var xdata = {"COMMON-LISP____DEFUN":[["Events","Programming"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))\u000A      y))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                              (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      1\u000A      (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (var1 ... varn) doc-string dcl ... dcl body),</code> \u000A \u000A <p>where <v>fn</v> is the symbol you wish to define and is a new symbolic name \u000A (see <see topic=\"ACL2____NAME\">name</see>), <v>(var1 ... varn)</v> is its list of formal parameters (see \u000A <see topic=\"ACL2____NAME\">name</see>), and <v>body</v> is its body.  The definitional axiom is logically \u000A admissible provided certain restrictions are met.  These are sketched \u000A below.</p> \u000A \u000A <p>Note that ACL2 does not support the use of <v>lambda-list</v> keywords (such \u000A as <v>&amp;optional</v>) in the formals list of functions.  We do support some such \u000A keywords in macros and often you can achieve the desired syntax by defining a \u000A macro in addition to the general version of your function.  See <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>.  The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string, <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt>, is optional; \u000A for a description of its form, see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p>The <i>declarations</i> (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>), <v>dcl</v>, are also optional. \u000A If more than one <v>dcl</v> form appears, they are effectively grouped together \u000A as one.  Perhaps the most commonly used ACL2 specific declaration is of the \u000A form <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard g :measure m))</v>.  This declaration in the \u000A <v>defun</v> of some function <v>fn</v> has the effect of making the ``<see topic=\"ACL2____GUARD\">guard</see>'' for <v>fn</v> be the term <v>g</v> and the ``measure'' be the term \u000A <v>m</v>.  The notion of ``measure'' is crucial to ACL2's definitional \u000A principle.  The notion of ``guard'' is not, and is discussed elsewhere; see \u000A <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> and see <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see>.  Note that \u000A the <v>:measure</v> is ignored in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode; see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.</p> \u000A \u000A <p>We now briefly discuss the ACL2 definitional principle, using the following \u000A definition form which is offered as a more or less generic example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (g x y)\u000A                  :measure (m x y)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (test x y)\u000A      (stop x y)\u000A    (step (fn (d x) y))))</code> \u000A \u000A <p>Note that in our generic example, <v>fn</v> has just two arguments, <v>x</v> \u000A and <v>y</v>, the <see topic=\"ACL2____GUARD\">guard</see> and measure terms involve both of them, and the \u000A body is a simple case split on <v>(test x y)</v> leading to a ``non-recursive'' \u000A branch, <v>(stop x y)</v>, and a ``recursive'' branch.  In the recursive branch, \u000A <v>fn</v> is called after ``decrementing'' <v>x</v> to <v>(d x)</v> and some step \u000A function is applied to the result.  Of course, this generic example is quite \u000A specific in form but is intended to illustrate the more general case.</p> \u000A \u000A <p>Provided this definition is admissible under the logic, as outlined below, \u000A it adds the following axiom to the logic.</p> \u000A \u000A <code>Defining Axiom:\u000A(fn x y)\u000A  =\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (test x y)\u000A    (stop x y)\u000A  (step (fn (d x) y)))</code> \u000A \u000A <p>Note that the <see topic=\"ACL2____GUARD\">guard</see> of <v>fn</v> has no bearing on this logical \u000A axiom.</p> \u000A \u000A <p>This defining axiom is actually implemented in the ACL2 system by a \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule, namely</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (test a b)\u000A           (stop a b)\u000A         (step (fn (d a) b)))).</code> \u000A \u000A <p>See <see topic=\"ACL2____DEFINITION\">definition</see> for a discussion of how definition rules are applied. \u000A Roughly speaking, the rule causes certain instances of <v>(fn x y)</v> to be \u000A replaced by the corresponding instances of the body above.  This is called \u000A ``opening up'' <v>(fn x y)</v>.  The instances of <v>(fn x y)</v> opened are \u000A chosen primarily by heuristics which determine that the recursive calls of \u000A <v>fn</v> in the opened body (after simplification) are more desirable than the \u000A unopened call of <v>fn</v>.</p> \u000A \u000A <p>This discussion has assumed that the definition of <v>fn</v> was admissible. \u000A Exactly what does that mean?  First, <v>fn</v> must be a previously \u000A unaxiomatized function symbol (however, see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>). \u000A Second, the formal parameters must be distinct variable names.  Third, the \u000A <see topic=\"ACL2____GUARD\">guard</see>, measure, and body should all be terms and should mention no free \u000A variables except the formal parameters.  Thus, for example, body may not \u000A contain references to ``global'' or ``special'' variables; ACL2 constants or \u000A additional formals should be used instead.</p> \u000A \u000A <p>The final conditions on admissibility concern the termination of the \u000A recursion.  Roughly put, all applications of <v>fn</v> must terminate.  In \u000A particular, there must exist a binary relation, <v>rel</v>, and some unary \u000A predicate <v>mp</v> such that <v>rel</v> is well-founded on objects satisfying \u000A <v>mp</v>, the measure term <v>m</v> must always produce something satisfying \u000A <v>mp</v>, and the measure term must decrease according to <v>rel</v> in each \u000A recursive call, under the hypothesis that all the tests governing the call are \u000A satisfied.  By the meaning of well-foundedness, we know there are no \u000A infinitely descending chains of successively <v>rel</v>-smaller <v>mp</v>-objects. \u000A Thus, the recursion must terminate.</p> \u000A \u000A <p>The only primitive well-founded relation in ACL2 is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> (see <see topic=\"ACL2____O_C3\">o&lt;</see>), which is known to be well-founded on the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s (see <see topic=\"ACL2____O-P\">o-p</see>). \u000A For the proof of well-foundedness, see <see topic=\"ACL2____PROOF-OF-WELL-FOUNDEDNESS\">proof-of-well-foundedness</see>. \u000A However it is possible to add new well-founded relations.  For details, see \u000A <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.  We discuss later how to specify which \u000A well-founded relation is selected by <v>defun</v> and in the present discussion \u000A we assume, without loss of generality, that it is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> on the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s.</p> \u000A \u000A <p>For example, for our generic definition of <v>fn</v> above, with measure term \u000A <v>(m x y)</v>, two theorems must be proved.  The first establishes that <v>m</v> \u000A produces an ordinal:</p> \u000A \u000A <code>(<see topic=\"ACL2____O-P\">o-p</see> (m x y)).</code> \u000A \u000A <p>The second shows that <v>m</v> decreases in the (only) recursive call of \u000A <v>fn</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (test x y))\u000A         (<see topic=\"ACL2____O_C3\">o&lt;</see> (m (d x) y) (m x y))).</code> \u000A \u000A <p>Observe that in the latter formula we must show that the ``<v>m</v>-size'' of \u000A <v>(d x)</v> and <v>y</v> is ``smaller than'' the <v>m</v>-size of <v>x</v> and \u000A <v>y</v>, provided the test, <v>(test x y)</v>, in the body fails, thus leading to \u000A the recursive call <v>(fn (d x) y)</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____O_C3\">o&lt;</see> for a discussion of this notion of ``smaller than.''  It \u000A should be noted that the most commonly used ordinals are the natural numbers \u000A and that on natural numbers, <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> is just the familiar ``less than'' \u000A relation (<tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>).  Thus, it is very common to use a measure <v>m</v> that \u000A returns a nonnegative integer, for then <v>(<see topic=\"ACL2____O-P\">o-p</see> (m x y))</v> becomes a simple \u000A conjecture about the type of <v>m</v> and the second formula above becomes a \u000A conjecture about the less-than relationship of nonnegative integer \u000A arithmetic.</p> \u000A \u000A <p>The most commonly used measure function is <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt>, which \u000A computes a nonnegative integer size for all ACL2 objects.  See <see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see>.</p> \u000A \u000A <p>Probably the most common recursive scheme in Lisp <see topic=\"ACL2____PROGRAMMING\">programming</see> is \u000A when some formal is supposed to be a list and in the recursive call it is \u000A replaced by its <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.  For example, <v>(test x y)</v> might be simply \u000A <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v> and <v>(d x)</v> might be <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>.  In that case, \u000A <v>(<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)</v> is a suitable measure because the <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> of a \u000A <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> is strictly larger than the <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt>s of its <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.  Thus, ``recursion by <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>'' and ``recursion by \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>'' are trivially admitted if <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> is used as the \u000A measure and the definition protects every recursive call by a test insuring \u000A that the decremented argument is a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>.  Similarly, ``recursion by \u000A <tt><see topic=\"COMMON-LISP____1-\">1-</see></tt>'' in which a positive integer formal is decremented by one in \u000A recursion, is also trivially admissible.  See <see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see> to extend \u000A the class of trivially admissible recursive schemes.</p> \u000A \u000A <p>We now turn to the question of which well-founded relation <v>defun</v> uses. \u000A It should first be observed that <v>defun</v> must actually select both a \u000A relation (e.g., <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>) and a domain predicate (e.g., <tt><see topic=\"ACL2____O-P\">o-p</see></tt>) on \u000A which that relation is known to be well-founded.  But, as noted elsewhere (see \u000A <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>), every known well-founded relation has a unique \u000A domain predicate associated with it and so it suffices to identify simply the \u000A relation here.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> field of a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> permits the explicit \u000A specification of any known well-founded relation with the keyword <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt>.  An example is given below.  If the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> for \u000A a <v>defun</v> specifies a well-founded relation, that relation and its \u000A associated domain predicate are used in generating the termination conditions \u000A for the definition.</p> \u000A \u000A <p>If no <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> is specified, <v>defun</v> uses the \u000A <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> specified in the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  See <see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> to see how to set \u000A the default well-founded relation (and, implicitly, its domain predicate). \u000A The initial default well-founded relation is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> (with domain predicate \u000A <tt><see topic=\"ACL2____O-P\">o-p</see></tt>).</p> \u000A \u000A <p>This completes the brief sketch of the ACL2 definitional principle. \u000A Optionally, see <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> for a more detailed discussion of the \u000A termination analysis and resulting proof obligations for admissibility, as \u000A well as a discussion of the relation to how ACL2 stores induction schemes.</p> \u000A \u000A <p>On very rare occasions ACL2 will seem to \"hang\" when processing a \u000A definition, especially if there are many subexpressions of the body whose \u000A function symbol is <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> (or which macroexpand to such an expression). \u000A In those cases you may wish to supply the following to <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>: \u000A <v>:normalize nil</v>.  This is an advanced feature that turns off ACL2's usual \u000A propagation upward of <v>if</v> tests.</p> \u000A \u000A <p>The following example illustrates all of the available declarations and \u000A most hint keywords, but is completely nonsensical.  For documentation, see \u000A <see topic=\"ACL2____XARGS\">xargs</see> and see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> example (x y z a b c i j)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> a b c)\u000A           (<see topic=\"COMMON-LISP____TYPE\">type</see> integer i j)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                  :measure (<see topic=\"COMMON-LISP____-\">-</see> i j)\u000A                  :ruler-extenders :basic\u000A                  :well-founded-relation my-wfr\u000A                  :hints ((&quot;Goal&quot;\u000A                           :do-not-induct t\u000A                           :do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see> fertilize)\u000A                           :expand ((<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x a) (<see topic=\"COMMON-LISP____MEMBER\">member</see> y z))\u000A                           :restrict ((&lt;-trans ((x x) (y (foo x)))))\u000A                           :hands-off (<see topic=\"COMMON-LISP____LENGTH\">length</see> binary-append)\u000A                           :in-theory (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see>\u000A                                        (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                                        '(<see topic=\"COMMON-LISP____ASSOC\">assoc</see>))\u000A                           :induct (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n a) (<see topic=\"COMMON-LISP____NTH\">nth</see> n b))\u000A                           :use ((:instance assoc-of-append\u000A                                            (x a) (y b) (z c))\u000A                                 (:functional-instance\u000A                                   (:instance p-f (x a) (y b))\u000A                                   (p consp)\u000A                                   (f assoc)))))\u000A                  :guard-hints ((&quot;Subgoal *1/3'&quot;\u000A                                 :use ((:instance assoc-of-append\u000A                                                  (x a) (y b) (z c)))))\u000A                  :mode :logic\u000A                  :normalize nil\u000A                  :verify-guards nil\u000A                  :non-executable t\u000A                  :otf-flg t))\u000A  (example-body x y z i j))</code>"],

 "ACL2____WORLD":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>The ACL2 logical world is a data structure that includes all \u000A logical content resulting from the <see topic=\"ACL2____COMMAND\">command</see>s evaluated, back through and \u000A including initialization, but not including commands that have been \u000A undone (see <see topic=\"ACL2____UBT\">ubt</see>).  Thus in particular, the world includes a \u000A represention of the current logical theory, as well as some extra-logical \u000A information such as the values of ACL2 <see topic=\"ACL2____TABLE\">table</see>s.  The rest of this topic \u000A focuses on the structure of the the ACL2 world and, more generally, the \u000A ``world'' data structure.</p> \u000A \u000A <p>A ``world'' is a list of triples, each of the form <v>(sym prop . val)</v>, \u000A implementing the ACL2 notion of property lists.  ACL2 permits the simultaneous \u000A existence of many property list worlds.  ``The world'' is often used as a \u000A shorthand for ``the ACL2 logical world'' which is the particular property list \u000A world used within the ACL2 system to maintain a database that contiains rules, \u000A <see topic=\"ACL2____TABLE\">table</see>s, and so on.</p> \u000A \u000A <p>Common Lisp provides the notion of ``property lists'' by which one can \u000A attach ``properties'' and their corresponding ``values'' to symbols.  For \u000A example, one can arrange for the <v>'color</v> property of the symbol \u000A <v>'box-14</v> to be <v>'purple</v> and the <v>'color</v> property of the symbol \u000A <v>'triangle-7</v> to be <v>'yellow</v>.  Access to property lists is given via \u000A the Common Lisp function <v>get</v>.  Thus, <v>(get 'box-14 'color)</v> might \u000A return <v>'purple</v>.  Property lists can be changed via the special form \u000A <v>setf</v>.  Thus, <v>(setf (get 'box-14 'color) 'blue)</v> changes the Common \u000A Lisp property list configuration so that <v>(get 'box-14 'color)</v> returns \u000A <v>'blue</v>.  It should be obvious that ACL2 cannot provide this facility, \u000A because Common Lisp's <v>get</v> ``function'' is not a function of its argument, \u000A but instead a function of some implicit state object representing the property \u000A list settings for all symbols.</p> \u000A \u000A <p>ACL2 provides the functions <v>getprop</v> and <v>putprop</v> which allow one \u000A to mimic the Common Lisp property list facility.  However, ACL2's <v>getprop</v> \u000A takes as one of its arguments a list that is a direct encoding of what was \u000A above called the ``state object representing the property list settings for \u000A all symbols.''  Because ACL2 already has a notion of ``<see topic=\"ACL2____STATE\">state</see>'' that is \u000A quite distinct from that used here, we call this property list object a \u000A ``world.''  A world is just a true list of triples.  Each triple is of the \u000A form <v>(sym prop . val)</v>.  This world can be thought of as a slightly \u000A elaborated form of association list and <v>getprop</v> is a slightly elaborated \u000A form of <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt> that takes two keys.  When <v>getprop</v> is called on a \u000A symbol, <v>s</v>, property <v>p</v>, and world, <v>w</v>, it scans <v>w</v> for the \u000A first triple whose <v>sym</v> is <v>s</v> and <v>prop</v> is <v>p</v> and returns the \u000A corresponding <v>val</v>. <v>Getprop</v> has two additional arguments: one \u000A controls what it returns if no such <v>sym</v> and <v>prop</v> exist in <v>w</v>, \u000A and the other allows an extremely efficient implementation.  To set some \u000A property's value for some symbol, ACL2 provides <v>putprop</v>.  <v>(<see topic=\"ACL2____PUTPROP\">putprop</see> sym\u000A prop val w)</v> merely returns a new world, <v>w'</v>, in which <v>(sym prop\u000A . val)</v> has been <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>ed onto the front of <v>w</v>, thus \u000A ``overwriting'' the <v>prop</v> value of <v>sym</v> in <v>w</v> to <v>val</v> and \u000A leaving all other properties in <v>w</v> unchanged.</p> \u000A \u000A <p>One aspect of ACL2's property list arrangement is that it is possible to \u000A have many different property list worlds.  For example, <v>'box-14</v> can have \u000A <v>'color</v> <v>'purple</v> in one world and can have <v>'color</v> <v>'yes</v> in \u000A another, and these two worlds can exist simultaneously because <v>getprop</v> is \u000A explicitly provided the world from which the property value is to be \u000A extracted.</p> \u000A \u000A <p>The efficiency alluded to above stems from the fact that Common Lisp \u000A provides property lists.  Using Common Lisp's provisions behind the scenes, \u000A ACL2 can ``install'' the properties of a given world into the Common Lisp \u000A property list state so as to make retrieval via <v>getprop</v> very fast in the \u000A special case that the world provided to <v>getprop</v> has been installed.  To \u000A permit more than one installed world, each of which is permitted to be changed \u000A via <v>putprop</v>, ACL2 requires that worlds be named and these names are used \u000A to distinquish installed versions of the various worlds.  At the moment we do \u000A not further document <v>getprop</v> and <v>putprop</v>.</p> \u000A \u000A <p>However, the ACL2 system uses a property list world, named \u000A <v>'current-acl2-world</v>, in which to store the succession of user <see topic=\"ACL2____COMMAND\">command</see>s and their effects on the logic.  This world is often referred to in \u000A our <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> as ``the world'' though it should be stressed that \u000A the user is permitted to have worlds and ACL2's is in no way distinguished \u000A except that the user is not permitted to modify it except via event <see topic=\"ACL2____COMMAND\">command</see>s.  The ACL2 world is part of the ACL2 <see topic=\"ACL2____STATE\">state</see> and may be \u000A obtained via <v>(w state)</v>.</p> \u000A \u000A <p><b>Warning</b>: The ACL2 world is very large.  Its length as of this \u000A writing (Version 2.5) is over <v>40,000</v> and it grows with each release. \u000A Furthermore, some of the values stored in it are pointers to old versions of \u000A itself.  Printing <v>(w state)</v> is something you should avoid because you \u000A likely will not have the patience to await its completion.  For these \u000A practical reasons, the only thing you should do with <v>(w state)</v> is provide \u000A it to <v>getprop</v>, as in the form</p> \u000A \u000A <code>(<see topic=\"ACL2____GETPROP\">getprop</see> sym prop default 'current-acl2-world (w state))</code> \u000A \u000A <p>to inspect properties within it, or to pass it to ACL2 primitives, such as \u000A theory functions, where it is expected.</p> \u000A \u000A <p>Some ACL2 <see topic=\"ACL2____COMMAND\">command</see> forms, such as theory expressions (see <see topic=\"ACL2____THEORIES\">theories</see>) and the values to be stored in tables (see <see topic=\"ACL2____TABLE\">table</see>), are \u000A permitted to use the variable symbol <v>world</v> freely with the understanding \u000A that when these forms are evaluated that variable is bound to <v>(w state)</v>. \u000A Theoretically, this gives those forms complete knowledge of the current \u000A logical configuration of ACL2.  However, at the moment, few world scanning \u000A functions have been documented for the ACL2 user.  Instead, supposedly \u000A convenient macro forms have been created and documented.  For example, \u000A <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v>, which is the theory expression which returns the \u000A currently <see topic=\"ACL2____ENABLE\">enable</see>d theory, actually macroexpands to \u000A <v>(current-theory-fn :here world)</v>.  When evaluated with <v>world</v> bound to \u000A <v>(w state)</v>, <v>current-theory-fn</v> scans the current ACL2 world and \u000A computes the set of <see topic=\"ACL2____RUNE\">rune</see>s currently <see topic=\"ACL2____ENABLE\">enable</see>d in it.</p>"],

 "ACL2____THEORIES":[["ACL2"],"ACL2 Sources","ACL2","<code>Example: '((:definition app) ; or (:d app)\u000A           (:executable-counterpart app)\u000A           (:i app)\u000A           rv\u000A           (rv)\u000A           assoc-of-app)</code> \u000A \u000A <p>See:</p> \u000A \u000A <p>A theory is a list of ``runic designators'' as described below.  Each runic \u000A designator denotes a set of ``runes'' (see <see topic=\"ACL2____RUNE\">rune</see>) and by unioning \u000A together the runes denoted by each member of a theory we define the set of \u000A runes corresponding to a theory.  Theories are used to control which rules are \u000A ``<see topic=\"ACL2____ENABLE\">enable</see>d,'' i.e., available for automatic application by the theorem \u000A prover.  There is always a ``current'' theory.  A rule is <see topic=\"ACL2____ENABLE\">enable</see>d \u000A precisely if its <see topic=\"ACL2____RUNE\">rune</see> is an element of the set of <see topic=\"ACL2____RUNE\">rune</see>s \u000A corresponding to the current theory.  At the top-level, the current theory is \u000A the theory selected by the most recent <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event, extended with \u000A the rule names introduced since then.  Inside the theorem prover, the \u000A <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint (see <see topic=\"ACL2____HINTS\">hints</see>) can be used to select a \u000A particular theory as current during the proof attempt for a particular \u000A goal.</p> \u000A \u000A <p>Theories are generally constructed by ``theory expressions.''  Formally, a \u000A theory expression is any term, containing at most the single free variable \u000A <tt><see topic=\"ACL2____WORLD\">world</see></tt>, that when evaluated with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the current \u000A ACL2 world (see <see topic=\"ACL2____WORLD\">world</see>) produces a theory.  ACL2 provides various \u000A functions for the convenient construction and manipulation of theories.  These \u000A are called ``theory functions''(see <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see>).  For example, \u000A the theory function <tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt> takes two theories and produces \u000A their union.  The theory function <tt><see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see></tt> returns the theory \u000A containing all known rule names as of the introduction of a given logical \u000A name.  But a theory expression can contain constants, e.g.,</p> \u000A \u000A <code>'(<see topic=\"ACL2____LEN\">len</see> (<see topic=\"ACL2____LEN\">len</see>) (:rewrite car-cons) car-cdr-elim)</code> \u000A \u000A <p>and user-defined functions.  The only important criterion is that a theory \u000A expression mention no variable freely except <tt><see topic=\"ACL2____WORLD\">world</see></tt> and evaluate to a \u000A theory.</p> \u000A \u000A <p>More often than not, theory expressions typed by the user do not mention \u000A the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  This is because user-typed theory expressions are \u000A generally composed of applications of ACL2's theory functions.  These \u000A ``functions'' are actually macros that expand into terms in which <tt><see topic=\"ACL2____WORLD\">world</see></tt> is used freely and appropriately.  Thus, the technical definition of \u000A ``theory expression'' should not mislead you into thinking that interestng \u000A theory expressions must mention <tt><see topic=\"ACL2____WORLD\">world</see></tt>; they probably do and you just \u000A didn't know it!</p> \u000A \u000A <p>One aspect of this arrangement is that theory expressions cannot generally \u000A be evaluated at the top-level of ACL2, because <tt><see topic=\"ACL2____WORLD\">world</see></tt> is not bound.  To \u000A see the value of a theory expression, <v>expr</v>, at the top-level, type</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">LET</see> ((<see topic=\"ACL2____WORLD\">WORLD</see> (W STATE))) expr).</code> \u000A \u000A <p>However, because the built-in theories are quite long, you may be sorry you \u000A printed the value of a theory expression!</p> \u000A \u000A <p>A theory is a true list of runic designators and to each theory there \u000A corresponds a set of <see topic=\"ACL2____RUNE\">rune</see>s, obtained by unioning together the sets of \u000A <see topic=\"ACL2____RUNE\">rune</see>s denoted by each runic designator.  For example, the theory \u000A constant</p> \u000A \u000A <code>'(<see topic=\"ACL2____LEN\">len</see> (<see topic=\"ACL2____LEN\">len</see>) (:e nth) (:rewrite car-cons) car-cdr-elim)</code> \u000A \u000A <p>corresponds to the set of <see topic=\"ACL2____RUNE\">rune</see>s</p> \u000A \u000A <code>{(:definition len)\u000A (:induction len)\u000A (:executable-counterpart len)\u000A (:executable-counterpart nth)\u000A (:elim car-cdr-elim)\u000A (:rewrite car-cons)} .</code> \u000A \u000A <p>Observe that the theory contains five elements but its runic correspondent \u000A contains six.  That is because runic designators can denote sets of several \u000A <see topic=\"ACL2____RUNE\">rune</see>s, as is the case for the first designator, <v>len</v>.  If the above \u000A theory were selected as current then the six rules named in its runic \u000A counterpart would be <see topic=\"ACL2____ENABLE\">enable</see>d and all other rules would be <see topic=\"ACL2____DISABLE\">disable</see>d.</p> \u000A \u000A <p>We now precisely define the runic designators and the set of <see topic=\"ACL2____RUNE\">rune</see>s \u000A denoted by each.  When we refer below to the ``macro-aliases dereference of'' \u000A a symbol, <v>symb</v>, we mean the (function) symbol corresponding <v>symb</v> in \u000A the macro-aliases-table if there is such a symbol, else <v>symb</v> itself; see \u000A <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.  For example, the macro-aliases dereference of \u000A <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> is <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>, and the macro-aliases dereference of \u000A <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> is <v>nth</v>.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o A <see topic=\"ACL2____RUNE\">rune</see> is a runic designator and denotes the singleton set \u000A containing that rune.</p> \u000A \u000A <p>o Suppose that <v>symb</v> is a symbol and <v>symb'</v> is the macro-aliases \u000A dereference of <v>symb</v>, where <v>symb'</v> is a function symbol introduced \u000A with a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>) event.  Then <v>symb</v> is a runic \u000A designator and denotes the set containing the runes <tt>(:definition \u000A symb')</tt> and <tt>(:induction symb')</tt>, omitting the latter if no such \u000A <see topic=\"ACL2____INDUCTION\">induction</see> rune exists (presumably because the definition of <v>symb'</v> \u000A is not singly recursive).</p> \u000A \u000A <p>o Suppose that <v>symb</v> is a symbol and <v>symb'</v> is the macro-aliases \u000A dereference of <v>symb</v>, where <v>symb'</v> is a function symbol introduced \u000A with a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>) event.  Then <v>(symb)</v> is a runic \u000A designator and denotes the singleton set containing the rune \u000A <tt>(:executable-counterpart symb')</tt>.</p> \u000A \u000A <p>o If <v>symb</v> is the name of a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> (or <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>) event \u000A that introduced at least one rule, then <v>symb</v> is a runic designator and \u000A denotes the set of the names of all rules introduced by the named event.</p> \u000A \u000A <p>o If <v>str</v> is the string naming some <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event and <v>symb</v> \u000A is the symbol returned by <v>(<see topic=\"COMMON-LISP____INTERN\">intern</see> str &quot;ACL2&quot;)</v>, then <v>symb</v> is a \u000A runic designator and denotes the singleton set containing <v>(:rewrite\u000A symb)</v>, which is the name of the rule stating the conditions under which the \u000A <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of <v>(<see topic=\"COMMON-LISP____INTERN\">intern</see> x str)</v> is <v>str</v>.</p> \u000A \u000A <p>o If <v>symb</v> is the name of a <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event, then <v>symb</v> is \u000A a runic designator and denotes the runic theory corresponding to \u000A <v>symb</v>.</p> \u000A \u000A <p>o Finally, suppose that <v>symb</v> is a symbol and <v>symb'</v> is the \u000A macro-aliases dereference of <v>symb</v>.  Then <v>(:KWD symb . rest)</v> is a \u000A runic designator if <v>(:KWD' symb' . rest)</v> is a <see topic=\"ACL2____RUNE\">rune</see>, where \u000A <v>:KWD</v> is one of <v>:d</v>, <v>:e</v>, <v>:i</v>, or <v>:t</v>, and correspondingly \u000A <v>:KWD'</v> is <v>:definition</v>, <v>:executable-counterpart</v>, <v>:induction</v>, \u000A or <v>:type-prescription</v>, respectively.  In this case, <v>(:KWD symb\u000A . rest)</v> denotes the runic theory corresponding to the rune <v>(:KWD' symb'\u000A . rest)</v>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Note that including a function name, e.g., <tt><see topic=\"ACL2____LEN\">len</see></tt>, in the current \u000A theory <see topic=\"ACL2____ENABLE\">enable</see>s that function but does not <see topic=\"ACL2____ENABLE\">enable</see> the executable \u000A counterpart.  Similarly, including <v>(<see topic=\"ACL2____LEN\">len</see>)</v> or <v>(:e len)</v> <see topic=\"ACL2____ENABLE\">enable</see>s \u000A the executable counterpart but not the symbolic definition.  And including the \u000A name of a proved lemma <see topic=\"ACL2____ENABLE\">enable</see>s all of the rules added by the event.  Of \u000A course, one can include explicitly the <see topic=\"ACL2____RUNE\">rune</see>s naming the rules in \u000A question and so can avoid entirely the use of non-runic elements in \u000A theories.</p> \u000A \u000A <p>Because a <see topic=\"ACL2____RUNE\">rune</see> is a runic designator denoting the set containing \u000A that <see topic=\"ACL2____RUNE\">rune</see>, a list of <see topic=\"ACL2____RUNE\">rune</see>s is a theory and denotes itself.  We \u000A call such theories ``runic theories.''  To every theory there corresponds a \u000A runic theory obtained by unioning together the sets denoted by each designator \u000A in the theory.  When a theory is selected as ``current'' it is actually its \u000A runic correspondent that is effectively used.  That is, a <see topic=\"ACL2____RUNE\">rune</see> is <see topic=\"ACL2____ENABLE\">enable</see>d iff it is a member of the runic correspondent of the current theory. \u000A The value of a theory defined with <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> is the runic \u000A correspondent of the theory computed by the defining theory expression.  The \u000A theory manipulation functions, e.g., <tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt>, actually convert \u000A their theory arguments to their runic correspondents before performing the \u000A required set operation.  The manipulation functions always return runic \u000A theories.  Thus, it is sometimes convenient to think of (non-runic) theories \u000A as merely abbreviations for their runic correspondents, abbreviations which \u000A are ``expanded'' at the first opportunity by theory manipulation functions and \u000A the ``theory consumer'' functions such as <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> and <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>.</p>"],

 "ACL2____TABLE":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TABLE\">table</see> tests 1 '(...))                ; set contents of tests[1] to '(...)\u000A(<see topic=\"ACL2____TABLE\">table</see> tests 25)                      ; get contents of tests[25]\u000A(<see topic=\"ACL2____TABLE\">table</see> tests)                         ; return table tests as an alist\u000A(<see topic=\"ACL2____TABLE\">table</see> tests nil nil :clear)          ; clear table tests\u000A(<see topic=\"ACL2____TABLE\">table</see> tests nil '((foo . 7)) :clear) ; set table tests to ((foo . 7))\u000A(<see topic=\"ACL2____TABLE\">table</see> tests nil nil :guard)          ; fetch the table guard\u000A(<see topic=\"ACL2____TABLE\">table</see> tests nil nil :guard term)     ; set the table guard\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____TABLE\">table</see> table-name key-term value-term op term)</code> \u000A \u000A <p>where <v>table-name</v> is a symbol that is the name of a (possibly new) \u000A table, <v>key-term</v> and <v>value-term</v>, if present, are arbitrary terms \u000A involving (at most) the single variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, <v>op</v>, if present, is \u000A one of the table operations below, and <v>term</v>, if present, is a term. \u000A <v>Table</v> returns an ACL2 ``error triple'' (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>).  The \u000A effect of <v>table</v> on <tt><see topic=\"ACL2____STATE\">state</see></tt> depends on <v>op</v> and how many \u000A arguments are presented.  Some invocations actually have no effect on the ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see> and hence an invocation of <v>table</v> is not always an ``event''. \u000A We explain below, after giving some background information.</p> \u000A \u000A <p><b>Important Note:</b> The <v>table</v> forms above are calls of a macro that \u000A expand to involve the special variable <tt><see topic=\"ACL2____STATE\">state</see></tt>.  This will prevent you \u000A from accessing a table from within a hint or theory where you do not have the \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> variable.  However, the form</p> \u000A \u000A <code>(table-alist 'tests world)</code> \u000A \u000A <p>returns the alist representation of the table named <v>test</v> in the given \u000A world.  Often you have access to <v>world</v>.</p> \u000A \u000A <p>The ACL2 system provides ``tables'' by which the user can associate one \u000A object with another.  Tables are in essence just conventional association \u000A lists &mdash; lists of pairs &mdash; but the ACL2 environment provides a means \u000A of storing these lists in the ``ACL2 world'' of the current <tt><see topic=\"ACL2____STATE\">state</see></tt>. \u000A The ACL2 user could accomplish the same ends by using ACL2 ``global \u000A variables;'' however, limitations on global variable names are imposed to \u000A ensure ACL2's soundness.  By convention, no table is important to ACL2's \u000A soundness, even though some features of the system use tables, and the user is \u000A invited to make free use of tables.  Because tables are stored in the ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see> they are restored by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and undone by \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>.  Many users of Nqthm requested a facility by which user \u000A data could be saved in Nqthm ``lib files'' and tables are ACL2's answer to \u000A that request.</p> \u000A \u000A <p>Abstractly, each table is an association list mapping ``keys'' to \u000A ``values.'' In addition, each table has a ``<v>:guard</v>,'' which is a term \u000A that must be true of any key and value used.  By setting the <v>:guard</v> on a \u000A table you may enforce an invariant on the objects in the table, e.g., that all \u000A keys are positive integers and all values are symbols.  Each table has a \u000A ``name,'' which must be a symbol.  Given a table name, the following \u000A operations can be performed on the table.</p> \u000A \u000A <p><v>:put</v> &mdash; associate a value with a key (possibly changing the value \u000A currently associated with that key).</p> \u000A \u000A <p><v>:get</v> &mdash; retrieve the value associated with a key (or nil if no \u000A value has been associated with that key).</p> \u000A \u000A <p><v>:alist</v> &mdash; return an alist showing all keys and non-nil values in \u000A the table.</p> \u000A \u000A <p><v>:clear</v> &mdash; clear the table (so that every value is nil), or if val \u000A is supplied then set table to that value (which must be an alist).</p> \u000A \u000A <p><v>:guard</v> &mdash; fetch or set the :guard of the table.</p> \u000A \u000A <p>When the operations above suggest that the table or its <v>:guard</v> are \u000A modified, what is actually meant is that the current <see topic=\"ACL2____STATE\">state</see> is redefined \u000A so that in it, the affected table name has the appropriate properties.  in \u000A such cases, the <v>table</v> form is an event (see <see topic=\"ACL2____EVENTS\">events</see>).  In the \u000A <v>:put</v> case, if the key is already in the table and associated with the \u000A proposed value, then the <v>table</v> event is redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).</p> \u000A \u000A <p><v>Table</v> forms are commonly typed by the user while interacting with the \u000A system.  <v>:Put</v> and <v>:get</v> forms are especially common.  Therefore, we \u000A have adopted a positional syntax that is intended to be convenient for most \u000A applications.  Essentially, some operations admit a ``short form'' of \u000A invocation.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name key-term value-term :put)   ; long form\u000A(<see topic=\"ACL2____TABLE\">table</see> name key-term value-term)        ; short form</code> \u000A \u000A <p>evaluates the key- and value-terms, obtaining two objects that we call \u000A <v>key</v> and <v>value</v>, checks that the <v>key</v> and <v>value</v> satisfy the \u000A <v>:guard</v> on the named table and then ``modifies'' the named table so that \u000A the value associated with <v>key</v> is <v>value</v>.  When used like this, \u000A <v>table</v> is actually an event in the sense that it changes the ACL2 <see topic=\"ACL2____WORLD\">world</see>.  In general, the forms evaluated to obtain the <v>key</v> and <v>value</v> \u000A may involve the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, which is bound to the then-current \u000A <see topic=\"ACL2____WORLD\">world</see> during the evaluation of the forms.  However, in the special case \u000A that the table in question is named <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, the <v>key</v> \u000A and <v>value</v> terms may not contain any variables.  Essentially, the keys and \u000A values used in <see topic=\"ACL2____EVENTS\">events</see> setting the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> must be \u000A explicitly given constants.  See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name key-term nil :get)          ; long form\u000A(<see topic=\"ACL2____TABLE\">table</see> name key-term)                   ; short form</code> \u000A \u000A <p>evaluates the key-term (see note below), obtaining an object, <v>key</v>, and \u000A returns the value associated with <v>key</v> in the named table (or, <v>nil</v> if \u000A there is no value associated with <v>key</v>).  When used like this, <v>table</v> \u000A is not an event; the value is simply returned.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name nil nil :alist)             ; long form\u000A(<see topic=\"ACL2____TABLE\">table</see> name)                            ; short form</code> \u000A \u000A <p>returns an alist representing the named table; for every key in the table \u000A with a non-<v>nil</v> associated value, the alist pairs the key and its value. \u000A The order in which the keys are presented is unspecified.  When used like \u000A this, <v>table</v> is not an event; the alist is simply returned.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name nil val :clear)</code> \u000A \u000A <p>sets the named table to the alist <v>val</v>, making the checks that \u000A <v>:put</v> makes for each key and value of <v>val</v>.  When used like this, \u000A <v>table</v> is an event because it changes the ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name nil nil :guard)</code> \u000A \u000A <p>returns the translated form of the guard of the named table.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> name nil nil :guard term)</code> \u000A \u000A <p>Provided the named table is empty and has not yet been assigned a \u000A <v>:guard</v> and <v>term</v> (which is not evaluated) is a term that mentions at \u000A most the variables <v>key</v>, <v>val</v> and <tt><see topic=\"ACL2____WORLD\">world</see></tt>, this event sets the \u000A <v>:guard</v> of the named table to <v>term</v>.  Whenever a subsequent <v>:put</v> \u000A occurs, <v>term</v> will be evaluated with <v>key</v> bound to the key argument of \u000A the <v>:put</v>, <v>val</v> bound to the <v>val</v> argument of the <v>:put</v>, and \u000A <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the then current <see topic=\"ACL2____WORLD\">world</see>.  An error will be caused \u000A by the <v>:put</v> if the result of the evaluation is <v>nil</v>.</p> \u000A \u000A <p>Note that it is not allowed to change the <v>:guard</v> on a table once it \u000A has been explicitly set.  Before the <v>:guard</v> is explicitly set, it is \u000A effectively just <v>t</v>.  After it is set it can be changed only by undoing \u000A the event that set it.  The purpose of this restriction is to prevent the user \u000A from changing the <v>:guards</v> on tables provided by other people or the \u000A system.</p> \u000A \u000A <p>The intuition behind the <v>:guard</v> mechanism on tables is to enforce \u000A invariants on the keys and values in a table, so that the values, say, can be \u000A used without run-time checking.  But if the <v>:guard</v> of a table is \u000A sensitive to the ACL2 <see topic=\"ACL2____WORLD\">world</see>, it may be possible to cause some value in \u000A the table to cease satisfying the <v>:guard</v> without doing any operations on \u000A the table.  Consider for example the <v>:guard</v> ``no value in this table is \u000A the name of an event.'' As described, that is enforced each time a value is \u000A stored.  Thus, <v>'bang</v> can be <v>:put</v> in the table provided there is no \u000A event named <v>bang</v>.  But once it is in the table, there is nothing to \u000A prevent the user from defining <v>bang</v> as a function, causing the table to \u000A contain a value that could not be <v>:put</v> there anymore.  Observe that not \u000A all state-sensitive <v>:guard</v>s suffer this problem.  The <v>:guard</v> ``every \u000A value is an event name'' remains invariant, courtesy of the fact that undoing \u000A back through an event name in the table would necessarily undo the <v>:put</v> \u000A of the name into the table.</p> \u000A \u000A <p><v>Table</v> was designed primarily for convenient top-level use.  Tables are \u000A not especially efficient.  Each table is represented by an alist stored on the \u000A property list of the table name.  <v>:Get</v> is just a <v>getprop</v> and <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>.  <v>:Put</v> does a <v>getprop</v> to the get the table alist, a \u000A <v>put-assoc-equal</v> to record the new association, and a <v>putprop</v> to \u000A store the new table alist &mdash; plus the overhead associated with \u000A <v>:guard</v>s and undoable <see topic=\"ACL2____EVENTS\">events</see>, and checking (for redundancy) if the \u000A key is already bound to its proposed value.  Note that there are never \u000A duplicate keys in the resulting <v>alist</v>; in particular, when the operation \u000A <v>:clear</v> is used to install new <v>alist</v>, duplicate keys are removed from \u000A that alist.</p> \u000A \u000A <p>A table name may be any symbol whatsoever.  Symbols already in use as \u000A function or theorem names, for example, may be used as table names.  Symbols \u000A in use only as table names may be defined with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, etc.  Because \u000A there are no restrictions on the user's choice of table names, table names are \u000A not included among the logical names.  Thus, <v>:pe name</v> will never display \u000A a table event (for a logical name other than <v>:here</v>).  Either <v>:pe\u000A name</v> will display a ``normal'' event such as <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> name ...)</v> or \u000A <v>(<see topic=\"ACL2____DEFTHM\">defthm</see> name ...)</v> or else <v>:pe name</v> will cause an error indicating \u000A that <v>name</v> is not a logical name.  This happens even if <v>name</v> is in \u000A use as a table name.  Similarly, we do not permit table names to have <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings, since the same name might already have a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string.  If you want to associate a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string \u000A with a table name that is being used no other way, define the name as a label \u000A and use the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> feature of <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> (see <see topic=\"ACL2____DEFLABEL\">deflabel</see>); also see <see topic=\"ACL2____DEFDOC\">defdoc</see>.</p>"],

 "ACL2____STOBJ":[["Programming"],"ACL2 Sources","ACL2","<p>In ACL2, a ``single-threaded object'' is a data structure whose use \u000A is so syntactically restricted that only one instance of the object need ever \u000A exist and its fields can be updated by destructive assignments.</p> \u000A \u000A <p>Note: Novices are advised to avoid using single-threaded objects, perhaps \u000A instead using <see topic=\"STD____DEFAGGREGATE\">std::defaggregate</see> or community book \u000A <v>books/data-structures/structures.lisp</v>.  At the least, consider using \u000A <v>(</v><tt><see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see></tt><v> 0)</v> to avoid <see topic=\"ACL2____GUARD\">guard</see> \u000A verification.</p> \u000A \u000A <p>The documentation in this section is laid out in the form of a tour that \u000A visits the documented topics in a reasonable order.  We recommend that you \u000A follow the tour the first time you read about stobjs.  The list of all stobj \u000A topics is shown below.  The tour starts immediately afterwards.  Also see \u000A <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> and, for so-called abstract stobjs, see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>.</p> \u000A \u000A <p>As noted, a ``single-threaded object'' is a data structure whose use is so \u000A syntactically restricted that only one instance of the object need ever exist. \u000A Updates to the object must be sequentialized.  This allows us to update its \u000A fields with destructive assignments without wrecking the axiomatic semantics \u000A of update-by-copy.  For this reason, single-threaded objects are sometimes \u000A called ``von Neumann bottlenecks.''</p> \u000A \u000A <p>From the logical perspective, a single-threaded object is an ordinary ACL2 \u000A object, e.g., composed of integers and conses.  Logically speaking, ordinary \u000A ACL2 functions are defined to allow the user to ``access'' and ``update'' its \u000A fields.  Logically speaking, when fields in the object, obj, are ``updated'' \u000A with new values, a new object, obj', is constructed.</p> \u000A \u000A <p>But suppose that by syntactic means we could ensure that there were no more \u000A references to the ``old'' object, obj.  Then we could create obj' by \u000A destructively modifying the memory locations involved in the representation of \u000A obj.  The syntactic means is pretty simple but draconian: the only reference \u000A to obj is in the variable named <v>OBJ</v>.</p> \u000A \u000A <p>The consequences of this simple rule are far-reaching and require some \u000A getting used to.  For example, if <v>OBJ</v> has been declared as a \u000A single-threaded object name, then the following consequences ensue (but see \u000A the discussion of congruent stobjs below for a slight relaxation).</p> \u000A \u000A <p>o <v>OBJ</v> is a top-level global variable that contains the current object, \u000A obj.</p> \u000A \u000A <p>o If a function uses the formal parameter <v>OBJ</v>, the only ``actual \u000A expression'' that can be passed into that slot is the variable <v>OBJ</v>, not \u000A merely a term that ``evaluates to an obj''; thus, such functions can only \u000A operate on the current object.  So for example, instead of <v>(FOO\u000A (UPDATE-FIELD1 3 ST))</v> write <v>(<see topic=\"COMMON-LISP____LET\">LET</see> ((ST (UPDATE-FIELD1 3 ST))) (FOO\u000A ST))</v>.</p> \u000A \u000A <p>o The accessors and updaters have a formal parameter named <v>OBJ</v>, so by \u000A the rule just above, those functions can only be applied to the current \u000A object.  The recognizer is the one exception to the rule: it may be applied \u000A either the <v>OBJ</v> or to an ordinary (non-stobj) object.</p> \u000A \u000A <p>o The ACL2 primitives, such as <v>CONS</v>, <v>CAR</v> and <v>CDR</v>, may not be \u000A applied to the variable <v>OBJ</v>.  Thus, for example, obj may not be consed \u000A into a list (which would create another pointer to it) or accessed or copied \u000A via ``unapproved'' means.</p> \u000A \u000A <p>o The updaters return a ``new <v>OBJ</v> object'', i.e., obj'; thus, when an \u000A updater is called, the only variable which can hold its result is \u000A <v>OBJ</v>.</p> \u000A \u000A <p>o If a function calls an <v>OBJ</v> updater, it must return an <v>OBJ</v> \u000A object (either as the sole value returned, or in <v>(<see topic=\"ACL2____MV\">mv</see> ... OBJ ...)</v>; see \u000A <see topic=\"ACL2____MV\">mv</see>).</p> \u000A \u000A <p>o When a top-level expression involving <v>OBJ</v> returns an <v>OBJ</v> \u000A object, that object becomes the new current value of <v>OBJ</v>.</p> \u000A \u000A <p>There are other functional languages supporting single-threadedness, for \u000A example Haskell's ``monads'' and Clean's ``uniqueness type system''.  Of \u000A course, ACL2 provides a theorem prover that can prove theorems that involve \u000A such constructs.</p> \u000A \u000A <p>Note that the syntactic restrictions noted above are enforced only when \u000A single-threaded objects are encountered directly in the top-level loop or are \u000A used in function definitions; the accessor and update functions for \u000A single-threaded objects may be used without restriction in formulas to be \u000A proved.  Since function evaluation is sometimes necessary during proofs, ACL2 \u000A must be able to evaluate these functions on logical constants representing the \u000A object, even when the constant is not ``the current object.''  Thus, ACL2 \u000A supports both the efficient von Neumann semantics and the clean applicative \u000A semantics, and uses the first in contexts where execution speed is paramount \u000A and the second during proofs.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFSTOBJ\">Defstobj</see></tt> and <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> <see topic=\"ACL2____EVENTS\">events</see> introduce stobjs. \u000A See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> for more details about stobjs.  In particular, a \u000A relatively advanced notion of ``congruent stobjs'' is discussed there.  The \u000A idea is to allow a stobj, <v>st2</v>, of the same ``shape'' as a given stobj, \u000A <v>st1</v>, to be used in place of <v>st1</v>.  Other <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> keywords \u000A allow inlining and renaming of stobj accessors and updaters.</p> \u000A \u000A <p>But we are getting ahead of ourselves.  To start the stobj tour, see <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see>.</p>"],

 "ACL2____STATE":[["Programming","Other"],"ACL2 Sources","ACL2","<p>Note: If you are interested in programming with state, see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see> after reading the information below.</p> \u000A \u000A <p>The ACL2 state object is used extensively in programming the ACL2 system, \u000A and has been used in other ACL2 programs as well.  However, most users, \u000A especially those interested in specification and verification (as opposed to \u000A programming <i>per se</i>), need not be aware of the role of the state object \u000A in ACL2, and will not write functions that use it explicitly.  We say more \u000A about this point at the end of this documentation topic.</p> \u000A \u000A <p>The ACL2 state object is an example of a single-threaded object or <see topic=\"ACL2____STOBJ\">stobj</see>.  ACL2 allows the user to define new single-threaded objects. \u000A Generally, ACL2 may need to access the ACL2 state but should not (cannot) \u000A change it except via a certain set of approved functions such as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  If you need a state-like object to which you have \u000A complete rights, you may want a <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>Key to the idea of our <v>state</v> is the notion of single-threadedness. \u000A For an explanation, see <see topic=\"ACL2____STOBJ\">stobj</see>.  The upshot of it is that <v>state</v> is \u000A a variable symbol with severe restrictions on its use, so that it can be \u000A passed into only certain functions in certain slots, and must be returned by \u000A those functions that ``modify'' it.  Henceforth, we do not discuss \u000A single-threaded objects in general (which the user can introduce with <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> and <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>) but one in particular, namely ACL2's \u000A <v>state</v> object.</p> \u000A \u000A <p>The <i>global table</i> is perhaps the most visible portion of the state \u000A object.  Using the interface functions <v>@</v> and <v>assign</v>, a user may bind \u000A global variables to the results of function evaluations (much as an Nqthm user \u000A exploits the Nqthm utility <v>r-loop</v>).  See <see topic=\"ACL2_____04\">@</see>, and see <see topic=\"ACL2____ASSIGN\">assign</see>. \u000A A particularly interesting global is <v>'current-acl2-world</v>, whose value is \u000A the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>ACL2 supports several facilities of a truly von Neumannesque state machine \u000A character, including file <see topic=\"ACL2____IO\">io</see> and global variables.  Logically speaking, \u000A the state is a true list of the 14 components described below.  There is a \u000A ``current'' state object at the top-level of the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop. \u000A This object is understood to be the value of what would otherwise be the free \u000A variable <v>state</v> appearing in top-level input.  When any <see topic=\"ACL2____COMMAND\">command</see> \u000A returns a state object as one of its values, that object becomes the new \u000A current state.  But ACL2 provides von Neumann style speed for state operations \u000A by maintaining only one physical (as opposed to logical) state object. \u000A Operations on the state are in fact destructive.  This implementation does not \u000A violate the applicative semantics because we enforce certain draconian \u000A syntactic rules regarding the use of state objects.  For example, one cannot \u000A ``hold on'' to an old state, access the components of a state arbitrarily, or \u000A ``modify'' a state object without passing it on to subsequent state-sensitive \u000A functions.</p> \u000A \u000A <p>Every routine that uses the state facilities (e.g. does <see topic=\"ACL2____IO\">io</see>, or calls \u000A a routine that does <see topic=\"ACL2____IO\">io</see>), must be passed a ``state object.'' And a \u000A routine must return a state object if the routine modifies the state in any \u000A way.  Rigid syntactic rules governing the use of state objects are enforced by \u000A the function <v>translate</v>, through which all ACL2 user input first passes. \u000A State objects can only be ``held'' in the formal parameter <v>state</v>, never \u000A in any other formal parameter and never in any structure (excepting a \u000A multiple-value return list field which is always a state object).  State \u000A objects can only be accessed with the primitives we specifically permit. \u000A Thus, for example, one cannot ask, in code to be executed, for the length of \u000A <v>state</v> or the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of <v>state</v>.  In the statement and proof of \u000A theorems, there are no syntactic rules prohibiting arbitrary treatment of \u000A state objects.</p> \u000A \u000A <p>Logically speaking, a state object is a true list whose members are as \u000A follows:</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>Open-input-channels</v>, an alist with keys that are symbols in package \u000A <v>&quot;ACL2-INPUT-CHANNEL&quot;</v>.  The value (<tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>) of each pair has the \u000A form <v>((:header type file-name open-time) . elements)</v>, where <v>type</v> is \u000A one of <v>:character</v>, <v>:byte</v>, or <v>:object</v> and <v>elements</v> is a \u000A list of things of the corresponding <v>type</v>, i.e. characters, integers of \u000A type <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> 255)</v>, or lisp objects in our theory.  <v>File-name</v> is a \u000A string.  <v>Open-time</v> is an integer.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p><v>Open-output-channels</v>, an alist with keys that are symbols in package \u000A <v>&quot;ACL2-OUTPUT-CHANNEL&quot;</v>.  The value of a pair has the form <v>((:header\u000A type file-name open-time) .  current-contents)</v>.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p><v>Global-table</v>, an alist associating symbols (to be used as ``global \u000A variables'') with values.  See <see topic=\"ACL2_____04\">@</see>, and see <see topic=\"ACL2____ASSIGN\">assign</see>.</p> \u000A \u000A <p><v>T-stack</v>, a list of arbitrary objects accessed and changed by the \u000A functions <v>aref-t-stack</v> and <v>aset-t-stack</v>.</p> \u000A \u000A <p><v>32-bit-integer-stack</v>, a list of arbitrary 32-bit-integers accessed and \u000A changed by the functions <v>aref-32-bit-integer-stack</v> and \u000A <v>aset-32-bit-integer-stack</v>.</p> \u000A \u000A <p><v>Big-clock-entry</v>, an integer, that is used logically to bound the \u000A amount of effort spent to evaluate a quoted form.</p> \u000A \u000A <p><v>Idates</v>, a list of dates and times, used to implement the function \u000A <v>print-current-idate</v>, which prints the date and time.</p> \u000A \u000A <p><v>Acl2-oracle</v>, a list of objects, used for example to implement the \u000A functions that let ACL2 report how much time was used, but inaccessible to the \u000A user.  See <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt> and also see <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>.</p> \u000A \u000A <p><v>File-clock</v>, an integer that is increased on every file opening and \u000A closing, and is used to maintain the consistency of the <tt><see topic=\"ACL2____IO\">io</see></tt> \u000A primitives.</p> \u000A \u000A <p><v>Readable-files</v>, an alist whose keys have the form <v>(<see topic=\"COMMON-LISP____STRING\">string</see> type\u000A time)</v>, where <tt><see topic=\"COMMON-LISP____STRING\">string</see></tt> is a file name and <v>time</v> is an integer.  The \u000A value associated with such a key is a list of characters, bytes, or objects, \u000A according to <v>type</v>.  The <v>time</v> field is used in the following way: \u000A when it comes time to open a file for input, we will only look for a file of \u000A the specified name and <v>type</v> whose time field is that of <v>file-clock</v>. \u000A This permits us to have a ``probe-file'' aspect to <v>open-file</v>: one can ask \u000A for a file, find it does not exist, but come back later and find that it does \u000A now exist.</p> \u000A \u000A <p><v>Written-files</v>, an alist whose keys have the form <v>(<see topic=\"COMMON-LISP____STRING\">string</see> type time1\u000A time2)</v>, where <tt><see topic=\"COMMON-LISP____STRING\">string</see></tt> is a file name, <v>type</v> is one of \u000A <v>:character</v>, <v>:byte</v> or <v>:object</v>, and <v>time1</v> and <v>time2</v> are \u000A integers.  <v>Time1</v> and <v>time2</v> correspond to the <v>file-clock</v> time at \u000A which the channel for the file was opened and closed.  This field is \u000A write-only; the only operation that affects this field is \u000A <v>close-output-channel</v>, which <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es a new entry on the front.</p> \u000A \u000A <p><v>Read-files</v>, a list of the form <v>(<see topic=\"COMMON-LISP____STRING\">string</see> type time1 time2)</v>, where \u000A <tt><see topic=\"COMMON-LISP____STRING\">string</see></tt> is a file name and <v>time1</v> and <v>time2</v> were the times at \u000A which the file was opened for reading and closed.  This field is write \u000A only.</p> \u000A \u000A <p><v>Writeable-files</v>, an alist whose keys have the form <v>(<see topic=\"COMMON-LISP____STRING\">string</see> type\u000A time)</v>.  To open a file for output, we require that the name, type, and time \u000A be on this list.</p> \u000A \u000A <p><v>List-all-package-names-lst</v>, a list of <v>true-listps</v>.  Roughly \u000A speaking, the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of this list is the list of all package names known \u000A to this Common Lisp right now and the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of this list is the value of \u000A this <v>state</v> variable after you look at its <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>.  The function, \u000A <v>list-all-package-names</v>, which takes the state as an argument, returns the \u000A <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s the list (returning a new state too).  This \u000A essentially gives ACL2 access to what is provided by CLTL's \u000A <v>list-all-packages</v>.  <tt><see topic=\"ACL2____DEFPKG\">Defpkg</see></tt> uses this feature to ensure that the \u000A about-to-be-created package is new in this lisp.  Thus, for example, in \u000A <v>akcl</v> it is impossible to create the package <v>&quot;COMPILER&quot;</v> with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> because it is on the list, while in Lucid that package name is not \u000A initially on the list.</p> \u000A \u000A <p><v>User-stobj-alist</v>, an alist which associates user-defined \u000A single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see>) with their values.</p></blockquote> \u000A \u000A <p>We recommend avoiding the use of the state object when writing ACL2 code \u000A intended to be used as a formal model of some system, for several reasons. \u000A First, the state object is complicated and contains many components that are \u000A oriented toward implementation and are likely to be irrelevant to the model in \u000A question.  Second, there is currently not much support for reasoning about \u000A ACL2 functions that manipulate the state object, beyond their logical \u000A definitions.  Third, the documentation about state is not as complete as one \u000A might wish.</p> \u000A \u000A <p>User-defined single-threaded objects offer the speed of <v>state</v> while \u000A giving the user complete access to all the fields.  See <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>Again, if you are interested in programming with state see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.</p>"],

 "ACL2____RUNE":[["Theories"],"ACL2 Sources","ACL2","<code>Examples:\u000A(:rewrite assoc-of-app)\u000A(:linear delta-aref . 2)\u000A(:definition length)\u000A(:executable-counterpart length)</code> \u000A \u000A <p>Note: This topic discusses a basic notion of ``rule name'', or ``rune'' for \u000A short.  Users often use abbrevitions for runes; for example, a <see topic=\"ACL2____THEORY\">theory</see> \u000A expression <v>(<see topic=\"ACL2____DISABLE\">DISABLE</see> APPEND)</v> abbreviates the following set of runes: \u000A <v>{(:DEFINITION BINARY-APPEND), (:INDUCTION BINARY-APPEND)}</v>.  See <see topic=\"ACL2____THEORIES\">theories</see> for a discussion of so-called ``runic designators'', which include \u000A expressions like <v>APPEND</v> (as above) as well as <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see>)</v> (for the \u000A executable-counterpart of <v>BINARY-APPEND</v>.  Runic designators can also be \u000A abbreviations including <v>(:d APPEND)</v>, <v>(:e APPEND)</v>, <v>(:i APPEND)</v>, \u000A and <v>(:t APPEND)</v>, which designate the definition, executable-counterpart, \u000A induction, and type-prescription rules for <v>BINARY-APPEND</v>.  For a complete \u000A description of runic designators, see <see topic=\"ACL2____THEORIES\">theories</see>; we return now to the \u000A more basic notion of a rune.</p> \u000A \u000A <p>Background: The theorem prover is driven from a database of rules.  The \u000A most common rules are <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules, which cause the simplifier \u000A to replace one term with another.  <see topic=\"ACL2____EVENTS\">Events</see> introduce rules into the \u000A database.  For example, a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event may introduce runes for \u000A symbolically replacing a function call by its instantiated body, for \u000A evaluating the function on constants, for determining the type of a call of \u000A the function, and for the induction scheme introduced upon defining the \u000A function.  <tt><see topic=\"ACL2____DEFTHM\">Defthm</see></tt> may introduce several rules, one for each of the \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> specified (where one rule class is specified if \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> is omitted, namely, <v>:rewrite</v>).</p> \u000A \u000A <p>Every rule in the system has a name.  Each name is a structured object \u000A called a ``rune,'' which is short for ``rule name''.  Runes are always of the \u000A form <v>(:token symbol . x)</v>, where <v>:token</v> is some keyword symbol \u000A indicating what kind of rule is named, <v>symbol</v> is the event name that \u000A created the rule (and is called the ``base symbol'' of the rune), and <v>x</v> \u000A is either <v>nil</v> or a natural number that makes the rule name distinct from \u000A that of rules generated by other <see topic=\"ACL2____EVENTS\">events</see> or by other <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> within the same event.</p> \u000A \u000A <p>For example, an event of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> name thm\u000A  :rule-classes ((:REWRITE :COROLLARY term1)\u000A                 (:REWRITE :COROLLARY term2)\u000A                 (:ELIM    :COROLLARY term3)))</code> \u000A \u000A <p>typically creates three rules, each with a unique rune.  The runes are</p> \u000A \u000A <code>(:REWRITE name . 1), (:REWRITE name . 2), and (:ELIM name).</code> \u000A \u000A <p>However, a given formula may create more than one rule, and all rules \u000A generated by the same <v>:corollary</v> formula will share the same rune. \u000A Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-thm\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar x)) x)\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo x)) x)))</code> \u000A \u000A <p>This is treated identically to the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-thm\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar x)) x)\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo x)) x))\u000A  :rule-classes ((:rewrite\u000A                  :corollary\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar x)) x)\u000A                       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo x)) x)))))</code> \u000A \u000A <p>In either case, two rules are created: one rewriting <v>(foo (bar x))</v> to \u000A <v>x</v>, and one rewriting <v>(bar (foo x))</v> to <v>x</v>.  However, only a \u000A single rune is created, <v>(:REWRITE MY-THM)</v>, because there is only one rule \u000A class.  But now consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-thm2\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar x)) x)\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo x)) x))\u000A  :rule-classes ((:rewrite\u000A                  :corollary\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar x)) x)\u000A                       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo x)) x)))\u000A                 (:rewrite\u000A                  :corollary\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (bar (foo x))) (foo x))\u000A                       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (foo (bar x))) (bar x))))))</code> \u000A \u000A <p>This time there are four rules created.  The first two rules are as before, \u000A and are assigned the rune <v>(:REWRITE MY-THM . 1)</v>.  The other two rules are \u000A similarly generated for the second <v>:corollary</v>, and are assigned the rune \u000A <v>(:REWRITE MY-THM . 2)</v>.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> will return the <see topic=\"ACL2____COROLLARY\">corollary</see> term \u000A associated with a given rune in a given <see topic=\"ACL2____WORLD\">world</see>.  Example:</p> \u000A \u000A <code>(<see topic=\"ACL2____COROLLARY\">corollary</see> '(:TYPE-PRESCRIPTION DIGIT-TO-CHAR) (w state))</code> \u000A \u000A <p>However, the preferred way to see the corollary term associated with a rune \u000A or a name is to use <v>:pf</v>; see <see topic=\"ACL2____PF\">pf</see>.</p> \u000A \u000A <p>The <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event creates as many as four rules.  <v>(:definition\u000A fn)</v> is the rune given to the equality axiom defining the function, <v>fn</v>. \u000A <v>(:executable-counterpart fn)</v> is the rune given to the rule for computing \u000A <v>fn</v> on known arguments.  A type prescription rule may be created under the \u000A name <v>(:type-prescription fn)</v>, and an <see topic=\"ACL2____INDUCTION\">induction</see> rule may be created \u000A under the name <v>(:induction fn)</v>.</p> \u000A \u000A <p>Runes may be individually <see topic=\"ACL2____ENABLE\">enable</see>d and <see topic=\"ACL2____DISABLE\">disable</see>d, according to \u000A whether they are included in the current theory.  See <see topic=\"ACL2____THEORIES\">theories</see>.  Thus, \u000A it is permitted to <see topic=\"ACL2____DISABLE\">disable</see> <v>(:elim name)</v>, say, while enabling the \u000A other rules derived from name.  Similarly, <v>(:definition fn)</v> may be <see topic=\"ACL2____DISABLE\">disable</see>d while <v>(:executable-counterpart fn)</v> and the type prescriptions \u000A for <v>fn</v> are <see topic=\"ACL2____ENABLE\">enable</see>d.</p> \u000A \u000A <p>Associated with most runes is the formula justifying the rule named.  This \u000A is called the ``<see topic=\"ACL2____COROLLARY\">corollary</see> formula'' of the rune and may be obtained via \u000A the function <tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt>, which takes as its argument a rune and a \u000A property list <see topic=\"ACL2____WORLD\">world</see>.  Also see <see topic=\"ACL2____PF\">pf</see>.  The <see topic=\"ACL2____COROLLARY\">corollary</see> formula \u000A for <v>(:rewrite name . 1)</v> after the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event above is \u000A <v>term1</v>.  The corollary formulas for <v>(:definition fn)</v> and \u000A <v>(:executable-counterpart fn)</v> are always identical: the defining axiom. \u000A Some runes, e.g., <v>(:definition car)</v>, do not have corollary formulas. \u000A <tt><see topic=\"ACL2____COROLLARY\">Corollary</see></tt> returns <v>nil</v> on such runes.  In any case, the corollary \u000A formula of a rune, when it is non-<v>nil</v>, is a theorem and may be used in \u000A the <v>:use</v> and <v>:by</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>Note: The system has built-in rules that, for regularity, ought to have \u000A names but don't because they can never be <see topic=\"ACL2____DISABLE\">disable</see>d.  One such rule is \u000A that implemented by the <see topic=\"ACL2____LINEAR\">linear</see> arithmetic package.  Because many of our \u000A subroutines are required by their calling conventions to return the justifying \u000A rune, we have invented the notion of ``fake runes.''  Fake runes always have \u000A the base symbol <v>nil</v>, use a keyword token that includes the phrase \u000A ``fake-rune'', and are always <see topic=\"ACL2____ENABLE\">enable</see>d.  Here is the list of fake \u000A runes.</p> \u000A \u000A <p><code>((:fake-rune-for-linear nil)\u000A (:fake-rune-for-type-set nil)\u000A (:fake-rune-for-nu-rewriter nil))</code></p> \u000A \u000A <p>Occasionally the system will print a fake rune where a rune is expected. \u000A For example, when <v>(:FAKE-RUNE-FOR-LINEAR NIL)</v> is reported among the rules \u000A used in a proof, it is an indication that the linear arithmetic package was \u000A used.  However, fake runes are not allowed in <see topic=\"ACL2____THEORIES\">theories</see>, they cannot be \u000A <see topic=\"ACL2____ENABLE\">enable</see>d or <see topic=\"ACL2____DISABLE\">disable</see>d, and they do not have associated <see topic=\"ACL2____COROLLARY\">corollary</see> formulas.  In short, despite the fact that the user may sometimes \u000A see fake runes printed, they should never be typed.</p>"],

 "ACL2____RULE-CLASSES":[["ACL2"],"ACL2 Sources","ACL2","<code>Example Form (from community book finite-set-theory/total-ordering.lisp):\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> &lt;&lt;-trichotomy\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (ordinaryp x)\u000A                (ordinaryp y))\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (&lt;&lt; x y)\u000A               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)\u000A               (&lt;&lt; y x)))\u000A  :rule-classes\u000A  ((:rewrite :corollary\u000A             (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (ordinaryp x)\u000A                           (ordinaryp y)\u000A                           (<see topic=\"COMMON-LISP____NOT\">not</see> (&lt;&lt; x y))\u000A                           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)))\u000A                      (&lt;&lt; y x)))))\u000A\u000AGeneral Form:\u000Aa true list of rule class objects as defined below\u000A\u000ASpecial Cases:\u000Aa symbol abbreviating a single rule class object</code> \u000A \u000A <p>When <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> is used to prove a named theorem, rules may be derived \u000A from the proved formula and stored in the database.  The user specifies which \u000A kinds of rules are to be built, by providing a list of rule class <i>names</i> \u000A or, more generally, rule class <i>objects</i>, which name the kind of rule to \u000A build and optionally specify various attributes of the desired rule.  The rule \u000A class names are <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt>, <v>:</v><tt><see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____EQUIVALENCE\">equivalence</see></tt>, <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>, <v>:</v><tt><see topic=\"ACL2____GENERALIZE\">generalize</see></tt>, <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>, <v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt>, <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>, <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, <v>:</v><tt><see topic=\"ACL2____TYPE-SET-INVERTER\">type-set-inverter</see></tt>, and <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt>.  Some classes <i>require</i> the user-specification of \u000A certain class-specific attributes.  Each class of rule affects the theorem \u000A prover's behavior in a different way, as discussed in the corresponding \u000A documentation topic.  In this topic we discuss the various attributes that may \u000A be attached to rule classes.</p> \u000A \u000A <p>Note that not all <see topic=\"ACL2____EVENTS\">events</see> generate rules.  For example, a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event that specifies <v>:rule-classes nil</v> does not generate a rule. \u000A Similarly, a <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> event generates an axiom that can be referenced \u000A by name in <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>, but it does not generate a rule.</p> \u000A \u000A <p>A rule class object is either one of the <v>:class</v> keywords or else is a \u000A list of the form shown below.  Those fields marked with ``(!)''  are required \u000A when the <v>:class</v> is as indicated.</p> \u000A \u000A <code>(:class\u000A  :COROLLARY term\u000A  :TRIGGER-FNS (fn1 ... fnk) ; provided :class = :META (!)\u000A  :TRIGGER-TERMS (t1 ... tk) ; provided :class = :FORWARD-CHAINING\u000A                             ;       or :class = :LINEAR\u000A  :TYPE-SET n                ; provided :class = :TYPE-SET-INVERTER\u000A  :TYPED-TERM term           ; provided :class = :TYPE-PRESCRIPTION\u000A  :CLIQUE (fn1 ... fnk)      ; provided :class = :DEFINITION\u000A  :CONTROLLER-ALIST alist    ; provided :class = :DEFINITION\u000A  :INSTALL-BODY directive    ; provided :class = :DEFINITION\u000A  :LOOP-STOPPER alist        ; provided :class = :REWRITE\u000A  :PATTERN term              ; provided :class = :INDUCTION (!)\u000A  :CONDITION term            ; provided :class = :INDUCTION\u000A  :SCHEME term               ; provided :class = :INDUCTION (!)\u000A  :MATCH-FREE all-or-once    ; provided :class = :REWRITE\u000A                                     or :class = :LINEAR\u000A                                     or :class = :FORWARD-CHAINING\u000A  :BACKCHAIN-LIMIT-LST limit ; provided :class = :REWRITE\u000A                                     or :class = :META\u000A                                     or :class = :LINEAR\u000A                                     or :class = :TYPE-PRESCRIPTION\u000A  :HINTS hints               ; provided instrs = nil\u000A  :INSTRUCTIONS instrs       ; provided  hints = nil\u000A  :OTF-FLG flg)</code> \u000A \u000A <p>When rule class objects are provided by the user, most of the fields are \u000A optional and their values are computed in a context sensitive way.  When a \u000A <v>:class</v> keyword is used as a rule class object, all relevant fields are \u000A determined contextually.  Each rule class object in <v>:rule-classes</v> causes \u000A one or more rules to be added to the database.  The <v>:class</v> keywords are \u000A documented individually under the following names.  Note that when one of \u000A these names is used as a <v>:class</v>, it is expected to be in the keyword \u000A package (i.e., the names below should be preceded by a colon but the ACL2 \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> facilities do not permit us to use keywords below).</p> \u000A \u000A <p>See also <tt><see topic=\"ACL2____FORCE\">force</see></tt>, <see topic=\"ACL2____CASE-SPLIT\">case-split</see>, <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt>, and <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> for ``pragmas'' one can wrap around individual hypotheses of \u000A certain classes of rules to affect how the hypothesis is relieved.</p> \u000A \u000A <p>Before we get into the discussion of rule classes, let us return to an \u000A important point.  In spite of the large variety of rule classes available, at \u000A present we recommend that new ACL2 users rely almost exclusively on \u000A (conditional) rewrite rules.  A reasonable but slightly bolder approach is to \u000A use <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules \u000A for ``type-theoretic'' rules, especially ones whose top-level function symbol \u000A is a common one like <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> or <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>; see <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> and see <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see>.  However, the rest of the \u000A rule classes are really not intended for widespread use, but rather are mainly \u000A for experts.</p> \u000A \u000A <p>We expect that we will write more about the question of which kind of rule \u000A to use.  For now: when in doubt, use a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule.</p> \u000A \u000A <p><v>:Rule-classes</v> is an optional keyword argument of the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> \u000A (and <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>) event.  In the following, let <v>name</v> be the name of \u000A the event and let <v>thm</v> be the formula to be proved or added as an \u000A axiom.</p> \u000A \u000A <p>If <v>:rule-classes</v> is not specified in a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> (or <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>) event, it is as though what was specified was to make one or more \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules, i.e., as though <v>:rule-classes</v> \u000A <v>((:rewrite))</v> had been used.  Use <v>:rule-classes</v> <v>nil</v> to specify \u000A that no rules are to be generated.</p> \u000A \u000A <p>If <v>:rule-classes</v> class is specified, where class is a non-<v>nil</v> \u000A symbol, it is as though <v>:rule-classes</v> <v>((class))</v> had been used. \u000A Thus, <v>:rule-classes</v> <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> is equivalent to \u000A <v>:rule-classes</v> <v>((:forward-chaining))</v>.</p> \u000A \u000A <p>We therefore now consider <v>:rule-classes</v> as a true list.  If any \u000A element of that list is a keyword, replace it by the singleton list containing \u000A that keyword.  Thus, <v>:rule-classes</v> <v>(:rewrite :elim)</v> is the same as \u000A <v>:rule-classes</v> <v>((:rewrite) (:elim))</v>.</p> \u000A \u000A <p>Each element of the expanded value of <v>:rule-classes</v> must be a true \u000A list whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is one of the rule class keyword tokens listed above, \u000A e.g., <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt>, etc., and whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> \u000A is a ``keyword alist'' alternately listing keywords and values.  The keywords \u000A in this alist must be taken from those shown below.  They may be listed in any \u000A order and most may be omitted, as specified below.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____COROLLARY\">Corollary</see></tt> &mdash; its value, <v>term</v>, must be a term.  If \u000A omitted, this field defaults to <v>thm</v>.  The <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> of a \u000A rule class object is the formula actually used to justify the rule created and \u000A thus determines the form of the rule.  Nqthm provided no similar capability: \u000A each rule was determined by <v>thm</v>, the theorem or axiom added.  ACL2 \u000A permits <v>thm</v> to be stated ``elegantly'' and then allows the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> of a rule class object to specify how that elegant statement is to \u000A be interpreted as a rule.  For the rule class object to be well-formed, its \u000A (defaulted) <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt>, <v>term</v>, must follow from <v>thm</v>. \u000A Unless <v>term</v> follows trivially from <v>thm</v> using little more than \u000A propositional logic, the formula <v>(<see topic=\"ACL2____IMPLIES\">implies</see> thm term)</v> is submitted to the \u000A theorem prover and the proof attempt must be successful.  During that proof \u000A attempt the values of <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>, <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt>, and \u000A <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt>, as provided in the rule class object, are provided as \u000A arguments to the prover.  Such auxiliary proofs give the sort of output that \u000A one expects from the prover.  However, as noted above, corollaries that follow \u000A trivially are not submitted to the prover; thus, such corollaries cause no \u000A prover output.</p> \u000A \u000A <p>Note that before <v>term</v> is stored, all calls of macros in it are \u000A expanded away.  See <see topic=\"ACL2____TRANS\">trans</see>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____HINTS\">Hints</see></tt>, <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt>, <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> \u000A &mdash; the values of these fields must satisfy the same restrictions placed \u000A on the fields of the same names in <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  These values are passed to \u000A the recursive call of the prover used to establish that the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> of the rule class object follows from the theorem or axiom \u000A <v>thm</v>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____TYPE-SET\">Type-set</see></tt> &mdash; this field may be supplied only if the \u000A <v>:class</v> is <v>:</v><tt><see topic=\"ACL2____TYPE-SET-INVERTER\">type-set-inverter</see></tt>.  When provided, the value must \u000A be a type-set, an integer in a certain range.  If not provided, an attempt is \u000A made to compute it from the corollary.  See <see topic=\"ACL2____TYPE-SET-INVERTER\">type-set-inverter</see>.</p> \u000A \u000A <p><v>:Typed-term</v> &mdash; this field may be supplied only if the <v>:class</v> \u000A is <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>.  When provided, the value is the term for \u000A which the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> is a type-prescription lemma.  If no \u000A <v>:typed-term</v> is provided in a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule class \u000A object, we try to compute heuristically an acceptable term.  See <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>.</p> \u000A \u000A <p><v>:Trigger-terms</v> &mdash; this field may be supplied only if the \u000A <v>:class</v> is <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>.  When \u000A provided, the value is a list of terms, each of which is to trigger the \u000A attempted application of the rule.  If no <v>:trigger-terms</v> is provided, we \u000A attempt to compute heuristically an appropriate set of triggers.  See <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> or see <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p><v>:Trigger-fns</v> &mdash; this field must (and may only) be supplied if the \u000A <v>:class</v> is <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>.  Its value must be a list of function \u000A symbols (except that a macro alias can stand in for a function symbol; see \u000A <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>).  Terms with these symbols trigger the application of \u000A the rule.  See <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <p><v>:Clique</v> and <v>:controller-alist</v> &mdash; these two fields may only \u000A be supplied if the <v>:class</v> is <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>.  If they are \u000A omitted, then ACL2 will attempt to guess them.  Suppose the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> of the rule is <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyp (equiv (fn a1 ... an) body))</v>.  The \u000A value of the <v>:clique</v> field should be a true list of function symbols, and \u000A if non-<v>nil</v> must include <v>fn</v>.  These symbols are all the members of \u000A the mutually recursive clique containing this definition of <v>fn</v>.  That is, \u000A a call of any function in <v>:clique</v> is considered a ``recursive call'' for \u000A purposes of the expansion heuristics.  The value of the <v>:controller-alist</v> \u000A field should be an alist that maps each function symbol in the <v>:clique</v> to \u000A a list of <v>t</v>'s and <v>nil</v>'s of length equal to the arity of the \u000A function.  For example, if <v>:clique</v> consists of just two symbols, <v>fn1</v> \u000A and <v>fn2</v>, of arities <v>2</v> and <v>3</v> respectively, then <v>((fn1 t nil)\u000A (fn2 nil t t))</v> is a legal value of <v>:controller-alist</v>.  The value \u000A associated with a function symbol in this alist is a ``mask'' specifying which \u000A argument slots of the function ``control'' the recursion for heuristic \u000A purposes.  Sloppy choice of <v>:clique</v> or <v>:controller-alist</v> can result \u000A in infinite expansion and stack overflow.</p> \u000A \u000A <p><v>:Install-body</v> &mdash; this field may only be supplied if the \u000A <v>:class</v> is <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>.  Its value must be <v>t</v>, <v>nil</v>, \u000A or the default, <v>:normalize</v>.  A value of <v>t</v> or <v>:normalize</v> will \u000A cause ACL2 to install this rule as the new body of the function being \u000A ``defined'' (<v>fn</v> in the paragraph just above); hence this definition will \u000A be installed for future <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>.  Furthermore, if this field \u000A is omitted or the value is <v>:normalize</v>, then this definition will be \u000A simplified using the so-called ``normalization'' procedure that is used when \u000A processing definitions made with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  You must explicitly specify \u000A <v>:install-body nil</v> in the following cases: <v>fn</v> (as above) is a member \u000A of the value of constant <v>*definition-minimal-theory*</v>, the arguments are \u000A not a list of distinct variables, <v>equiv</v> (as above) is not <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, \u000A or there are free variables in the hypotheses or right-hand side (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).  However, supplying <v>:install-body nil</v> will not affect \u000A the rewriter's application of the <v>:definition</v> rule, other than to avoid \u000A using the rule to apply <v>:expand</v> hints.  If a definition rule equates \u000A <v>(f a1 ... ak)</v> with <v>body</v> but there are hypotheses, <v>hyps</v>, then \u000A <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see> will replace terms <v>(f term1 ... termk)</v> by \u000A corresponding terms <v>(<see topic=\"COMMON-LISP____IF\">if</see> hyps body (<see topic=\"ACL2____HIDE\">hide</see> (f term1 ... termk)))</v>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">Loop-stopper</see></tt> &mdash; this field may only be supplied if the \u000A class is <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>.  Its value must be a list of entries each \u000A consisting of two variables followed by a (possibly empty) list of function \u000A symbols, for example <v>((x y binary-+) (<see topic=\"ACL2____U\">u</see> v foo bar))</v>.  It will be used to \u000A restrict application of rewrite rules by requiring that the list of instances \u000A of the second variables must be ``smaller'' than the list of instances of the \u000A first variables in a sense related to the corresponding functions listed; see \u000A <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.  The list as a whole is allowed to be <v>nil</v>, \u000A indicating that no such restriction shall be made.  Note that any such entry \u000A that contains a variable not being instantiated, i.e., not occurring on the \u000A left side of the rewrite rule, will be ignored.  However, for simplicity we \u000A merely require that every variable mentioned should appear somewhere in the \u000A corresponding <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula.</p> \u000A \u000A <p><v>:Pattern</v>, <v>:Condition</v>, <v>:Scheme</v> &mdash; the first and last of \u000A these fields must (and may only) be supplied if the class is <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt>.  <v>:Condition</v> is optional but may only be supplied if the class \u000A is <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt>.  The values must all be terms and indicate, \u000A respectively, the pattern to which a new induction scheme is to be attached, \u000A the condition under which the suggestion is to be made, and a term which \u000A suggests the new scheme.  See <see topic=\"ACL2____INDUCTION\">induction</see>.</p> \u000A \u000A <p><v>:Match-free</v> &mdash; this field must be <v>:all</v> or <v>:once</v> and may \u000A be supplied only if the <v>:class</v> is either <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, or <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>.  (This field is not \u000A implemented for other rule classes, including the <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule class.)  See <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> for a description \u000A of this field.  Note: Although this field is intended to be used for \u000A controlling retries of matching free variables in hypotheses, it is legal to \u000A supply it even if there are no such free variables.  This can simplify the \u000A automated generation of rules, but note that when <v>:match-free</v> is \u000A supplied, the warning otherwise provided for the presence of free variables in \u000A hypotheses will be suppressed.</p> \u000A \u000A <p><v>:Backchain-limit-lst</v> &mdash; this field may be supplied only if the \u000A <v>:class</v> is either <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, or <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>.  It is further required either \u000A only one rule is generated from the formula or, at least, every such rule has \u000A the same list of hypotheses.  The value for <v>:backchain-limit-lst</v> must be \u000A <v>nil</v>; a non-negative integer; or, except in the case of <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> \u000A rules, a true list each element of which is either <v>nil</v> or a non-negative \u000A integer.  If it is a list, its length must be equal to the number of \u000A hypotheses of the rule and each item in the list is the ``backchain limit'' \u000A associated with the corresponding hypothesis.  If <v>backchain-limit-lst</v> is \u000A a non-negative integer, it is defaulted to a list of the appropriate number of \u000A repetitions of that integer.  The backchain limit of a hypothesis is used to \u000A limit the effort that ACL2 will expend when relieving the hypothesis.  If it \u000A is <v>NIL</v>, no new limits are imposed; if it is an integer, the hypothesis \u000A will be limited to backchaining at most that many times.  Note that \u000A backchaining may be further limited by a global <v>backchain-limit</v>; see \u000A <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> for details.  For different ways to reign in the \u000A rewriter, see <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see> and see <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see>. \u000A Jared Davis has pointed out that you can set the <v>:backchain-limit-lst</v> to \u000A 0 to avoid any attempt to relieve <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses, which can lead to \u000A a significant speed-up in some cases.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Once <v>thm</v> has been proved (in the case of <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>) and each rule \u000A class object has been checked for well-formedness (which might require \u000A additional proofs), we consider each rule class object in turn to generate and \u000A add rules.  Let <v>:class</v> be the class keyword token of the <v>i</v>th class \u000A object (counting from left to right).  Generate the <see topic=\"ACL2____RUNE\">rune</see> <v>(:class\u000A name . x)</v>, where <v>x</v> is <v>nil</v> if there is only one class and otherwise \u000A <v>x</v> is <v>i</v>.  Then, from the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> of that object, \u000A generate one or more rules, each of which has the name <v>(:class name . x)</v>. \u000A See the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> entry for each rule class to see how formulas \u000A determine rules.  Note that it is in principle possible for several rules to \u000A share the same name; it happens whenever a <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> determines \u000A more than one rule.  This in fact only occurs for <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> class rules and only \u000A then if the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> is essentially a conjunction.  (See the \u000A documentation for <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> \u000A for details.)</p>"],

 "ACL2____REWRITE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <p>This doc topic discusses the rule-class <v>:rewrite</v>.  If you want a \u000A general discussion of how rewriting works in ACL2 and some guidance on how to \u000A construct effective rewrite rules, see <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> and then see <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see>.</p> \u000A \u000A <code>Examples:\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> plus-commutes                 ; Replace (<see topic=\"COMMON-LISP_____B2\">+</see> a b) by (<see topic=\"COMMON-LISP_____B2\">+</see> b a) provided\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x)))            ; certain heuristics approve the\u000A                                      ; permutation.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> plus-commutes                 ; equivalent to the above\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x))\u000A  :rule-classes ((:rewrite :corollary (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x))\u000A                           :loop-stopper ((x y binary-+))\u000A                           :match-free :all)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-nil                    ; Replace (<see topic=\"COMMON-LISP____APPEND\">append</see> a nil) by a, if\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)             ; (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> a) rewrites to t.\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-nil                    ; as above, but with defaults and\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)             ; a backchain limit\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) x))\u000A  :rule-classes ((:rewrite :corollary (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                                               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) x))\u000A                           :backchain-limit-lst (3) ; or equivalently, 3\u000A                           :match-free :all)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append                 ; Replace (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)) by\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see>                            ; (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e b) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e c) in\u000A   (<see topic=\"COMMON-LISP____AND\">and</see>                               ; contexts in which propositional\u000A    (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)                    ; equivalence is sufficient, provided\u000A    (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))                   ; b and c are true-lists.\u000A   (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A        (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...hi...)\u000A              (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...hk...)\u000A                       (<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A                            (equiv lhs rhs)\u000A                            ...)))\u000A     ...)</code> \u000A \u000A <p>Note: One <v>:rewrite</v> rule class object might create many rewrite rules \u000A from the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula.  To create the rules, we first \u000A translate the formula, expanding all macros (see <see topic=\"ACL2____TRANS\">trans</see>) and also \u000A expanding away calls of all so-called ``guard holders,'' <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt> and \u000A <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> (the latter resulting for example from calls of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____MBE\">mbe</see></tt>, or <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>), as well as expansions of the macro \u000A `<tt><see topic=\"COMMON-LISP____THE\">the</see></tt>'.  Next, we eliminate all <v>lambda</v>s; one may think of this \u000A step as simply substituting away every <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>, <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> in the formula.  We then flatten the <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> and <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> \u000A structure of the formula; for example, if the hypothesis or conclusion is of \u000A the form <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> term1 term2) term3)</v>, then we replace that by the \u000A ``flat'' term <v>(<see topic=\"COMMON-LISP____AND\">and</see> term1 term2 term3)</v>.  (The latter is actually an \u000A abbreviation for the right-associated term <v>(<see topic=\"COMMON-LISP____AND\">and</see> term1 (<see topic=\"COMMON-LISP____AND\">and</see> term2\u000A term3))</v>.)  The result is a conjunction of formulas, each of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> h1 ... hn) concl)</code> \u000A \u000A <p>where no hypothesis is a conjunction and <v>concl</v> is neither a \u000A conjunction nor an implication.  If necessary, the hypothesis of such a \u000A conjunct may be vacuous.  We then further coerce each <v>concl</v> into the form \u000A <v>(equiv lhs rhs)</v>, where <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation, \u000A by replacing any <v>concl</v> not of that form by <v>(<see topic=\"ACL2____IFF\">iff</see> concl t)</v>.  A \u000A <v>concl</v> of the form <v>(<see topic=\"COMMON-LISP____NOT\">not</see> term)</v> is considered to be of the form <v>(<see topic=\"ACL2____IFF\">iff</see>\u000A term nil)</v>.  By these steps we reduce the given <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> to a \u000A sequence of conjuncts, each of which is of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> h1 ... hn)\u000A         (equiv lhs rhs))</code> \u000A \u000A <p>where <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation.  See <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> for a general discussion of the introduction of new <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations.  At this point, we check whether <v>lhs</v> and <v>rhs</v> \u000A are the same term; if so, we cause an error, since this rule will loop.  (But \u000A this is just a basic check; the rule could loop in other cases, for example if \u000A <v>rhs</v> is an instance of <v>lhs</v>; see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.)</p> \u000A \u000A <p>We create a <v>:rewrite</v> rule for each such conjunct, if possible, and \u000A otherwise cause an error.  It is possible to create a rewrite rule from such a \u000A conjunct provided <v>lhs</v> is not a variable, a quoted constant, a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>-expression, a <v>lambda</v> application, or an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>-expression.</p> \u000A \u000A <p>A <v>:rewrite</v> rule is used when any instance of the <v>lhs</v> occurs in a \u000A context in which the <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation is an admissible <see topic=\"ACL2____CONGRUENCE\">congruence</see> relation.  First, we find a substitution that makes <v>lhs</v> equal \u000A to the target term.  Then we attempt to relieve the instantiated hypotheses of \u000A the rule.  Hypotheses that are fully instantiated are relieved by recursive \u000A rewriting.  Hypotheses that contain ``free variables'' (variables not assigned \u000A by the unifying substitution) are relieved by attempting to guess a suitable \u000A instance so as to make the hypothesis equal to some known assumption in the \u000A context of the target.  If the hypotheses are relieved, and certain \u000A restrictions that prevent some forms of infinite regress are met (see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>), the target is replaced by the instantiated <v>rhs</v>, which is \u000A then recursively rewritten.</p> \u000A \u000A <p>ACL2's rewriting process has undergone some optimization.  In particular, \u000A when a term <v>t1</v> is rewritten to a new term <v>t2</v>, the rewriter is then \u000A immediately applied to <v>t2</v>.  On rare occasions you may find that you do \u000A not want this behavior, in which case you may wish to use a trick involving \u000A <tt><see topic=\"ACL2____HIDE\">hide</see></tt>; see <see topic=\"ACL2____META\">meta</see>, near the end of that documentation.</p> \u000A \u000A <p>In another optimization, when the hypotheses and right-hand side are \u000A rewritten, ACL2 does not really first apply the substitution and then rewrite; \u000A instead, it as it rewrites those terms it looks up the <i>already \u000A rewritten</i> values of the bound variables.  Sometimes you may want those \u000A bindings rewritten again, e.g., because the variables occur in slots that \u000A admit additional equivalence relations.  See <i>double-rewrite</i>.</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> and see <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see> for an extended discussion of how to \u000A create effective rewrite rules.</p>"],

 "ACL2____MAKE-EVENT":[["Events","Macros"],"ACL2 Sources","ACL2","<p><v>Make-event</v> is a utility for generating <see topic=\"ACL2____EVENTS\">events</see>.  It \u000A provides a capability not offered by Lisp macros (see <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>), as it \u000A allows access to the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> and logical <see topic=\"ACL2____WORLD\">world</see>.  In essence, \u000A the expression <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form)</v> replaces itself with the result of \u000A evaluating <v>form</v>, say, <v>ev</v>, as though one had submitted <v>ev</v> \u000A instead of the <v>make-event</v> call.  For example, \u000A <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)))</v> is equivalent to the event <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A f (x) x)</v>.</p> \u000A \u000A <p>We break this documentation into the following sections.</p> \u000A \u000A <p><b>Introduction</b><br></br> \u000A \u000A <b>Detailed Documentation</b><br></br> \u000A \u000A <b>Error Reporting</b><br></br> \u000A \u000A <b>Restriction to Event Contexts</b><br></br> \u000A \u000A <b>Examples Illustrating How to Access State</b><br></br> \u000A \u000A <b>Advanced Expansion Control</b></p> \u000A \u000A <p>We begin with an informal introduction, which focuses on examples and \u000A introduces the key notion of ``expansion phase''.</p> \u000A \u000A <p><b>Introduction</b></p> \u000A \u000A <p><v>Make-event</v> is particularly useful for those who program using the ACL2 \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>; see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.  That is because the \u000A evaluation of <v>form</v> may read and even modify the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>Suppose for example that we want to define a constant <v>*world-length*</v>, \u000A that is the length of the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.  A <v>make-event</v> form \u000A can accomplish this task, as follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98883\u000AACL2 !&gt;(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> 'defconst '*world-length* (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))))\u000A\u000ASummary\u000AForm:  ( DEFCONST *WORLD-LENGTH* ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000ASummary\u000AForm:  ( MAKE-EVENT (<see topic=\"COMMON-LISP____LIST\">LIST</see> ...))\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A *WORLD-LENGTH*\u000AACL2 !&gt;*world-length*\u000A98883\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98890\u000AACL2 !&gt;</code> \u000A \u000A <p>How did this work?  First, evaluation of the form <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 'defconst\u000A '*world-length* (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)))</v> returned the event form <v>(<see topic=\"ACL2____DEFCONST\">defconst</see>\u000A *world-length* 98883)</v>.  Then that event form was automatically submitted to \u000A ACL2.  Of course, that changed the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, which is why the \u000A final value of <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))</v> is greater than its initial value.</p> \u000A \u000A <p>The example above illustrates how the evaluation of a <v>make-event</v> call \u000A takes place in two phases.  The first phase evaluates the argument of the \u000A call, in this case <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 'defconst '*world-length* (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)))</v>, \u000A to compute an event form, in this case <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *world-length* 98883)</v>. \u000A We call this evaluation the ``expansion'' phase.  Then the resulting event \u000A form is evaluated, which in this case defines the constant \u000A <v>*world-length*</v>.</p> \u000A \u000A <p>Now suppose we would like to introduce such a <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> form any \u000A time we like.  It is common practice to define macros to automate such tasks. \u000A Now we might be tempted simply to make the following definition.</p> \u000A \u000A <code>; WRONG!\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> define-world-length-constant (<see topic=\"ACL2____NAME\">name</see> state)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'defconst name (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))))</code> \u000A \u000A <p>But ACL2 rejects such a definition, because a macro cannot take the ACL2 \u000A state as a parameter; instead, the formal parameter to this macro named \u000A <v>&quot;STATE&quot;</v> merely represents an ordinary object.  You can try to \u000A experiment with other such direct methods to define such a macro, but they \u000A won't work.</p> \u000A \u000A <p>Instead, however, you can use the approach illustrated by the \u000A <v>make-event</v> example above to define the desired macro, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> define-world-length-constant (<see topic=\"ACL2____NAME\">name</see>)\u000A  `(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"COMMON-LISP____LIST\">list</see> 'defconst ',name (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)))))</code> \u000A \u000A <p>Here are example uses of this macro.</p> \u000A \u000A <code>ACL2 !&gt;(define-world-length-constant *foo*)\u000A\u000ASummary\u000AForm:  ( DEFCONST *FOO* ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000ASummary\u000AForm:  ( MAKE-EVENT (<see topic=\"COMMON-LISP____LIST\">LIST</see> ...))\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A *FOO*\u000AACL2 !&gt;*foo*\u000A98891\u000AACL2 !&gt;:pe *foo*\u000A          2:x(DEFINE-WORLD-LENGTH-CONSTANT *FOO*)\u000A\u000A&gt;             (<see topic=\"ACL2____DEFCONST\">DEFCONST</see> *FOO* 98891)\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98897\u000AACL2 !&gt;(define-world-length-constant *bar*)\u000A\u000ASummary\u000AForm:  ( DEFCONST *BAR* ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000ASummary\u000AForm:  ( MAKE-EVENT (<see topic=\"COMMON-LISP____LIST\">LIST</see> ...))\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A *BAR*\u000AACL2 !&gt;*bar*\u000A98897\u000AACL2 !&gt;:pe *bar*\u000A          3:x(DEFINE-WORLD-LENGTH-CONSTANT *BAR*)\u000A\u000A&gt;             (<see topic=\"ACL2____DEFCONST\">DEFCONST</see> *BAR* 98897)\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98903\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally, we note that the expansion phase can be used for computation that \u000A has side effects, generally by modifying state.  Here is a modification of the \u000A above example that does not change the world at all, but instead saves the \u000A length of the world in a state global.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'my-world-length (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)) state)\u000A         (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))</code> \u000A \u000A <p>Notice that this time, the value returned by the expansion phase is not an \u000A event form, but rather, is an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) whose \u000A value component is an event form, namely, the event form <v>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see>\u000A nil)</v>.  Evaluation of that event form does not change the ACL2 world (see \u000A <see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see>).  Thus, the sole purpose of the <v>make-event</v> call \u000A above is to change the <see topic=\"ACL2____STATE\">state</see> by associating the length of the current \u000A logical world with the state global named <v>'my-world-length</v>.  After \u000A evaluating this form, <v>(<see topic=\"ACL2_____04\">@</see> my-world-length)</v> provides the length of the ACL2 \u000A world, as illustrated by the following transcript.</p> \u000A \u000A <code>ACL2 !&gt;:pbt 0\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98883\u000AACL2 !&gt;(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A        (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'my-world-length (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)) state)\u000A                (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))\u000A\u000ASummary\u000AForm:  ( MAKE-EVENT (<see topic=\"ACL2____PPROGN\">PPROGN</see> ...))\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A NIL\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))\u000A98883\u000AACL2 !&gt;:pbt 0\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 !&gt;</code> \u000A \u000A <p>When <v>make-event</v> is invoked by a book, it is expanded during book \u000A certification but not, by default, when the book is included.  So for the \u000A example <v>(define-world-length-constant *foo*)</v> given above, if that form is \u000A in a book, then the value of <v>*foo*</v> will be the length of the world at the \u000A time this form was invoked during book certification, regardless of world \u000A length at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> time.  (The expansion is recorded in the book's \u000A <see topic=\"ACL2____CERTIFICATE\">certificate</see>, and re-used.)  To overcome this default, you can specify \u000A keyword value <v>:CHECK-EXPANSION t</v>.  This will cause an error if the \u000A expansion is different, but it can be useful for side effects.  For example, \u000A if you insert the following form in a book, then the length of the world will \u000A be printed when the form is encountered, whether during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A or during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____FMS\">fms</see> &quot;Length of current world: ~x0~|&quot;\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))))\u000A              *standard-co* state nil)\u000A         (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A :check-expansion t)</code> \u000A \u000A <p><b>Detailed Documentation</b></p> \u000A \u000A <code>Examples:\u000A\u000A; Trivial example: evaluate (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)) to obtain\u000A; (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x), which is then evaluated.\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)))\u000A\u000A; Evaluate (generate-form state) to obtain (<see topic=\"ACL2____MV\">mv</see> nil val state), and\u000A; then evaluate val.  (Generate-form is not specified here, but\u000A; imagine for example that it explores the state and then generates\u000A; some desired definition or theorem.)\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (generate-form state))\u000A\u000A; As above, but make sure that if this form is in a book, then when\u000A; we include the book, the evaluation of (generate-form state)\u000A; should return the same value as it did when the book was\u000A; certified.\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (generate-form state)\u000A            :CHECK-EXPANSION t)\u000A\u000A; As above (where the :CHECK-EXPANSION value can be included or\u000A; not), where if there is an error during expansion, then the error\u000A; message will explain that expansion was on behalf of the indicated\u000A; object, typically specified as the first argument.\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (generate-form state)\u000A            :ON-BEHALF-OF (generate-form state))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :CHECK-EXPANSION chk :ON-BEHALF-OF obj :EXPANSION? form)</code> \u000A \u000A <p>where <v>chk</v> is <v>nil</v> (the default), <v>t</v>, or the intended \u000A ``expansion result'' from the evaluation of <v>form</v> (as explained below); \u000A and if supplied, <v>obj</v> is an arbitrary ACL2 object, used only in reporting \u000A errors in expansion, i.e., in the evaluation of form.  The <v>:EXPANSION?</v> \u000A keyword is discussed in the final section, on Advanced Expansion Control.</p> \u000A \u000A <p>We strongly recommend that you browse some <v>.lisp</v> files in the \u000A community books directory <v>books/make-event/</v>.  You may even find it \u000A helpful, in order to understand <v>make-event</v>, to do so before continuing to \u000A read this documentation.  You may also find it useful to browse community book \u000A <v>books/misc/eval.lisp</v>, which contains definitions of macros \u000A <v>must-succeed</v> and <v>must-fail</v> that are useful for testing and are used \u000A in many books in the <v>books/make-event/</v> directory, especially \u000A <v>eval-tests.lisp</v>.  Another example, <v>books/make-event/defrule.lisp</v>, \u000A shows how to use macros whose calls expand to <v>make-event</v> forms, which in \u000A turn can generate <see topic=\"ACL2____EVENTS\">events</see>.  For more examples, see file \u000A <v>books/make-event/Readme.lsp</v>.  Other than the examples, the explanations \u000A here should suffice for most users.  If you want explanations of subtler \u000A details, see <see topic=\"ACL2____MAKE-EVENT-DETAILS\">make-event-details</see>.</p> \u000A \u000A <p>Note that <v>make-event</v> may only be used at the ``top level'' or where an \u000A event is expected.  See the section ``Restriction to Event Contexts'', \u000A below.</p> \u000A \u000A <p><v>Make-event</v> is related to Lisp macroexpansion in the sense that its \u000A argument is evaluated to obtain an expansion result, which is evaluated again. \u000A Let us elaborate on each of these notions in turn: ``is evaluated,'' \u000A ``expansion result'', and ``evaluated again.''  The final section, on Advanced \u000A Expansion Control, will generalize these processes in a way that we ignore for \u000A now.</p> \u000A \u000A <blockquote> \u000A \u000A <p>``is evaluated'' &mdash; The argument can be any expression, which is \u000A evaluated as would be any expression submitted to ACL2's top level loop. \u000A Thus, <tt><see topic=\"ACL2____STATE\">state</see></tt> and user-defined <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s may appear in the form \u000A supplied to <v>make-event</v>.  Henceforth, we will refer to this evaluation as \u000A ``expansion.''  Expansion is actually done in a way that restores ACL2's \u000A built-in <tt><see topic=\"ACL2____STATE\">state</see></tt> global variables, including the logical <see topic=\"ACL2____WORLD\">world</see>, \u000A to their pre-expansion values (with a few exceptions &mdash; see <see topic=\"ACL2____MAKE-EVENT-DETAILS\">make-event-details</see> &mdash; and where we note that changes to user-defined \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> global variables (see <see topic=\"ACL2____ASSIGN\">assign</see>) are preserved).  So, for \u000A example, events might be evaluated during expansion, but they will disappear \u000A from the logical <see topic=\"ACL2____WORLD\">world</see> after expansion returns its result.  Moreover, \u000A proofs are enabled by default at the start of expansion (see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>) if keyword <v>:CHECK-EXPANSION</v> is supplied and has a \u000A non-<v>nil</v> value.</p> \u000A \u000A <p>``expansion result'' &mdash; The above expansion may result in an ordinary \u000A (non-<tt><see topic=\"ACL2____STATE\">state</see></tt>, non-<tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>) value, which we call the ``expansion \u000A result.''  Or, expansion may result in a multiple value of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp\u000A val state)</v>, or, more generally, <v>(<see topic=\"ACL2____MV\">mv</see> erp val state stobj-1 ... stobj-k)</v> \u000A where each <v>stobj-i</v> is a <see topic=\"ACL2____STOBJ\">stobj</see>; then the expansion result is \u000A <v>val</v> unless <v>erp</v> is not <v>nil</v>, in which case there is no expansion \u000A result, and the original <v>make-event</v> evaluates to a soft error.  In either \u000A case (single or multiple value), either <v>val</v> is an embedded event form \u000A (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), or else the original <v>make-event</v> \u000A evaluates to a soft error, printed as described under ``Error Reporting'' \u000A below.</p> \u000A \u000A <p>``evaluated again'' &mdash; the expansion result is evaluated in place of \u000A the original <v>make-event</v>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>The expansion process can invoke subsidiary calls of <v>make-event</v>, and \u000A the expansion result can (perhaps after macroexpansion) be a call of \u000A <v>make-event</v>.  It can be useful to track all these <v>make-event</v> calls. \u000A The <see topic=\"ACL2____STATE\">state</see> global variable <v>make-event-debug</v> may be set to a \u000A non-<v>nil</v> value, for example <v>(<see topic=\"ACL2____ASSIGN\">assign</see> make-event-debug t)</v>, in order to \u000A see a trace of the expansion process, where a level is displayed (as in \u000A ``<v>3&gt;</v>'') to indicate the depth of subsidiary expansions.</p> \u000A \u000A <p>Expansion of a <v>make-event</v> call will yield an event that replaces the \u000A original <v>make-event</v> call.  In particular, if you put a <v>make-event</v> \u000A form in a book, then in essence it is replaced by its expansion result, \u000A created during the proof pass of the <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> process.  We now \u000A elaborate on this idea of keeping the original expansion.</p> \u000A \u000A <p>A <v>make-event</v> call generates a ``<v>make-event</v> replacement'' that may \u000A be stored by the system.  In the simplest case, this replacement is the \u000A expansion result.  When a book is certified, these replacements are stored in \u000A a book's certificate (technically, in the <v>:EXPANSION-ALIST</v> field).  Thus, \u000A although the book is not textually altered during certification, one may \u000A imagine a ``book expansion'' corresponding to the original book, in which \u000A events are substituted by replacements that were generated during the proof \u000A phase of certification.  A subsequent <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> will then include \u000A the book expansion corresponding to the indicated book.  When a book is \u000A compiled during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, it is actually the corresponding book \u000A expansion, stored as a temporary file, that is compiled instead.  That \u000A temporary file is deleted after compilation unless one first evaluates the \u000A form <v>(<see topic=\"ACL2____ASSIGN\">assign</see> keep-tmp-files t)</v>.  Note however that all of the original \u000A forms must still be legal <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.  So \u000A for example, if the first event in a book is <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-id (x)\u000A x))</v>, and is followed by <v>(my-id (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> ...))</v>, the final \u000A ``<v>include-book</v>'' pass of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> will fail because \u000A <v>my-id</v> is not defined when the <v>my-id</v> call is encountered.</p> \u000A \u000A <p>A <v>make-event</v> replacement might not be the expansion when either of the \u000A keyword arguments <v>:CHECK-EXPANSION</v> or <v>:EXPANSION?</v> is supplied.  We \u000A deal with the latter in the final section, on Advanced Expansion Control.  If \u000A <v>:CHECK-EXPANSION t</v> is supplied and the expansion is <v>exp</v>, then the \u000A replacement is obtained from the original <v>make-event</v> call, by \u000A substituting <v>exp</v> for <v>t</v> as the value of keyword \u000A <v>:CHECK-EXPANSION</v>.  Such a <v>make-event</v> call &mdash; during the second \u000A pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, or during event processing on behalf of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> other than when including a book near the end of its \u000A certification process &mdash; will do the expansion again and check that the \u000A expansion result is equal to the original expansion result, <v>exp</v>.  In the \u000A unusual case that you know the expected expansion result, <v>res</v>, you can \u000A specify <v>:CHECK-EXPANSION res</v> in the first place, so that the check is \u000A also done during the initial evaluation of the <v>make-event</v> form. \u000A IMPORTANT BUT OBSCURE DETAIL: That expansion check is only done when \u000A processing events, not during a preliminary load of a book's compiled file. \u000A The following paragraph elaborates.</p> \u000A \u000A <p>(Here are details on the point made just above, for those who use the \u000A <v>:CHECK-EXPANSION</v> argument to perform side-effects on the <see topic=\"ACL2____STATE\">state</see>. \u000A When you include a book, ACL2 generally loads a compiled file before \u000A processing the events in the book; see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.  While it is \u000A true that a non-<v>nil</v> <v>:CHECK-EXPANSION</v> argument causes <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to perform expansion of the <v>make-event</v> form during event \u000A processing it does <i>not</i> perform expansion when the compiled file (or \u000A expansion file; again, see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>) is loaded.)</p> \u000A \u000A <p>ACL2 performs the following space-saving optimization: when the expansion \u000A result is a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> event, then the <v>make-event</v> replacement is \u000A <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :ELIDED))</v>.</p> \u000A \u000A <p>The notion of ``expansion'' and ``replacement'' extend to the case that a \u000A call of <v>make-event</v> is found in the course of macroexpansion.  The \u000A following example illustrates this point.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-mac ()\u000A   '(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)))\u000A (my-mac))\u000A:pe :here</code> \u000A \u000A <p>The above call of <tt><see topic=\"ACL2____PE\">pe</see></tt> shows that the form <v>(my-mac)</v> has a \u000A <v>make-event</v> expansion (and replacement) of <v>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> NIL\u000A             (<see topic=\"COMMON-LISP____DEFMACRO\">DEFMACRO</see> MY-MAC\u000A                       NIL\u000A                       '(<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> '(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)))\u000A             (RECORD-EXPANSION (MY-MAC)\u000A                               (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)))</code> \u000A \u000A <p><b>Error Reporting</b></p> \u000A \u000A <p>Suppose that expansion produces a soft error as described above.  That is, \u000A suppose that the argument of a <v>make-event</v> call evaluates to a multiple \u000A value <v>(<see topic=\"ACL2____MV\">mv</see> erp val state ...)</v> where <v>erp</v> is not <v>nil</v>.  If <v>erp</v> \u000A is a string, then that string is printed in the error message.  If <v>erp</v> is \u000A a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is a string, then the error prints \u000A <v>&quot;~@0&quot;</v> with <v>#\\0</v> bound to that <v>cons</v> pair; see <see topic=\"ACL2____FMT\">fmt</see>.  Any \u000A other non-<v>nil</v> value of <v>erp</v> causes a generic error message to be \u000A printed.</p> \u000A \u000A <p><b>Restriction to Event Contexts</b></p> \u000A \u000A <p>A <v>make-event</v> call must occur either at the top level, or during \u000A <v>make-event</v> expansion, or as an argument of an event constructor.  We \u000A explain in more detail below.  This restriction is imposed to enable ACL2 to \u000A track expansions produced by <v>make-event</v>.</p> \u000A \u000A <p>The following examples illustrate this restriction.</p> \u000A \u000A <code>; Legal:\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A        :on summary\u000A        (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))))\u000A\u000A; Illegal:\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))\u000A        (<see topic=\"ACL2____MV\">mv</see> erp val state))</code> \u000A \u000A <p>More precisely: a <v>make-event</v> call that is not itself evaluated during \u000A <v>make-event</v> expansion is subject to the following requirement.  After \u000A macroexpansion has taken place, such a <v>make-event</v> call must be in an \u000A ``event context'', defined recursively as follows.  (All but the first two \u000A cases below correspond to similar cases for constructing events; see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.)</p> \u000A \u000A <blockquote> \u000A \u000A <p>o A form submitted at the top level, or more generally, supplied to a call \u000A of <tt><see topic=\"ACL2____LD\">ld</see></tt>, is in an event context.</p> \u000A \u000A <p>o A form occurring at the top level of a book is in an event context.</p> \u000A \u000A <p>o If <v>(</v><tt><see topic=\"ACL2____LOCAL\">local</see></tt><v> x1)</v> is in an event context, then so is \u000A <v>x1</v>.</p> \u000A \u000A <p>o If <v>(</v><tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt><v> x1)</v> is in an event context, then so is \u000A <v>x1</v>.</p> \u000A \u000A <p>o If <v>(</v><tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt><v> x ...)</v> is in an event context and its \u000A expansion <v>x1</v> is an embedded event form, then <v>x1</v> is in an event \u000A context.</p> \u000A \u000A <p>o If <v>(</v><tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt><v> ... x1)</v>, <v>(</v><tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt><v> ... x1 ...)</v>, or <v>(</v><tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt><v> ... x1)</v> is in an event context, then so is \u000A <v>x1</v>.</p> \u000A \u000A <p>o For any call of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, each of its arguments is \u000A in an event context.</p> \u000A \u000A <p>o For any call of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, each of its arguments except the \u000A first (the signature list) is in an event context.</p> \u000A \u000A <p>o If <v>(RECORD-EXPANSION x1 x2)</v> is in an event context, then <v>x1</v> and \u000A <v>x2</v> are in event contexts.  Note: <v>record-expansion</v> is intended for \u000A use only by the implementation, which imposes the additional restriction that \u000A <v>x1</v> and its subsidiary <v>make-event</v> calls (if any) must specify a \u000A <v>:CHECK-EXPANSION</v> argument that is a <see topic=\"COMMON-LISP____CONSP\">consp</see>.</p></blockquote> \u000A \u000A <p>Low-level remark, for system implementors.  There is the one exception to \u000A the above restriction: a single <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt> form immediately \u000A under a <v>progn!</v> call.  For example:</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> &lt;bindings&gt; (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> ...)))</code> \u000A \u000A <p>However, the following form may be preferable (see <see topic=\"ACL2____PROGN_12\">progn!</see>):</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> :STATE-GLOBAL-BINDINGS &lt;bindings&gt; (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> ...))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> for an interesting use of this \u000A exception.</p> \u000A \u000A <p><b>Examples Illustrating How to Access State</b></p> \u000A \u000A <p>You can modify the ACL2 <see topic=\"ACL2____STATE\">state</see> by doing your state-changing \u000A computation during the expansion phase, before expansion returns the event \u000A that is submitted.  Here are some examples.</p> \u000A \u000A <p>First consider the following.  Notice that expansion modifies state global \u000A <v>my-global</v> during <v>make-event</v> expansion, and then expansion returns a \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event to be evaluated.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A  (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____ASSIGN\">assign</see> my-global (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)))\u000A            (value '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))))</code> \u000A \u000A <p>Then we get:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> my-global)\u000A72271\u000AACL2 !&gt;:pe foo\u000A L        1:x(<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> (<see topic=\"ACL2____ER-PROGN\">ER-PROGN</see> # #))\u000A\u000A&gt;L            (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> X X))\u000AACL2 !&gt;</code> \u000A \u000A <p>Here's a slightly fancier example, where the computation affects the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  In a new session, execute:</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A  (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____ASSIGN\">assign</see> my-global (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state)))\u000A            (value `(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x ,(<see topic=\"ACL2_____04\">@</see> my-global))))))</code> \u000A \u000A <p>Then:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> my-global)\u000A72271\u000AACL2 !&gt;:pe foo\u000A L        1:x(<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> (<see topic=\"ACL2____ER-PROGN\">ER-PROGN</see> # #))\u000A\u000A&gt;L            (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> X 72271))\u000AACL2 !&gt;</code> \u000A \u000A <p>Note that ACL2 <see topic=\"ACL2____TABLE\">table</see> <see topic=\"ACL2____EVENTS\">events</see> may avoid the need to use <see topic=\"ACL2____STATE\">state</see> globals.  For example, instead of the example above, consider this \u000A example in a new session.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((world-len (<see topic=\"COMMON-LISP____LENGTH\">length</see> (w state))))\u000A    `(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____TABLE\">table</see> my-table :STORED-WORLD-LENGTH ,world-len)\u000A            (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x ,world-len)))))</code> \u000A \u000A <p>Then:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TABLE\">table</see> my-table)\u000A ((:STORED-WORLD-LENGTH . 72271))\u000AACL2 !&gt;:pe foo\u000A          1:x(<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> (<see topic=\"COMMON-LISP____LET\">LET</see> # #))\u000A\u000A&gt;L            (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> X 72271))\u000AACL2 !&gt;</code> \u000A \u000A <p>By the way, most built-in <see topic=\"ACL2____STATE\">state</see> globals revert after expansion.  But \u000A your own global (like <v>my-global</v> above) can be set during expansion, and \u000A the new value will persist.</p> \u000A \u000A <p><b>Advanced Expansion Control</b></p> \u000A \u000A <p>We conclude this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> section by discussing three kinds of \u000A additional control over <v>make-event</v> expansion.  These are all illustrated \u000A in community book <v>books/make-event/make-event-keywords-or-exp.lisp</v>.  The \u000A discussion below is split into the following three parts.</p> \u000A \u000A <p>(1) The value produced by expansion may have the form <v>(:DO-PROOFS\u000A exp)</v>, which specifies <v>exp</v> as the expansion result, to be evaluated \u000A without skipping proofs even when including a book.</p> \u000A \u000A <p>(2) The value produced by expansion may have the form <v>(:OR exp-1\u000A ... exp-k)</v>, which specifies that the first form <v>exp-i</v> to evaluate \u000A without error is the expansion result.</p> \u000A \u000A <p>(3) The keyword argument <v>:EXPANSION?</v> can serve to eliminate the \u000A storing of <v>make-event</v> replacements, as described above for the ``book \u000A expansion'' of a book.</p> \u000A \u000A <p>We now elaborate on each of these.</p> \u000A \u000A <p>(1) <v>:DO-PROOFS</v> ``call'' produced by expansion.</p> \u000A \u000A <p>We have discussed the expansion result produced by the expansion phase of \u000A evaluating a <v>make-event</v> call.  However, if the expansion phase produces \u000A an expression of the form <v>(:DO-PROOFS exp)</v>, then the expansion result is \u000A actually <v>exp</v>.  The <v>:DO-PROOFS</v> wrapper indicates that even if proofs \u000A are currently being skipped (see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>), then evaluation of \u000A <v>exp</v> should take place with proofs not skipped.  For example, proofs will \u000A be performed when evaluating the <v>make-event</v> expansion, namely the \u000A indicated <v>defthm</v> event, in the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> t state)\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(:DO-PROOFS\u000A              (<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc (<see topic=\"COMMON-LISP____EQUAL\">equal</see>\u000A                                 (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                                 (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)))))</code> \u000A \u000A <p>Note that such use of <v>:DO-PROOFS</v> causes proofs to be performed when \u000A evaluating the expansion while including an uncertified book.  But when \u000A including a certified book, then unless <v>:CHECK-EXPANSION</v> is supplied a \u000A non-<v>nil</v> value, the <v>make-event</v> replacement will just be the \u000A expansion, which does not include the <v>:DO-PROOFS</v> wrapper and hence will \u000A be evaluated with proofs skipped.</p> \u000A \u000A <p>(2) <v>:OR</v> ``call'' produced by expansion.</p> \u000A \u000A <p>There may be times where you want to try different expansions.  For \u000A example, the community book <v>books/make-event/proof-by-arith.lisp</v> attempts \u000A to admit a given event, which we'll denote <v>EV</v>, by trying events of the \u000A following form as <v>BOOK</v> varies over different community books.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> BOOK :DIR :SYSTEM))\u000A EV)</code> \u000A \u000A <p>A naive implementation of this macro would evaluate all such <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events until one succeeds, and then return that successful event \u000A as the expansion.  Then that event would need to be evaluated again!  With \u000A some hacking one could avoid that re-evaluation by using <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt>, \u000A but that won't work if you are trying to create a certified book without \u000A skipped proofs.  Instead, the implementation creates an expansion of the form \u000A <v>(:OR ev-1 ev-2 ... ev-k)</v>, where the list <v>(ev-1 ev-2 ... ev-k)</v> \u000A enumerates the generated encapsulate events.  In general, for this \u000A ``disjunctive case'' of a result from expansion, each <v>ev-i</v> is evaluated \u000A in sequence, and the first that succeeds without error is considered to be the \u000A expansion result &mdash; and a repeat evaluation is avoided.  If evaluation of \u000A each <v>ev-i</v> results in an error, then so does the <v>make-event</v> call.</p> \u000A \u000A <p>This special use of <v>:OR</v> in a value produced by expansion is only \u000A supported at the top level.  That is, the result can be <v>(:OR ev-1 ev-2\u000A ... ev-k)</v> but then each <v>ev-i</v> must be a legal expansion result, without \u000A such further use of <v>:OR</v> &mdash; except, <v>ev-i</v> may be <tt>(:DO-PROOFS \u000A ev-i')</tt>, where <v>ev-i'</v> then would serve as the expansion rather than \u000A <v>ev-i</v>.</p> \u000A \u000A <p>(3) The <v>:EXPANSION?</v> keyword argument.</p> \u000A \u000A <p>If keyword argument <v>:EXPANSION?</v> has a non<v>nil</v> value, then the \u000A <v>:CHECK-EXPANSION</v> keyword must be omitted or have value <v>nil</v> or \u000A <v>t</v>, hence not a cons pair.</p> \u000A \u000A <p>The idea of the <v>:EXPANSION?</v> keyword is to give you a way to avoid \u000A storing expansion results in a book's <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  Roughly speaking, \u000A when the expansion result matches the value of <v>:EXPANSION?</v>, then no \u000A expansion result is stored for the event by book certification; then when the \u000A book is later included, the value of <v>:EXPANSION?</v> is used as the \u000A expansion, thus bypassing the expansion phase.  One could say that the event \u000A is its own make-event replacement, but it is more accurate to say that there \u000A is no make-event replacement at all, since nothing is stored in the \u000A certificate for this event.  Below, we elaborate on make-event replacements \u000A when <v>:EXPANSION</v> is used and also discuss other properties of this \u000A keyword.</p> \u000A \u000A <p>We modify the notion of ``expansion result'' for <v>make-event</v> forms to \u000A comprehend the use of the <v>:EXPANSION?</v> keyword.  For that purpose, let's \u000A consider a call of <v>make-event</v> to be ``reducible'' if it has an \u000A <v>:EXPANSION?</v> keyword with non-<v>nil</v> value, <v>exp</v>, and its \u000A <v>:CHECK-EXPANSION</v> keyword is missing or has value <v>nil</v>, in which case \u000A the ``reduction'' of this <v>make-event</v> call is defined to be <v>exp</v>.  The \u000A expansion result as originally defined is modified by the following \u000A ``recursive reduction'' process: recur through the original expansion, passing \u000A through calls of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt>, <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt>, \u000A <tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt>, and <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, and \u000A replacing (recursively) any reducible call of <v>make-event</v> by its \u000A reduction.  Furthermore, we refer to two forms as ``reduction equivalent'' if \u000A their recursive reductions are equal.  Note that the recursive reduction \u000A process does not pass through <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, but that \u000A process is applied to the computation of expansions for their subsidiary \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> calls.</p> \u000A \u000A <p>To explain further the effect of <v>:EXPANSION? exp</v>, we split into the \u000A following two cases.</p> \u000A \u000A <p>o Case 1: Evaluation is not taking place when including a book or \u000A evaluating the second pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event; more precisely, \u000A the value of <v>(<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state)</v> is not the symbol <v>INCLUDE-BOOK</v>. \u000A There are two subcases.</p> \u000A \u000A <blockquote> \u000A \u000A <p>- Case 1a: The expansion result is not reduction-equivalent to <v>exp</v>. \u000A Then the <v>make-event</v> call is processed as though the <v>:EXPANSION?</v> \u000A keyword had been omitted.</p> \u000A \u000A <p>- Case 2a: The expansion result is reduction-equivalent to <v>exp</v>.  Then \u000A there is no <v>make-event</v> replacement for this call of <v>make-event</v>; no \u000A replacement will be put into the <see topic=\"ACL2____CERTIFICATE\">certificate</see> file for a book containing \u000A this <v>make-event</v> call.  When that book is subsequently included, the \u000A original form will be evaluated in the manner described in the next \u000A case.</p></blockquote> \u000A \u000A <p>o Case 2: Evaluation is taking place when including a book or evaluating \u000A the second pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event; more precisely, the value of \u000A <v>(<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state)</v> is the symbol <v>INCLUDE-BOOK</v>.  Then the \u000A expansion is <v>exp</v>.  The expansion phase is skipped unless \u000A <v>:CHECK-EXPANSION</v> is <v>t</v>.</p> \u000A \u000A <p>The <v>:EXPANSION?</v> keyword can be particularly useful in concert with the \u000A disjunctive (``<v>:OR</v>'') case (2) discussed above.  Suppose that expansion \u000A produces a value as discussed in (2) above, <v>(:OR exp-1 ... exp-k)</v>.  If \u000A one of these expressions <v>exp-i</v> is more likely than the others to be the \u000A expansion, then you may wish to specify <v>:EXPANSION? exp-i</v>, as this will \u000A avoid storing a <v>make-event</v> replacement in that common case.  This could \u000A be useful if the expressions are large, to avoid enlarging the <see topic=\"ACL2____CERTIFICATE\">certificate</see> file for a book containing the <v>make-event</v> call.</p> \u000A \u000A <p>It is legal to specify both <v>:EXPANSION? exp</v> and <v>:CHECK-EXPANSION\u000A t</v>.  When either <v>(<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state)</v> is the symbol \u000A <v>INCLUDE-BOOK</v>, or evaluation is taking place in raw Lisp, then this \u000A combination is treated the same as if <v>:EXPANSION?</v> is omitted and the \u000A value of <v>:CHECK-EXPANSION</v> is <v>exp</v>.  Otherwise, this combination is \u000A treated the same as <v>:CHECK-EXPANSION t</v>, modified to accommodate the \u000A effect of <v>:EXPANSION?</v>  as discussed above: if the expansion is indeed the \u000A value of <v>:EXPANSION?</v>, then no <v>make-event</v> replacement is \u000A generated.</p>"],

 "ACL2____LD":[["Other"],"ACL2 Sources","ACL2","<code>Examples: \u000A(LD \"foo.lisp\")              ; read and evaluate each form in file \u000A                             ; \"foo.lisp\", in order \u000A(LD \"foo.lisp\" :ld-pre-eval-print t) \u000A                             ; as above, but print each form to standard \u000A                             ; character output just before it is evaluated \u000A \u000AGeneral Form: \u000A(LD standard-oi                  ; open obj in channel, stringp file name \u000A                                 ; to open and close, or list of forms \u000A                                 ; Optional keyword arguments: \u000A    :dir                ...      ; use this add-include-book-dir directory \u000A    :standard-co        ...      ; open char out or file to open and close \u000A    :proofs-co          ...      ; open char out or file to open and close \u000A    :current-package    ...      ; known package name \u000A    :ld-skip-proofsp    ...      ; nil, 'include-book, or t \u000A                                 ;   (see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>) \u000A    :ld-redefinition-action ...  ; nil or '(:a . :b) \u000A    :ld-prompt          ...      ; nil, t, or some prompt printer fn \u000A    :ld-missing-input-ok ...     ; nil, t, :warn, or warning message \u000A    :ld-pre-eval-filter ...      ; :all, :query, or some new name \u000A    :ld-pre-eval-print  ...      ; nil, t, or :never \u000A    :ld-post-eval-print ...      ; nil, t, or :command-conventions \u000A    :ld-evisc-tuple     ...      ; nil or '(alist nil nil level length) \u000A    :ld-error-triples   ...      ; nil or t \u000A    :ld-error-action    ...      ; :return!, :return, :continue or :error \u000A    :ld-query-control-alist ...  ; alist supplying default responses \u000A    :ld-verbose         ...)     ; nil or t \u000A \u000A</code> \u000A \u000A <p><v>Ld</v> is the top-level ACL2 read-eval-print loop.  (When you call <tt><see topic=\"ACL2____LP\">lp</see></tt>, a little initialization is done in raw Common Lisp and then <v>ld</v> is \u000A called.)  <v>Ld</v> is also a general-purpose ACL2 file loader and a <see topic=\"ACL2____COMMAND\">command</see> interpreter.  <v>Ld</v> is actually a macro that expands to a function \u000A call involving <tt><see topic=\"ACL2____STATE\">state</see></tt>.  <v>Ld</v> returns an ``error triple'' <v>(<see topic=\"ACL2____MV\">mv</see> erp\u000A val state)</v> as explained below.  (For much more on error triples, see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.)</p> \u000A \u000A <p>See <see topic=\"ACL2____REBUILD\">rebuild</see> for a variant of <v>ld</v> that skips proofs.  See <see topic=\"ACL2____OUTPUT-TO-FILE\">output-to-file</see> for examples showing how to redirect output to a file.</p> \u000A \u000A <p>The arguments to <v>ld</v>, except for <v>:dir</v>, all happen to be global \u000A variables in <tt><see topic=\"ACL2____STATE\">state</see></tt> (see <see topic=\"ACL2____STATE\">state</see> and see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>).  For example, <v>'</v><tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> and \u000A <v>'</v><tt><see topic=\"ACL2____LD-VERBOSE\">ld-verbose</see></tt> are global variables, which may be accessed via <v>(<see topic=\"ACL2_____04\">@</see>\u000A current-package)</v> and <v>(<see topic=\"ACL2_____04\">@</see> ld-verbose)</v>.  When <v>ld</v> is called, it \u000A ``binds'' these variables.  By ``binds'' we actually mean the variables are \u000A globally set but restored to their old values on exit.  Because <v>ld</v> \u000A provides the illusion of <see topic=\"ACL2____STATE\">state</see> global variables being bound, they are \u000A called ``<v>ld</v> specials'' (after the Lisp convention of calling a variable \u000A ``special'' if it is referenced freely after having been bound).</p> \u000A \u000A <p>Note that all arguments but the first are passed via keyword.  Any variable \u000A not explicitly given a value in a call retains its pre-call value, with the \u000A exception of <v>:</v><tt><see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see></tt>, which defaults to <v>:return!</v> if \u000A not explicitly specified.</p> \u000A \u000A <p>Just as an example to drive the point home: If <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> is \u000A <v>&quot;ACL2&quot;</v> and you typed</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> *standard-oi* :current-package &quot;MY-PKG&quot;)</code> \u000A \u000A <p>you would find yourself in (an inner) read-eval-print loop in which the \u000A <see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see> was <v>&quot;MY-PKG&quot;</v>.  You could operate there as long \u000A as you wished, changing the current package at will.  But when you typed \u000A <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> you would return to the outer read-eval-print loop where the \u000A current package would still be <v>&quot;ACL2&quot;</v>.</p> \u000A \u000A <p>Roughly speaking, <v>ld</v> repeatedly reads a form from <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, \u000A evaluates it, and prints its result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  It does this \u000A until the form is <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> or evaluates to an error triple whose value \u000A component is <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt>, or until the input channel or list is emptied. \u000A However, <v>ld</v> has many bells and whistles controlled by the <v>ld</v> \u000A specials.  Each such special is documented individually.  For example, see the \u000A documentation for <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt>, <tt><see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see></tt>, etc.</p> \u000A \u000A <p>A more precise description of <v>ld</v> is as follows.  In the description \u000A below we use the <v>ld</v> specials as variables, e.g., we say ``a form is read \u000A from <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>.''  By this usage we refer to the current value of \u000A the named <see topic=\"ACL2____STATE\">state</see> global variable, e.g., we mean ``a form is read from \u000A the current value of <v>'</v><tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>.'' This technicality has an \u000A important implication: If while interacting with <v>ld</v> you change the value \u000A of one of the <v>ld</v> specials, e.g., <v>'</v><tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, you will \u000A change the behavior of <v>ld</v>, e.g., subsequent input will be taken from the \u000A new value.</p> \u000A \u000A <p>Three <v>ld</v> specials are treated as channels: <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is \u000A treated as an object input channel and is the source of forms evaluated by \u000A <v>ld</v>; <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt> are treated as character \u000A output channels and various flavors of output are printed to them.  However, \u000A the supplied values of these specials need not actually be channels; several \u000A special cases are recognized.</p> \u000A \u000A <p>If the supplied value of one of these is in fact an open channel of the \u000A appropriate type, that channel is used and is not closed by <v>ld</v>.  If the \u000A supplied value of one of these specials is a string, the string is treated as \u000A a file name in (essentially) Unix syntax (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>) and a channel \u000A of the appropriate type is opened to/from that file.  Any channel opened by \u000A <v>ld</v> during the binding of the <v>ld</v> specials is automatically closed by \u000A <v>ld</v> upon termination.  If <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt> are \u000A equal strings, only one channel to that file is opened and is used for \u000A both.</p> \u000A \u000A <p>As a special convenience, when <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is a string and the \u000A <v>:dir</v> argument provided and not <v>nil</v>, we look up <v>:dir</v> in the \u000A table of directories maintained by <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>, and prepend \u000A this directory to <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> to create the filename.  (In this case, \u000A however, we require that <v>standard-oi</v> is a relative pathname, not an \u000A absolute pathname.)  For example, one can write <v>(<see topic=\"ACL2____LD\">ld</see>\u000A &quot;arithmetic/top-with-meta.lisp&quot; :dir :system)</v> to <v>ld</v> that particular \u000A community books library.  (Of course, you should almost always load books like \u000A <v>arithmetic/top-with-meta</v> using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> instead of <v>ld</v>.) \u000A If <v>:dir</v> is not specified, then a relative pathname is resolved using the \u000A connected book directory; see <see topic=\"ACL2____CBD\">cbd</see>.</p> \u000A \u000A <p>Several other alternatives are allowed for <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>.  If <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is a true list then it is taken as the list of forms to be \u000A processed.  If <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is a list ending in an open channel, then \u000A <v>ld</v> processes the forms in the list and then reads and processes the forms \u000A from the channel.  Analogously, if <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is a list ending a \u000A string, an object input channel from the named file is opened and <v>ld</v> \u000A processes the forms in the list followed by the forms in the file.  That \u000A channel is closed upon termination of <v>ld</v>.</p> \u000A \u000A <p>In the cases that a string is to be converted to an object input channel \u000A &mdash; that is, when <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is a string or is a list ending in a \u000A string &mdash; an error occurs by default if the conversion fails, presumably \u000A because the file named by the string does not exist.  However, if keyword \u000A argument <v>:ld-missing-input-ok</v> is <v>t</v>, then <v>ld</v> immediately returns \u000A without error in this case, but without reading or executing any forms, as \u000A though <v>standard-oi</v> is <v>nil</v> and keyword arguments <v>:ld-verbose</v> and \u000A <v>ld-prompt</v> both have value <v>nil</v>.  The other legal values for \u000A <v>:ld-missing-input-ok</v> are <v>nil</v>, which gives the default behavior, and \u000A <v>:warn</v>, which behaves the same as <v>t</v> except that a warning is printed, \u000A which contains the same information as would be printed for the default error \u000A described above.</p> \u000A \u000A <p>The remaining <v>ld</v> specials are handled more simply and generally have \u000A to be bound to one of a finite number of tokens described in the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> entries for each <v>ld</v> special.  Should any <v>ld</v> special be supplied \u000A an inappropriate value, an error message is printed.</p> \u000A \u000A <p>Next, if <tt><see topic=\"ACL2____LD-VERBOSE\">ld-verbose</see></tt> is <v>t</v>, <v>ld</v> prints the message ``ACL2 \u000A loading name'' where <v>name</v> names the file or channel from which forms are \u000A being read.  At the conclusion of <v>ld</v>, it will print ``Finished loading \u000A name'' if <tt><see topic=\"ACL2____LD-VERBOSE\">ld-verbose</see></tt> is <v>t</v>.</p> \u000A \u000A <p>Finally, <v>ld</v> repeatedly executes the ACL2 read-eval-print step, which \u000A may be described as follows.  A <see topic=\"ACL2____PROMPT\">prompt</see> is printed to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> if <tt><see topic=\"ACL2____LD-PROMPT\">ld-prompt</see></tt> is non-<v>nil</v>.  The format of the <see topic=\"ACL2____PROMPT\">prompt</see> is determined by <tt><see topic=\"ACL2____LD-PROMPT\">ld-prompt</see></tt>.  If it is <v>t</v>, the default ACL2 \u000A <see topic=\"ACL2____PROMPT\">prompt</see> is used.  If it is any other non-<v>nil</v> value then it is \u000A treated as an ACL2 function that will print the desired <see topic=\"ACL2____PROMPT\">prompt</see>.  See \u000A <see topic=\"ACL2____LD-PROMPT\">ld-prompt</see>.  In the exceptional case where <v>ld</v>'s input is coming \u000A from the terminal <v>(<see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see>)</v> but its output is going to a different \u000A sink (i.e., <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> is not <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>), we also print \u000A the <see topic=\"ACL2____PROMPT\">prompt</see> to the terminal.</p> \u000A \u000A <p><v>Ld</v> then reads a form from <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>.  If the object read is \u000A a keyword, <v>ld</v> constructs a ``keyword command form'' by possibly reading \u000A several more objects.  See <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.  This construction process \u000A is sensitive to the value of <tt><see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see></tt>.  See <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see>.  Otherwise, the object read is treated as the command \u000A form.</p> \u000A \u000A <p><v>Ld</v> next decides whether to evaluate or skip this form, depending on \u000A <tt><see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see></tt>.  Initially, the filter must be either <v>:all</v>, \u000A <v>:query</v>, or a new name.  If it is <v>:all</v>, it means all forms are \u000A evaluated.  If it is <v>:query</v>, it means each form that is read is displayed \u000A and the user is queried.  Otherwise, the filter is a name and each form that \u000A is read is evaluated as long as the name remains new, but if the name is ever \u000A introduced then no more forms are read and <v>ld</v> terminates.  See <see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see>.</p> \u000A \u000A <p>If the form is to be evaluated, then <v>ld</v> first prints the form to \u000A <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>, if <tt><see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see></tt> is <v>t</v>.  With this \u000A feature, <v>ld</v> can process an input file or form list and construct a script \u000A of the session that appears as though each form was typed in.  See <see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see>.</p> \u000A \u000A <p><v>Ld</v> then evaluates the form, with <tt><see topic=\"ACL2____STATE\">state</see></tt> bound to the current \u000A <see topic=\"ACL2____STATE\">state</see>.  The result is some list of (multiple) values.  If a <see topic=\"ACL2____STATE\">state</see> is among the values, then <v>ld</v> uses that <see topic=\"ACL2____STATE\">state</see> as the \u000A subsequent current <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <p>Depending on <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt>, <v>ld</v> may interpret the result as \u000A an ``error.''  See <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see>.  We first discuss <v>ld</v>'s \u000A behavior if no error signal is detected (either because none was sent or \u000A because <v>ld</v> is ignoring them because <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt> is \u000A <v>nil</v>).</p> \u000A \u000A <p>In the case of a non-erroneous result, <v>ld</v> does two things: First, if \u000A the logical <see topic=\"ACL2____WORLD\">world</see> in the now current <see topic=\"ACL2____STATE\">state</see> is different than the \u000A <see topic=\"ACL2____WORLD\">world</see> before execution of the form, <v>ld</v> adds to the <see topic=\"ACL2____WORLD\">world</see> a \u000A ``<see topic=\"ACL2____COMMAND\">command</see> landmark'' containing the form evaluated.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.  Second, <v>ld</v> prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>, but only if <tt><see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see></tt> is not <v>nil</v>.  The \u000A result is printed as a list of (multiple) values unless <tt><see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see></tt> is <v>:command-conventions</v>, <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt> is \u000A <v>t</v>, and the result is an ``error triple'', i.e., of the form <v>(<see topic=\"ACL2____MV\">mv</see> * *\u000A state)</v> (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>).  In that case, only the non-erroneous \u000A ``value'' component of the result is printed.  See <see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see>.</p> \u000A \u000A <p>Whenever <v>ld</v> prints anything (whether the input form, a query, or some \u000A results) it ``eviscerates'' it if <v>ld-evisc-tuple</v> is non-<v>nil</v>. \u000A Essentially, evisceration is a generalization of Common Lisp's use of \u000A <v>*print-level*</v> and <v>*print-length*</v> to hide large substructures.  See \u000A <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> and also see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.</p> \u000A \u000A <p>We now return to the case of a form whose evaluation signals an error.  In \u000A this case, <v>ld</v> first restores the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> to what it was \u000A just before the erroneous form was evaluated.  Thus, a form that partially \u000A changes the <see topic=\"ACL2____WORLD\">world</see> (i.e., begins to store properties) and then signals \u000A an error, has no effect on the <see topic=\"ACL2____WORLD\">world</see>.  You may see this happen on <see topic=\"ACL2____COMMAND\">command</see>s that execute several <see topic=\"ACL2____EVENTS\">events</see> (e.g., an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or \u000A a <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> of several <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>): even though the output makes it \u000A appear that the initial <see topic=\"ACL2____EVENTS\">events</see> were executed, if an error is signalled \u000A by a later event the entire block of <see topic=\"ACL2____EVENTS\">events</see> is discarded.</p> \u000A \u000A <p>After rolling back, <v>ld</v> takes an action determined by <tt><see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see></tt>.  If the action is <v>:continue</v>, <v>ld</v> merely iterates \u000A the read-eval-print step.  Note that nothing suggestive of the value of the \u000A ``erroneous'' form is printed.  If the action is <v>:return</v>, <v>ld</v> \u000A terminates normally; similarly if the action is <v>:return!</v>, but a special \u000A value is returned that can cause superior <v>ld</v> commands to terminate; see \u000A <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see> for details.  If the action is <v>:error</v>, <v>ld</v> \u000A terminates signalling an error to its caller.  If its caller is in fact \u000A another instance of <v>ld</v> and that instance is watching out for error \u000A signals, the entire <see topic=\"ACL2____WORLD\">world</see> created by the inner <v>ld</v> will be \u000A discarded by the outer <v>ld</v> if the inner <v>ld</v> terminates with an \u000A error.</p> \u000A \u000A <p><v>Ld</v> returns an error triple, <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>.  <v>Erp</v> is \u000A <v>t</v> or <v>nil</v> indicating whether an error is being signalled.  If no \u000A error is signalled, <v>val</v> is the ``reason'' <v>ld</v> terminated and is one \u000A of <v>:exit</v> (meaning <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> was read), <v>:eof</v> (meaning the input \u000A source was exhausted), <v>:error</v> (meaning an error occurred but has been \u000A supressed), <v>:filter</v> (meaning the <tt><see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see></tt> terminated \u000A <v>ld</v>), or a cons pair whose first component is the symbol <v>:STOP-LD</v>, \u000A which typically indicates that an error occurred while the value of variable \u000A <v>'</v><tt><see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see></tt> was <v>:RETURN!</v>.  See <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see> \u000A for details of this last case.</p>"],

 "ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Software is complex, and ACL2 is a piece of software that is used \u000A to analyze software -- adding another layer of complexity.  Furthermore, \u000A instead of being limited to static analysis for certain fixed properties, ACL2 \u000A allows you -- indeed, forces you -- to formalize the problem and the \u000A questions.  It ``knows'' nothing inherent about your problem before you start \u000A to interact with it.  But it can be used to help answer the most complicated \u000A questions you can ask about software.</p> \u000A \u000A <p>All this is to say that it is not the kind of tool that you just install \u000A and then start to use effectively.  So OK, you've installed it or confirmed \u000A that you can invoke it.  Good for you.  Now you have to learn how to use it! \u000A Your success ultimately comes down to your understanding of your problem \u000A domain and your appropriate exploitation of ACL2's strengths and avoidance of \u000A its weaknesses.  So put aside the idea of sitting down and interacting with \u000A it.  Instead, learn about it.</p> \u000A \u000A <p>We assume you know some of the <see topic='ACL2____INTERESTING-APPLICATIONS'>industrial applications</see> of ACL2.  Realizing \u000A that such things can be done may sustain you during the long learning curve! \u000A We also assume you have taken both the <see topic='ACL2____A_02Flying_02Tour_02of_02ACL2'>Flying Tour</see> and the <see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'>Walking Tour</see>.  The tours give you a good \u000A overview of the whole system where this tutorial focuses on how to use the \u000A prover itself.</p> \u000A \u000A <p>If you haven't visited these links, please do so now.</p> \u000A \u000A <p>This tutorial will take you several hours -- maybe several days -- to work \u000A through.  Do not breeze through it as you might a blog.  <i>Think</i> your way \u000A through it.  <i>Remember</i> what you read.  Do not take short cuts.  If you \u000A start to use ACL2 before you really know how, it will only frustrate you.</p> \u000A \u000A <p>We recommend that you read this tutorial with an HTML browser so that you \u000A can see which links you've followed and which you have not.  To give you a \u000A sense of what is in store, here is a map of this document.  But don't navigate \u000A through it from here! Read it linearly, following the links when the text \u000A directs you to.</p> \u000A \u000A <code><see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> \u000A        <see topic=\"ACL2____SPECIAL-CASES-FOR-REWRITE-RULES\">special-cases-for-rewrite-rules</see> \u000A        <see topic=\"ACL2____EQUIVALENT-FORMULAS-DIFFERENT-REWRITE-RULES\">equivalent-formulas-different-rewrite-rules</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see> \u000A        <see topic=\"ACL2____DEALING-WITH-KEY-COMBINATIONS-OF-FUNCTION-SYMBOLS\">dealing-with-key-combinations-of-function-symbols</see> \u000A        <see topic=\"ACL2____GENERALIZING-KEY-CHECKPOINTS\">generalizing-key-checkpoints</see> \u000A        <see topic=\"ACL2____POST-INDUCTION-KEY-CHECKPOINTS\">post-induction-key-checkpoints</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see> \u000A        <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see> \u000A            <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-1\">practice-formulating-strong-rules-1</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-2\">practice-formulating-strong-rules-2</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-3\">practice-formulating-strong-rules-3</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-4\">practice-formulating-strong-rules-4</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-5\">practice-formulating-strong-rules-5</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-6\">practice-formulating-strong-rules-6</see> \u000A        <see topic=\"ACL2____SPECIFIC-KINDS-OF-FORMULAS-AS-REWRITE-RULES\">specific-kinds-of-formulas-as-rewrite-rules</see> \u000A        <see topic=\"ACL2____FURTHER-INFORMATION-ON-REWRITING\">further-information-on-rewriting</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-HINTS\">introduction-to-hints</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see> \u000A    <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1\">introductory-challenge-problem-1</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2\">introductory-challenge-problem-2</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3\">introductory-challenge-problem-3</see> \u000A        <i>(there are others but at least do a few)</i> \u000A    <see topic=\"ACL2____FREQUENTLY-ASKED-QUESTIONS-BY-NEWCOMERS\">frequently-asked-questions-by-newcomers</see> \u000A </code> \u000A \u000A <p>If any of the links above are marked as ``visited'' by your browser, use \u000A your browser's tools menu to mark all links as unvisited.  As you can see, we \u000A really think you'll get the most out of this document if you take it \u000A seriously.  As you read, you will see some links to ``advanced'' topics. \u000A These are marked with a tiny warning sign, ``<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>''. \u000A They lead out of this linear tutorial and into ACL2's hypertext reference \u000A manual.  We recommend that you <i>not</i> visit any of these advanced links \u000A until you have read the entire tutorial at least once.</p> \u000A \u000A <p>After you finish this tutorial material, we recommend that you look at the \u000A ACL2 Demos, at the ``Demos'' link of the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2'>ACL2 home page</a>.</p> \u000A \u000A <p>Most users of ACL2 have bought the book</p> \u000A \u000A <p><i>Computer-Aided Reasoning: An Approach</i>, Kaufmann, Manolios, and \u000A Moore, Kluwer Academic Publishers, June, 2000</p> \u000A \u000A <p>which is available <a href='http://www.lulu.com/content/1746161'>in \u000A paperback</a> from Lulu for approximately $20 (as of 2010).  That book \u000A contains hundreds of exercises in programming, proof, and using The Method \u000A described here to prove theorems.  Solutions to the exercises are online.  See \u000A this <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html#Books'> \u000A web page about the book</a>, which also includes information about its \u000A companion (also available on Lulu) describing applications of ACL2, some of \u000A which are from industry.</p> \u000A \u000A <p>Using ACL2 is akin to having a partner in the theorem proving enterprise. \u000A It will do some of the work and you will do some of the work.  It can't really \u000A be any other way because theorem proving is undecidable.  You bring a quirkly, \u000A error-prone, creative insight to the problem, and ACL2 brings accuracy, logic, \u000A and perserverance.</p> \u000A \u000A <p>Here we describe a ``model'' of how the system works and introduce some of \u000A the ideas and terminology you'll use repeatedly when interacting with it.</p> \u000A \u000A <p>This article is about the <i>theorem prover</i> itself, not the programming \u000A language and not the logic.  We assume you know enough about the ACL2 \u000A programming language that you can define simple functions, run them, and read \u000A and write ACL2 constants and terms.  For some examples of what we'll take for \u000A granted about ACL2 programming, see <see topic=\"ACL2____PROGRAMMING-KNOWLEDGE-TAKEN-FOR-GRANTED\">programming-knowledge-taken-for-granted</see>.</p> \u000A \u000A <p>We also assume you know enough about logic to understand, for example, the \u000A words we use to talk about formulas and proofs.  To see some examples of what \u000A we'll take for granted about your knowledge of logic terminology, see <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p> \u000A \u000A <p>When you give the theorem prover a goal formula to prove, it tries to prove \u000A it by breaking it down into subgoals, each of which must be proved in order to \u000A prove the original goal.  This breaking apart of the goal is done by various \u000A <i>proof techniques</i> built into ACL2.  Different proof techniques break the \u000A formula apart in different ways.  For example, the <i>simplifier</i> rips \u000A apart the propositional structure to isolate the various cases and applies \u000A rewrite rules to simplify the subterms of the formula, while the \u000A <i>induction</i> <i>engine</i> will attempt to break the goal into some base \u000A cases and some induction steps.</p> \u000A \u000A <p>The theorem prover's behavior is affected by a <i>database</i> of \u000A <i>rules</i> derived from axioms, definitions, and previously proved theorems. \u000A The database also records the <i>enabled status</i> of each rule; only enabled \u000A rules are seen by the prover and you can set the status of a rule.  There are \u000A many other user-settable switches and parameters affecting the behavior of the \u000A prover; you'll learn about some of them later.</p> \u000A \u000A <p>You guide the theorem prover most of the time simply by identifying lemmas \u000A for it to prove.  (A <i>lemma</i> is just a theorem that you think is useful \u000A in the proofs of other theorems.)</p> \u000A \u000A <p>Why does this guide the theorem prover?  Because every time you get the \u000A system to prove a theorem, it turns the theorem into a rule (unless you tell \u000A it not to) and stores the rule in the database.  That changes how the prover \u000A behaves subsequently.  But <i>you</i> determine the kind of rule ACL2 \u000A stores.</p> \u000A \u000A <p>To learn to ``drive'' the theorem prover you have to learn how various \u000A rules affect the system's behavior and how it turns proved formulas into \u000A rules.  But before we discuss this, we discuss a more mathematical activity: \u000A how do you figure out the lemmas ACL2 will need in order for it to prove \u000A interesting theorems?  ACL2 can often help you in this activity, if you use it \u000A in the right way.</p> \u000A \u000A <p>Here is the way we recommend you use ACL2.</p> \u000A \u000A <p><b>The Method</b>.</p> \u000A \u000A <p>(1) you present ACL2 with the goal conjecture to prove</p> \u000A \u000A <p>(2) typically, it fails to prove it (or you abort its attempt), but it \u000A prints some <i>Key Checkpoints</i></p> \u000A \u000A <p>(3) you look at the Key Checkpoints and decide that you know a fact that \u000A will help; this tutorial will present some helpful questions to keep in \u000A mind</p> \u000A \u000A <p>(4) you formalize your knowledge as a formula, along with directions for \u000A how ACL2 should turn the formula into a rule; this tutorial will tell you \u000A about the most commonly used rule, the <i>rewrite</i> rule</p> \u000A \u000A <p>(5) you recursively apply The Method to get ACL2 to prove your formula and \u000A to store it as the kind of rule you specified</p> \u000A \u000A <p>(6) go to step (1)</p> \u000A \u000A <p>Caveat: This approach simply won't work on some theorems!  Basically, this \u000A is a ``troubleshooting'' approach, where you're letting ACL2 determine the \u000A basic strategy and you're just helping with the subgoals.  But for some \u000A theorems, ACL2's approach will be misguided and no amount of troubleshooting \u000A will salvage its strategy.  You'll have a sense that this is happening when it \u000A happens because the formulas ACL2 presents to you will raise issues that feel \u000A irrelevant to you.  The basic truth is that if you think a formula is always \u000A true there are usually strong intuitive reasons behind your belief.  If you \u000A were asked to defend your belief, you'd start to explain your reasons and with \u000A training you can turn that into a proof.  So when ACL2's formulas present you \u000A with things you haven't thought of either (a) you'll have an ``Ah ha!'' \u000A moment when you realize you hadn't considered that case or (b) you'll realize \u000A that ACL2's approach is different from your intuitive ``proof.''</p> \u000A \u000A <p>But, surprisingly often, the troubleshooting approach to finding proofs \u000A works quite well, especially as you rein in your expectations and develop a \u000A sense of what ACL2 can handle on its own.  Of course, if you can decompose the \u000A proof into a couple of main lemmas before you start, so much the better: write \u000A down your sequence of lemmas, thinking about the rules you want them to \u000A generate, and get ACL2 to prove each of them before giving it the main \u000A theorem.  This <i>proof planning</i> approach will gradually become an \u000A integral part of your use of The Method.</p> \u000A \u000A <p>The only mechanized help we can offer with The Method, aside from the \u000A theorem prover itself, are tools to help you manage the stack of subgoals it \u000A generates when, in step (5) you recursively apply The Method to a lemma. \u000A There are both Emacs and Eclipse tools available.</p> \u000A \u000A <p>To use The Method you have to read the Key Checkpoints printed at the very \u000A end of <i>failed</i> proof attempts, just after the line that reads:</p> \u000A \u000A <code>The key checkpoint goals, below, may help you to debug this failure.</code> \u000A \u000A <p>Most users do not read the output from successful proofs and do not read \u000A the output <i>during</i> a proof -- they just let it stream by as a sort of \u000A gestalt meter on the theorem prover's progress or lack thereof.  For example, \u000A you'll be able to tell it is in a loop and needs to be interrupted.</p> \u000A \u000A <p>You will respond to most Key Checkpoints by formulating new lemmas for the \u000A system to prove and store as rules designed by you to alter ACL2's behavior so \u000A that it proves the Key Checkpoints.  You will give each lemma a <i>name</i> \u000A and write some <i>formula</i> to express the mathematical idea.  You'll \u000A command ACL2 to prove it by typing:</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        ...) \u000A</code> \u000A \u000A <p>In the ``...'' you may provide two other kinds of information: <i>hints</i> \u000A for how to prove <i>formula</i> and directives, called <i>rule-classes</i>, \u000A for how to convert <i>formula</i> into a rule after it has proved \u000A <i>formula</i>.  Note that <i>you</i> are in charge of determining what kind \u000A of rule ACL2 generates!  There are over a dozen different types of rules with \u000A many opportunities to specialize each type.  But the most common kind of rule \u000A you'll want to generate is a <i>rewrite</i> rule.</p> \u000A \u000A <p>We recommend that you read the following topics in the following order, \u000A without skipping anything except links into the ACL2 User's Manual, which are \u000A marked by the little warning sign, ``<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>''.</p> \u000A \u000A <p>(1) See <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> to read about the use \u000A and design of rewrite rules.</p> \u000A \u000A <p>(2) See <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see> to see how to use The Method \u000A to help you design rules.</p> \u000A \u000A <p>(3) See <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see> for general guidance on \u000A how to turn formulas into effective rules.</p> \u000A \u000A <p>(4) See <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see> to see how to issue commands \u000A that build different kinds of rules and that affect the enabled status of \u000A existing rules.</p> \u000A \u000A <p>(5) See <see topic=\"ACL2____INTRODUCTION-TO-HINTS\">introduction-to-hints</see> to see how to give the prover hints \u000A for how to prove specific theorems or even subgoals of specific proof \u000A attempts.</p> \u000A \u000A <p>(6) See <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see> for a few words \u000A about system aspects of interacting with ACL2.</p> \u000A \u000A <p>(7) See <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see> for a graduated sequence of good \u000A challenge problems for the new user to tackle.  <b>Do not skip this \u000A section!</b> It is here that you <i>really</i> learn how to use ACL2 -- by \u000A using it.</p> \u000A \u000A <p>(8) See <see topic=\"ACL2____FREQUENTLY-ASKED-QUESTIONS-BY-NEWCOMERS\">frequently-asked-questions-by-newcomers</see> for a list of \u000A questions that new users frequently ask, answered mainly by providing links \u000A into the ACL2 User's Manual.  We recommend that you skim through these \u000A questions and remember that you can find the answers here later.  We are very \u000A interested in receiving suggestions for how to improve this FAQ and this \u000A tutorial.  See the ACL2 home page, specifically the link ``Mailing \u000A Lists''.</p> \u000A \u000A <p>Please read all of the material cited above (skipping only the ACL2 User's \u000A Manual links (``<see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>'')) before you try to use ACL2 on problems of your \u000A own.</p> \u000A \u000A <p>By this point you should have read, at least, the following topics from \u000A this tutorial introduction to the theorem prover:</p> \u000A \u000A <code><see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> \u000A        <see topic=\"ACL2____SPECIAL-CASES-FOR-REWRITE-RULES\">special-cases-for-rewrite-rules</see> \u000A        <see topic=\"ACL2____EQUIVALENT-FORMULAS-DIFFERENT-REWRITE-RULES\">equivalent-formulas-different-rewrite-rules</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see> \u000A        <see topic=\"ACL2____DEALING-WITH-KEY-COMBINATIONS-OF-FUNCTION-SYMBOLS\">dealing-with-key-combinations-of-function-symbols</see> \u000A        <see topic=\"ACL2____GENERALIZING-KEY-CHECKPOINTS\">generalizing-key-checkpoints</see> \u000A        <see topic=\"ACL2____POST-INDUCTION-KEY-CHECKPOINTS\">post-induction-key-checkpoints</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see> \u000A        <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see> \u000A            <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-1\">practice-formulating-strong-rules-1</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-2\">practice-formulating-strong-rules-2</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-3\">practice-formulating-strong-rules-3</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-4\">practice-formulating-strong-rules-4</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-5\">practice-formulating-strong-rules-5</see> \u000A                <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-6\">practice-formulating-strong-rules-6</see> \u000A        <see topic=\"ACL2____SPECIFIC-KINDS-OF-FORMULAS-AS-REWRITE-RULES\">specific-kinds-of-formulas-as-rewrite-rules</see> \u000A        <see topic=\"ACL2____FURTHER-INFORMATION-ON-REWRITING\">further-information-on-rewriting</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-HINTS\">introduction-to-hints</see> \u000A    <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see> \u000A    <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1\">introductory-challenge-problem-1</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2\">introductory-challenge-problem-2</see> \u000A        <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3\">introductory-challenge-problem-3</see> \u000A        <i>(there are others but at least do a few)</i> \u000A    <see topic=\"ACL2____FREQUENTLY-ASKED-QUESTIONS-BY-NEWCOMERS\">frequently-asked-questions-by-newcomers</see> \u000A </code> \u000A \u000A <p>We also recommend that you look at the ACL2 Demos mentioned in the <see topic=\"ACL2____ACL2-TUTORIAL\">ACL2-tutorial</see>.</p> \u000A \u000A <p>Most users of ACL2 have bought the book</p> \u000A \u000A <p><i>Computer-Aided Reasoning: An Approach</i>, Kaufmann, Manolios, and \u000A Moore, Kluwer Academic Publishers, June, 2000</p> \u000A \u000A <p>which is <a href='http://www.lulu.com/content/1746161'>available in \u000A paperback from Lulu</a> for approximately $20 (as of 2010).  That book \u000A contains hundreds of exercises in programming, proof, and using The Method to \u000A prove theorems.  Solutions to the exercises are online.  See also this <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html#Books'> \u000A web page about the book</a>, which also includes information about its \u000A companion (also available on Lulu) describing applications of ACL2, some of \u000A which are from industry.</p> \u000A \u000A <p>Thank you for spending the time to get acquainted with the basics of the \u000A ACL2 theorem prover.  Don't hesitate to send further questions to the ACL2 \u000A Help address on the ``Mailing Lists'' link of the ACL2 home page.</p> \u000A \u000A <p><b>End of Tutorial Introduction to the Theorem Prover</b></p> \u000A \u000A <p>Below is a list of all of the topics cited on this page.</p>"],

 "ACL2____INCLUDE-BOOK":[["Books-reference","Books-tour","Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;my-arith&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;/home/smith/my-arith&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;/../../my-arith&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> file :load-compiled-file action\u000A                   :uncertified-okp t/nil      ; [default t]\u000A                   :defaxioms-okp t/nil        ; [default t]\u000A                   :skip-proofs-okp t/nil      ; [default t]\u000A                   :ttags ttags                ; [default nil]\u000A                   :dir directory\u000A                   :doc doc-string)</code> \u000A \u000A <p>where <v>file</v> is a book name.  See <see topic=\"ACL2____BOOKS\">books</see> for general information, \u000A see <see topic=\"ACL2____BOOK-NAME\">book-name</see> for information about book names, and see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see> \u000A for information about file names.  <v>Action</v> is one of <v>t</v>, <v>nil</v>, \u000A <v>:default</v>, <v>:warn</v>, or <v>:comp</v>; these values are explained below, \u000A and the default is <v>:default</v>.  The three <v>-okp</v> keyword arguments, \u000A which default to <v>t</v>, determine whether errors or warnings are generated \u000A under certain conditions explained below; when the argument is <v>t</v>, \u000A warnings are generated.  The <v>dir</v> argument, if supplied, is a keyword that \u000A represents an absolute pathname for a directory (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>), to be \u000A used instead of the current book directory (see <see topic=\"ACL2____CBD\">cbd</see>) for resolving the \u000A given <v>file</v> argument to an absolute pathname.  In particular, by default \u000A <v>:dir :system</v> resolves <v>file</v> using the <v>books/</v> directory of your \u000A ACL2 installation, unless your ACL2 executable was built somewhere other than \u000A where it currently resides; please see the ``Books Directory'' below.  To \u000A define other keywords that can be used for <v>dir</v>, see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see>.  <v>Doc-string</v> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.  If the book has no <tt><see topic=\"ACL2____CERTIFICATE\">certificate</see></tt>, if its \u000A <tt><see topic=\"ACL2____CERTIFICATE\">certificate</see></tt> is invalid or if the certificate was produced by a \u000A different <see topic=\"ACL2____VERSION\">version</see> of ACL2, a warning is printed and the book is \u000A included anyway; see <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  This can lead to serious errors, \u000A perhaps mitigated by the presence of a <v>.port</v> file from an earlier \u000A certification; see <see topic=\"ACL2____UNCERTIFIED-BOOKS\">uncertified-books</see>.  If the portcullis of the <see topic=\"ACL2____CERTIFICATE\">certificate</see> (see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>) cannot be raised in the host logical \u000A <see topic=\"ACL2____WORLD\">world</see>, an error is caused and no change occurs to the logic. \u000A Otherwise, the non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in file are assumed.  Then the \u000A <see topic=\"ACL2____KEEP\">keep</see> of the <see topic=\"ACL2____CERTIFICATE\">certificate</see> is checked to ensure that the correct \u000A files were read; see <see topic=\"ACL2____KEEP\">keep</see>.  A warning is printed if uncertified <see topic=\"ACL2____BOOKS\">books</see> were included.  Even if no warning is printed, <v>include-book</v> places \u000A a burden on you; see <see topic=\"ACL2____CERTIFICATE\">certificate</see>.</p> \u000A \u000A <p>If you use <see topic=\"ACL2____GUARD\">guard</see>s, please note <v>include-book</v> is executed as \u000A though <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> nil)</v> has been evaluated; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  If you want guards checked, please see <see topic=\"ACL2____LD\">ld</see> and/or \u000A see <see topic=\"ACL2____REBUILD\">rebuild</see>.</p> \u000A \u000A <p>The value of <v>:load-compiled-file</v> controls whether a compiled file for \u000A the given <v>file</v> is loaded by <v>include-book</v>.  Note that this keyword \u000A applies only to the given <v>file</v>, not to any included sub-books.  In order \u000A to skip loading all compiled files, for the given <v>file</v> as well as all \u000A included sub-books &mdash; for example, to avoid Lisp errors such as ``Wrong \u000A FASL version'' &mdash; use <v>(<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> nil state)</v>; see <see topic=\"ACL2____COMPILATION\">compilation</see>.  Otherwise, if keyword argument <v>:load-compiled-file</v> is \u000A missing or its value is the keyword <v>:default</v>, then it is treated as \u000A <v>:warn</v>.  If its value is <v>nil</v>, no attempt is made to load the compiled \u000A file for the book provided.  In order to load the compiled file, it must be \u000A more recent than the book's <see topic=\"ACL2____CERTIFICATE\">certificate</see> (except in raw mode, where it \u000A must be more recent than the book itself; see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>).  For \u000A non-<v>nil</v> values of <v>:load-compiled-file</v> that do not result in a loaded \u000A compiled file, ACL2 proceeds as follows.  Note that a load of a compiled file \u000A or expansion file aborts partway through whenever an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form \u000A is encountered for which no suitable compiled or expansion file can be loaded. \u000A For much more detail, see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>t</v>: Cause an error if the compiled file is not loaded.  This same \u000A requirement is imposed on every <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form evaluated during the \u000A course of evaluation of the present <v>include-book</v> form, except that for \u000A those subsidiary <v>include-book</v>s that do not themselves specify \u000A <v>:load-compiled-file t</v>, it suffices to load the expansion file (see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>).</p> \u000A \u000A <p><v>:warn</v>: An attempt is made to load the compiled file, and a warning is \u000A printed if that load fails to run to completion.</p> \u000A \u000A <p><v>:comp</v>: A compiled file is loaded as with value <v>t</v>, except that if \u000A a suitable ``expansion file'' exists but the compiled file does not, then the \u000A compiled file is first created.  See <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.</p></blockquote> \u000A \u000A <p>The three <v>-okp</v> arguments, <v>:uncertified-okp</v>, <v>defaxioms-okp</v>, \u000A and <v>skip-proofs-okp</v>, determine the system's behavior when the book or any \u000A subbook is found to be uncertified, when the book or any subbook is found to \u000A contain <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> events, and when the book or any subbook is found to \u000A contain <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> events, respectively.  All three default to \u000A <v>t</v>, which means it is ``ok'' for the condition to arise.  In this case, a \u000A warning is printed but the processing to load the book is allowed to proceed. \u000A When one of these arguments is <v>nil</v> and the corresponding condition \u000A arises, an error is signaled and processing is aborted.  <b>Exception</b>: \u000A <v>:uncertified-okp</v> is ignored if the <v>include-book</v> is being performed \u000A on behalf of a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A <p>The keyword argument <v>:ttags</v> may normally be omitted.  A few \u000A constructs, used for example if you are building your own system based on \u000A ACL2, may require it.  See <see topic=\"ACL2____DEFTTAG\">defttag</see> for an explanation of this \u000A argument.</p> \u000A \u000A <p><v>Include-book</v> is similar in spirit to <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> in that it is \u000A a single event that ``contains'' other <see topic=\"ACL2____EVENTS\">events</see>, in this case the <see topic=\"ACL2____EVENTS\">events</see> listed in the file named.  <v>Include-book</v> processes the non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event forms in the file, assuming that each is admissible.  <tt><see topic=\"ACL2____LOCAL\">Local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in the file are ignored.  You may use <v>include-book</v> \u000A to load several <see topic=\"ACL2____BOOKS\">books</see>, creating the logical <see topic=\"ACL2____WORLD\">world</see> that contains \u000A the definitions and theorems of all of them.</p> \u000A \u000A <p>If any non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event of the book attempts to define a <see topic=\"ACL2____NAME\">name</see> \u000A that has already been defined &mdash; and the book's definition is not \u000A syntactically identical to the existing definition &mdash; the attempt to \u000A include the book fails, an error message is printed, and no change to the \u000A logical <see topic=\"ACL2____WORLD\">world</see> occurs.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see> for the details.</p> \u000A \u000A <p>When a book is included, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) for the first event is always <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  That \u000A is, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> ``outside'' the book &mdash; in the \u000A environment in which <v>include-book</v> was called &mdash; is irrelevant to the \u000A book.  <see topic=\"ACL2____EVENTS\">Events</see> that change the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> are permitted within a \u000A book (provided they are not in <tt><see topic=\"ACL2____LOCAL\">local</see></tt> forms).  However, such changes \u000A within a book are not exported, i.e., at the conclusion of an \u000A <v>include-book</v>, the ``outside'' default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is always the \u000A same as it was before the <v>include-book</v>.</p> \u000A \u000A <p>Unlike every other event in ACL2, <v>include-book</v> puts a burden on you. \u000A Used improperly, <v>include-book</v> can be unsound in the sense that it can \u000A create an inconsistent extension of a consistent logical <see topic=\"ACL2____WORLD\">world</see>.  A \u000A certification mechanism is available to help you carry this burden &mdash; but \u000A it must be understood up front that even certification is no guarantee against \u000A inconsistency here.  The fundamental problem is one of file system security. \u000A See <see topic=\"ACL2____CERTIFICATE\">certificate</see> for a discussion of the security issues.</p> \u000A \u000A <p>At the beginning of execution of an <v>include-book</v> form, even before \u000A executing <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s, the value of <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> is restored to the value it had at startup.  After \u000A execution of an <v>include-book</v> form, the value of <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> is restored to what it was immediately before that \u000A <v>include-book</v> form was executed.  See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p><b>Books Directory.</b> We refer to the ``books directory'' of an \u000A executable image as the full pathname string of the directory associated with \u000A <v>include-book</v> keyword option <v>:dir :system</v> for that image.  By \u000A default, it is the <v>books/</v> subdirectory of the directory where the sources \u000A reside and the executable image is thus built (except for ACL2(r) &mdash; see \u000A <see topic=\"COMMON-LISP____REAL\">real</see> &mdash;, where it is <v>books/nonstd/</v>).  If those books reside \u000A elsewhere, the environment variable <v>ACL2_SYSTEM_BOOKS</v> can be set to the \u000A <v>books/</v> directory under which they reside (a Unix-style pathname, \u000A typically ending in <v>books/</v> or <v>books</v>, is permissible).  In most \u000A cases, your ACL2 executable is a small script in which you can set this \u000A environment variable just above the line on which the actual ACL2 image is \u000A invoked, for example:</p> \u000A \u000A <code>export ACL2_SYSTEM_BOOKS\u000AACL2_SYSTEM_BOOKS=/home/acl2/4-0/acl2-sources/books</code> \u000A \u000A <p>If you follow suggestions in the installation instructions, these books \u000A will be the ACL2 community books; see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p> \u000A \u000A <p>This concludes the guided tour through <see topic=\"ACL2____BOOKS\">books</see>.  See <see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see> for a subtle point about the interaction between \u000A <v>include-book</v> and on-the-fly <see topic=\"ACL2____COMPILATION\">compilation</see>.  See <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> \u000A for a discussion of how to certify a book.</p>"],

 "ACL2____HONS-AND-MEMOIZATION":[["ACL2"],"ACL2 Sources","ACL2","<p>Bob Boyer and Warren Hunt, and later Jared Davis and Sol Swords, \u000A have developed a canonical representation for ACL2 data objects and a function \u000A memoization mechanism to facilitate reuse of previously computed results. \u000A This facility includes procedures to read and print ACL2 expressions in such a \u000A way that repetition of some ACL2 objects is eliminated, thereby permitting a \u000A kind of on-the-fly file compression.  The implementation does not alter the \u000A semantics of ACL2 except to add a handful of definitions.</p> \u000A \u000A <p>We give the name ``ACL2(h)'' to the resulting experimental extension of the \u000A ACL2 system, which includes hash cons, function memoization, and fast \u000A association lists (applicative hash tables).  It is optimized for Clozure \u000A Common Lisp (CCL), but other ANSI-compliant host Lisp implementations may also \u000A work, provided there are sufficient machine resources.</p> \u000A \u000A <p>If you want to use ACL2(h), you might find it helpful to consult the \u000A document <v>centaur/README.html</v> in the ACL2 community books.</p> \u000A \u000A <p>An easy way is to build ACL2(h) is to include the following with a `make' \u000A command:</p> \u000A \u000A <code>ACL2_HONS=h</code> \u000A \u000A <p>So for example, to make an executable image and also documentation (which \u000A will appear in subdirectories <v>doc/EMACS</v> and <v>doc/HTML</v>):</p> \u000A \u000A <code>make large DOC ACL2_HONS=h</code> \u000A \u000A <p>Much of the documentation for the remainder of this topic is taken from the \u000A paper ``Function Memoization and Unique Object Representation for ACL2 \u000A Functions'' by Robert S. Boyer and Warren A. Hunt, Jr., which has appeared in \u000A the Sixth International Workshop on the ACL2 Theorem Prover and Its \u000A Applications, ACM Digital Library, 2006.</p> \u000A \u000A <p>In the implementation of the ACL2 logic, ACL2 data objects are represented \u000A by Common Lisp objects of the same type, and the ACL2 pairing operation is \u000A internally implemented by the Common Lisp <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> function.  In Common \u000A Lisp, <v>cons</v> is guaranteed to provide a new pair, distinct from any \u000A previously created pair.  We have defined a new ACL2 function <tt><see topic=\"ACL2____HONS\">hons</see></tt> \u000A that is logically identical to the ACL2 <v>cons</v> function, but whose \u000A implementation usually reuses an existing pair if its components are identical \u000A to the components of an existing pair.  A record of ACL2 HONS objects is kept, \u000A and when an ACL2 function calls <v>hons</v> ACL2 searches for an existing \u000A identical pair before allocating a new pair; this operation been called ``hash \u000A consing''.</p> \u000A \u000A <p>It appears that hash consing was first conceived by A. P. Ershov in 1957, \u000A to speed up the recognition of common subexpressions.  Ershov showed how to \u000A collapse trees to minimal DAGs by traversing trees bottom up, and he used \u000A hashing to eliminate the re-evaluation of common subexpressions.  Later, \u000A Eiichi Goto implemented a Lisp system with a built-in hash consing operation: \u000A his h-CONS cells were rewrite protected and free of duplicate copies, and Goto \u000A used this hash consing operation to facilitate the implementation of a \u000A symbolic algebra system he developed.</p> \u000A \u000A <p>Memoizing functions also has a long history.  In 1967, Donald Michie \u000A proposed using memoized functions to improve the performance of machine \u000A learning.  Rote learning was improved by a learning function not forgetting \u000A what it had previously learned; this information was stored as memoized \u000A function values.</p> \u000A \u000A <p>The use of hash consing has appeared many times.  For instance, Henry Baker \u000A used hash consing to improve the performance of the well-known Boyer rewriting \u000A benchmark.  Baker used both hash consing and function memoization to improve \u000A the speed of the Takeuchi function, exactly in the spirit of our \u000A implementation, but without the automated, system-wide integration we report \u000A here.</p> \u000A \u000A <p>The ACL2 implementation permits memoization of user-defined functions. \u000A During execution a user may enable or disable function memoization on an \u000A individual function basis, may clear memoization tables, and may even keep a \u000A stack of memoization tables.  This facility takes advantage of our \u000A implementation where we keep one copy of each distinct ACL2 data object.  Due \u000A to the functional nature of ACL2, it is sufficient to have at most one copy of \u000A any data structure; thus, a user may arrange to keep data canonicalized.  This \u000A implementation extends to the entire ACL2 system the benefits enjoyed by BDDs: \u000A canonicalization, memoization, and fast equality check.</p> \u000A \u000A <p>We have defined various algorithms using these features, and we have \u000A observed, in some cases, substantial performance increases.  For instance, we \u000A have implemented unordered set intersection and union operations that operate \u000A in time roughly linear in the size of the sets.  Without using arrays, we \u000A defined a canonical representation for Boolean functions using ACL2 objects. \u000A We have investigated the performance of rewriting and tree consensus \u000A algorithms to good effect, and we believe function memoization offers \u000A interesting opportunities to simplify algorithm definition while \u000A simultaneously providing performance improvements.</p> \u000A \u000A <p>We recommend that users focus at first on the logical definitions of <tt><see topic=\"ACL2____HONS\">hons</see></tt> and other primitives rather than their underlying Common Lisp \u000A implementations.  Integrated with this memoization system is a performance \u000A monitoring system, which can provide real-time feedback on the operation and \u000A usefulness of <tt><see topic=\"ACL2____HONS\">hons</see></tt> and function memoization.  For a more detailed \u000A description of these tools, please see the ACL2 2006 workshop paper mentioned \u000A above.</p> \u000A \u000A <p>The Fibonacci function is a small example that demonstrates the utility of \u000A function memoization.  The following definition exhibits a runtime that is \u000A exponential in its input argument.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"ACL2____MBE\">mbe</see>\u000A   :logic\u000A   (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> x) 0)\u000A         ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A         (t (<see topic=\"COMMON-LISP_____B2\">+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1)) (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))\u000A   :exec\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 2)\u000A       x\u000A     (<see topic=\"COMMON-LISP_____B2\">+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1)) (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))))</code> \u000A \u000A <p>Below we show how the ACL2 <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> utility can measure the time to \u000A execute a call to the <v>fib</v> function (with some editing to avoid noise from \u000A the underlying Common Lisp implementation).  <tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt> is actually an \u000A ACL2 macro that expands to a call of the ACL2 function used to identify a \u000A function for memoization; see <see topic=\"ACL2____MEMOIZE\">memoize</see>.  This function also accepts a \u000A well-formed term that must be true in order for the system to memoize a call \u000A of the memoized function; to ensure that an instance of the term is safe for \u000A evaluation in Common Lisp, a check is performed that if the <see topic=\"ACL2____GUARD\">guard</see> of \u000A the memoized function is satisfied, then this instance will execute without \u000A error.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TIME_42\">time$</see> (fib 40))\u000A; (EV-REC *RETURN-LAST-ARG3* ...) took\u000A; 0.99 seconds realtime, 0.98 seconds runtime\u000A; (1,296 bytes allocated).\u000A102334155\u000AACL2 !&gt;(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'fib)\u000A\u000ASummary\u000AForm:  ( TABLE MEMOIZE-TABLE ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A\u000ASummary\u000AForm:  ( PROGN (<see topic=\"ACL2____TABLE\">TABLE</see> MEMOIZE-TABLE ...) ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FIB\u000AACL2 !&gt;(<see topic=\"ACL2____TIME_42\">time$</see> (fib 40))\u000A; (EV-REC *RETURN-LAST-ARG3* ...) took\u000A; 0.00 seconds realtime, 0.00 seconds runtime\u000A; (2,864 bytes allocated).\u000A102334155\u000AACL2 !&gt;(<see topic=\"ACL2____TIME_42\">time$</see> (fib 100))\u000A; (EV-REC *RETURN-LAST-ARG3* ...) took\u000A; 0.00 seconds realtime, 0.00 seconds runtime\u000A; (7,024 bytes allocated).\u000A354224848179261915075\u000AACL2 !&gt;(<see topic=\"ACL2____UNMEMOIZE\">unmemoize</see> 'fib)</code> \u000A \u000A <p>We see that once the function <v>fib</v> is identified as a function for \u000A which function calls should be memoized, the execution times are substantially \u000A reduced.  Finally, we can prevent <v>fib</v> from being further memoized; in \u000A fact, <tt><see topic=\"ACL2____UNMEMOIZE\">unmemoize</see></tt> erases the previously memoized values.</p> \u000A \u000A <p>The implementation of hash consing, memoization, and applicative hash \u000A tables involves several facets: canonical representation of ACL2 data, \u000A function memoization, and the use of Lisp hash tables to improve the \u000A performance of association list operations.  We discuss each of these in turn, \u000A and we mention some subtle interrelationships.  Although it is not necessary \u000A to understand the discussion in this section, it may permit some users to \u000A better use this implementation.  This discussion may be confusing for some \u000A ACL2 users as it makes references to Lisp implementation features.</p> \u000A \u000A <p>The ACL2 system is actually implemented as a Lisp program that is layered \u000A on top of a Common Lisp system implementation.  ACL2 data constants are \u000A implemented with their corresponding counterparts in Common Lisp; that is, \u000A ACL2 cons pairs, strings, characters, numbers, and symbols are implemented \u000A with their specific Common Lisp counterparts.  This choice permits a number of \u000A ACL2 primitive functions to be implemented with their corresponding Common \u000A Lisp functions, but there are indeed differences.  ACL2 is a logic, and as \u000A such, it does not specify anything to do with physical storage or execution \u000A performance.  When ACL2 is used, the knowledgeable user can write functions to \u000A facilitate the reuse of some previously computed values.</p> \u000A \u000A <p>Recall the three mechanisms under discussion: hash consing, function \u000A memoization, and fast association list operations.  The function memoization \u000A mechanism takes advantage of the canonical representation of data objects \u000A provided by the <tt><see topic=\"ACL2____HONS\">hons</see></tt> operation as does the fast association list \u000A operation mechanism.  Each time <v>hons</v> is invoked, its arguments are \u000A themselves converted, if necessary, to uniquely represented objects.</p> \u000A \u000A <p>The ACL2 universe is recursively closed under the <v>cons</v> pairing \u000A operation and the atoms.  Hash consing (<tt><see topic=\"ACL2____HONS\">hons</see></tt>) is logically identical \u000A to <v>cons</v>, but a set of tables is used to record each <v>hons</v> pair.  When \u000A a <v>hons</v> pair is requested, the implementation checks, in the current set \u000A of tables, whether the requested pair already exists.  If not, a new pair is \u000A created and a record of that pair is made; otherwise, a reference to the \u000A previously created pair is returned.  Thus, any data object created with \u000A <v>hons</v> has a unique representation, as does every subcomponent.  We also \u000A arrange for strings to have a unique representation &mdash; only one copy of \u000A each different string is kept &mdash; and when any previously unseen string is \u000A an argument to <v>hons</v>, we add the string to a unique table of strings.  A \u000A system-wide benefit of having a canonical representation for data is that \u000A equality comparisons between any two data objects can be done in constant \u000A time.</p> \u000A \u000A <p>The definition of <tt><see topic=\"ACL2____HONS\">hons</see></tt> in no way changes the operation of <v>cons</v> \u000A &mdash; <v>hons</v> creates a <v>cons</v> pair.  When asked to create a <v>hons</v>, \u000A the implementation checks to see if there is a <v>cons</v> with the same <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> as the <v>hons</v> being requested.  Thus, the only \u000A difference between the results of a <v>hons</v> call and a corresponding \u000A <v>cons</v> call is a notation in some invisible (to the ACL2 logic) tables \u000A where we record which <v>cons</v> elements are also <v>hons</v> elements.  Since a \u000A <v>hons</v> is nothing but a <v>cons</v>, the operation of <v>car</v> and <v>cdr</v> \u000A is unchanged.  In fact, the implementation is designed so that at any time it \u000A is safe to clear the table identifying which <v>cons</v> elements are also \u000A considered <v>hons</v> elements.</p> \u000A \u000A <p>User-defined functions with defined and verified guards can be memoized. \u000A When a function is memoized, a user-supplied condition restricts the domain \u000A when memoization is attempted.  When the condition is satisfied, memoization \u000A is attempted (assuming that memoization for the function is presently \u000A enabled); otherwise, the function is just evaluated.  Each memoized function \u000A has a hash table that is used to keep track of a unique list of function \u000A arguments paired with their values.  If appropriate, for each function the \u000A corresponding table is checked to see if a previous call with exactly the same \u000A arguments already exists in the table: if so, then the associated value is \u000A returned; if not, then the function is evaluated and a new key-value pair is \u000A added to the table of memoized values so that some future call will benefit \u000A from the memoization.  With ACL2 user functions memoization can be dynamically \u000A enabled and disabled.  There is an ACL2 function that clears a specific \u000A memoization table.  And finally, just as with the <v>hons</v> table, a stack of \u000A these function memoization tables is maintained; that is, it is possible to \u000A memoize a function, use it a bit, set the memoized values aside, start a new \u000A table, use it, and then return to the original table.</p> \u000A \u000A <p>We next discuss the fast lookup operation for association lists.  When a \u000A pair is added to an association list using the functions <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> or \u000A <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt>, the system also records the key-value pair in an \u000A associated hash table.  As long as a user only used these two functions when \u000A placing key-value pairs on an association list, the key-value pairs in the \u000A corresponding hash table will be synchronized with the key-value pairs in the \u000A association list.  Later, if <tt><see topic=\"ACL2____HONS-GET\">hons-get</see></tt> is used to look up a key, then \u000A instead of performing a linear search of the association list we consult the \u000A associated hash table.  If a user does not strictly follow this discipline, \u000A then a linear search may be required.  In some sense, these association lists \u000A are much like ACL2 arrays, but without the burden of explicitly naming the \u000A arrays.  The ACL2 array <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> function is analogous to the \u000A functions <tt><see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see></tt> and <tt><see topic=\"ACL2____FAST-ALIST-CLEAN_12\">fast-alist-clean!</see></tt>.  There are \u000A user-level ACL2 functions that allow the associated hash tables to be cleared \u000A and removed.</p> \u000A \u000A <p>Finally, we would advise anyone who is using CCL in a research environment \u000A to stay plugged into the ``trunk'' or ``bleeding edge'' of CCL development. \u000A This is very easy to do by typing a few commands to a shell, for example \u000A standing above the target directory as follows, provided one has <v>svn</v> \u000A working.</p> \u000A \u000A <code>For linux:\u000A\u000A  rm -rf ccl\u000A  svn co http://svn.clozure.com/publicsvn/openmcl/trunk/linuxx8664/ccl\u000A\u000AFor an x86 Macintosh running the Darwin OS:\u000A\u000A  svn co http://svn.clozure.com/publicsvn/openmcl/trunk/darwinx8664/ccl\u000A\u000ATo keep up to date, you may find it sufficient to do:\u000A\u000A  cd ccl\u000A  svn update\u000A\u000AWhether obtaining a fresh CCL or just updating, finally issue these\u000Acommands.\u000A\u000A  ./lx86cl64\u000A  (rebuild-ccl :full t)\u000A  (<see topic=\"ACL2____QUIT\">quit</see>)\u000A  ./lx86cl64\u000A  (rebuild-ccl :full t)\u000A  (<see topic=\"ACL2____QUIT\">quit</see>)</code> \u000A \u000A <p>REFERENCES</p> \u000A \u000A <p>Baker, Henry G., The Boyer Benchmark at Warp Speed. ACM Lisp Pointers V,3 \u000A (Jul-Sep 1992), pages 13-14.</p> \u000A \u000A <p>Baker, Henry G., A Tachy 'TAK'.  ACM Lisp Pointers Volume 3, \u000A July-September, 1992, pages 22-23.</p> \u000A \u000A <p>Robert S. Boyer and Warren A. Hunt, Jr., Function Memoization and Unique \u000A Object Representation for ACL2 Functions, in the Sixth International Workshop \u000A on the ACL2 Theorem Prover and Its Applications, ACM Digital Library, \u000A 2006.</p> \u000A \u000A <p>A. P. Ershov.  On Programming of Arithmetic Operations.  In the \u000A Communications of the ACM, Volume 118, Number 3, August, 1958, pages \u000A 427-430.</p> \u000A \u000A <p>Eiichi Goto, Monocopy and Associative Algorithms in Extended Lisp, \u000A TR-74-03, University of Toyko, 1974.</p> \u000A \u000A <p>Richard P. Gabriel.  Performance and Evaluation of Lisp Systems.  MIT \u000A Press, 1985.</p> \u000A \u000A <p>Donald Michie.  Memo functions: a Language Feature with Rote Learning \u000A Properties.  Technical Report MIP-R-29, Department of Artificial Intelligence, \u000A University of Edinburgh, Scotland, 1967.</p> \u000A \u000A <p>Donald Michie.  Memo Functions and Machine Learning.  In Nature, Volumne \u000A 218, 1968, pages 19-22.</p>"],

 "ACL2____HINTS":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples: \u000AThe following :hints value is nonsensical.  Nevertheless, it \u000Aillustrates all of the available hint keywords except the \u000A``custom keywords'' (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) definable \u000Aby the user. \u000A \u000A:hints ((\"Goal\" \u000A         :do-not-induct t \u000A         :do-not '(generalize fertilize) \u000A         :expand ((assoc x a) \u000A                  :lambdas \u000A                  (:free (y) (:with member (member y z)))) \u000A         :restrict ((&lt;-trans ((x x) (y (foo x))))) \u000A         :hands-off (length binary-append) \u000A         :in-theory (set-difference-theories \u000A                      (current-theory :here) \u000A                      '(assoc)) \u000A         :induct (and (nth n a) (nth n b)) \u000A         :use ((:instance assoc-of-append \u000A                          (x a) (y b) (z c)) \u000A               (:functional-instance \u000A                 (:instance p-f (x a) (y b)) \u000A                 (p consp) \u000A                 (f assoc))) \u000A         :bdd (:vars (c a0 b0 a1 b1) :prove nil :bdd-constructors (cons)) \u000A         :clause-processor (:function cl-proc :hint (my-hint clause)) \u000A         :instructions (:x :prove) \u000A         :cases ((true-listp a) (consp a)) \u000A         :by (:instance rev-rev (x (cdr z))) \u000A         :nonlinearp t \u000A         :backchain-limit-rw 3 \u000A         :reorder (4 7 2) \u000A         :case-split-limitations (20 10) \u000A         :no-op t \u000A         :no-thanks t \u000A         :error (\"Bad value ~x0.\" 123) \u000A         :or (hint-kwd-alist-1 ... hint-kwd-alist-k) \u000A         :rw-cache-state nil \u000A         :backtrack (my-computed-hint clause processor clause-list))) \u000A</code> \u000A \u000A <p>A very common hint is the <v>:use</v> hint, which in general takes as its \u000A value a list of ``lemma instances'' (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>) but which \u000A allows a single lemma name as a special case.  Here are two examples, one \u000A using a single lemma name and one using a lemma instance:</p> \u000A \u000A <code>; Attach :use hint to the top-level goal, which is named &quot;Goal&quot;:\u000A:hints ((&quot;Goal&quot; :use lemma23))\u000A\u000A; Equivalent to the above: use the trivial instance (i.e., with the empty\u000A; substitution of lemma23:\u000A:hints ((&quot;Goal&quot; :use ((:instance lemma23))))\u000A\u000A; Attach :use hint to the named subgoal, where the indicated lemma is used\u000A; with the substitution that maps x to 17 and y to (foo z):\u000A:hints ((&quot;[1]Subgoal *1/1.2'&quot; :use ((:instance lemma23\u000A                                                 (x 17)\u000A                                                 (y (foo z))))))\u000A\u000A; Equivalent to the above: ACL2 allows you to omit the outer parentheses if\u000A; there is only one lemma used.\u000A:hints ((&quot;[1]Subgoal *1/1.2'&quot; :use (:instance lemma23\u000A                                                (x 17)\u000A                                                (y (foo z)))))</code> \u000A \u000A <p>ACL2 also provides ``custom keyword'' hints (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) and even more general ``computed hints'' for the \u000A advanced user (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>).</p> \u000A \u000A <p>Only the first hint applicable to a goal, as specified in the user-supplied \u000A list of <v>:hints</v> followed by the default hints (see <see topic=\"ACL2____DEFAULT-HINTS-TABLE\">default-hints-table</see>), will be applied to that goal.  For an advanced \u000A exception, see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>.  For a detailed discussion of how hints \u000A fit into the ACL2 waterfall, see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>.  For examples \u000A of the sophisticated use of hints, primarily for experts, see community book \u000A <v>books/hints/basic-tests.lisp</v>.</p> \u000A \u000A <p>Background: <v>Hints</v> are allowed in all <see topic=\"ACL2____EVENTS\">events</see> that use the \u000A theorem prover.  During <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> there are two different \u000A uses of the theorem prover: one to prove termination and another to verify the \u000A <see topic=\"ACL2____GUARD\">guard</see>s.  To pass a hint to the theorem prover during termination \u000A proofs, use the <v>:hints</v> keyword in the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'s <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> \u000A declaration.  To pass a hint to the theorem prover during the <see topic=\"ACL2____GUARD\">guard</see> \u000A verification portion of admitting a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, use the <v>:guard-hints</v> \u000A keyword in the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'s <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration.  The <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event and the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event also use the theorem prover. \u000A To pass hints to them, use the <v>:hints</v> keyword argument to the event.</p> \u000A \u000A <code>General Form of Common :hints:\u000A  ((<see topic=\"ACL2____GOAL-SPEC\">goal-spec</see> :key1 val1 ... :keyn valn)\u000A   ...\u000A   (<see topic=\"ACL2____GOAL-SPEC\">goal-spec</see> :key1 val1 ... :keyn valn))</code> \u000A \u000A <p>where <tt><see topic=\"ACL2____GOAL-SPEC\">goal-spec</see></tt> is as described elsewhere (see <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>) \u000A and the keys and their respective values are shown below with their \u000A interpretations.  We also provide ``computed hints'' but discuss them \u000A separately; see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.  The hint keywords below are considered \u000A in alphabetical order.</p> \u000A \u000A <dl> \u000A \u000A <dt><v>:backchain-limit-rw</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a natural number or <v>nil</v>, indicating the level of \u000A backchaining for <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____META\">meta</see>, and <see topic=\"ACL2____LINEAR\">linear</see> rules.  This \u000A overrides, for the current goal and (as with <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hints) \u000A descendent goals, the default <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> \u000A (see <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>).</p></dd> \u000A \u000A <dt><v>:backtrack</v></dt><p/> \u000A \u000A <dd><p>This is an advanced hint.  You can probably accomplish its effect by \u000A the use of ordinary computed hints; see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.  But if you are \u000A an expert, read on.  (See <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see> for some relevant \u000A background.)</p> \u000A \u000A <p><v>Value</v> is a computed hint, which is an expression that evaluates either \u000A to <v>nil</v> &mdash; indicating that the <v>:backtrack</v> hint is to have no \u000A effect &mdash; or to a non-empty alternating list of <v>:keyi :vali</v> pairs, \u000A as expected for a hint.  However, unlike ordinary computed hints, \u000A <v>:backtrack</v> hints are evaluated <b>after</b> a goal has been processed to \u000A yield zero or more subgoals, not before.  Moreover, variables <v>PROCESSOR</v> \u000A and <v>CLAUSE-LIST</v> are allowed, but variable \u000A <v>STABLE-UNDER-SIMPLIFICATIONP</v> is not.  We explain in more detail below, \u000A but first consider the following simple example.  First we define a standard \u000A list reversal function:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) nil))\u000A    nil))</code> \u000A \u000A <p>Now we prove:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (rev x)))</code> \u000A \u000A <p>The successful proof includes the following output.</p> \u000A \u000A <code>Subgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see>\u000A  (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A       (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))\u000A  (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))).\u000A\u000AThe destructor terms (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) and (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) can be\u000Aeliminated by using CAR-CDR-ELIM to replace X\u000Aby (<see topic=\"COMMON-LISP____CONS\">CONS</see> X1 X2), (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) by X1 and (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) by\u000AX2.  This produces the following goal.\u000A\u000ASubgoal *1/1''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X1 X2))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV X2)))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV X2) (<see topic=\"COMMON-LISP____LIST\">LIST</see> X1)))).</code> \u000A \u000A <p>But suppose that we attach a <v>:backtrack</v> hint to the goal above at \u000A which destructor elimination was applied:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (rev x))\u000A     :hints ((&quot;Subgoal *1/1'&quot;\u000A              :backtrack\u000A              (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (:do-not '(eliminate-destructors))))))</code> \u000A \u000A <p>Then when ACL2 applies destructor elimination as displayed above, this time \u000A the <v>:backtrack</v> hint applies, evaluating to <v>(:do-not\u000A '(eliminate-destructors))</v>.  Since this list is not <v>nil</v>, the prover \u000A decides not to keep the new subgoal, and instead supplies this <v>:do-not</v> \u000A hint before attacking the goal again.  In this example, ACL2 happens to use a \u000A technique later in its ``waterfall'' arsenal than destructor elimination, \u000A namely, generalization:</p> \u000A \u000A <code>Subgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see>\u000A  (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A       (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))\u000A  (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))).\u000A\u000A[Note:  A hint was supplied for our processing\u000Aof the goal above, because of a :backtrack hint\u000Athat is preventing destructor elimination.\u000AThanks!]\u000A\u000AWe generalize this conjecture, replacing\u000A(REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) by RV.  This produces\u000A\u000ASubgoal *1/1''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> RV))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> RV (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))).</code> \u000A \u000A <p>We now provide a careful explanation of how <v>:backtrack</v> hints work, but \u000A we suggest that you keep the example above in mind.  If ``<v>:backtrack\u000A form</v>'' is part of the hint that has been selected for a goal, then <v>form</v> \u000A is evaluated when one of ACL2's clause processors successfully applies to the \u000A current goal to produce a list of subgoals.  This evaluation takes place in an \u000A environment just like that for any computed hint (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>), \u000A with the following exceptions.  First, the variable \u000A <v>STABLE-UNDER-SIMPLIFICATIONP</v> is not allowed to occur free in <v>form</v>, \u000A but instead the following new variables are allowed to occur free and are \u000A bound for this evaluation as follows: <v>PROCESSOR</v> is bound to the processor \u000A in the list <v>*preprocess-clause-ledge*</v> that has applied to the goal, and \u000A <v>CLAUSE-LIST</v> is bound to the list of clauses (each a list of literals that \u000A is implicitly disjoined) returned by that clause processor.  Second, the \u000A variables <v>HIST</v> and <v>PSPV</v> are bound to the history and pspv returned \u000A by the clause processor, <b>not</b> the ones that were passed to the clause \u000A processor.  If this evaluation returns an error, then the proof aborts, as for \u000A any computed hint whose evaluation returns an error.  If this evaluation \u000A returns <v>nil</v>, then the <v>:backtrack</v> hint has no effect, and the goal is \u000A replaced by the list of goals (the value of <v>CLAUSE-LIST</v> described above), \u000A as usual.  Otherwise, the clause processor is deemed to have failed, and the \u000A goal clause is tried again starting at the top of the waterfall after \u000A selecting the hint returned by the above evaluation.  That hint will normally \u000A be an alternating list of hint keywords and their values, but if it is a \u000A custom keyword hint (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>), then it will be handled \u000A in the usual manner but with the first three variables above bound to the \u000A symbol <v>:OMITTED</v>.  Of course, if the new hint includes a value for \u000A <v>:BACKTRACK</v> then this process can loop; care should be taken to keep that \u000A from happening.</p> \u000A \u000A <p>A final note about <v>:BACKTRACK</v> hints: since these are a form of \u000A computed hints, <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> (if any) are applied to their evaluation \u000A result just as with any computed hint.  That is, the backtrack hint is \u000A successively modified with each override-hint, to produce a final hint that is \u000A actually used (or, ignored if that final hint is <v>nil</v>).  See <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>.</p></dd> \u000A \u000A <dt><v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt></dt><p/> \u000A \u000A <dd><p>This hint indicates that ACL2's built-in ordered binary decision \u000A diagrams (BDDs) with rewriting are to be used to prove or simplify the goal. \u000A See <see topic=\"ACL2____BDD\">bdd</see> for an introduction to the ACL2 BDD algorithm.</p> \u000A \u000A <p><v>Value</v> is a list of even length, such that every other element, \u000A starting with the first, is one of the keywords <v>:vars</v>, \u000A <v>:bdd-constructors</v>, <v>:prove</v>, or <v>:literal</v>.  Each keyword that is \u000A supplied should be followed by a value of the appropriate form, as shown \u000A below; for others, a default is used.  Although <v>:vars</v> must always be \u000A supplied, we expect that most users will be content with the defaults used for \u000A the other values.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>:vars</v> &mdash; A list of ACL2 variables, which are to be treated as \u000A Boolean variables.  The prover must be able to check, using trivial reasoning \u000A (see <see topic=\"ACL2____TYPE-SET\">type-set</see>), that each of these variables is Boolean in the context \u000A of the current goal.  Note that the prover will use very simple heuristics to \u000A order any variables that do not occur in <v>:vars</v> (so that they are \u000A ``greater than'' the variables that do occur in <v>:vars</v>), and these \u000A heuristics are often far from optimal.  In addition, any variables not listed \u000A may fail to be assumed Boolean by the prover, which is likely to seriously \u000A impede the effectiveness of ACL2's BDD algorithm.  Thus, users are encouraged \u000A <i>not</i> to rely on the default order, but to supply a list of variables \u000A instead.  Finally, it is allowed to use a value of <v>t</v> for <v>vars</v>.  This \u000A means the same as a <v>nil</v> value, except that the BDD algorithm is directed \u000A to fail unless it can guarantee that all variables in the input term are known \u000A to be Boolean (in a sense discussed elsewhere; see <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see>).</p> \u000A \u000A <p><v>:literal</v> &mdash; An indication of which part of the current goal \u000A should receive BDD processing.  Possible values are:</p> \u000A \u000A <code>:all     treat entire goal as a single literal (<see topic=\"COMMON-LISP____THE\">the</see> default)\u000A:conc    process the conclusion\u000An        process the hypothesis with index n (1, 2, ...)</code> \u000A \u000A <p><v>:bdd-constructors</v> &mdash; When supplied, this value should be a list \u000A of function symbols in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>; it is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see>)</v> by \u000A default, unless <v>:bdd-constructors</v> has a value in the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> by default, in which case that value is the default.  We \u000A expect that most users will be content with the default.  See <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see> for information about how this value is used.</p> \u000A \u000A <p><v>:prove</v> &mdash; When supplied, this value should be <v>t</v> or <v>nil</v>; \u000A it is <v>t</v> by default.  When the goal is not proved and this value is \u000A <v>t</v>, the entire proof will abort.  Use the value <v>nil</v> if you are happy \u000A to the proof to go on with the simplified term.</p> \u000A \u000A </blockquote></dd> \u000A \u000A <dt><v>:by</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>, <v>nil</v>, or a new event \u000A name. If the value is a <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>), \u000A then it indicates that the goal (when viewed as a clause) is either equal to \u000A the proposition denoted by the instance, or is subsumed by that proposition \u000A when both are viewed as clauses.  To view a formula as a clause, union \u000A together the negations of the hypotheses and add the conclusion.  For \u000A example,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (h1 t1) (h2 t2)) (c t1))</code> \u000A \u000A <p>may be viewed as the clause</p> \u000A \u000A <code>{~(h1 t1) ~(h2 t2) (c t1)}.</code> \u000A \u000A <p>Clause <v>c1</v> is ``subsumed'' by clause <v>c2</v> iff some instance of \u000A <v>c2</v> is a subset of <v>c1</v>.  For example, the clause above is subsumed by \u000A <v>{~(h1 x) (c x)}</v>, which when viewed as a formula is <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (h1 x) (c\u000A x))</v>.</p> \u000A \u000A <p>Note that if the value is the name of a function symbol introduced by \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, then the original form of the body of that definition is used. \u000A This behavior differs from that provided by a <v>:use</v> hint, where the \u000A so-called ``normalized'' body, for which ACL2 has propagated <v>IF</v> tests \u000A upward.</p> \u000A \u000A <p>If the value is <v>nil</v> or a new name, the prover does not even attempt to \u000A prove the goal to which this hint is attached.  Instead the goal is given a \u000A ``bye'', i.e., it is skipped and the proof attempt continues as though the \u000A goal had been proved.  If the prover terminates without error then it reports \u000A that the proof would have succeeded had the indicated goals been proved and it \u000A prints an appropriate <see topic=\"ACL2____DEFTHM\">defthm</see> form to define each of the <v>:by</v> names. \u000A The ``name'' <v>nil</v> means ``make up a name.''  Here is an example \u000A (admittedly contrived for illustration purposes).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A            :hints ((&quot;Subgoal *1/2'&quot; :by nil)))\u000A\u000AName the formula above *1.\u000A\u000A[[... output omitted here ...]]\u000A\u000A[Note:  A hint was supplied for our processing of the goal below.\u000AThanks!]\u000A\u000ASubgoal *1/2'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y) Z)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y Z)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000ABut we have been asked to pretend that this goal is subsumed by the\u000Ayet-to-be-proved |THM Subgoal *1/2'|.\u000A\u000ASubgoal *1/1\u000A[[... proof goes on; further output omitted here ...]]</code> \u000A \u000A <p>The system does not attempt to check the uniqueness of the <v>:by</v> names \u000A (supplied or made up), since by the time those goals are proved the namespace \u000A will be cluttered still further.  Therefore, the final list of ``appropriate'' \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> forms may be impossible to admit without some renaming by the \u000A user.  If you must invent new names, remember to substitute the new ones for \u000A the old ones in the <v>:by</v> hints themselves.</p></dd> \u000A \u000A <dt><v>:</v><tt><see topic=\"ACL2____CASE-SPLIT-LIMITATIONS\">case-split-limitations</see></tt></dt><p/> \u000A \u000A <dd><p><v>Value</v> is the same as for <tt><see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see></tt>.  The \u000A simplifier will behave as though the value had instead been supplied to \u000A <v>set-case-split-limitations</v>; see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>.  This \u000A behavior will persist through subgoals unless overridden by another \u000A <v>:CASE-SPLIT-LIMITATIONS</v> hint.</p></dd> \u000A \u000A <dt><v>:cases</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a non-empty list of terms.  For each term in the list, a \u000A new goal is created from the current goal by assuming that term; and also, in \u000A essence, one additional new goal is created by assuming all the terms in the \u000A list false.  We say ``in essence'' because if the disjunction of the terms \u000A supplied is a tautology, then that final goal will be a tautology and hence \u000A will in fact never actually be created.</p></dd> \u000A \u000A <dt><v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt></dt><p/> \u000A \u000A <dd><p><v>Value</v> specifies the application of a user-defined simplifier to \u000A the current goal.  See <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>, which provides necessary \u000A background and hint syntax.  Also see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see> \u000A for a discussion of ``trusted clause-processors'': goal-level simplifiers that \u000A may be external to ACL2 and do not need to be proved correct in ACL2.</p> \u000A \u000A <p>You can see all current <v>:clause-processor</v> rules by issuing the command \u000A <v>(print-clause-processor-rules)</v>, and you can see the names of all trusted \u000A clause-processors by issuing the command <v>(<see topic=\"ACL2____TABLE\">table</see>\u000A trusted-clause-processor-table)</v>.</p></dd> \u000A \u000A <dt><v>:do-not</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a term having at most the single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, which when evaluated (with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the current ACL2 \u000A logical <see topic=\"ACL2____WORLD\">world</see>) produces a list of symbols that is a subset of the \u000A list</p> \u000A \u000A <code>(preprocess ;propositional logic, simple rules\u000A simplify   ;as above plus rewriting, linear arithmetic\u000A eliminate-destructors\u000A fertilize  ;use of equalities\u000A generalize\u000A eliminate-irrelevance).</code> \u000A \u000A <p>The hint indicates that the ``processes'' named should not be used at or \u000A below the goal in question.  Thus, to prevent generalization and \u000A fertilization, say, include the hint</p> \u000A \u000A <code>:do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see> fertilize)</code> \u000A \u000A <p>If <v>value</v> is a single symbol, as in</p> \u000A \u000A <code>:do-not generalize,</code> \u000A \u000A <p>it is taken to be <v>'(value)</v>.</p></dd> \u000A \u000A <dt><v>:do-not-induct</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is <v>t</v>, <v>:otf-flg-override</v>, <v>:otf</v>, <v>name</v> or \u000A <v>nil</v>, indicating whether <see topic=\"ACL2____INDUCTION\">induction</see> is permitted under the specified \u000A goal.  If <v>value</v> is <v>t</v> or <v>:otf-flg-override</v>, then the attempt to \u000A apply <see topic=\"ACL2____INDUCTION\">induction</see> to the indicated goal or any subgoal under the \u000A indicated goal will immediately cause the theorem prover to report <see topic=\"ACL2____FAILURE\">failure</see>, except that if <v>:otf-flg t</v> is specified (see <see topic=\"ACL2____OTF-FLG\">otf-flg</see>) and \u000A <v>value</v> is <v>t</v>, then the proof will continue until the time at which the \u000A goal pushed for induction is finally considered.  The latter behavior is also \u000A what occurs if <v>value</v> is <v>:otf</v>.  Thus, any non-<v>nil</v> value requires \u000A the indicated goal to be proved entirely by simplification, destructor \u000A elimination, and the other ``waterfall'' processes.  <see topic=\"ACL2____INDUCTION\">Induction</see> to prove \u000A the indicated goal (or any subgoal) is not permitted.  See however the \u000A <v>:induct</v> hint below.  If <v>value</v> is a symbol other than <v>t</v>, \u000A <v>:otf-flg-override</v>, <v>:otf</v> or <v>nil</v>, the theorem prover will give a \u000A ``bye'' to any subgoal that would otherwise be attacked with induction.  This \u000A will cause the theorem prover to fail eventually but will collect the \u000A necessary subgoals.  If <v>value</v> is <v>nil</v>, this hint means <see topic=\"ACL2____INDUCTION\">induction</see> is permitted.  Since that is the default, there is no reason to use \u000A the value <v>nil</v>.  Note that a <v>:do-not-induct</v> hint is ignored for any \u000A goal on which an <v>:induct</v> hint is supplied.  For an advanced example of \u000A the use of value <v>:otf</v> with <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, see community book \u000A <v>books/hints/basic-tests.lisp</v>.</p></dd> \u000A \u000A <dt><v>:error</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is typically a ``fmt message'' to be printed by the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> tilde-directive ~@ but may be any object.  The effect of this hint is to \u000A cause an error when the hint is translated.  There is no reason to include an \u000A <v>:ERROR</v> hint in any user-typein, since it will only cause an error when \u000A the form is evaluated.  <v>:ERROR</v> hints are useful in the definition of \u000A functions that generate custom keyword hints (<see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) and \u000A computed hints (<see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>).  For example, if you wish to define a \u000A custom keyword hint <v>:my-hint val</v> and you wish the hint to signal an error \u000A if there is something inappropriate about <v>val</v> in the context of the hint, \u000A use the following code to generate the hint</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LIST\">list</see> :ERROR (<see topic=\"COMMON-LISP____CONS\">cons</see> &quot;Your specified value, ~x0, is inappropriate&quot;\u000A                   (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0 val))))</code> \u000A \u000A <p>which is equivalent to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LIST\">list</see> :ERROR (<see topic=\"ACL2____MSG\">msg</see> &quot;Your specified value, ~x0, is inappropriate&quot;\u000A                  val))</code> \u000A \u000A <p>which, if <v>val</v> has the value <v>123</v>, would evaluate to the hint</p> \u000A \u000A <code>(:ERROR (&quot;Your specified value, ~x0, is inappropriate&quot; (#0 . 123))).</code> \u000A \u000A <p>Note that any time an <v>:ERROR</v> keyword is produced during hint \u000A processing, including iterations of the expansions of custom keyword hints or \u000A of <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, an error will occur.</p></dd> \u000A \u000A <dt><v>:expand</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a true list of terms, each of which is of one of the \u000A forms <v>(<see topic=\"COMMON-LISP____LET\">let</see> ((v1 t1)...) b)</v> or <v>(fn t1 ... tn)</v>, where <v>fn</v> is a \u000A defined function symbol with formals <v>v1, ..., vn,</v> and <v>body</v> <v>b</v>. \u000A Such a term is said to be ``expandable:'' it can be replaced by the result of \u000A substituting the <v>ti</v>'s for the <v>vi</v>'s in <v>b</v>.  The terms listed in \u000A the <v>:expand</v> hint are expanded when they are encountered by the simplifier \u000A while working on the specified goal or any of its subgoals.  We permit \u000A <v>value</v> to be a single such term instead of a singleton list. \u000A <b>Remarks</b>: (1) Allowed are ``terms'' of the form <v>(:free (var1 var2 ...\u000A varn) pattern)</v> where the indicated variables are distinct and <v>pattern</v> \u000A is a term.  Such ``terms'' indicate that we consider the indicated variables \u000A to be instantiatable, in the following sense: whenever the simplifier \u000A encounters a term that can be obtained from <v>pattern</v> by instantiating the \u000A variables <v>(var1 var2 ...  varn)</v>, then it expands that term.  (2) Also \u000A allowed are ``terms'' of the form <v>(:with name term)</v>, where <v>name</v> is a \u000A function symbol, a macro name that denotes a function symbol (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), or a <see topic=\"ACL2____RUNE\">rune</see>.  The corresponding rule of class \u000A <v>:rewrite</v>, which is often a <see topic=\"ACL2____DEFINITION\">definition</see> rule but need not be, is \u000A then used in place of the current body for the function symbol of <v>term</v>; \u000A see <see topic=\"ACL2____SHOW-BODIES\">show-bodies</see> and see <see topic=\"ACL2____SET-BODY\">set-body</see>.  If the rule is of the form \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyp (equiv lhs rhs))</v>, then after matching <v>lhs</v> to the \u000A current term in a context that is maintaining equivalence relation <v>equiv</v>, \u000A ACL2 will replace the current term with <v>(<see topic=\"COMMON-LISP____IF\">if</see> hyp rhs (<see topic=\"ACL2____HIDE\">hide</see> term))</v>, or just \u000A <v>rhs</v> if the rule is just <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> lhs rhs)</v>.  (3) A combination of both \u000A <v>:free</v> and <v>:with</v>, as described above, is legal.  (4) The term \u000A <v>:LAMBDAS</v> is treated specially.  It denotes the list of all lambda \u000A applications (i.e., <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expressions) encountered during the proof. \u000A Conceptually, this use of <v>:LAMBDAS</v> tells ACL2 to treat lambda \u000A applications as a notation for substitutions, rather than as function calls \u000A whose opening is subject to the ACL2 rewriter's heuristics (specifically, not \u000A allowing lambda applications to open when they introduce ``too many'' if \u000A terms).</p></dd> \u000A \u000A <dt><v>:hands-off</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a true list of function symbols or lambda expressions, \u000A indicating that under the specified goal applications of these functions are \u000A not to be rewritten.  Note however that subterms will still be rewritten; see \u000A <see topic=\"ACL2____HIDE\">hide</see> if that is not what is intended.  (The community book \u000A <v>books/clause-processors/autohide.lisp</v> from Jared Davis may also be \u000A helpful in that case.) <v>Value</v> may also be a single function symbol or \u000A lambda expression instead of a list.</p></dd> \u000A \u000A <dt><v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a ``theory expression,'' i.e., a term having at most the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt> which when evaluated (with <tt><see topic=\"ACL2____WORLD\">world</see></tt> \u000A bound to the current ACL2 logical world (see <see topic=\"ACL2____WORLD\">world</see>)) will produce a \u000A theory to use as the current theory for the goal specified.  See <see topic=\"ACL2____THEORIES\">theories</see>.</p> \u000A \u000A <p>Note that an <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint will always be evaluated relative \u000A to the current ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, not relative to the theory of a \u000A previous goal.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> prop\u000A  (p (f (g x)))\u000A  :hints ((&quot;Goal&quot;      :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> f))\u000A          (&quot;Subgoal 3&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see>  g))))</code> \u000A \u000A <p>Consider in particular the theory in effect at <v>Subgoal 3</v>.  This call \u000A of the <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> macro enables <v>g</v> relative to the <tt><see topic=\"ACL2____CURRENT-THEORY\">current-theory</see></tt> of the current logical <see topic=\"ACL2____WORLD\">world</see>, <i>not</i> relative to \u000A the theory produced by the hint at <v>Goal</v>.  Thus, the <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> of \u000A <v>f</v> on behalf of the hint at <v>Goal</v> will be lost at <v>Subgoal 3</v>, and \u000A <v>f</v> will be enabled at <v>Subgoal 3</v> if was enabled globally when \u000A <v>prop</v> was submitted.</p></dd> \u000A \u000A <dt><v>:induct</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is either <v>t</v> or a term containing at least one \u000A recursively defined function symbol; if <v>t</v>, this hint indicates that the \u000A system should proceed to apply its induction heuristic to the specified goal \u000A produced (without trying simplification, etc.); if <v>value</v> is a term other \u000A than <v>t</v>, then not only should the system apply induction immediately, but \u000A it should analyze <v>value</v> rather than the goal to generate its <see topic=\"ACL2____INDUCTION\">induction</see> scheme.  Merging and the other <see topic=\"ACL2____INDUCTION\">induction</see> heuristics are \u000A applied.  Thus, if <v>value</v> contains several mergeable <see topic=\"ACL2____INDUCTION\">induction</see>s, \u000A the ``best'' will be created and chosen.  E.g., the <v>:induct</v> hint</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> i a) (<see topic=\"COMMON-LISP____NTH\">nth</see> j a))</code> \u000A \u000A <p>suggests simultaneous <see topic=\"ACL2____INDUCTION\">induction</see> on <v>i</v>, <v>j</v>, and <v>a</v>.</p> \u000A \u000A <p>If both an <v>:induct</v> and a <v>:do-not-induct</v> hint are supplied for a \u000A given goal then the indicated <see topic=\"ACL2____INDUCTION\">induction</see> is applied to the goal and the \u000A <v>:do-not-induct</v> hint is inherited by all subgoals generated.</p></dd> \u000A \u000A <dt><v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a list of <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> instructions; see <see topic=\"ACL2____INSTRUCTIONS\">instructions</see>.  Unlike other hint keywords described here, this one is \u000A actually a custom keyword hint (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) that \u000A generates a suitable <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hint.</p></dd> \u000A \u000A <dt><v>:no-op</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is any object and is irrelevant.  This hint does nothing. \u000A But empty hints, such as <v>(&quot;Goal&quot;)</v>, are illegal and there are occasions, \u000A especially when writing custom keyword hints \u000A (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) and computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) where it is convenient to be able to generate a non-empty \u000A no-op hint.  The standard idiom is <v>(&quot;Goal&quot; :NO-OP T)</v> but the <v>T</v> is \u000A completely ignored.  Unlike other hint keywords, multiple occurrences of the \u000A keyword <v>:NO-OP</v> are tolerated.</p></dd> \u000A \u000A <dt><v>:no-thanks</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is any object.  This hint does nothing, except that if \u000A <v>value</v> is non-<v>nil</v> then the usual ``[Note: A hint was \u000A supplied... Thanks!]'' is not printed.</p></dd> \u000A \u000A <dt><v>:nonlinearp</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is <v>t</v> or <v>nil</v>, indicating whether <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see> is active.  The default value is <v>nil</v>.  See <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>.</p></dd> \u000A \u000A <dt><v>:or</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a list <v>(kwd-val-listp-1 ... kwd-val-listp-k)</v>, where \u000A each <v>kwd-val-listp-i</v> is a list satisfying <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt>, \u000A i.e., an alternating list of keywords and values.  This hint causes an attempt \u000A to prove the specified goal using hints <v>kwd-val-listp-i</v> in \u000A sequence (first <v>kwd-val-listp-1</v>, then <v>kwd-val-listp-2</v>, and so on), \u000A until the first of these succeeds.  If none succeeds, then the prover proceeds \u000A after heuristically choosing the ``best'' result, taking into account the \u000A goals pushed in each case for proof by induction.</p> \u000A \u000A <p>The following (contrived but illustrative example illustrates how <v>:or</v> \u000A hints work.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (f x)\u000A            :hints\u000A            ((&quot;Goal&quot;\u000A              :expand ((<see topic=\"COMMON-LISP____NTH\">nth</see> x 3))\u000A              :or ((:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> car-cons))\u000A                   (:use cdr-cons :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> append)))\u000A              :do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see>))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AThe :OR hint for Goal gives rise to two disjunctive branches.  Proving\u000Aany one of these branches would suffice to prove Goal.  We explore\u000Athem in turn, describing their derivations as we go.\u000A\u000A---\u000ASubgoal D2\u000A( same formula as Goal ).\u000A\u000AThe first disjunctive branch (of 2) for Goal can be created by applying\u000Athe hint:\u000A(&quot;Subgoal D2&quot; :EXPAND ((<see topic=\"COMMON-LISP____NTH\">NTH</see> X 3))\u000A              :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> CAR-CONS)\u000A              :DO-NOT '(<see topic=\"ACL2____GENERALIZE\">GENERALIZE</see>)).\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000ANormally we would attempt to prove this formula by induction.  However,\u000Awe prefer in this instance to focus on the original input conjecture\u000Arather than this simplified special case.  We therefore abandon our\u000Aprevious work on this conjecture and reassign the name *1 to the original\u000Aconjecture.  (See :DOC otf-flg.)  [Note:  Thanks again for the hint.]\u000A\u000A---\u000ASubgoal D1\u000A( same formula as Goal ).\u000A\u000AThe second disjunctive branch (of 2) for Goal can be created by applying\u000Athe hint:\u000A(&quot;Subgoal D1&quot; :EXPAND ((<see topic=\"COMMON-LISP____NTH\">NTH</see> X 3))\u000A              :USE CDR-CONS\u000A              :IN-THEORY (<see topic=\"ACL2____ENABLE\">ENABLE</see> APPEND)\u000A              :DO-NOT '(<see topic=\"ACL2____GENERALIZE\">GENERALIZE</see>)).\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AACL2 Warning [Use] in ( THM ...):  It is unusual to :USE an enabled\u000A:REWRITE or :DEFINITION rule, so you may want to consider disabling\u000A(:REWRITE CDR-CONS).\u000A\u000AWe augment the goal with the hypothesis provided by the :USE hint.\u000AThe hypothesis can be obtained from CDR-CONS.  We are left with the\u000Afollowing subgoal.\u000A\u000ASubgoal D1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X Y)) Y)\u000A         (F X)).\u000A\u000ABy the simple :rewrite rule CDR-CONS we reduce the conjecture to\u000A\u000ASubgoal D1''\u000A(F X).</code> \u000A \u000A <p>... and so on.  This example illustrates how ACL2 processes <v>:or</v> hints \u000A in general.  For each <v>i</v> from 1 to <v>k</v>, a so-called ``disjunctive'' \u000A subgoal is created by splicing <v>kwd-val-listp-i</v> into the other hint values \u000A (if any) supplied for the given goal, in order.  A corresponding subgoal is \u000A created for each <v>i</v>, numbered in the usual manner (hence, counting down) \u000A except that the ``<v>D</v>'' is prefixed to each resulting goal.</p></dd> \u000A \u000A <dt><v>:reorder</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is a list of positive integers without duplicates, \u000A corresponding to the numbering of subgoals generated for the <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see> \u000A <v>&quot;G&quot;</v>, say <v>&quot;G.k&quot;</v> down to <v>&quot;G.1&quot;</v>.  Those subgoals are \u000A reordered so that if <v>value</v> is <v>(n1 n2 ... nk)</v>, then the goal now \u000A numbered <v>&quot;G.k&quot;</v> will be the goal originally numbered <v>&quot;G.n1&quot;</v>; the \u000A goal now numbered <v>&quot;G.k-1&quot;</v> will be the goal formerly numbered \u000A <v>&quot;G.n2&quot;</v>; and so on, down the list of <v>ni</v>, after which the goals not \u000A yet printed are printed in their original order.  Note that reordering for \u000A subgoals of a goal to be proved by induction, such as <v>*1</v>, is not \u000A supported.</p></dd> \u000A \u000A <dt><v>:restrict</v></dt><p/> \u000A \u000A <dd><p>Warning: This is a sophisticated hint, suggested by Bishop Brock, that \u000A is intended for advanced users.  In particular, <v>:restrict</v> hints are \u000A ignored by the preprocessor, so you might find it useful to give the hint \u000A <v>:do-not '(preprocess)</v> when using any <v>:restrict</v> hints, at least if \u000A the rules in question are abbreviations (see <see topic=\"ACL2____SIMPLE\">simple</see>).</p> \u000A \u000A <p><v>Value</v> is an association list.  Its members are of the form <v>(x\u000A subst1 subst2 ...)</v>, where: <v>x</v> is either (1) a <see topic=\"ACL2____RUNE\">rune</see> whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> or (2) an event name \u000A corresponding to one or more such <see topic=\"ACL2____RUNE\">rune</see>s; and <v>(subst1 subst2 ...)</v> \u000A is a non-empty list of substitutions, i.e., of association lists pairing \u000A variables with terms.  First consider the case that <v>x</v> is a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>.  Recall that without this \u000A hint, the rule named <v>x</v> is used by matching its left-hand side (call it \u000A <v>lhs</v>) against the term currently being considered by the rewriter, that \u000A is, by attempting to find a substitution <v>s</v> such that the instantiation of \u000A <v>lhs</v> using <v>s</v> is equal to that term.  If however the <v>:restrict</v> \u000A hint contains <v>(x subst1 subst2 ...)</v>, then this behavior will be modified \u000A by restricting <v>s</v> so that it must extend <v>subst1</v>; and if there is no \u000A such <v>s</v>, then <v>s</v> is restricted so that it must extend <v>subst2</v>; and \u000A so on, until the list of substitutions is exhausted.  If no such <v>s</v> is \u000A found, then the rewrite or definition rule named <v>x</v> is not applied to that \u000A term.  Finally, if <v>x</v> is an event name corresponding to one or more \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s (that is, \u000A <v>x</v> is the ``base symbol'' of such <see topic=\"ACL2____RUNE\">rune</see>s; see <see topic=\"ACL2____RUNE\">rune</see>), say \u000A <see topic=\"ACL2____RUNE\">rune</see>s <v>r1</v>, ... <v>rn</v>, then the meaning is the same except that \u000A <v>(x subst1 subst2 ...)</v> is replaced by <v>(ri subst1 subst2 ...)</v> for each \u000A <v>i</v>.  Once this replacement is complete, the hint may not contain two \u000A members whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is the same <see topic=\"ACL2____RUNE\">rune</see>.</p> \u000A \u000A <p>Note that the substitutions in <v>:restrict</v> hints refer to the variables \u000A actually appearing in the goals, not to the variables appearing in the rule \u000A being restricted.</p> \u000A \u000A <p>Here is an example, supplied by Bishop Brock.  Suppose that the database \u000A includes the following rewrite rule, which is probably kept <see topic=\"ACL2____DISABLE\">disable</see>d. \u000A (We ignore the question of how to prove this rule.)</p> \u000A \u000A <code>cancel-&lt;-*$free:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A              (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)\u000A              (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> z))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y z)\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A                    (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y) (<see topic=\"COMMON-LISP_____A2\">*</see> x z))\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 0)\u000A                      (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y) (<see topic=\"COMMON-LISP_____A2\">*</see> x z))\u000A                    (<see topic=\"ACL2____HIDE\">hide</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y z))))))</code> \u000A \u000A <p>Then ACL2 can prove the following theorem (unless other rules get in the \u000A way), essentially by multiplying both sides by <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 x))\u000A           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____F2\">/</see> x) 1))\u000A  :hints\u000A  ((&quot;Goal&quot;\u000A    :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> cancel-&lt;-*$free)\u000A    :restrict ((cancel-&lt;-*$free ((x x) (y (<see topic=\"COMMON-LISP_____F2\">/</see> x)) (z 1)))))))</code> \u000A \u000A <p>The <v>:restrict</v> hint above says that the variables <v>x</v>, <v>y</v>, and \u000A <v>z</v> in the rewrite rule <v>cancel-&lt;-*$free</v> above should be instantiated \u000A respectively by <v>x</v>, <v>(<see topic=\"COMMON-LISP_____F2\">/</see> x)</v>, and <v>1</v>.  Thus <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> y z)</v> becomes \u000A <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____F2\">/</see> x) 1)</v>, and this inequality is replaced by the corresponding \u000A instance of the right-hand-side of <v>cancel-&lt;-*$free</v>.  Since the current \u000A conjecture assumes <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 x)</v>, that instance of the right-hand side \u000A simplifies to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x (<see topic=\"COMMON-LISP_____F2\">/</see> x)) (<see topic=\"COMMON-LISP_____A2\">*</see> x 1))</code> \u000A \u000A <p>which in turn simplifies to <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 x)</v>, a hypothesis in the present \u000A theorem.</p></dd> \u000A \u000A <dt><v>:rw-cache-state</v></dt><p/> \u000A \u000A <dd><p><v>Value</v> is an element of the list constant \u000A <v>*legal-rw-cache-states*</v>: <v>:atom</v> (the default), <v>nil</v>, <v>t</v>, or \u000A <v>:disabled</v>.  This hint applies to the indicated goal and all its \u000A descendents, to set the so-called ``rw-cache-state'' to the indicated value; \u000A see <see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see>.</p></dd> \u000A \u000A <dt><v>:use</v></dt><p/> \u000A \u000A <dd><p>Examples of <v>:USE</v> hints are shown near the top of this \u000A documentation topic.</p> \u000A \u000A <p><v>Value</v> is a <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> or a true list of <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>s, indicating that the propositions denoted by the instances be \u000A added as hypotheses to the specified goal.  See <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>.  Note \u000A that <v>:use</v> makes the given instances available as ordinary hypotheses of \u000A the formula to be proved.  The <v>:instance</v> form of a <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> \u000A permits you to instantiate the free variables of previously proved theorems \u000A any way you wish; but it is up to you to provide the appropriate \u000A instantiations because once the instances are added as hypotheses their \u000A variables are no longer instantiable.  These new hypotheses participate fully \u000A in all subsequent rewriting, etc.  If the goal in question is in fact an \u000A instance of a previously proved theorem, you may wish to use <v>:by</v> below. \u000A Note that <see topic=\"ACL2____THEORIES\">theories</see> may be helpful when employing <v>:use</v> hints; see \u000A <see topic=\"ACL2____MINIMAL-THEORY\">minimal-theory</see>.</p> \u000A \u000A <p>Note that if the value is the name of a function symbol introduced by \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, then the ``normalized'' body of that definition is used, for \u000A which ACL2 has propagated <v>IF</v> tests upward.  This behavior differs from \u000A that provided by a <v>:by</v> hint, where the original body of the definition is \u000A used.</p></dd> \u000A \u000A </dl>"],

 "ACL2____FORCE":[["Miscellaneous","Rewrite"],"ACL2 Sources","ACL2","<p><v>Force</v> is the identity function: <v>(<see topic=\"ACL2____FORCE\">force</see> x)</v> is equal to \u000A <v>x</v>.  However, for rules of many classes (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>), a \u000A hypothesis of the form <v>(<see topic=\"ACL2____FORCE\">force</see> term)</v> is given special treatment, as \u000A described below.  This treatment takes place for rule classes <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> (actually in that case, the result of \u000A evaluating the hypothesis metafunction call), and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>.</p> \u000A \u000A <p>When a hypothesis of a conditional rule (of one of the classes listed \u000A above) has the form <v>(<see topic=\"ACL2____FORCE\">force</see> hyp)</v>, it is logically equivalent to <v>hyp</v> \u000A but has a pragmatic effect.  In particular, when the rule is considered, the \u000A needed instance of the hypothesis, <v>hyp'</v>, may be assumed if the usual \u000A process fails to prove it or its negation.  In that situation, if the rule is \u000A eventually applied, then a special case is generated, requiring the system to \u000A prove that <v>hyp'</v> is true in the current context.  The proofs of all such \u000A ``forced assumptions'' are, by default, delayed until the successful \u000A completion of the main goal.  See <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see> and see <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>.</p> \u000A \u000A <p>Note that the only time that ACL2 gives special treatment to calls of \u000A <v>force</v> is when it is considering the hypotheses of a conditional rule, as \u000A discussed above.  In particular, when the rewriter encounters a subterm of the \u000A goal currently being simplified, a call of <v>force</v> is not treated \u000A specially.  For example, if you provide a <v>:use</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>) \u000A that replaces a goal <v>G</v> by the goal</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ... (<see topic=\"ACL2____FORCE\">force</see> HYP) ...)\u000A                  concl)\u000A         G)</code> \u000A \u000A <p>then the rewriter will not give any special treatment to <tt>(force \u000A HYP)</tt>.  Instead, it will first rewrite <v>HYP</v> to, say, <v>HYP'</v>; and \u000A then, using the fact that <v>force</v> is the identity function, the rewriter \u000A will return <v>HYP'</v> as the rewritten value for <tt>(force HYP)</tt>.</p> \u000A \u000A <p>Forcing is generally used on hypotheses that are always expected to be \u000A true, as is commonly the case for <see topic=\"ACL2____GUARD\">guard</see>s of functions.  All the power \u000A of the theorem prover is brought to bear on a forced hypothesis and no \u000A backtracking is possible.  Forced goals can be attacked immediately (see <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>) or in a subsequent forcing round (see <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>).  Also see <see topic=\"ACL2____CASE-SPLIT\">case-split</see> for a related utility.  If the \u000A <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of the function <v>force</v> is <see topic=\"ACL2____DISABLE\">disable</see>d, then no hypothesis is forced.  For more on enabling and disabling \u000A forcing, see <see topic=\"ACL2____ENABLE-FORCING\">enable-forcing</see> and see <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>.</p> \u000A \u000A <p>It sometimes happens that a conditional rule is not applied because some \u000A hypothesis, <v>hyp</v>, could not be relieved, even though the required instance \u000A of <v>hyp</v>, <v>hyp'</v>, can be shown true in the context.  This happens when \u000A insufficient resources are brought to bear on <v>hyp'</v> at the time we try to \u000A relieve it.  A sometimes desirable alternative behavior is for the system to \u000A assume <v>hyp'</v>, apply the rule, and to generate explicitly a special case to \u000A show that <v>hyp'</v> is true in the context.  This is called ``forcing'' \u000A <v>hyp</v>.  It can be arranged by restating the rule so that the offending \u000A hypothesis, <v>hyp</v>, is embedded in a call of <v>force</v>, as in <v>(<see topic=\"ACL2____FORCE\">force</see>\u000A hyp)</v>.  By using the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> field of the <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> entry, a hypothesis can be forced without changing the statement \u000A of the theorem from which the rule is derived.</p> \u000A \u000A <p>Technically, <v>force</v> is just a function of one argument that returns \u000A that argument.  It is generally <see topic=\"ACL2____ENABLE\">enable</see>d and hence evaporates during \u000A simplification.  But its presence among the hypotheses of a conditional rule \u000A causes case splitting to occur if the hypothesis cannot be conventionally \u000A relieved.</p> \u000A \u000A <p>Since a forced hypothesis must be provable whenever the rule is otherwise \u000A applicable, forcing should be used only on hypotheses that are expected always \u000A to be true.</p> \u000A \u000A <p>A particularly common situation in which some hypotheses should be forced \u000A is in ``most general'' <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> lemmas.  If a single lemma \u000A describes the ``expected'' type of a function, for all ``expected'' arguments, \u000A then it is probably a good idea to force the hypotheses of the lemma.  Thus, \u000A every time a term involving the function arises, the term will be given the \u000A expected type and its arguments will be required to be of the expected type. \u000A In applying this advice it might be wise to avoid forcing those hypotheses \u000A that are in fact just type predicates on the arguments, since the routine that \u000A applies <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> lemmas has fairly thorough knowledge of the \u000A types of all terms.</p> \u000A \u000A <p><v>Force</v> can have the additional benefit of causing the ACL2 typing \u000A mechanism to interact with the ACL2 rewriter to establish the hypotheses of \u000A <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.  To understand this remark, think of the ACL2 \u000A type reasoning system as a rather primitive rule-based theorem prover for \u000A questions about Common Lisp types, e.g., ``does this expression produce a \u000A <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>?''  ``does this expression produce some kind of ACL2 number, \u000A e.g., an <tt><see topic=\"COMMON-LISP____INTEGERP\">integerp</see></tt>, a <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt>, or a <tt><see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see></tt>?'' etc.  It is driven by <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules. \u000A To relieve the hypotheses of such rules, the type system recursively invokes \u000A itself.  This can be done for any hypothesis, whether it is ``type-like'' or \u000A not, since any proposition, <v>p</v>, can be phrased as the type-like question \u000A ``does <v>p</v> produce an object of type <v>nil</v>?''  However, as you might \u000A expect, the type system is not very good at establishing hypotheses that are \u000A not type-like, unless they happen to be assumed explicitly in the context in \u000A which the question is posed, e.g., ``If <v>p</v> produces a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> then \u000A does <v>p</v> produce <v>nil</v>?''  If type reasoning alone is insufficient to \u000A prove some instance of a hypothesis, then the instance will not be proved by \u000A the type system and a <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rule with that hypothesis will \u000A be inapplicable in that case.  But by embedding such hypotheses in <v>force</v> \u000A expressions you can effectively cause the type system to ``punt'' them to the \u000A rest of the theorem prover.  Of course, as already noted, this should only be \u000A done on hypotheses that are ``always true.''  In particular, if rewriting is \u000A required to establish some hypothesis of a <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rule, then \u000A the rule will be found inapplicable because the hypothesis will not be \u000A established by type reasoning alone.</p> \u000A \u000A <p>The ACL2 rewriter uses the type reasoning system as a subsystem.  It is \u000A therefore possible that the type system will force a hypothesis that the \u000A rewriter could establish.  Before a forced hypothesis is reported out of the \u000A rewriter, we try to establish it by rewriting.</p> \u000A \u000A <p>This makes the following surprising behavior possible: A <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rule fails to apply because some true hypothesis is not \u000A being relieved.  The user changes the rule so as to <b>force</b> the \u000A hypothesis.  The system then applies the rule but reports no forcing.  How can \u000A this happen?  The type system ``punted'' the forced hypothesis to the \u000A rewriter, which established it.</p> \u000A \u000A <p>Finally, we should mention that the rewriter is never willing to force when \u000A there is an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> term present in the goal being simplified.  Since \u000A <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> terms and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> terms are merely abbreviations for <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> \u000A terms, they also prevent forcing.  Note that <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> terms are ultimately \u000A eliminated using the ordinary flow of the proof (but see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>), allowing <v>force</v> ultimately to function as \u000A intended.  Moreover, forcing can be disabled, as described above; also see \u000A <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>force</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> force (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "ACL2____ENCAPSULATE":[["Events"],"ACL2 Sources","ACL2","<p><v>Encapsulate</v> provides a way to execute a sequence of <see topic=\"ACL2____EVENTS\">events</see> and then hide some of the resulting effects.  There are two kinds of \u000A encapsulations: ``trivial'' and ``non-trivial''.  We discuss these briefly \u000A before providing detailed <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>A trivial encapsulation is an event of the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A () ; nil here indicates &quot;trivial&quot;\u000A &lt;event-1&gt;\u000A ...\u000A &lt;event-k&gt;)</code> \u000A \u000A <p>We use the term ``sub-events'' to refer to <v>&lt;event-1&gt;</v> through \u000A <v>&lt;event-k&gt;</v>.  Each sub-event <v>&lt;event-i&gt;</v> may be ``<see topic=\"ACL2____LOCAL\">local</see>'', that \u000A is, of the form <v>(<see topic=\"ACL2____LOCAL\">local</see> &lt;event-i'&gt;)</v>; the other sub-events are called \u000A ``non-local''.  When this <v>encapsulate</v> form is submitted to ACL2, it is \u000A processed in two passes.  On the first pass, each sub-event is printed (by \u000A default) and processed in sequence; admission of the <v>encapsulate</v> fails if \u000A any <v>&lt;event-i&gt;</v> fails to be admitted.  Then a second pass is made after \u000A rolling back the logical <see topic=\"ACL2____WORLD\">world</see> to what it was just before executing the \u000A <v>encapsulate</v> form.  In the second pass, only the non-<see topic=\"ACL2____LOCAL\">local</see> forms \u000A <v>&lt;event-i&gt;</v> are evaluated, again in order, and proofs are skipped.</p> \u000A \u000A <p>For example, the following trivial encapsulation exports a single event, \u000A <v>member-equal-reverse</v>.  The lemma <v>member-revappend</v> is used (as a \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> rule) to prove <v>member-equal-reverse</v> on the first pass, but \u000A since <v>member-revappend</v> is <see topic=\"ACL2____LOCAL\">local</see>, it is ignored on the second \u000A (final) pass.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see>\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> member-revappend\u000A    (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y))\u000A         (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a x)\u000A             (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a y)))\u000A    :hints ((&quot;Goal&quot; :induct (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y)))))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> member-equal-reverse\u000A   (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x))\u000A        (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a x))))</code> \u000A \u000A <p>Of course, one might prefer to prove these <see topic=\"ACL2____EVENTS\">events</see> at the top level, \u000A rather than within an encapsulation; but the point here is to illustrate that \u000A you can have <see topic=\"ACL2____LOCAL\">local</see> <see topic=\"ACL2____EVENTS\">events</see> that do not become part of the logical \u000A <see topic=\"ACL2____WORLD\">world</see>.  (Such a capability is also provided at the level of <see topic=\"ACL2____BOOKS\">books</see>; in particular, see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.)</p> \u000A \u000A <p>On the other hand, non-trivial encapsulations provide a way to introduce \u000A axioms about new function symbols, without introducing inconsistency and \u000A without introducing complete definitions.  The following example illustrates \u000A how that works.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A\u000A; The following list has a single signature, introducing a function foo of\u000A; one argument that returns one value.  (<see topic=\"COMMON-LISP____THE\">The</see> list is non-empty, so we call\u000A; this a &quot;non-trivial&quot; encapsulation.)\u000A ( ((foo *) =&gt; *) )\u000A\u000A; Introduce a ``witness'' (example) for foo, marked as local so that\u000A; it is not exported:\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))\u000A\u000A; Introduce a non-local property to be exported:\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-preserves-consp\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (<see topic=\"COMMON-LISP____CONSP\">consp</see> (foo x))))\u000A)</code> \u000A \u000A <p>The form above introduces a new function symbol, <v>foo</v>, with the \u000A indicated property and no definition.  In fact, the output from ACL2 concludes \u000A as follows.</p> \u000A \u000A <code>The following constraint is associated with the function FOO:\u000A\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (FOO X)))</code> \u000A \u000A <p>To understand this example, we consider how non-trivial encapsulations are \u000A processed.  The same two passes are made as for trivial encapsulations, and \u000A the (<see topic=\"ACL2____LOCAL\">local</see>) definition of <v>foo</v> is ignored on the second pass, and \u000A hence does not appear in the resulting ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.  But before \u000A the second pass, each <see topic=\"ACL2____SIGNATURE\">signature</see> is stored in the <see topic=\"ACL2____WORLD\">world</see>.  Thus, \u000A when the theorem <v>foo-preserves-consp</v> is encountered in the second pass, \u000A <v>foo</v> is a known function symbol with the indicated signature.</p> \u000A \u000A <p>If any event fails while evaluating a call of <v>encapsulate</v>, the entire \u000A <v>encapsulate</v> call is deemed to have failed, and the logical <see topic=\"ACL2____WORLD\">world</see> \u000A is rolled back to what it was immediately before the <v>encapsulate</v> \u000A call.</p> \u000A \u000A <p>We now provide detailed documentation.  But discussion of redundancy for \u000A <v>encapsulate</v> events may be found elsewhere; see <see topic=\"ACL2____REDUNDANT-ENCAPSULATE\">redundant-encapsulate</see>.</p> \u000A \u000A <code>Other Examples:\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((an-element *) =&gt; *))\u000A\u000A; The list of signatures above could also be written\u000A;            ((an-element (lst) t))\u000A\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> an-element (lst)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> lst) (<see topic=\"COMMON-LISP____CAR\">car</see> lst) nil)))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> member-equal-car\u000A            (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> lst (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst))\u000A                     (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst) lst))))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> thm1\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NULL\">null</see> lst) (<see topic=\"COMMON-LISP____NULL\">null</see> (an-element lst))))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> thm2\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NULL\">null</see> lst)))\u000A              (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (an-element lst) lst))))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A () ; empty signature: no constrained functions indicated\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> hack\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep x))\u000A                        (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep y)))\u000A                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y z)\u000A                          (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z)))))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> nthcdr-add1-conditional\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ZP\">zp</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> n)))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> n) x)\u000A                   (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (<see topic=\"ACL2____SIGNATURE\">signature</see> ... signature)\u000A  ev1\u000A  ...\u000A  evn)</code> \u000A \u000A <p>where each <tt><see topic=\"ACL2____SIGNATURE\">signature</see></tt> is a well-formed signature, each \u000A <v>signature</v> describes a different function symbol, and each <v>evi</v> is an \u000A embedded event form (See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).  Also see <see topic=\"ACL2____SIGNATURE\">signature</see>, in particular for a discussion of how a signature can assign a \u000A <see topic=\"ACL2____GUARD\">guard</see> to a function symbol.  There must be at least one <v>evi</v>.  The \u000A <v>evi</v> inside <tt><see topic=\"ACL2____LOCAL\">local</see></tt> special forms are called ``local'' <see topic=\"ACL2____EVENTS\">events</see> \u000A below.  <see topic=\"ACL2____EVENTS\">Events</see> that are not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> are sometimes said to be \u000A ``exported'' by the encapsulation.  We make the further restriction that no \u000A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> event may be introduced in the scope of an <v>encapsulate</v> \u000A (not even by <v>encapsulate</v> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> events that are among \u000A the <v>evi</v>).  Furthermore, no non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event \u000A is permitted in the scope of any <v>encapsulate</v> with a non-empty list of \u000A signatures.</p> \u000A \u000A <p>To be well-formed, an <v>encapsulate</v> event must have the properties that \u000A each event in the body (including the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> ones) can be successfully \u000A executed in sequence and that in the resulting theory, each function mentioned \u000A among the <see topic=\"ACL2____SIGNATURE\">signature</see>s was introduced via a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> event and has \u000A the <see topic=\"ACL2____SIGNATURE\">signature</see> listed.  (A utility is provided to assist in debugging \u000A failures of such execution; see <see topic=\"ACL2____REDO-FLAT\">redo-flat</see>.)  In addition, the body may \u000A contain no ``local incompatibilities'' which, roughly stated, means that the \u000A <see topic=\"ACL2____EVENTS\">events</see> that are not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> must not syntactically require \u000A symbols defined by <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____EVENTS\">events</see></tt>, except for the functions \u000A listed in the <see topic=\"ACL2____SIGNATURE\">signature</see>s.  See <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see>.  Finally, \u000A no non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> recursive definition in the body may involve in its \u000A suggested induction scheme any function symbol listed among the <see topic=\"ACL2____SIGNATURE\">signature</see>s.  See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.</p> \u000A \u000A <p>Observe that if the <see topic=\"ACL2____SIGNATURE\">signature</see>s list is empty, the resulting \u000A ``trivial'' <v>encapsulate</v> may still be useful for deriving theorems to be \u000A exported whose proofs require lemmas you prefer to hide (i.e., made <tt><see topic=\"ACL2____LOCAL\">local</see></tt>).  Whether trivial or not (i.e., whether the signature is empty or \u000A not), <v>encapsulate</v> exports the results of evaluating its non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see>, but its <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are ignored for the \u000A resulting logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>The result of a non-trivial <v>encapsulate</v> event is an extension of the \u000A logic in which, roughly speaking, the functions listed in the <see topic=\"ACL2____SIGNATURE\">signature</see>s are constrained to have the <see topic=\"ACL2____SIGNATURE\">signature</see>s listed and to \u000A satisfy the non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> theorems proved about them.  In fact, other \u000A functions introduced in the <v>encapsulate</v> event may be considered to have \u000A ``<see topic=\"ACL2____CONSTRAINT\">constraint</see>s'' as well.  (See <see topic=\"ACL2____CONSTRAINT\">constraint</see> for details, which are \u000A only relevant to functional instantiation.)  Since the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s were \u000A all theorems in the ``ephemeral'' or ``local'' theory, we are assured that the \u000A extension produced by <v>encapsulate</v> is sound.  In essence, the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> definitions of the constrained functions are just ``witness functions'' \u000A that establish the consistency of the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s.  Because those \u000A definitions are <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, they are not present in the theory produced by \u000A encapsulation.  After a non-trivial <v>encapsulate</v> event is admitted, \u000A theorems about the constrained function symbols may then be proved &mdash; \u000A theorems whose proofs necessarily employ only the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s.  Thus, \u000A those theorems may be later functionally instantiated, as with the \u000A <v>:functional-instance</v> lemma instance (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>), to \u000A derive analogous theorems about different functions, provided the constraints \u000A (see <see topic=\"ACL2____CONSTRAINT\">constraint</see>) can be proved about the new functions.</p> \u000A \u000A <p>The <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> for the first event in an encapsulation is \u000A the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> ``outside'' the encapsulation.  But since <see topic=\"ACL2____EVENTS\">events</see> changing the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> are permitted within the body of an \u000A <v>encapsulate</v>, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> may be changed.  However, \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> changes occurring within the body of the <v>encapsulate</v> \u000A are not exported.  In particular, the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> after an \u000A <v>encapsulate</v> is always the same as it was before the <v>encapsulate</v>, \u000A even though the <v>encapsulate</v> body might contain <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> changing \u000A <see topic=\"ACL2____EVENTS\">events</see>, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.  More generally, after execution of an <v>encapsulate</v> event, \u000A the value of <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> is restored to what it was \u000A immediately before that event was executed.  See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>We make some remarks on <see topic=\"ACL2____GUARD\">guard</see>s and evaluation.  Calls of functions \u000A introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see>s list cannot be evaluated in the ACL2 \u000A read-eval-print loop.  See <see topic=\"ACL2____DEFATTACH\">defattach</see> for a way to overcome this \u000A limitation.  Moreover, any <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> supplied in the signature is \u000A automatically associated in the <see topic=\"ACL2____WORLD\">world</see> with its corresponding function \u000A symbol, with no requirement other than that the guard is a legal term all of \u000A whose function symbols are in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode with their <see topic=\"ACL2____GUARD\">guard</see>s verified.  In particular, there need not be any relationship between a \u000A guard in a signature and the guard in a <v>local</v> witness function.  Finally, \u000A note that for functions introduced non-<see topic=\"ACL2____LOCAL\">local</see>ly inside a non-trivial \u000A <v>encapsulate</v> event, <see topic=\"ACL2____GUARD\">guard</see> verification is illegal unless ACL2 \u000A determines that the proof obligations hold outside the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A event as well.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((f (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A ;; ERROR!\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (f x)))\u000A   (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</code> \u000A \u000A <p>The order of the <see topic=\"ACL2____EVENTS\">events</see> in the vicinity of an <v>encapsulate</v> is \u000A confusing.  We discuss it in some detail here because when logical names are \u000A being used with theory functions to compute sets of rules, it is sometimes \u000A important to know the order in which <see topic=\"ACL2____EVENTS\">events</see> were executed.  (See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see> and see <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see>.)  What, for example, is the set \u000A of function names extant in the middle of an encapsulation?</p> \u000A \u000A <p>If the most recent event is <v>previous</v> and then you execute an \u000A <v>encapsulate</v> constraining <v>an-element</v> with two non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in its body, <v>thm1</v> and <v>thm2</v>, then the order of the <see topic=\"ACL2____EVENTS\">events</see> after the encapsulation is (reading chronologically forward): \u000A <v>previous</v>, <v>thm1</v>, <v>thm2</v>, <v>an-element</v> (the <v>encapsulate</v> \u000A itself).  Actually, between <v>previous</v> and <v>thm1</v> certain extensions \u000A were made to the <see topic=\"ACL2____WORLD\">world</see> by the superior <v>encapsulate</v>, to permit \u000A <v>an-element</v> to be used as a function symbol in <v>thm1</v>.</p> \u000A \u000A <p>Remark for ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).  For ACL2(r), <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> can \u000A be used to introduce classical and non-classical functions, as determined by \u000A the signatures; see <see topic=\"ACL2____SIGNATURE\">signature</see>.  Those marked as classical (respectively \u000A non-classical) must have classical (respectively, non-classical) <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A witness functions.  A related requirement applies to functional instantiation; \u000A see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>.</p>"],

 "ACL2____CERTIFY-BOOK":[["Books-reference","Books-tour"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot;)          ; certify in a world with 0 commands\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 3)        ; ... in a world with 3 commands\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; ?)        ; ... in a world without checking the\u000A                                     ;     number of commands\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 0 nil)    ; ... without compilation\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 0 t)      ; ... with compilation (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 0 t :ttags (foo))\u000A                                     ; ... allowing trust tag (ttag) foo\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 0 t :ttags :all)\u000A                                     ; ... allowing all trust tags (ttags)\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; t)        ; ... from world of old certificate\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-arith&quot; 0 nil :acl2x t)\u000A                                     ; ... writing or reading a .acl2x file\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> book-name\u000A              k                           ; [default 0]\u000A              compile-flg                 ; [default t]\u000A              :defaxioms-okp t/nil        ; [default nil]\u000A              :skip-proofs-okp t/nil      ; [default nil]\u000A              :ttags ttags                ; [default nil]\u000A              :acl2x t/nil                ; [default nil]\u000A              :ttagsx ttags               ; [default nil]\u000A              :write-port t/nil           ; [default t]\u000A              :pcert pcert                ; [default nil]\u000A              )</code> \u000A \u000A <p>where <v>book-name</v> is a book name (see <see topic=\"ACL2____BOOK-NAME\">book-name</see>), <v>k</v> is used \u000A to indicate your approval of the ``certification <see topic=\"ACL2____WORLD\">world</see>,'' and \u000A <v>compile-flg</v> can control whether the book is to be compiled.  The defaults \u000A for <v>compile-flg</v>, <v>skip-proofs-okp</v>, <v>acl2x</v>, <v>write-port</v>, and \u000A <v>pcert</v> can be affected by environment variables.  All of these arguments \u000A are described in detail below, except for <v>:pcert</v>.  (We assume below that \u000A the value of <v>:pcert</v> is <v>nil</v> (and environment variable \u000A <v>ACL2_PCERT_ARG</v> is unset or the empty string).  For a discussion of this \u000A argument, see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.)</p> \u000A \u000A <p>Certification occurs in some logical <see topic=\"ACL2____WORLD\">world</see>, called the \u000A ``certification <see topic=\"ACL2____WORLD\">world</see>.''  That <see topic=\"ACL2____WORLD\">world</see> must contain the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s needed to read and execute the forms in the book.  The <see topic=\"ACL2____COMMAND\">command</see>s necessary to recreate that <see topic=\"ACL2____WORLD\">world</see> from the ACL2 initial <see topic=\"ACL2____WORLD\">world</see> are called the ``<see topic=\"ACL2____PORTCULLIS\">portcullis</see> commands,'' and will be copied into \u000A the <see topic=\"ACL2____CERTIFICATE\">certificate</see> created for the book.  Those <see topic=\"ACL2____COMMAND\">command</see>s will be \u000A re-executed whenever the book is included, to ensure that the appropriate \u000A packages (and all other names used in the certification <see topic=\"ACL2____WORLD\">world</see>) are \u000A correctly defined.  Note that Step 1 of <v>certify-book</v> will fail if a \u000A package mentioned in the book is not defined before attempting the \u000A certification, i.e., defined by a portcullis command in the certification \u000A world.  For example, suppose that your book contains the symbol <v>FOO::X</v>, \u000A but the package \"FOO\" is not currently defined.  Then an error message will \u000A likely complain either about a missing package <v>&quot;FOO&quot;</v>, or about a symbol \u000A <v>FOO::X</v> that is ``not in any of the packages known to ACL2.''  A solution \u000A is to define the package \"FOO\", perhaps directly using <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> or by \u000A including a book that defines this package, before attempting the \u000A certification.</p> \u000A \u000A <p>The certified book will be more often usable if the certification <see topic=\"ACL2____WORLD\">world</see> is kept to a minimal extension of the ACL2 initial <see topic=\"ACL2____WORLD\">world</see> (for \u000A example, to prevent name clashes with functions defined in other books). \u000A Thus, before you call <v>certify-book</v> for the first time on a book, you may \u000A wish to get into the initial ACL2 <see topic=\"ACL2____WORLD\">world</see> (e.g., with <v>:ubt 1</v> or just \u000A starting a new version of ACL2), <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> the desired packages, and then \u000A invoke <v>certify-book</v>.</p> \u000A \u000A <p>The <v>k</v> argument to <v>certify-book</v> must be either a nonnegative \u000A integer or else one of the symbols <v>t</v> or <v>?</v> in any package.  If <v>k</v> \u000A is an integer, then it must be the number of <see topic=\"ACL2____COMMAND\">command</see>s that have been \u000A executed after the initial ACL2 <see topic=\"ACL2____WORLD\">world</see> to create the <see topic=\"ACL2____WORLD\">world</see> in \u000A which <v>certify-book</v> was called.  One way to obtain this number is by doing \u000A <v>:pbt :start</v> to see all the <see topic=\"ACL2____COMMAND\">command</see>s back to the first one.</p> \u000A \u000A <p>If <v>k</v> is <v>t</v> (or any symbol whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is \u000A <v>&quot;T&quot;</v>), it means that <v>certify-book</v> should use the same <see topic=\"ACL2____WORLD\">world</see> \u000A used in the last certification of this book.  <v>K</v> may have such a value \u000A only if you call <v>certify-book</v> in the initial ACL2 <see topic=\"ACL2____WORLD\">world</see> and there \u000A is a <see topic=\"ACL2____CERTIFICATE\">certificate</see> on file for the book being certified.  (Of course, the \u000A <see topic=\"ACL2____CERTIFICATE\">certificate</see> is probably invalid.)  In this case, <v>certify-book</v> \u000A reads the old <see topic=\"ACL2____CERTIFICATE\">certificate</see> to obtain the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s and executes them to recreate the certification <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>Finally, <v>k</v> may be <v>?</v> (or any symbol whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is \u000A <v>&quot;?&quot;</v>), in which case there is no check made on the certification world. \u000A That is, if <v>k</v> is such a value then no action related to the preceding two \u000A paragraphs is performed, which can be a nice convenience but at the cost of \u000A eliminating a potentially valuable check that the certification <see topic=\"ACL2____WORLD\">world</see> \u000A may be as expected.</p> \u000A \u000A <p>We next describe the meaning of <v>compile-flg</v> and how it defaults.  If \u000A explicit compilation has been suppressed by <v>(<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> nil\u000A state)</v>, then <v>compile-flg</v> is coerced to <v>nil</v>; see <see topic=\"ACL2____COMPILATION\">compilation</see>.  Otherwise <v>compile-flg</v> may be given the value of <v>t</v> (or \u000A <v>:all</v>, which is equivalent to <v>t</v> except during provisional \u000A certification; see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>), indicating that the book \u000A is to be compiled, or else <v>nil</v>.  (Note that compilation initially creates \u000A a compiled file with a temporary file name, and then moves that temporary file \u000A to the final compiled file name obtained by adding a suitable extension to the \u000A book name.  Thus, a compiled file will appear atomically in its intended \u000A location.)  Finally, suppose that <v>compile-flg</v> is not supplied (or is \u000A <v>:default</v>).  If environment variable <v>ACL2_COMPILE_FLG</v> is defined and \u000A not the empty string, then its value should be <v>T</v>, <v>NIL</v>, or <v>ALL</v> \u000A after converting to upper case, in which case <v>compile-flg</v> is considered \u000A to have value <v>t</v>, <v>nil</v>, or <v>:all</v> (respectively).  Otherwise \u000A <v>compile-flg</v> defaults to <v>t</v>.  Note that the value <v>:all</v> is \u000A equivalent to <v>t</v> except for during the Convert procedure of provisional \u000A certification; see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.</p> \u000A \u000A <p>Two keyword arguments, <v>:defaxioms-okp</v> and <v>:skip-proofs-okp</v>, \u000A determine how the system handles the inclusion of <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> events and \u000A <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> events, respectively, in the book.  The value <v>t</v> \u000A allows such events, but prints a warning message.  The value <v>nil</v> causes \u000A an error if such an event is found.  <v>Nil</v> is the default unless keyword \u000A argument <v>:acl2x t</v> is provided and state global <v>'write-acl2x</v> is a \u000A cons (see <see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-ACL2x</see>), in which case the default is <v>t</v>.</p> \u000A \u000A <p>The keyword argument <v>:ttags</v> may normally be omitted.  A few \u000A constructs, used for example if you are building your own system based on \u000A ACL2, may require it.  See <see topic=\"ACL2____DEFTTAG\">defttag</see> for an explanation of this \u000A argument.</p> \u000A \u000A <p>When book <v>B</v> is certified with value <v>t</v> (the default) for keyword \u000A argument <v>:write-port</v>, a file <v>B.port</v> is written by certification \u000A process.  This file contains all of the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s for \u000A <v>B</v>, i.e., all user commands present in the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> at \u000A the time <v>certify-book</v> is called.  if <v>B.lisp</v> later becomes \u000A uncertified, say because <see topic=\"ACL2____EVENTS\">events</see> from that file or an included book have \u000A been edited, then <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;B&quot;)</v> will consult <v>B.port</v> to \u000A evaluate forms in that file before evaluating the events in <v>B.lisp</v>.  On \u000A the other hand, <v>B.port</v> is ignored when including <v>B</v> if <v>B</v> is \u000A certified.</p> \u000A \u000A <p>If you use <see topic=\"ACL2____GUARD\">guard</see>s, please note <v>certify-book</v> is executed as \u000A though <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> nil)</v> has been evaluated; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  If you want guards checked, consider using <v>ld</v> \u000A instead, or in addition; see <see topic=\"ACL2____LD\">ld</see>.</p> \u000A \u000A <p>For a general discussion of books, see <see topic=\"ACL2____BOOKS\">books</see>.  <v>Certify-book</v> is \u000A akin to what we have historically called a ``proveall'': all the forms in the \u000A book are ``proved'' to guarantee their admissibility.  More precisely, \u000A <v>certify-book</v> (1) reads the forms in the book, confirming that the \u000A appropriate packages are defined in the certification <see topic=\"ACL2____WORLD\">world</see>; (2) does \u000A the full admissibility checks on each form (proving termination of recursive \u000A functions, proving theorems, etc.), checking as it goes that each form is an \u000A embedded event form (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>); (3) may roll back the \u000A <see topic=\"ACL2____WORLD\">world</see> (how far? ~-[] see below) and perform an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to \u000A check for <tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibilities (see <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see>); (4) writes a <see topic=\"ACL2____CERTIFICATE\">certificate</see> recording not only \u000A that the book was certified but also recording the <see topic=\"ACL2____COMMAND\">command</see>s necessary \u000A to recreate the certification <see topic=\"ACL2____WORLD\">world</see> (so the appropriate packages can be \u000A defined when the book is included in other <see topic=\"ACL2____WORLD\">world</see>s) and the check sums \u000A of all the <see topic=\"ACL2____BOOKS\">books</see> involved (see <see topic=\"ACL2____CERTIFICATE\">certificate</see>); (5) compiles the \u000A book if so directed (and then loads the object file in that case).  The result \u000A of executing a <v>certify-book</v> <see topic=\"ACL2____COMMAND\">command</see> is the creation of a single \u000A new event, which is actually an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  If you don't want \u000A its included <see topic=\"ACL2____EVENTS\">events</see> in your present <see topic=\"ACL2____WORLD\">world</see>, simply execute \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> <v>:here</v> afterwards.</p> \u000A \u000A <p>Technical Remark.  Step 3 above mentions rolling the logical <see topic=\"ACL2____WORLD\">world</see> \u000A back to check for local incompatibilies.  For efficiency, this retraction to \u000A an initial segment of the the world is skipped if a local event is not \u000A encountered as described below; otherwise, the world is rolled back to the \u000A point just before the first such event was admitted.  For this purpose, a \u000A relevant local event is one at the top level of the book or under a sequences \u000A of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> calls in the book.  Here, we include forms generated by \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  But this category does not include local <see topic=\"ACL2____EVENTS\">events</see> \u000A within an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, since those are ignored in the final \u000A processing of the <v>encapsulate</v> event.  We also do not consider local \u000A events within an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  End of Technical Remark.</p> \u000A \u000A <p>A utility is provided to assist in debugging failures of <v>certify-book</v>; \u000A see <see topic=\"ACL2____REDO-FLAT\">redo-flat</see>.)</p> \u000A \u000A <p><v>Certify-book</v> requires that the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) be <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> when certification is attempted. \u000A If the mode is not <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, an error is signalled.</p> \u000A \u000A <p>An error will occur if <v>certify-book</v> has to deal with any uncertified \u000A book other than the one on which it was called.  For example, if the book \u000A being certified includes another book, that subbook must already have been \u000A certified.</p> \u000A \u000A <p>If you have a certified book that has remained unchanged for some time you \u000A might well not remember the appropriate <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s for it, though they \u000A are stored in the <see topic=\"ACL2____CERTIFICATE\">certificate</see> file and (by default) also in the \u000A <v>.port</v> file.  If you begin to change the book, don't throw away its <see topic=\"ACL2____CERTIFICATE\">certificate</see> file just because it has become invalid!  It is an important \u000A historical document until the book is re-certified.  More important, don't \u000A throw away the <v>.port</v> file, as it will provide the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> \u000A commands when including the book as an uncertified book; see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.</p> \u000A \u000A <p>When <v>certify-book</v> is directed to produce a compiled file, it calls the \u000A Common Lisp function <v>compile-file</v> on the original source file.  This \u000A creates a compiled file with an extension known to ACL2, e.g., if the book is \u000A named <v>&quot;my-book&quot;</v> then the source file is <v>&quot;my-book.lisp&quot;</v> and the \u000A compiled file under GCL will be <v>&quot;my-book.o&quot;</v> while under SBCL it will be \u000A <v>&quot;my-book.fasl&quot;</v>.  The compiled file is then loaded.  When <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is used later on <v>&quot;my-book&quot;</v> it will automatically load the \u000A compiled file, provided the compiled file has a later write date than the \u000A source file.  The only effect of such <see topic=\"ACL2____COMPILATION\">compilation</see> and loading is that \u000A the functions defined in the book execute faster.  See <see topic=\"ACL2____GUARD\">guard</see> for a \u000A discussion of the issues, and if you want more details about <see topic=\"ACL2____BOOKS\">books</see> and \u000A compilation, see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.</p> \u000A \u000A <p>When <v>certify-book</v> is directed not to produce a compiled file, it will \u000A delete any existing compiled file for the book, so as not to mislead <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> into loading the now outdated compiled file.  Otherwise, \u000A <v>certify-book</v> will create a temporary ``expansion file'' to compile, \u000A obtained by appending the string \"@expansion.lsp\" to the end of the book \u000A name.  Remark: Users may ignore that file, which is automatically deleted \u000A unless <see topic=\"ACL2____STATE\">state</see> global variable <v>'save-expansion-file</v> has been set, \u000A presumably by a system developer, to a non-<v>nil</v> value; see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see> for more information about hit issue, including the role \u000A of environment variable <v>ACL2_SAVE_EXPANSION</v>.</p> \u000A \u000A <p>After execution of a <v>certify-book</v> form, the value of <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> is restored to what it was immediately before that \u000A <v>certify-book</v> form was executed.  See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>Those who use the relatively advanced features of trust tags (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) and <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> may wish to know how to create a <see topic=\"ACL2____CERTIFICATE\">certificate</see> file that avoids dependence on trust tags that are used only \u000A during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion.  For this, including documentation of the \u000A <v>:acl2x</v> and <v>:ttagsx</v> keyword arguments for <v>certify-book</v>, see \u000A <see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-ACL2x</see>.</p> \u000A \u000A <p>This completes the tour through the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____DEFUN-MODE":[["Defun"],"ACL2 Sources","ACL2","<p>Two ``<see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s'' are supported, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  Roughly speaking, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode allows you \u000A to prototype a function for execution without any proof burdens, while \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode allows you to add a new definitional axiom to the \u000A logic.  The system comes up in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  Execution of \u000A functions whose <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> may render ACL2 \u000A unsound!  See <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>.</p> \u000A \u000A <p>Note that calls of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> and of many <see topic=\"ACL2____EVENTS\">events</see> are skipped in \u000A <v>:program</v> mode; see <see topic=\"ACL2____PROGRAM\">program</see>.</p> \u000A \u000A <p>When you define a function in the ACL2 logic, that function can be run on \u000A concrete data.  But it is also possible to reason deductively about the \u000A function because each definition extends the underlying logic with a \u000A definitional axiom.  To ensure that the logic is sound after the addition of \u000A this axiom, certain restrictions have to be met, namely that the recursion \u000A terminates.  This can be quite challenging.</p> \u000A \u000A <p>Because ACL2 is a <see topic=\"ACL2____PROGRAMMING\">programming</see> language, you often may wish simply to \u000A program in ACL2.  For example, you may wish to define your system and test it, \u000A without any logical burden.  Or, you may wish to define ``utility'' functions \u000A &mdash; functions that are executed to help manage the task of building your \u000A system but functions whose logical properties are of no immediate concern. \u000A Such functions might be used to generate test data or help interpret the \u000A results of tests.  They might create files or explore the ACL2 database.  The \u000A termination arguments for such functions are an unnecessary burden provided no \u000A axioms about the functions are ever used in deductions.</p> \u000A \u000A <p>Thus, ACL2 introduces the idea of the ``<see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>'' of a function. \u000A The <v>:mode</v> keyword of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'s <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> <v>xarg</v> allows you \u000A to specify the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of a given definition.  If no <v>:mode</v> \u000A keyword is supplied, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is used; see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>.</p> \u000A \u000A <p>There are two <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s, each of which is written as a \u000A keyword:</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> &mdash; logically undefined but executable outside \u000A deductive contexts.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> &mdash; axiomatically defined as per the ACL2 \u000A definitional principle.</p> \u000A \u000A <p>It is possible to change the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of a function from \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  We discuss this below.</p> \u000A \u000A <p>We think of functions having <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode as ``dangerous'' \u000A functions, while functions having <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode are ``safe.''  The \u000A only requirement enforced on <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions is the \u000A syntactic one: each definition must be well-formed ACL2.  Naively speaking, if \u000A a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function fails to terminate then no harm is done \u000A because no axiom is added (so inconsistency is avoided) and some invocations \u000A of the function may simply never return.  This simplistic justification of \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode execution is faulty because it ignores the damage \u000A that might be caused by ``mis-guarded'' functions.  See <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>.</p> \u000A \u000A <p>We therefore implicitly describe an imagined implementation of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s that is safe and, we think, effective.  But please see <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>.</p> \u000A \u000A <p>The default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  This means that when \u000A you <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> a function the system will try to prove termination.  If you \u000A wish to introduce a function of a different <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> use the \u000A <v>:mode</v> <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword.  Below we show <v>fact</v> introduced as a \u000A function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)) (<see topic=\"COMMON-LISP_____D3\">=</see> n 0))\u000A      1\u000A    (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>No axiom is added to the logic as a result of this definition.  By \u000A introducing <v>fact</v> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode we avoid the burden of a \u000A termination proof, while still having the option of executing the function. \u000A For example, you can type</p> \u000A \u000A <code>ACL2 !&gt;(fact 3)</code> \u000A \u000A <p>and get the answer <v>6</v>.  If you type <v>(fact -1)</v> you will get a hard \u000A lisp error due to ``infinite recursion.''</p> \u000A \u000A <p>However, the ACL2 theorem prover knows no axioms about <v>fact</v>.  In \u000A particular, if the term <v>(fact 3)</v> arises in a proof, the theorem prover is \u000A unable to deduce that it is <v>6</v>.  From the perspective of the theorem \u000A prover it is as though <v>fact</v> were an undefined function symbol of arity \u000A <v>1</v>.  Thus, modulo certain important issues (see <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>), \u000A the introduction of this function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode does not \u000A imperil the soundness of the system &mdash; despite the fact that the \u000A termination argument for <v>fact</v> was omitted &mdash; because nothing of \u000A interest can be proved about <v>fact</v>.  Indeed, we do not allow <v>fact</v> to \u000A be used in logical contexts such as conjectures submitted for proof.</p> \u000A \u000A <p>It is possible to convert a function from <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode to \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode at the cost of proving that it is admissible.  This \u000A can be done by invoking</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> fact)</code> \u000A \u000A <p>which is equivalent to submitting the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>fact</v>, again, \u000A but in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)) (<see topic=\"COMMON-LISP_____D3\">=</see> n 0))\u000A      1\u000A    (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>This particular event will fail because the termination argument requires \u000A that <v>n</v> be nonnegative.  A repaired <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, for example with <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt> replaced by <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>, will succeed, and an axiom about <v>fact</v> will \u000A henceforth be available.</p> \u000A \u000A <p>Technically, <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> submits a redefinition of the \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function.  This is permitted, even when <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> is <v>nil</v>, because the new definition is identical \u000A to the old (except for its <v>:mode</v> and, possibly, other non-logical \u000A properties).</p> \u000A \u000A <p>See <see topic=\"ACL2____GUARD\">guard</see> for a discussion of how to restrict the execution of \u000A functions.  <see topic=\"ACL2____GUARD\">Guard</see>s may be ``verified'' for functions in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode; see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.</p>"],

 "ACL2____LINEAR":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> length-member-leq-length       If inequality reasoning begins to\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> e)           consider how (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a b))\u000A                (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))        compares to any other term, add to\u000A           (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))   the set of known inequalities the fact\u000A               (<see topic=\"COMMON-LISP____LENGTH\">length</see> x)))            that it is no larger than (<see topic=\"COMMON-LISP____LENGTH\">length</see> b),\u000A  :rule-classes :linear)               provided (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> a) and\u000A                                       (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b) rewrite to t.\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...hi...)\u000A              (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...hk...)\u000A                       (<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A                            (rel lhs rhs)\u000A                            ...)))\u000A     ...)</code> \u000A \u000A <p>Note: One <v>:linear</v> rule class object might create many linear \u000A arithmetic rules from the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula.  To create the \u000A rules, we first flatten the <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> and <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> structure of the \u000A formula, transforming it into a conjunction of formulas, each of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> h1 ... hn) (rel lhs rhs))</code> \u000A \u000A <p>where no hypothesis is a conjunction and <v>rel</v> is one of the inequality \u000A relations <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>, <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>, <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>, <tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt>, or <tt><see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see></tt>.  If \u000A necessary, the hypothesis of such a conjunct may be vacuous.  We create a \u000A <v>:linear</v> rule for each such conjunct, if possible, and otherwise cause an \u000A error.</p> \u000A \u000A <p>Each rule has one or more ``trigger terms'' which may be specified by the \u000A user using the <v>:trigger-terms</v> field of the rule class or which may be \u000A defaulted to values chosen by the system.  We discuss the determination of \u000A trigger terms after discussing how linear rules are used.</p> \u000A \u000A <p><v>:Linear</v> rules are used by an arithmetic decision procedure during \u000A rewriting.  See <see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see> and see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>. \u000A Here we assume that the reader is familiar with the material described in \u000A <tt><see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see></tt>.</p> \u000A \u000A <p>Recall that we eliminate the unknowns of an inequality in term-order, \u000A largest unknowns first.  (See <see topic=\"ACL2____TERM-ORDER\">term-order</see>.)  In order to facilitate this \u000A strategy, we store the inequalities in ``linear pots''.  For purposes of the \u000A present discussion, let us say that an inequality is ``about'' its largest \u000A unknown.  Then, all of the inequalities about a particular unknown are stored \u000A in the same linear pot, and the pot is said to be ``labeled'' with that \u000A unknown.  This storage layout groups all of the inequalities which are \u000A potential candidates for cancellation with each other into one place.  It is \u000A also key to the efficient operation of <v>:linear</v> rules.</p> \u000A \u000A <p>If the arithmetic decision procedure has stabilized and not yielded a \u000A contradiction, we scan through the list of linear pots examining each label as \u000A we go.  If the trigger term of some <v>:linear</v> rule can be instantiated to \u000A match the label, we so instantiate that rule and attempt to relieve the \u000A hypotheses with general-purpose rewriting.  If we are successful, we add the \u000A rule's instantiated conclusion to our set of inequalities.  This may let \u000A cancellation continue.</p> \u000A \u000A <p>Note: Problems may arise if you explicitly store a linear lemma under a \u000A trigger term that, when instantiated, is not the largest unknown in the \u000A instantiated concluding inequality.  Suppose for example you store the linear \u000A rule <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (fn i j) (<see topic=\"COMMON-LISP_____F2\">/</see> i (<see topic=\"COMMON-LISP_____A2\">*</see> j j)))</v> under the trigger term <v>(fn i j)</v>. \u000A Then when the system ``needs'' an inequality about <v>(fn a b)</v>, (i.e., \u000A because <v>(fn a b)</v> is the label of some linear pot, and hence the largest \u000A unknown in some inequality), it will appeal to the rule and deduce <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (fn\u000A a b) (<see topic=\"COMMON-LISP_____F2\">/</see> a (<see topic=\"COMMON-LISP_____A2\">*</see> b b)))</v>.  However, the largest unknown in this inequality is \u000A <v>(<see topic=\"COMMON-LISP_____F2\">/</see> a (<see topic=\"COMMON-LISP_____A2\">*</see> b b))</v> and hence it will be stored in a linear pot labeled with \u000A <v>(<see topic=\"COMMON-LISP_____F2\">/</see> a (<see topic=\"COMMON-LISP_____A2\">*</see> b b))</v>.  The original, triggering inequality which is in a pot \u000A about <v>(fn a b)</v> will therefore not be cancelled against the new one.  It \u000A is generally best to specify as a trigger term one of the ``maximal'' terms of \u000A the polynomial, as described below.</p> \u000A \u000A <p>We now describe how the trigger terms are determined.  Most of the time, \u000A the trigger terms are not specified by the user and are instead selected by \u000A the system.  However, the user may specify the terms by including an explicit \u000A <v>:trigger-terms</v> field in the rule class, e.g.,</p> \u000A \u000A <code>General Form of a Linear Rule Class:\u000A(:LINEAR :COROLLARY formula\u000A         :TRIGGER-TERMS (term1 ... termk))</code> \u000A \u000A <p>Each <v>termi</v> must be a term and must not be a variable, quoted constant, \u000A lambda application, <v>let-expression</v> or <v>if-expression</v>.  In addition, \u000A each <v>termi</v> must be such that if all the variables in the term are \u000A instantiated and then the hypotheses of the corollary formula are relieved \u000A (possibly instantiating additional free variables), then all the variables in \u000A the concluding inequality are instantiated.  We generate a linear rule for \u000A each conjuctive branch through the corollary and store each rule under each of \u000A the specified triggers.  Thus, if the corollary formula contains several \u000A conjuncts, the variable restrictions on the <v>termi</v> must hold for each \u000A conjunct.</p> \u000A \u000A <p>If <v>:trigger-terms</v> is omitted the system computes a set of trigger \u000A terms.  Each conjunct of the corollary formula may be given a unique set of \u000A triggers depending on the variables that occur in the conjunct and the addends \u000A that occur in the concluding inequality.  In particular, the trigger terms for \u000A a conjunct is the list of all ``maximal addends'' in the concluding \u000A inequality.</p> \u000A \u000A <p>The ``addends'' of <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y)</v> and <v>(<see topic=\"COMMON-LISP____-\">-</see> x y)</v> are the union of the \u000A addends of <v>x</v> and <v>y</v>.  The addends of <v>(<see topic=\"COMMON-LISP____-\">-</see> x)</v> and <v>(<see topic=\"COMMON-LISP_____A2\">*</see> n x)</v>, \u000A where <v>n</v> is a rational constant, is just <v>{x}</v>.  The addends of an \u000A inequality are the union of the addends of the left- and right-hand sides. \u000A The addends of any other term, <v>x</v>, is <v>{x}</v>.</p> \u000A \u000A <p>A term is maximal for a conjunct <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps concl)</v> of the corollary \u000A if (a) the term is a non-variable, non-quote, non-lambda application, \u000A non-<tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and non-<tt><see topic=\"COMMON-LISP____IF\">if</see></tt> expression, (b) the term contains enough \u000A variables so that when they are instantiated and the hypotheses are relieved \u000A (which may bind some free variables; see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) then all the \u000A variables in <v>concl</v> are instantiated, and (c) no other addend is always \u000A ``bigger'' than the term, in the technical sense described below.</p> \u000A \u000A <p>The technical notion referenced above depends on the notion of \u000A <i>fn-count</i>, the number of function symbols in a term, and \u000A <i>pseudo-fn-count</i>, which is essentially the number of function symbols \u000A implicit in a constant (see <see topic=\"ACL2____TERM-ORDER\">term-order</see>, specifically the discussion of \u000A ``pseudo-function application count'' at the end).  We say <v>term1</v> is \u000A always bigger than <v>term2</v> if all instances of <v>term1</v> have a larger \u000A fn-count (actually lexicographic order of fn-count and pseudo-fn-count) than \u000A the corresponding instances of <v>term2</v>.  This is equivalent to saying that \u000A the fn-count of <v>term1</v> is larger than that of <v>term2</v> (by ``fn-count'' \u000A here we mean the lexicographic order of fn-count and pseudo-fn-count) and the \u000A variable bag for <v>term2</v> is a subbag of that for <v>term1</v>.  For example, \u000A <v>(<see topic=\"COMMON-LISP_____F2\">/</see> a (<see topic=\"COMMON-LISP_____A2\">*</see> b b))</v> is always bigger than <v>(fn a b)</v> because the first has \u000A two function applications and <v>{a b}</v> is a subbag of <v>{a b b}</v>, but \u000A <v>(<see topic=\"COMMON-LISP_____F2\">/</see> a (<see topic=\"COMMON-LISP_____A2\">*</see> b b))</v> is not always bigger than <v>(fn a x)</v>.</p>"],

 "ACL2____DEFTTAG":[["Interfacing-tools","Events"],"ACL2 Sources","ACL2","<p><b>Introduction</b>.  This event is intended for advanced users \u000A who, in essence, want to build extensions of ACL2.  The typical intended use \u000A is to create <see topic=\"ACL2____BOOKS\">books</see> that extend the functionality of ACL2 in ways not \u000A allowed without a so-called ``active trust tag''.  A trust tag thus represents \u000A a contract: The writer of such a book is guaranteeing that the book extends \u000A ACL2 in a ``correct'' way as defined by the writer of the book.  The writer of \u000A the book will often have a small section of the book in the scope of an active \u000A trust tag that can be inspected by potential users of that book:</p> \u000A \u000A <code>&lt;initial part of book, which does not use trust tags&gt;\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :some-ttag) ; install :some-ttag as an active trust tag\u000A&lt;various code that requires an active trust tag&gt;\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil)        ; remove active trust tag\u000A&lt;initial part of book, which does not use trust tags&gt;</code> \u000A \u000A <p>Why might trust tags be needed?  The evaluation of certain functions can \u000A introduce bugs and even unsoundness, but can be useful in restricted ways that \u000A avoid such issues.  For example, <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> can be used in an unsafe \u000A way, for example to overwrite files, or worse; see <see topic=\"ACL2____SYS-CALL\">sys-call</see> for a \u000A frightening example from Bob Boyer.  The following example shows that the \u000A function <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> is restricted by default, but can be called after \u000A installing an active trust tag.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;pwd&quot; nil)\u000A\u000AACL2 Error in TOP-LEVEL:  The SYS-CALL function cannot be called unless\u000Aa trust tag is in effect.  See :DOC defttag.\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; Install :T as an active trust tag.\u000A\u000ATTAG NOTE: Adding ttag :T from the top level loop.\u000A T\u000AACL2 !&gt;(<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;pwd&quot; nil) ; print the current directory and return NIL\u000A/u/kaufmann\u000ANIL\u000AACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil) ; Remove the active trust tag (using value NIL).\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;pwd&quot; nil) ; Now we get the error again:\u000A\u000AACL2 Error in TOP-LEVEL:  The SYS-CALL function cannot be called unless\u000Aa trust tag is in effect.  See :DOC defttag.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Of course, using <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> with the Linux command <v>pwd</v> is not \u000A likely to cause any soundness problems!  So suppose we want to create a \u000A function that prints the working directory.  We might put the following <see topic=\"ACL2____EVENTS\">events</see> into a book that is to be certified.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :pwd-ttag)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> print-working-dir ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;pwd&quot; nil))\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil) ; optional (<see topic=\"ACL2____BOOKS\">books</see> end with this implicitly)</code> \u000A \u000A <p>We can certify this book with a specification that <v>:pwd-ttag</v> is a \u000A legal trust tag:</p> \u000A \u000A <code>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;pwd&quot; 0 t :ttags (:pwd-ttag))</code> \u000A \u000A <p>One can now use this book by executing <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with keyword \u000A parameter <v>:ttags (:pwd-ttag)</v> and then calling function \u000A <v>print-working-dir</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;pwd&quot; :ttags (:pwd-ttag))\u000A(print-working-dir) ; working directory is printed to terminal</code> \u000A \u000A <p><b>Detailed documentation.</b></p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> tag-name)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> tag-name :doc doc-string)</code> \u000A \u000A <p>where <v>tag-name</v> is a symbol.  The <v>:doc doc-string</v> argument is \u000A optional; if supplied, then it must be a valid <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string \u000A (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>), and the <v>defttag</v> call will generate a \u000A corresponding <tt><see topic=\"ACL2____DEFDOC\">defdoc</see></tt> event for <v>tag-name</v>.  (For the rest of this \u000A discussion we ignore the <v>:doc</v> argument.)</p> \u000A \u000A <p>Note however that (other than the <v>:doc</v> argument), if <v>tag-name</v> is \u000A not <v>nil</v> then it is converted to a ``corresponding <see topic=\"COMMON-LISP____KEYWORD\">keyword</see>'': a \u000A symbol in the <v>&quot;KEYWORD&quot;</v> package with the same <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> as \u000A <v>tag-name</v>.  Thus, for example, <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> foo)</v> is equivalent to \u000A <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)</v>.  Moreover, a non-<v>nil</v> symbol with a <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of <v>&quot;NIL&quot;</v> is illegal for trust tags; thus, for example, \u000A <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :nil)</v> is illegal.</p> \u000A \u000A <p>This event introduces or removes a so-called active trust tag (or ``ttag'', \u000A pronounced ``tee tag'').  An active ttag is a <see topic=\"COMMON-LISP____KEYWORD\">keyword</see> symbol that is \u000A associated with potentially unsafe evaluation.  For example, calls of <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> are illegal unless there is an active trust tag.  An active trust \u000A tag can be installed using a <v>defttag</v> event.  If one introduces an active \u000A ttag and then writes definitions that calls <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>, presumably in a \u000A defensibly ``safe'' way, then responsibility for those calls is attributed to \u000A that ttag.  This attribution (or blame!) is at the level of <see topic=\"ACL2____BOOKS\">books</see>; a \u000A book's <see topic=\"ACL2____CERTIFICATE\">certificate</see> contains a list of ttags that are active in that \u000A book, or in a book that is included (possibly <see topic=\"ACL2____LOCAL\">local</see>ly), or in a book \u000A included in a book that is included (either inclusion being potentially <see topic=\"ACL2____LOCAL\">local</see>), and so on.  We explain all this in more detail below.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____DEFTTAG\">Defttag</see> :tag-name)</v> is essentially equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :ttag :tag-name)</code> \u000A \u000A <p>and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  We say more \u000A about the scope of <v>defttag</v> forms below.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless executes the above <tt><see topic=\"ACL2____TABLE\">table</see></tt> event and hence changes the ACL2 \u000A logical <see topic=\"ACL2____WORLD\">world</see>, and is so recorded.  Although no event summary is \u000A printed, it is important to note that the ``TTAG NOTE'', discussed below, is \u000A always printed for a non-nil <v>:tag-name</v> (unless deferred; see <see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see>).</p> \u000A \u000A <p><b>Active ttags.</b> Suppose <v>tag-name</v> is a non-<v>nil</v> symbol.  Then \u000A <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :tag-name)</v> sets <v>:tag-name</v> to be the (unique) ``active \u000A ttag.''  There must be an active ttag in order for there to be any mention of \u000A certain function and macro symbols, including <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>; evaluate the \u000A form <v>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> *ttag-fns-and-macros*)</v> to see the full list of such \u000A symbols.  On the other hand, <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil)</v> removes the active ttag, if \u000A any; there is then no active ttag.  The scope of a <v>defttag</v> form in a book \u000A being certified or included is limited to subsequent forms in the same book \u000A before the next <v>defttag</v> (if any) in that book.  Similarly, if a \u000A <v>defttag</v> form is evaluated in the top-level loop, then its effect is \u000A limited to subsequent forms in the top-level loop before the next <v>defttag</v> \u000A in the top-level loop (if any).  Moreoever, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is illegal \u000A when a ttag is active; of course, in such a circumstance one can execute \u000A <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil)</v> in order to allow book certification.</p> \u000A \u000A <p><b>Ttag notes and the ``certifier.''</b> When a <v>defttag</v> is executed \u000A with an argument other than <v>nil</v>, output is printed, starting on a fresh \u000A line with: <v>TTAG NOTE</v>.  For example:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)\u000A\u000ATTAG NOTE: Adding ttag :FOO from the top level loop.\u000A :FOO\u000AACL2 !&gt;</code> \u000A \u000A <p>If the <v>defttag</v> occurs in an included book, the message looks like \u000A this.</p> \u000A \u000A <code>TTAG NOTE (for included book): Adding ttag :FOO from file /u/smith/acl2/my-book.lisp.</code> \u000A \u000A <p>The ``<v>TTAG NOTE</v>'' message is always printed on a single line.  The \u000A intention is that one can search the standard output for all such notes in \u000A order to find all <i>defttag</i> events.  In a sense, <i>defttag</i> events \u000A can allow you to define your own system on top of ACL2 (for example, see <see topic=\"ACL2____PROGN_12\">progn!</see>).  So in order for someone else (who we might call the ``certifier'') \u000A to be confident that your collection of <see topic=\"ACL2____BOOKS\">books</see> is meaningful, that \u000A certifier should certify all the user-supplied books from scratch and check \u000A either that no <v>:ttags</v> were supplied to <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, or else look \u000A for every <v>TTAG NOTE</v> in the standard output in order to locate all \u000A <v>defttag</v> <see topic=\"ACL2____EVENTS\">events</see> with non-<v>nil</v> tag name.  In this way, the \u000A certifier can in principle decide whether to be satisfied that those \u000A <v>defttag</v> events did not allow inappropriate forms in the user-supplied \u000A books.</p> \u000A \u000A <p>In order to eliminate much of the output from <v>TTAG NOTE</v>s, see <see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see>.  Note however that the resulting security is \u000A somewhat less; therefore, a <v>TTAG NOTE</v> is printed when invoking \u000A <v>set-deferred-ttag-notes</v> to defer printing of ttag notes.</p> \u000A \u000A <p><b>Allowed ttags when certifying and including books.</b> A <v>defttag</v> \u000A form may not be evaluated unless its argument is a so-called ``allowed'' ttag. \u000A All ttags are allowed in the interactive top-level loop.  However, during \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the set of allowed ttags is \u000A restricted according to the <v>:ttags</v> keyword argument.  If this argument is \u000A omitted then no ttag is allowed, so a <v>defttag</v> call will fail during book \u000A certification or inclusion in this case.  This restriction applies even to \u000A <v>defttag</v> forms already evaluated in the so-called certification <see topic=\"ACL2____WORLD\">world</see> at the time <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is called.  But note that <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see>\u000A nil)</v> is always legal.</p> \u000A \u000A <p>A <v>:ttags</v> argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> can \u000A have value <v>:all</v>, indicating that every ttag is allowed, i.e., no \u000A restriction is being placed on the arguments, just as in the interactive \u000A top-level loop.  In the case of <v>include-book</v>, an omitted <v>:ttags</v> \u000A argument or an argument of <v>:default</v> is treated as <v>:all</v>, except that \u000A warnings will occur when the book's <see topic=\"ACL2____CERTIFICATE\">certificate</see> includes ttags; but for \u000A <v>certify-book</v>, an omitted <v>ttags</v> argument is treated as <v>nil</v>. \u000A Otherwise, if the <v>:ttags</v> argument is supplied but not <v>:all</v>, then its \u000A value is a true list of ttag specifications, each having one of the following \u000A forms, where <v>sym</v> is a non-<v>nil</v> symbol which is treated as the \u000A corresponding <see topic=\"COMMON-LISP____KEYWORD\">keyword</see>.</p> \u000A \u000A <blockquote> \u000A \u000A <p>(1) <v>:sym</v></p> \u000A \u000A <p>(2) <v>(:sym)</v></p> \u000A \u000A <p>(3) <v>(:sym x1 x2 ... xk)</v>, where k &gt; 0 and each <v>xi</v> is a string, \u000A except that one <v>xi</v> may be <v>nil</v>.</p></blockquote> \u000A \u000A <p>In Case (1), <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :sym)</v> is allowed to occur in at most one book or \u000A else in the top-level loop (i.e., the certification world for a book under \u000A certification or a book being included).  Case (2) allows <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :sym)</v> \u000A to occur in an unlimited number of books.  For case (3) the <v>xi</v> specify \u000A where <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :sym)</v> may occur, as follows.  The case that <v>xi</v> is \u000A <v>nil</v> refers to the top-level loop, while all other <v>xi</v> are filenames, \u000A where the <v>&quot;.lisp&quot;</v> extension is optional and relative pathnames are \u000A considered to be relative to the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>). \u000A Note that the restrictions on <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :sym)</v> apply equally to any \u000A equivalent for based on the notion of ``corresponding keyword'' discussed \u000A above, e.g., <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> acl2::sym)</v>.</p> \u000A \u000A <p>An error message, as shown below, illustrates how ACL2 enforcess the notion \u000A of allowed ttags.  Suppose that you call <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> with argument \u000A <v>:ttags (:foo)</v>, where you have already executed <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)</v> in the \u000A certification world (i.e., before calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>).  Then ACL2 \u000A immediately associates the ttag <v>:foo</v> with <v>nil</v>, where again, <v>nil</v> \u000A refers to the top-level loop.  If ACL2 then encounters <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> foo)</v> \u000A inside that book, you will get the following error (using the full book name \u000A for the book, as shown):</p> \u000A \u000A <code>ACL2 Error in ( TABLE ACL2-DEFAULTS-TABLE ...):  The ttag :FOO associated\u000Awith file /u/smith/work/my-book.lisp is not among the set of ttags permitted\u000Ain the current context, specified as follows:\u000A  ((:FOO NIL)).\u000ASee :DOC defttag.</code> \u000A \u000A <p>In general the structure displayed by the error message, which is <v>((:FOO\u000A NIL))</v> in this case, represents the currently allowed ttags with elements as \u000A discussed in (1) through (3) above.  In this case, that list's unique element \u000A is <v>(:FOO NIL)</v>, meaning that ttag <v>:FOO</v> is only allowed at the top \u000A level (as represented by <v>NIL</v>).</p> \u000A \u000A <p><b>Associating ttags with books and with the top-level loop.</b> When a \u000A book is certified, each form <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> tag)</v> that is encountered for \u000A non-<v>nil</v> <v>tag</v> in that book or an included book is recorded in the \u000A generated <see topic=\"ACL2____CERTIFICATE\">certificate</see>, which associates the keyword corresponding to \u000A <v>tag</v> with the <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see> of the book containing that \u000A <v>deftag</v>.  If such a <v>defttag</v> form is encountered outside a book, hence \u000A in the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of the book being certified or one of its included \u000A books, then that keyword is associated with <v>nil</v> in the generated <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  Note that the notion of ``included book'' here applies to the \u000A recursive notion of a book either included directly in the book being \u000A certified or else included in such a book, where we account even for <see topic=\"ACL2____LOCAL\">local</see>ly included books.</p> \u000A \u000A <p>For examples of ways to take advantage of ttags, see <see topic=\"ACL2____HACKER\">hacker</see>, <see topic=\"ACL2____INCLUDE-RAW\">include-raw</see>, <see topic=\"ACL2____QUICKLISP\">quicklisp</see>, and more generally <see topic=\"ACL2____INTERFACING-TOOLS\">interfacing-tools</see>. \u000A See also <see topic=\"ACL2____TTAGS-SEEN\">ttags-seen</see>, <see topic=\"ACL2____PROGN_12\">progn!</see>, <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>, <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>, and <see topic=\"ACL2____SYS-CALL\">sys-call</see>.</p>"],

 "ACL2____DEFPKG":[["Events","Programming"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;MY-PKG&quot;\u000A        (<see topic=\"ACL2____UNION-EQ\">union-eq</see> *acl2-exports*\u000A                  *common-lisp-symbols-from-main-lisp-package*))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;name&quot; term doc-string)</code> \u000A \u000A <p>where <v>&quot;name&quot;</v> is a non-empty string consisting of standard characters \u000A (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>), none of which is lower case, that names the \u000A package to be created; <v>term</v> is a variable-free expression that evaluates \u000A to a list of symbols, where no two distinct symbols in the list may have the \u000A same <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt>, to be imported into the newly created package; and \u000A <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.  The name of the new package must be ``new'': the host lisp must \u000A not contain any package of that name.  There are two exceptions to this \u000A newness rule, discussed at the end of this documentation.</p> \u000A \u000A <p>(There is actually an additional argument, book-path, that is used for \u000A error reporting but has no logical content.  Users should generally ignore \u000A this argument, as well as the rest of this sentence: a book-path will be \u000A specified for <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events added by ACL2 to the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of \u000A a book's <see topic=\"ACL2____CERTIFICATE\">certificate</see>; see <see topic=\"ACL2____HIDDEN-DEATH-PACKAGE\">hidden-death-package</see>.)</p> \u000A \u000A <p><v>Defpkg</v> forms can be entered at the top-level of the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop.  They should not occur in <see topic=\"ACL2____BOOKS\">books</see> (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>).</p> \u000A \u000A <p>After a successful <v>defpkg</v> it is possible to ``intern'' a string into \u000A the package using <tt><see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see></tt>.  The result is a symbol \u000A that is in the indicated package, provided the imports allow it.  For example, \u000A suppose <v>'my-pkg::abc</v> is a symbol whose <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> is \u000A <v>&quot;MY-PKG&quot;</v>.  Suppose further that the imports specified in the \u000A <v>defpkg</v> for <v>&quot;MY-PKG&quot;</v> do not include a symbol whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is <v>&quot;XYZ&quot;</v>.  Then</p> \u000A \u000A <code>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;XYZ&quot; 'my-pkg::abc)</code> \u000A \u000A <p>returns a symbol whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is <v>&quot;XYZ&quot;</v> and whose <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> is <v>&quot;MY-PKG&quot;</v>.  On the other hand, if the imports to \u000A the <v>defpkg</v> does include a symbol with the name <v>&quot;XYZ&quot;</v>, say in the \u000A package <v>&quot;LISP&quot;</v>, then</p> \u000A \u000A <code>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;XYZ&quot; 'my-pkg::abc)</code> \u000A \u000A <p>returns that symbol (which is uniquely determined by the restriction on the \u000A imports list above).  See <see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see>.</p> \u000A \u000A <p>Upon admission of a <v>defpkg</v> event, the function <v>pkg-imports</v> is \u000A extended to compute a list of all symbols imported into the given package, \u000A without duplicates.</p> \u000A \u000A <p><v>Defpkg</v> is the only means by which an ACL2 user can create a new \u000A package or specify what it imports.  That is, ACL2 does not support the Common \u000A Lisp functions <v>make-package</v> or <v>import</v>.  Currently, ACL2 does not \u000A support exporting at all.</p> \u000A \u000A <p>The Common Lisp function <tt><see topic=\"COMMON-LISP____INTERN\">intern</see></tt> is weakly supported by ACL2; see \u000A <see topic=\"COMMON-LISP____INTERN\">intern</see>.  A more general form of that function is also provided: see \u000A <see topic=\"ACL2____INTERN_42\">intern$</see>.</p> \u000A \u000A <p>We now explain the two exceptions to the newness rule for package names. \u000A The careful experimenter will note that if a package is created with a \u000A <v>defpkg</v> that is subsequently undone, the host lisp system will contain the \u000A created package even after the undo.  Because ACL2 hangs onto <see topic=\"ACL2____WORLD\">world</see>s \u000A after they have been undone, e.g., to implement <v>:</v><tt><see topic=\"ACL2____OOPS\">oops</see></tt> but, more \u000A importantly, to implement error recovery, we cannot actually destroy a package \u000A upon undoing it.  Thus, the first exception to the newness rule is that \u000A <v>name</v> is allowed to be the name of an existing package if that package was \u000A created by an undone <v>defpkg</v> and the newly proposed set of imports is \u000A identical to the old one.  See <see topic=\"ACL2____PACKAGE-REINCARNATION-IMPORT-RESTRICTIONS\">package-reincarnation-import-restrictions</see>.  This exception does not violate \u000A the spirit of the newness rule, since one is disinclined to believe in the \u000A existence of undone packages.  The second exception is that <v>name</v> is \u000A allowed to be the name of an existing package if the package was created by a \u000A <v>defpkg</v> with identical set of imports.  That is, it is permissible to \u000A execute ``redundant'' <v>defpkg</v> <see topic=\"ACL2____COMMAND\">command</see>s.  The redundancy test is \u000A based on the values of the two import forms (comparing them after sorting and \u000A removing duplicates), not on the forms themselves.</p> \u000A \u000A <p>Finally, we explain why we require the package name to contain standard \u000A characters, none of which is lower case.  We have seen at least one \u000A implementation that handled lower-case package names incorrectly.  Since we \u000A see no need for lower-case characters in package names, which can lead to \u000A confusion anyhow (note for example that <v>foo::bar</v> is a symbol whose <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> is <v>&quot;FOO&quot;</v>, not <v>&quot;foo&quot;</v>), we simply disallow \u000A them.  Since the notion of ``lower case'' is only well-specified in Common \u000A Lisp for standard characters, we restrict to these.</p> \u000A \u000A <p>NOTE: Also see <see topic=\"ACL2____MANAGING-ACL2-PACKAGES\">managing-ACL2-packages</see> for contributed documentation \u000A on managing ACL2 packages.</p>"],

 "ACL2____IO":[["Interfacing-tools","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A  (channel state)\u000A  (<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> &quot;foo.lisp&quot; :object state)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (eofp obj state)\u000A          (<see topic=\"ACL2____READ-OBJECT\">read-object</see> channel state)\u000A          (.\u000A            .\u000A             (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____CLOSE-INPUT-CHANNEL\">close-input-channel</see> channel state)))\u000A                   (<see topic=\"ACL2____MV\">mv</see> final-ans state))..)))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____STD_F2IO\">std/io</see> and <see topic=\"ACL2____FILE-READING-EXAMPLE\">file-reading-example</see>.</p> \u000A \u000A <p>For advanced ways to control printing, see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>.</p> \u000A \u000A <p>For a discussion of formatted printing, see <see topic=\"ACL2____FMT\">fmt</see>.</p> \u000A \u000A <p>To control ACL2 abbreviation (``evisceration'') of objects before printing \u000A them, see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>, see <see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see>, and see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.</p> \u000A \u000A <p>To redirect output to a file, see <see topic=\"ACL2____OUTPUT-TO-FILE\">output-to-file</see>.</p> \u000A \u000A <p>ACL2 supports input and output facilities equivalent to a subset of those \u000A found in Common Lisp.  ACL2 does not support random access to files or \u000A bidirectional streams.  In Common Lisp, input and output are to or from \u000A objects of type <v>stream</v>.  In ACL2, input and output are to or from objects \u000A called ``channels,'' which are actually symbols.  Although a channel is a \u000A symbol, one may think of it intuitively as corresponding to a Common Lisp \u000A stream.  Channels are in one of two ACL2 packages, <v>&quot;ACL2-INPUT-CHANNEL&quot;</v> \u000A and <v>&quot;ACL2-OUTPUT-CHANNEL&quot;</v>.  When one ``opens'' a file one gets back a \u000A channel whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is the file name passed to ``open,'' \u000A postfixed with <v>-n</v>, where <v>n</v> is a counter that is incremented every \u000A time an open or close occurs.</p> \u000A \u000A <p>There are three channels which are open from the beginning and which cannot \u000A be closed:</p> \u000A \u000A <code>acl2-input-channel::standard-character-input-0\u000Aacl2-input-channel::standard-object-input-0\u000Aacl2-input-channel::standard-character-output-0</code> \u000A \u000A <p>All three of these are really Common Lisp's <v>*standard-input*</v> or \u000A <v>*standard-output*</v>, appropriately.</p> \u000A \u000A <p>For convenience, three global variables are bound to these rather tedious \u000A channel names:</p> \u000A \u000A <code>*standard-ci*\u000A*standard-oi*\u000A*standard-co*</code> \u000A \u000A <p>Common Lisp permits one to open a stream for several different kinds of \u000A <v>io</v>, e.g. character or byte.  ACL2 permits an additional type called \u000A ``object''.  In ACL2 an ``io-type'' is a keyword, either <v>:character</v>, \u000A <v>:byte</v>, or <v>:object</v>.  When one opens a file, one specifies a type, \u000A which determines the kind of io operations that can be done on the channel \u000A returned.  The types <v>:character</v> and <v>:byte</v> are familiar.  Type \u000A <v>:object</v> is an abstraction not found in Common Lisp.  An <v>:object</v> file \u000A is a file of Lisp objects.  One uses <v>read-object</v> to read from \u000A <v>:object</v> files and <v>print-object$</v> (or <v>print-object$-ser</v>) to print \u000A to <v>:object</v> files.  (The reading and printing are really done with the \u000A Common Lisp <v>read</v> and <v>print</v> functions.  For those familiar with \u000A <v>read</v>, we note that the <v>recursive-p</v> argument is <v>nil</v>.)  The \u000A function <v>read-object-suppress</v> is logically the same as <v>read-object</v> \u000A except that <v>read-object-suppress</v> throws away the second returned value, \u000A i.e. the value that would normally be read, simply returning <v>(<see topic=\"ACL2____MV\">mv</see> eof\u000A state)</v>; under the hood, <v>read-object-suppress</v> avoids errors, for example \u000A those caused by encountering symbols in packages unknown to ACL2.</p> \u000A \u000A <p>File-names are strings.  ACL2 does not support the Common Lisp type <tt><see topic=\"COMMON-LISP____PATHNAME\">pathname</see></tt>.  However, for the <v>file-name</v> argument of the output-related \u000A functions listed below, ACL2 supports a special value, <v>:STRING</v>.  For this \u000A value, the channel connects (by way of a Common Lisp output string stream) to \u000A a string rather than to a file: as characters are written to the channel they \u000A can be retrieved by using <v>get-output-stream-string$</v>.</p> \u000A \u000A <p>Here are the names, formals and output descriptions of the ACL2 io \u000A functions.</p> \u000A \u000A <code>Input Functions:\u000A  (<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> (file-name io-type state) (<see topic=\"ACL2____MV\">mv</see> channel state))\u000A  (<see topic=\"ACL2____OPEN-INPUT-CHANNEL-P\">open-input-channel-p</see> (channel io-type state) boolean)\u000A  (<see topic=\"ACL2____CLOSE-INPUT-CHANNEL\">close-input-channel</see> (channel state) state)\u000A  (<see topic=\"ACL2____READ-CHAR_42\">read-char$</see> (channel state) (<see topic=\"ACL2____MV\">mv</see> char/nil state)) ; nil for EOF\u000A  (<see topic=\"ACL2____PEEK-CHAR_42\">peek-char$</see> (channel state) boolean)\u000A  (<see topic=\"ACL2____READ-BYTE_42\">read-byte$</see> (channel state) (<see topic=\"ACL2____MV\">mv</see> byte/nil state)) ; nil for EOF\u000A  (<see topic=\"ACL2____READ-OBJECT\">read-object</see> (channel state) (<see topic=\"ACL2____MV\">mv</see> eof-read-flg obj-read state))\u000A  (read-object-suppress (channel state) (<see topic=\"ACL2____MV\">mv</see> eof-read-flg state))\u000A\u000AOutput Functions:\u000A  (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see>  (file-name io-type state) (<see topic=\"ACL2____MV\">mv</see> channel state))\u000A  (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see> (file-name io-type state) (<see topic=\"ACL2____MV\">mv</see> channel state))\u000A  (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL-P\">open-output-channel-p</see> (channel io-type state) boolean)\u000A  (<see topic=\"ACL2____CLOSE-OUTPUT-CHANNEL\">close-output-channel</see> (channel state) state)\u000A  (<see topic=\"ACL2____PRINC_42\">princ$</see> (obj channel state) state)\u000A  (<see topic=\"ACL2____WRITE-BYTE_42\">write-byte$</see> (byte channel state) state)\u000A  (<see topic=\"ACL2____PRINT-OBJECT_42\">print-object$</see> (obj channel state) state)\u000A  (print-object$-ser (obj serialize-character channel state) state)\u000A  (<see topic=\"ACL2____FMS\">fms</see>  (<see topic=\"COMMON-LISP____STRING\">string</see> alist channel state evisc-tuple) state)\u000A  (<see topic=\"ACL2____FMS_12\">fms!</see> (<see topic=\"COMMON-LISP____STRING\">string</see> alist channel state evisc-tuple) state)\u000A  (<see topic=\"ACL2____FMT\">fmt</see>  (<see topic=\"COMMON-LISP____STRING\">string</see> alist channel state evisc-tuple) (<see topic=\"ACL2____MV\">mv</see> col state))\u000A  (<see topic=\"ACL2____FMT_12\">fmt!</see> (<see topic=\"COMMON-LISP____STRING\">string</see> alist channel state evisc-tuple) (<see topic=\"ACL2____MV\">mv</see> col state))\u000A  (<see topic=\"ACL2____FMT1\">fmt1</see> (<see topic=\"COMMON-LISP____STRING\">string</see> alist col channel state evisc-tuple) (<see topic=\"ACL2____MV\">mv</see> col state))\u000A  (<see topic=\"ACL2____FMT1_12\">fmt1!</see> (<see topic=\"COMMON-LISP____STRING\">string</see> alist col channel state evisc-tuple) (<see topic=\"ACL2____MV\">mv</see> col state))\u000A  (<see topic=\"ACL2____CW\">cw</see> (<see topic=\"COMMON-LISP____STRING\">string</see> arg0 arg1 ... argn) nil)\u000A  (<see topic=\"ACL2____GET-OUTPUT-STREAM-STRING_42\">get-output-stream-string$</see> (channel state\u000A                              &amp;optional (close-p 't)\u000A                                        (ctx ''get-output-stream-string$))\u000A                             (<see topic=\"ACL2____MV\">mv</see> erp string state))</code> \u000A \u000A <p>The ``formatting'' functions are particularly useful; see <see topic=\"ACL2____FMT\">fmt</see> and \u000A see <see topic=\"ACL2____CW\">cw</see>.  In particular, <tt><see topic=\"ACL2____CW\">cw</see></tt> prints to a ``comment window'' and \u000A does not involve the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, so many may find it easier to use \u000A than <tt><see topic=\"ACL2____FMT\">fmt</see></tt> and its variants.  The functions <tt><see topic=\"ACL2____FMS_12\">fms!</see></tt>, <tt><see topic=\"ACL2____FMT_12\">fmt!</see></tt>, \u000A and <tt><see topic=\"ACL2____FMT1_12\">fmt1!</see></tt> are the same as their respective functions without the \u000A ``<v>!</v>,'' except that the ``<v>!</v>'' functions are guaranteed to print forms \u000A that can be read back in (at a slight readability cost).</p> \u000A \u000A <p>When one enters ACL2 with <v>(<see topic=\"ACL2____LP\">lp</see>)</v>, input and output are taken from <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt> to <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>.  Because these are synonyms for \u000A <v>*standard-input*</v> and <v>*standard-output*</v>, one can drive ACL2 io off of \u000A arbitrary Common Lisp streams, bound to <v>*standard-input*</v> and \u000A <v>*standard-output*</v> before entry to ACL2.</p> \u000A \u000A <p>The macro <v>get-output-stream-string$</v> returns the string accumulated \u000A into the given channel.  By default, a call of this macro closes the supplied \u000A output channel.  However, a third argument is optional (default <v>t</v>), and \u000A if it evaluates to <v>nil</v> then the channel remains open.  The fourth \u000A argument is an optional context, which generally evaluates to a symbol, for \u000A error reporting.  The following example illustrates.</p> \u000A \u000A <code>ACL2 !&gt;\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A   (channel state)\u000A   (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see> :string :object state)\u000A   (<see topic=\"ACL2____PPROGN\">pprogn</see> (print-object$-ser 17 nil channel state)\u000A           (print-object$-ser '(a b (c d)) nil channel state)\u000A           (er-let*\u000A             ((str1 (<see topic=\"ACL2____GET-OUTPUT-STREAM-STRING_42\">get-output-stream-string$</see>\u000A                     channel state\u000A                     nil))) ; keep the channel open\u000A             (<see topic=\"ACL2____PPROGN\">pprogn</see> (print-object$-ser 23 nil channel state)\u000A                     (print-object$-ser '((e f)) nil channel state)\u000A                     (er-let* ; close the channel\u000A                       ((str2 (<see topic=\"ACL2____GET-OUTPUT-STREAM-STRING_42\">get-output-stream-string$</see> channel state)))\u000A                       (value (<see topic=\"COMMON-LISP____CONS\">cons</see> str1 str2)))))))\u000A (&quot;\u000A17\u000A(A B (C D))&quot; . &quot;\u000A23\u000A((E F))&quot;)\u000AACL2 !&gt;</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see> for a discussion of formatted printing \u000A functions such as <v>fmt-to-string</v> that do not take a channel or <tt><see topic=\"ACL2____STATE\">state</see></tt> argument and return a string.</p> \u000A \u000A <p>By default, symbols are printed in upper case when vertical bars are not \u000A required, as specified by Common Lisp.  See <see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> for how to \u000A get ACL2 to print symbols in lower case.</p> \u000A \u000A <p>By default, numbers are printed in radix 10 (base 10).  See <see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see> for how to get ACL2 to print numbers in radix 2, 8, or \u000A 16.</p> \u000A \u000A <p>To see the <see topic=\"ACL2____GUARD\">guard</see> of an IO function, or indeed any function, see \u000A <see topic=\"ACL2____ARGS\">args</see> or call the function <v>guard</v>; but some built-in functions \u000A (including some IO functions) will print the result using the variable \u000A <v>STATE-STATE</v>.  While that is logically correct, if you want to execute the \u000A guard then you should replace that variable by <v>STATE</v> and also replace \u000A each built-in function symbol of the form <v>xxx-p1</v> by corresponding \u000A function symbol <v>xxx-p</v>.  Consider the following example.</p> \u000A \u000A <code>ACL2 !&gt;:args princ$\u000A\u000AFunction         PRINC$\u000AFormals:         (X CHANNEL STATE-STATE)\u000ASignature:       (<see topic=\"ACL2____PRINC_42\">PRINC$</see> * * STATE)\u000A                 =&gt; STATE\u000AGuard:           (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X)\u000A                          (<see topic=\"COMMON-LISP____CHARACTERP\">CHARACTERP</see> X)\u000A                          (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A                          (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X))\u000A                      (STATE-P1 STATE-STATE)\u000A                      (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> CHANNEL)\u000A                      (OPEN-OUTPUT-CHANNEL-P1 CHANNEL\u000A                                              :CHARACTER STATE-STATE))\u000AGuards Verified: T\u000ADefun-Mode:      :logic\u000AType:            (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"ACL2____PRINC_42\">PRINC$</see> X CHANNEL STATE-STATE))\u000ADocumentation available via :DOC\u000A PRINC$\u000AACL2 !&gt;(<see topic=\"ACL2____UNTRANSLATE\">untranslate</see> (<see topic=\"ACL2____GUARD\">guard</see> 'princ$ nil (w state)) t (w state))\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X)\u000A         (<see topic=\"COMMON-LISP____CHARACTERP\">CHARACTERP</see> X)\u000A         (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A         (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X))\u000A     (STATE-P1 STATE-STATE)\u000A     (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> CHANNEL)\u000A     (OPEN-OUTPUT-CHANNEL-P1 CHANNEL\u000A                             :CHARACTER STATE-STATE))\u000AACL2 !&gt;</code> \u000A \u000A <p>If you want to execute the guard for <tt><see topic=\"ACL2____PRINC_42\">princ$</see></tt>, then according to the \u000A suggestion above, you should consider the guard for <v>(<see topic=\"ACL2____PRINC_42\">princ$</see> x channel\u000A state)</v> to be as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X)\u000A         (<see topic=\"COMMON-LISP____CHARACTERP\">CHARACTERP</see> X)\u000A         (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A         (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X))\u000A     (STATE-P STATE)\u000A     (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> CHANNEL)\u000A     (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL-P\">OPEN-OUTPUT-CHANNEL-P</see> CHANNEL :CHARACTER STATE))</code> \u000A \u000A <p>For example, we can check the guard for a given value and channel as \u000A follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">let</see> ((x 3) (channel *standard-co*))\u000A         (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X)\u000A                  (<see topic=\"COMMON-LISP____CHARACTERP\">CHARACTERP</see> X)\u000A                  (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A                  (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X))\u000A              (STATE-P STATE)\u000A              (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> CHANNEL)\u000A              (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL-P\">OPEN-OUTPUT-CHANNEL-P</see> CHANNEL :CHARACTER STATE)))\u000AT\u000AACL2 !&gt;</code> \u000A \u000A <p>Comment for advanced users: Function <tt><see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see></tt> is \u000A identical as a function to <v>open-output-channel</v>, except that the former \u000A may be called even during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion and <tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> <see topic=\"ACL2____HINTS\">hints</see>, but requires that there is an active trust tag \u000A (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p> \u000A \u000A <p>Finally, we note that the <see topic=\"ACL2____STD_F2IO\">std/io</see> library contains useful file io \u000A functions whose definitions illustrate some of the features described \u000A above.</p>"],

 "ACL2____XARGS":[["Defun","Declare"],"ACL2 Sources","ACL2","<p>Common Lisp's <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> function does not easily allow one to \u000A pass extra arguments such as ``<see topic=\"ACL2____HINTS\">hints</see>''.  ACL2 therefore supports a \u000A peculiar new declaration (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>) designed explicitly for passing \u000A additional arguments to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> via a keyword-like syntax.  This \u000A declaration can also be used with <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>, but only for <v>xargs</v> \u000A keyword <v>:GUARD</v>; so we restrict attention below to use of <v>xargs</v> in \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>The following declaration is nonsensical but does illustrate all of the \u000A <v>xargs</v> keywords for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (which are the members of the list \u000A <v>*xargs-keywords*</v>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                :guard-hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____THEORY\">theory</see> batch1)))\u000A                :guard-debug t\u000A                :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____THEORY\">theory</see> batch1)))\u000A                :measure (<see topic=\"COMMON-LISP____-\">-</see> i j)\u000A                :ruler-extenders :basic\u000A                :mode :logic\u000A                :non-executable t\u000A                :normalize nil\u000A                :otf-flg t\u000A                :stobjs ($s)\u000A                :verify-guards t\u000A                :split-types t\u000A                :well-founded-relation my-wfr))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____XARGS\">xargs</see> :key1 val1 ... :keyn valn)</code> \u000A \u000A <p>where the keywords and their respective values are as shown below.  Note \u000A that once ``inside'' the xargs form, the ``extra arguments'' to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A are passed exactly as though they were keyword arguments.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt><br></br> \u000A <v>:GUARD</v><br></br> \u000A <v>:</v><v>GUARD</v><br></br> \u000A \u000A <v>Value</v> is a term involving only the formals of the function being defined. \u000A The actual <see topic=\"ACL2____GUARD\">guard</see> used for the definition is the conjunction of all the \u000A <see topic=\"ACL2____GUARD\">guard</see>s and types (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>) <see topic=\"COMMON-LISP____DECLARE\">declare</see>d.</p> \u000A \u000A <p><v>:GUARD-HINTS</v><br></br> \u000A \u000A <v>Value</v>: hints (see <see topic=\"ACL2____HINTS\">hints</see>), to be used during the <see topic=\"ACL2____GUARD\">guard</see> \u000A verification proofs as opposed to the termination proofs of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.</p> \u000A \u000A <p><v>:GUARD-DEBUG</v><br></br> \u000A \u000A <v>Value</v>: <v>nil</v> by default, else directs ACL2 to decorate each guard \u000A proof obligation with hypotheses indicating its sources.  See <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt><br></br> \u000A \u000A Value: hints (see <see topic=\"ACL2____HINTS\">hints</see>), to be used during the termination proofs as \u000A opposed to the <see topic=\"ACL2____GUARD\">guard</see> verification proofs of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.</p> \u000A \u000A <p><v>:MEASURE</v><br></br> \u000A \u000A <v>Value</v> is a term involving only the formals of the function being defined. \u000A This term is indicates what is getting smaller in the recursion.  The \u000A well-founded relation with which successive measures are compared is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>.  Also allowed is a special case, <v>(:? v1 ... vk)</v>, where <v>(v1\u000A ... vk)</v> enumerates a subset of the formal parameters such that some valid \u000A measure involves only those formal parameters.  However, this special case is \u000A only allowed for definitions that are redundant \u000A (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) or are executed when skipping proofs \u000A (see <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>).  Another special case is <v>:MEASURE nil</v>, which is \u000A treated the same as if <v>:MEASURE</v> is omitted.</p> \u000A \u000A <p><v>:RULER-EXTENDERS</v><br></br> \u000A \u000A For recursive definitions (possibly mutually recursive), <v>value</v> controls \u000A termination analysis and the resulting stored induction scheme.  See <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> for a discussion of legal values and their effects.</p> \u000A \u000A <p><v>:MODE</v><br></br> \u000A \u000A <v>Value</v> is <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, indicating the \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> mode of the function introduced.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.  If \u000A unspecified, the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> mode defaults to the default <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> mode \u000A of the current <see topic=\"ACL2____WORLD\">world</see>.  To convert a function from <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, see <see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>.</p> \u000A \u000A <p><v>:NON-EXECUTABLE</v><br></br> \u000A \u000A <v>Value</v> is normally <v>t</v> or <v>nil</v> (the default).  Rather than stating \u000A <v>:non-executable t</v> directly, which requires <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode and \u000A that the definitional body has a certain form, we suggest using the macro \u000A <v>defun-nx</v> or <v>defund-nx</v>; see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>.  A third value of \u000A <v>:non-executable</v> for advanced users is <v>:program</v>, which is generated \u000A by expansion of <v>defproxy</v> forms; see <see topic=\"ACL2____DEFPROXY\">defproxy</see>.  For another way to \u000A deal with non-executability, see <see topic=\"ACL2____NON-EXEC\">non-exec</see>.</p> \u000A \u000A <p><v>:NORMALIZE</v><br></br> \u000A \u000A Value is a flag telling <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> whether to propagate <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> tests \u000A upward.  Since the default is to do so, the value supplied is only of interest \u000A when it is <v>nil</v>. \u000A (See <see topic=\"COMMON-LISP____DEFUN\">defun</see>).</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt><br></br> \u000A \u000A Value is a flag indicating ``onward through the fog'' \u000A (see <see topic=\"ACL2____OTF-FLG\">otf-flg</see>).</p> \u000A \u000A <p><v>:STOBJS</v><br></br> \u000A \u000A <v>Value</v> is either a single <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> name or a true list of stobj names \u000A (see <see topic=\"ACL2____STOBJ\">stobj</see> and see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>, and perhaps see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>).  Every stobj name among the formals of the function must be \u000A listed, if the corresponding actual is to be treated as a stobj.  That is, if \u000A a function uses a stobj name as a formal parameter but the name is not \u000A declared among the <v>:stobjs</v> then the corresponding argument is treated as \u000A ordinary.  The only exception to this rule is <tt><see topic=\"ACL2____STATE\">state</see></tt>: whether you \u000A include it or not, <v>state</v> is always treated as a single-threaded object. \u000A This declaration has two effects.  One is to enforce the syntactic \u000A restrictions on single-threaded objects.  The other is to strengthen the \u000A <tt><see topic=\"ACL2____GUARD\">guard</see></tt> of the function being defined so that it includes conjuncts \u000A specifying that each declared single-threaded object argument satisfies the \u000A recognizer for the corresponding single-threaded object.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt><br></br> \u000A \u000A <v>Value</v> is <v>t</v> or <v>nil</v>, indicating whether or not <see topic=\"ACL2____GUARD\">guard</see>s are \u000A to be verified upon completion of the termination proof.  This flag should \u000A only be <v>t</v> if the <v>:mode</v> is unspecified but the default <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A mode is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, or else the <v>:mode</v> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.</p> \u000A \u000A <p><v>:</v><v>SPLIT-TYPES</v><br></br> \u000A \u000A <v>Value</v> is <v>t</v> or <v>nil</v>, indicating whether or not <see topic=\"COMMON-LISP____TYPE\">type</see>s are \u000A to be proved from the <see topic=\"ACL2____GUARD\">guard</see>s.  The default is <v>nil</v>, indicating that \u000A type declarations (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>) contribute to the <see topic=\"ACL2____GUARD\">guard</see>s.  If \u000A the value is <v>t</v>, then instead, the expressions corresponding to the type \u000A declarations (see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>) are conjoined into a ``split-type \u000A expression,'' and guard verification insists that this term is implied by the \u000A specified <v>:guard</v>.  Suppose for example that a definition has the \u000A following <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (p x y) :split-types t)\u000A         (<see topic=\"COMMON-LISP____TYPE\">type</see> integer x)\u000A         (<see topic=\"COMMON-LISP____TYPE\">type</see> (satisfies good-bar-p) y))</code> \u000A \u000A <p>Then for guard verification, <v>(p x y)</v> is assumed, and in addition to \u000A the usual proof obligations derived from the body of the definition, guard \u000A verification requires a proof that <v>(p x y)</v> implies both <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v> \u000A and <v>(good-bar-p y)</v>.  See community book \u000A <v>demos/split-types-examples.lisp</v> for small examples.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt><br></br> \u000A \u000A <v>Value</v> is a function symbol that is known to be a well-founded relation in \u000A the sense that a rule of class <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> has been \u000A proved about it.  See <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.</p>"],

 "ACL2____CERTIFICATE":[["Books-tour"],"ACL2 Sources","ACL2","<p>A book, say <v>&quot;arith&quot;</v>, is said to have a ``certificate'' if \u000A there is a file named <v>&quot;arith.cert&quot;</v>.  Certificates are created by the \u000A function <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and inspected by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  Check \u000A sums are used to help ensure that certificates are legitimate and that the \u000A corresponding book has not been modified since certification.  But because the \u000A file system is insecure and check sums are not perfect it is possible for the \u000A inclusion of a book to cause inconsistency even though the book carries an \u000A impeccable certificate.</p> \u000A \u000A <p>The certificate includes the version number of the certifying ACL2.  A book \u000A is considered uncertified if it is included in an ACL2 with a different <see topic=\"ACL2____VERSION\">version</see> number.</p> \u000A \u000A <p>The presence of a ``valid'' certificate file for a book attests to two \u000A things: all of the <see topic=\"ACL2____EVENTS\">events</see> of the book are admissible in a certain \u000A extension of the initial ACL2 logic, and the non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> \u000A of the book are independent of the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> ones (see <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see>).  In addition, the certificate contains the <see topic=\"ACL2____COMMAND\">command</see>s used to construct the <see topic=\"ACL2____WORLD\">world</see> in which certification occurred. \u000A Among those <see topic=\"ACL2____COMMAND\">command</see>s, of course, are the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s defining the \u000A packages used in the book.  When a book is included into a host <see topic=\"ACL2____WORLD\">world</see>, \u000A that <see topic=\"ACL2____WORLD\">world</see> is first extended by the <see topic=\"ACL2____COMMAND\">command</see>s listed in the \u000A certificate for the book.  Unless that causes an error due to name conflicts, \u000A the extension ensures that all the packages used by the book are identically \u000A defined in the host <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p><i>Security:</i></p> \u000A \u000A <p>Because the host file system is insecure, there is no way ACL2 can \u000A guarantee that the contents of a book remain the same as when its certificate \u000A was written.  That is, between the time a book is certified and the time it is \u000A used, it may be modified.  Furthermore, certificates can be counterfeited. \u000A Check sums (see <see topic=\"ACL2____CHECK-SUM\">check-sum</see>) are used to help detect such problems.  But \u000A check sums provide imperfect security: two different files can have the same \u000A check sum.</p> \u000A \u000A <p>Therefore, from the strictly logical point of view, one must consider even \u000A the inclusion of certified <see topic=\"ACL2____BOOKS\">books</see> as placing a burden on the user:</p> \u000A \u000A <blockquote> \u000A \u000A <p>The non-erroneous inclusion of a certified book is consistency preserving \u000A provided (a) the objects read by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> from the certificate \u000A were the objects written there by a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and (b) the forms \u000A read by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> from the book itself are the forms read by the \u000A corresponding <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>We say that a given execution of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is ``certified'' if a \u000A certificate file for the book is present and well-formed and the check sum \u000A information contained within it supports the conclusion that the <see topic=\"ACL2____EVENTS\">events</see> \u000A read by the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> are the ones checked by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>. \u000A When an uncertified <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> occurs, warnings are printed or \u000A errors are caused.  But even if no warning is printed, you must accept burdens \u000A (a) and (b) if you use <see topic=\"ACL2____BOOKS\">books</see>.  These burdens are easier to live with if \u000A you protect your <see topic=\"ACL2____BOOKS\">books</see> so that other users cannot write to them, you \u000A abstain from running concurrent ACL2 jobs, and you abstain from counterfeiting \u000A certificates.  But even on a single user uniprocessor, you can shoot yourself \u000A in the foot by using the ACL2 <see topic=\"ACL2____IO\">io</see> primitives to fabricate an \u000A inconsistent book and the corresponding certificate.</p> \u000A \u000A <p>Note that part (a) of the burden described above implies, in particular, \u000A that there are no guarantees when a certificate is copied.  When <see topic=\"ACL2____BOOKS\">books</see> \u000A are renamed (as by copying them), it is recommended that their certificates be \u000A removed and the <see topic=\"ACL2____BOOKS\">books</see> be recertified.  The expectation is that \u000A recertification will go through without a hitch if relative <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>s \u000A are used.  See <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>, which is not on the guided tour.</p> \u000A \u000A <p>Certificates essentially contain two parts, a <see topic=\"ACL2____PORTCULLIS\">portcullis</see> and a <see topic=\"ACL2____KEEP\">keep</see>.  There is a third part, an <v>expansion-alist</v>, in order to record \u000A expansions if <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> has been used, but the user need not be \u000A concerned with that level of detail.</p> \u000A \u000A <p>See <see topic=\"ACL2____PORTCULLIS\">portcullis</see> to continue the guided tour through <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____BOOKS":[["ACL2","Note1"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is about ACL2 <a \u000A href='https://en.wikipedia.org/wiki/Source_code'>source code</a> files. \u000A However, there are also <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html#Books'>traditional, \u000A paper books</a> published about ACL2 and its applications.</p> \u000A \u000A <p>You will almost surely want to organize your own ACL2 work into books. \u000A They facilitate reuse, allow you to reload proofs more quickly, allow you to \u000A rebuild parts of your proof in parallel, and so forth.  You will also want to \u000A be aware of the many <i>community books</i>, which provide useful tools and \u000A lemmas to build upon.  See <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> for more information, \u000A including how to contribute.</p> \u000A \u000A <h3>Introduction</h3> \u000A \u000A <p>A <b>book</b> is a file of ACL2 forms.  Books are prepared entirely by the \u000A user of the system, i.e., they are <i>source files</i> not <i>object \u000A files</i>.  Some of the forms in a book are marked <tt><see topic=\"ACL2____LOCAL\">local</see></tt> and the \u000A others are considered ``non-local.''</p> \u000A \u000A <p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> lets you load a book into any ACL2 <see topic=\"ACL2____WORLD\">world</see>.  A \u000A successful <v>include-book</v> extends the logic of the host <see topic=\"ACL2____WORLD\">world</see> by \u000A adding just the non-local <see topic=\"ACL2____EVENTS\">events</see> in the book.  Ordinarily, you might \u000A include a variety of books to load all of their definitions and rules.</p> \u000A \u000A <p>Successful book inclusion is consistency preserving, provided that the book \u000A itself is consistent, as discussed later.  However, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A assumes the <see topic=\"ACL2____EVENTS\">events</see> in a book are valid, so if you include a book that \u000A contains an inconsistency (e.g., an inadmissible definition) then the \u000A resulting theory is inconsistent!</p> \u000A \u000A <p><tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> lets you <i>certify</i> a book to guarantee that its \u000A successful inclusion is consistency preserving.  During certification, both \u000A the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> and non-local forms are processed.  This lets you mark as \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> any <see topic=\"ACL2____EVENTS\">events</see> you need for certification, but that you want \u000A to hide from users of the book&mdash;e.g., the hacks, crocks, and kludges on \u000A the way to a good set of <see topic=\"ACL2____REWRITE\">rewrite</see> rules.</p> \u000A \u000A <p>Certification can also <see topic='ACL2____COMPILATION'>compile</see> a book \u000A to speed up the execution of the functions defined within it.  The desire to \u000A compile books is largely responsible for the restrictions we put on the forms \u000A allowed in books.</p> \u000A \u000A <p>Extensive <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> is available on the various aspects of \u000A books.  We recommend that you read it all before using books.  It has been \u000A written so as to make sense when read in a certain linear sequence, called the \u000A ``guided tour'', though in general you may browse through it randomly.  If you \u000A are on the guided tour, you should next read <see topic=\"ACL2____BOOK-EXAMPLE\">book-example</see>.</p>"],

 "COMMON-LISP____DECLARE":[["Programming"],"ACL2 Sources","ACL2","<p>Common Lisp provides a declaration mechanism that allows the \u000A programmer to explain additional information to the compiler.  For \u000A instance:</p> \u000A \u000A <ul> \u000A \u000A <li>The programmer might declare that some variable always has some particular \u000A type.  The compiler might then, depending on its optimization/safety settings, \u000A either add run-time checks to ensure that this really is true, or optimize the \u000A compiled code by assuming the variable has the correct type.</li> \u000A \u000A <li>The programmer might declare that some variable is <v>ignore</v>d.  The \u000A compiler might then, instead of warning the programmer that the variable is \u000A never used, explicitly check to make sure at it really is never used.</li> \u000A \u000A </ul> \u000A \u000A <p>ACL2 supports the above kinds of declarations, and also adds its own kinds \u000A of declarations for specifying things like the <see topic=\"ACL2____GUARD\">guard</see>s and <see topic=\"ACL2____MEASURE\">measure</see>s of functions, as described in <see topic=\"ACL2____XARGS\">xargs</see>.</p> \u000A \u000A <p>There are also other kinds of Common Lisp declarations that ACL2 does not \u000A support, e.g., pertaining to inlining, safety settings, variable lifetime, and \u000A so forth.</p> \u000A \u000A <h3>Usage</h3> \u000A \u000A <p>Examples:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y z))\u000A(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> x y z)\u000A         (<see topic=\"COMMON-LISP____TYPE\">type</see> integer i j k)\u000A         (<see topic=\"COMMON-LISP____TYPE\">type</see> (satisfies integerp) m1 m2))\u000A(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                            (&amp;lt;= 0 i))\u000A                :guard-hints ((&quot;Goal&quot; :use (:instance lemma3\u000A                                              (x (<see topic=\"COMMON-LISP_____B2\">+</see> i j)))))))</code> \u000A \u000A <p>General Form:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> d1 ... dn)</code> \u000A \u000A <p>where, in ACL2, each <v>di</v> is of one of the following forms:</p> \u000A \u000A <dl> \u000A \u000A <dt><v>(<see topic=\"COMMON-LISP____IGNORE\">ignore</see> v1 ... vn)</v></dt> \u000A \u000A <dd>where each <v>vi</v> is a variable introduced in the immediately superior \u000A lexical environment.  These variables must not occur free in the scope of the \u000A declaration.  This declaration can be useful for inhibiting compiler warnings; \u000A see also <see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see>.</dd> \u000A \u000A <dt><v>(<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> v1 ... vn)</v></dt> \u000A \u000A <dd>where each <v>vi</v> is a variable introduced in the immediately superior \u000A lexical environment.  These variables need not occur free in the scope of the \u000A declaration.  This declaration can be useful for inhibiting compiler warnings; \u000A see also <see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see>.</dd> \u000A \u000A <dt><v>(<see topic=\"COMMON-LISP____TYPE\">type</see> type-spec v1 ... vn)</v></dt> \u000A \u000A <dd>where each <v>vi</v> is a variable introduced in the immediately superior \u000A lexical environment and <v>type-spec</v> is a type specifier (as described in \u000A the documentation for <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>).  This declaration can be useful for \u000A optimizing Common Lisp execution speed.</dd> \u000A \u000A <dt><v>(<see topic=\"ACL2____XARGS\">xargs</see> :key1 val1 ... :keyn valn)</v></dt> \u000A \u000A <dd>where the legal values of the keys and values are described in the \u000A documentation for <see topic=\"ACL2____XARGS\">xargs</see>.  These declarations are only allowed at the \u000A top level of definitions (<tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>, as shown below), \u000A and convey information such as the <see topic=\"ACL2____GUARD\">guard</see> and <see topic=\"ACL2____MEASURE\">measure</see> for a \u000A function.</dd> \u000A \u000A <dt><v>(<see topic=\"COMMON-LISP____OPTIMIZE\">optimize</see> ...)</v></dt> \u000A \u000A <dd>for example, <v>(<see topic=\"COMMON-LISP____OPTIMIZE\">optimize</see> (safety 3))</v>.  This is allowed only at the top \u000A level of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> forms and is probably only rarely of any interest.  See \u000A any Common Lisp documentation for more information.</dd> \u000A \u000A </dl> \u000A \u000A <p>Declarations in ACL2 may occur only where <v>dcl</v> occurs below:</p> \u000A \u000A <ul> \u000A <li><v>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> name args doc-string dcl ... dcl body)</v></li> \u000A <li><v>(<see topic=\"COMMON-LISP____DEFMACRO\">DEFMACRO</see> name args doc-string dcl ... dcl body)</v></li> \u000A <li><v>(<see topic=\"COMMON-LISP____LET\">LET</see> ((v1 t1) ...) dcl ... dcl body)</v></li> \u000A <li><v>(<see topic=\"ACL2____MV-LET\">MV-LET</see> (v1 ...) term dcl ... dcl body)</v></li> \u000A <li><v>(<see topic=\"COMMON-LISP____FLET\">FLET</see> ((<see topic=\"ACL2____NAME\">name</see> args dcl ... dcl body) ...))</v></li> \u000A </ul> \u000A \u000A <p>Of course, if a form macroexpands into one of these (e.g., as <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> \u000A expands into nested <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>s and our <v>er-let*</v> expands into nested \u000A <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>s) then declarations are permitted as handled by the macros \u000A involved.</p> \u000A \u000A <p><v>Declare</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p>"],

 "ACL2____TYPE-PRESCRIPTION":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> integerp-foo                       ; Assumes that foo has been\u000A  (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo x y))                     ; defined; then, states that\u000A  :rule-classes :type-prescription)        ; (foo x y) is of type integer.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> characterp-nth-type-prescription   ; (<see topic=\"COMMON-LISP____NTH\">Nth</see> n lst) is of type character\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see>                                 ; provided the hypotheses can be\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____CHARACTER-LISTP\">character-listp</see> lst)              ; established by type reasoning.\u000A        (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)\u000A        (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> lst)))\u000A   (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n lst)))\u000A  :rule-classes :type-prescription)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> characterp-nth-type-prescription-alt ; equivalent to the above\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____CHARACTER-LISTP\">character-listp</see> lst)\u000A        (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)\u000A        (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> lst)))\u000A   (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n lst)))\u000A  :rule-classes ((:type-prescription :typed-term (<see topic=\"COMMON-LISP____NTH\">nth</see> n lst))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> demodulize-type-for-quote-value  ; (Demodulize a lst 'value ans) is\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see>                               ; either a nonnegative integer or\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> a)                         ; of the same type as ans, provided\u000A        (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)                 ; the hyps can be established by type\u000A        (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> a lst))            ; reasoning\u000A   (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (demodulize a lst 'value ans))\u000A            (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> (demodulize a lst 'value ans) 0))\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (demodulize a lst 'value ans) ans)))\u000A  :rule-classes :type-prescription)</code> \u000A \u000A <p>To specify the term whose type (see <see topic=\"ACL2____TYPE-SET\">type-set</see>) is described by the \u000A rule, provide that term as the value of the <v>:typed-term</v> field of the rule \u000A class object.</p> \u000A \u000A <code>General Form (after preprocessing; see below):\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> hyps\u000A         (<see topic=\"COMMON-LISP____OR\">or</see> type-restriction1-on-pat\u000A             ...\u000A             type-restrictionk-on-pat\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> pat var1)\u000A             ...\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> pat varj)))</code> \u000A \u000A <p>where <v>pat</v> is the application of some function symbol to some \u000A arguments, each <v>type-restrictioni-on-pat</v> is a term involving <v>pat</v> and \u000A containing no variables outside of the occurrences of <v>pat</v>, and each \u000A <v>vari</v> is one of the variables of <v>pat</v>.  Generally speaking, the \u000A <v>type-restriction</v> terms ought to be terms that inform us as to the type of \u000A <v>pat</v>.  Ideally, they should be ``primitive recognizing expressions'' about \u000A <v>pat</v>; see <see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see>.  We describe preprocessing at the \u000A end of this topic.</p> \u000A \u000A <p>If the <v>:typed-term</v> is not provided in the rule class object, it is \u000A computed heuristically by looking for a term in the conclusion whose type is \u000A being restricted.  An error is caused if no such term is found.</p> \u000A \u000A <p>Roughly speaking, the effect of adding such a rule is to inform the ACL2 \u000A typing mechanism that <v>pat</v> has the type described by the conclusion, when \u000A the hypotheses are true.  In particular, the type of <v>pat</v> is within the \u000A union of the types described by the several disjuncts.  The ``type described \u000A by'' <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> pat vari)</v> is the type of <v>vari</v>.</p> \u000A \u000A <p>More operationally, when asked to determine the type of a term that is an \u000A instance of <v>pat</v>, ACL2 will first attempt to establish the hypotheses. \u000A <b>This is done by type reasoning alone, not rewriting!</b> However, if some \u000A hypothesis is a call of <tt><see topic=\"ACL2____FORCE\">force</see></tt>, then forcing may occur, which may \u000A ultimately invoke the rewriter; see <see topic=\"ACL2____FORCE\">force</see> and see <see topic=\"ACL2____CASE-SPLIT\">case-split</see>. \u000A So-called free variables in hypotheses are treated specially; see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>.  Provided the hypotheses are established by type reasoning, \u000A ACL2 then unions the types described by the <v>type-restrictioni-on-pat</v> \u000A terms together with the types of those subexpressions of <v>pat</v> identified \u000A by the <v>vari</v>.  The final type computed for a term is the intersection of \u000A the types implied by each applicable rule.  Type prescription rules may be \u000A disabled.</p> \u000A \u000A <p>You can limit the recursive establishment of hypotheses of rules; see <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>.</p> \u000A \u000A <p>Because only type reasoning is used to establish the hypotheses of \u000A <v>:type-prescription</v> rules, some care must be taken with the hypotheses. \u000A Suppose, for example, that the non-recursive function <v>my-statep</v> is \u000A defined as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-statep (x)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x) 2)))</code> \u000A \u000A <p>and suppose <v>(my-statep s)</v> occurs as a hypothesis of a \u000A <v>:type-prescription</v> rule that is being considered for use in the proof \u000A attempt for a conjecture with the hypothesis <v>(my-statep s)</v>.  Since the \u000A hypothesis in the conjecture is rewritten, it will become the conjunction of \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> s)</v> and <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> s) 2)</v>.  Those two terms will be \u000A assumed to have type <v>t</v> in the context in which the \u000A <v>:type-prescription</v> rule is tried.  But type reasoning will be unable to \u000A deduce that <v>(my-statep s)</v> has type <v>t</v> in this context.  Thus, either \u000A <v>my-statep</v> should be disabled (see <see topic=\"ACL2____DISABLE\">disable</see>) during the proof \u000A attempt or else the occurrence of <v>(my-statep s)</v> in the \u000A <v>:type-prescription</v> rule should be replaced by the conjunction into which \u000A it rewrites.</p> \u000A \u000A <p>While this example makes it clear how non-recursive predicates can cause \u000A problems, non-recursive functions in general can cause problems.  For example, \u000A if <v>(mitigate x)</v> is defined to be <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (<see topic=\"COMMON-LISP____1-\">1-</see> x) x)</v> then \u000A the hypothesis <v>(pred (mitigate s))</v> in the conjecture will rewrite, \u000A opening <v>mitigate</v> and splitting the conjecture into two subgoals, one in \u000A which <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> s)</v> and <v>(pred (<see topic=\"COMMON-LISP____1-\">1-</see> x))</v> are assumed and the other in \u000A which <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> s))</v> and <v>(pred x)</v> are assumed.  But <v>(pred\u000A (mitigate s))</v> will not be typed as <v>t</v> in either of these contexts.  The \u000A moral is: beware of non-recursive functions occuring in the hypotheses of \u000A <v>:type-prescription</v> rules.</p> \u000A \u000A <p>Because of the freedom one has in forming the conclusion of a \u000A type-prescription, we have to use heuristics to recover the pattern, <v>pat</v>, \u000A whose type is being specified.  In some cases our heuristics may not identify \u000A the intended term and the <v>:type-prescription</v> rule will be rejected as \u000A illegal because the conclusion is not of the correct form.  When this happens \u000A you may wish to specify the <v>pat</v> directly.  This may be done by using a \u000A suitable rule class token.  In particular, when the token \u000A <v>:type-prescription</v> is used it means ACL2 is to compute pat with its \u000A heuristics; otherwise the token should be of the form <v>(:type-prescription\u000A :typed-term pat)</v>, where <v>pat</v> is the term whose type is being \u000A specified.</p> \u000A \u000A <p>The defun event may generate a <v>:type-prescription</v> rule.  Suppose \u000A <v>fn</v> is the name of the function concerned.  Then <v>(:type-prescription\u000A fn)</v> is the rune given to the type-prescription, if any, generated for \u000A <v>fn</v> by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  (The trivial rule, saying <v>fn</v> has unknown type, \u000A is not stored, but <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> still allocates the rune and the corollary of \u000A this rune is known to be <v>t</v>.)</p> \u000A \u000A <p>We close with a discussion of how, before a term is parsed into a \u000A <v>:type-prescription</v> rule, it is preprocessed.  We describe this \u000A preprocessing in some detail below, but first consider the following \u000A (contrived) example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> append-tp-example\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((result (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)))\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NAT-LISTP\">nat-listp</see> x)\u000A             (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____LET\">let</see> ((second-hyp (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> y)))\u000A                        second-hyp)\u000A                      (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> result))))\u000A  :rule-classes :type-prescription)</code> \u000A \u000A <p>This theorem is parsed into a type-prescription rule with the following \u000A hypotheses and conclusion.</p> \u000A \u000A <code>(<see topic=\"ACL2____NAT-LISTP\">nat-listp</see> x) ; first hypothesis\u000A((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (second-hyp) second-hyp) (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> y)) ; second hypothesis\u000A(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> x y)) ; conclusion</code> \u000A \u000A <p>Notice that the top-level <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> was expanded, i.e., <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> \u000A was substituted for <v>result</v> &mdash; more accurately, <v>(<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> x\u000A y)</v> was substituted for <v>result</v>, since <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> is a macro that \u000A abbreviates <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>.  Also notice that the two hypotheses were \u000A ``flattened'' in the sense that they were gathered up into a list.  Finally, \u000A notice that the <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> in the second hypothesis was not expanded (it was \u000A merely translated to internal form, using <v>LAMBDA</v>).  If you actually \u000A submit the theorem above, you will get warnings, which you may choose to \u000A ignore; the application of <v>type-prescription</v> rules is somewhat subtle, so \u000A if you use them then you may wish to experiment to see which forms work best \u000A for you.</p> \u000A \u000A <p>Here is the detail promised above, for parsing a term into a \u000A <v>:type-prescription</v> rule.  There are two steps.  (1) ACL2 first translates \u000A the term, expanding all macros (see <see topic=\"ACL2____TRANS\">trans</see>) and also expanding away \u000A calls of all so-called ``guard holders,'' <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt> and <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> (the latter resulting for example from calls of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, \u000A <tt><see topic=\"ACL2____MBE\">mbe</see></tt>, or <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>), as well as expansions of the macro `<tt><see topic=\"COMMON-LISP____THE\">the</see></tt>'.  (2) Then the the translated term is traversed top-down, expanding away \u000A <v>lambda</v>s (<tt><see topic=\"COMMON-LISP____LET\">let</see></tt>, <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> expressions) and \u000A flattening the <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> structure, until the conclusion is exposed; \u000A then the conclusion's <v>lambda</v>s are also expanded away.  The simplest way \u000A to understand (2) may be to look at the definition of ACL2 source function \u000A <v>unprettyify-tp</v>, which implements Step (2), say by evaluating <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt><v> unprettyify-tp</v>.</p>"],

 "ACL2____WORMHOLE":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Example Form: \u000A; The following form enters a recursive read-eval-print loop on a \u000A; copy of the current state, allowing you to interact with that loop. \u000A; Note that the form does not mention the ACL2 state variable! \u000A; Evaluate the form below. Inside the resulting loop, define some function, \u000A; e.g., with <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)</v>.  Then exit with <v>:q</v> and observe, \u000A; e.g., with <v>:pe foo</v>, that the external state did not change. \u000A \u000A(wormhole 'foo \u000A          '(lambda (whs) (set-wormhole-entry-code whs :ENTER)) \u000A          nil \u000A          '(list 'hello 'there)) \u000A</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____WORMHOLE\">wormhole</see> name entry-lambda input form\u000A  :current-package    ...  ; known package name\u000A  :ld-skip-proofsp    ...  ; nil, t or 'include-book\u000A  :ld-redefinition-action  ; nil or '(:a . :b)\u000A  :ld-prompt          ...  ; nil, t, or some prompt printer fn\u000A  :ld-missing-input-ok ... ; nil, t, :warn, or warning message\u000A  :ld-pre-eval-filter ...  ; :all, :query, or some new name\u000A  :ld-pre-eval-print  ...  ; nil, t, or :never\u000A  :ld-post-eval-print ...  ; nil, t, or :command-conventions\u000A  :ld-evisc-tuple     ...  ; nil or '(alist level length hiding-cars)\u000A  :ld-error-triples   ...  ; nil or t\u000A  :ld-error-action    ...  ; :return!, :return, :continue, or :error\u000A  :ld-query-control-alist  ; alist supplying default responses\u000A  :ld-verbose         ...) ; nil or t</code> \u000A \u000A <p>The keyword arguments above are exactly those of <tt><see topic=\"ACL2____LD\">ld</see></tt> (see <see topic=\"ACL2____LD\">ld</see>) \u000A except that three of <tt><see topic=\"ACL2____LD\">ld</see></tt>'s keyword arguments are missing: the three \u000A that specify the channels <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt>, which default in <v>wormhole</v> to ACL2's comment window.</p> \u000A \u000A <p>There are two ways to create and enter a wormhole: <v>wormhole</v> as \u000A described here and the simpler <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt>.  We recommend you read \u000A this full account of wormholes before using <v>wormhole-eval</v>.</p> \u000A \u000A <p>Ignoring the use of <v>entry-lambda</v>, <v>wormhole</v> manufactures a named \u000A ``wormhole <see topic=\"ACL2____STATE\">state</see>'' and calls the general-purpose ACL2 read-eval-print \u000A loop <tt><see topic=\"ACL2____LD\">ld</see></tt> on it.  However, when <v>ld</v> exits, the wormhole evaporates \u000A and the function <v>wormhole</v> returns <v>nil</v>.  The manufactured state is \u000A like the ``current'' ACL2 <see topic=\"ACL2____STATE\">state</see> except for two things.  First, some \u000A information from the last wormhole state of this name is transferred into the \u000A new state; this allows a wormhole to maintain some state from one call to the \u000A next.  Second, some information from the wormhole call itself is transferred \u000A into the new state; this allows the wormhole to be sensitive to context. \u000A These two changes to the current state are reflected in the settings <v>(<see topic=\"ACL2_____04\">@</see>\u000A wormhole-status)</v> and <v>(<see topic=\"ACL2_____04\">@</see> wormhole-input)</v> discussed in detail below.</p> \u000A \u000A <p>Note that <v>wormhole</v> may be called from environments in which <tt><see topic=\"ACL2____STATE\">state</see></tt> is not bound.  It is still applicative because it always returns \u000A <v>nil</v>.</p> \u000A \u000A <p>There are some restrictions about what can be done inside a wormhole.  As \u000A you may imagine, we really do not ``copy the current state'' but rather just \u000A keep track of how we modified it and undo those modifications upon exit.  An \u000A error is signalled if you try to modify state in an unsupported way.  For this \u000A same reason, wormholes do not allow updating of any user-defined \u000A single-threaded objects.  See <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>One example wormhole is the implementation of the ACL2 <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> facility for tracking the frequency with which rules \u000A are tried.  To implement this feature directly the theorem prover would have \u000A to take the tracking data as an argument and pass it around so that updates \u000A could be accumulated.  This would greatly clutter the code.  Instead, the \u000A tracking data is maintained in a wormhole.  The theorem prover enters the \u000A wormhole to update the data as rules are tried.  When you request a display of \u000A the data, <tt><see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see></tt> enters the wormhole and prints \u000A the data.  But the data is never available outside that wormhole.  The ACL2 \u000A system uses a second wormhole to implement the <tt><see topic=\"ACL2____BRR\">brr</see></tt> facility, allowing \u000A the user to interact with the rewriter as rules are applied.</p> \u000A \u000A <p>We now specify the arguments and behavior of <v>wormhole</v>.</p> \u000A \u000A <p>The <v>name</v> argument must be a quoted constant and is typically a symbol. \u000A It will be the ``name'' of the wormhole.  A wormhole of that name will be \u000A created the first time either <v>wormhole</v> or <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt> is \u000A called.</p> \u000A \u000A <p>Every wormhole name has a ``status.''  The status of a wormhole is stored \u000A outside of ACL2; it is inaccessible to the ACL2 user except when in the named \u000A wormhole.  But the status of a wormhole may be set by the user from within the \u000A wormhole.</p> \u000A \u000A <p>Upon the first call of <v>wormhole</v> or <v>wormhole-eval</v> on a name, the \u000A status of that name is <v>nil</v>.  But in general you should arrange for the \u000A status to be a cons.  The status is set by the quoted <v>lambda</v> every time \u000A <v>wormhole</v> is called; but it may also be set in the <v>form</v> argument (the \u000A first form evaluated in the interactive loop) by assigning to the state global \u000A variable <v>wormhole-status</v>, as with</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status ...)</code> \u000A \u000A <p>or even by the user interacting with the loop if you do not exit the loop \u000A with the first form.  The <v>car</v> of the cons should be either <v>:ENTER</v> or \u000A <v>:SKIP</v> and is called the wormhole's ``entry code.''  The entry code of \u000A <v>nil</v> or an unexpectedly shaped status is <v>:ENTER</v>.  The <v>cdr</v> of the \u000A cons is arbitrary data maintained by you.</p> \u000A \u000A <p>When <v>wormhole</v> is invoked, the status of the specified name is \u000A incorporated into the manufactured wormhole state.  In particular, inside the \u000A wormhole, the status is the value of the state global variable \u000A <v>wormhole-status</v>.  That is, inside the wormhole, the status may be \u000A accessed by <v>(<see topic=\"ACL2_____04\">@</see> wormhole-status)</v> and set by <v>(<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status\u000A ...)</v>, <v>f-get-global</v> and <v>f-put-global</v>.  When <v>ld</v> exits -- \u000A typically because the form <v>:q</v> was read by <v>ld</v> -- the then-current \u000A value of wormhole-status is hidden away so that it can be restored when this \u000A wormhole is entered again.  The rest of the wormhole state is lost.</p> \u000A \u000A <p>This allows a sequence of entries and exits to a wormhole to maintain some \u000A history in the status and this information can be manipulated by ACL2 \u000A functions executing inside the wormhole.</p> \u000A \u000A <p>The second argument to <v>wormhole</v> must be a quoted lambda expression. \u000A We explain it later.</p> \u000A \u000A <p>The third argument, <v>input</v>, may be any term.  The value of the term is \u000A passed into the manufactured wormhole state, allowing you to pass in \u000A information about the calling context.  Inside the wormhole, the <v>input</v> is \u000A available via <v>(<see topic=\"ACL2_____04\">@</see> wormhole-input)</v>.  It could be reassigned via <v>(<see topic=\"ACL2____ASSIGN\">assign</see>\u000A wormhole-input ...)</v>, but there is no reason to do that.</p> \u000A \u000A <p>The fourth argument, <v>form</v>, may be any term; when <tt><see topic=\"ACL2____LD\">ld</see></tt> is called \u000A on the manufactured wormhole state, the first form evaluated by <v>ld</v> will \u000A be the value of <v>form</v>.  Note that <v>form</v> will be translated by <v>ld</v>. \u000A Errors, including guard violations, in the translation or execution of that \u000A first form will leave you in the interactive loop of the wormhole state.</p> \u000A \u000A <p>When used properly, the first form allows you to greet your user before \u000A reading the first interactive command or simply to do whatever computation you \u000A want to do inside the wormhole and exit silently.  We give examples below.</p> \u000A \u000A <p>Manufacturing a wormhole state is relatively expensive; in addition, the \u000A forms executed by <v>ld</v> must be read, translated, and interpreted as with \u000A any user type-in.  The <v>entry-lambda</v> offers a way to avoid this or, at \u000A least, to decide whether to incur that expense.</p> \u000A \u000A <p>Before the wormhole state is manufactured and entered, the \u000A <v>entry-lambda</v> is applied to the current wormhole status with <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt>.  That <v>lambda</v> application must produce a new wormhole \u000A status, which is stored as the wormhole's status.  The entry code for the new \u000A status determines whether <v>wormhole</v> actually manufactures a wormhole state \u000A and calls <v>ld</v>.</p> \u000A \u000A <p>If the entry code for that new status is <v>:ENTER</v> the wormhole state is \u000A manufactured and entered; otherwise, the new status is simply saved as the \u000A most recent status but the wormhole state is not manufactured or entered. \u000A Note therefore that the <v>entry-lambda</v> may be used to perform two \u000A functions: (a) to determine if it is really necessary to manufacture a state \u000A and (b) to update the data in the wormhole status as a function of the old \u000A status without invoking <v>ld</v>.</p> \u000A \u000A <p>The <v>entry-lambda</v> must be a quoted lambda expression of at most one \u000A argument.  Thus, the argument must be either</p> \u000A \u000A <code>'(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) &lt;body&gt;)</code> \u000A \u000A <p>or</p> \u000A \u000A <code>'(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> () &lt;body&gt;)</code> \u000A \u000A <p>Note the quote.  If a formal, e.g., <v>whs</v>, is provided, it must be used \u000A as a variable in the <v>lambda</v> body.  The <v>lambda</v>-expression may contain \u000A free variables, that is, the body may mention variables other than the \u000A <v>lambda</v> formal.  These free variables are understood in the caller's \u000A environment.  These conventions allow us to compile the <v>entry-lambda</v> \u000A application very efficiently when the guard has been verified.</p> \u000A \u000A <p>The guard on a call of <v>wormhole</v> is the conjunction of the guards on \u000A the arguments conjoined with the guard on the body of the <v>entry-lambda</v>. \u000A See <see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> for a discussion of the guard on the \u000A <v>lambda</v>-expression.</p> \u000A \u000A <p>The functions <tt><see topic=\"ACL2____WORMHOLE-STATUSP\">wormhole-statusp</see></tt>, <tt><see topic=\"ACL2____WORMHOLE-ENTRY-CODE\">wormhole-entry-code</see></tt>, <tt><see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see></tt>, <tt><see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see></tt>, <tt><see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see></tt>, \u000A and <tt><see topic=\"ACL2____MAKE-WORMHOLE-STATUS\">make-wormhole-status</see></tt> may be useful in manipulating entry codes and \u000A data in the <v>entry-lambda</v>.</p> \u000A \u000A <p>Note that you access and manipulate the wormhole's status in two different \u000A ways depending on whether you're ``outside'' of the wormhole applying the \u000A quoted <v>lambda</v> or ``inside'' the read-eval-print loop of the wormhole.</p> \u000A \u000A <p>OUTSIDE (<v>wormhole-eval</v>): access via the value of the <v>lambda</v> \u000A formal and set by returning the new status as the value of the <v>lambda</v> \u000A body.</p> \u000A \u000A <p>INSIDE (<v>ld</v> phase of <v>wormhole</v>): access via <v>(<see topic=\"ACL2_____04\">@</see>\u000A wormhole-status)</v>, and set via <v>(<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status ...)</v>.</p> \u000A \u000A <p>Pragmatic Advice on Designing a Wormhole: Suppose you are using wormholes \u000A to implement some extra-logical utility.  You must contemplate how you will \u000A use your wormhole's status to store hidden information.  You might be tempted \u000A to exploit the entry code as part of the status.  For example, you may think \u000A of <v>:ENTER</v> as indicating that your utility is ``turned on'' and <v>:SKIP</v> \u000A as indicating that your utility is ``turned off.''  We advise against such a \u000A design.  We recommend you base your decisions on the wormhole data.  We \u000A recommend that you set but not read the wormhole entry code to signal whether \u000A you wish to enter a full-fledged wormhole.  To use the entry code as a flag \u000A overloads it and invites confusion when your facility is ``turned off'' but \u000A you have to enter the wormhole for some reason.</p> \u000A \u000A <p>For a behind-the-scenes description of how wormholes work, See <see topic=\"ACL2____WORMHOLE-IMPLEMENTATION\">wormhole-implementation</see>.</p> \u000A \u000A <p>Here are some sample situations handled by <v>wormhole-eval</v> and \u000A <v>wormhole</v>.  Let the wormhole in question be named <v>DEMO</v>.  Initially \u000A its status is <v>NIL</v>.  The functions below all maintain the convention that \u000A the status is either <v>nil</v> or of the form <v>(:key . lst)</v>, where \u000A <v>:key</v> is either <v>:SKIP</v> or <v>:ENTER</v> and <v>lst</v> is a true-list of \u000A arbitrary objects.  But since there is no way to prevent the user from \u000A entering the <v>DEMO</v> wormhole interactively and doing something to the \u000A status, this convention cannot be enforced.  Thus, the functions below do what \u000A we say they do, e.g., remember all the values of <v>x</v> ever seen, only if \u000A they're the only functions messing with the <v>DEMO</v> status.  On the other \u000A hand, the guards of all the functions below can be verified.  We have \u000A explicitly declared that the guards on the functions below are to be verified, \u000A to confirm that they can be.  Guard verification is optional but wormholes \u000A (and <v>wormhole-eval</v> in particular) are more efficient when guards have \u000A been verified.  All of the functions defined below return <v>nil</v>.</p> \u000A \u000A <p>The examples below build on each other.  If you really want to understand \u000A wormholes we recommend that you evaluate each of the forms below, in the order \u000A they are discussed.</p> \u000A \u000A <p><b>Q.</b> How do I create a wormhole that prints its status to the comment \u000A window?</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> demo-status ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :verify-guards t))\u000A  (<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A                 '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs)\u000A                    (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;DEMO status:~%~x0~%&quot; whs)\u000A                            whs))\u000A                 nil))</code> \u000A \u000A <p>Note above that after printing the status to the comment window we return \u000A the new (unchanged) status <v>whs</v>.  Had we just written the call of <v>cw</v>, \u000A which returns <v>nil</v>, the function would print the status and then set it to \u000A <v>nil</v>!</p> \u000A \u000A <p><b>Q.</b> How do I use a wormhole to collect every symbol, <v>x</v>, passed \u000A to the function?</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> demo-collect (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :verify-guards t))\u000A  (<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A                 '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs)\u000A                    (<see topic=\"ACL2____MAKE-WORMHOLE-STATUS\">make-wormhole-status</see> whs\u000A                                          (<see topic=\"ACL2____WORMHOLE-ENTRY-CODE\">wormhole-entry-code</see> whs)\u000A                                          (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                                              (<see topic=\"COMMON-LISP____CONS\">cons</see> x (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))\u000A                                              (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))))\u000A                 nil))</code> \u000A \u000A <p>We could have also defined this function this way:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> demo-collect (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A      (<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A                     '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs)\u000A                        (<see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see> whs\u000A                                           (<see topic=\"COMMON-LISP____CONS\">cons</see> x (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))))\u000A                     nil)\u000A      nil))</code> \u000A \u000A <p>Both versions always return <v>nil</v> and both versions collect into the \u000A wormhole data field just the symbols <v>x</v> upon which <v>demo-collect</v> is \u000A called.</p> \u000A \u000A <p><b>Q.</b> How do I use <v>demo-collect</v>?  Below is a function that maps \u000A over a list and computes its length.  But it has been annotated with a call to \u000A <v>demo-collect</v> on every element.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-len (lst)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> lst)\u000A      0\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> 1\u000A         (<see topic=\"ACL2____PROG2_42\">prog2$</see> (demo-collect (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                 (my-len (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code> \u000A \u000A <p>Thus, for example:</p> \u000A \u000A <code>ACL2 !&gt;(my-len '(4 temp car &quot;Hi&quot; rfix))\u000A5\u000AACL2 !&gt;(demo-status)\u000ADEMO status:\u000A(:ENTER RFIX CAR TEMP)\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Q.</b> How do I set the entry code to <v>:ENTER</v> or <v>:SKIP</v> \u000A according to whether <v>name</v> is a <v>member-equal</v> of the list of things \u000A seen so far?  Note that we cannot check this condition outside the wormhole, \u000A because it depends on the list of things collected so far.  We make the \u000A decision inside the <v>lambda</v>-expression.  Note that we explicitly check \u000A that the guard of <v>member-equal</v> is satisfied by the current wormhole \u000A status, since we cannot rely on the invariant that no other function \u000A interferes with the status of the <v>DEMO</v> wormhole.  In the case that the \u000A status is ``unexpected'' we act like the status is <v>nil</v> and set it to \u000A <v>(:SKIP . NIL)</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> demo-set-entry-code (<see topic=\"ACL2____NAME\">name</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :verify-guards t))\u000A  (<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A                 '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs)\u000A                    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))\u000A                        (<see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see>\u000A                         whs\u000A                         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> name (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))\u000A                             :ENTER\u000A                             :SKIP))\u000A                        '(:SKIP . NIL)))\u000A                 nil))</code> \u000A \u000A <p>Thus</p> \u000A \u000A <code>ACL2 !&gt;(demo-set-entry-code 'monday)\u000ANIL\u000AACL2 !&gt;(demo-status)\u000ADEMO status:\u000A(:SKIP RFIX CAR TEMP)\u000ANIL\u000AACL2 !&gt;(demo-set-entry-code 'rfix)\u000ANIL\u000AACL2 !&gt;(demo-status)\u000ADEMO status:\u000A(:ENTER RFIX CAR TEMP)\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Q.</b> Suppose I want to collect every symbol and then, if the symbol \u000A has an <v>ABSOLUTE-EVENT-NUMBER</v> property in the ACL2 logical world, print \u000A the defining event with <v>:pe</v> and then enter an interactive loop; but if \u000A the symbol does not have an <v>ABSOLUTE-EVENT-NUMBER</v>, don't print anything \u000A and don't enter an interactive loop.</p> \u000A \u000A <p>Here it is not important to know what <v>ABSOLUTE-EVENT-NUMBER</v> is; this \u000A example just shows that we can use a wormhole to access the ACL2 logical \u000A world, even in a function that does not take the state as an argument.</p> \u000A \u000A <p>In the code below, we use <v>wormhole</v> instead of <v>wormhole-eval</v>, \u000A because we might have to access the logical world and enter an interactive \u000A loop.  But for efficiency we do as much as we can inside the entry \u000A <v>lambda</v>, where we can check whether <v>x</v> is symbol and collect it into \u000A the data field of the wormhole status.  Note that if we collect <v>x</v>, we \u000A also set the entry code to <v>:ENTER</v>.  If we don't collect <v>x</v>, we set \u000A the entry code to <v>:SKIP</v>.</p> \u000A \u000A <code>(defun collect-symbols-and-print-events (x) \u000A  (declare (xargs :guard t)) \u000A  (wormhole 'demo \u000A            '(lambda (whs) \u000A               (if (symbolp x) \u000A                   (make-wormhole-status whs \u000A                                         :ENTER \u000A                                         (cons x (wormhole-data whs))) \u000A                   (set-wormhole-entry-code whs :SKIP))) \u000A \u000A; The wormhole will not get past here is unless the entry code is \u000A; :ENTER.  If we get past here, we manufacture a state, put \u000A; x into <v>(<see topic=\"ACL2_____04\">@</see> wormhole-input)</v> and call ld in such a way that the \u000A; first form executed is the quoted if-expression below. \u000A \u000A            x \u000A            '(if (getprop (@ wormhole-input) \u000A                          'absolute-event-number \u000A                          nil \u000A                          'CURRENT-ACL2-WORLD (w state)) \u000A                 (er-progn \u000A                  (mv-let (col state) \u000A                          (fmt \"~%Entering a wormhole on the event name ~x0~%\" \u000A                               (list (cons #\\0 (@ wormhole-input))) \u000A                               *standard-co* state nil) \u000A                          (declare (ignore col)) \u000A                          (value nil)) \u000A                  (pe (@ wormhole-input)) \u000A                  (set-ld-prompt 'wormhole-prompt state) \u000A                  (value :invisible)) \u000A                 (value :q)) \u000A            :ld-verbose nil \u000A            :ld-prompt nil)) \u000A</code> \u000A \u000A <p>The ``first form'' (the <v>if</v>) asks whether the <v>wormhole-input</v> \u000A (i.e., <v>x</v>) has an <v>ABSOLUTE-EVENT-NUMBER</v> property.  If so, it enters \u000A an <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> to perform a sequence of commands, each of which returns \u000A an ACL2 error triple (see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>).  The first form uses \u000A <tt><see topic=\"ACL2____FMT\">fmt</see></tt> to print a greeting.  Since <v>fmt</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> col state)</v> \u000A and we must return an error triple, we embed the <v>fmt</v> term in an \u000A <v>(<see topic=\"ACL2____MV-LET\">mv-let</see> (col state) ... (value nil))</v>.  The macro <v>value</v> takes an \u000A object and returns a ``normal return'' error triple.  The second form in the \u000A <v>er-progn</v> uses the ACL2 history macro <v>pe</v> (see <see topic=\"ACL2____PE\">pe</see>) to print the \u000A defining event for a name.  The third form sets the prompt of this \u000A read-eval-print loop to the standard function for printing the wormhole \u000A prompt.  We silenced the printing of the prompt when we called <v>ld</v>, thanks \u000A to the <v>:ld-prompt nil</v> keyword option.  More on this below.  The fourth \u000A form returns the error triple value <v>:invisible</v> as the value of the first \u000A form.  This prevents <v>ld</v> from printing the value of the first form.  Since \u000A we have not exited <v>ld</v>, that function just continues by reading the next \u000A form from the comment window.  The user perceives this as entering a \u000A read-eval-print loop.  We continue in the loop until the user types \u000A <v>:q</v>.</p> \u000A \u000A <p>On the other branch of the <v>if</v>, if the symbol has no \u000A <v>ABSOLUTE-EVENT-NUMBER</v> property, we execute the form <v>(value :q)</v>, \u000A which is the programming equivalent of typing <v>:q</v>.  That causes the \u000A <v>ld</v> to exit.</p> \u000A \u000A <p>The <v>ld</v> special variables set in the call to <v>wormhole</v> and further \u000A manipulated inside the first form to <v>ld</v> may require explanation.  By \u000A setting <v>:</v><tt><see topic=\"ACL2____LD-VERBOSE\">ld-verbose</see></tt> to <v>nil</v>, we prevent <v>ld</v> from printing \u000A the familiar ACL2 banner when <v>ld</v> is called.  If <v>:ld-verbose nil</v> is \u000A deleted, then you would see something like</p> \u000A \u000A <code>ACL2 Version  4.0.  Level 2.\u000A...\u000AType (<see topic=\"ACL2____GOOD-BYE\">good-bye</see>) to quit completely out of ACL2.</code> \u000A \u000A <p>before the first form is read and evaluated.</p> \u000A \u000A <p>By setting <v>:</v><tt><see topic=\"ACL2____LD-PROMPT\">ld-prompt</see></tt> to <v>nil</v> we prevent <v>ld</v> from \u000A printing the prompt before reading and evaluating the first form.</p> \u000A \u000A <p>As this example shows, to use full-blown wormholes you must understand the \u000A protocol for using wormhole status to control whether a wormhole state is \u000A manufactured for <v>ld</v> and you must also understand programming with <tt><see topic=\"ACL2____STATE\">state</see></tt> and the effects of the various <tt><see topic=\"ACL2____LD\">ld</see></tt> ``special variables.''</p> \u000A \u000A <p>From the discussion above we see that wormholes can be used to create \u000A formatted output without passing in the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  For examples see \u000A <see topic=\"ACL2____CW\">cw</see>, in particular the discussion at the end of that documentation \u000A topic.</p>"],

 "ACL2____DEFSTOBJ":[["Events","Stobj"],"ACL2 Sources","ACL2","<p>Note: Novices are advised to avoid <v>defstobj</v>, perhaps instead \u000A using community books <see topic=\"STD____DEFAGGREGATE\">std::defaggregate</see> or \u000A <v>books/data-structures/structures.lisp</v>.  At the least, consider using \u000A <v>(</v><tt><see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see></tt><v> 0)</v> to avoid <see topic=\"ACL2____GUARD\">guard</see> \u000A verification.  On the other hand, after you learn to use <v>defstobj</v>, see \u000A <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see> for another way to introduce single-threaded objects.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *mem-size* 10) ; for use of *mem-size* just below\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st\u000A          (reg :type (array (unsigned-byte 31) (8))\u000A               :initially 0)\u000A          (p-c :type (unsigned-byte 31)\u000A               :initially 555)\u000A          halt                  ; = (halt :type t :initially nil)\u000A          (mem :type (array (unsigned-byte 31) (*mem-size*))\u000A               :initially 0 :resizable t))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> name\u000A          (field1 :type type1 :initially val1 :resizable b1)\u000A          ...\u000A          (fieldk :type typek :initially valk :resizable bk)\u000A          :renaming alist\u000A          :doc doc-string\u000A          :inline flg\u000A          :congruent-to old-stobj-name\u000A          :non-memoizable nm-flg)</code> \u000A \u000A <p>where <v>name</v> is a new symbol, each <v>fieldi</v> is a symbol, each \u000A <v>typei</v> is either a type-indicator (a <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt> or <see topic=\"ACL2____STOBJ\">stobj</see> \u000A name) or of the form <v>(ARRAY type-indicator max)</v>, each <v>vali</v> is an \u000A object satisfying <v>typei</v>, and each <v>bi</v> is <v>t</v> or <v>nil</v>.  Each \u000A pair <v>:initially vali</v> and <v>:resizable bi</v> may be omitted; more on this \u000A below.  The <v>:renaming alist</v> argument is optional and allows the user to \u000A override the default function names introduced by this event.  The <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is also optional.  The <v>:inline flg</v> Boolean argument is also \u000A optional and declares to ACL2 that the generated access and update functions \u000A for the stobj should be implemented as macros under the hood (which has the \u000A effect of inlining the function calls).  The optional <v>:congruent-to\u000A old-stobj-name</v> argument specifies an existing stobj with exactly the same \u000A structure, and is discussed below.  The optional <v>:non-memoizable nm-flg</v> \u000A Boolean argument is ignored when <v>nm-flg</v> is <v>nil</v> and is uninteresting \u000A when not using ACL2(h) (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>); otherwise, it \u000A instructs ACL2 to lay down faster code for functions that return the new stobj \u000A but disallows memoization of any function that takes the new stobj as an \u000A argument.  We describe further restrictions on the <v>fieldi</v>, <v>typei</v>, \u000A <v>vali</v>, and on <v>alist</v> below.  We recommend that you read about \u000A single-threaded objects (stobjs) in ACL2 before proceeding; see <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>The effect of this event is to introduce a new single-threaded object \u000A (i.e., a ``<see topic=\"ACL2____STOBJ\">stobj</see>''), named <v>name</v>, and the associated recognizers, \u000A creator, accessors, updaters, constants, and, for fields of <v>ARRAY</v> type, \u000A length and resize functions.</p> \u000A \u000A <p><i>The Single-Threaded Object Introduced</i></p> \u000A \u000A <p>The <v>defstobj</v> event effectively introduces a new global variable, named \u000A <v>name</v>, which has as its initial logical value a list of <v>k</v> elements, \u000A where <v>k</v> is the number of ``field descriptors'' provided.  The elements \u000A are listed in the same order in which the field descriptors appear.  If the \u000A <v>:type</v> of a field is <v>(ARRAY type-indicator (<see topic=\"COMMON-LISP____MAX\">max</see>))</v> then <v>max</v> is a \u000A non-negative integer or a symbol introduced by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>) whose value \u000A is a non-negative integer, and the corresponding element of the stobj is \u000A initially of length specified by <v>max</v>.</p> \u000A \u000A <p>Whether the value <v>:type</v> is of the form <v>(ARRAY type-indicator\u000A (<see topic=\"COMMON-LISP____MAX\">max</see>))</v> or, otherwise, just <v>type-indicator</v>, then <v>type-indicator</v> is \u000A typically a type-spec; see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>.  However, <v>type-indicator</v> can \u000A also be the name of a stobj that was previously introduced (by <v>defstobj</v> \u000A or <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>).  We ignore this ``nested stobj'' case below; see \u000A <see topic=\"ACL2____NESTED-STOBJS\">nested-stobjs</see> for a discussion of stobjs within stobjs.</p> \u000A \u000A <p>The keyword value <v>:initially val</v> specifies the initial value of a \u000A field, except for the case of a <v>:type</v> <v>(ARRAY type-indicator (<see topic=\"COMMON-LISP____MAX\">max</see>))</v>, \u000A in which case <v>val</v> is the initial value of the corresponding array.</p> \u000A \u000A <p>Note that the actual representation of the stobj in the underlying Lisp may \u000A be quite different; see <see topic=\"ACL2____STOBJ-EXAMPLE-2\">stobj-example-2</see>.  For the moment we focus \u000A entirely on the logical aspects of the object.</p> \u000A \u000A <p>In addition, the <v>defstobj</v> event introduces functions for recognizing \u000A and creating the stobj and for recognizing, accessing, and updating its \u000A fields.  For fields of <v>ARRAY</v> type, length and resize functions are also \u000A introduced.  Constants are introduced that correspond to the accessor \u000A functions.</p> \u000A \u000A <p><i>Restrictions on the Field Descriptions in Defstobj</i></p> \u000A \u000A <p>Each field descriptor is of the form:</p> \u000A \u000A <code>(fieldi :TYPE typei :INITIALLY vali)</code> \u000A \u000A <p>Note that the type and initial value are given in ``keyword argument'' \u000A format and may be given in either order.  The <v>typei</v> and <v>vali</v> \u000A ``arguments'' are not evaluated.  If omitted, the type defaults to <v>t</v> \u000A (unrestricted) and the initial value defaults to <v>nil</v>.</p> \u000A \u000A <p>Each <v>typei</v> must be either a <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt> or else a list of the \u000A form <v>(ARRAY type-spec (<see topic=\"COMMON-LISP____MAX\">max</see>))</v>.  (Again, we are ignoring the case of nested \u000A stobjs, discussed elsewhere; see <see topic=\"ACL2____NESTED-STOBJS\">nested-stobjs</see>.)  The latter forms are \u000A said to be ``array types.''  Examples of legal <v>typei</v> are:</p> \u000A \u000A <code>(INTEGER 0 31)\u000A(SIGNED-BYTE 31)\u000A(ARRAY (SIGNED-BYTE 31) (16))\u000A(ARRAY (SIGNED-BYTE 31) (*c*)) ; where *c* has a non-negative integer value</code> \u000A \u000A <p>The <v>typei</v> describes the objects which are expected to occupy the given \u000A field.  Those objects in <v>fieldi</v> should satisfy <v>typei</v>.  We are more \u000A precise below about what we mean by ``expected.''  We first present the \u000A restrictions on <v>typei</v> and <v>vali</v>.</p> \u000A \u000A <p>Non-Array Types</p> \u000A \u000A <p>When <v>typei</v> is a <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt> it restricts the contents, <v>x</v>, \u000A of <v>fieldi</v> according to the ``meaning'' formula given in the table for \u000A <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt>.  For example, the first <v>typei</v> above restricts the \u000A field to be an integer between 0 and 31, inclusive.  The second restricts the \u000A field to be an integer between -2^30 and (2^30)-1, inclusive.</p> \u000A \u000A <p>The initial value, <v>vali</v>, of a field description may be any ACL2 object \u000A but must satisfy <v>typei</v>.  Note that <v>vali</v> is not a form to be \u000A evaluated but an object.  A form that evaluates to <v>vali</v> could be written \u000A <v>'vali</v>, but <v>defstobj</v> does not expect you to write the quote mark. \u000A For example, the field description</p> \u000A \u000A <code>(days-off :initially (saturday sunday))</code> \u000A \u000A <p>describes a field named <v>days-off</v> whose initial value is the list \u000A consisting of the two symbols <v>SATURDAY</v> and <v>SUNDAY</v>.  In particular, \u000A the initial value is NOT obtained by applying the function <v>saturday</v> to \u000A the variable <v>sunday</v>!  Had we written</p> \u000A \u000A <code>(days-off :initially '(saturday sunday))</code> \u000A \u000A <p>it would be equivalent to writing</p> \u000A \u000A <code>(days-off :initially (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (saturday sunday)))</code> \u000A \u000A <p>which would initialize the field to a list of length two, whose first \u000A element is the symbol <v>quote</v> and whose second element is a list containing \u000A the symbols <v>saturday</v> and <v>sunday</v>.</p> \u000A \u000A <p>Array Types</p> \u000A \u000A <p>When <v>typei</v> is of the form <v>(ARRAY type-spec (<see topic=\"COMMON-LISP____MAX\">max</see>))</v>, the field is \u000A supposed to be a list of items, initially of length specified by <v>max</v>, \u000A each of which satisfies the indicated <v>type-spec</v>.  <v>Max</v> must be a \u000A non-negative integer or a defined constant evaluating to a non-negative \u000A integer. Thus, each of</p> \u000A \u000A <code>(ARRAY (SIGNED-BYTE 31) (16))\u000A(ARRAY (SIGNED-BYTE 31) (*c*)) ; given previous event (<see topic=\"ACL2____DEFCONST\">defconst</see> *c* 16)</code> \u000A \u000A <p>restricts the field to be a list of integers, initially of length 16, where \u000A each integer in the list is a <v>(SIGNED-BYTE 31)</v>.  We sometimes call such a \u000A list an ``array'' (because it is represented as an array in the underlying \u000A Common Lisp).  The elements of an array field are indexed by position, \u000A starting at 0.  Thus, the maximum legal index of an array field one less than \u000A is specified by <v>max</v>.  Note that the value of <v>max</v> must be less than \u000A the Common Lisp constant <v>array-dimension-limit</v>, and also (though this \u000A presumably follows) less than the Common Lisp constant \u000A <v>array-total-size-limit</v>.</p> \u000A \u000A <p>Note also that the <v>ARRAY</v> type requires that the <v>max</v> be enclosed \u000A in parentheses.  This makes ACL2's notation consistent with the Common Lisp \u000A convention of describing the (multi-)dimensionality of arrays.  But ACL2 \u000A currently supports only single dimensional arrays in stobjs.</p> \u000A \u000A <p>For array fields, the initial value <v>vali</v> must be an object satisfying \u000A the <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt> of the <v>ARRAY</v> description.  The initial value of the \u000A field is a list of <v>max</v> repetitions of <v>vali</v>.</p> \u000A \u000A <p>Array fields can be ``resized,'' that is, their lengths can be changed, if \u000A <v>:resizable t</v> is supplied as shown in the example and General Form above. \u000A The new length must satisfy the same restriction as does <v>max</v>, as \u000A described above.  Each array field in a <v>defstobj</v> event gives rise to a \u000A length function, which gives the length of the field, and a resize function, \u000A which modifies the length of the field if <v>:resizable t</v> was supplied with \u000A the field when the <v>defstobj</v> was introduced and otherwise causes an error. \u000A If <v>:resizable t</v> was supplied and the resize function specifies a new \u000A length <v>k</v>, then: if <v>k</v> is less than the existing array length, the \u000A array is shortened simply by dropping elements with index at least <v>k</v>; \u000A otherwise, the array is extended to length <v>k</v> by mapping the new indices \u000A to the initial value (supplied by <v>:initially</v>, else default <v>nil</v>).</p> \u000A \u000A <p>Array resizing is relatively slow, so we recommend using it somewhat \u000A sparingly.</p> \u000A \u000A <p><i>The Default Function Names</i></p> \u000A \u000A <p>To recap, in</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> name\u000A          (field1 :type type1 :initially val1)\u000A          ...\u000A          (fieldk :type typek :initially valk)\u000A          :renaming alist\u000A          :doc doc-string\u000A          :inline inline-flag)</code> \u000A \u000A <p><v>name</v> must be a new symbol, each <v>fieldi</v> must be a symbol, each \u000A <v>typei</v> must be a <tt><see topic=\"ACL2____TYPE-SPEC\">type-spec</see></tt> or <v>(ARRAY type-spec (<see topic=\"COMMON-LISP____MAX\">max</see>))</v>, and \u000A each <v>vali</v> must be an object satisfying <v>typei</v>.</p> \u000A \u000A <p>Roughly speaking, for each <v>fieldi</v>, a <v>defstobj</v> introduces a \u000A recognizer function, an accessor function, and an updater function.  The \u000A accessor function, for example, takes the stobj and returns the indicated \u000A component; the updater takes a new component value and the stobj and return a \u000A new stobj with the component replaced by the new value.  But that summary is \u000A inaccurate for array fields.</p> \u000A \u000A <p>The accessor function for an array field does not take the stobj and return \u000A the indicated component array, which is a list of length specified by \u000A <v>max</v>.  Instead, it takes an additional index argument and returns the \u000A indicated element of the array component.  Similarly, the updater function for \u000A an array field takes an index, a new value, and the stobj, and returns a new \u000A stobj with the indicated element replaced by the new value.</p> \u000A \u000A <p>These functions &mdash; the recognizer, accessor, and updater, and also \u000A length and resize functions in the case of array fields &mdash; have ``default \u000A names.''  The default names depend on the field name, <v>fieldi</v>, and on \u000A whether the field is an array field or not.  For clarity, suppose <v>fieldi</v> \u000A is named <v>c</v>. The default names are shown below in calls, which also \u000A indicate the arities of the functions.  In the expressions, we use <v>x</v> as \u000A the object to be recognized by field recognizers, <v>i</v> as an array index, \u000A <v>v</v> as the ``new value'' to be installed by an updater, and <v>name</v> as \u000A the single-threaded object.</p> \u000A \u000A <code>                 non-array field        array field\u000Arecognizer         (cP x)                (cP x)\u000Aaccessor           (c name)              (cI i name)\u000Aupdater            (UPDATE-c v name)     (UPDATE-cI i v name)\u000Alength                                   (c-LENGTH name)\u000Aresize                                   (RESIZE-c k name)</code> \u000A \u000A <p>Finally, a recognizer and a creator for the entire single-threaded object \u000A are introduced.  The creator returns the initial stobj, but may only be used \u000A in limited contexts; see <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>.  If the single-threaded \u000A object is named <v>name</v>, then the default names and arities are as shown \u000A below.</p> \u000A \u000A <code>top recognizer     (nameP x)\u000Acreator            (CREATE-name)</code> \u000A \u000A <p>For example, the event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">DEFSTOBJ</see> $S\u000A  (X :TYPE INTEGER :INITIALLY 0)\u000A  (A :TYPE (ARRAY (INTEGER 0 9) (3)) :INITIALLY 9))</code> \u000A \u000A <p>introduces a stobj named <v>$S</v>.  The stobj has two fields, <v>X</v> and \u000A <v>A</v>.  The <v>A</v> field is an array.  The <v>X</v> field contains an integer \u000A and is initially 0.  The <v>A</v> field contains a list of integers, each \u000A between 0 and 9, inclusively.  Initially, each of the three elements of the \u000A <v>A</v> field is 9.</p> \u000A \u000A <p>This event introduces the following sequence of definitions:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> XP (X) ...)               ; recognizer for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AP (X) ...)               ; recognizer of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> $SP ($S) ...)             ; top-level recognizer for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> CREATE-$S () ...)         ; creator for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> X ($S) ...)               ; accessor for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-X (V $S) ...)      ; updater for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A-LENGTH ($S) ...)        ; length of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> RESIZE-A (K $S) ...)      ; resizer for A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AI (I $S) ...)            ; accessor for A field at index I\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-AI (I V $S) ...)   ; updater for A field at index I</code> \u000A \u000A <p><i>Avoiding the Default Function Names</i></p> \u000A \u000A <p>If you do not like the default names listed above you may use the optional \u000A <v>:renaming</v> alist to substitute names of your own choosing.  Each element \u000A of <v>alist</v> should be of the form <v>(fn1 fn2)</v>, where <v>fn1</v> is a \u000A default name and <v>fn2</v> is your choice for that name.</p> \u000A \u000A <p>For example</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">DEFSTOBJ</see> $S\u000A  (X :TYPE INTEGER :INITIALLY 0)\u000A  (A :TYPE (ARRAY (INTEGER 0 9) (3)) :INITIALLY 9)\u000A  :renaming ((X XACCESSOR) (CREATE-$S MAKE$S)))</code> \u000A \u000A <p>introduces the following definitions</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> XP (X) ...)               ; recognizer for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AP (X) ...)               ; recognizer of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> $SP ($S) ...)             ; top-level recognizer for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> MAKE$S () ...)            ; creator for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> XACCESSOR ($S) ...)       ; accessor for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-X (V $S) ...)      ; updater for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A-LENGTH ($S) ...)        ; length of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> RESIZE-A (K $S) ...)      ; resizer for A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AI (I $S) ...)            ; accessor for A field at index I\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-AI (I V $S) ...)   ; updater for A field at index I</code> \u000A \u000A <p>Note that even though the renaming alist substitutes ``<v>XACCESSOR</v>'' for \u000A ``<v>X</v>'' the updater for the <v>X</v> field is still called ``<v>UPDATE-X</v>.'' \u000A That is because the renaming is applied to the default function names, not to \u000A the field descriptors in the event.</p> \u000A \u000A <p>Use of the <v>:renaming</v> alist may be necessary to avoid name clashes \u000A between the default names and and pre-existing function symbols.</p> \u000A \u000A <p><i>Constants</i></p> \u000A \u000A <p><v>Defstobj</v> events also introduce constant definitions (see <see topic=\"ACL2____DEFCONST\">defconst</see>).  One constant is introduced for each accessor function by \u000A prefixing and suffixing a `<v>*</v>' character on the function name.  The value \u000A of that constant is the position of the field being accessed.  For example, if \u000A the accessor functions are <v>a</v>, <v>b</v>, and <v>c</v>, in that order, then the \u000A following constant definitions are introduced.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* 0)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *b* 1)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *c* 2)</code> \u000A \u000A <p>These constants are used for certain calls of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> and <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt> that are displayed to the user in proof output.  For example, for \u000A stobj <v>st</v> with accessor functions <v>a</v>, <v>b</v>, and <v>c</v>, in that \u000A order, the term <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> '2 st)</v> would be printed during a proof as <v>(<see topic=\"COMMON-LISP____NTH\">nth</see>\u000A *c* st)</v>.  Also see <see topic=\"ACL2____TERM\">term</see>, in particular the discussion there of \u000A untranslated terms, and see <see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see>.</p> \u000A \u000A <p><i>Inspecting the Effects of a Defstobj</i></p> \u000A \u000A <p>Because the stobj functions are introduced as ``sub-events'' of the \u000A <v>defstobj</v> the history commands <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> and <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt> will \u000A not print the definitions of these functions but will print the superior \u000A <v>defstobj</v> event.  To see the definitions of these functions use the \u000A history command <v>:</v><tt><see topic=\"ACL2____PCB_12\">pcb!</see></tt>.</p> \u000A \u000A <p>To see an s-expression containing the definitions what constitute the raw \u000A Lisp implementation of the event, evaluate the form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NTH\">nth</see> 4 (global-val 'cltl-command (w state)))</code> \u000A \u000A <p><i>immediately after</i> the <v>defstobj</v> event has been processed.</p> \u000A \u000A <p>A <v>defstobj</v> is considered redundant only if the name, field \u000A descriptors, renaming alist, and inline flag are identical to a previously \u000A executed <v>defstobj</v>.  Note that a redundant <v>defstobj</v> does not reset \u000A the <see topic=\"ACL2____STOBJ\">stobj</see> fields to their initial values.</p> \u000A \u000A <p><i>Inlining and Performance</i></p> \u000A \u000A <p>The <v>:inline</v> keyword argument controls whether or not accessor, \u000A updater, and length functions are inlined (as macros under the hood, in raw \u000A Lisp).  If <v>:inline t</v> is provided then these are inlined; otherwise they \u000A are not.  The advantage of inlining is potentially better performance; there \u000A have been contrived examples, doing essentially nothing except accessing and \u000A updating array fields, where inlining reduced the time by a factor of 10 or \u000A more; and inlining has sped up realistic examples by a factor of at least 2. \u000A Inlining may get within a factor of 2 of C execution times for such contrived \u000A examples, and within a few percent of C execution times on realistic \u000A examples.</p> \u000A \u000A <p>A drawback to inlining is that redefinition may not work as expected, much \u000A as redefinition may not work as expected for macros: defined functions that \u000A call a macro, or inlined stobj function, will not see a subsequent \u000A redefinition of the macro or inlined function.  Another drawback to inlining \u000A is that because inlined functions are implemented as macros in raw Lisp, \u000A tracing (see <see topic=\"ACL2____TRACE_42\">trace$</see>) will not show their calls.  These drawbacks are \u000A avoided by default, but the user who is not concerned about them is advised to \u000A specify <v>:inline t</v>.</p> \u000A \u000A <p><i>Specifying Congruent Stobjs</i></p> \u000A \u000A <p>Two stobjs are may be considered to be ``congruent'' if they have the same \u000A structure, that is, their <v>defstobj</v> events are identical when ignoring \u000A field names.  In particular, every stobj is congruent to itself.  In order to \u000A tell ACL2 that a new stobj <v>st2</v> is indeed to be considered as congruent to \u000A an existing stobj <v>st1</v>, the <v>defstobj</v> event introducing <v>st2</v> is \u000A given the keyword argument <v>:congruent-to st1</v>.  Congruence is an \u000A equivalence relation: when you specify a new stobj to be congruent to an old \u000A one, you are also specifying that the new stobj is congruent to all other \u000A stobjs that are congruent to the old one.  Thus, continuing the example above, \u000A if you specify that <v>st3</v> is <v>:congruent-to st2</v>, then <v>st1</v>, \u000A <v>st2</v>, and <v>st3</v> will all be congruent to each other.</p> \u000A \u000A <p>When two stobjs are congruent, ACL2 allows you to substitute one for \u000A another in a function call.  Any number of stobjs may be replaced with \u000A congruent stobjs in the call, provided no two get replaced with the same \u000A stobj.  The return values are correspondingly modified: if stobj <v>st1</v> is \u000A replaced by <v>st2</v> at an argument position, and if <v>st1</v> is returned in \u000A the output <see topic=\"ACL2____SIGNATURE\">signature</see> of the function, then <v>st2</v> is returned in \u000A place of <v>st1</v>.</p> \u000A \u000A <p>The following example illustrates congruent stobjs.  For more examples of \u000A how to take advantage of congruent stobjs, and also of how to misuse them, see \u000A community book <v>books/misc/congruent-stobjs-test.lisp</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st1 fld1)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st2 fld2 :congruent-to st1)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st3 fld3 :congruent-to st2) ; equivalently, :congruent-to st1\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (st1 st2 st3)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (st1 st2 st3)))\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (fld2 st1) (fld3 st2) (fld1 st3)))\u000A(update-fld1 1 st1)\u000A(update-fld1 2 st2) ; notice use of update-fld1 on st2\u000A(update-fld1 3 st3) ; notice use of update-fld1 on st3\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f st3 st2 st1) '(3 2 1)))</code> \u000A \u000A <p>The following example shows an error that occurs when stobj arguments are \u000A repeated, i.e., at least two stobj arguments (in this case, three) get \u000A replaced by the same stobj.</p> \u000A \u000A <code>ACL2 !&gt;(f st1 st1 st1)\u000A\u000AACL2 Error in TOP-LEVEL:  The form ST1 is being used, as an argument\u000Ato a call of F, where the single-threaded object ST2 was expected,\u000Aeven though these are congruent stobjs.  See :DOC defstobj, in particular\u000Athe discussion of congruent stobjs.  Note:  this error occurred in\u000Athe context (F ST1 ST1 ST1).\u000A\u000AACL2 !&gt;</code>"],

 "ACL2____ACL2-DEFAULTS-TABLE":[["Other"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode) ; current default defun-mode\u000A(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program)\u000A           ; set default defun-mode to :program</code> \u000A \u000A <p>See <see topic=\"ACL2____TABLE\">table</see> for a discussion of tables in general.  The legal keys for \u000A this <see topic=\"ACL2____TABLE\">table</see> are shown below.  They may be accessed and changed via the \u000A general mechanisms provided by <see topic=\"ACL2____TABLE\">table</see>s.  However, there are often more \u000A convenient ways to access and/or change the defaults.  (See also the note \u000A below.)</p> \u000A \u000A <code>:defun-mode</code> \u000A \u000A <p>the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, which must be <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> for a general discussion of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s.  The <v>:</v><tt><see topic=\"ACL2____DEFUN-MODE\">defun-mode</see></tt> key may be conveniently set by \u000A keyword commands naming the new <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  See <see topic=\"ACL2____PROGRAM\">program</see> and see <see topic=\"ACL2____LOGIC\">logic</see>.</p> \u000A \u000A <code>:enforce-redundancy</code> \u000A \u000A <p>if <v>t</v>, cause ACL2 to insist that most events are redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>); if <v>:warn</v>, cause a warning instead of an error for \u000A such non-redundant events; else, <v>nil</v>.  See <see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see>.</p> \u000A \u000A <code>:ignore-doc-string-error</code> \u000A \u000A <p>if <v>t</v>, cause ACL2 to ignore ill-formed <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings \u000A rather than causing an error; if <v>:warn</v>, cause a warning instead of an \u000A error in such cases; else, <v>nil</v> (the default).  See <see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see>.</p> \u000A \u000A <code>:verify-guards-eagerness</code> \u000A \u000A <p>an integer between 0 and 2 indicating how eager the system is to verify the \u000A <see topic=\"ACL2____GUARD\">guard</see>s of a <see topic=\"COMMON-LISP____DEFUN\">defun</see> event.  See <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see>.</p> \u000A \u000A <code>:compile-fns</code> \u000A \u000A <p>When this key's value is <v>t</v>, functions are compiled when they are \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d; otherwise, the value is <v>nil</v>.  (Except, this key's value \u000A is ignored when explicit compilation is suppressed; see <see topic=\"ACL2____COMPILATION\">compilation</see>.) \u000A To set the flag, see <see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see>.</p> \u000A \u000A <code>:measure-function</code> \u000A \u000A <p>the default measure function used by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> when no <v>:measure</v> is \u000A supplied in <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>.  The default measure function must be a function \u000A symbol of one argument. Let <v>mfn</v> be the default measure function and \u000A suppose no <v>:measure</v> is supplied with some recursive function definition. \u000A Then <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> finds the first formal, <v>var</v>, that is tested along \u000A every branch and changed in each recursive call.  The system then ``guesses'' \u000A that <v>(mfn var)</v> is the <v>:measure</v> for that <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.</p> \u000A \u000A <code>:well-founded-relation</code> \u000A \u000A <p>the default well-founded relation used by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> when no \u000A <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> is supplied in <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>.  The default \u000A well-founded relation must be a function symbol, <v>rel</v>, of two arguments \u000A about which a <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> rule has been proved.  See \u000A <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.</p> \u000A \u000A <code>:bogus-defun-hints-ok</code> \u000A \u000A <p>When this key's value is <v>t</v>, ACL2 allows <v>:hints</v> for nonrecursive \u000A function definitions.  Otherwise, the value is the <v>nil</v> (the default) or \u000A <v>:warn</v> (which makes the check but merely warns when the check fails).  See \u000A <see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see>.</p> \u000A \u000A <code>:bogus-mutual-recursion-ok</code> \u000A \u000A <p>When this key's value is <v>t</v>, ACL2 skips the check that every function \u000A in a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> (or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>) ``clique'' calls at least \u000A one other function in that ``clique.''  Otherwise, the value is <v>nil</v> (the \u000A default) or <v>:warn</v> (which makes the check but merely warns when the check \u000A fails).  See <see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see>.</p> \u000A \u000A <code>:irrelevant-formals-ok</code> \u000A \u000A <p>When this key's value is <v>t</v>, the check for irrelevant formals is \u000A bypassed; otherwise, the value is the keyword <v>nil</v> (the default) or \u000A <v>:warn</v> (which makes the check but merely warns when the check fails).  See \u000A <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see> and see <see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see>.</p> \u000A \u000A <code>:ignore-ok</code> \u000A \u000A <p>When this key's value is <v>t</v>, the check for ignored variables is \u000A bypassed; otherwise, the value is the keyword <v>nil</v> (the default) or \u000A <v>:warn</v> (which makes the check but merely warns when the check fails).  See \u000A <see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see>.</p> \u000A \u000A <code>:bdd-constructors</code> \u000A \u000A <p>This key's value is a list of function symbols used to define the notion of \u000A ``BDD normal form.''  See <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see> and see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <code>:ttag</code> \u000A \u000A <p>This key's value, when non-<v>nil</v>, allows certain operations that extend \u000A the trusted code base beyond what is provided by ACL2.  See <see topic=\"ACL2____DEFTTAG\">defttag</see>. \u000A See <see topic=\"ACL2____DEFTTAG\">defttag</see>.</p> \u000A \u000A <code>:state-ok</code> \u000A \u000A <p>This key's value is either <v>t</v> or <v>nil</v> and indicates whether the \u000A user is aware of the syntactic restrictions on the variable symbol <v>STATE</v>. \u000A See <see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see>.</p> \u000A \u000A <code>:backchain-limit</code> \u000A \u000A <p>This key's value is a list of two ``numbers.''  Either ``number'' may \u000A optionally be <v>nil</v>, which is treated like positive infinity.  The numbers \u000A control backchaining through hypotheses during type-set reasoning and \u000A rewriting.  See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p> \u000A \u000A <code>:default-backchain-limit</code> \u000A \u000A <p>This key's value is a list of two ``numbers.''  Either ``number'' may \u000A optionally be <v>nil</v>, which is treated like positive infinity.  The numbers \u000A are used respectively to set the backchain limit of a rule if one has not been \u000A specified. See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p> \u000A \u000A <code>:step-limit</code> \u000A \u000A <p>This key's value is either <v>nil</v> or a natural number not exceeding the \u000A value of <v>*default-step-limit*</v>.  If the value is <v>nil</v> or the value of \u000A <v>*default-step-limit*</v>, there is no limit on the number of ``steps'' that \u000A ACL2 counts during a proof: currently, the number of top-level rewriting \u000A calls.  Otherwise, the value is the maximum number of such calls allowed \u000A during evaluation of any event.  See <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see>.</p> \u000A \u000A <code>:rewrite-stack-limit</code> \u000A \u000A <p>This key's value is a nonnegative integer less than <v>(<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 28)</v>.  It \u000A is used to limit the depth of calls of ACL2 rewriter functions.  See <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>.</p> \u000A \u000A <code>:let*-abstractionp</code> \u000A \u000A <p>This key affects how the system displays subgoals.  The value is either \u000A <v>t</v> or <v>nil</v>.  When t, let* expressions are introduced before printing \u000A to eliminate common subexpressions.  The actual goal being worked on is \u000A unchanged.</p> \u000A \u000A <code>:nu-rewriter-mode</code> \u000A \u000A <p>This key's value is <v>nil</v>, <v>t</v>, or <v>:literals</v>.  When the value is \u000A non-<v>nil</v>, the rewriter gives special treatment to expressions and \u000A functions defined in terms of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> and <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt>.  See <tt><see topic=\"ACL2____SET-NU-REWRITER-MODE\">set-nu-rewriter-mode</see></tt>.</p> \u000A \u000A <code>:case-split-limitations</code> \u000A \u000A <p>This key's value is a list of two ``numbers.''  Either ``number'' may \u000A optionally be <v>nil</v>, which is treated like positive infinity.  The numbers \u000A control how the system handles case splits in the simplifier.  See <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>.</p> \u000A \u000A <code>:include-book-dir-alist</code> \u000A \u000A <p>This key's value is used by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>'s <v>:DIR</v> argument to \u000A associate a directory with a keyword.  An exception is the keyword \u000A <v>:SYSTEM</v> for the <v>books/</v> directory; see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, in \u000A particular the section on ``Books Directory.''  Also see <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> and <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>.</p> \u000A \u000A <code>:match-free-default</code> \u000A \u000A <p>This key's value is either <v>:all</v>, <v>:once</v>, or <v>nil</v>.  See <see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see>.</p> \u000A \u000A <code>:match-free-override</code> \u000A \u000A <p>This key's value is a list of runes.  See <see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see>.</p> \u000A \u000A <code>:match-free-override-nume</code> \u000A \u000A <p>This key's value is an integer used in the implementation of <see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see>, so that only existing runes are affected by that \u000A event.</p> \u000A \u000A <code>:non-linearp</code> \u000A \u000A <p>This key's value is either <v>t</v> or <v>nil</v> and indicates whether the \u000A user wishes ACL2 to extend the linear arithmetic decision procedure to include \u000A non-linear reasoning.  See <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>.</p> \u000A \u000A <code>:tau-auto-modep</code> \u000A \u000A <p>This key's value is either <v>t</v> or <v>nil</v> and indicates whether the \u000A user wishes ACL2 to look for opportunities to create <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> \u000A rules from all suitable <v>defun</v>s and from all suitable <v>defthm</v>s (with \u000A non-<v>nil</v> <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>).  See <see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see>.</p> \u000A \u000A <code>:ruler-extenders</code> \u000A \u000A <p>This key's value may be a list of symbols, indicating those function \u000A symbols that are not to block the collection of rulers; see <see topic=\"COMMON-LISP____DEFUN\">defun</see>. \u000A Otherwise the value is <v>:all</v> to indicate all function symbols, i.e., so \u000A that no function symbol blocks the collection of rulers.  If a list is \u000A specified (rather than <v>:all</v>), then it may contain the keyword \u000A <v>:lambdas</v>, which has the special role of specifying all <v>lambda</v> \u000A applications.  No other keyword is permitted in the list.  See <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.</p> \u000A \u000A <code>:memoize-ideal-okp</code> \u000A \u000A <p>This key is only legal in an experimental <tt><see topic=\"ACL2____HONS\">hons</see></tt> version (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).  Its value must be either <v>t</v>, <v>nil</v>, or \u000A <v>:warn</v>.  If the value is <v>nil</v> or not present, then it is illegal by \u000A default to <see topic=\"ACL2____MEMOIZE\">memoize</see> a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function that has not \u000A been <see topic=\"ACL2____GUARD\">guard</see>-verified (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>), sometimes called an \u000A ``ideal-mode'' function.  This illegality is the default because such calls of \u000A such functions in the ACL2 loop are generally evaluated in the logic (using \u000A so-called ``executable counterpart'' definitions), rather than directly by \u000A executing calls of the corresponding (memoized) raw Lisp function.  However, \u000A such a raw Lisp call can be made when the function is called by a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function, so we allow you to override the default behavior by \u000A associating the value <v>t</v> or <v>:warn</v> with the key \u000A <v>:memoize-ideal-okp</v>, where with <v>:warn</v> you get a suitable warning. \u000A Note that you can also allow memoization of ideal-mode functions by supplying \u000A argument <v>:ideal-okp</v> to your memoization event (see <see topic=\"ACL2____MEMOIZE\">memoize</see>), in \u000A which case the value of <v>:memoize-ideal-okp</v> in the \u000A <v>acl2-defaults-table</v> is irrelevant.</p> \u000A \u000A <p>Note: Unlike all other <see topic=\"ACL2____TABLE\">table</see>s, <v>acl2-defaults-table</v> can affect \u000A the soundness of the system.  The <see topic=\"ACL2____TABLE\">table</see> mechanism therefore enforces on \u000A it a restriction not imposed on other <see topic=\"ACL2____TABLE\">table</see>s: when <tt><see topic=\"ACL2____TABLE\">table</see></tt> is \u000A used to update the <v>acl2-defaults-table</v>, the key and value must be \u000A variable-free forms.  Thus, while</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program),\u000A\u000A(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode ':program), and\u000A\u000A(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode (compute-mode *my-data*))</code> \u000A \u000A <p>are all examples of legal <see topic=\"ACL2____EVENTS\">events</see> (assuming <v>compute-mode</v> is a \u000A function of one non-<tt><see topic=\"ACL2____STATE\">state</see></tt> argument that produces a <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> \u000A as its single value),</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode (compute-mode (w state)))</code> \u000A \u000A <p>is not legal because the value form is <tt><see topic=\"ACL2____STATE\">state</see></tt>-sensitive.</p> \u000A \u000A <p>Consider for example the following three <see topic=\"ACL2____EVENTS\">events</see> which one might make \u000A into the text of a book.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table\u000A  :defun-mode\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state) :logic :program))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> crash-and-burn (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))</code> \u000A \u000A <p>The second event is illegal because its value form is <tt><see topic=\"ACL2____STATE\">state</see></tt>-sensitive.  If it were not illegal, then it would set the <v>:</v><tt><see topic=\"ACL2____DEFUN-MODE\">defun-mode</see></tt> to <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> when the book was being certified but \u000A would set the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> when the book was being \u000A loaded by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  That is because during certification, <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>nil</v> (proof obligations are generated and proved), but \u000A during book inclusion <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is non-<v>nil</v> (those \u000A obligations are assumed to have been satisfied.)  Thus, the above book, when \u000A loaded, would create a function in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode that does not \u000A actually meet the conditions for such status.</p> \u000A \u000A <p>For similar reasons, <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see> affecting \u000A <v>acl2-defaults-table</v> are illegal within the scope of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> forms. \u000A That is, the text</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> crash-and-burn (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))</code> \u000A \u000A <p>is illegal because <v>acl2-defaults-table</v> is changed locally.  If this \u000A text were acceptable as a book, then when the book was certified, \u000A <v>crash-and-burn</v> would be processed in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, but when \u000A the certified book was included later, <v>crash-and-burn</v> would have \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode because the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> event would be skipped.</p> \u000A \u000A <p>The text</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____PROGRAM\">program</see>) ;which is (<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> crash-and-burn (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))</code> \u000A \u000A <p>is acceptable and defines <v>crash-and-burn</v> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode, both during certification and subsequent inclusion.</p> \u000A \u000A <p>We conclude with an important observation about the relation between \u000A <v>acl2-defaults-table</v> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, and \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Including or certifying a book never has an effect on \u000A the <v>acl2-defaults-table</v>, nor does executing an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event; \u000A we always restore the value of this <see topic=\"ACL2____TABLE\">table</see> as a final act.  (Also see \u000A <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>, and see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>.) \u000A That is, no matter how a book fiddles with the <v>acl2-defaults-table</v>, its \u000A value immediately after including that book is the same as immediately before \u000A including that book.  If you want to set the <v>acl2-defaults-table</v> in a way \u000A that persists, you need to do so using <see topic=\"ACL2____COMMAND\">command</see>s that are not inside \u000A <see topic=\"ACL2____BOOKS\">books</see>.  It may be useful to set your favorite defaults in your <tt><see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see></tt> file; see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p>"],

 "ACL2____VERIFY-GUARDS":[["Events","Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____GUARD\">guard</see> for a general discussion of guards.</p> \u000A \u000A <p>Before discussing the <v>verify-guards</v> event, we first discuss guard \u000A verification, which can take place at definition time or, later, using \u000A <v>verify-guards</v>.  Typically, guard verification takes place at definition \u000A time if a guard (or type, or <see topic=\"ACL2____STOBJS\">stobjs</see>) has been supplied explicitly \u000A unless <v>:verify-guards nil</v> has been specified; see <see topic=\"COMMON-LISP____DEFUN\">defun</see> and see \u000A <see topic=\"ACL2____XARGS\">xargs</see>, and see <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> for how to change \u000A this default.  The point of guard verification is to ensure that during \u000A evaluation of an expression without free variables, no guard violation takes \u000A place.</p> \u000A \u000A <p>Technical note: the first argument of <v>verify-guards</v> must be a function \u000A symbol or the name of a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> or <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> event, not a \u000A macro-alias for a function symbol (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).  See <see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see> for a utility that does not have this restriction.</p> \u000A \u000A <p>Guard verification is intended to guarantee that for any call of a given \u000A function, if its <see topic=\"ACL2____GUARD\">guard</see> holds for that call then the <see topic=\"ACL2____GUARD\">guard</see> will \u000A hold for every function call in the body of that function.  Moreover, in order \u000A to avoid guard violations during evaluation of the function's guard itself, \u000A guard verification also is intended to guarantee that the guards are satisfied \u000A for all calls in the guard itself.  Consider the following simple example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A    17))</code> \u000A \u000A <p>If you evaluate <v>(f t)</v>, for example, in the top-level loop, you will \u000A (by default) get a guard error.  The point of guard verification is to \u000A guarantee the absence of guard errors, and we start by using this example to \u000A illustrate the proof obligations that guarantee such absence.</p> \u000A \u000A <p>The body of the above definition has the following function calls, where \u000A the first is the entire body.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A  17)\u000A(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) ; the test of the top-level IF call\u000A(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)             ; from (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) ; the true branch of the top-level IF call\u000A(<see topic=\"COMMON-LISP____CAR\">car</see> x)             ; from (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)             ; from (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</code> \u000A \u000A <p>We thus see potentially six conditions to prove, one for each call.  The \u000A guards of the function symbols of those calls are <v>t</v> for <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt>, <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</v> for both <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> and \u000A <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>, and finally that both arguments are rationals for <v>&lt;</v>. \u000A Moreover, we can take advantage of ``contextual assumptions'': the \u000A <v>if</v>-test conditions and the top-level <v>:guard</v>.  Thus, for \u000A <v>verify-guards</v> the proof obligation from the body of <v>f</v> is as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see>\u000A (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))) ; from the :guard\u000A (<see topic=\"COMMON-LISP____AND\">and</see> t ; from the top-level IF call\u000A      t ; from (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A      (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)) ; from the first (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A      (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A       (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) ; IF-test for calls in the true branch\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)) ; from (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A            (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)) ; from the second (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A            (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))) ; from the &lt; call\u000A            ))))</code> \u000A \u000A <p>But the <v>:guard</v> itself generates a similar sort of proof obligation. \u000A Note that the guard <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</v> is really an \u000A abbreviation (i.e. via the macro <tt><see topic=\"COMMON-LISP____AND\">and</see></tt>) for the term <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) nil)</v>.  The guard proof obligation for the guard itself is \u000A thus as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> t ; from (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A              (<see topic=\"COMMON-LISP____AND\">and</see> t         ; from (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) ;\u000A                   (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) ; from (<see topic=\"COMMON-LISP____CAR\">car</see> x) ;\u000A                   )))</code> \u000A \u000A <p>All of the above proof obligations are indeed theorems, and guard \u000A verification succeeds for the above definition of <v>f</v>.</p> \u000A \u000A <p>The example above illustrates the general procedure for generating the \u000A guard proof obligation.  Each function call is considered in the body or guard \u000A of the function, and it is required that the guard is met for that call, under \u000A certain ``contextual assumptions'', which are as follows.  In the case of the \u000A body of the named function, it is assumed that the guard holds for that \u000A function on its formal parameters.  And in both cases &mdash; the body of the \u000A named function and also its guard &mdash; the governing tests from superior \u000A calls of <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> are also assumed.</p> \u000A \u000A <p>As mentioned above, if the guard on a function is not <v>t</v>, then guard \u000A verification requires not only consideration of the body under the assumption \u000A that the guard is true, but also consideration of the guard itself.  Thus, for \u000A example, guard verification fails in the following example, even though there \u000A are no proof obligations arising from the body, because the guard itself can \u000A cause a guard violation when evaluated for an arbitrary value of <v>x</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A  x)</code> \u000A \u000A <p>We turn now to the <v>verify-guards</v> event as a way of verifying the <see topic=\"ACL2____GUARD\">guard</see>s for a function or theorem.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> flatten)\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> flatten\u000A               :hints ((&quot;Goal&quot; :use (:instance assoc-of-app)))\u000A               :otf-flg t\u000A               :guard-debug t ; default = nil\u000A               :doc &quot;string&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> name\u000A        :hints        hints\u000A        :otf-flg      otf-flg\u000A        :guard-debug  t ; typically t, but any value is legal\u000A        :doc          doc-string)</code> \u000A \u000A <p>In the General Form above, <v>name</v> is the name of a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A function (see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>) or of a theorem or axiom.  In the most common \u000A case <v>name</v> is the name of a function that has not yet had its <see topic=\"ACL2____GUARD\">guard</see>s verified, each subroutine of which has had its <see topic=\"ACL2____GUARD\">guard</see>s verified. \u000A The values <tt><see topic=\"ACL2____HINTS\">hints</see></tt>, <tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt>, and <tt><see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see></tt> are as \u000A described in the corresponding <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> entries; and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt>, if supplied, is a string <b>not</b> beginning with \u000A ``<v>:Doc-Section</v>''.  The four keyword arguments above are all optional.  To \u000A admit this event, the conjunction of the guard proof obligations must be \u000A proved.  If that proof is successful, <v>name</v> is considered to have had its \u000A <see topic=\"ACL2____GUARD\">guard</see>s verified.</p> \u000A \u000A <p>See <see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> for a utility that lets you view the \u000A formula to be proved by <v>verify-guards</v>, but without creating an event.</p> \u000A \u000A <p>If <v>name</v> is one of several functions in a mutually recursive clique, \u000A <v>verify-guards</v> will attempt to verify the <see topic=\"ACL2____GUARD\">guard</see>s of all of the \u000A functions.</p> \u000A \u000A <p>If <v>name</v> is a theorem or axiom name, <v>verify-guards</v> verifies the \u000A guards of the associated formula.  When a theorem has had its guards verified \u000A then you know that the theorem will evaluate to non-<v>nil</v> in all Common \u000A Lisps, without causing a runtime error (other than possibly a resource error). \u000A In particular, you know that the theorem's validity does not depend upon \u000A ACL2's arbitrary completion of the domains of partial Common Lisp \u000A functions.</p> \u000A \u000A <p>For example, if <v>app</v> is defined as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      y\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))</code> \u000A \u000A <p>then we can verify the guards of <v>app</v> and we can prove the theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> assoc-of-app\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c) (app a (app b c))))</code> \u000A \u000A <p>However, if you go into almost any Common Lisp in which <v>app</v> is defined \u000A as shown and evaluate</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app 1 2) 3) (app 1 (app 2 3)))</code> \u000A \u000A <p>we get an error or, perhaps, something worse like <v>nil</v>!  How can this \u000A happen since the formula is an instance of a theorem?  It is supposed to be \u000A true!</p> \u000A \u000A <p>It happens because the theorem exploits the fact that ACL2 has completed \u000A the domains of the partially defined Common Lisp functions like <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> \u000A and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>, defining them to be <v>nil</v> on all non-conses.  The formula \u000A above violates the guards on <v>app</v>.  It is therefore ``unreasonable'' to \u000A expect it to be valid in Common Lisp.</p> \u000A \u000A <p>But the following formula is valid in Common Lisp:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> a)\u000A         (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b))\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c) (app a (app b c)))\u000A    t)</code> \u000A \u000A <p>That is, no matter what the values of <v>a</v>, <v>b</v> and <v>c</v> the formula \u000A above evaluates to <v>t</v> in all Common Lisps (unless the Lisp engine runs out \u000A of memory or stack computing it).  Furthermore the above formula is a \u000A theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> guarded-assoc-of-app\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> a)\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b))\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c) (app a (app b c)))\u000A      t))</code> \u000A \u000A <p>This formula, <v>guarded-assoc-of-app</v>, is very easy to prove from \u000A <v>assoc-of-app</v>.  So why prove it?  The interesting thing about \u000A <v>guarded-assoc-of-app</v> is that we can verify the guards of the formula. \u000A That is, <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> guarded-assoc-of-app)</v> succeeds.  Note that it has \u000A to prove that if <v>a</v> and <v>b</v> are true lists then so is <v>(app a b)</v> to \u000A establish that the guard on the outermost <v>app</v> on the left is satisfied. \u000A By verifying the guards of the theorem we know it will evaluate to true in all \u000A Common Lisps.  Put another way, we know that the validity of the formula does \u000A not depend on ACL2's completion of the partial functions or that the formula \u000A is ``well-typed.''</p> \u000A \u000A <p>One last complication: The careful reader might have thought we could state \u000A <v>guarded-assoc-of-app</v> as</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> a)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c)\u000A                (app a (app b c))))</code> \u000A \u000A <p>rather than using the <v>if</v> form of the theorem.  We cannot!  The reason \u000A is technical: <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> is defined as a function in ACL2.  When it is \u000A called, both arguments are evaluated and then the obvious truth table is \u000A checked.  That is, <v>implies</v> is not ``lazy.''  Hence, when we write the \u000A guarded theorem in the <v>implies</v> form we have to prove the guards on the \u000A conclusion without knowing that the hypothesis is true.  It would have been \u000A better had we defined <v>implies</v> as a macro that expanded to the <v>if</v> \u000A form, making it lazy.  But we did not and after we introduced guards we did \u000A not want to make such a basic change.</p> \u000A \u000A <p>Recall however that <v>verify-guards</v> is almost always used to verify the \u000A guards on a function definition rather than a theorem.  We now return to that \u000A discussion.</p> \u000A \u000A <p>Because <v>name</v> is not uniquely associated with the <v>verify-guards</v> \u000A event (it necessarily names a previously defined function) the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string, <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt>, is not stored in the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> database.  Thus, we actually prohibit <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> from \u000A having the form of an ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p><v>Verify-guards</v> must often be used when the value of a recursive call of \u000A a defined function is given as an argument to a subroutine that is <see topic=\"ACL2____GUARD\">guard</see>ed.  An example of such a situation is given below.  Suppose <v>app</v> \u000A (read ``append'') has a <see topic=\"ACL2____GUARD\">guard</see> requiring its first argument to be a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  Consider</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A        (t (app (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))))))</code> \u000A \u000A <p>Observe that the value of a recursive call of <v>rev</v> is being passed into \u000A a <see topic=\"ACL2____GUARD\">guard</see>ed subroutine, <v>app</v>.  In order to verify the <see topic=\"ACL2____GUARD\">guard</see>s \u000A of this definition we must show that <v>(rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> produces a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>, since that is what the <see topic=\"ACL2____GUARD\">guard</see> of <v>app</v> requires.  How do \u000A we know that <v>(rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>?  The most elegant \u000A argument is a two-step one, appealing to the following two lemmas: (1) When \u000A <v>x</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>, <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  (2) \u000A When <v>z</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>, <v>(rev z)</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>. \u000A But the second lemma is a generalized property of <v>rev</v>, the function we \u000A are defining.  This property could not be stated before <v>rev</v> is defined \u000A and so is not known to the theorem prover when <v>rev</v> is defined.</p> \u000A \u000A <p>Therefore, we might break the admission of <v>rev</v> into three steps: \u000A define <v>rev</v> without addressing its <see topic=\"ACL2____GUARD\">guard</see> verification, prove some \u000A general properties about <v>rev</v>, and then verify the <see topic=\"ACL2____GUARD\">guard</see>s.  This \u000A can be done as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                  :verify-guards nil))    ; Note this additional xarg.\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A        (t (app (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-rev\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x2)\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (rev x2))))\u000A\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> rev)</code> \u000A \u000A <p>The ACL2 system can actually admit the original definition of <v>rev</v>, \u000A verifying the <see topic=\"ACL2____GUARD\">guard</see>s as part of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event.  The reason is \u000A that, in this particular case, the system's heuristics just happen to hit upon \u000A the lemma <v>true-listp-rev</v>.  But in many more complicated functions it is \u000A necessary for the user to formulate the inductively provable properties before \u000A <see topic=\"ACL2____GUARD\">guard</see> verification is attempted.</p> \u000A \u000A <p><b>Remark on computation of guard conjectures and evaluation</b>.  When \u000A ACL2 computes the <see topic=\"ACL2____GUARD\">guard</see> conjecture for the body of a function, it \u000A evaluates any ground subexpressions (those with no free variables), for calls \u000A of functions whose <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s are \u000A <tt><see topic=\"ACL2____ENABLE\">enable</see></tt>d.  Note that here, ``enabled'' refers to the current global \u000A <see topic=\"ACL2____THEORY\">theory</see>, not to any <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> given to the guard verification \u000A process; after all, the guard conjecture is computed even before its initial \u000A goal is produced.  Also note that this evaluation is done in an environment as \u000A though <v>:set-guard-checking :all</v> had been executed, so that we can trust \u000A that this evaluation takes place without guard violations; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.</p> \u000A \u000A <p>If you want to verify the <see topic=\"ACL2____GUARD\">guard</see>s on functions that are built into \u000A ACL2, you will first need to put them into <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  See \u000A <see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>, specifically the ``Remark on system functions'' in \u000A that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p>"],

 "ACL2____DEFINITION":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes \u000A and how they are used to build rules from formulas.  An example <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula from which a <v>:definition</v> rule might be built is:</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> open-len-twice\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x)\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NULL\">null</see> x)\u000A                      0\u000A                    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A                        1\u000A                      (<see topic=\"COMMON-LISP_____B2\">+</see> 2 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> x)))))))\u000A  :rule-classes :definition)\u000A\u000A; Same as above, with :controller-alist made explicit:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> open-len-twice\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x)\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NULL\">null</see> x)\u000A                      0\u000A                    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A                        1\u000A                      (<see topic=\"COMMON-LISP_____B2\">+</see> 2 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> x)))))))\u000A  :rule-classes ((:definition :controller-alist ((<see topic=\"ACL2____LEN\">len</see> t)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> hyp (equiv (fn a1 ... an) body))</code> \u000A \u000A <p>where <v>equiv</v> is an equivalence relation and <v>fn</v> is a function \u000A symbol other than <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, <tt><see topic=\"ACL2____HIDE\">hide</see></tt>, <tt><see topic=\"ACL2____FORCE\">force</see></tt> or <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt>.  Such rules allow ``alternative'' definitions of <v>fn</v> to be \u000A proved as theorems but used as definitions.  These rules are not true \u000A ``definitions'' in the sense that they (a) cannot introduce new function \u000A symbols and (b) do not have to be terminating recursion schemes.  They are \u000A just conditional rewrite rules that are controlled the same way we control \u000A recursive definitions.  We call these ``definition rules'' or ``generalized \u000A definitions''.</p> \u000A \u000A <p>Consider the general form above.  Generalized definitions are stored among \u000A the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules for the function ``defined,'' <v>fn</v> above, \u000A but the procedure for applying them is a little different.  During rewriting, \u000A instances of <v>(fn a1 ... an)</v> are replaced by corresponding instances of \u000A <v>body</v> provided the <v>hyp</v>s can be established as for a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule and the result of rewriting <v>body</v> satisfies the criteria for \u000A function expansion.  There are two primary criteria, either of which permits \u000A expansion.  The first is that the ``recursive'' calls of <v>fn</v> in the \u000A rewritten body have arguments that already occur in the goal conjecture.  The \u000A second is that the ``controlling'' arguments to <v>fn</v> are simpler in the \u000A rewritten body.</p> \u000A \u000A <p>The notions of ``recursive call'' and ``controllers'' are complicated by \u000A the provisions for mutually recursive definitions.  Consider a ``clique'' of \u000A mutually recursive definitions.  Then a ``recursive call'' is a call to any \u000A function defined in the clique and an argument is a ``controller'' if it is \u000A involved in the measure that decreases in all recursive calls.  These notions \u000A are precisely defined by the definitional principle and do not necessarily \u000A make sense in the context of generalized definitional equations as implemented \u000A here.</p> \u000A \u000A <p>But because the heuristics governing the use of generalized definitions \u000A require these notions, it is generally up to the user to specify which calls \u000A in body are to be considered recursive and what the controlling arguments are. \u000A This information is specified in the <v>:clique</v> and <v>:controller-alist</v> \u000A fields of the <v>:definition</v> rule class.</p> \u000A \u000A <p>The <v>:clique</v> field is the list of function symbols to be considered \u000A recursive calls of <v>fn</v>.  In the case of a non-recursive definition, the \u000A <v>:clique</v> field is empty; in a singly recursive definition, it should \u000A consist of the singleton list containing <v>fn</v>; otherwise it should be a \u000A list of all of the functions in the mutually recursive clique with this \u000A definition of <v>fn</v>.</p> \u000A \u000A <p>If the <v>:clique</v> field is not provided it defaults to <v>nil</v> if \u000A <v>fn</v> does not occur as a function symbol in <v>body</v> and it defaults to \u000A the singleton list containing <v>fn</v> otherwise.  Thus, <v>:clique</v> must be \u000A supplied by the user only when the generalized definition rule is to be \u000A treated as one of several in a mutually recursive clique.</p> \u000A \u000A <p>The <v>:controller-alist</v> is an alist that maps each function symbol in \u000A the <v>:clique</v> to a mask specifying which arguments are considered \u000A controllers.  The mask for a given member of the clique, <v>fn</v>, must be a \u000A list of <v>t</v>'s and <v>nil</v>'s of length equal to the arity of <v>fn</v>.  A \u000A <v>t</v> should be in each argument position that is considered a ``controller'' \u000A of the recursion.  For a function admitted under the principle of definition, \u000A an argument controls the recursion if it is one of the arguments measured in \u000A the termination argument for the function.  But in generalized definition \u000A rules, the user is free to designate any subset of the arguments as \u000A controllers.  Failure to choose wisely may result in the ``infinite \u000A expansion'' of definitional rules but cannot render ACL2 unsound since the \u000A rule being misused is a theorem.</p> \u000A \u000A <p>If the <v>:controller-alist</v> is omitted it can sometimes be defaulted \u000A automatically by the system.  If the <v>:clique</v> is <v>nil</v>, the \u000A <v>:controller-alist</v> defaults to <v>nil</v>.  If the <v>:clique</v> is a \u000A singleton containing <v>fn</v>, the <v>:controller-alist</v> defaults to the \u000A controller alist computed by <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn args body)</v>.  (The user can obtain \u000A some control over this analysis by setting the default ruler-extenders; see \u000A <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.)  If the <v>:clique</v> contains more than one function, \u000A the user must supply the <v>:controller-alist</v> specifying the controllers for \u000A each function in the clique.  This is necessary since the system cannot \u000A determine and thus cannot analyze the other definitional equations to be \u000A included in the clique.</p> \u000A \u000A <p>For example, suppose <v>fn1</v> and <v>fn2</v> have been defined one way and it \u000A is desired to make ``alternative'' mutually recursive definitions available to \u000A the rewriter.  Then one would prove two theorems and store each as a \u000A <v>:definition</v> rule.  These two theorems would exhibit equations \u000A ``defining'' <v>fn1</v> and <v>fn2</v> in terms of each other.  No provision is \u000A here made for exhibiting these two equations as a system of equations.  One is \u000A proved and then the other.  It just so happens that the user intends them to \u000A be treated as mutually recursive definitions.  To achieve this end, both \u000A <v>:definition</v> rules should specify the <v>:clique</v> <v>(fn1 fn2)</v> and \u000A should specify a suitable <v>:controller-alist</v>.  If, for example, the new \u000A definition of <v>fn1</v> is controlled by its first argument and the new \u000A definition of <v>fn2</v> is controlled by its second and third (and they each \u000A take three arguments) then a suitable <v>:controller-alist</v> would be <v>((fn1\u000A t nil nil) (fn2 nil t t))</v>.  The order of the pairs in the alist is \u000A unimportant, but there must be a pair for each function in the clique.</p> \u000A \u000A <p>Inappropriate heuristic advice via <v>:clique</v> and <v>:controller-alist</v> \u000A can cause ``infinite expansion'' of generalized definitions, but cannot render \u000A ACL2 unsound.</p> \u000A \u000A <p>Note that the actual definition of <v>fn1</v> has the runic name \u000A <v>(:definition fn1)</v>.  The runic name of the alternative definition is \u000A <v>(:definition lemma)</v>, where <v>lemma</v> is the name given to the event that \u000A created the generalized <v>:definition</v> rule.  This allows theories to switch \u000A between various ``definitions'' of the functions.</p> \u000A \u000A <p>By default, a <v>:definition</v> rule establishes the so-called ``body'' of a \u000A function.  The body is used by <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>, and it is also used \u000A heuristically by the theorem prover's preprocessing (the initial \u000A simplification using ``simple'' rules that is controlled by the \u000A <v>preprocess</v> symbol in <v>:do-not</v> <see topic=\"ACL2____HINTS\">hints</see>), induction analysis, and \u000A the determination for when to warn about non-recursive functions in rules. \u000A The body is also used by some heuristics involving whether a function is \u000A recursively defined, and by the <v>expand</v>, <v>x</v>, and <v>x-dumb</v> commands \u000A of the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a discussion of the optional field \u000A <v>:install-body</v> of <v>:definition</v> rules, which controls whether a \u000A <v>:definition</v> rule is used as described in the paragraph above.  Note that \u000A even if <v>:install-body nil</v> is supplied, the rewriter will still rewrite \u000A with the <v>:definition</v> rule; in that case, ACL2 just won't install a new \u000A body for the top function symbol of the left-hand side of the rule, which for \u000A example affects the application of <v>:expand</v> hints as described in the \u000A preceding paragraph.  Also see <see topic=\"ACL2____SET-BODY\">set-body</see> and see <see topic=\"ACL2____SHOW-BODIES\">show-bodies</see> for \u000A how to change the body of a function symbol.</p> \u000A \u000A <p>Note only that if you prove a definition rule for function <v>foo</v>, say, \u000A <v>foo-new-def</v>, you will need to refer to that definition as \u000A <v>foo-new-def</v> or as <v>(:DEFINITION foo-new-def)</v>.  That is because a \u000A <v>:definition</v> rule does not change the meaning of the symbol <v>foo</v> for \u000A <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>, nor does it change the meaning of the symbol <v>foo</v> \u000A in theory expressions; see <see topic=\"ACL2____THEORIES\">theories</see>, in particular the discussion there \u000A of runic designators.  Similarly <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> <v>foo</v> and <v>:</v><tt><see topic=\"ACL2____PF\">pf</see></tt> <v>foo</v> will still show the original definition of <v>foo</v>.</p> \u000A \u000A <p>The definitional principle, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, actually adds <v>:definition</v> \u000A rules.  Thus the handling of generalized definitions is exactly the same as \u000A for ``real'' definitions because no distinction is made in the implementation. \u000A Suppose <v>(fn x y)</v> is <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d to be <v>body</v>.  Note that <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt> or <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>) can compute the clique \u000A for <v>fn</v> from the syntactic presentation and it can compute the controllers \u000A from the termination analysis.  Provided the definition is admissible, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> adds the <v>:definition</v> rule <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x y) body)</v>.</p>"],

 "ACL2____FMT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>ACL2 provides the functions <v>fmt</v>, <tt><see topic=\"ACL2____FMT1\">fmt1</see></tt>, and <tt><see topic=\"ACL2____FMS\">fms</see></tt> \u000A as substitutes for Common Lisp's <v>format</v> function.  Also see <see topic=\"ACL2____FMT_12\">fmt!</see>, \u000A see <see topic=\"ACL2____FMT1_12\">fmt1!</see>, and see <see topic=\"ACL2____FMS_12\">fms!</see> for versions of these functions that \u000A write forms to files in a manner that allows them to be read, by avoiding \u000A using backslash (<v>\\</v>) to break long lines.  There are also analogues of \u000A these functions that return a string without taking <tt><see topic=\"ACL2____STATE\">state</see></tt> as an \u000A argument; see <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>.</p> \u000A \u000A <p>All three print a given string under an alist pairing character objects \u000A with values, interpreting certain ``tilde-directives'' in the string. \u000A <v>Channel</v> must be a character output channel (e.g., <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>).</p> \u000A \u000A <code>General Forms:                                            result\u000A(<see topic=\"ACL2____FMS\">fms</see> string alist channel state evisc-tuple)         ; state\u000A(<see topic=\"ACL2____FMT\">fmt</see> string alist channel state evisc-tuple)         ; (<see topic=\"ACL2____MV\">mv</see> col state)\u000A(<see topic=\"ACL2____FMT1\">fmt1</see> string alist column channel state evisc-tuple) ; (<see topic=\"ACL2____MV\">mv</see> col state)</code> \u000A \u000A <p><tt><see topic=\"ACL2____FMS\">Fms</see></tt> and <v>fmt</v> print an initial newline to put <v>channel</v> in \u000A column <v>0</v>; <tt><see topic=\"ACL2____FMT1\">Fmt1</see></tt> requires the current column as input.  Columns are \u000A numbered from <v>0</v>.  The current column is the column into which the next \u000A character will be printed.  (Thus, the current column number is also the \u000A number of <see topic=\"ACL2____CHARACTERS\">characters</see> printed since the last newline.)  The <v>col</v> \u000A returned by <v>fmt</v> and <tt><see topic=\"ACL2____FMT1\">fmt1</see></tt> is the current column at the conclusion \u000A of the formatting.  <v>Evisc-tuple</v> must be either <v>nil</v> (meaning no \u000A abbreviations are used when objects are printed) or an ``evisceration tuple''; \u000A see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>.</p> \u000A \u000A <p>We list the tilde-directives below.  The notation is explained after the \u000A chart.</p> \u000A \u000A <code>~xx  pretty print vx (maybe after printing a newline)\u000A~yx  pretty print vx starting in current column; end with newline\u000A~Xxy like ~xx but use vy as the evisceration tuple\u000A~Yxy like ~yx but use vy as the evisceration tuple\u000A~@x  if vx is a string, &quot;str&quot;,  recursively format &quot;str&quot;\u000A     if vx is (&quot;str&quot; . a), recursively format &quot;str&quot; under a+\u000A~#x~[...~/...~/ ... ~/...~] cases on vx\u000A     ^    ^     ...   ^  if 0&lt;=vx&lt;=k, choose vxth alternative\u000A     0    1     ...   k  if vx is a list of length 1, case 0; else 1\u000A~*x  iterator: vx must be of the form\u000A     (&quot;str0&quot; &quot;str1&quot; &quot;str2&quot; &quot;str3&quot; lst . a);\u000A     if lst is initially empty, format &quot;str0&quot; under a+; otherwise,\u000A     bind #\\* successively to the elements of lst and then\u000A     recursively format &quot;stri&quot; under a+, where i=1 if there is one\u000A     element left to process, i=2 if there are two left, and i=3\u000A     otherwise.\u000A~&amp;x  print elements of vx with ~x, separated by commas and a\u000A     final ``and''\u000A~vx  print elements of vx with ~x, separated by commas and a\u000A     final ``or''\u000A~nx  if vx is a small positive integer, print it as a word, e.g.,\u000A     seven;\u000A     if vx is a singleton containing a small positive integer, print\u000A       the corresponding ordinal as a word, e.g., seventh\u000A~Nx  like ~nx but the word is capitalized, e.g., Seven or Seventh\u000A~tx  tab out to column vx; newline first if at or past column vx\u000A~cx  vx is (n . w), print integer n right justified in field of\u000A     width w\u000A~fx  print object vx flat over as many lines as necessary\u000A~Fx  same as ~f, except that subsequent lines are indented to\u000A     start one character to the right of the first character printed\u000A~sx  if vx is a symbol, print vx, breaking on hyphens (unless the\u000A     symbol would normally be printed with surrounding vertical bar\u000A     characters (|), in which case print as with ~fx); if vx is a\u000A     string, print the characters in it, breaking on hyphens; else\u000A     vx is a number, to be printed using the current print-base and\u000A     print-radix\u000A~    tilde space: print a space\u000A~_x  print vx spaces\u000A~\u000A     tilde newline: skip following whitespace\u000A~%   output a newline\u000A~|   output a newline unless already on left margin\u000A~~   print a tilde\u000A~-   if close to rightmargin, output a hyphen and newline; else\u000A     skip this char</code> \u000A \u000A <p>If <v>x</v> is a character, then <v>vx</v> is the value of <v>#\\x</v> under the \u000A current alist.  Consider for example the discussion above for <v>~y</v>, \u000A ``<v>~yx pretty print vx</v>'', applied to the following expression: <v>(<see topic=\"ACL2____FMT\">fmt</see>\u000A &quot;HELLO ~y7&quot; (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\7 'world)) *standard-co* state nil)</v>.  Then in \u000A this example: <v>#\\x</v> is 7; and <v>vx</v> is the value of character <v>#\\7</v> \u000A under the given alist, which is the symbol, <v>WORLD</v>.  Thus, ACL2 will print \u000A <v>HELLO WORLD</v>.  When we say ``format <v>str</v> under <v>a+</v>'' we mean: \u000A process the given string under an alist obtained by appending <v>a</v> to the \u000A current alist.  The following example illustrates how this works.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____FMS\">fms</see> &quot;~@0&quot;\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____CONS\">cons</see> &quot;~x0 ~@1&quot; (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0  'abc))))\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\1 &quot;-- and now: ~x0 again~%&quot;))\u000A            *standard-co* state nil)\u000A\u000AABC -- and now: ABC again\u000A&lt;state&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p>Note: <v>~p</v>, <v>~q</v>, <v>~P</v>, and <v>~Q</v> are also currently supported, \u000A but are deprecated.  These are respectively the same as <v>~x</v>, <v>~y</v>, \u000A <v>~X</v>, and <v>~Y</v>, except that their arguments are expected to be terms, \u000A preferably untranslated (user-level) terms, that could be printed using infix \u000A notation in certain environments.  Infix printing is not currently supported \u000A but may be if there is sufficient need for it.</p> \u000A \u000A <p>ACL2's formatting functions print to the indicated channel, keeping track \u000A of which column they are in.  <tt><see topic=\"ACL2____FMT1\">Fmt1</see></tt> can be used if the caller knows \u000A which column the channel is in (i.e., how many <see topic=\"ACL2____CHARACTERS\">characters</see> have been \u000A printed since the last newline).  Otherwise, <v>fmt</v> or <tt><see topic=\"ACL2____FMS\">fms</see></tt> must be \u000A used, both of which output a newline so as to establish the column position at \u000A <v>0</v>.  Unlike Common Lisp's <v>format</v> routine, <v>fmt</v> and its relatives \u000A break the output into lines so that, by default, an attempt is made to avoid \u000A printing past column <v>77</v> (the value of constant \u000A <v>*fmt-hard-right-margin-default*</v>).  See <see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see> \u000A for a discussion of how linebreaks are inserted and how to change the relevant \u000A default settings.</p> \u000A \u000A <p>The formatting functions scan the string from left to right, printing each \u000A successive character unless it is a tilde <v>(~)</v>.  Upon encountering tildes \u000A the formatters take action determined by the character or <see topic=\"ACL2____CHARACTERS\">characters</see> \u000A immediately following the tilde.  The typical tilde-directive is a group of \u000A three successive <see topic=\"ACL2____CHARACTERS\">characters</see> from the string being printed.  For \u000A example, <v>~x0</v> is a 3 character <v>tilde-directive</v>.  The first character \u000A in a tilde-directive is always the tilde character itself.  The next character \u000A is called the ``command'' character.  The character after that is usually \u000A taken as the name of a ``format variable'' that is bound in the alist under \u000A which the string is being printed.  Format variables are, by necessity, <see topic=\"ACL2____CHARACTERS\">characters</see>.  The objects actually printed by a tilde-directive are the \u000A objects obtained by looking up the command's format variables in the alist. \u000A Typical format variable names are <v>0</v>, <v>1</v>, <v>2</v>, ..., <v>9</v>, <v>a</v>, \u000A <v>b</v>, <v>c</v>, etc., and if a tilde-directive uses the format variable \u000A <v>0</v>, as in <v>~x0</v>, then the character <v>#\\0</v> must be bound in the \u000A alist.  Some tilde commands take no arguments and others take more than one, \u000A so some directives are of length two and others are longer.</p> \u000A \u000A <p>It should be noted that this use of <see topic=\"ACL2____CHARACTERS\">characters</see> in the string to \u000A denote arguments is another break from Common Lisp's <v>format</v> routine.  In \u000A Common Lisp, the directives refer implicitly to the ``next item to be \u000A printed.''  But in ACL2 the directives name each item explicitly with our \u000A format variables.</p> \u000A \u000A <p>The following text contains examples that can be evaluated.  To make this \u000A process easier, we use a macro which is defined as part of ACL2 just for this \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.  The macro is named <v>fmx</v> and it takes up to eleven \u000A arguments, the first of which is a format string, <v>str</v>, and the others of \u000A which are taken as the values of format variables.  The variables used are \u000A <v>#\\0</v> through <v>#\\9</v>.  The macro constructs an appropriate alist, \u000A <v>a</v>, and then evaluates <v>(<see topic=\"ACL2____FMT\">fmt</see> str a *standard-co* state nil)</v>.</p> \u000A \u000A <p>Thus,</p> \u000A \u000A <code>(fmx &quot;Here is v0, ~x0, and here is v1, ~x1.&quot;\u000A     (<see topic=\"COMMON-LISP____CONS\">cons</see> 'value 0)\u000A     (<see topic=\"COMMON-LISP____CONS\">cons</see> 'value 1))</code> \u000A \u000A <p>is just an abbreviation for</p> \u000A \u000A <code>(<see topic=\"ACL2____FMT\">fmt</see> &quot;Here is v0, ~x0, and here is v1, ~x1.&quot;\u000A     (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____CONS\">cons</see> 'value 0))\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\1 (<see topic=\"COMMON-LISP____CONS\">cons</see> 'value 1)))\u000A     *standard-co*\u000A     state\u000A     nil)</code> \u000A \u000A <p>which returns <v>(<see topic=\"ACL2____MV\">mv</see> 53 state)</v> after printing the line</p> \u000A \u000A <code>Here is v0, (VALUE . 0), and here is v1, (VALUE . 1).</code> \u000A \u000A <p>We now devote special attention to three of the tilde-directives whose use \u000A is non-obvious.</p> \u000A \u000A <p><i>The Case Statement</i></p> \u000A \u000A <p><v>~#x</v> is essentially a ``case statement'' in the language of <v>fmt</v>. \u000A The proper form of the statement is</p> \u000A \u000A <code>~#x~[case-0~/case-1~/ ... ~/case-k~],</code> \u000A \u000A <p>where each of the <v>case-i</v> is a format string.  In the most common use, \u000A the variable <v>x</v> has an integer value, <v>vx</v>, between <v>0</v> and <v>k</v>, \u000A inclusive.  The effect of formatting the directive is to format \u000A <v>case-vx</v>.</p> \u000A \u000A <p>For example</p> \u000A \u000A <code>(fmx &quot;Go ~#0~[North~/East~/South~/West~].~%&quot; 1)</code> \u000A \u000A <p>will print ``Go East.'' followed by a newline and will return</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MV\">mv</see> 0 state)</v>, while if you change the <v>1</v> above to <v>3</v> (the \u000A maximum legal value), it will print ``Go West.''</p> \u000A \u000A <p>In order to make it easier to print such phrases as ``there are seven \u000A cases'' requiring agreement between subject and verb based on the number of \u000A elements of a list, the case statement allows its variable to take a list as \u000A its value and selects <v>case-0</v> if the list has length <v>1</v> and \u000A <v>case-1</v> otherwise.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____CASES\">cases</see> '(a b c)))\u000A  (fmx &quot;There ~#0~[is ~n1 case~/are ~n1 cases~].&quot;\u000A       cases\u000A       (<see topic=\"COMMON-LISP____LENGTH\">length</see> cases)))</code> \u000A \u000A <p>will print ``There are three cases.'' but if you change the</p> \u000A \u000A <p><v>'(a b c)</v> above simply to <v>'(a)</v> it will print ``There is one \u000A case.'' and if you change it to <v>nil</v> it will print ``There are zero \u000A cases.''</p> \u000A \u000A <p><i>Indirection</i></p> \u000A \u000A <p>Roughly speaking, <v>~@</v> will act as though the value of its argument is a \u000A format string and splice it into the current string at the current position. \u000A It is often used when the phrase to be printed must be computed.  For \u000A example,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((ev 'DEFUN))\u000A (fmx &quot;~x0 is an event~@1.&quot;\u000A      'foo\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> ev '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> defstub encapsulate))\u000A          &quot; that may introduce a function symbol&quot;\u000A          &quot;&quot;)))</code> \u000A \u000A <p>will print ``<v>foo</v> is an event that may introduce a function symbol,'' \u000A but if the value of <v>ev</v> is changed from <v>'</v><tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> to \u000A <v>'</v><tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, it prints ``<v>foo</v> is an event.''  The <v>~@</v> \u000A directive ``splices'' in the computed phrase (which might be empty).  Of \u000A course, this particular example could be done with the case statement</p> \u000A \u000A <code>~#1~[~/ that may introduce a function symbol~]</code> \u000A \u000A <p>where the value of <v>#\\1</v> is appropriately computed to be <v>0</v> or \u000A <v>1</v>.</p> \u000A \u000A <p>If the argument to <v>~@</v> is a pair, it is taken to be a format string \u000A <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>ed onto an alist, i.e., <v>(&quot;str&quot; . a)</v>, and the alist, <v>a</v>, \u000A is used to extend the current one before <v>&quot;str&quot;</v> is recursively \u000A processed.  This feature of <v>fmt</v> can be used to pass around ``phrases'' \u000A that contain computed contextual information in <v>a</v>.  The most typical use \u000A is as ``error messages.''  For example, suppose you are writing a function \u000A which does not have access to <tt><see topic=\"ACL2____STATE\">state</see></tt> and so cannot print an error \u000A message.  It may nevertheless be necessary for it to signal an error to its \u000A caller, say by returning two results, the first of which is interpreted as an \u000A error message if non-<v>nil</v>.  Our convention is to use a <v>~@</v> pair to \u000A represent such messages.  For example, the error value might be produced by \u000A the code:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A  &quot;Error:  The instruction ~x0 is illegal when the stack is ~x1.~%&quot;\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (current-instruction st))\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\1 (i-stack st))))</code> \u000A \u000A <p>If the <v>current-instruction</v> and <v>i-stack</v> (whatever they are) are \u000A <v>'(popi 3)</v> and <v>'(a b)</v> when the <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> above is evaluated, then \u000A it produces</p> \u000A \u000A <code>'(&quot;Error:  The instruction ~x0 is illegal when the stack is ~x1.~%&quot;\u000A  (#\\0 POPI 3)\u000A  (#\\1 A B))</code> \u000A \u000A <p>and if this pair is made the value of the <v>fmt</v> variable <v>0</v>, then \u000A <v>~@0</v> will print</p> \u000A \u000A <code>Error:  The instruction (POPI 3) is illegal when the stack is (A B).</code> \u000A \u000A <p>For example, evaluate</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see>\u000A ((pair\u000A  '(&quot;Error:  The instruction ~x0 is illegal when the stack is ~x1.~%&quot;\u000A    (#\\0 POPI 3)\u000A    (#\\1 A B))))\u000A (fmx &quot;~@0&quot; pair)).</code> \u000A \u000A <p>Thus, even though the function that produced the ``error'' could not print \u000A it, it could specify exactly what error message and data are to be \u000A printed.</p> \u000A \u000A <p>This example raises another issue.  Sometimes it is desirable to break \u000A lines in your format strings so as to make your source code more attractive. \u000A That is the purpose of the <v>tilde-newline</v> directive.  The following code \u000A produces exactly the same output as described above.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((pair '(&quot;Error:  The instruction ~x0 ~\u000A              is illegal when the stack is ~\u000A              ~x1.~%&quot;\u000A              (#\\0 POPI 3)\u000A              (#\\1 A B))))\u000A (fmx &quot;~@0&quot; pair)).</code> \u000A \u000A <p>Finally, observe that when <v>~@0</v> extends the current alist, <v>alist</v>, \u000A with the one, <v>a</v>, in its argument, the bindings from <v>a</v> are added to \u000A the front of <v>alist</v>, overriding the current values of any shared \u000A variables.  This ensures that the variable values seen by the recursively \u000A processed string, <v>&quot;str&quot;</v>, are those from <v>a</v>, but if <v>&quot;str&quot;</v> \u000A uses variables not bound in <v>a</v>, their values are as specified in the \u000A original alist.  Intuitively, variables bound in <v>a</v> are local to the \u000A processing of <v>(&quot;str&quot; . a)</v> but <v>&quot;str&quot;</v> may use ``global \u000A variables.''  The example above illustrates this because when the <v>~@0</v> is \u000A processed, <v>#\\0</v> is bound to the error message pair.  But when the \u000A <v>~x0</v> in the error string is processed, <v>#\\0</v> is bound to the illegal \u000A instruction.</p> \u000A \u000A <p><i>Iteration</i></p> \u000A \u000A <p>The <v>~*</v> directive is used to process each element of a list.  For \u000A example,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((lst '(a b c d e f g h))) ; a true-list whose elements we exhibit\u000A (fmx &quot;~*0&quot;\u000A      `(&quot;Whoa!&quot;          ; what to print if there's nothing to print\u000A        &quot;~x*!&quot;           ; how to print the last element\u000A        &quot;~x* and &quot;       ; how to print the 2nd to last element\u000A        &quot;~x*, &quot;          ; how to print all other elements\u000A        ,lst)))          ; the list of elements to print</code> \u000A \u000A <p>will print ``<v>A, B, C, D, E, F, G and H!</v>''.  Try this example with \u000A other true list values of <v>lst</v>, such as <v>'(a b)</v>, <v>'(a)</v>, and \u000A <v>nil</v>.  The tilde-directives <v>~&amp;0</v> and <v>~v0</v>, which take a true list \u000A argument and display its elements separated by commas and a final ``and'' or \u000A ``or,'' are implemented in terms of the more general <v>~*</v>.</p> \u000A \u000A <p>The <v>~*</v> directive allows the 5-tuple to specify in its final <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> an alist with which to extend the current one before processing the \u000A individual elements.</p> \u000A \u000A <p>We often use <v>~*</v> to print a series of phrases, separated by suitable \u000A punctuation, whitespace and noise words.  In such use, the <v>~*</v> handles the \u000A separation of the phrases and each phrase is generally printed by <v>~@</v>.</p> \u000A \u000A <p>Here is a complex example.  In the <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, below, we bind phrases to \u000A a list of <v>~@</v> pairs and then we create a <v>~*</v> 5-tuple to print out the \u000A conjunction of the phrases with a parenthetical ``finally!'' if the series is \u000A longer than 3.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((phrases\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____LIST\">list</see> &quot;simplifying with the replacement rules ~&amp;0&quot;\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 '(rewrite-rule1\u000A                                rewrite-rule2\u000A                                rewrite-rule3)))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> &quot;destructor elimination using ~x0&quot;\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 'elim-rule))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> &quot;generalizing the terms ~&amp;0&quot;\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 '((rev x) (app u v))))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> &quot;inducting on ~x0&quot;\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 'I))))\u000A       (5-tuple\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see>\u000A         &quot;magic&quot;                            ; no phrases\u000A         &quot;~@*&quot;                              ; last phrase\u000A         &quot;~@*, and~#f~[~/ (finally!)~] &quot;    ; second to last phrase\u000A         &quot;~@*, &quot;                            ; other phrases\u000A         phrases                            ; the phrases themselves\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\f\u000A               (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see>(<see topic=\"COMMON-LISP____LENGTH\">length</see> phrases) 3) 1 0))))) ;print ``finally''?\u000A  (fmx &quot;We did it by ~*0.&quot; 5-tuple))</code> \u000A \u000A <p>This <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> prints</p> \u000A \u000A <code>We did it by simplifying with the replacement rules REWRITE-RULE1,\u000AREWRITE-RULE2 and REWRITE-RULE3, destructor elimination using ELIM-\u000ARULE, generalizing the terms (REV X) and (APP U V), and (finally!)\u000Ainducting on I.</code> \u000A \u000A <p>You might wish to try evaluating the <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> after removing elements \u000A of phrases.</p> \u000A \u000A <p>Most of the output produced by ACL2 is produced via <v>fmt</v> statements. \u000A Thus, inspection of the source code will yield many examples.  A complicated \u000A example is the code that explains the simplifier's work.  See <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt> \u000A <v>simplify-clause-msg1</v>.  An ad hoc example is provided by the function \u000A <v>fmt-doc-example</v>, which takes two arguments: an arbitrary true list and \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  To see how <v>fmt-doc-example</v> works, <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> \u000A <v>fmt-doc-example</v>.</p> \u000A \u000A <code>(fmt-doc-example '(a b c d e f g h i j k l m n o p) state)</code> \u000A \u000A <p>will produce the output</p> \u000A \u000A <code>Here is a true list:  (A B C D E F G H I J K L M N O P).  It has 16\u000Aelements, the third of which is C.\u000A\u000AWe could print each element in square brackets:\u000A([A], [B], [C], [D], [E], [F], [G], [H], [I], [J], [K], [L], [M], [N],\u000A[almost there: O], [the end: P]).  And if we wished to itemize them\u000Ainto column 15 we could do it like this\u000A0123456789012345\u000A    0 (zeroth) A\u000A    1 (<see topic=\"COMMON-LISP____FIRST\">first</see>)  B\u000A    2 (<see topic=\"COMMON-LISP____SECOND\">second</see>) C\u000A    3 (<see topic=\"COMMON-LISP____THIRD\">third</see>)  D\u000A    4 (<see topic=\"COMMON-LISP____FOURTH\">fourth</see>) E\u000A    5 (<see topic=\"COMMON-LISP____FIFTH\">fifth</see>)  F\u000A    6 (<see topic=\"COMMON-LISP____SIXTH\">sixth</see>)  G\u000A    7 (<see topic=\"COMMON-LISP____SEVENTH\">seventh</see>)\u000A               H\u000A    8 (<see topic=\"COMMON-LISP____EIGHTH\">eighth</see>) I\u000A    9 (<see topic=\"COMMON-LISP____NINTH\">ninth</see>)  J\u000A   10 (<see topic=\"COMMON-LISP____TENTH\">tenth</see>)  K\u000A   11 (eleventh)\u000A               L\u000A   12 (twelfth)\u000A               M\u000A   13 (thirteenth)\u000A               N\u000A   14 (14th)   O\u000A   15 (15th)   P\u000AEnd of example.</code> \u000A \u000A <p>and return <v>(<see topic=\"ACL2____MV\">mv</see> 15 state)</v>.</p> \u000A \u000A <p>Finally, we should remind the reader that <v>fmt</v> and its subfunctions, \u000A most importantly <v>fmt0</v>, are written entirely in ACL2.  We make this \u000A comment for two reasons.  First, it illustrates the fact that quite low level \u000A code can be efficiently written in the language.  Second, it means that as a \u000A last resort for documentation purposes you can read the source code without \u000A changing languages.</p>"],

 "ACL2____ARRAYS":[["Programming"],"ACL2 Sources","ACL2","<p>Below we begin a detailed presentation of ACL2 arrays.  ACL2's \u000A single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see>) provide a similar functionality \u000A that is generally more efficient when there are updates (writes), but is also \u000A more restrictive.</p> \u000A \u000A <p>See <see topic=\"ACL2____ARRAYS-EXAMPLE\">arrays-example</see> for a brief introduction illustrating the use of \u000A ACL2 arrays.</p> \u000A \u000A <p>ACL2 provides relatively efficient 1- and 2-dimensional arrays.  Arrays are \u000A awkward to provide efficiently in an applicative language because the \u000A programmer rightly expects to be able to ``modify'' an array object with the \u000A effect of changing the behavior of the element accessing function on that \u000A object.  This, of course, does not make any sense in an applicative setting. \u000A The element accessing function is, after all, a function, and its behavior on \u000A a given object is immutable.  To ``modify'' an array object in an applicative \u000A setting we must actually produce a new array object.  Arranging for this to be \u000A done efficiently is a challenge to the implementors of the language.  In \u000A addition, the programmer accustomed to the von Neumann view of arrays must \u000A learn how to use immutable applicative arrays efficiently.</p> \u000A \u000A <p>In this note we explain 1-dimensional arrays.  In particular, we explain \u000A briefly how to create, access, and ``modify'' them, how they are implemented, \u000A and how to program with them.  2-dimensional arrays are dealt with by \u000A analogy.</p> \u000A \u000A <p><i>The Logical Description of ACL2 Arrays</i></p> \u000A \u000A <p>An ACL2 1-dimensional array is an object that associates arbitrary objects \u000A with certain integers, called ``indices.'' Every array has a dimension, \u000A <v>dim</v>, which is a positive integer.  The indices of an array are the \u000A consecutive integers from <v>0</v> through <v>dim-1</v>.  To obtain the object \u000A associated with the index <v>i</v> in an array <v>a</v>, one uses <v>(<see topic=\"ACL2____AREF1\">aref1</see> name a\u000A i)</v>.  <v>Name</v> is a symbol that is irrelevant to the semantics of <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> but affects the speed with which it computes.  We will talk more about \u000A array ``names'' later.  To produce a new array object that is like <v>a</v> but \u000A which associates <v>val</v> with index <v>i</v>, one uses <v>(<see topic=\"ACL2____ASET1\">aset1</see> name a i\u000A val)</v>.</p> \u000A \u000A <p>An ACL2 1-dimensional array is actually an alist.  There is no special ACL2 \u000A function for creating arrays; they are generally built with the standard list \u000A processing functions <tt><see topic=\"COMMON-LISP____LIST\">list</see></tt> and <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>.  However, there is a \u000A special ACL2 function, called <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>, for speeding up access to the \u000A elements of such an alist.  We discuss <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> later.</p> \u000A \u000A <p>One element of the alist must be the ``header'' of the array.  The <see topic=\"ACL2____HEADER\">header</see> of a 1-dimensional array with dimension <v>dim</v> is of the form:</p> \u000A \u000A <code>(:HEADER :DIMENSIONS (dim)\u000A         :MAXIMUM-LENGTH max\u000A         :DEFAULT obj ; optional\u000A         :NAME name   ; optional\u000A         :ORDER order ; optional values are &lt; (<see topic=\"COMMON-LISP____THE\">the</see> default), &gt;, or :none/nil\u000A         ).</code> \u000A \u000A <p><v>Obj</v> may be any object and is called the ``default value'' of the \u000A array.  <tt><see topic=\"COMMON-LISP____MAX\">Max</see></tt> must be an integer greater than <v>dim</v>.  <v>Name</v> must \u000A be a symbol.  The <v>:</v><tt><see topic=\"ACL2____DEFAULT\">default</see></tt> and <v>:name</v> entries are optional; \u000A if <v>:</v><tt><see topic=\"ACL2____DEFAULT\">default</see></tt> is omitted, the default value is <v>nil</v>.  The \u000A function <tt><see topic=\"ACL2____HEADER\">header</see></tt>, when given a name and a 1- or 2-dimensional array, \u000A returns the <see topic=\"ACL2____HEADER\">header</see> of the array.  The functions <tt><see topic=\"ACL2____DIMENSIONS\">dimensions</see></tt>, \u000A <tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt>, and <tt><see topic=\"ACL2____DEFAULT\">default</see></tt> are similar and return the \u000A corresponding fields of the <see topic=\"ACL2____HEADER\">header</see> of the array.  The role of the \u000A <v>:</v><tt><see topic=\"ACL2____DIMENSIONS\">dimensions</see></tt> field is obvious: it specifies the legal indices into \u000A the array.  The roles played by the <v>:</v><tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____DEFAULT\">default</see></tt> fields are described below.</p> \u000A \u000A <p>Aside from the <see topic=\"ACL2____HEADER\">header</see>, the other elements of the alist must each be \u000A of the form <v>(i . val)</v>, where <v>i</v> is an integer and <v>0 &lt;= i &lt; dim</v>, \u000A and <v>val</v> is an arbitrary object.</p> \u000A \u000A <p>The <v>:order</v> field of the header is ignored for 2-dimensional arrays. \u000A For 1-dimensional arrays, it specifies the order of keys (<v>i</v>, above) when \u000A the array is compressed as with <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>, as described below.  An \u000A <v>:order</v> of <v>:none</v> or <v>nil</v> specifies no reordering of the alist by \u000A <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>, and an order of <v>&gt;</v> specifies reordering by <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> so that keys are in descending order.  Otherwise, the alist is \u000A reordered by <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> so that keys are in ascending order.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____AREF1\">Aref1</see> name a i)</v> is <see topic=\"ACL2____GUARD\">guard</see>ed so that <v>name</v> must be a symbol, \u000A <v>a</v> must be an array and <v>i</v> must be an index into <v>a</v>.  The value of \u000A <v>(<see topic=\"ACL2____AREF1\">aref1</see> name a i)</v> is either <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> i a))</v> or else is the default \u000A value of <v>a</v>, depending on whether there is a pair in <v>a</v> whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is <v>i</v>.  Note that <v>name</v> is irrelevant to the value of an <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> expression.  You might <v>:pe aref1</v> to see how simple the definition \u000A is.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ASET1\">Aset1</see> name a i val)</v> is <see topic=\"ACL2____GUARD\">guard</see>ed analogously to the <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> expression.  The value of the <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> expression is essentially \u000A <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> i val) a)</v>.  Again, <v>name</v> is irrelevant.  Note <v>(<see topic=\"ACL2____ASET1\">aset1</see>\u000A name a i val)</v> is an array, <v>a'</v>, with the property that <v>(<see topic=\"ACL2____AREF1\">aref1</see> name a'\u000A i)</v> is <v>val</v> and, except for index <v>i</v>, all other indices into <v>a'</v> \u000A produce the same value as in <v>a</v>.  Note also that if <v>a</v> is viewed as an \u000A alist (which it is) the pair ``binding'' <v>i</v> to its old value is in <v>a'</v> \u000A but ``covered up'' by the new pair.  Thus, the length of an array grows by one \u000A when <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> is done.</p> \u000A \u000A <p>Because <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> covers old values with new ones, an array produced by \u000A a sequence of <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> calls may have many irrelevant pairs in it.  The \u000A function <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> can remove these irrelevant pairs.  Thus, \u000A <v>(<see topic=\"ACL2____COMPRESS1\">compress1</see> name a)</v> returns an array that is equivalent (vis-a-vis <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>) to <v>a</v> but which may be shorter.  For technical reasons, the alist \u000A returned by <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> may also list the pairs in a different order \u000A than listed in <v>a</v>.</p> \u000A \u000A <p>To prevent arrays from growing excessively long due to repeated <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> operations, <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> actually calls <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> on the new \u000A alist whenever the length of the new alist exceeds the <v>:</v><tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> entry, <tt><see topic=\"COMMON-LISP____MAX\">max</see></tt>, in the <see topic=\"ACL2____HEADER\">header</see> of the array.  See \u000A the definition of <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> (for example by using <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>).  This \u000A is primarily just a mechanism for freeing up <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> space consumed while \u000A doing <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> operations.  Note however that this <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> call \u000A is replaced by a hard error if the header specifies an <v>:order</v> of \u000A <v>:none</v> or <v>nil</v>.</p> \u000A \u000A <p>This completes the logical description of 1-dimensional arrays. \u000A 2-dimensional arrays are analogous.  The <v>:</v><tt><see topic=\"ACL2____DIMENSIONS\">dimensions</see></tt> entry of the \u000A <see topic=\"ACL2____HEADER\">header</see> of a 2-dimensional array should be <v>(dim1 dim2)</v>.  A pair of \u000A indices, <v>i</v> and <v>j</v>, is legal iff <v>0 &lt;= i &lt; dim1</v> and <v>0 &lt;= j &lt;\u000A dim2</v>.  The <v>:</v><tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> must be greater than <v>dim1*dim2</v>. \u000A <tt><see topic=\"ACL2____AREF2\">Aref2</see></tt>, <tt><see topic=\"ACL2____ASET2\">aset2</see></tt>, and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> are like their \u000A counterparts but take an additional <v>index</v> argument.  Finally, the pairs \u000A in a 2-dimensional array are of the form <v>((i . j) . val)</v>.</p> \u000A \u000A <p><i>The Implementation of ACL2 Arrays</i></p> \u000A \u000A <p>Very informally speaking, the function <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> ``creates'' an \u000A ACL2 array that provides fast access, while the function <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> \u000A ``maintains'' fast access.  We now describe this informal idea more \u000A carefully.</p> \u000A \u000A <p><tt><see topic=\"ACL2____AREF1\">Aref1</see></tt> is essentially <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt>.  If <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> were \u000A implemented naively the time taken to access an array element would be linear \u000A in the dimension of the array and the number of ``assignments'' to it (the \u000A number of <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> calls done to create the array from the initial \u000A alist).  This is intolerable; arrays are ``supposed'' to provide constant-time \u000A access and change.</p> \u000A \u000A <p>The apparently irrelevant names associated with ACL2 arrays allow us to \u000A provide constant-time access and change when arrays are used in \u000A ``conventional'' ways.  The implementation of arrays makes it clear what we \u000A mean by ``conventional.''</p> \u000A \u000A <p>Recall that array names are symbols.  Behind the scenes, ACL2 associates \u000A two objects with each ACL2 array name.  The first object is called the \u000A ``semantic value'' of the name and is an alist.  The second object is called \u000A the ``raw lisp array'' and is a Common Lisp array.</p> \u000A \u000A <p>When <v>(<see topic=\"ACL2____COMPRESS1\">compress1</see> name alist)</v> builds a new alist, <v>a'</v>, it sets the \u000A semantic value of <v>name</v> to that new alist.  Furthermore, it creates a \u000A Common Lisp array and writes into it all of the index/value pairs of <v>a'</v>, \u000A initializing unassigned indices with the default value.  This array becomes \u000A the raw lisp array of <v>name</v>.  <tt><see topic=\"ACL2____COMPRESS1\">Compress1</see></tt> then returns <v>a'</v>, the \u000A semantic value, as its result, as required by the definition of <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.</p> \u000A \u000A <p>When <v>(<see topic=\"ACL2____AREF1\">aref1</see> name a i)</v> is invoked, <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> first determines \u000A whether the semantic value of <v>name</v> is <v>a</v> (i.e., is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> to the \u000A alist <v>a</v>).  If so, <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> can determine the <v>i</v>th element of \u000A <v>a</v> by invoking Common Lisp's <v>aref</v> function on the raw lisp array \u000A associated with name.  Note that no linear search of the alist <v>a</v> is \u000A required; the operation is done in constant time and involves retrieval of two \u000A global variables, an <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> test and <v>jump</v>, and a raw lisp array \u000A access.  In fact, an ACL2 array access of this sort is about 5 times slower \u000A than a C array access.  On the other hand, if <v>name</v> has no semantic value \u000A or if it is different from <v>a</v>, then <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> determines the answer by \u000A linear search of <v>a</v> as suggested by the <v>assoc-like</v> definition of \u000A <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>.  Thus, <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> always returns the axiomatically specified \u000A result.  It returns in constant time if the array being accessed is the \u000A current semantic value of the name used.  The ramifications of this are \u000A discussed after we deal with <tt><see topic=\"ACL2____ASET1\">aset1</see></tt>.</p> \u000A \u000A <p>When <v>(<see topic=\"ACL2____ASET1\">aset1</see> name a i val)</v> is invoked, <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> does two <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es to create the new array.  Call that array <v>a'</v>.  It will be \u000A returned as the answer.  (In this discussion we ignore the case in which \u000A <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> does a <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.)  However, before returning, <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> determines if <v>name</v>'s semantic value is <v>a</v>.  If so, it makes \u000A the new semantic value of <v>name</v> be <v>a'</v> and it smashes the raw lisp \u000A array of <v>name</v> with <v>val</v> at index <v>i</v>, before returning <v>a'</v> as \u000A the result.  Thus, after doing an <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> and obtaining a new semantic \u000A value <v>a'</v>, all <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>s on that new array will be fast.  Any <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>s on the old semantic value, <v>a</v>, will be slow.</p> \u000A \u000A <p>To understand the performance implications of this design, consider the \u000A chronological sequence in which ACL2 (Common Lisp) evaluates expressions: \u000A basically inner-most first, left-to-right, call-by-value.  An array use, such \u000A as <v>(<see topic=\"ACL2____AREF1\">aref1</see> name a i)</v>, is ``fast'' (constant-time) if the alist supplied, \u000A <v>a</v>, is the value returned by the most recently executed <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> \u000A or <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> on the name supplied.  In the functional expression of \u000A ``conventional'' array processing, all uses of an array are fast.</p> \u000A \u000A <p>The <v>:name</v> field of the <see topic=\"ACL2____HEADER\">header</see> of an array is completely \u000A irrelevant.  Our convention is to store in that field the symbol we mean to \u000A use as the name of the raw lisp array.  But no ACL2 function inspects \u000A <v>:name</v> and its primary value is that it allows the user, by inspecting the \u000A semantic value of the array &mdash; the alist &mdash; to recall the name of \u000A the raw array that probably holds that value.  We say ``probably'' since there \u000A is no enforcement that the alist was compressed under the name in the <see topic=\"ACL2____HEADER\">header</see> or that all <v>aset</v>s used that name.  Such enforcement would be \u000A inefficient.</p> \u000A \u000A <p><i>Some Programming Examples</i></p> \u000A \u000A <p>In the following examples we will use ACL2 ``global variables'' to hold \u000A several arrays.  See <see topic=\"ACL2_____04\">@</see>, and see <see topic=\"ACL2____ASSIGN\">assign</see>.</p> \u000A \u000A <p>Let the <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable <v>a</v> be the 1-dimensional \u000A compressed array of dimension <v>5</v> constructed below.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> a (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                            '((:header :dimensions (5)\u000A                                       :maximum-length 15\u000A                                       :default uninitialized\u000A                                       :name demo)\u000A                              (0 . zero))))</code> \u000A \u000A <p>Then <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)</v> is <v>zero</v> and <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 1)</v> \u000A is <v>uninitialized</v>.</p> \u000A \u000A <p>Now execute</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> b (<see topic=\"ACL2____ASET1\">aset1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 1 'one))</code> \u000A \u000A <p>Then <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 0)</v> is <v>zero</v> and <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 1)</v> \u000A is <v>one</v>.</p> \u000A \u000A <p>All of the <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>s done so far have been ``fast.''</p> \u000A \u000A <p>Note that we now have two array objects, one in the global variable <v>a</v> \u000A and one in the global variable <v>b</v>.  <v>B</v> was obtained by assigning to \u000A <v>a</v>.  That assignment does not affect the alist <v>a</v> because this is an \u000A applicative language.  Thus, <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 1)</v> must <b>still</b> be \u000A <v>uninitialized</v>.  And if you execute that expression in ACL2 you will see \u000A that indeed it is.  However, a rather ugly comment is printed, namely that \u000A this array access is ``slow.''  The reason it is slow is that the raw lisp \u000A array associated with the name <v>demo</v> is the array we are calling <v>b</v>. \u000A To access the elements of <v>a</v>, <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> must now do a linear search. \u000A Any reference to <v>a</v> as an array is now ``unconventional;'' in a \u000A conventional language like Ada or Common Lisp it would simply be impossible to \u000A refer to the value of the array before the assignment that produced our \u000A <v>b</v>.</p> \u000A \u000A <p>Now let us define a function that counts how many times a given object, \u000A <v>x</v>, occurs in an array.  For simplicity, we will pass in the name and \u000A highest index of the array:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> cnt (<see topic=\"ACL2____NAME\">name</see> a i x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard\u000A                         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name a)\u000A                              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                              (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i -1)\u000A                              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name a))))\u000A                         :mode :logic\u000A                         :measure (<see topic=\"ACL2____NFIX\">nfix</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 i))))\u000A         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> i)) 0) ; return 0 if i is at most -1\u000A               ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"ACL2____AREF1\">aref1</see> name a i))\u000A                (<see topic=\"COMMON-LISP____1_B2\">1+</see> (cnt name a (<see topic=\"COMMON-LISP____1-\">1-</see> i) x)))\u000A               (t (cnt name a (<see topic=\"COMMON-LISP____1-\">1-</see> i) x))))</code> \u000A \u000A <p>To determine how many times <v>zero</v> appears in <v>(<see topic=\"ACL2_____04\">@</see> b)</v> we can \u000A execute:</p> \u000A \u000A <code>ACL2 !&gt;(cnt 'demo (<see topic=\"ACL2_____04\">@</see> b) 4 'zero)</code> \u000A \u000A <p>The answer is <v>1</v>.  How many times does <v>uninitialized</v> appear in \u000A <v>(<see topic=\"ACL2_____04\">@</see> b)</v>?</p> \u000A \u000A <code>ACL2 !&gt;(cnt 'demo (<see topic=\"ACL2_____04\">@</see> b) 4 'uninitialized)</code> \u000A \u000A <p>The answer is <v>3</v>, because positions <v>2</v>, <v>3</v> and <v>4</v> of the \u000A array contain that default value.</p> \u000A \u000A <p>Now imagine that we want to assign <v>'two</v> to index <v>2</v> and then count \u000A how many times the 2nd element of the array occurs in the array.  This \u000A specification is actually ambiguous.  In assigning to <v>b</v> we produce a new \u000A array, which we might call <v>c</v>.  Do we mean to count the occurrences in \u000A <v>c</v> of the 2nd element of <v>b</v> or the 2nd element of <v>c</v>?  That is, do \u000A we count the occurrences of <v>uninitialized</v> or the occurrences of <v>two</v>? \u000A If we mean the former the correct answer is <v>2</v> (positions <v>3</v> and \u000A <v>4</v> are <v>uninitialized</v> in <v>c</v>); if we mean the latter, the correct \u000A answer is <v>1</v> (there is only one occurrence of <v>two</v> in <v>c</v>).</p> \u000A \u000A <p>Below are ACL2 renderings of the two meanings, which we call <v>[former]</v> \u000A and <v>[latter]</v>.  (Warning: Our description of these examples, and of an \u000A example <v>[fast former]</v> that follows, assumes that only one of these three \u000A examples is actually executed; for example, they are not executed in sequence. \u000A See ``A Word of Warning'' below for more about this issue.)</p> \u000A \u000A <code>(cnt 'demo (<see topic=\"ACL2____ASET1\">aset1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 2 'two) 4 (<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 2))  ; [former]\u000A\u000A(<see topic=\"COMMON-LISP____LET\">let</see> ((c (<see topic=\"ACL2____ASET1\">aset1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 2 'two)))                           ; [latter]\u000A  (cnt 'demo c 4 (<see topic=\"ACL2____AREF1\">aref1</see> 'demo c 2)))</code> \u000A \u000A <p>Note that in <v>[former]</v> we create <v>c</v> in the second argument of the \u000A call to <v>cnt</v> (although we do not give it a name) and then refer to <v>b</v> \u000A in the fourth argument.  This is unconventional because the second reference \u000A to <v>b</v> in <v>[former]</v> is no longer the semantic value of <v>demo</v>. \u000A While ACL2 computes the correct answer, namely <v>2</v>, the execution of the \u000A <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> expression in <v>[former]</v> is done slowly.</p> \u000A \u000A <p>A conventional rendering with the same meaning is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 2)))                           ; [fast former]\u000A  (cnt 'demo (<see topic=\"ACL2____ASET1\">aset1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b) 2 'two) 4 x))</code> \u000A \u000A <p>which fetches the 2nd element of <v>b</v> before creating <v>c</v> by \u000A assignment.  It is important to understand that <v>[former]</v> and <v>[fast\u000A former]</v> mean exactly the same thing: both count the number of occurrences of \u000A <v>uninitialized</v> in <v>c</v>.  Both are legal ACL2 and both compute the same \u000A answer, <v>2</v>.  Indeed, we can symbolically transform <v>[fast former]</v> into \u000A <v>[former]</v> merely by substituting the binding of <v>x</v> for <v>x</v> in the \u000A body of the <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>.  But <v>[fast former]</v> can be evaluated faster than \u000A <v>[former]</v> because all of the references to <v>demo</v> use the then-current \u000A semantic value of <v>demo</v>, which is <v>b</v> in the first line and <v>c</v> \u000A throughout the execution of the <v>cnt</v> in the second line.  <v>[Fast\u000A former]</v> is the preferred form, both because of its execution speed and its \u000A clarity.  If you were writing in a conventional language you would have to \u000A write something like <v>[fast former]</v> because there is no way to refer to \u000A the 2nd element of the old value of <v>b</v> after smashing <v>b</v> unless it had \u000A been saved first.</p> \u000A \u000A <p>We turn now to <v>[latter]</v>.  It is both clear and efficient.  It creates \u000A <v>c</v> by assignment to <v>b</v> and then it fetches the 2nd element of <v>c</v>, \u000A <v>two</v>, and proceeds to count the number of occurrences in <v>c</v>.  The \u000A answer is <v>1</v>.  <v>[Latter]</v> is a good example of typical ACL2 array \u000A manipulation: after the assignment to <v>b</v> that creates <v>c</v>, <v>c</v> is \u000A used throughout.</p> \u000A \u000A <p>It takes a while to get used to this because most of us have grown \u000A accustomed to the peculiar semantics of arrays in conventional languages.  For \u000A example, in raw lisp we might have written something like the following, \u000A treating <v>b</v> as a ``global variable'':</p> \u000A \u000A <code>(cnt 'demo (aset 'demo b 2 'two) 4 (aref 'demo b 2))</code> \u000A \u000A <p>which sort of resembles <v>[former]</v> but actually has the semantics of \u000A <v>[latter]</v> because the <v>b</v> from which <v>aref</v> fetches the 2nd element \u000A is not the same <v>b</v> used in the <v>aset</v>!  The array <v>b</v> is destroyed \u000A by the <v>aset</v> and <v>b</v> henceforth refers to the array produced by the \u000A <v>aset</v>, as written more clearly in <v>[latter]</v>.</p> \u000A \u000A <p>A Word of Warning: Users must exercise care when experimenting with \u000A <v>[former]</v>, <v>[latter]</v> and <v>[fast former]</v>.  Suppose you have just \u000A created <v>b</v> with the assignment shown above,</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> b (<see topic=\"ACL2____ASET1\">aset1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 1 'one))</code> \u000A \u000A <p>If you then evaluate <v>[former]</v> in ACL2 it will complain that the <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> is slow and compute the answer, as discussed.  Then suppose you \u000A evaluate <v>[latter]</v> in ACL2.  From our discussion you might expect it to \u000A execute fast &mdash; i.e., issue no complaint.  But in fact you will find that \u000A it complains repeatedly.  The problem is that the evaluation of <v>[former]</v> \u000A changed the semantic value of <v>demo</v> so that it is no longer <v>b</v>.  To \u000A try the experiment correctly you must make <v>b</v> be the semantic value of \u000A <v>demo</v> again before the next example is evaluated.  One way to do that is \u000A to execute</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> b (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo (<see topic=\"ACL2_____04\">@</see> b)))</code> \u000A \u000A <p>before each expression.  Because of issues like this it is often hard to \u000A experiment with ACL2 arrays at the top-level.  We find it easier to write \u000A functions that use arrays correctly and efficiently than to so use them \u000A interactively.</p> \u000A \u000A <p>This last assignment also illustrates a very common use of <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.  While it was introduced as a means of removing irrelevant pairs \u000A from an array built up by repeated assignments, it is actually most useful as \u000A a way of insuring fast access to the elements of an array.</p> \u000A \u000A <p>Many array processing tasks can be divided into two parts.  During the \u000A first part the array is built.  During the second part the array is used \u000A extensively but not modified.  If your <see topic=\"ACL2____PROGRAMMING\">programming</see> task can be so \u000A divided, it might be appropriate to construct the array entirely with list \u000A processing, thereby saving the cost of maintaining the semantic value of the \u000A name while few references are being made.  Once the alist has stabilized, it \u000A might be worthwhile to treat it as an array by calling <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>, \u000A thereby gaining constant time access to it.</p> \u000A \u000A <p>ACL2's theorem prover uses this technique in connection with its \u000A implementation of the notion of whether a <see topic=\"ACL2____RUNE\">rune</see> is <see topic=\"ACL2____DISABLE\">disable</see>d or \u000A not.  Associated with every <see topic=\"ACL2____RUNE\">rune</see> is a unique integer <v>index</v>, called \u000A its ``nume.''  When each rule is stored, the corresponding nume is stored as a \u000A component of the rule.  <see topic=\"ACL2____THEORIES\">Theories</see> are lists of <see topic=\"ACL2____RUNE\">rune</see>s and \u000A membership in the ``current theory'' indicates that the corresponding rule is \u000A <see topic=\"ACL2____ENABLE\">enable</see>d.  But these lists are very long and membership is a linear-time \u000A operation.  So just before a proof begins we map the list of <see topic=\"ACL2____RUNE\">rune</see>s in \u000A the current theory into an alist that pairs the corresponding numes with \u000A <v>t</v>.  Then we compress this alist into an array.  Thus, given a rule we can \u000A obtain its nume (because it is a component) and then determine in constant \u000A time whether it is <see topic=\"ACL2____ENABLE\">enable</see>d.  The array is never modified during the \u000A proof, i.e., <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> is never used in this example.  From the logical \u000A perspective this code looks quite odd: we have replaced a linear-time \u000A membership test with an apparently linear-time <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt> after going to \u000A the trouble of mapping from a list of <see topic=\"ACL2____RUNE\">rune</see>s to an alist of numes.  But \u000A because the alist of numes is an array, the ``apparently linear-time <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt>'' is more apparent than real; the operation is constant-time.</p>"],

 "ACL2____BREAK-REWRITE":[["Debugging"],"ACL2 Sources","ACL2","<p>ACL2 allows the user to <see topic=\"ACL2____MONITOR\">monitor</see> the application of <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____DEFINITION\">definition</see>, and <see topic=\"ACL2____LINEAR\">linear</see> rules.  When <see topic=\"ACL2____MONITOR\">monitor</see>ed \u000A rules are about to be tried by the rewriter, an interactive break occurs and \u000A the user is allowed to watch and, in a limited sense, control the attempt to \u000A apply the rule.  This interactive loop, which is technically just a call of \u000A the standard top-level ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, on a ``<see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>'' (see <see topic=\"ACL2____WORMHOLE\">wormhole</see>), is called ``break-rewrite.'' \u000A While in break-rewrite, certain keyword commands are available for accessing \u000A information about the context in which the lemma is being tried.  These \u000A keywords are called break-rewrite ``commands.''</p> \u000A \u000A <p>Also see <see topic=\"ACL2____DMR\">dmr</see> (Dynamically Monitor Rewrites) for a related utility, \u000A which allows you to watch progress of the rewriter in real time.</p> \u000A \u000A <p>To abort from inside break-rewrite at any time, execute</p> \u000A \u000A <p>For further information, see the related <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> topics listed \u000A below.</p> \u000A \u000A <p>As explained in the documentation for <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt>, it is possible to \u000A cause the ACL2 rewriter to <see topic=\"ACL2____MONITOR\">monitor</see> the attempted application of \u000A selected rules.  When such a rule is about to be tried, the rewriter evaluates \u000A its break condition and if the result is non-<v>nil</v>, break-rewrite is \u000A entered.</p> \u000A \u000A <p>Break-rewrite permits the user to inspect the current <see topic=\"ACL2____STATE\">state</see> by \u000A evaluating break-rewrite commands.  Type <v>:</v><tt><see topic=\"ACL2____HELP\">help</see></tt> in break-rewrite \u000A to see what the break-rewrite commands are.  However, break-rewrite is \u000A actually just a call of the general ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, on \u000A a certain <see topic=\"ACL2____STATE\">state</see> and the break-rewrite commands are simply aliases \u000A provided by <v>ld-keyword-aliases</v> <see topic=\"ACL2____TABLE\">table</see> (see <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see>).  See <see topic=\"ACL2____LD\">ld</see> for details about this read-eval-print \u000A loop.  Thus, with a few exceptions, anything you can do at the ACL2 top-level \u000A can be done within break-rewrite.  For example, you can evaluate arbitrary \u000A expressions, use the keyword command hack, access <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, print \u000A <see topic=\"ACL2____EVENTS\">events</see>, and even define functions and prove theorems.  However, the \u000A ``certain <see topic=\"ACL2____STATE\">state</see>'' upon which <tt><see topic=\"ACL2____LD\">ld</see></tt> was called is a ``<see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>'' (see <see topic=\"ACL2____WORMHOLE\">wormhole</see>) because break-rewrite is not \u000A allowed to have any effect upon the behavior of rewrite.  What this means, \u000A very roughly but understandably, is that break-rewrite operates on a copy of \u000A the <see topic=\"ACL2____STATE\">state</see> being used by rewrite and when break-rewrite exits the <see topic=\"ACL2____WORMHOLE\">wormhole</see> closes and the <see topic=\"ACL2____STATE\">state</see> ``produced'' by break-rewrite \u000A disappears.  Thus, break-rewrite lets you query the state of the rewriter and \u000A even do experiments involving proofs, etc., but these experiments have no \u000A effect on the ongoing proof attempt.  In particular:</p> \u000A \u000A <p>Note that the output from break-rewrite is sometimes abbreviated by \u000A default, such as for the term causing the break.  This can be controlled by \u000A setting the <v>:term</v> evisc-tuple; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.  (Another \u000A option: use iprinting.  See <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.)  But as noted above, if you \u000A use <v>set-evisc-tuple</v> from inside the break-rewrite <see topic=\"ACL2____WORMHOLE\">wormhole</see>, its \u000A effect will disappear when you exit the break.  So you might want to issue a \u000A <v>set-evisc-tuple</v> command from the top level, outside break-rewrite.</p> \u000A \u000A <p>When you first enter break-rewrite a simple herald is printed such as:</p> \u000A \u000A <code>(3 Breaking (:rewrite lemma12) on (delta a (<see topic=\"COMMON-LISP_____B2\">+</see> 1 j)):</code> \u000A \u000A <p>The integer after the open parenthesis indicates the depth of nested \u000A break-rewrite calls.  In this discussion we use <v>3</v> consistently for this \u000A integer.  Unless you abort or somehow enter unbalanced parentheses into the \u000A script, the entire session at a given depth will be enclosed in balanced \u000A parentheses, making it easy to skip over them in Emacs.</p> \u000A \u000A <p>You then will see the break-rewrite <see topic=\"ACL2____PROMPT\">prompt</see>:</p> \u000A \u000A <code>3 ACL2 !&gt;</code> \u000A \u000A <p>The leading integer is, again, the depth.  Because breaks often occur \u000A recursively it is convenient always to know the level with which you are \u000A interacting.</p> \u000A \u000A <p>You may type arbitrary commands as in the top-level ACL2 loop.  For \u000A example, you might type:</p> \u000A \u000A <code>3 ACL2 !&gt;:help</code> \u000A \u000A <p>or</p> \u000A \u000A <code>3 ACL2 !&gt;:pe lemma12</code> \u000A \u000A <p>More likely, upon entering break-rewrite you will determine the context of \u000A the attempted application.  Here are some useful commands:</p> \u000A \u000A <code>3 ACL2 &gt;:target           ; the term being rewritten\u000A3 ACL2 &gt;:unify-subst      ; the unifying substitution\u000A3 ACL2 &gt;:path             ; the stack of goals pursued by the rewriter\u000A                          ; starting at the top-level clause being simplified\u000A                          ; and ending with the current application</code> \u000A \u000A <p>At this point in the interaction the system has not yet tried to apply the \u000A <see topic=\"ACL2____MONITOR\">monitor</see>ed rule.  That is, it has not tried to establish the hypotheses, \u000A considered the heuristic cost of backchaining, rewritten the right-hand side \u000A of the conclusion, etc.  When you are ready for it to try the rule you can \u000A type one of several different ``proceed'' commands.  The basic proceed \u000A commands are <v>:ok</v>, <v>:go</v>, and <v>:eval</v>.</p> \u000A \u000A <code>:ok</code> \u000A \u000A <p>exits break-rewrite without further interaction.  When break-rewrite exits \u000A it prints ``<v>3)</v>'', closing the parenthesis that opened the level <v>3</v> \u000A interaction.</p> \u000A \u000A <code>:go</code> \u000A \u000A <p>exits break-rewrite without further interaction, but prints out the result \u000A of the application attempt, i.e., whether the application succeeded, if so, \u000A what the <v>:target</v> term was rewritten to, and if not why the rule was not \u000A applicable.</p> \u000A \u000A <code>:eval</code> \u000A \u000A <p>causes break-rewrite to attempt to apply the rule but interaction at this \u000A level of break-rewrite resumes when the attempt is complete.  When control \u000A returns to this level of break-rewrite a message indicating the result of the \u000A application attempt (just as in <v>:go</v>) is printed, followed by the <see topic=\"ACL2____PROMPT\">prompt</see> for additional user input.</p> \u000A \u000A <p>Generally speaking, <v>:ok</v> and <v>:go</v> are used when the break in \u000A question is routine or uninteresting and <v>:eval</v> is used when the break is \u000A one that the user anticipates is causing trouble.  For example, if you are \u000A trying to determine why a lemma isn't being applied to a given term and the \u000A <v>:target</v> of the current break-rewrite is the term in question, you would \u000A usually <v>:eval</v> the rule and if break-rewrite reports that the rule failed \u000A then you are in a position to determine why, for example by carefully \u000A inspecting the <v>:</v><tt><see topic=\"ACL2____TYPE-ALIST\">type-alist</see></tt> of governing assumptions or why some \u000A hypothesis of the rule could not be established.</p> \u000A \u000A <p>It is often the case that when you are in break-rewrite you wish to change \u000A the set of <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s.  This can be done by using \u000A <v>:</v><tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> and <v>:</v><tt><see topic=\"ACL2____UNMONITOR\">unmonitor</see></tt> as noted above.  For \u000A example, you might want to <see topic=\"ACL2____MONITOR\">monitor</see> a certain rule, say \u000A <v>hyp-reliever</v>, just when it is being used while attempting to apply \u000A another rule, say <v>main-lemma</v>.  Typically then you would <see topic=\"ACL2____MONITOR\">monitor</see> \u000A <v>main-lemma</v> at the ACL2 top-level, start the proof-attempt, and then in \u000A the break-rewrite in which <v>main-lemma</v> is about to be tried, you would \u000A install a <see topic=\"ACL2____MONITOR\">monitor</see> on <v>hyp-reliever</v>.  If during the ensuing \u000A <v>:eval</v> <v>hyp-reliever</v> is broken you will know it is being used under \u000A the attempt to apply <v>main-lemma</v>.</p> \u000A \u000A <p>However, once <v>hyp-reliever</v> is being <see topic=\"ACL2____MONITOR\">monitor</see>ed it will be <see topic=\"ACL2____MONITOR\">monitor</see>ed even after <v>main-lemma</v> has been tried.  That is, if you let the \u000A proof attempt proceed then you may see many other breaks on <v>hyp-reliever</v>, \u000A breaks that are not ``under'' the attempt to apply <v>main-lemma</v>.  One way \u000A to prevent this is to <v>:eval</v> the application of <v>main-lemma</v> and then \u000A <v>:</v><tt><see topic=\"ACL2____UNMONITOR\">unmonitor</see></tt> <v>hyp-reliever</v> before exiting.  But this case \u000A arises so often that ACL2 supports several additional ``flavors'' of proceed \u000A commands.</p> \u000A \u000A <p><v>:Ok!</v>, <v>:go!</v>, and <v>:eval!</v> are just like their counterparts \u000A (<v>:ok</v>, <v>:go</v>, and <v>:eval</v>, respectively), except that while \u000A processing the rule that is currently broken no <see topic=\"ACL2____RUNE\">rune</see>s are <see topic=\"ACL2____MONITOR\">monitor</see>ed.  When consideration of the current rule is complete, the set of \u000A <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s is restored to its original setting.</p> \u000A \u000A <p><v>:Ok$</v>, <v>:go$</v>, and <v>:eval$</v> are similar but take an additional \u000A argument which must be a list of <see topic=\"ACL2____RUNE\">rune</see>s.  An example usage of \u000A <v>:eval$</v> is</p> \u000A \u000A <code>3 ACL2 !&gt;:eval$ ((:rewrite hyp-reliever))</code> \u000A \u000A <p>These three commands temporarily install unconditional breaks on the <see topic=\"ACL2____RUNE\">rune</see>s listed, proceed with the consideration of the currently broken rule, \u000A and then restore the set of <see topic=\"ACL2____MONITOR\">monitor</see>ed rules to its original \u000A setting.</p> \u000A \u000A <p>Thus, there are nine ways to proceed from the initial entry into \u000A break-rewrite although we often speak as though there are two, <v>:ok</v> and \u000A <v>:eval</v>, and leave the others implicit.  We group <v>:go</v> with <v>:ok</v> \u000A because in all their flavors they exit break-rewrite without further \u000A interaction (at the current level).  All the flavors of <v>:eval</v> require \u000A further interaction after the rule has been tried.</p> \u000A \u000A <p>To abort a proof attempt and return to the top-level of ACL2 you may at any \u000A time type <v>(<see topic=\"ACL2____A_12\">a!</see>)</v> followed by a carriage return.  If you are not in a raw \u000A Lisp break, you may type <v>:a!</v> instead.  The utility <v>p!</v> is completely \u000A analogous to <v>a!</v> except that it pops up only one <tt><see topic=\"ACL2____LD\">ld</see></tt> level.  If you \u000A have just entered the break-rewrite loop, this will pop you out of that loop, \u000A back to the proof.  See <see topic=\"ACL2____A_12\">a!</see> and see <see topic=\"ACL2____P_12\">p!</see>.</p> \u000A \u000A <p>We now address ourselves to the post-<v>:eval</v> interaction with \u000A break-rewrite.  As noted, that interaction begins with break-rewrite's report \u000A on the results of applying the rule: whether it worked and either what it \u000A produced or why it failed.  This information is also printed by certain \u000A keyword commands available after <v>:eval</v>, namely <v>:wonp</v>, \u000A <v>:rewritten-rhs</v> or (for <see topic=\"ACL2____LINEAR\">linear</see> rules) <v>:poly-list</v>, and \u000A <v>:failure-reason</v>.  In addition, by using <tt><see topic=\"ACL2____BRR_04\">brr@</see></tt> you can obtain this \u000A information in the form of ACL2 data objects.  This allows the development of \u000A more sophisticated ``break conditions''; see <see topic=\"ACL2____MONITOR\">monitor</see> for examples.  In \u000A this connection we point out the macro form <v>(<see topic=\"ACL2____OK-IF\">ok-if</see> term)</v>.  See <see topic=\"ACL2____OK-IF\">ok-if</see>.  This command exits break-rewrite if <v>term</v> evaluates to \u000A non-<v>nil</v> and otherwise does not exit.  Thus it is possible to define \u000A macros that provide other kinds of exits from break-rewrite.  The only way to \u000A exit break-rewrite after <v>:eval</v> is <v>:ok</v> (or, equivalently, the use of \u000A <tt><see topic=\"ACL2____OK-IF\">ok-if</see></tt>).</p> \u000A \u000A <p>ACL2 users who wish to know more about break-rewrite so that they can \u000A develop more convenient ways to <see topic=\"ACL2____MONITOR\">monitor</see> rules are encouraged to speak \u000A to J Moore.</p> \u000A \u000A <p>The rest of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> discusses a few implementation \u000A details of break-rewrite and may not be interesting to the typical user.</p> \u000A \u000A <p>There is no ACL2 function named break-rewrite.  It is an illusion created \u000A by appropriate calls to two functions named <v>brkpt1</v> and <v>brkpt2</v>.  As \u000A previously noted, break-rewrite is <tt><see topic=\"ACL2____LD\">ld</see></tt> operating on a <see topic=\"ACL2____WORMHOLE\">wormhole</see> \u000A <see topic=\"ACL2____STATE\">state</see>.  One might therefore wonder how break-rewrite can apply a rule \u000A and then communicate the results back to the rewriter running in the external \u000A <see topic=\"ACL2____STATE\">state</see>.  The answer is that it cannot.  Nothing can be communicated \u000A through a <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  In fact, <v>brkpt1</v> and <v>brkpt2</v> are each \u000A calls of <tt><see topic=\"ACL2____LD\">ld</see></tt> running on <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>s.  <v>Brkpt1</v> \u000A implements the pre-<v>:eval</v> break-rewrite and <v>brkpt2</v> implements the \u000A post-<v>:eval</v> break-rewrite.  The rewriter actually calls <v>brkpt1</v> before \u000A attempting to apply a rule and calls <v>brkpt2</v> afterwards.  In both cases, \u000A the rewriter passes into the <see topic=\"ACL2____WORMHOLE\">wormhole</see> the relevant information about \u000A the current context.  Logically <v>brkpt1</v> and <v>brkpt2</v> are no-ops and \u000A <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> ignores the <v>nil</v> they return.  But while control is in \u000A them, the execution of <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> is suspended and cannot proceed until \u000A the break-rewrite interactions complete.</p> \u000A \u000A <p>This design causes a certain anomoly that might be troubling.  Suppose that \u000A inside break-rewrite before <v>:evaling</v> a rule (i.e., in the <v>brkpt1</v> \u000A <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>) you define some function, <v>foo</v>.  Suppose \u000A then you <v>:eval</v> the rule and eventually control returns to break-rewrite \u000A (i.e., to <v>brkpt2</v> on a <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see> with the results of \u000A the application in it).  You will discover that <v>foo</v> is no longer defined! \u000A That is because the <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see> created during your \u000A <v>pre-:eval</v> interaction is lost when we exit the <see topic=\"ACL2____WORMHOLE\">wormhole</see> to resume \u000A the proof attempt.  The post-<v>:eval</v> <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see> is in \u000A fact identical to the initial pre-<v>:eval</v> <see topic=\"ACL2____STATE\">state</see> (except for the \u000A results of the application) because <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> did not change the \u000A external <see topic=\"ACL2____STATE\">state</see> and both <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>s are copies of it. \u000A A similar issue occurs with the use of <see topic=\"COMMON-LISP____TRACE\">trace</see> utilities: all effects of \u000A calling <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt> are erased when you proceed from a \u000A break in the break-rewrite loop.</p> \u000A \u000A <p>There is a lot more to know about break-rewrite, most of which is fairly \u000A easy to learn from looking at the code, since it is all expressed in ACL2. \u000A Feel free to ask questions of J Moore.</p>"],

 "COMMON-LISP____DOCUMENTATION":[["ACL2"],"ACL2 Sources","ACL2","<p>Here, we explain documentation for ACL2 and its community \u000A  books.</p> \u000A \u000A <p>If you are using a release copy of ACL2, then we recommend that you use a \u000A web browser to view the <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a>.  From that page, \u000A you also have the option of downloading a local copy of that manual (click on \u000A the ``down arrow'' icon at the top-right of that page) or of building a local \u000A copy (see the (@see xdoc) topic).</p> \u000A \u000A <p>Emacs users may prefer to read documentation inside a custom ``ACL2-Doc'' \u000A Emacs browser.  See <see topic=\"ACL2____ACL2-DOC\">ACL2-doc</see>.</p> \u000A \u000A <p>The ACL2 User's Manual is distributed with ACL2.  It contains ACL2 system \u000A documentation but unlike the ACL2+Books Manual, it does not contain \u000A documentation from the books.  A web-based copy is included with the ACL2 \u000A distribution in directory <v>doc/manual/</v>, and you can easily get to it by \u000A opening file <v>doc/home-page.html</v> in your browser.</p> \u000A \u000A <p>You can also read documentation at the terminal, for the ACL2 system and \u000A for documentation from any <v>books</v> that have been loaded into your session, \u000A by typing <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> followed by the topic name, for example</p> \u000A \u000A <code>:doc rewrite</code> \u000A \u000A <p>or</p> \u000A \u000A <code>(<see topic=\"ACL2____DOC\">doc</see> 'rewrite)</code> \u000A \u000A <p>ACL2 users are welcome to contribute to the ACL2 system documentation.  See \u000A community book <tt>books/system/doc/acl2-doc.lisp</tt>.</p> \u000A \u000A <p>The ACL2 system documentation includes topics not only for names of \u000A functions, macros, and constants, but also names of certain important ideas. \u000A For example, see <see topic=\"ACL2____REWRITE\">rewrite</see> and see <see topic=\"ACL2____META\">meta</see> to learn about \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules and <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules, respectively.  See \u000A <see topic=\"ACL2____HINTS\">hints</see> to learn about the structure of the <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> argument \u000A to the prover.</p> \u000A \u000A <p>If you want documentation on an ACL2 function or macro that is not \u000A documented, there are still several alternatives.</p> \u000A \u000A <code>ACL2 !&gt;:args fn</code> \u000A \u000A <p>will print the arguments and some other relevant information about the \u000A named function or macro.  This information is all gleaned from the definition \u000A and hence this is a definitive way to determine if <v>fn</v> is defined as a \u000A function or macro.</p> \u000A \u000A <p>You might also want to type:</p> \u000A \u000A <code>ACL2 !&gt;:pc fn</code> \u000A \u000A <p>which will print the <see topic=\"ACL2____COMMAND\">command</see> that introduced <v>fn</v>.  You should \u000A see <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see> for details on the kinds of input you can give \u000A the <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt> command.</p>"],

 "ACL2____META":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <p>Meta rules extend the ACL2 simplifier with hand-written code to transform \u000A certain terms to equivalent ones.  To add a meta rule, the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula must establish that the hand-written ``metafunction'' \u000A preserves the meaning of the transformed term.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-correct-1                ; Modify the rewriter to use fn to\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)                  ; transform terms that are calls of\u000A         (evl (fn x) a))            ; nth or of foo.\u000A  :rule-classes ((:meta :trigger-fns (<see topic=\"COMMON-LISP____NTH\">nth</see> foo))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-correct-2                ; As above, but this illustrates\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)    ; that without loss of generality we\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> a))         ; may restrict x to be shaped like a\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)         ; term and a to be an alist.\u000A                  (evl (fn x) a)))\u000A  :rule-classes ((:meta :trigger-fns (<see topic=\"COMMON-LISP____NTH\">nth</see> foo))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-correct-3                ; As above (with or without the\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)    ; hypotheses on x and a), with the\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> a)          ; additional restriction that the\u000A                (evl (hyp-fn x) a)) ; meaning of (hyp-fn x) is true in\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)         ; the current context.  That is, the\u000A                  (evl (fn x) a)))  ; applicability of the transformation\u000A  :rule-classes                     ; may be dependent upon some computed\u000A  ((:meta :trigger-fns (<see topic=\"COMMON-LISP____NTH\">nth</see> foo)))) ; hypotheses.</code> \u000A \u000A <p>While our intention is that the set of ACL2 documentation topics is \u000A self-contained, readers might find it useful to see the following paper for an \u000A introduction to meta reasoning in ACL2.</p> \u000A \u000A <blockquote><p>W. A. Hunt, Jr., R. B. Krug, M. Kaufmann, J S. Moore and \u000A E. W. Smith, ``Meta Reasoning in ACL2.''  TPHOLs 2005, ed. J. Hurd and \u000A T. F. Melham, LNCS 3603, Springer-Verlag, Berlin, 2005, \u000A pp. 163-178.</p></blockquote> \u000A \u000A <p>A non-<v>nil</v> list of function symbols must be supplied as the value of \u000A the <v>:trigger-fns</v> field in a <v>:meta</v> rule class object (except that a \u000A macro alias can stand in for a function symbol; see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>).</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)        ; this hyp is optional\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> a)              ; this hyp is optional\u000A              (ev (hyp-fn x ...) a)   ; this hyp is optional\u000A              ; meta-extract hyps may also be included (see below)\u000A              )\u000A         (equiv (ev x a)\u000A                (ev (fn x ...) a)))</code> \u000A \u000A <p>where <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation, <v>x</v> and <v>a</v> \u000A are distinct variable names, and <v>ev</v> is an evaluator function (see below), \u000A and <v>fn</v> is a function symbol, as is <v>hyp-fn</v> when provided.  The \u000A arguments to <v>fn</v> and <v>hyp-fn</v> should be identical.  In the most common \u000A case, both take a single argument, <v>x</v>, which denotes the term to be \u000A simplified.  If <v>fn</v> and/or <v>hyp-fn</v> are <see topic=\"ACL2____GUARD\">guard</see>ed, their <see topic=\"ACL2____GUARD\">guard</see>s should be trivially implied by <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt>.  We say the \u000A theorem above is a ``metatheorem'' or ``metalemma'' and <v>fn</v> is a \u000A ``metafunction'', and <v>hyp-fn</v> is a ``hypothesis metafunction''.</p> \u000A \u000A <p>If ``<v>...</v>'' is empty, i.e., the metafunctions take just one argument, \u000A we say they are ``vanilla flavored.''  If ``<v>...</v>'' is non-empty, we say \u000A the metafunctions are ``extended.''  Extended metafunctions can access <tt><see topic=\"ACL2____STATE\">state</see></tt> and context sensitive information to compute their results, within \u000A certain limits.  We discuss vanilla metafunctions here and recommend a \u000A thorough understanding of them before proceeding (at which time see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>).</p> \u000A \u000A <p>Additional hypotheses are supported, called ``meta-extract hypotheses''. \u000A These allow metafunctions to depend on the validity of certain terms extracted \u000A from the context or the logical <see topic=\"ACL2____WORLD\">world</see>.  These hypotheses provide a \u000A relatively advanced form of metatheorem so we explain them elsewhere; see \u000A <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>.</p> \u000A \u000A <p>One might think that metafunctions and (if supplied) hypothesis \u000A metafunctions must be executable: that is, not constrained (i.e., introduced \u000A in the <see topic=\"ACL2____SIGNATURE\">signature</see> of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>), and not <see topic=\"COMMON-LISP____DECLARE\">declare</see>d <v>:</v><tt><see topic=\"ACL2____NON-EXECUTABLE\">non-executable</see></tt>.  After all, there is no point in \u000A installing a simplifier that cannot be run!  However, such a restriction is \u000A not enforced, because one could introduce a metafunction using <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and then use <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> to attach it to an executable \u000A function; see <see topic=\"ACL2____DEFATTACH\">defattach</see>.</p> \u000A \u000A <p>We defer discussion of the case in which there is a hypothesis metafunction \u000A and for now address the case in which the other two hypotheses are \u000A present.</p> \u000A \u000A <p>In the discussion below, we refer to the argument, <v>x</v>, of <v>fn</v> and \u000A <v>hyp-fn</v> as a ``term.''  When these metafunctions are executed by the \u000A simplifier, they will be applied to (the quotations of) terms.  But during the \u000A proof of the metatheorem itself, <v>x</v> may not be the quotation of a term. \u000A If the <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> hypothesis is omitted, <v>x</v> may be any object. \u000A Even with the <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> hypothesis, <v>x</v> may merely ``look like a \u000A term'' but use non-function symbols or function symbols of incorrect arity. \u000A In any case, the metatheorem is stronger than necessary to allow us to apply \u000A the metafunctions to terms, as we do in the discussion below.  We return later \u000A to the question of proving the metatheorem.</p> \u000A \u000A <p>Suppose the general form of the metatheorem above is proved with the <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> and <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt> hypotheses.  Then when the simplifier \u000A encounters a term, <v>(h t1 ... tn)</v>, that begins with a function symbol, \u000A <v>h</v>, listed in <v>:trigger-fns</v>, it applies the metafunction, <v>fn</v>, to \u000A the quotation of the term, i.e., it evaluates <v>(fn '(h t1 ... tn))</v> to \u000A obtain some result, which can be written as <v>'val</v>.  If <v>'val</v> is \u000A different from <v>'(h t1 ... tn)</v> and <v>val</v> is a term, then <v>(h t1\u000A ... tn)</v> is replaced by <v>val</v>, which is then passed along for further \u000A rewriting.  Because the metatheorem establishes the correctness of <v>fn</v> for \u000A all terms (even non-terms!), there is no restriction on which function symbols \u000A are listed in the <v>:trigger-fns</v>.  Generally, of course, they should be the \u000A symbols that head up the terms simplified by the metafunction <v>fn</v>.  See \u000A <see topic=\"ACL2____TERM-TABLE\">term-table</see> for how one obtains some assistance towards guaranteeing \u000A that <v>val</v> is indeed a term.</p> \u000A \u000A <p>The ``evaluator'' function, <v>ev</v>, is a function that can evaluate a \u000A certain class of expressions, namely, all of those composed of variables, \u000A constants, and applications of a fixed, finite set of function symbols, \u000A <v>g1</v>, ..., <v>gk</v>.  Generally speaking, the set of function symbols \u000A handled by <v>ev</v> is chosen to be exactly the function symbols recognized and \u000A manipulated by the metafunctions being introduced.  For example, if <v>fn</v> \u000A manipulates expressions in which <v>'</v><tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> and <v>'</v><tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> occur as function symbols, then <v>ev</v> is generally specified \u000A to handle <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> and <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>.  The actual requirements on \u000A <v>ev</v> become clear when the metatheorem is proved.  The standard way to \u000A introduce an evaluator is to use the ACL2 macro <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt>, though \u000A this is not strictly necessary.  See <see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> if you want \u000A details.</p> \u000A \u000A <p>[Aside for the logic-minded.] Why are we justified in using metafunctions \u000A this way?  Suppose <v>(fn 'term1)</v> is <v>'term2</v>.  What justifies replacing \u000A <v>term1</v> by <v>term2</v>?  The first step is to assert that <v>term1</v> is \u000A <v>(ev 'term1 a)</v>, where <v>a</v> is an alist that maps <v>'var</v> to <v>var</v>, \u000A for each variable <v>var</v> in <v>term1</v>.  This step is incorrect, because \u000A <v>'term1</v> may contain function symbols other than the ones, <v>g1</v>, ..., \u000A <v>gk</v>, that <v>ev</v> knows how to handle.  But we can grow <v>ev</v> to a \u000A ``larger'' evaluator, <v>ev*</v>, an evaluator for all of the symbols that occur \u000A in <v>term1</v> or <v>term2</v>.  We can prove that <v>ev*</v> satisfies the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s on <v>ev</v>, provided no <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> events are adding \u000A constraints to <v>ev</v> (or callers of <v>ev</v>, and recursively); ACL2 checks \u000A this additional property.  Hence, the metatheorem holds for <v>ev*</v> in place \u000A of <v>ev</v>, by functional instantiation.  We can then carry out the proof of \u000A the <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> of <v>term1</v> and <v>term2</v> as follows: Fix <v>a</v> to \u000A be an alist that maps the quotations of the variables of <v>term1</v> and \u000A <v>term2</v> to themselves.  Then,</p> \u000A \u000A <code>term1 = (ev* 'term1 a)      ; (1) by construction of ev* and a\u000A      = (ev* (fn 'term1) a) ; (2) by the metatheorem for ev*\u000A      = (ev* 'term2 a)      ; (3) by evaluation of fn\u000A      = term2               ; (4) by construction of ev* and a</code> \u000A \u000A <p>Note that in line (2) above, where we appeal to the (functional \u000A instantiation of the) metatheorem, we can relieve its (optional) <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> and <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt> hypotheses by appealing to the facts that \u000A <v>term1</v> is a term and <v>a</v> is an alist by construction.  [End of Aside \u000A for the logic-minded.]</p> \u000A \u000A <p>There are subtleties related to the notion of ``growing'' <v>ev</v> to a \u000A ``larger'' evaluator, as mentioned in the paragraph just above.  For \u000A corresponding restrictions on <v>:meta</v> rules, see <see topic=\"ACL2____EVALUATOR-RESTRICTIONS\">evaluator-restrictions</see>.</p> \u000A \u000A <p>Finally, we turn to the second case, in which there is a hypothesis \u000A metafunction.  In that case, consider as before what happens when the \u000A simplifier encounters a term, <v>(h t1 ... tn)</v>, where <v>h</v> is listed in \u000A <v>:trigger-fns</v>.  This time, after it applies <v>fn</v> to <v>'(h t1 ... tn)</v> \u000A to obtain the quotation of some new term, <v>'val</v>, it then applies the \u000A hypothesis metafunction, <v>hyp-fn</v>.  That is, it evaluates <v>(hyp-fn '(h t1\u000A ... tn))</v> to obtain some result, which can be written as <v>'hyp-val</v>.  If \u000A <v>hyp-val</v> is not in fact a term, the metafunction is not used.  Provided \u000A <v>hyp-val</v> is a term, the simplifier attempts to establish (by conventional \u000A backchaining) that this term is non-<v>nil</v> in the current context.  If this \u000A attempt fails, then the meta rule is not applied.  Otherwise, <v>(h t1...tn)</v> \u000A is replaced by <v>val</v> as in the previous case (where there was no hypothesis \u000A metafunction).</p> \u000A \u000A <p>Why is it justified to make this extension to the case of hypothesis \u000A metafunctions?  First, note that the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> a)\u000A              (ev (hyp-fn x) a))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a)\u000A                (ev (fn x) a)))</code> \u000A \u000A <p>is logically equivalent to the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> a))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a)\u000A                (ev (new-fn x) a)))</code> \u000A \u000A <p>where <v>(new-fn x)</v> is defined to be <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 'if (hyp-fn x) (fn x) x)</v>. \u000A (If we're careful, we realize that this argument depends on making an \u000A extension of <v>ev</v> to an evaluator <v>ev*</v> that handles <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> and the \u000A functions manipulated by <v>hyp-fn</v>.)  If we write <v>'term</v> for the \u000A quotation of the present term, and if <v>(hyp-fn 'term)</v> and <v>(fn 'term)</v> \u000A are both terms, say <v>hyp1</v> and <v>term1</v>, then by the previous argument we \u000A know it is sound to rewrite term to <v>(<see topic=\"COMMON-LISP____IF\">if</see> hyp1 term1 term)</v>.  But since we \u000A have established in the current context that <v>hyp1</v> is non-<v>nil</v>, we may \u000A simplify <v>(<see topic=\"COMMON-LISP____IF\">if</see> hyp1 term1 term)</v> to <v>term1</v>, as desired.</p> \u000A \u000A <p>We now discuss the role of the <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> hypothesis. \u000A <v>(<see topic=\"ACL2____PSEUDO-TERMP\">Pseudo-termp</see> x)</v> checks that <v>x</v> has the shape of a term.  Roughly \u000A speaking, it ensures that <v>x</v> is a symbol, a quoted constant, or a true \u000A list consisting of a <v>lambda</v> expression or symbol followed by some \u000A pseudo-terms.  Among the properties of terms not checked by <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> are that variable symbols never begin with ampersand, \u000A <v>lambda</v> expressions are closed, and function symbols are applied to the \u000A correct number of arguments.  See <see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see>.</p> \u000A \u000A <p>There are two possible roles for <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> in the development of \u000A a metatheorem: it may be used as the <see topic=\"ACL2____GUARD\">guard</see> of the metafunction and/or \u000A hypothesis metafunction and it may be used as a hypothesis of the metatheorem. \u000A Generally speaking, the <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> hypothesis is included in a \u000A metatheorem only if it makes it easier to prove.  The choice is yours.  (An \u000A extreme example of this is when the metatheorem is invalid without the \u000A hypothesis!)  We therefore address ourselves the question: should a \u000A metafunction have a <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> <see topic=\"ACL2____GUARD\">guard</see>?  A <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> \u000A <see topic=\"ACL2____GUARD\">guard</see> for a metafunction, in connection with other considerations \u000A described below, improves the efficiency with which the metafunction is used \u000A by the simplifier.</p> \u000A \u000A <p>To make a metafunction maximally efficient you should (a) provide it with a \u000A <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> <see topic=\"ACL2____GUARD\">guard</see> and exploit the <see topic=\"ACL2____GUARD\">guard</see> when possible \u000A in coding the body of the function (see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>, \u000A especially the section on efficiency issues), (b) verify the <see topic=\"ACL2____GUARD\">guard</see>s of \u000A the metafunction (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>), and (c) compile the metafunction \u000A (see <see topic=\"ACL2____COMP\">comp</see>).  When these three steps have been taken the simplifier can \u000A evaluate <v>(fn 'term1)</v> by running the compiled ``primary code'' (see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>) for <v>fn</v> directly in Common Lisp.  (Note however \u000A that explicit compilation may be suppressed; see <see topic=\"ACL2____COMPILATION\">compilation</see>.)</p> \u000A \u000A <p>Before discussing efficiency issues further, let us review for a moment the \u000A general case in which we wish to evaluate <v>(fn 'obj)</v> for some <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> function.  We must first ask whether the <see topic=\"ACL2____GUARD\">guard</see>s of <v>fn</v> have \u000A been verified.  If not, we must evaluate <v>fn</v> by executing its logic \u000A definition.  This effectively checks the <see topic=\"ACL2____GUARD\">guard</see>s of every subroutine and \u000A so can be slow.  If, on the other hand, the <see topic=\"ACL2____GUARD\">guard</see>s of <v>fn</v> have been \u000A verified, then we can run the primary code for <v>fn</v>, provided <v>'obj</v> \u000A satisfies the <see topic=\"ACL2____GUARD\">guard</see> of <v>fn</v>.  So we must next evaluate the <see topic=\"ACL2____GUARD\">guard</see> of <v>fn</v> on <v>'obj</v>.  If the <see topic=\"ACL2____GUARD\">guard</see> is met, then we run the \u000A primary code for <v>fn</v>, otherwise we run the logic code.</p> \u000A \u000A <p>Now in the case of a metafunction for which the three steps above have been \u000A followed, we know the <see topic=\"ACL2____GUARD\">guard</see> is (implied by) <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> and \u000A that it has been verified.  Furthermore, we know without checking that the \u000A <see topic=\"ACL2____GUARD\">guard</see> is met (because <v>term1</v> is a term and hence <v>'term1</v> is a \u000A <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt>).  Hence, we can use the compiled primary code \u000A directly.</p> \u000A \u000A <p>We strongly recommend that you compile your metafunctions, as well as all \u000A their subroutines (unless explicit compilation is suppressed; see <see topic=\"ACL2____COMPILATION\">compilation</see>).  Guard verification is also recommended.</p> \u000A \u000A <p>Finally, we present a very simple example of the use of <v>:meta</v> rules, \u000A based on one provided by Robert Krug.  This example illustrates a trick for \u000A avoiding undesired rewriting after applying a metafunction or any other form \u000A of rewriting.  To elaborate: in general, the term <v>t2</v> obtained by applying \u000A a metafunction to a term <v>t1</v> is then handed immediately to the rewriter, \u000A which descends recursively through the arguments of function calls to rewrite \u000A <v>t2</v> completely.  But if <v>t2</v> shares a lot of structure with <v>t1</v>, \u000A then it might not be worthwhile to rewrite <v>t2</v> immediately.  (A rewrite of \u000A <v>t2</v> will occur anyhow the next time a goal is generated.)  The trick \u000A involves avoiding this rewrite by wrapping <v>t2</v> inside a call of <tt><see topic=\"ACL2____HIDE\">hide</see></tt>, which in turn is inside a call of a user-defined ``unhiding'' function, \u000A <v>unhide</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> unhide (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  x)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> unhide-hide\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (unhide (<see topic=\"ACL2____HIDE\">hide</see> x))\u000A         x)\u000A  :hints ((&quot;Goal&quot; :expand ((<see topic=\"ACL2____HIDE\">hide</see> x)))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> unhide))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-plus (x y)\u000A  (<see topic=\"COMMON-LISP_____B2\">+</see> x y))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> my-plus))\u000A\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A  ((my-plus x y)\u000A   (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y)\u000A   (unhide x)\u000A   (<see topic=\"ACL2____HIDE\">hide</see> x)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> meta-fn (<see topic=\"ACL2____TERM\">term</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> term)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> term)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> term) 3)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> term) 'my-plus))\u000A      `(UNHIDE (<see topic=\"ACL2____HIDE\">HIDE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> term) ,(<see topic=\"COMMON-LISP____CADDR\">caddr</see> term))))\u000A    term))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> my-meta-lemma\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl term a)\u000A         (evl (meta-fn term) a))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> my-plus)))\u000A  :rule-classes ((:meta :trigger-fns (my-plus))))</code> \u000A \u000A <p>Notice that in the following (silly) conjecture, ACL2 initially does only \u000A does the simplification directed by the metafunction; a second goal is \u000A generated before the commuativity of addition can be applied.  If the above \u000A calls of <v>UNHIDE</v> and <v>HIDE</v> had been stripped off, then <v>Goal'</v> \u000A would have been the term printed in <v>Goal''</v> below.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see>\u000A        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (my-plus b a)\u000A               ccc))\u000A\u000AThis simplifies, using the :meta rule MY-META-LEMMA and the :rewrite\u000Arule UNHIDE-HIDE, to\u000A\u000AGoal'\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> B A) CCC).\u000A\u000AThis simplifies, using the :rewrite rule COMMUTATIVITY-OF-+, to\u000A\u000AGoal''\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> A B) CCC).</code> \u000A \u000A <p>The discussion above probably suffices to make good use of this <v>(UNHIDE\u000A (<see topic=\"ACL2____HIDE\">HIDE</see> ...))</v> trick.  However, we invite the reader who wishes to understand \u000A the trick in depth to evaluate the following form before submitting the <tt><see topic=\"ACL2____THM\">thm</see></tt> form above.</p> \u000A \u000A <code>(<see topic=\"ACL2____TRACE_42\">trace$</see> (<see topic=\"ACL2____REWRITE\">rewrite</see> :entry (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"ACL2____TAKE\">take</see> 2 arglist))\u000A                 :exit (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> values)))\u000A        (rewrite-with-lemma :entry (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"ACL2____TAKE\">take</see> 2 arglist))\u000A                            :exit (<see topic=\"ACL2____TAKE\">take</see> 2 values)))</code> \u000A \u000A <p>The following annotated subset of the trace output (which may appear a bit \u000A different depending on the underlying Common Lisp implementation) explains how \u000A the trick works.</p> \u000A \u000A <code>    2&gt; (<see topic=\"ACL2____REWRITE\">REWRITE</see> ((MY-PLUS B A) NIL))&gt;\u000A      3&gt; (REWRITE-WITH-LEMMA\u000A              ((MY-PLUS B A)\u000A               (REWRITE-RULE (:META MY-META-LEMMA)\u000A                             1822\u000A                             NIL EQUAL META-FN NIL META NIL NIL)))&gt;\u000A\u000AWe apply the meta rule, then recursively rewrite the result, which is the\u000A(UNHIDE (<see topic=\"ACL2____HIDE\">HIDE</see> ...)) term shown just below.\u000A\u000A        4&gt; (<see topic=\"ACL2____REWRITE\">REWRITE</see> ((UNHIDE (<see topic=\"ACL2____HIDE\">HIDE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A)))\u000A                     ((A . A) (B . B))))&gt;\u000A          5&gt; (<see topic=\"ACL2____REWRITE\">REWRITE</see> ((<see topic=\"ACL2____HIDE\">HIDE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))\u000A                       ((A . A) (B . B))))&gt;\u000A\u000AThe HIDE protects its argument from being touched by the rewriter.\u000A\u000A          &lt;5 (<see topic=\"ACL2____REWRITE\">REWRITE</see> (<see topic=\"ACL2____HIDE\">HIDE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A)))&gt;\u000A          5&gt; (REWRITE-WITH-LEMMA\u000A                  ((UNHIDE (<see topic=\"ACL2____HIDE\">HIDE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A)))\u000A                   (REWRITE-RULE (:REWRITE UNHIDE-HIDE)\u000A                                 1806 NIL EQUAL (UNHIDE (<see topic=\"ACL2____HIDE\">HIDE</see> X))\u000A                                 X ABBREVIATION NIL NIL)))&gt;\u000A\u000ANow we apply UNHIDE-HIDE, then recursively rewrite its right-hand\u000Aside in an environment where X is bound to (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A).\u000A\u000A            6&gt; (<see topic=\"ACL2____REWRITE\">REWRITE</see> (X ((X BINARY-+ B A))))&gt;\u000A\u000ANotice that at this point X is cached, so REWRITE just returns\u000A(<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A).\u000A\u000A            &lt;6 (<see topic=\"ACL2____REWRITE\">REWRITE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))&gt;\u000A          &lt;5 (REWRITE-WITH-LEMMA T (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))&gt;\u000A        &lt;4 (<see topic=\"ACL2____REWRITE\">REWRITE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))&gt;\u000A      &lt;3 (REWRITE-WITH-LEMMA T (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))&gt;\u000A    &lt;2 (<see topic=\"ACL2____REWRITE\">REWRITE</see> (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> B A))&gt;</code>"],

 "ACL2____SIGNATURE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>We start with a gentle introduction to signatures, where we pretend \u000A that there are no single-threaded objects (more on that below &mdash; for now, \u000A if you don't know anything about single-threaded objects, that's fine!).  Here \u000A are some simple examples of signatures.</p> \u000A \u000A <code>((hd *) =&gt; *)\u000A((pair * *) =&gt; *)\u000A((foo * *) =&gt; (<see topic=\"ACL2____MV\">mv</see> * * *))</code> \u000A \u000A <p>The first of these says that <v>hd</v> is a function of one argument, while \u000A the other two say that <v>pair</v> and <v>foo</v> are functions that each take two \u000A arguments.  The first two say that <v>hd</v> and <v>pair</v> return a single \u000A value.  The third says that <v>foo</v> returns three values, much as the \u000A following definition returns three values:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x y)\u000A  (<see topic=\"ACL2____MV\">mv</see> y x (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)))</code> \u000A \u000A <p>Corresponding ``old-style'' signatures are as follows.  In each case, a \u000A function symbol is followed by a list of formal parameters and then either \u000A <v>t</v>, to denote a single value return, or <v>(<see topic=\"ACL2____MV\">mv</see> t t t)</v>, to denote a \u000A multiple value return (in this case, returning three values).</p> \u000A \u000A <code>(hd (x) t)\u000A(pair (x y) t)\u000A(foo (x y) (<see topic=\"ACL2____MV\">mv</see> t t t))</code> \u000A \u000A <p>That concludes our gentle introduction.  The documentation below is more \u000A general, for example covering single-threaded objects and keyword values such \u000A as <v>:guard</v>.  When reading what follows below, it is sufficient to know \u000A about single-threaded objects (or ``stobjs'') that each has a unique symbolic \u000A name and that <tt><see topic=\"ACL2____STATE\">state</see></tt> is the name of the only built-in single-threaded \u000A object.  All other stobjs are introduced by the user via <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> or \u000A <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>.  An object that is not a single-threaded object is said \u000A to be ``ordinary.''  For a discussion of single-threaded objects, see <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <code>Examples:\u000A((hd *) =&gt; *)\u000A((hd *) =&gt; * :formals (x) :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))\u000A((printer * state) =&gt; (<see topic=\"ACL2____MV\">mv</see> * * state))\u000A((mach * mach-state * state) =&gt; (<see topic=\"ACL2____MV\">mv</see> * mach-state))\u000A\u000AGeneral Form:\u000A((fn ...) =&gt; *)\u000A((fn ...) =&gt; stobj)\u000Aor\u000A((fn ...) =&gt; (<see topic=\"ACL2____MV\">mv</see> ...))\u000Aor for part1 and part2 as above,\u000A(part1 =&gt; part2 :kwd1 val1 ... :kwdn valn)</code> \u000A \u000A <p>where <v>fn</v> is the constrained function symbol, <v>...</v> is a list of \u000A asterisks and/or the names of single-threaded objects, <v>stobj</v> is a \u000A single-threaded object name, and the optional <v>:kwdi</v> and <v>:vali</v> are as \u000A described below.  ACL2 also supports an older style of signature, described \u000A below after we describe the preferred style.</p> \u000A \u000A <p>Signatures specify three syntactic aspects of a function symbol: (1) the \u000A ``arity'' or how many arguments the function takes, (2) the ``multiplicity'' \u000A or how many results it returns via <v>MV</v>, and (3) which of those arguments \u000A and results are single-threaded objects and which objects they are.</p> \u000A \u000A <p>A signature typically has the form <v>((fn x1 ... xn) =&gt; val)</v>.  Such a \u000A signature has two parts, separated by the symbol ``=&gt;''.  The first part, \u000A <v>(fn x1 ... xn)</v>, is suggestive of a call of the constrained function.  The \u000A number of ``arguments,'' <v>n</v>, indicates the arity of <v>fn</v>.  Each <v>xi</v> \u000A must be a symbol.  If a given <v>xi</v> is the symbol ``*'' then the \u000A corresponding argument must be ordinary.  If a given <v>xi</v> is any other \u000A symbol, that symbol must be the name of a single-threaded object and the \u000A corresponding argument must be that object.  No stobj name may occur twice \u000A among the <v>xi</v>.</p> \u000A \u000A <p>The second part, <v>val</v>, of a signature is suggestive of a term and \u000A indicates the ``shape'' of the output of <v>fn</v>.  If <v>val</v> is a symbol \u000A then it must be either the symbol ``*'' or the name of a single-threaded \u000A object.  In either case, the multiplicity of <v>fn</v> is 1 and <v>val</v> \u000A indicates whether the result is ordinary or a stobj.  Otherwise, <v>val</v> is \u000A of the form <v>(<see topic=\"ACL2____MV\">mv</see> y1 ... yk)</v>, where <v>k</v> &gt; 1.  Each <v>yi</v> must be \u000A either the symbol ``*'' or the name of a stobj.  Such a <v>val</v> indicates \u000A that <v>fn</v> has multiplicity <v>k</v> and the <v>yi</v> indicate which results \u000A are ordinary and which are stobjs.  No stobj name may occur twice among the \u000A <v>yi</v>, and a stobj name may appear in <v>val</v> only if appears among the \u000A <v>xi</v>.</p> \u000A \u000A <p>A signature may have the form <v>((fn x1 ... xn) =&gt; val . k)</v>, where \u000A <v>k</v> is a <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt>, i.e., an alternating list of keywords \u000A and values starting with a keyword.  In this case <v>((fn x1 ... xn) =&gt; val)</v> \u000A must be a legal signature as described above.  The legal keywords in <v>k</v> \u000A are <v>:GUARD</v> and <v>:FORMALS</v> (except that for ACL2(r), also see the \u000A remark about <v>:CLASSICALP</v> later in this topic).  The value following \u000A <v>:FORMALS</v> is to be the list of formal parameters of <v>fn</v>, while the \u000A value following <v>:GUARD</v> is a term that is to be the <see topic=\"ACL2____GUARD\">guard</see> of \u000A <v>fn</v>.  Note that this guard is never actually evaluated, and is not subject \u000A to the guard verification performed on functions introduced by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>).  Said differently: this guard need not itself have \u000A a guard of <v>t</v>.  Indeed, the guard is only used for attachments; see <see topic=\"ACL2____DEFATTACH\">defattach</see>.  Note that if <v>:GUARD</v> is supplied then <v>:FORMALS</v> must also \u000A be supplied (in order to related the variables occurring in the guard to the \u000A parameters of <v>fn</v>).  One final observation about guards: if the \u000A <v>:GUARD</v> keyword is omitted, then the guard defaults to <v>T</v>.</p> \u000A \u000A <p>Before ACL2 supported user-declared single-threaded objects there was only \u000A one single-threaded object: ACL2's built-in notion of <tt><see topic=\"ACL2____STATE\">state</see></tt>.  The \u000A notion of signature supported then gave a special role to the symbol \u000A <v>state</v> and all other symbols were considered to denote ordinary objects. \u000A ACL2 still supports the old form of signature, but it is limited to functions \u000A that operate on ordinary objects or ordinary objects and <v>state</v>.</p> \u000A \u000A <code>Old-Style General Form:\u000A(fn formals result . k)</code> \u000A \u000A <p>where <v>fn</v> is the constrained function symbol, <v>formals</v> is a \u000A suitable list of formal parameters for it, <v>k</v> is an optional <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt> (see below), and <v>result</v> is either a symbol denoting \u000A that the function returns one result or else <v>result</v> is an <tt><see topic=\"ACL2____MV\">mv</see></tt> \u000A expression, <v>(<see topic=\"ACL2____MV\">mv</see> s1 ... sn)</v>, where <v>n&gt;1</v>, each <v>si</v> is a symbol, \u000A indicating that the function returns <v>n</v> results.  At most one of the \u000A formals may be the symbol <v>STATE</v>, indicating that corresponding argument \u000A must be ACL2's built-in <tt><see topic=\"ACL2____STATE\">state</see></tt>.  If <v>state</v> appears in <v>formals</v> \u000A then <v>state</v> may appear once in <v>result</v>.  All ``variable symbols'' \u000A other than <v>state</v> in old style signatures denote ordinary objects, \u000A regardless of whether the symbol has been defined to be a single-threaded \u000A object name!</p> \u000A \u000A <p>The optional <v>k</v> is as described above for newer-style signatures, \u000A except that the user is also allowed to declare which symbols (besides \u000A <v>state</v>) are to be considered single-threaded object names.  Thus \u000A <v>:STOBJS</v> is also a legal keyword.  The form</p> \u000A \u000A <code>(fn formals result ... :stobjs names ...)</code> \u000A \u000A <p>specifies that <v>names</v> is either the name of a single-threaded object or \u000A else is a list of such names.  Every name in <v>names</v> must have been \u000A previously defined as a stobj via <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> or <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>.</p> \u000A \u000A <p>As promised above, we conclude with a remark about an additional keyword, \u000A <v>:CLASSICALP</v>, that is legal for ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).  The value of \u000A this keyword must be <v>t</v> (the default) or <v>nil</v>, indicating respectively \u000A whether <v>fn</v> is classical or not.</p>"],

 "ACL2____TRACE_42":[["Trace"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> foo bar)     ; trace foo and bar\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see>)             ; return current trace info (no new tracing specified)\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> (foo :entry  ; trace foo, printing first actual parameter upon entry\u000A             (<see topic=\"COMMON-LISP____CAR\">car</see> arglist)))\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> (foo :exit   ; trace foo, using fmt to print upon exit\u000A             (:fmt (<see topic=\"ACL2____MSG\">msg</see> &quot;Exiting FOO with ~x0&quot;\u000A                        value))))\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> (foo :native t))\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> spec1 spec2 ... specn) ; n &gt;= 1\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see>)</code> \u000A \u000A <p>where the <v>speci</v> are trace specs, as described below.</p> \u000A \u000A <p><v>Trace$</v> installs alternate code for the indicated functions that prints \u000A information upon entry to, and exit from, calls of the functions.  For an \u000A alternate tracing utility used for educational purposes in <a \u000A href='http://acl2s.ccs.neu.edu/acl2s/doc/'>ACL2s</a>, see community book \u000A <v>books/misc/trace-star.lisp</v>.</p> \u000A \u000A <p>From a logical perspective all trace printing is a fiction.  (But see <see topic=\"ACL2____TRACE_12\">trace!</see>  for a way to get around this and modify <see topic=\"ACL2____STATE\">state</see>.)  For a related \u000A fiction, see <see topic=\"ACL2____CW\">cw</see>.  <v>(<see topic=\"ACL2____TRACE_42\">Trace$</see>)</v> returns the list of currently-active \u000A trace specs, while the application of <v>trace$</v> to at least one argument \u000A returns the list of its arguments that are successfully acted on.</p> \u000A \u000A <p>Output from <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> normally goes to the screen, i.e., to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  But it can be redirected to a file; see <see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____UNTRACE_42\">untrace$</see> for how to undo the effect of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  Also see \u000A <see topic=\"COMMON-LISP____TRACE\">trace</see> for mention of modifications made to raw Lisp trace, which is \u000A accessible (as described below) using the <v>:native</v> keyword.</p> \u000A \u000A <p>Note that when <v>trace$</v> is applied to a function without option \u000A <v>:native</v>, that function's declarations and documentation are \u000A discarded.</p> \u000A \u000A <p>Next, we introduce tracing with some examples.  After that, we provide \u000A reference documentation for individual trace options allowed in a trace spec. \u000A Note that although our example focuses on user-defined functions, <v>trace$</v> \u000A can also be applied to built-in functions, though perhaps only system hackers \u000A should take advantage of this observation.</p> \u000A \u000A <p>We begin by illustrating the simplest sort of trace spec: a function \u000A symbol.  For example, the form <v>(<see topic=\"ACL2____TRACE_42\">trace$</see> foo bar)</v> directs the tracing of \u000A functions <v>foo</v> and <v>bar</v> by virtue of the two trace specs <v>foo</v> and \u000A <v>bar</v>.  We can see tracing in action by first defining:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (f x) 3))</code> \u000A \u000A <p>The following log then illustrates tracing of these two functions.  Notice \u000A that before <see topic=\"ACL2____GUARD\">guard</see>s have been verified, the so-called ``*1*'' functions \u000A (sometimes called ``executable counterpart functions'' or ``logic functions'') \u000A are called but the corresponding raw Lisp functions are not; but after guard \u000A verification of <v>f</v>, the raw Lisp counterpart of <v>f</v> is indeed called. \u000A (See <see topic=\"ACL2____GUARD\">guard</see> and see <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-LOG\">guard-evaluation-examples-log</see>.)</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> f g)\u000A ((F) (G))\u000AACL2 !&gt;(g 7)\u000A1&gt; (ACL2_*1*_ACL2::G 7)\u000A  2&gt; (ACL2_*1*_ACL2::F 7)\u000A  &lt;2 (ACL2_*1*_ACL2::F (7 . 7))\u000A&lt;1 (ACL2_*1*_ACL2::G ((7 . 7) 3))\u000A((7 . 7) 3)\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> f)\u000A\u000AComputing the guard conjecture for F....\u000A\u000AThe guard conjecture for F is trivial to prove.  F is compliant with\u000ACommon Lisp.\u000A\u000ASummary\u000AForm:  ( VERIFY-GUARDS F)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A F\u000AACL2 !&gt;(g 7)\u000A1&gt; (ACL2_*1*_ACL2::G 7)\u000A  2&gt; (ACL2_*1*_ACL2::F 7)\u000A    3&gt; (F 7)\u000A    &lt;3 (F (7 . 7))\u000A  &lt;2 (ACL2_*1*_ACL2::F (7 . 7))\u000A&lt;1 (ACL2_*1*_ACL2::G ((7 . 7) 3))\u000A((7 . 7) 3)\u000AACL2 !&gt;</code> \u000A \u000A <p>The following example introduces trace specs other than function symbols. \u000A Consider the following definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> n)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      1\u000A    (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>The following log illustrates the use of trace options <v>:cond</v> \u000A (condition for entering trace), <v>:entry</v> (what to print on entry), and \u000A <v>:exit</v> (what to print on exit).  The reason for two calls on argument 4 is \u000A that we are seeing such calls for the executable counterpart of <v>fact</v> and \u000A also its raw Lisp function.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> (fact :cond (<see topic=\"COMMON-LISP____EVENP\">evenp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> arglist))\u000A                     :entry (<see topic=\"COMMON-LISP____CONS\">cons</see> 'factorial-call arglist)\u000A                     :exit (<see topic=\"COMMON-LISP____CAR\">car</see> values)))\u000A ((FACT :COND (<see topic=\"COMMON-LISP____EVENP\">EVENP</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> ARGLIST))\u000A        :ENTRY (<see topic=\"COMMON-LISP____CONS\">CONS</see> 'FACTORIAL-CALL ARGLIST)\u000A        :EXIT (<see topic=\"COMMON-LISP____CAR\">CAR</see> VALUES)))\u000AACL2 !&gt;(fact 4)\u000A1&gt; (FACTORIAL-CALL 4)\u000A  2&gt; (FACTORIAL-CALL 4)\u000A    3&gt; (FACTORIAL-CALL 2)\u000A      4&gt; (FACTORIAL-CALL 0)\u000A      &lt;4 1\u000A    &lt;3 2\u000A  &lt;2 24\u000A&lt;1 24\u000A24\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that <v>VALUES</v> above is the list of all values returned, which is \u000A a one-element list unless <tt><see topic=\"ACL2____MV\">mv</see></tt> return is used, as illustrated in the \u000A following example, after defining: <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> two-vals (x) (<see topic=\"ACL2____MV\">mv</see> x 7))</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> two-vals)\u000A ((TWO-VALS))\u000AACL2 !&gt;(two-vals 3)\u000A1&gt; (ACL2_*1*_ACL2::TWO-VALS 3)\u000A&lt;1 (ACL2_*1*_ACL2::TWO-VALS 3 7)\u000A(3 7)\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> two-vals)\u000A\u000AComputing the guard conjecture for TWO-VALS....\u000A\u000AThe guard conjecture for TWO-VALS is trivial to prove, given the :executable-\u000Acounterpart of CONS.  TWO-VALS is compliant with Common Lisp.\u000A\u000ASummary\u000AForm:  ( VERIFY-GUARDS TWO-VALS)\u000ARules: ((:EXECUTABLE-COUNTERPART CONS))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A TWO-VALS\u000AACL2 !&gt;(two-vals 3)\u000A1&gt; (ACL2_*1*_ACL2::TWO-VALS 3)\u000A  2&gt; (TWO-VALS 3)\u000A  &lt;2 (TWO-VALS 3 7)\u000A&lt;1 (ACL2_*1*_ACL2::TWO-VALS 3 7)\u000A(3 7)\u000AACL2 !&gt;</code> \u000A \u000A <p>We now document all of the options that may appear in a trace spec.  A \u000A trace spec with options is of the form</p> \u000A \u000A <code>(fn :kwd1 val1 :kwd2 val2 ... :kwdn valn)</code> \u000A \u000A <p>and here we document each legal keyword <v>:kwdi</v> and corresponding \u000A expected value <v>vali</v>.  Note that <v>trace$</v> is intended primarily for \u000A functions defined in the ACL2 command loop (see <see topic=\"ACL2____LP\">lp</see>).  If you want to \u000A trace a function that is defined in raw Lisp, then you can use option \u000A <v>:native</v> (see below), but then many other <v>trace$</v> options will not be \u000A available to you: all of them except <v>:multiplicity</v> and <v>:native</v> \u000A itself will be passed directly to the <v>trace</v> utility of the underlying \u000A Common Lisp.</p> \u000A \u000A <p><v>:COND</v>, <v>:ENTRY</v>, and <v>:EXIT</v></p> \u000A \u000A <p><b>Introduction</b>.  For each of these three options, the value is a \u000A (user-level) term, except that for <v>:entry</v> and <v>:exit</v> the value can be \u000A of the form <v>(:fmt u)</v> or <v>(:fmt! u)</v>, where <v>u</v> is a user-level \u000A term.  We skip these two latter cases for now and return to them later.  Then \u000A the indicated term is evaluated as indicated in the next paragraph, and if the \u000A <v>:cond</v> term is omitted or evaluates to non-<v>nil</v>, then the value of the \u000A <v>:entry</v> term is printed on entry and the value of the <v>:exit</v> term is \u000A printed on exit.  By default, where <v>:entry</v> is omitted or is specified as \u000A <v>nil</v>, the value printed for <v>:entry</v> is the list obtained by consing \u000A the calling function symbol onto the list of actual parameters: in the \u000A notation described below, this is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> TRACED-FN ARGLIST)</v>.  Similarly, \u000A the default for printing at the exit of the function call, i.e. where \u000A <v>:exit</v> is omitted or is specified as <v>nil</v>, is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> TRACED-FN\u000A VALUES)</v> where <v>VALUES</v> is the list of values returned as described \u000A below.</p> \u000A \u000A <p>In the evaluations of the term described below upon a call of <v>fn</v>, each \u000A formal parameter of the definition of <v>fn</v> will be bound to the \u000A corresponding actual of the call, the variable <v>ARGLIST</v> will be bound to \u000A the list of actuals, and the variable <v>TRACED-FN</v> will be bound to the \u000A function being called (either <v>fn</v> or its executable counterpart function; \u000A see above).  Additionally in the case of <v>:exit</v>, the variable <v>VALUES</v> \u000A will be bound to the multiple values returned (thus, a one-element list if \u000A <tt><see topic=\"ACL2____MV\">mv</see></tt> is not used in the return).  Also for <v>:exit</v>, we bind \u000A <v>VALUE</v> to the logical value returned, i.e., to the suitable list of values \u000A returned in the <tt><see topic=\"ACL2____MV\">mv</see></tt> case and otherwise to the single value returned. \u000A So in the <v>mv</v> case, <v>VALUE</v> is the same as <v>VALUES</v>, and otherwise \u000A <v>VALUE</v> is <v>(<see topic=\"COMMON-LISP____CAR\">car</see> VALUES)</v>.  Other than these variables and <tt><see topic=\"ACL2____STATE\">state</see></tt>, no other variable may occur in the term, whose value must be a single \u000A non-<tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> value, unless there is an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p> \u000A \u000A <p>Now suppose <v>fn</v> is called.  First: If <v>:cond</v> is supplied and the \u000A result of evaluating the <v>:cond</v> term is <v>nil</v>, then no tracing is done. \u000A Otherwise tracing proceeds as follows.  First the <v>:entry</v> form is \u000A evaluated, and the result is printed.  Then the call of <v>fn</v> is evaluated. \u000A Finally the <v>:exit</v> term is evaluated and the result is printed.  As \u000A indicated above, the default for the <v>:entry</v> term if omitted or explicitly \u000A <v>nil</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> TRACED-FN ARGLIST)</v>, and the default for the <v>:exit</v> \u000A term if omitted or explicitly <v>nil</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> TRACED-FN VALUES)</v>.</p> \u000A \u000A <p>Note that if the function has a formal named <v>ARGLIST</v>, then \u000A <v>ARGLIST</v> will nevertheless refer to the entire list of formals, not the \u000A single formal named <v>ARGLIST</v>; similarly for <v>TRACED-FN</v>, and \u000A additionally for <v>VALUE</v> and <v>VALUES</v> in the case of <v>:exit</v>.</p> \u000A \u000A <p>As mentioned above, for each of <v>:entry</v> and <v>:exit</v>, a value of \u000A <v>nil</v> specifies the default behavior.  If you really want a value of \u000A <v>nil</v>, use a non-<v>nil</v> form that evaluates to <v>nil</v>, for example \u000A <v>(<see topic=\"COMMON-LISP____CAR\">car</see> nil)</v> or <v>'nil</v>.  However, for <v>:cond</v> a value of <v>nil</v> \u000A means what it says: do not evaluate the <v>:entry</v> or <v>:exit</v> forms.</p> \u000A \u000A <p>Finally we discuss the case that the <v>:entry</v> or <v>:exit</v> term is of \u000A the form <v>(:fmt u)</v> or <v>(:fmt! u)</v>.  In these cases, the term <v>u</v> is \u000A evaluated as described above to produce a value, say <v>msg</v>, but instead of \u000A printing <v>msg</v> directly, ACL2 calls <v>fmt1</v> using the string <v>&quot;~@0&quot;</v> \u000A and the alist that binds just character <v>#\\0</v> to <v>msg</v>.  The following \u000A example illustrates this point, where <v>fact</v> is defined as above.  Also see \u000A <see topic=\"ACL2____FMT\">fmt</see>.  Note that <v>(<see topic=\"ACL2____MSG\">msg</see> string . vals)</v> produces a value suitable for \u000A a <v>&quot;~@&quot;</v> directive to the <v>fmt</v> family of print functions.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see>\u000A        (fact\u000A         :entry (:fmt (<see topic=\"ACL2____MSG\">msg</see> &quot;Tracing ~x0 on ~x1&quot; traced-fn arglist))\u000A         :exit (<see topic=\"COMMON-LISP____CAR\">car</see> values)))\u000A ((FACT :ENTRY (:FMT (<see topic=\"ACL2____MSG\">MSG</see> &quot;Tracing ~x0 on ~x1&quot; TRACED-FN ARGLIST))\u000A        :EXIT (<see topic=\"COMMON-LISP____CAR\">CAR</see> VALUES)))\u000AACL2 !&gt;(fact 3)\u000A1&gt; Tracing ACL2_*1*_ACL2::FACT on (3)\u000A  2&gt; Tracing FACT on (3)\u000A    3&gt; Tracing FACT on (2)\u000A      4&gt; Tracing FACT on (1)\u000A        5&gt; Tracing FACT on (0)\u000A        &lt;5 1\u000A      &lt;4 1\u000A    &lt;3 2\u000A  &lt;2 6\u000A&lt;1 6\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p>If <v>:fmt!</v> is used instead of <v>:fmt</v>, then indentation as is the \u000A prefix string, <v>&quot;n&gt; &quot;</v> or <v>&quot;&lt;n &quot;</v>.  The following example \u000A illustrates the use of <v>:fmt!</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see>\u000A        (fact\u000A         :entry (:fmt! (<see topic=\"ACL2____MSG\">msg</see> &quot;Tracing ~x0 on ~x1&quot; traced-fn arglist))\u000A         :exit (:fmt! (<see topic=\"ACL2____MSG\">msg</see> &quot;From input ~x0: ~x1&quot;\u000A                           (<see topic=\"COMMON-LISP____CAR\">car</see> arglist) (<see topic=\"COMMON-LISP____CAR\">car</see> values)))))\u000A ((FACT :ENTRY (:FMT! (<see topic=\"ACL2____MSG\">MSG</see> &quot;Tracing ~x0 on ~x1&quot; TRACED-FN ARGLIST))\u000A        :EXIT (:FMT! (<see topic=\"ACL2____MSG\">MSG</see> &quot;From input ~x0: ~x1&quot; (<see topic=\"COMMON-LISP____CAR\">CAR</see> ARGLIST)\u000A                          (<see topic=\"COMMON-LISP____CAR\">CAR</see> VALUES)))))\u000AACL2 !&gt;(fact 3)\u000ATracing ACL2_*1*_ACL2::FACT on (3)\u000ATracing FACT on (3)\u000ATracing FACT on (2)\u000ATracing FACT on (1)\u000ATracing FACT on (0)\u000AFrom input 0: 1\u000AFrom input 1: 1\u000AFrom input 2: 2\u000AFrom input 3: 6\u000AFrom input 3: 6\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p>Here is the same example, with user-managed indentation.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see>\u000A        (fact\u000A         :entry (:fmt! (<see topic=\"ACL2____MSG\">msg</see> &quot;~t0Tracing ~x1 on ~x2&quot;\u000A                            (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"ACL2_____04\">@</see> trace-level)))\u000A                            traced-fn arglist))\u000A         :exit (:fmt! (<see topic=\"ACL2____MSG\">msg</see> &quot;~t0From input ~x1: ~x2&quot;\u000A                           (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"ACL2_____04\">@</see> trace-level)))\u000A                           (<see topic=\"COMMON-LISP____CAR\">car</see> arglist) (<see topic=\"COMMON-LISP____CAR\">car</see> values)))))\u000A ((FACT :ENTRY (:FMT! (<see topic=\"ACL2____MSG\">MSG</see> &quot;~t0Tracing ~x1 on ~x2&quot;\u000A                           (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"ACL2_____04\">@</see> TRACE-LEVEL)))\u000A                           TRACED-FN ARGLIST))\u000A        :EXIT (:FMT! (<see topic=\"ACL2____MSG\">MSG</see> &quot;~t0From input ~x1: ~x2&quot;\u000A                          (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"ACL2_____04\">@</see> TRACE-LEVEL)))\u000A                          (<see topic=\"COMMON-LISP____CAR\">CAR</see> ARGLIST)\u000A                          (<see topic=\"COMMON-LISP____CAR\">CAR</see> VALUES)))))\u000AACL2 !&gt;(fact 3)\u000A   Tracing ACL2_*1*_ACL2::FACT on (3)\u000A     Tracing FACT on (3)\u000A       Tracing FACT on (2)\u000A         Tracing FACT on (1)\u000A           Tracing FACT on (0)\u000A           From input 0: 1\u000A         From input 1: 1\u000A       From input 2: 2\u000A     From input 3: 6\u000A   From input 3: 6\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p><b>ADVANCED OPTIONS</b> (alphabetical list)</p> \u000A \u000A <p><v>:COMPILE</v></p> \u000A \u000A <p>The tracing of <v>fn</v> installs a substitute definition of <v>fn</v> that \u000A prints trace information.  If the <v>:compile</v> option is omitted or has value \u000A <v>:same</v>, then the new definition will be compiled if and only if the \u000A existing definition is already compiled.  Otherwise, the new definition will \u000A be compiled exactly when the value of <v>:compile</v> is not <v>nil</v>.</p> \u000A \u000A <p><v>:DEF</v>, <v>:MULTIPLICITY</v></p> \u000A \u000A <p>ACL2's <v>trace$</v> mechanism often needs to know the number of outputs of a \u000A traced function, in the sense of <tt><see topic=\"ACL2____MV\">mv</see></tt>.  If you trace a function that was \u000A not defined inside the ACL2 loop (hence you are using the <v>:native</v> \u000A option), or if you provide an alternative definition using option <v>:def</v> \u000A (see below) and the new definition changes the number of values returned, then \u000A a natural number value for <v>:multiplicity</v> informs the trace utility of the \u000A number of expected outputs of the function being traced.  In the case that \u000A <v>:native</v> is supplied, the effect of a non-<v>nil</v> <v>:multiplicity</v> \u000A value depends on the host Lisp.  In the case of Lisps for which ACL2 uses the \u000A built-in Lisp mechanism for returning multiple values (see <see topic=\"ACL2____MV\">mv</see>), which \u000A are CCL and threaded SBCL as of June, 2010, <v>:multiplicity</v> is not needed \u000A and is ignored with <v>:native t</v>.  For GCL and Allegro CL, \u000A <v>:multiplicity</v> is used to generate a suitable <v>:exit</v> form if the \u000A <v>:exit</v> keyword was not already supplied.  For the other Lisps, the \u000A <v>:multiplicity</v> value is treated essentially as 1 whether it is supplied or \u000A not, because we do not know how to pass suitable information based on this \u000A value to the host Lisp's built-in tracing mechanism.</p> \u000A \u000A <p>Note that even supplying a <v>:multiplicity</v> option does not change the \u000A meaning of the variable <v>values</v>.  See the discussion of <v>:native</v> \u000A below.</p> \u000A \u000A <p>A useful option can be to supply a definition as the value of <v>:def</v>. \u000A (Again, note that if <v>:native</v> is used, then all options other than \u000A <v>:multiplicity</v> are passed directly to the underlying Lisp; in particular, \u000A <v>:def</v> will have no effect with <v>:native</v> except in the unlikely case \u000A that the raw Lisp provides some sort of support for <v>:def</v>.)  Note that \u000A this definition should be like a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form, but without the leading \u000A <v>defun</v> symbol; and it should define the function symbol being traced, with \u000A the same formal parameter list.  However, tracing of the so-called \u000A ``executable counterpart'' of a function (sometimes referred to as the ``*1* \u000A function'', for evaluation in the ACL2 loop; see <see topic='ACL2____Guards'>guards</see> for related discussion) is not sensitive to the \u000A <v>:def</v> option; rather, if a function has an executable counterpart then \u000A that executable counterpart is traced.</p> \u000A \u000A <p><v>:EVISC-TUPLE</v></p> \u000A \u000A <p>The printing described above is, by default, done using the current default \u000A trace evisc-tuple, which can be set using <tt><see topic=\"ACL2____SET-TRACE-EVISC-TUPLE\">set-trace-evisc-tuple</see></tt> (for \u000A the shape of this tuple, see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>); see <see topic=\"ACL2____SET-TRACE-EVISC-TUPLE\">set-trace-evisc-tuple</see>.  This tuple is based by default on the raw Lisp \u000A variables <v>*print-level*</v> and <v>*print-length*</v>, and will hide the ACL2 \u000A <v>world</v> and handle <see topic=\"ACL2____STOBJ\">stobj</see>s appropriately.  You may override this \u000A default by supplying an evisc tuple with the <v>:evisc-tuple</v> argument in \u000A your trace spec.  Be careful to supply a valid evisc-tuple, or you may get a \u000A raw Lisp error!</p> \u000A \u000A <p>A special value, <v>:print</v>, is useful if you are doing system hacking \u000A that can produce objects that are not valid ACL2 objects, such as raw Lisp \u000A arrays or objects in supporting packages not visible in the ACL2 \u000A read-eval-print loop.  If you supply <v>:evisc-tuple :print</v>, then the \u000A printing described above will be done with raw Lisp printing rather than ACL2 \u000A printing: specifically, with <v>(format *trace-output* &quot;s%&quot; x)</v>, where \u000A <v>x</v> is the value to be printed.</p> \u000A \u000A <p>A second special value for <v>:evisc-tuple</v>, <v>:no-print</v>, avoids \u000A printing the values of the <v>:entry</v> and <v>:exit</v> forms (or their \u000A defaults, if not specified).  This option is of use for side effects; for an \u000A example see community book <v>books/misc/wet.lisp</v>.</p> \u000A \u000A <p>Note that if <v>:evisc-tuple X</v> is supplied, then the form <v>X</v> will be \u000A evaluated before the function body is entered.  You can thus pull some tricks \u000A to print extra information before the <v>:entry</v> form is evaluated, for \u000A example as follows for a factorial function, <v>fact</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> (fact :evisc-tuple\u000A                     (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;~|**** HERE IS CW ****~|&quot;)\u000A                             nil)))\u000A ((FACT :EVISC-TUPLE (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (<see topic=\"ACL2____CW\">CW</see> &quot;~|**** HERE IS CW ****~|&quot;)\u000A                             NIL)))\u000AACL2 !&gt;(fact 3)\u000A**** HERE IS CW ****\u000A1&gt; (ACL2_*1*_ACL2::FACT 3)\u000A**** HERE IS CW ****\u000A  2&gt; (ACL2_*1*_ACL2::FACT 2)\u000A**** HERE IS CW ****\u000A    3&gt; (ACL2_*1*_ACL2::FACT 1)\u000A**** HERE IS CW ****\u000A      4&gt; (ACL2_*1*_ACL2::FACT 0)\u000A      &lt;4 (ACL2_*1*_ACL2::FACT 1)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 6)\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p><v>:FORMALS</v></p> \u000A \u000A <p>Normally ACL2 can figure out the formals for a given function.  This is \u000A always the case for functions defined in the ACL2 command loop and when option \u000A <v>:def</v> is supplied.  If neither of these cases applies then you can still \u000A trace a function (even without using the <v>:native</v> option) by supplying \u000A option <v>:notinline :fncall</v>, but you will still need to supply the list of \u000A formal parameters.  The value of the <v>:formals</v> option should be the list \u000A of formals in this case.</p> \u000A \u000A <p><v>:HIDE</v></p> \u000A \u000A <p>The default value for this advanced option is <v>t</v>, which causes <see topic=\"ACL2____STOBJ\">stobj</see>s and the logical <see topic=\"ACL2____WORLD\">world</see> to be printed as single symbols, along \u000A with certain large structures of interest to developers (rewrite contants, \u000A enabled structures, and event and command index structures).  If however the \u000A value <v>nil</v> is supplied, then this default behavior is defeated.  In that \u000A case, you can still arrange to print the logical world as a symbol and to \u000A print <see topic=\"ACL2____STOBJ\">stobj</see>s without breaking the trace printing: see <see topic=\"ACL2____SET-TRACE-EVISC-TUPLE\">set-trace-evisc-tuple</see> for how to do this globally, or similarly use the \u000A <v>:evisc-tuple</v> option to <v>trace$</v> to do this with a single trace spec. \u000A Note however that with value <v>nil</v> specified for <v>:hide</v>, such use of an \u000A evisc-tuple will not deal properly with local stobjs (see <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>) or stobjs bound by <tt><see topic=\"ACL2____STOBJ-LET\">stobj-let</see></tt>, or with the \u000A aforementioned large structures other than the logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p><v>:NATIVE</v></p> \u000A \u000A <p>If <v>:native</v> is supplied with a non-<v>nil</v> value, then the trace spec \u000A is passed to the native Lisp trace (after removing the <v>:native</v> option). \u000A A trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) is required in order to use this option, \u000A because no syntactic check is made on the <v>:cond</v>, <v>:entry</v>, or \u000A <v>:exit</v> forms -- arbitrary raw Lisp may occur in them!</p> \u000A \u000A <p>Note that by ``native Lisp trace'' we mean the currently installed \u000A <v>trace</v>.  As discussed briefly elsewhere (see <see topic=\"COMMON-LISP____TRACE\">trace</see>), ACL2 has \u000A modified that trace to be more useful if the underlying host Lisp is GCL, \u000A Allegro CL, or CCL (OpenMCL).  If you need the original trace utility supplied \u000A for those Lisps, quit the ACL2 loop with <v>:q</v> and call <v>old-trace</v> and \u000A <v>old-untrace</v> in raw Lisp where you would otherwise call <v>trace</v> and \u000A <v>untrace</v>.  Note that the original trace utility supplied with a given Lisp \u000A will not hide the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> or give special treatment to <see topic=\"ACL2____STOBJ\">stobj</see>s.</p> \u000A \u000A <p>It is important to understand that if <v>:native t</v> is specified, then all \u000A other options are interpreted by the native Lisp trace.  For example, that \u000A trace probably has no understanding of the use of <v>:fmt</v> described above \u000A for <v>:entry</v> or <v>:exit</v>.  Indeed, the native trace may not even accept \u000A any of <v>:cond</v>, <v>:entry</v> or <v>:exit</v>, let alone any of the advanced \u000A options!  Moreover, if <v>:native t</v> is specified, then even a \u000A <v>:multiplicity</v> option does not provide the meaning of the variable \u000A <v>values</v> that one might desire.  In GCL for example, in the case of an \u000A <tt><see topic=\"ACL2____MV\">mv</see></tt> return of a function defined only in raw Lisp (not in ACL2), this \u000A variable will be bound to a list containing only the first result.</p> \u000A \u000A <p><v>:NOTINLINE</v></p> \u000A \u000A <p>By default, a new definition installed by <v>trace$</v> will include a \u000A <v>notinline</v> declaration so that recursive calls will always be traced.  To \u000A avoid this declaration, supply value <v>nil</v>.</p> \u000A \u000A <p>A special value for <v>:notinline</v>, <v>:fncall</v>, will cause the traced \u000A function to call its original definition.  Without this special value, the new \u000A installed definition for the traced function will include the body of the \u000A original definition.  This <v>:fncall</v> behavior is the default only in the \u000A following cases:</p> \u000A \u000A <blockquote><p>o for functions whose definitions are built into ACL2;</p> \u000A \u000A <p>o for functions that have been added (using a trust tag, an advanced \u000A feature, so most users can probably ignore this case) to either of the <tt><see topic=\"ACL2____STATE\">state</see></tt> global variables <v>program-fns-with-raw-code</v> or \u000A <v>logic-fns-with-raw-code</v>;</p> \u000A \u000A <p>o (`HONS' extension only; see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>) for <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>d functions.</p></blockquote> \u000A \u000A <p>The legal values for <v>:notinline</v> are <v>t</v> (the default for other than \u000A the cases displayed above), <v>nil</v>, and <v>:fncall</v>.  (Except: For the \u000A 'HONS' extension, only <v>:fncall</v> is legal.)</p> \u000A \u000A <p><b>Remarks</b>.</p> \u000A \u000A <p>(1) If some of the given trace specs have errors, then <v>trace$</v> will \u000A generally print error messages for those but will still process those that do \u000A not have errors.  The value returned will indicate the trace specs that were \u000A processed successfully.</p> \u000A \u000A <p>(2) If you certify or include a book that redundantly defines a function \u000A that is currently traced, then tracing behavior may disappear if a compiled \u000A definition is installed for the function or its in-the-logic (so-called `*1*') \u000A counterpart.</p> \u000A \u000A <p>(3) Some predefined functions are called during tracing.  In order to avoid \u000A infinite loops, such calls of traced predefined functions will be made using \u000A the original predefined functions, not using their code installed by \u000A <v>trace$</v>.</p>"],

 "ACL2____PROOF-CHECKER":[["ACL2","Debugging"],"ACL2 Sources","ACL2","<p>Call this up with <v>(<see topic=\"ACL2____VERIFY\">verify</see> ...)</v>.</p> \u000A \u000A <p>This is an interactive system for checking ACL2 theorems, or at least \u000A exploring their proofs.  One enters it using the <v>VERIFY</v> command (see \u000A <see topic=\"ACL2____VERIFY\">verify</see>), and then invokes commands at the resulting prompt to operate \u000A on a stack of goals, starting with the single goal that was supplied to \u000A <v>VERIFY</v>.  The final command (or ``instruction'') can be an <v>exit</v> \u000A command, which can print out a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event if the goal stack is \u000A empty; see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>, in particular the <v>exit</v> command. \u000A That resulting <v>defthm</v> event includes an <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt> \u000A parameter, which directs replay of the proof-checker commands (for example \u000A during certification of a book containing that event; see <see topic=\"ACL2____BOOKS\">books</see>).</p> \u000A \u000A <p>If you exit the proof-checker interactive loop, you may re-enter that \u000A session at the same point using the command <v>(<see topic=\"ACL2____VERIFY\">verify</see>)</v>, i.e., with no \u000A arguments.  The commands <v>save</v> and <v>retrieve</v> may be invoked to manage \u000A more than one session.</p> \u000A \u000A <p>The proof-checker can be invoked on a specific subgoal, and the resulting \u000A <v>:instructions</v> can be given as a hint to the theorem prover for that \u000A subgoal.  See <see topic=\"ACL2____INSTRUCTIONS\">instructions</see>.</p> \u000A \u000A <p>A tutorial is available on the world-wide web:<br></br> \u000A \u000A <a href='http://www.cs.utexas.edu/users/kaufmann/tutorial/rev3.html'>http://www.cs.utexas.edu/users/kaufmann/tutorial/rev3.html</a>.<br></br> \u000A \u000A The tutorial illustrates more than just the proof-checker.  The portion \u000A relevant to the proof-checker may be accessed directly:<br></br> \u000A \u000A <a \u000A href='http://www.cs.utexas.edu/users/kaufmann/tutorial/rev3.html#slide29'>http://www.cs.utexas.edu/users/kaufmann/tutorial/rev3.html#slide29</a></p> \u000A \u000A <p>See <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> for how to arrange that output is printed in \u000A abbreviated form.  In general, the proof-checker uses the <v>:TERM</v> <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> described in that documentation.</p> \u000A \u000A <p>Individual proof-checker commands are documented in subsection <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  When inside the interactive loop (i.e., after \u000A executing <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>), you may use the <tt><see topic=\"ACL2____HELP\">help</see></tt> command to get a list \u000A of legal instructions and <v>(<see topic=\"ACL2____HELP\">help</see> instr)</v> to get help for the instruction \u000A <v>instr</v>.</p>"],

 "COMMON-LISP____LET":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<h3>Introduction</h3> \u000A \u000A <p>Example</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP_____A2\">*</see> x x))\u000A      (y (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)))\u000A (<see topic=\"COMMON-LISP____LIST\">list</see> x y))</code> \u000A \u000A <p>If the form above is executed in an environment in which <v>x</v> has the \u000A value <v>-2</v>, then the result is <v>'(4 -4)</v>.</p> \u000A \u000A <p><v>Let</v> expressions bind variables so that their ``local'' values, the \u000A values they have when the ``body'' of the <v>let</v> is evaluated, are possibly \u000A different than their ``global'' values, the values they have in the context in \u000A which the <v>let</v> expression appears.  In the <v>let</v> expression above, the \u000A local variables bound by the <v>let</v> are <v>x</v> and <v>y</v>.  They are locally \u000A bound to the values delivered by the two forms <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x x)</v> and <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)</v>, \u000A respectively, that appear in the ``bindings'' of the <v>let</v>.  The body of \u000A the <v>let</v> is <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x y)</v>.</p> \u000A \u000A <p>Suppose that the <v>let</v> expression above occurs in a context in which \u000A <v>x</v> has the value <v>-2</v>.  (The global value of <v>y</v> is irrelevant to \u000A this example.)  For example, one might imagine that the <v>let</v> form above \u000A occurs as the body of some function, <v>fn</v>, with the formal parameter <v>x</v> \u000A and we are evaluating <v>(fn -2)</v>.</p> \u000A \u000A <p>To evaluate the <v>let</v> above in a context in which <v>x</v> is <v>-2</v>, we \u000A first evaluate the two forms specifying the local values of the variables. \u000A Thus, <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x x)</v> is evaluated and produces <v>4</v> (because <v>x</v> is \u000A <v>-2</v>) and <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)</v> is evaluated and produces <v>-4</v> (because <v>x</v> is \u000A <v>-2</v>).  Then <v>x</v> and <v>y</v> are bound to these values and the body of \u000A the <v>let</v> is evaluated.  Thus, when the body, <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x y)</v> is evaluated, \u000A <v>x</v> is <v>4</v> and <v>y</v> is <v>-4</v>.  Thus, the body produces <v>'(4\u000A -4)</v>.</p> \u000A \u000A <p>Note that the binding of <v>y</v>, which is written after the binding of \u000A <v>x</v> and which mentions <v>x</v>, nevertheless uses the global value of \u000A <v>x</v>, not the new local value.  That is, the local variables of the <v>let</v> \u000A are bound ``in parallel'' rather than ``sequentially.'' In contrast, if \u000A the example <v>let*</v> form:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((x (<see topic=\"COMMON-LISP_____A2\">*</see> x x))\u000A       (y (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)))\u000A (<see topic=\"COMMON-LISP____LIST\">list</see> x y))</code> \u000A \u000A <p>is evaluated when the global value of <v>x</v> is <v>-2</v>, then the result is \u000A <v>'(4 8)</v>, because the local value of <v>y</v> is computed after <v>x</v> has \u000A been bound to <v>4</v>.  <tt><see topic=\"COMMON-LISP____LET_A2\">Let*</see></tt> binds its local variables \u000A ``sequentially.''</p> \u000A \u000A <p>General <v>let</v> forms</p> \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((var1 term1) ... (varn termn)) body)\u000Aand\u000A(<see topic=\"COMMON-LISP____LET\">let</see> ((var1 term1) ... (varn termn))\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ... (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...)\u000A body)</code> \u000A \u000A <p>where the <v>vari</v> are distinct variables, the <v>termi</v> are terms \u000A involving only variables bound in the environment containing the <v>let</v>, and \u000A <v>body</v> is a term involving only the <v>vari</v> plus the variables bound in \u000A the environment containing the <v>let</v>.  Each <v>vari</v> must be used in \u000A <v>body</v> or else <see topic=\"COMMON-LISP____DECLARE\">declare</see>d ignored.</p> \u000A \u000A <p>A <v>let</v> form is evaluated by first evaluating each of the <v>termi</v>, \u000A obtaining for each a <v>vali</v>.  Then, each <v>vari</v> is bound to the \u000A corresponding <v>vali</v> and <v>body</v> is evaluated.</p> \u000A \u000A <p>Actually, <v>let</v> forms are just abbreviations for certain uses of \u000A <v>lambda</v> notation.  In particular</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((var1 term1) ... (varn termn)) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) body)</code> \u000A \u000A <p>is equivalent to</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (var1 ... varn)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...)\u000A   body)\u000A term1 ... termn).</code> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____LET_A2\">Let*</see></tt> forms are used when it is desired to bind the <v>vari</v> \u000A sequentially, i.e., when the local values of preceding <v>varj</v> are to be \u000A used in the computation of the local value for <v>vari</v>.</p> \u000A \u000A <p>General <v>let*</v> forms:</p> \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((var1 term1) ... (varn termn)) body)\u000Aand\u000A(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((var1 term1) ... (varn termn))\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x1 ... xm))\u000A body)</code> \u000A \u000A <p>where the <v>vari</v> are variables (not necessarily distinct), the \u000A <v>termi</v> are terms involving only variables bound in the environment \u000A containing the <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> and those <v>varj</v> such that <v>j&lt;i</v>, and \u000A <v>body</v> is a term involving only the <v>vari</v> plus the variables bound in \u000A the environment containing the <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>.  Each <v>vari</v> must be used \u000A either in some subsequent <v>termj</v> or in <v>body</v>, except that in the \u000A second form above we make an exception when <v>vari</v> is among the <v>xk</v>, in \u000A which case <v>vari</v> must not be thus used.  Note that <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> does not \u000A permit the inclusion of any <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms other than one as shown \u000A above.  In the second general form above, every <v>xk</v> must be among the \u000A <v>vari</v>, and furthermore, <v>xk</v> may not equal <v>vari</v> and <v>varj</v> for \u000A distinct <v>i</v>, <v>j</v>.</p> \u000A \u000A <p>The first <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> above is equivalent to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((var1 term1))\u000A ...\u000A (<see topic=\"COMMON-LISP____LET\">let</see> ((varn termn)) body)...)</code> \u000A \u000A <p>Thus, the <v>termi</v> are evaluated successively and after each evaluation \u000A the corresponding <v>vali</v> is bound to the value of <v>termi</v>.  The second \u000A <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> is similarly expanded, except that each for each <v>vari</v> that \u000A is among the <v>(x1 ... xm)</v>, the form <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> vari))</v> is \u000A inserted immediately after <v>(vari termi)</v>.</p> \u000A \u000A <p>Each <v>(vari termi)</v> pair in a <v>let</v> or <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> form is called a \u000A ``binding'' of <v>vari</v> and the <v>vari</v> are called the ``local variables'' \u000A of the <v>let</v> or <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>.  The common use of <v>let</v> and <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> \u000A is to save the values of certain expressions (the <v>termi</v>) so that they may \u000A be referenced several times in the body without suggesting their \u000A recomputation.</p> \u000A \u000A <p><v>Let</v> is part of Common Lisp.  See any Common Lisp documentation for \u000A more information.</p>"],

 "ACL2____TAU-SYSTEM":[["Rule-classes"],"ACL2 Sources","ACL2","<p>This documentation topic describes the syntactic form of \u000A ``tau-system'' rules; these rules extend ACL2's ``type checker.''  For an \u000A introduction to the tau system, see <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>.</p> \u000A \u000A <p>There happens to be a <i>function</i> named <v>tau-system</v>, defined as the \u000A identity function.  Its only role is to provide the rune \u000A <v>(:EXECUTABLE-COUNTERPART TAU-SYSTEM)</v>, which is used to enable and disable \u000A the tau system.  Otherwise the function <v>tau-system</v> has no purpose and we \u000A recommend that you avoid using it so you are free to enable and disable the \u000A tau system.</p> \u000A \u000A <p>When in the default (``greedy'') mode (see <tt><see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see></tt>), \u000A every <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and every <v>:corollary</v> (see <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>) \u000A of every <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> stored as a rule <i>of any</i> <v>:rule-class</v> is \u000A inspected to determine if it is of one of the forms below.  Rules of these \u000A forms are added to the tau database, even if they are not labeled as \u000A <v>:tau-system</v> rules, e.g., a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule might contribute to \u000A the tau database!  To add a rule to the tau database without adding any other \u000A kind of rule, tag it with <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>:tau-system</v>.  If a \u000A theorem has <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>nil</v>, it is not considered for the \u000A tau database.</p> \u000A \u000A <code>General Forms: \u000A<i>Boolean</i>: \u000A(booleanp (p v)) \u000A \u000A<i>Eval</i>: \u000A(p 'const) or \u000A(p *const*) \u000A \u000A<i>Simple</i>: \u000A(implies (p v) (q v)) \u000A \u000A<i>Conjunctive</i>: \u000A(implies (and (p1 v) ... (pk v)) (q v)), ; Here k must exceed 1. \u000A \u000A<i>Signature Form 1</i>: \u000A(implies (and (p1 x1) (p2 x2) ...) \u000A         (q (fn x1 x2 ...))) \u000A \u000A<i>Signature Form 2</i>: \u000A(implies (and (p1 x1) (p2 x2) ...) \u000A         (q (mv-nth 'n (fn x1 x2 ...)))) \u000A \u000A<i>Bounder Form 1 (or Form 2)</i>: \u000A(implies (and (tau-intervalp i1) \u000A              ... \u000A              (or (equal (tau-interval-dom i1) 'dom1-1) \u000A                  ...) \u000A              ... \u000A              (in-tau-intervalp x1 i1) \u000A              ...) \u000A         (and (tau-intervalp (bounder-fn i1 ...)) \u000A              (in-tau-intervalp <i>target</i> \u000A                                (bounder-fn i1 ...)))) \u000A \u000Awhere <i>target</i> is \u000A(fn x1 ... y1 ...)             in <i>Form 1</i>, and \u000A(mv-nth 'n (fn x1 ... y1 ...)) in <i>Form 2</i> \u000A \u000A<i>Big Switch</i>: \u000A(equal (fn . formals) body) \u000A \u000A<i>MV-NTH Synonym</i>: \u000A(equal (nth-alt x y) (mv-nth x y)) or \u000A(equal (mv-nth x y) (nth-alt x y)) \u000A</code> \u000A \u000A <p>The symbols <v>p</v>, <v>q</v>, <v>p1</v>, etc., denote monadic (one-argument) \u000A Boolean-valued function symbols, or equalities in which one argument is \u000A constant, arithmetic comparisons in which one argument is a rational or \u000A integer constant, or the logical negations of such terms.  By ``equalities'' \u000A we allow <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, and <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>.  By \u000A ``arithmetic comparison'' we mean <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>, <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>, <tt><see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see></tt>, or <tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt>.  Any of these tau predicates may appear negated.</p> \u000A \u000A <p>The notation <v>(p v)</v> above might stand for any one of:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> V 'MONDAY)\u000A(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> I 16)\u000A(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 'SUNDAY))</code> \u000A \u000A <p>The different rule forms above affect different aspects of the tau system. \u000A We discuss each form in more detail below.</p> \u000A \u000A <p>The documentation below is written as though the tau system is in auto \u000A mode!  To insure that the only rules added to the tau system are those \u000A explicitly assigned to <v>:rule-class</v> <v>:tau-system</v>, you should use \u000A <tt><see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see></tt> to select manual mode.</p> \u000A \u000A <code>General Form: <i>Boolean</i>: \u000A(booleanp (p v)) \u000A</code> \u000A \u000A <p>Here <v>p</v> must be a function symbol and <v>v</v> must be a variable.  Such \u000A a <v>:tau-system</v> rule adds <v>p</v> to the list of tau predicates.  If <v>p</v> \u000A was recognized as Boolean when it was defined, there is no need to state this \u000A rule.  This form is needed if you define a monadic Boolean function in such a \u000A way that the system does not recognize that it is Boolean.</p> \u000A \u000A <code>General Form: <i>Eval</i>: \u000A(p 'const) or \u000A(p *const*) \u000A</code> \u000A \u000A <p>Here <v>p</v> must be a function symbol.  In addition, recall that these \u000A general tau predicate forms may appear negated.  So the form above includes \u000A such theorems as <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (GOOD-STATEP *INITIAL-STATE*))</v>.  A theorem of this \u000A form thus records whether a named predicate is true or false on the given \u000A constant.</p> \u000A \u000A <p>Generally, when the tau system must determine whether an enabled tau \u000A predicate is true or false on a constant, it simply evaluates the predicate on \u000A the constant.  This can be impossible or very inefficient if <v>p</v> is not \u000A defined but constrained, or if <v>p</v> is defined in a hard-to-compute way \u000A (e.g., <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p (x) (<see topic=\"COMMON-LISP____EVENP\">evenp</see> (ack x x)))</v> where <v>ack</v> is the Ackermann \u000A function), or perhaps if the constant is very large.  By proving a \u000A <v>:tau-system</v> rule of Eval form, you cause the tau system to note the value \u000A of the predicate on the constant and henceforth to look it up instead of \u000A evaluating the definition.</p> \u000A \u000A <p>A difficulty, however, is determining that a slow down is due to the \u000A evaluation of tau predicates and not some other reason.  The first step is \u000A determining that tau is slowing the proof down.  See <tt><see topic=\"ACL2____TIME-TRACKER-TAU\">time-tracker-tau</see></tt> \u000A for an explanation of <v>TIME-TRACKER-NOTE</v>s output during some proofs \u000A involving tau reasoning.  These notes can alert you to the fact that \u000A significant amounts of time are being spent in the tau system.  <tt><see topic=\"ACL2____TIME-TRACKER-TAU\">Time-tracker-tau</see></tt> gives some ways of determining whether tau predicate \u000A evaluation is involved.  (If worse comes to worst, consider the following \u000A hack: In the ACL2 source file <v>tau.lisp</v>, immediately after the definition \u000A of the system function <v>ev-fncall-w-tau-recog</v>, there is a comment which \u000A contains some raw Lisp code that can be used to investigate whether tau's use \u000A of evaluation on constants is causing a problem.)  However, once a recognizer \u000A and the constants on which it is being evaluated are identified, the tau \u000A system can be sped up by proving Eval rules to pre-compute and store the \u000A values of the recognizer on those constants.  Alternatively, at the possible \u000A loss of some completeness in the tau system, the executable counterpart of the \u000A recognizer can be disabled.</p> \u000A \u000A <code>General Form: <i>Simple</i>: \u000A(implies (p v) (q v)) \u000A</code> \u000A \u000A <p>Here <v>v</v> must be a variable symbol.  This rule builds-in the information \u000A that anything satisfying <v>p</v> must also satisfy <v>q</v>, i.e., the ``type'' \u000A <v>q</v> includes the ``type'' <v>p</v>.  Recall that the forms may be negated. \u000A Most of the time, <v>p</v> and <v>q</v> will be predicate symbols but it is \u000A possible they will be equalities- or inequalities-with-constants.  Examples of \u000A Simple rules include the following, which are in fact built-in:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x))\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)))\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see> x) (<see topic=\"ACL2____ALISTP\">alistp</see> x))</code> \u000A \u000A <p>Because the tau system records the transitive closure of the Simple rules, \u000A any time a term is known to satisfy <v>natp</v> it is also known to satisfy \u000A <v>integerp</v> and <v>rationalp</v>, and known not to satisfy <v>true-listp</v>, \u000A and known to be non-negative.</p> \u000A \u000A <code>General Form: <i>Conjunctive</i>: \u000A(implies (and (p1 v) ... (pk v)) (q v)), ; Here k must exceed 1. \u000A</code> \u000A \u000A <p>The <v>pi</v> and <v>q</v> may be any tau predicates or their negations, <v>v</v> \u000A must be a variable symbol, and <v>i</v> must exceed 1 or else this is a Simple \u000A rule.  An obvious operational interpretation of this rule is that if an object \u000A is known to satisfy all of the <v>pi</v>, then it is known to satisfy <v>q</v>. \u000A However, the actual interpretation is more general.  For example, if an object \u000A is known to satisfy all but one of the <v>pi</v> and is known not to satisfy \u000A <v>q</v>, then the object is known not to satisfy the ``missing'' <v>pi</v>.</p> \u000A \u000A <p>For example, the following Conjunctive rule allows tau to conclude that if \u000A weekday <v>D</v> is not <v>MON</v>, <v>TUE</v>, <v>THU</v> or <v>FRI</v>, then it is \u000A <v>WED</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (weekdayp d)\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> d 'MON))\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> d 'TUE))\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> d 'WED))\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> d 'THU)))\u000A         (<see topic=\"COMMON-LISP____EQ\">eq</see> d 'FRI))</code> \u000A \u000A <p>The tau database is not closed under conjunctive rules; they are applied \u000A dynamically.</p> \u000A \u000A <code>General Form: <i>Signature Form 1</i>: \u000A(implies (and (p1 x1) (p2 x2) ... (pn xn) dep-hyp) \u000A         (q (fn x1 x2 ... xn))) \u000A</code> \u000A \u000A <p>The <v>pi</v> and <v>q</v> may be any tau predicates or their negations, \u000A <v>fn</v> must be a function symbol of arity <v>n</v>, the <v>xi</v> must be \u000A distinct variable symbols and <v>dep-hyp</v> may be any term, provided it is not \u000A of the <v>(pi xi)</v> shape and the only the variables in it are the \u000A <v>xi</v>.</p> \u000A \u000A <p>The Signature form actually allows multiple tau predicates to be applied to \u000A each variable, e.g., x1 might be required to be both an <v>INTEGERP</v> and \u000A <v>EVENP</v>.  The Signature form allows there to be multiple hypotheses \u000A classified as <v>dep-hyp</v>s, i.e., not fitting any of the previous shapes, and \u000A they are implicitly just conjoined.  The name ``dep-hyp'' is an abbreviation \u000A of ``dependent hypothesis'' and stems from the fact they often express \u000A relations between several of the function's inputs rather than type-like \u000A constraints on individual inputs.</p> \u000A \u000A <p>A Signature rule informs tau that the function <v>fn</v> returns an object \u000A satisfying <v>q</v> provided that the arguments satisfy the respective <v>pi</v> \u000A and provided that <v>dep-hyp</v> occurs in the current context.  Note: to be \u000A precise, dependent hypotheses are relieved only by applying ACL2's most \u000A primitive form of reasoning, <see topic=\"ACL2____TYPE-SET\">type-set</see>.  In particular, tau reasoning is \u000A not used to establish dependent hypotheses.  The presence of a <v>dep-hyp</v> in \u000A a signature rule may severely restrict its applicability.  We discuss this \u000A after showing a few mundane examples.</p> \u000A \u000A <p>An example Signature rule is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> x)\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> y))\u000A         (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)))</code> \u000A \u000A <p>Of course, a function may have multiple signatures:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> x)\u000A              (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> y))\u000A         (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)))</code> \u000A \u000A <p>Here is a Signature rule for the function <v>pairlis$</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> x)\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> y))\u000A         (<see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see> (<see topic=\"ACL2____PAIRLIS_42\">pairlis$</see> x y)))</code> \u000A \u000A <p>The tau system can consequently check this theorem by composing the last \u000A two rules shown and exploiting Simple rule stating that symbol-alists are also \u000A alists:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> a)\u000A                   (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> b)\u000A                   (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> y))\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> (<see topic=\"ACL2____PAIRLIS_42\">pairlis$</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> a b) y))))</code> \u000A \u000A <p>Since <v>a</v> and <v>b</v> are known to be lists of symbols and a signature \u000A for <v>append</v> is that it preserves that predicate, the first argument to the \u000A <v>pairlis$</v> expression is known to be a list of symbols.  This means the \u000A Signature rule for <v>pairlis$</v> tells us the result is a <v>symbol-alistp</v>, \u000A but the previously mentioned Simple rule, <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see> x)\u000A (<see topic=\"ACL2____ALISTP\">alistp</see> x))</v>, tells us the result is also an <v>alistp</v>.</p> \u000A \u000A <p>When a Signature rule has an <v>dep-hyp</v>, that hypothesis is not an \u000A expression in the tau system.  Tau is not used to check that hypothesis. \u000A Instead, tau uses the more primitive <see topic=\"ACL2____TYPE-SET\">type-set</see> mechanism of ACL2.  Here \u000A is an example of a Signature rule with a <v>dep-hyp</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> a)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> a)))\u000A         (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n a)))</code> \u000A \u000A <p>Note that the last hypothesis is a dependent hypothesis: it is not a tau \u000A predicate but a relationship between <v>n</v> and <v>a</v>.  It is relieved by \u000A <see topic=\"ACL2____TYPE-SET\">type-set</see>.  If one is trying to compute the signature of an <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> n\u000A a)</v> expression in a context in which <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> a))</v> is explicitly \u000A assumed, then this mechanism would establish the dependent hypothesis.  But \u000A one can easily imagine an almost identical context where, say <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see>\u000A (rev a)))</v> is explicitly assumed.  In that context, the Signature rule would \u000A not be fired because <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> cannot establish <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> a))</v> from \u000A <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> (rev a)))</v>, even though it would be easily proved by rewriting \u000A using the theorem <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (rev a)) (<see topic=\"ACL2____LEN\">len</see> a))</v>.</p> \u000A \u000A <p>Note also that if this signature could be phrased in a way that eliminates \u000A the dependency between <v>n</v> and <v>a</v> it would be more effective.  For \u000A example, here is a related Signature rule without a dependent hypothesis:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A              (register-filep a)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n 16))\u000A         (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n a)))</code> \u000A \u000A <p>In this theorem we require only that <v>n</v> be less than 16, which is a tau \u000A predicate and hence just an additional tau constraint on <v>n</v>.</p> \u000A \u000A <code>General Form: <i>Signature Form 2</i>: \u000A(implies (and (p1 x1) (p2 x2) ... (pn xn) dep-hyp) \u000A \u000A         (q (mv-nth 'n (fn x1 x2 ... xn)))) \u000A</code> \u000A \u000A <p>This form of signature rule is just like form 1 except that it is useful \u000A for functions that return multiple-values and allows us to ``type-check'' \u000A their individual outputs.</p> \u000A \u000A <code>General Form: <i>Bounder Forms 1 and 2</i>: \u000A(implies (and (tau-intervalp i1) \u000A              ... \u000A              (or (equal (tau-interval-dom i1) 'dom1-1) \u000A                  ...) \u000A              ... \u000A              (in-tau-intervalp x1 i1) \u000A              ...) \u000A         (and (tau-intervalp (bounder-fn i1 ...)) \u000A              (in-tau-intervalp <i>target</i> \u000A                                (bounder-fn i1 ...)))) \u000A</code> \u000A \u000A <p>where <i>target</i> is either <v>(fn x1 ... y1 ...)</v> in <i>Form 1</i> or \u000A <v>(<see topic=\"ACL2____MV-NTH\">mv-nth</see> 'n (fn x1 ... y1 ...))</v> in <i>Form 2</i>.</p> \u000A \u000A <p>This form is for advanced users only and the schema given above is just a \u000A reminder of the general shape.  A ``bounder'' for a given function symbol, \u000A <v>fn</v>, is a function symbol <v>bounder-fn</v> that computes an interval \u000A containing <v>(fn x1 ... y1 ...)</v> (or its <v>n</v>th component in the case of \u000A Form 2 rules) from the intervals containing certain of the arguments of \u000A <v>fn</v>.  The correctness theorem for a bounder function informs the tau \u000A system that bounds for <v>fn</v> are computed by <v>bounder-fn</v> and sets up the \u000A correspondence between the relevant arguments, <v>xi</v>, of <v>fn</v> and the \u000A intervals containing those arguments, <v>ii</v> to which <v>bounder-fn</v> is \u000A applied.  When the tau system computes the tau for a call of <v>fn</v>, it \u000A computes the tau of the relevant arguments and applies the bounder to the \u000A intervals of those tau.  This provides a domain and upper and/or lower bounds \u000A for the value of the term.  The tau system then further augments that with \u000A signature rules.  See <see topic=\"ACL2____BOUNDERS\">bounders</see> for details on intervals, bounders, and \u000A bounder correctness theorems.</p> \u000A \u000A <code>General Form: <i>Big Switch</i>: \u000A(equal (fn . formals) body) \u000A</code> \u000A \u000A <p>In the Big Switch form, <v>fn</v> must be a function symbol, <v>formals</v> \u000A must be a list of distinct variable symbols, and <v>body</v> must be a ``big \u000A switch'' term, i.e., one that case splits on tau predicates about a single \u000A variable and produces a term not involving that variable.  An example of a Big \u000A Switch rule is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (conditional-type x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)\u000A           (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> y)))</code> \u000A \u000A <p>The idea is that the tau system can treat calls of <v>conditional-type</v> as \u000A a tau-predicate after determining the tau of an argument.</p> \u000A \u000A <p>Since equality-to-constants are tau predicates, a more common example of a \u000A Big Switch rule is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (dtypep x expr)\u000A       (<see topic=\"COMMON-LISP____CASE\">case</see> x\u000A             (STMT (stmt-typep expr))\u000A             (EXPR (expr-typep expr))\u000A             (MODULE (module-typep expr))\u000A             (otherwise nil)))</code> \u000A \u000A <p>This is because <v>(<see topic=\"COMMON-LISP____CASE\">case</see> x (STMT ...) ...)</v> macroexpands in ACL2 to <v>(<see topic=\"COMMON-LISP____IF\">if</see>\u000A (<see topic=\"COMMON-LISP____EQL\">eql</see> x 'STMT) ... ...)</v> and <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> x 'STMT)</v> is a tau predicate about \u000A <v>x</v>.</p> \u000A \u000A <p>Big Switch rules are recognized when a function is defined (if tau is in \u000A automatic mode).  They generally do not have to be proved explicitly, though \u000A they might be when mutual recursion is involved.  Only the first detected Big \u000A Switch rule about a function <v>fn</v> is recognized.</p> \u000A \u000A <code>General Form: <i>MV-NTH Synonym</i>: \u000A(equal (nth-alt x y) (mv-nth x y)) or \u000A(equal (mv-nth x y) (nth-alt x y)) \u000A</code> \u000A \u000A <p>Rules of this form just tell the tau system that the user-defined function \u000A <v>nth-alt</v> is synonymous with the ACL2 primitive function <v>mv-nth</v>. \u000A Because ACL2's rewriter gives special handling to <v>mv-nth</v>, users sometimes \u000A define their own versions of that function so they can disable them and \u000A control rewriting better.  By revealing to the tau system that such a synonym \u000A has been introduced you allow Signature rules of Form 2 to be used.</p>"],

 "ACL2____TERM":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples of Terms:\u000A(<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____CAAR\">caar</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> t x)) (t 0))   ; an untranslated term\u000A\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (<see topic=\"COMMON-LISP____CONS\">cons</see> 't x) '0)    ; a translated term\u000A\u000A(<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y) 'nil v)              ; a pseudo-term</code> \u000A \u000A <p>In traditional first-order predicate calculus a ``term'' is a syntactic \u000A entity denoting some object in the universe of individuals.  Often, for \u000A example, the syntactic characterization of a term is that it is either a \u000A variable symbol or the application of a function symbol to the appropriate \u000A number of argument terms.  Traditionally, ``atomic formulas'' are built from \u000A terms with predicate symbols such as ``equal'' and ``member;'' ``formulas'' \u000A are then built from atomic formulas with propositional ``operators'' like \u000A ``not,'' ``and,'' and ``implies.'' Theorems are formulas.  Theorems are \u000A ``valid'' in the sense that the value of a theorem is true, in any model of \u000A the axioms and under all possible assignments of individuals to variables.</p> \u000A \u000A <p>However, in ACL2, terms are used in place of both atomic formulas and \u000A formulas.  ACL2 does not have predicate symbols or propositional operators as \u000A distinguished syntactic entities.  The ACL2 universe of individuals includes a \u000A ``true'' object (denoted by <v>t</v>) and a ``false'' object (denoted by \u000A <v>nil</v>), predicates and propositional operators are functions that return \u000A these objects.  Theorems in ACL2 are terms and the ``validity'' of a term \u000A means that, under no assignment to the variables does the term evaluate to \u000A <v>nil</v>.</p> \u000A \u000A <p>We use the word ``term'' in ACL2 in three distinct senses.  We will speak \u000A of ``translated'' terms, ``untranslated'' terms, and ``pseudo-'' terms.</p> \u000A \u000A <p><i>Translated Terms: The Strict Sense and Internal Form</i></p> \u000A \u000A <p>In its most strict sense, a ``term'' is either a legal variable symbol, a \u000A quoted constant, or the application of an n-ary function symbol or closed \u000A <v>lambda</v> expression to a true list of n terms.</p> \u000A \u000A <p>The legal variable symbols are symbols other than <v>t</v> or <v>nil</v> which \u000A are not in the keyword package, do not start with ampersand, do not start and \u000A end with asterisks, and if in the main Lisp package, do not violate an \u000A appropriate restriction (see <see topic=\"ACL2____NAME\">name</see>).</p> \u000A \u000A <p>Quoted constants are expressions of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> x)</v>, where <v>x</v> \u000A is any ACL2 object.  Such expressions may also be written <v>'x</v>.</p> \u000A \u000A <p>Closed <v>lambda</v> expressions are expressions of the form <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (v1\u000A ... vn) body)</v> where the <v>vi</v> are distinct legal variable symbols, \u000A <v>body</v> is a term, and the only free variables in <v>body</v> are among the \u000A <v>vi</v>.</p> \u000A \u000A <p>The function <v>termp</v>, which takes two arguments, an alleged term <v>x</v> \u000A and a logical world <v>w</v> (see <see topic=\"ACL2____WORLD\">world</see>), recognizes terms of a given \u000A extension of the logic.  <v>Termp</v> is defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode. \u000A Its definition may be inspected with <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> <v>termp</v> for a \u000A complete specification of what we mean by ``term'' in the most strict sense. \u000A Most ACL2 term-processing functions deal with terms in this strict sense and \u000A use <v>termp</v> as a <see topic=\"ACL2____GUARD\">guard</see>.  That is, the ``internal form'' of a term \u000A satisfies <v>termp</v>, the strict sense of the word ``term.''</p> \u000A \u000A <p><i>Untranslated Terms: What the User Types</i></p> \u000A \u000A <p>While terms in the strict sense are easy to explore (because their \u000A structure is so regular and simple) they can be cumbersome to type.  Thus, \u000A ACL2 supports a more sugary syntax that includes uses of macros and constant \u000A symbols.  Very roughly speaking, macros are functions that produce terms as \u000A their results.  Constants are symbols that are associated with quoted objects. \u000A Terms in this sugary syntax are ``translated'' to terms in the strict sense; \u000A the sugary syntax is more often called ``untranslated.''  Roughly speaking, \u000A translation just implements macroexpansion, the replacement of constant \u000A symbols by their quoted values, and the checking of all the rules governing \u000A the strict sense of ``term.''</p> \u000A \u000A <p>More precisely, macro symbols are as described in the documentation for \u000A <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>.  A macro, <v>mac</v>, can be thought of as a function, \u000A <v>mac-fn</v>, from ACL2 objects to an ACL2 object to be treated as an \u000A untranslated term.  For example, <tt><see topic=\"COMMON-LISP____CAAR\">caar</see></tt> is defined as a macro symbol; \u000A the associated macro function maps the object <v>x</v> into the object <v>(<see topic=\"COMMON-LISP____CAR\">car</see>\u000A (<see topic=\"COMMON-LISP____CAR\">car</see> x))</v>.  A macro form is a ``call'' of a macro symbol, i.e., a list whose \u000A <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is the macro symbol and whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is an arbitrary true \u000A list of objects, used as a term.  Macroexpansion is the process of replacing \u000A in an untranslated term every occurrence of a macro form by the result of \u000A applying the macro function to the appropriate arguments.  The ``appropriate'' \u000A arguments are determined by the exact form of the definition of the macro; \u000A macros support positional, keyword, optional and other kinds of arguments. \u000A See <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>.</p> \u000A \u000A <p>In addition to macroexpansion and constant symbol dereferencing, \u000A translation implements the mapping of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> forms into \u000A applications of <v>lambda</v> expressions and closes <v>lambda</v> expressions \u000A containing free variables.  Thus, the translation of</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____1_B2\">1+</see> i))) (<see topic=\"COMMON-LISP____CONS\">cons</see> x k))</code> \u000A \u000A <p>can be seen as a two-step process that first produces</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x k)) (<see topic=\"COMMON-LISP____1_B2\">1+</see> i))</code> \u000A \u000A <p>and then</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x k) (<see topic=\"COMMON-LISP____CONS\">cons</see> x k)) (<see topic=\"COMMON-LISP____1_B2\">1+</see> i) k) .</code> \u000A \u000A <p>Observe that the body of the <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and of the first <v>lambda</v> \u000A expression contains a free <v>k</v> which is finally bound and passed into the \u000A second <v>lambda</v> expression.</p> \u000A \u000A <p>Translation also maps <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt> forms into applications of <v>lambda</v> \u000A expressions.  See <see topic=\"COMMON-LISP____FLET\">flet</see>.</p> \u000A \u000A <p>When we say, of an event-level function such as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, that some argument ``must be a term'' we mean an untranslated term. \u000A The event functions translate their term-like arguments.</p> \u000A \u000A <p>To better understand the mapping between untranslated terms and translated \u000A terms it is convenient to use the keyword command <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt> to see \u000A examples of translations.  See <see topic=\"ACL2____TRANS\">trans</see> and also see <see topic=\"ACL2____TRANS1\">trans1</see>.</p> \u000A \u000A <p>Finally, we note that the theorem prover prints terms in untranslated form. \u000A But there can be more than one correct untranslated term corresponding to a \u000A given translated term.  For example, the translated term <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y 'nil)</v> \u000A can be untranslated as <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y nil)</v> and can also be untranslated as \u000A <v>(<see topic=\"COMMON-LISP____AND\">and</see> x y)</v>.  The theorem prover attempts to print an untranslated term \u000A that is as helpful to the user as possible.  In particular, consider a term of \u000A the form <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> k st)</v> where <v>st</v> is a single-threaded object (see <see topic=\"ACL2____STOBJ\">stobj</see>) and the <v>kth</v> accessor of <v>st</v> is, say, <v>kn</v>.  The theorem \u000A prover typically would expand <v>(kn st)</v> to <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> k st)</v>.  If <v>k</v> is \u000A large then it could be difficult for the user to make sense out of a proof \u000A transcript that mentions the expanded term.  Fortunately, the untranslation of \u000A <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> k st)</v> would be <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> *kn* st)</v>; here <v>*kn*</v> would be a \u000A constant (see <see topic=\"ACL2____DEFCONST\">defconst</see>) added by the <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event introducing \u000A <v>st</v>, defined to have value <v>k</v>.  The user can extend this user-friendly \u000A style of printing applications of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> to stobjs; see <see topic=\"ACL2____ADD-NTH-ALIAS\">add-nth-alias</see>.  These remarks about printing applications of function <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> extend naturally to function <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt>.  Moreover, the prover \u000A will attempt to treat terms as <see topic=\"ACL2____STOBJ\">stobj</see>s for the above purpose when \u000A appropriate.  For example, if function <v>foo</v> has <see topic=\"ACL2____SIGNATURE\">signature</see> <v>((foo\u000A * st) =&gt; (<see topic=\"ACL2____MV\">mv</see> * * * st))</v>, where <v>st</v> is introduced with <v>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st f0\u000A f1)</v>, then the <see topic=\"ACL2____TERM\">term</see> <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> '1 (<see topic=\"ACL2____MV-NTH\">mv-nth</see> '3 (foo x st0)))</v> will be \u000A printed as <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> *f1* (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 3 (foo x st0)))</v>.</p> \u000A \u000A <p><i>Pseudo-Terms: A Common Guard for Metafunctions</i></p> \u000A \u000A <p>Because <v>termp</v> is defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, it cannot be \u000A used effectively in conjectures to be proved.  Furthermore, from the \u000A perspective of merely guarding a term processing function, <v>termp</v> often \u000A checks more than is required.  Finally, because <v>termp</v> requires the \u000A logical <see topic=\"ACL2____WORLD\">world</see> as one of its arguments it is impossible to use \u000A <v>termp</v> as a <see topic=\"ACL2____GUARD\">guard</see> in places where the logical <see topic=\"ACL2____WORLD\">world</see> is not \u000A itself one of the arguments.</p> \u000A \u000A <p>For these reasons we support the idea of ``pseudo-terms.''  A pseudo-term \u000A is either a symbol (but not necessarily one having the syntax of a legal \u000A variable symbol), a true list beginning with <v>quote</v> (but not necessarily \u000A well-formed), or the ``application of'' a symbol or pseudo <v>lambda</v> \u000A expression to a true list of pseudo-terms.  A pseudo <v>lambda</v> expression is \u000A an expression of the form <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (v1 ... vn) body)</v> where the <v>vi</v> are \u000A all symbols and <v>body</v> is a pseudo-term.</p> \u000A \u000A <p>Pseudo-terms are recognized by the unary function <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt>.  If \u000A <v>(termp x w)</v> is true, then <v>(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)</v> is true.  However, if \u000A <v>x</v> fails to be a (strict) term it may nevertheless still be a pseudo-term. \u000A For example, <v>(<see topic=\"COMMON-LISP____CAR\">car</see> a b)</v> is not a term, because <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is applied to \u000A the wrong number of arguments, but it is a pseudo-term.</p> \u000A \u000A <p>The structures recognized by <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> can be recursively \u000A explored with the same simplicity that terms can be.  In particular, if <v>x</v> \u000A is not a <v>variablep</v> or an <v>fquotep</v>, then <v>(ffn-symb x)</v> is the \u000A function (<v>symbol</v> or <v>lambda</v> expression) and <v>(fargs x)</v> is the \u000A list of argument pseudo-terms.  A metafunction (see <see topic=\"ACL2____META\">meta</see>) or \u000A clause-processor (see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>) may use <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> as \u000A the <see topic=\"ACL2____GUARD\">guard</see>.</p>"],

 "ACL2____SET-GUARD-CHECKING":[["Guard"],"ACL2 Sources","ACL2","<p>Detailed comments about the arguments of this function may be found \u000A elsewhere: see <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see>.  Here we provide an introduction \u000A to the use of <v>set-guard-checking</v>.</p> \u000A \u000A <p>New users are encouraged to execute one of the following forms in order to \u000A avoid evaluation errors due to <see topic=\"ACL2____GUARD\">guard</see>s:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)\u000A(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> nil)</code> \u000A \u000A <p>The former avoids all guard-checking on user-defined functions and should \u000A generally work fine for new users, the only drawback being efficiency loss on \u000A compute-intensive problems.  All settings other than <v>:none</v> check guards, \u000A but a value of <v>nil</v> allows evaluation to continue in the logic when guards \u000A fail (avoiding the raw Lisp definition in that case).</p> \u000A \u000A <p>You may put one of the above forms in the <v>&quot;acl2-customization.lsp&quot;</v> \u000A file in your current directory (see <see topic=\"ACL2____CBD\">cbd</see>) or your home directory; see \u000A <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>Note that <see topic=\"ACL2____GUARD\">guard</see>s are not part of the ACL2 logic, and hence new users \u000A can completely ignore the notion of <see topic=\"ACL2____GUARD\">guard</see> (and the rest of this \u000A documentation section after this paragraph!).  For example, <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v> and \u000A <v>nil</v> can be proved equal in the ACL2 logic, as follows, even though the \u000A <see topic=\"ACL2____GUARD\">guard</see> on <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> requires its first argument to be a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> \u000A pair or <v>nil</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> 3) nil))</code> \u000A \u000A <p>Moreover, unless your functions or top-level forms call built-in ACL2 \u000A functions that are defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, the following \u000A property will hold.</p> \u000A \u000A <blockquote><p>Evaluation of <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)</v> will allow \u000A evaluation of forms such as <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v> to take place without error in the \u000A top level loop, not only when proving theorems.</p></blockquote> \u000A \u000A <p>If you feel bold, then you may wish to read the rest of this documentation \u000A topic; also see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see> for a succinct table, with associated \u000A discussion, that covers in detail the material presented in the rest of the \u000A present topic.</p> \u000A \u000A <p>The top-level ACL2 loop has a variable which controls which sense of \u000A execution is provided.  To turn ``<see topic=\"ACL2____GUARD\">guard</see> checking on,'' by which we mean \u000A that <see topic=\"ACL2____GUARD\">guard</see>s are checked at runtime, execute the top-level form \u000A <v>:set-guard-checking t</v>.  To allow guard violations, do \u000A <v>:set-guard-checking nil</v>, or do <v>:set-guard-checking :none</v> to turn off \u000A all guard-checking, so that raw Lisp definitions of user-defined functions are \u000A avoided unless their <see topic=\"ACL2____GUARD\">guard</see> is <v>t</v>. The status of guard-checking is \u000A reflected in the <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>means <see topic=\"ACL2____GUARD\">guard</see> checking is on and</p> \u000A \u000A <code>ACL2 &gt;</code> \u000A \u000A <p>means <see topic=\"ACL2____GUARD\">guard</see> checking is off.  The exclamation mark can be thought of \u000A as ``barring'' certain computations.  The absence of the mark suggests the \u000A absence of error messages or unbarred access to the logical axioms.  Thus, for \u000A example</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will signal an error, while</p> \u000A \u000A <code>ACL2 &gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will return <v>nil</v>.</p> \u000A \u000A <p>We will return at the end of this documentation topic to discuss two other \u000A values, <v>:all</v> and <v>:nowarn</v>, for <v>:set-guard-checking</v>.  We also \u000A note that evaluation of built-in <v>:program</v> mode functions always takes \u000A place in raw Lisp.</p> \u000A \u000A <p>Whether <see topic=\"ACL2____GUARD\">guard</see>s are checked during evaluation is independent of the \u000A <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt>.  We note this simply because it is easy to confuse \u000A ``<v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode'' with ``evaluation in Common Lisp'' and thus \u000A with ``<see topic=\"ACL2____GUARD\">guard</see> checking on;'' and it is easy to confuse ``<v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode'' with ``evaluation in the logic'' and with ``<see topic=\"ACL2____GUARD\">guard</see> \u000A checking off.''  But the <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt> determines whether newly \u000A submitted definitions introduce programs or add logical axioms.  That mode is \u000A independent of whether evaluation checks <see topic=\"ACL2____GUARD\">guard</see>s or not.  You can \u000A operate in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode with runtime <see topic=\"ACL2____GUARD\">guard</see> checking on or \u000A off.  Analogously, you can operate in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode with runtime \u000A <see topic=\"ACL2____GUARD\">guard</see> checking on or off.</p> \u000A \u000A <p>For further discussion on evaluation and guards see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>, in particular the exception for safe-mode in the \u000A ``Aside'' there.  See <see topic=\"ACL2____GUARD\">guard</see> for a general discussion of <see topic=\"ACL2____GUARD\">guard</see>s.</p> \u000A \u000A <p>Now we fulfill our promise above to discuss two other values for \u000A <v>:set-guard-checking</v>:</p> \u000A \u000A <code>:set-guard-checking :nowarn\u000A:set-guard-checking :all</code> \u000A \u000A <p>The meaning of these values is perhaps best described by the following \u000A example provided by David Rager.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-test (expr)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> expr)\u000A                         :verify-guards nil))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> expr)\u000A             expr\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> (my-test (<see topic=\"COMMON-LISP____CAR\">car</see> expr))\u000A                 (my-test (<see topic=\"COMMON-LISP____CDR\">cdr</see> expr)))))\u000A\u000AThe admission of MY-TEST is trivial, using the relation O&lt; (which is\u000Aknown to be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> EXPR).  We could deduce no constraints on the type of MY-\u000ATEST.  However, in normalizing the definition we used primitive type\u000Areasoning.\u000A\u000ASummary\u000AForm:  ( DEFUN MY-TEST ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A MY-TEST\u000AACL2 !&gt;(my-test '(a b c))\u000A\u000AACL2 Warning [Guards] in TOP-LEVEL:  Guard-checking will be inhibited\u000Aon recursive calls of the executable counterpart (i.e., in the ACL2\u000Alogic) of MY-TEST.  To check guards on all recursive calls:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :all)\u000ATo leave behavior unchanged except for inhibiting this message:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :nowarn)\u000A\u000A(A B C)\u000AACL2 !&gt;</code> \u000A \u000A <p>If you think about evaluation of <v>(my-test '(a b c))</v>, you will see that \u000A it leads to the recursive call <v>(my-test 'a)</v>, which one might expect to \u000A cause a guard violation since the symbol <v>a</v> is not a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>. \u000A However, as the warning above explains, we do not by default check guards on \u000A recursive calls.  The reason is efficiency &mdash; imagine a simple definition \u000A with a guard that is slow to evaluate.  The values <v>:nowarn</v> and <v>:all</v> \u000A for <v>:set-guard-checking</v> have been introduced as ways of dealing with the \u000A above warning.  The value <v>:nowarn</v> simply turns off the warning above. \u000A The value <v>:all</v> causes all guards to be checked, even on recursive calls \u000A and even on all calls of non-built-in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions \u000A &mdash; unless, of course, a call is made of a function whose guard has been \u000A verified (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>), where the arguments satisfy the guard, in \u000A which case the corresponding call is made in raw Lisp without subsidiary \u000A guard-checking.  We still say that ``guard-checking is on'' after \u000A <v>:set-guard-checking</v> is invoked with values <v>t</v>, <v>:nowarn</v>, and \u000A <v>:all</v>, otherwise (after value <v>nil</v>) we say ``guard-checking is \u000A off.</p> \u000A \u000A <p>For technical reasons, <v>:all</v> does not have its advertised effect in the \u000A case of built-in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>-mode functions.  If you are interested \u000A in this technical detail, see the comment ``In the boot-strap world...'' in \u000A source function <v>oneify-cltl-code</v>.</p> \u000A \u000A <p>We conclude with a remark about the use of <v>:set-guard-checking</v> for \u000A experimenting with ACL2 as a logic or as a programming language.  If one views \u000A ACL2 as a logic, one may wish to use <v>:set-guard-checking :none</v>, while if \u000A instead one views ACL2 as a functional programming language, one may wish to \u000A use <v>:set-guard-checking :all</v>.  The following transcript illustrates this \u000A distinction by way of example.  Specifically, <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v> is equal to \u000A <v>nil</v> in the ACL2 logic, but may be viewed as a programming error.  The \u000A default of <v>:set-guard-checking t</v> is problematic for learning ACL2 using \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions, since one can get raw Lisp errors.  In \u000A the example below, the raw Lisp error occurs because <v>foo</v> implicitly has a \u000A <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>, hence <v>(foo 3)</v> is evaluated in raw Lisp, which \u000A leads to a raw Lisp call of c[(car 3)].</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A         (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FOO\u000AACL2 !&gt;(foo 3)\u000AError: Attempt to take the car of 3 which is not listp.\u000A  [condition type: TYPE-ERROR]\u000A\u000ARestart actions (select using :continue):\u000A 0: Abort entirely from this (lisp) process.\u000A[Current process: Initial Lisp Listener]\u000A[1] ACL2(1): [RAW LISP] :pop\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(foo 3)\u000ANIL\u000AACL2 &gt;:set-guard-checking :all\u000A\u000ATurning guard checking on, value :ALL.\u000A\u000AACL2 !&gt;(foo 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol CAR, which\u000Ais (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL)), is violated by the arguments in the\u000Acall (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3).  See :DOC trace for a useful debugging utility.  See :DOC\u000Aset-guard-checking for information about suppressing this check with\u000A(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 !&gt;</code>"],

 "ACL2____DEFUN-SK":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-x-p0-and-q0 (y z)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> x\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z))))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-x-p0-and-q0 (y z) ; equivalent to the above\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> (x)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z))))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-x-y-p0-and-q0 (z)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> (x y)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z)))\u000A  :strengthen t)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> fn (var1 ... varn) body\u000A  &amp;key rewrite doc quant-ok skolem-name thm-name witness-dcls strengthen)</code> \u000A \u000A <p>where <v>fn</v> is the symbol you wish to define and is a new symbolic name \u000A (see <see topic=\"ACL2____NAME\">name</see>), <v>(var1 ... varn)</v> is its list of formal parameters (see \u000A <see topic=\"ACL2____NAME\">name</see>), and <v>body</v> is its body, which must be quantified as described \u000A below.  The <v>&amp;key</v> argument <tt><see topic=\"ACL2____DOC\">doc</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string to be associated with <v>fn</v>; for a description of its form, see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.  In the case that <v>n</v> is 1, the list <v>(var1)</v> may be \u000A replaced by simply <v>var1</v>.  The other arguments are explained below.</p> \u000A \u000A <p>For a simple example, see <see topic=\"ACL2____DEFUN-SK-EXAMPLE\">defun-sk-example</see>.  For a more elaborate \u000A example, see <see topic=\"ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE\">Tutorial4-Defun-Sk-Example</see>.  See <see topic=\"ACL2____QUANTIFIER-TUTORIAL\">quantifier-tutorial</see> for a careful beginner's introduction that takes you \u000A through typical kinds of quantifier-based reasoning in ACL2.  Also see <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see> for an example illustrating how the use of recursion, rather than \u000A explicit quantification with <v>defun-sk</v>, may be preferable.</p> \u000A \u000A <p>Below we describe the <v>defun-sk</v> event precisely.  First, let us \u000A consider the examples above.  The first example, again, is:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-x-p0-and-q0 (y z)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> x\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z))))</code> \u000A \u000A <p>It is intended to represent the predicate with formal parameters <v>y</v> and \u000A <v>z</v> that holds when for some <v>x</v>, <v>(<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))</v> \u000A holds.  In fact <v>defun-sk</v> is a macro that adds the following two <see topic=\"ACL2____EVENTS\">events</see>, as shown just below.  The first event guarantees that if this new \u000A predicate holds of <v>y</v> and <v>z</v>, then the term shown, \u000A <v>(exists-x-p0-and-q0-witness y z)</v>, is an example of the <v>x</v> that is \u000A therefore supposed to exist.  (Intuitively, we are axiomatizing \u000A <v>exists-x-p0-and-q0-witness</v> to pick a witness if there is one.  We comment \u000A below on the use of <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt>; for now, consider <v>defun-nx</v> to be \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.)  Conversely, the second event below guarantees that if there \u000A is any <v>x</v> for which the term in question holds, then the new predicate \u000A does indeed hold of <v>y</v> and <v>z</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> exists-x-p0-and-q0 (y z)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((x (exists-x-p0-and-q0-witness y z)))\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> exists-x-p0-and-q0-suff\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))\u000A           (exists-x-p0-and-q0 y z)))</code> \u000A \u000A <p>Now let us look at the third example from the introduction above:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-x-y-p0-and-q0 (z)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> (x y)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z))))</code> \u000A \u000A <p>The intention is to introduce a new predicate <v>(forall-x-y-p0-and-q0 z)</v> \u000A which states that the indicated conjunction holds of all <v>x</v> and all <v>y</v> \u000A together with the given <v>z</v>.  This time, the axioms introduced are as shown \u000A below.  The first event guarantees that if the application of function \u000A <v>forall-x-y-p0-and-q0-witness</v> to <v>z</v> picks out values <v>x</v> and <v>y</v> \u000A for which the given term <v>(<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))</v> holds, then the new \u000A predicate <v>forall-x-y-p0-and-q0</v> holds of <v>z</v>.  Conversely, the \u000A (contrapositive of) the second axiom guarantees that if the new predicate \u000A holds of <v>z</v>, then the given term holds for all choices of <v>x</v> and \u000A <v>y</v> (and that same <v>z</v>).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> forall-x-y-p0-and-q0 (z)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (x y)\u000A          (forall-x-y-p0-and-q0-witness z)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-x-y-p0-and-q0-necc\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z)))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (forall-x-y-p0-and-q0 z))))</code> \u000A \u000A <p>The examples above suggest the critical property of <v>defun-sk</v>: it \u000A indeed does introduce the quantified notions that it claims to introduce.</p> \u000A \u000A <p>Notice that the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event just above, \u000A <v>forall-x-y-p0-and-q0-necc</v>, may not be of optimal form as a rewrite rule. \u000A Users sometimes find that when the quantifier is <v>forall</v>, it is useful to \u000A state this rule in a form where the new quantified predicate is a hypothesis \u000A instead.  In this case that form would be as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-x-y-p0-and-q0-necc\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (forall-x-y-p0-and-q0 z)\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z) (q0 x y z))))</code> \u000A \u000A <p>ACL2 will turn this into one <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule for each conjunct, \u000A <v>(p0 x y z)</v> and <v>(q0 x y z)</v>, with hypothesis <v>(forall-x-y-p0-and-q0\u000A z)</v> in each case.  In order to get this effect, use <v>:rewrite :direct</v>, in \u000A this case as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-x-y-p0-and-q0 (z)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> (x y)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (p0 x y z)\u000A               (q0 x y z)))\u000A  :rewrite :direct)</code> \u000A \u000A <p>We now turn to a detailed description of <v>defun-sk</v>, starting with a \u000A discussion of its arguments as shown in the \"General Form\" above.</p> \u000A \u000A <p>The third argument, <v>body</v>, must be of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____Q\">Q</see> bound-vars term)</code> \u000A \u000A <p>where: <v>Q</v> is the symbol <tt><see topic=\"ACL2____FORALL\">forall</see></tt> or <tt><see topic=\"ACL2____EXISTS\">exists</see></tt> (in the \u000A \"ACL2\" package), <v>bound-vars</v> is a variable or true list of variables \u000A disjoint from <v>(var1 ... varn)</v> and not including <tt><see topic=\"ACL2____STATE\">state</see></tt>, and \u000A <v>term</v> is a term.  The case that <v>bound-vars</v> is a single variable \u000A <v>v</v> is treated exactly the same as the case that <v>bound-vars</v> is \u000A <v>(v)</v>.</p> \u000A \u000A <p>The result of this event is to introduce a ``Skolem function,'' whose name \u000A is the keyword argument <v>skolem-name</v> if that is supplied, and otherwise is \u000A the result of modifying <v>fn</v> by suffixing \"-WITNESS\" to its name.  The \u000A following definition and one of the following two theorems (as indicated) are \u000A introduced for <v>skolem-name</v> and <v>fn</v> in the case that <v>bound-vars</v> \u000A (see above) is a single variable <v>v</v>.  The name of the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event \u000A may be supplied as the value of the keyword argument <v>:thm-name</v>; if it is \u000A not supplied, then it is the result of modifying <v>fn</v> by suffixing \u000A \"-SUFF\" to its name in the case that the quantifier is <tt><see topic=\"ACL2____EXISTS\">exists</see></tt>, and \u000A \"-NECC\" in the case that the quantifier is <tt><see topic=\"ACL2____FORALL\">forall</see></tt>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> fn (var1 ... varn)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((v (skolem-name var1 ... varn)))\u000A    term))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-suff ;in case the quantifier is EXISTS\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> term\u000A           (fn var1 ... varn)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-necc ;in case the quantifier is FORALL\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> term)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (fn var1 ... varn))))</code> \u000A \u000A <p>In the <v>forall</v> case, however, the keyword pair <v>:rewrite :direct</v> \u000A may be supplied after the body of the <v>defun-sk</v> form, in which case the \u000A contrapositive of the above form is used instead:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-necc ;in case the quantifier is FORALL\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (fn var1 ... varn)\u000A           term))</code> \u000A \u000A <p>This is often a better choice for the \"-NECC\" rule, provided ACL2 can \u000A parse <v>term</v> as a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule.  A second possible value of \u000A the <v>:rewrite</v> argument of <v>defun-sk</v> is <v>:default</v>, which gives the \u000A same behavior as when <v>:rewrite</v> is omitted.  Otherwise, the value of \u000A <v>:rewrite</v> should be the term to use as the body of the <v>fn-necc</v> \u000A theorem shown above; ACL2 will attempt to do the requisite proof in this case. \u000A If that term is weaker than the default, the properties introduced by \u000A <v>defun-sk</v> may of course be weaker than they would be otherwise.  Finally, \u000A note that the <v>:rewrite</v> keyword argument for <v>defun-sk</v> only makes \u000A sense if the quantifier is <v>forall</v>; it is thus illegal if the quantifier \u000A is <v>exists</v>.  Enough said about <v>:rewrite</v>!</p> \u000A \u000A <p>In the case that <v>bound-vars</v> is a list of at least two variables, say \u000A <v>(bv1 ... bvk)</v>, the definition above (with no keywords) is the following \u000A instead, but the theorem remains unchanged.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> fn (var1 ... varn)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (bv1 ... bvk)\u000A          (skolem-name var1 ... varn)\u000A          term))</code> \u000A \u000A <p>In order to emphasize that the last element of the list, <v>body</v>, is a \u000A term, <v>defun-sk</v> checks that the symbols <tt><see topic=\"ACL2____FORALL\">forall</see></tt> and <tt><see topic=\"ACL2____EXISTS\">exists</see></tt> \u000A do not appear anywhere in it.  However, on rare occasions one might \u000A deliberately choose to violate this convention, presumably because <tt><see topic=\"ACL2____FORALL\">forall</see></tt> or <tt><see topic=\"ACL2____EXISTS\">exists</see></tt> is being used as a variable or because a macro call \u000A will be eliminating ``calls of'' <tt><see topic=\"ACL2____FORALL\">forall</see></tt> and <tt><see topic=\"ACL2____EXISTS\">exists</see></tt>.  In these \u000A cases, the keyword argument <v>quant-ok</v> may be supplied a non-<v>nil</v> \u000A value.  Then <v>defun-sk</v> will permit <tt><see topic=\"ACL2____FORALL\">forall</see></tt> and <tt><see topic=\"ACL2____EXISTS\">exists</see></tt> in \u000A the body, but it will still cause an error if there is a real attempt to use \u000A these symbols as quantifiers.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt> above, rather than <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, disables \u000A certain checks that are required for evaluation, in particular the \u000A single-threaded use of <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s.  However, there is a price: calls of \u000A these defined functions cannot be evaluated; see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>.  Normally \u000A that is not a problem, since these notions involve quantifiers.  But you are \u000A welcome to replace this <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form with your own, as follows: if you \u000A supply a list of <v>declare</v> forms to keyword argument <v>:witness-dcls</v>, \u000A these will become the declare forms in the generated <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  Note that \u000A if your value of <v>witness-dcls</v> does not contain the form <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A (<see topic=\"ACL2____XARGS\">xargs</see> :non-executable t))</v>, then the appropriate wrapper for non-executable \u000A functions will not be added automatically, i.e., <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> will be used in \u000A place of <v>defun-nx</v>.  Note also that if <see topic=\"ACL2____GUARD\">guard</see> verification is \u000A attempted, then it will likely fail with an error message complaining that \u000A ``guard verification may depend on local properties.''  In that case, you may \u000A wish to delay guard verification, as in the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> foo (x)\u000A   (<see topic=\"ACL2____EXISTS\">exists</see> n (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                  (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n x)))\u000A   :witness-dcls ((<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                                  :verify-guards nil))))\u000A (<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo))</code> \u000A \u000A <p><v>Defun-sk</v> is a macro implemented using <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>.  Hence, it \u000A should only be executed in <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>; see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and see <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  Advanced feature: If argument \u000A <v>:strengthen t</v> is passed to <v>defun-sk</v>, then <v>:strengthen t</v> will \u000A generate the extra constraint that that is generated for the corresponding \u000A <v>defchoose</v> event; see <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  You can use the command \u000A <v>:</v><tt><see topic=\"ACL2____PCB_12\">pcb!</see></tt> to see the event generated by a call of the <v>defun-sk</v> \u000A macro.</p> \u000A \u000A <p>If you find that the rewrite rules introduced with a particular use of \u000A <v>defun-sk</v> are not ideal, even when using the <v>:rewrite</v> keyword \u000A discussed above (in the <v>forall</v> case), then at least two reasonable \u000A courses of action are available for you.  Perhaps the best option is to prove \u000A the <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules you want.  If you see a pattern for creating rewrite \u000A rules from your <v>defun-sk</v> events, you might want to write a macro that \u000A executes a <v>defun-sk</v> followed by one or more <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events. \u000A Another option is to write your own variant of the <v>defun-sk</v> macro, say, \u000A <v>my-defun-sk</v>, for example by modifying a copy of the definition of \u000A <v>defun-sk</v> from the ACL2 sources.</p> \u000A \u000A <p>If you want to represent nested quantifiers, you can use more than one \u000A <v>defun-sk</v> event.  For example, in order to represent</p> \u000A \u000A <code>(<see topic=\"ACL2____FORALL\">forall</see> x (<see topic=\"ACL2____EXISTS\">exists</see> y (p x y z)))</code> \u000A \u000A <p>you can use <v>defun-sk</v> twice, for example as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-y-p (x z)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> y (p x y z)))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-x-exists-y-p (z)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> x (exists-y-p x z)))</code> \u000A \u000A <p>Some distracting and unimportant warnings are inhibited during \u000A <v>defun-sk</v>.</p> \u000A \u000A <p>Note for ACL2(r) users (see <see topic=\"COMMON-LISP____REAL\">real</see>): In ACL2(r), the keyword \u000A <v>:CLASSICALP</v> is also supported.  Its legal values are <v>t</v> (the default) \u000A and <v>nil</v>, and it determines whether or not (respectively) ACL2(r) will \u000A consider <v>fn</v> to be a classical function.  It must be the case that the \u000A value is <v>t</v> (perhaps implicitly, by default) if and only if <v>body</v> is \u000A classical.</p> \u000A \u000A <p>Note that this way of implementing quantifiers is not a new idea.  Hilbert \u000A was certainly aware of it 60 years ago!  Also see <see topic=\"ACL2____CONSERVATIVITY-OF-DEFCHOOSE\">conservativity-of-defchoose</see> for a technical argument that justifies the \u000A logical conservativity of the <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> event in the sense of the \u000A paper by Kaufmann and Moore entitled ``Structured Theory Development for a \u000A Mechanized Logic'' (Journal of Automated Reasoning 26, no. 2 (2001), \u000A pp. 161-203).</p>"],

 "ACL2____MONITOR":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____MONITOR\">monitor</see> '(:rewrite assoc-of-app) 't)\u000A:monitor (:rewrite assoc-of-app) t\u000A:monitor (:definition app) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :target) '(app c d))\u000A:monitor (:linear rule3) t\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____MONITOR\">monitor</see> rune term)</code> \u000A \u000A <p>where <v>rune</v> is a <see topic=\"ACL2____RUNE\">rune</see> and <v>term</v> is a term, called the \u000A ``break condition.'' <v>Rune</v> must be a rune corresponding to a rule of class \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>.</p> \u000A \u000A <p>When a <see topic=\"ACL2____RUNE\">rune</see> is <see topic=\"ACL2____MONITOR\">monitor</see>ed any attempt to apply it may result \u000A in an interactive break in an ACL2 ``<see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see>.'' There you \u000A will get a chance to see how the application proceeds.  See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> for a description of the interactive loop entered.  Whether an \u000A interactive break occurs depends on the value of the break condition \u000A expression associated with the <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>.</p> \u000A \u000A <p>NOTE: Some <v>:rewrite</v> rules are considered ``simple abbreviations''; see \u000A <see topic=\"ACL2____SIMPLE\">simple</see>.  These can be be monitored, but only at certain times during \u000A the proof.  Monitoring is carried out by code inside the rewriter but \u000A abbreviation rules may be applied by a special purpose simplifier inside the \u000A so-called <i>preprocess</i> phase of a proof.  If you desire to monitor an \u000A abbreviation rule, a warning will be printed suggesting that you may want to \u000A supply the hint <v>:DO-NOT '(PREPROCESS)</v>; see <see topic=\"ACL2____HINTS\">hints</see>.  Without such a \u000A hint, an abbreviation rule can be applied during the preprocess phase of a \u000A proof, and no such application will cause an interactive break.</p> \u000A \u000A <p>To remove a <see topic=\"ACL2____RUNE\">rune</see> from the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s, use \u000A <v>unmonitor</v>.  To see which <see topic=\"ACL2____RUNE\">rune</see>s are <see topic=\"ACL2____MONITOR\">monitor</see>ed and what their \u000A break conditions are, evaluate <v>(<see topic=\"ACL2____MONITORED-RUNES\">monitored-runes</see>)</v>.</p> \u000A \u000A <p><v>Monitor</v>, <v>unmonitor</v> and <v>monitored-runes</v> are macros that \u000A expand into expressions involving <v>state</v>.  While these macros appear to \u000A return the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s this is an illusion.  They \u000A all print <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see> information to the comment window and \u000A then return error triples (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) instructing <v>ld</v> to \u000A print nothing.  It is impossible to return the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s because it exists only in the <see topic=\"ACL2____WORMHOLE\">wormhole</see> <see topic=\"ACL2____STATE\">state</see> with which \u000A you interact when a break occurs.  This allows you to change the <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s and their conditions during the course of a proof \u000A attempt without changing the <see topic=\"ACL2____STATE\">state</see> in which the the proof is being \u000A constructed.</p> \u000A \u000A <p>Unconditional break points are obtained by using the break condition \u000A <v>t</v>.  We now discuss conditional break points.  The break condition, \u000A <v>expr</v>, must be a term that contains no free variables other than \u000A <v>state</v> and that returns a single non-<v>state</v> result.  In fact, the \u000A result should be <v>nil</v>, <v>t</v>, or a true list of commands to be fed to the \u000A resulting interactive break.  Whenever the system attempts to use the \u000A associated rule, <v>expr</v> is evaluated in the <see topic=\"ACL2____WORMHOLE\">wormhole</see> interaction \u000A <see topic=\"ACL2____STATE\">state</see>.  A break occurs only if the result of evaluating <v>expr</v> is \u000A non-<v>nil</v>.  If the result is a true list, that list is appended to the \u000A front of <v>standard-oi</v> and hence is taken as the initial user commands \u000A issued to the interactive break.</p> \u000A \u000A <p>In order to develop effective break conditions it must be possible to \u000A access context sensitive information, i.e., information about the context in \u000A which the <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see> is being tried.  The <tt><see topic=\"ACL2____BRR_04\">brr@</see></tt> macro \u000A may be used in break conditions to access such information as the term being \u000A rewritten and the current governing assumptions.  This information is not \u000A stored in the proof <see topic=\"ACL2____STATE\">state</see> but is transferred into the <see topic=\"ACL2____WORMHOLE\">wormhole</see> \u000A <see topic=\"ACL2____STATE\">state</see> when breaks occur.  The macro form is <v>(<see topic=\"ACL2____BRR_04\">brr@</see> :sym)</v> where \u000A <v>:sym</v> is one of several keyword symbols, including <v>:target</v> (the term \u000A being rewritten), <v>:unify-subst</v> (the substitution that instantiates the \u000A left-hand side of the conclusion of the rule so that it is the target term), \u000A and <v>:</v><tt><see topic=\"ACL2____TYPE-ALIST\">type-alist</see></tt> (the governing assumptions).  See <see topic=\"ACL2____BRR_04\">brr@</see>.</p> \u000A \u000A <p>For example,</p> \u000A \u000A <code>ACL2 !&gt;:monitor (:rewrite assoc-of-app)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :target) '(app a (app b c)))</code> \u000A \u000A <p>will monitor <v>(:rewrite assoc-of-app)</v> but will cause an interactive \u000A break only when the target term, the term being rewritten, is <v>(app a (app b\u000A c))</v>.</p> \u000A \u000A <p>Because break conditions are evaluated in the interaction environment, the \u000A user developing a break condition for a given <see topic=\"ACL2____RUNE\">rune</see> can test candidate \u000A break conditions before installing them.  For example, suppose an \u000A unconditional break has been installed on a <see topic=\"ACL2____RUNE\">rune</see>, that an interactive \u000A break has occurred and that the user has determined both that this particular \u000A application is uninteresting and that many more such applications will likely \u000A occur.  An appropriate response would be to develop an expression that \u000A recognizes such applications and returns <v>nil</v>.  Of course, the hard task \u000A is figuring out what makes the current application uninteresting.  But once a \u000A candidate expression is developed, the user can evaluate it in the current \u000A context simply to confirm that it returns <v>nil</v>.</p> \u000A \u000A <p>Recall that when a break condition returns a non-<v>nil</v> true list that \u000A list is appended to the front of <v>standard-oi</v>.  For example,</p> \u000A \u000A <code>ACL2 !&gt;:monitor (:rewrite assoc-of-app) '(:go)</code> \u000A \u000A <p>will cause <v>(:rewrite assoc-of-app)</v> to be <see topic=\"ACL2____MONITOR\">monitor</see>ed and will \u000A make the break condition be <v>'(:go)</v>.  This break condition always \u000A evaluates the non-<v>nil</v> true list <v>(:go)</v>.  Thus, an interactive break \u000A will occur every time <v>(:rewrite assoc-of-app)</v> is tried.  The break is fed \u000A the command <v>:go</v>.  Now the command <v>:go</v> causes <v>break-rewrite</v> to \u000A (a) evaluate the attempt to apply the lemma, (b) print the result of that \u000A attempt, and (c) exit from the interactive break and let the proof attempt \u000A continue.  Thus, in effect, the above <v>:monitor</v> merely ``traces'' the \u000A attempted applications of the <see topic=\"ACL2____RUNE\">rune</see> but never causes an interactive \u000A break requiring input from the user.</p> \u000A \u000A <p>It is possible to use this feature to cause a conditional break where the \u000A effective break condition is tested <b>after</b> the lemma has been tried. \u000A For example:</p> \u000A \u000A <code>ACL2 !&gt;:monitor (:rewrite lemma12)\u000A                '(:unify-subst\u000A                  :eval$ nil\u000A                  :ok-if (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp))\u000A                             (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :rewritten-rhs) '(foo a))))\u000A                  :rewritten-rhs)</code> \u000A \u000A <p>causes the following behavior when <v>(:rewrite lemma12)</v> is tried.  A \u000A break always occurs, but it is fed the commands above.  The first, \u000A <v>:unify-subst</v>, causes <v>break-rewrite</v> to print out the unifying \u000A substitution.  Then in response to <v>:eval$</v> <v>nil</v> the lemma is tried but \u000A with all <see topic=\"ACL2____RUNE\">rune</see>s temporarily <see topic=\"ACL2____UNMONITOR\">unmonitor</see>ed.  Thus no breaks will \u000A occur during the rewriting of the hypotheses of the lemma.  When the attempt \u000A has been made, control returns to <v>break-rewrite</v> (which will print the \u000A results of the attempt, i.e., whether the lemma was applied, if so what the \u000A result is, if not why it failed).  The next command, the <v>:ok-if</v> with its \u000A following expression, is a conditional exit command.  It means exit \u000A <v>break-rewrite</v> if either the attempt was unsuccessful, <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____BRR_04\">brr@</see>\u000A :wonp))</v>, or if the result of the rewrite is any term other than <v>(foo\u000A a)</v>.  If this condition is met, the break is exited and the remaining break \u000A commands are irrelevant.  If this condition is not met then the next command, \u000A <v>:rewritten-rhs</v>, prints the result of the application (which in this \u000A contrived example is known to be <v>(foo a)</v>).  Finally, the list of supplied \u000A commands is exhausted but <v>break-rewrite</v> expects more input.  Therefore, \u000A it begins prompting the user for input.  The end result, then, of the above \u000A <v>:monitor</v> command is that the <see topic=\"ACL2____RUNE\">rune</see> in question is elaborately \u000A traced and interactive breaks occur whenever it rewrites its target to <v>(foo\u000A a)</v>.</p> \u000A \u000A <p>We recognize that the above break condition is fairly arcane.  We suspect \u000A that with experience we will develop some useful idioms.  For example, it is \u000A straightforward now to define macros that monitor <see topic=\"ACL2____RUNE\">rune</see>s in the ways \u000A suggested by the following names: <v>trace-rune</v>, <v>break-if-target-is</v>, \u000A and <v>break-if-result-is</v>.  For example, the last could be defined as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> break-if-result-is (<see topic=\"ACL2____RUNE\">rune</see> term)\u000A  `(<see topic=\"ACL2____MONITOR\">monitor</see> ',rune\u000A            '(<see topic=\"COMMON-LISP____QUOTE\">quote</see> (:eval :ok-if\u000A                           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :rewritten-rhs) ',term))))))</code> \u000A \u000A <p>(Note however that the submitted term must be in translated form.)</p> \u000A \u000A <p>Since we don't have any experience with this kind of control on lemmas we \u000A thought it best to provide a general (if arcane) mechanism and hope that the \u000A ACL2 community will develop the special cases that we find most \u000A convenient.</p>"],

 "ACL2____DOC-STRING":[["Legacy-documentation"],"ACL2 Sources","ACL2","<code>Examples:\u000A&quot;:Doc-Section name\u000Aone-liner~/notes~/details&quot;\u000A\u000A&quot;:Doc-Section name\u000Aone-liner~/\u000Anotes~/\u000Adetails~/\u000A:cite old-name1\u000A:cited-by old-name2&quot;</code> \u000A \u000A <p>Use <v>(get-doc-string 'name state)</v> to see other examples.</p> \u000A \u000A <p><see topic=\"COMMON-LISP____DOCUMENTATION\">Documentation</see> strings not beginning with ``<v>:Doc-Section</v>'' (case \u000A is irrelevant) are ignored.  See <see topic=\"ACL2____MARKUP\">markup</see> for how to supply formatting \u000A information (such as fonts and displayed text) in <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A strings.</p> \u000A \u000A <p>ACL2 attaches special importance to <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings beginning \u000A with the header ``<v>:Doc-Section</v>'' (or any variant thereof obtained by \u000A changing case).  Any <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string that does not begin with such \u000A a header is considered unformatted and is ignored.  For the rest of this \u000A discussion, we use the phrase ``<see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string'' as though it \u000A read ``formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string.''</p> \u000A \u000A <p><see topic=\"COMMON-LISP____DOCUMENTATION\">Documentation</see> strings are always processed in the context of some \u000A symbol, <v>name</v>, being defined.  (Indeed, if an event defines no symbol, \u000A e.g., <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> or <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>, then it is not permitted to \u000A have a formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string.)  The string will be associated \u000A with name in the ``<see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> database.'' The database is divided \u000A into ``sections'' and each section is named by a symbol.  Among the sections \u000A are <tt><see topic=\"ACL2____EVENTS\">events</see></tt>, <tt><see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see></tt>, <tt><see topic=\"ACL2____HISTORY\">history</see></tt>, <tt><see topic=\"ACL2____OTHER\">other</see></tt>, and \u000A <tt><see topic=\"ACL2____MISCELLANEOUS\">miscellaneous</see></tt>.  A complete list of the sections may be obtained by \u000A typing <v>:docs *</v> at the terminal.  You can create new sections.  The main \u000A purpose of sections is simply to partition the large set of names into smaller \u000A subsets whose contents can be enumerated separately.  The idea is that the \u000A user may remember (or recognize) the relevant section name and then read its \u000A contents to find interesting items.</p> \u000A \u000A <p>Within a section are ``<see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> tuples'' which associate with \u000A each documented name its <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string and a list of related \u000A documented names, called the ``related names'' of the name.  When <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> prints the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for name, it always lists the related \u000A names.</p> \u000A \u000A <p>When a formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string is submitted with the defining \u000A event of some name, the section name and an initial set of related names are \u000A parsed from the string.  In addition, the formatted string contains various \u000A ``levels'' of detail that are printed out at different times.  Finally, it is \u000A possible for a string to cause the newly documented name to be added to the \u000A related names of any previously documented name.  Thus, as new names are \u000A introduced they can be grouped with old ones.</p> \u000A \u000A <p>The general form of an ACL2 formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string is</p> \u000A \u000A <code>&quot;:DOC-SECTION &lt;section-name&gt;\u000A  &lt;one-liner&gt;~/\u000A  &lt;notes&gt;~/\u000A  &lt;details&gt;~/\u000A  :CITE &lt;n1&gt;\u000A  ...\u000A  :CITE &lt;nn&gt;\u000A  :CITED-BY &lt;m1&gt;\u000A  ...\u000A  :CITED-BY &lt;mm&gt;&quot;</code> \u000A \u000A <p>Before we explain this, let it be noted that <v>(get-doc-string name\u000A state)</v> will return the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string associated with <v>name</v> \u000A in the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> database.  You may want to call \u000A <v>get-doc-string</v> on <v>'</v><tt><see topic=\"ACL2____PE\">pe</see></tt> and <v>'</v><tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt> just \u000A to see some concrete <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings.  This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string, which is rather long, is under <v>'doc-string</v>.</p> \u000A \u000A <p>A formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string has five parts: the header and \u000A section-name (terminating in the first <v>#\\Newline</v>), the <v>&lt;one-liner&gt;</v>, \u000A <v>&lt;notes&gt;</v>, and <v>&lt;details&gt;</v> (each terminating in a tilde-slash \u000A (``<v>~/</v>'') pair), and a citation part.  These five parts are parsed into \u000A six components.  <v>&lt;section-name&gt;</v> is read as the name of a symbol, \u000A section-name.  <v>&lt;one-liner&gt;</v>, <v>&lt;notes&gt;</v>, and <v>&lt;details&gt;</v> are \u000A arbitrary sequences of <see topic=\"ACL2____CHARACTERS\">characters</see> (ignoring initial white space and not \u000A including the tilde-slash pairs which terminate them).  The <v>&lt;ni&gt;</v> are read \u000A as symbols and assembled into a list called the ``cite'' symbols.  The \u000A <v>&lt;mi&gt;</v> are read as symbols and assembled into a list called the \u000A ``cited-by'' symbols.  See the warning below regarding the hackish nature of \u000A our symbol reader.</p> \u000A \u000A <p><v>Section-name</v> must either be a previously documented symbol or else be \u000A <v>name</v>, the symbol being documented.  To open a new section of the \u000A database, named <v>section-name</v>, you should define the logical name \u000A section-name (as by <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> or any other event; also see <see topic=\"ACL2____DEFDOC\">defdoc</see>) and attach to it a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string for section \u000A section-name.  You might wish to print out the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string we \u000A use for some of our section names, e.g., <v>(get-doc-string 'events state)</v>. \u000A By forcing section names to be documented symbols, we permit sections \u000A themselves to have one line descriptions and discussions, presented by the \u000A standard <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> facilities like the facilities <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> \u000A and <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt> that may be used at the terminal.</p> \u000A \u000A <p>Each of the <v>ni</v>'s and <v>mi</v>'s must be previously documented \u000A symbols.</p> \u000A \u000A <p>Both <v>&lt;one-liner&gt;</v> and <v>&lt;details&gt;</v> must be non-empty, i.e., must \u000A contain some non-whitespace <see topic=\"ACL2____CHARACTERS\">characters</see>.  <v>&lt;notes&gt;</v> may be empty. \u000A The <v>:cite</v>s and <v>:cited-by</v>s pairs may be intermingled and may be \u000A separated by either newlines or spaces.  The citation part may be empty.  When \u000A the citation part is empty, the tilde-slash pair terminating the \u000A <v>&lt;details&gt;</v> part may be omitted.  Thus, the simplest form of a formatted \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string is:</p> \u000A \u000A <code>&quot;:Doc-Section &lt;section-name&gt;\u000A &lt;one-liner&gt;~/~/\u000A &lt;details&gt;&quot;</code> \u000A \u000A <p>Since white space at the front of <v>&lt;one-liner&gt;</v>, <v>&lt;notes&gt;</v> and \u000A <v>&lt;details&gt;</v> is ignored, we often precede those parts by <v>#\\Newline</v>s to \u000A make the strings easier to read in our source files.  We also typically indent \u000A all of the text in the string by starting each line with a few spaces.  (The \u000A Emacs commands for formatting Lisp get confused if you have arbitrary <see topic=\"ACL2____CHARACTERS\">characters</see> on the left margin.)  We assume that every line in \u000A <v>&lt;one-liner&gt;</v>, <v>&lt;notes&gt;</v>, and <v>&lt;details&gt;</v> starts with at least as \u000A many spaces as <v>&lt;one-liner&gt;</v> does, i.e., we assume they are all indented \u000A the same amount (or more).  Let <v>d</v> be the number of spaces separating \u000A <v>&lt;one-liner&gt;</v> from the <v>#\\Newline</v> preceding it.  When the various \u000A parts are printed, we ``de-indent'' by stripping out the first d spaces \u000A following each <v>#\\Newline</v>.</p> \u000A \u000A <p>However, we find that when <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> is printed flush against \u000A the left margin it is difficult to distinguish the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> text \u000A from previous output.  We therefore prefix each line we print by a special pad \u000A of <see topic=\"ACL2____CHARACTERS\">characters</see>.  By default, this pad is ``<v>| </v>'' so that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> text has a vertical bar running down the left margin.  But the \u000A pad is just the value of the global variable <v>doc-prefix</v> and you may \u000A <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> it any string you wish.</p> \u000A \u000A <p>To add such a string to the database under the symbol <v>name</v> we make a \u000A new entry in the section-name section of the database.  The entry associates \u000A <v>name</v> with the string and uses the string's cites list as the initial \u000A value of the related names field.  In addition, we add <v>name</v> to the \u000A related names field of each of the names listed in the string's cited-by list. \u000A We also add <v>name</v> to the related names field of its section-name.  Observe \u000A that the cites list in a string is only the initial value of the related names \u000A of the names.  Future <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings may add to it via \u000A <v>:cited-by</v> or <v>:Doc-Section</v>.  Indeed, this is generally the case.  We \u000A discuss this further below.</p> \u000A \u000A <p>When a brief description of <v>name</v> is required (as by <v>:docs **</v>), \u000A <v>name</v> and <v>&lt;one-liner&gt;</v> are printed.  <v>&lt;one-liner&gt;</v> is usually \u000A printed starting in column 15 (however see <see topic=\"ACL2____PRINT-DOC-START-COLUMN\">print-doc-start-column</see>). \u000A Despite its name, <v>&lt;one-liner&gt;</v> need not be one line.  It usually is one \u000A line, however.</p> \u000A \u000A <p>When you type <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> name at the terminal, the first response \u000A will be to print <v>name</v> and <v>&lt;one-liner&gt;</v>.  Then <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> \u000A prints <v>&lt;notes&gt;</v>, if any.  Then, if <v>name</v> is the name of a section, it \u000A prints the <v>&lt;one-liner&gt;</v>s for each of its related names.  For example, try \u000A <v>:doc events</v>.  If <v>name</v> is not a section name but does have some \u000A related names, they are merely listed but not explained.  Try <v>:doc\u000A theory-functions</v>.  <v>:more-doc name</v> prints <v>&lt;details&gt;</v>.</p> \u000A \u000A <p>Our style is to let each new concept add itself to the related names of old \u000A concepts.  To do otherwise increases the chances that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A gets outdated because one often forgets to update supposedly complete lists of \u000A the relevant topics when new topics are invented.  For example, <v>:doc\u000A theory-functions</v> lists each available theory function.  But \u000A <v>get-doc-string</v> of <v>'</v><tt><see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see></tt> just shows a few \u000A examples and has an empty cites list.  From where do we get the names of the \u000A theory functions listed by <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>?  The answer is that each theory \u000A function has its own <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string and those strings each \u000A specify <v>:cited-by</v> <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see>.  See for example \u000A <v>get-doc-string</v> of <v>'</v><tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt>.  So by the time the \u000A entire system is assembled, the related names of <v>'</v><tt><see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see></tt> contains all the (documented) theory functions.  This makes \u000A it easy to add new theory functions without changing the general discussion in \u000A <v>'</v><tt><see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see></tt>.</p> \u000A \u000A <p>When an event or <see topic=\"ACL2____COMMAND\">command</see> form is printed, as by <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt>, that contains a formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string, we do \u000A not print the actual <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (since they are usually large \u000A and distracting).  Instead we print the string:</p> \u000A \u000A <code>&quot;Documentation available via :doc&quot;</code> \u000A \u000A <p>inviting you to use <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> and <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt> (or \u000A <v>get-doc-string</v>) if you wish to see the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> at the \u000A terminal.</p> \u000A \u000A <p><i>Warning on Reading Symbols from Strings:</i> When we read a symbol, such \u000A as the section-symbol, from a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string, we use a quick and \u000A dirty imitation of the much more powerful CLTL <v>read</v> program.  In \u000A particular, we scan past any whitespace, collect all the <see topic=\"ACL2____CHARACTERS\">characters</see> we \u000A see until we get to more whitespace or the end of the string, convert the \u000A <see topic=\"ACL2____CHARACTERS\">characters</see> to upper case, make a string out of them, and <tt><see topic=\"COMMON-LISP____INTERN\">intern</see></tt> \u000A that string.  Thus, if you typed <v>&quot;:Doc-Section 123 ...&quot;</v> we would read \u000A the <v>123</v> as the symbol <v>|123|</v>.  Observe that special <see topic=\"ACL2____CHARACTERS\">characters</see>, such as parentheses and escape <see topic=\"ACL2____CHARACTERS\">characters</see>, are not \u000A afforded their usual reverence by our hack.  Furthermore, the question arises: \u000A in which package do we <tt><see topic=\"COMMON-LISP____INTERN\">intern</see></tt> the symbol?  The answer is, usually, the \u000A package containing the name being defined.  I.e., if you are documenting \u000A <v>my-pkg::name</v> and you attach a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string that begins \u000A <v>&quot;:Doc-Section: Machines ...&quot;</v> then the section-symbol will be \u000A <v>my-pkg::machines</v>.  We recognize two special cases. If the first character \u000A read is a colon, we use the <v>keyword</v> package.  If the first five <see topic=\"ACL2____CHARACTERS\">characters</see> read are <v>acl2::</v> then we <see topic=\"COMMON-LISP____INTERN\">intern</see> in the <v>&quot;ACL2&quot;</v> \u000A package.  Our own section names, e.g., <tt><see topic=\"ACL2____EVENTS\">events</see></tt>, are in the \u000A <v>&quot;ACL2&quot;</v> package.</p> \u000A \u000A <p>In a related area, when you ask for the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of a name, \u000A e.g., when you type <v>:doc name</v> at the terminal, that name is read with the \u000A full ACL2 reader, not the hack just described.  That name is read into the \u000A current package.  Thus, if you are operating <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> \u000A <v>&quot;MY-PKG&quot;</v> and type <v>:doc events</v>, what is read is \u000A <v>my-pkg::events</v>.  The database may not contain an entry for this symbol. \u000A Before reporting that no <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> exists, we try \u000A <v>acl2::events</v>.</p> \u000A \u000A <p>One last note: <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> permits a formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string, which is associated in the database with the name of the package.  But \u000A the name of the package is a string, not a symbol.  It is permitted to access \u000A the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of a string (i.e., package name).  But there are no \u000A facilities for getting such a <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> name into the related names of \u000A another name nor of making such <see topic=\"COMMON-LISP____STRINGP\">stringp</see> names be section names.  That \u000A is because we always read symbols from strings and never read strings from \u000A strings.  I.e., if you did write <v>&quot;Doc-Section \\&quot;MY-PKG\\&quot; ...&quot;</v> it \u000A would read in as a weird symbol.</p>"],

 "ACL2____DEFATTACH":[["Events"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is organized into the following \u000A sections:</p> \u000A \u000A <p><b>Introductory example.</b><br></br> \u000A \u000A <b>Syntax and semantics of defattach.</b><br></br> \u000A \u000A <b>Three primary uses of defattach.</b><br></br> \u000A \u000A <b>Miscellaneous remarks, with discussion of possible user errors.</b></p> \u000A \u000A <p>Please see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> if you intend to use <v>defattach</v> but are \u000A not already familiar with the use of <v>encapsulate</v> to introduce constrained \u000A functions.</p> \u000A \u000A <p>See community book <v>books/misc/defattach-example.lisp</v> for a small \u000A example.  it illustrates how <v>defattach</v> may be used to build something \u000A like ``higher-order'' programs, in which constrained functions may be refined \u000A to different executable functions.  More uses of <v>defattach</v> may be found \u000A in the ACL2 source code, specifically, file <v>boot-strap-pass-2.lisp</v>.</p> \u000A \u000A <p>The argument <v>:skip-checks t</v> enables easy experimentation with \u000A <v>defattach</v>, by permitting use of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions and \u000A the skipping of semantic checks.  Also permitted is <v>:skip-checks nil</v> (the \u000A default) and <v>:skip-checks :cycles</v>, which turns off only the update of the \u000A extended ancestor relation (see below) and hence the check for cycles in this \u000A relation; see below.  We do not make any logical claims when the value of \u000A <v>:skip-checks</v> is non-<v>nil</v>; indeed, a trust tag is required in this \u000A case (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  Remark for those who use the experimental HONS \u000A extension (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>): the interaction of memoization \u000A and attachments is not tracked for attachments introduced with a non-<v>nil</v> \u000A value of <v>:skip-checks</v>.  For more discussion of <v>:skip-checks t</v>, see \u000A <see topic=\"ACL2____DEFPROXY\">defproxy</see>; we do not discuss <v>:skip-checks</v> further, here.</p> \u000A \u000A <p><b>Introductory example.</b></p> \u000A \u000A <p>We begin with a short log illustrating the use of <v>defattach</v>.  Notice \u000A that after evaluating the event <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)</v>, a call of the \u000A constrained function <v>f</v> is evaluated by instead calling <v>g</v> on the \u000A arguments.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A        ((f (x) t :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x))\u000A        (<see topic=\"ACL2____DEFTHM\">defthm</see> f-property\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> (f x)))))\u000A[... output omitted ...]\u000A T\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____NULL\">null</see> x))))\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> 17 (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A[... output omitted ...]\u000A G\u000AACL2 !&gt;(f '(3 4)) ; undefined function error\u000A\u000AACL2 Error in TOP-LEVEL:  ACL2 cannot ev the call of undefined function\u000AF on argument list:\u000A\u000A((3 4))\u000A\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)\u000A[... output omitted ...]\u000A :ATTACHMENTS-RECORDED\u000AACL2 !&gt;(f '(3 4)) ; f is evaluated using g\u000A(17 . 3)\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> f g)\u000A ((F) (G))\u000AACL2 !&gt;(f '(3 4)) ; f is evaluated using g\u000A1&gt; (ACL2_*1*_ACL2::F (3 4))\u000A  2&gt; (ACL2_*1*_ACL2::G (3 4))\u000A    3&gt; (G (3 4))\u000A    &lt;3 (G (17 . 3))\u000A  &lt;2 (ACL2_*1*_ACL2::G (17 . 3))\u000A&lt;1 (ACL2_*1*_ACL2::F (17 . 3))\u000A(17 . 3)\u000AACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> f nil) ; unattach f (<see topic=\"COMMON-LISP____REMOVE\">remove</see> its attachment)\u000A[... output omitted ...]\u000A :ATTACHMENTS-RECORDED\u000AACL2 !&gt;(f '(3 4)) ; undefined function error once again\u000A1&gt; (ACL2_*1*_ACL2::F (3 4))\u000A\u000AACL2 Error in TOP-LEVEL:  ACL2 cannot ev the call of undefined function\u000AF on argument list:\u000A\u000A((3 4))\u000A\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Syntax and semantics of defattach.</b></p> \u000A \u000A <p>The log above shows that the event <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)</v> allows <v>g</v> to be \u000A used for evaluating calls of <v>f</v>.  From a logical perspective, the \u000A evaluation takes place in the addition to the current session of an \u000A ``attachment equation'' axiom (universally quantified over all <v>x</v>) for \u000A each <v>defattach</v> event:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) (g x)) ;;; attachment equation axiom for (<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)</code> \u000A \u000A <p>Below we explain <v>defattach</v> in some detail.  But it is important to \u000A keep in mind that evaluation with the attachment equations takes place in an \u000A extension of the logical theory of the session.  ACL2 guarantees that this \u000A so-called ``evaluation theory'' remains consistent, assuming the absence of \u000A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> <see topic=\"ACL2____EVENTS\">events</see> from the user.  This guarantee is a consequence \u000A of a more general guarantee: an ACL2 logical <see topic=\"ACL2____WORLD\">world</see> exists in which \u000A (loosely speaking) the attachment equation for <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)</v>, as \u000A <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (...) (g ...))</v>, takes the place of the original defining event \u000A for <v>f</v>, for each <v>defattach</v> event.  This more general guarantee holds \u000A even if there are <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> events, though as explained below, no \u000A function symbol that syntactically supports a <v>defaxiom</v> formula is allowed \u000A to get an attachment.  A deeper discussion of the logical issues is available \u000A (but not intended to be read by most users) in a long comment in the ACL2 \u000A source code labeled ``Essay on Defattach.''</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)   ; call g in place of calling constrained function f\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g)) ; same as just above\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo)))))\u000A                  ; equivalent to first form above, except with hints for the\u000A                  ; proof that the guard of f implies the guard of g\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo)))\u000A                :otf-flg t))\u000A                  ; as above, except with an :otf-flg of t for the proof that\u000A                  ; the guard of f implies the guard of g\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g)\u000A           :hints ((&quot;Goal&quot; :use my-thm)))\u000A                  ; equivalent to first form above, except with hints for the\u000A                  ; proof that the constraints on f hold for g\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g)\u000A           :hints ((&quot;Goal&quot; :use my-thm))\u000A           :otf-flg t)\u000A                  ; as above, except with an :otf-flg of t for the proof that\u000A                  ; the constraints on f hold for g\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g)\u000A           (h j)) ; Attach g to f and attach j to h\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g :attach nil)\u000A           (h j)) ; Same as just above, including the same proof obligations,\u000A                  ; except for one difference: because of :attach nil, calls\u000A                  ; of f will not be evaluated, i.e., there will be no\u000A                  ; executable attachment of g to f\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f nil)\u000A           (h j)) ; Attach j to h and unattach f\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo))))\u000A           (h j :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> bar))))\u000A           :hints ((&quot;Goal&quot; :use my-thm)))\u000A                  ; Attach g to f and attach j to h, with hints:\u000A                  ; - For proving that the guard of f implies the guard of g,\u000A                  ;   enable foo;\u000A                  ; - For proving that the guard of h implies the guard of j,\u000A                  ;   enable bar; and\u000A                  ; - For proving that the constraints on f and h hold for\u000A                  ;   g and j (respectively), use theorem my-thm.\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f nil)   ; remove the attachment of f, if any (e.g., g above)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f nil)) ; same as just above\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)   ; single attach or, if g is nil, unattach\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f1 g1 :kwd val ...)\u000A           ...\u000A           (fk gk :kwd' val' ...)\u000A           :kwd'' val'' ...)</code> \u000A \u000A <p>where each indicated keyword-value pair is optional and each keyword is one \u000A of <v>:ATTACH</v>, <v>:HINTS</v>, <v>:OTF-FLG</v>, or <v>:INSTRUCTIONS</v>.  The value \u000A of each <v>:ATTACH</v> keyword is either <v>t</v> or <v>nil</v>, with default <v>t</v> \u000A except that the value of <v>:ATTACH</v> at the ``top level,'' after each entry \u000A <v>(fi gi ...)</v>, is the default for each <v>:ATTACH</v> keyword supplied in \u000A such an entry.  We discuss the <v>:ATTACH</v> keyword later in this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.  The associated values for the other keywords have the \u000A usual meanings for the proof obligations described below: the guard proof \u000A obligation for keywords within each <v>(fi gi ...)</v> entry, and the constraint \u000A proof obligation for keywords at the top level.  No keyword may occur twice in \u000A the same context, i.e., within the same <v>(fi gi ...)</v> entry or at the top \u000A level; and <v>:INSTRUCTIONS</v> may not occur in the same context with \u000A <v>:HINTS</v> or <v>:OTF-FLG</v>.</p> \u000A \u000A <p>The first General Form above is simply an abbreviation for the form \u000A <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g))</v>, which is an instance of the second General Form above. \u000A For the second General Form we say that <v>gi</v> is ``attached to'' <v>fi</v> (by \u000A the <v>defattach</v> event) if <v>gi</v> is not <v>nil</v>, and otherwise we say \u000A that <v>fi</v> is ``unattached'' (by the <v>defattach</v> event).  It is also \u000A convenient to refer to <v>&lt;fi,gi&gt;</v> as an ``attachment pair'' (of the event) \u000A if <v>gi</v> is not <v>nil</v>.  We may refer to the set of <v>fi</v> as the \u000A ``attachment nest'' of each <v>fi</v>.</p> \u000A \u000A <p>We start with a brief introduction to the first General Form in the case \u000A that <v>g</v> is not <v>nil</v>.  This form arranges that during evaluation, with \u000A exceptions noted below, every call of the constrained function symbol <v>f</v> \u000A will in essence be replaced by a call of the function symbol <v>g</v> on the \u000A same arguments.  We may then refer to <v>g</v> as the ``attachment of'' <v>f</v>, \u000A or say that ``<v>g</v> is attached to <v>f</v>.''  Notable exceptions, where we do \u000A not use attachments during evaluation, are for macroexpansion, evaluation of \u000A <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> and <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> terms, evaluation during <tt><see topic=\"ACL2____TABLE\">table</see></tt> \u000A events, some <see topic=\"ACL2____STOBJ\">stobj</see> operations including all <see topic='ACL2____STOBJ'>updates</see>, and especially evaluation of ground terms (terms \u000A without free variables) during proofs.  However, even for these cases we allow \u000A the use of attachments in the first argument of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> and, more \u000A generally, the next-to-last (i.e., second) argument of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> \u000A when its first argument is not of the form <v>'m</v> for some macro, <v>m</v>.</p> \u000A \u000A <p>To see why attachments are disallowed during evaluation of ground terms \u000A during proofs (except for the <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> and <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> cases \u000A mentioned above), consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f (x) t)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP_____B2\">+</see> 3 x))\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)</code> \u000A \u000A <p>If the form <v>(f 2)</v> is submitted at the ACL2 prompt, the result will be \u000A <v>5</v> because the attachment <v>g</v> of <v>f</v> is called on the argument, \u000A <v>2</v>.  However, during a proof the term <v>(f 2)</v> will not be simplified to \u000A <v>5</v>, since that would be unsound, as there are no axioms about <v>f</v> that \u000A would justify such a simplification.</p> \u000A \u000A <p>For the case that <v>g</v> is <v>nil</v> in the first General Form above, the \u000A result is the removal of the existing attachment to <v>f</v>, if any.  After \u000A this removal, calls of <v>f</v> will once again cause errors saying that ``ACL2 \u000A cannot ev the call of undefined function <v>f</v> ...''.  In this case not only \u000A is the previous attachment to <v>f</v> removed; moreover, for every function \u000A symbol <v>f'</v> in the attachment nest of <v>f</v> in the <v>defattach</v> event \u000A that introduced the existing attachment to <v>f</v>, then <v>f'</v> is unattached. \u000A (An example near the end of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic shows why this \u000A unattachment needs to be done.) Such removal takes place before the current \u000A <v>defattach</v> is processed, but is restored if the new event fails to be \u000A admitted.</p> \u000A \u000A <p>We focus henceforth on the second General Form.  There must be at least one \u000A attachment, i.e., <v>i</v> must be at least 1.  All keywords are optional; their \u000A role is described below.  The <v>fi</v> must be distinct constrained function \u000A symbols, that is, function symbols all introduced in <see topic=\"ACL2____SIGNATURE\">signature</see>s of \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> (or macros such as <tt><see topic=\"ACL2____DEFSTUB\">defstub</see></tt> that \u000A generate <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events).  Each non-<v>nil</v> <v>gi</v> is a \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode function symbol that has had its guards verified, \u000A with the same <see topic=\"ACL2____SIGNATURE\">signature</see> as <v>fi</v> (though formal parameters for \u000A <v>fi</v> and <v>gi</v> may have different names).  (Note: The macro \u000A <v>defattach!</v>, defined in community book <v>books/misc/defattach-bang</v>, \u000A avoids this restriction.)  This event generates proof obligations and an \u000A ordering check, both described below.  The effect of this event is first to \u000A remove any existing attachments for all the function symbols <v>fi</v>, as \u000A described above for the first General Form, and then to attach each <v>gi</v> to \u000A <v>fi</v>.</p> \u000A \u000A <p>Proof obligations must be checked before making attachments.  For this \u000A discussion we assume that each <v>gi</v> is non-<v>nil</v> (otherwise first remove \u000A all attachment pairs <v>&lt;fi,gi&gt;</v> for which <v>gi</v> is nil).  Let <v>s</v> be \u000A the functional substitution mapping each <v>fi</v> to <v>gi</v>.  For any term \u000A <v>u</v>, we write <v>u\\s</v> for the result of applying <v>s</v> to <v>u</v>; that \u000A is, <v>u\\s</v> is the ``functional instance'' obtained by replacing each \u000A <v>fi</v> by <v>gi</v> in <v>u</v>.  Let <v>G_fi</v> and <v>G_gi</v> be the guards of \u000A <v>fi</v> and <v>gi</v>, respectively.  Let <v>G_fi'</v> be the result of replacing \u000A each formal of <v>fi</v> by the corresponding formal of <v>gi</v> in <v>G_fi</v>. \u000A ACL2 first proves, for each <v>i</v> (in order), the formula <v>(<see topic=\"ACL2____IMPLIES\">implies</see> G_fi'\u000A G_gi)\\s</v>.  If this sequence of proofs succeeds, then the remaining formula \u000A to prove is the functional instance <v>C\\s</v> of the conjunction <v>C</v> of the \u000A constraints on the symbols <v>fi</v>; see <see topic=\"ACL2____CONSTRAINT\">constraint</see>.  This last proof \u000A obligation is thus similar to the one generated by functional instantiation \u000A (see <see topic=\"ACL2____CONSTRAINT\">constraint</see>).  As with functional instantiation, ACL2 stores the \u000A fact that such proofs have been done so that they are avoided in future events \u000A (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>).  Thus, you will likely avoid some proofs with the \u000A sequence</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f nil)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f nil)\u000A...</code> \u000A \u000A <p>rather than the sequence:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)\u000A:u\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g)\u000A:u\u000A...</code> \u000A \u000A <p>It remains to describe an ordering check.  We begin with the following \u000A motivating example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f (x) t) ; constrained function with no constraints\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (<see topic=\"COMMON-LISP____NOT\">not</see> (f x)))\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f g) ; ILLEGAL!</code> \u000A \u000A <p>Were the above <v>defattach</v> event to succeed, the evaluation theory \u000A (discussed above) would be inconsistent: <v>(f x)</v> equals <v>(g x)</v> by the \u000A new attachment equation, which in turn equals <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (f x))</v> by definition \u000A of <v>g</v>.  The evaluation would therefore be meaningless.  Also, from a \u000A practical perspective, there would be an infinite loop resulting from any call \u000A of <v>f</v>.</p> \u000A \u000A <p>We consider a function symbol <v>g</v> to be an ``extended immediate ancestor \u000A of'' a function symbol <v>f</v> if either of the following two criteria is met: \u000A (a) <v>g</v> occurs in the formula that introduces <v>f</v> (i.e., definition body \u000A or constraint) and <v>g</v> is introduced by an event different from (earlier \u000A than) the event introducing <v>f</v>; or (b) <v>g</v> is attached to <v>f</v>.  For \u000A a proposed <v>defattach</v> event, we check that this relation has no cycles, \u000A where for condition (b) we include all attachment pairs that would result, \u000A including those remaining from earlier <v>defattach</v> events.</p> \u000A \u000A <p>Of course, a special case is that no function symbol may be attached to \u000A itself.  Similarly, no function symbol may be attached to any of its \u000A ``siblings'' &mdash; function symbols introduced by the same event &mdash; as \u000A siblings are considered equivalent for purposes of the acyclicity check.</p> \u000A \u000A <p><b>Three primary uses of defattach.</b><br></br></p> \u000A \u000A <p>We anticipate three uses of <v>defattach</v>:</p> \u000A \u000A <p>(1) Constrained function execution</p> \u000A \u000A <p>(2) Sound modification of the ACL2 system</p> \u000A \u000A <p>(3) Program refinement</p> \u000A \u000A <p>We discuss these in turn.</p> \u000A \u000A <p>(1) The example at the beginning of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> illustrates \u000A constrained function execution.</p> \u000A \u000A <p>(2) ACL2 is written essentially in itself.  Thus, there is an opportunity \u000A to attaching to system functions.  For example, encapsulated function \u000A <v>too-many-ifs-post-rewrite</v>, in the ACL2 source code, receives an \u000A attachment of <v>too-many-ifs-post-rewrite-builtin</v>, which implements a \u000A heuristic used in the rewriter.  To find all such examples, search the source \u000A code for the string `-builtin'.</p> \u000A \u000A <p>Over time, we expect to continue replacing ACL2 source code in a similar \u000A manner.  We invite the ACL2 community to assist in this ``open architecture'' \u000A enterprise; feel free to email the ACL2 implementors if you are interested in \u000A such activity.</p> \u000A \u000A <p>(3) Recall that for an attachment pair <v>&lt;f,g&gt;</v>, a proof obligation is \u000A (speaking informally) that <v>g</v> satisfies the constraint on <v>f</v>.  Yet \u000A more informally speaking, <v>g</v> is ``more defined'' than <v>f</v>; we can think \u000A of <v>g</v> as ``refining'' <v>f</v>.  With these informal notions as motivation, \u000A we can view defattach as providing refinement though the following formal \u000A observation: the evaluation theory extends the theory of the ACL2 session, \u000A specifically by the addition of all attachment equations.  For the \u000A logic-inclined, it may be useful to think model-theoretically: The class of \u000A models of the evaluation theory is non-empty but is a subset of the class of \u000A models of the current session theory.</p> \u000A \u000A <p><b>Miscellaneous remarks, with discussion of possible user errors.</b></p> \u000A \u000A <p>We conclude with remarks on some details.</p> \u000A \u000A <p>A <v>defattach</v> event is never redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>); in \u000A that sense it is analogous to <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>.</p> \u000A \u000A <p>As mentioned above, the use of attachments is disabled for evaluation of \u000A ground terms during proofs.  However, attachments can be used on code during \u000A the proof process, essentially when the ``program refinement'' is on theorem \u000A prover code rather than on functions we are reasoning about.  The attachment \u000A to <v>too-many-ifs-post-rewrite</v> described above provides one example of such \u000A attachments.  Meta functions and clause-processor functions can also have \u000A attachments, with the restriction that no common ancestor with the evaluator \u000A can have an attachment; see <see topic=\"ACL2____EVALUATOR-RESTRICTIONS\">evaluator-restrictions</see>.</p> \u000A \u000A <p>For an attachment pair <v>&lt;f,g&gt;</v>, evaluation of <v>f</v> never consults the \u000A <see topic=\"ACL2____GUARD\">guard</see> of <v>f</v>.  Rather, control passes to <v>g</v>, whose guard is \u000A checked if necessary.  The proof obligation related to guards, as described \u000A above, guarantees that any legal call of <v>f</v> is also a legal call of \u000A <v>g</v>.  Thus for guard-verified code that results in calls of <v>f</v> in raw \u000A Lisp, it is sound to replace these calls with corresponding calls of \u000A <v>g</v>.</p> \u000A \u000A <p><v>Defattach</v> events are illegal inside any <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event with \u000A a non-empty <see topic=\"ACL2____SIGNATURE\">signature</see> unless they are <see topic=\"ACL2____LOCAL\">local</see> to the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p> \u000A \u000A <p>We next discuss a restriction based on a notion of a function symbol \u000A syntactically supporting an event.  Function symbol <v>f</v> is <i>ancestral</i> \u000A in event <v>E</v> if either <v>f</v> occurs in <v>E</v>, or (recursively) <v>f</v> \u000A occurs in an event <v>E'</v> that introduces some function symbol <v>g</v> that is \u000A ancestral in <v>E</v>.  We require that no function symbol ancestral in the \u000A formula of a <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> event may have an attachment.  Theoretical \u000A reasons are discussed in comments in the ACL2 source code, but here we give a \u000A little example showing the need for some such restriction: without it, we show \u000A how to prove <v>nil</v>!</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFN\">defn</see> g1 () 1)\u000A(<see topic=\"ACL2____DEFN\">defn</see> g2 () 2)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f1 () t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f2 () t)\u000A(<see topic=\"ACL2____DEFUND\">defund</see> p (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A  t)\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A  ((p x)))\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> f1-is-f2\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f1) (f2)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> meta-fn (x)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f1) (f2))\u000A         x)\u000A        (t *nil*)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad-meta-rule\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)\u000A         (evl (meta-fn x) a))\u000A  :rule-classes ((:meta :trigger-fns (p))))\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f1 g1)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f2 g2)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use ((:instance (:theorem (<see topic=\"COMMON-LISP____NOT\">not</see> (p x)))\u000A                                   (x t)))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>To see all attachments: <v>(all-attachments (w state))</v>.  (Note that \u000A attachments introduced with a non-<v>nil</v> value of <v>:skip-checks</v> will be \u000A omitted from this list.)</p> \u000A \u000A <p>Next we discuss the <v>:ATTACH</v> keyword.  There is rarely if ever a reason \u000A to specify <v>:ATTACH T</v>, but the following (admittedly contrived) example \u000A shows why it may be necessary to specify <v>:ATTACH NIL</v>.  First we introduce \u000A three new function symbols.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f (x) t)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A  (f x))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ((h (x) t))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x) (g x)))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> h-prop\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (h x) (g x))))</code> \u000A \u000A <p>Now suppose we want to attach the function <tt><see topic=\"ACL2____ACL2-NUMBERP\">ACL2-numberp</see></tt> to both \u000A <v>f</v> and <v>h</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f acl2-numberp) (h acl2-numberp))</code> \u000A \u000A <p>Such an attempt fails, because the following constraint is generated but is \u000A not a theorem: <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X) (G X))</v>.  Clearly we also need to \u000A attach to <v>g</v> as well.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f acl2-numberp) (h acl2-numberp) (g acl2-numberp))</code> \u000A \u000A <p>But this fails for a different reason, as explained by the error \u000A message:</p> \u000A \u000A <code>ACL2 Error in ( DEFATTACH (F ACL2-NUMBERP) ...):  It is illegal to\u000Aattach to function symbol G, because it was introduced with DEFUN.\u000ASee :DOC defattach.</code> \u000A \u000A <p>That is: logically, we need to attach <v>acl2-numberp</v> to <v>g</v>, but we \u000A cannot actually attach to <v>g</v> because it was introduced with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, \u000A not with <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  So we specify <v>:ATTACH NIL</v> for the \u000A attachment to <v>g</v>, saying that no actual attachment should be made to the \u000A code for <v>g</v>, even though for logical purposes we should consider that \u000A <v>g</v> has been given the indicated attachment.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f acl2-numberp) (h acl2-numberp) (g acl2-numberp :attach nil))</code> \u000A \u000A <p>Finally, we can check that <v>f</v>, <v>g</v>, and <v>h</v> execute as \u000A expected.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____AND\">and</see> (f 3)\u000A                   (<see topic=\"COMMON-LISP____NOT\">not</see> (f t))\u000A                   (g 3)\u000A                   (<see topic=\"COMMON-LISP____NOT\">not</see> (g t))\u000A                   (h 3)\u000A                   (<see topic=\"COMMON-LISP____NOT\">not</see> (h t))))\u000A :PASSED\u000AACL2 !&gt;</code> \u000A \u000A <p>We conclude with an example promised above, showing why it is necessary in \u000A general to unattach all function symbols in an existing attachment nest when \u000A unattaching any one of those function symbols.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f1 () t)\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ((f2 () t))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 () (f1)))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> f2=f1 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f2) (f1))))\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ((f3 () t))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f3 () (f1)))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> f3=f1 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f3) (f1))))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> four () (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) 4)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> five () (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) 5)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f1 four) (f2 four))\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f1 five) (f3 five))</code> \u000A \u000A <p>The second <v>defattach</v> replaces erases the existing attachment pair \u000A <v>&lt;f1,four&gt;</v> before installing the new attachment pairs <v>&lt;f1,five&gt;</v> and \u000A <v>&lt;f3,five&gt;</v>.  After the second defattach, both <v>(f1)</v> and <v>(f3)</v> \u000A evaluate to 5.  Now suppose that the attachment pair <v>&lt;f2,four&gt;</v> were not \u000A erased.  Then we would have <v>(f1)</v> evaluating to 5 and <v>(f2)</v> evaluating \u000A to 4, contradicting the constraint <v>f2=f1</v>.  The evaluation theory would \u000A thus be inconsistent, and at a more concrete level, the user might well be \u000A surprised by evaluation results if the code were written with the assumption \u000A specified in the constraint <v>f2=f1</v>.</p>"],

 "ACL2____MBE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The macro <v>mbe</v> (``must be equal'') can be used in function \u000A definitions in order to cause evaluation to use alternate code to that \u000A provided for the logic.  An example is given below.  However, the use of \u000A <v>mbe</v> can lead to non-terminating computations.  See <see topic=\"ACL2____DEFEXEC\">defexec</see>, \u000A perhaps after reading the present documentation, for a way to prove \u000A termination.</p> \u000A \u000A <p>In the ACL2 logic, <v>(<see topic=\"ACL2____MBE\">mbe</see> :exec exec-code :logic logic-code)</v> equals \u000A <v>logic-code</v>; the value of <v>exec-code</v> is ignored.  However, in raw Lisp \u000A it is the other way around: this form macroexpands simply to <v>exec-code</v>. \u000A ACL2's <see topic=\"ACL2____GUARD\">guard</see> verification mechanism ensures that the raw Lisp code is \u000A only evaluated when appropriate, since the guard proof obligations generated \u000A for (the macroexpansion of) this call of <v>mbe</v> include not only the guard \u000A proof obligations from <v>exec-code</v>, but also, under suitable contextual \u000A assumptions, the term <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> exec-code logic-code)</v>.  See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> (in particular, for discussion of the contextual assumptions \u000A from the <v>:guard</v> and <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>-tests) and, for general discussion of \u000A guards, see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p>Normally, during evaluation of an <v>mbe</v> call, only the <v>:logic</v> code \u000A is evaluated unless the call is in the body of a <see topic=\"ACL2____GUARD\">guard</see>-verified \u000A function or under a call of a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function; in those \u000A cases only the <v>:exec</v> code is evaluated.  This implies that equality of \u000A <v>:exec</v> and <v>:logic</v> code is never checked at runtime. (Rather, such \u000A equality is proved when verifying guards.)  We qualified with ``normally'' \u000A above because there are two exceptions.  During a ``safe mode'', which is used \u000A in macroexpansion and evaluation of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> forms, the <v>:logic</v> \u000A and <v>:exec</v> code are both evaluated and their equality is checked.  Second, \u000A when guard-checking is set to <v>:all</v> or <v>:none</v>, then for any <v>mbe</v> \u000A call in the body of a <v>:logic</v> mode definition, only the <v>:logic</v> code \u000A will be evaluated.</p> \u000A \u000A <p>Note that the <v>:exec</v> and the <v>:logic</v> code in an <v>mbe</v> call must \u000A have the same output signature.  For example, one cannot return <v>(</v><tt><see topic=\"ACL2____MV\">mv</see></tt><v> * *)</v> while the other returns just a single value.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____MBT\">mbt</see>, which stands for ``must be true.''  You may find it \u000A more natural to use <tt><see topic=\"ACL2____MBT\">mbt</see></tt> for certain applications, as described in its \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>Here is an example of the use of <v>mbe</v>.  Suppose that you want to define \u000A factorial in the usual recursive manner, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      1\u000A    (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>But perhaps you want to be able to execute calls of <v>fact</v> on large \u000A arguments that cause stack overflows, perhaps during proofs.  (This isn't a \u000A particularly realistic example, but it should serve.)  So, instead you can \u000A define this tail-recursive version of factorial:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact1 (n acc)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> acc))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      acc\u000A    (fact1 (<see topic=\"COMMON-LISP____1-\">1-</see> n) (<see topic=\"COMMON-LISP_____A2\">*</see> n acc))))</code> \u000A \u000A <p>We are now ready to define <v>fact</v> using <v>mbe</v>.  Our intention is that \u000A logically, <v>fact</v> is as shown in the first definition above, but that \u000A <v>fact</v> should be executed by calling <v>fact1</v>.  Notice that we defer \u000A <see topic=\"ACL2____GUARD\">guard</see> verification, since we are not ready to prove the correspondence \u000A between <v>fact1</v> and <v>fact</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0))\u000A                  :verify-guards nil))\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :exec  (fact1 n 1)\u000A       :logic (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A                  1\u000A                (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n))))))</code> \u000A \u000A <p>Next, we prove the necessary correspondence lemmas.  Notice the inclusion \u000A of a community book to help with the arithmetic reasoning.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;books/arithmetic/top-with-meta&quot;)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fact1-fact\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> acc)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fact1 n acc)\u000A                  (<see topic=\"COMMON-LISP_____A2\">*</see> acc (fact n)))))</code> \u000A \u000A <p>We may now do guard verification for <v>fact</v>, which will allow the \u000A execution of the raw Lisp <v>fact</v> function, where the above <v>mbe</v> call \u000A expands simply to <v>(fact1 n 1)</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)</code> \u000A \u000A <p>Now that guards have been verified, a trace of function calls illustrates \u000A that the evaluation of calls of <v>fact</v> is passed to evaluation of calls of \u000A <v>fact1</v>.  The outermost call below is of the logical function stored for \u000A the definition of <v>fact</v>; all the others are of actual raw Common Lisp \u000A functions.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact fact1)\u000ANIL\u000AACL2 !&gt;(fact 3)\u000A1&gt; (ACL2_*1*_ACL2::FACT 3)\u000A  2&gt; (FACT 3)\u000A    3&gt; (FACT1 3 1)\u000A      4&gt; (FACT1 2 3)\u000A        5&gt; (FACT1 1 6)\u000A          6&gt; (FACT1 0 6)\u000A          &lt;6 (FACT1 6)\u000A        &lt;5 (FACT1 6)\u000A      &lt;4 (FACT1 6)\u000A    &lt;3 (FACT1 6)\u000A  &lt;2 (FACT 6)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 6)\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p>You may occasionally get warnings when you compile functions defined using \u000A <v>mbe</v>.  (For commands that invoke the compiler, see <see topic=\"ACL2____COMPILATION\">compilation</see>.) \u000A These can be inhibited by using an <v>ignorable</v> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form.  Here \u000A is a simple but illustrative example.  Note that the declarations can \u000A optionally be separated into two <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> x)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)))\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :logic x :exec y))</code> \u000A \u000A <p>Finally, we observe that when the body of a function contains a term of the \u000A form <v>(<see topic=\"ACL2____MBE\">mbe</see> :exec exec-code :logic logic-code)</v>, the user would be unlikely \u000A to notice any difference in the theorem prover if this term were replaced by \u000A <v>logic-code</v>.  ACL2 takes various steps to ensure this.  For example, the \u000A proof obligations generated for admitting a function treat the above <v>mbe</v> \u000A term simply as <v>logic-code</v>.  Function expansion, <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>, \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules, generation of <see topic=\"ACL2____CONSTRAINT\">constraint</see>s for \u000A functional instantiation, and creation of rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> \u000A and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> also treat <v>mbe</v> calls as their \u000A <v>:logic</v> code.</p>"],

 "ACL2____FORWARD-CHAINING":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Examples:\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> p-and-r-forward           ; When (p a) appears in a formula to be\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p x) (r x))       ; simplified, try to establish (p a) and\u000A          (<see topic=\"ACL2____Q\">q</see> (f x)))              ; (r a) and, if successful, add (<see topic=\"ACL2____Q\">q</see> (f a))\u000A :rule-classes :forward-chaining) ; to the known assumptions.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> p-and-r-forward           ; as above with most defaults filled in\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p x) (r x))\u000A           (<see topic=\"ACL2____Q\">q</see> (f x)))\u000A  :rule-classes ((:forward-chaining :trigger-terms ((p x))\u000A                                    :corollary (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p x) (r x))\u000A                                                        (<see topic=\"ACL2____Q\">q</see> (f x)))\u000A                                    :match-free :all)))</code> \u000A \u000A <p>To specify the triggering terms provide a non-empty list of terms as the \u000A value of the <v>:trigger-terms</v> field of the rule class object.</p> \u000A \u000A <code>General Form:\u000AAny theorem, provided an acceptable triggering term exists.</code> \u000A \u000A <p>The structure of this documentation is as follows.  First we give a brief \u000A overview of forward chaining and contrast it to backchaining (rewriting). \u000A Then we lay out the syntactic restrictions on <v>:forward-chaining</v> rules. \u000A Then we give more details about the process and point to a tool to assist you \u000A in debugging your <v>:forward-chaining</v> rules.</p> \u000A \u000A <p><i>Overview and When to Use Forward Chaining</i></p> \u000A \u000A <p>Forward chaining is performed as part of the simplification process: before \u000A the goal is rewritten a <i>context</i> is established.  The context tells the \u000A theorem prover what may be assumed during rewriting, in particular, to \u000A establish hypotheses of rewrite rules.  Forward chaining is used to extend the \u000A context before rewriting begins.  For example, the <v>:forward-chaining</v> rule \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (p1 x))</v> would add <v>(p1 A)</v> to the context, where <v>A</v> \u000A is some term, if <v>(p A)</v> is already in the context.</p> \u000A \u000A <p>Forward chaining and backchaining are duals.  If a rewrite rule requires \u000A that <v>(p1 A)</v> be established and <v>(p A)</v> is known, it could be done \u000A either by making <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (p1 x))</v> a <v>:forward-chaining</v> rule or \u000A a <v>:rewrite</v> rule.  Which should you choose?</p> \u000A \u000A <p>As a rule of thumb, if a conclusion like <v>(p1 A)</v> is expected to be \u000A widely needed, it is better to derive it via forward chaining because then it \u000A is available ``for free'' during the rewriting after paying the one-time cost \u000A of forward chaining.  Alternatively, if <v>(p1 A)</v> is a rather special \u000A hypothesis of key importance to only a few rewrite rules, it is best to derive \u000A it only when needed.  Thus forward chaining is pro-active and backward \u000A chaining (rewriting) is reactive.</p> \u000A \u000A <p><i>Syntactic Restrictions</i></p> \u000A \u000A <p>Forward chaining rules are generated from the corollary term (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) as follows.  First, every <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expression is expanded \u000A away (hence, so is every <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> and <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> expression), as is \u000A every call of a so-called ``guard holder,'' <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt> or <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> (the latter resulting from macroexpansion of calls of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt> or <tt><see topic=\"ACL2____MBE\">mbe</see></tt>), <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, and a few \u000A others), or `<tt><see topic=\"COMMON-LISP____THE\">the</see></tt>'.  If the resulting term has the form <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyp\u000A concl)</v>, then <v>concl</v> is treated as a conjunction, with one forward \u000A chaining rule with hypothesis <v>hyp</v> created for each conjunct.  In the \u000A other case, where the corollary term is not an <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt>, we process it \u000A as we process the conclusion in the first case.</p> \u000A \u000A <p>Note that unlike rewrite rules, a nested implication is not folded into a \u000A single implication.  Consider for example the following term.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (p1 x)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x)\u000A                  (p3 x)))</code> \u000A \u000A <p>Although this term is parsed for a rewrite rule as <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x)\u000A (p2 x)) (p3 x))</v>, that is not the case when this term is parsed for a \u000A forward-chaining rule, in which case <v>(p1 x)</v> is treated as the hypothesis \u000A and <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x) (p3 x))</v> is treated as the conclusion.</p> \u000A \u000A <p>The <v>:trigger-terms</v> field of a <v>:forward-chaining</v> rule class object \u000A should be a non-empty list of terms, if provided, and should have certain \u000A properties described below.  If the <v>:trigger-terms</v> field is not provided, \u000A it defaults to the singleton list containing the ``atom'' of the first \u000A hypothesis of the formula.  (The atom of <v>(<see topic=\"COMMON-LISP____NOT\">not</see> x)</v> is <v>x</v>; the atom of \u000A any other term is the term itself.)  If there are no hypotheses and no \u000A <v>:trigger-terms</v> were provided, an error is caused.</p> \u000A \u000A <p>A triggering term is acceptable if it is not a variable, a quoted constant, \u000A a lambda application, a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>- (or <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>-) expression, or a \u000A <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt>-expression, and every variable symbol in the conclusion of the \u000A theorem either occurs in the hypotheses or occurs in the trigger.</p> \u000A \u000A <p><i>More Details about Forward Chaining</i></p> \u000A \u000A <p><v>:Forward-chaining</v> rules are used by the simplifier <i>before</i> it \u000A begins to rewrite the literals of the goal.  (Forward chaining is thus carried \u000A out from scratch for each goal.)  If any term in the goal is an instance of a \u000A trigger of some forward chaining rule, we try to establish the hypotheses of \u000A that forward chaining theorem (from the negation of the goal).  To relieve a \u000A hypothesis we only use type reasoning, evaluation of ground terms, and \u000A presence among our known assumptions.  We do not use rewriting.  So-called \u000A free variables in hypotheses are treated specially; see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>. \u000A If all hypotheses are relieved, and certain heuristics approve of the newly \u000A derived conclusion, we add the instantiated conclusion to our known \u000A assumptions.  Since this might introduce new terms into the assumptions, \u000A forward chaining is repeated.  Heuristic approval of each new addition is \u000A necessary to avoid infinite looping as would happen with the rule <v>(<see topic=\"ACL2____IMPLIES\">implies</see>\u000A (p x) (p (f x)))</v>, which might otherwise forward chain from <v>(p A)</v> to \u000A <v>(p (f A))</v> to <v>(p (f (f A)))</v>, etc.</p> \u000A \u000A <p><i>Caution</i>.  Forward chaining does not actually add terms to the goals \u000A displayed during proof attempts.  Instead, it extends an associated \u000A <i>context</i>, called ``assumptions'' in the preceding paragraph, that ACL2 \u000A builds from the goal currently being proved.  (For insiders: forward chaining \u000A extends the <tt><see topic=\"ACL2____TYPE-ALIST\">type-alist</see></tt>.)  The context starts out with ``obvious'' \u000A consequences of the negation of the goal.  For example, if the goal is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p A) (<see topic=\"ACL2____Q\">q</see> (f A)))\u000A         (c A))</code> \u000A \u000A <p>then the context notes that <v>(p A)</v> and <v>(<see topic=\"ACL2____Q\">q</see> (f A))</v> are non-<v>nil</v> \u000A and <v>(c A)</v> is <v>nil</v>.  Forward chaining is then used to expand the \u000A context.  For example, if a forward chaining rule has <v>(f x)</v> as a trigger \u000A term and has body <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (r (f x)))</v>, then the context is extended \u000A by binding <v>(r (f A))</v> to non-<v>nil</v>, provided the heuristics approve of \u000A this extension.  Note however that since <v>(r (f A))</v> is put into the \u000A context, not the goal, you will not see it in the goal formula.  Furthermore, \u000A the assumption added to the context is just the instantiation of the \u000A conclusion of the rule, with no simplification or rewriting applied.  Thus, \u000A for example, if it contains an enabled non-recursive function symbol it is \u000A unlikely ever to match a (rewritten) term arising during subsequent \u000A simplification of the goal.</p> \u000A \u000A <p>However, forward-chaining does support the linear arithmetic reasoning \u000A package.  For example, suppose that forward-chaining puts <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (f x) (g x))</v> \u000A into the context.  Then this inequality also goes into the linear arithmetic \u000A database, together with suitable instances of linear lemmas whose trigger term \u000A is a call of <v>g</v>.  See <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p>Debugging <v>:forward-chaining</v> rules can be difficult since their effects \u000A are not directly visible on the goal being simplified.  Tools are available to \u000A help you discover what forward chaining has occurred see <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see>.</p>"],

 "ACL2____FREE-VARIABLES":[["Rule-classes","Rewrite"],"ACL2 Sources","ACL2","<p>As described elsewhere (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>), ACL2 rules are \u000A treated as implications for which there are zero or more hypotheses <v>hj</v> to \u000A prove.  In particular, rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> may look like \u000A this:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> h1 ... hn)\u000A         (fn lhs rhs))</code> \u000A \u000A <p>Variables of <v>hi</v> are said to occur <i>free</i> in the above \u000A <v>:rewrite</v> rule if they do not occur in <v>lhs</v> or in any <v>hj</v> with \u000A <v>j&lt;i</v>.  (To be precise, here we are only discussing those variables that \u000A are not in the scope of a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>/<tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>/<v>lambda</v> that binds \u000A them.)  We also refer to these as the <i>free variables</i> of the rule.  ACL2 \u000A may issue a warning or error when there are free variables in a rule, as \u000A described below.  (Variables of <v>rhs</v> may be considered free if they do not \u000A occur in <v>lhs</v> or in any <v>hj</v>.  But we do not consider those in this \u000A discussion.)</p> \u000A \u000A <p>In general, the <i>free variables</i> of rules are those variables \u000A occurring in their hypotheses (not <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>/<tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>/<v>lambda</v>-bound) \u000A that are not bound when the rule is applied.  For rules of class <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>, variables are bound by a trigger \u000A term.  (See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a discussion of the <v>:trigger-terms</v> \u000A field).  For rules of class <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, variables are \u000A bound by the <v>:typed-term</v> field.</p> \u000A \u000A <p>Let us discuss the method for relieving hypotheses of <see topic=\"ACL2____REWRITE\">rewrite</see> rules \u000A with free variables.  Similar considerations apply to <see topic=\"ACL2____LINEAR\">linear</see> and <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules, and <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.</p> \u000A \u000A <p>See <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES\">free-variables-examples</see> for more examples of how this all works, \u000A including illustration of how the user can exercise some control over it.  In \u000A particular, see <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES-REWRITE\">free-variables-examples-rewrite</see> for an explanation of \u000A output from the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> facility in the presence of rewriting \u000A failures involving free variables, as well as an example exploring ``binding \u000A hypotheses'' as described below.</p> \u000A \u000A <p>Note that the <v>:match-free</v> mechanism discussed below does not apply to \u000A <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.  See <see topic=\"ACL2____FREE-VARIABLES-TYPE-PRESCRIPTION\">free-variables-type-prescription</see> \u000A for a discussion of how to control free-variable matchng for <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.</p> \u000A \u000A <p>We begin with an example.  Does the proof of the <tt><see topic=\"ACL2____THM\">thm</see></tt> below \u000A succeed?</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> p2 (x y) t)\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (p2 x z) t))\u000A  :rule-classes ((:rewrite :match-free :all)))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))</code> \u000A \u000A <p>Consider what happens when the proof of the <v>thm</v> is attempted.  The \u000A ACL2 rewriter attempts to apply rule <v>p2-trans</v> to the conclusion, <v>(p2 a\u000A d)</v>.  So, it binds variables <v>x</v> and <v>z</v> from the left-hand side of the \u000A conclusion of <v>p2-trans</v> to terms <v>a</v> and <v>d</v>, respectively, and then \u000A attempts to relieve the hypotheses of <v>p2-trans</v>.  The first hypothesis of \u000A <v>p2-trans</v>, <v>(p2 x y)</v>, is considered first.  Variable <v>y</v> is free in \u000A that hypothesis, i.e., it has not yet been bound.  Since <v>x</v> is bound to \u000A <v>a</v>, the rewriter looks through the context for a binding of <v>y</v> such \u000A that <v>(p2 a y)</v> is true, and it so happens that it first finds the term \u000A <v>(p2 a b)</v>, thus binding <v>y</v> to <v>b</v>.  Now it goes on to the next \u000A hypothesis, <v>(p2 y z)</v>.  At this point <v>y</v> and <v>z</v> have already been \u000A bound to <v>b</v> and <v>d</v>; but <v>(p2 b d)</v> cannot be proved.</p> \u000A \u000A <p>So, in order for the proof of the <tt><see topic=\"ACL2____THM\">thm</see></tt> to succeed, the rewriter \u000A needs to backtrack and look for another way to instantiate the first \u000A hypothesis of <v>p2-trans</v>.  Because <v>:match-free :all</v> has been \u000A specified, backtracking does take place.  This time <v>y</v> is bound to <v>c</v>, \u000A and the subsequent instantiated hypothesis becomes <v>(p2 c d)</v>, which is \u000A true.  The application of rule <v>(p2-trans)</v> succeeds and the theorem is \u000A proved.</p> \u000A \u000A <p>If instead <v>:match-free :all</v> had been replaced by <v>:match-free\u000A :once</v> in rule <v>p2-trans</v>, then backtracking would not occur, and the \u000A proof of the <tt><see topic=\"ACL2____THM\">thm</see></tt> would fail.</p> \u000A \u000A <p>Next we describe in detail the steps used by the rewriter in dealing with \u000A free variables.</p> \u000A \u000A <p>ACL2 uses the following sequence of steps to relieve a hypothesis with free \u000A variables, except that steps (1) and (3) are skipped for \u000A <v>:forward-chaining</v> rules and step (3) is skipped for \u000A <v>:type-prescription</v> rules.  First, if the hypothesis is of the form \u000A <v>(<see topic=\"ACL2____FORCE\">force</see> hyp0)</v> or <v>(<see topic=\"ACL2____CASE-SPLIT\">case-split</see> hyp0)</v>, then replace it with \u000A <v>hyp0</v>.</p> \u000A \u000A <blockquote><p>(1) Suppose the hypothesis has the form <v>(equiv var term)</v> \u000A where <v>var</v> is free and no variable of <v>term</v> is free, and either \u000A <v>equiv</v> is <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> or else <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A relation and <v>term</v> is a call of <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt>.  We call this a \u000A ``binding hypothesis.''  Then bind <v>var</v> to the result of rewriting \u000A <v>term</v> in the current context.</p> \u000A \u000A <p>(2) Look for a binding of the free variables of the hypothesis so that the \u000A corresponding instance of the hypothesis is known to be true in the current \u000A context.</p> \u000A \u000A <p>(3) Search all <tt><see topic=\"ACL2____ENABLE\">enable</see></tt>d, hypothesis-free rewrite rules of the form \u000A <v>(equiv lhs rhs)</v>, where <v>lhs</v> has no variables (other than those bound \u000A by <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>, <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, or <v>lambda</v>), <v>rhs</v> is known to be true in \u000A the current context, and <v>equiv</v> is typically <v>equal</v> but can be any \u000A equivalence relation appropriate for the current context (see <see topic=\"ACL2____CONGRUENCE\">congruence</see>); then attempt to bind the free variables so that the instantiated \u000A hypothesis is <v>lhs</v>.</p></blockquote> \u000A \u000A <p>If all attempts fail and the original hypothesis is a call of <tt><see topic=\"ACL2____FORCE\">force</see></tt> \u000A or <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt>, where forcing is enabled (see <see topic=\"ACL2____FORCE\">force</see>) then the \u000A hypothesis is relieved, but in the split-off goals, all free variables are \u000A bound to unusual names that call attention to this odd situation.</p> \u000A \u000A <p>When a <see topic=\"ACL2____REWRITE\">rewrite</see> or <see topic=\"ACL2____LINEAR\">linear</see> rule has free variables in the \u000A hypotheses, the user generally needs to specify whether to consider only the \u000A first instance found in steps (2) and (3) above, or instead to consider them \u000A all.  Below we discuss how to specify these two options as ``<v>:once</v>'' or \u000A ``<v>:all</v>'' (the default), respectively.</p> \u000A \u000A <p>Is it better to specify <v>:once</v> or <v>:all</v>?  We believe that <v>:all</v> \u000A is generally the better choice because of its greater power, provided the user \u000A does not introduce a large number of rules with free variables, which has been \u000A known to slow down the prover due to combinatorial explosion in the search \u000A (Steps (2) and (3) above).</p> \u000A \u000A <p>Either way, it is good practice to put the ``more substantial'' hypotheses \u000A first, so that the most likely bindings of free variables will be found first \u000A (in the case of <v>:all</v>) or found at all (in the case of <v>:once</v>).  For \u000A example, a rewrite rule like</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x y)\u000A              (p2 x y))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar x) (bar-prime x)))</code> \u000A \u000A <p>may never succeed if <v>p1</v> is nonrecursive and enabled, since we may well \u000A not find calls of <v>p1</v> in the current context.  If however <v>p2</v> is \u000A disabled or recursive, then the above rule may apply if the two hypotheses are \u000A switched.  For in that case, we can hope for a match of <v>(p2 x y)</v> in the \u000A current context that therefore binds <v>x</v> and <v>y</v>; then the rewriter's \u000A full power may be brought to bear to prove <v>(p1 x y)</v> for that <v>x</v> and \u000A <v>y</v>.</p> \u000A \u000A <p>Moreover, the ordering of hypotheses can affect the efficiency of the \u000A rewriter.  For example, the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)\u000A              (foo x y))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar x) (bar-prime x)))</code> \u000A \u000A <p>may well be sub-optimal.  Presumably the intention is to rewrite <v>(bar\u000A x)</v> to <v>(bar-prime x)</v> in a context where <v>(foo x y)</v> is explicitly \u000A known to be true for some rational number <v>y</v>.  But <v>y</v> will be bound \u000A first to the first term found in the current context that is known to \u000A represent a rational number.  If the 100th such <v>y</v> that is found is the \u000A first one for which <v>(foo x y)</v> is known to be true, then wasted work will \u000A have been done on behalf of the first 99 such terms <v>y</v> &mdash; unless \u000A <v>:once</v> has been specified, in which case the rule will simply fail after \u000A the first binding of <v>y</v> for which <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)</v> is known to be true. \u000A Thus, a better form of the above rule is almost certainly the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (foo x y)\u000A              (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar x) (bar-prime x)))</code> \u000A \u000A <p><i>Specifying `once' or `all'.</i></p> \u000A \u000A <p>As noted above, the following discussion applies only to <see topic=\"ACL2____REWRITE\">rewrite</see>, \u000A <see topic=\"ACL2____LINEAR\">linear</see>, and <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules.  See <see topic=\"ACL2____FREE-VARIABLES-TYPE-PRESCRIPTION\">free-variables-type-prescription</see> for a discussion of analogous considerations \u000A for <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.</p> \u000A \u000A <p>One method for specifying <v>:once</v> or <v>:all</v> for free-variable \u000A matching is to provide the <v>:match-free</v> field of the <v>:rule-classes</v> of \u000A the rule, for example, <v>(:rewrite :match-free :all)</v>.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.  However, there are global events that can be used to specify \u000A <v>:once</v> or <v>:all</v>; see <see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> and see <see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see>.  Here are some examples.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :once)    ; future rules without a :match-free field\u000A                                  ; are stored as :match-free :once (but this\u000A                                  ; behavior is local to a book)\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t) ; existing rules are treated as\u000A                                  ; :match-free :once regardless of their\u000A                                  ; original :match-free fields\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once (:rewrite foo) (:rewrite bar . 2))\u000A                                  ; the two indicated rules are treated as\u000A                                  ; :match-free :once regardless of their\u000A                                  ; original :match-free fields</code> \u000A \u000A <p><i>Some history.</i> Before Version 2.7 the ACL2 rewriter performed Step \u000A (2) above first.  More significantly, it always acted as though <v>:once</v> had \u000A been specified.  That is, if Step (2) did not apply, then the rewriter took \u000A the first binding it found using either Steps (1) or (3), in that order, and \u000A proceeded to relieve the remaining hypotheses without trying any other \u000A bindings of the free variables of that hypothesis.</p>"],

 "ACL2____OVERRIDE-HINTS":[["Hints"],"ACL2 Sources","ACL2","<p>This is an advanced feature, originally implemented to help system \u000A designers to create ``modes'' that control the way hints are supplied to the \u000A theorem prover.  Please see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see> for the much more usual way \u000A to install hints that may be applied by default.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> (w state))\u000A((computed-hint-1 clause keyword-alist processor)\u000A (computed-hint-2 clause keyword-alist stable-under-simplificationp))</code> \u000A \u000A <p><v>Override-hints</v> returns a list of computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) which, unlike other computed hints, may mention the variable \u000A <v>KEYWORD-ALIST</v>.</p> \u000A \u000A <p>Before reading further, please see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see> to review \u000A the basics of how <see topic=\"ACL2____HINTS\">hints</see> are applied during a proof.  In particular, we \u000A assume familiarity with the notion of selecting a hint to be applied to the \u000A current goal.  If there are override-hints, that hint selection is tentative, \u000A because if it reduced to <v>nil</v> after the application of override-hints, \u000A then that hint will be skipped and the attempt will continue for selecting an \u000A applicable hint.  (Craft your override-hints so that <v>:no-op t</v> is returned \u000A in such cases instead of <v>nil</v>, if you don't want the hint to be skipped.) \u000A But we must explain what is meant by ``the application of override-hints'', \u000A and we do that now.</p> \u000A \u000A <p>Suppose that there are override-hints when a hint is selected for the \u000A current goal.  That selected hint is a keyword-alist, which is an alternating \u000A list of hint keywords and their values, whose source is either an explicit \u000A hint <v>(goal-name :key1 val1 ... :keyn valn)</v> where the <v>:keyi</v> are \u000A allowed to be custom hint keywords (which are expanded away; see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>), or else is the non-<v>nil</v> keyword-alist produced by \u000A evaluating a computed hint.  Then the override-hints are applied to that \u000A keyword-alist as follows, one at a time, in order of their occurrence in the \u000A list of override-hints (as determined by the use of <tt><see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see></tt> \u000A and <tt><see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see></tt>).  The first override-hint is evaluated, in the \u000A usual manner of evaluating computed hints but with the variable \u000A <v>KEYWORD-ALIST</v> bound to the above keyword-alist.  That evaluation produces \u000A a result that should also be a keyword-alist, or else an error occurs.  Any \u000A custom keyword hints are then eliminated from that keyword-alist.  The \u000A resulting keyword-alist must not contain the <v>:ERROR</v> hint keyword and must \u000A not start with the <v>:COMPUTED-HINT-REPLACEMENT</v> keyword; otherwise an error \u000A occurs.  With <v>KEYWORD-ALIST</v> bound to this result, the second \u000A override-hint is similarly evaluated.  This process continues, and the \u000A keyword-alist returned by the final override-hint is the one used when \u000A processing the goal at hand.  Except: If that keyword-alist is <v>nil</v>, then \u000A the next hint among the pending hints is tentatively selected and the process \u000A repeats, applying each override hint to that new tentative selection.  Of \u000A course we might obtain <v>nil</v> again, in which case we tentatively select the \u000A next pending hint; and so on.</p> \u000A \u000A <p>If finally no hint is selected for the current goal, then \u000A <v>KEYWORD-ALIST</v> is bound to <v>nil</v> and the override-hints are applied as \u000A described above.  But note that this final step is skipped if hint selection \u000A is being performed because <v>stable-under-simplificationp</v> has just become \u000A true, rather than at the top of the waterfall.  (Otherwise the override-hints \u000A could easily keep firing uselessly yet putting us back at the top of the \u000A waterfall, with no change to the given goal, resulting in an infinite \u000A loop.)</p> \u000A \u000A <p>As mentioned above, the <v>:COMPUTED-HINT-REPLACEMENT</v> keyword is illegal \u000A for the value of an override-hint.  But a selected hint may be a computed hint \u000A that evaluates to a keyword-alist beginning with prefix \u000A <v>:COMPUTED-HINT-REPLACEMENT val</v>.  What value does ACL2 return for such a \u000A computed hint in the presence of override-hints?  First, this prefix is \u000A stripped off before passing the resulting keyword-alist to the override-hints \u000A as described above.  If the result of applying override-hints to that \u000A keyword-alist is not <v>nil</v>, then the prefix is put back on the front of \u000A that resulting keyword-alist after doing internal processing of the hint, \u000A including expansion of any custom keyword hints.  Otherwise, the application \u000A of override-hints to the computed hint is <v>nil</v>, so this hint is not \u000A selected after all.</p> \u000A \u000A <p><b>WARNING</b>: Unlike ordinary computed hints, a value of <v>nil</v> for an \u000A override-hint is not ignored.  That is: When an ordinary computed hint \u000A evaluates to <v>nil</v>, it is deemed not to apply, and the next available hint \u000A is consulted.  But when an override-hint is evaluated, the result is always \u000A supplied for the next binding of the variable <v>KEYWORD-ALIST</v>, even if that \u000A result is <v>nil</v>.  If you want an override-hint to be a no-op, return as the \u000A expression the variable <v>KEYWORD-ALIST</v> rather than an expression that \u000A evaluates to <v>nil</v>.</p> \u000A \u000A <p>This feature can be used in order to implement a form of additive hints. \u000A Suppose for example that you want a hint that turns off generalization.  A \u000A simple but inadequate solution is:</p> \u000A \u000A <code>(<see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> '((<see topic=\"COMMON-LISP____QUOTE\">quote</see> (:do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see>)))))</code> \u000A \u000A <p>The problem is that if there is any explicit hint supplied for a given \u000A goal, then it will be the one selected, and the above will be ignored.  But \u000A suppose that the explicit hint supplied is of the form <v>(&quot;Subgoal x.y&quot;\u000A :do-not '(fertilize))</v>.  What we would really want in this case is to \u000A generate the hint for the indicated subgoal that binds <v>:do-not</v> to a list \u000A indicating that both fertilization _and_ generalization are disabled for that \u000A goal.  A solution is to merge, for example as follows.  (The use of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> and <tt><see topic=\"ACL2____CW\">cw</see></tt> is of course optional, included here to provide debug \u000A printing.)</p> \u000A \u000A <code>(<see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see>\u000A '((<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((tmp (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :do-not KEYWORD-ALIST))\u000A          (new-keyword-alist\u000A           (<see topic=\"COMMON-LISP____COND\">cond</see> (tmp (<see topic=\"COMMON-LISP____LIST_A2\">list*</see> :do-not\u000A                             `(<see topic=\"COMMON-LISP____CONS\">cons</see> 'generalize ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> tmp))\u000A                             (remove-keyword :do-not KEYWORD-ALIST)))\u000A                 (t (<see topic=\"COMMON-LISP____LIST_A2\">list*</see> :do-not ''(<see topic=\"ACL2____GENERALIZE\">generalize</see>) KEYWORD-ALIST)))))\u000A     (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;New: ~x0~|&quot; new-keyword-alist)\u000A             new-keyword-alist))))</code> \u000A \u000A <p>REMARKS</p> \u000A \u000A <p>(1) The utilities <tt><see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see></tt>, <tt><see topic=\"ACL2____ADD-OVERRIDE-HINTS_12\">add-override-hints!</see></tt>, \u000A <tt><see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see></tt>, <tt><see topic=\"ACL2____SET-OVERRIDE-HINTS_12\">set-override-hints!</see></tt>, <tt><see topic=\"ACL2____REMOVE-OVERRIDE-HINTS\">remove-override-hints</see></tt>, and <tt><see topic=\"ACL2____REMOVE-OVERRIDE-HINTS_12\">remove-override-hints!</see></tt> are also available, \u000A in complete analogy to their default-hints versions.</p> \u000A \u000A <p>(2) The community book <v>hints/basic-tests.lisp</v> illustrates the use of \u000A override-hints and illuminates a number of corner cases; search in that file \u000A for ``Test override-hints.''</p> \u000A \u000A <p>(3) The community book <v>hints/merge-hint.lisp</v> provides support for \u000A merging hints that might be useful for writers of override-hint expressions \u000A (see the examples at the end of that file).</p> \u000A \u000A <p>(4) Override-hints are used in the processing of <v>:BACKTRACK</v> hints (see \u000A <see topic=\"ACL2____HINTS\">hints</see>).</p>"],

 "ACL2____LD-SKIP-PROOFSP":[["Ld"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> t state)\u000A T\u000AACL2 !s&gt;(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> nil state)\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> 'include-book state)\u000A INCLUDE-BOOK\u000AACL2 !s&gt;</code> \u000A \u000A <p>A global variable in the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, called <v>'ld-skip-proofsp</v>, \u000A determines the thoroughness with which ACL2 processes your <see topic=\"ACL2____COMMAND\">command</see>s. \u000A This variable may take on one of three values: <v>t</v>, <v>nil</v> or \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  When <v>ld-skip-proofsp</v> is non-<v>nil</v>, the \u000A system assumes that which ought to be proved and is thus unsound.  The form \u000A <v>(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> flg state)</v> is the general-purpose way of setting \u000A <v>ld-skip-proofsp</v>.  This global variable is an ``<tt><see topic=\"ACL2____LD\">ld</see></tt> special,'' \u000A which is to say, you may call <tt><see topic=\"ACL2____LD\">ld</see></tt> in such a way as to ``bind'' this \u000A variable for the dynamic extent of the <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>When <v>ld-skip-proofsp</v> is non-<v>nil</v>, the default <see topic=\"ACL2____PROMPT\">prompt</see> \u000A displays the character <v>s</v>.  Thus, the <see topic=\"ACL2____PROMPT\">prompt</see></p> \u000A \u000A <code>ACL2 !s&gt;</code> \u000A \u000A <p>means that the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> (otherwise \u000A the character <v>p</v>, for <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, would also be printed; see \u000A <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>) but ``proofs are being skipped.''</p> \u000A \u000A <p>Observe that there are two legal non-<v>nil</v> values, <v>t</v> and \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  When <v>ld-skip-proofsp</v> is <v>t</v>, ACL2 skips \u000A all proof obligations but otherwise performs all other required analysis of \u000A input <see topic=\"ACL2____EVENTS\">events</see>.  When <v>ld-skip-proofsp</v> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, \u000A ACL2 skips not only proof obligations but all analysis except that required to \u000A compute the effect of successfully executed <see topic=\"ACL2____EVENTS\">events</see>.  To explain the \u000A distinction, let us consider one particular event, say a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  Very \u000A roughly speaking, a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event normally involves a check of the \u000A syntactic well-formedness of the submitted definition, the generation and \u000A proof of the termination conditions, and the computation and storage of \u000A various rules such as a <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule and some <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.  By ``normally'' above we mean when \u000A <v>ld-skip-proofsp</v> is <v>nil</v>.  How does a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> behave when \u000A <v>ld-skip-proofsp</v> is non-<v>nil</v>?</p> \u000A \u000A <p>If <v>ld-skip-proofsp</v> is <v>t</v>, then <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> performs the \u000A syntactic well-formedness checks and computes and stores the various rules, \u000A but it does not actually carry out the termination proofs.  If \u000A <v>ld-skip-proofsp</v> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> does not do \u000A the syntactic well-formedness check nor does it carry out the termination \u000A proof.  Instead, it merely computes and stores the rules under the assumption \u000A that the checks and proofs would all succeed.  Observe that a setting of \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is ``stronger'' than a setting of <v>t</v> in the \u000A sense that <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> causes <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> to assume even more \u000A about the admissibility of the event than <v>t</v> does.</p> \u000A \u000A <p>As one might infer from the choice of name, the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event \u000A sets <v>ld-skip-proofsp</v> to <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> when processing the \u000A <see topic=\"ACL2____EVENTS\">events</see> in a book being loaded.  Thus, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> does the \u000A miminal work necessary to carry out the effects of every event in the book. \u000A The syntactic checks and proof obligations were, presumably, successfully \u000A carried out when the book was certified.</p> \u000A \u000A <p>A non-<v>nil</v> value for <v>ld-skip-proofsp</v> also affects the system's \u000A output messages.  Event summaries (the paragraphs that begin ``Summary'' and \u000A display the event forms, rules used, etc.) are not printed when \u000A <v>ld-skip-proofsp</v> is non-<v>nil</v>.  Warnings and observations are printed \u000A when <v>ld-skip-proofsp</v> is <v>t</v> but are not printed when it is \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>Intuitively, <v>ld-skip-proofsp</v> <v>t</v> means skip just the proofs and \u000A otherwise do all the work normally required for an event; while \u000A <v>ld-skip-proofsp</v> <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is ``stronger'' and means do \u000A as little as possible to process <see topic=\"ACL2____EVENTS\">events</see>.  In accordance with this \u000A intuition, <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are processed when <v>ld-skip-proofsp</v> \u000A is <v>t</v> but are skipped when <v>ld-skip-proofsp</v> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>The ACL2 system itself uses only two settings, <v>nil</v> and <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the latter being used only when executing the <see topic=\"ACL2____EVENTS\">events</see> \u000A inside of a book being included.  The <v>ld-skip-proofsp</v> setting of <v>t</v> \u000A is provided as a convenience to the user.  For example, suppose one has a file \u000A of <see topic=\"ACL2____EVENTS\">events</see>.  By loading it with <tt><see topic=\"ACL2____LD\">ld</see></tt> with <v>ld-skip-proofsp</v> set \u000A to <v>t</v>, the <see topic=\"ACL2____EVENTS\">events</see> can all be checked for syntactic correctness and \u000A assumed without proof.  This is a convenient way to recover a state lost by a \u000A system crash or to experiment with a modification of an <see topic=\"ACL2____EVENTS\">events</see> \u000A file.</p> \u000A \u000A <p>The foregoing discussion is actually based on a lie.  <v>ld-skip-proofsp</v> \u000A is allowed two other values, <v>'initialize-acl2</v> and \u000A <v>'include-book-with-locals</v>.  The first causes behavior similar to <v>t</v> \u000A but skips <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> and avoids some error checks that would \u000A otherwise prevent ACL2 from properly booting.  The second is identical to \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> but also executes <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see>. \u000A These additional values are not intended for use by the user, but no barriers \u000A to their use have been erected.</p> \u000A \u000A <p>We close by reminding the user that ACL2 is potentially unsound if \u000A <v>ld-skip-proofsp</v> is ever set by the user.  We provide access to it simply \u000A to allow experimentation and rapid reconstruction of lost or modified logical \u000A <see topic=\"ACL2____WORLD\">world</see>s.</p>"],

 "ACL2____EVENTS":[["ACL2"],"ACL2 Sources","ACL2","<p>Any extension of the syntax of ACL2 (i.e., the definition of a new \u000A constant or macro), the axioms (i.e., the definition of a function), or the \u000A rule database (i.e., the proof of a theorem), constitutes a logical ``event.'' \u000A Events change the ACL2 logical world (see <see topic=\"ACL2____WORLD\">world</see>).  Indeed, the only way \u000A to change the ACL2 <see topic=\"ACL2____WORLD\">world</see> is via the successful evaluation of an event \u000A function.  Every time the <see topic=\"ACL2____WORLD\">world</see> is changed by an event, a landmark is \u000A left on the <see topic=\"ACL2____WORLD\">world</see> and it is thus possible to identify the <see topic=\"ACL2____WORLD\">world</see> \u000A ``as of'' the evaluation of a given event.  An event may introduce new logical \u000A names.  Some events introduce no new names (e.g., <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>), some \u000A introduce exactly one (e.g., <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>), and some \u000A may introduce many (e.g., <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> ).</p> \u000A \u000A <p>ACL2 typically completes processing of an event by printing a summary. \u000A Unless proofs are skipped (see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>) or summary output is \u000A inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), information about the proof \u000A attempt (if any) is printed that includes a list of rules used, a summary of \u000A warnings, and the number of ``prover steps'' (if any; see <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>).  A breakdown of the time used is also printed, which \u000A by default is runtime (cpu time), but can be changed to realtime (wall clock \u000A time); see <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see> for a discussion of events permitted in \u000A <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____EQUIVALENCE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> r-equal-is-an-equivalence ; assumes that r-equal has been defined\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (r-equal x y))\u000A       (r-equal x x)\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (r-equal x y) (r-equal y x))\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (r-equal x y)\u000A                     (r-equal y z))\u000A                (r-equal x z)))\u000A  :rule-classes :equivalence)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFEQUIV\">defequiv</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (equiv x y))\u000A     (equiv x x)\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv x y) (equiv y x))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (equiv x y)\u000A                   (equiv y z))\u000A              (equiv x z)))</code> \u000A \u000A <p>except that the order of the conjuncts and terms and the choice of variable \u000A symbols is unimportant.  The effect of such a rule is to identify <v>equiv</v> \u000A as an equivalence relation.  Note that only Boolean 2-place function symbols \u000A can be treated as equivalence relations.  See <see topic=\"ACL2____CONGRUENCE\">congruence</see> and see <see topic=\"ACL2____REFINEMENT\">refinement</see> for closely related concepts.</p> \u000A \u000A <p>The macro form <v>(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> equiv)</v> is an abbreviation for a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> of rule-class <v>:equivalence</v> that establishes that <v>equiv</v> is an \u000A equivalence relation.  It generates the formula shown above.  See <see topic=\"ACL2____DEFEQUIV\">defequiv</see>.</p> \u000A \u000A <p>When <v>equiv</v> is marked as an equivalence relation, its reflexivity, \u000A symmetry, and transitivity are built into the system in a deeper way than via \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules.  More importantly, after <v>equiv</v> has been \u000A shown to be an equivalence relation, lemmas about <v>equiv</v>, e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps (equiv lhs rhs)),</code> \u000A \u000A <p>when stored as <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules, cause the system to rewrite \u000A certain occurrences of (instances of) <v>lhs</v> to (instances of) <v>rhs</v>. \u000A Roughly speaking, an occurrence of <v>lhs</v> in the <v>kth</v> argument of some \u000A <v>fn</v>-expression, <v>(fn ... lhs' ...)</v>, can be rewritten to produce <v>(fn\u000A ...  rhs' ...)</v>, provided the system ``knows'' that the value of <v>fn</v> is \u000A unaffected by <v>equiv</v>-substitution in the <v>kth</v> argument.  Such \u000A knowledge is communicated to the system via ``congruence lemmas.''</p> \u000A \u000A <p>For example, suppose that <v>r-equal</v> is known to be an equivalence \u000A relation.  The <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> lemma</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (r-equal s1 s2)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn s1 n) (fn s2 n)))</code> \u000A \u000A <p>informs the rewriter that, while rewriting the first argument of \u000A <v>fn</v>-expressions, it is permitted to use <v>r-equal</v> rewrite-rules.  See \u000A <see topic=\"ACL2____CONGRUENCE\">congruence</see> for details about <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> lemmas. \u000A Interestingly, congruence lemmas are automatically created when an equivalence \u000A relation is stored, saying that either of the equivalence relation's arguments \u000A may be replaced by an equivalent argument.  That is, if the equivalence \u000A relation is <v>fn</v>, we store congruence rules that state the following \u000A fact:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (fn x1 y1)\u000A              (fn x2 y2))\u000A         (<see topic=\"ACL2____IFF\">iff</see> (fn x1 x2) (fn y1 y2)))</code> \u000A \u000A <p>Another aspect of equivalence relations is that of ``refinement.''  We say \u000A <v>equiv1</v> ``refines'' <v>equiv2</v> iff <v>(equiv1 x y)</v> implies <v>(equiv2 x\u000A y)</v>.  <v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt> rules permit you to establish such connections \u000A between your equivalence relations.  The value of refinements is that if the \u000A system is trying to rewrite something while maintaining <v>equiv2</v> it is \u000A permitted to use as a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule any refinement of \u000A <v>equiv2</v>.  Thus, if <v>equiv1</v> is a refinement of <v>equiv2</v> and there \u000A are <v>equiv1</v> rewrite-rules available, they can be brought to bear while \u000A maintaining <v>equiv2</v>.  See <see topic=\"ACL2____REFINEMENT\">refinement</see>.</p> \u000A \u000A <p>The system initially has knowledge of two equivalence relations, equality, \u000A denoted by the symbol <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, and propositional equivalence, denoted by \u000A <tt><see topic=\"ACL2____IFF\">iff</see></tt>.  <tt><see topic=\"COMMON-LISP____EQUAL\">Equal</see></tt> is known to be a refinement of all equivalence \u000A relations and to preserve equality across all arguments of all functions.</p> \u000A \u000A <p>Typically there are five steps involved in introducing and using a new \u000A equivalence relation, equiv.</p> \u000A \u000A <blockquote> \u000A \u000A <p>(1) Define <v>equiv</v>,</p> \u000A \u000A <p>(2) prove the <v>:equivalence</v> lemma about <v>equiv</v>,</p> \u000A \u000A <p>(3) prove the <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> lemmas that show where <v>equiv</v> \u000A can be used to maintain known relations,</p> \u000A \u000A <p>(4) prove the <v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt> lemmas that relate <v>equiv</v> to \u000A known relations other than equal, and</p> \u000A \u000A <p>(5) develop the theory of conditional <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules that \u000A drive equiv rewriting.</p> \u000A \u000A </blockquote> \u000A \u000A <p>More will be written about this as we develop the techniques.  For now, \u000A here is an example that shows how to make use of equivalence relations in \u000A rewriting.</p> \u000A \u000A <p>Among the theorems proved below is</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> insert-sort-is-id\u000A  (perm (insert-sort x) x))</code> \u000A \u000A <p>Here <v>perm</v> is defined as usual with <v>delete</v> and is proved to be an \u000A equivalence relation and to be a congruence relation for <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>.</p> \u000A \u000A <p>Then we prove the lemma</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> insert-is-cons\u000A  (perm (insert a x) (<see topic=\"COMMON-LISP____CONS\">cons</see> a x)))</code> \u000A \u000A <p>which you must think of as you would <v>(insert a x) = (<see topic=\"COMMON-LISP____CONS\">cons</see> a x)</v>.</p> \u000A \u000A <p>Now prove <v>(perm (insert-sort x) x)</v>.  The base case is trivial.  The \u000A induction step is</p> \u000A \u000A <code>   (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A &amp; (perm (insert-sort (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A\u000A-&gt; (perm (insert-sort x) x).</code> \u000A \u000A <p>Opening <v>insert-sort</v> makes the conclusion be</p> \u000A \u000A <code>(perm (insert (<see topic=\"COMMON-LISP____CAR\">car</see> x) (insert-sort (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))) x).</code> \u000A \u000A <p>Then apply the induction hypothesis (rewriting <v>(insert-sort (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> \u000A to <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>), to make the conclusion be</p> \u000A \u000A <code>(perm (insert (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) x)</code> \u000A \u000A <p>Then apply <v>insert-is-cons</v> to get <v>(perm (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) x)</v>. \u000A But we know that <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> is <v>x</v>, so we get <v>(perm x\u000A x)</v> which is trivial, since <v>perm</v> is an equivalence relation.</p> \u000A \u000A <p>Here are the events.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((lt * *) =&gt; *))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> lt (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y)) nil))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> lt-non-symmetric (<see topic=\"ACL2____IMPLIES\">implies</see> (lt x y) (<see topic=\"COMMON-LISP____NOT\">not</see> (lt y x)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> insert (x lst)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) (<see topic=\"COMMON-LISP____LIST\">list</see> x))\u000A        ((lt x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) (<see topic=\"COMMON-LISP____CONS\">cons</see> x lst))\u000A        (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst) (insert x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> insert-sort (lst)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) nil)\u000A        (t (insert (<see topic=\"COMMON-LISP____CAR\">car</see> lst) (insert-sort (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> del (x lst)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) nil)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A        (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst) (del x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mem (x lst)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) nil)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) t)\u000A        (t (mem x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> perm (lst1 lst2)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst1) (<see topic=\"COMMON-LISP____ATOM\">atom</see> lst2))\u000A        ((mem (<see topic=\"COMMON-LISP____CAR\">car</see> lst1) lst2)\u000A         (perm (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst1) (del (<see topic=\"COMMON-LISP____CAR\">car</see> lst1) lst2)))\u000A        (t nil)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-reflexive\u000A  (perm x x))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-cons\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (mem a x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (perm x (<see topic=\"COMMON-LISP____CONS\">cons</see> a y))\u000A                  (perm (del a x) y)))\u000A  :hints ((&quot;Goal&quot; :induct (perm x y))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-symmetric\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (perm x y) (perm y x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> mem-del\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (mem a (del b x)) (mem a x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-mem\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (perm x y)\u000A                (mem a x))\u000A           (mem a y)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> mem-del2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (mem a x)\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a b)))\u000A           (mem a (del b x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> comm-del\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (del a (del b x)) (del b (del a x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-del\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (perm x y)\u000A           (perm (del a x) (del a y))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> perm-transitive\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (perm x y) (perm y z)) (perm x z)))\u000A\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> perm)\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> perm\u000A                    perm-reflexive\u000A                    perm-symmetric\u000A                    perm-transitive))\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> perm perm (<see topic=\"COMMON-LISP____CONS\">cons</see> x y) 2)\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> perm iff (mem x y) 2)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> atom-perm\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)) (perm x nil))\u000A  :rule-classes :forward-chaining\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> perm))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> insert-is-cons\u000A  (perm (insert a x) (<see topic=\"COMMON-LISP____CONS\">cons</see> a x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> insert-sort-is-id\u000A  (perm (insert-sort x) x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)) y))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (app (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))) nil))\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> perm perm (app x y) 2)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> app-cons\u000A  (perm (app a (<see topic=\"COMMON-LISP____CONS\">cons</see> b c)) (<see topic=\"COMMON-LISP____CONS\">cons</see> b (app a c))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> app-commutes\u000A  (perm (app a b) (app b a)))\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> perm perm (app x y) 1\u000A  :hints ((&quot;Goal&quot; :induct (app y x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rev-is-id (perm (rev x) x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> == (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> y))\u000A               (== (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)))\u000A          nil)\u000A      (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> y))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ==-reflexive (== x x))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ==-symmetric (<see topic=\"ACL2____IMPLIES\">implies</see> (== x y) (== y x)))\u000A\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> ==)\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> ==-symmetric ==-reflexive))\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> == == (<see topic=\"COMMON-LISP____CONS\">cons</see> x y) 2)\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> == iff (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) 1)\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> == == (app x y) 2)\u000A\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> == == (app x y) 1)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rev-rev (== (rev (rev x)) x))</code>"],

 "ACL2____TYPE-SET":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>To help you experiment with type-sets we briefly note the following \u000A utility functions.</p> \u000A \u000A <p><v>(type-set-quote x)</v> will return the type-set of the object <v>x</v>.  For \u000A example, <v>(type-set-quote &quot;test&quot;)</v> is <v>2048</v> and <v>(type-set-quote\u000A '(a b c))</v> is <v>512</v>.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____TYPE-SET\">type-set</see> 'term nil nil nil (ens state) (w state) nil nil nil)</v> will \u000A return the type-set of <v>term</v>.  For example,</p> \u000A \u000A <code>(<see topic=\"ACL2____TYPE-SET\">type-set</see> '(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) nil nil nil (ens state) (w state) nil nil nil)</code> \u000A \u000A <p>will return (mv 192 nil).  192, otherwise known as <v>*ts-boolean*</v>, is \u000A the type-set containing <v>t</v> and <v>nil</v>.  The second result may be ignored \u000A in these experiments.  <v>Term</v> must be in the <v>translated</v>, internal form \u000A shown by <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt>.  See <see topic=\"ACL2____TRANS\">trans</see> and see <see topic=\"ACL2____TERM\">term</see>.</p> \u000A \u000A <p><v>(type-set-implied-by-term 'x nil 'term (ens state)(w state) nil)</v> will \u000A return the type-set deduced for the variable symbol <v>x</v> assuming the \u000A <v>translated</v> term, <v>term</v>, true.  The second result may be ignored in \u000A these experiments.  For example,</p> \u000A \u000A <code>(type-set-implied-by-term 'v nil '(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v)\u000A                          (ens state) (w state) nil)</code> \u000A \u000A <p>returns <v>11</v>.</p> \u000A \u000A <p><v>(convert-type-set-to-term 'x ts (ens state) (w state) nil)</v> will return \u000A a term whose truth is equivalent to the assertion that the term <v>x</v> has \u000A type-set <v>ts</v>.  The second result may be ignored in these experiments.  For \u000A example</p> \u000A \u000A <code>(convert-type-set-to-term 'v 523 (ens state) (w state) nil)</code> \u000A \u000A <p>returns a term expressing the claim that <v>v</v> is either an integer or a \u000A non-<v>nil</v> true-list.  <v>523</v> is the <v>logical-or</v> of <v>11</v> (which \u000A denotes the integers) with <v>512</v> (which denotes the non-<v>nil</v> \u000A true-lists).</p> \u000A \u000A <p>The ``actual primitive types'' of ACL2 are listed in \u000A <v>*actual-primitive-types*</v>, whose elements are shown below.  Each actual \u000A primitive type denotes a set &mdash; sometimes finite and sometimes not \u000A &mdash; of ACL2 objects and these sets are pairwise disjoint.  For example, \u000A <v>*ts-zero*</v> denotes the set containing 0 while <v>*ts-positive-integer*</v> \u000A denotes the set containing all of the positive integers.</p> \u000A \u000A <code>*TS-ZERO*                  ;;; {0}\u000A*TS-POSITIVE-INTEGER*      ;;; positive integers\u000A*TS-POSITIVE-RATIO*        ;;; positive non-integer rationals\u000A*TS-NEGATIVE-INTEGER*      ;;; negative integers\u000A*TS-NEGATIVE-RATIO*        ;;; negative non-integer rationals\u000A*TS-COMPLEX-RATIONAL*      ;;; complex rationals\u000A*TS-NIL*                   ;;; {nil}\u000A*TS-T*                     ;;; {t}\u000A*TS-NON-T-NON-NIL-SYMBOL*  ;;; symbols other than nil, t\u000A*TS-PROPER-CONS*           ;;; null-terminated non-empty lists\u000A*TS-IMPROPER-CONS*         ;;; conses that are not proper\u000A*TS-STRING*                ;;; strings\u000A*TS-CHARACTER*             ;;; characters</code> \u000A \u000A <p>The actual primitive types were chosen by us to make theorem proving \u000A convenient.  Thus, for example, the actual primitive type <v>*ts-nil*</v> \u000A contains just <v>nil</v> so that we can encode the hypothesis ``<v>x</v> is \u000A <v>nil</v>'' by saying ``<v>x</v> has type <v>*ts-nil*</v>'' and the hypothesis \u000A ``<v>x</v> is non-<v>nil</v>'' by saying ``<v>x</v> has type complement of \u000A <v>*ts-nil*</v>.''  We similarly devote a primitive type to <v>t</v>, <v>*ts-t*</v>, \u000A and to a third type, <v>*ts-non-t-non-nil-symbol*</v>, to contain all the other \u000A ACL2 symbols.</p> \u000A \u000A <p>Let <v>*ts-other*</v> denote the set of all Common Lisp objects other than \u000A those in the actual primitive types.  Thus, <v>*ts-other*</v> includes such \u000A things as floating point numbers and CLTL array objects.  The actual primitive \u000A types together with <v>*ts-other*</v> constitute what we call <v>*universe*</v>. \u000A Note that <v>*universe*</v> is a finite set containing one more object than \u000A there are actual primitive types; that is, here we are using <v>*universe*</v> \u000A to mean the finite set of primitive types, not the infinite set of all objects \u000A in all of those primitive types.  <v>*Universe*</v> is a partitioning of the set \u000A of all Common Lisp objects: every object belongs to exactly one of the sets in \u000A <v>*universe*</v>.</p> \u000A \u000A <p>Abstractly, a ``type-set'' is a subset of <v>*universe*</v>.  To say that a \u000A term, <v>x</v>, ``has type-set <v>ts</v>'' means that under all possible \u000A assignments to the variables in <v>x</v>, the value of <v>x</v> is a member of \u000A some member of <v>ts</v>.  Thus, <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x y)</v> has type-set \u000A <v>{*ts-proper-cons* *ts-improper-cons*}</v>.  A term can have more than one \u000A type-set.  For example, <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x y)</v> also has the type-set \u000A <v>{*ts-proper-cons* *ts-improper-cons* *ts-nil*}</v>.  Extraneous types can be \u000A added to a type-set without invalidating the claim that a term ``has'' that \u000A type-set.  Generally we are interested in the smallest type-set a term has, \u000A but because the entire theorem-proving problem for ACL2 can be encoded as a \u000A type-set question, namely, ``Does <v>p</v> have type-set complement of \u000A <v>*ts-nil*</v>?,'' finding the smallest type-set for a term is an undecidable \u000A problem.  When we speak informally of ``the'' type-set we generally mean ``the \u000A type-set found by our heuristics'' or ``the type-set assumed in the current \u000A context.''</p> \u000A \u000A <p>Note that if a type-set, <v>ts</v>, does not contain <v>*ts-other*</v> as an \u000A element then it is just a subset of the actual primitive types.  If it does \u000A contain <v>*ts-other*</v> it can be obtained by subtracting from <v>*universe*</v> \u000A the complement of <v>ts</v>.  Thus, every type-set can be written as a (possibly \u000A complemented) subset of the actual primitive types.</p> \u000A \u000A <p>By assigning a unique bit position to each actual primitive type we can \u000A encode every subset, <v>s</v>, of the actual primitive types by the nonnegative \u000A integer whose ith bit is on precisely if <v>s</v> contains the ith actual \u000A primitive type.  The type-sets written as the complement of <v>s</v> are encoded \u000A as the <v>twos-complement</v> of the encoding of <v>s</v>.  Those type-sets are \u000A thus negative integers.  The bit positions assigned to the actual primitive \u000A types are enumerated from <v>0</v> in the same order as the types are listed in \u000A <v>*actual-primitive-types*</v>.  At the concrete level, a type-set is an \u000A integer between <v>*min-type-set*</v> and <v>*max-type-set*</v>, inclusive.</p> \u000A \u000A <p>For example, <v>*ts-nil*</v> has bit position <v>6</v>.  The type-set \u000A containing just <v>*ts-nil*</v> is thus represented by <v>64</v>.  If a term has \u000A type-set <v>64</v> then the term is always equal to <v>nil</v>.  The type-set \u000A containing everything but <v>*ts-nil*</v> is the twos-complement of <v>64</v>, \u000A which is <v>-65</v>.  If a term has type-set <v>-65</v>, it is never equal to \u000A <v>nil</v>.  By ``always'' and ``never'' we mean under all, or under no, \u000A assignments to the variables, respectively.</p> \u000A \u000A <p>Here is a more complicated example.  Let <v>s</v> be the type-set containing \u000A all of the symbols and the natural numbers.  The relevant actual primitive \u000A types, their bit positions and their encodings are:</p> \u000A \u000A <code>actual primitive type       bit    value\u000A\u000A*ts-zero*                    0       1\u000A*ts-positive-integer*        1       2\u000A*ts-nil*                     6      64\u000A*ts-t*                       7     128\u000A*ts-non-t-non-nil-symbol*    8     256</code> \u000A \u000A <p>Thus, the type-set <v>s</v> is represented by <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 2 64 128 256)</v> = \u000A <v>451</v>.  The complement of <v>s</v>, i.e., the set of all objects other than \u000A the natural numbers and the symbols, is <v>-452</v>.</p>"],

 "ACL2____LOCAL":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> hack1\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                       (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y)\u000A                       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y) 1))\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> y (<see topic=\"COMMON-LISP_____F2\">/</see> x)))))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> double-naturals-induction (a b)\u000A         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> b) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 a) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 b))\u000A                (double-naturals-induction (<see topic=\"COMMON-LISP____1-\">1-</see> a) (<see topic=\"COMMON-LISP____1-\">1-</see> b)))\u000A               (t (<see topic=\"COMMON-LISP____LIST\">list</see> a b)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____LOCAL\">local</see> ev)</code> \u000A \u000A <p>where <v>ev</v> is an event form.  If the current default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> \u000A (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> and <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>nil</v> or <v>t</v>, then <v>(<see topic=\"ACL2____LOCAL\">local</see> ev)</v> is equivalent to \u000A <v>ev</v>.  But if the current default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or if <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, then \u000A <v>(<see topic=\"ACL2____LOCAL\">local</see> ev)</v> is a <v>no-op</v>.  Thus, if such forms are in the event list of \u000A an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event or in a book, they are processed when the \u000A encapsulation or book is checked for admissibility in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A but are skipped when extending the host <see topic=\"ACL2____WORLD\">world</see>.  Such <see topic=\"ACL2____EVENTS\">events</see> are \u000A thus considered ``local'' to the verification of the encapsulation or book. \u000A The non-local <see topic=\"ACL2____EVENTS\">events</see> are the ones ``exported'' by the encapsulation or \u000A book.  See <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> for a thorough discussion.  Also see <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see> for a discussion of a commonly encountered problem with \u000A such event hiding: you can't make an event local if its presence is required \u000A to make sense of a non-local one.</p> \u000A \u000A <p>Note that <see topic=\"ACL2____EVENTS\">events</see> that change the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, and in \u000A fact any <see topic=\"ACL2____EVENTS\">events</see> that set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, are \u000A disallowed inside the scope of <v>local</v>.  See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.</p>"],

 "ACL2____COMP":[["Events","Programming"],"ACL2 Sources","ACL2","<p>NOTE: <v>Comp</v> is a no-op if explicit compilation is suppressed; \u000A see <see topic=\"ACL2____COMPILATION\">compilation</see>.  The documentation here assumes that this is not the \u000A case.</p> \u000A \u000A <code>Examples:\u000A:comp t          ; compile all uncompiled ACL2 functions\u000A(<see topic=\"ACL2____COMP\">comp</see> t)         ; same as above, but can be put into a book\u000A(<see topic=\"ACL2____COMP\">comp</see> :exec)     ; compile all uncompiled logic (``*1*'') definitions\u000A:comp foo        ; compile the defined function foo\u000A:comp (:raw foo) ; compile the raw Lisp version of the defined function foo\u000A                   but not the corresponding logic definition\u000A:comp (foo bar)  ; compile the defined functions foo and bar\u000A:comp (foo (:raw bar))  ; compile the defined functions foo and bar, but for\u000A                        ; bar do not compile the corresponding logic definition\u000A\u000AGeneral Form:\u000A:comp specifier\u000Awhere specifier is one of the following:\u000A\u000A  t                     compile all user-defined ACL2 functions that are\u000A                          currently uncompiled (redefined built-in functions\u000A                          are not recompiled)\u000A  :exec                 same as t, except that only logic versions are\u000A                          compiled (see below), not raw Lisp definitions\u000A  :raw                  same as t, except that only raw Lisp definitions are\u000A                          compiled, not logic version (see below)\u000A  (name-1 ... name-k)   a non-empty list of names of functions defined by\u000A                          DEFUN in ACL2, except that each name-i can be of\u000A                          the form (:raw sym) or (:exec sym), where sym is\u000A                        the name of such a function\u000A  name                  same as (<see topic=\"ACL2____NAME\">name</see>)</code> \u000A \u000A <p>When you define a function in ACL2, you are really causing two definitions \u000A to be made ``under the hood'' in Common Lisp: the definition is submitted \u000A explicitly to raw Lisp, but so is a corresponding ``logic definition''.  If \u000A guards have not been verified, then only the logic definition will be \u000A evaluated; see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>, in particular the section titled \u000A ``Guards and evaluation V: efficiency issues''.</p> \u000A \u000A <p>Thus, if you are not verifying <see topic=\"ACL2____GUARD\">guard</see>s and you want the benefit of \u000A Lisp compilation for speed and space efficiency, then you may want to place \u000A the form <v>(<see topic=\"ACL2____COMP\">comp</see> :exec)</v> in your <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p>Generally it is not necessary to place the form <v>(<see topic=\"ACL2____COMP\">comp</see> t)</v>, or the form \u000A <v>(<see topic=\"ACL2____COMP\">comp</see> :raw)</v>, in a book, because <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> compiles the raw \u000A Lisp definitions anyhow, by default.  But you may wish to put <v>(<see topic=\"ACL2____COMP\">comp</see> t)</v> or \u000A <v>(<see topic=\"ACL2____COMP\">comp</see> fn1 fn2 ... fnk)</v> in a book when such a form precedes expensive \u000A calls of functions, for example for proofs involving calls of functions on \u000A large constants, or to support computationally expensive macroexpansion.</p> \u000A \u000A <p>As suggested by the examples above, if a function specifier is of the form \u000A <v>(:raw fn)</v>, then <v>fn</v> will be compiled in raw Common Lisp but its \u000A corresponding logic definition will not be compiled; and for <v>(:exec fn)</v>, \u000A it's the other way around.</p> \u000A \u000A <p>The use of <v>:comp</v> may create various files whose names start with \u000A ``<v>TMP*</v>'', but it then deletes them.  If you want to save these files, \u000A evaluate <v>(<see topic=\"ACL2____ASSIGN\">assign</see> keep-tmp-files t)</v>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see> for a way to compile each function as it is \u000A defined.  But note that <v>set-compile-fns</v> is ignored during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>Note that if functions are traced (see <see topic=\"ACL2____TRACE_42\">trace$</see>), then <v>comp</v> will \u000A first untrace the functions that are to be compiled, then will do the \u000A compile(s), and finally will re-trace the functions that it untraced (using \u000A their original trace specs).  In particular, if you have traced a function and \u000A then you compile it using <v>:comp</v>, the resulting traced function will be \u000A compiled as well unless you specified <v>:compile nil</v> in your trace spec; \u000A and after you untrace the function it will definitely run compiled.</p> \u000A \u000A <p>We conclude with a technical remark only for those who use trust tags to \u000A write raw Lisp code.  <v>:Comp</v> generally creates files to compile unless it \u000A is given a single function to compile.  Those files contain the ACL2 \u000A definitions of all functions to compile, omitting those in the lists obtained \u000A by evaluating the forms <v>(<see topic=\"ACL2_____04\">@</see> logic-fns-with-raw-code)</v> and <v>(<see topic=\"ACL2_____04\">@</see>\u000A program-fns-with-raw-code)</v>.  <v>:Comp</v> skips compilation for functions that \u000A are already compiled, as is typically the case when you redefine functions in \u000A raw Lisp using <see topic=\"ACL2____INCLUDE-RAW\">include-raw</see>.  But if you define interpreted (as opposed \u000A to compiled) functions with raw Lisp code, say by using trust tags (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) and <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, then you are advised to add all such symbols to \u000A one of the lists stored in the two <see topic=\"ACL2____STATE\">state</see> globals above: to \u000A <v>logic-fns-with-raw-code</v> if the function symbol is in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A mode, else to <v>program-fns-with-raw-code</v>.  Then, instead of the \u000A corresponding ACL2 definition (without raw Lisp code) being written to a file, \u000A the function symbol will be passed directly to the Lisp <v>compile</v> function. \u000A Note that the above two state globals are both untouchable, so you may need to \u000A deal with that before modifying them, for example as follows (also see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((temp-touchable-vars t set-temp-touchable-vars))\u000A (<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'logic-fns-with-raw-code\u000A                       (<see topic=\"COMMON-LISP____CONS\">cons</see> 'my-fn (<see topic=\"ACL2_____04\">@</see> logic-fns-with-raw-code))\u000A                       state)))</code>"],

 "ACL2____DEFTHM":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> assoc-of-app\u000A        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c)\u000A               (app a (app b c))))</code> \u000A \u000A <p>The following nonsensical example illustrates all the optional arguments \u000A but is illegal because not all combinations are permitted.  See <see topic=\"ACL2____HINTS\">hints</see> \u000A for a complete list of <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> main\u000A        (<see topic=\"ACL2____IMPLIES\">implies</see> (hyps x y z) (concl x y z))\u000A       :rule-classes (:REWRITE :GENERALIZE)\u000A       :instructions (<see topic=\"ACL2____INDUCT\">induct</see> prove promote (dive 1) x\u000A                             (dive 2) = top (drop 2) prove)\u000A       :hints ((&quot;Goal&quot;\u000A                :do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see> fertilize)\u000A                :in-theory (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see>\u000A                             (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                             '(<see topic=\"COMMON-LISP____ASSOC\">assoc</see>))\u000A                :induct (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n a) (<see topic=\"COMMON-LISP____NTH\">nth</see> n b))\u000A                :use ((:instance assoc-of-append\u000A                                 (x a) (y b) (z c))\u000A                      (:functional-instance\u000A                        (:instance p-f (x a) (y b))\u000A                        (p consp)\u000A                        (f assoc)))))\u000A       :otf-flg t)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> name term\u000A        :rule-classes rule-classes\u000A        :instructions instructions\u000A        :hints        hints\u000A        :otf-flg      otf-flg\u000A        :doc          doc-string)</code> \u000A \u000A <p>where <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>), <v>term</v> is a \u000A term alleged to be a theorem, and <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>, <tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt>, \u000A <tt><see topic=\"ACL2____HINTS\">hints</see></tt>, <tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> are as described in \u000A their respective <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.  The five keyword arguments above are \u000A all optional, however you may not supply both <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>, since one drives the proof checker and the other drives \u000A the theorem prover.  If <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> is not specified, the list \u000A <v>(:rewrite)</v> is used; if you wish the theorem to generate no rules, specify \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>nil</v>.</p> \u000A \u000A <p>When ACL2 processes a <v>defthm</v> event, it first tries to prove the <see topic=\"ACL2____TERM\">term</see> using the indicated hints (see <see topic=\"ACL2____HINTS\">hints</see>) or <see topic=\"ACL2____INSTRUCTIONS\">instructions</see> (see \u000A <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>).  If it is successful, it stores the rules described by \u000A the rule-classes (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>), proving the necessary \u000A corollaries.</p>"],

 "ACL2____PROGRAM":[["Defun-mode"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;:program\u000AACL2 p!&gt;</code> \u000A \u000A <p>Typing the keyword <v>:program</v> sets the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to \u000A <v>:program</v>.</p> \u000A \u000A <p>Functions defined in <v>:program</v> mode are logically undefined but can be \u000A executed on constants outside of deductive contexts.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.</p> \u000A \u000A <p>Calls of the following macros are ignored (skipped) when in <v>:program</v> \u000A mode.</p> \u000A \u000A <code>local\u000Averify-guards\u000Averify-termination\u000Adefaxiom\u000Adefthm\u000Adeftheory\u000Ain-theory\u000Ain-arithmetic-theory\u000Aregenerate-tau-database\u000Atheory-invariant\u000Adefchoose</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> for a discussion of the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s available \u000A and what their effects on the logic are.  See <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> for a \u000A discussion of how the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is used.  This event is \u000A equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program)</v>, and hence \u000A is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in \u000A which it occurs. See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>Recall that the top-level form <v>:program</v> is equivalent to \u000A <v>(<see topic=\"ACL2____PROGRAM\">program</see>)</v>; see <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.  Thus, to change the default \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to <v>:program</v> in a book, use <v>(<see topic=\"ACL2____PROGRAM\">program</see>)</v>, which is an \u000A embedded event form, rather than <v>:program</v>, which is not a legal form for \u000A <see topic=\"ACL2____BOOKS\">books</see>.  See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.</p>"],

 "ACL2____CLAUSE-PROCESSOR":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <p>We will introduce clause-processor rules by way of the following example. \u000A But note that the clause-processor utility is more general than this example \u000A may suggest; for example, the second argument of <v>evl0</v> in the hypothesis \u000A need not be the same as its second argument in the conclusion.</p> \u000A \u000A <code>; Example (which we'll return to, below):\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> correctness-of-note-fact-clause-processor\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp cl)\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> a)\u000A                (evl0 (conjoin-clauses\u000A                       (note-fact-clause-processor cl term))\u000A                      a))\u000A           (evl0 (disjoin cl) a))\u000A  :rule-classes :clause-processor)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see> for documentation of an \u000A analogous utility that does not require the clause-processor to be proved \u000A correct.  But please read the present documentation before reading about that \u000A utility.  Both utilities designate functions ``clause-processors''.  Such \u000A functions must be executable &mdash; hence not constrained by virtue of being \u000A introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see> of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> &mdash; and must \u000A respect <see topic=\"ACL2____STOBJ\">stobj</see> and output arity restrictions.  For example, something \u000A like <v>(<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____MV\">mv</see> ...))</v> is illegal; also see <see topic=\"ACL2____SIGNATURE\">signature</see>.</p> \u000A \u000A <p>We begin this documentation with an introduction, focusing on the example \u000A above, and then conclude with a detailed general discussion of \u000A clause-processor rules.  You might find it most useful simply to look at the \u000A examples in community books directory <v>books/clause-processors/</v>; see file \u000A <v>Readme.lsp</v> in that directory.</p> \u000A \u000A <h4>INTRODUCTION</h4> \u000A \u000A <p>A <v>:clause-processor</v> rule installs a simplifier at the level of goals, \u000A where a goal is represented as a <i>clause</i>: a list of <see topic=\"ACL2____TERM\">term</see>s that is \u000A implicitly viewed as a disjunction (the application of <tt><see topic=\"COMMON-LISP____OR\">or</see></tt>).  For \u000A example, if ACL2 prints a goal in the form <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p q) r)</v>, then \u000A the clause might be the one-element list containing the internal \u000A representation of this term &mdash; <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____IF\">if</see> p q 'nil) r)</v> &mdash; but \u000A more likely, the corresponding clause is <v>((<see topic=\"COMMON-LISP____NOT\">not</see> p) (<see topic=\"COMMON-LISP____NOT\">not</see> q) r)</v>.  Note that \u000A the members of a clause are <i>translated</i> terms; see <see topic=\"ACL2____TERM\">term</see>.  For \u000A example, they do not contain calls of the macro <v>AND</v>, and constants are \u000A quoted.</p> \u000A \u000A <p>Note that clause-processor simplifiers are similar to metafunctions, and \u000A similar efficiency considerations apply.  See <see topic=\"ACL2____META\">meta</see>, in particular the \u000A discussion on how to ``make a metafunction maximally efficient.''</p> \u000A \u000A <p>Unlike rules of class <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>, rules of class \u000A <v>:clause-processor</v> must be applied by explicit <v>:clause-processor</v> \u000A <see topic=\"ACL2____HINTS\">hints</see>; they are not applied automatically (unless by way of computed \u000A hints; see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>).  But <v>:clause-processor</v> rules can be \u000A useful in situations for which it is more convenient to code a simplifier that \u000A manipulates the entire goal clause rather than individual subterms of terms in \u000A the clause.</p> \u000A \u000A <p>We begin with a simple illustrative example: a clause-processor that \u000A assumes an alleged fact (named <v>term</v> in the example) and creates a \u000A separate goal to prove that fact.  We can extend the hypotheses of the current \u000A goal (named <v>cl</v> in the example) with a term by adding the negation of that \u000A term to the clause (disjunctive) representation of that goal.  So the \u000A following returns a list of two clauses: the result of adding <v>term</v> as a \u000A hypothesis to the input clause, as just described, and a second clause \u000A consisting only of that term.  This list of two clauses can be viewed as the \u000A conjunction of the first clause and the second clause (where again, each \u000A clause is viewed as a disjunction).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> note-fact-clause-processor (cl term)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) ; optional, for better efficiency\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____LIST\">list</see> 'not term)\u000A              cl)\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> term)))</code> \u000A \u000A <p>As with <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules, we need to introduce a suitable \u000A evaluator; see <see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> if you want details.  Since we expect to \u000A reason about the function <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt>, because of its role in \u000A <v>note-fact-clause-processor</v> as defined above, we include <v>NOT</v> in the \u000A set of functions known to this evaluator.  We also include <v>IF</v>, as is \u000A often a good idea.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl0 evl0-list\u000A  ((<see topic=\"COMMON-LISP____NOT\">not</see> x) (<see topic=\"COMMON-LISP____IF\">if</see> x y z)))</code> \u000A \u000A <p>ACL2 can now prove the following theorem automatically.  (This is the \u000A example displayed at the outset of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.)  Of \u000A course, <v>:clause-processor</v> rules about clause-processor functions less \u000A trivial than <v>note-fact-clause-processor</v> may require lemmas to be proved \u000A first!  The function <v>disjoin</v> takes a clause and returns its disjunction \u000A (the result of applying <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> to its members), and <v>conjoin-clauses</v> \u000A applies <v>disjoin</v> to every element of a given list of clauses and then \u000A conjoins (applies <v>AND</v>) to the corresponding list of resulting terms.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> correctness-of-note-fact-clause-processor\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp cl)\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> a)\u000A                (evl0 (conjoin-clauses\u000A                       (note-fact-clause-processor cl term))\u000A                      a))\u000A           (evl0 (disjoin cl) a))\u000A  :rule-classes :clause-processor)</code> \u000A \u000A <p>Now let us submit a silly but illustrative example theorem to ACL2, to show \u000A how a corresponding <v>:clause-processor</v> hint is applied.  The hint says to \u000A apply the clause-processor function, <v>note-fact-clause-processor</v>, to the \u000A current goal clause and a ``user hint'' as the second argument of that \u000A function, in this case <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> a a)</v>.  Thus, a specific variable, \u000A <v>clause</v>, is always bound to the current goal clause for the evaluation of \u000A the <v>:clause-processor</v> hint, to produce a list of clauses.  Since two \u000A subgoals are created below, we know that this list contained two clauses. \u000A Indeed, these are the clauses returned when <v>note-fact-clause-processor</v> is \u000A applied to two arguments: the current clause, which is the one-element list \u000A <v>((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) x))</v>, and the user hint, <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> a a)</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A                   x)\u000A            :hints\u000A            ((&quot;Goal&quot;\u000A              :clause-processor\u000A              (note-fact-clause-processor clause '(<see topic=\"COMMON-LISP____EQUAL\">equal</see> a a)))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AWe now apply the verified :CLAUSE-PROCESSOR function NOTE-FACT-CLAUSE-\u000APROCESSOR to produce two new subgoals.\u000A\u000ASubgoal 2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> A A)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X Y)) X)).\u000A\u000ABut we reduce the conjecture to T, by the :executable-counterpart of\u000AIF and the simple :rewrite rule CAR-CONS.\u000A\u000ASubgoal 1\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> A A).\u000A\u000ABut we reduce the conjecture to T, by primitive type reasoning.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:EXECUTABLE-COUNTERPART IF)\u000A        (:EXECUTABLE-COUNTERPART NOT)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:REWRITE CAR-CONS))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>That concludes our introduction to clause-processor rules and hints.  We \u000A turn now to detailed documentation.</p> \u000A \u000A <h4>DETAILED DOCUMENTATION</h4> \u000A \u000A <p>The <see topic=\"ACL2____SIGNATURE\">signature</see> of a clause-processor function, <v>CL-PROC</v>, must \u000A have one of the following forms.  Here, each <v>st_i</v> is a <see topic=\"ACL2____STOBJ\">stobj</see> \u000A (possibly <v>state</v>) while the other parameters and results are not stobjs \u000A (see <see topic=\"ACL2____STOBJ\">stobj</see>).  Note that there need not be input stobjs in [3] &mdash; \u000A i.e., <v>k</v> can be 0 &mdash; and even if there are, there need not be output \u000A stobjs.</p> \u000A \u000A <code>[1]  ((CL-PROC cl) =&gt; cl-list)\u000A\u000A[2]  ((CL-PROC cl hint) =&gt; cl-list)\u000A\u000A[3]  ((CL-PROC cl hint st_1 ... st_k) =&gt; (<see topic=\"ACL2____MV\">mv</see> erp cl-list st_i1 ... st_in))</code> \u000A \u000A <p>In [3], we think of the first component of the result as an error flag. \u000A Indeed, a proof will instantly abort if that error flag is not <v>nil</v>.</p> \u000A \u000A <p>We next discuss the legal forms of <v>:clause-processor</v> rules, followed \u000A below by a discussion of <v>:clause-processor</v> <see topic=\"ACL2____HINTS\">hints</see>.  In the \u000A discussion below, we use lower-case names to represent specific symbols, for \u000A example <v>implies</v>, and we use upper-case names to represent more arbitrary \u000A pieces of syntax (which we will describe), for example, <v>CL</v>.</p> \u000A \u000A <p>If a <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> specification includes \u000A <v>:clause-processor</v>, then the corresponding term must have the following \u000A form.  (Additional ``meta-extract'' hypotheses, not shown or discussed below, \u000A may be included as desired in order to use facts from the logical <tt><see topic=\"ACL2____WORLD\">world</see></tt> to help prove the rule; see <see topic=\"ACL2____META-EXTRACT\">meta-extract</see> for explanation of this \u000A advanced feature.)</p> \u000A \u000A <code>; General Form (omitting possible meta-extract hypotheses)\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp CL)\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> A)\u000A              (EVL (conjoin-clauses &lt;CL-LIST&gt;)\u000A                    B))\u000A         (EVL (disjoin CL) A))</code> \u000A \u000A <p>Here <v>EVL</v> is a known evaluator; <v>CL</v> and <v>A</v> are distinct \u000A non-stobj variables; and <v>&lt;CL-LIST&gt;</v> is an expression representing the \u000A clauses returned by the clause-processor function <v>CL-PROC</v>, whose form \u000A depends on the <see topic=\"ACL2____SIGNATURE\">signature</see> of that function, as follows.  Typically \u000A <v>B</v> is <v>A</v>, but it can be any term (useful when generalization is \u000A occurring; see the example ``Test generalizing alist'' in community book \u000A <v>books/clause-processors/basic-examples.lisp</v>).  For cases [1] and [2] \u000A above, <v>&lt;CL-LIST&gt;</v> is of the form <v>(CL-PROC CL)</v> or <v>(CL-PROC CL\u000A HINT)</v>, respectively, where in the latter case <v>HINT</v> is a non-stobj \u000A variable distinct from the variables <v>CL</v> and <v>A</v>.  For case [3], \u000A <v>&lt;CL-LIST&gt;</v> is of the form</p> \u000A \u000A <code>(clauses-result (CL-PROC CL HINT st_1 ... st_k))</code> \u000A \u000A <p>where the <v>st_i</v> are the specific stobj names mentioned in [3]. \u000A Logically, <v>clauses-result</v> returns the <tt><see topic=\"COMMON-LISP____CADR\">cadr</see></tt> if the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is \u000A <v>NIL</v>, and otherwise (for the error case) returns a list containing the \u000A empty (false) clause.  So in the non-error case, <v>clauses-result</v> picks out \u000A the second result, denoted <v>cl-list</v> in [3] above, and in the error case \u000A the implication above trivially holds.</p> \u000A \u000A <p>In the above theorem, we are asked to prove <v>(EVL (disjoin CL) A)</v> \u000A assuming that the conjunction of all clauses produced by the clause processor \u000A evaluates to a non-<v>nil</v> value under some alist <v>B</v>.  In fact, we can \u000A choose <v>B</v> so as to allow us to assume evaluations of the generated clauses \u000A over many different alists.  This technique is discussed in the community book \u000A <v>books/clause-processors/multi-env-trick.lisp</v>, which introduces some \u000A macros that may be helpful in accomplishing proofs of this type.</p> \u000A \u000A <p>The clause-processor function, <v>CL</v>, must have a guard that ACL2 can \u000A trivially prove from the hypotheses that the first argument of <v>CL</v> is \u000A known to be a <v>pseudo-term-listp</v> and any <see topic=\"ACL2____STOBJ\">stobj</see> arguments are \u000A assumed to satisfy their stobj predicates.</p> \u000A \u000A <p>Next we specify the legal forms for <v>:clause-processor</v> <see topic=\"ACL2____HINTS\">hints</see>. \u000A These depend on the signature as described in [1] through [3] above.  Below, \u000A as above, <v>CL-PROC</v> is the clause-processor function, and references to \u000A ``<v>clause</v>'' refer to that exact variable (not, for example, to <v>cl</v>). \u000A In each of the three cases, the forms shown for that case are equivalent; in \u000A particular, the <v>:function</v> syntax is simply a convenience for the final \u000A form in each case.</p> \u000A \u000A <p>Signature [1], <v>((cl-proc cl) =&gt; cl-list)</v>:</p> \u000A \u000A <code>:clause-processor CL-PROC\u000A:clause-processor (:function CL-PROC)\u000A:clause-processor (CL-PROC clause)</code> \u000A \u000A <p>or any term macroexpanding to <v>(CL-PROC clause)</v>.</p> \u000A \u000A <p>Signature [2], ((cl-proc cl hint) =&gt; cl-list):</p> \u000A \u000A <code>:clause-processor (:function CL-PROC :hint HINT)\u000A:clause-processor (CL-PROC clause HINT)</code> \u000A \u000A <p>or any term macroexpanding to <v>(CL-PROC clause HINT)</v>, where <v>HINT</v> \u000A is any term with at most <v>CLAUSE</v> free.</p> \u000A \u000A <p>Signature [3], ((CL-PROC cl hint ...) =&gt; (mv erp cl-list ...))</p> \u000A \u000A <code>:clause-processor (:function CL-PROC :hint HINT)\u000A:clause-processor (CL-PROC clause HINT st_1 ... st_k)</code> \u000A \u000A <p>or any term macroexpanding to <v>(CL-PROC clause HINT st_1 ... st_k)</v>, \u000A where <v>HINT</v> is any term with at most <v>CLAUSE</v> free.</p> \u000A \u000A <p>A <v>:clause-processor</v> hint causes the proof to abort if the result \u000A returned by evaluating the suitable <v>CL-PROC</v> call, as above, is not a list \u000A of clauses, i.e., a list of (translated) <see topic=\"ACL2____TERM\">term</see> lists.  The proof also \u000A aborts if in case [3] the first (<v>erp</v>) value returned is not <v>nil</v>, in \u000A which case <v>erp</v> is used for printing an error message as follows: if it is \u000A a string, then that string is printed; but if it is a non-empty true list \u000A whose first element is a string, then it is printed as though by <v>(<see topic=\"ACL2____FMT\">fmt</see> ~@0\u000A (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 erp)) ...)</v> (see <see topic=\"ACL2____FMT\">fmt</see>).  Otherwise, a non-<v>nil</v> \u000A <v>erp</v> value causes a generic error message to be printed.</p> \u000A \u000A <p>If there is no error as above, but the <v>CL-PROC</v> call returns clause \u000A list whose single element is equal to the input clause, then the hint is \u000A ignored since we are left with the goal with which we started.  In that case, \u000A the other prover processes are then applied as usual.</p> \u000A \u000A <p>You can see all current <v>:clause-processor</v> rules by issuing the \u000A following command: <v>(print-clause-processor-rules)</v>.</p> \u000A \u000A <p>The following paper discusses ACL2 clause-processors at a high level \u000A suitable for a non-ACL2 audience:</p> \u000A \u000A <blockquote> \u000A \u000A <p>M. Kaufmann, J S. Moore, S. Ray, and E. Reeber, ``Integrating External \u000A Deduction Tools with ACL2.''  <i>Journal of Applied Logic</i> (Special Issue: \u000A Empirically Successful Computerized Reasoning), Volume 7, Issue 1, March 2009, \u000A pp. 3--25.  Also published online (DOI <v>10.1016/j.jal.2007.07.002</v>). \u000A Preliminary version in: Proceedings of the 6th International Workshop on the \u000A Implementation of Logics (IWIL 2006) (C. Benzmueller, B. Fischer, and \u000A G. Sutcliffe, editors), <a href='http://ceur-ws.org/Vol-212/'>CEUR Workshop \u000A Proceedings Vol. 212</a>, Phnom Penh, Cambodia, pp. 7-26, November \u000A 2006.</p></blockquote> \u000A \u000A "],

 "ACL2____SET-RAW-MODE":[["Defttag"],"ACL2 Sources","ACL2","<p>Below we discuss raw-mode.  In brief: The simplest way to turn \u000A raw-mode on is <v>:SET-RAW-MODE-ON!</v>, and to turn it off, <v>:SET-RAW-MODE\u000A NIL</v>.  Also see <see topic=\"ACL2____SET-RAW-MODE-ON_12\">set-raw-mode-on!</see>.</p> \u000A \u000A <p>ACL2 users often find its careful syntax checking to be helpful during code \u000A development.  Sometimes it is even useful to do code development in \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, where ACL2 can be used to check termination of \u000A (mutually) recursive functions, verify guards, or even prove properties of the \u000A functions.</p> \u000A \u000A <p>However, loading code using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is much slower than using \u000A Common Lisp <v>load</v> in raw Lisp, and in this sense ACL2 can get in the way \u000A of efficient execution.  Unfortunately, it is error-prone to use ACL2 sources \u000A (or their compilations) in raw Lisp, primarily because a number of ACL2 \u000A primitives will not let you do so.  Perhaps you have seen this error message \u000A when trying to do so:</p> \u000A \u000A <code>HARD ACL2 ERROR in ACL2-UNWIND-PROTECT:  Apparently you have tried\u000Ato execute a form in raw Lisp that is only intended to be executed\u000Ainside the ACL2 loop.</code> \u000A \u000A <p>Even without this problem it is important to enter the ACL2 loop (see <see topic=\"ACL2____LP\">lp</see>), for example in order to set the <tt><see topic=\"ACL2____CBD\">cbd</see></tt> and (to get more technical) \u000A the readtable.</p> \u000A \u000A <p>ACL2 provides a ``raw mode'' for execution of raw Lisp forms.  In this \u000A mode, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> reduces essentially to a Common Lisp <v>load</v>. \u000A More generally, the ACL2 logical <tt><see topic=\"ACL2____WORLD\">world</see></tt> is not routinely extended in \u000A raw mode (some sneaky tricks are probably required to make that happen).  To \u000A turn raw mode off or on:</p> \u000A \u000A <code>:set-raw-mode t   ; turn raw mode on\u000A:set-raw-mode nil ; turn raw mode off</code> \u000A \u000A <p>The way you can tell that you are in raw mode is by looking at the prompt \u000A (see <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>), which uses a capital ``<v>P</v>'' (suggesting \u000A something like program mode, but more so).</p> \u000A \u000A <code>ACL2 P&gt;</code> \u000A \u000A <p>Typical benefits of raw mode are fast loading of source and compiled files \u000A and the capability to hack arbitrary Common Lisp code in an environment with \u000A the ACL2 sources loaded (and hence with ACL2 primitives available).  In \u000A addition, ACL2 hard errors will put you into the Lisp debugger, rather than \u000A returning you to the ACL2 loop, and this may be helpful for debugging; see \u000A <see topic=\"ACL2____HARD-ERROR\">hard-error</see> and see <see topic=\"ACL2____ILLEGAL\">illegal</see>, but also see <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>. \u000A However, it probably is generally best to avoid raw mode unless these \u000A advantages seem important.  We expect the main benefit of raw mode to be in \u000A deployment of applications, where load time is much faster than the time \u000A required for a full-blown <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, although in certain cases the \u000A fast loading of books and treatment of hard errors discussed above may be \u000A useful during development.</p> \u000A \u000A <p>Raw mode is also useful for those who want to build extensions of ACL2. \u000A For example, the following form can be put into a certifiable book to load an \u000A arbitrary Common Lisp source or compiled file.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____DEFTTAG\">defttag</see> my-application)\u000A       (<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t)\u000A               (load &quot;some-file&quot;)))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____INCLUDE-RAW\">include-raw</see> and <see topic=\"ACL2____WITH-RAW-MODE\">with-raw-mode</see>.  See <see topic=\"ACL2____DEFTTAG\">defttag</see>, \u000A and see <see topic=\"ACL2____PROGN_12\">progn!</see>.</p> \u000A \u000A <p>Below are several disadvantages to raw mode.  These should discourage users \u000A from using it for general code development, as <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode is \u000A generally preferable.</p> \u000A \u000A <ul> \u000A <li>Forms are in essence executed in raw Lisp.  Hence: \u000A     <ul> \u000A     <li>Syntax checking is turned off; and</li> \u000A     <li>Guard checking is completely disabled.</li> \u000A     </ul></li> \u000A <li>Table events, including <tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, are ignored, as are many \u000A     other <tt><see topic=\"ACL2____EVENTS\">events</see></tt>, including <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> and <tt><see topic=\"ACL2____COMP\">comp</see></tt>.</li> \u000A <li>Soundness claims are weakened for any ACL2 session in which raw \u000A     mode was ever entered; see <see topic=\"ACL2____DEFTTAG\">defttag</see>.</li> \u000A <li>The normal undoing mechanism (see <see topic=\"ACL2____UBT\">ubt</see>) is not supported.</li> \u000A <li>Unexpected behavior may occur when you return from raw-mode. \u000A    For example, if you redefine a :logic mode function whose guards \u000A    have not been verified, you will not see the change inside the \u000A    ACL2 loop because there, the raw Common Lisp definition is only \u000A    executed after guards have been verified; see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see> \u000A    and see <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see>.</li> \u000A </ul> \u000A \u000A <p>We conclude with some details.</p> \u000A \u000A <p><i>Printing results</i>.  The rules for printing results are unchanged for \u000A raw mode, with one exception.  If the value to be printed would contain any \u000A Lisp object that is not a legal ACL2 object, then the <v>print</v> routine is \u000A used from the host Lisp, rather than the usual ACL2 printing routine.  The \u000A following example illustrates the printing used when an illegal ACL2 object \u000A needs to be printed.  Notice how that ``command conventions'' are observed \u000A (see <see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see>); the ``<v>[Note</v>'' occurs one space over in \u000A the second example, and no result is printed in the third example.</p> \u000A \u000A <code>ACL2 P&gt;(find-package &quot;ACL2&quot;)\u000A[Note:  Printing non-ACL2 result.]\u000A#&lt;The ACL2 package&gt;\u000AACL2 P&gt;(<see topic=\"ACL2____MV\">mv</see> nil (find-package &quot;ACL2&quot;) state)\u000A [Note:  Printing non-ACL2 result.]\u000A#&lt;The ACL2 package&gt;\u000AACL2 P&gt;(<see topic=\"ACL2____MV\">mv</see> t (find-package &quot;ACL2&quot;) state)\u000AACL2 P&gt;(<see topic=\"ACL2____MV\">mv</see> 3 (find-package &quot;ACL2&quot;))\u000A[Note:  Printing non-ACL2 result.]\u000A(3 #&lt;The ACL2 package&gt;)\u000AACL2 P&gt;</code> \u000A \u000A <p>If you have trouble with large structures being printed out, you might want \u000A to execute appropriate Common Lisp forms in raw mode, for example, <v>(setq\u000A *print-length* 5)</v> and <v>(setq *print-level* 5)</v>.</p> \u000A \u000A <p><i>Include-book</i>.  The <see topic=\"ACL2____EVENTS\">events</see> <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>, \u000A <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>, <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt>, and <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt> have been designed to work with raw mode.  However, \u000A if you enter raw mode and then evaluate such forms, then the effects of these \u000A forms will disappear when you exit raw mode, in which case you can expect to \u000A see a suitable warning.  Regarding <i>include-book</i> itself: it should work \u000A in raw mode as you might expect, at least if a compiled file or expansion file \u000A was created when the book was certified; see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>.</p> \u000A \u000A <p><i>Packages</i>.  Raw mode disallows the use of <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>.  If you \u000A want to create a new package, first exit raw mode with <v>:set-raw-mode nil</v>; \u000A you can subsequently re-enter raw mode with <v>:set-raw-mode t</v> if you \u000A wish.</p>"],

 "ACL2____O-P":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Using the nonnegative integers and lists we can represent the \u000A ordinals up to <v>epsilon-0</v>. The ordinal representation used in ACL2 has \u000A changed as of Version_2.8 from that of Nqthm-1992, courtesy of Pete Manolios \u000A and Daron Vroon; additional discussion may be found in ``Ordinal Arithmetic in \u000A ACL2'', <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/'>proceedings of \u000A ACL2 Workshop 2003</a>.  Previously, ACL2's notion of ordinal was very similar \u000A to the development given in ``New Version of the Consistency Proof for \u000A Elementary Number Theory'' in The Collected Papers of Gerhard Gentzen, \u000A ed. M.E. Szabo, North-Holland Publishing Company, Amsterdam, 1969, pp \u000A 132-213.</p> \u000A \u000A <p>The following essay is intended to provide intuition about ordinals.  The \u000A truth, of course, lies simply in the ACL2 definitions of <v>o-p</v> and <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>.</p> \u000A \u000A <p>Very intuitively, think of each non-zero natural number as by being denoted \u000A by a series of the appropriate number of strokes, i.e.,</p> \u000A \u000A <code>0             0\u000A1             |\u000A2             ||\u000A3             |||\u000A4             ||||\u000A...           ...</code> \u000A \u000A <p>Then ``<v>omega</v>,'' here written as <v>w</v>, is the ordinal that might be \u000A written as</p> \u000A \u000A <code>w             |||||...,</code> \u000A \u000A <p>i.e., an infinite number of strokes.  Addition here is just concatenation. \u000A Observe that adding one to the front of <v>w</v> in the picture above produces \u000A <v>w</v> again, which gives rise to a standard definition of <v>w</v>: <v>w</v> is \u000A the least ordinal such that adding another stroke at the beginning does not \u000A change the ordinal.</p> \u000A \u000A <p>We denote by <v>w+w</v> or <v>w*2</v> the ``<v>doubly infinite</v>'' sequence \u000A that we might write as follows.</p> \u000A \u000A <code>w*2           |||||... |||||...</code> \u000A \u000A <p>One way to think of <v>w*2</v> is that it is obtained by replacing each \u000A stroke in <v>2</v> <v>(||)</v> by <v>w</v>.  Thus, one can imagine <v>w*3</v>, \u000A <v>w*4</v>, etc., which leads ultimately to the idea of ``<v>w*w</v>,'' the \u000A ordinal obtained by replacing each stroke in <v>w</v> by <v>w</v>.  This is also \u000A written as ``<v>omega</v> squared'' or <v>w^2</v>, or:</p> \u000A \u000A <code> 2\u000Aw             |||||... |||||... |||||... |||||... |||||... ...</code> \u000A \u000A <p>We can analogously construct <v>w^3</v> by replacing each stroke in <v>w</v> by \u000A <v>w^2</v> (which, it turns out, is the same as replacing each stroke in \u000A <v>w^2</v> by <v>w</v>).  That is, we can construct <v>w^3</v> as <v>w</v> copies of \u000A <v>w^2</v>,</p> \u000A \u000A <code> 3              2       2       2       2\u000Aw              w  ...  w  ...  w  ...  w ... ...</code> \u000A \u000A <p>Then we can construct <v>w^4</v> as <v>w</v> copies of <v>w^3</v>, <v>w^5</v> as \u000A <v>w</v> copies of <v>w^4</v>, etc., ultimately suggesting <v>w^w</v>.  We can then \u000A stack <v>omega</v>s, i.e., <v>(w^w)^w</v> etc.  Consider the ``limit'' of all of \u000A those stacks, which we might display as follows.</p> \u000A \u000A <code>       .\u000A      .\u000A     .\u000A    w\u000A   w\u000A  w\u000A w\u000Aw</code> \u000A \u000A <p>That is epsilon-0.</p> \u000A \u000A <p>Below we begin listing some ordinals up to <v>epsilon-0</v>; the reader can \u000A fill in the gaps at his or her leisure.  We show in the left column the \u000A conventional notation, using <v>w</v> as ``<v>omega</v>,'' and in the right column \u000A the ACL2 object representing the corresponding ordinal.</p> \u000A \u000A <code>ordinal            ACL2 representation\u000A\u000A0                  0\u000A1                  1\u000A2                  2\u000A3                  3\u000A...                ...\u000Aw                 '((1 . 1) . 0)\u000Aw+1               '((1 . 1) . 1)\u000Aw+2               '((1 . 1) . 2)\u000A...                ...\u000Aw*2               '((1 . 2) . 0)\u000A(w*2)+1           '((1 . 2) . 1)\u000A...                ...\u000Aw*3               '((1 . 3) . 0)\u000A(w*3)+1           '((1 . 3) . 1)\u000A...                ...\u000A\u000A 2\u000Aw                 '((2 . 1) . 0)\u000A...                ...\u000A\u000A 2\u000Aw +w*4+3          '((2 . 1) (1 . 4) . 3)\u000A...                ...\u000A\u000A 3\u000Aw                 '((3 . 1) . 0)\u000A...                ...\u000A\u000A w\u000Aw                 '((((1 . 1) . 0) . 1) . 0)\u000A...                ...\u000A\u000A w  99\u000Aw +w  +w4+3       '((((1 . 1) . 0) . 1) (99 . 1) (1 . 4) . 3)\u000A...                ...\u000A\u000A  2\u000A w\u000Aw                 '((((2 . 1) . 0) . 1) . 0)\u000A\u000A...                ...\u000A\u000A  w\u000A w\u000Aw                 '((((((1 . 1) . 0) . 1) . 0) . 1) . 0)\u000A...               ...</code> \u000A \u000A <p>Observe that the sequence of <v>o-p</v>s starts with the natural numbers \u000A (which are recognized by <tt><see topic=\"ACL2____NATP\">natp</see></tt>). This is convenient because it means \u000A that if a term, such as a measure expression for justifying a recursive \u000A function (see <see topic=\"ACL2____O_C3\">o&lt;</see>) must produce an <v>o-p</v>, it suffices for it to \u000A produce a natural number.</p> \u000A \u000A <p>The ordinals listed above are listed in ascending order.  This is the \u000A ordering tested by <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>.</p> \u000A \u000A <p>The ``<v>epsilon-0</v> ordinals'' of ACL2 are recognized by the recursively \u000A defined function <v>o-p</v>.  The base case of the recursion tells us that \u000A natural numbers are <v>epsilon-0</v> ordinals.  Otherwise, an <v>epsilon-0</v> \u000A ordinal is a list of <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pairs whose final <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is a natural \u000A number, <v>((a1 . x1) (a2 . x2) ... (an . xn) . p)</v>.  This corresponds to the \u000A ordinal <v>(w^a1)x1 + (w^a2)x2 + ... + (w^an)xn + p</v>.  Each <v>ai</v> is an \u000A ordinal in the ACL2 representation that is not equal to 0.  The sequence of \u000A the <v>ai</v>'s is strictly decreasing (as defined by <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>). Each <v>xi</v> \u000A is a positive integer (as recognized by <tt><see topic=\"ACL2____POSP\">posp</see></tt>).</p> \u000A \u000A <p>Note that infinite ordinals should generally be created using the ordinal \u000A constructor, <tt><see topic=\"ACL2____MAKE-ORD\">make-ord</see></tt>, rather than <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>. The functions <tt><see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see></tt>, <tt><see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see></tt>, and <tt><see topic=\"ACL2____O-RST\">o-rst</see></tt> are ordinals \u000A destructors.  Finally, the function <tt><see topic=\"ACL2____O-FINP\">o-finp</see></tt> and the macro <tt><see topic=\"ACL2____O-INFP\">o-infp</see></tt> tell whether an ordinal is finite or infinite, respectively.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> compares two <v>epsilon-0</v> ordinals, <v>x</v> and \u000A <v>y</v>.  If both are integers, <v>(<see topic=\"ACL2____O_C3\">o&lt;</see> x y)</v> is just <v>x&lt;y</v>.  If one is an \u000A integer and the other is a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, the integer is the smaller. \u000A Otherwise, <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> recursively compares the <tt><see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see></tt>s of the \u000A ordinals to determine which is smaller.  If they are the same, the <tt><see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see></tt>s of the ordinals are compared.  If they are equal, the <tt><see topic=\"ACL2____O-RST\">o-rst</see></tt>s of the ordinals are recursively compared.</p> \u000A \u000A <p>Fundamental to ACL2 is the fact that <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> is well-founded on \u000A <v>epsilon-0</v> ordinals.  That is, there is no ``infinitely descending chain'' \u000A of such ordinals.  See <see topic=\"ACL2____PROOF-OF-WELL-FOUNDEDNESS\">proof-of-well-foundedness</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>o-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o-p (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x)\u000A           (<see topic=\"ACL2____NATP\">natp</see> x)\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                (<see topic=\"ACL2____O-P\">o-p</see> (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x))\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> 0 (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x)))\u000A                (<see topic=\"ACL2____POSP\">posp</see> (<see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see> x))\u000A                (<see topic=\"ACL2____O-P\">o-p</see> (<see topic=\"ACL2____O-RST\">o-rst</see> x))\u000A                (<see topic=\"ACL2____O_C3\">o&lt;</see> (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> (<see topic=\"ACL2____O-RST\">o-rst</see> x))\u000A                    (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x)))))</code>"],

 "ACL2____MUTUAL-RECURSION":[["Events","Programming","Defun"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> evenlp (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (oddlp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) t))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> oddlp (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (evenlp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) nil)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see> def1 ... defn)</code> \u000A \u000A <p>where each <v>defi</v> is a call of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFUND\">defund</see></tt>, <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt>, or <v>defund-nx</v>.</p> \u000A \u000A <p>When mutually recursive functions are introduced it is necessary to do the \u000A termination analysis on the entire clique of definitions.  Each <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A form specifies its own measure, either with the <v>:measure</v> keyword \u000A <v>xarg</v> (see <see topic=\"ACL2____XARGS\">xargs</see>) or by default to <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt>.  When a \u000A function in the clique calls a function in the clique, the measure of the \u000A callee's actuals must be smaller than the measure of the caller's formals \u000A &mdash; just as in the case of a simply recursive function.  But with mutual \u000A recursion, the callee's actuals are measured as specified by the callee's \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> while the caller's formals are measured as specified by the \u000A caller's <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  These two measures may be different but must be \u000A comparable in the sense that <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> decreases through calls.</p> \u000A \u000A <p>If you want to specify <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> or <v>:guard-hints</v> (see <see topic=\"ACL2____XARGS\">xargs</see>), you can put them in the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration of any of the \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> forms, as the <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> from each form will be \u000A appended together, as will the <tt><see topic=\"ACL2____GUARD-HINTS\">guard-hints</see></tt> from each form.</p> \u000A \u000A <p>You may find it helpful to use a lexicographic order, the idea being to \u000A have a measure that returns a list of two arguments, where the first takes \u000A priority over the second.  Here is an example.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;ordinals/lexicographic-ordering&quot; :dir :system)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> l&lt;) ; will be treated as LOCAL\u000A\u000A (<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x) 1)))\u000A    (bar x))\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (y)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> y) 0)))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> y) y (foo (<see topic=\"COMMON-LISP____1-\">1-</see> y))))))</code> \u000A \u000A <p>The <tt><see topic=\"ACL2____GUARD\">guard</see></tt> analysis must also be done for all of the functions at \u000A the same time.  If any one of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s specifies the <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> <v>xarg</v> to be <v>nil</v>, then <see topic=\"ACL2____GUARD\">guard</see> verification is \u000A omitted for all of the functions.  Similarly, if any one of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s \u000A specifies the <v>:non-executable</v> <v>xarg</v> to be <v>t</v>, or if any of the \u000A definitions uses <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt> or <v>defund-nx</v>, then every one of the \u000A definitions will be treated as though it specifies a <v>:non-executable</v> \u000A <v>xarg</v> of <v>t</v>.</p> \u000A \u000A <p>Technical Note: Each <v>defi</v> above must be a call of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, \u000A <tt><see topic=\"ACL2____DEFUND\">defund</see></tt>, <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt>, or <v>defund-nx</v>.  In particular, it is not \u000A permitted for a <v>defi</v> to be an arbitrary form that macroexpands into a \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form.  This is because <v>mutual-recursion</v> is itself a macro, \u000A and since macroexpansion occurs from the outside in, at the time \u000A <v>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see> def1 ... defk)</v> is expanded the <v>defi</v> have not yet \u000A been macroexpanded.</p> \u000A \u000A <p>Suppose you have defined your own <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>-like macro and wish to use \u000A it in a <v>mutual-recursion</v> expression.  Well, you can't.  (!)  But you can \u000A define your own version of <v>mutual-recursion</v> that allows your <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>-like form.  Here is an example.  Suppose you define</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-defun (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args) (my-defun-fn args))</code> \u000A \u000A <p>where <v>my-defun-fn</v> takes the arguments of the <v>my-defun</v> form and \u000A produces from them a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form.  As noted above, you are not allowed \u000A to write <v>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see> (my-defun ...) ...)</v>.  But you can define the \u000A macro <v>my-mutual-recursion</v> so that</p> \u000A \u000A <code>(my-mutual-recursion (my-defun ...) ... (my-defun ...))</code> \u000A \u000A <p>expands into <v>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> ...) ... (<see topic=\"COMMON-LISP____DEFUN\">defun</see> ...))</v> by \u000A applying <v>my-defun-fn</v> to each of the arguments of \u000A <v>my-mutual-recursion</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-mutual-recursion-fn (lst)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> lst)))\u000A\u000A; Each element of lst must be a consp (whose car, we assume, is always\u000A; MY-DEFUN).  We apply my-defun-fn to the arguments of each element and\u000A; collect the resulting list of DEFUNs.\u000A\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) nil)\u000A        (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (my-defun-fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst)))\u000A                 (my-mutual-recursion-fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-mutual-recursion (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> lst)\u000A\u000A; Each element of lst must be a consp (whose car, we assume, is always\u000A; MY-DEFUN).  We obtain the DEFUN corresponding to each and list them\u000A; all inside a MUTUAL-RECURSION form.\u000A\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> lst)))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 'mutual-recursion (my-mutual-recursion-fn lst))).</code>"],

 "ACL2____MEMOIZE":[["Events","Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to an experimental \u000A extension of ACL2 under development by Bob Boyer and Warren Hunt.  See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see> for a general discussion of memoization and the related \u000A features of hash consing and applicative hash tables.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo)                      ; remember the values of calls\u000A                                    ;   of foo\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :condition t)         ; same as above\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :condition '(test x)) ; memoize for args satisfying\u000A                                    ;   the given condition\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :condition-fn 'test)  ; memoize for args satisfying\u000A                                    ;   a call of the given function\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :recursive nil)       ; don't memoize recursive calls\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :aokp t)              ; attachments OK for stored results\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'foo :ideal-okp t)         ; memoize even if foo is in :logic mode\u000A                                    ;   but has not been guard-verified\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> fn                         ; memoizes fn and returns fn\u000A         :condition    condition    ; optional (<see topic=\"ACL2____DEFAULT\">default</see> t)\u000A         :condition-fn condition-fn ; optional\u000A         :hints        hints        ; optional, for verifying the\u000A                                    ;   guards of condition-fn\u000A         :otf-flg      otf-flg      ; optional, for verifying the\u000A                                    ;   guards of condition-fn\u000A         :recursive    t/nil        ; optional (<see topic=\"ACL2____DEFAULT\">default</see> t)\u000A         :commutative  t/lemma-name ; optional (<see topic=\"ACL2____DEFAULT\">default</see> nil)\u000A         :forget       t/nil        ; optional (<see topic=\"ACL2____DEFAULT\">default</see> nil)\u000A         :memo-table-init-size size ; optional (<see topic=\"ACL2____DEFAULT\">default</see> *mht-default-size*)\u000A         :aokp         t/nil        ; optional (<see topic=\"ACL2____DEFAULT\">default</see> nil)\u000A         :ideal-okp    t/:warn/nil  ; optional (<see topic=\"ACL2____DEFAULT\">default</see> nil)\u000A         :verbose      t/nil        ; optional (<see topic=\"ACL2____DEFAULT\">default</see> t)\u000A         )</code> \u000A \u000A <p>where <v>fn</v> evaluates to a user-defined function symbol; <v>condition</v> \u000A is either <v>t</v> (the default), <v>'t</v>, <v>nil</v>, or <v>'nil</v>, or else \u000A evaluates to an expression whose free variables are among the formal \u000A parameters of <v>fn</v>; and <v>condition-fn</v> is either <v>nil</v> (the default) \u000A or else evaluates to a legal function symbol.  Further restrictions and \u000A options are discussed below.  Note that all arguments are evaluated (but for \u000A the special handling of value <v>t</v> for <v>:commutative</v>, the argument must \u000A literally be <v>t</v>; see below).</p> \u000A \u000A <p>Generally <v>fn</v> must evaluate to a defined function symbol.  However, \u000A this value can be the name of a macro that is associated with such a function \u000A symbol; see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.  That associated function symbol is \u000A the one called ``memoized'' in the discussion below, but we make no more \u000A mention of this subtlety.</p> \u000A \u000A <p>In the most common case, <v>memoize</v> takes a single argument, which \u000A evaluates to a function symbol.  We call this function symbol the ``memoized \u000A function'' because ``memos'' are saved and re-used, in the following sense. \u000A When a call of the memoized function is evaluated, the result is ``memoized'' \u000A by associating the call's arguments with that result, in a suitable table. \u000A But first an attempt is made to avoid such evaluation, by doing a lookup in \u000A that table on the given arguments for the result, as stored for a previous \u000A call on those arguments.  If such a result is found, then it is returned \u000A without further computation.  This paragraph also applies if <v>:condition</v> \u000A is supplied but is <v>t</v> or <v>'t</v>.</p> \u000A \u000A <p>If keyword argument <v>:condition-fn</v> is supplied, but <v>:condition</v> is \u000A not, then the result of evaluating <v>:condition-fn</v> must be a defined \u000A function symbol whose formal parameter list and <tt><see topic=\"ACL2____GUARD\">guard</see></tt> are the same as \u000A for the function being memoized.  If <v>fn</v> is in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, \u000A then <see topic=\"ACL2____GUARD\">guard</see>s must have been verified for <v>:condition-fn</v>.  Such a \u000A ``condition function'' will be run whenever the memoized function is called, \u000A on the same parameters, and the lookup or table store described above are only \u000A performed if the result from the condition function call is non-<v>nil</v>.</p> \u000A \u000A <p>Suppose however that <v>:condition</v> is supplied.  If the value supplied is \u000A <v>t</v> or <v>'t</v>, then the lookup and table store described above are always \u000A done.  If the value is <v>nil</v> or <v>'nil</v>, then this lookup and table store \u000A are never done, although statistics may be gathered; see <see topic=\"ACL2____PROFILE\">profile</see>.  Now \u000A consider other values for <v>:condition</v>.  An attempt will be made to define \u000A a condition function whose <see topic=\"ACL2____GUARD\">guard</see> and formal parameters list are the \u000A same as those of the memoized function, and whose body is the result, <v>r</v>, \u000A of evaluating the given <v>condition</v>.  The name of that condition function \u000A is the result of evaluating <v>:condition-fn</v> if supplied, else is the result \u000A of concatenating the string <v>&quot;-MEMOIZE-CONDITION&quot;</v> to the end of the name \u000A of the memoized function.  The condition function will be defined with <see topic=\"ACL2____GUARD\">guard</see> verification turned off, but that definition will be followed \u000A immediately by a <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event; and this is where the optional \u000A <v>:hints</v> and <v>:otf-flg</v> are attached.  At evaluation time the condition \u000A function is used as described in the preceding paragraph; so in effect, the \u000A condition (<v>r</v>, above) is evaluated, with its variables bound to the \u000A corresponding actuals of the memoized function call, and the memoized function \u000A attempts a lookup or table store if and only if the result of that evaluation \u000A is non-<v>nil</v>.</p> \u000A \u000A <p>Note that <v>fn</v> can be either a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function or a \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function.  However, only the corresponding raw Lisp \u000A function is actually memoized, so <see topic=\"ACL2____GUARD\">guard</see> violations can defeat \u000A memoization, and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions without their <see topic=\"ACL2____GUARD\">guard</see>s verified will only be memoized when called by <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode functions.  (See <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see> for more information about \u000A guards and evaluation in ACL2.)  If <v>fn</v> is a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A function and <v>:condition</v> is supplied and not <v>t</v> or <v>nil</v>, then the \u000A condition must be a <see topic=\"ACL2____GUARD\">guard</see>-verified function.</p> \u000A \u000A <p>Calls of this macro generate events of the form <v>(<see topic=\"ACL2____TABLE\">table</see> memoize-table fn\u000A ((:condition-fn fn) ...))</v>.  When successful, the returned value is of the \u000A form <v>(<see topic=\"ACL2____MV\">mv</see> nil function-symbol state)</v>.</p> \u000A \u000A <p>Suppose that a function is already memoized.  Then it is illegal to memoize \u000A that function.  Moreover, if the function was memoized with an associated \u000A condition (i.e., was memoized with keyword <v>:condition</v> or \u000A <v>:condition-fn</v> having value other than <v>t</v> or <v>nil</v>), then it is \u000A also illegal to convert the function from <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> to \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode (see <see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>).  To turn off \u000A memoization, see <see topic=\"ACL2____UNMEMOIZE\">unmemoize</see>.</p> \u000A \u000A <p><v>Memoize</v> is illegal for a function if its arguments include <tt><see topic=\"ACL2____STATE\">state</see></tt> or if it returns any <see topic=\"ACL2____STOBJ\">stobj</see>s.  A stobj can be an input of a \u000A memoized function, but in that case, the memoization table for that stobj will \u000A be cleared every time that stobj is updated.</p> \u000A \u000A <p><v>Memoize</v> never allows attachments to be used (see <see topic=\"ACL2____DEFATTACH\">defattach</see>). \u000A If an attachment is used during evaluation, then the evaluation result will \u000A not be stored.</p> \u000A \u000A <p>We conclude with by documenting keyword parameters not discussed above.</p> \u000A \u000A <p>Keyword parameter <v>:recursive</v> is <v>t</v> by default, which means that \u000A recursive calls of <v>fn</v> will be memoized just as ``top-level'' calls of \u000A <v>fn</v>.  When <v>:recursive</v> is instead set to <v>nil</v>, memoization is only \u000A done at the top level.  Using <v>:recursive nil</v> is similar to writing a \u000A wrapper function that just calls <v>fn</v>, and memoizing the wrapper instead of \u000A <v>fn</v>.</p> \u000A \u000A <p>If <v>:trace</v> has a non-<v>nil</v> value, then <v>memoize</v> also traces in a \u000A traditional Lisp style.  If <v>:trace</v> has value <v>notinline</v> or \u000A <v>notinline</v>, then a corresponding declaration is added at the beginning of \u000A the new definition of <v>fn</v>.</p> \u000A \u000A <p>A non-<v>nil</v> value for <v>:commutative</v> can be supplied if <v>fn</v> is a \u000A binary function in <v>:logic</v> mode.  If the <v>memoize</v> event is successful, \u000A then subsequently: whenever each argument to <v>fn</v> is either a rational \u000A number or a <see topic=\"ACL2____HONS\">hons</see>, then when the evaluation of <v>fn</v> on those \u000A arguments is memoized, the evaluation of <v>fn</v> on the swap of those \u000A arguments is, in essence, also memoized.  If <v>:commutative</v> is supplied and \u000A is not <v>nil</v> or <v>t</v>, then it should be the name of a previously-proved \u000A theorem whose formula states the commutativity of <v>fn</v>, i.e., is the \u000A formula <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x y) (fn y x))</v> for a pair <v>{x,y}</v> of distinct \u000A variables.  If <v>:commutative</v> is <v>t</v> &mdash; but not merely an \u000A expression that evaluates to <v>t</v> &mdash; then an attempt to prove such a \u000A lemma will be made on-the-fly.  The name of the lemma is the symbol in the \u000A same package as <v>fn</v>, obtained by adding the suffix <v>&quot;-COMMUTATIVE&quot;</v> \u000A to the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of <v>fn</v>.  If the proof attempt fails, then you \u000A may want first to prove the lemma yourself with appropriate hints and perhaps \u000A supporting lemmas, and then supply the name of that lemma as the value of \u000A <v>:commutative</v>.</p> \u000A \u000A <p>If <v>:commutative</v> is supplied, and a non-commutative condition is \u000A provided by <v>:condition</v> or <v>:condition-fn</v>, then although the results \u000A will be correct, the extra memoization afforded by <v>:commutative</v> is \u000A unspecified.</p> \u000A \u000A <p>If <v>:memo-table-init-size</v> is supplied, then it should be a positive \u000A integer specifying the initial size of an associated hash table.</p> \u000A \u000A <p>Argument <v>:aokp</v> is relevant only when attachments are used; see <see topic=\"ACL2____DEFATTACH\">defattach</see> for background on attachments.  When <v>:aokp</v> is <v>nil</v>, the \u000A default, computed values are not stored when an attachment was used, or even \u000A when an attachment may have been used because a function was called that had \u000A been memoized using <v>:aokp t</v>.  Otherwise, computed values are always \u000A stored, but saved values are not used except when attachments are allowed.  To \u000A summarize:</p> \u000A \u000A <code>aokp=nil (<see topic=\"ACL2____DEFAULT\">default</see>): ``Pure'', i.e., values do not depend on attachments\u000A- Fetch: always legal\u000A- Store: only store resulting value when attachments were not used\u000A\u000Aaokp=t: ``Impure'', i.e., values may depend on attachments\u000A- Fetch: only legal when attachments are allowed (e.g., not during proofs)\u000A- Store: always legal</code> \u000A \u000A <p>If <v>:ideal-okp</v> is supplied and not <v>nil</v>, then it is permitted to \u000A memoize an ``ideal-mode'' function: one in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode whose \u000A <see topic=\"ACL2____GUARD\">guard</see>s have not been verified.  In general, it is ill-advised to \u000A memoize an ideal-mode function, because its calls are typically evaluated ``in \u000A the logic'' without calling a memoized ``raw Lisp'' version of the function. \u000A However, if the function is called by a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function, \u000A evaluation can transfer to raw Lisp before reaching the call of the memoized \u000A function, in which case memoization will take place.  For such situations you \u000A can provide value <v>:warn</v> or <v>t</v> for keyword parameter <v>:ideal-okp</v>. \u000A Both of these values allow memoization of ideal-mode functions, but if \u000A <v>:warn</v> is supplied then a warning will take place.  Note that you may set \u000A the key <v>:memoize-ideal-okp</v> of the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> to value \u000A <v>t</v> or <v>:warn</v> to change the default, but if parameter <v>:ideal-okp</v> \u000A is supplied, the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> value is ignored.</p> \u000A \u000A <p>If <v>:verbose</v> is supplied, it should either be <v>nil</v>, which will \u000A inhibit proof, event, and summary output (see <see topic=\"ACL2____WITH-OUTPUT\">with-output</see>), or else \u000A <v>t</v> (the default), which does not inhibit output.  If the output baffles \u000A you, try</p> \u000A \u000A <code>:trans1 (<see topic=\"ACL2____MEMOIZE\">memoize</see> ...)</code> \u000A \u000A <p>to see the single-step macroexpansion of your <v>memoize</v> call.</p> \u000A \u000A <p>The default for <v>:forget</v> is <v>nil</v>.  If <v>:forget</v> is supplied, and \u000A not <v>nil</v>, then it must be <v>t</v>, which causes all memoization done for a \u000A top-level call of <v>fn</v> to be forgotten when that top-level call \u000A exits.</p>"],

 "ACL2____PORTCULLIS":[["Books-tour"],"ACL2 Sources","ACL2","<p>The certificate (see <see topic=\"ACL2____CERTIFICATE\">certificate</see> for general information) of \u000A a certified file is divided into two parts, a portcullis and a <see topic=\"ACL2____KEEP\">keep</see>. \u000A These names come from castle lore.  The portcullis of a castle is an iron \u000A grate that slides up through the ceiling of the tunnel-like entrance.  The \u000A portcullis of a book ensures that <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> does not start to read \u000A the book until the appropriate context has been created.</p> \u000A \u000A <p>Technically, the portcullis consists of the <see topic=\"ACL2____VERSION\">version</see> number of the \u000A certifying ACL2, a list of <see topic=\"ACL2____COMMAND\">command</see>s used to create the ``certification \u000A <see topic=\"ACL2____WORLD\">world</see>'' and an alist specifying the check sums of all the <see topic=\"ACL2____BOOKS\">books</see> \u000A included in that <see topic=\"ACL2____WORLD\">world</see>.  The portcullis is constructed automatically by \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> from the <see topic=\"ACL2____WORLD\">world</see> in which <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is \u000A called, but that <see topic=\"ACL2____WORLD\">world</see> must have certain properties described below. \u000A After listing the properties we discuss the issues in a more leisurely \u000A manner.</p> \u000A \u000A <p>Each <see topic=\"ACL2____COMMAND\">command</see> in the portcullis must be either a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> form \u000A or an embedded event form (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).</p> \u000A \u000A <p>Consider a book to be certified.  The book is a file containing event \u000A forms.  Suppose the file contains references to such symbols as \u000A <v>my-pkg::fn</v> and <v>acl2-arith::cancel</v>, but that the book itself does not \u000A create the packages.  Then a hard Lisp error would be caused merely by the \u000A attempt to read the expressions in the book.  The corresponding <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s cannot be written into the book itself because the book must be \u000A compilable and Common Lisp compilers differ on the rules concerning the inline \u000A definition of new packages.  The only safe course is to make all <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s occur outside of compiled files.</p> \u000A \u000A <p>More generally, when a book is certified it is certified within some \u000A logical <see topic=\"ACL2____WORLD\">world</see>.  That ``certification <see topic=\"ACL2____WORLD\">world</see>'' contains not only \u000A the necessary <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s but also, perhaps, function and constant \u000A definitions and maybe even references to other <see topic=\"ACL2____BOOKS\">books</see>.  When <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> creates the <see topic=\"ACL2____CERTIFICATE\">certificate</see> for a file it recovers from the \u000A certification <see topic=\"ACL2____WORLD\">world</see> the <see topic=\"ACL2____COMMAND\">command</see>s used to create that <see topic=\"ACL2____WORLD\">world</see> from the initial ACL2 <see topic=\"ACL2____WORLD\">world</see>.  Those <see topic=\"ACL2____COMMAND\">command</see>s become part \u000A of the portcullis for the certified book.  In addition, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A records in the portcullis the check sums (see <see topic=\"ACL2____CHECK-SUM\">check-sum</see>) of all the \u000A <see topic=\"ACL2____BOOKS\">books</see> included in the certification <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> presumes that it is impossible even to read the \u000A contents of a certified book unless the portcullis can be ``raised.'' To raise \u000A the portcullis we must be able to execute (possibly redundantly, but certainly \u000A without error), all of the <see topic=\"ACL2____COMMAND\">command</see>s in the portcullis and then verify \u000A that the <see topic=\"ACL2____BOOKS\">books</see> thus included were identical to those used to build the \u000A certification <see topic=\"ACL2____WORLD\">world</see> (up to check sum).  This raising of the portcullis \u000A must be done delicately since <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s are present: we cannot even read \u000A a <see topic=\"ACL2____COMMAND\">command</see> in the portcullis until we have successfully executed the \u000A previous ones, since packages are being defined.</p> \u000A \u000A <p>Clearly, a book is most useful if it is certified in the most elementary \u000A extension possible of the initial logic.  If, for example, your certification \u000A <see topic=\"ACL2____WORLD\">world</see> happens to contain a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> for <v>&quot;MY-PKG&quot;</v> and the \u000A function <v>foo</v>, then those definitions become part of the portcullis for \u000A the book.  Every time the book is included, those names will be defined and \u000A will have to be either new or redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).  But if \u000A those names were not necessary to the certification of the book, their \u000A presence would unnecessarily restrict the utility of the book.</p> \u000A \u000A <p>See <see topic=\"ACL2____KEEP\">keep</see> to continue the guided tour of <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____HIDE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p><v>Hide</v> is actually the <see topic=\"COMMON-LISP____IDENTITY\">identity</see> function: <v>(<see topic=\"ACL2____HIDE\">hide</see> x) =\u000A x</v> for all <v>x</v>.  However, terms of the form <v>(<see topic=\"ACL2____HIDE\">hide</see> x)</v> are ignored by \u000A the ACL2 rewriter, except when explicit <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see> are given \u000A for such terms (see <see topic=\"ACL2____HINTS\">hints</see>) or when rewrite rules explicitly about \u000A <v>hide</v> are available.  An <v>:expand</v> hint that removes all calls of \u000A <v>hide</v> is:</p> \u000A \u000A <code>:expand ((:free (x) (<see topic=\"ACL2____HIDE\">hide</see> x)))</code> \u000A \u000A <p>The above hint can be particularly useful when ACL2's equality heuristics \u000A apply <v>hide</v> to an equality after substituting it into the rest of the \u000A goal, if that goal (or a subgoal of it) fails to be proved.</p> \u000A \u000A <p><v>Hide</v> terms are also ignored by the induction heuristics.</p> \u000A \u000A <p>Sometimes the ACL2 simplifier inserts <v>hide</v> terms into a proof attempt \u000A out of the blue, as it were.  Why and what can you do about it?  Suppose you \u000A have a constrained function, say <v>constrained-fn</v>, and you define another \u000A function, say <v>another-fn</v>, in terms of it, as in:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> another-fn (x y z)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (big-hairy-test x y z)\u000A      (constrained-fn x y z)\u000A      t))</code> \u000A \u000A <p>Suppose the term <v>(another-fn 'a 'b 'c)</v> arises in a proof.  Since the \u000A arguments are all constants, ACL2 will try to reduce such a term to a constant \u000A by executing the definition of <v>another-fn</v>.  However, after a possibly \u000A extensive computation (because of <v>big-hairy-test</v>) the execution fails \u000A because of the unevaluable call of <v>constrained-fn</v>.  To avoid subsequent \u000A attempts to evaluate the term, ACL2 embeds it in a <v>hide</v> expression, i.e., \u000A rewrites it to <v>(<see topic=\"ACL2____HIDE\">hide</see> (another-fn 'a 'b 'c))</v>.</p> \u000A \u000A <p>You might think this rarely occurs since all the arguments of \u000A <v>another-fn</v> must be constants.  You would be right except for one special \u000A case: if <v>another-fn</v> takes no arguments, i.e., is a constant function, \u000A then every call of it fits this case.  Thus, if you define a function of no \u000A arguments in terms of a constrained function, you will often see \u000A <v>(another-fn)</v> rewrite to <v>(<see topic=\"ACL2____HIDE\">hide</see> (another-fn))</v>.</p> \u000A \u000A <p>We do not hide the term if the executable counterpart of the function is \u000A disabled -- because we do not try to evaluate it in the first place.  Thus, to \u000A prevent the insertion of a <v>hide</v> term into the proof attempt, you can \u000A globally disable the executable counterpart of the offending defined function, \u000A e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart another-fn))).</code> \u000A \u000A <p>It is conceivable that you cannot afford to do this: perhaps some calls of \u000A the offending function must be computed while others cannot be.  One way to \u000A handle this situation is to leave the executable counterpart enabled, so that \u000A <v>hide</v> terms are introduced on the calls that cannot be computed, but prove \u000A explicit :<tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules for each of those <v>hide</v> terms.  For \u000A example, suppose that in the proof of some theorem, thm, it is necessary to \u000A leave the executable counterpart of <v>another-fn</v> enabled but that the call \u000A <v>(another-fn 1 2 3)</v> arises in the proof and cannot be computed.  Thus the \u000A proof attempt will introduce the term <v>(<see topic=\"ACL2____HIDE\">hide</see> (another-fn 1 2 3))</v>.  Suppose \u000A that you can show that <v>(another-fn 1 2 3)</v> is <v>(constrained-fn 1 2 3)</v> \u000A and that such a step is necessary to the proof.  Unfortunately, proving the \u000A rewrite rule</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> thm-helper\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (another-fn 1 2 3) (constrained-fn 1 2 3)))</code> \u000A \u000A <p>would not help the proof of thm because the target term is hidden inside \u000A the <v>hide</v>.  However,</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> thm-helper\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____HIDE\">hide</see> (another-fn 1 2 3)) (constrained-fn 1 2 3)))</code> \u000A \u000A <p>would be applied in the proof of thm and is the rule you should prove.</p> \u000A \u000A <p>Now to prove <v>thm-helper</v> you need to use the two ``tricks'' which have \u000A already been discussed.  First, to eliminate the <v>hide</v> term in the proof \u000A of <v>thm-helper</v> you should include the hint <v>:expand</v> <v>(<see topic=\"ACL2____HIDE\">hide</see>\u000A (another-fn 1 2 3))</v>.  Second, to prevent the <v>hide</v> term from being \u000A reintroduced when the system tries and fails to evaluate <v>(another-fn 1 2\u000A 3)</v> you should include the hint <v>:in-theory</v> <v>(<see topic=\"ACL2____DISABLE\">disable</see>\u000A (:executable-counterpart another-fn))</v>.  Thus, <v>thm-helper</v> will actually \u000A be:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> thm-helper\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____HIDE\">hide</see> (another-fn 1 2 3)) (constrained-fn 1 2 3))\u000A  :hints\u000A  ((&quot;Goal&quot; :expand (<see topic=\"ACL2____HIDE\">hide</see> (another-fn 1 2 3))\u000A           :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart another-fn)))))</code> \u000A \u000A <p>See <see topic=\"ACL2____EVISCERATE-HIDE-TERMS\">eviscerate-hide-terms</see> for how to affect the printing of \u000A <v>hide</v> terms.</p> \u000A \u000A <p><b>Function: </b><srclink>hide</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hide (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "ACL2____CONSTRAINT":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Suppose that a given theorem, <v>thm</v>, is to be functionally \u000A instantiated using a given functional substitution, <v>alist</v>.  (See <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>, or for an example, see <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE\">functional-instantiation-example</see>.)  What is the set of proof obligations \u000A generated?  It is the set obtained by applying <v>alist</v> to all terms, \u000A <v>tm</v>, such that (a) <v>tm</v> mentions some function symbol in the domain of \u000A <v>alist</v>, and (b) either (i) <v>tm</v> arises from the ``constraint'' on a \u000A function symbol ancestral in <v>thm</v> or in some <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> or (ii) \u000A <v>tm</v> is the body of a <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>.  Here, a function symbol is \u000A ``ancestral'' in <v>thm</v> if either it occurs in <v>thm</v>, or it occurs in the \u000A definition of some function symbol that occurs in <v>thm</v>, and so on.</p> \u000A \u000A <p>The remainder of this note explains what we mean by ``constraint'' in the \u000A words above.</p> \u000A \u000A <p>In a certain sense, function symbols are introduced in essentially two \u000A ways.  The most common way is to use <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (or when there is mutual \u000A recursion, <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>).  There is also a \u000A mechanism for introducing ``witness functions''; see <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  The \u000A documentation for these <see topic=\"ACL2____EVENTS\">events</see> describes the axioms they introduce, \u000A which we will call here their ``definitional axioms.''  These definitional \u000A axioms are generally the constraints on the function symbols that these axioms \u000A introduce.</p> \u000A \u000A <p>However, when a function symbol is introduced in the scope of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, its constraints may differ from the definitional axioms \u000A introduced for it.  For example, suppose that a function's definition is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>; that is, suppose the function is \u000A introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see> of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Then its \u000A constraints include, at the least, those non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> theorems and \u000A definitions in the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> that mention the function symbol.</p> \u000A \u000A <p>Actually, it will follow from the discussion below that if the <see topic=\"ACL2____SIGNATURE\">signature</see> is empty for an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, then the constraint on each of \u000A its new function symbols is exactly the definitional axiom introduced for it. \u000A Intuitively, we view such <v>encapsulates</v> just as we view <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  But the general case, where the <see topic=\"ACL2____SIGNATURE\">signature</see> \u000A is not empty, is more complicated.</p> \u000A \u000A <p>In the discussion that follows we describe in detail exactly which \u000A constraints are associated with which function symbols that are introduced in \u000A the scope of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event.  In order to simplify the \u000A exposition we make two cuts at it.  In the first cut we present an \u000A over-simplified explanation that nevertheless captures the main ideas.  In the \u000A second cut we complete our explanation by explaining how we view certain <see topic=\"ACL2____EVENTS\">events</see> as being ``lifted'' out of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, resulting in a \u000A possibly smaller <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, which becomes the target of the \u000A algorithm described in the first cut.</p> \u000A \u000A <p>At the end of this note we present an example showing why a more naive \u000A approach is unsound.</p> \u000A \u000A <p>Finally, before we start our ``first cut,'' we note that any information \u000A you want ``exported'' outside an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event must be there as an \u000A explicit definition or theorem.  For example, even if a function <v>foo</v> has \u000A output type <v>(<see topic=\"ACL2____MV\">mv</see> t t)</v> in its <see topic=\"ACL2____SIGNATURE\">signature</see>, the system will not know \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (foo x))</v> merely on account of this information.  Thus, if you \u000A are using functions like <v>foo</v> (constrained <tt><see topic=\"ACL2____MV\">mv</see></tt> functions), then you \u000A may find it useful to prove (inside the <v>encapsulate</v>, to be exported) a \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule for the constrained function, for \u000A example, the <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (foo\u000A x))</v>.</p> \u000A \u000A <p><i>First cut at constraint-assigning algorithm.</i> Quite simply, the \u000A formulas introduced in the scope of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> are conjoined, and \u000A each function symbol introduced by the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is assigned that \u000A conjunction as its constraint.</p> \u000A \u000A <p>Clearly this is a rather severe algorithm.  Let us consider two possible \u000A optimizations in an informal manner before presenting our second cut.</p> \u000A \u000A <p>Consider the (rather artificial) event below.  The function <v>before1</v> \u000A does not refer at all, even indirectly, to the locally-introduced function \u000A <v>sig-fn</v>, so it is unfortunate to saddle it with constraints about \u000A <v>sig-fn</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> before1 (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       (before1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A     x))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> sig-fn-prop\u000A   (<see topic=\"COMMON-LISP____CONSP\">consp</see> (sig-fn x)))\u000A )</code> \u000A \u000A <p>We would like to imagine moving the definition of <v>before1</v> to just in \u000A front of this <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> before1 (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (before1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A    x))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> sig-fn-prop\u000A   (<see topic=\"COMMON-LISP____CONSP\">consp</see> (sig-fn x)))\u000A )</code> \u000A \u000A <p>Thus, we will only assign the constraint <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> (sig-fn x))</v>, from the \u000A theorem <v>sig-fn-prop</v>, to the function <v>sig-fn</v>, not to the function \u000A <v>before1</v>.</p> \u000A \u000A <p>More generally, suppose an event in an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event does not \u000A mention any function symbol in the <see topic=\"ACL2____SIGNATURE\">signature</see> of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, nor any function symbol that mentions any such function symbol, \u000A and so on.  (We might say that no function symbol from the <see topic=\"ACL2____SIGNATURE\">signature</see> is \u000A an ``ancestor'' of any function symbol occurring in the event.)  Then we \u000A imagine moving the event, so that it appears in front of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  We don't actually move it, but we pretend we do when it comes \u000A time to assign constraints.  Thus, such definitions only introduce \u000A definitional axioms as the constraints on the function symbols being defined. \u000A In the example above, the event <v>sig-fn-prop</v> introduces no constraints on \u000A function <v>before1</v>.</p> \u000A \u000A <p>Once this first optimization is performed, we have in mind a set of \u000A ``constrained functions.''  These are the functions introduced in the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> that would remain after moving some of them in front, as \u000A indicated above.  Consider the collection of all formulas introduced by the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, except the definitional axioms, that mention these \u000A constrained functions.  So for example, in the event below, no such formula \u000A mentions the function symbol <v>after1</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> sig-fn-prop\u000A   (<see topic=\"COMMON-LISP____CONSP\">consp</see> (sig-fn x)))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> after1 (x)\u000A   (sig-fn x))\u000A )</code> \u000A \u000A <p>We can see that there is really no harm in imagining that we move the \u000A definition of <v>after1</v> out of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, to just after the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p> \u000A \u000A <p>Many subtle aspects of this rearrangement process have been omitted.  For \u000A example, suppose the function <v>fn</v> uses <v>sig-fn</v>, the latter being a \u000A function in the signature of the encapsulation.  Suppose a formula about \u000A <v>fn</v> is proved in the encapsulation.  Then from the discussion above \u000A <v>fn</v> is among the constrained functions of the encapsulate: it cannot be \u000A moved before the encapsulate and it cannot be moved after the encapsulation. \u000A But why is <v>fn</v> constrained?  The reason is that the theorem proved about \u000A <v>fn</v> may impose or express constraints on <v>sig-fn</v>.  That is, the \u000A theorem proved about <v>fn</v> may depend upon properties of the witness used \u000A for <v>sig-fn</v>.  Here is a simple example:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) 0))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (lst)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> lst)\u000A       t\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (sig-fn (<see topic=\"COMMON-LISP____CAR\">car</see> lst)))\u000A            (fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> fn-always-true\u000A   (fn lst)))</code> \u000A \u000A <p>In this example, there are no <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events that mention \u000A <v>sig-fn</v> explicitly.  One might therefore conclude that it is completely \u000A unconstrained.  But the witness we chose for it always returns an integer. \u000A The function <v>fn</v> uses <v>sig-fn</v> and we prove that <v>fn</v> always returns \u000A true.  Of course, the proof of this theorem depends upon the properties of the \u000A witness for <v>sig-fn</v>, even though those properties were not explicitly \u000A ``called out'' in theorems proved about <v>sig-fn</v>.  It would be unsound to \u000A move <v>fn-always-true</v> after the encapsulate.  It would also be unsound to \u000A constrain <v>sig-fn</v> to satisfy just <v>fn-always-true</v> without including in \u000A the constraint the relation between <v>sig-fn</v> and <v>fn</v>.  Hence both \u000A <v>sig-fn</v> and <v>fn</v> are constrained by this encapsulation and the \u000A constraint imposed on each is the same and states the relation between the two \u000A as characterized by the equation defining <v>fn</v> as well as the property that \u000A <v>fn</v> always returns true.  Suppose, later, one proved a theorem about \u000A <v>sig-fn</v> and wished to functionally instantiate it.  Then one must also \u000A functionally instantiate <v>fn</v>, even if it is not involved in the theorem, \u000A because it is only through <v>fn</v> that <v>sig-fn</v> inherits its constrained \u000A properties.</p> \u000A \u000A <p>This is a pathological example that illustrate a trap into which one may \u000A easily fall: rather than identify the key properties of the constrained \u000A function the user has foreshadowed its intended application and constrained \u000A those notions.  Clearly, the user wishing to introduce the <v>sig-fn</v> above \u000A would be well-advised to use the following instead:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) 0))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> integerp-sig-fn\u000A   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (sig-fn x))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (lst)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> lst)\u000A      t\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (sig-fn (<see topic=\"COMMON-LISP____CAR\">car</see> lst)))\u000A         (fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> fn-always-true\u000A   (fn lst)))</code> \u000A \u000A <p>Note that <v>sig-fn</v> is constrained merely to be an integer.  It is the \u000A only constrained function.  Now <v>fn</v> is introduced after the encapsulation, \u000A as a simple function that uses <v>sig-fn</v>.  We prove that <v>fn</v> always \u000A returns true, but this fact does not constrain <v>sig-fn</v>.  Future uses of \u000A <v>sig-fn</v> do not have to consider <v>fn</v> at all.</p> \u000A \u000A <p>Sometimes it is necessary to introduce a function such as <v>fn</v> within \u000A the <v>encapsulate</v> merely to state the key properties of the undefined \u000A function <v>sig-fn</v>.  But that is unusual and the user should understand that \u000A both functions are being constrained.</p> \u000A \u000A <p>Another subtle aspect of encapsulation that has been brushed over so far \u000A has to do with exactly how functions defined within the encapsulation use the \u000A signature functions.  For example, above we say ``Consider the collection of \u000A all formulas introduced by the encapsulate, <i>except the definitional \u000A axioms</i>, that mention these constrained functions.''  We seem to suggest \u000A that a definitional axiom which mentions a constrained function can be moved \u000A out of the encapsulation and considered part of the ``post-encapsulation'' \u000A extension of the logical <see topic=\"ACL2____WORLD\">world</see>, if the defined function is not used in \u000A any non-definitional formula proved in the encapsulation.  For example, in the \u000A encapsulation above that constrained <v>sig-fn</v> and introduced <v>fn</v> within \u000A the encapsulation, <v>fn</v> was constrained because we proved the formula \u000A <v>fn-always-true</v> within the encapsulation.  Had we not proved \u000A <v>fn-always-true</v> within the encapsulation, <v>fn</v> could have been moved \u000A after the encapsulation.  But this suggests an unsound rule because whether \u000A such a function can be moved after the encapsulate depend on whether its \u000A <i>admission</i> used properties of the witnesses!  In particular, we say a \u000A function is ``subversive'' if any of its governing tests or the actuals in any \u000A recursive call involve a function in which the signature functions are \u000A ancestral.  See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.</p> \u000A \u000A <p>(Aside: The definition of <v>fn</v> in the first enapsulation above that \u000A defines <v>fn</v>, i.e., the encapsulation with <v>fn-always-true</v> inside, is \u000A subversive because the call of the macro <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> expands to a call of \u000A <v>IF</v> that governs a recursive call of <v>fn</v>, in this case:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (lst)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> lst)\u000A      t\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (sig-fn (<see topic=\"COMMON-LISP____CAR\">car</see> lst)))\u000A          (fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A        nil))).</code> \u000A \u000A <p>If we switch the order of conjuncts in <v>fn</v>, then the definition of \u000A <v>fn</v> is no longer subversive, but it still ``infects'' the constraint \u000A generated for the encapsulation, hence for <v>sig-fn</v>, because \u000A <v>fn-always-true</v> blocks the definition of <v>fn</v> from being moved back (to \u000A after the encapsulation).  If we both switch the order of conjuncts and drop \u000A <v>fn-always-true</v> from the encapsulation, then the definition of <v>fn</v> is \u000A in essence moved back to after the encapsulation, and the constraint for \u000A <v>sig-fn</v> no longer includes the definition of <v>fn</v>.  End of aside.)</p> \u000A \u000A <p>Another aspect we have not discussed is what happens to nested \u000A encapsulations when each introduces constrained functions.  We say an \u000A <v>encapsulate</v> event is ``trivial'' if it introduces no constrained \u000A functions, i.e., if its signatures is <v>nil</v>.  Trivial encapsulations are \u000A just a way to wrap up a collection of events into a single event.</p> \u000A \u000A <p>From the foregoing discussion we see we are interested in exactly how we \u000A can ``rearrange'' the events in a non-trivial encapsulation -- moving some \u000A ``before'' the encapsulation and others ``after'' the encapsulation.  We are \u000A also interested in which functions introduced by the encapsulation are \u000A ``constrained'' and what the ``constraints'' on each are.  We may summarize \u000A the observations above as follows, after which we conclude with a more \u000A elaborate example.</p> \u000A \u000A <p><i>Second cut at constraint-assigning algorithm.</i> First, we focus only \u000A on non-trivial encapsulations that neither contain nor are contained in \u000A non-trivial encapsulations.  (Nested non-trivial encapsulations are not \u000A rearranged at all: do not put anything in such a nest unless you mean for it \u000A to become part of the constraints generated.)  Second, in what follows we only \u000A consider the non-<v>local</v> events of such an <v>encapsulate</v>, assuming that \u000A they satisfy the restriction of using no locally defined function symbols \u000A other than the signature functions.  Given such an <v>encapsulate</v> event, \u000A move, to just in front of it and in the same order, all definitions and \u000A theorems for which none of the signature functions is ancestral.  Now collect \u000A up all formulas (theorems) introduced in the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> other than \u000A definitional axioms.  Add to this set any of those definitional equations that \u000A is either subversive or defines a function used in a formula in the set.  The \u000A conjunction of the resulting set of formulas is called the ``constraint'' and \u000A the set of all the signature functions of the <v>encapsulate</v> together with \u000A all function symbols defined in the <v>encapsulate</v> and mentioned in the \u000A constraint is called the ``constrained functions.''  Assign the constraint to \u000A each of the constrained functions.  Move, to just after the <v>encapsulate</v>, \u000A the definitions of all function symbols defined in the <v>encapsulate</v> that \u000A have been omitted from the constraint.</p> \u000A \u000A <p>Implementation note.  In the implementation we do not actually move <see topic=\"ACL2____EVENTS\">events</see>, but we create constraints that pretend that we did.</p> \u000A \u000A <p>Here is an example illustrating our constraint-assigning algorithm.  It \u000A builds on the preceding examples.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((sig-fn *) =&gt; *))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> before1 (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       (before1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A     x))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> sig-fn (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> sig-fn-prop\u000A   (<see topic=\"COMMON-LISP____CONSP\">consp</see> (sig-fn x)))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> during (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       x\u000A     (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (sig-fn x))\u000A           17)))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> before2 (x)\u000A   (before1 x))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> before2-prop\u000A   (<see topic=\"COMMON-LISP____ATOM\">atom</see> (before2 x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> during-prop\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A                 (before2 x))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (during x))\u000A                   (<see topic=\"COMMON-LISP____CAR\">car</see> (sig-fn x)))))\u000A\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> after1 (x)\u000A   (sig-fn x))\u000A\u000A (<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> after2 (x) (<see topic=\"ACL2____U\">u</see>)\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> u x) (during x)))\u000A )</code> \u000A \u000A <p>Only the functions <v>sig-fn</v> and <v>during</v> receive extra constraints. \u000A The functions <v>before1</v> and <v>before2</v> are viewed as moving in front of \u000A the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, as is the theorem <v>before2-prop</v>.  The functions \u000A <v>after1</v> and <v>after2</v> are viewed as being moved past the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  The implementation reports the following.</p> \u000A \u000A <code>In addition to SIG-FN, we export AFTER2, AFTER1, BEFORE2, DURING and\u000ABEFORE1.\u000A\u000AThe following constraint is associated with both of the functions DURING and\u000ASIG-FN:\u000A\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (DURING X)\u000A            (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                X (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (SIG-FN X)) 17)))\u000A     (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (SIG-FN X))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____ATOM\">ATOM</see> X) (BEFORE2 X))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (DURING X))\u000A                     (<see topic=\"COMMON-LISP____CAR\">CAR</see> (SIG-FN X)))))</code> \u000A \u000A <p>Notice that the formula <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> (during x))</v> is not a conjunct of the \u000A constraint.  During the first pass of the <v>encapsulate</v>, this formula is \u000A stored as a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule deduced during the definition \u000A of the function <v>during</v>.  However, the rule is not exported because of a \u000A rather subtle soundness issue.  (If you are interested in details, see the \u000A comments in source function <v>putprop-type-prescription-lst</v>.)</p> \u000A \u000A <p>We conclude by asking (and to a certain extent, answering) the following \u000A question: Isn't there an approach to assigning constraints that avoids \u000A over-constraining more simply than our ``second cut'' above?  Perhaps it seems \u000A that given an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, we should simply assign to each locally \u000A defined function the theorems exported about that function.  If we adopted \u000A that simple approach the events below would be admissible.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((foo *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A   (foo x))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> bar-prop\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar x) x)\u000A   :rule-classes nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-id\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x)\u000A  :hints ((&quot;Goal&quot; :use bar-prop)))\u000A\u000A; The following event is not admissible in ACL2.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch!\u000A  nil\u000A  :rule-classes nil\u000A  :hints\u000A  ((&quot;Goal&quot; :use\u000A    ((:functional-instance foo-id\u000A                           (foo (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))))))))</code> \u000A \u000A <p>Under the simple approach we have in mind, <v>bar</v> is constrained to \u000A satisfy both its definition and <v>bar-prop</v> because <v>bar</v> mentions a \u000A function declared in the signature list of the encapsulation.  In fact, \u000A <v>bar</v> is so-constrained in the ACL2 semantics of encapsulation and the \u000A first two events above (the <v>encapsulate</v> and the consequence that <v>foo</v> \u000A must be the identity function) are actually admissible.  But under the simple \u000A approach to assigning constraints, <v>foo</v> is unconstrained because no \u000A theorem about it is exported.  Under that approach, <v>ouch!</v> is provable \u000A because <v>foo</v> can be instantiated in <v>foo-id</v> to a function other than \u000A the identity function.</p> \u000A \u000A <p>It's tempting to think we can fix this by including definitions, not just \u000A theorems, in constraints.  But consider the following slightly more elaborate \u000A example.  The problem is that we need to include as a constraint on <v>foo</v> \u000A not only the definition of <v>bar</v>, which mentions <v>foo</v> explicitly, but \u000A also <v>abc</v>, which has <v>foo</v> as an ancestor.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((foo *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-prop\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x)))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A   (foo x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> abc (x)\u000A   (bar x))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> abc-prop\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (abc x) x)\u000A   :rule-classes nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-id\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x)\u000A  :hints ((&quot;Goal&quot; :use abc-prop)))\u000A\u000A; The following event is not admissible in ACL2.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch!\u000A  nil\u000A  :rule-classes nil\u000A  :hints\u000A  ((&quot;Goal&quot; :use\u000A    ((:functional-instance foo-id\u000A                           (foo (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A                           (bar (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))))))))</code>"],

 "ACL2____ACCUMULATED-PERSISTENCE":[["Debugging"],"ACL2 Sources","ACL2","<code>Useful Forms:\u000A(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> t)              ; Activate statistics gathering.\u000A(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)           ; As above, ``enhanced'' (see below)\u000A\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames)   ; Display statistics ordered by\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :tries)    ; frames built, times tried,\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :ratio)    ; or their ratio.\u000A\u000A(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> nil)            ; Deactivate.\u000A\u000AAdvanced forms:\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames-s) ; The `s', `f', and `a' suffixes\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames-f) ; stand for `success' (`useful'),\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames-a) ; `failure' (`useless'), and `all',\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :tries-s)  ; respectively.  The only effect of\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :tries-f)  ; the `s' and `f' versions is to\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :tries-a)  ; sort first by useful or useless\u000A                                         ; applications, respectively (see\u000A                                         ; below).  The `a' versions avoid\u000A                                         ; showing the useful/useless\u000A                                         ; breakdown.\u000A\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :runes)    ; Just show runes alphabetically.</code> \u000A \u000A <p>Note: <v>set-accumulated-persistence</v> is equivalent to \u000A <v>accumulated-persistence</v>.</p> \u000A \u000A <p>See the end of this item for a discussion of ``enhanced statistics \u000A gathering,'' which can be useful for more fine-grained proof debugging.</p> \u000A \u000A <p>Generally speaking, the more ACL2 knows, the slower it runs.  That is \u000A because the search space grows with the number of alternative rules.  Often, \u000A the system tries to apply rules that you have forgotten were even there, if \u000A you knew about them in the first place!  ``Accumulated-persistence'' is a \u000A statistic (originally developed for Nqthm) that helps you identify the rules \u000A that are causing ACL2's search space to explode.</p> \u000A \u000A <p>For other proof debugging utilities, see <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> and see <see topic=\"ACL2____DMR\">dmr</see>.</p> \u000A \u000A <p>Accumulated persistence tracking can be turned on or off.  It is generally \u000A off.  When on, proofs may take perhaps 50% more time than otherwise!  But some \u000A useful numbers are collected.  When it is turned on, by</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> t)</code> \u000A \u000A <p>an accumulation site is initialized and henceforth data about which rules \u000A are being tried is accumulated into that site.  That accumulated data can be \u000A displayed with <v>show-accumulated-persistence</v>, as described in detail \u000A below.  When accumulated persistence is turned off, with \u000A <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> nil)</v>, the accumulation site is wiped out and the \u000A data in it is lost.</p> \u000A \u000A <p>The ``accumulated persistence'' of a <see topic=\"ACL2____RUNE\">rune</see> is the number of <see topic=\"ACL2____RUNE\">rune</see>s the system has attempted to apply (since accumulated persistence was \u000A last activated) while the given <see topic=\"ACL2____RUNE\">rune</see> was being tried.</p> \u000A \u000A <p>Consider a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule named <tt><see topic=\"ACL2____RUNE\">rune</see></tt>.  For simplicity, \u000A let us imagine that <tt><see topic=\"ACL2____RUNE\">rune</see></tt> is tried only once in the period during which \u000A accumulated persistence is being monitored.  Recall that to apply a rewrite \u000A rule we must match the left-hand side of the conclusion to some term we are \u000A trying to rewrite, establish the hypotheses of <tt><see topic=\"ACL2____RUNE\">rune</see></tt> by rewriting, and, \u000A if successful, then rewrite the right-hand side of the conclusion.  We say \u000A <tt><see topic=\"ACL2____RUNE\">rune</see></tt> is ``being tried'' from the time we have matched its left-hand \u000A side to the time we have either abandoned the attempt or finished rewriting \u000A its right-hand side.  (By ``match'' we mean to include any loop-stopper \u000A requirement; see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.)  During that period of time other rules \u000A might be tried, e.g., to establish the hypotheses.  The rules tried while \u000A <tt><see topic=\"ACL2____RUNE\">rune</see></tt> is being tried are ``billed'' to <tt><see topic=\"ACL2____RUNE\">rune</see></tt> in the sense that \u000A they are being considered here only because of the demands of <tt><see topic=\"ACL2____RUNE\">rune</see></tt>. \u000A Thus, if no other rules are tried during that period, the accumulated \u000A persistence of <tt><see topic=\"ACL2____RUNE\">rune</see></tt> is <v>1</v> &mdash; we ``bill'' <tt><see topic=\"ACL2____RUNE\">rune</see></tt> once \u000A for its own application attempt.  If, on the other hand, we tried <v>10</v> \u000A rules on behalf of that application of <tt><see topic=\"ACL2____RUNE\">rune</see></tt>, then <tt><see topic=\"ACL2____RUNE\">rune</see></tt>'s \u000A accumulated persistence would be <v>11</v>.</p> \u000A \u000A <p>One way to envision accumulated persistence is to imagine that every time a \u000A <see topic=\"ACL2____RUNE\">rune</see> is tried it is pushed onto a stack.  The rules tried on behalf of \u000A a given application of a <see topic=\"ACL2____RUNE\">rune</see> are thus pushed and popped on the stack \u000A above that <see topic=\"ACL2____RUNE\">rune</see>.  A lot of work might be done on its behalf &mdash; the \u000A stack above the <see topic=\"ACL2____RUNE\">rune</see> grows and shrinks repeatedly as the search \u000A continues for a way to use the <see topic=\"ACL2____RUNE\">rune</see>.  All the while, the <see topic=\"ACL2____RUNE\">rune</see> \u000A itself ``persists'' in the stack, until we finish with the attempt to apply \u000A it, at which time we pop it off.  The accumulated persistence of a <see topic=\"ACL2____RUNE\">rune</see> \u000A application is thus the number of stack frames built while that <see topic=\"ACL2____RUNE\">rune</see> \u000A was on the stack.</p> \u000A \u000A <p>Note that accumulated persistence is tallied whether or not the attempt to \u000A apply a <see topic=\"ACL2____RUNE\">rune</see> is successful.  Each of the rules tried on its behalf \u000A might have failed and the attempt to apply the <see topic=\"ACL2____RUNE\">rune</see> might have also \u000A failed.  The ACL2 proof script would make no mention of the <see topic=\"ACL2____RUNE\">rune</see> or the \u000A rules tried on its behalf because they did not contribute to the proof.  But \u000A time was spent pursuing the possible application of the <see topic=\"ACL2____RUNE\">rune</see> and \u000A accumulated persistence is a measure of that time.</p> \u000A \u000A <p>A high accumulated persistence might come about in two extreme ways.  One \u000A is that the rule causes a great deal of work every time it is tried.  The \u000A other is that the rule is ``cheap'' but is tried very often.  We therefore \u000A keep track of the number of times each rule is tried as well as its \u000A persistence.  The ratio between the two is the average amount of work done on \u000A behalf of the rule each time it is tried.</p> \u000A \u000A <p>When the accumulated persistence totals are displayed by the function \u000A <v>show-accumulated-persistence</v> we sort them so that the most expensive \u000A <see topic=\"ACL2____RUNE\">rune</see>s are shown first.  We can sort according to one of three basic \u000A keys:</p> \u000A \u000A <code>:frames - the number of frames built on behalf of the rune\u000A:tries  - the number of times the rune was tried\u000A:ratio  - frames built per try</code> \u000A \u000A <p>The key simply determines the order in which the information is presented. \u000A If no argument is supplied to <v>show-accumulated-persistence</v>, <v>:frames</v> \u000A is used.</p> \u000A \u000A <p>The display breaks each total into ``useful'' and ``useless'' subtotals.  A \u000A ``useful'' rule try is one that is viewed as contributing to the progress of \u000A the proof, and the rest are ``useless'' rule applications.  For example, if a \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule is tried but its hypotheses are not successfully \u000A relieved, then that rule application and all work done on behalf of those \u000A hypotheses is ``useless'' work.  In general, an attempt to apply a <see topic=\"ACL2____RUNE\">rune</see> \u000A is viewed as ``useful'' unless the attempt fails or the attempt is on the \u000A stack (as described above) for a <see topic=\"ACL2____RUNE\">rune</see> application that ultimately \u000A fails.  A large number of ``useless'' <v>:frames</v> or <v>:tries</v> along with \u000A correspondingly small ``useful'' counts may suggest <see topic=\"ACL2____RUNE\">rune</see>s to consider \u000A disabling (see <see topic=\"ACL2____DISABLE\">disable</see> and see <see topic=\"ACL2____IN-THEORY\">in-theory</see>).  Thus, here is a more \u000A complete list of the arguments that may be supplied to \u000A <v>show-accumulated-persistence</v>.  Suffixes ``s'', ``f'', and ``a'' are \u000A intended to suggest ``success'' (``useful''), ``failure'' (``useless''), and \u000A ``all''.</p> \u000A \u000A <code>:frames     - sort by the number of frames built on behalf of the rune\u000A   :frames-s -   as above, but sort by useful applications\u000A   :frames-f -   as above, but sort by useless applications\u000A   :frames-a -   as above, but inhibit display of ``useful'' and\u000A                 ``useless'' subtotals\u000A:tries      - sort by the number of times the rune was tried\u000A   :tries-s  -   as above, but sort by useful applications\u000A   :tries-f  -   as above, but sort by useless applications\u000A   :tries-a  -   as above, but inhibit display of ``useful'' and\u000A                 ``useless'' subtotals\u000A:ratio      - sort by frames built per try\u000A:useless    - show only the runes tried whose tries were all ``useless''</code> \u000A \u000A <p>For a given line of the report, every frame credited to a ``useful'' \u000A (respectively, ``useless'') rule application is considered ``useful'' \u000A (respectively, ``useless'').  We illustrate with the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A  (<see topic=\"ACL2____DEFSTUB\">defstub</see> hyp (x) t)\u000A  (<see topic=\"ACL2____DEFSTUB\">defstub</see> concl (x) t)\u000A  (<see topic=\"ACL2____DEFSTUB\">defstub</see> bad (x) t)\u000A  (<see topic=\"ACL2____DEFSTUB\">defstub</see> good (x) t)\u000A  (<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> good-ax\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (good x) (hyp x)))\u000A  (<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> bad-ax\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (bad x) (hyp x)))\u000A  (<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> hyp-implies-concl\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (hyp x) (concl x)))\u000A  )\u000A(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> t)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (good x) (concl x)))\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see>)</code> \u000A \u000A <p>To prove the <tt><see topic=\"ACL2____THM\">thm</see></tt> form, ACL2 attempts to rewrite <v>(concl x)</v> to \u000A true by applying rule <v>hyp-implies-concl</v>.  It then attempts to establish \u000A <v>(hyp x)</v> first by trying rule <v>bad-ax</v>, which fails, and second by \u000A trying rule <v>good-ax</v>, which succeeds.  As expected, the report labels as \u000A ``useless'' the failure of the attempt to establish the hypothesis, <v>(bad\u000A x)</v>.</p> \u000A \u000A <code>--------------------------------\u000A      1        1 (    1.00) (:REWRITE BAD-AX)\u000A      0        0    [useful]\u000A      1        1    [useless]\u000A--------------------------------</code> \u000A \u000A <p>Now consider the top-level application of rule <v>hyp-implies-concl</v>. \u000A Even though the above report shows the application of <v>bad-ax</v> as \u000A ``useless'', note that this rule was applied on behalf of the successful \u000A (``useful'') application of <v>hyp-implies-concl</v>, and hence is incorporated \u000A into the ``useful'' line for <v>hyp-implies-concl</v>, as follows.</p> \u000A \u000A <code>--------------------------------\u000A      3        1 (    3.00) (:REWRITE HYP-IMPLIES-CONCL)\u000A      3        1    [useful]\u000A      0        0    [useless]\u000A--------------------------------</code> \u000A \u000A <p>In summary: categorization of <v>:frames</v> as ``useful'' or ``useless'' is \u000A based on whether they support ``useful'' or ``useless'' <v>:tries</v>.</p> \u000A \u000A <p>Note that a <see topic=\"ACL2____RUNE\">rune</see> with high accumulated persistence may not actually \u000A be the ``culprit.''  For example, suppose <v>rune1</v> is reported to have a \u000A <v>:ratio</v> of <v>101</v>, meaning that on the average a hundred and one frames \u000A were built each time <v>rune1</v> was tried.  Suppose <v>rune2</v> has a \u000A <v>:ratio</v> of <v>100</v>.  It could be that the attempt to apply <v>rune1</v> \u000A resulted in the attempted application of <v>rune2</v> and no other <see topic=\"ACL2____RUNE\">rune</see>. \u000A Thus, in some sense, <v>rune1</v> is ``cheap'' and <v>rune2</v> is the ``culprit'' \u000A even though it is reported as costing less than <v>rune1</v>.</p> \u000A \u000A <p>If a proof is aborted, then in general, <tt><see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see></tt> will only display totals for runes whose \u000A attempted application is complete: that is, if the rewriter was in the process \u000A of relieving hypotheses for a rule, then information for that rule will not be \u000A included in the tally.  We say ``in general'' because, as indicated near the \u000A top of the output from <tt><see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see></tt> when such \u000A incomplete information is omitted, you can get this information by using \u000A argument <v>:frames-a</v> or <v>:tries-a</v>.</p> \u000A \u000A <p>There are other subtleties in how rune applications are tallied, documented \u000A elsewhere: see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE-SUBTLETIES\">accumulated-persistence-subtleties</see>.</p> \u000A \u000A <p>We conclude with a discussion of ``enhanced'' statistics gathering, which \u000A is enabled by supplying <v>accumulated-persistence</v> the argument \u000A <v>:ALL</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)</code> \u000A \u000A <p>At some additional performance expense (but probably well under a factor of \u000A 2 altogether), ACL2 then gathers additional statistics for individual \u000A hypotheses of rules as well as their conclusions.  To understand how this \u000A works, suppose <v>rn</v> is a <see topic=\"ACL2____RUNE\">rune</see>.  Then we prepend the keyword \u000A <v>:CONC</v> to <v>rn</v> to form what we call its ``conclusion xrune'', and for \u000A its <v>I</v>-th hypothesis we prepend <v>:HYP I</v> to <v>rn</v> to form its \u000A <v>I</v>-th ``hypothesis xrune.''  Here, ``xrune'' is pronounced ``ex rune'', \u000A and is mnemonic for ``extended rune.''  For example, if <v>(<see topic=\"ACL2____REWRITE\">REWRITE</see> FOO)</v> is \u000A a <see topic=\"ACL2____RUNE\">rune</see> then <v>(:CONC REWRITE FOO)</v> is its conclusion xrune, and \u000A <v>(:HYP 2 REWRITE FOO)</v> is a hypothesis xrune corresponding to the second \u000A hypothesis of the corresponding rewrite rule.</p> \u000A \u000A <p>With <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)</v>, we instruct ACL2 to track not \u000A only runes but also xrunes.  Then, <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see>)</v> will \u000A display information for all xrunes in a format that we consider to be ``raw'', \u000A in the sense that data for xrunes are displayed just as for runes.  But a \u000A ``merged'' format is also available.  Here is a summary of display commands, \u000A followed below by further discussion.</p> \u000A \u000A <code>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames t) ; t is optional, i.e., the default\u000A   ; Display enhanced statistics sorted by frames, in a ``raw'' format.\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames :merge)\u000A   ; Display enhanced statistics sorted by frames, in a ``merged'' format.\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames nil)\u000A   ; Display regular statistics sorted by frames, without the enhancements.\u000A\u000A; More generally, the descriptions just above apply for any legal first\u000A; argument:\u000A\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY t)\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY :merge)\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY nil)\u000A\u000A; Note also these alternate forms, equivalent to the first of the two forms\u000A; just above, i.e., the form with second argument of t:\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY :raw)\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY)</code> \u000A \u000A <p>There is a significant difference between how runes are tracked and how \u000A ACL2 tracks hypothesis and conclusion xrunes: unlike regular runes, these \u000A xrunes do not contribute to the accumulated <v>:frames</v> counts.  Rather, they \u000A serve as accumulation sites without contributing their <v>:tries</v> to any \u000A accumulation.  Consider for example the snippet below, taken from a report \u000A created with the <v>:merge</v> option (to be discussed further below), i.e., by \u000A evaluating the form <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :frames :merge)</v>.</p> \u000A \u000A <code>:frames   :tries    :ratio  rune\u000A--------------------------------\u000A    462      211 (    2.18) (:REWRITE PERM-MEM)\u000A     13        6    [useful]\u000A    449      205    [useless]\u000A   .............................\u000A    251       47 (    5.34) (:HYP 2 :REWRITE PERM-MEM)\u000A      6        6    [useful]\u000A    245       41    [useless]\u000A   .............................\u000A      0      211 (    0.00) (:HYP 1 :REWRITE PERM-MEM)\u000A      0        6    [useful]\u000A      0      205    [useless]\u000A   .............................\u000A      0        7 (    0.00) (:CONC :REWRITE PERM-MEM)\u000A      0        6    [useful]\u000A      0        1    [useless]\u000A--------------------------------</code> \u000A \u000A <p>Notice that while <v>:tries</v> are recorded for the xrune <v>(:HYP 1\u000A :REWRITE PERM-MEM)</v>, no <v>:frames</v> are recorded.  This is because no stack \u000A frames were built for runes while this xrune was on the stack &mdash; only for \u000A the xrune itself, which as we explained above is not accumulated into the \u000A total <v>:frames</v> counts.  As it turns out, this lack of stack frames is \u000A explained by the fact that the rewrite rule <v>PERM-MEM</v> has a free variable \u000A in the first hypothesis.</p> \u000A \u000A <code>ACL2 !&gt;:pe perm-mem\u000A         18  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> PERM-MEM\u000A                     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (PERM X Y) (MEM A X))\u000A                              (MEM A Y))\u000A                     :RULE-CLASSES ((:REWRITE :MATCH-FREE :ONCE)))\u000AACL2 !&gt;</code> \u000A \u000A <p>The second hypothesis, however, does cause additional rewriting in order to \u000A rewrite it to true, resulting in 251 stack frames for runes.  We see that the \u000A conclusion does not lead to creation of any rune stack frames, which might \u000A seem to suggest that only 251 stack frames for runes were created on behalf of \u000A this rule application &mdash; yet, we see that 462 frames were actually \u000A created.  The difference is the 211 frames created for the rewrite rule \u000A itself.  Even if the total had been a bit more than 462, one need not be \u000A surprised, as there could be some work recorded during application of the \u000A rewrite rule, such as type-prescription reasoning, that is not done during \u000A rewriting of a hypothesis or the conclusion.</p> \u000A \u000A <p>Now suppose we have executed <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)</v> and \u000A attempted some proofs, and now we are ready to see statistics.  The form \u000A <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see>)</v> displays statistics exactly as described \u000A above, treating these extra xrunes just as though they are runes; similarly \u000A for the form <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> KEY)</v>, for any legal <v>KEY</v>. \u000A A second optional argument may however be supplied to \u000A <v>show-accumulated-persistence</v>.  The default for that second argument is \u000A <v>t</v>, and a second argument of <v>:raw</v> is treated the same as <v>t</v>; \u000A thus, these arguments provide the behavior just described, where data for \u000A xrunes are displayed just as for runes.  You may restrict output to runes, \u000A ignoring hypothesis and conclusion xrunes, by giving a second argument of \u000A <v>nil</v>.  (This gives the same behavior as if we had started with the command \u000A <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> t)</v> instead of the command \u000A <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)</v>.)  Finally, you may give a second argument \u000A of <v>:merge</v>, in which case output will be sorted and displayed as though \u000A only runes were tracked (not the extra xrunes), but each data item for a \u000A non-rune xrune will be merged so that it is displayed in suitable order just \u000A below its corresponding rune, as in the <v>PERM-MEM</v> example displayed \u000A above.</p> \u000A \u000A <p>We close by mentioning two aspects of enhanced statistics display for \u000A <v>:CONC</v> xrunes that have potential to be confusing.  First consider the \u000A following example.</p> \u000A \u000A <code>  :frames   :tries    :ratio  rune\u000A--------------------------------\u000A     14        4 (    3.50) (:REWRITE DEFAULT-+-2)\u000A      0        0    [useful]\u000A     14        4    [useless]\u000A   .............................\u000A     10        4 (    2.50) (:HYP 1 :REWRITE DEFAULT-+-2)\u000A      0        0    [useful]\u000A     10        4    [useless]\u000A--------------------------------</code> \u000A \u000A <p>It may be surprising that no data is displayed for the corresponding \u000A <v>:CONC</v> xrune.  The explanation, however, is simple: the hypothesis never \u000A rewrote to true, so the conclusion was never rewritten.  This is consistent \u000A with the marking as ``useless'' of all <v>:frames</v> and <v>:tries</v> for the \u000A rune and the hypothesis xrune.  Note by the way, once again, that the \u000A hypothesis xrune does not contribute to any <v>:frames</v> count.</p> \u000A \u000A <p>Another reason not to see data displayed for a <v>:CONC</v> xrune is that if \u000A a rule has no hypotheses, then no such data is collected.  This decision was \u000A made because in the case of no hypotheses, we expect it to be very rare that \u000A information for the <v>:CONC</v> xrune will add any useful insight.</p> \u000A \u000A <p>On a final note: <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :runes)</v> may be used \u000A simply to see a list of all <see topic=\"ACL2____RUNE\">rune</see>s (or xrunes) displayed \u000A alphabetically.</p> \u000A \u000A <p>Users are encouraged to think about other meters we could install in ACL2 \u000A to help diagnose performance problems.</p>"],

 "ACL2____SYNTAXP":[["Miscellaneous","Rewrite"],"ACL2 Sources","ACL2","<p>A call of <v>syntaxp</v> in the hypothesis of a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule is treated \u000A specially, as described below.  Similar treatment is given to the evaluation \u000A of a <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rule's hypothesis function call.</p> \u000A \u000A <p>For example, consider the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule created from the \u000A following formula.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                            (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'NORM))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (LXD X)\u000A                (LXD (NORM X)))).</code> \u000A \u000A <p>The <v>syntaxp</v> hypothesis in this rule will allow the rule to be applied \u000A to <v>(lxd (trn a b))</v> but will not allow it to be applied to <v>(lxd (norm\u000A a))</v>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> test)</code> \u000A \u000A <p><v>Syntaxp</v> always returns <v>t</v> and so may be added as a vacuous \u000A hypothesis.  However, when relieving the hypothesis, the test ``inside'' the \u000A <v>syntaxp</v> form is actually treated as a meta-level proposition about the \u000A proposed instantiation of the rule's variables and that proposition must \u000A evaluate to true (non-<v>nil</v>) to ``establish'' the <v>syntaxp</v> \u000A hypothesis.</p> \u000A \u000A <p>Note that the test of a <v>syntaxp</v> hypothesis does not, in general, deal \u000A with the meaning or semantics or values of the terms, but rather with their \u000A syntactic forms.  In the example above, the <v>syntaxp</v> hypothesis allows the \u000A rule to be applied to every target of the form <v>(lxd u)</v>, provided <v>u</v> \u000A is not of the form <v>(norm v)</v>.  Observe that without this syntactic \u000A restriction the rule above could loop, producing a sequence of increasingly \u000A complex targets <v>(lxd a)</v>, <v>(lxd (norm a))</v>, <v>(lxd (norm (norm a)))</v>, \u000A etc.  An intuitive reading of the rule might be ``<v>norm</v> the argument of \u000A <v>lxd</v> unless it has already been <v>norm</v>ed.''</p> \u000A \u000A <p>Note also that a <v>syntaxp</v> hypothesis deals with the syntactic form used \u000A internally by ACL2, rather than that seen by the user.  In some cases these \u000A are the same, but there can be subtle differences with which the writer of a \u000A <v>syntaxp</v> hypothesis must be aware.  You can use <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt> to \u000A display this internal representation.</p> \u000A \u000A <p>There are two types of <v>syntaxp</v> hypotheses.  The simpler type may be a \u000A hypothesis of a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, or \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule provided <v>test</v> contains at least one variable \u000A but no free variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).  In particular, <v>test</v> \u000A may not use <see topic=\"ACL2____STOBJ\">stobj</see>s; any stobj name will be treated as an ordinary \u000A variable.  The case of <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules is similar to the above, \u000A except that it applies to the result of applying the hypothesis metafunction. \u000A (Later below we will describe the second type, an <i>extended</i> <v>syntaxp</v> \u000A hypothesis, which may use <tt><see topic=\"ACL2____STATE\">state</see></tt>.)</p> \u000A \u000A <p>We illustrate the use of simple <v>syntaxp</v> hypotheses by slightly \u000A elaborating the example given above.  Consider a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> \u000A rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)\u000A              (<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                                 (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'NORM)))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (LXD X)\u000A                (LXD (NORM X))))</code> \u000A \u000A <p>How is this rule applied to <v>(lxd (trn a b))</v>?  First, we form a \u000A substitution that instantiates the left-hand side of the conclusion of the \u000A rule so that it is identical to the target term.  In the present case, the \u000A substitution replaces <v>x</v> with <v>(trn a b)</v>.</p> \u000A \u000A <code>(LXD X) ==&gt; (LXD (trn a b)).</code> \u000A \u000A <p>Then we backchain to establish the hypotheses, in order.  Ordinarily this \u000A means that we instantiate each hypothesis with our substitution and then \u000A attempt to rewrite the resulting instance to true.  Thus, in order to relieve \u000A the first hypothesis above, we rewrite</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> (trn a b)).</code> \u000A \u000A <p>If this rewrites to true, we continue.</p> \u000A \u000A <p>Of course, many users are aware of some exceptions to this general \u000A description of the way we relieve hypotheses.  For example, if a hypothesis \u000A contains a ``free-variable'' &mdash; one not bound by the current substitution \u000A &mdash; we attempt to extend the substitution by searching for an instance of \u000A the hypothesis among known truths.  See <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>.  <tt><see topic=\"ACL2____FORCE\">Force</see></tt>d \u000A hypotheses are another exception to the general rule of how hypotheses are \u000A relieved.</p> \u000A \u000A <p>Hypotheses marked with <v>syntaxp</v>, as in <v>(<see topic=\"ACL2____SYNTAXP\">syntaxp</see> test)</v>, are also \u000A exceptions.  We instantiate such a hypothesis; but instead of rewriting the \u000A instantiated instance, we evaluate the instantiated <v>test</v>.  More \u000A precisely, we evaluate <v>test</v> in an environment in which its variable \u000A symbols are bound to the quotations of the terms to which those variables are \u000A bound in the instantiating substitution.  So in the case in point, we (in \u000A essence) evaluate</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> '(trn a b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> '(trn a b)) 'NORM))).</code> \u000A \u000A <p>This clearly evaluates to <v>t</v>.  When a <v>syntaxp</v> test evaluates to \u000A true, we consider the <v>syntaxp</v> hypothesis to have been established; this \u000A is sound because logically <v>(<see topic=\"ACL2____SYNTAXP\">syntaxp</see> test)</v> is <v>t</v> regardless of \u000A <v>test</v>.  If the test evaluates to <v>nil</v> (or fails to evaluate because of \u000A <see topic=\"ACL2____GUARD\">guard</see> violations) we act as though we cannot establish the hypothesis \u000A and abandon the attempt to apply the rule; it is always sound to give up.</p> \u000A \u000A <p>The acute reader will have noticed something odd about the form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> '(trn a b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> '(trn a b)) 'NORM))).</code> \u000A \u000A <p>When relieving the first hypothesis, <v>(<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)</v>, we substituted \u000A <v>(trn a b)</v> for <v>X</v>; but when relieving the second hypothesis, \u000A <v>(<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'NORM))))</v>, we substituted the \u000A quotation of <v>(trn a b)</v> for <v>X</v>.  Why the difference?  Remember that in \u000A the first hypothesis we are talking about the value of <v>(trn a b)</v> &mdash; \u000A is it rational &mdash; while in the second one we are talking about its \u000A syntactic form.  Remember also that Lisp, and hence ACL2, evaluates the \u000A arguments to a function before applying the function to the resulting \u000A values. Thus, we are asking ``Is the list <v>(trn a b)</v> a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> and \u000A if so, is its <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> the symbol <v>NORM</v>?''  The <v>quote</v>s on both \u000A <v>(trn a b)</v> and <v>NORM</v> are therefore necessary.  One can verify this by \u000A defining <v>trn</v> to be, say <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, and then evaluating forms such \u000A as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> '(trn a b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> '(trn a b)) 'NORM))\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (trn a b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (trn a b)) NORM))\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (trn 'a 'b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (trn 'a 'b)) NORM))\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> '(trn a b)) (<see topic=\"COMMON-LISP____EQ\">EQ</see> '(<see topic=\"COMMON-LISP____CAR\">CAR</see> (trn a b)) ''NORM))</code> \u000A \u000A <p>at the top-level ACL2 prompt.</p> \u000A \u000A <p>See <see topic=\"ACL2____SYNTAXP-EXAMPLES\">syntaxp-examples</see> for more examples of the use of \u000A <v>syntaxp</v>.</p> \u000A \u000A <p>An extended <v>syntaxp</v> hypothesis is similar to the simple type described \u000A above, but it uses two additional variables, <v>mfc</v> and <v>state</v>, which \u000A must not be bound by the left hand side or an earlier hypothesis of the rule. \u000A They must be the last two variables mentioned by <v>form</v>; first <v>mfc</v>, \u000A then <v>state</v>.  These two variables give access to the functions \u000A <v>mfc-</v>xxx; see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  As described there, <v>mfc</v> \u000A is bound to the so-called metafunction-context and <v>state</v> to ACL2's <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____SYNTAXP-EXAMPLES\">syntaxp-examples</see> for an example of the use of these \u000A extended <v>syntaxp</v> hypotheses.</p> \u000A \u000A <p>We conclude with an example illustrating an error that may occur if you \u000A forget that a <v>syntaxp</v> hypothesis will be evaluated in an environment \u000A where variables are bound to syntactic terms, not to values.  Consider the \u000A following <see topic=\"ACL2____STOBJ\">stobj</see> introduction (see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st\u000A  (fld1 :type (signed-byte 3) :initially 0)\u000A  fld2)</code> \u000A \u000A <p>The following <v>syntaxp</v> hypothesis is ill-formed for evaluation. \u000A Indeed, ACL2 causes an error because it anticipates that when trying to \u000A relieve the <v>syntaxp</v> hypothesis of this rule, ACL2 would be evaluating \u000A <v>(fld1 st)</v> where <v>st</v> is bound to a term, not to an actual <v>stobj</v> \u000A as required by the function <v>fld1</v>.  The error message is intended to \u000A explain this problem.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep (fld1 st)))\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (stp st)\u000A                         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> st)\u000A                              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> st) 2)\u000A                              (fld1p (<see topic=\"COMMON-LISP____CAR\">car</see> st))))))\u000A\u000AACL2 Error in ( DEFTHM BAD ...):  The form (QUOTEP (FLD1 ST)), from\u000Aa SYNTAXP hypothesis, is not suitable for evaluation in an environment\u000Awhere its variables are bound to terms.  See :DOC SYNTAXP.  Here is\u000Afurther explanation:\u000A     The form ST is being used, as an argument to a call of FLD1, where\u000Athe single-threaded object of that name is required.  But in the current\u000Acontext, the only declared stobj name is STATE.  Note:  this error\u000Aoccurred in the context (FLD1 ST).\u000A\u000ASummary\u000AForm:  ( DEFTHM BAD ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AACL2 Error in ( DEFTHM BAD ...):  See :DOC failure.\u000A\u000A******** FAILED ********\u000AACL2 !&gt;</code> \u000A \u000A <p>Presumably the intention was to rewrite the term <v>(stp st)</v> when the \u000A <v>fld1</v> component of <v>st</v> is seen to be an explicit constant.  As \u000A explained elsewhere (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), we can obtain the result of \u000A rewriting <v>(fld1 st)</v> by binding a fresh variable to that term using \u000A <v>EQUAL</v>, as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> good\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> f (fld1 st))\u000A                (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep f)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (stp st)\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> st)\u000A                       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> st) 2)\u000A                       (fld1p (<see topic=\"COMMON-LISP____CAR\">car</see> st))))))</code> \u000A \u000A <p>The event above is admitted by ACL2.  We can see it in action by disabling \u000A the definition of <v>stp</v> so that only the rule above, <v>good</v>, is \u000A available for reasoning about <v>stp</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> stp))</code> \u000A \u000A <p>Then the proof fails for the following, because the <v>syntaxp</v> hypothesis \u000A of the rule, <v>good</v>, fails: <v>(quotep f)</v> evaluates to <v>nil</v> when \u000A <v>f</v> is bound to the term <v>(fld1 st)</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (stp st))</code> \u000A \u000A <p>However, the proof succeeds for the next form, as we explain below.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (stp (<see topic=\"COMMON-LISP____LIST\">list</see> 3 rest)))</code> \u000A \u000A <p>Consider what happens in that case when rule <v>good</v> is applied to the \u000A term <v>(stp (<see topic=\"COMMON-LISP____LIST\">list</see> 3 rest))</v>.  (See <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> for relevant \u000A background.)  The first hypothesis of <v>good</v> binds the variable <v>f</v> to \u000A the result of rewriting <v>(fld1 st)</v>, where <v>st</v> is bound to the \u000A (internal form of) the term <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 3 rest)</v> &mdash; and that result is \u000A clearly the term, <v>'3</v>.  Then the <v>syntaxp</v> hypothesis is successfully \u000A relieved, because the evaluation of <v>(quotep f)</v> returns <v>t</v> in the \u000A environment that binds <v>f</v> to <v>'3</v>.</p>"],

 "ACL2____REDUNDANT-EVENTS":[["Miscellaneous","Note1"],"ACL2 Sources","ACL2","<p>Sometimes an event will announce that it is ``redundant'', meaning \u000A that the the form is not evaluated because ACL2 determines that its effect is \u000A already incorporated into the logical <see topic=\"ACL2____WORLD\">world</see>.  Thus, when this happens, \u000A no change to the logical <see topic=\"ACL2____WORLD\">world</see> takes place.  This feature permits two \u000A independent <see topic=\"ACL2____BOOKS\">books</see>, each of which defines some name, to be included \u000A sequentially provided they use exactly the same definition.</p> \u000A \u000A <p>Note that by the definition above, a form can have no effect on the logical \u000A <see topic=\"ACL2____WORLD\">world</see> yet not be considered to be redundant.  Here is an example of \u000A such a form.</p> \u000A \u000A <code>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Hello world.~%&quot;))</code> \u000A \u000A <p>When are two <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see> definitions considered ``the same''?  It \u000A depends upon the kind of name being defined.  We consider these below in \u000A alphabetical order.  See also the Notes below.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> is redundant if there is already an identical \u000A <v>defabsstobj</v> event in the logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> event is never redundant.  Note that it doesn't define \u000A any name.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event is redundant if there is already \u000A an axiom or theorem of the given name and both the formula (after \u000A macroexpansion) and the <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> are syntactically identical.  Note \u000A that a <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> can make a subsequent <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> redundant, and a \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> can make a subsequent <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> redundant as well.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> is redundant if the name is already defined either with \u000A a syntactically identical <v>defconst</v> event or one that defines it to have \u000A the same value.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFDOC\">defdoc</see></tt> event is never redundant because it doesn't define any \u000A name.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> event is never redundant.  This means that if you have a \u000A <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> in a book and that book has been included (without error), \u000A then references to that label denote the point in <see topic=\"ACL2____HISTORY\">history</see> at which the \u000A book introduced the label.  See the note about shifting logical names, \u000A below.</p> \u000A \u000A <p>A <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> event is redundant if there is already a macro defined \u000A with the same name and syntactically identical arguments, <see topic=\"ACL2____GUARD\">guard</see>, and \u000A body.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event is redundant if a package of the same name with \u000A exactly the same imports has been defined.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event is redundant if there is already a <v>defstobj</v> \u000A event with the same name that has exactly the same field descriptors (see \u000A <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>), in the same order, and with the same <v>:renaming</v> value if \u000A <v>:renaming</v> is supplied for either event.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event is redundant according to the criterion given above \u000A in the discussion of <v>defaxiom</v>.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> is never redundant.  The ``natural'' notion of \u000A equivalent <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> forms is that the names and values of the two \u000A theory expressions are the same.  But since most theory expressions are \u000A sensitive to the context in which they occur, it seems unlikely to us that two \u000A <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>s coming from two sequentially included <see topic=\"ACL2____BOOKS\">books</see> will \u000A ever have the same values.  So we prohibit redundant theory definitions.  If \u000A you try to define the same theory name twice, you will get a ``name in use'' \u000A error.</p> \u000A \u000A <p>A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>, or <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event is \u000A redundant if for each function to be introduced, there has already been \u000A introduced a function with the same name, the same formals, and syntactically \u000A identical <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt>, <v>:measure</v>, type declarations, <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt><v>s</v> declarations and <v>body</v> (before macroexpansion), and an \u000A appropriate <v>mode</v> (see the ``Note About Appropriate Modes'' below). \u000A Moreover, the order of the combined <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> and type declarations \u000A must be the same in both cases.  Exceptions:<br></br> \u000A \u000A (1) If either definition is declared <v>:non-executable</v> (see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>), then the two events must be identical.<br></br> \u000A \u000A (2) It is permissible for one definition to have a <v>:</v><see topic=\"ACL2____GUARD\">guard</see> of \u000A <v>t</v> and the other to have no explicit guard (hence, the guard is implicitly \u000A <v>t</v>).<br></br> \u000A \u000A (3) The <v>:measure</v> check is avoided if the old definition is non-recursive \u000A (and not defined within a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>) or we are skipping proofs \u000A (for example, during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>).  Otherwise, the new definition may \u000A have a <v>:measure</v> of <v>(:? v1 ... vk)</v>, where <v>(v1 ... vk)</v> enumerates \u000A the variables occurring in the measure stored for the old definition.<br></br> \u000A \u000A (4) If either the old or new event is a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event, then \u000A redundancy requires that both are <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> events that define \u000A the same set of function symbols.</p> \u000A \u000A <p>An <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event is most commonly redundant when a \u000A syntactically identical <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> has already been executed under \u000A the same <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt>, <tt><see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see></tt>, and \u000A <tt><see topic=\"ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS\">default-verify-guards-eagerness</see></tt>.  The full criterion for redundancy of \u000A <v>encapsulate</v> events is more complex, for example ignoring contents of \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____REDUNDANT-ENCAPSULATE\">redundant-encapsulate</see>.</p> \u000A \u000A <p>An <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event is never redundant.  Note that it doesn't define \u000A any name.</p> \u000A \u000A <p>An <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event is redundant if the book has already been \u000A included.</p> \u000A \u000A <p>A call of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> is never redundant, as its argument is always \u000A evaluated to obtain the make-event expansion.  However, that expansion may of \u000A course be redundant.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event is redundant according to the criteria in \u000A the discussion above for the case of a <v>defun</v> event.</p> \u000A \u000A <p>A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> event is never redundant: its subsidiary <see topic=\"ACL2____EVENTS\">events</see> are \u000A always considered.  Of course, its sub-events may themselves be redundant.  If \u000A all of its sub-events are redundant &mdash; or more generally, if none of the \u000A sub-events changes the logical <see topic=\"ACL2____WORLD\">world</see> &mdash; then the <v>progn</v> event \u000A also won't change the world.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt> event is redundant if every name supplied is \u000A already a member of the corresponding list of untouchable symbols.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____REGENERATE-TAU-DATABASE\">regenerate-tau-database</see></tt> event is never redundant.  Note that it \u000A doesn't define any name.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt> event is redundant if no name supplied is a \u000A member of the corresponding list of untouchable symbols.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt> event is redundant if it does not cause any \u000A change.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____SET-BODY\">set-body</see></tt> event is redundant if the indicated body is already the \u000A current body.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____TABLE\">table</see></tt> event not define any name.  It is redundant when it sets \u000A the value already associated with a key of the table, or when it sets an \u000A entire table (using keyword <v>:clear</v>) to its existing value; see <see topic=\"ACL2____TABLE\">table</see>.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event is redundant if the function has already had \u000A its <see topic=\"ACL2____GUARD\">guard</see>s verified.</p> \u000A \u000A <p><i>Note About Built-in (Predefined) Functions and Macros:</i></p> \u000A \u000A <p>Redundancy is restricted for built-in macros and functions that have \u000A special raw Lisp code.  Such redundancy is only legal in the context of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  This restriction is needed for soundness, for technical reasons \u000A omitted here (details may be found in a long comment about redundant-events in \u000A source function <v>chk-acceptable-defuns-redundancy</v>).</p> \u000A \u000A <p><i>Note About Appropriate Modes:</i></p> \u000A \u000A <p>Suppose a function is being redefined and that the formals, guards, types, \u000A stobjs, and bodies are identical.  When are the modes (<v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A or <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>) ``appropriate?''  Identical modes are appropriate. \u000A But what if the old mode was <v>:program</v> and the new mode is <v>:logic</v>? \u000A This is appropriate, provided the definition meets the requirements of the \u000A logical definitional principle.  That is, you may redefine ``redundantly'' a \u000A <v>:program</v> mode function as a <v>:logic</v> mode function provide the measure \u000A conjectures can be proved.  This is what <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> does.  Now \u000A consider the reverse style of redefinition.  Suppose the function was defined \u000A in <v>:logic</v> mode and is being identically redefined in <v>:program</v> mode. \u000A ACL2 will treat the redefinition as redundant, provided the appropriate \u000A criteria are met (as though it were in :logic mode).</p> \u000A \u000A <p><i>Note About Shifting Logical Names:</i></p> \u000A \u000A <p>Suppose a book defines a function <v>fn</v> and later uses <v>fn</v> as a \u000A logical name in a theory expression.  Consider the value of that theory \u000A expression in two different sessions.  In session A, the book is included in a \u000A <see topic=\"ACL2____WORLD\">world</see> in which <v>fn</v> is not already defined, i.e., in a <see topic=\"ACL2____WORLD\">world</see> \u000A in which the book's definition of <v>fn</v> is not redundant.  In session B, the \u000A book is included in a <see topic=\"ACL2____WORLD\">world</see> in which <v>fn</v> is already identically \u000A defined.  In session B, the book's definition of <v>fn</v> is redundant.  When \u000A <v>fn</v> is used as a logical name in a theory expression, it denotes the point \u000A in <see topic=\"ACL2____HISTORY\">history</see> at which <v>fn</v> was introduced.  Observe that those points \u000A are different in the two sessions.  Hence, it is likely that theory \u000A expressions involving <v>fn</v> will have different values in session A than in \u000A session B.</p> \u000A \u000A <p>This may adversely affect the user of your book.  For example, suppose your \u000A book creates a theory via <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> that is advertised just to contain \u000A the names generated by the book.  But suppose you compute the theory as the \u000A very last event in the book using:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)\u000A                         (<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> fn))</code> \u000A \u000A <p>where <v>fn</v> is the very first event in the book and happens to be a \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event.  This expression returns the advertised set if <v>fn</v> is \u000A not already defined when the book is included.  But if <v>fn</v> were previously \u000A (identically) defined, the theory is larger than advertised.</p> \u000A \u000A <p>The moral of this is simple: when building <see topic=\"ACL2____BOOKS\">books</see> that other people \u000A will use, it is best to describe your <see topic=\"ACL2____THEORIES\">theories</see> in terms of logical \u000A names that will not shift around when the <see topic=\"ACL2____BOOKS\">books</see> are included.  The best \u000A such names are those created by <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>.</p> \u000A \u000A <p><i>Note About Unfortunate Redundancies.</i></p> \u000A \u000A <p>Notice that our syntactic criterion for redundancy of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> may not allow redefinition to take effect unless there is a syntactic \u000A change in the definition.  The following example shows how an attempt to \u000A redefine a function can fail to make any change.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:warn . :overwrite) state)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'car x))\u000A(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> nil state)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x)) ; redundant, unfortunately; foo does not change\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo 3) 3)) ; succeeds, showing that redef of foo didn't happen</code> \u000A \u000A <p>The call of macro <v>mac</v> was expanded away before storing the first \u000A definition of <v>foo</v> for the theorem prover.  Therefore, the new definition \u000A of <v>mac</v> does not affect the expansion of <v>foo</v> by the theorem prover, \u000A because the new definition of <v>foo</v> is ignored.</p> \u000A \u000A <p>One workaround is first to supply a different definition of <v>foo</v>, just \u000A before the last definition of <v>foo</v> above.  Then that final definition will \u000A no longer be redundant.  However, as a courtesy to users, we strengthen the \u000A redundancy check for function definitions when redefinition is active.  If in \u000A the example above we remove the form <v>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> nil\u000A state)</v>, then the problem goes away:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:warn . :overwrite) state)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'car x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x)) ; no longer redundant\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo 3) 3)) ; fails, as we would like</code> \u000A \u000A <p>To summarize: If a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form is submitted that meets the usual \u000A redundancy criteria, then it may be considered redundant even if a macro \u000A called in the definition has since been redefined.  The analogous problem \u000A applie to constants, i.e., symbols defined by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> that occur in \u000A the definition body.  However, if redefinition is currently active the problem \u000A goes away: that is, the redundancy check is strengthened to check the \u000A ``translated'' body, in which macro calls and constants defined by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> are expanded away.</p> \u000A \u000A <p>The above discussion for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> forms applies to <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> \u000A forms as well.  However, for <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> forms ACL2 always checks \u000A translated bodies, so such bogus redundancy does not occur.</p> \u000A \u000A <p>Here is more complex example illustrating the limits of redefinition, based \u000A on one supplied by Grant Passmore.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> n3 () 0)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> n4 () 1)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> n5 () (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (n3) (n4))) ; body is normalized to nil\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (n5) nil)) ; succeeds, trivially\u000A(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:warn . :overwrite) state)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> n3 () 2)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (n5) nil)) ; still succeeds, sadly</code> \u000A \u000A <p>We may expect the final <tt><see topic=\"ACL2____THM\">thm</see></tt> call to fail because of the following \u000A reasoning: <v>(n5)</v> = <v>(<see topic=\"COMMON-LISP_____E3\">&gt;</see> (n3) (n4))</v> = <v>(<see topic=\"COMMON-LISP_____E3\">&gt;</see> 2 1)</v> = <v>t</v>. \u000A Unfortunatly, the body of <v>n5</v> was simplified (``normalized'') to <v>nil</v> \u000A when <v>n5</v> was admitted, so the redefinition of <v>n3</v> is ignored during \u000A the final <v>thm</v> call.  (Such normalization can be avoided; see the brief \u000A discussion of <v>:normalize</v> in the documentation for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.)  So, \u000A given this unfortunate situation, one might expect at this point simply to \u000A redefine <v>n5</v> using the same definition as before, in order to pick up the \u000A new definition of <v>n3</v>.  Such ``redefinition'' would, however, be \u000A redundant, for the same reason as in the previous example: no syntactic change \u000A was made to the definition.  Even with redefinition active, there is no change \u000A in the body of <v>n5</v>, even with macros and constants (defined by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>) expanded; there are none such!  The same workaround applies as \u000A before: redefine <v>n5</v> to be something different, and then redefine <v>n5</v> \u000A again to be as desired.</p> \u000A \u000A <p>A related phenomenon can occur for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  As explained \u000A above, an <v>encapsulate</v> event is redundant if it is identical to one \u000A already in the database.  (But a weaker condition applies in general; see \u000A <see topic=\"ACL2____REDUNDANT-ENCAPSULATE\">redundant-encapsulate</see>.)  Consider then the following contrived \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) x)\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x)))\u000A(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:warn . :overwrite) state)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'car x))\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (mac x)))</code> \u000A \u000A <p>The last <v>encapsulate</v> event is redundant because it meets the criterion \u000A for redundancy: it is identical to the earlier <v>encapsulate</v> event.  Even \u000A though redefinition is active, and hence ACL2 ``should'' be able to see that \u000A the new <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>foo</v> is not truly redundant, nevertheless the \u000A criterion for redundancy of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> allows the new \u000A <v>encapsulate</v> form to be redundant.</p> \u000A \u000A <p>A workaround can be to add something trivial to the <v>encapsulate</v>, for \u000A example:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A  (<see topic=\"ACL2____DEFLABEL\">deflabel</see> try2) ; ``Increment'' to try3 next time, and so on.\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x))</code>"],

 "ACL2____LOOP-STOPPER":[["Rewrite"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a discussion of the syntax of the \u000A <v>:loop-stopper</v> field of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule-classes.  Here we \u000A describe how that field is used, and also how that field is created when the \u000A user does not explicitly supply it.</p> \u000A \u000A <p>For example, the built-in <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule \u000A <v>commutativity-of-+</v>,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A              (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x))),</code> \u000A \u000A <p>creates a rewrite rule with a loop-stopper of <v>((x y binary-+))</v>.  This \u000A means, very roughly, that the term corresponding to <v>y</v> must be ``smaller'' \u000A than the term corresponding to <v>x</v> in order for this rule to apply. \u000A However, the presence of <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> in the list means that certain \u000A functions that are ``invisible'' with respect to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> (by default, \u000A <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> is the only such function) are more or less ignored when \u000A making this ``smaller'' test.  We are much more precise below.</p> \u000A \u000A <p>Our explanation of loop-stopping is in four parts.  First we discuss ACL2's \u000A notion of ``term order.''  Next, we bring in the notion of ``invisibility'', \u000A and use it together with term order to define orderings on terms that are used \u000A in the loop-stopping algorithm.  Third, we describe that algorithm.  These \u000A topics all assume that we have in hand the <v>:loop-stopper</v> field of a given \u000A rewrite rule; the fourth and final topic describes how that field is \u000A calculated when it is not supplied by the user.</p> \u000A \u000A <p>ACL2 must sometimes decide which of two terms is syntactically simpler.  It \u000A uses a total ordering on terms, called the ``term order.''  Under this \u000A ordering constants such as <v>'(a b c)</v> are simpler than terms containing \u000A variables such as <v>x</v> and <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 x)</v>.  Terms containing variables are \u000A ordered according to how many occurrences of variables there are.  Thus <v>x</v> \u000A and <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 x)</v> are both simpler than <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x x)</v> and <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y)</v>.  If \u000A variable counts do not decide the order, then the number of function \u000A applications are tried.  Thus <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x x)</v> is simpler than <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP_____B2\">+</see> 1\u000A y))</v> because the latter has one more function application.  Finally, if the \u000A number of function applications do not decide the order, a lexicographic \u000A ordering on Lisp objects is used.  See <see topic=\"ACL2____TERM-ORDER\">term-order</see> for details.</p> \u000A \u000A <p>When the loop-stopping algorithm is controlling the use of permutative \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules it allows <v>term1</v> to be moved leftward over \u000A <v>term2</v> only if <v>term1</v> is smaller, in a suitable sense.  Note: The \u000A sense used in loop-stopping is <b>not</b> the above explained term order but a \u000A more complicated ordering described below.  The use of a total ordering stops \u000A rules like commutativity from looping indefinitely because it allows <v>(<see topic=\"COMMON-LISP_____B2\">+</see> b\u000A a)</v> to be permuted to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b)</v> but not vice versa, assuming <v>a</v> is \u000A smaller than <v>b</v> in the ordering.  Given a set of permutative rules that \u000A allows arbitrary permutations of the tips of a tree of function calls, this \u000A will normalize the tree so that the smallest argument is leftmost and the \u000A arguments ascend in the order toward the right.  Thus, for example, if the \u000A same argument appears twice in the tree, as <v>x</v> does in the <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> tree denoted by the term <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a x b x)</v>, then when the allowed \u000A permutations are done, all occurrences of the duplicated argument in the tree \u000A will be adjacent, e.g., the tree above will be normalized to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b x\u000A x)</v>.</p> \u000A \u000A <p>Suppose the loop-stopping algorithm used term order, as noted above, and \u000A consider the <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> tree denoted by <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y (<see topic=\"COMMON-LISP____-\">-</see> x))</v>.  The \u000A arguments here are in ascending term order already.  Thus, no permutative \u000A rules are applied.  But because we are inside a <v>+-expression</v> it is very \u000A convenient if <v>x</v> and <v>(<see topic=\"COMMON-LISP____-\">-</see> x)</v> could be given virtually the same position \u000A in the ordering so that <v>y</v> is not allowed to separate them.  This would \u000A allow such rules as <v>(<see topic=\"COMMON-LISP_____B2\">+</see> i (<see topic=\"COMMON-LISP____-\">-</see> i) j) = j</v> to be applied.  In support of this, \u000A the ordering used in the control of permutative rules allows certain unary \u000A functions, e.g., the unary minus function above, to be ``invisible'' with \u000A respect to certain ``surrounding'' functions, e.g., <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> function \u000A above.</p> \u000A \u000A <p>Briefly, a unary function symbol <v>fn1</v> is invisible with respect to a \u000A function symbol <v>fn2</v> if <v>fn2</v> belongs to the value of <v>fn1</v> in \u000A <tt><see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see></tt>; also see <see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see>, which \u000A explains its format and how it can be set by the user.  Roughly speaking, \u000A ``invisible'' function symbols are ignored for the purposes of the term-order \u000A test.</p> \u000A \u000A <p>Consider the example above, <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y (<see topic=\"COMMON-LISP____-\">-</see> x))</v>.  The translated version of \u000A this term is <v>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> y (<see topic=\"ACL2____UNARY--\">unary--</see> x)))</v>.  The initial <tt><see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see></tt> makes <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> invisible with repect to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>.  The commutativity rule for <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> will attempt to swap \u000A <v>y</v> and <v>(<see topic=\"ACL2____UNARY--\">unary--</see> x)</v> and the loop-stopping algorithm is called to \u000A approve or disapprove.  If term order is used, the swap will be disapproved. \u000A But term order is not used.  While the loop-stopping algorithm is permuting \u000A arguments inside a <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> expression, <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> is invisible. \u000A Thus, insted of comparing <v>y</v> with <v>(<see topic=\"ACL2____UNARY--\">unary--</see> x)</v>, the loop-stopping \u000A algorithm compares <v>y</v> with <v>x</v>, approving the swap because <v>x</v> comes \u000A before <v>y</v>.</p> \u000A \u000A <p>Here is a more precise specification of the total order used for \u000A loop-stopping with respect to a list, <v>fns</v>, of functions that are to be \u000A considered invisible.  Let <v>x</v> and <v>y</v> be distinct terms; we specify \u000A when ``<v>x</v> is smaller than <v>y</v> with respect to <v>fns</v>.''  If <v>x</v> is \u000A the application of a unary function symbol that belongs to <v>fns</v>, replace \u000A <v>x</v> by its argument.  Repeat this process until the result is not the \u000A application of such a function; let us call the result <v>x-guts</v>.  Similarly \u000A obtain <v>y-guts</v> from <v>y</v>.  Now if <v>x-guts</v> is the same term as \u000A <v>y-guts</v>, then <v>x</v> is smaller than <v>y</v> in this order iff <v>x</v> is \u000A smaller than <v>y</v> in the standard term order.  On the other hand, if \u000A <v>x-guts</v> is different than <v>y-guts</v>, then <v>x</v> is smaller than <v>y</v> \u000A in this order iff <v>x-guts</v> is smaller than <v>y-guts</v> in the standard term \u000A order.</p> \u000A \u000A <p>Now we may describe the loop-stopping algorithm.  Consider a rewrite rule \u000A with conclusion <v>(equiv lhs rhs)</v> that applies to a term <v>x</v> in a given \u000A context; see <see topic=\"ACL2____REWRITE\">rewrite</see>.  Suppose that this rewrite rule has a \u000A loop-stopper field (technically, the <v>:heuristic-info</v> field) of <v>((x1 y1\u000A . fns-1) ... (xn yn . fns-n))</v>.  (Note that this field can be observed by \u000A using the command <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> with the name of the rule; see <see topic=\"ACL2____PR\">pr</see>.) \u000A We describe when rewriting is permitted.  The simplest case is when the \u000A loop-stopper list is <v>nil</v> (i.e., <v>n</v> is <v>0</v>); in that case, \u000A rewriting is permitted.  Otherwise, for each <v>i</v> from 1 to <v>n</v> let \u000A <v>xi'</v> be the actual term corresponding to the variable <v>xi</v> when \u000A <v>lhs</v> is matched against the term to be rewritten, and similarly correspond \u000A <v>yi'</v> with <v>y</v>.  If <v>xi'</v> and <v>yi'</v> are the same term for all \u000A <v>i</v>, then rewriting is not permitted.  Otherwise, let <v>k</v> be the least \u000A <v>i</v> such that <v>xi'</v> and <v>yi'</v> are distinct.  Let <v>fns</v> be the list \u000A of all functions that are invisible with respect to every function in \u000A <v>fns-k</v>, if <v>fns-k</v> is non-empty; otherwise, let <v>fns</v> be <v>nil</v>. \u000A Then rewriting is permitted if and only if <v>yi'</v> is smaller than <v>xi'</v> \u000A with respect to <v>fns</v>, in the sense defined in the preceding paragraph.</p> \u000A \u000A <p>It remains only to describe how the loop-stopper field is calculated for a \u000A rewrite rule when this field is not supplied by the user.  (On the other hand, \u000A to see how the user may specify the <v>:loop-stopper</v>, see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.)  Suppose the conclusion of the rule is of the form <v>(equiv\u000A lhs rhs)</v>.  First of all, if <v>rhs</v> is not an instance of the left hand \u000A side by a substitution whose range is a list of distinct variables, then the \u000A loop-stopper field is <v>nil</v>.  Otherwise, consider all pairs <v>(<see topic=\"ACL2____U\">u</see> . v)</v> \u000A from this substitution with the property that the first occurrence of <v>v</v> \u000A appears in front of the first occurrence of <v>u</v> in the print representation \u000A of <v>rhs</v>.  For each such <v>u</v> and <v>v</v>, form a list <v>fns</v> of all \u000A functions <v>fn</v> in <v>lhs</v> with the property that <v>u</v> or <v>v</v> (or \u000A both) appears as a top-level argument of a subterm of <v>lhs</v> with function \u000A symbol <v>fn</v>.  Then the loop-stopper for this rewrite rule is a list of all \u000A lists <v>(<see topic=\"ACL2____U\">u</see> v . fns)</v>.</p>"],

 "ACL2____GUARD":[["Miscellaneous","Note1"],"ACL2 Sources","ACL2","<p>The ACL2 system provides a mechanism for restricting a function to \u000A a particular domain.  Such restrictions are called ``guards.''  A definition's \u000A guard has no effect on the logical axiom added when that definition is \u000A accepted by ACL2, and novices are often best served by avoiding guards. \u000A However, guards can be useful as a specification device or for increasing \u000A execution efficiency.  To make such a restriction, use the <v>:guard</v> \u000A keyword (see <see topic=\"ACL2____XARGS\">xargs</see>) or, especially if you want the host Lisp compiler \u000A to use this information, use <v>type</v> declarations (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>; also \u000A see <see topic=\"ACL2____XARGS\">xargs</see> for a discussion of the <v>split-types</v> keyword).  The \u000A general issue of guards and guard verification is discussed in the topics \u000A listed below.</p> \u000A \u000A <p>To begin further discussion of guards, see <see topic=\"ACL2____GUARD-INTRODUCTION\">guard-introduction</see>.  That \u000A section directs the reader to further sections for more details.  To see just \u000A a summary of some <see topic=\"ACL2____COMMAND\">command</see>s related to guards, see <see topic=\"ACL2____GUARD-QUICK-REFERENCE\">guard-quick-reference</see>.  For a discussion of the use of proof to verify the \u000A absence of guard violations, see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.</p>"],

 "ACL2____DEFABSSTOBJ":[["Events","Stobj"],"ACL2 Sources","ACL2","<p>We assume familiarity with single-threaded objects; see <see topic=\"ACL2____STOBJ\">stobj</see> and see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.  The event <v>defabsstobj</v> defines a \u000A so-called ``abstract stobj'', a notion we introduce briefly now and then \u000A explain in more depth below.</p> \u000A \u000A <p>The evaluation of a <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> event produces logical definitions for \u000A several functions: a recognizer, which characterizes the <see topic=\"ACL2____STOBJ\">stobj</see> in terms \u000A of lists; a creator, which produces an initial suitable list structure; and \u000A field accessors and updators, defined in terms of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> and <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt>.  <v>Defabsstobj</v> provides a way to define alternate definitions \u000A for ``stobj primitives'' for a corresponding single-threaded object.  These \u000A stobj primitives include a recognizer, a creator, and other ``exported'' \u000A functions.  In essence, <v>defabsstobj</v> establishes interface functions, or \u000A ``exports'', on a new stobj that is a copy of an indicated ``concrete'' stobj \u000A that already exists.</p> \u000A \u000A <p>We begin below with an introduction to abstract <see topic=\"ACL2____STOBJ\">stobj</see>s.  We then \u000A explain the <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> event by way of an example.  We conclude by \u000A giving summary documentation for the <v>defabsstobj</v> event.</p> \u000A \u000A <p>For another introduction to abstract stobjs, see the paper ``Abstract \u000A Stobjs and Their Application to ISA Modeling'' by Shilpi Goel, Warren A. Hunt, \u000A Jr., and Matt Kaufmann, in the proceedings of <a \u000A href='http://www.cs.uwyo.edu/~ruben/acl2-13'>ACL2 Workshop 2013</a>.</p> \u000A \u000A <p><b>INTRODUCTION</b></p> \u000A \u000A <p>We start with a brief review of <see topic=\"ACL2____STOBJ\">stobj</see>s and some potential problems \u000A with them, followed by an introduction to abstract stobjs and how they can \u000A avoid these problems.  Prior experience with stobjs will probably help the \u000A reader to absorb the ideas below.</p> \u000A \u000A <p>Recall that single-threaded objects, or <see topic=\"ACL2____STOBJ\">stobj</see>s, provide a way for \u000A ACL2 users to stay within the ACL2 logic, where every data object is an atom \u000A or a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> of data objects, while obtaining the benefits of fast \u000A evaluation through destructive updates.  Consider for example this very simple \u000A event.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)</code> \u000A \u000A <p>This event introduces a recognizer, <v>stp</v>, and a creator, \u000A <v>create-st</v>, for a data structure consisting of a single field accessed and \u000A updated by functions <v>fld</v> and <v>update-fld</v>, respectively.  Each of \u000A these four primitive functions has both a logical definition, which is used \u000A when the prover reasons about the function, and an executable definition, \u000A which is used in raw Lisp.  In the logic, <v>stp</v> recognizes objects that \u000A have the requisite fields.  In raw Lisp, there is a ``live stobj'', which is \u000A an array object whose fields correspond to those specified by the <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event, implemented as Lisp arrays.</p> \u000A \u000A <p>Here are the logical definition and the executable definition, \u000A respectively, that are introduced for the field accessor, <v>fld</v>, introduced \u000A above.  Notice that since a stobj is represented in raw Lisp using an array, \u000A the raw Lisp accessor uses a raw Lisp array accessor, <v>svref</v>.  (You can \u000A see all the logical and executable definitions by evaluating the form \u000A <v>(<see topic=\"ACL2____TRACE_42\">trace$</see> defstobj-axiomatic-defs defstobj-raw-defs)</v> before evaluating the \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> form.)</p> \u000A \u000A <code>; logical definition\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fld (st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (stp st)\u000A                  :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____NTH\">nth</see> 0 st))\u000A\u000A; executable (raw Lisp) definition\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fld (st)\u000A  (svref st 0))</code> \u000A \u000A <p>Sophisticated programming with stobjs can provide efficient implementations \u000A of algorithms, but may require the preservation of a complex invariant.  One \u000A can, of course, define a function to implement such an invariant after \u000A introducing the stobj, as follows.</p> \u000A \u000A <code>; Introduce a stobj.\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld1 ... fldk)\u000A\u000A; Define an invariant on that stobj.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> good-stp (st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs st))\u000A  ...)\u000A\u000A; Define some basic functions that update the stobj and preserve the\u000A; invariant.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-st (... st ...)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs st\u000A                  :guard (<see topic=\"COMMON-LISP____AND\">and</see> (good-stp st) ...)))\u000A  ...)\u000A...\u000A\u000A; Prove that the invariant is indeed preserved by those basic functions.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> good-stp-update-st\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (good-stp st)\u000A                ...)\u000A           (good-stp (update-st ... st ...))))\u000A...\u000A\u000A; Implement algorithms built on the basic functions.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (... st ...)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs st\u000A                  :guard (<see topic=\"COMMON-LISP____AND\">and</see> (good-stp st) ...)))\u000A  ... (update-st ... st ...) ...)\u000A\u000A; Prove invariance theorems about these algorithms.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> good-stp-foo\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (good-stp st)\u000A                ...)\u000A           (good-stp (foo ... st ...))))\u000A...\u000A\u000A; Prove other properties of these algorithms.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-is-correct\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (good-stp st)\u000A                ...)\u000A           (some-property (foo ... st ...))))\u000A...</code> \u000A \u000A <p>But there are at least two potential difficulties in using stobjs as \u000A described above.</p> \u000A \u000A <blockquote><p>1. When <v>foo</v> is executed on concrete data in the ACL2 loop, \u000A the guard check may be expensive because <v>(good-stp st)</v> is expensive.</p> \u000A \u000A <p>2. Reasoning about <v>foo</v> (using rules like <v>foo-is-correct</v> above) \u000A involves proving hypotheses of invariance theorems, which may be complicated \u000A for the user to manage or slow for the theorem prover.</p></blockquote> \u000A \u000A <p>The <v>defabsstobj</v> event offers an opportunity to address these issues. \u000A It introduces a new stobj, which we call an ``abstract stobj'', which is \u000A associated with a corresponding ``concrete stobj'' introduced by an earlier \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event.  The <v>defabsstobj</v> event specifies a logical \u000A (<v>:LOGIC</v>) and an executable (<v>:EXEC</v>) definition for each primitive \u000A operation, or ``stobj primitive'', involving that stobj.  As is the case for \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>, the logical definition is what ACL2 reasons about, and is \u000A appropriate to apply to an ACL2 object satisfying the logical definition of \u000A the recognizer function for the stobj.  The executable definition is applied \u000A in raw Lisp to a live stobj, which is an array object associated with the \u000A given stobj name.</p> \u000A \u000A <p>We can picture a sequence of updates to corresponding abstract and concrete \u000A stobjs as follows.  Initially in this picture, <v>st$a0</v> and <v>st$c0</v> are a \u000A corresponding abstract and concrete stobj (respectively).  Then an update, \u000A <v>u1</v>, is applied with <v>:LOGIC</v> and <v>:EXEC</v> functions <v>u$a1</v> and \u000A <v>u$c1</v>, respectively.  The resulting abstract and concrete stobj, \u000A <v>st$a1</v> and <v>st$c1</v>, correspond as before.  Then a second update, \u000A <v>u2</v>, is applied with <v>:LOGIC</v> and <v>:EXEC</v> functions <v>u$a2</v> and \u000A <v>u$c2</v>, respectively &mdash; again preserving the correspondence.  And so \u000A on.</p> \u000A \u000A <code>Abstract               u$a1       u$a2       u$a3\u000A(:logic)         st$a0  --&gt; st$a1  --&gt; st$a2  --&gt;   ...\u000A\u000A                   ^          ^          ^               ^\u000ACorrespondence     |          |          |          ...  |\u000A                   v          v          v               v\u000A\u000A                       u$c1       u$c2       u$c3\u000AConcrete         st$c0  --&gt; st$c1  --&gt; st$c2  --&gt;   ...\u000A(:exec)</code> \u000A \u000A <p>We conclude this introduction with some remarks about implementation. \u000A Consider an abstract stobj <v>st</v> with corresponding concrete stobj \u000A <v>st$c</v>.  The live stobjs for <v>st</v> and <v>st$c</v> have the same structure, \u000A but are distinct arrays.  Indeed, the raw Lisp creator function for <v>st$c</v> \u000A is called to create a new initial live stobj for <v>st</v>.  As we will see \u000A below, reads and writes in raw Lisp to the live stobj for <v>st</v> are \u000A ultimately performed using the primitive accessors and updaters defined for \u000A <v>st$c</v>.  One might think of the live stobjs for <v>st</v> and <v>st$c</v> as \u000A being congruent stobjs (see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>), except that the stobjs \u000A themselves are not congruent: the stobj primitives introduced for <v>st</v> may \u000A be applied to <v>st</v> but not arbitrary field updaters of <v>st$c</v>, for \u000A example.  As one might expect, the <v>:EXEC</v> function for an exported \u000A function is applied to the live stobj for <v>st</v> in raw Lisp.</p> \u000A \u000A <p><b>EXAMPLE</b></p> \u000A \u000A <p>We present examples, with detailed comments intended to explain abstract \u000A stobjs, in two community books: <v>books/misc/defabsstobj-example-1.lisp</v> and \u000A <v>books/misc/defabsstobj-example-2.lisp</v>.  In this section we outline the \u000A first of these.  We suggest that after you finish this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A topic, you read through those two books.</p> \u000A \u000A <p>Here is the first of two closely related <v>defabsstobj</v> <see topic=\"ACL2____EVENTS\">events</see> \u000A from the book <v>defabsstobj-example-1.lisp</v>, but in expanded form.  We will \u000A show the abbreviated form later, which omits most of data in the form that is \u000A immediately below.  Thus most of the information shown here is default \u000A information.  We believe that the comments below explain most or all of what \u000A you need to know in order to start using <v>defabsstobj</v>, and that you will \u000A learn the remainder when you see error messages.  For example, we do not say \u000A in the comments below that every <v>:LOGIC</v> and <v>:EXEC</v> function must be \u000A <see topic=\"ACL2____GUARD\">guard</see>-verified, but that is indeed a requirement.</p> \u000A \u000A <code>(defabsstobj st ; The new abstract stobj is named st. \u000A \u000A; The concrete stobj corresponding to st is st$c: \u000A \u000A  :concrete st$c \u000A \u000A; The recognizer for the new abstract stobj is stp, which is defined to be \u000A; st$ap in the logic, and is executed on the live stobj in raw Lisp using \u000A; st$cp. \u000A \u000A  :recognizer (stp :logic st$ap :exec st$cp) \u000A \u000A; The initial stobj is defined as create-st (a function of no arguments), \u000A; which is defined logically as create-st$a, though create-st$c is invoked to \u000A; create the initial live stobj for st.  The :correspondence and :preserved \u000A; keywords refer to proof obligations, discussed below. \u000A \u000A  :creator (create-st :logic create-st$a :exec create-st$c \u000A                      :correspondence create-st{correspondence} \u000A                      :preserved create-st{preserved}) \u000A \u000A; Proof obligations are generated that involve a correspondence between the \u000A; new abstract stobj and corresponding concrete stobj.  The function \u000A; st$corr, which need not be executable (see :DOC defun-nx), takes two \u000A; arguments, a concrete stobj and an abstract stobj.  This function symbol is \u000A; used in the statements of the proof obligations. \u000A \u000A  :corr-fn st$corr \u000A \u000A; In this example we have four exports.  In each case a new function is \u000A; introduced that has the same signature as its :EXEC function, except that \u000A; st$c is replaced by st.  The :LOGIC and :EXEC functions are as specified, \u000A; and the other keywords refer to proof obligations that we discuss below. \u000A \u000A  :exports ((lookup :logic lookup$a \u000A                    :exec mem$ci \u000A                    :correspondence lookup{correspondence} \u000A                    :guard-thm lookup{guard-thm}) \u000A            (update :logic update$a \u000A                    :exec update-mem$ci \u000A                    :correspondence update{correspondence} \u000A                    :preserved update{preserved} \u000A                    :guard-thm update{guard-thm}) \u000A            (misc :logic misc$a \u000A                  :exec misc$c \u000A                  :correspondence misc{correspondence}) \u000A            (update-misc :logic update-misc$a \u000A                         :exec update-misc$c \u000A                         :correspondence update-misc{correspondence} \u000A                         :preserved update-misc{preserved})) \u000A  :doc nil) \u000A</code> \u000A \u000A <p>Note that all stobj primitives (recognizer, creator, and exported \u000A functions) are defined in the ACL2 loop in terms of their <v>:LOGIC</v> \u000A functions and in raw Lisp in terms of their <v>:EXEC</v> functions.  In the ACL2 \u000A loop, a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form defines a function, while in raw Lisp, a <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> form defines a macro (for efficiency).  We first illustrate how that \u000A works for the recognizer.  (You can see all the logical and executable \u000A definitions by evaluating the form <v>(<see topic=\"ACL2____TRACE_42\">trace$</see> defabsstobj-axiomatic-defs\u000A defabsstobj-raw-defs)</v> before evaluating the <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> form.)</p> \u000A \u000A <code>; In the ACL2 loop:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> stp (st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard 't))\u000A  (st$ap st))\u000A\u000A; In raw Lisp:\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> stp (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args) (<see topic=\"COMMON-LISP____CONS\">cons</see> 'st$cp args))</code> \u000A \u000A <p>The definitions are made similarly for exported functions, with <see topic=\"ACL2____GUARD\">guard</see>s derived from their <v>:LOGIC</v> functions as follows.  Consider the \u000A exported function <v>update</v> in our example.  Its <v>:LOGIC</v> function, \u000A <v>update$a</v>, has formals <v>(k val st$a)</v> and the following guard.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> k) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 k) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k 49))\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> val) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 val))\u000A     (st$ap st$a)\u000A     (mem$c-entryp val))</code> \u000A \u000A <p>The formals of <v>update</v> are obtained by starting with the formals of its \u000A <v>:EXEC</v> function, <v>update-mem$ci</v> &mdash; which are <v>(i v st$c)</v> \u000A &mdash; and replacing the concrete stobj name <v>st$c</v> by the new stobj name \u000A <v>st</v>.  The formals of <v>update</v> are thus <v>(i v st)</v>.  The guard for \u000A <v>update</v> is obtained in two steps.  The first step is to substitute the \u000A formals of <v>update</v> for the formals of <v>update$a</v> in the guard for \u000A <v>update$a</v>, to obtain the following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49))\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v))\u000A     (st$ap st)\u000A     (mem$c-entryp v))</code> \u000A \u000A <p>The second step is to replace, for each new stobj primitive <v>p</v>, the \u000A <v>:LOGIC</v> function for <v>p</v> by <v>p</v> itself.  The only <v>:LOGIC</v> \u000A function occurring in the formula just above is <v>st$ap</v>, which is the \u000A <v>:LOGIC</v> funcction for <v>stp</v>.  The guard for <v>update</v> is thus as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49))\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v))\u000A     (stp st)\u000A     (mem$c-entryp v))</code> \u000A \u000A <p>We turn now to the proof obligations, as promised above.  There are three \u000A types: <v>:CORRESPONDENCE</v>, <v>:PRESERVED</v>, and <v>:GUARD-THM</v>.  All \u000A required lemmas may be printed simply by defining the necessary <v>:LOGIC</v> \u000A and <v>:EXEC</v> functions and then submitting the <v>defabsstobj</v> event.  (To \u000A advanced users: also see <see topic=\"ACL2____DEFABSSTOBJ-MISSING-EVENTS\">defabsstobj-missing-events</see> for a utility that \u000A returns the required formulas in translated form.)  Although the \u000A <v>defabsstobj</v> event will fail if the required lemmas have not been proved, \u000A first it will print the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> forms that must be admitted in order to \u000A complete submission of the <v>defabsstobj</v> event.  (Note that although the \u000A those theorems are stated exactly in the form expected by the system, you are \u000A welcome to supply whatever <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> you prefer, even though \u000A the system creates <v>:rule-classes nil</v> by default.)</p> \u000A \u000A <p>The detailed theory explaining the need for these lemmas may be found in a \u000A comment in ACL2 source file <v>other-events.lisp</v>, in a comment entitled \u000A ``Essay on the Correctness of Abstract Stobjs''.  Here, we give an informal \u000A sense of the importance of these lemmas as we present examples of them. \u000A Fundamental is the notion of evaluation in the logic versus evaluation using \u000A live stobjs, where one imagines tracking the current value of each abstract \u000A stobj during each of these two evaluations.</p> \u000A \u000A <p>We start with the <v>:CORRESPONDENCE</v> lemmas.  These guarantee that \u000A evaluation in the logic agrees with evaluation using live stobjs, in the sense \u000A that the only difference is between a logical stobj and a live stobj, where \u000A the two correspond in the sense of the function specified by <v>:CORR-FN</v>. \u000A We start with the <v>:CREATOR</v> function where the statement is quite simple, \u000A stating that the <v>:CORR-FN</v> holds initially.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> create-st{correspondence}\u000A  (st$corr (create-st$c) (create-st$a)))</code> \u000A \u000A <p>For the exported functions, there are essentially two cases.  If an \u000A exported function returns other than the new abstract stobj, then the theorem \u000A asserts the equality of the results of applying the <v>:LOGIC</v> and <v>:EXEC</v> \u000A functions for the exported function.  Hypotheses include the <v>:CORR-FN</v> \u000A correspondence followed by the <see topic=\"ACL2____GUARD\">guard</see> for the <v>:LOGIC</v> function, \u000A which is stated in terms of the formal parameters of the <v>:EXEC</v> function \u000A except using the abstract stobj (here, <v>st</v>) in place of the concrete stobj \u000A (here, <v>st$c</v>).  The conclusion uses the <v>:EXEC</v> formals, modified in \u000A the call of the <v>:LOGIC</v> function (here, <v>lookup$a</v>) to use the abstract \u000A stobj, as in the hypotheses.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lookup{correspondence}\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (st$corr st$c st)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49)\u000A                (st$ap st))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (mem$ci i st$c)\u000A                  (lookup$a i st)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>By contrast, if the exported function returns the new abstract stobj, then \u000A the conclusion uses the correspondence function insted of <v>EQUAL</v>, as in \u000A the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> update{correspondence}\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (st$corr st$c st)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v)\u000A                (st$ap st)\u000A                (mem$c-entryp v))\u000A           (st$corr (update-mem$ci i v st$c)\u000A                    (update$a i v st)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>For exported functions that return multiple values, such conclusions are \u000A conjoined together over the returned values.</p> \u000A \u000A <p>The <v>:PRESERVED</v> lemmas guarantee that updates to the abstract stobj \u000A preserve its recognizer.  The fact that every exported function has this \u000A property provides justification for an optimization performed by ACL2 during \u000A generation of proof obligations for <see topic=\"ACL2____GUARD\">guard</see> verification, by assuming \u000A that the recognizer always holds.  The <v>:PRESERVED</v> lemma for the \u000A <v>:CREATOR</v> shows that the recognizer holds initially.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> create-st{preserved}\u000A  (st$ap (create-st$a)))</code> \u000A \u000A <p>Here is a typical such lemma, for the exported function <v>update</v>.  Note \u000A that there is no such lemma for <v>lookup</v>, since <v>lookup</v> does not return \u000A <v>st</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> update{preserved}\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v)\u000A                (st$ap st)\u000A                (mem$c-entryp v))\u000A           (st$ap (update$a i v st))))</code> \u000A \u000A <p>Finally, we consider the <v>:GUARD-THM</v> lemmas.  These serve to guarantee \u000A that the <see topic=\"ACL2____GUARD\">guard</see> holds for each call of an <v>:EXEC</v> function.  During \u000A guard verification, logical definitions are used; in particular, since each \u000A exported function is defined in the logic as the corresponding call of its \u000A <v>:LOGIC</v> function, guard verification shows that each call of the \u000A <v>:LOGIC</v> function for an exported function satisfies that function's guard. \u000A But why is this true for raw Lisp evaluation using live stobjs, where the \u000A <v>:EXEC</v> function is called for an exported function?  The <v>:GUARD-THM</v> \u000A lemmas provide the answer, as they state that if the <v>:LOGIC</v> function's \u000A guard holds, then the <v>:EXEC</v> function's guard holds.  Here is an example. \u000A Note that the hypotheses come from the correspondence of the concrete and \u000A abstract function as guaranteed by the <v>:CORR</v> function, together with the \u000A guard of the <v>:LOGIC</v> function; and the conclusion comes from the guard of \u000A the <v>:EXEC</v> function.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lookup{guard-thm}\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (st$corr st$c c)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i 49)\u000A                (st$ap st))\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (mem$c-length st$c))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>We conclude this EXAMPLE section by showing a short form for the \u000A <v>defabsstobj</v> form displayed above.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see> st\u000A  :exports ((lookup :exec mem$ci)\u000A            (update :exec update-mem$ci)\u000A            misc update-misc))</code> \u000A \u000A <p><b>SUMMARY DOCUMENTATION</b></p> \u000A \u000A <p>The General Form is as shown below, where the order of keywords is \u000A unimportant.  Duplicate keywords are discouraged; while permitted, only the \u000A first (leftmost) occurrence of a given keyword is used.  Only the \u000A <v>:exports</v> keyword is required.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see> st\u000A  :concrete concrete\u000A  :recognizer recognizer\u000A  :creator creator\u000A  :corr-fn corr-fn\u000A  :congruent-to congruent-to\u000A  :protect-default protect-default\u000A  :exports (e1 ... ek)\u000A  :doc doc)</code> \u000A \u000A <p>The keyword argument <v>:EXPORTS</v> must be supplied, and missing or \u000A <v>nil</v> keyword arguments have defaults as indicated below.  All arguments \u000A must satisfy the conditions below.</p> \u000A \u000A <p>Before we describe the arguments, we define a notion of a ``function spec'' \u000A and its ``completion''.  A function spec is either a symbol or else a list of \u000A the form</p> \u000A \u000A <code>(fn <v>:kwd1</v> val1 ... <v>:kwdn</v> valn), </code> \u000A \u000A <p>that is, a symbol followed by a <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt>.  We view the \u000A case of a symbol, <v>s</v>, as the function spec <v>(s)</v>, with no keywords. \u000A There must be no duplicate keywords.  In each case that we expect a function \u000A spec, the context provides a set of valid keywords for that function spec; it \u000A is an error to provide any other keyword in the function spec.  Each function \u000A spec is interpreted as its ``completion'', obtained by extending the function \u000A spec with a default value for each valid keyword as indicated below.  With \u000A that interpretation, the ``exported function'' of a function spec is its \u000A <v>car</v>, and that function symbol and each keyword value must be a \u000A guard-verified function symbol; and moreover, the <v>:EXEC</v> function must not \u000A include the new abstract stobj name, <v>st</v>, among its formals.</p> \u000A \u000A <p>We are ready to describe the arguments of <v>defabsstobj</v>.</p> \u000A \u000A <blockquote><p><v>St</v> is a symbol, which names the new abstract stobj.</p> \u000A \u000A <p><v>Concrete</v> is the name of an existing stobj that is not an abstract \u000A stobj, i.e., was introduced with <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> (not <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>).</p> \u000A \u000A <p><v>Recognizer</v> is a function spec (for the recognizer function).  The \u000A valid keywords are <v>:LOGIC</v> and <v>:EXEC</v>.  The default for \u000A <v>recognizer</v> is obtained by adding the suffix <v>&quot;P&quot;</v> to <v>name</v>.  The \u000A default value for <v>:LOGIC</v> is formed by adding the suffix <v>&quot;$AP&quot;</v> to \u000A <v>recognizer</v>; for <v>:EXEC</v>, by adding the suffix <v>&quot;$CP&quot;</v>.  The \u000A <v>:EXEC</v> function must be the recognizer for the specified <v>:CONCRETE</v> \u000A stobj.</p> \u000A \u000A <p><v>Creator</v> is a function spec (for the creator function).  The valid \u000A keywords are <v>:LOGIC</v> and <v>:EXEC</v>.  The default for <v>creator</v> is \u000A obtained by adding the prefix <v>&quot;CREATE-&quot;</v> to <v>name</v>.  The default \u000A value for <v>:LOGIC</v> is formed by adding the suffix <v>&quot;$A&quot;</v> to \u000A <v>creator</v>; for <v>:EXEC</v>, by adding the suffix <v>&quot;$C&quot;</v>.  The \u000A <v>:CREATOR</v> function must be the creator for the specified <v>:CONCRETE</v> \u000A stobj, as ACL2 checks that the <v>:CREATOR</v> function takes no arguments and \u000A returns the <v>:CONCRETE</v> stobj.</p> \u000A \u000A <p><v>Corr-fn</v> is a known function symbol that takes two arguments (for the \u000A correspondence theorems).  The default for <v>corr-fn</v> is obtained by adding \u000A the suffix <v>&quot;$CORR&quot;</v> to <v>name</v>.</p> \u000A \u000A <p><v>Congruent-to</v> should either be <v>nil</v> (the default) or the name of an \u000A abstract stobj previously introduced (by <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>).  In the latter \u000A case, the current and previous abstract stobj should have the same concrete \u000A stobj (not merely congruent concrete stobjs), and their <v>:EXPORTS</v> fields \u000A should have the same length and also correspond, as follows: the ith export of \u000A each should have the same <v>:LOGIC</v> and <v>:EXEC</v> symbols.  See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> for more about congruent stobjs.  Note that if two names are \u000A congruent, then they are either both ordinary stobjs or both abstract \u000A stobjs.</p> \u000A \u000A <p><v>Protect-default</v> should either be <v>nil</v> (the default) or <v>t</v>.  It \u000A provides the value of keyword <v>:PROTECT</v> for each member of <v>exports</v> \u000A that does not explicitly specify <v>:PROTECT</v>.  See the discussion of \u000A <v>exports</v> below.</p> \u000A \u000A <p>An important aspect of the <v>congruent-to</v> parameter is that if it is not \u000A <v>nil</v>, then the checks for lemmas &mdash; <v>{CORRESPONDENCE}</v>, \u000A <v>{GUARD-THM}</v>, and <v>{PRESERVED}</v> &mdash; are omitted.  Thus, the values \u000A of keyword <v>:CORR-FN</v>, and the values of keywords <v>:CORRESPONDENCE</v>, \u000A <v>:GUARD-THM</v>, and <v>:PRESERVED</v> in each export (as we discuss next), are \u000A irrelevant; they are not inferred and they need not be supplied.</p> \u000A \u000A <p>The value of <v>:EXPORTS</v> is a non-empty true list.  Each <v>ei</v> is a \u000A function spec (for an exported function).  The valid keywords are <v>:LOGIC</v>, \u000A <v>:EXEC</v>, <v>:CORRESPONDENCE</v>, and <v>:GUARD-THM</v>, <v>:PROTECT</v>, and also \u000A <v>:PRESERVED</v> if and only if the specified <v>:EXEC</v> function returns the \u000A <v>:CONCRETE</v> stobj.  The default values for all of these keywords except \u000A <v>:PROTECT</v> are obtained by respectively adding the suffix <v>&quot;$A&quot;</v> \u000A <v>&quot;$C&quot;</v>, <v>&quot;{CORRESPONDENCE}&quot;</v>, <v>&quot;{GUARD-THM}&quot;</v>, or \u000A <v>&quot;{PRESERVED}&quot;</v>.  For <v>:PROTECT</v>, the default is <v>nil</v> unless the \u000A <v>defabsstobj</v> event specifies <v>:PROTECT-DEFAULT t</v>.</p> \u000A \u000A <p><v>Doc</v>, if non-<v>nil</v>, is a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).</p></blockquote> \u000A \u000A <p>Not shown is the keyword, <v>:MISSING</v>; the effect of <v>:missing t</v> is \u000A to turn the call of <v>defabsstobj</v> into a corresponding call of <tt><see topic=\"ACL2____DEFABSSTOBJ-MISSING-EVENTS\">defabsstobj-missing-events</see></tt>.</p> \u000A \u000A <p>Note that a <v>defabsstobj</v> event will fail if the required lemmas &mdash; \u000A that is, those for valid keywords <v>:CORRESPONDENCE</v>, <v>:GUARD-THM</v>, and \u000A <v>:PRESERVED</v> &mdash; have not been proved, unless proofs are being skipped. \u000A The exemption when skipping proofs allows the supporting lemmas to be <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  If the <tt><see topic=\"ACL2____LD\">ld</see></tt> special <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>t</v>, then the missing <see topic=\"ACL2____EVENTS\">events</see> \u000A are printed with a warning before the <v>defabsstobj</v> event is admitted; but \u000A if <v>ld-skip-proofsp</v> is the symbol <v>INCLUDE-BOOK</v>, then that warning is \u000A omitted.  (Also see <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> and see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.)  If \u000A however proofs are not being skipped, then the <v>defabsstobj</v> event will \u000A fail after printing the missing events.  Advanced users may wish to see <see topic=\"ACL2____DEFABSSTOBJ-MISSING-EVENTS\">defabsstobj-missing-events</see> for a utility that returns a data structure \u000A containing the missing lemmas.</p> \u000A \u000A <p>Let <v>st</v> be an abstract stobj with corresponding concrete stobj \u000A <v>st$c</v>.  let <v>f</v> be an exported function for <v>st</v> and let <v>f$a</v> \u000A and <v>f$c</v> be the corresponding <v>:LOGIC</v> and <v>:EXEC</v> functions, \u000A respectively.  The formals of <v>f</v> are obtained by taking the formals of \u000A <v>f$c</v> and replacing <v>st$c</v> by <v>st</v>.  The <see topic=\"ACL2____GUARD\">guard</see> for <v>f</v> is \u000A derived as follows from the guard of <v>f$a</v>.  First, the formals of <v>f$a</v> \u000A are replaced by the formals of <v>f</v> in the guard of <v>f$a</v>, to obtain a \u000A term we denote here as <v>guard-pre</v>.  Now for each exported function symbol \u000A <v>g</v> of <v>st</v> with corresponding <v>:LOGIC</v> function <v>g$a</v>, form a \u000A functional substitution by consing <v>g$a</v> with <v>g</v>.  Finally, apply that \u000A functional substitution to <v>guard-pre</v>; the result is the guard of <v>f</v>. \u000A That guard must satisfy the usual conditions of a guard: thus, it must return \u000A a single non-<see topic=\"ACL2____STOBJ\">stobj</see> value and satisfy normal syntactic restrictions, \u000A including single-threadedness in its handling of stobjs.</p> \u000A \u000A <p><v>Remark</v>.  Because of how guards are created for exported functions, and \u000A in particular because <v>:LOGIC</v> functions are replaced as discussed above, a \u000A good discipline is to define <v>:LOGIC</v> functions that are not intended for \u000A general use, but are intended only for use as <v>:LOGIC</v> functions of \u000A corresponding stobj primitives.  For example, suppose that you use <v>length</v> \u000A as the <v>:LOGIC</v> function for some stobj primitive, <v>f</v> (as opposed to \u000A using your own function, say, <v>foo-length</v> or <v>foo$a</v>).  Then every call \u000A of <v>length</v> will be replaced by <v>f</v> when creating the guard of a stobj \u000A primitive from the guard of its <v>:LOGIC</v> function.  This might not be what \u000A you intended if you were using <v>length</v> in that guard simply to compute the \u000A length of an ordinary list.</p> \u000A \u000A <p>There are a few additional restrictions, as follows.</p> \u000A \u000A <blockquote><p>All exported function names must be new (unless redefinition is \u000A on; see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>), and there must be no duplicates among \u000A them.</p> \u000A \u000A <p>The <v>:CONCRETE</v> stobj name must be a formal parameter of the <v>:EXEC</v> \u000A fn of every function spec, except for the <v>:CREATOR</v> function spec.  Also \u000A the input signatures of the <v>:LOGIC</v> and <v>:EXEC</v> function for a function \u000A spec must agree, except perhaps at the position of that <v>:CONCRETE</v> \u000A formal.</p> \u000A \u000A <p>For function specs other than the <v>:CREATOR</v> function spec, the output \u000A signatures of the <v>:LOGIC</v> and <v>:EXEC</v> functions must have the same \u000A length and must agree, except perhaps at position <v>p_out</v> of the \u000A <v>:CONCRETE</v> stobj in the <v>:EXEC</v> function's output.  If <v>p_in</v> is the \u000A position of the <v>:CONCRETE</v> stobj in the <v>:EXEC</v> function's formals, \u000A then the <v>:LOGIC</v> function's output at position <v>p_out</v> should match the \u000A <v>:LOGIC</v> function's formal at position <v>p_in</v>.</p> \u000A \u000A <p>The <v>:PROTECT</v> keyword is something that you should ignore unless you \u000A get an error message about it, pertaining to modifying the concrete stobj \u000A non-atomically.  In that case, you can eliminate the error by providing \u000A <v>:PROTECT t</v> in the function spec, or by providing <v>defabsstobj</v> keyword \u000A argument <v>:PROTECT-DEFAULT t</v> at the top level.  The above explanation is \u000A probably all you need to know about <v>:PROTECT</v>, but just below is a more \u000A complete explanation for those who desire it.  Further information is also \u000A available if you need it; see <see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see>, and see the example \u000A uses of these keywords in community book \u000A <v>books/misc/defabsstobj-example-2.lisp</v>.</p></blockquote> \u000A \u000A <p>For those who are interested, here is a more detailed discussion of \u000A <v>:PROTECT</v> and <v>:PROTECT-DEFAULT</v>, as promised above.  It applies to any \u000A function spec for an export (hence not to the <v>:CREATOR</v> function spec). \u000A If the <v>:EXEC</v> function is a stobj primitive, then clearly the following \u000A property holds: any execution of a call of that function can only update the \u000A concrete stobj at most once &mdash; i.e., modification of the concrete stobj \u000A is atomic.  ACL2 can deduce this property not only for stobj primitives but \u000A for many other functions as well.  However, if ACL2 cannot deduce this \u000A property, then it will cause an error saying that the <v>:EXEC</v> function \u000A ``appears capable of modifying the concrete stobj, <v>&lt;stobj_name&gt;</v>, \u000A non-atomically.''  That message also explains how to eliminate this error: \u000A provide <v>:PROTECT t</v> for the function spec.  Alternatively, all function \u000A specs without an explicit <v>:PROTECT</v> keyword can be implicitly supplied \u000A <v>:PROTECT t</v> by supplying the value <v>t</v> for the <v>:PROTECT-DEFAULT</v> \u000A keyword parameter of the <v>defabsstobj</v> event.  However, beware that when \u000A <v>:PROTECT</v> is <v>t</v>, the generated raw Lisp code runs slightly less \u000A efficiently &mdash; though perhaps with negligible efficiency loss if the \u000A <v>:EXEC</v> function is not trivial.  Community books \u000A <v>books/misc/defabsstobj-example-3.lisp</v> and \u000A <v>books/misc/defabsstobj-example-4.lisp</v> provide related information.</p> \u000A \u000A <p>We conclude with some remarks.</p> \u000A \u000A <p>Unlike <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>, there is no <v>:renaming</v> argument.  Instead, the \u000A scheme described above provides a flexible way to assign names.  Also unlike \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>, there is no <v>:inline</v> or <v>:non-memoizable</v> argument; \u000A <v>:inline</v> is essentially t, in the sense that stobj primitives are macros \u000A in raw Lisp; and the <v>:non-memoizable</v> argument is derived implicitly from \u000A the value of that argument (<v>nil</v> by default) for the corresponding \u000A concrete stobj.</p> \u000A \u000A <p>Those who use the experimental extension ACL2(h), which includes function \u000A memoization (see <see topic=\"ACL2____MEMOIZE\">memoize</see>), may be aware that the memo table for a \u000A function is flushed whenever it is the case that one of its stobj inputs is \u000A updated.  In fact, such flushing happens even when a stobj that is congruent \u000A to one of its stobj inputs is updated.  For that purpose, an abstract stobj is \u000A considered to be congruent to its corresponding concrete stobj.</p>"],

 "ACL2____CBD":[["Books-reference"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;:cbd\u000A&quot;/usr/home/smith/&quot;</code> \u000A \u000A <p>The connected book directory is a nonempty string that specifies a \u000A directory as an absolute pathname.  (See <see topic=\"COMMON-LISP____PATHNAME\">pathname</see> for a discussion of \u000A file naming conventions.)  When <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is given a relative book \u000A name it elaborates it into a full book name, essentially by appending the \u000A connected book directory string to the left and <v>&quot;.lisp&quot;</v> to the right. \u000A (For details, see <see topic=\"ACL2____BOOK-NAME\">book-name</see> and also see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>.) \u000A Furthermore, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> temporarily sets the connected book \u000A directory to the directory string of the resulting full book name so that \u000A references to inferior <see topic=\"ACL2____BOOKS\">books</see> in the same directory may omit the \u000A directory.  See <see topic=\"ACL2____SET-CBD\">set-cbd</see> for how to set the connected book directory \u000A string.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____CBD\">cbd</see>)</code> \u000A \u000A <p>This is a macro that expands into a term involving the single free variable \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  It returns the connected book directory string.</p> \u000A \u000A <p>The connected book directory (henceforth called the ``<v>cbd</v>'') is used \u000A by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to elaborate the supplied book name into a full book \u000A name (see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>).  For example, if the <v>cbd</v> is \u000A <v>&quot;/usr/home/smith/&quot;</v> then the elaboration of the <see topic=\"ACL2____BOOK-NAME\">book-name</see> \u000A <v>&quot;project/task-1/arith&quot;</v> (to the <v>&quot;.lisp&quot;</v> extension) is \u000A <v>&quot;/usr/home/smith/project/task-1/arith.lisp&quot;</v>.  That <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see> is what <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> opens to read the source text for \u000A the book.</p> \u000A \u000A <p>The <v>cbd</v> may be changed using <tt><see topic=\"ACL2____SET-CBD\">set-cbd</see></tt> (see <see topic=\"ACL2____SET-CBD\">set-cbd</see>). \u000A Furthermore, during the processing of the <see topic=\"ACL2____EVENTS\">events</see> in a book, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> sets the <v>cbd</v> to be the directory string of the <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see> of the book.  Thus, if the <v>cbd</v> is \u000A <v>&quot;/usr/home/smith/&quot;</v> then during the processing of <see topic=\"ACL2____EVENTS\">events</see> by</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;project/task-1/arith&quot;)</code> \u000A \u000A <p>the <v>cbd</v> will be set to <v>&quot;/usr/home/smith/project/task-1/&quot;</v>.  Note \u000A that if <v>&quot;arith&quot;</v> recursively includes a subbook, say <v>&quot;naturals&quot;</v>, \u000A that resides on the same directory, the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event for it may \u000A omit the specification of that directory.  For example, <v>&quot;arith&quot;</v> might \u000A contain the event</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;naturals&quot;).</code> \u000A \u000A <p>In general, suppose we have a superior book and several inferior <see topic=\"ACL2____BOOKS\">books</see> which are included by <see topic=\"ACL2____EVENTS\">events</see> in the superior book.  Any inferior \u000A book residing on the same directory as the superior book may be referenced in \u000A the superior without specification of the directory.</p> \u000A \u000A <p>We call this a ``relative'' as opposed to ``absolute'' naming.  The use of \u000A relative naming is preferred because it permits <see topic=\"ACL2____BOOKS\">books</see> (and their \u000A accompanying inferiors) to be moved between directories while maintaining \u000A their <see topic=\"ACL2____CERTIFICATE\">certificate</see>s and utility.  Certified <see topic=\"ACL2____BOOKS\">books</see> that reference \u000A inferiors by absolute file names are unusable (and rendered uncertified) if \u000A the inferiors are moved to new directories.</p> \u000A \u000A <p><i>Technical Note and a Challenge to Users:</i></p> \u000A \u000A <p>After elaborating the book name to a full book name, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A opens a channel to the file to process the <see topic=\"ACL2____EVENTS\">events</see> in it.  In some host \u000A Common Lisps, the actual file opened depends upon a notion of ``connected \u000A directory'' similar to our connected book directory.  Our intention in always \u000A elaborating book names into absolute filename strings (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see> for \u000A terminology) is to circumvent the sensitivity to the connected directory.  But \u000A we may have insufficient control over this since the ultimate file naming \u000A conventions are determined by the host operating system rather than Common \u000A Lisp (though, we do check that the operating system ``appears'' to be one that \u000A we ``know'' about).  Here is a question, which we'll pose assuming that we \u000A have an operating system that calls itself ``Unix.''  Suppose we have a file \u000A name, filename, that begins with a slash, e.g., <v>&quot;/usr/home/smith/...&quot;</v>. \u000A Consider two successive invocations of CLTL's</p> \u000A \u000A <code>(open filename :direction :input)</code> \u000A \u000A <p>separated only by a change to the operating system's notion of connected \u000A directory.  Must these two invocations produce streams to the same file?  A \u000A candidate string might be something like \u000A <v>&quot;/usr/home/smith/*/usr/local/src/foo.lisp&quot;</v> which includes some \u000A operating system-specific special character to mean ``here insert the \u000A connected directory'' or, more generally, ``here make the name dependent on \u000A some non-ACL2 aspect of the host's state.''  If such ``tricky'' name strings \u000A beginning with a slash exist, then we have failed to isolate ACL2 adequately \u000A from the operating system's file naming conventions.  Once upon a time, ACL2 \u000A did not insist that the <v>cbd</v> begin with a slash and that allowed the \u000A string <v>&quot;foo.lisp&quot;</v> to be tricky because if one were connected to \u000A <v>&quot;/usr/home/smith/&quot;</v> then with the empty <v>cbd</v> <v>&quot;foo.lisp&quot;</v> is a \u000A full book name that names the same file as <v>&quot;/usr/home/smith/foo.lisp&quot;</v>. \u000A If the actual file one reads is determined by the operating system's state \u000A then it is possible for ACL2 to have two distinct ``full book names'' for the \u000A same file, the ``real'' name and the ``tricky'' name.  This can cause ACL2 to \u000A include the same book twice, not recognizing the second one as \u000A redundant.</p>"],

 "ACL2____RETURN-LAST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Return-last</v> is an ACL2 function that returns its last \u000A argument.  It is used to implement common utilities such as <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> and \u000A <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>.  For most users, this may already be more than one needs to \u000A know about <v>return-last</v>; for example, ACL2 tends to avoid printing calls \u000A of <v>return-last</v> in its output, printing calls of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> or <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> (or other such utilities) instead.</p> \u000A \u000A <p>If you encounter a call of <v>return-last</v> during a proof, then you may \u000A find it most useful to consider <v>return-last</v> simply as a function defined \u000A by the following equation.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____RETURN-LAST\">return-last</see> x y z) z)</code> \u000A \u000A <p>It may also be useful to know that unlike other ACL2 functions, \u000A <v>return-last</v> can take a multiple value as its last argument, in which case \u000A this multiple value is returned.  The following contrived definition \u000A illustrates this point.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (fn x y z)\u000A         (<see topic=\"ACL2____RETURN-LAST\">return-last</see> fn x (<see topic=\"ACL2____MV\">mv</see> y z)))\u000A\u000ASince FOO is non-recursive, its admission is trivial.  We observe that\u000Athe type of FOO is described by the theorem\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (FOO FN X Y Z)) (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (FOO FN X Y Z))).  We used\u000Aprimitive type reasoning.\u000A\u000A(FOO * * * *) =&gt; (<see topic=\"ACL2____MV\">MV</see> * *).\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FOO\u000AACL2 !&gt;(foo 'bar 3 4 5)\u000A(4 5)\u000AACL2 !&gt;(<see topic=\"ACL2____MV-LET\">mv-let</see> (a b)\u000A               (foo 'bar 3 4 5)\u000A               (<see topic=\"COMMON-LISP____CONS\">cons</see> b a))\u000A(5 . 4)\u000AACL2 !&gt;</code> \u000A \u000A <p>Most readers would be well served to avoid reading the rest of this \u000A documentation of <v>return-last</v>.  For reference, however, below we document \u000A it in some detail.  We include some discussion of its evaluation, in \u000A particular its behavior in raw Lisp, because we expect that most who read \u000A further are working with raw Lisp code (and trust tags).</p> \u000A \u000A <p><v>Return-last</v> is an ACL2 function that can arise from macroexpansion of \u000A certain utilities that return their last argument, which may be a multiple \u000A value.  Consider for example the simplest of these, <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>:</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Some CW printing...~%&quot;) (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A (<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'PROGN\u000A              (<see topic=\"ACL2____CW\">CW</see> &quot;Some CW printing...~%&quot;)\u000A              (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that a call of <v>prog2$</v> macroexpands to a call of \u000A <v>return-last</v> in which the first argument is <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> progn)</v>.  Although \u000A <v>return-last</v> is a function in the ACL2 world, it is implemented ``under \u000A the hood'' as a macro in raw Lisp, as the following log (continuing the \u000A example above) illustrates.</p> \u000A \u000A <code>ACL2 !&gt;:q\u000A\u000AExiting the ACL2 read-eval-print loop.  To re-enter, execute (<see topic=\"ACL2____LP\">LP</see>).\u000A? [RAW LISP] (macroexpand-1 '(<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'PROGN\u000A                                           (<see topic=\"ACL2____CW\">CW</see> &quot;Some CW printing...~%&quot;)\u000A                                           (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4)))\u000A(<see topic=\"COMMON-LISP____PROGN\">PROGN</see> (<see topic=\"COMMON-LISP____LET\">LET</see> ((*AOKP* T)) (<see topic=\"ACL2____CW\">CW</see> &quot;SOME CW PRINTING...~%&quot;)) (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000AT\u000A? [RAW LISP]</code> \u000A \u000A <p>Thus, the original <v>prog2$</v> call generates a corresponding call of \u000A <v>progn</v> in raw Lisp, which in turn causes evaluation of each argument and \u000A returns whatever is returned by evaluation of the last (second) argument.</p> \u000A \u000A <p>(Remark for those who use <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>.  The binding of <v>*aokp*</v> to \u000A <v>t</v> is always included for the second argument as shown except when the \u000A first argument is of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">QUOTE</see> M)</v> where <v>M</v> is a macro, or (less \u000A important) when the first argument is a symbol or a cons whose car is \u000A <v>QUOTE</v>.  This binding allows ACL2 to use attachments in the second \u000A argument of <v>return-last</v> (hence, in the first argument of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>), \u000A even in contexts such as proofs in which attachments are normally not allowed. \u000A Those who use the experimental HONS version of ACL2 (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>) will see an additional binding in the above single-step \u000A macroexpansion, which allows the storing of memoized results even when that \u000A would otherwise be prevented because of the use of attachments.)</p> \u000A \u000A <p>In general, a form <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> F) X Y)</v> macroexpands to <v>(F X\u000A Y)</v>, where <v>F</v> is defined in raw Lisp to return its last argument.  The \u000A case that <v>F</v> is <v>progn</v> is a bit misleading, because it is so simple. \u000A More commonly, macroexpansion produces a call of a macro defined in raw Lisp \u000A that may produce side effects.  Consider for example the ACL2 utility <tt><see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see></tt>, which is intended to change the <see topic=\"ACL2____GUARD\">guard</see>-checking \u000A mode to the indicated value (see <see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see>).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> :none (<see topic=\"COMMON-LISP____CAR\">car</see> 3)) ; no guard violation\u000ANIL\u000AACL2 !&gt;:trans1 (<see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> :none (<see topic=\"COMMON-LISP____CAR\">car</see> 3))\u000A (WITH-GUARD-CHECKING1 (CHK-WITH-GUARD-CHECKING-ARG :NONE)\u000A                       (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))\u000AACL2 !&gt;:trans1 (WITH-GUARD-CHECKING1 (CHK-WITH-GUARD-CHECKING-ARG :NONE)\u000A                                     (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))\u000A (<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'WITH-GUARD-CHECKING1-RAW\u000A              (CHK-WITH-GUARD-CHECKING-ARG :NONE)\u000A              (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))\u000AACL2 !&gt;:q\u000A\u000AExiting the ACL2 read-eval-print loop.  To re-enter, execute (<see topic=\"ACL2____LP\">LP</see>).\u000A? [RAW LISP] (macroexpand-1\u000A              '(<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'WITH-GUARD-CHECKING1-RAW\u000A                             (CHK-WITH-GUARD-CHECKING-ARG :NONE)\u000A                             (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3)))\u000A(WITH-GUARD-CHECKING1-RAW (CHK-WITH-GUARD-CHECKING-ARG :NONE) (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))\u000AT\u000A? [RAW LISP] (pprint\u000A              (macroexpand-1\u000A               '(WITH-GUARD-CHECKING1-RAW\u000A                 (CHK-WITH-GUARD-CHECKING-ARG :NONE)\u000A                 (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))))\u000A\u000A(<see topic=\"COMMON-LISP____LET\">LET</see> ((ACL2_GLOBAL_ACL2::GUARD-CHECKING-ON\u000A       (CHK-WITH-GUARD-CHECKING-ARG :NONE)))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (SPECIAL ACL2_GLOBAL_ACL2::GUARD-CHECKING-ON))\u000A  (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3))\u000A? [RAW LISP]</code> \u000A \u000A <p>The above raw Lisp code binds the state global variable \u000A <v>guard-checking-on</v> to <v>:none</v>, as <v>chk-with-guard-checking-arg</v> is \u000A just the identity function except for causing a hard error for an illegal \u000A input.</p> \u000A \u000A <p>The intended use of <v>return-last</v> is that the second argument is \u000A evaluated first in a normal manner, and then the third argument is evaluated \u000A in an environment that may depend on the value of the second argument.  (For \u000A example, the macro <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt> macroexpands to a call of \u000A <v>return-last</v> with a first argument of <v>'WITH-PROVER-TIME-LIMIT1-RAW</v>, a \u000A second argument that evaluates to a numeric time limit, and a third argument \u000A that is evaluated in an environment where the theorem prover is restricted to \u000A avoid running longer than that time limit.)  Although this intended usage \u000A model is not strictly enforced, it is useful to keep in mind in the following \u000A description of how calls of <v>return-last</v> are handled by the ACL2 \u000A evaluator.</p> \u000A \u000A <p>When a form is submitted in the top-level loop, it is handled by ACL2's \u000A custom evaluator.  That evaluator is specified to respect the semantics of the \u000A expression supplied to it: briefly put, if an expression <v>E</v> evaluates to a \u000A value <v>V</v>, then the equality <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> E (<see topic=\"COMMON-LISP____QUOTE\">quote</see> V))</v> should be a theorem. \u000A Notice that this specification does not discuss the side-effects that may \u000A occur when evaluating a call of <v>return-last</v>, so we discuss that now. \u000A Suppose that the ACL2 evaluator encounters the call <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see> 'fn expr1\u000A expr2)</v>.  First it evaluates <v>expr1</v>.  If this evaluation succeeds without \u000A error, then it constructs an expression of the form <v>(fn *x* ev-form)</v>, \u000A where *x* is a Lisp variable bound to the result of evaluating <v>expr1</v> and \u000A <v>ev-form</v> is a call of the evaluator for <v>expr2</v>.  (Those who want \u000A implementation details are invited to look at function <v>ev-rec-return-last</v> \u000A in ACL2 source file <v>translate.lisp</v>.)  There are exceptions if <v>fn</v> is \u000A <v>progn</v>, <v>ec-call1-raw</v>, <v>with-guard-checking1-raw</v>, or \u000A <v>mbe1-raw</v>, but the main idea is the same: do a reasonable job emulating \u000A the behavior of a raw-Lisp call of <v>return-last</v>.</p> \u000A \u000A <p>The following log shows how a <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> call can generate a call of the \u000A evaluator for the last argument of <v>return-last</v> (arguent <v>expr2</v>, \u000A above).  We use <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> to show single-step macroexpansions, \u000A which indicate how a call of <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> expands to a call of \u000A <v>return-last</v>.  The implementation actually binds the Lisp variable \u000A <v>*RETURN-LAST-ARG3*</v> to <v>expr2</v> before calling the ACL2 evaluator, \u000A <v>ev-rec</v>.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A (TIME$1 (<see topic=\"COMMON-LISP____LIST\">LIST</see> 0 NIL NIL NIL NIL)\u000A         (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000AACL2 !&gt;:trans1 (TIME$1 (<see topic=\"COMMON-LISP____LIST\">LIST</see> 0 NIL NIL NIL NIL)\u000A                       (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A (<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'TIME$1-RAW\u000A              (<see topic=\"COMMON-LISP____LIST\">LIST</see> 0 NIL NIL NIL NIL)\u000A              (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000AACL2 !&gt;(<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A; (EV-REC *RETURN-LAST-ARG3* ...) took\u000A; 0.00 seconds realtime, 0.00 seconds runtime\u000A; (1,120 bytes allocated).\u000A7\u000AACL2 !&gt;</code> \u000A \u000A <p>We now show how things can go wrong in other than the ``intended use'' case \u000A described above.  In the example below, the macro <v>mac-raw</v> is operating \u000A directly on the syntactic representation of its first argument, which it \u000A obtains of course as the second argument of a <v>return-last</v> call.  Again \u000A this ``intended use'' of <v>return-last</v> requires that argument to be \u000A evaluated and then only its result is relevant; its syntax is not supposed to \u000A matter.  We emphasize that only top-level evaluation depends on this \u000A ``intended use''; once evaluation is passed to Lisp, the issue disappears.  We \u000A illustrate below how to use the <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt> utility to avoid this issue; \u000A see <see topic=\"ACL2____TOP-LEVEL\">top-level</see>.  The example uses the utility <v>defmacro-last</v> to \u000A ``install'' special handling of the raw-Lisp macro <v>mac-raw</v> by \u000A <v>return-last</v>; later below we discuss <v>defmacro-last</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A\u000ATTAG NOTE: Adding ttag :T from the top level loop.\u000A T\u000AACL2 !&gt;(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A         (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t)\u000A         (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac-raw (x y)\u000A           `(<see topic=\"COMMON-LISP____PROGN\">progn</see> (print (<see topic=\"COMMON-LISP____QUOTE\">quote</see> ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))\u000A                   (terpri) ; newline\u000A                   ,y)))\u000A\u000ASummary\u000AForm:  ( PROGN! (<see topic=\"ACL2____SET-RAW-MODE\">SET-RAW-MODE</see> T) ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> mac)\u000A[[ ... output omitted ... ]]\u000A RETURN-LAST-TABLE\u000AACL2 !&gt;(<see topic=\"ACL2____RETURN-LAST\">return-last</see> 'mac-raw '3 nil)\u000A\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  Fault during read of memory address #x120000300006\u000A***********************************************\u000A\u000AIf you didn't cause an explicit interrupt (Control-C),\u000Athen the root cause may be call of a :program mode\u000Afunction that has the wrong guard specified, or even no\u000Aguard specified (i.e., an implicit guard of t).\u000ASee :DOC guards.\u000A\u000ATo enable breaks into the debugger (also see :DOC acl2-customization):\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000AACL2 !&gt;(<see topic=\"ACL2____TOP-LEVEL\">top-level</see> (<see topic=\"ACL2____RETURN-LAST\">return-last</see> 'mac-raw '3 nil))\u000A\u000A3\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p>We next describe how to extend the behavior of <v>return-last</v>.  This \u000A requires an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), and is accomplished by \u000A extending a <see topic=\"ACL2____TABLE\">table</see> provided by ACL2, see <see topic=\"ACL2____RETURN-LAST-TABLE\">return-last-table</see>. \u000A Rather than using <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see> directly for this purpose, it is \u000A probably more convenient to use a macro, <v>defmacro-last</v>.  We describe the \u000A community book <v>books/misc/profiling.lisp</v> in order to illustrate how this \u000A works.  The events in that book include the following, which are described \u000A below.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :profiling)\u000A\u000A(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t)\u000A (load (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string (<see topic=\"ACL2____CBD\">cbd</see>) &quot;profiling-raw.lsp&quot;)))\u000A\u000A(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> with-profiling)</code> \u000A \u000A <p>The first event introduces a trust tag.  The second loads a file into raw \u000A Lisp that defines a macro, <v>with-profiling-raw</v>, which can do profiling for \u000A the form to be evaluated.  The third introduces an ACL2 macro \u000A <v>with-profiling</v>, whose calls expand into calls of the form <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see>\u000A (<see topic=\"COMMON-LISP____QUOTE\">quote</see> with-profiling-raw) &amp; &amp;)</v>.  The third event also extends <tt><see topic=\"ACL2____RETURN-LAST-TABLE\">return-last-table</see></tt> so that these calls will expand in raw Lisp to calls of \u000A <v>with-profiling-raw</v>.</p> \u000A \u000A <p>The example above illustrates the following methodology for introducing a \u000A macro that returns its last argument but produces useful side-effects with raw \u000A Lisp code.</p> \u000A \u000A <blockquote><p>(1) Introduce a trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p> \u000A \u000A <p>(2) Using <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, load into raw Lisp a file defining a macro, \u000A <v>RAW-NAME</v>, that takes two arguments, returning its last (second) argument \u000A but using the first argument to produce desired side effects during evaluation \u000A of that last argument.</p> \u000A \u000A <p>(3) Evaluate the form <v>(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> NAME :raw RAW-NAME)</v>.  This will \u000A introduce <v>NAME</v> as an ACL2 macro that expands to a corresponding call of \u000A <v>RAW-NAME</v> (see (2) above) in raw Lisp.  The specification of keyword value \u000A of <v>:raw</v> as <v>RAW-NAME</v> may be omitted if <v>RAW-NAME</v> is the result of \u000A modifying the symbol <v>NAME</v> by suffixing the string <v>&quot;-RAW&quot;</v> to the \u000A <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of <v>NAME</v>.</p></blockquote> \u000A \u000A <p>WARNING: Not every use of <v>return-last</v> can be soundly evaluated outside \u000A a function body.  The reason is that ACL2's evaluator, <v>ev-rec</v>, recurs \u000A through terms that are presented in the top-level loop, and handles \u000A <v>return-last</v> calls in a special manner: basically, the call of <v>ev-rec</v> \u000A on the form <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see> 'mac-raw x y)</v> leads to evaluation of a macro \u000A call of the form <v>(mac-raw *return-last-arg2* (ev-rec ...))</v>, where \u000A *return-last-arg2* is a global variable bound to the result of evaluating \u000A <v>x</v> with <v>ev-rec</v>.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A(set-raw-mode-on state)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac-raw (str y) ; print message is an atom\u000A `(<see topic=\"COMMON-LISP____LET\">let</see> ((result (<see topic=\"COMMON-LISP____CONSP\">consp</see> ,y))\u000A        (str ,str))\u000A    (<see topic=\"COMMON-LISP____OR\">or</see> result\u000A        (<see topic=\"ACL2____PROG2_42\">prog2$</see> (fmx ,str ',y)\u000A                nil))))\u000A(set-raw-mode-off state)\u000A(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> mac)\u000A; Horrible error:\u000A(mac &quot;Not a cons: ~x0~%&quot; 17)\u000A; Works, but probably many would consider it awkward to use top-level:\u000A(<see topic=\"ACL2____TOP-LEVEL\">top-level</see> (mac &quot;Not a cons: ~x0~%&quot; 17))</code> \u000A \u000A <p>In such cases we suggest supplying keyword <v>:top-level-ok nil</v> to the \u000A call of <v>defmacro-last</v>, for example:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> mac :top-level-ok nil)</code> \u000A \u000A <p>Then any attempt to call <v>mac</v> at the top level, as opposed to inside a \u000A function body, will cause a clean error before evaluation begins.</p> \u000A \u000A <p>It is useful to explore what is done by <v>defmacro-last</v>.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> with-profiling)\u000A (<see topic=\"COMMON-LISP____PROGN\">PROGN</see> (<see topic=\"COMMON-LISP____DEFMACRO\">DEFMACRO</see> WITH-PROFILING (X Y)\u000A                  (<see topic=\"COMMON-LISP____LIST\">LIST</see> 'RETURN-LAST\u000A                        (<see topic=\"COMMON-LISP____LIST\">LIST</see> 'QUOTE 'WITH-PROFILING-RAW)\u000A                        X Y))\u000A        (<see topic=\"ACL2____TABLE\">TABLE</see> RETURN-LAST-TABLE 'WITH-PROFILING-RAW\u000A               'WITH-PROFILING))\u000AACL2 !&gt;:trans1 (with-profiling '(<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> fgetprop rewrite) (mini-proveall))\u000A (<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'WITH-PROFILING-RAW\u000A              '(<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> FGETPROP REWRITE)\u000A              (MINI-PROVEALL))\u000AACL2 !&gt;:q\u000A\u000AExiting the ACL2 read-eval-print loop.  To re-enter, execute (<see topic=\"ACL2____LP\">LP</see>).\u000A? [RAW LISP] (macroexpand-1\u000A              '(<see topic=\"ACL2____RETURN-LAST\">RETURN-LAST</see> 'WITH-PROFILING-RAW\u000A                             '(<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> FGETPROP REWRITE)\u000A                             (MINI-PROVEALL)))\u000A(WITH-PROFILING-RAW '(<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> FGETPROP REWRITE) (MINI-PROVEALL))\u000AT\u000A? [RAW LISP]</code> \u000A \u000A <p>To understand the macro <v>with-profiling-raw</v> you could look at the \u000A community book loaded above: <v>books/misc/profiling-raw.lsp</v>.</p> \u000A \u000A <p>We mentioned above that ACL2 tends to print calls of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> or \u000A <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> (or other such utilities) instead of calls of <v>return-last</v>. \u000A Here we elaborate that point.  ACL2's `<v>untranslate</v>' utility treats \u000A <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> F) X Y)</v> as <v>(G X Y)</v> if <v>F</v> corresponds to the \u000A symbol <v>G</v> in <v>return-last-table</v>.  However, it is generally rare to \u000A encounter such a term during a proof, since calls of <v>return-last</v> are \u000A generally expanded away early during a proof.</p> \u000A \u000A <p>Calls of <v>return-last</v> that occur in code &mdash; forms submitted in the \u000A top-level ACL2 loop, and definition bodies other than those marked as <tt><see topic=\"ACL2____NON-EXECUTABLE\">non-executable</see></tt> (see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>) &mdash; have the following restriction: \u000A if the first argument is of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> F)</v>, then <v>F</v> must be an \u000A entry in <v>return-last-table</v>.  There are however four exceptions: the \u000A following symbols are considered to be keys of <v>return-last-table</v> even if \u000A they are no longer associated with non-<v>nil</v> values, say because of a \u000A <tt><see topic=\"ACL2____TABLE\">table</see></tt> event with keyword <v>:clear</v>.</p> \u000A \u000A <blockquote><p>* <v>progn</v>, associated with <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt><br></br> \u000A \u000A * <v>mbe1-raw</v>, associated with <v>mbe1</v>, a version of <v>mbe</v><br></br> \u000A \u000A * <v>ec-call1-raw</v>, associated with <v>ec-call1</v> (a variant of <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>)<br></br> \u000A \u000A * <v>with-guard-checking1-raw</v>, associated with <v>with-guard-checking1</v> (a \u000A variant of <tt><see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see></tt>)</p></blockquote> \u000A \u000A <p>Note that because of its special status, it is illegal to trace \u000A <v>return-last</v>.</p> \u000A \u000A <p>We conclude by warning that as a user, you take responsibility for not \u000A compromising the soundness or error handling of ACL2 when you define a macro \u000A in raw Lisp and especially when you install it as a key of <tt><see topic=\"ACL2____RETURN-LAST-TABLE\">return-last-table</see></tt>, either directly or (more likely) using <v>defmacro-last</v>. \u000A In particular, be sure that you are defining a macro of two arguments that \u000A always returns the value of its last argument, returning the complete multiple \u000A value if that last argument evaluates to a multiple value.</p> \u000A \u000A <p>The following is correct, and illustrates care taken to return multiple \u000A values.</p> \u000A \u000A <code>:q\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-time1-raw (val form)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> val))\u000A  `(<see topic=\"COMMON-LISP____LET\">let</see>  ((start-time (get-internal-run-time))\u000A          (result (multiple-value-list ,form))\u000A          (end-time (get-internal-run-time)))\u000A     (format t &quot;Total time: ~s~%&quot;\u000A             (float (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____-\">-</see> end-time start-time)\u000A                       internal-time-units-per-second)))\u000A     (values-list result)))\u000A(<see topic=\"ACL2____LP\">lp</see>)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> my-time1)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-time (form)\u000A  `(my-time1 nil ,form))</code> \u000A \u000A <p>Then for example:</p> \u000A \u000A <code>ACL2 !&gt;(my-time (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 1000000) (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 1000000)))\u000ATotal time: 0.12\u000AT\u000AACL2 !&gt;</code> \u000A \u000A <p>But if instead we provide the following more naive implementation, of the \u000A above raw Lisp macro, the above evaluation can produce an error, for example \u000A if the host Lisp is CCL.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-time1-raw (val form)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> val))\u000A    `(<see topic=\"COMMON-LISP____LET\">let</see>  ((start-time (get-internal-run-time))\u000A            (result ,form)\u000A            (end-time (get-internal-run-time)))\u000A       (format t &quot;Total time: ~s~%&quot;\u000A               (float (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____-\">-</see> end-time start-time)\u000A                         internal-time-units-per-second)))\u000A       result)) ; WRONG -- need multiple values returned!</code> \u000A \u000A <p>Here is a second, similar example.  This time we'll start with the error; \u000A can you spot it?</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t)\u000A (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo-raw (x y)\u000A   `(prog1\u000A        ,y\u000A      (<see topic=\"ACL2____CW\">cw</see> &quot;Message showing argument 1: ~x0~%&quot; ,x))))\u000A(<see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> foo)</code> \u000A \u000A <p>We then can wind up with a hard Lisp error:</p> \u000A \u000A <code>ACL2 !&gt;(foo 3 (<see topic=\"ACL2____MV\">mv</see> 4 5))\u000AMessage showing argument 1: 3\u000A\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  value NIL is not of the expected type REAL.\u000A***********************************************\u000A\u000AIf you didn't cause an explicit interrupt (Control-C),\u000Athen the root cause may be call of a :program mode\u000Afunction that has the wrong guard specified, or even no\u000Aguard specified (i.e., an implicit guard of t).\u000ASee :DOC guards.\u000A\u000ATo enable breaks into the debugger (also see :DOC acl2-customization):\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000AACL2 !&gt;</code> \u000A \u000A <p>Here is a corrected version of the above macro.  The point here is that \u000A <v>prog1</v> returns a single value, while <v>our-multiple-value-prog1</v> returns \u000A all the values that are returned by its first argument.</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t)\u000A (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo-raw (x y)\u000A   `(our-multiple-value-prog1 ;; better\u000A     ,y\u000A     (<see topic=\"ACL2____CW\">cw</see> &quot;Message showing argument 1: ~x0~%&quot; ,x))))</code> \u000A \u000A <p><b>Function: </b><srclink>return-last</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> return-last (fn eager-arg last-arg)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> fn eager-arg)\u000A                (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> fn 'mbe1-raw)\u000A                                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> last-arg eager-arg)\u000A                                  t)))\u000A       last-arg)</code>"],

 "ACL2____Pages_02Written_02Especially_02for_02the_02Tours":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>The ACL2 Home Page is generated from ACL2's online documentation \u000A strings. \u000A (How else could we achieve the total integration of ACL2's online \u000A documentation with the home page?)  This page is just an artifact of the \u000A structure of our documentation strings: each string must belong to a ``major \u000A section'' of the documentation database.  This page is not structured to be \u000A used by a person browsing via the Web.  It contains, in an arbitrary order, \u000A the pages written specificially for the Web user.</p> \u000A \u000A <p>Furthermore, browsing the pages below as text, in particular using the \u000A ACL2 :DOC command, is often unsatisfying because because of the lack of \u000A support for displaying gif files or for going ``back'' to a node just visited. \u000A If you wish to look at the pages below, we strongly recommend that you do so \u000A via a HTML-based Web browser.  Indeed, you should simply visit ACL2's Home \u000A Page and take one of the Tours.</p> \u000A \u000A <p>Generally, the topics listed above will not be of use to the ACL2 \u000A user.</p>"],

 "ACL2____ORDINALS":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Ordinals are used in ACL2 for proving termination in the admission \u000A of recursive function definitions.  For a proof that the ACL2 ordinals are \u000A well-founded, see <see topic=\"ACL2____PROOF-OF-WELL-FOUNDEDNESS\">proof-of-well-foundedness</see>.</p> \u000A \u000A <p>The representation of ordinals changed in ACL2 Version_2.8, and is due to \u000A Pete Manolios and Daron Vroon.  They have also defined algorithms for ordinal \u000A arithmetic, created a library of theorems to reason about ordinal arithmetic, \u000A and written the rest of this documentation in order to explain this change. \u000A We thank them for their efforts.  Although they have provided the \u000A implementation and even modified the community books as needed, we have looked \u000A over their work and are maintaining it (and this documentation); if there are \u000A any bugs, they should be considered ours (Matt Kaufmann and J Moore).</p> \u000A \u000A <p>A book is included for compatibility with the representation before \u000A Version_2.8.  For books that contain events relying on the previous ordinal \u000A implementation, insert the following lines before the first such event:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;ordinals/e0-ordinal&quot; :dir :system)\u000A(<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> e0-ord-&lt;)</code> \u000A \u000A <p>The new ordinal representation is based on a slightly different version of \u000A Cantor Normal Form than that used by the old ordinals.  An advantage of the \u000A new representation is that it is exponentially more succinct than the old \u000A representation.</p> \u000A \u000A <p>While pre-Version_2.8 ACL2 versions provided built-in functions for \u000A checking if an object is an ordinal and for comparing two ordinals, they did \u000A not provide support for reasoning about and constructing ordinals.  The \u000A community books directory <v>books/ordinals</v> provides such support.  First, \u000A it provides efficient algorithms for ordinal arithmetic (including addition, \u000A subtraction, multiplication, and exponentiation).  The algorithms and their \u000A complexity are described in the following paper.</p> \u000A \u000A <code>Manolios, Panagiotis &amp; Vroon, Daron.\u000AAlgorithms for ordinal arithmetic.\u000ABaader, Franz (ed),\u000A19th International Conference on Automated Deduction--CADE-19.\u000APages 243-257 of LNAI, vol. 2741.  Springer-Verlag.</code> \u000A \u000A <p>Second, the algorithms are mechanically verified and libraries of theorems \u000A which can be used to automate reasoning involving the ordinals are provided. \u000A For details, see the following paper.</p> \u000A \u000A <code>Manolios, Panagiotis &amp; Vroon, Daron. \u000AOrdinal arithmetic in ACL2. \u000AKaufmann, Matt, &amp; Moore, J Strother (eds). \u000AFourth International Workshop on the ACL2 Theorem \u000AProver and Its Applications (ACL2-2003), \u000AJuly, 2003. \u000ASee <a href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/'>http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/</a>. \u000A</code> \u000A \u000A <p>We now describe aspects of the above mentioned books in more detail.</p> \u000A \u000A <p>The new ordering function is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> and the new ordinal recognizer is \u000A <tt><see topic=\"ACL2____O-P\">o-p</see></tt>.  See also <tt><see topic=\"ACL2____NATP\">natp</see></tt>, <tt><see topic=\"ACL2____POSP\">posp</see></tt>, <tt><see topic=\"ACL2____O_C3_D3\">o&lt;=</see></tt>, <tt><see topic=\"ACL2____O_E3\">o&gt;</see></tt>, \u000A <tt><see topic=\"ACL2____O_E3_D3\">o&gt;=</see></tt>, <tt><see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see></tt>, <tt><see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see></tt>, <tt><see topic=\"ACL2____O-RST\">o-rst</see></tt>, \u000A <tt><see topic=\"ACL2____MAKE-ORD\">make-ord</see></tt>, <tt><see topic=\"ACL2____O-FINP\">o-finp</see></tt>, and <tt><see topic=\"ACL2____O-INFP\">o-infp</see></tt>.</p> \u000A \u000A <p>The old ordinals were based on the following formulation of Cantor Normal \u000A Form:</p> \u000A \u000A <p>For any ordinal, <v>a &lt; epsilon-0</v>, there exist natural numbers <v>p</v> and \u000A <v>n</v>, and ordinals <v>a1 &gt;= a2 &gt;= ... &gt;= an &gt; 0</v> such that <v>a &gt; a1</v> and \u000A <v>a = w^(a1) + w^(a2) + ... + w^(an) + p</v>.</p> \u000A \u000A <p>Thus, a predicate recognizing ACL2's old ordinals is given by the following \u000A definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> e0-ordinalp (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (e0-ordinalp (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 0))\u000A           (e0-ordinalp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A               (<see topic=\"COMMON-LISP____NOT\">not</see> (e0-ord-&lt; (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))))\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A         (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0))))</code> \u000A \u000A <p>The new representation is based on a corollary to the above theorem, which \u000A we get by the left distributive property of ordinal multiplication over \u000A ordinal addition. Thus, <v>w^a + w^a = (w^a)2</v>, <v>w^a + w^a + w^a = (w^a)3</v> \u000A and so forth. The corollary is as follows:</p> \u000A \u000A <p>For any ordinal, <v>a &lt; epsilon-0</v>, there exist natural numbers <v>p</v> and \u000A <v>n</v>, positive integers <v>x1, x2, ..., xn</v> and ordinals <v>a1 &gt; a2 &gt; ... &gt;\u000A an &gt; 0</v> such that <v>a &gt; a1</v> and <v>a = w^(a1)x1 + w^(a2)x2 + ... + w^(an)xn\u000A + p</v>.</p> \u000A \u000A <p>Instead of representing an ordinal as a list of non-increasing ordinals, we \u000A represent it as a list of exponent-coefficient pairs, such that the exponents \u000A are strictly decreasing (see <see topic=\"ACL2____O-P\">o-p</see>).  Note that this representation is \u000A exponentially more efficient than the old representation.</p> \u000A \u000A <p>The ordinal arithmetic functions: <v>o+</v>, <v>o-</v>, <v>o*</v>, and <v>o^</v> \u000A are defined in the ordinals library (in the community books directory \u000A <v>books/ordinals</v>). To use them, include the book \u000A <v>ordinals-without-arithmetic</v> or <v>ordinals</v>, depending on whether you \u000A want the arithmetic books included or not (<v>ordinals</v> includes community \u000A book <v>books/arithmetic/top-with-meta</v>). To use the old ordinals, include \u000A the book <v>e0-ordinal</v> and run the command <v>(<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see>\u000A e0-ord-&lt;)</v></p> \u000A \u000A <p>The community book <see topic=\"ACL2____ARITHMETIC_F2NATP-POSP\">arithmetic/natp-posp</see> is a book for reasoning \u000A about <v>posp</v> and <v>natp</v>.  We recommend using this book if you have to \u000A reason about <v>posp</v> and <v>natp</v>.  It is included in community book \u000A <v>books/arithmetic/top</v>, which is included in community book \u000A <v>books/arithmetic/top-with-meta</v>, which is included in community book \u000A <v>books/ordinals/ordinals</v>.</p> \u000A \u000A <p>If you have a good reason to use the old definitions of the ordinals (e.g., \u000A because of legacy code and theorems), then we provide a convenient way to do \u000A this.  The book <v>ordinal-isomorphism</v> proves that the new ordinals are \u000A order-isomorphic to the old ordinals and thus theorems proved in one context \u000A can be directly transferred to the other.  For an example of how to do this, \u000A look at the book <v>defmul</v> in the community books directory \u000A <v>books/workshops/2000/ruiz/multiset</v>.</p> \u000A \u000A <p>The ordinals books have been used to prove non-trivial theorems.  For a \u000A good example, see the books in the community books directory \u000A <v>books/workshops/2003/sustik/support</v>, where Matyas Sustik proves Dickson's \u000A lemma.</p> \u000A \u000A <p>Finally, many termination proofs can be carried out with weaker orderings \u000A than the ordinals up to <v>epsilon-0</v>.  For example, many inductive theorem \u000A provers only know that the lexicographic ordering on natural numbers is \u000A well-founded.  The book <v>lexicographic-ordering</v> contains a definition of \u000A such an ordering <v>l&lt;</v> whose arguments are either a list of natural numbers, \u000A or a natural number.  In the book we prove that <v>l&lt;</v> is well-founded (that \u000A is, we prove a <v>:well-founded-relation</v> <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> and provide a macro \u000A <v>llist</v> to simplify the generation of measure functions.  We also show how \u000A to use <v>l&lt;</v> to prove that the famous Ackermann function terminates. \u000A Finally, since <v>l&lt;</v> does something reasonable with natural numbers, it gets \u000A along with <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt>, the default measure chosen by ACL2.</p>"],

 "ACL2____LOGIC":[["Defun-mode"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 p!&gt;:logic\u000AACL2 !&gt;</code> \u000A \u000A <p>Typing the keyword <v>:logic</v> sets the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to \u000A <v>:logic</v>.</p> \u000A \u000A <p>Functions defined in <v>:logic</v> mode are logically defined.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> for a discussion of the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s available \u000A and what their effects on the logic are.  See <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> for a \u000A discussion of how the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is used.  This event is \u000A equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :logic)</v>, and hence \u000A is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in \u000A which it occurs. See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>Recall that the top-level form <v>:logic</v> is equivalent to <v>(<see topic=\"ACL2____LOGIC\">logic</see>)</v>; \u000A see <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.  Thus, to change the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to \u000A <v>:logic</v> in a book, use <v>(<see topic=\"ACL2____LOGIC\">logic</see>)</v>, which is an embedded event form, \u000A rather than <v>:logic</v>, which is not a legal form for <see topic=\"ACL2____BOOKS\">books</see>.  See \u000A <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.</p>"],

 "ACL2____COMPUTED-HINTS":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>General Form of :hints:\u000A(hint1 hint2 ... hintk)</code> \u000A \u000A <p>Each element, hinti, must be either a common hint or a computed hint.</p> \u000A \u000A <p>A common hint is of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____GOAL-SPEC\">goal-spec</see> :key1 val1 ... :keyn valn)</code> \u000A \u000A <p>where <v>goal-spec</v> is as specified in <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see> and each \u000A <v>:keyi</v> and <v>vali</v> is as specified in <see topic=\"ACL2____HINTS\">hints</see>.  Among the ``common \u000A hints'' we include both the primitive hints and user-defined custom keyword \u000A hints (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>).</p> \u000A \u000A <p>A computed hint may be a function symbol, <v>fn</v>, of three, four or seven \u000A arguments.  Otherwise, a computed hint is a term with the following \u000A properties:</p> \u000A \u000A <p>(a) the only free variables allowed in the term are <v>ID</v>, <v>CLAUSE</v>, \u000A <v>WORLD</v>, <v>STABLE-UNDER-SIMPLIFICATIONP</v>, <v>HIST</v>, <v>PSPV</v>, <v>CTX</v>, \u000A and <tt><see topic=\"ACL2____STATE\">state</see></tt>;</p> \u000A \u000A <p>(b) the output signature of the term is either <v>(<see topic=\"ACL2____MV\">MV</see> * * STATE)</v>, which \u000A is to be treated as an error triple (see below), or is <v>*</v>, denoting a \u000A single non-<see topic=\"ACL2____STOBJ\">stobj</see> value; and</p> \u000A \u000A <p>(c) in the former case of (b) above, the term is single-threaded in <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>If a computed hint is a function symbol <v>fn</v>, whose arity n is therefore \u000A three, four, or seven, then it is treated as the term resulting from applying \u000A that <v>fn</v> to the first n variables shown in (a) above.  Notice that it must \u000A then return a single non-<see topic=\"ACL2____STOBJ\">stobj</see> value, not an error triple, since \u000A <v>state</v> is not one of the first seven variables shown in (a).</p> \u000A \u000A <p>Note: Error triples are an ACL2 idiom for implementing ``errors''; see \u000A <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.  If a computation returns <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> in a \u000A context in which ACL2 is respecting the error triple convention (see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see> and see <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see>), then an error is deemed to \u000A have occurred if <v>erp</v> is non-<v>nil</v>.  The computation is expected to \u000A have printed an appropriate error message to <tt><see topic=\"ACL2____STATE\">state</see></tt> and further \u000A computation is aborted.  On the other hand, if a computation returns an error \u000A triple in which <v>erp</v> is nil, then ``value'' of the computation is taken to \u000A be the second component, <v>val</v>, of the triple (along with the possibly \u000A modified <tt><see topic=\"ACL2____STATE\">state</see></tt>), and computation continues.  For more information \u000A about programming with error triples, see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.</p> \u000A \u000A <p>The function symbol cases are treated as abbreviations of the term <v>(fn\u000A ID CLAUSE WORLD)</v>, <v>(fn ID CLAUSE WORLD STABLE-UNDER-SIMPLIFICATIONP)</v>, or \u000A <v>(fn ID CLAUSE WORLD STABLE-UNDER-SIMPLIFICATIONP HIST PSPV CTX)</v> as \u000A appropriate for the arity of <v>fn</v>.  (Note that this tells you which \u000A argument of <v>fn</v> is which.)  Moreover, in these cases the value returned \u000A must be a single ordinary (non-<see topic=\"ACL2____STOBJ\">stobj</see>) value, not an error triple.  In \u000A the discussion below we assume all computed hints are of the term form. \u000A Indeed, we almost assume all computed hints are of the 3 and 4 argument forms. \u000A We only comment briefly on the 7 argument form in <see topic=\"ACL2____USING-COMPUTED-HINTS-8\">using-computed-hints-8</see>.</p> \u000A \u000A <p>The semantics of a computed hint term is as follows.  On every subgoal, the \u000A term is evaluated in an environment in which the variables mentioned in (a) \u000A above are bound to context-sensitive values explained below.  Either the \u000A computed hint signals an error, in which the proof attempt aborts, or else it \u000A returns a value, <v>val</v> and a new state, <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Any changes to \u000A those parts of <tt><see topic=\"ACL2____STATE\">state</see></tt> that affect logical soundness are undone; more \u000A specifically, the values of symbols (sometimes called ``state global \u000A variables'') in the list <v>*protected-system-state-globals*</v> in the global \u000A table of the state (see <see topic=\"ACL2____STATE\">state</see>) are restored when changed during \u000A evaluation.  The value, <v>val</v>, of a non-erroneous computed hint calculation \u000A is either <v>nil</v>, which means the computed hint did not apply to the subgoal \u000A in question, or it is an alternating list of <v>:keyi vali</v> pairs as \u000A specified in <see topic=\"ACL2____HINTS\">hints</see>.  With one exception, those new hints are applied to \u000A the given subgoal as though the user had typed them explicitly.</p> \u000A \u000A <p>The exception is that the first keyword in the returned <v>val</v> is allowed \u000A to be <v>:COMPUTED-HINT-REPLACEMENT</v>.  Its value should be <v>nil</v>, <v>t</v>, \u000A or a list of terms.  If this keyword is not present, the default value of \u000A <v>nil</v> is provided.  We explain <v>:COMPUTED-HINT-REPLACEMENT</v> below.</p> \u000A \u000A <p>The evaluation of a hint term is done with guard checking turned off (see \u000A <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>); e.g., the form <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 23)</v> in a computed hint \u000A returns <v>nil</v> as per the axioms.</p> \u000A \u000A <p>When a non-<v>nil</v> value is returned, the keyword/value pairs (other than \u000A the optional <v>:COMPUTED-HINT-REPLACEMENT</v>) are used as the hint for the \u000A subgoal in question.  Thus, your job as the programmer of computed hints is \u000A either to cause an error, typically by invoking <tt><see topic=\"ACL2____ER\">er</see></tt>, or to return a \u000A non-erroneous value triple whose value is the list of keys and values you \u000A would have typed had you supplied a common hint for the subgoal. (In \u000A particular, any theory expressions in it are evaluated with respect to the \u000A global current-theory, not whatever theory is active at the subgoal in \u000A question.)  If the generated list of keywords and values is illegal, an error \u000A will be signaled and the proof attempt will be aborted.</p> \u000A \u000A <p>The purpose of the <v>:COMPUTED-HINT-REPLACEMENT</v> keyword and its value, \u000A <v>chr</v>, is to change the list of hints.  If <v>chr</v> is <v>nil</v>, then the \u000A hint which was applied is removed from the list of hints that is passed down \u000A to the children of the subgoal in question.  This is the default.  If <v>chr</v> \u000A is <v>t</v>, then the hint is left in the list of hints.  This means that the \u000A same hint may act on the children of the subgoal.  Otherwise, <v>chr</v> must be \u000A a list of terms, each of which is treated as a computed hint.  The hint which \u000A was applied is deleted from the list of hints and the hints in <v>chr</v> are \u000A added to the list of hints passed to the children of the subgoal.  The ability \u000A to compute new hints and pass them down allows strange and wonderful \u000A behavior.</p> \u000A \u000A <p>For these purposes, the goals produced by induction and the top-level goals \u000A of forcing rounds are not considered children; all original hints are \u000A available to them.</p> \u000A \u000A <p>Only the first hint applicable to a goal, as specified in the user-supplied \u000A list of <v>:hints</v> followed by the <tt><see topic=\"ACL2____DEFAULT-HINTS-TABLE\">default-hints-table</see></tt>, will be \u000A applied to that goal.  (For an advanced exception, see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>.)</p> \u000A \u000A <p>It remains only to describe the bindings of the free variables.</p> \u000A \u000A <p>Suppose the theorem prover is working on some clause, clause, named by some \u000A <tt><see topic=\"ACL2____GOAL-SPEC\">goal-spec</see></tt>, e.g., \"Subgoal *1/2'''\" in some logical world, world. \u000A Corresponding to the printed <v>goal-spec</v> is an internal data structure \u000A called a ``clause identifier'' id.  See <see topic=\"ACL2____CLAUSE-IDENTIFIER\">clause-identifier</see>.</p> \u000A \u000A <p>In the case of a common hint, the hint applies if the goal-spec of the hint \u000A is the same as the goal-spec of the clause in question.</p> \u000A \u000A <p>In the case of a computed hint, the variable <v>ID</v> is bound to the clause \u000A id, the variable <v>CLAUSE</v> is bound to the (translated form of the) clause, \u000A and the variable <v>WORLD</v> is bound to the current ACL2 world.  The variable \u000A <v>STABLE-UNDER-SIMPLIFICATIONP</v> is bound to <v>t</v> or <v>nil</v>.  It is bound \u000A to <v>t</v> only if the clause is known to be stable under simplification.  That \u000A is, the simplifier has been applied to the clause and did not change it.  Such \u000A a clause is sometimes known as a ``simplification checkpoint.''  It is \u000A frequently useful to inject hints (e.g., to enable a rule or provide a \u000A <v>:use</v> hint) only when the goal in question has stabilized.  If a hint is \u000A provided, the processing of the clause starts over with simplification.</p> \u000A \u000A <p>As for <v>CTX</v> and <tt><see topic=\"ACL2____STATE\">state</see></tt>, they are provided so that you can pass \u000A them to the <tt><see topic=\"ACL2____ER\">er</see></tt> macro to print error messages.  We recommend not \u000A writing computed hints that otherwise change <tt><see topic=\"ACL2____STATE\">state</see></tt>!</p> \u000A \u000A <p>The remaining variables, <v>HIST</v> and <v>PSPV</v> are not documented yet. \u000A Only users familiar with the internals of ACL2 are likely to need them or \u000A understand their values.</p> \u000A \u000A <p>For some instruction about how to use computed hints, see <see topic=\"ACL2____USING-COMPUTED-HINTS\">using-computed-hints</see>.</p>"],

 "ACL2____SKIP-PROOFS":[["Other"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) nil (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (foo (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> form)</code> \u000A \u000A <p>where <v>form</v> is processed as usual except that the proof obligations \u000A usually generated are merely assumed.</p> \u000A \u000A <p>Normally <v>form</v> is an event; see <see topic=\"ACL2____EVENTS\">events</see>.  If you want to put \u000A <v>skip-proofs</v> around more than one event, consider the following (see <see topic=\"COMMON-LISP____PROGN\">progn</see>): <v>(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> (<see topic=\"COMMON-LISP____PROGN\">progn</see> event1 event2 ... eventk))</v>.</p> \u000A \u000A <p>WARNING: <v>Skip-proofs</v> allows inconsistent <see topic=\"ACL2____EVENTS\">events</see> to be admitted \u000A to the logic.  Use it at your own risk!</p> \u000A \u000A <p>Sometimes in the development of a formal model or proof it is convenient to \u000A skip the proofs required by a given event.  By embedding the event in a \u000A <v>skip-proofs</v> form, you can avoid the proof burdens generated by the event, \u000A at the risk of introducing unsoundness.  Below we list four illustrative \u000A situations in which you might find <v>skip-proofs</v> useful.</p> \u000A \u000A <p>1. The termination argument for a proposed function definition is \u000A complicated.  You presume you could admit it, but are not sure that your \u000A definition has the desired properties.  By embedding the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event \u000A in a <v>skip-proofs</v> you can ``admit'' the function and experiment with \u000A theorems about it before undoing (see <see topic=\"ACL2____UBT\">ubt</see>) and then paying the price of \u000A its admission.  Note however that you might still have to supply a measure. \u000A The set of formals used in some valid measure, known as the ``measured \u000A subset'' of the set of formals, is used by ACL2's induction heuristics and \u000A therefore needs to be suitably specified.  You may wish to specify the special \u000A measure of <v>(:? v1 ... vk)</v>, where <v>(v1 ... vk)</v> enumerates the measured \u000A subset.</p> \u000A \u000A <p>2. You intend eventually to verify the <see topic=\"ACL2____GUARD\">guard</see>s for a definition but \u000A do not want to take the time now to pursue that.  By embedding the <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event in a <v>skip-proofs</v> you can get the system to behave as \u000A though the <see topic=\"ACL2____GUARD\">guard</see>s were verified.</p> \u000A \u000A <p>3. You are repeatedly recertifying a book while making many experimental \u000A changes.  A certain <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> in the book takes a very long time to prove \u000A and you believe the proof is not affected by the changes you are making.  By \u000A embedding the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event in a <v>skip-proofs</v> you allow the theorem \u000A to be assumed without proof during the experimental recertifications.</p> \u000A \u000A <p>4. You are constructing a proof top-down and wish to defer the proof of a \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> until you are convinced of its utility.  You can embed the \u000A <v>defthm</v> in a <v>skip-proofs</v>.  Of course, you may find later (when you \u000A attempt prove the theorem) that the proposed <v>defthm</v> is not a theorem.</p> \u000A \u000A <p>Unsoundness or Lisp errors may result if the presumptions underlying a use \u000A of <v>skip-proofs</v> are incorrect.  Therefore, <v>skip-proofs</v> must be \u000A considered a dangerous (though useful) tool in system development.</p> \u000A \u000A <p>Roughly speaking, a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> embedded in a <v>skip-proofs</v> is \u000A essentially a <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>, except that it is not noted as an axiom for \u000A the purposes of functional instantiation (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>).  But a \u000A skipped <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is much more subtle since not only is the definitional \u000A equation being assumed but so are formulas relating to termination and type. \u000A The situation is also difficult to characterize if the <v>skip-proofs</v> <see topic=\"ACL2____EVENTS\">events</see> are within the scope of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> in which constrained \u000A functions are being introduced.  In such contexts no clear logical story is \u000A maintained; in particular, constraints aren't properly tracked for \u000A definitions.  A proof script involving <v>skip-proofs</v> should be regarded as \u000A work-in-progress, not as a completed proof with some unproved assumptions.  A \u000A <v>skip-proofs</v> event represents a promise by the author to admit the given \u000A event without further axioms.  In other words, <v>skip-proofs</v> should only be \u000A used when the belief is that the proof obligations are indeed theorems in the \u000A existing ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>ACL2 allows the certification of <see topic=\"ACL2____BOOKS\">books</see> containing <v>skip-proofs</v> \u000A <see topic=\"ACL2____EVENTS\">events</see> by providing the keyword argument <v>:skip-proofs-okp t</v> to the \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command.  This is contrary to the spirit of certified \u000A <see topic=\"ACL2____BOOKS\">books</see>, since one is supposedly assured by a valid <see topic=\"ACL2____CERTIFICATE\">certificate</see> \u000A that a book has been ``blessed.''  But certification, too, takes the view of \u000A <v>skip-proofs</v> as ``work-in-progress'' and so allows the author of the book \u000A to promise to finish.  When such <see topic=\"ACL2____BOOKS\">books</see> are certified, a warning to the \u000A author is printed, reminding him or her of the incurred obligation.  When \u000A <see topic=\"ACL2____BOOKS\">books</see> containing <v>skip-proofs</v> are included into a session, a \u000A warning to the user is printed, reminding the user that the book is in fact \u000A incomplete and possibly inconsistent.  This warning is in fact an error if \u000A <v>:skip-proofs-okp</v> is <v>nil</v> in the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form; see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.</p> \u000A \u000A <p>We conclude with a technical note.  <v>Skip-proofs</v> works by binding the \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> special <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> to <v>t</v> unless it is already \u000A bound to a non-<v>nil</v> value; see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.</p>"],

 "ACL2____SET-GAG-MODE":[["Prover-output"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A:set-gag-mode t      ; enable gag-mode, suppressing most proof commentary\u000A(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> t)     ; same as above\u000A:set-gag-mode :goals ; same as above, but print names of goals when produced\u000A:set-gag-mode nil    ; disable gag-mode\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> val)\u000A:set-gag-mode val</code> \u000A \u000A <p>where <v>val</v> is one of <v>t</v>, <v>nil</v>, or <v>:goals</v>.</p> \u000A \u000A <p>The basic idea of <see topic=\"ACL2____GAG-MODE\">gag-mode</see> is to avoid much of the verbose output \u000A from the theorem prover, leaving only output that is expected to be helpful. \u000A The first two forms below set gag-mode on, while the other turns it off; these \u000A may be placed in your ACL2 customization file; see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> :goals) ; (<see topic=\"ACL2____DEFAULT\">default</see>) avoid most prover output; show goal names\u000A(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> t)      ; avoid most prover output; also, hide goal names\u000A(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> nil)    ; allow prover output</code> \u000A \u000A <p>Gag-mode focuses attention on so-called ``key checkpoints''.  By default, a \u000A checkpoint is a goal that cannot be simplified.  (Below we discuss how to \u000A change this default.)  A key checkpoint is a checkpoint that is not descended \u000A from another checkpoint.  (Technical point: ``descended'' signifies that both \u000A goals are at the top level in the same forcing round, or are in the same proof \u000A by induction.)  Successful ACL2 users generally focus their attention on key \u000A checkpoints, or less often, induction schemes.  (To suppress or abbreviate \u000A induction schemes in gag-mode, see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>, specifically, the \u000A discussion of <v>:GAG-MODE</v>.)  For a discussion of how to use ACL2 prover \u000A output in an effective manner, see <see topic=\"ACL2____THE-METHOD\">the-method</see>, and see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for a more detailed tutorial.  In \u000A gag-mode, a key checkpoint is only displayed when ACL2 is unable to make any \u000A further progress on that goal or some descendent of it, other than with a \u000A proof by induction.</p> \u000A \u000A <p>Evaluation of <v>set-gag-mode t</v> enters gag-mode, so that only key \u000A checkpoints are printed.  Evaluation of <v>set-gag-mode :goals</v> also enters \u000A gag-mode, but will additionally cause the name of a goal to be printed as soon \u000A as it is generated (by invoking <see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see>).  The <v>:goals</v> \u000A setting is the default, and is useful for cases in which the prover spends \u000A very little of its time generating goals to be proved by induction, yet you \u000A want to see that it is making progress.  For finer-grained feedback about the \u000A simplifier's activity, see <see topic=\"ACL2____DMR\">dmr</see>.</p> \u000A \u000A <p>The current value of <see topic=\"ACL2____GAG-MODE\">gag-mode</see> is returned by a macro of the same \u000A name:</p> \u000A \u000A <code>(<see topic=\"ACL2____GAG-MODE\">gag-mode</see>) ; evaluates to t, nil, or :goals</code> \u000A \u000A <p>An alternative to gag-mode is to use proof-trees; see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>. \u000A With proof-trees it is not so important to avoid excessive prover output, \u000A since the proof-tree display provides structure that makes it easy to monitor \u000A proof attempts and navigate output for a proof that has failed or seems to be \u000A failing.  Still, output can take time to print, so you may get better \u000A performance with gag-mode.</p> \u000A \u000A <p>The intention of gag-mode is to show you only the parts of a proof attempt \u000A that are relevant for debugging a failure; additional output is generally more \u000A likely to be distracting than truly helpful.  But on occasion you may want to \u000A see the full proof output after an attempt made with gag-mode.  This can be \u000A done provided proof output is not inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) during the proof attempt; see <see topic=\"ACL2____PSO\">pso</see> and see \u000A <see topic=\"ACL2____PSO_12\">pso!</see>.  Since <v>set-gag-mode</v> takes responsibility for the saving of \u000A output, related utility <tt><see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see></tt> is disabled when gag-mode is \u000A active.  Also note that calling <v>set-gag-mode</v> erases the currently saved \u000A output, if any.</p> \u000A \u000A <p>You may notice that gag-mode tends to print relatively little information \u000A about goals pushed for proof by sub-induction &mdash; i.e., a proof of *i.j, \u000A *i.j.k, etc.  That feature emphasizes that unsuccessful sub-inductions should \u000A generally be avoided, not analyzed for ways to make them succeed.  Instead, \u000A the key checkpoint that generated the goal pushed for this induction is more \u000A appropriate to analyze.  In general, the ``higher level'' the checkpoint, the \u000A more worthy it is of attention.  Thus, we suggest that look at the top-level \u000A checkpoints before looking at those labeled ``Key checkpoints under a \u000A top-level induction''.</p> \u000A \u000A <p>We conclude with remarks for advanced users.</p> \u000A \u000A <p>The notion of ``checkpoint'' can be modified by the user.  The default, as \u000A discussed above, is for a checkpoint to be a goal that cannot be simplified. \u000A Put differently, a checkpoint is acted on by one of the processes in the value \u000A of the form <v>(<see topic=\"ACL2_____04\">@</see> checkpoint-processors)</v>; see <see topic=\"ACL2_____04\">@</see>.  Any or all of the \u000A symbols <v>eliminate-destructors-clause</v>, <v>fertilize-clause</v>, \u000A <v>generalize-clause</v>, or <v>eliminate-irrelevance-clause</v> can be removed \u000A from this value in order that invocation of the corresponding proof process \u000A does not cause its input goal to be labeled a checkpoint.  For example, if you \u000A do not want destructor elimination to be treated differently from \u000A simplification for purposes of labeling checkpoints, you can evaluate the \u000A following form (see <see topic=\"ACL2____ASSIGN\">assign</see>):</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> checkpoint-processors\u000A        (<see topic=\"COMMON-LISP____REMOVE\">remove</see> 'eliminate-destructors-clause\u000A                (<see topic=\"ACL2_____04\">@</see> checkpoint-processors)))</code> \u000A \u000A <p>Note that the value of <v>(<see topic=\"ACL2_____04\">@</see> checkpoint-processors)</v> also affects the \u000A proof tree display; see <see topic=\"ACL2____PROOF-TREE-DETAILS\">proof-tree-details</see>.  End of Remark.)</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>, in particular the discussion there of \u000A <v>:GAG-MODE</v>, for how to influence slightly just what is printed in \u000A gag-mode.</p>"],

 "ACL2____CONGRUENCE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes \u000A and how they are used to build rules from formulas.  An example <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula from which a <v>:congruence</v> rule might be built, assuming \u000A that <v>set-equal</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation, is:</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> set-equal-implies-iff-memb-2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (set-equal x y)\u000A           (<see topic=\"ACL2____IFF\">iff</see> (memb e x) (memb e y)))\u000A  :rule-classes :congruence)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFCONG\">defcong</see> and see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>.</p> \u000A \u000A <p>NOTE: This topic discusses so-called ``classic'' congruence rules.  A more \u000A general class of rules, so-called ``patterned'' congruence rules, is \u000A supported.  We discuss only classic congruence rules below; for a discussion \u000A of patterned congruence rules, first read the present topic and then see <see topic=\"ACL2____PATTERNED-CONGRUENCE\">patterned-congruence</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 xk xk-equiv)\u000A         (equiv2 (fn x1... xk       ...xn)\u000A                 (fn x1... xk-equiv ...xn)))</code> \u000A \u000A <p>where <v>equiv1</v> and <v>equiv2</v> are known equivalence relations, <v>fn</v> \u000A is an <v>n-ary</v> function symbol other than <v>if</v>, and the <v>xi</v> and \u000A <v>xk-equiv</v> are all distinct variables.  The effect of such a rule is to \u000A record that the <v>equiv2</v>-equivalence of <v>fn</v>-expressions can be \u000A maintained if, while rewriting the <v>kth</v> argument position, \u000A <v>equiv1</v>-equivalence is maintained.  See <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> for a general \u000A discussion of the issues.  We say that <v>equiv2</v>, above, is the ``outside \u000A equivalence'' in the rule and <v>equiv1</v> is the ``inside equivalence for the \u000A <v>k</v>th argument.''</p> \u000A \u000A <p>The macro form <v>(<see topic=\"ACL2____DEFCONG\">defcong</see> equiv1 equiv2 (fn x1 ... x1) k)</v> is an \u000A abbreviation for a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> of rule-class <v>:congruence</v> that attempts \u000A to establish that <v>equiv2</v> is maintained by maintaining <v>equiv1</v> in \u000A <v>fn</v>'s <v>k</v>th argument.  The <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> macro automatically \u000A generates the general formula shown above.  See <see topic=\"ACL2____DEFCONG\">defcong</see>.</p> \u000A \u000A <p>The <v>memb</v> example above tells us that <v>(memb e x)</v> is \u000A propositionally equivalent to <v>(memb e y)</v>, provided <v>x</v> and <v>y</v> are \u000A <v>set-equal</v>.  The outside equivalence is <tt><see topic=\"ACL2____IFF\">iff</see></tt> and the inside \u000A equivalence for the second argument is <v>set-equal</v>.  If we see a <v>memb</v> \u000A expression in a propositional context, e.g., as a literal of a clause or test \u000A of an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> (but not, for example, as an argument to <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>), we \u000A can rewrite its second argument maintaining <v>set-equality</v>.  For example, a \u000A rule stating the commutativity of <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> (modulo set-equality) could \u000A be applied in this context.  Since equality is a refinement of all equivalence \u000A relations, all equality rules are always available.  See <see topic=\"ACL2____REFINEMENT\">refinement</see>.</p> \u000A \u000A <p>All known <v>:congruence</v> rules about a given outside equivalence and \u000A <v>fn</v> can be used independently.  That is, consider two <v>:congruence</v> \u000A rules with the same outside equivalence, <v>equiv</v>, and about the same \u000A function <v>fn</v>.  Suppose one says that <v>equiv1</v> is the inside equivalence \u000A for the first argument and the other says <v>equiv2</v> is the inside \u000A equivalence for the second argument.  Then <v>(fn a b)</v> is <v>equiv</v> <tt>(fn \u000A a' b')</tt> provided <v>a</v> is <v>equiv1</v> to <v>a'</v> and <v>b</v> is \u000A <v>equiv2</v> to <v>b'</v>.  This is an easy consequence of the transitivity of \u000A <v>equiv</v>.  It permits you to think independently about the inside \u000A equivalences.</p> \u000A \u000A <p>Furthermore, it is possible that more than one inside equivalence for a \u000A given argument slot will maintain a given outside equivalence.  For example, \u000A <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> a)</v> is equal to <tt>(length a')</tt> if <v>a</v> and <v>a'</v> are \u000A related either by <v>list-equal</v> or by <tt><see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see></tt>.  You may prove \u000A two (or more) <v>:congruence</v> rules for the same slot of a function.  The \u000A result is that the system uses a new, ``generated'' equivalence relation for \u000A that slot with the result that rules of both (or all) kinds are available \u000A while rewriting.</p> \u000A \u000A <p><v>:Congruence</v> rules can be disabled.  For example, if you have two \u000A different inside equivalences for a given argument position and you find that \u000A the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules for one are unexpectedly preventing the \u000A application of the desired rule, you can disable the rule that introduced the \u000A unwanted inside equivalence.</p> \u000A \u000A <p><i>Remark on Replacing IFF by EQUAL.</i> You may encounter a warning \u000A suggesting that a congruence rule ``can be strengthened by replacing the \u000A second equivalence relation, IFF, by EQUAL.''  Suppose for example that this \u000A warning occurs when you submit the following rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONG\">defcong</see> equiv1 iff (fn x y) 2)</code> \u000A \u000A <p>which is shorthand for the following:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> equiv1-implies-iff-fn-2\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 y y-equiv)\u000A                (<see topic=\"ACL2____IFF\">iff</see> (fn x y) (fn x y-equiv)))\u000A       :rule-classes (:congruence))</code> \u000A \u000A <p>The warning is telling you that ACL2 was able to deduce that <v>fn</v> always \u000A returns a Boolean, and hence a trivial but useful consequence is obtained by \u000A replacing <tt><see topic=\"ACL2____IFF\">iff</see></tt> by <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> &mdash;</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONG\">defcong</see> equiv1 equal (fn x y) 2)</code> \u000A \u000A <p>&mdash; which is shorthand for the following:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> equiv1-implies-equal-fn-2\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 y y-equiv)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x y) (fn x y-equiv)))\u000A       :rule-classes (:congruence))</code> \u000A \u000A <p>If you have difficulty proving the latter directly, you can derive it from \u000A the former by giving a suitable hint, minimally as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONG\">defcong</see> equiv1 equal (fn x y) 2\u000A  :hints ((&quot;Goal&quot;\u000A           :use equiv1-implies-iff-fn-2\u000A           :in-theory\u000A           (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> '((:type-prescription fn))\u000A                           (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)))))</code> \u000A \u000A <p>By heeding this warning, you may avoid unnecessary <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt> \u000A warnings later.  We now explain why, but see <see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> for \u000A relevant background material.</p> \u000A \u000A <p>For example, suppose you have proved the ``<v>iff</v>'' version of the \u000A congruence rule above, and later you submit the following rewrite rule.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> equal-list-perm\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 x y)\u000A           (fn x y)))</code> \u000A \u000A <p>Since <v>fn</v> is known to return a Boolean, ACL2 performs an optimization \u000A that stores this rule as though it were the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> equal-list-perm\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 x y)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x y) t)))</code> \u000A \u000A <p>Thus, if ACL2's rewriter sees a term <v>(fn a b)</v> in a context where the \u000A equivalence relation <tt><see topic=\"ACL2____IFF\">iff</see></tt> is not being maintained, then it cannot use \u000A rule <v>equiv1-implies-iff-fn-2</v>, so it rewrites argument <v>a</v> without the \u000A benefit of knowing that it suffices to maintain <v>equiv1</v>; and then it \u000A caches the result.  When ACL2 subsequently attempts to relieve the hypothesis \u000A <v>(equiv1 x y)</v>, it will rewrite <v>x</v> simply by returning the rewritten \u000A value of <v>a</v> from the result cache.  This is unfortunate if <v>a</v> could \u000A have been rewritten more completely under maintainance of the equivalence \u000A relation <v>equiv1</v> &mdash; which is legal in the hypothesis since <v>a</v> is \u000A an argument of <v>equiv1</v>, which is an <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation.  The user \u000A who observes the warning from rule <v>equiv1-implies-iff-fn-2</v>, and replaces \u000A it with <v>equiv1-implies-equal-fn-2</v>, will avoid this unfortunate \u000A case.</p>"],

 "ACL2____PROOF-TREE":[["Debugging"],"ACL2 Sources","ACL2","<p>A view of ACL2 proofs may be obtained by way of ``proof tree \u000A displays,'' which appear in proof output (see <see topic=\"ACL2____PROOFS-CO\">proofs-co</see>) when \u000A proof-tree output is enabled (see below) When ACL2 starts a proof and \u000A proof-tree output is enabled, the proof output begins with the following \u000A string.</p> \u000A \u000A <code>&lt;&lt; Starting proof tree logging &gt;&gt;</code> \u000A \u000A <p>Then for each goal encountered during the proof, a corresponding proof tree \u000A display (as described below) is printed into the proof output: first the \u000A characters in the constant string <v>*proof-tree-start-delimiter*</v> are \u000A printed, then the proof tree display, and finally the characters in the \u000A constant string <v>*proof-tree-end-delimiter*</v>.</p> \u000A \u000A <p>External tools may present proof tree displays in a separate window.  In \u000A particular, a tool distributed with the ACL2 community books customizes the \u000A emacs environment to provide window-based proof tree displays together with \u000A commands for traversing the proof transcript; see the discussion of ``ACL2 \u000A proof-tree support'' in file <v>emacs/emacs-acl2.el</v> distributed with \u000A ACL2.</p> \u000A \u000A <p>The command <v>:start-proof-tree</v> enables proof-tree output, while \u000A <v>:stop-proof-tree</v> disables proof-tree output; see <see topic=\"ACL2____START-PROOF-TREE\">start-proof-tree</see> \u000A and see <see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see>.</p> \u000A \u000A <p>Here is an example of a proof tree display, with comments.  Lines marked \u000A with ``c'' are considered ``checkpoints,'' i.e., goals whose scrutiny may be \u000A of particular value.</p> \u000A \u000A <code>( DEFTHM PLUS-TREE-DEL ...)    ;currently proving PLUS-TREE-DEL\u000A   1 Goal preprocess   ;&quot;Goal&quot; creates 1 subgoal by preprocessing\u000A   2 |  Goal' simp     ;&quot;Goal'&quot; creates 2 subgoals by simplification\u000Ac  0 |  |  Subgoal 2 PUSH *1   ;&quot;Subgoal 2&quot; pushes &quot;*1&quot; for INDUCT\u000A++++++++++++++++++++++++++++++ ;first pass thru waterfall completed\u000Ac  6 *1 INDUCT                 ;Proof by induction of &quot;*1&quot; has\u000A     |  &lt;5 more subgoals&gt;      ; created 6 top-level subgoals.  At\u000A                               ; this point, one of those 6 has been\u000A                               ; proved, and 5 remain to be proved.\u000A                               ; We are currently working on the\u000A                               ; first of those 5 remaining goals.</code> \u000A \u000A <p>See <see topic=\"ACL2____PROOF-TREE-EXAMPLES\">proof-tree-examples</see> for many examples that contain proof tree \u000A displays.  But first, we summarize the kinds of lines that may appear in a \u000A proof tree display.  The simplest form of a proof tree display is a header \u000A showing the current event, followed by list of lines, each having one of the \u000A following forms.</p> \u000A \u000A <code>n &lt;goal&gt; &lt;process&gt; ...</code> \u000A \u000A <p>Says that the indicated goal created <v>n</v> subgoals using the indicated \u000A process.  Here ``...'' refers to possible additional information.</p> \u000A \u000A <code>c   n &lt;goal&gt; &lt;process&gt; ...</code> \u000A \u000A <p>As above, but calls attention to the fact that this goal is a \u000A ``checkpoint'' in the sense that it may be of particular interest.  Some \u000A displays may overwrite ``c'' with ``&gt;'' to indicate the current checkpoint \u000A being shown in the proof transcript.</p> \u000A \u000A <code>|  &lt;goal&gt; ...\u000A|  |  &lt;k subgoals&gt;</code> \u000A \u000A <p>Indicates that the goal just above this line, which is pointed to by the \u000A rightmost vertical bar (``|''), has <v>k</v> subgoals, none of which have yet \u000A been processed.</p> \u000A \u000A <code>|  &lt;goal&gt; ...\u000A|  |  &lt;k more subgoals&gt;</code> \u000A \u000A <p>As above, except that some subgoals have already been processed.</p> \u000A \u000A <code>++++++++++++++++++++++++++++++</code> \u000A \u000A <p>Separates successive passes through the ``waterfall''.  Thus, this \u000A ``fencepost'' mark indicates the start of a new proof by induction or of a new \u000A forcing round.</p> \u000A \u000A <p>See <see topic=\"ACL2____PROOF-TREE-EXAMPLES\">proof-tree-examples</see> for detailed examples.  See <see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see> to learn how to mark goals as checkpoints that <see topic=\"ACL2____FORCE\">force</see> the creation of goals in forcing rounds.  Finally, see <see topic=\"ACL2____PROOF-TREE-DETAILS\">proof-tree-details</see> for some points not covered elsewhere.</p>"],

 "ACL2____BIND-FREE":[["Miscellaneous","Rewrite"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> LHS)\u000A              (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> RHS)\u000A              (<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see> (FIND-MATCH-IN-PLUS-NESTS LHS RHS) (X)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> LHS RHS)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) LHS) (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) RHS))))\u000A\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see>\u000A                (FIND-RATIONAL-MATCH-IN-TIMES-NESTS LHS RHS MFC STATE)\u000A                (X))\u000A              (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)\u000A              (<see topic=\"ACL2____CASE-SPLIT\">CASE-SPLIT</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 0))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> LHS RHS)\u000A                (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 X)\u000A                    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> X) LHS) (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> X) RHS))\u000A                   (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> X) RHS) (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> X) LHS)))))</code> \u000A \u000A <p>General Forms:</p> \u000A \u000A <code>(<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see> term var-list)\u000A(<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see> term t)\u000A(<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see> term)</code> \u000A \u000A <p>A rule which uses a <v>bind-free</v> hypothesis has similarities to both a \u000A rule which uses a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> hypothesis and to a <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rule. \u000A <v>Bind-free</v> is like <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt>, in that it logically always returns \u000A <v>t</v> but may affect the application of a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule when it is called at the top-level \u000A of a hypothesis.  It is like a <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rule, in that it allows the \u000A user to perform transformations of terms under progammatic control.</p> \u000A \u000A <p>Note that a <v>bind-free</v> hypothesis does not, in general, deal with the \u000A meaning or semantics or values of the terms, but rather with their syntactic \u000A forms.  Before attempting to write a rule which uses <v>bind-free</v>, the user \u000A should be familiar with <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> and the internal form that ACL2 uses \u000A for terms.  This internal form is similar to what the user sees, but there are \u000A subtle and important differences.  <tt><see topic=\"ACL2____TRANS\">Trans</see></tt> can be used to view this \u000A internal form.</p> \u000A \u000A <p>Just as for a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> hypothesis, there are two basic types of \u000A <v>bind-free</v> hypotheses.  The simpler type of <v>bind-free</v> hypothesis may \u000A be used as the nth hypothesis in a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule whose <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> is \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> hyp1 ... hypn ... hypk) (equiv lhs rhs))</v> provided <v>term</v> \u000A is a term, <v>term</v> contains at least one variable, and every variable \u000A occuring freely in <v>term</v> occurs freely in <v>lhs</v> or in some <v>hypi</v>, \u000A <v>i&lt;n</v>.  In addition, <v>term</v> must not use any stobjs.  Later below we \u000A will describe the second type, an <i>extended</i> <v>bind-free</v> hypothesis, \u000A which may use <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Whether simple or extended, a <v>bind-free</v> \u000A hypothesis may return an alist that binds free variables, as explained below, \u000A or it may return a list of such alists.  We focus on the first of these cases: \u000A return of a single binding alist.  We conclude our discussion with a section \u000A that covers the other case: return of a list of alists.</p> \u000A \u000A <p>We begin our description of <v>bind-free</v> by examining the first example \u000A above in some detail.</p> \u000A \u000A <p>We wish to write a rule which will cancel ``like'' addends from both sides \u000A of an equality.  Clearly, one could write a series of rules such as</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">DEFTHM</see> THE-HARD-WAY-2-1\u000A   (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> A X B)\u000A                 (<see topic=\"COMMON-LISP_____B2\">+</see> X C))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> A B)\u000A                 (<see topic=\"ACL2____FIX\">FIX</see> C))))</code> \u000A \u000A <p>with one rule for each combination of positions the matching addends might \u000A be found in (if one knew before-hand the maximum number of addends that would \u000A appear in a sum).  But there is a better way.  (In what follows, we assume the \u000A presence of an appropriate set of rules for simplifying sums.)</p> \u000A \u000A <p>Consider the following definitions and theorem:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> INTERSECTION-EQUAL (X Y)\u000A  (<see topic=\"COMMON-LISP____COND\">COND</see> ((<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X)\u000A         NIL)\u000A        ((<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) Y)\u000A         (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"ACL2____INTERSECTION-EQUAL\">INTERSECTION-EQUAL</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y)))\u000A        (T\u000A         (<see topic=\"ACL2____INTERSECTION-EQUAL\">INTERSECTION-EQUAL</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> PLUS-LEAVES (<see topic=\"ACL2____TERM\">TERM</see>)\u000A  (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____EQ\">EQ</see> (FN-SYMB TERM) 'BINARY-+)\u000A      (<see topic=\"COMMON-LISP____CONS\">CONS</see> (FARGN TERM 1)\u000A            (PLUS-LEAVES (FARGN TERM 2)))\u000A    (<see topic=\"COMMON-LISP____LIST\">LIST</see> TERM)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FIND-MATCH-IN-PLUS-NESTS (LHS RHS)\u000A  (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____EQ\">EQ</see> (FN-SYMB LHS) 'BINARY-+)\u000A           (<see topic=\"COMMON-LISP____EQ\">EQ</see> (FN-SYMB RHS) 'BINARY-+))\u000A      (<see topic=\"COMMON-LISP____LET\">LET</see> ((COMMON-ADDENDS (<see topic=\"ACL2____INTERSECTION-EQUAL\">INTERSECTION-EQUAL</see> (PLUS-LEAVES LHS)\u000A                                                (PLUS-LEAVES RHS))))\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> COMMON-ADDENDS\u000A            (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> 'X (<see topic=\"COMMON-LISP____CAR\">CAR</see> COMMON-ADDENDS)))\u000A          NIL))\u000A    NIL))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">DEFTHM</see> CANCEL-MATCHING-ADDENDS-EQUAL\u000A  (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> LHS)\u000A                (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> RHS)\u000A                (<see topic=\"ACL2____BIND-FREE\">BIND-FREE</see> (FIND-MATCH-IN-PLUS-NESTS LHS RHS) (X)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> LHS RHS)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) LHS) (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) RHS)))))</code> \u000A \u000A <p>How is this rule applied to the following term?</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c))\u000A       (<see topic=\"COMMON-LISP_____B2\">+</see> (bar b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n)))</code> \u000A \u000A <p>As mentioned above, the internal form of an ACL2 term is not always what \u000A one sees printed out by ACL2.  In this case, by using <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt> one \u000A can see that the term is stored internally as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                 (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c)))\u000A       (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n))).</code> \u000A \u000A <p>When ACL2 attempts to apply <v>cancel-matching-addends-equal</v> to the term \u000A under discussion, it first forms a substitution that instantiates the \u000A left-hand side of the conclusion so that it is identical to the target term. \u000A This substitution is kept track of by the substitution alist:</p> \u000A \u000A <code>((LHS . (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                   (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c))))\u000A (RHS . (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n)))).</code> \u000A \u000A <p>ACL2 then attempts to relieve the hypotheses in the order they were given. \u000A Ordinarily this means that we instantiate each hypothesis with our \u000A substitution and then attempt to rewrite the resulting instance to true. \u000A Thus, in order to relieve the first hypothesis, we rewrite:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                      (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c)))).</code> \u000A \u000A <p>Let us assume that the first two hypotheses rewrite to <v>t</v>.  How do we \u000A relieve the <v>bind-free</v> hypothesis?  Just as for a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> \u000A hypothesis, ACL2 evaluates <v>(find-match-in-plus-nests lhs rhs)</v> in an \u000A environment where <v>lhs</v> and <v>rhs</v> are instantiated as determined by the \u000A substitution.  In this case we evaluate</p> \u000A \u000A <code>(FIND-MATCH-IN-PLUS-NESTS '(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                                      (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c)))\u000A                          '(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n))).</code> \u000A \u000A <p>Observe that, just as in the case of a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> hypothesis, we \u000A substitute the quotation of the variables bindings into the term to be \u000A evaluated.  See <see topic=\"ACL2____SYNTAXP\">syntaxp</see> for the reasons for this.  The result of this \u000A evaluation, <v>((X . (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N)))</v>, is then used to extend the substitution \u000A alist:</p> \u000A \u000A <code>((X . (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N))\u000A (LHS . (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                   (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n) (foo a c))))\u000A (RHS . (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> a n)))),</code> \u000A \u000A <p>and this extended substitution determines \u000A <v>cancel-matching-addends-equal</v>'s result:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) LHS) (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> X) RHS))\u000A==&gt;\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N)) 3 (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N) (FOO A C))\u000A       (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N)) (BAR B) (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> A N))).</code> \u000A \u000A <p>Question: What is the internal form of this result?<br></br> \u000A \u000A Hint: Use <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt>.</p> \u000A \u000A <p>When this rule fires, it adds the negation of a common term to both sides \u000A of the equality by selecting a binding for the otherwise-free variable <v>x</v>, \u000A under programmatic control.  Note that other mechanisms such as the binding of \u000A <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> may also extend the substitution alist.</p> \u000A \u000A <p>Just as for a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> test, a <v>bind-free</v> form signals failure \u000A by returning <v>nil</v>.  However, while a <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> test signals success \u000A by returning true, a <v>bind-free</v> form signals success by returning an alist \u000A which is used to extend the current substitution alist.  Because of this use \u000A of the alist, there are several restrictions on it &mdash; in particular the \u000A alist must only bind variables, these variables must not be already bound by \u000A the substitution alist, and the variables must be bound to ACL2 terms.  If \u000A <v>term</v> returns an alist and the alist meets these restrictions, we append \u000A the alist to the substitution alist and use the result as the new current \u000A substitution alist.  This new current substitution alist is then used when we \u000A attempt to relieve the next hypothesis or, if there are no more, instantiate \u000A the right hand side of the rule.</p> \u000A \u000A <p>There is also a second, optional, <v>var-list</v> argument to a \u000A <v>bind-free</v> hypothesis.  If provided, it must be either <v>t</v> or a list of \u000A variables.  If it is not provided, it defaults to <v>t</v>.  If it is a list of \u000A variables, this second argument is used to place a further restriction on the \u000A possible values of the alist to be returned by <v>term</v>: any variables bound \u000A in the alist must be present in the list of variables.  We strongly recommend \u000A the use of this list of variables, as it allows some consistency checks to be \u000A performed at the time of the rule's admittance which are not possible \u000A otherwise.</p> \u000A \u000A <p>An extended <v>bind-free</v> hypothesis is similar to the simple type \u000A described above, but it uses two additional variables, <v>mfc</v> and \u000A <v>state</v>, which must not be bound by the left hand side or an earlier \u000A hypothesis of the rule.  They must be the last two variables mentioned by \u000A <v>term</v>: first <v>mfc</v>, then <v>state</v>.  These two variables give access \u000A to the functions <v>mfc-</v>xxx; see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  As \u000A described there, <v>mfc</v> is bound to the so-called metafunction-context and \u000A <v>state</v> to ACL2's <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____BIND-FREE-EXAMPLES\">bind-free-examples</see> for \u000A examples of the use of these extended <v>bind-free</v> hypotheses.</p> \u000A \u000A <p><b>SECTION</b>: Returning a list of alists.</p> \u000A \u000A <p>As promised above, we conclude with a discussion of the case that \u000A evaluation of the <v>bind-free</v> term produces a list of alists, <v>x</v>, \u000A rather than a single alist.  In this case each member <v>b</v> of <v>x</v> is \u000A considered in turn, starting with the first and proceeding through the list. \u000A Each such <v>b</v> is handled exactly as discussed above, as though it were the \u000A result of evaluating the <v>bind-free</v> term.  Thus, each <v>b</v> extends the \u000A current variable binding alist, and all remaining hypotheses are then \u000A relieved, as though <v>b</v> had been the value obtained by evaluating the \u000A <v>bind-free</v> term.  As soon as one such <v>b</v> leads to successful relieving \u000A of all remaining hypotheses, the process of relieving hypotheses concludes, so \u000A no further members of <v>x</v> are considered.</p> \u000A \u000A <p>We illustrate with a simple pedagogical example.  First introduce functions \u000A <v>p1</v> and <v>p2</v> such that a rewrite rule specifies that <v>p2</v> implies \u000A <v>p1</v>, but with a free variable.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> p1 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p2 (x y) t)\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-implies-p1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x y)\u000A           (p1 x)))</code> \u000A \u000A <p>If we add the following axiom, then <v>(p1 x)</v> follows logically for all \u000A <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-instance\u000A  (p2 v (<see topic=\"COMMON-LISP____CONS\">cons</see> v 4)))</code> \u000A \u000A <p>Unfortunately, evaluation of <v>(<see topic=\"ACL2____THM\">thm</see> (p1 a))</v> fails, because ACL2 fails to \u000A bind the free variable <v>y</v> in order to apply the rule <v>p2-instance</v>.</p> \u000A \u000A <p>Let's define a function that produces a list of alists, each binding the \u000A variable <v>y</v>.  Of course, we know that only the middle one below is \u000A necessary in this simple example.  In more complex examples, one might use \u000A heuristics to construct such a list of alists.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-alists (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'y (fcons-term* 'cons x ''3)))\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'y (fcons-term* 'cons x ''4)))\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'y (fcons-term* 'cons x ''5)))))</code> \u000A \u000A <p>The following rewrite rule uses <v>bind-free</v> to return a list of \u000A candidate alists binding <v>y</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> p2-implies-p1-better\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BIND-FREE\">bind-free</see> (my-alists x)\u000A                           (y)) ; the second argument, (y), is optional\u000A                (p2 x y))\u000A           (p1 x)))</code> \u000A \u000A <p>Now the proof succeeds for <v>(<see topic=\"ACL2____THM\">thm</see> (p1 a))</v>.  Why?  When ACL2 applies the \u000A <v>rewrite</v> rule <v>p2-implies-p1-better</v>, it evaluates <v>my-alists</v>, as \u000A we can see from the following <see topic=\"COMMON-LISP____TRACE\">trace</see>, to bind <v>y</v> in three different \u000A alists.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (p1 a))\u000A1&gt; (ACL2_*1*_ACL2::MY-ALISTS A)\u000A&lt;1 (ACL2_*1*_ACL2::MY-ALISTS (((Y CONS A '3))\u000A                              ((Y CONS A '4))\u000A                              ((Y CONS A '5))))\u000A\u000AQ.E.D.</code> \u000A \u000A <p>The first alist, binding <v>y</v> to <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> a '3)</v>, fails to allow the \u000A hypothesis <v>(p2 x y)</v> to be proved.  But the next binding of <v>y</v>, to \u000A <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> a '4)</v>, succeeds: then the current binding alist is <v>((x . a) (y\u000A . (<see topic=\"COMMON-LISP____CONS\">cons</see> a '4)))</v>, for which the hypothesis <v>(p2 x y)</v> rewrites to true \u000A using the rewrite rule <v>p2-instance</v>.</p>"],

 "ACL2____TIME_42":[["ACL2-built-ins","Programming","Other"],"ACL2 Sources","ACL2","<p>Semantically, <v>(<see topic=\"ACL2____TIME_42\">time$</see> x ...)</v> equals <v>x</v>.  However, its \u000A evaluation may write timing output to the trace output (which is usually the \u000A terminal), as explained further below.</p> \u000A \u000A <code>Examples:\u000A\u000A; Basic examples:\u000A\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo 3 4))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (mini-proveall))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) (<see topic=\"ACL2____TIME_42\">time$</see> (f x)))\u000A\u000A; Custom examples, which use a custom timing message rather than a built-in\u000A; message from Lisp:\u000A\u000A; Report only if real time is at least 1/2 second (two equivalent forms).\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo) :mintime 1/2)\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo) :real-mintime 1/2)\u000A\u000A; Report only if allocation is at least 1000 bytes (<see topic=\"COMMON-LISP____AND\">and</see> if the Lisp supports\u000A; :minalloc).\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo) :minalloc 1000)\u000A\u000A; Report only if real time is at least 1/2 second and (<see topic=\"COMMON-LISP____IF\">if</see> the Lisp supports\u000A; :minalloc) allocation is at least 931 bytes.\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo) :real-mintime 1/2 :minalloc 931)\u000A\u000A; Print &quot;Hello Moon, Goodbye World&quot; instead of any timing data.\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo)\u000A       :msg &quot;Hello ~s0, ~s1 World.&quot;\u000A       :args (<see topic=\"COMMON-LISP____LIST\">list</see> &quot;Moon&quot; &quot;Goodbye&quot;))\u000A\u000A; Print default custom timing message (same as omitting :mintime 0):\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (foo)\u000A       :mintime 0)\u000A\u000A; Print supplied custom timing message.\u000A(<see topic=\"COMMON-LISP____LET\">let</see> ((bar ...))\u000A  (<see topic=\"ACL2____TIME_42\">time$</see> (foo)\u000A         :msg &quot;The execution of ~xf took ~st seconds of real ~\u000A               time and ~sc seconds of run time (cpu time), and ~\u000A               allocated ~sa bytes.  In an unrelated note, bar ~\u000A               currently has the value ~x0.~%&quot;\u000A         :args (<see topic=\"COMMON-LISP____LIST\">list</see> bar)))\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____TIME_42\">time$</see> form)\u000A(<see topic=\"ACL2____TIME_42\">time$</see> form ; arguments below are optional\u000A       :real-mintime  &lt;rational number of seconds&gt;\u000A       :run-mintime   &lt;rational number of seconds&gt;\u000A       :minalloc      &lt;number of bytes&gt;\u000A       :msg           &lt;fmt string&gt;\u000A       :args          &lt;list of arguments for msg&gt;\u000A       )\u000A; Note: :real-mintime can be replaced by :mintime</code> \u000A \u000A <p>where <v>form</v> is processed as usual except that the host Common Lisp \u000A times its evaluation.</p> \u000A \u000A <p>The simplest form is <v>(<see topic=\"ACL2____TIME_42\">time$</see> x)</v>, which will call the <v>time</v> utility \u000A in the underlying Lisp, and will print a small default message.  If you want \u000A to see a message printed by the host Lisp, use <v>(<see topic=\"ACL2____TIME_42\">time$</see> x :mintime nil)</v> \u000A instead, which may provide detailed, implementation-specific data such as the \u000A amounts of time spent in user and system mode, the gc time, the number of page \u000A faults encountered, and so on.  Of you can create a custom message, configured \u000A using the <v>:msg</v> and <v>:args</v> parameters.  <v>Time$</v> can also be made to \u000A report timing information only conditionally: the <v>:real-mintime</v> (or \u000A equivalently, <v>:mintime</v>), <v>:run-mintime</v>, and <v>:minalloc</v> arguments \u000A can be used to avoid reporting timing information for computations that take a \u000A small amount of time (perhaps as might be expected in ordinary cases), but to \u000A draw the user's attention to computations that take longer or allocate more \u000A memory than expected.</p> \u000A \u000A <p>We next document the keyword arguments in some detail.</p> \u000A \u000A <blockquote><p>Keyword arguments <v>:real-mintime</v> (or <v>:mintime</v>) and \u000A <v>:run-mintime</v> can be used to specify a minimum time threshold for time \u000A reporting.  That is, no timing information will be printed if the execution of \u000A <v>form</v> takes less than the specified number of seconds of real (total) time \u000A or run (cpu) time, respectively.  Note that rational numbers like 1/2 may be \u000A used to specify a fractional amount of seconds.  It is an error to specify \u000A both <v>:real-mintime</v> and its synonym, <v>:mintime</v>.</p> \u000A \u000A <p>Keyword argument <v>:minalloc</v> is not supported on all Lisps.  When it is \u000A not supported, it is ignored.  But on supported Lisps, <v>:minalloc</v> can be \u000A used to specify a minimum memory allocation threshold.  If <v>form</v> results \u000A in fewer than this many bytes being allocated, then no timing information will \u000A be reported.</p> \u000A \u000A <p>Keyword argument <v>:msg</v>, when provided, should be a string accepted by \u000A the <v>fmt</v> family of functions (see <see topic=\"ACL2____FMT\">fmt</see>), and it may refer to the \u000A elements of <v>:args</v> by their positions, just as for <v>cw</v> (see <see topic=\"ACL2____CW\">cw</see>).</p></blockquote> \u000A \u000A <p>The following directives allow you to report timing information using the \u000A <v>:msg</v> string.  The examples at the top of this documentation topic \u000A illustrate the use of these directives.</p> \u000A \u000A <blockquote><p><v>~xf</v> &mdash; the form that was executed</p> \u000A \u000A <p><v>~sa</v> &mdash; the amount of memory allocated, in bytes (in supported \u000A Lisps)</p> \u000A \u000A <p><v>~st</v> &mdash; the real time taken, in seconds</p> \u000A \u000A <p><v>~sc</v> &mdash; the run time (cpu time) taken, in seconds</p> \u000A \u000A <p>The following apply only when the host Lisp is GCL.  The two system times \u000A will likely be <v>nil</v> unless the GCL version is 2.6.10 or later.  Note the \u000A upper-case characters for child times.</p> \u000A \u000A <p><v>~sC</v> &mdash; the child run time (cpu time) taken, in seconds</p> \u000A \u000A <p><v>~ss</v> &mdash; the system time taken, in seconds</p> \u000A \u000A <p><v>~sS</v> &mdash; the child system time taken, in seconds</p></blockquote> \u000A \u000A <p>We turn now to an example that illustrates how <v>time$</v> can be called in \u000A function bodies.  Consider the following definition of the Fibonacci function, \u000A followed by the definition of a function that times <v>k</v> calls of this \u000A function.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib (n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      1\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> n 1)\u000A        1\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> n 1))\u000A         (fib (<see topic=\"COMMON-LISP____-\">-</see> n 2))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> time-fib (k)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> k)\u000A      nil\u000A    (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A     (<see topic=\"ACL2____TIME_42\">time$</see> (fib k)\u000A            :mintime 1/2\u000A            :msg &quot;(fib ~x0) took ~st seconds, ~sa bytes allocated.~%&quot;\u000A            :args (<see topic=\"COMMON-LISP____LIST\">list</see> k))\u000A     (time-fib (<see topic=\"COMMON-LISP____1-\">1-</see> k)))))</code> \u000A \u000A <p>The following log shows a sample execution of the function defined just \u000A above.</p> \u000A \u000A <code>ACL2 !&gt;(time-fib 36)\u000A(fib 36) took 3.19 seconds, 1280 bytes allocated.\u000A(fib 35) took 1.97 seconds, 1280 bytes allocated.\u000A(fib 34) took 1.21 seconds, 1280 bytes allocated.\u000A(fib 33) took 0.75 seconds, 1280 bytes allocated.\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p>Notes:</p> \u000A \u000A <p>(1) Common Lisp specifies that the <v>time</v> utility prints to ``trace \u000A output'', and <v>time$</v> follows this convention.  Thus, if you have opened a \u000A <see topic=\"COMMON-LISP____TRACE\">trace</see> file (see <see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see>), then you can expect to find \u000A the <v>time$</v> output there.</p> \u000A \u000A <p>(2) Unless the <v>:msg</v> argument is supplied, an explicit call of \u000A <v>time$</v> in the top-level loop will show that the form being timed is a call \u000A of the ACL2 evaluator function <v>ev-rec</v>.  This is normal; the curious are \u000A invited, at their own risk, to see <see topic=\"ACL2____RETURN-LAST\">return-last</see> for an \u000A explanation.</p>"],

 "ACL2____LP":[["Ld"],"ACL2 Sources","ACL2","<p>To enter the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop from Common Lisp, call the \u000A Common Lisp program <v>lp</v> (which stands for ``loop,'' as in \u000A ``read-eval-print loop'' or ``<see topic=\"ACL2____COMMAND\">command</see> loop.'')  The ACL2 <see topic=\"ACL2____COMMAND\">command</see> \u000A loop is actually coded in ACL2 as the function <tt><see topic=\"ACL2____LD\">ld</see></tt> (which stands for \u000A ``load'').  The <see topic=\"ACL2____COMMAND\">command</see> loop is just what you get by loading from the \u000A standard object input channel, <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt>.  Calling <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A directly from Common Lisp is possible but fragile because hard lisp errors or \u000A aborts throw you out of <tt><see topic=\"ACL2____LD\">ld</see></tt> and back to the top-level of Common Lisp. \u000A <v>Lp</v> calls <tt><see topic=\"ACL2____LD\">ld</see></tt> in such a way as to prevent this and is thus the \u000A standard way to get into the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop.  Also see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see> for information on the loading of an initialization \u000A file.</p> \u000A \u000A <p>All of the visible functionality of <v>lp</v> is in fact provided by <tt><see topic=\"ACL2____LD\">ld</see></tt>, which is written in ACL2 itself.  Therefore, you should see <see topic=\"ACL2____LD\">ld</see> for \u000A detailed <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop.  We sketch it \u000A below, for novice users.</p> \u000A \u000A <p>Every expression typed to the ACL2 top-level must be an ACL2 \u000A expression.</p> \u000A \u000A <p>Any ACL2 expression may be submitted for evaluation.  Well-formedness is \u000A checked.  Some well-formed expressions cannot be evaluated because they \u000A involve (at some level) undefined constrained functions (see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>).  In addition, ACL2 does not allow ``global variables'' in \u000A expressions to be evaluated.  Thus, <v>(<see topic=\"COMMON-LISP____CAR\">car</see> '(a b c))</v> is legal and evaluates \u000A to <v>A</v>, but <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> is not, because there is no ``global context'' or \u000A binding environment that gives meaning to the variable symbol <v>x</v>.</p> \u000A \u000A <p>There is an exception to the global variable rule outlined above: \u000A single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see>) may be used as global variables in \u000A top-level expressions.  The most commonly used such object is the ACL2 \u000A ``current state,'' which is the value of the variable symbol <tt><see topic=\"ACL2____STATE\">state</see></tt>. \u000A This variable may occur in the top-level form to be evaluated, but must be \u000A passed only to ACL2 functions ``expecting'' <v>state</v> as described in the \u000A documentation for <tt><see topic=\"ACL2____STATE\">state</see></tt> and for <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s in general.  If the \u000A form returns a new <see topic=\"ACL2____STATE\">state</see> object as one of its values, then that is \u000A considered the new ``current'' <see topic=\"ACL2____STATE\">state</see> for the evaluation of the \u000A subsequent form.  See <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <p>ACL2 provides some support for the functionality usually provided by global \u000A variables in a read-eval-print loop, namely the saving of the result of a \u000A computation for subsequent re-use in another expression.  See <see topic=\"ACL2____ASSIGN\">assign</see> \u000A and see <see topic=\"ACL2_____04\">@</see>.</p> \u000A \u000A <p>If the form read is a single keyword, e.g., <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>, then special procedures are followed.  See <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____COMMAND\">command</see> loop keeps track of the <see topic=\"ACL2____COMMAND\">command</see>s you have typed \u000A and allows you to review them, display them, and roll the logical <see topic=\"ACL2____STATE\">state</see> \u000A back to that created by any <see topic=\"ACL2____COMMAND\">command</see>.  See <see topic=\"ACL2____HISTORY\">history</see>.</p> \u000A \u000A <p>ACL2 makes the convention that if a top-level form returns three values, \u000A the last of which is an ACL2 <see topic=\"ACL2____STATE\">state</see>, then the first is treated as a flag \u000A that means ``an error occurred,'' the second is the value to be printed if no \u000A error occurred, and the third is (of course) the new <see topic=\"ACL2____STATE\">state</see>.  When ``an \u000A error occurs'' no value is printed.  Thus, if you execute a top-level form \u000A that happens to return three such values, only the second will be printed (and \u000A that will only happen if the first is <v>nil</v>!).  See <see topic=\"ACL2____LD\">ld</see> for \u000A details.</p>"],

 "ACL2____EMBEDDED-EVENT-FORM":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hd (x) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> x) 0))\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma23 ...))\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn1 ...)\u000A       (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn2 ...))\u000A       ...)\u000A\u000AGeneral Form:\u000AAn embedded event form is a term, x, such that:</code> \u000A \u000A <ul> \u000A \u000A <li><v>x</v> is a call of an event function other than <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> (see \u000A <see topic=\"ACL2____EVENTS\">events</see> for a listing of the event functions);</li> \u000A \u000A <li><v>x</v> is of the form <v>(</v><tt><see topic=\"ACL2____LOCAL\">local</see></tt><v> x1)</v> where <v>x1</v> is an \u000A embedded event form;</li> \u000A \u000A <li><v>x</v> is of the form <v>(</v><tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt><v> x1)</v> where <v>x1</v> is \u000A an embedded event form;</li> \u000A \u000A <li><v>x</v> is of the form <v>(</v><tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt><v> &amp;)</v>, where <v>&amp;</v> is \u000A any term whose expansion is an embedded event (see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>);</li> \u000A \u000A <li><v>x</v> is of the form <v>(</v><tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt><v> ... x1)</v>, \u000A <v>(</v><tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt><v> ... x1 ...)</v>, or <v>(</v><tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt><v> ... x1)</v>, where <v>x1</v> is an embedded event \u000A form;</li> \u000A \u000A <li><v>x</v> is a call of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, \u000A or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>;</li> \u000A \u000A <li><v>x</v> macroexpands to one of the forms above; or</li> \u000A \u000A <li>[intended only for the implementation] <v>x</v> is <v>(RECORD-EXPANSION x1\u000A x2)</v>, where <v>x1</v> and <v>x2</v> are embedded event forms.</li> \u000A \u000A </ul> \u000A \u000A <p>However, we add the following restrictions for <tt><see topic=\"ACL2____LOCAL\">local</see></tt> contexts.</p> \u000A \u000A <ul> \u000A \u000A <li>An embedded event form may not set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> when in \u000A the context of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  Thus for example, the form \u000A \u000A <code>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :defun-mode :program))</code> \u000A \u000A is not an embedded event form, nor is the form <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____PROGRAM\">program</see>))</v>, \u000A since the latter sets the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> implicitly.  An example \u000A at the end of the discussion below illustrates why there is this \u000A restriction.</li> \u000A \u000A <li>A call of <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> is illegal in the context of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>. \u000A Without this restriction, one could locally assert a strong axiom like \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> t nil)</v> and then non-locally prove that formula, leaving you in an \u000A ACL2 logical <see topic=\"ACL2____WORLD\">world</see> in which it appears that the formula is actually \u000A provable without such an axiom.</li> \u000A \u000A <li>A call of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt> or <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt> is illegal in the context of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  For an \u000A explanation, see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see>.</li> \u000A \u000A </ul> \u000A \u000A <p>Only embedded event forms are allowed in a book after its initial <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> form.  See <see topic=\"ACL2____BOOKS\">books</see>.  However, you may find that <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> allows you to get the effect you want for a form that is not an \u000A embedded event form.  For example, you can put the following into a book, \u000A which assigns the value 17 to <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable <v>x</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____ASSIGN\">assign</see> x 17)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A            :check-expansion t)</code> \u000A \u000A <p>When an embedded event is executed while <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, those parts of it inside <tt><see topic=\"ACL2____LOCAL\">local</see></tt> forms are \u000A ignored.  Thus,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 () 1)\u000A       (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 () 2))\u000A       (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f3 () 3))</code> \u000A \u000A <p>will define <v>f1</v>, <v>f2</v>, and <v>f3</v> when <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is \u000A <v>nil</v> or <v>t</v>, but will define only <v>f1</v> and <v>f3</v> when <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p><i>Discussion:</i></p> \u000A \u000A <p><tt><see topic=\"ACL2____ENCAPSULATE\">Encapsulate</see></tt>, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> place \u000A restrictions on the kinds of forms that may be processed.  These restrictions \u000A ensure that the non-local <see topic=\"ACL2____EVENTS\">events</see> are indeed admissible provided that \u000A the sequence of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> and non-local <see topic=\"ACL2____EVENTS\">events</see> is admissible when \u000A proofs are done, i.e., when <v>ld-skip-proofs</v> is <v>nil</v>.  But <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> places no such restrictions, hence is potentially dangerous and should \u000A be avoided unless you understand the ramifications; so it is illegal unless \u000A there is an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p> \u000A \u000A <p><tt><see topic=\"ACL2____LOCAL\">Local</see></tt> permits the hiding of an event or group of <see topic=\"ACL2____EVENTS\">events</see> in \u000A the sense that <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are processed when we are trying to \u000A establish the admissibility of a sequence of <see topic=\"ACL2____EVENTS\">events</see> embedded in <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms or in <see topic=\"ACL2____BOOKS\">books</see>, but are ignored when we are \u000A constructing the <see topic=\"ACL2____WORLD\">world</see> produced by assuming that sequence.  Thus, for \u000A example, a particularly ugly and inefficient <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule might \u000A be made <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to an <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> that ``exports'' a desirable \u000A theorem whose proof requires the ugly lemma.</p> \u000A \u000A <p>To see why we can't allow just anything as an embedded event, consider \u000A allowing the form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state)\u000A    (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () 2)\u000A    (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () 1))</code> \u000A \u000A <p>followed by</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-is-1 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo) 1)).</code> \u000A \u000A <p>When we process the <see topic=\"ACL2____EVENTS\">events</see> with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>nil</v>, \u000A the second <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is executed and the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> succeeds.  But \u000A when we process the <see topic=\"ACL2____EVENTS\">events</see> with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the second <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is executed, so that <v>foo</v> no \u000A longer has the same definition it did when we proved <v>foo-is-1</v>.  Thus, an \u000A invalid formula is assumed when we process the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> while skipping \u000A proofs.  Thus, the first form above is not a legal embedded event form.</p> \u000A \u000A <p>If you encounter a situation where these restrictions seem to prevent you \u000A from doing what you want to do, then you may find <v>make-event</v> to be \u000A helpful.  See <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFPKG\">Defpkg</see></tt> is not allowed because it affects how things are read after \u000A it is executed.  But all the forms embedded in an event are read before any \u000A are executed.  That is,</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> nil\u000A             (<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;MY-PKG&quot; nil)\u000A             (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () 'my-pkg::bar))</code> \u000A \u000A <p>makes no sense since <v>my-pkg::bar</v> must have been read before the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> for <v>&quot;MY-PKG&quot;</v> was executed.</p> \u000A \u000A <p>Finally, let us elaborate on the restriction mentioned earlier related to \u000A the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  Consider the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____PROGRAM\">program</see>))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 0 x)\u000A       0\u000A     (<see topic=\"COMMON-LISP____1_B2\">1+</see> (foo (<see topic=\"COMMON-LISP____-\">-</see> x))))))</code> \u000A \u000A <p>See <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see> for a discussion of how <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> processes event forms.  Briefly, on the first pass through the \u000A <see topic=\"ACL2____EVENTS\">events</see> the definition of <v>foo</v> will be accepted in <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A mode <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, and hence accepted.  But on the second pass the \u000A form <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____PROGRAM\">program</see>))</v> is skipped because it is marked as <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, \u000A and hence <v>foo</v> is accepted in <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> mode <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>. \u000A Yet, no proof has been performed in order to admit <v>foo</v>, and in fact, it \u000A is not hard to prove a contradiction from this definition!</p>"],

 "ACL2____VERIFY-TERMINATION":[["Events"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> fact)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> fn dcl ... dcl)\u000A(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> (fn1 dcl ... dcl)\u000A                    (fn2 dcl ... dcl)\u000A                    ...)</code> \u000A \u000A <p>where <v>fn</v> and the <v>fni</v> are function symbols having <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode (see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>) and all of the <v>dcl</v>s are either \u000A <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms or <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings.  The first form above \u000A is an abbreviation for</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> (fn dcl ... dcl))</code> \u000A \u000A <p>so we limit our discussion to the second form.  Each of the <v>fni</v> must \u000A be in the same clique of mutually recursively defined functions, but not every \u000A function in the clique need be among the <v>fni</v>.</p> \u000A \u000A <p><v>Verify-termination</v> attempts to establish the admissibility of the \u000A <v>fni</v>. <v>Verify-termination</v> retrieves their definitions, creates \u000A modified definitions using the <v>dcl</v>s supplied above, and resubmits these \u000A definitions.  You could avoid using <v>verify-termination</v> by typing the new \u000A definitions yourself.  So in that sense, <v>verify-termination</v> adds no new \u000A functionality.  But if you have prototyped your system in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and tested it, you can use <v>verify-termination</v> to resubmit \u000A your definitions and change their <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, \u000A addings <see topic=\"ACL2____HINTS\">hints</see> without having to retype or recopy the code.</p> \u000A \u000A <p>The <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____COMMAND\">command</see> executed by <v>verify-termination</v> is \u000A obtained by retrieving the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> (or <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>) <see topic=\"ACL2____COMMAND\">command</see> that introduced the clique in question and then possibly modifying \u000A each definition as follows.  Consider a function, <v>fn</v>, in the clique.  If \u000A <v>fn</v> is not among the <v>fni</v> above, its definition is left unmodified \u000A other than to add <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic))</v>.  Otherwise, <v>fn</v> is \u000A some <v>fni</v> and we modify its definition by inserting into it the \u000A corresponding <v>dcl</v>s listed with <v>fni</v> in the arguments to \u000A <v>verify-termination</v>, as well as <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic))</v>.  In \u000A addition, we throw out from the old declarations in <v>fn</v> the <v>:mode</v> \u000A specification and anything that is specified in the new <v>dcl</v>s.</p> \u000A \u000A <p>For example, suppose that <v>fact</v> was introduced with:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer n)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n))))).</code> \u000A \u000A <p>Suppose later we do <v>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> fact)</v>.  Then the following \u000A definition is submitted.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer n))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n))))).</code> \u000A \u000A <p>Observe that this is the same definition as the original one, except the \u000A old specification of the <v>:mode</v> has been deleted so that the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> now defaults to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  Although the termination \u000A proof succeeds, ACL2 also tries to verify the <see topic=\"ACL2____GUARD\">guard</see>, because we have \u000A (implicitly) provided a <see topic=\"ACL2____GUARD\">guard</see>, namely <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)</v>, for this \u000A function.  (See <see topic=\"ACL2____GUARD\">guard</see> for a general discussion of guards, and see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see> for a discussion of how type declarations are used in guards.) \u000A Unfortunately, the <see topic=\"ACL2____GUARD\">guard</see> verification fails, because the subterm <v>(<see topic=\"ACL2____ZP\">zp</see>\u000A n)</v> requires that <v>n</v> be nonnegative, as can be seen by invoking <v>:args\u000A zp</v>.  (For a discussion of termination issues relating to recursion on the \u000A naturals, see <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.)  So we might be tempted to submit the \u000A following:</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>\u000A fact\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n))))).</code> \u000A \u000A <p>However, this is considered a changing of the guard (from <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see>\u000A n)</v>), which is illegal.  If we instead change the guard in the earlier \u000A <v>defun</v> after undoing that earlier definition with <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt><v>\u000A fact</v>, then <v>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> fact)</v> will succeed.</p> \u000A \u000A <p><b>Remark on system functions.</b> There may be times when you want to \u000A apply <v>verify-termination</v> (and also, perhaps, <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>) to \u000A functions that are predefined in ACL2.  It may be necessary in such cases to \u000A modify the system code first.  See Part II of the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/open-architecture/'>open \u000A architecture page for ACL2</a> for a discussion of the process for \u000A contributing updates to the system code and <see topic=\"ACL2____BOOKS\">books</see> with such \u000A <v>verify-termination</v> or <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, perhaps \u000A resulting in more system functions being built-in as <see topic=\"ACL2____GUARD\">guard</see>-verified. \u000A To see which built-in functions have already received such treatment, see \u000A community books directory <v>books/system/</v>; or, evaluate the constant \u000A <v>*system-verify-guards-alist*</v>, each of whose entries associates the name \u000A of a community book with a list of functions whose guard-verification is \u000A proved by including that book.  See the above URL for more details.</p> \u000A \u000A <p>Note that if <v>fn1</v> is already in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, then the \u000A <v>verify-termination</v> call has no effect.  It is generally considered to be \u000A redundant, in the sense that it returns without error; but if the <v>fn1</v> is \u000A a constrained function (i.e., introduced in the signature of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, or by <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>), then an error occurs.  This error is \u000A intended to highlight unintended uses of <v>verify-termination</v>; but if you \u000A do not want to see an error in this case, you can write and use your own macro \u000A in place of <v>verify-termination</v>.  The following explanation of the \u000A implementation of <v>verify-termination</v> may help with such a task.</p> \u000A \u000A <p>We conclude with a discussion of the use of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> to implement \u000A <v>verify-termination</v>.  This discussion can be skipped; we include it only \u000A for those who want to create variants of <v>verify-termination</v>, or who are \u000A interested in seeing an application of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.</p> \u000A \u000A <p>Consider the following proof of <v>nil</v>, which succeeded up through \u000A Version_3.4 of ACL2.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"ACL2____ZP\">zp</see> y))\u000A       (<see topic=\"COMMON-LISP____LIST\">list</see> x y)\u000A     (foo (<see topic=\"COMMON-LISP____1_B2\">1+</see> x) (<see topic=\"COMMON-LISP____1-\">1-</see> y))))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> y)))\u000A          (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"ACL2____ZP\">zp</see> y))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> x y)\u000A            (foo (<see topic=\"COMMON-LISP____1_B2\">1+</see> x) (<see topic=\"COMMON-LISP____1-\">1-</see> y)))))\u000A (<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> foo))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad-lemma\u000A  (<see topic=\"ACL2____ZP\">zp</see> x)\u000A  :hints ((&quot;Goal&quot; :induct (foo x 1)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>How did this work?  In the first pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, the \u000A second <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>foo</v> promoted <v>foo</v> from <v>:program</v> to \u000A <v>:logic</v> mode, with <v>y</v> as the unique measured variable.  The following \u000A call to <v>verify-termination</v> was then redundant.  However, on the second \u000A pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, the second (<tt><see topic=\"ACL2____LOCAL\">local</see></tt>) definition of \u000A <v>foo</v> was skipped, and the <v>verify-termination</v> event then used the \u000A first definition of <v>foo</v> to guess the measure, based (as with all guesses \u000A of measures) on a purely syntactic criterion.  ACL2 incorrectly chose \u000A <v>(<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)</v> as the measure, installing <v>x</v> as the unique measured \u000A variable, which in turn led to an unsound induction scheme subsequently used \u000A to prove <v>nil</v> (lemma <v>bad-lemma</v>, above)</p> \u000A \u000A <p>Now, <v>verify-termination</v> is a macro whose calls expand to <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> calls.  So in the first pass above, the <v>verify-termination</v> \u000A call generated a <v>defun</v> event identical to the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A of <v>foo</v>, which was correctly identified as redundant.  That expansion was \u000A recorded, and on the second pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, the expansion was \u000A recalled and used in place of the <v>verify-termination</v> call (that is how \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> works).  So instead of a measure being guessed for the \u000A <v>verify-termination</v> call on the second pass, the same measure was used as \u000A was used on the first pass, and a sound induction scheme was stored.  The \u000A attempt to prove <v>nil</v> (lemma <v>bad-lemma</v>) then failed.</p>"],

 "ACL2____PROGN_12":[["Events"],"ACL2 Sources","ACL2","<p><b>WARNING!</b> This event is intended for advanced users who, in \u000A essence, want to build extensions of ACL2.  See see <see topic=\"ACL2____DEFTTAG\">defttag</see>, in \u000A particular, the ``WARNING'' there, and see the warning about <see topic=\"ACL2____STOBJ\">stobj</see>s at \u000A the end of this documentation topic.</p> \u000A \u000A <p><v>Progn!</v> can be used like <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, even in <see topic=\"ACL2____BOOKS\">books</see>.  But \u000A unlike <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, <v>progn!</v> does not require its constituent forms to be \u000A <see topic=\"ACL2____EVENTS\">events</see> (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), except that the first form \u000A cannot be a symbol unless it is <v>:state-global-bindings</v> (advanced feature, \u000A described below).  However, see <see topic=\"ACL2____MAKE-EVENT\">make-event</see> for a ``Restriction to the \u000A Top Level'' that still applies under a call of <v>progn!</v>.</p> \u000A \u000A <p>Because <v>progn!</v> allows non-events, it differs from <v>progn</v> in \u000A another important respect: <v>progn!</v> is illegal unless there is an active \u000A ttag; see <see topic=\"ACL2____DEFTTAG\">defttag</see>.</p> \u000A \u000A <p>See community book <see topic=\"ACL2____HACKER\">hacker</see> for two macros, <see topic=\"ACL2____WITH-RAW-MODE\">with-raw-mode</see> and \u000A <see topic=\"ACL2____WITH-REDEF-ALLOWED\">with-redef-allowed</see>, each defined in terms of <v>progn!</v>, that allow \u000A arbitrary forms in contexts that would normally require legal embedded event \u000A forms.</p> \u000A \u000A <p>Given a form <v>(<see topic=\"ACL2____PROGN_12\">progn!</see> form1 form2 ... formk)</v>, ACL2 will evaluate each \u000A <v>formi</v> in turn (for i from 1 to k).  If a form returns more than one value \u000A (see <see topic=\"ACL2____MV\">mv</see>) where the first value returned is not <v>nil</v>, then no later \u000A form will be evaluated and the result returned by the <v>progn!</v> call will be \u000A <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> for some non-<v>nil</v> value <v>erp</v>, signifying an \u000A error (see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see>).  Otherwise the evaluation is considered \u000A to have succeeded, and will continue with later forms.  The value returned by \u000A a call of <v>progn!</v> with no such error is of the form <v>(<see topic=\"ACL2____MV\">mv</see> nil v state)</v>, \u000A where <v>v</v> depends on the last form as follows.  If the last form evaluates \u000A to a single value, then <v>v</v> is that value, except if the value is a <see topic=\"ACL2____STOBJ\">stobj</see>, say <v>ST</v>, then <v>v</v> is the symbol <v>REPLACED-ST</v>.  Otherwise \u000A the last form evaluates to some <v>(<see topic=\"ACL2____MV\">mv</see> nil x ...)</v>, and <v>v</v> is <v>x</v> \u000A unless after the final form's evaluation we are in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), in which case the <v>progn!</v> call returns <v>nil</v> (so that \u000A ACL2 can at least print the result &mdash; imagine Lisp returning a pathname \u000A object from a <v>load</v> call, for example).</p> \u000A \u000A <p>The normal undoing mechanism does not generally apply to forms within a \u000A <v>progn!</v> that are not legal ACL2 <see topic=\"ACL2____EVENTS\">events</see> (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).  In particular, note that a non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> call of \u000A <v>progn!</v> in an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event will generally be evaluated twice: \u000A once on each pass.  This fact is worth keeping in mind if you are using \u000A <v>progn!</v> to change the state of the system; ask yourself if it is \u000A acceptable to apply that state-changing operation more than once.</p> \u000A \u000A <p>Please note that <v>progn!</v> may differ from <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> in the following \u000A sense: definitions within a call of <v>progn!</v> might not be compiled.  For \u000A example, consider the following book.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :test)\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see>  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x) x))\u000A(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x) x))</code> \u000A \u000A <p>If the underlying Lisp is GCL 2.6.7, then after including this certified \u000A book (where the default certification took place, creating a compiled file), \u000A then <v>f1</v> is a compiled function but <v>f2</v> is not.  For other Lisps \u000A supported by ACL2, both <v>f1</v> and <v>f2</v> are compiled, though we are not \u000A sure that every function under every call of <v>progn!</v> would similarly be \u000A compiled.</p> \u000A \u000A <p>We now describe, for system hackers only, a sophisticated extension of \u000A <v>progn!</v> not mentioned above: support for keyword argument \u000A <v>:state-global-bindings</v>.  If the first argument of <v>progn!</v> is this \u000A keyword, then the second argument is treated as a list of bindings as expected \u000A by ACl2 system function <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt>.  Thus, in the ACL2 \u000A loop,</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> :state-global-bindings bindings form1 form2 ... formk)</code> \u000A \u000A <p>is treated as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> bindings (<see topic=\"ACL2____PROGN_12\">progn!</see> form1 form2 ... formk)))</code> \u000A \u000A <p>However, in raw Lisp the former is just:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> form1 form2 ... formk)</code> \u000A \u000A <p>Thus, one should use the <v>:state-global-bindings</v> argument with care, \u000A since the behavior in the ACL2 loop can differ from that in Common Lisp.  The \u000A intention is that one bind only <see topic=\"ACL2____STATE\">state</see> global variables that are \u000A relevant to evaluation of the forms within the ACL2 loop and are harmlessly \u000A ignored for evaluation of those forms in raw Lisp.  Here is a typical sort of \u000A example, as <see topic=\"ACL2____STATE\">state</see> global <v>ld-redefinition-action</v> is not relevant to \u000A the evaluation of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> in raw Lisp.</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> 'ld-redefinition-action nil)\u000A        (<see topic=\"ACL2____PROGN_12\">progn!</see> :state-global-bindings\u000A                ((<see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see> '(:doit . :overwrite)))\u000A                (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A        (<see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see> 'ld-redefinition-action nil))</code> \u000A \u000A <p>Finally, we point out a pitfall of <v>progn!</v> related to <see topic=\"ACL2____STOBJ\">stobj</see>s. \u000A The following book can cause a hard Lisp error, depending on the host Common \u000A Lisp, when certified with a non-<v>nil</v> value for <v>compile-flg</v> (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :my-ttag)\u000A(<see topic=\"ACL2____PROGN_12\">progn!</see> (update-fld 3 st))</code> \u000A \u000A <p>The problem is that the <see topic=\"ACL2____STOBJ\">stobj</see> variable <v>st</v> is not known to raw \u000A Lisp.  The compilation problem disappears if the last form above is replaced \u000A with the following two forms.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;hacking/hacker&quot; :dir :system)\u000A(with-raw-mode (update-fld 3 *the-live-st*))</code>"],

 "ACL2____MV":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Mv</v> is the mechanism provided by ACL2 for returning two or more \u000A values.  Logically, <v>(<see topic=\"ACL2____MV\">mv</see> x1 x2 ... xn)</v> is the same as <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x1 x2\u000A ... xn)</v>, a list of the indicated values.  However, ACL2 avoids the cost of \u000A building this list structure, with the cost that <v>mv</v> may only be used in a \u000A certain style in definitions: if a function ever returns using <v>mv</v> (either \u000A directly, or by calling another function that returns a multiple value), then \u000A this function must always return the same number of values.</p> \u000A \u000A <p>For more explanation of the multiple value mechanism, see <see topic=\"ACL2____MV-LET\">mv-let</see>. \u000A Also see <see topic=\"ACL2____MV-LIST\">mv-list</see> for a way to convert a multiple value into an ordinary \u000A list.</p> \u000A \u000A <p>ACL2 does not support the Common Lisp construct <v>values</v>, whose logical \u000A meaning seems difficult to characterize.  <v>Mv</v> is the ACL2 analogue of that \u000A construct.</p>"],

 "ACL2____LD-REDEFINITION-ACTION":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-redefinition-action</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The accessor is <v>(<see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see> state)</v> and the updater is \u000A <v>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> val state)</v>.</p> \u000A \u000A <p><b>WARNING!</b> If <v>ld-redefinition-action</v> is non-<v>nil</v> then ACL2 is \u000A liable to be made unsafe or unsound either by ill-considered definitions, or \u000A because redefining a macro or inlined function called in the body of a \u000A function, <v>g</v>, may not cause the new definition to be called by <v>g</v>. \u000A The keyword command <v>:</v><tt><see topic=\"ACL2____REDEF\">redef</see></tt> will set <v>ld-redefinition-action</v> \u000A to a convenient setting allowing unsound redefinition.  See below.</p> \u000A \u000A <p>When <v>ld-redefinition-action</v> is <v>nil</v>, redefinition is prohibited. \u000A In that case, an error message is printed upon any attempt to introduce a name \u000A that is already in use.  There is one exception to this rule.  It is permitted \u000A to redefine a function symbol in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode to be a function \u000A symbol in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode provided the formals and body remain the \u000A same.  This is the standard way a function ``comes into'' logical \u000A existence.</p> \u000A \u000A <p>Throughout the rest of this discussion we exclude from our meaning of \u000A ``redefinition'' the case in which a function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode is \u000A identically redefined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  At one time, ACL2 freely \u000A permitted the <see topic=\"ACL2____SIGNATURE\">signature</see>-preserving redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions but it no longer does.  See <see topic=\"ACL2____REDEFINING-PROGRAMS\">redefining-programs</see>.</p> \u000A \u000A <p>When <v>ld-redefinition-action</v> is non-<v>nil</v>, you are allowed to \u000A redefine a name that is already in use.  <b>The system may be rendered \u000A unsound</b> by such an act.  It is important to understand how dangerous \u000A redefinition is.  Suppose <v>fn</v> is a function symbol that is called from \u000A within some other function, say <v>g</v>.  Suppose <v>fn</v> is redefined so that \u000A its arity changes.  Then the definition of <v>g</v> is rendered syntactically \u000A ill-formed by the redefinition.  This can be devastating since the entire ACL2 \u000A system assumes that terms in its database are well-formed.  For example, if \u000A ACL2 executes <v>g</v> by running the corresponding function in raw Common Lisp \u000A the redefinition of <v>fn</v> may cause raw lisp to break in irreparable ways. \u000A As Lisp programmers we live with this all the time by following the simple \u000A rule: after changing the syntax of a function don't run any function that \u000A calls it via its old syntax.  This rule also works in the context of the \u000A evaluation of ACL2 functions, but it is harder to follow in the context of \u000A ACL2 deductions, since it is hard to know whether the database contains a path \u000A leading the theorem prover from facts about one function to facts about \u000A another.  Finally, of course, even if the database is still syntactically \u000A well-formed there is no assurance that all the rules stored in it are valid. \u000A For example, theorems proved about <v>g</v> survive the redefinition of <v>fn</v> \u000A but may have crucially depended on the properties of the old <v>fn</v>.  In \u000A summary, we repeat the warning: <b>all bets are off if you set</b> \u000A <v>ld-redefinition-action</v> to <b>non</b>-<v>nil</v>.</p> \u000A \u000A <p>ACL2 provides some enforcement of the concern above, by disabling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> if any <see topic=\"ACL2____WORLD\">world</see>-changing <see topic=\"ACL2____EVENTS\">events</see> exist in the \u000A certification <see topic=\"ACL2____WORLD\">world</see> that were executed with a non-<v>nil</v> value of \u000A <v>'ld-redefinition-action</v>.  (This value is checked at the end of each \u000A top-level command, but the value does not change during evaluation of embedded \u000A event forms; see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.)</p> \u000A \u000A <p>If at any point in a session you wish to see the list of all names that \u000A have been redefined, see <see topic=\"ACL2____REDEFINED-NAMES\">redefined-names</see>.</p> \u000A \u000A <p>That said, we'll give you enough rope to hang yourself.  When \u000A <v>ld-redefinition-action</v> is non-<v>nil</v>, it must be a pair, <v>(a .  b)</v>. \u000A The value of <v>a</v> determines how the system interacts with you when a \u000A redefinition is submitted.  The value of <v>b</v> allows you to specify how the \u000A property list of the redefined name is to be ``renewed'' before the \u000A redefinition.</p> \u000A \u000A <p>There are several dimensions to the space of possibilities controlled by \u000A part a: Do you want to be queried each time you redefine a name, so you can \u000A confirm your intention?  (We sometimes make typing mistakes or simply forget \u000A we have used that name already.)  Do you want to see a warning stating that \u000A the name has been redefined?  Do you want ACL2 system functions given special \u000A protection from possible redefinition?  Below are the choices for part a:</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>:query</v> &mdash; every attempt to redefine a name will produce a query. \u000A The query will allow you to abort the redefinition or proceed.  It will will \u000A also allow you to specify the part <v>b</v> for this redefinition.  <v>:Query</v> \u000A is the recommended setting for users who wish to dabble in redefinition.</p> \u000A \u000A <p><v>:warn</v> &mdash; any user-defined function may be redefined but a \u000A post-redefinition warning is printed.  The attempt to redefine a system name \u000A produces a query.  If you are prototyping and testing a big system in ACL2 \u000A this is probably the desired setting for part <v>a</v>.</p> \u000A \u000A <p><v>:doit</v> &mdash; any user-defined function may be redefined silently \u000A (without query or warning) but when an attempt is made to redefine a system \u000A function, a query is made.  This setting is recommended when you start making \u000A massive changes to your prototyped system (and tire of even the warning \u000A messages issued by <v>:warn</v>).</p> \u000A \u000A </blockquote> \u000A \u000A <p>In support of our own ACL2 systems <see topic=\"ACL2____PROGRAMMING\">programming</see> there are two other \u000A settings.  We suggest ordinary users not use them.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>:warn!</v> &mdash; every attempt to redefine a name produces a warning but \u000A no query.  Since ACL2 system functions can be redefined this way, this setting \u000A should be used by the only-slightly-less-than supremely confident ACL2 system \u000A hacker.</p> \u000A \u000A <p><v>:doit!</v> &mdash; this setting allows any name to be redefined silently \u000A (without query or warnings).  ACL2 system functions are fair game.  This \u000A setting is reserved for the supremely confident ACL2 system hacker. \u000A (Actually, this setting is used when we are loading massively modified \u000A versions of the ACL2 source files.)</p> \u000A \u000A </blockquote> \u000A \u000A <p>Part <v>b</v> of <v>ld-redefinition-action</v> tells the system how to \u000A ``renew'' the property list of the name being redefined.  There are two \u000A choices:</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>:erase</v> &mdash; erase all properties stored under the name, or</p> \u000A \u000A <p><v>:overwrite</v> &mdash; preserve existing properties and let the redefining \u000A overwrite them.</p> \u000A \u000A </blockquote> \u000A \u000A <p>It should be stressed that neither of these <v>b</v> settings is guaranteed \u000A to result in an entirely satisfactory state of affairs after the redefinition. \u000A Roughly speaking, <v>:erase</v> returns the property list of the name to the \u000A state it was in when the name was first introduced.  Lemmas, type information, \u000A etc., stored under that name are lost.  Is that what you wanted?  Sometimes it \u000A is, as when the old definition is ``completely wrong.'' But other times the \u000A old definition was ``almost right'' in the sense that some of the work done \u000A with it is still (intended to be) valid.  In that case, <v>:overwrite</v> might \u000A be the correct <v>b</v> setting.  For example if <v>fn</v> was a function and is \u000A being re-<tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d with the same <see topic=\"ACL2____SIGNATURE\">signature</see>, then the properties \u000A stored by the new <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> should overwrite those stored by the old \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> but the properties stored by <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>s will be \u000A preserved.</p> \u000A \u000A <p>In addition, neither setting will cause ACL2 to erase properties stored \u000A under other symbols!  Thus, if <v>FOO</v> names a rewrite rule which rewrites a \u000A term beginning with the function symbol <v>BAR</v> and you then redefine \u000A <v>FOO</v> to rewrite a term beginning with the function symbol <v>BAZ</v>, then \u000A the old version of <v>FOO</v> is still available (because the rule itself was \u000A added to the rewrite rules for <v>BAR</v>, whose property list was not cleared \u000A by redefining <v>FOO</v>).</p> \u000A \u000A <p>The <v>b</v> setting is only used as the default action when no query is \u000A made.  If you choose a setting for part a that produces a query then you will \u000A have the opportunity, for each redefinition, to specify whether the property \u000A list is to be erased or overwritten.</p> \u000A \u000A <p>The keyword command <v>:</v><tt><see topic=\"ACL2____REDEF\">redef</see></tt> sets <v>ld-redefinition-action</v> to \u000A the pair <v>(:query . :overwrite)</v>.  Since the resulting query will give you \u000A the chance to specify <v>:erase</v> instead of <v>:overwrite</v>, this setting is \u000A quite convenient.  But when you are engaged in heavy-duty prototyping, you may \u000A wish to use a setting such as <v>:warn</v> or even <v>:doit</v>.  For that you \u000A will have to invoke a form such as:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:doit . :overwrite) state) .</code> \u000A \u000A <p><tt><see topic=\"ACL2____ENCAPSULATE\">Encapsulate</see></tt> causes somewhat odd interaction with the user if \u000A redefinition occurs within the encapsulation because the <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>d \u000A event list is processed several times.  For example, if the redefinition \u000A action causes a query and a non-local definition is actually a redefinition, \u000A then the query will be posed twice, once during each pass.  C'est la vie.</p> \u000A \u000A <p>Finally, it should be stressed again that redefinition is dangerous because \u000A not all of the rules about a name are stored on the property list of the name. \u000A Thus, redefinition can render ill-formed terms stored elsewhere in the \u000A database or can preserve now-invalid rules.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>, in \u000A particular the section ``Note About Unfortunate Redundancies,'' for more \u000A discussion of potential pitfalls of redefinition.</p>"],

 "ACL2____INDUCTION":[["Rule-classes"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> recursion-by-sub2-induction-rule\u000A  t\u000A  :rule-classes ((:induction :pattern (<see topic=\"COMMON-LISP_____A2\">*</see> 1/2 i)\u000A                             :condition (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i 0))\u000A                             :scheme (recursion-by-sub2 i))))</code> \u000A \u000A <p>In ACL2, as in Nqthm, the functions in a conjecture ``suggest'' the \u000A inductions considered by the system.  Because every recursive function must be \u000A admitted with a justification in terms of a measure that decreases in a \u000A well-founded way on a given set of ``controlling'' arguments, every recursive \u000A function suggests a dual induction scheme that ``unwinds'' the function from a \u000A given application.</p> \u000A \u000A <p>For example, since <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> (actually <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>, but \u000A we'll ignore the distinction here) decomposes its first argument by successive \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s as long as it is a non-<v>nil</v> true list, the induction scheme \u000A suggested by <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> has a base case supposing <v>x</v> to be either \u000A not a true list or to be <v>nil</v> and then has an induction step in which the \u000A induction hypothesis is obtained by replacing <v>x</v> by <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>.  This \u000A substitution decreases the same measure used to justify the definition of \u000A <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>.  Observe that an induction scheme is suggested by a recursive \u000A function application only if the controlling actuals are distinct variables, a \u000A condition that is sufficient to ensure that the ``substitution'' used to \u000A create the induction hypothesis is indeed a substitution and that it drives \u000A down a certain measure.  In particular, <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> (foo x) y)</v> does not \u000A suggest an induction unwinding <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> because the induction scheme \u000A suggested by <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> requires that we substitute <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> for \u000A <v>x</v> and we cannot do that if <v>x</v> is not a variable symbol.</p> \u000A \u000A <p>Once ACL2 has collected together all the suggested induction schemes it \u000A massages them in various ways, combining some to simultaneously unwind certain \u000A cliques of functions and vetoing others because they ``flaw'' others.  We do \u000A not further discuss the induction heuristics here; the interested reader \u000A should see Chapter XIV of A Computational Logic (Boyer and Moore, Academic \u000A Press, 1979) which represents a fairly complete description of the induction \u000A heuristics of ACL2.</p> \u000A \u000A <p>However, unlike Nqthm, ACL2 provides a means by which the user can \u000A elaborate the rules under which function applications suggest induction \u000A schemes.  Such rules are called <v>:induction</v> rules.  The definitional \u000A principle automatically creates an <v>:induction</v> rule, named <v>(:induction\u000A fn)</v>, for each admitted recursive function, <v>fn</v>.  It is this rule that \u000A links applications of <v>fn</v> to the induction scheme it suggests.  Disabling \u000A <v>(:induction fn)</v> will prevent <v>fn</v> from suggesting the induction scheme \u000A derived from its recursive definition.  It is possible for the user to create \u000A additional <v>:induction</v> rules by using the <v>:induction</v> rule class in \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.</p> \u000A \u000A <p>Technically we are ``overloading'' <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> by using it in the \u000A creation of <v>:induction</v> rules because no theorem need be proved to set up \u000A the heuristic link represented by an <v>:induction</v> rule.  However, since \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> is generally used to create rules and rule-class objects are \u000A generally used to specify the exact form of each rule, we maintain that \u000A convention and introduce the notion of an <v>:induction</v> rule.  An \u000A <v>:induction</v> rule can be created from any lemma whatsoever.</p> \u000A \u000A <code>General Form of an :induction Lemma or Corollary:\u000AT\u000A\u000AGeneral Form of an :induction rule-class:\u000A(:induction :pattern pat-term\u000A            :condition cond-term\u000A            :scheme scheme-term)</code> \u000A \u000A <p>where <v>pat-term</v>, <v>cond-term</v>, and <v>scheme-term</v> are all terms, \u000A <v>pat-term</v> is the application of a function symbol, <v>fn</v>, \u000A <v>scheme-term</v> is the application of a function symbol, <v>rec-fn</v>, that \u000A suggests an induction, and, finally, every free variable of <v>cond-term</v> and \u000A <v>scheme-term</v> is a free variable of <v>pat-term</v>.  We actually check that \u000A <v>rec-fn</v> is either recursively defined &mdash; so that it suggests the \u000A induction that is intrinsic to its recursion &mdash; or else that another \u000A <v>:induction</v> rule has been proved linking a call of <v>rec-fn</v> as the \u000A <v>:pattern</v> to some scheme.</p> \u000A \u000A <p>The induction rule created is used as follows.  When an instance of the \u000A <v>:pattern</v> term occurs in a conjecture to be proved by induction and the \u000A corresponding instance of the <v>:condition</v> term is known to be non-<v>nil</v> \u000A (by type reasoning alone), the corresponding instance of the <v>:scheme</v> term \u000A is created and the rule ``suggests'' the induction, if any, suggested by that \u000A term.  (Analysis of that term may further involve induction rules, though the \u000A applied rule is removed from consideration during that further analysis, in \u000A order to avoid looping.)  If <v>rec-fn</v> is recursive, then the suggestion is \u000A the one that unwinds that recursion.</p> \u000A \u000A <p>Consider, for example, the example given above,</p> \u000A \u000A <code>(:induction :pattern (<see topic=\"COMMON-LISP_____A2\">*</see> 1/2 i)\u000A            :condition (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i 0))\u000A            :scheme (recursion-by-sub2 i)).</code> \u000A \u000A <p>In this example, we imagine that <v>recursion-by-sub2</v> is the \u000A function:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> recursion-by-sub2 (i)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 i))\u000A      (recursion-by-sub2 (<see topic=\"COMMON-LISP____-\">-</see> i 2))\u000A      t))</code> \u000A \u000A <p>Observe that this function recursively decomposes its integer argument by \u000A subtracting <v>2</v> from it repeatedly and stops when the argument is <v>1</v> or \u000A less.  The value of the function is irrelevant; it is its induction scheme \u000A that concerns us.  The induction scheme suggested by <v>(recursion-by-sub2\u000A i)</v> is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 i)))   ; base case\u000A              (:p i))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 1 i))    ; induction step\u000A                   (:p (<see topic=\"COMMON-LISP____-\">-</see> i 2)))\u000A              (:p i)))</code> \u000A \u000A <p>We can think of the base case as covering two situations.  The first is \u000A when <v>i</v> is not an integer.  The second is when the integer <v>i</v> is \u000A <v>0</v> or <v>1</v>.  In the base case we must prove <v>(:p i)</v> without further \u000A help.  The induction step deals with those integer <v>i</v> greater than <v>1</v>, \u000A and inductively assumes the conjecture for <v>i-2</v> while proving it for \u000A <v>i</v>.  Let us call this scheme ``induction on <v>i</v> by twos.''</p> \u000A \u000A <p>Suppose the above <v>:induction</v> rule has been added.  Then an occurrence \u000A of, say, <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 1/2 k)</v> in a conjecture to be proved by induction would \u000A suggest, via this rule, an induction on <v>k</v> by twos, provided <v>k</v> was \u000A known to be a nonnegative integer.  This is because the induction rule's \u000A <v>:pattern</v> is matched in the conjecture, its <v>:condition</v> is satisfied, \u000A and the <v>:scheme</v> suggested by the rule is that derived from \u000A <v>(recursion-by-sub2 k)</v>, which is induction on <v>k</v> by twos.  Similarly, \u000A the term <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 1/2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))</v> would suggest no induction via this rule, \u000A even though the rule ``fires'' because it creates the <v>:scheme</v> \u000A <v>(recursion-by-sub2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))</v> which suggests no inductions unwinding \u000A <v>recursion-by-sub2</v> (since the controlling argument of \u000A <v>recursion-by-sub2</v> in this <v>:scheme</v> is not a variable symbol).</p> \u000A \u000A <p>Continuing this example one step further illustrates the utility of \u000A <v>:induction</v> rules.  We could define the function <v>recursion-by-cddr</v> \u000A that suggests the induction scheme decomposing its <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> argument two \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s at a time.  We could then add the <v>:induction</v> rule linking \u000A <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 1/2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> x))</v> to <v>(recursion-by-cddr x)</v> and arrange for <v>(<see topic=\"COMMON-LISP_____A2\">*</see>\u000A 1/2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))</v> to suggest induction on <v>l</v> by <tt><see topic=\"COMMON-LISP____CDDR\">cddr</see></tt>.</p> \u000A \u000A <p>Observe that <v>:induction</v> rules require no proofs to be done.  Such a \u000A rule is merely a heuristic link between the <v>:pattern</v> term, which may \u000A occur in conjectures to be proved by induction, and the <v>:scheme</v> term, \u000A from which an induction scheme may be derived.  Hence, when an <v>:induction</v> \u000A rule-class is specified in a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event, the theorem proved is \u000A irrelevant.  The easiest theorem to prove is, of course, <v>t</v>.  Thus, we \u000A suggest that when an <v>:induction</v> rule is to be created, the following form \u000A be used:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> name T\u000A  :rule-classes ((:induction :pattern pat-term\u000A                             :condition cond-term\u000A                             :scheme scheme-term)))</code> \u000A \u000A <p>The name of the rule created is <v>(:induction name)</v>.  When that rune is \u000A disabled the heuristic link between <v>pat-term</v> and <v>scheme-term</v> is \u000A broken.</p>"],

 "ACL2____IN-THEORY":[["Events","Theories"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see>\u000A             (<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)\u000A             '(flatten (:executable-counterpart flatten))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> term :doc doc-string)</code> \u000A \u000A <p>where <v>term</v> is a term that when evaluated will produce a theory (see \u000A <see topic=\"ACL2____THEORIES\">theories</see>), and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string not beginning with ``<v>:Doc-Section</v> ...''.  Because no unique name \u000A is associated with an <v>in-theory</v> event, there is no way we can store the \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> in our <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A database.  Hence, we actually prohibit <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> from having the form \u000A of an ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p>Except for the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, <v>term</v> must contain no free \u000A variables.  <v>Term</v> is evaluated with the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to \u000A the current <see topic=\"ACL2____WORLD\">world</see> to obtain a theory and the corresponding runic theory \u000A (see <see topic=\"ACL2____THEORIES\">theories</see>) is then made the current theory.  Thus, immediately \u000A after the <v>in-theory</v>, a rule is <see topic=\"ACL2____ENABLE\">enable</see>d iff its rule name is a \u000A member of the runic interpretation (see <see topic=\"ACL2____THEORIES\">theories</see>) of some member of the \u000A value of <v>term</v>.  See <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see> for a list of the commonly \u000A used theory manipulation functions.</p> \u000A \u000A <p>Note that it is often useful to surround <v>in-theory</v> <see topic=\"ACL2____EVENTS\">events</see> with \u000A <v>local</v>, that is, to use <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____IN-THEORY\">in-theory</see> ...))</v>.  This use of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> in <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events and <see topic=\"ACL2____BOOKS\">books</see> will prevent the effect \u000A of this theory change from being exported outside the context of that \u000A <v>encapsulate</v> or book.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____HINTS\">hints</see> for a discussion of the <v>:in-theory</v> hint, \u000A including some explanation of the important point that an <v>:in-theory</v> hint \u000A will always be evaluated relative to the current ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, \u000A not relative to the theory of a previous goal.</p> \u000A \u000A <p><tt><see topic=\"ACL2____IN-THEORY\">In-theory</see></tt> returns an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>).  When \u000A the return is without error, the value is of the form <v>(<see topic=\"ACL2____MV\">mv</see> nil\u000A (:NUMBER-OF-ENABLED-RUNES k) state)</v> where <v>k</v> is the length of the new \u000A current theory.  This value of <v>k</v> is what is printed when an \u000A <v>in-theory</v> event evaluates without error at the top level.</p>"],

 "ACL2____DEFCHOOSE":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> choose-x-for-p1-and-p2 (x) (y z)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x y z)\u000A       (p2 x y z)))\u000A\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> choose-x-for-p1-and-p2 x (y z) ; equivalent to the above\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x y z)\u000A       (p2 x y z)))\u000A\u000A; The following is as above, but strengthens the axiom added to pick a sort\u000A; of canonical witness, as described below.\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> choose-x-for-p1-and-p2 x (y z)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x y z)\u000A       (p2 x y z))\u000A  :strengthen t)\u000A\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> choose-x-and-y-for-p1-and-p2 (x y) (z)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x y z)\u000A       (p2 x y z)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> fn\u000A           (bound-var1 ... bound-varn)\u000A           (free-var1 ... free-vark)\u000A           body\u000A           :doc doc-string\u000A           :strengthen b),</code> \u000A \u000A <p>where <v>fn</v> is the symbol you wish to define and is a new symbolic name \u000A (see <see topic=\"ACL2____NAME\">name</see>), <v>(bound-var1 ... bound-varn)</v> is a list of distinct \u000A `bound' variables (see below), <v>(free-var1 ... free-vark)</v> is the list of \u000A formal parameters of <v>fn</v> and is disjoint from the bound variables, and \u000A <v>body</v> is a term.  The use of <v>lambda-list</v> keywords (such as \u000A <v>&amp;optional</v>) is not allowed.  The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string argument, \u000A <v>:doc doc-string</v>, is optional; for a description of the form of \u000A <v>doc-string</v> see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.  The <v>:strengthen</v> keyword argument \u000A is optional; if supplied, it must be <v>t</v> or <v>nil</v>.</p> \u000A \u000A <p>The system treats <v>fn</v> very much as though it were declared in the <see topic=\"ACL2____SIGNATURE\">signature</see> of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, with a single axiom exported as \u000A described below.  If you supply a <v>:use</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>), <v>:use\u000A fn</v>, it will refer to that axiom.  No rule (of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or \u000A otherwise; see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) is created for <v>fn</v>.</p> \u000A \u000A <p><v>Defchoose</v> is only executed in <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>; \u000A see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.  Also see <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>.</p> \u000A \u000A <p>In the most common case, where there is only one bound variable, it is \u000A permissible to omit the enclosing parentheses on that variable.  The effect is \u000A the same whether or not those parentheses are omitted.  We describe this case \u000A first, where there is only one bound variable, and then address the other \u000A case.  Both cases are discussed assuming <v>:strengthen</v> is <v>nil</v>, which \u000A is the default.  We deal with the case <v>:strengthen t</v> at the end.</p> \u000A \u000A <p>The effect of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> fn bound-var (free-var1 ... free-vark)\u000A  body)</code> \u000A \u000A <p>is to introduce a new function symbol, <v>fn</v>, with formal parameters \u000A <v>(free-var1 ... free-vark)</v>.  Now consider the following axiom, which \u000A states that <v>fn</v> picks a value of <v>bound-var</v> so that the body will be \u000A true, if such a value exists:</p> \u000A \u000A <code>(1)   (<see topic=\"ACL2____IMPLIES\">implies</see> body\u000A               (<see topic=\"COMMON-LISP____LET\">let</see> ((bound-var (fn free-var1 ... free-vark)))\u000A                 body))</code> \u000A \u000A <p>This axiom is ``clearly conservative'' under the conditions expressed \u000A above: the function <v>fn</v> simply picks out a ``witnessing'' value of \u000A <v>bound-var</v> if there is one.  For a rigorous statement and proof of this \u000A conservativity claim, see <see topic=\"ACL2____CONSERVATIVITY-OF-DEFCHOOSE\">conservativity-of-defchoose</see>.</p> \u000A \u000A <p>Next consider the case that there is more than one bound variable, i.e., \u000A there is more than one bound-var in the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> fn\u000A           (bound-var1 ... bound-varn)\u000A           (free-var1 ... free-vark)\u000A           body)</code> \u000A \u000A <p>Then <v>fn</v> returns a multiple value with <v>n</v> components, and formula \u000A (1) above is expressed using <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> body\u000A         (<see topic=\"ACL2____MV-LET\">mv-let</see> (bound-var1 ... bound-varn)\u000A                 (fn free-var1 ... free-vark)\u000A                 body))</code> \u000A \u000A <p>We now discuss the case that <v>:strengthen t</v> is supplied.  For \u000A simplicity we return to our simplest case, with <v>defchoose</v> applied to \u000A function <v>fn</v>, a single free variable <v>y</v>, and a single bound variable \u000A <v>bound-var</v>.  The idea is that if we pick the ``smallest'' witnessing \u000A <v>bound-var</v> for two different free variables <v>y</v> and <v>y1</v>, then \u000A either those two witnesses are the same, or else one is less than the other, \u000A in which case the smaller one is a witness for its free variable but not for \u000A the other.  (See comments in source function <v>defchoose-constraint-extra</v> \u000A for more details.)  Below, <v>body1</v> is the result of replacing <v>y</v> by \u000A <v>y1</v> in <v>body</v>.</p> \u000A \u000A <code>(2)   (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn y) (fn y1))\u000A          (<see topic=\"COMMON-LISP____LET\">let</see> ((bound-var (fn y)))\u000A            (<see topic=\"COMMON-LISP____AND\">and</see> body\u000A                 (<see topic=\"COMMON-LISP____NOT\">not</see> body1)))\u000A          (<see topic=\"COMMON-LISP____LET\">let</see> ((bound-var (fn y1)))\u000A            (<see topic=\"COMMON-LISP____AND\">and</see> body1\u000A                 (<see topic=\"COMMON-LISP____NOT\">not</see> body))))</code> \u000A \u000A <p>An important application of this additional axiom is to be able to define a \u000A ``fixing'' function that picks a canonical representative of each equivalence \u000A class, for a given equivalence relation.  The following events illustrate this \u000A point.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((equiv (x y) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> equiv (x y) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)))\u000A (<see topic=\"ACL2____DEFEQUIV\">defequiv</see> equiv))\u000A\u000A(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> efix (x) (y)\u000A  (equiv x y)\u000A  :strengthen t)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> equiv-implies-equal-efix-1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv y y1)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (efix y) (efix y1)))\u000A  :hints ((&quot;Goal&quot; :use efix))\u000A  :rule-classes (:congruence))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> efix-fixes\u000A  (equiv (efix x) x)\u000A  :hints ((&quot;Goal&quot; :use ((:instance efix (y x))))))</code> \u000A \u000A <p>If there is more than one bound variable, then (2) is modified in complete \u000A analogy to (1) to use <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> in place of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>.</p> \u000A \u000A <p>Comment for logicians: As we point out in the documentation for <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>, <v>defchoose</v> is ``appropriate,'' by which we mean that it is \u000A conservative, even in the presence of <v>epsilon-0</v> induction.  For a proof, \u000A See <see topic=\"ACL2____CONSERVATIVITY-OF-DEFCHOOSE\">conservativity-of-defchoose</see>.</p>"],

 "ACL2____COMMAND":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>...the word ``command'' usually refers to a top-level form whose \u000A evaluation produces a new logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <code>Typical commands are:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> consp-foo (<see topic=\"COMMON-LISP____CONSP\">consp</see> (foo x)))\u000A(<see topic=\"ACL2____DEFREC\">defrec</see> pair (hd . tl) nil)</code> \u000A \u000A <p>The first two forms are examples of commands that are in fact primitive \u000A <see topic=\"ACL2____EVENTS\">events</see>.  See <see topic=\"ACL2____EVENTS\">events</see>.  <v>Defrec</v>, on the other hand, is a macro \u000A that expands into a <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> of several primitive <see topic=\"ACL2____EVENTS\">events</see>.  In \u000A general, a <see topic=\"ACL2____WORLD\">world</see> extending command generates one or more <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>Both <see topic=\"ACL2____EVENTS\">events</see> and commands leave landmarks on the <see topic=\"ACL2____WORLD\">world</see> that \u000A enable us to determine how the given <see topic=\"ACL2____WORLD\">world</see> was created from the \u000A previous one.  Most of your interactions will occur at the command level, \u000A i.e., you type commands, you print previous commands, and you undo back \u000A through commands.  Commands are denoted by command descriptors.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p>"],

 "ACL2____FORWARD-CHAINING-REPORTS":[["Forward-chaining","Debugging"],"ACL2 Sources","ACL2","<p>Debugging forward-chaining rules can be hard because their effects \u000A are not directly visible on the goal.  In this documentation we tell you how \u000A to get reports on the forward chaining activity occurring in your proof \u000A attempts.  This documentation is written in several parts.  The first part is \u000A an introduction for the first-time user of forward chaining reports.  The next \u000A two parts describe how to read reports.  The last part describes how to \u000A monitor forward chaining activity only for selected runes, etc.  We recommend \u000A the new user of these reports read everything!</p> \u000A \u000A <p><i>A Quick Introduction to Forward Chaining Reports</i></p> \u000A \u000A <p>Caution: The reporting mechanism maintains some state, and if you have \u000A already used forward chaining reporting in a session, the directions below may \u000A not work as advertised!  To return to the default forward chaining reporting \u000A state, execute this form at the top level:</p> \u000A \u000A <code>(<see topic=\"ACL2____RESET-FC-REPORTING\">reset-fc-reporting</see>)</code> \u000A \u000A <p>You can get a report about all forward chaining activity in subsequent \u000A proofs by doing:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> t)</code> \u000A \u000A <p>Options will be discussed later that allow you to monitor the activity \u000A caused by particular <v>:forward-chaining</v> rules or terms.</p> \u000A \u000A <p>Then do a proof that is expected to cause some forward chaining.  In the \u000A proof output you will see lines like this:</p> \u000A \u000A <code>(Forward Chaining on behalf of PREPROCESS-CLAUSE:  (<see topic=\"ACL2____FC-REPORT\">FC-Report</see> 1))</code> \u000A \u000A <p>This is the only difference you should see in the proof output.</p> \u000A \u000A <p>After the proof attempt has terminated, you can execute:</p> \u000A \u000A <code>(<see topic=\"ACL2____FC-REPORT\">fc-report</see> k)</code> \u000A \u000A <p>for any <v>k</v> printed during the immediately preceding proof attempt. \u000A That will print a much longer report describing the activity that occurred \u000A during the <v>k</v>th use of forward chaining in that proof attempt.</p> \u000A \u000A <p>If you want to see these reports in real time (embedded in the proof \u000A output), do this before invoking the prover:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-FC-REPORT-ON-THE-FLY\">set-fc-report-on-the-fly</see> t)</code> \u000A \u000A <p>Collecting the data used to generate these reports slows down the prover. \u000A If you no longer wish to see such reports, do</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> nil)</code> \u000A \u000A <p><i>How To Read FC Reports</i></p> \u000A \u000A <p>The report printed by <v>(<see topic=\"ACL2____FC-REPORT\">fc-report</see> k)</v> is of the form:</p> \u000A \u000A <code>Forward Chaining Report <i>k</i>: \u000ACaller: <i>token</i> \u000AClause: (<i>lit1</i> <i>lit2</i> ... <i>litn</i>) \u000ANumber of Rounds: <i>m</i> \u000AContradictionp: <i>bool</i> \u000AActivations: (<i>act1</i> <i>act2</i> ...) \u000A</code> \u000A \u000A <p>This report means that the <i>k</i>th use of forward chaining in the most \u000A recent proof attempt was done on behalf of <i>token</i> (see below).  The \u000A initial context (set of assumptions) consisted of the negations of the \u000A literals listed in the clause shown and the initial candidate trigger terms \u000A are all those appearing in that clause.  This invocation of forward chaining \u000A proceeded to do <i>m</i> rounds of successive extensions of the initial \u000A context and ultimately either reached a contradiction (<i>bool</i> = <v>T</v>) \u000A or returned an extended context (<i>bool</i> = <v>NIL</v>).  Note that reaching \u000A a contradiction from the negations of all the literals in a clause is ``good'' \u000A because it means the clause is true.  The report concludes with the final \u000A status of all the forward chaining rules fired during the process.  We explain \u000A how to read one of these activation reports in the next section.</p> \u000A \u000A <p>Forward chaining is done on behalf of many proof techniques in the system. \u000A Each is associated with a <i>token</i>.  The main proof technique that uses \u000A forward chaining is <v>SIMPLIFY-CLAUSE</v>.  This is the call of forward \u000A chaining that sets up the context used by the rewriter to relieve hypotheses \u000A during backchaining.  Another common caller of forward chaining is \u000A <v>PREPROCESS-CLAUSE</v>, the first process in the ACL2 waterfall (see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>).  Forward chaining often proves ``near \u000A propositional'' goals (those depending just on boolean implications between \u000A basic predicates).  Other tokens you may see include <v>INDUCT</v>, which uses \u000A forward chaining to set up a context for applying <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> \u000A rules, and the definitional principle (and related utilities such as <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> and <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>) which uses forward chaining \u000A during the construction of both measure conjectures and guard conjectures. \u000A When used this way, the <i>token</i> is <v>defun-or-guard-verification</v>.</p> \u000A \u000A <p><i>How to Read Activation Reports</i></p> \u000A \u000A <p>The forward chaining report concludes with a list of activation \u000A reports.</p> \u000A \u000A <code>Activations: (<i>act1</i> <i>act2</i> ...) \u000A</code> \u000A \u000A <p>Each <i>acti</i> is of the form:</p> \u000A \u000A <code>(<i>rune</i> \u000A (:TRIGGER <i>inst-trig</i>) \u000A ((:UNIFY-SUBST <i>subst</i>) \u000A  (:DISPOSITION <i>outcome-part1</i> <i>outcome-part2</i> <i>inst-term</i>)) \u000A ...) \u000A</code> \u000A \u000A <p>where the <v>...</v> indicates that the rest of the report consists of more \u000A of those tuples listing a <v>:UNIFY-SUBST</v> and <v>:DISPOSITION</v>.  We call \u000A each tuple a <i>disposition</i> of the activation and each disposition \u000A describes a substitution <i>subst</i> identifying the final instantiation of \u000A the rule and how the activation fared.  Suppose there are <i>n</i> \u000A dispositions.  (If the rule in question contains no free variables, <i>n</i> \u000A will be 1.)</p> \u000A \u000A <p>This activation report means that during the forward chaining process in \u000A question, the <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> <see topic=\"ACL2____RUNE\">rune</see> <i>rune</i> was fired \u000A due to the presence in the evolving context of the trigger term \u000A <i>inst-trig</i>.  (Note that <i>inst-trig</i> is an instantiation of the \u000A trigger term of the named rule.  That is, the variable symbols you see in \u000A <i>inst-trig</i> are those of the clause printed in the forward chaining \u000A report.)  The activation of <i>rune</i> by <i>inst-trig</i> proceeded to split \u000A <i>n</i> ways as different choices were made for the <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> \u000A occuring among the hypotheses.  Each of those <i>n</i> choices gave rise to a \u000A different substitution <i>subst</i>, and each succeeded or failed as described \u000A by the corresponding <v>:DISPOSITION</v>.</p> \u000A \u000A <p>The <v>:DISPOSITION</v> of an activation is described in three parts, \u000A <i>outcome-part1</i>, <i>outcome-part2</i>, and <i>inst-term</i>. \u000A <i>Outcome-part1</i> is either <v>SUCCESS</v> or <v>BLOCKED</v>, meaning that the \u000A instance given by <i>subst</i> either succeeded in the sense that all of its \u000A instantiated hypotheses were found in the context, or failed because some \u000A instantiated hypothesis was not found.</p> \u000A \u000A <p>If <v>outcome-part1</v> is <v>SUCCESS</v> then <i>inst-term</i> is the \u000A instantiated conclusion produced by the rule.  <i>Outcome-part2</i> is \u000A <v>APPROVED</v> if the instantiated conclusion was acceptable to our heuristics \u000A designed to prevent looping and not already known in the evolving context. \u000A <i>Outcome-part2</i> is <v>REJECTED</v> if the instantiated conclusion was not \u000A approved by our heuristics.  <i>Outcome-part2</i> is <v>REDUNDANT</v> if the \u000A instantiated conclusion was approved by the heuristics but already known true \u000A in the current evolving context.  If <v>APPROVED</v>, the truth of the \u000A instantiated conclusion is added to the evolving context.  Otherwise, it is \u000A not.</p> \u000A \u000A <p>If <v>outcome-part1</v> is <v>BLOCKED</v> then <v>outcome-part2</v> is one of \u000A three possible things: <v>FALSE</v>, in which case <i>inst-term</i> is an \u000A instantiated hypothesis of the rule that is assumed false in the current \u000A context, <v>UNRELIEVED-HYP</v>, in which case <i>inst-term</i> is an \u000A instantiated hypothesis whose truthvalue is not determined by the context, or \u000A <v>UNRELIEVED-HYP-FREE</v>, in which case <i>inst-term</i> is an oddly \u000A instantiated hypothesis whose truthvalue is not determined by the context and \u000A which also contains free variables.  In the last case, the ``odd'' \u000A instantiation was by the substitution <i>subst</i> but extended so that free \u000A variables in the hypothesis are renamed to start with the prefix \u000A <v>UNBOUND-FREE-</v> to draw your attention to them.</p> \u000A \u000A <p>Note: All of the terms printed in the report are instantiated with the \u000A relevant unifying substitution (possibly extended to bind free variables).</p> \u000A \u000A <p><i>Specifying the Tracking Criteria</i></p> \u000A \u000A <p>During a proof attempt, the forward chaining module stores information \u000A about the activations satisfying certain criteria.  The <i>criteria</i> is a \u000A list of triples.  Each triple consists of a forward chaining rune, an \u000A instantiated trigger term, and an instantiated conclusion to watch for. \u000A However, any or all of the components of such a triple may be <v>t</v> and that \u000A is given special significance.</p> \u000A \u000A <p>An activation <v>satisfies</v> a criteria if it satisfies at least one of the \u000A triples.  An activation satisfies a triple if it satisfies all three of the \u000A components.  Every activation satisfies the component <v>t</v>.  An activation \u000A satisfies a rune if the activation describes a firing of the named rule.  An \u000A activation satisfies an instantiated trigger term if the activation was \u000A created by that trigger being present in the context.  An activation satisfies \u000A an instantiated conclusion if the activation <i>could</i> produce the \u000A instantiated conclusion (with the right choice of any free variables).</p> \u000A \u000A <p>Thus, the criteria is interpreted as a disjunction of conjunctions, making \u000A it possible to track a specific set of runes, triggers, and conclusions.</p> \u000A \u000A <p>For example, here is a triple that might appear in the criteria:</p> \u000A \u000A <code>((:FORWARD-CHAINING ALISTP-FORWARD-TO-TRUE-LISTP)\u000A t\u000A t).</code> \u000A \u000A <p>This triple would cause every activation of the given rule to be tracked. \u000A However, the triple</p> \u000A \u000A <code>((:FORWARD-CHAINING ALISTP-FORWARD-TO-TRUE-LISTP)\u000A (<see topic=\"ACL2____ALISTP\">ALISTP</see> (MAKE-BINDINGS VARS (TOP-N (OP1 INST) (STACK S))))\u000A t)</code> \u000A \u000A <p>would only track activations of that rule fired by the specific term shown \u000A as the second element of the triple.  Futhermore</p> \u000A \u000A <code>(t\u000A (<see topic=\"ACL2____ALISTP\">ALISTP</see> (MAKE-BINDINGS VARS (TOP-N (OP1 INST) (STACK S))))\u000A t)</code> \u000A \u000A <p>would track any forward chaining rule triggered by that term, and</p> \u000A \u000A <code>(t\u000A t\u000A (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (MAKE-BINDINGS VARS (TOP-N (OP1 INST) (STACK S)))))</code> \u000A \u000A <p>would track any rule fired by any trigger that might lead to the specific \u000A term given as the third component above.</p> \u000A \u000A <p>Note: The condition on how an activation satisfies an instantiated \u000A conclusion is a little subtle.  Consider the activation of the forward \u000A chaining rule</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> x)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y)))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (make-bindings x y)))</code> \u000A \u000A <p>triggered by <v>(<see topic=\"ACL2____SYMBOL-LISTP\">SYMBOL-LISTP</see> VARS)</v> arising in the current context.  This \u000A activation <i>could</i> produce the specific conclusion shown in the last \u000A triple above, if it just happened that <v>(TOP-N (OP1 INST) (STACK S))</v> were \u000A chosen as the binding of the free variable <v>y</v>.  Thus, the activation of \u000A this rule triggered by <v>(<see topic=\"ACL2____SYMBOL-LISTP\">SYMBOL-LISTP</see> VARS)</v> satisfies the last triple \u000A above.</p> \u000A \u000A <p>Observe that the triple</p> \u000A \u000A <code>(t t t)</code> \u000A \u000A <p>is satisfied by every activation of any rule by any trigger term producing \u000A any conclusion.</p> \u000A \u000A <p>The function <v>set-fc-criteria</v> sets the criteria describing which \u000A activations are to be tracked.  For example, if you execute:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> ((:FORWARD-CHAINING LEMMA1)\u000A                  t t)\u000A                 ((:FORWARD-CHAINING LEMMA2)\u000A                  (<see topic=\"ACL2____ALISTP\">ALISTP</see> (BASIC-MAPPER A B))\u000A                  t)\u000A                 (t t (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (DLT D)))),</code> \u000A \u000A <p>the system would track all activations of the forward-chaining rule \u000A <v>LEMMA1</v>, plus those activations of forward-chaining rule <v>LEMMA2</v> \u000A triggered by the term given in the second triple, plus any activation of any \u000A rule that might derive <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (DLT D))</v>.</p> \u000A \u000A <p>Because criteria generally mention variable symbols used in a specific \u000A conjecture, it is probably best to reconsider your criteria every time you \u000A want to track forward chaining.</p> \u000A \u000A <p>If the criteria is <v>nil</v>, then nothing is tracked.  Setting the criteria \u000A to <v>nil</v> is the way you turn off tracking and reporting of forward chaining \u000A activity.  You may do this either by <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>)</v> or by \u000A <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> nil)</v>.  (Technically the second form is an odd use of \u000A <v>set-fc-criteria</v>, which expects any supplied arguments to be triples; if \u000A the ``triple'' <v>nil</v> is the only one supplied, we take it to mean that the \u000A entire criteria should be <v>nil</v>.)</p> \u000A \u000A <p>To track every forward chaining activation you may set the criteria with \u000A either <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> (t t t))</v> or use the abbreviation \u000A <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> t)</v>.</p> \u000A \u000A <p>If, when you read a forward chaining report, you see no mention of an \u000A activation you have in mind, e.g., of a certain rune or deriving a certain \u000A conclusion, and you have set the criteria correctly, then the activation never \u000A happened.  (This is akin to using <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt> and <v>:</v><tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> \u000A to monitor the application of a rewrite rule and then seeing no interactive \u000A break.)</p> \u000A \u000A <p>For some relevant functions to help you manage criteria and when the full \u000A reports are printed see <tt><see topic=\"ACL2____FC-REPORT\">fc-report</see></tt>, <tt><see topic=\"ACL2____SHOW-FC-CRITERIA\">show-fc-criteria</see></tt>, <tt><see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see></tt>, <tt><see topic=\"ACL2____RESET-FC-REPORTING\">reset-fc-reporting</see></tt>, and <tt><see topic=\"ACL2____SET-FC-REPORT-ON-THE-FLY\">set-fc-report-on-the-fly</see></tt>.</p>"],

 "ACL2____EQUALITY-VARIANTS":[["Programming"],"ACL2 Sources","ACL2","<p>The ACL2 environment includes not only a logic but also a \u000A programming language, which is based on Common Lisp.  Execution efficiency may \u000A be increased by using fast equality tests: <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> for symbols and <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> for numbers, symbols, and characters (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>).  Several \u000A list-processing functions built into ACL2 thus have three variants, depending \u000A on whether the equality function used is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, or <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>; a list is provided below.  ACL2 has taken measures to ensure that one \u000A can reason about a single logical function even when one uses these different \u000A variants.</p> \u000A \u000A <p>Consider for example the case of list membership.  Common Lisp provides a \u000A utility for this purposes, <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, which can take a <v>:TEST</v> keyword \u000A argument, default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>.  So for example, one might write</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">member</see> a x :TEST 'eq)</code> \u000A \u000A <p>if either <v>a</v> is a symbol or <v>x</v> is a list of symbols, so that the \u000A fastest equality test (<tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>) may be used when comparing <v>a</v> to \u000A successive elements of the list, <v>x</v>.  One might elsewhere write <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see>\u000A b (foo y))</v>, which is equivalent to <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> b (foo y) :TEST 'eql)</v>, for \u000A example if <v>b</v> is a number.  If one wants to reason about both <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> a\u000A x :TEST 'eq)</v> and <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> b y)</v>, it might be helpful for both calls of \u000A <v>member</v> to be the same logically, even though Common Lisp will execute \u000A them differently (using <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> or <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, respectively).  ACL2 \u000A arranges that in fact, both references to <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> generate calls of \u000A <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> in the theorem prover.</p> \u000A \u000A <p>In fact, since <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> can take the optional <v>:TEST</v> keyword \u000A argument, then in ACl2 it must be defined as a macro, not a function (see \u000A <see topic=\"COMMON-LISP____DEFUN\">defun</see>).  ACL2 arranges that a call of <v>member</v> generates a \u000A corresponding call of the function <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, regardless of the \u000A value of <v>TEST</v>, in a manner that produces <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> in prover \u000A output.  More generally, you can expect ACL2 to treat your use of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> as though you had written <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, for example in the way \u000A it stores <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules and other kinds of rules as well (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>).  We say little here about how this is all arranged by ACL2, \u000A other than to mention that <tt><see topic=\"ACL2____MBE\">mbe</see></tt> is utilized (so, you might see mention \u000A in proof logs) of the function <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> that implements <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.  Such details, which involve a notion of ``macro alias'' and probably \u000A can be ignored by most users, may be found elsewhere; see <see topic=\"ACL2____EQUALITY-VARIANTS-DETAILS\">equality-variants-details</see>.</p> \u000A \u000A <p>As a convenience to the user, the macro <v>member-eq</v> is provided that \u000A expands to a corresponding call of <v>member</v> with <v>:TEST 'eq</v>, as \u000A follows.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> (foo x) (bar y))\u000A (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (FOO X) (BAR Y) :TEST 'EQ)\u000AACL2 !&gt;</code> \u000A \u000A <p>For efficiency we recommend using the <v>-equal</v> equality variant, for \u000A example <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> or <v>(</v><tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt><v> ... :TEST 'equal)</v>, \u000A in certain contexts: <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>, <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, and \u000A <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt> forms.  However, the implementation of equality variants \u000A has been designed so that when defining a function, one may choose freely in a \u000A definition an equality variant of primitive <v>F</v>, to get efficient execution \u000A but where subsequent reasoning is about <v>F-equal</v>.  For details about the \u000A above recommendation and for a discussion of the implementation, see <see topic=\"ACL2____EQUALITY-VARIANTS-DETAILS\">equality-variants-details</see>.</p> \u000A \u000A <p>The following alphabetical list includes all primitives that have equality \u000A variants.  Each macro <v>F</v> listed below takes an optional <v>:TEST</v> keyword \u000A argument of <v>'eq</v>, <v>'eql</v>, or <v>'equal</v>, where <v>'eql</v> is the \u000A default.  For each such <v>F</v>, a function <v>F-equal</v> is defined such that \u000A for logical purposes (in particular theorem proving), each call of <v>F</v> \u000A expands to a corresponding call of <v>F-equal</v>.  For convenience, a macro \u000A <v>F-eq</v> is also defined, so that a call of <v>F-eq</v> expands to a \u000A corresponding call of <v>F</v> with <v>:TEST 'eq</v>.</p> \u000A \u000A <code><tt><see topic=\"ACL2____ADD-TO-SET\">add-to-set</see></tt> \u000A<tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt> \u000A<tt><see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see></tt> \u000A<tt><see topic=\"ACL2____INTERSECTION_42\">intersection$</see></tt> ; (see Note below) \u000A<tt><see topic=\"ACL2____INTERSECTP\">intersectp</see></tt> \u000A<tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> \u000A<tt><see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see></tt> \u000A<v>position-ac</v> \u000A<tt><see topic=\"COMMON-LISP____POSITION\">position</see></tt> \u000A<tt><see topic=\"ACL2____PUT-ASSOC\">put-assoc</see></tt> \u000A<tt><see topic=\"COMMON-LISP____RASSOC\">rassoc</see></tt> \u000A<tt><see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see></tt> \u000A<tt><see topic=\"ACL2____REMOVE1\">remove1</see></tt> \u000A<tt><see topic=\"COMMON-LISP____REMOVE\">remove</see></tt> \u000A<tt><see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see></tt> ; (see Note below) \u000A<tt><see topic=\"COMMON-LISP____SUBSETP\">subsetp</see></tt> \u000A<tt><see topic=\"ACL2____UNION_42\">union$</see></tt> ; (see Note below) \u000A</code> \u000A \u000A <p>Note: Three of the macros above have names ending with the character, \u000A `<v>$</v>': <tt><see topic=\"ACL2____INTERSECTION_42\">intersection$</see></tt>, <tt><see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see></tt>, and <tt><see topic=\"ACL2____UNION_42\">union$</see></tt>. \u000A In each case there is a corresponding Common Lisp primitive without the \u000A trailing `<v>$</v>': <v>intersection</v>, <v>set-difference</v>, and <v>union</v>. \u000A However, Common Lisp does not specify the order of elements in the list \u000A returned by those primitives, so ACL2 has its own.  Nevertheless, the only use \u000A of the trailing `<v>$</v>' is to distinguish the primitives; associated \u000A functions and macros, for example <v>union-eq</v> and <v>intersection-equal</v>, \u000A do not include the `<v>$</v>' character in their names.</p> \u000A \u000A <p>We conclude with a brief discussion of <see topic=\"ACL2____GUARDS\">guards</see>.  The expansion of any \u000A of the above macros depends on the keyword argument, which generates a \u000A function call with a guard suitable for the equality test being used. \u000A Consider for example the call <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test 'eq)</v>, or equivalently, \u000A <v>(<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> x lst)</v>.  Expanding these macros leads to a call of <tt><see topic=\"ACL2____MBE\">mbe</see></tt>; you can see how that goes by using <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt>.  Ultimately, \u000A the guard being checked is that of the function <v>member-eq-exec</v>, which is \u000A as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A    (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A  (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> lst))</code> \u000A \u000A <p>Care has been taken to ensure that this guard is checked during evaluation \u000A and also that it generates suitable proof obligations for guard verification \u000A (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>).  A guard violation might look something like \u000A this:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> 3 '(4 5))\u000A\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call\u000A(MEMBER-EQ-EXEC$GUARD-CHECK X LST), which is\u000A(<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X) (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> LST) (<see topic=\"ACL2____SYMBOL-LISTP\">SYMBOL-LISTP</see> LST)), is violated by\u000Athe arguments in the call (MEMBER-EQ-EXEC$GUARD-CHECK 3 '(4 5)).\u000ASee :DOC set-guard-checking for information about suppressing this\u000Acheck with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Above, <v>member-eq-exec$guard-check</v> is a function generated as part of \u000A ACL2's expansion of <v>member</v> with <v>:test 'eq</v>, and this function symbol \u000A can be quite reasonably ignored.  The important thing is that it refers to the \u000A guard for <v>member-eq-exec</v>, which as the name may suggest is intended to \u000A guard the execution of a call of <tt><see topic=\"ACL2____MEMBER-EQ\">member-eq</see></tt>, or a call of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> with <v>:test 'eq</v>.  The important part of the message above is the \u000A guard actually being violated: <v>(<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X) (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see>\u000A LST) (<see topic=\"ACL2____SYMBOL-LISTP\">SYMBOL-LISTP</see> LST))</v>.</p>"],

 "ACL2____LEMMA-INSTANCE":[["Hints"],"ACL2 Sources","ACL2","<p>Lemma instances are the objects one provides via <v>:use</v> and \u000A <v>:by</v> <see topic=\"ACL2____HINTS\">hints</see> \u000A (see <see topic=\"ACL2____HINTS\">hints</see>) to bring to the theorem prover's attention some previously \u000A proved or easily provable fact.  A typical use of the <v>:use</v> hint is given \u000A below.  The value specified is a list of five lemma instances.</p> \u000A \u000A <code>:use (reverse-reverse\u000A      (:type-prescription app)\u000A      (:instance assoc-of-app\u000A                 (x a) (y b) (z c))\u000A      (:functional-instance p-f\u000A                            (p consp) (f flatten))\u000A      (:instance (:theorem (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x))\u000A                 (x (flatten a))))</code> \u000A \u000A <p>Observe that an event name can be a lemma instance.  The <v>:use</v> hint \u000A allows a single lemma instance to be provided in lieu of a list, as in:</p> \u000A \u000A <code>:use reverse-reverse</code> \u000A \u000A <p>or</p> \u000A \u000A <code>:use (:instance assoc-of-app (x a) (y b) (z c))</code> \u000A \u000A <p>A lemma instance denotes a formula which is either known to be a theorem or \u000A which must be proved to be a theorem before it can be used.  To use a lemma \u000A instance in a particular subgoal, the theorem prover adds the formula as a \u000A hypothesis to the subgoal before the normal theorem proving heuristics are \u000A applied.</p> \u000A \u000A <p>A lemma instance, or <v>lmi</v>, is of one of the following five forms:</p> \u000A \u000A <p>(1) <v>name</v>, where <v>name</v> names a previously proved theorem, axiom, or \u000A definition and denotes the formula (theorem) of that name.</p> \u000A \u000A <p>(2) <v>rune</v>, where <v>rune</v> is a <see topic=\"ACL2____RUNE\">rune</see> (see <see topic=\"ACL2____RUNE\">rune</see>) denoting \u000A the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> justifying the rule named by the <see topic=\"ACL2____RUNE\">rune</see>.</p> \u000A \u000A <p>(3) <v>(:theorem term)</v>, where <v>term</v> is any term alleged to be a \u000A theorem.  Such a lemma instance denotes the formula <v>term</v>.  But before \u000A using such a lemma instance the system will undertake to prove <v>term</v>.</p> \u000A \u000A <p>(4) <v>(:instance lmi (v1 t1) ... (vn tn))</v>, where <v>lmi</v> is recursively \u000A a lemma instance, the <v>vi</v>'s are distinct variables and the <v>ti</v>'s are \u000A terms.  Such a lemma instance denotes the formula obtained by instantiating \u000A the formula denoted by <v>lmi</v>, replacing each <v>vi</v> by <v>ti</v>.  Normally \u000A ACL2 enforces the requirement that every variable <v>vi</v> must be bound in the \u000A formula denoted by <v>lmi</v>.  However, the keyword <v>:extra-bindings-ok</v> may \u000A be inserted immediately after the lemma instance in order to remove that \u000A requirement: <v>(:instance lmi :extra-bindings-ok (v1 t1) ... (vn tn))</v>.</p> \u000A \u000A <p>(5) <v>(:functional-instance lmi (f1 g1) ... (fn gn))</v>, where <v>lmi</v> is \u000A recursively a lemma instance and each <v>fi</v> is an ``instantiable'' function \u000A symbol of arity <v>ni</v> and <v>gi</v> is a function symbol, a macro alias for a \u000A function symbol <v>gi'</v> (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>) in which case we \u000A treat <v>gi</v> as <v>gi'</v>, or a pseudo-lambda expression of arity <v>ni</v>.  An \u000A instantiable function symbol is any defined or constrained function symbol \u000A except the primitives <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt>, <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt>, and <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>, and a few others, as listed by the constant \u000A <v>*non-instantiable-primitives*</v>.  These are built-in in such a way that we \u000A cannot recover the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s on them.  (Special case: a function \u000A introduced in the <v>:partial-theory</v> of a dependent clause-processor is not \u000A instantiable; see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>.)  A pseudo-lambda \u000A expression is an expression of the form <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (v1 ... vn) body)</v> where \u000A the <v>vi</v> are distinct variable symbols and <v>body</v> is any term.  No <i>a \u000A priori</i> relation is imposed between the <v>vi</v> and the variables of \u000A <v>body</v>, i.e., <v>body</v> may ignore some <v>vi</v>'s and may contain ``free'' \u000A variables.  However, we do not permit <v>v</v> to occur freely in <v>body</v> if \u000A the functional substitution is to be applied to any formula (<v>lmi</v> or the \u000A <see topic=\"ACL2____CONSTRAINT\">constraint</see>s to be satisfied) in a way that inserts <v>v</v> into the \u000A scope of a binding of <v>v</v> by <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> (or, <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt>).  If you happen to violate this restriction, an informative error \u000A message will be printed.  That message will list for you the potentially \u000A illegal choices for <v>v</v> in the context in which the functional substitution \u000A is offered.  A <v>:functional-instance</v> lemma instance denotes the formula \u000A obtained by functionally instantiating the formula denoted by <v>lmi</v>, \u000A replacing <v>fi</v> by <v>gi</v>.  However, before such a lemma instance can be \u000A used, the system will generate proof obligations arising from the replacement \u000A of the <v>fi</v>'s by the <v>gi</v>'s in constraints that ``support'' the lemma to \u000A be functionally instantiated; see <see topic=\"ACL2____CONSTRAINT\">constraint</see>.  One might expect that if \u000A the same instantiated constraint were generated on behalf of several events, \u000A then each of those instances would have to be proved.  However, for the sake \u000A of efficiency, ACL2 stores the fact that such an instantiated constraint has \u000A been proved and avoids it in future events.</p> \u000A \u000A <p>Note that ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>) imposes additional requirements for \u000A functional instantiation.  See <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-IN-ACL2R\">functional-instantiation-in-ACL2r</see>.</p> \u000A \u000A <p>Obscure case for <see topic=\"ACL2____DEFINITION\">definition</see>s.  If the lemma instance refers to a \u000A <v>:definition</v> <see topic=\"ACL2____RUNE\">rune</see>, then it refers to the <tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula \u000A of that rune, which can be a simplified (``normalized'') form of the original \u000A formula.  However, if the hint is a <v>:by</v> hint and the lemma instance is \u000A based on a name (i.e., a symbol), rather than a rune, then the formula is the \u000A original formula of the event, as shown by <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>, rather than the \u000A normalized version, as shown by <v>:</v><tt><see topic=\"ACL2____PF\">pf</see></tt>.  This is as one would \u000A expect: If you supply the name of an event, you expect it to refer to the \u000A original event.  For <v>:use</v> hints we use the simplified (normalized) form \u000A instead, which is reasonable since one would expect simplification during the \u000A proof that re-traces the normalization done at the time the rule was \u000A created.</p> \u000A \u000A <p>See <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE\">functional-instantiation-example</see> for an example of the use of \u000A <v>:functional-instance</v> (so-called ``functional instantiation).''</p>"],

 "ACL2____NOTE-2-8":[["Release-notes"],"ACL2 Sources","ACL2","<p>BRIEF SUMMARY.</p> \u000A \u000A <p>The Version_2.8 notes are divided into the indicated subtopics.  Here we \u000A give only a brief summary of just a few of the major new features and changes \u000A that seem most likely to impact existing proofs.  Not included in this brief \u000A summary, but included in the subtopics, are descriptions of many improvements \u000A (including bug fixes and new functionality) that should not get in the way of \u000A existing proof efforts.  In the description below we also omit discussion of \u000A changes that will become clear by way of error messages if they affect \u000A you.</p> \u000A \u000A <p>In particular, please see <see topic=\"ACL2____NOTE-2-8-NEW-FUNCTIONALITY\">note-2-8-new-functionality</see> for discussion \u000A of a number of new features that you may find useful.</p> \u000A \u000A <p>Acknowledgements and elaboration, as well as other changes, can be found in \u000A the subtopics listed below.</p> \u000A \u000A <p>o Some of the bug fixes (see <see topic=\"ACL2____NOTE-2-8-BUG-FIXES\">note-2-8-bug-fixes</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ Some soundness bugs were fixed.</p> \u000A \u000A <p>+ The handling of free variables in hypotheses (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) \u000A of rewrite and linear rules had a bug that prevented some proofs from going \u000A through.  Now that this bug has been fixed, you may find some proofs running \u000A much more slowly than before.  You can use <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> and \u000A <tt><see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see></tt> to remedy this situation; see <see topic=\"ACL2____NOTE-2-8-BUG-FIXES\">note-2-8-bug-fixes</see> for details.</p> \u000A \u000A <p>+ The <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see> in the current logical <see topic=\"ACL2____WORLD\">world</see> are no \u000A longer ignored by <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.</p> \u000A \u000A <p>+ Forms violating guard-checking such as <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *silly* (<see topic=\"COMMON-LISP____CAR\">car</see> 3))</v> \u000A are now allowed in <see topic=\"ACL2____BOOKS\">books</see>.</p></blockquote> \u000A \u000A <p>o Some of the new functionality (see <see topic=\"ACL2____NOTE-2-8-NEW-FUNCTIONALITY\">note-2-8-new-functionality</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ WARNING: You may find that <v>control-d</v> (in emacs, <v>control-c\u000A control-d</v>) can throw you completely out of Lisp where it had not formerly \u000A done so.</p> \u000A \u000A <p>+ ACL2 now starts up inside the ACL2 loop &mdash; that is, <v>(</v><tt><see topic=\"ACL2____LP\">lp</see></tt><v>)</v> is executed automatically &mdash; when built on CLISP or Allegro CL. \u000A This was already the case for GCL and CMUCL, and it still is not true for \u000A LispWorks.</p> \u000A \u000A <p>+ See <see topic=\"ACL2____NOTE-2-8-ORDINALS\">note-2-8-ordinals</see> for a discussion of a significant change in \u000A ordinal represtation, and in particular, for how to preserve existing proofs \u000A that depend on the previous ordinal representation.</p> \u000A \u000A <p>+ Macros <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (``must be equal''), <tt><see topic=\"ACL2____MBT\">mbt</see></tt> (``must be true''), \u000A and <tt><see topic=\"ACL2____DEFEXEC\">defexec</see></tt> have been introduced, which allow the user to attach \u000A alternate executable definitions to functions.</p> \u000A \u000A <p>+ The user can now control multiple matching for free variables in \u000A hypotheses for <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules, as has already been \u000A supported for <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules.</p> \u000A \u000A <p>+ It is no longer necessary to specify <v>(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> nil)</v> in \u000A order to avoid errors when a rule with free variables in its hypotheses is \u000A missing the <v>:match-free</v> field.</p> \u000A \u000A <p>+ The form <v>(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>)</v> causes, at least for most Lisps, entry \u000A into the Lisp debugger whenever ACL2 causes an error.</p> \u000A \u000A <p>+ A new <tt><see topic=\"ACL2____TABLE\">table</see></tt> has been provided so that advanced users can override \u000A the built-in <v>untranslate</v> functionality.  See <see topic=\"ACL2____USER-DEFINED-FUNCTIONS-TABLE\">user-defined-functions-table</see>.</p> \u000A \u000A <p>+ The <tt><see topic=\"ACL2____PSTACK\">pstack</see></tt> (`process [prover] stack'') mechanism, formerly \u000A denoted <v>checkpoints</v>, has been improved.  One of these improvements is to \u000A show actual parameters with <v>(<see topic=\"ACL2____PSTACK\">pstack</see> t)</v> rather than formals.</p> \u000A \u000A <p>+ The <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event is now allowed to take an <v>:inline</v> \u000A argument, which can speed up execution.</p> \u000A \u000A <p>+ Macro <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> no longer takes arguments for the <v>gstack</v> or \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  To print terms in full rather than abbreviated: <v>(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see>\u000A :evisc-tuple nil)</v>.</p> \u000A \u000A <p>+ The <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event now has an additional (optional) keyword, \u000A <v>:dir</v>.  In particular, <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo/bar&quot; :dir :system)</v> will \u000A include the indicated book after prepending the path of the built-in \u000A <v>books/</v> directory.  You will probably not find <v>:dir :system</v> to be \u000A useful if you move the executable image or distributed books; see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, in particular its ``soundness warning''.</p> \u000A \u000A <p>+ The printing of results in raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) may now be \u000A partially controlled by the user: see <see topic=\"ACL2____ADD-RAW-ARITY\">add-raw-arity</see>.</p> \u000A \u000A <p>+ For those using Unix/Linux `make': A <v>cert.acl2</v> file can contain \u000A forms to be evaluated before an appropriate <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command is \u000A invoked automatically (not included in <v>cert.acl2</v>).</p></blockquote> \u000A \u000A <p>o Some of the changes in the proof engine (see <see topic=\"ACL2____NOTE-2-8-PROOFS\">note-2-8-proofs</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ ACL2 now prevents certain rewriting loops; see <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>.</p> \u000A \u000A <p>+ Small changes have been made to heuristics for controlling rewriting \u000A during proofs by induction and in handling certain ``weak'' <see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see> rules.</p> \u000A \u000A <p>+ The handling of free variables in a hypothesis of a <see topic=\"ACL2____REWRITE\">rewrite</see> rule \u000A (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) has been improved in the case that the hypothesis \u000A is of the form <v>(equiv x y)</v>, where <v>equiv</v> is a known equivalence \u000A relation (see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>).</p> \u000A \u000A <p>+ We have modified how the ACL2 simplifier handles the application of a \u000A defined function symbol to constant arguments, by avoiding the introduction of \u000A <see topic=\"ACL2____HIDE\">hide</see> when evaluation fails if the term can be rewritten.</p> \u000A \u000A <p>+ The generation of \"Goal\" for recursive (and mutually-recursive) \u000A definitions now uses the subsumption/replacement limitation (default 500). \u000A See <see topic=\"ACL2____CASE-SPLIT-LIMITATIONS\">case-split-limitations</see>.</p> \u000A \u000A <p>+ Default hints now apply to hints given in definitions, not just theorems. \u000A See <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p>+ Linear arithmetic now uses the conclusions of <tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> \u000A rules, and <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> now uses a small amount of linear reasoning when \u000A deciding inequalities.</p></blockquote> \u000A \u000A <p>o Some of the changes in rules, definitions, and constants (see <see topic=\"ACL2____NOTE-2-8-RULES\">note-2-8-rules</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ See the above doc topic.</p></blockquote> \u000A \u000A <p>o Guard-related changes are described in see <see topic=\"ACL2____NOTE-2-8-BUG-FIXES\">note-2-8-bug-fixes</see>.</p> \u000A \u000A <p>o Some of the proof-checker changes (see <see topic=\"ACL2____NOTE-2-8-PROOF-CHECKER\">note-2-8-proof-checker</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ Added new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>wrap1</v>, <v>wrap</v>, and \u000A <v>wrap-induct</v>, to combine multiple conjuncts or goals.</p> \u000A \u000A <p>+ The <v>type-alist</v> command now takes optional arguments that control \u000A whether or not the governors and/or conclusion are used in computing the \u000A context.</p></blockquote> \u000A \u000A <p>o Some of the system-level changes (see <see topic=\"ACL2____NOTE-2-8-SYSTEM\">note-2-8-system</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ ACL2 now runs on OpenMCL and on MCL 5.0.</p></blockquote> \u000A \u000A <p>o Some of the other changes (see <see topic=\"ACL2____NOTE-2-8-OTHER\">note-2-8-other</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ Emacs file <v>emacs/emacs-acl2.el</v> has been updated (see <see topic=\"ACL2____NOTE-2-8-OTHER\">note-2-8-other</see> for details).</p> \u000A \u000A <p>+ When <v>:pl</v> is given a term other than a symbol, it will print all \u000A rewrite rules that match that term.</p> \u000A \u000A <p>+ A new function, <tt><see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see></tt>, returns a symbol in the given \u000A package.</p> \u000A \u000A <p>+ The list constant <v>*acl2-exports*</v> has been extended.</p> \u000A \u000A <p>+ A new release of the rtl library has been included: <v>books/rtl/rel4/</v>. \u000A See the <v>README</v> file in that directory.</p></blockquote> \u000A \u000A <p>Again, please proceed to the subtopics for more thorough release \u000A notes.</p>"],

 "ACL2____EXTENDED-METAFUNCTIONS":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>General Form of an Extended :Meta theorem:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)              ; this hyp is optional\u000A              (<see topic=\"ACL2____ALISTP\">alistp</see> a)                    ; this hyp is optional\u000A              (ev (hyp-fn x mfc state) a)   ; this hyp is optional\u000A              ; meta-extract hyps may also be included (see below)\u000A              )\u000A         (equiv (ev x a)\u000A                (ev (fn x mfc state) a)))</code> \u000A \u000A <p>where the restrictions are as described in the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for \u000A <tt><see topic=\"ACL2____META\">meta</see></tt> where <v>state</v> is literally the symbol <v>STATE</v>, and <v>x</v>, \u000A <v>a</v>, <v>mfc</v>, and <v>state</v> are distinct variable symbols.  A <v>:meta</v> \u000A theorem of the above form installs <v>fn</v> as a metatheoretic simplifier with \u000A hypothesis function <v>hyp-fn</v>, exactly as for vanilla metafunctions.  The \u000A only difference is that when the metafunctions are applied, some contextual \u000A information is passed in via the <v>mfc</v> argument and the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A is made available.</p> \u000A \u000A <p>See <see topic=\"ACL2____META\">meta</see> for a discussion of vanilla flavored metafunctions.  This \u000A documentation assumes you are familiar with the simpler situation, in \u000A particular, how to define a vanilla flavored metafunction, <v>fn</v>, and its \u000A associated hypothesis metafunction, <v>hyp-fn</v>, and how to state and prove \u000A metatheorems installing such functions.  Defining extended metafunctions \u000A requires that you also be familiar with many ACL2 implementation details. \u000A This documentation is sketchy on these details; see the ACL2 source code or \u000A email the <see topic=\"ACL2____ACL2-HELP\">ACL2-help</see> list if you need more help.</p> \u000A \u000A <p>Additional hypotheses are supported, called ``meta-extract hypotheses'', \u000A that allow metafunctions to depend on the validity of certain terms extracted \u000A from the context or the logical <see topic=\"ACL2____WORLD\">world</see>.  These hypotheses provide an \u000A even more advanced form of metatheorem so we explain them elsewhere; see <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>.</p> \u000A \u000A <p>The metafunction context, <v>mfc</v>, is a list containing many different \u000A data structures used by various internal ACL2 functions.  We do not document \u000A the form of <v>mfc</v>.  Your extended metafunction should just take <v>mfc</v> as \u000A its second formal and pass it into the functions mentioned below.  The ACL2 \u000A <v>state</v> is well-documented (see <see topic=\"ACL2____STATE\">state</see>).  Below we present \u000A expressions below that can be useful in defining extended metafunctions.  Some \u000A of these expressions involve keyword arguments, <v>:forcep</v> and <v>:ttree</v>, \u000A which are optional and in most cases are fine to omit, and which we explain \u000A after we present the useful expressions.</p> \u000A \u000A <p><v>(mfc-clause mfc)</v>: returns the current goal, in clausal form.  A clause \u000A is a list of ACL2 terms, implicitly denoting the disjunction of the listed \u000A terms.  The clause returned by <v>mfc-clause</v> is the clausal form of the \u000A translation (see <see topic=\"ACL2____TRANS\">trans</see>) of the goal or subgoal on which the rewriter is \u000A working.  When a metafunction calls <v>mfc-clause</v>, the term being rewritten \u000A by the metafunction either occurs somewhere in this clause or, perhaps more \u000A commonly, is being rewritten on behalf of some lemma to which the rewriter has \u000A backchained while trying to rewrite a term in the clause.</p> \u000A \u000A <p><v>(mfc-ancestors mfc)</v>: returns an alist whose keys are the negations of \u000A the backchaining hypotheses being pursued.  In particular, <v>(<see topic=\"COMMON-LISP____NULL\">null</see>\u000A (mfc-ancestors mfc))</v> will be true exactly when rewriting is on part of the \u000A current goal.  Exception: An element of this alist whose key is of the form \u000A <v>(:binding-hyp hyp unify-subst)</v> indicates that <v>hyp</v> has been \u000A encountered as a hypothesis of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term)</v> or <v>(equiv var\u000A (double-rewrite-term))</v>, in each case binding variable <v>var</v> to the result \u000A of rewriting <v>term</v> under <v>unify-subst</v>.</p> \u000A \u000A <p><v>(mfc-rdepth mfc)</v>: returns the remaining stack depth for calls of the \u000A rewriter (by default, <v>*default-rewrite-stack-limit*</v> at the top level; see \u000A <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>).  When this is 0, no further calls of the rewriter \u000A can be made without error.</p> \u000A \u000A <p><v>(mfc-type-alist mfc)</v>: returns the type-alist governing the occurrence \u000A of the term, <v>x</v>, being rewritten by the metafunction.  A type-alist is an \u000A association list, each element of which is of the form <v>(<see topic=\"ACL2____TERM\">term</see> ts . ttree)</v>. \u000A Such an element means that the term <v>term</v> has the <see topic=\"ACL2____TYPE-SET\">type-set</see> <v>ts</v>; \u000A see <see topic=\"ACL2____TYPE-ALIST\">type-alist</see>.  The <v>ttree</v> component is probably irrelevant here. \u000A All the terms in the type-alist are in translated form (see <see topic=\"ACL2____TRANS\">trans</see>). \u000A The <v>ts</v> are numbers denoting finite Boolean combinations of ACL2's \u000A primitive types (see <see topic=\"ACL2____TYPE-SET\">type-set</see>).  The type-alist includes not only \u000A information gleaned from the conditions governing the term being rewritten but \u000A also that gleaned from forward chaining from the (negations of the) other \u000A literals in the clause returned by <v>mfc-clause</v>.</p> \u000A \u000A <p><v>(mfc-unify-subst mfc)</v>: returns <v>nil</v> except when evaluating a \u000A <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> or <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypothesis, in which case, returns the \u000A unifying substitution present at the start of that evaluation.</p> \u000A \u000A <p><v>(mfc-world mfc)</v>: returns the ACL2 logical <tt><see topic=\"ACL2____WORLD\">world</see></tt>.</p> \u000A \u000A <p><v>(mfc-ts term mfc state :forcep forcep :ttreep ttreep)</v>: returns the \u000A <v>type-set</v> of <v>term</v> in the current context; see <see topic=\"ACL2____TYPE-SET\">type-set</see>.</p> \u000A \u000A <p><v>(mfc-rw term obj equiv-info mfc state)</v>: returns the result of \u000A rewriting <v>term</v> in the current context, <v>mfc</v>, with objective <v>obj</v> \u000A and the equivalence relation described by <v>equiv-info</v>.  <v>Obj</v> should be \u000A <v>t</v>, <v>nil</v>, or <v>?</v>, and describes your objective: try to show that \u000A <v>term</v> is true, false, or anything.  <v>Equiv-info</v> is either <v>nil</v>, \u000A <v>t</v>, a function symbol <v>fn</v> naming a known equivalence relation, or a \u000A list of congruence rules.  <v>Nil</v> means return a term that is <v>equal</v> to \u000A <v>term</v>.  <v>T</v> means return a term that is propositionally equivalent \u000A (i.e., in the <v>iff</v> sense) to <v>term</v>, while <v>fn</v> means return a term \u000A <v>fn</v>-equivalent to <v>term</v>.  The final case, which is intended only for \u000A advanced users, allows the specification of generated equivalence relations, \u000A as supplied to the <v>geneqv</v> argument of <v>rewrite</v>.  Generally, if you \u000A wish to establish that <v>term</v> is true in the current context, use the \u000A idiom</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (mfc-rw term t t mfc state) *t*)</code> \u000A \u000A <p>The constant <v>*t*</v> is set to the internal form of the constant term \u000A <v>t</v>, i.e., <v>'t</v>.</p> \u000A \u000A <p><v>(mfc-rw+ term alist obj equiv-info mfc state)</v>: if <v>alist</v> is \u000A <v>nil</v> then this is equivalent to <v>(mfc-rw term obj equiv-info mfc\u000A state)</v>.  However, the former takes an argument, <v>alist</v>, that binds \u000A variables to terms, and returns the result of rewriting <v>term</v> under that \u000A <v>alist</v>, where this rewriting is as described for <v>mfc-rw</v> above.  The \u000A function <v>mfc-rw+</v> can be more efficient than <v>mfc-rw</v>, because the \u000A terms in the binding alist have generally already been rewritten, and it can \u000A be inefficient to rewrite them again.  For example, consider a rewrite rule of \u000A the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A              (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (... (mfc-rw `(bar ,x) ...) ...))\u000A              ...)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (... x ...) ...))</code> \u000A \u000A <p>Here, <v>x</v> is bound in the conclusion to the result of rewriting some \u000A term, say, <v>tm</v>.  Then the above call of <v>mfc-rw</v> will rewrite <v>tm</v>, \u000A which is probably unnecessary.  So a preferable form of the rule above may be \u000A as follows, so that <v>tm</v> is not further rewritten by <v>mfc-rw+</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ...\u000A              (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (... (mfc-rw+ '(bar v) `((v . ,x)) ...) ...))\u000A              ...)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (... x ...) ...))</code> \u000A \u000A <p>However, you may find that the additional rewriting done by <v>mfc-rw</v> is \u000A useful in some cases.</p> \u000A \u000A <p><v>(mfc-ap term mfc state)</v>: returns <v>t</v> or <v>nil</v> according to \u000A whether linear arithmetic can determine that <v>term</v> is false.  To the \u000A cognoscenti: returns the contradiction flag produced by linearizing <v>term</v> \u000A and adding it to the <v>linear-pot-lst</v>.</p> \u000A \u000A <p><v>(mfc-relieve-hyp hyp alist rune target bkptr mfc state)</v>: returns \u000A <v>t</v> or <v>nil</v> according to whether the indicated hypothesis term, \u000A <v>hyp</v>, can be relieved (proved) under the giving variable bindings, \u000A <v>alist</v>.  Note that this function returns <v>nil</v> if <v>hyp</v> has free \u000A variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).  Here, <v>hyp</v> is the hypothesis of \u000A the indicated <tt><see topic=\"ACL2____RUNE\">rune</see></tt> at (one-based) position <v>bkptr</v>, and <v>target</v> \u000A is an instantiated term to which <v>rune</v> is being applied.  Note that no \u000A indication is returned for whether any assumptions have been generated by \u000A <tt><see topic=\"ACL2____FORCE\">force</see></tt> or <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt>.  (If you need such a feature, feel free \u000A to request it of the ACL2 implementors.)</p> \u000A \u000A <p>We explain the <v>:forcep</v> and <v>:ttreep</v> keyword arguments provided in \u000A some expressions, as promised above.  Their defaults are <v>:same</v> and \u000A <v>nil</v>, respectively.  A value of <v>:same</v> for <v>:forcep</v> means that \u000A forcing will be allowed if and only if it is allowed in the current rewriting \u000A environment; see <see topic=\"ACL2____FORCE\">force</see>.  A value of <v>t</v> or <v>nil</v> for <v>:forcep</v> \u000A overrides this default and allows or disallows forcing, respectively.  By \u000A default these functions return a single value, <v>val</v>, but if <v>:ttreep</v> \u000A is <v>t</v> then they return <v>(<see topic=\"ACL2____MV\">mv</see> val ttree)</v>, where <v>ttree</v> is the \u000A tag-tree (see <see topic=\"ACL2____TTREE\">ttree</see>) returned by the indicated operation, with an input \u000A tag-tree of <v>nil</v>).</p> \u000A \u000A <p>During the execution of a metafunction by the theorem prover, the \u000A expressions above compute the results specified.  Typically, you should \u000A imagine that there are no axioms about the <v>mfc-</v> function symbols: treat \u000A them as uninterpreted function symbols.  There is an advanced feature, \u000A meta-extract hypotheses, that can avoid this logical weakness in some cases \u000A when proving <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules; see <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>.  But we assume \u000A for the rest of the present <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic that you do not use \u000A meta-extract hypotheses.  Thus, in the proof of the correctness of a \u000A metafunction, no information is available about the results of these \u000A functions: you should <i>use these functions for heuristic purposes \u000A only</i>.</p> \u000A \u000A <p>For example, your metafunction may use these functions to decide whether to \u000A perform a given transformation, but the transformation must be sound \u000A regardless of the value that your metafunction returns.  We illustrate this \u000A below.  It is sometimes possible to use the hypothesis metafunction, \u000A <v>hyp-fn</v>, to generate a sufficient hypothesis to justify the \u000A transformation.  The generated hypothesis might have already been ``proved'' \u000A internally by your use of <v>mfc-ts</v> or <v>mfc-rw</v>, but the system will have \u000A to prove it ``officially'' by relieving it.  We illustrate this below \u000A also.</p> \u000A \u000A <p>We conclude with a script that defines, verifies, and uses several extended \u000A metafunctions.  This script is based on one provided by Robert Krug, who was \u000A instrumental in the development of this style of metafunction and whose help \u000A we gratefully acknowledge.</p> \u000A \u000A <code>; Here is an example.  I will define (foo i j) simply to be (<see topic=\"COMMON-LISP_____B2\">+</see> i j).\u000A; But I will keep its definition disabled so the theorem prover\u000A; doesn't know that.  Then I will define an extended metafunction\u000A; that reduces (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) to 0 provided i has integer type and the\u000A; expression (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i) occurs as a hypothesis in the clause.\u000A\u000A; Note that (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) is 0 in any case.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (i j) (<see topic=\"COMMON-LISP_____B2\">+</see> i j))\u000A\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> eva eva-lst ((foo i j)\u000A                           (<see topic=\"ACL2____UNARY--\">unary--</see> i)\u000A\u000A; I won't need these two cases until the last example below, but I\u000A; include them now.\u000A\u000A                           (<see topic=\"COMMON-LISP____IF\">if</see> x y z)\u000A                           (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)))\u000A\u000A(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> metafn (x mfc state)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'foo)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CADDR\">caddr</see> x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'unary-- (<see topic=\"COMMON-LISP____CADR\">cadr</see> x))))\u000A\u000A; So x is of the form (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)).  Now I want to check some other\u000A; conditions.\u000A\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (ts-subsetp (mfc-ts (<see topic=\"COMMON-LISP____CADR\">cadr</see> x) mfc state)\u000A                            *ts-integer*)\u000A                (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> `(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> '10 ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))\u000A                              (mfc-clause mfc)))\u000A           (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> 0)))\u000A          (t x)))\u000A   (t x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> metafn-correct\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (eva x a) (eva (metafn x mfc state) a))\u000A  :rule-classes ((:meta :trigger-fns (foo))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> foo))\u000A\u000A; The following will fail because the metafunction won't fire.\u000A; We don't know enough about i.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0))\u000A\u000A; Same here.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 i)) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0)))\u000A\u000A; But this will work.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0)))\u000A\u000A; This won't, because the metafunction looks for (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i) literally,\u000A; not just something that implies it.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 11 i))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0)))\u000A\u000A; Now I will undo the defun of metafn and repeat the exercise, but\u000A; this time check the weaker condition that (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i) is provable\u000A; (<see topic=\"ACL2____BY\">by</see> rewriting it) rather than explicitly present.\u000A\u000A(<see topic=\"ACL2____UBT\">ubt</see> 'metafn)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> metafn (x mfc state)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'foo)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CADDR\">caddr</see> x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'unary-- (<see topic=\"COMMON-LISP____CADR\">cadr</see> x))))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (ts-subsetp (mfc-ts (<see topic=\"COMMON-LISP____CADR\">cadr</see> x) mfc state)\u000A                            *ts-integer*)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (mfc-rw `(<see topic=\"COMMON-LISP_____C3\">&lt;</see> '10 ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> x)) t t mfc state)\u000A                       *t*))\u000A\u000A; The mfc-rw above rewrites (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i) with objective t and iffp t.  The\u000A; objective means the theorem prover will try to establish it.  The\u000A; iffp means the theorem prover can rewrite maintaining propositional\u000A; equivalence instead of strict equality.\u000A\u000A           (<see topic=\"COMMON-LISP____QUOTE\">quote</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> 0)))\u000A          (t x)))\u000A   (t x)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> metafn-correct\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (eva x a) (eva (metafn x mfc state) a))\u000A  :rule-classes ((:meta :trigger-fns (foo))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> foo))\u000A\u000A; Now it will prove both:\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 10 i))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0)))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 11 i))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo i (<see topic=\"COMMON-LISP____-\">-</see> i)) 0)))\u000A\u000A; Now I undo the defun of metafn and change the problem entirely.\u000A; This time I will rewrite (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo i j)) to t.  Note that\u000A; this is true if i and j are integers.  I can check this\u000A; internally, but have to generate a hyp-fn to make it official.\u000A\u000A(<see topic=\"ACL2____UBT\">ubt</see> 'metafn)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> metafn (x mfc state)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'integerp)\u000A         (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)) 'foo))\u000A\u000A; So x is (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo i j)).  Now check that i and j are\u000A; ``probably'' integers.\u000A\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (ts-subsetp (mfc-ts (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)) mfc state)\u000A                            *ts-integer*)\u000A                (ts-subsetp (mfc-ts (<see topic=\"COMMON-LISP____CADDR\">caddr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)) mfc state)\u000A                            *ts-integer*))\u000A           *t*)\u000A          (t x)))\u000A   (t x)))\u000A\u000A; To justify this transformation, I generate the appropriate hyps.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hyp-fn (x mfc state)\u000A\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> mfc state))\u000A\u000A; The hyp-fn is run only if the metafn produces an answer different\u000A; from its input.  Thus, we know at execution time that x is of the\u000A; form (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo i j)) and we know that metafn rewrote\u000A; (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) and (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j) both to t.  So we just produce their\u000A; conjunction.  Note that we must produce a translated term; we\u000A; cannot use the macro AND and must quote constants!  Sometimes you\u000A; must do tests in the hyp-fn to figure out which case the metafn\u000A; produced, but not in this example.\u000A\u000A           `(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> ,(<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> ,(<see topic=\"COMMON-LISP____CADDR\">caddr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))\u000A                'nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> metafn-correct\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (eva (hyp-fn x mfc state) a)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (eva x a) (eva (metafn x mfc state) a)))\u000A  :rule-classes ((:meta :trigger-fns (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see>))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> foo))\u000A\u000A; This will not be proved.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo i j))))\u000A\u000A; But this will be.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo i j))))</code>"],

 "ACL2____SET-EVISC-TUPLE":[["Io"],"ACL2 Sources","ACL2","<p>ACL2 output is generally printed in full.  However, ACL2 can be \u000A directed to abbreviate, or ``eviscerate'', objects before printing them, \u000A though the use of a so-called ``evisc-tuple''.  See <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> for a \u000A discussion of evisc-tuples.  The utility <v>set-evisc-tuple</v> modifies certain \u000A global evisc-tuples, as explained below, to affect the extent to which ACL2 \u000A eviscerates objects during printing, for example during proof output or when \u000A printing top-level evaluation results.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> evisc-tuple  ; a legal evisc-tuple, or :DEFAULT\u000A                :iprint val   ; one of *iprint-actions*\u000A                :sites sites) ; either :ALL, or an element or a subset of\u000A                              ;   the list (:TERM :LD :TRACE :ABBREV)</code> \u000A \u000A <p>where the value of <v>:iprint</v> is passed to <tt><see topic=\"ACL2____SET-IPRINT\">set-iprint</see></tt>, <v>:sites\u000A :all</v> abbreviates <v>:sites '(:TERM :LD :TRACE :ABBREV)</v>, and other \u000A documentation is provided below.  Note that all arguments are evaluated.</p> \u000A \u000A <p>See <see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> for how to avoid evisceration for ACL2 output.</p> \u000A \u000A <p>The following example illustrates an invocation of <v>set-evisc-tuple</v> \u000A that limits the print-level to 3 &mdash; only three descents into list \u000A structures are permitted before eviscerating a subterm &mdash; and limits the \u000A print-length to 4 &mdash; only the first four elements of any list structure \u000A will be printed.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 3   ; print-level\u000A                                     4   ; print-length\u000A                                     nil ; alist\u000A                                     nil ; hiding-cars\u000A                                     )\u000A                        :iprint :same ; better yet, T\u000A                        :sites :all)\u000A (:TERM :LD :TRACE :ABBREV)\u000AACL2 !&gt;'((a b ((c d)) e f g) u v w x y)\u000A((A B (#) E ...) U V W ...)\u000AACL2 !&gt;</code> \u000A \u000A <p>We recommend however using <v>:iprint t</v> so that eviscerated terms may be \u000A read back in; see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.  Indeed, the <v>:iprint</v> argument is \u000A required as a reminder to the user to consider that issue, unless iprinting \u000A has been enabled at least once.  If <v>:sites</v> or a required <v>:iprint</v> \u000A argument is omitted, however, ACL2 will query the user for the missing \u000A arguments rather than causing an error.</p> \u000A \u000A <p>ACL2 eviscerates by default only in a few cases, primarily in informational \u000A messages for errors, warnings, and queries (i.e., in the <v>:EVISC</v> case \u000A below).  Users can modify the default behavior by supplying a suitable \u000A argument to <v>set-evisc-tuple</v>.  The argument may be <v>:default</v>, which \u000A denotes the evisceration provided when ACL2 starts up.  Otherwise that \u000A argument is an evisc-tuple, which is either <v>nil</v> (no evisceration) or as \u000A described above.  Moreover, there are four evisc-tuple ``evisceration \u000A contexts'', each with its own evisceration control.  The value returned by \u000A <v>set-evisc-tuple</v> indicates the evisceration contexts whose evisc-tuple has \u000A been set.  The evisceration contexts are as follows, all of which use a \u000A default value of <v>nil</v> for the hiding-cars.  Accessors are also shown for \u000A retrieving the corresponding evisc-tuple.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o <v>:TERM</v> &mdash; used for printing terms.  The accessor is \u000A <v>(term-evisc-tuple flg state)</v>, where <v>flg</v> is ignored if \u000A <v>set-evisc-tuple</v> has been called for <v>:term</v> with value other than \u000A <v>:default</v>, and otherwise (hence initially): a <v>flg</v> of <v>nil</v> \u000A indicates an evisc-tuple of <v>nil</v>, and otherwise the term-evisc-tuple has a \u000A print-level of 3 and print-length of 4.</p> \u000A \u000A <p>o <v>:ABBREV</v> &mdash; used for printing informational messages for errors, \u000A warnings, and queries.  Initially, the alist abbreviates the ACL2 <v>world</v>, \u000A print-level is 5, and print-level is 7.  The accessor is \u000A <v>(abbrev-evisc-tuple state)</v>.</p> \u000A \u000A <p>o <v>:GAG-MODE</v> &mdash; used for printing induction schemes (and perhaps, \u000A in the future, for other printing) when <see topic=\"ACL2____GAG-MODE\">gag-mode</see> is on.  If gag-mode is \u000A off, the value used for this <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> is <v>(term-evisc-tuple nil\u000A state)</v>.  But if gag-mode is on (i.e., <v>(<see topic=\"ACL2____GAG-MODE\">gag-mode</see>)</v> evaluates to a \u000A non-<v>nil</v> value), then with one exception, the value is an evisc-tuple or \u000A <v>nil</v>, to be used in gag-mode for printing of induction schemes and, during \u000A proofs, the ``The non-trivial part of the guard conjecture''.  The exceptional \u000A value is <v>t</v>, which indicates that in gag-mode, no printing of induction \u000A schemes should be and the guard conjecture should be printed using \u000A <v>(term-evisc-tuple t state)</v>.  The accessor is <v>(gag-mode-evisc-tuple\u000A state)</v>.</p> \u000A \u000A <p>o <v>:LD</v> &mdash; used by the ACL2 read-eval-print loop.  The accessor is \u000A <v>(</v><tt><see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see></tt><v> state)</v>.</p> \u000A \u000A <p>o <v>:TRACE</v> &mdash; used for printing <see topic=\"COMMON-LISP____TRACE\">trace</see> output.  No accessor \u000A is available (though in raw Lisp, <v>(trace-evisc-tuple)</v> returns the \u000A trace-evisc-tuple).</p></blockquote> \u000A \u000A <p>Each context <v>ectx</v> also has an updater, <v>(set-ectx-evisc-tuple val\u000A state)</v>, where <v>val</v> is a legal value for <v>set-evisc-tuple</v> as \u000A described above: <v>:default</v> or an <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> (possibly \u000A <v>nil</v>).</p> \u000A \u000A <p>Note that the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> commands and the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A generally do their printing using the term-evisc-tuple.</p>"],

 "ACL2____COMPILATION":[["Programming"],"ACL2 Sources","ACL2","<p>ACL2 has several mechanisms to speed up the evaluation of function \u000A calls by <i>compiling</i> functions: see <see topic=\"ACL2____COMP\">comp</see>, see <see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see>, and see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>.  The intention is that \u000A compilation never changes the value returned by a function call, though it \u000A could cause the call to succeed rather than fail, for example by avoiding a \u000A stack overflow.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable <v>'compiler-enabled</v> is set \u000A automatically when the system is built, and may depend on the underlying Lisp \u000A implementation.  (In order to disable the compiler at build time, which will \u000A defeat the speed-up but usually be pretty harmless when the host Lisp is CCL \u000A or SBCL, see the discussion of <v>ACL2_COMPILER_DISABLED</v> in distributed file \u000A <v>GNUmakefile</v>.)  The value of <v>'compiler-enabled</v>, as returned by <v>(<see topic=\"ACL2_____04\">@</see>\u000A compiler-enabled)</v>, can be <v>t</v>, <v>:books</v>, or <v>nil</v>.  If the value is \u000A <v>nil</v>, then <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> coerce their \u000A arguments <v>:load-compile-file</v> and <v>compile-flg</v> arguments \u000A (respectively) to <v>nil</v>.  Otherwise, the value is <v>:books</v> or <v>t</v> and \u000A there is no such coercion; but if the value is not <v>t</v>, then <tt><see topic=\"ACL2____COMP\">comp</see></tt> \u000A and <tt><see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see></tt> are no-ops, which is probably desirable for Lisps \u000A such as CCL and SBCL that compile on-the-fly even when the compiler is not \u000A explicitly invoked.</p> \u000A \u000A <p>However, you may have reason to want to change the above (default) \u000A behavior.  To enable compilation by default for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> but not for <tt><see topic=\"ACL2____COMP\">comp</see></tt> or <tt><see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see></tt>:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> :books state)</code> \u000A \u000A <p>To enable compilation not only as above but also for <tt><see topic=\"ACL2____COMP\">comp</see></tt> and \u000A <tt><see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see></tt>:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> t state)</code> \u000A \u000A <p>To suppress compilation and loading of compiled files by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> (for example, if you get a raw Lisp error such as ``Wrong FASL \u000A version''):</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> nil state)</code> \u000A \u000A <p>Remark for users of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  If <v>set-compiler-enabled</v> is \u000A invoked during <v>make-event</v> expansion, its effect on <tt><see topic=\"ACL2____STATE\">state</see></tt> global \u000A variable <v>'compiler-enabled</v> will persist after evaluation completes for \u000A that <v>make-event</v> form.  So for example, one might use the following idiom \u000A in a book so that for all books included on behalf of a given <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form, no compiled files are loaded, but (optionally) no such \u000A effect takes place for later <v>include-book</v> forms in that book.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'saved-compiler-enabled (<see topic=\"ACL2_____04\">@</see> compiler-enabled) state)\u000A         (<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> nil state)\u000A         (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A :check-expansion t)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;my-book&quot;)\u000A; optional\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____SET-COMPILER-ENABLED\">set-compiler-enabled</see> (<see topic=\"ACL2_____04\">@</see> saved-compiler-enabled) state)\u000A         (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A :check-expansion t)</code> \u000A \u000A <p>Upon completion of an invocation of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, the value of <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable \u000A <v>'compiler-enabled</v> is restored to the value it had immediately before that \u000A invocation.</p> \u000A \u000A <p>See <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see> for more discussion about compilation and \u000A <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p>We close with a discussion of a feature that allows control over the \u000A loading of <v>.port</v> files in close analogy to how loading of compiled files \u000A is controlled by <v>set-compiler-enabled</v>, as described above. \u000A (See <see topic=\"ACL2____UNCERTIFIED-BOOKS\">uncertified-books</see> for a discussion of <v>.port</v> files.)  A <see topic=\"ACL2____STATE\">state</see> global variable, <v>'port-file-enabled</v> exists for this purpose, and \u000A it is set as follows.</p> \u000A \u000A <code>(set-port-file-enabled t state)   ; permit loading of .port files (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(set-port-file-enabled nil state) ; skip loading of .port files</code> \u000A \u000A <p>Just as described above for state global <v>compiler-enabled</v>, the value \u000A of <v>'port-file-enabled</v> persists after <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion and is \u000A restored after <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  The idiom \u000A displayed above, for avoiding loading of compiled files, can be modified or \u000A extended in the obvious way to avoid loading of <v>.port</v> files.</p>"],

 "ACL2____BACKCHAIN-LIMIT":[["Rewrite","Meta","Linear","Type-prescription"],"ACL2 Sources","ACL2","<p>Before ACL2 can apply a rule with hypotheses, it must establish \u000A that the hypotheses are true.  (We ignore the relaxing of this requirement \u000A afforded by <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt>s and <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses.)  ACL2 \u000A typically establishes each hypothesis by backchaining &mdash; instantiating \u000A the hypothesis and then rewriting it recursively.  Here we describe how ACL2 \u000A allows the user to limit backchaining.  At the end, below, we describe the \u000A function <tt><see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see></tt>.</p> \u000A \u000A <p>Each hypothesis of a <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <tt><see topic=\"ACL2____META\">meta</see></tt>, <tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, or \u000A <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule is assigned a backchain-limit when the rule is \u000A stored.  By default, this limit is <v>nil</v>, denoting infinity (no limit). \u000A However, the value used for the default may be set to a non-negative integer \u000A (or to <v>nil</v>) by the user; see <see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see>.  The \u000A default is overridden when a <v>:backchain-limit-lst</v> is supplied explicitly \u000A with the rule; see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.  The number of recursive applications \u000A of backchaining starting with the hypothesis of a rule is limited to the \u000A backchain-limit associated with that hypothesis.</p> \u000A \u000A <p>Moreover, the user may set global backchain-limits that limit the total \u000A backchaining depth.  See <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>.  One limit is for the use \u000A of <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <tt><see topic=\"ACL2____META\">meta</see></tt>, and <tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules, while the other \u000A limit is for so-called ``<see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning'', which uses rules of \u000A class <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.  The two limits operate independently. \u000A Below, we discuss the first kind of backchain limits, i.e., for other than \u000A <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules, except as otherwise indicated; but the \u000A mechanism for those rules is similar.</p> \u000A \u000A <p>Below we lay out the precise sense in which a global backchain-limit \u000A interacts with the backchain-limits of individual rules in order to limit \u000A backchaining.  But first we note that when further backchaining is disallowed, \u000A ACL2 can still prove a hypothesis in a given context by using that contextual \u000A information.  In fact, <see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning may be used (except that a \u000A weaker version of it is used in the second case above, i.e., where we are \u000A already doing type-set reasoning).  Thus, the relieving of hypotheses may be \u000A limited to the use of contextual information (without backchaining, i.e., \u000A without recursively rewriting hypotheses) by executing <v>:set-backchain-limit\u000A 0</v>.</p> \u000A \u000A <p>Recall that there are two sorts of backchain limits: those applied to \u000A hypotheses of individual rules, as assigned by their <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> or else taken from the default (see <see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see>); and the global limit, initially <v>nil</v> (no \u000A limit) but settable with <v>:</v><tt><see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see></tt>.  Here is how these \u000A two types of limits interact to limit backchaining, i.e., recursive rewriting \u000A of hypotheses.  ACL2 maintains a current backchain limit, which is the limit \u000A on the depth of recursive calls to the rewriter, as well as a current \u000A backchain depth, which is initially 0 and is incremented each time ACL2 \u000A backchains (and is decremented when a backchain completes).  When ACL2 begins \u000A to rewrite a literal (crudely, one of the ``top-level'' terms of the goal \u000A currently being worked on), it sets the current backchain-limit to the global \u000A value, which is initially <v>nil</v> but can be set using <v>:</v><tt><see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see></tt>.  When ACL2 is preparing to relieve a hypothesis by \u000A backchaining (hence, after it has already tried type-set reasoning), it first \u000A makes sure that the current backchain limit is greater than the current \u000A backchain depth.  If not, then it refuses to relieve that hypothesis. \u000A Otherwise, it increments the current backchain depth and calculates a new \u000A current backchain-limit by taking the minimum of two values: the existing \u000A current backchain-limit, and the sum of the current backchain depth and the \u000A backchain-limit associated with the hypothesis.  Thus, ACL2 only modifies the \u000A current backchain-limit if it is necessary to decrease that limit in order to \u000A respect the backchain limit associated with the hypothesis.</p> \u000A \u000A <p>We illustrate with the following examples.</p> \u000A \u000A <code>; We stub out some functions so that we can reason about them.\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p0 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p1 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p2 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p3 (x) t)\u000A\u000A; Initially, the default-backchain-limit is nil, or infinite.\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-implies-p1-limitless\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x)\u000A           (p1 x)))\u000A\u000A; The following rule will have a backchain-limit of 0.\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p1-implies-p0-limit-0\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p1 x)\u000A           (p0 x))\u000A  :rule-classes ((:rewrite :backchain-limit-lst 0)))\u000A\u000A; We have (p2 x) ==&gt; (p1 x) ==&gt; (p0 x).  We wish to establish that\u000A; (p2 x) ==&gt; (p0 x).  Normally, this would be no problem, but here\u000A; we fail because ACL2 cannot establish (p0 x) by type-set reasoning\u000A; alone.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x)\u000A           (p0 x)))\u000A\u000A; We set the default-backchain-limit (for rewriting) to 1.\u000A\u000A:set-default-backchain-limit 1\u000A\u000A; The following is more powerful than p1-implies-p0-limit-0\u000A; because it can use rewrite rules to establish (p1 x).\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p1-implies-p0-limit-1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p1 x)\u000A           (p0 x)))\u000A\u000A; This theorem will succeed:\u000A\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 x)\u000A           (p0 x)))\u000A\u000A; We return the default-backchain-limit to its initial value.\u000A\u000A:set-default-backchain-limit nil\u000A\u000A; Here is our last axiom.\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p3-implies-p2-limitless\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p3 x)\u000A           (p2 x)))\u000A\u000A; We now have (p3 x) ==&gt; (p2 x) ==&gt; (p1 x) ==&gt; (p0 x).  However the\u000A; rule p1-implies-p0-limit-1 has a backchain-limit of 1; hence we\u000A; are not allowed to backchain far enough back to use\u000A; p3-implies-p2-limitless.  We therefore lose.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> will-fail\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (p3 x)\u000A           (p0 x)))</code> \u000A \u000A <p>Finally, we remark that to see the current global backchain-limits, issue \u000A the following commands.</p> \u000A \u000A <code>(<see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> wrld :ts) ; backchain limit for type-set reasoning\u000A(<see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> wrld :rewrite) ; backchain limit for rewriting</code>"],

 "ACL2____WITH-OUTPUT":[["Prover-output"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A; Turn off all output during evaluation of the indicated thm form.\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A :off :all\u000A :gag-mode nil\u000A (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000A; Prove the indicated theorem with the event summary turned off and\u000A; using the :goals setting for gag-mode.\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A   :off summary\u000A   :gag-mode :goals\u000A   (<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000A; Same effect as just above:\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A   :on summary\u000A   :summary nil\u000A   :gag-mode :goals\u000A   (<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000A; Turn on only the indicated parts of the summary.\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A   :on summary\u000A   :summary (time rules)\u000A   :gag-mode :goals  ; use gag-mode, with goal names printed\u000A   (<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000A; Same as specifying :off :all, but showing all output types\u000A; (i.e., the value of constant *valid-output-names*):\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A :off (error warning warning! observation prove proof-checker event history\u000A             summary proof-tree)\u000A :gag-mode nil\u000A (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000A; Same as above, but :stack :push says to save the current\u000A; inhibit-output-lst, which can be restored in a subsidiary with-output call\u000A; that specifies :stack :pop.\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A :stack :push\u000A :off :all\u000A :gag-mode nil\u000A (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app x y) z) (app x (app y z)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> :key1 val1 ... :keyk valk form)</code> \u000A \u000A <p>where each <v>:keyi</v> is either <v>:off</v>, <v>:on</v>, <v>:stack</v>, \u000A <v>:summary</v>, or <v>:gag-mode</v>; <v>form</v> evaluates to an error triple (see \u000A <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>); and <v>vali</v> is as follows.  If <v>:keyi</v> is \u000A <v>:off</v> or <v>:on</v>, then <v>vali</v> can be <v>:all</v>, and otherwise is a \u000A symbol or non-empty list of symbols representing output types that can be \u000A inhibited; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.  If <v>:keyi</v> is \u000A <v>:gag-mode</v>, then <v>vali</v> is one of the legal values for <v>:</v><tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt>.  If <v>:keyi</v> is <v>:summary</v>, then <v>vali</v> is either \u000A <v>:all</v> or a true-list of symbols each of which belongs to the list \u000A <v>*summary-types*</v>.  Otherwise <v>:keyi</v> is <v>:stack</v>, in which case \u000A <v>:vali</v> is <v>:push</v> or <v>:pop</v>; for now assume that <v>:stack</v> is not \u000A specified (we'll return to it below).  The result of evaluating the General \u000A Form above is to evaluate <v>form</v>, but in an environment where output occurs \u000A as follows.  If <v>:on :all</v> is specified, then every output type is turned \u000A on except as inhibited by <v>:off</v>; else if <v>:off :all</v> is specified, then \u000A every output type is inhibited except as specified by <v>:on</v>; and otherwise, \u000A the currently-inhibited output types are reduced as specified by <v>:on</v> and \u000A then extended as specified by <v>:off</v>.  But if <v>:gag-mode</v> is specified, \u000A then before modifying how output is inhibited, <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> is set for the \u000A evaluation of <v>form</v> as specified by the value of <v>:gag-mode</v>; see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>.  If <v>summary</v> is among the output types that are turned on \u000A (not inhibited), then if <v>:summary</v> is specified, the only parts of the \u000A summary to be printed will be those specified by the value of <v>:summary</v>. \u000A The correspondence should be clear, except perhaps that <v>header</v> refers to \u000A the line containing only the word <v>Summary</v>, and <v>value</v> refers to the \u000A value of the form printed during evaluation of sequences of events as for \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p> \u000A \u000A <p>Note that the handling of the <v>:stack</v> argument pays no attention to the \u000A <v>:summary</v> argument.</p> \u000A \u000A <p>Note: When the scope of <v>with-output</v> is exited, then all modifications \u000A are undone, reverting <v>gag-mode</v> and the state of output inhibition to \u000A those which were present before the <v>with-output</v> call was entered.</p> \u000A \u000A <p>The <v>:stack</v> keyword's effect is illustrated by the following example, \u000A where ``<v>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> nil)</v>'' may replaced by ``<v>(<see topic=\"COMMON-LISP____PROGN\">progn</see></v>'' without any \u000A change to the output that is printed.</p> \u000A \u000A <code>(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A :stack :push :off :all\u000A (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x) x)\u000A   (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> :stack :pop (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x) x))\u000A   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f3 (x) x)\u000A   (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> :stack :pop :off warning (<see topic=\"ACL2____IN-THEORY\">in-theory</see> nil))\u000A   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f4 (x) x)))</code> \u000A \u000A <p>The outer <v>with-output</v> call saves the current output settings (as may \u000A have been modified by earlier calls of <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>), by \u000A pushing them onto a stack, and then turns off all output.  Each inner \u000A <v>with-output</v> call temporarily pops that stack, restoring the starting \u000A output settings, until it completes and undoes the effects of that pop. \u000A Unless <v>event</v> output was inhibited at the top level (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), the following output is shown:</p> \u000A \u000A <code>Since F2 is non-recursive, its admission is trivial.  We observe that\u000Athe type of F2 is described by the theorem (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (F2 X) X).</code> \u000A \u000A <p>And then, if <v>summary</v> output was not inhibited at the top level, we get \u000A the rest of this output:</p> \u000A \u000A <code>Summary\u000AForm:  ( DEFUN F2 ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000ASummary\u000AForm:  ( IN-THEORY NIL)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)</code> \u000A \u000A <p>Note that the use of <v>:off warning</v> supresses a <v>&quot;Theory&quot;</v> warning \u000A for the <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> nil)</v> event, and that in no case will output be printed \u000A for definitions of <v>f1</v>, <v>f3</v>, or <v>f4</v>, or for the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event itself.</p> \u000A \u000A <p>The following more detailed explanation of <v>:stack</v> is intended only for \u000A advanced users.  After <v>:gag-mode</v> is handled (if present) but before \u000A <v>:on</v> or <v>:off</v> is handled, the value of <v>:stack</v> is handled as \u000A follows.  If the value is <v>:push</v>, then <see topic=\"ACL2____STATE\">state</see> global \u000A <v>inhibit-output-lst-stack</v> is modified by pushing the value of <see topic=\"ACL2____STATE\">state</see> \u000A global <v>inhibit-output-lst</v> onto the value of <see topic=\"ACL2____STATE\">state</see> global \u000A <v>inhibit-output-lst-stack</v>, which is <v>nil</v> at the top level.  If the \u000A value is <v>:pop</v>, then <see topic=\"ACL2____STATE\">state</see> global <v>inhibit-output-lst-stack</v> is \u000A modified only if non-<v>nil</v>, in which case its top element is popped and \u000A becomes the value of of <see topic=\"ACL2____STATE\">state</see> global <v>inhibit-output-lst</v>.</p> \u000A \u000A <p>Warning: <v>With-output</v> has no effect in raw Lisp, and hence is \u000A disallowed in function bodies.  However, you can probably get the effect you \u000A want as illustrated below, where <v>&lt;form&gt;</v> must return an error triple \u000A <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>; see <see topic=\"ACL2____LD\">ld</see> and see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.</p> \u000A \u000A <code>Examples avoiding with-output, for use in function definitions:\u000A\u000A; Inhibit all output:\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((inhibit-output-lst *valid-output-names*))\u000A &lt;form&gt;)\u000A\u000A; Inhibit all warning output:\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((inhibit-output-lst\u000A   (<see topic=\"ACL2____UNION-EQ\">union-eq</see> (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'inhibit-output-lst state)\u000A             '(warning warning!))))\u000A &lt;form&gt;)</code> \u000A \u000A <p>Note that <v>with-output</v> is allowed in books.  See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.</p>"],

 "ACL2____O_C3":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>If <v>x</v> and <v>y</v> are both <v>o-p</v>s (see <see topic=\"ACL2____O-P\">o-p</see>) then \u000A <v>(<see topic=\"ACL2____O_C3\">o&lt;</see> x y)</v> is true iff <v>x</v> is strictly less than <v>y</v>.  <v>o&lt;</v> is \u000A well-founded on the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s.  When <v>x</v> and <v>y</v> are both nonnegative \u000A integers, <v>o&lt;</v> is just the familiar ``less than'' relation (<tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>).</p> \u000A \u000A <p><v>o&lt;</v> plays a key role in the formal underpinnings of the ACL2 logic.  In \u000A order for a recursive definition to be admissible it must be proved to \u000A ``terminate.''  By terminate we mean that the arguments to the function ``get \u000A smaller'' as the function recurses and this sense of size comparison must be \u000A such that there is no ``infinitely descending'' sequence of ever smaller \u000A arguments.  That is, the relation used to compare successive arguments must be \u000A well-founded on the domain being measured.</p> \u000A \u000A <p>The most basic way ACL2 provides to prove termination requires the user to \u000A supply (perhaps implicitly) a mapping of the argument tuples into the ordinals \u000A with some ``measure'' expression in such a way that the measures of the \u000A successive argument tuples produced by recursion decrease according to the \u000A relation <v>o&lt;</v>.  The validity of this method rests on the well-foundedness \u000A of <v>o&lt;</v> on the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s.</p> \u000A \u000A <p>Without loss of generality, suppose the definition in question introduces \u000A the function <v>f</v>, with one formal parameter <v>x</v> (which might be a list \u000A of objects).  Then we require that there exist a measure expression, <v>(m\u000A x)</v>, that always produces an <tt><see topic=\"ACL2____O-P\">o-p</see></tt>.  Furthermore, consider any \u000A recursive call, <v>(f (d x))</v>, in the body of the definition.  Let <v>hyps</v> \u000A be the conjunction of terms, each of which is either the test of an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> \u000A in the body or else the negation of such a test, describing the path through \u000A the body to the recursive call in question.  Then it must be a theorem \u000A that</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyps (<see topic=\"ACL2____O_C3\">O&lt;</see> (m (d x)) (m x))).</code> \u000A \u000A <p>When we say <v>o&lt;</v> is ``well-founded'' on the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s we mean that \u000A there is no infinite sequence of <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s such that each is smaller than \u000A its predecessor in the sequence.  Thus, the theorems that must be proved about \u000A <v>f</v> when it is introduced establish that it cannot recur forever because \u000A each time a recursive call is taken <v>(m x)</v> gets smaller.  From this, and \u000A the syntactic restrictions on definitions, it can be shown (as on page 44 in \u000A ``A Computational Logic'', Boyer and Moore, Academic Press, 1979) that there \u000A exists a function satisfying the definition; intuitively, the value assigned \u000A to any given <v>x</v> by the alleged function is that computed by a sufficiently \u000A large machine.  Hence, the logic is consistent if the axiom defining <v>f</v> is \u000A added.</p> \u000A \u000A <p>See <see topic=\"ACL2____O-P\">o-p</see> for a discussion of the ordinals and how to compare two \u000A ordinals.</p> \u000A \u000A <p>The definitional principle permits the use of relations other than <v>o&lt;</v> \u000A but they must first be proved to be well-founded on some domain.  See <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.  Roughly put, alternative relations are shown \u000A well-founded by providing an order-preserving mapping from their domain into \u000A the ordinals.  See <see topic=\"COMMON-LISP____DEFUN\">defun</see> for details on how to specify which \u000A well-founded relation is to be used.</p> \u000A \u000A <p><b>Function: </b><srclink>o&lt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o&lt; (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (o&lt;g x) (o&lt;g y))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____O-FINP\">o-finp</see> x) (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____O-INFP\">o-infp</see> y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)))\u000A             ((<see topic=\"ACL2____O-FINP\">o-finp</see> y) nil)\u000A             ((<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x)\u000A                          (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> y)))\u000A              (<see topic=\"ACL2____O_C3\">o&lt;</see> (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x) (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> y)))\u000A             ((<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see> x) (<see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see> y)))\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see> x) (<see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see> y)))\u000A             (t (<see topic=\"ACL2____O_C3\">o&lt;</see> (<see topic=\"ACL2____O-RST\">o-rst</see> x) (<see topic=\"ACL2____O-RST\">o-rst</see> y)))))</code>"],

 "ACL2____NON-LINEAR-ARITHMETIC":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>This documentation topic is divided into two parts.  We first \u000A discuss the practical aspect of how to use the non-linear arithmetic extension \u000A to ACL2, and then the theory behind it.  We assume that the reader is familiar \u000A with the material in <tt><see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see></tt> and that on <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules.</p> \u000A \u000A <p>We begin our discussion of how to use non-linear arithmetic with a simple \u000A example.  Assume that we wish to prove:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A               (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)\u000A               (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> z)\u000A               (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 y)\u000A               (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> y z)))\u000A          (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> x y) z)))</code> \u000A \u000A <p>Note that <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see> x y) &lt;= (<see topic=\"COMMON-LISP_____F2\">/</see> x y)</v>.  Note also that if we divide both \u000A sides of <v>x &lt; (<see topic=\"COMMON-LISP_____A2\">*</see> y z)</v> by <v>y</v>, since <v>0 &lt; y</v>, we obtain <v>(<see topic=\"COMMON-LISP_____F2\">/</see> x y) &lt;\u000A z</v>.  By chaining these two inequalities together, we get the inequality we \u000A desired to prove.</p> \u000A \u000A <p>We now proceed with our example session:</p> \u000A \u000A <code>(skip-proofs \u000A (progn \u000A \u000A; Since the truth of this theorem depends on the linear properties \u000A; of floor, we will need the linear lemma: \u000A \u000A   (defthm floor-bounds-1 \u000A       (implies (and (rationalp x) \u000A                     (rationalp y)) \u000A                (and (&lt; (+ (/ x y) -1) \u000A                        (floor x y)) \u000A                     (&lt;= (floor x y) \u000A                         (/ x y)))) \u000A       :rule-classes ((:linear :trigger-terms ((floor x y))))) \u000A \u000A; We now disable floor, so that the linear lemma will be used. \u000A \u000A   (in-theory (disable floor)) \u000A \u000A; We create five rewrite rules which we will use during non-linear \u000A; arithmetic.  The necessity for these is due to one of the differences in \u000A; ACL2's behaviour when non-linear arithmetic is turned on.  Although \u000A; the conclusions of linear lemmas have always been rewritten before \u000A; they are used, now, when non-linear arithmetic is turned on, the \u000A; conclusions are rewritten under a different theory than under ``normal'' \u000A; rewriting.  This theory is also used in other, similar, circumstances \u000A; described below. \u000A \u000A   (defthm |arith (* -1 x)| \u000A       (equal (* -1 x) \u000A              (- x))) \u000A \u000A   (defthm |arith (* 1 x)| \u000A       (equal (* 1 x) \u000A              (fix x))) \u000A \u000A   (defthm |arith (* x (/ x) y)| \u000A       (equal (* x (/ x) y) \u000A              (if (equal (fix x) 0) \u000A                  0 \u000A                  (fix y)))) \u000A \u000A   (defthm |arith (* y x)| \u000A       (equal (* y x) \u000A              (* x y))) \u000A \u000A   (defthm |arith (fix x)| \u000A       (implies (acl2-numberp x) \u000A                (equal (fix x) \u000A                       x)))) \u000A )  ; End skip-proofs. \u000A \u000A; We disable the above rewrite rules from normal use. \u000A \u000A(in-theory (disable |arith (* -1 x)| \u000A                    |arith (* 1 x)| \u000A                    |arith (* x (/ x) y)| \u000A                    |arith (* y x)| \u000A                    |arith (fix x)|)) \u000A \u000A; We create an arithmetic-theory.  Note that we must give a quoted \u000A; constant for the theory &mdash; none of the normal <tt><see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see></tt> \u000A; are applicable to in-arithmetic-theory. \u000A \u000A(in-arithmetic-theory '(|arith (* -1 x)| \u000A                        |arith (* 1 x)| \u000A                        |arith (* x (/ x) y)| \u000A                        |arith (* y x)| \u000A                        |arith (fix x)|)) \u000A \u000A; We turn non-linear arithmetic on. \u000A \u000A(set-non-linearp t) \u000A \u000A; We can now go ahead and prove our theorem. \u000A \u000A(thm \u000A (implies (and (rationalp x) \u000A               (rationalp y) \u000A               (rationalp z) \u000A               (&lt; 0 y) \u000A               (&lt; x (* y z))) \u000A          (&lt; (floor x y) z))) \u000A</code> \u000A \u000A <p>The above example illustrates the two practical requirements for using \u000A non-linear arithmetic in ACL2.  First, one must set up an arithmetic-theory. \u000A Usually, one would not set up an arithmetic-theory on one's own but would \u000A instead load a library book or books which do so.  Second, one must turn the \u000A non-linear arithmetic extension on.  This one must do explicitly &mdash; no \u000A book can do this for you.</p> \u000A \u000A <p>For a brief discussion of why this is so, even though <v>(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see>\u000A t)</v> is an embeddable event, see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see> (in particular, \u000A the final paragraph).  (Note that <v>(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see> t)</v> modifies the \u000A <v>acl2-defaults-table</v>.)  Also see <see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see>, see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>, and see <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>You can also enable non-linear arithmetic with the hint <v>:nonlinearp t</v>. \u000A See <see topic=\"ACL2____HINTS\">hints</see>.  We, in fact, recommend the use of a hint which will enable \u000A nonlinear arithmetic only when the goal has stabilized under rewriting.  Using \u000A <tt><see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see></tt> can make this easier.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nonlinearp-default-hint (stable-under-simplificationp hist pspv)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> (stable-under-simplificationp\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ACCESS\">access</see> rewrite-constant\u000A                          (<see topic=\"ACL2____ACCESS\">access</see> prove-spec-var pspv :rewrite-constant)\u000A                          :nonlinearp))\u000A             '(:computed-hint-replacement t :nonlinearp t)\u000A           nil))\u000A        ((<see topic=\"ACL2____ACCESS\">access</see> rewrite-constant\u000A                 (<see topic=\"ACL2____ACCESS\">access</see> prove-spec-var pspv :rewrite-constant)\u000A                 :nonlinearp)\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAAR\">caar</see> hist) 'SETTLED-DOWN-CLAUSE))\u000A             '(:computed-hint-replacement t :nonlinearp nil)\u000A           nil))\u000A        (t nil)))\u000A\u000A(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> '((nonlinearp-default-hint stable-under-simplificationp\u000A                                              hist pspv)))</code> \u000A \u000A <p>This has proven to be a helpful strategy which allows faster proof \u000A times.</p> \u000A \u000A <p>We now proceed to briefly describe the theory behind the non-linear \u000A extension to ACL2.  In <tt><see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see></tt> it was stated that, \u000A ``[L]inear polynomial inequalities can be combined by cross-multiplication and \u000A addition to permit the deduction of a third inequality....''  That is, if</p> \u000A \u000A <code>0 &lt; poly1,\u000A0 &lt; poly2,</code> \u000A \u000A <p>and <v>c</v> and <v>d</v> are positive rational constants, then</p> \u000A \u000A <code>0 &lt; c*poly1 + d*poly2.</code> \u000A \u000A <p>Similarly, given the above,</p> \u000A \u000A <code>0 &lt; poly1*poly2.</code> \u000A \u000A <p>In the linear arithmetic case, we are taking advantage of the facts that \u000A multiplication by a positive rational constant does not change the sign of a \u000A polynomial and that the sum of two positive polynomials is itself positive. \u000A In the non-linear arithmetic case, we are using the fact that the product of \u000A two positive polynomials is itself positive.</p> \u000A \u000A <p>For example, suppose we have the three assumptions:</p> \u000A \u000A <code>p1:  3*x*y + 7*a &lt; 4\u000Ap2:            3 &lt; 2*x  or p2': 0 &lt; -3 + 2*x\u000Ap3:            1 &lt; y    or p3': 0 &lt; -1 + y,</code> \u000A \u000A <p>and we wish to prove that <v>a &lt; 0</v>.  As described elsewhere (see <see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see>), we proceed by assuming the negation of our goal:</p> \u000A \u000A <code>p4:            0 &lt;= a,</code> \u000A \u000A <p>and looking for a contradiction.</p> \u000A \u000A <p>There are no cancellations which can be performed by linear arithmetic in \u000A the above situation.  (Recall that two polynomials are cancelled against each \u000A other only when they have the same largest unknown.)  However, <v>p1</v> has a \u000A product as its largest unknown, and for each of the factors of that product \u000A there is a poly that has that factor as a largest unknown.  When non-linear \u000A arithmetic is enabled, ACL2 will therefore multiply <v>p1'</v> and <v>p2'</v> \u000A obtaining</p> \u000A \u000A <code>p5:            0 &lt; 3 + -2*x + -3*y + 2*x*y.</code> \u000A \u000A <p>The addition of this polynomial will allow cancelation to continue and, in \u000A this case, we will prove our goal.  Thus, just as ACL2 adds two polynomials \u000A together when they have the same largest unknown of opposite signs in order to \u000A create a new ``smaller'' polynomial; so ACL2 multiplies polynomials together \u000A when the product of their largest unknowns is itself the largest unknown of \u000A another polynomial.  As the use of <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> lemmas to further seed \u000A the arithmetic database may allow cancellation to proceed, so may the use of \u000A non-linear arithmetic.</p> \u000A \u000A <p>This multiplication of polynomials is the motivation for an \u000A arithmetic-theory distinct from than the normal one.  Because this may be done \u000A so often, and because the individual factors have presumably already been \u000A rewritten, it is important that this be done in an efficient way.  The use of \u000A a small, specialized, theory helps avoid the repeated application of rewrite \u000A rules to already stabilized terms.</p>"],

 "ACL2____MV-LET":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (x y z)              ; local variables\u000A        (<see topic=\"ACL2____MV\">mv</see> 1 2 3)           ; multi-valued expression\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> y)) ; optional declarations\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> x z))          ; body</code> \u000A \u000A <p>The form above binds the three ``local variables,'' <v>x</v>, <v>y</v>, and \u000A <v>z</v>, to the three results returned by the multi-valued expression and then \u000A evaluates the body.  The result is <v>'(1 . 3)</v>.  The second local, <v>y</v>, \u000A is <see topic=\"COMMON-LISP____DECLARE\">declare</see>d <v>ignore</v>d.  The multi-valued expression can be any ACL2 \u000A expression that returns <v>k</v> results, where <v>k</v> is the number of local \u000A variables listed.  Often however it is simply the application of a \u000A <v>k</v>-valued function.  <v>Mv-let</v> is the standard way to invoke a \u000A multi-valued function when the caller must manipulate the vector of results \u000A returned.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (var1 ... vark)\u000A        term\u000A        body)\u000Aor\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (var1 ... vark)\u000A        term\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ... (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...)\u000A        body)</code> \u000A \u000A <p>where the <v>vari</v> are distinct variables, <v>term</v> is a term that \u000A returns <v>k</v> results and mentions only variables bound in the environment \u000A containing the <v>mv-let</v> expression, and <v>body</v> is a term mentioning only \u000A the <v>vari</v> and variables bound in the environment containing the \u000A <v>mv-let</v>.  Each <v>vari</v> must occur in <v>body</v> unless it is <see topic=\"COMMON-LISP____DECLARE\">declare</see>d <v>ignore</v>d or <v>ignorable</v> in one of the optional <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms, unless this requirement is turned off; see <see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see>.  The value of the <v>mv-let</v> term is the result of evaluating \u000A <v>body</v> in an environment in which the <v>vari</v> are bound, in order, to the \u000A <v>k</v> results obtained by evaluating <v>term</v> in the environment containing \u000A the <v>mv-let</v>.</p> \u000A \u000A <p>Here is an extended example that illustrates both the definition of a \u000A multi-valued function and the use of <v>mv-let</v> to call it.  Consider a \u000A simple binary tree whose interior nodes are <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es and whose leaves \u000A are non-<tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es.  Suppose we often need to know the number, <v>n</v>, of \u000A interior nodes of such a tree; the list, <v>syms</v>, of symbols that occur as \u000A leaves; and the list, <v>ints</v>, of integers that occur as leaves.  (Observe \u000A that there may be leaves that are neither symbols nor integers.)  Using a \u000A multi-valued function we can collect all three results in one pass.</p> \u000A \u000A <p>Here is the first of two definitions of the desired function.  This \u000A definition is ``primitive recursive'' in that it has only one argument and \u000A that argument is reduced in size on every recursion.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> count-and-collect (x)\u000A\u000A; We return three results, (<see topic=\"ACL2____MV\">mv</see> n syms ints) as described above.\u000A\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A\u000A; X is a leaf.  Thus, there are 0 interior nodes, and depending on\u000A; whether x is a symbol, an integer, or something else, we return\u000A; the list containing x in as the appropriate result.\u000A\u000A         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x) (<see topic=\"ACL2____MV\">mv</see> 0 (<see topic=\"COMMON-LISP____LIST\">list</see> x) nil))\u000A               ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)(<see topic=\"ACL2____MV\">mv</see> 0 nil      (<see topic=\"COMMON-LISP____LIST\">list</see> x)))\u000A               (t           (<see topic=\"ACL2____MV\">mv</see> 0 nil      nil))))\u000A        (t\u000A\u000A; X is an interior node.  First we process the car, binding n1, syms1, and\u000A; ints1 to the answers.\u000A\u000A           (<see topic=\"ACL2____MV-LET\">mv-let</see> (n1 syms1 ints1)\u000A                   (count-and-collect (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000A; Next we process the cdr, binding n2, syms2, and ints2.\u000A\u000A                   (<see topic=\"ACL2____MV-LET\">mv-let</see> (n2 syms2 ints2)\u000A                           (count-and-collect (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000A; Finally, we compute the answer for x from those obtained for its car\u000A; and cdr, remembering to increment the node count by one for x itself.\u000A\u000A                           (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> n1 n2))\u000A                               (<see topic=\"COMMON-LISP____APPEND\">append</see> syms1 syms2)\u000A                               (<see topic=\"COMMON-LISP____APPEND\">append</see> ints1 ints2)))))))</code> \u000A \u000A <p>This use of a multiple value to ``do several things at once'' is very \u000A common in ACL2.  However, the function above is inefficient because it <see topic=\"COMMON-LISP____APPEND\">append</see>s <v>syms1</v> to <v>syms2</v> and <v>ints1</v> to <v>ints2</v>, copying the \u000A list structures of <v>syms1</v> and <v>ints1</v> in the process.  By adding \u000A ``accumulators'' to the function, we can make the code more efficient.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> count-and-collect1 (x n syms ints)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x) (<see topic=\"ACL2____MV\">mv</see> n (<see topic=\"COMMON-LISP____CONS\">cons</see> x syms) ints))\u000A               ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"ACL2____MV\">mv</see> n syms (<see topic=\"COMMON-LISP____CONS\">cons</see> x ints)))\u000A               (t (<see topic=\"ACL2____MV\">mv</see> n syms ints))))\u000A        (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (n2 syms2 ints2)\u000A                   (count-and-collect1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____1_B2\">1+</see> n) syms ints)\u000A                   (count-and-collect1 (<see topic=\"COMMON-LISP____CAR\">car</see> x) n2 syms2 ints2)))))</code> \u000A \u000A <p>We claim that <v>(count-and-collect x)</v> returns the same triple of results \u000A as <v>(count-and-collect1 x 0 nil nil)</v>.  The reader is urged to study this \u000A claim until convinced that it is true and that the latter method of computing \u000A the results is more efficient.  One might try proving the theorem</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> count-and-collect-theorem\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (count-and-collect1 x 0 nil nil) (count-and-collect x))).</code> \u000A \u000A <p>Hint: the inductive proof requires attacking a more general theorem.</p> \u000A \u000A <p>ACL2 does not support the Common Lisp construct <v>multiple-value-bind</v>, \u000A whose logical meaning seems difficult to characterize.  <v>Mv-let</v> is the \u000A ACL2 analogue of that construct.  Also see <see topic=\"ACL2____MV\">mv</see> and see <see topic=\"ACL2____MV-LIST\">mv-list</see>.</p>"],

 "ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM":[["Tau-system"],"ACL2 Sources","ACL2","<p>This doc topic is the main source of information about the tau \u000A system and discusses the general idea behind the procedure and how to exploit \u000A it.</p> \u000A \u000A <p><i>A ``Type-Checker'' for an Untyped Language</i></p> \u000A \u000A <p>Because ACL2 is an untyped language it is impossible to type check it.  All \u000A functions are total.  An <i>n</i>-ary function may be applied to any \u000A combination of <i>n</i> ACL2 objects.  The syntax of ACL2 stipulates that \u000A <v>(</v><i>fn a1...an</i><v>)</v> is a well-formed term if <i>fn</i> is a function \u000A symbol of <i>n</i> arguments and the <i>ai</i> are well-formed terms.  No \u000A mention is made of the ``types'' of terms.  That is what is meant by saying \u000A ACL2 is an untyped language.</p> \u000A \u000A <p>Nevertheless, the system provides a variety of monadic Boolean function \u000A symbols, like <tt><see topic=\"ACL2____NATP\">natp</see></tt>, <tt><see topic=\"COMMON-LISP____INTEGERP\">integerp</see></tt>, <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>, etc., that \u000A recognize different ``types'' of objects at runtime.  Users typically define \u000A many more such recognizers for domain-specific ``types.''  Because of the \u000A prevalence of such ``types,'' ACL2 must frequently reason about the inclusion \u000A of one ``type'' in another.  It must also reason about the consequences of \u000A functions being defined so as to produce objects of certain ``types'' when \u000A given arguments of certain other ``types.''</p> \u000A \u000A <p>Because the word ``type'' in computer science tends to imply syntactic or \u000A semantic restrictions on functions, we avoid using that word henceforth. \u000A Instead, we just reason about monadic Boolean predicates.  You may wish to \u000A think of ``tau'' as synonymous with ``type'' but without any suggestion of \u000A syntactic or semantic restrictions.</p> \u000A \u000A <p><i>Design Philosophy</i></p> \u000A \u000A <p>The following basic principles were kept in mind when developing tau \u000A checker and may help you exploit it.</p> \u000A \u000A <p>(1) The tau system is supposed to be a lightweight, fast, and helpful \u000A decision procedure for an elementary subset of the logic focused on monadic \u000A predicates and function signatures.</p> \u000A \u000A <p>(2) Most subgoals produced by the theorem prover are not in any decidable \u000A subset of the logic!  Thus, decision procedures fail to prove the vast \u000A majority of the formulas they see and will be net time-sinks if tried too \u000A often no matter how fast they are.</p> \u000A \u000A <p>Tau reasoning is used by the prover as part of <v>preprocess-clause</v>, one \u000A of the first proof techniques the system tries.  The tau system filters out \u000A ``obvious'' subgoals.  The tau system is only tried when subgoals first enter \u000A the waterfall and when they are stable under simplification.</p> \u000A \u000A <p>(3) The tau system is ``benign'' in the sense that the only way it \u000A contributes to a proof is to eliminate (prove!) subgoals.  It does not \u000A rewrite, simplify, or change formulas.  Tau reasoning is not used by the \u000A rewriter.  The tau system either eliminates a subgoal by proving it or leaves \u000A it unchanged.</p> \u000A \u000A <p>(4) It is impossible to infer automatically the relations between arbitrary \u000A recursively defined predicates and functions.  Thus, the tau system's \u000A knowledge of tau relationships and function signatures is gleaned from \u000A theorems stated by the user and proved by the system.</p> \u000A \u000A <p>(5) Users wishing to build effective ``type-checkers'' for their models \u000A must learn how rules affect the tau system's behavior.  There are two main \u000A forms of tau rules: those that reveal inclusion/exclusion relations between \u000A named tau predicates, e.g., that 16-bit naturals are also 32-bit naturals,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (n16p x) (n32p x)),</code> \u000A \u000A <p>and signatures for all relevant functions, e.g., writing a 32-bit natural \u000A to a legal slot in a register file produces a register file:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n 16)\u000A              (n32p val)\u000A              (register-filep regs))\u000A         (register-filep (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> n val regs))).</code> \u000A \u000A <p>For a complete description of acceptable forms see <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>.</p> \u000A \u000A <p>(6) The tau system is ``greedy'' in its efforts to augment its database. \u000A Its database is potentially augmented when rules of <i>any</i> \u000A <v>:rule-class</v> (see <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>) are proved.  For example, if \u000A you make a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule which \u000A expresses a relationship between one tau and another (e.g., that <v>(P x)</v> \u000A implies <v>(<see topic=\"ACL2____Q\">Q</see> x)</v>), ACL2 will build it into the tau database.  The rule-class \u000A <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> can be used to add a rule to the tau database without \u000A adding any other kind of rule.</p> \u000A \u000A <p>(7) Greediness is forced into the design by benignity: the tau system may \u000A ``know'' some fact that the rewriter does not, and because tau reasoning is \u000A not used in rewriting, that missing fact must be conveyed to the rewriter \u000A through some other class of rule, e.g., a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> or <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rule.  By \u000A making the tau system greedy, we allow the user to program the rewriter and \u000A the tau system simultaneously while keeping them separate.  However, this \u000A means you must keep in mind the effects of a rule on <i>both</i> the rewriter \u000A and the tau system and use <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> rules explicitly when you \u000A want to ``talk'' just to the tau system.</p> \u000A \u000A <p>(8) Tau rules are built into the database with as much preprocessing as \u000A possible (e.g., the system transitively closes inclusion/exclusion \u000A relationships at rule-storage time) so the checker can be fast.</p> \u000A \u000A <p>(9) For speed, tau does not track dependencies and is not sensitive to the \u000A enabled/disabled status (see <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt>) of rules, \u000A other than <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> rules.  Once a fact has been built \u000A into the tau database, the only way to prevent that fact from being used is by \u000A disabling the entire tau system, by disabling <v>(:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt><v> tau-system)</v>.  If any tau reasoning is used in a \u000A proof, the rune <v>(:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt><v> tau-system)</v> is \u000A reported in the summary.  For a complete list of all the runes in the tau \u000A database, evaluate <v>(global-val 'tau-runes (w state))</v>.  Any of these \u000A associated theorems could have been used.</p> \u000A \u000A <p>These design criteria are not always achieved!  For example, the tau \u000A system's ``greediness'' can be turned off (see <tt><see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see></tt>), the \u000A tau database can be regenerated from scratch to ignore disabled rules (see \u000A <tt><see topic=\"ACL2____REGENERATE-TAU-DATABASE\">regenerate-tau-database</see></tt>), and disabling the <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> of a tau predicate symbol will prevent the tau system \u000A from trying to run the predicate on constants.  The tau system's benignity can \u000A be frustrating since it might ``know'' something the rewriter does not.  More \u000A problematically, the tau system is not always ``fast'' and not always \u000A ``benign!''  The typical way tau reasoning can slow a proof down is by \u000A evaulating expensive tau predicates on constants.  The typical way tau \u000A reasoning can hurt a previously successful proof is by proving some subgoals \u000A (!) and thus causing the remaining subgoals to have different <see topic=\"ACL2____CLAUSE-IDENTIFIER\">clause-identifier</see>s, thus making explicit hints no longer applicable.  We deal \u000A with such problems in <tt><see topic=\"ACL2____DEALING-WITH-TAU-PROBLEMS\">dealing-with-tau-problems</see></tt>.</p> \u000A \u000A <p><i>Technical Details</i></p> \u000A \u000A <p>The tau system consists of both a database and an algorithm for using the \u000A database.  The database contains theorems that match certain schemas allowing \u000A them to be stored in the tau database.  Roughly speaking the schemas encode \u000A ``inclusion'' and ``exclusion'' relations, e.g., that <v>natp</v> implies \u000A <v>integerp</v> and that <v>integerp</v> implies not <v>consp</v>, and they encode \u000A ``signatures'' of functions, e.g., theorems that relate the output of a \u000A function to the input, provided only tau predicates are involved.</p> \u000A \u000A <p>By ``tau predicates'' we mean the application of a monadic Boolean-valued \u000A function symbol, the equality of something to a quoted constant, an arithmetic \u000A ordering relation between something and a rational constant, or the logical \u000A negation of such a term.  Here are some examples of tau predicates:</p> \u000A \u000A <code>(<see topic=\"ACL2____NATP\">natp</see> i)\u000A(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))\u000A(<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 'MONDAY)\u000A(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> 23 k))\u000A(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 8 max)\u000A(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> max 24)</code> \u000A \u000A <p>Synonyms for <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> include <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>, <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, and <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>. \u000A Note that negated equalites are also allowed.  The arithmetic ordering \u000A relations that may be used are <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>, <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>, <tt><see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see></tt>, and <tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt>.  One of the arguments to every arithmetic ordering relation must be an \u000A integer or rational constant for the term to be treated as a tau \u000A predicate.</p> \u000A \u000A <p>A ``tau'' is a data object representing a set of signed (positive or \u000A negative) tau predicates whose meaning is the conjunction of the literals in \u000A the set.</p> \u000A \u000A <p>When we say that a term ``has'' a given tau we mean the term satisfies all \u000A of the recognizers in that tau.</p> \u000A \u000A <p>The tau algorithm is a decision procedure for the logical theory described \u000A (only) by the rules in the database.  The algorithm takes a term and a list of \u000A assumptions mapping subterms (typically variable symbols) to tau, and returns \u000A the tau of the given term.</p> \u000A \u000A <p>When the system is called upon to decide whether a term satisfies a given \u000A monadic predicate, it computes the tau of the term and asks whether the \u000A predicate is in that set.  More generally, to determine if a term satisfies a \u000A tau, <i>s</i>, we compute a tau, <i>r</i>, for the term and ask whether \u000A <i>s</i> is a subset of <i>r</i>.  To determine whether a constant, <i>c</i>, \u000A satisfies tau <i>s</i> we apply each of the literals in <i>s</i> to <i>c</i>. \u000A Evaluation might, of course, be time-consuming for complex user-defined \u000A predicates.</p> \u000A \u000A <p>The tau database contains rules derived from definitions and theorems \u000A stated by the user.  See <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> for a description of the \u000A acceptable forms of tau rules.</p> \u000A \u000A <p>To shut off the greedy augmentation of the tau database, see <see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see>.  This may be of use to users who wish to tightly control \u000A the rules in the tau database.  To add a rule to the tau database without \u000A adding any other kind of rule, use the rule class <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>.</p> \u000A \u000A <p>There are some slight complexities in the design related to how we handle \u000A events with both <v>:tau-system</v> corollaries and corollaries of other \u000A <v>:rule-classes</v>, see <tt><see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see></tt>.</p> \u000A \u000A <p>To prevent tau reasoning from being used, disable the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <v>tau-system</v>, i.e., execute</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart tau-system)))</code> \u000A \u000A <p>or, equivalently,</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>)))</code> \u000A \u000A <p>To prevent tau from being used in the proof of a particular subgoal, \u000A locally disable the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <v>tau-system</v> \u000A with a local <v>:in-theory</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>).</p> \u000A \u000A <p>The event command <tt><see topic=\"ACL2____TAU-STATUS\">tau-status</see></tt> is a macro that can be used to toggle \u000A both whether tau reasoning is globally enabled and whether the tau database is \u000A augmented greedily.  For example, the event</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-STATUS\">tau-status</see> :system nil :auto-mode nil)</code> \u000A \u000A <p>prevents the tau system from being used in proofs and prevents the \u000A augmentation of the tau database by rules other than those explicitly labeled \u000A <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>.</p> \u000A \u000A <p>To see what the tau system ``knows'' about a given function symbol see \u000A <see topic=\"ACL2____TAU-DATA\">tau-data</see>.  To see the entire tau database, see <see topic=\"ACL2____TAU-DATABASE\">tau-database</see>.  To \u000A regenerate the tau database using only the runes listed in the current enabled \u000A theory, see <see topic=\"ACL2____REGENERATE-TAU-DATABASE\">regenerate-tau-database</see>.</p>"],

 "ACL2____DMR":[["Debugging","Break-rewrite","Accumulated-persistence"],"ACL2 Sources","ACL2","<p>In addition to utilities that allow you to set breakpoints or print \u000A rewriting information to the screen &mdash; see <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> &mdash; \u000A ACL2 provides a utility for watching the activity of the rewriter and some \u000A other proof processes, in real time.  This utility is called ``dmr'', which is \u000A an acronym for ``dynamically monitor rewrites''.  The utility comes in two \u000A parts: an ACL2 component that frequently updates a file (the ``dmr file'') \u000A containing the relevant information, and an Emacs component that frequently \u000A updates an Emacs buffer (the ``dmr buffer'') with the contents of that \u000A file.  Other editors could, in principle, be programmed to display that file; \u000A anyone developing such a capability is invited to contribute it to the ACL2 \u000A community.</p> \u000A \u000A <p>The dmr utility can be extremely helpful for expensive proofs, especially \u000A when ACL2 is not providing any output to the terminal.  The <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> and <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> utilities may be a bit \u000A easier to use, so you might want to try those first.  But the dmr utility can \u000A be a very helpful debugging aide, as it can visually give you a sense of where \u000A ACL2 is spending its time.</p> \u000A \u000A <p>The Emacs portion of this utility is already loaded if you load the \u000A distributed Emacs file <v>emacs/emacs-acl2.el</v>.  Otherwise, invoke the \u000A following Emacs command, say by typing <v>Control-X Control-E</v> after the \u000A right parenthesis, where <v>DIR</v> is the directory of your ACL2 \u000A distribution.</p> \u000A \u000A <code>(load &quot;&lt;DIR&gt;/emacs/monitor.el&quot;) ; absolute pathnames might work best</code> \u000A \u000A <p>You only need to do that once.  Then each time you want to observe the \u000A rewriter in action, invoke the following to see it displayed in a buffer, \u000A which we call the ``dmr buffer'':</p> \u000A \u000A <code>Control-t 1</code> \u000A \u000A <p>But first you will need to enable monitoring at the ACL2 level:</p> \u000A \u000A <code>(dmr-start)</code> \u000A \u000A <p>Monitoring has some cost.  So if you have started it, then at some point \u000A you may want to turn it off when not using it.  Any time the dmr buffer \u000A (generally called <v>&quot;acl2-dmr-&lt;user_name&gt;&quot;</v>) is not visible, Emacs \u000A monitoring is turned off.  You can also turn off Emacs monitoring explicitly, \u000A with:</p> \u000A \u000A <code>Control-t 2</code> \u000A \u000A <p>At the ACL2 level you can disable monitoring as follows:</p> \u000A \u000A <code>(dmr-stop)</code> \u000A \u000A <p><i>Interpreting the dmr buffer display.</i></p> \u000A \u000A <p>We explain the dmr buffer display by way of the following example.  It is a \u000A snapshot of a dmr buffer taken from one of the community books, \u000A <v>books/workshops/2004/legato/support/proof-by-generalization-mult.lisp</v>.</p> \u000A \u000A <code> 0. (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> . WP-ZCOEF-G-MULTIPLIES)\u000A 1. SIMPLIFY-CLAUSE\u000A   2. Rewriting (to simplify) the atom of literal 18; argument(s) 1\u000A   4. Rewriting (to simplify) the expansion; argument(s) 3|2\u000A   7. Applying (:DEFINITION WP-ZCOEF-G)\u000A*  8. Rewriting (to simplify) the rewritten body; argument(s) 2|1|2|2\u000A*  13. Applying (:REWRITE MOD-ZERO . 2)\u000A*    14. Rewriting (to establish) the atom of hypothesis 4\u000A*    15. Applying (:META META-INTEGERP-CORRECT)</code> \u000A \u000A <p>Each line indicates an ACL2 activity that leads to the activity shown on \u000A the line just below it.  Moreover, lines are sometimes collapsed to make the \u000A display more compact.  Consider for example the first few lines.  Above, we \u000A are proving a theorem named <v>WP-ZCOEF-G-MULTIPLIES</v>.  Lines 1 and 2 show \u000A the clause simplification process invokes the rewriter on the 18th literal. \u000A (Recall that a clause is a disjunction of literals; for example the clause \u000A <v>{(<see topic=\"COMMON-LISP____NOT\">NOT</see> A), (<see topic=\"COMMON-LISP____NOT\">NOT</see> B), C}</v> would be displayed as <v>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> A B) C)</v>.) \u000A This 18th literal mentioned on line 2 is a function call <v>(f arg1 ...)</v>, \u000A and ``<v>argument(s) 1</v>'' indicates that the rewriter, which works \u000A inside-out, is considering the first argument (``<v>arg1</v>'').  Thus the \u000A display could instead have shown the following.</p> \u000A \u000A <code>2. Rewriting (to simplify) the atom of literal 18\u000A3. Rewriting (to simplify) the first argument\u000A4. Rewriting (to simplify) the expansion; argument(s) 3|2</code> \u000A \u000A <p>But it saved space to combine lines 2 and 3.  Line 4 suggests that the \u000A above <v>arg1</v> is a function call that has been opened up because of an \u000A <v>:expand</v> hint or, perhaps, an expansion directed explicitly by the prover \u000A (as can happen during induction).  The annotation ``<v>argument(s) 3|2</v>'' \u000A then indicates that the rewriter is diving into the third argument of the \u000A expansion, and then into the second argument of that.  Let us call the result \u000A <v>term7</v> (since it is the one to be considered on line 7).</p> \u000A \u000A <p>Now consider the next two lines:</p> \u000A \u000A <code>   7. Applying (:DEFINITION WP-ZCOEF-G)\u000A*  8. Rewriting (to simplify) the rewritten body; argument(s) 2|1|2|2</code> \u000A \u000A <p>Line 7 is saying that <v>term7</v> (defined above) is modified by applying \u000A the definition of <v>WP-ZCOEF-G</v> to it.  Line 8 then says that the body of \u000A this definition has been rewritten (with its formals bound to the actuals from \u000A <v>term7</v>) and the rewriter is diving into the subterms of that rewritten \u000A body, as indicated.  Notice also that line 8 is the first line marked with an \u000A asterisk (``<v>*</v>'') in the margin.  This line is the first that is different \u000A from what was shown the previous time the display was updated (about 1/10 \u000A second earlier, by default).  When a line is marked with an asterisk, so are \u000A all the lines below it; so the lines without an asterisk are those that have \u000A been stable since the last display.  In this example we may see line 7 marked \u000A without an asterisk for a while, which suggests that the rule <v>(:DEFINITION\u000A WP-ZCOEF-G)</v> is expensive.  (Also see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>.)  In \u000A general, a line that persists for awhile without a leading asterisk can \u000A suggest why the proof is taking a long time.</p> \u000A \u000A <p>Finally, note the indentation of line 14 relative to line 13.  Extra \u000A indentation occurs when an attempt is being made to relieve a hypothesis \u000A (i.e., rewrite it to <v>t</v>).  In particular, rewrites that will be \u000A incorporated directly into a (top-level) literal are all indented just two \u000A spaces, starting with the first rewrite directly under a process such as \u000A <v>SIMPLIFY-CLAUSE</v> (shown line 1 above).  If the indentation is at least the \u000A value of raw Lisp variable <v>*dmr-indent-max*</v> (by default, 20), then the \u000A indentation is restricted to that column, but ACL2 prints <v>{n}</v> where n is \u000A the column that would have been used for indentation if there were no \u000A maximum.</p> \u000A \u000A <p>You can move the cursor around in the dmr buffer even while it is being \u000A updated.  But emacs will attempt to keep the cursor no later than the first \u000A asterisk (``<v>*</v>'') in the margin.  Thus, you can move the cursor around in \u000A the stable part of the display, and emacs will keep the cursor in that stable \u000A part.</p> \u000A \u000A <p>WARNING: Things could go terribly wrong if the same user runs two different \u000A ACL2 sessions with dmr active, because the same file will be written by two \u000A different ACL2 processes.</p> \u000A \u000A <p>WARNING: For dmr to work, emacs and ACL2 need to be run on the same machine \u000A because the file used to communicate between ACL2 and emacs is under \u000A <v>/tmp</v>.  Except, you can probably hack around that restriction by changing \u000A <v>*dmr-file-name*</v> in ACL2 (in raw Lisp) and correspondingly in Emacs file \u000A <v>monitor.el</v>.</p> \u000A \u000A <p>More generally, advanced users are welcome to search for the string</p> \u000A \u000A <code>User-settable dmr variables</code> \u000A \u000A <p>in ACL2 files <v>interface-raw.lisp</v> and <v>emacs/monitor.el</v> in order to \u000A customize their dmr environments.</p> \u000A \u000A <p>In order to update the dmr file with the latest stack information, \u000A interrupt ACL2 and then evaluate: <v>(dmr-flush)</v>.  In order to support \u000A resumption of the interrupted proof (assuming your host Common Lisp supports \u000A resumption), evaluation of <v>(dmr-start)</v> will automatically enable the \u000A debugger if it is not already enabled and not fully disabled with value \u000A <v>:never</v> (see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>).  If such automatic enabling \u000A takes place, then <v>(dmr-stop)</v> will restore the old setting of the debugger \u000A unless, after <v>(dmr-start)</v> enables the debugger but before <v>(dmr-stop)</v> \u000A is called, you call <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt> (or more precisely: function \u000A <v>set-debugger-enable-fn</v> is called).</p> \u000A \u000A <p>Note for users of the experimental extension ACL2(p) (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>): when waterfall-parallelism has been set to a non-<v>nil</v> value \u000A (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>), statistics about parallel execution \u000A are printed instead of the usual information.</p>"],

 "ACL2____BOOKS-CERTIFICATION":[["Community-books"],"ACL2 Sources","ACL2","<p>Starting in ACL2 6.4 we recommend using the new Community Books \u000A<v>make</v> system to certify the books.  If you encounter problems using the new \u000Asystem (described below) or need some feature that is no longer available, \u000Aplease see <see topic=\"ACL2____BOOKS-CERTIFICATION-ALT\">books-certification-alt</see> for alternate instructions.  We have \u000A<i>not</i> changed how <v>make regression</v> works from the <v>acl2-sources</v> \u000Adirectory.</p> \u000A \u000A \u000A<h3>Prerequisites</h3> \u000A \u000A<p>We assume that you have already downloaded and installed ACL2 as per the <a \u000Ahref=\"http://www.cs.utexas.edu/users/moore/acl2/v6-5/HTML/installation/installation.html\">ACL2 installation instructions</a> on \u000Athe ACL2 home page.</p> \u000A \u000A<p>We assume you know the path to your ACL2 executable.  Typically this is a \u000Ascript named <v>saved_acl2</v> in your <v>acl2-sources</v> directory.</p> \u000A \u000A<p>We assume the ACL2 <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> are installed in the <v>books/</v> \u000Asubdirectory of your ACL2 distribution, as is the case when you have followed \u000Athe ACL2 installation instructions above.</p> \u000A \u000A<p>The instructions below are suitable for ACL2 and its experimental \u000Aextensions, ACL2(p) and ACL2(h).  If you are using ACL2(r), please instead see \u000A<see topic=\"ACL2____BOOKS-CERTIFICATION-ALT\">books-certification-alt</see>.</p> \u000A \u000A<h3>A Basic Build</h3> \u000A \u000A<p>In previous versions of ACL2, building the Community Books could take \u000Aseveral hours.  Starting in ACL2 6.4, the default build has been made <b>much \u000Afaster by excluding many books</b> by default.</p> \u000A \u000A<p>The new default <v>make</v> target, called <v>basic</v>, now certifies only the \u000Afollowing, widely used books:</p> \u000A \u000A<ul> \u000A<li>arithmetic</li> \u000A<li>arithmetic-2</li> \u000A<li>arithmetic-3</li> \u000A<li>arithmetic-5</li> \u000A<li><see topic=\"ACL2____IHS\">ihs</see></li> \u000A<li>misc</li> \u000A<li>tools</li> \u000A<li><see topic=\"ACL2____STD\">std</see></li> \u000A<li><see topic=\"ACL2____XDOC\">xdoc</see></li> \u000A<li>data-structures</li> \u000A</ul> \u000A \u000A<p>To certify these books, you should be able to run <v>make</v> as follows.  The \u000A<v>-j 2</v> part of this command is suitable for a computer with two cores.  If \u000Ayou have, e.g., a quad-core computer, you should probably use <v>-j 4</v> \u000Ainstead, and so on.</p> \u000A \u000A<code>$ cd /path/to/acl2-sources/books\u000A$ make ACL2=/path/to/acl2-sources/saved_acl2 -j 2 basic</code> \u000A \u000A<p>If you configure your <v>PATH</v> so that you can launch ACL2 by typing \u000A<v>acl2</v>, then you may omit the <v>ACL2=...</v> part.</p> \u000A \u000A \u000A<h3>Certifying Additional Books</h3> \u000A \u000A<p>We expect that most ACL2 users will want to certify at least the <v>basic</v> \u000Abooks described above.  But what if you also need other books?</p> \u000A \u000A<p>One option is to do a full build (see below).  But it is usually much faster \u000Ato simply tell <v>make</v> to build the books you actually want to use.</p> \u000A \u000A<p>There are <v>make</v> targets corresponding to most directory names.  For \u000Ainstance, to build the books under <v>coi</v> and <v>rtl</v> and <v>cgen</v>, you can \u000Arun:</p> \u000A \u000A<code>$ cd /path/to/acl2-sources/books\u000A$ make ACL2=/path/to/acl2-sources/saved_acl2 coi rtl cgen -j 2</code> \u000A \u000A<p>For finer grained control, you can name individual books.  For instance, if \u000Ayou want the <v>rtl/rel9</v> library, you could run:</p> \u000A \u000A<code>$ cd /path/to/acl2-sources/books\u000A$ make ACL2=/path/to/acl2-sources/saved_acl2 rtl/rel9/lib/top.cert -j 2</code> \u000A \u000A \u000A<h3>A Full Build</h3> \u000A \u000A<p>Building all of the books can take hours and is usually unnecessary, but is \u000Aeasy to do: just run <v>make all</v>, e.g.,</p> \u000A \u000A<code>$ cd /path/to/acl2-sources/books\u000A$ make ACL2=/path/to/acl2-sources/saved_acl2 -j 2 all</code> \u000A \u000A<p>This actually still skips a few books that are very slow.  If you \u000A<i>really</i> need to certify absolutely everything, you can run <v>make\u000Aeverything</v>, but this will likely add hours to your build!</p> \u000A \u000A \u000A<h3>ACL2 Extensions and Additional Software</h3> \u000A \u000A<p>Note that some books require experimental extensions of ACL2 (e.g., a book \u000Amay work with ACL2(h) but not with ordinary ACL2).  Other books require certain \u000Aadditional software.</p> \u000A \u000A<p>The build system will automatically determine which kind of ACL2 you are \u000Arunning (e.g., ACL2, ACL2(h), ACL2(p)) and, based on this, may prevent \u000Aincompatible books from being certified.  The output of <v>make</v> should \u000Aexplain which books are being excluded and why.</p> \u000A \u000A<p>Some books, especially <see topic=\"ACL2____INTERFACING-TOOLS\">interfacing-tools</see> like <see topic=\"ACL2____OSLIB\">oslib</see> and the \u000AACL2 <see topic=\"ACL2____BRIDGE\">bridge</see>, require <see topic=\"ACL2____QUICKLISP\">quicklisp</see>, a tool for installing Common Lisp \u000Alibraries.  Quicklisp does not work with all Lisps that can run ACL2, so you \u000Amust explicitly enable it by setting <v>USE_QUICKLISP=1</v> in your <v>make</v> \u000Acommand.  For instance:</p> \u000A \u000A<code>make ACL2=... USE_QUICKLISP=1 doc/top.cert -j 4</code> \u000A \u000A<p>Using Quicklisp should definitely work for CCL.  We have not tested it with \u000Aother Lisps, but it will likely work for SBCL and CMUCL, and will almost \u000Acertainly <i>not</i> work for GCL.</p> \u000A \u000A<p>Some other books based on <see topic=\"ACL2____SATLINK\">satlink</see> and <see topic=\"ACL2____GL\">gl</see> require a SAT solver, \u000Atypically Glucose, to be installed; see <see topic=\"SATLINK____SAT-SOLVER-OPTIONS\">satlink::sat-solver-options</see> for \u000Ainstallation options.  The build system should automatically determine if \u000AGlucose is installed on your system, and will avoid trying to certify these \u000Abooks unless Glucose is present.</p> \u000A \u000A \u000A<h3>Cleaning Up</h3> \u000A \u000A<p>If you want to delete generated files, you can run <v>make clean</v> to remove \u000Acertificates, compiled files, and build logs.</p> \u000A \u000A<p>If you just want to remove the files in a particular subdirectory (and its \u000Asubdirectories), you can go into that directory and then run the \u000A<v>build/clean.pl</v> script.  This will delete, starting from your current \u000Adirectory, recursively, all certificates, logs, compiled files, etc.</p> \u000A \u000A<p>Note that <v>make clean</v> doesn't remove some files, e.g., <see topic=\"ACL2____XDOC\">xdoc</see> \u000Amanuals.  To remove everything, try <v>make moreclean</v>.</p> \u000A \u000A \u000A<h3>Further Resources</h3> \u000A \u000A<p>The new build system is largely based on <see topic=\"ACL2____CERT.PL\">cert.pl</see>.  There is \u000Aconsiderable documentation about <v>cert.pl</v>, and we highly recommend using it \u000Ato manage your own ACL2 projects.</p> \u000A \u000A<p>The main build script is <v>books/GNUmakefile</v>.  There are considerable \u000Acomments at the start of this file, and you can also inspect it to see what \u000Atargets are available.</p> \u000A \u000A<p>Please feel absolutely free to contact the <see topic=\"ACL2____ACL2-HELP\">ACL2-help</see> mailing list \u000Awith any questions about building the community books.</p>"],

 "ACL2____ACL2-BUILT-INS":[["Programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is a parent topic under which we \u000A include documentation for built-in functions, macros, and special forms that \u000A are typically used in programming.  For others, including those typically used \u000A as top-level commands or those that create <see topic=\"ACL2____EVENTS\">events</see> (<tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, and so on), documentation may be found as a subtopic of some \u000A other parent topic.  We do not document some of the more obscure functions \u000A provided by ACL2 that do not correspond to functions of Common Lisp.</p> \u000A \u000A <p>See any documentation for Common Lisp for more details on many of these \u000A functions.</p>"],

 "ACL2____WELL-FOUNDED-RELATION":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lex2p-is-well-founded-relation\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (pairp x) (<see topic=\"ACL2____O-P\">o-p</see> (ordinate x)))\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pairp x)\u000A                     (pairp y)\u000A                     (lex2p x y))\u000A                (<see topic=\"ACL2____O_C3\">o&lt;</see> (ordinate x) (ordinate y))))\u000A  :rule-classes :well-founded-relation)</code> \u000A \u000A <p>The example above creates a <v>:well-founded-relation</v> rule, where of \u000A course the functions <v>pairp</v>, <v>lex2p</v>, and <v>ordinate</v> would have to \u000A be defined first.  It establishes that <v>lex2p</v> is a well-founded relation \u000A on <v>pairp</v>s.  We explain and give details below.</p> \u000A \u000A <p>Exactly two general forms are recognized:</p> \u000A \u000A <code>General Forms\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (mp x) (<see topic=\"ACL2____O-P\">O-P</see> (fn x)))              ; Property 1\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (mp x)                       ; Property 2\u000A                   (mp y)\u000A                   (rel x y))\u000A              (<see topic=\"ACL2____O_C3\">O&lt;</see> (fn x) (fn y)))),</code> \u000A \u000A <p>or</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____O-P\">O-P</see> (fn x))                               ; Property 1\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (rel x y)                         ; Property 2\u000A              (<see topic=\"ACL2____O_C3\">O&lt;</see> (fn x) (fn y))))</code> \u000A \u000A <p>where <v>mp</v>, <v>fn</v>, and <v>rel</v> are function symbols, <v>x</v> and \u000A <v>y</v> are distinct variable symbols, and no other <v>:well-founded-relation</v> \u000A theorem about <v>fn</v> has been proved.  When the second general form is used, \u000A we act as though the first form were used with <v>mp</v> being the function that \u000A ignores its argument and returns <v>t</v>.  The discussion below therefore \u000A considers only the first general form.</p> \u000A \u000A <p>Note: We are very rigid when checking that the submitted formula is of one \u000A of these forms.  For example, in the first form, we insist that all the \u000A conjuncts appear in the order shown above.  Thus, interchanging the two \u000A properties in the top-level conjunct or rearranging the hyptheses in the \u000A second conjunct both produce unrecognized forms.  The requirement that each \u000A <v>fn</v> be proved well-founded at most once ensures that for each well-founded \u000A relation, <v>fn</v>, there is a unique <v>mp</v> that recognizes the domain on \u000A which <v>rel</v> is well-founded.  We impose this requirement simply so that \u000A <v>rel</v> can be used as a short-hand when specifying the well-founded \u000A relations to be used in definitions; otherwise the specification would have to \u000A indicate which <v>mp</v> was to be used.</p> \u000A \u000A <p>We also insist that the new ordering be embedded into the ordinals as \u000A handled by <tt><see topic=\"ACL2____O-P\">o-p</see></tt> and <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> and not some into previously admitted \u000A user-defined well-founded set and relation.  This restriction should pose no \u000A hardship.  If <v>mp</v> and <v>rel</v> were previously shown to be well-founded \u000A via the embedding <v>fn</v>, and you know how to embed some new set and relation \u000A into <v>mp</v> and <v>rel</v>, then by composing <v>fn</v> with your new embedding \u000A and using the previously proved well-founded relation lemma you can embed the \u000A new set and relation into the ordinals.</p> \u000A \u000A <p><v>Mp</v> is a predicate that recognizes the objects that are supposedly \u000A ordered in a well-founded way by <v>rel</v>.  We call such an object an \u000A ``<v>mp</v>-measure'' or simply a ``measure'' when <v>mp</v> is understood. \u000A Property 1 tells us that every measure can be mapped into an ACL2 ordinal. \u000A (See <see topic=\"ACL2____O-P\">o-p</see>.)  This mapping is performed by <v>fn</v>.  Property 2 tells us \u000A that if the measure <v>x</v> is smaller than the measure <v>y</v> according to \u000A <v>rel</v> then the image of <v>x</v> under <v>fn</v> is a smaller than that of \u000A <v>y</v>, according to the well-founded relation <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>.  (See <see topic=\"ACL2____O_C3\">o&lt;</see>.) \u000A Thus, the general form of a <v>:well-founded-relation</v> formula establishes \u000A that there exists a <v>rel</v>-order preserving embedding (namely via <v>fn</v>) \u000A of the <v>mp</v>-measures into the ordinals.  We can thus conclude that <v>rel</v> \u000A is well-founded on <v>mp</v>-measures.</p> \u000A \u000A <p>Such well-founded relations are used in the admissibility test for \u000A recursive functions, in particular, to show that the recursion terminates.  To \u000A illustrate how such information may be used, consider a generic function \u000A definition</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____IF\">if</see> (test x) (g (step x)) (base x))).</code> \u000A \u000A <p>If <v>rel</v> has been shown to be well-founded on <v>mp</v>-measures, then \u000A <v>g</v>'s termination can be ensured by finding a measure, <v>(m x)</v>, with the \u000A property that <v>m</v> produces a measure:</p> \u000A \u000A <code>(mp (m x)),                                     ; Defun-goal-1</code> \u000A \u000A <p>and that the argument to <v>g</v> gets smaller (when measured by <v>m</v> and \u000A compared by <v>rel</v>) in the recursion,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (test x) (rel (m (step x)) (m x))).    ; Defun-goal-2</code> \u000A \u000A <p>If <v>rel</v> is selected as the <v>:well-founded-relation</v> to be used in \u000A the definition of <v>g</v>, the definitional principal will generate and attempt \u000A to prove <v>defun-goal-1</v> and <v>defun-goal-2</v> to justify <v>g</v>.  We show \u000A later why these two goals are sufficient to establish the termination of \u000A <v>g</v>.  Observe that neither the ordinals nor the embedding, <v>fn</v>, of the \u000A <v>mp</v>-measures into the ordinals is involved in the goals generated by the \u000A definitional principal.</p> \u000A \u000A <p>Suppose now that a <v>:well-founded-relation</v> theorem has been proved for \u000A <v>mp</v> and <v>rel</v>.  How can <v>rel</v> be ``selected as the \u000A <v>:well-founded-relation</v>'' by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>?  There are two ways.  First, \u000A an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword to the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event allows the specification \u000A of a <v>:well-founded-relation</v>.  Thus, the definition of <v>g</v> above might \u000A be written</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :well-founded-relation (mp . rel)))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (test x) (g (step x)) (base x)))</code> \u000A \u000A <p>Alternatively, <v>rel</v> may be specified as the \u000A <v>:default-well-founded-relation</v> in <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> by \u000A executing the event</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> rel).</code> \u000A \u000A <p>When a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event does not explicitly specify the relation in its \u000A <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> the default relation is used.  When ACL2 is initialized, the \u000A default relation is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt>.</p> \u000A \u000A <p>Finally, though it is probably obvious, we now show that <v>defun-goal-1</v> \u000A and <v>defun-goal-2</v> are sufficient to ensure the termination of <v>g</v> \u000A provided <v>property-1</v> and <v>property-2</v> of <v>mp</v> and <v>rel</v> have been \u000A proved.  To this end, assume we have proved <v>defun-goal-1</v> and \u000A <v>defun-goal-2</v> as well as <v>property-1</v> and <v>property-2</v> and we show \u000A how to admit <v>g</v> under the primitive ACL2 definitional principal (i.e., \u000A using only the ordinals).  In particular, consider the definition event</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :well-founded-relation o&lt;\u000A                 :measure (fn (m x))))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (test x) (g (step x)) (base x)))</code> \u000A \u000A <p>Proof that <v>g</v> is admissible: To admit the definition of <v>g</v> above we \u000A must prove</p> \u000A \u000A <code>(<see topic=\"ACL2____O-P\">o-p</see> (fn (m x)))                        ; *1</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (test x)                               ; *2\u000A         (<see topic=\"ACL2____O_C3\">o&lt;</see> (fn (m (step x))) (fn (m x)))).</code> \u000A \u000A <p>But *1 can be proved by instantiating <v>property-1</v> to get</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (mp (m x)) (<see topic=\"ACL2____O-P\">o-p</see> (fn (m x)))),</code> \u000A \u000A <p>and then relieving the hypothesis with <v>defun-goal-1</v>, <v>(mp (m\u000A x))</v>.</p> \u000A \u000A <p>Similarly, *2 can be proved by instantiating <v>property-2</v> to get</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (mp (m (step x)))\u000A              (mp (m x))\u000A              (rel (m (step x)) (m x)))\u000A         (<see topic=\"ACL2____O_C3\">o&lt;</see> (fn (m (step x))) (fn (m x))))</code> \u000A \u000A <p>and relieving the first two hypotheses by appealing to two instances of \u000A <v>defun-goal-1</v>, thus obtaining</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (rel (m (step x)) (m x))\u000A         (<see topic=\"ACL2____O_C3\">o&lt;</see> (fn (m (step x))) (fn (m x)))).</code> \u000A \u000A <p>By chaining this together with <v>defun-goal-2</v>,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (test x)\u000A         (rel (m (step x)) (m x)))</code> \u000A \u000A <p>we obtain *2.  Q.E.D.</p>"],

 "ACL2____PROGRAMMING-WITH-STATE":[["State"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> section introduces some common techniques \u000A for programming using the ACL2 state object.  A prerequisite is thus a basic \u000A understanding of that object; see <see topic=\"ACL2____STATE\">state</see>.  We hope this section is \u000A useful, and we invite suggestions for improvements and additions.</p> \u000A \u000A <p>A supplement to this section is the ACL2 source code, which uses most (and \u000A probably all) of the techniques discussed here.  That code is thus a source of \u000A many examples, which can serve as ``templates'' to guide one's own programming \u000A with state.</p> \u000A \u000A <p>Recall that ``ACL2'' stands for ``A Computational Logic for Applicative \u000A Common Lisp''.  In particular, the language is applicative: there are no \u000A global variables or side effects.  For many purposes this does not feel \u000A restrictive; for example, an ACL2 user who is programming in raw Lisp may well \u000A be more comfortable coding a factorial function applicatively, using \u000A recursion, rather than using iteration with repeated assignment to the same \u000A variable.</p> \u000A \u000A <p>However, there are situations that call for reading or modifying the system \u000A state, such as performing input and output, signalling errors, saving \u000A information from one computation for use in a later one, or reading and \u000A updating system-level or environmental data.  This section provides an \u000A introductory guide for writing functions that traffic in state.  We emphasize \u000A that this guide is intended as an introduction; more complete documentation \u000A may often be found by following links to documentation of individual \u000A utilities, and again, more examples may be found by searching the ACL2 source \u000A code for uses of the functions and macros mentioned below.  The rest of this \u000A section is organized as follows.</p> \u000A \u000A <code>ENABLING PROGRAMMING WITH STATE\u000ASTATE GLOBALS AND THE ACL2 LOGICAL WORLD\u000AA REMARK ON GUARDS\u000AERRORS AND ERROR TRIPLES\u000ASEQUENTIAL PROGRAMMING\u000ABINDING VARIABLES USING ERROR TRIPLES\u000ABINDING STATE GLOBAL VARIABLES\u000AINPUT AND OUTPUT\u000ATIMINGS\u000AENVIRONMENT AND SYSTEM\u000AREMARKS ON EVENTS AND LD\u000AADVANCED TOPICS</code> \u000A \u000A <p>ENABLING PROGRAMMING WITH STATE</p> \u000A \u000A <p>In order to submit a definition that takes <tt><see topic=\"ACL2____STATE\">state</see></tt> as a formal \u000A parameter, you must either declare <v>state</v> as a <v>:</v><tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> (see \u000A <see topic=\"ACL2____XARGS\">xargs</see>) or first evaluate the following form at the top level: \u000A <v>(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)</v>.</p> \u000A \u000A <p>Consider for example the following trivial definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"ACL2____MV\">mv</see> 3 state))</code> \u000A \u000A <p>If you submit the above definition in a fresh ACL2 session, you will get \u000A this error message.</p> \u000A \u000A <code>ACL2 Error in ( DEFUN FOO ...):  The variable symbol STATE should not\u000Abe used as a formal parameter of a defined function unless you are\u000Aaware of its unusual status and the restrictions enforced on its use.\u000ASee :DOC set-state-ok.</code> \u000A \u000A <p>If first you evaluate <v>(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)</v>, you can admit the above \u000A definition.  Alternatively, you can declare <v>state</v> as a <v>:</v><tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A  (<see topic=\"ACL2____MV\">mv</see> 3 state))</code> \u000A \u000A <p>A difference in the two approaches is that for the latter, a <see topic=\"ACL2____GUARD\">guard</see> \u000A proof obligation is generated by default.  See the section below entitled ``A \u000A remark on guards''.</p> \u000A \u000A <p>STATE GLOBALS AND THE ACL2 LOGICAL WORLD</p> \u000A \u000A <p>Recall (see <see topic=\"ACL2____STATE\">state</see>) that one of the fields of the ACL2 state object \u000A is the global-table, which logically is an alist associating symbols, known as \u000A ``state globals'' or ``state global variables'', with values.  But no such \u000A alist actually exists in the implementation.  Instead, ACL2 provides utilities \u000A for reading state globals &mdash; see <see topic=\"ACL2_____04\">@</see> and see <see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> \u000A &mdash; and utilities for writing them &mdash; see <see topic=\"ACL2____ASSIGN\">assign</see> and see <see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see>.  The following log shows how they work; further explanation \u000A follows below.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> my-var (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A 7\u000AACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> my-var)\u000A7\u000AACL2 !&gt;(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'my-var (<see topic=\"COMMON-LISP_____B2\">+</see> 1 5) state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'my-var state)\u000A6\u000AACL2 !&gt;</code> \u000A \u000A <p>Note that the first result is indented by one space.  This is ACL2's way to \u000A indicate that the <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> expression returned an ``error triple'' and \u000A that no error was signalled.  We discuss error triples in more detail below; \u000A also see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.</p> \u000A \u000A <p>As illustrated above, the output signatures of the utilities for assigning \u000A to state globals differ from each other as follows: <tt><see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see></tt> \u000A returns <v>state</v>, but <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> returns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil val\u000A state)</v> where <v>val</v> is the value assigned to the state global.  The output \u000A signatures of the utilities for reading, <v>@</v> and <v>f-get-global</v>, are \u000A identical.  In fact, the form <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'my-var state)</v> is the \u000A single-step macroexpansion of the form <v>(<see topic=\"ACL2_____04\">@</see> my-var)</v>, as can be confirmed \u000A using <tt><see topic=\"ACL2____TRANS1\">trans1</see></tt>.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2_____04\">@</see> my-var)\u000A (<see topic=\"ACL2____F-GET-GLOBAL\">F-GET-GLOBAL</see> 'MY-VAR STATE)\u000AACL2 !&gt;</code> \u000A \u000A <p>State globals are useful for conveying persistent state information. \u000A Consider for example the utility <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>.  The form \u000A <v>(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> '(prove proof-tree))</v> is approximately equivalent \u000A to (assign inhibit-output-lst '(prove proof-tree)).  We say ``approximately'' \u000A because <v>set-inhibit-output-lst</v> additionally does some error checking to \u000A insure that all the tokens in the new list are legal.  When deciding whether \u000A to print output, the ACL2 system reads the value of state global variable \u000A <v>inhibit-output-lst</v>.</p> \u000A \u000A <p>A particularly useful state global is <v>current-acl2-world</v>, whose value \u000A is the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.  Because the ACL2 world is commonly accessed \u000A in applications that use the ACL2 state, ACL2 provides a function that returns \u000A the world: <v>(w state) = (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'current-acl2-world state)</v>.  While \u000A it is common to read the world, only functions <v>set-w</v> and <v>set-w!</v> are \u000A available to write the world, but these are untouchable and these should \u000A generally be avoided except by system implementors \u000A (pl[remove-untouchable]).</p> \u000A \u000A <p>A REMARK ON GUARDS</p> \u000A \u000A <p>For a function definition (see <see topic=\"COMMON-LISP____DEFUN\">defun</see>), if <v>state</v> is specified as \u000A a <see topic=\"ACL2____STOBJ\">stobj</see> as with the form <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))</v>, then the \u000A <see topic=\"ACL2____GUARD\">guard</see> for that function is considered to include the condition \u000A <v>(state-p state)</v>.  By default, <see topic=\"ACL2____GUARD\">guard</see> verification will then be \u000A performed.</p> \u000A \u000A <p>We can illustrate this point by modifying the example above as follows, to \u000A read the value of state global <v>gag-mode</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A  (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'gag-mode state))</code> \u000A \u000A <p>If you try this in a fresh ACL2 session, the proof will fail with the \u000A following key checkpoint, which says that the state global <v>gag-mode</v> is \u000A bound in the global-table of the state.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (STATE-P1 STATE)\u000A         (<see topic=\"ACL2____ASSOC-EQUAL\">ASSOC-EQUAL</see> 'GAG-MODE (<see topic=\"COMMON-LISP____NTH\">NTH</see> 2 STATE)))</code> \u000A \u000A <p>How can we deal with this proof failure?  One way is simply to ignore the \u000A issue by defining the function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                  :mode :program))\u000A  (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'gag-mode state))</code> \u000A \u000A <p>Perhaps a better way is to strengthen the guard to assert that the \u000A indicated state global is bound, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (boundp-global 'gag-mode state)\u000A                  :stobjs state))\u000A  (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'gag-mode state))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____GUARD-MISCELLANY\">guard-miscellany</see> for a discussion of how guards are \u000A generated from <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> fields of <see topic=\"COMMON-LISP____DECLARE\">declare</see> forms, specifically, for \u000A keywords <v>:guard</v> and <v>:stobjs</v>.</p> \u000A \u000A <p>ERRORS AND ERROR TRIPLES</p> \u000A \u000A <p>When evaluation returns three values, where the first two are ordinary \u000A objects and the third is the ACL2 state, the result may be called an ``error \u000A triple''.  (Whether it is treated as an error triple depends on the \u000A programmer.)  Error triples are often denoted <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>, and \u000A common ACL2 programming idioms treat <v>erp</v> as a flag indicating whether an \u000A error is being signalled and <v>val</v> as the ``value'' computed.  Also see \u000A <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.</p> \u000A \u000A <p>Even ACL2 users who are not programmers encounter error triples, because \u000A these are the values returned by evaluation of ACL2 <see topic=\"ACL2____EVENTS\">events</see>.  Consider \u000A the following log, where the only user input is the <v>defun</v> form following \u000A the <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A\u000ASince FOO is non-recursive, its admission is trivial.  We observe that\u000Athe type of FOO is described by the theorem (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (FOO X) X).\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 !&gt;</code> \u000A \u000A <p>All output above results from explicit calls of output functions, except \u000A for the next-to-last line, which contains <v>FOO</v>.  Notice the single-space \u000A indentation preceding <v>FOO</v>.  That space indicates that in fact, the value \u000A returned by evaluation of the <v>defun</v> form is the error triple whose error \u000A flag is <v>nil</v> and whose computed value is <v>FOO</v>.  By default, ACL2 \u000A prints any error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil val state)</v> by inserting a space before \u000A printing <v>val</v>.  You can change the default by setting state global <tt><see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see></tt> to <v>t</v>; notice how the same result is printed \u000A below.</p> \u000A \u000A <code>ACL2 !&gt;:u\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 !&gt;(set-ld-post-eval-print t state)\u000A(NIL T &lt;state&gt;)\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A\u000ASince FOO is non-recursive, its admission is trivial.  We observe that\u000Athe type of FOO is described by the theorem (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (FOO X) X).\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A(NIL FOO &lt;state&gt;)\u000AACL2 !&gt;</code> \u000A \u000A <p>The way error triples are printed by <v>ld</v> is controlled not only by \u000A state global <v>ld-post-eval-print</v>, but also by state global \u000A <v>ld-error-triples</v>.  These are examples of ``ld specials''; see <see topic=\"ACL2____LD\">ld</see>, \u000A see <see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see>, and see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see>.</p> \u000A \u000A <p>It is so common to produce an error triple whose first (error flag) \u000A component is <v>nil</v> that ACL2 provides a handy macro, <v>value</v>, for this \u000A purpose.  Thus, <v>(value &lt;expression&gt;)</v> is equivalent to <v>(<see topic=\"ACL2____MV\">mv</see> nil\u000A &lt;expression&gt; state)</v>.  Also see <see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> for a similar construct \u000A that is a legal event form.</p> \u000A \u000A <p>We turn now to the topic of errors.  The macro <tt><see topic=\"ACL2____ER\">er</see></tt> ``causes'' an \u000A error, but there are really two quite different kinds of errors: ``soft'' and \u000A ``hard'' errors.  We use the term ``soft error'' to refer to a form that \u000A returns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> for which <v>erp</v> is \u000A non-<v>nil</v>.  Soft errors do not interrupt the normal flow of evaluation: the \u000A error triple is returned to the caller which interprets the <v>erp</v> flag and \u000A <v>val</v> as directed by the programmer.  Macros discussed below make it \u000A convenient to think about soft errors as short-circuiting the computation. \u000A Hard errors, on the other hand, do actually rip control away from the current \u000A evaluation and return it to the top-level loop.  Logically speaking, \u000A expressions that cause hard errors return <v>nil</v> in the error case, but the \u000A <v>nil</v> is never seen in actual evaluation because control does not return to \u000A the caller.</p> \u000A \u000A <p>Note that the function <tt><see topic=\"ACL2____ABORT_12\">abort!</see></tt>, which you can invoke by typing \u000A <v>:</v><tt><see topic=\"ACL2____A_12\">a!</see></tt>, always returns to the top level.  Note that ACL2 can prove \u000A that <v>(<see topic=\"ACL2____ABORT_12\">abort!</see>)</v> returns <v>nil</v> but that this cannot be confirmed by \u000A computation.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____ABORT_12\">abort!</see>) nil))\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:TYPE-PRESCRIPTION ABORT!))\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____ABORT_12\">abort!</see>) nil)\u000AAbort to ACL2 top-level\u000A...\u000AACL2 !&gt;</code> \u000A \u000A <p>(What actually happens with a hard error, including non-default cases, is a \u000A bit subtle; most readers will probably want to skip this paragraph.  The \u000A read-eval-print loop implemented by <tt><see topic=\"ACL2____LD\">ld</see></tt> is implemented by a call of the \u000A ACL2 evaluator function, <v>trans-eval</v>, on each input form.  If a hard error \u000A occurs during evaluation of an input form, its <v>trans-eval</v> call will \u000A return with a soft error.  <tt><see topic=\"ACL2____LD\">Ld</see></tt>, in turn handles that soft error \u000A appropriately; see <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see>.)</p> \u000A \u000A <p>The most common way to signal errors is the macro <tt><see topic=\"ACL2____ER\">er</see></tt>, which prints \u000A a formatted error message and returns a soft or hard error as specified by the \u000A call.  Note however that soft errors are signalled using <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode functions.</p> \u000A \u000A <p>Since the output signatures of soft and hard errors are different &mdash; \u000A hard errors ``return'' a single value while soft errors return a triple \u000A &mdash; mixing them in an expression requires embedding the hard error form in \u000A (an irrelevant) triple, as illustrated below.  All branches of the expression \u000A must produce an error triple if any branch does.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> chk-find-or-abort (e x state)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A             (value                          ; Note use of VALUE!\u000A               (<see topic=\"ACL2____ER\">er</see> hard 'chk-find-or-abort\u000A                   &quot;Did not find ~x0!&quot;\u000A                    e))\u000A             (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                 (<see topic=\"ACL2____ER\">er</see> soft 'chk-find-or-abort\u000A                     &quot;Non-integer, ~x0, in list!&quot;\u000A                     (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                 (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) e)\u000A                     (value x)\u000A                     (chk-find-or-abort e (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) state)))))\u000A\u000ASummary\u000AForm:  ( DEFUN CHK-FIND-OR-ABORT ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A CHK-FIND-OR-ABORT\u000AACL2 !&gt;(chk-find-or-abort 3 '(1 2 3 4 5) state)\u000A (3 4 5)\u000AACL2 !&gt;(chk-find-or-abort 3 '(1 A 3 4 5) state)\u000A\u000AACL2 Error in CHK-FIND-OR-ABORT:  Non-integer, A, in list!\u000A\u000AACL2 !&gt;(chk-find-or-abort 3 '(1 2 4 5) state)\u000A\u000AHARD ACL2 ERROR in CHK-FIND-OR-ABORT:  Did not find 3!\u000A...\u000AACL2 !&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____ER\">er</see> for further discussion of errors.  For some other individual \u000A topics related to errors see <see topic=\"ACL2____ASSERT_42\">assert$</see>, see <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>, see \u000A <see topic=\"ACL2____ERROR1\">error1</see>, see <see topic=\"ACL2____HARD-ERROR\">hard-error</see>, see <see topic=\"ACL2____ILLEGAL\">illegal</see>, and see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see>.</p> \u000A \u000A <p>In the next section we discuss soft errors further, in the context of \u000A programming.</p> \u000A \u000A <p>SEQUENTIAL PROGRAMMING</p> \u000A \u000A <p>This section describes handy ways to modify state in steps, using macros \u000A that implement a sequence of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> bindings.  For \u000A example, suppose you want to assign the values 1 and 2 to two state globals \u000A <v>one-var</v> and <v>two-var</v>, respectively.  Because of ACL2's syntactic \u000A restrictions on <tt><see topic=\"ACL2____STATE\">state</see></tt>, it is not legal simply to write \u000A <v>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'two-var 2 (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'one-var 1 state))</v>.  However, \u000A <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> comes to the rescue as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'one-var 1 state)))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'two-var 2 state)))\u000A    state))</code> \u000A \u000A <p>It is so common to bind state successively in such a manner that ACL2 \u000A provides a macro, <tt><see topic=\"ACL2____PPROGN\">pprogn</see></tt>, for this purpose.  Thus, an equivalent \u000A solution to the problem above is</p> \u000A \u000A <code>(<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'one-var 1 state)\u000A        (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'two-var 2 state)\u000A        state)</code> \u000A \u000A <p>or, more simply, as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'one-var 1 state)\u000A        (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'two-var 2 state))</code> \u000A \u000A <p>See <see topic=\"ACL2____PPROGN\">pprogn</see>.  Note that the last form is allowed to return multiple \u000A values; the only requirement on the last form is that its value include \u000A <v>state</v>.</p> \u000A \u000A <p>It is also common to update the state using a sequence of forms such that \u000A each returns an error triple, where the intention is for evaluation to \u000A short-circuit immediately if a soft error is encountered.  Suppose \u000A <v>&lt;expr1&gt;</v> and <v>&lt;expr2&gt;</v> are expressions that return error triples, where \u000A the <v>state</v> components of the error triples might be updated, and one \u000A wishes to evaluate <v>&lt;expr1&gt;</v> and then <v>&lt;expr2&gt;</v>, returning the \u000A (multiple) values returned by <v>&lt;expr2&gt;</v> unless the error triple returned by \u000A <v>&lt;expr1&gt;</v> is a soft error, in which case that error triple is returned. \u000A One can of course do so as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        &lt;expr1&gt;\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp val state))\u000A              (t &lt;expr2&gt;)))</code> \u000A \u000A <p>But ACL2 provides a handy macro, <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt>, for this purpose.  The \u000A following code is equivalent to the code just above.</p> \u000A \u000A <code>(<see topic=\"ACL2____ER-PROGN\">er-progn</see> &lt;expr1&gt; &lt;expr2&gt;)</code> \u000A \u000A <p>See <see topic=\"ACL2____ER-PROGN\">er-progn</see> for more details.  Note that unlike <tt><see topic=\"ACL2____PPROGN\">pprogn</see></tt>, the \u000A return <see topic=\"ACL2____SIGNATURE\">signature</see> for the last expression must be the same as that of \u000A the others: an error triple.</p> \u000A \u000A <p>Let's consider how to use <v>pprogn</v> and <v>er-progn</v> together.  In the \u000A following example <v>f1</v> and <v>f2</v> both return <v>state</v>, while each of \u000A <v>g1</v> and <v>g2</v> returns an error triple.  The following code modifies \u000A state by executing these in the order <v>f1</v>, <v>g1</v>, <v>f2</v>, and finally \u000A <v>g2</v>, returning <v>(<see topic=\"ACL2____MV\">mv</see> nil val state)</v> where <v>val</v> is the value \u000A component of the error triple returned by <v>g2</v> &mdash; except we return a \u000A soft error if <v>g1</v> or <v>g2</v> returns a soft error.</p> \u000A \u000A <code>(<see topic=\"ACL2____PPROGN\">pprogn</see> (f1 x state)\u000A        (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (g1 x state)\u000A                  (<see topic=\"ACL2____PPROGN\">pprogn</see> (f2 x state)\u000A                          (g2 x state))))</code> \u000A \u000A <p>Finally, consider the <see topic=\"ACL2____EVENTS\">events</see> <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>. \u000A These have similar behavior to that of <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt>.  However, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> may only be used in event contexts, for example at \u000A the top level or immediately underneath a call of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, while <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> has no such restriction.  So when \u000A writing code, use <v>er-progn</v> rather than <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>. \u000A In particular, the body of a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> must not have any calls of \u000A <v>progn</v> (or of <v>progn!</v>  either), and the same restriction holds for any \u000A code to be executed, such as the body of a <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> form.</p> \u000A \u000A <p>BINDING VARIABLES USING ERROR TRIPLES</p> \u000A \u000A <p>In this section we discuss the macro <v>er-let*</v>, which is a variant of \u000A the special form, <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, that is useful when programming with \u000A state.</p> \u000A \u000A <p>The macro <v>er-let*</v> is useful when binding variables to the value \u000A components of error triples.  It is actually quite similar to <v>er-progn</v>, \u000A described above, except that <v>er-let*</v> binds variables.  First consider the \u000A following example.</p> \u000A \u000A <code>(er-let* ((x1 (f1 state))\u000A          (x2 (f2 x1 state)))\u000A  (value (<see topic=\"COMMON-LISP____CONS\">cons</see> x1 x2)))</code> \u000A \u000A <p>The code just above is essentially equivalent to writing the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp x1 state)\u000A        (f1 state)\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp x1 state))\u000A              (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp x2 state)\u000A                         (f2 x1 state)\u000A                         (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp x2 state))\u000A                               (t (value (<see topic=\"COMMON-LISP____CONS\">cons</see> x1 x2))))))))</code> \u000A \u000A <p>As suggested by the example above, <v>er-let*</v> has the same syntax as \u000A <v>let*</v>, except that declarations are not supported.  (But note that \u000A <v>ignore</v> declarations are not needed; all variables that are bound are also \u000A used, at least in the error case.  Consider replacing <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x1 x2)</v> by \u000A <v>nil</v> in the example displayed immediately above, and note that <v>x1</v> and \u000A <v>x2</v> are still used.)  However, unlike <v>let*</v>, <v>er-let*</v> requires \u000A that for each binding <v>(var expr)</v>, the expression <v>expr</v> must evaluate \u000A to an error triple and, moreover, it requires that the second argument (the \u000A ``body'') of <v>er-let*</v> must evaluate to an error triple.  If one of the \u000A variable expressions (e.g., the <v>f1</v> and <v>f2</v> calls above) signals an \u000A error, its error triple is returned as the value of the <v>er-let*</v>.</p> \u000A \u000A <p>Of course, soft errors can be ``caught'' by using <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> instead of \u000A <v>er-let*</v> and simply ignoring the error flag or, more generally, by \u000A returning a non-erroneous error triple even if the error flag was on.</p> \u000A \u000A <p>BINDING STATE GLOBAL VARIABLES</p> \u000A \u000A <p>In this section we introduce a utility, <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt>, that is \u000A an analogue of <v>let*</v> for state global variables.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((inhibit-output-lst (<see topic=\"ACL2____ADD-TO-SET-EQ\">add-to-set-eq</see> 'summary (<see topic=\"ACL2_____04\">@</see> inhibit-output-lst))))\u000A (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x)))</code> \u000A \u000A <p>This form binds state global variable <v>inhibit-output-lst</v> to the result \u000A of adding the symbol, <v>summary</v>, to the current value of that state global. \u000A Thus (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), the usual summary is not printed \u000A when evaluating this call of <tt><see topic=\"ACL2____THM\">thm</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> for more complete <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>INPUT AND OUTPUT</p> \u000A \u000A <p>In ACL2, most input and output involves the ACL2 state.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p>TIMINGS</p> \u000A \u000A <p>For how to obtain the time elapsed since the start of the ACL2 session, see \u000A <see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see>.</p> \u000A \u000A <p>For a utility for saving times into the ACL2 state and for printing those \u000A saved times, see the community book <v>misc/save-time.lisp</v>.</p> \u000A \u000A <p>To time an evaluation (though this really isn't about state), see <see topic=\"ACL2____TIME_42\">time$</see>.</p> \u000A \u000A <p>ENVIRONMENT AND SYSTEM</p> \u000A \u000A <p>Next, we mention briefly some ways in which ACL2 interacts with its \u000A environment using the ACL2 state.</p> \u000A \u000A <p>For how to read and write environment variables, see <see topic=\"ACL2____GETENV_42\">getenv$</see> and see \u000A <see topic=\"ACL2____SETENV_42\">setenv$</see>.</p> \u000A \u000A <p>For how to run a command in the host operating system, see <see topic=\"ACL2____SYS-CALL\">sys-call</see> and <see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see>.</p> \u000A \u000A <p>REMARKS ON EVENTS AND LD</p> \u000A \u000A <p>In general, undefined or surprising behavior may occur when using ACL2 \u000A <see topic=\"ACL2____EVENTS\">events</see> or calling <see topic=\"ACL2____LD\">ld</see> in your programs.  In some cases ACL2 \u000A enforces restrictions against these uses.  We strongly discourage using <tt><see topic=\"ACL2____LD\">ld</see></tt> in programs, as it has been designed to be called only at the top level of \u000A a read-eval-print loop.</p> \u000A \u000A <p>There is also a restriction on contexts in which <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> may be \u000A called: it may only be called in a context where an event is expected, such as \u000A the top level, in a book, or as an argument of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>.  The reason is that ACL2 does very subtle and careful tracking of \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansions; and it is only able to do this in event \u000A contexts, where it is able to carry out such tracking accurately.</p> \u000A \u000A <p>ADVANCED TOPICS</p> \u000A \u000A <p>ACL2 provides the function <v>trans-eval</v> to evaluate an arbitrary form \u000A (after translating it to a <see topic=\"ACL2____TERM\">term</see>, i.e., into internal form).  For more \u000A information, we refer to reader to comments in the definition of \u000A <v>trans-eval</v> in the ACL2 source code.  There are also many examples of its \u000A use in the ACL2 sources.</p> \u000A \u000A <p>For a function that provides the true absolute filename, with soft links \u000A resolved, see <see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see>.</p> \u000A \u000A <p>For a function that returns a check-sum on the characters in a channel, see \u000A <see topic=\"ACL2____CHECK-SUM\">check-sum</see>.</p> \u000A \u000A <p>To obtain a random number, see <see topic=\"ACL2____RANDOM_42\">random$</see>.</p> \u000A \u000A <p>If you are programming in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) or in raw \u000A Lisp, use the variable <v>*the-live-state*</v> in place of the variable \u000A <v>state</v>.</p> \u000A \u000A <p>We invite suggestions for additional advanced topics.</p>"],

 "ACL2____DOUBLE-REWRITE":[["Miscellaneous","Rewrite"],"ACL2 Sources","ACL2","<p>Logically, <v>double-rewrite</v> is the <tt><see topic=\"COMMON-LISP____IDENTITY\">identity</see></tt> function: \u000A <v>(<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x)</v> is equal to <v>x</v>.  However, the ACL2 rewriter treats \u000A calls of <v>double-rewrite</v> in the following special manner.  When it \u000A encounters a term <v>(<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> u)</v>, it first rewrites <v>u</v> in the \u000A current context, and then the rewriter rewrites the result.</p> \u000A \u000A <p>Such double-rewriting is rarely necessary, but it can be useful when \u000A rewriting under non-trivial equivalence relations (see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>). \u000A The following example will illustrate the issue.</p> \u000A \u000A <code>; Define an equivalence relation.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-equiv (x y)\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> my-equiv)\u000A\u000A; Define a unary function whose argument is preserved by my-equiv.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A  t)\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> my-equiv equal (foo x) 1)\u000A\u000A; Define some other unary functions.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h1 (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h2 (x) x)\u000A\u000A; Prove some lemmas and then disable the functions above.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-1\u000A  (my-equiv (h1 x) (h2 x)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2\u000A  (foo (h2 x)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-3\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (foo x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g x) x)))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)\u000A                           '(lemma-1 lemma-2 lemma-3\u000A                             my-equiv-implies-equal-foo-1)))\u000A\u000A; Attempt to prove a simple theorem that follows ``obviously'' from the\u000A; events above.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g (h1 a)) (h1 a)))</code> \u000A \u000A <p>We might expect the proof of this final <v>thm</v> to succeed by the \u000A following reasoning.  It is immediate from <v>lemma-3</v> provided we can \u000A establish <v>(foo (h1 a))</v>.  By the <v>defcong</v> event above, we know that \u000A <v>(foo (h1 a))</v> equals <v>(foo (h2 a))</v> provided <v>(my-equiv (h1 a) (h2\u000A a))</v>; but this is immediate from <v>lemma-1</v>.  And finally, <v>(foo (h2\u000A a))</v> is true by <v>lemma-2</v>.</p> \u000A \u000A <p>Unfortunately, the proof fails.  But fortunately, ACL2 gives the following \u000A useful warning when <v>lemma-3</v> is submitted:</p> \u000A \u000A <code>ACL2 Warning [Double-rewrite] in ( DEFTHM LEMMA-3 ...):  In the :REWRITE\u000Arule generated from LEMMA-3, equivalence relation MY-EQUIV is maintained\u000Aat one problematic occurrence of variable X in hypothesis (FOO X),\u000Abut not at any binding occurrence of X.  Consider replacing that occurrence\u000Aof X in this hypothesis with (<see topic=\"ACL2____DOUBLE-REWRITE\">DOUBLE-REWRITE</see> X).  See :doc double-\u000Arewrite for more information on this issue.</code> \u000A \u000A <p>We can follow the warning's advice by changing <v>lemma-3</v> to the \u000A following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-3\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (foo (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g x) x)))</code> \u000A \u000A <p>With this change, the proof succeeds for the final <v>thm</v> above.</p> \u000A \u000A <p>In practice, it should suffice for users to follow the advice given in the \u000A ``<v>Double-rewrite</v>'' warnings, by adding calls of <v>double-rewrite</v> \u000A around certain variable occurrences.  But this can cause inefficiency in large \u000A proof efforts.  For that reason, and for completeness, it seems prudent to \u000A explain more carefully what is going on; and that is what we do for the \u000A remainder of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.  Optionally, also see the paper \u000A ``Double Rewriting for Equivalential Reasoning in ACL2'' by Matt Kaufmann and \u000A J Strother Moore, in the proceedings of the 2006 ACL2 Workshop (paper is \u000A published in ACM Digital Library, <a \u000A href='http://portal.acm.org/toc.cfm?id=1217975'>http://portal.acm.org/toc.cfm?id=1217975</a>).</p> \u000A \u000A <p><b>Suggesting congruence rules.</b></p> \u000A \u000A <p>Sometimes the best way to respond to a ``<v>Double-rewrite</v>'' warning may \u000A be to prove a congruence rule.  Consider for example this rule.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> insert-sort-is-id\u000A  (perm (insert-sort x) x))</code> \u000A \u000A <p>Assuming that <v>perm</v> has been identified as an <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A relation (see <see topic=\"ACL2____DEFEQUIV\">defequiv</see>), we will get the following warning.</p> \u000A \u000A <code>ACL2 Warning [Double-rewrite] in ( DEFTHM INSERT-SORT-IS-ID ...):\u000AIn a :REWRITE rule generated from INSERT-SORT-IS-ID, equivalence relation\u000APERM is maintained at one problematic occurrence of variable X in the\u000Aright-hand side, but not at any binding occurrence of X.  Consider\u000Areplacing that occurrence of X in the right-hand side with\u000A(<see topic=\"ACL2____DOUBLE-REWRITE\">DOUBLE-REWRITE</see> X).  See :doc double-rewrite for more information on\u000Athis issue.</code> \u000A \u000A <p>The problem is that the second occurrence of <v>x</v> (the right-hand side of \u000A the rule <v>insert-sort-is-id</v>) is in a context where <v>perm</v> is to be \u000A maintained, yet in this example, the argument <v>x</v> of <v>insert-sort</v> on \u000A the left-hand side of that rule is in a context where <v>perm</v> will not be \u000A maintained.  This can lead one to consider the possibility that <v>perm</v> \u000A could be maintained in that left-hand side occurrence of <v>x</v>, and if so, to \u000A prove the following congruence rule.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONG\">defcong</see> perm perm (insert-sort x) 1)</code> \u000A \u000A <p>This will eliminate the above warning for <v>insert-sort-is-id</v>.  More \u000A important, this <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> event would probably be useful, since it would \u000A allow rewrite rules with equivalence relation <v>perm</v> to operate on the \u000A first argument of any call of <v>insert-sort</v> whose context calls for \u000A maintaining <v>perm</v>.</p> \u000A \u000A <p><b>Details on double-rewrite.</b></p> \u000A \u000A <p>The reader who wants these details may first wish to see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A for relevant review.</p> \u000A \u000A <p>The ACL2 rewriter takes a number of contextual arguments, including the \u000A generated equivalence relation being maintained (see <see topic=\"ACL2____CONGRUENCE\">congruence</see>) and an \u000A association list that maps variables to terms.  We call the latter alist the \u000A <v>unify-subst</v> because it is produced by unifying (actually matching) a \u000A pattern against a current term; let us explain this point by returning to the \u000A example above.  Consider what happens when the rewriter is given the top-level \u000A goal of the <v>thm</v> above.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g (h1 a)) (h1 a))</code> \u000A \u000A <p>This rewrite is performed with the empty alist (<v>unify-subst</v>), and is \u000A begun by rewriting the first argument (in that same empty \u000A <v>unify-subst</v>):</p> \u000A \u000A <code>(g (h1 a))</code> \u000A \u000A <p>Note that the only equivalence relation being maintained at this point is \u000A <v>equal</v>.  Now, the rewriter notices that the left-hand side of \u000A <v>lemma-3</v>, which is <v>(g x)</v>, matches <v>(g (h1 a))</v>.  The rewriter thus \u000A creates a <v>unify-subst</v> binding <v>x</v> to <v>(h1 a)</v>: <v>((x . (h1 a)))</v>. \u000A It now attempts to rewrite the hypothesis of <v>lemma-3</v> to <v>t</v> under this \u000A <v>unify-subst</v>.</p> \u000A \u000A <p>Consider what happens now if the hypothesis of <v>lemma-3</v> is <v>(foo\u000A x)</v>.  To rewrite this hypothesis under a <v>unify-subst</v> of <v>((x . (h1\u000A a)))</v>, it will first rewrite <v>x</v> under this <v>unify-subst</v>.  The key \u000A observation here is that this rewrite takes place simply by returning the \u000A value of <v>x</v> in the <v>unify-subst</v>, namely <v>(h1 a)</v>.  No further \u000A rewriting is done!  The efficiency of the ACL2 rewriter depends on such \u000A caching of previous rewriting results.</p> \u000A \u000A <p>But suppose that, instead, the hypothesis of <v>lemma-3</v> is <v>(foo\u000A (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x))</v>.  As before, the rewriter dives to the first argument of \u000A this call of <v>foo</v>.  But this time the rewriter sees the call \u000A <v>(<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x)</v>, which it handles as follows.  First, <v>x</v> is \u000A rewritten as before, yielding <v>(h1 a)</v>.  But now, because of the call of \u000A <v>double-rewrite</v>, the rewriter takes <v>(h1 a)</v> and rewrites it under the \u000A empty <v>unify-subst</v>.  What's more, because of the <v>defcong</v> event above, \u000A this rewrite takes place in a context where it suffices to maintain the \u000A equivalence relation <v>my-equiv</v>.  This allows for the application of \u000A <v>lemma-1</v>, hence <v>(h1 a)</v> is rewritten (under <v>unify-subst</v> = \u000A <v>nil</v>) to <v>(h2 a)</v>.  Popping back up, the rewriter will now rewrite the \u000A call of <v>foo</v> to <v>t</v> using <v>lemma-2</v>.</p> \u000A \u000A <p>The example above explains how the rewriter treats calls of \u000A <v>double-rewrite</v>, but it may leave the unfortunate impression that the user \u000A needs to consider each <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule \u000A carefully, just in case a call of <v>double-rewrite</v> may be appropriate. \u000A Fortunately, ACL2 provides a ``[Double-rewrite]'' warning to inform the user \u000A of just this sort of situation.  If you don't see this warning when you submit \u000A a (<v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>) rule, then the issue \u000A described here shouldn't come up for that rule.  Such warnings may appear for \u000A hypotheses or right-hand side of a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule, and for \u000A hypotheses or full conclusion (as opposed to just the trigger term) of a \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule.</p> \u000A \u000A <p>If you do see a ``[Double-rewrite]'' warning, then should you add the \u000A indicated call(s) of <v>double-rewrite</v>?  At the time of writing this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, the answer is not clear.  Early experiments with double \u000A rewriting suggested that it may be too expensive to call <v>double-rewrite</v> \u000A in every instance where a warning indicates that there could be an advantage \u000A to doing so.  And at the time of this writing, the ACL2 regression suite has \u000A about 1900 such warnings (but note that books were developed before \u000A <v>double-rewrite</v> or the ``[Double-rewrite]'' warning were implemented), \u000A which suggests that one can often do fine just ignoring such warnings. \u000A However, it seems advisable to go ahead and add the calls of \u000A <v>double-rewrite</v> indicated by the warnings unless you run across efficiency \u000A problems caused by doing so.  Of course, if you decide to ignore all such \u000A warnings you can execute the event:<br></br> \u000A \u000A <v>(</v><tt><see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see></tt><v> &quot;Double-rewrite&quot;)</v>.</p> \u000A \u000A <p>Finally, we note that it is generally not necessary to call \u000A <v>double-rewrite</v> in order to get its effect in the following case, where \u000A the discussion above might have led one to consider a call of \u000A <v>double-rewrite</v>: a hypothesis is a variable, or more generally, we are \u000A considering a variable occurrence that is a branch of the top-level <v>IF</v> \u000A structure of a hypothesis.  The automatic handling of this case, by a form of \u000A double rewriting, was instituted in ACL2 Version_2.9 and remains in place with \u000A the introduction of <v>double-rewrite</v>.  Here is a simple illustrative \u000A example.  Notice that <v>foo-holds</v> applies to prove the final <tt><see topic=\"ACL2____THM\">thm</see></tt> \u000A below, even without a call of <v>double-rewrite</v> in the hypothesis of \u000A <v>foo-holds</v>, and that there is no ``[Double-rewrite]'' warning when \u000A submitting <v>foo-holds</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((foo *) =&gt; *)\u000A  ((bar *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-holds\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> x\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) t)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> bar-holds-propositionally\u000A   (<see topic=\"ACL2____IFF\">iff</see> (bar x) t)))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (foo (bar y)))</code>"],

 "ACL2____CHARACTERS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>ACL2 accepts 256 distinct characters, which are the characters \u000A obtained by applying the function <tt><see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see></tt> to each integer from \u000A <v>0</v> to <v>255</v>.  Among these, Common Lisp designates certain ones as \u000A <i>standard characters</i>, namely those of the form <v>(<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> n)</v> where \u000A <v>n</v> is from <v>33</v> to <v>126</v>, together with <v>#\\Newline</v> and \u000A <v>#\\Space</v>.  The actual standard characters may be viewed by evaluating the \u000A <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> <v>*standard-chars*</v>.</p> \u000A \u000A <p>To be more precise, Common Lisp does not specify the precise relationship \u000A between <tt><see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see></tt> and the standard characters.  However, we check that \u000A the underlying Common Lisp implementation uses a particular relationship that \u000A extends the usual ASCII coding of characters.  We also check that Space, Tab, \u000A Newline, Page, and Rubout correspond to characters with respective <tt><see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see></tt>s <v>32</v>, <v>9</v>, <v>10</v>, <v>12</v>, and <v>127</v>.</p> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____CODE-CHAR\">Code-char</see></tt> has an inverse, <tt><see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see></tt>.  Thus, when <tt><see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see></tt> is applied to an ACL2 character, <v>c</v>, it returns a number <v>n</v> \u000A between <v>0</v> and <v>255</v> inclusive such that <v>(<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> n)</v> = \u000A <v>c</v>.</p> \u000A \u000A <p>The preceding paragraph implies that there is only one ACL2 character with \u000A a given character code.  CLTL allows for ``attributes'' for characters, which \u000A could allow distinct characters with the same code, but ACL2 does not allow \u000A this.</p> \u000A \u000A <p><i>The Character Reader</i></p> \u000A \u000A <p>ACL2 supports the `<v>#\\</v>' notation for characters provided by Common \u000A Lisp, with some restrictions.  First of all, for every character <v>c</v>, the \u000A notation</p> \u000A \u000A <code>#\\c</code> \u000A \u000A <p>may be used to denote the character object <v>c</v>.  That is, the user may \u000A type in this notation and ACL2 will read it as denoting the character object \u000A <v>c</v>.  In this case, the character immediately following <v>c</v> must be one \u000A of the following ``terminating characters'': a Tab, a Newline, a Page \u000A character, a space, or one of the characters:</p> \u000A \u000A <code>&quot;  '  (  )  ;  `  ,</code> \u000A \u000A <p>Other than the notation above, ACL2 accepts alternate notation for five \u000A characters.</p> \u000A \u000A <code>#\\Space\u000A#\\Tab\u000A#\\Newline\u000A#\\Page\u000A#\\Rubout</code> \u000A \u000A <p>Again, in each of these cases the next character must be from among the set \u000A of ``terminating characters'' described in the single-character case.  Our \u000A implementation is consistent with IS0-8859, even though we don't provide \u000A <v>#\\</v> syntax for entering characters other than that described above.</p> \u000A \u000A <p>Finally, we note that it is our intention that any object printed by ACL2's \u000A top-level-loop may be read back into ACL2.  Please notify the implementors if \u000A you find a counterexample to this claim.</p>"],

 "ACL2____WITH-PROVER-STEP-LIMIT":[["Other"],"ACL2 Sources","ACL2","<p>Logically, <v>(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> expr form)</v> is equivalent to \u000A <v>form</v>, except that if the number of ``prover steps'' executed during \u000A evaluation of <v>form</v> exceeds a bound specified by the value of <v>expr</v>, \u000A then that proof will abort.  See <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> for a related \u000A utility that sets the limit on prover steps globally instead of setting it for \u000A just one form, and for a discussion of the notion of ``prover steps'', which \u000A could change in future ACL2 releases.  For a related utility based on time \u000A instead of prover steps, see <see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>; but as discussed \u000A in the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <tt><see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see></tt>, there is at \u000A best a loose connection between the counting of steps and <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>.</p> \u000A \u000A <p>The arguments of <v>(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> expr form)</v> are evaluated. \u000A However, the (optional) argument <v>flg</v> is not evaluated in \u000A <v>(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> expr flg form)</v>.</p> \u000A \u000A <p>Depending on the machine you are using, you may have less than a half-hour \u000A of time before the number of prover steps exceeds the maximum limit on prover \u000A steps that may be imposed, which is one less than the value of \u000A <v>*default-step-limit*</v>.  But there is no limit unless you explicitly call \u000A <v>with-prover-step-limit</v> or <tt><see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see></tt>.</p> \u000A \u000A <p>For examples of how step-limits work besides those presented below, see the \u000A community book <v>books/misc/misc2/step-limits.lisp</v>.</p> \u000A \u000A <p>For a utility that returns an indicator of the number of prover steps most \u000A recently taken, see <see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see>.</p> \u000A \u000A <p>Note that <v>with-prover-step-limit</v> may not be called inside definitions, \u000A and that it is simply the identity macro in raw Lisp.  However, \u000A <v>with-prover-step-limit!</v> may be called in place of \u000A <v>with-prover-step-limit</v>, with the effect described here even in raw \u000A Lisp.</p> \u000A \u000A <code>Examples:\u000A\u000A; Limit (mini-proveall) to 100,000 prover steps (which happens to suffice)\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> 100000 (mini-proveall))\u000A\u000A; Same as above for the inner call of with-prover-step-limit; so the\u000A; mini-proveall call completes, but then we get an error because the second\u000A; argument of the outer with-prover-step-limit call took more than 200\u000A; steps.\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>\u000A 200\u000A (<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> 100000 (mini-proveall)))\u000A\u000A; Same as preceding form just above, except that this time there is no error,\u000A; because the inner call of with-prover-step-limit has an extra argument\u000A; of t inserted into the second argument position, which specifies that this\u000A; entire inner call is treated as though it uses no prover steps.\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>\u000A 200\u000A (<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> 100000 t (mini-proveall)))\u000A\u000A; The inner call limits (mini-proveall) to 200 prover steps, which fails\u000A; almost immediately.\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> 100000 (<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> 200 (mini-proveall)))\u000A\u000A; Do not limit the number of prover steps, regardless of such a limit imposed\u000A; globally or by the surrounding context.\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> nil (mini-proveall))\u000A\u000A; Same as just above (indeed, nil above is converted to\u000A; *default-step-limit*):\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> *default-step-limit* (mini-proveall))\u000A\u000A; Advanced example: Limit the indicated theorem to 100 steps, and when the\u000A; proof does not complete, then put down a label instead.\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        (<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>\u000A         100\u000A         (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x x) x)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">append</see> x x x))))\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> erp\u000A            (<see topic=\"ACL2____DEFLABEL\">deflabel</see> foo :doc &quot;Attempt failed.&quot;)\u000A          (value (<see topic=\"COMMON-LISP____LIST\">list</see> :succeeded-with val))))\u000A\u000A; Use extra argument of t to avoid ``charging'' steps for the indicated\u000A; form.\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>\u000A 500\u000A (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ()\u000A  (<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>\u000A   500\u000A   t ; Don't charge prover steps for this first defthm.\u000A   (<see topic=\"ACL2____DEFTHM\">defthm</see> test1\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))\u000A            (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z))\u000A     :rule-classes nil))\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> test2\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))\u000A           (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z))\u000A    :rule-classes nil)))\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> expr form)\u000A(<see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> expr flg form)</code> \u000A \u000A <p>where <v>form</v> is an arbitrary form to evaluate, and <v>expr</v> evaluates \u000A to one of the following: <v>nil</v>; a natural number not exceeding the value of \u000A <v>*default-step-limit*</v>; or the special value, <v>:START</v>.  The optional \u000A extra argument in the second position, <v>flg</v>, must be Boolean if \u000A supplied.</p> \u000A \u000A <p>If the value of <v>expr</v> is a natural number less than the value of \u000A <v>*default-step-limit*</v>, then that value is the maximum number of prover \u000A steps permitted during evaluation of <v>form</v> before an error occurs.  If \u000A however the value of <v>expr</v> is <v>nil</v> or is the value of \u000A <v>*default-step-limit*</v>, then no limit is placed on the number of prover \u000A steps during processing of <v>form</v>.  Otherwise, the value of <v>expr</v> \u000A should be the keyword <v>:START</v>, which is intended for use by the ACL2 \u000A implementation and may have semantics that change with new ACL2 versions.</p> \u000A \u000A <p>Finally we describe the optional extra Boolean argument, <v>flg</v>.  If \u000A <v>flg</v> is <v>nil</v> or omitted, then a running count of prover steps is \u000A maintained after <v>form</v> is evaluated; otherwise, that count is not affected \u000A by evaluation of <v>form</v>.  To see how this works when <v>flg</v> is nil or \u000A omitted, consider an event of shape <v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> form1 form2)</v>, where we are \u000A tracking prover steps (say, because of a superior call of \u000A <v>with-prover-step-limit</v>).  If <v>n</v> is the number of prover steps \u000A available when the <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> form is entered, and if <v>s</v> prover steps \u000A are executed while evaluating <v>form1</v>, then <v>n-s</v> steps are available \u000A for evaluation of <v>form2</v> provided <v>s</v> does not exceed <v>n</v>; \u000A otherwise, an error occurs.  In particular, this is the case if <v>form1</v> is \u000A an event <tt>(with-prover-step-limit k form1')</tt>.  However, if <v>form1</v> \u000A is an event <tt>(with-prover-step-limit k t form1')</tt>, then because of the \u000A extra argument of <v>t</v>, no steps are ``charged'' to <v>form</v>; that is, all \u000A <v>n</v> steps, rather than <v>n-s</v> steps, are available for evaluation of \u000A <v>form2</v>.</p>"],

 "ACL2____THEORY-INVARIANT":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> '(:rewrite left-to-right))\u000A                            (<see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> '(:rewrite right-to-left))))\u000A                  :key my-invariant\u000A                  :error nil)\u000A\u000A; Equivalent to the above:\u000A(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> (<see topic=\"ACL2____INCOMPATIBLE\">incompatible</see> (:rewrite left-to-right)\u000A                                (:rewrite right-to-left))\u000A                  :key my-invariant\u000A                  :error nil)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> term &amp;key key error)</code> \u000A \u000A <p>where:</p> \u000A \u000A <blockquote> \u000A \u000A <p>o <v>term</v> is a term that uses no variables other than <v>ens</v> and <tt><see topic=\"ACL2____STATE\">state</see></tt>;</p> \u000A \u000A <p>o <v>key</v> is an arbitrary ``name'' for this invariant (if omitted, an \u000A integer is generated and used); and</p> \u000A \u000A <p>o <v>:error</v> specifies the action to be taken when an invariant is \u000A violated &mdash; either <v>nil</v> if a warning is to be printed, else <v>t</v> \u000A (the default) if an error is to be caused.</p> \u000A \u000A </blockquote> \u000A \u000A <p><v>Theory-invariant</v> is an event that adds to or modifies the <see topic=\"ACL2____TABLE\">table</see> \u000A of user-supplied theory invariants that are checked each time a theory \u000A expression is evaluated.</p> \u000A \u000A <p>The theory invariant mechanism is provided via a table (see <see topic=\"ACL2____TABLE\">table</see>) \u000A named <v>theory-invariant-table</v>.  In fact, the <v>theory-invariant</v> \u000A ``event'' is just a macro that expands into a use of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> event. \u000A More general access to the <v>theory-invariant</v> <see topic=\"ACL2____TABLE\">table</see> is provided by \u000A <tt><see topic=\"ACL2____TABLE\">table</see></tt> itself.  For example, the <see topic=\"ACL2____TABLE\">table</see> can be inspected or \u000A cleared with <tt><see topic=\"ACL2____TABLE\">table</see></tt>; you can clear an individual theory invariant by \u000A setting the invariant to <v>t</v>, or eliminate all theory invariants with the \u000A command <v>(<see topic=\"ACL2____TABLE\">table</see> theory-invariant-table nil nil :clear)</v>.</p> \u000A \u000A <p><v>Theory-invariant-table</v> maps arbitrary keys to records containing terms \u000A that mention, at most, the variables <v>ens</v> and <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Every time \u000A an alleged theory expression is evaluated, e.g., in the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> \u000A event or <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, each of the terms in \u000A <v>theory-invariant-table</v> is evaluated with <v>ens</v> bound to a so-called \u000A ``enabled structure'' obtained from the theory expression and <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A bound to the current ACL2 state (see <see topic=\"ACL2____STATE\">state</see>).  Users generally need not \u000A know about the enabled structure, other than that it can be accessed using the \u000A macros <v>active-runep</v> and <v>incompatible</v>; see <see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> and \u000A see <see topic=\"ACL2____INCOMPATIBLE\">incompatible</see>.  If the result is <v>nil</v>, a message is printed and \u000A an error occurs (except, only a warning occurs if <v>:error nil</v> is \u000A specified).  Thus, the <see topic=\"ACL2____TABLE\">table</see> can be thought of as a list of conjuncts. \u000A Each <v>term</v> in the <see topic=\"ACL2____TABLE\">table</see> has a ``name,'' which is just the key under \u000A which the term is stored.  When a theory violates the restrictions specified \u000A by some term, both the name and the term are printed.  By calling \u000A <v>theory-invariant</v> with a new term but the same name, you can overwrite \u000A that conjunct of the theory invariant; but see the Local Redefinition Caveat \u000A at the end of this note.  You may want to avoid using explicit names, since \u000A otherwise the subsequent inclusion of another book that defines a theory \u000A invariant with the same name will override your theory invariant.</p> \u000A \u000A <p>Theory invariants are particularly useful in the context of large rule sets \u000A intended for re-use.  Such sets often contain conflicting rules, e.g., rules \u000A that are to be <see topic=\"ACL2____ENABLE\">enable</see>d when certain function symbols are <see topic=\"ACL2____DISABLE\">disable</see>d, rules that rewrite in opposite directions and thus loop if \u000A simultaneously <see topic=\"ACL2____ENABLE\">enable</see>d, groups of rules which should be <see topic=\"ACL2____ENABLE\">enable</see>d \u000A in concert, etc.  The developer of such rule sets understands these \u000A restrictions and probably documents them.  The theory invariant mechanism \u000A allows the developer to codify his restrictions so that the user is alerted \u000A when they are violated.</p> \u000A \u000A <p>Since theory invariants are arbitrary terms, macros may be used to express \u000A commonly used restrictions.  For example, executing the event</p> \u000A \u000A <code>(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> (<see topic=\"ACL2____INCOMPATIBLE\">incompatible</see> (:rewrite left-to-right)\u000A                                (:rewrite right-to-left)))</code> \u000A \u000A <p>would subsequently cause an error any time the current theory contained \u000A both of the two <see topic=\"ACL2____RUNE\">rune</see>s shown.  Of course, <see topic=\"ACL2____INCOMPATIBLE\">incompatible</see> is just \u000A defined as a macro.  Its definition may be inspected with <v>:pe\u000A incompatible</v>.</p> \u000A \u000A <p>In order for a <v>theory-invariant</v> event to be accepted, the proposed \u000A theory invariant must be satisfied by the current theory (see <see topic=\"ACL2____CURRENT-THEORY\">current-theory</see>).  The value returned upon successful execution of the event \u000A is the key (whether user-supplied or generated).</p> \u000A \u000A <p>Local Redefinition Caveat.  Care needs to be taken when redefining a theory \u000A invariant in a <see topic=\"ACL2____LOCAL\">local</see> context.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>\u000A (<see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> '(:definition binary-append))\u000A :key app-inv)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> t :key app-inv))\u000A (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> binary-append))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> ...))</code> \u000A \u000A <p>The second pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> will fail, because the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event violates the original <v>theory-invariant</v> and the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <v>theory-invariant</v> is skipped in the second pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Of course, <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt>s in <see topic=\"ACL2____BOOKS\">books</see> can cause the analogous problem in the second (<tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>) \u000A pass of a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  In both cases, though, the theory invariants \u000A are only checked at the conclusion of the (<v>include-book</v> or \u000A <v>encapsulate</v>) event.  Indeed, theory invariants are checked at the end of \u000A every event related to <see topic=\"ACL2____THEORIES\">theories</see>, including <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>, <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, \u000A except for events executed on behalf of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or the second \u000A pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p>"],

 "ACL2____SET-IPRINT":[["Io"],"ACL2 Sources","ACL2","<p>The following log may be sufficient for you to see how to use \u000A <v>set-iprint</v>; more explanation is below.  The example is taken from a \u000A session that used the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> utility, but familiarity with that \u000A utility is not necessary in order to understand this example.  What it shows \u000A is that the form <v>(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t)</v> allows you to recover, using <see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see>, output that had been hidden.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (p y))\u000A\u000A(1 Breaking (:REWRITE AX) on (P Y):\u000A1 ACL2 &gt;:eval\u000A\u000A1x (:REWRITE AX) failed because :HYP 1 rewrote to\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> Y '(NIL NIL NIL NIL ...)).\u000A\u000A1 ACL2 &gt;:a!\u000AAbort to ACL2 top-level\u000AHere is the current pstack [see :DOC pstack]:\u000A(REWRITE-ATM SIMPLIFY-CLAUSE WATERFALL)\u000A\u000A*** Note: No checkpoints to print. ***\u000A\u000AACL2 Version 6.4.  Level 1.  Cbd &quot;u/smith/&quot;.\u000ASystem books directory &quot;/u/smith/acl2/v6-4/books/&quot;.\u000AType :help for help.\u000AType (<see topic=\"ACL2____GOOD-BYE\">good-bye</see>) to quit completely out of ACL2.\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t)\u000A\u000AACL2 Observation in SET-IPRINT:  Iprinting has been enabled.\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (p y))\u000A\u000A(1 Breaking (:REWRITE AX) on (P Y):\u000A1 ACL2 &gt;:eval\u000A\u000A1x (:REWRITE AX) failed because :HYP 1 rewrote to\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> Y '(NIL NIL NIL NIL . #@2#)).\u000A\u000A1 ACL2 &gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> '(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> Y '(NIL NIL NIL NIL . #@2#)))\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> Y\u000A       '(NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A             NIL NIL NIL NIL NIL NIL NIL NIL NIL))\u000A1 ACL2 &gt;</code> \u000A \u000A <p>This concludes the introductory example.</p> \u000A \u000A <p>When ACL2 pretty-prints large expressions using formatted printing \u000A (see <see topic=\"ACL2____FMT\">fmt</see>), it may save time and space by printing tokens `<v>#</v>' or \u000A `<v>...</v>' in place of certain subexpressions.  By default this only happens \u000A for a few settings such as error and warning messages; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> for controlling such elision in general.  The full expression \u000A is unavailable to the user when `<v>#</v>' or `<v>...</v>' is printed, but that is \u000A easily solved by evaluating the form</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t)</code> \u000A \u000A <p>to enable a mode called ``iprinting''.  Then, instead of printing `<v>#</v>' \u000A or `<v>...</v>', ACL2 prints `<v>#@i#</v>' for <v>i</v> = 1,2,3,... &mdash; all in \u000A base 10.  ACL2 can read back in such `<v>#@i#</v>' because under the hood, \u000A <v>i</v> is associated with its corresponding elided form.  Thus the term \u000A ``iprint'' can be viewed as suggesting ``interactive print'' or ``index \u000A print''.  We also think of ``iprint'' as suggesting ``<v>i</v> printing'', to \u000A suggest the printing of token `<v>#@i#</v>'.  We call <v>i</v> the ``iprint \u000A index'' of that token.</p> \u000A \u000A <p>The following example should suffice to illustrate how to recover elided \u000A subexpressions.  (Below this example we provide details that may be of \u000A interest only to advanced users.)  Here we cause an error by defining a macro \u000A of one argument and then calling it with two arguments.  By default, error \u000A messages abbreviate subexpressions deeper than level 5 with `<v>#</v>' and past \u000A the 7th element of a list with `<v>...</v>'.  We see this behavior below.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (x) x)\u000A\u000ASummary\u000AForm:  ( DEFMACRO FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.02 seconds (prove: 0.00, print: 0.00, other: 0.02)\u000A FOO\u000AACL2 !&gt;(foo arg1 (a (b (c (d (e (f (g)))))) h i j k l m n))\u000A\u000AACL2 Error in macro expansion:  Wrong number of args in macro expansion\u000Aof (FOO ARG1 (A (B (C (D #))) H I J K L ...)).  (See :DOC set-iprint\u000Ato be able to see elided values in this message.)\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t)\u000A\u000AACL2 Observation in SET-IPRINT:  Iprinting has been enabled.\u000AACL2 !&gt;(foo arg1 (a (b (c (d (e (f (g)))))) h i j k l m n))\u000A\u000AACL2 Error in macro expansion:  Wrong number of args in macro expansion\u000Aof (FOO ARG1 (A (B (C (D #@1#))) H I J K L . #@2#)).\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> '(FOO ARG1 (A (B (C (D #@1#))) H I J K L . #@2#)))\u000A23\u000AACL2 !&gt;</code> \u000A \u000A <p>Sometimes you may want to supply the abbreviated form not to compute with \u000A it, as in the <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> example just above, but so that you can see \u000A it.  The macro <tt><see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see></tt> eliminates elision during printing. \u000A Below we show two ways to use this utility: first, we use it to print the \u000A elided form, and then, we use it instead on the original input form to print \u000A the entire error message.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> '(FOO ARG1 (A (B (C (D #@1#))) H I J K L . #@2#)))\u000A(FOO ARG1\u000A     (A (B (C (D (E (F (G))))))\u000A        H I J K L M N))\u000AACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> (foo arg1 (a (b (c (d (e (f (g)))))) h i j k l m n)))\u000A\u000AACL2 Error in macro expansion:  Wrong number of args in macro expansion\u000Aof (FOO ARG1 (A (B (C (D (E (F (G)))))) H I J K L M N)).\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>The documentation above probably suffices for most users.  For those who \u000A want more details, below we detail all the ways to use the <v>set-iprint</v> \u000A utility.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t)   ; enable iprinting (elision with #@i@)\u000A(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> nil) ; disable iprinting\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> action        ; t, nil, :reset, :reset-enable, or :same\u000A            :soft-bound s ; initially  1000\u000A            :hard-bound h ; initially 10000)</code> \u000A \u000A <p>where all arguments are optional, but ACL2 queries for <v>action</v> if it is \u000A omitted.  We defer the explanations of <v>:soft-bound</v> and <v>:hard-bound</v>. \u000A The values for <v>action</v> are as follows:</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>T</v> &mdash; Enable iprinting.</p> \u000A \u000A <p><v>NIL</v> &mdash; Disable iprinting.</p> \u000A \u000A <p><v>:RESET</v> &mdash; Reset iprinting to its initial disabled state, so that \u000A when enabled, the first index <v>i</v> for which `<v>#@i#</v> is printed will be \u000A 1.  Note that all stored information for existing iprint indices will be \u000A erased.</p> \u000A \u000A <p><v>:RESET-ENABLE</v> &mdash; Reset iprinting as with <v>:reset</v>, and then \u000A enable iprinting.</p> \u000A \u000A <p><v>:SAME</v> &mdash; Make no change to the iprinting state (other than \u000A setting the <v>:soft-bound</v> and/or <v>:hard-bound</v> if specified, as \u000A explained below).</p></blockquote> \u000A \u000A <p>Immediately after a top-level form is read, hence before it is evaluated, a \u000A check is made for whether the latest iprint index exceeds a certain bound, \u000A <v>(iprint-soft-bound state)</v> &mdash; 1000, by default.  If so, then the \u000A <v>(iprint-last-index state)</v> is set back to 0.  This soft bound can be \u000A changed to any positive integer <v>k</v> by calling <v>set-iprint</v> with \u000A <v>:soft-bound k</v>, typically <v>(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> :same :soft-bound k</v>)].</p> \u000A \u000A <p>The above ``soft bound'' is applied once for each top-level form, but you \u000A may want finer control by applying a bound after the pretty-printing of each \u000A individual form (since many forms may be pretty-printed between successive \u000A evaluations of top-level forms).  That bound is <v>(iprint-hard-bound\u000A state)</v>, and can be set with the <v>:hard-bound</v> argument in analogy to how \u000A <v>:soft-bound</v> is set, as described above.</p> \u000A \u000A <p>A ``rollover'' is the detection that the soft or hard bound has been \u000A exceeded, along with a state update so that the next iprint index will be 1. \u000A When a rollover occurs, any index beyond the latest iprint index is no longer \u000A available for reading.  At the top level of the ACL2 read-eval-print loop, \u000A this works as follows: ACL2 reads the next top-level form according to the \u000A current iprint state, then handles a rollover if the latest iprint index \u000A exceeded the current soft bound.  The following log illustrates a rollover, \u000A which follows the description above.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see> t :soft-bound 3)\u000A\u000AACL2 Observation in SET-IPRINT:  The soft-bound for iprinting has been\u000Aset to 3.\u000A\u000AACL2 Observation in SET-IPRINT:  Iprinting has been enabled.\u000AACL2 !&gt;(<see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 2 3 nil nil) :sites :ld)\u000A (:LD)\u000AACL2 !&gt;'((a b c d e f g) (a b c d e f g) (a b c d e f g))\u000A((A B C . #@1#) (A B C . #@2#) (A B C . #@3#))\u000AACL2 !&gt;'((a b c d e f g) (a b c d e f g) (a b c d e f g))\u000A((A B C . #@4#) (A B C . #@5#) (A B C . #@6#))\u000AACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> '((A B C . #@4#) (A B C . #@5#) (A B C . #@6#)))\u000A((A B C D E F G)\u000A (A B C D E F G)\u000A (A B C D E F G))\u000AACL2 !&gt;'(1 2 3 4 5)\u000A(1 2 3 . #@1#)\u000AACL2 !&gt;'((a b c d e f g) (a b c d e f g) (a b c d e f g))\u000A((A B C . #@2#) (A B C . #@3#) (A B C . #@4#))\u000AACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> '((A B C . #@4#) (A B C . #@5#) (A B C . #@6#)))\u000A((A B C D E F G)\u000A (A B C D E F G)\u000A (A B C D E F G))\u000AACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> '((A B C . #@4#) (A B C . #@5#) (A B C . #@6#)))\u000A\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  Out-of-bounds index in #@5#.  See :DOC set-iprint.\u000A***********************************************\u000A\u000AIf you didn't cause an explicit interrupt (Control-C),\u000Athen the root cause may be call of a :program mode\u000Afunction that has the wrong guard specified, or even no\u000Aguard specified (i.e., an implicit guard of t).\u000ASee :DOC guards.\u000A\u000ATo enable breaks into the debugger (also see :DOC acl2-customization):\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000AACL2 !&gt;</code> \u000A \u000A <p>We conclude by mentioning two cases where iprinting and evisc-tuples are \u000A ignored.  (1) This is typically the case when printing results in raw Lisp \u000A outside the ACL2 loop.  To use iprinting and evisc-tuples in raw Lisp, use \u000A raw-mode; see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>.  In raw-mode, results that are ACL2 objects \u000A will be printed in the same way that ACL2 would print those results if not in \u000A raw-mode.  (2) Iprinting and evisc-tuples are ignored by <tt><see topic=\"ACL2____PRINT-OBJECT_42\">print-object$</see></tt>, which however is sensitive to many settings that do not affect \u000A formatted (<tt><see topic=\"ACL2____FMT\">fmt</see></tt> etc.) printing; see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>.</p> \u000A \u000A <p>The reader interested in design and implementation issues considered during \u000A the addition of iprinting to ACL2 is invited to read the paper ``Abbreviated \u000A Output for Input in ACL2: An Implementation Case Study''; see the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2009/'>proceedings of \u000A ACL2 Workshop 2009</a>.</p>"],

 "ACL2____RULER-EXTENDERS":[["Defun"],"ACL2 Sources","ACL2","<p><b>Introduction</b></p> \u000A \u000A <p>Consider the following recursive definition, which returns a list of threes \u000A of length one more than the length of <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 3\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (f (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          nil)))</code> \u000A \u000A <p>One might expect ACL2's termination analysis to admit this function, since \u000A we know that <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> is ``smaller'' than <v>x</v> if <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> is \u000A true.  (By default, ACL2's notion of ``smaller'' is ordinary natural-number \u000A <v>&lt;</v>, and the argument <v>x</v> is measured by applying function \u000A <v>acl2-count</v> to <v>x</v>.)  However, that termination analysis does not \u000A consider <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> tests, like <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> above, when they occur under \u000A calls of functions other than <v>IF</v>, such as <v>CONS</v> in the case \u000A above.</p> \u000A \u000A <p>One way to overcome this problem is to ``lift'' the <v>IF</v> test to the top \u000A level, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 (f (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A    (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 nil)))</code> \u000A \u000A <p>But another way to overcome the problem is to tell ACL2 to extend its \u000A termination (and induction) analysis through calls of <v>cons</v>, as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :ruler-extenders (<see topic=\"COMMON-LISP____CONS\">cons</see>)))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 3\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (f (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          nil)))</code> \u000A \u000A <p>You may even wish to provide value <v>:all</v> instead of an explicit list of \u000A ruler-extenders, so that no function call blocks the termination analysis:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :ruler-extenders :all))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 3\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (f (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          nil)))</code> \u000A \u000A <p>Alternatively, you can omit the <v>XARGS</v> <v>:RULER-EXTENDERS</v> form, \u000A instead modifying the global default set of ruler-extenders:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :all)\u000A\u000A; or, for example:\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> '(<see topic=\"COMMON-LISP____CONS\">cons</see> return-last))</code> \u000A \u000A <p>You can call the function <tt><see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see></tt> as follows to see \u000A the current global default set of ruler-extenders:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see> (w state))</code> \u000A \u000A <p>We conclude this introduction by considering the handling of <v>LET</v> \u000A expressions by termination analysis.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____THE\">the</see> (integer 1 *)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> n)\u000A           (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))\u000A         1)))</code> \u000A \u000A <p>ACL2 treats the call of <tt><see topic=\"COMMON-LISP____THE\">the</see></tt> in the body of this definition as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((var (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> n)\u000A               (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))\u000A             1)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> var) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 1 var))\u000A      var\u000A    &lt;some_error&gt;))</code> \u000A \u000A <p>A <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expression, in turn, is treated as a <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> \u000A application:</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (var)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> var)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> var 1))\u000A         nil)\u000A       var\u000A     &lt;some_error&gt;))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> n)\u000A     (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))\u000A   1))</code> \u000A \u000A <p>Notice that the <tt><see topic=\"ACL2____POSP\">posp</see></tt> test, which governs the recursive call of \u000A <v>fact</v>, is inside an argument of a function application, namely the \u000A application of the <v>LAMBDA</v> expression.  So by default, ACL2 will not \u000A consider this <tt><see topic=\"ACL2____POSP\">posp</see></tt> test in its termination analysis.  The keyword \u000A <v>:LAMBDAS</v> in the list of ruler-extenders denotes all calls of lambda \u000A expressions, much as the inclusion of <v>CONS</v> in the ruler-extenders denotes \u000A all calls of <v>CONS</v>.  The following definition is thus accepted by \u000A ACL2.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :ruler-extenders (:lambdas)))\u000A  (<see topic=\"COMMON-LISP____THE\">the</see> (integer 1 *)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> n)\u000A           (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))\u000A         1)))</code> \u000A \u000A <p>As a convenience, ACL2 allows the symbol <v>:lambdas</v> in place of \u000A <v>(:lambdas)</v>, and in fact the former will also include the default \u000A ruler-extenders: <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> (which comes from macroexpansion of calls \u000A of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, and others) and <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt>.</p> \u000A \u000A <p>IMPORTANT REMARKS.  (1) Notice that the argument to \u000A <v>set-ruler-extenders</v> is evaluated, but the argument to \u000A <v>:RULER-EXTENDERS</v> in <v>XARGS</v> is not evaluated.  (2) Do not put macro \u000A names in your list of ruler-extenders.  For example, if you intend that <v>+</v> \u000A should not block the termination analysis, in analogy to <v>cons</v> in the \u000A example above, then the list of ruler-extenders should include <v>binary-+</v>, \u000A not <v>+</v>.  Of course, if you use <v>:all</v> then this is not an issue, but \u000A see the next remark.  (3) Also please note that by taking advantage of the \u000A ruler-extenders, you may be complicating the induction scheme stored for the \u000A function, whose computation takes similar advantage of the additional <v>IF</v> \u000A structure that you are specifying.</p> \u000A \u000A <p>Below we describe the notion of ruler-extenders in detail, as well as how \u000A to set its default using <v>set-ruler-extenders</v>.</p> \u000A \u000A <p><b>Details</b></p> \u000A \u000A <p>We begin by discussing how to set the ruler-extenders by using the macro \u000A <v>set-ruler-extenders</v>; below we will discuss the use of keyword \u000A <v>:ruler-extenders</v> in <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :basic) ; return to default\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> *basic-ruler-extenders*) ; same as immediately above\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :all) ; every governing IF test rules a recursive call\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :lambdas) ; LET does not block termination analysis\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> :lambdas *basic-ruler-extenders*))\u000A                               ; same as immediately above\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> '(f g)) ; termination analysis goes past calls of f, g\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> val)</code> \u000A \u000A <p>where <v>val</v> evaluates to one of <v>:basic</v>, <v>:all</v>, <v>:lambdas</v>, \u000A or a true list of symbols containing no keyword other than, optionally, \u000A <v>:lambdas</v>.</p> \u000A \u000A <p>When a recursive definition is submitted to ACL2 (in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A mode), the recursion must be proved to terminate; see <see topic=\"COMMON-LISP____DEFUN\">defun</see>.  More \u000A precisely, ACL2 explores the <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> structure of the body of the \u000A definition to accumulate the tests that ``rule'' any given recursive call. \u000A The following example reviews how this works.  Suppose that <v>f</v> has already \u000A been defined.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> y))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (g (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)\u000A        (f (g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)))\u000A      (f (<see topic=\"COMMON-LISP____LIST\">list</see> x y)))))</code> \u000A \u000A <p>ACL2 makes the following response to this proposed definition.  Notice that \u000A the <v>:measure</v> proposed above must be proved to be an ACL2 ordinal &mdash; \u000A that is, to satisfy <v>O-P</v> &mdash; and that the arguments to each recursive \u000A call must be smaller (in the sense of that measure and <v>O&lt;</v>, which here \u000A reduces to the ordinary <v>&lt;</v> relation) than the formals under the assumption \u000A of the ruling <v>IF</v> tests.  The first <v>IMPLIES</v> term below thus \u000A corresponds to the recursive call <v>(g (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)</v>, while the second \u000A corresponds to the recursive call <v>(g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))</v>.</p> \u000A \u000A <code>For the admission of G we will use the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> Y)).  The non-trivial part of the measure\u000Aconjecture is\u000A\u000AGoal\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____O-P\">O-P</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> Y)))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"ACL2____O_C3\">O&lt;</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> Y))\u000A                  (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> Y))))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)) (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y))\u000A              (<see topic=\"ACL2____O_C3\">O&lt;</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> Y)))\u000A                  (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> Y))))).</code> \u000A \u000A <p>Now consider the following alternate version of the above definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> y))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (g (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)\u000A    (f (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)\u000A           (g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> x y)))))</code> \u000A \u000A <p>The first test, <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v>, still rules the first recursive call, <v>(g\u000A (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)</v>.  And the negation of that test, namely <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))</v>, \u000A still rules the second recursive call <v>(g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))</v>.  But the call of \u000A <v>f</v> blocks the top-down exploration of the <v>IF</v> structure of the body of \u000A <v>g</v>, so <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> y)</v> does not rule that second recursive call, which \u000A (again) is <v>(g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))</v>.  As a result, ACL2 fails to admit the above \u000A definition.</p> \u000A \u000A <p><v>Set-ruler-extenders</v> is provided to overcome the sort of blocking \u000A described above.  Suppose for example that the following event is \u000A submitted:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> '(f))</code> \u000A \u000A <p>Then the alternate definition of <v>g</v> above is admissible, because the \u000A call of <v>f</v> no longer blocks the top-down exploration of the <v>IF</v> \u000A structure of the body of <v>g</v>: that is, <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> y)</v> becomes a ruler of \u000A the recursive call <v>(g x (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))</v>.  In this case, we say that <v>f</v> is a \u000A ``ruler-extender''.  The same result obtains if we first submit</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :all)</code> \u000A \u000A <p>as this removes all function calls as blockers of the top-down analysis. \u000A In other words, with <v>:all</v> it is the case that for every recursive call, \u000A every test argument of a superior call of <v>IF</v> contributes a ruler of that \u000A recursive call.</p> \u000A \u000A <p>ACL2 handles <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> (and <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>) expressions by translating them \u000A to <v>LAMBDA</v> expressions (see <see topic=\"ACL2____TERM\">term</see>).  The next examples illustrates \u000A termination analysis involving such expressions.  First consider the following \u000A (admittedly inefficient) definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((k (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> n) n 0)))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> k 0)\u000A        1\u000A      (<see topic=\"COMMON-LISP_____A2\">*</see> k (fact (<see topic=\"COMMON-LISP_____B2\">+</see> -1 k))))))</code> \u000A \u000A <p>ACL2 translates the body of this definition to a <v>LAMBDA</v> application, \u000A essentially:</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (k)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> k 0)\u000A       1\u000A     (<see topic=\"COMMON-LISP_____A2\">*</see> k (fact (<see topic=\"COMMON-LISP_____B2\">+</see> -1 k)))))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> n) n 0))</code> \u000A \u000A <p>As with the application of any function other than <v>IF</v>, the top-down \u000A termination analysis does not dive into arguments: the <v>LAMBDA</v> blocks the \u000A continuation of the analysis into its argument.  But here, the argument of the \u000A <v>LAMBDA</v> is <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> n) n 0)</v>, which has no recursive calls to \u000A consider anyhow.  What is more interesting: ACL2 does continue its termination \u000A analysis into the body of the <v>LAMBDA</v>, in an environment binding the \u000A <v>LAMBDA</v> formals to its actuals.  In this case, the termination analysis \u000A thus continues into the term</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> k 0)\u000A    1\u000A  (<see topic=\"COMMON-LISP_____A2\">*</see> k (fact (<see topic=\"COMMON-LISP_____B2\">+</see> -1 k))))</code> \u000A \u000A <p>in the environment that binds <v>k</v> to the term <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> n) n 0)</v>. \u000A Thus, the proof obligation is successfully discharged, as reported by \u000A ACL2:</p> \u000A \u000A <code>For the admission of FACT we will use the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> N).  The non-trivial part of the measure conjecture is\u000A\u000AGoal\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"ACL2____NATP\">NATP</see> N) N 0) 0))\u000A         (<see topic=\"ACL2____O_C3\">O&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (<see topic=\"COMMON-LISP_____B2\">+</see> -1 (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"ACL2____NATP\">NATP</see> N) N 0)))\u000A             (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> N))).\u000A.....\u000AQ.E.D.\u000A\u000AThat completes the proof of the measure theorem for FACT.</code> \u000A \u000A <p>But now consider the following definition, in which the recursion takes \u000A place inside the argument of the <v>LAMBDA</v> rather than inside the \u000A <v>LAMBDA</v> body.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((result (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A                    y\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                        (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (our-test result)\u000A        result\u000A      0)))</code> \u000A \u000A <p>Writing the body in <v>LAMBDA</v> notation:</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (result)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (our-test result)\u000A       result\u000A     0))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A     y\u000A   (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A         (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))</code> \u000A \u000A <p>By default, the <v>LAMBDA</v> call blocks the top-down termination analysis \u000A from proceeding into the term <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x) ...)</v>.  To solve this, one can \u000A submit the event:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> :lambdas)</code> \u000A \u000A <p>The above definition of <v>app</v> is then admitted by ACL2, because the \u000A termination analysis is no longer blocked by the <v>LAMBDA</v> call.</p> \u000A \u000A <p>The example just above illustrates that the heuristically-chosen measure is \u000A suitably sensitive to the ruler-extenders.  Specifically: that measure is the \u000A application of <v>acl2-count</v> to the first formal parameter of the function \u000A that is tested along every branch of the relevant <v>IF</v> structure (as \u000A determined by the rulers) and occurs as a proper subterm at the same argument \u000A position in every recursive call.  The heuristics for choosing the \u000A controller-alist for a <tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule are similarly sensitive to the \u000A ruler-extenders (see <see topic=\"ACL2____DEFINITION\">definition</see>).</p> \u000A \u000A <p>The remarks above for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are equally applicable \u000A when a definition sits inside a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event, except of \u000A course that in this case, a ``recursive call'' is a call of any function being \u000A defined by that <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> are sensitive to \u000A <v>set-ruler-extenders</v> in analogy to the case of <v>defun</v> <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>This macro generates a call <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :ruler-extenders\u000A val)</v> and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs. See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  The current \u000A list of ruler-extenders may be obtained as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> :ruler-extenders\u000A     (table-alist 'acl2-defaults-table (w state))))</code> \u000A \u000A <p>or more conveniently, as:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see> (w state))</code> \u000A \u000A <p>Note that evaluation of <v>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> lst)</v>, where <v>lst</v> \u000A evaluates to a list, does not necessarily include the default ruler-extenders \u000A &mdash; i.e., those included for the argument, <v>:basic</v> &mdash; which are \u000A the elements of the list constant <v>*basic-ruler-extenders*</v>, namely <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> and <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt>.  You may, of course, include these \u000A explicitly in your list argument.</p> \u000A \u000A <p>We conclude our discussion by noting that the set of ruler-extenders can \u000A affect the induction scheme that is stored with a recursive definition.  The \u000A community book <v>books/misc/misc2/ruler-extenders-tests.lisp</v> explains how \u000A induction schemes are derived in this case.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> tree-of-nils-p (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (tree-of-nils-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A           (tree-of-nils-p (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A    (<see topic=\"COMMON-LISP____NULL\">null</see> x)))</code> \u000A \u000A <p>The above definition generates the following induction scheme.  Note that \u000A <v>(<see topic=\"COMMON-LISP____AND\">and</see> u v)</v> expands to <v>(<see topic=\"COMMON-LISP____IF\">if</see> u v nil)</v>, which explains why the term \u000A <v>(tree-of-nils-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))</v> rules the recursive call <v>(tree-of-nils-p (<see topic=\"COMMON-LISP____CDR\">cdr</see>\u000A x))</v>, resulting in the hypothesis <v>(tree-of-nils-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))</v> in the final \u000A conjunct below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)) (:P X))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                   (<see topic=\"COMMON-LISP____NOT\">NOT</see> (TREE-OF-NILS-P (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A                   (:P (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A              (:P X))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                   (TREE-OF-NILS-P (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))\u000A                   (:P (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))\u000A                   (:P (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A              (:P X)))</code> \u000A \u000A <p>Now consider the following variant of the above definition, in which a call \u000A of the function <v>identity</v> blocks the termination analysis.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> tree-of-nils-p (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____IDENTITY\">identity</see> (<see topic=\"COMMON-LISP____AND\">and</see> (tree-of-nils-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (tree-of-nils-p (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))\u000A    (<see topic=\"COMMON-LISP____NULL\">null</see> x)))</code> \u000A \u000A <p>This time the induction scheme is as follows, since only the top-level \u000A <v>IF</v> test contributes rulers to the termination analysis.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)) (:P X))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                   (:P (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))\u000A                   (:P (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A              (:P X)))</code> \u000A \u000A <p>But now suppose we first designate <v>identity</v> as a ruler-extender.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-RULER-EXTENDERS\">set-ruler-extenders</see> '(<see topic=\"COMMON-LISP____IDENTITY\">identity</see>))</code> \u000A \u000A <p>Then the induction scheme generated for the both of the above variants of \u000A <v>tree-of-nils-p</v> is the one shown for the first variant, which is \u000A reasonable because both definitions now produce essentially the same \u000A termination analysis.</p>"],

 "ACL2____PC":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pc 3    ; print the third command executed\u000A:pc :max ; print the most recent command\u000A:pc :x   ; print the most recent command\u000A:pc fn   ; print the command that introduced fn</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Pc</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and prints the \u000A <see topic=\"ACL2____COMMAND\">command</see> identified by that descriptor.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>. \u000A For example</p> \u000A \u000A <code>ACL2 !&gt;:pc foo\u000A LVd     52 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)</code> \u000A \u000A <p><v>Pc</v> always prints a space first, followed by three (possibly blank) \u000A <see topic=\"ACL2____CHARACTERS\">characters</see> (``LVd'' above) explained below (four, in the experimental \u000A HONS version, as discussed further below).  Then <v>pc</v> prints the <see topic=\"ACL2____COMMAND\">command</see> number, a number uniquely identifying the <see topic=\"ACL2____COMMAND\">command</see>'s position \u000A in the sequence of <see topic=\"ACL2____COMMAND\">command</see>s since the beginning of the user's session. \u000A Finally, the <see topic=\"ACL2____COMMAND\">command</see> itself is printed.</p> \u000A \u000A <p>While <v>pc</v> always prints a space first, some <see topic=\"ACL2____HISTORY\">history</see> <see topic=\"ACL2____COMMAND\">command</see>s, for example <v>:</v><tt><see topic=\"ACL2____PCS\">pcs</see></tt> and <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>, use the first \u000A column of output to delimit a region of <see topic=\"ACL2____COMMAND\">command</see>s or to point to a \u000A particular event within a <see topic=\"ACL2____COMMAND\">command</see>.</p> \u000A \u000A <p>For example, <v>:pcs 52 54</v> will print something like</p> \u000A \u000A <code>/LVd     52 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)\u000A LV      53 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> BAR (X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> X X))\u000A\\        54 (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> FOO-BAR (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (BAR X)) (FOO X)))\u000A          : ...\u000A        127 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> LATEST (X) X)</code> \u000A \u000A <p>Here, the two slash <see topic=\"ACL2____CHARACTERS\">characters</see> in the first column are intended to \u000A suggest a bracket delimiting <see topic=\"ACL2____COMMAND\">command</see>s 52 through 54.  The last <see topic=\"ACL2____COMMAND\">command</see> printed by <tt><see topic=\"ACL2____PCS\">pcs</see></tt> is always the most recent <see topic=\"ACL2____COMMAND\">command</see>, \u000A i.e., the <see topic=\"ACL2____COMMAND\">command</see> at <v>:here</v>, and is separated from the rest of the \u000A display by an elipsis if some <see topic=\"ACL2____COMMAND\">command</see>s are omitted.</p> \u000A \u000A <p>Similarly, the <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> <see topic=\"ACL2____COMMAND\">command</see> will print a particular \u000A event within a <see topic=\"ACL2____COMMAND\">command</see> block and will indicate that event by printing a \u000A ``<tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt>'' in the first column.  The symbol is intended to be an arrow \u000A pointing at the event in question.</p> \u000A \u000A <p>For example, <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> <v>true-listp-app</v> might print:</p> \u000A \u000A <code>         1 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;list-book&quot;)\u000A            \\\u000A&gt;           (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> TRUE-LISTP-APP\u000A                    (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (APP A B)) (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> B)))</code> \u000A \u000A <p>using the arrow to indicate the event itself.  The slash printed to connect \u000A the <see topic=\"ACL2____COMMAND\">command</see>, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, with the event, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, is \u000A intended to suggest a tree branch indicating that the event is inferior to \u000A (and part of) the <see topic=\"ACL2____COMMAND\">command</see>.</p> \u000A \u000A <p>The mysterious <see topic=\"ACL2____CHARACTERS\">characters</see> sometimes preceding a <see topic=\"ACL2____COMMAND\">command</see> have \u000A the following interpretations.  The first two have to do with the function \u000A symbols introduced by the <see topic=\"ACL2____COMMAND\">command</see> and are blank if no symbols were \u000A introduced.</p> \u000A \u000A <p>At any time we can classify our function symbols into disjoint sets, which \u000A we will here name with <see topic=\"ACL2____CHARACTERS\">characters</see>.  The ``<v>P</v>'' functions are those \u000A in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  The ``<v>L</v>'' functions are those in \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode whose <see topic=\"ACL2____GUARD\">guard</see>s have not been verified.  The \u000A ``<v>V</v>'' functions are those in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode whose <see topic=\"ACL2____GUARD\">guard</see>s \u000A have been verified.  You may also see the use of (lower-case) ``<v>v</v>'' to \u000A indicate functions introduced by <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Note that <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> and <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> cause function symbols to be \u000A reclassified.  If a <see topic=\"ACL2____COMMAND\">command</see> introduces function symbols then the first \u000A mysterious character indicates the class of the symbols at the time of \u000A introduction and the second character indicates the current class of the \u000A symbols (if the current class is different from the introductory class).</p> \u000A \u000A <p>Thus, the display</p> \u000A \u000A <code>PLd     52 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)</code> \u000A \u000A <p>tells us that <see topic=\"ACL2____COMMAND\">command</see> 52 introduced a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> function \u000A but that some <see topic=\"ACL2____COMMAND\">command</see> after 52 changed its mode to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A and that the <see topic=\"ACL2____GUARD\">guard</see>s of <v>foo</v> have not been verified.  That is, \u000A <v>foo</v>'s termination has been verified even though it was not verified as \u000A part of the <see topic=\"ACL2____COMMAND\">command</see> that introduced <v>foo</v>.  Had a subsequent <see topic=\"ACL2____COMMAND\">command</see> verified the <see topic=\"ACL2____GUARD\">guard</see>s of <v>foo</v>, the display would contain a \u000A <v>V</v> where the <v>L</v> is.</p> \u000A \u000A <p>The display</p> \u000A \u000A <code>P d     52 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)</code> \u000A \u000A <p>indicates that <v>foo</v> was introduced in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and \u000A still is in that mode.</p> \u000A \u000A <p>The third character indicates the enabled/disabled status of the <see topic=\"ACL2____RUNE\">rune</see>s introduced by the <see topic=\"ACL2____COMMAND\">command</see>.  If the status character is blank \u000A then all the <see topic=\"ACL2____RUNE\">rune</see>s (if any) introduced are <see topic=\"ACL2____ENABLE\">enable</see>d.  If the \u000A status character is ``<v>D</v>'' then some <see topic=\"ACL2____RUNE\">rune</see>s were introduced and they \u000A are all <see topic=\"ACL2____DISABLE\">disable</see>d.  If the status character is ``<v>d</v>'' then at least \u000A one, but not all, of the <see topic=\"ACL2____RUNE\">rune</see>s introduced is <see topic=\"ACL2____DISABLE\">disable</see>d.  Thus, in \u000A the display</p> \u000A \u000A <code>L d     52 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO (X) X)</code> \u000A \u000A <p>we see that some <see topic=\"ACL2____RUNE\">rune</see> introduced by <see topic=\"ACL2____COMMAND\">command</see> 52 is <see topic=\"ACL2____DISABLE\">disable</see>d.  As noted in the documentation for <see topic=\"ACL2____RUNE\">rune</see>, a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A <see topic=\"ACL2____COMMAND\">command</see> introduces many <see topic=\"ACL2____RUNE\">rune</see>s, e.g., the axiomatic definition \u000A rule, <v>(:definition fn)</v>, the executable counterpart rule, \u000A <v>(:executable-counterpart fn)</v>, and <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>s, \u000A <v>(:type-prescription fn)</v>.  The display above does not say which of the \u000A <see topic=\"ACL2____RUNE\">rune</see>s based on <v>foo</v> is <see topic=\"ACL2____DISABLE\">disable</see>d, but it does tell us one of \u000A them is; see <see topic=\"ACL2____DISABLEDP\">disabledp</see> for how to obtain the disabled runes for a given \u000A function symbol.</p> \u000A \u000A <p>Finally, for the experimental HONS version only (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>), a fourth character is printed, indicating whether \u000A functions are memoized.  A symbol may be memoized if it is a function symbol \u000A that is not constrained (i.e., introduced by <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> or in the <see topic=\"ACL2____SIGNATURE\">signature</see> of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event).  If the command introduces no \u000A symbol that may be memoized, then a space is printed.  Otherwise, if every \u000A memoizable symbol is memoized, an ``<v>M</v>'' is printed.  Otherwise, an \u000A ``<v>m</v>'' is printed.</p>"],

 "ACL2____BRR":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example:\u000A:brr t       ; enable\u000A:brr nil     ; disable\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____BRR\">brr</see> flg)</code> \u000A \u000A <p>where <v>flg</v> evaluates to <v>t</v> or <v>nil</v>.  This function modifies \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> so that the attempted application of certain rewrite rules are \u000A ``broken.'' ``<v>Brr</v>'' stands for ``break-rewrite'' and can be thought of as \u000A a mode with two settings.  The normal mode is ``disabled.''</p> \u000A \u000A <p>For a more thorough introduction to the break rewrite system see <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>.</p> \u000A \u000A <p>When <v>brr</v> mode is ``enabled'' the ACL2 rewriter monitors the attempts \u000A to apply certain rules and advises the user of those attempts by entering an \u000A interactive wormhole break.  From within this break the user can watch \u000A selected application attempts.  The user can also interact with the system \u000A during <v>brr</v> breaks via <tt><see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see></tt>.</p> \u000A \u000A <p>The rules monitored are selected by using the <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> and <tt><see topic=\"ACL2____UNMONITOR\">unmonitor</see></tt> commands.  It is possible to break a rune ``conditionally'' in the \u000A sense that an interactive break will occur only if a specified predicate is \u000A true of the environment at the time of the attempted application.  See <see topic=\"ACL2____MONITOR\">monitor</see> and see <see topic=\"ACL2____UNMONITOR\">unmonitor</see>.</p> \u000A \u000A <p>Even if a non-empty set of rules has been selected, no breaks will occur \u000A unless <v>brr</v> mode is enabled.  Thus, the first time in a session that you \u000A wish to monitor a rewrite rule, use <v>:brr</v> <v>t</v> to enable <v>brr</v> mode. \u000A Thereafter you may select runes to be monitored with <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> and \u000A <tt><see topic=\"ACL2____UNMONITOR\">unmonitor</see></tt> with the effect that whenever monitored rules are tried (and \u000A their break conditions are met) an interactive break will occur.  Be advised \u000A that when <v>brr</v> mode is enabled the rewriter is somewhat slower than \u000A normal.  Furthermore, that sluggishness persists even if no runes are \u000A monitored.  You may regain normal performance &mdash; regardless of what runes \u000A are monitored &mdash; by disabling <v>brr</v> mode with <v>:brr</v> <v>nil</v>.</p> \u000A \u000A <p>Why isn't <v>brr</v> mode disabled automatically when no runes are monitored? \u000A More generally, why does ACL2 have <v>brr</v> mode at all?  Why not just test \u000A whether there are monitored runes?  If you care about the answers, see <see topic=\"ACL2____WHY-BRR\">why-brr</see>.</p> \u000A \u000A <p>BRR Mode and Console Interrupts: If the system is operating in <v>brr</v> \u000A mode and you break into raw Lisp (as by causing a console interrupt or \u000A happening upon a signalled Lisp error; see <see topic=\"ACL2____BREAKS\">breaks</see>), you can return to \u000A the ACL2 top-level, outside any <v>brr</v> environment, by executing \u000A <v>(</v><tt><see topic=\"ACL2____ABORT_12\">abort!</see></tt><v>)</v>.  Otherwise, the normal way to quit from such a \u000A break (for example <v>:q</v> in GCL, <v>:reset</v> in Allegro CL, and <v>q</v> in \u000A CMU CL) will return to the innermost ACL2 read-eval-print loop, which may or \u000A may not be the top-level of your ACL2 session!  In particular, if the break \u000A happens to occur while ACL2 is within the <v>brr</v> environment (in which it is \u000A preparing to read <tt><see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see></tt>), the abort will merely return to that \u000A <v>brr</v> environment.  Upon exiting that environment, normal theorem proving \u000A is continued (and the <v>brr</v> environment may be entered again in response to \u000A subsequent monitored rule applications).  Before returning to the <v>brr</v> \u000A environment, ACL2 ``cleans up'' from the interrupted <v>brr</v> processing. \u000A However, it is not possible (given the current implementation) to clean up \u000A perfectly.  This may have two side-effects.  First, the system may \u000A occasionally print the self-explanatory ``Cryptic BRR Message 1'' (or 2), \u000A informing you that the system has attempted to recover from an aborted \u000A <v>brr</v> environment.  Second, it is possible that subsequent <v>brr</v> \u000A behavior in that proof will be erroneous because the cleanup was done \u000A incorrectly.  The moral is that you should not trust what you learn from \u000A <v>brr</v> if you have interrupted and aborted <v>brr</v> processing during the \u000A proof.  These issues do not affect the behavior or soundness of the theorem \u000A prover.</p>"],

 "ACL2____A_02Tiny_02Warning_02Sign":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='warning.gif'></img></p> \u000A \u000A <p>This warning sign, which usually appears as ``<icon src='twarning.gif'/>'', \u000A indicates that the link it marks takes you into ACL2's online \u000A documentation.</p> \u000A \u000A <p>The documentation is a vast graph of documented topics intended to help the \u000A <i>user</i> of ACL2 rather than the <i>potential user</i>.  If you are \u000A exploring ACL2's home page to learn about the system, perhaps you should go \u000A back rather than follow the link marked with this sign.  But you are welcome \u000A to explore the online documentation as well.  Good luck.</p>"],

 "ACL2____WITH-PROVER-TIME-LIMIT":[["Other"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A; Limit (mini-proveall) to about 1/4 second:\u000A(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> 1/4 (mini-proveall))\u000A\u000A; Limit (mini-proveall) to about 1/4 second, even if surrounding call of\u000A; with-prover-time-limit provides for a more restrictive bound:\u000A(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> '(1/4) (mini-proveall))\u000A\u000A; Limit the indicated theorem to about 1/50 second, and if the proof does not\u000A; complete or it fails, then put down a label instead.\u000A(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        (<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>\u000A         1/50\u000A         (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x x) x)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">append</see> x x x))))\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> erp\u000A            (<see topic=\"ACL2____DEFLABEL\">deflabel</see> foo :doc &quot;Attempt failed.&quot;)\u000A          (value (<see topic=\"COMMON-LISP____LIST\">list</see> :succeeded-with val))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> time form)</code> \u000A \u000A <p>where <v>time</v> evaluates to a positive rational number or to a list \u000A containing such, and <v>form</v> is arbitrary.  Logically, \u000A <v>(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> time form)</v> is equivalent to <v>form</v>.  However, \u000A if the time for evaluation of <v>form</v> exceeds the value specified by \u000A <v>time</v>, and if ACL2 notices this fact during a proof, then that proof will \u000A abort, for example like this:</p> \u000A \u000A <code>ACL2 Error in ( DEFTHM PERM-REFLEXIVE ...):  Out of time in the rewriter.</code> \u000A \u000A <p>If there is already a surrounding call of <v>with-prover-time-limit</v> that \u000A has set up an expiration time, the inner <v>with-prover-time-limit</v> call is \u000A not allowed to push that time further into the future unless the inner time is \u000A specified as a list containing a rational, rather than as a rational.</p> \u000A \u000A <p>Note that by default, the time used is runtime (cpu time); to switch to \u000A realtime (elapsed time), see <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.</p> \u000A \u000A <p>For a related utility based on prover steps instead of time, see <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>; also see <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see>.  Those \u000A utilities have the advantage of having platform-independent behavior, unlike \u000A time limits, which of course are generally less restrictive for faster \u000A processors.  But note that the prover steps counted need not correspond \u000A closely to prover time.</p> \u000A \u000A <p>Although <v>with-prover-time-limit</v> behaves like an ACL2 function in the \u000A sense that it evaluates both its arguments, it is however actually a macro \u000A that behaves as follows.  (1) The value of its first (time limit) argument \u000A affects the evaluation of its second argument (by causing an error during that \u000A evaluation if the time for completion is insufficient).  (2) The second \u000A argument can return multiple values (see <see topic=\"ACL2____MV\">mv</see>), which are then returned \u000A by the call of <v>with-prover-time-limit</v>.  (3) Thus, there is not a fixed \u000A number of values returned by <v>with-prover-time-limit</v>.</p> \u000A \u000A <p>If you find that the time limit appears to be implemented too loosely, it \u000A may be because the prover only checks the time elapsed at certain points \u000A during the proof process, for example at entry to the rewriter.  For example, \u000A if you write your own <tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> that does an expensive \u000A computation, the time is unlikely to be checked during its execution.  If \u000A however you find the time limit seems to be ignored even during ordinary \u000A prover operation, you are encouraged to email an example to the ACL2 \u000A implementors with instructions on how to observe the undesirable behavior. \u000A This information can perhaps be used to improve ACL2 by the insertion of more \u000A checks for expiration of the time limit.</p> \u000A \u000A <p>The rest of this documentation topic explains the rather subtle logical \u000A story, and is not necessary for understanding how to use \u000A <v>with-prover-time-limit</v>.  The ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> object logically contains \u000A a field called the <v>acl2-oracle</v>, which is an arbitrary true list of \u000A objects; see <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>.  Our claim is that every ACL2 session \u000A makes sense for <b>some</b> value of <v>acl2-oracle</v> in the initial \u000A <v>state</v> for that session.  Logically, <v>with-prover-time-limit</v> is a \u000A no-op, just returning its second value.  But under the hood, it provides a \u000A ``hint'' for the <v>acl2-oracle</v>, so that (logically speaking) when its first \u000A element (<tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>) is consulted by ACL2's prover to see if the time limit \u000A has expired, it gets the ``right'' answer (specifically, either nil if all is \u000A well or else a message to print if the time limit has expired).  Logically, \u000A the <v>acl2-oracle</v> is then <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>'ed &mdash; that is, its first element \u000A is popped off &mdash; so that future results from <v>read-acl2-oracle</v> are \u000A independent of the one just obtained.</p>"],

 "ACL2____SPLITTER":[["Debugging"],"ACL2 Sources","ACL2","<p>The application of a rule to a term may cause a goal to simplify to \u000A more than one subgoal.  A rule with such an application is called a \u000A ``splitter''.  Here, we explain the output produced for splitters when proof \u000A output is enabled \u000A (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) and such reporting is turned on (as it is \u000A by default) &mdash; that is, when the value of <v>(</v><tt><see topic=\"ACL2____SPLITTER-OUTPUT\">splitter-output</see></tt><v>)</v> is true.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> for how to turn off, or on, the reporting of \u000A splitters.  Also see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> for information on how \u000A to control case splits.  Note that since splitters are rule applications, \u000A splitter output is not generated for case splits that are caused by other than \u000A rules, such as the mere presence of calls of the function symbol, <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, \u000A in the goal.</p> \u000A \u000A <p>We begin by describing three types of splitters.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>if-intro</v>: The rule application may have introduced a call of <v>IF</v>, \u000A in the sense discussed at the end below.</p> \u000A \u000A <p><v>case-split</v>: For the application of a rule with hypothesis of the form \u000A <v>(<see topic=\"ACL2____CASE-SPLIT\">case-split</see> &lt;hyp&gt;)</v>, <v>&lt;hyp&gt;</v> did not simplify to true or false.</p> \u000A \u000A <p><v>immed-forced</v>: For the application of a rule with hypothesis of the \u000A form <v>(<see topic=\"ACL2____FORCE\">force</see> &lt;hyp&gt;)</v>, <v>&lt;hyp&gt;</v> did not simplify to true or false, where \u000A immediate-force-modep is enabled (see <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>).</p></blockquote> \u000A \u000A <p>These three annotations &mdash; <v>if-intro</v>, <v>case-split</v>, and \u000A <v>immed-forced</v> &mdash; may be used in proof output and summaries for \u000A describing rule applications, as discussed below.  A fourth annotation, \u000A <v>forced</v>, maybe also be used in proof output to indicate the application of \u000A a rule with hypothesis of the form <v>(<see topic=\"ACL2____FORCE\">force</see> &lt;hyp&gt;)</v> when <v>&lt;hyp&gt;</v> did not \u000A simplify to true or false, where immediate-force-modep is disabled (see <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>).  We don't consider such uses of <tt><see topic=\"ACL2____FORCE\">force</see></tt> to be \u000A splitters, because they do not cause case splits (though they do produce goals \u000A to prove after lower-case ``q.e.d.'' is printed); see <see topic=\"ACL2____FORCE\">force</see>.</p> \u000A \u000A <p>There are three kinds of output affected by splitters, illustrated in turn \u000A below using examples.</p> \u000A \u000A <blockquote><p>(a) During the proof, <see topic=\"ACL2____GAG-MODE\">gag-mode</see> off<br></br> \u000A \u000A (b) During the proof, <see topic=\"ACL2____GAG-MODE\">gag-mode</see> on<br></br> \u000A \u000A (c) Summary</p></blockquote> \u000A \u000A <p>Of course, (a) and (b) are skipped if proof output is inhibited, which (c) \u000A is skipped if summary output is inhibited; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.</p> \u000A \u000A <p>(a) During the proof, <see topic=\"ACL2____GAG-MODE\">gag-mode</see> off</p> \u000A \u000A <p>With <see topic=\"ACL2____GAG-MODE\">gag-mode</see> off (or when using <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt>, <v>:</v><tt><see topic=\"ACL2____PSOF\">psof</see></tt>, or <v>:</v><tt><see topic=\"ACL2____PSOG\">psog</see></tt>) one normally gets an English commentary.  The \u000A following output indicates that at least one application of each rule <v>F</v> \u000A and <v>G</v> is of type <v>if-intro</v>, at least one application of rules <v>G</v> \u000A and <v>R1</v> are of type <v>case-split</v>, and at least one application of rule \u000A <v>R3</v> is of type <v>immed-forced</v>.  If <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see> is off \u000A then ``<v>immed-forced</v>'' would be replaced by ``<v>forced</v>''.</p> \u000A \u000A <code>This simplifies, using the :definitions F (<see topic=\"ACL2____IF-INTRO\">if-intro</see>), G (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> and\u000Aif-intro) and H and the :rewrite rules R1, R2 (<see topic=\"ACL2____CASE-SPLIT\">case-split</see>), and\u000AR3 (<see topic=\"ACL2____IMMED-FORCED\">immed-forced</see>), to the following two conjectures.</code> \u000A \u000A <p>Note that any such printing of ``<v>forced</v>'' is done even if \u000A <v>(<see topic=\"ACL2____SPLITTER-OUTPUT\">splitter-output</see>)</v> is false.  Such forcing indication is also made when \u000A raw proof format is used &mdash; see <see topic=\"ACL2____SET-RAW-PROOF-FORMAT\">set-raw-proof-format</see> &mdash; but \u000A in that case, no indication is made for splitters in the proof output.</p> \u000A \u000A <p>(b) During the proof, <see topic=\"ACL2____GAG-MODE\">gag-mode</see> on</p> \u000A \u000A <p>With <see topic=\"ACL2____GAG-MODE\">gag-mode</see> on the proof output is greatly abbreviated.  However, \u000A ``Splitter Notes'' are printed so that even with <see topic=\"ACL2____GAG-MODE\">gag-mode</see> on, one can \u000A get important information to help control large case splits, by disabling \u000A splitter rules as appropriate.  These are printed at the point when a goal \u000A splits into subgoals.  Here, for example, is the Splitter Note that \u000A corresponds to the output shown in (a) above.  It shows the goal whose \u000A simplification has produced a split into more than one subgoal, and it shows \u000A how many subgoals have been created.</p> \u000A \u000A <code>Splitter note (see :DOC splitter) for Subgoal *1/2.2.1' (2 subgoals).\u000A  case-split: ((:DEFINITION G) (:REWRITE R2))\u000A  immed-forced: ((:REWRITE R3))\u000A  if-intro: ((:DEFINITION G) (:DEFINITION F))</code> \u000A \u000A <p>No such splitter notes are printed for the use of <tt><see topic=\"ACL2____FORCE\">force</see></tt> (when <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see> is off).</p> \u000A \u000A <p>(c) Summary</p> \u000A \u000A <p>Here is a possible summary corresponding to our running example.  In the \u000A summary, ``Splitter rules'' is omitted if there are no splitter rules, and a \u000A splitter type is only mentioned if there is at least one corresponding \u000A splitter rule.</p> \u000A \u000A <code>Summary\u000AForm:  ( THM ...)\u000ARules: ((:DEFINITION F)\u000A        (:DEFINITION G)\u000A        (:DEFINITION H)\u000A        (:REWRITE R1)\u000A        (:REWRITE R2)\u000A        (:REWRITE R3))\u000ASplitter rules (see :DOC splitter):\u000A  case-split: ((:DEFINITION G) (:REWRITE R2))\u000A  immed-forced: ((:REWRITE R3))\u000A  if-intro: ((:DEFINITION G) (:DEFINITION F))\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000AProver steps counted:  145</code> \u000A \u000A <p>No indication for ``<v>forced</v>'' is given for ``Splitter rules''.  (As \u000A discussed earlier above, the <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses are not considered \u000A relevant for determining splitter rule applications unless <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see> is on.)</p> \u000A \u000A <p>We conclude by giving the criteria for a <see topic=\"ACL2____REWRITE\">rewrite</see> or <see topic=\"ACL2____DEFINITION\">definition</see> rule application to be a splitter of type <v>if-intro</v>.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Reporting of splitter rules is on, i.e., the value of <v>(</v><tt><see topic=\"ACL2____SPLITTER-OUTPUT\">splitter-output</see></tt><v>)</v> is true.</p> \u000A \u000A <p>o At least two subgoals are created, even before considering subgoals \u000A generated by hypotheses that are calls of <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt> or <tt><see topic=\"ACL2____FORCE\">force</see></tt>.</p> \u000A \u000A <p>o The term to which the rule is applied is at the top level, rather than \u000A being encountered when trying to establish the hypothesis of a rule.</p> \u000A \u000A <p>o The rule is a <see topic=\"ACL2____REWRITE\">rewrite</see> rule, a <see topic=\"ACL2____DEFINITION\">definition</see> rule, or a <see topic=\"ACL2____META\">meta</see> rule.</p> \u000A \u000A <p>o There is a call of the function symbol <v>IF</v> in the right-hand side of \u000A the <see topic=\"ACL2____REWRITE\">rewrite</see> rule; or, in the case of a <see topic=\"ACL2____DEFINITION\">definition</see> rule, in the \u000A body of the definition; or, in the case of a <see topic=\"ACL2____META\">meta</see> rule, in the result \u000A of applying the metafunction.</p> \u000A \u000A <p>o There is a call of the function symbol <v>IF</v> in the result of \u000A rewriting: the right-hand side (for a <see topic=\"ACL2____REWRITE\">rewrite</see> rule), the definition \u000A body (for a <see topic=\"ACL2____DEFINITION\">definition</see> rule), or the metafunction application (for a \u000A <see topic=\"ACL2____META\">meta</see> rule).</p></blockquote> \u000A \u000A <p>Any rule application meeting the above criteria will be considered a \u000A splitter of type <v>if-intro</v>, even if the call does not actually cause a \u000A case split.  For example, if you are proving <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (hyp x) (conc x))</v> \u000A and rule R rewrites <v>(hyp x)</v> to <v>(<see topic=\"COMMON-LISP____IF\">if</see> (h1 x) (h2 x) nil)</v>, which is \u000A really the term <v>(<see topic=\"COMMON-LISP____AND\">and</see> (h1 x) (h2 x))</v>, then R may be labelled as a splitter \u000A rule.  If you want to find the causes of case-splitting, the list of \u000A <v>if-intro</v> splitters can help you narrow your search, but may include \u000A irrelevant rules as well.</p> \u000A \u000A <p>Finally, note that you may see splits not attributed to splitters.  We \u000A believe that this will be uncommon during simplification, though it can occur \u000A for example when a call of <v>IF</v> is in the body of a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expression, \u000A i.e., in a call of a <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> expression.  But splits caused by other \u000A processes, notably destructor elimination (see <see topic=\"ACL2____ELIM\">elim</see>), will typically \u000A not be attributed to splitters.</p>"],

 "ACL2____SAVE-EXEC":[["Interfacing-tools","Command-line"],"ACL2 Sources","ACL2","<p><v>Save-exec</v> saves your ACL2 state so that you can immediately \u000A re-start later in that same state.  This utility can be useful for a project \u000A with <see topic=\"ACL2____BOOKS\">books</see> to be included every time ACL2 is started, to avoid time \u000A taken to run <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  Another use of <v>save-exec</v> is to save \u000A an executable that takes command-line arguments beyond those normally passed \u000A to the host Lisp executable.  All arguments of a call of <v>save-exec</v> are \u000A evaluated.</p> \u000A \u000A <code>Examples:\u000A\u000A; Save an executable script named my-saved_acl2, with the indicated message\u000A; added to the start-up banner:\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot;\u000A           &quot;This saved image includes Version 7 of Project Foo.&quot;)\u000A\u000A; Same as above, but instead with a generic comment in the start-up banner:\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot; nil)\u000A\u000A; Arrange that the generated script passes the indicated arguments to be\u000A; processed by the Lisp (<see topic=\"ACL2____ACL2\">ACL2</see>) executable (where this example is specific to\u000A; the case that CCL is the host Lisp):\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot; nil\u000A           :host-lisp-args &quot;--no-init -Z 256M&quot;)\u000A\u000A; Arrange that the generated script passes along the indicated arguments\u000A; to Lisp (<see topic=\"ACL2____ACL2\">ACL2</see>), but that they are not processed by Lisp other than to\u000A; record the additional arguments (see (6) below).\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot; nil\u000A           :inert-args &quot;abc xyz -i foo&quot;)\u000A\u000A; Combining the preceding two examples:\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot; nil\u000A           :host-lisp-args &quot;--no-init -Z 256M&quot;\u000A           :inert-args &quot;abc xyz -i foo&quot;)\u000A\u000A; Arrange that ACL2 evaluates (with LD) the three forms shown.\u000A; In this example, the THM form fails, but that does not stop the definition\u000A; of BAR from being evaluated: LD continues on, just as the top-level loop\u000A; lets you continue after a failure.\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-acl2&quot; &quot;test&quot;\u000A           :init-forms\u000A           '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x))\u000A             (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x))\u000A             (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) x)))\u000A\u000A; Essentially as just above, except that the call of LD returns after the THM\u000A; failure, so BAR does not get defined.\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-acl2&quot; &quot;test&quot;\u000A           :init-forms\u000A           '((<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x))\u000A                   (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x))\u000A                   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) x))\u000A                 :ld-pre-eval-print t :ld-verbose nil)))\u000A\u000A; Immediately exit the ACL2 read-eval-print loop after starting up.\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-acl2&quot; nil\u000A           :return-from-lp t)\u000A\u000A; Immediately exit the ACL2 read-eval-print loop after starting up and\u000A; defining function FOO in the logic.\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-acl2&quot; &quot;Start with foo defined.&quot;\u000A           :return-from-lp '(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A                             :off :all\u000A                             (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)))\u000A\u000A; Immediately exit the ACL2 read-eval-print loop after starting up and\u000A; defining variable xxx in raw Lisp.\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-acl2&quot; &quot;Start with xxx defined.&quot;\u000A           :return-from-lp '(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A                             :off :all\u000A                             (<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"ACL2____SET-RAW-MODE-ON_12\">set-raw-mode-on!</see>)\u000A                                   (defvar xxx (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 10))\u000A                                   (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> nil)\u000A                                   (<see topic=\"ACL2____U\">u</see>)))))</code> \u000A \u000A <p>Each example above generates a file named \"my-saved_acl2\".  That file is \u000A quite similar in form to the script generated when building ACL2 directly from \u000A source code; details are below.  For example, here are the contents of that \u000A generated file if the host Lisp is CCL (but where dates and pathnames are \u000A specific to one's environment).  Here, we break lines using `\\', but the \u000A <v>exec</v> command is actually on a single line.</p> \u000A \u000A <code>#!/bin/sh\u000A\u000A# Saved August 16, 2013  23:06:49\u000A#  then August 17, 2013  11:01:56\u000A\u000Aexport CCL_DEFAULT_DIRECTORY=&quot;/projects/acl2/lisps/ccl/15542/ccl&quot;\u000Aexec &quot;/projects/ccl/lx86cl64&quot; -I &quot;/u/smith/my-saved_acl2.lx86cl64&quot; \\\u000A     -Z 64M -K ISO-8859-1 -e &quot;(acl2::acl2-default-restart)&quot; \\\u000A     --no-init -Z 256M \\\u000A     -- \\\u000A     abc xyz -i foo \\\u000A     &quot;$@&quot;</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> exec-filename extra-startup-string\u000A           :host-lisp-args host-lisp-args\u000A           :inert-args inert-args\u000A           :return-from-lp return-from-lp\u000A           :init-forms init-forms)</code> \u000A \u000A <p>where the keyword arguments are optional, and arguments are as follows.</p> \u000A \u000A <blockquote><p><v>Exec-filename</v> is the filename of the proposed \u000A executable.</p> \u000A \u000A <p><v>Extra-startup-string</v> is a non-empty string to be printed after the \u000A normal ACL2 startup message when you start up the saved image.  However, \u000A <v>extra-startup-string</v> is allowed to be <v>nil</v>, in which case a generic \u000A string will be printed instead.</p> \u000A \u000A <p><v>Host-lisp-args</v> can be <v>nil</v> (the default), but if it is a \u000A non-<v>nil</v> value, then it is a string to be inserted into the command line \u000A in the saved script, specifying additional arguments that are to be processed \u000A by the host Lisp executable.  (Note for SBCL only: these are runtime options; \u000A for toplevel options, see (8) below.)</p> \u000A \u000A <p><v>Inert-args</v> can be <v>nil</v> (the default), but if it is a non-<v>nil</v> \u000A value, then it is a string to be inserted into the command line in the saved \u000A script, specifying additional arguments that are not to be processed by the \u000A host Lisp executable.</p></blockquote> \u000A \u000A <p><v>Return-from-lp</v> and <v>init-forms</v> are <v>nil</v> by default. \u000A Regardless of their value, ACL2 starts up and enters its read-eval-print loop \u000A in the usual way; see <see topic=\"ACL2____LP\">lp</see>.  Normally you'll stay inside that loop, but \u000A if <v>return-from-lp</v> or <v>init-forms</v> is not <v>nil</v>, then the indicated \u000A forms are evaluated in the ACL2 read-eval-print loop: a single form in the \u000A case of <v>return-from-lp</v>, and a list of forms in the case of \u000A <v>init-forms</v>.  (Essentially, the <tt><see topic=\"ACL2____LD\">ld</see></tt> special <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> is \u000A provided to a call of <tt><see topic=\"ACL2____LD\">ld</see></tt> as a list with the indicated forms on the \u000A front.)  Moreover, if the case of <v>return-from-lp</v>, the loop is exited \u000A after evaluation of the form, leaving you in raw Lisp.  Evaluation of \u000A <v>return-from-lp</v> is done with <tt><see topic=\"ACL2____LD\">ld</see></tt> options that minimize output (also \u000A see <see topic=\"ACL2____WITH-OUTPUT\">with-output</see> to minimize output).  Suggestion: let \u000A <v>return-from-lp</v> be <v>t</v> if you simply want to exit the read-eval-print \u000A loop at startup, without evaluating any (nontrivial) form.</p> \u000A \u000A <p>NOTE: It is illegal to supply non-<v>nil</v> values for both \u000A <v>:return-from-lp</v> and <v>:init-forms</v>.  But if you add <v>(value :q)</v> to \u000A the end of your list of <v>:init-forms</v>, and your other forms do not result \u000A in an error, then you will be left in raw Lisp (as was presumably intended if \u000A you were planning to use <v>:return-from-lp</v>).</p> \u000A \u000A <p>The remainder of this documentation focuses on the options other than \u000A <v>return-from-lp</v> and <v>init-forms</v>.</p> \u000A \u000A <p><b>Details</b>:</p> \u000A \u000A <p>(1) You must first exit the ACL2 read-eval-print loop, typically by \u000A executing <v>:q</v>, before evaluating a <v>save-exec</v> call; otherwise an error \u000A occurs.</p> \u000A \u000A <p>(2) The image will be saved so that in the new image, the raw Lisp package \u000A and the package in the ACL2 read-eval-print loop (see <see topic=\"ACL2____LP\">lp</see>) will be the \u000A same as their respective values at the time <v>save-exec</v> is called.</p> \u000A \u000A <p>(3) <v>Save-exec</v> generates a small script file (e.g., \u000A <v>&quot;my-saved_acl2&quot;</v> in the examples above), similar in form (see (4) below) \u000A to the script generated when building ACL2 directly from source code, but with \u000A a comment line indicating the time at which the new script is written. \u000A <v>Save-exec</v> also saves an associated binary file.  The binary file's name \u000A is obtained by putting a suffix on the script filename; for example, if the \u000A host Lisp is GCL running on a Linux or Darwin (MacOS) system, then that binary \u000A file has the name <v>my-saved_acl2.gcl</v> in the examples above.</p> \u000A \u000A <p>(4) If <v>inert-args</v> is <v>nil</v> (for example if keyword <v>:inert-args</v> \u000A is omitted), then when the generated ACL2 script is invoked with command line \u000A arguments, those arguments will be passed to the host Lisp; otherwise they \u000A will not.  Thus for the example above, suppose we invoke the generated script \u000A as follows.</p> \u000A \u000A <code>my-saved_acl2 -a bcd -e fgh</code> \u000A \u000A <p>If <v>my-saved_acl2</v> was generated using a <v>save-exec</v> command with a \u000A non-<v>nil</v> value specified for keyword <v>:inert-args</v>, then the arguments \u000A ``<v>-a bcd -e fgh</v>'' will not be passed to the host Lisp; otherwise, they \u000A will be.  Note that for ACL2 executable scripts generated by an ordinary ACL2 \u000A build from sources, the latter case (i.e., without <v>inert-args</v>) takes \u000A place.</p> \u000A \u000A <p>(5) The generated script, which specifies execution with <v>/bin/sh</v>, will \u000A generally contain a line of one of the following forms.  (But for SBCL, see \u000A (8) below.)  In the examples that follow, <v>ACL2_options</v> is a suitable list \u000A of command-line arguments given to the ACL2 executable.  The quoted string \u000A <v>&quot;$@&quot;</v> is intended to allow the user to pass additional command-line \u000A arguments to that executable.</p> \u000A \u000A <blockquote> \u000A \u000A <p>If <v>host-lisp-args</v> and <v>inert-args</v> are omitted (or <v>nil</v>):</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; &lt;ACL2_options&gt; &quot;$@&quot;</code> \u000A \u000A <p>More generally, <v>host-lisp-args</v> is inserted immediately after \u000A <v>&lt;ACL2_options&gt;</v>, but only if it is non-<v>nil</v> (hence a string).  If \u000A <v>inert-args</v> is <v>nil</v>, we thus get:</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; &lt;ACL2_options&gt; host-lisp-args &quot;$@&quot;</code> \u000A \u000A <p>If <v>host-lisp-args</v> redefines a value from <v>&lt;ACL2_options&gt;</v>, then it \u000A is up to the host lisp which value to use.  For example, experiments show that \u000A in CCL, if <v>-Z</v> appears twice, each with a legal value, then the second \u000A value is the one that is used (i.e. it does indeed override the original value \u000A written out by ACL2 in <v>&lt;ACL2_options&gt;</v>.  But experiments also show that in \u000A LispWorks, where ``<v>-init -</v>'' is included in <v>&lt;ACL2_options&gt;</v>, then \u000A inclusion of ``<v>-init foo.lisp</v>'' in <v>host-lisp-args</v> is ignored.</p> \u000A \u000A <p>The remaining cases below are for a non-<v>nil</v> value of <v>inert-args</v>. \u000A In each case, if <v>host-lisp-args</v> is <v>nil</v> then it should be omitted \u000A from the displayed command.</p> \u000A \u000A <p>If <v>inert-args</v> is <v>t</v> then an additional argument, `<v>--</v>', \u000A indicates that when ACL2 is given command line arguments, these should not be \u000A processed by the host Lisp (other than recording them; see (6) below):</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; &lt;ACL2_options&gt; host-lisp-args -- &quot;$@&quot;</code> \u000A \u000A <p>If <v>inert-args</v> is a string then the result is similar to the above, \u000A except that <v>inert-args</v> is added immediately after `<v>--</v>':</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; &lt;ACL2_options&gt; host-lisp-args -- inert-args &quot;$@&quot;</code> \u000A \u000A </blockquote> \u000A \u000A <p>(6) See <see topic=\"OSLIB____ARGV\">oslib::argv</see> for a utility that returns a list of all \u000A <v>inert-args</v> from an invocation of ACL2.</p> \u000A \u000A <p>(7) Suppose that you invoke an ACL2 script, say <v>&quot;my-saved_acl2&quot;</v>, \u000A that was generated by <v>save-exec</v>, and then optionally evaluate some forms. \u000A Then you may save a new ACL2 script with <v>save-exec</v>.  The new script will \u000A contain comment lines that extend comment lines in <v>&quot;my-saved_acl2&quot;</v> with \u000A a new write date, but otherwise will be identical to the script that would \u000A have been generated by executing the new <v>save-exec</v> call after invoking \u000A the original ACL2 executable (built directly from ACL2 sources) instead of \u000A <v>&quot;my-saved_acl2&quot;</v>.  In other words, the options added by the earlier \u000A <v>save-exec</v> call that created <v>&quot;my-saved_acl2&quot;</v> are discarded by the \u000A new <v>save-exec</v> call.  However, the <v>.core</v> file will built on top of \u000A the <v>.core</v> file that was consulted when <v>&quot;my-saved_acl2&quot;</v> was \u000A invoked.</p> \u000A \u000A <p>(8) The following note pertains only to the case that the host Lisp is \u000A SBCL.  For SBCL, the scripts written are analogous to, but slightly different \u000A from, those shown above.  Please note that for SBCL, the <v>host-lisp-args</v> \u000A are what the SBCL manual calls ``runtime options''.  For SBCL only, an extra \u000A keyword argument, <v>:toplevel-args</v>, may be used for specifying what the \u000A SBCL manual calls ``toplevel options.  As with <v>:host-lisp-args</v>, this \u000A value, <v>toplevel-args</v>, should be <v>nil</v> (the default) or a string.  Here \u000A is an example.</p> \u000A \u000A <code>(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;my-saved_acl2&quot; nil\u000A           :host-lisp-args &quot;--dynamic-space-size 12000&quot;\u000A           :toplevel-args &quot;--eval '(print \\&quot;HELLO\\&quot;)'&quot;\u000A           :inert-args &quot;--my-option my-value&quot;)</code> \u000A \u000A <p>The script generated by this example call of <v>save-exec</v> contains a line \u000A such as the following (with the same convention for `\\' as before)</p> \u000A \u000A <code>exec &quot;/projects/sbcl-1.1.7-x86-64-linux/src/runtime/sbcl&quot; \\\u000A     --dynamic-space-size 2000 --control-stack-size 8 \\\u000A     --core &quot;/u/smith/my-saved_acl2.core&quot; --dynamic-space-size 12000 \\\u000A     --end-runtime-options \\\u000A     --no-userinit --eval '(acl2::sbcl-restart)' \\\u000A     --eval '(print &quot;HELLO&quot;)' \\\u000A     --end-toplevel-options \\\u000A     --my-option my-value \\\u000A     &quot;$@&quot;</code> \u000A \u000A <p>In general, the generated script is of one of the following forms (with the \u000A same convention for `\\' as before).</p> \u000A \u000A <blockquote> \u000A \u000A <p>For the case that <v>inert-args</v> is <v>nil</v>:</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; \\\u000A     &lt;ACL2_runtime_options&gt; host-lisp-args --end-runtime-options \\\u000A     &lt;ACL2_toplevel_options&gt; host-lisp-args \\\u000A     &quot;$@&quot;</code> \u000A \u000A <p>For the case that <v>inert-args</v> is non-<v>nil</v>:</p> \u000A \u000A <code>exec &lt;lisp_executable&gt; \\\u000A     &lt;ACL2_runtime_options&gt; host-lisp-args --end-runtime-options \\\u000A     &lt;ACL2_toplevel_options&gt; host-lisp-args --end-toplevel-options \\\u000A     inert-args &quot;$@&quot;</code> \u000A \u000A </blockquote> \u000A \u000A <p>Notice that as before, when the generated script is invoked (for example, \u000A at the shell), additional command-line arguments provided at that time are \u000A passed to Lisp if and only if <v>inert-args</v> is <v>nil</v>.  For SBCL, when \u000A they are passed to Lisp they are passed as toplevel options, not as runtime \u000A options.</p>"],

 "ACL2____PUFF":[["History"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;:puff :max\u000AACL2 !&gt;:puff :x\u000AACL2 !&gt;:puff 15\u000AACL2 !&gt;:puff &quot;book&quot;\u000A\u000AGeneral Form:\u000A:puff cd</code> \u000A \u000A <p>where <v>cd</v> is a <see topic=\"ACL2____COMMAND\">command</see> descriptor (see <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>) for a ``puffable'' <see topic=\"ACL2____COMMAND\">command</see> (see below).  <v>Puff</v> \u000A replaces the <see topic=\"ACL2____COMMAND\">command</see> at <v>cd</v> by the immediate subevents of the <see topic=\"ACL2____COMMAND\">command</see>, executed as <see topic=\"ACL2____COMMAND\">command</see>s.  <v>Puff</v> then prints, using <tt><see topic=\"ACL2____PCS\">pcs</see></tt>, the <v>puff</v>ed region.</p> \u000A \u000A <p>We consider <v>puff</v> to be a sort of hack; it is generally robust and \u000A sound, but that is not guaranteed.  If any existing ACL2 event resulted from \u000A <v>puff</v>, ACL2 considers proofs to have been skipped, and thus <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is disallowed until such events have been undone (see <see topic=\"ACL2____UBT\">ubt</see>).</p> \u000A \u000A <p>A ``puffable'' <see topic=\"ACL2____COMMAND\">command</see> is an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____COMMAND\">command</see>, an \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____COMMAND\">command</see>, or any <see topic=\"ACL2____COMMAND\">command</see> other than those \u000A consisting of a single primitive event.  For example, since <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is a \u000A primitive event, a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____COMMAND\">command</see> is not puffable.  But a macro \u000A form that expands into several <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> is puffable.  The \u000A only primitive <see topic=\"ACL2____EVENTS\">events</see> that are puffable are calls of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  A puffable <see topic=\"ACL2____COMMAND\">command</see> contains \u000A (interesting) subevents, namely, the <see topic=\"ACL2____EVENTS\">events</see> in the body of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, in the file of the book included, or in the <see topic=\"ACL2____COMMAND\">command</see> \u000A block.</p> \u000A \u000A <p>(Obscure exceptions.  (1) An <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> command generated by the \u000A macro <tt><see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see></tt> is not puffable.  (2) If a use \u000A of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> results in <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in the scope of \u000A an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event within a command, then an \u000A attempt to <v>puff</v> that command will result in an error, leaving the <see topic=\"ACL2____WORLD\">world</see> unchanged, if any such <tt><see topic=\"ACL2____LOCAL\">local</see></tt> event was necessary to support \u000A other events in the command.  (3) An attempt to <v>puff</v> an <v>include-book</v> \u000A command may fail for a book that has been modified, as describe late in this \u000A documentation topic.)</p> \u000A \u000A <p>The puff <see topic=\"ACL2____COMMAND\">command</see> ``lifts'' the immediate subevents of the indicated \u000A <see topic=\"ACL2____COMMAND\">command</see> so that they become <see topic=\"ACL2____COMMAND\">command</see>s themselves.  The <see topic=\"ACL2____COMMAND\">command</see> <tt><see topic=\"ACL2____PUFF_A2\">puff*</see></tt> recursively puffs the newly introduced <see topic=\"ACL2____COMMAND\">command</see>s. \u000A See <see topic=\"ACL2____PUFF_A2\">puff*</see>, which also gives an example illustrating both <v>puff</v> and \u000A <tt><see topic=\"ACL2____PUFF_A2\">puff*</see></tt>.  <v>Puff</v> undoes the <see topic=\"ACL2____COMMAND\">command</see> at <v>cd</v> and replaces it \u000A by its immediate subevents.  Thus, in general the length of the <see topic=\"ACL2____HISTORY\">history</see> \u000A grows when a puff <see topic=\"ACL2____COMMAND\">command</see> is executed.  If <v>puff</v> causes an error \u000A (see below), the logical <see topic=\"ACL2____WORLD\">world</see> remains unchanged from its initial \u000A configuration.</p> \u000A \u000A <p>The intended use of <v>puff</v> is to allow the user access to the <see topic=\"ACL2____EVENTS\">events</see> ``hidden'' inside compound <see topic=\"ACL2____COMMAND\">command</see>s.  For example, while trying \u000A to prove some theorem, <v>p</v>, about a constrained function, <v>fn</v>, one \u000A might find that the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, <v>cd</v>, that introduced <v>fn</v> \u000A failed to include an important <see topic=\"ACL2____CONSTRAINT\">constraint</see>, <v>q</v>.  Without <v>puff</v>, \u000A the only way to proceed is to undo back through <v>cd</v>, create a suitable \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> that proves and exports <v>q</v> as well as the old <see topic=\"ACL2____CONSTRAINT\">constraint</see>s, re-execute the new <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, re-execute the <see topic=\"ACL2____EVENTS\">events</see> since <v>cd</v>, and then try <v>p</v> again.  Unfortunately, it may be \u000A hard to prove <v>q</v> and additional <see topic=\"ACL2____EVENTS\">events</see> may have to be inserted into \u000A the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> to prove it.  It may also be hard to formulate the \u000A ``right'' <v>q</v>, i.e., one that is provable in the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and \u000A provides the appropriate facts for use in the proof of <v>p</v>.</p> \u000A \u000A <p>Using <v>puff</v>, the user can erase the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> at <v>cd</v>, \u000A replacing it by the <see topic=\"ACL2____EVENTS\">events</see> in its body.  Now the formerly constrained \u000A function, <v>fn</v>, is defined as its witness.  The user can experiment with \u000A formulations and proofs of <v>q</v> suitable for <v>p</v>.  Of course, to get into \u000A the ultimately desired <see topic=\"ACL2____STATE\">state</see> &mdash; where <v>fn</v> is constrained \u000A rather than defined and <v>q</v> is exported by an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> at \u000A <v>cd</v> &mdash; the user must ultimately undo back to <v>cd</v> and carry out \u000A the more tedious program described above.  But by using <v>puff</v> it is easier \u000A to experiment.</p> \u000A \u000A <p>Similar applications of <v>puff</v> allow the user of a book to expose the \u000A innards of the book as though they had all be typed as <see topic=\"ACL2____COMMAND\">command</see>s.  The \u000A user might then ``partially undo'' the book, keeping only some of the <see topic=\"ACL2____EVENTS\">events</see> in it.</p> \u000A \u000A <p><v>Puff</v> operates as follows.  First, it determines the list of immediate \u000A subevents of the <see topic=\"ACL2____COMMAND\">command</see> indicated by <v>cd</v>.  It causes an error if \u000A there is only one subevent and that subevent is identical to the <see topic=\"ACL2____COMMAND\">command</see> &mdash; i.e., if the <see topic=\"ACL2____COMMAND\">command</see> at <v>cd</v> is a primitive.  Next, \u000A <v>puff</v> undoes back through the indicated <see topic=\"ACL2____COMMAND\">command</see>.  This not only \u000A erases the <see topic=\"ACL2____COMMAND\">command</see> at <v>cd</v> but all the <see topic=\"ACL2____COMMAND\">command</see>s executed \u000A after it.  Finally, <v>puff</v> re-executes the subevents of (the now erased) \u000A <v>cd</v> followed by all the <see topic=\"ACL2____COMMAND\">command</see>s that were executed afterwards.</p> \u000A \u000A <p>Observe that the <see topic=\"ACL2____COMMAND\">command</see>s executed after <v>cd</v> will generally have \u000A higher <see topic=\"ACL2____COMMAND\">command</see> numbers than they did before the puff.  For example, \u000A suppose 100 <see topic=\"ACL2____COMMAND\">command</see>s have been executed and that <v>:puff 80</v> is then \u000A executed.  Suppose <see topic=\"ACL2____COMMAND\">command</see> 80 contains 5 immediate subevents (i.e., is \u000A an encapsulation of five <see topic=\"ACL2____EVENTS\">events</see>).  Then, after puffing, <see topic=\"ACL2____COMMAND\">command</see> \u000A 80 is the first event of the puffed <see topic=\"ACL2____COMMAND\">command</see>, <see topic=\"ACL2____COMMAND\">command</see> 81 is the \u000A second, and so on; 104 <see topic=\"ACL2____COMMAND\">command</see>s appear to have been executed.</p> \u000A \u000A <p>When puffing an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____COMMAND\">command</see>s are executed.  Note that this will replace constrained \u000A functions by their witnesses.</p> \u000A \u000A <p>Finally, it is impossible to <v>puff</v> in the presence of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> for certified books that have been altered since \u000A they were included.  (Note that this restriction only applies to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, not to <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.)  To be specific, suppose \u000A <v>&quot;arith&quot;</v> is a certified book that has been included in a session. \u000A Suppose that after <v>&quot;arith&quot;</v> was included, the source file is modified. \u000A (This might happen if the user of <v>&quot;arith&quot;</v> is not its author and the \u000A author happens to be working on a new version of <v>&quot;arith&quot;</v> during the \u000A same time period.)  Now suppose the user tries to <v>puff</v> the <see topic=\"ACL2____COMMAND\">command</see> \u000A that included <v>&quot;arith&quot;</v>.  The attempt to obtain the subevents in \u000A <v>&quot;arith&quot;</v> will discover that the check sum of <v>&quot;arith&quot;</v> has changed \u000A and an error will be caused.  No change is made in the logical <see topic=\"ACL2____WORLD\">world</see>. \u000A A similar error is caused if, in this same situation, the user tries to puff \u000A any command that occurred before the inclusion of <v>&quot;arith&quot;</v>!  That is, \u000A <v>puff</v> may cause an error and leave the <see topic=\"ACL2____WORLD\">world</see> unchanged even if the \u000A <see topic=\"ACL2____COMMAND\">command</see> puffed is not one involving the modified book.  This happens \u000A because in order to reconstruct the <see topic=\"ACL2____WORLD\">world</see> after the puffed <see topic=\"ACL2____COMMAND\">command</see>, <v>puff</v> must obtain the <see topic=\"ACL2____EVENTS\">events</see> in the book and if the \u000A book's source file has changed there is no assurance that the reconstructed \u000A <see topic=\"ACL2____WORLD\">world</see> is the one the user intends.</p> \u000A \u000A <p>Warning: We do not detect changes to uncertified <see topic=\"ACL2____BOOKS\">books</see> that have \u000A been included and are then puffed or re-included!  The act of including an \u000A uncertified book leaves no trace of the check sum of the book.  Furthermore, \u000A the act prints a warning message disclaiming soundness.  In light of this, \u000A <v>:puff</v> quietly ``re-''executes the current contents of the book.</p>"],

 "ACL2____META-EXTRACT":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>For this advanced topic, we assume familiarity with metatheorems \u000A and metafunctions (see <see topic=\"ACL2____META\">meta</see>), as well as extended metafunctions (see \u000A <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>).  The capability described here &mdash; \u000A so-called ``meta-extract hypotheses'' for a <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> or a \u000A <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> rule &mdash; provides an advanced form of \u000A meta-level reasoning that was initially designed largely by Sol Swords, who \u000A also provided a preliminary implementation.</p> \u000A \u000A <p>A meta rule or clause-processor rule may have so-called ``meta-extract'' \u000A hypotheses that take forms displayed below.  Here <v>evl</v> is the evaluator, \u000A <v>obj</v> is an arbitrary term, <v>mfc</v> is the metafunction context (which is \u000A a variable other than the symbol <v>STATE</v> that represents the metafunction \u000A context; see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>), <v>state</v> is literally the \u000A symbol <v>STATE</v>, <v>a</v> is the second argument of <v>evl</v> in both arguments \u000A of the conclusion of the rule, and <v>aa</v> is an arbitrary term.</p> \u000A \u000A <code>(evl (<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see> obj mfc state) a)\u000A(evl (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT\">meta-extract-global-fact</see> obj state) aa)) ; equivalent to the next form\u000A(evl (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT_B2\">meta-extract-global-fact+</see> obj state state) aa)\u000A(evl (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT_B2\">meta-extract-global-fact+</see> obj st state) aa)</code> \u000A \u000A <p>The first form is only legal for <v>:meta</v> rules for which the \u000A metafunction is an extended metafunction.  The remaining forms are legal for \u000A both <v>:meta</v> rules and <v>:clause-processor</v> rules.</p> \u000A \u000A <p>Sol Swords has contributed a community book, \u000A <v>clause-processors/meta-extract-user.lisp</v>, that uses a Skolemization trick \u000A to allow one to use at most one <v>meta-extract-global-fact+</v> hypothesis and \u000A at most one <v>meta-extract-contextual-fact</v> hypothesis.</p> \u000A \u000A <p>These additional hypotheses may be necessary in order to prove a proposed \u000A metatheorem or (for the second type of hypothesis above) clause-processor \u000A rule, in particular when the correctness of the metafunction depends on the \u000A correctness of utilities extracting formulas from the logical <see topic=\"ACL2____WORLD\">world</see> or \u000A (for the first type) facts from the metafunction context (mfc).  After the \u000A rule is proved, however, the meta-extract hypotheses have no effect on how the \u000A rule is applied during a proof.  An argument for correctness of using \u000A meta-extract hypotheses is given in the ACL2 source code within a comment \u000A entitled ``Essay on Correctness of Meta Reasoning''.  In the documentation \u000A below, we focus primarily on <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules, since the use of \u000A <v>meta-extract-global-fact</v> hypotheses in <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> \u000A rules is entirely analogous.  (At the end, though, we discuss the last of the \u000A four forms displayed above.)  And for <v>:meta</v> rules we focus not on the \u000A application of rules but, rather, on how the use of meta-extract hypotheses \u000A allow you to prove correctness of metafunctions that use facts from the \u000A logical <see topic=\"ACL2____WORLD\">world</see> or the metafunction context (mfc).</p> \u000A \u000A <p>Below we describe properties of <v>meta-extract-contextual-fact</v> and \u000A <v>meta-extract-global-fact</v>, but only after we illustrate their utility with \u000A an example.  But even before we present that example, we first give a sense of \u000A how to think about these functions by showing a theorem that one can prove \u000A about the first of them.  If this snippet doesn't help your intuition, then \u000A just skip over it and start with the example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A  ((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y) (<see topic=\"ACL2____TYPESPEC-CHECK\">typespec-check</see> x y)))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A      (<see topic=\"COMMON-LISP____NOT\">not</see> (bad-atom (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> 'x alist))))\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl (<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see> (<see topic=\"COMMON-LISP____LIST\">list</see> :typeset 'x)\u000A                                                mfc\u000A                                                state)\u000A                  alist)\u000A             (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 0 ; indicates non-empty intersection\u000A                         (<see topic=\"COMMON-LISP____LOGAND\">logand</see> (type-set-quote ; type-set of a constant\u000A                                  (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> 'x alist)))\u000A                                 (mfc-ts-fn 'x mfc state nil)))))))</code> \u000A \u000A <p>The following example comes from the community book, \u000A <v>books/clause-processors/meta-extract-simple-test.lisp</v> (after it defines \u000A the evaluator), which presents very basic (and contrived) examples that \u000A nevertheless illustrate meta-extract hypotheses.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> plus-identity-2-meta\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT\">meta-extract-global-fact</see> '(:formula bar-posp)\u000A                                                      state)\u000A                            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'u\u000A                                        (nthmeta-ev (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> term))\u000A                                                    a))))\u000A                (nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see>\u000A                             `(:typeset ,(<see topic=\"COMMON-LISP____CADDR\">caddr</see> term)) mfc state)\u000A                            a))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (nthmeta-ev term a)\u000A                  (nthmeta-ev (plus-identity-2-metafn term mfc state) a)))\u000A  :rule-classes ((:meta :trigger-fns (<see topic=\"ACL2____BINARY-_B2\">binary-+</see>))))</code> \u000A \u000A <p>The two hypotheses illustratate the two basic kinds of meta-extract \u000A hypotheses: applications of the evaluator to a call of \u000A <v>meta-extract-global-fact</v> and to a call of \u000A <v>meta-extract-contextual-fact</v>.  Here is the definition of the metafunction \u000A used in the above rule, slightly simplified here from what is found in the \u000A above book (but adequate for proving the two events that follow it in the \u000A above book).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> plus-identity-2-metafn (<see topic=\"ACL2____TERM\">term</see> mfc state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state :verify-guards nil))\u000A  (<see topic=\"ACL2____CASE-MATCH\">case-match</see> term\u000A    (('binary-+ ('bar &amp;) y)\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A      ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____META-EXTRACT-FORMULA\">meta-extract-formula</see> 'bar-posp state)\u000A              '(<see topic=\"ACL2____POSP\">POSP</see> (BAR U)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (ts= (mfc-ts y mfc state :forcep nil)\u000A                *ts-character*)\u000A           (<see topic=\"COMMON-LISP____CADR\">cadr</see> term)\u000A         term))\u000A      (t term)))\u000A    (&amp; term)))</code> \u000A \u000A <p>This metafunction returns its input term unchanged except in the case that \u000A the term is of the form <v>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar x) y)</v> and the following two \u000A conditions are met, in which case it returns <v>(bar x)</v>.</p> \u000A \u000A <code>(1)  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____META-EXTRACT-FORMULA\">meta-extract-formula</see> 'bar-posp state)\u000A            '(<see topic=\"ACL2____POSP\">POSP</see> (BAR U)))\u000A\u000A(2)  (ts= (mfc-ts y mfc state :forcep nil)\u000A          *ts-character*)</code> \u000A \u000A <p>So suppose that term is <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 'binary-+ (<see topic=\"COMMON-LISP____LIST\">list</see> 'bar x) y)</v>.  We show how \u000A the meta-extract hypotheses together with (1) and (2) imply that the \u000A conclusion of the above <v>:meta</v> rule holds.  Here is that conclusion after \u000A a bit of simplification.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (nthmeta-ev (<see topic=\"COMMON-LISP____LIST\">list</see> 'binary-+ (<see topic=\"COMMON-LISP____LIST\">list</see> 'bar x) y) a)\u000A       (nthmeta-ev (<see topic=\"COMMON-LISP____LIST\">list</see> 'bar x) a))</code> \u000A \u000A <p>This equality simplifies as follows using the evaluator properties of \u000A <v>nthmeta-ev</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (bar (nthmeta-ev x a))\u000A                 (nthmeta-ev y a))\u000A       (bar (nthmeta-ev x a)))</code> \u000A \u000A <p>Since a positive number plus a character is that number, it clearly \u000A suffices to show:</p> \u000A \u000A <code>(A)  (<see topic=\"ACL2____POSP\">posp</see> (bar (nthmeta-ev x a)))\u000A\u000A(B)  (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (nthmeta-ev y a))</code> \u000A \u000A <p>It remains then to show that these follow from (1) and (2) together with \u000A the meta-extract hypotheses.</p> \u000A \u000A <p>First consider (A).  We show that it is just a simplification of the first \u000A meta-extract hypothesis.</p> \u000A \u000A <code>(nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT\">meta-extract-global-fact</see> '(:formula bar-posp)\u000A                                      state)\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'u\u000A                        (nthmeta-ev (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> term))\u000A                                    a))))\u000A= {by our assumption that term is (<see topic=\"COMMON-LISP____LIST\">list</see> 'binary-+ (<see topic=\"COMMON-LISP____LIST\">list</see> 'bar x) y)}\u000A(nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT\">meta-extract-global-fact</see> '(:formula bar-posp)\u000A                                      state)\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'u\u000A                        (nthmeta-ev x a))))\u000A= {by definition of meta-extract-global-fact, as discussed later}\u000A(nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-FORMULA\">meta-extract-formula</see> 'bar-posp state)\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'u\u000A                        (nthmeta-ev x a))))\u000A= {by (1)}\u000A(nthmeta-ev '(<see topic=\"ACL2____POSP\">posp</see> (bar u))\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'u\u000A                        (nthmeta-ev x a))))\u000A= {by evaluator properties of nthmeta-ev}\u000A(<see topic=\"ACL2____POSP\">posp</see> (bar (nthmeta-ev x a)))</code> \u000A \u000A <p>Now consider (B).  We show that it is just a simplification of the second \u000A meta-extract hypothesis.</p> \u000A \u000A <code>(nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see>\u000A             `(:typeset ,(<see topic=\"COMMON-LISP____CADDR\">caddr</see> term)) mfc state)\u000A            a)\u000A= {by our assumption that term is (<see topic=\"COMMON-LISP____LIST\">list</see> 'binary-+ (<see topic=\"COMMON-LISP____LIST\">list</see> 'bar x) y)}\u000A(nthmeta-ev (<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see> (<see topic=\"COMMON-LISP____LIST\">list</see> ':typeset y) mfc state)\u000A            a)\u000A= {by definition of meta-extract-contextual-fact, as discussed later}\u000A(nthmeta-ev (<see topic=\"COMMON-LISP____LIST\">list</see> 'typespec-check\u000A                  (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote\u000A                        (mfc-ts y mfc state :forcep nil))\u000A                  y)\u000A            a)\u000A= {by (2)}\u000A(nthmeta-ev (<see topic=\"COMMON-LISP____LIST\">list</see> 'typespec-check\u000A                  (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote *ts-character*)\u000A                  y)\u000A            a)\u000A= {by evaluator properties of nthmeta-ev}\u000A(<see topic=\"ACL2____TYPESPEC-CHECK\">typespec-check</see> *ts-character* (nthmeta-ev y a))\u000A= {by definition of typespec-check}\u000A(<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (nthmeta-ev y a))</code> \u000A \u000A <p>Note the use of <v>:forcep nil</v> above.  All of the <v>mfc-xx</v> functions \u000A take a keyword argument <v>:forcep</v>.  Calls of <v>mfc-xx</v> functions made on \u000A behalf of <v>meta-extract-contextual-fact</v> always use <v>:forcep nil</v>, so in \u000A order to reason about these calls in your own metafunctions, you will want to \u000A use <v>:forcep nil</v>.  We have contemplated adding a utility like \u000A <v>meta-extract-contextual-fact</v> that allows forcing but returns a tag-tree \u000A (see <see topic=\"ACL2____TTREE\">ttree</see>), and may do so if there is demand for it.</p> \u000A \u000A <p>Finally, we document what is provided logically by calls of \u000A <v>meta-extract-global-fact</v> and <v>meta-extract-contextual-fact</v>.  Of \u000A course, you are invited to look at the definitions of these function in the \u000A ACL2 source code, or by using <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>.  Note that both of these \u000A functions are non-executable (each of their bodies is inside a call of <tt><see topic=\"ACL2____NON-EXEC\">non-exec</see></tt>); their purpose is purely logical, not for execution.  The functions \u000A return <v>*t*</v>, i.e., <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> t)</v>, in cases that they provide no \u000A information.</p> \u000A \u000A <p>First we consider the value of <v>(<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT\">meta-extract-global-fact</see> obj state)</v> \u000A for various values of <v>obj</v>.  When we refer below to concepts like ``body'' \u000A and ``evaluation'', we refer to these with respect to the logical world of the \u000A input <v>state</v>.</p> \u000A \u000A <blockquote><p>Case <v>obj</v> = <v>(<see topic=\"COMMON-LISP____LIST\">list</see> :formula FN)</v>:<br></br> \u000A \u000A The value reduces to the value of <v>(<see topic=\"ACL2____META-EXTRACT-FORMULA\">meta-extract-formula</see> FN state)</v>, which \u000A returns the ``formula'' of <v>FN</v> in the following sense.  If <v>FN</v> is a \u000A function symbol with formals <v>(X1 ... Xk)</v>, then the formula is the <see topic=\"ACL2____CONSTRAINT\">constraint</see> on <v>FN</v> if <v>FN</v> is constrained or introduced by <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>, and otherwise is <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (FN X1 ... Xk) BODY)</v>, where <v>BODY</v> \u000A is the (unsimplified) body of the definition of <v>FN</v>.  Otherwise, if \u000A <v>FN</v> is the name of a theorem, the formula is just what is stored for that \u000A theorem.  Otherwise, the formula is <v>*t*</v>.</p> \u000A \u000A <p>Case <v>obj</v> = <v>(<see topic=\"COMMON-LISP____LIST\">list</see> :lemma FN N)</v>:<br></br> \u000A \u000A Assume <v>N</v> is a natural number; otherwise, treat <v>N</v> as 0.  If <v>FN</v> \u000A is a function symbol with more than <v>N</v> associated lemmas &mdash; \u000A ``associated'' in the sense of being either a <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule \u000A for <v>FN</v> or a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule for <v>FN</v> whose left-hand side \u000A has a top function symbol of <v>FN</v> &mdash; then the value is the <v>N</v>th \u000A such lemma (with zero-based indexing).  Otherwise the value is <v>*t*</v>.</p> \u000A \u000A <p>Case <v>obj</v> = <v>(<see topic=\"COMMON-LISP____LIST\">list</see> :fncall FN ARGLIST)</v>:<br></br> \u000A \u000A Assume that <v>FN</v> is a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode function symbol and that \u000A <v>ARGLIST</v> is a true list of values of the same length as list of formal \u000A parameters for <v>FN</v> (i.e., as the input arity of <v>FN</v>).  Also assume \u000A that the application of <v>FN</v> to actual parameter list <v>ARGLIST</v> returns \u000A a result <v>(<see topic=\"ACL2____MV\">mv</see> nil x)</v>.  Let <v>(QARG1 ... QARGk)</v> be the result of quoting \u000A each element of <v>ARGLIST</v>, i.e., replacing each <v>y</v> in <v>ARGLIST</v> by \u000A the two-element list <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> y)</v>.  Then the value is the term <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (FN\u000A QARG1 ... QARGk) (<see topic=\"COMMON-LISP____QUOTE\">quote</see> x))</v>.</p> \u000A \u000A <p>For any other values of <v>obj</v>, the value is <v>*t*</v>.</p></blockquote> \u000A \u000A <p>Finally, the value of <v>(<see topic=\"ACL2____META-EXTRACT-CONTEXTUAL-FACT\">meta-extract-contextual-fact</see> obj mfc state)</v> is \u000A as follows for various values of <v>obj</v>.  Note a difference from the \u000A semantics of <v>meta-extract-global-fact</v>: below, the relevant logical world \u000A is the one stored in the metafunction context, <v>mfc</v>, not in the input \u000A <v>state</v>.</p> \u000A \u000A <blockquote><p>Case <v>obj</v> = (list :typeset TERM ...):<br></br> \u000A \u000A The value is the value of <v>(<see topic=\"ACL2____TYPESPEC-CHECK\">typespec-check</see> ts TERM)</v>, where <v>ts</v> is the \u000A value of <v>(mfc-ts TERM mfc state :forcep nil :ttreep nil)</v>, and where \u000A <v>(<see topic=\"ACL2____TYPESPEC-CHECK\">typespec-check</see> ts val)</v> is defined to be true when <v>val</v> has type-set \u000A <v>ts</v>.  (Exception: If <v>val</v> satisfies <v>bad-atom</v> then \u000A <v>typespec-check</v> is true when <v>ts</v> is negative.)</p> \u000A \u000A <p>Case <v>obj</v> = (list :rw+ TERM ALIST OBJ EQUIV ...):<br></br> \u000A \u000A We assume below that <v>EQUIV</v> is a symbol that represents an equivalence \u000A relation, where <v>nil</v> represents <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <v>t</v> represents <tt><see topic=\"ACL2____IFF\">iff</see></tt>, and otherwise <v>EQUIV</v> represents itself (an equivalence relation in \u000A the current logical <see topic=\"ACL2____WORLD\">world</see>).  For any other <v>EQUIV</v> the value is \u000A <v>*t*</v>.  Now let <v>rhs</v> be the value of <v>(mfc-rw+ TERM ALIST OBJ EQUIV\u000A mfc state :forcep nil :ttreep nil)</v>.  Then the value is the term <v>(<see topic=\"COMMON-LISP____LIST\">list</see>\u000A 'equv (sublis-var ALIST TERM) rhs)</v>, where equv is the equivalence relation \u000A represented by <v>EQUIV</v>, and <v>sublis-var</v> is defined to substitute a \u000A variable-binding alist into a term.</p> \u000A \u000A <p>Case <v>obj</v> = (list :rw TERM OBJ EQUIV ...):<br></br> \u000A \u000A The value is the same as above but for an <v>ALIST</v> of <v>nil</v>, i.e., for \u000A the case that <v>obj</v> is <v>(<see topic=\"COMMON-LISP____LIST\">list</see> :rw+ TERM nil OBJ EQUIV ...)</v>.</p> \u000A \u000A <p>Case <v>obj</v> = (list :ap TERM ...):<br></br> \u000A \u000A The value is <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 'not TERM)</v> if <v>(mfc-ap TERM mfc state :forcep nil)</v> \u000A is true, else is <v>*t*</v>.</p> \u000A \u000A <p>Case <v>obj</v> = (list :relieve-hyp HYP ALIST RUNE TARGET BKPTR \u000A ...):<br></br> \u000A \u000A The value is <v>(sublis-var alist hyp)</v> &mdash; see above for a discussion of \u000A <v>sublis-var</v> &mdash; if the following is true.</p> \u000A \u000A <code>(mfc-relieve-hyp hyp alist rune target bkptr mfc state\u000A                 :forcep nil :ttreep nil)</code> \u000A \u000A <p>Otherwise the value is <v>*t*</v>.</p> \u000A \u000A <p>If no case above applies, then the value is <v>*t*</v>.</p></blockquote> \u000A \u000A <p>We conclude by considering the fourth of the four forms above (and \u000A implicitly, its special case represented by the third form above):</p> \u000A \u000A <code>(evl (<see topic=\"ACL2____META-EXTRACT-GLOBAL-FACT_B2\">meta-extract-global-fact+</see> obj st state) aa)</code> \u000A \u000A <p>The discussion above is for the function <v>meta-extract-global-fact+</v>, \u000A but assumes that the logical <see topic=\"ACL2____WORLD\">world</see>s of <v>st</v> and <v>state</v> are \u000A equal; otherwise the value returned is <v>*t*</v>.  Of course, since a call of \u000A <v>meta-extract-global-fact</v> expands to a corresponding call of \u000A <v>meta-extract-global-fact+</v> in which the last two arguments are both \u000A <v>state</v>, that condition holds automatically for that case.  But the \u000A <v>state</v> mentioned in the meta-extract hypotheses of a <see topic=\"ACL2____META\">meta</see> rule or \u000A <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> rule is in essence an initial state.  In the case of a \u000A clause-processor rule, the clause-processor function may modify that initial \u000A state (say, by printing or modifying some state globals) without changing its \u000A world, and then pass that modified state to <v>fncall-term</v>.  While \u000A <v>fncall-term</v> may produce a different result for this modified state than \u000A for the initial state, both are valid: the state used for heuristic purposes, \u000A such as determining whether guard-checking may cause an error.  A useful \u000A instance of the hypothesis displayed above will be one in which <v>st</v> is \u000A that modified state.</p>"],

 "ACL2____ELIM":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <p>The following example of an <v>:elim</v> rule is an important one, and is \u000A built into ACL2.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> car-cdr-elim\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) x))\u000A  :rule-classes :elim)</code> \u000A \u000A <p>The class of <v>:elim</v> rules is fundamentally quite different from the \u000A more common class of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules.  Briefly put, a \u000A <v>:rewrite</v> rule replaces instances of its left-hand side with corresponding \u000A instances of its right-hand side.  But an <v>:elim</v> rule, on the other hand, \u000A has the effect of generalizing so-called ``destructor'' function applications \u000A to variables.  In essence, applicability of a <v>:rewrite</v> rule is based on \u000A matching its left-hand side, while applicability of an <v>:elim</v> rule is \u000A based on the presence of at least one destructor term.</p> \u000A \u000A <p>For example, a conjecture about <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> can be \u000A replaced by a conjecture about new variables <v>x1</v> and <v>x2</v>, as shown in \u000A the following example.  (Run the command <v>:mini-proveall</v> and search for \u000A <v>CAR-CDR-ELIM</v> to see the full proof containing this excerpt.)</p> \u000A \u000A <code>Subgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (APP (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))).\u000A\u000AThe destructor terms (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) and (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) can be eliminated by using\u000ACAR-CDR-ELIM to replace X by (<see topic=\"COMMON-LISP____CONS\">CONS</see> X1 X2), (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) by X1 and (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)\u000Aby X2.  This produces the following goal.\u000A\u000ASubgoal *1/1''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X1 X2))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV X2)))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (APP (REV X2) (<see topic=\"COMMON-LISP____LIST\">LIST</see> X1)))).\u000A\u000AThis simplifies, using primitive type reasoning, to\u000A\u000ASubgoal *1/1'''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV X2))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (APP (REV X2) (<see topic=\"COMMON-LISP____LIST\">LIST</see> X1)))).</code> \u000A \u000A <p>The resulting conjecture is often simpler and hence more amenable to \u000A proof.</p> \u000A \u000A <p>The application of an <v>:elim</v> rule thus replaces a variable by a term \u000A that contains applications of so-called ``destructor'' functions to that \u000A variable.  The example above is typical: the variable <v>x</v> is replaced by \u000A the term <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v>, which applies a so-called \u000A ``constructor'' function, <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, to applications <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> and \u000A <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> of destructor functions <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> to that same \u000A variable, <v>x</v>.  But that is only part of the story.  ACL2 then generalizes \u000A the destructor applications <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> to new variables \u000A <v>x1</v> and <v>x2</v>, respectively, and ultimately the result is a simpler \u000A conjecture.</p> \u000A \u000A <p>More generally, the application of an <v>:elim</v> rule replaces a variable \u000A by a term containing applications of destructors; there need not be a \u000A clear-cut notion of ``constructor.''  But the situation described above is \u000A typical, and we will focus on it, giving full details when we introduce the \u000A ``General Form'' below.</p> \u000A \u000A <p>Notice that the situation can be complicated a bit by a rule's hypotheses. \u000A For example, the replacement specified by the rule <v>car-cdr-elim</v> (shown \u000A near the beginning of this discussion) is only valid if the variable being \u000A replaced is a cons structure.  Thus, when ACL2 applies <v>car-cdr-elim</v> to \u000A replace a variable <v>v</v>, it will split into two cases: one case in which \u000A <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> v)</v> is true, in which <v>v</v> is replaced by <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> v) (<see topic=\"COMMON-LISP____CDR\">cdr</see>\u000A v))</v> and then <v>(<see topic=\"COMMON-LISP____CAR\">car</see> v)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> v)</v> are generalized to new variables; \u000A and one case in which <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> v)</v> is false.  In practice, <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> v)</v> is \u000A often provable, perhaps even literally present as a hypotheses; then of course \u000A there is no need to introduce the second case.  That is why there is no such \u000A second case in the example above.</p> \u000A \u000A <p>You might find <v>:elim</v> rules to be useful whenever you have in mind a \u000A data type that can be built up from its fields with a ``constructor'' function \u000A and whose fields can be accessed by corresponding ``destructor'' functions. \u000A So for example, if you have a ``house'' data structure that represents a house \u000A in terms of its address, price, and color, you might have a rule like the \u000A following.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (house-p x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (make-house (address x)\u000A                            (price x)\u000A                            (color x))\u000A                x))</code> \u000A \u000A <p>The application of such a rule is entirely analogous to the application of \u000A the rule <v>car-cdr-elim</v> discussed above.  We discuss such rules and their \u000A application more carefully below.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> hyp (equiv lhs x))</code> \u000A \u000A <p>where <v>equiv</v> is a known equivalence relation (see <see topic=\"ACL2____DEFEQUIV\">defequiv</see>); \u000A <v>x</v> is a variable symbol; and <v>lhs</v> contains one or more terms (called \u000A ``destructor terms'') of the form <v>(fn v1 ... vn)</v>, where <v>fn</v> is a \u000A function symbol and the <v>vi</v> are distinct variable symbols, <v>v1</v>, ..., \u000A <v>vn</v> include all the variable symbols in the formula, no <v>fn</v> occurs in \u000A <v>lhs</v> in more than one destructor term, and all occurrences of <v>x</v> in \u000A <v>lhs</v> are inside destructor terms.</p> \u000A \u000A <p>To use an <v>:elim</v> rule, the theorem prover waits until a conjecture has \u000A been maximally simplified.  It then searches for an instance of some \u000A destructor term <v>(fn v1 ... vn)</v> in the conjecture, where the instance for \u000A <v>x</v> is some variable symbol, <v>vi</v>, and every occurrence of <v>vi</v> \u000A outside the destructor terms is in an <v>equiv</v>-hittable position.  If such \u000A an instance is found, then the theorem prover instantiates the <v>:elim</v> \u000A formula as indicated by the destructor term matched; splits the conjecture \u000A into two goals, according to whether the instantiated hypothesis, <v>hyp</v>, \u000A holds; and in the case that it does hold, generalizes all the instantiated \u000A destructor terms in the conjecture to new variables and then replaces <v>vi</v> \u000A in the conjecture by the generalized instantiated <v>lhs</v>.  An occurrence of \u000A <v>vi</v> is ``<v>equiv</v>-hittable'' if sufficient congruence rules (see <see topic=\"ACL2____DEFCONG\">defcong</see>) have been proved to establish that the propositional value of the \u000A clause is not altered by replacing that occurrence of <v>vi</v> by some \u000A <v>equiv</v>-equivalent term.</p> \u000A \u000A <p>If an <v>:elim</v> rule is not applied when you think it should have been, \u000A and the rule uses an equivalence relation, <v>equiv</v>, other than <v>equal</v>, \u000A it is most likely that there is an occurrence of the variable that is not \u000A <v>equiv</v>-hittable.  Easy occurrences to overlook are those in the governing \u000A hypotheses.  If you see an unjustified occurrence of the variable, you must \u000A prove the appropriate congruence rule to allow the <v>:elim</v> to fire.</p> \u000A \u000A <p>Further examples of how ACL2 <v>:elim</v> rules are used may be found in the \u000A corresponding discussion of ``Elimation of Destructors'' for Nqthm, in Section \u000A 10.4 of A Computational Logic Handbook.</p>"],

 "ACL2____SET-PROVER-STEP-LIMIT":[["Other"],"ACL2 Sources","ACL2","<p>This event provides a way to limit the number of so-called ``prover \u000A steps'' permitted for an event.  See <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see> for a way \u000A to specify the limit on prover steps for a single event, rather than globally. \u000A For a related utility based on time instead of prover steps, see <see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>.  For examples of how step limits work, see the \u000A community book <v>books/misc/misc2/step-limits.lisp</v>.  For a utility that \u000A returns an indicator of the number of prover steps most recently taken, see \u000A <see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see>.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> *default-step-limit*) ; no limit on prover steps\u000A(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> nil)   ; abbreviation for the form just above\u000A(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> 10000) ; allow at most 10,000 prover steps per event\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> expr)</code> \u000A \u000A <p>where <v>expr</v> evaluates either to <v>nil</v> or else to a natural number \u000A not exceeding the value of <v>*default-step-limit*</v>.  If that value is \u000A <v>nil</v> or the value of <v>*default-step-limit*</v>, then no default limit is \u000A placed on the number of prover ``steps'' (see below) during processing of an \u000A event.  Otherwise, that value is the maximum number of prover steps permitted \u000A before an error occurs.</p> \u000A \u000A <p>This event specifies the limit on the number of ``steps'' counted by the \u000A ACL2 prover during processing of an event.  Currently, a step is counted for \u000A each call of the system functions <v>rewrite</v> and <v>expand-abbreviations</v>. \u000A However, the steps counted may change in future releases of ACL2, so users \u000A would probably be well served by avoiding the assumption that only the above \u000A two calls are counted as prover steps.</p> \u000A \u000A <p>Depending on the computer you are using, you may have less than a half-hour \u000A of time before the number of prover steps exceeds the maximum step-limit, \u000A which is one less than the value of <v>*default-step-limit*</v>.  Note however \u000A the exception stated above: if the ``limit'' is <v>nil</v> or is the value of \u000A <v>*default-step-limit*</v>, then no limit is imposed.</p> \u000A \u000A <p>There is at best a loose connection between the counting of steps and \u000A <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>.  In particular, for a call of <v>mfc-rw</v> or \u000A any <v>mfc-</v> function (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>), the steps taken \u000A during that call are forgotten when returning from that call.</p> \u000A \u000A <p>The limit is relevant for every event, as well as for calls of <tt><see topic=\"ACL2____THM\">thm</see></tt> \u000A and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> &mdash; and more generally, to any form that creates \u000A a ``summary context'' to print the usual event summary.  The limit is also put \u000A in force when entering the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.  A call of \u000A <v>set-prover-step-limit</v> applies to each subsequent form unless the call of \u000A <v>set-prover-step-limit</v> is within a summary context, in which case its \u000A effect disappears when exiting that summary context.</p> \u000A \u000A <p>The limit applies to each event, not just ``atomic'' events.  Consider the \u000A following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> 500)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ()\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-1 ; takes 380 steps\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A    :rule-classes nil)\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2 ; would take 319 steps\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)) (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y)))\u000A    :rule-classes nil))</code> \u000A \u000A <p>The first <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event, <v>lemma-1</v> takes 380 steps (as of this \u000A writing), as shown in the summary:</p> \u000A \u000A <code>Prover steps counted:  380\u000ALEMMA-1</code> \u000A \u000A <p>The second <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event, <v>lemma-2</v>, takes 319 steps (as of this \u000A writing) when evaluated at the top level.  However, in the context above, 380 \u000A steps of the available 500 steps (from the <v>set-prover-step-limit</v> event \u000A above) have already been taken under the above <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event. \u000A Thus, when the number of steps would exceed 120, the proof of <v>lemma-2</v> is \u000A aborted:</p> \u000A \u000A <code>ACL2 Error in STEP-LIMIT:  The prover step-limit, which is 120 in the\u000Acurrent context, has been exceeded.  See :DOC set-prover-step-limit.</code> \u000A \u000A <p>The summary for <v>lemma-2</v> reflects that situation:</p> \u000A \u000A <code>Prover steps counted:  More than 120</code> \u000A \u000A <p>The summary for the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event then indicates that the \u000A available steps for that event have also been exceeded:</p> \u000A \u000A <code>Prover steps counted:  More than 500</code> \u000A \u000A <p>The discussion above applies to any event that contains other events, hence \u000A applies similarly to <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> events.</p> \u000A \u000A <p>For those who use <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, we note that prover steps in the \u000A expansion phase similarly contribute to the total number of steps counted. \u000A For example, suppose that the limit is 500 prover steps as above, and you \u000A submit <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> EXPR)</v>, where 300 prover steps take place during \u000A evaluation of <v>EXPR</v>, producing event <v>EV</v>.  Then evaluation of <v>EV</v> \u000A will cause an error if it takes more than 200 prover steps.  This observation \u000A actually can be used to count prover steps for sequences of forms that are not \u000A all legal <tt><see topic=\"ACL2____EVENTS\">events</see></tt> (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), such as calls of \u000A <tt><see topic=\"ACL2____THM\">thm</see></tt>.  For example, a small built-in ACL2 test suite that includes \u000A <tt><see topic=\"ACL2____THM\">thm</see></tt> forms can be run by evaluating the form <v>(mini-proveall)</v>, and \u000A the steps can be counted as shown below.  (Here we assume a fresh ACL2 \u000A session; an error would occur if first, we evaluate the event \u000A <v>(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> 500)</v> displayed above.)</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (mini-proveall) (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))\u000A[[... output omitted here ...]]\u000ASummary\u000AForm:  ( MAKE-EVENT (<see topic=\"ACL2____ER-PROGN\">ER-PROGN</see> ...))\u000ARules: NIL\u000AWarnings:  Double-rewrite, Equiv, Subsume and Non-rec\u000ATime:  0.38 seconds (prove: 0.04, print: 0.29, other: 0.05)\u000AProver steps counted:  41090\u000A NIL\u000AACL2 !&gt;</code>"],

 "ACL2____BOOKS-CERTIFICATION-CLASSIC":[["Books-certification"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic explains an approach to certifying \u000A directories of books, which we call ``classic ACL2 `make'-based \u000A certification''.</p> \u000A \u000A <p>Warning: The capability described in this section might be replaced at any \u000A time by a capability based on corresponding support for community books (see \u000A <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see>).  If you think that would be a hardship, please \u000A contact the ACL2 implementors.</p> \u000A \u000A <p>This topic discusses a way to certify a directory of books other than the \u000A ACL2 community books.  See <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see> for how to certify the \u000A set of ACL2 community <see topic=\"ACL2____BOOKS\">books</see>.  There is also a section in that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic, ``Restricting to Specific Directories and Books'', that \u000A provides an alternative to classic ACL2 `make'-based certification (as \u000A discussed in the present topic) for certifying specified sets of books.</p> \u000A \u000A <p>We assume here a familiarity with Unix/Linux `make'.  We also assume that \u000A you are using GNU `make' rather than some other flavor of `make'.  And \u000A finally, we assume, as is typically the case by following the standard \u000A installation instructions, that you install the ACL2 community books in the \u000A <v>books/</v> subdirectory of your ACL2 distribution.  We will refer below to \u000A that directory as <v>BOOKS</v>.</p> \u000A \u000A <p>In summary: to use `make' to certify <see topic=\"ACL2____BOOKS\">books</see> under a given directory, \u000A you may create a simple Makefile in that directory (as explained below) so \u000A that when you stand in that directory, you can submit the command, `make', to \u000A certify those books.  If you have a multi-processor machine or the like, then \u000A you can use the `<v>-j</v> flag `make'-level parallelism by specifying the \u000A number of concurrent processes.  For example:</p> \u000A \u000A <code>make -j 4</code> \u000A \u000A <p>For each book <v>foo.lisp</v>, a file <v>foo.out</v> in the same directory as \u000A <v>foo.lisp</v> will contain the output from the corresponding certification \u000A attempt.  If you have previously executed such a command, then you might first \u000A want to delete <see topic=\"ACL2____CERTIFICATE\">certificate</see> files and other generated files by executing \u000A the following command.</p> \u000A \u000A <code>make clean</code> \u000A \u000A <p>Note that when you run `make', then by default, the first error will cause \u000A the process to stop.  You can use <v>make -i</v> to force `make' to ignore \u000A errors, thus continuing past them.  Or, use <v>make -k</v> to keep going, but \u000A skipping certification for any book that includes another whose certification \u000A has failed.</p> \u000A \u000A <p>By default, your acl2-customization file (see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>) is \u000A ignored by such `make' commands.  However, you can specify the use of an \u000A acl2-customization file by setting the value of environment variable \u000A <v>ACL2_CUSTOMIZATION</v> to the empty string, indicating a default such file, \u000A or to the desired absolute pathname.  For example:</p> \u000A \u000A <code>make ACL2_CUSTOMIZATION=''\u000Amake ACL2_CUSTOMIZATION='~/acl2-customization.lisp'</code> \u000A \u000A <p>We now discuss how to create makefiles to support `make' commands as \u000A discussed above.</p> \u000A \u000A <p>First we give five steps for creating a <v>Makefile</v> to support \u000A certification of a directory of books, without subdirectories.  For examples \u000A of such Makefiles you can look in community book directories (which, however, \u000A might disappear in future versions of ACL2).</p> \u000A \u000A <blockquote><p>1. Include the file <v>Makefile-generic</v> from the <v>books/</v> \u000A subdirectory of your ACL2 sources directory, but first perhaps define the \u000A variable `<v>ACL2</v>'.  Consider the following example.</p> \u000A \u000A <code>ACL2 ?= /Users/john_doe/acl2/acl2-sources/saved_acl2\u000Ainclude /Users/john_doe/acl2/acl2-sources/books/Makefile-generic</code> \u000A \u000A <p>In this example, you can omit the first line, because the default ACL2 \u000A executable is file <v>saved_acl2</v> in the directory immediately above the \u000A directory of the specified <v>Makefile-generic</v> file.  Indeed, that is the \u000A common case.  Note the use of <v>?=</v> instead of <v>=</v> or <v>:=</v>, so that \u000A <v>ACL2</v> can instead be defined by the environment or provided on the command \u000A line as part of the `make' command.</p> \u000A \u000A <p>2. (Optional; usually skipped.)  Set the <v>INHIBIT</v> variable if you want \u000A to see more than the summary output.  For example, if you want to see the same \u000A output as you would normally see at the terminal, put this line in your \u000A Makefile after the `<v>include</v>' lines.</p> \u000A \u000A <code>INHIBIT = (<see topic=\"ACL2____ASSIGN\">assign</see> inhibit-output-lst (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> proof-tree)))</code> \u000A \u000A <p>For other values to use for <v>INHIBIT</v>, see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> \u000A and see the original setting of <v>INHIBIT</v> in \u000A <v>books/Makefile-generic</v>.</p> \u000A \u000A <p>3. Specify the books to be certified.  Normally, every file with extension \u000A <v>.lisp</v> will be a book that you want to certify, in which case you can skip \u000A this step.  Otherwise, put a line in your <v>Makefile</v> after the ones above \u000A that specifies the books to be certified.  The following example, from an old \u000A version of community books file <v>books/finite-set-theory/osets/Makefile</v>, \u000A should make this clear.</p> \u000A \u000A <code>BOOKS = computed-hints fast instance map membership outer primitives \\\u000A        quantify set-order sets sort</code> \u000A \u000A <p>But better yet, use the extension <v>.lsp</v> for any Lisp or ACL2 files that \u000A are not to be certified, so that the definition of <v>BOOKS</v> can be \u000A omitted.</p> \u000A \u000A <p>4. Create <v>.acl2</v> files for books that are to be certified in other than \u000A the initial ACL2 world (see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>).  For example, if you look in \u000A community books file <v>books/arithmetic/equalities.acl2</v> you will see <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> forms followed by a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command, because it was \u000A determined that <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> forms were necessary in the certification world \u000A in order to certify the <v>equalities</v> book.  In general, for each \u000A <v>&lt;book-name&gt;.lisp</v> whose certification requires a non-initial certification \u000A world, you will need a corresponding <v>&lt;book-name&gt;.acl2</v> file that ends with \u000A the appropriate <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command.</p> \u000A \u000A <p>You also have the option of creating a file <v>cert.acl2</v> that has a \u000A special role.  When file <v>&lt;book-name&gt;.lisp</v> is certified, if there is no \u000A file <v>&lt;book-name&gt;.acl2</v> but there is a file <v>cert.acl2</v>, then \u000A <v>cert.acl2</v> will be used as <v>&lt;book-name&gt;.acl2</v> would have been used, as \u000A described in the preceding paragraph, except that the appropriate <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command will be generated automatically.  Thus, no \u000A <v>certify-book</v> command should occur in <v>cert.acl2</v>.</p> \u000A \u000A <p>It is actually allowed to put raw lisp forms in a <v>.acl2</v> file \u000A (presumably preceded by <v>:q</v> or <v>(value :q)</v> and followed by <v>(<see topic=\"ACL2____LP\">lp</see>)</v>). \u000A But this is not recommended; we make no guarantees about certification \u000A performed any time after raw Lisp has been entered in the ACL2 session.</p> \u000A \u000A <p>5. Generally, the next step is to include the following line after the \u000A `<v>include</v>' of <v>Makefile-generic</v> (see the first step above).</p> \u000A \u000A <code>-include Makefile-deps</code> \u000A \u000A <p>This will cause `make' to create and then include a file <v>Makefile-deps</v> \u000A that contains ``dependency'' lines needed by `make'.  If those dependencies \u000A are somehow flawed, it may be because you have <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms that \u000A are not truly including books, for example in multi-line comments \u000A (<v>#|..|#</v>).  These will be ignored if preceded by a semicolon (<v>;</v>), or \u000A if you add a line break after ``<v>include-book</v>.''  But instead of adding \u000A the `<v>-include</v>' line above, you can create dependency lines yourself by \u000A running the command</p> \u000A \u000A <code>make dependencies</code> \u000A \u000A <p>and pasting the result into the end of your <v>Makefile</v>, and editing as \u000A you see fit.</p></blockquote> \u000A \u000A <p>This concludes the basic instructions for creating a <v>Makefile</v> in a \u000A directory including books.  Here are some other capabilities offered by \u000A community books file <v>books/Makefile-subdirs</v>.  Not included below is a \u000A discussion of how to increase parallelism by avoiding the need to certify \u000A included books before certifying a given book; see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.</p> \u000A \u000A <p><b>Subdirectory Support</b></p> \u000A \u000A <p>There is support for using `make' to certify books in subdirectories. \u000A Consider the following example.</p> \u000A \u000A <code>DIRS = pass1 bind-free floor-mod\u000Ainclude ../Makefile-subdirs</code> \u000A \u000A <p>This indicates that we are to run `make' in subdirectories <v>pass1/</v>, \u000A <v>bind-free/</v>, and <v>floor-mod/</v> of the current directory.</p> \u000A \u000A <p>You can combine this subdirectory support with the support already \u000A discussed for certifying books in the top-level directory.  Here is an \u000A example, which as of this writing is in community books file \u000A <v>books/arithmetic-3/Makefile</v> contains the following lines.</p> \u000A \u000A <code>arith-top: top all\u000Aall: top\u000A\u000ADIRS = pass1 bind-free floor-mod\u000Ainclude ../Makefile-subdirs\u000Ainclude ../Makefile-generic\u000A\u000A-include Makefile-deps</code> \u000A \u000A <p>The `<v>top</v>' target is defined in <v>../Makefile-subdirs</v> to call `make' \u000A in each subdirectory specified in <v>DIRS</v>.  We have set the default target \u000A in the example above to a new name, <v>arith-top</v>, that makes that <v>top</v> \u000A target before making the `<v>all</v>' target which, in turn, is the default \u000A target in any <v>Makefile-generic</v>, and is responsible for certifying books \u000A in the current directory as discussed in the five steps displayed above.</p> \u000A \u000A <p>Use <v>Makefile-psubdirs</v> instead of <v>Makefile-subdirs</v> if \u000A certification of a book in a subdirectory never depends on certification of a \u000A book in a different subdirectory, because then the <v>-j</v> option of `make' \u000A can allow subdirectories to be processed in parallel.</p> \u000A \u000A <p><b>Cleaning Up</b></p> \u000A \u000A <p>We note that there is a <v>clean</v> target.  Thus,</p> \u000A \u000A <code>make clean</code> \u000A \u000A <p>will remove generated files including <v>.cert</v>, <v>.out</v> files, and \u000A compiled files.</p> \u000A \u000A <p><b>System Books</b></p> \u000A \u000A <p>An environment variable <v>ACL2_SYSTEM_BOOKS</v> is generally set \u000A automatically, so you can probably skip reading the following paragraph unless \u000A your attempt to certify books fails to locate those books properly.</p> \u000A \u000A <p>The environment variable <v>ACL2_SYSTEM_BOOKS</v> can be set to the top-level \u000A directory of the ACL2 community books.  A Unix-style pathname, typically \u000A ending in <v>books/</v> or <v>books</v>, is permissible.  In most cases, your ACL2 \u000A executable is a small script in which you can set this environment variable \u000A just above the line on which the actual ACL2 image is invoked, for \u000A example:</p> \u000A \u000A <code>export ACL2_SYSTEM_BOOKS\u000AACL2_SYSTEM_BOOKS=/home/acl2/v3-2/acl2-sources/books</code> \u000A \u000A <p>However, you can also set <v>ACL2_SYSTEM_BOOKS</v> as a `make' variable, by \u000A setting it in your <v>Makefile</v> before the first target definition, e.g.:</p> \u000A \u000A <code>ACL2_SYSTEM_BOOKS ?= /home/acl2/v3-2/acl2-sources/books</code> \u000A \u000A <p><b>Compilation Support</b></p> \u000A \u000A <p>The file <v>books/Makefile-generic</v> provides support for compiling books \u000A that are already certified (but see <see topic=\"ACL2____COMPILATION\">compilation</see> for an exception).  For \u000A example, suppose that you have certified books using GCL as the host Lisp, \u000A resulting in compiled files with the <v>.o</v> extension.  Now suppose you would \u000A like to compile the books for Allegro Common Lisp, whose compiled files have \u000A the <v>.fasl</v> extension.  The following command will work if you have \u000A included <v>books/Makefile-generic</v> in your <v>Makefile</v>.</p> \u000A \u000A <code>make fasl</code> \u000A \u000A <p>In general, the compiled file extension for a Lisp supported by ACL2 will \u000A be a target name for building compiled files for all your books (after \u000A certifying the books, if not already up-to-date on certification).</p> \u000A \u000A <p>If you run into problems, you can get help by joining the <v>acl2-help</v> \u000A email list (follow the link from the ACL2 home page) and sending a message to \u000A that list.  Also consider trying another version of GNU `make'; for example, \u000A we have found that versions 3.81 and 3.82 sometimes cause errors on Linux \u000A where version 3.80 does not.</p>"],

 "ACL2____ACL2-CUSTOMIZATION":[["Other"],"ACL2 Sources","ACL2","<p>ACL2 provides a mechanism to load automatically a so-called ``ACL2 \u000A customization file,'' via <tt><see topic=\"ACL2____LD\">ld</see></tt>, the first time <tt><see topic=\"ACL2____LP\">lp</see></tt> is called in \u000A an ACL2 session.  ACL2 looks for this file as follows.</p> \u000A \u000A <ol> \u000A \u000A <li>If the host Lisp reads a non-empty value for the system's \u000A environment variable <v>ACL2_CUSTOMIZATION</v>, then that string value is used \u000A for the customization file name.  In this case, if the file does not exist or \u000A if the string is \"NONE\" then there is no customization file.  Notes: \u000A \u000A     <ul> \u000A \u000A     <li>If the customization file name is a relative pathname (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>), then the pathname is considered relative to the connected book \u000A     directory (see <see topic=\"ACL2____CBD\">cbd</see>).</li> \u000A \u000A     <li>If this variable is not already defined, then its value is set to \u000A     <v>NONE</v> when books are certified using <see topic=\"ACL2____CERT.PL\">cert.pl</see> or other, legacy \u000A     Make-based certification tools.</li> \u000A \u000A     </ul></li> \u000A \u000A <li>Otherwise (empty environment variable value), file \u000A <v>&quot;acl2-customization.lsp&quot;</v> or <v>&quot;acl2-customization.lisp&quot;</v> on the \u000A connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>), generally the current directory, is \u000A the customization file (in that order) if either exists.</li> \u000A \u000A <li>Otherwise file <v>&quot;acl2-customization.lsp&quot;</v> or \u000A <v>&quot;acl2-customization.lisp&quot;</v> on your home directory is the customization \u000A file (in that order), if either exists (except, this case is skipped on \u000A Windows operating systems.</li> \u000A \u000A </ol> \u000A \u000A <p>Except for the fact that this <tt><see topic=\"ACL2____LD\">ld</see></tt> command is not typed explicitly by \u000A you, it is a standard <tt><see topic=\"ACL2____LD\">ld</see></tt> command, with one exception: any settings of \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> specials are remembered once this call of <tt><see topic=\"ACL2____LD\">ld</see></tt> has completed. \u000A For example, suppose that you start your customization file with \u000A <v>(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> t state)</v>, so that proofs are skipped as it is loaded \u000A with <tt><see topic=\"ACL2____LD\">ld</see></tt>.  Then the <tt><see topic=\"ACL2____LD\">ld</see></tt> special <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> will \u000A remain <v>t</v> after the <tt><see topic=\"ACL2____LD\">ld</see></tt> has completed, causing proofs to be skipped \u000A in your ACL2 session, unless your customization file sets this variable back \u000A to <v>nil</v>, say with <v>(<see topic=\"ACL2____SET-LD-SKIP-PROOFSP\">set-ld-skip-proofsp</see> nil state)</v>.</p> \u000A \u000A <p>If the customization file exists, it is loaded with <tt><see topic=\"ACL2____LD\">ld</see></tt> using the \u000A usual default values for the <tt><see topic=\"ACL2____LD\">ld</see></tt> specials (see <see topic=\"ACL2____LD\">ld</see>).  Thus, if an \u000A error is encountered, no subsequent forms in the file will be evaluated.</p> \u000A \u000A <p>To create a customization file it is recommended that you first give it a \u000A name other than <v>&quot;acl2-customization.lsp&quot;</v> or \u000A <v>&quot;acl2-customization.lisp&quot;</v> so that ACL2 does not try to include it \u000A prematurely when you next enter <tt><see topic=\"ACL2____LP\">lp</see></tt>.  Then, while in the uncustomized \u000A <tt><see topic=\"ACL2____LP\">lp</see></tt>, explicitly invoke <tt><see topic=\"ACL2____LD\">ld</see></tt> on your evolving (but renamed) \u000A customization file until all forms are successfully evaluated.  The same \u000A procedure is recommended if for some reason ACL2 cannot successfully evaluate \u000A all forms in your customization file: temporarily rename your customization \u000A file so that ACL2 does not try to <tt><see topic=\"ACL2____LD\">ld</see></tt> it automatically and then debug \u000A the new file by explicit calls to <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>WARNING!  If you certify a book after the (automatic) loading of a \u000A customization file, the forms in that file will be part of the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of the <see topic=\"ACL2____BOOKS\">books</see> you certify!  That is, the forms in your \u000A customization file at certification time will be loaded whenever anybody uses \u000A the <see topic=\"ACL2____BOOKS\">books</see> you are certifying.  Since customization files generally \u000A contain idiosyncratic <see topic=\"ACL2____COMMAND\">command</see>s, you may not want yours to be part of \u000A the <see topic=\"ACL2____BOOKS\">books</see> you create for others.  Thus, if you have a customization \u000A file then you may want to invoke <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt><v> 1</v> before certifying \u000A any <see topic=\"ACL2____BOOKS\">books</see>; alternatively, see <see topic=\"ACL2____CERTIFY-BOOK_12\">certify-book!</see> for automatic \u000A invocation of <tt><see topic=\"ACL2____UBT\">ubt</see></tt>.</p> \u000A \u000A <p>On the other hand, if you wish to prevent undoing commands from the \u000A customization file, see <see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see>.</p> \u000A \u000A <p>Finally, we note that except on Windows-based systems, if there is a file \u000A <v>acl2-init.lsp</v> in your home directory, then it will be loaded into raw \u000A Lisp when ACL2 is invoked.</p>"],

 "ACL2____PRINT-CONTROL":[["Io"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____IO\">io</see> for a summary of printing in ACL2.  Here we document \u000A some advanced ways to control what is printed by ACL2's primary printing \u000A routines.</p> \u000A \u000A <p>See <tt><see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see></tt>, <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt>, <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt>, and <tt><see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see></tt> for discussions of the most \u000A common ways to control what is printed.  Indeed, these are the only ways to \u000A control the behavior of <tt><see topic=\"ACL2____PRINC_42\">princ$</see></tt> and <v>prin1$</v>.</p> \u000A \u000A <p>See <tt><see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see></tt> for how to set the right margin for \u000A prover output and, more generally, all output from formatted printing \u000A functions (see <tt><see topic=\"ACL2____FMT\">fmt</see></tt>).  Note that <v>set-print-right-margin</v>, mentioned \u000A below, does not affect such printing.</p> \u000A \u000A <p>The rest of this topic is for advanced users of ACL2.  We refer to Common \u000A Lisp behavior, as described in any good Common Lisp documentation.</p> \u000A \u000A <p><b>Print-control variables</b>.  <tt><see topic=\"ACL2____SET-PRINT-BASE\">Set-print-base</see></tt>, <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt>, and <tt><see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see></tt> assign to corresponding so-called \u000A ``<see topic=\"ACL2____STATE\">state</see> global variables'' <v>'print-base</v>, <v>'print-radix</v>, and \u000A <v>'print-case</v>, which can be accessed using the expressions <v>(<see topic=\"ACL2_____04\">@</see>\u000A print-base)</v>, <v>(<see topic=\"ACL2_____04\">@</see> print-radix)</v>, and <v>(<see topic=\"ACL2_____04\">@</see> print-case)</v>, respectively; \u000A see <see topic=\"ACL2____ASSIGN\">assign</see>.  Here is a table showing all print-control variables, their \u000A setters, and their defaults.  Also see <tt><see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see></tt>.</p> \u000A \u000A <code>print-base          set-print-base          10\u000Aprint-case          set-print-case          :upcase\u000Aprint-circle        set-print-circle        nil\u000A  [but see remark on print-circle-files, below]\u000Aprint-escape        set-print-escape        t\u000Aprint-length        set-print-length        nil\u000Aprint-level         set-print-level         nil\u000Aprint-lines         set-print-lines         nil\u000Aprint-pretty        set-print-pretty        nil\u000Aprint-radix         set-print-radix         nil\u000Aprint-readably      set-print-readably      nil\u000Aprint-right-margin  set-print-right-margin  nil</code> \u000A \u000A <p>Each ACL2 print-control variable <v>print-xxx</v> can correspond in function \u000A to Common Lisp variable <v>*PRINT-XXX*</v>.  Specifically, the evaluation of \u000A forms <v>(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> t)</v>, <v>(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> t)</v>, and \u000A <v>(<see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> t)</v> affects ACL2 printing functions in much the same way \u000A that setting to <v>t</v> Common Lisp variables <v>*PRINT-BASE*</v>, \u000A <v>*PRINT-RADIX*</v>, and <v>*PRINT-CASE*</v>, respectively, affects Common Lisp \u000A printing.  The same is true for <v>print-escape</v>, except that this does not \u000A affect <tt><see topic=\"ACL2____PRINC_42\">princ$</see></tt> or <v>prin1$</v>, which correspond to Common Lisp \u000A functions <v>princ</v> and <v>prin1</v>: <v>princ</v> treats <v>*PRINT-ESCAPE*</v> as \u000A <v>nil</v> while <v>prin1</v> treats <v>*PRINT-ESCAPE*</v> as <v>t</v>.  Moreover, all \u000A print-control variables not mentioned in this paragraph are set to their \u000A defaults in <tt><see topic=\"ACL2____PRINC_42\">princ$</see></tt> and <v>prin1$</v>, as indicated by ACL2 constant \u000A <v>*print-control-defaults*</v>, except that <v>print-readably</v> is set to \u000A <v>nil</v> in <v>princ$</v>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____FMT\">Fmt</see></tt> and its related functions are sensitive to state globals \u000A <v>'print-base</v>, <v>'print-radix</v>, <v>'print-case</v>, <v>'print-escape</v>, and \u000A <v>'print-readably</v>, in analogy with Common Lisp functions that don't fix \u000A <v>*PRINT-ESCAPE*</v> or <v>*PRINT-READABLY*</v>.  But the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> functions \u000A do not respect settings of other print-control variables; for example, they \u000A act as though <v>'print-circle</v> is <v>nil</v>.  Since ACL2 output is produced \u000A using the same underlying print routines as the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> functions, it also \u000A is insensitive to all print-control variables except for the five above.  To \u000A control the print-level and print-length used for producing ACL2 output, see \u000A <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.</p> \u000A \u000A <p><see topic=\"ACL2____PRINT-OBJECT_42\">Print-object$</see> is sensitive to all of the print-control \u000A variables.</p> \u000A \u000A <p>Evaluate <v>(<see topic=\"ACL2____RESET-PRINT-CONTROL\">reset-print-control</see>)</v> to restore all print-control variables \u000A to their original settings, as stored in constant \u000A <v>*print-control-defaults*</v>.</p> \u000A \u000A <p>(Remark on <v>print-circle-files</v>: ACL2 typically binds <v>'print-circle</v> \u000A to <v>t</v> before writing <see topic=\"ACL2____CERTIFICATE\">certificate</see> files, or auxiliary files that are \u000A compiled when <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms are present in a book, or files in \u000A support of <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> commands.  This binding allows for structure \u000A sharing that can keep these files from growing large.  End of Remark.)</p> \u000A \u000A <p>(Remark for those using ACL2 built on CLtL1 (non-ANSI) Gnu Common \u000A Lisp (GCL): Note that Common Lisp variables <v>*PRINT-LINES*</v>, \u000A <v>*PRINT-MISER-WIDTH*</v>, <v>*PRINT-READABLY*</v>, <v>*PRINT-PPRINT-DISPATCH*</v>, \u000A and <v>*PRINT-RIGHT-MARGIN*</v> do not have any effect for such GCL \u000A versions.)</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>You might think that in order to use the theorem prover you have to \u000A know the mathematical logic supported by ACL2.  But you need to know a lot \u000A less about it than you might think.</p> \u000A \u000A <p>Technically, a theorem is a formula that can be derived from axioms by \u000A using rules of inference.  Thus, to do a proof you have to know (a) the syntax \u000A of formulas, (b) the axioms, and (c) the rules of inference.  Traditionally, \u000A these things are spelled out in excruciating detail in treatments of \u000A mathematical logic -- and for good reason.</p> \u000A \u000A <p>The whole point of proving theorems is that it is a way to determine that a \u000A formula is ``always true'' (under some model of the axioms).  By ``always \u000A true'' we actually mean what logicians mean when they say the formula is \u000A <i>valid</i>: true in the model, for all possible values of the variables. \u000A Here by ``model of the axioms'' we mean an understanding of the meaning of the \u000A various function symbols so that the axioms are true for all values of the \u000A variables.  If the variables in your conjecture can take on an infinite number \u000A of values, proof is often the <b>only</b> way to determine that a conjecture \u000A is ``always true.''  So if proof is being used to determine that a \u000A questionable formula is always true the proof must be carried out flawlessly. \u000A Thus, the (a) syntax, (b) axioms, and (c) rules of inference must be described \u000A precisely and followed to the letter.</p> \u000A \u000A <p>But formal mathematical logic was invented to explain how people reason. \u000A To the extent that logic mimics human reasoning, proofs can be seen as just \u000A extremely carefully crafted arguments.  Given that ACL2 is responsible for \u000A following the rules ``to the letter,'' your main job is ``explain'' the big \u000A leaps.</p> \u000A \u000A <p>To use the theorem prover you must understand (a) the syntax, because you \u000A must be able to write formulas flawlessly.  But you don't have to know (b) the \u000A axioms and (c) the rules of inference at nearly the same level of precision, \u000A as long as you understand the basic structure and language of proofs.</p> \u000A \u000A <p>Below is part of a proof of a certain theorem.  You ought to be able to \u000A understand the following.  Since what we describe is a proof of one case of \u000A the formula, we hope that you're <i>convinced</i> that the formula holds for \u000A that case.</p> \u000A \u000A <p>Read this and follow the links to confirm that you understand what happens. \u000A Be sure to then use your browser's <b>Back Button</b> to return to this page \u000A and continue.</p> \u000A \u000A <p><b>An Annotated Proof of</b></p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>``We will prove that reversing the reverse of a <v>true-listp</v> yields the \u000A original list.  The formula stating this is above.  We will prove it by <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF'>induction</see> on the \u000A list structure of <v>z</v>.</p> \u000A \u000A <p>The <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-BASE-CASE'>base \u000A case</see> of the induction is:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))).</code> \u000A \u000A <p>This formula is equivalent, by <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see>, to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p><see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING'>Rewriting</see> with the \u000A definition of <v>endp</v> produces:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p><see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING-REPEATEDLY'>Rewriting \u000A repeatedly</see> starting with the definition of <v>true-listp</v> produces:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z nil))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>Then using the second <i>hypothesis</i>, just <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-EQUALS-FOR-EQUALS'>substituting equals for \u000A equals</see>, we get</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z nil))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev nil)) nil))</code> \u000A \u000A <p>Since the <i>conclusion</i> involves no variables, we can <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-EVALUATION'>evaluate</see> it, getting</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z nil))\u000A         T)</code> \u000A \u000A <p>But this is an <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see> of the <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>tautology</see> \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> p T)</v>.  Thus, the base case is proved.''</p> \u000A \u000A <p>Now it is time for a little quiz.  There are just three questions.</p> \u000A \u000A <p><b>Q1</b>: The case above was the Base Case of an inductive proof of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>in which we did induction on the structure of the linear list <v>z</v>.  What \u000A is the Induction Step?  That is, what do you have to prove besides the Base \u000A Case to complete this inductive proof?</p> \u000A \u000A <p>Below are four commonly given answers; choose one.  Then look <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q1-ANSWER'>here</see> to find \u000A out if you're right.</p> \u000A \u000A <code><i>Induction Step -- Choice (i)</i>: \u000A(implies (not (endp z)) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A \u000A<i>Induction Step -- Choice (ii)</i>: \u000A(implies (true-listp (cdr z)) \u000A         (equal (rev (rev (cdr z))) (cdr z))) \u000A \u000A<i>Induction Step -- Choice (iii)</i>: \u000A(implies (and (not (endp z)) \u000A              (equal (rev (rev (cdr x))) (cdr x))) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A \u000A<i>Induction Step -- Choice (iv)</i>: \u000A(implies (and (not (endp z)) \u000A              (implies (true-listp (cdr z)) \u000A                       (equal (rev (rev (cdr z))) (cdr z)))) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p><b>Q2</b>: To prove the Induction Step we must prove one or more of the \u000A goals below.</p> \u000A \u000A <p>Which combinations are sufficient to imply the Induction Step?  Decide what \u000A is required and then look <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q2-ANSWER'>here</see> to find out if you're \u000A right.  To help you, the Induction Step is of the form:</p> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and <i>c</i> \u000A              (implies <i>p'</i> <i>q'</i>)) \u000A         (implies <i>p</i> <i>q</i>)) \u000A</code> \u000A \u000A <p>and beside each candidate subgoal we show its structure in those terms.</p> \u000A \u000A <code><i>Subgoal (i)</i>: \u000A(implies (and (not (endp z))                        ; (implies (and <i>c</i> \u000A              (true-listp z))                       ;               <i>p</i>) \u000A         (true-listp (cdr z)))                      ;          <i>p'</i>) \u000A \u000A<i>Subgoal (ii)</i>: \u000A(implies (and (not (endp z))                        ; (implies (and <i>c</i> \u000A              (true-listp z)                        ;               <i>p</i> \u000A              (equal (rev (rev (cdr z))) (cdr z)))  ;               <i>q'</i>) \u000A         (equal (rev (rev z)) z))                   ;          <i>q</i>) \u000A \u000A<i>Subgoal (iii)</i>: \u000A(implies (and (not (endp z))                        ; (implies (and <i>c</i> \u000A              (equal (rev (rev (cdr z))) (cdr z)))  ;               <i>q'</i>) \u000A         (equal (rev (rev z)) z))                   ;          <i>q</i>) \u000A \u000A<i>Subgoal (iv)</i>: \u000A(implies (and (not (endp z))                        ; (implies (and <i>c</i> \u000A              (true-listp (cdr z))                  ;               <i>p'</i> \u000A              (equal (rev (rev (cdr z))) (cdr z)))  ;               <i>q'</i>) \u000A         (equal (rev (rev z)) z))                   ;          <i>q</i>) \u000A</code> \u000A \u000A <p><b>Q3</b>: Suppose you know the theorem</p> \u000A \u000A <code><i>Theorem</i>: \u000A(implies (p (f x)) \u000A         (equal (g (h x)) \u000A                x)) \u000A</code> \u000A \u000A <p>and you wish to rewrite the target <v>(g (h a))</v> to <v>a</v> in</p> \u000A \u000A <code><i>Goal Conjecture</i>: \u000A(implies (and (q (f a)) \u000A              (r a)) \u000A         (s (g (h a)))) \u000A</code> \u000A \u000A <p>What must you prove to relieve the hypothesis of <i>Theorem</i>?</p> \u000A \u000A <p>After you've thought about it, look <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q3-ANSWER'>here</see> for our answer.</p> \u000A \u000A <p><b>End of the Quiz</b></p> \u000A \u000A <p>If this page made sense, you're ready to read the introduction to the \u000A theorem prover.</p> \u000A \u000A <p>If you are reading this as part of the tutorial introduction to the theorem \u000A prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____EXECUTABLE-COUNTERPART":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples:\u000A(:executable-counterpart length)</code> \u000A \u000A <p>which may be abbreviated in <see topic=\"ACL2____THEORIES\">theories</see> as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LENGTH\">length</see>)</code> \u000A \u000A <p>Every <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> introduces at least two rules used by the theorem \u000A prover.  Suppose <v>fn</v> is the name of a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d function.  Then \u000A <v>(:definition fn)</v> is the rune (see <see topic=\"ACL2____RUNE\">rune</see>) naming the rule that \u000A allows the simplifier to replace calls of <v>fn</v> by its instantiated body. \u000A <v>(:executable-counterpart fn)</v> is the <see topic=\"ACL2____RUNE\">rune</see> for the rule for how to \u000A evaluate the function on known constants.</p> \u000A \u000A <p>When typing <see topic=\"ACL2____THEORIES\">theories</see> it is convenient to know that <v>(fn)</v> is a \u000A runic designator that denotes <v>(:executable-counterpart fn)</v>.  See <see topic=\"ACL2____THEORIES\">theories</see>.</p> \u000A \u000A <p>If <v>(:executable-counterpart fn)</v> is <see topic=\"ACL2____ENABLE\">enable</see>d, then when \u000A applications of <v>fn</v> to known constants are seen by the simplifier they are \u000A computed out by executing the Common Lisp code for <v>fn</v> (with the \u000A appropriate handling of <see topic=\"ACL2____GUARD\">guard</see>s).  Suppose <v>fact</v> is defined as the \u000A factorial function.  If the executable counterpart <see topic=\"ACL2____RUNE\">rune</see> of <v>fact</v>, \u000A <v>(:executable-counterpart fact)</v>, is <see topic=\"ACL2____ENABLE\">enable</see>d when the simplifier \u000A encounters <v>(fact 12)</v>, then that term will be ``immediately'' expanded to \u000A <v>479001600</v>.  Note that even if subroutines of <v>fn</v> have disabled \u000A executable counterparts, <v>fn</v> will call their Lisp code nonetheless: once \u000A an executable counterpart function is applied, no subsidiary enable checks are \u000A made.</p> \u000A \u000A <p>Such one-step expansions are sometimes counterproductive because they \u000A prevent the anticipated application of certain lemmas about the subroutines of \u000A the expanded function.  Such computed expansions can be prevented by disabling \u000A the executable counterpart <see topic=\"ACL2____RUNE\">rune</see> of the relevant function.  For example, \u000A if <v>(:executable-counterpart fact)</v> is <see topic=\"ACL2____DISABLE\">disable</see>d, <v>(fact 12)</v> will \u000A not be expanded by computation.  In this situation, <v>(fact 12)</v> may be \u000A rewritten to <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 12 (fact 11))</v>, using the rule named <v>(:definition\u000A fact)</v>, provided the system's heuristics permit the introduction of the term \u000A <v>(fact 11)</v>.  Note that lemmas about multiplication may then be applicable \u000A (while such lemmas would be inapplicable to <v>479001600</v>).  In many proofs \u000A it is desirable to <see topic=\"ACL2____DISABLE\">disable</see> the executable counterpart <see topic=\"ACL2____RUNE\">rune</see>s of \u000A certain functions to prevent their expansion by computation.  See <see topic=\"ACL2____EXECUTABLE-COUNTERPART-THEORY\">executable-counterpart-theory</see>.</p> \u000A \u000A <p>Finally: What do we do about functions that are ``constrained'' rather than \u000A defined, such as the following?  (See <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.)</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((foo *) =&gt; *))\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)))</code> \u000A \u000A <p>Does <v>foo</v> have an executable counterpart?  Yes: since the vast majority \u000A of functions have sensible executable counterparts, it was decided that \u000A <b>all</b> functions, even such ``constrained'' ones, have executable \u000A counterparts.  We essentially ``trap'' when such calls are inappropriate. \u000A Thus, consider for example:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> x 1)\u000A    (foo x)))</code> \u000A \u000A <p>If the term <v>(bar '3)</v> is encountered by the ACL2 rewriter during a \u000A proof, and if the <v>:executable-counterpart</v> of <v>bar</v> is <see topic=\"ACL2____ENABLE\">enable</see>d, \u000A then it will be invoked to reduce this term to <v>'4</v>.  However, if the term \u000A <v>(bar 'a)</v> is encountered during a proof, then since <v>'a</v> is not a \u000A <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt> and since the <v>:executable-counterpart</v> of <v>foo</v> is \u000A only a ``trap,'' then this call of the <v>:executable-counterpart</v> of \u000A <v>bar</v> will result in a ``trap.'' In that case, the rewriter will return the \u000A term <v>(<see topic=\"ACL2____HIDE\">hide</see> (bar 'a))</v> so that it never has to go through this process \u000A again.  See <see topic=\"ACL2____HIDE\">hide</see>.</p>"],

 "ACL2____LD-ERROR-ACTION":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-error-action</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see> state)</v> and the updater is \u000A <v>(set-ld-error-action val state)</v>.  <v>Ld-error-action</v> must be \u000A <v>:continue</v>, <v>:return</v>, <v>:return!</v>, or <v>:error</v>.  The initial \u000A value of <v>ld-error-action</v> is <v>:continue</v>, which means that the \u000A top-level ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) will not exit when an \u000A error is caused by <v>user-typein</v>.  But the default value for \u000A <v>ld-error-action</v> on calls of <tt><see topic=\"ACL2____LD\">ld</see></tt> is <v>:return!</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-error-action</v> is one of them.  Suppose that <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt> is \u000A <v>t</v> and a form evaluates to an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>; see \u000A <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.  If the ``error component'', <v>erp</v>, is non-<v>nil</v>, \u000A then an error is said to have occurred.  If an error occurs, <tt><see topic=\"ACL2____LD\">ld</see></tt>'s \u000A action depends on <v>ld-error-action</v>.  If it is <v>:continue</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A just continues processing the forms in <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>.  If it is \u000A <v>:return</v> or <v>:return!</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> stops and returns as though it had \u000A emptied the channel.  If it is <v>:error</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> stops and returns, \u000A signalling an error to its caller by returning an error triple with \u000A non-<v>nil</v> error component, and reverting the logical <see topic=\"ACL2____WORLD\">world</see> to its \u000A value just before that call of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>To see this effect of <v>:ERROR</v> for <v>ld-error-action</v>, consider the \u000A following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x) (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bad (x)) (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)))</code> \u000A \u000A <p>When the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>bad</v> fails (because its body is missing), \u000A evaluation of the <v>ld</v> call stops; thus, the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>g</v> is not \u000A evaluated.  The definition of <v>f</v> will be removed from the logical <see topic=\"ACL2____WORLD\">world</see> before the call of <v>ld</v> returns.</p> \u000A \u000A <p>However, by default each user call of <v>ld</v> is made with a \u000A <v>ld-error-action</v> of <v>:RETURN!</v> (not <v>:ERROR</v>).  In the common case \u000A that all nested calls of <v>ld</v> inside the ACL2 loop are made this way, an \u000A error will not roll back the logical <see topic=\"ACL2____WORLD\">world</see>.  However, it will still \u000A halt evaluation of forms for the current call of <v>ld</v> and any parent calls \u000A of <v>ld</v> (other than the call made on behalf of <v>lp</v> that entered the \u000A ACL2 loop in the first place), as though there were no more forms to \u000A evaluate.</p> \u000A \u000A <p>We have already discussed the behavior of <v>ld</v> when an error occurs. \u000A But there is another case when <v>ld</v> can stop processing more forms: when \u000A <v>ld-error-action</v> is not <v>:CONTINUE</v>, <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt> is \u000A <v>t</v>, and evaluation of a form returns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil val\u000A state)</v>, where <v>nil</v> is the error component and whose ``value component'', \u000A <v>val</v> is a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is the symbol <v>:STOP-LD</v>. \u000A Let <v>val</v> be the pair <v>(:STOP-LD . x)</v>.  Then the call of <v>ld</v> \u000A returns the error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD n . x) state)</v>, where <v>n</v> is \u000A the value of <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable <v>'ld-level</v> at the time of \u000A termination.  The following example illustrates how this works.</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x) x)\u000A      (<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x) x)\u000A            (<see topic=\"ACL2____MV\">mv</see> nil '(:STOP-LD my-error more-info) state)\u000A            (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g2 (x) x)))\u000A      (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g1 (x) x)))</code> \u000A \u000A <p>The outer call of <v>ld</v> returns the value</p> \u000A \u000A <code>(:STOP-LD 2 3 MY-ERROR MORE-INFO)</code> \u000A \u000A <p>and leaves us in a world the includes definitions for <v>f1</v> and <v>f2</v>, \u000A but no definition for <v>g1</v> or <v>g2</v> since neither of their two <v>defun</v> \u000A forms was evaluated.  The value of <tt><see topic=\"ACL2____STATE\">state</see></tt> global <v>'ld-level</v> is \u000A incremented from 1 to 2 when the outer <v>ld</v> is entered and then again to 3 \u000A when the inner <v>ld</v> is entered.  When the inner <v>ld</v> escounters the \u000A error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD my-error more-info) state)</v>, it sees \u000A <v>:STOP-LD</v> in the <v>car</v> of the value component and pushes the current \u000A value of <v>'ld-level</v>, 3, onto the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of that value, to return the \u000A value triple <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD my-error 3 more-info) state)</v>.  The outer \u000A of <v>ld</v> then sees this value and returns <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD my-error 2 3\u000A more-info) state)</v>, since its current value of <v>'ld-level</v> is 2 after the \u000A inner <v>ld</v> exits.</p> \u000A \u000A <p>That concludes our discussion of how these special <v>:STOP-LD</v> values are \u000A handled; but how are they created?  While they can be created directly by \u000A evaluation results as suggested in the example above, that is not the standard \u000A way.  Rather, <v>ld</v> returns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD n)\u000A state)</v>, where <v>n</v> is the value of variable <v>ld-level</v> at the time of \u000A termination, when the following conditions hold: an error occurs, \u000A <v>ld-error-action</v> is <v>RETURN!</v> (which is the default), and <tt><see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see></tt> is <v>t</v> (the default).  The following example, which is a \u000A bit similar to the preceding one, illustrates both creation and handling of \u000A the special <v>:STOP-LD</v> values.</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x) x)\u000A      (<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x) x)\u000A            (<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f3 (x) x)\u000A                  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bad (x)) ; ERROR -- missing the body\u000A                  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g3 (x) x)))\u000A            (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g2 (x) x)))\u000A      (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g1 (x) x)))</code> \u000A \u000A <p>The result is that <v>f1</v>, <v>f2</v>, and <v>f3</v> are defined, but none of \u000A <v>g1</v>, <v>g2</v>, or <v>g3</v> is defined.  Let's see why.  The innermost call \u000A of <tt><see topic=\"ACL2____LD\">ld</see></tt> has a default <v>:ld-error-action</v> of <v>:RETURN!</v> (as do the \u000A other calls).  So when the definition of <v>bad</v> fails, then the innermost \u000A <v>ld</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD 4) state)</v>.  The middle <v>ld</v> sees \u000A this value, and since its <v>:ld-error-action</v> is not <v>:CONTINUE</v> (because \u000A it has the default value of <v>:RETURN!</v>), it returns before considering the \u000A definition of <v>g2</v>, with value <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD 3 4) state)</v>.  The \u000A topmost call of <v>ld</v> similarly sees the <v>:STOP-LD</v>; stops evaluation of \u000A forms, without defining <v>g1</v>; and returns <v>(<see topic=\"ACL2____MV\">mv</see> nil (:STOP-LD 2 3 4)\u000A state)</v>.</p>"],

 "ACL2____INSTRUCTIONS":[["Proof-checker"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> for an introduction to the interactive \u000A ``proof-checker'' goal manager, which supports much more direct control of the \u000A proof process than is available by direct calls to the prover (as are normally \u000A made using <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> or <tt><see topic=\"ACL2____THM\">thm</see></tt>).  In brief, typical use is to \u000A evaluate the form <v>(<see topic=\"ACL2____VERIFY\">verify</see> SOME-GOAL)</v>, where <v>SOME-GOAL</v> is a \u000A formula (i.e., term) that you would like to prove.  Various \u000A commands (instructions) are available at the resulting prompt; see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  When the proof is completed, suitable invocation of \u000A the <v>exit</v> command will print out a form containing an <v>:instructions</v> \u000A field that provides the instructions that you gave interactively, so that this \u000A form can be evaluated non-interactively.</p> \u000A \u000A <p>Thus, also see <see topic=\"ACL2____DEFTHM\">defthm</see> for the role of <v>:instructions</v> in place of \u000A <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>.  As illustrated by the following example, the value \u000A associated with <v>:instructions</v> is a list of <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A commands.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> associativity-of-append\u000A        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A               (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A        :instructions\u000A        (:induct (:dv 1) (:dv 1) :x :up :x (:dv 2) := (:drop 2)\u000A         :top (:dv 2) :x :top :s :bash))</code> \u000A \u000A <p>When you are inside the interactive loop (i.e., after executing <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>), you may invoke <tt><see topic=\"ACL2____HELP\">help</see></tt> to get a list of legal instructions and \u000A <v>(<see topic=\"ACL2____HELP\">help</see> instr)</v> to get help for the instruction <v>instr</v>.</p> \u000A \u000A <p>Below, we describe a capability for supplying <v>:instructions</v> as \u000A <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>.</p> \u000A \u000A <p>The most basic utilities for directing the discharge of a proof obligation \u000A are <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> and (less commonly) <v>:instructions</v>.  Individual \u000A instructions may call the prover with <v>:hints</v>; in that sense, prover hints \u000A may occur inside instructions.  We now describe how, on the other hand, \u000A instructions may occur inside hints.</p> \u000A \u000A <p>ACL2 supports <v>:instructions</v> as a hints keyword.  The following example \u000A forms the basis for our running example.  This example does not actually need \u000A hints, but imagine that the inductive step &mdash; which is \"Subgoal *1/2\" \u000A &mdash; was difficult.  You could submit that goal to <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, do an \u000A interactive proof, submit <v>(<see topic=\"ACL2____EXIT\">exit</see> t)</v> to obtain the list of \u000A <v>:instructions</v>, and then paste in those instructions.  When you submit the \u000A resulting event, you might see the following.  Below we'll explain the hint \u000A processing.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A            :hints ((&quot;Subgoal *1/2&quot;\u000A                     :instructions\u000A                     (:promote (:dv 1) (:dv 1) :x :up :x (:dv 2) :=\u000A                      (:drop 2) :top (:dv 2) :x :top :s))))\u000A\u000AName the formula above *1.\u000A\u000APerhaps we can prove *1 by induction.  Three induction schemes are\u000Asuggested by this conjecture.  Subsumption reduces that number to two.\u000AHowever, one of these is flawed and so we are left with one viable\u000Acandidate.\u000A\u000AWe will induct according to a scheme suggested by (<see topic=\"ACL2____BINARY-APPEND\">BINARY-APPEND</see> X Y).\u000AThis suggestion was produced using the :induction rule BINARY-APPEND.\u000AIf we let (:P X Y Z) denote *1 above then the induction scheme we'll\u000Ause is\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X)) (:P (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y Z))\u000A              (:P X Y Z))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X) (:P X Y Z))).\u000AThis induction is justified by the same argument used to admit BINARY-APPEND.\u000AWhen applied to the goal at hand the above induction scheme produces\u000Atwo nontautological subgoals.\u000A\u000A[Note:  A hint was supplied for our processing of the goal below.\u000AThanks!]\u000A\u000ASubgoal *1/2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y) Z)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y Z)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000ABut the trusted :CLAUSE-PROCESSOR function PROOF-CHECKER-CL-PROC replaces\u000Athis goal by T.\u000A\u000ASubgoal *1/1\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000ABy the simple :definition ENDP we reduce the conjecture to\u000A\u000ASubgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000ABut simplification reduces this to T, using the :definition BINARY-APPEND\u000Aand primitive type reasoning.\u000A\u000AThat completes the proof of *1.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:DEFINITION BINARY-APPEND)\u000A        (:DEFINITION ENDP)\u000A        (:DEFINITION NOT)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:INDUCTION BINARY-APPEND))\u000ATime:  0.02 seconds (prove: 0.01, print: 0.01, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>To understand how the <v>:instructions</v> supplied above were processed, \u000A observe proof-checker instruction interpreter may be viewed as a \u000A ``clause-processor'': a function that takes an input goal and returns a list \u000A of goals (which can be the empty list).  Such a function has the property that \u000A if all goals in that returned list are theorems, then so is the input goal. \u000A This view of the proof-checker instruction interpreter as a clause-processor \u000A leads to the following crucial observation.</p> \u000A \u000A <p><b>IMPORTANT!</b>.  Each call of the proof-checker instruction interpreter \u000A is treated as a standalone clause-processor that is insensitive to the \u000A surrounding prover environment.  In particular:</p> \u000A \u000A <blockquote> \u000A \u000A <p>o The proof-checker's theory is not sensitive to <v>:in-theory</v> <see topic=\"ACL2____HINTS\">hints</see> already processed in the surrounding proof.  Indeed, the current theory \u000A for which proof-checker commands are processed is just the current theory of \u000A the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, i.e., the value of <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v>. \u000A Moreover, references to <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v> in a proof-checker \u000A <v>in-theory</v> command, even implicit references such as provided by <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> expressions, are also references to the current \u000A theory of the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>o The <see topic=\"ACL2____RUNE\">rune</see>s used during an <v>:instructions</v> hint are not tracked \u000A beyond that hint, hence may not show up in the summary of the overall proof. \u000A Again, think of the <v>:instructions</v> hint as a <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> call, \u000A which has some effect not tracked by the surrounding proof other than for the \u000A child goals that it returns.</p></blockquote> \u000A \u000A <p>We continue now with our discussion of the proof-checker instruction \u000A interpreter as a clause-processor.</p> \u000A \u000A <p>In the example above, the input goal (<v>&quot;Subgoal *1/2&quot;</v>) was processed \u000A by the proof-checker instruction interpreter.  The result was the empty goal \u000A stack, therefore proving the goal, as reported in the output, which we repeat \u000A here.</p> \u000A \u000A <code>[Note:  A hint was supplied for our processing of the goal below.\u000AThanks!]\u000A\u000ASubgoal *1/2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y) Z)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y Z)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000ABut the trusted :CLAUSE-PROCESSOR function PROOF-CHECKER-CL-PROC replaces\u000Athis goal by T.</code> \u000A \u000A <p><b>Remark.</b> This brief remark can probably be ignored, but we include it \u000A for completeness.  The <v>:CLAUSE-PROCESSOR</v> message above may be surprising, \u000A since the hint attached to <v>&quot;Subgoal *1/2&quot;</v> is an <v>:instructions</v> \u000A hint, not a <v>:clause-processor</v> hint.  But <v>:instructions</v> is actually a \u000A custom keyword hint (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>), and may be thought of \u000A as a macro that expands to a <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hint, one that \u000A specifies <v>proof-checker-cl-proc</v> as the clause-processor function.  The \u000A keen observer may notice that the clause-processor is referred to as \u000A ``trusted'' in the above output.  Normally one needs a trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) to install a trusted clause-processor, but that is not the case for \u000A the built-in clause-processor, <v>proof-checker-cl-proc</v>.  Finally, we note \u000A that <v>:instructions</v> <see topic=\"ACL2____HINTS\">hints</see> are ``spliced'' into the hints as \u000A follows: the appropriate <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hint replaces the \u000A <v>:instructions</v> hint, and the other hints remain intact.  It may seems \u000A surprising that one can thus, for example, use <v>:instructions</v> and \u000A <v>:in-theory</v> together; but although the <v>:in-theory</v> hint will have no \u000A effect on execution of the <v>:instructions</v> (see first bullet above), the \u000A <v>:in-theory</v> hint will apply in the usual manner to any child goals (see \u000A <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>).  End of Remark.</p> \u000A \u000A <p>Now consider the case that the supplied instructions do not prove the goal. \u000A That is, suppose that the execution of those instructions results in a \u000A non-empty goal stack.  In that case, the resulting goals become children of \u000A the input goals.  The following edited log provides an illustration using a \u000A modification of the above example, this time with a single instruction that \u000A splits into two cases.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A            :hints ((&quot;Subgoal *1/2&quot;\u000A                     :instructions\u000A                     ((:casesplit (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))))))\u000A\u000A[[ ... output omitted ... ]]\u000A\u000ASubgoal *1/2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y) Z)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y Z)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y) Z)\u000A                (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y Z))).\u000A\u000AWe now apply the trusted :CLAUSE-PROCESSOR function PROOF-CHECKER-CL-PROC\u000Ato produce two new subgoals.\u000A\u000ASubgoal *1/2.2\u000A\u000A[[ ... output omitted ... ]]\u000A\u000ASubgoal *1/2.1\u000A\u000A[[ ... output omitted ... ]]</code> \u000A \u000A <p>We have seen that an <v>:instructions</v> hint may produce zero or more \u000A subgoals.  There may be times where you wish to insist that it produce zero \u000A subgoals, i.e., that it prove the desired goal.  The proof-checker \u000A `<v>finish</v>' command works nicely for this purpose.  For example, the \u000A following form is successfully admitted, but if you delete some of the \u000A commands (for example, the <v>:s</v> command at the end), you will see an \u000A informative error message.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A            (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A     :hints ((&quot;Subgoal *1/2&quot;\u000A              :instructions\u000A              ((finish :promote (:dv 1) (:dv 1) :x :up :x (:dv 2) :=\u000A                       (:drop 2) :top (:dv 2) :x :top :s)))))</code> \u000A \u000A <p>If an :instructions hint of the form <v>((finish ...))</v> fails to prove the \u000A goal, the clause-processor is deemed to have caused an error.  Indeed, any \u000A ``failure'' of a supplied proof-checker instruction will be deemed to cause an \u000A error.  In this case, you should see an error message such as the \u000A following:</p> \u000A \u000A <code>Saving proof-checker error state; see :DOC instructions.  To retrieve:\u000A\u000A(<see topic=\"ACL2____RETRIEVE\">RETRIEVE</see> :ERROR1)</code> \u000A \u000A <p>In this case, you can evaluate the indicated <tt><see topic=\"ACL2____RETRIEVE\">retrieve</see></tt> command in \u000A the ACL2 read-eval-print loop, to get to the point of failure.</p> \u000A \u000A <p>You may have noticed that there is no output from the proof-checker in the \u000A examples above.  This default behavior prevents confusion that could arise \u000A from use of proof-checker commands that call the theorem prover such as \u000A <v>prove</v>, <v>bash</v>, <v>split</v>, and <v>induct</v>.  These commands produce \u000A output for what amounts to a fresh proof attempt, which could confuse attempts \u000A to understand the surrounding proof log.  You can override the default \u000A behavior by providing a command of the form</p> \u000A \u000A <code> <v>(comment inhibit-output-lst VAL)</v> </code> \u000A \u000A <p>where <v>VAL</v> is either the keyword <v>:SAME</v> (indicating that no change \u000A should be made to which output is inhibited) or else is a legal value for \u000A inhibited output; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.  The following two \u000A variants of the immediately preceding <v>THM</v> form will each produce output \u000A from the proof-checker commands, assuming in the first variant that output \u000A hasn't already been inhibited.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A            :hints ((&quot;Subgoal *1/2&quot;\u000A                     :instructions\u000A                     ((comment inhibit-output-lst :same)\u000A                      (:casesplit (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A            :hints ((&quot;Subgoal *1/2&quot;\u000A                     :instructions\u000A                     ((comment inhibit-output-lst (<see topic=\"ACL2____PROOF-TREE\">proof-tree</see>))\u000A                      (:casesplit (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))))))</code> \u000A \u000A <p>Note that such a <v>comment</v> instruction must be provided explicitly \u000A (i.e., not by way of a proof-checker <see topic=\"ACL2____MACRO-COMMAND\">macro-command</see>) as the first \u000A instruction, in order to have the effect on inhibited output that is described \u000A above.</p> \u000A \u000A <p>The following contrived example gives a sense of how one might want to use \u000A <v>:instructions</v> within <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>.  If you submit the following \u000A theorem</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x)) x)))</code> \u000A \u000A <p>then you will see the following checkpoint printed with the summary.</p> \u000A \u000A <code>Subgoal *1/3''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) NIL) NIL)\u000A                     (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A                           NIL)\u000A                X))</code> \u000A \u000A <p>This suggests proving the following theorem.  Here we state it using <tt><see topic=\"ACL2____DEFTHMD\">defthmd</see></tt>, so that it is immediately disabled.  Normally disabling would be \u000A unnecessary, but for our contrived example it is useful to imagine disabling \u000A it, say because we are following a methodology that tends to keep <see topic=\"ACL2____REWRITE\">rewrite</see> rules disabled.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">defthmd</see> revappend-revappend\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y) z)\u000A         (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> y (<see topic=\"COMMON-LISP____APPEND\">append</see> x z))))</code> \u000A \u000A <p>We might then enter the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> to prove the original theorem \u000A interactively, as follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____VERIFY\">verify</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x)) x)))\u000A-&gt;: bash\u000A***** Now entering the theorem prover *****\u000AGoal'\u000A\u000A([ A key checkpoint:\u000A\u000AGoal'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL) NIL)\u000A                X))\u000A\u000AGoal' is subsumed by a goal yet to be proved.\u000A\u000A])\u000A\u000AQ.E.D.\u000A\u000ACreating one new goal:  (MAIN . 1).\u000A\u000AThe proof of the current goal, MAIN, has been completed.  However,\u000Athe following subgoals remain to be proved:\u000A  (MAIN . 1).\u000ANow proving (MAIN . 1).\u000A-&gt;: th ; show current goal (&quot;th&quot; for &quot;theorem&quot;)\u000A*** Top-level hypotheses:\u000A1. (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)\u000A\u000AThe current subterm is:\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL) NIL)\u000A       X)\u000A-&gt;: p ; show current subterm only\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL) NIL)\u000A       X)\u000A-&gt;: 1 ; dive to first argument\u000A-&gt;: p\u000A(<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL) NIL)\u000A-&gt;: sr ; show-rewrites\u000A\u000A1. REVAPPEND-REVAPPEND (disabled)\u000A  New term: (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> NIL (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X NIL))\u000A  Hypotheses: &lt;none&gt;\u000A  Equiv: EQUAL\u000A\u000A2. REVAPPEND\u000A  New term: (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL))\u000A                 (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL))\u000A                            (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> X NIL)))))\u000A  Hypotheses: &lt;none&gt;\u000A  Equiv: EQUAL\u000A-&gt;: (r 1) ; rewrite with rule #1 above\u000ARewriting with REVAPPEND-REVAPPEND.\u000A-&gt;: p\u000A(<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> NIL (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X NIL))\u000A-&gt;: top ; move to the top of the conclusion, making it the current subterm\u000A-&gt;: p\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> NIL (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X NIL)) X)\u000A-&gt;: prove ; finish the proof\u000A***** Now entering the theorem prover *****\u000A\u000AQ.E.D.\u000A\u000A*!*!*!*!*!*!* All goals have been proved! *!*!*!*!*!*!*\u000AYou may wish to exit.\u000A-&gt;: (<see topic=\"ACL2____EXIT\">exit</see> t) ; the argument, t, causes :instructions to be printed\u000A(<see topic=\"ACL2____DEFTHM\">DEFTHM</see> T\u000A        (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)\u000A                 (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVERSE\">REVERSE</see> (<see topic=\"COMMON-LISP____REVERSE\">REVERSE</see> X)) X))\u000A        :INSTRUCTIONS (:BASH (:DV 1)\u000A                             (:REWRITE REVAPPEND-REVAPPEND)\u000A                             :TOP :PROVE))\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see>\u000A        (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)\u000A                 (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVERSE\">REVERSE</see> (<see topic=\"COMMON-LISP____REVERSE\">REVERSE</see> X)) X))\u000A        :hints ((&quot;Goal&quot;\u000A                 :INSTRUCTIONS ; copy what was printed above:\u000A                 (:BASH (:DV 1)\u000A                        (:REWRITE REVAPPEND-REVAPPEND)\u000A                        :TOP :PROVE))))\u000AGoal'\u000A\u000AQ.E.D.\u000A\u000AQ.E.D.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: NIL\u000AHint-events: ((:CLAUSE-PROCESSOR PROOF-CHECKER-CL-PROC))\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally we present an even more contrived example, based on the one above. \u000A This example illustrates that there is actually no limit imposed on the \u000A nesting of <v>:instructions</v> within <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> within \u000A <v>:instructions</v>, and so on.  Notice the indication of nesting levels: \u000A ``<v>1&gt;</v>'' to ``<v>&lt;1</v>'' for output from nesting level 1, and ``<v>2&gt;</v>'' to \u000A ``<v>&lt;2</v>'' for output from nesting level 2.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x)) x))\u000A     :hints ((&quot;Goal&quot;\u000A              :instructions\u000A              ((comment inhibit-output-lst :same)\u000A               (:prove\u000A                :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> append))\u000A                        (&quot;Subgoal *1/3''&quot;\u000A                         :instructions\u000A                         ((comment inhibit-output-lst :same)\u000A                          :bash\u000A                          (:dv 1)\u000A                          (:rewrite revappend-revappend)))))))))</code> \u000A \u000A <p>Here is an edited version of the resulting log.</p> \u000A \u000A <code>[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000A[[1&gt; Executing proof-checker instructions]]\u000A\u000A-&gt;: (COMMENT INHIBIT-OUTPUT-LST :SAME)\u000A-&gt;: (:PROVE\u000A         :HINTS\u000A         ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> APPEND))\u000A          (&quot;Subgoal *1/3''&quot; :INSTRUCTIONS ((COMMENT INHIBIT-OUTPUT-LST :SAME)\u000A                                           :BASH (:DV 1)\u000A                                           (:REWRITE REVAPPEND-REVAPPEND)))))\u000A***** Now entering the theorem prover *****\u000A\u000A[[ ... output omitted ... ]]\u000A\u000A[Note:  A hint was supplied for our processing of the goal below.\u000AThanks!]\u000A\u000ASubgoal *1/3''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) NIL) NIL)\u000A                     (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____REVAPPEND\">REVAPPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A                           NIL)\u000A                X)).\u000A\u000A[[2&gt; Executing proof-checker instructions]]\u000A\u000A-&gt;: (COMMENT INHIBIT-OUTPUT-LST :SAME)\u000A-&gt;: :BASH\u000A***** Now entering the theorem prover *****\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000ABut we have been asked to pretend that this goal is subsumed by the\u000Ayet-to-be-proved |PROOF-CHECKER Goal|.\u000A\u000AQ.E.D.\u000A\u000ACreating one new goal:  (MAIN . 1).\u000A\u000AThe proof of the current goal, MAIN, has been completed.  However,\u000Athe following subgoals remain to be proved:\u000A  (MAIN . 1).\u000ANow proving (MAIN . 1).\u000A-&gt;: (:DV 1)\u000A-&gt;: (:REWRITE REVAPPEND-REVAPPEND)\u000ARewriting with REVAPPEND-REVAPPEND.\u000A\u000A[[&lt;2 Completed proof-checker instructions]]\u000A\u000AWe now apply the trusted :CLAUSE-PROCESSOR function PROOF-CHECKER-CL-PROC\u000Ato produce one new subgoal.\u000A\u000ASubgoal *1/3'''\u000A\u000A[[ ... output omitted ... ]]\u000A\u000A[[&lt;1 Completed proof-checker instructions]]</code> \u000A \u000A <p>The nesting levels are independent of whether or not output is enabled; for \u000A example, if the first <v>(comment ...)</v> form below is omitted, then we will \u000A see only the output bracketed by ``<v>2&gt;</v>'' to ``<v>&lt;2</v>''.  Note also that \u000A these levels are part of the error states saved for access by <tt><see topic=\"ACL2____RETRIEVE\">retrieve</see></tt> \u000A (as indicated above); for example, a failure at level 1 would be associated \u000A with symbol <v>:ERROR1</v> as indicated above, while a failure at level 2 would \u000A be associated with symbol <v>:ERROR2</v>.</p>"],

 "ACL2____HINTS-AND-THE-WATERFALL":[["Hints"],"ACL2 Sources","ACL2","<p>Below we describe the flow of an ACL2 proof attempt, with special \u000A attention to how <see topic=\"ACL2____HINTS\">hints</see> are applied during a proof.  For most ACL2 \u000A users, only one point is important to take away from this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A topic: you may specify hints during a proof (see <see topic=\"ACL2____HINTS\">hints</see>; perhaps also \u000A see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see> and see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>), and they can be \u000A expected to behave intuitively.  See <see topic=\"ACL2____THE-METHOD\">the-method</see> for a summary of how to \u000A interact with the ACL2 prover; see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> \u000A for a more detailed tutorial; and see <see topic=\"ACL2____HINTS\">hints</see> for an introduction to ACL2 \u000A hints, including detailed <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for specific hint types.</p> \u000A \u000A <p>The remainder of this topic serves as a reference in case one needs a \u000A deeper understanding of the workings of ACL2's handling of hints.  Also, for \u000A examples of the sophisticated use of hints, primarily for experts, see \u000A community book <v>books/hints/basic-tests.lisp</v>.</p> \u000A \u000A <p>First, we describe the ACL2 ``waterfall'', which handles each goal either \u000A by replacing it with a list (possibly empty) of child goals, or else by \u000A putting the goal into a ``pool'' for later proof by induction.  Then, we \u000A describe how hints are handled by the waterfall.</p> \u000A \u000A <p><b>The Waterfall.</b></p> \u000A \u000A <p>Each goal considered by the ACL2 prover passes through a series of proof \u000A processes, called the ``waterfall processes'', as stored in the constant \u000A <v>*preprocess-clause-ledge*</v>.  The top process applies top-level hints, \u000A including <v>:use</v> hints; the next is a lightweight ``preprocess'' simplifier \u000A for ``simple'' rules (see <see topic=\"ACL2____SIMPLE\">simple</see>); the next is the main ACL2 \u000A simplifier; and finally ACL2 attempts (in this order) destructor elimination, \u000A fertilization (heuristic use of equalities), generalization, and elimination \u000A of irrelevance.  Each process may ``hit'', creating zero or more child goals \u000A that are each then handled at the top of the waterfall; or it may ``miss'', in \u000A which case the next process in the above sequence is considered.  If all \u000A processes miss, then a ``push'' process defers the goal until it is later \u000A considered for proof by induction.  When all goals have been thus handled, the \u000A goal most recently pushed for proof by induction is considered, and the \u000A process repeats.</p> \u000A \u000A <p>We next describe the two additional ways in which control can be returned \u000A to the top of the waterfall.</p> \u000A \u000A <p>When the simplification process is attempted unsuccessfully for a goal, the \u000A goal is deemed to have ``settled down''.  In this case, and if no ancestor of \u000A the goal has settled down, then the ``settled-down'' process is deemed to have \u000A ``hit'' on the goal, the effect being that the goal makes a new pass through \u000A all the waterfall processes.  (Other processes can then notice that settling \u000A down has occurred and modify their heuristics accordingly.)  For example, if \u000A <v>&quot;Goal&quot;</v> simplifies to <v>&quot;Subgoal 2&quot;</v> (among others), and \u000A <v>&quot;Subgoal 2&quot;</v> simplifies to <v>&quot;Subgoal 2.3&quot;</v> (among others), which in \u000A turn is not further simplified, then the ``settled-down'' process hits on \u000A <v>&quot;Subgoal 2.3&quot;</v> but not on any of its children, their children, and so \u000A on.</p> \u000A \u000A <p>When simplification has missed (and thus the goal has settled down), the \u000A next proof process is normally destructor elimination.  However, if a computed \u000A hint is suitable (in a sense described below; also see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>, \u000A especially the discussion of <v>stable-under-simplificationp</v>), then that \u000A hint is selected as control is returned to the top of the waterfall.  A \u000A subtlety is that in this case, if the most recent hit had been from settling \u000A down, then the prover ``changes its mind'' and considers that the goal has not \u000A yet settled down after all as it continues through the waterfall.</p> \u000A \u000A <p>Each time a goal is considered at the top of the waterfall, then before \u000A passing through the proof processes as described above, ACL2 searches for a \u000A relevant hint to select unless it has already been provided a hint in the \u000A ``<v>stable-under-simplificationp</v>'' case mentioned above.  We turn now to a \u000A more thorough discussion of how hints are selected and applied.</p> \u000A \u000A <p><b>The handling of hints.</b></p> \u000A \u000A <p>In the discussion below we will ignore forcing rounds, as each forcing \u000A round is simply treated as a new proof attempt that uses the list of hints \u000A provided at the start of the proof.</p> \u000A \u000A <p>When the theorem prover is called by <tt><see topic=\"ACL2____THM\">thm</see></tt> or <see topic=\"ACL2____EVENTS\">events</see> such as \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, and <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, it gathers up the \u000A hints that have been supplied, often provided as a <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> \u000A argument, but for example using a <v>:guard-hints</v> argument for <see topic=\"ACL2____GUARD\">guard</see> \u000A verification proofs.  (ACL2(r) users (see <see topic=\"COMMON-LISP____REAL\">real</see>) may also employ \u000A <v>:std-hints</v>.)  It then appends these to the front of the list of default \u000A hints (see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>).  The resulting list becomes the initial \u000A value of the list of ``pending hints'', one of two critical lists maintained \u000A by the theorem prover to manage hints.  The other critical list is a list of \u000A ``hint settings''; the two lists are maintained as follows.</p> \u000A \u000A <p>When a goal is first considered, a hint is selected from the list of \u000A pending hints if any is found to apply, as described below.  If a hint is \u000A selected, then it takes effect and is removed from the pending hints.  Except: \u000A if the selected hint is a computed hint with value <v>t</v> specified for \u000A <v>:computed-hint-replacement</v>, then it is not removed; and if that value is \u000A a list of hints, then that list is appended to the front of the list of \u000A pending hints after the selected hint is removed (also see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>).  The selected hint is also used to update the hint settings, \u000A as described below.</p> \u000A \u000A <p>The list of hint settings associates hint keywords with values.  It is \u000A passed from the current goal to its children (and hence the children's \u000A children, and so on), though modified by hints selected from pending hints, as \u000A described below.  This list is maintained so that when a goal is pushed for \u000A proof by induction, the hint settings are applied at the start of the proof by \u000A induction.  Note that the list of hint settings is not re-applied to \u000A descendents of a goal in the current waterfall; a hint is applied only when it \u000A is selected (and also perhaps later as just described, through the stored hint \u000A settings at the start of a proof by induction).  For example, if the hint \u000A selected for <v>&quot;Subgoal 3&quot;</v> includes <v>:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo)</v>, then \u000A the hint settings are correspondingly updated when processing <v>&quot;Subgoal\u000A 3&quot;</v>, and they persist at subgoals such as <v>&quot;Subgoal 3.2&quot;</v> and \u000A <v>&quot;Subgoal 3.2.1&quot;</v> (unless overriden by hints on those goals); but the \u000A theory specifying <v>foo</v> is not re-installed at every such subgoal.</p> \u000A \u000A <p>When a hint is selected, the list of hint settings is updated so that for \u000A each keyword <v>:kwd</v> and associated value <v>val</v> from the hint, <v>:kwd</v> \u000A is associated with <v>val</v> in the hint settings, discarding any previous \u000A association of <v>:kwd</v> with a value in the hint settings.  Except, certain \u000A ``top-level'' hints are never saved in the hint settings: <v>:use</v>, \u000A <v>:cases</v>, <v>:by</v>, <v>:bdd</v>, <v>:or</v>, and <v>:clause-processor</v>.</p> \u000A \u000A <p>For example, suppose that we specify the following hints, with no default \u000A hints.</p> \u000A \u000A <code>((&quot;Goal&quot; :expand ((bar x y)))\u000A (&quot;Subgoal 3&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo)))</code> \u000A \u000A <p>These hints then become the initial list of pending hints.  When the proof \u000A attempt begins, the prover encounters the top-level goal (<v>&quot;Goal&quot;</v>) and \u000A pulls the <v>&quot;Goal&quot;</v> hint from the pending hints, so that the list of hint \u000A settings contains a value only for keyword <v>:expand</v>.  This hint setting \u000A will remain for all children of the top-level goal as well, and their \u000A children, and so on, and will be inherited by induction &mdash; in other \u000A words, it will remain throughout the entire proof.  Now consider what happens \u000A when the proof reaches <v>&quot;Subgoal 3&quot;</v>.  At this point there is only one \u000A pending hint, which is in fact attached to that subgoal.  Therefore, this hint \u000A is pulled from the pending hints (leaving that list empty), and the hint \u000A settings are extended by associating the <v>:in-theory</v> keyword with the \u000A theory represented by <v>(<see topic=\"ACL2____ENABLE\">enable</see> foo)</v>.  That theory is immediately installed \u000A until the prover finishes addressing <v>&quot;Subgoal 3&quot;</v>, its children, their \u000A children, and so on; and until that completion is reached, the <v>:in-theory</v> \u000A keyword remains associated with the <v>(<see topic=\"ACL2____ENABLE\">enable</see> foo)</v> in the hint settings, \u000A although of course there is no re-installation of the theory at any ensuing \u000A child goal.  When finally <v>&quot;Subgoal 3&quot;</v> and its descendents have been \u000A completed and the prover is about to consider <v>&quot;Subgoal 2&quot;</v>, the \u000A <v>:in-theory</v> association is removed from the hint settings and the global \u000A theory is re-installed.  However, the list of pending hints remains empty.</p> \u000A \u000A <p>It remains to describe how a hint is selected for a goal.  When a goal is \u000A first considered (hence at the top of the waterfall), the list of pending \u000A hints is scanned, in order, until one of the hints is suitable for the goal. \u000A An explicit hint <v>(goal-name :kwd1 val1 ... :kwdn valn)</v> is suitable if \u000A <v>goal-name</v> is the name of the current goal and there is at least one \u000A keyword.  A computed hint is suitable if it evaluates to a non-<v>nil</v> value. \u000A As indicated earlier in this documentation topic, an exception occurs when a \u000A computed hint is selected after simplification fails (the \u000A ``<v>stable-under-simplificationp</v>'' case): in that case, the goal returns to \u000A the top of the waterfall with that hint as the selected hint, and no \u000A additional search for a hint to select is made at that time.</p> \u000A \u000A <p>The following slightly tricky example illustrates handling of hints.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> '((&quot;Goal&quot; :do-not '(preprocess))))\u000A ((&quot;Goal&quot; :DO-NOT '(PREPROCESS)))\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A            :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> car-cons))))\u000A\u000AACL2 Warning [Hints] in ( THM ...):  The goal-spec &quot;Goal&quot; is explicitly\u000Aassociated with more than one hint.  All but the first of these hints\u000Amay be ignored.  If you intended to give all of these hints, combine\u000Athem into a single hint of the form (&quot;Goal&quot; :kwd1 val1 :kwd2 val2 ...).\u000ASee :DOC hints-and-the-waterfall.\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AName the formula above *1.</code> \u000A \u000A <p>The warning above is printed because <v>&quot;Goal&quot;</v> is associated with two \u000A pending hints: one given by the <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt> call and one \u000A supplied by the <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> keyword of the <tt><see topic=\"ACL2____THM\">thm</see></tt> form.  The \u000A <v>:in-theory</v> hint is selected because user-supplied hints are ahead of \u000A default hints in the list of pending hints; we then get the first ``Note'' \u000A above.  The goal progresses through the waterfall without any proof process \u000A applying to the goal; in particular, it cannot be further simplified.  After \u000A the simplification process, a ``settled-down'' process applies, as discussed \u000A above, immediately causing another trip through the waterfall.  Since the \u000A <v>:in-theory</v> hint was earlier removed from the list of pending hints when \u000A it was applied, the default (<v>:do-not</v>) hint is now the only pending hint. \u000A That hint is applied, resulting in the second ``Note'' above.</p> \u000A \u000A <p>Again, more examples may be found in the community book \u000A <v>books/hints/basic-tests.lisp</v>.  A particularly tricky but informative \u000A example in that book is the one related to <v>nonlinearp-default-hint</v>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for an advanced feature that allows \u000A modification of the hint selected for a goal.</p>"],

 "ACL2____EC-CALL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The name ``<v>ec-call</v>'' represents ``executable-counterpart \u000A call.''  This utility is intended for users who are familiar with guards.  See \u000A <see topic=\"ACL2____GUARD\">guard</see> for a general discussion of guards.</p> \u000A \u000A <p>Logically, <v>ec-call</v> behaves like the identity macro; during proofs, \u000A <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> TERM)</v> is typically replaced quickly by <v>TERM</v> during a proof \u000A attempt.  However, <v>ec-call</v> causes function calls to be evaluated in the \u000A ACL2 logic rather than raw Lisp, as explained below.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____EC-CALL\">ec-call</see> (fn term1 ... termk))</code> \u000A \u000A <p>where <v>fn</v> is a known function symbol other than those in the list that \u000A is the value of the constant <v>*ec-call-bad-ops*</v>.  In particular, <v>fn</v> \u000A is not a macro.  Semantically, <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> (fn term1 ... termk))</v> equals \u000A <v>(fn term1 ... termk)</v>.  However, this use of <v>ec-call</v> has two \u000A effects.</p> \u000A \u000A <blockquote> \u000A \u000A <p>(1) <see topic=\"ACL2____GUARD\">Guard</see> verification generates no proof obligations from the guard \u000A of <v>fn</v> for this call.  Indeed, guards need not have been verified for \u000A <v>fn</v>.</p> \u000A \u000A <p>(2) During evaluation, after the arguments of <v>fn</v> are evaluated as \u000A usual, the executable counterpart of <v>fn</v> is called, rather than <v>fn</v> as \u000A defined in raw Lisp.  That is, the call of <v>fn</v> is made on its evaluated \u000A arguments as though this call is being made in the ACL2 top-level loop, rather \u000A than in raw Lisp.  In particular, the <see topic=\"ACL2____GUARD\">guard</see> of <v>fn</v> is checked, at \u000A least by default (see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>).</p></blockquote> \u000A \u000A <p>Note that in the term (ec-call (fn term1 ... termk))<v></v>, only the \u000A indicated call of <v>fn</v> is made in the logic; each <v>termi</v> is evaluated \u000A in the normal manner.  If you want an entire term evaluated in the logic, wrap \u000A <v>ec-call</v> around each function call in the term (other than calls of \u000A <v>if</v> and <v>ec-call</v>).</p> \u000A \u000A <p><b>Technical Remark</b> (probably best ignored).  During evaluation of a \u000A call of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> or <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> in raw Lisp, a form <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> (fn\u000A term1 ... termk))</v> is treated as <v>(fn term1 ... termk)</v>, that is, without \u000A calling the executable counterpart of <v>fn</v>.  This situation occurs when \u000A loading a compiled file (or expansion file) on behalf of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  The reason is technical: executable counterparts are \u000A defined below a book's events in the book's compiled file.  End of Technical \u000A Remark.</p> \u000A \u000A <p>Here is a small example.  We define <v>foo</v> recursively but with guard \u000A verification inhibited on the recursive call, which is to be evaluated in the \u000A ACL2 logic.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)))\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"ACL2____EC-CALL\">ec-call</see> (foo (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))))\u000A          (<see topic=\"COMMON-LISP____CAR\">car</see> y)))\u000A\u000AThe admission of FOO is trivial, using the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).  We could deduce no constraints on the type of FOO.\u000A\u000AComputing the guard conjecture for FOO....\u000A\u000AThe guard conjecture for FOO is trivial to prove.  FOO is compliant\u000Awith Common Lisp.\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 !&gt;(foo '(2 3 4 5) '(6 7))\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (FOO X Y),\u000Awhich is (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y), is violated by the arguments in the call\u000A(FOO '(4 5) NIL).  To debug see :DOC print-gv, see :DOC trace, and\u000Asee :DOC wet.  See :DOC set-guard-checking for information about suppressing\u000Athis check with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new\u000Ausers.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>The error above arises because eventually, <v>foo</v> recurs down to a value \u000A of parameter <v>y</v> that violates the guard.  This is clear from tracing (see \u000A <see topic=\"ACL2____TRACE_42\">trace$</see> and see <see topic=\"COMMON-LISP____TRACE\">trace</see>).  Each call of the executable counterpart \u000A of <v>foo</v> (the so-called ``*1*'' function for <v>foo</v>) checks the guard and \u000A then invokes the raw Lisp version of <v>foo</v>.  The raw Lisp version calls the \u000A executable counterpart on the recursive call.  When the guard check fails we \u000A get a violation.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> foo)\u000A ((FOO))\u000AACL2 !&gt;(foo '(2 3 4 5) '(6 7))\u000A1&gt; (ACL2_*1*_ACL2::FOO (2 3 4 5) (6 7))\u000A  2&gt; (FOO (2 3 4 5) (6 7))\u000A    3&gt; (ACL2_*1*_ACL2::FOO (3 4 5) (7))\u000A      4&gt; (FOO (3 4 5) (7))\u000A        5&gt; (ACL2_*1*_ACL2::FOO (4 5) NIL)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (FOO X Y),\u000Awhich is (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y), is violated by the arguments in the call\u000A(FOO '(4 5) NIL).  To debug see :DOC print-gv, see :DOC trace, and\u000Asee :DOC wet.  See :DOC set-guard-checking for information about suppressing\u000Athis check with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new\u000Ausers.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>If we turn off guard errors then we can see the trace as above, but where \u000A we avoid calling the raw Lisp function when the guard fails to hold.</p> \u000A \u000A <code>ACL2 !&gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(foo '(2 3 4 5) '(6 7))\u000A1&gt; (ACL2_*1*_ACL2::FOO (2 3 4 5) (6 7))\u000A  2&gt; (FOO (2 3 4 5) (6 7))\u000A    3&gt; (ACL2_*1*_ACL2::FOO (3 4 5) (7))\u000A      4&gt; (FOO (3 4 5) (7))\u000A        5&gt; (ACL2_*1*_ACL2::FOO (4 5) NIL)\u000A          6&gt; (ACL2_*1*_ACL2::FOO (5) NIL)\u000A            7&gt; (ACL2_*1*_ACL2::FOO NIL NIL)\u000A            &lt;7 (ACL2_*1*_ACL2::FOO NIL)\u000A          &lt;6 (ACL2_*1*_ACL2::FOO (5))\u000A        &lt;5 (ACL2_*1*_ACL2::FOO (4 5))\u000A      &lt;4 (FOO (3 4 5))\u000A    &lt;3 (ACL2_*1*_ACL2::FOO (3 4 5))\u000A  &lt;2 (FOO (2 3 4 5))\u000A&lt;1 (ACL2_*1*_ACL2::FOO (2 3 4 5))\u000A(2 3 4 5)\u000AACL2 &gt;</code>"],

 "ACL2____DEFUN-INLINE":[["Defun","Events"],"ACL2 Sources","ACL2","<p>You may be able to improve performance by replacing an event \u000A <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f ...)</v> with a corresponding event <v>(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> f ...)</v>, in \u000A order to encourage the host Lisp compiler to inline calls of <v>f</v>.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> lng (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x) 0 (<see topic=\"COMMON-LISP____1_B2\">1+</see> (lng (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> fn (var1 ... varn) doc-string dcl ... dcl body)</code> \u000A \u000A <p>satisfying the same requirements as in the General Form for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>. \u000A The effect is to define a macro <v>fn</v> and a function <v>fn$inline</v> (i.e., a \u000A symbol in the same package as <v>fn</v> but whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> has the \u000A suffix <v>&quot;$INLINE&quot;</v>, such that each call of <v>fn</v> expands to a call of \u000A the function symbol <v>fn$inline</v> on the same arguments.  If <v>doc-string</v> \u000A is supplied, then it is associated with <v>fn</v>, not with <v>fn$inline</v>. \u000A Moreover, <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are generated that allow the use of \u000A <v>fn</v> in <see topic=\"ACL2____THEORY\">theory</see> expressions to represent <v>fn$inline</v> and that \u000A cause any untranslated (user-level) call of <v>fn$inline</v> to be printed as \u000A the corresponding call of <v>fn</v>.</p> \u000A \u000A <p>A form <v>(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> f ...)</v> actually defines a function named \u000A <v>f$inline</v> and a corresponding macro named <v>f</v> whose calls expand to \u000A calls of <v>f$inline</v>, while providing the illusion that there is just the \u000A ``function'' <v>f</v>.  For example, the Example Form above macroexpands in one \u000A step to the following form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> lng (non-stobj-var0)\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'lng$inline non-stobj-var0))\u000A       (<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> lng lng$inline)\u000A       (<see topic=\"COMMON-LISP____DEFUN\">defun</see> lng$inline (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x) 0 (<see topic=\"COMMON-LISP____1_B2\">1+</see> (lng (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code> \u000A \u000A <p>Note that the above call of <tt><see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see></tt> generates the \u000A aforementioned two table events (see <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see>), which provide the \u000A illusion that we are just defining a function <v>lng</v>, as you can see in the \u000A following log: <v>lng</v> appears rather than <v>lng$inline</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> nil)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (lng (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> (lng x) (lng y)))\u000A            :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> lng))))\u000A\u000A[.. output omitted ..]\u000A\u000ASubgoal *1/2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (LNG (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> (LNG (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (LNG Y))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (LNG (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (LNG X) (LNG Y)))).\u000A\u000A[.. output omitted ..]</code> \u000A \u000A <p>Under the hood, ACL2 arranges that every function symbol with suffix \u000A <v>&quot;$INLINE&quot;</v> is presented to the compiler as one whose calls we would \u000A prefer to inline.  Technically: the Common Lisp form <v>(declaim (inline\u000A f$inline))</v> is generated for a function symbol <v>f$inline</v> before that \u000A symbol's definition is submitted.  However, the Common Lisp spec explicitly \u000A avoids requiring that the compiler respect this <v>declaim</v> form. \u000A Fortunately, Common Lisp implementations often do respect it.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFUND-INLINE\">defund-inline</see>, see <see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see>, and see <see topic=\"ACL2____DEFUND-NOTINLINE\">defund-notinline</see>.</p> \u000A \u000A <p><i>Remarks</i>.</p> \u000A \u000A <p>(1) None of these macros (including <v>defun-inline</v>) is supported for use \u000A inside a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>.</p> \u000A \u000A <p>(2) Every function symbol defined in ACL2 whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> has the \u000A suffix <v>&quot;$INLINE&quot;</v> is proclaimed to be inline; similarly for \u000A <v>&quot;$NOTINLINE&quot;</v> and notinline.</p> \u000A \u000A <p>(3) No special treatment for inlining (or notinlining) is given for \u000A function symbols locally defined by <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt>, with two exceptions: when \u000A explicitly declared <v>inline</v> or <v>notinline</v> by the <v>flet</v> form, and \u000A for symbols discussed in (1) and (2) above that, at some point in the current \u000A ACL2 session, were defined as function symbols in ACL2 (even if not currently \u000A defined because of undoing or being <see topic=\"ACL2____LOCAL\">local</see>).</p> \u000A \u000A <p>(4) The function symbol actually being defined by <v>(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> foo\u000A ...)</v>  is <v>foo$inline</v>.  As mentioned above, one can be oblivious to this \u000A fact when writing <see topic=\"ACL2____THEORY\">theory</see> expressions or perusing prover output. \u000A However, for other purposes (for example, <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> and <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> <v>:exports</v>) you will need to supply the name of the function \u000A symbol rather than the name of the macro; e.g., for the above form \u000A <v>(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> foo ...)</v>, you may subsequently issue the event \u000A <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo$inline)</v> but not <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)</v>.</p> \u000A \u000A <p>(5) Obscure Remark.  Suppose that you certify a book with compilation (the \u000A default) in one host Lisp, saving the expansion file.  Suppose that you then \u000A compile in another host Lisp by using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with argument \u000A <v>:load-compiled-file :comp</v>.  Then in subsequent sessions, including that \u000A book with the second host Lisp will not result in any inline or notinline \u000A behavior for functions defined in the book.  This may be fixed in a future \u000A release if someone complains.</p>"],

 "ACL2____TERM-ORDER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>ACL2 must occasionally choose which of two terms is syntactically \u000A smaller.  The need for such a choice arises, for example, when using equality \u000A hypotheses in conjectures (the smaller term is substituted for the larger \u000A elsewhere in the formula), in stopping loops in permutative rewrite rules \u000A (see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>), and in choosing the order in which to try to cancel \u000A the addends in linear arithmetic inequalities.  When this notion of syntactic \u000A size is needed, ACL2 uses ``term order.''  Popularly speaking, term order is \u000A just a lexicographic ordering on terms.  But the situation is actually more \u000A complicated.</p> \u000A \u000A <p>We define term order only with respect to terms in translated form.  See \u000A <see topic=\"ACL2____TRANS\">trans</see>.  Constants are viewed as built up by <i>pseudo-function</i> \u000A applications, as described at the end of this documentation.</p> \u000A \u000A <p><v>Term1</v> comes before <v>term2</v> in the term order iff</p> \u000A \u000A <blockquote> \u000A \u000A <p>(a) the number of variable occurrences in <v>term1</v> is less than that in \u000A <v>term2</v>, or</p> \u000A \u000A <p>(b) the numbers of variable occurrences in the two terms are equal but the \u000A number of function applications in <v>term1</v> is less than that in <v>term2</v>, \u000A or</p> \u000A \u000A <p>(c) the numbers of variable occurrences in the two terms are equal, the \u000A numbers of functions applications in the two terms are equal, but \u000A pseudo-function application count for <v>term1</v> is less than that for \u000A <v>term2</v>, or</p> \u000A \u000A <p>(d) the numbers of variable occurrences in the two terms are equal, the \u000A numbers of functions applications in the two terms are equal, the \u000A pseudo-function application counts for the two terms are equal, and <v>term1</v> \u000A comes before <v>term2</v> in a lexicographic ordering, <tt><see topic=\"ACL2____LEXORDER\">lexorder</see></tt>, based \u000A their structure as Lisp objects: see <see topic=\"ACL2____LEXORDER\">lexorder</see>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>The function <v>term-order</v>, when applied to the translations of two ACL2 \u000A terms, returns <v>t</v> iff the first is ``less than or equal'' to the second in \u000A the term order.</p> \u000A \u000A <p>By ``number of variable occurrences'' we do not mean ``number of distinct \u000A variables'' but ``number of times a variable symbol is mentioned.''  <v>(<see topic=\"COMMON-LISP____CONS\">Cons</see>\u000A x x)</v> has two variable occurrences, not one.  Thus, perhaps \u000A counterintuitively, a large term that contains only one variable occurrence, \u000A e.g., <v>(<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x)))</v> comes before <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x x)</v> in \u000A the term order.</p> \u000A \u000A <p>Since constants contain no variable occurrences and non-constant \u000A expressions must contain at least one variable occurrence, constants come \u000A before non-constants in the term order, no matter how large the constants. \u000A For example, the list constant</p> \u000A \u000A <code>'(monday tuesday wednesday thursday friday)</code> \u000A \u000A <p>comes before <v>x</v> in the term order.  Because term order is involved in \u000A the control of permutative rewrite rules and used to shift smaller terms to \u000A the left, a set of permutative rules designed to allow the permutation of any \u000A two tips in a tree representing the nested application of some function will \u000A always move the constants into the left-most tips.  Thus,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> x 3 (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> klst)) (dx i j)) ,</code> \u000A \u000A <p>which in translated form is</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x\u000A          (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A                    (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (dx i j)\u000A                              (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> klst))))),</code> \u000A \u000A <p>will be permuted under the built-in commutativity rules to</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '3\u000A          (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x\u000A                    (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> klst))\u000A                              (dx i j))))</code> \u000A \u000A <p>or</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> 3 x (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> klst)) (dx i j)).</code> \u000A \u000A <p>Two terms with the same numbers of variable occurrences and function \u000A applications and the same pseudo-function application count are ordered by \u000A lexicographic means, based on their structures.  See <see topic=\"ACL2____LEXORDER\">lexorder</see>.  Thus, \u000A if two terms <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> ...)</v> and <v>(<see topic=\"COMMON-LISP____REVERSE\">reverse</see> ...)</v> contain the same numbers \u000A of variable occurrences and function applications, and no quoted constants, \u000A then the <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> term is first in the term order because <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> \u000A comes before <tt><see topic=\"COMMON-LISP____REVERSE\">reverse</see></tt> in the term order (which is here reduced to \u000A alphabetic ordering).</p> \u000A \u000A <p>It remains to discuss the notion of <i>pseudo-function application \u000A count</i>.</p> \u000A \u000A <p>Clearly, two constants are ordered using cases (c) and (d) of term order, \u000A since they each contain 0 variable occurrences and no function calls.  This \u000A raises the question ``How many function applications are in a constant?'' \u000A Because we regard the number of function applications as a more fundamental \u000A measure of the size of a constant than lexicographic considerations, we \u000A decided that for the purposes of term order, constants would be seen as being \u000A built by primitive constructor functions.  These constructor functions are not \u000A actually defined in ACL2 but merely imagined for the purposes of term order. \u000A We here use suggestive names for these imagined functions, ignoring entirely \u000A the prior use of these names within ACL2.  The imagined applications of these \u000A functions are what we refer to as <i>pseudo-function</i> applications.</p> \u000A \u000A <p>The constant function <v>z</v> constructs <v>0</v>.  Positive integers are \u000A constructed from <v>(z)</v> by the successor function, <v>s</v>.  Thus <v>2</v> is \u000A <v>(s (s (z)))</v> and contains three function applications.  <v>100</v> contains \u000A one hundred and one applications.  Negative integers are constructed from \u000A their positive counterparts by <tt><see topic=\"COMMON-LISP____-\">-</see></tt>.  Thus, <v>-2</v> is <v>(<see topic=\"COMMON-LISP____-\">-</see> (s (s\u000A (z))))</v> and has four applications.  Ratios are constructed by the dyadic \u000A function <tt><see topic=\"COMMON-LISP_____F2\">/</see></tt>.  Thus, <v>-1/2</v> is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____-\">-</see> (s (z))) (s (s (z))))</code> \u000A \u000A <p>and contains seven applications.  Complex rationals are similarly \u000A constructed from rationals.  All character objects are considered primitive \u000A and are constructed by constant functions of the same name.  Thus <v>#\\a</v> \u000A and <v>#\\b</v> both contain one application.  Strings are built from the empty \u000A string, <v>(o)</v> by the ``string-cons'' function written <v>cs</v>.  Thus \u000A <v>&quot;AB&quot;</v> is <v>(cs (#\\a) (cs (#\\b) (o)))</v> and contains five \u000A applications.  Symbols are obtained from strings by ``packing'' the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> with the unary function <v>p</v>.  Thus <v>'ab</v> is</p> \u000A \u000A <code>(p (cs (#\\a) (cs (#\\b) (o))))</code> \u000A \u000A <p>and has six applications.  Note that packages are here ignored and thus \u000A <v>'acl2::ab</v> and <v>'my-package::ab</v> each contain just six applications. \u000A Finally, <see topic=\"COMMON-LISP____CONS\">cons</see>es are built with <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, as usual.  So <v>'(1\u000A . 2)</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> '1 '2)</v> and contains six applications, since <v>'1</v> \u000A contains two and <v>'2</v> contains three.  This, for better or worse, answers \u000A the question ``How many function applications are in a constant?''</p> \u000A \u000A <p>Finally, when we refer to the ``pseudo-function application count'', we \u000A mean the number of pseudo-function applications as described above, except \u000A that we bound this number by the constant <v>(fn-count-evg-max-val)</v>.  (This \u000A bound is important for efficiency, so that constants that are very large \u000A <v>cons</v> structures do not cause significant slowdown as ACL2 attempts to \u000A walk through them while computing their pseudo-function application \u000A count.)</p>"],

 "ACL2____SET-WATERFALL-PARALLELISM":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> nil)        ; never parallelize (serial execution)\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> :full)      ; always parallelize\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> :top-level) ; parallelize top-level subgoals\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>             ; parallelize if sufficient resources\u000A  :resource-based)                     ;   (recommended setting)\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> t)          ; alias for :resource-based\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>             ; parallelize if sufficient resources\u000A  :resource-and-timing-based           ;   and suggested by prior attempts\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>             ; never parallelize but use parallel\u000A  :pseudo-parallel)                    ;   code base (a debug mode)</code> \u000A \u000A <p><v>Set-waterfall-parallelism</v> evaluates its argument, which specifies the \u000A enabling or disabling of the <see topic=\"ACL2____PARALLEL\">parallel</see> execution of ACL2's main proof \u000A process, the waterfall.</p> \u000A \u000A <p>It also sets <see topic=\"ACL2____STATE\">state</see> global <v>waterfall-printing</v> to an appropriate \u000A value.  See <see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see>.</p> \u000A \u000A <p>Note that not all ACL2 features are supported when waterfall-parallelism is \u000A set to non-<v>nil</v> (see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see>).</p> \u000A \u000A <p>A value of <v>t</v> is treated the same as a value of <v>:resource-based</v> \u000A and is provided for user convenience.</p> \u000A \u000A <p><v>:Resource-based</v> waterfall parallelism typically achieves the best \u000A performance in ACL2(p), while maintaining system stability, so \u000A <v>:resource-based</v> (or equivalently, <v>t</v>) is the recommended value.</p> \u000A \u000A <p>A value of <v>nil</v> indicates that ACL2(p) should never prove subgoals in \u000A parallel.</p> \u000A \u000A <p>A value of <v>:full</v> indicates that ACL2(p) should always prove \u000A independent subgoals in parallel.</p> \u000A \u000A <p>A value of <v>:top-level</v> indicates that ACL2(p) should prove each of the \u000A top-level subgoals in parallel but otherwise prove subgoals in a serial \u000A manner.  This mode is useful when the user knows that there are enough \u000A top-level subgoals, many of which take a non-trivial amount of time to be \u000A proved, such that proving them in parallel will result in a useful reduction \u000A in overall proof time.</p> \u000A \u000A <p>A value of <v>:resource-based</v> (or equivalently, <v>t</v>) indicates that \u000A ACL2(p) should use its built-in heuristics to determine whether CPU core \u000A resources are available for parallel execution.  Note that ACL2(p) does not \u000A hook into the operating system to determine the workload on the machine. \u000A ACL2(p) works off the assumption that it is the only process using significant \u000A CPU resources, and it optimizes the amount of parallelism based on the number \u000A of CPU cores in the system.  (Note that ACL2(p) knows how to obtain the number \u000A of CPU cores from the operating system in CCL, but that, in SBCL and in \u000A Lispworks, a constant is used instead).</p> \u000A \u000A <p>During the first proof attempt of a given conjecture, a value of \u000A <v>:resource-and-timing-based</v> results in the same behavior as with \u000A <v>:resource-based</v>.  However, on subsequent proof attempts, the time it took \u000A to prove each subgoal will be considered when deciding whether to parallelize \u000A execution.  If a particular theorem's proof is already achieving satisfactory \u000A speedup via <v>:resource-based</v> parallelism, there is no reason to try this \u000A setting.  However, if the user wishes to experiment, the \u000A <v>:resource-and-timing-based</v> setting may improve performance.  Note that \u000A since the initial run does not have the subgoal proof times available, this \u000A mode will never be better than the <v>:resource-based</v> setting for \u000A non-interactive theorem proving.</p> \u000A \u000A <p>A value of <v>:pseudo-parallel</v> results in using the parallel waterfall \u000A code, but with serial execution.  This setting is useful for debugging the \u000A code base that supports parallel execution of the waterfall.  For example, you \u000A may wish to use this mode if you are an ``ACL2 Hacker'' who would like to see \u000A comprehensible output from tracing (see <see topic=\"ACL2____TRACE_42\">trace$</see>) the <v>@par</v> versions \u000A of the waterfall functions.</p> \u000A \u000A <p>The following remark pertains to those using the `HONS' experimental \u000A extension of ACL2 (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>; in particular, see <see topic=\"ACL2____MEMOIZE\">memoize</see>).  Since memoization is not supported when waterfall parallelism is \u000A enabled (see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see>), then when \u000A <v>set-waterfall-parallelism</v> is called with a non-<v>nil</v> value, all \u000A memoized functions are unmemoized.  When <v>set-waterfall-parallelism</v> is \u000A again called with a <v>nil</v> value, those memoization settings are \u000A restored.</p> \u000A \u000A <p><v>Set-waterfall-parallelism</v> is an embedded event form.  However, a call \u000A of this macro will not affect waterfall-parallelism when including a certified \u000A book that contains that call.  For such an effect, you may use the following \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> form; also see <see topic=\"ACL2____NON-PARALLEL-BOOK\">non-parallel-book</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> :full)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A            :check-expansion t)</code> \u000A \u000A <p>To enable waterfall parallelism for book certification using ACL2(p), see \u000A <see topic=\"ACL2____WATERFALL-PARALLELISM-FOR-BOOK-CERTIFICATION\">waterfall-parallelism-for-book-certification</see>.</p>"],

 "ACL2____SET-CASE-SPLIT-LIMITATIONS":[["Other"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> '(500 100))\u000A(<see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> 'nil)\u000A(<see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> '(500 nil))</code> \u000A \u000A <p>The first of these prevents <v>clausify</v> from trying the \u000A subsumption/replacement (see below) loop if more than 500 clauses are \u000A involved.  It also discourages the clause simplifier from splitting into more \u000A than 100 cases at once.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____HINTS\">hints</see> for discussion of a related hint, \u000A <v>:case-split-limitations</v>.  Also see <see topic=\"ACL2____SPLITTER\">splitter</see> for information about \u000A reports on rules that may be responsible for case splits.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> lst)</code> \u000A \u000A <p>where <v>lst</v> is either <v>nil</v> (denoting a list of two <v>nil</v>s), or a \u000A list of length two, each element of which is either <v>nil</v> or a natural \u000A number.  When <v>nil</v> is used as an element, it is treated as positive \u000A infinity.  The default setting is <v>(500 100)</v>.</p> \u000A \u000A <p>The first number specifies the maximum number of clauses that may \u000A participate in the ``subsumption/replacement'' loop.  Because of the expensive \u000A nature of that loop (which compares every clause to every other in a way that \u000A is quadratic in the number of literals in the clauses), when the number of \u000A clauses exceeds about 1000, the system tends to ``go into a black hole,'' \u000A printing nothing and not even doing many garbage collections (because the \u000A subsumption/replacement loop does not create new clauses so much as it just \u000A tries to delete old ones).  The initial setting is lower than the threshold at \u000A which we see noticeably bad performance, so you probably will not see this \u000A behavior unless you have raised or disabled the limit.</p> \u000A \u000A <p>Why raise the subsumption/replacement limit?  The subsumption/replacement \u000A loop cleans up the set of subgoals produced by the simplifier.  For example, \u000A if one subgoal is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p q r) s)            [1]</code> \u000A \u000A <p>and another is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p (<see topic=\"COMMON-LISP____NOT\">not</see> q) r) s)      [2]</code> \u000A \u000A <p>then the subsumption/replacement loop would produce the single subgoal</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p r) s)              [3]</code> \u000A \u000A <p>instead.  This cleanup process is simply skipped when the number of \u000A subgoals exceeds the subsumption/replacement limit.  But each subgoal must \u000A nonetheless be proved.  The proofs of [1] and [2] are likely to duplicate much \u000A work, which is only done once in proving [3].  So with a low limit, you may \u000A find the system quickly produces a set of subgoals but then takes a long time \u000A to prove that set.  With a higher limit, you may find the set of subgoals to \u000A be ``cleaner'' and faster to prove.</p> \u000A \u000A <p>Why lower the subsumption/replacement limit?  If you see the system go into \u000A a ``black hole'' of the sort described above (no output, and few garbage \u000A collections), it could due to the subsumption/replacement loop working on a \u000A large set of large subgoals.  You might temporarily lower the limit so that it \u000A begins to print the uncleaned set of subgoals.  Perhaps by looking at the \u000A output you will realize that some function can be disabled so as to prevent \u000A the case explosion.  Then raise or disable the limit again!</p> \u000A \u000A <p>The second number in the case-split-limitations specifies how many case \u000A splits the simplifier will allow before it begins to shut down case splitting. \u000A In normal operation, when a literal rewrites to a nest of <v>IF</v>s, the system \u000A simplifies all subsequent literals in all the contexts generated by walking \u000A through the nest in all possible ways.  This can also cause the system to ``go \u000A into a black hole'' printing nothing except garbage collection messages.  This \u000A ``black hole'' behavior is different from than mentioned above because space \u000A is typically being consumed at a prodigious rate, since the system is \u000A rewriting the literals over and over in many different contexts.</p> \u000A \u000A <p>As the simplifier sweeps across the clause, it keeps track of the number of \u000A cases that have been generated.  When that number exceeds the second number in \u000A case-split-limitations, the simplifier stops rewriting literals.  The \u000A remaining, unrewritten, literals are copied over into the output clauses. \u000A <v>IF</v>s in those literals are split out, but the literals themselves are not \u000A rewritten.  Each output clause is then attacked again, by subsequent \u000A simplification, and eventually the unrewritten literals in the tail of the \u000A clause will be rewritten because the earlier literals will stabilize and stop \u000A producing case splits.</p> \u000A \u000A <p>The default setting of 100 is fairly low.  We have seen successful proofs \u000A in which thousands of subgoals were created by a simplification.  By setting \u000A the second number to small values, you can force the system to case split \u000A slowly.  For example, a setting of 5 will cause it to generate ``about 5'' \u000A subgoals per simplification.</p> \u000A \u000A <p>You can read about how the simplifier works in the book Computer-Aided \u000A Reasoning: An Approach (Kaufmann, Manolios, Moore); also see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for a detailed tutorial on using the ACL2 \u000A prover.  If you think about it, you will see that with a low case limit, the \u000A initial literals of a goal are repeatedly simplified, because each time a \u000A subgoal is simplified we start at the left-most subterm.  So when case \u000A splitting prevents the later subterms from being fully split out, we revisit \u000A the earlier terms before getting to the later ones.  This can be good. \u000A Perhaps it takes several rounds of rewriting before the earlier terms are in \u000A normal form and then the later terms rewrite quickly.  But it could happen \u000A that the earlier terms are expensive to rewrite and do not change, making the \u000A strategy of delayed case splits less efficient.  It is up to you.</p> \u000A \u000A <p>Sometimes the simplifier produces more clauses than you might expect, even \u000A with case-split-limitations in effect.  As noted above, once the limit has \u000A been exceeded, the simplifier does not rewrite subsequent literals.  But \u000A <v>IF</v>s in those literals are split out nonetheless.  Furthermore, the \u000A enforcement of the limit is -- as described above -- all or nothing: if the \u000A limit allows us to rewrite a literal then we rewrite the literal fully, \u000A without regard for limitations, and get as many cases as ``naturally'' are \u000A produced.  It quite often happens that a single literal, when expanded, may \u000A grossly exceed the specified limits.</p> \u000A \u000A <p>If the second ``number'' is <v>nil</v> and the simplifier is going to produce \u000A more than 1000 clauses, a ``helpful little message'' to this effect is printed \u000A out.  This output is printed to the system's ``comment window'' not the \u000A standard proofs output.</p>"],

 "ACL2____GUARDS-AND-EVALUATION":[["Guard"],"ACL2 Sources","ACL2","<p>The guard has no effect on the logical axiom added by the \u000A definition of a function.  It does, however, have consequences for how calls \u000A of that function are evaluated in ACL2.  We begin by explaining those \u000A consequences, when ACL2 is in its default ``mode,'' i.e., as originally \u000A brought up.  In subsequent discussion we'll consider other ways that guards \u000A can interact with evaluation.</p> \u000A \u000A <p>For more about guards in general, see <see topic=\"ACL2____GUARD\">guard</see>.  For in-depth \u000A discussion of the interaction between the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and guard \u000A checking, see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>, see <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see>, \u000A see <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-SCRIPT\">guard-evaluation-examples-script</see>, and see <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-LOG\">guard-evaluation-examples-log</see>.  Also see <see topic=\"ACL2____GENERALIZED-BOOLEANS\">generalized-booleans</see> for \u000A discussion about a subtle issue in the evaluation of certain Common Lisp \u000A functions.</p> \u000A \u000A <p><i>Guards and evaluation I: the default</i></p> \u000A \u000A <p>Consider the following very simple definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</code> \u000A \u000A <p>First consider how raw Common Lisp behaves when evaluating calls of this \u000A function.  To evaluate <v>(foo x)</v> for some expression <v>x</v>, first <v>x</v> \u000A is evaluated to some value <v>v</v>, and then <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> is evaluated \u000A with <v>x</v> bound to <v>v</v>.  For example, if <v>v</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 'a 3)</v>, then \u000A Common Lisp computes <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 1 3)</v>.  But if (for example) <v>v</v> is a \u000A number, e.g., <v>7</v>, then there is no way to predict what Common Lisp might \u000A do.  Some implementations would cause ``sensible'' errors, others might return \u000A nonsense, still others might crash the host machine.  The results tend toward \u000A the catastrophic if the call of <v>foo</v> in question is in compiled code.</p> \u000A \u000A <p>Now by default, ACL2 evaluates calls of <v>foo</v> exactly as Common Lisp \u000A does, except that it uses guards to check the ``legality'' of each function \u000A call.  So for example, since <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> has a guard of <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</v>, the call <v>(foo 7)</v> would cause a ``guard violation,'' as \u000A illustrated below.</p> \u000A \u000A <code>ACL2 !&gt;(foo 7)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol CDR, which\u000Ais (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL)), is violated by the arguments in the\u000Acall (<see topic=\"COMMON-LISP____CDR\">CDR</see> 7).\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Thus, the relation between evaluation in ACL2 and evaluation in Common Lisp \u000A is that the two produce the very same results, provided there is no guard \u000A violation.</p> \u000A \u000A <p><i>Guards and evaluation II:</i> <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>.</p> \u000A \u000A <p>The ACL2 logic is a logic of total functions.  That is, every application \u000A of a function defined has a completely specified result.  See the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for each individual primitive for the specification of what it \u000A returns when its guard is violated; for example, see <see topic=\"COMMON-LISP____CDR\">cdr</see>.</p> \u000A \u000A <p>The presence of guards thus introduces a choice in the sense of evaluation. \u000A When you type a form for evaluation do you mean for guards to be checked or \u000A not?  Put another way, do you mean for the form to be evaluated in Common Lisp \u000A (if possible) or in the ACL2 logic?  Note: If Common Lisp delivers an answer, \u000A it will be the same as in the logic, but it might be erroneous to execute the \u000A form in Common Lisp.  For example, the ACL2 logic definition of <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> \u000A implies that the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of an <see topic=\"COMMON-LISP____ATOM\">atom</see> is <v>nil</v>; see <see topic=\"COMMON-LISP____CDR\">cdr</see>. \u000A So: should <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> 7)</v> cause a guard violation error or return <v>nil</v>?</p> \u000A \u000A <p>The top-level ACL2 loop has a variable which controls which sense of \u000A execution is provided.  By default, ``guard checking'' is on, by which we mean \u000A that guards are checked at runtime, in the sense already described.  To allow \u000A execution to proceed in the logic when there is a guard violation, do \u000A <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> nil</v>; or evaluate <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :none</v> to skip guard checking entirely.  To turn \u000A ``guard checking'' back on, execute the top-level form <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> t</v>.  The status of guard checking reflected in the \u000A <see topic=\"ACL2____PROMPT\">prompt</see>; guard-checking is ``on'' when the <see topic=\"ACL2____PROMPT\">prompt</see> contains an \u000A exclamation mark (also see <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>).  For example,</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>means guard checking is on and</p> \u000A \u000A <code>ACL2 &gt;</code> \u000A \u000A <p>means guard checking is off.  The exclamation mark can be thought of as \u000A ``barring'' certain computations.  The absence of the mark suggests the \u000A absence of error messages or unbarred access to the logical axioms.  Thus, for \u000A example</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will signal an error, while</p> \u000A \u000A <code>ACL2 &gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will return <v>nil</v>.  To return to our previous example: with guard \u000A checking off, <v>(foo 7)</v> evaluates to <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 1 nil)</v>.  Also see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.</p> \u000A \u000A <p><i>Guards and evaluation III: guard verification</i></p> \u000A \u000A <p>Consider the defininition of <v>foo</v> given above, but modified so that a \u000A reasonable guard of <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> is specified, as shown below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</code> \u000A \u000A <p>We say ``reasonable guard'' above because if <v>x</v> is such that <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see>\u000A x)</v> holds, then the call of <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> in the evaluation of <v>(foo x)</v> \u000A will not cause a guard violation.  Thus, it ``should'' be legal to evaluate \u000A <v>(foo x)</v>, for any such <v>x</v>, simply by evaluating this form in raw \u000A Common Lisp.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event has been provided for this purpose. \u000A Details may be found elsewhere; see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.  Briefly, for any \u000A defined function <v>fn</v>, the event <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fn)</v> attempts to check \u000A the condition discussed above, that whenever <v>fn</v> is called on arguments \u000A that satisfy its guard, the evaluation of this call will proceed without any \u000A guard violation.  (Moreover, the guard itself should be evaluable without any \u000A guard violation.)  If this check is successful, then future calls of this sort \u000A will be evaluated in raw Common Lisp.</p> \u000A \u000A <p>Returning to our example above, the <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)</v> will succeed \u000A because the guard <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> of <v>foo</v> implies the guard generated from \u000A the call <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> in the body of the definition, namely, <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</v> (see <see topic=\"COMMON-LISP____CDR\">cdr</see>).  Then the evaluation of <v>(foo (<see topic=\"COMMON-LISP____CONS\">cons</see> 'a\u000A 3))</v> will take place in raw Common Lisp, because <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 'a 3)</v> satisfies \u000A the guard of <v>foo</v>.</p> \u000A \u000A <p>This ability to dive into raw Common Lisp hinges on the proof that the \u000A guards you attach to your own functions are sufficient to ensure that the \u000A guards encountered in the body are satisfied.  This is called ``guard \u000A verification.'' Once a function has had its guards verified, then ACL2 can \u000A evaluate the function somewhat faster (but see ``Guards and evaluation V: \u000A efficiency issues'' below).  Perhaps more importantly, ACL2 can also guarantee \u000A that the function will be evaluated correctly by any implementation of Common \u000A Lisp (provided the guard of the function is satisfied on the input).  That is, \u000A if you have verified the guards of a system of functions and you have \u000A determined that they work as you wish in your host ACL2 (perhaps by proving \u000A it, perhaps by testing), then they will work identically in any Common \u000A Lisp.</p> \u000A \u000A <p>There is a subtlety to our treatment of evaluation of calls of functions \u000A whose guards have been verified.  If the function's guard is not satisfied by \u000A such a call, then no further attempt is made to evaluate any call of that \u000A function in raw lisp during the course of evaluation of that call.  This is \u000A obvious if guard checking is on, because an error is signalled the first time \u000A its guard is violated; but in fact it is also true when guard checking is off. \u000A See <see topic=\"ACL2____GUARD-EXAMPLE\">guard-example</see> for an example.</p> \u000A \u000A <p><i>Guards and evaluation IV: functions having :program mode</i></p> \u000A \u000A <p>Strictly speaking, functions in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode (see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>) do not have definitions in the ACL2 logic.  So what does it mean \u000A to evaluate calls of such functions in ACL2?  In general we treat <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions much as we treat <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> functions whose guards \u000A have been verified, except that when no error occurs then the corresponding \u000A raw Lisp function is always called.  (We say ``in general'' because there are \u000A exceptions, discussed in the ``Aside'' just below.)  Note that when the guard \u000A of a function in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode is violated, there is still a value \u000A that the ACL2 logic proves is equal to the given call.  But the same cannot be \u000A said for a function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  Nevertheless, for the sake \u000A of convenience we go ahead and evaluate the corresponding raw Lisp function \u000A except in the situation where the guard is violated and guard-checking is on, \u000A aside from the following:</p> \u000A \u000A <p><b>Aside</b>.  There are exceptions to the use of raw Lisp, discussed just \u000A above, to evaluate calls of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions.  The primary \u000A one is that after <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :none</v>, evaluation of \u000A user-defined <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function calls is done in the ACL2 \u000A logic, not in raw Lisp.  The more obscure exception is that during expansion \u000A of macros and <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms, and during evaluation of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> forms, ACL2 enters a ``safe mode'' in which this escape to raw Lisp \u000A is prevented.  The following example illustrates how the user can experiment \u000A directly with safe mode, though it is preferred to use <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :none</v> if you are happy to skip all guard checking and \u000A evaluate forms in the logic.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program :guard t))\u000A         (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 !&gt;(foo 3)\u000AError: Attempt to take the car of 3 which is not listp.\u000A  [condition type: SIMPLE-ERROR]\u000A\u000ARestart actions (select using :continue):\u000A 0: Return to Top Level (an &quot;abort&quot; restart).\u000A 1: Abort entirely from this process.\u000A[1] ACL2(2): :pop\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> safe-mode t)\u000A T\u000AACL2 !&gt;(foo 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol CAR, which\u000Ais (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL)), is violated by the arguments in the\u000Acall (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3).  See :DOC trace for a useful debugging utility.  See\u000A:DOC set-guard-checking for information about suppressing this check\u000Awith (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> safe-mode nil)\u000A NIL\u000AACL2 !&gt;(foo 3)\u000AError: Attempt to take the car of 3 which is not listp.\u000A  [condition type: SIMPLE-ERROR]\u000A\u000ARestart actions (select using :continue):\u000A 0: Return to Top Level (an &quot;abort&quot; restart).\u000A 1: Abort entirely from this process.\u000A[1] ACL2(2):</code> \u000A \u000A <p>The other exception occurs after <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> can be called \u000A with a value of <v>:all</v>; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  <b>End of \u000A aside.</b></p> \u000A \u000A <p>Thus, as with <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> functions: when a guard has been \u000A satisfied on a call of a function with <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, no \u000A subsidiary guard checking will be done.</p> \u000A \u000A <p>Notice that by treating functions in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode like \u000A functions whose guards have been verified, we are using raw lisp to compute \u000A their values when their guards are met.  We do not check guards any further \u000A once raw lisp is invoked.  This can lead to hard lisp errors if the guards are \u000A not appropriate, as illustrated below.</p> \u000A \u000A <code>ACL2 &gt;:program\u000AACL2 p&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.02 seconds (prove: 0.00, print: 0.00, proof tree: 0.00, other: 0.02)\u000A FOO\u000AACL2 p&gt;(foo 3)\u000A\u000AError: 3 is not of type LIST.\u000AFast links are on: do (use-fast-links nil) for debugging\u000AError signalled by CDR.\u000ABroken at COND.  Type :H for Help.\u000AACL2&gt;&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>.</p> \u000A \u000A <p>However, here is a way to get ACL2 to do run-time guard checking for \u000A user-defined <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions.  With this method, ACL2 \u000A will evaluate calls of user-defined <v>:program</v> mode functions in a manner \u000A that follows their ACL2 definitions.  Simply execute the following in the ACL2 \u000A loop to put ACL2 into a ``safe mode.''</p> \u000A \u000A <code>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'safe-mode t state)</code> \u000A \u000A <p>Let us revisit the example above, using safe mode.  Notice that the guard \u000A of <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is now being checked, because the executable counterpart of \u000A <v>foo</v> is being called even though the <see topic=\"ACL2____GUARD\">guard</see> is <v>t</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'safe-mode t state)\u000A&lt;state&gt;\u000AACL2 !&gt;:program\u000AACL2 p!&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 p!&gt;(foo 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol CDR, which\u000Ais (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL)), is violated by the arguments in the\u000Acall (<see topic=\"COMMON-LISP____CDR\">CDR</see> 3).  See :DOC trace for a useful debugging utility.  See\u000A:DOC set-guard-checking for information about suppressing this check\u000Awith (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 p!&gt;</code> \u000A \u000A <p>If we go back into ``unsafe'' mode, then we once again see a raw Lisp \u000A error, as we now illustrate.</p> \u000A \u000A <code>ACL2 p!&gt;(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'safe-mode nil state)\u000A&lt;state&gt;\u000AACL2 p!&gt;(foo 3)\u000A\u000AError: 3 is not of type LIST.\u000AFast links are on: do (si::use-fast-links nil) for debugging\u000AError signalled by CDR.\u000ABroken at COND.  Type :H for Help.\u000AACL2&gt;&gt;</code> \u000A \u000A <p><i>Guards and evaluation V: efficiency issues</i></p> \u000A \u000A <p>We have seen that by verifying the guards for a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A function, we arrange that raw lisp is used for evaluation of calls of such \u000A functions when the arguments satisfy its guard.</p> \u000A \u000A <p>This has several apparent advantages over the checking of guards as we go. \u000A First, the savings is magnified as your system of functions gets deeper: the \u000A guard is checked upon the top-level entry to your system and then raw Common \u000A Lisp does all the computing.  Second, if the raw Common Lisp is compiled (see \u000A <see topic=\"ACL2____COMPILATION\">compilation</see>), enormous speed-ups are possible.  Third, if your Common \u000A Lisp or its compiler does such optimizations as <v>tail-recursion</v> removal, \u000A raw Common Lisp may be able to compute your functions on input much ``bigger'' \u000A than ACL2 can.</p> \u000A \u000A <p>The first of these advantages is quite important if you have complicated \u000A guards.  However, the other two advantages are probably not very important, as \u000A we now explain.</p> \u000A \u000A <p>When a function is defined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, its <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A is executed in raw Common Lisp.  (We might call this the ``primary'' raw lisp \u000A definition of the function.)  However, a corresponding ``logic definition'' is \u000A also executed.  The ``logic definition'' is a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> in raw lisp that \u000A checks guards at runtime and escapes to the primary raw lisp definition if the \u000A guard holds of the arguments and the function has already had its guards \u000A verified.  Otherwise the logic definition executes the body of the function by \u000A calling the logic definitions of each subroutine.  Now it is true that <see topic=\"ACL2____COMPILATION\">compilation</see> generally speeds up execution enormously.  However, the \u000A <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> command (see <see topic=\"ACL2____COMP\">comp</see>) compiles both of the raw lisp \u000A definitions associated with a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> function.  Also, we have \u000A attempted to arrange that for every tail recursion removal done on the actual \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, a corresponding tail recursion removal is done on the ``logic \u000A definition.''</p> \u000A \u000A <p>We believe that in most cases, the logic definition executes almost as fast \u000A as the primary raw lisp definition, at least if the evaluation of the guards \u000A is fast.  So, the main advantage of guard verification is probably that it \u000A lets you know that the function may be executed safely in raw lisp, returning \u000A the value predicted by the ACL2 logic, whenever its arguments satisfy its \u000A guard.  We envision the development of systems of applicative lisp functions \u000A that have been developed and reasoned about using ACL2 but which are intended \u000A for evaluation in raw Common Lisp (perhaps with only a small ``core'' of ACL2 \u000A loaded), so this advantage of guard verification is important.</p> \u000A \u000A <p>Nevertheless, guard verification might be important for optimal efficiency \u000A when the functions make use of type declarations.  For example, at this \u000A writing, the GCL implementation of Common Lisp can often take great advantage \u000A of <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms that assign small integer types to formal parameters. \u000A Note that while type declarations contributed to the guard by default, they \u000A can be proved from the guard instead; see <see topic=\"ACL2____XARGS\">xargs</see> for a discussion of the \u000A <v>:SPLIT-TYPES</v> keyword.</p> \u000A \u000A <p>To continue the discussion of guards, see <see topic=\"ACL2____GUARDS-FOR-SPECIFICATION\">guards-for-specification</see> \u000A to read about the use of guards as a specification device.</p>"],

 "ACL2____BOOK-COMPILED-FILE":[["Books-reference"],"ACL2 Sources","ACL2","<p>An effect of <see topic=\"ACL2____COMPILATION\">compilation</see> is to speed up the execution of the \u000A functions defined in a book.  Compilation can also remove tail recursion, thus \u000A avoiding stack overflows.  The presence of compiled code for the functions in \u000A the book should not otherwise affect the performance of ACL2.  See <see topic=\"ACL2____GUARD\">guard</see> for a discussion; also See <see topic=\"ACL2____COMPILATION\">compilation</see>.</p> \u000A \u000A <p>By default, the <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command compiles the book that it \u000A certifies.  see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> for how to control this behavior.</p> \u000A \u000A <p>By default, the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> command loads the compiled file for \u000A the book.  The details of how this loading works are subtle, and do not need \u000A to be understood by most users.  The ACL2 source code contains an ``Essay on \u000A Hash Table Support for Compilation'' that explains such details for those \u000A interested.  All that users should generally need to know about this is that \u000A the compiled file is always the result of compiling a so-called ``expansion \u000A file'', which contains certain additional code besides the book itself.  The \u000A relevance to users of the expansion file is that it can be loaded if the \u000A compiled file is missing (except when <v>:load-compiled-file t</v> is specified \u000A by the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form), and its existence is required in order for \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to create a book's compiled file, as described below.</p> \u000A \u000A <p>Most users can skip the remainder of this documentation topic, which \u000A addresses the uncommon activity of using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to compile \u000A books.</p> \u000A \u000A <p><v>Include-book</v> can be made to compile a book by supplying its keyword \u000A argument <v>:load-compiled-file</v> the value <v>:comp</v>.  However, a compiled \u000A file can only be produced if there is already an <i>expansion file</i> that is \u000A at least as recent as the book's <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  Such a file, whose name \u000A happens to be the result of concatenating the string <v>&quot;@expansion.lsp&quot;</v> \u000A to the book name (without the <v>&quot;.lisp&quot;</v> suffix), is created by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> when state global variable <v>'save-expansion-file</v> has a \u000A non-<v>nil</v> value.  That will be the case if ACL2 started up when environment \u000A variable <v>ACL2_SAVE_EXPANSION</v> was <v>t</v> (or any value that is not the \u000A empty string and whose <tt><see topic=\"COMMON-LISP____STRING-UPCASE\">string-upcase</see></tt> is not <v>&quot;NIL&quot;</v>), until the \u000A time (if any) that <v>'save-expansion-file</v> is assigned a different value by \u000A the user.  In most respects, the <v>:comp</v> setting is treated exactly the \u000A same as <v>:warn</v>; but after all events in the book are processed, the \u000A expansion file is compiled if a compiled file was not loaded, after which the \u000A resulting compiled file is loaded.</p> \u000A \u000A <p>One can thus, for example, compile books for several different host Lisps \u000A &mdash; useful when installing ACL2 executables at the same site that are \u000A built on different host Lisps.  A convenient way to do this in an environment \u000A that provides Gnu `make' is to certify the community books using the shell \u000A command ``<v>make regression</v>'' in the <v>acl2-sources/</v> directory, after \u000A setting environment variable <v>ACL2_SAVE_EXPANSION</v> to <v>t</v>, and then \u000A moving to the <v>books</v> directory and executing the appropriate `make' \u000A commands to compile the books (targets <v>fasl</v>, <v>o</v>, and so on, according \u000A to the compiled file extension for the host Lisp).</p> \u000A \u000A <p>We conclude by saying more about the <v>:load-compiled-file</v> argument of \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  We assume that <see topic=\"ACL2____STATE\">state</see> global \u000A <v>'compiler-enabled</v> has a non-<v>nil</v> value; otherwise \u000A <v>:load-compiled-file</v> is always treated as <v>nil</v>.</p> \u000A \u000A <p>We do not consider raw mode below (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), which presents \u000A a special case: ACL2 will attempt to load the book itself whenever it would \u000A otherwise load the expansion or compiled file, but cannot (either because the \u000A <v>:load-compiled-file</v> argument is <v>nil</v>, or for each of the expansion \u000A and compiled files, either it does not exist or it is out of date with respect \u000A to the <v>.cert</v> file).</p> \u000A \u000A <p>The <v>:load-compiled-file</v> argument is not recursive: calls of \u000A <v>include-book</v> that are inside the book supplied to <v>include-book</v> use \u000A their own <v>:load-compiled-file</v> arguments.  However, those subsidiary \u000A <v>include-book</v> calls can nevertheless be sensitive to the \u000A <v>:load-compiled-file</v> arguments of enclosing <v>include-book</v> calls, as \u000A follows.  If <v>:load-compiled-file</v> has value <v>t</v>, then every subsidiary \u000A <v>include-book</v> is required to load a compiled file.  Moreover, if a book's \u000A compiled file or expansion file is loaded in raw Lisp, then an attempt will be \u000A made to load the compiled file or expansion file for any <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A form encountered during that load.  If that attempt fails, then that load \u000A immediately aborts, as does its parent load, and so on up the chain.  If, when \u000A going up the chain, an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is aborted for which keyword \u000A argument <v>:load-compiled-file</v> has value <v>t</v>, then an error occurs.</p> \u000A \u000A <p>When loading a book's compiled file or expansion file, <v>FILE</v>, it is \u000A possible to encounter an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form for a book that has no \u000A suitable compiled file or expansion file.  In that case, the load of <v>FILE</v> \u000A is aborted at that point.  Similarly, the load of <v>FILE</v> is aborted in the \u000A case that this <v>include-book</v> form has a suitable compiled file or \u000A expansion file whose load is itself aborted.  Thus, whenever any \u000A <v>include-book</v> aborts, so do all of its parent <v>include-book</v>s, up the \u000A chain.  Such an abort causes an error when the <v>include-book</v> form \u000A specifies a <v>:load-compiled-file</v> value of <v>t</v>.</p>"],

 "ACL2____SET-WRITE-ACL2X":[["Books-reference"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> nil state)\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> t state)\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> '(nil) state) ; same as just above, but allow inclusion of\u000A                               ; uncertified books during certify-book\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> '(t) state)\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> '(include-book-with-locals) state)</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> val state)</code> \u000A \u000A <p>where <v>val</v> evaluates to <v>t</v>, <v>nil</v>, or a one-element list whose \u000A element is a legal value for the global <v>'ld-skip-proofsp</v>; see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.  The value returned is an error triple, which in the \u000A non-error case is <v>(<see topic=\"ACL2____MV\">mv</see> nil v state)</v>, where <v>v</v> is the value of <v>val</v> \u000A and <v>state</v> is the result of updating the input <see topic=\"ACL2____STATE\">state</see> by assigning \u000A state global <v>'write-acl2x</v> the value <v>v</v>.</p> \u000A \u000A <p>The command <v>(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> val state)</v> assigns the value of <v>val</v> \u000A to the <see topic=\"ACL2____STATE\">state</see> global variable <v>'write-acl2x</v>, affecting whether or \u000A not <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> writes out a file with extension <v>acl2x</v>, called a \u000A ``<v>.acl2x</v> file'' and pronounced ``dot-acl2x file''.  Such a file is read \u000A or written by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> when it is supplied with keyword argument \u000A <v>:acl2x t</v>.  By default, such a call of <v>certify-book</v> reads a \u000A <v>.acl2x</v> file; but if the value of state global variable <v>'write-acl2x</v> \u000A is not <v>nil</v>, then <v>certify-book</v> writes a <v>.acl2x</v> file (in which \u000A case it is illegal to specify a non-<v>nil</v> value for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A keyword argument <v>:pcert</v>).  Consider for example <v>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;foo&quot;\u000A 0 nil :acl2x t)</v>.  By default, this command reads file <v>foo.acl2x</v>, which \u000A supplies replacements for some forms in <v>foo.lisp</v>, as described later \u000A below.  But if the value of state global <v>'write-acl2x</v> is not <v>nil</v>, \u000A then instead, this <v>certify-book</v> command writes such a file \u000A <v>foo.acl2x</v>.</p> \u000A \u000A <p>Before we discuss the function of <v>.acl2x</v> files, we first explain more \u000A about how a non-<v>nil</v> value of <see topic=\"ACL2____STATE\">state</see> global <v>'write-acl2x</v> \u000A affects the behavior of a command <v>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> ... :acl2x t ...)</v>.  A \u000A significant effect on the behavior is that after processing events in the \u000A given book, ACL2 writes out a <v>.acl2x</v> file and then returns, skipping the \u000A other subsequent actions typically performed by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>: a <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see> check, writing of a <see topic=\"ACL2____CERTIFICATE\">certificate</see> file, and \u000A possibly <see topic=\"ACL2____COMPILATION\">compilation</see>.  Another effect is that proofs may be skipped \u000A when processing <see topic=\"ACL2____EVENTS\">events</see> assuming that the the <v>certify-book</v> command \u000A does not explicitly specify <v>:skip-proofs-okp nil</v>, as we now explain.  A \u000A non-<v>nil</v> value of <v>'write-acl2x</v> should either be <v>t</v> or a \u000A one-element list <v>(x)</v>, where <v>x</v> is a legal value for the <see topic=\"ACL2____STATE\">state</see> \u000A global <v>'ld-skip-proofsp</v> (see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>).  In both cases, \u000A <v>certify-book</v> will process <see topic=\"ACL2____EVENTS\">events</see> to write out a <v>.acl2x</v> file \u000A as described above.  But in the latter (list) case, event processing will take \u000A place according to the value of <v>x</v>: in particular, proofs will be skipped \u000A when <v>x</v> is not <v>nil</v>, and if moreover <v>x</v> is the symbol \u000A <v>include-book-with-locals</v>, then only one pass will be made through each \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form.  A third effect of a non-<v>nil</v> value of \u000A <v>'write-acl2x</v>, which is restricted to the list case, is that <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> events encountered during event processing are allowed to \u000A succeed on uncertified books, something that is prohibited during most calls \u000A of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is used to write out a <v>.acl2x</v> file, there is \u000A typically a subsequent run of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> that reads that file. \u000A Consider how this can work with a book <v>foo.lisp</v>.  In the first call of \u000A <v>certify-book</v>, a file <v>foo.acl2x</v> is written that contains all <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansions, but <v>foo.cert</v> is not written.  In the second call \u000A of <v>certify-book</v>, no <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion typically takes place, \u000A because <v>foo.acl2x</v> supplies the expansions.  The command \u000A <v>(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> t state)</v> should be evaluated before the first \u000A certification (though another legal non-<v>nil</v> value may be used in place of \u000A <v>t</v>), setting the value of <tt><see topic=\"ACL2____STATE\">state</see></tt> global <v>'write-acl2x</v> to \u000A <v>t</v>, to enable writing of <v>foo.acl2x</v>; and the command \u000A <v>(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> nil state)</v> may be evaluated before the second run \u000A (though this is not necessary in a fresh ACL2 session) in order to complete \u000A the certification (writing out <v>foo.cert</v>) using <v>foo.acl2x</v> to supply \u000A the <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansions.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> is supplied with keyword argument <v>:acl2x t</v> \u000A it will read or write the book's <v>.acl2x</v> file; when supplied with \u000A <v>:acl2x nil</v>, it will not read or write that <v>.acl2x</v> file.  The value \u000A of <v>:acl2x</v> is <v>nil</v> by default.  The interaction of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> with the corresponding <v>.acl2x</v> file is as follows.</p> \u000A \u000A <code>o If <v>:acl2x</v> is <v>t</v>, then: \u000A  - If <v>set-write-acl2x</v> has been (most recently) called with a value of \u000A    <v>t</v> for its first argument,then ACL2 writes the corresponding \u000A    <v>.acl2x</v> file. \u000A  - If <v>set-write-acl2x</v> has been (most recently) called with a value of \u000A    <v>nil</v> for its first argument, or not called at all, then ACL2 insists \u000A    on a corresponding <v>.acl2x</v> file that is at least as recent as the \u000A    corresponding <v>.lisp</v> file, causing an error otherwise. \u000A o If <v>:acl2x</v> is <v>nil</v>,then: \u000A  - If <v>set-write-acl2x</v> has been (most recently) called with a value \u000A    <v>t</v> for its first argument, or if argument <v>:ttagsx</v> is supplied, \u000A    then an error occurs. \u000A  - If the <v>.acl2x</v> file exists, then regardless of whether or how \u000A    <v>set-write-acl2x</v> has been called, ACL2 ignores the <v>.acl2x</v> file \u000A    but issues a warning about it. \u000A</code> \u000A \u000A <p>Suppose you use the two-runs approach: first write a <v>.acl2x</v> file, then \u000A certify using (reading) that <v>.acl2x</v> file.  Then with scripts such as \u000A makefiles, then you may wish to provide a single <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command \u000A to use for both runs.  For that purpose, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> supports the \u000A keyword argument <v>:ttagsx</v>.  If this argument is supplied and \u000A <v>write-acl2x</v> is true, then this argument is treated as the <v>:ttags</v> \u000A argument, overriding a <v>:ttags</v> argument if present.  That is, for the two \u000A runs, <v>:ttagsx</v> may be used to specify the trust tags used in the first \u000A certification while <v>:ttags</v> specifies the trust tags, if any (else \u000A <v>:ttags</v> may be omitted), used in the second certification.  Note: If the \u000A argument <v>:ttagsx</v> is not supplied, then its value defaults to the \u000A (explicit or default) value of the <v>:ttags</v> argument.</p> \u000A \u000A <p>The built-in ACL2 Makefile support automatically generates suitable \u000A dependencies if you create a <v>.acl2</v> file with a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> call \u000A matching the following regular expression, case-independent:</p> \u000A \u000A <code>(certify-book[^;]*:acl2x t</code> \u000A \u000A <p>For an example <v>.acl2</v> file with a <v>certify-book</v> call matching the \u000A above pattern, see community books file \u000A <v>books/make-event/double-cert-test-1.acl2</v>.</p> \u000A \u000A <p>Note that <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is generally not affected by \u000A <v>set-write-acl2x</v>, other than through the indirect effect on <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  More precisely: All expansions are stored in the <see topic=\"ACL2____CERTIFICATE\">certificate</see> file, so when <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is applied to a certified \u000A book, the <v>.acl2x</v> file is not consulted.</p> \u000A \u000A <p>An example of how to put this all together may be found in community book \u000A <v>books/make-event/double-cert-test-1.lisp</v>.  There, we see the following \u000A form.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____DEFTTAG\">defttag</see> :my-ttag)\u000A        (<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"COMMON-LISP____LET\">let</see> ((val (<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;pwd&quot; nil)))\u000A                  (value (<see topic=\"COMMON-LISP____LIST\">list</see> 'defun 'foo () val))))))</code> \u000A \u000A <p>Imagine that in place of the binding computed using <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>, which \u000A by the way requires a trust tag, is some computation of your choice (such as \u000A reading forms from a file) that is used to construct your own event, in place \u000A of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event constructed above.  The <v>Makefile</v> in that \u000A directory contains the following added dependency, so that file \u000A <v>double-cert-test-1.acl2x</v> will be created:</p> \u000A \u000A <code>double-cert-test-1.cert: double-cert-test-1.acl2x</code> \u000A \u000A <p>There is also the file <v>double-cert-test-1.acl2</v> in that directory, \u000A which contains a single form as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;double-cert-test-1&quot; ? t :ttagsx :all :ttags nil)</code> \u000A \u000A <p>Thus, a call of `make' first creates file <v>double-cert-test-1.acl2x</v>, \u000A which uses the above <v>:ttagsx</v> argument in order to support the use of \u000A <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion.  Then, `make' goes on to \u000A cause a second certification in which no trust tags are involved.  As a \u000A result, the parent book <v>double-cert-test.lisp</v> is ultimately certified \u000A without requiring any trust tags.</p> \u000A \u000A <p>The discussion above is probably sufficient for most users of the two-run \u000A approach it describes.  We conclude with further details for those who want \u000A more information.  Those who wish to see a yet lower-level explanation of how \u000A all this works are invited to read the comment in the ACL2 source code \u000A entitled ``Essay on .acl2x Files (Double Certification).</p> \u000A \u000A <p>Consider the <v>.acl2x</v> file produced by the first run as described above. \u000A It contains a single expression, which is an association list whose keys are \u000A all positive integers, which occur in increasing order.  When the <v>.acl2x</v> \u000A file is present and at least as recent as the corresponding <v>.lisp</v> file, \u000A then for a subsequent <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> with argument <v>:acl2x t</v> and the \u000A (default) value of <v>nil</v> for <see topic=\"ACL2____STATE\">state</see> global <v>'write-acl2x</v>, that \u000A association list will be applied to the top-level events in the book, as \u000A follows.  Suppose the entry <v>(n . ev)</v> belongs to the association list in \u000A the <v>.acl2x</v> file.  Then <v>n</v> is a positive integer, and the <v>n</v>th \u000A top-level event in the book &mdash; where the <v>0</v>th event is the initial \u000A <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> form &mdash; will be replaced by <v>ev</v>.  In practice, \u000A <v>ev</v> is the <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion created during certification for \u000A the <v>nth</v> top-level event in the book; and this will always be the case if \u000A the <v>.acl2x</v> file is created by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> after execution of the \u000A form <v>(<see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-acl2x</see> t state)</v>.  However, you are welcome to associate \u000A indices manually with any <see topic=\"ACL2____EVENTS\">events</see> you wish into the alist stored in the \u000A <v>.acl2x</v> file.</p> \u000A \u000A <p>Note: Also see the community book <v>make-event/acl2x-help.lisp</v> for a \u000A useful utility that can be used to skip proofs during the writing of \u000A <v>.acl2x</v> files.</p>"],

 "ACL2____PARALLELISM":[["ACL2"],"ACL2 Sources","ACL2","<p>This documentation topic relates to an experimental extension of \u000A ACL2, ACL2(p), created initially by David L. Rager.  See <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see> for how to build an executable image that supports parallel \u000A execution.  Also see community books directory <v>books/parallel/</v> for \u000A examples.  For a completely different sort of parallelism, at the system \u000A level, see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.</p> \u000A \u000A <p>IMPORTANT NOTE.  We hope and expect that every evaluation result is \u000A correctly computed by ACL2(p), and that every formula proved using ACL2(p) is \u000A a theorem of the ACL2 logic (and in fact is provable using ACL2).  However, we \u000A do not guarantee these properties.  Since ACL2(p) is intended to be an aid in \u000A efficient evaluation and proof development, we focus less on ironclad \u000A soundness and more on providing an efficient and working implementation. \u000A Nevertheless, if you encounter a case where ACL2(p) computes an incorrect \u000A result, or produces a proof where ACL2 fails to do so (and this failure is not \u000A discussed in <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see>), please notify \u000A the implementors.</p> \u000A \u000A <p>The ACL2 source code provides experimental parallel execution and proof \u000A capabilities.  For example, one of ACL2's strengths lies in its ability to \u000A simulate industrial models efficiently, and it can also decrease the time \u000A required for proofs about such models both by making use of parallel \u000A evaluation and by dispatching proof subgoals in parallel.</p> \u000A \u000A <p>While we aim to support Clozure Common Lisp (CCL), Steel Bank Common Lisp \u000A (SBCL), and Lispworks, SBCL and Lispworks both currently sometimes experience \u000A problems when evaluating the ACL2 proof process (the ``waterfall'') in \u000A parallel.  Therefore, CCL is the recommend Lisp for anyone that wants to use \u000A parallelism and isn't working on fixing those problems.</p>"],

 "ACL2____ORACLE-APPLY":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Oracle-apply</v> evaluates its first argument to produce an ACL2 \u000A function symbol, <v>FN</v>, and then applies <v>FN</v> to the value of the second \u000A argument, which should be a true list whose length is the number of inputs for \u000A <v>FN</v>.  The return value is of the form <v>(<see topic=\"ACL2____MV\">mv</see> call-result state)</v>.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> 'cons '(3 4) state) = (<see topic=\"ACL2____MV\">mv</see> '(3 . 4) &lt;state&gt;)\u000A(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> (<see topic=\"COMMON-LISP____CAR\">car</see> '(<see topic=\"COMMON-LISP____FLOOR\">floor</see> foo)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 6 7) 5) state) = (<see topic=\"ACL2____MV\">mv</see> 2 &lt;state&gt;)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see> for a related utility.</p> \u000A \u000A <p>Note that calls of <v>oracle-funcall</v> and <v>oracle-apply</v> return two \u000A values: the result of the function application, and a modified <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p><v>Oracle-apply</v> is defined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, and in fact is \u000A <see topic=\"ACL2____GUARD\">guard</see>-verified.  However, you will not be able to prove much about this \u000A function, because it is defined in the logic using the <v>acl2-oracle</v> field \u000A of the ACL2 <see topic=\"ACL2____STATE\">state</see>; see <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>.  The behavior \u000A described above &mdash; i.e., making a function call &mdash; takes place when \u000A the third argument is the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, so during proofs (when that can \u000A never happen), a term <v>(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> 'fn '...)</v> will not simplify using a \u000A call of <v>fn</v>.</p> \u000A \u000A <p>The guard for <v>(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> fn args state)</v> is the term \u000A <v>(oracle-apply-guard fn args state)</v>, which we describe below.</p> \u000A \u000A <p><b>Function: </b><srclink>oracle-apply-guard</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oracle-apply-guard (fn args state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (f-boundp-global 'temp-touchable-fns\u000A                             state)\u000A            (ev-fncall-w-guard fn args (w state)\u000A                               (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'temp-touchable-fns\u000A                                             state))))</code> \u000A \u000A <p>where:</p> \u000A \u000A <p><b>Function: </b><srclink>ev-fncall-w-guard</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A ev-fncall-w-guard\u000A (fn args wrld temp-touchable-fns)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A (<see topic=\"COMMON-LISP____AND\">and</see>\u000A  (plist-worldp wrld)\u000A  (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> fn)\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> fn 'if))\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> fn *ttag-fns-and-macros*))\u000A  (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> args)\u000A  (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((formals (<see topic=\"ACL2____GETPROP\">getprop</see> fn 'formals\u000A                           t 'current-acl2-world\u000A                           wrld))\u000A         (stobjs-in (stobjs-in fn wrld))\u000A         (untouchable-fns (global-val 'untouchable-fns wrld)))\u000A        (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> formals t))\u000A             (<see topic=\"COMMON-LISP____EQL\">eql</see> (<see topic=\"ACL2____LEN\">len</see> formals) (<see topic=\"ACL2____LEN\">len</see> args))\u000A             (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> untouchable-fns)\u000A             (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> fn untouchable-fns))\u000A                 (<see topic=\"COMMON-LISP____AND\">and</see> temp-touchable-fns\u000A                      (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> t temp-touchable-fns)\u000A                          (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> temp-touchable-fns)\u000A                               (<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> fn temp-touchable-fns)))))\u000A             (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NULL\">null</see> formals)\u000A                       (<see topic=\"ACL2____GETPROP\">getprop</see> fn 'stobj-function\u000A                                nil 'current-acl2-world\u000A                                wrld)))\u000A             (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> stobjs-in)\u000A             (all-nils stobjs-in)))))</code> \u000A \u000A <p>These definitions say that <v>fn</v> is a function symbol other than <v>if</v>; \u000A it is not among the keys of the alist, <v>*ttag-fns-and-macros*</v>; and it does \u000A not take or create a <see topic=\"ACL2____STOBJ\">stobj</see> (see <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>).  Moreover, the \u000A second argument, <v>args</v>, must be a true list whose length is the number of \u000A formal parameters of <v>fn</v>.  These requirements may be a bit onerous for \u000A guard verification of functions that call <v>oracle-apply</v>, but this is \u000A easily overcome by using <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, for example as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A  (<see topic=\"ACL2____EC-CALL\">ec-call</see> (<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> 'car (<see topic=\"COMMON-LISP____LIST\">list</see> x) state)))</code> \u000A \u000A <p>This use of <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt> will, however, cause the <see topic=\"ACL2____GUARD\">guard</see> of \u000A <v>oracle-apply</v> to be checked at runtime.</p> \u000A \u000A <p>If the <see topic=\"ACL2____GUARD\">guard</see> for <v>oracle-apply</v> fails to hold but there is no \u000A guard violation because guard-checking is suppressed (see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>), then the value returned is computed using its logical \u000A definition &mdash; which, as mentioned above, uses the ACL2 oracle &mdash; and \u000A hence the value computed is unpredictable (indeed, the function argument will \u000A not actually be called).</p> \u000A \u000A <p>The value returned by <v>oracle-apply</v> is always a single value obtained \u000A by calling the executable counterpart of its function argument, as we now \u000A explain.  Consider a form <v>(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> fn args state)</v> that evaluates to \u000A <tt>(mv VAL state')</tt>, where <v>fn</v> evaluates to the function symbol \u000A <v>F</v>.  If <v>F</v> returns multiple values, then <v>VAL</v> is the first value \u000A computed by the call of <v>F</v> on the value of <v>args</v>.  More precisely, \u000A <v>oracle-apply</v> actually invokes the executable counterpart of <v>F</v>; thus, \u000A if <v>args</v> is the expression <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x1 ... xk)</v>, then <v>VAL</v> is the \u000A same as (first) value returned by evaluating <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> (F x1 x2 ... xk))</v>. \u000A See <see topic=\"ACL2____EC-CALL\">ec-call</see>.</p> \u000A \u000A <p>(Remark.  If you identify a need for a version of <v>oracle-apply</v> to \u000A return multiple values, we can perhaps provide such a utility; feel free to \u000A contact the ACL2 implementors to request it.)</p> \u000A \u000A <p>A subtlety is that the evaluation takes place in so-called ``safe mode'', \u000A which avoids raw Lisp errors due to calls of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode \u000A functions.  The use of safe mode is unlikely to be noticed if the value of the \u000A first argument of <v>oracle-apply</v> is a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function \u000A symbol.  However, for <v>:program</v> mode functions with side effects due to \u000A special raw Lisp code, as may be the case for built-in functions or for custom \u000A functions defined with active trust tags (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), use of the \u000A function <tt><see topic=\"ACL2____ORACLE-APPLY-RAW\">oracle-apply-raw</see></tt> may be preferable.  It is a much less \u000A restrictive version of <v>oracle-apply</v>, which avoids safe mode and (for \u000A example) can apply a function that has a definition in the host Lisp but not \u000A in the ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>oracle-apply</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oracle-apply (fn args state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                       :guard (oracle-apply-guard fn args state)))\u000A       (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A               (<see topic=\"ACL2____READ-ACL2-ORACLE\">read-acl2-oracle</see> state)\u000A               (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> erp))\u000A               (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> val)\u000A                        (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> val) fn)\u000A                        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> val) args)\u000A                        (<see topic=\"COMMON-LISP____CADDR\">caddr</see> val))\u000A                   state)))</code>"],

 "ACL2____MARKUP":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings make special use of the tilde \u000A character \u000A (~).  In particular, we describe here a ``markup language'' for which the \u000A tilde character plays a special role.  The markup language is valuable if you \u000A want to write <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> that is to be displayed outside your ACL2 \u000A session.  If you are not writing such <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, and if also you do \u000A not use the character `~', then there is no need to read on.</p> \u000A \u000A <p>Three uses of the tilde character (~) in <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings are \u000A as follows.  Below we explain the uses that constitute the ACL2 markup \u000A language.</p> \u000A \u000A <blockquote><p><v>~/</v><br></br> \u000A \u000A Indicates the end of a documentation <b>section</b>; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p><v>~~</v><br></br> \u000A \u000A Indicates the literal insertion of a tilde character (~).</p> \u000A \u000A <p><v>~]</v><br></br> \u000A \u000A This directive in a documentation string is effective only during the \u000A processing of part 2, the details (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>), and controls how \u000A much is shown on each round of <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> processing when printing to \u000A the terminal.  If the system is not doing <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> processing, then \u000A it acts as though the ~] is not present.  Otherwise, the system put out a \u000A newline and halts documentation printing on the present topic, which can be \u000A resumed if the user types <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> at the terminal.</p></blockquote> \u000A \u000A <p>The other uses of the tilde character are of the following form.</p> \u000A \u000A <code>~key[arg]</code> \u000A \u000A <p>Before launching into an explanation of how this works in detail, let us \u000A consider some small examples.</p> \u000A \u000A <p>Here is a word that is code:</p> \u000A \u000A <code>~c[function-name].</code> \u000A \u000A <p>Here is a phrase with an ``emphasized'' word, ``not'':</p> \u000A \u000A <code>Do ~em[not] do that.</code> \u000A \u000A <p>Here is the same phrase, but where ``not'' receives stronger emphasis \u000A (presumably boldface in a printed version):</p> \u000A \u000A <code>Do ~st[not] do that.</code> \u000A \u000A <p>Here is a passage that is set off as a display, in a fixed-width font:</p> \u000A \u000A <code>~bv[]\u000AThis passage has been set off as ``verbatim''.\u000AThe present line starts just after a line break.  Normally, printed\u000Atext is formatted, but inside ~bv[]...~ev[], line breaks are taken\u000Aliterally.\u000A~ev[]</code> \u000A \u000A <p>In general, the idea is to provide a ``markup language'' that can be \u000A reasonably interpreted not only at the terminal (via <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>), but \u000A also via translators into other languages.  In fact, translators have been \u000A written into Texinfo and HTML.</p> \u000A \u000A <p>Let us turn to a more systematic consideration of how to mark text in <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings using expressions of the form <v>~key[arg]</v>, which we \u000A will call ``<see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde directives.''  The idea is that <v>key</v> \u000A informs the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> printer (which could be the terminal, a \u000A hardcopy printer, or some hypertext tool) about the ``style'' used to display \u000A <v>arg</v>.  The intention is that each such printer should do the best it can. \u000A For example, we have seen above that <v>~em[arg]</v> tells the printer to \u000A <i>emphasize</i> <v>arg</v> if possible, using an appropriate display to \u000A indicate emphasis (italics, or perhaps surrounding <v>arg</v> with some \u000A character like <v>_</v>, or ...).  For another example, the directive for bold \u000A font, <v>~b[arg]</v>, says that printed text for <v>arg</v> should be in bold if \u000A possible, but if there is no bold font available (such as at the terminal), \u000A then the argument should be printed in some other reasonable manner (for \u000A example, as ordinary text).  The <v>key</v> part is case-insensitive; for \u000A example, you can use ~BV[] or ~Bv[] or ~bV[] in place of ~bv[].</p> \u000A \u000A <p>Every form below may have any string as the argument (inside <v>[..]</v>), as \u000A long as it does not contain a newline (more on that below).  However, when an \u000A argument does not make much sense to us, we show it below as the empty string, \u000A e.g., ``<v>~bv[]</v>'' rather than ``<v>~bv[arg]</v>''.</p> \u000A \u000A <code>~-[]      Print the equivalent of a dash \u000A \u000A~b[arg]   Print the argument in bold font, if available \u000A \u000A~bid[arg]   ``Begin implementation dependent'' -- Ignores argument at \u000A          terminal. \u000A \u000A~bf[]     Begin formatted text (respecting spaces and line breaks), \u000A          but in ordinary font (rather than, say, fixed-width font) \u000A          if possible \u000A \u000A~bq[]     Begin quotation (indented text, if possible) \u000A \u000A~bv[]     Begin verbatim (print in fixed-width font, respecting \u000A          spaces and line breaks) \u000A \u000A~c[arg]   Print arg as ``code'', such as in a fixed-width font \u000A \u000A~ef[]     End format; balances ~bf[] \u000A \u000A~eid[arg]   ``End implementation dependent'' -- Ignores argument at \u000A          terminal. \u000A \u000A~em[arg]  Emphasize arg, perhaps using italics \u000A \u000A~eq[]     End quotation; balances ~bq[] \u000A \u000A~ev[]     End verbatim; balances ~bv[] \u000A \u000A~i[arg]   Print arg in italics font \u000A \u000A~id[arg]   ``Implementation dependent'' -- Ignores argument at \u000A          terminal. \u000A \u000A~il[arg]  Print argument as is, but make it a link to another doc \u000A          topic (for true hypertext environments).  Note that the \u000A          link argument must match the package name for the desired \u000A          topic unless the package name is ``ACL2''.  That is, to \u000A          link to documentation on ``foo::topic'', one needs to use \u000A          the full name for ``foo::topic'', including its package \u000A          name. \u000A \u000A~ilc[arg] Same as ~il[arg], except that arg should be printed as \u000A          with ~c[arg] \u000A \u000A~l[arg]   Ordinary link to another doc topic; prints as ``See :DOC \u000A          arg'' at the terminal (but also see ~pl below, which puts \u000A          ``see'' in lower case) \u000A \u000A~nl[]     Print a newline \u000A \u000A~par[]    Paragraph mark, of no significance at the terminal \u000A          (can be safely ignored; see also notes below) \u000A \u000A~pl[arg]  Parenthetical link (borrowing from Texinfo):  same as \u000A          ~l[arg], except that ``see'' is in lower case.  This is \u000A          typically used at other than the beginning of a sentence. \u000A \u000A~sc[arg]  Print arg in (small, if possible) capital letters \u000A \u000A~st[arg]  Strongly emphasize arg, perhaps using a bold font \u000A \u000A~t[arg]   Typewriter font; similar to ~c[arg], but leaves less \u000A          doubt about the font that will be used. \u000A \u000A~terminal[arg]  Terminal only; arg is to be ignored except when \u000A          reading documentation at the terminal, using :DOC. \u000A \u000A~url[arg] Print arg as HTML hyperlink if possible (else print like <v>~c</v>) \u000A</code> \u000A \u000A <p><i>Style notes and further details</i></p> \u000A \u000A <p>It is not a good idea to put <see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde directives inside \u000A verbatim environments, <v>~bv[] ... ~ev[]</v>.</p> \u000A \u000A <p>Do not nest <see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde directives; that is, do not write</p> \u000A \u000A <code>The ~c[~il[append] function ...</code> \u000A \u000A <p>but note that the ``equivalent'' expression</p> \u000A \u000A <code>The ~ilc[append] function ...</code> \u000A \u000A <p>is fine.  The following phrase is also acceptable:</p> \u000A \u000A <code>~bf[]This is\u000A~em[formatted] text.\u000A~ef[]</code> \u000A \u000A <p>because the nesting is only conceptual, not literal.</p> \u000A \u000A <p>We recommend that for displayed text, <v>~bv[]</v> and <v>~ev[]</v> should \u000A usually each be on lines by themselves.  That way, printed text may be less \u000A encumbered with excessive blank lines.  Here is an example.</p> \u000A \u000A <code>Here is some normal text.  Now start a display:\u000A~bv[]\u000A  2 + 2 = 4\u000A~ev[]\u000AAnd here is the end of that paragraph.\u000A\u000AHere is the start of the next paragraph.</code> \u000A \u000A <p>The analogous consideration applies to <v>~bf[]</v> and <v>~ef[]</v> as well as \u000A <v>~bq[]</v> and <v>~eq[]</v>.</p> \u000A \u000A <p>You may ``quote'' <see topic=\"ACL2____CHARACTERS\">characters</see> inside the <v>arg</v> part of \u000A <v>~key[arg]</v>, by preceding them with ~.  This is, in fact, the only legal \u000A way to use a newline character or a right bracket (]) inside the argument to a \u000A <see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde directive.</p> \u000A \u000A <p>Write your <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings without hyphens.  Otherwise, you \u000A may find your text printed on paper (via TeX, for example) like this \u000A &mdash;</p> \u000A \u000A <code>Here is a hyphe- nated word.</code> \u000A \u000A <p>even if what you had in mind was:</p> \u000A \u000A <code>Here is a hyphe-\u000Anated word.</code> \u000A \u000A <p>When you want to use a dash (as opposed to a hyphen), consider using ~-[], \u000A which is intended to be interpreted as a ``dash.''  For example:</p> \u000A \u000A <code>This sentence ~-[] which is broken with dashes ~-[] is boring.</code> \u000A \u000A <p>would be written to the terminal (using <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>) by replacing \u000A <v>~-[]</v> with two hyphen <see topic=\"ACL2____CHARACTERS\">characters</see>, but would presumably be printed \u000A on paper with a dash.</p> \u000A \u000A <p>Be careful to balance the ``begin'' and ``end'' pairs, such as <v>~bv[]</v> \u000A and <v>~ev[]</v>.  Also, do not use two ``begin'' directives (<v>~bf[]</v>, \u000A <v>~bq[]</v>, or <v>~bv[]</v>) without an intervening ``end'' directive.  It is \u000A permissible (and perhaps this is not surprising) to use the <see topic=\"ACL2____DOC-STRING\">doc-string</see> \u000A part separator <v>~/</v> while between such a begin-end pair.</p> \u000A \u000A <p>Because of a bug in texinfo (as of this writing), you may wish to avoid \u000A beginning a line with (any number of spaces followed by) the <tt><see topic=\"COMMON-LISP____-\">-</see></tt> \u000A character or <v>~-[]</v>.</p> \u000A \u000A <p>The ``paragraph'' directive, <v>~par[]</v>, is rarely if ever used.  There is \u000A a low-level capability, not presently documented, that interprets two \u000A successive newlines as though they were <v>~par[]</v>.  This is useful for the \u000A HTML driver.  For further details, see the authors of ACL2.</p> \u000A \u000A <p>Emacs code is available for manipulating <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings that \u000A contain <see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde-directives (for example, for doing a \u000A reasonable job filling such <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings).  See the authors if \u000A you are interested.</p> \u000A \u000A <p>We tend to use <v>~em[arg]</v> for ``section headers,'' such as ``Style notes \u000A and further details'' above.  We tend to use <v>~st[arg]</v> for emphasis of \u000A words inside text.  This division seems to work well for our Texinfo driver. \u000A Note that <v>~st[arg]</v> causes <v>arg</v> to be printed in upper-case at the \u000A terminal (using <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>), while <v>~em[arg]</v> causes <v>arg</v> to be \u000A printed at the terminal as though <v>arg</v> were not marked for emphasis.</p> \u000A \u000A <p>Our Texinfo and HTML drivers both take advantage of capabilities for \u000A indicating which <see topic=\"ACL2____CHARACTERS\">characters</see> need to be ``escaped,'' and how.  Unless \u000A you intend to write your own driver, you probably do not need to know more \u000A about this issue; otherwise, contact the ACL2 authors.  We should probably \u000A mention, however, that Texinfo makes the following requirement: when using \u000A <v>~l[arg]</v>, where <v>arg</v> contains one of the special <see topic=\"ACL2____CHARACTERS\">characters</see> \u000A <tt><see topic=\"ACL2_____04\">@</see></tt>, <v>{</v>, or <v>}</v>, you must immediately follow this use with a \u000A period or comma.  Also, the Emacs ``info'' <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> that we \u000A generate by using our Texinfo driver has the property that in node names, \u000A <v>:</v> has been replaced by <v>|</v> (because of quirks in info); so for \u000A example, the ``<see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'' simplification command, <v>s</v>, is \u000A documented under <v>acl2-pc||s</v> rather than under <v>acl2-pc::s</v>.</p> \u000A \u000A <p>We have tried to keep this markup language fairly simple; in particular, \u000A there is no way to refer to a link by other than the actual name.  So for \u000A example, when we want to make <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> an invisible link in ``code'' \u000A font, we write the following form, which indicates that <v>:</v> should be in \u000A that font and then <tt><see topic=\"ACL2____DOC\">doc</see></tt> should both be in that font and be an invisible \u000A link.</p> \u000A \u000A <code>~c[:]~ilc[doc]</code>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>We start by showing classical induction on the natural numbers in \u000A an ACL2 setting before turning to a more general treatment of induction.</p> \u000A \u000A <p><b>Classical Induction on Natural Numbers</b>: Induction is familiar in the \u000A arithmetic setting.  Let <v>(p n)</v> denote some formula involving the variable \u000A <v>n</v> (and perhaps some other variables which we don't exhibit).  Then to \u000A prove <v>(p n)</v>, for all <v>n</v>, by classical induction on the construction \u000A of the natural numbers, prove each of the following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (zp n) (p n)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (zp n)) \u000A              (p (- n 1))) \u000A         (p n)) \u000A</code> \u000A \u000A <p>The Base Case establishes that <v>p</v> holds for <v>0</v>.  In fact, because \u000A of the definition of <tt><see topic=\"ACL2____ZP\">zp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, it \u000A establishes that <v>(p n)</v> holds when <v>n</v> is <v>0</v> and it holds when \u000A <v>n</v> is not a natural number.</p> \u000A \u000A <p>The Induction Step establishes that if <v>n</v> is a natural number other \u000A than <v>0</v>, and if <v>p</v> holds for <v>n</v>-1, then <v>p</v> holds for <v>n</v>. \u000A The hypothesis <v>(p (<see topic=\"COMMON-LISP____-\">-</see> n 1))</v> above is called the <i>induction \u000A hypothesis</i>.</p> \u000A \u000A <p>Note that if the Base Case and Induction Step are valid, then we know <v>(p\u000A n)</v>, for all <v>n</v>.  You can convince yourself of this by picking any object \u000A and asking ``how do I know <v>p</v> holds for this object?''  For example, <v>(p\u000A -7)</v>, <v>(p 'abc)</v>, and <v>(p 0)</v> are all established by the Base Case. \u000A What about <v>(p 1)</v>?  That follows from <v>(p 0)</v>, given the Induction \u000A Step.  Why?  To prove <v>(p 1)</v> using the Induction Step, you have to \u000A establish <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ZP\">zp</see> 1))</v>, which is true, and <v>(p (<see topic=\"COMMON-LISP____-\">-</see> 1 1))</v>, which is \u000A <v>(p 0)</v>, which is true by the Base Case.  So <v>(p 1)</v> is true.  Similar \u000A reasoning proves <v>(p 2)</v> from from <v>(p 1)</v>, etc.  Clearly, for every \u000A natural number other than <v>0</v> we could reason like this to show that <v>p</v> \u000A holds.  Since the Base Case handled all the objects that are not natural \u000A numbers, and handled <v>0</v>, we know <v>(p n)</v>, for all <v>n</v>.</p> \u000A \u000A <p>There is a duality between recursion and induction that ACL2 exploits.  The \u000A fact that the Base and Induction steps above are sufficient to prove <v>p</v> \u000A for all objects is related to the fact that the following recursion defines a \u000A total, terminating function:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nat-recursion (n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      n\u000A      (nat-recursion (<see topic=\"COMMON-LISP____-\">-</see> n 1))))</code> \u000A \u000A <p>When this function is admitted we have to prove that if <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> does \u000A not hold, then <v>(<see topic=\"COMMON-LISP____-\">-</see> n 1)</v> is smaller, in some sense, than <v>n</v>.  This \u000A sense of ``smaller'' is determined by some <i>measure</i> of the arguments. \u000A That measure must return an ordinal (<see topic=\"ACL2____ORDINALS\">ordinals</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>), \u000A but the most common measures return natural numbers, which are among the \u000A ordinals.  Furthermore, that measure should insure that the terms in the \u000A recursive calls are smaller than the formals, i.e., the measure of <v>(<see topic=\"COMMON-LISP____-\">-</see> n\u000A 1)</v> must be smaller than the measure of <v>n</v>, when the recursive branches \u000A are taken.  This sense of ``smaller'' must be <i>well-founded</i>: it must be \u000A impossible to have an infinitely descending chain of smaller things.  This is \u000A true of the less-than relation on the ordinals (see <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A Well-foundedness means that eventually any recursion must ``bottom out'' \u000A because things can't keep getting smaller forever.</p> \u000A \u000A <p>The recursion in <v>nat-recursion</v> <b>suggests</b> the induction shown \u000A above: the Base Case is defined by the <v>if</v> branch that does not lead to a \u000A recursive call.  The Induction Step is defined by the other branch.  The \u000A induction hypothesis is defined by what we recur on, i.e., <v>(<see topic=\"COMMON-LISP____-\">-</see> n 1)</v>.  The \u000A theorems proved when <v>nat-recursion</v> is introduced <i>justify</i> the \u000A classical induction scheme noted above.</p> \u000A \u000A <p>Every recursively defined ACL2 function suggests a legal induction and vice \u000A versa.</p> \u000A \u000A <p>Furthermore, every call of a recursively defined function on distinct \u000A variable symbols also suggests a legal induction: just take the induction \u000A suggested by the function's recursive definition after renaming the formal \u000A parameters to be the variables in the call.</p> \u000A \u000A <p>For example, it should be clear that <v>(nat-recursion a)</v> suggests a Base \u000A Case defined by <v>(<see topic=\"ACL2____ZP\">zp</see> a)</v>, and induction step defined by <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ZP\">zp</see> a))</v> \u000A and an induction hypothesis about <v>(<see topic=\"COMMON-LISP____-\">-</see> a 1)</v>.</p> \u000A \u000A <p>Note that the term <v>(fact n)</v> suggests the same classical induction on \u000A natural numbers shown above, where <v>fact</v> is defined as follows (even \u000A though we've used the formal parameter <v>k</v> below).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (k)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> k)\u000A      1\u000A      (<see topic=\"COMMON-LISP_____A2\">*</see> k (fact (<see topic=\"COMMON-LISP____-\">-</see> k 1)))))</code> \u000A \u000A <p>The induction suggested by a term like <v>(fact n)</v> is insensitive to the \u000A name of the formal parameter used in the <v>defun</v>.</p> \u000A \u000A <p>The induction suggested by a function or term is insensitive to the value \u000A returned by the function or term.</p> \u000A \u000A <p>It doesn't matter what the function returns in its ``base case'' (e.g., \u000A <v>1</v> in <v>fact</v>) or what the function ``does'' to its recursive call \u000A (e.g., multiply by <v>k</v> in the <v>defun</v> of <v>fact</v>).</p> \u000A \u000A <p>All that matters is (i) how the <v>if</v> structure breaks down the cases on \u000A <v>k</v>, (ii) which branches lead to recursion, and (iii) what arguments are \u000A passed to the recursive calls.  Those things determine (i) the case analysis \u000A of the induction scheme, (ii) which cases are Base Cases and which are \u000A Induction Steps, and (iii) what the induction hypotheses are.</p> \u000A \u000A <p>For a selection of common inductions schemes in ACL2 (e.g., on the \u000A structure of natural numbers, lists, and trees and on several variables at \u000A once, multiple base cases, multiple induction hypotheses, multiple induction \u000A steps, etc.)  <see topic='ACL2____EXAMPLE-INDUCTIONS'>check this \u000A link</see>.</p> \u000A \u000A <p>Every legal ACL2 induction corresponds to an admissible recursive function \u000A and vice versa.  Similarly, every legal ACL2 induction corresponds to a call \u000A of a recursively defined function on distinct variables.</p> \u000A \u000A <p>ACL2 chooses which induction to do by looking at the terms that occur in \u000A the conjecture.  For many elementary theorems, ACL2 chooses the right \u000A induction by itself.</p> \u000A \u000A <p>You may occasionally need to tell it what induction to do.  You do that by \u000A showing it a term that suggests the induction you want.  We'll explain how you \u000A communicate this to ACL2 later.  If you understand how recursive functions \u000A suggest inductions, then you know what you need to know to use ACL2.</p> \u000A \u000A <p>The main point of this discussion of induction is familiarize you with the \u000A basic terms: <i>Base Case</i> (of which there may be several), <i>Induction \u000A Step</i> (of which there may be several), <i>Induction Hypothesis</i> (of \u000A which there may be several in each Induction Step), <i>measure</i> and \u000A <i>well-founded relation</i> <i>justifying</i> an induction, and the induction \u000A <i>suggested</i> by a term or recursive function definition.  Furthermore, \u000A every Induction Hypothesis is always an <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see> of the conjecture \u000A being proved: each induction hypothesis is obtained from the conjecture being \u000A proved by applying a <i>substitution</i> replacing variables by terms.</p> \u000A \u000A <p>If you are reviewing the material taken for granted about logic while \u000A working your way through the introduction to the theorem prover, please use \u000A your browser's <b>Back Button</b> to return to the example proof in <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____IF_A2":[["Bdd"],"ACL2 Sources","ACL2","<p>The function <v>IF*</v> is defined to be <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, but it is used \u000A in a special way by ACL2's <see topic=\"ACL2____BDD\">bdd</see> package.</p> \u000A \u000A <p><b>Function: </b><srclink>if*</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> if* (x y z) (<see topic=\"COMMON-LISP____IF\">if</see> x y z))</code> \u000A \u000A <p>As explained elsewhere (see <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see>), ACL2's <see topic=\"ACL2____BDD\">bdd</see> \u000A algorithm gives special treatment to <see topic=\"ACL2____TERM\">term</see>s of the form <v>(<see topic=\"ACL2____IF_A2\">IF*</see> TEST TBR\u000A FBR)</v>.  In such cases, the algorithm simplifies <v>TEST</v> first, and the \u000A result of that simplification must be a constant (normally <v>t</v> or <v>nil</v>, \u000A but any non-<v>nil</v> explicit value is treated like <v>t</v> here).  Otherwise, \u000A the algorithm aborts.</p> \u000A \u000A <p>Thus, <v>IF*</v> may be used to implement a sort of conditional rewriting for \u000A ACL2's <see topic=\"ACL2____BDD\">bdd</see> package, even though this package only nominally supports \u000A unconditional rewriting.  The following contrived example should make this \u000A point clear.</p> \u000A \u000A <p>Suppose that we want to prove that <v>(<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))</v> is \u000A equal to <v>y</v>, but that we would be happy to prove this only for lists \u000A having length 4.  We can state such a theorem as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3)))\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A         y))</code> \u000A \u000A <p>If we want to prove this formula with a <v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt> hint, then we \u000A need to have appropriate rewrite rules around.  First, note that <v>LENGTH</v> \u000A is defined as follows (try <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> <tt><see topic=\"COMMON-LISP____LENGTH\">length</see></tt>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LENGTH\">length</see> x)\u000A =\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A    (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list))\u000A    (<see topic=\"ACL2____LEN\">len</see> x))</code> \u000A \u000A <p>Since <see topic=\"ACL2____BDD\">bdd</see>-based rewriting is merely very simple unconditional \u000A rewriting (see <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see>), we expect to have to prove a rule \u000A reducing <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> of a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> stringp-cons\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A         nil))</code> \u000A \u000A <p>Now we need a rule to compute the <v>LEN</v> of <v>X</v>, because the \u000A definition of <v>LEN</v> is recursive and hence not used by the <see topic=\"ACL2____BDD\">bdd</see> \u000A package.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> len-cons\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> a x))\u000A         (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"ACL2____LEN\">len</see> x))))</code> \u000A \u000A <p>We imagine this rule simplifying <v>(<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3))</v> in terms of \u000A <v>(<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____LIST\">LIST</see> X1 X2 X3))</v>, and so on, and then finally <v>(<see topic=\"ACL2____LEN\">LEN</see> nil)</v> should \u000A be computed by execution (see <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see>).</p> \u000A \u000A <p>We also need to imagine simplifying <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y)</v>, where still <v>X</v> \u000A is bound to <v>(<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3)</v>.  The following two rules suffice for this \u000A purpose (but are needed, since <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>, actually <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>, \u000A is recursive).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> append-cons\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> a x) y)\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-nil\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> nil x)\u000A         x))</code> \u000A \u000A <p>Finally, we imagine needing to simplify calls of <tt><see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see></tt>, where the \u000A first argument is a number (initially, the length of <v>(<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3)</v>, \u000A which is 4).  The second lemma below is the traditional way to accomplish that \u000A goal (when not using BDDs), by proving a conditional rewrite rule.  (The first \u000A lemma is only proved in order to assist in the proof of the second lemma.)</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> fold-constants-in-+\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep x))\u000A                (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep y)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y z)\u000A                  (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> nthcdr-add1-conditional\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ZP\">zp</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> n)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> n) x)\u000A                  (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code> \u000A \u000A <p>The problem with this rule is that its hypothesis makes it a conditional \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> rule, and conditional rewrite rules are not used by the <see topic=\"ACL2____BDD\">bdd</see> package.  (See <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see> for a discussion of ``BDD rules.'') \u000A (Note that the hypothesis cannot simply be removed; the resulting formula \u000A would be false for <v>n = -1</v> and <v>x = '(a)</v>, for example.)  We can solve \u000A this problem by using <v>IF*</v>, as follows; comments follow.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> nthcdr-add1\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n) x)\u000A         (<see topic=\"ACL2____IF_A2\">if*</see> (<see topic=\"ACL2____ZP\">zp</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> n))\u000A              x\u000A              (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code> \u000A \u000A <p>How is <v>nthcdr-add1</v> applied by the <see topic=\"ACL2____BDD\">bdd</see> package?  Suppose that \u000A the <see topic=\"ACL2____BDD\">bdd</see> computation encounters a <see topic=\"ACL2____TERM\">term</see> of the form <v>(<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> (<see topic=\"COMMON-LISP_____B2\">+</see>\u000A 1 N) X)</v>.  Then the <see topic=\"ACL2____BDD\">bdd</see> package will apply the <see topic=\"ACL2____REWRITE\">rewrite</see> rule \u000A <v>nthcdr-add1</v>.  The first thing it will do when attempting to simplify the \u000A right hand side of that rule is to attempt to simplify the term <v>(<see topic=\"ACL2____ZP\">ZP</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see>\u000A N))</v>.  If <v>N</v> is an explicit number (which is the case in the scenario we \u000A envision), this test will reduce (assuming the executable counterparts of \u000A <tt><see topic=\"ACL2____ZP\">zp</see></tt> and <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> are <see topic=\"ACL2____ENABLE\">enable</see>d) to <v>t</v> or to <v>nil</v>. \u000A In fact, the lemmas above (not including the lemma \u000A <v>nthcdr-add1-conditional</v>) suffice to prove our goal:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3)))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A              y))\u000A     :hints ((&quot;Goal&quot; :bdd (:vars nil))))</code> \u000A \u000A <p>If we execute the following form that disables the definition and \u000A executable counterpart of the function <tt><see topic=\"ACL2____ZP\">zp</see></tt></p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> zp (<see topic=\"ACL2____ZP\">zp</see>)))</code> \u000A \u000A <p>before attempting the proof of the theorem above, we can see more clearly \u000A the point of using <v>IF*</v>.  In this case, the prover makes the following \u000A report.</p> \u000A \u000A <code>ACL2 Error in ( THM ...):  Unable to resolve test of IF* for term\u000A\u000A(<see topic=\"ACL2____IF_A2\">IF*</see> (<see topic=\"ACL2____ZP\">ZP</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 N)) X (<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> N (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A\u000Aunder the bindings\u000A\u000A((X (<see topic=\"COMMON-LISP____CONS\">CONS</see> X0 (<see topic=\"COMMON-LISP____CONS\">CONS</see> X1 (<see topic=\"COMMON-LISP____CONS\">CONS</see> X2 #)))) (N '3))\u000A\u000A-- use SHOW-BDD to see a backtrace.</code> \u000A \u000A <p>If we follow the advice above, we can see rather clearly what happened. \u000A See <see topic=\"ACL2____SHOW-BDD\">show-bdd</see>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)\u000A\u000ABDD computation on Goal yielded 21 nodes.\u000A==============================\u000A\u000ABDD computation was aborted on Goal, and hence there is no\u000Afalsifying assignment that can be constructed.  Here is a backtrace\u000Aof calls, starting with the top-level call and ending with the one\u000Athat led to the abort.  See :DOC show-bdd.\u000A\u000A(<see topic=\"COMMON-LISP____LET\">LET</see> ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3)))\u000A     (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> (<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> X) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y)) Y))\u000A  alist: ((Y Y) (X3 X3) (X2 X2) (X1 X1) (X0 X0))\u000A\u000A(<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> (<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> X) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X Y))\u000A  alist: ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3)) (Y Y))\u000A\u000A(<see topic=\"ACL2____IF_A2\">IF*</see> (<see topic=\"ACL2____ZP\">ZP</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 N)) X (<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> N (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A  alist: ((X (<see topic=\"COMMON-LISP____LIST_A2\">LIST*</see> X0 X1 X2 X3 Y)) (N 3))\u000AACL2 !&gt;</code> \u000A \u000A <p>Each of these term-alist pairs led to the next, and the test of the last \u000A one, namely <v>(<see topic=\"ACL2____ZP\">ZP</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 N))</v> where <v>N</v> is bound to <v>3</v>, was not \u000A simplified to <v>t</v> or to <v>nil</v>.</p> \u000A \u000A <p>What would have happened if we had used <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> in place of <v>IF*</v> in \u000A the rule <v>nthcdr-add1</v>?  In that case, if <v>ZP</v> and its executable \u000A counterpart were disabled then we would be put into an infinite loop!  For, \u000A each time a term of the form <v>(<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> k V)</v> is encountered by the BDD \u000A package (where k is an explicit number), it will be rewritten in terms of \u000A <v>(<see topic=\"COMMON-LISP____NTHCDR\">NTHCDR</see> k-1 (<see topic=\"COMMON-LISP____CDR\">CDR</see> V))</v>.  We would prefer that if for some reason the term \u000A <v>(<see topic=\"ACL2____ZP\">ZP</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 N))</v> cannot be decided to be <v>t</v> or to be <v>nil</v>, then the \u000A BDD computation should simply abort.</p> \u000A \u000A <p>Even if there were no infinite loop, this kind of use of <v>IF*</v> is useful \u000A in order to provide feedback of the form shown above whenever the test of an \u000A <v>IF</v> term fails to simplify to <v>t</v> or to <v>nil</v>.</p>"],

 "ACL2____SUBVERSIVE-RECURSIONS":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Subtleties arise when one of the ``constrained'' functions, <v>f</v>, \u000A introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see> of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, is \u000A involved in the termination argument for a non-local recursively defined \u000A function, <v>g</v>, in that <v>encapsulate</v>.  During the processing of the \u000A encapsulated events, <v>f</v> is locally defined to be some witness function, \u000A <v>f'</v>.  Properties of <v>f'</v> are explicitly proved and exported from the \u000A encapsulate as the constraints on the undefined function <v>f</v>.  But if \u000A <v>f</v> is used in a recursive <v>g</v> defined within the encapsulate, then the \u000A termination proof for <v>g</v> may use properties of <v>f'</v> &mdash; the witness \u000A &mdash; that are not explicitly set forth in the constraints stated for \u000A <v>f</v>.</p> \u000A \u000A <p>Such recursive <v>g</v> are said be ``subversive'' because if naively treated \u000A they give rise to unsound induction schemes or (via functional instantiation) \u000A recurrence equations that are impossible to satisfy.  We illustrate what could \u000A go wrong below.</p> \u000A \u000A <p>Subversive recursions are not banned outright.  Instead, they are treated \u000A as part of the constraint.  That is, in the case above, the definitional \u000A equation for <v>g</v> becomes one of the constraints on <v>f</v>.  This is \u000A generally a severe restriction on future functional instantiations of <v>f</v>. \u000A In addition, ACL2 removes from its knowledge of <v>g</v> any suggestions about \u000A legal inductions to ``unwind'' its recursion.</p> \u000A \u000A <p>What should you do?  Often, the simplest response is to move the offending \u000A recursive definition, e.g., <v>g</v>, out of the encapsulate.  That is, \u000A introduce <v>f</v> by constraint and then define <v>g</v> as an ``independent'' \u000A event.  You may need to constrain ``additional'' properties of <v>f</v> in order \u000A to admit <v>g</v>, e.g., constrain it to reduce some ordinal measure.  However, \u000A by separating the introduction of <v>f</v> from the admission of <v>g</v> you will \u000A clearly identify the necessary constraints on <v>f</v>, functional \u000A instantiations of <v>f</v> will be simpler, and <v>g</v> will be a useful function \u000A which suggests inductions to the theorem prover.</p> \u000A \u000A <p>Note that the functions introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see> should not even \u000A occur ancestrally in the termination proofs for non-local recursive functions \u000A in the encapsulate.  That is, the constrained functions of an encapsulate \u000A should not be reachable in the dependency graph of the functions used in the \u000A termination arguments of recursive functions in encapsulate.  If they are \u000A reachable, their definitions become part of the constraints.</p> \u000A \u000A <p>The following event illustrates the problem posed by subversive \u000A recursions.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((f *) =&gt; *))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____NOT\">not</see> (g (f x))) t)))</code> \u000A \u000A <p>Suppose, contrary to how ACL2 works, that the encapsulate above were to \u000A introduce no constraints on <v>f</v> on the bogus grounds that the only use of \u000A <v>f</v> in the encapsulate is in an admissible function.  We discuss the \u000A plausibility of this bogus argument in a moment.</p> \u000A \u000A <p>Then it would be possible to prove the theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> f-not-identity\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f '(a . b)) '(a . b)))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use (:instance g (x '(a . b))))))</code> \u000A \u000A <p>simply by observing that if <v>(f '(a . b))</v> were <v>'(a . b)</v>, then \u000A <v>(g '(a . b))</v> would be <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (g '(a . b)))</v>, which is impossible.</p> \u000A \u000A <p>But then we could functionally instantiate <v>f-not-identity</v>, replacing \u000A <v>f</v> by the identity function, to prove <v>nil</v>!  This is bad.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A  nil\u000A  :rule-classes nil\u000A  :hints\u000A  ((&quot;Goal&quot; :use (:functional-instance f-not-identity (f identity)))))</code> \u000A \u000A <p>This sequence of events was legal in versions of ACL2 prior to Version 1.5. \u000A When we realized the problem we took steps to make it illegal.  However, our \u000A steps were insufficient and it was possible to sneak in a subversive function \u000A (via mutual recursion) as late as Version 2.3.</p> \u000A \u000A <p>We now turn to the plausibility of the bogus argument above.  Why might one \u000A even be tempted to think that the definition of <v>g</v> above poses no \u000A constraint on <v>f</v>?  Here is a very similar encapsulate.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((f *) =&gt; *))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> map (x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> (f x) (map (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A      nil)))</code> \u000A \u000A <p>Here <v>map</v> plays the role of <v>g</v> above.  Like <v>g</v>, <v>map</v> calls \u000A the constrained function <v>f</v>.  But <v>map</v> truly does not constrain \u000A <v>f</v>.  In particular, the definition of <v>map</v> could be moved ``out'' of \u000A the encapsulate so that <v>map</v> is introduced afterwards.  The difference \u000A between <v>map</v> and <v>g</v> is that the constrained function plays no role in \u000A the termination argument for the one but does for the other.</p> \u000A \u000A <p>As a ``user-friendly'' gesture, ACL2 implicitly moves <v>map</v>-like \u000A functions out of encapsulations; logically speaking, they are introduced after \u000A the encapsulation.  This simplifies the constraint.  When the constraint \u000A cannot be thus simplfied the user is advised, via the ``infected'' warning, to \u000A phrase the encapsulation in the simplest way possible.</p> \u000A \u000A <p>The lingering bug between Versions 1.5 and 2.3 mentioned above was due to \u000A our failure to detect the <v>g</v>-like nature of some functions when they were \u000A defined in mutually recursively cliques with other functions.  The singly \u000A recursive case was recognized.  The bug arose because our detection \u000A ``algorithm'' was based on the ``suggested inductions'' left behind by \u000A successful definitions.  We failed to recall that mutually-recursive \u000A definitions do not, as of this writing, make any suggestions about inductions \u000A and so did not leave any traces of their subversive natures.</p> \u000A \u000A <p>We conclude by elaborating on the criterion ``involved in the termination \u000A argument'' mentioned at the outset.  Suppose that function <v>f</v> is \u000A recursively defined in an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, where the body of <v>f</v> has no \u000A ``ancestor'' among the functions introduced in the signature of that <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, i.e.: the body contains no call of a signature function, no call \u000A of a function whose body calls a signature function, and so on.  Then ACL2 \u000A treats <v>f</v> as though it is defined in front of that <v>encapsulate</v> form; \u000A so <v>f</v> is not <v>constrained</v> by the encapsulate, and its definition is \u000A hence certainly not subversive in the sense discussed above.  But suppose to \u000A the contrary that some function introduced in the signature is an ancestor of \u000A the body of <v>f</v>.  Then the definition of <v>f</v> is subversive if moreover, \u000A its termination proof obligation has an ancestor among the signature \u000A functions.  Now, that proof obligation involves terms from the first argument \u000A of selected calls of <v>IF</v>, as well as recursive calls; for a detailed \u000A discussion, see <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.  The important point here is that for \u000A the recursive calls, only the arguments in ``measured'' positions are relevant \u000A to the termination proof obligation.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((h *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (n) n))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (i n)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> i)\u000A       n\u000A     (f (<see topic=\"COMMON-LISP____1-\">1-</see> i) (h n)))))</code> \u000A \u000A <p>ACL2 heuristically picks the measure <v>(<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> i)</v> for the definition \u000A of <v>f</v>; thus, <v>i</v> is the only ``measured formal'' of <v>f</v>.  Since \u000A <v>i</v> is the first formal of <v>f</v>, then for the recursive call of <v>f</v>, \u000A only the first argument contributes to the termination proof obligation: in \u000A this case, <v>(<see topic=\"COMMON-LISP____1-\">1-</see> i)</v> but not <v>(h n)</v>.  Thus, even though <v>h</v> is a \u000A signature function, the definition of <v>f</v> is not considered to be \u000A subversive; an induction scheme is thus stored for <v>f</v>.  (This restriction \u000A to measured formal positions of recursive calls, for determining subversive \u000A definitions, is new in Version_3.5 of ACL2.)</p>"],

 "ACL2____RELEASE-NOTES":[["About-ACL2"],"ACL2 Sources","ACL2","<p>This section of the online <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> contains notes on \u000A the changes that distinguish successive released versions of ACL2.</p> \u000A \u000A <p>The current version of ACL2 is the value of the constant <v>(<see topic=\"ACL2_____04\">@</see>\u000A acl2-version)</v>.</p>"],

 "ACL2____PROVISIONAL-CERTIFICATION":[["Books-certification"],"ACL2 Sources","ACL2","<p>Provisional certification is a process that can increase \u000A parallelism at the system level, typically using `make', when certifying a \u000A collection of <see topic=\"ACL2____BOOKS\">books</see>.  We got this idea from Jared Davis, who developed \u000A rudimentary provisional certification schemes first at Rockwell Collins and \u000A later for his `Milawa' project.  Our design has also benefited from \u000A conversations with Sol Swords.</p> \u000A \u000A <p>To invoke provisional certification, see <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see>.  For \u000A example, you could issue the following command.</p> \u000A \u000A <code>ACL2_PCERT=t cert.pl -j 4 `find . -name '*.lisp'`</code> \u000A \u000A <p>Alternatively, see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see> for a discussion of \u000A classic ACL2 `make'-based certification (which may disappear in a future ACL2 \u000A release); here we extend those instructions to show how to use provisional \u000A certification.  (Also, you may wish to look at community books file \u000A <v>books/system/pcert/Makefile</v> for an example.)  We begin by describing a \u000A few ways to do that.  A simple way is to add the line `<v>ACL2_PCERT=t</v>' to a \u000A `make' command that you use for book certification, for example as \u000A follows.</p> \u000A \u000A <code>make -j 4 ACL2_PCERT=t</code> \u000A \u000A <p>The following works too, in a bash shell.</p> \u000A \u000A <code>(export ACL2_PCERT=t ; time make -j 4)</code> \u000A \u000A <p>Alternatively, add the line</p> \u000A \u000A <code>ACL2_PCERT ?= t</code> \u000A \u000A <p>to the <v>Makefile</v> residing in the directory, placed above the line that \u000A specifies the `<v>include</v>' of file <v>Makefile-generic</v>.  A successful \u000A `make' will result in creating the desired <see topic=\"ACL2____CERTIFICATE\">certificate</see> (<v>.cert</v>) \u000A files.</p> \u000A \u000A <p>Warning: If you put the line ``<v>ACL2_PCERT ?= t</v>'' below the include of \u000A <v>Makefile-generic</v>, it might have no effect.  For example, try editing \u000A community books file <v>books/system/pcert/Makefile</v> by moving the line \u000A ``<v>ACL2_PCERT ?= t</v>'' to the bottom of the file, and watch ``<v>make\u000A top.cert</v>'' fail to invoke provisional certification.</p> \u000A \u000A <p>The description above may be sufficient for you to use provisional \u000A certification.  We provide additional documentation below for the reader who \u000A wants to understand more about this process, for example when not using \u000A `make'.  Below we assume prior familiarity with <see topic=\"ACL2____BOOKS\">books</see>, in particular \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  The remainder of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is divided into sections: Summary, Correctness Claim and \u000A Issues, Combining Pcertify and Convert into Pcertify+, and Further \u000A Information.</p> \u000A \u000A <p><b>Summary</b></p> \u000A \u000A <p>Recall that certification of a book, <v>bk</v>, produces a <see topic=\"ACL2____CERTIFICATE\">certificate</see> \u000A file <v>bk.cert</v>.  The provisional certification process produces this file \u000A as well, but as the last of the following three steps.  All of these steps are \u000A carried out by calls of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> using its <v>:pcert</v> keyword \u000A argument.  We typically call these steps ``procedures'', to distinguish them \u000A from the steps of an individual call of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A <blockquote><p>o The ``Pcertify'' procedure (sometimes called the ``Create'' \u000A procedure) is invoked by calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> with keyword argument \u000A <v>:pcert :create</v>.  It produces a file <v>bk.pcert0</v>, sometimes called the \u000A ``<v>.pcert0</v>'' file (pronounced ``dot pee cert zero'').  Proofs are skipped \u000A during this procedure, which can be viewed as an elaborate syntax check, \u000A augmented by compilation if specified (as it is by default).</p> \u000A \u000A <p>o The ``Convert'' procedure is invoked by calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> with \u000A keyword argument <v>:pcert :convert</v>.  It creates file <v>bk.pcert1</v> from \u000A <v>bk.pcert0</v>, by doing proofs for all events in <v>bk.lisp</v>.  Note that the \u000A third argument (the `<v>compile-flg</v>' argument) is ignored by such a call of \u000A <v>certify-book</v> unless its value is <v>:all</v> (either explicitly or by way \u000A of environment variable <v>ACL2_COMPILE_FLG</v>).  A subtlety is that if there \u000A is a compiled file at least as recent as the corresponding <v>.pcert0</v> file, \u000A then that compiled file's write date will be updated to the current time at \u000A the end of this procedure.  The usual <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see> check at \u000A the end of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is omitted for the Convert procedure, since it \u000A was performed towards the end of the Create procedure.</p> \u000A \u000A <p>o The ``Complete'' procedure is invoked by calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A with keyword argument <v>:pcert :complete</v>.  It checks that every included \u000A book (including every one that is <tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly included) has a <v>.cert</v> \u000A file that is at least as recent as the corresponding book.  The effect is to \u000A move <v>bk.pcert1</v> to <v>bk.cert</v>.  Note that all arguments of \u000A <v>certify-book</v> other than the <v>:pcert</v> argument are ignored for this \u000A procedure, other than for some trivial argument checking.</p></blockquote> \u000A \u000A <p>You can combine the Pcertify and Convert procedures into a single \u000A procedure, Pcertify+, which may be useful for books that contain expensive \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> but do few proofs.  We defer discussion of \u000A that feature to the section below, ``Combining Pcertify and Convert into \u000A Pcertify+''.</p> \u000A \u000A <p>The main idea of provisional certification is to break sequential \u000A dependencies caused by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, that is, so that a book's proofs \u000A are carried out even when it includes books (sometimes called ``sub-books'') \u000A that have not themselves been fully certified.  For example, suppose that a \u000A proof development consists of books <v>A.lisp</v>, <v>B.lisp</v>, and <v>C.lisp</v>, \u000A where file <v>A.lisp</v> contains the form <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;B&quot;)</v> and file \u000A <v>B.lisp</v> contains the form <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;C&quot;)</v>.  Normally one would \u000A first certify <v>C</v>, then <v>B</v>, and finally, <v>A</v>.  However, the \u000A provisional certification process can speed up the process on a multi-core \u000A machine, as follows: the Pcertify (pronounced ``pee certify'') procedure \u000A respects this order but (one hopes) is fast since proofs are skipped; the \u000A Convert procedure essentially completes the certification in parallel by doing \u000A proofs and creating <v>.pcert1</v> files based on <v>.pcert0</v> files; and the \u000A Complete procedure respects book order when quickly renaming <v>.pcert1</v> \u000A files to <v>.cert</v> files.  In our example, the steps would be as follows, but \u000A note that we need not finish all Pcertify steps before starting some Convert \u000A steps, nor need we finish all Convert steps before starting some Complete \u000A steps, as explained further below.</p> \u000A \u000A <blockquote><p>o Pcertify books \"C\", \"B\",and then \"A\", sequentially, \u000A skipping proofs but doing compilation.</p> \u000A \u000A <p>o Do proofs in parallel for the books using the Convert procedure, where \u000A each book relies on the existence of its own <v>.pcert0</v> file as well as a \u000A <v>.cert</v>, <v>.pcert0</v>, or <v>.pcert1</v> file for each of its included \u000A sub-books.  Write out a <v>.pcert1</v> file for the book when the proof \u000A succeeds.</p> \u000A \u000A <p>o Rename the <v>.pcert1</v> file to a corresponding <v>.cert</v> file when a \u000A <v>.cert</v> file exists and is up-to-date for each included \u000A book.</p></blockquote> \u000A \u000A <p>The Convert step can begin for <v>bk.lisp</v> any time after <v>bk.pcert0</v> \u000A is built.  The Complete step can begin for this book any time after \u000A <v>bk.pcert1</v> and every <v>sub-bk.cert</v> are built, for <v>sub-bk</v> a \u000A sub-book of <v>bk</v>.</p> \u000A \u000A <p>The new procedures &mdash; Pcertify, Convert, and Complete &mdash; are \u000A invoked by supplying a value for the keyword argument <v>:pcert</v> of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, namely <v>:create</v>, <v>:convert</v>, or <v>:complete</v>, \u000A respectively.  Typically, and by default, the <v>compile-flg</v> argument of \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is <v>t</v> for the Pcertify procedure, so that <see topic=\"ACL2____COMPILATION\">compilation</see> can take full advantage of parallelism.  This argument is treated \u000A as <v>nil</v> for the other procedures except when its value is <v>:all</v> in the \u000A Convert procedure, as mentioned above.</p> \u000A \u000A <p>For those who use <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, we note that expansion is done in the \u000A Pcertify procedure; the later steps use the expansion resulting from that \u000A procedure.  The reason is that although a call of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> is \u000A similar to a macro call, a difference is that the expansion of a \u000A <v>make-event</v> form can depend on the <see topic=\"ACL2____STATE\">state</see>.  Therefore, we insist on \u000A doing such an expansion only once, so that all books involved agree on the \u000A expansion.  We say more about <v>make-event</v> below.</p> \u000A \u000A <p><b>Correctness Claim and Issues</b></p> \u000A \u000A <p>The Basic Claim for certification is the same whether or not the \u000A provisional certification process is employed: all books should be certified \u000A from scratch, with no files written to the directories of the books except by \u000A ACL2.  Moreover, no trust tags should be used (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), or else it \u000A is the responsibility of the user to investigate every occurrence of ``<v>TTAG\u000A NOTE</v>'' that is printed to standard output.</p> \u000A \u000A <p>But common practice is to certify a set of books in stages: certify a few \u000A books, fix some books, re-certify changed books, certify some more books, and \u000A so on.  In practice, we expect this process to be sound even though it does \u000A not meet the preconditions for the Basic Claim above.  In particular, we \u000A expect that the use of checksums in <see topic=\"ACL2____CERTIFICATE\">certificate</see>s will make it \u000A exceedingly unlikely that a book is still treated as certified after any \u000A events in the book or any sub-book, or any <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s \u000A of the book or any sub-book, have been modified.</p> \u000A \u000A <p>Provisional certification makes it a bit easier for a determined user to \u000A subvert correctness.  For example, the Complete procedure only checks write \u000A dates to ensure that each sub-book's <v>.cert</v> file is no older than the \u000A corresponding <v>.lisp</v> file, but it does not look inside <v>.cert</v> files of \u000A sub-books; in particular it does not look at their checksum information.  Of \u000A course, the automatic dependency analysis provided by classic ACL2 \u000A `make'-based certification avoids accidental problems of this sort.  And, \u000A checksum information will indeed be applied at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> time, at \u000A least for sub-books included non-<see topic=\"ACL2____LOCAL\">local</see>ly.</p> \u000A \u000A <p>In short: while we believe that the provisional certification process can \u000A be trusted, we suggest that for maximum trust, it is best for all books in a \u000A project to be certified from scratch without the provisional certification \u000A process.</p> \u000A \u000A <p><b>Combining Pcertify and Convert into Pcertify+</b></p> \u000A \u000A <p>You can combine the Pcertify and Convert procedure into a single procedure, \u000A Pcertify+, which may be useful for books that contain expensive <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> but do few proofs.  If you are using `make' to do \u000A provisional certification as described above, just set `make' variable \u000A <v>ACL2_BOOKS_PCERT_ARG_T</v> to the list of books for which you want the \u000A Pcertify+ procedure performed instead of separate Pcertify and Convert \u000A procedures.  Either of two common methods may be used to set this variable, as \u000A illustrated below for the case that books <v>sub.lisp</v> and <v>mid.lisp</v> are \u000A the ones on which you want Pcertify+ performed.  One method is to add the \u000A following to your directory's Makefile, above the <v>include</v> of \u000A <v>Makefile-generic</v>.</p> \u000A \u000A <code>ACL2_BOOKS_PCERT_ARG_T = sub mid</code> \u000A \u000A <p>Alternatively, you can specify the desired books on the command line, for \u000A example as follows.</p> \u000A \u000A <code>make -j 4 ACL2_BOOKS_PCERT_ARG_T='sub mid'</code> \u000A \u000A <p>Note that the books are given without their <v>.lisp</v> extensions.</p> \u000A \u000A <p>At the ACL2 level, the Pcertify+ procedure is performed when the value \u000A <v>t</v> is supplied to the <v>:pcert</v> keyword argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  Thus, <v>:pcert t</v> can be thought of as a combination of \u000A <v>:pcert :create</v> and <v>:pcert :convert</v>.  However, what ACL2 actually \u000A does is to perform the Pcertify step without skipping proofs, and at the end \u000A of the <v>certify-book</v> run, it writes out both the <v>.pcert0</v> and \u000A <v>.pcert1</v> file, with essentially the same contents.  (We say \u000A ``essentially'' because the implementation writes <v>:PCERT-INFO :PROVED</v> to \u000A the end of the <v>.pcert0</v> file, but not to the <v>.pcert1</v> file.)</p> \u000A \u000A <p><b>Further Information</b></p> \u000A \u000A <p>Some errors during provisional certification cannot be readily solved.  For \u000A example, if there are circular directory dependencies (for example, some book \u000A in directory D1 includes some book in directory D2 and vice-versa), then \u000A classic ACL2 `make'-based certification will quite possibly fail.  For another \u000A example, perhaps your directory's Makefile is awkward to convert to one with \u000A suitable dependencies.  When no fix is at hand, it might be best simply to \u000A avoid provisional certification.  If you are using classic ACL2 `make'-based \u000A certification, you can simply add the following line to your directory's \u000A <v>Makefile</v>, or use ``<v>ACL2_PCERT= </v>'' on the `make' command line, to \u000A avoid provisional certification.</p> \u000A \u000A <code>override ACL2_PCERT =</code> \u000A \u000A <p>We invite anyone who has troubleshooting tips to contact the ACL2 \u000A developers with suggestions for adding such tips to this section.</p> \u000A \u000A <p>Our next remark is relevant only to users of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, and \u000A concerns the interaction of that utility with state global <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt>.  Normally, the global value of <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is \u000A unchanged during <v>make-event</v> expansion, except that it is bound to \u000A <v>nil</v> when the <v>make-event</v> form has a non-<v>nil</v> \u000A <v>:check-expansion</v> argument.  But during the Pcertify procedure (not the \u000A Pcertify+ procedure), <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is always bound to <v>nil</v> at \u000A the start of <v>make-event</v> expansion.  To see why, consider for example the \u000A community book <v>books/make-event/proof-by-arith.lisp</v>.  This book \u000A introduces a macro, <v>proof-by-arith</v>, that expands to a call of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  This <v>make-event</v> form expands by trying to prove a given \u000A theorem using a succession of included arithmetic books, until the proof \u000A succeeds.  Now proofs are skipped during the Pcertify procedure, and if proofs \u000A were also skipped during <v>make-event</v> expansion within that procedure, the \u000A first arithmetic book's <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form would always be saved \u000A because the theorem's proof ``succeeded'' (as it was skipped!).  Of course, \u000A the theorem's proof could then easily fail during the Convert step.  If you \u000A really want to inhibit proofs during <v>make-event</v> expansion in the Pcertify \u000A step, consider using a form such as the following: <v>(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> nil)) ...)</v>.</p> \u000A \u000A <p>Finally, we describe what it means for there to be a valid <see topic=\"ACL2____CERTIFICATE\">certificate</see> file for including a certified book.  Normally, this is a file \u000A with extension <v>.cert</v>.  However, if that <v>.cert</v> file does not exist, \u000A then ACL2 looks for a <v>.pcert0</v> file instead; and if that also does not \u000A exist, it looks for a <v>.pcert1</v> file.  (To see why does the <v>.pcert0</v> \u000A file take priority over the <v>.pcert1</v> file, note that the Convert procedure \u000A copies a <v>.pcert0</v> file to a <v>.pcert1</v> file, so both might exist &mdash; \u000A but the <v>.pcert1</v> file might be incomplete if copying is in progress.) \u000A Once the candidate certificate file is thus selected, it must be valid in \u000A order for the book to be considered certified (see <see topic=\"ACL2____CERTIFICATE\">certificate</see>).  For \u000A the certificate file as chosen above, then in order for a compiled file to be \u000A loaded, it must be at least as recent as that certificate file.</p> \u000A \u000A <p>Again, as discussed above, a <v>.pcert0</v> or <v>.pcert1</v> file may serve as \u000A a valid certificate file when the <v>.cert</v> file is missing.  But when that \u000A happens, a warning may be printed that a ``provisionally certified'' book has \u000A been included.  No such warning occurs if environment variable <v>ACL2_PCERT</v> \u000A has a non-empty value, or if that warning is explicitly inhibited (see <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> and see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>).</p>"],

 "ACL2____OOPS":[["History"],"ACL2 Sources","ACL2","<p>The keyword <see topic=\"ACL2____COMMAND\">command</see> <v>:oops</v> will undo the most recent \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> (or <v>:u</v>, which we here consider just another \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>).  A second <v>:oops</v> will undo the next most recent \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>, a third will undo the <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> before that one, \u000A and a fourth <v>:oops</v> will return the logical <see topic=\"ACL2____WORLD\">world</see> to its \u000A configuration before the first <v>:oops</v>.</p> \u000A \u000A <p>Consider the logical world (see <see topic=\"ACL2____WORLD\">world</see>) that represents the current \u000A extension of the logic and ACL2's rules for dealing with it.  The <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> and <v>:u</v> <see topic=\"ACL2____COMMAND\">command</see>s ``roll back'' to some previous <see topic=\"ACL2____WORLD\">world</see> \u000A (see <see topic=\"ACL2____UBT\">ubt</see>).  Sometimes these <see topic=\"ACL2____COMMAND\">command</see>s are used to inadvertently \u000A undo useful work and user's wish they could ``undo the last undo.''  That is \u000A the function provided by <v>:oops</v>.</p> \u000A \u000A <p><v>:Oops</v> is best described in terms of an implementation.  Imagine a ring \u000A of four <see topic=\"ACL2____WORLD\">world</see>s and a marker (<v>*</v>) indicating the current ACL2 <see topic=\"ACL2____WORLD\">world</see>:</p> \u000A \u000A <code>      *\u000A    w0\u000A  /    \\\u000Aw3      w1\u000A  \\    /\u000A    w2</code> \u000A \u000A <p>This is called the ``kill ring'' and it is maintained as follows.  When you \u000A execute an event the current <see topic=\"ACL2____WORLD\">world</see> is extended and the kill ring is not \u000A otherwise affected.  When you execute <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> or <v>:u</v>, the \u000A current <see topic=\"ACL2____WORLD\">world</see> marker is moved one step counterclockwise and that <see topic=\"ACL2____WORLD\">world</see> in the ring is replaced by the result, say <v>w0'</v>, of the \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> or <v>:u</v>.</p> \u000A \u000A <code>     w0\u000A   /    \\\u000A*w0'     w1\u000A   \\    /\u000A     w2</code> \u000A \u000A <p>If you were to execute <see topic=\"ACL2____EVENTS\">events</see> at this point, <v>w0'</v> would be \u000A extended and no other changes would occur in the kill ring.</p> \u000A \u000A <p>When you execute <v>:oops</v>, the marker is moved one step clockwise.  Thus \u000A the kill ring becomes</p> \u000A \u000A <code>      *\u000A    w0\u000A  /    \\\u000Aw0'     w1\u000A  \\    /\u000A    w2</code> \u000A \u000A <p>and the current ACL2 <see topic=\"ACL2____WORLD\">world</see> is <v>w0</v> once again.  That is, \u000A <v>:oops</v> ``undoes'' the <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> that produced <v>w0'</v> from \u000A <v>w0</v>.  Similarly, a second <v>:oops</v> will move the marker to <v>w1</v>, \u000A undoing the undo that produced <v>w0</v> from <v>w1</v>.  A third <v>:oops</v> makes \u000A <v>w2</v> the current <see topic=\"ACL2____WORLD\">world</see>.  Note however that a fourth <v>:oops</v> \u000A restores us to the configuration previously displayed above in which <v>w0'</v> \u000A has the marker.</p> \u000A \u000A <p>In general, the kill ring contains the current <see topic=\"ACL2____WORLD\">world</see> and the three \u000A most recent <see topic=\"ACL2____WORLD\">world</see>s in which a <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> or <v>:u</v> were \u000A done.</p> \u000A \u000A <p>While <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> may appear to discard the information in the <see topic=\"ACL2____EVENTS\">events</see> undone, we can see that the <see topic=\"ACL2____WORLD\">world</see> in which the <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> occurred is still available.  No information has been lost about that \u000A <see topic=\"ACL2____WORLD\">world</see>.  But <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> does discard information!  <v>:</v><tt><see topic=\"ACL2____UBT\">Ubt</see></tt> discards the information necessary to recover from the third most recent \u000A <tt><see topic=\"ACL2____UBT\">ubt</see></tt>!  An <v>:oops</v>, on the other hand, discards no information, it \u000A just selects the next available <see topic=\"ACL2____WORLD\">world</see> on the kill ring and doing enough \u000A <v>:oops</v>es will return you to your starting point.</p> \u000A \u000A <p>We can put this another way.  You can freely type <v>:oops</v> and inspect \u000A the <see topic=\"ACL2____WORLD\">world</see> that you thus obtain with <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>, <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt>, \u000A and other <see topic=\"ACL2____HISTORY\">history</see> <see topic=\"ACL2____COMMAND\">command</see>s.  You can repeat this as often as you \u000A wish without risking the permanent loss of any information.  But you must be \u000A more careful typing <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> or <v>:u</v>.  While <v>:oops</v> makes \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> seem ``safe'' because the most recent <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> can \u000A always be undone, information is lost when you execute <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>.</p> \u000A \u000A <p>We note that <v>:ubt</v> and <v>:u</v> may remove compiled definitions (but \u000A note that in some Lisps, including CCL (OpenMCL) and SBCL, functions are \u000A always compiled).  When the original world is restored using <v>:oops</v>, \u000A restored functions will not generally be compiled (except for Lisps as above), \u000A though the user can remedy this situation; see <see topic=\"ACL2____COMP\">comp</see>.</p> \u000A \u000A <p>Finally, we note that our implementation of <v>oops</v> can use a significant \u000A amount of memory, because of the saving of old logical <see topic=\"ACL2____WORLD\">world</see>s.  Most \u000A users are unlikely to experience a memory problem, but if you do, then you may \u000A want to disable <v>oops</v> by evaluting <v>(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> 0 state)</v>; see \u000A <see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see>.</p>"],

 "ACL2____LOCAL-INCOMPATIBILITY":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Sometimes a ``<tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibility'' is reported while \u000A attempting to embed some <see topic=\"ACL2____EVENTS\">events</see>, as in an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  This is generally due to the use of a locally defined name in \u000A a non-local event or the failure to make a witnessing definition <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____LOCAL\">Local</see></tt> incompatibilities may be detected while trying to execute the \u000A strictly non-local <see topic=\"ACL2____EVENTS\">events</see> of an embedding.  For example, <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> operates by first executing all the <see topic=\"ACL2____EVENTS\">events</see> (<tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A and non-local) with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> <v>nil</v>, to confirm that they are \u000A all admissible.  Then it attempts merely to assume the non-local ones to \u000A create the desired theory, by executing the <see topic=\"ACL2____EVENTS\">events</see> with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> set to <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  Similarly, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> assumes the non-local ones, with the understanding that a \u000A previously successful <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> has performed the admissiblity \u000A check.</p> \u000A \u000A <p>How can a sequence of <see topic=\"ACL2____EVENTS\">events</see> admitted with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> \u000A <v>nil</v> fail when <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>?  The \u000A key observation is that in the latter case only the non-local <see topic=\"ACL2____EVENTS\">events</see> \u000A are processed.  The <tt><see topic=\"ACL2____LOCAL\">local</see></tt> ones are skipped and so the non-local ones \u000A must not depend upon them.</p> \u000A \u000A <p>Two typical mistakes are suggested by the detection of a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A incompatibility: (1) a locally defined function or macro was used in a \u000A non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event (and, in the case of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, was not \u000A included among the <see topic=\"ACL2____SIGNATURE\">signature</see>s) and (2) the witnessing definition of a \u000A function that was included among the <see topic=\"ACL2____SIGNATURE\">signature</see>s of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> was not made <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.</p> \u000A \u000A <p>An example of mistake (1) would be to include among your <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>d <see topic=\"ACL2____EVENTS\">events</see> both <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn ...))</v> and <v>(<see topic=\"ACL2____DEFTHM\">defthm</see>\u000A lemma (<see topic=\"ACL2____IMPLIES\">implies</see> (fn ...) ...))</v>.  Observe that <v>fn</v> is defined locally but \u000A a formula involving <v>fn</v> is defined non-locally.  In this case, either the \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> should be made <tt><see topic=\"ACL2____LOCAL\">local</see></tt> or the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> should be \u000A made non-local.</p> \u000A \u000A <p>An example of mistake (2) would be to include <v>(fn (x) t)</v> among your \u000A <see topic=\"ACL2____SIGNATURE\">signature</see>s and then to write <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x) ...)</v> in your <see topic=\"ACL2____EVENTS\">events</see>, instead of <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn ...))</v>.</p> \u000A \u000A <p>One subtle aspect of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is that if you constrain any \u000A member of a mutually recursive clique you must define the entire clique \u000A locally and then you must constrain those members of it you want axiomatized \u000A non-locally.</p> \u000A \u000A <p>Errors due to <tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibility should never occur in the \u000A assumption of a fully certified book.  Certification ensures against it. \u000A Therefore, if <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> reports an incompatibility, we assert that \u000A earlier in the processing of the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> a warning was printed \u000A advising you that some book was uncertified.  If this is not the case &mdash; \u000A if <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> reports an incompatibility and there has been no prior \u000A warning about lack of certification &mdash; please report it to us.</p> \u000A \u000A <p>When a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibility is detected, we roll-back to the <see topic=\"ACL2____WORLD\">world</see> in which we started the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>. \u000A That is, we discard the intermediate <see topic=\"ACL2____WORLD\">world</see> created by trying to process \u000A the <see topic=\"ACL2____EVENTS\">events</see> skipping proofs.  This is clean, but we realize it is very \u000A frustrating because the entire sequence of <see topic=\"ACL2____EVENTS\">events</see> must be processed \u000A from scratch.  Assuming that the embedded <see topic=\"ACL2____EVENTS\">events</see> were, once upon a \u000A time, processed as top-level <see topic=\"ACL2____COMMAND\">command</see>s (after all, at some point you \u000A managed to create this sequence of <see topic=\"ACL2____COMMAND\">command</see>s so that the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A and non-local ones together could survive a pass in which proofs were done), \u000A it stands to reason that we could define a predicate that would determine \u000A then, before you attempted to embed them, if <tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibilities \u000A exist.  We hope to do that, eventually.</p> \u000A \u000A <p>We conclude with a subtle example of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> incompatibility.  The \u000A problem is that in order for <v>foo-type-prescription</v> to be admitted using \u000A the specified <v>:typed-term</v> <v>(foo x)</v>, the conclusion <v>(my-natp (foo\u000A x))</v> depends on <v>my-natp</v> being a <see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see>.  This is \u000A fine on the first pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, during which lemma \u000A <v>my-natp-cr</v> is admitted.  But <v>my-natp-cr</v> is skipped on the second \u000A pass because it is marked <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, and this causes \u000A <v>foo-type-prescription</v> to fail on the second pass.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-natp (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"ACL2____NFIX\">nfix</see> x))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> my-natp-cr\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (my-natp x)\u000A                 (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                      (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x)))\u000A          :rule-classes :compound-recognizer))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-type-prescription\u000A   (my-natp (foo x))\u000A   :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> foo)))\u000A   :rule-classes ((:type-prescription :typed-term (foo x)))))</code>"],

 "ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Rewrite rules make ACL2 replace one term by another.  This is done \u000A by the rewriter, which is part of ACL2's simplifier.  The rewriter sweeps \u000A through the goal formula trying all the rewrite rules it knows.  Here's an \u000A example.  Just pretend that you have made a rewrite rule from the formula \u000A below.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> i)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"ACL2____LEN\">len</see> a)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (put i v (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A                (<see topic=\"COMMON-LISP____APPEND\">append</see> (put i v a) b)))</code> \u000A \u000A <p>Then every time the rewriter sees a target term that <i>matches</i></p> \u000A \u000A <code>(put i v (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))</code> \u000A \u000A <p>it considers the rule, with the variables <v>i</v>, <v>v</v>, <v>a</v>, and \u000A <v>b</v> of the rule <i>bound</i> to whatever terms matched them in the target, \u000A say the terms <i>i</i>, <i>v</i>, <i>a</i>, and <i>b</i>.  To consider the \u000A rule, the rewriter first tries to establish (``relieve'') the hypotheses.  In \u000A particular, it rewrites:</p> \u000A \u000A <code>(natp <i>i</i>)           ; <i>hyp 1</i> \u000A</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(&lt; <i>i</i> (len <i>a</i>)). ; <i>hyp 2</i> \u000A</code> \u000A \u000A <p>If both hyptheses rewrite to true, then the rule <i>fires</i> and replaces \u000A the target by:</p> \u000A \u000A <code>(append (put <i>i</i> <i>v</i> <i>a</i>) <i>b</i>). \u000A</code> \u000A \u000A <p>In short, rewrite rules direct ACL2 to rearrange the terms in the goal \u000A formulas.</p> \u000A \u000A <p>We are more precise later, but for now we turn to the question of how do \u000A you make a rewrite rule from a formula?  The answer is, you prove the formula \u000A with the <v>defthm</v> command.  Recall that</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        ...) \u000A</code> \u000A \u000A <p>commands ACL2 to try to prove <i>formula</i> and, if successful, build it \u000A into the database as a rule according to your specification in the \u000A <i>rule-classes</i> argument of the ... part of the command.</p> \u000A \u000A <p>To make it easy for you to generate rewrite rules, <v>defthm</v> has a simple \u000A heuristic: if you don't tell it what kind of rule to generate from \u000A <i>formula</i>, it generates a rewrite rule!  Thus, if this command</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i>) \u000A</code> \u000A \u000A <p>is successful, ACL2 will have a new rewrite rule in the database, even \u000A though you did not <i>explicitly</i> tell it to add a rule.</p> \u000A \u000A <p>A common mistake for new users is to forget that the above command adds a \u000A rewrite rule.  This often results in a tangle of rules that lead nowhere or to \u000A infinite rewriting that you will have to interrupt.  It is also good to \u000A remember that the command <i>only</i> adds a rule.  It does not magically make \u000A ACL2 aware of all the mathematical consequences of the formula: it just makes \u000A a rewrite rule.</p> \u000A \u000A <p>When you prove a theorem with <v>defthm</v> you are <i>programming</i> ACL2. \u000A Being careless in your statement of lemmas is tantamount to being careless in \u000A your programming.</p> \u000A \u000A <p>ACL2 can generate rewrite rules from formulas that look like this:</p> \u000A \u000A <code>(IMPLIES (AND <i>hyp1</i> ... <i>hypk</i>) \u000A         (<i>eqv</i> <i>lhs</i> <i>rhs</i>)) \u000A</code> \u000A \u000A <p>where <i>eqv</i> is either <v>EQUAL</v> or <v>IFF</v>, and <i>lhs</i> is not a \u000A variable symbol, not a constant, and not a call of the function <v>IF</v>, and \u000A not a call of an abbreviation (``macro'') that expands to any of these.  So \u000A illegal <i>lhs</i> include <v>X</v>, <v>0</v>, <v>(<see topic=\"COMMON-LISP____IF\">IF</see> X Y Y)</v>, and <v>(<see topic=\"COMMON-LISP____OR\">OR</see> p q)</v>. \u000A The last is illegal because <v>OR</v> is just an abbreviation for a certain \u000A <v>IF</v>-expression.</p> \u000A \u000A <p>Technical Note: This tutorial introduction to the theorem prover takes \u000A liberties with the truth!  We are trying to give you a useful predictive model \u000A of the system without burdening you with all the details, which are discussed \u000A in the ACL2 User's Manual.  For example, using directives in the rule-classes \u000A you can rearrange the proved formula into the form you want your rule to take, \u000A and you can make ACL2 take advantage of equivalence relations <i>eqv</i> other \u000A than just <v>EQUAL</v> and <v>IFF</v>.  But we'll ignore these fine points for \u000A now.</p> \u000A \u000A <p>We call the <i>hyp</i> terms the <i>hypotheses</i> of the rule.  We call \u000A <i>lhs</i> the <i>left-hand side</i> of the rule, and we call <i>rhs</i> the \u000A <i>right-hand side</i> of the rule.  If the conclusion of the rule is an \u000A <v>EQUAL</v> term we call it an <i>equality</i> rule.  Otherwise, it is a \u000A <i>propositional equivalence</i> rule.  If there are no hypotheses, \u000A <i>k</i>=0, we say the rule is an <i>unconditional</i> rewrite rule; otherwise \u000A it is <i>conditional</i>.</p> \u000A \u000A <p>ACL2 allows several special cases of the shapes above.  See <see topic=\"ACL2____SPECIAL-CASES-FOR-REWRITE-RULES\">special-cases-for-rewrite-rules</see>, but come back here and continue.</p> \u000A \u000A <p>A rewrite rule makes ACL2 seek out occurrences of terms that match the \u000A left-hand side of the rule and replace those occurrences using the right-hand \u000A side, provided all the hypotheses rewrite to true in the context of the \u000A application of the rule.</p> \u000A \u000A <p>That is, the left-hand side is treated as a <i>pattern</i> that triggers \u000A the rule.  The hypotheses are <i>conditions</i> that have to be proved in \u000A order for the rule to fire.  The right-hand side is the <i>replacement</i> and \u000A is put into the formula where the pattern occurred.</p> \u000A \u000A <p>Now for some clarifications. ACL2 only considers enabled rules.  And ACL2 \u000A will use a propositional rule to replace a target only if the target occurs in \u000A a propositional place in the formula.  Generally that means it occurs in the \u000A argument of a propositional connective, like <v>AND</v>, <v>OR</v>, <v>NOT</v>, \u000A <v>IMPLIES</v>, and <v>IFF</v>, or in the test of an <v>IF</v>.  When we say that \u000A the left-hand side of the rule must <i>match</i> the target we mean that we \u000A can instantiate the variables in the rule to make the left-hand side identical \u000A to the target.  To <i>relieve</i> or establish the hypotheses of the rule, \u000A ACL2 just applies other rewrite rules to try to prove the instantiated \u000A hypotheses from the assumptions governing the occurrence of the target.  When \u000A ACL2 replaces the target, it replaces it with the instantiated right-hand side \u000A of the rule and then applies rewrite rules to that.</p> \u000A \u000A <p>If a hypothesis has variables that do not occur in the left-hand side of \u000A the rule, then the pattern matching process won't find values for those \u000A variables.  We call those <i>free variables</i>.  They must be instantiated \u000A before ACL2 can relieve that hypothesis.  To instantiate them, ACL2 has to \u000A guess values that would make the hypothesis true in this context, i.e., true \u000A given the assumptions of the goal theorem.  So if you're trying to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> A)\u000A              (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> P) A)\u000A              (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> P) A))\u000A         ...)</code> \u000A \u000A <p>and the target you're rewriting is in the ``...'' part of the formula, the \u000A rewriter knows that <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> A)</v> <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> P) A)</v> and <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see>\u000A (<see topic=\"COMMON-LISP____CDR\">CDR</see> P) A)</v> are true.  So if a rewrite rule is considered and the rule has \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)</v> as a hypothesis, where <v>e</v> is a free variable but <v>x</v> \u000A was bound to <v>A</v> in the pattern matching, then it will guess that <v>e</v> \u000A must be <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> P)</v> or <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> P)</v>, even though there are many other \u000A possibilities that would make <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e A)</v> true.  Of course, whatever \u000A guess it makes must also satisfy all the other hypotheses that mention <v>e</v> \u000A in the rule.  It simply isn't very imaginative at guessing!</p> \u000A \u000A <p>The most predictable rewrite rules have no free variables.  You can add \u000A pragmatic advice to help ACL2 with free variables, telling it to try all the \u000A possibilities it finds, to try just the first, or even to compute a \u000A ``creative'' guess.</p> \u000A \u000A <p>It is possible to make the rewriting process loop forever, e.g., by \u000A rewriting <i>alpha</i> to <i>beta</i> with one set of rules and rewriting \u000A <i>beta</i> to <i>alpha</i> with another.  Even a single rule can make the \u000A process loop; we'll show you an example of that later in the tutorial.  ACL2 \u000A can handle commutativity rules without looping.  It uses <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see>\u000A y x))</v> to replace <v>(<see topic=\"COMMON-LISP_____B2\">+</see> B A)</v> by <v>(<see topic=\"COMMON-LISP_____B2\">+</see> A B)</v>, but not vice versa.  (It is \u000A sensitive to alphabetic ordering when dealing with <i>permutative</i> \u000A rules.)</p> \u000A \u000A <p>Logically equivalent formulas can generate radically different rewrite \u000A rules!  Rearranging the propositional structure of the formula or swapping the \u000A left and right sides of an equality -- while having no effect on the \u000A mathematical meaning of a formula -- can have a drastic impact on the \u000A pragmatic meaning as a rule.  To see an illustration of this, see <see topic=\"ACL2____EQUIVALENT-FORMULAS-DIFFERENT-REWRITE-RULES\">equivalent-formulas-different-rewrite-rules</see>.</p> \u000A \u000A <p>Developing an effective set of rewrite rules is key to success at using \u000A ACL2.  We'll look more at this later in the tutorial.</p> \u000A \u000A <p>If you are working your way through the tutorial for the theorem prover, \u000A use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.  If you are reading just about how to \u000A make effective rewrite rules, go on to <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see>.</p>"],

 "ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>We assume you've read about rewrite rules; see <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see>.</p> \u000A \u000A <p>When a proof attempt fails, ACL2 prints some <i>key checkpoints</i>.  These \u000A are formulas that we think you should look at.  There are two kinds printed: \u000A key checkpoints <i>before an induction</i>, and key checkpoints <i>under a \u000A top-level induction</i>.  (Key checkpoints under deeper inductions and \u000A checkpoints that aren't considered ``key'' may exist in the proof attempt, but \u000A ACL2 doesn't print them at the end of failed proofs because you shouldn't be \u000A distracted by them.)</p> \u000A \u000A <p>Below is a list of questions to ask yourself about the key checkpoints. \u000A Initially, we recommend just picking one key checkpoint <i>before</i> an \u000A induction (perhaps the simplest looking one) and asking these questions. \u000A These questions may lead you to look at other key checkpoints.  As you gain \u000A more experience you'll elaborate and generalize this advice.</p> \u000A \u000A <p><b>(1) Do you believe this formula is a theorem?</b> If you don't think it \u000A is, it's pointless to try to prove it!  You should reconsider your top-level \u000A formula in light of the special case suggested by this key checkpoint.</p> \u000A \u000A <p><b>(2) Can it be simplified?</b> Is there some combination of function \u000A symbols in it that could be eliminated or simplified by exploiting some \u000A simpler fact?  By a ``simpler fact'' we mean a theorem about a few of the \u000A symbols in this formula.  For an example of this see <see topic=\"ACL2____DEALING-WITH-KEY-COMBINATIONS-OF-FUNCTION-SYMBOLS\">dealing-with-key-combinations-of-function-symbols</see>.  Don't think about the \u000A deep question ``how can I prove the checkpoint?'' until you've got it into its \u000A simplest form.</p> \u000A \u000A <p><b>(3) Is the simpler fact already in the database?</b> If there is some \u000A simpler fact that would help clean up the checkpoint but you believe the \u000A simpler fact is already in the database, you can use <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A and other <i>history</i> commands to inspect the database; (see <see topic=\"ACL2____HISTORY\">history</see> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>).  But if you find the allegedly relevant simpler \u000A fact in the database, you must ask: <b>why wasn't it used?</b> There are four \u000A principal reasons:</p> \u000A \u000A <p>(3a) it is disabled -- so enable it; you'll learn how when you read the \u000A coming sections on <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see> and <see topic=\"ACL2____INTRODUCTION-TO-HINTS\">introduction-to-hints</see>.</p> \u000A \u000A <p>(3b) its left-hand side doesn't match the target -- so improve the rule by \u000A generalizing its left-hand side or prove a new rule for this situation; if you \u000A decide to remove the old rule from the database, see <i>undo</i> commands in \u000A <see topic=\"ACL2____HISTORY\">history</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>(3c) it is an <v>IFF</v> rule but the target doesn't occur propositionally -- \u000A so see if you you can strengthen the rule to an <v>EQUAL</v> rule or weaken the \u000A context of the target by changing the conjecture to use the target \u000A propositionally; if you decide to remove the old rule from the database, see \u000A <i>undo</i> commands in <see topic=\"ACL2____HISTORY\">history</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>(3d) the hypotheses of the rule cannot be relieved for this occurrence of \u000A the target; this can be caused by the rule's hypotheses being too strong \u000A (requiring more than they should), or by the hypotheses of the current \u000A conjecture being too weak (you forgot some key hypothesis), or by ACL2 not \u000A having the rules it needs to prove that the conjecture's hypotheses really do \u000A imply the rule's.  Tools are available (<v>:</v>see <see topic=\"ACL2____BRR\">brr</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A help you figure out why the rule failed, so use them and improve the rule, or \u000A the current conjecture, or the database as appropriate.</p> \u000A \u000A <p><b>(4) If the simpler fact is not already known, prove it.</b> This means \u000A you must create a new <v>defthm</v> event with appropriate <i>rule-classes</i> \u000A to store your new theorem so that it will be used.  See <see topic=\"ACL2____DEALING-WITH-KEY-COMBINATIONS-OF-FUNCTION-SYMBOLS\">dealing-with-key-combinations-of-function-symbols</see>.  Then you must start using \u000A The Method recursively to prove your new lemma.</p> \u000A \u000A <p><b>(5) Otherwise, is this formula something you'd prove by induction?</b> \u000A If you can't simplify it, it may be because you ``need this fact to prove this \u000A fact,'' in which case, induction is the right thing to do.  But first, \u000A remember that in order for a formulas to be provable by induction, it must be \u000A very general.  Why must it be general?  Because in an inductive proof, the \u000A main thing you have to work with is the induction hypothesis, which is an \u000A instance of the theorem you're proving.  If the theorem is not general enough, \u000A you won't be able to assume an instance that will help you.  ACL2 may try \u000A induction even on formulas that are not general enough.  Don't assume that the \u000A formula is ripe for induction just because ACL2 found an induction to do! \u000A Before you ``approve'' a formula for induction, ask whether it is perhaps a \u000A special case of some more general theorem.  See <see topic=\"ACL2____GENERALIZING-KEY-CHECKPOINTS\">generalizing-key-checkpoints</see> now and then come back here.  If you found a \u000A generalization, you should probably be proving that formula instead of this \u000A one.  So formulate the appropriate <v>defthm</v> and use The Method recursively \u000A to prove it.</p> \u000A \u000A <p><b>(6) If the formula is right for induction, did ACL2 do an induction for \u000A it?</b> You can answer that without looking at the proof.  Just see if there \u000A are any key checkpoints after induction.  If not, why didn't ACL2 induct? \u000A Perhaps you told ACL2 not to induct!  Perhaps no term in the conjecture \u000A suggests an appropriate induction?  You could remedy this by extending ACL2's \u000A induction analysis by adding to the database.  Or you could just tell ACL2 \u000A what induction to do for this formula.  You'll learn about both later (when \u000A you read coming sections of the tutorial).</p> \u000A \u000A <p><b>(7) If ACL2 did do an induction, was it the right one?</b> You can find \u000A the induction scheme used by reading the first induction message in the output \u000A log after you submitted the conjecture to ACL2.  But most often you will \u000A realize the ``wrong'' induction was done just by looking at the post-induction \u000A key checkpoints, keeping in mind that each is supposed to be a natural special \u000A case of the theorem you're proving.  Is the case analysis inappropriate?  Are \u000A induction hypotheses missing?  If so, you should look at the induction scheme. \u000A If you determine the wrong induction was done, extend ACL2's induction \u000A analysis or tell it which induction to do, which you'll learn about in the \u000A coming sections of the tutorial.  For more advice about looking at \u000A post-induction key checkpoints, see <see topic=\"ACL2____POST-INDUCTION-KEY-CHECKPOINTS\">post-induction-key-checkpoints</see> now \u000A and then come back here.</p> \u000A \u000A <p><b>(8) If the post-induction key checkpoints seems plausible,</b> <b>then \u000A repeat the questions above for each one of them,</b> <b>perhaps starting with \u000A the simplest.</b></p> \u000A \u000A <p>In any case, after successfully taking whatever action you've decided on, \u000A e.g., proving some new lemma and adding it as a rule:</p> \u000A \u000A <p><b>Start over trying to prove your main conjecture.</b> This is important! \u000A Do not just scroll back to the key checkpoints generated the last time you \u000A tried to prove it.  Instead, re-generate them in the context of your new, \u000A improved database and hints.</p> \u000A \u000A <p>You will be following this general outline almost all of the time that \u000A you're interacting with ACL2.  You will not often be asking ``Why is ACL2 \u000A making me think about this subgoal?  What did ACL2 do to get here?  How does \u000A ACL2 work?''</p> \u000A \u000A <p>Two other ideas are helpful to keep in mind.</p> \u000A \u000A <p><b>Is a key checkpoint unexpectedly complicated?</b> Pay special attention \u000A to the case where the complication seems to be the introduction of low-level \u000A details you thought you'd dealt with or by the introduction of symbols you \u000A didn't expect to see in this proof.  These can be signs that you ought to \u000A disable some rules in the database (e.g., a definition of a complicated \u000A function about which you've proved all the necessary lemmas or some lemma that \u000A transforms the problem as was appropriate for some other proof).</p> \u000A \u000A <p><b>Does the theorem prover just hang up, printing nothing?</b> If this \u000A happens, you must interrupt it.  How you interrupt the prover is dependent on \u000A which Common Lisp and which interface you're using.  But most Common Lisps \u000A treat control-c as a console interrupt.  If you're in Emacs running ACL2 as a \u000A shell process, you must type control-c control-c.  If you're in ACL2s, hit the \u000A <i>Interrupt Session</i> button.  Interrupting ACL2 can leave you in an \u000A interactive loop similar in appearance but different from ACL2's top-level! \u000A So pay careful attention to the prompt and see <see topic=\"ACL2____BREAKS\">breaks</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>Once you've regained control from the ``runaway'' theorem prover, there are \u000A several tools you can use to find out what it is doing in real-time. \u000A Generally speaking, when the theorem prover goes silent for a very long time \u000A it is either in some kind of rewrite loop caused by rules that cause it to \u000A flip back and forth between various supposedly normal forms, or else it has \u000A split the problem into a huge number of cases and suffering a combinatoric \u000A explosion.  See <see topic=\"ACL2____DMR\">dmr</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A and, perhaps, see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>If you are reading this as part of the tutorial introduction to the theorem \u000A prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR":[["Events"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> assumes familiarity with \u000A <v>:clause-processor</v> rules; see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>.  Briefly put, a \u000A <i>clause-processor</i> is a user-defined function that takes as input the \u000A ACL2 representation of a goal &mdash; a <i>clause</i> &mdash; and returns a \u000A list of goals (i.e., a list of clauses).  A <v>:clause-processor</v> rule is a \u000A way to inform ACL2 that a clause-processor has been proved correct and now may \u000A be specified in <v>:clause-processor</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>Here we describe a utility, <v>define-trusted-clause-processor</v>, that \u000A provides another way to inform ACL2 that a function is to be considered a \u000A clause-processor that can be specified in a <v>:clause-processor</v> hint.  You \u000A can find examples of correct and incorrect use of this utility in community \u000A book <v>books/clause-processors/basic-examples</v>.</p> \u000A \u000A <p>Consider the simple example already presented for <v>:clause-processor</v> \u000A rules (again, see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>), for a simple clause-processor \u000A named <v>note-fact-clause-processor</v>.  Instead of introducing an evaluator \u000A and proving a correctness theorem with <v>:rule-classes :clause-processor</v>, \u000A we can simply inform ACL2 that we trust the function \u000A <v>note-fact-clause-processor</v> to serve as a clause-processor.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>\u000A  note-fact-clause-processor\u000A  nil\u000A  :ttag my-ttag)</code> \u000A \u000A <p>A non-nil <v>:ttag</v> argument generates a <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> event in order to \u000A acknowledge the dependence of the ACL2 session on the (unproved) correctness \u000A of this clause-processor.  That argument can be omitted if there is currently \u000A an active trust tag.  See <see topic=\"ACL2____DEFTTAG\">defttag</see>.  Because we are trusting this \u000A clause-processor, rather than having proved it correct, we refer to it as a \u000A ``trusted'' clause-processor to contrast with a proved-correct, or \u000A ``verified'', clause-processor.</p> \u000A \u000A <p>Now that the event displayed above has established \u000A <v>note-fact-clause-processor</v> as a (trusted) clause-processor, we can use it \u000A in a <v>:clause-processor</v> hint, for example as follows.  Notice that the \u000A output is identical to that for the corresponding example presented for the \u000A verified case (see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>), except that the word ``verified'' \u000A has been replaced by the word ``trusted''.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A                   x)\u000A            :hints\u000A            ((&quot;Goal&quot;\u000A              :clause-processor\u000A              (note-fact-clause-processor clause '(<see topic=\"COMMON-LISP____EQUAL\">equal</see> a a)))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AWe now apply the trusted :CLAUSE-PROCESSOR function NOTE-FACT-CLAUSE-\u000APROCESSOR to produce two new subgoals.\u000A\u000ASubgoal 2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> A A)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X Y)) X)).\u000A\u000ABut we reduce the conjecture to T, by the :executable-counterpart of\u000AIF and the simple :rewrite rule CAR-CONS.\u000A\u000ASubgoal 1\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> A A).\u000A\u000ABut we reduce the conjecture to T, by primitive type reasoning.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:EXECUTABLE-COUNTERPART IF)\u000A        (:EXECUTABLE-COUNTERPART NOT)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:REWRITE CAR-CONS))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>Indeed, if one runs this example first and subsequently verifies the \u000A clause-processor, one will see the word ``trusted'' change to \u000A ``verified''.</p> \u000A \u000A <p>The general form is as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>\u000A  cl-proc           ;;; clause-processor function\u000A  supporters        ;;; see below\u000A  &amp;key\u000A  label             ;;; optional, but required if doc is non-nil\u000A  doc               ;;; optional\u000A  ttag              ;;; discussed above\u000A  partial-theory    ;;; optional encapsulate event\u000A  )</code> \u000A \u000A <p>If a <v>:label</v> <v>LAB</v> is supplied, then a subsidiary <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> \u000A event will be generated with name <v>LAB</v>, which will enable you to to undo \u000A this <v>define-trusted-clause-processor</v> event using: <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt><v>\u000A LAB</v>.  If you supply a <v>:label</v> then you may supply a <v>:doc</v> argument \u000A to use with that generated <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> event.  We discussed the \u000A <v>:ttag</v> argument above.  The entire form is considered redundant (skipped) \u000A if it is identical to one already executed in the current ACL2 world; but if \u000A it is not redundant, then <v>cl-proc</v> must not already have been similarly \u000A designated as a trusted clause-processor.</p> \u000A \u000A <p>Note that <v>cl-proc</v> may be defined either in <v>:program</v>-mode or \u000A <v>:logic</v>-mode.</p> \u000A \u000A <p>The <v>supporters</v> argument should be a true list of function symbols in \u000A the current ACL2 world.  It is important that this list include user-defined \u000A functions whose definitions support the correctness of the clause-processor \u000A function.  Otherwise, <tt><see topic=\"ACL2____LOCAL\">local</see></tt> definitions of those missing supporters \u000A can render the use of this clause-processor unsound, as discussed in the paper \u000A referenced at the end of the <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> documentation topic. \u000A Moreover, ACL2 assumes for dependent clause-processors (discussed below) that \u000A every function symbol constrained by the ``promised encapsulate'' of that \u000A event is either among those <v>supporters</v> or ancestral in one of them \u000A (i.e. a supporter of a supporter, a supporter of one of those, etc.).</p> \u000A \u000A <p><b>Dependent clause-processors and promised encapsulates</b>: The \u000A <v>:partial-theory</v> argument</p> \u000A \u000A <p>Suppose you want to introduce a clause-processor to reason about a complex \u000A hardware simulator that is implemented outside ACL2.  Sawada and Reeber had \u000A just such a problem, as reported in their FMCAD 2006 paper.  Indeed, they used \u000A <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> to implement a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>-mode function in ACL2 \u000A that can invoke that simulator.  In principle one could code the simulator \u000A directly in ACL2; but it would be a tremendous amount of work that has no \u000A practical purpose, given the interface to the external simulator.  So: In what \u000A sense can we have a clause-processor that proves properties about a simulator \u000A when that simulator is not fully axiomatized in ACL2?  Our answer, in a \u000A nutshell, is this: The above <v>:partial-theory</v> argument provides a way to \u000A write merely some of the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s on the external tool (or even no \u000A constraints at all), with the understanding that such constraints are present \u000A implicitly in a stronger ``promised'' <v>encapsulate</v>, for example by \u000A exporting the full definition.</p> \u000A \u000A <p>If a trusted clause-processor is introduced with a <v>:partial-theory</v> \u000A argument, we call it a ``dependent'' clause-processor, because its correctness \u000A is dependent on the constraints implicitly introduced by the \u000A <v>:partial-theory</v> <v>encapsulate</v> form.  The implicit constraints should \u000A logically imply the constraints actually introduced by the explicit \u000A <v>encapsulate</v>, but they should also be sufficient to justify every possible \u000A invocation of the clause-processor in a <v>:clause-processor</v> hint.  The user \u000A of a <v>define-trusted-clause-processor</v> form is making a guarantee &mdash; \u000A or, is relying on a guarantee provided by the writer of that form &mdash; that \u000A in principle, there exists a so-called ``promised encapsulate'': an \u000A <v>encapsulate</v> form with the same <see topic=\"ACL2____SIGNATURE\">signature</see> as the \u000A <v>:partial-theory</v> <v>encapsulate</v> form associated with the trusted \u000A clause-processor, but whose constraints introduced are the aforementioned \u000A implicit constraints.</p> \u000A \u000A <p>There are several additional requirements on a <v>:partial-theory</v> \u000A argument.  First, it must be an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event with non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>.  Moreover, the functions introduced by that event must be exactly \u000A those specified in the signature, and no more.  And further still, the \u000A <v>define-trusted-clause-processor</v> form cannot be executed inside any <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form with non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>; we can think of this \u000A situation as attempting to associate more than one <v>encapsulate</v> with the \u000A functions introduced in the inner <v>encapsulate</v>.</p> \u000A \u000A <p>The <v>:partial-theory</v> event will (in essence) be executed as part of the \u000A evaluation of the <v>define-trusted-clause-processor</v> form.  Again, a \u000A critical obligation rests on the user who provides a <v>:partial-theory</v>: \u000A there must exist (in principle at least) a corresponding promised encapsulate \u000A form with the same <see topic=\"ACL2____SIGNATURE\">signature</see> that could logically be admitted, whenever \u000A the above <v>define-trusted-clause-processor</v> form is evaluated successfully, \u000A that justifies the designation of <v>cl-proc</v> as a clause-processor.  See \u000A also the paper mentioned above for more about promised encapsulates.  A key \u000A consequence is that the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s are unknown for the functions \u000A introduced in (the signature of) a <v>:partial-theory</v> <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A form.  Thus, functional instantiation (see <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE\">functional-instantiation-example</see>) is disabled for function in the signature \u000A of a <v>:partial-theory</v> form.</p> \u000A \u000A <p><b>A remark on the underlying implementation</b></p> \u000A \u000A <p>You can see all of the current trusted clause-processors by issuing the \u000A command <v>(<see topic=\"ACL2____TABLE\">table</see> trusted-clause-processor-table)</v>.  Those that are dependent \u000A clause-processors will be associated in the resulting association list with a \u000A pair whose <v>car</v> is the list of supporters and whose <v>cdr</v> is <v>t</v>, \u000A i.e., with <v>(supporters . t)</v>; the others will be associated just with \u000A <v>(supporters)</v>.</p> \u000A \u000A <p>Thus, <v>define-trusted-clause-processor</v> is actually a macro that \u000A generates (among other things) a <v>table</v> event for a table named \u000A <v>trusted-clause-processor-table</v>; see <see topic=\"ACL2____TABLE\">table</see>.  You are invited to use \u000A <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> to see expansions of calls of this macro.</p> \u000A \u000A <p><b>A technique for using raw Lisp to define a trusted \u000A clause-processor</b></p> \u000A \u000A <p>The following code is intended to give an idea for how one might define the \u000A ``guts'' of a trusted clause-processor in raw Lisp.  The idea is to stub out \u000A functions, such as <v>acl2-my-prove below</v>, that you want to define in raw \u000A Lisp; and then, load a raw Lisp file to overwrite any such function with the \u000A real code.  But then we make any such overwritten function untouchable.  (This \u000A last step is important because otherwise, one can prove <v>nil</v> using a \u000A <v>:functional-instance</v> <v>:use</v> hint, by exploiting the fact that this \u000A function has executable code for which there is no corresponding definitional \u000A axiom.)  Note: The point here is only to illustrate the use of raw Lisp, so we \u000A do not bother to define or explain functions <v>hint-to-termlist</v> or \u000A <v>disjoin-clause-segments-to-clause</v>, which this example assumes are defined \u000A elsewhere; their meanings are not important for this example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> acl2-my-prove (<see topic=\"ACL2____TERM\">term</see> hint) t)\u000A\u000A(<see topic=\"ACL2____PROGRAM\">program</see>)\u000A\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :my-cl-proc)\u000A\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A\u000A; We wrap everything here in a single progn, so that the entire form is\u000A; atomic.  That's important because we want the use of push-untouchable to\u000A; prevent anything besides my-clause-processor from calling acl2-my-prove.\u000A\u000A  (<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A\u000A   (set-raw-mode-on state)\u000A\u000A   (load &quot;my-hint-raw.lsp&quot;) ; defines my-prove in raw Lisp\u000A\u000A   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> acl2-my-prove (<see topic=\"ACL2____TERM\">term</see> hint)\u000A     (my-prove term hint)))\u000A\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-clause-processor (cl hint)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (pseudo-term-listp cl)\u000A                    :mode :program))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (acl2-my-prove (disjoin cl) hint)\u000A        (disjoin-clause-segments-to-clause\u000A         (<see topic=\"ACL2____PAIRLIS_42\">pairlis$</see> (hint-to-termlist hint) nil)\u000A         cl)\u000A      (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;~|~%NOTE: Unable to prove goal with ~\u000A                  my-clause-processor and indicated hint.~|&quot;)\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> cl))))\u000A\u000A  (<see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see> acl2-my-prove t)\u000A  )</code>"],

 "ACL2____COMPOUND-RECOGNIZER":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> alistp-implies-true-listp-compound-recognizer\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____ALISTP\">alistp</see> x)                 ; When (<see topic=\"ACL2____ALISTP\">alistp</see> x) is assumed true, add\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))            ; the additional hypothesis that x is\u000A  :rule-classes :compound-recognizer) ; of primitive type true-listp.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> natp-compound-recognizer      ; See discussion below.\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NATP\">natp</see> x)\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x)))\u000A  :rule-classes :compound-recognizer)</code> \u000A \u000A <p>Before presenting the General Forms, we start with a motivating example: \u000A the second <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> form above, which provides a nice example of a \u000A <v>:compound-recognizer</v> rule that is built into ACL2.  To see how this rule \u000A might be useful, consider the following (admittedly very simple) <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> triple (x)\u000A  (<see topic=\"COMMON-LISP_____A2\">*</see> 3 x))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> triple-preserves-integerp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A           (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (triple x))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> triple natp))</code> \u000A \u000A <p>If the above <v>:compound-recognizer</v> rule is disabled, then the following \u000A trivial theorem fails as shown; we explain below.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> x)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (triple x)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> natp-compound-recognizer))))</code> \u000A \u000A <p>The problem is that when ACL2 tries to rewrite the term <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see>\u000A (triple x))</v> using the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule \u000A <v>triple-preserves-integerp</v>, it needs to rewrite the hypothesis \u000A <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v> to <v>t</v>, but instead what is known is <v>(<see topic=\"ACL2____NATP\">natp</see> x)</v>.  If \u000A we remove the hint, then the proof succeeds because the above \u000A <v>:compound-recognizer</v> rule tells ACL2 that when assuming <v>(<see topic=\"ACL2____NATP\">natp</see> x)</v> to \u000A be true, it should actually assume both <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v> and <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x)</v> to \u000A be true.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (fn x) concl)               ; (1)\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (fn x)) concl)         ; (2)\u000A(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (fn x) concl1)         ; (3)\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (fn x)) concl2))\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (fn x) concl1 concl2)            ; (4)\u000A(<see topic=\"ACL2____IFF\">iff</see> (fn x) concl)                   ; (5)\u000A(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fn x) concl)                 ; (6)</code> \u000A \u000A <p>where <v>fn</v> is a Boolean valued function of one argument, <v>x</v> is a \u000A variable symbol, and the system can deduce some restriction on the primitive \u000A type of <v>x</v> from the assumption that <v>concl</v> holds.  The last \u000A restriction is vague but one way to understand it is to weaken it a little to \u000A ``and <v>concl</v> is a non-tautological conjunction or disjunction of the \u000A primitive type recognizers listed below.''</p> \u000A \u000A <p>The primitive ACL2 types and a suitable primitive recognizing expression \u000A for each are listed below.</p> \u000A \u000A <code>type                suitable primitive recognizer\u000A\u000Azero                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)\u000Anegative integers   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0))\u000Apositive integers   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 0))\u000Anegative ratio      (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A                         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0))\u000Apositive ratio      (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A                         (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 0))\u000Acomplex rational    (<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> x)\u000Anil                 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)\u000At                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x t)\u000Aother symbols       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))\u000A                         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x t)))\u000Aproper conses       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                         (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))\u000Aimproper conses     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000Astrings             (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000Acharacters          (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)</code> \u000A \u000A <p>Thus, a suitable <v>concl</v> to recognize the naturals would be <v>(<see topic=\"COMMON-LISP____OR\">or</see>\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0) (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 0)))</v> but it turns out that we also permit \u000A <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0))</v>.  Similarly, the true-lists could be \u000A specified by</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil) (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))</code> \u000A \u000A <p>but we in fact allow <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)</v>.  When time permits we will \u000A document more fully what is allowed or implement a macro that permits direct \u000A specification of the desired type in terms of the primitives.</p> \u000A \u000A <p>There are essentially four forms of <v>:compound-recognizer</v> rules, as the \u000A forms labeled (3) and (4) above are equivalent, as are those labeled (5) and \u000A (6).  We explain how such rules are used by considering the individual \u000A forms.</p> \u000A \u000A <p>Consider form (1), <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (fn x) concl)</v>.  The effect of such a rule \u000A is that when the rewriter assumes <v>(fn x)</v> true, as it would while diving \u000A through <v>(<see topic=\"COMMON-LISP____IF\">if</see> (fn x) xxx ...)</v> to rewrite <v>xxx</v>, it restricts the type of \u000A <v>x</v> as specified by <v>concl</v>.  For example, if <v>concl</v> is the term \u000A <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v>, then when rewriting <v>xxx</v>, <v>x</v> will be assumed to be \u000A an integer.  However, when assuming <v>(fn x)</v> false, as necessary in <v>(<see topic=\"COMMON-LISP____IF\">if</see>\u000A (fn x) ... xxx)</v>, the rule permits no additional assumptions about the type \u000A of <v>x</v>.  For example, if <v>fn</v> is <v>primep</v>, i.e., the predicate that \u000A recognizes prime numbers, then <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (primep x) (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x\u000A 0)))</v> is a compound recognizer rule of the first form.  When <v>(primep x)</v> \u000A is assumed true, the rewriter gains the additional information that <v>x</v> is \u000A a natural number.  When <v>(primep x)</v> is assumed false, no additional \u000A information is gained &mdash; since <v>x</v> may be a non-prime natural or may \u000A not even be a natural.</p> \u000A \u000A <p>Form (2) is the symmetric case, when assuming <v>(fn x)</v> false permits \u000A type restrictions on <v>x</v> but assuming <v>(fn x)</v> true permits no such \u000A restrictions.  For example, if we defined <v>exprp</v> to be the recognizer for \u000A well-formed expressions for some language in which all symbols, numbers, \u000A character objects and strings were well-formed &mdash; e.g., the \u000A well-formedness rules only put restrictions on expressions represented by \u000A <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>s &mdash; then the theorem <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (exprp x)) (<see topic=\"COMMON-LISP____CONSP\">consp</see>\u000A x))</v> is a rule of the second form.  Assuming <v>(exprp x)</v> true tells us \u000A nothing about the type of <v>x</v>; assuming it false tells us <v>x</v> is a \u000A <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>.</p> \u000A \u000A <p>Forms (3) and (4), which are really equivalent, address themselves to the \u000A case where one type may be deduced from <v>(fn x)</v> and a generally unrelated \u000A type may be deduced from its negation.  If we modified the expression \u000A recognizer above so that character objects are illegal, then rules of the \u000A forms (3) and (4) are</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (exprp x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (exprp x)) (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))).\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (exprp x)\u000A    (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x))\u000A  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))</code> \u000A \u000A <p>Finally, rules of forms (5) and (6) address the case where <v>fn</v> \u000A recognizes all and only the objects whose type is described.  In these cases, \u000A <v>fn</v> is really just a new name for some ``compound recognizers.''  The \u000A classic example is <v>(<see topic=\"ACL2____BOOLEANP\">booleanp</see> x)</v>, which is just a handy combination of two \u000A primitive types:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"ACL2____BOOLEANP\">booleanp</see> x) (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x t) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))).</code> \u000A \u000A <p>Often it is best to disable <v>fn</v> after proving that it is a compound \u000A recognizer, since otherwise the term <v>(fn x)</v> will be expanded and thus \u000A disappear.</p> \u000A \u000A <p>Every time you prove a new compound recognizer rule about <v>fn</v> it \u000A overrides all previously proved compound recognizer rules about <v>fn</v>. \u000A Thus, if you want to establish the type implied by <v>(fn x)</v> and you want to \u000A establish the type implied by <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (fn x))</v>, you must prove a compound \u000A recognizer rule of the third, fourth, fifth, or sixth forms.  Proving a rule \u000A of the first form followed by one of the second only leaves the second fact in \u000A the database.</p> \u000A \u000A <p>Compound recognizer rules can be disabled with the effect that older rules \u000A about <v>fn</v>, if any, are exposed.</p> \u000A \u000A <p>If you prove more than one compound recognizer rule for a function, you may \u000A see a <b>warning</b> message to the effect that the new rule is not as \u000A ``restrictive'' as the old.  That is, the new rules do not give the rewriter \u000A strictly more type information than it already had.  The new rule is stored \u000A anyway, overriding the old, if enabled.  You may be playing subtle games with \u000A enabling or rewriting.  But two other interpretations are more likely, we \u000A think.  One is that you have forgotten about an earlier rule and should merely \u000A print it out to make sure it says what you intend, and then discard your new \u000A rule.  The other is that you meant to give the system more information and the \u000A system has simply been unable to extract the intended type information from \u000A the term you placed in the conclusion of the new rule.  Given our lack of \u000A specificity in saying how type information is extracted from rules, you can \u000A hardly blame yourself for this problem.  Sorry.  If you suspect you've been \u000A burned this way, you should rephrase the new rule in terms of the primitive \u000A recognizing expressions above and see if the warning is still given.  It would \u000A also be helpful to let us see your example so we can consider it as we \u000A redesign this stuff.</p> \u000A \u000A <p>Compound recognizer rules are similar to <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> \u000A rules in that the system deduces new information from the act of assuming \u000A something true or false.  If a compound recognizer rule were stored as a \u000A forward chaining rule it would have essentially the same effect as described, \u000A when it has any effect at all.  The important point is that <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules, because of their more general and expensive form, are \u000A used ``at the top level'' of the simplification process: we forward chain from \u000A assumptions in the goal being proved.  But compound recognizer rules are built \u000A in at the bottom-most level of the simplifier, where type reasoning is \u000A done.</p> \u000A \u000A <p>All that said, compound recognizer rules are a rather fancy, specialized \u000A mechanism.  It may be more appropriate to create <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules instead of <v>:compound-recognizer</v> rules.</p>"],

 "ACL2____ACL2-DOC":[["Documentation"],"ACL2 Sources","ACL2","<p>As discussed elsewhere (see <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>), the web-based \u000A <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a> provides a way to browse the combined \u000A documentation for the ACL2 system and community books.  This documentation can \u000A also be read at the terminal using the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> command, though \u000A documentation for <see topic=\"ACL2____BOOKS\">books</see> will only be included for those books that have \u000A been included in the session.  In this topic we describe how to browse the \u000A documentation using ACL2-Doc, a browser for reading ACL2 and books \u000A documentation inside Emacs.</p> \u000A \u000A <p>While ACL2-Doc is much like Emacs Info, it is a separate system that \u000A provides some additional functionality.  In order to use ACL2-Doc, load the \u000A distributed file <tt>emacs/acl2-doc.el</tt> into Emacs.  This will happen \u000A automatically if you load <tt>emacs/emacs-acl2.el</tt>, which will happen \u000A automatically if you put the following form in your <tt>~/.emacs</tt> file, \u000A replacing <v>DIR</v> by a path to your ACL2 installation.</p> \u000A \u000A <code>(load &quot;DIR/emacs/emacs-acl2.el&quot;)</code> \u000A \u000A <p>Then to start the browser at the top-level topic, either execute the Emacs \u000A command</p> \u000A \u000A <code>meta-x acl2-doc</code> \u000A \u000A <p>or else type:</p> \u000A \u000A <code>Control-t g</code> \u000A \u000A <p>By default you will browse the ACL2+Books Manual, though if you are using \u000A an svn version between ACL2 releases then you may be queried; more on that \u000A below.  You can enter the ACL2-Doc browser at a specific documentation topic \u000A as follows (in analogy to Emacs command Meta-.):</p> \u000A \u000A <code>Control-t .</code> \u000A \u000A <p>In each of the cases above, you will now be in a buffer called \u000A \"<tt>acl2-doc</tt>\", which will be displaying the top-level ACL2 topic in a \u000A special mode, the ACL2-Doc major mode.  That mode provides the following key \u000A bindings; you can also see these by typing <tt>Control-h m</tt> while in that \u000A buffer.</p> \u000A \u000A <code>&lt;Return&gt;      acl2-doc-go!\u000Ag             acl2-doc-go\u000Ah             acl2-doc-help\u000Ai             acl2-doc-index\u000A,             acl2-doc-index-next\u000Al             acl2-doc-last\u000An             acl2-doc-search-next\u000Aq             acl2-doc-quit\u000Ar             acl2-doc-return\u000As             acl2-doc-search\u000AS             acl2-doc-re-search\u000At             acl2-doc-top\u000Au             acl2-doc-up\u000Aw             acl2-doc-where\u000ASPC           scroll-up\u000ATAB           acl2-doc-tab\u000AD             acl2-doc-rendered-combined-download\u000AH             acl2-doc-history\u000AI             acl2-doc-initialize</code> \u000A \u000A <p>You can see the documentation for each of these in the usual way, using \u000A <tt>Control-h k {key}</tt> or <tt>Control-h f {command}</tt>.  Here is what \u000A you will find in each case if you do that.</p> \u000A \u000A <code>&lt;Return&gt;      acl2-doc-go!\u000A   Go to the topic occurring at the cursor position.\u000A\u000Ag             acl2-doc-go\u000A   Go to the specified topic; performs completion.\u000A\u000Ah             acl2-doc-help\u000A   Go to the ACL2-DOC topic to read about how to use the ACL2-Doc browser.\u000A\u000Ai             acl2-doc-index\u000A   Go to the specified topic or else one containing it as a substring;\u000A   performs completion.  If the empty string is supplied, then go to the\u000A   index buffer.  Note that the index buffer is in ACL2-Doc mode; thus, in\u000A   particular, you can type &lt;RETURN&gt; while standing on a topic in order to go\u000A   directly to that topic.\u000A\u000A,             acl2-doc-index-next\u000A   Find the next topic containing, as a substring, the topic from the\u000A   previous i command.\u000A\u000Al             acl2-doc-last\u000A   Go to the last topic visited.\u000A\u000An             acl2-doc-search-next\u000A   Find the next occurrence for the most recent search or regular expression\u000A   search.\u000A\u000Aq             acl2-doc-quit\u000A   Quit the ACL2-Doc browser.\u000A\u000Ar             acl2-doc-return\u000A   Return to the last topic visited, popping the stack of such topics.\u000A\u000As             acl2-doc-search\u000A   Search forward from the top of the manual for the input string.  If the\u000A   search succeeds, then go to that topic with the cursor put immediately\u000A   after the found text, with the topic name displayed in the minibuffer.\u000A\u000AS             acl2-doc-re-search\u000A   Perform a regular expression search, forward from the top of the manual,\u000A   for the input string.  If the search succeeds, then go to that topic with\u000A   the cursor put immediately after the found text, with the topic name\u000A   displayed in the minibuffer.\u000A\u000At             acl2-doc-top\u000A   Go to the top topic.\u000A\u000Au             acl2-doc-up\u000A   Go to the parent of the current topic.\u000A\u000Aw             acl2-doc-where\u000A   Display the topic name in the minibuffer, together with the manual name\u000A   (ACL2+Books Manual or ACL2 User's Manual)\u000A\u000ASPC           scroll-up\u000A   Scroll up (same as Control-v)\u000A\u000ATAB           acl2-doc-tab\u000A   Visit the next link after the cursor on the current page, searching from\u000A   the top if no link is below the cursor.\u000A\u000AD\u000A   Download the ``bleeding edge'' ACL2+Books Manual from the web; then\u000A   restart the ACL2-Doc browser to view that manual.\u000A\u000AH             acl2-doc-history\u000A   Visit a buffer that displays the names of all visited topics in order,\u000A   newest at the bottom.  That buffer is in acl2-doc mode; thus the usual\u000A   acl2-doc commands may be used.  In particular, you can visit a displayed\u000A   topic name by putting your cursor on it and typing &lt;RETURN&gt;.\u000A\u000AI             acl2-doc-initialize\u000A   Restart the ACL2-Doc browser, clearing its state.  With prefix argument,\u000A   toggle between the ACL2 User's Manual (<see topic=\"COMMON-LISP____THE\">the</see> default) and the ACL2+Books\u000A   Manual.  For the latter, it will be necessary first to create file\u000A   books/system/doc/rendered-doc-combined.lsp; see :DOC acl2-doc.</code> \u000A \u000A <h4>The Two Manuals</h4> \u000A \u000A <p>ACL2-Doc can display the ACL2 User's Manual, which includes documentation \u000A for the ACL2 system but not for the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  But by default, \u000A ACL2-Doc will display the ACL2+Books Manual, which includes documentation for \u000A those books as well.  To change which of these two manuals you display, just \u000A give a prefix argument to the \"<tt>I</tt>\" command, as indicated above.</p> \u000A \u000A <p>If you are using an svn version of ACL2 and the books, between releases, \u000A then you may need to download an extra file in order to browse the ACL2+Books \u000A Manual.  Most likely you will just answer <v>y</v> when queried about \u000A downloading the file when first using ACL2-Doc.  If you want more details, see \u000A the last of the Notes below.</p> \u000A \u000A <h4>Notes</h4> \u000A \u000A <ul> \u000A \u000A <li>You might find that when you attempt to follow <see topic=\"ACL2____SOME-BROKEN-LINK\">some-broken-link</see>, \u000A you find yourself at the <see topic=\"ACL2____BROKEN-LINK\">broken-link</see> topic.  If you are using the ACL2 \u000A User's Manual rather than the ACL2+Books Manual, the reason might be that \u000A <v>some-broken-link</v> is documented in a book, not in the ACL2 system.  In \u000A that case, the <v>broken-link</v> page will show you where to find that book; \u000A but if you want to read the documentation for <v>some-broken-link</v> in the \u000A ACL2-Doc browser, you can do so by switching to the ACL2+Books Manual.  See \u000A the <v>I</v> command, documented above.<p/></li> \u000A \u000A <li>Files with names ending in <tt>.acl2-doc</tt> will come up in ACL2-Doc \u000A mode.  Thus, you may wish to save a file with that extension, for example \u000A <tt>bookmarks.acl2-doc</tt>, that contains your favorite bookmarks.  You may \u000A wish to use the history command (<tt>H</tt>) to obtain a list of names of \u000A visited topics, in order to create an initial such file.<p/></li> \u000A \u000A <li>Many commands offer defaults, and many offer completion.  The default is \u000A determined by cursor position: if the cursor is sitting on a letter of a \u000A documentation topic name, or on a space character immediately after it, then \u000A that name will be offered as the default.<p/></li> \u000A \u000A <li>Square brackets indicate documentation topic names, for example: \u000A <tt>[acl2-doc]</tt>.  The square brackets are really there, for example when \u000A you are searching using \"s\", \"S\", or \"n\".  However, for purposes of \u000A determining the default name (see above), the only effect of the enclosing \u000A square brackets is to extend the region in which the default is offered.  For \u000A example, consider the string \"<tt>[acl2-doc]</tt>\": the default name of \u000A \"<tt>acl2-doc</tt>\" is offered if the cursor is on either square \u000A bracket.  But links have some idiosyncrasies.<p/> \u000A \u000A <ol> \u000A \u000A   <li>Topic names, including links `<tt>[..]</tt>' to topic names, are printed \u000A   relative to the ACL2 package.  Especially in the case of the ACL2+Books \u000A   Manual, you may therefore see links that include package prefixes.  Here, \u000A   for example, is a sentence from the documentation for <see topic=\"ACL2____GL\">gl</see> in the \u000A   ACL2+Books Manual. \u000A \u000A   <code>We call these structures [gl::symbolic-objects]. \u000A   </code> \u000A \u000A   The \"<tt>gl</tt>\" package prefix allows commands to pick up \u000A   \"<tt>gl::symbolic-objects</tt>\" as the name to use as a default, so that \u000A   for example, hitting <v>&lt;Return&gt;</v> will take you to that topic.  But when \u000A   reading the sentence, for best results you should ignore package prefixes. \u000A   So for example, you would read the sentence above as follows. \u000A \u000A   <code>We call these structures symbolic-objects. \u000A   </code><p/></li> \u000A \u000A   <li>Topic names that originally contained spaces now have underscores in \u000A   place of the spaces.  So for example, the topic <see topic=\"ACL2____CERT.PL\">cert.pl</see> in the \u000A   ACL2+Books Manual contains a link to the topic originally named as follows. \u000A \u000A   <code>1. Certifying Simple Books</code> \u000A \u000A   This link shows up in the ACL2-Doc browser (and in output from the \u000A   <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> command at the terminal) as: \u000A   <v>[1._Certifying_Simple_Books]</v>.  When you see a potential link that does \u000A   include whitespace, then it will not work, probably because the original \u000A   markup specified English text that differs from the topic name.  For \u000A   example, consider the following passage from the documentation for <see topic=\"ACL2____GL\">gl</see>. \u000A \u000A   <code>GL requires ACL2(h) because it makes extensive use of \u000A[hons-and-memoization]. Some optional parts of GL also require \u000A[trust tags]. \u000A   </code> \u000A \u000A   The first link will take you to the topic, <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  But \u000A   the second link was actually written to be a link to the documentation for \u000A   <see topic=\"ACL2____DEFTTAG\">defttag</see>.  The <v>&lt;Return&gt;</v> command will not work on that second \u000A   link; unfortunately, the alternative is to use the <v>g</v> command and \u000A   specify <v>defttag</v> in the minibuffer.<p/></li> \u000A \u000A </ol> \u000A \u000A Of course, the web-based browser avoids these idiosyncrasies (see <see topic=\"XDOC____SAVE\">xdoc::save</see>), hence may be more appropriate for those who have no particular \u000A preference for using Emacs to browse the documentation.<p/></li> \u000A \u000A <li>Searching using the \"s\" or \"S\" command is carried out by searching \u000A top-to-bottom in a hidden Emacs buffer that contains all of the \u000A documentation.  The topics are listed in the following order according to \u000A topic name:<p/> \u000A \u000A   <ol> \u000A \u000A   <li>All topics whose names reside in the \"<tt>ACL2</tt>\" package;</li> \u000A \u000A   <li>All topics whose names reside in the \"<tt>ACL2-PC</tt>\" package; and, \u000A   for the ACL2+Books Manual,</li> \u000A \u000A   <li>All other topics, sorted by <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> and then by <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt>.</li> \u000A \u000A   </ol><p/></li> \u000A \u000A <li>You may be queried, regarding whether you want to browse the ACL2+Books \u000A Manual, which is preferred, or the ACL2 User's Manual, which omits \u000A documentation for the books.  Both of these manuals are based on files that \u000A you will have if you are using a released version of ACL2 (after Version 6.3). \u000A But if you are using an <a href='http://acl2-devel.googlecode.com'>svn \u000A version</a>, then to use the ACL2+Books Manual you will need an extra file. \u000A You can build this file yourself, as described below but you may prefer to \u000A download it: for example, when you start ACL2-Doc, you may be given the option \u000A of downloading <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/manuals/current/rendered-doc-combined.lsp.gz'>a \u000A tarball for the latest ``bleeding edge'' copy</a> and extracting into \u000A directory <v>system/doc/</v> of your community books directory.  Indeed, the \u000A system will do all this for you if you answer <v>y</v> to that query. \u000A Alternatively, you can insist on a download of a ``bleeding edge'' version by \u000A using the `<tt>D</tt>' command.  However, if you prefer to browse the ACL2 \u000A User's Manual (without the books), you can put the following form into your \u000A <tt>~/.emacs</tt> file, above the form that loads the code for ACL2-Doc (see \u000A above).<p/> \u000A \u000A <code>(defvar *acl2-doc-top-default* 'TOP)</code> \u000A \u000A If you prefer to build <v>rendered-doc-combined.lsp</v> yourself, you can do so \u000A as follows.  For best results, use a directory separate from your regular ACL2 \u000A installation unless you already are using ACL2(h) (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).  Specifically, this separation can avoid subsequent \u000A problems when trying to include <see topic=\"ACL2____BOOKS\">books</see> with ACL2 that were certified \u000A using ACL2(h)).<p/> \u000A \u000A <ol> \u000A \u000A   <li>Build ACL2(h): \u000A   <code>make large ACL2_HONS=h</code></li> \u000A \u000A   <li>Build the manual, optionally supplying your \"<tt>make</tt>\" command \u000A   with a \"<tt>-j</tt>\" argument.  If \"<tt>acl2h</tt>\" invokes the \u000A   ACL2(h) executable that you just built, then you may omit \u000A   \"<tt>ACL2=acl2h</tt>\" below; otherwise replace \"<tt>acl2h</tt>\" by a \u000A   full pathname for that executable. \u000A   <code>cd books\u000Amake all USE_QUICKLISP=1 ACL2_BOOK_CERTS=doc/top.cert ACL2=acl2h</code></li> \u000A \u000A   <li>Build the file <tt>books/system/doc/rendered-doc-combined.lsp</tt> as \u000A   follows, still standing in the <tt>books</tt> directory, perhaps modifying \u000A   \"<tt>ACL2=acl2h</tt>\" as discussed above. \u000A   <code>cd books\u000Amake doc/top.cert USE_QUICKLISP=1 ACL2=acl2h</code></li> \u000A \u000A </ol></li></ul>"],

 "COMMON-LISP____EQUAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v> is equal to <v>t</v> or <v>nil</v>, according to \u000A whether or not <v>x</v> and <v>y</v> are the same value.</p> \u000A \u000A <p>For a discussion of the various idioms for testing against 0, See <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p>"],

 "COMMON-LISP____CDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-cdr</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A        ((<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A        (t nil)))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</code> \u000A \u000A <p>Notice that in the ACL2 logic, <v>cdr</v> returns <v>nil</v> for every <see topic=\"COMMON-LISP____ATOM\">atom</see>.</p>"],

 "COMMON-LISP____CAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-car</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A        ((<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A        (t nil)))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</code> \u000A \u000A <p>Notice that in the ACL2 logic, <v>car</v> returns <v>nil</v> for every <see topic=\"COMMON-LISP____ATOM\">atom</see>.</p>"],

 "ACL2____ZERO-TEST-IDIOMS":[["Programming"],"ACL2 Sources","ACL2","<p>Below are six commonly used idioms for testing whether <v>x</v> is \u000A <v>0</v>.  <tt><see topic=\"ACL2____ZIP\">Zip</see></tt> and <tt><see topic=\"ACL2____ZP\">zp</see></tt> are the preferred termination tests for \u000A recursions down the integers and naturals, respectively.</p> \u000A \u000A <code>idiom       logical              guard                 primary\u000A            meaning                                 compiled code*\u000A\u000A(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)          t                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)\u000A\u000A(<see topic=\"COMMON-LISP____EQL\">eql</see> x 0)   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)          t                   (<see topic=\"COMMON-LISP____EQL\">eql</see> x 0)\u000A\u000A(<see topic=\"COMMON-LISP____ZEROP\">zerop</see> x)   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)          x is a number       (<see topic=\"COMMON-LISP_____D3\">=</see> x 0)\u000A\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> x 0)     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)          x is a number       (<see topic=\"COMMON-LISP_____D3\">=</see> x 0)\u000A\u000A(<see topic=\"ACL2____ZIP\">zip</see> x)     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____IFIX\">ifix</see> x) 0)   x is an integer     (<see topic=\"COMMON-LISP_____D3\">=</see> x 0)\u000A\u000A(<see topic=\"ACL2____ZP\">zp</see> x)      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> x) 0)   x is a natural      (<see topic=\"ACL2____INT_D3\">int=</see> x 0)\u000A\u000A(<see topic=\"ACL2____ZPF\">zpf</see> x)     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> x) 0)   x is a fixnum &gt;= 0  (<see topic=\"COMMON-LISP____EQL\">eql</see> (the-fixnum x) 0)</code> \u000A \u000A <p>*See <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>, especially the subsection titled \u000A ``Guards and evaluation V: efficiency issues''.  Primary code is relevant only \u000A if <see topic=\"ACL2____GUARD\">guard</see>s are verified.  The ``compiled code'' shown is only \u000A suggestive.</p> \u000A \u000A <p>The first four idioms all have the same logical meaning and differ only \u000A with respect to their executability and efficiency.  In the absence of \u000A compiler optimizing, <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x 0)</v> is probably the most efficient, <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x\u000A 0)</v> is probably the least efficient, and <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> x 0)</v> is in between. \u000A However, an optimizing compiler could always choose to compile <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x\u000A 0)</v> as <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> x 0)</v> and, in situations where <v>x</v> is known at \u000A compile-time to be numeric, <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> x 0)</v> as <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x 0)</v>.  So efficiency \u000A considerations must, of course, be made in the context of the host \u000A compiler.</p> \u000A \u000A <p>Note also that <v>(<see topic=\"COMMON-LISP____ZEROP\">zerop</see> x)</v> and <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x 0)</v> are indistinguishable.  They \u000A have the same meaning and the same <see topic=\"ACL2____GUARD\">guard</see>, and can reasonably be \u000A expected to generate equally efficient code.</p> \u000A \u000A <p>Note that <v>(<see topic=\"ACL2____ZIP\">zip</see> x)</v> and <v>(<see topic=\"ACL2____ZP\">zp</see> x)</v> do not have the same logical \u000A meanings as the others or each other.  They are not simple tests for equality \u000A to <v>0</v>.  They each coerce <v>x</v> into a restricted domain, <tt><see topic=\"ACL2____ZIP\">zip</see></tt> to \u000A the integers and <tt><see topic=\"ACL2____ZP\">zp</see></tt> to the natural numbers, choosing <v>0</v> for <v>x</v> \u000A when <v>x</v> is outside the domain.  Thus, <v>1/2</v>, <v>#c(1 3)</v>, and \u000A <v>'abc</v>, for example, are all ``recognized'' as zero by both <tt><see topic=\"ACL2____ZIP\">zip</see></tt> and \u000A <tt><see topic=\"ACL2____ZP\">zp</see></tt>.  But <tt><see topic=\"ACL2____ZIP\">zip</see></tt> reports that <v>-1</v> is different from <v>0</v> \u000A while <tt><see topic=\"ACL2____ZP\">zp</see></tt> reports that <v>-1</v> ``is'' <v>0</v>.  More precisely, <v>(<see topic=\"ACL2____ZIP\">zip</see>\u000A -1)</v> is <v>nil</v> while <v>(<see topic=\"ACL2____ZP\">zp</see> -1)</v> is <v>t</v>.</p> \u000A \u000A <p>Note that the last five idioms all have <see topic=\"ACL2____GUARD\">guard</see>s that restrict their \u000A Common Lisp executability.  If these last five are used in situations in which \u000A <see topic=\"ACL2____GUARD\">guard</see>s are to be verified, then proof obligations are incurred as the \u000A price of using them.  If guard verification is not involved in your project, \u000A then the first five can be thought of as synonymous.</p> \u000A \u000A <p><tt><see topic=\"ACL2____ZIP\">Zip</see></tt> and <tt><see topic=\"ACL2____ZP\">zp</see></tt> are not provided by Common Lisp but are \u000A ACL2-specific functions.  Why does ACL2 provide these functions?  The answer \u000A has to do with the admission of recursively defined functions and efficiency. \u000A <tt><see topic=\"ACL2____ZP\">Zp</see></tt> is provided as the zero-test in situations where the controlling \u000A formal parameter is understood to be a natural number.  <tt><see topic=\"ACL2____ZIP\">Zip</see></tt> is \u000A analogously provided for the integer case.  We illustrate below.</p> \u000A \u000A <p>Here is an admissible definition of factorial</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>Observe the classic recursion scheme: a test against <v>0</v> and recursion \u000A by <tt><see topic=\"COMMON-LISP____1-\">1-</see></tt>.  Note however that the test against <v>0</v> is expressed with \u000A the <tt><see topic=\"ACL2____ZP\">zp</see></tt> idiom.  Note also the absence of a <see topic=\"ACL2____GUARD\">guard</see> making explicit \u000A our intention that <v>n</v> is a natural number.</p> \u000A \u000A <p>This definition of factorial is readily admitted because when <v>(<see topic=\"ACL2____ZP\">zp</see>\u000A n)</v></p> \u000A \u000A <p>is false (i.e., <v>nil</v>) then <v>n</v> is a natural number other than <v>0</v> \u000A and so <v>(<see topic=\"COMMON-LISP____1-\">1-</see> n)</v> is less than <v>n</v>.  The base case, where <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> is \u000A true, handles all the ``unexpected'' inputs, such as arise with <v>(fact -1)</v> \u000A and <v>(fact 'abc)</v>.  When calls of <v>fact</v> are evaluated, <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> \u000A checks <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)</v> and <v>(<see topic=\"COMMON-LISP_____E3\">&gt;</see> n 0)</v>.  <see topic=\"ACL2____GUARD\">Guard</see> verification is \u000A unsuccessful for this definition of <v>fact</v> because <tt><see topic=\"ACL2____ZP\">zp</see></tt> requires its \u000A argument to be a natural number and there is no <see topic=\"ACL2____GUARD\">guard</see> on <v>fact</v>, \u000A above.  Thus the primary raw lisp for <v>fact</v> is inaccessible and only the \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> definition (which does runtime ``type'' checking) is used \u000A in computation.  In summary, this definition of factorial is easily admitted \u000A and easily manipulated by the prover but is not executed as efficiently as it \u000A could be.</p> \u000A \u000A <p>Runtime efficiency can be improved by adding a <see topic=\"ACL2____GUARD\">guard</see> to the \u000A definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>This <see topic=\"ACL2____GUARD\">guard</see>ed definition has the same termination conditions as \u000A before -- termination is not sensitive to the <see topic=\"ACL2____GUARD\">guard</see>.  But the <see topic=\"ACL2____GUARD\">guard</see>s can be verified.  This makes the primary raw lisp definition \u000A accessible during execution.  In that definition, the <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> above is \u000A compiled as <v>(<see topic=\"COMMON-LISP_____D3\">=</see> n 0)</v>, because <v>n</v> will always be a natural number when \u000A the primary code is executed.  Thus, by adding a <see topic=\"ACL2____GUARD\">guard</see> and verifying \u000A it, the elegant and easily used definition of factorial is also efficiently \u000A executed on natural numbers.</p> \u000A \u000A <p>Now let us consider an alternative definition of factorial in which <v>(<see topic=\"COMMON-LISP_____D3\">=</see> n\u000A 0)</v> is used in place of <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> n 0) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>This definition does not terminate.  For example <v>(fact -1)</v> gives rise \u000A to a call of <v>(fact -2)</v>, etc.  Hence, this alternative is inadmissible.  A \u000A plausible response is the addition of a <see topic=\"ACL2____GUARD\">guard</see> restricting <v>n</v> to the \u000A naturals:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0))))\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> n 0) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))</code> \u000A \u000A <p>But because the termination argument is not sensitive to the <see topic=\"ACL2____GUARD\">guard</see>, \u000A it is still impossible to admit this definition.  To influence the termination \u000A argument one must change the conditions tested.  Adding a runtime test that \u000A <v>n</v> is a natural number would suffice and allow both admission and <see topic=\"ACL2____GUARD\">guard</see> verification.  But such a test would slow down the execution of the \u000A compiled function.</p> \u000A \u000A <p>The use of <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> as the test avoids this dilemma.  <tt><see topic=\"ACL2____ZP\">Zp</see></tt> \u000A provides the logical equivalent of a runtime test that <v>n</v> is a natural \u000A number but the execution efficiency of a direct <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt> comparison with \u000A <v>0</v>, at the expense of a <see topic=\"ACL2____GUARD\">guard</see> conjecture to prove.  In addition, if \u000A <see topic=\"ACL2____GUARD\">guard</see> verification and most-efficient execution are not needed, then \u000A the use of <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> allows the admission of the function without a <see topic=\"ACL2____GUARD\">guard</see> or other extraneous verbiage.</p> \u000A \u000A <p>While general rules are made to be broken, it is probably a good idea to \u000A get into the habit of using <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> as your terminating ``<v>0</v> test'' \u000A idiom when recursing down the natural numbers.  It provides the logical power \u000A of testing that <v>n</v> is a non-<v>0</v> natural number and allows efficient \u000A execution.</p> \u000A \u000A <p>We now turn to the analogous function, <tt><see topic=\"ACL2____ZIP\">zip</see></tt>.  <tt><see topic=\"ACL2____ZIP\">Zip</see></tt> is the \u000A preferred <v>0</v>-test idiom when recursing through the integers toward <v>0</v>. \u000A <tt><see topic=\"ACL2____ZIP\">Zip</see></tt> considers any non-integer to be <v>0</v> and otherwise just \u000A recognizes <v>0</v>.  A typical use of <tt><see topic=\"ACL2____ZIP\">zip</see></tt> is in the definition of \u000A <tt><see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see></tt>, shown below.  (ACL2 can actually accept this \u000A definition, but only after appropriate lemmas have been proved.)</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> integer-length (i)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZIP\">zip</see> i)\u000A      0\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> i -1)\u000A      0\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> i 2))))))</code> \u000A \u000A <p>Observe that the function recurses by <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see> i 2)</v>.  Hence, calling the \u000A function on <v>25</v> causes calls on <v>12</v>, <v>6</v>, <v>3</v>, <v>1</v>, and \u000A <v>0</v>, while calling it on <v>-25</v> generates calls on <v>-13</v>, <v>-7</v>, \u000A <v>-4</v>, <v>-2</v>, and <v>-1</v>.  By making <v>(<see topic=\"ACL2____ZIP\">zip</see> i)</v> the first test, we \u000A terminate the recursion immediately on non-integers.  The <see topic=\"ACL2____GUARD\">guard</see>, if \u000A present, can be verified and allows the primary raw lisp definition to check \u000A <v>(<see topic=\"COMMON-LISP_____D3\">=</see> i 0)</v> as the first terminating condition (because the primary code is \u000A executed only on integers).</p>"],

 "ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>For a general introduction to ACL2(p), an experimental extension of \u000A ACL2 that supports parallel execution and proof, see <see topic=\"ACL2____PARALLELISM\">parallelism</see>. \u000A Please note that although this extension is usable and, we hope, robust in its \u000A behavior, there are still known issues to address beyond those listed \u000A explicitly below.  While we expect ACL2(p) to perform correctly, it may never \u000A have the same level of attention to correctness as is given to ACL2; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>, specifically the ``IMPORTANT NOTE'' there.</p> \u000A \u000A <p>Below we list proof features of ACL2 that are not yet supported when \u000A parallel execution is enabled for the primary ACL2 proof process, generally \u000A known as ``the waterfall'', typically by calling <tt><see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see></tt>.</p> \u000A \u000A <p>Please note that this topic is limited to the case that such waterfall \u000A parallelism is enabled.  We believe that all ACL2 proof procedures are \u000A supported when waterfall parallelism is disabled, even in executables that \u000A support parallelism (see <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see>).</p> \u000A \u000A <p>Without a trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>): We support <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>s, <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>, and <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see> \u000A that do not modify <see topic=\"ACL2____STATE\">state</see>, but we do not permit <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, \u000A regardless of whether they modify state.  With a trust tag, the user can use \u000A <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>s that modify state and can also use <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED\">set-waterfall-parallelism-hacks-enabled</see> for a \u000A convenient mechanism for adding a trust tag).  See <see topic=\"ACL2____ERROR-TRIPLES-AND-PARALLELISM\">error-triples-and-parallelism</see> for a discussion of how to avoid modifying \u000A state in such situations.  Regardless of whether a trust tag is active: We do \u000A not support checkers of <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see> to be anything but the \u000A default checker.</p> \u000A \u000A <p>GNU Make versions 3.81 and 3.82 formerly caused a lot of problems (version \u000A 3.80 somewhat less so), at least on Linux, when certifying books with ACL2 \u000A built on a host Lisp of CCL using `make'.  CCL was updated around March 23, \u000A 2011 to fix this problem, so if you get segfaults (for example) with CCL, try \u000A updating your CCL installation.</p> \u000A \u000A <p>Book certification should generally work but may present some issues, \u000A including the following.</p> \u000A \u000A <blockquote><p>o The standard `make'-based process for book certification will \u000A not use <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see>, which is disabled by default (even when \u000A <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see> by using the <v>ACL2_PAR</v> flag).  See <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see> and see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>, which explain \u000A that <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see> files are ignored during that process unless \u000A specified explicitly on the command line or in the environment.</p> \u000A \u000A <p>o A book certified with ACL2(p) might subsequently cause an error when \u000A included with ACL2.  As of this writing, however, we have only seen this for a \u000A book in which <tt><see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see></tt> is used.</p> \u000A \u000A <p>o In general, ACL2(p) is primarily intended to support more rapid \u000A interactive development.  While we are unaware of an unsoundness likely to \u000A affect an ACL2(p) user, we suggest using ACL2 for final book certification, \u000A rather than ACL2(p), to lower the risk of unsound book \u000A certification.</p></blockquote> \u000A \u000A <p>Proof output can contain repeated printing of the same subgoal name.</p> \u000A \u000A <p><see topic=\"ACL2____GAG-MODE\">Gag-mode</see> isn't officially supported, although it has proved helpful \u000A to use ACL2(p) in conjunction with <v>(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> t)</v> (because this setting \u000A suppresses the output that occurs outside the waterfall).  This being said, \u000A ACL2(p) also prints key checkpoints (for example see <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>), but with a notion of ``key checkpoint'' \u000A that does not take into account whether the goal is later proved by induction. \u000A See <see topic=\"ACL2____ACL2P-KEY-CHECKPOINTS\">ACL2p-key-checkpoints</see> for further explanation of these key \u000A checkpoints.  Note that <tt><see topic=\"ACL2____PSO\">pso</see></tt> is also not supported.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt> utility is not supported.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> utility is not supported.</p> \u000A \u000A <p>Tracking for <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> is not supported (see <see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>).</p> \u000A \u000A <p>Time limits (see <see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>) aren't supported.</p> \u000A \u000A <p>The timing information printed at the end of a proof attempt, which is \u000A intended to represent cpu time (not wall-clock time), may be somewhat \u000A inaccurate when <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see> is non-<v>nil</v>.  Consider using \u000A <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> to obtain timing information.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>s is not recommended, as there may be race \u000A conditions.</p> \u000A \u000A <p>Output specific to <v>:OR</v> <see topic=\"ACL2____HINTS\">hints</see> is disabled.</p> \u000A \u000A <p>Proof trees are likely not to work as originally designed.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt> may not fully inhibit proof \u000A output.</p> \u000A \u000A <p>Reporting of <see topic=\"ACL2____SPLITTER\">splitter</see> rules is currently unsupported when \u000A waterfall-parallelism is on.</p> \u000A \u000A <p>Interrupting a proof attempt is not yet properly supported.  At a minimum, \u000A interrupts are trickier with waterfall parallelism enabled.  For one, the user \u000A typically needs to issue the interrupt twice before the proof attempt is \u000A actually interrupted.  Additionally, on rare occasions the theorem is \u000A registered as proved, even though the prover did not finish the proof.  If \u000A this occurs, issue a <v>:u</v> (see <see topic=\"ACL2____UBT\">ubt</see>) and you will likely be at a \u000A stable state.</p> \u000A \u000A <p>Also with regards to interrupting a proof attempt, sometimes the user may \u000A need to issue a <v>:q</v> and <v>lp</v> to reset properly the parallelism \u000A implementation to a stable state.  The primary symptom that the user is \u000A experiencing this issue is that threads will continue to compute in the \u000A background, even though there should be no proof attempt in progress.  The \u000A user can observe this symptom by examining the CPU utilization of their ACL2 \u000A process, for example on Linux/Unix with the shell process <v>top</v>.  Lisp \u000A usage greater than a few percent is indicative of this problem.</p> \u000A \u000A <p>Because of how ACL2 <see topic=\"ACL2____ARRAYS\">arrays</see> are designed, the user may find that, in \u000A practice, ACL2 arrays work (but perhaps with some <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see> \u000A messages).  However, we are aware of race conditions that can cause \u000A problems.</p> \u000A \u000A <p>Instead of dynamically monitoring rewrites, <see topic=\"ACL2____DMR\">dmr</see> instead dynamically \u000A outputs information helpful for debugging the performance of proof \u000A parallelism.  The instructions concerning how to see this debugging \u000A information are the same as the instructions for enabling <see topic=\"ACL2____DMR\">dmr</see> mode.</p> \u000A \u000A <p>If you are working with LispWorks 6.0 or 6.0.1, then you may see messages \u000A about misaligned conses.  The state of the system may be corrupted after such \u000A a message has been printed.  This LispWorks bug is fixed in LispWorks 6.1.</p> \u000A \u000A <p>The waterfall parallelism mode <v>:resource-and-timing-based</v> is not fully \u000A supported when the host Lisp is other than CCL.  It may work, but we have not \u000A attempted to address a potential race condition.</p> \u000A \u000A <p>Proof output for splitter rules (see <see topic=\"ACL2____SPLITTER\">splitter</see>) is currently \u000A unsupported when waterfall-parallelism is enabled.</p> \u000A \u000A <p>(Comment for ACL2(h) users; see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.)  Memoization \u000A may not work as intended when executing in parallel (including the waterfall). \u000A In an effort to be helpful to the user, the functions automatically memoized \u000A by ACL2(h) are unmemoized when setting waterfall parallelism to anything but \u000A <v>nil</v>.  Those exact functions are again memoized once waterfall parallelism \u000A is disabled.  Additionally, any functions memoized within the ACL2 loop (by a \u000A call of <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>) are also unmemoized when enabling waterfall \u000A parallelism and once again memoized when disabling waterfall parallelism. \u000A This is implemented by returning the memoization state to what it was before \u000A enabling waterfall parallelism.  As such, the user should be aware that any \u000A changes made to the memoization state while waterfall parallelism is enabled \u000A will be lost once waterfall parallelism is disabled.</p>"],

 "ACL2____TIME-TRACKER":[["Debugging","Time$"],"ACL2 Sources","ACL2","<p>The <v>time-tracker</v> macro is a utility for displaying time spent \u000A during specified evaluation.  In general, the user provides this \u000A specification.  However, ACL2 itself uses this utility for tracking uses of \u000A its <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see> reasoning utility (see <see topic=\"ACL2____TIME-TRACKER-TAU\">time-tracker-tau</see>).  We \u000A discuss that use as an example before discussing the general form for calls of \u000A <v>time-tracker</v>.</p> \u000A \u000A <p>Note that by default, the time being tracked is runtime (cpu time); to \u000A switch to realtime (elapsed time), see <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.</p> \u000A \u000A <p>Remark for ACL2(p) users (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>): <v>time-tracker</v> is \u000A merely a no-op in ACL2(p).</p> \u000A \u000A <p>During the development of the <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>, we were concerned about \u000A the possibility that it would slow down proofs without any indication of how \u000A one might avoid the problem.  We wanted a utility that would alert the user in \u000A such situations.  However, the tau-system code does not return <see topic=\"ACL2____STATE\">state</see>, \u000A so we could not track time spent in the state.  We developed the \u000A <v>time-tracker</v> utility to track time and print messages, and we did it in a \u000A general way so that others can use it in their own code.  Here is an example \u000A of such a message that could be printed during a proof.</p> \u000A \u000A <code>TIME-TRACKER-NOTE [:TAU]: Elapsed runtime in tau is 2.58 secs; see\u000A:DOC time-tracker-tau.</code> \u000A \u000A <p>And here is an example of such a message that could be printed at the end \u000A of the proof.</p> \u000A \u000A <code>TIME-TRACKER-NOTE [:TAU]: For the proof above, the total time spent\u000Ain the tau system was 20.29 seconds.  See :DOC time-tracker-tau.</code> \u000A \u000A <p>The <v>time-tracker</v> utility tracks computation time spent on behalf of a \u000A user-specified ``tag''.  In the case of the tau-system, we chose the tag, \u000A <v>:tau</v>.  The first argument of <v>time-tracker</v> is the tag, which in our \u000A running example is always <v>:tau</v>.  Note that although all arguments of \u000A <v>time-tracker</v> are evaluated, the first argument is typically a keyword and \u000A the second is always a keyword, and such arguments evaluate to themselves.</p> \u000A \u000A <p>An ACL2 function invoked at the start of a proof includes approximately the \u000A following code.</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_42\">progn$</see>\u000A (<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :end)\u000A (<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :init\u000A               :times '(1 2 3 4 5)\u000A               :interval 5\u000A               :msg &quot;Elapsed runtime in tau is ~st secs; see :DOC ~\u000A                     time-tracker-tau.~|~%&quot;)\u000A ...)</code> \u000A \u000A <p>The first <v>time-tracker</v> call (above) ends any existing time-tracking \u000A for tag <v>:tau</v>.  One might have expected it be put into code managing the \u000A proof summary, but we decided not to rely on that code being executed, say, in \u000A case of an interrupt.  When a given tag is not already being time-tracked, \u000A then <v>:end</v> is a no-op (rather than an error).</p> \u000A \u000A <p>The second <v>time-tracker</v> call (above) initiates time-tracking for the \u000A tag, <v>:tau</v>.  Moreover, it specifies the effect of the <v>:print?</v> \u000A keyword.  Consider the following abbreviated definition from the ACL2 source \u000A code.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> tau-clausep-lst-rec (clauses ens wrld ans ttree state calist)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____ENDP\">endp</see> clauses)\u000A    (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> ans nil) ttree calist))\u000A   (t (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A       (flg1 ttree1 calist)\u000A       (tau-clausep (<see topic=\"COMMON-LISP____CAR\">car</see> clauses) ens wrld state calist)\u000A       (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :print?)\u000A               (tau-clausep-lst-rec (<see topic=\"COMMON-LISP____CDR\">cdr</see> clauses) ...))))))</code> \u000A \u000A <p>Notice that <v>(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :print?)</v> is executed immediately after \u000A <v>tau-clausep</v> is called.  The idea is to check whether the total time \u000A elapsed inside the tau-system justifies printing a message to the user.  The \u000A specification of <v>:times '(1 2 3 4 5)</v> in the <v>:init</v> form above says \u000A that a message should be printed after 1 second, after 2 seconds, ..., and \u000A after 5 seconds.  Thereafter, the specification of <v>:interval 5</v> in the \u000A <v>:init</v> form above says that each time we print, at least 5 additional \u000A seconds should have been tracked before <v>(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :print?)</v> \u000A prints again.  Finally, the <v>:msg</v> keyword above specifies just what should \u000A be printed.  If it is omitted, then a reasonable default message is printed \u000A (as discussed below), but in this case we wanted to print a custom message. \u000A The <v>:msg</v> string above is what is printed using formatted printing (see \u000A <see topic=\"ACL2____FMT\">fmt</see>), where the character <v>#\\t</v> is bound to a string giving a \u000A decimal representation with two decimal points of the time tracked so far for \u000A tag <v>:tau</v>.  (As our general description below points out, <v>:msg</v> can \u000A also be a ``message'' list rather than a string.)</p> \u000A \u000A <p>But when is time actually tracked for <v>:tau</v>?  Consider the following \u000A definition from the ACL2 source code.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> tau-clausep-lst (clauses ens wrld ans ttree state calist)\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :start)\u000A          (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A           (clauses ttree calist)\u000A           (tau-clausep-lst-rec clauses ens wrld ans ttree state calist)\u000A           (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :stop)\u000A                   (<see topic=\"ACL2____MV\">mv</see> clauses ttree calist)))))</code> \u000A \u000A <p>The two calls of <v>time-tracker</v> above first start, and then stop, \u000A time-tracking for the tag, <v>:tau</v>.  Thus, time is tracked during evaluation \u000A of the call of <v>tau-clausep-lst-rec</v>, which is the function (discussed \u000A above) that does the <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>'s work.</p> \u000A \u000A <p>Finally, as noted earlier above, ACL2 may print a time-tracking message for \u000A tag <v>:tau</v> at the end of a proof.  The ACL2 function <v>print-summary</v> \u000A contains essentially the following code.</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> :tau :print?\u000A              :min-time 1\u000A              :msg &quot;For the proof above, the total runtime ~\u000A                    spent in the tau system was ~st seconds.  ~\u000A                    See :DOC time-tracker-tau.~|~%&quot;)</code> \u000A \u000A <p>The use of <v>:min-time</v> says that we are to ignore the <v>:times</v> and \u000A <v>:interval</v> established by the <v>:init</v> call described above, and \u000A instead, print a message if and only if at least 1 second (since 1 is the \u000A value of keyword <v>:min-time</v>) has been tracked for tag <v>:tau</v>. \u000A Formatted printing (see <see topic=\"ACL2____FMT\">fmt</see>) is used for the value of <v>:msg</v>, where \u000A the character <v>#\\t</v> is bound to a decimal string representation of the \u000A time in seconds, as described above.</p> \u000A \u000A <p>The example above covers all legal values for the second argument of \u000A <v>time-tracker</v> and discusses all the additional legal keyword arguments. \u000A We conclude with a precise discussion of all arguments.  Note that all \u000A arguments are evaluated; thus when we refer to an argument, we are discussing \u000A the value of that argument.  All times discussed are runtimes, i.e., cpu \u000A times, unless that default is changed; see <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.</p> \u000A \u000A <code>General forms:\u000A\u000A(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> t)        ; enable time-tracking (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A\u000A(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> nil)      ; disable time-tracking\u000A\u000A(<see topic=\"ACL2____TIME-TRACKER\">time-tracker</see> tag       ; a symbol other than t or nil\u000A              option    ; :init, :end, :start, :stop, or :print?\u000A              ;; keyword arguments:\u000A              :times    ; non-nil if and only if option is :init\u000A              :interval ; may only be non-nil with :init option\u000A              :min-time ; may only be non-nil with :print? option\u000A              :msg      ; may only be non-nil with :init and :print? options</code> \u000A \u000A <p>Time-tracking is enabled by default.  If the first argument is <v>t</v> or \u000A <v>nil</v>, then no other arguments are permitted and time-tracking is enabled \u000A or disabled, respectively.  When time-tracking is disabled, nothing below \u000A takes place.  Thus we assume time-tracking is enabled for the remainder of \u000A this discussion.  We also assume below that the first argument is neither \u000A <v>t</v> nor <v>nil</v>.</p> \u000A \u000A <p>We introduce some basic notions about time-tracking.  A given tag, such as \u000A <v>:tau</v> in the example above, might or might not currently be ``tracked'': \u000A <v>:init</v> causes the specified tag to be tracked, while <v>:end</v> causes the \u000A specified tag not to be tracked.  If the tag is being tracked, the tag might \u000A or might not be ``active'': <v>:start</v> causes the tag to be in an active \u000A state, whie <v>:stop</v> causes the tag not to be active.  Note that only \u000A tracked tags can be in an active or inactive state.  For a tag that is being \u000A tracked, the ``accumulated time'' is the total time spent in an active state \u000A since the time that the tag most recently started being tracked, and the \u000A ``checkpoint list'' is a non-empty list of rational numbers specifying when \u000A printing may take place, as described below.</p> \u000A \u000A <p>We now consider each legal value for the second argument, or ``option'', \u000A for a call of <v>time-tracker</v> on a given tag.</p> \u000A \u000A <p><v>:Init</v> specifies that the tag is to be tracked.  It also establishes \u000A defaults for the operation of <v>:print?</v>, as described below, using the \u000A <v>:times</v>, <v>:interval</v>, and <v>:msg</v> keywords.  Of these three, only \u000A <v>:times</v> is required, and its value must be a non-empty list of rational \u000A numbers specifying the initial checkpoint list for the tag.  It is an error to \u000A specify <v>:init</v> if the tag is already being tracked.  (So if you don't care \u000A whether or not the tag is already being tracked and you want to initiate \u000A tracking for that tag, use <v>:end</v> first.)</p> \u000A \u000A <p><v>:End</v> specifies that if the tag is being tracked, then it should nstop \u000A being tracked.  If the tag is not being tracked, then <v>:end</v> has no \u000A effect.</p> \u000A \u000A <p><v>:Start</v> specifies that the tag is to be active.  It is an error to \u000A specify <v>:start</v> if the tag is not being tracked or is already active.</p> \u000A \u000A <p><v>:Stop</v> specifies that the tag is to stop being active.  It is an error \u000A to specify <v>:stop</v> if the tag is not being tracked or is not active.</p> \u000A \u000A <p><v>:Print?</v> specifies that if the tag is being tracked (not necessarily \u000A active), then a message should be printed if a suitable condition is met.  The \u000A nature of that message and that condition depend on the keyword options \u000A supplied with <v>:print?</v> as well as those supplied with the <v>:init</v> \u000A option that most recently initiated tracking.  <v>:Print?</v> has no effect if \u000A the tag is not being tracked, except that if certain keyword values are \u000A checked if supplied with <v>:print?</v>: <v>:min-time</v> must be a rational \u000A number or <v>nil</v>, and <v>:msg</v> must be either a string, a true-list whose \u000A <v>car</v> is a string, or <v>nil</v>.  The remainder of this documentation \u000A describes the <v>:print?</v> option in detail under the assumption that the tag \u000A is being tracked: first, giving the conditions under which it causes a message \u000A to be printed, and second, explaining what is printed.</p> \u000A \u000A <p>When <v>:print?</v> is supplied a non-<v>nil</v> value of <v>:min-time</v>, that \u000A value must be a rational number, in which case a message is printed if the \u000A accumulated time for the tag is at least that value.  Otherwise a message is \u000A printed if the accumulated time is greater than or equal to the <v>car</v> of \u000A the checkpoint list for the tag.  In that case, the tracking state for the tag \u000A is updated in the following two ways.  First, the checkpoint list is scanned \u000A from the front and as long as the accumulated time is greater than or equal to \u000A the <v>car</v> of the remaining checkpoint list, that <v>car</v> is popped off the \u000A checkpoint list.  Second, if the checkpoint list has been completely emptied \u000A and a non-<v>nil</v> <v>:interval</v> was supplied when tracking was most recently \u000A initiated with the <v>:init</v> option, then the checkpoint list is set to \u000A contain a single element, namely the sum of the accumulated time with that \u000A value of <v>:interval</v>.</p> \u000A \u000A <p>Finally, suppose the above criteria are met, so that <v>:print?</v> results \u000A in printing of a message.  We describe below the message, <v>msg</v>, that is \u000A printed.  Here is how it is printed (see <see topic=\"ACL2____FMT\">fmt</see>), where \u000A <v>seconds-as-decimal-string</v> is a string denoting the number of seconds of \u000A accumulated time for the tag, with two digits after the decimal point.</p> \u000A \u000A <code>(<see topic=\"ACL2____FMS\">fms</see> &quot;TIME-TRACKER-NOTE [~x0]: ~@1~|&quot;\u000A     (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0 tag)\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> #1 msg)\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> #t seconds-as-decimal-string))\u000A     (<see topic=\"ACL2____PROOFS-CO\">proofs-co</see> state) state nil)</code> \u000A \u000A <p>The value of <v>msg</v> is the value of the <v>:msg</v> keyword supplied with \u000A <v>:print?</v>, if non-<v>nil</v>; else, the value of <v>:msg</v> supplied when most \u000A recently initialization with the <v>:init</v> option, if non-<v>nil</v>; and \u000A otherwise, the string <v>&quot;~st s&quot;</v> (the final ``s'' abbreviating the word \u000A ``seconds'').  It is convenient to supply <v>:msg</v> as a call <v>(<see topic=\"ACL2____MSG\">msg</see> str\u000A arg-0 arg-1 ... arg-k)</v>, where <v>str</v> is a string and each <v>arg-i</v> is \u000A the value to be associated with <v>#\\i</v> upon formatted printing (as with \u000A <tt><see topic=\"ACL2____FMT\">fmt</see></tt>) of the string <v>str</v>.</p>"],

 "ACL2____THE-METHOD":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for a more \u000A detailed tutorial on how to prove theorems with ACL2.</p> \u000A \u000A <p>Many users develop proof scripts in an Emacs buffer and submit one event at \u000A a time to the theorem prover running in a <v>*shell*</v> buffer.  The script \u000A buffer is logically divided into two regions: the events that have been \u000A accepted by the theorem prover and those that have not yet been accepted.  An \u000A imaginary ``barrier'' divides these two regions.  The region above the barrier \u000A describes the state of the <v>*shell*</v> buffer (and ACL2's logical world). \u000A The region below the barrier is the ``to do'' list.</p> \u000A \u000A <p>We usually start a proof project by typing the key lemmas, and main goal \u000A into the to do list.  Definitions are here just regarded as theorems to prove \u000A (i.e., the measure conjectures).  Then we follow ``The Method.''</p> \u000A \u000A <p>(1) Think about the proof of the first theorem in the to do list. \u000A Structure the proof either as an induction followed by simplification or just \u000A simplification.  Have the necessary lemmas been proved? That is, are the \u000A necessary lemmas in the done list already?  If so, proceed to Step 2. \u000A Otherwise, add the necessary lemmas at the front of the to do list and repeat \u000A Step 1.</p> \u000A \u000A <p>(2) Call the theorem prover on the first theorem in the to do list and let \u000A the output stream into the *shell* buffer.  Abort the proof if it runs more \u000A than a few seconds.</p> \u000A \u000A <p>(3) If the theorem prover succeeded, advance the barrier past the \u000A successful command and go to Step 1.</p> \u000A \u000A <p>(4) Otherwise, inspect the failed proof attempt, starting from the \u000A beginning, not the end.  Basically you should look for the first place the \u000A proof attempt deviates from your imagined proof.  If your imagined proof was \u000A inductive, inspect the induction scheme used by ACL2.  If that is ok, then \u000A find the first subsequent subgoal that is stable under simplification and \u000A think about why it was not proved by the simplifier.  If your imagined proof \u000A was not inductive, then think about the first subgoal stable under \u000A simplification, as above.  Modify the script appropriately.  It usually means \u000A adding lemmas to the to do list, just in front of the theorem just tried.  It \u000A could mean adding hints to the current theorem.  In any case, after the \u000A modifications go to Step 1.</p> \u000A \u000A <p>We do not seriously suggest that this or any rotely applied algorithm will \u000A let you drive ACL2 to difficult proofs.  Indeed, to remind you of this we call \u000A this ``The Method'' rather than ``the method.''  That is, we are aware of the \u000A somewhat pretentious nature of any such advice.  But these remarks have helped \u000A many users approach ACL2 in a constructive and disciplined way.</p> \u000A \u000A <p>We say much more about The Method in the ACL2 book.  See the home page. \u000A Also see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> for a discussion of a way for ACL2 to help you to \u000A use The Method.  And again, see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for \u000A a more detailed tutorial.</p> \u000A \u000A <p>Learning to read failed proofs is a useful skill.  There are several kinds \u000A of ``checkpoints'' in a proof: (1) a formula to which induction is being (or \u000A would be) applied, (2) the first formula stable under simplification, (3) a \u000A formula that is possibly generalized, either by cross-fertilizing with and \u000A throwing away an equivalence hypothesis or by explicit generalization of a \u000A term with a new variable.</p> \u000A \u000A <p>At the induction checkpoint, confirm that you believe the formula being \u000A proved is a theorem and that it is appropriately strong for an inductive \u000A proof.  Read the selected induction scheme and make sure it agrees with your \u000A idea of how the proof should go.</p> \u000A \u000A <p>At the post-simplification checkpoint, which is probably the most commonly \u000A seen, consider whether there are additional rewrite rules you could prove to \u000A make the formula simplify still further.  Look for compositions of function \u000A symbols you could rewrite.  Look for contradictions among hypotheses and prove \u000A the appropriate implications: for example, the checkpoint might contain the \u000A two hypotheses <v>(P (F A))</v> and <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"ACL2____Q\">Q</see> (G (F A))))</v> and you might \u000A realize that <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (<see topic=\"ACL2____Q\">q</see> (g x)))</v> is a theorem.  Look for signs that \u000A your existing rules did not apply, e.g., for terms that should have been \u000A rewritten, and figure out why they were not.  Possible causes include that \u000A they do not exactly match your old rules, that your old rules have hypotheses \u000A that cannot be relieved here -- perhaps because some other rules are missing, \u000A or perhaps your old rules are disabled.  If you cannot find any further \u000A simplifications to make in the formula, ask yourself whether it is valid.  If \u000A so, sketch a proof.  Perhaps the proof is by appeal to a combination of lemmas \u000A you should now prove?</p> \u000A \u000A <p>At the two generalization checkpoints --- where hypotheses are discarded or \u000A terms are replaced by variables --- ask yourself whether the result is a \u000A theorem.  It often is not.  Think about rewrite rules that would prove the \u000A formula.  These are often restricted versions of the overly-general formulas \u000A created by the system's heuristics.</p> \u000A \u000A <p>See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> for a discussion of a tool to help you navigate \u000A through ACL2 proofs.</p>"],

 "ACL2____STOBJ-EXAMPLE-1":[["Stobj"],"ACL2 Sources","ACL2","<p>Suppose we want to sweep a tree and (1) count the number of \u000A interior nodes, (2) count the number of tips and (3) keep a record of every \u000A tip we encounter that is an integer.  We could use a single-threaded object as \u000A our ``accumulator''.  Such an object would have three fields, one holding the \u000A number of nodes seen so far, one holding the number of tips, and one holding \u000A all the integer tips seen.</p> \u000A \u000A <p>The following event declares <v>counters</v> to be a single-threaded \u000A object.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> counters\u000A  (NodeCnt     :type integer :initially 0)\u000A  (TipCnt      :type integer :initially 0)\u000A  (IntTipsSeen :type t       :initially nil))</code> \u000A \u000A <p>It has three fields, <v>NodeCnt</v>, <v>TipCnt</v>, and <v>IntTipsSeen</v>.  (As \u000A always in ACL2, capitalization is irrelevant in simple symbol names, so the \u000A first name could be written <v>nodecnt</v> or <v>NODECNT</v>, etc.) Those are the \u000A name of the accessor functions for the object.  The corresponding update \u000A functions are named <v>update-NodeCnt</v>, <v>update-TipCnt</v> and \u000A <v>update-IntTipsSeen</v>.</p> \u000A \u000A <p>If you do not like the default function names chosen above, there is a \u000A feature in the <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event that allows you to specify other \u000A names.</p> \u000A \u000A <p>If you want to see the ACL2 definitions of all the functions defined by \u000A this event, look at <see topic=\"ACL2____STOBJ-EXAMPLE-1-DEFUNS\">stobj-example-1-defuns</see>.</p> \u000A \u000A <p>If, after this event, we evaluate the top-level ``global variable'' \u000A <v>counters</v> in the ACL2 read-eval-print loop we get:</p> \u000A \u000A <code>ACL2 !&gt;counters\u000A&lt;counters&gt;</code> \u000A \u000A <p>Note that the value printed is ``<v>&lt;counters&gt;</v>''.  Actually, the value of \u000A <v>counters</v> in the logic is <v>(0 0 NIL)</v>.  But ACL2 always prints \u000A single-threaded objects in this non-informative way because they are usually \u000A so big that to do otherwise would be unpleasant.</p> \u000A \u000A <p>Had you tried to evaluate the ``global variable'' <v>counters</v> before \u000A declaring it a single-threaded object, ACL2 would have complained that it does \u000A not support global variables.  So a lesson here is that once you have declared \u000A a new single-threaded object your top-level forms can reference it.  In \u000A versions of ACL2 prior to Version 2.4 the only variable enjoying this status \u000A was <v>STATE</v>.  single-threaded objects are a straightforward generalization \u000A of the long-implemented von Neumann <tt><see topic=\"ACL2____STATE\">state</see></tt> feature of ACL2.</p> \u000A \u000A <p>We can access the fields of <v>counters</v> as with:</p> \u000A \u000A <code>ACL2 !&gt;(NodeCnt counters)\u000A0\u000AACL2 !&gt;(IntTipsSeen counters)\u000ANIL</code> \u000A \u000A <p>and we can set the fields of <v>counters</v> as with:</p> \u000A \u000A <code>ACL2 !&gt;(update-NodeCnt 3 counters)\u000A&lt;counters&gt;\u000AACL2 !&gt;(NodeCnt counters)\u000A3</code> \u000A \u000A <p>Observe that when we evaluate an expression that returns a counter object, \u000A that object becomes the ``current value'' of <v>counters</v>.</p> \u000A \u000A <p>Here is a function that ``converts'' the <v>counters</v> object to its \u000A ``ordinary'' representation:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> show-counters (counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (NodeCnt counters)\u000A        (TipCnt counters)\u000A        (IntTipsSeen counters)))</code> \u000A \u000A <p>Observe that we <i>must</i> declare, at the top of the <v>defun</v>, that we \u000A mean to use the formal parameter <v>counters</v> as a single-threaded object! \u000A If we did not make this declaration, the body of <v>show-counters</v> would be \u000A processed as though <v>counters</v> were an ordinary object.  An error would be \u000A caused because the accessors used above cannot be applied to anything but the \u000A single-threaded object <v>counters</v>.  If you want to know why we insist on \u000A this declaration, see <see topic=\"ACL2____DECLARE-STOBJS\">declare-stobjs</see>.</p> \u000A \u000A <p>When <v>show-counters</v> is admitted, the following message is printed:</p> \u000A \u000A <code>Since SHOW-COUNTERS is non-recursive, its admission is trivial.  We\u000Aobserve that the type of SHOW-COUNTERS is described by the theorem\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (SHOW-COUNTERS COUNTERS))\u000A     (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (SHOW-COUNTERS COUNTERS))).\u000AWe used primitive type reasoning.\u000A\u000A(SHOW-COUNTERS COUNTERS) =&gt; *.\u000A\u000AThe guard conjecture for SHOW-COUNTERS is trivial to prove.\u000ASHOW-COUNTERS is compliant with Common Lisp.</code> \u000A \u000A <p>The line above containing the ``=&gt;'' is called the ``signature'' of \u000A <v>show-counters</v>; it conveys the information that the first argument is the \u000A single-threaded object <v>counters</v> and the only result is an ordinary \u000A object.  Here is an example of another signature:</p> \u000A \u000A <code>(PROCESSOR * * COUNTERS) =&gt; (<see topic=\"ACL2____MV\">MV</see> * COUNTERS)</code> \u000A \u000A <p>which indicates that the function <v>PROCESSOR</v> (which we haven't shown \u000A you) takes three arguments, the third of which is the <v>COUNTERS</v> stobj, and \u000A returns two results, the second of which is the modified <v>COUNTERS</v>.</p> \u000A \u000A <p>Returning to the admission of <v>show-counters</v> above, the last sentence \u000A printed indicates that the <tt><see topic=\"ACL2____GUARD\">guard</see></tt> conjectures for the function were \u000A proved.  When some argument of a function is declared to be a single-threaded \u000A object via the <v>xargs</v> <v>:stobj</v>, we automatically add (conjoin) to the \u000A guard the condition that the argument satisfy the recognizer for that \u000A single-threaded object.  In the case of <v>show-counters</v> the guard is \u000A <v>(countersp counters)</v>.</p> \u000A \u000A <p>Here is an example of <v>show-counters</v> being called:</p> \u000A \u000A <code>ACL2 !&gt;(show-counters counters)\u000A(3 0 NIL)</code> \u000A \u000A <p>This is what we would see had we set the <v>NodeCnt</v> field of the initial \u000A value of <v>counters</v> to <v>3</v>, as we did earlier in this example.</p> \u000A \u000A <p>We next wish to define a function to reset the <v>counters</v> object.  We \u000A could define it this way:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> reset-counters (counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((counters (update-NodeCnt 0 counters)))\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((counters (update-TipCnt 0 counters)))\u000A      (update-IntTipsSeen nil counters))))</code> \u000A \u000A <p>which ``successively'' sets the <v>NodeCnt</v> field to <v>0</v>, then the \u000A <v>TipCnt</v> field to <v>0</v>, and then the <v>IntTipsSeen</v> field to <v>nil</v> \u000A and returns the resulting object.</p> \u000A \u000A <p>However, the nest of <v>let</v> expressions is tedious and we use this \u000A definition instead.  This definition exploits a macro, here named ``<v>seq</v>'' \u000A (for ``sequentially'') which evaluates each of the forms given, binding their \u000A results successively to the stobj name given.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> reset-counters (counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (seq counters\u000A       (update-NodeCnt 0 counters)\u000A       (update-TipCnt 0 counters)\u000A       (update-IntTipsSeen nil counters)))</code> \u000A \u000A <p>This definition is syntactically identical to the one above, after macro \u000A expansion.  Our definition of <v>seq</v> is shown below and is not part of \u000A native ACL2.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> seq (<see topic=\"ACL2____STOBJ\">stobj</see> &amp;rest rst)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> rst) stobj)\u000A        ((<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> rst)) (<see topic=\"COMMON-LISP____CAR\">car</see> rst))\u000A        (t `(<see topic=\"COMMON-LISP____LET\">let</see> ((,stobj ,(<see topic=\"COMMON-LISP____CAR\">car</see> rst)))\u000A             (seq ,stobj ,@(<see topic=\"COMMON-LISP____CDR\">cdr</see> rst))))))</code> \u000A \u000A <p>The signature printed for <v>reset-counters</v> is</p> \u000A \u000A <code>(RESET-COUNTERS COUNTERS) =&gt; COUNTERS.</code> \u000A \u000A <p>Here is an example.</p> \u000A \u000A <code>ACL2 !&gt;(show-counters counters)\u000A(3 0 NIL)\u000AACL2 !&gt;(reset-counters counters)\u000A&lt;counters&gt;\u000AACL2 !&gt;(show-counters counters)\u000A(0 0 NIL)</code> \u000A \u000A <p>Here finally is a function that uses <v>counters</v> as a single-threaded \u000A accumulator to collect the desired information about the tree <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> sweep-tree (x counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A         (seq counters\u000A              (update-TipCnt (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (TipCnt counters)) counters)\u000A              (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                  (update-IntTipsSeen (<see topic=\"COMMON-LISP____CONS\">cons</see> x (IntTipsSeen counters))\u000A                                  counters)\u000A                counters)))\u000A        (t (seq counters\u000A                (update-NodeCnt (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (NodeCnt counters)) counters)\u000A                (sweep-tree (<see topic=\"COMMON-LISP____CAR\">car</see> x) counters)\u000A                (sweep-tree (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) counters)))))</code> \u000A \u000A <p>We can paraphrase this definition as follows.  If <v>x</v> is an atom, then \u000A increment the <v>TipCnt</v> field of <v>counters</v> and <i>then</i>, if <v>x</v> is \u000A an integer, add <v>x</v> to the <v>IntTipsSeen</v> field, and return \u000A <v>counters</v>.  On the other hand, if <v>x</v> is not an atom, then increment \u000A the <v>NodeCnt</v> field of <v>counters</v>, and <i>then</i> sweep the <v>car</v> of \u000A <v>x</v> and <i>then</i> sweep the <v>cdr</v> of <v>x</v> and return the result.</p> \u000A \u000A <p>Here is an example of its execution.  We have displayed the input tree in \u000A full dot notation so that the number of interior nodes is just the number of \u000A dots.</p> \u000A \u000A <code>ACL2 !&gt;(sweep-tree '((((a . 1) . (2 . b)) . 3)\u000A                     . (4 . (5 . d)))\u000A                   counters)\u000A&lt;counters&gt;\u000AACL2 !&gt;(show-counters counters)\u000A(7 8 (5 4 3 2 1))\u000AACL2 !&gt;(reset-counters counters)\u000A&lt;counters&gt;\u000AACL2 !&gt;(show-counters counters)\u000A(0 0 NIL)</code> \u000A \u000A <p>The <v>counters</v> object has two integer fields and a field whose type is \u000A unrestricted.  single-threaded objects support other types of fields, such as \u000A arrays.  We deal with that in the <see topic=\"ACL2____STOBJ-EXAMPLE-2\">stobj-example-2</see>.  But we recommend \u000A that you first consider the implementation issues for the <v>counters</v> \u000A example (in <see topic=\"ACL2____STOBJ-EXAMPLE-1-IMPLEMENTATION\">stobj-example-1-implementation</see>) and then consider the proof \u000A issues (in <see topic=\"ACL2____STOBJ-EXAMPLE-1-PROOFS\">stobj-example-1-proofs</see>).</p> \u000A \u000A <p>To continue the stobj tour, see <see topic=\"ACL2____STOBJ-EXAMPLE-2\">stobj-example-2</see>.</p>"],

 "ACL2____SLOW-ARRAY-WARNING":[["Arrays"],"ACL2 Sources","ACL2","<p>If you use ACL2 <see topic=\"ACL2____ARRAYS\">arrays</see> you may sometimes see a <b>slow \u000A array</b> warning.  We explain below what that warning means and some likely \u000A ``mistakes'' it may signify.</p> \u000A \u000A <p>First, we note that you can control whether or not you get a warning and, \u000A if so, whether or not a break (error from which you can continue; see <see topic=\"ACL2____BREAK_42\">break$</see>) ensues:</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> slow-array-action :warning) ; warn on slow array access (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> slow-array-action :break)   ; warn as above, and then call break$\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> slow-array-action nil) ; do not warn or break on slow array access</code> \u000A \u000A <p>If you are using ACL2 arrays, then you probably care about performance, in \u000A which case it is probably best to avoid the <v>nil</v> setting.  Below we assume \u000A the default behavior: a warning, but no break.</p> \u000A \u000A <p>The discussion in the documentation for <see topic=\"ACL2____ARRAYS\">arrays</see> defines what we mean \u000A by the semantic value of a name.  As noted there, behind the scenes ACL2 \u000A maintains the invariant that with some names there is associated a pair \u000A consisting of an ACL2 array <v>alist</v>, called the semantic value of the name, \u000A and an equivalent raw lisp array.  Access to ACL2 array elements, as in \u000A <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist i)</v>, is executed in constant time when the array alist \u000A is the semantic value of the name, because we can just use the corresponding \u000A raw lisp array to obtain the answer.  <tt><see topic=\"ACL2____ASET1\">Aset1</see></tt> and <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> \u000A modify the raw lisp array appropriately to maintain the invariant.</p> \u000A \u000A <p>If <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> is called on a name and alist, and the alist is not the \u000A then-current semantic value of the name, the correct result is computed but it \u000A requires linear time because the alist must be searched.  When this happens, \u000A <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> prints a <b>slow array</b> warning message to the comment \u000A window.  <tt><see topic=\"ACL2____ASET1\">Aset1</see></tt> behaves similarly because the array it returns will \u000A cause the <b>slow array</b> warning every time it is used.</p> \u000A \u000A <p>From the purely logical perspective there is nothing ``wrong'' about such \u000A use of <see topic=\"ACL2____ARRAYS\">arrays</see> and it may be spurious to print a warning message.  But \u000A because <see topic=\"ACL2____ARRAYS\">arrays</see> are generally used to achieve efficiency, the <b>slow \u000A array</b> warning often means the user's intentions are not being realized. \u000A Sometimes merely performance expectations are not met; but the message may \u000A mean that the functional behavior of the program is different than \u000A intended.</p> \u000A \u000A <p>Here are some ``mistakes'' that might cause this behavior.  In the \u000A following we suppose the message was printed by <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> about an array \u000A named <v>name</v>.  Suppose the alist supplied <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> is <v>alist</v>.</p> \u000A \u000A <p>(1) <tt><see topic=\"ACL2____COMPRESS1\">Compress1</see></tt> was never called on <v>name</v> and <v>alist</v>.  That \u000A is, perhaps you created an alist that is an <tt><see topic=\"ACL2____ARRAY1P\">array1p</see></tt> and then proceeded \u000A to access it with <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> but never gave ACL2 the chance to create a raw \u000A lisp array for it.  After creating an alist that is intended for use as an \u000A array, you must do <v>(<see topic=\"ACL2____COMPRESS1\">compress1</see> name alist)</v> and pass the resulting \u000A <v>alist'</v> as the array.</p> \u000A \u000A <p>(2) <v>Name</v> is misspelled.  Perhaps the array was compressed under the \u000A name <v>'delta-1</v> but accessed under <v>'delta1</v>?</p> \u000A \u000A <p>(3) An <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> was done to modify <v>alist</v>, producing a new array, \u000A <v>alist'</v>, but you subsequently used <v>alist</v> as an array.  Inspect all \u000A <v>(<see topic=\"ACL2____ASET1\">aset1</see> name ...)</v> occurrences and make sure that the alist modified is \u000A never used subsequently (either in that function or any other).  It is good \u000A practice to adopt the following syntactic style.  Suppose the alist you are \u000A manipulating is the value of the local variable <v>alist</v>.  Suppose at some \u000A point in a function definition you wish to modify <v>alist</v> with <tt><see topic=\"ACL2____ASET1\">aset1</see></tt>.  Then write</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((alist (<see topic=\"ACL2____ASET1\">aset1</see> name alist i val))) ...)</code> \u000A \u000A <p>and make sure that the subsequent function body is entirely within the \u000A scope of the <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>.  Any uses of <v>alist</v> subsequently will refer to \u000A the new alist and it is impossible to refer to the old alist.  Note that if \u000A you write</p> \u000A \u000A <code>(foo (<see topic=\"COMMON-LISP____LET\">let</see> ((alist (<see topic=\"ACL2____ASET1\">aset1</see> name alist i val))) ...)  ; arg 1\u000A     (bar alist))                                  ; arg 2</code> \u000A \u000A <p>you have broken the rules, because in <v>arg 1</v> you have modified \u000A <v>alist</v> but in <v>arg 2</v> you refer to the old value.  An appropriate \u000A rewriting is to lift the <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> out:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((alist (<see topic=\"ACL2____ASET1\">aset1</see> name alist alist i val)))\u000A  (foo ...                                         ; arg 1\u000A       (bar alist)))                               ; arg 2</code> \u000A \u000A <p>Of course, this may not mean the same thing.</p> \u000A \u000A <p>(4) A function which takes <v>alist</v> as an argument and modifies it with \u000A <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> fails to return the modified version.  This is really the same \u000A as (3) above, but focuses on function interfaces.  If a function takes an \u000A array <v>alist</v> as an argument and the function uses <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> (or a \u000A subfunction uses <tt><see topic=\"ACL2____ASET1\">aset1</see></tt>, etc.), then the function probably ``ought'' to \u000A return the result produced by <tt><see topic=\"ACL2____ASET1\">aset1</see></tt>.  The reasoning is as follows.  If \u000A the array is passed into the function, then the caller is holding the array. \u000A After the function modifies it, the caller's version of the array is obsolete. \u000A If the caller is going to make further use of the array, it must obtain the \u000A latest version, i.e., that produced by the function.</p>"],

 "ACL2____SET-TAU-AUTO-MODE":[["Tau-system"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see> t)      ; select automatic (``greedy'') mode\u000A(<see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see> nil)    ; select manual mode</code> \u000A \u000A <p>This event is equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :tau-auto-modep\u000A &lt;t-or-nil&gt;)</v>, and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>. \u000A See <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> for background details.</p> \u000A \u000A <p>The tau system gathers rules for its database in one of two ways: greedily \u000A or only at the explicit command of the user.  ``Greedy'' mode is officially \u000A called ``automatic mode'' and is the default.  The other mode is called \u000A ``manual mode.''</p> \u000A \u000A <p>In automatic mode, all rules processed by ACL2 are also considered for \u000A inclusion in the tau database: if the <v>:corollary</v> of some proved theorem \u000A happens to fit into one of the forms described in <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>, \u000A that rule is quietly added to the tau database <i>regardless of what</i> \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> the user named for the <v>:corollary</v>.  Of course, \u000A such rules are also stored in the ways named by the user.  See the <i>Design \u000A Philosophy</i> section of <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> for a \u000A discussion of why the tau system is greedy by default.  More details are given \u000A on automatic mode after we explain manual mode.</p> \u000A \u000A <p>To more tightly control the rules available to the tau system, the user may \u000A select manual mode by executing <v>(<see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see> nil)</v>.  In manual mode, \u000A the only events that create <v>:tau-system</v> rules are <v>defthm</v> events \u000A explicitly specifying the <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> rule class in the \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> argument.  Of course, for a <v>:tau-system</v> rule \u000A to be created from a proved formula (or its specified <v>:corollary</v>), the \u000A formula must be of the appropriate shape (syntactic form). See <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>.  In manual mode, if the <v>:tau-system</v> rule class is specified \u000A but the formula is not of an appropriate form an error is signalled.  (Note: \u000A even in manual mode, monadic functions that are recognized as Boolean are \u000A classified as tau predicates; but no rules are created for them.)</p> \u000A \u000A <p>Returning to our discussion of automatic mode, a <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> \u000A rule may be created by any of the events below, provided the definition or \u000A formula proved is of an appropriate shape:</p> \u000A \u000A <p>(1) <v>defun</v> events introducing ``big switch'' or ``<v>mv-nth</v> \u000A synonyms,''</p> \u000A \u000A <p>(2) <v>defun</v> events creating type-prescription rules that may be also \u000A represented as ``signature rules'' of form 1, and</p> \u000A \u000A <p>(3) any <v>defthm</v> event with a non-<v>nil</v> <v>:rule-classes</v> argument \u000A   if no <v>:tau-system</v> rule is among the rule classes and the formula proved \u000A   is in the shape of any <v>tau-system</v> rule.</p> \u000A \u000A <p>Of course, events such as <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> and <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> may \u000A also add rules to the tau database when they execute the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events implicit in their descriptions.  See <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see> \u000A for a description of the various shapes mentioned above.</p> \u000A \u000A <p>Note that any rule (of any rule class) created when the tau system is in \u000A manual mode is also created in automatic mode.  For example, if an event would \u000A create a <v>:DEFINITION</v>, <v>:TYPE-PRESCRIPTION</v>, <v>FORWARD-CHAINING</v>, or \u000A <v>:REWRITE</v> rule when the tau system is in manual mode, then the event will \u000A create that same rule when the tau system is in automatic mode.  Automatic \u000A mode just means that some additional <v>:tau-system</v> rules may be \u000A created.</p> \u000A \u000A <p>Of course, if a <v>defthm</v> event explicitly specifies a <v>:tau-system</v> \u000A rule class, then even if the tau system is in automatic mode, that tau rule is \u000A created from the proved formula (or the specified <v>:corollary</v>) or else an \u000A error is caused.  But if the tau system is in automatic mode and a <v>defthm</v> \u000A event doesn't explicitly specify a <v>:tau-system</v> rule class, then the \u000A system quietly checks each specified <v>:corollary</v> &mdash; or, in the \u000A absence of any <v>:corollary</v>, it checks the proved formula &mdash; for \u000A whether it can be stored as a tau rule.  If so, then the system stores a tau \u000A rule, in addition to storing the specified rule.  Of course, no error is \u000A signalled if a proved formula of some non-<v>:tau-system</v> rule class fails to \u000A be of an appropriate shape for the tau system.</p> \u000A \u000A <p>In automatic mode, if the <v>:rule-classes</v> specified for <v>defthm</v> \u000A included several corollaries and any one of them is of class <v>:tau-system</v> \u000A then the only tau system rules created are those explicitly classed as \u000A <v>:tau-system</v> rules.  For example, suppose a <v>defthm</v> has one \u000A <v>:corollary</v> stored as a <v>:rewrite</v> rule and another <v>:corollary</v> \u000A stored as a <v>:tau-system</v> rule.  But suppose the <v>:rewrite</v> rule happens \u000A to also to fit the form of a <v>:tau-system</v> rule.  Is it added to the tau \u000A database or not?  The answer is no.  If you have taken the trouble to specify \u000A <v>:tau-system</v> corollaries for an event, then those corollaries are the only \u000A ones stored as tau sytem rules from that event.  Note that had both \u000A corollaries been classed as <v>:rewrite</v> rules (and been of acceptable \u000A <v>:tau-system</v> form) both would have also been made <v>:tau-system</v> rules. \u000A This also allows you be in automatic mode and state a <v>:rewrite</v> or other \u000A non-<v>:tau-system</v> rule and prevent it from being also made a tau system \u000A rule: just add a frivolous <v>:tau-system</v> <v>:corollary</v> like <v>(<see topic=\"ACL2____BOOLEANP\">booleanp</see>\u000A (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))</v>.</p> \u000A \u000A <p>Recall that the use of tau rules is controlled by the rune \u000A <v>(:EXECUTABLE-COUNTERPART TAU-SYSTEM)</v>.  When that rune is disabled, no tau \u000A rules are <i>used</i> in proofs.  However, the tau system continues to collect \u000A tau rules if the system is in automatic mode.  Thus, if and when the tau \u000A system is re-enabled, rules automatically generated while the tau system was \u000A disabled will be used as usual by the tau system.</p> \u000A \u000A <p>Finally, note that <v>defthm</v> events with <v>:rule-classes</v> <v>nil</v> do \u000A not create <v>:tau-system</v> rules even if the formula proved is of an \u000A appropriate shape, regardless of whether the tau system is in automatic or \u000A manual mode.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____TAU-STATUS\">tau-status</see></tt> provides a convenient way to enable/disable \u000A the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <v>tau-system</v> and/or to switch \u000A between manual and automatic modes.  It may also be used to determine the \u000A current settings of those two flags.</p>"],

 "ACL2____PROOF-CHECKER-COMMANDS":[["Proof-checker"],"ACL2 Sources","ACL2","<p>This documentation section contains documentation for individual \u000A commands that can be given inside the interactive <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> loop \u000A that is entered using <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>.</p>"],

 "ACL2____PARALLELISM-TUTORIAL":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>In this topic we introduce the ACL2 parallelism primitives using the \u000A example of a doubly-recursive Fibonacci function, whose basic definition is as \u000A follows.  See <see topic=\"ACL2____PARALLELISM\">parallelism</see> for a very high-level summary of the \u000A parallelism capability described here, and see <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see> for how \u000A to build an executable image that supports parallel execution.  Here, we \u000A assume that such an executable is being used.</p> \u000A \u000A <p><b>Serial Fibonacci</b></p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"COMMON-LISP_____B2\">+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1)) (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))))</code> \u000A \u000A <p><b>Introducing</b> <tt><see topic=\"ACL2____PARGS\">Pargs</see></tt></p> \u000A \u000A <p>A simple way to introduce parallelism into this function definition is to \u000A wrap the addition expression with a call of <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, and the arguments \u000A to the addition will be computed in parallel whenever resources are available. \u000A As a result, we end up with a very similar and thus intuitive function \u000A definition.  Note that we replaced <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> by <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>, since <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> expects a function call, not a macro call.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                            (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))))</code> \u000A \u000A <p><b>Introducing the Granularity Problem</b></p> \u000A \u000A <p>After you submit the above two versions of the Fibonacci function, test \u000A them with the following forms.</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME_42\">time$</see> (fib 10))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (pfib 10))</code> \u000A \u000A <p>Now increase the argument by increments of 5 to 10 until you find your \u000A curiosity satisfied or your patience wearing thin.  You can interrupt \u000A evaluation if necessary and return to the ACL2 loop.  You will immediately \u000A notice that you have not increased execution speed, at least not by much, by \u000A introducing parallelism.</p> \u000A \u000A <p>First, consider the computation of <v>(pfib 4)</v>.  Assuming resources are \u000A available, <v>(pfib 4)</v> will create a thread for computing <v>(pfib 3)</v> and \u000A another thread for <v>(pfib 2)</v>.  It is easy to imagine that setting up each \u000A thread takes much longer than the entire computation of <v>(fib 4)</v>.</p> \u000A \u000A <p>Second, we must realize that if we have two threads available for computing \u000A <v>(fib 10)</v>, then the evaluation of <v>(fib 8)</v> will probably complete \u000A before the evaluation of <v>(fib 9)</v>.  Once <v>(fib 8)</v> finishes, \u000A parallelism resources will become available for the next recursive call made \u000A on behalf of <v>(fib 9)</v>.  If for example that call is <v>(fib 3)</v>, we will \u000A waste a lot of cycles just handing work to the thread that does this \u000A relatively small computation.  We need a way to ensure that parallelism \u000A resources are only used on problems of a \"large\" size.  Ensuring that only \u000A \"large\" problems are spawned is called the ``granularity problem.''</p> \u000A \u000A <p>In summary: We want to tell ACL2 that it can evaluate the arguments of \u000A <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> in parallel only when the parameter of <v>pfib</v> is greater than \u000A some threshold.  Our tests using CCL have suggested that 27 is a reasonable \u000A threshold.</p> \u000A \u000A <p><b>Explicit Programming for the Granularity Problem</b></p> \u000A \u000A <p>One way to avoid the granularity problem is to duplicate code as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 27) ; the granularity test\u000A               (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                                (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A             (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                       (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))))</code> \u000A \u000A <p>Duplicating code is fundamentally a bad design principle, because it can \u000A double the work for future maintenance.  A ``granularity form'' is an \u000A expression</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> &lt;form&gt;))</code> \u000A \u000A <p>that can allow threads to be spawned (without duplicating code) whenever \u000A the evaluation of <v>&lt;form&gt;</v> results in a non-<v>nil</v> value.  It may be \u000A placed inside any call of a parallelism primitive, in a position documentated \u000A separately for each primitive.  Here is a definition of <v>pfib</v> using this \u000A feature for a call of the parallelism primitive <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PARGS\">pargs</see>\u000A            (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 27)))\u000A            (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                      (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))))</code> \u000A \u000A <p>Test each version as follows (or substitute your own natural number for \u000A 33).</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME_42\">time$</see> (fib 33))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (pfib 33))</code> \u000A \u000A <p><b>Another Granularity Issue Related to Thread Limitations</b></p> \u000A \u000A <p>Our implementation of parallelism primitives has the property that once a \u000A thread is assigned a computation, that assignment stays in effect until the \u000A computation is complete.  In particular, if a thread encounters a parallelism \u000A primitive that spawns child threads, the parent thread stays assigned, waiting \u000A until the child computations complete before it can continue its own \u000A computation.  In the meantime, the parent thread reduces the number of \u000A additional threads that Lisp can provide by 1, rather than being reassigned to \u000A do other work.</p> \u000A \u000A <p>How can this lack of reassignment affect the user?  Consider, for example, \u000A the application of a recursive function to a long list.  Imagine that this \u000A function is written so that the body contains a recursive call, for example as \u000A <v>(<see topic=\"ACL2____PARGS\">pargs</see> (process (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (recur (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</v>.  Each such <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> call \u000A that spawns child work must wait for its children, one of which is the work of \u000A evaluating <v>(recur (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v>, to complete.  There is an ACL2 limit on how \u000A much pending work can be in the system, limiting the number of <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> \u000A calls that can result in parallel execution.  For example, if the ACL2 limit \u000A is k and each call of <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> actually spawns threads for evaluating its \u000A arguments, then after k <v>cdr</v>s there will be no further parallel \u000A execution.</p> \u000A \u000A <p>Possible solutions may include reworking of algorithms (for example to be \u000A tree-based rather than list-based) or using appropriate granularity forms.  We \u000A hope that future implementations will allow thread ``re-deployment'' in order \u000A to mitigate this problem.  This problem applies to <tt><see topic=\"ACL2____PLET\">plet</see></tt>, <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, <tt><see topic=\"ACL2____PAND\">pand</see></tt>, <tt><see topic=\"ACL2____POR\">por</see></tt>, and <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt>.</p> \u000A \u000A <p><b>Introducing</b> <tt><see topic=\"ACL2____PLET\">Plet</see></tt></p> \u000A \u000A <p>We can use a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> binding to compute the recursive calls of <v>fib</v> \u000A and then add the bound variables together, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"COMMON-LISP____LET\">let</see> ((a (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1)))\u000A                 (b (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A             (<see topic=\"COMMON-LISP_____B2\">+</see> a b)))))</code> \u000A \u000A <p>By using the parallelism primitive <tt><see topic=\"ACL2____PLET\">plet</see></tt>, we can introduce \u000A parallelism much as we did using <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, with an optional granularity \u000A form, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PLET\">plet</see>\u000A            (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 27)))\u000A            ((a (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1)))\u000A             (b (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A            (<see topic=\"COMMON-LISP_____B2\">+</see> a b)))))</code> \u000A \u000A <p>Notice that this time, we were able to use <v>+</v> rather than being forced \u000A to use <v>binary-+</v>.  Unlike <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, which expects a function call \u000A (not a macro call), <tt><see topic=\"ACL2____PLET\">plet</see></tt> allows macros at the top level of its \u000A body.</p> \u000A \u000A <p><b>Introducing</b> <tt><see topic=\"ACL2____PAND\">Pand</see></tt> (By Way of a Tree Validation Example)</p> \u000A \u000A <p>Consider ``genetic trees'' that contains leaves of DNA elements, in the \u000A sense that each tip is one of the symbols <v>A</v>, <v>G</v>, <v>C</v>, or <v>T</v>. \u000A First we define the function <v>valid-tip</v> which recognizes whether a tip \u000A contains one of these symbols.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tip (tip)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> tip 'A)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> tip 'G)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> tip 'C)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> tip 'T)))</code> \u000A \u000A <p>Now we define a function that traverses the tree, checking that every tip \u000A is valid.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tree-serial (tree)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> tree)\u000A      (valid-tip tree)\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (valid-tree-serial (<see topic=\"COMMON-LISP____CAR\">car</see> tree))\u000A         (valid-tree-serial (<see topic=\"COMMON-LISP____CDR\">cdr</see> tree)))))</code> \u000A \u000A <p>We also define a parallel version.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tree-parallel (tree)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> tree)\u000A      (valid-tip tree)\u000A    (<see topic=\"ACL2____PAND\">pand</see> (valid-tree-parallel (<see topic=\"COMMON-LISP____CAR\">car</see> tree))\u000A          (valid-tree-parallel (<see topic=\"COMMON-LISP____CDR\">cdr</see> tree)))))</code> \u000A \u000A <p>Before we can time the functions, we need to create test trees.  We have \u000A found that test trees need to be approximately 1 gigabyte before we clearly \u000A see speedup, and we make them asymmetric to demonstrate the ability of our \u000A implementation to adapt to asymmetric data.  We can create the trees with the \u000A execution of the following forms.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-valid-binary-tree (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'C 'G) (<see topic=\"COMMON-LISP____CONS\">cons</see> 'A 'T))\u000A    (<see topic=\"COMMON-LISP____CONS\">cons</see> (make-valid-binary-tree (<see topic=\"COMMON-LISP____-\">-</see> x 2)) ; 2 to make asymmetric\u000A          (make-valid-binary-tree (<see topic=\"COMMON-LISP____-\">-</see> x 1)))))\u000A\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *valid-tree* (make-valid-binary-tree 30)) ; takes awhile\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *invalid-tree* (<see topic=\"COMMON-LISP____CONS\">cons</see> *valid-tree* nil)) ; nil is invalid tip</code> \u000A \u000A <p>We can time our functions with the forms:</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-serial *valid-tree*))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-parallel *valid-tree*))</code> \u000A \u000A <p>Unfortunately, the serial version runs faster than the parallelism version; \u000A however, there is more to this story.</p> \u000A \u000A <p><b>Demonstrating Early Termination with an Invalid Tree</b></p> \u000A \u000A <p>Now observe this magic:</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-serial   *invalid-tree*))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-parallel *invalid-tree*))</code> \u000A \u000A <p>The serial version took awhile, while the parallel version finished almost \u000A immediately.  The test for validation was split into testing the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> \u000A and the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of the <v>*invalid-tree*</v> root, and since the <v>cdr</v> \u000A was equal to <v>nil</v>, its test returned immediately.  This immediate return \u000A quickly interrupted the computation associated with the <v>car</v>, and returned \u000A the result.</p> \u000A \u000A <p><b>Granularity with</b> <tt><see topic=\"ACL2____PAND\">Pand</see></tt></p> \u000A \u000A <p>We can also define a parallel version with a granularity form:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tree-parallel (tree depth)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> depth)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> tree)\u000A      (valid-tip tree)\u000A    (<see topic=\"ACL2____PAND\">pand</see>\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> depth 5)))\u000A     (valid-tree-parallel (<see topic=\"COMMON-LISP____CAR\">car</see> tree) (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))\u000A     (valid-tree-parallel (<see topic=\"COMMON-LISP____CDR\">cdr</see> tree) (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth)))))</code> \u000A \u000A <p>We can test this form by executing our previous forms.  You will probably \u000A find some speedup on a machine with several cores available, but more speedup \u000A can likely be obtained with an expensive test on the leaves in place of \u000A <v>valid-tip</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-serial   *valid-tree*))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (valid-tree-parallel *valid-tree* 0))</code> \u000A \u000A <p><b>Introducing</b> <tt><see topic=\"ACL2____POR\">Por</see></tt></p> \u000A \u000A <p><tt><see topic=\"ACL2____POR\">Por</see></tt> can be used in the same way as <tt><see topic=\"ACL2____PAND\">pand</see></tt>, but with early \u000A termination occurring when an argument evaluates to a non-<v>nil</v> value, in \u000A which case the value returned is <v>t</v>.  Finally, <tt><see topic=\"ACL2____POR\">por</see></tt> also supports \u000A the use of a <see topic=\"ACL2____GRANULARITY\">granularity</see> form.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>It is impossible in this short introduction to teach you \u000A propositional calculus if you don't already know it!</p> \u000A \u000A <p>A typical use of propositional calculus is to observe that</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z)))</code> \u000A \u000A <p>is equivalent to:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>If this is surprising and you know propositional calculus, then the problem \u000A might be our notation.  We're exploiting the tautology</p> \u000A \u000A <code><i>(p ---&gt; (q ---&gt; r)) &lt;---&gt; ((p &amp; q) ---&gt; r)</i> \u000A</code> \u000A \u000A <p>where <i>---&gt;</i> and <i>&lt;---&gt;</i> are meant to be the traditional \u000A arrows denoting logical implication and logical equivalence.</p> \u000A \u000A <p>If you don't know propositional calculus, we'll say just a few things to \u000A help ease your journey.</p> \u000A \u000A <p>A <i>propositional formula</i>, in ACL2, is any formula written entirely in \u000A terms of variable symbols, <v>T</v>, <v>NIL</v>, and the propositional functions \u000A <v>AND</v>, <v>OR</v>, <v>NOT</v>, <v>IMPLIES</v>, and <v>IFF</v>.  The ``tautology'' \u000A above in traditional notation is this propositional formula in ACL2:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">IFF</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> P (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> Q R))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> P Q) R)).</code> \u000A \u000A <p>If you have a formula like</p> \u000A \u000A <code>(implies <i>hyp</i> \u000A         <i>concl</i>) \u000A</code> \u000A \u000A <p>then we say that formula is an <i>implication</i>, that <i>hyp</i> is the \u000A <i>hypothesis</i>, and that <i>concl</i> is the conclusion.  If the hypothesis \u000A is an <v>and</v> expression, as in</p> \u000A \u000A <code>(implies (and <i>hyp1</i> \u000A              <i>hyp2</i> \u000A              ...) \u000A         <i>concl</i>) \u000A</code> \u000A \u000A <p>then we call <i>hyp1</i> is the <i>first hypothesis</i>, <i>hyp2</i> is the \u000A <i>second hypothesis</i>, etc.</p> \u000A \u000A <p>If a term is of the form</p> \u000A \u000A <code>(and <i>term1</i> <i>term2</i> ...) \u000A</code> \u000A \u000A <p>we say it is a <i>conjunction</i> and that <i>term1</i> is the <i>first \u000A conjunct</i>, <i>term2</i> is the <i>second conjunct</i>, etc.  We treat an \u000A <v>or</v>-term analogously but call it a <i>disjunction</i> and its arguments \u000A are <i>disjuncts</i>.</p> \u000A \u000A <p>A <i>tautology</i> is any propositional formula that can be proved by \u000A testing it under all combinations of Boolean assignments to its variables.  We \u000A give an example of such a <i>truth-table proof</i> below, but hasten to add \u000A that ACL2 does not generally use truth tables to recognize tautologies.  It \u000A primarily uses <v>IF</v>-normalization and BDDs to recognize tautologies, which \u000A can be seen as a mix of symbolic manipulation and case analysis.</p> \u000A \u000A <p>Many tautologies have names, but ACL2 doesn't refer to them by name because \u000A it derives them from first principles.  We list a few here because we \u000A sometimes use the names in our documentation; more importantly, you should \u000A look at these formulas and convince yourself that they're always true for all \u000A Boolean values of the variables:</p> \u000A \u000A <code><i>Double Negation</i>: \u000A(iff (not (not p)) p) \u000A \u000A<i>DeMorgan</i>: \u000A(iff (not (and p q)) \u000A     (or (not p) (not q))) \u000A \u000A<i>Distributivity</i>: \u000A(iff (and p (or q r)) \u000A     (or (and p q) \u000A         (and p r))) \u000A \u000A<i>Promotion</i>: \u000A(iff (implies p (implies q r)) \u000A     (implies (and p q) r)) \u000A \u000A<i>Implicative Disjunction</i>: \u000A(iff (implies p q) \u000A     (or (not p) q)) \u000A \u000A<i>Contrapositive</i>: \u000A(iff (implies p q) \u000A     (implies (not q) (not p))) \u000A \u000A<i>Generalized Contrapositive</i>: \u000A(iff (implies (and p r) q) \u000A     (implies (and p (not q)) (not r))) \u000A \u000A</code> \u000A \u000A <p>There are, of course, many others, even with these same names!  For \u000A example, there is a dual version of DeMorgan showing how <v>not</v> distributes \u000A over <v>or</v>, a dual version of Distributivity for <v>or</v> over <v>and</v>, \u000A etc.</p> \u000A \u000A <p>Dealing with propositional calculus will not generally be a problem for you \u000A because it is decidable and ACL2 has procedures that decide propositional \u000A formulas.  However, propositional calculus can lead to exponential explosion \u000A and can thus explain why ACL2 has ``gone out to lunch.''  In addition, \u000A sometimes if you are curious as to <i>why</i> ACL2 is working on a certain \u000A subgoal the reason can be traced back to propositional calculus.</p> \u000A \u000A <p>The most common example of this is that to prove a formula of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____IMPLIES\">implies</see> p1 q1)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> p2 q2))</code> \u000A \u000A <p>propositional calculus will convert it to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p2 (<see topic=\"COMMON-LISP____NOT\">not</see> p1)) q2)\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p2 q1) q2))</code> \u000A \u000A <p>Many users are surprised that the first conjunct above does not have \u000A <v>q1</v> as a hypothesis.  If you ever stare at an ACL2 goal and say to \u000A yourself ``A hypothesis is missing!'' the chances are that propositional \u000A calculus is ``to blame.''  In particular, if you are trying to prove that \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> p1 q1)</v> implies something, you must deal with the case that \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> p1 q1)</v> is true because <v>p1</v> is false.  Think about it.</p> \u000A \u000A <p>Now we illustrate the truth table method for deciding tautologies, even \u000A though that is not what ACL2 generally uses.  Consider the formula called \u000A Promotion above:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">IFF</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> P (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> Q R))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> P Q) R))</code> \u000A \u000A <p>The formula above is a tautology.  It contains three variables, <v>P</v>, \u000A <v>Q</v>, and <v>R</v>, and so there are 8 combinations of Boolean assignments to \u000A them.  If we let</p> \u000A \u000A <code><i>formula1</i>:  (IMPLIES P (IMPLIES Q R)) \u000A<i>formula2</i>:  (IMPLIES (AND P Q) R) \u000A</code> \u000A \u000A <p>then we wish to test the formula <v>(<see topic=\"ACL2____IFF\">IFF</see> </v><i>formula1 \u000A formula2</i><v>)</v>:</p> \u000A \u000A <code>P   Q   R       <i>formula1</i>   <i>formula2</i>   (IFF <i>formula1</i> <i>formula2</i>) \u000A--------- \u000AT   T   T            T         T       T \u000AT   T   NIL          NIL       NIL     T \u000AT   NIL T            T         T       T \u000AT   NIL NIL          T         T       T \u000ANIL T   T            T         T       T \u000ANIL T   NIL          T         T       T \u000ANIL NIL T            T         T       T \u000ANIL NIL NIL          T         T       T \u000A</code> \u000A \u000A <p>So we see that the formula always returns <v>T</v> and is thus a \u000A tautology.</p> \u000A \u000A <p>Recall that in the original example at the top of this page we were trying \u000A to prove the formula</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z)))</code> \u000A \u000A <p>This formula is an <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see> of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> p (<see topic=\"ACL2____IMPLIES\">implies</see> q r)).</code> \u000A \u000A <p>The substitution required by the match is</p> \u000A \u000A <code><i>sigma</i>: \u000A((p    (endp z)) \u000A (q    (true-listp z)) \u000A (r    (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>Since we know the tautology:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"ACL2____IMPLIES\">implies</see> p (<see topic=\"ACL2____IMPLIES\">implies</see> q r))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> p q) r)).</code> \u000A \u000A <p>is always true no matter what Boolean values <v>p</v>, <v>q</v>, and <v>r</v> \u000A have, then we know this instance of it (obtained by applying the substitution \u000A <i>sigma</i> above) is always true:</p> \u000A \u000A <code>(iff (implies (endp z)                            <i>formula1'</i> \u000A              (implies (true-listp z) \u000A                       (equal (rev (rev z)) z))) \u000A     (implies (and (endp z)                       <i>formula2'</i> \u000A                   (true-listp z)) \u000A              (equal (rev (rev z)) z))). \u000A</code> \u000A \u000A <p>Thus, if we're trying to prove <i>formula1'</i> it is permitted to try to \u000A to prove <i>formula2'</i> instead, because they return the same \u000A truthvalue.</p> \u000A \u000A <p>This sketch of propositional reasoning in ACL2 is a little suspect because \u000A we didn't address the possibility that the substitution might replace the \u000A propositional variables by non-propositional terms.  But the tautology was \u000A verified only on Boolean values for those variables.  This actually works out \u000A because in ACL2 all propositional testing is done against <v>nil</v> and any \u000A non-<v>nil</v> value, including <v>t</v>, is as good as another.  However, the \u000A tautology allows us to replace one formula by the other only in contexts in \u000A which we just care about propositional truth, i.e., whether the formula is \u000A <v>nil</v> or not.  When we prove a formula in ACL2 we are really establishing \u000A that it never returns <v>nil</v>, i.e., no matter what the values of the \u000A variables, the value of the formula is non-<v>nil</v>.</p> \u000A \u000A <p>A very simple example of this is with Double Negation.</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p)) p)</code> \u000A \u000A <p>is a tautology.  This means that if we were trying to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p)) ...)</code> \u000A \u000A <p>we could transform it to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> p ...).</code> \u000A \u000A <p>But if we were trying to prove:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p)) p)</code> \u000A \u000A <p>we could not prove it by using Double Negation!  The formula above claims \u000A that <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p))</v> and <v>p</v> have identical values.  They do not!  For \u000A example, <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> 3))</v> is <v>t</v>, not <v>3</v>.  However, <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see>\u000A 3))</v> and <v>t</v> are propositionally equivalent (i.e., satisfy <v>iff</v>) \u000A because one is as good as the other in a test.  <i>That</i> is what Double \u000A Negation says.</p> \u000A \u000A <p>As long as you only use propositional formulas in propositional places this \u000A aspect of ACL2 should not affect you.</p> \u000A \u000A <p>Now please use your browser's <b>Back Button</b> to return to the example \u000A proof in <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>You should design your rewrite rules to canonicalize the terms in \u000A your problem, that is, your rules should drive terms into some normal form so \u000A that different but equivalent terms are rewritten into the preferred shape, \u000A making equivalent terms identical.  You are very familiar with this idea from \u000A algebra, where you learned to normalize polynomials.  Thus, when you see \u000A <i>(2x + 6)(3x - 9)</i> you automaticaly normalize it, by ``multiplying out \u000A and collecting like terms,'' to get <i>(6x^2 - 54)</i>.  This normalization \u000A strategy allows you to recognize equivalent terms presented differently, such \u000A as <i>6(x^2 - 9)</i>.</p> \u000A \u000A <p>The ACL2 user is responsible for making up the rules.  (Standard ``books'' \u000A -- files of ACL2 definitions and theorems -- can often provide rules for some \u000A sets of functions, e.g., arithmetic.)  This is a heavy burden on you but it \u000A means you are in charge of your own normal forms.  For example, if you use the \u000A function <v>nthcdr</v>, which returns the <v>n</v>th <v>cdr</v> of a list, you might \u000A see both <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> i x))</v> and <v>(<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> i (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v>.  These two \u000A expressions are equivalent but not identical.  You will want to decide which \u000A you want to see and prove the rewrite rule that converts the other to that \u000A preferred form.</p> \u000A \u000A <p>Most good users develop an implicit ordering on terms and rewrite ``heavy'' \u000A terms to ``lighter'' ones.  This insures that there are no loops in their \u000A rewrite rules.  But this ordering changes according to the user and the \u000A problem.</p> \u000A \u000A <p>Generally, the lightest terms are primitives such as <v>IF</v>, <v>EQUAL</v>, \u000A arithmetic, etc.  Functions defined without explicit recursion tend to be \u000A ignored because they are just expanded away (but see below).  Recursively \u000A defined functions tend to be heavier than any other recursive function used in \u000A their definitions, so, for example, if <v>rev</v> is defined in terms of \u000A <v>append</v>, <v>rev</v> is heavier than <v>append</v>.  But the size and subtlety \u000A of recursively defined functions also affects their place in the ordering.</p> \u000A \u000A <p>But rewrite rules are surprisingly subtle.  Recall that a rewrite rule can \u000A be made from a formula of this form:</p> \u000A \u000A <code>(IMPLIES (AND <i>hyp1</i> ... <i>hypk</i>) \u000A         (<i>eqv</i> <i>lhs</i> <i>rhs</i>)) \u000A</code> \u000A \u000A <p>where <i>eqv</i> is either <v>EQUAL</v> or <v>IFF</v>, and <i>lhs</i> is a call \u000A of a function other than <v>IF</v>.  In such a rule, <i>lhs</i> is the pattern \u000A responsible for triggering the rule, the <i>hypi</i> are conditions which must \u000A be satisfied by the context of the target being rewritten, and <i>rhs</i> is \u000A the replacement.  The replacement only happens if the rule is enabled, the \u000A pattern matches, the conditions are satisfied, and (in the case of an <v>IFF</v> \u000A rule) the target occurs propositionally.  There are other heuristic \u000A restrictions that we won't discuss here.</p> \u000A \u000A <p>So how should you phrase a theorem in order to make it an effective \u000A rule?</p> \u000A \u000A <p>General Principles:</p> \u000A \u000A <p>* <b>Strengthen the Formula</b>: The fewer hypotheses a formula has the \u000A better the rewrite rule formed from it.  The more general the left-hand side \u000A the better the rule.  The fewer free variables in the hypothesis, the better. \u000A The idea is to form a rule that fires predictably.  Later in this tutorial \u000A you'll get some practice formulating strong rules.</p> \u000A \u000A <p>* <b>Choosing the Conclusion</b>: If a lemma is an implication, you have to \u000A choose what the conclusion will be. (You'll also have to ``orient'' that \u000A conclusion by choosing a left-hand side and a right-hand side, but we discuss \u000A that below).  You can swap the conclusion and any hypothesis by negating both, \u000A producing a different conclusion.  There are generally two (possibly \u000A conflicting) heuristics for deciding which part of the formula should be the \u000A conclusion:</p> \u000A \u000A <p><i>Choosing the Conclusion Heuristic 1</i>: Can you make the conclusion be \u000A an <v>EQUAL</v> or <v>IFF</v> expression that has a ``heavy term'' on one side? \u000A That will make a rule that replaces the heavy term with a lighter one.  We \u000A discuss this situation more below.</p> \u000A \u000A <p><i>Choosing the Conclusion Heuristic 2</i>: Can you make the conclusion be \u000A a non-propositional term that contains all the variables mentioned in the \u000A hypotheses?  By ``non-propositional'' we mean a term that is not just the \u000A propositional combination (e.g., with <v>AND</v> or <v>OR</v>) of other terms but \u000A instead some call of some ``heavy'' function?  If your conclusion contains all \u000A the variables mentioned in the hypotheses, matching it will instantiate all \u000A the variables in the hypotheses.  That way ACL2 will not have to guess \u000A instantiations of unbound variables when it tries to relieve the hypotheses. \u000A It is not very good at guessing.</p> \u000A \u000A <p>* <b>Orienting the Conclusion</b>: If the conclusion is an <v>EQUAL</v> or an \u000A <v>IFF</v>, you have to decide which is the left-hand side and which is the \u000A right.  If the conclusion is <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see></v> <i>lhs</i><v>)</v>, then the left-hand \u000A side is <i>lhs</i> and the right-hand side is <v>NIL</v>.  If the conclusion is \u000A not an <v>EQUAL</v>, an <v>IFF</v>, or a <v>NOT</v> then the conclusion itself will \u000A be the left-hand side and the right-hand side will be <v>T</v>.  If your lemma \u000A was created by looking at Key Checkpoints while using The Method, the \u000A left-hand side should match some term in that checkpoint.</p> \u000A \u000A <p>Remember, the left-hand side is the ``trigger'' that will make the rule \u000A fire.  It is the pattern that ACL2 will be looking for.</p> \u000A \u000A <p>* <b>Pay attention to the free variables</b>: Look at the variables that \u000A occur in the pattern (the left-hand side) and compare them to the variables \u000A that occur in the hypotheses.  Does some hypothesis contain a variable, say \u000A <i>v</i>, that is not in the pattern?  We call <i>v</i> a <i>free variable</i> \u000A because it will not be assigned a value (``bound'') by the process of pattern \u000A matching.  ACL2 will have to guess a value for <i>v</i>.  If some hypothesis \u000A contains <i>v</i> as a free variable, ask whether more than one hypothesis \u000A contains <i>v</i>?  ACL2 uses the first hypothesis containing a free <i>v</i> \u000A to guide its guess for <i>v</i>.  To ``guess'' a value for <i>v</i>, ACL2 uses \u000A that hypothesis as a pattern and tries to match it against the assumptions in \u000A the checkpoint formula being proved.  This means that key hypothesis must be \u000A in normal form, to match the rewritten assumptions of the goal.  It also means \u000A that you should reorder the hypotheses to put the most unusual hypothesis \u000A containing a free <i>v</i> first in the list of conjuncts.  For example, if \u000A <v>v</v> is free in two hypotheses, <v>(<see topic=\"ACL2____NATP\">natp</see> v)</v> and <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> v a)\u000A b)</v>, then we recommend putting the <v>member</v> term first.  There are likely \u000A to be many terms in the goal satisfying the <v>natp</v> hypothesis -- or none if \u000A <v>natp</v> has expanded to an integer inequality -- while there are likely to \u000A be few terms satisfying the <v>member</v> hypothesis, especially if <v>a</v> and \u000A <v>b</v> are bound by the left-hand side of the rule.</p> \u000A \u000A <p>Here are some (possibly conflicting) heuristics for choosing the left-hand \u000A side:</p> \u000A \u000A <p><i>Choose a Left-Hand Side that Occurs in a Key Checkpoint</i>: If you use \u000A the Method you will tend to do this anyway, because you'll see terms in the \u000A Key Checkpoints that you want to get rid of.  But many moderately experienced \u000A users ``look ahead'' to how the proof will go and formulate a few anticipatory \u000A rules with the idea of guiding ACL2 down the preferred path to the proof. \u000A When you do that, you risk choosing left-hand sides that won't actually arise \u000A in the problem.  So when you formulate anticipatory rules, pay special \u000A attention to the functions and terms you put in the left-hand sides.  The next \u000A few paragraphs deal with specific cases.</p> \u000A \u000A <p><i>Avoid Non-Recursive Functions in the Left-Hand Side</i>: If the \u000A left-hand side contains a call of a defined function whose definition is not \u000A recursive, then it will almost never match any target in the formula being \u000A rewritten unless the function is disabled.  Suppose for example you have \u000A defined <v>SQ</v> so that <v>(SQ x)</v> is <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x x)</v>.  Suppose you considered \u000A choosing a left-hand side like <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (SQ x) (SQ y))</v>.  Suppose you hoped it \u000A would hit the target <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (SQ A) (SQ B))</v> in some formula.  But when ACL2 \u000A simplifies the formula, it will first rewrite that target to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> A A) (<see topic=\"COMMON-LISP_____A2\">*</see> B B))</code> \u000A \u000A <p>by expanding the definition of <v>SQ</v>, since it could do so without \u000A introducing any recursive calls.  But now the target won't match your rule. \u000A By choosing a left-hand side that occurs in a Key Checkpoint (and is not one \u000A of a handful of abbreviations ACL2 uses in its output like <v>AND</v>, \u000A <v>NOT</v>), you'll avoid this problem since <v>SQ</v> will have already been \u000A expanded before the Key Checkpoint is printed.</p> \u000A \u000A <p><i>Disable Non-Recursive Functions</i>: If you insist on a left-hand side \u000A that contains calls of non-recursive functions, remember to disable those \u000A non-recursive functions after you've proved all the rules you want about them. \u000A By disabling <v>SQ</v> you can prevent ACL2 from expanding the definition as it \u000A did above.  Sometimes you will define a function non-recursively to formalize \u000A some concept that is common in your application and you will want to create a \u000A sort of algebra of rules about the function.  By all means do so, so you can \u000A conduct your formal reasoning in terms of the concepts you're informally \u000A manipulating.  But after proving the required laws, disable the non-recursive \u000A concept so that ACL2 just uses your laws and not the messy definition.</p> \u000A \u000A <p><i>Choose a Left-Hand Side Already in Simplest Form</i>: This is a \u000A generalization of the advice above.  If any rule will rewrite your left-hand \u000A side, it will prevent your rule from matching any target.  For example, if you \u000A write a left-hand side like <v>(foo (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)))</v> then it would never \u000A match any target!  The reason is that even if <v>(FOO (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> A B)))</v> did \u000A occur in some goal formula, before ACL2 would try your rule about <v>foo</v> it \u000A will use the obvious rule about <v>CAR</v> and <v>CONS</v> to transform your \u000A imagined target to <v>(FOO A)</v>.  Thus, your rule would not match.  So you \u000A have to keep in mind <i>all your other rules</i> when you choose a left-hand \u000A side (and when you choose the hypotheses to guide free variable selection). \u000A If you always choose a pattern that matches a term in a Key Checkpoint, you \u000A avoid this problem.  Also see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> example \u000A <v>books/demos/knuth-bendix-problem-1.lisp</v>.</p> \u000A \u000A <p><i>Make Sure the Left-Hand Side is ``Heavier'' than the Right</i>: \u000A Sometimes this is obvious, as when you choose <v>(REV (REV x))</v> for the \u000A left-hand side and <v>x</v> for the right.  But what do you about <v>(REV\u000A (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y))</v> versus <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV y) (REV x))</v>?  Most of the time we \u000A choose to drive the heaviest function (in this case <v>REV</v>) down toward the \u000A variables, lifting the lighter function (<v>APPEND</v>) up so that we can reason \u000A about the lighter function's interaction with the surrounding ``matrix'' of \u000A the formula.  So we'd rewrite <v>(REV (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y))</v> to <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV y)\u000A (REV x))</v>, not vice versa.</p> \u000A \u000A <p><i>Alternative Ways to Talk About the Same Thing</i>: If your problem and \u000A specification use two different ways to talk about the same thing, choose one \u000A form and rewrite the other into that form.  For example, the ACL2 built-in \u000A <v>nth</v> returns the nth element of a list, and the built-in function \u000A <v>nthcdr</v> returns the nth <v>cdr</v> of a list.  They are defined \u000A independently.  But <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> n x)</v> is the same thing as <v>(<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n\u000A x))</v>.  Since <v>nth</v> can be expressed in terms of <v>nthcdr</v> but not vice \u000A versa, it is clear we should prove <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n x) (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n x)))</v> \u000A as a rewrite rule if both <v>nth</v> and <v>nthcdr</v> are involved in the \u000A problem.</p> \u000A \u000A <p><i>Don't Let Computational Efficiency Dictate the Terms</i>: If you have \u000A two functions that are equivalent (perhaps one was defined to be \u000A computationally more efficient), prove their equivalence as a rewrite rule \u000A that eliminates the more complicated function.  An extreme example would be a \u000A model that uses a sophisticated data structure (like a balanced binary tree, \u000A red-black tree, ordered array, or hash table) to implement something simple \u000A like an association of keys to values.  By proving the equivalence as stated \u000A you can eliminate the messy function early and do the bulk of your reasoning \u000A in terms of its simple specification.</p> \u000A \u000A <p>The best ACL2 users become very good at keeping all these things in mind \u000A when designing their rewrite rules.  Practice makes perfect.  Don't be afraid \u000A during your learning of ACL2 to undo the rules you first invented and try to \u000A make better ones.</p> \u000A \u000A <p>Finally, be patient!  There will be times when you think to yourself ``Why \u000A should I spend my time thinking of rules that guide ACL2?  I know the proof!'' \u000A There are two reasons.  First, you may ``know'' the proof but you may well be \u000A wrong and part-way through this whole exercise you may realize that you're \u000A missing a major hypothesis or special case that breaks your whole conception \u000A of the problem.  The proof is in the details.  Second, most of the time the \u000A library of rules you develop in this process will be used over and over again \u000A on variants of the main problem in the months and years ahead.  This is \u000A sometimes called the <i>proof maintenance</i> problem.  Theorems don't suffer \u000A bit rot!  But the artifacts you're modeling change and you will need to prove \u000A new versions of old theorems.  A good general purpose library makes this much \u000A easier.</p> \u000A \u000A <p>We now recommend that you practice inventing strong rules; see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>.</p> \u000A \u000A <p>For advice on handling specific kinds of formulas and definitions, see \u000A <see topic=\"ACL2____SPECIFIC-KINDS-OF-FORMULAS-AS-REWRITE-RULES\">specific-kinds-of-formulas-as-rewrite-rules</see>.</p> \u000A \u000A <p>For more information about the rewriter works and how rules are created, \u000A see <see topic=\"ACL2____FURTHER-INFORMATION-ON-REWRITING\">further-information-on-rewriting</see>.</p> \u000A \u000A <p>If you are working your way through the tutorial introduction to the \u000A theorem prover, use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____GUARD-EVALUATION-TABLE":[["Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> for an introduction to the topic \u000A discussed here.  Also see <see topic=\"ACL2____GUARD\">guard</see> for a general discussion of guards, and \u000A see <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-SCRIPT\">guard-evaluation-examples-script</see> for a script that illustrates \u000A combinations presented below.</p> \u000A \u000A <p>Note: The default setting for guard-checking (that is, the initial value \u000A for <v>(<see topic=\"ACL2_____04\">@</see> guard-checking-on)</v>) is <v>T</v>.</p> \u000A \u000A <p>The table below illustrates the interaction of the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> with \u000A the value supplied to <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>.  The first row considers \u000A functions defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode; the other two consider \u000A functions defined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  The columns correspond to four \u000A values of state global <v>'guard-checking-on</v>, as supplied to <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>.  (A fifth value, <v>:nowarn</v>, is similar to <v>t</v> but \u000A suppresses warnings encountered with <v>t</v> (as explained in those warning \u000A messages), and is not considered here.)  During proofs, \u000A <v>'guard-checking-on</v> is set to <v>nil</v> regardless of how this variable has \u000A been set in the top-level loop.</p> \u000A \u000A <p>Below this table, we make some comments about its entries, ordered by row \u000A and then by column.  For example, when we refer to ``b2'' we are discussing \u000A the execution of a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function whose guards have not \u000A been verified, after having executed <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v>\u000A :all</v>.</p> \u000A \u000A <code>  guard-checking-on:  (1)t      (2):all   (3):none   (4)nil\u000A\u000A(a) :program             a1        a2        a3        a4\u000A(b) guards not verified  b1        b2        b3        b4\u000A(c) guards verified      c1        c2        c3        c4</code> \u000A \u000A <p>a1. Check the <see topic=\"ACL2____GUARD\">guard</see> upon entry, then use the raw Lisp code if the \u000A guard checks (else cause an error).  This is a common setting when one wants a \u000A little guard checking but also wants the efficiency of raw Lisp.  But note \u000A that you can get raw Lisp errors.  For example, if you make the definition \u000A <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))</v> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and execute \u000A <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> t</v>, and then execute <v>(foo 3)</v>, you \u000A will likely get an error from the call <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v> made in raw Lisp.</p> \u000A \u000A <p>a2. For built-in (predefined) functions, see a1 instead. \u000A Otherwise:<br></br> \u000A \u000A Check the <see topic=\"ACL2____GUARD\">guard</see>, without exception.  Thus, we never run the raw Lisp \u000A code in this case.  This can be useful when testing <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode \u000A functions, but you may want to run <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt><v> t</v> or at least \u000A <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt><v> :exec</v> in this case, so that the execution is done \u000A using compiled code.</p> \u000A \u000A <p>a3. For built-in (predefined) functions, see a4 instead. \u000A Otherwise:<br></br> \u000A \u000A Do not check the <see topic=\"ACL2____GUARD\">guard</see>.  For <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions, we \u000A never run the raw Lisp code in this case; so if you care about efficiency, see \u000A the comment in a2 above about <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>.  This combination is useful \u000A if you are using ACL2 as a programming language and do not want to prove \u000A theorems about your functions or suffer <see topic=\"ACL2____GUARD\">guard</see> violations.  In this \u000A case, you can forget about any connection between ACL2 and Common Lisp.</p> \u000A \u000A <p>a4. Run the raw Lisp code without checking <see topic=\"ACL2____GUARD\">guard</see>s at all.  Thus, for \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions, the <v>nil</v> setting is often preferable \u000A to the <v>:none</v> setting because you get the efficiency of raw Lisp \u000A execution.  However, with <v>nil</v> you can therefore get hard Lisp errors as \u000A in a1 above.</p> \u000A \u000A <p>b1. Guards are checked at the top-level, though not at self-recursive \u000A calls.  We never run the raw Lisp code in this case; guards would need to be \u000A verified first.</p> \u000A \u000A <p>b2. Unlike the <v>t</v> setting, guards are checked even on self-recursive \u000A calls.  But like the <v>t</v> setting, we do not run the raw Lisp code.  Use \u000A this setting if you want guards checked on each recursive call in spite of the \u000A cost of doing so.</p> \u000A \u000A <p>b3, b4. Execution avoids the raw Lisp code and never checks guards.  The \u000A <v>nil</v> and <v>:none</v> settings behave the same in this case (i.e., for \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions whose guards have not been verified), \u000A except that recursive calls are never inlined for <v>:none</v> and tracing (see \u000A <see topic=\"COMMON-LISP____TRACE\">trace</see>) will show recursive calls for <v>:none</v> but not for \u000A <v>nil</v>.</p> \u000A \u000A <p>c1, c2. Guards are checked.  If the checks pass, evaluation takes place \u000A using the raw Lisp code.  If the checks fail, we get a guard violation. \u000A Either way, we do not execute ``in the logic''; we only execute using the raw \u000A Lisp code.  Note that <v>t</v> and <v>:all</v> behave the same in this case, \u000A (i.e. for <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions whose <see topic=\"ACL2____GUARD\">guard</see>s have been \u000A verified).</p> \u000A \u000A <p>c3, c4. For the <v>:none</v> and <v>nil</v> settings, <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A functions whose guards have been verified will never cause guard violations. \u000A However, with <v>nil</v> and for built-in functions in <v>:logic</v> mode, guards \u000A are still checked: if the check succeeds, then evaluation is done using the \u000A raw Lisp code, and if not, it is done by the ``logic'' code, including \u000A self-recursive calls (though unlike the <v>t</v> case, we will not see a warning \u000A about this).  But with <v>:none</v> for user-defined functions, no guard \u000A checking is done, and the only time the raw Lisp code will be executed is when \u000A the guard is <v>t</v> and guards are verified at the time the executable \u000A counterpart of the function is defined (i.e., when the function is admitted \u000A unless it is later defined again and compiled using <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>). \u000A Thus, if you use <v>:none</v> and you want a function <v>(foo x)</v> with guard \u000A <v>(g x)</v> to execute using raw Lisp code, you can write a ``wrapper''function \u000A with a guard of <v>t</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo-wrap (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (g x)\u000A      (foo x)\u000A    'do-not-case))</code> \u000A \u000A <p>If you want the speed of executing raw Lisp code and you have non-trivial \u000A guards on functions that you want to call at the top-level, use <v>nil</v> \u000A rather than <v>:none</v>.</p>"],

 "ACL2____FORCING-ROUND":[["Force"],"ACL2 Sources","ACL2","<p>If ACL2 ``<see topic=\"ACL2____FORCE\">force</see>s'' some hypothesis of some rule to be true, \u000A it is obliged later to prove the hypothesis.  See <see topic=\"ACL2____FORCE\">force</see>.  ACL2 delays \u000A the consideration of <see topic=\"ACL2____FORCE\">force</see>d hypotheses until the main goal has been \u000A proved.  It then undertakes a new round of proofs in which the main goal is \u000A essentially the conjunction of all hypotheses <see topic=\"ACL2____FORCE\">force</see>d in the preceding \u000A proof.  Call this round of proofs the ``Forcing Round.''  Additional \u000A hypotheses may be <see topic=\"ACL2____FORCE\">force</see>d by the proofs in the Forcing Round.  The \u000A attempt to prove these hypotheses is delayed until the Forcing Round has been \u000A successfully completed.  Then a new Forcing Round is undertaken to prove the \u000A recently <see topic=\"ACL2____FORCE\">force</see>d hypotheses and this continues until no hypotheses are \u000A <see topic=\"ACL2____FORCE\">force</see>d.  Thus, there is a succession of Forcing Rounds.</p> \u000A \u000A <p>The Forcing Rounds are enumerated starting from 1.  The Goals and Subgoals \u000A of a Forcing Round are printed with the round's number displayed in square \u000A brackets.  Thus, <v>&quot;[1]Subgoal 1.3&quot;</v> means that the goal in question is \u000A Subgoal 1.3 of the 1st forcing round.  To supply a hint for use in the proof \u000A of that subgoal, you should use the goal specifier <v>&quot;[1]Subgoal 1.3&quot;</v>. \u000A See <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>.</p> \u000A \u000A <p>When a round is successfully completed &mdash; and for these purposes you \u000A may think of the proof of the main goal as being the 0th forcing round &mdash; \u000A the system collects all of the assumptions <see topic=\"ACL2____FORCE\">force</see>d by the just-completed \u000A round.  Here, an assumption should be thought of as an implication, \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> context hyp)</v>, where context describes the context in which hyp \u000A was assumed true.  Before undertaking the proofs of these assumptions, we try \u000A to ``clean them up'' in an effort to reduce the amount of work required.  This \u000A is often possible because the <see topic=\"ACL2____FORCE\">force</see>d assumptions are generated by the \u000A same rule being applied repeatedly in a given context.</p> \u000A \u000A <p>By delaying and collecting the <v>forced</v> assumptions until the completion \u000A of the ``main goal'' we gain two advantages.  First, the user gets \u000A confirmation that the ``gist'' of the proof is complete and that all that \u000A remains are ``technical details.''  Second, by delaying the proofs of the \u000A <see topic=\"ACL2____FORCE\">force</see>d assumptions ACL2 can undertake the proof of each assumption only \u000A once, no matter how many times it was <see topic=\"ACL2____FORCE\">force</see>d in the main goal.</p> \u000A \u000A <p>In order to indicate which proof steps of the previous round were \u000A responsible for which <see topic=\"ACL2____FORCE\">force</see>d assumptions, we print a sentence \u000A explaining the origins of each newly <see topic=\"ACL2____FORCE\">force</see>d goal.  For example,</p> \u000A \u000A <code>[1]Subgoal 1, below, will focus on\u000A(GOOD-INPUTP (XTRANS I)),\u000Awhich was forced in\u000A Subgoal 14, above,\u000A  by applying (:REWRITE PRED-CRUNCHER) to\u000A  (PRED (XTRANS I) I),\u000A and\u000A Subgoal 28, above,\u000A  by applying (:REWRITE PRED-CRUNCHER) to\u000A  (PRED (XTRANS I) I).</code> \u000A \u000A <p>In this entry, ``[1]Subgoal 1'' is the name of a goal which will be proved \u000A in the next forcing round.  On the next line we display the <see topic=\"ACL2____FORCE\">force</see>d \u000A hypothesis, call it <v>x</v>, which is <v>(good-inputp (xtrans i))</v> in this \u000A example.  This term will be the conclusion of the new subgoal.  Since the new \u000A subgoal will be printed in its entirety when its proof is undertaken, we do \u000A not here exhibit the context in which <v>x</v> was <see topic=\"ACL2____FORCE\">force</see>d.  The sentence \u000A then lists (possibly a succession of) a goal name from the just-completed \u000A round and some step in the proof of that goal that <see topic=\"ACL2____FORCE\">force</see>d <v>x</v>.  In \u000A the example above we see that Subgoals 14 and 28 of the just-completed proof \u000A <see topic=\"ACL2____FORCE\">force</see>d <v>(good-inputp (xtrans i))</v> by applying <v>(:rewrite\u000A pred-cruncher)</v> to the term <v>(pred (xtrans i) i)</v>.</p> \u000A \u000A <p>If one were to inspect the theorem prover's description of the proof steps \u000A applied to Subgoals 14 and 28 one would find the word ``<see topic=\"ACL2____FORCE\">force</see>d'' (or \u000A sometimes ``forcibly'') occurring in the commentary.  Whenever you see that \u000A word in the output, you know you will get a subsequent forcing round to deal \u000A with the hypotheses <see topic=\"ACL2____FORCE\">force</see>d.  Similarly, if at the beginning of a \u000A forcing round a <see topic=\"ACL2____RUNE\">rune</see> is blamed for causing a <see topic=\"ACL2____FORCE\">force</see> in some \u000A subgoal, inspection of the commentary for that subgoal will reveal the word \u000A ``<see topic=\"ACL2____FORCE\">force</see>d'' after the rule name blamed.</p> \u000A \u000A <p>Most <see topic=\"ACL2____FORCE\">force</see>d hypotheses come from within the prover's simplifier. \u000A When the simplifier encounters a hypothesis of the form <v>(<see topic=\"ACL2____FORCE\">force</see> hyp)</v> it \u000A first attempts to establish it by rewriting <v>hyp</v> to, say, <v>hyp'</v>.  If \u000A the truth or falsity of <v>hyp'</v> is known, forcing is not required. \u000A Otherwise, the simplifier actually <see topic=\"ACL2____FORCE\">force</see>s <v>hyp'</v>.  That is, the \u000A <v>x</v> mentioned above is <v>hyp'</v>, not <v>hyp</v>, when the <see topic=\"ACL2____FORCE\">force</see>d \u000A subgoal was generated by the simplifier.</p> \u000A \u000A <p>Once the system has printed out the origins of the newly <see topic=\"ACL2____FORCE\">force</see>d \u000A goals, it proceeds to the next forcing round, where those goals are \u000A individually displayed and attacked.</p> \u000A \u000A <p>At the beginning of a forcing round, the <see topic=\"ACL2____ENABLE\">enable</see>d structure defaults \u000A to the global <see topic=\"ACL2____ENABLE\">enable</see>d structure.  For example, suppose some <see topic=\"ACL2____RUNE\">rune</see>, <v>rune</v>, is globally <see topic=\"ACL2____ENABLE\">enable</see>d.  Suppose in some event you <see topic=\"ACL2____DISABLE\">disable</see> the <see topic=\"ACL2____RUNE\">rune</see> at <v>&quot;Goal&quot;</v> and successfully prove the goal but \u000A <see topic=\"ACL2____FORCE\">force</see> <v>&quot;[1]Goal&quot;</v>.  Then during the proof of <v>&quot;[1]Goal&quot;</v>, \u000A <see topic=\"ACL2____RUNE\">rune</see> is <see topic=\"ACL2____ENABLE\">enable</see>d ``again.''  The right way to think about this is \u000A that the <see topic=\"ACL2____RUNE\">rune</see> is ``still'' <see topic=\"ACL2____ENABLE\">enable</see>d.  That is, it is <see topic=\"ACL2____ENABLE\">enable</see>d globally and each forcing round resumes with the global <see topic=\"ACL2____ENABLE\">enable</see>d structure.</p>"],

 "ACL2____BUILT-IN-CLAUSE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> acl2-count-abl\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                     (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)))\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (abl x))\u000A                   (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)))\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                     (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> nil x)))\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (abl x))\u000A                   (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))))\u000A  :rule-classes :built-in-clause)</code> \u000A \u000A <p>A <v>:built-in-clause</v> rule can be built from any formula other than \u000A propositional tautologies.  Roughly speaking, the system uses the list of \u000A built-in clauses as the first method of proof when attacking a new goal.  Any \u000A goal that is subsumed by a built in clause is proved ``silently.''</p> \u000A \u000A <p>ACL2 maintains a set of ``built-in'' clauses that are used to short-circuit \u000A certain theorem proving tasks.  We discuss this at length below.  When a \u000A theorem is given the rule class <v>:built-in-clause</v> ACL2 flattens the <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> and <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> structure of the <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> formula so \u000A as to obtain a set of formulas whose conjunction is equivalent to the given \u000A corollary.  It then converts each of these to clausal form and adds each \u000A clause to the set of built-in clauses.</p> \u000A \u000A <p>The example above (regardless of the definition of <v>abl</v>) will build in \u000A two clauses,</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (abl x)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))}</code> \u000A \u000A <p>and</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> nil x)\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (abl x)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))}.</code> \u000A \u000A <p>We now give more background.</p> \u000A \u000A <p>Recall that a clause is a set of terms, implicitly representing the \u000A disjunction of the terms.  Clause <v>c1</v> is ``subsumed'' by clause <v>c2</v> if \u000A some instance of <v>c2</v> is a subset <v>c1</v>.</p> \u000A \u000A <p>For example, let <v>c1</v> be</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> l))\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> l))}.</code> \u000A \u000A <p>Then <v>c1</v> is subsumed by <v>c2</v>, shown below,</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))\u000A ; second term omitted here\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))}</code> \u000A \u000A <p>because we can instantiate <v>x</v> in <v>c2</v> with <v>l</v> to obtain a subset \u000A of <v>c1</v>.</p> \u000A \u000A <p>Observe that <v>c1</v> is the clausal form of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> l)\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a (<see topic=\"COMMON-LISP____CAR\">car</see> l))))\u000A         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> l))),</code> \u000A \u000A <p><v>c2</v> is the clausal form of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> l)\u000A         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> l)))</code> \u000A \u000A <p>and the subsumption property just means that <v>c1</v> follows trivially from \u000A <v>c2</v> by instantiation.</p> \u000A \u000A <p>The set of built-in clauses is just a set of known theorems in clausal \u000A form.  Any formula that is subsumed by a built-in clause is thus a theorem. \u000A If the set of built-in theorems is reasonably small, this little theorem \u000A prover is fast.  ACL2 uses the ``built-in clause check'' in four places: (1) \u000A at the top of the iteration in the prover -- thus if a built-in clause is \u000A generated as a subgoal it will be recognized when that goal is considered, (2) \u000A within the simplifier so that no built-in clause is ever generated by \u000A simplification, (3) as a filter on the clauses generated to prove the \u000A termination of recursively <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d functions and (4) as a filter on \u000A the clauses generated to verify the guards of a function.</p> \u000A \u000A <p>The latter two uses are the ones that most often motivate an extension to \u000A the set of built-in clauses.  Frequently a given formalization problem \u000A requires the definition of many functions which require virtually identical \u000A termination and/or guard proofs.  These proofs can be short-circuited by \u000A extending the set of built-in clauses to contain the most general forms of the \u000A clauses generated by the definitional schemes in use.</p> \u000A \u000A <p>The attentive user might have noticed that there are some recursive \u000A schemes, e.g., recursion by <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> after testing <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>, that ACL2 \u000A just seems to ``know'' are ok, while for others it generates measure clauses \u000A to prove.  Actually, it always generates measure clauses but then filters out \u000A any that pass the built-in clause check.  When ACL2 is initialized, the clause \u000A justifying <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> recursion after a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> test is added to the \u000A set of built-in clauses.  (That clause is <v>c2</v> above.)</p> \u000A \u000A <p>Note that only a subsumption check is made; no rewriting or simplification \u000A is done.  Thus, if we want the system to ``know'' that <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> recursion \u000A is ok after a negative <tt><see topic=\"COMMON-LISP____ATOM\">atom</see></tt> test (which, by the definition of <tt><see topic=\"COMMON-LISP____ATOM\">atom</see></tt>, is the same as a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> test), we have to build in a second \u000A clause.  The subsumption algorithm does not ``know'' about commutative \u000A functions.  Thus, for predictability, we have built in commuted versions of \u000A each clause involving commutative functions.  For example, we build in \u000A both</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x)\u000A (<see topic=\"COMMON-LISP_____D3\">=</see> x 0)\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP_____B2\">+</see> -1 x)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))}</code> \u000A \u000A <p>and the commuted version</p> \u000A \u000A <code>{(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x)\u000A (<see topic=\"COMMON-LISP_____D3\">=</see> 0 x)\u000A (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP_____B2\">+</see> -1 x)) (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))}</code> \u000A \u000A <p>so that the user need not worry whether to write <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x 0)</v> or <v>(<see topic=\"COMMON-LISP_____D3\">=</see> 0\u000A x)</v> in definitions.</p> \u000A \u000A <p><v>:built-in-clause</v> rules added by the user can be enabled and \u000A disabled.</p>"],

 "ACL2____BDD-ALGORITHM":[["Bdd"],"ACL2 Sources","ACL2","<p>The BDD algorithm in ACL2 uses a combination of manipulation of \u000A <v>IF</v> terms and unconditional rewriting.  In this discussion we begin with \u000A some relevant mathematical theory.  This is followed by a description of how \u000A ACL2 does BDDs, including concluding discussions of soundness, completeness, \u000A and efficiency.</p> \u000A \u000A <p>We recommend that you read the other documentation about BDDs in ACL2 \u000A before reading the rather technical material that follows.  See <see topic=\"ACL2____BDD\">bdd</see>.</p> \u000A \u000A <p>Here is an outline of our presentation.  Readers who want a user \u000A perspective, without undue mathematical theory, may wish to skip to Part (B), \u000A referring to Part (A) only on occasion if necessary.</p> \u000A \u000A <p>(A) <b>Mathematical Considerations</b></p> \u000A \u000A <blockquote> \u000A \u000A <p>(A1) BDD term order</p> \u000A \u000A <p>(A2) BDD-constructors and BDD terms, and their connection with aborting the \u000A BDD algorithm</p> \u000A \u000A <p>(A3) Canonical BDD terms</p> \u000A \u000A <p>(A4) A theorem stating the equivalence of provable and syntactic equality \u000A for canonical BDD terms</p> \u000A \u000A </blockquote> \u000A \u000A <p>(B) <b>Algorithmic Considerations</b></p> \u000A \u000A <blockquote> \u000A \u000A <p>(B1) BDD rules (rules used by the rewriting portion of the ACL2 BDD \u000A algorithm)</p> \u000A \u000A <p>(B2) Terms ``known to be Boolean''</p> \u000A \u000A <p>(B3) An ``IF-lifting'' operation used by the algorithm, as well as an \u000A iterative version of that operation</p> \u000A \u000A <p>(B4) The ACL2 BDD algorithm</p> \u000A \u000A <p>(B5) Soundness and Completeness of the ACL2 BDD algorithm</p> \u000A \u000A <p>(B6) Efficiency considerations</p></blockquote> \u000A \u000A <p>(A) <b>Mathematical Considerations</b></p> \u000A \u000A <p>(A1) <i>BDD term order</i></p> \u000A \u000A <p>Our BDD algorithm creates a total ``BDD term order'' on ACL2 terms, on the \u000A fly.  We use this order in our discussions below of IF-lifting and of \u000A canonical BDD terms, and in the algorithm's use of commutativity.  The \u000A particular order is unimportant, except that we guarantee (for purposes of \u000A commutative functions) that constants are smaller in this order than \u000A non-constants.</p> \u000A \u000A <p>(A2) <i>BDD-constructors</i> (assumed to be <v>'(<see topic=\"COMMON-LISP____CONS\">cons</see>)</v>) and <i>BDD \u000A terms</i></p> \u000A \u000A <p>We take as given a list of function symbols that we call the \u000A ``BDD-constructors.''  By default, the only BDD-constructor is <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, \u000A although it is legal to specify any list of function symbols as the \u000A BDD-constructors, either by using the <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see> (see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>) or by supplying a <v>:BDD-CONSTRUCTORS</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>).  Warning: this capability is largely untested and may produce \u000A undesirable results.  Henceforth, except when explicitly stated to the \u000A contrary, we assume that BDD-constructors is <v>'(<see topic=\"COMMON-LISP____CONS\">cons</see>)</v>.</p> \u000A \u000A <p>Roughly speaking, a <see topic=\"ACL2____BDD\">bdd</see> term is the sort of <see topic=\"ACL2____TERM\">term</see> produced by \u000A our BDD algorithm, namely a tree with all <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> nodes lying above all \u000A non-<v>CONS</v> nodes.  More formally, a <see topic=\"ACL2____TERM\">term</see> is said to be a <see topic=\"ACL2____BDD\">bdd</see> \u000A term if it contains <b>no</b> subterm of either of the following forms, where \u000A <v>f</v> is not <v>CONS</v>.</p> \u000A \u000A <code>(f ... (<see topic=\"COMMON-LISP____CONS\">CONS</see> ...) ...)\u000A\u000A(f ... 'x ...)  ; where (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) = t</code> \u000A \u000A <p>We will see that whenever the BDD algorithm attempts to create a <see topic=\"ACL2____TERM\">term</see> that is not a <see topic=\"ACL2____BDD\">bdd</see> term, it aborts instead.  Thus, whenever the \u000A algorithm completes without aborting, it creates a <see topic=\"ACL2____BDD\">bdd</see> term.</p> \u000A \u000A <p>(A3) <i>Canonical BDD terms</i></p> \u000A \u000A <p>We can strengthen the notion of ``BDD term'' to a notion of ``canonical BDD \u000A term'' by imposing the following additional requirements, for every subterm of \u000A the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> x y z)</v>:</p> \u000A \u000A <blockquote><p>(a) <v>x</v> is a variable, and it precedes (in the BDD term \u000A order) every variable occurring in <v>y</v> or <v>z</v>;</p> \u000A \u000A <p>(b) <v>y</v> and <v>z</v> are syntactically distinct; and,</p> \u000A \u000A <p>(c) it is not the case that <v>y</v> is <v>t</v> and <v>z</v> is \u000A <v>nil</v>.</p></blockquote> \u000A \u000A <p>We claim that it follows easily from our description of the BDD algorithm \u000A that every term it creates is a canonical BDD term, assuming that the \u000A variables occurring in all such terms are treated by the algorithm as being \u000A Boolean (see (B2) below) and that the terms contain no function symbols other \u000A than <v>IF</v> and <v>CONS</v>.  Thus, under those assumptions the following \u000A theorem shows that the BDD algorithm never creates distinct terms that are \u000A provably equal, a property that is useful for completeness and efficiency (as \u000A we explain in (B5) and (B6) below).</p> \u000A \u000A <p>(A4) <i>Provably equal canonical BDD terms are identical</i></p> \u000A \u000A <p>We believe that the following theorem and proof are routine extensions of a \u000A standard result and proof to terms that allow calls of <v>CONS</v>.</p> \u000A \u000A <p><b>Theorem</b>.  Suppose that <v>t1</v> and <v>t2</v> are canonical BDD terms \u000A that contain no function symbols other than <v>IF</v> and <v>CONS</v>.  Also \u000A suppose that <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 t2)</v> is a theorem.  Then <v>t1</v> and <v>t2</v> are \u000A syntactically identical.</p> \u000A \u000A <p>Proof of theorem: By induction on the total number of symbols occurring in \u000A these two terms.  First suppose that at least one term is a variable; without \u000A loss of generality let it be <v>t1</v>.  We must prove that <v>t2</v> is \u000A syntactically the same as <v>t1</v>.  Now it is clearly consistent that \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 t2)</v> is false if <v>t2</v> is a call of <v>CONS</v> (to see this, \u000A simply let <v>t1</v> be an value that is not a <v>CONSP</v>).  Similarly, <v>t2</v> \u000A cannot be a constant or a variable other than <v>t1</v>.  The remaining \u000A possibility to rule out is that <v>t2</v> is of the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> t3 t4 t5)</v>, \u000A since by assumption its function symbol must be <v>IF</v> or <v>CONS</v> and we \u000A have already handled the latter case.  Since <v>t2</v> is canonical, we know \u000A that <v>t3</v> is a variable.  Since <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 t2)</v> is provable, i.e.,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 (<see topic=\"COMMON-LISP____IF\">if</see> t3 t4 t5))</code> \u000A \u000A <p>is provable, it follows that we may substitute either <v>t</v> or <v>nil</v> \u000A for <v>t3</v> into this equality to obtain two new provable equalities.  First, \u000A suppose that <v>t1</v> and <v>t3</v> are distinct variables.  Then these \u000A substitutions show that <v>t1</v> is provably equal to both <v>t4</v> and <v>t5</v> \u000A (since <v>t3</v> does not occur in <v>t4</v> or <v>t5</v> by property (a) above, as \u000A <v>t2</v> is canonical), and hence <v>t4</v> and <v>t5</v> are provably equal to \u000A each other, which implies by the inductive hypothesis that they are the same \u000A term &mdash; and this contradicts the assumption that <v>t2</v> is canonical \u000A (property (b)).  Therefore <v>t1</v> and <v>t3</v> are the same variable, i.e., \u000A the equality displayed above is actually <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 (<see topic=\"COMMON-LISP____IF\">if</see> t1 t4 t5))</v>. \u000A Substituting <v>t</v> and then <v>nil</v> for <v>t1</v> into this provable equality \u000A lets us prove <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t t4)</v> and <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> nil t5)</v>, which by the \u000A inductive hypothesis implies that <v>t4</v> is (syntactically) the term <v>t</v> \u000A and <v>t5</v> is <v>nil</v>.  That is, <v>t2</v> is <v>(<see topic=\"COMMON-LISP____IF\">IF</see> t1 t nil)</v>, which \u000A contradicts the assumption that <v>t2</v> is canonical (property (c)).</p> \u000A \u000A <p>Next, suppose that at least one term is a call of <v>IF</v>.  Our first \u000A observation is that the other term is also a call of <v>IF</v>.  For if the \u000A other is a call of <v>CONS</v>, then they cannot be provably equal, because the \u000A former has no function symbols other than <v>IF</v> and hence is Boolean when \u000A all its variables are assigned Boolean values.  Also, if the other is a \u000A constant, then both branches of the <v>IF</v> term are provably equal to that \u000A constant and hence these branches are syntactically identical by the inductive \u000A hypothesis, contradicting property (b).  Hence, we may assume for this case \u000A that both terms are calls of <v>IF</v>; let us write them as follows.</p> \u000A \u000A <code>t0:  (<see topic=\"COMMON-LISP____IF\">IF</see> t1 t2 t3)\u000Au0:  (<see topic=\"COMMON-LISP____IF\">IF</see> u1 u2 u3)</code> \u000A \u000A <p>Note that <v>t1</v> and <v>u1</v> are variables, by property (a) of canonical \u000A BDD terms.  First we claim that <v>t1</v> does not strictly precede <v>u1</v> in \u000A the BDD term order.  For suppose <v>t1</v> does strictly precede <v>u1</v>.  Then \u000A property (a) of canonical BDD terms guarantees that <v>t1</v> does not occur in \u000A <v>u0</v>.  Hence, an argument much like one used above shows that <v>u0</v> is \u000A provably equal to both <v>t2</v> (substituting <v>t</v> for <v>t1</v>) and <v>t3</v> \u000A (substituting <v>nil</v> for <v>t1</v>), and hence <v>t2</v> and <v>t3</v> are \u000A provably equal.  That implies that they are identical terms, by the inductive \u000A hypothesis, which then contradicts property (b) for <v>t0</v>.  Similarly, \u000A <v>u1</v> does not strictly precede <v>t1</v> in the BDD term order.  Therefore, \u000A <v>t1</v> and <v>u1</v> are the same variable.  By substituting <v>t</v> for this \u000A variable we see that <v>t2</v> and <v>u2</v> are provably equal, and hence they \u000A are equal by the inductive hypothesis.  Similarly, by substituting <v>nil</v> \u000A for <v>t1</v> (and <v>u1</v>) we see that <v>t3</v> and <v>u3</v> are provably, hence \u000A syntactically, equal.</p> \u000A \u000A <p>We have covered all cases in which at least one term is a variable or at \u000A least one term is a call of <v>IF</v>.  If both terms are constants, then \u000A provable and syntactic equality are clearly equivalent.  Finally, then, we may \u000A assume that one term is a call of <v>CONS</v> and the other is a constant or a \u000A call of <v>CONS</v>.  The constant case is similar to the <v>CONS</v> case if the \u000A constant is a <v>CONSP</v>, so we omit it; while if the constant is not a \u000A <v>CONSP</v> then it is not provably equal to a call of <v>CONS</v>; in fact it is \u000A provably <i>not</i> equal!</p> \u000A \u000A <p>So, we are left with a final case, in which canonical BDD terms <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> t1\u000A t2)</v> and <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> u1 u2)</v> are provably equal, and we want to show that \u000A <v>t1</v> and <v>u1</v> are syntactically equal as are <v>t2</v> and <v>u2</v>.  These \u000A conclusions are easy consequences of the inductive hypothesis, since the ACL2 \u000A axiom <v>CONS-EQUAL</v> (which you can inspect using <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>) shows \u000A that equality of the given terms is equivalent to the conjunction of <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see>\u000A t1 t2)</v> and <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> u1 u2)</v>.  Q.E.D.</p> \u000A \u000A <p>(B) <b>Algorithmic Considerations</b></p> \u000A \u000A <p>(B1) <i>BDD rules</i></p> \u000A \u000A <p>A rule of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) is said to \u000A be a ``<see topic=\"ACL2____BDD\">bdd</see> rewrite rule'' if and only if it satisfies the following \u000A criteria.  (1) The rule is <see topic=\"ACL2____ENABLE\">enable</see>d.  (2) Its <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A relation is <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.  (3) It has no hypotheses.  (4) Its <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field is <v>nil</v>, i.e., it is not a permutative rule.  (5) All \u000A variables occurring in the rule occur in its left-hand side (i.e., there are \u000A no ``free variables''; see <see topic=\"ACL2____REWRITE\">rewrite</see>).  A rule of class <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) is said to be a ``<see topic=\"ACL2____BDD\">bdd</see> definition \u000A rule'' if it satisfies all the criteria above (except (4), which does not \u000A apply), and moreover the top function symbol of the left-hand side was not \u000A recursively (or mutually recursively) defined.  Technical point: Note that \u000A this additional criterion is independent of whether or not the indicated \u000A function symbol actually occurs in the right-hand side of the rule.</p> \u000A \u000A <p>Both BDD rewrite rules and BDD definition rules are said to be ``BDD \u000A rules.''</p> \u000A \u000A <p>(B2) <i>Terms ''known to be Boolean''</i></p> \u000A \u000A <p>We apply the BDD algorithm in the context of a top-level goal to prove, \u000A namely, the goal at which the <v>:BDD</v> hint is attached.  As we run the BDD \u000A algorithm, we allow ourselves to say that a set of <see topic=\"ACL2____TERM\">term</see>s is ``known to \u000A be Boolean'' if we can verify that the goal is provable from the assumption \u000A that at least one of the terms is not Boolean.  Equivalently, we allow \u000A ourselves to say that a set of terms is ``known to be Boolean'' if we can \u000A verify that the original goal is provably equivalent to the assertion that if \u000A all terms in the set are Boolean, then the goal holds.  The notion ``known to \u000A be Boolean'' is conservative in the sense that there are generally sets of \u000A terms for which the above equivalent criteria hold and yet the sets of terms \u000A are not noted as as being ``known to be Boolean.''  However, ACL2 uses a \u000A number of tricks, including <see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning and analysis of the \u000A structure of the top-level goal, to attempt to establish that a sufficiently \u000A inclusive set of terms is known to be Boolean.</p> \u000A \u000A <p>From a practical standpoint, the algorithm determines a set of terms known \u000A to be Boolean; we allow ourselves to say that each term in this set is ``known \u000A to be Boolean.''  The algorithm assumes that these terms are indeed Boolean, \u000A and can make use of that assumption.  For example, if <v>t1</v> is known to be \u000A Boolean then the algorithm simplifies <v>(<see topic=\"COMMON-LISP____IF\">IF</see> t1 t nil)</v> to <v>t1</v>; see (iv) \u000A in the discussion immediately below.</p> \u000A \u000A <p>(B3) <i>IF-lifting</i> and the <i>IF-lifting-for-IF loop</i></p> \u000A \u000A <p>Suppose that one has a <see topic=\"ACL2____TERM\">term</see> of the form <v>(f ... (<see topic=\"COMMON-LISP____IF\">IF</see> test x y)\u000A ...)</v>, where <v>f</v> is a function symbol other than <v>CONS</v>.  Then we say \u000A that ``IF-lifting'' <v>test</v> ``from'' this term produces the following term, \u000A which is provably equal to the given term.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> test\u000A    (f ... x ...)  ; resulting true branch\u000A    (f ... y ...)) ; resulting false branch</code> \u000A \u000A <p>Here, we replace each argument of <v>f</v> of the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> test .. ..)</v>, \u000A for the same <v>test</v>, in the same way.  In this case we say that \u000A ``IF-lifting applies to'' the given term, ``yielding the test'' <v>test</v> and \u000A with the ``resulting two branches'' displayed above.  Whenever we apply \u000A IF-lifting, we do so for the available <v>test</v> that is least in the BDD term \u000A order (see (A1) above).</p> \u000A \u000A <p>We consider arguments <v>v</v> of <v>f</v> that are ``known to be Boolean'' \u000A (see above) to be replaced by <v>(<see topic=\"COMMON-LISP____IF\">IF</see> v t nil)</v> for the purposes of \u000A IF-lifting, i.e., before IF-lifting is applied.</p> \u000A \u000A <p>There is one special case, however, for IF-lifting.  Suppose that the given \u000A term is of the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> v y z)</v> where <v>v</v> is a variable and is the test \u000A to be lifted out (i.e., it is least in the BDD term order among the potential \u000A tests).  Moroever, suppose that neither <v>y</v> nor <v>z</v> is of the form \u000A <v>(<see topic=\"COMMON-LISP____IF\">IF</see> v W1 W2)</v> for that same <v>v</v>.  Then IF-lifting does not apply to the \u000A given term.</p> \u000A \u000A <p>We may now describe the IF-lifting-for-IF loop, which applies to terms of \u000A the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> test tbr fbr)</v> where the algorithm has already produced \u000A <v>test</v>, <v>tbr</v>, and <v>fbr</v>.  First, if <v>test</v> is <v>nil</v> then we \u000A return <v>fbr</v>, while if <v>test</v> is a non-<v>nil</v> constant or a call of \u000A <v>CONS</v> then we return <v>tbr</v>.  Otherwise, we see if IF-lifting applies. \u000A If IF-lifting does not apply, then we return <v>(<see topic=\"COMMON-LISP____IF\">IF</see> test tbr fbr)</v>. \u000A Otherwise, we apply IF-lifting to obtain a term of the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> x y z)</v>, \u000A by lifting out the appropriate test.  Now we recursively apply the \u000A IF-lifting-for-IF loop to the term <v>(<see topic=\"COMMON-LISP____IF\">IF</see> x y z)</v>, unless any of the \u000A following special cases apply.</p> \u000A \u000A <blockquote><p>(i) If <v>y</v> and <v>z</v> are the same term, then return \u000A <v>y</v>.</p> \u000A \u000A <p>(ii) Otherwise, if <v>x</v> and <v>z</v> are the same term, then replace <v>z</v> \u000A by <v>nil</v> before recursively applying IF-lifting-for-IF.</p> \u000A \u000A <p>(iii) Otherwise, if <v>x</v> and <v>y</v> are the same term and <v>y</v> is known \u000A to be Boolean, then replace <v>y</v> by <v>t</v> before recursively applying \u000A IF-lifting-for-IF.</p> \u000A \u000A <p>(iv) If <v>z</v> is <v>nil</v> and either <v>x</v> and <v>y</v> are the same term \u000A or <v>x</v> is ``known to be Boolean'' and <v>y</v> is <v>t</v>, then return \u000A <v>x</v>.</p></blockquote> \u000A \u000A <p>NOTE: When a variable <v>x</v> is known to be Boolean, it is easy to see that \u000A the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> x t nil)</v> is always reduced to <v>x</v> by this algorithm.</p> \u000A \u000A <p>(B4) <i>The ACL2 BDD algorithm</i></p> \u000A \u000A <p>We are now ready to present the BDD algorithm for ACL2.  It is given an \u000A ACL2 <see topic=\"ACL2____TERM\">term</see>, <v>x</v>, as well as an association list <v>va</v> that maps \u000A variables to terms, including all variables occurring in <v>x</v>.  We maintain \u000A the invariant that whenever a variable is mapped by <v>va</v> to a term, that \u000A term has already been constructed by the algorithm, except: initially <v>va</v> \u000A maps every variable occurring in the top-level term to itself.  The algorithm \u000A proceeds as follows.  We implicitly ordain that whenever the BDD algorithm \u000A attempts to create a <see topic=\"ACL2____TERM\">term</see> that is not a <see topic=\"ACL2____BDD\">bdd</see> term (as defined \u000A above in (A2)), it aborts instead.  Thus, whenever the algorithm completes \u000A without aborting, it creates a <see topic=\"ACL2____BDD\">bdd</see> term.</p> \u000A \u000A <blockquote> \u000A \u000A <p>If <v>x</v> is a variable, return the result of looking it up in <v>va</v>.</p> \u000A \u000A <p>If <v>x</v> is a constant, return <v>x</v>.</p> \u000A \u000A <p>If <v>x</v> is of the form <v>(<see topic=\"COMMON-LISP____IF\">IF</see> test tbr fbr)</v>, then first run the \u000A algorithm on <v>test</v> with the given <v>va</v> to obtain <v>test'</v>.  If \u000A <v>test'</v> is <v>nil</v>, then return the result <v>fbr'</v> of running the \u000A algorithm on <v>fbr</v> with the given <v>va</v>.  If <v>test'</v> is a constant \u000A other than <v>nil</v>, or is a call of <v>CONS</v>, then return the result \u000A <v>tbr'</v> of running the algorithm on <v>tbr</v> with the given <v>va</v>.  If \u000A <v>tbr</v> is identical to <v>fbr</v>, return <v>tbr</v>.  Otherwise, return the \u000A result of applying the IF-lifting-for-IF loop (described above) to the term \u000A <tt>(IF test' tbr' fbr')</tt>.</p> \u000A \u000A <p>If <v>x</v> is of the form <v>(<see topic=\"ACL2____IF_A2\">IF*</see> test tbr fbr)</v>, then compute the result \u000A exactly as though <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> were used rather than <tt><see topic=\"ACL2____IF_A2\">if*</see></tt>, except that if \u000A <v>test'</v> is not a constant or a call of <v>CONS</v> (see paragraph above), \u000A then abort the BDD computation.  Informally, the tests of <tt><see topic=\"ACL2____IF_A2\">if*</see></tt> terms \u000A are expected to ``resolve.''  NOTE: This description shows how <tt><see topic=\"ACL2____IF_A2\">if*</see></tt> can \u000A be used to implement conditional rewriting in the BDD algorithm.</p> \u000A \u000A <p>If <v>x</v> is a <v>LAMBDA</v> expression <v>((<see topic=\"COMMON-LISP____LAMBDA\">LAMBDA</see> vars body) . args)</v> \u000A (which often corresponds to a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> term; see <see topic=\"COMMON-LISP____LET\">let</see>), then first \u000A form an alist <v>va'</v> by binding each <v>v</v> in <v>vars</v> to the result of \u000A running the algorithm on the corresponding member of <v>args</v>, with the \u000A current alist <v>va</v>.  Then, return the result of the algorithm on <v>body</v> \u000A in the alist <v>va'</v>.</p> \u000A \u000A <p>Otherwise, <v>x</v> is of the form <v>(f x1 x2 ... xn)</v>, where <v>f</v> is a \u000A function symbol other than <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> or <tt><see topic=\"ACL2____IF_A2\">if*</see></tt>.  In that case, let \u000A <v>xi'</v> be the result of running the algorithm on <v>xi</v>, for <v>i</v> from 1 \u000A to <v>n</v>, using the given alist <v>va</v>.  First there are a few special \u000A cases.  If <v>f</v> is <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> then we return <v>t</v> if <v>x1'</v> is \u000A syntactically identical to <v>x2'</v> (where this test is very fast; see (B6) \u000A below); we return <v>x1'</v> if it is known to be Boolean and <v>x2'</v> is \u000A <v>t</v>; and similarly, we return <v>x2'</v> if it is known to be Boolean and \u000A <v>x1'</v> is <v>t</v>.  Next, if each <v>xi'</v> is a constant and the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <v>f</v> is enabled, then the result is obtained by \u000A computation.  Next, if <v>f</v> is <tt><see topic=\"ACL2____BOOLEANP\">booleanp</see></tt> and <v>x1'</v> is known to be \u000A Boolean, <v>t</v> is returned.  Otherwise, we proceed as follows, first possibly \u000A swapping the arguments if they are out of (the BDD term) order and if <v>f</v> \u000A is known to be commutative (see below).  If a BDD rewrite rule (as defined \u000A above) matches the term <tt>(f x1'... xn')</tt>, then the most recently stored \u000A such rule is applied.  If there is no such match and <v>f</v> is a \u000A BDD-constructor, then we return <tt>(f x1'... xn')</tt>.  Otherwise, if a BDD \u000A definition rule matches this term, then the most recently stored such rule \u000A (which will usually be the original definition for most users) is applied.  If \u000A none of the above applies and neither does IF-lifting, then we return <tt>(f \u000A x1'... xn')</tt>.  Otherwise we apply IF-lifting to <tt>(f x1'... xn')</tt> to \u000A obtain a term <v>(<see topic=\"COMMON-LISP____IF\">IF</see> test tbr fbr)</v>; but we aren't done yet.  Rather, we run \u000A the BDD algorithm (using the same alist) on <v>tbr</v> and <v>fbr</v> to obtain \u000A terms <v>tbr'</v> and <v>fbr'</v>, and we return <tt>(IF test tbr' fbr')</tt> \u000A unless <v>tbr'</v> is syntactically identical to <v>fbr'</v>, in which case we \u000A return <v>tbr'</v>.</p></blockquote> \u000A \u000A <p>When is it the case that, as said above, ``<v>f</v> is known to be \u000A commutative''?  This happens when an enabled rewrite rule is of the form \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (f X Y) (f Y X))</v>.  Regarding swapping the arguments in that case: \u000A recall that we may assume very little about the BDD term order, essentially \u000A only that we swap the two arguments when the second is a constant and the \u000A first is not, for example, in <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x 1)</v>.  Other than that situation, one \u000A cannot expect to predict accurately when the arguments of commutative \u000A operators will be swapped.</p> \u000A \u000A <p>(B5) Soundness and Completeness of the ACL2 BDD algorithm</p> \u000A \u000A <p>Roughly speaking, ``soundness'' means that the BDD algorithm should give \u000A correct answers, and ``completeness'' means that it should be powerful enough \u000A to prove all true facts.  Let us make the soundness claim a little more \u000A precise, and then we'll address completeness under suitable hypotheses.</p> \u000A \u000A <p><b>Claim</b> (Soundness).  If the ACL2 BDD algorithm runs to completion on \u000A an input term <v>t0</v>, then it produces a result that is provably equal to \u000A <v>t0</v>.</p> \u000A \u000A <p>We leave the proof of this claim to the reader.  The basic idea is simply \u000A to check that each step of the algorithm preserves the meaning of the term \u000A under the bindings in the given alist.</p> \u000A \u000A <p>Let us start our discussion of completeness by recalling the theorem proved \u000A above in (A4).</p> \u000A \u000A <p><b>Theorem</b>.  Suppose that <v>t1</v> and <v>t2</v> are canonical BDD terms \u000A that contain no function symbols other than <v>IF</v> and <v>CONS</v>.  Also \u000A suppose that <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1 t2)</v> is a theorem.  Then <v>t1</v> and <v>t2</v> are \u000A syntactically identical.</p> \u000A \u000A <p>Below we show how this theorem implies the following completeness property \u000A of the ACL2 BDD algorithm.  We continue to assume that <v>CONS</v> is the only \u000A BDD-constructor.</p> \u000A \u000A <p><b>Claim</b> (Completeness).  Suppose that <v>t1</v> and <v>t2</v> are provably \u000A equal terms, under the assumption that all their variables are known to be \u000A Boolean.  Assume further that under this same assumption, top-level runs of \u000A the ACL2 BDD algorithm on these terms return terms that contain only the \u000A function symbols <v>IF</v> and <v>CONS</v>.  Then the algorithm returns the same \u000A term for both <v>t1</v> and <v>t2</v>, and the algorithm reduces <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> t1\u000A t2)</v> to <v>t</v>.</p> \u000A \u000A <p>Why is this claim true?  First, notice that the second part of the \u000A conclusion follows immediately from the first, by definition of the algorithm. \u000A Next, notice that the terms <v>u1</v> and <v>u2</v> obtained by running the \u000A algorithm on <v>t1</v> and <v>t2</v>, respectively, are provably equal to <v>t1</v> \u000A and <v>t2</v>, respectively, by the Soundness Claim.  It follows that <v>u1</v> \u000A and <v>u2</v> are provably equal to each other.  Since these terms contain no \u000A function symbols other than <v>IF</v> or <v>CONS</v>, by hypothesis, the Claim now \u000A follows from the Theorem above together with the following lemma.</p> \u000A \u000A <p><b>Lemma</b>.  Suppose that the result of running the ACL2 BDD algorithm on \u000A a top-level term <v>t0</v> is a term <v>u0</v> that contains only the function \u000A symbols <v>IF</v> and <v>CONS</v>, where all variables of <v>t0</v> are known to be \u000A Boolean.  Then <v>u0</v> is a canonical BDD term.</p> \u000A \u000A <p>Proof: left to the reader.  Simply follow the definition of the algorithm, \u000A with a separate argument for the IF-lifting-for-IF loop.</p> \u000A \u000A <p>Finally, let us remark on the assumptions of the Completeness Claim above. \u000A The assumption that all variables are known to be Boolean is often true; in \u000A fact, the system uses the forward-chaining rule <v>boolean-listp-forward</v> \u000A (you can see it using <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>) to try to establish this assumption, \u000A if your theorem has a form such as the following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 ...))\u000A      (y (<see topic=\"COMMON-LISP____LIST\">list</see> y0 y1 ...)))\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> x)\u000A                (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> y))\u000A           ...))</code> \u000A \u000A <p>Moreover, the <v>:BDD</v> hint can be used to force the prover to abort if it \u000A cannot check that the indicated variables are known to be Boolean; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>Finally, consider the effect in practice of the assumption that the terms \u000A resulting from application of the algorithm contain calls of <v>IF</v> and \u000A <v>CONS</v> only.  Typical use of BDDs in ACL2 takes place in a theory (see \u000A <see topic=\"ACL2____THEORIES\">theories</see>) in which all relevant non-recursive function symbols are \u000A enabled and all recursive function symbols possess enabled BDD rewrite rules \u000A that tell them how open up.  For example, such a rule may say how to expand on \u000A a given function call's argument that has the form <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> a x)</v>, while \u000A another may say how to expand when that argument is <v>nil</v>).  (See for \u000A example the rules <v>append-cons</v> and <v>append-nil</v> in the documentation \u000A for <tt><see topic=\"ACL2____IF_A2\">if*</see></tt>.)  We leave it to future work to formulate a theorem that \u000A guarantees that the BDD algorithm produces terms containing calls only of \u000A <v>IF</v> and <v>CONS</v> assuming a suitably ``complete'' collection of rewrite \u000A rules.</p> \u000A \u000A <p>(B6) <i>Efficiency considerations</i></p> \u000A \u000A <p>Following Bryant's algorithm, we use a graph representation of <see topic=\"ACL2____TERM\">term</see>s \u000A created by the BDD algorithm's computation.  This representation enjoys some \u000A important properties.</p> \u000A \u000A <blockquote> \u000A \u000A <p>(Time efficiency) The test for syntactic equality of BDD terms is very \u000A fast.</p> \u000A \u000A <p>(Space efficiency) Equal BDD data structures are stored identically in \u000A memory.</p></blockquote> \u000A \u000A <p><i>Implementation note.</i> The representation actually uses a sort of hash \u000A table for BDD terms that is implemented as an ACL2 1-dimensional array.  See \u000A <see topic=\"ACL2____ARRAYS\">arrays</see>.  In addition, we use a second such hash table to avoid \u000A recomputing the result of applying a function symbol to the result of running \u000A the algorithm on its arguments.  We believe that these uses of hash tables are \u000A standard.  They are also discussed in Moore's paper on BDDs; see <see topic=\"ACL2____BDD\">bdd</see> \u000A for the reference.</p>"],

 "COMMON-LISP____CONS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x y)</v> is a pair whose first component is <v>x</v> and \u000A second component is <v>y</v>.  If <v>y</v> is a list, then <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x y)</v> is a \u000A list that has an additional element <v>x</v> on the front.</p>"],

 "ACL2____MISCELLANEOUS":[["ACL2"],"ACL2 Sources","ACL2","<p>Perhaps as the system matures this section will become more \u000A  structured.</p>"],

 "ACL2____GUARD-DEBUG":[["Guard","Debugging"],"ACL2 Sources","ACL2","<p>ACL2 guard verification (see <see topic=\"ACL2____GUARD\">guard</see>) is often best avoided by \u000A beginning users of ACL2.  When guard verification is employed, it can generate \u000A numerous goals, some of which may not be theorems if the definition being \u000A processed has bugs.  It can be difficult to find such bugs.  This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic explains a capability provided by ACL2 to help find such \u000A bugs.</p> \u000A \u000A <p>We begin with the following example.  Although it is contrived and a bit \u000A simplistic, it illustrates how the guard-debug utility works.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> length-repeat (x)\u000A  (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x x)))\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> length-repeat :guard-debug t)</code> \u000A \u000A <p>The output produces two top-level key checkpoints, as follows.</p> \u000A \u000A <code>Subgoal 2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____EXTRA-INFO\">EXTRA-INFO</see> '(:GUARD (:BODY LENGTH-REPEAT))\u000A                     '(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X))\u000A\u000ASubgoal 1\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____EXTRA-INFO\">EXTRA-INFO</see> '(:GUARD (:BODY LENGTH-REPEAT))\u000A                          '(<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X)))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X))))\u000A         (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X)))</code> \u000A \u000A <p>The upper subgoal (numbered 2) says that the body of the definition of \u000A <v>length-repeat</v> contains a call <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X)</v>, which is the source of \u000A the goal.  In this case, that makes sense: the <see topic=\"ACL2____GUARD\">guard</see> for a call \u000A <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> arg1 arg2)</v> is <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> arg1)</v>, which in this case is \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)</v>.  The lower subgoal (numbered 1) says that the same \u000A definition contains the call <v>(<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X))</v>, which generates the \u000A proof obligation that if <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> X X)</v> does not satisfy <v>true-listp</v>, \u000A then it must satisfy <v>stringp</v>.  That proof obligation comes directly from \u000A the <see topic=\"ACL2____GUARD\">guard</see> for <tt><see topic=\"COMMON-LISP____LENGTH\">length</see></tt>.</p> \u000A \u000A <p>Of course, the example above is a bit obvious.  But for large definitional \u000A bodies such information can be very helpful.  Note that guard-debug can be \u000A specified not only in <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> events but also in <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> \u000A <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> events.</p> \u000A \u000A <p>We now describe the guard-debug utility in some detail.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____EXTRA-INFO\">Extra-info</see> x y)</v> always returns <v>t</v> by definition.  However, if \u000A <see topic=\"ACL2____GUARD\">guard</see> verification takes place with a non-<v>nil</v> setting of \u000A <v>guard-debug</v> (see below), then the goals generated for guard verification \u000A can include hypotheses that are calls of <v>extra-info</v>.  Typically, such a \u000A hypothesis is of the following form:</p> \u000A \u000A <code>(<see topic=\"ACL2____EXTRA-INFO\">extra-info</see> '(:guard (:body F))\u000A            '(G ARG1 ... ARGk))</code> \u000A \u000A <p>The above form indicates that the goal in which it occurs was generated to \u000A verify that the <see topic=\"ACL2____GUARD\">guard</see> of the function <v>F</v> is satisfied by the \u000A arguments <v>ARG1</v> through <v>ARGk</v>, where the term <v>(G ARG1 ... ARGk)</v> \u000A occurs in the body of the function <v>F</v> whose guard verification is in \u000A progress.  If however the above call of <v>G</v> occurs in the guard of <v>F</v> \u000A instead of the body of <v>F</v>, then <v>:body</v> is replaced by <v>:guard</v> \u000A above:</p> \u000A \u000A <code>(<see topic=\"ACL2____EXTRA-INFO\">extra-info</see> '(:guard (:guard F))\u000A            '(G ARG1 ... ARGk))</code> \u000A \u000A <p>If the same proof obligation (goal clause) arises from more than one \u000A occurrence of the same call, then a single goal will be generated, which has \u000A several <v>extra-info</v> hypotheses added to show the multiple sources of that \u000A proof obligation.</p> \u000A \u000A <p>All rules (see <see topic=\"ACL2____RUNE\">rune</see>) associated with <v>extra-info</v> are <see topic=\"ACL2____DISABLE\">disable</see>d by default, so that hypotheses of the form described above are not \u000A simplified to <v>t</v>.  These hypotheses are intended to ride along for free: \u000A you can generally expect the proof to have the same structure whether or not \u000A the <v>:guard-debug</v> option is supplied, though on rare occasions the proofs \u000A may diverge.</p> \u000A \u000A <p>It remains to explain the notion of a <v>guard-debug</v> setting of <v>t</v>, \u000A that is, to explain how to obtain the additional hypotheses described above. \u000A If guards are being verified during processing of a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event \u000A (whether or not inside a call of <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>), then one specifies \u000A <v>:guard-debug t</v> in an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration of a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form; \u000A see <see topic=\"ACL2____XARGS\">xargs</see>.  If however the guard verification is on behalf of a <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> call, whether for a definition or a theorem, then one specifies \u000A the keyword argument <v>:guard-debug t</v>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____PRINT-GV\">print-gv</see> for a utility for debugging guard violations, in \u000A contrast to the above guard-debug mechanism, which is for debugging failed \u000A proofs arising from guard verification.</p>"],

 "ACL2____DEFUN-MODE-CAVEAT":[["Common-lisp"],"ACL2 Sources","ACL2","<p>Technically speaking, in the current implementation, the execution \u000A of functions having <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> may damage the \u000A ACL2 system in a way that renders it unsound.  In practice, we have never seen \u000A this happen; so, the explanations below can be viewed as extremely paranoid. \u000A Nevertheless, here we document this concern, even if it should be taken with \u000A more than a grain of salt.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> for a discussion of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s.  That \u000A discussion describes an imagined implementation that is slightly different \u000A from this one.  This note explains that the current implementation is open to \u000A unsoundness.</p> \u000A \u000A <p>For discussion of a different soundness issue that is also related to \u000A function execution, see <see topic=\"ACL2____GENERALIZED-BOOLEANS\">generalized-booleans</see>.</p> \u000A \u000A <p>The execution of a function having <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A may violate Common Lisp <see topic=\"ACL2____GUARD\">guard</see>s on the subroutines used.  (This may be \u000A true even for calls of a function on arguments that satisfy its <see topic=\"ACL2____GUARD\">guard</see>, \u000A because ACL2 has not verified that its <see topic=\"ACL2____GUARD\">guard</see> is sufficient to protect \u000A its subroutines.)  When a <see topic=\"ACL2____GUARD\">guard</see> is violated at runtime all bets are \u000A off.  That is, no guarantees are made either about the answer being ``right'' \u000A or about the continued rationality of the ACL2 system itself.</p> \u000A \u000A <p>For example, suppose you make the following <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> crash (i)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> i))</code> \u000A \u000A <p>Note that the declared guard does not in fact adequately protect the \u000A subroutines in the body of <v>crash</v>; indeed, satisfying the guard to \u000A <v>crash</v> will guarantee that the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> expression is in violation of \u000A its guard.  Because this function is admitted in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>-mode, \u000A no checks are made concerning the suitability of the guard.  Furthermore, in \u000A the current ACL2 implementation, <v>crash</v> is executed directly in Common \u000A Lisp.  Thus if you call <v>crash</v> on an argument satisfying its guard you \u000A will cause an erroneous computation to take place.</p> \u000A \u000A <code>ACL2 !&gt;(crash 7)\u000A\u000AError: Caught fatal error [memory may be damaged]\u000A...</code> \u000A \u000A <p>There is no telling how much damage is done by this errant computation.  In \u000A some lisps your ACL2 job may actually crash back to the operating system.  In \u000A other lisps you may be able to recover from the ``hard error'' and resume ACL2 \u000A in a damaged but apparently functional image.</p> \u000A \u000A <p>THUS, HAVING A FUNCTION WITH <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> IN \u000A YOUR SYSTEM ABSOLVES US, THE ACL2 IMPLEMENTORS, FROM RESPONSIBILITY FOR THE \u000A SOUNDNESS OF OUR SYSTEM.</p> \u000A \u000A <p>Furthermore</p> \u000A \u000A <p>ACL2 DOES NOT YET PROVIDE ANY MEANS OF REGAINING ASSURANCES OF SOUNDNESS \u000A AFTER THE INTRODUCTION OF A FUNCTION IN <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> MODE, EVEN IF IT \u000A IS ULTIMATELY CONVERTED TO <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> MODE (since its execution could \u000A have damaged the system in a way that makes it possible to verify its \u000A termination and <see topic=\"ACL2____GUARD\">guard</see>s unsoundly).</p> \u000A \u000A <p>Finally,</p> \u000A \u000A <p>THE VAST MAJORITY OF ACL2 SYSTEM CODE IS IN <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> MODE AND \u000A SO ALL BETS ARE OFF FROM BEFORE YOU START!</p> \u000A \u000A <p>This hopeless state of current affairs will change, we think.  We think we \u000A have defined our functions ``correctly'' in the sense that they can be \u000A converted, without ``essential'' modification, to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode. \u000A We think it very unlikely that a mis-guarded function in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode (whether ours or yours) will cause unsoundness without some sort of hard \u000A lisp error accompanying it.  We think that ultimately we can make it possible \u000A to execute your functions (interpretively) without risk to the system, even \u000A when some have <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  In that imagined implementation, \u000A code using functions having <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode would run more slowly, \u000A but safely.  These functions could be introduced into the logic ex post facto, \u000A whereupon the code's execution would speed up because Common Lisp would be \u000A allowed to execute it directly.  We therefore ask that you simply pretend that \u000A this is that imagined implementation, introduce functions in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, use them as convenient and perhaps ultimately introduce some of \u000A them in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode and prove their properties.  If you use the \u000A system this way we can develop (or dismiss) this style of formal system \u000A development.  BUT BE ON THE LOOKOUT FOR SCREWUPS DUE TO DAMAGE CAUSED BY THE \u000A EXECUTION OF YOUR FUNCTIONS HAVING <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> MODE!</p>"],

 "ACL2____DEFEXEC":[["Events","Mbe"],"ACL2 Sources","ACL2","<p>Suppose you define a function <v>(fn x)</v> with a <see topic=\"ACL2____GUARD\">guard</see> of \u000A <v>(good-input-p x)</v>, and you know that when the guard holds, the measure \u000A decreases on each recursive call.  Unfortunately, the definitional principle \u000A (see <see topic=\"COMMON-LISP____DEFUN\">defun</see>) ignores the guard.  For example, if the definition has the \u000A form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (good-input-p x)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (not-done-yet x)\u000A      (... (fn (destr x)) ...)\u000A    ...))</code> \u000A \u000A <p>then in order to admit this definition, ACL2 must prove the appropriate \u000A formula asserting that <v>(destr x)</v> is ``smaller than'' <v>x</v> under the \u000A assumption <v>(not-done-yet x)</v> but without the assumption <v>(good-input-p\u000A x)</v>, even if <v>(not-done-yet x)</v> is true.  In essence, it may be necessary \u000A to submit instead the following definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (good-input-p x)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (good-input-p x)\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (not-done-yet x)\u000A          (... (fn (destr x)) ...)\u000A        ...)\u000A    nil)</code> \u000A \u000A <p>But it is unfortunate that when calls of <v>fn</v> are evaluated, for example \u000A when <v>fn</v> is applied to an explicit constant during a proof, then a call of \u000A <v>good-input-p</v> must now be evaluated on each recursive call.</p> \u000A \u000A <p>Fortunately, <v>defexec</v> provides a way to keep the execution efficient. \u000A For the example above we could use the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFEXEC\">defexec</see> fn (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (good-input-p x)))\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____IF\">if</see> (good-input-p x)\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> (not-done-yet x)\u000A                      (... (fn (destr x)) ...)\u000A                    ...)\u000A                nil)\u000A       :exec  (<see topic=\"COMMON-LISP____IF\">if</see> (not-done-yet x)\u000A                  (... (fn (destr x)) ...)\u000A                ...)))</code> \u000A \u000A <p>Here ``<tt><see topic=\"ACL2____MBE\">mbe</see></tt>'' stands for ``must be equal'' and, roughly speaking, \u000A its call above is logically equal to the <v>:logic</v> form but is evaluated \u000A using the <v>:exec</v> form when the guard holds.  See <see topic=\"ACL2____MBE\">mbe</see>.  The effect \u000A is thus to define <v>fn</v> as shown in the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form above, but to \u000A cause execution of <v>fn</v> using the <v>:exec</v> body.  The use of <v>defexec</v> \u000A instead of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> in the example above causes a termination proof to be \u000A performed, in order to guarantee that evaluation always theoretically \u000A terminates, even when using the <v>:exec</v> form for evaluation.</p> \u000A \u000A <code>Example:\u000A\u000A; Some of the keyword arguments in the declarations below are irrelevant or\u000A; unnecessary, but they serve to illustrate their use.\u000A\u000A(<see topic=\"ACL2____DEFEXEC\">defexec</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP_____B2\">+</see> 15 (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x))\u000A                  :ruler-extenders :basic\u000A                  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> nth)))\u000A                  :guard-hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> last)))\u000A                  :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 25)))\u000A           (exec-xargs\u000A                  :test (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x))\u000A                  :default-value 'undef ; defaults to nil\u000A                  :measure (<see topic=\"ACL2____NFIX\">nfix</see> x)\u000A                  :ruler-extenders :basic\u000A                  :well-founded-relation o&lt;))\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> x)\u000A                  1\u000A                (<see topic=\"COMMON-LISP_____A2\">*</see> x (f (<see topic=\"COMMON-LISP____-\">-</see> x 1))))\u000A       :exec  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> x 0)\u000A                  1\u000A                (<see topic=\"COMMON-LISP_____A2\">*</see> x (f (<see topic=\"COMMON-LISP____-\">-</see> x 1))))))</code> \u000A \u000A <p>The above example macroexpands to the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> ()\u000A (<see topic=\"ACL2____LOCAL\">LOCAL</see>\u000A  (<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> ()\u000A   (<see topic=\"ACL2____SET-IGNORE-OK\">SET-IGNORE-OK</see> T)\u000A   (<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">SET-IRRELEVANT-FORMALS-OK</see> T)\u000A   (<see topic=\"ACL2____LOCAL\">LOCAL</see> (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> F (X)\u000A            (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see>\u000A             (<see topic=\"ACL2____XARGS\">XARGS</see> :VERIFY-GUARDS NIL\u000A                    :HINTS ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> NTH)))\u000A                    :MEASURE (<see topic=\"ACL2____NFIX\">NFIX</see> X)\u000A                    :RULER-EXTENDERS :BASIC\u000A                    :WELL-FOUNDED-RELATION O&lt;))\u000A            (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 X))\u000A                (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP_____D3\">=</see> X 0) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> X (F (<see topic=\"COMMON-LISP____-\">-</see> X 1))))\u000A                'UNDEF)))\u000A   (<see topic=\"ACL2____LOCAL\">LOCAL</see> (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> F-GUARD-IMPLIES-TEST\u000A            (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 X) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> X 25))\u000A                     (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 X)))\u000A            :RULE-CLASSES NIL))))\u000A (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> F (X)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :MEASURE (<see topic=\"COMMON-LISP_____B2\">+</see> 15 (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X))\u000A                   :RULER-EXTENDERS :BASIC\u000A                   :HINTS ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> NTH)))\u000A                   :GUARD-HINTS ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> LAST)))\u000A                   :GUARD (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 X) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> X 25))))\u000A   (<see topic=\"ACL2____MBE\">MBE</see> :LOGIC\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"ACL2____ZP\">ZP</see> X) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> X (F (<see topic=\"COMMON-LISP____-\">-</see> X 1))))\u000A        :EXEC\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP_____D3\">=</see> X 0) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> X (F (<see topic=\"COMMON-LISP____-\">-</see> X 1)))))))</code> \u000A \u000A <p>Notice that in the example above, the <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> in the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> definition of <v>F</v> are inherited from the <v>:hints</v> in the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> of the <v>defexec</v> form.  We discuss such inheritance below.</p> \u000A \u000A <p>CAVEAT: Termination is not considered for calls of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> under the \u000A top-level call.  Moreover, the <v>:exec</v> part of an <tt><see topic=\"ACL2____MBE\">mbe</see></tt> call under \u000A the <v>:logic</v> part of any superior <v>mbe</v> call is completely ignored.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFEXEC\">defexec</see> fn (var1 ... varn) doc-string dcl ... dcl\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :LOGIC logic-body\u000A       :EXEC  exec-body))</code> \u000A \u000A <p>where the syntax is identical to the syntax of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> where the body \u000A is a call of <v>mbe</v>, with the exceptions described below.  Thus, <v>fn</v> is \u000A the symbol you wish to define and is a new symbolic name and <v>(var1\u000A ... varn)</v> is its list of formal parameters (see <see topic=\"ACL2____NAME\">name</see>).  The first \u000A exception is that at least one <v>dcl</v> (i.e., <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form) must \u000A specify a <v>:guard</v>, <v>guard</v>.  The second exception is that one of the \u000A <v>dcl</v>s is allowed to contain an element of the form <v>(exec-xargs ...)</v>. \u000A The <v>exec-xargs</v> form, if present, must specify a non-empty <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt> each of whose keys is one of <v>:test</v>, \u000A <v>:default-value</v>, or one of the standard <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keys of \u000A <v>:measure</v>, <v>:ruler-extenders</v>, <v>:well-founded-relation</v>, \u000A <v>:hints</v>, or <v>:stobjs</v>.  Any of these five standard <v>xargs</v> keys that \u000A is present in an <v>xargs</v> of some <v>dcl</v> but is not specified in the \u000A (possibly nonexistent) <v>exec-xargs</v> form is considered to be specified in \u000A the <v>exec-xargs</v> form, as illustrated in the example above for <v>:hints</v>. \u000A (So for example, if you want <v>:hints</v> in the final, non-local definition \u000A but not in the local definition, then specify the <v>:hints</v> in the \u000A <v>xargs</v> but specify <v>:hints nil</v> in the <v>exec-xargs</v>.)  If <v>:test</v> \u000A is specified and not <v>nil</v>, let <v>test</v> be its value; otherwise let \u000A <v>test</v> default to <v>guard</v>.  If <v>:default-value</v> is specified, let \u000A <v>default-value</v> be its value; else <v>default-value</v> is <v>nil</v>. \u000A <v>Default-value</v> should have the same <see topic=\"ACL2____SIGNATURE\">signature</see> as <v>exec-body</v>; \u000A otherwise the <v>defexec</v> form will fail to be admitted.</p> \u000A \u000A <p>The above General Form's macroexpansion is of the form <v>(<see topic=\"COMMON-LISP____PROGN\">PROGN</see> encap\u000A final-def)</v>, where <v>encap</v> and <v>final-def</v> are as follows. \u000A <v>Final-def</v> is simply the result of removing the <v>exec-xargs</v> \u000A declaration (if any) from its <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form, and is the result of \u000A evaluating the given <v>defexec</v> form, since <v>encap</v> is of the following \u000A form.</p> \u000A \u000A <code>; encap\u000A(<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> ()\u000A  (<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> t)             ; harmless for proving termination\u000A  (<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> t) ; harmless for proving termination\u000A  (<see topic=\"ACL2____LOCAL\">local</see> local-def)\u000A  (<see topic=\"ACL2____LOCAL\">local</see> local-thm))</code> \u000A \u000A <p>The purpose of <v>encap</v> is to ensure the the executable version of \u000A <v>name</v> terminates on all arguments.  Thus, <v>local-def</v> and \u000A <v>local-thm</v> are as follows, where the <v>xargs</v> of the <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A form are the result of adding <v>:VERIFY-GUARDS NIL</v> to the result of \u000A removing the <v>:test</v> and (optional) <v>:default-value</v> from the \u000A <v>exec-xargs</v>.</p> \u000A \u000A <code>; local-def\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> fn formals\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :VERIFY-GUARDS NIL ...))\u000A  (<see topic=\"COMMON-LISP____IF\">IF</see> test\u000A      exec-body\u000A    default-value))\u000A\u000A; local-thm\u000A(<see topic=\"ACL2____DEFTHM\">DEFTHM</see> fn-EXEC-GUARD-HOLDS\u000A  (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> guard test)\u000A  :RULE-CLASSES NIL)</code> \u000A \u000A <p>We claim that if the above <v>local-def</v> and <v>local-thm</v> are admitted, \u000A then all evaluations of calls of <v>fn</v> terminate.  The concern is that the \u000A use of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> in <v>final-def</v> allows for the use of <v>exec-body</v> for \u000A a call of <v>fn</v>, as well as for subsequent recursive calls, when <v>guard</v> \u000A holds and assuming that the guards have been verified for <v>final-def</v>. \u000A However, by <v>local-thm</v> we can conclude in this case that <v>test</v> holds, \u000A in which case the call of <v>fn</v> may be viewed as a call of the version of \u000A <v>fn</v> defined in <v>local-def</v>.  Moreover, since guards have been verified \u000A for <v>final-def</v>, then guards hold for subsequent evaluation of \u000A <v>exec-body</v>, and in particular for recursive calls of <v>fn</v>, which can \u000A thus continue to be viewed as calls using <v>local=def</v>.</p>"],

 "ACL2____WET":[["Trace"],"ACL2 Sources","ACL2","<p>The acronym ``wet'' stands for ``with-error-trace''.  <v>Wet</v> \u000A provides a convenient way to obtain a backtrace when evaluation causes a guard \u000A violation or other error.</p> \u000A \u000A <p>The basic idea is that <v>(<see topic=\"ACL2____WET\">wet</see> form)</v> evaluates <v>form</v> and, if there is \u000A an error, shows a backtrace of calls that led to that error.  Note however \u000A that by default only calls of user-defined (not built-in) functions \u000A ``supporting'' <v>form</v> in the following sense will show up in the backtrace: \u000A those that occur in the macroexpansion of <v>form</v> or (recursively) support \u000A any of those functions.  So for example, since <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form)</v> \u000A macroexpands to <v>(make-event-fn (<see topic=\"COMMON-LISP____QUOTE\">quote</see> form) ...)</v>, calls of functions \u000A occurring in <v>form</v> will likely not show up in the backtrace by default. \u000A The option <v>:fns all</v> overrides this default, with potential loss of speed; \u000A more on this below.</p> \u000A \u000A <p>The following example explains the use of <v>wet</v>.  First, submit the \u000A following three definitions:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))) (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) (foo (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (bar (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</code> \u000A \u000A <p>Now imagine you have obtained the following guard violation:</p> \u000A \u000A <code>ACL2 !&gt;(g '(3 4))\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (FOO X),\u000Awhich is (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X), is violated by the arguments in the call (FOO NIL).\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.  See\u000A:DOC set-guard-checking for information about suppressing this check\u000Awith (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>With <v>wet</v>, you can get a backtrace of user-defined functions.  The \u000A package prefixes shown below, <v>ACL2_*1*_</v>, indicate that the executable \u000A (logical) counterparts of the corresponding raw Lisp functions are being \u000A called; see <see topic=\"ACL2____GUARD\">guard</see>.  Don't forget to start with <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>\u000A &quot;misc/wet&quot; :dir :system)</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WET\">wet</see> (g '(3 4)))\u000A; Fast loading /projects/acl2/devel/books/misc/wet.fasl\u000A\u000ATTAG NOTE: Adding ttag :TRACE! from the top level loop.\u000A\u000AACL2 Error in WET:  The guard for the function call (FOO X), which\u000Ais (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X), is violated by the arguments in the call (FOO NIL).\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.  See\u000A:DOC set-guard-checking for information about suppressing this check\u000Awith (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000ABacktrace stack:\u000A----------------\u000A1. (ACL2_*1*_ACL2::FOO NIL)\u000A2. (ACL2_*1*_ACL2::BAR (4))\u000A3. (ACL2_*1*_ACL2::G (3 4))\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>By default, large structures are hidden during the printing of the \u000A backtrace stack.  But you can supply a value for keyword argument \u000A <v>:evisc-tuple</v> to modify the printing: <v>nil</v> to avoid hiding, else a \u000A suitable evisc-tuple, as shown below (see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WET\">wet</see> (g '(3 4)) :evisc-tuple (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 1 1 nil nil))\u000A; Fast loading /projects/acl2/devel/books/misc/wet.fasl\u000A\u000ATTAG NOTE: Adding ttag :TRACE! from the top level loop.\u000A\u000AACL2 Error in WET:  The guard for the function call (FOO X), which\u000Ais (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X), is violated by the arguments in the call (FOO NIL).\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.  See\u000A:DOC set-guard-checking for information about suppressing this check\u000Awith (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000ABacktrace stack:\u000A----------------\u000A1. (ACL2_*1*_ACL2::FOO ...)\u000A2. (ACL2_*1*_ACL2::BAR ...)\u000A3. (ACL2_*1*_ACL2::G ...)\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>For a backtrace as a data object, evaluate the form <v>(<see topic=\"ACL2_____04\">@</see> wet-stack)</v>. \u000A But note that this object may not be a legal ACL2 value, for example because \u000A of the ``<v>*1*</v>'' symbols shown above.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____WET\">wet</see> form           ; an arbitrary form\u000A     :book bk-form  ; optional, not evaluated\u000A     ;;; the rest are optional and evaluated:\u000A     :evisc-tuple e ; an evisc-tuple\u000A     :fns fns       ; :all, or a list of functions to show in a backtrace\u000A     :compile c     ; :same, t, or nil; default :same (nil if :fns supplied)</code> \u000A \u000A <p><v>Form</v> is evaluated.  If there is an error, a backtrace stack is printed \u000A to the standard output (<tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>), containing (by default) the \u000A user-defined function calls made before the error.  Such printing is \u000A controlled by the <v>:evisc-tuple</v> if supplied; otherwise, hiding of large \u000A structures will occur.  (Technical detail: by default the global \u000A abbrev-evisc-tuple is used, if bound; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.</p> \u000A \u000A <p>The <v>:fns</v> option.  As mentioned above, by default the <v>wet</v> \u000A backtrace shows user-defined functions that syntactically ``support'' the form \u000A being evaluated.  This default can be overridden by supplying an explicit \u000A list, <v>fns</v>, of functions, using option <v>:fns fns</v>; these will then be \u000A the functions whose calls are eligible for inclusion in the backtrace.  The \u000A special value <v>:fns :all</v> will allow all user-defined function calls in the \u000A backtrace.  This value can be useful when using <tt><see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see></tt>, for \u000A example, since the function being applied isn't typically included as a \u000A syntactic supporter of the form being evaluated.</p> \u000A \u000A <p>The <v>:compile</v> option.  <v>Wet</v> uses the <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> utility to \u000A modify the definitions of affected functions so that they can record \u000A information for the backtrace.  As described above, these affected functions \u000A are those that syntactically ``support'' the form unless specified by the \u000A <v>:fns</v> option.  As is the case for <v>trace$</v> &mdash; see <see topic=\"ACL2____TRACE_42\">trace$</see> \u000A &mdash; the new definitions of these affected functions may or may not be \u000A compiled.  For <v>trace$</v> and for <v>wet</v>, the default is to compile the new \u000A definition if and only if the original definition was compiled, except: For \u000A <v>wet</v>, if option <v>:fns :all</v> is provided, then the default is not to \u000A compile the affected definitions.  And for <v>trace$</v> and <v>wet</v>, the \u000A <v>:compile</v> option overrides the default, to specify what will be compiled: \u000A value <v>:same</v> to compile each affected function if and only if its original \u000A definition was compiled, value <v>t</v> to compile all affected functions, and \u000A value <v>nil</v> to skip compilation.</p> \u000A \u000A <p>The <v>:book</v> option.  <v>Wet</v> actually works by temporarily including a \u000A community book,</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/wet&quot; :dir :system)</code> \u000A \u000A <p>and then passing its arguments to macro <v>wet!</v>, defined in that book. \u000A The keyword argument <v>:book</v> allows you to specify a different book that \u000A defines a macro <v>wet!</v> to which to pass its arguments.  If the value of \u000A <v>:book</v> is a string, then the book named by that string is temporarily \u000A included using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>: <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;bk&quot;)</v>.  Otherwise \u000A <v>:book</v> should be a list of arguments, to be provided (unevaluated) to \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, for example <v>(&quot;my-wet&quot; :dir :my-utils)</v>.  Thus you \u000A can experiment by copying community book <v>books/misc/wet.lisp</v> to your own \u000A directory and making modifications to the copy.  If you make changes, we \u000A invite you to share them with the ACL2 community (see <see topic=\"ACL2____BOOKS\">books</see>).  Note \u000A that you can also supply <v>:book nil</v>, in which case the definition of \u000A <v>wet!</v> in your current session will be used without including a book.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____TRACE_42\">trace$</see> for a general tracing utility.  As mentioned above, \u000A <v>wet</v> is implemented using <v>trace$</v>.  <v>Wet</v> actually first applies \u000A <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt>; upon completion, <v>wet</v> then applies <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> to \u000A re-trace any functions that it had untraced, using their original trace \u000A specs.</p>"],

 "ACL2____VERSION":[["About-ACL2"],"ACL2 Sources","ACL2","<p>To determine the version number of your copy of ACL2, evaluate the \u000A form <v>(<see topic=\"ACL2_____04\">@</see> acl2-version)</v>.  The value will be a string.  For example,</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> acl2-version)\u000A&quot;ACL2 Version 3.4&quot;</code> \u000A \u000A <p>The part of the string after <v>&quot;ACL2 Version &quot;</v> is of the form <v>x.y</v> \u000A or <v>x.y.z</v>, optionally followed by a succession of values in parentheses, \u000A where <v>x</v>, <v>y</v>, and <v>z</v> are natural numbers.  If <v>z</v> is omitted \u000A then it is implicitly 0.  We refer to <v>X</v>, <v>y</v>, and <v>z</v> as the \u000A ``major'', ``minor'', and ``incrl'' fields, respectively.  The incrl field is \u000A used for incremental releases.  The discussion just below assumes that \u000A incremental releases are not employed at the user's site, i.e., the incrl \u000A fields are always 0.  We remove this assumption when we discuss incremental \u000A releases at the end of this documenttation topic.</p> \u000A \u000A <p><see topic=\"ACL2____BOOKS\">Books</see> are considered certified only in the same version of ACL2 in \u000A which the certification was done.  The <see topic=\"ACL2____CERTIFICATE\">certificate</see> file records the \u000A version number of the certifying ACL2 and <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> considers the \u000A book uncertified if that does not match the current version number.  Thus, \u000A each time we release a new version of ACL2, previously certified books should \u000A be recertified.</p> \u000A \u000A <p>Note that there are over 150 constants in the system, most having to do \u000A with the fact that ACL2 is coded in ACL2.  Many of these, for example \u000A <v>*common-lisp-specials-and-constants*</v> and <v>*acl2-exports*</v>, may change \u000A from version to version, and this can cause unsoundness.  For example, the \u000A symbol <v>'set-difference-eq</v> was added to <v>*acl2-exports*</v> in \u000A Version_2.9, so we can certify a book in Version_2.8 containing the following \u000A theorem, which is false in Version_2.9.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> 'set-difference-eq *acl2-exports*))</code> \u000A \u000A <p>Therefore, we need to disallow inclusion of such a book in a Version_2.9 \u000A session, which otherwise would allow us to prove <v>nil</v>.  Furthermore, it is \u000A possible that from one version of the system to another we might change, say, \u000A the default values on some system function or otherwise make ``intentional'' \u000A changes to the axioms.  It is even possible one version of the system is \u000A discovered to be unsound and we release a new version to correct our \u000A error.</p> \u000A \u000A <p>Therefore we adopted the draconian policy that books are certified by a \u000A given version of ACL2 and ``must'' be recertified to be used in other \u000A versions.  We put ``must'' in quotes because in fact, ACL2 allows a book that \u000A was certified in one ACL2 version to be included in a later version, using \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  But ACL2 does not allow <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to succeed \u000A when such an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is executed on its behalf.  Also, you may \u000A experience undesirable behavior if you avoid recertification when moving to a \u000A different version.  Hence we recommend that you stick to the draconion policy \u000A of recertifying books when updating to a new ACL2 version.</p> \u000A \u000A <p>Furthermore, all bets are off if you certify a book using ACL2 built on one \u000A host Lisp implementation and include the book using ACL2 built on a different \u000A host Lisp implementation.  For example, for most host Lisp implementations, \u000A the Lisp reader will interpret the token <v>2f</v> as a symbol; however, Common \u000A Lisp allows for the possibility that <v>2f</v> is read as a number.  In such a \u000A case, one could use the former ACL2 executable to certify a book containing \u000A the form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-thm\u000A  (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> '2f)\u000A  :rule-classes nil)</code> \u000A \u000A <p>but then use the latter ACL2 executable to include the book, even though \u000A for the latter ACL2 executable, <v>2f</v> is a number, not a symbol.  Of course, \u000A one would expect a checksum error in that case; but checksums do not provide \u000A guaranteed protection against including uncertified books.</p> \u000A \u000A <p><i>Incremental releases.</i></p> \u000A \u000A <p>From time to time, so-called ``incremental releases'' of ACL2 are made \u000A available.  These releases are thoroughly tested on at least two platforms; \u000A ``normal'' releases, on the other hand, are thoroughly tested on many more \u000A platforms (perhaps a dozen or so) and are accompanied by updates to the ACL2 \u000A home page.  We provide incremental releases in order to provide timely updates \u000A for ACL2 users who want them, without imposing unnecessary burdens on either \u000A on the ACL2 implementors or on ACL2 users who prefer to update less \u000A frequently.  The implementors expect users to update their copies of ACL2 when \u000A normal releases are made available, but not necessarily when incremental \u000A releases are made available.</p> \u000A \u000A <p>Incremental releases are accompanied by a bump in the incrl field of the \u000A version field, while normal releases are accompanied by a bump in the minor or \u000A (much less frequently) major field and zeroing out of the incrl field.  Note \u000A that incremental releases are full-fledged releases.</p>"],

 "ACL2____SET-DEBUGGER-ENABLE":[["Debugging"],"ACL2 Sources","ACL2","<code>Forms (see below for explanations and GCL exceptions):\u000A\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> t)         ; enable breaks into the raw Lisp debugger\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :break)    ; same as above\u000A:set-debugger-enable t          ; same as above\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :break-bt) ; as above, but print a backtrace first\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :bt-break) ; as above, but print a backtrace first\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :bt)       ; print a backtrace but do not enter debugger\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :never)    ; disable all breaks into the debugger\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> nil)       ; disable debugger except when calling break$</code> \u000A \u000A <p><i>Introduction.</i> Suppose we define <v>foo</v> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode to take the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of its argument.  This can cause a raw Lisp \u000A error.  ACL2 will then return control to its top-level loop unless you enable \u000A the Lisp debugger, as shown below (except: the error message can take quite a \u000A different form in non-ANSI GCL).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program)) (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 !&gt;(foo 3)\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  Attempt to take the car of 3 which is not listp.\u000A***********************************************\u000A\u000AIf you didn't cause an explicit interrupt (Control-C),\u000Athen the root cause may be call of a :program mode\u000Afunction that has the wrong guard specified, or even no\u000Aguard specified (i.e., an implicit guard of t).\u000ASee :DOC guards.\u000A\u000ATo enable breaks into the debugger (also see :DOC acl2-customization):\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000A&lt;state&gt;\u000AACL2 !&gt;(foo 3)\u000AError: Attempt to take the car of 3 which is not listp.\u000A  [condition type: TYPE-ERROR]\u000A\u000ARestart actions (select using :continue):\u000A 0: Abort entirely from this (lisp) process.\u000A[Current process: Initial Lisp Listener]\u000A[1] ACL2(1): [RAW LISP]</code> \u000A \u000A <p><i>Details.</i> ACL2 usage is intended to take place inside the ACL2 \u000A read-eval-print loop (see <see topic=\"ACL2____LP\">lp</see>).  Indeed, in most Lisp implementations \u000A ACL2 comes up inside that loop, as evidenced by the prompt:</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>However, one can occasionally hit a raw Lisp error.  Here is the above \u000A example again, this time for a GCL implementation, which unfortunately gives a \u000A slightly less aesthetic report.</p> \u000A \u000A <code>ACL2 !&gt;(foo 3)\u000A\u000AError: 3 is not of type LIST.\u000AFast links are on: do (si::use-fast-links nil) for debugging\u000AError signalled by CAR.\u000ABacktrace: funcall &gt; system:top-level &gt; lisp:lambda-closure &gt; lp &gt; acl2_*1*_acl2::foo &gt; foo &gt; car &gt; system:universal-error-handler &gt; system::break-level-for-acl2 &gt; let* &gt; UNLESS\u000AACL2 !&gt;</code> \u000A \u000A <p>Here, the user has defined <v>foo</v> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, with an \u000A implicit <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.  The ACL2 evaluator therefore called the Lisp \u000A evaluator, which expected <v>nil</v> or a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> argument to <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>.</p> \u000A \u000A <p>By default, ACL2 will return to its top-level loop (at the same level of \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt>) when there is a raw Lisp error, as though a call of <tt><see topic=\"ACL2____ER\">er</see></tt> \u000A with flag <v>HARD</v> has been evaluated.  If instead you want to enter the raw \u000A Lisp debugger in such cases, evaluate the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> t)</code> \u000A \u000A <p>You can subsequently return to the default behavior with:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> nil)</code> \u000A \u000A <p>Either way, you can enter the Lisp debugger from within the ACL2 loop by \u000A evaluating <v>(</v><tt><see topic=\"ACL2____BREAK_42\">break$</see></tt><v>)</v>.  If you want <v>break$</v> disabled, then \u000A evaluate the following, which disables entry to the Lisp debugger not only for \u000A Lisp errors but also when executing <v>(<see topic=\"ACL2____BREAK_42\">break$</see>)</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :never)</code> \u000A \u000A <p>The discussion above also applies to interrupts (from <v>Control-C</v>) in \u000A some, but not all, host Common Lisps.</p> \u000A \u000A <p>It remains to discuss options <v>:break</v>, <v>:bt</v>, <v>:break-bt</v>, and \u000A <v>:bt-break</v>.  Option <v>:break</v> is synonymous with option <v>t</v>, while \u000A option <v>:bt</v> prints a backtrace.  Options <v>:break-bt</v> and <v>:bt-break</v> \u000A are equivalent, and each has the combined effect of <v>:bt</v> and <v>:break</v>: \u000A a backtrace is printed and then the debugger is entered.</p> \u000A \u000A <p>Note that <v>set-debugger-enable</v> applies not only to raw Lisp errors, but \u000A also to ACL2 errors: those affected by <tt><see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see></tt>.  However, for \u000A ACL2 errors, entering the debugger is controlled only by <v>break-on-error</v>, \u000A not by <v>set-debugger-enable</v>.  For ACL2 errors encountered after evaluating \u000A <v>(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> t)</v>, the <v>set-debugger-enable</v> values of <v>:bt</v>, \u000A <v>:break-bt</v>, and <v>:bt-break</v> will result in the same effect: in many \u000A host LIsps, this effect will be to cause a backtrace to be printed.</p> \u000A \u000A <p>Remark for Common Lisp hackers (except for the case that the host Lisp is \u000A non-ANSI GCL).  You can customize the form of the backtrace printed by \u000A entering raw Lisp (with <v>:q</v>) and then redefining function \u000A <v>print-call-history</v>, whose definition immediately precedes that of \u000A <v>break-on-error</v> in ACL2 source file <v>ld.lisp</v>.  Of course, all bets are \u000A off when defining any function in raw Lisp, but as a practical matter you are \u000A probably fine as long as your books are ultimately certified with an \u000A unmodified copy of ACL2.  If you come up with improvements to \u000A <v>print-call-history</v>, please pass them along to the ACL2 \u000A implementors.</p>"],

 "ACL2____REDO-FLAT":[["Other"],"ACL2 Sources","ACL2","<p>When one submits an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command and there is a failure, ACL2 restores its logical <see topic=\"ACL2____WORLD\">world</see> as though the command had not been run.  But sometimes one would like \u000A to debug the failure by re-executing all sub-events that succeeded up to the \u000A point of failure, and then re-executing the failed sub-event.  Said \u000A differently, imagine that the <see topic=\"ACL2____EVENTS\">events</see> under an <v>encapsulate</v>, \u000A <v>progn</v>, or <v>certify-book</v> form were flattened into a list of events \u000A that were then submitted to ACL2 up to the point of failure.  This would put \u000A us in the state in which the original failed event had failed, so we could now \u000A replay that failed event and try modifying it, or first proving additional \u000A events, in order to get it admitted.</p> \u000A \u000A <p><v>Redo-flat</v> is provided for this purpose.  Consider the following \u000A (rather nonsensical) example, in which the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>f3</v> fails (the \u000A body is <v>y</v> but the formal parameter list is <v>(x)</v>).</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x) x)\u000A (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A              (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> hack (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) x))))\u000A (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A              (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> hack (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x)))))\u000A (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A              (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x) x))\u000A              (<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> foo (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x) :rule-classes nil)\u000A                     (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f3 (x) y)))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f4 (x) x)\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f5 (x) y))</code> \u000A \u000A <p>After this attempt fails, you can evaluate the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____REDO-FLAT\">redo-flat</see>)</code> \u000A \u000A <p>This will first lay down a <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> event, <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> r)</v>, so \u000A that you can eventually remove your debugging work with <v>(:ubt! r)</v>.  Then \u000A the successful sub-events that preceded the failure will be executed with \u000A proofs skipped (so that this execution is fast).  Then, the failed event will \u000A be executed.  Finally, a <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> command is executed so that you can \u000A see a summary of the events that executed successfully.</p> \u000A \u000A <p>You can eliminate some of the steps above by supplying keyword values, as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____REDO-FLAT\">redo-flat</see>\u000A :succ  succ ; Skip the successful sub-events if val is nil.\u000A :fail  fail ; Skip the failed sub-event if val is nil.\u000A :label lab  ; Skip deflabel if lab or succ is nil, else use (<see topic=\"ACL2____DEFLABEL\">deflabel</see> lab).\u000A :pbt   val  ; Skip the final :pbt if val, lab, or succ is nil.\u000A )</code> \u000A \u000A <p>Also, you can avoid skipping proofs for the successful sub-events by \u000A supplying keyword <v>:succ-ld-skip-proofsp</v> with a valid value for \u000A <v>ld-skip-proofsp</v>; see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.  For example, you might want \u000A to execute <v>(<see topic=\"ACL2____REDO-FLAT\">redo-flat</see> :succ-ld-skip-proofsp nil)</v> if you use the \u000A <v>must-fail</v> utility from community book <v>make-event/eval.lisp</v>, since \u000A for example <v>(must-fail (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)))</v> normally succeeds but would \u000A cause an error if proofs are skipped.  For another example: you may wish to do \u000A proofs when re-running certain <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms, for example, when the \u000A expansion is <v>(:OR expr1 expr2)</v> and the proof initially succeeds for \u000A <v>expr2</v> rather than <v>expr1</v> (see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>).</p> \u000A \u000A <p>If you prefer only to see the successful and failed sub-events, without any \u000A events being re-executed, you may evaluate the following form instead.</p> \u000A \u000A <code>(<see topic=\"ACL2____REDO-FLAT\">redo-flat</see> :show t)</code> \u000A \u000A <p>For the example above, this command produces the following output.</p> \u000A \u000A <code>List of events preceding the failure:\u000A\u000A((<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> F1 (X) X)\u000A (<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> NIL\u000A              (<see topic=\"ACL2____LOCAL\">LOCAL</see> (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> HACK (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X Y)) X))))\u000A (<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> NIL\u000A              (<see topic=\"ACL2____LOCAL\">LOCAL</see> (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> HACK (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> X Y) (<see topic=\"COMMON-LISP_____B2\">+</see> Y X)))))\u000A (<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> '(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> F2 (X) X))\u000A (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> FOO (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X X)\u000A         :RULE-CLASSES NIL))\u000A\u000AFailed event:\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> F3 (X) Y)\u000AACL2 !&gt;</code> \u000A \u000A <p><v>Redo-flat</v> uses a scheme that should not cause spurious name conflicts \u000A for <tt><see topic=\"ACL2____LOCAL\">local</see></tt> events.  Above, it is mentioned that events are \u000A ``flattened''; now we clarify this notion.  Each sub-event that succeeds and \u000A is an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> is left intact.  Only such events \u000A that fail are replaced by their component events.  Thus, in the example above, \u000A there is no conflict between the two <tt><see topic=\"ACL2____LOCAL\">local</see></tt> sub-events named \u000A ``<v>hack</v>,'' because these are contained in successful <v>encapsulate</v> \u000A sub-events, which are therefore not flattened.  The <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and two \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events surrounding the definition of <v>f3</v> are, however, \u000A flattened, because that definition failed to be admitted.</p> \u000A \u000A <p>Normally, <v>redo-flat</v> will have the desired effect even if you \u000A interrupted a proof (with control-c).  However, <v>redo-flat</v> will not \u000A produce the desired result after an interrupt if you have enabled the debugger \u000A using <v>(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> t)</v>,</p>"],

 "ACL2____QUANTIFIER-TUTORIAL":[["Defun-sk"],"ACL2 Sources","ACL2","<p>The initial version of this tutorial was written by Sandip Ray. \u000A Additions and revisions are welcome.  Sandip has said:</p> \u000A \u000A <blockquote><p>``This is a collection of notes that I wrote to remind myself \u000A of how to reason about quantifiers when I just started.  Most users after they \u000A have gotten the hang of quantifiers probably will not need this and will be \u000A able to use their intuitions to guide them in the process.  But since many \u000A ACL2 users are not used to quantification, I am hoping that this set of notes \u000A might help them to think clearly while reasoning about quantifiers in \u000A ACL2.''</p></blockquote> \u000A \u000A <p>Many ACL2 papers start with the sentence ``ACL2 is a quantifier-free \u000A first-order logic of recursive functions.''  It is true that the <i>syntax</i> \u000A of ACL2 is quantifier-free; every formula is assumed to be universally \u000A quantified over all free variables in the formula.  But the <i>logic</i> in \u000A fact does afford arbitrary first-order quantification.  This is obtained in \u000A ACL2 using a construct called <v>defun-sk</v>.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>.</p> \u000A \u000A <p>Many ACL2 users do not think in terms of <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see>.  The focus is \u000A almost always on defining recursive functions and reasoning about them using \u000A induction.  That is entirely justified, in fact, since proving theorems about \u000A recursive functions by induction plays to the strengths of the theorem prover. \u000A Nevertheless there are situations where it is reasonable and often useful to \u000A think in terms of quantifiers.  However, reasoning about quantifiers requires \u000A that you get into the mindset of thinking about theorems in terms of \u000A quantification.  This note is about how to do this effectively given ACL2's \u000A implementation of quantification.  This does not discuss <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> in \u000A detail, but merely shows some examples.  A detailed explanation of the \u000A implementation is in the ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> (see <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>); also \u000A see <see topic=\"ACL2____CONSERVATIVITY-OF-DEFCHOOSE\">conservativity-of-defchoose</see>.</p> \u000A \u000A <p>[Note: Quantifiers can be used for some pretty cool things in ACL2. \u000A Perhaps the most interesting example is the way of using quantifiers to \u000A introduce arbitrary tail-recursive equations; see the paper ``Partial \u000A Functions in ACL2'' by Panagiotis Manolios and J Strother Moore.  This note \u000A does not address applications of quantifiers, but merely how you would reason \u000A about them once you think you want to use them.]</p> \u000A \u000A <p>Assume that you have some function <v>P</v>.  I have just left <v>P</v> as a \u000A unary function stub below, since I do not care about what <v>P</v> is.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> P (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)</code> \u000A \u000A <p>Now suppose you want to specify the concept that ``there exists some <v>x</v> \u000A such that <v>(P x)</v> holds''.  ACL2 allows you to write that directly using \u000A quantifiers.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-P () (<see topic=\"ACL2____EXISTS\">exists</see> x (P x)))</code> \u000A \u000A <p>If you submit the above form in ACL2 you will see that the theorem prover \u000A specifies two functions <v>exists-p</v> and <v>exists-p-witness</v>, and exports \u000A the following constraints:</p> \u000A \u000A <code>1.  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> exists-P () (P (exists-P-witness)))\u000A2.  (<see topic=\"ACL2____DEFTHM\">defthm</see> exists-P-suff (<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (exists-p)))</code> \u000A \u000A <p>Here <v>exists-P-witness</v> is a new function symbol in the current ACL2 \u000A theory.  What do the constraints above say?  Notice the constraint \u000A <v>exists-p-suff</v>.  It says that if you can provide any <v>x</v> such that \u000A <v>(P x)</v> holds, then you know that <v>exists-p</v> holds.  Think of the other \u000A constraint (definition of <v>exists-p</v>) as going the other way.  That is, it \u000A says that if <v>exists-p</v> holds, then there is some <v>x</v>, call it \u000A <v>(exists-p-witness)</v>, for which <v>P</v> holds.  Notice that nothing else is \u000A known about <v>exists-p-witness</v> than the two constraints above.</p> \u000A \u000A <p>[Note: <v>exists-p-witness</v> above is actually defined in ACL2 using a \u000A special form called <v>defchoose</v>.  See <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  This note does not \u000A talk about <v>defchoose</v>.  So far as this note is concerned, think of \u000A <v>exists-p-witness</v> as a new function symbol that has been generated somehow \u000A in ACL2, about which nothing other than the two facts above is known.]</p> \u000A \u000A <p>Similarly, you can talk about the concept that ``for all <v>x</v> <v>(P x)</v> \u000A holds.'' This can be specified in ACL2 by the form:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-P () (<see topic=\"ACL2____FORALL\">forall</see> x (P x)))</code> \u000A \u000A <p>This produces the following two constraints:</p> \u000A \u000A <code>1.  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> forall-P () (P (forall-p-witness)))\u000A2.  (<see topic=\"ACL2____DEFTHM\">defthm</see> forall-p-necc (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (P x)) (<see topic=\"COMMON-LISP____NOT\">not</see> (forall-p))))</code> \u000A \u000A <p>To understand these, think of <v>for-all-p-witness</v> as producing some \u000A <v>x</v> which does not satisfy <v>P</v>, if such a thing exists.  The constraint \u000A <v>forall-p-necc</v> merely says that if <v>forall-p</v> holds then <v>P</v> is \u000A satisfied for every <v>x</v>.  (To see this more clearly, just think of the \u000A contrapositive of the formula shown.) The other constraint (definition of \u000A <v>forall-p</v>) implies that if <v>forall-p</v> does not hold then there is some \u000A <v>x</v>, call it <v>(forall-p-witness)</v>, which does not satisfy <v>P</v>.  To \u000A see this, just consider the following formula which is immediately derivable \u000A from the definition.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (forall-p)) (<see topic=\"COMMON-LISP____NOT\">not</see> (P (forall-witness))))</code> \u000A \u000A <p>The description above suggests that to reason about quantifiers, the \u000A following Rules of Thumb, familiar to most any student of logic, are \u000A useful.</p> \u000A \u000A <blockquote><p>RT1: To prove <v>(exists-p)</v>, construct some object <v>A</v> \u000A such that <v>P</v> holds for <v>A</v> and then use <v>exists-P-suff</v>.</p> \u000A \u000A <p>RT2: If you assume <v>exists-P</v> in your hypothesis, use the definition of \u000A <v>exists-p</v> to know that <v>P</v> holds for <v>exists-p-witness</v>.  To use \u000A this to prove a theorem, you must be able to derive the theorem based on the \u000A hypothesis that <v>P</v> holds for something, whatever the something is.</p> \u000A \u000A <p>RT3: To prove <v>forall-P</v>, prove the theorem <v>(P x)</v> (that is, that \u000A <v>P</v> holds for an arbitrary <v>x</v>), and then simply instantiate the \u000A definition of <v>forall-p</v>, that is, show that <v>P</v> holds for the \u000A witness.</p> \u000A \u000A <p>RT4: If you assume <v>forall-p</v> in the hypothesis of the theorem, see how \u000A you can prove your conclusion if indeed you were given <v>(P x)</v> as a \u000A theorem.  Possibly for the conclusion to hold, you needed that <v>P</v> holds \u000A for some specific set of <v>x</v> values.  Then use the theorem \u000A <v>forall-p-necc</v> by instantiating it for the specific <v>x</v> values you care \u000A about.</p></blockquote> \u000A \u000A <p>Perhaps the above is too terse.  In the remainder of the note, we will \u000A consider several examples of how this is done to prove theorems in ACL2 that \u000A involve quantified notions.</p> \u000A \u000A <p>Let us consider two trivial theorems.  Assume that for some unary function \u000A <v>r</v>, you have proved <v>(r x)</v> as a theorem.  Let us see how you can prove \u000A that (1) there exists some x such that <v>(r x)</v> holds, and (2) for all \u000A <v>x</v> <v>(r x)</v> holds.</p> \u000A \u000A <p>We first model these things using <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>.  Below, <v>r</v> is \u000A simply some function for which <v>(r x)</v> is a theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((r *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> r (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> r-holds (r x)))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> exists-r () (<see topic=\"ACL2____EXISTS\">exists</see> x (r x)))\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-r () (<see topic=\"ACL2____FORALL\">forall</see> x (r x)))</code> \u000A \u000A <p>ACL2 does not have too much reasoning support for quantifiers.  So in most \u000A cases, one would need <v>:use</v> hints to reason about quantifiers.  In order \u000A to apply <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>, it is preferable to keep the function \u000A definitions and theorems disabled.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> exists-r exists-r-suff forall-r forall-r-necc))</code> \u000A \u000A <p>Let us now prove that there is some <v>x</v> such that <v>(r x)</v> holds. \u000A Since we want to prove <v>exists-r</v>, we must use <v>exists-r-suff</v> by RT1. \u000A We do not need to construct any instance here since <v>r</v> holds for all \u000A <v>x</v> by the theorem above.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> exists-r-holds\u000A  (exists-r)\u000A  :hints ((&quot;Goal&quot; :use ((:instance exists-r-suff)))))</code> \u000A \u000A <p>Let us now prove the theorem that for all <v>x</v>, <v>(r x)</v> holds.  By \u000A RT3, we must be able to prove it by definition of <v>forall-r</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-r-holds\u000A  (forall-r)\u000A  :hints ((&quot;Goal&quot; :use ((:instance (:definition forall-r))))))</code> \u000A \u000A <p>[Note: Probably no ACL2 user in his or her right mind would prove the \u000A theorems <v>exists-r-holds</v> and <v>forall-r-holds</v> above.  The theorems \u000A shown are only for demonstration purposes.]</p> \u000A \u000A <p>For the remainder of this note we will assume that we have two stubbed out \u000A unary functions <v>M</v> and <v>N</v>, and we will look at proving some quantified \u000A properties of these functions.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> M (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> N (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)</code> \u000A \u000A <p>Let us now define the predicates <v>all-M</v>, <v>all-N</v>, <v>ex-M</v>, and \u000A <v>ex-N</v> specifying the various quantifications.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> all-M () (<see topic=\"ACL2____FORALL\">forall</see> x (M x)))\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> all-N () (<see topic=\"ACL2____FORALL\">forall</see> x (N x)))\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-M () (<see topic=\"ACL2____EXISTS\">exists</see> x (M x)))\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-N () (<see topic=\"ACL2____EXISTS\">exists</see> x (N x)))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> all-M all-N all-M-necc all-N-necc))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> some-M some-N some-M-suff some-N-suff))</code> \u000A \u000A <p>Let us prove the classic distributive properties of quantification: the \u000A distributivity of universal quantification over conjunction, and the \u000A distributivity of existential quantification over disjunction.  We can state \u000A these properties informally in ``pseudo ACL2'' notation as follows:</p> \u000A \u000A <code>1.  (<see topic=\"ACL2____EXISTS\">exists</see> x: (M x)) or (<see topic=\"ACL2____EXISTS\">exists</see> x: (N x)) &lt;=&gt; (<see topic=\"ACL2____EXISTS\">exists</see> x: (M x) or (N x))\u000A2.  (<see topic=\"ACL2____FORALL\">forall</see> x: (M x)) and (forall: x (N x)) &lt;=&gt; (<see topic=\"ACL2____FORALL\">forall</see> x: (M x) and (N x))</code> \u000A \u000A <p>To make these notions formal we of course need to define the formulas at \u000A the right-hand sides of 1 and 2.  So we define <v>some-MN</v> and <v>all-MN</v> to \u000A capture these concepts.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-MN () (<see topic=\"ACL2____EXISTS\">exists</see> x (<see topic=\"COMMON-LISP____OR\">or</see> (M x) (N x))))\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> all-MN () (<see topic=\"ACL2____FORALL\">forall</see> x (<see topic=\"COMMON-LISP____AND\">and</see> (M x) (N x))))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> all-MN all-MN-necc some-MN some-MN-suff))</code> \u000A \u000A <p>First consider proving property 1.  The formal statement of this theorem \u000A would be: <v>(<see topic=\"ACL2____IFF\">iff</see> (some-MN) (<see topic=\"COMMON-LISP____OR\">or</see> (some-M) (some-N)))</v>.</p> \u000A \u000A <p>How do we prove this theorem?  Looking at RT1-RT4 above, note that they \u000A suggest how one should reason about quantification when one has an \u000A ``implication''.  But here we have an ``equivalence''.  This suggests another \u000A rule of thumb.</p> \u000A \u000A <blockquote><p>RT5: Whenever possible, prove an equivalence involving \u000A quantifiers by proving two implications.</p></blockquote> \u000A \u000A <p>Let us apply RT5 to prove the theorems above.  So we will first prove: \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (some-MN) (<see topic=\"COMMON-LISP____OR\">or</see> (some-M) (some-N)))</v></p> \u000A \u000A <p>How can we prove this?  This involves assuming a quantified predicate \u000A <v>(some-MN)</v>, so we must use RT2 and apply the definition of <v>some-MN</v>. \u000A Since the conclusion involves a disjunction of two quantified predicates, by \u000A RT1 we must be able to construct two objects <v>A</v> and <v>B</v> such that \u000A either <v>M</v> holds for <v>A</v> or <v>N</v> holds for <v>B</v>, so that we can then \u000A invoke <v>some-M-suff</v> and <v>some-N-suff</v> to prove the conclusion.  But now \u000A notice that if <v>some-MN</v> is true, then there is already an object, in fact \u000A <v>some-MN-witness</v>, such that either <v>M</v> holds for it, or <v>N</v> holds \u000A for it.  And we know this is the case from the definition of <v>some-MN</v>! So \u000A we will simply prove the theorem instantiating <v>some-M-suff</v> and \u000A <v>some-N-suff</v> with this witness.  The conclusion is that the following \u000A event will go through with ACL2.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> le1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (some-MN)\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (some-M) (some-N)))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition some-MN))\u000A                  (:instance some-M-suff\u000A                             (x (some-MN-witness)))\u000A                  (:instance some-N-suff\u000A                             (x (some-MN-witness)))))))</code> \u000A \u000A <p>This also suggests the following rule of thumb:</p> \u000A \u000A <blockquote><p>RT6: If a conjecture involves assuming an existentially \u000A quantified predicate in the hypothesis from which you are trying to prove an \u000A existentially quantified predicate, use the witness of the existential \u000A quantification in the hypothesis to construct the witness for the existential \u000A quantification in the conclusion.</p></blockquote> \u000A \u000A <p>Let us now try to prove the converse of le1, that is: <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____OR\">or</see>\u000A (some-M) (some-N)) (some-MN))</v></p> \u000A \u000A <p>Since the hypothesis is a disjunction, we will just prove each case \u000A individually instead of proving the theorem by a :<v>cases</v> hint.  So we \u000A prove the following two lemmas.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> le2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (some-M) (some-MN))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition some-M))\u000A                  (:instance some-MN-suff\u000A                             (x (some-M-witness)))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> le3\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (some-N) (some-MN))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition some-N))\u000A                  (:instance some-MN-suff\u000A                             (x (some-N-witness)))))))</code> \u000A \u000A <p>Note that the hints above are simply applications of RT6 as in <v>le1</v>. \u000A With these lemmas, of course the main theorem is trivial.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">defthmd</see> |some disjunction|\u000A  (<see topic=\"ACL2____IFF\">iff</see> (some-MN) (<see topic=\"COMMON-LISP____OR\">or</see> (some-M) (some-N)))\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance le1)\u000A                  (:instance le2)\u000A                  (:instance le3)))))</code> \u000A \u000A <p>Let us now prove the distributivity of universal quantification over \u000A conjunction, that is, the formula: <v>(<see topic=\"ACL2____IFF\">iff</see> (all-MN) (<see topic=\"COMMON-LISP____AND\">and</see> (all-M)\u000A (all-N)))</v></p> \u000A \u000A <p>Applying RT5, we will again decompose this into two implications.  So \u000A consider first the one-way implication: <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (all-M) (all-N))\u000A (all-MN))</v>.</p> \u000A \u000A <p>Here we get to assume <v>all-M</v> and <v>all-N</v>.  Thus by RT4 we can use \u000A <v>all-M-necc</v> and <v>all-N-necc</v> to think as if we are given the formulas \u000A <v>(M x)</v> and <v>(N x)</v> as theorems.  The conclusion here is also a \u000A universal quantification, namely we have to prove <v>all-MN</v>.  Then RT3 tells \u000A us to proceed as follows.  Take any object <v>y</v>.  Try to find an \u000A instantiation <v>z</v> of the hypothesis that implies <v>(<see topic=\"COMMON-LISP____AND\">and</see> (M y) (N y))</v>. \u000A Then instantiate <v>y</v> with <v>all-MN-witness</v>.  Note that the hypothesis \u000A lets us assume <v>(M x)</v> and <v>(N x)</v> to be theorems.  Thus to justify we \u000A need to instantiate <v>x</v> with <v>y</v>, and in this case, therefore, with \u000A <v>all-MN-witness</v>.  To make the long story short, the following event goes \u000A through with ACL2:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lf1\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (all-M) (all-N))\u000A            (all-MN))\u000A    :rule-classes nil\u000A    :hints ((&quot;Goal&quot;\u000A              :use ((:instance (:definition all-MN))\u000A                    (:instance all-M-necc (x (all-MN-witness)))\u000A                    (:instance all-N-necc (x (all-MN-witness)))))))</code> \u000A \u000A <p>This suggests the following rule of thumb which is a dual of RT6:</p> \u000A \u000A <blockquote><p>RT7: If a conjecture assumes some universally quantified \u000A predicate in the hypothesis and its conclusion asserts a universallly \u000A quantified predicate, then instantiate the ``necessary condition'' \u000A (<v>forall-mn-necc</v>) of the hypothesis with the witness of the conclusion to \u000A prove the conjecture.</p></blockquote> \u000A \u000A <p>Applying RT7 now we can easily prove the other theorems that we need to \u000A show that universal quantification distributes over conjunction.  Let us just \u000A go through this motion in ACL2.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lf2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (all-MN)\u000A           (all-M))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition all-M))\u000A                  (:instance all-MN-necc\u000A                             (x (all-M-witness)))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lf3\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (all-MN)\u000A           (all-N))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition all-N))\u000A                  (:instance all-MN-necc\u000A                             (x (all-N-witness)))))))\u000A\u000A(<see topic=\"ACL2____DEFTHMD\">defthmd</see> |all conjunction|\u000A  (<see topic=\"ACL2____IFF\">iff</see> (all-MN)\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (all-M) (all-N)))\u000A :hints ((&quot;Goal&quot; :use ((:instance lf1)\u000A                       (:instance lf2)\u000A                       (:instance lf3)))))</code> \u000A \u000A <p>The rules of thumb for universal and existential quantification should make \u000A you realize the duality of their use.  Every reasoning method about universal \u000A quantification can be cast as a way of reasoning about existential \u000A quantification, and vice versa.  Whether you reason using universal and \u000A existential quantifiers depends on what is natural in a particular context. \u000A But just for the sake of completeness let us prove the duality of universal \u000A and existential quantifiers.  So what we want to prove is the following:</p> \u000A \u000A <code>3.  (<see topic=\"ACL2____FORALL\">forall</see> x (<see topic=\"COMMON-LISP____NOT\">not</see> (M x))) = (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____EXISTS\">exists</see> x (M x)))</code> \u000A \u000A <p>We first formalize the notion of <v>(<see topic=\"ACL2____FORALL\">forall</see> x (<see topic=\"COMMON-LISP____NOT\">not</see> (M x)))</v> as a \u000A quantification.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> none-M () (<see topic=\"ACL2____FORALL\">forall</see> x (<see topic=\"COMMON-LISP____NOT\">not</see> (M x))))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> none-M none-M-necc))</code> \u000A \u000A <p>So we now want to prove: <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (none-M) (<see topic=\"COMMON-LISP____NOT\">not</see> (some-M)))</v>.</p> \u000A \u000A <p>As before, we should prove this as a pair of implications.  So let us prove \u000A first: <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (none-M) (<see topic=\"COMMON-LISP____NOT\">not</see> (some-M)))</v>.</p> \u000A \u000A <p>This may seem to assert an existential quantification in the conclusion, \u000A but rather, it asserts the <i>negation</i> of an existential quantification. \u000A We are now trying to prove that something does not exist.  How do we do that? \u000A We can show that nothing satisfies <v>M</v> by just showing that \u000A <v>(some-M-witness)</v> does not satisfy <v>M</v>.  This suggests the following \u000A rule of thumb:</p> \u000A \u000A <blockquote><p>RT8: When you encounter the negation of an existential \u000A quantification think in terms of a universal quantification, and \u000A vice-versa.</p></blockquote> \u000A \u000A <p>Ok, so now applying RT8 and RT3 you should be trying to apply the \u000A definition of <v>some-M</v>.  The hypothesis is just a pure (non-negated) \u000A universal quantification so you should apply RT4.  A blind application lets us \u000A prove the theorem as below.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> nl1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (none-M) (<see topic=\"COMMON-LISP____NOT\">not</see> (some-M)))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition some-M))\u000A                  (:instance none-M-necc (x (some-M-witness)))))))</code> \u000A \u000A <p>How about the converse implication?  I have deliberately written it as \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (none-M)) (some-M))</v> instead of switching the left-hand and \u000A right-hand sides of <v>nl1</v>, which would have been equivalent.  Again, RH8 \u000A tells us how to reason about it, in this case using RH2, and we succeed.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> nl2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (none-M)) (some-M))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition none-M))\u000A                  (:instance some-M-suff (x (none-M-witness)))))))</code> \u000A \u000A <p>So finally we just go through the motions of proving the equality.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">defthmd</see> |forall not = not exists|\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (none-M) (<see topic=\"COMMON-LISP____NOT\">not</see> (some-M)))\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance nl1)\u000A                  (:instance nl2)))))</code> \u000A \u000A <p>Let us now see if we can prove a slightly more advanced theorem which can \u000A be stated informally as: If there is a natural number <v>x</v> which satisfies \u000A <v>M</v>, then there is a least natural number <v>y</v> that satisfies <v>M</v>.</p> \u000A \u000A <p>[Note: Any time I have had to reason about existential quantification I \u000A have had to do this particular style of reasoning and state that if there is \u000A an object satisfying a predicate, then there is also a ``minimal'' object \u000A satisfying the predicate.]</p> \u000A \u000A <p>Let us formalize this concept.  We first define the concept of existence of \u000A a natural number satisfying <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-nat-M () (<see topic=\"ACL2____EXISTS\">exists</see> x (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> x) (M x))))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> some-nat-M some-nat-M-suff))</code> \u000A \u000A <p>We now talk about what it means to say that <v>x</v> is the least number \u000A satisfying <v>M</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> none-below (y)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> r (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> r) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> r y)) (<see topic=\"COMMON-LISP____NOT\">not</see> (M r))))))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> none-below none-below-necc))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> min-M () (<see topic=\"ACL2____EXISTS\">exists</see> y (<see topic=\"COMMON-LISP____AND\">and</see> (M y) (<see topic=\"ACL2____NATP\">natp</see> y) (none-below y))))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> min-M min-M-suff))</code> \u000A \u000A <p>The predicate <v>none-below</v> says that no natural number less than <v>y</v> \u000A satisfies <v>M</v>.  The predicate <v>min-M</v> says that there is some natural \u000A number <v>y</v> satisfying <v>M</v> such that <v>none-below</v> holds for \u000A <v>y</v>.</p> \u000A \u000A <p>So the formula we want to prove is: <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (some-nat-M)\u000A (min-M))</v>.</p> \u000A \u000A <p>Since the formula requires that we prove an existential quantification, RT1 \u000A tells us to construct some object satisfying the predicate over which we are \u000A quantifying.  We should then be able to instantiate <v>min-M-suff</v> with this \u000A object.  That predicate says that the object must be the least natural number \u000A that satisfies <v>M</v>.  Since such an object is uniquely computable if we know \u000A that there exists some natural number satisfying <v>M</v>, let us just write a \u000A recursive function to compute it.  This function is <v>least-M</v> below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> least-M-aux (i bound)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____NFIX\">nfix</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> bound) i))))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____NATP\">natp</see> i))\u000A             (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____NATP\">natp</see> bound))\u000A             (<see topic=\"COMMON-LISP_____E3\">&gt;</see> i bound))\u000A         0)\u000A       ((M i) i)\u000A       (t (least-M-aux (<see topic=\"COMMON-LISP_____B2\">+</see> i 1) bound))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> least-M (bound) (least-M-aux 0 bound))</code> \u000A \u000A <p>Let us now reason about this function as one does typically.  So we prove \u000A that this object is indeed the least natural number that satisfies <v>M</v>, \u000A assuming that <v>bound</v> is a natural number that satisfies <v>M</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> least-aux-produces-an-M\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> i)\u000A                (<see topic=\"ACL2____NATP\">natp</see> bound)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i bound)\u000A                (M bound))\u000A           (M (least-M-aux i bound))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> least-&lt;=bound\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 bound)\u000A           (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (least-M-aux i bound) bound)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> least-aux-produces-least\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> i)\u000A                (<see topic=\"ACL2____NATP\">natp</see> j)\u000A                (<see topic=\"ACL2____NATP\">natp</see> bound)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i j)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> j bound)\u000A                (M j))\u000A            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (least-M-aux i bound) j)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> least-aux-produces-natp\u000A  (<see topic=\"ACL2____NATP\">natp</see> (least-M-aux i bound)))\u000A\u000A(<see topic=\"ACL2____DEFTHMD\">defthmd</see> least-is-minimal-satisfying-m\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> bound)\u000A                (<see topic=\"ACL2____NATP\">natp</see> i)\u000A                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (least-M bound)))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (M i)))\u000A  :hints ((&quot;Goal&quot;\u000A            :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> least-aux-produces-least least-&lt;=bound)\u000A            :use ((:instance least-&lt;=bound\u000A                             (i 0))\u000A                  (:instance least-aux-produces-least\u000A                             (i 0)\u000A                             (j i))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> least-has-m\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> bound)\u000A                (m bound))\u000A           (M (least-M bound))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> least-is-natp\u000A  (<see topic=\"ACL2____NATP\">natp</see> (least-M bound)))</code> \u000A \u000A <p>So we have done that, and hopefully this is all that we need about \u000A <v>least-M</v>.  So we disable everything.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> least-M natp))</code> \u000A \u000A <p>Now of course we note that the statement of the conjecture we are \u000A interested in has two quantifiers, an inner <v>forall</v> (from <v>none-below</v>) \u000A and an outer <v>exists</v> (from <v>min-M</v>).  Since ACL2 is not very good with \u000A quantification, we hold its hands to reason with the quantifier part.  So we \u000A will first prove something about the <v>forall</v> and then use it to prove what \u000A we need about the <v>exists</v>.</p> \u000A \u000A <blockquote><p>RT9: When you face nested quantifiers, reason about each \u000A nesting separately.</p></blockquote> \u000A \u000A <p>So what do we want to prove about the inner quantifier?  Looking carefully \u000A at the definition of <v>none-below</v> we see that it is saying that for all \u000A natural numbers <v>r</v> &lt; <v>y</v>, <v>(M r)</v> does not hold.  Well, how would \u000A we want to use this fact when we want to prove our final theorem?  We expect \u000A that we will instantiate <v>min-M-suff</v> with the object <v>(least-M bound)</v> \u000A where we know (via the outermost existential quantifier) that <v>M</v> holds for \u000A <v>bound</v>, and we will then want to show that <v>none-below</v> holds for \u000A <v>(least-M bound)</v>.  So let us prove that for any natural number (call it \u000A <v>bound</v>), <v>none-below</v> holds for <v>(least-M bound)</v>.  For the final \u000A theorem we only need it for natural numbers satisfying <v>M</v>, but note that \u000A from the lemma <v>least-is-minimal-satisfying-m</v> we really do not need that \u000A <v>bound</v> satisfies <v>M</v>.</p> \u000A \u000A <p>So we are now proving: <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> bound) (none-below (least-M\u000A bound)))</v>.</p> \u000A \u000A <p>Well since this is a standard case of proving a universally quantified \u000A predicate, we just apply RT3.  We have proved that for all naturals <v>i</v> \u000A &lt; <v>(least-M bound)</v>, <v>i</v> does not satisfy <v>M</v> (lemma \u000A <v>least-is-minimal-satisfying-M</v>), so we merely need the instantiation of \u000A that lemma with <v>none-below-witness</v> of the thing we are trying to prove, \u000A that is, <v>(least-M bound)</v>.  The theorem below thus goes through.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> least-is-minimal\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> bound)\u000A           (none-below (least-M bound)))\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance (:definition none-below)\u000A                             (y (least-M bound)))\u000A                  (:instance least-is-minimal-satisfying-m\u000A                             (i (none-below-witness (least-M bound))))))))</code> \u000A \u000A <p>Finally we are in the outermost existential quantifier, and are in the \u000A process of applying <v>min-M-suff</v>.  What object should we instantiate it \u000A with?  We must instantiate it with <v>(least-M bound)</v> where <v>bound</v> is an \u000A object which must satisfy <v>M</v> and is a natural.  We have such an object, \u000A namely <v>(some-nat-M-witness)</v> which we know have all these qualities given \u000A the hypothesis.  So the proof now is just RT1 and RT2.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> |minimal exists|\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (some-nat-M) (min-M))\u000A  :hints ((&quot;Goal&quot;\u000A            :use ((:instance min-M-suff\u000A                             (y (least-M (some-nat-M-witness))))\u000A                  (:instance (:definition some-nat-M))))))</code> \u000A \u000A <p>If you are comfortable with the reasoning above, then you are comfortable \u000A with quantifiers and probably will not need these notes any more.  In my \u000A opinion, the best way of dealing with ACL2 is to ask yourself why you think \u000A something is a theorem, and the rules of thumb above are simply guides to the \u000A questions that you need to ask when you are dealing with quantification.</p> \u000A \u000A <p>Here are a couple of simple exercises for you to test if you understand the \u000A reasoning process.</p> \u000A \u000A <p><b>Exercise 1</b>.  Formalize and prove the following theorem.  Suppose \u000A there exists <v>x</v> such that <v>(R x)</v> and suppose that all <v>x</v> satisfy \u000A <v>(P x)</v>.  Then prove that there exists <v>x</v> such that <v>(P x) &amp; (R x)</v>. \u000A (See <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/contrib/quantifier-exercise-1-solution.html'>http://www.cs.utexas.edu/users/moore/acl2/contrib/quantifier-exercise-1-solution.html</a> \u000A for a solution.)</p> \u000A \u000A <p><b>Exercise 2</b>.  Recall the example just before the preceding exercise, \u000A where we showed that if there exists a natural number <v>x</v> satisfying <v>M</v> \u000A then there is another natural number <v>y</v> such that <v>y</v> satisfies <v>M</v> \u000A and for every natural number <v>z</v> &lt; <v>y</v>, <v>z</v> does not.  What would \u000A happen if we remove the restriction of <v>x</v>, <v>y</v>, and <v>z</v> being \u000A naturals?  Of course, we will not talk about <v>&lt;</v> any more, but suppose you \u000A use a total order on all ACL2 objects such as <see topic=\"ACL2_____C3_C3\">&lt;&lt;</see>.  More concretely, \u000A consider the definition of <v>some-M</v> above.  Let us now define two other \u000A functions:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/total-order&quot; :dir :system)\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> none-below-2 (y)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> r (<see topic=\"ACL2____IMPLIES\">implies</see> (&lt;&lt; r y) (<see topic=\"COMMON-LISP____NOT\">not</see> (M r)))))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> min-M2 () (<see topic=\"ACL2____EXISTS\">exists</see> y (<see topic=\"COMMON-LISP____AND\">and</see> (M y) (none-below-2 y))))</code> \u000A \u000A <p>The question is whether <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (some-M) (min-M2))</v> is a theorem.  Can \u000A you prove it?  Can you disprove it?</p>"],

 "ACL2____NOTE-2-9-3":[["Release-notes"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____NOTE-2-9-1\">note-2-9-1</see> and see <see topic=\"ACL2____NOTE-2-9-2\">note-2-9-2</see> for other \u000A changes since the last non-incremental release (Version_2.9).</p> \u000A \u000A <p>We fixed a soundness bug that exploited the ability to define <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions that are improperly guarded, and then to use those \u000A functions in <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> forms.  The fix is to evaluate <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> \u000A forms using the same ``safe-mode'' that is used in macroexpansion (see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>).  Here is a proof of <v>nil</v> that succeeded in \u000A Allegro Common Lisp (but not, for example, GCL).  See also a long comment in \u000A source function <v>defconst-fn</v> for an example that does not require the use \u000A of <v>:set-guard-checking</v>.</p> \u000A \u000A <code>:set-guard-checking nil ; execute before certifying the book below\u000A\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 ()\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A          (<see topic=\"COMMON-LISP____CHAR-UPCASE\">char-upcase</see> (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 224))))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFCONST\">defconst</see> *b* (f1)))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 ()\u000A   (<see topic=\"COMMON-LISP____CHAR-UPCASE\">char-upcase</see> (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 224)))\u000A (<see topic=\"ACL2____DEFCONST\">defconst</see> *b* (f1))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> *b* (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 224)))\u000A   :rule-classes nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use bad))\u000A  :rule-classes nil)</code> \u000A \u000A <p>We fixed a soundness hole due to the fact that the \"LISP\" package does \u000A not exist in OpenMCL.  We now explicitly disallow this package name as an \u000A argument to <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>.  Thanks to Bob Boyer and Warren Hunt for bringing \u000A an issue to our attention that led to this fix.</p> \u000A \u000A <p>ACL2 now requires all package names to consist of standard characters (see \u000A <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>, none of which is lower case.  The reason is that we \u000A have seen at least one lisp implementation that does not handle lower case \u000A package names correctly.  Consider for example the following raw lisp log \u000A (some newlines omitted).</p> \u000A \u000A <code>&gt;(make-package &quot;foo&quot;)\u000A#&lt;&quot;foo&quot; package&gt;\u000A&gt;(package-name (symbol-package 'FOO::A))\u000A&quot;foo&quot;\u000A&gt;(package-name (symbol-package '|FOO|::A))\u000A&quot;foo&quot;\u000A&gt;</code> \u000A \u000A <p>Distributed book <v>books/textbook/chap10/compiler</v>, as well as workshop \u000A books in directory <v>books/workshops/2004/cowles-gamboa/support/</v>, were \u000A modified to accommodate the above change.</p> \u000A \u000A <p>Added <v>newline</v>, <tt><see topic=\"ACL2____ADD-TO-SET-EQL\">add-to-set-eql</see></tt>, <v>the-fixnum</v>, and \u000A <v>the-fixnum!</v>  to <v>*acl2-exports*</v>.  Thanks to Jared Davis for bringing \u000A these to our attention.</p> \u000A \u000A <p>Added a line to <v>acl2.lisp</v> to support CMUCL running on Mac OSX, thanks \u000A to a suggestion from Fabricio Chalub Barbosa do Rosario.</p> \u000A \u000A <p>The executable scripts for saved ACL2 images now include <v>$*</v>, so that \u000A command-line arguments will be passed along.</p> \u000A \u000A <p>(For GCL profiling only) Fixed a colon (<v>:</v>) that should have been a \u000A semicolon (<v>;</v>) in file <v>save-gprof.lsp</v>.  Thanks to David Hardin for \u000A pointing out this bug.</p> \u000A \u000A <p>The documentation for <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt> rules has been expanded and \u000A improved, thanks to useful feedback from Hanbing Liu.</p> \u000A \u000A <p>Fixed a bug in the guard for function <v>include-book-dir</v>.</p> \u000A \u000A <p>For those who want to experiment with an alternate implementation of <tt><see topic=\"ACL2____MV\">mv</see></tt> and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>, there is now support for under-the-hood implementation \u000A of these in terms of raw Lisp functions <v>values</v> and \u000A <v>multiple-value-bind</v>, respectively.  The regression suite has seen about a \u000A 10% speed-up in Allegro CL and about an 8% slowdown in GCL for builds with \u000A this change.  See the makefile (<v>GNUmakefile</v>) for examples of how to build \u000A ACL2 by including the feature, <v>:acl2-mv-as-values</v>.  Source file \u000A <v>init.lsp</v> has been renamed to <v>init.lisp</v> in support of this change \u000A (technical detail: otherwise GCL loads the init file too soon, before its \u000A <v>-eval</v> argument is evaluated).  Thanks to David Rager for inspiring this \u000A change, by pointing out the problematic use of globals by the existing <tt><see topic=\"ACL2____MV\">mv</see></tt> implementation from the standpoint of supporting parallel evaluation. \u000A This capability is experimental: there is likely to be some remaining work to \u000A be done on it.</p> \u000A \u000A <p>A change related to the one just above is that we now limit the maximum \u000A number of arguments to any call of <tt><see topic=\"ACL2____MV\">mv</see></tt> to 32.  Thanks to Bob Boyer for \u000A raising a question that lead to this change.</p> \u000A \u000A <p>Eliminated some compiler warnings in OpenMCL.</p> \u000A \u000A <p>In the rtl library (<v>books/rtl/rel4/</v>), functions <v>bits</v> and \u000A <v>setbits</v> have had their <see topic=\"ACL2____GUARD\">guard</see>s improved (as they had been too \u000A restrictive, especially for <v>setbits</v>).</p> \u000A \u000A <p>A new function <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> permits timing of forms, by using (under the \u000A hood) the host Common Lisp's <v>time</v> utility.</p> \u000A \u000A <p>We fixed an infinite loop that could occur during destructor elimination \u000A (see <see topic=\"ACL2____ELIM\">elim</see>).  Thanks to Sol Swords to bringing this to our attention and \u000A sending a nice example, and to Doug Harper for sending a second example that \u000A we also found useful.</p> \u000A \u000A <p>The method of speeding up GCL-based builds (see <see topic=\"ACL2____NOTE-2-9-2\">note-2-9-2</see>) has \u000A changed slightly from Version_2.9.2.  Now, in the `make' command:</p> \u000A \u000A <code>LISP='gcl -eval &quot;(defparameter user::*fast-acl2-gcl-build* t)&quot;</code> \u000A \u000A <p>We improved the pretty-printer's handling of keywords.  For example, before \u000A this change one might see the following printed by ACL2.</p> \u000A \u000A <code>(MODIFY TH S :KEY1 VAL1 :KEY2\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____IF\">IF</see> X Y Z) AAAAAAAAAA BBBBBBB))</code> \u000A \u000A <p>Now, the above might print as follows.  Notice that we have avoided \u000A breaking after a keyword (see <see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see>) that is preceded by other forms \u000A on the same line.</p> \u000A \u000A <code>(MODIFY TH S\u000A        :KEY1 VAL1\u000A        :KEY2 (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____IF\">IF</see> X Y Z) AAAAAAAAAA BBBBBBB))</code> \u000A \u000A <p>See <see topic=\"ACL2____NOTE-2-9-3-PPR-CHANGE\">note-2-9-3-ppr-change</see> for a detailed discussion of this \u000A change.</p> \u000A \u000A <p>(GCL ONLY) Evaluation in a break is no longer inhibited by ACL2 when built \u000A on top of GCL, so GCL now matches other Common Lisps in this respect.</p> \u000A \u000A <p>For ACL2 built on most host Common Lisps, you will see the string <v>[RAW\u000A LISP]</v> in the prompt, at least at a break, to emphasize that one is inside a \u000A break and hence should probably quit from the break.  See <see topic=\"ACL2____BREAKS\">breaks</see>.</p> \u000A \u000A <p>Jared Davis suggested improvements to lemmas <v>len-update-nth</v> (in source \u000A file <v>axioms.lisp</v>) and <v>append-true-listp-type-prescription</v> (in \u000A <v>books/meta/term-defuns.lisp</v>), which have been incorporated.  The former \u000A required a change in <v>books/workshops</v> book \u000A <v>2004/ruiz-et-al/support/q-dag-unification.cert</v>, which has been made.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>rewrite</v> allows further binding of \u000A free variables in hypotheses, with new optional argument \u000A <v>instantiate-free</v>.  Proof-checker command <v>show-rewrites</v> (<v>sr</v>) \u000A gives corresponding additional information.  Documentation for these commands \u000A has been improved; see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  Thanks to John Matthews \u000A and Bill Young for suggestions and feedback leading to these improvements.</p> \u000A \u000A <p>Fixed downcase printing so that the package name of a symbol is also \u000A downcased.  For example, after execution of <v>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;FOO&quot; nil)</v> and \u000A <v>(set-acl2-print-case :downcase)</v>, <v>'foo::ab</v> will print back as the \u000A same, rather than as <v>'FOO::ab</v>.</p> \u000A \u000A <p>It is now possible to control the output so that numbers are printed in \u000A binary, octal, or hex, though the default is still radix 10.  See <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see>.  Note that in support of this change, built-in functions \u000A <tt><see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see></tt> and <v>explode-atom</v> now take an extra \u000A <v>print-base</v> argument.  Different support for radix conversion may be found \u000A in a book newly contributed by Jun Sawada, <v>books/misc/radix.lisp</v>.</p> \u000A \u000A <p>Built-in axiom <v>car-cdr-elim</v> is now only an <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt> rule. \u000A It was formerly both an <v>:elim</v> rule and a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule.  A \u000A new rule, <v>cons-car-cdr</v>, takes the place of the old <v>:rewrite</v> rule, \u000A but is instead a hypothesis-free rule that can cause a case split (see source \u000A file <v>axioms.lisp</v>).  Thanks to Jared Davis for suggesting this change.</p> \u000A \u000A <p>Lemmas about <tt><see topic=\"ACL2____ALPHORDER\">alphorder</see></tt> (<v>alphorder-reflexive</v>, \u000A <v>alphorder-transitive</v>, <v>alphorder-anti-symmetric</v>, and \u000A <v>alphorder-total</v>) are now available.  (They had been <tt><see topic=\"ACL2____LOCAL\">local</see></tt> in \u000A source file <v>axioms.lisp</v>.)  Thanks to Serita Nelesen for bringing this \u000A issue to our attention.</p> \u000A \u000A <p>ACL2 has, for some time, printed a space in the event summary after the \u000A open parenthesis for a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event, in order to ease backward \u000A searching for the original form, for example <v>(<see topic=\"ACL2____DEFTHM\">defthm</see> bar ...)</v>:</p> \u000A \u000A <code>Form:  ( DEFTHM BAR ...)</code> \u000A \u000A <p>The intention was that this extra space should be printed for every event \u000A form; but it was missing in some cases, for example, for <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.  This has been fixed.</p> \u000A \u000A <p>In analogy to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, now <tt><see topic=\"ACL2____LD\">ld</see></tt> takes the (optional) \u000A keyword argument <v>:dir</v>.  Thanks to Jared Davis for providing an \u000A implementation of this feature and to Eric Smith and Jeff Marshall for \u000A requesting this feature.</p> \u000A \u000A <p>We fixed a bug in <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> that could cause an error when \u000A redefinition is on, for example:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-REDEFINITION-ACTION\">set-ld-redefinition-action</see> '(:warn! . :overwrite) state)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;/u/acl2/books/arithmetic/top&quot;)</code> \u000A \u000A <p>The behavior of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> now matches the documentation: \u000A handling of compiled files for uncertified <see topic=\"ACL2____BOOKS\">books</see> will follow the same \u000A rules as for certified books.  In particular, if you create an object file in \u000A raw Lisp for some book, then including that book will load that object file. \u000A Thanks to Jared Davis for bringing this issue to our attention.</p> \u000A \u000A <p>New documentation explains the interaction of redefinition and redundancy. \u000A See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see> &mdash; the ``Note About Unfortunate \u000A Redundancies'' is new.  Thanks to Grant Passmore for providing examples that \u000A led us to write this additional documentation.</p> \u000A \u000A <p>Solutions to exercises in <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/how-to-prove-thms'> \u000A How To Prove Theorems Formally</a> are now available in distributed book \u000A <v>books/misc/how-to-prove-thms.lisp</v>.  Also in that directory may be found a \u000A new book <v>hanoi.lisp</v> that contains a solution to the Towers of Hanoi \u000A problem.</p>"],

 "ACL2____NOTE-2-9-1":[["Release-notes"],"ACL2 Sources","ACL2","<p> \u000A (GCL only) A bug in <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> has been fixed that could be \u000A exploited to prove <v>nil</v>, and hence is a soundness bug.  Thanks to Dave \u000A Greve for sending us an example of a problem with <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> (see below) \u000A that led us to this discovery.</p> \u000A \u000A <p>ACL2 now warns when <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> specifies <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> as the first \u000A equivalence relation, e.g., <v>(<see topic=\"ACL2____DEFCONG\">defcong</see> equal iff (<see topic=\"COMMON-LISP____MEMBER\">member</see> x y) 2)</v>.  The \u000A warning says that the rule has no effect because <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> already refines \u000A all other equivalence relations.  Formerly, this caused an error unless \u000A <v>:event-name</v> was supplied (see <see topic=\"ACL2____DEFCONG\">defcong</see>), and in fact the error was \u000A a nasty raw Lisp error on GCL platforms due to some mishandling of packages by \u000A ACL2 that has been fixed (see the paragraph about <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> \u000A above).  Thanks to Dave Greve for sending a helpful example in his report of \u000A this problem.</p> \u000A \u000A <p>(GCL only) The build process was broken for GCL 2.6.0 (and perhaps some \u000A earlier versions), and has been fixed.  Thanks to Jose Luis Ruiz-Reyna for \u000A bringing this problem to our attention.</p> \u000A \u000A <p>(GCL only) We have increased the hole size to at least 20% of max-pages, \u000A which may eliminate some garbage collection at the expense of larger virtual \u000A memory (not larger resident memory or larger image).  Thanks to Camm Maguire \u000A for helpful explanations on this topic.</p> \u000A \u000A <p>We have clarified the <see topic=\"ACL2____GUARD\">guard</see> warning message that is printed during \u000A evaluation of recursively-defined functions whose <see topic=\"ACL2____GUARD\">guard</see>s have not been \u000A verified, for example:</p> \u000A \u000A <code>ACL2 Warning [Guards] in TOP-LEVEL:  Guard-checking may be inhibited\u000Aon some recursive calls of executable counterparts (i.e., in the ACL2\u000Alogic), including perhaps EVENLP.  To check guards on all recursive\u000Acalls:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :all)\u000ATo leave behavior unchanged except for inhibiting this message:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :nowarn)</code> \u000A \u000A <p>And, ACL2 no longer prints that message when the <see topic=\"ACL2____GUARD\">guard</see> was \u000A unspecified for the function or was specified as <v>T</v>.  Thanks to Serita \u000A Nelesen for bringing the latter issue to our attention.  Finally, ACL2 now \u000A prints such a warning at most once during the evaluation of any top-level \u000A form; thanks to Bill Young for pointing out this issue.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see></tt> has been enhanced to allow specified \u000A prover functions <b>not</b> to be traced.  See <see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see>.</p> \u000A \u000A <p>Added <tt><see topic=\"ACL2____LP\">lp</see></tt>, <v>wet</v>, and <tt><see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see></tt> to \u000A <v>*acl2-exports*</v>, and hence to the <v>&quot;</v><tt><see topic=\"ACL2____ACL2-USER\">ACL2-user</see></tt><v>&quot;</v> \u000A package.</p> \u000A \u000A <p>The distributed book <v>books/arithmetic-3/bind-free/integerp.lisp</v> has \u000A been modified in order to prevent potential looping; specifically, the \u000A definition of function <v>reduce-integerp-+-fn-1</v>.  Thanks to Robert Krug for \u000A providing this change.</p> \u000A \u000A <p>A small improvement was made in the <v>wet</v> failure message when the error \u000A occurs during translation to internal form.  Thanks to Jared Davis for \u000A pointing out the obscurity of some <v>wet</v> error messages.</p> \u000A \u000A <p>We have improved ACL2's evaluation mechanism for the function \u000A <v>bad-atom&lt;=</v>, which now is specified to return <v>nil</v> if neither argument \u000A is a so-called ``bad atom'' (as recognized by function <v>bad-atom</v>).  The \u000A following events had caused a hard error, for example.  (We're sorry that \u000A <v>bad-atom</v> and <v>bad-atom&lt;=</v> are not documented, but we also consider it \u000A unlikely that anyone needs such documentation; otherwise, please contact the \u000A implementors.)</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (bad-atom&lt;= x y))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (foo x y))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar 3 4) 7))</code> \u000A \u000A <p>We have also changed the guard on <tt><see topic=\"ACL2____ALPHORDER\">alphorder</see></tt> to require both \u000A arguments to be atoms.</p> \u000A \u000A <p>For forms <v>(<see topic=\"ACL2____LOCAL\">local</see> x)</v> that are skipped during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, or \u000A during the second pass of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, ACL2 \u000A had nevertheless checked that <v>x</v> is a legal event form.  This is no longer \u000A the case.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> now does non-linear arithmetic when appropriate. \u000A It had formerly ignored <tt><see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see></tt> executed in the ACL2 command \u000A loop.</p> \u000A \u000A <p>Incremental releases are now supported.  See <see topic=\"ACL2____VERSION\">version</see> and {obsolete \u000A after Version 4.3} set-tainted-okp.  Thanks to Hanbing Liu for discovering a \u000A flaw in our original design.</p> \u000A \u000A <p>The pattern-matching algorithm for <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules has been \u000A made slightly more restrictive, thanks to a suggestion and examples from \u000A Robert Krug.  For example, previously one could get an infinite loop as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x) t)\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> foo-axiom\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x))\u000A         (foo x)))\u000A(<see topic=\"ACL2____THM\">thm</see> (foo 0)) ; or replace 0 by any integer!</code> \u000A \u000A <p>That is because the term <v>(foo 0)</v> was considered to match against the \u000A pattern <v>(foo (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x))</v>, with <v>x</v> bound to <v>-1</v>.  While such matching \u000A is sound, it leads to an infinite loop since it allows <v>foo-axiom</v> to \u000A rewrite <v>(foo 0)</v> to <v>(foo -1)</v>, and then <v>(foo -1)</v> to <v>(foo\u000A -2)</v>, and so on.  The fix is to insist that the new value, in this case \u000A <v>-1</v>, is no larger in size according to <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> than the old \u000A value, in this case <v>0</v>.  Since that test fails, the match is considered to \u000A fail and the loop no longer occurs.  An analogous fix has been made for \u000A multiplication, where now we only match when the new term is still a non-zero \u000A integer.  That change avoids a loop here.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x) t)\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> foo-axiom\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x))\u000A         (foo x)))\u000A(<see topic=\"ACL2____THM\">thm</see> (foo 0)) ; or try (<see topic=\"ACL2____THM\">thm</see> (foo 4))</code> \u000A \u000A <p>Added macro <v>find-lemmas</v> in <v>books/misc/find-lemmas.lisp</v> (see brief \u000A documentation there) for finding all lemmas that mention all function symbols \u000A in a given list.</p> \u000A \u000A <p><v>:Restrict</v> <see topic=\"ACL2____HINTS\">hints</see> now work for <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules, \u000A though they continue to be ignored by the preprocessor and hence you may want \u000A to use <v>:do-not '(preprocess)</v> with any restrict hints.  Thanks to John \u000A Matthews for pointing out the lack of support for <v>:definition</v> rules in \u000A <v>:restrict</v> hints.</p> \u000A \u000A <p>Some books have been updated.  In particular, there is a new directory \u000A <v>books/workshops/2004/</v> in workshops distribution, for the 2004 ACL2 \u000A workshop.  There is also a new version of Jared Davis's ordered sets library, \u000A formerly in <v>books/finite-set-theory/osets-0.81/</v> but now in \u000A <v>books/finite-set-theory/osets/</v>.</p> \u000A \u000A <p>Fixed a bug in the (under-the-hood) raw Lisp definition of <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>, which had been causing a warning in CMU Common Lisp.</p> \u000A \u000A <p>[Technical improvements related to the use of ``<v>make dependencies</v>'' \u000A for certifying distributed books:]<br></br> File <v>books/Makefile-generic</v> \u000A now does a better job with ``<v>make dependencies</v>,'' specifically with \u000A respect to handling <v>*.acl2</v> files and handling <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A commands with <v>:dir :system</v>.  Regarding the latter, suppose for example \u000A that book <v>basic.lisp</v> contains the line:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic/top-with-meta&quot; :dir :system)</code> \u000A \u000A <p>Then <v>make dependencies</v> would generate the following line:</p> \u000A \u000A <code>basic.cert: $(ACL2_SRC_BOOKS)/arithmetic/top-with-meta.cert</code> \u000A \u000A <p>Thus, if <v>:dir :system</v> is used with <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the \u000A corresponding <v>Makefile</v> should define the variable <v>ACL2_SRC_BOOKS</v>.  A \u000A standard <v>Makefile</v> header for a books directory could thus be as \u000A follows.</p> \u000A \u000A <code># The following variable should represent the ACL2 source directory.  It is the\u000A# only variable in this Makefile that may need to be edited.\u000AACL2_SRC = ../../../../../..\u000A\u000AACL2_SRC_BOOKS = $(ACL2_SRC)/books\u000Ainclude $(ACL2_SRC_BOOKS)/Makefile-generic\u000AACL2 = $(ACL2_SRC)/saved_acl2</code> \u000A \u000A <p>Finally, the ``<v>-s</v>'' flag may now be omitted when running ``<v>make\u000A dependencies</v>.''</p>"],

 "ACL2____NOTE-2-7-BUG-FIXES":[["Note-2-7"],"ACL2 Sources","ACL2","<p>Francisco J. Martin-Mateos emailed us a soundness bug (!) in our \u000A handling of functional instantiation (for example see <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE\">functional-instantiation-example</see>).  We are grateful for that email, which \u000A clearly illustrated the problem.  It is included just below the definition of \u000A <v>push-clause</v> in ACL2 source file <v>prove.lisp</v>, where we have fixed the \u000A bug.  This bug was fixed in a re-release of Version 2.6 in February, 2002.</p> \u000A \u000A <p>Rob Sumners emailed us a soundness bug (!) in function <v>commutative-p1</v>, \u000A which is used by the ACL2 <see topic=\"ACL2____BDD\">bdd</see> package.  We are grateful for his help; \u000A his email gave a proof of nil and also pointed to the problem function.  This \u000A bug was fixed in a re-release of Version 2.6 in February, 2002.</p> \u000A \u000A <p>We discovered and fixed a soundness bug illustrated by the book below, \u000A which was certifiable in Version 2.6 and ends in a proof of <v>nil</v>.  The \u000A event <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)</v> should have been rejected, because <v>foo</v> \u000A calls a function whose guards have not been verified, namely, <v>bar</v>. \u000A However, ACL2 did not notice the call of function <v>bar</v> in the body of \u000A <v>foo</v> because it was looking in the simplified (normalized) body of \u000A <v>foo</v> rather than in the original body of <v>foo</v>.  During processing of \u000A the book below, the logical definition of <v>zp</v> is used before \u000A <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)</v>, and <v>(<see topic=\"ACL2____ZP\">zp</see> -3)</v> reduces to <v>t</v> in the logic. \u000A After <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)</v>, ACL2 simplifies <v>(foo -3)</v> by going into \u000A raw Lisp, where <v>(<see topic=\"ACL2____ZP\">zp</see> -3)</v> is evaluated and reduces to <v>nil</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A  (<see topic=\"ACL2____ZP\">zp</see> x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> zp-false-on-negatives\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A           (bar x))\u000A  :rule-classes :type-prescription)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                  :verify-guards nil))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (bar x) 0 1) ; simplified body reduces this line to 0\u000A    17))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-of-minus-3-is-0\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo -3) 0)\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-of-minus-3-is-1\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo -3) 1)\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bug\u000A  nil\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use (foo-of-minus-3-is-0 foo-of-minus-3-is-1))))</code> \u000A \u000A <p>The above bug exploited the fact that <tt><see topic=\"ACL2____ZP\">zp</see></tt> has a different definition \u000A in raw Lisp than in the logic for arguments that violate its guard).  The \u000A following example caused a hard error in raw Lisp, though not a soundness \u000A error.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                  :verify-guards nil))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (bar x) x nil))\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bug\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo 3) t)\u000A  :rule-classes nil)</code> \u000A \u000A <p>We have made a minor change to the notion of the <i>formula</i> of a \u000A function symbol, related to the change above, which however is unlikely to be \u000A noticeable.</p> \u000A \u000A <p>In order to make it harder to hit problems like the guard problem above, we \u000A have slighly modified the raw Lisp definition of <tt><see topic=\"ACL2____ZP\">zp</see></tt>.</p> \u000A \u000A <p>A <tt><see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see></tt> command, <v>:ancestors</v>, was broken, but has been \u000A fixed.  Thanks to Eric Smith for bringing the problem to our attention, and to \u000A Robert Krug for supplying the final part of the fix.</p> \u000A \u000A <p>Some <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands caused errors when all goals have \u000A already been proved.  This has been fixed.  Thanks to Matt Wilding for \u000A reporting this bug.</p> \u000A \u000A <p>Fixed a bug in <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>.  When compiling uncompiled functions \u000A with very large definitions, ACL2 was inserted a backslash (<v>\\</v>) character \u000A into generated files.</p> \u000A \u000A <p>Fixed the <v>:type-alist</v> <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt> command (see <see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see>), whose output was difficult to read when typed after an \u000A <v>:eval</v>..</p> \u000A \u000A <p>Fixed some clumsy handling of errors when including an uncertified book, \u000A for example, with the error message when including an uncertified book with a \u000A bad <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event.  Thanks to Eric Smith for pointing out this \u000A problem.</p> \u000A \u000A <p>Two modifications to <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> now cause it to reflect natural \u000A expectations with respect to soundness.  First, it now has default values of \u000A <v>nil</v> instead of <v>t</v> for keyword arguments <v>:skip-proofs-okp</v> and \u000A <v>:defaxioms-okp</v>.  Thanks to Robert Krug for suggesting this change and the \u000A ACL2 seminar at the University of Texas for discussing it.  Second, when \u000A <v>:skip-proofs-okp</v> (respectively, <v>:defaxioms-okp</v>) is <v>nil</v>, either \u000A explicitly or by default, then <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> commands (respectively, \u000A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> events) are disallowed inside any included books, regardless \u000A of the keyword parameters passed to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  This had not been \u000A the case for previous versions of ACL2, regardless of the values of \u000A <v>:skip-proofs-okp</v> or <v>:defaxioms-okp</v> passed to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>Improved warnings and errors for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to mention the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> as a possible source of <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> and <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>s.</p> \u000A \u000A <p>ACL2 formerly caused an error when <see topic=\"ACL2____HINTS\">hints</see> in a <v>:</v><tt><see topic=\"ACL2____COROLLARY\">corollary</see></tt> were not well-formed.  This situation could arise as follows when \u000A certifying a book.  A lemma FOO is proved <tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly to the book (or, is \u000A present in a sub-book that is included locally).  The <v>:corollary</v> of a \u000A subsequent theorem, BAR, disables that rule in a hint.  When BAR is proved, \u000A this is not a problem.  But <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> makes a second pass after \u000A processing the events in a book: it essentially does an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>. \u000A During the <v>include-book</v> pass, FOO is not known (because it was <tt><see topic=\"ACL2____LOCAL\">local</see></tt>), and therefore ACL2 fails to process the <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> of FOO in an \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint.  The fix is that during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <see topic=\"ACL2____HINTS\">hints</see> are ignored in corollaries just as they have been for the main theorem \u000A (or definition).</p> \u000A \u000A <p>It was possible for guard verification to succeed where it should have \u000A failed.  We have fixed the bug (which was in source function (ironically \u000A named!) <v>fcons-term-smart</v>).  Thanks to Robert Krug for sending us an \u000A example of bungled guard verification.  It turns out that this bug was also \u000A present in Version_2.6.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>=</v> has been improved.  Formerly, it \u000A could fail to apply when certain <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> terms were in the context. \u000A Thanks to Pete Manolios for bringing this problem to our attention.</p> \u000A \u000A <p>The command <tt><see topic=\"ACL2____ADD-BINOP\">add-binop</see></tt> failed to work.  This has been fixed.  Thanks \u000A to Rob Sumners for pointing out this problem.  Also see <see topic=\"ACL2____NOTE-2-7-OTHER\">note-2-7-other</see> \u000A for a discussion of how this and another <see topic=\"ACL2____TABLE\">table</see> are no longer part of \u000A the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.</p> \u000A \u000A <p>Book certification could cause a segmentation fault in cases where the \u000A certification world (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>) has a very large number of \u000A events.  This has been fixed.</p> \u000A \u000A <p>We now allow empty <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see> and empty hints, as requested by \u000A Eric Smith.  Examples:</p> \u000A \u000A <code>(&quot;Goal&quot; :use ())\u000A(&quot;Goal&quot;)</code> \u000A \u000A <p>A large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nest could cause a stack overflow when \u000A executing either <v>:pr FN</v>, <v>:pr! FN</v>, or <v>:monitor (:definition FN)\u000A t</v>, where <v>FN</v> is in that large mutual recursion nest.  This has been \u000A fixed (implementation detail: function <v>actual-props</v> has been made \u000A tail-recursive).  NOTE: If you just want the definition of <v>FN</v>, \u000A <v>:</v><tt><see topic=\"ACL2____PF\">pf</see></tt><v> FN</v> can be much faster than <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt><v> FN</v> if \u000A <v>FN</v> is in a large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>.</p> \u000A \u000A <p>Hard Lisp errors could occur when including uncertified books.  This has \u000A been fixed; ACL2 now does syntax-checking formerly omitted when including \u000A uncertified books.</p> \u000A \u000A <p>Previously, the evaluation of <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> and <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> \u000A forms could cause ``undefined'' warnings when the form was compiled.  This has \u000A been fixed.  Thanks to Eric Smith for bring a <v>mutual-recursion</v> example to \u000A our attention.</p> \u000A \u000A <p>A bug has been fixed in the syntactic check for valid <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> values.  Formerly, valid <v>:loop-stopper</v> values were \u000A erroneously restricted to lists of length at most 2 (a minor problem, since \u000A these lists typically have length 1), and the function symbol(s) need not have \u000A been defined in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.  Thanks to Eric Smith for \u000A sending an example to demonstrate the latter problem.</p> \u000A \u000A <p>Functions definitions that are <v>:non-executable</v> (see <see topic=\"ACL2____XARGS\">xargs</see>) had \u000A never been recognized as redundant, but this has been fixed.  Thanks to Vernon \u000A Austel for pointing out this problem.</p> \u000A \u000A <p>Compilation using <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> now compiles user-defined <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions.  Formerly only <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions \u000A could be compiled using <v>:comp</v>.</p> \u000A \u000A <p>Handling of <v>:by</v> hints has been improved in essentially three ways. \u000A The primary change is that now, when the current goal exactly matches the \u000A supplied lemma instance, the subsumption test will always succeeds (see <see topic=\"ACL2____HINTS\">hints</see>, in particular the discussion of <v>:by</v>).  Second, certain proof \u000A failures involving <v>:by</v> <see topic=\"ACL2____HINTS\">hints</see> were failing silently, with duplicate \u000A messages ``As indicated by the hint, this goal is subsumed by....''  This \u000A could happen when the original goal was among the goals generated by applying \u000A the hint.  This problem has been fixed by no longer considering this proof \u000A step to be specious (see <see topic=\"ACL2____SPECIOUS-SIMPLIFICATION\">specious-simplification</see>).  Third and finally, \u000A when the <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> refers to a definition, the original body of \u000A that definition is used rather than the simplfied (``normalized'') body.</p> \u000A \u000A <p>In addition to the obove, we now recognize more cases of specious \u000A simplification (see <see topic=\"ACL2____SPECIOUS-SIMPLIFICATION\">specious-simplification</see>).  Thanks to Eric Smith for \u000A bringing this issue to our attention.</p> \u000A \u000A <p>Fixed building of ACL2 under CLISP so that (1) the appropriate ACL2 startup \u000A message is printed out when ACL2 starts up, and (2) the lisp process supplied \u000A to make, e.g., LISP=/usr/bin/clisp, is the one written out to the saved ACL2 \u000A file.  Thanks to Dave Greve and Noah Friedman for suggesting (2).  Also, ACL2 \u000A now works with CLISP 2.30.  We have accommodated a change in CLISP's handling \u000A of streams and its package-locking mechanism, as well as certain non-standard \u000A characters that formerly could cause CLISP 2.30 to break, even when those \u000A characters are in comments.</p> \u000A \u000A <p>Eliminated compiler warnings for CMU Lisp.</p> \u000A \u000A <p>Fixed an incorrect error supplied when book certification proceeded so \u000A quickly that the file write dates of the book (<v>.lisp</v> file) and the \u000A corresponding compiled file are equal.  Now that error only occurs if the \u000A compiled file has a strictly earlier write date, which probably should never \u000A happen.</p> \u000A \u000A <p>Fixed an infinite loop when executing <v>make clean-books</v> (and hence \u000A `make' with targets that call <v>clean-books</v>, namely, \u000A <v>certify-books-fresh</v>, <v>regression-fresh</v>, and \u000A <v>regression-nonstd-fresh</v>), which could occur when any subdirectories of \u000A <v>books/</v> are missing &mdash; even <v>workshops/</v>, which is intended to be \u000A optional.  Thanks to Pete Manolios for pointing out this bug.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> command now works properly even when filenames, or \u000A their directories or parent directories (etc.) are links.  Thanks to Matt \u000A Wilding for pointing out this problem.</p> \u000A \u000A <p>The commands <v>:</v><tt><see topic=\"ACL2____PUFF\">puff</see></tt> <v>:</v><tt><see topic=\"ACL2____PUFF_A2\">puff*</see></tt> have been fixed. \u000A Formerly, there was a bug when <v>:puff</v> or <v>:puff*</v> caused the execution \u000A of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> for an absolute <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>, <v>P</v>, that was \u000A other than the current connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>).  When \u000A including <v>P</v>, any subsidiary <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with a relative pathname \u000A would be erroneously considered relative to the current <tt><see topic=\"ACL2____CBD\">cbd</see></tt> rather \u000A than relative to the directory of <v>P</v>.  Thanks to Pete Manolios and Matt \u000A Wilding for pointing out this problem.</p> \u000A \u000A <p>It had been possible in a ``large'' ACL2 image to call <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> successfully on built-in function <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>, with \u000A undesirable results.  This hole has been plugged.  Thanks to Rob Sumners for \u000A pointing out this problem.  The new function <tt><see topic=\"ACL2____GC_42\">gc$</see></tt> must also stay in \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.</p> \u000A \u000A <p>ACL2 no longer warns when certifying a book based on <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A functions whose <see topic=\"ACL2____GUARD\">guard</see>s have not yet been verified.  Thanks to Pete \u000A Manolios for pointing out this issue.</p> \u000A \u000A <p>An occasional ``slow array warning'' had been possible during proofs.  The \u000A following sequence shows how to evoke that warning in previous versions.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> binary-append))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> binary-append))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> binary-append))\u000A(<see topic=\"ACL2____UBT\">ubt</see> 2)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) x))</code> \u000A \u000A <p>(See <see topic=\"ACL2____NOTE-2-7-OTHER\">note-2-7-other</see> for a discussion of a change to <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> in support of this fix; however, users should not need to read that \u000A discussion.)</p> \u000A \u000A <p>The raw Lisp code for <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> had a small bug, which was only \u000A evidenced in CLISP implementations as far as we know.  It has been fixed.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____LD\">ld</see></tt> is applied to a stringp file name, it now temporarily sets \u000A the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>) to the directory of that file \u000A while evaluating forms in that file.  To see the effect of this change, \u000A imagine a subdirectory <v>&quot;sub&quot;</v> of the current directory, and imagine \u000A executing <v>(<see topic=\"ACL2____LD\">ld</see> &quot;sub/foo.lisp&quot;)</v>, where file <v>foo.lisp</v> contains the \u000A form <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;bar&quot;)</v>.  Presumably the intention was to consider \u000A the file <v>bar.lisp</v> in the same directory, <v>sub/</v>, as <v>foo.lisp</v>. \u000A <v>Ld</v> now honors that intention, but in previous versions <v>&quot;bar.lisp&quot;</v> \u000A would have been a reference to a file in the current directory, not in \u000A <v>sub/</v>.</p> \u000A \u000A <p>For users of <v>run-acl2</v> [perhaps there are none!]: A fix has been \u000A provided by a Debian user via Camm Maguire so that acl2-mode anyone using \u000A that?] will work in Xemacs, which apparently uses variable \u000A <v>lisp-mode-shared-map</v> rather than <v>shared-lisp-mode-map</v>.</p> \u000A \u000A <p>ACL2 has, for a long time (always?), had a mechanism for avoiding \u000A re-proving <see topic=\"ACL2____CONSTRAINT\">constraint</see>s generated by <v>:functional-instance</v> <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>s in <v>:use</v> and <v>:by</v> hints.  But this mechanism had not \u000A applied to defined (as opposed to constrained) functions.  This has been \u000A fixed.  Thanks to Francisco J. Martin-Mateos (ChesKo) for pointing out this \u000A problem by sending a clear example.</p>"],

 "ACL2____MAKE-EVENT-DETAILS":[["Make-event"],"ACL2 Sources","ACL2","<p>The normal user of <v>make-event</v> can probably ignore this \u000A section, but we include it for completeness.  We assume that the reader has \u000A read and understood the basic documentation for <v>make-event</v> (see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>), but we begin below with a summary of expansion.</p> \u000A \u000A <p><b>Introduction</b></p> \u000A \u000A <p>Here is a summary of how we handle expansion involving <v>make-event</v> \u000A forms.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion nil)</v></p> \u000A \u000A <p>This shows the <v>:check-expansion</v> default of <v>nil</v>, and is typical \u000A user input.  We compute the expansion <v>exp</v> of <v>form</v>, which is the \u000A expansion of the original <v>make-event</v> expression and is evaluated in place \u000A of that expression.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion t)</v></p> \u000A \u000A <p>The user presumably wants it checked that the expansion doesn't change in \u000A the future, in particular during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  If the expansion of \u000A <v>form</v> is <v>exp</v>, then we will evaluate <v>exp</v> to obtain the value as \u000A before, but this time we record that the expansion of the original \u000A <v>make-event</v> expression is <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion exp)</v> \u000A rather than simply <v>exp</v>.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion exp) ; exp a cons</v></p> \u000A \u000A <p>This is generated for the case that <v>:check-expansion</v> is <v>t</v>, as \u000A explained above.  Evaluation is handled as described in that above case, \u000A except here we check that the expansion result is the given <v>exp</v>. \u000A (Actually, the user is also allowed supply such a form.)  The original \u000A <v>make-event</v> expression does not undergo any expansion (intuitively, it \u000A expands to itself).</p> \u000A \u000A <p>Now let us take a look at how we expand <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> forms (<tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is handled similarly).</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> ... (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion nil) ...)</v></p> \u000A \u000A <p>The expansion is obtained by replacing the <v>make-event</v> form as follows. \u000A Let <v>exp</v> be the expansion of <v>form</v>, Then replace the above \u000A <v>make-event</v> form, which we denote as <v>F</v>, by <v>(record-expansion F\u000A exp)</v>.  Here, <v>record-expansion</v> is a macro that returns its second \u000A argument.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> ... (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion t) ...)</v></p> \u000A \u000A <p>The expansion is of the form <v>(record-expansion F exp)</v> as in the \u000A <v>nil</v> case above, except that this time <v>exp</v> is <tt>(make-event form \u000A :check-expansion exp')</tt>, where <v>exp'</v> is the expansion of \u000A <v>form</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> ... (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion exp) ...) ; exp a\u000A cons</v></p> \u000A \u000A <p>No expansion takes place unless expansion takes place for at least one of \u000A the other subforms of the <v>progn</v>, in which case each such form <v>F</v> is \u000A replaced by <v>(record-expansion F exp)</v> where <v>exp</v> is the expansion of \u000A <v>F</v>.</p> \u000A \u000A <p><b>Detailed semantics</b></p> \u000A \u000A <p>In our explanation of the semantics of <v>make-event</v>, we assume \u000A familiarity with the notion of ``embedded event form'' (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).</p> \u000A \u000A <p>Let's say that the ``actual embedded event form'' corresponding to a given \u000A form is the underlying call of an ACL2 event: that is, <tt><see topic=\"ACL2____LOCAL\">local</see></tt>s are \u000A dropped when <v>ld-skip-proofsp</v> is <v>'include-book</v>, and macros are \u000A expanded away, thus leaving us with a <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, a <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, or \u000A an event form (possibly <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>), any of which might have \u000A surrounding <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt>, or <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> \u000A calls.</p> \u000A \u000A <p>Thus, such an actual embedded event form can be viewed as having the form \u000A <v>(rebuild-expansion wrappers base-form)</v> where <v>base-form</v> is a \u000A <v>progn</v>, a <v>make-event</v>, or an event form (possibly <v>encapsulate</v>), \u000A and <v>wrappers</v> are (as in ACL2 source function <v>destructure-expansion</v>) \u000A the result of successively removing the event form from the result of \u000A macroexpansion, leaving a sequence of <v>(<see topic=\"ACL2____LOCAL\">local</see>)</v>, <v>(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>)</v>, and \u000A <v>(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> ...)</v> forms.  In this case we say that the form \u000A ``destructures into'' the indicated <v>wrappers</v> and <v>base-form</v>, and that \u000A it can be ``rebuilt from'' those <v>wrappers</v> and <v>base-form</v>.</p> \u000A \u000A <p>Elsewhere we define the notion of the ``expansion result'' from an \u000A evaluation (see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>), and we mention that when expansion \u000A concludes, the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and most of the <v>state</v> are \u000A restored to their pre-expansion values.  Specifically, after evaluation of the \u000A argument of <v>make-event</v> (even if it is aborted), the ACL2 logical world is \u000A restored to its pre-evaluation value, as are all state global variables in the \u000A list <v>*protected-system-state-globals*</v>.  Thus, assignments to user-defined \u000A state globals (see <see topic=\"ACL2____ASSIGN\">assign</see>) do persist after expansion, since they are \u000A not in that list.</p> \u000A \u000A <p>We recursively define the combination of evaluation and expansion of an \u000A embedded event form, as follows.  We also simultaneously define the notion of \u000A ``expansion takes place,'' which is assumed to propagate upward (in a sense \u000A that will be obvious), such that if no expansion takes place, then the \u000A expansion of the given form is considered to be itself.  It is useful to keep \u000A in mind a goal that we will consider later: Every <v>make-event</v> subterm of \u000A an expansion result has a <v>:check-expansion</v> field that is a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>, \u000A where for this purpose <v>make-event</v> is viewed as a macro that returns its \u000A <v>:check-expansion</v> field.  (Implementation note: The latest expansion of a \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is \u000A stored in state global <v>'last-make-event-expansion</v>, except that if no \u000A expansion has taken place for that form then <v>'last-make-event-expansion</v> \u000A has value <v>nil</v>.)</p> \u000A \u000A <blockquote> \u000A \u000A <p>If the given form is not an embedded event form, then simply cause a soft \u000A error, <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> where <v>erp</v> is not <v>nil</v>.  Otherwise:</p> \u000A \u000A <p>If the evaluation of the given form does not take place (presumably because \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> events are being skipped), then no expansion takes place. \u000A Otherwise:</p> \u000A \u000A <p>Let <v>x</v> be the actual embedded event form corresponding to the given \u000A form, which destructures into wrappers <v>W</v> and base-form <v>B</v>.  Then the \u000A original form is evaluated by evaluating <v>x</v>, and its expansion is as \u000A follows.</p> \u000A \u000A <p>If <v>B</v> is <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> form :check-expansion val)</v>, then expansion \u000A takes place if and only if <v>val</v> is not a <v>consp</v> and no error occurs, \u000A as now described.  Let <v>R</v> be the expansion result from protected \u000A evaluation of <v>form</v>, if there is no error.  <v>R</v> must be an embedded \u000A event form, or it is an error.  Then evaluate/expand <v>R</v>, where if <v>val</v> \u000A is not <v>nil</v> then state global <v>'ld-skip-proofsp</v> is initialized to \u000A <v>nil</v>.  (This initialization is important so that subsequent expansions are \u000A checked in a corresponding environment, i.e., where proofs are turned on in \u000A both the original and subsquent environments.)  It is an error if this \u000A evaluation causes an error.  Otherwise, the evaluation yields a value, which \u000A is the result of evaluation of the original <v>make-event</v> expression, as \u000A well as an expansion, <v>E_R</v>.  Let <v>E</v> be rebuilt from <v>W</v> and \u000A <v>E_R</v>.  The expansion of the original form is <v>E</v> if <v>val</v> is \u000A <v>nil</v>, and otherwise is the result of replacing the original form's \u000A <v>:check-expansion</v> field with <v>E</v>, with the added requirement that if \u000A <v>val</v> is not <v>t</v> (thus, a <v>consp</v>) then <v>E</v> must equal <v>val</v> or \u000A else we cause an error.</p> \u000A \u000A <p>If <v>B</v> is either <v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> form1 form2 ...)</v> or <v>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> sigs\u000A form1 form2 ...)</v>, then after evaluating <v>B</v>, the expansion of the \u000A original form is the result of rebuilding from <v>B</v>, with wrappers <v>W</v>, \u000A after replacing each <v>formi</v> in <v>B</v> for which expansion takes place by \u000A <tt>(record-expansion formi formi')</tt>, where <v>formi'</v> is the expansion \u000A of <v>formi</v>.  Note that these expansions are determined as the <v>formi</v> \u000A are evaluated in sequence (where in the case of <v>encapsulate</v>, this \u000A determination occurs only during the first pass).  Except, if no expansion \u000A takes place for any <v>formi</v>, then the expansion of the original form is \u000A itself.</p> \u000A \u000A <p>Otherwise, the expansion of the original form is itself.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Similarly to the <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> cases above, book \u000A certification causes a book to be replaced by its so-called ``book \u000A expansion.''  There, each event <v>ev</v> for which expansion took place during \u000A the proof pass of certification &mdash; say, producing <v>ev'</v> &mdash; is \u000A replaced by <tt>(record-expansion ev ev')</tt>.</p> \u000A \u000A <p>Implementation Note.  The book expansion is actually implemented by way of \u000A the <v>:expansion-alist</v> field of its <see topic=\"ACL2____CERTIFICATE\">certificate</see>, which associates \u000A 0-based positions of top-level forms in the book (not including the initial \u000A <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> form) with their expansions.  Thus, the book's source file \u000A is not overwritten; rather, the certificate's expansion-alist is applied when \u000A the book is included or compiled.  End of Implementation Note.</p> \u000A \u000A <p>It is straightforward by computational induction to see that for any \u000A expansion of an embedded event form, every <v>make-event</v> sub-event has a \u000A <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> <v>:check-expansion</v> field.  Here, by ``sub-event'' we mean to \u000A expand macros; and we also mean to traverse <v>progn</v> and <v>encapsulate</v> \u000A forms as well as <v>:check-expansion</v> fields of <v>make-event</v> forms.  Thus, \u000A we will only see <v>make-event</v> forms with <v>consp</v> <v>:check-expansion</v> \u000A fields in the course of <v>include-book</v> forms, the second pass of \u000A <v>encapsulate</v> forms, and raw Lisp.  This fact guarantees that an event form \u000A will always be treated as its original expansion.</p> \u000A \u000A <p><b>Notes on ttags</b></p> \u000A \u000A <p>See <see topic=\"ACL2____DEFTTAG\">defttag</see> for documentation of the notion of ``trust tag'' \u000A (``ttag'').  We note here that even if an event <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> tag-name)</v> for \u000A non-<v>nil</v> <v>tag-name</v> is admitted only during the expansion phase of a \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> form, then such expansion will nevertheless still cause \u000A <v>tag-name</v> to be recorded in the logical <see topic=\"ACL2____WORLD\">world</see> (assuming that the \u000A <v>make-event</v> form is admitted).  So in order to certify such a book, a \u000A suitable <v>:ttags</v> argument must be supplied; see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>.</p> \u000A \u000A <p>ACL2 does provide a way to avoid the need for <v>:ttags</v> arguments in such \u000A cases.  The idea is to certify a book twice, where the results of \u000A <v>make-event</v> expansion are saved from the first call of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and provided to the second call.  See <see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-ACL2x</see>.</p> \u000A \u000A <p>Finally, we discuss a very unusual case where certification does not \u000A involve trust tags but a subsequent <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> does involve trust \u000A tags: a <v>make-event</v> call specifying <v>:check-expansion t</v>, whose \u000A expansion generates a <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> event during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> but \u000A not <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  Consider the following book.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____ER-PROGN\">er-progn</see>\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2_____04\">@</see> skip-notify-on-defttag) ; non-nil when including a certified book\u000A      (<see topic=\"ACL2____PPROGN\">pprogn</see>\u000A       (<see topic=\"ACL2____FMS\">fms</see> &quot;Value of (<see topic=\"ACL2_____04\">@</see> skip-notify-on-defttag): ~x0~|&quot;\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0 (<see topic=\"ACL2_____04\">@</see> skip-notify-on-defttag)))\u000A            *standard-co* state nil)\u000A       (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A        ()\u000A        (<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)\u000A        (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> &quot;Imagine something bad here!&quot;)))\u000A    (value nil))\u000A  (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :some-value)))\u000A :check-expansion t)</code> \u000A \u000A <p>This book certifies successfully without the need for a <v>:ttags</v> \u000A argument for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  Indeed, the above book's <see topic=\"ACL2____CERTIFICATE\">certificate</see> does not specify <v>:foo</v> as a trust tag associated with the \u000A book, because no <v>defttag</v> event was executed during book certification. \u000A Unfortunately, if we try to include this book without specifying a value of \u000A <v>:ttags</v> that allows <v>:foo</v>, book inclusion will cause executing of the \u000A above <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt>.  If that inclusion happens in the context of certifying \u000A some superior book and the appropriate <v>:ttags</v> arguments have not been \u000A provided, that certification will fail.</p>"],

 "ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>ACL2 is implemented in Common Lisp.  There are many different \u000A Common Lisps and they differ in details relating to interacting with the \u000A system.  We sometimes refer to the host Common Lisp as ``raw Lisp.''  The new \u000A user is advised not to operate in raw Lisp as it is possible to, say, redefine \u000A ACL2 system faclities like <v>defthm</v>.</p> \u000A \u000A <p>Most people use Emacs (see <see topic=\"ACL2____EMACS\">Emacs</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) or \u000A the ACL2 Sedan (Eclipse) interface (see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-Sedan</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A They provide protection against certain common mistakes, e.g., trying to edit \u000A a block of input text after the operating system has buffered it up and sent \u000A it to the Lisp reader which is parsing it as you type.  More on this below. \u000A In addition, the Sedan provides helpful syntax checking and a disciplined \u000A approach to the stack of lemmas generated by The Method.  But the variety of \u000A interfaces to the variety of Lisps mean that there is great variation in what \u000A one must type to interact with ACL2.</p> \u000A \u000A <p>The best example is perhaps trying to interrupt a running proof.  If your \u000A host Common Lisp is GCL or Allegro and you are typing directly to the Common \u000A Lisp process, then you can interrupt a computation by typing ``ctrl-c'' (hold \u000A down the Control key and hit the ``c'' key once).  But other Lisps may require \u000A some other control sequence.  If you are typing to an Emacs process which is \u000A running the GCL or Allegro Common Lisp process in a shell buffer, you should \u000A type ctrl-c ctrl-c.  If you are running the ACL2 Sedan, you can use the \u000A <i>Interrupt Session</i> button on the tool bar.  The environment you enter \u000A when you interrupt depends on various factors and basically you should \u000A endeavor to get back to the top level ACL2 command loop, perhaps by typing \u000A some kind of Lisp depenent ``abort'' command like <v>A</v> or <v>:q</v>, or \u000A <v>:abort!</v>.  You can usually determine what environment you're in by paying \u000A attention to the prompt, which we discuss below.</p> \u000A \u000A <p>The ACL2 ``interactive loop'' is called <tt><see topic=\"ACL2____LP\">lp</see></tt> (<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A and is generally invoked automatically from your Common Lisp when you start up \u000A the ACL2 process.  <v>LP</v> is just a special case of an ACL2 function called \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, which the user can call from within the ACL2 \u000A interactive loop to enter the loop recursively.  New users don't have to know \u000A this except that it helps explain why some commands have the string \u000A ``<v>-ld-</v>'' in their names!</p> \u000A \u000A <p>ACL2 presents itself as a ``read-eval-print'' loop: you're repeatedly \u000A prompted for some type-in, which is read, evaluated, and may cause some \u000A printing.  The prompt tells you something about ACL2's state.  In the standard \u000A environment, the prompt is</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>The ``<v>ACL2</v>'' tells you that the symbols you use in your command are \u000A those defined in the standard ACL2 namespace (or, in the jargon of Lisp, the \u000A ``current package,'' see <see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A You could create a new namespace (see <see topic=\"ACL2____DEFPKG\">defpkg</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A and set the current package to it (see <see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A The next part of the prompt above \u000A (``<v>!</v>''), the exclamation mark) tells you that before ACL2 evaluates your \u000A type-in it will check to see whether <tt><see topic=\"ACL2____GUARD\">guard</see></tt>s (<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A are respected, i.e., whether the functions used in your command are being \u000A supplied arguments in their ``expected domains.''  If evaluation is allowed by \u000A the guards, it proceeds exactly according to the ACL2 axioms; if evaluation is \u000A not allowed, an error is signaled.  ACL2 event commands check their arguments \u000A thoroughly at run-time, regardless of Lisp's notion of ``expected \u000A domains.''</p> \u000A \u000A <p>If the exclamation mark is missing from the prompt,</p> \u000A \u000A <code>ACL2 &gt;</code> \u000A \u000A <p>then evaluation occurs strictly according to the ACL2 axioms, without \u000A regard for any declared guards.</p> \u000A \u000A <p>You can switch between these two prompts by typing</p> \u000A \u000A <code>ACL2 !&gt;:set-guard-checking nil</code> \u000A \u000A <p>to turn guard checking off and</p> \u000A \u000A <code>ACL2 &gt;:set-guard-checking t</code> \u000A \u000A <p>to turn it on.  Try typing <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 7)</v> to each prompt.</p> \u000A \u000A <p>If there is a ``<v>p</v>'' in the prompt,</p> \u000A \u000A <code>ACL2 p!&gt;</code> \u000A \u000A <p>with or without the exclamation mark:</p> \u000A \u000A <code>ACL2 p&gt;</code> \u000A \u000A <p>it means you are in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> (<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A mode rather than <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> (<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A mode.  In <v>:program</v> mode, <v>defun</v> just defines Common Lisp programs \u000A that you can evaluation but it adds no axioms and you cannot use such defined \u000A functions in theorems or invoke <v>defthm</v>.  <v>:Program</v> mode is often used \u000A to prototype a model.</p> \u000A \u000A <p>Most commands are just typical parenthesized Lisp expressions, like</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> rev-rev\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev x)) x)))</code> \u000A \u000A <p>but some are typed as keywords followed by a certain number of \u000A arguments.</p> \u000A \u000A <p>For example, to undo the last event you may type</p> \u000A \u000A <code>ACL2 !&gt;:u</code> \u000A \u000A <p>or to undo back through the introduction of <v>rev</v> you may type</p> \u000A \u000A <code>ACL2 !&gt;:ubt rev</code> \u000A \u000A <p>The first is equivalent to evaluating <v>(<see topic=\"ACL2____U\">u</see>)</v> and the second is equivalent \u000A to evaluating <v>(<see topic=\"ACL2____UBT\">ubt</see> 'rev)</v>.  See <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A So if you see a sentence like ``to turn on the break rewrite facility, execute \u000A <v>:brr t</v>,'' we mean type</p> \u000A \u000A <code>ACL2 !&gt;:brr t</code> \u000A \u000A <p>or equivalently</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____BRR\">brr</see> t)</code> \u000A \u000A <p>If you see a prompt that doesn't look like those above you are probably not \u000A typing commands to the standard ACL2 read-eval-print loop!  If you've somehow \u000A called <v>LD</v> recursively, the prompt ``gets deeper,'' e.g.,</p> \u000A \u000A <code>ACL2 !&gt;&gt;</code> \u000A \u000A <p>and you can pop out one level with <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A (for ``quit'') or pop to the outermost ACL2 loop with <v>:</v><v>abort!</v> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A If you are in the outermost call of the ACL2 interactive loop and you type \u000A <v>:q</v>, you pop out into raw lisp.  The prompt there is generally different \u000A from the ACL2 prompt but that is outside our our control and varies from Lisp \u000A to Lisp.  We have arranged for many (but not all) Lisps to use a raw lisp \u000A prompt involving the string <v>&quot;[RAW LISP]&quot;</v>.  To get back into the ACL2 \u000A interactive loop from raw lisp, evaluate <v>(<see topic=\"ACL2____LP\">LP</see>)</v>.</p> \u000A \u000A <p>If you see a prompt that looks like an ACL2 prompt but has a number in \u000A front of it, e.g.,</p> \u000A \u000A <code>1 ACL2 &gt;</code> \u000A \u000A <p>then you're talking to the break rewrite facility (and you are 1 level deep \u000A in the example above).  Presumably at earlier time in this session you enabled \u000A that facility, with <v>:brr t</v>, installed a monitor on some rule, invoked the \u000A prover, entered the break, and forgot.  Everything you have done (e.g., lemmas \u000A you might have proved with <v>defthm</v>) inside that break will be lost when \u000A you exit the break.</p> \u000A \u000A <p>Since the break rewrite facility is ``ours'' we can tell you how to exit \u000A it!  To exit our breaks and return to the top-level ACL2 loop, execute \u000A <v>:abort!</v>.</p> \u000A \u000A <p>If you discover you've been working in a <v>brr</v> break, exit, turn off the \u000A break facility wih <v>:brr nil</v>, and redo whatever <v>defun</v>s and \u000A <v>defthm</v>s you did while in that break.</p> \u000A \u000A <p>Users of the Emacs interface may occasionally type commands directly in the \u000A <v>*shell*</v> buffer running ACL2.  This can be ``dangerous'' for two reasons. \u000A One is that if you type an event, like a <v>defun</v> or <v>defthm</v>, directly \u000A to the shell, it will not be recorded in your ``script'' buffer and you may \u000A forget it in your final script.  The other is that if you attempt to edit a \u000A multi-line command on any but the most recent line, e.g., to correct the \u000A spelling of <v>defthm</v> below after you've typed the ``<v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see>\u000A x)</v>'' you will confuse the Lisp parser because it has already read \u000A ``<v>(defth rev-rev</v>''.</p> \u000A \u000A <code>ACL2 !&gt;(defth rev-rev\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)</code> \u000A \u000A <p>This usually provokes the raw Lisp to enter a low level error break from \u000A which you must abort, possibly reenter the ACL2 loop, and re-type the \u000A corrected command from scratch.</p> \u000A \u000A <p>Another common mistake when using interfaces other than the ACL2 Sedan is \u000A to type an ill-formed ACL2 expression containing dots or commas, which also \u000A often provokes a break into the raw Lisp's error handler.</p> \u000A \u000A <p>The fundamental lesson is that you should pay attention to the prompt and \u000A learn what the different prompts mean -- or use the ACL2 Sedan.</p> \u000A \u000A <p>If you have been working your way through the tutorial introduction to the \u000A theorem prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____DEFPROXY":[["Events"],"ACL2 Sources","ACL2","<p>This event is provided for those who want to experiment with <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> using <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions, and without proof \u000A obligations or constraints on cycles in the extended ancestors graph; see \u000A <see topic=\"ACL2____DEFATTACH\">defattach</see>.  If you merely want to define a stub or a non-executable \u000A function, see <see topic=\"ACL2____DEFSTUB\">defstub</see> or see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>, respectively.</p> \u000A \u000A <p>See community book <v>books/misc/defproxy-test.lisp</v> for an extended (but \u000A simple) example.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____DEFPROXY\">defproxy</see> subr1 (<see topic=\"COMMON-LISP_____A2\">*</see> *) =&gt; *)\u000A(<see topic=\"ACL2____DEFPROXY\">defproxy</see> add-hash (<see topic=\"COMMON-LISP_____A2\">*</see> * hashtable) =&gt; (<see topic=\"ACL2____MV\">mv</see> * hashtable))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFPROXY\">defproxy</see> name args-sig =&gt; output-sig)</code> \u000A \u000A <p>where <v>name</v> is a new function symbol and <v>(<see topic=\"ACL2____NAME\">name</see> . args-sig) =&gt;\u000A output-sig)</v> is a signature; see <see topic=\"ACL2____SIGNATURE\">signature</see>.</p> \u000A \u000A <p>The macro <v>defproxy</v> provides a convenient way to introduce a ``proxy'': \u000A a <v>:program</v> mode function that can be given attachments for execution (see \u000A <see topic=\"ACL2____DEFATTACH\">defattach</see>), assuming that there is an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  Thus, a <v>defproxy</v> calls expands to a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form with \u000A the following <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form: <v>:non-executable\u000A :program</v>.  Note that <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> is not permitted for such a \u000A function.  However, it is permitted to put the proxy function into \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode by use of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event; indeed, this \u000A is the way to ``upgrade'' an attachment so that the normal checks are \u000A performed and no trust tag is necessary.</p> \u000A \u000A <p>In order to take advantage of a <tt><see topic=\"ACL2____DEFPROXY\">defproxy</see></tt> form, one provides a \u000A subsequent <v>defattach</v> form to attach an executable function to the \u000A <v>defproxy</v>-introduced function.  When <v>:skip-checks t</v> is provided in a \u000A <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> form, the usual checks for <v>defattach</v> <see topic=\"ACL2____EVENTS\">events</see> are \u000A skipped, including proof obligations and the check that the extended ancestor \u000A relation has no cycles (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  There must be an active trust \u000A tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) in order to use <v>:skip-checks t</v>.  In that case \u000A the use of <v>:skip-checks t</v> is permitted; but note that its use is in fact \u000A required if a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function is involved, and even if a \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function is involved that has not been <see topic=\"ACL2____GUARD\">guard</see>-verified.</p> \u000A \u000A <p>The following log shows a simple use of defproxy.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFPROXY\">defproxy</see> foo-stub (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)\u000A\u000ASummary\u000AForm:  ( DEFUN FOO-STUB ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FOO-STUB\u000AACL2 !&gt;(foo-stub '(3 4 5))\u000A\u000AACL2 Error in TOP-LEVEL:  ACL2 cannot ev the call of undefined function\u000AFOO-STUB on argument list:\u000A\u000A((3 4 5))\u000A\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo-impl (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program\u000A                         :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))))\u000A         (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000ASummary\u000AForm:  ( DEFUN FOO-IMPL ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO-IMPL\u000AACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A\u000ATTAG NOTE: Adding ttag :T from the top level loop.\u000A T\u000AACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> (foo-stub foo-impl) :skip-checks t)\u000A\u000ASummary\u000AForm:  ( DEFATTACH (FOO-STUB FOO-IMPL) ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A :ATTACHMENTS-RECORDED\u000AACL2 !&gt;(foo-stub '(3 4 5))\u000A3\u000AACL2 !&gt;</code> \u000A \u000A <p>One can replace this attachment with one that uses <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A functions and does not skip checks.  The idea is to reintroduce the proxy \u000A function using an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form, which does not require \u000A redefinition (see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>) to be enabled, and either to \u000A put the attachment into <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode with the <see topic=\"ACL2____GUARD\">guard</see> \u000A verified, as we do in the example below, or else to attach to a different \u000A <see topic=\"ACL2____GUARD\">guard</see>-verified <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> (foo-stub nil) :skip-checks t) ; remove attachment\u000A\u000ASummary\u000AForm:  ( DEFATTACH (FOO-STUB NIL) ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A :ATTACHMENTS-RECORDED\u000AACL2 !&gt;(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A        ((foo-stub (x) t :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo-stub (x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A        (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-stub-reduces-acl2-count\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                   (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (foo-stub x))\u000A                      (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)))))\u000A\u000A[[ ... output omitted here ... ]]\u000A\u000AThe following constraint is associated with the function FOO-STUB:\u000A\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (FOO-STUB X)) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X)))\u000A\u000ASummary\u000AForm:  ( ENCAPSULATE ((FOO-STUB ...) ...) ...)\u000ARules: NIL\u000AWarnings:  Non-rec\u000ATime:  0.02 seconds (prove: 0.01, print: 0.00, other: 0.01)\u000A T\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> foo-impl)\u000A\u000ASince FOO-IMPL is non-recursive, its admission is trivial.  We could\u000Adeduce no constraints on the type of FOO-IMPL.\u000A\u000AComputing the guard conjecture for FOO-IMPL....\u000A\u000AThe guard conjecture for FOO-IMPL is trivial to prove.  FOO-IMPL is\u000Acompliant with Common Lisp.\u000A\u000ASummary\u000AForm:  ( DEFUN FOO-IMPL ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000ASummary\u000AForm:  ( MAKE-EVENT (VERIFY-TERMINATION-FN ...))\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FOO-IMPL\u000AACL2 !&gt;(<see topic=\"ACL2____DEFTTAG\">defttag</see> nil) ; optional\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> (foo-stub foo-impl))\u000A\u000AThe guard proof obligation is\u000A\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> X)\u000A         (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQ\">EQ</see> X NIL))).\u000A\u000ABut we reduce the conjecture to T, by primitive type reasoning.\u000A\u000AQ.E.D.\u000A\u000AThis concludes the guard proof.\u000A\u000AWe now prove that the attachment satisfies the required constraint.\u000AThe goal to prove is\u000A\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (FOO-IMPL X))\u000A            (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X))).\u000A\u000A[[ ... output omitted here ... ]]\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( DEFATTACH (FOO-STUB FOO-IMPL))\u000ARules: ((:DEFINITION ACL2-COUNT)\u000A        (:DEFINITION FOO-IMPL)\u000A        (:ELIM CAR-CDR-ELIM)\u000A        (:FAKE-RUNE-FOR-LINEAR NIL)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:REWRITE CAR-CONS)\u000A        (:REWRITE CDR-CONS)\u000A        (:TYPE-PRESCRIPTION ACL2-COUNT))\u000ATime:  0.02 seconds (prove: 0.01, print: 0.01, other: 0.00)\u000A :ATTACHMENTS-RECORDED\u000AACL2 !&gt;</code> \u000A \u000A <p>We close with some remarks on the checking of <see topic=\"ACL2____GUARD\">guard</see>s in the case \u000A that <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> has been called with keyword argument <v>:skip-checks\u000A t</v>.  We illustrate with examples, where we assume an attachment pair <v>(f\u000A . g)</v> created by an event <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> ... (f g) ... :skip-checks t ...)</v>. \u000A A good model for the treatment of <v>:skip-checks t</v> is dependent on whether \u000A <v>f</v> was introduced with <v>defproxy</v> or with <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>: for \u000A <v>defproxy</v>, the normal guard-related checks are treated as skipped, while \u000A for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, they are assumed to hold.</p> \u000A \u000A <p>First suppose that <v>f</v> was introduced using <v>defproxy</v>, and consider \u000A the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPROXY\">defproxy</see> f (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)) ; not guard-verified; implicit guard of t is too weak\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; trust tag needed for :skip-checks t\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g) :skip-checks t)</code> \u000A \u000A <p>If we try to evaluate the form <v>(f 3)</v> in ACL2, then the top-level \u000A so-called ``executable counterpart'' (i.e., the logically-defined funcction, \u000A also known as the ``*1*'' function) of <v>f</v> is invoked.  It calls the \u000A executable counterpart of <v>g</v>, which calls the executable counterpart of \u000A <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>, which in turn checks the <see topic=\"ACL2____GUARD\">guard</see> of <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and causes a \u000A guard violation error (unless we first turn off guard-checking; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> f g)\u000A ((F) (G))\u000AACL2 !&gt;(f 3)\u000A1&gt; (ACL2_*1*_ACL2::F 3)\u000A  2&gt; (ACL2_*1*_ACL2::G 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (<see topic=\"COMMON-LISP____CAR\">CAR</see> X),\u000Awhich is (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL)), is violated by the arguments\u000Ain the call (<see topic=\"COMMON-LISP____CAR\">CAR</see> 3).  To debug see :DOC print-gv, see :DOC trace, and\u000Asee :DOC wet.  See :DOC set-guard-checking for information about suppressing\u000Athis check with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new\u000Ausers.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Little changes if we modify the example above by strengtheing the guard of \u000A <v>g</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPROXY\">defproxy</see> f (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; *)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; trust tag needed for :skip-checks t\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g) :skip-checks t)</code> \u000A \u000A <p>The result of evaluating <v>(f 3)</v> is as before, except that this time the \u000A guard violation occurs at the time that <v>g</v> is called.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> f g)\u000A ((F) (G))\u000AACL2 !&gt;(f 3)\u000A1&gt; (ACL2_*1*_ACL2::F 3)\u000A  2&gt; (ACL2_*1*_ACL2::G 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (G X), which\u000Ais (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X), is violated by the arguments in the call (G 3).  To debug\u000Asee :DOC print-gv, see :DOC trace, and see :DOC wet.  See :DOC set-\u000Aguard-checking for information about suppressing this check with (set-\u000Aguard-checking :none), as recommended for new users.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Now consider a slight variation of the example just above, in which <v>f</v> \u000A is introduced using <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> instead of using <v>defproxy</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ( ((f *) =&gt; *) )\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; trust tag needed for :skip-checks t\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (f g) :skip-checks t)</code> \u000A \u000A <p>Since <v>f</v> was introduced by <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> instead of by \u000A <v>defproxy</v>, ACL2 assumes that the usual guard properties hold.  In \u000A particular, it assumes that (informally speaking) the guard of <v>f</v> implies \u000A the guard of <v>g</v>; see <see topic=\"ACL2____DEFATTACH\">defattach</see> for details.  So in this case, ACL2 \u000A proceeds under that assumption even though it's actually false, and the result \u000A is a raw Lisp error.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> f g)\u000A ((F) (G))\u000AACL2 !&gt;(f 3)\u000A1&gt; (ACL2_*1*_ACL2::F 3)\u000A  2&gt; (G 3)\u000A\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  Attempt to take the car of 3 which is not listp.\u000A***********************************************\u000A\u000AIf you didn't cause an explicit interrupt (Control-C),\u000Athen the root cause may be call of a :program mode\u000Afunction that has the wrong guard specified, or even no\u000Aguard specified (i.e., an implicit guard of t).\u000ASee :DOC guards.\u000A\u000ATo enable breaks into the debugger (also see :DOC acl2-customization):\u000A(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">SET-DEBUGGER-ENABLE</see> T)\u000AACL2 !&gt;</code> \u000A \u000A <p>If you replace <v>g</v> by its definition in the first example of this \u000A series, i.e. with a guard (implicitly) of <v>t</v>, you will see the same error, \u000A this time because the <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> event assumed that <v>g</v> was \u000A guard-verified.</p>"],

 "ACL2____CONSERVATIVITY-OF-DEFCHOOSE":[["Defchoose"],"ACL2 Sources","ACL2","<p>This documentation topic provides underlying theory.  It is of \u000A theoretical interest only; it has no relationship to the effective use of \u000A ACL2.</p> \u000A \u000A <p>The argument below for the conservativity of <see topic=\"ACL2____DEFCHOOSE\">defchoose</see> replaces the \u000A terse and somewhat misleading reference to a forcing argument in Appendix B of \u000A the paper by ACL2 authors Kaufmann and Moore, ``Structured Theory Development \u000A for a Mechanized Logic'' (Journal of Automated Reasoning 26, no. 2 (2001), \u000A pp. 161-203).</p> \u000A \u000A <p>Our basic idea is to to take a (countable) first-order structure for ACL2, \u000A M, together with a function symbol, f, introduced by <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>, and \u000A find a way to expand M with an interpretation of f (without changing the \u000A universe of M) so that e0-induction continues to hold in the expansion.  A \u000A remark at the end of this documentation topic shows why care is necessary.  A \u000A concept called ``forcing'', originally introduced by Paul Cohen for set \u000A theory, has long since been adapted by logicians (in a simplified form) to \u000A model theory.  This simplified model-theoretic forcing provides the means for \u000A making our careful expansion.</p> \u000A \u000A <p>The forcing argument presented below is intended to be completely \u000A self-contained for those familiar with basic first-order logic and ACL2.  No \u000A background in forcing (model-theoretic or otherwise) is expected, though we do \u000A expect a rudimentary background in first-order logic and familiarity with the \u000A following.</p> \u000A \u000A <p>Preliminaries.  We write s[p&lt;-p0] to denote the result of extending or \u000A modifying the assignment s by binding p to p0.  Now let A be a subset of the \u000A universe U of a first-order structure M.  A is said to be ``first-order \u000A definable with parameters'' in M if for some formula phi, variable x, and \u000A assignment s binding the free variables of phi except perhaps for x, A = {a \u000A \\in U: M |= phi[s[x&lt;-a]].  Note that we are writing ``\\in'' to denote set \u000A membership.  Finally, we indicate the end of a proof (or of a theorem \u000A statement, when the proof is omitted) with the symbol ``-|''.</p> \u000A \u000A <p>We gratefully acknowledge very helpful feedback from John Cowles, who found \u000A several errors in a draft of this note and suggested the exercises.  We also \u000A thank Ruben Gamboa for helpful feedback, and we thank Jim Schmerl for an \u000A observation that led us directly to this proof in the first place.</p> \u000A \u000A <p>We are given a consistent first-order theory T, extending the ACL2 \u000A ground-zero theory, that satisfies the e0-induction scheme.  We wish to show \u000A that the extension of T by the following arbitrary defchoose event is \u000A conservative, where g is a new function symbol.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> g &lt;bound-vars&gt; &lt;free-vars&gt; &lt;body&gt;)</code> \u000A \u000A <p>Note that by ``the extension of T'' here we mean the extension of T by not \u000A only the new defchoose axiom displayed just below, but also the addition of \u000A e0-induction axioms for formulas in the language with the new defchoose \u000A function symbol, g.</p> \u000A \u000A <code>&lt;body&gt; -&gt; (<see topic=\"COMMON-LISP____LET\">LET</see> &lt;free-vars&gt; = g(&lt;bound-vars&gt;) in &lt;body&gt;)</code> \u000A \u000A <p>By definition of conservativity, since proofs are finite, it clearly \u000A suffices to consider an arbitrary finite subset of T.  Then by the \u000A completeness, soundness, and downward Lowenheim-Skolem theorems of first-order \u000A logic, it suffices to show that an arbitrary countable model of T can be \u000A expanded (i.e., by interpreting the new symbol g without changing the universe \u000A of the model) to a model of the corresponding defchoose axiom above, in which \u000A all e0-induction axioms hold in the language of that model.</p> \u000A \u000A <p>Below, we will carry out a so-called <i>forcing</i> construction, which \u000A allows us to expand any countable model M of T to a model M[G] that satisfies \u000A e0-induction and also satisfies the above axiom generated from the above \u000A defchoose event.  The ideas in this argument are standard in model theory; no \u000A novelty is claimed here.</p> \u000A \u000A <p>Fix a countable model M of a theory T that satisfies e0-induction and \u000A extends the ACL2 ground-zero theory.  Also fix the above defchoose axiom, \u000A where g is not in the language of T.</p> \u000A \u000A <p>We start by defining a partial order P as follows.  Let Nb and Nf be the \u000A lengths of &lt;bound-vars&gt; and &lt;free-vars&gt;, respectively.  P consists \u000A of all fn in M such that the following formula is true in M.  Roughly \u000A speaking, it says that fn is a finite function witnessing the above \u000A requirement for g.</p> \u000A \u000A <code>alistp(fn) &amp;\u000Ano-duplicatesp-equal(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see>(fn)) &amp;\u000A(<see topic=\"ACL2____FORALL\">forall</see> &lt;bound-vars&gt;, &lt;free-vars&gt; .\u000A   (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see>(<see topic=\"COMMON-LISP____CONS\">cons</see>(&lt;bound-vars&gt;,&lt;free-vars&gt;), fn) -&gt;\u000A    (<see topic=\"COMMON-LISP____LENGTH\">length</see>(&lt;bound-vars&gt;) = Nb &amp;\u000A     length(&lt;free-vars&gt;)  = Nf &amp;\u000A     ((<see topic=\"ACL2____EXISTS\">exists</see> &lt;free-vars&gt; . &lt;body&gt;) -&gt; &lt;body&gt;))))</code> \u000A \u000A <p>P is ordered by subset, i.e., we say that p2 <i>extends</i> p1 if p1 is a \u000A subset (not necessarily proper) of p2 (more precisely, M |= \u000A subsetp-equal(p1,p2)).</p> \u000A \u000A <p>Remark.  The original argument in Appendix B of the aforementioned paper \u000A can essentially be salvaged, as we now show.  The key observation is that the \u000A particular choice of P is nearly irrelevant for the argument that follows \u000A below.  In particular, we can instead define P to consist of finite one-one \u000A functions with domain contained in the set of natural numbers.  More \u000A precisely, consider the following definitions.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> function-p (fn)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ALISTP\">alistp</see> fn)\u000A       (<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">no-duplicatesp-equal</see> (<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> fn))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nat-listp (l)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l)\u000A         (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A        (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                (<see topic=\"ACL2____NAT-LISTP\">nat-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nat-function-p (x)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (function-p x)\u000A       (<see topic=\"ACL2____NAT-LISTP\">nat-listp</see> (<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> x))))</code> \u000A \u000A <p>and define inverse as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> inverse (fn)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> fn)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> fn)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CDAR\">cdar</see> fn) (<see topic=\"COMMON-LISP____CAAR\">caar</see> fn))\u000A          (inverse (<see topic=\"COMMON-LISP____CDR\">cdr</see> fn)))))</code> \u000A \u000A <p>Then P may instead be defined to consist of those fn for which \u000A nat-function-p(fn) &amp; function-p(inverse(fn)).  With this alternate \u000A definition of P, the argument below then goes through virtually unchanged, and \u000A we get an expansion M[G] of M in which there is a definable enumeration of the \u000A universe.  The conservativity of defchoose then follows easily because the \u000A function being introduced can be defined explicitly using that enumeration \u000A (namely, always pick the least witness in the sense of the enumeration).</p> \u000A \u000A <p>End of Remark.</p> \u000A \u000A <p>Next we present the relevant forcing concepts from model theory.</p> \u000A \u000A <p>A <i>dense</i> subset of P is a subset D of P such that for every p \\in P, \u000A there is d \\in D such that d extends p.  A subset G of P is <i>generic</i> \u000A with respect to a collection Ds of dense subsets of P, also written ``G is \u000A Ds-generic,'' if G is closed under subset (if p2 \\in G and p2 extends p1 then \u000A p1 \\in G), G is pairwise compatible (the union-equal of any two elements of G \u000A is in G), and every set in Ds has non-empty intersection with G.</p> \u000A \u000A <p>For p \\in P, we say that a subset D of P is <i>dense beyond</i> p if for \u000A all p1 extending p there exists p2 extending p1 such that p2 \\in D.  This \u000A notion makes sense even for D not a subset of P if we treat elements of D not \u000A in P as nil.</p> \u000A \u000A <p>Proposition 1.  For any partial order P and countable collection Ds of \u000A dense subsets of P, there is a Ds-generic subset of P.</p> \u000A \u000A <p>Proof.  Let Ds = {D0,D1,D2,...}.  Define a sequence &lt;p_0,p_1,...&gt; \u000A such that for all i, p_i \\in Di and p_(i+1) extends p_i.  Let G = {p \\in P: \u000A for some i, pi extends p}.  Then G is Ds-generic. -|</p> \u000A \u000A <p>Note that P is first-order definable (with parameters) in M.  Let Df be the \u000A set of dense subsets of P that are first-order definable (with parameters) in \u000A M.  A standard argument shows there are only countably many first-order \u000A definitions with parameters in a countable model M &mdash; for example, we can \u000A Goedel number all terms and then all formulas &mdash; hence, Df is \u000A countable.</p> \u000A \u000A <p>By Proposition 1, let G be Df-generic.  Notice that for any list x of \u000A length Nb in M, the set of elements f of P for which x is in the domain of f \u000A is dense and first-order definable.  We may thus define a function g0 as \u000A follows: g0(x_1,...,x_Nb) = y if there is some element of G containing the \u000A pair ((x_1 ... x_Nb) . y).  It is easy to see that g0 is a total function on \u000A M.  Let L be the language of T and let L[g] be the union of L with a set \u000A containing a single new function symbol, g.  Let M[G] be the expansion of M to \u000A L[g] obtained by interpreting g to be g0 (see also Proposition 5 below).</p> \u000A \u000A <p>So now we have fixed M, P, Df, G, and g0, where G is Df-generic.</p> \u000A \u000A <p>Proposition 2.  Let Df be the set of dense subsets of P that are \u000A first-order definable (with parameters) in M.  Suppose that p \\in G and D \u000A \\in Df.  Then for some q \\in G extending p, q \\in D.</p> \u000A \u000A <p>Proof.  Let D0 be the set of p' \\in D that either extend p or have no \u000A extension in D that extends p.  We leave it as a straightforward exercise to \u000A show that D0 is dense, and D0 is clearly first-order definable (with \u000A parameters) in M.  So by genericity of G, we may pick q \\in D0 such that q \u000A \\in G.  Thus q \\in D.  By definition of generic, some extension q1 of both p \u000A and q belongs to G.  Pick q2 \\in D extending q1; thus q has an extension in D \u000A that extends p (namely, q2), so by definition of D0, q extends p. -|</p> \u000A \u000A <p>Definition of forcing.  Let phi(x1,...,xk) be a first-order formula in L[g] \u000A and let p \\in P.  We define a formula of L, denoted ``p ||- phi'' (``p forces \u000A phi''), by recursion on phi (in the metatheory) as follows.  (Here, we view \u000A ``or'' and ``forall'' as abbreviations.)</p> \u000A \u000A <blockquote><p>If phi is atomic, then let phi'(A) be the result of replacing, \u000A   inside-out, each subterm of the form g(x_1,...,x_Nb) with the term (cdr \u000A   (assoc-equal (list x_1 ... x_Nb) A)), where A is neither p nor a variable \u000A   occurring in phi.  Then p ||- phi is defined as follows: ``The set {A \\in \u000A   P: A extends p and phi'(A)} is dense beyond p''.  That is, p ||- phi is the \u000A   following formula:</p> \u000A \u000A <code>(<see topic=\"ACL2____FORALL\">forall</see> p1 \\in P extending p)\u000A (<see topic=\"ACL2____EXISTS\">exists</see> p2 \\in P extending p1) phi'(p2).</code> \u000A \u000A <p>p ||- ~phi is: (forall p' \\in P extending p) ~(p' ||- phi)</p> \u000A \u000A <p>p ||- phi_1 &amp; phi_2 is: (p ||- phi_1) &amp; (p ||- phi_2)</p> \u000A \u000A <p>p ||- (exists x) phi is: (exists x) (p ||- phi)</p></blockquote> \u000A \u000A <p>We will need the following definition later.</p> \u000A \u000A <p>Definition.  p ||-w phi (p <i>weakly forces</i> phi) is an abbreviation for \u000A p ||- ~~phi.</p> \u000A \u000A <p>The following exercises were suggested by John Cowles as a means for \u000A gaining familiarity with the definition of forcing.</p> \u000A \u000A <p>Exercise 1. Consider the formula (phi_1 OR phi_2) as an abbreviation for \u000A ~(~phi_1 &amp; ~phi_2), Show that p ||- (phi_1 OR phi_2) is equivalent to the \u000A following.</p> \u000A \u000A <code>(<see topic=\"ACL2____FORALL\">forall</see> p' \\in P extending p) (<see topic=\"ACL2____EXISTS\">exists</see> p'' \\in P extending p')\u000A ((p'' ||- phi_1) OR (p'' ||- phi_2))</code> \u000A \u000A <p>Exercise 2. Consider the formula (forall x)phi as an abbreviation for \u000A ~(exists x)~phi, Show that p ||- (forall x)phi is equivalent to the \u000A following.</p> \u000A \u000A <code>(<see topic=\"ACL2____FORALL\">forall</see> x)\u000A (<see topic=\"ACL2____FORALL\">forall</see> p1 \\in P extending p)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> p2 \\in P extending p1) (p2 ||- phi).</code> \u000A \u000A <p>Exercise 3. Prove that p ||-w phi is equivalent to the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____FORALL\">forall</see> p' \\in P extending p)\u000A (<see topic=\"ACL2____EXISTS\">exists</see> p'' \\in P extending p') (p'' ||- phi).</code> \u000A \u000A <p>Exercise 4. Let phi be a formula of L[g].  Prove: M |= (p ||- \u000A      phi)[s[p&lt;-p0]] implies M |= (p ||-w phi)[s[p&lt;-p0]].</p> \u000A \u000A <p>Exercise 5. Let phi be a formula of L[g].  Prove: M |= (p ||- \u000A      ~phi)[s[p&lt;-p0]] iff M |= (p ||-w ~phi)[s[p&lt;-p0]].</p> \u000A \u000A <p>[End of exercises.]</p> \u000A \u000A <p>The definition of forcing stipulates how to view ``p ||- phi(x1,...,xk)'' \u000A as a new formula theta(p,x1,...,xk).  That is, ``||-'' transforms formulas, so \u000A for any first-order formula phi, ``p ||- phi'' is just another first-order \u000A formula.  That observation shows that a formula such as ((p ||- phi) OR (p ||- \u000A ~phi)) is really just another first-order formula.  The following proposition \u000A thus follows easily.</p> \u000A \u000A <p>Proposition 3. For any formula phi of L[g], {p0: M |= ((p ||- phi) OR (p \u000A ||- ~phi))[s[p&lt;-p0]]]} is a dense subset of P, which (since it is \u000A first-order definable with parameters in M) intersects G. -|</p> \u000A \u000A <p>The following proposition is easily proved by a structural induction on \u000A phi, and is left to the reader.</p> \u000A \u000A <p>Proposition 4. Let phi be a formula of L[g].  Suppose <v>p0 in P</v>, <v>p1\u000A in P</v>,<br></br> \u000A \u000A M |= (p ||- phi)[s[p&lt;-p0]] and p1 extends p0.  Then<br></br> \u000A \u000A M |= (p ||- phi)[s[p&lt;-p1]]. -|</p> \u000A \u000A <p>We will also need the following.</p> \u000A \u000A <p>Proposition 5. The following is dense for any finite set S of Nb-tuples: {p \u000A \\in P: for some &lt;x_1 ... x_Nb&gt; \\in S, (list x_1 ... x_Nb) \\in \u000A strip-cars(p)}.  Thus, the function g0 is a total function. -|</p> \u000A \u000A <p>The next lemma tells us that the sentences true in M[G] are those that are \u000A forced by an element of G.</p> \u000A \u000A <p>Truth Lemma.  Let phi be a formula in L[g], let s be an assignment to the \u000A free variables of phi, and let p be a variable not in the domain of s.  Then \u000A M[G] |= phi[s] iff for some p0 \\in G, M |= (p ||- phi)[s[p&lt;-p0]].</p> \u000A \u000A <p>Proof.  The proof is by induction on the structure of phi.  First suppose \u000A phi is atomic.  Let D* be the set of elements p0 \\in P such that every \u000A assoc-equal evaluation from the definition of forcing phi returns a pair when \u000A A is bound to p0.  (Intuitively, this means that p0 is a sufficiently large \u000A approximation from any G containing p0 to make sense of phi in M[G].)  We make \u000A the following claim.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____A2\">*</see>)   For all p0 \\in G such that p0 \\in D*,\u000A      M[G] |= phi[s] iff M |= (p ||- phi)[s[p&lt;-p0]].</code> \u000A \u000A <p>To prove the claim, fix p0 in both G and D*, and recall the function g0 \u000A constructed from G in the definition of M[G].  Suppose that t_1, ..., t_Nb are \u000A terms and g(t_1, ..., t_Nb) is a subterm of phi.  Then s assigns a value in M \u000A to each of the t_i.  Let a_i be the value assigned by s to t_i.  Then g0(a_1, \u000A ..., a_Nb) = (cdr (assoc-equal (list a_1 ... a_Nb) p0)), as the assoc-equal is \u000A a pair (since p0 \\in D*) and has the indicated value (because p0 \\in G).  It \u000A follows by the definition of formula phi' in the definition of forcing:</p> \u000A \u000A <code>M[G] |= phi[s]  iff  M |= phi'(p)[s[p&lt;-p0]]</code> \u000A \u000A <p>Moreover, because p0 \\in D* it is clear that this holds if p0 is replaced \u000A by an arbitrary extension of p0.  Then (*) easily follows.</p> \u000A \u000A <p>By Proposition 5, D* is dense, so there is some p0 in the intersection of \u000A D* and G.  The forward direction of the conclusion then follows by (*).  The \u000A reverse direction is clear from (*) by application of Proposition 2 to D* and \u000A Proposition 4.</p> \u000A \u000A <p>Next, suppose M[G] |= ~phi[x].  Then it is not the case that M[G] |= phi, \u000A so by the inductive hypothesis, there is no p0 \\in G for which M |= (p ||- \u000A phi)[s[p&lt;-p0]].  By Proposition 3, there is p0 \\in G for which M |= (p ||- \u000A ~phi)[s[p&lt;-p0]].  For the other direction, suppose it is not the case that \u000A M[G] |= ~phi[s].  So M[G] |= phi[s], and by the inductive hypothesis, there is \u000A p0 \\in G for which M |= (p ||- phi)[s[p&lt;-p0]].  It follows that there is \u000A no p1 \\in G for which M |= (p ||- ~phi)[s[p&lt;-p1]], since from such p1 we \u000A can find a common extension p2 of p0 and p1 (since G is generic), and since p2 \u000A extends p0 then by Proposition 4, M |= (p ||- phi)[s[p&lt;-p2]], contradicting \u000A (by definition of forcing) M |= (p ||- ~phi)[s[p&lt;-p1]] since p2 extends \u000A p1.</p> \u000A \u000A <p>The case (phi_1 &amp; phi_2) follows easily from the inductive hypothesis. \u000A For the forward direction, apply Proposition 4 and the observation that by \u000A genericity, if p0 \\in G and p1 \\in G then p0 and p1 they have a common \u000A extension in G.</p> \u000A \u000A <p>Finally, the case (exists x) phi follows trivially from the inductive \u000A hypothesis. -|</p> \u000A \u000A <p>Truth Lemma Corollary.  The Truth Lemma holds with ||-w replacing ||-.</p> \u000A \u000A <p>Proof.  This is clear by applying the Truth Lemma to ~~phi. -|</p> \u000A \u000A <p>Here is our main theorem.  Recall that all first-order theories in our ACL2 \u000A context satisfy the e0-induction scheme.</p> \u000A \u000A <p>Theorem.  M[G] satisfies e0-induction.</p> \u000A \u000A <p>Proof.  We consider an arbitrary instance of e0-induction in L[g], stated \u000A using a strict well-founded relation &lt;| and a formula phi.  We write phi(y) \u000A to indicate that y may be among the free variables of phi, and phi(y&lt;-x) to \u000A denote the result of substituting x for y in phi.</p> \u000A \u000A <code>theta(y):   (<see topic=\"ACL2____FORALL\">forall</see> y) [((<see topic=\"ACL2____FORALL\">forall</see> x &lt;| y) phi(y&lt;-x)) -&gt; phi(y)]\u000A         -&gt; (<see topic=\"ACL2____FORALL\">forall</see> y) phi(y)</code> \u000A \u000A <p>Our goal is to prove that theta holds in M[G].</p> \u000A \u000A <p>Below, we abuse notation by leaving assignments implicit and by writing ``p \u000A ||- phi(y0)'' to signify that the formula (p ||- phi(y)) is true in M under \u000A the extension of the explicit assignment that binds y to y0.  We believe that \u000A the intended meaning will be clear.</p> \u000A \u000A <p>Consider the following set D.</p> \u000A \u000A <code>D = {p \\in P: either p ||-w phi(y0) for all y0,\u000A              or else\u000A              for some y0, p ||- ~phi(y0) and\u000A                           for all y1 &lt;| y0 p ||-w phi(y1)}.</code> \u000A \u000A <p>The set D is clearly first-order definable (with parameters) in M.  We \u000A claim that D is a dense subset of P.  For suppose p0 \\in P; we find p1 \\in D \u000A extending p0, as follows.  If p0 ||-w phi(y0) for all y0, then we may take p1 \u000A to be p0.  Otherwise, by definition of ||-w and ||-, there is some y0 such \u000A that for some extension p0' of p0, p0' ||- ~phi(y0).  Pick a &lt;|-minimal \u000A such y0, and correspondingly pick p1 so that p1 extends p0 and p1 ||- \u000A ~phi(y0).  In order to show that p1 \\in D, it remains to show that for all y1 \u000A &lt;| y0, p1 ||-w phi(y1), i.e., there is no q extending p1 such that q ||- \u000A ~phi(y1).  This is indeed the case since otherwise q and y1 would contradict \u000A the &lt;|-minimality of y0.</p> \u000A \u000A <p>Applying the genericity of G and just-proved density of D, pick p0 \\in G \u000A such that p0 \\in D.  If p0 ||-w phi(y0) for all y0, then by the Truth Lemma \u000A Corollary, M[G] |= phi(y0) for all y0, and thus M[G] |= theta.  Otherwise, \u000A since p0 \\in D we may choose y0 such that p0 ||- ~phi(y0) and for all y1 \u000A &lt;| y0, p0 ||-w phi(y1).  By the Truth Lemma and its corollary, since p0 \u000A \\in G we have:</p> \u000A \u000A <code>(1)   M[G] |= ~phi(y0).\u000A(2)   For all y1 &lt;| y0, M[G] |= phi(y1).</code> \u000A \u000A <p>It follows that the antecedent of theta is false in M[G], as witnessed by y \u000A = y0; thus M[G] |= theta. -|</p> \u000A \u000A <p>Remark.  We close by returning, as promised above, to the question of why \u000A so much care is necessary in constructing an expansion of M.  We assume \u000A familiarity here with the notion of a ``non-standard'' natural number of M, \u000A i.e., one that is greater than the interpretation of any term that has the \u000A form (+ 1 1 1 ... 1).  Here is a very simple example that illustrates the need \u000A for some care.  Consider the following event, which introduces a function foo \u000A with the following property: for all x, if natp(x) then natp(foo(x)).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCHOOSE\">defchoose</see> foo (y) (x)\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> x) (<see topic=\"ACL2____NATP\">natp</see> y)))</code> \u000A \u000A <p>Certainly we can build a model of the above property from a model M of the \u000A ground-zero theory, by interpreting foo so that for all x for which M \u000A satisfies natp(x), foo(x) is also a natp in M.  But suppose we start with a \u000A non-standard model M of the ground-zero theory, and we happen to define foo(x) \u000A to be 1 for all non-standard natural numbers x and 0 for all other x.  The \u000A resulting expansion of M will not satisfy the e0-induction scheme or even the \u000A ordinary natural number induction scheme: foo(0)=0 holds in that expansion as \u000A does the implication foo(n)=0 =&gt; foo(n+1)=0 for every natural number n of \u000A M, standard or not; and yet foo(k)=0 fails for every non-standard natural \u000A number k of M.</p>"],

 "ACL2____BOOK-NAME":[["Books-tour"],"ACL2 Sources","ACL2","<code>Examples:\u000A&quot;list-processing&quot;\u000A&quot;/usr/home/smith/my-arith&quot;</code> \u000A \u000A <p>Book names are string constants that can be elaborated into file names.  We \u000A elaborate book names by concatenating the ``connected book directory'' (see \u000A <see topic=\"ACL2____CBD\">cbd</see>) string on the left and some ``extension,'' such as <v>&quot;.lisp&quot;</v>, \u000A on the right.  However, the connected book directory is not added if the book \u000A name itself already represents an absolute file name.  Furthermore, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> temporarily reset the connected book \u000A directory to be the directory of the book being processed.  This allows <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms to use file names without explicit mention of the \u000A enclosing book's directory.  This in turn allows <see topic=\"ACL2____BOOKS\">books</see> (together with \u000A those that they include, using <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>) to be moved between \u000A directories while maintaining their certification and utility.</p> \u000A \u000A <p>You may wish to read elsewhere for details of ACL2 file name conventions \u000A (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>), for a discussion of the filename that is the result of \u000A the elaboration described here (see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>), and for details of \u000A the concept of the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>).  For details of \u000A how <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> (see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>) and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>) use these concepts, see below.</p> \u000A \u000A <p>Often a book name is simply the familiar name of the file.  (See <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see> for discussion of the notions of ``directory string,'' \u000A ``familiar name,'' and ``extension''.  These concepts are not on the guided \u000A tour through <see topic=\"ACL2____BOOKS\">books</see> and you should read them separately.)  However, it \u000A is permitted for book names to include a directory or part of a directory \u000A name.  Book names never include the extension, since ACL2 must routinely tack \u000A several different extensions onto the name during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  For \u000A example, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> uses the <v>&quot;.lisp&quot;</v>, <v>&quot;.cert&quot;</v> and \u000A possibly the <v>&quot;.o&quot;</v> or <v>&quot;.lbin&quot;</v> extensions of the book name.</p> \u000A \u000A <p>Book names are elaborated into full file names by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  This elaboration is sensitive to the ``connected book \u000A directory.'' The connected book directory is an absolute filename string (see \u000A <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>) that is part of the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  (You may wish to see \u000A <see topic=\"ACL2____CBD\">cbd</see> and to see <see topic=\"ACL2____SET-CBD\">set-cbd</see> &mdash; note that these are not on the \u000A guided tour).  If a book name is an absolute filename string, ACL2 elaborates \u000A it simply by appending the desired extension to the right.  If a book name is \u000A a relative filename string, ACL2 appends the connected book directory on the \u000A left and the desired extension on the right.</p> \u000A \u000A <p>Note that it is possible that the book name includes some partial \u000A specification of the directory.  For example, if the connected book directory \u000A is <v>&quot;/usr/home/smith/&quot;</v> then the book name <v>&quot;project/task-1/arith&quot;</v> \u000A is a book name that will be elaborated to</p> \u000A \u000A <code>&quot;/usr/home/smith/project/task-1/arith.lisp&quot;.</code> \u000A \u000A <p>Observe that while the <see topic=\"ACL2____EVENTS\">events</see> in this <v>&quot;arith&quot;</v> book are being \u000A processed the connected book directory will temporarily be set to</p> \u000A \u000A <code>&quot;/usr/home/smith/project/task-1/&quot;.</code> \u000A \u000A <p>Thus, if the book requires other <see topic=\"ACL2____BOOKS\">books</see>, e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;naturals&quot;)</code> \u000A \u000A <p>then it is not necessary to specify the directory on which they reside \u000A provided that directory is the same as the superior book.</p> \u000A \u000A <p>This inheritance of the connected book directory and its use to elaborate \u000A the names of inferior <see topic=\"ACL2____BOOKS\">books</see> makes it possible to move <see topic=\"ACL2____BOOKS\">books</see> and \u000A their inferiors to new directories, provided they maintain the same relative \u000A relationship.  It is even possible to move with ease whole collections of \u000A <see topic=\"ACL2____BOOKS\">books</see> to different filesystems that use a different operating system \u000A than the one under which the original certification was performed.</p> \u000A \u000A <p>The <v>&quot;.cert&quot;</v> extension of a book, if it exists, is presumed to \u000A contain the most recent <see topic=\"ACL2____CERTIFICATE\">certificate</see> for the book.  See <see topic=\"ACL2____CERTIFICATE\">certificate</see> (or, if you are on the guided tour, wait until the tour gets \u000A there).</p> \u000A \u000A <p>See <see topic=\"ACL2____BOOK-CONTENTS\">book-contents</see> to continue the guided tour.</p>"],

 "ACL2____TYPE-SPEC":[["Declare","The"],"ACL2 Sources","ACL2","<p>ACL2 permits the use of type declarations in certain contexts; see \u000A <see topic=\"COMMON-LISP____DECLARE\">declare</see> for background.  Here is an example of a type declaration; here \u000A the symbol, <v>integer</v>, is what we refer to as a ``type-spec'':</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP_____B2\">+</see> a b)))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer x))   ;; &lt;-- type declaration\u000A  (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x))</code> \u000A \u000A <p>A Common Lisp compiler might be able to use the above declaration to \u000A improve the execution efficiency of the resulting code.  In particular, the \u000A Common Lisp <see topic=\"COMMON-LISP_____B2\">+</see> operation is rather elaborate: it must be capable of \u000A adding together integers, rationals, floats, etc.  When we tell the compiler \u000A that this <v>x</v> is surely an integer, it may be able to use a more efficient \u000A version of <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 x)</v> that only deals with the integer case.</p> \u000A \u000A <p>Type declarations can be added to functions, <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> bindings, and \u000A other places as described in <see topic=\"COMMON-LISP____DECLARE\">declare</see>, and interact with the ACL2's \u000A <see topic=\"ACL2____GUARD\">guard</see> mechanism.  For instance, during guard verification, the above \u000A type declaration will result in a guard obligation: we will need to prove that \u000A <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b)</v> is always an integer.  Type declarations about the formals of a \u000A function generally become part of the guard of the function, but see also \u000A <see topic=\"ACL2____SPLIT-TYPES\">split-types</see> for a way to gain more control over this.</p> \u000A \u000A <p>Whether or not a particular type declaration will actually improve the \u000A efficiency of your functions depends on the Lisp compiler.  For instance, many \u000A Lisp compilers will not benefit much from a plain <v>integer</v> declaration. \u000A If you are trying to optimize code by adding type declarations, it may be \u000A useful to use <see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> to inspect the impact that your declarations \u000A have on the resulting code.</p> \u000A \u000A <h3>Type Specs</h3> \u000A \u000A <p>The syntax that Common Lisp compilers use for these type \u000A declarations&mdash;e.g., the symbol <v>integer</v> above&mdash;is different than \u000A the usual syntax of ACL2.</p> \u000A \u000A <p>We use the name <b>type-spec</b> to refer to the supported types that can \u000A be mentioned in declarations.  For instance:</p> \u000A \u000A <table> \u000A <tr><th>Declaration</th>                            <th>Type-Spec</th></tr> \u000A <tr><td><v>(<see topic=\"COMMON-LISP____TYPE\">type</see> string x y z)</v></td>     <td><v>string</v></td></tr> \u000A <tr><td><v>(<see topic=\"COMMON-LISP____TYPE\">type</see> (integer 0 7) x)</v></td>  <td><v>(integer 0 7)</v></td></tr> \u000A <tr><td><v>(<see topic=\"COMMON-LISP____TYPE\">type</see> rational x)</v></td>       <td><v>rational</v></td></tr> \u000A <tr><td><v>(<see topic=\"COMMON-LISP____TYPE\">type</see> (rational 1 *) x)</v></td> <td><v>(rational 1 *)</v></td></tr> \u000A </table> \u000A \u000A <p>The type-specs and their meanings (when applied to the variable <v>x</v> as \u000A in <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> type-spec x))</v> are given below.</p> \u000A \u000A <code>type-spec              meaning\u000A----------------------------------------------------------------------\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> type1 ... typek)  (<see topic=\"COMMON-LISP____AND\">AND</see> (p1 X) ... (pk X))\u000A                       where (pj x) is the meaning for type-spec typej\u000AATOM                   (<see topic=\"COMMON-LISP____ATOM\">ATOM</see> X)\u000ABIT                    (<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 1) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 0))\u000ACHARACTER              (<see topic=\"COMMON-LISP____CHARACTERP\">CHARACTERP</see> X)\u000ACOMPLEX                (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____COMPLEX-RATIONALP\">COMPLEX-RATIONALP</see> X)\u000A                            (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> (<see topic=\"COMMON-LISP____REALPART\">REALPART</see> X))\u000A                            (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> (<see topic=\"COMMON-LISP____IMAGPART\">IMAGPART</see> X)))\u000A(<see topic=\"COMMON-LISP____COMPLEX\">COMPLEX</see> RATIONAL)     same as COMPLEX, above\u000A(<see topic=\"COMMON-LISP____COMPLEX\">COMPLEX</see> type)         (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____COMPLEX-RATIONALP\">COMPLEX-RATIONALP</see> X)\u000A                            (p (<see topic=\"COMMON-LISP____REALPART\">REALPART</see> X))\u000A                            (p (<see topic=\"COMMON-LISP____IMAGPART\">IMAGPART</see> X)))\u000A                       where (p x) is the meaning for type-spec type\u000ACONS                   (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000AINTEGER                (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)\u000A(INTEGER i j)          (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)   ; See notes below\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i X)\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X j))\u000A(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> x1 ... xn)     (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> X '(x1 ... xn))\u000A(<see topic=\"COMMON-LISP____MOD\">MOD</see> i)                same as (INTEGER 0 i-1)\u000ANIL                    NIL\u000A(<see topic=\"COMMON-LISP____NOT\">NOT</see> type)             (<see topic=\"COMMON-LISP____NOT\">NOT</see> (p X))\u000A                       where (p x) is the meaning for type-spec type\u000ANULL                   (<see topic=\"COMMON-LISP____EQ\">EQ</see> X NIL)\u000A(<see topic=\"COMMON-LISP____OR\">OR</see> type1 ... typek)   (<see topic=\"COMMON-LISP____OR\">OR</see> (p1 X) ... (pk X))\u000A                       where (pj x) is the meaning for type-spec typej\u000ARATIO                  (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X) (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)))\u000ARATIONAL               (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)\u000A(RATIONAL i j)         (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)  ; See notes below\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i X)\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X j))\u000AREAL                   (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)       ; (REALP X) in ACL2(r)\u000A(<see topic=\"COMMON-LISP____REAL\">REAL</see> i j)             (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> X)  ; See notes below\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i X)\u000A                            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X j))\u000A(SATISFIES pred)       (pred X) ; Lisp requires a unary function, not a macro\u000ASIGNED-BYTE            (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)\u000A(SIGNED-BYTE i)        same as (INTEGER k m) where k=-2^(i-1), m=2^(i-1)-1\u000ASTANDARD-CHAR          (STANDARD-CHARP X)\u000ASTRING                 (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A(<see topic=\"COMMON-LISP____STRING\">STRING</see> max)           (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> X) max))\u000ASYMBOL                 (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X)\u000AT                      T\u000AUNSIGNED-BYTE          same as (INTEGER 0 *)\u000A(UNSIGNED-BYTE i)      same as (INTEGER 0 (2^i)-1)\u000A----------------------------------------------------------------------</code> \u000A \u000A <h5>Notes</h5> \u000A \u000A <p>In general, <v>(integer i j)</v> means</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X)\u000A     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> i X)\u000A     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X j)).</code> \u000A \u000A <p>But if <v>i</v> is the symbol <v>*</v>, the first inequality is omitted.  If \u000A <v>j</v> is the symbol <v>*</v>, the second inequality is omitted.  If instead of \u000A being an integer, the second element of the type specification is a list \u000A containing an integer, <v>(i)</v>, then the first inequality is made strict.  An \u000A analogous remark holds for the <v>(j)</v> case.  The <v>rational</v> and <v>real</v> \u000A type specifiers are similarly generalized.</p>"],

 "ACL2____SPECIOUS-SIMPLIFICATION":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Occasionally the ACL2 theorem prover reports that the current goal \u000A simplifies to itself or to a set including itself.  Such simplifications are \u000A said to be ``specious'' and are ignored in the sense that the theorem prover \u000A acts as though no simplification were possible and tries the next available \u000A proof technique.  Specious simplifications are almost always caused by \u000A forcing.</p> \u000A \u000A <p>The simplification of a formula proceeds primarily by the local application \u000A of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, and other rules to \u000A its various subterms.  If no rewrite rules apply, the formula cannot be \u000A simplified and is passed to the next ACL2 proof technique, which is generally \u000A the elimination of destructors.  The experienced ACL2 user pays special \u000A attention to such ``maximally simplified'' formulas; the presence of \u000A unexpected terms in them indicates the need for additional rules or the \u000A presence of some conflict that prevents existing rules from working \u000A harmoniously together.</p> \u000A \u000A <p>However, consider the following interesting possibility: local rewrite \u000A rules apply but, when applied, reproduce the goal as one of its own subgoals. \u000A How can rewrite rules apply and reproduce the goal?  Of course, one way is for \u000A one rule application to undo the effect of another, as when commutativity is \u000A applied twice in succession to the same term.  Another kind of example is when \u000A two rules conflict and undermine each other.  For example, under suitable \u000A hypotheses, <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> x)</v> might be rewritten to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</v> \u000A by the <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> of <tt><see topic=\"COMMON-LISP____LENGTH\">length</see></tt> and then a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule might be used to ``fold'' that back to <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> x)</v>. \u000A Generally speaking the presence of such ``looping'' rewrite rules causes \u000A ACL2's simplifier either to stop gracefully because of heuristics such as that \u000A described in the documentation for <tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> or to cause a stack \u000A overflow because of indefinite recursion.</p> \u000A \u000A <p>A more insidious kind of loop can be imagined: two rewrites in different \u000A parts of the formula undo each other's effects ``at a distance,'' that is, \u000A without ever being applied to one another's output.  For example, perhaps the \u000A first hypothesis of the formula is simplified to the second, but then the \u000A second is simplified to the first, so that the end result is a formula \u000A propositionally equivalent to the original one but with the two hypotheses \u000A commuted.  This is thought to be impossible unless forcing or case-splitting \u000A occurs, but if those features are exploited (see <see topic=\"ACL2____FORCE\">force</see> and see <see topic=\"ACL2____CASE-SPLIT\">case-split</see>) it can be made to happen relatively easily.</p> \u000A \u000A <p>Here is a simple example.  Declare <v>foo</v> to be a function of one \u000A argument returning one result:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> p1 (x) t)</code> \u000A \u000A <p>Prove the following silly rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (p1 x))\u000A           (p1 x)))</code> \u000A \u000A <p>Now suppose we try the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (p1 x))</code> \u000A \u000A <p>The <see topic=\"ACL2____REWRITE\">rewrite</see> rule <v>bad</v> will rewrite <v>(p1 x)</v> to <v>t</v>, but it \u000A will be unable to prove the hypothesis <v>(<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (p1 x))</v>.  As a result, \u000A the prover will spawn a new goal, to prove <v>(p1 x)</v>.  However, since this \u000A new goal is the same as the original goal, ACL2 will recognize the \u000A simplification as <i>specious</i> and consider the attempted simplification to \u000A have failed.</p> \u000A \u000A <p>In other words, despite the rewriting, no progress was made!  In more \u000A common cases, the original goal may simplify to a set of subgoals, one of \u000A which includes the original goal.</p> \u000A \u000A <p>If ACL2 were to adopt the new set of subgoals, it would loop indefinitely. \u000A Therefore, it checks whether the input goal is a member of the output \u000A subgoals.  If so, it announces that the simplification is ``specious'' and \u000A pretends that no simplification occurred.</p> \u000A \u000A <p>``Maximally simplified'' formulas that produce specious simplifications are \u000A maximally simplified in a very technical sense: were ACL2 to apply every \u000A applicable rule to them, no progress would be made.  Since ACL2 can only apply \u000A every applicable rule, it cannot make further progress with the formula.  But \u000A the informed user can perhaps identify some rule that should not be applied \u000A and make it inapplicable by disabling it, allowing the simplifier to apply all \u000A the others and thus make progress.</p> \u000A \u000A <p>When specious simplifications are a problem it might be helpful to <see topic=\"ACL2____DISABLE\">disable</see> all forcing (including <see topic=\"ACL2____CASE-SPLIT\">case-split</see>s) and resubmit the formula \u000A to observe whether forcing is involved in the loop or not.  See <see topic=\"ACL2____FORCE\">force</see>. \u000A The commands</p> \u000A \u000A <code>ACL2 !&gt;:disable-forcing\u000Aand\u000AACL2 !&gt;:enable-forcing</code> \u000A \u000A <p><see topic=\"ACL2____DISABLE\">disable</see> and <see topic=\"ACL2____ENABLE\">enable</see> the pragmatic effects of both <v>force</v> \u000A and <v>case-split</v>.  If the loop is broken when forcing is <see topic=\"ACL2____DISABLE\">disable</see>d, \u000A then it is very likely some <see topic=\"ACL2____FORCE\">force</see>d hypothesis of some rule is \u000A ``undoing'' a prior simplification.  The most common cause of this is when we \u000A <see topic=\"ACL2____FORCE\">force</see> a hypothesis that is actually false but whose falsity is somehow \u000A temporarily hidden (more below).  To find the offending rule, compare the \u000A specious simplification with its non-specious counterpart and look for rules \u000A that were speciously applied that are not applied in the non-specious case. \u000A Most likely you will find at least one such rule and it will have a <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypothesis.  By disabling that rule, at least for the subgoal in \u000A question, you may allow the simplifier to make progress on the subgoal.</p>"],

 "ACL2____NOTE-4-0":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 3.6.1 into the \u000A following categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, distributed books, Emacs support, and experimental \u000A versions.  Each change is described in just one category, though of course \u000A many changes could be placed in more than one category.  Also see <see topic=\"ACL2____NOTE-3-6-1\">note-3-6-1</see> for other changes since Version 3.6.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>There have been extensive changes to the handling of compiled files for \u000A books, which may generally be invisible to most users.  The basic idea is that \u000A when compiled files are loaded on behalf of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, they are now \u000A loaded before events in the book are processed, not afterwards.  This can \u000A speed up calls of <v>include-book</v>, especially if the underlying Lisp \u000A compiles all definitions on the fly (as is the case for CCL and SBCL).  One \u000A functional change is that for keyword argument <v>:load-compiled-file</v> of \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, the values <v>:comp-raw</v>, <v>:try</v>, and <v>:comp!</v> \u000A are no longer legal.  (Note that the handling of <v>:comp-raw</v> was actually \u000A broken, so it seems that this value wasn't actually used by anyone; also, the \u000A handling of <v>:comp!</v> formerly could cause an error in some Lisp platforms, \u000A including SBCL.)  Another change is that if <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is called \u000A with <v>:load-compiled-file t</v>, then each sub-book must have a compiled file \u000A or a so-called ``expansion file''; see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.  In the \u000A unlikely event that this presents a problem, the makefile provides a way to \u000A build with compilation disabled; see <see topic=\"ACL2____COMPILATION\">compilation</see>.  Users of raw mode \u000A (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) will be happy to know that <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> now \u000A works if there an up-to-date compiled file for the book, since <tt><see topic=\"ACL2____PORTCULLIS\">portcullis</see></tt> commands are now incorporated into that compiled file.  The \u000A mechanism for saving expansion files has changed, and the <v>:save-expansion</v> \u000A argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> has been eliminated; see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>. \u000A More discussion of ACL2's new handling of book compilation is described in a \u000A new documentation topic; see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>.</p> \u000A \u000A <p>It was possible to get a hard Lisp error when certifying a book with a \u000A redundant <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> form whose term is not identical to the existing \u000A <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> form for the same name.  The following example illustrates \u000A the problem, which has been fixed (as part of the change in handling of \u000A compiled files for books, described above).  Imagine that after the initial \u000A <v>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)</v> form, file <v>foo.lisp</v> has just the form \u000A <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* (<see topic=\"COMMON-LISP____APPEND\">append</see> nil nil))</v>.  Then before the fix, we could have:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* nil)\u000A[[output omitted]]\u000AACL2 !&gt;(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;foo&quot; 1)\u000A[[initial output omitted]\u000A* Step 5:  Compile the functions defined in &quot;/v/joe/foo.lisp&quot;.\u000ACompiling /v/joe/foo.lisp.\u000AEnd of Pass 1.\u000AEnd of Pass 2.\u000AOPTIMIZE levels: Safety=0 (No runtime error checking), Space=0, Speed=3\u000AFinished compiling /vjoe/foo.lisp.\u000ALoading /v/joe/foo.lisp\u000A\u000AError: Illegal attempt to redeclare the constant *A*.</code> \u000A \u000A <p>The <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt> facility has been changed to repair a bug that allowed \u000A guard violations to go undetected.  The major change has to do with the \u000A treatment of what used to be called the ``pseudo-flag'' argument which has \u000A been replaced by a quoted lambda expression.  See <see topic=\"ACL2____NOTE-4-0-WORMHOLE-CHANGES\">note-4-0-wormhole-changes</see> for help in converting calls of <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>. \u000A Also see see <see topic=\"ACL2____WORMHOLE\">wormhole</see> and see <see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see>.</p> \u000A \u000A <p>The function <v>assign-wormhole-output</v> has been eliminated but its \u000A functionality can be provided by <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt>.</p> \u000A \u000A <p>The ACL2 tutorial has been greatly expanded, for example to include a \u000A self-contained discussion of rewriting.  See <see topic=\"ACL2____ACL2-TUTORIAL\">ACL2-tutorial</see>.</p> \u000A \u000A <p>Formerly, the <tt><see topic=\"ACL2____MBE\">mbe</see></tt> macro and <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt> function were \u000A disallowed in any definition within an encapsulate having a non-empty \u000A signature.  Now, these are allowed provided the definition has been declared \u000A to be non-executable (see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>).  As a result, <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> may now include <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt> among the function symbols \u000A known by the evaluator; this had not previously been allowed.  Thanks to Sol \u000A Swords for discussions leading to this relaxation for <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____PRINC_42\">Princ$</see></tt> now prints strings more efficiently.  Thanks to Jared Davis \u000A for suggesting the improvements to <v>princ$</v>.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration <v>:non-executable t</v> no longer \u000A requires the absence of <tt><see topic=\"ACL2____STATE\">state</see></tt> or declared <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s among the \u000A formal parameters of a function definition.  As before, the use of \u000A <v>:non-executable t</v> turns off single-threadedness checking for the body, \u000A and also as before, attempts to execute the function will fail.  Thanks to Sol \u000A Swords for requesting this relaxation (for automatic generation of so-called \u000A ``flag functions'' for definitions using <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>).</p> \u000A \u000A <p>The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been improved for explaining to advanced users \u000A the details of the ACL2 hint mechanism; see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>, \u000A and see the example about <v>nonlinearp-default-hint</v> in distributed book \u000A <v>books/hints/basic-tests.lisp</v>.  Thanks to Robert Krug for useful \u000A discussions, in particular suggesting the above example as one to be explained \u000A with the documentation.</p> \u000A \u000A <p>The <v>time$</v> macro has been enhanced to allow user control of the timing \u000A message that is printed, and of when it is printed.  See <see topic=\"ACL2____TIME_42\">time$</see>.  Thanks \u000A to Jared Davis for providing the essential design, helpful documentation \u000A (largely incorporated), and an initial implementation for raw Lisp.</p> \u000A \u000A <p>The <v>:ttags</v> argument to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> had been required when \u000A including a certified book that uses trust tags.  This is no longer the case: \u000A essentially, <v>:ttags</v> defaults to <v>:all</v> except that warnings will be \u000A printed.  Thanks to Jared Davis for requesting such a relaxation, and to him \u000A and Sandip Ray for useful discussions.</p> \u000A \u000A <p>The definition of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> has been modified so that the single-step \u000A macroexpansion (see <see topic=\"ACL2____TRANS1\">trans1</see>) of its calls can be evaluated.  Thanks to \u000A Pete Manolios for bringing this evaluation issue to our attention and ensuing \u000A discussions.</p> \u000A \u000A <p>All calls of so-called ``guard-holders'' &mdash; <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt> (from calls of see <see topic=\"ACL2____MBE\">mbe</see>), <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, and <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt> &mdash; are now removed before storing hypotheses of rules of class \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>.  Thanks to Sol Swords for \u000A requesting this enhancement and sending the following example in the case of \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> rules.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> foo\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;asdf&quot;)\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) x)\u000A               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) y))))</code> \u000A \u000A <p>The handling of <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive <v>~s</v> has been modified so that if \u000A the argument is a symbol that would normally be printed using vertical bars \u000A (<v>|</v>), then that symbol is printed as with <v>~f</v>.  Thanks to Jared Davis \u000A for providing the following example showing that treatment of <v>~s</v> was a \u000A bit unexpected: <v>(<see topic=\"ACL2____CW\">cw</see> &quot;~s0.~%&quot; '|fo\\|o|)</v>.</p> \u000A \u000A <p>Error messages have been improved for ill-formed terms (in ACL2's so-called \u000A ``translation'' routines).  Thanks to Jared Davis for requesting such an \u000A enhancement.</p> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> so that it executes in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode. \u000A Previously, evaluation of a <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> event in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode caused a somewhat inscrutable error, but now, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode \u000A is treated the same as <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode for purposes of <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>.</p> \u000A \u000A <p>The ``system hacker'' commands <v>(</v><tt><see topic=\"ACL2____REDEF_B2\">redef+</see></tt><v>)</v> and <v>(</v><tt><see topic=\"ACL2____REDEF-\">redef-</see></tt><v>)</v> are now embedded event forms (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), \u000A hence may be used in <see topic=\"ACL2____BOOKS\">books</see> as well as in <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  Also, these two commands are now no-ops in raw \u000A Lisp.</p> \u000A \u000A <p>The function symbol <v>worldp</v> (in the <v>&quot;ACL2&quot;</v> package) has been \u000A renamed to <v>plist-worldp</v>.</p> \u000A \u000A <p>The function <v>gc$-fn</v> (resulting from macroexpansion of <tt><see topic=\"ACL2____GC_42\">gc$</see></tt>) is \u000A now in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  Thanks to Jared Davis for requesting this \u000A change.</p> \u000A \u000A <p>The user now has control over whether compilation is used, for example \u000A whether or not <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> compiles by default, using function \u000A <v>set-compiler-enabled</v>.  See <see topic=\"ACL2____COMPILATION\">compilation</see>.</p> \u000A \u000A <p>Modified the conversion of relative to absolute pathnames in <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s for book certification.  Now, more pathnames \u000A remain as relative pathnames.</p> \u000A \u000A <p>The <v>&quot;Ttags&quot;</v> warning that can be printed by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is \u000A now given even if <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt> has specified `<v>warning</v>'. \u000A To suppress it, specify <v>warning!</v> instead, for example, \u000A <v>(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> '(acl2::warning! acl2::proof-tree))</v>.</p> \u000A \u000A <p>On occasion, ACL2 prints the message ``Flushing current installed world'' \u000A as it cleans up when certain actions (installing a <see topic=\"ACL2____WORLD\">world</see>) are \u000A interrupted.  This operation has been sped up considerably.  If your session \u000A includes many <see topic=\"ACL2____EVENTS\">events</see>, you can probably speed up any such operation \u000A further by invoking <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt>.  Thanks to Jared Davis for \u000A sending a query that led us to make this improvement.</p> \u000A \u000A <p>Calls of the form <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> (<see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see> logic exec))</v> are no longer \u000A allowed, since we do not have confidence that they would be handled \u000A correctly.</p> \u000A \u000A <p>The underlying function for <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> (and hence for <tt><see topic=\"ACL2____EXIT\">exit</see></tt> \u000A and <tt><see topic=\"ACL2____QUIT\">quit</see></tt>) is now in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  Thanks to Jared Davis \u000A for requesting this enhancement.</p> \u000A \u000A <p>We now require that every function symbol in the <see topic=\"ACL2____SIGNATURE\">signature</see> of an \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event have a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode definition at the \u000A end of the first pass, not merely a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode definition \u000A (which formerly was sufficient).  You can still define such a function in \u000A <v>:program</v> mode, provided it is followed by a <v>:logic</v> mode definition \u000A (where of course both definitions are <see topic=\"ACL2____LOCAL\">local</see>, since we are discussing \u000A functions is introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see>).  Thanks to Carl Eastlund for \u000A bringing this issue to our attention.  (Note: An analogous modification has \u000A been made for <v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt> <see topic=\"ACL2____HINTS\">hints</see> as well.)</p> \u000A \u000A <p>The following functions now have raw Lisp implementations that may run \u000A faster than their ACL2 definitions: <tt><see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see></tt>, <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>, \u000A <tt><see topic=\"ACL2____MEMBER-EQ\">member-eq</see></tt>, <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, <v>subsetp-eq</v>, <tt><see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see></tt>, <tt><see topic=\"ACL2____REMOVE-EQ\">remove-eq</see></tt>, <tt><see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see></tt>, <tt><see topic=\"ACL2____POSITION-EQ\">position-eq</see></tt>, \u000A and <tt><see topic=\"ACL2____POSITION-EQUAL\">position-equal</see></tt>.  Thanks to Jared Davis for suggesting that we \u000A consider such an improvement.</p> \u000A \u000A <p>We now avoid infinite loops caused when tracing functions that implement \u000A <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  Thanks to Rob Sumners and Eric Smith for useful \u000A discussions.</p> \u000A \u000A <p>The implementation of <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt> has been modified slightly, to \u000A accommodate the fix for ``some holes in the handling of trust tags'' described \u000A later, below.</p> \u000A \u000A <p>This item applies unless the host Lisp is GCL.  An interrupt (control-c) \u000A will now cause a proof to exit normally in most cases, by simulating a \u000A timeout, as though <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt> had been called with a \u000A time-limit of 0.  If the first interrupt doesn't terminate the proof, a second \u000A one should do so (because a different, more ``severe'' mechanism is used after \u000A the first attempt).  As a result, <tt><see topic=\"ACL2____REDO-FLAT\">redo-flat</see></tt> should work as one might \u000A expect even if a proof is interrupted.  Thanks to Dave Greve for requesting \u000A this enhancement to <tt><see topic=\"ACL2____REDO-FLAT\">redo-flat</see></tt>.  Technical note: for reasons related to \u000A this change, time-limits are no longer checked in evaluator functions \u000A (<v>ev-fncall</v>, <v>ev</v>, <v>ev-lst</v>, <v>ev-fncall-w</v>, <v>ev-w</v>, and \u000A <v>ev-w-lst</v>).</p> \u000A \u000A <p>It is now legal for <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <see topic=\"ACL2____MACRO-COMMAND\">macro-command</see>s to appear in \u000A <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt> that are used in place of <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>. \u000A Thanks to Sandip Ray for (most recently) requesting this feature.</p> \u000A \u000A <p>The value of <v>:command-conventions</v> for <tt><see topic=\"ACL2____LD\">ld</see></tt> special variable \u000A <v>ld-post-eval-print</v> is now treated as though it were <v>t</v> if the value \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> special variable <v>ld-error-triples</v> is <v>nil</v>.  The following \u000A example illustrates this change.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see> state) ; default\u000A:COMMAND-CONVENTIONS\u000AACL2 !&gt;(<see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see> state) ; default\u000AT\u000AACL2 !&gt;(set-ld-error-triples nil state)\u000A*** Then, before the change:\u000AACL2 !&gt;(<see topic=\"ACL2____MV\">mv</see> t 3 state)\u000A 3\u000A*** Instead, after the change:\u000AACL2 !&gt;(<see topic=\"ACL2____MV\">mv</see> t 3 state)\u000A(T 3 &lt;state&gt;)</code> \u000A \u000A <p>The default behavior of <tt><see topic=\"ACL2____LD\">ld</see></tt> has been changed.  Formerly when an \u000A error occurred that halted a subsidiary call of <v>ld</v>, then the parent \u000A <v>ld</v> would continue.  That is no longer the case.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A            (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bad (x)) ; ERROR -- missing the body\u000A            ))\u000A      (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)))</code> \u000A \u000A <p>Formerly, <v>g</v> would be defined in the resulting logical <see topic=\"ACL2____WORLD\">world</see>. \u000A Now, the error halts not only the inner <v>ld</v> but also the outer <v>ld</v>. \u000A See <see topic=\"ACL2____LD\">ld</see>, and for details of the new default value for \u000A <v>:ld-error-action</v>, <v>:RETURN!</v>, see see <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see>.  Also \u000A see the paragraph below about a new utility, <v>:</v><tt><see topic=\"ACL2____P_12\">p!</see></tt>.  Thanks to \u000A Robert Krug and Sandip Ray for helpful discussions.</p> \u000A \u000A <p>Environment variable <v>ACL2-CUSTOMIZATION</v> has been replaced by \u000A <v>ACL2_CUSTOMIZATION</v> &mdash; that is, the hyphen has been replaced by an \u000A underscore &mdash; so that it can be set conveniently in the <v>bash</v> shell. \u000A See <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>The ``<v>Warnings:</v>'' summary is now omitted when there are no warnings, \u000A where formerly ``<v>Warnings: None</v>'' was printed.  Thanks to Jared Davis for \u000A suggesting this change.</p> \u000A \u000A <p>We have modified the generation of <v>constraint</v>s for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in two primary ways, neither of them likely to affect many \u000A users.  One change is that the virtual movement of definitions and theorems to \u000A in front of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, or of definitions to behind that \u000A event, is no longer inhibited in the case of nested encapsulates with \u000A non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>s.  The following example illustrates the other \u000A change, as discussed below.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((f (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x (f x)))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x) (g x))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> h-is-f (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (h x)) x)))</code> \u000A \u000A <p>Previously, the <see topic=\"ACL2____CONSTRAINT\">constraint</see> on <v>f</v> and <v>h</v> was essentially the \u000A conjunction of the definition of <v>h</v> and the theorem <v>h-is-f</v>.  Now, the \u000A definition of <v>g</v> is conjoined as well; moreover, <v>g</v> receives the same \u000A <see topic=\"ACL2____CONSTRAINT\">constraint</see> as do <v>f</v> and <v>h</v>, where previously <v>g</v> was only \u000A constrained by its definition.  While we are not aware of a soundness bug \u000A caused by the previous approach, the new approach follows more precisely the \u000A intended notion of <see topic=\"ACL2____CONSTRAINT\">constraint</see>.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> (or <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt>) option <v>:multiplicity</v> had \u000A been required when option <v>:native</v> was supplied.  This is no longer the \u000A case.  Also, a bug has been fixed that had prevented <v>:multiplicity</v> from \u000A working properly in GCL and Allegro CL.</p> \u000A \u000A <p>Several errors have been eliminated that formerly occurred when the \u000A constraints for a function symbol were unknown because it was constrained \u000A using a dependent clause-processor (see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>.  Now, it is assumed that the <v>supporters</v> \u000A argument in a <tt><see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see></tt> event is such that every \u000A ancestor of any function symbol constrained by the ``promised encapsulate'' of \u000A that event among, or ancestral in, those <v>supporters</v>.  Thanks to Sol \u000A Swords, Sandip Ray, and Jared Davis for helpful discussions.</p> \u000A \u000A <p>The notion of <see topic=\"ACL2____CONSTRAINT\">constraint</see> for functions introduced by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A has been modified slightly.  No longer do we remove from the body of the \u000A definition calls of so-called ``guard-holders'': <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>, <see topic=\"ACL2____EC-CALL\">ec-call</see>, and <see topic=\"ACL2____MV-LIST\">mv-list</see>, and uses of <v>the-error</v> \u000A generated by <tt><see topic=\"COMMON-LISP____THE\">the</see></tt>.  Also, we now expand calls of <v>the-error</v> with \u000A the same aggressive heuristics applied to a number of other functions \u000A (technically, adding it to the list \u000A <v>*expandable-boot-strap-non-rec-fns*</v>).</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>, allows evaluation of calls of constrained \u000A (<tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>d) functions.  In particular, users can now, in principle, \u000A soundly modify ACL2 source code; please feel free to contact the ACL2 \u000A implementors if you are interested in doing so.  See <see topic=\"ACL2____DEFATTACH\">defattach</see>.</p> \u000A \u000A <p>Eric Smith has noticed that if you exit the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> loop using \u000A <v>:</v><tt><see topic=\"ACL2____A_12\">a!</see></tt> during an <tt><see topic=\"ACL2____LD\">ld</see></tt> of a file, then all changes to the \u000A logical <see topic=\"ACL2____WORLD\">world</see> are discarded that were made during that call of <tt><see topic=\"ACL2____LD\">ld</see></tt>.  A new utility, <v>:</v><tt><see topic=\"ACL2____P_12\">p!</see></tt>, pops just one level instead, and avoids \u000A discarding that work.  (This change is related to an item above, ``The default \u000A behavior of <tt><see topic=\"ACL2____LD\">ld</see></tt> has been changed.'')  Thanks to Eric for pointing out \u000A this issue.</p> \u000A \u000A <p>New function <tt><see topic=\"ACL2____MV-LIST\">mv-list</see></tt> is the identity function logically, but \u000A converts multiple values to lists.  The first argument is the number of \u000A values, so an example form is as follows, where <v>foo</v> returns three values: \u000A <v>(<see topic=\"ACL2____MV-LIST\">mv-list</see> 3 (foo x y))</v>.  Thanks to Sol Swords for requesting this feature \u000A and for reporting a soundness bug in one of our preliminary \u000A implementations.</p> \u000A \u000A <p>A new <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable, <v>host-lisp</v>, has as its value a \u000A keyword whose value depends on the underlying Common Lisp implementation.  Use \u000A <v>(<see topic=\"ACL2_____04\">@</see> host-lisp)</v> to see its value.</p> \u000A \u000A <p>It is now possible to write <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for HTML without error \u000A when there are links to nonexistent documentation topics.  See the comments in \u000A macro <v>acl2::write-html-file</v> at the end of file \u000A <v>doc/write-acl2-html.lisp</v>.  When there are such errors, they should be \u000A easier to understand than previously.  Thanks to Alan Dunn for providing the \u000A initial modifications.</p> \u000A \u000A <p>It is now possible to inhibit specified parts of the Summary printed at the \u000A conclusion of an event.  See <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see>.  Also see \u000A <see topic=\"ACL2____WITH-OUTPUT\">with-output</see>, in particular the discussion of the new <v>:summary</v> \u000A keyword.  Thanks to Sol Swords for requesting more control over the \u000A Summary.</p> \u000A \u000A <p>A new <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> keyword, <v>:case-split-limitations</v>, can \u000A override the default case-split-limitations settings (see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>) in the simplifier.  Thanks to Ian Johnson for \u000A requesting this addition and providing an initial implementation.</p> \u000A \u000A <p>It is now possible to defer and avoid some ttag-related output; see <see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see>.  Thanks to Jared Davis for requesting less verbosity \u000A from ttag-related output.</p> \u000A \u000A <p>A new <see topic=\"ACL2____COMMAND\">command</see>, <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt>, allows you to restrict the rewrite \u000A rules printed that apply to a given term.  See <see topic=\"ACL2____PL2\">pl2</see>.  Thanks to Robert \u000A Krug for requesting such a capability.</p> \u000A \u000A <p>ACL2 now provides a utility for canonicalizing filenames, so that soft \u000A links are resolved; see <see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see>.  Moreover, ACL2 uses this \u000A utility in its own sources, which can eliminate some issues.  In particular, \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with argument <v>:ttags :all</v> no longer breaks when \u000A given a book name differing from the book name that was used at certification \u000A time; thanks to Sol Swords for reporting that problem.  Also, certain errors \u000A have been eliminated involving the combination of packages in the \u000A certification world and trust tags; thanks to Jared Davis for sending an \u000A example of that problem.</p> \u000A \u000A <p>You can now suppress or enable guard-checking for an individual form; see \u000A <see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see>.  Thanks to Sol Swords for requesting this \u000A feature.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____WALKABOUT\">walkabout</see></tt> utility has been documented (thanks to Rob Sumners \u000A for suggesting this documentation).  This utility can make it easy to explore \u000A a large <v>cons</v> tree.  New interactive commands <v>(pp n)</v> and <v>(pp\u000A print-level print-length)</v> have been added to restrict how much of the \u000A current object is displayed.  See <see topic=\"ACL2____WALKABOUT\">walkabout</see>.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> may now be provided a \u000A <v>:backchain-limit-lst</v> keyword.  The default behavior is unchanged, but now \u000A <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> is sensitive not only to the new <v>:backchain-limit-lst</v> of \u000A a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule (if supplied) but to the <tt><see topic=\"ACL2____DEFAULT-BACKCHAIN-LIMIT\">default-backchain-limit</see></tt> of the current logical <see topic=\"ACL2____WORLD\">world</see>.  Setting of \u000A backchain-limits can now specify either the new (type-set) limit or the old \u000A limit (for rewriting); see <see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see> and see <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>.  Moreover, the functions <tt><see topic=\"ACL2____DEFAULT-BACKCHAIN-LIMIT\">default-backchain-limit</see></tt> \u000A and <tt><see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see></tt> now take a second argument of <v>:ts</v> or \u000A <v>:rewrite</v> to specify which backchain-limit is desired.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>The so-called ``too-many-ifs'' heuristic has been modified.  Such a \u000A heuristic has been employed in ACL2 (and previous Boyer-Moore provers) for \u000A many years, in order to limit the introduction of calls of <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> by \u000A non-recursive functions.  Most users need not be concerned with this change, \u000A but two proofs in the regression suite (out of thousands) needed trivial \u000A adjustment, so user proofs could need tweaking.  In one application, this \u000A modification sped up proofs by 15%; but the change in runtime for the \u000A regression suite is negligible, so such speedups may vary.  Thanks to Sol \u000A Swords for providing a test from ACL2 runs at Centaur Technology, which was \u000A useful in re-tuning this heuristic.</p> \u000A \u000A <p>Guard proof obligations could have size quadratic in the number of clauses \u000A in a <tt><see topic=\"COMMON-LISP____CASE\">case</see></tt> statement.  This inefficiency has been removed with a change \u000A that eliminates a hypothesis of the form <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> term constant))</v> when \u000A there is already a stronger hypothesis, equating the same term with a \u000A different constant.  Thanks to Sol Swords for bringing this problem to our \u000A attention and suggesting an alternate approach to solving it, which we may \u000A consider in the future if related efficiency problems persist.</p> \u000A \u000A <p>We adjusted the heuristics for determining induction schemes in the \u000A presence of <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>, when handling calls of a function symbol \u000A that is a ruler-extender, in either of two cases: either the function takes \u000A only one argument; or the function is <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> or <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, and \u000A the first argument contains no recursive call.  These cases are treated more \u000A directly as though the ruler-extender call is replaced by the unique (in the \u000A case of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> and <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, the second) argument.</p> \u000A \u000A <p>A new <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule, <v>true-listp-append</v>, has \u000A been added:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b)\u000A         (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> a b)))</code> \u000A \u000A <p>If you are interested in the motivation for adding this rule, see comments \u000A in <v>true-listp-append</v> in ACL2 source file <v>axioms.lisp</v>.</p> \u000A \u000A <p>The use of <v>:forward-chaining</v> lemmas has been improved slightly.  In \u000A previous versions, a conclusion derived by forward chaining was discarded if \u000A it was derivable by type-set reasoning, since it was ``already provable.'' \u000A But this heuristic prevented the conclusion from triggering further forward \u000A chaining.  This has been fixed.  Thanks to Dave Greve for pointing out this \u000A problem.</p> \u000A \u000A <p>The fundamental utility that turns an <v>IF</v> expression into a set of \u000A clauses has been optimized to better handle tests of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x\u000A 'constant)</v> and their negations.  This eliminates an exponential explosion in \u000A large case analyses.  But it comes at the inconveience of sometimes reordering \u000A the clauses produced.  The latter aspect of this change may require you to \u000A change some Subgoal numbers in proof hints.  We apologize for the \u000A inconvenience.</p> \u000A \u000A <p>Certification can now run faster (specifically, the compilation phase) for \u000A books with very large structures generated by <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, when there \u000A is significant sharing of substructure, because of a custom optimization of \u000A the Lisp reader.  Thanks to Sol Swords for bringing this efficiency issue to \u000A our attention.</p> \u000A \u000A <p>Jared Davis reported inefficiency in certain <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> evaluation \u000A due to a potentially expensive ``bad lisp object'' check on the expansion \u000A produced by the <v>make-event</v>.  This check has been eliminated except in the \u000A case that the expansion introduces packages (for example, by including a book \u000A during the expansion phase that introduces packages).  Thanks to Jared for \u000A providing a helpful example.</p> \u000A \u000A <p>The application of rules of class <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> had the potential \u000A to loop (as commented in ACL2 source function <v>apply-induction-rule</v>). \u000A This has been fixed.  Thanks to Daron Vroon and Pete Manolios for sending nice \u000A examples causing the loop.</p> \u000A \u000A <p>Heuristics have been tweaked so that false goals may be simplified to \u000A <v>nil</v> that had formerly been left unchanged by simplification, perhaps \u000A resulting in useless and distracting proofs by induction.  Thanks to Pete \u000A Manolios for pointing out this issue by sending the following example: <v>(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)) 0))</v>.  (Technical explanation: When every literal in \u000A a clause simplifies to <v>nil</v>, even though we might not normally delete one \u000A or more such literals, we will replace the entire clause by the false \u000A clause.)</p> \u000A \u000A <p>Improved the efficiency of the built-in function, <tt><see topic=\"ACL2____TAKE\">take</see></tt>.  Thanks to \u000A Bob Boyer for suggesting this improvement.</p> \u000A \u000A <p>ACL2 can now use evaluation to relieve hypotheses when applying \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.  Thanks to Peter Dillinger and Dave \u000A Greve for requesting this enhancement, and to Robert Krug for a relevant \u000A discussion long ago.</p> \u000A \u000A <p>Evaluation has been sped up during theorems for calls of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>, by \u000A avoiding repeated evaluation of the expression to which its variables are \u000A bound.  Thanks to Sol Swords for requesting this improvement and sending an \u000A illustrative example.</p> \u000A \u000A <p>Modified a heuristic to avoid the opening up non-recursive function calls \u000A on calls of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> involving <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>-expressions.  For example, the \u000A <tt><see topic=\"ACL2____THM\">thm</see></tt> form below is now admitted</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x)\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar (<see topic=\"ACL2____HIDE\">hide</see> (<see topic=\"COMMON-LISP____IF\">if</see> a b c)))\u000A     (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____HIDE\">hide</see> (<see topic=\"COMMON-LISP____IF\">if</see> a b c)) (<see topic=\"ACL2____HIDE\">hide</see> (<see topic=\"COMMON-LISP____IF\">if</see> a b c)))))</code> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a soundness bug in destructor elimination, which was preventing some \u000A cases from being generated.  Thanks to Eric Smith for reporting this bug and \u000A sending a helpful example.  (Technical detail: the fixes were in ACL2 source \u000A functions <v>apply-instantiated-elim-rule</v> and \u000A <v>eliminate-destructors-clause1</v>, and comments in the former contain Eric's \u000A example.)</p> \u000A \u000A <p>Fixed a bug that supported a proof of <v>nil</v> by exploiting the fact that \u000A <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s were not included in check-sum computations \u000A in a book's <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  For such a proof of <v>nil</v>, see the \u000A relevant comment in the ACL2 source file <v>ld.lisp</v> under <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see>\u000A note-4-0 ...)</v>.</p> \u000A \u000A <p>Changed the implementation of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>.  The previous \u000A implementation could allow relative pathnames to be stored in the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s of <see topic=\"ACL2____CERTIFICATE\">certificate</see>s of <see topic=\"ACL2____BOOKS\">books</see>, which \u000A perhaps could lead to unsoundness (though we did not try to exploit this to \u000A prove <v>nil</v>).  Thanks to Jared Davis for reporting a bug in our first new \u000A implementation.  An additional change to both <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> and \u000A <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt> is that these now work in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>).  (Thanks to Dave Greve for suggesting a reduction in the \u000A warnings we produced related to raw-mode.)  Note that it is no longer \u000A permitted to make a direct call of the form <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table\u000A :include-book-dir-alist ...)</v>; use <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> instead.</p> \u000A \u000A <p>Fixed a soundness bug related to <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword \u000A <v>:non-executable</v>.  New macros, <v>defun-nx</v> and <v>defund-nx</v>, have been \u000A provided for declaring functions to be non-executable; see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see>. \u000A While we expect this bug to occur only rarely if at all in practice, the \u000A following example shows how it could be evoked.</p> \u000A \u000A <code>;;;;;;;;;;;;;;;;;;;;\u000A;;; Book sub.lisp\u000A;;;;;;;;;;;;;;;;;;;;\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                  :non-executable t))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (a b c)\u000A          (<see topic=\"ACL2____MV\">mv</see> 3 4)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> a b))\u000A          c))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____MV-LET\">mv-let</see> (x y z)\u000A                  (<see topic=\"ACL2____MV\">mv</see> 2 3 4)\u000A                  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y z))\u000A                  nil)\u000A          (f)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> g-nil\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g) nil)\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (f))))\u000A  :rule-classes nil)\u000A;;;;;;;;;;;;;;;;;;;;\u000A;;; Book top.lisp\u000A;;;;;;;;;;;;;;;;;;;;\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;sub&quot;)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use g-nil))\u000A  :rule-classes nil)</code> \u000A \u000A <p>The modification described above pertaining to <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt> also \u000A prevents execution of non-executable functions that have been <see topic=\"COMMON-LISP____TRACE\">trace</see>d. \u000A The following example illustrates the problem; now, the following <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>g</v> is illegal, and the problem disappears if <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt> is \u000A used instead.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) ; Use defun-nx to avoid an error after Version_3.6.1.\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :non-executable t))\u000A  x)\u000A(g 3) ; causes error, as expected\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> g)\u000A(g 3) ; returned 3 before the bug fix; after fix, causes error as expected</code> \u000A \u000A <p>A hard error was possible when attempting to include an uncertified book \u000A containing <see topic=\"ACL2____EVENTS\">events</see> of the form <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"ACL2____LOCAL\">local</see> ...))</v>.  This has \u000A been fixed.  Thanks to Sol Swords for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>Fixed a bug in the heuristic improvement described for Version_3.6 (see \u000A <see topic=\"ACL2____NOTE-3-6\">note-3-6</see>) as ``We simplified induction schemes....''  The bug had \u000A prevented, in unusual cases such as the following (notice the impossible \u000A case), a proof by induction.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (a x)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       (<see topic=\"COMMON-LISP____CASE\">case</see> a\u000A         (0 (foo (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A         (1 (foo (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A         (0 (foo a (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))))))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:type-prescription foo)))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> (foo a x)))</code> \u000A \u000A <p>Macro <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> did not work with an <v>:evisc-tuple</v> argument. \u000A This has been fixed by changing <v>cw-gstack</v> so that it now evaluates its \u000A arguments.  Thanks to Sol Swords for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> during the printing of failure messages \u000A for termination proofs.</p> \u000A \u000A <p>Fixed a bug in the handling of <v>#.</v> (see <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>). \u000A Thanks to Bob Boyer for bringing this bug to our attention.</p> \u000A \u000A <p>Replaced a hard Lisp error with a clean error, in certain cases that a \u000A <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> value is erroneously supplied as a non-<v>nil</v> atom. \u000A Example: <v>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x) :hints 3)</v>.</p> \u000A \u000A <p>Fixed a bug in the interaction of function tracing with conversion of a \u000A function from <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  The \u000A following example illustrates what had been wrong.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(f 3) ; raw Lisp hard error\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> f)\u000A(f 3) ; raw Lisp hard error (still)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)) ; upgrade f to :logic mode\u000A(f 3) ; clean guard violation; f is no longer traced\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see>) ; uh oh - f is shown as traced\u000A(<see topic=\"ACL2____UNTRACE_42\">untrace$</see> f)\u000A(f 3) ; OUCH: hard Lisp error because old :program mode definition of\u000A      ; the executable counterpart (sometimes called *1*f) was restored!</code> \u000A \u000A <p>Made a fix so that when building ACL2 with `make' option \u000A <v>ACL2_SAFETY=3</v>, there will no longer be any safety-0 compiled code \u000A generated.  Thanks to Gary Byers for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in the handling of <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> that generate custom \u000A keyword hints (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>) involving the variable \u000A <v>stable-under-simplificationp</v>.  Thanks to Ian Johnson for bringing this \u000A bug to our attention with explanation that included a helpful example, \u000A included as comment in the ACL2 source code for function \u000A <v>apply-override-hint</v>.</p> \u000A \u000A <p>The <v>saved_acl2</v> script in CLISP could contain unexpected characters \u000A where simple newlines were expected.  Dave Greve found this in a Cygwin \u000A environment on Windows.  Thanks to Dave for reporting this bug and \u000A experimenting with a fix, and thanks to the CLISP folks for providing helpful \u000A information.</p> \u000A \u000A <p>Fixed a bug that could make <v>:</v><tt><see topic=\"ACL2____OOPS\">oops</see></tt> cause an error.  Also, the \u000A <tt><see topic=\"ACL2____OOPS\">oops</see></tt> command can no longer take you back before a <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt> event.</p> \u000A \u000A <p>(GCL only) Fixed a bug that could occur when calling <v>trace</v> in raw Lisp \u000A in GCL.</p> \u000A \u000A <p>Proof summaries have been improved, so that they account for <see topic=\"ACL2____RUNE\">rune</see>s \u000A used in rewriting that takes place when generating goals to be proved in a \u000A forcing round.  Thanks to Jared Davis for sending us an example illustrating \u000A this issue.</p> \u000A \u000A <p>Fixed a bug that (at least in CCL) could put extra backslashes (`<v>\\</v>') \u000A in a pathname that ACL2 writes out to the executable script created by a \u000A build.  Thanks to Gary Byers for explaining that the CCL behavior is legal \u000A (for our previous use of Common Lisp function <v>merge-pathnames</v>).</p> \u000A \u000A <p>We closed some holes in the handling of trust tags (also known as \u000A ``ttags''; see <see topic=\"ACL2____DEFTTAG\">defttag</see>) by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  The following example \u000A illustrates this rather subtle situation.  Consider the following book.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____ER-PROGN\">er-progn</see>\u000A  (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A   ()\u000A   (<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)\u000A   (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> &quot;Imagine something bad here!&quot;))\u000A  (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :some-value)))\u000A :check-expansion t)</code> \u000A \u000A <p>Formerly, the following commands succeeded.</p> \u000A \u000A <code>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;test3&quot; 0 t :ttags :all)\u000A:u\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;test3&quot; :ttags nil)</code> \u000A \u000A <p>But because of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> keyword argument <v>:check-expansion t</v>, \u000A we know that the event <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :foo)</v> is evaluated by the above <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form, and hence the <v>:ttags</v> argument of <v>include-book</v>, \u000A above, should have specified <v>:foo</v>.  The problem was that <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> \u000A forms evaluated during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion did not contribute to the \u000A trust tag information stored in the book's <see topic=\"ACL2____CERTIFICATE\">certificate</see>.  Note: Because \u000A of this change, one should avoid using <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> with \u000A <v>:check-expansion t</v> when the expansion would introduce a <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> \u000A event during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> but not <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> time.  For an \u000A example illustrating this issue, see <see topic=\"ACL2____MAKE-EVENT-DETAILS\">make-event-details</see>, specifically \u000A the new version of the section labeled ``A note on ttags'' at the end of that \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.</p> \u000A \u000A <p>Closed a small loophole that had the potential, in rare circumstances, to \u000A violate atomicity of under-the-hood updates for ACL2 <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p>The following example was formerly allowed, but resulted in a \u000A guard-verified function (here, <v>g</v>) whose guard proof obligation is not a \u000A theorem outside the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event.  We now disallow <see topic=\"ACL2____GUARD\">guard</see> \u000A verification for functions introduced non-<see topic=\"ACL2____LOCAL\">local</see>ly inside an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event unless we determine that the proof obligations hold outside \u000A the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event as well.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((f (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A ;; ERROR!\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (f x)))\u000A   (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</code> \u000A \u000A <p>The use of <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> on <see topic=\"ACL2____STOBJ\">stobj</see> functions had potentially \u000A caused a hard Lisp error; for example, this could occur when <v>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> foo\u000A fld)</v> was followed by <v>:comp foop</v>.  This has been fixed.</p> \u000A \u000A <p>Fixed a bug that could cause a raw Lisp error when the first argument of \u000A <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt> is not a symbol.</p> \u000A \u000A <p>It had been possible to use the reserved keyword \u000A :computed-hints-replacement as the name of a custom keyword hint (see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>).  This has been fixed.  Thanks to Dave Greve, who \u000A pointed out a confusing hint error message (which has also been fixed) that \u000A led us to this issue.</p> \u000A \u000A <p>Fixed a bug that could cause a hard Lisp error, instead of a graceful ACL2 \u000A error, if keyword <v>:backchain-limit-lst</v> in a rule class is given a cons \u000A that is not a true list, such as <v>(1 . 1)</v>.</p> \u000A \u000A <p>Eliminated an error that could occur when redefining a function as a macro \u000A and then compiling, as in the example below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A:redef!\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (x) x)\u000A:comp t</code> \u000A \u000A <p>Thanks to Eric Smith for sending the above example in his bug report.</p> \u000A \u000A <p>Fixed a bug that could result in an assertion when a <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> causes an error.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS AND RELATED INFRASTRUCTURE</b></p> \u000A \u000A <p>See the <a href='http://code.google.com/p/acl2-books/source/list'>log \u000A entries</a> for a record of books changed or added since the preceding \u000A release, with log entries.</p> \u000A \u000A <p>We note in particular the new <v>system/</v> directory, which begins to \u000A specify ACL2 system code in anticipation of opening the architecture of ACL2 \u000A (see <see topic=\"ACL2____DEFATTACH\">defattach</see> for a relevant tool).  Some system functions were \u000A changed slightly (but with the expectation of not generally affecting ACL2 \u000A behavior) in support of the development of this directory.  Those interested \u000A in contributing to further such efforts are invited to contact the ACL2 \u000A implementors.</p> \u000A \u000A <p>New utilities have been provided for certifying most of the distributed \u000A books with more `make'-level parallelism.  For example, we have obtained close \u000A to a 12x reduction in time by using `<v>make -j 24 regression-fast</v>' on a \u000A 24-processor machine.  For more information see <v>books/make-targets</v>, or to \u000A include the <v>books/workshops</v> in the regression run, see \u000A <v>books/regression-targets</v>.  Thanks to Sol Swords for providing these nice \u000A utilities.</p> \u000A \u000A <p>The top-level makefile, <v>GNUmakefile</v>, has been fixed so that the build \u000A processes (which are inherently sequential) will ignore the <v>-j</v> option of \u000A `make'.  Note that regressions can still, however, be done in parallel, as the \u000A <v>-j</v> option will be passed automatically to the appropriate `make' \u000A command.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p><b>EXPERIMENTAL VERSIONS</b></p> \u000A \u000A <p>The HONS version, supported primarily by Bob Boyer and Warren Hunt (see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>), has undergone numerous improvements.  For \u000A example, keyword argument <v>:FORGET</v> is now supported when calling <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> from within the ACL2 loop, and system function <v>worse-than</v> is \u000A <see topic=\"ACL2____MEMOIZE\">memoize</see>d with the <v>:condition</v> that both terms are function \u000A applications (clearing the memo-table after each prover invocation).  Thanks \u000A to Jared Davis and Sol Swords for investigating the memoization of \u000A <v>worse-than</v>, and with suitable <v>condition</v>.  Thanks also to Jared Davis \u000A for contributing structural modifications to the implementation of <tt><see topic=\"ACL2____HONS\">hons</see></tt>.</p> \u000A \u000A <p>David Rager contributed modifications to the parallel version (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>), which include taking advantage of atomic increments available \u000A at least since Version 1.0.21 of SBCL and Version 1.3 of CCL.</p>"],

 "ACL2____NOTE-2-9-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____NOTE-2-9-1\">note-2-9-1</see> for other changes since the last \u000A non-incremental release \u000A (Version_2.9).</p> \u000A \u000A <p>There was a bug in non-linear arithmetic (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>) \u000A that caused the following error:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;rtl/rel4/lib/top&quot; :dir :system)\u000A....\u000AACL2 !&gt;(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see> t)\u000A T\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (bvecp a 77)\u000A               (bvecp b 50))\u000A          (bvecp (fl (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP_____A2\">*</see> a b) (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 23)))\u000A                 104))\u000A :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> bvecp))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000ABy the simple :definition BVECP, the :executable-counterparts of EXPT\u000Aand UNARY-/ and the simple :rewrite rule ASSOCIATIVITY-OF-* we reduce\u000Athe conjecture to\u000A\u000AGoal'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> A)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 A)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> A 151115727451828646838272)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> B)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 B)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> B 1125899906842624))\u000A         (BVECP (FL (<see topic=\"COMMON-LISP_____A2\">*</see> A B 1/8388608)) 104)).\u000A\u000AHARD ACL2 ERROR in VARIFY:  This should not have happened.  The supposed\u000Avariable, '1/8388608, is instead a constant.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Thanks to Robert Krug for providing a fix for the above error.</p> \u000A \u000A <p>Guard-checking was being inhibited (since v2-9) for calls of built-in \u000A primitives on explicit values, e.g., <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v>.  This has been fixed.</p> \u000A \u000A <p>Guard-related warnings could be printed during proofs (this bug was \u000A introduced in Version_2.9.1).  These warnings have been eliminated.</p> \u000A \u000A <p>Compound-recognizer rules <v>natp-compound-recognizer</v> and \u000A <v>posp-compound-recognizer</v> are now built into ACL2 for predicates <tt><see topic=\"ACL2____NATP\">natp</see></tt> and <tt><see topic=\"ACL2____POSP\">posp</see></tt>, and hence have been deleted from book \u000A <v>natp-posp.lisp</v> (where they were called <v>natp-cr</v> and <v>posp-cr</v>, \u000A respectively).</p> \u000A \u000A <p>The function <v>file-clock-p</v>, which recognizes a component of the ACL2 \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>, is now defined using <tt><see topic=\"ACL2____NATP\">natp</see></tt> instead of <tt><see topic=\"COMMON-LISP____INTEGERP\">integerp</see></tt>. \u000A Thanks to Jared Davis for suggesting this change.  (Technical explanation \u000A about functions in ACL2 source file <v>axioms.lisp</v>: With a <v>file-clock</v> \u000A of -1, the call of <v>make-input-channel</v> in <v>open-input-channel</v> will \u000A create a channel that can't be closed; see the guard of \u000A <v>close-input-channel</v>.)</p> \u000A \u000A <p>(Allegro CL users only) Support is now provided for building an Allegro CL \u000A application, provided you have an Allegro CL dynamic runtime license.  (Our \u000A belief is that with such a license, many users can use the same application, \u000A rather than each user needing a separate license.)  See new GNUmakefile target \u000A <v>allegro-app</v> and file <v>build-allegro-exe.cl</v> for more information.</p> \u000A \u000A <p>The new home page now contains a link to a new page \u000A <v>other-releases.html</v>, which contains information about other ACL2 \u000A releases.  (This is in one's local home page, but may not show up on the \u000A central ACL2 home page until the next non-incremental release.)  Thanks to \u000A Warren Hunt for suggesting this addition.</p> \u000A \u000A <p>We thank Erik Reeber for suggesting a solution to output redirection using \u000A <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>, which we have described at the end of its documentation.</p> \u000A \u000A <p>A new documentation topic fixes the flawed argument for conservativity of \u000A the <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> event that appears in Appendix B of Kaufmann and Moore's \u000A paper, ``Structured Theory Development for a Mechanized Logic'' (Journal of \u000A Automated Reasoning 26, no. 2 (2001), pp. 161-203).  See <see topic=\"ACL2____CONSERVATIVITY-OF-DEFCHOOSE\">conservativity-of-defchoose</see>.  Thanks to John Cowles and Ruben Gamboa for \u000A helpful feedback on drafts of this note.</p> \u000A \u000A <p>The solution to exercise 6.15 in <v>books/textbook/chap6/solutions.txt</v> \u000A has been fixed.  Thanks to Aaron Smith for pointing out the problem.</p> \u000A \u000A <p>A new documentation topic <see topic=\"ACL2____DEFUN-SK-EXAMPLE\">defun-sk-example</see> gives a little more help \u000A in using <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> effectively.  Thanks to Julien Schmaltz for \u000A presenting this example as a challenge.</p> \u000A \u000A <p>(GCL only) There is now a way to speed up GCL builds of ACL2, at the cost \u000A of perhaps a percent or so in performance of the resulting image.  Using \u000A `make' one supplies the following.</p> \u000A \u000A <code>LISP='gcl -eval &quot;(defparameter user::*fast-acl2-gcl-build* t)&quot;</code> \u000A \u000A <p>Various makefiles have been improved in several ways.</p> \u000A \u000A <blockquote><p>(1) Parallel book certification, using GNU make's <v>-j</v> \u000A option, can be used.</p> \u000A \u000A <p>(2) Book certifications now stops at the first failure if \u000A <v>books/Makefile</v> or <v>books/Makefile-generic</v> is used, and returns \u000A non-zero exit status.  However, the various make targets in the ACL2 source \u000A directory (<v>regression</v>, <v>certify-books</v>, etc.) still continue past \u000A failures unless you provide <v>ACL2_IGNORE=' '</v> on the `make' command \u000A line.</p> \u000A \u000A <p>(3) The build process has been modified (file <v>GNUmakefile</v>) so that it \u000A stops upon a failed compile or a failed initialization.</p> \u000A \u000A <p>(4) The automatic dependency generation (from ``<v>make dependencies</v>'' \u000A has been improved so that commands of the form <v>(<see topic=\"ACL2____LD\">ld</see> &quot;my-book.lisp&quot;)</v> in \u000A <v>.acl2</v> files cause the appropriate depedencies to be \u000A generated.</p></blockquote> \u000A \u000A <p>Thanks to comments from several users that led to the above Makefile \u000A improvements: Ray Richards, Doug Harper, and the Rockwell ACL2 users for (1) \u000A and (2) (and inspiring (4)), and David Rager for (2) and (3).  In particular, \u000A Doug Harper sent a replacement for the <v>.date</v> mechanism, which was \u000A interfering with <v>make -n</v>; so, these files are no longer written.</p> \u000A \u000A <p>A mechanism has been added for saving output.  In particular, you can now \u000A call <tt><see topic=\"ACL2____LD\">ld</see></tt> on a file with output turned off, for efficiency, and yet when \u000A a proof fails you can then display the proof attempt for the failed (last) \u000A event.  See <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>.  Another new command &mdash; see <see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see> &mdash; causes subgoal numbers to be printed during \u000A proof attempts when output is inhibited.</p> \u000A \u000A <p>Documentation has been added for using ACL2's makefile support to automate \u000A the certification of collections of books.  See <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt> that was causing hard Lisp \u000A errors.</p> \u000A \u000A <p>Improved <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> to allow a <v>:frames</v> argument to specify a \u000A range of one or more frames to be printed.  see <see topic=\"ACL2____CW-GSTACK\">cw-gstack</see>.</p> \u000A \u000A <p>Fixed a bug in <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>forwardchain</v>.  Thanks to \u000A Ming-Hsiu Wang for bringing this bug to our attention.</p> \u000A \u000A <p>We have provided a mechanism for saving an executable image.  See <see topic=\"ACL2____SAVING-AND-RESTORING\">saving-and-restoring</see> and see <see topic=\"ACL2____SAVE-EXEC\">save-exec</see>.  We have eliminated obsolete \u000A functions <v>note-lib</v> and <v>make-lib</v>.</p> \u000A \u000A <p>Modified the <tt><see topic=\"ACL2____GROUND-ZERO\">ground-zero</see></tt> <see topic=\"ACL2____THEORY\">theory</see> so that it contains all of \u000A the built-in rules (in ACL2 source file <v>axioms.lisp</v>).  It had formerly \u000A failed to include rules from some definitions and theorems near the end of \u000A <v>axioms.lisp</v>.</p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see></tt>, allows the enforcement of \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, and most other events during book development. \u000A See <see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see>.</p> \u000A \u000A <p>A bug has been fixed that had allowed <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> to \u000A cause a hard Lisp error when calling <tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt> on ill-formed \u000A theories after, for example:</p> \u000A \u000A <code>:set-guard-checking nil\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> '((:rewrite no-such-rule))\u000A                           (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> 'ground-zero)))</code> \u000A \u000A <p>The handling of <see topic=\"ACL2____GUARD\">guard</see> checking has been modified somewhat in a way \u000A that should only very rarely affect users.  (An ``Essay on Guard Checking'' in \u000A the ACL2 source code explains this point to anyone interested in \u000A implementation details.)</p> \u000A \u000A <p>(GCL ONLY) Removed the -dir setting in the ACL2 wrapper script for GCL. \u000A This should generally have no effect for most users, but it eliminates a \u000A potential source of error down the road.</p> \u000A \u000A <p>Several interesting new definitions and lemmas have been added to the rtl \u000A library developed at AMD, and incorporated into <v>books/rtl/rel4/lib/</v>. \u000A Other book changes include a change to lemma <v>truncate-rem-elim</v> in \u000A <v>books/ihs/quotient-remainder-lemmas.lisp</v>, as suggested by Jared \u000A Davis.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see></tt> may now be referred to in <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> and <see topic=\"ACL2____HINTS\">hints</see>, thanks to a new <tt><see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see></tt> event.  Thanks to Jared Davis for this suggestion.</p> \u000A \u000A <p>ACL2 terms of the form <v>(<see topic=\"COMMON-LISP____IF\">if</see> p 'nil 't)</v> are now printed as <v>(<see topic=\"COMMON-LISP____NOT\">not</see> p)</v>, \u000A where in some setting they had been printed as <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p) t)</v>.  Thanks \u000A to Robert Krug for this improvement.</p> \u000A \u000A <p>(GCL ONLY) Added profiling support, based heavily on code supplied by Camm \u000A Maguire.  See file <v>save-gprof.lsp</v> for instructions.  Thanks to Camm, and \u000A also to David Hardin for inspiring this addition.</p> \u000A \u000A <p>Added support for preprocessing before printing (untranslating) a term. \u000A See <see topic=\"ACL2____USER-DEFINED-FUNCTIONS-TABLE\">user-defined-functions-table</see>, in particular the discussion of \u000A <v>untranslate-preprocess</v>.  Thanks to Jared Davis for inspiring this \u000A addition, and for providing a book that takes advantage of it \u000A (<v>books/misc/untranslate-patterns.lisp</v>).</p> \u000A \u000A <p>The documentation has been improved for explaining how <see topic=\"ACL2____RUNE\">rune</see>s are \u000A assigned; see <see topic=\"ACL2____RUNE\">rune</see>.  Thanks to Robert Krug for pointing out \u000A inaccuracies in the existing documentation.</p>"],

 "ACL2____NOTE-2-8-BUG-FIXES":[["Note-2-8"],"ACL2 Sources","ACL2","<p>We have fixed a soundness bug in the tautology checker's handling \u000A of expressions of the form <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____NOT\">not</see> x))</v>.  This bug has gone back at \u000A least as far as Version_2.4.  All of the regression tests passed after the \u000A fix, without modification.  So we hope that this bug has rarely bitten anyone. \u000A Thanks to Qiang Zhang for sending us a proof of <v>nil</v> that led us to this \u000A fix: <v>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____AND\">and</see> p q) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p) (<see topic=\"COMMON-LISP____NOT\">not</see> q)))))</v>.  And thanks to \u000A Matyas Sustik for an observation that led to an improvement of our initial \u000A fix.</p> \u000A \u000A <p>The preceding version (2.7) introduced a soundness bug in handling of ACL2 \u000A <see topic=\"ACL2____ARRAYS\">arrays</see>, in which functions <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> were \u000A returning the input alist rather than compressing it appropriately.  Here is a \u000A proof of <v>nil</v> that no longer succeeds, based on a bug report from Warren \u000A Hunt, who we thank for bringing this problem to our atttention.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((ar2 (<see topic=\"ACL2____ASET1\">aset1</see> 'my-array ar1 3 10))\u000A              (ar3 (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'my-array ar2))\u000A              (ar4 (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> ar2)))\u000A              (ar5 (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'my-array ar4)))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> ar2 ar4)\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> ar3 ar5)))))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use\u000A           ((:instance bad\u000A                       (ar1 (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'my-array\u000A                                       '((3 . 5)\u000A                                         (:HEADER :DIMENSIONS (5)\u000A                                                  :MAXIMUM-LENGTH 6\u000A                                                  :DEFAULT 0\u000A                                                  :NAME MY-ARRAY)))))))))</code> \u000A \u000A <p>On a related note, a new function <tt><see topic=\"ACL2____FLUSH-COMPRESS\">flush-compress</see></tt> can be used for \u000A subtle control of under-the-hood raw Lisp support for fast array access, \u000A although we expect it to be very rare that users need this extra support.</p> \u000A \u000A <p>Previous versions have had two soundness bugs that can occur when using the \u000A <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>:</p> \u000A \u000A <blockquote><p>o The first bug pertains to the <v>expand</v> command, and hence \u000A <v>x</v> and <v>x-dumb</v> commands (which call <v>expand</v>); see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  The bug can occur when applying the above commands \u000A when the current term is a call of a constrained function symbol for which \u000A there is a <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule.  Now, the <v>expand</v> command will \u000A succeed only when the function symbol of the current term is a defined \u000A function symbol, in which case the original definition is always used, in \u000A analogy to how the <v>:expand</v> hint works in the prover; see <see topic=\"ACL2____HINTS\">hints</see>. \u000A Thanks to John Erickson for sending an example that led us to wonder if there \u000A might be a soundness problem.</p> \u000A \u000A <p>o The second bug pertains to the <v>s</v> command (and commands that call it, \u000A e.g., <v>s-prop</v>).  The proof-checker forms a context out of the top-level \u000A hypotheses and the <v>if</v>-terms governing the current term.  If there is a \u000A contradiction in the top-level hypotheses, the proof-checker can appropriately \u000A consider the goal to be proved, and it does so.  But formerly, the criterion \u000A was weaker: the contradiction could involve the combination of the top-level \u000A hypotheses and <v>if</v>-term governors.  Thanks to Rob Sumners for noticing \u000A this bug.</p></blockquote> \u000A \u000A <p>A soundness bug could be provoked in some Lisps by applying <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> \u000A to the empty string.  This has been disallowed.</p> \u000A \u000A <p>We fixed a soundness bug related to packages caused by a failure to track \u000A axioms introduced <tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly on behalf of <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events.  See \u000A <see topic=\"ACL2____HIDDEN-DEATH-PACKAGE\">hidden-death-package</see>.</p> \u000A \u000A <p>We fixed a soundness bug caused by a failure to check that a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule can be processed when proofs are skipped or under a \u000A <tt><see topic=\"ACL2____DEFEQUIV\">defequiv</see></tt> event.  The former case can occur when processing an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event, where the rule could depend on a \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <v>:</v><tt><see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see></tt> rule preceding the proposed \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule under the same <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  See <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see> for such an \u000A example.</p> \u000A \u000A <p>We fixed a potential soundness bug relating to reclassifying a \u000A <v>:program</v> mode function to <v>:logic</v> mode (as done by <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> or the submission of an appropriate ``redundant'' \u000A definition) without adequate checking that <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> usage was identical. \u000A Allegedly redundant definitions must now preserve the <v>stobjs</v> declaration \u000A as well as the formals, body, guard and type declarations.  We thank Vernon \u000A Austel for pointing out this problem.</p> \u000A \u000A <p>It was possible to get a raw Lisp error by introducing a <tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly \u000A defined function with <see topic=\"ACL2____GUARD\">guard</see> verification inhibited and then \u000A subsequently introducing the same definition non-locally without that \u000A inhibition.  The following example will clarify.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> nil\u000A  (<see topic=\"ACL2____LOCAL\">local</see>\u000A    (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards nil)) (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A; The following causes a raw lisp error because ACL2 runs the Common Lisp\u000A; definition of foo, because it thinks that foo's guard of t was verified.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo 3) xxx))</code> \u000A \u000A <p>Thanks to Jared Davis for bringing this problem to our attention.  We are \u000A particularly grateful to Jared because his example exploited this bug by \u000A applying it to a function defined using <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (introduced in this same \u000A version, 2.8), in order to prove <v>nil</v>!</p> \u000A \u000A <p>The sort of error message shown below can legitimately occur when \u000A certifying a book in a certification world where there was an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> command with a relative pathname (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>). \u000A However, it was occurring more often than necessary.  This has been fixed.</p> \u000A \u000A <blockquote><p>ACL2 Error in (CERTIFY-BOOK \"foo\" ...): The certification \u000A world has include-book commands for book \"bar\" that correspond to different \u000A full pathnames, namely \"/u/dir1/bar\" and \"/u/dir2/bar\".  ACL2 cannot \u000A currently certify a book in such a world.  To work around this problem, use an \u000A absolute pathname for at least one of these books (see :DOC \u000A pathname).</p></blockquote> \u000A \u000A <p>Bugs were fixed in <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt>, in particular related to the use of \u000A values <v>:all</v>.  Also, documentation for <v>with-output</v> has been improved. \u000A Thanks to Vernon Austel for pointing out the bugs.</p> \u000A \u000A <p>Fixed a lisp error occurring when <v>bash</v> proof-checker command was given \u000A illegal syntax, e.g., <v>(bash ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see>\u000A binary-append))))</v> instead of <v>(bash (&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see>\u000A binary-append)))</v>.</p> \u000A \u000A <p>We added an appropriate guard to <tt><see topic=\"ACL2____FIND-RULES-OF-RUNE\">find-rules-of-rune</see></tt>, which will \u000A avoid hard lisp errors when this function is called on non-<see topic=\"ACL2____RUNE\">rune</see> \u000A arguments.  Thanks to Eric Smith for pointing out this issue.</p> \u000A \u000A <p>It was possible for a redundant <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) to leave a <see topic=\"ACL2____COMMAND\">command</see> in the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> \u000A and to cause (re-)loading of a compiled file.  These behaviors have been \u000A fixed.  In particular, if <v>book1</v> has already been included in the current \u000A ACL2 <see topic=\"ACL2____WORLD\">world</see> and <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;book1&quot;)</v> occurs in <v>book2</v>, then \u000A the compiled file for <v>book1</v> will not be loaded again when <v>book2</v> is \u000A included.  Thanks to Dave Greve for bringing our attention to these problems, \u000A and to Eric Smith for bringing up a special case earlier (where \"//\" \u000A occurred in the book name).</p> \u000A \u000A <p>The summary printed at the end of a proof had not listed <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> rules used in a proof.  This has been corrected.</p> \u000A \u000A <p>The use of proof trees in emacs redefined `<v>control-c control-c</v>' in \u000A such a way that in telnet mode, the telnet session was interrupted and perhaps \u000A could not be continued.  This has been fixed.</p> \u000A \u000A <p>Source function <v>load-theory-into-enabled-structure</v> contained a \u000A guard-violating call of <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.  Thanks to Vernon Austel for \u000A bringing this problem to our attention; even though this bug was benign (as he \u000A pointed out), we like keeping the source code free of guard violations.</p> \u000A \u000A <p>A number of proof-checker atomic macros caused a hard error when all goals \u000A have already been proved.  This has been fixed.  Thanks to John Erickson for \u000A sending an example of the issue.</p> \u000A \u000A <p>A bug has been fixed in <tt><see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see></tt>.  Formerly, a \u000A <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____GUARD\">guard</see> violation occurred when calling <tt><see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see></tt> more than once with first argument other than \u000A <v>:clear</v>.</p> \u000A \u000A <p>Defininitions of functions involving large constants could cause stack \u000A overflows.  This has been fixed, at least in some of the most egregious cases \u000A (by making a source function <v>fn-count-evg</v> tail-recursive).  Thanks to \u000A Jared Davis for bringing this problem to our attention.</p> \u000A \u000A <p>Evaluation of computed hints could cause stack overflows.  This has been \u000A fixed.  Thanks to Eric Smith for bringing this problem to our attention.</p> \u000A \u000A <p>Evaluation of <v>:</v><tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> on <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s is now fast even if the specified function is part of a very large \u000A <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nest.  Thanks to Eric Smith for sending an example \u000A showing that this wasn't always the case.</p> \u000A \u000A <p>Fixed a bug in <v>books/bdd/cbf.lisp</v> that was causing certification of \u000A distributed bdd books to fail when the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>) differs from the current working directory.  Thanks to Scott Guthery for \u000A bringing this bug to our attention and supplying a helpful log.</p> \u000A \u000A <p>Duplicate rule names have been eliminated from warnings generated upon the \u000A use of enabled <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules. \u000A Thanks to Eric Smith for pointing out this problem.</p> \u000A \u000A <p>The trace utilities (see <see topic=\"COMMON-LISP____TRACE\">trace</see>), as modified for GCL and Allegro \u000A Common Lisp, had failed to show more than the first return value for so-called \u000A ``<v>*1*</v>'' functions (essentially, <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> functions) \u000A when they were returning multiple values (via <see topic=\"ACL2____MV\">mv</see>).  This has been \u000A fixed.  Thanks to Erik Reeber for pointing out this problem.  Also, it is now \u000A possible to refer to <v>arglist</v> in <see topic=\"ACL2____TRACE_42\">trace$</see> forms when ACL2 is built on \u000A GCL, not just when ACL2 is built on Allegro Common Lisp.</p> \u000A \u000A <p>Uses of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> introduced during proofs by failed attempts to \u000A evaluate constrained functions (see <see topic=\"ACL2____HIDE\">hide</see>) are now tracked, so that the \u000A <see topic=\"ACL2____RUNE\">rune</see> <v>(:DEFINITION HIDE)</v> will show up in the summary.</p> \u000A \u000A <p>The following bug, introduced back in Version 2.7, has been fixed.  The bug \u000A applied only to GCL and may well not have affected anyone.  But the function \u000A proclamation computed by ACL2 for compilation usually had an output type of \u000A <v>nil</v> where it should have been <v>t</v>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____GC_42\">gc$</see></tt> had a bug exhibited when it was supplied one or more \u000A arguments.  This has been fixed.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFABBREV\">defabbrev</see></tt> broke when supplied a string and no \u000A documentation, e.g., <v>(<see topic=\"ACL2____DEFABBREV\">defabbrev</see> foo () &quot;&quot;)</v>.  Thanks to Rob Sumners for \u000A noticing this problem and providing a fix, which we have incorporated.</p> \u000A \u000A <p>For ACL2 executables built on Allegro Common Lisp, a Lisp error occurred \u000A when <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> was called on other than a defined function symbol.  Now \u000A ACL2 prints a more useful error message.</p> \u000A \u000A <p>The proof-checker no longer accepts a <v>(</v><tt><see topic=\"ACL2____VERIFY\">verify</see></tt><v>)</v> command \u000A when some function symbol in the original goal no longer exists in the current \u000A ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.  Thanks to John Erickson for bringing this issue to \u000A our attention.</p> \u000A \u000A <p>The function <v>ld-redefinition-action</v> may now be called by the user. \u000A Thanks to Vernon Austel for suggesting that we remove this symbol from the \u000A list of so-called untouchables.</p> \u000A \u000A <p>The handling of free variables in hypotheses (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) of \u000A rewrite and linear rules had a bug that prevented some proofs from going \u000A through.  Here is a simple example, essentially provided by Diana Moisuc, who \u000A we thank for bringing this issue to our attention.  The proof of the <tt><see topic=\"ACL2____THM\">thm</see></tt> below had failed, but now will succeed.  This particular bug prevented, \u000A for example, the <v>:all</v> behavior from occurring when the first hypothesis \u000A of the rule does not have free variables.  NOTE: Now that this bug has been \u000A fixed, you may find some proofs running much more slowly than before.  You can \u000A use <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> to locate rules that are slowing down your \u000A proofs because of excessive attention to free variables, and then execute \u000A <tt><see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see></tt> for those rules (or, just change the rules \u000A themselves to specify <v>:once</v> in the <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo1 (<see topic=\"COMMON-LISP_____A2\">*</see> ) =&gt; *)\u000A(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> aux-foo1\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a)\u000A                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo1 0)  (<see topic=\"COMMON-LISP____LIST\">list</see> 0 i)))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo1 a) (<see topic=\"COMMON-LISP____LIST\">list</see> 0 (<see topic=\"COMMON-LISP_____B2\">+</see> a i))))\u000A   :rule-classes ((:rewrite :match-free :all))))\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a)\u000A               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo1 0) (<see topic=\"COMMON-LISP____LIST\">list</see> 0 i)))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo1 a) (<see topic=\"COMMON-LISP____LIST\">list</see> 0 (<see topic=\"COMMON-LISP_____B2\">+</see> a i)))))</code> \u000A \u000A <p>Formerly, creation of large arrays could cause an error in the underlying \u000A Common Lisp implementation without helpful messages for the user.  Now, we \u000A check Common Lisp restrictions on arrays and print a helpful error message if \u000A they are violated, namely: each dimension must be less than the value of \u000A Common Lisp constant <v>array-dimension-limit</v>, and the product of the \u000A dimensions must be less than the value of Common Lisp constant \u000A <v>array-total-size-limit</v>.  Thanks to Warren Hunt for bringing this issue to \u000A our attention.  Note: this change also removes a former restriction of <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> array fields to size smaller than 2^28-1, provided the underlying Lisp \u000A can support larger arrays.</p> \u000A \u000A <p>The <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see> in the current logical <see topic=\"ACL2____WORLD\">world</see> were ignored \u000A by <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.  This has been fixed.  Thanks to Jared Davis for \u000A pointing out this bug and sending a helpful example.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____BRR\">brr</see></tt> mechanism has been cleaned up in order to avoid hard errors \u000A and infinite loops that can arrive when typing interrupts (<v>control-c</v>) or \u000A end-of-files (<v>control-d</v>) inside the <tt><see topic=\"ACL2____BRR\">brr</see></tt> loop.  Thanks to Dave \u000A Greve, Olga Matlin, Eric Smith, and Serita Van Groningen for bringing this \u000A issue to our attention.  As a byproduct, if you type <v>control-d</v> (or if \u000A inside emacs, <v>control-c control-d</v>), you may now quit entirely out of ACL2 \u000A and lisp (see <see topic=\"ACL2____GOOD-BYE\">good-bye</see>) in some cases where you formerly would not \u000A have, for example when sitting at the ACL2 prompt (which formerly, in Allegro \u000A Common Lisp for example, would merely take you into raw Lisp rather than \u000A quitting everything).</p> \u000A \u000A <p>We have eliminated structural flaws in the HTML documentation pages that \u000A could make them unreadable in some browsers.  Thanks to Bill Young for \u000A bringing this issue to our attention and to Joe Hendrix for diagnosing the \u000A problem.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> could run very slowly after many instructions in a \u000A given session.  This has been fixed; thanks to Art Flatau for bringing this \u000A problem to our attention.  (Implementation detail: We now keep tag-trees \u000A duplicate-free when we accumulate them into state.  This change could have \u000A minor speed advantages for some top-level proofs too, not just in the \u000A proof-checker.)</p> \u000A \u000A <p>The printing of accesses to stobjs using nth or update-nth has been done \u000A using symbolic constants since ACL2 Version_2.6.  However, there was a bug \u000A that prevented this feature from working for <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt> except at a \u000A top-level call.  This has been fixed.  Thanks to Julien Schmaltz for bringing \u000A this problem to our attention.  For example, consider these events:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st field0 field1)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> 1 (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 0 17 st)) (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> xxx yyy)))\u000A     :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> nth update-nth))))</code> \u000A \u000A <p>Before the fix, the proof attempt of the above silly thm printed the \u000A following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 (<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> *FIELD0* 17 ST))</code> \u000A \u000A <p>After the fix, we instead see the following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NTH\">NTH</see> *FIELD1* (<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> *FIELD0* 17 ST))</code> \u000A \u000A <p>It is now possible to certify and subsequently include <see topic=\"ACL2____BOOKS\">books</see> that \u000A require guard-checking to be off.  For example, the book can contain the form \u000A <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *silly* (<see topic=\"COMMON-LISP____CAR\">car</see> 3))</v> even though <v>3</v> fails to satisfy the guard \u000A of <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>.  Formerly, it was necessary to execute <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> nil</v> before a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> in order for such a form to be handled without error.  Thanks to \u000A Hanbing Liu for bringing this problem to our attention.</p> \u000A \u000A <p>Fixed a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> bug that could cause probably cause strange \u000A error, ``Attempt to access the plist field''.  Thanks to Bill Young for \u000A bringing this problem to our attention.</p> \u000A \u000A <p>Fixed a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> bug that was failing to record applications of \u000A rewrite rules using the proof-checker's <v>:rewrite</v> command, causing the \u000A proof summary to omit mention of that rule (for example, when using the \u000A proof-checker's <v>:exit</v> command to generate an <v>:instructions</v> hint). \u000A Thanks to Bill Young for pointing out this bug.</p> \u000A \u000A <p>Modernized some of the proof-tree emacs and infix printing stuff, thanks to \u000A suggestions made by Camm Maguire.</p>"],

 "ACL2____NOTE-2-7-NEW-FUNCTIONALITY":[["Note-2-7"],"ACL2 Sources","ACL2","<p>ACL2 now has a more powerful technique for relieving a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule's hypothesis that contains free \u000A variables.  A new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> section has been written describing the \u000A handling free variables in rules; see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>.  In brief, the \u000A primary change is that when a free-variable match for the current hypothesis \u000A fails to allow subsequent hypotheses to be relieved, then additional matches \u000A may be attempted until they have all been tried.  Also see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> (discussion of <v>:match-free</v>).  Also see <see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see>, see <see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see>, and see <see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> for interfaces provided to the user for controlling \u000A the way ACL2 deals with free variables in hypotheses.  We thank Rob Sumners \u000A for several helpful discussions about the designs of those interfaces, as well \u000A as Eric Smith and Robert Krug for helpful related discussions.  Robert Krug \u000A also found a performance bug in a preliminary version, for which we are \u000A grateful.</p> \u000A \u000A <p>WARNING: Book certification attempts may take much longer now that, by \u000A default, ACL2 looks for more free variable matches (see paragraph just above). \u000A You can get the old behavior by inserting the form</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :once)</code> \u000A \u000A <p>just after the initial <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> form.  However, rules from \u000A included books that have free variables can still slow down certification. \u000A This can be fixed by inserting</p> \u000A \u000A <code>(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t)</code> \u000A \u000A <p>before the first event in the file that generates a proof.</p> \u000A \u000A <p><see topic=\"ACL2____FORWARD-CHAINING\">Forward-chaining</see> has been made more powerful in the presence of free \u000A variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), thanks to a contribution by Erik \u000A Reeber.  Both before and now, when an attempt is made to relieve (prove) a \u000A hypothesis of a <v>:forward-chaining</v> rule in the case that at least one \u000A variable in that hypothesis is not yet bound, ACL2 looks in the current \u000A context for an instance of that hypothesis.  If it finds one, then it binds \u000A the unbound variables and continues to the next hyopothesis.  What is new is \u000A that ACL2 can now looks for multiple instances of that hypothesis.  Consider \u000A the following example; an explanation is below.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> (((op * *) =&gt; *))\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op (x y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)))\u000A             (<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-op\u000A               (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op x y) (op y z)) (op x z))\u000A               :rule-classes :forward-chaining))\u000A\u000A; fails in Version_2.6; succeeds in in Version_2.7\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op a b) (op b c) (op b e)) (op a c)))</code> \u000A \u000A <p>Before Version_2.7, the proof of the <v>thm</v> above fails.  When the \u000A <v>:forward-chaining</v> rule <v>transitivity-of-op</v> binds <v>x</v> to <v>a</v> and \u000A <v>y</v> to <v>b</v>, it then looks for an instance of <v>(op y z)</v> in the \u000A current context, with <v>y</v> bound to <v>b</v> but <v>z</v> unbound.  It happens \u000A to find <v>(op b e)</v> before <v>(op b c)</v>, and it then adds <v>(op a e)</v> to \u000A the context.  But starting with Version_2.7, it continues to look for \u000A additional instances and finds <v>(op b c)</v> in the context as well, chaining \u000A forward to <v>(op a c)</v> and thus proving the theorem.</p> \u000A \u000A <p>A new macro, <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt>, provides a simple way to get much or most \u000A of the power of <see topic=\"ACL2____META\">meta</see>functions.  Thanks to Eric Smith for coming up with \u000A the idea and to Robert Krug for providing an implementation (which we modified \u000A only very slightly) and documentation. See <see topic=\"ACL2____BIND-FREE\">bind-free</see> and see <see topic=\"ACL2____BIND-FREE-EXAMPLES\">bind-free-examples</see>.</p> \u000A \u000A <p>With the addition of <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> (mentioned above), <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> \u000A has become a macro, although that change should be transparent to the user. \u000A More importantly, the argument of <v>syntaxp</v> may now refer to variables \u000A <v>mfc</v> and <v>state</v>, giving <v>syntaxp</v> some of the power of extended \u000A metafunctions; see <see topic=\"ACL2____SYNTAXP\">syntaxp</see> and see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>. \u000A Thanks to Robert Krug for implementing that extension.  Also, the argument of \u000A <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> may now include calls of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions. \u000A See <see topic=\"ACL2____SYNTAXP\">syntaxp</see> and see <see topic=\"ACL2____SYNTAXP-EXAMPLES\">syntaxp-examples</see> (thanks to Robert Krug for \u000A updating the former and creating the latter documentation).</p> \u000A \u000A <p>The linear-arithmetic decision procedure (see <see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see>) has \u000A now been extended so that ACL2 can reason about non-linear arithmetic as well \u000A (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see> for how to turn on this feature).  We thank \u000A Robert Krug for the initial implementation of this, and Eric Smith for finding \u000A a couple of bugs in it.</p> \u000A \u000A <p>Some <tt><see topic=\"COMMON-LISP____TRACE\">trace</see></tt> utilities have been made available in the ACL2 loop.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Function <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> (and also <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt>) calls the \u000A corresponding underlying Lisp routine <v>trace</v> (and <v>untrace</v>), which \u000A however continues (as it has for some time) to be enhanced for GCL and Allegro \u000A CL.</p> \u000A \u000A <p>o Macro <tt><see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see></tt> causes trace output to go to a specified \u000A file.  Macro <tt><see topic=\"ACL2____CLOSE-TRACE-FILE\">close-trace-file</see></tt> causes trace output to go to the screen \u000A (which is the default).</p> \u000A \u000A <p>o Macro <v>with-error-trace</v> (or, <v>wet</v> for short) causes a backtrace \u000A to be written out for many failures, including guard violations.  See <see topic=\"COMMON-LISP____TRACE\">trace</see>, see <see topic=\"ACL2____TRACE_42\">trace$</see>, and see :DOC <v>wet</v> [** NOTE: eliminated after \u000A Version 3.3].</p> \u000A \u000A </blockquote> \u000A \u000A <p>A new <tt><see topic=\"ACL2____THEORY\">theory</see></tt>, <tt><see topic=\"ACL2____MINIMAL-THEORY\">minimal-theory</see></tt> has been provided (see <see topic=\"ACL2____THEORIES\">theories</see>).  It can be particularly useful for speeding up proofs involving \u000A <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>New <tt><see topic=\"ACL2____EVENTS\">events</see></tt> <tt><see topic=\"ACL2____DEFUND\">defund</see></tt> and <tt><see topic=\"ACL2____DEFTHMD\">defthmd</see></tt> behave exactly like \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, respectively, except that these new events \u000A disable the new name.</p> \u000A \u000A <p>The new macro <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> can be used to suppress output that would \u000A normally result from evaluation of a form.</p> \u000A \u000A <p>The form <v>(</v><tt><see topic=\"ACL2____PSTACK\">pstack</see></tt><v>)</v> can give the user an idea of what the \u000A prover has been up to during a proof, or after a user-aborted proof. \u000A Moreover, by evaluating <v>(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> t)</v> (see <see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see>) \u000A one can get <see topic=\"COMMON-LISP____TRACE\">trace</see>-like information about prover functions, including \u000A time summaries, printed to the screen during a proof.  Thanks to Bill Legato \u000A and Robert Krug for initiating this work and to Robert for providing some \u000A initial implementation.</p> \u000A \u000A <p>The new command <v>:</v><tt><see topic=\"ACL2____COMP-GCL\">comp-gcl</see></tt> is identical in functionality, \u000A except that it always leaves <v>.c</v> and <v>.h</v> files when compiling in GCL. \u000A Thanks to Rob Sumners and Vernon Austel for suggesting such a capability.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____E_F2D\">e/d</see></tt> provides a convenient way to <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> some \u000A rules and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> others.  It was formerly in a book supplied with the \u000A distribution, <v>books/ihs/ihs-init.lisp</v>, written by Bishop Brock (who we \u000A thank for providing this useful macro).</p> \u000A \u000A <p>New distributed books include those in <v>books/ordinals/</v>, \u000A <v>books/rtl/rel3/</v>, and <v>books/misc/simplify-defuns.lisp</v> (which is \u000A documented in <v>books/misc/simplify-defuns.txt</v>).</p> \u000A \u000A <p>The <v>:expand</v> hint now accepts a special value, <v>:LAMBDAS</v>, that \u000A tells the ACL2 rewriter to expand all lambda applications (<tt><see topic=\"COMMON-LISP____LET\">let</see></tt> \u000A expressions).  See <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>A new function <tt><see topic=\"ACL2____ZPF\">zpf</see></tt> has been added as fast test against 0 for \u000A nonnegative fixnums.</p> \u000A \u000A <p>A new macro <tt><see topic=\"ACL2____GC_42\">gc$</see></tt> allows the user to call the garbage collector of \u000A the underlying Common Lisp.  Thanks to Rob Sumners for suggesting this \u000A feature.</p> \u000A \u000A <p>It is now possible to <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> <see topic=\"ACL2____SIMPLE\">simple</see> (abbreviation) rules. \u000A However, as a warning explains, they are still not considered monitored during \u000A preprocessing; see <see topic=\"ACL2____MONITOR\">monitor</see>.  Thanks to Robert Krug for providing this \u000A improvement.</p> \u000A \u000A <p>The second argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, if supplied, formerly had to \u000A be either <v>t</v> or a non-negative integer.  Now it can be the symbol <v>?</v>, \u000A in the <v>ACL2</v> package, indicating that the usual check should be suppressed \u000A on the number of commands that have been executed to create the world in which \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> was called.</p>"],

 "ACL2____NOTE-2-7":[["Release-notes"],"ACL2 Sources","ACL2","<p>The Version_2.7 notes are divided into the subtopics below.  Here \u000A we give only a brief summary of a few of the changes that seem most likely to \u000A impact existing proofs.  Not included in this brief summary, but included in \u000A the subtopics, are descriptions of improvements (including bug fixes and new \u000A functionality) that should not get in the way of existing proof efforts.</p> \u000A \u000A <p>In particular, please see <see topic=\"ACL2____NOTE-2-7-NEW-FUNCTIONALITY\">note-2-7-new-functionality</see> for discussion \u000A of a number of new features that you may find useful.</p> \u000A \u000A <p>Acknowledgements and elaboration, as well as other changes, can be found in \u000A the subtopics listed below.</p> \u000A \u000A <p>o Bug fixes (see <see topic=\"ACL2____NOTE-2-7-BUG-FIXES\">note-2-7-bug-fixes</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ Three soundness bugs were fixed.  These bugs were probably rarely hit, so \u000A users may well not notice these changes.</p> \u000A \u000A <p>+ <tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> now requires <v>:skip-proofs-ok t</v> (respectively, \u000A <v>:defaxioms-okp t</v>) if there are <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> (respectively, <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>) events in the book or any included sub-books.</p> \u000A \u000A <p>+ When <v>:by</v> hints refer to a definition, they now use the original body \u000A of that definition rather than the simplfied (``normalized'') body.</p> \u000A \u000A <p>+ When <tt><see topic=\"ACL2____LD\">ld</see></tt> is applied to a stringp file name, it now temporarily \u000A sets the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>) to the directory of that \u000A file while evaluating forms in that file.</p></blockquote> \u000A \u000A <p>o New functionality (see <see topic=\"ACL2____NOTE-2-7-NEW-FUNCTIONALITY\">note-2-7-new-functionality</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ ACL2 now works harder to apply <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules with free variables in the hypotheses.  See <see topic=\"ACL2____NOTE-2-7-NEW-FUNCTIONALITY\">note-2-7-new-functionality</see>, in particular its first two paragraphs, for \u000A details.  <see topic=\"ACL2____FORWARD-CHAINING\">Forward-chaining</see> also does more with free \u000A variables.</p></blockquote> \u000A \u000A <p>o Changes in proof engine (see <see topic=\"ACL2____NOTE-2-7-PROOFS\">note-2-7-proofs</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ Some prover heuristics have changed slightly.  Among other consequences, \u000A this can cause subgoal <see topic=\"ACL2____HINTS\">hints</see> to change.  For example, suppose that the \u000A Version_2.6 proof of a particular theorem generated \"Subgoal 2\" and \u000A \"Subgoal 1\" while Version_2.7 only generates the second of these.  Then a \u000A subgoal hint attached to \"Subgoal 1\" in Version_2.6 would have to be \u000A attached to \"Goal'\" in Version_2.7.  (See <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>.)  The full topic \u000A has details (see <see topic=\"ACL2____NOTE-2-7-PROOFS\">note-2-7-proofs</see>).</p></blockquote> \u000A \u000A <p>o Changes in rules and definitions (see <see topic=\"ACL2____NOTE-2-7-RULES\">note-2-7-rules</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ The package name of a generated variable has changed for <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt>.</p></blockquote> \u000A \u000A <p>o Guard-related changes (see <see topic=\"ACL2____NOTE-2-7-GUARDS\">note-2-7-guards</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ <tt><see topic=\"ACL2____GUARD\">Guard</see></tt> verification formerly succeeded in a few cases where it \u000A should have failed.</p> \u000A \u000A <p>+ Guards generated from type declarations now use functions \u000A <v>signed-byte-p</v> and <v>unsigned-byte-p</v>, now defined in source file \u000A <v>axioms.lisp</v> and formerly defined rather similarly under \u000A <v>books/ihs/</v>.</p></blockquote> \u000A \u000A <p>o Proof-checker changes (see <see topic=\"ACL2____NOTE-2-7-PROOF-CHECKER\">note-2-7-proof-checker</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ See the above doc topic.</p></blockquote> \u000A \u000A <p>o System-level changes (see <see topic=\"ACL2____NOTE-2-7-SYSTEM\">note-2-7-system</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ See the above doc topic.</p></blockquote> \u000A \u000A <p>o Other changes (see <see topic=\"ACL2____NOTE-2-7-OTHER\">note-2-7-other</see>):</p> \u000A \u000A <blockquote> \u000A \u000A <p>+ A new <tt><see topic=\"ACL2____TABLE\">table</see></tt>, <tt><see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see></tt>, takes the place of the \u000A handling of invisible functions in the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>,</p> \u000A \u000A <p>+ The <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> event has been modified so that the default \u000A action is an error rather than a warning.</p> \u000A \u000A <p>+ Proof output that reports destructor elimination no longer uses the word \u000A ``generalizing''.</p></blockquote> \u000A \u000A <p>Again, please proceed to the subtopics for more thorough release \u000A notes.</p>"],

 "ACL2____NESTED-STOBJS":[["Stobj"],"ACL2 Sources","ACL2","<p>For this topic we assume that you already understand the basics of \u000A single-threaded objects in ACL2.  See <see topic=\"ACL2____STOBJ\">stobj</see>, and in particular, see \u000A <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.  The latter topic mentions briefly that a stobj field can \u000A itself be a stobj or an array of stobjs.  That discussion is the subject of \u000A the present <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.</p> \u000A \u000A <p>Our presentation is in four sections.  First we augment the documentation \u000A for <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> by explaining how stobjs may be specified for fields in a \u000A new stobj definition.  Then we explain an aliasing problem, which accounts for \u000A a prohibition against making direct calls to accessors and updaters involving \u000A stobj fields of stobjs.  Next, we introduce an ACL2 primitive, <v>stobj-let</v>, \u000A which provides the only way to read and write stobj components of stobjs.  The \u000A final section provides precise documentation for <v>stobj-let</v>.</p> \u000A \u000A <p>See also ACL2 community book <v>demos/modeling/nested-stobj-toy-isa.lisp</v> \u000A for a worked example, which applies nested stobj structures to the problem of \u000A defining interpreters.  A variety of small additional examples may be found in \u000A ACL2 community book <v>misc/nested-stobj-tests.lisp</v>.  For further \u000A discussion, you are welcome to read the ``Essay on Nested Stobjs'', a long \u000A comment in ACL2 source file <v>other-events.lisp</v>.  However, this \u000A documentation topic is intended to be self-contained for those familiar with \u000A <see topic=\"ACL2____STOBJ\">stobj</see>s.</p> \u000A \u000A <h3>SECTION: Extension of <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> to permit <see topic=\"ACL2____STOBJ\">stobj</see>s within \u000A stobjs</h3> \u000A \u000A <p>Recall that the <v>:type</v> keyword of a <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> field descriptor \u000A can be a ``type-indicator'' that specifies the type of the field as a \u000A type-spec (see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>).  For example, the following specifies an \u000A integer field and a field that is an array of bytes.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st\u000A  (int-field :type integer :initially 0)\u000A  (ar-field :type (array unsigned-byte (10)) :initially 0))</code> \u000A \u000A <p>But the <v>:type</v> of a stobj field descriptor may instead be based on a \u000A stobj.  For example, the following sequence of three <see topic=\"ACL2____EVENTS\">events</see> is legal. \u000A The first field descriptor of <v>top</v>, named <v>sub1-field</v>, illustrates one \u000A new kind of value for <v>:type</v>: the name of a previously-introduced stobj, \u000A which here is <v>sub1</v>. The second field descriptor of <v>top</v>, named \u000A <v>sub2-ar-field</v>, illustrates the other new kind of value for <v>:type</v>: an \u000A array whose elements are specified by the name of a previously-introduced \u000A stobj, in this case, the stobj <v>sub2</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> sub1 fld1)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> sub2 fld2)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> top\u000A  (sub1-field :type sub1)\u000A  (sub2-ar-field :type (array sub2 (10))))</code> \u000A \u000A <p>The <v>:initially</v> keyword is illegal for fields whose <v>:type</v> is a \u000A stobj or an array of stobjs.  Each such initial value is provided by a \u000A corresponding call of the stobj creator for that stobj.  In particular, in the \u000A case of an array of stobjs, the stobj creator is called once for each element \u000A of the array, so that the array elements are distinct.  For example, each \u000A element of <v>sub2-ar-field</v> in the example above is initially provided by a \u000A separate call of <v>create-sub2</v>.  Each initial element is thus unique, and \u000A in particular is distinct from the initial global value of the stobj. \u000A Similarly, the initial global stobj for <v>sub1</v> is distinct from the initial \u000A <v>sub1-field</v> field of the global stobj for <v>top</v>, as these result from \u000A separate calls of <v>create-sub1</v>.</p> \u000A \u000A <p>When a stobj is used in a field of another stobj, we may refer to the \u000A former field as a ``child stobj'' and the latter stobj as a ``parent stobj''. \u000A So in the example above, <v>sub1-field</v> is a child stobj of type <v>sub1</v> \u000A for parent stobj <v>top</v>, and <v>sub2-ar-field</v> is an array of child stobjs \u000A of type <v>sub2</v> for parent stobj <v>top</v>.  A child stobj has the same \u000A structural shape as the global stobj of its type, but as explained above, \u000A these are distinct structures.  We follow standard terminology by saying \u000A ``isomorphic'' to indicate the same structural shape.  So for example, (the \u000A value of) <v>sub1-field</v> is isomorphic to <v>sub1</v>, though these are \u000A distinct structures.</p> \u000A \u000A <p>ACL2 enforces the following restriction, which can allow for greater \u000A efficiency in the raw Lisp code generated for <v>stobj-let</v> forms, as per the \u000A discussion below about clearing memoization tables.  If the parent stobj is \u000A introduced by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> using keyword argument <v>:non-memoizable t</v>, \u000A then this is required to have been the case for every child stobj as well.</p> \u000A \u000A <h3>SECTION: An aliasing problem</h3> \u000A \u000A <p>Before introducing <v>stobj-let</v> below, we provide motivation for this \u000A ACL2 primitive.</p> \u000A \u000A <p>Consider the following <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child fld)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> parent\u000A  (fld2 :type child))</code> \u000A \u000A <p>Now suppose we could evaluate the following code, to be run immediately \u000A after admitting the two <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> events above.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((child (fld2 parent))\u000A       (child (update-fld 3 child)))\u000A  (<see topic=\"ACL2____MV\">mv</see> child parent))</code> \u000A \u000A <p>Now logically there is no change to <v>parent</v>: <v>parent</v> is passed \u000A through unchanged.  We can indeed prove that fact!</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1\u000A                    (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((child (fld2 parent))\u000A                           (child (update-fld 3 child)))\u000A                      (<see topic=\"ACL2____MV\">mv</see> child parent)))\u000A            parent))</code> \u000A \u000A <p>But recall that stobjs are updated with destructive assignments.  That is, \u000A we really are updating <v>(fld2 parent)</v> to be the new value of <v>child</v>, \u000A whether this is explained logically or not.  Thus, evaluation of the above \u000A <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> form does in fact change the actual global stobj, \u000A <v>parent</v>.</p> \u000A \u000A <p>(Aside: Here is an explanation involving raw Lisp, for those who might find \u000A this useful.  We escape to raw Lisp and execute the following; note that \u000A <v>*the-live-parent*</v> is the Lisp variable representing the global value of \u000A <v>parent</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((parent *the-live-parent*))\u000A  (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((child (fld2 parent))\u000A         (child (update-fld 4 child)))\u000A    (<see topic=\"ACL2____MV\">mv</see> child parent)))</code> \u000A \u000A <p>Then, in raw Lisp, <v>(fld (fld2 *the-live-parent*))</v> evaluates to <v>4</v>, \u000A illustrating the destructive update.  End of Aside.)</p> \u000A \u000A <p>Such aliasing can permit a change to a child stobj to cause a \u000A logically-inexplicable change to the parent stobj.  Similarly, unfettered \u000A accessing of stobj fields can result in logically inexplicable changes to the \u000A child stobj when the parent stobj is changed.  Thus, ACL2 disallows direct \u000A calls of stobj accessors and updaters for fields whose <v>:type</v> is a stobj \u000A or an array of stobjs.  Instead, ACL2 provides <v>stobj-let</v> for reading and \u000A writing such fields in a sound manner.</p> \u000A \u000A <h3>SECTION: Accessing and updating stobj fields of stobjs using \u000A <v>stobj-let</v></h3> \u000A \u000A <p>ACL2 provides a primitive, <v>stobj-let</v>, to access and update stobj \u000A fields of stobjs, in a manner that avoids the aliasing problem discussed \u000A above.  In this section we provide an informal introduction to <v>stobj-let</v>, \u000A using examples, to be followed in the next section by precise \u000A documentation.</p> \u000A \u000A <p>We begin by returning to a slight variant of the example above.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child fld)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> parent\u000A  (fld2 :type child)\u000A  fld3)</code> \u000A \u000A <p>The following form returns the result of updating the <v>fld2</v> field of \u000A <v>parent</v>, which is a stobj isomorphic to <v>child</v>, to have a value of 3. \u000A Below we explain the terms ``bindings'', ``producer variables'', ``producer'', \u000A and ``consumer'', as well as how to understand this form.</p> \u000A \u000A <code>(<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A ((child (fld2 parent)))  ; bindings\u000A (child)                  ; producer variable(s)\u000A (update-fld 3 child)     ; producer\u000A (update-fld3 'a parent)) ; consumer</code> \u000A \u000A <p>The four lines under ``<v>stobj-let</v>'' just above can be understood as \u000A follows.</p> \u000A \u000A <ul> \u000A <li>Bindings:<br/> \u000A     <blockquote>Bind <v>child</v> to <v>(fld2 parent)</v>.</blockquote></li> \u000A <li>Producer variable(s) and producer:<br/> \u000A     <blockquote>Then bind the variable, <v>child</v>, to the value of \u000A     the producer, <v>(update-fld 3 child)</v>.</blockquote></li> \u000A <li>Implicit update of parent:<br/> \u000A     <blockquote>Update <v>fld2</v> of <v>parent</v> with the producer variable, \u000A     <v>child</v>.</blockquote></li> \u000A <li>Consumer:<br/> \u000A     <blockquote>Finally, return <v>(update-fld3 'a\u000A     parent)</v>.</blockquote></li> \u000A </ul> \u000A \u000A <p>Thus, the logical expansion of the <v>stobj-let</v> form above is the \u000A following expression, though this is approximate (see below).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((child (fld2 parent))) ; bindings\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((child (update-fld 3 child))) ; bind producer vars to producer\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((parent (update-fld2 child parent))) ; implicit update of parent\u000A      (update-fld3 'a parent))))</code> \u000A \u000A <p>The bindings always bind distinct names to child stobjs of a unique parent \u000A stobj, where the child stobj corresponds to the <v>:type</v> associated with the \u000A indicated accessor in the <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> form for the parent stobj.  Thus in \u000A this case, for the unique binding, variable <v>child</v> is bound to the stobj \u000A of `type' <v>child</v> for accessor <v>fld2</v> of the parent stobj, <v>parent</v>. \u000A We refer to <v>child</v> from the bindings as a ``stobj-let-bound variable''. \u000A Note also that the ``implicit extra step'' mentioned above is generated by \u000A macroexpansion of <v>stobj-let</v>; it logically updates the parent with new \u000A child values, just before calling the consumer.  Implementation note for \u000A ACL2(h) only (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>): Destructive updating in raw \u000A Lisp lets us omit this implicit extra step, though the raw Lisp code generated \u000A for <v>stobj-let</v> will clear the memoization table for every function taking \u000A the parent stobj as an input, if any child stobj bound in the bindings is \u000A among the producer variables &mdash; unless the parent stobj was introduced by \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> using keyword argument <v>:non-memoizable t</v>.</p> \u000A \u000A <p>The form above is equivalent to the form displayed just below, which \u000A differs only in specifying an explicit stobj updater corresponding to the \u000A stobj accessor, <v>fld2</v>.  Here we show the default updater name, whose name \u000A has <v>&quot;UPDATE-&quot;</v> prepended to the name of the accessor.  But if the \u000A <v>:RENAMING</v> field of the <v>defstobj</v> event specified a different updater \u000A name corresponding to <v>fld2</v>, then that would need to be included where we \u000A have added <v>update-fld2</v> below.</p> \u000A \u000A <code>(<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A ((child (fld2 parent) update-fld2)) ; bindings, including updater(s)\u000A (child)                  ; producer variables\u000A (update-fld 3 child)     ; producer\u000A (update-fld3 'a parent)) ; consumer</code> \u000A \u000A <p>You can experiment using <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> to see the single-step \u000A macroexpansion of a <v>stobj-let</v> form in the logic.  For example, here is \u000A how that works for a <v>stobj-let</v> form that binds three fields and updates \u000A two of them.  Notice that because more than one field is updated, an <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> form is generated to bind the two fields to their values returned by \u000A the producer, rather than a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> form as previously generated.  First, \u000A let's introduce some events.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child1 child1-fld)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child2 child2-fld)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child3 child3-fld)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> mom\u000A  (fld1 :type child1)\u000A  (fld2 :type child2)\u000A  (fld3 :type child3))\u000A; Silly stub:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-last-op (op mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> op))\u000A  mom)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> new-mom (mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom))\u000A  (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A   ((child1 (fld1 mom))\u000A    (child2 (fld2 mom))\u000A    (child3 (fld3 mom)))\u000A   (child1 child3)\u000A   (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((child1 (update-child1-fld 'one child1))\u000A          (child3 (update-child3-fld 'three child3)))\u000A     (<see topic=\"ACL2____MV\">mv</see> child1 child3))\u000A   (update-last-op 'my-compute mom)))</code> \u000A \u000A <p>Now let's look at the single-step macroexpansion of the above \u000A <v>stobj-let</v> form.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A                ((child1 (fld1 mom))\u000A                 (child2 (fld2 mom))\u000A                 (child3 (fld3 mom)))\u000A                (child1 child3)\u000A                (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((child1 (update-child1-fld 'one child1))\u000A                       (child3 (update-child3-fld 'three child3)))\u000A                  (<see topic=\"ACL2____MV\">mv</see> child1 child3))\u000A                (update-last-op 'my-compute mom))\u000A (<see topic=\"ACL2____PROGN_42\">PROGN$</see>\u000A  (<see topic=\"COMMON-LISP____LET\">LET</see>\u000A   ((CHILD1 (FLD1 MOM))\u000A    (CHILD2 (FLD2 MOM))\u000A    (CHILD3 (FLD3 MOM)))\u000A   (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> CHILD1 CHILD2 CHILD3))\u000A   (<see topic=\"ACL2____MV-LET\">MV-LET</see>\u000A      (CHILD1 CHILD3)\u000A      (CHECK-VARS-NOT-FREE (MOM)\u000A                           (<see topic=\"COMMON-LISP____LET_A2\">LET*</see> ((CHILD1 (UPDATE-CHILD1-FLD 'ONE CHILD1))\u000A                                  (CHILD3 (UPDATE-CHILD3-FLD 'THREE CHILD3)))\u000A                                 (<see topic=\"ACL2____MV\">MV</see> CHILD1 CHILD3)))\u000A      (<see topic=\"COMMON-LISP____LET_A2\">LET*</see> ((MOM (UPDATE-FLD1 CHILD1 MOM))\u000A             (MOM (UPDATE-FLD3 CHILD3 MOM)))\u000A            (CHECK-VARS-NOT-FREE (CHILD1 CHILD2 CHILD3)\u000A                                 (UPDATE-LAST-OP 'MY-COMPUTE MOM))))))\u000AACL2 !&gt;</code> \u000A \u000A <p>If you try to evaluate a <v>stobj-let</v> form directly in the top-level \u000A loop, rather than from within a function body, you will get an error.  The \u000A example above illustrates how <v>stobj-let</v> may be used in function bodies; \u000A here is another example, presented using an edited log.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> child fld)\u000A\u000ASummary\u000AForm:  ( DEFSTOBJ CHILD ...)\u000ARules: NIL\u000ATime:  0.02 seconds (prove: 0.00, print: 0.00, other: 0.02)\u000A CHILD\u000AACL2 !&gt;(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> parent\u000A         (fld2 :type child)\u000A         fld3)\u000A\u000ASummary\u000AForm:  ( DEFSTOBJ PARENT ...)\u000ARules: NIL\u000ATime:  0.02 seconds (prove: 0.00, print: 0.00, other: 0.02)\u000A PARENT\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (parent)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs parent))\u000A         (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A          ((child (fld2 parent)))   ; bindings\u000A          (child)                   ; producer variables\u000A          (update-fld 3 child)      ; producer\u000A          (update-fld3 'a parent))) ; consumer\u000A[[output omitted]]\u000A F\u000AACL2 !&gt;(f parent)\u000A&lt;parent&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> check-f (parent)\u000A         ; returns the value of the field of the child stobj\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs parent))\u000A         (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A          ((child (fld2 parent))) ; bindings\u000A          (val)                   ; producer variables\u000A          (fld child)             ; producer\u000A          val))                   ; consumer\u000A[[output omitted]]\u000A CHECK-F\u000AACL2 !&gt;(check-f parent)\u000A3\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that the second function defined above, <v>check-f</v>, uses a \u000A <v>stobj-let</v> form that returns an ordinary value: it reads a value from a \u000A child stobj, but does not write to the child stobj, as indicated by the lack \u000A of a child stobj among the producer variables.  So for that <v>stobj-let</v> \u000A form, there is no implicit extra step.</p> \u000A \u000A <p>We labeled a <v>stobj-let</v> expansion above as ``approximate'' for two \u000A reasons, which we give here informally.  (Now you know how to apply \u000A <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> to that <v>stobj-let</v> call to see the precise \u000A expansion.)  First, <v>stobj-let</v> declares the stobj-let-bound variables to \u000A be <v>ignorable</v> for the top <v>let</v> bindings.  Second, and more \u000A importantly, <v>stobj-let</v> imposes the following restrictions on the producer \u000A and consumer, to avoid the aliasing problem: it disallows references to the \u000A parent stobj in the producer and it also disallows references to any bound \u000A stobj (i.e., bound in the bindings) in the consumer.</p> \u000A \u000A <p>We conclude this section with examples based on a slight variation of the \u000A nested stobj example from the first section above.  These events can also be \u000A found in ACL2 community book <v>misc/nested-stobj-tests.lisp</v>, immediately \u000A under the following comment:</p> \u000A \u000A <code>; As promised in :doc stobj-let, we begin with an example from that :doc.</code> \u000A \u000A <p>Note that some lemmas were needed in order to complete the <see topic=\"ACL2____GUARD\">guard</see> \u000A proof for the function <v>update-top</v>, which may be found in the above file; \u000A they are omitted below.</p> \u000A \u000A <p>First we introduce three stobjs.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> kid1 fld1)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> kid2 fld2)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> mom\u000A  (kid1-field :type kid1)\u000A  (kid2-ar-field :type (array kid2 (5)))\u000A  last-op)</code> \u000A \u000A <p>The next function takes a given index and a <v>mom</v> stobj, and swaps the \u000A value stored in the stobj in <v>mom</v>'s <v>kid2-ar-field</v> array at that index \u000A with the value stored in the stobj in <v>mom</v>'s <v>kid1-field</v> field.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mom-swap-fields (index mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom\u000A                  :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> index)\u000A                              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> index (kid2-ar-field-length mom)))))\u000A  (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A   ((kid1 (kid1-field mom))\u000A    (kid2 (kid2-ar-fieldi index mom)))\u000A   (kid1 kid2)\u000A   (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((val1 (fld1 kid1))\u000A          (val2 (fld2 kid2))\u000A          (kid1 (update-fld1 val2 kid1))\u000A          (kid2 (update-fld2 val1 kid2)))\u000A     (<see topic=\"ACL2____MV\">mv</see> kid1 kid2))\u000A   (update-last-op 'swap mom)))</code> \u000A \u000A <p>Function <v>mom.kid1-fld1</v> stores a given value in the given <v>mom</v>'s \u000A <v>kid1-fld1</v> field.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mom.kid1-fld1 (val mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom))\u000A  (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A   ((kid1 (kid1-field mom)))\u000A   (kid1)\u000A   (update-fld1 val kid1)\u000A   (update-last-op val mom)))</code> \u000A \u000A <p>We next combine the two functions above, according to an <v>op</v> argument, \u000A as indicated by the following definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-mom (op mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> op)\u000A              (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> op) 'swap)\u000A              (<see topic=\"ACL2____NATP\">natp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> op))\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> op) (kid2-ar-field-length mom)))\u000A         (mom-swap-fields (<see topic=\"COMMON-LISP____CDR\">cdr</see> op) mom))\u000A        (t (mom.kid1-fld1 op mom))))</code> \u000A \u000A <p>The following checker function uses a <v>stobj-let</v> form like the ones \u000A above, a major difference being that the producer variable is not a stobj, \u000A since the producer does not modify any stobjs.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> check-update-mom (index val1 val2 last-op mom)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom\u000A                    :mode :program\u000A                    :guard\u000A                    (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NULL\">null</see> index)\u000A                        (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> index)\u000A                             (<see topic=\"COMMON-LISP_____C3\">&lt;</see> index (kid2-ar-field-length mom))))))\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (last-op mom) last-op)\u000A         (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A          ((kid1 (kid1-field mom))\u000A           (kid2 (kid2-ar-fieldi index mom)))\u000A          (val) ; producer variables\u000A          (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> val1 (fld1 kid1))\u000A               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> val2 (fld2 kid2)))\u000A          val)))</code> \u000A \u000A <p>Now let us run our update function to populate some fields within the \u000A <v>mom</v> stobj.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((mom ; set mom to (3 (x0 x1 x2 x3 x4))\u000A         (update-mom 3 mom))\u000A        (mom ; set mom to (x1 (x0 3 x2 x3 x4))\u000A         (update-mom '(swap . 1) mom))\u000A        (mom ; set mom to (7 (x0 3 x2 x3 x4))\u000A         (update-mom 7 mom))\u000A        (mom ; set mom to (x0 (7 3 x2 x3 x4))\u000A         (update-mom '(swap . 0) mom))\u000A        (mom ; set mom to (5 (7 3 x2 x3 x4))\u000A         (update-mom 5 mom))\u000A        (mom ; set mom to (7 (5 3 x2 x3 x4))\u000A         (update-mom '(swap . 0) mom)))\u000A   mom)</code> \u000A \u000A <p>Are the above values of 7, 5, and 3 as expected, with a last operation \u000A being a swap?  Yes!</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____AND\">and</see> (check-update-mom 0 7 5 'swap mom)\u000A            (check-update-mom 1 7 3 'swap mom))\u000AT\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that above, we never tried to access two different entries of the \u000A array.  This can be done, but we need to bind two different stobjs to those \u000A fields.  Fortunately, congruent stobjs make this possible; see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>, in particular the discussion of congruent stobjs.  Since we want to \u000A bind two stobjs to values in the array that are isomorphic to the stobj \u000A <v>kid2</v>, we introduce a stobj congruent to <v>kid2</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> kid2a fld2a :congruent-to kid2)</code> \u000A \u000A <p>Then we can define our swapping function as follows.  The <see topic=\"ACL2____GUARD\">guard</see> \u000A proof obligation includes the requirement that the two indices be distinct, \u000A again to avoid an aliasing problem.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mom-swap-indices (i1 i2 mom)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs mom\u000A                  :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> i1)\u000A                              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i1 (kid2-ar-field-length mom))\u000A                              (<see topic=\"ACL2____NATP\">natp</see> i2)\u000A                              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i2 (kid2-ar-field-length mom))\u000A                              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> i1 i2)))))\u000A  (<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A   ((kid2 (kid2-ar-fieldi i1 mom))\u000A    (kid2a (kid2-ar-fieldi i2 mom)))\u000A   (kid2 kid2a)\u000A   (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((val2 (fld2 kid2))\u000A          (val2a (fld2 kid2a))\u000A          (kid2 (update-fld2 val2a kid2))\u000A          (kid2a (update-fld2 val2 kid2a)))\u000A     (<see topic=\"ACL2____MV\">mv</see> kid2 kid2a))\u000A   mom))</code> \u000A \u000A <p>The aforementioned community book, <v>misc/nested-stobj-tests.lisp</v>, \u000A contains a corresponding checker immediately following this definition.</p> \u000A \u000A <h3>SECTION: Precise documentation for <v>stobj-let</v></h3> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____STOBJ-LET\">stobj-let</see>\u000A BINDINGS\u000A PRODUCER-VARIABLES\u000A PRODUCER\u000A CONSUMER)</code> \u000A \u000A <p>where <v>PRODUCER-VARIABLES</v> is a non-empty true list of legal variable \u000A names without duplicates, <v>PRODUCER</v> and <v>CONSUMER</v> are expressions, and \u000A <v>BINDINGS</v> is a list subject to the following requirements.</p> \u000A \u000A <p><v>BINDINGS</v> is a non-empty true list of tuples, each of which has the \u000A form <v>(VAR ACCESSOR)</v> or <v>(VAR ACCESSOR UPDATER)</v>.  There is a stobj \u000A name, <v>ST</v>, previously introduced by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> (not <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>), such that each <v>accessor</v> is of the form <v>(ACC ST)</v> or \u000A <v>(ACCi I ST)</v>, with the same stobj name (<v>ST</v>) for each binding.  In the \u000A case <v>(ACC ST)</v>, <v>ACC</v> is the accessor for a non-array field of <v>ST</v>. \u000A In the case <v>(ACCi I ST)</v>, <v>ACCi</v> is the accessor for an array field of \u000A <v>ST</v>, and <v>I</v> is either a variable, a natural number, a list <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A N)</v> where <v>N</v> is a natural number, or a symbol introduced by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>.  If <v>UPDATER</v> is supplied, then it is a symbol that is the name \u000A of the stobj updater for the field of <v>ST</v> accessed by <v>ACCESSOR</v>.  If \u000A <v>UPDATER</v> is not supplied, then for the discussion below we consider it to \u000A be, implicitly, the symbol in the same package as the function symbol of \u000A <v>ACCESSOR</v> (i.e., <v>ACC</v> or <v>ACCi</v>), obtained by prepending the string \u000A <v>&quot;UPDATE-&quot;</v> to the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of that function symbol.  Finally, \u000A <v>ACCESSOR</v> has a <see topic=\"ACL2____SIGNATURE\">signature</see> specifying a return value that is either \u000A <v>ST</v> or is stobj that is congruent to <v>ST</v>.</p> \u000A \u000A <p>If the conditions above are met, then the General Form expands to the one \u000A of the following expressions, depending on whether the list \u000A <v>PRODUCER-VARIABLES</v> has one member or more than one member, respectively. \u000A (But see below for extra code that may be inserted if there are stobj array \u000A accesses in <v>BINDINGS</v>.)  Here we write <v>STOBJ-LET-BOUND-VARS</v> for the \u000A list of variables <v>VAR</v> discussed above, i.e., for <v>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see>\u000A BINDINGS)</v>.  And, we write <v>UPDATES</v> for the result of mapping through \u000A <v>PRODUCER-VARIABLES</v> and, for each variable <v>VAR</v> that has a binding \u000A <v>(VAR ACCESSOR UPDATER)</v> in <v>BINDINGS</v> (where <v>UPDATER</v> may be \u000A implicit, as discussed above), collect into <v>UPDATES</v> the tuple <v>(ST\u000A (UPDATER VAR ST))</v>.</p> \u000A \u000A <p>For <v>PRODUCER-VARIABLES</v> = <v>(PRODUCER-VAR)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> BINDINGS\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> . STOBJ-LET-BOUND-VARIABLES))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((PRODUCER-VAR PRODUCER))\u000A    (<see topic=\"COMMON-LISP____LET_A2\">let*</see> UPDATES\u000A      CONSUMER)))</code> \u000A \u000A <p>Otherwise:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> BINDINGS\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> . STOBJ-LET-BOUND-VARIABLES))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> PRODUCER-VARS\u000A          PRODUCER\u000A          (<see topic=\"COMMON-LISP____LET_A2\">let*</see> UPDATES\u000A            CONSUMER)))</code> \u000A \u000A <p>Moreover, ACL2 places restrictions on the resulting expression: <v>ST</v> \u000A must not occur free in <v>PRODUCER</v>, and every variable in \u000A <v>STOBJ-LET-BOUND-VARIABLES</v> must not occur free in <v>CONSUMER</v>.</p> \u000A \u000A <p><v>Stobj-let</v> forms can be evaluated using ordinary objects in theorem \u000A contexts, much as any form.  They can also, of course, appear in function \u000A bodies.  However, a <v>stobj-let</v> form cannot be evaluated directly in the \u000A top-level loop or other top-level contexts for execution (such as during \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion).</p> \u000A \u000A <p>Finally, let <v>FORM</v> denote the form displayed above (either case).  We \u000A explain how <v>FORM</v> is actually replaced by an expression of the form \u000A <v>(<see topic=\"ACL2____PROGN_42\">PROGN$</see> ... FORM)</v>.  This expression generates an extra <see topic=\"ACL2____GUARD\">guard</see> proof \u000A obligation, which guarantees that no aliasing occurs from binding two \u000A stobj-let-bound variables to the same array access.  So fix a stobj array \u000A accessor <v>ACCi</v> for which some stobj is bound to <v>(ACCi I ST)</v> in \u000A <v>BINDINGS</v>; we define an expression <v>ACCi-CHECK</v> as follows.  Collect up \u000A all such index expressions <v>I</v>, where if <v>I</v> is of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A N)</v> then replace <v>I</v> by <v>N</v>.  If the resulting list of index \u000A expressions for <v>ACCi</v> consists solely of distinct numbers, or if it is of \u000A length 1, then no extra check is generated for <v>ACCi</v>.  Otherwise, let \u000A <v>ACCi-CHECK</v> be the form <v>(chk-no-duplicatesp (<see topic=\"COMMON-LISP____LIST\">list</see> I1 ... Ik))</v>, where \u000A <v>I1</v>, ..., <v>Ik</v> are the index expressions for <v>ACCi</v>.  Note: \u000A <v>chk-no-duplicatesp</v> is a function that returns nil, but has a <see topic=\"ACL2____GUARD\">guard</see> \u000A that its argument is an <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt> that satisfies <tt><see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see></tt>.  Finally, <v>FORM</v> is replaced by <v>(<see topic=\"ACL2____PROGN_42\">PROGN$</see> CHK1 ... CHKn\u000A FORM)</v>, where the <v>CHKm</v> range over all of the above \u000A <v>ACCi-CHECK</v>.</p>"],

 "ACL2____INTRODUCTION-TO-THE-DATABASE":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>We assume you've read <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> \u000A and <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p> \u000A \u000A <p>The theorem prover's heuristics are influenced by the database of rules and \u000A the enabled/disabled status of the rules.  You can think of the database as a \u000A <i>global</i> hint, potentially affecting all parts of a proof attempt.</p> \u000A \u000A <p>However, in addition to the ``global hint,'' it is possible to give \u000A <i>local</i> hints that affect the theorem prover's behavior on specific \u000A subgoals.  We discuss the database here and discuss local hints later in the \u000A tutorial.</p> \u000A \u000A <p>The theorem prover's ``database'' is called the <i>ACL2 world</i>.  You \u000A change the world by issuing commands called <i>events</i>.  The most common \u000A events are <v>defun</v> for defining new functions (and predicates) and \u000A <v>defthm</v> for proving new theorems.  Both add rules to the database.  Here \u000A are some commonly used events.</p> \u000A \u000A <p>We recommend that upon the first reading of this tutorial you do not follow \u000A the links shown below!  The links take you into the hypertext reference \u000A manual, where it is easy to get lost unless you're looking for detail about \u000A one specific feature.</p> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DEFUN\">defun</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to define a new function or predicate symbol. \u000A Definitional axioms are just a kind of <v>rewrite</v> rule, but <v>defun</v> may \u000A also add rules affecting the determination of the type of a term and rules \u000A affecting the induction analysis.  When you issue a <v>defun</v> command you \u000A will always supply the <i>name</i> of the function or predicate, the list of \u000A <i>formal parameters</i>, <i>v1,...vn</i>, and the <i>body</i>:</p> \u000A \u000A <code>(defun <i>name</i> (<i>v1 ... vn</i>) \u000A   <i>body</i>) \u000A</code> \u000A \u000A <p>If the event is accepted, a <i>definitional axiom</i> is added to the \u000A world, <v>(</v><i>name</i> <i>v1...vn</i><v>)</v>=<i>body</i>, stored as a special \u000A kind of unconditional rewrite rule.  However, the <v>defun</v> event may require \u000A theorems to be proved.  Specifically, <i>measure theorems</i> must be proved \u000A to establish that recursively defined functions always terminate, by proving \u000A that an <i>ordinal</i> measure of the formal parameters decreases in a \u000A <i>well-founded</i> way in every recursive call.  In addition, if \u000A <i>guards</i> are being used to declare the <i>expected domain</i> of the \u000A newly defined function, <i>guard theorems</i> might be proved to establish \u000A that all functions stay within their expected domains.  In any case, you may \u000A provide additional information to the <v>defun</v> event, coded as part of the \u000A <v>declaration</v> that Common Lisp allows:</p> \u000A \u000A <code>(defun <i>name</i> (<i>v1 ... vn</i>) \u000A   (declare (xargs ...)) \u000A   <i>body</i>) \u000A</code> \u000A \u000A <p>The <v>xargs</v> (``extra arguments to <v>defun</v>'') entry may specify, among \u000A other things, the measure to use in the termination proof, hints for use by \u000A the prover during the termination proof, the guard of the new function, and \u000A hints for use by the prover during the guard verification step.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFTHM\">defthm</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to prove a theorem and to add it as a rule of one \u000A or more specified rule-classes.  When you issue a <v>defthm</v> command you \u000A always specify a <i>name</i> for the theorem you're trying to prove and a \u000A <i>formula</i> stating the theorem.  You may optionally supply some local \u000A hints as we describe later in the tutorial.  You may also optionally supply \u000A some <i>rule classes</i> indicating how you want your formula stored as a \u000A rule, after it is proved.  We discuss the <v>defthm</v> <i>rule classes</i> \u000A below.</p> \u000A \u000A <p>See <see topic=\"ACL2____IN-THEORY\">in-theory</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to enable or disable rules.  Rules have names \u000A derived from the names you give to functions and theorems, e.g., <v>(:REWRITE\u000A LEFT-IDENTITY-OF-FOO . 2)</v> for the second rewrite rule you created from the \u000A theorem named <v>LEFT-IDENTITY-OF-FOO</v>.  Rule names are called <i>runes</i>. \u000A A <i>theory</i> is just a set (list) of runes.  The <i>current theory</i> is \u000A the list of enabled runes and the <v>in-theory</v> event can add runes to or \u000A delete runes from the current theory.</p> \u000A \u000A <p>See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> to \u000A change the world by loading a certified file of other events.  The most common \u000A use of <v>include-book</v> is to load ``community books'' -- books written by \u000A other ACL2 users who have released them for distribution to the community. \u000A The most common books loaded are probably the arithmetic books:</p> \u000A \u000A <code>; * for the most elementary arithmetic, needed for any problem \u000A;   that involves even simple addition and multiplication like \u000A; <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> 2 y) -3)</v>: \u000A \u000A    (include-book \"arithmetic/top-with-meta\" :dir :system) \u000A \u000A; * for more complicated arithmetic involving non-linear terms like \u000A; <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x y)</v>, <v>(<see topic=\"COMMON-LISP____EXPT\">expt</see> x (<see topic=\"COMMON-LISP_____B2\">+</see> i j))</v>, and <v>floor</v> and <v>mod</v> \u000A \u000A    (include-book \"arithmetic-5/top\" :dir :system) \u000A</code> \u000A \u000A <p>But for a complete list of system books, see <see topic=\"ACL2____BOOKS\">books</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> to \u000A certify a file of events for reuse later.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFCONST\">defconst</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to define a new constant, allowing you to write a \u000A symbol, e.g., <v>*weekdays*</v> in place of some object, e.g., <v>'(MON TUE WED\u000A THU FRI)</v> in formulas.</p> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to define a new syntactic abbreviation.  The macro \u000A facility in Lisp is quite powerful, allowing you to <i>compute</i> the form to \u000A which some type-in expands.  For example, the primitive macro <v>COND</v> is \u000A defined so that <v>(<see topic=\"COMMON-LISP____COND\">COND</see> ((P X) 1)((<see topic=\"ACL2____Q\">Q</see> X) 2)(T 3))</v> expands to <v>(<see topic=\"COMMON-LISP____IF\">IF</see> (P X)\u000A 1 (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"ACL2____Q\">Q</see> X) 2 3))</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to introduce a <i>single-threaded object</i> that \u000A your functions may modify ``destructively'' provided they follow strict \u000A syntactic rules.</p> \u000A \u000A <p>See <see topic=\"ACL2____EVENTS\">events</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a complete list of the ACL2 events.  There are \u000A events to allow mutually recursive definitions, to introduce some new function \u000A symbols constrained to satisfy given axioms, to allow the temporary \u000A introduction of a ``local'' event to help prove some goal theorem and then \u000A disappear, to provide the power of first-order quantification and a choice \u000A operator, and many other features.</p> \u000A \u000A <p>There are also commands that allow you to inspect the world, e.g., to print \u000A the command that introduced a given name, to show all the commands back to a \u000A certain one, undo the last command or more generally roll-back to an earlier \u000A command.  See <see topic=\"ACL2____HISTORY\">history</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>The Defthm Rule-Classes</b></p> \u000A \u000A <p>We've already discussed the key role that rules play in controlling the \u000A behavior of the system.  New rules are introduced primiarily with the \u000A <v>defthm</v> event, though <v>defun</v> and other events may introduce rules.</p> \u000A \u000A <p>To prove <i>formula</i> and generate, say a <v>:rewrite</v> rule and a \u000A <v>:generalize</v> rule from it, you would write</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :rule-classes (:rewrite :generalize)) \u000A</code> \u000A \u000A <p>If you wanted to rearrange the shape of the formula before generating the \u000A <v>:rewrite</v> rule you could provide a <v>:corollary</v> modifier to the \u000A <v>:rewrite</v> rule class:</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :rule-classes ((:rewrite :corollary ...) \u000A                       :generalize)). \u000A</code> \u000A \u000A <p>There are many classes of rules, affecting different parts of the system. \u000A Each class is denoted by a keyword, e.g., <v>:REWRITE</v>, <v>:LINEAR</v>, etc. \u000A You are responsible for specifying the class(es) of rules to be generated from \u000A a given formula and several different rules (possibly of different classes) \u000A may be derived from a single formula.  Each class admits optional modifiers \u000A that allow you finer control over each rule.  Each class admits the \u000A <v>:corollary</v> modifier with which you can rearrange the formula before a \u000A rule of that class is generated.  This allows you to state a theorem in its \u000A most elegant form for publication purposes but store it as a rule with the \u000A most appropriate hypotheses and conclusion.  Other modifiers tend to be \u000A specific to certain rule classes, but for example, <v>:rewrite</v> rule \u000A modifiers include an optional limit on the depth of backchaining and options \u000A for handling free variables.</p> \u000A \u000A <p>We give some links below to other classes of rules.  However, we recommend \u000A that you not follow these links upon your first reading of this tutorial!</p> \u000A \u000A <p>See <see topic=\"ACL2____REWRITE\">rewrite</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a description of how to create a rewrite \u000A rule.</p> \u000A \u000A <p>See <see topic=\"ACL2____LINEAR\">linear</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a description of how to store theorems \u000A concluding with arithmetic inequalities.  The trouble with storing</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"ACL2____LEN\">len</see> (delete e x)) (<see topic=\"ACL2____LEN\">len</see> x))</code> \u000A \u000A <p>as a rewrite rule is that it only matches instances of that inequality and \u000A thus fails to match</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"ACL2____LEN\">LEN</see> (DELETE E X)) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">LEN</see> X)))</code> \u000A \u000A <p>ACL2 contains an extensible linear arithmetic decision procedure and by \u000A storing inequalities as <v>:linear</v> rules you can make that decision \u000A procedure aware of the basic inequalities between non-primitive numerically \u000A valued terms.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, see <see topic=\"ACL2____CONGRUENCE\">congruence</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A and see <see topic=\"ACL2____REFINEMENT\">refinement</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> to learn how to introduce a new equivalence \u000A relation to the rewriter.  For example, suppose you define <v>set-equal</v> so \u000A that it returns t precisely if its two arguments are lists containing the same \u000A elements, regardless of order or number of occurrences.  Note that under this \u000A sense of ``equivalence'', <v>(rev x)</v> is the identity function and \u000A <v>append</v> is commutative, for example.</p> \u000A \u000A <code>(set-equal (rev x) x)\u000A\u000A(set-equal (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) (<see topic=\"COMMON-LISP____APPEND\">append</see> y x))</code> \u000A \u000A <p>You can make ACL2 use these two theorems as <v>:rewrite</v> rules to replace \u000A instances of <v>(REV x)</v> and <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y)</v> by <v>set-equal</v> terms, even \u000A though the results are not actually <v>EQUAL</v>.  This is possible provided the \u000A target occurs in a context admitting <v>set-equal</v> as a congruence relation. \u000A For example, the <v>:congruence</v> rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (set-equal a b)\u000A         (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e a)\u000A              (<see topic=\"COMMON-LISP____MEMBER\">member</see> e b)))</code> \u000A \u000A <p>gives the rewriter permission to use the above <v>set-equal</v> rules as \u000A rewrite rules in the second argument of any <v>member</v> expression being used \u000A in a propositional way.</p> \u000A \u000A <p>See <see topic=\"ACL2____ELIM\">elim</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a description of how to make the system adopt a \u000A ``change of variable'' tactic that can trade in <i>destructor</i> functions \u000A for <i>constructor</i> functions.  In analogy with how ACL2 eliminates <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see>\u000A X)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> X)</v> by replacing <v>X</v> with <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> A B)</v>, you can make \u000A it eliminate other destructors.  For example, the community book \u000A <v>&quot;arithmetic-5/top&quot;</v> provides an <v>elim</v> rule that eliminates <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see>\u000A x y)</v> and <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> x y)</v> by replacing <v>x</v> by <v>(<see topic=\"COMMON-LISP_____B2\">+</see> r (<see topic=\"COMMON-LISP_____A2\">*</see> y q))</v>, so that \u000A the <v>floor</v> expression becomes <v>q</v> and the <v>mod</v> expression becomes \u000A <v>r</v>.  When introducing your own <v>elim</v> rules you will probably also need \u000A to introduce <v>generalize</v> rules (see below) so that the new variables are \u000A appropriately constrained.</p> \u000A \u000A <p>See <see topic=\"ACL2____GENERALIZE\">generalize</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a description of how you can make ACL2 restrict \u000A the new variables it introduces when generalizing.  ACL2 will sometimes \u000A replace a term by a new variable and with <v>generalize</v> rules you can insure \u000A that the new variable symbol has certain properties of the term it \u000A replaces.</p> \u000A \u000A <p>See <see topic=\"ACL2____INDUCTION\">induction</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a description of how to tailor the inductions \u000A suggested by a term.  Most of the time when ACL2 chooses the ``wrong'' \u000A induction, the easiest fix is with a local <v>:induct</v> hint (see below).  But \u000A if the same problem arises repeatedly in several theorems, you might want to \u000A ``educate'' ACL2's induction heuristic.</p> \u000A \u000A <p>For a complete list of rule-classes, See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>If you are reading this as part of the tutorial introduction to the theorem \u000A prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____INTERESTING-APPLICATIONS":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>ACL2 is an interactive system in which you can model digital \u000A artifacts and guide the system to mathematical proofs about the behavior of \u000A those models.  It has been used at such places as AMD, Centaur, IBM, and \u000A Rockwell Collins to verify interesting properties of commercial designs.  It \u000A has been used to verify properties of models of microprocessors, microcode, \u000A the Sun Java Virtual Machine, operating system kernels, other verifiers, and \u000A interesting algorithms.</p> \u000A \u000A <p>Here we list just a few of the industrially-relevant results obtained with \u000A ACL2.  Reading the list may help you decide you want to learn how to use ACL2. \u000A If you do decide you want to learn more, we recommend that you take <see \u000A topic='ACL2____The_02Tours'>The Tours</see> after you leave this page.</p> \u000A \u000A <p>ACL2 was used at <b>Motorola Government Systems</b> to certify several \u000A microcode programs for the <b>Motorola CAP digital signal processor</b>, \u000A including a comparator sort program that is particularly subtle. In the same \u000A project, ACL2 was used to model the CAP at both the pipelined architectural \u000A level and the instruction set level.  The architectural model was bit- and \u000A cycle-accurate: it could be used to predict every bit of memory on every \u000A cycle.  The models were proved equivalent under certain hypotheses, the most \u000A important being a predicate that analyzed the microcode for certain pipeline \u000A hazards. This predicate defined what the hazards were, syntactically, and the \u000A equivalence of the two models established the correctness of this syntactic \u000A characterization of hazards.  Because ACL2 is a functional programming \u000A language, the ACL2 models and the hazard predicate could be executed.  ACL2 \u000A executed microcode interpretr several times faster than the hardware simulator \u000A could execute it -- with assurance that the answers were equivalent.  In \u000A addition, the ACL2 hazard predicate was executed on over fifty microcode \u000A programs written by Motorola engineers and extracted from the ROM \u000A mechanically. Hazards were found in some of these.  (See, for example, Bishop \u000A Brock and Warren. A. Hunt, Jr. ``Formal analysis of the motorola CAP DSP.'' In \u000A <i>Industrial-Strength Formal Methods</i>. Springer-Verlag, 1999.)</p> \u000A \u000A <p>ACL2 was used at <b>Advanced Micro Devices</b> (AMD) to verify the \u000A compliance of the <b>AMD Athon</b>'s (TM) elementary floating point operations \u000A with their IEEE 754 specifications. This followed ground-breaking work in 1995 \u000A when ACL2 was used to prove the correctness of the microcode for \u000A floating-point division on the <b>AMD K5</b>. The AMD Athlon work proved \u000A addition, subtraction, multiplication, division, and square root compliant \u000A with the IEEE standard. Bugs were found in RTL designs. These bugs had \u000A survived undetected in hundreds of millions of tests but were uncovered by \u000A ACL2 proof attempts. The RTL in the fabricated Athlon FPU has been \u000A mechanically verified by ACL2.  Similar ACL2 proofs have been carried out for \u000A every major AMD FPU design fabricated since the Athlon.  (See for example, \u000A David Russinoff. ``A mechanically checked proof of correctness of the AMD5K86 \u000A floating-point square root microcode''.  <i>Formal Methods in System \u000A Design</i> Special Issue on Arithmetic Circuits, 1997.)</p> \u000A \u000A <p>ACL2 was used at <b>IBM</b> to verify the floating point divide and square \u000A root on the <b>IBM Power 4</b>.  (See Jun Sawada. ``Formal verification of \u000A divide and square root algorithms using series calculation''. In \u000A <i>Proceedings of the ACL2 Workshop 2002</i>, Grenoble, April 2002.)</p> \u000A \u000A <p>ACL2 was used to verify floating-point addition/subtraction instructions \u000A for the <b>media unit</b> from <b>Centaur Technology</b>'s 64-bit, \u000A X86-compatible microprocessor. This unit implements over one hundred \u000A instructions, with the most complex being floating-point \u000A addition/subtraction. The media unit can add/subtract four pairs of \u000A floating-point numbers every clock cycle with an industry-leading two-cycle \u000A latency. The media unit was modeled by translating its Verilog design into an \u000A HDL deeply embedded in the ACL2 logic. The proofs used a combination of AIG- \u000A and BDD-based symbolic simulation, case splitting, and theorem proving.  (See \u000A Warren A. Hunt, Jr. and Sol Swords. ``Centaur Technology Media Unit \u000A Verification''. In <i>CAV '09: Proceedings of the 21st International</i> \u000A <b>Conference on Computer Aided Verification</b>, pages 353--367, Berlin, \u000A Heidelberg, 2009. Springer-Verlag.)</p> \u000A \u000A <p><b>Rockwell Collins</b> used ACL2 to prove information flow properties \u000A about its <b>Advanced Architecture MicroProcessor 7 Government Version \u000A (AAMP7G)</b>, a Multiple Independent Levels of Security (MILS) device for use \u000A in cryptographic applications. The AAMP7G provides MILS capability via a \u000A verified secure hardware-based separation kernel. The AAMP7G's design was \u000A proved to achieve MILS using ACL2, in accordance with the standards set by \u000A EAL-7 of the Common Criteria and Rockwell Collins has received National \u000A Security Agency (NSA) certification for the device based on this work.  (See \u000A David S. Hardin, Eric W. Smith, and William. D. Young. ``A robust machine code \u000A proof framework for highly secure applications''. In <i>Proceedings of the \u000A sixth international workshop on the ACL2 theorem prover</i> <i>and its \u000A applications</i>, pages 11--20, New York, NY, USA, 2006. ACM.)</p> \u000A \u000A <p>Key properties of the <b>Sun Java Virtual Machine</b> and its <b>bytecode \u000A verifier</b> were verified in ACL2. Among the properties proved were that \u000A certain invariants are maintained by <b>class loading</b> and that the \u000A bytecode verifier insures that execution is safe. In addition, various <b>JVM \u000A bytecode programs</b> have been verified using this model of the JVM.  (See \u000A Hanbing Liu. <i>Formal Specification and</i> <i>Verification of a JVM and its \u000A Bytecode Verifier</i>. PhD thesis, University of Texas at Austin, 2006.)</p> \u000A \u000A <p>The <b>Boyer-Moore fast string searching algorithm</b> was verified with \u000A ACL2, including a model of the JVM bytecode for the search algorithm itself \u000A (but not the preprocessing).  (See J S. Moore and Matt Martinez. ``A \u000A mechanically checked proof of the correctness of the Boyer-Moore fast string \u000A searching algorithm.'' In <i>Engineering Methods and Tools for Software Safety \u000A and Security</i> pages 267--284. IOS Press, 2009.)</p> \u000A \u000A <p>ACL2 was used to verify the fidelity between an <b>ACL2-like theorem \u000A prover</b> and a simple (``trusted by inspection'') <b>proof checker</b>, \u000A thereby establishing (up to the soundness of ACL2) the soundness of the \u000A ACL2-like theorem prover.  This project was only part of a much larger project \u000A in which the resulting ACL2 proof script was then hand-massaged into a script \u000A suitable for the ACL2-like theorem prover to process, generating a formal \u000A proof of its fidelity that has been checked by the trusted proof checker. \u000A (See Jared Davis.  <i>Milawa: A Self-Verifying Theorem Prover</i>. \u000A Ph.D. Thesis, University of Texas at Austin, December, 2009.)</p> \u000A \u000A <p>These are but a few of the interesting projects carried out with ACL2. \u000A Many of the authors mentioned above have versions of the papers on their web \u000A pages.  In addition, see <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html'>Books \u000A and Papers about ACL2 and its Applications</a>.  Also, see the presentations \u000A in each of the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/workshops.html'>ACL2 \u000A Workshops</a>.</p>"],

 "ACL2____HISTORY":[["ACL2"],"ACL2 Sources","ACL2","<p>ACL2 keeps track of the <see topic=\"ACL2____COMMAND\">command</see>s that you have executed that \u000A have extended the logic or the rule database, as by the definition of macros, \u000A functions, etc.  Using the facilities in this section you can review the \u000A sequence of <see topic=\"ACL2____COMMAND\">command</see>s executed so far.  For example, you can ask to see \u000A the most recently executed <see topic=\"ACL2____COMMAND\">command</see>, or the <see topic=\"ACL2____COMMAND\">command</see> <v>10</v> \u000A before that, or the <see topic=\"ACL2____COMMAND\">command</see> that introduced a given function symbol. \u000A You can also undo back through some previous <see topic=\"ACL2____COMMAND\">command</see>, restoring the \u000A logical <see topic=\"ACL2____WORLD\">world</see> to what it was before the given <see topic=\"ACL2____COMMAND\">command</see>.</p> \u000A \u000A <p>The annotations printed in the margin in response to some of these commands \u000A (such as `P', `L', and `V') are explained in the documentation for \u000A <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt>.</p> \u000A \u000A <p>Several technical terms are used in the documentation of the history <see topic=\"ACL2____COMMAND\">command</see>s.  You must understand these terms to use the <see topic=\"ACL2____COMMAND\">command</see>s.  These \u000A terms are documented via <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> entries of their own.  See <see topic=\"ACL2____COMMAND\">command</see>, see <see topic=\"ACL2____EVENTS\">events</see>, see <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>, and see <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p>"],

 "ACL2____GUARD-EVALUATION-EXAMPLES-LOG":[["Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-SCRIPT\">guard-evaluation-examples-script</see> for a script that shows \u000A the interaction of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s with the value set by <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>.  Here, we present a log resulting from running this \u000A script.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> for discussion of the interaction between \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s and <see topic=\"ACL2____GUARD\">guard</see>-checking that is illustrated by this \u000A script.  Also see <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see> for a succinct table, with \u000A associated discussion, that covers in detail the interactions illustrated \u000A here.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :mode :program))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000ASummary\u000AForm:  ( DEFUN FACT ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 !&gt;:set-guard-checking t\u000A\u000AGuard-checking-on already has value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the :program function symbol\u000AFACT, which is (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call\u000A(FACT T).  See :DOC trace for a useful debugging utility.  See :DOC\u000Aset-guard-checking for information about suppressing this check with\u000A(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the :program function symbol\u000AFACT, which is (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call\u000A(FACT T).  See :DOC trace for a useful debugging utility.  See :DOC\u000Aset-guard-checking for information about suppressing this check with\u000A(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:u\u000A\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 &gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                         :mode :program))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000ASummary\u000AForm:  ( DEFUN FACT ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 &gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:u\u000A\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 &gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :verify-guards nil\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000AFor the admission of FACT we will use the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).  The non-trivial part of the measure conjecture is\u000A\u000A[[output omitted here]]\u000A\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 &gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A[[Comment added to the log:\u000A  Normally you would get a message about guard-checking being\u000A  inhibited on recursive calls.  However, when a function is\u000A  traced the guard-checking is done on recursive calls unless\u000A  the guards have been verified (see :DOC verify-guards).\u000A]]\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A[[Comment added to the log:\u000A  In spite of the warning above, guards are checked here on\u000A  self-recursive calls, because the function is traced.\u000A]]\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)\u000A\u000AComputing the guard conjecture for FACT....\u000A\u000AThe guard conjecture for FACT is trivial to prove, given the :compound-\u000Arecognizer rule POSP-COMPOUND-RECOGNIZER, primitive type reasoning\u000Aand the :type-prescription rule FACT.  FACT is compliant with Common\u000ALisp.\u000A\u000ASummary\u000AForm:  ( VERIFY-GUARDS FACT)\u000ARules: ((:COMPOUND-RECOGNIZER POSP-COMPOUND-RECOGNIZER)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:TYPE-PRESCRIPTION FACT))\u000AWarnings:  None\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A FACT\u000AACL2 &gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:u\u000A L        1:x(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FACT (X) ...)\u000AACL2 &gt;:u\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 &gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000AFor the admission of FACT we will use the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).  The non-trivial part of the measure conjecture is\u000A\u000A[[output omitted here]]\u000A\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 &gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol FACT, which\u000Ais (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X), is violated by the arguments in the call (FACT T).\u000ASee :DOC trace for a useful debugging utility.  See :DOC set-guard-\u000Achecking for information about suppressing this check with (set-guard-\u000Achecking :none), as recommended for new users.\u000A\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:u\u000A          0:x(<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 &gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                         :verify-guards nil\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000AFor the admission of FACT we will use the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).  The non-trivial part of the measure conjecture is\u000A\u000A[[output omitted here]]\u000A\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 &gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A ((FACT))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 1)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 0)\u000A    &lt;3 (ACL2_*1*_ACL2::FACT 1)\u000A  &lt;2 (ACL2_*1*_ACL2::FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)\u000A\u000AComputing the guard conjecture for FACT....\u000A\u000AThe guard conjecture for FACT is trivial to prove, given the :compound-\u000Arecognizer rule POSP-COMPOUND-RECOGNIZER and the :type-prescription\u000Arule FACT.  FACT is compliant with Common Lisp.\u000A\u000ASummary\u000AForm:  ( VERIFY-GUARDS FACT)\u000ARules: ((:COMPOUND-RECOGNIZER POSP-COMPOUND-RECOGNIZER)\u000A        (:TYPE-PRESCRIPTION FACT))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000A\u000A[[Note added to log: No need to trace fact again after verify-guards.]]\u000A\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :all\u000A\u000ALeaving guard checking on, but changing value to :ALL.\u000A\u000AACL2 !&gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 !&gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 !&gt;:set-guard-checking :none\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;:set-guard-checking nil\u000A\u000AMasking guard violations but still checking guards except for self-\u000Arecursive calls.  To avoid guard checking entirely, :SET-GUARD-CHECKING\u000A:NONE.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fact 2)\u000A1&gt; (ACL2_*1*_ACL2::FACT 2)\u000A  2&gt; (FACT 2)\u000A    3&gt; (FACT 1)\u000A      4&gt; (FACT 0)\u000A      &lt;4 (FACT 1)\u000A    &lt;3 (FACT 1)\u000A  &lt;2 (FACT 2)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 2)\u000A2\u000AACL2 &gt;(fact t)\u000A1&gt; (ACL2_*1*_ACL2::FACT T)\u000A  2&gt; (FACT T)\u000A  &lt;2 (FACT 1)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 1)\u000A1\u000AACL2 &gt;</code>"],

 "ACL2____EVALUATOR-RESTRICTIONS":[["Meta"],"ACL2 Sources","ACL2","<p>Note: This topic, which explains some subtleties for evaluators, \u000A can probably be skipped by most readers.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> and of class <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> are stated using so-called ``evaluator'' functions.  Here we \u000A explain some restrictions related to evaluators.  Below we refer primarily to \u000A <v>:meta</v> rules, but the discussion applies equally to <v>:clause-processor</v> \u000A rules.</p> \u000A \u000A <p>In a nutshell, we require that a rule's evaluator does not support other \u000A functions in the rule, and we require that the evaluator not be introduced \u000A under a non-trivial encapsulate.  We also require that no function has an \u000A attachment (see <see topic=\"ACL2____DEFATTACH\">defattach</see>) that is both ancestral in the evaluator and \u000A also ancestral in the meta or clause-processor functions.  We explain these \u000A restrictions in detail below.</p> \u000A \u000A <p>An argument given elsewhere (see <see topic=\"ACL2____META\">meta</see>, in particular ``Aside for the \u000A logic-minded'') explains that the correctness argument for applying \u000A metatheoretic simplifiers requires that one be able to ``grow'' an evaluator \u000A (see <see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see>) to handle all functions in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.  Then we may, in essence, functionally instantiate the original \u000A evaluator to the new (``grown'') evaluator, provided that the new evaluator \u000A satisfies all of the axioms of the original.  We therefore require that the \u000A evaluator function does not support the formula of any <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> event. \u000A This notion of ``support'' (sometimes denoted ``is an ancestor of'') is \u000A defined recursively as follows: a function symbol supports a formula if either \u000A it occurs in that formula, or else it supports the definition or constraint \u000A for some function symbol that occurs in that formula.  Moreover, we require \u000A that neither the evaluator function nor its list version support the \u000A definition or constraint for any other function symbol occurring in the \u000A proposed <v>:meta</v> theorem.</p> \u000A \u000A <p>We also require that the evaluator does not support the formula of a \u000A <v>:meta</v> rule's metafunction (nor, if there is one, hypothesis metafunction) \u000A or of a <v>:clause-processor</v> rule's clause-processor function.  This \u000A requirement, along with with the analogous requirement for <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> stated above, are necessary in order to carry out the functional \u000A instantiation argument alluded to above, as follows (where the reader may find \u000A it useful to have some familiarity with the paper ``Structured Theory \u000A Development for a Mechanized Logic'' (Journal of Automated Reasoning 26, no. 2 \u000A (2001), pages 161-203).  By the usual conservativity argument, we know that \u000A the rule follows logically from the axiomatic events for its supporters.  This \u000A remains true if we functionally instantiate the evaluator with one \u000A corresponding to all the functions symbols of the current session, since none \u000A of the definitions of supporters of defaxioms or metafunctions are hit by that \u000A functional substitution.</p> \u000A \u000A <p>Notice though that the argument above depends on knowing that the rule is \u000A not itself an axiom about the evaluator!  Therefore, we also restrict \u000A evaluators so that they are not defined in the scope of a superior <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event with non-empty signature, in order to avoid an even more \u000A subtle problem.  The aforementioned correctness argument depends on knowing \u000A that the rule is provable from the axioms on the evaluator and metafunction \u000A (and hypothesis metafunction, if any).  The additional restriction avoids \u000A unsoundness!  The following events, if allowed, produce a proof that <v>(f\u000A x)</v> equals <v>t</v> even though, as shown below, that does not follow logically \u000A from the axioms introduced.</p> \u000A \u000A <code>; Introduce our metafunction.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-cancel (<see topic=\"ACL2____TERM\">term</see>)\u000A  (<see topic=\"ACL2____CASE-MATCH\">case-match</see> term\u000A    (('f ('g))\u000A     *t*)\u000A    (&amp; term)))\u000A\u000A; Introduce our evaluator and prove our meta rule, but in the same\u000A; encapsulate!\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((f (x) t))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A\u000A (<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A   ((f x)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> correctness-of-my-cancel\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)\u000A          (evl (my-cancel x) a))\u000A   :rule-classes ((:meta :trigger-fns (f)))))\u000A\u000A; Prove that (f x) = t.\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFSTUB\">defstub</see> c () t))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A         ()\u000A         (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g () (c)))\u000A         (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> g (g))))\u000A         (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> f-g\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (g)) t)\u000A                  :rule-classes nil))\u000A         (<see topic=\"ACL2____DEFTHM\">defthm</see> f-c\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (c)) t)\u000A           :hints ((&quot;Goal&quot; :use f-g\u000A                    :in-theory (<see topic=\"ACL2____E_F2D\">e/d</see> (g) (correctness-of-my-cancel))))\u000A           :rule-classes nil)))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> f-t\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) t)\u000A   :hints ((&quot;Goal&quot; :by (:functional-instance\u000A                        f-c\u000A                        (c (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> () x)))))\u000A   :rule-classes nil))</code> \u000A \u000A <p>To see that the term <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) t)</v> does not follow logically from \u000A the axiomatic <see topic=\"ACL2____EVENTS\">events</see> above, consider following the above definition of \u000A <v>my-cancel</v> with the following <see topic=\"ACL2____EVENTS\">events</see> instead.</p> \u000A \u000A <code>; (<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-cancel (<see topic=\"ACL2____TERM\">term</see>) ...) as before, then:\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g ()\u000A  nil)\u000A\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A   ((f x) (g)))</code> \u000A \u000A <p>These events imply the axiomatic events above, because we still have the \u000A definition of <v>my-cancel</v>, we have a stronger <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> event, \u000A and we can now prove <v>correctness-of-my-cancel</v> exactly as it is stated \u000A above.  So, the rule <v>f-t</v> is a logical consequence of the chronology of \u000A the current session.  However, in the current session we can also prove the \u000A following rule, which contradicts <v>f-t</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> f-not-t\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f t) nil)\u000A  :rule-classes nil)</code> \u000A \u000A <p>It follows that the current session logically yields a contradiction!</p> \u000A \u000A <p>Erik Reeber has taken the above example and modified it to prove <v>nil</v> \u000A in ACL2 Version_3.1, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-cancel (<see topic=\"ACL2____TERM\">term</see>)\u000A   (<see topic=\"ACL2____CASE-MATCH\">case-match</see> term\u000A     (('f ('g))\u000A      *t*)\u000A     (('f2 ('g2))\u000A      *t*)\u000A     (&amp; term)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x)\u000A   (<see topic=\"COMMON-LISP____NOT\">not</see> x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g2 ()\u000A   nil)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ((f (x) t))\u000A\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A\u000A  (<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A    ((f x)\u000A     (f2 x)\u000A     (g2)))\u000A\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> correctness-of-my-cancel\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)\u000A           (evl (my-cancel x) a))\u000A    :rule-classes ((:meta :trigger-fns (f)))))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ()\u000A\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFSTUB\">defstub</see> c () t))\u000A\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A          ()\u000A          (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g () (c)))\u000A          (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> g (g))))\u000A          (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> f-g\u000A                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (g)) t)\u000A                   :rule-classes nil))\u000A          (<see topic=\"ACL2____DEFTHM\">defthm</see> f-c\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (c)) t)\u000A            :hints ((&quot;Goal&quot; :use f-g\u000A                     :in-theory (<see topic=\"ACL2____E_F2D\">e/d</see> (g) (correctness-of-my-cancel))))\u000A            :rule-classes nil)))\u000A\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> f-t\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) t)\u000A    :hints ((&quot;Goal&quot; :by (:functional-instance\u000A                         f-c\u000A                         (c (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> () x)))))\u000A    :rule-classes nil))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g ()\u000A   nil)\u000A\u000A; Below is the expansion of the following defevaluator, changed slightly as\u000A; indicated by comments.\u000A; (<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl2 evl2-list ((f x) (f2 x) (g) (g2)))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see>\u000A  (((EVL2 * *) =&gt; *)\u000A   ((EVL2-LIST * *) =&gt; *))\u000A  (<see topic=\"ACL2____SET-INHIBIT-WARNINGS\">SET-INHIBIT-WARNINGS</see> &quot;theory&quot;)\u000A  (<see topic=\"ACL2____LOCAL\">LOCAL</see> (<see topic=\"ACL2____IN-THEORY\">IN-THEORY</see> *DEFEVALUATOR-FORM-BASE-THEORY*))\u000A  (<see topic=\"ACL2____LOCAL\">LOCAL</see>\u000A   (<see topic=\"ACL2____MUTUAL-RECURSION\">MUTUAL-RECURSION</see> (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> EVL2 (X A)\u000A                       (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :VERIFY-GUARDS NIL\u000A                                       :MEASURE (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X)\u000A                                       :WELL-FOUNDED-RELATION O&lt;\u000A                                       :MODE :LOGIC))\u000A                       (<see topic=\"COMMON-LISP____COND\">COND</see> ((<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X) (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> X A)))\u000A                             ((<see topic=\"COMMON-LISP____ATOM\">ATOM</see> X) NIL)\u000A                             ((<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'QUOTE) (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A                             ((<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))\u000A                              (EVL2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))\u000A                                    (<see topic=\"ACL2____PAIRLIS_42\">PAIRLIS$</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A                                              (EVL2-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))))\u000A                             ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'F) ; changed f to f2 just below\u000A                              (F2 (EVL2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) A)))\u000A                             ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'F2)\u000A                              (F2 (EVL2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) A)))\u000A                             ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'G) (G))\u000A                             ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'G2) (G2))\u000A                             (T NIL)))\u000A                     (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> EVL2-LIST (X-LST A)\u000A                       (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :MEASURE (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X-LST)\u000A                                       :WELL-FOUNDED-RELATION O&lt;))\u000A                       (<see topic=\"COMMON-LISP____COND\">COND</see> ((<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X-LST) NIL)\u000A                             (T (<see topic=\"COMMON-LISP____CONS\">CONS</see> (EVL2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X-LST) A)\u000A                                      (EVL2-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X-LST) A)))))))\u000A\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-1\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X)\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A)\u000A                    (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> X A)))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-2\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'QUOTE))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A) (<see topic=\"COMMON-LISP____CADR\">CADR</see> X))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-3\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A)\u000A                    (EVL2 (<see topic=\"COMMON-LISP____CADDAR\">CADDAR</see> X)\u000A                          (<see topic=\"ACL2____PAIRLIS_42\">PAIRLIS$</see> (<see topic=\"COMMON-LISP____CADAR\">CADAR</see> X)\u000A                                    (EVL2-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-4\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X-LST))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2-LIST X-LST A) NIL)))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-5\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X-LST)\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2-LIST X-LST A)\u000A                    (<see topic=\"COMMON-LISP____CONS\">CONS</see> (EVL2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X-LST) A)\u000A                          (EVL2-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X-LST) A)))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-6\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'F))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A) ; changed f to f2 just below\u000A                    (F2 (EVL2 (<see topic=\"COMMON-LISP____CADR\">CADR</see> X) A)))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-7\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'F2))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A)\u000A                    (F2 (EVL2 (<see topic=\"COMMON-LISP____CADR\">CADR</see> X) A)))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-8\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'G))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A) (G))))\u000A  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> EVL2-CONSTRAINT-9\u000A    (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'G2))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (EVL2 X A) (G2)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> f2-t\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f2 x) t)\u000A   :hints ((&quot;Goal&quot; :by (:functional-instance\u000A                        f-t\u000A                        (f f2)\u000A                        (evl evl2)\u000A                        (evl-list evl2-list)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bug-implies-nil\u000A   nil\u000A   :hints ((&quot;Goal&quot; :use ((:instance f2-t (x t)))))\u000A   :rule-classes nil)</code> \u000A \u000A <p>Finally, we also require that no function has an attachment (see <see topic=\"ACL2____DEFATTACH\">defattach</see>) that is both ancestral in the evaluator and also ancestral in the \u000A meta or clause-processor functions.  (If you don't use <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> then \u000A you can ignore this condition.)  Without this restriction, the following \u000A events prove <v>nil</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f () t)\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A  ((f)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-meta-fn (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x '(f))\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (f))\u000A    x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> my-meta-fn-correct\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (evl x a)\u000A         (evl (my-meta-fn x) a))\u000A  :rule-classes ((:meta :trigger-fns (f))))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> constant-nil ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  nil)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> f constant-nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> f-is-nil\u000A; proved using my-meta-fn-correct\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f) nil)\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use ((:functional-instance\u000A                         f-is-nil\u000A                         (f (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> () t))))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>To see why this restriction is sufficient, see a comment in the ACL2 source \u000A code entitled ``; Essay on Correctness of Meta Reasoning.''</p>"],

 "ACL2____WORMHOLE-IMPLEMENTATION":[["Wormhole"],"ACL2 Sources","ACL2","<p>What happens when you call <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>?  Recall that a typical \u000A call of the function looks like this:</p> \u000A \u000A <code>(<see topic=\"ACL2____WORMHOLE\">wormhole</see> 'name\u000A          '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) ...)\u000A          input\u000A          form\u000A          :ld-verbose ...\u000A          ...)</code> \u000A \u000A <p>A brief recap of the advertised semantics for <v>wormhole</v> establishes our \u000A terminology: When the above <v>wormhole</v> is evaluated, the \u000A <v>lambda</v>-expression is applied to the wormhole's status and the result is \u000A stored as the new status.  Then, if the entry-code of the new status is \u000A <v>:ENTER</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> is invoked on a copy of the ``current state'' with the \u000A specified <v>ld-</v> ``special variables;'' output is directed to the comment \u000A window.  In that copy of the state, the state-global variable \u000A <v>wormhole-input</v> is set to the value of <v>input</v> and the state-global \u000A variable <v>wormhole-status</v> is set to the (new) status computed by the \u000A <v>lambda</v>-expression.  Thus, inside the wormhole, <v>(<see topic=\"ACL2_____04\">@</see> wormhole-input)</v> \u000A returns the list of inputs, <v>(<see topic=\"ACL2_____04\">@</see> wormhole-status)</v> returns the current \u000A status, and <v>(<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status ...)</v> sets the wormhole's status.  The \u000A first form executed by the <v>ld</v> is the value of <v>form</v> and unless that \u000A form returns <v>(value :q)</v>, causing the <v>ld</v> to quit, the <v>ld</v> \u000A proceeds to take subsequent input from the comment window.  Upon exiting from \u000A <v>ld</v>, the wormhole state ``evaporates.''  The wormhole's status upon exit \u000A is remembered and restored the next time the wormhole is entered.</p> \u000A \u000A <p>Here is what really happens.</p> \u000A \u000A <p>Each wormhole's status is recorded in an alist stored in a Common Lisp \u000A global variable named <v>*wormhole-status-alist*</v>.  This variable is not part \u000A of the ACL2 state.  If you exit the ACL2 loop with <v>:q</v> you can inspect the \u000A value of <v>*wormhole-status-alist*</v>.  When the <v>lambda</v>-expression is \u000A evaluated it is applied to the value associated with <v>name</v> in the alist \u000A and the result is stored back into that alist.  This step is performed by \u000A <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt>.  To make things more efficient, <v>wormhole-eval</v> is \u000A just a macro that expands into a <v>let</v> that binds the <v>lambda</v> formal to \u000A the current status and whose body is the <v>lambda</v> body.  Guard clauses are \u000A generated from the body, with one exception: the <v>lambda</v> formal is \u000A replaced by a new variable so that no prior assumptions are available about \u000A the value of the the wormhole status.</p> \u000A \u000A <p>If the newly computed status has an entry code of <v>:ENTER</v> <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A will be invoked.  But we don't really copy state, of course.  Instead we will \u000A invoke <v>ld</v> on the live state, which is always available in the von Neumann \u000A world in which ACL2 is implemented.  To give the illusion of copying state, we \u000A will undo changes to the state upon exiting.  To support this, we do two \u000A things just before invoking <v>ld</v>: we bind a Common Lisp special variable is \u000A to <v>t</v> to record that ACL2 is in a wormhole, and we initialize an \u000A accumulator that will be used to record state changes made while in the \u000A wormhole.</p> \u000A \u000A <p>Then <v>ld</v> is invoked, with first argument, <v>standard-oi</v>, being set \u000A to <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> form *standard-oi*)</v>.  According to the standard semantics of \u000A <v>ld</v>, this reads and evaluates <v>form</v> and then the forms in the \u000A specified channel.  The standard channels are directed to and from the \u000A terminal, which is the physical realization of the comment window.</p> \u000A \u000A <p>All state modifying functions of ACL2 are sensitive to the special variable \u000A that indicates that evaluation is in a wormhole.  Some ACL2 state-modifying \u000A functions (e.g., those that modify the file system like <tt><see topic=\"ACL2____WRITE-BYTE_42\">write-byte$</see></tt>) \u000A are made to cause an error if invoked inside a wormhole on a file other than \u000A the terminal.  Others, like <v>f-put-global</v> (the function behind such \u000A features as <v>assign</v> and maintenance of the ACL2 logical world by such \u000A events as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>) are made to record the old value \u000A of the state component being changed; these records are kept in the \u000A accumulator initialized above.</p> \u000A \u000A <p>Upon exit from <v>ld</v> for any reason, the final value of <v>(<see topic=\"ACL2_____04\">@</see>\u000A wormhole-status)</v> is stored in <v>*wormhole-status-alist*</v> and then the \u000A accumulator is used to ``undo'' all the state changes.</p> \u000A \u000A <p><v>Wormhole</v> always returns <v>nil</v>.</p>"],

 "ACL2____USING-COMPUTED-HINTS-7":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>A problem with the example in <see topic=\"ACL2____USING-COMPUTED-HINTS-6\">using-computed-hints-6</see> is that \u000A exactly one simplification occurs between each (effective) firing of the hint. \u000A Much more commonly we wish to fire a hint once a subgoal has become stable \u000A under simplification.</p> \u000A \u000A <p>A classic example of this is when we are dealing with an interpreter for \u000A some state machine.  We typically do not want the ``step'' function to open up \u000A on the symbolic representation of a state until that state has been maximally \u000A simplified.  We will illustrate with a simple state machine.</p> \u000A \u000A <p>Let us start by defining the step function, <v>stp</v>, and the corresponding \u000A <v>run</v> function that applies it a given number of times.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> stp (s)\u000A  (<see topic=\"COMMON-LISP_____B2\">+</see> 1 s))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> run (s n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      s\u000A      (run (stp s) (<see topic=\"COMMON-LISP____-\">-</see> n 1))))</code> \u000A \u000A <p>The step function here is trivial: a state is just a number and the step \u000A function increments it.  In this example we will not be interested in the \u000A theorems we prove but in how we prove them.  The formula we will focus on \u000A is</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (run s 7) xxx))</code> \u000A \u000A <p>This is not a theorem, of course.  But we want to test our advice on \u000A non-theorems because we do not want the advice to work only for proofs that \u000A succeed.  (In the past, we gave advice about using computed hints and that \u000A advice caused the theorem prover to run forever when given formulas that it \u000A couldn't prove -- but most of the time the system is presented with formulas \u000A it cannot prove!)</p> \u000A \u000A <p>Furthermore, without some kind of additional rules, the <v>(run s 7)</v> \u000A expression in the conjecture above will not expand at all, because ACL2's \u000A heuristics do not approve.</p> \u000A \u000A <p>In fact, we do not want to take chances that <v>run</v> will be expanded -- \u000A we want to control its expansion completely.  Therefore, disable <v>run</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> run))</code> \u000A \u000A <p>Now, what do we want?  (That is always a good question to ask!)  We want \u000A <v>(run s 7)</v> to expand ``slowly.''  In particular, we want it to expand \u000A once, to <v>(run (stp s) 6)</v>.  Then we want the <v>stp</v> to be expanded and \u000A fully simplified before the <v>run</v> expression is expanded again.  That is, \u000A we want to force the expansion of <v>run</v> whenever the goal is stable under \u000A simplification.  This is sometimes called ``staged simplification.''</p> \u000A \u000A <p>We can achieve staged simplification for any given function symbol by \u000A defining the functions shown below and then using a simple computed hint:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (run s 7) xxx)\u000A     :hints ((stage run)))</code> \u000A \u000A <p>By inspecting how <v>stage</v> is defined you can see how to extend it, but \u000A we explain as we go.  To experiment, you can just paste the definitions (and \u000A defmacro) below into your ACL2 shell and then try the <v>thm</v> command.</p> \u000A \u000A <p>First, define this pair of mutually recursive functions. \u000A <v>Find-first-call</v> finds the first call of the function symbol <v>fn</v> in a \u000A given term.</p> \u000A \u000A <code>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-first-call (fn term)\u000A ; Find the first call of fn in term.\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((variablep term) nil)\u000A        ((fquotep term) nil)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ffn-symb term) fn)\u000A         term)\u000A        (t (find-first-call-lst fn (fargs term)))))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-first-call-lst (fn lst)\u000A ; Find the first call of fn in a list of terms.\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A        (t (<see topic=\"COMMON-LISP____OR\">or</see> (find-first-call fn (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A               (find-first-call-lst fn (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))))</code> \u000A \u000A <p>We will arrange for the computed hint to generate an <v>:EXPAND</v> hint for \u000A the first call of <v>fn</v>, whenever the goal becomes stable under \u000A simplification.  If no call is found, the hint will do nothing.  To make sure \u000A the hint will not loop indefinitely (for example, by forcing <v>fn</v> to expand \u000A only to have the rewriter ``fold'' it back up again), we will provide the hint \u000A with a bound that stops it after some number of iterations.  Here is the basic \u000A function that creates the <v>expand</v> hint and replaces itself to count \u000A down.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> stage1 (fn max clause flg)\u000A; If the clause is stable under simplification and there is a call of\u000A; fn in it, expand it.  But don't do it more than max times.\u000A (<see topic=\"COMMON-LISP____LET\">let</see> ((temp (<see topic=\"COMMON-LISP____AND\">and</see> flg\u000A                  (find-first-call-lst fn clause))))\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> temp\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> max)\u000A           (<see topic=\"ACL2____CW\">cw</see> &quot;~%~%HINT PROBLEM:  The maximum repetition count of ~\u000A                your STAGE hint been reached without eliminating ~\u000A                all of the calls of ~x0.  You could supply a larger ~\u000A                count with the optional second argument to STAGE ~\u000A                (which defaults to 100).  But think about what is ~\u000A                happening! Is each stage permanently eliminating a ~\u000A                call of ~x0?~%~%&quot;\u000A               fn)\u000A         `(:computed-hint-replacement\u000A            ((stage1 ',fn ,(<see topic=\"COMMON-LISP____-\">-</see> max 1)\u000A                     clause\u000A                     stable-under-simplificationp))\u000A           :expand (,temp)))\u000A     nil)))</code> \u000A \u000A <p>Suppose that when <v>stage1</v> is called, <v>fn</v> is the function we want to \u000A expand, <v>max</v> is the maximum number of iterations of this expansion, \u000A <v>clause</v> is the current goal clause, and <v>flg</v> is the value of the \u000A <v>stable-under-simplificationp</v> flag.  Then if <v>clause</v> is stable and we \u000A can find a call of <v>fn</v> in it, we ask whether <v>max</v> is exhausted.  If \u000A so, we print an ``error message'' to the comment window with <tt><see topic=\"ACL2____CW\">cw</see></tt> and \u000A return <v>nil</v> (the value of <v>cw</v>).  That <v>nil</v> means the hint does \u000A nothing.  But if <v>max</v> is not yet exhausted, we return a new hint.  As you \u000A can see above, the hint replaces itself with another <v>stage1</v> hint with the \u000A same <v>fn</v> and a decremented <v>max</v> to be applied to the new <v>clause</v> \u000A and the then-current value of <v>stable-under-simplificationp</v>.  The hint \u000A also contains an <v>:expand</v> directive for the call of <v>fn</v> found.</p> \u000A \u000A <p>Thus, if the computed hint was:</p> \u000A \u000A <code>(stage1 'run 5 clause stable-under-simplificationp)</code> \u000A \u000A <p>and <v>(run s 7)</v> occurs in the clause, then it will generate</p> \u000A \u000A <code>(:computed-hint-replacement\u000A  ((stage1 'run 4 clause stable-under-simplificationp))\u000A :expand ((run s 7)))</code> \u000A \u000A <p>which will in turn replace the old <v>stage1</v> hint with the new one and \u000A will apply <v>:expand ((run s 7))</v> to the current goal.</p> \u000A \u000A <p>We can make this more convenient by defining the macro:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> stage (fn &amp;optional (<see topic=\"COMMON-LISP____MAX\">max</see> '100))\u000A `(stage1 ',fn ,max clause stable-under-simplificationp))</code> \u000A \u000A <p>Note that the macro allows us to either provide the maximum bound or let it \u000A default to 100.</p> \u000A \u000A <p>Henceforth, we can type</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (run s 7) xxx)\u000A     :hints ((stage run)))</code> \u000A \u000A <p>to stage the opening of <v>run</v> up to 100 times, or we can write</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (run s 7) xxx)\u000A     :hints ((stage run 5)))</code> \u000A \u000A <p>to stage it only 5 times.  In the latter example, the system with print a \u000A ``error message'' after the fifth expansion.</p> \u000A \u000A <p>Note that if we executed</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> '((stage run)))</code> \u000A \u000A <p>then we could attack all theorems (involving <v>run</v>) with staged \u000A simplification (up to bound 100), without typing an explicit hint.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (run s 7) xxx))</code> \u000A \u000A <p>Using techniques similar to those above we have implemented ``priority \u000A phased simplification'' and provided it as a book.  See community book \u000A <v>books/misc/priorities.lisp</v>.  This is an idea suggested by Pete Manolios, \u000A by which priorities may be assigned to rules and then the simplifier \u000A simplifies each subgoal maximally under the rules of a given priority before \u000A enabling the rules of the next priority level.  The book above documents both \u000A how we implement it with computed hints and how to use it.</p> \u000A \u000A <p>Here is another example of using the <v>stable-under-simplificationp</v> flag \u000A to delay certain actions.  It defines a default hint, see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>, which will enable <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see> on precisely \u000A those goals which are stable-under-simplificationp.  It also uses the \u000A <v>HISTORY</v> and <v>PSPV</v> variables to determine when toggling <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see> is appropriate.  These variables are documented only in \u000A the source code.  If you start using these variables extensively, please \u000A contact the developers of ACL2 or Robert Krug (<v>rkrug@cs.utexas.edu</v>) and \u000A let us know how we can help.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nonlinearp-default-hint (stable-under-simplificationp hist pspv)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> (stable-under-simplificationp\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ACCESS\">access</see> rewrite-constant\u000A                          (<see topic=\"ACL2____ACCESS\">access</see> prove-spec-var pspv :rewrite-constant)\u000A                          :nonlinearp))\u000A             '(:computed-hint-replacement t\u000A               :nonlinearp t)\u000A           nil))\u000A        ((<see topic=\"ACL2____ACCESS\">access</see> rewrite-constant\u000A                 (<see topic=\"ACL2____ACCESS\">access</see> prove-spec-var pspv :rewrite-constant)\u000A                 :nonlinearp)\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAAR\">caar</see> hist) 'SETTLED-DOWN-CLAUSE))\u000A             '(:computed-hint-replacement t\u000A               :nonlinearp nil)\u000A           nil))\u000A        (t\u000A         nil)))</code>"],

 "ACL2____TIPS":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>We present here some tips for using ACL2 effectively.  Though this \u000A collection is somewhat <i>ad hoc</i>, we try to provide some organization, \u000A albeit somewhat artificial: for example, the sections overlap, and no \u000A particular order is intended.  This material has been adapted by Bill Young \u000A from a very similar list for Nqthm that appeared in the conclusion of: \u000A ``Interaction with the Boyer-Moore Theorem Prover: A Tutorial Study Using the \u000A Arithmetic-Geometric Mean Theorem,'' by Matt Kaufmann and Paolo Pecchiari, CLI \u000A Technical Report 100, June, 1995.  We also draw from a similar list in Chapter \u000A 13 of ``A Computational Logic Handbook'' by R.S. Boyer and J \u000A S. Moore (Academic Press, 1988).  We'll refer to this as ``ACLH'' below.</p> \u000A \u000A <p>These tips are organized roughly as follows.</p> \u000A \u000A <blockquote><p>A. ACL2 Basics</p> \u000A \u000A <p>B. Strategies for creating events</p> \u000A \u000A <p>C. Dealing with failed proofs</p> \u000A \u000A <p>D. Performance tips</p> \u000A \u000A <p>E. Miscellaneous tips and knowledge</p> \u000A \u000A <p>F. Some things you DON'T need to know</p></blockquote> \u000A \u000A <p><i>ACL2 BASICS</i></p> \u000A \u000A <p><b>A1. The ACL2 logic.</b><br></br> \u000A \u000A This is a logic of total functions.  For example, if <v>A</v> and <v>B</v> are \u000A less than or equal to each other, then we need to know something more in order \u000A to conclude that they are equal (e.g., that they are numbers).  This kind of \u000A twist is important in writing definitions; for example, if you expect a \u000A function to return a number, you may want to apply the function <tt><see topic=\"ACL2____FIX\">fix</see></tt> or \u000A some variant (e.g., <tt><see topic=\"ACL2____NFIX\">nfix</see></tt> or <tt><see topic=\"ACL2____IFIX\">ifix</see></tt>) in case one of the formals \u000A is to be returned as the value.</p> \u000A \u000A <p>ACL2's notion of ordinals is important on occasion in supplying ``measure \u000A <see topic=\"ACL2____HINTS\">hints</see>'' for the acceptance of recursive definitions.  Be sure that your \u000A measure is really an ordinal.  Consider the following example, which ACL2 \u000A fails to admit (as explained below).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> cnt (<see topic=\"ACL2____NAME\">name</see> a i x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"COMMON-LISP_____B2\">+</see> 1 i)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 i))\u000A         0)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"ACL2____AREF1\">aref1</see> name a i))\u000A         (<see topic=\"COMMON-LISP____1_B2\">1+</see> (cnt name a (<see topic=\"COMMON-LISP____1-\">1-</see> i) x)))\u000A        (t (cnt name a (<see topic=\"COMMON-LISP____1-\">1-</see> i) x))))</code> \u000A \u000A <p>One might think that <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 i)</v> is a reasonable measure, since we know \u000A that <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 i)</v> is a positive integer in any recursive call of <v>cnt</v>, and \u000A positive integers are ACL2 ordinals (see <see topic=\"ACL2____O-P\">o-p</see>).  However, the ACL2 logic \u000A requires that the measure be an ordinal unconditionally, not just under the \u000A governing assumptions that lead to recursive calls.  An appropriate fix is to \u000A apply <tt><see topic=\"ACL2____NFIX\">nfix</see></tt> to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 i)</v>, i.e., to use</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____NFIX\">nfix</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 i))))</code> \u000A \u000A <p>in order to guarantee that the measure will always be an ordinal (in fact, \u000A a positive integer).</p> \u000A \u000A <p>For more about admissibility of recursive definitions, see <see topic=\"COMMON-LISP____DEFUN\">defun</see>, in \u000A particular the discussion of termination.</p> \u000A \u000A <p><b>A2. Simplification.</b><br></br> \u000A \u000A The ACL2 simplifier is basically a rewriter, with some ``<see topic=\"ACL2____LINEAR\">linear</see> \u000A arithmetic'' thrown in.  One needs to understand the notion of conditional \u000A rewriting.  See <see topic=\"ACL2____REWRITE\">rewrite</see>.</p> \u000A \u000A <p><b>A3. Parsing of rewrite rules.</b><br></br></p> \u000A \u000A <p>ACL2 parses <see topic=\"ACL2____REWRITE\">rewrite</see> rules roughly as explained in ACLH, \u000A <i>except</i> that it never creates ``unusual'' rule classes.  In ACL2, if you \u000A want a <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule, for example, you must specify <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> in the <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>, and also \u000A see <see topic=\"ACL2____REWRITE\">rewrite</see> and see <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p><b>A4. Linear arithmetic.</b><br></br> \u000A \u000A On this subject, it should suffice to know that the prover can handle truths \u000A about <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> and <tt><see topic=\"COMMON-LISP____-\">-</see></tt>, and that <see topic=\"ACL2____LINEAR\">linear</see> rules (see above) are \u000A somehow ``thrown in the pot'' when the prover is doing such reasoning. \u000A Perhaps it's also useful to know that <see topic=\"ACL2____LINEAR\">linear</see> rules can have hypotheses, \u000A and that conditional rewriting is used to relieve those hypotheses.</p> \u000A \u000A <p><b>A5. Events.</b><br></br> \u000A \u000A Over time, the expert ACL2 user will know some subtleties of its <see topic=\"ACL2____EVENTS\">events</see>.  For example, <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> and <see topic=\"ACL2____HINTS\">hints</see> are \u000A important, and they distinguish between a function and its executable \u000A counterpart.</p> \u000A \u000A <p><i>B. STRATEGIES FOR CREATING EVENTS</i></p> \u000A \u000A <p>In this section, we concentrate on the use of definitions and <see topic=\"ACL2____REWRITE\">rewrite</see> rules.  There are quite a few kinds of rules allowed in ACL2 besides \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> rules, though most beginning users probably won't usually need \u000A to be aware of them.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for details.  In particular, \u000A there is support for <see topic=\"ACL2____CONGRUENCE\">congruence</see> rewriting.  Also see <see topic=\"ACL2____RUNE\">rune</see> \u000A (``RUle NamE'') for a description of the various kinds of rules in the \u000A system.</p> \u000A \u000A <p><b>B1. Use high-level strategy.</b><br></br> \u000A \u000A Decompose theorems into ``manageable'' lemmas (admittedly, experience helps \u000A here) that yield the main result ``easily.''  It's important to be able to \u000A outline non-trivial proofs by hand (or in your head).  In particular, avoid \u000A submitting goals to the prover when there's no reason to believe that the goal \u000A will be proved and there's no ``sense'' of how an induction argument would \u000A apply.  It is often a good idea to avoid induction in complicated theorems \u000A unless you have a reason to believe that it is appropriate.</p> \u000A \u000A <p><b>B2. Write elegant definitions.</b><br></br> \u000A \u000A Try to write definitions in a reasonably modular style, especially recursive \u000A ones.  Think of ACL2 as a <see topic=\"ACL2____PROGRAMMING\">programming</see> language whose procedures are \u000A definitions and lemmas, hence we are really suggesting that one follow good \u000A <see topic=\"ACL2____PROGRAMMING\">programming</see> style (in order to avoid duplication of ``code,'' for \u000A example).</p> \u000A \u000A <p>When possible, complex functions are best written as compositions of \u000A simpler functions.  The theorem prover generally performs better on primitive \u000A recursive functions than on more complicated recursions (such as those using \u000A accumulating parameters).</p> \u000A \u000A <p>Avoid large non-recursive definitions which tend to lead to large case \u000A explosions.  If such definitions are necessary, try to prove all relevant \u000A facts about the definitions and then <see topic=\"ACL2____DISABLE\">disable</see> them.</p> \u000A \u000A <p>Whenever possible, avoid mutual recursion if you care to prove anything \u000A about your functions.  The induction heuristics provide essentially no help \u000A with reasoning about mutually defined functions.  Mutually recursive functions \u000A can usually be combined into a single function with a ``flag'' argument. \u000A (However, see <see topic=\"ACL2____MUTUAL-RECURSION-PROOF-EXAMPLE\">mutual-recursion-proof-example</see> for a small example of \u000A proof involving mutually recursive functions.)</p> \u000A \u000A <p><b>B3. Look for analogies.</b><br></br> \u000A \u000A Sometimes you can easily edit sequences of lemmas into sequences of lemmas \u000A about analogous functions.</p> \u000A \u000A <p><b>B4. Write useful rewrite rules.</b><br></br> \u000A \u000A As explained in A3 above, every <see topic=\"ACL2____REWRITE\">rewrite</see> rule is a directive to the \u000A theorem prover, usually to replace one <see topic=\"ACL2____TERM\">term</see> by another.  The directive \u000A generated is determined by the syntax of the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> submitted.  Never \u000A submit a <see topic=\"ACL2____REWRITE\">rewrite</see> rule unless you have considered its interpretation as \u000A a proof directive.</p> \u000A \u000A <p><b>B4a.  Rewrite rules should simplify.</b><br></br> \u000A \u000A Try to write <see topic=\"ACL2____REWRITE\">rewrite</see> rules whose right-hand sides are in some sense \u000A ``simpler than'' (or at worst, are variants of) the left-hand sides.  This \u000A will help to avoid infinite loops in the rewriter.</p> \u000A \u000A <p><b>B4b.  Avoid needlessly expensive rules.</b><br></br> \u000A \u000A Consider a rule whose conclusion's left-hand side (or, the entire conclusion) \u000A is a <see topic=\"ACL2____TERM\">term</see> such as <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> that matches many <see topic=\"ACL2____TERM\">term</see>s \u000A encountered by the prover.  If in addition the rule has complicated \u000A hypotheses, this rule could slow down the prover greatly.  Consider switching \u000A the conclusion and a complicated hypothesis (negating each) in that case.</p> \u000A \u000A <p><b>B4c. The ``Knuth-Bendix problem''.</b><br></br> \u000A \u000A Be aware that left sides of <see topic=\"ACL2____REWRITE\">rewrite</see> rules should match the ``normalized \u000A forms'', where ``normalization'' (rewriting) is inside out.  Be sure to avoid \u000A the use of nonrecursive function symbols on left sides of <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rules, except when those function symbols are <see topic=\"ACL2____DISABLE\">disable</see>d, because they \u000A tend to be expanded away before the rewriter would encounter an instance of \u000A the left side of the rule.  Also assure that subexpressions on the left hand \u000A side of a rewrite rule are in simplified form; see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> \u000A example <v>books/demos/knuth-bendix-problem-1.lisp</v>.</p> \u000A \u000A <p><b>B4d. Avoid proving useless rules.</b><br></br> \u000A \u000A Sometimes it's tempting to prove a <see topic=\"ACL2____REWRITE\">rewrite</see> rule even before you see how \u000A it might find application.  If the rule seems clean and important, and not \u000A unduly expensive, that's probably fine, especially if it's not too hard to \u000A prove.  But unless it's either part of the high-level strategy or, on the \u000A other hand, intended to get the prover past a particular unproved goal, it may \u000A simply waste your time to prove the rule, and then clutter the database of \u000A rules if you are successful.</p> \u000A \u000A <p><b>B4e. State rules as strongly as possible, usually.</b><br></br> \u000A \u000A It's usually a good idea to state a rule in the strongest way possible, both \u000A by eliminating unnecessary hypotheses and by generalizing subexpressions to \u000A variables.</p> \u000A \u000A <p>Advanced users may choose to violate this policy on occasion, for example \u000A in order to avoid slowing down the prover by excessive attempted application \u000A of the rule.  However, it's a good rule of thumb to make the strongest rule \u000A possible, not only because it will then apply more often, but also because the \u000A rule will often be easier to prove (see also B6 below).  New users are \u000A sometimes tempted to put in extra hypotheses that have a ``type restriction'' \u000A appearance, without realizing that the way ACL2 handles (total) functions \u000A generally lets it handle trivial cases easily.</p> \u000A \u000A <p><b>B4f. Avoid circularity.</b><br></br> \u000A \u000A A stack overflow in a proof attempt almost always results from circular \u000A rewriting.  Use <tt><see topic=\"ACL2____BRR\">brr</see></tt> to investigate the stack; see <see topic=\"ACL2____BREAK-LEMMA\">break-lemma</see>. \u000A Because of the complex heuristics, it is not always easy to define just when a \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> will cause circularity.  See the very good discussion of this \u000A topic in ACLH.</p> \u000A \u000A <p>See <see topic=\"ACL2____BREAK-LEMMA\">break-lemma</see> for a trick involving use of the forms <v>brr t</v> \u000A and <v>(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see>)</v> for inspecting loops in the rewriter.</p> \u000A \u000A <p><b>B4g. Remember restrictions on permutative rules.</b><br></br> \u000A \u000A Any rule that permutes the variables in its left hand side could cause \u000A circularity.  For example, the following axiom is automatically supplied by \u000A the system:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> commutativity-of-+\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (<see topic=\"COMMON-LISP_____B2\">+</see> y x))).</code> \u000A \u000A <p>This would obviously lead to dangerous circular rewriting if such \u000A ``permutative'' rules were not governed by a further restriction.  The \u000A restriction is that such rules will not produce a <see topic=\"ACL2____TERM\">term</see> that is \u000A ``lexicographically larger than'' the original <see topic=\"ACL2____TERM\">term</see> (see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>).  However, this sometimes prevents intended rewrites.  See \u000A Chapter 13 of ACLH for a discussion of this problem.</p> \u000A \u000A <p><b>B5. Conditional vs. unconditional rewrite rules.</b><br></br> \u000A \u000A It's generally preferable to form unconditional <see topic=\"ACL2____REWRITE\">rewrite</see> rules unless \u000A there is a danger of case explosion.  That is, rather than pairs of rules such \u000A as</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> p\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> term1 term2))</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> p)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> term1 term3))</code> \u000A \u000A <p>consider:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> term1\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> p term2 term3))</code> \u000A \u000A <p>However, sometimes this strategy can lead to case explosions: <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> \u000A <see topic=\"ACL2____TERM\">term</see>s introduce cases in ACL2.  Use your judgment.  (On the subject of \u000A <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>: <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt>, <tt><see topic=\"COMMON-LISP____CASE\">case</see></tt>, <tt><see topic=\"COMMON-LISP____AND\">and</see></tt>, and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> are macros \u000A that abbreviate <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> forms, and propositional functions such as <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> quickly expand into <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> <see topic=\"ACL2____TERM\">term</see>s.)</p> \u000A \u000A <p><b>B6. Create elegant theorems.</b><br></br> \u000A \u000A Try to formulate lemmas that are as simple and general as possible.  For \u000A example, sometimes properties about several functions can be ``factored'' into \u000A lemmas about one function at a time.  Sometimes the elimination of unnecessary \u000A hypotheses makes the theorem easier to prove, as does generalizing first by \u000A hand.</p> \u000A \u000A <p><b>B7. Use</b> <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>s <b>temporarily to explore \u000A possibilities.</b><br></br> \u000A \u000A When there is a difficult goal that seems to follow immediately (by a \u000A <v>:use</v> hint or by rewriting) from some other lemmas, you can create those \u000A lemmas as <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> <see topic=\"ACL2____EVENTS\">events</see> (or, the application of <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> to <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see topic=\"ACL2____EVENTS\">events</see>) and then double-check that the \u000A difficult goal really does follow from them.  Then you can go back and try to \u000A turn each <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> into a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  When you do that, it's \u000A often useful to <see topic=\"ACL2____DISABLE\">disable</see> any additional <see topic=\"ACL2____REWRITE\">rewrite</see> rules that you \u000A prove in the process, so that the ``difficult goal'' will still be proved from \u000A its lemmas when the process is complete.</p> \u000A \u000A <p>Better yet, rather than disabling <see topic=\"ACL2____REWRITE\">rewrite</see> rules, use the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> mechanism offered by <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> to make temporary rules \u000A completely <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the problem at hand.  See <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> and \u000A see <see topic=\"ACL2____LOCAL\">local</see>.</p> \u000A \u000A <p><b>B9. Use books.</b><br></br> \u000A \u000A Consider using previously certified <see topic=\"ACL2____BOOKS\">books</see>, especially for <see topic=\"ACL2____ARITHMETIC\">arithmetic</see> reasoning.  This cuts down the duplication of effort and starts \u000A your specification and proof effort from a richer foundation.  See <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p> \u000A \u000A <p><i>C. DEALING WITH FAILED PROOFS</i></p> \u000A \u000A <p><b>C1. Look in proof output for goals that can't be further \u000A simplified.</b><br></br> \u000A \u000A Use the ``<see topic=\"ACL2____PROOF-TREE\">proof-tree</see>'' utility to explore the proof space.  However, \u000A you don't need to use that tool to use the ``checkpoint'' strategy.  The idea \u000A is to think of ACL2 as a ``simplifier'' that either proves the theorem or \u000A generates some goal to consider.  That goal is the first ``checkpoint,'' i.e., \u000A the first goal that does not further simplify.  Exception: it's also important \u000A to look at the induction scheme in a proof by induction, and if induction \u000A seems appropriate, then look at the first checkpoint <i>after</i> the \u000A induction has begun.</p> \u000A \u000A <p>Consider whether the goal on which you focus is even a theorem.  Sometimes \u000A you can execute it for particular values to find a counterexample.</p> \u000A \u000A <p>When looking at checkpoints, remember that you are looking for any reason \u000A at all to believe the goal is a theorem.  So for example, sometimes there may \u000A be a contradiction in the hypotheses.</p> \u000A \u000A <p>Don't be afraid to skip the first checkpoint if it doesn't seem very \u000A helpful.  Also, be willing to look a few lines up or down from the checkpoint \u000A if you are stuck, bearing in mind however that this practice can be more \u000A distracting than helpful.</p> \u000A \u000A <p><b>C2. Use the ``break rewrite'' facility.</b><br></br> \u000A \u000A <tt><see topic=\"ACL2____BRR\">Brr</see></tt> and related utilities let you inspect the ``rewrite stack.'' \u000A These can be valuable tools in large proof efforts.  See <see topic=\"ACL2____BREAK-LEMMA\">break-lemma</see> \u000A for an introduction to these tools, and see <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> for more \u000A complete information.</p> \u000A \u000A <p>The break facility is especially helpful in showing you why a particular \u000A rewrite rule is not being applied.</p> \u000A \u000A <p><b>C3. Use induction hints when necessary.</b> Of course, if you can define \u000A your functions so that they suggest the correct inductions to ACL2, so much \u000A the better!  But for complicated inductions, induction <see topic=\"ACL2____HINTS\">hints</see> are \u000A crucial.  See <see topic=\"ACL2____HINTS\">hints</see> for a description of <v>:induct</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p><b>C4. Use the ``Proof Checker'' to explore.</b><br></br> \u000A \u000A The <tt><see topic=\"ACL2____VERIFY\">verify</see></tt> command supplied by ACL2 allows one to explore problem \u000A areas ``by hand.''  However, even if you succeed in proving a conjecture with \u000A <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, it is useful to prove it without using it, an activity that \u000A will often require the discovery of <see topic=\"ACL2____REWRITE\">rewrite</see> rules that will be useful \u000A in later proofs as well.</p> \u000A \u000A <p><b>C5. Don't have too much patience.</b><br></br> \u000A \u000A Interrupt the prover fairly quickly when simplification isn't succeeding.</p> \u000A \u000A <p><b>C6. Simplify rewrite rules.</b><br></br> \u000A \u000A When it looks difficult to relieve the hypotheses of an existing <see topic=\"ACL2____REWRITE\">rewrite</see> rule that ``should'' apply in a given setting, ask yourself if you \u000A can eliminate a hypothesis from the existing <see topic=\"ACL2____REWRITE\">rewrite</see> rule.  If so, it \u000A may be easier to prove the new version from the old version (and some \u000A additional lemmas), rather than to start from scratch.</p> \u000A \u000A <p><b>C7. Deal with base cases first.</b><br></br> \u000A \u000A Try getting past the base case(s) first in a difficult proof by induction. \u000A Usually they're easier than the inductive step(s), and rules developed in \u000A proving them can be useful in the inductive step(s) too.  Moreover, it's \u000A pretty common that mistakes in the statement of a theorem show up in the base \u000A case(s) of its proof by induction.</p> \u000A \u000A <p><b>C8. Use</b> <v>:expand</v> <b>hints.</b> Consider giving <v>:expand</v> \u000A <see topic=\"ACL2____HINTS\">hints</see>.  These are especially useful when a proof by induction is \u000A failing.  It's almost always helpful to open up a recursively defined function \u000A that is supplying the induction scheme, but sometimes ACL2 is too timid to do \u000A so; or perhaps the function in question is <see topic=\"ACL2____DISABLE\">disable</see>d.</p> \u000A \u000A <p><i>D. PERFORMANCE TIPS</i></p> \u000A \u000A <p><b>D1. Disable rules.</b><br></br> \u000A \u000A There are a number of instances when it is crucial to <see topic=\"ACL2____DISABLE\">disable</see> rules, \u000A including (often) those named explicitly in <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see>.  Also, \u000A <see topic=\"ACL2____DISABLE\">disable</see> recursively defined functions for which you can prove what seem \u000A to be all the relevant properties.  The prover can spend significant time \u000A ``behind the scenes'' trying to open up recursively defined functions, where \u000A the only visible effect is slowness.</p> \u000A \u000A <p><b>D2. Turn off the ``break rewrite'' facility.</b> Remember to execute \u000A <v>:brr nil</v> after you've finished with the ``break rewrite'' utility (see \u000A <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>), in order to bring the prover back up to full speed.</p> \u000A \u000A <p><i>E. MISCELLANEOUS TIPS AND KNOWLEDGE</i></p> \u000A \u000A <p><b>E1. Order of application of rewrite rules.</b><br></br> \u000A \u000A Keep in mind that the most recent <see topic=\"ACL2____REWRITE\">rewrite</see> rules in the <see topic=\"ACL2____HISTORY\">history</see> \u000A are tried first.</p> \u000A \u000A <p><b>E2. Relieving hypotheses is not full-blown theorem proving.</b><br></br> \u000A \u000A Relieving hypotheses on <see topic=\"ACL2____REWRITE\">rewrite</see> rules is done by rewriting and <see topic=\"ACL2____LINEAR\">linear</see> arithmetic alone, not by case splitting or by other prover processes \u000A ``below'' simplification.</p> \u000A \u000A <p><b>E3. ``Free variables'' in rewrite rules.</b><br></br> The set of ``free \u000A  variables'' of a <see topic=\"ACL2____REWRITE\">rewrite</see> rule is defined to contain those variables \u000A  occurring in the rule that do not occur in the left-hand side of the rule. \u000A  It's often a good idea to avoid rules containing free variables because they \u000A  are ``weak,'' in the sense that hypotheses containing such variables can \u000A  generally only be proved when they are ``obviously'' present in the current \u000A  context.  This weakness suggests that it's important to put the most \u000A  ``interesting'' (specific) hypotheses about free variables first, so that the \u000A  right instances are considered.  For example, suppose you put a very general \u000A  hypothesis such as <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> first.  If the context has several <see topic=\"ACL2____TERM\">term</see>s around that are known to be <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>s, then <v>x</v> may be bound \u000A  to the wrong one of them.  For much more information on free variables, see \u000A  <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>.</p> \u000A \u000A <p><b>E4. Obtaining information</b><br></br> \u000A \u000A Use <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> <v>foo</v> to inspect <see topic=\"ACL2____REWRITE\">rewrite</see> rules whose left hand \u000A sides are applications of the function <v>foo</v>.  Another approach to seeing \u000A which <see topic=\"ACL2____REWRITE\">rewrite</see> rules apply is to enter the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> with \u000A <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, and use the <v>show-rewrites</v> or <v>sr</v> command.</p> \u000A \u000A <p><b>E5. Consider esoteric rules with care.</b><br></br> \u000A \u000A If you care to see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> and peruse the list of subtopics (which \u000A will be listed right there in most versions of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>), \u000A you'll see that ACL2 supports a wide variety of rules in addition to \u000A <v>:</v><see topic=\"ACL2____REWRITE\">rewrite</see> rules.  Should you use them?  This is a complex question \u000A that we are not ready to answer with any generality.  Our general advice is to \u000A avoid relying on such rules as long as you doubt their utility.  More \u000A specifically: be careful not to use conditional type prescription rules, as \u000A these have been known to bring ACL2 to its knees, unless you are conscious \u000A that you are doing so and have reason to believe that they are working \u000A well.</p> \u000A \u000A <p><i>F. SOME THINGS YOU DON'T NEED TO KNOW</i></p> \u000A \u000A <p>Most generally: you shouldn't usually need to be able to predict too much \u000A about ACL2's behavior.  You should mainly just need to be able to react to \u000A it.</p> \u000A \u000A <p><b>F1. Induction heuristics.</b><br></br> \u000A \u000A Although it is often important to read the part of the prover's \u000A output that gives the induction scheme chosen by the prover, it is \u000A not necessary to understand how the prover made that choice. \u000A (Granted, advanced users may occasionally gain minor insight from such \u000A knowledge.  But it's truly minor in many cases.)  What <i>is</i> important is \u000A to be able to tell it an appropriate induction when it doesn't pick the right \u000A one (after noticing that it doesn't).  See C3 above.</p> \u000A \u000A <p><b>F2. Heuristics for expanding calls of recursively defined \u000A functions.</b><br></br> \u000A \u000A As with the previous topic, the important thing isn't to understand these \u000A heuristics but, rather, to deal with cases where they don't seem to be \u000A working.  That amounts to supplying <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see> for those calls \u000A that you want opened up, which aren't.  See also C8 above.</p> \u000A \u000A <p><b>F3. The ``waterfall''.</b><br></br> \u000A \u000A As discussed many times already, a good strategy for using ACL2 is to look for \u000A checkpoints (goals stable under simplification) when a proof fails, perhaps \u000A using the <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> facility.  Thus, it is reasonable to ignore almost \u000A all the prover output, and to avoid pondering the meaning of the other \u000A ``processes'' that ACL2 uses besides simplification (such as elimination, \u000A cross-fertilization, generalization, and elimination of irrelevance).  For \u000A example, you don't need to worry about prover output that mentions ``type \u000A reasoning'' or ``abbreviations,'' for example.</p>"],

 "ACL2____PROOF-TREE-EXAMPLES":[["Proof-tree"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> for an introduction to proof trees, and for a \u000A list of related topics.  Here we present a detailed example followed by a \u000A shorter example that illustrates proof by induction.</p> \u000A \u000A <p>Consider the <see topic=\"ACL2____GUARD\">guard</see> proof for the definition of a function \u000A <v>cancel_equal_plus</v>; the body of this definition is of no importance here. \u000A The first proof tree display is:</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A     |  &lt;18 subgoals&gt;</code> \u000A \u000A <p>This is to be read as follows.</p> \u000A \u000A <blockquote><p>At this stage of the proof we have encountered the top-level \u000A  goal, named \"Goal\", which generated 18 subgoals using the ``preprocess'' \u000A  process.  We have not yet begun to work on those subgoals.</p></blockquote> \u000A \u000A <p>The corresponding message from the ordinary prover output is:</p> \u000A \u000A <blockquote><p>By case analysis we reduce the conjecture to the following 18 \u000A  conjectures.</p></blockquote> \u000A \u000A <p>Note that the field just before the name of the goal (<v>&quot;Goal&quot;</v>), which \u000A here contains the number 18, indicates the number of cases (children) created \u000A by the goal using the indicated process.  This number will remain unchanged as \u000A long as this goal is displayed.</p> \u000A \u000A <p>The next proof tree display is:</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   1 |  Subgoal 18 simp\u000A     |  |  &lt;1 subgoal&gt;\u000A     |  &lt;17 more subgoals&gt;</code> \u000A \u000A <p>which indicates that at this point, the prover has used the simplification \u000A (``simp'') process on Subgoal 18 to create one subgoal (``&lt;1 \u000A subgoal&gt;'').  The vertical bar (``|'') below ``Subgoal 18'', accompanied by \u000A the line below it, signifies that there are 17 siblings of Subgoal 18 that \u000A remain to be processed.</p> \u000A \u000A <p>The next proof tree displayed is:</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   1 |  Subgoal 18 simp\u000Ac  2 |  |  Subgoal 18' ELIM\u000A     |  |  |  &lt;2 subgoals&gt;\u000A     |  &lt;17 more subgoals&gt;</code> \u000A \u000A <p>Let us focus on the fourth line of this display:</p> \u000A \u000A <code>c  2 |  |  Subgoal 18' ELIM</code> \u000A \u000A <p>The ``c'' field marks this goal as a ``checkpoint'', i.e., a goal worthy of \u000A careful scrutiny.  In fact, any goal that creates children by a process other \u000A than ``preprocess'' or ``simp'' is marked as a checkpoint.  In this case, the \u000A destructor-elimination (``<see topic=\"ACL2____ELIM\">elim</see>'') process has been used to create \u000A subgoals of this goal.  The indentation shows that this goal, Subgoal 18', is \u000A a child of Subgoal 18.  The number ``2'' indicates that 2 subgoals have been \u000A created (by <see topic=\"ACL2____ELIM\">elim</see>).  Note that this information is consistent with the \u000A line just below it, which says ``&lt;2 subgoals&gt;''.</p> \u000A \u000A <p>Finally, the last line of this proof tree display,</p> \u000A \u000A <code>|  &lt;17 more subgoals&gt;</code> \u000A \u000A <p>is connected by vertical bars (``|'') up to the string <v>&quot;Subgoal 18&quot;</v>, \u000A which suggests that there are 17 immediate subgoals of Goal remaining to \u000A process after Subgoal 18.  Note that this line is indented one level from the \u000A second line, which is the line for the goal named <v>&quot;Goal&quot;</v>.  The display \u000A is intended to suggest that the subgoals of Goal that remain to be proved \u000A consist of Subgoal 18 together with 17 more subgoals.</p> \u000A \u000A <p>The next proof tree display differs from the previous one only in that now, \u000A Subgoal 18' has only one more subgoal to be processed.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   1 |  Subgoal 18 simp\u000Ac  2 |  |  Subgoal 18' ELIM\u000A     |  |  |  &lt;1 more subgoal&gt;\u000A     |  &lt;17 more subgoals&gt;</code> \u000A \u000A <p>Note that the word ``more'' in ``&lt;1 more subgoal&gt;'' tells us that \u000A there was originally more than one subgoal of Subgoal 18.  In fact that \u000A information already follows from the line above, which (as previously \u000A explained) says that Subgoal 18' originally created 2 subgoals.</p> \u000A \u000A <p>The next proof tree display occurs when the prover completes the proof of \u000A that ``1 more subgoal'' referred to above.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A     |  &lt;17 more subgoals&gt;</code> \u000A \u000A <p>Then, Subgoal 17 is processed and creates one subgoal, by \u000A simplification:</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   1 |  Subgoal 17 simp\u000A     |  |  &lt;1 subgoal&gt;\u000A     |  &lt;16 more subgoals&gt;</code> \u000A \u000A <p>... and so on.</p> \u000A \u000A <p>Later in the proof one might find the following successive proof tree \u000A displays.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A     |  &lt;9 more subgoals&gt;\u000A\u000A( DEFUN CANCEL_EQUAL_PLUS ...)\u000A\u000A  18 Goal preprocess\u000A   0 |  Subgoal 9 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A     |  &lt;8 more subgoals&gt;</code> \u000A \u000A <p>These displays tell us that Subgoal 9 simplified to <v>t</v> (note that the \u000A ``0'' shows clearly that no subgoals were created), but that some rule's \u000A hypotheses were <see topic=\"ACL2____FORCE\">force</see>d.  Although this goal is not checkpointed (i.e., \u000A no ``c'' appears on the left margin), one can cause such goals to be \u000A checkpointed; see <see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see>.</p> \u000A \u000A <p>In fact, the proof tree displayed at the end of the ``main proof''(the 0-th \u000A forcing round) is as follows.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   0 |  Subgoal 9 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A   0 |  Subgoal 8 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A   0 |  Subgoal 7 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A   0 |  Subgoal 6 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A   0 |  Subgoal 4 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)\u000A   0 |  Subgoal 3 simp (<see topic=\"ACL2____FORCED\">FORCED</see>)</code> \u000A \u000A <p>This is followed by the following proof tree display at the start of the \u000A forcing round.</p> \u000A \u000A <code>  18 Goal preprocess\u000A   0 |  Subgoal 9 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 4)\u000A   0 |  Subgoal 8 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 6)\u000A   0 |  Subgoal 7 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 1)\u000A   0 |  Subgoal 6 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 3)\u000A   0 |  Subgoal 4 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 5)\u000A   0 |  Subgoal 3 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 2)\u000A++++++++++++++++++++++++++++++\u000A   6 [1]Goal FORCING-ROUND\u000A   2 |  [1]Subgoal 6 preprocess\u000A     |  |  &lt;2 subgoals&gt;\u000A     |  &lt;5 more subgoals&gt;</code> \u000A \u000A <p>This display shows which goals to ``blame'' for the existence of each goal \u000A in the forcing round.  For example, Subgoal 9 is to blame for the creation of \u000A [1]Subgoal 4.</p> \u000A \u000A <p>Actually, there is no real goal named <v>&quot;[1]Goal&quot;</v>.  However, the \u000A line</p> \u000A \u000A <code>6 [1]Goal FORCING-ROUND</code> \u000A \u000A <p>appears in the proof tree display to suggest a ``parent'' of the six \u000A top-level goals in that forcing round.  As usual, the numeric field before the \u000A goal name contains the original number of children of that (virtual, in this \u000A case) goal &mdash; in this case, 6.</p> \u000A \u000A <p>In our example proof, Subgoal 6 eventually gets proved, without doing any \u000A further forcing.  At that point, the proof tree display looks as follows.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   0 |  Subgoal 9 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 4)\u000A   0 |  Subgoal 7 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 1)\u000A   0 |  Subgoal 6 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 3)\u000A   0 |  Subgoal 4 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 5)\u000A   0 |  Subgoal 3 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 2)\u000A++++++++++++++++++++++++++++++\u000A   6 [1]Goal FORCING-ROUND\u000A     |  &lt;5 more subgoals&gt;</code> \u000A \u000A <p>Notice that the line for Subgoal 8,</p> \u000A \u000A <code>0 |  Subgoal 8 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 6)</code> \u000A \u000A <p>no longer appears.  That is because the goal [1]Subgoal 6 has been proved, \u000A along with all its children; and hence, the proof of Subgoal 8 no longer \u000A depends on any further reasoning.</p> \u000A \u000A <p>The final two proof tree displays in our example are as follows.</p> \u000A \u000A <code>( DEFUN CANCEL_EQUAL_PLUS ...)\u000A  18 Goal preprocess\u000A   0 |  Subgoal 7 simp (<see topic=\"ACL2____FORCED\">FORCED</see> [1]Subgoal 1)\u000A++++++++++++++++++++++++++++++\u000A   6 [1]Goal FORCING-ROUND\u000A   2 |  [1]Subgoal 1 preprocess\u000A   1 |  |  [1]Subgoal 1.1 preprocess\u000A   1 |  |  |  [1]Subgoal 1.1' simp\u000Ac  3 |  |  |  |  [1]Subgoal 1.1'' ELIM\u000A     |  |  |  |  |  &lt;1 more subgoal&gt;\u000A\u000A( DEFUN CANCEL_EQUAL_PLUS ...)\u000A&lt;&lt;PROOF TREE IS EMPTY&gt;&gt;</code> \u000A \u000A <p>The explanation for the empty proof tree is simple: once [1]Subgoal 1.1.1 \u000A was proved, nothing further remained to be proved.  In fact, the much \u000A sought-after ``Q.E.D.'' appeared shortly after the final proof tree was \u000A displayed.</p> \u000A \u000A <p>Let us conclude with a final, brief example that illustrates proof by \u000A induction.  Partway through the proof one might come across the following \u000A proof tree display.</p> \u000A \u000A <code>( DEFTHM PLUS-TREE-DEL ...)\u000A   1 Goal preprocess\u000A   2 |  Goal' simp\u000Ac  0 |  |  Subgoal 2 PUSH *1\u000A     |  |  &lt;1 more subgoal&gt;</code> \u000A \u000A <p>This display says that in the attempt to prove a theorem called \u000A <v>plus-tree-del</v>, preprocessing created the only child Goal' from Goal, and \u000A Goal' simplified to two subgoals.  Subgoal 2 is immediately pushed for proof \u000A by induction, under the name ``*1''.  In fact if Subgoal 1 simplifies to \u000A <v>t</v>, then we see the following successive proof tree displays after the one \u000A shown above.</p> \u000A \u000A <code>( DEFTHM PLUS-TREE-DEL ...)\u000A   1 Goal preprocess\u000A   2 |  Goal' simp\u000Ac  0 |  |  Subgoal 2 PUSH *1\u000A\u000A( DEFTHM PLUS-TREE-DEL ...)\u000A   1 Goal preprocess\u000A   2 |  Goal' simp\u000Ac  0 |  |  Subgoal 2 PUSH *1\u000A++++++++++++++++++++++++++++++\u000Ac  6 *1 INDUCT\u000A     |  &lt;5 more subgoals&gt;</code> \u000A \u000A <p>The separator ``+++++...'' says that we are beginning another trip through \u000A the waterfall.  In fact this trip is for a proof by induction (as opposed to a \u000A forcing round), as indicated by the word ``INDUCT''.  Apparently *1.6 was \u000A proved immediately, because it was not even displayed; a goal is only \u000A displayed when there is some work left to do either on it or on some goal that \u000A it brought (perhaps indirectly) into existence.</p> \u000A \u000A <p>Once a proof by induction is completed, the ``PUSH'' line that refers to \u000A that proof is eliminated (``pruned'').  So for example, when the present proof \u000A by induction is completed, the line</p> \u000A \u000A <code>c  0 |  |  Subgoal 2 PUSH *1</code> \u000A \u000A <p>is eliminated, which in fact causes the lines above it to be eliminated \u000A (since they no longer refer to unproved children).  Hence, at that point one \u000A might expect to see:</p> \u000A \u000A <code>( DEFTHM PLUS-TREE-DEL ...)\u000A&lt;&lt;PROOF TREE IS EMPTY&gt;&gt;</code> \u000A \u000A <p>However, if the proof by induction of *1 necessitates further proofs by \u000A induction or a forcing round, then this ``pruning'' will not yet be \u000A done.</p>"],

 "ACL2____PROGRAMMING-KNOWLEDGE-TAKEN-FOR-GRANTED":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This brief review of the programming language is presented as a \u000A sequence of questions and answers meant to test your knowledge of the ACL2 \u000A programming language.  If you want a gentle introduction to the programming \u000A language, see <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/gentle-intro-to-acl2-programming.html'>http://www.cs.utexas.edu/users/moore/publications/gentle-intro-to-acl2-programming.html</a>.</p> \u000A \u000A <p>Before we get started with the programming drill, let us remind you that \u000A all we're interested in here is the language, not the ``program development \u000A environment.''  It's impossible to program in ACL2 or any other language \u000A without a decent environment, one that at the very least includes a way to \u000A prepare and edit files of programs.  The two most popular program development \u000A environments among ACL2 users are <see topic=\"ACL2____EMACS\">Emacs</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A the Eclipse-based <see topic=\"ACL2____ACL2-SEDAN\">ACL2-Sedan</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A The Sedan provides the most support for the new user, including real-time \u000A syntax checking and a facility for testing among many other features.  But in \u000A this drill we're not interested in the program development environment, we're \u000A interested in your understanding of the ACL2 language.</p> \u000A \u000A <p><b>Q</b>: What do you think this command does?</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))</code> \u000A \u000A <p><b>A</b>: It defines a function named <v>rev</v> that takes one argument, \u000A treats it like a list, and reverses the order of the elements in that list. \u000A To figure this out from the definition, you have to know that <v>append</v> \u000A concatenates two lists.  Logically speaking, the <v>defun</v> of <v>rev</v> adds \u000A the axiom:</p> \u000A \u000A <code>(rev x)\u000A=\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A    nil\u000A    (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))),</code> \u000A \u000A <p>implicitly quantified for all <v>x</v>.</p> \u000A \u000A <p><b>Q</b>: Given the <v>defun</v> of <v>rev</v> above, what are the <i>formal \u000A parameters</i>?  What is the <i>body</i> of the definition?  Write down a \u000A <i>call</i> of <i>append</i> that occurs in the body of <v>rev</v>.  What are \u000A the <i>actuals</i> of that call?  <b>A</b>: The <i>formals</i> of <v>rev</v> is \u000A the list of variables after the first <v>rev</v> in the <v>defun</v>, namely \u000A <v>(x)</v>.  We say <v>x</v> is the first (and only) <i>formal</i> here.  The \u000A <i>body</i> of <v>rev</v> is the entire <v>if</v>-expression.  The only \u000A <i>call</i> of <v>append</v> in the body is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</code> \u000A \u000A <p>and the <i>actuals</i> of that call are, respectively, <v>(rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v> \u000A and <v>(<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))</v>.</p> \u000A \u000A <p><b>Q</b>: What do you get if you evaluate <v>(rev '(a b c d))</v>?  <b>A</b>: \u000A <v>(D C B A)</v>.</p> \u000A \u000A <p><b>Q</b>: How did <v>rev</v> change the case of the elements, e.g., lowercase \u000A <v>a</v> was in the input list but uppercase <v>A</v> was in the output? \u000A <b>A</b>: This is a trick question.  <v>Rev</v> doesn't change the case of the \u000A elements.  ACL2 is case-insensitive when dealing with symbols.  The symbol \u000A <v>a</v> is read in as the symbol <v>A</v>.  Thus, when writing function names, \u000A for example, we can write <v>rev</v>, <v>Rev</v>, <v>REV</v>, or even <v>ReV</v> and \u000A always be referring to the function <v>REV</v>.  By default, ACL2 prints symbols \u000A in uppercase.</p> \u000A \u000A <p><b>Q</b>: What does <v>(rev '((a b c) &quot;Abc&quot; &quot;a&quot; b #\\c))</v> return? \u000A <b>A</b>: <v>(#\\c B &quot;a&quot; &quot;Abc&quot; (A B C))</v>.  If you thought the answer was \u000A any of these, then you need to think or read more carefully:</p> \u000A \u000A <code>(#\\C B &quot;A&quot; &quot;ABC&quot; (A B C))\u000A\u000A(#\\C B &quot;A&quot; &quot;ABC&quot; (C B A))</code> \u000A \u000A <p>The first wrong answer above is wrong because Lisp is ``case insensitive'' \u000A only for symbols, not for character objects like <v>#\\c</v> (the lowercase \u000A character c) or for strings.  Furthermore, <v>&quot;A&quot;</v> is a string, not a \u000A symbol; it is different from <v>A</v>.  The second wrong answer above is wrong \u000A because <v>rev</v> does not go into the individual elements of the list, it just \u000A reverses the order of the elements.  So it doesn't change the element <v>(A B\u000A C)</v> to <v>(C B A)</v>.</p> \u000A \u000A <p><b>Q</b>: In the question about what <v>(rev '(a b c d))</v> returns, we put \u000A a quote mark before the <v>(a b c d)</v> but not before the answer, <v>(D C B\u000A A)</v>.  Why?  <b>A</b>: The phrase ``<i>x</i> evaluates to <i>y</i>'' treats \u000A <i>x</i> as a <i>term</i> to be evaluated and <i>y</i> as an <i>object</i>. \u000A <v>(Rev '(a b c d))</v> is a term to be evaluated and denotes a call of the \u000A function <v>rev</v> on the value of the argument term <v>'(a b c d)</v>.  The \u000A value of that argument term is the object <v>(a b c d)</v>.  The value of the \u000A call of <v>rev</v> is the object <v>(d c b a)</v>.  If you have an object, \u000A <i>obj</i>, and you wish to create a term whose value is <i>obj</i>, then you \u000A put a quote mark in front of it, <i>'obj</i>.</p> \u000A \u000A <p><b>Q</b>: Can <v>rev</v> be applied to something other than a list? \u000A <b>A</b>: Yes, every ACL2 function can be applied to any object.  ACL2 is an \u000A untyped programming language: every variable ranges over the entire universe \u000A of objects.  In normal usage, <v>rev</v> is applied to lists but there is \u000A nothing about the syntax of the language that prevents it being applied to \u000A non-lists.</p> \u000A \u000A <p><b>Q</b>: So what does <v>(rev 23)</v> evaluate to?  <b>A</b>: <v>Nil</v>.</p> \u000A \u000A <p><b>Q</b>: Why?  <b>A</b>: Because <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> 23)</v> is <v>t</v>, because \u000A <v>endp</v> is defined:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> endp (x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))</code> \u000A \u000A <p>Thus, if <v>rev</v> is applied to anything that is not a cons, it returns \u000A <v>nil</v>.</p> \u000A \u000A <p><b>Q</b>: So what does <v>(rev '(a b c . d))</v> evaluate to?  <b>A</b>: \u000A <v>(c b a)</v>.  To explain why requires demonstrating that you know what <v>(a\u000A b c . d)</v> means.  It is the object computed by evaluating:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CONS\">cons</see> 'a\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> 'b\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> 'c\u000A                  'd))).</code> \u000A \u000A <p>That is, it is a list whose ``terminal marker'' is the atom <v>D</v>. \u000A <v>Rev</v> treats that list exactly as it treats the <v>nil</v>-terminated list of \u000A the same elements, <v>(a b c)</v>, because <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> 'D)</v> = <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> nil)</v> = \u000A <v>t</v>.</p> \u000A \u000A <p><b>Q</b>: What does <v>(rev 1 2 3)</v> evaluate to?  <b>A</b>: That's a trick \u000A question.  <v>Rev</v> takes one argument, not three.  So <v>(rev 1 2 3)</v> is an \u000A ill-formed term.</p> \u000A \u000A <p><b>Q</b>: What does <v>(rev '(a b c . d . nil))</v> evaluate to?  <b>A</b>: \u000A That is a trick question.  There is no such object.  In Lisp's ``dot \u000A notation'' every dot must be followed by a well-formed object and then a close \u000A parenthesis.  Usually that ``well-formed object'' is an atom.  If it is not an \u000A atom, i.e., if it is a cons, then the entire expression could have been \u000A written without that dot.  For example, <v>(a b c . (d e))</v> is an object, but \u000A it could be written <v>(a b c d e)</v>.</p> \u000A \u000A <p><b>Q</b>: Do <v>(rev (rev x))</v> and <v>x</v> always evaluate to the same \u000A object?  <b>A</b>: No.  <v>(Rev (rev 23))</v> evaluates to <v>nil</v>, not \u000A <v>23</v>.</p> \u000A \u000A <p><b>Q</b>: Do <v>(rev (rev x))</v> and <v>x</v> always evaluate to the same \u000A object when <v>x</v> is a cons?  <b>A</b>: No.  <v>(rev (rev '(a b c . d)))</v> \u000A evaluates to <v>(a b c)</v>, not <v>(a b c . d)</v>.</p> \u000A \u000A <p><b>Q</b>: When are <v>(rev (rev x))</v> and <v>x</v> equal?  <b>A</b>: When the \u000A terminal marker of <v>x</v> is <v>nil</v>.</p> \u000A \u000A <p><b>Q</b>: Can you define a Lisp function that recognizes \u000A <v>nil</v>-terminated lists?  <b>A</b>: Yes, but it is not necessary for the \u000A user to define that concept because Common Lisp provides such a function which \u000A is logically defined as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> true-listp (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))).</code> \u000A \u000A <p>This can be paraphrased: <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)</v> means that if <v>x</v> is a \u000A <v>cons</v>, its <v>cdr</v> is a <v>true-listp</v> and if <v>x</v> is not a <v>cons</v>, \u000A it must be <v>nil</v>.  Thus, <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> '(a b c))</v> is <v>t</v> and \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> '(a b c . d))</v> is <v>nil</v>.</p> \u000A \u000A <p><b>Q</b>: Can you write a Lisp formula that says ``If <v>z</v> is a \u000A <v>nil</v>-terminated list then reversing the result of reversing <v>z</v> is \u000A <v>z</v>''?</p> \u000A \u000A <p><b>A</b>: Yes:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z)).</code> \u000A \u000A <p><b>Q</b>: Is this all there is to ACL2 programming?  <b>A</b>: No!  ACL2 \u000A provides many other features.  For a full list of all the primitive functions \u000A in ACL2 see <see topic=\"ACL2____PROGRAMMING\">programming</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A Some highlights for the beginner are mentioned below, but all of the links \u000A below ought to be tagged with the <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A sign.</p> \u000A \u000A <p>* <tt><see topic=\"COMMON-LISP____LIST\">list</see></tt>: build a <v>nil</v>-terminated list from the values of \u000A <i>n</i> terms, e.g., <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x) (<see topic=\"COMMON-LISP_____B2\">+</see> 2 x))</v> returns <v>(3 4 5)</v> if \u000A <v>x</v> is <v>3</v>.</p> \u000A \u000A <p>* <see topic=\"COMMON-LISP____LIST_A2\">list*</see>: build a non-<v>nil</v> terminated list of <i>n</i> objects \u000A from the values of <i>n+1</i> terms, e.g., <v>(<see topic=\"COMMON-LISP____LIST_A2\">list*</see> x (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x) (<see topic=\"COMMON-LISP_____B2\">+</see> 2 x) (<see topic=\"COMMON-LISP_____A2\">*</see> -1\u000A x))</v> returns the list <v>(3 4 5 . -3)</v> if <v>x</v> is <v>3</v>.</p> \u000A \u000A <p>* <tt><see topic=\"COMMON-LISP____AND\">and</see></tt>, <tt><see topic=\"COMMON-LISP____OR\">or</see></tt>, <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt>, <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt>, <tt><see topic=\"ACL2____IFF\">iff</see></tt>: The \u000A propositional connectives.  <v>And</v> and <v>or</v> are allowed to take a varying \u000A number of arguments, e.g., <v>(<see topic=\"COMMON-LISP____AND\">and</see> p q r)</v> is just an abbreviation for \u000A <v>(<see topic=\"COMMON-LISP____AND\">and</see> p (<see topic=\"COMMON-LISP____AND\">and</see> q r))</v>.  In Lisp, <v>and</v> returns <v>nil</v> if any of its \u000A arguments evaluates to <v>nil</v>; otherwise it returns the value of the last \u000A argument!  Thus, <v>(<see topic=\"COMMON-LISP____AND\">and</see> t t 3)</v> returns <v>3</v>!  If you object to the idea \u000A that <v>and</v> is not Boolean, don't give it non-Boolean arguments!  Similarly, \u000A <v>or</v> returns the value of the first argument that evaluates to \u000A non-<v>nil</v>, or <v>nil</v> if they all evaluate to <v>nil</v>.  Both <v>and</v> and \u000A <v>or</v> can be thought of as ``lazy'' in that they don't always have to \u000A evaluate all their arguments.  This is really accomplished by treating \u000A <v>and</v> and <v>or</v> as abbrevations for <v>if</v> nests.</p> \u000A \u000A <p>* <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt>, <tt><see topic=\"COMMON-LISP_____A2\">*</see></tt>, <tt><see topic=\"COMMON-LISP____-\">-</see></tt>, <tt><see topic=\"COMMON-LISP_____F2\">/</see></tt>, <tt><see topic=\"COMMON-LISP____FLOOR\">floor</see></tt>, <tt><see topic=\"COMMON-LISP____MOD\">mod</see></tt>, \u000A <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>, <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>, <tt><see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see></tt>, <tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt>: the Lisp elementary arithmetic \u000A operators.  Both <v>+</v> and <v>*</v> allow varying numbers of arguments.  All \u000A the arithmetic operators default non-numeric arguments to <v>0</v>.  If you \u000A don't like the idea that <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 2 t)</v> is <v>3</v>, don't ask <v>+</v> to add \u000A <v>t</v> to something!</p> \u000A \u000A <p>* <tt><see topic=\"ACL2____NATP\">natp</see></tt>, <tt><see topic=\"COMMON-LISP____INTEGERP\">integerp</see></tt>, <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt>, <tt><see topic=\"COMMON-LISP____CHARACTERP\">characterp</see></tt>, \u000A <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt>, <tt><see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see></tt>, <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>: the recognizers for the \u000A primitive data types.  The first three recognize subsets of the ACL2 numeric \u000A universe.  The naturals are a subset of the integers, the integers are a \u000A subset of the rationals, and the rationals are a subset of the objects \u000A recognized by <tt><see topic=\"ACL2____ACL2-NUMBERP\">ACL2-numberp</see></tt>, which also includes the <tt><see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see></tt>s.  The other recognizers listed above recognize characters, \u000A strings, symbols, and conses.</p> \u000A \u000A <p>* <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt>: a convenient way to write a cascading nest of <v>if</v>s, \u000A e.g.,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____NATP\">natp</see> x)) 'non-natural)\u000A      ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0) 'zero)\u000A      ((<see topic=\"COMMON-LISP____EVENP\">evenp</see> x) 'positive-even)\u000A      (t 'positive-odd))</code> \u000A \u000A <p>abbreviates</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____NATP\">natp</see> x))\u000A    'non-natural\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)\u000A        'zero\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EVENP\">evenp</see> x)\u000A            'positive-even\u000A            'positive-odd))).</code> \u000A \u000A <p>* <tt><see topic=\"COMMON-LISP____CASE\">case</see></tt>: a convenient way to case split on the identity of an \u000A object.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CASE\">case</see> key\u000A  (non-natural -1)\u000A  (zero 0)\u000A  ((positive-even positive-odd) 'positive-natural)\u000A  (otherwise 'unknown))</code> \u000A \u000A <p>abbreviates</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQL\">eql</see> key 'non-natural) -1)\u000A      ((<see topic=\"COMMON-LISP____EQL\">eql</see> key 'zero) 0)\u000A      ((<see topic=\"COMMON-LISP____MEMBER\">member</see> key '(positive-even positive-odd))\u000A       'positive-natural)\u000A      (t 'unknown)).</code> \u000A \u000A <p>* user defined macros: using <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A you can introduce your own abbreviations.  We recommend you not do this until \u000A you're good at list processing since macros are functions that build objects \u000A representing terms.</p> \u000A \u000A <p>* <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>: allows you to define mutually-recursive \u000A functions.</p> \u000A \u000A <p>* <tt><see topic=\"ACL2____MV\">mv</see></tt> and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>: allow functions to return \u000A ``multiple-values''.  In Lisp, such functions return vectors of values, the \u000A vectors are represented as lists of values, but the implementations are \u000A generally more efficient.  For example, <v>(<see topic=\"ACL2____MV\">mv</see> x y z)</v> returns a ``vector'' \u000A consisting of the values of <v>x</v>, <v>y</v>, and <v>z</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (a b c)\u000A        (foo x)\u000A        (bar a b c x))</code> \u000A \u000A <p>evaluates <v>(foo x)</v>, treats the result as a vector of three values, \u000A binds the variables <v>a</v>, <v>b</v>, and <v>c</v> to those three values, and \u000A evaluates and returns <v>(bar a b c x)</v>.</p> \u000A \u000A <p>ACL2 also provides many other features, such as single-threaded objects \u000A which may be ``destructively modified'' (see <see topic=\"ACL2____STOBJ\">stobj</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A including a very special single-threaded object that records the <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> of the ACL2 system), file input and output (see \u000A <see topic=\"ACL2____IO\">io</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>), applicative arrays (see <see topic=\"ACL2____ARRAYS\">arrays</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A and property lists (see <see topic=\"ACL2____GETPROP\">getprop</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A and other facilities necessary for it to be a practical programming language. \u000A However, we <b>strongly</b> recommend that as a new user you stay away from \u000A these features until you are good at proving theorems about elementary list \u000A processing!</p> \u000A \u000A <p>If this little drill made sense to you, you know enough of the programming \u000A language to get started. Use your browser's <b>Back Button</b> now to return \u000A to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p> \u000A \u000A <p>If you are uncomfortable with ACL2 programming, we recommend that you study \u000A <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/gentle-intro-to-acl2-programming.html'>http://www.cs.utexas.edu/users/moore/publications/gentle-intro-to-acl2-programming.html</a> \u000A and <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-programming-exercises1.html'>http://www.cs.utexas.edu/users/moore/publications/acl2-programming-exercises1.html</a>.</p> \u000A \u000A <p>However, we strongly recommend that you first invest in learning either the \u000A <see topic=\"ACL2____EMACS\">Emacs</see> or Eclipse-based <see topic=\"ACL2____ACL2-SEDAN\">ACL2-Sedan</see> program development \u000A environments, since it is foolish to try to learn how to program in a \u000A stand-alone read-eval-print loop!</p> \u000A \u000A <p>While getting started, many users find the Hyper-Card a handy index into \u000A the documentation for the ACL2 language:</p> \u000A \u000A <p><a \u000A href='http://www.cs.utexas.edu/users/moore/publications/hyper-card.html'>http://www.cs.utexas.edu/users/moore/publications/hyper-card.html</a></p> \u000A \u000A <p>Once you are comfortable with the ACL2 programming language, use your \u000A browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____PATTERNED-CONGRUENCE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>This topic assumes familiarity with the basics of congruence \u000A rules; see <see topic=\"ACL2____CONGRUENCE\">congruence</see>.</p> \u000A \u000A <p>We begin our discussion by showing some patterned congruence rules and \u000A using them to illustrate some terminology.</p> \u000A \u000A <code>Example forms:\u000A\u000A; ``Shallow'' patterned congruence rule:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (e1 y1 y2)\u000A         (e2 (f1 3 y1 (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A             (f1 3 y2 (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))))\u000A\u000A; ``Deep'' patterned congruence rule:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (e1 y1 y2)\u000A         (e2 (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (foo x y1 z))\u000A             (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (foo x y2 z))))\u000A\u000A; ``Deep'' patterned congruence rule:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (e1 y1 y2)\u000A         (e2 (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (foo x (g y1) z))\u000A             (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (foo x (g y2) z))))</code> \u000A \u000A <p>In the example forms above, <v>e1</v> and <v>e2</v> are known equivalence \u000A relations, which (as for classic congruence rules) we call the ``inner'' and \u000A ``outer'' equivalences.  The examples above are not classic, because the \u000A ``function symbol'' of the rule &mdash; <v>f1</v> in the first example, \u000A <v>mv-nth</v> in the second and third examples &mdash; is not being applied to \u000A distinct variables.  The first example is called ``shallow'' because the \u000A ``variable'' of the rule, <v>y1</v>, is a top-level argument of the ``lhs'' of \u000A the rule, which is the first call of <v>f1</v>; the others do not have this \u000A property, as <v>y1</v> is buried under further function calls.</p> \u000A \u000A <p>We invite you to browse the community book \u000A <tt>demos/patterned-congruences.lisp</tt>, which provides several examples of \u000A patterned congruence rules and their use, as well as several examples of \u000A formulas that are illegal as patterned congruence rules for various \u000A reasons.</p> \u000A \u000A <p>We now define the class of patterned congruence rules.  The general form of \u000A a patterned congruence rule is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 x y)\u000A         (equiv2 lhs rhs)),</code> \u000A \u000A <p>where the rule is not classic (see <see topic=\"ACL2____CONGRUENCE\">congruence</see>) and the following \u000A conditions hold.  <v>Equiv1</v> and <v>equiv2</v> are known equivalence relations, \u000A which (as in the classic case) we call the ``inner'' and ``outer'' \u000A equivalences, respectively.  The terms <v>lhs</v> and <v>rhs</v> are function \u000A calls, and we call these the <v>lhs</v> and <v>rhs</v> of the rule, respectively. \u000A The variable <v>x</v> occurs in <v>lhs</v> and the variable <v>y</v> occurs in \u000A <v>rhs</v>.  These must be the only occurrences of <v>x</v> and <v>y</v> in either \u000A <v>lhs</v> or <v>rhs</v>, and <v>rhs</v> must be the result of substituting <v>y</v> \u000A for <v>x</v> in <v>lhs</v>.  None of the following may occur as a function symbol \u000A of <v>lhs</v> (or, equivalently, <v>rhs</v>): <v>if</v>, <v>implies</v>, <v>equal</v>, \u000A or a <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt>.</p> \u000A \u000A <p>Patterned congruence rules are used, much like classic congruence rules, by \u000A the ACL2 rewriter to determine which equivalence relations to maintain as it \u000A dives into a term.  Recall (see <see topic=\"ACL2____CONGRUENCE\">congruence</see>) that if it suffices for the \u000A rewriter to maintain <v>e2</v> when rewriting a term, then it suffices to \u000A maintain <v>e1</v> when rewriting an argument of that term provided there is an \u000A applicable congruence rule with outer equivalence <v>e2</v> and inner \u000A equivalence <v>e1</v>.  An analogous principle holds for patterned congruence \u000A rules, which by the way consider refinements of the outer equivalence just as \u000A is done by classic congruence rules.  But there is a new wrinkle because the \u000A lhs of a patterned congruence rule is a function call that can have \u000A non-variable arguments.  Consider the following events.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> e1 (x y)\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> e1)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f10 (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 3 x x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f11 (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> y))\u000A  x)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> e1-implies-iff-f11-cong-2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (e1 y1 y2)\u000A           (<see topic=\"ACL2____IFF\">iff</see> (f11 (f10 x) y1)\u000A                (f11 (f10 x) y2)))\u000A  :rule-classes (:congruence))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> f11 (:t f11) e1))</code> \u000A \u000A <p>The following proof fails, because the indicated call of <v>f10</v> expands \u000A before the rewriter is applied to <v>b2</v> &mdash; otherwise <v>b2</v> would \u000A rewrite to <v>b1</v> because <v>(e1 b1 b2)</v> is known in that context and by the \u000A rule above, it suffices to maintain <v>e1</v> when rewriting <v>b2</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (e1 b1 b2)\u000A              (<see topic=\"ACL2____IFF\">iff</see> (f11 (f10 a)\u000A                        b1)\u000A                   (f11 (f10 a) ; expands before b2 is rewritten\u000A                        b2))))</code> \u000A \u000A <p>On the other hand, the following succeeds because the rewrite discussed \u000A above does indeed take place when the indicated call of <v>f10</v> does not \u000A expand.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (e1 b1 b2)\u000A           (<see topic=\"ACL2____IFF\">iff</see> (f11 (f10 a)\u000A                     b1)\u000A                (f11 (f10 a) ; does not expand\u000A                     b2)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> f10))))</code> \u000A \u000A <p>This inherent sequentiality of matching is important for soundness, as is \u000A illustrated by examples using the function <v>some-consp</v> in the \u000A aforementioned community book, <tt>demos/patterned-congruences.lisp</tt>.</p> \u000A \u000A <p>The example above illustrates the following point: as the rewriter dives \u000A into a term, then when matching is attempted using a patterned congruence \u000A rule, rewriting has already taken place to the left of the current subterm. \u000A By contrast, note that this pass through the rewriter will not yet have \u000A completed rewriting of terms to the right of the current subterm when matching \u000A a patterned congruence rule.</p> \u000A \u000A <p>Patterned congruence rules are used primarily during the process of \u000A <i>rewriting</i>.  In particular, unlike classic congruence rules, they are \u000A <i>not</i> used to do equality substitution at the goal level.  The simplest \u000A way to understand this point may be with the following trivial example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x y z) t)\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u (<see topic=\"COMMON-LISP_____A2\">*</see> a b))\u000A              (foo u u b)))</code> \u000A \u000A <p>The theorem fails, of course, but what is interesting is that ACL2 \u000A simplifies it by replacing the variable <v>u</v> by the term <v>(<see topic=\"COMMON-LISP_____A2\">*</see> a b)</v> \u000A throughout the goal and then dropping the equality, to produce the following \u000A goal.</p> \u000A \u000A <code>(FOO (<see topic=\"COMMON-LISP_____A2\">*</see> A B) (<see topic=\"COMMON-LISP_____A2\">*</see> A B) B)</code> \u000A \u000A <p>This sort of substitution can also be made when <v>equal</v> is replaced by a \u000A known equivalence relation, if a classic congruence rule for <v>foo</v> \u000A justifies the substitution.  But that is not the case for our implementation \u000A of patterned congruence rules.</p> \u000A \u000A <p>The discussion above is intended to suffice for effective use of patterned \u000A congruence rules.  If you are interested in their <i>implementation</i>, we \u000A invite you to read the long comment entitled ``Essay on Patterned Congruences \u000A and Equivalences'' in ACL2 source file <tt>rewrite.lisp</tt>.</p>"],

 "ACL2____NOTE-3-6":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 3.5 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, distributed books, Emacs support, and experimental \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see> version.  Each change is described in just one \u000A category, though of course many changes could be placed in more than one \u000A category.</p> \u000A \u000A <p>Note that (starting with ACL2 Version 3.5) LispWorks is no longer supported \u000A as a platform for ACL2, as the LispWorks compiler could not handle the ACL2 \u000A sources; see comments in the ACL2 sources about ``function size'' being ``too \u000A large''.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>In the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form, the function symbol (or symbols, \u000A plural, in the case of <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>) being defined may now be used \u000A in the specified <v>:measure</v> and <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> terms.  Note that, the \u000A definition(s) of the new symbol(s) are still not used in the termination \u000A proof.  Thanks to Daron Vroon for discussions leading to this addition for the \u000A measure and to Dave Greve for requesting such an enhancement for the \u000A guard.</p> \u000A \u000A <p>Processing of the <v>:guard-hints</v> in an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A form is now delayed until the start of <see topic=\"ACL2____GUARD\">guard</see> verification.  As a \u000A result, the function symbol(s) being defined may now be used as one might \u000A expect in such hints, for example in an <v>:in-theory</v> form.  Thanks to Jared \u000A Davis for suggesting that we make such an improvement and providing an \u000A example.</p> \u000A \u000A <p>Made a low-level change to <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> in support of the ACL2s \u000A utility ``<v>dynamic-make-event</v>''.  Thanks to Peter Dillinger for explaining \u000A the issue leading to this change.</p> \u000A \u000A <p>Modified the failure message printed when a measure conjecture fails to be \u000A proved, to indicate whether or not the hint <v>:ruler-extenders :all</v> would \u000A create a different measure conjecture.  Thanks to Peter Dillinger for \u000A suggesting such a modification.</p> \u000A \u000A <p>A call of <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt> had added hints to the end of the \u000A current list of default hints.  Now, it adds them to the beginning of that \u000A list, so that they are tried first.  However, <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt> now \u000A takes a keyword argument, <v>:at-end</v>.  If that argument is supplied and \u000A evaluates to other than <v>nil</v>, then the previous behavior occurs.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> is used to save ACL2 images, the build dates are now \u000A printed on separate lines at startup and in the executable script.  Thanks To \u000A Bob Boyer for requesting some newlines.</p> \u000A \u000A <p><tt><see topic=\"ACL2____FORWARD-CHAINING\">Forward-chaining</see></tt> rules are now generated so that every <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> \u000A (also <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> and <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt>) expression is expanded away, as is \u000A every call of a so-called ``guard holder'' (<tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>, <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, <tt><see topic=\"COMMON-LISP____THE\">the</see></tt>).  These were formerly only expanded \u000A away in the conclusion.  Thanks to Konrad Slind for a helpful email leading to \u000A this change.</p> \u000A \u000A <p><tt><see topic=\"ACL2____CURRENT-THEORY\">Current-theory</see></tt> now causes a hard error when applied to a name not \u000A found in the current ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, rather than simply returning \u000A <v>nil</v>.</p> \u000A \u000A <p>When the underlying Common Lisp is GCL, ACL2 no longer uses the <v>#n=</v> \u000A reader macro when writing out certain files, including <see topic=\"ACL2____CERTIFICATE\">certificate</see> \u000A files.  In all other Lisps, it now uses the <v>#n=</v> ``print-circle'' feature \u000A not only for <see topic=\"ACL2____CERTIFICATE\">certificate</see> files and ``<v>expansion.lsp</v>'' files written \u000A for example in support of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, but also for files written in \u000A support of <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>.  This is all managed with new <see topic=\"ACL2____STATE\">state</see> \u000A global variable <v>print-circle-files</v>; see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>.  Thanks to \u000A Dave Greve for pointing out that GCL is limited by default to 1024 indices for \u000A <v>#n=</v>.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>A documentation topic explains in some detail how <see topic=\"ACL2____HINTS\">hints</see> work with \u000A the ACL2 proof ``waterfall'': see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>.  This topic \u000A may be useful to those who write computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) or \u000A other advanced hints.</p> \u000A \u000A <p>Added a new hint keyword, <v>:no-thanks</v>, that avoids printing the usual \u000A ``Thanks'' message for <see topic=\"ACL2____HINTS\">hints</see>.  Thanks to Peter Dillinger for requesting \u000A this feature.</p> \u000A \u000A <p>Added a new hint keyword, <v>:backtrack</v>, that checks the goals produced \u000A by processing a goal, and can cause the goal to be re-processed using a new \u000A hint.  See <see topic=\"ACL2____HINTS\">hints</see>.  Thanks to Pete Manolios for a conversation that led \u000A to the idea of this hint.</p> \u000A \u000A <p>Added a new class of hints, <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, that is similar to <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>, except that override-hints are always applied, even if the \u000A user has supplied a hint explicitly for the goal.  See <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>. \u000A Thanks to Pete Manolios and Harsh Raju Chamarthi for useful discussions on \u000A this topic, including its application to testing.</p> \u000A \u000A <p>When a goal ready to be pushed for proof by induction is given the new hint \u000A ``<v>:do-not-induct :otf</v>'', it is indeed pushed for proof by induction, \u000A rather than causing immediate failure as in the case of the hint \u000A ``<v>:do-not-induct t</v>''.  Instead, the proof fails when the goal is later \u000A picked to be proved by induction.  Thanks to Peter Dillinger for discussions \u000A leading to this feature.</p> \u000A \u000A <p>Related to computed hints only: Each history entry in the list stored in \u000A variable <v>HIST</v> (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) now has a <v>:CLAUSE</v> field, \u000A which provide's access to a goal's parent, parent's parent, and so on (within \u000A the same induction and forcing round only).</p> \u000A \u000A <p>It is now possible to inhibit warnings produced by <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> and <see topic=\"ACL2____HINTS\">hints</see> that occur when certain built-in definitions and \u000A executable-counterparts are disabled: just evaluate <v>(<see topic=\"ACL2____ASSIGN\">assign</see>\u000A verbose-theory-warning nil)</v>.  Thanks to Jared Davis (and probably others in \u000A the past) for requesting such a mechanism.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>A source function (<v>linearize-lst</v>) was replaced by tail-recursive code, \u000A which can avoid a stack overflow.  Thanks to Dave Greve for sending a helpful \u000A example.</p> \u000A \u000A <p>The heuristics for limiting <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> have been slightly \u000A relaxed, to allow derivations based on the occurrence of all arguments of the \u000A forward-chaining rule's conclusion in the goal (after stripping leading calls \u000A of <v>NOT</v>).  Thanks to Dave Greve for contributing this improvement and \u000A providing a motivating example.</p> \u000A \u000A <p>We simplified induction schemes by eliminating each hypothesis of the form \u000A <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> term (<see topic=\"COMMON-LISP____QUOTE\">quote</see> const)))</v> for which some other hypothesis in the \u000A same case equates term with some (presumably other) quoted constant.  Thanks \u000A to Dave Greve for requesting an improvement of this sort.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a soundness bug related to redundancy of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events \u000A (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) and <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.  A proof of <v>nil</v> \u000A in ACL2 Version 3.5 appears in a comment in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-3-6 ...)</v> in \u000A the ACL2 source code.  The fix is to insist that in order for one <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event to be redundant with another, they must be evaluated with \u000A the same <see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see>.  Analogous to this issue of <see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see> for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>s is an issue of the <see topic=\"ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS\">default-verify-guards-eagerness</see>, which has similarly been fixed.</p> \u000A \u000A <p>Fixed soundness bugs related to the handling of <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see> for <see topic=\"ACL2____CONSTRAINT\">constraint</see>s.  Proofs of <v>nil</v> in ACL2 \u000A Version 3.5 appear in a comment in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-3-6 ...)</v> in the ACL2 \u000A source code.</p> \u000A \u000A <p>Fixed a bug that could cause the following error during calls of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> in the presence of calls of <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> in the book:</p> \u000A \u000A <code>ACL2 Warning [Skip-proofs] in\u000A\u000AHARD ACL2 ERROR in FMT0: Illegal Fmt Syntax.  The tilde-@ directive at\u000Aposition 0 of the string below is illegal because its variable evaluated\u000Ato 0, which is neither a string nor a list.\u000A\u000A&quot;~@0&quot;</code> \u000A \u000A <p>Thanks to Dave Greve for reporting this bug and making available a very \u000A helpful test case.</p> \u000A \u000A <p>The <v>:corollary</v> of a rule (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) failed to use the \u000A <tt><see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see></tt> of the logical <see topic=\"ACL2____WORLD\">world</see>.  This has been fixed.</p> \u000A \u000A <p>(CCL only) We removed a call, for CCL 1.3 (and beyond) only, of foreign \u000A function <v>sync</v> in the closing of output channels.  Thanks to Daron Vroon \u000A for reporting issues with such a call on a non-Intel platform.</p> \u000A \u000A <p>Fixed a bug in reporting failures when monitoring rewrite rules with free \u000A variables in the hypotheses, that could cause a hard Lisp error (from which \u000A ACL2 continues, however).  Thanks to Eric Smith for sending a very helpful \u000A example with his bug report.</p> \u000A \u000A <p>Fixed the handling of <v>:induct</v> <see topic=\"ACL2____HINTS\">hints</see>, which had thrown away hint \u000A information from parent goals.  For example, the <tt><see topic=\"ACL2____THM\">thm</see></tt> form below failed \u000A to prove even though the second hint is in some sense superfluous; induction \u000A occurs automatically at <v>&quot;Goal'&quot;</v> even without the hint on that.  The \u000A failure was due to discarding the hint information on <v>&quot;Goal&quot;</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> append))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z))) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A     :hints\u000A     ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> append))\u000A      (&quot;Goal'&quot; :induct t) ; failed unless this line is commented out\u000A      ))</code> \u000A \u000A <p>Fixed a bug in the <tt><see topic=\"ACL2____ARGS\">args</see></tt> command that was failing to show the \u000A formals of primitives (built-in functions like <v>consp</v> that do not come \u000A with explicit definitions).  Thanks to John Cowles for pointing out this bug. \u000A (At a lower level, the bug was that primitives failed to have <v>'stobjs-in</v> \u000A or <v>'stobjs-out</v> properties.)</p> \u000A \u000A <p>Fixed bugs in the utility supporting moving directories of certified books, \u000A sometimes used in Debian builds (as described in source function \u000A <v>make-certificate-file</v>).  Thanks to Alan Dunn for pointing out such a bug, \u000A in paths associated with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events in <see topic=\"ACL2____PORTCULLIS\">portcullis</see> commands \u000A in <see topic=\"ACL2____CERTIFICATE\">certificate</see>s (which are used for error reporting).  There were also \u000A bugs, now fixed, that prevented renaming some book paths.  Please note that \u000A this mechanism is not guaranteed to be sound; in particular, it can probably \u000A misbehave when macros are used to generate portcullis events.  However, it \u000A seems likely that such issues will be very rare.</p> \u000A \u000A <p>Eliminated warnings that could arise when tracing a function with <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  Now, when <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> is applied to a function without option \u000A <v>:native</v>, that function's declarations and documentation are \u000A discarded.</p> \u000A \u000A <p>Fixed a bug that could cause a failure when building an executable image \u000A using SBCL as the underlying Common Lisp.  Thanks to Jun Sawada for reporting \u000A this failure.  We made a similar fix for CMUCL.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> in the case that an absolute pathnanme is \u000A supplied.  Thanks to Jared Davis for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in the use of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> with the <v>:native</v> and \u000A <v>:multiplicity</v> options that caused hard errors for some underlying \u000A Lisps.</p> \u000A \u000A <p>Fixed a bug in the interaction of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>, \u000A which caused error as <tt><see topic=\"ACL2____COMP\">comp</see></tt> tried to re-trace functions that it \u000A temporarily untraced.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS AND RELATED INFRASTRUCTURE</b></p> \u000A \u000A <p>See <a \u000A href='https://code.google.com/p/acl2-books/wiki/ReleaseVersionNumbers'>ReleaseVersionNumbers</a> \u000A for a list of books in Version 3.6 of ACL2 but not Version 3.5.  For \u000A <i>changes</i> to existing books rather than <i>additions</i>, see the <a \u000A href='http://code.google.com/p/acl2-books/source/list'>log entries</a> \u000A starting with revision r286 up through revision r329.</p> \u000A \u000A <p>It is no longer required to specify a value for environment (or `make') \u000A variable <v>ACL2_SYSTEM_BOOKS</v> when running `make' in the distributed book \u000A directory, even when that directory is not under the directory containing the \u000A ACL2 executable.  Thanks to Peter Dillinger for providing this improvement, by \u000A modifying <v>books/Makefile-generic</v> and, as needed, distributed book \u000A Makefiles.</p> \u000A \u000A <p>Thanks to Peter Dillinger, some books in support of the ACL2 Sedan (ACL2s) \u000A are more easily available for ACL2 users who do not use ACL2s.  See <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see>.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>If the following form is evaluated before the load of \u000A <v>emacs/emacs-acl2.el</v>, then variables are now set to reflect the directory \u000A containing that file.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (boundp '*acl2-sources-dir*)\u000A    (makunbound '*acl2-sources-dir*))</code> \u000A \u000A <p>Fixed info-dir-entry line in generated info file (by patching \u000A <v>doc/write-acl2-texinfo.lisp</v>).  Thanks to Alan Dunn for providing this \u000A patch.</p> \u000A \u000A <p><b>EXPERIMENTAL HONS VERSION</b></p> \u000A \u000A <p>Bob Boyer and others have contributed numerous changes for the experimental \u000A ``<v>hons</v>'' version of ACL2 (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).  A number of \u000A these have been crafted to work specifically with CCL (Clozure Common Lisp, \u000A formerly OpenMCL), which is now required as the underlying Lisp for the \u000A ``<v>hons</v>'' version of ACL2.</p> \u000A \u000A <p>A heuristic (source function <v>too-many-ifs</v> has been made more scalable \u000A (for the non-HONS version as well, in fact), but with no functional change. \u000A Thanks to Jared Davis for noticing performance issues and suggesting \u000A fixes.</p> \u000A \u000A <p>Other changes including the following, quoting Bob Boyer:</p> \u000A \u000A <blockquote><p>The CCL CASE macro now does better than a dumb linear search in \u000A some CASEes.</p> \u000A \u000A <p>SH and CSH are functions to talk to the underlying Gnu-Linux from CCL. \u000A Good for repeated calling when you simply cannot afford the copying cost of a \u000A FORK because you are using, say, a dozen gigabytes.</p> \u000A \u000A <p>Added CCL compiler-macros for IF and OR, to support some 'coverage' \u000A analysis, cf. IF-REPORT, extending the profiling.</p> \u000A \u000A <p>Introduced the type 'mfixnum' so that things like counting honses and \u000A counting calls to memoized or profiled functions can run fast in CCL 64 bits \u000A and yet still run at all under 32 bits.</p> \u000A \u000A <p>Moved all HONSes to CCL's newish static space, which permits the address of \u000A a cons to be used as a hash key, as in most Lisps.  (CCL moves most conses and \u000A most everything when it does a compacting-gc.)</p> \u000A \u000A <p>Quite a few changes in the memoize-fn reporting.</p> \u000A \u000A <p>Added a timer facility, cf. call-with-timeout.  Good for running under \u000A throttle some gross thoughts like 'Is it true that you can't fit 12 pigeons \u000A into 11 holes' on some propositional calculus systems/functions.</p> \u000A \u000A <p>Added rwx-size, pid-owner-cmdlines, rss, and proc-stat to help see what is \u000A really going on virtually in Gnu-Linux.</p> \u000A \u000A <p>Fixed at least one bug in compact-print-file and helped make its \u000A integration into ACL2's read-object$ a little more sound.  Still worried some \u000A about *print-readably* vs. readtable-case.  Does anyone else stay awake late \u000A at night worrying about readtable-case?</p> \u000A \u000A <p>Revised how the *watch-dog-process* interrupts the main process for the \u000A thousandth time, cf. watch.  Haven't changed it in weeks, which means that (a) \u000A it is getting tolerable or (b) I've run out of gas.</p></blockquote>"],

 "ACL2____NOTE-3-3":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 3.2.1 into new \u000A features, bug fixes, prover algorithm enhancements, and miscellaneous.  Also \u000A see <see topic=\"ACL2____NOTE-3-2-1\">note-3-2-1</see> for other changes since Version 3.2.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>A new ``gag-mode'' provides less verbose, more helpful feedback from the \u000A theorem prover, in support of The Method (see <see topic=\"ACL2____THE-METHOD\">the-method</see>).  See <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>.  We recommend the use of gag-mode, which may become the default \u000A in future ACL2 releases, and we welcome suggestions for improvement.  We thank \u000A Robert Krug and Sandip Ray for helpful feedback in the design of <see topic=\"ACL2____GAG-MODE\">gag-mode</see>.  Note that when proofs fail, then even without gag-mode and even if \u000A proof output is inhibited, the summary will contain a useful listing of \u000A so-called ``key checkpoints'' (see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>).</p> \u000A \u000A <p>Added support for a leading `<v>~</v>' in filenames.  Thanks to Bob Boyer for \u000A suggesting this enhancement.  Note that since `<v>~/</v>' depends on the user, \u000A it is disallowed in <see topic=\"ACL2____BOOKS\">books</see> to be certified (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>), \u000A since otherwise an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form in a book, <v>b</v>, could have a \u000A different meaning at certification time than at the time <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A is later executed on book <v>b</v>.</p> \u000A \u000A <p>Made a change to allow <v>(<see topic=\"ACL2____TIME_42\">time$</see> FORM)</v> and <v>(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>\u000A TIME FORM)</v> when <v>FORM</v> includes <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> calls that change the \u000A ACL2 <see topic=\"ACL2____WORLD\">world</see>.  Thanks to Jared Davis for requesting such support for \u000A <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>.</p> \u000A \u000A <p>Computed <see topic=\"ACL2____HINTS\">hints</see> (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) may now produce a \u000A so-called ``error triple'', i.e., a result of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp val\u000A state)</v>, where a non-<v>nil</v> <v>erp</v> causes an error, and otherwise \u000A <v>val</v> is the value of the hint.  It remains legal for a computed hint to \u000A return a single ordinary value; indeed, the symbol form of a computed hint \u000A must still be a function that returns an ordinary single value.</p> \u000A \u000A <p>New hints provide additional control of the theorem prover, as follows. \u000A See <see topic=\"ACL2____HINTS\">hints</see> for more details, and see new distributed book directory \u000A <v>books/hints/</v> for examples, in particular file <v>basic-tests.lisp</v> in \u000A that directory for simple examples.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o The hint <v>:OR (hints-1 ... hints-k)</v> causes an attempt to prove the \u000A specified goal using each <v>hints-i</v> in turn, until the first of these \u000A succeeds.  If none succeeds, then the prover proceeds after heuristically \u000A choosing the ``best'' result, taking into account the goals pushed in each \u000A case for proof by induction.</p> \u000A \u000A <p>o A custom hint is a keyword that the user associates with a corresponding \u000A hint-generating function by invoking <tt><see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see></tt>.  Thus, a \u000A custom hint may be viewed as a convenient sort of computed hint.</p> \u000A \u000A <p>o A custom hint, <v>:MERGE</v>, is implemented in distributed book \u000A <v>books/hints/merge.lisp</v>.  It is useful for combining hints.</p> \u000A \u000A <p>o A sophisticated yet useful custom hint is the <v>:CONSIDER</v> hint \u000A implemented in distributed book <v>books/hints/consider-hint.lisp</v>.  With \u000A this hint, you can for example give the equivalent of a <v>:USE</v> hint without \u000A the need to supply an instantiation.  Include that book in order to see \u000A documentation online with <v>:doc consideration</v>, and see the book \u000A <v>books/hints/consider-hint-tests.lisp</v> for examples.</p></blockquote> \u000A \u000A <p>A new hint, <v>:</v><tt><see topic=\"ACL2____REORDER\">reorder</see></tt>, allows the specification of which \u000A subgoals are to be considered first.  Thanks to Sandip Ray for putting forward \u000A this idea.</p> \u000A \u000A <p>Enhanced <tt><see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see></tt> by supporting a second argument of \u000A <v>:same</v>, which avoids changing which output is inhibited.</p> \u000A \u000A <p>Added macros <v>thm?</v> and <v>not-thm?</v> to distributed book \u000A <v>books/make-event/eval.lisp</v>, so that it's easy to test within a certified \u000A book that a proof attempt succeeds or that it fails.</p> \u000A \u000A <p>Added printing function <tt><see topic=\"ACL2____CW_12\">cw!</see></tt>, which is analogous to <tt><see topic=\"ACL2____CW\">cw</see></tt> just \u000A as <tt><see topic=\"ACL2____FMT_12\">fmt!</see></tt> is to <tt><see topic=\"ACL2____FMT\">fmt</see></tt>, i.e., printing so that the result can be \u000A read back in.  Thanks to Jared Davis for suggesting this enhancement (after \u000A doing his own implementation).</p> \u000A \u000A <p>The ACL2 customization file can now be specified using environment variable \u000A <v>ACL2-CUSTOMIZATION</v> [note: starting with Version_4.0, \u000A <v>ACL2_CUSTOMIZATION</v>].  See <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.  Thanks to Peter \u000A Dillinger for requesting this feature.</p> \u000A \u000A <p>Added new emacs capabilities for proof trees (all documented in emacs):</p> \u000A \u000A <blockquote> \u000A \u000A <p>o New function start-proof-tree-noninteractive, for example<br></br> \u000A \u000A   (start-proof-tree-noninteractive \"*shell*\")</p> \u000A \u000A <p>o C-z o Switch to another frame</p> \u000A \u000A <p>o C-z b Switch to prooftree buffer</p> \u000A \u000A <p>o C-z B Switch to prooftree buffer in \"prooftree-frame\" \u000A frame</p></blockquote> \u000A \u000A <p>Added Common Lisp function, <v>search</v>, as a macro in <tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, \u000A with limited support for keyword arguments.  Thanks to Warren Hunt for \u000A requesting this addition.</p> \u000A \u000A <p>Sandip Ray has contributed a book, <v>books/make-event/defrefine.lisp</v>, \u000A that provides a collection of macros to aid in reasoning about ACL2 functions \u000A via refinement.</p> \u000A \u000A <p>Wrote and incorporated new utility for listing all the theorems in an \u000A included book.  See <v>books/misc/book-thms.lisp</v>.  Thanks to Jared Davis for \u000A requesting this functionality.</p> \u000A \u000A <p>The new distributed book <v>misc/defp.lisp</v> generalizes the <tt><see topic=\"ACL2____DEFPUN\">defpun</see></tt> \u000A macro to allow more general forms of tail recursion.</p> \u000A \u000A <p>(Low-level printing improvement) A new function, \u000A <v>set-ppr-flat-right-margin</v>, allows the right margin for certain kinds of \u000A ``flat'' printing to exceed column 40.  Thanks to Jared Davis for pointing out \u000A that state global variables <v>'fmt-hard-right-margin</v> and \u000A <v>'fmt-soft-right-margin</v> are not alone sufficient to extend the right \u000A margin in all cases.</p> \u000A \u000A <p>The event <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> can now take a relative pathname as \u000A an argument.  Formerly, it required an absolute pathname.</p> \u000A \u000A <p>A new book, <v>books/misc/defopener.lisp</v>, provides a utility creating a \u000A theorem that equates a term with its simplification.</p> \u000A \u000A <p>ACL2 now provides limited support for the Common Lisp primitive <v>FLET</v>, \u000A which supports local function bindings.  See <see topic=\"COMMON-LISP____FLET\">flet</see>.  Thanks to Warren \u000A Hunt for requesting this feature.</p> \u000A \u000A <p>Added a definition of <tt><see topic=\"ACL2____BOOLE_42\">boole$</see></tt>, a close analogue of Common Lisp \u000A function <v>boole</v>.  Thanks to Bob Boyer for providing an initial \u000A implementation.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> to inhibit a potentially useless theory warning.</p> \u000A \u000A <p>Fixed a bug in the application of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to relative \u000A pathnames for files in other than the current directory.  Thanks to Amr Helmy \u000A for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> for displaying rules \u000A of class <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>.  Thanks to Jared Davis for finding this bug and \u000A providing a fix.</p> \u000A \u000A <p>Formerly, <tt><see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see></tt> was not a legal event form \u000A for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms and <see topic=\"ACL2____BOOKS\">books</see>.  This has been fixed.  Thanks \u000A to Robert Krug and Sandip Ray for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed the handling of <see topic=\"ACL2____HINTS\">hints</see> in <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands for the \u000A prover, such as <v>bash</v> &mdash; see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see> &mdash; so \u000A that the user can override the default settings of hints, in particular of \u000A <v>:do-not</v> and <v>:do-not-induct</v> hints attached to <v>&quot;Goal&quot;</v>.  This \u000A fix also applies to the distributed book <v>misc/bash.lisp</v>, where Robert \u000A Krug noticed that he got an error with <v>:hints ((&quot;Goal&quot; :do-not\u000A '(preprocess)))</v>; we thank Robert for pointing out this problem.</p> \u000A \u000A <p>Fixed a bug in handling of <see topic=\"ACL2____STOBJ\">stobj</see>s occurring in guards of functions \u000A whose <see topic=\"ACL2____GUARD\">guard</see>s have been verified.  In such cases, a raw Lisp error was \u000A possible when proving theorems about non-''live'' <see topic=\"ACL2____STOBJ\">stobj</see>s.  We thank \u000A Daron Vroon for sending us an example that highlighted this bug.  The \u000A following (simpler) example causes such an error in previous versions of \u000A ACL2.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs st :guard (fld st)))\u000A  st)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo '(3))\u000A            '(3)))</code> \u000A \u000A <p>The <see topic=\"ACL2____DMR\">dmr</see> feature for dynamic monitoring of rewrites had a bug, where \u000A the file used for communicating with emacs was the same for all users, based \u000A on who built the ACL2 executable image.  This has been fixed.  Thanks to \u000A Robert Krug for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in some warnings, in particular the warning for including an \u000A uncertified book, that was giving an incorrect warning summary string.</p> \u000A \u000A <p>Inclusion of uncertified books erroneously re-used <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> \u000A expansions that were stored in stale <see topic=\"ACL2____CERTIFICATE\">certificate</see>s.  This is no longer \u000A the case.  Thanks to Jared Davis for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug that was disallowing calls of <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> in <see topic=\"ACL2____EVENTS\">events</see> that were executing before calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A <p>Modified the functionality of <v>binop-table</v> so other than binary \u000A function symbols are properly supported (hence with no action based on \u000A right-associated arguments).  See <see topic=\"ACL2____ADD-BINOP\">add-binop</see>.</p> \u000A \u000A <p>Fixed small <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> issues related to packages.  Emacs \u000A commands <v>ctrl-t d</v> and <v>ctrl-t ctrl-d</v> now work properly with colon \u000A (`<v>:</v>') and certain other punctuation characters.  The <v>p-top</v> command \u000A now prints ``<v>***</v>'' regardless of the current package.</p> \u000A \u000A <p>Fixed a bug that allowed <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to succeed without specifying \u000A value <v>t</v> for keyword argument <v>:skip-proofs-okp</v>, even with <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in the certification <see topic=\"ACL2____WORLD\">world</see> depending on \u000A events executed under <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt>.</p> \u000A \u000A <p>Improved <tt><see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see></tt> in the following two ways. \u000A Thanks to Robert Krug and Bill Young for requesting these improvements and for \u000A providing useful feedback.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o It can provide more complete information when aborting a proof.</p> \u000A \u000A <p>o The <v>:frames</v> reported for a rule are categorized as ``useful'' and \u000A ``useless'' according to whether they support ``useful'' or ``useless'' \u000A <v>:tries</v> of that rule, respectively.  See <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> \u000A for further explanation.</p></blockquote> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> so that the reported time and warnings include \u000A those from the expansion phase.  In analogy with <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, the rules reported still do not include those from subsidiary \u000A events (including the expansion phase).  A related change to <tt><see topic=\"ACL2____LD\">ld</see></tt> avoids \u000A resetting summary information (time, warnings) with each top-level form \u000A evaluation; <see topic=\"ACL2____EVENTS\">events</see> already handle this information themselves.</p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt> so that all warnings are inhibited \u000A when <v>warning!</v> but not <v>warning</v> is included in the list.  Formerly, \u000A only soundness-related warnings were inhibited in this case.  Thanks to Eric \u000A Smith for bringing this bug to our attention.</p> \u000A \u000A <p>Distributed directory <v>doc/HTML/</v> now again includes installation \u000A instructions (which was missing in Version_3.2.1), in \u000A <v>doc/HTML/installation/installation.html</v>.</p> \u000A \u000A <p>Some fixes have been made for <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> support.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o <see topic=\"ACL2____PROOF-TREE\">Proof-tree</see> output is no longer inhibited automatically during \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, though it continues to be inhibited by default (i.e., \u000A ACL2 continues to start up as though <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt> has been \u000A called with argument <v>'(<see topic=\"ACL2____PROOF-TREE\">proof-tree</see>)</v>).</p> \u000A \u000A <p>o Fixed a bug in Xemacs support for <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> help keys <v>C-z h</v> \u000A and <v>C-z ?</v>.</p> \u000A \u000A <p>o Fixed a bug in <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>s that was failing to deal with the case \u000A that a goal pushed for proof by induction is subsumed by such a goal to be \u000A proved later.  Now, the proof-tree display regards such subsumed goals as \u000A proved, as is reported in the theorem prover's output.</p></blockquote> \u000A \u000A <p>Fixed a bug that was disallowing <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt> forms inside <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms in a certification <see topic=\"ACL2____WORLD\">world</see> (see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>).</p> \u000A \u000A <p>If the <v>:load-compiled-file</v> argument of a call of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A is <v>:comp</v>, then an existing compiled file will be loaded, provided it is \u000A more recent than the corresponding book (i.e., <v>.lisp</v> file).  A bug was \u000A causing the compiled file to be deleted and then reconstructed in the case of \u000A <v>:comp</v>, where this behavior was intended only for <v>:comp!</v>.</p> \u000A \u000A <p>Fixed a bug that was avoiding compilation of some executable counterparts \u000A (sometimes called ``*1* functions'') during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, and also \u000A during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with <v>:load-compiled-file</v> value of <v>:comp</v> \u000A or <v>:comp!</v>).  Thanks to Eric Smith for sending a small example to bring \u000A this bug to our attention.</p> \u000A \u000A <p>Incorporated a fix from Eric Smith for a typo (source function \u000A <v>ancestors-check1</v>) that could cause hard Lisp errors.  Thanks, Eric!</p> \u000A \u000A <p>Fixed the following issues with packages and book <see topic=\"ACL2____CERTIFICATE\">certificate</see>s.  See \u000A <see topic=\"ACL2____HIDDEN-DEATH-PACKAGE\">hidden-death-package</see> if you are generally interested in such issues, \u000A and for associated examples, see comments on ``Fixed the following issues with \u000A packages'' in <v>note-3-3</v> in the ACL2 source code.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Reduced the size of <v>.cert</v> files by eliminating some unnecessary \u000A <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events generated for the <see topic=\"ACL2____PORTCULLIS\">portcullis</see>.</p> \u000A \u000A <p>o Fixed a bug that has caused errors when reading symbols from a <see topic=\"ACL2____PORTCULLIS\">portcullis</see> that are in undefined packages defined in locally included \u000A books.</p> \u000A \u000A <p>o Fixed a bug that could lead to failure of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> caused by \u000A a subtle interaction between <tt><see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see></tt> and <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events \u000A generated for the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of a <see topic=\"ACL2____CERTIFICATE\">certificate</see>.</p></blockquote> \u000A \u000A <p><b>PROVER ALGORITHM ENHANCEMENTS</b></p> \u000A \u000A <p>Non-linear arithmetic (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>) has been improved \u000A to be more efficient or more powerful in some cases.  Thanks to Robert Krug \u000A for contributing these improvements.</p> \u000A \u000A <p>Improved certain (so-called ``<tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt>'') reasoning about whether \u000A or not expressions denote integers.  Thanks to Robert Krug for contributing \u000A code to implement this change, along with examples illustrating its power that \u000A are now distributed in the book \u000A <v>books/misc/integer-type-set-test.lisp</v>.</p> \u000A \u000A <p>Improved ACL2's heuristics for relieving hypotheses, primarily to use \u000A linear reasoning on conjuncts and disjuncts of the test of an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> \u000A expression.  For example, given a hypothesis of the form <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> term1\u000A term2) ...)</v>, ACL2 will now use linear reasoning to attempt to prove both \u000A <v>term1</v> and <v>term2</v>, not merely for <v>term2</v>.  Thanks to Robert Krug \u000A for supplying examples illustrating the desirability of such an improvement \u000A and for useful discussions about the fix.</p> \u000A \u000A <p>Made a slight heuristic change, so that when a hypothesis with <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> \u000A or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> subterms (i.e. <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> subterms) rewrites to <v>t</v>, \u000A then that hypothesis is necessarily eliminated.  Thanks to Jared Davis for \u000A sending an example that led us to develop this change, and thanks to Robert \u000A Krug for a helpful discussion.</p> \u000A \u000A <p><b>MISCELLANEOUS</b></p> \u000A \u000A <p>Added documentation on how to use <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> to avoid duplicating \u000A expensive computations, thanks to Jared Davis.  See <see topic=\"ACL2____USING-TABLES-EFFICIENTLY\">using-tables-efficiently</see>.</p> \u000A \u000A <p>Modified the error message for calls of undefined functions to show the \u000A arguments.  Thanks to Bob Boyer for requesting this enhancement.</p> \u000A \u000A <p>Modified utilies <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt>, <v>:</v><tt><see topic=\"ACL2____PR_12\">pr!</see></tt>, <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>, and \u000A <v>:</v><tt><see topic=\"ACL2____SHOW-BODIES\">show-bodies</see></tt> to incorporate code contributed by Jared Davis. \u000A That code defines low-level source functions <v>info-for-xxx</v> that collect \u000A information about rules, which is thus available to advanced users.</p> \u000A \u000A <p>Dynamic monitoring of rewrites (see <see topic=\"ACL2____DMR\">dmr</see>) has been improved in the \u000A following ways, as suggested by Robert Krug.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Some stale entries from the rewrite stack are no longer printed, in \u000A particular above <v>ADD-POLYNOMIAL-INEQUALITIES</v>.</p> \u000A \u000A <p>o An additional rewrite stack entry is made when entering non-linear \u000A arithmetic (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>).</p> \u000A \u000A <p>o An <v>ADD-POLYNOMIAL-INEQUALITIES</v> entry is printed with a counter, to \u000A show how often this process is called.</p></blockquote> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> so that the newly-saved image will have the same \u000A raw Lisp package as the existing saved image.  This is a very technical change \u000A that will likely not impact most users; for example, the package in the ACL2 \u000A read-eval-print loop (see <see topic=\"ACL2____LP\">lp</see>) had already persisted from the original \u000A to newly-saved image.  Thanks to Jared Davis for suggesting this change.</p> \u000A \u000A <p>Changed <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion so that changes to <tt><see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see></tt>, <tt><see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see></tt>, \u000A <v>set-fmt-soft-right-margin</v>, and <v>set-fmt-hard-right-margin</v> will \u000A persist after being evaluated during <v>make-event</v> expansion. \u000A (Specifically, <v>*protected-system-state-globals*</v> has been modified; see \u000A <see topic=\"ACL2____MAKE-EVENT-DETAILS\">make-event-details</see>.)  Thanks to Jared Davis for bringing this issue to \u000A our attention.</p> \u000A \u000A <p>Output from the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> is now always enabled when invoking \u000A <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, even if it is globally inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>).</p> \u000A \u000A <p>Improved the message printed when an <v>:induct</v> hint fails, to give more \u000A information in some cases.  Thanks to Rex Page for suggesting where an \u000A improvement could be made and providing useful feedback on an initial \u000A improvement.</p> \u000A \u000A <p>Added a warning for <see topic=\"ACL2____CONGRUENCE\">congruence</see> rules (see <see topic=\"ACL2____DEFCONG\">defcong</see>) that \u000A specify <tt><see topic=\"ACL2____IFF\">iff</see></tt> as the second equivalence relation when <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> can \u000A be used instead.  Those who heed these warnings can eliminate certain \u000A subsequent <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt> warnings for <see topic=\"ACL2____REWRITE\">rewrite</see> rules with \u000A conclusions of the form <v>(<see topic=\"ACL2____IFF\">iff</see> term1 term2)</v>, and hence implicitly for \u000A Boolean conclusions <v>term1</v> that are interpreted as <v>(<see topic=\"ACL2____IFF\">iff</see> term1 t)</v>. \u000A Thanks to Sarah Weissman for sending us an example that highlighted the need \u000A for such a warning.</p> \u000A \u000A <p>Modified macro <v>:</v><tt><see topic=\"ACL2____REDEF_12\">redef!</see></tt> (which is for system implementors) so \u000A that it eliminates untouchables.</p> \u000A \u000A <p>Several improvements have been made to the experimental hons/memoization \u000A version of ACL2.  See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>The distributed books directory, <v>(<see topic=\"ACL2_____04\">@</see> distributed-books-dir)</v>, is now \u000A printed in the start-up message.</p>"],

 "ACL2____NOTE-3-2-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>(OpenMCL and multi-threaded SBCL only) Fixed a soundness bug in the \u000A evaluation of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> forms in the presence of Lisp feature \u000A <v>:acl2-mv-as-values</v>.  Thanks to Sol Swords for reporting this problem and \u000A sending a simple proof of <v>nil</v> (which can be found in a comment in the \u000A ACL2 sources, in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-3-2-1 ...)</v>).</p> \u000A \u000A <p>Added a new utility, <tt><see topic=\"ACL2____DMR\">dmr</see></tt> (Dynamicaly Monitor Rewrites), for \u000A watching the activity of the rewriter and some other proof processes.  See \u000A <see topic=\"ACL2____DMR\">dmr</see>.  We thank Robert Krug for useful contributions.</p> \u000A \u000A <p>Fixed a bug in evaluation of calls of <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>.</p> \u000A \u000A <p>Fixed the writing of executable scripts when building ACL2, so that the \u000A build-time value of environment variable <v>ACL2_SYSTEM_BOOKS</v> is no longer \u000A written there.  Thanks to Dave Greve for discussing this change.</p> \u000A \u000A <p>Fixed bugs in <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> (which are similarly present in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>sr</v> (<v>show-rewrites</v>) command.  The first bug was \u000A evident from the following forms sent by Robert Krug, which caused an \u000A error.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-3/floor-mod/floor-mod&quot; :dir :system)\u000A:pl (<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 x) n)</code> \u000A \u000A <p>The second bug was due to a failure to take note of which rules are \u000A disabled, and could be seen by executing the following (very slow!).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> modulus () t)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-3/floor-mod/floor-mod&quot; :dir :system)\u000A:pl (<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (modulus))</code> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> so that by default, all \u000A executable-counterpart functions (sometimes called ``*1* functions'') are \u000A compiled.  This is the behavior that was already supported with a \u000A <v>compile-flg</v> argument of <v>:all</v>; the change is that argument <v>t</v> now \u000A has this behavior as well (and <v>:all</v> is supported only for legacy \u000A purposes).  A new value for <v>compile-flg</v>, <v>:raw</v>, gives the behavior \u000A formerly produced by value <v>t</v>, namely where executable-counterpart \u000A functions are not compiled.  The above changes are irrelevant if compilation \u000A is suppressed; see <see topic=\"ACL2____COMPILATION\">compilation</see>.  Finally, if environment variable \u000A <v>ACL2_COMPILE_FLG</v> is set, then after converting to upper-case this \u000A environment variable's value of <v>&quot;T&quot;</v>, <v>&quot;NIL&quot;</v>, or <v>&quot;:RAW&quot;</v> \u000A will determine the value of the optional <v>compile-flg</v> argument to be \u000A <v>t</v>, <v>nil</v>, or <v>:raw</v>, respectively, when this argument is not \u000A explicitly supplied.</p> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> so that <v>:comp</v> argument now acts like \u000A <v>:comp!</v>, i.e., compiling a file that includes the file together with all \u000A executable counterpart (so-called ``*1*'') functions.  A new argument, \u000A <v>:comp-raw</v>, has the behavior that <v>:comp</v> had formerly, i.e., compiling \u000A the actual book only.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see></tt> is now computed in raw \u000A Lisp by calling <tt><see topic=\"COMMON-LISP____FLOOR\">floor</see></tt> on its arguments.  This change was suggested by \u000A Peter Dillinger, in order to avoid stack overflows such as reported by Daron \u000A Vroon.  A new book, <v>books/misc/misc2/misc.lisp</v>, contains a proof of \u000A equivalence of <tt><see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see></tt> and <tt><see topic=\"COMMON-LISP____FLOOR\">floor</see></tt>, and \u000A serves as a repository for other miscellaeous proofs, including those \u000A justifying ACL2 modifications such as this one.</p> \u000A \u000A <p>Enhanced <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> to break down results by useful \u000A vs.  useless rule applications.  In particular, this provides information \u000A about which rules were ever applied successfully, as requested by Bill \u000A Young.</p> \u000A \u000A <p>Added coverage of <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules to the <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> statistics.</p> \u000A \u000A <p>Fixed a bug that was causing a <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hint to fire \u000A on a subgoal of the goal to which it was attached, when the original \u000A application didn't change the clause.  Thanks to Dave Greve for pointing out \u000A this bug and providing a useful example.</p> \u000A \u000A <p>Fixed a bug in handling of computed <see topic=\"ACL2____HINTS\">hints</see> related to the \u000A <v>stable-under-simplificationp</v> parameter (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>). \u000A There were actually two bugs.  A minor but confusing bug was that the same \u000A goal was printed twice upon application of such a hint.  The major bug was \u000A that <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see> (as well as other ``top'' hints: <v>:by</v>, \u000A <v>:cases</v>, and <v>:clause-processor</v>) were not being applied properly. \u000A Thanks to Jared Davis for sending an example some time ago that showed the \u000A duplicate printing, and to Dave Greve for sending an example showing \u000A mis-application of <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> <see topic=\"ACL2____HINTS\">hints</see>.  Note that you \u000A may find that existing computed hints using the \u000A <v>stable-under-simplificationp</v> parameter no longer have the same behavior; \u000A see a comment about computed hints in <v>note-3-2-1</v>, ACL2 source file \u000A <v>ld.lisp</v>, for an example of how you might want to fix such computed \u000A hints.</p> \u000A \u000A <p>David Russinoff has contributed an updated version of \u000A <v>books/quadratic-reciprocity/</v> including minor modifications of the \u000A treatment of prime numbers and a proof that there exist infinitely many \u000A primes.  Thanks to David for contributing this work, and to Jose Luis \u000A Ruiz-Reina for posing the challenge.</p> \u000A \u000A <p>Reduced the sizes of some <see topic=\"ACL2____CERTIFICATE\">certificate</see> (<v>.cert</v>) files by relaxing \u000A the test that allows original <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> <see topic=\"ACL2____EVENTS\">events</see> to be placed there, \u000A rather than evaluating the import list term into an explicit list of \u000A symbols.</p> \u000A \u000A <p>Improved execution efficiency slightly for function <v>rcdp</v> in file \u000A <v>books/misc/records.lisp</v>, by using <tt><see topic=\"ACL2____MBE\">mbe</see></tt> to introduce a \u000A tail-recursive body.</p> \u000A \u000A <p>The executable script generated by <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> (and by the normal \u000A build process) now includes a time stamp as a comment.  Thanks to Jared Davis \u000A for suggesting this change in order to support his use of <v>omake</v>.  In the \u000A process, we also arranged that the startup banner for an executable created by \u000A <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> shows all of the build (save) times, not just the one for \u000A the original image.</p> \u000A \u000A <p>Sped up most redundant <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> <see topic=\"ACL2____EVENTS\">events</see> by avoiding evaluation \u000A and sorting of the imports list in the case of identical event forms.  And, \u000A for <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events that are not redundant, sped up their processing in \u000A Allegro CL (and perhaps other Lisps, but apparently not GCL) by using our own \u000A <v>import</v> function.</p> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> so that it refuses to associate a \u000A keyword with a different directory string than one it is already bound to. \u000A See <see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see> for how to remove the existing binding \u000A first.  Thanks to Robert Krug for pointing out that without this change, one \u000A can find it difficult to debug a failure due to rebinding a keyword with \u000A <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>.</p> \u000A \u000A <p>Added a new value for the <v>:do-not-induct</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>), \u000A <v>:otf-flg-override</v>, which causes ACL2 to ignore the <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> \u000A when considering whether to abort the proof because of a <v>:do-not-induct</v> \u000A hint.  Thanks to Daron Vroon for suggesting such an addition.</p> \u000A \u000A <p>Modified the printing of messages for entering and exiting raw mode (see \u000A <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), so that in particular they are inhibited during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or whenever <v>observation</v>s are inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>).  Thanks to Peter Dillinger for suggesting such a \u000A change.</p> \u000A \u000A <p>(For system hackers only.)  The handling of <see topic=\"ACL2____EVENTS\">events</see> of the form \u000A <v>(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> ...))</v> had a bug that was causing bindings to \u000A be evaluated twice.  Moreover, the use of system function <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt> is suspect in raw Lisp.  We have eliminated special \u000A treatment of <v>state-global-let*</v> by <v>progn!</v> in favor of a new keyword \u000A argument, <v>state-global-bindings</v>, that provides the intended \u000A functionality.  See <see topic=\"ACL2____PROGN_12\">progn!</see>.  Moreover, special handling that allowed \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms under <v>state-global-let*</v> has been removed; the \u000A desired effect can be obtained using (progn! :state-global-bindings ...). \u000A Thanks to Peter Dillinger for pointing out the above bug and collaborating on \u000A these changes.</p> \u000A \u000A <p>Incorporated backward-compatible enhancements to \u000A <v>books/misc/expander.lisp</v> from Daron Vroon (documented near the top of \u000A that file).</p> \u000A \u000A <p>The specification of <v>:backchain-limit-lst</v> had required that only a \u000A single (<v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, or <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>) \u000A rule be generated.  We have weakened this restriction to allow more than one \u000A rule provided that each rule has the same list of hypotheses.  For example, \u000A the rule class <v>(:rewrite :backchain-limit-lst 1)</v> is now legal for the \u000A corollary formula <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (f x) (<see topic=\"COMMON-LISP____AND\">and</see> (g x) (h x)))</v>, where this was not \u000A formerly the case.  Thanks to Dave Greve for bringing this issue to our \u000A attention.</p>"],

 "ACL2____NOTE-3-0-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the \u000A documentation has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Fixed a soundness bug, introduced in the previous release, due to a failure \u000A to disallow <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see> that set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> in a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> context.  Here is a proof of <v>nil</v> \u000A that exploits the bug.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____PROGRAM\">program</see>))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo ()\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure 17))\u000A   (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (foo))))\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A nil\u000A :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> foo (foo)) :use foo)))</code> \u000A \u000A <p>Fixed a bug in the alternatives to <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt>, which are the <tt><see topic=\"ACL2____EXIT\">exit</see></tt> and <tt><see topic=\"ACL2____QUIT\">quit</see></tt> commands.  Thanks to Jared Davis and Peter Dillinger \u000A for pointing this out right away.</p> \u000A \u000A <p>The definition of <tt><see topic=\"ACL2____LEN\">len</see></tt> has been highly optimized in raw Lisp. \u000A Thanks to Bob Boyer and Warren Hunt for suggesting such an improvement and \u000A providing a lot of help in coming up with the current implementation.</p> \u000A \u000A <p>The clause subsumption algorithms have been improved, both to improve \u000A efficiency during warnings for <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules and to punt when \u000A the subsumption computation for induction appears to be blowing up.  Thanks to \u000A Robert Krug for bringing this issue to our attention and supplying a useful \u000A example.</p> \u000A \u000A <p>A bug has been fixed that prevented <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> from working properly in \u000A OpenMCL and multi-threaded SBCL (actually, in any ACL2 image where feature \u000A <v>:acl2-mv-as-values</v> is present).  Thanks to Sol Swords for bringing this \u000A problem to our attention.</p> \u000A \u000A <p>A <see topic=\"ACL2____TYPE-SPEC\">type-spec</see> of the form <v>(satisfies pred)</v> carries the \u000A requirement that <v>pred</v> be a unary function symbol in the current ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see>; otherwise, it is illegal.  Thanks to Dave Greve for pointing out \u000A that Common Lisp has this requirement.</p> \u000A \u000A <p>Installed a fix provided by Gary Byers (for ACL2 source function \u000A <v>install-new-raw-prompt</v>), for OpenMCL, that fixes an issue exposed in some \u000A versions of OpenMCL when compiler optimization is off.</p> \u000A \u000A <p>Fixed a bug in contributed book <v>misc/untranslate-patterns.lisp</v> that \u000A was causing calls of <v>add-untranslate-pattern</v> to be rejected in <see topic=\"ACL2____BOOKS\">books</see>.  Thanks to Ray Richards for pointing out this bug and to Jared Davis \u000A for assisting in the fix.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> when keywords <v>:initially</v> and \u000A <v>:resizable</v> are both supplied.  In this case, the definition of the \u000A resizing function mistakenly failed to quote the <v>:initially</v> value, even \u000A though this value is not to be evaluated.  One could even get an error in this \u000A case, as in the following example supplied by Erik Reeber, whom we thank for \u000A bringing this bug to our attention:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> $test\u000A  (test-x :type (array t (5)) :initially (0) :resizable t))</code> \u000A \u000A <p>A new feature, <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, allows the setting of time \u000A limits during proofs.  This is <b>not</b> a general-purpose time-limit \u000A utility, nor is it guaranteed to implement a strict bound; it only attempts to \u000A limit time approximately during proofs.  Thanks to Pete Manolios and Daron \u000A Vroon, who made the most recent request for such a feature, and to Robert Krug \u000A for a helpful discussion.</p> \u000A \u000A <p>(GCL only) Fixed a bug in the procedure for building a profiling image. \u000A Thanks to Sol Swords for bringing this bug to our attention and to Eric Smith \u000A for bringing a subsequent problem to our attention.</p> \u000A \u000A <p>Handling of <see topic=\"ACL2____THEORIES\">theories</see> can now use significantly less time and space. \u000A A regression suite run took about 25% longer before this change than it did \u000A after making this change (and also the ones in the next two paragraphs). \u000A Thanks to Vernon Austel for bringing this issue to our attention and for \u000A supplying code, quite some time ago, that provided detailed, useful \u000A implementation suggestions.  Also thanks to the folks at Rockwell Collins, \u000A Inc. for pushing the limits of the existing implementation, thus encouraging \u000A this improvement.</p> \u000A \u000A <p>Fixed a performance bug in obtaining executable counterpart symbols.</p> \u000A \u000A <p>We now avoid certain computations made on behalf of warnings, when such \u000A warnings are disabled.</p> \u000A \u000A <p>We have relaxed the checks made when including an uncertified book, to \u000A match the checks made when including a certified book.  Thanks to Eric Smith \u000A for suggesting this change.</p> \u000A \u000A <p>Fixed a bug in <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> (see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>) that caused \u000A an error when printing the time summary.</p> \u000A \u000A <p>Made fixes to avoid potential hard Lisp errors caused by the use of \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions.  The fix was to use a ``safe mode,'' \u000A already in use to prevent such errors during macroexpansion; see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>.  However, such errors were possible during evaluation \u000A of macro <see topic=\"ACL2____GUARD\">guard</see>s, for example as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (foo 3)))\u000A  x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A  (mac x))</code> \u000A \u000A <p>A similar issue existed for calls of <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>, \u000A <tt><see topic=\"ACL2____TABLE\">table</see></tt>, <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, and <v>value-triple</v>, but has been fixed \u000A for all but <v>in-theory</v> and <v>make-event</v>, where technical issues have \u000A caused us to defer this change.</p> \u000A \u000A <p>Fixed a bug in <v>wet</v> that caused problems in OpenMCL, and perhaps other \u000A Lisp implementations, when the argument to <v>wet</v> calls, or depends on, \u000A certain built-ins including <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>, <tt><see topic=\"ACL2____MBE\">mbe</see></tt>, and \u000A <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>.  Thanks to David Rager for bringing this problem to our \u000A attention.</p> \u000A \u000A <p>The file <v>books/Makefile-generic</v> has been improved so that when book \u000A certification fails with `make', the failure message contains the book \u000A filename.</p> \u000A \u000A <p>Documentation has been written to explain how to avoid an expensive \u000A immediate rewrite of the result of applying a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rule.  See <see topic=\"ACL2____META\">meta</see>.  Thanks to Robert Krug for \u000A supplying this trick, and to Eric Smith and Dave Greve for useful \u000A discussions.</p> \u000A \u000A <p>(OpenMCL only) OpenMCL-based ACL2 image names formerly had extension \u000A <v>&quot;.dppccl&quot;</v>, which was correct only for some platforms (including 32-bit \u000A Darwin PPC).  That has been fixed, thanks to a suggestion from Gary Byers.</p> \u000A \u000A <p>It is now legal to attach both a <v>:use</v> and a <v>:cases</v> hint at the \u000A same goal.  Thanks to Eric Smith for (most recently) requesting this \u000A feature.</p> \u000A \u000A <p>It is now permissible to include the same symbol more than once in the \u000A imports list of a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> form (i.e., its second argument).  Also, the \u000A evaluation of <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> forms with long import lists now uses a \u000A reasonably efficient sorting routine to check for two different symbols with \u000A the same name (see also <v>books/misc/sort-symbols.lisp</v>).  If you currently \u000A call a function like <v>remove-duplicates-eql</v> for your imports list, as had \u000A been suggested by a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> error message, then you may experience some \u000A speed-up by removing that call.  Thanks to Eric Smith for helping to discover \u000A this issue through profiling.</p> \u000A \u000A <p>Made miscellaneous efficiency improvements not listed above (for example, \u000A following a suggestion of Eric Smith to avoid checking for so-called ``bad \u000A Lisp objects'' during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, which saved almost 3% in time on \u000A one large example).</p> \u000A \u000A <p>Modified the notion of ``untouchable'' to separate the notion of \u000A untouchable functions and macros from the notion of untouchable state global \u000A variables.  See <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>.  Thanks to Bob Boyer for sending an \u000A example, <v>(put-global 'ld-evisc-tuple t state)</v>, that suggested to us the \u000A need for more restrictive handling of untouchables.  In particular, many \u000A <v>ld</v> specials (see <see topic=\"ACL2____LD\">ld</see>) are now untouchable.  You may be able to work \u000A around this restriction by calling <tt><see topic=\"ACL2____LD\">ld</see></tt>; see for example the change to \u000A <v>books/misc/expander.lisp</v>.  But please contact the ACL2 implementors if \u000A this sort of workaround does not appear to be sufficient for your \u000A purposes.</p> \u000A \u000A <p>Fixed a bug in function <v>set-standard-oi</v> (see <see topic=\"ACL2____STANDARD-OI\">standard-oi</see>).</p> \u000A \u000A <p>Fixed a bug in the use of <tt><see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see></tt>.  The bad behavior was an \u000A improper use of the print-level and print-length components of the tuple \u000A (specifically, taking its <tt><see topic=\"COMMON-LISP____CADDR\">caddr</see></tt> and <tt><see topic=\"COMMON-LISP____CADDDR\">cadddr</see></tt> instead of taking \u000A its <tt><see topic=\"COMMON-LISP____CADR\">cadr</see></tt> and <tt><see topic=\"COMMON-LISP____CADDR\">caddr</see></tt>).  Thanks to Bob Boyer for bringing this \u000A bug to our attention.</p> \u000A \u000A <p>A new argument to the <v>compile-flg</v> argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, \u000A <v>:all</v>, causes creation of a file to be compiled in place of the given \u000A book, where that file contains not only a copy of the book (with <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms expanded) but also contains definitions of the so-called \u000A ``executable counterparts'' of the functions defined in the book.  Then, \u000A functions defined in the book will be run compiled when including the book, \u000A even for functions whose <see topic=\"ACL2____GUARD\">guard</see>s have not been verified, or are in \u000A <v>:program</v> mode and running in so-called ``safe mode'' (for example, during \u000A expansion of macros).  The default behavior, value <v>t</v> of <v>compile-flg</v>, \u000A is unchanged.  Moreover, a new <v>:comp!</v> argument of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A now compiles the executable counterparts when creating the book's compiled \u000A file, and unlike <v>:comp</v>, always deletes the old compiled file first so \u000A that one always gets a fresh compile.</p> \u000A \u000A <p>Now, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> gives a \"Guards\" warning only for <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions that are defined in the given book but have not had \u000A their guards verified.  Previously, it also warned about such functions that \u000A were defined in the certification world or in sub-books.</p> \u000A \u000A <p>A new command, <tt><see topic=\"ACL2____REDO-FLAT\">redo-flat</see></tt>, facilitates the debugging of failed \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> forms by evaluating preliminary forms in \u000A order to leave one at the point of failure.  See <see topic=\"ACL2____REDO-FLAT\">redo-flat</see>.  Thanks to \u000A Ray Richards and others for asking for such a utility, and to Sandip Ray for \u000A useful discussions.</p> \u000A \u000A <p>We have changed the automatic declaration of of function types (still done \u000A in GCL and OpenMCL only, for now).  Our motivation was to avoid the assumption \u000A that Common Lisp functions return one value when ACL2 says that they do; \u000A thanks to Bob Boyer for bringing this issue to our attention with the example \u000A of defining <v>(foo x y)</v> to be <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see> x y)</v>.  ACL2 was saying that \u000A <v>foo</v> returns a single value, but because <v>floor</v> returns two values in \u000A raw Lisp, so does <v>foo</v>.  Other changes to automatic declaration include \u000A comprehending <tt><see topic=\"ACL2____DEFUND\">defund</see></tt>, not just <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.</p> \u000A \u000A <p>A new function, <tt><see topic=\"ACL2____MOD-EXPT\">mod-expt</see></tt>, computes <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> base exp) m)</v>, \u000A and does so efficiently in some implementations (currently only in GCL 2.7.0, \u000A which is not yet released).  Thanks to Warren Hunt for suggesting such an \u000A addition.</p> \u000A \u000A <p>New functions <tt><see topic=\"ACL2____GETENV_42\">getenv$</see></tt> and <tt><see topic=\"ACL2____SETENV_42\">setenv$</see></tt> have been made available \u000A for reading and writing environment variables.  Thanks to Jun Sawada for \u000A requesting these utilities.</p> \u000A \u000A <p>The query utility <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> has been improved in several ways.  As \u000A before, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules are only printed if the argument is a symbol; \u000A but the information printed for them is now more appropriate.  The following \u000A are changes for the case that the argument is not a symbol, but rather, a \u000A term.  (1) Rules are displayed that have <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations other \u000A than <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.  (2) All matching <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules are \u000A displayed, where previously <v>:definition</v> rules were only shown if they \u000A were ``simple'' rules (sometimes known as ``abbreviations''); see <see topic=\"ACL2____SIMPLE\">simple</see>.  (3) The ``Equiv'' field is printed for terms, not just symbols.  (4) \u000A The substitution is shown that, when applied to the left-hand side of the \u000A rule, will yield the specified term.  Thanks to Eric Smith for suggesting \u000A these changes.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>;show-rewrites</v> has been improved to \u000A match the changes described above for <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>.  In particular, \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules that are not ``<see topic=\"ACL2____SIMPLE\">simple</see>'' are now \u000A displayed by the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>show-rewrites</v> (and <v>sr</v>) \u000A command, and the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>rewrite</v> command has been \u000A correspondingly modified to accept these <v>:definition</v> rules.</p> \u000A \u000A <p>Fixed `make' targets <v>copy-distribution</v>, <v>copy-workshops</v>, and \u000A <v>copy-nonstd</v> so that they should also work for non-developers.</p> \u000A \u000A <p>Fixed a bug that was causing <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> to display <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> \u000A hypotheses oddly in some cases, in particular <v>(<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP____LET\">let</see> ...))</v>.  (The \u000A problem was in the ``untranslate'' display of the internal form of \u000A <v>syntaxp</v> calls.)  Thanks to Robert Krug for bringing this problem to our \u000A attention.  We also removed the restriction on <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> that its \u000A argument could not be a variable, a constant, or (more interestingly) a <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> application (i.e., a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> expression).</p>"],

 "ACL2____NOTE-2-9-5":[["Release-notes"],"ACL2 Sources","ACL2","<p>Fixed a bug in <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> that was causing a hard error when \u000A attempting to report on a forced assumption.  Thanks to Jared Davis for \u000A pointing this out and sending an example that helped us to determine a \u000A fix.</p> \u000A \u000A <p>Added <tt><see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see></tt> to the set of legal <see topic=\"ACL2____EVENTS\">events</see> that \u000A can be placed in <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms and <see topic=\"ACL2____BOOKS\">books</see>.  Thanks to John \u000A Cowles for bringing this issue to our attention.</p> \u000A \u000A <p>Fixed a bug that broke <v>wet</v>.  Thanks to David Rager for bringing this \u000A bug to our attention.</p> \u000A \u000A <p>Guard verification now evaluates ground subexpressions (those with no free \u000A variables) when computing the guard conjecture for the body of a function. \u000A Thanks to Jared Davis for useful conversations leading to this change.  See \u000A <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>, in particular its ``Note on computation of guard \u000A conjectures and evaluation'' near the end of that topic, for more details.</p> \u000A \u000A <p>Added a warning when a <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> is redefined.  Thanks to \u000A Jared Davis for suggesting a warning in this case and providing an informative \u000A example.  Also, <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt>s are now maintained more completely, \u000A as they are checked at the end of every event except for events executed on \u000A behalf of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or the second pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p> \u000A \u000A <p>Fixed the handling of runic designators to match their specification (see \u000A <see topic=\"ACL2____THEORIES\">theories</see>), so that disabling the name of a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event <tt><see topic=\"ACL2____DISABLE\">disable</see></tt>s all rules generated for that event.</p> \u000A \u000A <p>(For those who do numerous builds using feature <v>:acl2-mv-as-values</v>, \u000A currently only OpenMCL and multi-threaded SBCL by default:) You can speed up \u000A builds by adding the following parameter to `make', under conditions described \u000A in <v>GNUmakefile</v>: <v>USE_ACL2_PROCLAIMS=:REUSE</v>.</p> \u000A \u000A <p>Arranged that traced functions (see <see topic=\"ACL2____TRACE_42\">trace$</see>) are automatically \u000A untraced when events are undone (for example see <see topic=\"ACL2____UBT\">ubt</see>), at least for \u000A most underlying Common Lisp implementations.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> now creates non-executable functions, which \u000A allows <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s to be used where they had previously been prohibited. \u000A More generally, the user now has control over <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms to be used \u000A by the underlying <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>'d function; see <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>.  Thanks to \u000A Sandip Ray for pointing out the need for such a modification.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____DEFINITION\">Definition</see></tt> rules are now treated, at least by default, as \u000A truly first-class definitions.  In particular, <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see> use \u000A the latest <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule by default.  You may specify \u000A <v>:install-body nil</v> to get the previous behavior of <v>:definition</v> rules; \u000A See <see topic=\"ACL2____DEFINITION\">definition</see>, and you may choose a previously-installed \u000A <v>:definition</v> rule to provide the current body; see <see topic=\"ACL2____SET-BODY\">set-body</see>.  Also \u000A see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for details of the <v>:install-body</v> field, and see \u000A <see topic=\"ACL2____HINTS\">hints</see> to see a new <v>:with</v> directive for controlling expansion.  The \u000A <v>:with</v> directive for <v>:expand</v> hints can even direct the use of a \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule for expansion!  Thanks to various people, including \u000A Sandip Ray and Rob Sumners, for discussions on the issue of the applicability \u000A of <v>:definition</v> rules for <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p><see topic=\"ACL2____CONSTRAINT\">Constraint</see>s for functional instantiation now use the original \u000A definition rather than a simplified (``normalized'') version of it.</p> \u000A \u000A <p>Fixed a bug that caused the prompt to stay the same when guard-checking is \u000A off (see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>) and raw-mode is changed (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>).</p> \u000A \u000A <p>Lemma names in directory <v>books/ordinals</v> have been changed by replacing \u000A <v>/\\</v> with <v>&amp;</v> and replacing <v>\\/</v> with <v>V</v>.  We made this change \u000A because backslash is an escape character and hence disappears unless it is \u000A itself escaped.</p> \u000A \u000A <p>Fixed <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output so that failed non-proof events do not cause \u000A the proof-tree to be re-printed.  Thus for example, if you have already \u000A advanced the checkpoint marker, it will not be reset by subequent failed \u000A non-proof events.  Thanks to Pete Manolios and Peter Dillinger for bringing \u000A this bug to our attention.</p> \u000A \u000A <p>Fixed a bug that was preventing the printing of <see topic=\"ACL2____STOBJ\">stobj</see> fields as \u000A constants instead of numbers in certain cases.  (Note that this bug only \u000A affected printing, not soundness.) Thanks to Eric Smith for bringing this \u000A problem to our attention and providing the following example (which now works \u000A fine).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld1 fld2)\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> update-nth))\u000A(<see topic=\"ACL2____DEFUND\">defund</see> run (st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (st))) ;adding this didn't seem to help..\u000A  st)\u000A\u000A;works great; *fld1* prints as  *fld1*\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> *fld1* 'abc st)\u000A            (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))))\u000A\u000A;*fld1* gets printed as 0, presumably because the call to run intervenes.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> *fld1* 'abc (run st))\u000A            (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))))</code> \u000A \u000A <p>The macro <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> now allows the use of macros defined within its \u000A bodies even when at the event level, as illustrated by the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-defun (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A         `(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ,@args))\u000A       (my-defun g (x) x))</code> \u000A \u000A <p>Thanks to Anna Slobodova for bringing this issue to our attention.  A \u000A related change is that all arguments of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> must now be embedded \u000A event forms (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), so use <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> instead \u000A if this is not the case.</p> \u000A \u000A <p>The change to <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> mentioned above also fixes a bug in handling \u000A <see topic=\"ACL2____LOCAL\">local</see> events inside a <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> that is inside an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or in a book.  For example, the following form formerly caused an \u000A error.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A (<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) x))\u000A        (<see topic=\"COMMON-LISP____DEFUN\">defun</see> abc (x) x)))</code> \u000A \u000A <p>We fixed two bugs in <v>:</v><tt><see topic=\"ACL2____PUFF\">puff</see></tt> and <v>:</v><tt><see topic=\"ACL2____PUFF_A2\">puff*</see></tt>.  The \u000A first, brought to our attention by Eric Smith (who we thank), caused a cryptic \u000A error message when puffing a command with no subsidiary stored events; try, \u000A for example, <v>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> 3))</v>.  The second was due to a \u000A failure to restore the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  Suppose for example that \u000A we have certified the book <v>foo.lisp</v>, which contains <v>(</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt><v>)</v> followed by some definitions and/or theorems.  Now suppose we \u000A start ACL2 and execute the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot;)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> test-thm\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x)\u000A  :rule-classes nil)</code> \u000A \u000A <p>If we now execute <v>:</v><tt><see topic=\"ACL2____PUFF\">puff</see></tt><v> 1</v>, ACL2 will roll back the world \u000A to before the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>; then ``puff'' the include-book, which will \u000A leave us in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode; and finally skip re-execution of the \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> because such <see topic=\"ACL2____EVENTS\">events</see> are skipped in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode.  The fix is to re-install the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> immediately \u000A after the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to its pre-<tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> value.</p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, provides something like macros that take \u000A state.  For example, one can use it to put tests into certified books, do \u000A proof search, and generate new function names.  Many examples appear in \u000A directory <v>books/make-event/</v>.  See <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.  Thanks to Bob Boyer \u000A and Jared Davis for useful feedback and to Warren Hunt, David Rager, and \u000A Sandip Ray for helpful discussions leading to some of the examples in \u000A directory <v>books/make-event/</v>.</p> \u000A \u000A <p>In support of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, which is described in the preceding \u000A paragraph, <v>certify-book</v> has a new keyword argument, <v>:save-expansion</v>, \u000A that controls whether the result of expanding <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms is \u000A written out to a file.  See <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>; and for a discussion of book \u000A expansion files, see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.</p> \u000A \u000A <p>We fixed a soundness bug that did not correctly detect <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A events.  For example, the following event was admitted.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see>\u000A  (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A   ()\u000A   (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____PROGRAM\">program</see>))) ; or, (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> :off summary (<see topic=\"ACL2____PROGRAM\">program</see>)))\u000A   (<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> t)\u000A   (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)))\u000A     (<see topic=\"COMMON-LISP____1_B2\">1+</see> (foo x)))))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> inconsistent\u000A   nil\u000A   :hints ((&quot;Goal&quot; :use foo))\u000A   :rule-classes nil))</code> \u000A \u000A <p>A new value for <see topic=\"ACL2____GUARD\">guard</see> checking, <v>:none</v>, is now allowed.  If you \u000A execute <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :none</v>, then no guard checking \u000A will take place (but raw Lisp code will not be executed in this case).  As a \u000A result, you should never see a guard violation, even for calls of \u000A <v>:</v><v>program</v> mode functions.  We thank Pete Manolios, who has long \u000A wanted this feature, and also Peter Dillinger, for asking for it.  New \u000A documentation explains the interaction between the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and the \u000A value supplied to <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>.  See <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see>, see <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-SCRIPT\">guard-evaluation-examples-script</see>, and see \u000A <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-LOG\">guard-evaluation-examples-log</see>.</p> \u000A \u000A <p>In the course of adding the <see topic=\"ACL2____GUARD\">guard</see>-checking value <v>:none</v> \u000A described in the paragraph above, we eliminated an optimization that \u000A eliminated guard checking for some recursive calls of <v>:</v><see topic=\"ACL2____LOGIC\">logic</see> mode \u000A mutually-recursive functions that have not had their guards verified.  But we \u000A doubt that this change will be noticed by any users!)</p> \u000A \u000A <p>The ACL2 hyper-card has been enhanced, thanks to David Rager, with a \u000A listing of ``Useful EMACS Commands'' to match comments in \u000A <v>emacs/emacs-acl2.el</v>.</p> \u000A \u000A <p>Users contributed books following the <v>Readme.lsp</v> methodology: \u000A <v>data-structures/memories</v> and <v>unicode</v> (Jared Davis), <v>proofstyles</v> \u000A (Sandip Ray and J Moore).</p> \u000A \u000A <p>Made some improvements to <v>books/Makefile-generic</v> (a file discussed \u000A elsewhere; see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>).  In particular, improved \u000A handling of <v>.acl2</v> files for <v>dependencies</v> target.</p> \u000A \u000A <p>(Only OpenMCL and, with feature <v>:acl2-mv-as-values</v>, GCL) Fixed a bug \u000A that was causing proclaiming to fail when definitions are submitted \u000A interactively.</p> \u000A \u000A <p>The default stack size has been increased for several lisps.</p> \u000A \u000A <p>(Very technical) A restriction has been weakened on the use of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____STOBJ\">stobj</see>s under a call of an ACL2 evaluator (<v>trans-eval</v> or \u000A <v>simple-translate-and-eval</v>).  Now, the error can only take place for \u000A <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> names that occur in the term being evaluated.  Thanks to Erik \u000A Reeber for bringing this issue to our attention.</p> \u000A \u000A <p>The notion of ``ancestor'' has been changed slightly.  This notion is used \u000A by extended metafunctions and <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see> and see <see topic=\"ACL2____BRR_04\">brr@</see>), and also with backchain limits \u000A (see <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> and see <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>).  Basically, \u000A each time a hypothesis is encountered during application of a <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rule, that hypothesis is pushed (after instantiating and negating) onto the \u000A current list of ancestors before it is rewritten.  However, hypotheses of the \u000A form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term)</v>, where <v>var</v> is free (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), had not been included in the ancestors (similarly for \u000A <v>(equiv var (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> term))</v> where <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation).  Now such ``binding hypotheses'' are included in a \u000A special way in ancestors data structures.  In particular, <v>(<see topic=\"COMMON-LISP____NULL\">null</see>\u000A (mfc-ancestors mfc))</v> will now be true if and only if the term being \u000A rewritten is part of the current goal as opposed to a hypothesis from a rule \u000A encountered during backchaining, even if that hypothesis is a binding \u000A hypothesis.  Thanks to Dave Greve for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>Termination and induction analysis now continue through both arguments of \u000A <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, not just the second.  (Normally, the gathering up of <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> tests stops at function calls; but it continued through the second \u000A argument of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, and now it will continue through both arguments.) \u000A Thanks to Sol Swords for discussion leading to this change.</p> \u000A \u000A <p>The ACL2 distribution is now kept on the http server rather than the ftp \u000A server (but the home page has not been moved).  Thanks to Robert Krug for \u000A letting us know that some ACL2 users have found it inconvenient to fetch ACL2 \u000A using ftp.</p> \u000A \u000A <p>The file <v>books/README.html</v> has been renamed to <v>books/Readme.html</v>, \u000A since some browsers don't show the former in the directory listing.</p>"],

 "ACL2____NOTE-2-8-NEW-FUNCTIONALITY":[["Note-2-8"],"ACL2 Sources","ACL2","<p>WARNING: You may find that <v>control-d</v> (in emacs, <v>control-c\u000A control-d</v>) can throw you completely out of Lisp where it had not formerly \u000A done so.</p> \u000A \u000A <p>(CLISP and Allegro CL only) ACL2 now starts up inside the ACL2 loop &mdash; \u000A that is, <v>(</v><tt><see topic=\"ACL2____LP\">lp</see></tt><v>)</v> is executed automatically &mdash; when built \u000A on CLISP or Allegro CL.  This was already the case for GCL and CMUCL, and it \u000A still is not true for LispWorks.  Thanks to Joe Corneli for bringing the CLISP \u000A command-line option <v>&quot;-i&quot;</v> to our attention, which led to this CLISP \u000A change and inspired reconsideration of how to do this for Allegro CL.</p> \u000A \u000A <p>Pete Manolios and Daron Vroon have changed the representation of ordinals \u000A in ACL2, defined algorithms for ordinal arithmetic, and created a library of \u000A theorems to reason about ordinal arithmetic.  We thank them for these nice \u000A contributions.  See <see topic=\"ACL2____NOTE-2-8-ORDINALS\">note-2-8-ordinals</see> for details, in particular, for \u000A how to preserve existing proofs that depend on the previous ordinal \u000A representation.</p> \u000A \u000A <p>Sometimes users create rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> that cause an \u000A infinite loop in the ACL2 rewriter.  This has lead to Lisp stack overflows and \u000A even segmentation faults.  Now, the depth of calls of functions in the ACL2 \u000A rewriter is limited, and under user control.  See <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>.</p> \u000A \u000A <p>Macros <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (``must be equal'') and <tt><see topic=\"ACL2____MBT\">mbt</see></tt> (``must be true'') \u000A have been introduced, which allow the user to attach fast executable \u000A definitions to (presumably slower) <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions.  Thanks \u000A to Vernon Austel for a key idea.  Also provided is a macro <tt><see topic=\"ACL2____DEFEXEC\">defexec</see></tt>, \u000A which employs <tt><see topic=\"ACL2____MBE\">mbe</see></tt> but enforces the requirement that the executable \u000A definition also terminates.  Thanks to Jose Luis Ruiz Reina for collaborating \u000A in the design and development of <tt><see topic=\"ACL2____DEFEXEC\">defexec</see></tt>, and for useful comments from \u000A a number of others as well in the development of <v>mbe</v> including Joe \u000A Hendrix and Rob Sumners.</p> \u000A \u000A <p>Definitions have been added for functions <tt><see topic=\"ACL2____RASSOC-EQ\">rassoc-eq</see></tt> and <tt><see topic=\"ACL2____RASSOC-EQUAL\">rassoc-equal</see></tt>, which are like <tt><see topic=\"COMMON-LISP____RASSOC\">rassoc</see></tt> but use different tests and have \u000A different guards.  (Compare <tt><see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see></tt> and <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>, which \u000A are in similar relation to <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt>.)</p> \u000A \u000A <p>The user can now control multiple matching for free variables in hypotheses \u000A for <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules, as has already been supported for \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules.  For \u000A <v>:forward-chaining</v> rules, ``free variables'' are those in the hypotheses \u000A not bound by a given trigger term.  As for <v>:rewrite</v> and <v>:linear</v> \u000A rules, free-variable matching may be limited to the first successful attempt \u000A by specifying <v>:match-free :once</v> with <v>:forward-chaining</v> in the \u000A <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt>, and <tt><see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see></tt> may be used to \u000A modify the behavior of an existing rule.  Thanks to Erik Reeber for most of \u000A the implementation of these new capabilities, as well as significant \u000A assistance with a corresponding new documentation topic (see <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES-FORWARD-CHAINING\">free-variables-examples-forward-chaining</see>).</p> \u000A \u000A <p>It is no longer necessary to specify <v>(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> nil)</v> in \u000A order to avoid errors when a rule with free variables in its hypotheses is \u000A missing the <v>:match-free</v> field.  (This was already true during book \u000A certification, but now it is the case in interactive sessions as well.)</p> \u000A \u000A <p>The form <v>(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>)</v> causes, at least for most Lisps, entry into \u000A the Lisp debugger whenever ACL2 causes an error.  See <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>. \u000A Thanks to John Erickson for providing encouragement to provide this \u000A feature.</p> \u000A \u000A <p>A new <tt><see topic=\"ACL2____TABLE\">table</see></tt> has been provided so that advanced users can override \u000A the built-in <v>untranslate</v> functionality.  See <see topic=\"ACL2____USER-DEFINED-FUNCTIONS-TABLE\">user-defined-functions-table</see>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____PSTACK\">pstack</see></tt> mechanism (formerly denoted <v>checkpoints</v>) has been \u000A improved.  The ``process [prover] stack,'' or pstack, is automatically printed \u000A when proofs abort.  Evaluation of function calls on explicit arguments during \u000A proofs is now tracked.  Actual parameters are shown with <v>(<see topic=\"ACL2____PSTACK\">pstack</see> t)</v> \u000A rather than formals.  Thanks to Bill Legato for suggesting the first two of \u000A these improvements and, in general, encouraging changes that make ACL2 easier \u000A to use.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event is now allowed to take an <v>:inline</v> argument, \u000A which can speed up execution.  Thanks to Rob Sumners for suggesting and \u000A implementing this new feature.</p> \u000A \u000A <p>Macro <tt><see topic=\"ACL2____ASSERT_42\">assert$</see></tt> has been added in order to make it easy to write \u000A assertions in one's code.  Semantically, <v>(<see topic=\"ACL2____ASSERT_42\">assert$</see> test form)</v> is the same \u000A as <v>form</v>, but it causes a hard error (using <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt>) if <v>test</v> \u000A evaluates to <v>nil</v>.</p> \u000A \u000A <p>Macro <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> no longer takes arguments for the gstack or <tt><see topic=\"ACL2____STATE\">state</see></tt>.  However, it now takes a keyword argument (which is optional), \u000A <v>:evisc-tuple</v>, that can be used to control how it prints terms.  In \u000A particular, <v>cw-gstack</v> abbreviates large terms by default, but \u000A <v>(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :evisc-tuple nil)</v> causes terms to be printed in full.  Thanks \u000A to Robert Krug and Eric Smith for requesting this improvement.</p> \u000A \u000A <p>The advanced user now has more control over the evisceration of terms.  See \u000A <see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see>, in particular the new paragraph on ``The printing of \u000A error messages and warnings.''</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event now has an additional (optional) keyword, \u000A <v>:dir</v>.  The value of <v>:dir</v> should be a keyword that is associated with \u000A an absolute directory pathname to be used in place of the current book \u000A directory (see <see topic=\"ACL2____CBD\">cbd</see>) for resolving the first argument of \u000A <v>include-book</v> to an absolute pathname.  At start-up, the only such keyword \u000A is <v>:system</v>, so that for example <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic/top&quot; :dir\u000A :system)</v> will include the book <v>&quot;arithmetic/top&quot;</v> under the \u000A <v>&quot;books/&quot;</v> directory of your ACL2 installation.  But you can associate \u000A ``projects'' with keywords using <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>, e.g., \u000A <v>(<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> :my-project &quot;/u/smith/project0/&quot;)</v>.  See <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> and also see <see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see> and see \u000A <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.  Note: You will probably not find <v>:dir :system</v> to be \u000A useful if the distributed books are not placed in the path of their original \u000A location, pointed to by <v>:dir :system</v>, which will often happen if the \u000A executable image is obtained from another site.  Also see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, \u000A in particular its ``soundness warning''.</p> \u000A \u000A <p>The printing of results in raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) may now be \u000A partially controlled by the user: see <see topic=\"ACL2____ADD-RAW-ARITY\">add-raw-arity</see>.  Also, newlines \u000A are printed when necessary before the value is printed.</p> \u000A \u000A <p>For those using Unix/Linux `make': A <v>cert.acl2</v> file can contain forms \u000A to be evaluated before an appropriate <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command is invoked \u000A automatically (not included in <v>cert.acl2</v>).</p> \u000A \u000A <p>Jared Davis has contributed a new set of books for ordered finite set \u000A theory to the standard distribution, <v>books/finite-set-theory/osets-0.81/</v>. \u000A See the <v>README</v> file in that directory.  Thanks, Jared.</p> \u000A \u000A <p>Robert Krug has contributed two related changes (thanks, Robert!) in \u000A support of stronger arithmetic reasoning.  First, one can now enable and \u000A disable nonlinear arithmetic with a <v>:nonlinearp</v> hint, which will override \u000A the default provided by <tt><see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see></tt> (initially, <v>nil</v>).  See \u000A <see topic=\"ACL2____HINTS\">hints</see>.  Second, <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see> can now have access to the \u000A <v>HISTORY</v>, <v>PSPV</v>, and <v>CTX</v> variables of the waterfall, which (for \u000A example) allows the writing of a hint which will enable nonlinear arithmetic \u000A on precisely those goals that are <v>stable-under-simplificationp</v>.  See \u000A <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.</p> \u000A \u000A <p>Robert Krug has contributed a new set of arithmetic books to the standard \u000A distribution, <v>books/arithmetic-3/</v>.  See the <v>README</v> file in that \u000A directory.  Thanks, Robert.</p>"],

 "ACL2____NOTE-2-7-PROOFS":[["Note-2-7"],"ACL2 Sources","ACL2","<p>An improvement in the linear arithmetic heuristics has been \u000A provided by Robert Krug.  For information about this change, search for the \u000A comment in <v>add-linear-lemma</v> (file <v>rewrite.lisp</v>) that begins as \u000A follows.</p> \u000A \u000A <code>; Previous to Version_2.7, we just went ahead and used the result of</code> \u000A \u000A <p>Thanks, Robert!  Also thanks to Eric Smith for providing a motivating \u000A example.</p> \u000A \u000A <p>The non-linear-arithmetic addition (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>) led \u000A to several small changes in the linear-arithmetic decision procedure (see \u000A <see topic=\"ACL2____LINEAR-ARITHMETIC\">linear-arithmetic</see>).  Two of these changes could affect existing \u000A proofs.</p> \u000A \u000A <blockquote> \u000A \u000A <p>First, when we are setting up the initial arithmetic database (which we \u000A call the ``pot-lst''), we have always scanned it to see if there were any \u000A pairs of inequalities from which we could derive a previously unknown \u000A equality.  In some cases we added this equality to the clause and in others we \u000A used it to rewrite the clause, substituting one side of the equality for the \u000A other throughout the clause.  Previously, the heuristics that we used to \u000A determine whether we performed the substitution differed from those used in \u000A several other places in the code.  This has now been regularized, and similar \u000A heuristics are now used throughout the code.</p> \u000A \u000A <p>The second change to the linear-arithmetic decision procedure is that we \u000A now explicitly add inequalities derived from type reasoning to the pot-lst. \u000A Previously, we performed cancellations against these inequalities without \u000A adding them to the pot-lst.  This change results in there being more \u000A inequalities in the pot-lst than before, and so more chances for there to be a \u000A pair of inequalities from which an equality can be derived.  In effect, \u000A certain simple consequences of the current goal (see <see topic=\"ACL2____TYPE-SET\">type-set</see>) may now \u000A be added as hypotheses of the goal or used to peform equality \u000A substitutions.</p> \u000A \u000A </blockquote> \u000A \u000A <p>A slight improvement has been made to the way certain rewrite rules are \u000A stored.  It was already the case that a rewrite rule rule whose conclusion \u000A <v>C</v> is not a call of a known equivalence relation (or <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, or <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>) is stored as <v>(<see topic=\"ACL2____IFF\">iff</see> C t)</v>, except that if ACL2 can \u000A determine (using its <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> mechanism) that <v>C</v> is Boolean, then \u000A the rule is stored as <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> C t)</v>.  The iprovement is that if <v>C</v> and \u000A <v>C'</v> are Boolean, then a rule stated as <tt>(iff C C')</tt> is stored as \u000A <tt>(equal C C')</tt>.  Thanks to Pete Manolios for providing an example that \u000A led us to consider this improvement.</p> \u000A \u000A <p>The heuristic use of equalities (fertilization) has been modified. \u000A Previously, ACL2 would sometimes substitute using an equality but keep the \u000A equality, and then undo the substitution by using the equality again.  Now, \u000A when ACL2 keeps an equality after using it, it puts the equality inside a call \u000A of <tt><see topic=\"ACL2____HIDE\">hide</see></tt>.  Descendents of that goal that are unchanged by \u000A simplification will have this call of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> removed so that the \u000A equality can once again contribute to the proof.  This change can cause some \u000A proofs to succeed that otherwise would fail.  In the unlikely event that a \u000A proof fails that formerly succeeded, the following hint on \"Goal\" may fix \u000A the problem (see <see topic=\"ACL2____HINTS\">hints</see>):</p> \u000A \u000A <code>:expand ((:free (x) (<see topic=\"ACL2____HIDE\">hide</see> x)))</code> \u000A \u000A <p>We have refined the heuristics employed when an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> form is assumed \u000A true or false.  Our previous attempt (see <see topic=\"ACL2____NOTE-2-6-PROOFS\">note-2-6-proofs</see> for the \u000A original announcement) was not as general as we had believed.  We have also \u000A improved some low-level code responsible for rewriting <v>IF</v> expressions. \u000A In earlier versions of ACL2, it was possible to have the truth or falsity of \u000A an <v>IF</v> expression explicitly recorded in the type-alist, and yet not use \u000A this information during rewriting.  This problem has been corrected.  Thanks \u000A to Robert Krug for noticing this problem and implementing the fix.</p> \u000A \u000A <p>We have sped up the rewriter in some cases where there are large \u000A collections of mutually-recursive functions (see <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>). \u000A (Implementation notes: technically, we have modified the way function \u000A <v>being-openedp</v> operates on the <v>fnstack</v>, and we have modified \u000A <v>*current-acl2-world-key-ordering*</v> as described in the essay above its \u000A definition.)</p> \u000A \u000A <p><see topic=\"ACL2____FORWARD-CHAINING\">Forward-chaining</see> is now done in the preprocessing phase of proof \u000A attempts (see the discussion of <v>:DO-NOT</v> &mdash; see <see topic=\"ACL2____HINTS\">hints</see>).  This \u000A is part of a technical change, made in support of translation of type \u000A declarations to <see topic=\"ACL2____GUARD\">guard</see>s (see <see topic=\"ACL2____NOTE-2-7-GUARDS\">note-2-7-guards</see>).  Previously, \u000A whenever ACL2 checked for <see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see>s, it then looked for a \u000A contradiction using <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> reasoning if it did not find a suitable \u000A built-in clause.  The change is to perform forward-chaining in such cases \u000A (i.e., when a built-in clause is not found).</p> \u000A \u000A <p>A couple of changes have been made in the generation of goals for <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>s.  Thanks to Eric Smith for bringing issues to our attention \u000A that led to these changes.  For one, <see topic=\"ACL2____GUARD\">guard</see>s are no longer relevant in \u000A such goal generation.  Formerly, the addition of a guard could make a proof \u000A fail that otherwise succeeded.  Secondly, contextual information is now always \u000A kept when it involves a constrained constant, i.e., a zero-ary function \u000A introduced in the signature of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.</p>"],

 "ACL2____NOTE-2-7-OTHER":[["Note-2-7"],"ACL2 Sources","ACL2","<p>Made several minor <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> improvements.  We are \u000A grateful to Eric Smith for suggesting (most of) these.</p> \u000A \u000A <p>Improved <v>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)</v> (see <see topic=\"ACL2____BDD\">bdd</see>) to give more useful feedback when \u000A there are ``leaf'' terms not known to be Boolean.</p> \u000A \u000A <p>Sped up processing of large mutual-recursion nests.  In one large example \u000A the speedup was roughly two orders of magnitude.</p> \u000A \u000A <p>Modified event printing so that if both <v>'prove</v> and <v>'event</v> are \u000A inhibited, then events are no longer printed on behalf of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, or <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>.  Thanks to Eric Smith \u000A for prompting consideration of such a change.</p> \u000A \u000A <p>The following technical change was made to support <v>with-error-trace</v> \u000A and <v>wet</v> (see <see topic=\"ACL2____NOTE-2-7-NEW-FUNCTIONALITY\">note-2-7-new-functionality</see>), but may be of interest \u000A to those who do low-level programming using the ACL2 logical <tt><see topic=\"ACL2____WORLD\">world</see></tt>. \u000A The <v>'unnormalized-body</v> property is now stored not only for functions \u000A defined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, but also for functions defined by the \u000A user in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  (<v>:Program</v> mode Functions built into \u000A ACL2 still have their <v>'unnormalized-body</v> property omitted, in order to \u000A save space.)</p> \u000A \u000A <p>The handling of ``invisible'' functions for purposes of controlling \u000A rewriting (see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>) has been moved to a new table; see <see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see>.  Macros that access and modify this table are called \u000A ``<v>...-invisible-fns-table</v>'' in place of their former names, \u000A ``<v>...-invisible-fns-alist</v>.''  This feature was formerly implemented in \u000A the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, which prevented a book from exporting lists \u000A of invisible functions intended to work with the <see topic=\"ACL2____REWRITE\">rewrite</see> rules \u000A developed in the book.  Thanks to Eric Smith and Rob Sumners for suggesting \u000A this change.  See <see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see> (formerly \u000A <v>set-invisible-fns-alist</v>), and also see <see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> and see \u000A <see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see>, which provides ways to incrementally add to and \u000A remove from this table, respectively.  The handling of printing binary \u000A function call nests using macros (See <see topic=\"ACL2____ADD-BINOP\">add-binop</see>) has also been moved \u000A out of the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> as suggested by Eric and Rob, but this \u000A feature didn't work anyhow (see <see topic=\"ACL2____NOTE-2-7-BUG-FIXES\">note-2-7-bug-fixes</see>).  Incidentally, the \u000A symbols <v>binop-table</v>, <tt><see topic=\"ACL2____ADD-BINOP\">add-binop</see></tt>, and <tt><see topic=\"ACL2____REMOVE-BINOP\">remove-binop</see></tt> have all \u000A been added to the list <v>*acl2-exports*</v> (see <see topic=\"ACL2____ACL2-USER\">ACL2-user</see>), <tt><see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see></tt> and <tt><see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see></tt> have been added to that \u000A list, and <v>set-invisible-fns-alist</v> has been replaced in that list by \u000A <tt><see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see></tt>.  Function <v>invisible-fns-alistp</v> is no \u000A longer defined and has been removed from <v>*acl2-exports*</v>.</p> \u000A \u000A <p>We now enforce the stated restriction on the pairings in \u000A <v>macro-aliases-table</v> (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), namely, that it \u000A associates names of macros with names of funcions (with respect to the current \u000A ACL2 logical <see topic=\"ACL2____WORLD\">world</see>).  We make a similar requirement on <tt><see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see></tt>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> event has been modified so that the default \u000A action is an error rather than a warning.  Thanks to Eric Smith for suggesting \u000A this change.  Also, the value returned upon successful execution of a <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> event is now the key.</p> \u000A \u000A <p>Proof output that reports destructor elimination no longer uses the word \u000A ``generalizing''.  This small change may help in browsing proof output, since \u000A now ``generaliz'' takes you to true uses of generalization.  Thanks to Matyas \u000A Sustik for suggesting such a change.</p> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> now prints an abbreviated controller-alist for \u000A <v>;</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules.  Formerly the output from <v>:pl</v> could be \u000A overwhelming when the supplied function was part of a large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nest.</p> \u000A \u000A <p>The defaults for keyword parameters of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> have changed. \u000A See <see topic=\"ACL2____NOTE-2-7-BUG-FIXES\">note-2-7-bug-fixes</see>, in particular, the discussion there of two \u000A modifications to <v>certify-book</v>.</p> \u000A \u000A <p>Technical changes have been made to <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> \u000A that should usually be invisible to users.  The next paragraph describes them \u000A in detail, only for competeness (i.e., that description can be ignored by most \u000A users).  But first, here is an example showing an effect on users.  The slow \u000A array warning was not there previously.  Notice that the warning only arises \u000A if the event form is changed.  The solution is to be sure that redundant \u000A <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> forms are syntactically identical.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                                '((:header :dimensions (5)\u000A                                           :maximum-length 15\u000A                                           :default uninitialized\u000A                                           :name demo)\u000A                                  (1 . one)\u000A                                  (0 . zero))))\u000A\u000ASummary\u000AForm:  ( DEFCONST *A* ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A *A*\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo *a* 0)\u000AZERO\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                                '((:header :dimensions (5)\u000A                                           :maximum-length 15\u000A                                           :default uninitialized\u000A                                           :name demo)\u000A                                  (1 . one)\u000A                                  (0 . zero))))\u000A\u000AThis event is redundant.  See :DOC redundant-events.\u000A\u000ASummary\u000AForm:  ( DEFCONST *A* ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A :REDUNDANT\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo *a* 0)\u000AZERO\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                                '((:header :dimensions (5)\u000A                                           :maximum-length 15\u000A                                           :default uninitialized\u000A                                           :name demo)\u000A                                  (0 . zero)\u000A                                  (1 . one))))\u000A\u000AThis event is redundant.  See :DOC redundant-events.\u000A\u000ASummary\u000AForm:  ( DEFCONST *A* ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A :REDUNDANT\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo *a* 0)\u000A\u000A**********************************************************\u000ASlow Array Access!  A call of AREF1 on an array named\u000ADEMO is being executed slowly.  See :DOC slow-array-warning\u000A**********************************************************\u000A\u000AZERO\u000AACL2 !&gt;</code> \u000A \u000A <p>As before, the von Neumann structure stored in the <v>'acl2-array</v> \u000A property of the array name contains the array list object in its <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>. \u000A However, previously it was the case that <v>compress1</v> and <v>compress2</v> did \u000A not update that <v>car</v> when its new value would be equal to its old value. \u000A This was done largely in support of some type-set tables defined using <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> in <v>type-set-b.lisp</v>.  The new versions of <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and \u000A <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> are simpler in that no such exception is made in the case of \u000A equal lists, although instead the entire compression process is \u000A short-circuited when the input array list object is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> to the <v>car</v> \u000A of the <v>'acl2-array</v> property.  This change was made because the equality \u000A test was causing a ``slow array access'' warning to be printed in rare cases \u000A during proofs, as described elswhere (see <see topic=\"ACL2____NOTE-2-7-BUG-FIXES\">note-2-7-bug-fixes</see>).</p> \u000A \u000A <p>We no longer distribute documentation specific to Lucid Emacs.  The Info \u000A documentation in directory <v>doc/EMACS/</v> works well both for Gnu Emacs and \u000A XEmacs.</p> \u000A \u000A <p>A little-advertised macro, <v>value</v>, has long been allowed for top-level \u000A forms in <see topic=\"ACL2____BOOKS\">books</see>; see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.  This has been replaced \u000A by a new macro, <v>value-triple</v>.  The two have the same semantics at the \u000A top-level of books, where <tt><see topic=\"ACL2____STATE\">state</see></tt> is ``live''.  However, \u000A <v>value-triple</v> should be used at the top-level of a book, while <v>value</v> \u000A should be used in function definitions (as before).  This change eliminates a \u000A warning put out by the Allegro Common Lisp compiler for top-level <v>value</v> \u000A forms in <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____NOTE-2-6-PROOFS":[["Note-2-6"],"ACL2 Sources","ACL2","<p>Certain optimizations are performed when converting terms to \u000A clausal form.  For example, <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 1)</v> is known to be <v>t</v>, <v>(<see topic=\"ACL2____HARD-ERROR\">HARD-ERROR</see>\u000A ctx str alist)</v> is known to be <v>nil</v>, and <v>(<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> n)</v> is known to \u000A imply <v>(<see topic=\"COMMON-LISP____RATIONALP\">RATIONALP</see> n)</v>.</p> \u000A \u000A <p>In earlier versions of ACL2, the conversion of a term to clausal form \u000A expanded <v>LAMBDA</v> applications.  That may no longer occur.  Some proofs may \u000A slow down (or fail) because your <v>LAMBDA</v>-expressions are not expanded away \u000A when you ``expected'' them to be.</p> \u000A \u000A <p>Robert Krug found a soundness bug in our linear arithmetic package.  The \u000A bug was caused by the derivation of an equation from two inequalities without \u000A taking adequate precautions to ensure that both sides of the inequalities were \u000A numeric.  Robert also kindly provided a fix which we adopted.  Thanks \u000A Robert!</p> \u000A \u000A <p>We fixed a bug that could prevent the application of a metatheorem.</p> \u000A \u000A <p>A bug has been fixed that had caused bogus forcing rounds (see <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>).  The bug could occur when the hypothesis of a rule was forced \u000A (see <see topic=\"ACL2____FORCE\">force</see>) before the prover decided to start over and prove the \u000A original goal by induction.  Thanks to Rob Sumners for drawing our attention \u000A to this problem.</p> \u000A \u000A <p>Some low-level fixes have been made that prevent certain infinite loops, \u000A based on reports by users.  We thank Yunja Choi, Matt Wilding, and Pete \u000A Manolios for reporting such problems.</p> \u000A \u000A <p>An obscure potential soundness hole has been fixed by redoing the way \u000A evaluation takes place in the ACL2 loop and during theorem proving.  We expect \u000A that users will see no difference based on this change.  (Those interested in \u000A the details can see the long comment ``Essay on Evaluation in ACL2'' in source \u000A file interface-raw.lisp.)</p> \u000A \u000A <p>A small change was made in computation for a heuristic that controls \u000A backchaining.  This will speed up proofs dramatically in a very few cases but \u000A should have a very small impact in general.</p> \u000A \u000A <p>The simplifier has been modified to avoid eliminating hypotheses of goals \u000A that can be established by contextual (specifically, type-set) reasoning \u000A alone.  We believe that this change will generally strengthen ACL2's reasoning \u000A engine, although on rare occasions a lemma that formerly was provable may \u000A require user assistance.  Thanks to Robert Krug for suggesting this change and \u000A providing its implementation.</p> \u000A \u000A <p>Case splits are now limited, by default.  This may allow some proof \u000A attempts to provide output where previously the prover would appear to ``go \u000A out to lunch.''  For a more complete discussion, including instructions for \u000A how users can control case splitting, see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>.</p> \u000A \u000A <p>A bug has been fixed in the handling of <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> \u000A rules by the <see topic=\"ACL2____BDD\">bdd</see> package.  Thanks to Rob Sumners for discovering this \u000A bug and supplying a helpful example.</p> \u000A \u000A <p>ACL2 may now use the built-in induction scheme for a function symbol even \u000A if that function symbol is disabled.  Formerly, if a function symbol was \u000A disabled then its induction scheme was only considered if an explicit \u000A induction hint was supplied, other than <v>:induct t</v>.</p> \u000A \u000A <p>We eliminated the rule-class <v>linear-alias</v>.  This rule class was seldom \u000A used and complicated the linear arithmetic decision procedure in ways that \u000A made it difficult to extend to handle some non-linear special cases.  The only \u000A use of the rule-class that we know of was in our own <v>nqthm</v> books, which \u000A were an attempt to provide an embedding of the Nqthm logic and theorem prover \u000A into ACL2.  But that facility was also practically never used, as far as we \u000A know.  So both <v>linear-alias</v> rules and the <v>nqthm</v> books have been \u000A eliminated.</p> \u000A \u000A <p>In earlier versions of ACL2, when the <v>IF</v>-form of <v>(<see topic=\"COMMON-LISP____AND\">AND</see> p q)</v> was \u000A assumed true -- as when rewriting the <v>alpha</v> expression in <v>(<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____AND\">AND</see> p\u000A q) alpha beta)</v> -- the assumption mechanism did not deduce that <v>p</v> and \u000A <v>q</v> are true, only that their conjunction, in its <v>IF</v>-form, is true. \u000A This has long been known as a deficiency in both ACL2 and the earlier Nqthm \u000A but it was tedious to do better when one considered the full range of \u000A <v>IF</v>-forms one might encounter in the test of another <v>IF</v>.  Rather than \u000A code all the cases, we just waited until clausification got rid of them. \u000A Robert Krug developed a pretty nice treatment of the general case and we added \u000A it in this version.  This also involved a surprising number of changes \u000A elsewhere in the system because the improved handling of assumptions caused \u000A the theorem prover often to ``erase'' hypotheses provided by <v>:use</v> hints \u000A because it could simplify them to <v>t</v>.  Thank you Robert!</p> \u000A \u000A <p>In response to a suggestion from Robert Krug, we added <v>mfc-ap</v> so that \u000A extended metafunctions can take advantage of linear arithmetic.  See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.</p> \u000A \u000A <p>There is less delay in printing goals.  In previous versions, a goal was \u000A not printed until its subgoals were created (or the goal was proved).  Now, \u000A the goal is printed essentially as soon as it is created.</p> \u000A \u000A <p>A small technical change has been made in the function <tt><see topic=\"ACL2____TERM-ORDER\">term-order</see></tt>, \u000A to give priority on the function symbol count over the weighting of constants. \u000A So for example, while previously the term <v>(f)</v> preceded the constant 2, \u000A that is no longer the case.  If this change is noticed at all, it will \u000A probably be noticed in how so-called <i>permutative</i> rewrite rules are \u000A applied; see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.  Thanks to Robert Krug for suggesting this \u000A improvement and providing part of the implemtation.</p>"],

 "ACL2____NOTE-2-6-NEW-FUNCTIONALITY":[["Note-2-6"],"ACL2 Sources","ACL2","<p>A fundamental change is the provision of the ``nu-rewriter'' for \u000A simplifying expressions composed of <v>NTH</v>, <v>UPDATE-NTH</v>, and \u000A <v>UPDATE-NTH-ARRAY</v> applications and <v>LET</v> expressions and other calls of \u000A non-recursive functions or <v>LAMBDA</v> expressions involving those symbols. \u000A The nu-rewriter applies the obvious rewrite rule for <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> i (<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> j v\u000A s))</v> and the analogous rule for <v>UPDATE-NTH-ARRAY</v>.  See <see topic=\"ACL2____NU-REWRITER\">nu-rewriter</see> The nu-rewriter can be enabled with <tt><see topic=\"ACL2____SET-NU-REWRITER-MODE\">set-nu-rewriter-mode</see></tt>.</p> \u000A \u000A <p>A new flag has been added to the <v>xargs</v> of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> permitting the \u000A declaration that the function is <v>non-executable</v>.  The usage is \u000A <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :non-executable t))</v> and the effect is that the function \u000A has no executable counterpart.  On the positive side: the function is \u000A permitted to use single-threaded object names and functions arbitrarily, as in \u000A theorems rather than as in executable definitions.  Such functions are not \u000A permitted to declare any names <v>:</v><tt><see topic=\"ACL2____STOBJ\">stobj</see></tt><v>s</v> but accessors, etc., \u000A may be used, just as in theorems.</p> \u000A \u000A <p>A new flag has been added to permit the system to abbreviate output by \u000A introducing <v>LET*</v> notation identifying common subterms.  The formula being \u000A proved is not affected; this flag changes its displayed form only.  See <see topic=\"ACL2____SET-LET_A2-ABSTRACTIONP\">set-let*-abstractionp</see>.</p> \u000A \u000A <p>A ``raw mode'' has been added, primarily for faster loading of \u000A applications.  see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>.</p> \u000A \u000A <p>Functions <tt><see topic=\"ACL2____ALPHORDER\">alphorder</see></tt> and <tt><see topic=\"ACL2____LEXORDER\">lexorder</see></tt> have been put in \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  <v>Lexorder</v> is now a total order ordering of the \u000A ACL2 universe, and theorems are included to that effect.  Thanks to Pete \u000A Manolios for suggesting the idea and providing events to use, and to Rob \u000A Sumners for assistance with some modifications.  See also the new book \u000A <v>books/misc/total-order</v> for an irreflexive total order.</p> \u000A \u000A <p>The ACL2 user can now make system calls to the host operating system.  See \u000A <see topic=\"ACL2____SYS-CALL\">sys-call</see> and see <see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see>.  Thanks to Rob Sumners for \u000A working out this idea with Pete Manolios and Robert Krug, who we also thank, \u000A and for working out the implementation with us.</p> \u000A \u000A <p>It is no longer required to use absolute <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>s in <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms that have been executed before a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>. \u000A Any relative pathname strings in such contexts will be expanded into absolute \u000A pathnames before they are saved in the <tt><see topic=\"ACL2____PORTCULLIS\">portcullis</see></tt> of the <tt><see topic=\"ACL2____CERTIFICATE\">certificate</see></tt> of the book being certified.</p> \u000A \u000A <p>ACL2 can now be built on top of Allegro Common Lisp 6.0, and also on \u000A Windows platforms on top of Allegro Common Lisp and GCL.  Thanks to Pete \u000A Manolios and Vinay K. Siddhavanahalli for their help with Windows.</p> \u000A \u000A <p>Rob Sumners has designed and provided an initial implementation for two \u000A improvements to <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> (also see <see topic=\"ACL2____STOBJ\">stobj</see>).  First, array fields \u000A can now be resized.  Resize and length functions are provided for array \u000A fields, which can be used to resize stobj array fields dynamically.  The \u000A recognizers for array fields have been simplified to accommodate this change, \u000A so that they only check that each element of the array field has the specified \u000A type.  Second, performance has been improved for stobjs with a large number of \u000A fields, by changing their Common Lisp implementation to store the fields in a \u000A simple vector instead of a list.</p> \u000A \u000A <p>Now <see topic=\"ACL2____STOBJ\">stobj</see>s may be bound locally; see <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>. \u000A Thanks to Rob Sumners, who encouraged us to implement this capability, was an \u000A early user of it, and participated usefully in discussions on its design.</p> \u000A \u000A <p>New functions <tt><see topic=\"ACL2____FMS_12\">fms!</see></tt>, <tt><see topic=\"ACL2____FMT_12\">fmt!</see></tt>, and <tt><see topic=\"ACL2____FMT1_12\">fmt1!</see></tt> are the same as \u000A their respective functions without the ``<v>!</v>,'' except that the ``<v>!</v>'' \u000A functions are guaranteed to print forms that can be read back in (at a slight \u000A readability cost).</p> \u000A \u000A <p>We added <tt><see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see></tt>, metafunctions which allow <tt><see topic=\"ACL2____STATE\">state</see></tt> and context sensitive rewriting to some extent.  We thank Robert Krug \u000A for pushing for and on this idea.</p> \u000A \u000A <p>The documentation has been improved.  In particular, a new documentation \u000A topic provides a gentle introduction to ACL2 <tt><see topic=\"ACL2____ARRAYS\">arrays</see></tt> &mdash; see <see topic=\"ACL2____ARRAYS-EXAMPLE\">arrays-example</see> &mdash; and additional documentation has been provided for \u000A getting started with proof trees in emacs &mdash; see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>New Makefile targets <v>fasl</v> and <v>o</v> have been added to the \u000A <v>books/</v> directory of the distribution.  For example, you might first \u000A certify books using an ACL2 built on top of GCL (which creates compiled files \u000A with suffix <v>o</v>).  Then, when standing in the <v>books/</v> directory, you \u000A might execute the command</p> \u000A \u000A <blockquote> \u000A \u000A <p>make fasl ACL2=my-allegro-acl2</p> \u000A \u000A </blockquote> \u000A \u000A <p>which will create compiled (<v>.fasl</v>) files for Allegro Common Lisp, \u000A assuming that <v>my-allegro-acl2</v> starts up an ACL2 built on that Common \u000A Lisp.</p> \u000A \u000A <p>The macro <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> now allows variables to be declared ignored.  See \u000A <see topic=\"COMMON-LISP____LET_A2\">let*</see> and see <see topic=\"COMMON-LISP____LET\">let</see>.</p> \u000A \u000A <p>The user may now control backchaining.  This feature was designed and \u000A primarily implemented by Robert Krug (though the authors of ACL2 are \u000A resposible for any errors); thanks, Robert!  See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p> \u000A \u000A <p>It is now possible to ``slow down'' the rate at which case splits are \u000A generated by the simplifier.  See <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>.</p> \u000A \u000A <p>Accesses to <see topic=\"ACL2____STOBJ\">stobj</see>s using <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> or <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt> are now \u000A displayed using symbolic constants instead of numeric indices.  For example, \u000A given the event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> foo a b :renaming ((b c)))</code> \u000A \u000A <p>then the term <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> 0 foo)</v> will be displayed (for example, during \u000A proofs) as <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> *a* foo)</v> while <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> 1 foo)</v> will be displayed as \u000A <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> *c* foo)</v>.  The <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event now correspondingly introduces \u000A a <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> event for each field accessor function, introducing a \u000A constant whose name is obtained from the accessor's name by prefixing and \u000A suffixin a ``<v>*</v>,'' as in the example above: accessor <v>a</v> generates \u000A <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* 0)</v> and accessor <v>c</v> generates <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *c* 1)</v>. See \u000A <see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see> for how to extend this feature for alternate names of \u000A <see topic=\"ACL2____STOBJ\">stobj</see>s.</p> \u000A \u000A <p>Computed hints have been improved.  It is now possible to detect within a \u000A computed hint whether the goal clause is stable under simplification; it is \u000A also possible for a computed hint to change the list of available hints.  See \u000A <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.</p> \u000A \u000A <p>It is now possible to provide ``default hints'' that are appended to the \u000A hints explicitly provided.  See <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>.</p> \u000A \u000A <p>Using computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) and default hints (see \u000A <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>) it is possible to implement a book that supports \u000A ``priority phased simplification.''  Using this book you can assign priorities \u000A to your rules and cause the theorem prover to simplify each goal maximally \u000A under all the rules of one priority before enabling rules of the next \u000A priority.  See <v>books/misc/priorities.lisp</v>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFABBREV\">defabbrev</see></tt> has been improved to allow <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A forms and documentation strings and to do more error-checking.  Thanks to Rob \u000A Sumners for designing this enhancement and providing the first implementation. \u000A See <see topic=\"ACL2____DEFABBREV\">defabbrev</see>.</p> \u000A \u000A <p>Further changes were made to support CMU Lisp.  Wolfhard Buss helped with \u000A these changes.</p> \u000A \u000A <p>A new table was added that is used when printing proof output, so that \u000A nests of right-associated calls of a binary function are replaced by \u000A corresponding macro calls, as has been the case for <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> and \u000A <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt>, <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> and <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>, and so on.  See <see topic=\"ACL2____ADD-BINOP\">add-binop</see>.</p> \u000A \u000A <p>Operators <tt><see topic=\"COMMON-LISP____LOGAND\">logand</see></tt>, <tt><see topic=\"COMMON-LISP____LOGIOR\">logior</see></tt>, <tt><see topic=\"COMMON-LISP____LOGXOR\">logxor</see></tt>, and <tt><see topic=\"COMMON-LISP____LOGEQV\">logeqv</see></tt> are now macros (formerly, they were functions) that call corresponding \u000A binary functions (e.g., <v>binary-logand</v>) defined in source file \u000A <v>&quot;axioms.lisp&quot;</v>.  Thanks to Rob Sumners for this enhancement.  Proof \u000A output will however continue to show calls of <tt><see topic=\"COMMON-LISP____LOGAND\">logand</see></tt>, <tt><see topic=\"COMMON-LISP____LOGIOR\">logior</see></tt>, \u000A <tt><see topic=\"COMMON-LISP____LOGXOR\">logxor</see></tt>, and <tt><see topic=\"COMMON-LISP____LOGEQV\">logeqv</see></tt>.</p> \u000A \u000A <p>Function <v>(</v><tt><see topic=\"ACL2____ALLOCATE-FIXNUM-RANGE\">allocate-fixnum-range</see></tt><v> fixnum-lo fixnum-hi)</v> sets \u000A aside more \"permanent\" fixnums in GCL.</p> \u000A \u000A <p>ACL2 now runs under <v>CLISP</v>.  Thanks to Wolfhard Buss and Sam Steingold \u000A for their assistance with the port.</p> \u000A \u000A <p>Michael ``Bogo'' Bogomolny has created a search engine, accessible from the \u000A ACL2 home page.  For that purpose he modified the HTML translator to create \u000A one file per topic (a good idea in any case).  Thanks, Bogo!</p> \u000A \u000A <p>An emacs file of potential (but optional) use for ACL2 users may be found \u000A in <v>emacs/emacs-acl2.el</v>.  In particular, this file supports the use of \u000A proof trees (see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>).</p> \u000A \u000A <p>Some <see topic=\"ACL2____BOOKS\">books</see> have been added or modified.  In particular, Robert Krug \u000A has contributed <v>books/arithmetic-2/</v>, which provides an alternative to the \u000A existing collection of books about arithmetic, <v>books/arithmetic/</v>.  For a \u000A discussion of the distributed books see the link to <v>README.html</v> in the \u000A installation instructions.</p>"],

 "ACL2____NOTE-2-5":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important Changes:</p> \u000A \u000A <p>Concurrent with the release of ACL2 Version 2.5 is the publication of two \u000A books about ACL2.  See the ``Books and Papers about ACL2 and Its \u000A Applications'' on the ACL2 Home Page.</p> \u000A \u000A <p>The <v>books</v> subdirectory now contains many new certifiable books, \u000A including solutions to the exercises in the two published books and full \u000A scripts for the case studies.  See <v>books/README.html</v>.</p> \u000A \u000A <p>Improved Unix <v>Makefile</v> support for book certification has also been \u000A written.  See <v>books/README.html</v>.</p> \u000A \u000A <p>The list of symbols in <v>*acl2-exports*</v> has been considerably expanded. \u000A If you have packages built by importing <v>*acl2-exports*</v> you might want to \u000A look carefully at the new value of that constant.  The new value includes all \u000A <v>:logic</v> mode functions as of Version 2.5, as well as all documented macros \u000A and all built-in theorem names.</p> \u000A \u000A <p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> were modified to have some \u000A additional keyword arguments.  It is possible to certify a book containing \u000A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> and/or <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> events and get warning messages or \u000A errors signaled, according to the settings of these new flags.  In addition, \u000A it is possible to specify in <v>include-book</v> whether the book must be \u000A certified (under penalty of error if not).  The default values of these new \u000A arguments cause warnings to be printed rather than errors signaled.</p> \u000A \u000A <p>The above change involved altering the form of certificate files.  When \u000A books certified under previous versions are included, more warnings will be \u000A generated because these books are considered possibly to contain <v>defaxiom</v> \u000A and/or <v>skip-proofs</v> events.</p> \u000A \u000A <p>We anticipate further changes to this aspect of books and consider the \u000A current mechanisms (for controlling whether warnings or errors are signaled) \u000A just a prototype.  See also the discussion below of ``soundness related'' \u000A warnings.  Your suggestions are welcome.</p> \u000A \u000A <p>A discrepancy between ACL2 and Common Lisp was fixed, having to do with \u000A <v>declare ignore</v>.  In past versions of ACL2, a formal parameter of a \u000A <v>defun</v> was considered ignored if it was not used in the body, the guard or \u000A the measure of the <v>defun</v>.  That meant that a variable used only in the \u000A guard could not be declared ignored in ACL2; but some Common Lisp compilers \u000A would complain because the variable was not used in the body.  Now, ACL2 \u000A considers a variable ignored if it is not used in the body.</p> \u000A \u000A <p>ACL2 can now be built in releases 5.0 and later of Allegro Common Lisp. \u000A (Other releases of Allegro Common Lisp and of other lisps continue to be \u000A supported as well.)  This includes Allegro Common Lisp running on Windows 98 \u000A platforms.  John Cowles helped us do some testing and answered questions for \u000A us.  Thanks John!</p> \u000A \u000A <p>We incorporated Ruben Gamboa's changes to allow the building of a variant, \u000A ACL2(r), of ACL2, in which the user can reason about the real numbers using \u000A non-standard analysis.  See <see topic=\"COMMON-LISP____REAL\">real</see>.  Note that ACL2(r) and ACL2 have \u000A different underlying theories, and books certified in one system may not be \u000A included in the other.  For backward compatibility and to ensure a smooth \u000A transition, ACL2 is built by default, not ACL2(r).  This is a compile-time \u000A switch; see the makefile for instructions.  There should be no changes to ACL2 \u000A resulting from the capability of building ACL2(r) from the same sources.  Also \u000A see <see topic=\"ACL2____ACKNOWLEDGMENTS\">acknowledgments</see> for more on the history of ACL2(r).</p> \u000A \u000A <p>A large number of bugs (some affecting soundness) were fixed, and many \u000A small new features were added.  See below.</p> \u000A \u000A <p>Less Important Changes:</p> \u000A \u000A <p>Some warnings are now considered ``soundness related,'' namely, those that \u000A advise you that an uncertified book has been included or that a book \u000A containing <v>DEFAXIOM</v> or <v>SKIP-PROOFS</v> events.  (Technically, \u000A <v>DEFAXIOM</v>s do not imperil soundness in the proof- theoretic sense, though \u000A they may imperil the validity of theorems.  But you sould know when a book has \u000A added an axiom to your logic!)  In previous versions of ACL2, all warnings \u000A were inhibited if the token <v>warning</v> was included in the argument to \u000A <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>.  Now, soundness related warnings are printed \u000A even if <v>warning</v>s have been inhibited.  To inhibit all warnings, supply \u000A the token <v>warning!</v> to <v>set-inhibit-output-lst</v>.</p> \u000A \u000A <p>Several bugs in <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> were fixed, relating to the possibility \u000A that some of the subfunctions introduced by the <v>defstobj</v> were already \u000A defined.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____PUFF\">Puff</see></tt> no longer tries to expand <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> events. \u000A Previously, the attempt would cause a hard error.</p> \u000A \u000A <p>A soundness bug was fixed.  The bug might have been exercised if you had an \u000A alternative definition (implies hyps (equiv (fn ...) body)) in which equiv is \u000A an equivalence relation other than EQUAL.  In this case, calls of fn might \u000A have been expanded to body in places that were not equiv-hittable.</p> \u000A \u000A <p>An obscure soundness bug was fixed.  The bug was exercised only if you had \u000A a metafunction with a computed hypothesis (i.e., a ``meta hypothesis \u000A function''), the hypothesis contained a free variable, i.e., a variable not \u000A involved in the term being rewritten, and the free variable occurred in the \u000A output of the metafunction.  The possibility of this bug was brought to our \u000A attention by Robert Krug.</p> \u000A \u000A <p>We fixed a bug in the handling of <v>hide</v> related to the question of \u000A whether a variable symbol occurs in a term.  The old code did not find the \u000A variable and could cause the system to throw away a hypothesis about it on the \u000A grounds that it was never mentioned.  Rob Sumners helped discover this \u000A problem.</p> \u000A \u000A <p>The handling of <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt> rules was generalized, permitting \u000A arbitrary known equivalence relations instead of merely <v>equal</v> in the \u000A concluding equality.</p> \u000A \u000A <p>The printing of runes (rule names; see <see topic=\"ACL2____RUNE\">rune</see>) used has been made \u000A \"deterministic,\" both in proof output and in proof attempt summaries, by \u000A sorting the runes before printing.</p> \u000A \u000A <p>The handling of free variables has been improved for hypotheses such as \u000A <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 X)</v>, and more generally, any hypotheses involving a comparison with \u000A <v>0</v> (even for example <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> X 1)</v> where <v>X</v> is known to be an integer, \u000A which is handled as <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X 0)</v>).  Thanks to Robert Krug for bringing \u000A relevant examples to our attention.</p> \u000A \u000A <p>A new value, <v>:comp</v>, has been implemented for the \u000A <v>:load-compiled-file</v> keyword of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  If this value is \u000A supplied, then a compiled file will always be loaded, even if that requires \u000A creating the compiled file first.</p> \u000A \u000A <p>The event <v>include-book</v> now generates a warning when a compiled file is \u000A expected but not found (see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>).  Formerly, it only did so \u000A when executed at the top level; it failed to generate the warning when \u000A executed on behalf of a surrounding <v>include-book</v> command.</p> \u000A \u000A <p>Certain redefinition warnings generated by Allegro Common Lisp have been \u000A eliminated.</p> \u000A \u000A <p>A new key has been implemented for the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, \u000A <v>:bogus-mutual-recursion-ok</v>, set with <v>:</v><tt><see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see></tt>.  Thanks to David Russinoff for pointing out \u000A the utility of such a key.</p> \u000A \u000A <p>A bug was fixed in <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> that prevented its generated events \u000A from being accepted when guard verification is being performed.  Thanks to \u000A Bill Young for bringing this problem to our attention.  A second bug was \u000A brought to our attention by Pete Manolios, which was causing certain <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> events to be rejected.  That problem has been fixed, and an \u000A \"Infected\" warning has also been eliminated.</p> \u000A \u000A <p>The command <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> now works with Allegro Common Lisp.</p> \u000A \u000A <p>A low-level bug was fixed that could, for example, cause an error such as \u000A \"Error: Expected 5 args but received 4 args\" when interrupting a <v>local</v> \u000A event.</p> \u000A \u000A <p>A bug has been fixed in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> related to definition \u000A expansion.  Thanks to Pete Manolios for bringing this to our attention with a \u000A simple example.</p> \u000A \u000A <p>A bug has been fixed related to the <v>:</v><see topic=\"ACL2____BDD\">bdd</see> hint in the presence \u000A of <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations.  Thanks to Pete Manolios for bringing this to \u000A our attention with a simple example.</p> \u000A \u000A <p>The functions <tt><see topic=\"COMMON-LISP____POSITION\">position</see></tt> and <tt><see topic=\"ACL2____POSITION-EQUAL\">position-equal</see></tt> formerly required \u000A the second argument to be a true list.  In accordance with Common Lisp, we now \u000A also allow the second argument to be a string.  This could cause earlier \u000A proofs about these functions to fail unless <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> is known to \u000A hold where necessary.</p> \u000A \u000A <p>Robert Krug wrote a patch, which has been incorporated, to prevent certain \u000A infinite loops that can arise in linear arithmetic.  Thanks, Robert!</p> \u000A \u000A <p>The macro <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> no longer requires the bound variables to be \u000A distinct.</p> \u000A \u000A <p>An obscure bug was fixed related to congruence rules.  The bug would \u000A sometimes cause ACL2 to behave as though no rules (other than equality) were \u000A available for some argument positions.  Thanks to Pete Manolios for bringing \u000A this bug to our attention.</p> \u000A \u000A <p>Documentation topics have been added for <tt><see topic=\"ACL2____HARD-ERROR\">hard-error</see></tt> and <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, and the documentation for <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt> has been improved.  Thanks \u000A to Rob Sumners for a useful suggestion in the examples in documentation for \u000A <v>prog2$</v> and a fix in documentation for <tt><see topic=\"COMMON-LISP____SUBLIS\">sublis</see></tt>.</p> \u000A \u000A <p>The event form <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> was made more secure, in that it can \u000A now catch attempts to write a book to disk during its certification.  Thanks \u000A to Rob Sumners for pointing out the insecurity of the existing mechanism.</p> \u000A \u000A <p>A Y2K problem was fixed with our applicative handling of dates.</p> \u000A \u000A <p>Accessors and updaters for <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s have been made more efficient \u000A when the underlying lisp is Allegro Common Lisp, by the use of appropriate \u000A simple array declarations.</p> \u000A \u000A <p>A raw Lisp break had been possible when a certified book that had no guard \u000A verification was included in a session after <v>(</v><tt><see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see></tt><v> 2)</v>.  This has been fixed.</p> \u000A \u000A <p>The keyword command <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> can now be used to compile only raw \u000A Lisp functions, excluding executable counterparts, by supplying the argument \u000A <v>:raw</v>.</p> \u000A \u000A <p>Rewrite rule <v>nth-of-character-listp</v> was removed from source file \u000A axioms.lisp since it is essentially subsumed by <v>characterp-nth</v>.</p> \u000A \u000A <p>Printing has been sped up.  In one example the improvement was over 50% in \u000A both Allegro and GCL.</p> \u000A \u000A <p>We now allow printing in a \"downcase\" mode, where symbols are printed in \u000A lower case.  All printing functions except <v>print-object$</v> now print \u000A characters in lower case for a symbol when the ACL2 state global variable \u000A <v>print-case</v> has value <v>:downcase</v> and vertical bars are not necessary \u000A for printing that symbol.  See <see topic=\"ACL2____IO\">io</see> for a discussion of the macros \u000A <v>acl2-print-case</v> and <v>set-acl2-print-case</v>.  The default printing \u000A remains unchanged, i.e., symbols are printed in upper case when vertical bars \u000A are not required.</p> \u000A \u000A <p>A low-level printing function (<v>prin1$</v>) was modified so that it is not \u000A sensitive to various Common Lisp globals related to printing.  So for example, \u000A the function <tt><see topic=\"ACL2____FMT\">fmt</see></tt> is no longer sensitive to the value of Common Lisp \u000A global <v>*print-case*</v>.  (The preceding paragraph explains how to control \u000A the case for printing in ACL2.)</p> \u000A \u000A <p>The definition of <tt><see topic=\"ACL2____ARRAY1P\">array1p</see></tt> was fixed so that the \u000A <v>:maximum-length</v> of an array must be strictly greater than the number \u000A specified in the <v>:dimensions</v> field; they may no longer be equal.  This \u000A was always the intention; the documentation (see <see topic=\"ACL2____ARRAYS\">arrays</see>) has remained \u000A unchanged.  The corresponding change was also made to <tt><see topic=\"ACL2____ARRAY2P\">array2p</see></tt>. \u000A Allegro Common Lisp formerly caused an error when <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> was called \u000A on an array where the numbers above were equal; now, we get a guard violation \u000A instead, which is appropriate.</p> \u000A \u000A <p>In the context of theories, a name now represents not just the \u000A corresponding <v>:definition</v> <see topic=\"ACL2____RUNE\">rune</see>, as it has done in earlier versions \u000A of ACL2, but also the corresponding <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> rune.  See <see topic=\"ACL2____THEORIES\">theories</see> for a discussion of runic designators.  Most users will rarely, if \u000A ever, notice this change.  One situation where this change will make a \u000A difference is after executing <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> 'foo))</v> followed \u000A by <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> bar))</v>, where function <v>bar</v> is introduced after \u000A event <v>foo</v>, and <v>bar</v> is recursively defined.  The latter <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> form now enables the rune <v>(:induction bar)</v>, which implies that \u000A the prover can use the induction scheme stored at definition time for \u000A <v>bar</v>.  Formerly, the rune <v>(:induction bar)</v> was not enabled by \u000A <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> bar))</v>, and hence the induction scheme for <v>bar</v> was \u000A ignored even when explicit <v>:induct</v> hints were supplied.</p> \u000A \u000A <p>You may now supply <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword pair <v>:normalize nil</v> in order \u000A to prevent certain definitions from ``hanging'' when there are many \u000A <v>if</v>-subexpressions.  see <see topic=\"COMMON-LISP____DEFUN\">defun</see>.</p> \u000A \u000A <p>We now translate type declarations of <v>real</v> into guards, as we have \u000A already done for other types such as <v>rational</v>.  For example, <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A (<see topic=\"COMMON-LISP____TYPE\">type</see> real x))</v> generates the <see topic=\"ACL2____GUARD\">guard</see> <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</v>.  See <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>.</p> \u000A \u000A <p>The theorem prover now behaves reasonably under the combination of \u000A specifying a value of <v>t</v> both for <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> and for a hint \u000A <v>:do-not-induct</v>.  Previously, it aborted the first time it would have \u000A otherwise pushed a goal for induction, but now, it will continue and wait \u000A until all induction subgoals have been pushed before it aborts.</p> \u000A \u000A <p>We changed slightly the definition of <tt><see topic=\"COMMON-LISP____ROUND\">round</see></tt>.  However, we believe \u000A that the new definition is equivalent to the old.</p> \u000A \u000A <p>The definition of Common Lisp function <tt><see topic=\"COMMON-LISP____SUBSTITUTE\">substitute</see></tt> has been \u000A added.</p> \u000A \u000A <p>The following changes have been made in the use of file names within ACL2. \u000A We thank Warren Hunt and John Cowles for running some tests of these changes \u000A on Macintosh and Windows 98 platforms (respectively).</p> \u000A \u000A <blockquote> \u000A \u000A <p>(1) Names of directories and files now use a syntax like that used for Unix \u000A (trademark of AT&amp;T), where directories are separated using the ``<v>/</v>'' \u000A character even when the operating system is not Unix or Linux.  See <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>.  ACL2 also continues to support its notion of <i>structured \u000A pathnames</i> from Version 2.4 and before, but might not do so in future \u000A releases and hence no longer documents such syntax.</p> \u000A \u000A <p>(2) The command <v>:</v><tt><see topic=\"ACL2____SET-CBD\">set-cbd</see></tt> may now take a relative pathname as \u000A an argument.</p> \u000A \u000A <p>(3) When the macro <tt><see topic=\"ACL2____LD\">ld</see></tt> is given a file name as a value for <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, then if that file name is a relative pathname it refers to the \u000A result of prepending the connected book directory (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>, see \u000A <see topic=\"ACL2____CBD\">cbd</see>, and see <see topic=\"ACL2____SET-CBD\">set-cbd</see>) in order to obtain an absolute pathname. \u000A Simiarly for the <v>ld</v> specials <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>It is no longer necessary to issue <v>:</v><tt><see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see></tt><v> t</v> if you \u000A include a <see topic=\"ACL2____STOBJ\">stobj</see> declaration for <tt><see topic=\"ACL2____STATE\">state</see></tt>, for example:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))</code> \u000A \u000A <p>See <see topic=\"ACL2____DECLARE-STOBJS\">declare-stobjs</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> has been cleaned up a bit, including the \u000A documentation and the capability (once again) to define pc-macro commands (see \u000A <see topic=\"ACL2____DEFINE-PC-MACRO\">define-pc-macro</see>) and proof-checker meta commands (see <see topic=\"ACL2____DEFINE-PC-META\">define-pc-meta</see>).</p> \u000A \u000A <p>Recall that events generate summaries that include a line beginning with \u000A ``<v>Warnings:</v>'', which is followed (on the same line) by zero or more brief \u000A strings that summarize the warnings generated by that event.  Formerly, this \u000A warnings summary for an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event did \u000A not include the summary strings for warnings generated by subsidiary events. \u000A This has been fixed.</p> \u000A \u000A <p>Macro <tt><see topic=\"ACL2____CW\">cw</see></tt> has been documented and now expands to a call of a \u000A <v>;</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function.  See <see topic=\"ACL2____CW\">cw</see> for a way to print to the \u000A screen without having to involve the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thanks to Rob \u000A Sumners for suggesting that we document this useful utility.</p> \u000A \u000A <p>Functions <v>duplicates</v>, <v>add-to-set-equal</v>, <v>intersection-eq</v>, \u000A <v>evens</v>, and <v>odds</v> are now <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>First we give two examples of rewriting.  Then we give a rather \u000A detailed description.  We recommend you read the description, even if you \u000A understand the two examples, just so that you learn our terminology.</p> \u000A \u000A <p><b>Example 1</b>: Suppose your goal conjecture is:</p> \u000A \u000A <code><i>Goal Conjecture</i>: \u000A(implies (and (endp z) \u000A              (true-listp z)) \u000A         (equal (rev (rev z)) z)) \u000A</code> \u000A \u000A <p>Then you can use the following theorem (actually the definitional axiom \u000A introduced by the <v>defun</v> of <v>endp</v>):</p> \u000A \u000A <code><i>Definitional Axiom</i>: endp \u000A(equal (endp x) \u000A       (not (consp x))). \u000A</code> \u000A \u000A <p>to <i>rewrite</i> the <i>Goal Conjecture</i> to</p> \u000A \u000A <code><i>Rewritten Goal Conjecture</i>: \u000A(implies (and (not (consp z)) \u000A              (true-listp z)) \u000A         (equal (rev (rev z)) z)) \u000A</code> \u000A \u000A <p>Note that in this example, rewriting replaced the call of <v>endp</v> by its \u000A body after instantiating its body with the actuals from the call.  This is \u000A sometimes just called <i>expanding</i> the definition of <v>endp</v>.  (The \u000A notions of <i>formal</i>, <i>body</i>, <i>call</i>, and <i>actuals</i> are \u000A discussed in <see topic=\"ACL2____PROGRAMMING-KNOWLEDGE-TAKEN-FOR-GRANTED\">programming-knowledge-taken-for-granted</see>.)</p> \u000A \u000A <p>Expanding a definition is an example of <i>unconditional</i> rewriting. \u000A All definitions in ACL2 are just bare equalities relating a call of the \u000A function on its formals to its body.  Any time you use an equality theorem, \u000A whether a definitional equality or something more general like</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A       (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))</code> \u000A \u000A <p>to replace an instance of one side by the corresponding instance of the \u000A other in a goal conjecture, we call that <i>unconditional</i> rewriting with \u000A the equality.</p> \u000A \u000A <p><b>Example 2</b>: Suppose your goal conjecture is:</p> \u000A \u000A <code><i>Goal Conjecture</i>: \u000A(implies (and (subsetp a b) \u000A              (true-listp b) \u000A              (member e a)) \u000A         (&lt; (len (rm e b)) (len b))). \u000A</code> \u000A \u000A <p>This conjecture may be read ``if <v>a</v> is a subset of the <v>true-listp</v> \u000A <v>b</v> and <v>e</v> is a member of <v>a</v>, then the result of removing <v>e</v> \u000A from <v>b</v> has a shorter length than <v>b</v>.''</p> \u000A \u000A <p>You can use the following theorem:</p> \u000A \u000A <code><i>Theorem</i>: \u000A(implies (member u v) \u000A         (equal (len (rm u v)) \u000A                (- (len v) 1))) \u000A</code> \u000A \u000A <p>to <i>rewrite</i> the <i>Goal Conjecture</i> to</p> \u000A \u000A <code><i>Rewritten Goal Conjecture</i>: \u000A(implies (and (subsetp a b) \u000A              (true-listp b) \u000A              (member e a)) \u000A         (&lt; (- (len b) 1) (len b))). \u000A</code> \u000A \u000A <p>To do this you must know that the following subgoal is provable:</p> \u000A \u000A <code><i>Subgoal to Relieve Hyp 1</i>: \u000A(implies (and (subsetp a b) \u000A              (true-listp b) \u000A              (member e a)) \u000A         (member e b)). \u000A</code> \u000A \u000A <p>This is an example of <i>conditional</i> rewriting.  In order to use the \u000A <i>Theorem</i> we had to establish that its hypotheses are satisfied.  That is \u000A called <i>relieving the hypotheses</i> and was done by proving the <i>Subgoal \u000A to Relieve Hyp 1</i>.  Conditional rewriting is the most commonly used proof \u000A technique in ACL2.</p> \u000A \u000A <p>Unconditional rewriting is just a special case, where there are no \u000A hypotheses to relieve.</p> \u000A \u000A <p>Expanding a definition is just another special case, where there are no \u000A hypotheses to relieve and the pattern is easy to match because it is a call of \u000A a function on distinct variables.</p> \u000A \u000A <p>This page discusses <i>rewriting</i> from the logical perspective.  It is \u000A important that you are familiar with the notions of a <i>pattern</i> term \u000A being an <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see> of a <i>target</i> \u000A term.  We often say the pattern <i>matches</i> the target.  These notions \u000A involve a corresponding <i>substitution</i> of terms for variables.  All these \u000A notions are discussed in the link for ``<see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see>'' above and we \u000A recommend you read it before continuing.  Then use your browser's <b>Back \u000A Button</b> to come back here.</p> \u000A \u000A <p>You should also be aware of the terms introduced in our discussion of <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see>.</p> \u000A \u000A <p><i>Rewriting</i> is a fundamental rule of inference in our system.  The \u000A rule allows you to use a theorem, i.e., an axiom, lemma, or definition, to \u000A replace one term by another in the goal conjecture you're trying to prove.</p> \u000A \u000A <p>Suppose you have a theorem that is of the form (or can be put into the \u000A form):</p> \u000A \u000A <code><i>Theorem</i>: \u000A(implies (and <i>hyp1</i> \u000A              ... \u000A              <i>hypk</i>) \u000A         (equal <i>pattern</i> \u000A                <i>replacement</i>)) \u000A</code> \u000A \u000A <p>From the logical perspective we don't care how the theorem was actually \u000A written when it was proved.  It might have no hypotheses (in which case the \u000A <i>hypi</i> could just be <v>t</v>), or it could have been written in a \u000A different but equivalent propositional style, <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see></v> <i>hyp1</i><v>)\u000A ...)</v>, or the equality could have been written the other way around, \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> </v><i>replacement</i> <i>pattern</i><v>)</v>.  Such syntactic details \u000A don't matter.  Just take a theorem and use propositional calculus to rearrange \u000A it equivalently into this form for the purposes of this one rewrite step.</p> \u000A \u000A <p>Suppose <i>pattern</i> is an instance of some target term, <i>target</i> \u000A that occurs in your goal conjecture.  Let the corresponding substitution be \u000A <i>sigma</i>.  If <i>sigma</i> does not contain a binding for every variable \u000A that occurs in <i>Theorem</i>, then extend <i>sigma</i> to <i>sigma'</i> by \u000A adding one binding for each such variable.  (This is necessary only if \u000A <i>pattern</i> does not contain every variable in <i>Theorem</i>.)</p> \u000A \u000A <p>Let <i>replacement'</i> be the result of instantiating <i>replacement</i> \u000A with <i>sigma'</i>.  Let <i>hypi'</i> be the result of instantiating \u000A <i>hypi</i> with <i>sigma'</i>.</p> \u000A \u000A <p>Then the <b>Rewrite Rule of Inference</b> tells us it is permitted to \u000A replace that occurrence of <i>target</i> in the goal by <i>replacement'</i> -- \u000A <b>if you can prove</b> each <i>hypi'</i> in this context.  We make this last \u000A condition clear in a moment.</p> \u000A \u000A <p>The justification for this is that <i>Theorem</i> is true for all values of \u000A the variables.  Hence, it is true for the values specified by <i>sigma'</i>. \u000A If the <i>hypi'</i> are true, then the target is really equal to \u000A <i>replacement'</i>.  But it is always permitted to replace something by \u000A something it's equal to.</p> \u000A \u000A <p>Rewriting thus involves several steps:</p> \u000A \u000A <p>(1) Finding a <i>target</i> and a <i>theorem</i> to use to rewrite it to \u000A some more desirable <i>replacement</i>.</p> \u000A \u000A <p>(2) Instantiating <i>pattern</i> in the (rearranged) theorem to match \u000A <i>target</i>.</p> \u000A \u000A <p>(3) Extending <i>sigma</i> to <i>sigma'</i> to bind all the variables in \u000A <i>Theorem</i>.</p> \u000A \u000A <p>(4) Establishing that the <i>sigma'</i> instances of each of the \u000A <i>hypi</i> hold.  This is called <i>relieving the hypotheses</i> of the \u000A theorem and is discussed in greater detail below.</p> \u000A \u000A <p>(5) Replacing the occurrence of <i>target</i> in the goal conjecture by the \u000A <i>sigma'</i> instance of <i>replacement</i>, provided all the hypotheses are \u000A relieved.</p> \u000A \u000A <p>Step (4) above, <i>relieving the hypotheses</i>, requires first identifying \u000A the ``context'' of the target in the goal conjecture.  To do this, use \u000A propositional calculus to rearrange the goal conjecture so the occurrence of \u000A <i>target</i> is in the conclusion and let <i>context</i> be the \u000A hypothesis.</p> \u000A \u000A <code><i>Rearranged Conjecture</i>: \u000A(implies <i>context</i> \u000A         (... <i>target</i> ...)) \u000A</code> \u000A \u000A <p>To relieve the hypotheses you must then prove each of the following \u000A subgoals:</p> \u000A \u000A <code><i>Subgoal to Relieve Hyp i</i>: \u000A(implies <i>context</i> <i>hypi'</i>). \u000A</code> \u000A \u000A <p>It is important to note that this description of rewriting with \u000A <i>Theorem</i> describes the process from a strictly logical perspective.  The \u000A syntax of the theorem and the goal don't matter.  You're free to use \u000A propositional calculus to rearrange them to put them into the appropriate \u000A forms to fit the descriptions given.  Clearly, if you have a candidate Theorem \u000A in the ``wrong'' form and but it can be rearranged with propositional calculus \u000A into the ``right'' form, then that rearranged theorem is also a <i>Theorem</i> \u000A and can be used as described.  But in the actual implementation of ACL2, the \u000A syntactic form of a proved <i>Theorem</i> affects how it is used by rewriting. \u000A If a proved theorem takes the form of an implication concluding with an \u000A equality, ACL2 treats the left-hand side of the equality as <i>pattern</i> and \u000A the right-hand side as <i>replacement</i>, unless you tell it otherwise. \u000A We'll discuss this later.</p> \u000A \u000A <p>Furthermore, being from the logical perspective this discussion of \u000A rewriting does not address (a) how you extend <i>simga</i> to <i>sigma'</i> -- \u000A any extension will do provided it allows you to relieve the hypotheses.  The \u000A ACL2 theorem prover uses certain heuristics which we'll discuss later, \u000A including methods by which you can guide the system in the selection.</p> \u000A \u000A <p>Crucially, it does not discuss whether it is a <i>good idea</i> to do a \u000A particular rewrite!  For example, the definitional equality:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A           0\u000A           (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code> \u000A \u000A <p>may be used repeatedly, endlessly, to replace <v>(<see topic=\"ACL2____LEN\">len</see> a)</v> by an ever \u000A growing sequence of terms:</p> \u000A \u000A <code>(<see topic=\"ACL2____LEN\">len</see> a)\u000A=\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> a)\u000A    0\u000A    (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> a))))\u000A=\u000A(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> a)\u000A    0\u000A    (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> a))\u000A             0\u000A             (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> a)))))))\u000A= ...</code> \u000A \u000A <p>The ACL2 implmentation of rewriting uses certain heuristics and the you can \u000A guide the system in its choices.  We'll discuss this later.</p> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the example proof in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This answer is in the form of a script sufficient to lead ACL2 to a \u000A proof, with a brief prologue.</p> \u000A \u000A <p>We wish to collect one copy of each element in x.  We'll actually define \u000A the method two ways, primitive recursively and tail-recursively, the latter \u000A method being analogous to the program:</p> \u000A \u000A <code>a = nil;\u000Awhile (x not empty) {\u000A  a = if (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a) then a else (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a);\u000A  x = (<see topic=\"COMMON-LISP____CDR\">cdr</see> x);\u000A  }\u000Areturn a;</code> \u000A \u000A <p>We'll prove the two ``equivalent'' and we'll prove that they return a \u000A subset of <v>x</v> that contains no duplications.</p> \u000A \u000A <p>This page is organized into four sections.  (A) We will start by proving \u000A that the primitive recursive version correct: it returns a subset of its \u000A argument that is duplication free.  This will be straightforward.  (B) Then \u000A we'll define the <v>while</v>-loop version and we will prove it ``equivalent'' \u000A to the primitive recursive version.  This will be challenging primarily \u000A because the two methods collect their answers in different orders; even \u000A stating the relationship between the two is interesting.  Proving it will \u000A involve a few lemmas.  But once we prove their ``equivalence'' the correctness \u000A of the <v>while</v>-loop version will be straightforward from the correctness of \u000A the primitive recursive version.  (C) We will disable the rules we prove about \u000A the <v>while</v>-loop version and prove it correct directly, without exploiting \u000A the primitive recursive version.  This requires leading the theorem prover \u000A more carefully because reasoning about tail-recursive functions that \u000A accumulate results is sometimes delicate.  (D) Lessons learned -- a narrative \u000A that summarizes what we learn from these examples.</p> \u000A \u000A <p>We follow The Method, which, recall, involves us in recursive attempts to \u000A prove lemmas.  We use a notation to indicate our sequence of proof attempts. \u000A Here is an example (although in actual use we print things across multiple \u000A lines).  The number in bracket indicates our ``stack depth''.  The ``key \u000A term'' is some term from a Key Checkpoint in the failed proof which is \u000A responsible for our subsequent action.  Sometimes instead of a Key Term we \u000A just give an English explanation of what we're thinking.</p> \u000A \u000A <code>[0] (<see topic=\"ACL2____DEFTHM\">defthm</see> main ...)     Failed!    Key Term: ...\u000A[1] (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-1 ...)  Succeeded!\u000A[0] (<see topic=\"ACL2____DEFTHM\">defthm</see> main ...)     Failed!    Key Term: ...\u000A[1] (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2 ...)  Failed!    Key Term: ...\u000A[2] (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2a ...) Succeeded!\u000A[2] (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2b ...) Succeeded!\u000A[1] (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2 ...)  Succeeded!\u000A[0] (<see topic=\"ACL2____DEFTHM\">defthm</see> main ...)     Succeeded!</code> \u000A \u000A <p>The rest of this page is just a re-playable script.</p> \u000A \u000A <code>; -----------------------------------------------------------------\u000A; Section A:  The Primitive Recursive Version and Its Correctness\u000A\u000A; The property of having duplications is defined as:\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> dupsp (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          t\u000A          (dupsp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A; The primitive recursive method of collecting one copy of each element is:\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> collect-once (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          (collect-once (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (collect-once (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-collect-once\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (collect-once x) x))\u000A; Succeeded!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-collect-once\u000A;   (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (collect-once x))))\u000A; Failed!\u000A; Key Term:  (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (COLLECT-ONCE (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-collect-once\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (collect-once a))\u000A       (<see topic=\"COMMON-LISP____MEMBER\">member</see> e a)))\u000A; Succeeded!\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-collect-once\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (collect-once x))))\u000A; Succeeded!\u000A\u000A; That was really easy!\u000A\u000A;-----------------------------------------------------------------\u000A; Section B:  The While-Loop Version and Its Correctness --\u000A;  presented in two parts:  its equivalence to the primitive recursive\u000A;  version and then its correctness proved via that equivalence\u000A\u000A; The tail-recursive, or while-loop version, is defined as follows.  The\u000A; function below is the loop itself and it ought to be called with a = nil to\u000A; implement the initialization of a in the pseudo-code above.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> while-loop-version (x a)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      a\u000A      (while-loop-version (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A                          (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a)\u000A                              a\u000A                              (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a)))))\u000A\u000A; We wish to prove that the two are equivalent.  But they are actually\u000A; very different.  For example,\u000A\u000A; (collect-once '(2 4 1 3 1 2 3 4))           = (1 2 3 4)\u000A; (while-loop-version '(2 4 1 3 1 2 3 4) nil) = (3 1 4 2)\u000A\u000A; Things get a little more complicated if a is non-nil:\u000A; (while-loop-version '(2 4 1 3 1 2 3 4) '(2 2 4 4)) = (3 1 2 2 4 4)\u000A\u000A; Several observations help explain what is happening.  (1) Collect-once\u000A; collects the last occurrence of each element, in the order of their last\u000A; occurrences.  So, for example, since the last occurrence of 2 preceeds the\u000A; last occurrence of 3 in '(2 4 1 3 1 2 3 4)), then the collected 2 preceeds\u000A; the collected 3 in the answer.  But while-loop-version collects the first\u000A; occurrence of each element, in the reverse order of that occurrence.  So it\u000A; adds 2 to its accumulator first and adds 3 last, making 3 preceed 2 in the\u000A; answer.\u000A\u000A; (2) The while-loop-version does not collect anything already in a and indeed\u000A; just adds stuff to the front of a, returning everything initially in a plus\u000A; one occurrence of everything in x not in a.\u000A\u000A; To state the relationship that holds between these two we have to define two\u000A; other functions.\u000A\u000A; This is our familiar list reverse function...\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A\u000A; And this function ``removes'' from x all the elements in y, i.e., copies x\u000A; while dropping the elements of y.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> list-minus (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) y)\u000A          (list-minus (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (list-minus (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))))\u000A\u000A; The specific equivalence we're really interested in is\u000A; (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x nil)\u000A;        (collect-once (rev x)))\u000A\u000A; But we will not be able to prove that by induction because it has the\u000A; constant nil where we need a variable, a, in order to admit an appropriate\u000A; inductive instance.  So we will attack the most general problem.  What is\u000A; (while-loop-version x a) equal to, in terms of collect-once?\u000A\u000A; The most general relationship between the two collection functions is:\u000A\u000A; (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A;        (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a))\u000A\u000A; This formula bears thinking about!  If you're like us, you won't believe it\u000A; until it is proved!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> general-equivalence\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A;          (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a)))\u000A; Failed!\u000A; Key term in checkpoint:\u000A; (LIST-MINUS (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))) A)\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> list-minus-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (list-minus (<see topic=\"COMMON-LISP____APPEND\">append</see> a b) c)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> (list-minus a c)\u000A                 (list-minus b c))))\u000A; Succeeded!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> general-equivalence\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A;          (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a)))\u000A; Failed!\u000A; Key term in checkpoint:\u000A; (COLLECT-ONCE (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (LIST-MINUS (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) A) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))\u000A\u000A; [1]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> collect-once-append\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (collect-once (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A;          (<see topic=\"COMMON-LISP____APPEND\">append</see> (list-minus (collect-once a) b)\u000A;                  (collect-once b))))\u000A; Failed!\u000A; Key term:\u000A; (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) B))\u000A\u000A; [2]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e a)\u000A           (<see topic=\"COMMON-LISP____MEMBER\">member</see> e b))))\u000A; Succeeded!\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> collect-once-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (collect-once (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> (list-minus (collect-once a)\u000A                             b)\u000A                 (collect-once b))))\u000A; Succeeded!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> general-equivalence\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A;          (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a)))\u000A; Failed!\u000A; Key term:\u000A; (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (LIST-MINUS (COLLECT-ONCE (LIST-MINUS (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) A))\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> assoc-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> a b) c)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))))\u000A; Succeeded!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> general-equivalence\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A;          (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a)))\u000A; Failed!\u000A; Key term:\u000A; (LIST-MINUS (COLLECT-ONCE (LIST-MINUS (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) A)) ...)\u000A\u000A; This key term makes us think of the lemma to move the LIST-MINUS inside the\u000A; COLLECT-ONCE.  But when that's done, we will have two LIST-MINUS terms\u000A; nestled together and we will want to combine them into one.  Call these two\u000A; lemmas (a) and (b).\u000A\u000A; [1] (a)\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> list-minus-collect-once\u000A;   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (list-minus (collect-once x) a)\u000A;          (collect-once (list-minus x a))))\u000A; Failed!\u000A; Key term:\u000A; (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (LIST-MINUS (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))\u000A\u000A; [2] (A pretty fact)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-list-minus\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (list-minus x a))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A            (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e a)))))\u000A; Succeeded!\u000A\u000A; [1] (a)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> list-minus-collect-once\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (list-minus (collect-once x) a)\u000A         (collect-once (list-minus x a))))\u000A; Succeeded!\u000A\u000A; [1] (b)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> list-minus-list-minus\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (list-minus (list-minus x a) b)\u000A         (list-minus x (<see topic=\"COMMON-LISP____APPEND\">append</see> b a))))\u000A; Succeeded!\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> general-equivalence\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (while-loop-version x a)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> (collect-once (list-minus (rev x) a)) a)))\u000A; Succeeded!\u000A\u000A; That completes the proof of the ``equivalence'' of the two methods.\u000A\u000A; Now we prove (1) that the result of while-loop-version is a subset, and (2)\u000A; that it contains no duplications.  We prove the two conjuncts separately.\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-while-loop\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x nil) x))\u000A; Succeeded!\u000A\u000A; But the theorem prover works harder to do the proof above than one might have\u000A; expected because it doesn't turn into an instance of\u000A; main-theorem-1-about-collect-once because of the presence of the rev term.\u000A; However, we're content that ACL2 managed to do the proof on its own.\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-while-loop\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (while-loop-version x nil))))\u000A\u000A; So we see that the proof of correctness of while-loop-version isn't hard,\u000A; after we establish the relationship with the primitive recursive version.\u000A; But finding and proving the relationship is fairly challenging.\u000A\u000A; -----------------------------------------------------------------\u000A; Section C:  A Direct Proof of the Correctness of the While-Loop Version\u000A\u000A; Some would consider the proof in Section B ``indirect'' because we first showed\u000A; how while-loop-version could be expressed as a collect-once and then proved\u000A; our main theorems about while-loop-version, which means those main proofs\u000A; were conducted in terms of collect-once, not while-loop-version.\u000A\u000A; It is interesting to compare this proof with the ``direct'' one in which\u000A; we don't use collect-once at all and reason only about while-loop-version.\u000A\u000A; So to do that comparison, let's disable all the lemmas we've proved about\u000A; while-loop-version and try to prove the two main theorems above about\u000A; while-loop-version.\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> general-equivalence\u000A                    main-theorem-1-about-while-loop\u000A                    main-theorem-2-about-while-loop))\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-while-loop-redux\u000A;   (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x nil) x))\u000A; Failed!  [Well, the truth is below...]\u000A\u000A; We don't even submit this event above because we recognize that it is not\u000A; general enough to permit proof by induction.  We need to deal with the nil in\u000A; the second argument of while-loop-version.  Experience with induction tells\u000A; us this should be a variable, so we can assume an appropriate inductive\u000A; instance.  Therefore, we adopt this subgoal immediately:\u000A\u000A; [1]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-lemma-1-about-while-loop-version\u000A;   (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x a) (<see topic=\"COMMON-LISP____APPEND\">append</see> x a)))\u000A; Failed!\u000A; Key Term:  Does the wrong induction.\u000A\u000A; [1]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-lemma-1-about-while-loop-version\u000A;   (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x a) (<see topic=\"COMMON-LISP____APPEND\">append</see> x a))\u000A;   :hints ((&quot;Goal&quot; :induct (while-loop-version x a))))\u000A; Failed!  Two key terms are suggested\u000A; Key term: (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> ... (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (WHILE-LOOP-VERSION (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A)))\u000A;                    (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (WHILE-LOOP-VERSION (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A) (<see topic=\"COMMON-LISP____CONS\">CONS</see> ... (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))))\u000A; Key term: (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> A A)\u000A; So we'll prove both before trying again.\u000A; [2]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-cons\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a b)\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e b))))\u000A; Succeeded!\u000A\u000A; [2]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-reflexive\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a a))\u000A; Succeeded!\u000A\u000A; [1]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-lemma-1-about-while-loop-version\u000A;   (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x a) (<see topic=\"COMMON-LISP____APPEND\">append</see> x a))\u000A;   :hints ((&quot;Goal&quot; :induct (while-loop-version x a))))\u000A; Failed!\u000A; Key Term:\u000A; (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> ...\u000A;               (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (WHILE-LOOP-VERSION (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) A))\u000A;                        (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) A))))\u000A;          (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (WHILE-LOOP-VERSION (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) A))\u000A;                   (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))))\u000A\u000A; We'd be done if we could rewrite the\u000A; (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) A))\u000A; to\u000A; (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A))\u000A; These two terms are not equal!  But they are ``set-equal'' and this kind of\u000A; rewriting is possible using user-defined equivalences and congruence rules.\u000A; But the new user should not dive into congruences yet.  So we will do this\u000A; with ordinary lemmas:\u000A\u000A; The plan then is to prove\u000A; (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))\u000A;      (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))))\u000A\u000A; Consider the first half of this bi-implication:\u000A; (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))            ; hyp1\u000A;          (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))))           ; concl\u000A; Notice that if we knew\u000A; (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)) (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))) ; hyp2\u000A; then we could use hyp1 and hyp2 together with the transitivity of\u000A; subsetp to get concl.\u000A\u000A; The proof in the other direction is comparable but requires the\u000A; (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)) (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))\u000A\u000A; Thus, our plan is prove\u000A; (a) transitivity of subsetp\u000A; (b) (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)) (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)))\u000A; (c) (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)) (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))\u000A\u000A; in order to prove\u000A; (d) (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))\u000A;         (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))))\u000A\u000A; [2] (a)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> trans-subsetp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a b)\u000A                (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> b c))\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a c)))\u000A; Succeeded!\u000A\u000A; [2] (b)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-cons-v-cons-append-1\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c))\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))))\u000A; Succeeded!\u000A\u000A; [2] (c)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-cons-v-cons-append-2\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c))\u000A           (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c))))\u000A; Succeeded!\u000A\u000A; [2] (d)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-cons-cons-append\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b (<see topic=\"COMMON-LISP____CONS\">cons</see> e c)))\u000A       (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)))))\u000A; Succeeded!\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-lemma-1-about-while-loop-version\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x a) (<see topic=\"COMMON-LISP____APPEND\">append</see> x a))\u000A  :hints ((&quot;Goal&quot; :induct (while-loop-version x a))))\u000A; Succeeded!\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-while-loop-version\u000A;   (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x nil) x))\u000A; Failed!  [But the truth is below...]\u000A\u000A; But we don't submit this because we don't expect it to be proved\u000A; from the main lemma just proved:  they don't match!  But\u000A; note that if we instantiated the main lemma, replacing a by nil,\u000A; we get:\u000A\u000A; (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x nil) (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil))\u000A\u000A; and we could simplify the (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) to x in this context, with\u000A; another congruence rule -- if we were using them.  So let's prove\u000A; first that we can simplify (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) inside a subsetp:\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-nil\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y nil))\u000A       (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y)))\u000A; Succeeded!\u000A\u000A; and then just tell ACL2 how to use the lemma to get the main theorem.  Note\u000A; that we give a hint to instantiate main-lemma-1... but we also disable\u000A; main-lemma-1... because otherwise it will rewrite itself away!  Once the\u000A; instance of main-lemma-1... is sitting around as a hypothesis,\u000A; subsetp-append-nil will rewrite the (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil) to x for us and finish the\u000A; proof.\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-while-loop-version\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (while-loop-version x nil) x)\u000A  :hints ((&quot;Goal&quot;\u000A           :use (:instance main-lemma-1-about-while-loop-version\u000A                           (x x)\u000A                           (a nil))\u000A           :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> main-lemma-1-about-while-loop-version))))\u000A; Succeeded!\u000A\u000A; Recall that the main-theorem-1... just proved is just half of what we want.\u000A; We also want:\u000A\u000A; [0]\u000A; (<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-while-loop-version\u000A;   (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (while-loop-version x nil))))\u000A; Failed!  [But the truth is below...]\u000A\u000A; But, again, we don't submit that because the nil makes it not general enough for\u000A; induction.  Instead we go immediately to:\u000A\u000A; [1]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-lemma-2-about-while-loop-version\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp a))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (while-loop-version x a)))))\u000A; Succeeded!\u000A\u000A; This time we know our main-lemma-2... will match (there's no (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil)\u000A; in there to mess things up) and so we can complete the proof with:\u000A\u000A; [0]\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-while-loop-version\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (while-loop-version x nil))))\u000A; Succeeded!\u000A\u000A;-----------------------------------------------------------------\u000A; Section D:  Lessons Learned\u000A\u000A; The most obvious lesson is that it is easier to reason about the primitive\u000A; recursive collect-once than about the while-loop-version.  Thus, if your only\u000A; need is for a function that collects one occurrence of each element of a list\u000A; and you don't care about the order in which you collect them and you don't\u000A; need it to be very sparing of stack space when it executes, then use the\u000A; primitive recursive definition and don't even think about while loops!\u000A\u000A; So why might you be driven to while-loop-version?  One possibility is that\u000A; the list you wish to process is very long and the primitive recursive version\u000A; would produce a stack overflow.  In ACL2, that would mean the list would have\u000A; to be several thousand long.  Is your application really so demanding?\u000A\u000A; Another possibility is that you are modeling in Lisp a while loop expressed\u000A; in some other programming language.  In that case, the fidelity of your model to\u000A; the artifact being modeled is important and you should use while-loop-version.\u000A\u000A; Another possibility is that for some reason order matters and you really are\u000A; interested in collecting the first occurrence rather than the last.  Of\u000A; course this is most likely to be relevant in more interesting applications\u000A; where the occurrences are somehow distinguishable.\u000A\u000A; If you are forced to deal with the while-loop-version the question is do you\u000A; do an indirect proof as in Section B or a direct proof as in Section C?\u000A; The indirect proof involved 10 theorems and the direct proof involved 11.\u000A; That is not a significant difference.\u000A\u000A; But our sense is that the indirect proof is easier to find, once you figure\u000A; out the basic shape of the relation between while-loop-version collect-once.\u000A; In particular, we had to give the theorem prover two hints in the direct\u000A; proof (versus no hints in the indirect proof).  One of our hints was about\u000A; what induction to do and the other was about how to use a previously proved\u000A; instance of a lemma involving an accumulator.  Furthermore, we had to think\u000A; carefully about the use of the transitivity of subsetp and we had to hack our\u000A; way around rewriting (<see topic=\"COMMON-LISP____APPEND\">append</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> e b)) to (<see topic=\"COMMON-LISP____CONS\">cons</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> a b)) in a\u000A; subsetp-expression.\u000A\u000A; Some of these ``set'' problems could have been handled a lot more elegantly by\u000A; defining set-equal as an equivalence relation and proving the congruence\u000A; rules to allow the rewriting of set-equal terms to set-equal terms inside\u000A; certain expressions like subsetp and member.  However, that involves a lot of\u000A; overhead in the form of congruence rules showing that set-equality is\u000A; maintained by replacement of set-equals by set-equals in various argument\u000A; positions of the various functions.  See :doc congruence.  In general, we\u000A; find congruence-based reasoning extremely neat and powerful when the\u000A; appropriate infrastructure has been built up.  But because the infrastructure\u000A; is ``heavy'' we tend not to invest in it for small projects.\u000A\u000A; In summary, different users might take home different lessons about whether a\u000A; direct or indirect proof is better here.  This is in part due to the\u000A; complexity of the functional relationship between collect-once and\u000A; while-loop-version, which additionall involved append, list-minus, and rev.\u000A; Had the relationship been simpler, the indirect proof would have been\u000A; preferred.\u000A\u000A; An undeniable lesson, however, is that it is helpful to know both styles of\u000A; proof and to be able to explore both as needed in your applications.</code> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4\">introductory-challenge-problem-4</see>.</p>"],

 "ACL2____INTRODUCTION-TO-HINTS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>We assume you've read <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see>, \u000A <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>, and <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see>.</p> \u000A \u000A <p>You may give the theorem prover a hint that is specific to a particular \u000A subgoal generated during a proof attempt.  Of course, you learn the name of \u000A the subgoal by inspecting the key checkpoints or other proof output.  You are \u000A not expected to anticipate the need for hints at specific subgoals; instead, \u000A you usually deduce that a hint is required because the subgoals is not proved \u000A but you see that existing rules and context make it provable.</p> \u000A \u000A <p>The most common hint is to enable and/or disable a particular rule on some \u000A particular subgoal.</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :hints ((\"Subgoal *1/3.2''\" :in-theory (disable nth-nthcdr)))) \u000A</code> \u000A \u000A <p>The hint above tells the rewriter that just before it begins to work on \u000A <v>Subgoal *1/3.2''</v> it should switch to a local theory in which all of the \u000A rules generated from the event <v>nth-nthcdr</v> are disabled.  That local \u000A theory remains the one in use for all descendent subgoals generated from the \u000A one named, until and unless one of those descendent subgoals has an \u000A <v>:in-theory</v> hint associated with it.  There are many kinds of hints \u000A besides <v>:in-theory</v> and in general, after each subgoal name, you can give \u000A various forms of advice and list various adjustments you wish to make to the \u000A context in which the prover is operating when it begins addressing the subgoal \u000A named.</p> \u000A \u000A <p>The top-level goal is always named <v>Goal</v>.  Thus</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :hints ((\"Goal\" ...<i>hints1</i>...) \u000A                (\"Subgoal *1/3.2''\" ...<i>hints2</i>...))) \u000A</code> \u000A \u000A <p>has the effect of using <i>hints1</i> for the top-level goal and all of its \u000A children throughout the entire proof, except for <v>Subgoal *1/3.2''</v> and its \u000A children, where <i>hints2</i> is used instead.</p> \u000A \u000A <p>There are a few hints which ``take effect'' exactly on the subgoal to which \u000A they are attached and are not inherited by their descendents.</p> \u000A \u000A <p>Here is an incomplete list of some of the more common hints; we note the \u000A ones that do not pass on their effects to their descendents.  We recommend \u000A that you not follow the advanced links (marked ``<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>'') \u000A below until you have read the entire tutorial.</p> \u000A \u000A <p>See <see topic=\"ACL2____IN-THEORY\">in-theory</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for how to enable and/or disable rules.  The new \u000A theory is computed by a ``theory expression'' (see <see topic=\"ACL2____THEORIES\">theories</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A such as <v>(<see topic=\"ACL2____DISABLE\">disable</see> nth-nthcdr)</v> and typically makes adjustments such as \u000A additions or deletions to the global current theory.  All the relevant new \u000A theories are computed before the proof begins.  Thus, in</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :hints ((\"Goal\" :in-theory (disable rule1)) \u000A                (\"Subgoal *1/3.2''\" (disable rule2)))) \u000A</code> \u000A \u000A <p>the theory mentioned for <v>Goal</v> is the global current theory minus \u000A <v>rule1</v>, while the theory mentioned for its descendent, <v>Subgoal\u000A *1/3.2''</v>, is the global current theory minus <v>rule2</v>.  In particular, if \u000A both <v>rule1</v> and <v>rule2</v> are enabled in the global current theory, then \u000A <v>rule1</v> is enabled during the processing of <v>Subgoal *1/3.2''</v> because \u000A it was not removed explicitly there.</p> \u000A \u000A <p>See <see topic=\"ACL2____USE\">use</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for how to force the theorem prover to take note of \u000A particular instances of particular theorems; in particular, the instances are \u000A created and added as hypotheses to the subgoal in question.  The hint is not \u000A inherited by the descendents of the subgoal (since the formula being proved \u000A has been modified by the hint).  If the rule you are using (with a <v>:use</v> \u000A hint) is an enabled rewrite rule, it might interfere with the added hypothesis \u000A -- by rewriting itself to <v>T</v> -- and thus often you will both <v>:use ...</v> \u000A and <v>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> ...)</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EXPAND\">expand</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for how to tell the theorem prover to expand one or \u000A more function calls whenever encountered.</p> \u000A \u000A <p>See <see topic=\"ACL2____CASES\">cases</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for how to force the theorem prover to do a case \u000A split to prove the subgoal under each of an exhaustive list of cases given in \u000A the hint.  This hint takes action specifically at the named subgoal and is not \u000A passed down to its children.</p> \u000A \u000A <p>See <see topic=\"ACL2____INDUCT\">induct</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for how to tell the theorem prover to go \u000A immediately into induction for the subgoal in question, and to use the \u000A induction scheme suggested by the hint rather than the one suggested by the \u000A terms in the subgoal itself.  This hint is not inherited by its \u000A descendents.</p> \u000A \u000A <p>See <see topic=\"ACL2____HINTS\">hints</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a complete list of all hints, and see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a more thorough description of how the effects \u000A of hints at a subgoal are inherited by the descendents.</p> \u000A \u000A <p>If you are reading this as part of the tutorial introduction to the theorem \u000A prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____GUARD-MISCELLANY":[["Guard"],"ACL2 Sources","ACL2","<p>The discussion of guards concludes here with a few miscellaneous \u000A remarks.  (Presumably you found this documentation by following a link; see \u000A <see topic=\"ACL2____GUARDS-FOR-SPECIFICATION\">guards-for-specification</see>.)  For further information related to guards \u000A other than what you find under ``<see topic=\"ACL2____GUARD\">guard</see>,'' see any of the following \u000A documentation topics: <see topic=\"ACL2____GUARD-EXAMPLE\">guard-example</see>, <tt><see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see></tt>, <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt>, <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, and (for a discussion of keyword <v>:SPLIT-TYPES</v>) <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>.</p> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____DEFUN\">Defun</see></tt> can be made to try to verify the guards on a function.  This \u000A is controlled by the ``<see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>'' of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>; see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.  The <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is either as specified with the <v>:mode</v> \u000A <v>xarg</v> of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or else defaults to the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.  See <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>.  If the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of the \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> and either a <see topic=\"ACL2____GUARD\">guard</see> is specified \u000A explicitly or <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> <v>t</v> is specified in the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>, then we attempt to verify the guards of the function.  Otherwise we do \u000A not.  (But see <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> for how to modify this \u000A behavior.)</p> \u000A \u000A <p>It is sometimes impossible for the system to verify the guards of a \u000A recursive function at definition time.  For example, the guard conjectures \u000A might require the invention and proof of some inductively derived property of \u000A the function (as often happens when the value of a recursive call is fed to a \u000A guarded subroutine).  So sometimes it is necessary to define the function \u000A using <v>:verify-guards nil</v> then to state and prove key theorems about the \u000A function, and only then have the system attempt guard verification. \u000A Post-<tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> guard verification is achieved via the event <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.  See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.</p> \u000A \u000A <p>It should be emphasized that guard verification affects only two things: \u000A how fast ACL2 can evaluate the function and whether the function is executed \u000A correctly by raw Common Lisp, without guard violations.  Since ACL2 does not \u000A use the raw Common Lisp definition of a function to evaluate its calls unless \u000A that function's guards have been verified, the latter effect is felt only if \u000A you run functions in raw Common Lisp rather than via ACL2's command loop.</p> \u000A \u000A <p>Guard verification does not otherwise affect the theorem prover or the \u000A semantics of a definition.  If you are not planning on running your function \u000A on ``big'' inputs and you don't care if your function runs correctly in raw \u000A Common Lisp (e.g., you have formalized some abstract mathematical property and \u000A just happened to use ACL2 as your language), there is no need to suffer \u000A through guard verification.  Often users start by not doing guard verification \u000A and address that problem later.  Sometimes you are driven to it, even in \u000A mathematical projects, because you find that you want to run your functions \u000A particularly fast or in raw Common Lisp.</p> \u000A \u000A <p>If <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is used to compile a file, and the file contains \u000A functions with unverified guard conjectures, then you will be warned that the \u000A compiled file cannot be loaded into raw Common Lisp with the expectation that \u000A the functions will run correctly.  This is just the same point we have been \u000A making: ACL2 and Common Lisp agree only on the restricted domains specified by \u000A our guards.  When guards are violated, Common Lisp can do anything.  When you \u000A call a compiled function on arguments violating its guards, the chances are \u000A only increased that Common Lisp will go berserk, because compiled functions \u000A generally check fewer things at runtime and tend to be more fragile than \u000A interpreted ones.</p> \u000A \u000A <p>Finally, we note that ACL2 collects up <see topic=\"ACL2____GUARD\">guard</see>s from <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A forms in order of appearance.  So for example, the <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (foo x))\u000A         (<see topic=\"COMMON-LISP____TYPE\">type</see> string x)</code> \u000A \u000A <p>will generate the guard <v>(<see topic=\"COMMON-LISP____AND\">and</see> (foo x) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x))</v>, while the form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> string x)\u000A         (<see topic=\"ACL2____XARGS\">xargs</see> :guard (foo x))</code> \u000A \u000A <p>will generate the guard <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x) (foo x))</v>.  The only exception \u000A to this rule is the case that <v>:guard</v> and <v>:stobjs</v> are specified in \u000A the same <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> form, in which case the <v>:stobjs</v> form will be \u000A treated as through it comes before the <v>:guard</v> form.</p>"],

 "ACL2____FREE-VARIABLES-EXAMPLES-REWRITE":[["Free-variables-examples"],"ACL2 Sources","ACL2","<p>The following examples illustrate ACL2's handling of free variables \u000A in <see topic=\"ACL2____REWRITE\">rewrite</see> rules, as well as user control over how such free variables \u000A are handled.  See <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> for a background discussion.</p> \u000A \u000A <code>;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Example 1\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p2 (x y) t) ; introduce unconstrained function\u000A\u000A; Get warning because of free variable.  This would be an error if you had\u000A; first executed (<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> t) in order to force yourself to\u000A; specify :match-free (illustrated later, below).\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (p2 x z)))\u000A\u000A; Succeeds.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A; The following causes an error because p2-trans is not a rune.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once p2-trans)\u000A\u000A; After the following, the rewrite rule p2-trans will only allow one\u000A; attempt per hypothesis to bind free variables.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once (:rewrite p2-trans))\u000A\u000A; Now this same theorem fails to be proved.  Here's why.  The\u000A; context for proving (p2 a d) happens to include the hypotheses in\u000A; reverse order.  So when the first hypothesis of p2-trans, namely\u000A; (p2 x y), is relieved, where x is bound to a (as we are attempting\u000A; to rewrite the current literal (p2 a d)), we find (p2 a b) in the\u000A; context before (p2 a c) and hence y is bound to b.  The\u000A; instantiated second hypothesis of p2-trans is thus (p2 b d), and\u000A; the proof fails.  Before the add-match-free-override form above,\u000A; the proof succeeded because the rewriter was allowed to backtrack\u000A; and find the other binding for the first hypothesis of p2-trans,\u000A; namely, y bound to c.  Then the instantiated second hypothesis of\u000A; p2-trans is (p2 c d), which is known to be true in the current\u000A; context.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A; Return to original behavior for binding free variables.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all t)\u000A\u000A; Succeeds once again.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A(<see topic=\"ACL2____U\">u</see>) ; undo (<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all t)\u000A\u000A; This is an error, since no further arguments should appear after\u000A; :clear.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :clear t)\u000A\u000A; Return all rules to original behavior for binding free variables,\u000A; regardless of which previous add-match-free-override forms have\u000A; been executed.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :clear)\u000A\u000A; This succeeds just as it did originally.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A(<see topic=\"ACL2____UBT_12\">ubt!</see> 'p2-trans) ; back to the start, except retain the defstub\u000A\u000A; Require that :match-free be specified for :linear and :rewrite rules with\u000A; free variables.\u000A(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> t)\u000A\u000A; Fails because :match-free is missing.\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (p2 x z)))\u000A\u000A; Fails because :match-free must be followed by :once or :all.\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (p2 x z))\u000A  :rule-classes ((:rewrite :match-free nil)))\u000A\u000A; Succeeds, this time with no warning at all.\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (p2 x z))\u000A  :rule-classes ((:rewrite :match-free :once)))\u000A\u000A; Fails because we only bind once (see earlier long comment).\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A; Treat p2-trans as though `:match-free :all' had been specified.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all (:rewrite p2-trans))\u000A\u000A; Succeeds since more than one binding is allowed for p2-trans.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A(<see topic=\"ACL2____U\">u</see>)\u000A(<see topic=\"ACL2____U\">u</see>)\u000A\u000A; Specify that future :linear and :rewrite rules with free variables\u000A; that do not have :match-free specified are treated as though\u000A; `:match-free :once' were specified.\u000A(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :once)\u000A\u000A; Succeeds without error since `:match-free' is specified, as described\u000A; above.  But there is a warning, since :match-free is not specified for this\u000A; :rewrite rule.\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-trans\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p2 y z))\u000A           (p2 x z)))\u000A\u000A; Fails since only single bindings are allowed for p2-trans.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A; Treat p2-trans as though `:match-free :all' had been specified.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all t)\u000A\u000A; Succeeds.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 a c)\u000A                   (p2 a b)\u000A                   (p2 c d))\u000A              (p2 a d)))\u000A\u000A; Test searching of ground units, i.e. rewrite rules without variables on the\u000A; left side of the conclusion, for use in relieving hypotheses with free\u000A; variables.  This is a very contrived example.\u000A\u000A(<see topic=\"ACL2____UBT_12\">ubt!</see> 1) ; back to the start\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((p1 *) =&gt; *)\u000A  ((p2 * *) =&gt; *)\u000A  ((p3 *) =&gt; *)\u000A  ((a) =&gt; *)\u000A  ((b) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p2 (x y) (<see topic=\"COMMON-LISP____LIST\">list</see> x y)))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p3 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> a () 0))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> b () 0)))\u000A\u000A; Allow default of :match-free :all (form may be omitted).\u000A(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> nil)\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> ax1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p2 x y)\u000A                (p1 y))\u000A           (p3 x)))\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-a-b\u000A  (p2 (a) (b)))\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p2-a-a\u000A  (p2 (a) (a)))\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> p1-b\u000A  (p1 (b)))\u000A\u000A; Succeeds; see long comment below on next attempt to prove this\u000A; theorem.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 (a) y)\u000A              (p3 (a))))\u000A\u000A; Now ax1 will only relieve hypothesis (p2 x y) for one binding of y:\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t)\u000A\u000A; Fails when ax1 attempts to rewrite the conclusion to true, because\u000A; the most recent ground unit for hypothesis (p2 x y) with x bound\u000A; to (a) is rule p2-a-a, which binds y to (a).  If more than one ground\u000A; unit could be used then we would backtrack and apply rule p2-a-b,\u000A; which binds y to (b) and hence hypothesis (p1 y) of ax1 is\u000A; relieved by rule p1-b.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 (a) y)\u000A              (p3 (a))))\u000A\u000A; Return rules to original :match-free behavior.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :clear)\u000A\u000A; Succeeds once again.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 (a) y)\u000A              (p3 (a))))\u000A\u000A; Just for kicks, change the behavior of a built-in rule irrelevant\u000A; to the proof at hand.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once (:rewrite string&lt;-l-trichotomy))\u000A\u000A; Still succeeds.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (p2 (a) y)\u000A              (p3 (a))))</code> \u000A \u000A <code>;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Example 2\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;</code> \u000A \u000A <p>The next example illustrates the use of the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> facility \u000A to get information about handling of free variables by the rewriter. \u000A Explanation is given after this (edited) transcript.  Input begins on lines \u000A with a prompt (search for ``ACL2''); the rest is output.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A        ((p1 (<see topic=\"ACL2____U\">u</see> x) t)\u000A         (bad (x) t)\u000A         (p2 (x y z) t)\u000A         (bar (x y) t)\u000A         (foo (x y) t)\u000A         (poo (x y) t)\u000A         (prop (<see topic=\"ACL2____U\">u</see>) t))\u000A\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (<see topic=\"ACL2____U\">u</see> x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> u x)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bad (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p2 (x y z) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y z)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> poo (x y) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x y)) nil))\u000A        (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> prop (<see topic=\"ACL2____U\">u</see>) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> u)) t))\u000A\u000A        (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-poo\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 'y3))\u000A                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x y)\u000A                          (poo x y))))\u000A\u000A        (<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-1\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p1 u x)\u000A                        (bad x)\u000A                        (p2 x y z)\u000A                        (bar x y)\u000A                        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x) ; admittedly silly!\u000A                        (foo x y))\u000A                   (prop u))\u000A          :rule-classes ((:rewrite :match-free :all))))\u000A\u000A; [[ output omitted ]]\u000A\u000ASummary\u000AForm:  ( ENCAPSULATE ((P1 ...) ...) ...)\u000ARules: NIL\u000AWarnings:  Subsume and Non-rec\u000ATime:  0.08 seconds (prove: 0.00, print: 0.01, other: 0.06)\u000A T\u000AACL2 !&gt;:brr t\u000AThe monitored runes are:\u000ANIL\u000A T\u000AACL2 !&gt;:monitor (:rewrite lemma-1) t\u000A(((:REWRITE LEMMA-1) 'T))\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p1 u0 x1)\u000A                          (bad x1)\u000A                          (bad x3)\u000A                          (bar x3 y1)\u000A                          (bar x3 y3)\u000A                          (p1 u0 x2)\u000A                          (p1 u0 x3)\u000A                          (p2 x3 y1 z1)\u000A                          (p2 x3 y3 z1))\u000A                     (prop u0)))\u000A\u000A(1 Breaking (:REWRITE LEMMA-1) on (PROP U0):\u000A1 ACL2 &gt;:eval\u000A\u000A1x (:REWRITE LEMMA-1) failed because :HYP 1 contains free variables.\u000AThe following display summarizes the attempts to relieve hypotheses\u000Aby binding free variables; see :DOC free-variables.\u000A\u000A    [1] X : X1\u000AFailed because :HYP 3 contains free variables Y and Z, for which no\u000Asuitable bindings were found.\u000A    [1] X : X2\u000AFailed because :HYP 2 rewrote to (BAD X2).\u000A    [1] X : X3\u000A        [3] Z : Z1\u000A            Y : Y1\u000AFailed because :HYP 6 rewrote to (FOO X3 Y1).\u000A        [3] Z : Z1\u000A            Y : Y3\u000AFailed because :HYP 6 rewrote to (POO X3 Y3).\u000A\u000A1 ACL2 &gt;:unify-subst\u000A     U : U0\u000A1 ACL2 &gt;</code> \u000A \u000A <p>The <v>:eval</v> command above asks the rewriter to attempt to apply the \u000A rewrite rule <v>lemma-1</v> to the term <v>(prop u0)</v>, shown just above the \u000A line with <v>:eval</v>.  As we can see at the end, the variable <v>u</v> in the \u000A conclusion of <v>lemma-1</v> is being bound to the variable <v>u0</v> in the \u000A conjecture.  The first hypothesis of <v>lemma-1</v> is <v>(p1 u x)</v>, so the \u000A rewriter looks for some <v>x</v> for which <v>(p1 u0 x)</v> is known to be true. \u000A It finds <v>x1</v>, and then goes on to consider the second hypothesis, <v>(bad\u000A x)</v>.  Since the theorem we are proving has <v>(bad x1)</v> in the hypothesis \u000A and <v>x</v> is currently bound to <v>x1</v>, the rewriter is satisfied and moves \u000A on to the third hypothesis of <v>lemma-1</v>, <v>(p2 x y z)</v>.  However, <v>x</v> \u000A is bound to <v>x1</v> and there are no instances of <v>y</v> and <v>z</v> for which \u000A <v>(p2 x1 y z)</v> is known in the current context.  All of the above analysis \u000A is summarized in the first part of the output from <v>:eval</v> above:</p> \u000A \u000A <code>    [1] X : X1\u000AFailed because :HYP 3 contains free variables Y and Z, for which no\u000Asuitable bindings were found.</code> \u000A \u000A <p>Thus, the binding of <v>x</v> to <v>x1</v> on behalf of the first hypothesis \u000A has failed.</p> \u000A \u000A <p>The rewriter now backs up to look for other values of <v>x</v> that satisfy \u000A the first hypothesis, and finds <v>x2</v> because our current theorem has a \u000A hypothesis of <v>(p1 u0 x2)</v>.  But this time, the second hypothesis of \u000A <v>lemma-1</v>, <v>(bad x)</v>, is not known to be true for <v>x</v>; that is, \u000A <v>(bad x2)</v> does not rewrite to <v>t</v>; in fact, it rewrites to itself. \u000A That explains the next part of the output from <v>:eval</v> above:</p> \u000A \u000A <code>    [1] X : X2\u000AFailed because :HYP 2 rewrote to (BAD X2).</code> \u000A \u000A <p>The rewriter now backs up again to look for other values of <v>x</v> that \u000A satisfy the first hypothesis, and finds <v>x3</v> because our current theorem \u000A has a hypothesis of <v>(p1 u0 x3)</v>.  This time, the second hypothesis of \u000A <v>lemma-1</v> is not a problem, and moreover, the rewriter is able to bind \u000A <v>y</v> and <v>z</v> to <v>y1</v> and <v>z1</v>, respectively, in order to satisfy \u000A the third hypothesis, <v>(p2 x y z)</v>: that is, <v>(p2 x2 y1 z1)</v> is known in \u000A the current context.  That explains more of the above output from \u000A <v>:eval</v>:</p> \u000A \u000A <code>[1] X : X3\u000A    [3] Z : Z1\u000A        Y : Y1</code> \u000A \u000A <p>Unfortunately, the sixth hypothesis, <v>(foo x y)</v>, rewrites to itself \u000A under the above bindings:</p> \u000A \u000A <code>Failed because :HYP 6 rewrote to (FOO X3 Y1).</code> \u000A \u000A <p>So the rewriter looks for other bindings to satisfy the third hypothesis \u000A and finds these.</p> \u000A \u000A <code>[3] Z : Z1\u000A    Y : Y3</code> \u000A \u000A <p>This time, the sixth hypothesis can be rewritten under the above bindings, \u000A from <v>(foo x3 y3)</v> to <v>(poo x3 y3)</v> by lemma <v>foo-poo</v>, but still not \u000A to <v>t</v>.</p> \u000A \u000A <code>Failed because :HYP 6 rewrote to (POO X3 Y3).</code> \u000A \u000A <p>There are no more free variable bindings to try, so this concludes the \u000A output from <v>:eval</v>.</p> \u000A \u000A <code>;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Example 3\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;</code> \u000A \u000A <p>The next pair of examples illustrates so-called ``binding hypotheses'' (see \u000A <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) and explores some of their subtleties.  The first shows \u000A binding hypotheses in action on a simple example.  The second shows how \u000A binding hypotheses interact with equivalence relations and explains the role \u000A of <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt>.</p> \u000A \u000A <p>Our first example sets up a theory with two user-supplied rewrite rules, \u000A one of which has a binding hypothesis.  Below we explain how that binding \u000A hypothesis contributes to the proof.</p> \u000A \u000A <code>; Define some unary functions.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> k (x) x)\u000A\u000A; Prove some simple lemmas.  Note the binding hypothesis in g-rewrite.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> f-k-h\u000A  (f (k (h x))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> g-rewrite\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> y (k x)) ; binding hypothesis\u000A                     (f y))\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g x) y)))\u000A\u000A; Restrict to a theory that includes the above lemmas but avoids the above\u000A; definitions.\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)\u000A                           '(f-k-h g-rewrite)))\u000A\u000A; Prove a theorem.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g (h a)) (k (h a))))</code> \u000A \u000A <p>Let us look at how ACL2 uses the above binding hypothesis in the proof of \u000A the preceding <v>thm</v> form.  The rewriter considers the term <v>(g (h a))</v> \u000A and finds a match with the left-hand side of the rule <v>g-rewrite</v>, binding \u000A <v>x</v> to <v>(h a)</v>.  The first hypothesis binds <v>y</v> to the result of \u000A rewriting <v>(k x)</v> in the current context, where the variable <v>x</v> is \u000A bound to the term <v>(h a)</v>; thus <v>y</v> is bound to <v>(k (h a))</v>.  The \u000A second hypothesis, <v>(f y)</v>, is then rewritten under this binding, and the \u000A result is <v>t</v> by application of the rewrite rule <v>f-k-h</v>.  The rule \u000A <v>g-rewrite</v> is then applied under the already-mentioned binding of <v>x</v> \u000A to <v>(h a)</v>.  This rule application triggers a recursive rewrite of the \u000A right-hand side of <v>g-rewrite</v>, which is <v>y</v>, in a context where <v>y</v> \u000A is bound (as discussed above) to <v>(k (h a))</v>.  The result of this rewrite \u000A is that same term, <v>(k (h a))</v>.  The original call of <v>equal</v> then \u000A trivially rewrites to <v>t</v>.</p> \u000A \u000A <p>We move on now to our second example, which is similar but involves a \u000A user-defined equivalence relation.  You may find it helpful to review \u000A <v>:equivalence</v> rules; see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>.</p> \u000A \u000A <p>Recall that when a hypothesis is a call of an equivalence relation other \u000A than <v>equal</v>, the second argument must be a call of <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt> \u000A in order for the hypothesis to be treated as a binding hypothesis.  That is \u000A indeed the case below; an explanation follows.</p> \u000A \u000A <code>; Define an equivalence relation.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-equiv (x y) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> my-equiv) ; introduces rule MY-EQUIV-IS-AN-EQUIVALENCE\u000A\u000A; Define some unary functions\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h1 (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h2 (x) x)\u000A\u000A; Prove some simple lemmas.  Note the binding hypothesis in lemma-3.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-1\u000A  (my-equiv (h1 x) (h2 x)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-2\u000A  (f (h2 x)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma-3\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (my-equiv y (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x)) ; binding hypothesis\u000A                     (f y))\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g x) y)))\u000A\u000A; Restrict to a theory that includes the above lemmas but avoids the above\u000A; definitions.\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)\u000A                           '(lemma-1 lemma-2 lemma-3\u000A                                     my-equiv-is-an-equivalence)))\u000A\u000A; Prove a theorem.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g (h1 a)) (h2 a)))</code> \u000A \u000A <p>The proof succeeds much as in the first example, but the following \u000A observation is key: when ACL2 binds <v>y</v> upon considering the first \u000A hypothesis of <v>lemma-3</v>, it rewrites the term <v>(<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x)</v> in a \u000A context where it need only preserve the equivalence relation <v>my-equiv</v>. \u000A At this point, <v>x</v> is bound by applying <v>lemma-3</v> to the term <v>(g (h1\u000A a))</v>; so, <v>x</v> is bound to <v>(h1 a)</v>.  The rule <v>lemma-1</v> then applies \u000A to rewrite this occurrence of <v>x</v> to <v>(h2 a)</v>, but only because it \u000A suffices to preserve <v>my-equiv</v>.  Thus <v>y</v> is ultimately bound to <v>(h2\u000A a)</v>, and the proof succeeds as one would expect.</p> \u000A \u000A <p>If we tweak the above example slightly by disabling the user's <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> <see topic=\"ACL2____RUNE\">rune</see>, then the proof of the <tt><see topic=\"ACL2____THM\">thm</see></tt> form fails because \u000A the above rewrite of <v>(<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> x)</v> is done in a context where it no \u000A longer suffices to preserve <v>my-equiv</v> as we dive into the second argument \u000A of <v>my-equiv</v> in the first hypothesis of <v>lemma-3</v>; so, <v>lemma-1</v> \u000A does not apply this time.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)\u000A                           '(lemma-1 lemma-2 lemma-3)))\u000A\u000A; Proof fails in this case!\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g (h1 a)) (h2 a)))</code>"],

 "ACL2____EQUALITY-VARIANTS-DETAILS":[["Equality-variants"],"ACL2 Sources","ACL2","<p>Here we present details about equality variants, none of which is \u000A likely to be important to the majority of ACL2 users.  Please see <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for relevant background.</p> \u000A \u000A <p>We begin by presenting <see topic=\"ACL2____EVENTS\">events</see> that implement the equality variants \u000A for <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, as these illustrate the events introduced for all macros \u000A having equality variants.  The definition of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, just below, calls \u000A the macro <v>let-mbe</v>, which in turn is just an abbreviation for a \u000A combination of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.  Here is a simplified version of \u000A the definition of this macro.  For relevant background, see <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> let-mbe (bindings &amp;key logic exec)\u000A  `(<see topic=\"COMMON-LISP____LET\">let</see> ,bindings\u000A     (<see topic=\"ACL2____MBE\">mbe</see> :logic ,logic\u000A          :exec ,exec)))</code> \u000A \u000A <p>This use of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> arranges that each argument of a call of \u000A <v>member</v> is evaluated only once.</p> \u000A \u000A <p>The actual definition of the macro <v>let-mbe</v> is a bit more complex, in \u000A order to guarantee that <see topic=\"ACL2____GUARD\">guard</see>s are appropriately checked.  For purposes \u000A of this discussion we ignore this simplification.  (You can find the \u000A definition of <v>let-mbe</v> in ACL2 source file <v>axioms.lisp</v>.)</p> \u000A \u000A <p>Consider the following definition from ACL2 source file <v>axioms.lisp</v>. \u000A Notice that it invokes the macro <v>let-mbe</v>, discussed above.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> member (x l &amp;key (test ''eql))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> test ''eq)\u000A                             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> test ''eql)\u000A                             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> test ''equal))))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> test ''eq)\u000A    `(let-mbe ((x ,x) (l ,l))\u000A              :logic (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x l)\u000A              :exec  (member-eq-exec x l)))\u000A   ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> test ''eql)\u000A    `(let-mbe ((x ,x) (l ,l))\u000A              :logic (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x l)\u000A              :exec  (member-eql-exec x l)))\u000A   (t ; (<see topic=\"COMMON-LISP____EQUAL\">equal</see> test 'equal)\u000A    `(<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> ,x ,l))))</code> \u000A \u000A <p>Inspection of the definition above shows that every call of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> \u000A expands to one that is logically equivalent to the corresponding call of \u000A <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, which is defined as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> member-equal (x lst)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) lst)\u000A        (t (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))</code> \u000A \u000A <p>The following two definitions model equality variants of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> for \u000A tests <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> and <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, respectively.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> member-eq-exec (x lst)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                             (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                           (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> lst))))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A        ((<see topic=\"COMMON-LISP____EQ\">eq</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) lst)\u000A        (t (member-eq-exec x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> member-eql-exec (x lst)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> x)\u000A                             (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                           (<see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see> lst))))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A        ((<see topic=\"COMMON-LISP____EQL\">eql</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) lst)\u000A        (t (member-eql-exec x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))</code> \u000A \u000A <p>At this point the user can write <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> x y)</v> or <v>(<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x\u000A y)</v> to call equality variants of <v>member</v> with test <v>eql</v> or \u000A <v>equal</v>, respectively.  We thus provide the following macro for the <v>eq</v> \u000A variant.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> member-eq (x lst)\u000A  `(<see topic=\"COMMON-LISP____MEMBER\">member</see> ,x ,lst :test 'eq))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> proof obligations generated by calls of <v>member</v> will \u000A include those based on its use of <v>mbe</v>, and are supported by the following \u000A two lemmas.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-eq-exec-is-member-equal\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (member-eq-exec x l)\u000A         (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x l)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-eql-exec-is-member-equal\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (member-eql-exec x l)\u000A         (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x l)))</code> \u000A \u000A <p>Finally, the following two events arrange that in certain contexts such as \u000A <see topic=\"ACL2____THEORIES\">theories</see> (including the use of <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> in <see topic=\"ACL2____EVENTS\">events</see> and \u000A <see topic=\"ACL2____HINTS\">hints</see>), <tt><see topic=\"ACL2____MEMBER-EQ\">member-eq</see></tt> and <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> are treated as references \u000A to <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> member-eq member-equal)\u000A(<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> member member-equal)</code> \u000A \u000A <p>Note however that these events do not affect printing of calls during \u000A proofs: calls of <v>member</v> and <v>member-eq</v> will be macroexpanded away, \u000A leaving you with calls of <v>member-equal</v> that are displayed in proof \u000A output.  For a way to change this behavior, see <tt><see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see></tt>.</p> \u000A \u000A <p>We conclude this topic by exploring the following recommendation made in \u000A the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see>.</p> \u000A \u000A <blockquote><p>For efficiency we recommend using the <v>-equal</v> equality \u000A variant, for example <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> or <v>(</v><tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt><v> ... :TEST\u000A 'equal)</v>, in certain contexts: <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>, <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, and <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt> forms.</p></blockquote> \u000A \u000A <p>ACL2 relies on the underlying Common Lisp for evaluation.  It also \u000A processes events in the ACL2 logic.  In order to guarantee consistency of its \u000A logical and Common Lisp evaluations, ACL2 uses a ``safe mode'' to avoid \u000A ill-guarded calls.  In particular, consider the use of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> in \u000A execution of a call of an equality variant of a primitive, <v>F</v>, other than \u000A its <v>F-equal</v> variant.  The <tt><see topic=\"ACL2____MBE\">mbe</see></tt> call discussed above requires a \u000A connection to be established between the <v>:logic</v> and <v>:exec</v> forms. \u000A For example, if <v>F</v> is called with <v>:TEST 'eql</v> (either explicitly or as \u000A the default), then ACL2 will call both <v>F-eql-exec</v> and <v>F-equal</v>, and \u000A check that the two results are equal.</p> \u000A \u000A <p>The following partial log illustrates the point above.  We define a macro \u000A that calls <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, and when a call of this macro is expanded during \u000A processing of a subsequent definition, we see that two membership functions \u000A are called on the same arguments.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (lst)\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                           (<see topic=\"COMMON-LISP____MEMBER\">member</see> 'c lst :test 'eq))))\u000A\u000ASummary\u000AForm:  ( DEFMACRO MAC ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A MAC\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> member-equal member-eq-exec)\u000A ((<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see>) (MEMBER-EQ-EXEC))\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f () (mac (a b c d)))\u000A1&gt; (ACL2_*1*_ACL2::MEMBER-EQ-EXEC C (A B C D))\u000A  2&gt; (MEMBER-EQ-EXEC C (A B C D))\u000A  &lt;2 (MEMBER-EQ-EXEC (C D))\u000A&lt;1 (ACL2_*1*_ACL2::MEMBER-EQ-EXEC (C D))\u000A1&gt; (ACL2_*1*_ACL2::MEMBER-EQUAL C (A B C D))\u000A  2&gt; (<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see> C (A B C D))\u000A  &lt;2 (<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see> (C D))\u000A&lt;1 (ACL2_*1*_ACL2::MEMBER-EQUAL (C D))\u000A\u000ASince F is non-recursive, its admission is trivial.</code> \u000A \u000A <p>If performance is an issue then we can avoid such a problem, for example as \u000A follows.  In a fresh session, let us define a suitable wrapper for calling \u000A <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> with <v>:TEST 'eq</v>.  This time, the trace in our partial log \u000A shows that we have avoided calling two membership functions.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mem-eq (x lst)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                                    (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                                  (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> lst))))\u000A         (<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test 'eq))\u000A[[ ... output omitted here ... ]]\u000A MEM-EQ\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (lst)\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                           (mem-eq 'c lst))))\u000A\u000ASummary\u000AForm:  ( DEFMACRO MAC ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A MAC\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_42\">trace$</see> member-equal member-eq-exec mem-eq)\u000A ((<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see>)\u000A  (MEMBER-EQ-EXEC)\u000A  (MEM-EQ))\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f () (mac (a b c d)))\u000A1&gt; (ACL2_*1*_ACL2::MEM-EQ C (A B C D))\u000A  2&gt; (MEM-EQ C (A B C D))\u000A  &lt;2 (MEM-EQ (C D))\u000A&lt;1 (ACL2_*1*_ACL2::MEM-EQ (C D))\u000A\u000ASince F is non-recursive, its admission is trivial.</code>"],

 "ACL2____DEALING-WITH-KEY-COMBINATIONS-OF-FUNCTION-SYMBOLS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Suppose <v>REV</v> reverses a list, <v>MEMBER</v> checks that its first \u000A argument is an element of its second, and <v>SQUARES-PLUS-3P</v> is some \u000A complicated predicate.  Suppose you're proving some Main Theorem that involves \u000A those concepts and the theorem prover presents you with the following hideous \u000A formula as a key checkpoint.  What action should you take?</p> \u000A \u000A <p>Hint: Don't read the formula ``for sense,'' i.e., don't try to understand \u000A what this formula is saying!  Just look at every subterm involving a nest of \u000A two function symbols and ask if you know something about those two symbols \u000A that allows you to <i>simplify</i> that one subterm.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I)) (REV X))\u000A              (LIST-OF-INTEGERS X)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> I)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 I)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> K)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 K)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> I K)\u000A              (SQUARES-PLUS-3P K X)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I))))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I)) X)))\u000A         (SQUARES-PLUS-3P K (REV X)))?</code> \u000A \u000A <p>The experienced ACL2 user will stop reading at the second hypothesis!</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I)) (REV X))</code> \u000A \u000A <p>The combination of <v>MEMBER</v> and <v>REV</v> can be simplified.  The \u000A question ``is <v>e</v> a member of <v>(REV x)</v>'' can be answered by asking ``is \u000A <v>e</v> a member of <v>x</v>''.  The two questions are equivalent.  This insight \u000A comes from your intuition about the <i>semantics</i> of <v>REV</v> -- it just \u000A reorders the elements but doesn't add or delete any.  The second question is \u000A simpler since it doesn't mention <v>REV</v>, so this is a good transformation to \u000A make.  And the theorem that they are equivalent is simpler than the key \u000A checkpoint above because it involves fewer functions and smaller \u000A expressions.</p> \u000A \u000A <p>You might formalize this insight as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))\u000A       (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))</code> \u000A \u000A <p>But this conjecture is <i>not</i> a theorem, because <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)</v> \u000A returns the <v>cdr</v> of <v>x</v> that begins with <v>e</v>, not just a Boolean \u000A (<v>t</v> or <v>nil</v>) indicating whether <v>e</v> is an element of <v>x</v>.  The \u000A location of the first <v>e</v> in <v>(rev x)</v> is generally different than the \u000A location in <v>x</v>.  So when we say the two questions are ``equivalent'' we \u000A don't mean they are equal.  We mean that they're propositionally equivalent: \u000A both <v>nil</v> or both non-<v>nil</v>.  This sense of equivalence is called ``if \u000A and only if'' and is checked by the function <v>iff</v>.</p> \u000A \u000A <p>So our intuitive insight can be phrased as this theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))\u000A     (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))</code> \u000A \u000A <p>Suggesting that this formulation of the insight is ``obvious'' begs many \u000A questions.  Mathematically, we could have avoided <v>iff</v> and just written \u000A two implications:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x)))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x)) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))).</code> \u000A \u000A <p>or</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x)))\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))))).</code> \u000A \u000A <p>Or we could have used <v>iff</v> but ``oriented'' it the other way:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A     (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x)))</code> \u000A \u000A <p>We choose to write</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))\u000A     (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))</code> \u000A \u000A <p>because of <i>our knowledge of how ACL2 turns formulas into rules!</i></p> \u000A \u000A <p>We deal with this at greater length later.  But just to drive the point \u000A home, if we issue the command:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-rev\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))\u000A       (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)))</code> \u000A \u000A <p>ACL2 will build in a rule that causes every propositional occurrence of \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (REV x))</v> to be replaced by <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e x)</v>.  (By \u000A ``propositional occurrence'' we mean an occurrence in which the value is \u000A tested, as by <v>IF</v> or the propositional connectives.  Remember, one might \u000A use <v>member</v> to determine the location of an element too.)</p> \u000A \u000A <p>Note carefully: <i>if you do not tell ACL2 how to make a rule</i> from a \u000A theorem, it makes a rewrite rule.  Rewrite rules always replace instances of \u000A the left-hand side by the corresponding instances of the right-hand side. \u000A That is, when interpreted as a rewrite rule, <v>(<see topic=\"ACL2____IFF\">iff</see> </v><i>alpha</i> \u000A <i>beta</i><v>)</v> makes ACL2 replace <i>alpha</i> by <i>beta</i>.</p> \u000A \u000A <p>Probably the biggest mistake new users make is forgetting that every \u000A theorem they prove creates a very specific rule.  You must remember that you \u000A are <i>programming</i> ACL2 with these rules.  Being careless in your \u000A statement of theorems is tantamount to being careless in your programming. \u000A What you get is a mess.</p> \u000A \u000A <p>Had we proved the same equivalence, but with the <v>iff</v> commuted, we \u000A would be giving ACL2 <i>bad advice</i>.  We would be telling it ``replace \u000A instances of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e x)</v> by the corresponding instances of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e\u000A (REV x))</v>''!  If ACL2 had that rule and ever tried to simplify any \u000A <v>member</v> expression, e.g., <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A B)</v>, it would get into an infinite \u000A loop, e.g., producing the following sequence of transformations:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A B)\u000A(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A (REV B))\u000A(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A (REV (REV B)))\u000A...</code> \u000A \u000A <p>until it eventually exhausted some resource.</p> \u000A \u000A <p>Recall that we entertained the idea of phrasing our insight about \u000A <v>member</v> and <v>rev</v> with implications rather than <v>iff</v>.  Generally \u000A speaking, implications produce weaker rules -- rules that apply less often. \u000A We discuss that later.</p> \u000A \u000A <p>Now suppose we've proved <v>member-rev</v>, oriented so as to rewrite \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev x))</v> to <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)</v>, and built it in as a rewrite \u000A rule.  Then suppose we repeated the attempt to prove our Main Theorem.  This \u000A time, when the prover is processing the hideous Key Checkpoint printed above, \u000A our new lemma, <v>member-rev</v>, will hit it.  It will transform the formula \u000A to:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I)) X)   ; &lt;-- the hyp has simplified\u000A              (LIST-OF-INTEGERS X)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> I)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 I)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> K)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 K)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> I K)\u000A              (SQUARES-PLUS-3P K X)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I))))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 (<see topic=\"COMMON-LISP_____A2\">*</see> I I)) X)))\u000A         (SQUARES-PLUS-3P K (REV X)))?</code> \u000A \u000A <p>and then that will collapse to <v>T</v>, since the <v>IMPLIES</v> has \u000A contradictory hypotheses (note the last hypothesis above).</p> \u000A \u000A <p>By proving <v>member-rev</v> we proved the hideous checkpoint.  We never had \u000A to look at the rest of the formula or think about why it is a theorem. \u000A Furthermore, attacking the main theorem again, from scratch, with \u000A <v>member-rev</v> in the database, may eliminate other checkpoints that came up \u000A the last time we tried to prove our main goal.  So we recommend addressing one \u000A checkpoint at a time.</p> \u000A \u000A <p>This example illustrates that purely <i>local</i> thinking -- looking for \u000A simplifiable combinations of function symbols -- can sometimes lead to proofs \u000A and should always be your first reaction to a key checkpoint: what local fact \u000A do you know that would clean up the formula?  Don't think about deep questions \u000A like ``why is this true?''  until you can't see any way to make it \u000A simpler.</p> \u000A \u000A <p>It is important to train yourself to see combinations of function symbols \u000A and to create strong rules for eliminating them.  We will give you \u000A opportunities to practice this later in the tutorial.</p> \u000A \u000A <p>If you have been reading the tutorial introduction to the theorem prover, \u000A use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p>"],

 "ACL2____BOUNDERS":[["Tau-system"],"ACL2 Sources","ACL2","<code><i>Bounder Forms 1 and 2</i>: \u000A(implies (and (tau-intervalp i1) \u000A              ... \u000A              (or (equal (tau-interval-dom i1) 'dom1-1) \u000A                  ...) \u000A              ... \u000A              (in-tau-intervalp x1 i1) \u000A              ...) \u000A         (and (tau-intervalp (bounder-fn i1 ...)) \u000A              (in-tau-intervalp <i>target</i> \u000A                                (bounder-fn i1 ...)))) \u000A</code> \u000A \u000A <p>where <i>target</i> is either <v>(fn x1 ... y1 ...)</v> or <v>(<see topic=\"ACL2____MV-NTH\">mv-nth</see> 'n (fn\u000A x1 ... y1 ...))</v>, depending on whether we are in the <i>Form 1</i> or <i>Form \u000A 2</i> case, respectively.  However, the shape above is meant just as a \u000A reminder.  Details are given below.</p> \u000A \u000A <p>This topic first explains the basic shape of <i>Bounder Form 1</i>.  Then \u000A it illustrates <i>Bounder Form 2</i>.  Finally, it deals briefly with proving \u000A bounder correctness theorems.  The community book \u000A <v>tau-bounders/elementary-bounders</v> contains bounders for various elementary \u000A functions including <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt>, <tt><see topic=\"COMMON-LISP_____A2\">*</see></tt>, <tt><see topic=\"COMMON-LISP_____F2\">/</see></tt>, <tt><see topic=\"COMMON-LISP____FLOOR\">floor</see></tt>, <tt><see topic=\"COMMON-LISP____MOD\">mod</see></tt>, <tt><see topic=\"COMMON-LISP____LOGAND\">logand</see></tt>, <tt><see topic=\"COMMON-LISP____LOGNOT\">lognot</see></tt>, <tt><see topic=\"COMMON-LISP____LOGIOR\">logior</see></tt>, <tt><see topic=\"COMMON-LISP____LOGORC1\">logorc1</see></tt>, <tt><see topic=\"COMMON-LISP____LOGEQV\">logeqv</see></tt>, <tt><see topic=\"COMMON-LISP____LOGXOR\">logxor</see></tt>, and <tt><see topic=\"COMMON-LISP____ASH\">ash</see></tt>.  You might look at or include this \u000A book to see more example theorems, to see how proofs of such theorems are \u000A managed, and to experiment with their effects on proving theorems involving \u000A arithmetic over finite or half-finite intervals.</p> \u000A \u000A <p>A bounder correctness theorem establishes that <v>bounder-fn</v> is a \u000A ``bounder'' for the function <v>fn</v>.  That means that when trying to compute \u000A a tau for a call of <v>fn</v> (or, in the case of <i>Form 2</i>, for the \u000A <v>n</v>th component of the multiple-value vector returned by a call of <v>fn</v>) \u000A the tau system can call <v>bounder-fn</v> on the intervals containing certain \u000A arguments of <v>fn</v>.</p> \u000A \u000A <p>Let us start with an example.  Let <v>fn</v> be the addition function, <v>+</v> \u000A (actually, <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>).  Consider the target term <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y)</v> and \u000A contemplate the question: if you know intervals containing <v>x</v> and <v>y</v>, \u000A say <v>intx</v> and <v>inty</v> respectively, what is an interval containing their \u000A sum?  The answer is pretty easy to state in English: the domain of the answer \u000A interval is the less restrictive of the domains of <v>intx</v> and <v>inty</v>. \u000A The lower bound of the answer interval is the sum of the lower bounds of \u000A <v>intx</v> and <v>inty</v>, and the lower relation is the stronger of the lower \u000A relations of <v>intx</v> and <v>inty</v>.  Analogous comments define the upper \u000A bound and relation of the answer interval.  So for example, if <v>x</v> is an \u000A <v>INTEGERP</v> such that <v>0 &lt;= x &lt;= 10</v> and <v>y</v> is a <v>RATIONALP</v> such \u000A that <v>0 &lt; y &lt;= 20</v>, then <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x y)</v> is a <v>RATIONALP</v> such that <v>0 &lt;\u000A (<see topic=\"COMMON-LISP_____B2\">+</see> x y) &lt;= 30</v>.</p> \u000A \u000A <p>Defining this precisely is more tedious than describing it in English \u000A because one must make precise the notions of ``less restrictive'' domains, \u000A ``weaker'' relations, and the possibility that either or both of the bounds \u000A could be ``infinite.''  But we can easily imagine defining the function \u000A <v>bounder-for-+</v> that returns the answer interval described, given <v>intx</v> \u000A and <v>inty</v>.</p> \u000A \u000A <p>Then the following <i>Bounder Form 1</i> formula establishes the \u000A correctness of <v>bounder-for-+</v> and allows the tau system to use it to \u000A produce bounds in the tau computed for <v>+</v>-expressions:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> intx)\u000A              (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> inty)\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> x intx)\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> y inty))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (bounder-for-+ intx inty))\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y)\u000A                                (bounder-for-+ intx inty))))</code> \u000A \u000A <p>For example, suppose we have a formula with the following hypotheses</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a)\u000A     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 a)\u000A     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> a 10)\u000A     (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> b)\u000A     (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 b)\u000A     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> b 20))</code> \u000A \u000A <p>and suppose the tau system encounters the term <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b)</v>.  When the term \u000A is enountered, the tau for <v>a</v> would include an <v>INTEGERP</v> interval such \u000A that <v>0 &lt;= a &lt;= 10</v> and the tau for <v>b</v> would include a <v>RATIONALP</v> \u000A interval such that <v>0 &lt; b &lt;= 20</v>.  In its most primitive configuration, the \u000A tau system would only know that the tau for <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b)</v> includes the \u000A recognizer <v>RATIONALP</v> (and all that it is known to imply).  But after the \u000A bounder theorem above is proved and available as a <v>:tau-system</v> rule the \u000A tau system would infer that <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b)</v> was in the <v>RATIONALP</v> interval \u000A such that <v>0 &lt; (<see topic=\"COMMON-LISP_____B2\">+</see> a b) &lt;= 30</v>.</p> \u000A \u000A <p>Thus, by defining bounder functions and proving them correct the user can \u000A give the tau system the ability to compute the bounds on function calls as a \u000A function of the known bounds on their actuals.</p> \u000A \u000A <p>It is sometimes useful to restrict the domains of the intervals to be \u000A considered.  For example, in bounding <v>*</v>-expressions it is simplifying to \u000A restrict one's attention to intervals over the integers or rationals (and thus \u000A exclude the complex rationals so one need not think about the getting negative \u000A bounds by multiplying two ``positive'' complex rationals or how to ``round \u000A up'' from complex bounds to the rationals required by our intervals).</p> \u000A \u000A <p>If we were to define <v>bounder-for-*</v> so that it works correctly to bound \u000A <v>*</v>-expressions, but only for integer or rational arguments, its \u000A correctness theorem would be:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> intx)                             ; (a)\u000A              (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> inty)\u000A              (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> intx) 'INTEGERP)    ; (b)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> intx) 'RATIONALP))\u000A              (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> inty) 'INTEGERP)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> inty) 'RATIONALP))\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> x intx)                        ; (c)\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> y inty))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (bounder-for-* intx inty))       ; (d)\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y)                        ; (e)\u000A                                (bounder-for-* intx inty))))</code> \u000A \u000A <p>In this case, <v>bounder-for-*</v> would be applied to the intervals for \u000A <v>x</v> and <v>y</v> only if those intervals were over the integers or the \u000A rationals.</p> \u000A \u000A <p>The above theorem for <v>bounder-for-*</v> begins to suggest the general form \u000A of a bounder theorem and we will use it to explain the general form.</p> \u000A \u000A <p>The hypotheses of a bounder theorem must be a conjunction and the conjuncts \u000A must be partitionable into three parts, (a), (b), and (c).  The conclusion, \u000A must be a conjunction, must contain at least two conjuncts, (d) and (e), and \u000A is allowed to contain others that are simply ignored for purposes of bounders. \u000A (See the note below about why we allow but ignore additional conjuncts in the \u000A conclusion.)</p> \u000A \u000A <p>Part (a) introduces some distinct ``interval variables,'' here called \u000A ``ivars,'' that are known to denote intervals; for the example above, the \u000A ivars are <v>intx</v> and <v>inty</v>.  Each hypothesis in part (a) is of the form \u000A <v>(<see topic=\"ACL2____TAU-INTERVALP\">TAU-INTERVALP</see> ivar)</v>.</p> \u000A \u000A <p>Part (b) allows us to restrict the domains of some of the intervals.  Each \u000A hypothesis in part (b) must be a disjunction and each of the disjuncts must be \u000A of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">TAU-INTERVAL-DOM</see> ivar) 'dom)</v>, where <v>ivar</v> is one \u000A of the interval variables and <v>dom</v> is one of <v>INTEGERP</v>, \u000A <v>RATIONALP</v>, <v>ACL2-NUMBERP</v>, or <v>NIL</v>.  It is not necessary to \u000A restrict every interval variable.  Indeed, part (b) may be empty, as in the \u000A theorem for <v>bounder-for-+</v> above.</p> \u000A \u000A <p>Part (c) consists of a set of <v>(<see topic=\"ACL2____IN-TAU-INTERVALP\">IN-TAU-INTERVALP</see> avar ivar)</v> hypotheses \u000A where each <v>avar</v> is a variable and no two hypotheses in part (c) use the \u000A same <v>avar</v> or <v>ivar</v>.  We call the set of all such <v>avar</v> the \u000A ``actual variables'' or ``avars.''  The avars and ivars must be distinct. \u000A Part (c) sets up a correspondence between the avars and the ivars, each avar \u000A is in an interval denoted by one ivar.</p> \u000A \u000A <p>Part (d) introduces the name of the bounder function, here \u000A <v>bounder-for-*</v>, and the order of its ivar arguments.  We see that \u000A <v>bounder-for-*</v> takes two arguments and they correspond, in order, to the \u000A intervals containing <v>x</v> and <v>y</v>.  Part (d) also establishes that the \u000A bounder function always returns an interval under hypotheses (a), (b), and \u000A (c).  Note that it is sometimes useful to return the ``universal interval'' \u000A (one that contains everything) if you don't want to compute a better interval \u000A for some case; see <tt><see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see></tt> or <tt><see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see></tt>.</p> \u000A \u000A <p>Part (e) introduces the name of the function being bounded, here <v>*</v>, \u000A and the order of its arguments.  It establishes that the function being \u000A bounded really is bounded by the interval computed by the bounder function. \u000A In general, the function being bounded may take additional arguments.  It is \u000A possible that the function being bounded takes some arguments that do not \u000A affect the bounds of its output.</p> \u000A \u000A <p>Thus, parts (c) and (e) together establish a mapping between the actuals of \u000A a call of the function being bounded and the intervals to be supplied to the \u000A bounder.</p> \u000A \u000A <p>The parts identified above may be presented in any order and the literals \u000A constituting those parts may be mingled.  Thus, for example, here is another \u000A version of the theorem above that generates the same bounding information for \u000A the tau system.  In this version, the hypotheses and conclusions are \u000A rearranged, <v>bounder-for-*</v> takes its arguments in the opposite order, and \u000A the theorem includes an additional conclusion.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> intx)                             ; (a)\u000A              (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> intx) 'INTEGERP)    ; (b)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> intx) 'RATIONALP))\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> x intx)                        ; (c)\u000A\u000A              (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> inty)                             ; (a)\u000A              (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> inty) 'INTEGERP)    ; (b)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> inty) 'RATIONALP))\u000A              (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> y inty))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y)                        ; (e)\u000A                                (bounder-for-* inty intx))\u000A              (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (bounder-for-* inty intx))        ; (d)))\u000A\u000A              (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> (bounder-for-* inty intx))\u000A                         'INTEGERP)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> (bounder-for-* inty intx))\u000A                         'RATIONALP))</code> \u000A \u000A <p><i>Note on why bounder forms allow additional conjuncts in the \u000A conclusion</i>: It is often the case that one creates bounders by composing \u000A other bounders.  To prove compositional bounds correct one must often prove \u000A more than the mere correctness of the components.  For example, one might need \u000A to prove that the domain of the new bounding interval is <v>INTEGERP</v> or \u000A otherwise restricted.  We allow such ``unnecessary'' conclusions simply to \u000A save the user the burden of stating multiple theorems.</p> \u000A \u000A <p><i>An Illustration of Bounder Form 2</i>: Suppose <v>(quad i)</v> is defined \u000A so that truncates the integer <v>i</v> to the largest multiple of 4 weakly below \u000A <v>i</v> and, additionally, returns the remainder.  For example, <v>(quad 26)</v> \u000A returns <v>(<see topic=\"ACL2____MV\">mv</see> 24 2)</v>.  Then here are bounders for each of its return \u000A values:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> quad-bounds-0 (i)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see> i)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 (<see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see> i)))\u000A         (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'integerp nil 0 nil (<see topic=\"ACL2____TAU-INTERVAL-HI\">tau-interval-hi</see> i)))\u000A        (t (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> nil nil nil nil nil))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> quad-bounds-1 (i)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see> i)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 (<see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see> i)))\u000A         (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'integerp nil 0 nil 3))\u000A        (t (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> nil nil nil nil nil))))</code> \u000A \u000A <p>Note that the bounders assume <v>i</v> is an <v>INTEGERP</v> and return the \u000A universal interval when <v>i</v> is not a natural.</p> \u000A \u000A <p>As noted in the discussion below about how to prove bounder correctness \u000A theorems, proving these bounders correct will require an arithmetic book, \u000A e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-5/top&quot; :dir :system)</code> \u000A \u000A <p>Here then are two bounder correctness theorems of <i>Form 2</i>:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> quad-bounds-0-correct\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> i)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> i) 'INTEGERP)\u000A                (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> x i))\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (quad-bounds-0 i))\u000A                (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 0 (quad x))\u000A                                  (quad-bounds-0 i))))\u000A  :rule-classes :tau-system)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> quad-bounds-1-correct\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> i)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> i) 'INTEGERP)\u000A                (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> x i))\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (quad-bounds-1 i))\u000A                (<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (quad x)) (quad-bounds-1 i))))\u000A  :rule-classes :tau-system)</code> \u000A \u000A <p>As noted above, if these bounders are to be used in constructing other \u000A bounders, we might include (in the first theorem) an additional concluding \u000A conjunct, such as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> (quad-bounds-0 i)) 'INTEGERP)</code> \u000A \u000A <p>so that we can keep <v>quad-bounds-0</v> disabled to allow us to use \u000A <v>quad-bounds-0-correct</v> as a <v>:rewrite</v> or other rule and still relieve \u000A hypotheses about the domain of the interval it produces.  These hypotheses \u000A would arise if some other verified bounder was called on the produced \u000A interval.  In addition, as noted below, we might replace the \u000A <v>:rule-classes</v> above with</p> \u000A \u000A <code>:rule-classes\u000A ((:rewrite)\u000A  (:forward-chaining :trigger-terms ((quad-bounds-0 i))))</code> \u000A \u000A <p>Since the theorem is being stored as some kind of rule and since it \u000A satisfies the <i>Bounder Form 2</i> shape, it will additionally be stored as a \u000A <v>:tau-system</v> rule.</p> \u000A \u000A <p><i>Note on proving bounder theorems</i>: Proving bounder theorems is just \u000A like proving any other arithmetic theorem and you will need whatever libraries \u000A are appropriate for the problem domain you are working in.  Do not expect the \u000A tau system to be of much use in proving bounder theorems.  A typical bounder \u000A theorem might require you to prove a subgoal like <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (fn x y) (g\u000A (<see topic=\"ACL2____TAU-INTERVAL-HI\">tau-interval-hi</see> int1) int2))</v>.  But tau deals with inequalities relating \u000A terms to constants, e.g., <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> ... 16)</v>.  A bounder theorem is a sort of \u000A ``metatheorem'' about <i>how to construct</i> bounded intervals from other \u000A bounded intervals.  So when you undertake to define a bounder and prove it \u000A correct, go into the project with your eyes open!</p> \u000A \u000A <p>But bounder functions can be broadly divided into two classes, those \u000A defined in terms of arithmetic on the interval bounds and those defined in \u000A terms of other bounders.  For example, given that</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LOGXOR\">LOGXOR</see> x y) = (<see topic=\"COMMON-LISP____LOGNOT\">LOGNOT</see> (<see topic=\"COMMON-LISP____LOGEQV\">LOGEQV</see> x y))</code> \u000A \u000A <p>an interval for bounding <v>LOGXOR</v> can be constructed by composing the \u000A constructions of intervals for <v>LOGEQV</v> and <v>LOGNOT</v>.  So some bounder \u000A correctness proofs will involve direct manipulation of arithmetic inequalities \u000A and others might involve appeal to the correctness of other bounders, \u000A depending on how the new bounder is defined.</p> \u000A \u000A <p>Regardless of which style of bounder we are dealing with, we have found it \u000A useful to prove the basic theorems relating the tau interval accessors to \u000A <tt><see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see></tt>, e.g.,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) dom)</code> \u000A \u000A <p>and then disable those functions to avoid seeing excessive <v>car</v>s and \u000A <v>cdr</v>s.</p> \u000A \u000A <p>When dealing with bounders defined in the direct, arithmetic style, we tend \u000A to keep <tt><see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see></tt> and <tt><see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see></tt> enabled so they \u000A unfold and expose the algebra.</p> \u000A \u000A <p>When dealing with bounders defined compositionally in terms of other \u000A verified bounders, we tend to keep <tt><see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see></tt> and <tt><see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see></tt> disabled so we can rely on the previously proved bounder \u000A theorems as rewrite and forward chaining rules.</p> \u000A \u000A <p>Note that this last remark means that when you prove bounder correctness \u000A theorems you should include corollaries that are useful <v>:rewrite</v> and \u000A possibly <v>:forward-chaining</v> rules if you anticipate using that bounder in \u000A more complex ones.  We tend to trigger the forward chaining with the bounder \u000A expression itself, rather than one of the hypotheses.  For example in the rule \u000A above for <v>bounder-for-*</v> we would include <v>(:forward-chaining\u000A :trigger-terms ((tau-bounder-expt2 int2)))</v> and let the <v>in-tau-intervalp</v> \u000A hypotheses select the free variables <v>x</v> and <v>y</v>.</p>"],

 "ACL2____BOOKS-CERTIFICATION-ALT":[["Books-certification"],"ACL2 Sources","ACL2","<p>For background on the ACL2 community books, see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  Here we explain how to certify those books, or some of \u000A those books, with ACL2.  We thank Bishop Brock, Jared Davis, and Sol Swords \u000A for their substantial contributions to this methodology.  See \u000A <v>books/GNUmakefile</v>, in the community books, for more about ``Credits and \u000A History'', and for additional technical details not covered in this topic.</p> \u000A \u000A <p>For more information about installing ACL2, see the <a href=\"http://www.cs.utexas.edu/users/moore/acl2/v6-5/HTML/installation/installation.html\">ACL2 installation instructions</a>.  For information \u000A about so-called ``classic ACL2 `make'-based certification'', which provides \u000A support for certifying directories of books but may disappear in a future ACL2 \u000A release, see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>.</p> \u000A \u000A <p><b>The Basics</b></p> \u000A \u000A <p>We make the following assumptions.</p> \u000A \u000A <ul> \u000A  <li>Gnu `make' is available on your system via the `make' command \u000A (rather than some other flavor of `make').  (Execute `<v>make --version</v> to \u000A verify this.)</li> \u000A \u000A  <li>You have built or obtained an ACL2 executable.</li> \u000A \u000A  <li>The ACL2 <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> are installed in the <v>books/</v> subdirectory of \u000A your ACL2 distribution, as is the case when you have followed the standard \u000A installation instructions.</li> \u000A </ul> \u000A \u000A <box><p>Note: All commands shown below are issued in the top-level (ACL2 \u000A sources) directory of your ACL2 distribution.</p></box> \u000A \u000A <p>By default the ACL2 executable is file <v>saved_acl2</v> in your ACL2 sources \u000A directory, and you can issue the following command to the shell in order to do \u000A a ``regression run'' that certifies all of the community books using that \u000A executable.</p> \u000A \u000A <code>make regression</code> \u000A \u000A <p>Better yet, save a log file in case there are problems, for example as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE\">make</see> regression) &gt;&amp; make-regression.log</code> \u000A \u000A <p>or perhaps better yet:</p> \u000A \u000A <code>(time nice make regression) &gt;&amp; make-regression.log</code> \u000A \u000A <p>For the sake of brevity, below we'll skip mentioning any of `<v>time</v>', \u000A `<v>nice</v>', or `<v>&gt;&amp; make-regression.log</v>'.  But saving a log file, in \u000A particular, is useful in case you encounter problems to report.</p> \u000A \u000A <p>If you fetched the community books using svn, then you will have a \u000A directory <v>books/workshops/</v> that is not necessary for certifying the other \u000A books.  If you want to skip certification of the books under \u000A <v>books/workshops/</v>, use target `<v>certify-books</v>' instead of target \u000A `<v>regression</v>', for example as follows.</p> \u000A \u000A <code>(time nice make certify-books) &gt;&amp; make-certify-books.log</code> \u000A \u000A <p>Whether you use target `<v>regression</v>' or target `<v>certify-books</v>', \u000A then for each book <v>foo.lisp</v> whose certification is attempted, a file \u000A <v>foo.cert.out</v> in the same directory will contain the output from the \u000A book's certification attempt.</p> \u000A \u000A <p>A regression run may take a few hours, but if you have a multiprocessing \u000A computer, you can speed it up by certifying some books in parallel, by \u000A providing a value for `make' option <v>-j</v>.  For example, if you have 8 \u000A hardware threads then you might want to issue the following command.</p> \u000A \u000A <code>make regression -j 8</code> \u000A \u000A <p><b>Specifying the ACL2 Executable</b></p> \u000A \u000A <p>If your ACL2 executable is not file <v>saved_acl2</v> in the ACL2 sources \u000A directory, then you will need to specify that executable.  You can do that by \u000A setting variable <v>ACL2</v>, either as an environment variable or, as displayed \u000A below, as a `make' variable.  Either way, you will need to avoid relative \u000A pathnames.  For example, the first two forms below are legal, but the third is \u000A not, assuming that <v>my-acl2</v> is on your <v>PATH</v> in a Unix-like \u000A environment (e.g., linux or MacOS) and that <v>my-saved_acl2</v> is just a \u000A pathname relative to your ACL2 sources directory, which is not on your \u000A path.</p> \u000A \u000A <code>make regression -j 8 ACL2=my-acl2\u000Amake regression -j 8 ACL2=/u/smith/bin/acl2\u000A# The following only works if my-saved_acl2 is on your path (see above).\u000Amake regression -j 8 ACL2=my-saved_acl2</code> \u000A \u000A <p><b>Cleaning</b></p> \u000A \u000A <p>You can delete files generated by book certification (including <v>.cert</v> \u000A files, <v>.out</v> files, compiled files, and more) by issuing the following \u000A command (again, in your ACL2 sources directory).</p> \u000A \u000A <code>make clean-books</code> \u000A \u000A <p>If you want to cause such deletion and then do a regression, simply replace \u000A the `<v>regression</v>' or `<v>certify-books</v>' target by \u000A `<v>regression-fresh</v>' or `<v>certify-books-fresh</v>', respectively, for \u000A example as follows.  follows.</p> \u000A \u000A <code>make -j 4 regression-fresh\u000Amake -j 4 certify-books-fresh</code> \u000A \u000A <p>If however you only want to clean up generated files residing under a given \u000A directory (or its subdirectories, and recursively), you can issue the \u000A following command while standing in that directory, where <v>DIR</v> is a \u000A pathname of your <v>books</v> directory.</p> \u000A \u000A <code>DIR/clean.pl</code> \u000A \u000A <p>For example, to clean up generated files under <v>books/arithmetic</v>, you \u000A could do the following.</p> \u000A \u000A <code>cd books/arithmetic\u000A../clean.pl\u000Acd - # to return to the ACL2 sources directory, if you wish to do so</code> \u000A \u000A <p><b>Restricting to Specific Directories and Books</b></p> \u000A \u000A <p>You can specify which books you want certified by using any or all of the \u000A variables <v>EXCLUDED_PREFIXES</v>, <v>ACL2_BOOK_CERTS</v>, or \u000A <v>ACL2_BOOK_DIRS</v>.  First, the set of desired <v>.cert</v> files is restricted \u000A to those that do not start with any string that is one of the words in the \u000A value of <v>EXCLUDED_PREFIXES</v>.  Then <v>ACL2_BOOK_CERTS</v> and \u000A <v>ACL2_BOOK_DIRS</v>, if supplied, specify which books should be certified, as \u000A illustrated by the following example.</p> \u000A \u000A <code>make -j 8 regression-fresh \\\u000A ACL2_BOOK_DIRS=&quot;symbolic paco&quot; \\\u000A ACL2_BOOK_CERTS=&quot; \\\u000A  workshops/2006/cowles-gamboa-euclid/Euclid/ed6a.cert \\\u000A  workshops/2006/cowles-gamboa-euclid/Euclid/ed4bb.cert \\\u000A  &quot;</code> \u000A \u000A <p>Then all book in directories <v>symbolic</v> and <v>paco</v> will be certified, \u000A as will the books <v>workshops/2006/cowles-gamboa-euclid/Euclid/ed6a.lisp</v> \u000A and <v>workshops/2006/cowles-gamboa-euclid/Euclid/ed4bb.lisp</v>.  Note that all \u000A pathnames should be relative to your community books directory; in particular, \u000A they should not be absolute pathnames.  Also notice the <v>.cert</v> extension \u000A used in files supplied for <v>ACL2_BOOK_CERTS</v>.</p> \u000A \u000A <p>Alternatively, you may wish to invoke <v>books/cert.pl</v> while standing in \u000A a directory under which you want to certify books.  This will certify not only \u000A those books, but all supporting books &mdash; even those not under the current \u000A directory &mdash; that do not have up-to-date <v>.cert</v> files.  The following \u000A is a simple command to invoke that will certify all books in the current \u000A directory, where if the <v>books/</v> directory is not on your path, you will \u000A need to provide a suitable filename, e.g. <v>../../cert.pl</v> or \u000A <v>~/acl2/books/cert.pl</v>.</p> \u000A \u000A <code>cert.pl -j 4 *.lisp</code> \u000A \u000A <p>Here is a more complex command, which illustrates a way to certify books in \u000A subdirectories (as well as the current directory), the use of provisional \u000A certification (see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>), and `make'-level \u000A parallelism (in this case specifying four parallel processes).</p> \u000A \u000A <code>ACL2_PCERT=t cert.pl -j 4 `find . -name '*.lisp'`</code> \u000A \u000A <p>Note that with this approach, unlike classic ACL2 `make'-based \u000A certification (see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>, out-of-date <v>.cert</v> \u000A files that are not under the current directory will also be built.  For \u000A documentation of <v>cert.pl</v> invoke:</p> \u000A \u000A <code>cert.pl -h</code> \u000A \u000A <p>See the top of <v>cert.pl</v> for authorship and copyright information.</p> \u000A \u000A <p>Finally, we give a brief summary of how to use so-called ``classic ACL2 \u000A `make'-based certification'' for community books; see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see> for details.  Note that support for this approach \u000A might be eliminated in a future ACL2 release.  We welcome comments from the \u000A ACL2 community about whether or not that would be a good thing to do.  See the \u000A discussion above about <v>ACL2_BOOK_DIRS</v> for the ``modern'' way to \u000A accomplish the same thing.</p> \u000A \u000A <p>Many community book directories have a <v>Makefile</v>.  If you modify books \u000A only in such a directory, you can recertify by standing in that directory and \u000A issuing a `make' command.  This command can optionally specify an ACL2 \u000A executable as well as parallelism, for example as follows, where the first \u000A line (<v>make clean</v>) is optional.</p> \u000A \u000A <code>make clean\u000A(time nice make -j 8 ACL2=my-acl2)</code> \u000A \u000A <p><b>ACL2 Customization Files</b></p> \u000A \u000A <p>By default, your acl2-customization file (see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>) is \u000A ignored by all flavors of ``<v>make regression</v>''.  However, you can specify \u000A the use of an acl2-customization file by setting the value of environment \u000A variable <v>ACL2_CUSTOMIZATION</v> to the empty string, indicating a default \u000A such file, or to the desired absolute pathname.  For example:</p> \u000A \u000A <code>make regression ACL2_CUSTOMIZATION=''\u000Amake regression ACL2_CUSTOMIZATION='~/acl2-customization.lisp'</code> \u000A \u000A <p><b>Regressions for Experimental Extensions of ACL2</b></p> \u000A \u000A <p>The instructions are unchanged if you are using ACL2(h) (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).  However, note that the default executable (when \u000A <v>ACL2</v> is not specified) remains <v>saved_acl2</v> in your ACL2 sources \u000A directory, not <v>saved_acl2h</v> as one might expect for ACL2(h)(p), \u000A respectively.  So probably you'll want to supply ``<v>ACL2=&lt;your_acl2&gt;</v>'' \u000A explicitly with your `make' command.</p> \u000A \u000A <p>The comments above also pertain pertain to using ACL2(p) (see <see topic=\"ACL2____PARALLEL\">parallel</see>); the default is <v>saved_acl2</v> rather than <v>saved_acl2p</v>. \u000A However, we recommend that you use ACL2, not ACL2(p), for your regression. \u000A Then you can use ACL2(p) for your own proof developments.  (The analogous \u000A comment applies to ACL2(hp), which combines capabilities of ACL2(h) and \u000A ACL2(p), i.e., we recommend that you use ACl2(h) for your regression in that \u000A case.)  However, if you want to use ACL2(p) or ACL2(hp) for your regression, \u000A see <see topic=\"ACL2____WATERFALL-PARALLELISM-FOR-BOOK-CERTIFICATION\">waterfall-parallelism-for-book-certification</see>.</p> \u000A \u000A <p>If you intend to certify books in the <v>nonstd</v> subdirectory of the \u000A community books, you will want to use ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).  In that \u000A case, you will probably specify use of the default executable (i.e., when \u000A <v>ACL2</v> is not specified), which is <v>saved_acl2r</v> in your ACL2 sources \u000A directory, rather than <v>saved_acl2</v>.</p> \u000A \u000A <p><b>Provisional Certification</b></p> \u000A \u000A <p>To use provisional certification (see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>), \u000A supply <v>ACL2_PCERT=t</v> with your `make' command.  Here is an example.</p> \u000A \u000A <code>time nice make regression -j 4 ACL2_BOOK_DIRS=deduction ACL2_PCERT=t</code> \u000A \u000A <p><b>Miscellany</b></p> \u000A \u000A <p>Other control of the certification process may be found by perusing \u000A community books file <v>books/make_cert</v>.  In particular, the <v>INHIBIT</v> \u000A variable may be set to a call of <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>, for example \u000A as follows to obtain the output one would get by default in an (interactive) \u000A ACL2 session.</p> \u000A \u000A <code>time nice make regression -j 4 ACL2_BOOK_DIRS=arithmetic \\\u000A  INHIBIT='(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> proof-tree)'</code> \u000A \u000A <p><b>Troubleshooting</b></p> \u000A \u000A <p>If you run into problems, you can get help by joining the <v>acl2-help</v> \u000A email list (follow the link from the ACL2 home page) and sending a message to \u000A that list.  Also consider trying another version of GNU `make'; for example, \u000A we have found that versions 3.81 and 3.82 sometimes cause errors on Linux \u000A where version 3.80 does not.  Note however that Version 3.80 does not print \u000A certain informational messages that are printed by later versions.</p>"],

 "ACL2____BOOK-EXAMPLE":[["Books-tour"],"ACL2 Sources","ACL2","<p>Suppose you have developed a sequence of admissible <see topic=\"ACL2____EVENTS\">events</see> \u000A which you want to turn into a book.  We call this ``publishing'' the book. \u000A This note explains how to do that.</p> \u000A \u000A <p>A key idea of <see topic=\"ACL2____BOOKS\">books</see> is that they are ``incremental'' in the sense \u000A that when you include a book in a host logical <see topic=\"ACL2____WORLD\">world</see>, the <see topic=\"ACL2____WORLD\">world</see> \u000A is incrementally extended by the results established in that book.  This is \u000A allowed only if every name defined by the incoming book is either new or is \u000A already identically defined.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.  This is exactly \u000A the same problem faced by a programmer who wishes to provide a utility to \u000A other people: how can he make sure he doesn't create name conflicts?  The \u000A solution, in Common Lisp, is also the same: use packages.  While <see topic=\"ACL2____BOOKS\">books</see> \u000A and packages have a very tenuous formal connection (every book must start with \u000A an <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt>), the creation of a book is intimately concerned with \u000A the package issue.  Having motivated what would otherwise appear as an \u000A unnecessary fascination with packages below, we now proceed with a description \u000A of how to publish a book.</p> \u000A \u000A <p>Just to be concrete, let's suppose you have already gotten ACL2 to accept \u000A the following sequence of <see topic=\"ACL2____COMMAND\">command</see>s, starting in the ACL2 initial <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;ACL2-MY-BOOK&quot;\u000A        (<see topic=\"ACL2____UNION-EQ\">union-eq</see> *common-lisp-symbols-from-main-lisp-package*\u000A                  *acl2-exports*))\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2-MY-BOOK&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)) y))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (app (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))) nil))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rev-app-hack\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (app a (<see topic=\"COMMON-LISP____LIST\">list</see> x))) (<see topic=\"COMMON-LISP____CONS\">cons</see> x (rev a))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rev-rev\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">acl2::true-listp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev x)) x)))</code> \u000A \u000A <p>Observe that the first form above defines a package (which imports the \u000A symbols defined in CLTL such as <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> and <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> and the symbols \u000A used to <see topic=\"ACL2____COMMAND\">command</see> ACL2 such as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>).  The \u000A second form selects that package as the current one.  All subsequent forms are \u000A read into that package.  The remaining forms are just event forms: <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>s in this case.</p> \u000A \u000A <p>Typically you would have created a file with Emacs containing these forms \u000A and you will have submitted each of them interactively to ACL2 to confirm that \u000A they are all admissible.  That interactive verification should start in ACL2's \u000A initial <see topic=\"ACL2____WORLD\">world</see> &mdash; although you might, of course, start your \u000A sequence of <see topic=\"ACL2____EVENTS\">events</see> with some <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>s to build a more \u000A elaborate <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>The first step towards publishing a book containing the results above is to \u000A create a file that starts with the <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> and then contains the \u000A rest of the forms.  Let's call that file <v>&quot;my-book.lisp&quot;</v>.  The name is \u000A unimportant, except it must end with <v>&quot;.lisp&quot;</v>.  If there are <see topic=\"ACL2____EVENTS\">events</see> that you do not wish to be available to the user of the book &mdash; \u000A e.g., lemmas you proved on your way toward proving the main ones &mdash; you \u000A may so mark them by enclosing them in <tt><see topic=\"ACL2____LOCAL\">local</see></tt> forms.  See <see topic=\"ACL2____LOCAL\">local</see>. \u000A Let us suppose you wish to hide <v>rev-app-hack</v> above.  You may also add \u000A standard Lisp comments to the file.  The final content of \u000A <v>&quot;my-book.lisp&quot;</v> might be:</p> \u000A \u000A <code>; This book contains my app and rev functions and the theorem\u000A; that rev is its own inverse.\u000A\u000A  (<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2-MY-BOOK&quot;)\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)) y))\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (app (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))) nil))\u000A\u000A; The following hack is not exported.\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> rev-app-hack\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (app a (<see topic=\"COMMON-LISP____LIST\">list</see> x))) (<see topic=\"COMMON-LISP____CONS\">cons</see> x (rev a)))))\u000A\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> rev-rev\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">acl2::true-listp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev x)) x)))</code> \u000A \u000A <p>The file shown above <b>is</b> the book.  By the time this note is done you \u000A will have seen how to certify that the book is correct, how to compile it, and \u000A how to use it in other host <see topic=\"ACL2____WORLD\">world</see>s.  Observe that the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> is \u000A not in the book.  It cannot be: Common Lisp compilers disagree on how to treat \u000A new package definitions appearing in files to be compiled.</p> \u000A \u000A <p>Since a book is just a source file typed by the user, ACL2 provides a \u000A mechanism for checking that the <see topic=\"ACL2____EVENTS\">events</see> are all admissible and then \u000A marking the file as checked.  This is called certification.  To certify \u000A <v>&quot;my-book.lisp&quot;</v> you should first get into ACL2 with an initial <see topic=\"ACL2____WORLD\">world</see>.  Then, define the package needed by the book, by typing the following \u000A <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> to the ACL2 <see topic=\"ACL2____PROMPT\">prompt</see>:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;ACL2-MY-BOOK&quot;\u000A               (<see topic=\"ACL2____UNION-EQ\">union-eq</see> *common-lisp-symbols-from-main-lisp-package*\u000A                         *acl2-exports*))</code> \u000A \u000A <p>Then execute the <see topic=\"ACL2____COMMAND\">command</see>:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-book&quot; 1 t) ; the `t' is in fact the default</code> \u000A \u000A <p>Observe that you do not type the <v>&quot;.lisp&quot;</v> part of the file name.  For \u000A purposes of <see topic=\"ACL2____BOOKS\">books</see>, the book's name is <v>&quot;my-book&quot;</v> and by the time \u000A all is said and done, there will be several extensions in addition to the \u000A <v>&quot;.lisp&quot;</v> extension associated with it.</p> \u000A \u000A <p>The <v>1</v> tells <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> that you acknowledge that there is \u000A one command in this ``certification <see topic=\"ACL2____WORLD\">world</see>'' (namely the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>).  To use the book, any prospective host <see topic=\"ACL2____WORLD\">world</see> must be extended \u000A by the addition of whatever <see topic=\"ACL2____COMMAND\">command</see>s occurred before certification.  It \u000A would be a pity to certify a book in a <see topic=\"ACL2____WORLD\">world</see> containing junk because \u000A that junk will become the ``<see topic=\"ACL2____PORTCULLIS\">portcullis</see>'' guarding entrance to the book. \u000A The <v>t</v> above tells <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> that you wish to compile \u000A <v>&quot;my-book.lisp&quot;</v> also (but see <see topic=\"ACL2____COMPILATION\">compilation</see> for an exception). \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> makes many checks but by far the most important and \u000A time-consuming one is that it ``proves'' every event in the file.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is done it will have created two new files.  The \u000A first will be called <v>&quot;my-book.cert&quot;</v> and contains the ``<see topic=\"ACL2____CERTIFICATE\">certificate</see>'' attesting to the admissibility of the <see topic=\"ACL2____EVENTS\">events</see> in \u000A <v>&quot;my-book.lisp&quot;</v>.  The <see topic=\"ACL2____CERTIFICATE\">certificate</see> contains the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> and \u000A any other forms necessary to construct the certification <see topic=\"ACL2____WORLD\">world</see>.  It \u000A also contains various check sums used to help you keep track of which version \u000A of <v>&quot;my-book.lisp&quot;</v> was certified.</p> \u000A \u000A <p>The second file that may be created by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is the compiled \u000A version of <v>&quot;my-book.lisp&quot;</v> and will have a name that is assigned by the \u000A host compiler (e.g., <v>&quot;my-book.o&quot;</v> in GCL, <v>&quot;my-book.fasl&quot;</v> in \u000A SBCL).  <tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> will also load this object file.  When <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is done, you may throw away the logical <see topic=\"ACL2____WORLD\">world</see> it created, \u000A for example by executing the <see topic=\"ACL2____COMMAND\">command</see> <v>:u</v>.</p> \u000A \u000A <p>To use the book later in any ACL2 session, just execute the event \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;my-book&quot;)</v>.  This will do the necessary <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, \u000A load the non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in <v>&quot;my-book.lisp&quot;</v> and then may \u000A load the compiled code for the non-local functions defined in that file. \u000A Checks are made to ensure that the <see topic=\"ACL2____CERTIFICATE\">certificate</see> file exists and \u000A describes the version of <v>&quot;my-book.lisp&quot;</v> that is read.  The compiled \u000A code is loaded if and only if it exists and has a later write date than the \u000A source file (but see <see topic=\"ACL2____COMPILATION\">compilation</see> for an exception).</p> \u000A \u000A <p>Since <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is itself an event, you may put such forms into \u000A other <see topic=\"ACL2____BOOKS\">books</see>.  Thus it is possible for the inclusion of a single book to \u000A lead to the inclusion of many others.  The check sum information maintained in \u000A <see topic=\"ACL2____CERTIFICATE\">certificate</see>s helps deal with the version control problem of the \u000A referenced <see topic=\"ACL2____BOOKS\">books</see>.  I.e., if this version of <v>&quot;my-book&quot;</v> is used \u000A during the certification of <v>&quot;your-book&quot;</v>, then the <see topic=\"ACL2____CERTIFICATE\">certificate</see> \u000A for <v>&quot;your-book&quot;</v> includes the check sum of this version of \u000A <v>&quot;my-book&quot;</v>.  If a later <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;your-book&quot;)</v> finds a \u000A version of <v>&quot;my-book&quot;</v> with a different check sum, an error is signalled. \u000A But check sums are not perfect and the insecurity of the host file system \u000A prevents ACL2 from guaranteeing the logical soundness of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event, even for a book that appears to have a valid <see topic=\"ACL2____CERTIFICATE\">certificate</see> (they can be forged, after all).  (See <see topic=\"ACL2____CERTIFICATE\">certificate</see> for \u000A further discussion.)</p> \u000A \u000A <p>This concludes the example of how to create, certify and use a book.  If \u000A you wish, you could now review the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for book-related \u000A topics (see <see topic=\"ACL2____BOOKS\">books</see>) and browse through them.  They'll probably make \u000A sense in this context.  Alternatively, you could continue the ``guided tour'' \u000A through the rest of the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of <see topic=\"ACL2____BOOKS\">books</see>.  See <see topic=\"ACL2____BOOK-NAME\">book-name</see>, following the pointer given at the conclusion.</p>"],

 "ACL2____BDD-INTRODUCTION":[["Bdd"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BDD\">bdd</see> for a brief introduction to BDDs in ACL2 and for \u000A pointers to other documentation on BDDs in ACL2.  Here, we illustrate the use \u000A of BDDs in ACL2 by way of some examples.  For a further example, see <see topic=\"ACL2____IF_A2\">if*</see>.</p> \u000A \u000A <p>Let us begin with a really simple example.  (We will explain the <v>:bdd</v> \u000A hint <v>(:vars nil)</v> below.)</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____IF\">if</see> a b c) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> a) c b))\u000A            :hints ((&quot;Goal&quot; :bdd (:vars nil)))) ; Prove with BDDs\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000ABut simplification with BDDs (7 nodes) reduces this to T, using the\u000A:definitions EQUAL and NOT.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:DEFINITION EQUAL) (:DEFINITION NOT))\u000AWarnings:  None\u000ATime:  0.18 seconds (prove: 0.05, print: 0.02, other: 0.12)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>The <v>:bdd</v> hint <v>(:vars nil)</v> indicates that BDDs are to be used on \u000A the indicated goal, and that any so-called ``variable ordering'' may be used: \u000A ACL2 may use a convenient order that is far from optimal.  It is beyond the \u000A scope of the present documentation to address the issue of how the user may \u000A choose good variable orderings.  Someday our implementation of BDDs may be \u000A improved to include heuristically-chosen variable orderings rather than rather \u000A random ones.</p> \u000A \u000A <p>Here is a more interesting example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> v-not (x)\u000A; Complement every element of a list of Booleans.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (v-not (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A    nil))\u000A\u000A; Now we prove a rewrite rule that explains how to open up v-not on\u000A; a consp.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> v-not-cons\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (v-not (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____NOT\">not</see> x) (v-not y))))\u000A\u000A; Finally, we prove for 7-bit lists that v-not is self-inverting.\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3 x4 x5 x6)))\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> x)\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (v-not (v-not x)) x)))\u000A :hints ((&quot;Goal&quot; :bdd\u000A                 ;; Note that this time we specify a variable order.\u000A                 (:vars (x0 x1 x2 x3 x4 x5 x6)))))</code> \u000A \u000A <p>It turns out that the variable order doesn't seem to matter in this \u000A example; using several orders we found that 30 nodes were created, and the \u000A proof time was about 1/10 of a second on a (somewhat enhanced) Sparc 2.  The \u000A same proof took about a minute and a half without any <v>:bdd</v> hint!  This \u000A observation is a bit misleading perhaps, since the theorem for arbitrary \u000A <v>x</v>,</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> x)\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (v-not (v-not x)) x)))</code> \u000A \u000A <p>only takes about 1.5 times as long as the <v>:bdd</v> proof for 7 bits, \u000A above!  Nevertheless, BDDs can be very useful in reducing proof time, \u000A especially when there is no regular structure to facilitate proof by \u000A induction, or when the induction scheme is so complicated to construct that \u000A significant user effort is required to get the proof by induction to go \u000A through.</p> \u000A \u000A <p>Finally, consider the preceding example, with a <v>:bdd</v> hint of (say) \u000A <v>(:vars nil)</v>, but with the rewrite rule <v>v-not-cons</v> above disabled. \u000A In that case, the proof fails, as we see below.  That is because the BDD \u000A algorithm in ACL2 uses hypothesis-free <v>:</v><see topic=\"ACL2____REWRITE\">rewrite</see> rules, \u000A <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt><v>s</v>, and nonrecursive definitions, but \u000A it does not use recursive definitions.</p> \u000A \u000A <p>Notice that when we issue the <v>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)</v> command, the system's \u000A response clearly shows that we need a rewrite rule for simplifying terms of \u000A the form <v>(v-not (<see topic=\"COMMON-LISP____CONS\">cons</see> ...))</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see>\u000A        (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3 x4 x5 x6)))\u000A          (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> x)\u000A                   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (v-not (v-not x)) x)))\u000A        :hints ((&quot;Goal&quot; :bdd (:vars nil)\u000A                 :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> v-not-cons))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AACL2 Error in ( THM ...):  Attempted to create V-NOT node during BDD\u000Aprocessing with an argument that is a call of a bdd-constructor,\u000Awhich would produce a non-BDD term (as defined in :DOC\u000Abdd-algorithm).  See :DOC show-bdd.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.58 seconds (prove: 0.13, print: 0.00, other: 0.45)\u000A\u000A******** FAILED ********  See :DOC failure  ******** FAILED ********\u000AACL2 !&gt;(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)\u000A\u000ABDD computation on Goal yielded 17 nodes.\u000A==============================\u000A\u000ABDD computation was aborted on Goal, and hence there is no\u000Afalsifying assignment that can be constructed.  Here is a backtrace\u000Aof calls, starting with the top-level call and ending with the one\u000Athat led to the abort.  See :DOC show-bdd.\u000A\u000A(<see topic=\"COMMON-LISP____LET\">LET</see> ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3 X4 X5 ...)))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"ACL2____BOOLEAN-LISTP\">BOOLEAN-LISTP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (V-NOT (V-NOT X)) X)))\u000A  alist: ((X6 X6) (X5 X5) (X4 X4) (X3 X3) (X2 X2) (X1 X1) (X0 X0))\u000A\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (V-NOT (V-NOT X)) X)\u000A  alist: ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3 X4 X5 ...)))\u000A\u000A(V-NOT (V-NOT X))\u000A  alist: ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3 X4 X5 ...)))\u000A\u000A(V-NOT X)\u000A  alist: ((X (<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3 X4 X5 ...)))\u000AACL2 !&gt;</code> \u000A \u000A <p>The term that has caused the BDD algorithm to abort is thus <v>(V-NOT X)</v>, \u000A where <v>X</v> has the value <v>(<see topic=\"COMMON-LISP____LIST\">LIST</see> X0 X1 X2 X3 X4 X5 ...)</v>, i.e., <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see>\u000A X0 (<see topic=\"COMMON-LISP____LIST\">LIST</see> X1 X2 X3 X4 X5 ...))</v>.  Thus, we see the utility of introducing a \u000A rewrite rule to simplify terms of the form <v>(V-NOT (<see topic=\"COMMON-LISP____CONS\">CONS</see> ...))</v>.  The moral \u000A of this story is that if you get an error of the sort shown above, you may \u000A find it useful to execute the command <v>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)</v> and use the result as \u000A advice that suggests the left hand side of a rewrite rule.</p> \u000A \u000A <p>Here is another sort of failed proof.  In this version we have omitted the \u000A hypothesis that the input is a bit vector.  Below we use <v>show-bdd</v> to see \u000A what went wrong, and use the resulting information to construct a \u000A counterexample.  This failed proof corresponds to a slightly modified input \u000A theorem, in which <v>x</v> is bound to the 4-element list <v>(<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2\u000A x3)</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see>\u000A        (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3)))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (v-not (v-not x)) x))\u000A        :hints ((&quot;Goal&quot; :bdd\u000A                 ;; This time we do not specify a variable order.\u000A                 (:vars nil))))\u000A\u000A[Note:  A hint was supplied for our processing of the goal above.\u000AThanks!]\u000A\u000AACL2 Error in ( THM ...):  The :BDD hint for the current goal has\u000Asuccessfully simplified this goal, but has failed to prove it.\u000AConsider using (<see topic=\"ACL2____SHOW-BDD\">SHOW-BDD</see>) to suggest a counterexample; see :DOC\u000Ashow-bdd.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.18 seconds (prove: 0.07, print: 0.00, other: 0.12)\u000A\u000A******** FAILED ********  See :DOC failure  ******** FAILED ********\u000AACL2 !&gt;(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)\u000A\u000ABDD computation on Goal yielded 73 nodes.\u000A==============================\u000A\u000AFalsifying constraints:\u000A((X0 &quot;Some non-nil value&quot;)\u000A (X1 &quot;Some non-nil value&quot;)\u000A (X2 &quot;Some non-nil value&quot;)\u000A (X3 &quot;Some non-nil value&quot;)\u000A ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> 'T X0) T)\u000A ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> 'T X1) T)\u000A ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> 'T X2) T)\u000A ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> 'T X3) NIL))\u000A\u000A==============================\u000A\u000ATerm obtained from BDD computation on Goal:\u000A\u000A(<see topic=\"COMMON-LISP____IF\">IF</see> X0\u000A    (<see topic=\"COMMON-LISP____IF\">IF</see> X1\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> X2 (<see topic=\"COMMON-LISP____IF\">IF</see> X3 (<see topic=\"COMMON-LISP____IF\">IF</see> # # #) (<see topic=\"COMMON-LISP____IF\">IF</see> X3 # #))\u000A            (<see topic=\"COMMON-LISP____IF\">IF</see> X2 'NIL (<see topic=\"COMMON-LISP____IF\">IF</see> X3 # #)))\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> X1 'NIL\u000A            (<see topic=\"COMMON-LISP____IF\">IF</see> X2 (<see topic=\"COMMON-LISP____IF\">IF</see> X3 # #) (<see topic=\"COMMON-LISP____IF\">IF</see> X2 # #))))\u000A    (<see topic=\"COMMON-LISP____IF\">IF</see> X0 'NIL\u000A        (<see topic=\"COMMON-LISP____IF\">IF</see> X1 (<see topic=\"COMMON-LISP____IF\">IF</see> X2 (<see topic=\"COMMON-LISP____IF\">IF</see> X3 # #) (<see topic=\"COMMON-LISP____IF\">IF</see> X2 # #))\u000A            (<see topic=\"COMMON-LISP____IF\">IF</see> X1 'NIL (<see topic=\"COMMON-LISP____IF\">IF</see> X2 # #)))))\u000A\u000AACL2 Query (:SHOW-BDD):  Print the term in full?  (N, Y, W or ?):\u000An ; I've seen enough.  The assignment shown above suggests\u000A  ; (though not conclusively) that if we bind x3 to a non-nil\u000A  ; value other than T, and bind x0, x1, and x2 to t, then we\u000A  ; this may give us a counterexample.\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">let</see> ((x0 t) (x1 t) (x2 t) (x3 7))\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____LIST\">list</see> x0 x1 x2 x3)))\u000A           ;; Let's use LIST instead of EQUAL to see how the two\u000A           ;; lists differ.\u000A           (<see topic=\"COMMON-LISP____LIST\">list</see> (v-not (v-not x)) x)))\u000A((T T T T) (T T T 7))\u000AACL2 !&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____IF_A2\">if*</see> for another example.</p>"],

 "ACL2____ALTERNATIVE-INTRODUCTION":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>This section contains introductory material on ACL2 including what \u000A ACL2 is, how to get started using the system, how to read the output, and \u000A other introductory topics.  It was written almost entirely by Bill Young of \u000A Computational Logic, Inc.</p> \u000A \u000A <p>You might also find CLI Technical Report 101 helpful, especially if you are \u000A familiar with Nqthm.  If you would like more familiarity with Nqthm, we \u000A suggest CLI Technical Report 100.</p> \u000A \u000A <p><i>OVERVIEW</i></p> \u000A \u000A <p>ACL2 is an automated reasoning system developed (for the first 9 years) at \u000A Computational Logic, Inc. and (from January, 1997) at the University of Texas \u000A at Austin.  It is the successor to the Nqthm (or Boyer-Moore) logic and proof \u000A system and its Pc-Nqthm interactive enhancement.  The acronym ACL2 actually \u000A stands for ``A Computational Logic for Applicative Common Lisp''.  This title \u000A suggests several distinct but related aspects of ACL2.</p> \u000A \u000A <p>We assume that readers of the ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> have at least a \u000A very slight familiarity with some Lisp-like language.  We will address the \u000A issue of prerequisites further, in ``ABOUT THIS TUTORIAL'' below.</p> \u000A \u000A <p>As a <b>logic</b>, ACL2 is a formal system with rigorously defined syntax \u000A and semantics.  In mathematical parlance, the ACL2 logic is a first-order \u000A logic of total recursive functions providing mathematical induction on the \u000A ordinals up to epsilon-0 and two extension principles: one for recursive \u000A definition and one for constrained introduction of new function symbols, here \u000A called encapsulation.  The syntax of ACL2 is that of Common Lisp; ACL2 \u000A specifications are ``also'' Common Lisp programs in a way that we will make \u000A clear later.  In less formal language, the ACL2 logic is an integrated \u000A collection of rules for defining (or axiomatizing) recursive functions, \u000A stating properties of those functions, and rigorously establishing those \u000A properties.  Each of these activities is mechanically supported.</p> \u000A \u000A <p>As a <b>specification language</b>, ACL2 supports modeling of systems of \u000A various kinds.  An ACL2 function can equally be used to express purely formal \u000A relationships among mathematical entities, to describe algorithms, or to \u000A capture the intended behavior of digital systems.  For digital systems, an \u000A ACL2 specification is a mathematical <b>model</b> that is intended to \u000A formalize relevant aspects of system behavior.  Just as physics allows us to \u000A model the behavior of continuous physical systems, ACL2 allows us to model \u000A digital systems, including many with physical realizations such as computer \u000A hardware.  As early as the 1930's Church, Kleene, Turing and others \u000A established that recursive functions provide an expressive formalism for \u000A modeling digital computation.  Digital computation should be understood in a \u000A broad sense, covering a wide variety of activities including almost any \u000A systematic or algorithmic activity, or activity that can be reasonably \u000A approximated in that way.  This ranges from the behavior of a digital circuit \u000A to the behavior of a programming language compiler to the behavior of a \u000A controller for a physical system (as long as the system can be adequately \u000A modeled discretely).  All of these have been modeled using ACL2 or its \u000A predecessor Nqthm.</p> \u000A \u000A <p>ACL2 is a <b>computational</b> logic in at least three distinct senses. \u000A First, the theory of recursive functions is often considered the mathematics \u000A of computation.  Church conjectured that any ``effective computation'' can be \u000A modeled as a recursive function.  Thus, ACL2 provides an expressive language \u000A for modeling digital systems.  Second, many ACL2 specifications are \u000A executable.  In fact, recursive functions written in ACL2 <b>are</b> Common \u000A Lisp functions that can be submitted to any compliant Common Lisp compiler and \u000A executed (in an environment where suitable ACL2-specific macros and functions \u000A are defined).  Third, ACL2 is computational in the sense that calculation is \u000A heavily integrated into the reasoning process.  Thus, an expression with \u000A explicit constant values but no free variables can be simplified by \u000A calculation rather than by complex logical manipulations.</p> \u000A \u000A <p>ACL2 is a powerful, automated <b>theorem prover</b> or proof checker.  This \u000A means that a competent user can utilize the ACL2 system to discover proofs of \u000A theorems stated in the ACL2 logic or to check previously discovered proofs. \u000A The basic deductive steps in an ACL2-checked proof are often quite large, due \u000A to the sophisticated combination of decision procedures, conditional \u000A rewriting, mathematical and structural induction, propositional \u000A simplification, and complex heuristics to orchestrate the interactions of \u000A these capabilities.  Unlike some automated proof systems, ACL2 does not \u000A produce a formal proof.  However, we believe that if ACL2 certifies the \u000A ``theoremhood'' of a given conjecture, then such a formal proof exists and, \u000A therefore, the theorem is valid.  The ultimate result of an ACL2 proof session \u000A is a collection of ``<see topic=\"ACL2____EVENTS\">events</see>,'' possibly grouped into ``<see topic=\"ACL2____BOOKS\">books</see>,'' \u000A that can be replayed in ACL2.  Therefore, a proof can be independently \u000A validated by any ACL2 user.</p> \u000A \u000A <p>ACL2 may be used in purely automated mode in the shallow sense that \u000A conjectures are submitted to the prover and the user does not interact with \u000A the proof attempt (except possibly to stop it) until the proof succeeds or \u000A fails.  However, any non-trivial proof attempt is actually interactive, since \u000A successful proof ``<see topic=\"ACL2____EVENTS\">events</see>'' influence the subsequent behavior of the \u000A prover.  For example, proving a lemma may introduce a rule that subsequently \u000A is used automatically by the prover.  Thus, any realistic proof attempt, even \u000A in ``automatic'' mode, is really an interactive dialogue with the prover to \u000A craft a sequence of <see topic=\"ACL2____EVENTS\">events</see> building an appropriate theory and proof \u000A rules leading up to the proof of the desired result.  Also, ACL2 supports \u000A annotating a theorem with ``<see topic=\"ACL2____HINTS\">hints</see>'' designed to guide the proof \u000A attempt.  By supplying appropriate <see topic=\"ACL2____HINTS\">hints</see>, the user can suggest proof \u000A strategies that the prover would not discover automatically.  There is a \u000A ``<see topic=\"ACL2____PROOF-TREE\">proof-tree</see>'' facility (see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>) that allows the user to \u000A <see topic=\"ACL2____MONITOR\">monitor</see> the progress and structure of a proof attempt in real-time. \u000A Exploring failed proof attempts is actually where heavy-duty ACL2 users spend \u000A most of their time.</p> \u000A \u000A <p>ACL2 can also be used in a more explicitly interactive mode.  The ``<see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'' subsystem of ACL2 allows exploration of a proof on a fairly \u000A low level including expanding calls of selected function symbols, invoking \u000A specific <see topic=\"ACL2____REWRITE\">rewrite</see> rules, and selectively navigating around the proof. \u000A This facility can be used to gain sufficient insight into the proof to \u000A construct an automatic version, or to generate a detailed interactive-style \u000A proof that can be replayed in batch mode.</p> \u000A \u000A <p>Because ACL2 is all of these things &mdash; computational logic, \u000A specification language, <see topic=\"ACL2____PROGRAMMING\">programming</see> system, and theorem prover &mdash; \u000A it is more than the sum of its parts.  The careful integration of these \u000A diverse aspects has produced a versatile automated reasoning system suitable \u000A for building highly reliable digital systems.  In the remainder of this \u000A tutorial, we will illustrate some simple uses of this automated reasoning \u000A system.</p> \u000A \u000A <p><i>ABOUT THIS TUTORIAL</i></p> \u000A \u000A <p>ACL2 is a complex system with a vast array of features, bells and whistles. \u000A However, it is possible to perform productive work with the system using only \u000A a small portion of the available functionality.  The goals of this tutorial \u000A are to:</p> \u000A \u000A <blockquote> \u000A \u000A <p>familiarize the new user with the most basic features of and modes of \u000A interaction with ACL2;</p> \u000A \u000A <p>familiarize her with the form of output of the system; and</p> \u000A \u000A <p>work through a graduated series of examples.</p></blockquote> \u000A \u000A <p>The more knowledge the user brings to this system, the easier it will be to \u000A become proficient.  On one extreme: the <b>ideal</b> user of ACL2 is an expert \u000A Common Lisp programmer, has deep understanding of automated reasoning, and is \u000A intimately familiar with the earlier Nqthm system.  Such ideal users are \u000A unlikely to need this tutorial.  However, without some background knowledge, \u000A the beginning user is likely to become extremely confused and frustrated by \u000A this system.  We suggest that a new user of ACL2 should:</p> \u000A \u000A <blockquote> \u000A \u000A <p>(a) have a little familiarity with Lisp, including basic Lisp programming \u000A and prefix notation (a Lisp reference manual such as Guy Steele's ``Common \u000A Lisp: The Language'' is also helpful);</p> \u000A \u000A <p>(b) be convinced of the utility of formal modeling; and</p> \u000A \u000A <p>(c) be willing to gain familiarity with basic automated theorem proving \u000A topics such as rewriting and algebraic simplification.</p></blockquote> \u000A \u000A <p>We will not assume any deep familiarity with Nqthm (the so-called \u000A ``Boyer-Moore Theorem Prover''), though the book ``A Computational Logic \u000A Handbook'' by Boyer and Moore (Academic Press, 1988) is an extremely useful \u000A reference for many of the topics required to become a competent ACL2 user. \u000A We'll refer to it as ACLH below.</p> \u000A \u000A <p>As we said in the introduction, ACL2 has various facets.  For example, it \u000A can be used as a Common Lisp <see topic=\"ACL2____PROGRAMMING\">programming</see> system to construct \u000A application programs.  In fact, the ACL2 system itself is a large Common Lisp \u000A program constructed almost entirely within ACL2.  Another use of ACL2 is as a \u000A specification and modeling tool.  That is the aspect we will concentrate on in \u000A the remainder of this tutorial.</p> \u000A \u000A <p><i>GETTING STARTED</i></p> \u000A \u000A <p>This section is an abridged version of what's available elsewhere; feel \u000A free to see <see topic=\"ACL2____STARTUP\">startup</see> for more details.</p> \u000A \u000A <p>How you start ACL2 will be system dependent, but you'll probably type \u000A something like ``acl2'' at your operating system prompt.  Consult your system \u000A administrator for details.</p> \u000A \u000A <p>When you start up ACL2, you'll probably find yourself inside the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop, as indicated by the following <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>If not, you should type <v>(<see topic=\"ACL2____LP\">LP</see>)</v>.  See <see topic=\"ACL2____LP\">lp</see>, which has a lot more \u000A information about the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop.</p> \u000A \u000A <p>There are two ``modes'' for using ACL2, <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>.  When you begin ACL2, you will ordinarily be in the \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  This means that any new function defined is not \u000A only executable but also is axiomatically defined in the ACL2 logic.  (See \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>.)  Roughly speaking, \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode is available for using ACL2 as a <see topic=\"ACL2____PROGRAMMING\">programming</see> \u000A language without some of the logical burdens necessary for formal reasoning. \u000A In this tutorial we will assume that we always remain in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A mode and that our purpose is to write formal models of digital systems and to \u000A reason about them.</p> \u000A \u000A <p>Now, within the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop you can carry out various kinds of \u000A activities, including the folllowing.  (We'll see examples later of many of \u000A these.)</p> \u000A \u000A <blockquote> \u000A \u000A <p>define new functions (see <see topic=\"COMMON-LISP____DEFUN\">defun</see>);</p> \u000A \u000A <p>execute functions on concrete data;</p> \u000A \u000A <p>pose and attempt to prove conjectures about previously defined functions \u000A (see <see topic=\"ACL2____DEFTHM\">defthm</see>);</p> \u000A \u000A <p>query the ACL2 ``<see topic=\"ACL2____WORLD\">world</see>'' or database (e.g., see <see topic=\"ACL2____PE\">pe</see>); and</p> \u000A \u000A <p>numerous other things.</p></blockquote> \u000A \u000A <p>In addition, there is extensive on-line <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, of which this \u000A tutorial introduction is a part.</p> \u000A \u000A <p><i>INTERACTING WITH ACL2</i></p> \u000A \u000A <p>The standard means of interacting with ACL2 is to submit a sequence of \u000A forms for processing by the ACL2 system.  These forms are checked for \u000A syntactic and semantic acceptability and appropriately processed by the \u000A system.  These forms can be typed directly at the ACL2 <see topic=\"ACL2____PROMPT\">prompt</see>. \u000A However, most successful ACL2 users prefer to do their work using the Emacs \u000A text editor, maintaining an Emacs ``working'' buffer in which forms are \u000A edited.  Those forms are then copied to the ACL2 interaction buffer, which is \u000A often the <v>&quot;*shell*&quot;</v> buffer.</p> \u000A \u000A <p>In some cases, processing succeeds and makes some change to the ACL2 \u000A ``logical <see topic=\"ACL2____WORLD\">world</see>,'' which affects the processing of subsequent forms. \u000A How can this processing fail?  For example, a proposed theorem will be \u000A rejected unless all function symbols mentioned have been previously defined. \u000A Also the ability of ACL2 to discover the proof of a theorem may depend on the \u000A user previously having proved other theorems.  Thus, the order in which forms \u000A are submitted to ACL2 is quite important.  Maintaining forms in an appropriate \u000A order in your working buffer will be helpful for re-playing the proof \u000A later.</p> \u000A \u000A <p>One of the most common <see topic=\"ACL2____EVENTS\">events</see> in constructing a model is introducing \u000A new functions.  New functions are usually introduced using the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A form; we'll encounter some exceptions later.  Proposed function definitions \u000A are checked to make sure that they are syntactically and semantically \u000A acceptable (e.g., that all mentioned functions have been previously defined) \u000A and, for recursive functions, that their recursive calls <b>terminate</b>.  A \u000A recursive function definition is guaranteed to terminate if there is some some \u000A ``measure'' of the arguments and a ``well-founded'' ordering such that the \u000A arguments to the function get smaller in each recursive call.  See <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.</p> \u000A \u000A <p>For example, suppose that we need a function that will append two lists \u000A together.  (We already have one in the ACL2 <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> function; but \u000A suppose perversely that we decide to define our own.)  Suppose we submit the \u000A following definition (you should do so as well and study the system \u000A output):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-app (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      y\u000A    (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (my-app x y))))</code> \u000A \u000A <p>The system responds with the following message:</p> \u000A \u000A <code>ACL2 Error in ( DEFUN MY-APP ...):  No :MEASURE was supplied with\u000Athe definition of MY-APP.  Our heuristics for guessing one have not\u000Amade any suggestions.  No argument of the function is tested along\u000Aevery branch and occurs as a proper subterm at the same argument\u000Aposition in every recursive call.  You must specify a :MEASURE.  See\u000A:DOC defun.</code> \u000A \u000A <p>This means that the system could not find an expression involving the \u000A formal parameters <v>x</v> and <v>y</v> that decreases under some well-founded \u000A order in every recursive call (there is only one such call).  It should be \u000A clear that there is no such measure in this case because the only recursive \u000A call doesn't change the arguments at all.  The definition is obviously flawed; \u000A if it were accepted and executed it would loop forever.  Notice that a \u000A definition that is rejected is not stored in the system database; there is no \u000A need to take any action to have it ``thrown away.''  Let's try again with the \u000A correct definition.  The interaction now looks like (we're also putting in the \u000A ACL2 <see topic=\"ACL2____PROMPT\">prompt</see>; you don't type that):</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-app (x y)\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A             y\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (my-app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))\u000A\u000AThe admission of MY-APP is trivial, using the relation O&lt;\u000A(which is known to be well-founded on the domain recognized by\u000AO-P) and the measure (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).  We observe that the\u000Atype of MY-APP is described by the theorem\u000A(<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (MY-APP X Y)) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (MY-APP X Y) Y)).\u000AWe used primitive type reasoning.\u000A\u000ASummary\u000AForm:  ( DEFUN MY-APP ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.07 seconds (prove: 0.00, print: 0.00, other: 0.07)\u000AMY-APP</code> \u000A \u000A <p>Notice that this time the function definition was accepted.  We didn't have \u000A to supply a measure explicitly; the system inferred one from the form of the \u000A definition.  On complex functions it may be necessary to supply a measure \u000A explicitly.  (See <see topic=\"ACL2____XARGS\">xargs</see>.)</p> \u000A \u000A <p>The system output provides several pieces of information.</p> \u000A \u000A <blockquote> \u000A \u000A <p>The revised definition is acceptable.  The system realized that there is a \u000A particular measure (namely, <v>(<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x)</v>) and a well-founded relation \u000A (<v>o&lt;</v>) under which the arguments of <v>my-app</v> get smaller in recursion. \u000A Actually, the theorem prover proved several theorems to admit <v>my-app</v>. \u000A The main one was that when <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v> is false the <v>acl2-count</v> of \u000A <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> is less than (in the <v>o&lt;</v> sense) the <v>acl2-count</v> of \u000A <v>x</v>.  <tt><see topic=\"ACL2____ACL2-COUNT\">Acl2-count</see></tt> is the most commonly used measure of the ``size`` \u000A of an ACL2 object.  <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> is the ordering relation on ordinals less than \u000A epsilon-0.  On the natural numbers it is just ordinary ``&lt;''.</p> \u000A \u000A <p>The observation printed about ``the type of MY-APP'' means that calls of \u000A the function <v>my-app</v> will always return a value that is either a <see topic=\"COMMON-LISP____CONS\">cons</see> pair or is equal to the second parameter.</p> \u000A \u000A <p>The summary provides information about which previously introduced \u000A definitions and lemmas were used in this proof, about some notable things to \u000A watch out for (the Warnings), and about how long this event took to \u000A process.</p></blockquote> \u000A \u000A <p>Usually, it's not important to read this information.  However, it is a \u000A good habit to scan it briefly to see if the type information is surprising to \u000A you or if there are Warnings.  We'll see an example of them later.</p> \u000A \u000A <p>After a function is accepted, it is stored in the database and available \u000A for use in other function definitions or lemmas.  To see the definition of any \u000A function use the <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> command (see <see topic=\"ACL2____PE\">pe</see>).  For example,</p> \u000A \u000A <code>ACL2 !&gt;:pe my-app\u000A L       73:x(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> MY-APP (X Y)\u000A                    (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____ATOM\">ATOM</see> X)\u000A                        Y (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (MY-APP (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))))</code> \u000A \u000A <p>This displays the definition along with some other relevant information. \u000A In this case, we know that this definition was processed in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode (the ``<v>L</v>'') and was the 73rd <see topic=\"ACL2____COMMAND\">command</see> processed in the \u000A current session.</p> \u000A \u000A <p>We can also try out our newly defined function on some sample data.  To do \u000A that, just submit a form to be evaluated to ACL2.  For example,</p> \u000A \u000A <code>ACL2 !&gt;(my-app '(0 1 2) '(3 4 5))\u000A(0 1 2 3 4 5)\u000AACL2 !&gt;(my-app nil nil)\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p>Now suppose we want to prove something about the function just introduced. \u000A We conjecture, for example, that the length of the <see topic=\"COMMON-LISP____APPEND\">append</see> of two lists \u000A is the sum of their lengths.  We can formulate this conjecture in the form of \u000A the following ACL2 <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> form.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-app-length\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (my-app x y))\u000A         (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y))))</code> \u000A \u000A <p>First of all, how did we know about the functions <v>len</v> and <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt>, \u000A etc.?  The answer to that is somewhat unsatisfying &mdash; we know them from \u000A our past experience in using Common Lisp and ACL2.  It's hard to know that a \u000A function such as <v>len</v> exists without first knowing some Common Lisp.  If \u000A we'd guessed that the appropriate function was called <tt><see topic=\"COMMON-LISP____LENGTH\">length</see></tt> (say, \u000A from our knowledge of Lisp) and tried <v>:pe length</v>, we would have seen that \u000A <tt><see topic=\"COMMON-LISP____LENGTH\">length</see></tt> is defined in terms of <v>len</v>, and we could have explored \u000A from there.  Luckily, you can write a lot of ACL2 functions without knowing \u000A too many of the primitive functions.</p> \u000A \u000A <p>Secondly, why don't we need some ``type'' hypotheses?  Does it make sense \u000A to append things that are not lists?  Well, yes.  ACL2 and Lisp are both quite \u000A weakly typed.  For example, inspection of the definition of <v>my-app</v> shows \u000A that if <v>x</v> is not a <see topic=\"COMMON-LISP____CONS\">cons</see> pair, then <v>(my-app x y)</v> always \u000A returns <v>y</v>, no matter what <v>y</v> is.</p> \u000A \u000A <p>Thirdly, would it matter if we rewrote the lemma with the equality \u000A reversed, as follows?</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-app-length2\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y))\u000A         (<see topic=\"ACL2____LEN\">len</see> (my-app x y)))).</code> \u000A \u000A <p>The two are <b>logically</b> equivalent, but...yes, it would make a big \u000A difference.  Recall our remark that a lemma is not only a ``fact'' to be \u000A proved; it also is used by the system to prove other later lemmas.  The \u000A current lemma would be stored as a <see topic=\"ACL2____REWRITE\">rewrite</see> rule.  (See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.)  For a <see topic=\"ACL2____REWRITE\">rewrite</see> rule, a conclusion of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see>\u000A LHS RHS)</v> means to replace instances of the <v>LHS</v> by the appropriate \u000A instance of the <v>RHS</v>.  Presumably, it's better to <see topic=\"ACL2____REWRITE\">rewrite</see> <v>(<see topic=\"ACL2____LEN\">len</see>\u000A (my-app x y))</v> to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y))</v> than the other way around.  The \u000A reason is that the system ``knows'' more about <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> than it does about \u000A the new function symbol <v>my-app</v>.</p> \u000A \u000A <p>So let's see if we can prove this lemma.  Submitting our preferred <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> to ACL2 (do it!), we get the following interaction:</p> \u000A \u000A <code>          --------------------------------------------------\u000AACL2 !&gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> my-app-length\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (my-app x y))\u000A         (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y))))\u000A\u000AName the formula above *1.\u000A\u000APerhaps we can prove *1 by induction.  Three induction schemes are\u000Asuggested by this conjecture.  These merge into two derived\u000Ainduction schemes.  However, one of these is flawed and so we are\u000Aleft with one viable candidate.\u000A\u000AWe will induct according to a scheme suggested by (<see topic=\"ACL2____LEN\">LEN</see> X), but\u000Amodified to accommodate (MY-APP X Y).  If we let (:P X Y) denote *1\u000Aabove then the induction scheme we'll use is\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)) (:P X Y))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (:P (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))\u000A              (:P X Y))).\u000AThis induction is justified by the same argument used to admit LEN,\u000Anamely, the measure (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X) is decreasing according to the\u000Arelation O&lt; (which is known to be well-founded on the domain\u000Arecognized by O-P).  When applied to the goal at hand the\u000Aabove induction scheme produces the following two nontautological\u000Asubgoals.\u000A\u000ASubgoal *1/2\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (MY-APP X Y))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> X) (<see topic=\"ACL2____LEN\">LEN</see> Y)))).\u000A\u000ABut simplification reduces this to T, using the :definitions of FIX,\u000ALEN and MY-APP, the :type-prescription rule LEN, the :rewrite rule\u000AUNICITY-OF-0 and primitive type reasoning.\u000A\u000ASubgoal *1/1\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (MY-APP (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"ACL2____LEN\">LEN</see> Y))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (MY-APP X Y))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> X) (<see topic=\"ACL2____LEN\">LEN</see> Y)))).\u000A\u000AThis simplifies, using the :definitions of LEN and MY-APP, primitive\u000Atype reasoning and the :rewrite rules COMMUTATIVITY-OF-+ and\u000ACDR-CONS, to\u000A\u000ASubgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (MY-APP (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> Y) (<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">LEN</see> (MY-APP (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) Y)))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> Y) 1 (<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))).\u000A\u000ABut simplification reduces this to T, using linear arithmetic,\u000Aprimitive type reasoning and the :type-prescription rule LEN.\u000A\u000AThat completes the proof of *1.\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( DEFTHM MY-APP-LENGTH ...)\u000ARules: ((:REWRITE UNICITY-OF-0)\u000A        (:DEFINITION FIX)\u000A        (:REWRITE COMMUTATIVITY-OF-+)\u000A        (:DEFINITION LEN)\u000A        (:REWRITE CDR-CONS)\u000A        (:DEFINITION MY-APP)\u000A        (:TYPE-PRESCRIPTION LEN)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:FAKE-RUNE-FOR-LINEAR NIL))\u000AWarnings:  None\u000ATime:  0.30 seconds (prove: 0.13, print: 0.05, other: 0.12)\u000A MY-APP-LENGTH\u000A          --------------------------------------------------</code> \u000A \u000A <p>Wow, it worked!  In brief, the system first tried to <see topic=\"ACL2____REWRITE\">rewrite</see> and \u000A simplify as much as possible.  Nothing changed; we know that because it said \u000A ``Name the formula above *1.''  Whenever the system decides to name a formula \u000A in this way, we know that it has run out of techniques to use other than proof \u000A by induction.</p> \u000A \u000A <p>The induction performed by ACL2 is structural or ``Noetherian'' induction. \u000A You don't need to know much about that except that it is induction based on \u000A the structure of some object.  The heuristics infer the structure of the \u000A object from the way the object is recursively decomposed by the functions used \u000A in the conjecture.  The heuristics of ACL2 are reasonably good at selecting an \u000A induction scheme in simple cases.  It is possible to override the heuristic \u000A choice by providing an <v>:induction</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>).  In the case \u000A of the theorem above, the system inducts on the structure of <v>x</v> as \u000A suggested by the decomposition of <v>x</v> in both <v>(my-app x y)</v> and <v>(<see topic=\"ACL2____LEN\">len</see>\u000A x)</v>.  In the base case, we assume that <v>x</v> is not a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>.  In the \u000A inductive case, we assume that it is a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> and assume that the \u000A conjecture holds for <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>.</p> \u000A \u000A <p>There is a close connection between the analysis that goes on when a \u000A function like <v>my-app</v> is accepted and when we try to prove something \u000A inductively about it.  That connection is spelled out well in Boyer and \u000A Moore's book ``A Computational Logic,'' if you'd like to look it up.  But it's \u000A pretty intuitive.  We accepted <v>my-app</v> because the ``size'' of the first \u000A argument <v>x</v> decreases in the recursive call.  That tells us that when we \u000A need to prove something inductively about <v>my-app</v>, it's a good idea to try \u000A an induction on the size of the first argument.  Of course, when you have a \u000A theorem involving several functions, it may be necessary to concoct a more \u000A complicated <see topic=\"ACL2____INDUCTION\">induction</see> schema, taking several of them into account. \u000A That's what's meant by ``merging'' the induction schemas.</p> \u000A \u000A <p>The proof involves two cases: the base case, and the inductive case. \u000A You'll notice that the subgoal numbers go <b>down</b> rather than up, so you \u000A always know how many subgoals are left to process.  The base case (<v>Subgoal\u000A *1/2</v>) is handled by opening up the function definitions, simplifying, doing \u000A a little rewriting, and performing some reasoning based on the types of the \u000A arguments.  You'll often encounter references to system defined lemmas (like \u000A <v>unicity-of-0</v>).  You can always look at those with <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>; but, \u000A in general, assume that there's a lot of simplification power under the hood \u000A that's not too important to understand fully.</p> \u000A \u000A <p>The inductive case (<v>Subgoal *1/1</v>) is also dispatched pretty easily. \u000A Here we assume the conjecture true for the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of the list and try to \u000A prove it for the entire list.  Notice that the prover does some simplification \u000A and then prints out an updated version of the goal (<v>Subgoal *1/1'</v>). \u000A Examining these gives you a pretty good idea of what's going on in the \u000A proof.</p> \u000A \u000A <p>Sometimes one goal is split into a number of subgoals, as happened with the \u000A induction above.  Sometimes after some initial processing the prover decides \u000A it needs to prove a subgoal by induction; this subgoal is given a name and \u000A pushed onto a stack of goals.  Some steps, like generalization (see ACLH), are \u000A not necessarily validity preserving; that is, the system may adopt a false \u000A subgoal while trying to prove a true one.  (Note that this is ok in the sense \u000A that it is not ``unsound.''  The system will fail in its attempt to establish \u000A the false subgoal and the main proof attempt will fail.)  As you gain facility \u000A with using the prover, you'll get pretty good at recognizing what to look for \u000A when reading a proof script.  The prover's <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> utility helps \u000A with monitoring an ongoing proof and jumping to designated locations in the \u000A proof (see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>).  See <see topic=\"ACL2____TIPS\">tips</see> for a number of useful \u000A pointers on using the theorem prover effectively.</p> \u000A \u000A <p>When the prover has successfully proved all subgoals, the proof is \u000A finished.  As with a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, a summary of the proof is printed.  This \u000A was an extremely simple proof, needing no additional guidance.  More realistic \u000A examples typically require the user to look carefully at the failed proof log \u000A to find ways to influence the prover to do better on its next attempt.  This \u000A means either: proving some rules that will then be available to the prover, \u000A changing the global state in ways that will affect the proof, or providing \u000A some <see topic=\"ACL2____HINTS\">hints</see> locally that will influence the prover's behavior.  Proving \u000A this lemma (<v>my-app-length</v>) is an example of the first.  Since this is a \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> rule, whenever in a later proof an instance of the form <v>(<see topic=\"ACL2____LEN\">LEN</see>\u000A (MY-APP X Y))</v> is encountered, it will be rewritten to the corresponding \u000A instance of <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">LEN</see> X) (<see topic=\"ACL2____LEN\">LEN</see> Y))</v>.  Disabling the rule by executing the \u000A <see topic=\"ACL2____COMMAND\">command</see></p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> my-app-length)),</code> \u000A \u000A <p>is an example of a global change to the behavior of the prover since this \u000A <see topic=\"ACL2____REWRITE\">rewrite</see> will not be performed subsequently (unless the rule is again \u000A <see topic=\"ACL2____ENABLE\">enable</see>d).  Finally, we can add a (local) <see topic=\"ACL2____DISABLE\">disable</see> ``hint'' to a \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, meaning to <see topic=\"ACL2____DISABLE\">disable</see> the lemma only in the proof of one \u000A or more subgoals.  For example:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> my-app-length-commutativity\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (my-app x y))\u000A         (<see topic=\"ACL2____LEN\">len</see> (my-app y x)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> my-app-length))))</code> \u000A \u000A <p>In this case, the hint supplied is a bad idea since the proof is much \u000A harder with the hint than without it.  Try it both ways.</p> \u000A \u000A <p>By the way, to undo the previous event use <v>:u</v> (see <see topic=\"ACL2____U\">u</see>).  To undo \u000A back to some earlier event use <v>:ubt</v> (see <see topic=\"ACL2____UBT\">ubt</see>).  To view the \u000A current event use <v>:pe :here</v>.  To list several <see topic=\"ACL2____EVENTS\">events</see> use <v>:pbt</v> \u000A (see <see topic=\"ACL2____PBT\">pbt</see>).</p> \u000A \u000A <p>Notice the form of the hint in the previous example (see <see topic=\"ACL2____HINTS\">hints</see>).  It \u000A specifies a goal to which the hint applies.  <v>&quot;Goal&quot;</v> refers to the \u000A top-level goal of the theorem.  Subgoals are given unique names as they are \u000A generated.  It may be useful to suggest that a function symbol be <see topic=\"ACL2____DISABLE\">disable</see>d only for Subgoal 1.3.9, say, and a different function <see topic=\"ACL2____ENABLE\">enable</see>d \u000A only on Subgoal 5.2.8.  Overuse of such <see topic=\"ACL2____HINTS\">hints</see> often suggests a poor \u000A global proof strategy.</p> \u000A \u000A <p>We now recommend that you visit <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> on additional \u000A examples.  See <see topic=\"ACL2____ANNOTATED-ACL2-SCRIPTS\">annotated-ACL2-scripts</see>.</p>"],

 "ACL2____ACKNOWLEDGMENTS":[["About-ACL2"],"ACL2 Sources","ACL2","<p>The development of ACL2 was initially made possible by funding from \u000A the U. S. Department of Defense, including ARPA and ONR.  We thank all the \u000A organizations that have contributed support, including the following (in \u000A alphabetical order).</p> \u000A \u000A <blockquote> \u000A \u000A <p>o AMD, for providing significant time over several years for Matt Kaufmann \u000A to carry out ACL2 research, support, and development<br></br> \u000A \u000A o Computational Logic, Inc. and its president, Don Good, where the first \u000A eight years of ACL2 development occurred<br></br> \u000A \u000A o Centaur Technology<br></br> \u000A \u000A o DARPA<br></br> \u000A \u000A o Digital Equipment Corporation<br></br> \u000A \u000A o EDS, which provided some time for Matt Kaufmann's ACL2 work 1998-1999<br></br> \u000A \u000A o ForrestHunt and, more generally, Warren A. Hunt, Jr. (see below)<br></br> \u000A \u000A o IBM<br></br> \u000A \u000A o NSF<br></br> \u000A \u000A o ONR<br></br> \u000A \u000A o Rockwell Collins<br></br> \u000A \u000A o SRC<br></br> \u000A \u000A o Sun Microsystems<br></br> \u000A \u000A o University of Texas at Austin (in particular support to J Moore through the \u000A Admiral B. R.  Inman Chair of Computing Theory)</p> \u000A \u000A </blockquote> \u000A \u000A <p>We are especially grateful to Warren A. Hunt, Jr. for his unrivaled efforts \u000A in securing support for the entire ACL2 research group at both Computational \u000A Logic, Inc., and the University of Texas at Austin.  Without his efforts, we \u000A would have spent less time working on the system and fewer students would have \u000A been funded to apply it.</p> \u000A \u000A <p>ACL2 was started in August, 1989 by Boyer and Moore working together.  They \u000A co-authored the first versions of axioms.lisp and basis.lisp, with Boyer \u000A taking the lead in the formalization of ``<see topic=\"ACL2____STATE\">state</see>'' and the most \u000A primitive <see topic=\"ACL2____IO\">io</see> functions.  Boyer also had a significant hand in the \u000A development of the early versions of the files interface-raw.lisp and \u000A translate.lisp.  For several years, Moore alone was responsible for developing \u000A the ACL2 system code, though he consulted often with both Boyer and Kaufmann. \u000A In August, 1993, Kaufmann became jointly responsible with Moore for developing \u000A the system.  Boyer has continued to provide valuable consulting on an informal \u000A basis.</p> \u000A \u000A <p>Bishop Brock was the heaviest early user of ACL2, and provided many \u000A suggestions for improvements.  In particular, the <v>:cases</v> and \u000A <v>:restrict</v> <see topic=\"ACL2____HINTS\">hints</see> were his idea; he developed an early version of \u000A congruence-based reasoning for Nqthm; and he helped in the development of some \u000A early <see topic=\"ACL2____BOOKS\">books</see> about arithmetic.  In a demonstration of his courage and \u000A faith in us, he pushed for Computational Logic, Inc., to agree to the Motorola \u000A CAP contract -- which required formalizing a commercial DSP in the untested \u000A ACL2 -- and moved to Scottsdale, AZ, to do the work with the Motorola design \u000A team.  His demonstration of ACL2's utility was an inspiration, even to those \u000A of us designing ACL2.</p> \u000A \u000A <p>John Cowles also helped in the development of some early <see topic=\"ACL2____BOOKS\">books</see> about \u000A arithmetic, and also provided valuable feedback and bug reports.</p> \u000A \u000A <p>Other early users of ACL2 at Computational Logic, Inc. helped influence its \u000A development.  In particular, Warren Hunt helped with the port to Macintosh \u000A Common Lisp, and Art Flatau and Mike Smith provided useful general \u000A feedback.</p> \u000A \u000A <p>Mike Smith helped develop the Emacs portion of the implementation of proof \u000A trees.</p> \u000A \u000A <p>Bill Schelter made some enhancements to akcl (now gcl) that helped to \u000A enhance ACL2 performance in that Common Lisp implementation, and more \u000A generally, responded helpfully to our bug reports.  Camm Maguire has since \u000A provided wonderful gcl support, and has created a Debian package for ACL2 \u000A built on GCL.  We are also grateful to developers of other Common Lisp \u000A implementations.</p> \u000A \u000A <p>Kent Pitman helped in our interaction with the ANSI Common Lisp \u000A standardization committee, X3J13.</p> \u000A \u000A <p>John Cowles helped with the port to Windows (98) by answering questions and \u000A running tests.</p> \u000A \u000A <p>Ruben Gamboa created a modification of ACL2 to allow reasoning about the \u000A real numbers using non-standard analysis.  His work has been incorporated into \u000A the ACL2 distribution; see <see topic=\"COMMON-LISP____REAL\">real</see>.</p> \u000A \u000A <p>Rob Sumners has made numerous useful suggestions.  In particular, he has \u000A designed and implemented improvements for <see topic=\"ACL2____STOBJ\">stobj</see>s and been key in our \u000A development of locally-bound stobjs; see <see topic=\"ACL2____NOTE-2-6\">note-2-6</see>.</p> \u000A \u000A <p>Robert Krug has designed and implemented many changes in the vicinity of \u000A the linear arithmetic package and its connection to type-set and rewrite.  He \u000A was also instrumental in the development of <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.</p> \u000A \u000A <p>Pete Manolios has made numerous useful suggestions.  In particular, Pete \u000A helped us to organize the first workshop and was a wonderful equal partner \u000A with the two of us (Kaufmann and Moore) in producing the books that arose from \u000A that workshop.  Pete and his student, Daron Vroon, provided the current \u000A implementation of <see topic=\"ACL2____ORDINALS\">ordinals</see>.</p> \u000A \u000A <p>Jared Davis and Sol Swords have our gratitude for starting the <a \u000A href='http://acl2-books.googlecode.com/'>ACL2 Books repository</a>.</p> \u000A \u000A <p>We thank David L. Rager for contributing an initial version of the support \u000A for <see topic=\"ACL2____PARALLELISM\">parallelism</see> in an experimental extension of ACL2.</p> \u000A \u000A <p>Bob Boyer and Warren A. Hunt, Jr. developed a canonical representation for \u000A ACL2 data objects and a function memoization mechanism to facilitate reuse of \u000A previously computed results.  We thank them for their extensive efforts for \u000A the corresponding experimental (as of 2008 and 2009) extension of ACL2; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>We also thank the contributors to the ACL2 workshops for some suggested \u000A improvements and for the extensive collection of publicly distributed \u000A benchmark problems.  And we thank participants at the ACL2 seminar at the \u000A University of Texas for useful feedback.  More generally, we thank the ACL2 \u000A community for feedback, contributed <see topic=\"ACL2____BOOKS\">books</see> (see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>), \u000A and their interest in the ACL2 project.</p> \u000A \u000A <p><i>Regarding the documentation:</i></p> \u000A \u000A <blockquote><p>Bill Young wrote significant portions of the original \u000A <v>acl2-tutorial</v> section of the ACL2 documentation, including what is now \u000A called <see topic=\"ACL2____ALTERNATIVE-INTRODUCTION\">alternative-introduction</see>.  This was an especially important task \u000A in the early years when there was no guide for how to use ACL2 and we are very \u000A grateful.  He, Bishop Brock, Rich Cohen, and Noah Friedman read over \u000A considerable amounts of the documentation, and made many useful comments. \u000A Others, particularly Bill Bevier and John Cowles, have also made useful \u000A comments on the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>Art Flatau helped develop the ACL2 <see topic=\"ACL2____MARKUP\">markup</see> language and translators \u000A from that language to Texinfo and HTML.  Michael ``Bogo'' Bogomolny created a \u000A search engine, beginning with Version 2.6, and for that purpose modified the \u000A HTML translator to create one file per topic (a good idea in any case).</p> \u000A \u000A <p>Laura Lawless provided many hours of help in marking up appropriate parts \u000A of the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> in typewriter font.</p> \u000A \u000A <p>Noah Friedman developed an Emacs tool that helped us insert ``invisible \u000A links'' into the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, which improve the usability of that \u000A documentation under HTML readers such as Mosaic.</p> \u000A \u000A <p>Richard Stallman contributed a texinfo patch, to be found in the file \u000A <v>doc/texinfo.tex</v>.</p></blockquote> \u000A \u000A "],

 "ACL2____USING-COMPUTED-HINTS-6":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>So far none of our computed hints have used the \u000A <v>:COMPUTED-HINT-REPLACEMENT</v> feature.  We now illustrate that.</p> \u000A \u000A <p>The <v>:computed-hint-replacement</v> feature can easily lead to loops.  So \u000A as you experiment with the examples in this section and your own hints using \u000A this feature, be ready to interrupt the theorem prover and abort!</p> \u000A \u000A <p>A non-looping use of the <v>:computed-hint-replacement</v> feature would be a \u000A hint like this:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (certain-terms-present clause)\u000A    '(:computed-hint-replacement t\u000A      :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> lemma25))\u000A    '(:computed-hint-replacement t\u000A      :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> lemma25)))</code> \u000A \u000A <p>In this hint, if certain terms are present in <v>clause</v>, as determined by \u000A the function with the obvious name (here undefined), then this hint enables \u000A <v>lemma25</v> and otherwise disables it.  <v>Lemma25</v> might be a very \u000A expensive lemma, e.g., one that matches frequently and has an expensive and \u000A rarely established hypothesis.  One might wish it enabled only under certain \u000A conditions.  Recall that theories are inherited by children.  So once \u000A <v>lemma25</v> is enabled it ``stays'' enabled for the children, until disabled; \u000A and vice versa.  If the <v>:computed-hint-replacement</v> feature were not \u000A present and computed hints were always deleted after they had been used, then \u000A <v>lemma25</v> would be left enabled (or disabled) for all the childen produced \u000A by the first firing of the hint.  But with the arrangement here, every subgoal \u000A gets a theory deemed suitable by the hint, and the hint persists.</p> \u000A \u000A <p>Now we will set up a toy to allow us to play with computed hints to \u000A understand them more deeply.  To follow the discussion it is best to execute \u000A the following events.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> wrapper (x) t)\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> wrapper-axiom (wrapper x) :rule-classes nil)</code> \u000A \u000A <p>Now submit the following event and watch what happens.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints (`(:use (:instance wrapper-axiom (x a)))))</code> \u000A \u000A <p>The theorem prover adds <v>(wrapper a)</v> to the goal and then abandons the \u000A proof attempt because it cannot prove the subgoal.  Since the computed hint is \u000A deleted upon use, the hint is not applied to the subgoal (i.e., the child of \u000A the goal).</p> \u000A \u000A <p>What happens if we do the following?</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints (`(:computed-hint-replacement t\u000A            :use (:instance wrapper-axiom (x a)))))</code> \u000A \u000A <p>As one might expect, this loops forever: The hint is applied to the child \u000A and adds the hypothesis again.  When the hint fires, nothing is actually \u000A changed, since <v>(wrapper a)</v> is already in the subgoal.</p> \u000A \u000A <p>So let's change the experiment a little.  Let's make the hint add the \u000A hypothesis <v>(wrapper p)</v> where <v>p</v> is the first literal of the clause. \u000A This is silly but it allows us to explore the behavior of computed hints a \u000A little more.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints (`(:use (:instance wrapper-axiom (x ,(<see topic=\"COMMON-LISP____CAR\">car</see> clause))))))</code> \u000A \u000A <p>So in this case, the theorem prover changes the goal to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (WRAPPER (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V)) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))</code> \u000A \u000A <p>which then simplifies to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (WRAPPER NIL) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))</code> \u000A \u000A <p>because the concluding equality can be assumed false in the hypothesis \u000A (e.g., think of the contrapositive version).  Nothing else happens because the \u000A hint has been removed and so is not applicable to the child.</p> \u000A \u000A <p>Now consider the following -- and be ready to interrupt it and abort!</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints (`(:computed-hint-replacement t\u000A            :use (:instance wrapper-axiom (x ,(<see topic=\"COMMON-LISP____CAR\">car</see> clause))))))</code> \u000A \u000A <p>This time the hint is not removed and so is applied to the child.  So from \u000A <v>Goal</v> we get</p> \u000A \u000A <code>Goal'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (WRAPPER (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))</code> \u000A \u000A <p>and then</p> \u000A \u000A <code>Goal''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (WRAPPER (<see topic=\"COMMON-LISP____NOT\">NOT</see> (WRAPPER (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))))\u000A              (WRAPPER (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V))</code> \u000A \u000A <p>etc.</p> \u000A \u000A <p>First, note that the hint is repeatedly applied to its children.  That is \u000A because we wrote <v>:computed-hint-replacement t</v>.  But second, note that \u000A <v>Goal'</v> is not even being simplified before <v>Goal''</v> is produced from \u000A it.  If it were being simplified, the <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)</v>'s in the hypotheses \u000A would be replaced by <v>nil</v>.  This is a feature.  It means after a computed \u000A hint has fired, other hints are given a chance at the result, even the hint \u000A itself unless it is removed from the list of hints.</p> \u000A \u000A <p>As an exercise, let's arrange for the hint to stay around and be applied \u000A indefinitely but with a simplification between each use of the the hint.  To \u000A do this we need to pass information from one application of the hint to the \u000A next, essentially to say ``stay around but don't fire.''</p> \u000A \u000A <p>First, we will define a function to use in the hint.  This is more than a \u000A mere convenience; it allows the hint to ``reproduce itself'' in the \u000A replacement.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> wrapper-challenge (clause parity)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> parity\u000A      `(:computed-hint-replacement ((wrapper-challenge clause nil))\u000A        :use (:instance wrapper-axiom (x ,(<see topic=\"COMMON-LISP____CAR\">car</see> clause))))\u000A      `(:computed-hint-replacement ((wrapper-challenge clause t)))))</code> \u000A \u000A <p>Note that this function is not recursive, even though it uses its own name. \u000A That is because the occurrence of its name is in a quoted constant.</p> \u000A \u000A <p>Now consider the following.  What will it do?</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints ((wrapper-challenge clause t)))</code> \u000A \u000A <p>First, observe that this is a legal hint because it is a term that mentions \u000A only the free variable <v>CLAUSE</v>.  When defining hint functions you may \u000A sometimes think their only arguments are the four variables <v>id</v>, \u000A <v>clause</v>, <v>world</v>, and <v>stable-under-simplificationp</v>.  That is not \u000A so.  But in your hints you must call those functions so that those are the \u000A only free variables.  Note also that the occurrence of <v>clause</v> inside the \u000A <v>:computed-hint-replacement</v> is not an occurrence of the variable clause \u000A but just a constant.  Just store this note away for a moment.  We'll return to \u000A it momentarily.</p> \u000A \u000A <p>Second, the basic cleverness of this hint is that every time it fires it \u000A reproduces itself with the opposite parity.  When the parity is <v>t</v> it \u000A actually changes the goal by adding a hypothesis.  When the parity is <v>nil</v> \u000A it doesn't change the goal and so allows simplification to proceed -- but it \u000A swaps the parity back to <v>t</v>.  What you can see with this simple toy is \u000A that we can use the computed hints to pass information from parent to \u000A child.</p> \u000A \u000A <p>Ok, so what happens when the event above is executed?  Try it.  You will \u000A see that ACL2 applied the hint the first time.  It doesn't get around to \u000A printing the output because an error is caused before it can print.  But here \u000A is a blow-by-blow description of what happens.  The hint is evaluated on \u000A <v>Goal</v> with the <v>clause</v> <v>((<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v))</v>.  It produces a hint exactly \u000A as though we had typed:</p> \u000A \u000A <code>(&quot;Goal&quot; :use (:instance wrapper-axiom (x (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v))))</code> \u000A \u000A <p>which is applied to this goal. In addition, it produces the new hints \u000A argument</p> \u000A \u000A <code>:hints ((wrapper-challenge clause nil)).</code> \u000A \u000A <p>By applying the <v>&quot;Goal&quot;</v> hint we get the new subgoal</p> \u000A \u000A <code>Goal'\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (wrapper (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v))</code> \u000A \u000A <p>but this is not printed because, before printing it, the theorem prover \u000A looks for hints to apply to it and finds</p> \u000A \u000A <code>(wrapper-challenge clause nil)</code> \u000A \u000A <p>That is evaluated and produces a hint exactly as though we had typed:</p> \u000A \u000A <code>(&quot;Goal'&quot; )</code> \u000A \u000A <p>and the new hints argument:</p> \u000A \u000A <code>:hints ((wrapper-challenge clause nil)).</code> \u000A \u000A <p>But if you supply the hint <v>(&quot;Goal'&quot; )</v>, ACL2 will signal an error \u000A because it does not allow you to specify an empty hint!</p> \u000A \u000A <p>So the definition of <v>wrapper-challenge</v> above is almost correct but \u000A fatally flawed.  We need a non-empty ``no-op'' hint.  One such hint is to tell \u000A the system to expand a term that will always be expanded anyway.  So undo \u000A <v>wrapper-challenge</v>, redefine it, and try the proof again.  Now remember \u000A the observation about <v>clause</v> that we asked you to ``store'' above.  The \u000A new definition of <v>wrapper-challenge</v> illustrates what we meant.  Note that \u000A the first formal parameter of <v>wrapper-challenge</v>, below, is no longer \u000A named <v>clause</v> but is called <v>cl</v> instead.  But the ``call'' of \u000A <v>wrapper-challenge</v> in the replacements is on <v>clause</v>.  This may seem \u000A to violate the rule that a function definition cannot use variables other than \u000A the formals.  But the occurrences of <v>clause</v> below are not variables but \u000A constants in an object that will eventually be treated as hint term.</p> \u000A \u000A <code>:ubt wrapper-challenge\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> wrapper-challenge (cl parity)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> parity\u000A      `(:computed-hint-replacement ((wrapper-challenge clause nil))\u000A        :use (:instance wrapper-axiom (x ,(<see topic=\"COMMON-LISP____CAR\">car</see> cl))))\u000A      `(:computed-hint-replacement ((wrapper-challenge clause t))\u000A        :expand ((<see topic=\"COMMON-LISP____ATOM\">atom</see> zzz)))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)\u000A  :hints ((wrapper-challenge clause t)))</code> \u000A \u000A <p>This time, things go as you might have expected!  <v>Goal'</v> is produced \u000A and simplified, to</p> \u000A \u000A <code>Goal''\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (wrapper nil)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u v)).</code> \u000A \u000A <p>Simplification gets a chance because when the new hint \u000A <v>(wrapper-challenge clause nil)</v> is fired it does not change the goal.  But \u000A it does change the parity in the hints argument so that before <v>Goal''</v> is \u000A simplified again, the hint fires and adds the hypothesis:</p> \u000A \u000A <code>Goal'''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (WRAPPER (<see topic=\"COMMON-LISP____NOT\">NOT</see> (WRAPPER NIL)))\u000A              (WRAPPER NIL))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V)).</code> \u000A \u000A <p>This simplifies, replacing the first <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (WRAPPER NIL))</v> by <v>NIL</v>, \u000A since <v>(WRAPPER NIL)</v> is known to be true here.  Thus the goal simplifies \u000A to</p> \u000A \u000A <code>Goal'4'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (WRAPPER NIL) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> U V)).</code> \u000A \u000A <p>The process repeats indefinitely.</p> \u000A \u000A <p>So we succeeded in getting a hint to fire indefinitely but allow a full \u000A simplification between rounds.</p>"],

 "ACL2____STOBJ-EXAMPLE-3":[["Stobj"],"ACL2 Sources","ACL2","<p>The event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> $s\u000A  (x :type integer :initially 0)\u000A  (a :type (array (integer 0 9) (3)) :initially 9 :resizable t))</code> \u000A \u000A <p>introduces a stobj named <v>$S</v>.  The stobj has two fields, <v>X</v> and \u000A <v>A</v>.  The <v>A</v> field is an array.  The <v>X</v> field contains an integer \u000A and is initially 0.  The <v>A</v> field contains a list of integers, each \u000A between 0 and 9, inclusively.  (Under the hood, this ``list'' is actually \u000A implemented as an array.)  Initially, the <v>A</v> field has three elements, \u000A each of which is 9.</p> \u000A \u000A <p>This event introduces the following sequence of function definitions:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> XP (X) ...)               ; recognizer for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AP (X) ...)               ; recognizer of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> $SP ($S) ...)             ; top-level recognizer for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> CREATE-$S NIL ...)        ; creator for stobj $S\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> X ($S) ...)               ; accessor for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-X (V $S) ...)      ; updater for X field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A-LENGTH ($S) ...)        ; length of A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> RESIZE-A (K $S) ...)      ; resizer for A field\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> AI (I $S) ...)            ; accessor for A field at index I\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-AI (I V $S) ...)   ; updater for A field at index I</code> \u000A \u000A <p>Here is the definition of <v>$SP</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> $SP ($S)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD T :VERIFY-GUARDS T))\u000A  (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> $S)\u000A       (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP____LENGTH\">LENGTH</see> $S) 2)\u000A       (XP (<see topic=\"COMMON-LISP____NTH\">NTH</see> 0 $S))\u000A       (AP (<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S))\u000A       T))</code> \u000A \u000A <p>This reveals that in order to satisfy <v>$SP</v> an object must be a true \u000A list of length 2 whose first element satisfies <v>XP</v> and whose second \u000A satisfies <v>AP</v>.  By printing the definition of <v>AP</v> one learns that it \u000A requires its argument to be a true list, each element of which is an integer \u000A between 0 and 9.</p> \u000A \u000A <p>The initial value of stobj <v>$S</v> is given by zero-ary ``creator'' \u000A function <v>CREATE-$S</v>.  Creator functions may only be used in limited \u000A contexts.  See <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>.</p> \u000A \u000A <p>Here is the definition of <v>UPDATE-AI</v>, the updater for the <v>A</v> field \u000A at index <v>I</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> UPDATE-AI (I V $S)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD\u000A                  (<see topic=\"COMMON-LISP____AND\">AND</see> ($SP $S)\u000A                       (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> I)\u000A                       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 I)\u000A                       (<see topic=\"COMMON-LISP_____C3\">&lt;</see> I (A-LENGTH $S))\u000A                       (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> V) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 V) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> V 9)))\u000A                  :VERIFY-GUARDS T))\u000A  (UPDATE-NTH-ARRAY 1 I V $S))</code> \u000A \u000A <p>By definition <v>(UPDATE-NTH-ARRAY 1 I V $S)</v> is <v>(<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> 1\u000A (<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> I V (<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S)) $S)</v>.  This may be a little surprising but should \u000A be perfectly clear.</p> \u000A \u000A <p>First, ignore the guard, since it is irrelevant in the logic.  Reading from \u000A the inside out, <v>(UPDATE-AI I V $S)</v> extracts <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S)</v>, which is \u000A array <v>a</v> of <v>$S</v>.  (Recall that <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> is 0-based.)  The next \u000A higher expression in the definition above, <v>(<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> I V a)</v>, \u000A ``modifies'' <v>a</v> by setting its <v>I</v>th element to <v>V</v>.  Call this \u000A <v>a'</v>.  The next higher expression, <v>(<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> 1 a' $S)</v>, ``modifies'' \u000A <v>$S</v> by setting its 1st component to <v>a'</v>.  Call this result <v>$s'</v>. \u000A Then <v>$s'</v> is the result returned by <v>UPDATE-AI</v>.</p> \u000A \u000A <p>So the first useful observation is that from the perspective of the logic, \u000A the type ``restrictions'' on stobjs are irrelevant.  They are ``enforced'' by \u000A ACL2's guard mechanism, not by the definitions of the updater functions.</p> \u000A \u000A <p>As one might also imagine, the accessor functions do not really ``care,'' \u000A logically, whether they are applied to well-formed stobjs or not.  For \u000A example, <v>(AI I $S)</v> is defined to be <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> I (<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S))</v>.</p> \u000A \u000A <p>Thus, you will not be able to prove that (AI 2 $S) is an integer.  That \u000A is,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (AI 2 $S))</code> \u000A \u000A <p>is not a theorem, because <v>$S</v> may not be well-formed.</p> \u000A \u000A <p>Now <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (AI 2 $S))</v> will always evaluate to <v>T</v> in the \u000A top-level ACL2 command loop, because we insist that the current value of the \u000A stobj <v>$S</v> always satisfies <v>$SP</v> by enforcing the guards on the \u000A updaters, independent of whether guard checking is on or off; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  But in a theorem <v>$S</v> is just another variable, \u000A implicitly universally quantified.</p> \u000A \u000A <p>So <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (AI 2 $S))</v> is not a theorem because it is not true when \u000A the variable <v>$S</v> is instantiated with, say,</p> \u000A \u000A <code>'(1 (0 1 TWO))</code> \u000A \u000A <p>because, logically speaking, <v>(AI 2 '(1 (0 1 TWO)))</v> evaluates to the \u000A symbol <v>TWO</v>.  That is,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (AI 2 '(1 (0 1 TWO))) 'TWO)</code> \u000A \u000A <p>is true.</p> \u000A \u000A <p>However,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> ($SP $S) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 2 (A-LENGTH $S))) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (AI 2 $S)))</code> \u000A \u000A <p>is a theorem.  To prove it, you will have to prove a lemma about <v>AP</v>. \u000A The following will do:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> ap-nth\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (AP x)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"ACL2____LEN\">len</see> x)))\u000A           (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> i x)))).</code> \u000A \u000A <p>Similarly,</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (A-LENGTH $S))\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> v 9))\u000A         ($SP (UPDATE-AI i v $S)))</code> \u000A \u000A <p>is not a theorem until you add the additional hypothesis <v>($SP $S)</v>.  To \u000A prove the resulting theorem, you will need a lemma such as the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> ap-update-nth\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (AP a)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 v)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> v 9)\u000A                (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"ACL2____LEN\">len</see> a)))\u000A           (AP (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i v a))))</code> \u000A \u000A <p>The moral here is that from the logical perspective, you must provide the \u000A hypotheses that, as a programmer, you think are implicit on the structure of \u000A your stobjs, and you must prove their invariance.  This is a good area for \u000A further support, perhaps in the form of a library of macros.</p> \u000A \u000A <p><i>Resizing Array Fields</i></p> \u000A \u000A <p>Recall the specification of the array field, <v>A</v> for the stobj <v>$S</v> \u000A introduced above:</p> \u000A \u000A <code>(a :type (array (integer 0 9) (3)) :initially 9 :resizable t)</code> \u000A \u000A <p>Logically, this field is a list, initially of length 3.  Under the hood, \u000A this field is implemented using a Common Lisp array with 3 elements.  In some \u000A applications, one may wish to lengthen an array field, or even (to reclaim \u000A space) to shrink an array field.  The <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event provides \u000A functions to access the current length of an array field and to change the \u000A array field, with default names obtained by suffixing the field name with \u000A ``<v>LENGTH-</v>'' or prefixing it with ``<v>RESIZE-</v>,'' respectively.  The \u000A following log shows the uses of these fields in the above example.</p> \u000A \u000A <code>ACL2 !&gt;(A-LENGTH $S)\u000A3\u000AACL2 !&gt;(RESIZE-A 10 $S) ; change length of A to 10\u000A&lt;$s&gt;\u000AACL2 !&gt;(A-LENGTH $S)\u000A10\u000AACL2 !&gt;(AI 7 $S)        ; new elements get value from :initially\u000A9\u000AACL2 !&gt;(RESIZE-A 2 $S)  ; truncate A down to first 2 elements\u000A&lt;$s&gt;\u000AACL2 !&gt;(A-LENGTH $S)\u000A2\u000AACL2 !&gt;(AI 7 $S)        ; error:  access past array bound\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol AI, which\u000Ais (<see topic=\"COMMON-LISP____AND\">AND</see> ($SP $S) (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> I) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 I) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> I (A-LENGTH $S))), is violated\u000Aby the arguments in the call (AI 7 $S).\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>Here are the definitions of the relevant functions for the above example; \u000A also see <see topic=\"ACL2____RESIZE-LIST\">resize-list</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A-LENGTH ($S)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD ($SP $S) :VERIFY-GUARDS T))\u000A  (<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> RESIZE-A (K $S)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD ($SP $S) :VERIFY-GUARDS T))\u000A  (<see topic=\"ACL2____UPDATE-NTH\">UPDATE-NTH</see> 1\u000A              (<see topic=\"ACL2____RESIZE-LIST\">RESIZE-LIST</see> (<see topic=\"COMMON-LISP____NTH\">NTH</see> 1 $S) K 9)\u000A              $S))</code> \u000A \u000A <p>It is important to note that the implementation of array resizing in ACL2 \u000A involves copying the entire array into a newly allocated space and thus can be \u000A quite costly if performed often.  This approach was chosen in order to make \u000A array access and update as efficient as possible, with the suspicion that for \u000A most applications, array access and update are considerably more frequent than \u000A resizing (especially if the programmer is aware of the relative costs \u000A beforehand).</p> \u000A \u000A <p>It should also be noted that computations of lengths of stobj array fields \u000A should be fast (constant-time) in all or most Common Lisp implementations.</p> \u000A \u000A <p>Finally, if <v>:resizable t</v> is not supplied as shown above, then an \u000A attempt to resize the array will result in an error.  If you do not intend to \u000A resize the array, it is better to omit the <v>:resizable</v> option (or to \u000A supply <v>:resizable nil</v>), since then the length function will be defined to \u000A return a constant, namely the initial length, which can simplify guard proofs \u000A (compare with the definition of <v>A-LENGTH</v> above).</p> \u000A \u000A <p>This completes the tour through the documentation of <see topic=\"ACL2____STOBJ\">stobj</see>s. \u000A However, you may now wish to read the documentation for the event that \u000A introduces a new single-threaded object; see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.</p>"],

 "ACL2____REDEFINING-PROGRAMS":[["Programming"],"ACL2 Sources","ACL2","<p>ACL2 does not in general allow the redefinition of functions \u000A because logical inconsistency can result: previously stored theorems can be \u000A rendered invalid if the axioms defining the functions involved are changed. \u000A However, to permit prototyping of both <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode systems, ACL2 permits redefinition if the user has accepted \u000A logical responsibility for the consequences by setting <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> to an appropriate non-<v>nil</v> value.  The refusal of \u000A ACL2 to support the unrestricted redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode \u000A functions may appear somewhat capricious.  After all, what are the logical \u000A consequences of changing a definition if no axioms are involved?</p> \u000A \u000A <p>Three important points should be made before we discuss redefinition \u000A further.</p> \u000A \u000A <p>The first is that ACL2 does support redefinition (of both <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> functions) when <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> is non-<v>nil</v>.</p> \u000A \u000A <p>The second is that a ``redefinition'' that does not change the mode, \u000A formals, guards, type declarations, stobjs, or body of a function is \u000A considered redundant and is permitted even when <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> \u000A is <v>nil</v>.  We recognize and permit redundant definitions because it is not \u000A uncommon for two distinct <see topic=\"ACL2____BOOKS\">books</see> to share identical function \u000A definitions.  When determining whether the body of a function is changed by a \u000A proposed redefinition, we actually compare the untranslated versions of the \u000A two bodies.  See <see topic=\"ACL2____TERM\">term</see>.  For example, redundancy is not recognized if \u000A the old body is <v>(<see topic=\"COMMON-LISP____LIST\">list</see> a b)</v> and the new body is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> a (<see topic=\"COMMON-LISP____CONS\">cons</see> b\u000A nil))</v>.  We use the untranslated bodies because of the difficulty of \u000A translating the new body in the presence of the old syntactic information, \u000A given the possibility that the redefinition might attempt to change the <see topic=\"ACL2____SIGNATURE\">signature</see> of the function, i.e., the number of formals, the number of \u000A results, or the position of single-threaded objects in either.</p> \u000A \u000A <p>The third important point is that a ``redefinition'' that preserves the \u000A formals, guard, types, stobjs, and body but changes the mode from <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> is permitted even when <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> is <v>nil</v>.  That is what <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> \u000A does.</p> \u000A \u000A <p>This note addresses the temptation to allow redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions in situations other than the three described above. \u000A Therefore, suppose <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> is <v>nil</v> and consider the \u000A cases.</p> \u000A \u000A <p>Case 1.  Suppose the new definition attempts to change the formals or more \u000A generally the <see topic=\"ACL2____SIGNATURE\">signature</see> of the function.  Accepting such a redefinition \u000A would render ill-formed other <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions which call the \u000A redefined function.  Subsequent attempts to evaluate those callers could \u000A arbitrarily damage the Common Lisp image.  Thus, redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions under these circumstances requires the user's active \u000A approval, as would be sought with <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> <v>'(:query\u000A . :overwrite)</v>.</p> \u000A \u000A <p>Case 2.  Suppose the new definition attempts to change the body (even \u000A though it preserves the <see topic=\"ACL2____SIGNATURE\">signature</see>).  At one time we believed this was \u000A acceptable and ACL2 supported the quiet redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode functions in this circumstance.  However, because such functions can be \u000A used in macros and redundancy checking is based on untranslated bodies, this \u000A turns out to be unsound!  (Aside: Perhaps this is not an issue if the function \u000A takes <tt><see topic=\"ACL2____STATE\">state</see></tt> or a user-defined <see topic=\"ACL2____STOBJ\">stobj</see> argument; but we do not \u000A further consider this distinction.)  Such redefinition is therefore now \u000A prohibited.  We illustrate such an unsoundness below.  Let <v>foo-thm1.lisp</v> \u000A be a book with the following contents.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program)) (<see topic=\"COMMON-LISP____LIST\">list</see> 'if x 't 'nil))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> p (x) (p1 x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (p x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-thm1 (<see topic=\"ACL2____IFF\">iff</see> (foo x) x) :rule-classes nil)</code> \u000A \u000A <p>Note that the macro form <v>(p x)</v> translates to <v>(<see topic=\"COMMON-LISP____IF\">if</see> x t nil)</v>.  The \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> function <v>p1</v> is used to generate this translation. \u000A The function <v>foo</v> is defined so that <v>(foo x)</v> is <v>(p x)</v> and a \u000A theorem about <v>foo</v> is proved, namely, that <v>(foo x)</v> is true iff <v>x</v> \u000A is true.</p> \u000A \u000A <p>Now let <v>foo-thm2.lisp</v> be a book with the following contents.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program)) (<see topic=\"COMMON-LISP____LIST\">list</see> 'if x 'nil 't))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> p (x) (p1 x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (p x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-thm2 (<see topic=\"ACL2____IFF\">iff</see> (foo x) (<see topic=\"COMMON-LISP____NOT\">not</see> x)) :rule-classes nil)</code> \u000A \u000A <p>In this book, the <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> function <v>p1</v> is defined so that \u000A <v>(p x)</v> means just the negation of what it meant in the first book, namely, \u000A <v>(<see topic=\"COMMON-LISP____IF\">if</see> x nil t)</v>.  The function <v>foo</v> is defined identically &mdash; more \u000A precisely, the <i>untranslated</i> body of <v>foo</v> is identical in the two \u000A <see topic=\"ACL2____BOOKS\">books</see>, but because of the difference between the two versions of the \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> function <v>p1</v> the axioms defining the two <v>foo</v>s \u000A are different.  In the second book we prove the theorem that <v>(foo x)</v> is \u000A true iff <v>x</v> is nil.</p> \u000A \u000A <p>Now consider what would happen if the <see topic=\"ACL2____SIGNATURE\">signature</see>-preserving \u000A redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions were permitted and these two \u000A <see topic=\"ACL2____BOOKS\">books</see> were included.  When the second book is included the redefinition \u000A of <v>p1</v> would be permitted since the <see topic=\"ACL2____SIGNATURE\">signature</see> is preserved and \u000A <v>p1</v> is just a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>.  But then when the redefinition of \u000A <v>foo</v> is processed it would be considered redundant and thus be permitted. \u000A The result would be a logic in which it was possible to prove that <v>(foo\u000A x)</v> is equivalent to both <v>x</v> and <v>(<see topic=\"COMMON-LISP____NOT\">not</see> x)</v>.  In particular, the \u000A following sequence leads to a proof of nil:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo-thm1&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo-thm2&quot;)\u000A(<see topic=\"ACL2____THM\">thm</see> nil :hints ((&quot;Goal&quot; :use (foo-thm1 foo-thm2))))</code> \u000A \u000A <p>It might be possible to loosen the restrictions on the redefinition of \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions by allowing <see topic=\"ACL2____SIGNATURE\">signature</see>-preserving \u000A redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> functions not involved in macro \u000A definitions.  Alternatively, we could implement definition redundancy checking \u000A based on the translated bodies of functions (though that is quite \u000A problematic).  Barring those two changes, we believe it is necessary simply to \u000A impose the same restrictions on the redefinition of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode \u000A functions as we do on <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions.</p>"],

 "ACL2____POINTERS":[["Documentation"],"ACL2 Sources","ACL2",""],

 "ACL2____NOTE5":[["Release-notes"],"ACL2 Sources","ACL2","<p>Acl2 now allows ``complex rationals,'' which are complex numbers \u000A whose real parts are rationals and whose imaginary parts are non-zero \u000A rationals.  See <see topic=\"COMMON-LISP____COMPLEX\">complex</see>.</p> \u000A \u000A <p>A new way of handling <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses has been implemented. \u000A Rather than cause a case split at the time the <tt><see topic=\"ACL2____FORCE\">force</see></tt> occurs, we \u000A complete the main proof and then embark on one or more ``forcing rounds'' in \u000A which we try to prove the <see topic=\"ACL2____FORCE\">force</see>d hypotheses.  See <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>. \u000A To allow us to compare the new handling of <tt><see topic=\"ACL2____FORCE\">force</see></tt> with the old, Version \u000A 1.5 implements both and uses a flag in <tt><see topic=\"ACL2____STATE\">state</see></tt> to determine which method \u000A should be used.  Do <v>(<see topic=\"ACL2____ASSIGN\">assign</see> old-style-forcing t)</v> if you want <tt><see topic=\"ACL2____FORCE\">force</see></tt> to be handled as it was in Version 1.4.  However, we expect to \u000A eliminate the old-style forcing eventually because we think the new style is \u000A more effective.  To see the difference between the two approaches to forcing, \u000A try proving the associativity of <see topic=\"COMMON-LISP____APPEND\">append</see> under both settings of \u000A <v>old-style-forcing</v>.  To get the new behavior invoke:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> a) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> b))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> a b) c)\u000A                     (<see topic=\"COMMON-LISP____APPEND\">append</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)))))</code> \u000A \u000A <p>Then <v>(<see topic=\"ACL2____ASSIGN\">assign</see> old-style-forcing t)</v> and invoke the <v>thm</v> <see topic=\"ACL2____COMMAND\">command</see> above again.</p> \u000A \u000A <p>A new <v>:cases</v> <see topic=\"ACL2____HINTS\">hints</see> allows proof by cases.  See <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> now restore the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> when they complete.  See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> and see \u000A <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see>s on many Acl2 primitives defined in <v>axioms.lisp</v> have \u000A been weakened to permit them to be used in accordance with lisp custom and \u000A tradition.</p> \u000A \u000A <p>It is possible to attach heuristic filters to <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules \u000A to limit their applicability.  See <see topic=\"ACL2____SYNTAXP\">syntaxp</see>.</p> \u000A \u000A <p>A tutorial has been added (but as of Version_3.6.1 it has become \u000A obsolete).</p> \u000A \u000A <p><see topic=\"ACL2____EVENTS\">Events</see> now print the Summary paragraph listing <see topic=\"ACL2____RUNE\">rune</see>s used, \u000A time, etc., whether they succeed or fail.  The format of the ``<see topic=\"ACL2____FAILURE\">failure</see> \u000A banner'' has been changed but still has multiple asterisks in it.  <v>Thm</v> \u000A also prints a Summary, whether it succeeds or fails; but <v>thm</v> is not an \u000A event.</p> \u000A \u000A <p>A new event form <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> has been added; see <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>.</p> \u000A \u000A <p>A user-specific customization facility has been added in the form of a book \u000A that is automatically included, if it exists on the current directory.  See \u000A <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>A facility for conditional metalemmas has been implemented; see <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <p>The acceptable values for <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> have changed.  In the old \u000A version (Version 1.4), a value of <v>t</v> meant that proofs and <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> are to be skipped.  In Version 1.5, a value of <v>t</v> means \u000A proofs (but not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see>) are to be skipped.  A value of \u000A <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> means proofs and <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are to \u000A be skipped.  There are two other, more obscure, acceptable values.  See <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.</p> \u000A \u000A <p>In order to turn off the forcing of assumptions, one should now <see topic=\"ACL2____DISABLE\">disable</see> the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <tt><see topic=\"ACL2____FORCE\">force</see></tt> (rather \u000A than the <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> of <tt><see topic=\"ACL2____FORCE\">force</see></tt>, as in the previous \u000A release); see <see topic=\"ACL2____FORCE\">force</see>.</p> \u000A \u000A <p>The macros <tt><see topic=\"ACL2____ENABLE-FORCING\">enable-forcing</see></tt> and <tt><see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see></tt> make it \u000A convenient to <see topic=\"ACL2____ENABLE\">enable</see> or <see topic=\"ACL2____DISABLE\">disable</see> forcing.  See <see topic=\"ACL2____ENABLE-FORCING\">enable-forcing</see> and see <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>.</p> \u000A \u000A <p>The new commands <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> and <v>:</v><tt><see topic=\"ACL2____PR_12\">pr!</see></tt> print the rules \u000A created by an event or command.  See <see topic=\"ACL2____PR\">pr</see> and see <see topic=\"ACL2____PR_12\">pr!</see>.</p> \u000A \u000A <p>The new <see topic=\"ACL2____HISTORY\">history</see> <see topic=\"ACL2____COMMAND\">command</see>s <v>:</v><tt><see topic=\"ACL2____PUFF\">puff</see></tt> and <v>:</v><tt><see topic=\"ACL2____PUFF_A2\">puff*</see></tt> will replace a compound <see topic=\"ACL2____COMMAND\">command</see> such as an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> by the sequence of <see topic=\"ACL2____EVENTS\">events</see> in it.  That is, they \u000A ``<see topic=\"ACL2____PUFF\">puff</see> up'' or ``lift'' the subevents of a <see topic=\"ACL2____COMMAND\">command</see> to the <see topic=\"ACL2____COMMAND\">command</see> level, eliminating the formerly superior <see topic=\"ACL2____COMMAND\">command</see> and \u000A lengthening the <see topic=\"ACL2____HISTORY\">history</see>.  This is useful if you want to ``partially \u000A undo'' an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book or other compound <see topic=\"ACL2____COMMAND\">command</see> so you \u000A can experiment.  See <see topic=\"ACL2____PUFF\">puff</see> and see <see topic=\"ACL2____PUFF_A2\">puff*</see>.</p> \u000A \u000A <p>Theory expressions now are allowed to use the free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt> \u000A and prohibited from using the free variable <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____THEORIES\">theories</see>, although it is essentially the same as before except it mentions \u000A <tt><see topic=\"ACL2____WORLD\">world</see></tt> instead of <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____WORLD\">world</see> for a discussion of \u000A the Acl2 logical <see topic=\"ACL2____WORLD\">world</see>.  Allowing <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> to be \u000A state-sensitive violated an important invariant about how <see topic=\"ACL2____BOOKS\">books</see> \u000A behaved.</p> \u000A \u000A <p><tt><see topic=\"ACL2____TABLE\">Table</see></tt> keys and values now are allowed to use the free variable \u000A <tt><see topic=\"ACL2____WORLD\">world</see></tt> and prohibited from using the free variable <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See \u000A the note above about theory expressions for some explanation.</p> \u000A \u000A <p>The macro for minus, <tt><see topic=\"COMMON-LISP____-\">-</see></tt>, used to expand <v>(<see topic=\"COMMON-LISP____-\">-</see> x 3)</v> to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> x\u000A -3)</v> and now expands it to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> -3 x)</v> instead.  The old macro, if used in \u000A the left-hand sides of rewrite rules, produced inapplicable rules because the \u000A constant occurs in the second argument of the <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt>, but potential target \u000A terms generally had the constant in the first argument position because of the \u000A effect of <v>commutativity-of-+</v>.</p> \u000A \u000A <p>A new class of rule, <v>:linear-alias</v> rules, allows one to implement the \u000A nqthm package and similar hacks in which a <see topic=\"ACL2____DISABLE\">disable</see>d function is to be \u000A known equivalent to an arithmetic function.</p> \u000A \u000A <p>A new class of rule, <v>:built-in-clause</v> rules, allows one to extend the \u000A set of clauses proved silently by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> during measure and <see topic=\"ACL2____GUARD\">guard</see> processing.  See <see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see>.</p> \u000A \u000A <p>The new command <tt><see topic=\"ACL2____PCB_12\">pcb!</see></tt> is like <tt><see topic=\"ACL2____PCB\">pcb</see></tt> but sketches the <see topic=\"ACL2____COMMAND\">command</see> and then prints its subsidiary <see topic=\"ACL2____EVENTS\">events</see> in full.  See <see topic=\"ACL2____PCB_12\">pcb!</see>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____REWRITE\">Rewrite</see></tt> class rules may now specify the <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> and see <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.</p> \u000A \u000A <p>The rules for how <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>s control permutative rewrite rules \u000A have been changed.  One effect of this change is that now when the built-in \u000A commutativity rules for <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> are used, the terms <v>a</v> and <v>(<see topic=\"COMMON-LISP____-\">-</see> a)</v> \u000A are permuted into adjacency.  For example, <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b (<see topic=\"COMMON-LISP____-\">-</see> a))</v> is now \u000A normalized by the commutativity rules to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a (<see topic=\"COMMON-LISP____-\">-</see> a) b)</v>; in Version 1.4, \u000A <v>b</v> was considered syntactically smaller than <v>(<see topic=\"COMMON-LISP____-\">-</see> a)</v> and so <v>(<see topic=\"COMMON-LISP_____B2\">+</see> a b\u000A (<see topic=\"COMMON-LISP____-\">-</see> a))</v> is considered to be in normal form.  Now it is possible to arrange \u000A for unary functions be be considered ``invisible'' when they are used in \u000A certain contexts.  By default, <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> is considered invisible when \u000A its application appears in the argument list of <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>.  See <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see> and see :DOC set-invisible-fns-table.</p> \u000A \u000A <p>Extensive documentation has been provided on the topic of Acl2's ``term \u000A ordering.''  See <see topic=\"ACL2____TERM-ORDER\">term-order</see>.</p> \u000A \u000A <p>Calls of <tt><see topic=\"ACL2____LD\">ld</see></tt> now default <tt><see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see></tt> to <v>:return</v> \u000A rather than to the current setting.</p> \u000A \u000A <p>The <see topic=\"ACL2____COMMAND\">command</see> descriptor <v>:x</v> has been introduced and is synonymous \u000A with <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt>, the most recently executed <see topic=\"ACL2____COMMAND\">command</see>.  <see topic=\"ACL2____HISTORY\">History</see> <see topic=\"ACL2____COMMAND\">command</see>s such as <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> print a <v>:x</v> beside the \u000A most recent <see topic=\"ACL2____COMMAND\">command</see>, simply to indicate that it <b>is</b> the most \u000A recent one.</p> \u000A \u000A <p>The <see topic=\"ACL2____COMMAND\">command</see> descriptor <v>:x-23</v> is synonymous with <v>(:x -23)</v>. \u000A More generally, every symbol in the keyword package whose first character is \u000A <v>#\\x</v> and whose remaining <see topic=\"ACL2____CHARACTERS\">characters</see> parse as a negative integer is \u000A appropriately understood.  This allows <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> <v>:x-10</v> where \u000A <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> <v>(:max -10)</v> or <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> <v>(:here -10)</v> were \u000A previously used.  The old forms are still legal.</p> \u000A \u000A <p>The order of the arguments to <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> has been changed.</p> \u000A \u000A <p>The simplifier now reports the use of unspecified built-in type information \u000A about the primitives with the phrase ``primitive type reasoning.''  This \u000A phrase may sometimes occur in situations where ``propositional calculus'' was \u000A formerly credited with the proof.</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____PAIRLIS\">pairlis</see></tt> has been replaced in the code by a new \u000A function <tt><see topic=\"ACL2____PAIRLIS_42\">pairlis$</see></tt>, because Common Lisp does not adequately specify its \u000A <tt><see topic=\"COMMON-LISP____PAIRLIS\">pairlis</see></tt> function.</p> \u000A \u000A <p>Some new Common Lisp functions have been added, including <tt><see topic=\"COMMON-LISP____LOGTEST\">logtest</see></tt>, \u000A <tt><see topic=\"COMMON-LISP____LOGCOUNT\">logcount</see></tt>, <tt><see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see></tt>, <tt><see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see></tt>, <tt><see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see></tt>, <tt><see topic=\"COMMON-LISP____STRING\">string</see></tt>, and <tt><see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see></tt>.  The source file \u000A <v>/slocal/src/acl2/axioms.lisp</v> is the ultimate reference regarding Common \u000A Lisp functions in Acl2.</p> \u000A \u000A <p>The functions <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt> and <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> have been \u000A documented.  See <see topic=\"ACL2____DEFUNS\">defuns</see> and see <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>.</p> \u000A \u000A <p>A few symbols have been added to the list <v>*acl2-exports*</v>.</p> \u000A \u000A <p>A new key has been implemented for the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, \u000A <v>:irrelevant-formals-ok</v>.  See <see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see>.</p> \u000A \u000A <p>The connected book directory, <tt><see topic=\"ACL2____CBD\">cbd</see></tt>, must be nonempty and begin and \u000A end with a slash.  It is set (and displayed) automatically upon your first \u000A entry to <tt><see topic=\"ACL2____LP\">lp</see></tt>.  You may change the setting with <tt><see topic=\"ACL2____SET-CBD\">set-cbd</see></tt>.  See \u000A <see topic=\"ACL2____CBD\">cbd</see>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____OOPS\">oops</see></tt> will undo the last <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>.  See <see topic=\"ACL2____OOPS\">oops</see>.</p> \u000A \u000A <p>Documentation has been written about the ordinals.  See :DOC \u000A <v>e0-ordinalp</v> and see :DOC <v>e0-ord-&lt;</v>.  [Note added later: Starting with \u000A Version_2.8, instead see <see topic=\"ACL2____O-P\">o-p</see> and see <see topic=\"ACL2____O_C3\">o&lt;</see>.</p> \u000A \u000A <p>The color <see topic=\"ACL2____EVENTS\">events</see> &mdash; (red), (pink), (blue), and (gold) &mdash; \u000A may no longer be enclosed inside calls of <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, for soundness \u000A reasons.  In fact, neither may any event that sets the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see> for an example of how to change the exit \u000A keyword from <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> to something else.</p> \u000A \u000A <p>The attempt to install a <see topic=\"ACL2____MONITOR\">monitor</see> on <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules \u000A stored as simple abbreviations now causes an error because the application of \u000A abbreviations is not tracked.</p> \u000A \u000A <p>A new message is sometimes printed by the theorem prover, indicating that a \u000A given simplification is ``specious'' because the subgoals it produces include \u000A the input goal.  In Version 1.4 this was detected but not reported, causing \u000A behavior some users found bizarre.  See <see topic=\"ACL2____SPECIOUS-SIMPLIFICATION\">specious-simplification</see>.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____DEFINITION\">Definition</see></tt> rules are no longer always required to specify the \u000A <v>:clique</v> and <v>:controller-alist</v> fields; those fields can be defaulted \u000A to system-determined values in many common instances.  See <see topic=\"ACL2____DEFINITION\">definition</see>.</p> \u000A \u000A <p>A warning is printed if a macro form with keyword arguments is given \u000A duplicate keyword values.  Execute <v>(<see topic=\"ACL2____THM\">thm</see> t :doc nil :doc &quot;ignored&quot;)</v> and \u000A read the warning printed.</p> \u000A \u000A <p>A new restriction has been placed on <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> recursive definitions inside the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> may not use, in \u000A their tests and recursive calls, the constrained functions introduced by the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.  (Note added in \u000A Version 2.3: Subversive recursions were first recognized by us here in Version \u000A 1.5, but our code for recognizing them was faulty and the bug was not fixed \u000A until Version 2.3.)</p> \u000A \u000A <p>The <see topic=\"ACL2____EVENTS\">events</see> <tt><see topic=\"ACL2____DEFEQUIV\">defequiv</see></tt>, <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt>, <tt><see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see></tt>, \u000A and <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> have been reimplemented so that they are just macros \u000A that expand into appropriate <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>; they are no longer primitive <see topic=\"ACL2____EVENTS\">events</see>.  See the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of each affected event.</p> \u000A \u000A <p>The <v>defcor</v> event, which was a shorthand for a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> that \u000A established a <see topic=\"ACL2____COROLLARY\">corollary</see> of a named, previously proved event, has been \u000A eliminated because its implementation relied on a technique we have decided to \u000A ban from our code.  If you want the effect of a <v>defcor</v> in Version 1.5 you \u000A must submit the corresponding <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> with a <v>:by</v> hint naming the \u000A previously proved event.</p> \u000A \u000A <p>Error reporting has been improved for inappropriate <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____HINTS\">hints</see> and <see topic=\"ACL2____EVENTS\">events</see>, and for syntax errors in rule classes, and for \u000A non-existent filename arguments to <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>Technical Note: We now maintain the Third Invariant on <v>type-alists</v>, as \u000A described in the Essay on the Invariants on Type-alists, and Canonicality. \u000A This change will affect some proofs, for example, by causing a to rewrite more \u000A quickly to <v>c</v> when <v>(equiv a b)</v> and <v>(equiv b c)</v> are both known \u000A and <v>c</v> is the canonical representative of the three.</p>"],

 "ACL2____NOTE-6-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 6.0 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>More system functions are in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, <see topic=\"ACL2____GUARD\">guard</see>-verified.  Evaluate</p> \u000A \u000A <code>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> &quot;system/top&quot; *system-verify-guards-alist*)))</code> \u000A \u000A <p>for the list of functions checked to be guard-verifiable in the community \u000A books.  Thanks to those who have contributed to this effort, as shown in file \u000A headers in directory <v>system/</v> of the community books.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFUND\">defund</see></tt> now avoids an error when <v>:mode :program</v> has \u000A been specified in an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> form of a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form, for \u000A example: <v>(<see topic=\"ACL2____DEFUND\">defund</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program)) x)</v>.  It does this \u000A by avoiding the generation of <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in such cases. \u000A Thanks to David Rager and Jared Davis for requesting such a change, and for \u000A ensuing helpful discussions.</p> \u000A \u000A <p>Added a field <v>:UNIFY-SUBST</v> to metafunction contexts (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>), accessed with function <v>mfc-unify-subst</v>.  Thanks \u000A to Sol Swords for requesting this enhancement.</p> \u000A \u000A <p>The functions <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> and <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt> are now <see topic=\"ACL2____GUARD\">guard</see>-verified <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode functions.</p> \u000A \u000A <p>It had been the case that if any supporter of a dependent clause processor \u000A (see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>) is among the ancestors of a given \u000A formula, then it was illegal to apply functional instantiation (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>) to that formula.  Now, this is illegal only if some such \u000A supporter is in the domain of the functional substitution.</p> \u000A \u000A <p>The tau system (see <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>, or if you are unfamiliar with the \u000A tau system, see <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>) now allows the user to \u000A define and verify functions that compute bounds on arithmetic expressions. \u000A See <see topic=\"ACL2____BOUNDERS\">bounders</see>.</p> \u000A \u000A <p>The utility <v>print-summary-user</v> has been replaced by <tt><see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see></tt>, which is described below.  If you previously attached a \u000A function to <v>print-summary-user</v>, say <v>my-print-summary-user</v>, then you \u000A can get the effect you had previously as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-finalize-event-user (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic :stobjs state))\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (my-print-summary-user state)\u000A          state))\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> finalize-event-user my-finalize-event-user)</code> \u000A \u000A <p>It had been the case that when you <tt><see topic=\"ACL2____LD\">ld</see></tt> a file, the connected book \u000A directory (see <see topic=\"ACL2____CBD\">cbd</see>) was set to the canonical pathname of that file's \u000A directory for the duration of the <v>LD</v> call.  This could cause problems, \u000A however, if the file is actually a soft link: an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form in \u000A the book with a relative pathname for the book would be resolved with respect \u000A to the absolute pathname for that link, which is probably not what was \u000A intended.  So soft links are no longer followed when computing the above \u000A connected book directory.  The following example, which is how we discovered \u000A this problem, may clarify.  We attempted to execute the form <v>(<see topic=\"ACL2____LD\">ld</see>\u000A &quot;top.lisp&quot;)</v> using ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>) in community books directory \u000A <v>nonstd/arithmetic/</v>, where all of the <v>.lisp</v> files are soft links to \u000A files in <v>arithmetic/</v>.  Thus, the form <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;equalities&quot;)</v> \u000A attempted to include <v>arithmetic/equalities</v> instead of \u000A <v>nonstd/arithmetic/equalities</v>, which caused an error.</p> \u000A \u000A <p>We no longer document the use of value <v>:START</v> for <tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt>.  This value has always been used by the ACL2 \u000A implementation and may have semantics that change with new ACL2 versions.  If \u000A you have reason to use this value, please contact the ACL2 implementors.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>By default, the prover now gives information about case splits.  See <see topic=\"ACL2____SPLITTER\">splitter</see>.  Thanks to many ACL2 users, most recently David Rager, for \u000A requesting such a capability.  Also thanks to David Rager and Jared Davis for \u000A helpful discussions, and thanks to Robert Krug for feedback on the initial \u000A implementation and documentation that led us to make improvements.</p> \u000A \u000A <p>New utilities <tt><see topic=\"ACL2____INITIALIZE-EVENT-USER\">initialize-event-user</see></tt> and <tt><see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see></tt> \u000A allow the user to run <see topic=\"ACL2____STATE\">state</see>-modifying code at the start and end of \u000A <see topic=\"ACL2____EVENTS\">events</see>.  Thanks to Harsh Raju Chamarthi for requesting these \u000A capabilities.  Note that <tt><see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see></tt> replaces \u000A <v>print-summary-user</v>.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>Several heuristic improvements have been made to the tau system, even if \u000A you do not explicitly use the new capability for computing bounds on \u000A arithmetic expressions, mentioned above.  See <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>, or if you are \u000A unfamiliar with the tau system, see <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>A soundness bug has been fixed that exploited the use of expansion files \u000A (see <see topic=\"ACL2____BOOK-COMPILED-FILE\">book-compiled-file</see>) together with <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>.  For an \u000A example illustrating this bug, see the comment about ``Expansion/Defstobj \u000A Bug'' in the form <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-1 ...)</v> in ACL2 source file \u000A <v>ld.lisp</v>.</p> \u000A \u000A <p>We fixed a soundness bug involving system function <tt><see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see></tt> and (most likely) other functions in the former value of \u000A constant <v>*unattachable-primitives*</v>.  Thanks to Jared Davis and Sol Swords \u000A for bringing this bug to our attention by way of an example.  We include a \u000A very slight variant of that example in a comment within the form <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see>\u000A note-6-1 ...)</v> in ACL2 source file <v>ld.lisp</v>.</p> \u000A \u000A <p>There was a soundness bug that allowed attachments to prove <v>nil</v> in a \u000A consistent logical <see topic=\"ACL2____WORLD\">world</see> involving <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> <see topic=\"ACL2____EVENTS\">events</see>. \u000A This has been fixed, by requiring that no function symbol ancestral in a \u000A <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> formula is allowed to get an attachment.  See <see topic=\"ACL2____DEFATTACH\">defattach</see>, in particular discussion of ``a restriction based on a notion of a \u000A function symbol syntactically supporting an event'', which concludes with a \u000A proof of <v>nil</v> that is no longer possible.</p> \u000A \u000A <p>(ACL2(h) only) We fixed a soundness bug in the interaction of memoization \u000A with congruent stobjs, in cases where the <v>:congruent-to</v> field of <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> was not the canonical representative in the congruence class.  For \u000A an example illustrating this bug, see the comment about ``memoize/congruent \u000A stobj bug'' in the form <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-1 ...)</v>  in ACL2 source file \u000A <v>ld.lisp</v>.</p> \u000A \u000A <p>Functions defined by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> had failed to be compiled when \u000A certifying books, except in host Lisps that compile on-the-fly (CCL, SBCL). \u000A This has been fixed for all host Lisps.  A related change, probably less \u000A significant, was made for <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>.  Thanks to Sol Swords for \u000A reporting bugs that turned out to be mistakes in a preliminary implementation \u000A of this change.</p> \u000A \u000A <p>Fixed an assertion error involving linear arithmetic.  Thanks to Sol Swords \u000A for sending an example illustrating the bug (now appearing as a comment in \u000A ACL2 source function <v>linearize1</v>).</p> \u000A \u000A <p>Fixed a bug that was breaking the ACL2s build mechanism (see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see>) by causing certain needless evaluation of ``hidden <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>'' forms in <see topic=\"ACL2____CERTIFICATE\">certificate</see> files when executing a call of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  The bug could also affect rare error messages arising from \u000A ill-formed <see topic=\"ACL2____CERTIFICATE\">certificate</see> files.  Thanks to Harsh Raju Chamarthi for \u000A bringing this bug to our attention by sending us an example script of the sort \u000A that was breaking during an ACL2s build.</p> \u000A \u000A <p>Fixed handling of pathnames by some low-level code (system function \u000A <v>our-truename</v>) that could cause errors, for example for host-Lisp GCL on \u000A some platforms when environment variable <v>HOME</v> points to a non-existent \u000A directory.  Thanks to Camm Maguire for bringing this issue to our attention \u000A and helping with the debugging.</p> \u000A \u000A <p>Fixed a coding bug in generation of stobj resizing functions for a stobj \u000A named <v>OLD</v>.  The following example illustrates the bug.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> old\u000A  (fld :type (array (unsigned-byte 31) (8))\u000A        :initially 0 :resizable t))\u000A(resize-fld 10 old)\u000A; The following returned 8 but should have returned 10:\u000A(fld-length old)</code> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____DEFABSSTOBJ-MISSING-EVENTS\">defabsstobj-missing-events</see></tt> (which macroexpanded \u000A incorrectly).  Thanks to Sol Swords for bringing this bug to our \u000A attention.</p> \u000A \u000A <p>Fixed two bugs in the handling of step-limits.  Thanks to Hanbing Liu for \u000A bringing the main such bug to our attention, which was that ACL2 could report \u000A a step-limit violation during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> (in fact, during any \u000A compound event such as a call of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>), even \u000A without direct user involvement in managing step-limits (see <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> and see <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>).  The other bug \u000A was that a bad argument to <tt><see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see></tt> could result in a raw \u000A Lisp error, for example: <v>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> '(a b)))</v>.</p> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL</b></p> \u000A \u000A <p>The <v>books/</v> directory no longer needs to exist in order to build an \u000A ACL2 executable.  Thanks to Robert Krug for pointing out that the installation \u000A instructions had suggested that this was already the case.</p> \u000A \u000A <p>Many changes have been made to the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  For example, \u000A some community books now include <v>std/lists/rev.lisp</v>, which contains the \u000A rule <v>revappend-removal</v>, which may cause some proofs involving <tt><see topic=\"COMMON-LISP____REVAPPEND\">revappend</see></tt> to fail where they formerly succeeded, or vice-versa.  When a proof \u000A fails that formerly succeeded, it may be useful for you to look over the <see topic=\"ACL2____RUNE\">rune</see>s printed in the event summary.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p><b>EXPERIMENTAL/ALTERNATE VERSIONS</b></p> \u000A \u000A <p>For ACL2(p), <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt> is now locked by default; thanks to \u000A David Rager for suggesting this change.  But there is a way to avoid the lock; \u000A see <see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see>.  In particular, the lock is avoided in the \u000A implementations of <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> and <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see>, which are not supported in ACL2(p) (see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see>).</p>"],

 "ACL2____NOTE-4-3":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 4.2 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level and to distributed books, \u000A Emacs support, and experimental versions.  Each change is described in just \u000A one category, though of course many changes could be placed in more than one \u000A category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>Significant changes have been made for list-processing primitives such as \u000A <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> and <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt>; see <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see>.  In summary: \u000A instead of separate functions based on <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, and <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, there is essentially just one function, which is based on <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>; the <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> and <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> variants are logically just the <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> variant.  For example, <tt><see topic=\"ACL2____MEMBER-EQ\">member-eq</see></tt> and <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> are macros \u000A that generate corresponding calls of <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> in the logic, \u000A although in raw Lisp they will execute using tests <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> and <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, \u000A respectively.  References to any of these in logical contexts such as <see topic=\"ACL2____THEORIES\">theories</see> are now references to the function based on <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>; for \u000A example, the hint <v>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> member)</v> is completely equivalent to \u000A the hint <v>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> member-equal)</v>.  Distributed books have been \u000A modified as necessary to accommodate this change.  While the need for such \u000A changes was relatively infrequent, changes were for example needed in contexts \u000A where terms are manipulated directly; for example, <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> needs \u000A to mention <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> rather than <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>, just as it was \u000A already the case to mention, say, <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> rather than <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>.  Again, see <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for more information about \u000A equality variants.</p> \u000A \u000A <p>A few improvements were made in support of the modified treatment of \u000A equality variants discussed above.  The changes include the following.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o We now allow the use of macro aliases (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see> in \u000A <v>:trigger-fns</v> of rules (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>).</p> \u000A \u000A <p>o We now remove so-called ``guard holders'' (including calls of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>, hence of <tt><see topic=\"ACL2____MBE\">mbe</see></tt>) in <v>:trigger-terms</v> of rules.</p> \u000A \u000A <p>o We also remove guard holders in formulas of <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> and \u000A <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.</p> \u000A \u000A <p>o Macros <v>union-eq</v> and <v>intersection-eq</v> can now take any positive \u000A number of arguments, and <tt><see topic=\"ACL2____UNION-EQ\">union-eq</see></tt> can take zero arguments.  (Thanks to \u000A Jared Davis for requesting this enhancement.)  The same can be said for new \u000A macros <tt><see topic=\"ACL2____UNION_42\">union$</see></tt> and <tt><see topic=\"ACL2____INTERSECTION_42\">intersection$</see></tt>, respectively.</p> \u000A \u000A <p>o A few changes were made to built-in theorems from source file \u000A <v>axioms.lisp</v>, in particular disabling <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule \u000A <v>consp-assoc-equal</v> (formerly two enabled rules, <v>consp-assoc-eq</v> and \u000A <v>consp-assoc</v>) but adding this theorem as a <v>:forward-chaining</v> rule, \u000A and similarly for <v>true-list-listp-forward-to-true-listp-assoc-equal</v> (and \u000A eliminating rule <v>true-list-listp-forward-to-true-listp-assoc-eq</v>; and \u000A disabling rule <v>true-listp-cadr-assoc-eq-for-open-channels-p</v>.  Also, \u000A theorem <v>all-boundp-preserves-assoc</v> has been renamed to \u000A <v>all-boundp-preserves-assoc-equal</v> and also strengthened.</p> \u000A \u000A <p>o Some <see topic=\"ACL2____GUARD\">guard</see>s were slightly improved (logically weaker or the \u000A same).</p></blockquote> \u000A \u000A <p>Improved <v>get-output-stream-string$</v> to allow for a context and to do \u000A genuine error printing instead of using <tt><see topic=\"ACL2____CW\">cw</see></tt>.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p>Added the symbols <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt> and <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt> to \u000A <v>*acl2-exports*</v>.</p> \u000A \u000A <p>A small change was made to the processing of more than one <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> declaration for the same function.  In particular, a guard of <v>t</v> is \u000A essentially ignored.</p> \u000A \u000A <p>Attachments are now allowed during evaluation of the first argument of \u000A <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> even in contexts (such as proofs) during which the use of \u000A attachments is normally prohibited.  More generally, the second of the three \u000A arguments of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> has this property, except in the case of \u000A <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (or related macros like <tt><see topic=\"ACL2____MBE1\">mbe1</see></tt>), where the <v>exec</v> \u000A argument may provide the value.  Thanks to Sol Swords for useful discussions \u000A leading us to implement this enhancement.</p> \u000A \u000A <p>The restriction has been removed that prohibited the use of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> \u000A inside <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> with a non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>. \u000A This restriction was introduced in Version 3.4, but has not been necessary \u000A since Version 4.0, when we first started disallowing <see topic=\"ACL2____GUARD\">guard</see> verification \u000A for functions introduced non-<see topic=\"ACL2____LOCAL\">local</see>ly inside such <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A events.</p> \u000A \u000A <p>We weakened the checks involving common ancestors for evaluator and <see topic=\"ACL2____META\">meta</see> (and <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>) functions (see <see topic=\"ACL2____EVALUATOR-RESTRICTIONS\">evaluator-restrictions</see>) so that except in the <tt><see topic=\"ACL2____MBE\">mbe</see></tt> case, the \u000A next-to-last argument of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> is not considered.  Thanks to Sol \u000A Swords for bringing this issue to our attention.</p> \u000A \u000A <p>The macro <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> no longer requires at least two arguments.  Thanks \u000A to Dave Greve for requesting this enhancement.</p> \u000A \u000A <p>(Mostly for system hackers) Now, <tt><see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see></tt> breaks at a more \u000A appropriate (earlier) time for certain system functions that do not return \u000A state, such as <v>translate11</v>.  Thanks to David Rager for requesting this \u000A improvement.</p> \u000A \u000A <p><tt><see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">Show-accumulated-persistence</see></tt> may take a new argument, <v>:runes</v>, \u000A which simply causes an alphabetical list of <see topic=\"ACL2____RUNE\">rune</see>s to be printed \u000A out.</p> \u000A \u000A <p>Improved <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> so that <v>:entry</v>, <v>:exit</v>, and <v>:cond</v> \u000A forms may reference <v>state</v> even if the function being traced does not \u000A include <v>state</v> as a formal.</p> \u000A \u000A <p>The system function <v>acl2x-expansion-alist</v> now takes a second argument, \u000A namely <tt><see topic=\"ACL2____STATE\">state</see></tt>.  This change allows for more flexibility in the sorts of \u000A attachments that can be made to this function (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  Thanks \u000A to Jared Davis and Sol Swords for requesting this enhancement and providing a \u000A preliminary implementation.</p> \u000A \u000A <p>An obscure <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> change, unlikely to affect users, replaces \u000A the state global variables <v>erp</v>, <v>val</v>, <v>print-macroexpansion-flg</v>, \u000A and <v>print-prompt-and-instr-flg</v> by <v>pc-erp</v>, <v>pc-val</v>, \u000A <v>pc-print-macroexpansion-flg</v>, and <v>pc-print-prompt-and-instr-flg</v>, \u000A respectively.</p> \u000A \u000A <p><see topic=\"ACL2____STATE\">State</see> globals <v>fmt-hard-right-margin</v> and \u000A <v>fmt-soft-right-margin</v> are now untouchable (see <see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see> and see <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>).  If you bind \u000A these <v>state</v> globals with <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt>, then you will need to \u000A do so with appropriate setters to restore their values, for example as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((fmt-hard-right-margin 500 set-fmt-hard-right-margin)\u000A  (fmt-soft-right-margin 480 set-fmt-soft-right-margin))\u000A ...)</code> \u000A \u000A <p>The error message has been improved for the case of evaluating an undefined \u000A function that has an attachment (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  Thanks to Jared Davis \u000A for sending the following example, which illustrates the additional part of \u000A the message.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x) t)\u000A[[... output omitted ...]]\u000AACL2 !&gt;(<see topic=\"ACL2____DEFATTACH\">defattach</see> foo identity)\u000A[[... output omitted ...]]\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *x* (foo 3))\u000A\u000AACL2 Error in ( DEFCONST *X* ...):  ACL2 cannot ev the call of undefined\u000Afunction FOO on argument list:\u000A\u000A(3)\u000A\u000ANote that because of logical considerations, attachments (including\u000AIDENTITY) must not be called in this context.\u000A\u000A[[... additional output omitted ...]]</code> \u000A \u000A <p>The directory string supplied to <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> no longer \u000A must terminate with the `<v>/</v>' character, as had been required in some Lisp \u000A implementations.  Thanks to Sol Swords for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>We no longer print induction schemes with <see topic=\"ACL2____GAG-MODE\">gag-mode</see>; use <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> if you want to see them.  Thanks to Dave Greve for this suggestion.</p> \u000A \u000A <p>It is now legal to supply a constant for a <see topic=\"ACL2____STOBJ\">stobj</see> array dimension. \u000A See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.  Thanks to Warren Hunt for requesting this \u000A enhancement.</p> \u000A \u000A <p>We cleaned up a few issues with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o It is no longer illegal to submit a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> form in raw-mode (see \u000A <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>).  Thanks to Jun Sawada for reporting an example in which \u000A an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form submitted in raw-mode caused an error because of \u000A a `hidden' <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> form (see <see topic=\"ACL2____HIDDEN-DEFPKG\">hidden-defpkg</see>).  There will no \u000A longer be an error in such cases.</p> \u000A \u000A <p>o It had been the case that <see topic=\"ACL2____LOCAL\">local</see>ly including a book could make it \u000A possible to use a package defined by that book.  Consider for example the \u000A following book, <v>foo.lisp</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic/top&quot; :dir :system))</code> \u000A \u000A <p>After certifying this book, it had been possible to admit the following \u000A events in a new session.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot;)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> acl2-asg::*foo* 3)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *c* 'acl2-asg::xyz)</code> \u000A \u000A <p>In Version_4.3, neither of these <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> events is admitted.</p> \u000A \u000A <p>o A hard Lisp error is now avoided that had been possible in rare cases \u000A when including books with hidden packages (see <see topic=\"ACL2____HIDDEN-DEFPKG\">hidden-defpkg</see>). An \u000A example may be found in a comment in the <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> for <v>note-4-3</v> \u000A (in ACL2 source file <v>ld.lisp</v>).</p></blockquote> \u000A \u000A <p>The undocumented (but sometimes useful) functions <v>packn1</v> and \u000A <v>packn</v> are now <see topic=\"ACL2____GUARD\">guard</see>-verified <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions. \u000A Thanks to Sandip Ray for requesting this enhancement.</p> \u000A \u000A <p>It had been the case that when including a book, functions defined in the \u000A book's certification <see topic=\"ACL2____WORLD\">world</see> (that is, in its <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s) were typically not given compiled code.  That has been fixed.</p> \u000A \u000A <p>The commands <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt> have been improved, \u000A primarily by printing information for more rule classes.  See <see topic=\"ACL2____PL\">pl</see> and \u000A see <see topic=\"ACL2____PL2\">pl2</see>.  See also the item below about the new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A command, <v>show-type-prescriptions</v>.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>New macros <tt><see topic=\"ACL2____MV_F3-LET\">mv?-let</see></tt> and <tt><see topic=\"ACL2____MV_F3\">mv?</see></tt> extend the funtionality of \u000A <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> and <tt><see topic=\"ACL2____MV\">mv</see></tt> (respectively) to the case of a single \u000A value.</p> \u000A \u000A <p>Macro <see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see> is available for system programmers who wish \u000A bind <tt><see topic=\"ACL2____STATE\">state</see></tt> locally, essentially using <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>.  But \u000A this should only be done with extreme care, and it requires an active trust \u000A tag; see <see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see>.</p> \u000A \u000A <p>Formatted printing functions now have analogues that print to strings and \u000A do not take an output channel or <tt><see topic=\"ACL2____STATE\">state</see></tt> as arguments.  See <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>.</p> \u000A \u000A <p>The system function <v>ancestors-check</v> is now available for verified \u000A modification by users, i.e., attachment using <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> ancestors-check\u000A &lt;your_function&gt;)</v>.  Thanks to Robert Krug for providing the necessary proof \u000A support, which we modified only in small ways.</p> \u000A \u000A <p>New macros, <v>observation-cw</v> and <v>warning$-cw</v>, provide formatted \u000A printing of <tt><see topic=\"ACL2____OBSERVATION\">observation</see></tt>s and warnings (respectively) without <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thanks to Harsh Raju Chamarthi and David Rager for requests leading \u000A to these utilities.  <v>Observation-cw</v> is now used in some of the \u000A distributed books (thanks to Robert Krug for useful interaction for that).</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>type-alist</v> (see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>) now takes an optional third argument that causes the \u000A production of forward-chaining reports (see <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see>). \u000A Thanks to Dave Greve for requesting such an enhancement.</p> \u000A \u000A <p>The reports generated by forward-chaining, <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see>, \u000A have been changed to indicate when a conclusion reached by forward chaining is \u000A <v>REDUNDANT</v> with respect to the type information already known.  Thanks to \u000A Dave Greve for suggesting this enhancement.</p> \u000A \u000A <p>The utility <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt> is now legal for <see topic=\"ACL2____EVENTS\">events</see> \u000A (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).  For example, the following is now \u000A legal.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>\u000A  2\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> append-assoc\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A           (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))))))</code> \u000A \u000A <p>The new utility <tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt> is analogous to the utility \u000A <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, but counts ``prover steps'' rather than \u000A checking for time elapsed.  See <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>.  Also see <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> to provide a default step-limit.  Note that just as \u000A <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt> may now be used to create <see topic=\"ACL2____EVENTS\">events</see>, as \u000A discussed just above, <tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt> may also be used to \u000A create <see topic=\"ACL2____EVENTS\">events</see>.  Thanks to Carl Eastlund for requesting support for \u000A step-limits.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____PROGN_42\">progn$</see></tt> is analogous to <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, but allows an \u000A arbitrary number of arguments.  For example:</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____PROGN_42\">progn$</see> (f1 x) (f2 x) (f3 x))\u000A (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (F1 X) (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (F2 X) (F3 X)))\u000AACL2 !&gt;</code> \u000A \u000A <p>Thanks to David Rager for contributing this macro.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> may now be supplied the argument \u000A <v>:skip-checks :cycles</v>.  In this case, as with argument <v>:skip-checks\u000A t</v>, a trust tag is reuired (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), and no logical claims are \u000A made.  The effect is to avoid the usual check that the extended ancestor \u000A relation has no cycles (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  Thanks to Dave Greve for \u000A requesting this feature.</p> \u000A \u000A <p>You can now limit the printing of subgoal names when using <v>:</v><tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt><v> :goals</v>.  See <see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see>.  Thanks to Karl \u000A Hoech for a suggestion leading to this enhancement.</p> \u000A \u000A <p>A new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command, <v>show-type-prescriptions</v>, or \u000A <v>st</v> for short, provides information about <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> \u000A rules that match a given term.  Thanks to Dave Greve for requesting this \u000A enhancement.  See also the item above about related improvements to commands \u000A <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt>.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>ACL2 now avoids some repeated attempts to rewrite hypotheses of rewrite \u000A rules.  See <see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> for a discussion of this behavior and \u000A how to avoid it.  The default behavior has been observed to reduce by 11% the \u000A overall time required to complete a regression.  Here are the directories that \u000A had the top three time decreases and top three time increases, shown in \u000A seconds.</p> \u000A \u000A <code>-368 coi/gacc (1064 down to 696: decrease of 35%)\u000A-220 workshops/1999/ste (664 down to 444: decrease of 33%)\u000A-148 unicode (331 down to 183: decrease of 45%)\u000A....\u000A  +7 workshops/2002/cowles-flat/support (229 up to 236: increase of 3%)\u000A  +8 workshops/1999/ivy/ivy-v2/ivy-sources (508 up to 516: increase of 2%)\u000A +12 workshops/2009/hardin/deque-stobj (78 up to 91: increase of 17%)</code> \u000A \u000A <p>The so-called ``ancestors check,'' which is used to limit backchaining, has \u000A been strengthened so that two calls of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> are considered the same \u000A even if their arguments appear in the opposite order.  Thanks to Robert Krug \u000A for providing an implementation and a useful discussion.</p> \u000A \u000A <p>The check for <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see> in processing of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s has \u000A been made more efficient.  Thanks to Eric Smith for reporting this issue in \u000A 2001 (!) and thanks to Warren Hunt for recently sending an example.  For that \u000A example, we have seen the time for the <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see> check reduced \u000A from about 10 seconds to about 0.04 seconds.</p> \u000A \u000A <p>(GCL only) The macro <tt><see topic=\"ACL2____MV\">mv</see></tt> has been modified so that certain fixnum \u000A boxing can be avoided.</p> \u000A \u000A <p>(Allegro CL only) We have set to <v>nil</v> four Allegro CL variables that \u000A otherwise enable storing of certain source information (for details, see the \u000A discussion of ``cross-referencing'' in ACL2 source file <v>acl2-init.lisp</v>). \u000A As a result of this change we have about a 6% speedup on the regression suite, \u000A but a 27% time reduction on an example that includes a lot of books.</p> \u000A \u000A <p>Exhaustive matching for the case of <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> has been extended \u000A to <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules, in analogy to the default setting \u000A <v>:match-free :all</v> already in place for <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, and \u000A <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules.  See <see topic=\"ACL2____FREE-VARIABLES-TYPE-PRESCRIPTION\">free-variables-type-prescription</see>. \u000A Thanks to Dave Greve for requesting this enhancement.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>A soundness bug was fixed in some raw-Lisp code implementing the function, \u000A <tt><see topic=\"ACL2____TAKE\">take</see></tt>.  Thanks to Sol Swords for pointing out this bug with \u000A (essentially) the following proof of <v>nil</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">defthmd</see> take-1-nil-logic\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAKE\">take</see> 1 nil) '(nil))\u000A  :hints((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____TAKE\">take</see>)))))\u000A(<see topic=\"ACL2____THM\">thm</see> nil :hints ((&quot;Goal&quot; :use take-1-nil-logic)))</code> \u000A \u000A <p>Calls of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> in ``safe-mode'' situations &mdash; i.e., during \u000A evaluation of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt>, and <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> \u000A forms, and during macroexpansion &mdash; are now guard-checked.  Thus, in \u000A these situations both the <v>:logic</v> and <v>:exec</v> forms will be evaluated, \u000A with an error if the results are not equal.  Formerly, only the <v>:logic</v> \u000A form was evaluated, which was a soundness bug that could be exploited to prove \u000A <v>nil</v>.  For a such a proof and a bit of further explanation, see the \u000A example at the top of the comments for <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-4-3 ..)</v> in ACL2 \u000A source file <v>ld.lisp</v>.</p> \u000A \u000A <p>It had been possible to prove <v>nil</v> by proving the following theorem \u000A using ACL2 built on CCL and then proving its negation using ACL2 built on a \u000A different host Lisp.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> host-lisp-is-ccl\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> 'host-lisp *initial-global-table*))\u000A         :ccl)\u000A  :rule-classes nil)</code> \u000A \u000A <p>This hole has been plugged by moving the setting of <v>'host-lisp</v> out of \u000A the constant <v>*initial-global-table*</v>.</p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> for arguments that are <see topic=\"ACL2____STOBJ\">stobj</see> accessors or \u000A updaters.  It also gives an informative error in this case when the accessor \u000A or updater is a macro (because the introducing <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event \u000A specified <v>:inline t</v>).</p> \u000A \u000A <p>Avoided a potential error that could occur when no user home directory is \u000A located.  Our previous solution for Windows simply avoided looking for ACL2 \u000A customization files (see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>) and <v>acl2-init.lsp</v> \u000A files in a user's home directory.  With this change, we handle such files the \u000A same for Windows as for non-Windows systems: we always look for ACL2 \u000A customization files (see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>) and <v>acl2-init.lsp</v> \u000A files in a user's home directory, but only if such a directory exists.  Thanks \u000A to Hanbing Liu for reporting this issue.</p> \u000A \u000A <p>(GCL only) Fixed a bug that prevented the use of <tt><see topic=\"ACL2____GET-OUTPUT-STREAM-STRING_42\">get-output-stream-string$</see></tt> when the host Lisp is GCL.</p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see></tt> to work properly for executable counterparts \u000A (so-called ``*1*'' functions).</p> \u000A \u000A <p>Fixed a bug in the error message caused by violating the <see topic=\"ACL2____GUARD\">guard</see> of a \u000A macro call.</p> \u000A \u000A <p>Fixed a bug in an error message that one could get when calling <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> with argument <v>:skip-checks t</v> to attach to a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function symbol that was introduced with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  (This \u000A is indeed an error, but the message was confusing.)  Thanks to Robert Krug for \u000A bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in the loop-checking done on behalf of <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>, which \u000A could miss a loop.  For an example, see the comment about loop-checking in the \u000A comments for <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-4-3 ..)</v> in ACL2 source file <v>ld.lisp</v>.</p> \u000A \u000A <p>Terms of the form <v>(<see topic=\"ACL2____HIDE\">hide</see> &lt;term&gt;)</v> without free variables could be \u000A simplified, contrary to the purpose of <tt><see topic=\"ACL2____HIDE\">hide</see></tt>.  This is no longer the \u000A case, Thanks to Dave Greve for reporting this issue.</p> \u000A \u000A <p>An infinite loop could occur when an error was encountered in a call of \u000A <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt>, for example with the following form, and this has been \u000A fixed.</p> \u000A \u000A <code>(<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A               '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> ()\u000A                  (<see topic=\"ACL2____ER\">er</see> hard 'my-top &quot;Got an error!&quot;))\u000A               nil)</code> \u000A \u000A <p>Fixed a bug in detection of package redefinition.  While we have no example \u000A demonstrating this as a soundness bug, we cannot rule it out.</p> \u000A \u000A <p>Fixed a bug in the message produced by an erroneous call of <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt>. \u000A Thanks to Jared Davis for reporting this bug and sending a helpful \u000A example.</p> \u000A \u000A <p>For a failed <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event, we now avoid \u000A printing <see topic=\"ACL2____RUNE\">rune</see>s that are used only in processing proposed rules to be \u000A stored, but not in the proof itself.  Thanks to Dave Greve for sending us an \u000A example that led us to make this fix.</p> \u000A \u000A <p>ACL2 did not reliably enforce the restriction against non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> inside <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events, as \u000A illustrated by the following examples.</p> \u000A \u000A <code>; not permitted (as expected)\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot;))\u000A\u000A; permitted (as expected)\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot;)))\u000A\u000A; formerly permitted (surprisingly); now, not permitted\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> () (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot;)))</code> \u000A \u000A <p>Moreover, the corresponding error message has been fixed.  Thanks to Jared \u000A Davis and Sandip Ray for relevant discussions.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is given a first argument that is not a string, a \u000A more graceful error now occurs, where previously an ugly raw Lisp error had \u000A occurred.  Thanks to Eric Smith for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in an error message that was printed when an unexpected \u000A expression has occurred where a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form is expected.</p> \u000A \u000A <p>(Since all functions are compiled when the host Lisp is CCL or SBCL, the \u000A following bug fix did not occur for those host Lisps.)  After evaluation of \u000A <v>(</v><tt><see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see></tt><v> t)</v>, all defined functions are expected to \u000A run with compiled code; but this was not the case for functions exported from \u000A an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event.  This has been fixed.</p> \u000A \u000A <p>It had been the case that the <v>:</v><tt><see topic=\"ACL2____PUFF\">puff</see></tt> command was broken for \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form whose book had been certified in a world with an \u000A <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> event.  This has been fixed.</p> \u000A \u000A <p>Evaluation of <see topic=\"ACL2____STOBJ\">stobj</see> updaters (see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>) may no longer use \u000A attachments (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  This is a subtle point that will likely \u000A not affect many users.  Thanks to Jared Davis for bringing this issue to our \u000A attention; a slight variant of his example appears in a comment in ACL2 source \u000A function <v>oneify-cltl-code</v>.</p> \u000A \u000A <p>It had been the case that even when a <see topic=\"ACL2____STOBJ\">stobj</see> creator function was \u000A declared to be untouchable (see <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>), a <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt> form based on that same stobj was permitted.  Now, such \u000A forms are not admitted.  Thanks to Jared Davis for a query leading to this \u000A fix.</p> \u000A \u000A <p>Fixed a buggy message upon <see topic=\"ACL2____GUARD\">guard</see> violations, which was suggesting \u000A the use of <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)</v> in some cases when guard-checking \u000A was already set to <v>:none</v>.</p> \u000A \u000A <p>It had been possible to get a hard Lisp error when computing with <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt> in <see topic=\"ACL2____BOOKS\">books</see>.  The following is an example of such a book, whose \u000A certification no longer causes an error.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *c* (<see topic=\"ACL2____EC-CALL\">ec-call</see> (f 3)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))</code> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt>, and also the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands \u000A <v>rewrite</v> and <v>show-rewrites</v> (and hence their respective aliases <v>r</v> \u000A and <v>sr</v>), now take rule-id arguments that can be <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> \u000A <see topic=\"ACL2____RUNE\">rune</see>s.  These commands dealt with definition rules already, e.g.</p> \u000A \u000A <code>:pl2 (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) binary-append</code> \u000A \u000A <p>but they did not allow explicit specification of <v>:definition</v> runes, \u000A e.g.:</p> \u000A \u000A <code>:pl2 (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) (:definition binary-append)</code> \u000A \u000A <p>The following example illustrates a bug in the processing of (admittedly \u000A obscure) <see topic=\"ACL2____HINTS\">hints</see> of the form <v>:do-not-induct name</v>, where <v>name</v> is \u000A not <v>t</v>, <v>:otf-flg-override</v>, <v>:otf</v>, or <v>nil</v>.  In this example, \u000A ACL2 had essentially ignored the hint and reverted to prove the original goal \u000A by induction, rather than to skip the goal temporarily as is expected for such \u000A hints.  Thanks to David Rager for a helpful discussion.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x2 y2) z2) (<see topic=\"COMMON-LISP____APPEND\">append</see> x2 y2 z2)))\u000A     :hints ((&quot;Subgoal 1&quot; :do-not-induct some-name)))</code> \u000A \u000A <p>Fixed a slight bug in the definitions of built-in <tt><see topic=\"ACL2____THEORIES\">theories</see></tt>.  For \u000A example, in a fresh ACL2 session the value of the following form is <v>nil</v>, \u000A but formerly included several <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____WORLD\">world</see> (w state)))\u000A  (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> :here)\u000A                           (<see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> 'ground-zero)))</code> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL AND TO DISTRIBUTED BOOKS</b></p> \u000A \u000A <p>Many changes have been made to the distributed books, as recorded in svn \u000A logs under the `Source' and 'Updates' links at the <a \u000A href='http://acl2-books.googlecode.com/'>ACL2 Books</a> web page.  Here we \u000A list some of the more significant changes.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o A large library has been graciously contributed by the formal \u000A verification group at Centaur Technology.  See <v>books/centaur/</v> and, in \u000A particular, file <v>books/centaur/README</v>, which explains how the library \u000A depends on the experimental HONS extension (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).</p> \u000A \u000A <p>o Among the new books is an illustration of <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>, \u000A <v>books/misc/defattach-example.lisp</v>, as well as a variant of defattach that \u000A avoids the need for <see topic=\"ACL2____GUARD\">guard</see> verification, \u000A <v>books/misc/defattach-bang.lisp</v>.</p> \u000A \u000A <p>o Distributed book <v>books/misc/trace1.lisp</v> has been deleted.  It had \u000A provided slightly more friendly <see topic=\"COMMON-LISP____TRACE\">trace</see> output for new users, but \u000A distributed book <v>books/misc/trace-star.lisp</v> may be better suited for that \u000A purpose.</p></blockquote> \u000A \u000A <p>ACL2 can once again be built on LispWorks (i.e., as the host Lisp), at \u000A least with LispWorks 6.0.  Thanks to David Rager for useful conversations. \u000A Several changes have been made from previous LispWorks-based ACL2 \u000A executables:<br></br> \u000A \u000A o ACL2 now starts up in its read-eval-print loop.<br></br> \u000A \u000A o You can save an image with <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt>.<br></br> \u000A \u000A o Multiprocessing is not enabled.<br></br> \u000A \u000A o The stack size is managed using a LispWorks variable that causes the stack \u000A to grow as needed.<br></br> \u000A \u000A o When ACL2 is built a script file is written, as is done for other host \u000A Lisps.  Thus, (assuming that no <v>PREFIX</v> is specified), <v>saved_acl2</v> is \u000A just a small text file that invokes a binary executable, which for Lispworks \u000A is <v>saved_acl2.lw</v>.</p> \u000A \u000A <p>The HTML documentation no longer has extra newlines in &lt;pre&gt; \u000A environments.</p> \u000A \u000A <p>Statistics on ACL2 code size may be found in distributed file \u000A <v>doc/acl2-code-size.txt</v>.  This file and other information can be found in \u000A a new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic, <see topic=\"ACL2____ABOUT-ACL2\">about-ACL2</see>.</p> \u000A \u000A <p>Fixed the build process to pay attention to environment variable \u000A <v>ACL2_SYSTEM_BOOKS</v> (which may be supplied as a command-line argument to \u000A `make').  An ACL2 executable can thus now be built even when there is no \u000A <v>books/</v> subdirectory if a suitable replacement directory is supplied.</p> \u000A \u000A <p>Some warnings from the host Lisp are now suppressed that could formerly \u000A appear.  For example, the warnings shown below occurs in Version 4.2 using \u000A Allegro CL, but not in Version 4.3.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> t)\u000A              (<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> t)\u000A              (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x y)\u000A                x))\u000A[[.. output omitted ..]]\u000A BAR\u000AACL2 !&gt;:comp bar\u000A; While compiling BAR:\u000AWarning: Variable Y is never used.\u000A; While compiling (LABELS ACL2_*1*_ACL2::BAR ACL2_*1*_ACL2::BAR):\u000AWarning: Variable Y is never used.\u000A BAR\u000AACL2 !&gt;</code> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>The distributed Emacs file <v>emacs/emacs-acl2.el</v> now indents calls of \u000A <v>er@par</v> and <v>warning$@par</v> the same way that calls of <v>defun</v> are \u000A indented.</p> \u000A \u000A <p><b>EXPERIMENTAL VERSIONS</b></p> \u000A \u000A <p>The parallel version (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>) now supports parallel \u000A evaluation of the ``waterfall'' part of the ACL2 prover; see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>.  Thanks to David Rager for doing the primary \u000A design and implementation work.</p> \u000A \u000A <p>A new macro, <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt>, supports speculative and parallel \u000A execution in the parallel version, courtesy of David Rager.</p> \u000A \u000A <p>Among the enhancements for the HONS version (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>) are the following.</p> \u000A \u000A <blockquote><p><tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt>d functions may now be traced (see <see topic=\"ACL2____TRACE_42\">trace$</see>).  Thanks to Sol Swords for requesting this enhancement.</p> \u000A \u000A <p><tt><see topic=\"ACL2____MEMOIZE-SUMMARY\">Memoize-summary</see></tt> and <tt><see topic=\"ACL2____CLEAR-MEMOIZE-STATISTICS\">clear-memoize-statistics</see></tt> are now \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions that return <v>nil</v>.  Thanks to Sol Swords \u000A for this enhancement.</p> \u000A \u000A <p><tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt> is now explicitly illegal for constrained functions. \u000A (Already such memoization was ineffective.)</p> \u000A \u000A <p>A new keyword argument, <v>:AOKP</v>, controls whether or not to allow \u000A memoization to take advantage of attachments; see <see topic=\"ACL2____MEMOIZE\">memoize</see> and for \u000A relevant background, see <see topic=\"ACL2____DEFATTACH\">defattach</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt> is now illegal by default for <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A functions that have not had their guards verified.  See <see topic=\"ACL2____MEMOIZE\">memoize</see> \u000A (keyword <v>:ideal-okp</v>) and see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see> (key \u000A <v>:memoize-ideal-okp</v>) for and explanation of this restriction and how to \u000A avoid it.</p> \u000A \u000A <p><see topic=\"ACL2____HISTORY\">History</see> commands such as <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> and <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> now \u000A display ``<v>M</v>'' or ``<v>m</v>'' to indicate memoized functions.  See <see topic=\"ACL2____PC\">pc</see>.</p></blockquote>"],

 "ACL2____NOTE-4-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 4.2 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, distributed books, Emacs support, and experimental \u000A versions.  Each change is described in just one category, though of course \u000A many changes could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>The <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> utility can now do finer-grained \u000A tracking, providing data for individual hypotheses and the conclusion of a \u000A rule.  See <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>.  To try this out, evaluate the form \u000A <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :all)</v>; then see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> \u000A for a discussion of display options using <v>show-accumulated-persistence</v>. \u000A Thanks to Dave Greve for suggesting this new capability and collaborating on \u000A its design and implementation.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> utility now permits the use of <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode functions, though this requires the use of a trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  See <see topic=\"ACL2____DEFATTACH\">defattach</see> and for discussion of the new capability, see \u000A <see topic=\"ACL2____DEFPROXY\">defproxy</see>, which explains how part of this change involves allowing \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions to be declared <see topic=\"ACL2____NON-EXECUTABLE\">non-executable</see>.</p> \u000A \u000A <p>Redefinition (see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>) is no longer permitted for \u000A functions that have attachments (see <see topic=\"ACL2____DEFATTACH\">defattach</see>).  In such cases, the \u000A attachment must be removed first, e.g. with <v>(<see topic=\"ACL2____DEFATTACH\">defattach</see> foo nil)</v>.</p> \u000A \u000A <p>Made small changes to <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> and <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> in order to \u000A permit guard verification of functions introduced with more than one \u000A quantified variable in a <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> form.  The change to <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> \u000A is to weaken the <see topic=\"ACL2____GUARD\">guard</see> by eliminating the requirement that the second \u000A argument satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>, and replacing the call of <tt><see topic=\"COMMON-LISP____ENDP\">endp</see></tt> in \u000A the definition body by a corresponding call of <tt><see topic=\"COMMON-LISP____ATOM\">atom</see></tt>.  The new \u000A definition of <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> is thus logically equivalent to the old \u000A definition, but with a weaker guard.  Thanks to Sol Swords for sending the \u000A following example, for which the final <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> form had failed \u000A but now succeeds.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (a b c) nil)\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-a-b-foo (c)\u000A   (<see topic=\"ACL2____FORALL\">forall</see> (a b) (foo a b c))\u000A   :witness-dcls ((<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">Xargs</see> :guard t\u000A                                  :verify-guards nil))))\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> forall-a-b-foo)</code> \u000A \u000A <p>The implementations of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>, <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, <tt><see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see></tt>, <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (and <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>), and <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt> have changed.  See the discussion below \u000A of the new utility, <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>.  A consequence is that <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> \u000A is explicitly disallowed for these and related symbols, which formerly could \u000A cause hard Lisp errors, because they are now macros.  Tracing of return-last \u000A is also disallowed.  Another consequence is that time$ now prints a more \u000A abbreviated message by default, but a version of the old behavior can be \u000A obtained with <v>:mintime nil</v>.</p> \u000A \u000A <p>The following utilities no longer print an observation about raw-mode \u000A transitions: <v>set-raw-mode-on</v>, <tt><see topic=\"ACL2____SET-RAW-MODE-ON_12\">set-raw-mode-on!</see></tt>, <tt><see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see></tt>, and <v>set-raw-mode-off</v>.  Thanks to Jared Davis for \u000A suggestion this change in the case of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> (which proved \u000A awkward to restrict to that case).</p> \u000A \u000A <p>The system function <v>translate-and-test</v> now permits its <v>LAMBDA</v> \u000A form to refer to the variable <v>WORLD</v>, which is bound to the current ACL2 \u000A logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>Modified abort handling to avoid talking about an interrupt when the error \u000A was caused by a Lisp error rather than an interrupt.</p> \u000A \u000A <p>The value of the constant <v>*acl2-exports*</v>, which is still a list, has \u000A been extended significantly, though only with the addition of symbols that one \u000A might reasonably have expected all along to belong to this list.  A new \u000A distributed book, <v>books/misc/check-acl2-exports.lisp</v>, checks (at \u000A certification time) that no documented constant, macro, or function symbol in \u000A the <v>&quot;ACL2&quot;</v> package has been accidentally omitted from \u000A <v>*acl2-exports*</v>.  Thanks to Dave Greve for helpful discussions related to \u000A this change.</p> \u000A \u000A <p>Improved the built-in `<v>untranslate</v>' functions to produce <v>let*</v> \u000A expressions when appropriate (more to help with tools that call \u000A <v>untranslate</v> and the like, than to help with proof output).</p> \u000A \u000A <p>The utility <tt><see topic=\"ACL2____REDO-FLAT\">redo-flat</see></tt> now works for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> failures, \u000A just as it continues to work for failures of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>.</p> \u000A \u000A <p>The following only affects users who use trust tags to add to list values \u000A of either of the <tt><see topic=\"ACL2____STATE\">state</see></tt> global variables <v>program-fns-with-raw-code</v> \u000A or <v>logic-fns-with-raw-code</v>.  For functions that belong to either of the \u000A above two lists, <v>trace$</v> will supply a default value of <v>:fncall</v> to \u000A keyword <v>:notinline</v>, to avoid discarding raw-Lisp code for the \u000A function.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of the macro <tt><see topic=\"COMMON-LISP____INTERN\">intern</see></tt> has been strengthened so that \u000A its second argument may no longer be either the symbol \u000A <v>*main-lisp-package-name*</v> or the string <v>&quot;COMMON-LISP&quot;</v>.  That change \u000A supports another change, namely that the following symbols in the \u000A <v>&quot;COMMON-LISP&quot;</v> package are no longer allowed into ACL2: symbols in that \u000A package that are not members of the list constant \u000A <v>*common-lisp-symbols-from-main-lisp-package*</v> yet are imported into the \u000A <v>&quot;COMMON-LISP&quot;</v> package from another package.  See <see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> and \u000A see <see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see>.  To see why we made that change, consider for \u000A example the following theorem, which ACL2 was able to prove when the host Lisp \u000A is GCL.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x &quot;ALLOCATE&quot;) (y 'car))\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> y)\u000A                       &quot;COMMON-LISP&quot;))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y))\u000A                  &quot;SYSTEM&quot;)))</code> \u000A \u000A <p>Now suppose that one includes a book with this theorem (with <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>nil</v>), using an ACL2 built on top of a different host Lisp, \u000A say CCL, that does not import the symbol <v>SYSTEM::ALLOCATE</v> into the \u000A <v>&quot;COMMON-LISP&quot;</v> package.  Then then one can prove <v>nil</v> by giving this \u000A theorem as a <v>:use</v> hint.</p> \u000A \u000A <p>The axioms introduced by <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> have changed.  See the discussion \u000A of <tt><see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see></tt> under ``NEW FEATURES'' below.</p> \u000A \u000A <p>The error message for free variables (e.g., in definition bodies and \u000A guards) now supplies additional information when there are governing IF \u000A conditions.  Thanks to Jared Davis for requesting this enhancement and \u000A collaborating in its design.</p> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____REDEF-\">redef-</see></tt> now turns off redefinition.</p> \u000A \u000A <p>Improved proof output in the case of a <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hint \u000A that proves the goal, so that the clause-processor function name is \u000A printed.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see></tt> command `<v>then</v>' now stops at the first \u000A failure (if any).</p> \u000A \u000A <p>It is no longer permitted to submit definitions in <v>:logic</v> mode for \u000A merely part of an existing <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event.  Such an action \u000A left the user in an odd state and seemed a potential soundness hole.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____BREAK_42\">break$</see></tt> is now in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  Thanks to \u000A Jared Davis for requesting this enhancement.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> now provides clearer output in the \u000A case that it is redundant.  More important perhaps, as a courtesy it now \u000A causes an error when applied to a constrained function, since presumably such \u000A an application was unintended (as the constrained function could never have \u000A been in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode).  Note that if one desires different \u000A behavior, one can create one's own version of <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> (but \u000A with a different name).</p> \u000A \u000A <p>Improved the <see topic=\"ACL2____GUARD\">guard</see>s for the following functions, often weakening \u000A them, to reflect more precisely the requirements for calling <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>: \u000A <v>alist-difference-eq</v>, <v>intersection-eq</v>, <v>intersection1-eq</v>, <tt><see topic=\"ACL2____INTERSECTP-EQ\">intersectp-eq</see></tt>, <v>not-in-domain-eq</v>, <v>set-difference-assoc-eq</v>, \u000A <v>set-equalp-eq</v>, and <tt><see topic=\"ACL2____UNION-EQ\">union-eq</see></tt>.  Thanks to Jared Davis for pointing \u000A out this issue for <tt><see topic=\"ACL2____INTERSECTP-EQ\">intersectp-eq</see></tt>.</p> \u000A \u000A <p>(CCL only) Made a change that can reduce the size of a compiled file \u000A produced by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> when the host Lisp is CCL, by discarding \u000A source information (for example, discarding <tt><see topic=\"ACL2____LOCAL\">local</see></tt> events).</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>See the discussion above about new statistics that can be gathered by the \u000A <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt> utility.</p> \u000A \u000A <p>A new hint, <v>:</v><tt><see topic=\"ACL2____INSTRUCTIONS\">instructions</see></tt>, allows use of the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> at the level of <see topic=\"ACL2____HINTS\">hints</see> to the prover.  Thanks to Pete \u000A Manolios for requesting this feature (in 2001!).  See <see topic=\"ACL2____INSTRUCTIONS\">instructions</see>.</p> \u000A \u000A <p>(For system hackers) There are new versions of system functions \u000A <v>translate1</v> and <v>translate</v>, namely <v>translate1-cmp</v> and \u000A <v>translate-cmp</v> respectively, that do not take or return <tt><see topic=\"ACL2____STATE\">state</see></tt>. \u000A See the Essay on Context-message Pairs for relevant information.  Thanks to \u000A David Rager for collaborating on this enhancement.</p> \u000A \u000A <p>A new utility, <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>, is now the unique ACL2 function that \u000A can pass back a multiple value result from one of its arguments.  Thus, now \u000A the following are macros whose calls ultimately expand to calls of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>: <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>, <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, \u000A <tt><see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see></tt>, <tt><see topic=\"ACL2____MBE\">mbe</see></tt> (and <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>), and \u000A <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>.  With an active trust tag, an advanced user can now write \u000A code that has side effects in raw Lisp; see <see topic=\"ACL2____RETURN-LAST\">return-last</see>.  Thanks to \u000A Jared Davis for requesting this feature.</p> \u000A \u000A <p>A new function, <tt><see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see></tt>, specifies the list of symbols imported \u000A into a given package.  The axioms for <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> have been strengthened, \u000A taking advantage of this function.  Now one can prove theorems using ACL2 that \u000A we believe could not previously be proved using ACL2, for example the \u000A following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> str t))\u000A       (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> str nil)))</code> \u000A \u000A <p>Thanks to Sol Swords for a helpful report, which included the example \u000A above.  See <see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> and see <see topic=\"ACL2____DEFPKG\">defpkg</see>.</p> \u000A \u000A <p>Added function <tt><see topic=\"ACL2____NO-DUPLICATESP-EQ\">no-duplicatesp-eq</see></tt>.</p> \u000A \u000A <p>Added a new hint keyword, <v>:</v><tt><see topic=\"ACL2____BACKCHAIN-LIMIT-RW\">backchain-limit-rw</see></tt>, to control the \u000A level of backchaining for <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____META\">meta</see>, and <see topic=\"ACL2____LINEAR\">linear</see> \u000A rules.  This overrides, for the current goal and (as with <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hints) descendent goals, the default <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> (see \u000A <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see>).  Thanks to Jared Davis for requesting this \u000A feature.</p> \u000A \u000A <p>Support is now provided for creating and certifying books that do not \u000A depend on trust tags, in the case that the only use of trust tags is during \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion.  See <see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-ACL2x</see>.  Thanks to Sol \u000A Swords for reporting a couple of bugs in a preliminary implementation.</p> \u000A \u000A <p>Function <v>(file-write-date$ filename state)</v> has been added, giving the \u000A write date of the given file.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for how to get new reports on the \u000A forward chaining activity occurring in your proof attempts.  Thanks to Dave \u000A Greve for inspiring the addition of this utility.</p> \u000A \u000A <p>It is now possible to use ACL2's printing utilities to return strings, by \u000A opening output channels to the keyword <v>:STRING</v> rather than to filenames. \u000A See <see topic=\"ACL2____IO\">io</see>.  Thanks to Jared Davis for a helpful conversation that led us \u000A to add this feature.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>We have slightly improved the handling of <v>:</v><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> \u000A rules that contain free variables.  Formerly, such rules might fire only once, \u000A when the first match for a free variable is discovered, and would not fire \u000A again even if subsequent forward chaining made available another match.  This \u000A made it difficult to predict whether a rule with free variables would fire or \u000A not, depending as it did on the order in which newly derived conclusions were \u000A added.  The new handling is a little slower but more predictable.  Thanks to \u000A Dave Greve for sending a helpful example that led us to consider making such \u000A an improvement.</p> \u000A \u000A <p>We have slightly improved the so-called ``<see topic=\"ACL2____TYPE-SET\">type-set</see>'' heuristics to \u000A work a bit harder on terms of the form <v>(rec term)</v>, where <v>rec</v> is a \u000A so-called ``compound-recognizer'' function, that is, a function with a \u000A corresponding enabled <v>:</v><tt><see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see></tt> rule.  Thanks to Jared \u000A Davis for sending a helpful example (found, in essence, in the modified \u000A function <v>type-set-rec</v>, source file <v>type-set-b.lisp</v>).</p> \u000A \u000A <p>We made three heuristic improvements in the way contexts (so-called \u000A ``type-alists'') are computed from goals (``clauses'').  Although these \u000A changes did not noticeably affect timing results for the ACL2 regression \u000A suite, they can be very helpful for goals with many hypotheses.  Thanks to \u000A Dave Greve for sending a useful example (one where we found a goal with 233 \u000A hypotheses!).</p> \u000A \u000A <p>The algorithm for substituting alists into terms was modified.  This change \u000A is unlikely to affect many users, but in one example it resulted in a speed-up \u000A of about 21%.  Thanks to Dave Greve for supplying that example.</p> \u000A \u000A <p>Sped up <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> a bit by memoizing checksums of symbols. \u000A (This change pertains to ``normal'' ACL2 only, not the <tt><see topic=\"ACL2____HONS\">hons</see></tt> version \u000A (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>, where such memoization already occurred.) \u000A We found about a 23% speed-up on an example from Dave Greve.</p> \u000A \u000A <p>Made a small change to the algorithm used to prove hypotheses of \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules (ACL2 source function \u000A <v>type-set-relieve-hyps</v>).  One change avoids a linear walk through the \u000A context (the ``type-alist'' structure), while the other could avoid storing \u000A unnecessary <tt><see topic=\"ACL2____FORCE\">force</see></tt>d assumptions (into the so-called ``tag-tree'').</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a long-standing soundness bug caused by the interaction of <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses with destructor <see topic=\"ACL2____ELIM\">elim</see>ination.  The fix was to avoid \u000A using forcing when building the context (so-called ``type-alist'') when the \u000A goal is considered for destructor elimination; those who are interested can \u000A see a discussion in source function <v>eliminate-destructors-clause1</v>, which \u000A includes a proof of <v>nil</v> that no longer succeeds.  A similar fix was made \u000A for generalization, though we have not exploited the previous code to prove \u000A <v>nil</v> in that case.</p> \u000A \u000A <p>Fixed a bug that allowed book certification to ignore <v>skip-proofs</v> \u000A around <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  Thus, a book could contain an event \u000A of the form <v>(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> (<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ...))</v>, and a call of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> on that book could succeed even without supplying keyword \u000A <v>:skip-proofs-okp t</v>.  This bug was introduced in Version 3.5 (May, \u000A 2009).</p> \u000A \u000A <p>Fixed a bug that could occur when including a book that attempts to \u000A redefine a function as a macro, or vice-versa.  (For details of the issue, see \u000A the comment in the definition of variable <v>*hcomp-fn-macro-restore-ht*</v> in \u000A source file <v>other-events.lisp</v>.)</p> \u000A \u000A <p>(Windows only) Fixed handling of the Windows drive so that an executable \u000A image saved on one machine can be used on another, even with a different \u000A drive.  Thanks to Harsh Raju Chamarthi for reporting this issue and doing a \u000A lot of testing and collaboration to help us get this right.</p> \u000A \u000A <p>Made a change to avoid possible low-level errors, such as bus errors, when \u000A quitting ACL2 by calling <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> or its synonyms.  This was occurring \u000A in CCL, and we are grateful to Gary Byers for helping us find the source of \u000A those errors (which basically was that ACL2 was attempting to quit while \u000A already in the process of quitting).</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see></tt>, which was being ignored in \u000A function bodies.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt>, which was not reverting the logical <see topic=\"ACL2____WORLD\">world</see> when an error resulted from evaluation of the given form.  Thanks to \u000A Jared Davis for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a long-standing bug (back through Version 2.7) that was discarding \u000A changes to the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>) when exiting and then \u000A re-entering the top-level ACL2 loop (with <tt><see topic=\"ACL2____LP\">lp</see></tt>).</p> \u000A \u000A <p>In some host Lisps, it has been possible to be in a situation where it is \u000A impossible to interrupt checkpoint printing during the summary.  We had \u000A thought this solved when the host Lisp was CCL, but Sol Swords sent us an \u000A example (for which we are grateful) illustrating that this behavior could \u000A occur.  This has been fixed.</p> \u000A \u000A <p>Fixed a bug in a proof obligation generated for <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> rules, that the <see topic=\"ACL2____GUARD\">guard</see> on the \u000A metafunction or clause-processor function, <v>fn</v>, holds under suitable \u000A assumptions.  Those assumptions include not only that the first argument of \u000A <v>fn</v> satisfies <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt>, but also that all <see topic=\"ACL2____STOBJ\">stobj</see> inputs \u000A satisfy the corresponding stobj recognizer predicates.  We had erroneously \u000A considered stobj outputs of <v>fn</v> instead of stobj inputs.  Thanks to Sol \u000A Swords for bringing this bug to our attention with a simple example, and \u000A correctly pointing us to the bug in our code.</p> \u000A \u000A <p>Fixed the following bugs in <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>.  We hadn't always been \u000A applying the full functional substitution when generating guard proof \u000A obligations.  We had been able to hit an assertion when reattaching to more \u000A than one function.  Attachment was permitted in the case of an untouchable \u000A function (see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>).  Finally, the guard proof obligation \u000A could fail in the case that the two functions have different formal parameter \u000A lists, as in the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((foo (x) x :guard (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x2)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x2)))\u000A  x2)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> foo bar)</code> \u000A \u000A <p>Fixed a raw Lisp error that could be caused by including a book using \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> to define a function symbol in a locally-introduced \u000A package.  An example appears in a comment in ACL2 source function \u000A <v>write-expansion-file</v>.</p> \u000A \u000A <p>Made a change that can prevent an error near the end of book certification \u000A when the underlying Host Lisp is Allegro Common Lisp, in the case that \u000A environment variable <v>ACL2_SYSTEM_BOOKS</v> has been set to the name of a \u000A directory with a parent that is a soft link.  Thanks to Dave Greve for \u000A supplying an example to led us to this fix, which involves avoiding Allegro \u000A CL's implementation of the Common Lisp function, <v>truename</v>.</p> \u000A \u000A <p>Fixed a bug that was failing to substitute fully using bindings of free \u000A variables in <tt><see topic=\"ACL2____FORCE\">force</see></tt>d hypotheses.  A related change is that instead of \u000A binding such a free variable to a new variable of the form <v>???-Y</v>, the new \u000A variable is now of the form <v>UNBOUND-FREE-Y</v>.</p> \u000A \u000A <p>Fixed a bug that could inhibit the printing of certain theory warnings (and \u000A probably, in the other direction, cause inappropriate such printing).</p> \u000A \u000A <p>We eliminated excessive <v>&quot;Raw-mode&quot;</v> warnings about <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> that could be generated by the use of raw-mode during \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  Thanks to Dave Greve for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>Fixed the printing of results from forms within an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, so \u000A that they are abbreviated according to the <tt><see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see></tt>.</p> \u000A \u000A <p>It is now possible to evaluate <see topic=\"ACL2____STOBJ\">stobj</see>-related forms after evaluating \u000A <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> <v>:none</v> or <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> <v>nil</v>, even in cases where such evaluation formerly \u000A caused a guard violation due to a bug in ACL2.  Here is an example of an error \u000A that no longer occurs.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A\u000ASummary\u000AForm:  ( DEFSTOBJ ST ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A ST\u000AACL2 !&gt;(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)\u000A\u000ATurning off guard checking entirely.  To allow execution in raw Lisp\u000Afor functions with guards other than T, while continuing to mask guard\u000Aviolations, :SET-GUARD-CHECKING NIL.  See :DOC set-guard-checking.\u000A\u000AACL2 &gt;(fld st)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (FLD ST),\u000Awhich is (STP ST), is violated by the arguments in the call (FLD ST).\u000A[... etc. ...]</code> \u000A \u000A <p>You can understand how things now work by imagining that when a function \u000A introduced by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> is called, <tt><see topic=\"ACL2____GUARD\">guard</see></tt>-checking values of \u000A <v>:none</v> or <v>nil</v> are temporarily converted to <v>t</v>.  Thanks to Pete \u000A Manolios, Ian Johnson, and Harsh Raju Chamarthi for requesting this \u000A improvement.</p> \u000A \u000A <p>Fixed a bug in which the wrong attachment could be made when the same \u000A function has an attachment in a book and another in the certification world of \u000A that book (possibly even built into ACL2), if the load of a compiled file is \u000A aborted because a sub-book's compiled file is missing.  The bug has been \u000A present since the time that <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> was added (Version_4.0).  An \u000A example may be found in a comment in the <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> for <v>note-4-2</v> \u000A (ACL2 source file <v>ld.lisp</v>).</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> and related utilities now cause a clean error when \u000A provided other than a symbol.  Thanks to Jared Davis for pointing out the raw \u000A Lisp error that had occurred in such cases.</p> \u000A \u000A <p>It had been the case that in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), it was \u000A possible to confuse <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> when including a book in a directory \u000A different from the current directory.  This has been fixed.  Thanks to Hanbing \u000A Liu for bringing this problem to our attention with a small example.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS AND RELATED INFRASTRUCTURE</b></p> \u000A \u000A <p>Many changes have been made to the distributed books, thanks to an active \u000A ACL2 community.  You can contribute books and obtain updates between ACL2 \u000A releases by visiting the <a href='http://acl2-books.googlecode.com/'>ACL2 \u000A Books</a> web page.</p> \u000A \u000A <p>There is new <v>Makefile</v> support for certifying just some of the \u000A distributed books.  See <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>, in particular \u000A discussion of the variable <v>ACL2_BOOK_DIRS</v>.  Thanks to Sandip Ray for \u000A requesting this enhancement.</p> \u000A \u000A <p>The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> now points to a new book, \u000A <v>books/make-event/defrule.lisp</v>, that shows how <v>make-event</v> can be used \u000A to do macroexpansion before generating <see topic=\"ACL2____EVENTS\">events</see>.  Thanks to Carl Eastlund \u000A for useful interaction on the acl2-help mailing list that led us to add this \u000A example.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>Incorporated a version of changes from Jared Davis to the <v>control-t f</v> \u000A emacs utility (distributed file <v>emacs/emacs-acl2.el</v>), so that one can \u000A fill a format string from anywhere within the string.</p> \u000A \u000A <p><b>EXPERIMENTAL VERSIONS</b></p> \u000A \u000A <p>We refrain from listing changes here to experimental versions, other than \u000A an enhancement to the <see topic=\"ACL2____HONS\">hons</see> version that can reduce sizes of <see topic=\"ACL2____CERTIFICATE\">certificate</see> files, by applying <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> to introduce structure \u000A sharing (ACL2 source function <v>make-certificate-file1</v>).</p>"],

 "ACL2____NOTE-4-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 4.1 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, distributed books, Emacs support, and experimental \u000A versions.  Each change is described in just one category, though of course \u000A many changes could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> associated with calls of the macro, <tt><see topic=\"COMMON-LISP____SEARCH\">search</see></tt>, has \u000A been weakened so that now, given strings are no longer restricted to contain \u000A only standard characters unless the <v>:test</v> argument is <tt><see topic=\"COMMON-LISP____CHAR-EQUAL\">char-equal</see></tt>.</p> \u000A \u000A <p>Modified the writing of ``hidden <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>'' forms into <see topic=\"ACL2____CERTIFICATE\">certificate</see> files (see <see topic=\"ACL2____HIDDEN-DEFPKG\">hidden-defpkg</see>), to support moving certificate \u000A files for distributed books, as is done by ACL2s (see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see>) and \u000A Debian releases of ACL2.  Thanks to Camm Maguire for reporting a problem with \u000A Debian releases of ACL2 that led to this change.</p> \u000A \u000A <p>Expanded the constant <v>*acl2-exports*</v> by adding <v>intersection-equal</v> \u000A to the list.  Thanks to Jared Davis for requesting this change.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> utility now compiles functions that have code \u000A conditionalized for raw Lisp only (presumably because a trust tag was active \u000A when they were defined).  Previously, this was not the case when <v>:comp</v> \u000A was applied to more than a single function symbol.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>A new macro, <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt>, allows evaluation directly in the top level \u000A loop for forms that normally need to be evaluated inside function bodies, such \u000A as <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt>.  See <see topic=\"ACL2____TOP-LEVEL\">top-level</see>.  Thanks to Jared Davis for \u000A requesting such a utility.</p> \u000A \u000A <p>Added <tt><see topic=\"COMMON-LISP____COUNT\">count</see></tt>, a Common Lisp function, to ACL2.  In support of that \u000A addition, also added rewrite rule <v>eqlablep-nth</v>.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>[None this time.]</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>We fixed a soundness bug that could occur when a function that returns \u000A multiple values that is called in its own guard.  Thanks to Sol Swords for \u000A reporting this bug and sending a small self-contained example, which is \u000A included in a comment in the function <v>chk-acceptable-defuns1</v> in ACL2 \u000A source file <v>defuns.lisp</v>.</p> \u000A \u000A <p>It was possible to cause an error when giving theory hints during \u000A redefinition of functions.  This has been fixed.  Thanks to Ian Johnson for \u000A sending an example that nicely illustrated this problem.</p> \u000A \u000A <p>Fixed system function <v>io?</v> for the case that formal parameter \u000A <v>commentp</v> is <v>t</v> and <v>vars</v> is non-empty.  Thanks to David Rager for \u000A bringing to our attention the fact that <v>io?</v> was broken for such a \u000A combination of parameters.</p> \u000A \u000A <p>Not exactly a bug fix, but: <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> was breaking when a \u000A <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> is specified, so we have improved the documentation (see \u000A <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>) to explain how to provide verified guards for a function \u000A introduced by <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>.  Thanks to Jared Davis for bringing this issue \u000A to our attention.</p> \u000A \u000A <p>Made a fix to the handling of interrupts, which in rare cases might have \u000A left one in a state where all subsequent proof attempts were labeled as \u000A ``Aborting due to an interrupt''.</p> \u000A \u000A <p>Fixed <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> and related utilities, so that when proof output is \u000A redirected to a file, all summary output goes to that file rather than to the \u000A terminal.</p> \u000A \u000A <p>(GCL on Windows only) Removed an inappropriate check, resulting in an error \u000A about ``pathname-device,'' that could prevent Windows GCL builds of ACL2. \u000A Thanks to Camm Maguire for reporting this problem and a helpful \u000A discussion.</p> \u000A \u000A <p>(Windows only) Modified the computation of canonical pathnames to avoid \u000A issues of case-insensitivity, in particular for the drive (e.g., <v>&quot;C:&quot;</v> \u000A vs. <v>&quot;c:&quot;</v>).  Thanks to Harsh Raju Chamarthi for reporting this issue and \u000A helping with its debugging.</p> \u000A \u000A <p>(Windows only) The value of <v>(<see topic=\"ACL2_____04\">@</see> distributed-books-dir)</v> no longer will \u000A be missing the Windows drive prefix, for example, <v>&quot;C:&quot;</v>.  Thanks to \u000A Harsh Raju Chamarthi for reporting this issue and helping with its \u000A debugging.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS AND RELATED INFRASTRUCTURE</b></p> \u000A \u000A <p>See the <a href='http://code.google.com/p/acl2-books/source/list'>log \u000A entries</a> for a record of books changed or added since the preceding \u000A release, with log entries.</p> \u000A \u000A <p>Modified <v>books/Makefile-generic</v> by adding a new <v>BOOKS_SKIP_COMP</v> \u000A variable, which is used in <v>Makefile</v>s in some subdirectories of \u000A <v>books/</v>, in order to avoid errors when compiling certified books for \u000A multiple Lisps.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>Distributed file <v>emacs/emacs-acl2.el</v> has been modified so that the \u000A forms <v>control-t e</v> and <v>control-t control-e</v> now pick up package \u000A markers (see <see topic=\"ACL2____SHARP-BANG-READER\">sharp-bang-reader</see>), in the following sense: if the \u000A top-level form is preceded by a line starting with <v>#!</v>, then that line is \u000A included in the inserted string.  Thanks to Jared Davis for suggesting this \u000A enhancement and providing a preliminary implementation.</p> \u000A \u000A <p><b>EXPERIMENTAL VERSIONS</b></p> \u000A \u000A <p>For the <v>HONS</v> version there have been some changes to <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>:</p> \u000A \u000A <blockquote><p><tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt> accepts a new keyword, <v>:recursive</v>, that is \u000A a synonym for the existing keyword <v>:inline</v>.  Thanks to Sol Swords for \u000A requesting this addition.  Moreover, it is now enforced that these keywords \u000A have Boolean values.</p> \u000A \u000A <p><tt><see topic=\"ACL2____MEMOIZE\">Memoize</see></tt> may now be called on <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions. \u000A Thanks to Sol Swords for requesting this enhancement.</p> \u000A \u000A <p>A bug has been fixed.  Now, if <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> is called with a \u000A <v>:condition-fn</v> (with value other than <v>nil</v> or <v>t</v>), then the <see topic=\"ACL2____GUARD\">guard</see> of the memoized function and the <v>:condition-fn</v> must be the same. \u000A Previously, one could exploit the lack of such a check to get a hard Lisp \u000A error, for example as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> cf (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))) (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'f :condition-fn 'cf)\u000A(f 3)</code> \u000A \u000A <p>Memoization is now illegal for built-in functions that use underlying raw \u000A Lisp in their implementations.  To see why, consider the form <v>(<see topic=\"ACL2____GC_42\">gc$</see>)</v>, \u000A which is a macro call expanding to <v>(gc$-fn nil)</v>.  Previously, after \u000A evaluation of <v>(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'gc$-fn)</v>, a call of <v>gc$</v> would no longer call \u000A the garbage collector, which had been invoked by raw Lisp code.  Now, \u000A evaluation of <v>(<see topic=\"ACL2____MEMOIZE\">memoize</see> 'gc$-fn)</v> causes an error.</p></blockquote>"],

 "ACL2____NOTE-3-5":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 3.4 into the following \u000A categories: changes to existing features, new features, heuristic \u000A improvements, bug fixes, new and updated books, Emacs support, and \u000A experimental <tt><see topic=\"ACL2____HONS\">hons</see></tt> version.  Each change is described in just one \u000A category, though of course many changes could be placed in more than one \u000A category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>Many improvements have been made to ACL2's ``evisceration'' mechanism for \u000A hiding substructures of objects before they are printed, and to related \u000A documentation:</p> \u000A \u000A <blockquote> \u000A \u000A <p>o A new documentation topic explains evisc-tuples.  See <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>.</p> \u000A \u000A <p>o A new interface, <tt><see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see></tt>, has been provided for setting \u000A the four global evisc-tuples.  See <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.</p> \u000A \u000A <p>o A new mode, ``iprinting'', allows eviscerated output to be read back in. \u000A See <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.</p> \u000A \u000A <p>o Function <v>default-evisc-tuple</v> has been deprecated and will probably \u000A be eliminated in future releases; use <v>abbrev-evisc-tuple</v> instead.  Also \u000A eliminated is the brr-term-evisc-tuple (also the user-brr-term-evisc-tuple). \u000A The term-evisc-tuple controls printing formerly controlled by the \u000A brr-term-evisc-tuple or user-brr-term-evisc-tuple.</p> \u000A \u000A <p>o ACL2 output is done in a more consistent manner, respecting the intention \u000A of those four global evisc-tuples.  In particular, more proof output is \u000A sensitive to the term-evisc-tuple.  Again, see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.</p> \u000A \u000A <p>o A special value, <v>:DEFAULT</v>, may be provided to <tt><see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see></tt> in order to restore these <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>s to their \u000A original settings.</p> \u000A \u000A <p>o (Details for heavy users of the evisc-tuple mechanism) (1) There are no \u000A longer <see topic=\"ACL2____STATE\">state</see> globals named <v>user-term-evisc-tuple</v> or \u000A <v>user-default-evisc-tuple</v>.  (2) Because of the above-mentioned \u000A <v>:DEFAULT</v>, if you have referenced state globals directly, you should use \u000A accessors instead, for example <v>(abbrev-evisc-tuple state)</v> instead of \u000A <v>(<see topic=\"ACL2_____04\">@</see> abbrev-evisc-tuple)</v>.  (3) For uniformity, <tt><see topic=\"ACL2____SET-TRACE-EVISC-TUPLE\">set-trace-evisc-tuple</see></tt> now takes a second argument, \u000A <v>state</v>.</p></blockquote> \u000A \u000A <p>Improved <tt><see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see></tt> in several ways.  First, it breaks earlier \u000A in a more appropriate place.  Thanks to Dave Greve for highlighting this \u000A problem with the existing implementation.  Also, <tt><see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see></tt> now \u000A breaks on hard errors, not only soft errors (see <see topic=\"ACL2____ER\">er</see>, options <v>hard</v> \u000A and <v>hard?</v>).  Thanks to Warren Hunt and Anna Slobodova for sending an \u000A example that showed a flaw in an initial improvement.  Finally, new options \u000A cause printing of the call stack for some host Common Lisps.  See <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>.  Thanks to Bob Boyer for requesting this feature.</p> \u000A \u000A <p><tt><see topic=\"ACL2____TRACE_12\">Trace!</see></tt> may now be used in raw Lisp (though please note that all \u000A soundness claims are off any time you evaluate forms in raw Lisp!).  Thanks to \u000A Bob Boyer for feedback that led to this enhancement.</p> \u000A \u000A <p>ACL2 now searches for file <v>acl2-customization.lsp</v> in addition to (and \u000A just before) its existing search for <v>acl2-customization.lisp</v>; See <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.  Thanks to Jared Davis for suggesting this change, which \u000A supports the methodology that files with a <v>.lisp</v> extension are \u000A certifiable books (thus avoiding the need to set the <v>BOOKS</v> variable in \u000A makefiles; see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>).</p> \u000A \u000A <p>Improved the error message for illegal <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms of the form \u000A <v>(<see topic=\"COMMON-LISP____TYPE\">type</see> (satisfies ...))</v>.  Thanks to Dave Greve for sending an example \u000A highlighting the issue.</p> \u000A \u000A <p>If trace output is going to a file (because <tt><see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see></tt> has \u000A been executed), then a note will be printed to that effect at the time that a \u000A call of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> or <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt> is applied to one or more <see topic=\"COMMON-LISP____TRACE\">trace</see> specs.</p> \u000A \u000A <p>The notion of redundancy (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) has been made more \u000A restrictive for <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> events.  Now, if either the old or \u000A new event is a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> event, then redundancy requires that \u000A both are <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> events that define the same set of function \u000A symbols.  Although we are not aware of any soundness bugs fixed by this \u000A modification, nevertheless we believe that it reduces the risk of soundness \u000A bugs in the future.</p> \u000A \u000A <p>The definition of <v>trace*</v> has been moved to a book, \u000A <v>misc/trace1.lisp</v>.  A new version, used in ACL2s, is in book \u000A <v>misc/trace-star.lisp</v>.  <see topic=\"COMMON-LISP____TRACE\">Trace</see> utilities <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt> are still built into ACL2.  [Note: File <v>misc/trace1.lisp</v> was \u000A deleted after Version 4.2.]</p> \u000A \u000A <p>Certain <see topic=\"ACL2____CERTIFICATE\">certificate</see> files will now be much smaller, by printing in a \u000A way that takes advantage of structure sharing.  Certifying the following \u000A example produces a <v>.cert</v> file of over 3M before this change, but less \u000A than 1K after the change.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nest (i)\u000A  ;; Makes an exponentially-sized nest of conses i deep.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> i)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((next (nest (<see topic=\"COMMON-LISP____1-\">1-</see> i))))\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> next next))))\u000A\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A `(<see topic=\"ACL2____DEFCONST\">defconst</see> *big* ',(nest 20)))</code> \u000A \u000A <p>Thanks to Sol Swords for providing the above example and to him as well as \u000A to Bob Boyer, Jared Davis, and Warren Hunt for encouraging development of this \u000A improvement.  We have also applied this improvement to the printing of \u000A function definitions to files on behalf of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____COMP\">comp</see></tt>.</p> \u000A \u000A <p>Names of symbols are now printed with vertical bars according to the Common \u000A Lisp spec.  Formerly, if the first character of a symbol name could be the \u000A first character of the print representation of a number, then the symbol was \u000A printed using vertical bars (<v>|..|</v>) around its name.  Now, a much more \u000A restrictive test for ``potential numbers'' is used, which can result in fewer \u000A such vertical bars.  Base 16 is now carefully considered as well; see <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see>.  Thanks to Bob Boyer for requesting this improvement.  Note \u000A that macros <v>set-acl2-print-base</v> and <v>set-acl2-print-case</v> have been \u000A replaced by functions; see <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> and see <see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see>.</p> \u000A \u000A <p>The ACL2 reader now supports `<v>#.</v>' syntax in place of the `<v>#,</v> \u000A syntax formerly supported.  Thanks to Bob Boyer for requesting this change. \u000A See <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>.  NOTE that because of this change, `<v>#.</v>' no \u000A longer causes an abort; instead please use <v>(<see topic=\"ACL2____A_12\">a!</see>)</v> or optionally, if in the \u000A ACL2 loop, <v>:a!</v>; see <see topic=\"ACL2____A_12\">a!</see>.</p> \u000A \u000A <p>Some small changes have been made related to <see topic=\"ACL2____GAG-MODE\">gag-mode</see>:</p> \u000A \u000A <blockquote> \u000A \u000A <p>o <see topic=\"ACL2____GAG-MODE\">Gag-mode</see> now suppresses some messages that were being printed upon \u000A encountering disjunctive splits from <v>:OR</v> <see topic=\"ACL2____HINTS\">hints</see>.  Thanks to Sol \u000A Swords for suggesting this improvement.</p> \u000A \u000A <p>o ACL2 had printed ``<v>Q.E.D.</v>'' with all output suppressed and <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> enabled.  Now, ``<v>Q.E.D.</v>'' will be suppressed when <v>PROVE</v> \u000A and <v>SUMMARY</v> output are suppressed, even if <v>gag-mode</v> is enabled.</p> \u000A \u000A <p>o The use of <tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt> had drastic effects on the inhibited \u000A output (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), basically inhibiting nearly all \u000A output (even most warnings) when turning on gag-mode and enabling all output \u000A except <v>proof-tree</v> output when turning off gag-mode.  Now, <tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt> only inhibits or enables proof (<v>PROVE</v>) output, according to \u000A whether gag-mode is being turned on or off (respectively).  The related \u000A utility <tt><see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see></tt> has also been modified, basically to \u000A eliminate <v>:all</v> as a first argument and to allow <v>t</v> and <v>:all</v> as \u000A second arguments, for inhibiting prover output or virtually all output, \u000A respectively (see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>).</p></blockquote> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFSTUB\">defstub</see></tt> event <see topic=\"ACL2____SIGNATURE\">signature</see> specifying output of the form \u000A <v>(<see topic=\"ACL2____MV\">mv</see> ...)</v> now introduces a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule asserting \u000A that the new function returns a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> result.  Thanks to Bob \u000A Boyer for sending the following example, which motivated this change.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> census (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; (<see topic=\"ACL2____MV\">mv</see> * *))\u000A(<see topic=\"ACL2____DEFN\">defn</see> foo (x)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (a1 a2)\u000A          (census x)\u000A          (<see topic=\"COMMON-LISP____LIST\">list</see> a1 a2)))</code> \u000A \u000A <p>Improved the efficiency of <tt><see topic=\"ACL2____STRING-APPEND\">string-append</see></tt> so that in raw Lisp, it \u000A calls <tt><see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see></tt>.  Thanks to Jared Davis for suggesting this change, \u000A including the use of <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.  A minor change was made to the definition \u000A of <tt><see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see></tt> to support this change, and the lemma \u000A <v>append-to-nil</v> was added (see below).</p> \u000A \u000A <p>The checksum algorithm used for <see topic=\"ACL2____CERTIFICATE\">certificate</see> files of <see topic=\"ACL2____BOOKS\">books</see> \u000A has been changed.  Thanks to Jared Davis for contributing the new code.  This \u000A change will likely not be noticed unless one is using the experimental <tt><see topic=\"ACL2____HONS\">hons</see></tt> version of ACL2, where it can greatly speed up book certification and \u000A inclusion because of function memoization (of source function \u000A <v>fchecksum-obj</v>).</p> \u000A \u000A <p>Fewer calls are made to the checksum algorithm on behalf of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and a few other operations.  Thanks to Jared Davis for providing \u000A data that helped lead us to these changes.</p> \u000A \u000A <p>Formatted printing directives <v>~p</v>, <v>~q</v>, <v>~P</v>, and <v>~Q</v> are \u000A deprecated, though still supported.  See <see topic=\"ACL2____FMT\">fmt</see>.  Instead, please use \u000A <v>~x</v>, <v>~y</v>, <v>~X</v>, and <v>~Y</v> (respectively).  As a by-product, rule \u000A names in proof output are no longer hyphenated.</p> \u000A \u000A <p>A new keyword, <v>:multiplicity</v>, is available for tracing raw Lisp \u000A functions using the ACL2 <see topic=\"COMMON-LISP____TRACE\">trace</see> utility.  See <see topic=\"ACL2____TRACE_42\">trace$</see>.</p> \u000A \u000A <p>Users may now control whether or not a slow array access results in a \u000A warning printed to the screen (which is the default, as before), and if so, \u000A whether or not the warning is followed by a break.  See <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>.</p> \u000A \u000A <p>On linux-like systems (including Mac OS X and SunOS), <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> \u000A will now write its temporary files into the <v>&quot;/tmp&quot;</v> directory, which is \u000A the value of <tt><see topic=\"ACL2____STATE\">state</see></tt> global <v>'tmp-dir</v>.  You can change that \u000A directory with <v>(<see topic=\"ACL2____ASSIGN\">assign</see> tmp-dir &quot;&lt;your_temp_directory_path&gt;&quot;)</v>.</p> \u000A \u000A <p>The messages printed for uncertified books have been enhanced.  Thanks to \u000A Jared Davis for requesting such an improvement.</p> \u000A \u000A <p>A function definition that would be redundant if in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A mode is now considered redundant even if it (the new definition) is in \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  That is, if a definition is ``downgraded'' from \u000A <v>:logic</v> to <v>:program</v> mode, the latter (<v>:program</v> mode) definition \u000A is considered redundant.  Previously, such redundancy was disallowed, but we \u000A have relaxed that restriction because of a scenario brought to our attention \u000A by Jared Davis: include a book with the <v>:logic</v> mode definition, and then \u000A include a book with the <v>:program</v> mode definition followed by <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt>.  Thanks, Jared.</p> \u000A \u000A <p>The ACL2 reader no longer accepts characters other than those recognized by \u000A <tt><see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see></tt> except for <v>#\\Tab</v>, <v>#\\Page</v>, and \u000A <v>#\\Rubout</v> (though it still accepts strings containing such characters). \u000A As a result, no <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion is allowed to contain any such \u000A unacceptable character or string.  Thanks to Sol Swords for sending an example \u000A that led us to make this restriction.  A simple example is the following \u000A book:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *my-null* (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0))\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> `(<see topic=\"ACL2____DEFCONST\">defconst</see> *new-null* ,*my-null*))</code> \u000A \u000A <p>For this book, a call of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> formerly broke during the \u000A compilation phase, but if there was no compilation, then a call of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> broke.  Now, the error occurs upon evaluation of the <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> form.</p> \u000A \u000A <p>ACL2 now collects up <see topic=\"ACL2____GUARD\">guard</see>s from <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms more as a \u000A user might expect, without introducing an unexpected ordering of conjuncts. \u000A We thank Jared Davis for sending us the following illustrative example, \u000A explained below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                              (<see topic=\"ACL2____NATP\">natp</see> n)\u000A                              (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) n)))\u000A           (<see topic=\"COMMON-LISP____TYPE\">type</see> string x)\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x n))\u000A  t)</code> \u000A \u000A <p>Formerly, a guard was generated for this example by unioning the conjuncts \u000A from the <v>:guard</v> onto a list containing the term <v>(<see topic=\"COMMON-LISP____STRING\">string</see> x)</v> generated \u000A from the <v>type</v> declaration, resulting in an effective guard of:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A     (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) n)\u000A     (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x))</code> \u000A \u000A <p>The guard of this guard failed to be verified because <v>(<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x))</v> now \u000A comes after the call <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> x)</v>.  With the fix, contributions to the \u000A guards are collected up in the order in which they appear.  So in the above \u000A example, the effective guard is the specified <v>:guard</v>; the contribution \u000A <v>(<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)</v> comes later, and is thus dropped since it is redundant.  NOTE \u000A by the way that if <v>:guard</v> and <v>:stobjs</v> are specified in the same \u000A <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> form, then for purposes of collecting up the effective guard as \u000A described above, <v>:stobjs</v> will be treated as through it comes before the \u000A <v>:guard</v>.</p> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____CLOSE-OUTPUT-CHANNEL\">close-output-channel</see></tt> to try to do a better job flushing \u000A buffers.  Thanks to Bob Boyer for helpful correspondence.</p> \u000A \u000A <p>The notion of ``subversive recursion'' has been modified so that some \u000A functions are no longer marked as subversive.  See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>, in particular the discussion elaborating on the notion \u000A of ``involved in the termination argument'' at the end of that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic.</p> \u000A \u000A <p>Formerly, <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules for new definitions inside \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms were sometimes added as <see topic=\"ACL2____CONSTRAINT\">constraint</see>s.  This is \u000A no longer the case.  See also discussion of the ``soundness bug in the forming \u000A of constraints'', which is related.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>It is now possible to affect ACL2's termination analysis (and resulting \u000A induction analysis).  Thanks to Peter Dillinger for requesting this feature. \u000A The default behavior is essentially unchanged.  But for example, the following \u000A definition is accepted by ACL2 because of the use of the new \u000A <v>:ruler-extenders</v> features; See <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :ruler-extenders :all))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 3\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A            (f (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          nil)))</code> \u000A \u000A <p>The following lemma was added in support of the improvement to <tt><see topic=\"ACL2____STRING-APPEND\">string-append</see></tt> described above:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> append-to-nil\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x nil)\u000A                  x)))</code> \u000A \u000A <p>A mechanism has been provided for users to contribute documentation.  See \u000A <see topic=\"ACL2____MANAGING-ACL2-PACKAGES\">managing-ACL2-packages</see> for an example, which contains a link to an \u000A external web page on effective use of ACL2 packages, kindly provided by Jared \u000A Davis.  ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings may now link to external web pages \u000A using the new symbol, <v>~url</v>; see <see topic=\"ACL2____MARKUP\">markup</see>.  Of course, those links \u000A appear in the web version of the documentation, but you made need to take a \u000A bit of action in order for these to appear as links in the Emacs Info version; \u000A see <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>Added <tt><see topic=\"ACL2____INTERSECTP\">intersectp</see></tt> (similar to <tt><see topic=\"ACL2____INTERSECTP-EQ\">intersectp-eq</see></tt> and <tt><see topic=\"ACL2____INTERSECTP-EQUAL\">intersectp-equal</see></tt>).</p> \u000A \u000A <p>The user now has more control over how ACL2 prints forms; See <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>.  Thanks to Bob Boyer for useful discussions leading to this \u000A enhancement.</p> \u000A \u000A <p>Some Common Lisp implementations only allow the syntax \u000A <v>pkg-name::expression</v> when <v>expression</v> is a symbol.  The ACL2 reader \u000A has been modified to support a package prefix for arbitrary expressions; see \u000A <see topic=\"ACL2____SHARP-BANG-READER\">sharp-bang-reader</see>.  Thanks to Hanbing Liu for a query that led to this \u000A feature and to Pascal J. Bourguignon for suggesting an implmentation.</p> \u000A \u000A <p>Ill-formed <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings need not cause an error.  See <see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see>.  Thanks to Bob Boyer for requesting this \u000A feature.</p> \u000A \u000A <p>Type declarations are now permitted in <v>let*</v> forms; see <see topic=\"COMMON-LISP____LET_A2\">let*</see>, \u000A see <see topic=\"COMMON-LISP____DECLARE\">declare</see>, and see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>.</p> \u000A \u000A <p>(For Lisp programmers) Macro <v>with-live-state</v> has been provided for \u000A programmers who refer to free variable <v>STATE</v>, for example with macros \u000A that generate uses of <v>STATE</v>, and want to avoid compiler warnings when \u000A evaluating in raw Lisp.  For example, the following form can be submitted \u000A either inside or outside the ACL2 loop to get the desired effect (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>): <v>(<see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'doc-prefix &quot; &quot; state))</v>. \u000A For another example use of this macro, see the definition of <v>trace$</v> (ACL2 \u000A source file <v>other-events.lisp</v>).</p> \u000A \u000A <p>(System hackers only) Added <v>:</v><tt><see topic=\"ACL2____REDEF-\">redef-</see></tt> to undo the effect of \u000A <v>:</v><tt><see topic=\"ACL2____REDEF_B2\">redef+</see></tt>.  See <see topic=\"ACL2____REDEF-\">redef-</see>.</p> \u000A \u000A <p>Function <tt><see topic=\"ACL2____RANDOM_42\">random$</see></tt> is a built-in random number generator.  See <see topic=\"ACL2____RANDOM_42\">random$</see>.  Thanks to Sol Swords for requesting this feature and providing an \u000A initial implementation.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>Sped up <see topic=\"ACL2____GUARD\">guard</see> generation for some functions with large if-then-else \u000A structures in their bodies.  Thanks to Sol Swords for sending an illustrative \u000A example.</p> \u000A \u000A <p>Sped up <see topic=\"ACL2____GUARD\">guard</see> generation in some cases by evaluating ground \u000A (variable-free) subexpressions.  Thanks to Bob Boyer for sending a motivating \u000A example: <v>(<see topic=\"ACL2____DEFN\">defn</see> foo (x) (<see topic=\"COMMON-LISP____CASE\">case</see> x ((1 2) 1) ((3 4) 3) ... ((999 1000)\u000A 999)))</v>.</p> \u000A \u000A <p>Modified slightly a heuristic association of ``size'' with constants, which \u000A can result in significant speed-ups in proofs involving constants that are \u000A very large <v>cons</v> trees.</p> \u000A \u000A <p>Added a restriction in the linear arithmetic procedure for deleting \u000A polynomial inequalities from the linear database.  Formerly, an inequality \u000A could be deleted if it was implied by another inequality.  Now, such deletion \u000A requires that certain heuristic ``parent tree'' information is at least as \u000A restrictive for the weaker inequality as for the stronger.  Thanks to Dave \u000A Greve for bringing a relevant example to our attention and working with us to \u000A figure out some surprising behavior, and to Robert Krug for making a key \u000A observation leading to the fix.</p> \u000A \u000A <p>(GCL especially) Improved compiled code slightly by communicating to raw \u000A Lisp the output type when a function body is of the form <v>(<see topic=\"COMMON-LISP____THE\">the</see> character\u000A ...)</v>.  This tiny improvement will probably only be observed in GCL, if at \u000A all.</p> \u000A \u000A <p>Applied a correction suggested by Robert Krug to the variant of <tt><see topic=\"ACL2____TERM-ORDER\">term-order</see></tt> used in parts of ACL2's arithmetic reasoning.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed bugs in the handling of <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt> expressions, one of which had \u000A the capability of rendering ACL2 unsound.  Thanks to Sol Swords for pointing \u000A out two issues and sending examples.  One example illustrated how ACL2 was in \u000A essence throwing away outer <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt> bindings when processing an inner \u000A <v>flet</v>.  We have exploited that example to prove a contradiction, as \u000A follows: this book was certifiable before this fix.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> a (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'c x))\u000A\u000A; Example from Sol Swords, which failed to be admitted (claiming that\u000A; function A is undefined) without the above definition of A.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo1 (x y)\u000A   (<see topic=\"COMMON-LISP____FLET\">flet</see> ((a (x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'a x)))\u000A     (<see topic=\"COMMON-LISP____FLET\">flet</see> ((b (y) (<see topic=\"COMMON-LISP____LIST\">list</see> 'b y)))\u000A       (b (a (<see topic=\"COMMON-LISP____LIST\">list</see> x y))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> not-true\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo1 3 4)\u000A         '(b (c (3 4))))\u000A  :hints ((&quot;Goal&quot;\u000A           :in-theory\u000A           (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart foo1))))\u000A  :rule-classes nil)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use not-true))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Sol's second example, below, pointed to a second bug related to computing \u000A output signatures in the presence of nested <v>flet</v> expressions, which we \u000A have also fixed: this form failed before the fix.</p> \u000A \u000A <code>:trans (<see topic=\"COMMON-LISP____FLET\">flet</see> ((foo (a) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____FLET\">flet</see> ((bar (b) b)) a)))) x)</code> \u000A \u000A <p>Fixed a subtle soundness bug in the forming of constraints from deduced \u000A type prescriptions.  As a result, when ACL2 prints a warning message labeling \u000A encapsulated functions as ``subversive'', ACL2 will no longer deduce \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules for those functions.  Examples that \u000A exploit the bug in ACL2 Version_3.4 may be found in comments in ACL2 source \u000A function <v>convert-type-set-to-term</v> (file <v>other-processes.lisp</v>) and \u000A especially in function <v>putprop-type-prescription-lst</v> (file \u000A <v>defuns.lisp</v>).  For more on the general issue of ``subversive \u000A recursions,'' see <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.)</p> \u000A \u000A <p>Fixed a soundness bug in the handling of inequalities by the <see topic=\"ACL2____TYPE-SET\">type-set</see> mechanism, which was using the inequality database inside the body \u000A of a <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt>.</p> \u000A \u000A <p>Fixed a long-standing soundness bug in <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt>, whose raw Lisp code gave the logically incorrect result in the \u000A case of a single entry other than the <tt><see topic=\"ACL2____HEADER\">header</see></tt>, where that entry mapped \u000A an index to the <tt><see topic=\"ACL2____DEFAULT\">default</see></tt> value.  Also fixed soundness bugs in <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>, in the case of <v>:order &gt;</v>, where the raw Lisp code could drop \u000A the <tt><see topic=\"ACL2____HEADER\">header</see></tt> from the result or, when the input alist had entries in \u000A ascending order, fail to return an alist in descending order.  For example, \u000A the following book certified successfully.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> true-formula-1\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'a '((:HEADER :DIMENSIONS (4) :MAXIMUM-LENGTH 5\u000A                                  :DEFAULT 1 :NAME A :ORDER &lt;)\u000A                         (1 . 1)))\u000A         '((:HEADER :DIMENSIONS (4) :MAXIMUM-LENGTH 5\u000A                    :DEFAULT 1 :NAME A :ORDER &lt;)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see>))))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch-1\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use true-formula-1))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> true-formula-2\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'a '((:HEADER :DIMENSIONS (4) :MAXIMUM-LENGTH 5\u000A                                  :DEFAULT NIL :NAME A :ORDER &gt;)\u000A                         (1 . 1)\u000A                         (2 . 2)))\u000A         '((:HEADER :DIMENSIONS (4) :MAXIMUM-LENGTH 5\u000A                    :DEFAULT NIL :NAME A :ORDER &gt;)\u000A           (2 . 2)\u000A           (1 . 1)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see>))))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch-2\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use true-formula-2))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> true-formula-3\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'a '((:HEADER :DIMENSIONS (3) :MAXIMUM-LENGTH 4\u000A                                  :NAME A :ORDER &gt;)\u000A                         (1 . B)\u000A                         (0 . A)))\u000A         '((:HEADER :DIMENSIONS (3) :MAXIMUM-LENGTH 4\u000A                    :NAME A :ORDER &gt;)\u000A           (1 . B)\u000A           (0 . A)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____COMPRESS1\">compress1</see>))))\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ouch-3\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use true-formula-3))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Fixed a soundness bug involving measured subsets and <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt>, by changing <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> so that it uses \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  See <see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>, in particular the \u000A discussion about <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> near the end of that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A topic.  Peter Dillinger first raised the idea to us of making such a change; \u000A when we found this soundness bug, we were certainly happy to do so!</p> \u000A \u000A <p>Fixed a bug that could cause a hard Lisp error but not, apparently, \u000A unsoundness.  The bug was in the lack of attention to the order of guard and \u000A type declarations when checking for redundancy.  In the following example, the \u000A second definition was redundant during the first pass of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form.  The second definition, however, was stored on the second \u000A pass with a guard of <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))</v>, which caused a hard \u000A Lisp error when evaluating <v>(foo 3)</v> due to a misguided attempt to evaluate \u000A <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v> in raw Lisp.  The fix is to restrict redundancy of definitions so \u000A that the guard and type declarations must be in the same order for the two \u000A definitions.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))))\u000A          x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))))\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A   x))\u000A; Now we get a hard Lisp error from evaluation of the guard of foo:\u000A(foo 3)</code> \u000A \u000A <p>Fixed a bug in the guard violation report for function <tt><see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see></tt>.  Thanks to Dave Greve for bringing this bug to \u000A our attention.</p> \u000A \u000A <p>Made a change to allow certain <see topic=\"ACL2____HINTS\">hints</see>, in particular certain \u000A <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> hints, that had previously caused errors during \u000A termination proofs by viewing the function being defined as not yet existing. \u000A Thanks to Sol Swords for bringing this issue to our attention with a nice \u000A example.</p> \u000A \u000A <p>ACL2 now properly handles interrupts (via control-c) issued during printing \u000A of the checkpoint summary.  Previously it was possible on some platforms to \u000A make ACL2 hang when interrupting both during a proof and during the ensuing \u000A printing of the checkpoint summary.  Thanks to Jared Davis and Sol Swords for \u000A bringing this problem to our attention.</p> \u000A \u000A <p>Fixed a bug that was preventing, inside some book <v>&quot;b&quot;</v>, the use of a \u000A <v>:dir</v> argument to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> that refers to a directory defined \u000A using <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> earlier in the book <v>&quot;b&quot;</v>.  (We found \u000A this ourselves, but we thank John Cowles for observing it independently and \u000A sending us a nice example.)</p> \u000A \u000A <p>(GCL and CCL only) Fixed a bug in certain under-the-hood type inferencing. \u000A Thanks to Sol Swords for sending an example using <see topic=\"ACL2____STOBJ\">stobj</see>s defined with \u000A the <v>:inline</v> keyword, along with a helpful backtrace in CCL, and to Gary \u000A Byers for his debugging help.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____PRINT-GV\">print-gv</see></tt>, which was mishandling calls of functions \u000A with more than one argument.</p> \u000A \u000A <p>Fixed a bug in the handling of <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> terms by the \u000A <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>DV</v>, including numeric (``diving'') commands. \u000A Thanks for Mark Reitblatt for bringing this problems to our attention with a \u000A helpful example.</p> \u000A \u000A <p>Fixed printing of goal names resulting from the application of <v>:OR</v> \u000A <see topic=\"ACL2____HINTS\">hints</see> so that they aren't ugly when working in other than the \u000A <v>&quot;ACL2&quot;</v> package.  Thanks to Sol Swords for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>Fixed <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> printing so that interrupts will not cause problems \u000A with hiding ordinary output because of incomplete proof-tree output.  Thanks \u000A to Peter Dillinger for pointing out this issue.</p> \u000A \u000A <p>Fixed a hard error that could be caused by mishandling a <tt><see topic=\"ACL2____FORCE\">force</see></tt>d \u000A hypothesis during <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see>.  Thanks to Peter Dillinger for \u000A bringing this bug to our attention by sending a useful example.</p> \u000A \u000A <p>Fixed a bug that could cause simplifications to fail because of alleged \u000A ``specious simplification.''  This bug could appear when deriving an equality \u000A from the linear arithmetic database, and then attempting to add this equality \u000A to the current goal's hypotheses when it was already present.  Thanks to Eric \u000A Smith for sending a helpful example (in July 2005!) that helped us debug this \u000A issue.</p> \u000A \u000A <p>Fixed a bug in processing of <v>:</v><tt><see topic=\"ACL2____TYPE-SET-INVERTER\">type-set-inverter</see></tt> rules.</p> \u000A \u000A <p>Fixed a bug that was causing an error, at least for an underlying Lisp of \u000A CCL (OpenMCL), when <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt> was applied to a term returning multiple \u000A values.  Thanks to Sol Swords for sending an example that brought this bug to \u000A our attention.</p> \u000A \u000A <p>Fixed handling of array orders to treat keyword value <v>:order :none</v> \u000A correctly from an array's <tt><see topic=\"ACL2____HEADER\">header</see></tt>.  Previously, there were two \u000A problems.  One problem was that <v>:order :none</v> was treated like the default \u000A for <v>:order</v>, <v>&lt;</v>, while <v>:order nil</v> was treated in the manner \u000A specified by <v>:order :none</v> (see <see topic=\"ACL2____ARRAYS\">arrays</see>).  Now, both <v>:order\u000A :none</v> and <v>:order nil</v> are treated as <v>:order nil</v> had been treated, \u000A i.e., so that there is no reordering of the alist by <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.  The \u000A other problem with this case of <v>:order</v> was that the <v>:maximum-length</v> \u000A field of the <tt><see topic=\"ACL2____HEADER\">header</see></tt> was not being respected: the length could grow \u000A without bound.  Now, as previously explained (but not previously implemented) \u000A &mdash; see <see topic=\"ACL2____ARRAYS\">arrays</see> &mdash; a <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> call made on behalf of \u000A <v>aset1</v> causes a hard error if the header of the supplied array specifies \u000A an <v>:order</v> of <v>:none</v> or <v>nil</v>.</p> \u000A \u000A <p>An <v>ignorable</v> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form had caused an error in some contexts \u000A when it should have been allowed.  In particular, this problem could arise \u000A when using an <v>ignorable</v> declaration at the top level in a <tt><see topic=\"ACL2____DEFABBREV\">defabbrev</see></tt> form.  It could also arise upon calling <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> \u000A when the corresponding <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form contained an <v>ignorable</v> \u000A declaration at the top level.  These bugs have been fixed.</p> \u000A \u000A <p>Contrary to existing documentation (see <see topic=\"ACL2____MAKE-EVENT-DETAILS\">make-event-details</see>), the \u000A value of ``<tt><see topic=\"ACL2____LD\">ld</see></tt> special variable'' <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> was always \u000A set to <v>nil</v> during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion, not merely when the \u000A <v>make-event</v> form has a non-<v>nil</v> value for keyword parameter \u000A <v>:check-expansion</v>.  This has been fixed.  Thanks to Sol Swords for \u000A bringing this issue to our attention.</p> \u000A \u000A <p>We have disallowed the certification of a book when not at the top-level, \u000A either directly in the top-level loop or at the top level of <tt><see topic=\"ACL2____LD\">ld</see></tt>. \u000A Before this restriction, the following would certify a book with a definition \u000A such as <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (h x))</v> that calls function <v>h</v> before defining \u000A it, if the certification was by way of the form such as:</p> \u000A \u000A <code>(<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x) x) (<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;my-book&quot;))</code> \u000A \u000A <p>But a subsequent <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> of <v>&quot;my-book&quot;</v> would then fail, \u000A because <v>h</v> is not defined at the top level.</p> \u000A \u000A <p>Printing with <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive <v>~c</v> now works properly even when \u000A the print-base is other than 10.  Thanks to Sol Swords for reporting this bug \u000A and providing a fix for it.</p> \u000A \u000A <p>(SBCL, CMUCL, and CCL only) Fixed a bug in <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt> in the \u000A case that the underlying Common Lisp is SBCL, CMUCL, or CCL (OpenMCL).  Thanks \u000A to Jun Sawada for bringing this bug to our attention and providing a fix.</p> \u000A \u000A <p>Fixed a bug that was preventing <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> events in \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events.  Thanks to Jared Davis for bringing this bug to \u000A our attention.</p> \u000A \u000A <p>Fixed a bug evidenced by error message ``Unexpected form in certification \u000A world'', which could result from attempting to certify a book after evaluating \u000A an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form with a local <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>.  Thanks to Jared \u000A Davis for pointing out this bug and sending the example:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (x) `(<see topic=\"ACL2____TABLE\">table</see> foo 'bar ,x)))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (foo 3)))</code> \u000A \u000A <p>Formerly, evaluating a <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> form inside a <see topic=\"ACL2____WORMHOLE\">wormhole</see> such as \u000A the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> loop could leave the user in a bad state after \u000A returning to the top level, in which that function could not be untraced. \u000A This has been fixed.  Note however that when you proceed from a break in the \u000A <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> loop, the tracing state will be the same as it was when \u000A you entered that break: all effects of calling <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt> are erased when you proceed from the break.</p> \u000A \u000A <p>A <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> of <v>(<see topic=\"COMMON-LISP____AND\">and</see>)</v> is no longer ignored.  Thanks to Sol \u000A Swords for bringing this bug to our attention.</p> \u000A \u000A <p>A bug has been fixed that could result in needlessly weak induction schemes \u000A in the case that a recursive call is made in the first argument of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>.  This has been fixed by including <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> as a default \u000A ruler-extender in the new ruler-extenders feature (see above, and see <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>).  For details on this bug see Example 11 in distributed book \u000A <v>books/misc/misc2/ruler-extenders-tests.lisp</v>.</p> \u000A \u000A <p>(For CCL/OpenMCL on Windows) ACL2 should now build on CCL (OpenMCL) on \u000A Windows.  Thanks to David Rager for bringing this issue to our attention and \u000A helping with a fix that worked for CCL 1.2, and to the CCL team for improving \u000A handling of Windows-style filenames in CCL 1.3.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS</b></p> \u000A \u000A <p>See <a \u000A href='https://code.google.com/p/acl2-books/wiki/ReleaseVersionNumbers'>ReleaseVersionNumbers</a> \u000A for a list of books in Version 3.5 of ACL2 but not Version 3.4.</p> \u000A \u000A <p>Run the shell command</p> \u000A \u000A <code> <v>svn log -r 94:HEAD</v> </code> \u000A \u000A <p>to see all changes to <v>books/</v> since the release of Version 3.4.</p> \u000A \u000A <p>Here are just a few highlights.</p> \u000A \u000A <p>Thanks largely to Jared Davis, many <v>Makefile</v>s have been improved to do \u000A automatic dependency analysis.  See <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see> for how \u000A to get your own <v>Makefile</v>s to do this by adding a line: <v>-include\u000A Makefile-deps</v>.</p> \u000A \u000A <p>Libraries <v>books/arithmetic-4/</v> and <v>books/rtl/rel7/</v> have been \u000A eliminated from the default book certification (<v>make regression</v>), in \u000A favor of new libraries <v>books/arithmetic-5/</v> and <v>books/rtl/rel8/</v> \u000A contributed by Robert Krug and Hanbing Liu, respectively.  They and Jun Sawada \u000A have arranged the compatibility of these libraries; i.e., it is possible to \u000A evaluate both of the following in the same session:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-5/top&quot; :dir :system)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;rtl/rel8/lib/top&quot; :dir :system)</code> \u000A \u000A <p>Library <v>books/rtl/rel1/</v> is no longer certified by default (though it \u000A is still distributed in support of ACL2(r); see <see topic=\"COMMON-LISP____REAL\">real</see>).</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>Slightly modified <v>Control-t e</v> (defined in <v>emacs/emacs-acl2.el</v>) to \u000A comprehend the notion of an ``ACL2 scope'', and added <v>Control-t o</v> to \u000A insert a superior <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> defining such a scope.  See the Emacs \u000A documentation for <v>Control-t e</v> (generally obtained after typing \u000A <v>Control-h k</v>).</p> \u000A \u000A <p>Modified distributed file <v>emacs/emacs-acl2.el</v> so that if you put the \u000A following two forms in your <v>~/.emacs</v> file above the form that loads \u000A <v>emacs/emacs-acl2.el</v>, then Emacs will not start up a shell.  Thanks to \u000A Terry Parks for leading us to this modification.</p> \u000A \u000A <code>(defvar acl2-skip-shell nil)\u000A(setq acl2-skip-shell t)</code> \u000A \u000A <p><b>EXPERIMENTAL HONS VERSION</b></p> \u000A \u000A <p>Bob Boyer and others have contributed numerous changes for the experimental \u000A ``<v>hons</v>'' version of ACL2 (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>).</p> \u000A \u000A <p>The ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> can now be queried with <v>(<see topic=\"ACL2_____04\">@</see> hons-enabled)</v> so \u000A that a result of <v>t</v> says that one is in the experimental <v>hons</v> \u000A version, while <v>nil</v> says the opposite.</p>"],

 "ACL2____NOTE-3-4":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 3.3 into changes to \u000A existing features, new features, bug fixes, new and updated books, and Emacs \u000A support.  Each change is described just once, though of course many changes \u000A could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>Fixed a long-standing potential infinite loop in the rewriter.  Thanks to \u000A Sol Swords for sending a concise example illustrating the looping behavior. \u000A (Those interested in details are welcome to look at the comment about loop \u000A behavior in source function <v>rewrite-equal</v>.)</p> \u000A \u000A <p>Incorporated a slight strengthening of non-linear arithmetic contributed by \u000A Robert Krug (thanks, Robert).  With non-linear arithmetic enabled, the problem \u000A was essentially that ACL2 made the following ``optimization'': given \u000A inequalities <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> a u)</v> and <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> b v)</v>, for positive rational constants \u000A <v>a</v> and <v>b</v> terms <v>u</v> and <v>v</v> of which at least one is known to be \u000A rational, infer <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> a b) (<see topic=\"COMMON-LISP_____A2\">*</see> u v))</v>.  Without this optimization, however, \u000A ACL2 now infers the stronger inequality obtained by direct multiplication of \u000A the two given inequalities.  To see the effect of this change, submit the \u000A event <v>(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see> t)</v> followed by:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 3 x)\u000A                   (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 4 y))\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 (<see topic=\"COMMON-LISP_____B2\">+</see> 12 (<see topic=\"COMMON-LISP_____A2\">*</see> -4 x) (<see topic=\"COMMON-LISP_____A2\">*</see> -3 y) (<see topic=\"COMMON-LISP_____A2\">*</see> x y)))))</code> \u000A \u000A <p>The utility <tt><see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see></tt> has been modified in \u000A several ways: it now takes a single argument (no longer takes <tt><see topic=\"ACL2____STATE\">state</see></tt> as \u000A an argument); a natural number <v>n</v> abbreviates the pair <v>(n . n)</v>; the \u000A argument is no longer evaluated, but it still optionally may be quoted; and a \u000A new value, <v>t</v>, suppresses all printing of the checkpoint summary.  Thanks \u000A to Jared Davis for suggesting most of these improvements.</p> \u000A \u000A <p>There was formerly a restriction on <tt><see topic=\"ACL2____MBE\">mbe</see></tt> that the <v>:exec</v> \u000A argument may not contain a call of <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.  This restriction has been \u000A removed, thanks to a request from Jared Davis and Sol Swords.  Thanks also to \u000A Sandip Ray, who pointed out that this restriction may have been in place in \u000A order that <tt><see topic=\"ACL2____DEFEXEC\">defexec</see></tt> can guarantee termination using the <v>:exec</v> \u000A code; its <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has therefore been updated to clarify this \u000A situation.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> are now stored by performing certain \u000A logical simplifications on the left side of the conclusion: <v>(<see topic=\"ACL2____PROG2_42\">prog2$</see> X Y)</v> \u000A is replaced by <v>Y</v>, <v>(<see topic=\"ACL2____MBE\">mbe</see> :logic X :exec Y)</v> is replaced by <v>X</v> (more \u000A precisely, the analogous change is made to the generated call of <tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>); and <v>(<see topic=\"COMMON-LISP____THE\">the</see> TYPE X)</v> is replaced by <v>X</v> (again, the \u000A change is actually made on the macroexpanded form).  Thanks to Jared Davis and \u000A Sol Swords for requesting this change.  An analogous change has also been made \u000A for rules of class <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> utility has been reimplemented to work independently of \u000A the underlying Lisp trace.  It thus works the same for every host Lisp, except \u000A as provided by an interface to the underlying host Lisp trace (the \u000A <v>:native</v> option).  Note that the host Lisp trace continues to be modified \u000A for GCL, Allegro CL, and CCL (OpenMCL); see <see topic=\"COMMON-LISP____TRACE\">trace</see>.  See <see topic=\"ACL2____TRACE_42\">trace$</see> \u000A for updated detailed documentation on tracing options, many of which are new, \u000A for example an <v>:evisc-tuple</v> option that can be set to <v>:no-print</v> if \u000A you want the function traced without the usual entry and exit printing.  The \u000A previous <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> had some issues, including the following, which have \u000A all been fixed.  Thanks to Peter Dillinger for assistance in determining \u000A desired functionality of the new <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and for helping to test \u000A it.</p> \u000A \u000A <blockquote> \u000A \u000A <p>Recursive calls were not always shown in the trace for two reasons.  (1) \u000A Compiler inlining could prevent recursive calls from being shown in the trace, \u000A in particular in CCL (OpenMCL).  Thanks to Jared Davis and Warren Hunt for \u000A pointing out this issue and requesting a fix, and to Bob Boyer and Gary Byers \u000A for relevant helpful discussions.  (2) ACL2's algorithm for producing \u000A executable counterparts prevented tracing of recursive calls even after \u000A <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)</v>.  Thanks to Peter Dillinger for requesting a \u000A fix.</p> \u000A \u000A <p>It was possible to exploit a bug in the interaction of multiple values and \u000A trace to prove a contradiction.  An example is in a comment in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see>\u000A note-3-4 ...)</v> in the ACL2 source code.</p> \u000A \u000A <p>Certain large structures could cause expensive computations for printing \u000A even when a <v>:cond</v> condition was specified and evaluated to <v>nil</v>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____TRACE_12\">Trace!</see></tt> now suppresses printing of the event summary, and returns \u000A the value that would be returned (if there is an active trust tag) by the \u000A corresponding call of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.</p> \u000A \u000A <p>Some bugs have been fixed in the underlying native trace installed by ACL2 \u000A for GCL, Allegro CL, and CCL (OpenMCL), including the following.  In GCL it \u000A had been impossible to use the variable <v>ARGLIST</v> in a <v>:cond</v> \u000A expression.  In Allegro CL and CCL, a <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> bug mishandled tracing \u000A non-ACL2 functions when directives such as <v>:entry</v> and <v>:exit</v> were \u000A supplied.  GCL trace now hides the world even when tracing non-ACL2 functions. \u000A Tracing in CCL no longer causes a Lisp error when untracing a traced function \u000A defined outside the ACL2 loop; for example <v>(<see topic=\"ACL2____TRACE_42\">trace$</see> len1)</v> followed by \u000A <v>(<see topic=\"ACL2____UNTRACE_42\">untrace$</see> len1)</v> no longer causes an error.</p></blockquote> \u000A \u000A <p>The macro <v>wet</v> has been changed, for the better we think.  see <see topic=\"ACL2____WET\">wet</see>.</p> \u000A \u000A <p>The generation of goals for <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>s has been changed to avoid \u000A dropping assumptions formerly deemed ``irrelevant''.  (A simple example may be \u000A found in a comment in source function <v>unencumber-assumption</v>, source file \u000A <v>prove.lisp</v>.)  Thanks to Jared Davis for sending us an example that led us \u000A to make this change.</p> \u000A \u000A <p>Modified the implementation of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> so that in the <see topic=\"ACL2____CERTIFICATE\">certificate</see> of a book, <tt><see topic=\"ACL2____LOCAL\">local</see></tt> events arising from <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> \u000A forms are elided.  For example, if <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> &lt;form&gt;)</v> expands to \u000A <v>(<see topic=\"ACL2____LOCAL\">local</see> &lt;expanded-form&gt;)</v>, then where the latter had been stored in the \u000A certificate, now instead <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :ELIDED))</v> will be stored. \u000A Thanks to Eric Smith for requesting this improvement.  He has reported that a \u000A preliminary version of this improvement shrunk a couple of his <v>.cert</v> \u000A files from perhaps 40MB each to about 140K each.</p> \u000A \u000A <p>Now, a <tt><see topic=\"ACL2____TABLE\">table</see></tt> event that sets the entire table, <v>(<see topic=\"ACL2____TABLE\">table</see> tbl nil\u000A alist :clear)</v>, is redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) when the supplied \u000A <v>alist</v> is equal to the current value of the table.  Thanks to Peter \u000A Dillinger for requesting this change.</p> \u000A \u000A <p>The event constructor <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> now returns the value that is returned \u000A by evaluation of its final form if no error occurs, except that it still \u000A returns <v>nil</v> if the that final evaluation leaves ACL2 in raw-mode.</p> \u000A \u000A <p><v>:</v><tt><see topic=\"ACL2____PSO\">Pso</see></tt> and <v>:</v><tt><see topic=\"ACL2____PSOG\">psog</see></tt> have been improved so that they \u000A show the key checkpoint summary at the end of a failed proof.  (For a \u000A discussion of key checkpoints, see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>.)  As a result, a call \u000A of <tt><see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see></tt> now affects subsequent evaluation of \u000A <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> and <v>:</v><tt><see topic=\"ACL2____PSOG\">psog</see></tt>.  In particular, you no longer need \u000A to reconstruct a proof (by calling <tt><see topic=\"ACL2____THM\">thm</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>) in order to \u000A see key checkpoints that were omitted due to the limit; just call <tt><see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see></tt> and then run <v>:pso</v> or <v>:psog</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> behaves a little differently under <see topic=\"ACL2____GAG-MODE\">gag-mode</see>.  Now, proof-checker commands that call the theorem prover to create \u000A new proof-checker goals, such as <v>bash</v> and <v>induct</v> (see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>), will show key checkpoints when in <see topic=\"ACL2____GAG-MODE\">gag-mode</see>. \u000A As before, proof-checker commands <v>pso</v> and <v>pso!</v> (and now, also \u000A <v>psog</v>) &mdash; see <see topic=\"ACL2____PSO\">pso</see>, see <see topic=\"ACL2____PSOG\">psog</see>, and see <see topic=\"ACL2____PSO_12\">pso!</see> \u000A &mdash; can then show the unedited proof log.  However, unlike proof attempts \u000A done in the ACL2 loop, such proof attempts will not show a summary of key \u000A checkpoints at the end, because from a prover perspective, all such goals were \u000A considered to be temporarily ``proved'' by giving them ``byes'', to be \u000A dispatched by later proof-checker commands.</p> \u000A \u000A <p>A little-known feature had been that a <see topic=\"ACL2____MEASURE\">measure</see> of <v>0</v> was treated \u000A as though no measure was given.  This has been changed so that now, a <see topic=\"ACL2____MEASURE\">measure</see> of <v>nil</v> is treated as though no measure was given.</p> \u000A \u000A <p>Expanded <v>*acl2-exports*</v> to include every documented symbol whose name \u000A starts with <v>&quot;SET-&quot;</v>.  Thanks to Jared Davis for remarking that <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt> was omitted from <v>*acl2-exports*</v>, which led to this \u000A change.</p> \u000A \u000A <p>The <see topic=\"COMMON-LISP____TRACE\">trace</see> mechanism has been improved so that the <v>:native</v> and \u000A <v>:multiplicity</v> options can be used together for Lisps that support the \u000A trace <v>:exit</v> keyword.  These Lisps include GCL and Allegro CL, whose \u000A native trace utilities have been modified for ACL2.  For SBCL and CCL \u000A (OpenMCL), which use the built-in Lisp mechanism for returning multiple values \u000A in ACL2 (see <see topic=\"ACL2____MV\">mv</see>), the use of <v>:multiplicity</v> with <v>:native</v> \u000A remains unnecessary and will be ignored.  In support of this change, the \u000A modification of native Allegro CL tracing for ACL2 was fixed to handle \u000A <v>:exit</v> forms correctly that involve <tt><see topic=\"ACL2____MV\">mv</see></tt>.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____REDEF_12\">redef!</see></tt> is just like <v>:</v><tt><see topic=\"ACL2____REDEF\">redef</see></tt>, but \u000A prints a warning rather than doing a query.  The old version of <v>:redef!</v> \u000A was for system hackers and has been renamed to <v>:</v><tt><see topic=\"ACL2____REDEF_B2\">redef+</see></tt>.</p> \u000A \u000A <p>Introduced a new utility for evaluating a function call using the so-called \u000A executable counterpart &mdash; that is, executing the call in the logic rather \u000A than in raw Lisp.  See <see topic=\"ACL2____EC-CALL\">ec-call</see>.  Thanks to Sol Swords for requesting \u000A this utility and participating in its high-level design.</p> \u000A \u000A <p>See <see topic=\"ACL2____PRINT-GV\">print-gv</see> for a new utility that assists with debugging guard \u000A violations. Thanks to Jared Davis for requesting more tool assistance for \u000A debugging guard violations.</p> \u000A \u000A <p>Improved the guard violation error message to show the positions of the \u000A formals, following to a suggestion of Peter Dillinger.</p> \u000A \u000A <p>Added new <tt><see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see></tt> capability to assist in debugging failed \u000A attempts at <see topic=\"ACL2____GUARD\">guard</see> verification.  See <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see>.  Thanks to \u000A Jared Davis for requesting a tool to assist in such debugging and to him, \u000A Robert Krug, and Sandip Ray for useful discussions.</p> \u000A \u000A <p>New utilities provide the formula to be proved by <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>. \u000A See <see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> and see <see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see>, Thanks to \u000A Mark Reitblatt for making a request leading to these utilities.  These \u000A utilities can be applied to a term, not just an event name; thanks to Peter \u000A Dillinger for correspondence that led to this extension.</p> \u000A \u000A <p>A new utility causes runes to be printed as lists in proof output from \u000A simplification, as is done already in proof summaries.  See <see topic=\"ACL2____SET-RAW-PROOF-FORMAT\">set-raw-proof-format</see>.  Thanks to Jared Davis for requesting this utility.</p> \u000A \u000A <p>An experimental capability allows for parallel evaluation.  See <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  Thanks to David Rager for providing an initial implementation \u000A of this capability.</p> \u000A \u000A <p>Defined <tt><see topic=\"ACL2____XOR\">xor</see></tt> in analogy to <tt><see topic=\"ACL2____IFF\">iff</see></tt>.  Thanks to Bob Boyer, Warren \u000A Hunt, and Sol Swords for providing this definition.</p> \u000A \u000A <p>Improved distributed file <v>doc/write-acl2-html.lisp</v> so that it can now \u000A be used to build HTML documentation files for <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings in \u000A user <see topic=\"ACL2____BOOKS\">books</see>.  See the comment in the definition of macro \u000A <v>acl2::write-html-file</v> at the end of that file.  Thanks to Dave Greve and \u000A John Powell for requesting this improvement.</p> \u000A \u000A <p>It is now possible to specify <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> for non-recursive \u000A function definitions (which can be useful when definitions are automatically \u000A generated).  See <see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see>.  Thanks to Sol Swords for \u000A requesting such a capability.</p> \u000A \u000A <p>Keyword argument <v>:dir</v> is now supported for <tt><see topic=\"ACL2____REBUILD\">rebuild</see></tt> just as it \u000A has been for <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>We relaxed the criteria for functional substitutions, so that a function \u000A symbol can be bound to a macro symbol that corresponds to a function symbol in \u000A the sense of <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>.  So for example, a functional \u000A substitution can now contain the doublet <v>(f +)</v>, where previously it would \u000A have been required instead to contain <v>(f binary-+)</v>.</p> \u000A \u000A <p>We now allow arbitrary packages in raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) \u000A &mdash; thanks to Jared Davis for requesting this enhancement &mdash; and more \u000A than that, we allow arbitrary Common Lisp in raw mode.  Note however that for \u000A arbitrary packages, you need to be in raw mode when the input is read, not \u000A just when the input form is evaluated.</p> \u000A \u000A <p>Two new keywords are supported by the <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> macro.  A \u000A <v>:gag-mode</v> keyword argument suppresses some prover output as is done by \u000A <tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt>.  Thanks to Jared Davis for asking for a convenient way \u000A to set <see topic=\"ACL2____GAG-MODE\">gag-mode</see> inside a book, in particular perhaps for a single \u000A theorem; this keyword provides that capability.  A <v>:stack</v> keyword allows \u000A sub-<see topic=\"ACL2____EVENTS\">events</see> of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> to ``pop'' the \u000A effect of a superior <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> call.  Thanks to Peter Dillinger for \u000A requesting such a feature.  See <see topic=\"ACL2____WITH-OUTPUT\">with-output</see>.</p> \u000A \u000A <p>The command <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> and its aliases <tt><see topic=\"ACL2____EXIT\">exit</see></tt> and <tt><see topic=\"ACL2____QUIT\">quit</see></tt> \u000A now all take an optional status argument, which provides the Unix exit status \u000A for the underlying process.  Thanks to Florian Haftmann for sending a query to \u000A the ACL2 email list that led to this enhancement.</p> \u000A \u000A <p>Keyword commands now work for macros whose argument lists have lambda list \u000A keywords.  For a macro with a <v>lambda</v> list keyword in its argument list, \u000A the corresponding keyword command reads only the minimum number of required \u000A arguments.  See <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.</p> \u000A \u000A <p>It is now legal to <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> variables <v>ignorable</v> in <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt> \u000A forms, as in <v>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((x (<see topic=\"COMMON-LISP_____B2\">+</see> a b)) ...) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> x)) ...)</v>. \u000A Thanks to Jared Davis for requesting this enhancement.</p> \u000A \u000A <p>Added a warning when more than one hint is supplied explicitly for the same \u000A goal.  It continues to be the case that only the first hint applicable to a \u000A given goal will be applied, as specified in the user-supplied list of \u000A <v>:hints</v> followed by the <tt><see topic=\"ACL2____DEFAULT-HINTS-TABLE\">default-hints-table</see></tt>.  Thanks to Mark \u000A Reitblatt for sending a question that led both to adding this clarification to \u000A the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> and to adding this warning.</p> \u000A \u000A <p>You may now use <tt><see topic=\"ACL2____SET-NON-LINEAR\">set-non-linear</see></tt>, <tt><see topic=\"ACL2____SET-LET_A2-ABSTRACTION\">set-let*-abstraction</see></tt>, \u000A <v>set-tainted-ok</v>, and <tt><see topic=\"ACL2____SET-LD-SKIP-PROOFS\">set-ld-skip-proofs</see></tt> in place of their versions \u000A ending in ``<v>p</v>''.  Thanks to Jared Davis for suggesting consideration of \u000A such a change.  All ``<v>set-</v>'' utilites now have a version without the \u000A final ``<v>p</v>'' (and most do not have a version with the final \u000A ``<v>p</v>'').</p> \u000A \u000A <p>Added a \"Loop-Stopper\" warning when a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule is \u000A specified with a <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field that contains illegal \u000A entries that will be ignored.  Thanks to Jared Davis for recommending such a \u000A warning.</p> \u000A \u000A <p>Added a substantial documentation topic that provides a beginner's guide to \u000A the use of quantification with <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> in ACL2.  Thanks to Sandip Ray \u000A for contributing this guide, to which we have made only very small \u000A modifications.  See <see topic=\"ACL2____QUANTIFIER-TUTORIAL\">quantifier-tutorial</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFUN-SK\">Defun-sk</see></tt> now allows the keyword option <v>:strengthen t</v>, which \u000A will generate the extra constraint that that is generated for the \u000A corresponding <v>defchoose</v> event; see <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  Thanks to Dave \u000A Greve for suggesting this feature.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a soundness bug related to the use of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> inside <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events.  An example proof of <v>nil</v> (before the fix) is in a \u000A comment in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-3-4 ...)</v> in the ACL2 source code.  We therefore \u000A no longer allow calls of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> inside <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events that \u000A have non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>s.</p> \u000A \u000A <p>Fixed a bug related to the definition of a function supporting the macro \u000A <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt>.  Although this bug was very unlikely to affect any user, \u000A it could be carefully exploited to make ACL2 unsound:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> fact\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAADR\">caadr</see> (<see topic=\"COMMON-LISP____CADDR\">caddr</see> (value-triple-fn '(foo 3) nil nil)))\u000A         'value) ; but it's state-global-let* in the logic\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :use fact :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (value-triple-fn))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> definitions of functions or macros are no longer \u000A considered redundant with built-ins when the built-ins have special raw Lisp \u000A code, because ACL2 was unsound without this restriction!  A comment about \u000A redundant definitions in source function <v>chk-acceptable-defuns</v> shows how \u000A one could prove <v>nil</v> without this new restriction.  Note that system \u000A utility <v>:</v><tt><see topic=\"ACL2____REDEF_B2\">redef+</see></tt> removes this restriction.</p> \u000A \u000A <p>Although ACL2 already prohibited the use of certain built-in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions for <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> and during \u000A macroexpansion, we have computed a much more complete list of functions that \u000A need such restrictions, the value of constant \u000A <v>*primitive-program-fns-with-raw-code*</v>.</p> \u000A \u000A <p>Modified what is printed when a proof fails, to indicate more clearly which \u000A event failed.</p> \u000A \u000A <p>Fixed a problem with <tt><see topic=\"ACL2____DMR\">dmr</see></tt> in CCL (OpenMCL) that was causing a raw \u000A Lisp break after an interrupt in some cases.  Thanks to Gary Byers for a \u000A suggestion leading to this fix.</p> \u000A \u000A <p>Fixed bugs in <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> code for dealing with free variables in \u000A hypotheses.</p> \u000A \u000A <p>Upon an abort, the printing of <tt><see topic=\"ACL2____PSTACK\">pstack</see></tt> and <see topic=\"ACL2____GAG-MODE\">gag-mode</see> summary \u000A information for other than GCL was avoided when inside a call of <tt><see topic=\"ACL2____LD\">ld</see></tt>. \u000A This has been fixed.</p> \u000A \u000A <p>(Windows only) Fixed bugs for ACL2 built on SBCL on Windows, including one \u000A that prevented <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> parameters <v>:dir :system</v> from working, \u000A and one that prevented certain compilation.  Thanks to Peter Dillinger for \u000A bringing these to our attention and supplying a fix for the second.  Thanks \u000A also to Andrew Gacek for bringing <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> issues to our \u000A attention.  Also, fixed writing of file <v>saved_acl2</v> at build time so that \u000A for Windows, Unix-style pathnames are used.</p> \u000A \u000A <p>Fixed a hard Lisp error that could occur with keywords as <tt><see topic=\"ACL2____TABLE\">table</see></tt> \u000A names, e.g., <v>(<see topic=\"ACL2____TABLE\">table</see> :a :a nil :put)</v>.  Thanks to Dave Greve for bringing \u000A this problem to our attention and providing this example.</p> \u000A \u000A <p>Fixed handling of <v>:OR</v> <see topic=\"ACL2____HINTS\">hints</see> so that proof attempts under an \u000A <v>:OR</v> hint do not abort (reverting to induction on the original input \u000A conjecture) prematurely.  Thanks to Robert Krug for pointing out this problem \u000A and pointing to a possible initial fix.</p> \u000A \u000A <p>(SBCL and CLISP only) It is now possible to read symbols in the \u000A <v>&quot;COMMON-LISP&quot;</v> package inside the ACL2 command loop (see <see topic=\"ACL2____LP\">lp</see>). \u000A This could cause a raw Lisp error in previous versions of ACL2 whose host \u000A Common Lisp was SBCL or CLISP.  Thanks to Peter Dillinger for bringing this \u000A issue to our attention.</p> \u000A \u000A <p>Fixed a bug that was preventing certain <see topic=\"ACL2____HINTS\">hints</see>, such as <v>:do-not</v> \u000A hints, from being used after the application of an <v>:or</v> hint.  Thanks to \u000A Robert Krug for bringing this bug to our attention.</p> \u000A \u000A <p>(Hons version only) Fixed a bug in the interaction of <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> \u000A (<tt><see topic=\"ACL2____HONS\">hons</see></tt> version only) with event processing, specifically in interaction \u000A with failures inside a call of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Thanks \u000A to Jared Davis for bringing this bug to our attention and sending an example. \u000A A simplified example may be found in a comment in source function \u000A <v>table-cltl-cmd</v>, source file <v>history-management.lisp</v>; search for \u000A ``Version_3.3'' there.</p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt> so that its <v>:evisc-tuple</v> is applied to the top \u000A clause, instead of using <v>(4 5 nil nil)</v> in all cases.  If no \u000A <v>:evisc-tuple</v> is supplied then <v>(term-evisc-tuple t state)</v> is used for \u000A the top clause, as it is already used for the rest of the stack.</p> \u000A \u000A <p>Fixed a bug in the interaction of <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>s with <v>:induct</v> hint \u000A value <v>:otf-flg-override</v>.  Thanks to Peter Dillinger for reporting this \u000A bug and sending an example that evokes it.</p> \u000A \u000A <p>Fixed bugs in <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> and <tt><see topic=\"ACL2____FIND-RULES-OF-RUNE\">find-rules-of-rune</see></tt> for the case \u000A of rule class <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt>.  Thanks to Robert Krug and Jared Davis for \u000A bringing these related bugs to our attention.</p> \u000A \u000A <p>Improved failure messages so that the key checkpoints are printed only once \u000A when there is a proof failure.  Formerly, a proof failure would cause the key \u000A checkpoints to be printed for every <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> superior to the proof attempt.</p> \u000A \u000A <p>Fixed a bug in generation of <see topic=\"ACL2____GUARD\">guard</see>s for calls of <tt><see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see></tt>.  Thanks to Mark Reitblatt for sending an example showing this \u000A bug.  The bug can be in play when you see the message: ``HARD ACL2 ERROR in \u000A MAKE-LAMBDA-APPLICATION: Unexpected unbound vars (\"\")''.  A distillation of \u000A Mark's example that causes this hard error is as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((y x)) (<see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see> y)))</code> \u000A \u000A <p>The <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt> macro now accepts test/value pairs of the form <v>(T\u000A val)</v> in other than the last position, such as the first such pair in \u000A <v>(<see topic=\"COMMON-LISP____COND\">cond</see> (t 1) (nil 2) (t 3))</v>.  Thanks to Jared Davis for sending this \u000A example and pointing out that ACL2 was sometimes printing goals that have such \u000A a form, and hence cannot be submitted back to ACL2.  A few macros \u000A corresponding to <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt> in some books under <v>books/rtl</v> and \u000A <v>books/bdd</v> were similarly modified.  (A second change will probably not be \u000A noticeable, because it doesn't affect the final result: singleton <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt> \u000A clauses now generate a call of <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> in a single step of macroexpansion, \u000A not of <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>.  For example, <v>(<see topic=\"COMMON-LISP____COND\">cond</see> (a) (b x) (t y))</v> now expands to \u000A <v>(<see topic=\"COMMON-LISP____OR\">OR</see> A (<see topic=\"COMMON-LISP____IF\">IF</see> B X Y))</v> instead of <v>(<see topic=\"COMMON-LISP____IF\">IF</see> A A (<see topic=\"COMMON-LISP____IF\">IF</see> B X Y))</v>.  See the source \u000A code for <v>cond-macro</v> for a comment about this change.)</p> \u000A \u000A <p>Fixed a bug in the interaction of <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>DV</v>, \u000A including numeric (``diving'') commands, with the <tt><see topic=\"ACL2____ADD-BINOP\">add-binop</see></tt> event. \u000A Specifically, if you executed <v>(<see topic=\"ACL2____ADD-BINOP\">add-binop</see> mac fn)</v> with <v>fn</v> having \u000A arity other than 2, a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command such as 3 or <v>(dv 3)</v> at \u000A a call of <v>mac</v> could have the wrong effect.  We also fixed a bug in diving \u000A with <v>DV</v> into certain <v>AND</v> and <v>OR</v> calls.  Thanks for Mark \u000A Reitblatt for bringing these problems to our attention with helpful \u000A examples.</p> \u000A \u000A <p>Fixed a couple of bugs that were causing an error, ``HARD ACL2 ERROR in \u000A RENEW-NAME/OVERWRITE''.  Thanks to Sol Swords for bringing the first of these \u000A bugs to our attention.</p> \u000A \u000A <p>Fixed a bug that could cause <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to fail in certain cases \u000A where there are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____START-PROOF-TREE\">start-proof-tree</see></tt> that could cause Lisp to hang or \u000A produce an error.  Thanks to Carl Eastlund for sending an example to bring \u000A this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in the proof output, which was failing to report cases where \u000A the current goal simplifies to itself or to a set including itself (see <see topic=\"ACL2____SPECIOUS-SIMPLIFICATION\">specious-simplification</see>).</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt> that was causing a raw Lisp \u000A error for a bad first argument.  Thanks to Peter Dillinger for pointing out \u000A this bug.</p> \u000A \u000A <p>The following was claimed in <v>:doc</v> <tt><see topic=\"ACL2____NOTE-3-3\">note-3-3</see></tt>, but was not fixed \u000A until the present release:<br></br> \u000A \u000A Distributed directory <v>doc/HTML/</v> now again includes installation \u000A instructions, in <v>doc/HTML/installation/installation.html</v>.</p> \u000A \u000A <p>In certain Common Lisp implementations &mdash; CCL (OpenMCL) and LispWorks, \u000A at least &mdash; an interrupt could leave you in a break such that quitting \u000A the break would not show the usual summary of key checkpoints.  This has been \u000A fixed.</p> \u000A \u000A <p><b>NEW AND UPDATED BOOKS</b></p> \u000A \u000A <p>Updated <v>books/clause-processors/SULFA/</v> with a new version from Erik \u000A Reeber; thanks, Erik.</p> \u000A \u000A <p>Added new books directory <v>tools/</v> from Sol Swords.  See \u000A <v>books/tools/Readme.lsp</v> for a summary of what these books provide.</p> \u000A \u000A <p>The distributed book <v>books/misc/file-io.lisp</v> includes a new utility, \u000A <v>write-list!</v>, which is like <v>write-list</v> except that it calls <tt><see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see></tt> instead of <tt><see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see></tt>.  Thanks to \u000A Sandip Ray for requesting this utility and assisting with its \u000A implementation.</p> \u000A \u000A <p>Added <v>record-update</v> macro supplied by Sandip Ray to distributed book \u000A <v>books/misc/records.lisp</v>.</p> \u000A \u000A <p>Sandip Ray has contributed books that prove soundness and completeness of \u000A different proof strategies used in sequential program verification. \u000A Distributed directory <v>books/proofstyles/</v> has three new directories \u000A comprising that contribution: <v>soundness/</v>, <v>completeness/</v>, and \u000A <v>counterexamples/</v>.  The existing <v>books/proofstyles/</v> directory has \u000A been moved to its subdirectory <v>invclock/</v>.</p> \u000A \u000A <p>Jared Davis has contributed a profiling utility for ACL2 built on CCL \u000A (OpenMCL).  See <v>books/misc/oprof.lisp</v>.  Thanks, Jared.</p> \u000A \u000A <p>ACL2 utilities <tt><see topic=\"ACL2____GETPROP\">getprop</see></tt> and <tt><see topic=\"ACL2____PUTPROP\">putprop</see></tt> take advantage of \u000A under-the-hood Lisp (hashed) property lists.  The new book \u000A <v>books/misc/getprop.lisp</v> contains an example showing how this works.</p> \u000A \u000A <p>Added the following new book directories: <v>books/paco/</v>, which includes \u000A a small ACL2-like prover; and <v>books/models/jvm/m5</v>, which contains the \u000A definition of one of the more elaborate JVM models, M5, along with other files \u000A including JVM program correctness proofs.  See files <v>Readme.lsp</v> in these \u000A directories, and file <v>README</v> in the latter.</p> \u000A \u000A <p>Added <i>books</i> about sorting in <v>books/sorting</v>.  See \u000A <v>Readme.lsp</v> in that directory for documentation.</p> \u000A \u000A <p>Added book <v>books/misc/computed-hint-rewrite.lisp</v> to provide an \u000A interface to the rewriter for use in computed hints.  Thanks to Jared Davis \u000A for requesting this feature.</p> \u000A \u000A <p>Jared Davis has provided a pseudorandom number generator, in \u000A <v>books/misc/random.lisp</v>.</p> \u000A \u000A <p>Robert Krug has contributed a new library, <v>books/arithmetic-4/</v>, for \u000A reasoning about arithmetic.  He characterizes it as being more powerful than \u000A its predecessor, <v>books/arithmetic-3/</v>, and without its predecessor's \u000A rewriting loops, but significantly slower than its predecessor on some \u000A theorems.</p> \u000A \u000A <p>Incorporated changes from Peter Dillinger to verify guards for functions in \u000A <v>books/ordinals/lexicographic-ordering.lisp</v> (and one in \u000A <v>ordinal-definitions.lisp</v> in that directory).</p> \u000A \u000A <p>A new directory, <v>books/hacking/</v>, contains a library for those who wish \u000A to use trust tags to modify or extend core ACL2 behavior.  Thanks to Peter \u000A Dillinger for contributing this library.  Obsolete version \u000A <v>books/misc/hacker.lisp</v> has been deleted.  Workshop contribution \u000A <v>books/workshops/2007/dillinger-et-al/code/</v> is still included with the \u000A workshops/ tar file, but should be considered deprecated.</p> \u000A \u000A <p>In <v>books/make-event/assert.lisp</v>, changed <v>assert!</v> and \u000A <v>assert!-stobj</v> to return <v>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :success)</v> upon success instead \u000A of <v>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)</v>, following a suggestion from Jared Davis.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>Changed <v>emacs/emacs-acl2.el</v> so that the fill column default (for the \u000A right margin) is only set (still to 79) in lisp-mode.</p> \u000A \u000A <p>Modified Emacs support in file <v>emacs/emacs-acl2.el</v> so that names of \u000A events are highlighted just as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> has been highlighted when it is \u000A called.  Search in the above file for <v>font-lock-add-keywords</v> for \u000A instructions on how to eliminate this change.</p> \u000A \u000A <p>The name of the temporary file used by some Emacs utilities defined in file \u000A <v>emacs/emacs-acl2.el</v> has been changed to have extension <v>.lsp</v> instead \u000A of <v>.lisp</v>; thus it is now <v>temp-emacs-file.lsp</v>.  Also, `make' commands \u000A to `clean' books will delete such files (specifically, \u000A <v>books/Makefile-generic</v> has been changed to delete \u000A <v>temp-emacs-file.lsp</v>).</p>"],

 "ACL2____NOTE-3-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Before this release, a raw Lisp error could put the ACL2 user into the \u000A debugger of the host Common Lisp.  Now, such cases will generally put the user \u000A back at the top-level loop after an informative message.  For details, see \u000A <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>; also see <see topic=\"ACL2____BREAK_42\">break$</see>.</p> \u000A \u000A <p>Fixed a soundness bug that was allowing unknown packages to sneak into a \u000A book and wreak havoc.  Thanks to Peter Dillinger for sending an interesting \u000A example that led us to an exploration resulting in finding this bug.  (A \u000A comment in the source code for <v>note-3-2</v> shows such an example.)  That \u000A example led us to fix a couple of other bugs related to packages.  See <see topic=\"ACL2____HIDDEN-DEATH-PACKAGE\">hidden-death-package</see> if you are generally interested in such issues, and for \u000A associated examples, see comments in <v>note-3-2</v> in the ACL2 source \u000A code.</p> \u000A \u000A <p>Fixed subtle soundness bugs related to <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules by \u000A restricting evaluators (see <see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see>), as discussed in a new \u000A documentation topic: see <see topic=\"ACL2____EVALUATOR-RESTRICTIONS\">evaluator-restrictions</see>.</p> \u000A \u000A <p>Fixed a soundness bug that was allowing redefinition from <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> to <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  This prohibition had been in ACL2 for \u000A awhile but was accidentally removed in the preceding version.</p> \u000A \u000A <p>Fixed a soundness bug related to <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  Thanks to Peter Dillinger \u000A for bringing it to our attention and for useful discussions, and providing a \u000A proof of <v>nil</v>, the essence of which is illustrated as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2____TRACE_42\">trace$</see> (bar :entry (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () 17))))</code> \u000A \u000A <p>Thus, <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> could be used to cause destructive raw Lisp behavior. \u000A Now, <v>trace$</v> fails unless it is either given a list of symbols or else \u000A there is an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>); otherwise, consider using \u000A <v>trace!</v> instead.</p> \u000A \u000A <p>Closed a loophole that could be viewed as compromising soundness.  It was \u000A possible to write files during book certification by exploiting <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion, but that is no longer the case by default.  A new \u000A function <tt><see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see></tt> is identical as a function to \u000A <v>open-output-channel</v>, except that the new function may be called even \u000A during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion and <tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> <see topic=\"ACL2____HINTS\">hints</see>, \u000A but requires that there is an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  Thanks \u000A to Peter Dillinger for producing a convincing example (forging a <see topic=\"ACL2____CERTIFICATE\">certificate</see> during book certification; see <see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see>) and \u000A to him, Sandip Ray, and Jared Davis for useful discussions on the topic.</p> \u000A \u000A <p>Added book <v>books/defexec/reflexive/reflexive.lisp</v> to illustrate \u000A reflexive functions.</p> \u000A \u000A <p>ACL2 now generate scripts that invoke the saved image with <v>exec</v>. \u000A (Previously this was only done for GCL and CLISP.)  The benefit of this change \u000A can be to avoid the lingering of ACL2 processes after enclosing processes have \u000A exited.  Thanks to Peter Dillinger for pointing out this issue.</p> \u000A \u000A <p>ACL2 has a better implementation of <v>(</v><tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt><v>)</v> (hence of \u000A synonyms <v>(</v><tt><see topic=\"ACL2____QUIT\">quit</see></tt><v>)</v> and <v>(</v><tt><see topic=\"ACL2____EXIT\">exit</see></tt><v>)</v>).  As a result, \u000A you should now be able to exit ACL2 and Lisp from within the ACL2 \u000A read-eval-print loop with any of the above; formerly, this was not supported \u000A for some Lisp implementations, and was slow in OpenMCL.  Thanks to SBCL \u000A developer Harald Hanche-Olsen for useful advice.</p> \u000A \u000A <p>Fixed a bug in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) that was causing hard \u000A errors when evaluating calls of <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt>, or of macros expanding to \u000A such calls.</p> \u000A \u000A <p>Fixed a few Makefile dependencies, necessary only for parallel `make'.</p> \u000A \u000A <p>A new book, <v>misc/defpun-exec-domain-example.lisp</v>, provides an example \u000A showing how partial functions which return a unique value for arguments in a \u000A specified domain can be efficiently executed with ACL2.  Execution is achieved \u000A using the <tt><see topic=\"ACL2____MBE\">mbe</see></tt> construct.  Thanks to Sandip Ray for providing this \u000A example.</p> \u000A \u000A <p>Existing function <tt><see topic=\"ACL2____MOD-EXPT\">mod-expt</see></tt> computes <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> base exp) mod)</v> \u000A with great efficiency in GCL, but not in other Lisps.  Now, the book \u000A <v>arithmetic-3/floor-mod/mod-expt-fast.lisp</v> defines a function \u000A <v>mod-expt-fast</v> that should provide significantly improved performance for \u000A such expressions in other Lisps as well, though still probably not as fast as \u000A when using <v>mod-expt</v> in GCL.  Thanks to Warren Hunt, with contributions \u000A from Robert Krug, for providing this book,</p> \u000A \u000A <p>Modified macro <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> to print of an error message before \u000A entering a break, and to cause a hard error if the underlying Lisp cannot \u000A handle it (formerly, a raw Lisp break would occur).  Thanks to Bob Boyer for \u000A bringing these issues to our attention.</p> \u000A \u000A <p>The book <v>books/misc/defpun.lisp</v>, as well as other books related to the \u000A <v>defpun</v> macro, has been modified to avoid namespace collisions by \u000A prefixing function symbol names with <v>&quot;DEFPUN-&quot;</v>; for example <v>base</v> \u000A has been replaced by <v>defpun-base</v>.  Thanks to Dave Greve for providing a \u000A first version of this update to <v>defpun.lisp</v>.</p> \u000A \u000A <p>A theory, <v>base</v>, in <v>books/arithmetic-3/bind-free/top.lisp</v>, has \u000A been renamed <v>arithmetic-3-bind-free-base</v>, to avoid potential name \u000A conflicts.</p> \u000A \u000A <p>Fixed <v>books/arithmetic-3/bind-free/banner.lisp</v> to print (as before) a \u000A message about how to turn on non-linear arithmetic, by modifying the call of \u000A <v>value-triple</v> to use <v>:on-skip-proofs t</v>.  Thanks to Robert Krug for \u000A bringing this issue to our attention.</p> \u000A \u000A <p>Modified <v>books/Makefile-subdirs</v> and <v>books/Makefile-psubdirs</v> so \u000A that they can be used with <v>books/Makefile-generic</v>.  Thus, one can set \u000A things up so that `make' can be used to certify books both in the current \u000A directory and subdirectories, for example as follows.</p> \u000A \u000A <code>ACL2 = ../../saved_acl2\u000A\u000Aarith-top: top all\u000Aall: top\u000A\u000ADIRS = pass1 bind-free floor-mod\u000Ainclude ../Makefile-subdirs\u000Ainclude ../Makefile-generic\u000A\u000Atop.cert: top.lisp\u000Atop.cert: bind-free/top.cert\u000Atop.cert: floor-mod/floor-mod.cert\u000Atop.cert: floor-mod/mod-expt-fast.cert</code> \u000A \u000A <p>An experimental extension of ACL2 is under development by Bob Boyer and \u000A Warren Hunt to support function memoization, hash conses, and an applicative \u000A version of hash tables.  The default build of ACL2 does not include this \u000A extension, other than simple logic definitions of functions in new source file \u000A <v>hons.lisp</v>.  Future versions of ACL2 may fully incorporate this \u000A experimental extension.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> event macro has been modified primarily by adding \u000A a new constraint as follows, where <v>evl</v> is the evaluator.  The idea is \u000A that for the evaluation of a function call, one may replace each argument by \u000A the quotation of its evaluation and then also replace the alist environment \u000A with <v>nil</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">DEFTHMD</see> UNHIDE-evl-CONSTRAINT-0\u000A  (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A                (<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> A ''NIL)))\u000A                (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'QUOTE)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (evl X A)\u000A                  (evl (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)\u000A                             (<see topic=\"ACL2____KWOTE-LST\">KWOTE-LST</see> (UNHIDE-evl-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) A)))\u000A                       NIL))))</code> \u000A \u000A <p>In order to support this change, there is another change: an evaluator maps \u000A <v>nil</v> to <v>nil</v> (note <v>(<see topic=\"COMMON-LISP____AND\">AND</see> X (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> X A)))</v> in place of \u000A <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> X A))</v> below).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">DEFTHM</see> UNHIDE-evl-CONSTRAINT-1\u000A  (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> X)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (UNHIDE-evl X A)\u000A                  (<see topic=\"COMMON-LISP____AND\">AND</see> X (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQ\">ASSOC-EQ</see> X A))))))</code> \u000A \u000A <p>With the new <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt>, Dave Greve has been able to do a proof \u000A about beta reduction that seemed impossible before (see \u000A <v>books/misc/beta-reduce.lisp</v>).  Thanks to Dave for suggesting an initial \u000A version of this change.</p> \u000A \u000A <p>Explicit compilation is now avoided for OpenMCL, resulting in fewer files \u000A to manage (no more files resulting from compilation) and, according to some \u000A tests, slightly faster run times.  See <see topic=\"ACL2____COMPILATION\">compilation</see>.  Thanks to Bob \u000A Boyer and Warren Hunt for suggesting this possibility.</p> \u000A \u000A <p>Now, the <v>term-evisc-tuple</v> (see <see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see>) is overridden by \u000A state global <v>user-term-evisc-tuple</v> in all cases.  Formerly, this was only \u000A the case when <v>term-evisc-tuple</v> was called with non-<v>nil</v> first \u000A argument.</p> \u000A \u000A <p>Symbols with the dot (<v>.</v>) character are generally no longer printed \u000A with vertical bars.  For example, before this change:</p> \u000A \u000A <code>ACL2 !&gt;'ab.c\u000A|AB.C|\u000AACL2 !&gt;</code> \u000A \u000A <p>After this change:</p> \u000A \u000A <code>ACL2 !&gt;'ab.c\u000AAB.C\u000AACL2 !&gt;</code> \u000A \u000A <p>Thanks to Jared Davis for suggesting this improvement.</p> \u000A \u000A <p>Fixed bugs in <v>guard</v> verification for theorems.  The following examples \u000A illustrate these bugs.  If either theorem's body is executed in raw Lisp there \u000A is likely to be a hard Lisp error, even though <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> was \u000A supposed to ensure against that behavior.</p> \u000A \u000A <code>; Example: Verify-guards failed to check that all functions in the theorem\u000A; had already been guard-verified.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-car (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> my-car-compute-example (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (my-car 3) (my-car 3)))\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> my-car-compute-example)\u000A\u000A; Example: Verify guards of a theorem whose body uses state improperly.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad-state-handler\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (state-p state)\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> state) (<see topic=\"COMMON-LISP____CAR\">car</see> state))\u000A    t)\u000A  :rule-classes nil)\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> bad-state-handler)</code> \u000A \u000A <p>See <see topic=\"ACL2____GCL\">gcl</see> for an example, developed with Warren Hunt and Serita \u000A Nelesen, that shows how to get fast fixnum (small integer) arithmetic \u000A operations in GCL.</p> \u000A \u000A <p>Fixnum declarations are now realized as <v>(signed-byte 30)</v> and \u000A <v>(unsigned-byte 29)</v> instead of what was generally <v>(signed-byte 29)</v> \u000A and <v>(unsigned-byte 28)</v>.  MCL users may thus find better performance if \u000A they switch to OpenMCL.  Note that some definitions have changed \u000A correspondingly; for example, <tt><see topic=\"ACL2____ZPF\">zpf</see></tt> now <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt>s its argument to \u000A be of type <v>(unsigned-byte 29)</v> instead of <v>(unsigned-byte 28)</v>.  A few \u000A <see topic=\"ACL2____BOOKS\">books</see> may thus need to be adjusted; for example, changes were made to \u000A books in <v>books/data-structures/memories/</v>.</p> \u000A \u000A <p>ACL2's rewriter now avoids removing certain true hypotheses and false \u000A conclusions.  When a hypothesis rewrites to true or a conclusion rewrites to \u000A false, ACL2 formerly removed that hypothesis or conclusion.  Now, it only does \u000A such removal when the hypothesis or conclusion is either a call of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> or an equivalence relation (see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>), or else is \u000A sufficiently trivial (roughly, either redundant with another hypothesis or \u000A conclusion or else trivially true without considering the rest of the goal). \u000A A specific example may be found in source file <v>simplify.lisp</v>; search for \u000A ``; But we need to do even more work''.  Thanks to Robert Krug for providing \u000A the idea for this improvement and its initial implementation.  As is common \u000A with heuristic changes, you may find it necessary on occasion to rename some \u000A subgoals in your <see topic=\"ACL2____HINTS\">hints</see>.  And in this case, you might also find it \u000A necessary on rare occasions to add <v>:do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see>)</v> <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>A new function, <v>mfc-relieve-hyp</v>, allows (for example) for more \u000A powerful <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypotheses, by providing an interface to the \u000A rewriter's routine for relieving hypotheses.  See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  Thanks to Robert Krug for providing the idea for \u000A this feature and its initial implementation.</p> \u000A \u000A <p>Two improvements have been made to non-linear arithmetic (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>).  One allows for deducing strict inequality (<v>&lt;</v>) \u000A for the result of certain polynomial multiplications, where previously only \u000A non-strict inequality (<v>&lt;=</v>) was deduced.  A second allows the use of the \u000A product of two polynomials when at least one of them is known to be rational. \u000A We had previously restricted the use of the product to the case where both \u000A were known to be rational.  Thanks to Robert Krug for these improvements.</p> \u000A \u000A <p>(OpenMCL and Allegro CL only) Fixed ACL2's redefinitions of raw Lisp \u000A <v>trace</v> and <v>untrace</v> in OpenMCL and Allegro CL so that when given no \u000A arguments, they return the list of traced functions.  For <v>trace</v>, this is \u000A an ANSI spec requirement.  Note that <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> and <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt> \u000A continue to return <v>nil</v> in the ACL2 loop.</p> \u000A \u000A <p>Fixed a bug that was allowing the symbol <v>&amp;whole</v> to appear in other \u000A than the first argument position for a <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> event, in violation of \u000A the Common Lisp spec (and leading to potentially surprising behavior).  Thanks \u000A to Peter Dillinger for bringing this bug to our attention.</p> \u000A \u000A <p>It had been illegal to use <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> under some calls of <tt><see topic=\"ACL2____LD\">ld</see></tt>.  This has been fixed.  Thanks to Jared Davis for bringing this issue to \u000A our attention with a simple example, in essence:</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> '((<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-defun (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args) `(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ,@args)))\u000A      (my-defun f (x) x)))</code> \u000A \u000A <p>ACL2 no longer prohibits certain <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> forms when including \u000A uncertified <see topic=\"ACL2____BOOKS\">books</see>.  Thanks to Peter Dillinger for first bringing this \u000A issue to our attention.</p> \u000A \u000A <p>Hard errors arose when using <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> stack display commands, \u000A in particular <v>:path</v> and <v>:frame</v>, from inside the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.  This has been fixed.</p> \u000A \u000A <p>Fixed a bug that could cause functions that call system built-ins \u000A <v>f-put-global</v>, <v>f-get-global</v>, or <v>f-boundp-global</v> to cause a raw \u000A Lisp error even when proving theorems.  Thanks to Peter Dillinger, for \u000A reporting such a failure for the form <v>(<see topic=\"ACL2____THM\">thm</see> (w '(1 2 3)))</v>.</p> \u000A \u000A <p>Renamed the formal parameters of function <v>set-equal</v> in distributed \u000A book <v>books/arithmetic-3/bind-free/normalize.lisp</v> so that more distributed \u000A books can be included together in the same session.  In particular books \u000A <v>books/data-structures/set-theory</v> and \u000A <v>books/arithmetic-3/extra/top-ext</v> can now be included together.  Thanks to \u000A Carl Eastlund for bringing this problem to our attention and to Robert Krug \u000A for suggesting the formals renaming as a fix.</p> \u000A \u000A <p>Metafunctions must now be executable.  See <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <p>New utilities allow for user-defined simplifiers at the goal level, both \u000A verified and unverified (``trusted''), where the latter can even be defined by \u000A programs outside ACL2.  See <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>, which points to a new \u000A directory <v>books/clause-processors/</v> that contains examples of these new \u000A utilities, including for example a system (``SULFA'') contributed by Erik \u000A Reeber that implements a decision procedure (thanks, Erik).  Also see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see> for the new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command \u000A <v>clause-processor</v> (or for short, <v>cl-proc</v>).</p> \u000A \u000A <p>The rewriter has been tweaked to run faster in some cases involving very \u000A large terms.  Thanks to Eric Smith and Jared Davis for providing a helpful \u000A example that helped us locate the source of this inefficiency.</p> \u000A \u000A <p>Added <v>books/make-event/defspec.lisp</v>.  This book shows how one can \u000A mimic certain limited forms of higher-order statements in ACL2 by use of \u000A macros, <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, and <tt><see topic=\"ACL2____TABLE\">table</see></tt> events.  Thanks to Sandip Ray \u000A for his contribution.</p> \u000A \u000A <p>A new release of the RTL library, <v>books/rtl/rel7/</v>, replaces the \u000A previous version, <v>books/rtl/rel6/</v>.  Thanks to Hanbing Liu and David \u000A Russinoff for providing this new version.</p> \u000A \u000A <p>We thank David Russinoff for providing a proof of the law of quadratic \u000A reciprocity.  See <v>books/quadratic-reciprocity/Readme.lsp</v>.</p> \u000A \u000A <p>Eliminated a slow array warning (see <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>) that could \u000A occur when exiting a <see topic=\"ACL2____WORMHOLE\">wormhole</see> after executing an <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> \u000A event in that wormhole.  Thanks to Dave Greve for bringing this problem to our \u000A attention.</p> \u000A \u000A <p>A new accessor, <v>(mfc-rdepth mfc)</v>, provides a new field, the remaining \u000A rewrite stack depth, which has been added to metafunction context structures; \u000A see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  Thanks to Eric Smith for suggesting this \u000A addition.</p> \u000A \u000A <p>The algorithms were modified for collecting up rule names and other \u000A information used in proofs, into so-called ``tag-trees''.  Tag-trees are now \u000A free of duplicate objects, and this change can dramatically speed up some \u000A proofs that involve many different rules.  Thanks to Eric Smith for doing some \u000A profiling that brought this issue to our attention, and for reporting that \u000A this change reduced proof time on an example by about 47% (from 3681.46 \u000A reported seconds down to 1954.69).</p> \u000A \u000A <p>All legal <v>xargs</v> keywords may now be used in <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see>.  In particular, this is the case for <v>:normalize</v>.</p> \u000A \u000A <p>(SBCL and CMUCL only) Fixed a problem with stobj array resizing functions \u000A that was causing a hard error in ACL2 images built on SBCL or CMUCL.</p> \u000A \u000A <p>A new <see topic=\"ACL2____TABLE\">table</see>, <tt><see topic=\"ACL2____EVISC-TABLE\">evisc-table</see></tt>, allows you to introduce print \u000A abbreviations, for example for large constants.  Moreover, a new reader macro \u000A &mdash; <v>#,</v> &mdash; makes it convenient to reference constants even inside \u000A a quote.  See <see topic=\"ACL2____EVISC-TABLE\">evisc-table</see>.  Thanks to Bob Boyer and Warren Hunt for \u000A useful discussions leading to this feature.</p> \u000A \u000A <p>The macros in <v>books/misc/expander.lisp</v> now have a new keyword \u000A argument, <v>:simplify-hyps-p</v>.  The default behavior is as before, but now \u000A case splitting from hypothesis simplification can be avoided.  For details, \u000A evaluate <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/expander&quot; :dir :system)</v> and then <v>:doc!\u000A defthm?</v> and <v>:doc! symsym</v>.  Thanks to Daron Vroon for sending a question \u000A that prompted this additional functionality.</p> \u000A \u000A <p>ACL2 failed to apply <v>:</v><tt><see topic=\"ACL2____RESTRICT\">restrict</see></tt> hints to rules of class \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, except for the simplest sorts (see <see topic=\"ACL2____SIMPLE\">simple</see>). \u000A This has been fixed.  Thanks to Jared Davis for pointing out this bug by \u000A sending a small example.</p> \u000A \u000A <p>Added a new <v>:msg</v> argument to <v>assert-event</v>; see <see topic=\"ACL2____ASSERT-EVENT\">assert-event</see>.  The implementation of <v>value-triple</v> has been modified to \u000A support this change.</p> \u000A \u000A <p>Fixed a bug in macro <v>io?</v> that now allows the <v>commentp</v> argument to \u000A be <v>t</v>.  This provides a way other than <v>cw</v> to print without modifying \u000A state, for example as follows.  (Warning: Certain errors may leave you in a \u000A <see topic=\"ACL2____WORMHOLE\">wormhole</see>, in which case use <v>:a!</v> to abort.)</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____PROG2_42\">prog2$</see> (io? event t state\u000A                    ()\u000A                    (<see topic=\"ACL2____FMS\">fms</see> &quot;Howdy~%&quot; nil *standard-co* state nil))\u000A               (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A\u000AHowdy\u000A7\u000AACL2 !&gt;:set-inhibit-output-lst (<see topic=\"ACL2____PROOF-TREE\">proof-tree</see> event)\u000A (<see topic=\"ACL2____PROOF-TREE\">PROOF-TREE</see> EVENT)\u000AACL2 !&gt;(<see topic=\"ACL2____PROG2_42\">prog2$</see> (io? event t state\u000A                    ()\u000A                    (<see topic=\"ACL2____FMS\">fms</see> &quot;Howdy~%&quot; nil *standard-co* state nil))\u000A               (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A7\u000AACL2 !&gt;</code> \u000A \u000A <p>ACL2 now disallows calls of <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> inside function bodies, just as \u000A it already disallowed such calls of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, since in both cases the \u000A Common Lisp meaning differs from the ACL2 meaning.</p> \u000A \u000A <p>Redefinition of system functions now always requires an active trust tag \u000A (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  This restriction was intended before, but there was a \u000A hole that allowed a second redefinition without an active trust tag.  Thanks \u000A to Peter Dillinger for pointing out this bug.</p> \u000A \u000A <p><tt><see topic=\"ACL2____VERIFY-TERMINATION\">Verify-termination</see></tt> has been disabled for a few more built-in \u000A functions that are in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.  (If you are curious about \u000A which ones they are, evaluate <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'built-in-program-mode-fns\u000A state)</v>.)  [Note added for Version_3.4: This state global has been changed to \u000A 'program-fns-with-raw-code.]  Moreover, such functions now will execute only \u000A their raw Lisp code, so for example they cannot be called during \u000A macroexpansion.  Thanks to Peter Dillinger and Sandip Ray for useful \u000A discussions on details of the implementation of this restriction.</p> \u000A \u000A <p>New untouchable state global variables, <v>temp-touchable-vars</v> and \u000A <v>temp-touchable-fns</v>, can control the enforcement of untouchability.  See \u000A <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>.  Thanks to Peter Dillinger for suggesting these \u000A features.</p> \u000A \u000A <p>The ``TTAG NOTE'' string was being printed by <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events \u000A whenever an active trust tag was already in effect (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), even \u000A if the encapsulate event contained no <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> event.  This has been \u000A fixed.  Thanks to Peter Dillinger for a query leading to this fix.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> that could leave the user in raw-mode (see \u000A <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>).  This could occur when certifying a book with a \u000A <v>compile-flg</v> value of <v>t</v> (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>), when that book \u000A contained a <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> event setting raw-mode to <v>t</v> without setting \u000A raw-mode back to <v>nil</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> t) ...)</code>"],

 "ACL2____NOTE-3-0-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the \u000A documentation has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Fixed soundness bugs in the handling of primitive function <tt><see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see></tt>, and improved its documentation.  (The executable counterpart \u000A returned an incorrect default value, and the axiom \u000A <v>symbol-package-name-pkg-witness-name</v> needed <v>pkg-name</v> to be other \u000A than <v>&quot;&quot;</v> in order to avoid the default value of \"ACL2\".)  As fallout, \u000A a new built-in <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rule, \u000A <v>symbol-package-name-of-symbol-is-not-empty-string</v>, now asserts that the \u000A <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of a symbol is never <v>&quot;&quot;</v>.  Thanks to Mike \u000A Gordon for bringing these soundness bugs to our attention by attempting to \u000A prove translations of ACL2 axioms in HOL4.</p> \u000A \u000A <p>Fixed a soundness bug in linear arithmetic, due to incomplete tracking of \u000A forced assumptions while deriving inequalities.  Thanks to Robert Krug for \u000A providing a fix and a proof of <v>nil</v> before the fix.</p> \u000A \u000A <p>Fixed a soundness bug in the redundancy criterion for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> events, \u000A which has been modified; see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.  This bug is illustrated \u000A below.  Thanks to Peter Dillinger and Jared Davis for contributions to an \u000A email thread that led us to discover this bug.  The solution is that for a \u000A definition to be redundant with an earlier definition, ACL2 no longer ignores \u000A <v>:</v><tt><see topic=\"ACL2____MEASURE\">measure</see></tt> <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> except when skipping proofs (e.g., during \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>).  However, a new ``measure'', <v>(:? v1 ... vk)</v>, is \u000A supported, for specifying a measured subset of the set of formals, i.e., a set \u000A of formals that serves as the set of parameters for some valid measure.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> y)))\u000A          (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> y))\u000A              (foo (<see topic=\"COMMON-LISP____CONS\">cons</see> x x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))\u000A            y)))\u000A\u000A; So the following is redundant -- but it guesses a measure\u000A; of (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> x), which isn't right!\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> y))\u000A       (foo (<see topic=\"COMMON-LISP____CONS\">cons</see> x x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))\u000A     y)))\u000A\u000A; end of encapsulate\u000A\u000A; Now we prove a non-theorem by exploiting the bug above,\u000A; erroneously replacing formal y by a constant in the induction\u000A; scheme hinted below.  (This should not be allowed, as y should be\u000A; labeled as a measured formal.)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A  (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :induct (foo x '(3)))))\u000A\u000A; It's easy to get a contradiction by instantiating the\u000A; non-theorem just above.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use ((:instance bad (x '(7)))))))</code> \u000A \u000A <p>Fixed a bug in <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s \u000A <v>show-rewrites</v> (<v>sr</v>) command that was causing a Lisp break.  For \u000A <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>, also improved the display of unifying substitutions, \u000A modified output to take binding hypotheses <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term)</v> into account \u000A properly, and arranged for inclusion of <see topic=\"ACL2____META\">meta</see> rules that modify the \u000A given term.  Thanks to Eric Smith for bringing these issues to our \u000A attention.</p> \u000A \u000A <p>Introduced new utilities for undoing <see topic=\"ACL2____COMMAND\">command</see>s, <v>:</v><tt><see topic=\"ACL2____UBU\">ubu</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____UBU_12\">ubu!</see></tt>, which are analogous to <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> and <v>:</v><tt><see topic=\"ACL2____UBT_12\">ubt!</see></tt>  (respectively) except that they only undo back up to, but not \u000A including, the indicated command.</p> \u000A \u000A <p>Fixed a performance bug, pointed out by Eric Smith, that was negating \u000A efforts made for the preceding release to avoid computation for disabled \u000A warnings.</p> \u000A \u000A <p>Added <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> and <v>value-triple</v> to <v>*acl2-exports*</v>.  Thanks \u000A to Bob Boyer and Erik Reeber (respectively) for bringing these issues to our \u000A attention.</p> \u000A \u000A <p>Improved the automatic proclaiming of function types for GCL and OpenMCL, \u000A specifically to use an output format consistent with the Common Lisp spec. \u000A Thanks to Bob Boyer for bringing this issue to our attention.</p> \u000A \u000A <p>Added <v>books/misc/transfinite.lisp</v>, which deals with transfinite \u000A induction in ACL2.  Thanks to Eric Smith for contributing this book.</p> \u000A \u000A <p>Added <v>books/misc/process-book-readme.lisp</v> to the distribution.  Thanks \u000A to Sandip Ray for pointing out its omission.</p> \u000A \u000A <p>Added contributions <v>books/concurrent-programs/bakery/</v> and \u000A <v>books/concurrent-programs/german-protocol/</v>.  These contributions can be \u000A used as tutorials, especially by new ACL2 users, for learning how to model \u000A concurrent protocols in ACL2 and the steps involved in reasoning about their \u000A correctness.  Thanks to Sandip Ray for these contributions.  See the \u000A <v>Readme.lsp</v> files in these directories.</p> \u000A \u000A <p>Theory invariants may now involve the variable <v>ENS</v> instead of the \u000A variable <v>THEORY</v>.  The practical effect of this change is that any \u000A expression of the form <v>(<see topic=\"ACL2____MEMBER-EQUAL\">MEMBER-EQUAL</see> rune THEORY)</v> occurring in a <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> expression should be replaced by <v>(<see topic=\"ACL2____ACTIVE-RUNEP\">ACTIVE-RUNEP</see> rune)</v>. \u000A See <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>.  Thanks to Eric Smith and Dave Greve for pointing \u000A out an inefficiency in the handling of theory invariants that led to this \u000A change, which can speed up their handling by orders of magnitude on large \u000A examples, and to Eric for testing this change and pointing out problems with \u000A an early implementation of it.</p> \u000A \u000A <p>Theory invariants (see <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>) are no longer checked on \u000A theories defined by <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  After all, one can \u000A define a theory with <v>deftheory</v> that is not intended to be used as the \u000A current theory, but rather is intended to be combined with other <see topic=\"ACL2____THEORIES\">theories</see> (see <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see>).  Thanks to Eric Smith for bringing \u000A this issue to our attention.</p> \u000A \u000A <p><tt><see topic=\"ACL2____THEORY-INVARIANT\">Theory-invariant</see></tt> errors had been reported with very little detail \u000A when warnings were inhibited.  This problem has been fixed; thanks to Eric \u000A Smith for bringing it to our attention and providing an example.  We have also \u000A improved the handling of redundancy for <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> now has a new optional keyword, <v>rewrite</v>, \u000A that can be used to change the form of the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule \u000A generated when the quantifier is <tt><see topic=\"ACL2____FORALL\">forall</see></tt>.  Thanks to Eric Smith and \u000A Sandip Ray for useful discussions on this topic.  We have also slightly \u000A modified the <see topic=\"ACL2____HINTS\">hints</see> for the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event underneath a \u000A <v>defun-sk</v> in order to make the proof more reliably efficient.</p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt>, allows setting of a barrier before \u000A which undoing is illegal.  An argument to this macro allows the barrier to be \u000A made permanent; otherwise, it can be removed with <v>:</v><tt><see topic=\"ACL2____UBT-PREHISTORY\">ubt-prehistory</see></tt>.  Thanks to Peter Dillinger for useful conversations leading \u000A to the addition of <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt>.</p> \u000A \u000A <p>A new query, <v>(</v><tt><see topic=\"ACL2____WORMHOLE-P\">wormhole-p</see></tt><v> </v><tt><see topic=\"ACL2____STATE\">state</see></tt><v>)</v>, allows \u000A users to determine whether or not they are in a <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>.  Thanks to \u000A Peter Dillinger for providing this utility.</p> \u000A \u000A <p><v>Value-triple</v> no longer evaluates its form during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, \u000A and in raw Lisp its calls trivially macroexpand to <v>nil</v>, without any \u000A consideration of its argument.  This change avoids errors and warnings when \u000A <see topic=\"ACL2____STOBJ\">stobj</see> names occur in the argument.</p> \u000A \u000A <p>We fixed what could be considered a soundness hole that could occur by \u000A exploiting redefinition in a particular way.  Thanks to Peter Dillinger for \u000A raising a question that led to discovery of this hole.</p> \u000A \u000A <p>A bug has been fixed in handling of illegal <see topic=\"ACL2____THEORY\">theory</see> expressions. \u000A Thanks to Eric Smith, who reported this problem and provided the example \u000A <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> '((:definition natp) (:rewrite doesntexist)))</v> to show how a \u000A hard error could occur.</p> \u000A \u000A <p>Improved error reporting by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> when the certification \u000A <see topic=\"ACL2____WORLD\">world</see> contains inadmissible forms.</p> \u000A \u000A <p>Modified <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> to add two new keyword arguments.  There is now \u000A a <v>:doc</v> keyword argument; previously, an optional documentation string \u000A (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>) was to be placed just before the body, without a \u000A keyword.  There is also a <v>:strengthen</v> argument that strengthens the axiom \u000A added, which allows for the definition of ``fixing'' functions for equivalence \u000A relations that choose canonical representatives of equivalence classes.  See \u000A <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.  Thanks for Dave Greve for useful discussions that led us to \u000A this <v>:strengthen</v> enhancement.</p> \u000A \u000A <p>Added <v>books/misc/bash.lisp</v>, which provides utilities for simplifying a \u000A goal into a list of subgoals (as documented at the top of that file).  Thanks \u000A to Dave Greve for requesting this utility and suggesting refinements to its \u000A functionality, which have been incorporated.</p> \u000A \u000A <p>(For Emacs users only) The command <v>meta-x new-shell</v> provided by file \u000A <v>emacs/emacs-acl2.el</v> now puts you in shell-mode, which for example \u000A supports directory tracking.  Thanks to Jared Davis for suggesting this \u000A change.</p> \u000A \u000A <p>Fixed some mishandling of <see topic=\"ACL2____STOBJ\">stobj</see>s by <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> \u000A expansion.</p> \u000A \u000A <p>Introduced a new event, <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt>, that introduces a ``trust tag'' \u000A (``ttag'') allowing for extensions of ACL2 and for the use of generally unsafe \u000A ACL2 constructs.  Thanks to Peter Dillinger, Sandip Ray, and Erik Reeber for \u000A useful discussions on <v>defttag</v> and the following related items.</p> \u000A \u000A <blockquote> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt>, can be used to give users access \u000A to system functions and data structures.  We also fixed a bug in <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt>; and, it no longer is a no-op in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode. \u000A Thanks to Peter Dillinger for proposing <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt> and \u000A suggesting that it and <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt> be functional in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode.</p> \u000A \u000A <p>Raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) no longer disables <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>. \u000A However, <tt><see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see></tt> is now disallowed unless there is an active ttag \u000A (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  If you want to execute <v>(</v><tt><see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see></tt><v>\u000A t)</v> and there is no active ttag, consider executing <v>(</v><tt><see topic=\"ACL2____SET-RAW-MODE-ON_12\">set-raw-mode-on!</see></tt><v>)</v> instead.</p> \u000A \u000A <p>Redefinition of system functions is disallowed unless there is an active \u000A ttag.  However, <tt><see topic=\"ACL2____REDEF_12\">redef!</see></tt> now introduces <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :redef!)</v> in order \u000A to allow redefinition of system functions.</p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, is a legal embedded event form that can go in \u000A <see topic=\"ACL2____BOOKS\">books</see> and both <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> forms (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), and is similar to <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> except that it allows \u000A arbitrary forms.  Thus, a <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> form is potentially dangerous and can \u000A only be evaluated if there is an active ttag.</p> \u000A \u000A <p>See <see topic=\"ACL2____TTAGS-SEEN\">ttags-seen</see> for information about how to find the ttags known in \u000A the current ACL2 <see topic=\"ACL2____WORLD\">world</see>, and for related caveats.</p> \u000A \u000A <p>A new book created with Peter Dillinger, <v>books/misc/hacker.lisp</v> (added \u000A after Version_3.3: now <v>books/hacking/hacker.lisp</v>), uses <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> to \u000A define utiliities <v>with-raw-mode</v> and <v>with-redef-allowed</v>, which \u000A respectively allow raw Lisp evaluation and redefinition to take place within a \u000A certifiable book (!).</p></blockquote> \u000A \u000A <p>Macro <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> is no longer allowed in function bodies because \u000A it does not have (and has never had) any effect in raw Lisp.  See <see topic=\"ACL2____WITH-OUTPUT\">with-output</see> for a workaround.</p> \u000A \u000A <p>Fixed a bug in redundancy of <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> in raw Lisp, which caused an \u000A error when certifying a book with a redundant <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event whose \u000A <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> had already been modified.  Here is an example:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A(update-fld 3 st)\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;foo&quot; 1) ; where foo.lisp contains (<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)</code> \u000A \u000A <p>New books illustrating <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> have been contributed in \u000A directory <v>books/make-event/</v>: <v>dotimes.lisp</v> (David Rager), \u000A <v>stobj-test.lisp</v>, and <v>logical-tangent.lisp</v> (Peter Dillinger).</p> \u000A \u000A <p>Modified <v>print-object$</v> (see <see topic=\"ACL2____IO\">io</see>) so that it no longer prints an \u000A extra space at the end.</p> \u000A \u000A <p>Replaced the ``draconian restriction to avoid capture'' that had prevented \u000A some <v>:functional-instance</v> <see topic=\"ACL2____HINTS\">hints</see> from being legal.  The \u000A corresponding check now only requires that no variable free in the functional \u000A substitution is captured by a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> (or <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt>) binding.  See <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>.</p> \u000A \u000A <p>Added new extended metafunction, <v>mfc-rw+</v>, which is equivalent to \u000A <v>mfc-rw</v> except that it takes an alist argument, which may be useful for \u000A efficiency.  See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  Thanks to Robert Krug for \u000A suggesting this more efficient variant of <v>mfc-rw</v>.</p> \u000A \u000A <p>Added support for the <v>ignorable</v> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form.</p> \u000A \u000A <p>We now cause an error on a call of <v>open-input-channel</v> (see <see topic=\"ACL2____IO\">io</see>) \u000A with an argument string whose first character is the <v>|</v> character.  Thanks \u000A to Bob Boyer for providing an example (several months ago) showing the danger \u000A of such calls, namely that the following command would log you out and kill \u000A all of your processes when running on top of GCL in Linux:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> &quot;|kill -9 -1&quot; :object state)</v></p> \u000A \u000A <p>Restricted the use of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> to contexts in which it can be \u000A tracked properly, under legal <see topic=\"ACL2____EVENTS\">events</see> (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>). \u000A Thanks to Peter Dillinger for bringing an example to our attention that led to \u000A this fix.</p> \u000A \u000A <p>Fixed a bug that was avoiding <see topic=\"ACL2____GUARD\">guard</see>-checking for the functions \u000A <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt>.  Thanks to David Rager for bringing \u000A this bug to our attention.</p> \u000A \u000A <p>Added an error message when a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> \u000A event fails, to clarify whether failure is for the <see topic=\"ACL2____MEASURE\">measure</see> conjecture \u000A or for the <see topic=\"ACL2____GUARD\">guard</see> conjecture.  Thanks to David Rager for requesting \u000A clarification for such failures.</p> \u000A \u000A <p>Fixed a bug in reporting of <see topic=\"ACL2____GUARD\">guard</see> violations (hard Lisp error) when \u000A certain macros (for example, <tt><see topic=\"COMMON-LISP____COND\">cond</see></tt>) are used in the <see topic=\"ACL2____GUARD\">guard</see>. \u000A Thanks to Jared Davis for bringing this problem to our attention and providing \u000A assistance with the solution, in particular by providing a helpful \u000A example.</p> \u000A \u000A <p>Grant Passmore has contributed a resolution/paramodulation prover written \u000A in ACL2, in directory <v>books/deduction/passmore/</v>.  Thanks, Grant.</p> \u000A \u000A <p>Improved the error message when illegal theories are encountered.</p> \u000A \u000A <p>Improved the suppression of output for <v>inhibit-output</v> arguments of \u000A routines in the book <v>books/misc/expander.lisp</v>.  Thanks to Qiang Zhang for \u000A pointing out the possibility for improvement here.</p> \u000A \u000A <p>Added a new directory <v>books/arithmetic-3/extra/</v> that extends \u000A <v>books/arithmetic-3</v> with additional rules, contributed by Alex Spiridonov \u000A with guidance from Robert Krug.  WARNING: This directory is under development. \u000A It may undergo large changes in future releases, so please consider it \u000A experimental and subject to change.  Feedback is welcomed.</p> \u000A \u000A <p>As part of the work mentioned just above, Robert Krug and Alex Spiridonov \u000A contributed improvements to <v>books/arithmetic-3/</v>:</p> \u000A \u000A <blockquote> \u000A \u000A <p>o A new rule <v>|(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> x) (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> x n)))|</v> in \u000A <v>bind-free/collect.lisp</v>, which is important for reducing <v>collect-*</v> \u000A expressions though it slowed down one proof (see comment above this rule in \u000A <v>bind-free/collect.lisp</v>).</p> \u000A \u000A <p>o Slight improvements of rules <v>integerp-mod</v> and <v>rationalp-mod</v> in \u000A <v>floor-mod/floor-mod.lisp</v>.</p> \u000A \u000A <p>o To avoid conflict with <v>books/rtl/rel6/arithmetic/</v>, renamed rule \u000A <v>mod-minus</v> to <v>mod-neg</v> in <v>floor-mod/floor-mod.lisp</v>, and renamed \u000A <v>integerp-+-reduce-leading-constant</v> to \u000A <v>integerp-+-reduce-leading-rational-constant</v> in \u000A <v>bind-free/integerp.lisp</v>.</p></blockquote> \u000A \u000A <p>(GCL on Windows only) Made a low-level change to avoid multiplying stacks \u000A for GCL on Windows, since GCL 2.6.6 broke while doing this.</p> \u000A \u000A <p>Fixed bugs in linear arithmetic (rarely evidenced, it seems) involving \u000A using <v>&lt;</v> to compare complex rational constants.  Thanks to Robert Krug for \u000A helping with the fixes.</p> \u000A \u000A <p>Added a new event, <tt><see topic=\"ACL2____ASSERT-EVENT\">assert-event</see></tt>, for checking that forms evaluate \u000A to non-<v>nil</v> values.  Thanks to Peter Dillinger for suggesting and \u000A collaborating on this addition.</p>"],

 "ACL2____NOTE-2-9-4":[["Release-notes"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____NOTE-2-9-1\">note-2-9-1</see>, see <see topic=\"ACL2____NOTE-2-9-2\">note-2-9-2</see>, and see <see topic=\"ACL2____NOTE-2-9-3\">note-2-9-3</see> for other changes since the last non-incremental \u000A release (Version_2.9).</p> \u000A \u000A <p>A soundness bug has been fixed that was due to inadequate checking of \u000A <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules in the presence of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see>. \u000A Specifically, a <v>local</v> <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> event is insufficient for \u000A supporting a <v>:meta</v> rule (an example is shown in source function \u000A <v>chk-acceptable-rules</v>).  Thanks to Dave Greve and Jared Davis for bringing \u000A this bug to our attention, by sending a proof of <v>nil</v> that exploited this \u000A bug.  The fix is to check legality of <v>:meta</v> rules even when skipping \u000A proofs during an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event or the second pass of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event.</p> \u000A \u000A <p>Fixed problem with parallel make for workshop books by adding a dependency \u000A line to <v>books/workshops/2003/Makefile</v>.</p> \u000A \u000A <p>Default hints (see <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>) no longer prevent the use of \u000A <v>:INSTRUCTIONS</v> (see <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>).  Thanks to Jared Davis for \u000A pointing out this problem.</p> \u000A \u000A <p>New functions <tt><see topic=\"ACL2____REMOVE-EQ\">remove-eq</see></tt> and <tt><see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see></tt> have been defined, \u000A in analogy to <tt><see topic=\"COMMON-LISP____REMOVE\">remove</see></tt>.  These two symbols have also been added to \u000A <v>*acl2-exports*</v>.  Thanks to David Rager for pointing out that \u000A <v>remove-equal</v> was missing.  Moreover, the definitions of <v>delete1-eq</v> \u000A and <v>delete1-equal</v> have been eliminated.  Function <v>remove1-eq</v>, now in \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode in source file <v>axioms.lisp</v>, serves in place of \u000A <v>delete1-eq</v>, with corresponding new function definitions for <tt><see topic=\"ACL2____REMOVE1\">remove1</see></tt> and <tt><see topic=\"ACL2____REMOVE1-EQUAL\">remove1-equal</see></tt>.</p> \u000A \u000A <p>The symbol <tt><see topic=\"ACL2____ASSERT_42\">assert$</see></tt> has been added to <v>*acl2-exports*</v>.  Thanks \u000A to Jared Davis for the suggestion.</p> \u000A \u000A <p>Added SBCL support.  Thanks to Juho Snellman for significant assistance \u000A with the port.  Thanks to Bob Boyer for suggesting the use of feature \u000A <v>:acl2-mv-as-values</v> with SBCL, which can allow thread-level parallelism in \u000A the underlying lisp; we have done so when feature <v>:sb-thread</v> is \u000A present.</p> \u000A \u000A <p>We have continued to incorporate suggestions for wording improvements in \u000A documentation and error messages.  Thanks to all who send these suggestions, \u000A especially to Eric Smith, who has suggested the vast majority of them.</p> \u000A \u000A <p>Made a small improvement to errors and warnings caused on behalf of <see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see>, to indicate when an event of the same name already \u000A exists.</p> \u000A \u000A <p>Fixed a bug in <v>books/misc/rtl-untranslate.lisp</v> that was causing a \u000A guard violation when adding a new entry for an existing key.</p> \u000A \u000A <p>Fixed a bug in translation to internal form that caused <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> \u000A and <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> to have difficulties handling ignored variables in \u000A <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> forms.  Thanks to Sandip Ray to bringing this issue to our \u000A attention with a helpful example.</p> \u000A \u000A <p>The form <v>(push :acl2-mv-as-values *features*)</v> has been added in source \u000A file <v>acl2-init.lisp</v> for SBCL and OpenMCL only, in order to support \u000A parallel execution (looking to the future...).</p> \u000A \u000A <p>Default-hints (see <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>) were being ignored inside the \u000A <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>, but no longer.  Thanks to John Erickson for bringing \u000A this problem to our attention and providing a simple example of it.</p> \u000A \u000A <p>Modified the <v>TAGS</v> <v>&quot;make&quot;</v> target (specifically, function \u000A <v>make-tags</v>) so that it is gracefully skipped if the <v>etags</v> program is \u000A not found.  Thanks to David Rager for pointing out this issue.</p> \u000A \u000A <p>Sandip Ray has re-worked the supporting materials for his ACL2 Workshop \u000A 2003 talk (originally with John Matthews and Mark Tuttle), to run in a few \u000A minutes.  The result is in <v>workshops/2003/ray-matthews-tuttle/support/</v> \u000A and is included in the full ACL2 regression suite.  Thanks, Sandip.</p> \u000A \u000A <p>Debian releases of ACL2 had created superfluous <v>.cert.final</v> files when \u000A certifying books.  This has been fixed; thanks to Jared Davis for noticing \u000A this problem.</p> \u000A \u000A <p>Jared Davis has pointed out that ``If you add a <v>:backchain-limit-lst 0</v> \u000A to a rewrite rule whose hypotheses are all forced, then ACL2 `really assumes \u000A them' without trying to relieve them right there through rewriting.'' \u000A Relevant documentation has been added for <v>:backchain-limit-lst</v>; see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p>A new version of the rtl library has been included in <v>books/rtl/rel5/</v>. \u000A Thanks to David Russinoff for contributing hand proofs for the new lemmas, and \u000A to Matt Kaufmann for carrying out their mechanization.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> that was failing to set the initial \u000A <v>cbd</v> according to the current directory when starting up ACL2.  Thanks to \u000A Camm Maguire for bringing our attention to this problem.</p> \u000A \u000A <p>Variables introduced during <v>let*</v> abstraction are now in the current \u000A package.  Thanks to Jared Davis for suggesting such a change.  See <see topic=\"ACL2____SET-LET_A2-ABSTRACTIONP\">set-let*-abstractionp</see>.</p> \u000A \u000A <p>It is now allowed for two definitions to be considered the same from the \u000A standpoint of redundancy (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) when one specifies a \u000A <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> of <v>t</v> and the other has no explicit <v>:guard</v> \u000A (hence, the guard is implicitly <v>t</v>).  Thanks to Jared Davis for bringing \u000A this issue to our attention.</p> \u000A \u000A <p>(For users of <v>emacs/emacs-acl2.el</v>) There have been a few enhancements \u000A to distributed file <v>emacs/emacs-acl2. el</v> (skip this paragraph if you \u000A don't use that file):</p> \u000A \u000A <blockquote><p>o <v>Control-t q</v> continues to compare windows ignoring \u000A whitespace, but now, a prefix argument can be given to control case is also \u000A ignored (ignore case if positive, else use case).</p> \u000A \u000A <p>o <v>Control-t Control-l</v> has been defined to be similar to <v>Control-t\u000A    l</v>, except that proofs are skipped and output is suppressed.</p> \u000A \u000A <p>o <v>Control-t u</v> has been defined to print, to the shell buffer, a \u000A   <v>:</v><tt><see topic=\"ACL2____UBT_12\">ubt!</see></tt> form for the command containing the cursor.</p> \u000A \u000A <p>o Control-t Control-f buries the current buffer.</p> \u000A \u000A <p>o <v>Meta-x new-shell</v> now puts the new shell buffer in <v>shell-mode</v> \u000A   (thanks to David Rager for noticing this issue).</p></blockquote> \u000A \u000A <p>Linear arithmetic has been modified so that we do not generate the equality \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> term1 term2)</v> from the pair of inequalities <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> term1 term2)</v> \u000A and <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> term2 term1)</v> in the case that we would have to <tt><see topic=\"ACL2____FORCE\">force</see></tt> both \u000A <v>term1</v> and <v>term2</v> to be <tt><see topic=\"ACL2____ACL2-NUMBERP\">ACL2-numberp</see></tt>s.  Thanks to Dave Greve \u000A for providing a motivating example and to Robert Krug for providing a fix.</p> \u000A \u000A <p>The event <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt> had not been allowed inside <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> forms.  This was an oversight, and has been \u000A fixed.</p> \u000A \u000A <p>Sandip Ray has contributed a new library of books to support proofs of \u000A partial and total correctness of sequential programs based on assertional \u000A reasoning, in <v>books/symbolic/</v>.  This work is based on the paper \u000A J. Matthews, J S. Moore, S. Ray, and D. Vroon, ``A Symbolic Simulation \u000A Approach to Assertional Program Verification,'' currently in draft form.  In \u000A particular, the books include the macro <v>defsimulate</v>, which automatically \u000A transforms inductive assertion proofs of correctness of sequential programs to \u000A the corresponding interpreter proofs.  See the <v>README</v> in that \u000A directory.</p> \u000A \u000A <p>We have changed the implementation of <v>:dir :system</v> for <tt><see topic=\"ACL2____LD\">ld</see></tt> and \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  This change will not affect you if you build an ACL2 \u000A executable in the normal manner, leaving in place the <v>books/</v> subdirectory \u000A of the source directory; nor will it affect you if you download a GCL Debian \u000A binary distribution.  The change is that if environment variable \u000A <v>ACL2_SYSTEM_BOOKS</v> is set, then it specifies the distributed books \u000A directory, i.e., the directory determined by <v>:dir :system</v>.  You may find \u000A it convenient to set this variable in your ACL2 script file (typically, \u000A <v>saved_acl2</v>).  If it is set when you build ACL2, the generated script for \u000A running ACL2 will begin by setting <v>ACL2_SYSTEM_BOOKS</v> to that value. \u000A Thanks to various people who have discussed this issue, in particular Jared \u000A Davis who sent an email suggesting consideration of the use of an environment \u000A variable, and to Eric Smith who helped construct this paragraph.  (Note that \u000A this use of <v>ACL2_SYSTEM_BOOKS</v> replaces the use of <v>ACL2_SRC_BOOKS</v> \u000A described previously; see <see topic=\"ACL2____NOTE-2-9-1\">note-2-9-1</see>.)</p> \u000A \u000A <p>ACL2 now automatically deletes files <v>TMP*.lisp</v> created during the \u000A build process and created by <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt>.  If you want these to be \u000A saved, evaluate <v>(<see topic=\"ACL2____ASSIGN\">assign</see> keep-tmp-files t)</v> in the ACL2 loop or in raw \u000A Lisp.  The <v>clean</v> target for the standard `make' process for certifying \u000A books (see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>) will however delete all files \u000A <v>TMP*.*</v>.</p> \u000A \u000A <p>The <v>TMP</v> files discussed just above now generally include the current \u000A process ID in their names, e.g., <v>TMP@16388@1.lisp</v> instead of \u000A <v>TMP1.lisp</v>.  Thanks to Bob Boyer for suggesting this measure, which will \u000A reduce the possibility that two different processes will attempt to access the \u000A same temporary file.</p> \u000A \u000A <p>Now, <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> will print the information formerly printed by \u000A <v>:pe!</v>, slightly enhanced to work for logical names that are strings, not \u000A just symbols.  Thanks to Warren Hunt for leading us to this change by \u000A suggesting that <v>:pe nth</v> print the definition of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt>.</p> \u000A \u000A <p>We eliminated spurious warnings that could occur in raw mode in OpenMCL or \u000A CMUCL when <see topic=\"ACL2____STOBJ\">stobj</see>s are present.  We thank Juho Snellman for pointing out \u000A the relevant bug and appropriate fix.</p> \u000A \u000A <p><v>Mfc-rw</v> now takes a third argument that can specify an arbitrary known \u000A equivalence relation; see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.  Thanks to Dave Greve \u000A for discussions suggesting this improvement.</p> \u000A \u000A <p>A small modification to a symbol-reading function allows documentation \u000A string processing on Windows systems that use CR/LF for line breaks.  Thanks \u000A to William Cook for bringing this issue to our attention.</p> \u000A \u000A <p>The documentation has been improved on how to control the printing of ACL2 \u000A terms.  See <see topic=\"ACL2____USER-DEFINED-FUNCTIONS-TABLE\">user-defined-functions-table</see>.  Thanks to Sandip Ray for \u000A asking a question that led to the example presented there.</p> \u000A \u000A <p>We fixed an inefficiency that could cause an <tt><see topic=\"ACL2____LD\">ld</see></tt> command to seem to \u000A hang at its conclusion.  Thanks to Sandip Ray for pointing out this \u000A problem.</p> \u000A \u000A <p>We checked that ACL2 runs under LispWorks 4.4.5, and have inhibited \u000A redefinition warnings.</p> \u000A \u000A <p>Two changes have been made on behalf of congruence-based reasoning.  Thanks \u000A to Dave Greve for examples and discussions that have led to these changes, and \u000A to Eric Smith and Vernon Austel, who also sent relevant examples.</p> \u000A \u000A <blockquote><p>o When a call of the new unary function <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt> \u000A is encountered by the rewriter, its argument will be rewritten twice.  This \u000A solves certain problems encountered in congruence-based rewriting.  Warnings \u000A for <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules will suggest when \u000A calls of <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt> on variables in hypotheses are likely to be a \u000A good idea.  See <see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see>.</p> \u000A \u000A <p>o Hypotheses of the form <v>(equiv var (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> term))</v>, where \u000A <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation and <v>var</v> is a free \u000A variable (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), will bind <v>var</v> to the result of \u000A rewriting <v>term</v> twice.  Previously, hypotheses of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var\u000A term)</v> would bind a free variable <v>var</v>, but the call had to be of \u000A <v>equal</v> rather than of an arbitrary known equivalence \u000A relation.</p></blockquote> \u000A \u000A <p>The following improvements were made in support of ACL2 on top of \u000A OpenMCL.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o New versions of OpenMCL that do not have <v>:mcl</v> in Lisp variable \u000A <v>*features*</v> will now work with ACL2.  Thanks to David Rager for bringing \u000A this issue to our attention.</p> \u000A \u000A <p>o Added support for OpenMCL 1.0 for 64-bit DarwinPPC/MacOS X, thanks to \u000A Robert Krug.</p> \u000A \u000A <p>o Fixed tracing in OpenMCL so that the level is reset to 1 even if there \u000A has been an abort.</p> \u000A \u000A <p>o Added support in OpenMCL for <v>WET</v>.</p> \u000A \u000A <p>o Incorporated suggestions from Gary Byers for printing the ``Welcome to \u000A OpenMCL'' prompt before initially entering the ACL2 loop and, and for setting \u000A useful environment variable <v>CCL_DEFAULT_DIRECTORY</v> in the ACL2 \u000A script.</p></blockquote> \u000A \u000A <p>Fixed a long-standing bug in forward-chaining, where variable-free \u000A hypotheses were being evaluated even if the <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see>s of \u000A their function symbols had been disabled.  Thanks to Eric Smith for bringing \u000A this bug to our attention by sending a simple example that exhibited the \u000A problem.</p> \u000A \u000A <p>Improved reporting by the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> utility upon failure to \u000A relieve hypotheses in the presence of free variables, so that information is \u000A shown about the attempting bindings.  See <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES-REWRITE\">free-variables-examples-rewrite</see>.  Thanks to Eric Smith for requesting this \u000A improvement.  Also improved the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> loop so that terms, in \u000A particular from unifying substitutions, are printed without hiding subterms by \u000A default.  The user can control such hiding (``evisceration''); see :DOC \u000A <v>set-brr-term-evisc-tuple</v>.</p> \u000A \u000A <p>A new directory <v>books/defexec/</v> contains books that illustrate the use \u000A of <tt><see topic=\"ACL2____MBE\">mbe</see></tt> and <tt><see topic=\"ACL2____DEFEXEC\">defexec</see></tt>.  Thanks to the contributors of those books \u000A (see the <v>README</v> file in that directory).</p> \u000A \u000A <p>The directories <v>books/rtl/rel2</v> and <v>books/rtl/rel3</v> are no longer \u000A distributed.  They are still available by email request.  (Subdirectory \u000A <v>rel1/</v> supports some of the optional <v>workshop/</v> books, so it is still \u000A distributed.)</p> \u000A \u000A <p>Added book <v>books/misc/sticky-disable.lisp</v> to manage <see topic=\"ACL2____THEORIES\">theories</see> \u000A that might otherwise be modified adversely by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  Thanks to \u000A Ray Richards for a query that led to our development of this tool.</p> \u000A \u000A <p>The commands <v>(<see topic=\"ACL2____EXIT\">exit</see>)</v> and <v>(<see topic=\"ACL2____QUIT\">quit</see>)</v> may now be used to quit ACL2 and \u000A Lisp completely; in fact they macroexpand to calls of the same function as \u000A does <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> (which is now a macro).  Thanks to Jared Davis for \u000A suggesting the new aliases.  (OpenMCL-only comment:) These all work for \u000A OpenMCL even inside the ACL2 loop.</p> \u000A \u000A <p>The macro <v>wet</v> now hides structure by default on large expressions. \u000A However, a new optional argument controls this behavior, for example avoiding \u000A such hiding if that argument is <v>nil</v>.  Thanks to Hanbing Liu for pointing \u000A out that <v>wet</v> was not helpful for very large terms.</p> \u000A \u000A <p>We have fixed a bug in the forward-chaining mechanism that, very rarely, \u000A could cause a proof to be aborted needlessly with an obscure error message. \u000A Thanks to Jared Davis for sending us an example that evoked this bug.</p> \u000A \u000A <p>Fixed a bug that was causing proof output on behalf of \u000A <v>:functional-instance</v> to be confusing, because it failed to mention that \u000A the number of constraints may be different from the number of subgoals \u000A generated.  Thanks to Robert Krug for pointing out this confusing output.  The \u000A fix also causes the reporting of rules used when silently simplifying the \u000A constraints to create the subgoals.</p> \u000A \u000A <p>Fixed a bug in handling of leading <v>./</v> in pathnames, as in: \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;./foo&quot;)</v>.  Thanks to Jared Davis for bringing this bug to \u000A our attention.</p> \u000A \u000A <p>Made a small fix for handling of free variables of <v>:</v><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules, which had erroneously acted as though a hypothesis \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term)</v> can bind the variable <v>var</v>.</p> \u000A \u000A <p>A small change has been made for <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules for \u000A hypotheses of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term)</v>, where <v>var</v> is a free \u000A variable and no variable of <v>term</v> is free (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).  As \u000A with <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules, we now bind \u000A <v>var</v> to <v>term</v> even if <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> u term)</v> happens to be known in the \u000A current context for some term <v>u</v>.  Also as with <v>:rewrite</v> and \u000A <v>:linear</v> rules, similar handling is given to hypotheses <v>(equiv var\u000A (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> term))</v> where <v>equiv</v> is a known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A relation.</p> \u000A \u000A <p>We changed the handling of free variables in hypotheses of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules being handled by the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>rewrite</v> \u000A (<v>r</v>) command, in complete analogy to the change described just above for \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.</p> \u000A \u000A <p>The installation instructions have been updated for obtaining GCL on a \u000A Macintosh.  Thanks to Robert Krug for supplying this information and to Camm \u000A Maguire for simplifying the process by eliminating the <v>gettext</v> \u000A dependency.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____COMP\">comp</see></tt> is now an event, so it may be placed in <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p>Previously, a <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> call could fail because of file permission \u000A issues, yet ACL2 (and the underlying Lisp) would quit anyhow.  This has been \u000A fixed.  Thanks to Peter Dillinger for bringing this problem to our \u000A attention.</p> \u000A \u000A <p>Jared Davis, with assistance from David Rager, has updated his ordered sets \u000A library, <v>books/finite-set-theory/osets/</v>.  See file <v>CHANGES.html</v> in \u000A that directory.</p> \u000A \u000A <p>A new function, <tt><see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see></tt>, has been provided for the rare \u000A user who encounters memory limitations.  See <see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see>.</p>"],

 "ACL2____NOTE-2-9-3-PPR-CHANGE":[["Note-2-9-3"],"ACL2 Sources","ACL2","<p>We have improved pretty-printing in ACL2 Version_2.9.3 to handle \u000A keywords a little differently.  To see a discussion of the basics of this \u000A change, see <see topic=\"ACL2____NOTE-2-9-3\">note-2-9-3</see>.  In this note we describe it in considerable \u000A detail.</p> \u000A \u000A <p>Those who wish to understand the ACL2 pretty-printer's implementation can \u000A now find considerably more comments on it in the source code.  In this note, \u000A we do not focus on the implementation.  Rather, we motivate the change and \u000A show how the improved prettyprinter performs.</p> \u000A \u000A <p>Why do we want better keyword handling?  Imagine a macro that builds a new \u000A state from an old state by changing the values in the affected fields, leaving \u000A everything else unchanged.  One could write</p> \u000A \u000A <code>(modify th s :key1 val1 :key2 val2 :key3 val3)</code> \u000A \u000A <p>where the three keys identify fields in the state.</p> \u000A \u000A <p>To make it easier to read new concrete states, we may have a function that \u000A prints them ``relative'' to a given base state, expressing the new state as a \u000A modification of the given base state.  So we may find ourselves prettyprinting \u000A modify forms like that above.</p> \u000A \u000A <p>The previous prettyprinter will sometimes print the form above as \u000A follows.</p> \u000A \u000A <code>(modify th s :key1\u000A        val1\u000A        :key2 val2 :key3 val3)</code> \u000A \u000A <p>This can be unpleasant to read, because of the way <v>:key1</v> and <v>val1</v> \u000A are separated.  Here is an example of the old prettyprinter and the new one, \u000A both printing an expression from the ACL2 source code in a width of 40:</p> \u000A \u000A <code>Old:\u000A(ADD-TO-TAG-TREE\u000A 'ASSUMPTION\u000A (<see topic=\"ACL2____MAKE\">MAKE</see>\u000A  ASSUMPTION :TYPE-ALIST TYPE-ALIST\u000A  :TERM TERM :REWRITTENP REWRITTENP\u000A  :IMMEDIATEP IMMEDIATEP :ASSUMNOTES\u000A  (<see topic=\"COMMON-LISP____LIST\">LIST</see>\u000A   (<see topic=\"ACL2____MAKE\">MAKE</see>\u000A        ASSUMNOTE :CL-ID\u000A        NIL :RUNE RUNE :TARGET TARGET)))\u000A TTREE)\u000A\u000ANew:\u000A(ADD-TO-TAG-TREE\u000A     'ASSUMPTION\u000A     (<see topic=\"ACL2____MAKE\">MAKE</see> ASSUMPTION\u000A           :TYPE-ALIST TYPE-ALIST\u000A           :TERM TERM\u000A           :REWRITTENP REWRITTENP\u000A           :IMMEDIATEP IMMEDIATEP\u000A           :ASSUMNOTES\u000A           (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"ACL2____MAKE\">MAKE</see> ASSUMNOTE\u000A                       :CL-ID NIL\u000A                       :RUNE RUNE\u000A                       :TARGET TARGET)))\u000A     TTREE)</code> \u000A \u000A <p>Basically the change we made forces the prettyprinter to print each \u000A <v>:key</v> on a new line unless they all fit on a single line.  So we would now \u000A get either</p> \u000A \u000A <code>(modify th s :key1 val1 :key2 :val2 :key3 val3)</code> \u000A \u000A <p>or</p> \u000A \u000A <code>(modify th s\u000A        :key1 val1\u000A        :key2 val2\u000A        :key3 val3)</code> \u000A \u000A <p>Furthermore, we fixed it so that if <v>val1</v> (say) is a big s-expression \u000A we may still print it on the same line as its key.  The old prettyprinter \u000A enforced the rule that if you wanted to print <v>(foo a b)</v> and <v>b</v> gets \u000A broken up into several lines, then it has to start on a new line.  Thus, we'd \u000A never print</p> \u000A \u000A <code>(foo a (bbb\u000A        (mum x)))</code> \u000A \u000A <p>but would print instead</p> \u000A \u000A <code>(foo a\u000A     (bbb\u000A      (mum x)))</code> \u000A \u000A <p>Now, if a is a keyword, we can print the first way.</p> \u000A \u000A <p>So here are some nice examples of prettyprinted keyword forms.  All of \u000A these are printed for a page of width 40.</p> \u000A \u000A <code>&lt;--            40 chars               -&gt;\u000Axxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\u000A\u000A(MODIFY TH S :KEY1 V1 :KEY2 V2)\u000A\u000A(MODIFY TH S :KEY1 V1 :KEY2 V2 :KEY3 V3)\u000A\u000A(MODIFY TH S1                               ; Because of the extra char\u000A        :KEY1 V1                            ; in S1 the flat size exceeds\u000A        :KEY2 V2                            ; 40 and we break it.\u000A        :KEY3 V3)</code> \u000A \u000A <p>The old ppr would have printed this as:</p> \u000A \u000A <code>(MODIFY\u000A     TH S1 :KEY1 V1 :KEY2 V2 :KEY3 V3)</code> \u000A \u000A <p>Returning to new examples:</p> \u000A \u000A <code>&lt;--            40 chars               -&gt;\u000Axxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\u000A\u000A(MODIFY TH S\u000A        :KEY1 (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____IF\">IF</see> X Y Z) AAAA BBBB)\u000A        :KEY2 VAL2\u000A        :KEY3 VAL3)</code> \u000A \u000A <p>Now we extend <v>AAAA</v> and <v>BBBB</v> by one char each, so it would \u000A overflow the right margin if printed as above, and we get:</p> \u000A \u000A <code>&lt;--            40 chars               -&gt;\u000Axxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\u000A\u000A(MODIFY\u000A     TH S\u000A     :KEY1 (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____IF\">IF</see> X Y Z) AAAAX BBBBX)\u000A     :KEY2 VAL2\u000A     :KEY3 VAL3)</code> \u000A \u000A <p>If we make these names even longer we force the value off the line \u000A containing <v>:key1</v>:</p> \u000A \u000A <code>(MODIFY\u000A     TH S\u000A     :KEY1\u000A     (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____IF\">IF</see> X Y Z) AAAAXXXXX BBBBXXXXX)\u000A     :KEY2 VAL2\u000A     :KEY3 VAL3)</code> \u000A \u000A <p>Here are some examples from the ACL2 source code, printed in 40 \u000A characters:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">DEFTHM</see>\u000A ALPHORDER-ANTI-SYMMETRIC\u000A (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A               (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y))\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y))\u000A :HINTS\u000A ((&quot;Goal&quot;\u000A   :IN-THEORY\u000A   (<see topic=\"ACL2____UNION-THEORIES\">UNION-THEORIES</see>\u000A    '(<see topic=\"COMMON-LISP____STRING_C3\">STRING&lt;</see> SYMBOL-&lt;)\u000A    (<see topic=\"ACL2____DISABLE\">DISABLE</see>\u000A       CODE-CHAR-CHAR-CODE-IS-IDENTITY))\u000A   :USE\u000A   ((:INSTANCE SYMBOL-EQUALITY (S1 X)\u000A               (S2 Y))\u000A    (:INSTANCE BAD-ATOM&lt;=-ANTISYMMETRIC)\u000A    (:INSTANCE\u000A         CODE-CHAR-CHAR-CODE-IS-IDENTITY\u000A         (C Y))\u000A    (:INSTANCE\u000A         CODE-CHAR-CHAR-CODE-IS-IDENTITY\u000A         (C X)))))\u000A :RULE-CLASSES\u000A ((:FORWARD-CHAINING\u000A   :COROLLARY\u000A   (<see topic=\"ACL2____IMPLIES\">IMPLIES</see>\u000A      (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A           (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A           (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y)))\u000A      (<see topic=\"ACL2____IFF\">IFF</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y)))\u000A   :HINTS\u000A   ((&quot;Goal&quot;\u000A     :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ALPHORDER))))))</code> \u000A \u000A <p>Here is that same one, printed in a width of 60.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">DEFTHM</see>\u000A ALPHORDER-ANTI-SYMMETRIC\u000A (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A               (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y))\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y))\u000A :HINTS\u000A ((&quot;Goal&quot;\u000A     :IN-THEORY\u000A     (<see topic=\"ACL2____UNION-THEORIES\">UNION-THEORIES</see>\u000A          '(<see topic=\"COMMON-LISP____STRING_C3\">STRING&lt;</see> SYMBOL-&lt;)\u000A          (<see topic=\"ACL2____DISABLE\">DISABLE</see> CODE-CHAR-CHAR-CODE-IS-IDENTITY))\u000A     :USE ((:INSTANCE SYMBOL-EQUALITY (S1 X)\u000A                      (S2 Y))\u000A           (:INSTANCE BAD-ATOM&lt;=-ANTISYMMETRIC)\u000A           (:INSTANCE CODE-CHAR-CHAR-CODE-IS-IDENTITY (C Y))\u000A           (:INSTANCE CODE-CHAR-CHAR-CODE-IS-IDENTITY\u000A                      (C X)))))\u000A :RULE-CLASSES\u000A ((:FORWARD-CHAINING\u000A      :COROLLARY (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A                               (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A                               (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y)))\u000A                          (<see topic=\"ACL2____IFF\">IFF</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y)))\u000A      :HINTS ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ALPHORDER))))))</code> \u000A \u000A <p>Just for comparison, here is the above printed in 60 columns by the old \u000A prettyprinter.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">DEFTHM</see>\u000A ALPHORDER-ANTI-SYMMETRIC\u000A (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A               (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y))\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A               (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X))\u000A          (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y))\u000A :HINTS\u000A ((&quot;Goal&quot; :IN-THEORY\u000A          (<see topic=\"ACL2____UNION-THEORIES\">UNION-THEORIES</see>\u000A               '(<see topic=\"COMMON-LISP____STRING_C3\">STRING&lt;</see> SYMBOL-&lt;)\u000A               (<see topic=\"ACL2____DISABLE\">DISABLE</see> CODE-CHAR-CHAR-CODE-IS-IDENTITY))\u000A          :USE\u000A          ((:INSTANCE SYMBOL-EQUALITY (S1 X)\u000A                      (S2 Y))\u000A           (:INSTANCE BAD-ATOM&lt;=-ANTISYMMETRIC)\u000A           (:INSTANCE CODE-CHAR-CHAR-CODE-IS-IDENTITY (C Y))\u000A           (:INSTANCE CODE-CHAR-CHAR-CODE-IS-IDENTITY\u000A                      (C X)))))\u000A :RULE-CLASSES\u000A ((:FORWARD-CHAINING\u000A       :COROLLARY\u000A       (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> X Y)\u000A                     (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000A                     (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> Y)))\u000A                (<see topic=\"ACL2____IFF\">IFF</see> (<see topic=\"ACL2____ALPHORDER\">ALPHORDER</see> Y X) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X Y)))\u000A       :HINTS\u000A       ((&quot;Goal&quot; :IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ALPHORDER))))))</code> \u000A \u000A <p>Of course, given that you cannot tell for sure whether the keywords you're \u000A seeing are part of a keyword/value parameter list or part of some constant \u000A containing random keywords, the prettyprinter can't solve the problem \u000A perfectly.  We just tried to make it work nicely on well-formed keyword/value \u000A parameter lists.</p> \u000A \u000A <p>For example, here is a form printed by the each prettyprinter.</p> \u000A \u000A <code>Old:\u000A(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see>\u000A X\u000A '(:MONDAY\u000A      :MON :TUESDAY :TUES :WEDNESDAY\u000A      :WED :THURSDAY :THURS :FRIDAY\u000A      :FRI :SATURDAY :SAT :SUNDAY :SUN))\u000A\u000ANew:\u000A(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> X\u000A        '(:MONDAY :MON\u000A                  :TUESDAY :TUES\u000A                  :WEDNESDAY :WED\u000A                  :THURSDAY :THURS\u000A                  :FRIDAY :FRI\u000A                  :SATURDAY :SAT\u000A                  :SUNDAY :SUN))</code> \u000A \u000A <p>The new way is not how one would print it by hand!  But then, neither is \u000A the old way.</p>"],

 "ACL2____NOTE-2-9":[["Release-notes"],"ACL2 Sources","ACL2","<code><b>TABLE OF CONTENTS.</b><br></br> \u000A \u000A============================== \u000ABUG FIXES. \u000ANEW FUNCTIONALITY. \u000ACHANGES IN PROOF ENGINE. \u000AGUARD-RELATED CHANGES. \u000APROOF-CHECKER CHANGES. \u000ASYSTEM-LEVEL CHANGES. \u000ABOOK CHANGES. \u000AMISCELLANEOUS CHANGES. \u000A============================== \u000A</code> \u000A \u000A <p><b>BUG FIXES.</b></p> \u000A \u000A <p>We fixed a soundness bug due to a conflict between user-supplied package \u000A names and internal package names (obtained by prepending a Lisp constant, \u000A <v>*1*-package-prefix*</v>) and user-supplied package names.  For example, the \u000A form <v>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;ACL2_*1*_MYPKG&quot; ())</v> is no longer legal.  Thanks to Robert \u000A Krug for asking a question that led directly to the discovery of this bug.</p> \u000A \u000A <p>We fixed a soundness bug that allows <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions to \u000A call <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions.  The fix furthermore prevents \u000A functions with <see topic=\"ACL2____GUARD\">guard</see>s verified from calling functions with guards not \u000A verified.  We had thought we already prevented all this, but there was a \u000A problem with the interaction of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> definitions and redundancy \u000A checking (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).</p> \u000A \u000A <p>We fixed a soundness bug that could occur when built-in functions were \u000A called during macroexpansion (a hole in so-called ``safe-mode'').</p> \u000A \u000A <p>Fixed a minor bug in system functions <v>genvar1</v> and <v>genvar</v>, where \u000A <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> had been used in place of <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>.  This bug was discovered \u000A during the plugging of a hole in safe-mode, mentioned just above.</p> \u000A \u000A <p>We fixed handling of the <v>:inline</v> keyword for <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>, which \u000A previously could cause raw Lisp errors in OpenMCL and CMU Common Lisp.  Thanks \u000A to John Matthews for bringing this issue to our attention.</p> \u000A \u000A <p>Calls of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> could result in a state for which some \u000A function definitions were not compiled that should have been.  The result \u000A could be performance degradation or even stack overflows.  This situation \u000A could arise in the following two ways.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Inclusion of a book with an absolute pathname that differs from the \u000A absolute pathname at certification time, presumably because of the use of soft \u000A links.  Thanks to Bob Boyer and Warren Hunt for bringing a stack overflow to \u000A our attention that led us to this fix.</p> \u000A \u000A <p>o Large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nests (more than 20 functions) are \u000A executed in a superior book.</p> \u000A \u000A </blockquote> \u000A \u000A <p>We fixed some performance bugs that can increase the speed of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> calls by a factor of close to 2.  Thanks to Eric Smith for \u000A asking if we could speed up <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> processing; we have done so \u000A in the past, but primarily focusing on large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nests \u000A (which have nothing in particular to do with the current improvements).  Also, \u000A thanks to Rob Sumners for a very useful remark early in the process that kept \u000A us from drawing an incorrect conclusion at that point.</p> \u000A \u000A <p>We fixed <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> so that it can be run on a form that returns \u000A multiple values, which it could not do previouslly.  Thanks to Eric Smith for \u000A pointing out this problem.</p> \u000A \u000A <p>Fixed a bug in the Allegro ACL2 trace utility (see <see topic=\"ACL2____TRACE_42\">trace$</see>) that was \u000A causing ``<v>10&gt;</v>'' to be printed as ``<v>9&gt;</v>'', ``<v>11&gt;</v>'' to be printed \u000A as ``<v>10 &gt;</v>'', ``<v>12&gt;</v>'' to be printed as ``<v>11 &gt;</v>'', and so on.</p> \u000A \u000A <p>Fixed a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> bug that was preventing the use of the <v>DV</v> \u000A command (or a numerical command) on <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expressions.  Thanks to Bill \u000A Young for pointing out this problem.</p> \u000A \u000A <p>Fixed a bug in a comment on how to set <v>ACL2_BOOKS_DIR</v> in the makefile. \u000A Thanks to Dave Greve for pointing out this problem.</p> \u000A \u000A <p>Fixed a potential soundness bug in the linear arithmetic routines.  Thanks \u000A to Jared Davis for noticing this problem and to Robert Krug for implementing \u000A the fix.  (Technical details: We had been assuming that polynomials were being \u000A normalized -- see the definition of good-polyp in linear-a.lisp -- but this \u000A assumption was false.)</p> \u000A \u000A <p>When the macro <tt><see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see></tt> is opened twice in succession, it \u000A now automatically closes the first trace output channel before opening \u000A another.</p> \u000A \u000A <p>It is now possible to use `make' to build ACL2 on Windows systems that \u000A support `make'.  Thanks to Pete Manolios and Mike Thomas for pointing out the \u000A problem, to Jared Davis and Mike for helping us to analyze the problem, and to \u000A Jared for testing the fix.</p> \u000A \u000A <p>Fixed a bug in the <see topic=\"ACL2____GUARD\">guard</see> of <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt> that was causing a \u000A needless guard violation.</p> \u000A \u000A <p><b>NEW FUNCTIONALITY.</b></p> \u000A \u000A <p>The new events <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt> and <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see></tt> \u000A allow one to append to or subtract from the current list of default hints. \u000A The event <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt> continues to set the list of default \u000A hints, discarding the previous value of the <tt><see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see></tt>.  Note that \u000A <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt> is still <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or \u000A book in which it occurs, and <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt> has the same property, \u000A although neither is implemented any longer using the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  New events <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS_12\">add-default-hints!</see></tt>, <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS_12\">remove-default-hints!</see></tt>, and <tt><see topic=\"ACL2____SET-DEFAULT-HINTS_12\">set-default-hints!</see></tt> are the same as <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt>, <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see></tt>, and <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt>, respectively, except that the former three events are not \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to their enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.  Thanks to Jared \u000A Davis for suggesting these enhancements.</p> \u000A \u000A <p>OpenMCL's tracing routines have been modified in a similar manner as those \u000A of Allegro CL.  Thanks to Robert Krug for providing this enhancement.</p> \u000A \u000A <p>Guard-checking can now be caused to happen on recursive calls.  See \u000A ``GUARD-RELATED CHANGES'' below for details.</p> \u000A \u000A <p>Advanced users can now inhibit compilation of so-called ``*1* functions'' \u000A with the <v>:comp</v> command; see <see topic=\"ACL2____COMP\">comp</see>.  Thanks to Rob Sumners for \u000A suggesting this enhancement.</p> \u000A \u000A <p>Added new legal argument <v>hard?</v> for the <tt><see topic=\"ACL2____ER\">er</see></tt> macro, which is now \u000A documented.  See <see topic=\"ACL2____ER\">er</see>.  Thanks to Rob Sumners for a discussion leading to \u000A this change.  Also, the three legal first arguments to <tt><see topic=\"ACL2____ER\">er</see></tt> &mdash; \u000A <v>hard</v>, <v>hard?</v>, and <v>soft</v> &mdash; may now be in any package (thanks \u000A to Jared Davis for bringing this issue to our attention).</p> \u000A \u000A <p>We have removed the requirement that for a rule's hypothesis <v>(<see topic=\"ACL2____BIND-FREE\">bind-free</see>\u000A term var-list)</v>, at least one variable must occur free in <v>term</v>.  For \u000A example, the expression <v>(<see topic=\"ACL2____BIND-FREE\">bind-free</see> (bind-divisor a b) (x))</v> was legal \u000A because <v>a</v> and <v>b</v> occur free in <v>(bind-divisor a b)</v>; but \u000A <v>(<see topic=\"ACL2____BIND-FREE\">bind-free</see> (foo (bar)) (x))</v> was not legal.  The latter is no longer \u000A disallowed. (Technical note: this allows <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> to be used to \u000A create explicit substitutions in metafunction hypotheses.)</p> \u000A \u000A <p>The following two enhancements have been implemented for rules of class \u000A <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>.  Thanks to Eric Smith for requesting more control of \u000A reasoning with <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules, which led to these enhancements, and \u000A to him and Robert Krug for helpful discussions.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o It is now possible to control backchaining in rules of class <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> by providing a <v>:backchain-limit-lst</v> argument, as was already \u000A allowed for rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>. \u000A See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.  However, unlike those other two rule classes, the \u000A value for <v>:backchain-limit-lst</v> is prohibited from being a non-empty list; \u000A it must be either <v>nil</v> or a non-negative integer.</p> \u000A \u000A <p>o (For advanced users.) It is now legal for hypothesis metafunctions to \u000A generate, in essense, calls of <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> and <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt>, handled \u000A essentially as they are handled in hypotheses of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> and \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules.  We say ``essentially'' primarily because both \u000A <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> and <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> are actually macros, but hypothesis \u000A metafunctions must generate translated terms (see <see topic=\"ACL2____TERM\">term</see>).  The \u000A enterprising advanced user can call <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt> to see examples of \u000A translated terms corresponding to calls of <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> and <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>A new command <v>:</v><tt><see topic=\"ACL2____TRANS_12\">trans!</see></tt> has been added, which is like \u000A <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt> except that <v>:</v><tt><see topic=\"ACL2____TRANS_12\">trans!</see></tt> ignored issues of \u000A single-threadedness.  See <see topic=\"ACL2____TRANS_12\">trans!</see>.  Thanks to Eric Smith for suggesting \u000A this addition.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____PF\">pf</see></tt> command now works when the argument is the name of a \u000A macro associated with a function by <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.</p> \u000A \u000A <p><b>CHANGES IN PROOF ENGINE.</b></p> \u000A \u000A <p>The simplifier has been changed slightly in order to avoid using <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> facts derived from a literal (essentially, a top-level \u000A hypothesis or conclusion) that has been rewritten.  As a practical matter, \u000A this may mean that the user should not expect forward-chaining to take place \u000A on a term that can be rewritten for any reason (generally function expansion \u000A or application of rewrite rules).  Formerly, the restriction was less severe: \u000A forward-chaining facts from a hypothesis could be used as long as the \u000A hypothesis was not rewritten to <v>t</v>.  Thanks to Art Flatau for providing an \u000A example that led us to make this change; see the comments in source function \u000A <v>rewrite-clause</v> for details.</p> \u000A \u000A <p>The rewriter has been modified to work slightly harder in relieving \u000A hypotheses.  Thanks to Eric Smith for providing an example that inspired the \u000A following, which illustrates the issue.  Suppose we introduce functions \u000A <v>foo</v> and <v>bar</v> with the (non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt>) properties shown below.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((foo *) =&gt; *)\u000A  ((bar *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-holds\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> x\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) t)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> bar-holds-propositionally\u000A   (<see topic=\"ACL2____IFF\">iff</see> (bar x) t)))</code> \u000A \u000A <p>Consider what happens when ACL2's rewriter is used to prove the following \u000A theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (foo (bar y)))</code> \u000A \u000A <p>With ACL2's inside-out rewriting, <v>(bar y)</v> is first considered, but \u000A rewrite rule <v>bar-holds-propositionally</v> does not apply because the context \u000A requires preserving equality, not mere Boolean (<v>iff</v>) equivalence.  Then \u000A the rewriter moves its attention outward and sees the term <v>(foo (bar y))</v>. \u000A It attempts to apply the rule <v>foo-holds</v>, in a context created by binding \u000A its variable <v>x</v> to the term <v>(bar y)</v>.  It then attempts to relieve the \u000A hypothesis <v>x</v> of rule <v>foo-holds</v> in that context.  Before this change, \u000A ACL2 basically assumed that since rewriting was inside out, then <v>(bar y)</v> \u000A had already been rewritten as much as possible, so the rewrite of <v>x</v> in \u000A the aforementioned context (binding <v>x</v> to <v>(bar y)</v>) simply returned \u000A <v>(bar y)</v>, and the attempt to relieve the hypothesis of <v>foo-holds</v> \u000A failed.  The change is essentially for ACL2's rewriter to make a second pass \u000A through the rewriter when the attempt fails to rewrite a variable to <v>t</v>, \u000A this time using the fact that we are in a Boolean context.  (We mention that \u000A source function <v>rewrite-solidify-plus</v> implements this idea, for those who \u000A want to dig deeply into this issue.)  In our example, that means that the \u000A rewriter considers <v>(bar y)</v> in a Boolean context, where it may apply the \u000A rule <v>bar-holds-propositionally</v> to relieve the hypothesis \u000A successfully.</p> \u000A \u000A <p>When <v>(</v><tt><see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see></tt><v> t)</v> has been executed, <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see> can now be applied in some cases for which it \u000A previously was not.  Thanks to Robert Krug for supplying this modification and \u000A to Julien Schmaltz for providing a motivating example.</p> \u000A \u000A <p>We modified the rewriter to avoid certain infinite loops caused by an \u000A interaction of the opening of recursive functions with equality reasoning. \u000A (This change is documented in detail in the source code, in particular \u000A functions <v>rewrite-fncall</v> and <v>fnstack-term-member</v>.)  Thanks to Fares \u000A Fraij for sending us an example that led us to make this change.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of function <tt><see topic=\"ACL2____HIDE\">hide</see></tt> is now \u000A disabled when ACL2 starts up.  This removes an anomoly, for example that</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 1 (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"ACL2____HIDE\">hide</see> 0) a))))</code> \u000A \u000A <p>succeeded while</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 1 (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"ACL2____HIDE\">hide</see> 0) a))) (foo nil)))</code> \u000A \u000A <p>failed.  Now both fail.</p> \u000A \u000A <p>The theory <v>*s-prop-theory*</v> is no longer used by the \u000A <i>proof-checker</i>; it has been replaced by <v>(<see topic=\"ACL2____THEORY\">theory</see> '</v><tt><see topic=\"ACL2____MINIMAL-THEORY\">minimal-theory</see></tt>.  We have left the constant <v>*s-prop-theory*</v> defined in \u000A the source code in support of existing books, however.  This change eliminates \u000A annoying theory warnings printed upon invocation of <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A commands <v>s-prop</v>, <v>sl</v>, and <v>split</v>.</p> \u000A \u000A <p>Terms are now kept in an internal form that avoids calls of primitive \u000A functions (built-ins without explicit definitions; see code for <v>cons-term</v> \u000A for details), in favor of the constants that result from evlaluating those \u000A calls.  So for example, the internal form for <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 1 2)</v> is <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> (1\u000A . 2))</v>.  This change was made at around the same time as changes in support \u000A of <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt>; see above.  One consequence is that the splitting of \u000A goals into cases (technically, source function <v>clausify</v> and even more \u000A technically, source function <v>call-stack</v>) has been modified, which can \u000A cause subgoal numbers to change.</p> \u000A \u000A <p><b>GUARD-RELATED CHANGES.</b></p> \u000A \u000A <p>Guard-checking can now be caused to happen on recursive calls, where this \u000A was formerly not the case for <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions and, \u000A perhaps more important, for <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions whose <see topic=\"ACL2____GUARD\">guard</see>s have not been verified.  Moreover, a warning is printed when ACL2 does \u000A not rule out the exclusion of guard-checking on recursive calls.  See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  Thanks to David Rager for bringing this issue to our \u000A attention, and to Rob Sumners and the Univ. of Texas ACL2 seminar in general \u000A for their feedback.</p> \u000A \u000A <p>Guard violations are reported with less of the offending term hidden. \u000A Thanks to Jared Davis for suggesting that we look at this issue.</p> \u000A \u000A <p><b>PROOF-CHECKER CHANGES.</b></p> \u000A \u000A <p>We fixed the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> so that diving works as you might expect \u000A for a macro call <v>(op a b c)</v> representing <v>(binary-op a (binary-op b\u000A c))</v>.  In the past, if the current term was of the form <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> t1 t2\u000A t3)</v>, then <v>(DV 3)</v> (and <v>3</v>) would dive to <v>t3</v> even though the \u000A corresponding function call is <v>(<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> t1 (<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> t2 t3))</v>. \u000A This is still the case, but now this behavior holds for any macro associated \u000A with a function in <v>binop-table</v> (see <see topic=\"ACL2____ADD-BINOP\">add-binop</see>).  Moreover, users \u000A can now write customized diving functions; see <see topic=\"ACL2____DIVE-INTO-MACROS-TABLE\">dive-into-macros-table</see>, \u000A and also see <v>books/misc/rtl-untranslate.lisp</v> for example calls to <tt><see topic=\"ACL2____ADD-DIVE-INTO-MACRO\">add-dive-into-macro</see></tt>.  Of course, the old behavior can still be obtained using \u000A the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>DIVE</v> command; see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.</p> \u000A \u000A <p>The <v>runes</v> command in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> now shows only the <see topic=\"ACL2____RUNE\">rune</see>s used by the most recent primitive or macro command (as shown by \u000A <v>:comm</v>), unless it is given a non-<v>nil</v> argument.  Also, <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>lemmas-used</v> has been added as, in essence, an \u000A alias for <v>runes</v>.</p> \u000A \u000A <p>(The following two items are also mentioned above under ``BUG FIXES.'')</p> \u000A \u000A <p>Fixed a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> bug that was preventing the use of the <v>DV</v> \u000A command (or a numerical command) on <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expressions.  Thanks to Bill \u000A Young for pointing out this problem.</p> \u000A \u000A <p>The theory <v>*s-prop-theory*</v> is no longer used by the \u000A <i>proof-checker</i>; it has been replaced by <v>(<see topic=\"ACL2____THEORY\">theory</see> '</v><tt><see topic=\"ACL2____MINIMAL-THEORY\">minimal-theory</see></tt>.  We have left the constant <v>*s-prop-theory*</v> defined in \u000A the source code in support of existing books, however.  This change eliminates \u000A annoying theory warnings printed upon invocation of <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A commands <v>s-prop</v>, <v>sl</v>, and <v>split</v>.</p> \u000A \u000A <p><b>SYSTEM-LEVEL CHANGES.</b></p> \u000A \u000A <p>Fixed a problem with building ACL2 on CMUCL in some systems (source \u000A function <v>save-acl2-in-cmulisp</v>).  Thanks to Bill Pase for bringing this to \u000A our attention.</p> \u000A \u000A <p>The installation instructions have been extended to include instructions \u000A for building on GCL in Mac OS X.  Thanks to Jun Sawada and Camm Maguire.</p> \u000A \u000A <p>Initial pre-allocation of space has been updated for GCL to reflect more \u000A current GCL executables (we considered GCL 2.6.1-38).  This can help avoid \u000A running out of memory for large ACL2 sessions.</p> \u000A \u000A <p>The main <v>Makefile</v> has been replaced by <v>GNUmakefile</v>, in order to \u000A enforce the use of GNU `make'.  If you use another `make' program, you'll get \u000A an error message that may help you proceed.</p> \u000A \u000A <p>(GCL only) SGC is no longer turned on for GCL 2.6 sub-versions through \u000A 2.6.3 if <v>si::*optimize-maximum-pages*</v> is bound to <v>T</v>, due to an \u000A apparent issue with their interaction in those sub-versions.  Also, we have \u000A eliminated preallocation for all versions after 2.6.1 because GCL doesn't need \u000A it (comments are in source function <v>save-acl2-in-akcl</v>).  Thanks to Camm \u000A Maguire for excellent GCL help and guidance, and to Camm and Bob Boyer for \u000A useful discussions.</p> \u000A \u000A <p>We have removed support for so-called ``small'' images.  Thus, <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>, <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> and <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt>, <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt>, and \u000A other commands are fully supported in ACL2 saved images.  Because of this and \u000A other changes in the generation of the so-called ``*1*'' logical functions, \u000A related to guards (as described above in -GUARD-RELATED CHANGES'', and related \u000A to the discussion of safe-mode in ``BUG FIXES'' above), image sizes have \u000A increased substantially.</p> \u000A \u000A <p>We no longer <v>time</v> or run ``<v>nice</v>'' the certification of individual \u000A books.  The file <v>books/Makefile-generic</v> had done these by default, and \u000A some individual distributed and workshop book directories had <v>Makefile</v>s \u000A that did so as well.  Thanks to Mike Thomas, who pointed out the lack of \u000A <v>nice</v> on some Windows systems (and we decided on this simple solution). \u000A Overall targets in <v>books/Makefile</v> still <v>time</v> their runs by default, \u000A and the partiular <v>time</v> program is now controlled by a <v>Makefile</v> \u000A variable.</p> \u000A \u000A <p>Failures during <v>make certify-books</v> or <v>make regression</v> now show up \u000A in the log as ``<v>**CERTIFICATION FAILED**</v>'', regardless of the operating \u000A system (as long as it supports `make').  Formerly, one searched for \u000A ``<v>**</v>'' but this did not appear in openMCL runs.</p> \u000A \u000A <p>We have eliminated ``Undefined function'' warnings that could occur in \u000A OpenMCL.</p> \u000A \u000A <p><b>BOOK CHANGES.</b></p> \u000A \u000A <p>Reconciled the definitions of <v>firstn</v> in <v>book/misc/csort.lisp</v>, \u000A <v>books/bdd/bdd-primitives.lisp</v>, \u000A <v>books/ordinals/ordinal-definitions.lisp</v>, and \u000A <v>books/data-structures/list-defuns.lisp</v>.  Thanks to Ray Richards for \u000A bringing this issue to our attention.</p> \u000A \u000A <p>Distributed book <v>books/misc/defpun</v> now can handle <see topic=\"ACL2____STOBJ\">stobj</see>s where \u000A it did not previously.  Thanks to John Matthews for bringing this issue to our \u000A attention.</p> \u000A \u000A <p>The \"make\" variable <v>COMPILE_FLG</v> in file <v>books/Makefile-generic</v> \u000A formerly only had an effect if there was a <v>cert.acl2</v> file present.  That \u000A oversight has been remedied.</p> \u000A \u000A <p>File <v>&quot;books/arithmetic/certify.lsp&quot;</v> was missing a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command for <v>&quot;natp-posp&quot;</v>.  Thanks to John Cowles for \u000A noticing this deficiency and supplying a fix.  (This file is of use to those \u000A who want to certify the <v>&quot;books/arithmetic/&quot;</v> books without using \u000A <v>&quot;make&quot;</v>.)</p> \u000A \u000A <p>A few small changes have been made to <v>&quot;books/rtl/rel4&quot;</v>.</p> \u000A \u000A <p>Small changes were made to books <v>misc/symbol-btree</v> and \u000A <v>misc/rtl-untranslate</v>.  In particular, the definition of \u000A <v>symbol-btreep</v> was strengthened.</p> \u000A \u000A <p>We made a minor fix to <v>books/ordinals/e0-ordinal.lisp</v>, adding \u000A <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> ob+)</v> and hence <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> ocmp)</v> as well.  This \u000A was necessitated by the fix prohibiting functions with guards verified from \u000A calling functions with guards not verified (see also the related discussion \u000A under ``BUG FIXES'' above).</p> \u000A \u000A <p><b>MISCELLANEOUS CHANGES.</b></p> \u000A \u000A <p>Further sped up processing of large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nests \u000A (extending what was done for Version_2.7), perhaps by a factor of two in some \u000A cases.</p> \u000A \u000A <p>As promised in Version_2.5 (see <see topic=\"ACL2____NOTE-2-5\">note-2-5</see>), structured pathnames are \u000A no longer supported.  So for example, the argument to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A must now be a string constant.</p> \u000A \u000A <p>Some documentation has been improved, for <see topic=\"ACL2____STOBJ\">stobj</see>s thanks to \u000A suggestions by John Matthews and much of the rest thanks to feedback from Eric \u000A Smith.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> is now available to users (it has been \u000A taken off the list of so-called ``untouchables'').  Thanks to Jared Davis for \u000A bringing this issue to our attention.</p> \u000A \u000A <p>The documentation for topic <see topic=\"ACL2____USING-COMPUTED-HINTS-7\">using-computed-hints-7</see> has been \u000A improved.  Thanks to Doug Harper and Eric Smith for inspiring this \u000A improvement.</p> \u000A \u000A <p>We added several symbols to <v>*acl2-exports*</v>: <tt><see topic=\"ACL2____CW\">cw</see></tt>, <tt><see topic=\"ACL2____ER\">er</see></tt>, \u000A <tt><see topic=\"ACL2____INTERN_42\">intern$</see></tt>, <tt><see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see></tt>, and <tt><see topic=\"ACL2____SET-DIFFERENCE-EQ\">set-difference-eq</see></tt>.  Thanks to Jared Davis for suggesting most of these.</p> \u000A \u000A <p>Now, a <tt><see topic=\"ACL2____TABLE\">table</see></tt> event that sets the value for a key, <v>(<see topic=\"ACL2____TABLE\">table</see> tbl key\u000A val :put)</v>, is redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) when it does not \u000A change the value associated with an existing key of the table.  In particular, \u000A <tt><see topic=\"ACL2____DEFINE-PC-MACRO\">define-pc-macro</see></tt> is now fully redundant when it does not change an \u000A existing <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> macro-command definition.  Thanks to Bill Young \u000A for bringing the latter issue to our attention.</p> \u000A \u000A <p>The definitions of unused system functions <v>ev-w</v> and <v>ev-w-lst</v> have \u000A been deleted.</p> \u000A \u000A <p>ACL2 now prints a warning if a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event introduces a package \u000A name with lower-case letters, since there is opportunity for later confusion \u000A in that case.  Thanks to Frederic Peschanski for bringing this problem to our \u000A attention and Sandip Ray for encouragement.</p> \u000A \u000A <p>ACL2 now works in Version 19 of CMU Common Lisp.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> has been modified so that for ACL2 built on \u000A Allegro Common Lisp in Unix or Linux, the existing environment is used. \u000A Thanks to Erik Reeber for bringing this issue to our attention.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____DISABLEDP\">disabledp</see></tt> can now be given a macro name that has a \u000A corresponding function; see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.  Also, <tt><see topic=\"ACL2____DISABLEDP\">disabledp</see></tt> now has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v> but causes a hard error on an \u000A inappropriate argument.</p>"],

 "ACL2____NOTE-2-8-PROOFS":[["Note-2-8"],"ACL2 Sources","ACL2","<p>ACL2 now prevents certain rewriting loops; see <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>.</p> \u000A \u000A <p>During the computation of <tt><see topic=\"ACL2____CONSTRAINT\">constraint</see></tt>s for functional instantiation, \u000A <v>(<see topic=\"ACL2____PROG2_42\">prog2$</see> term1 term2)</v> and <v>(<see topic=\"COMMON-LISP____THE\">the</see> type term2)</v> are now treated as \u000A <v>term2</v>.</p> \u000A \u000A <p>A change has been made in heuristics for controlling rewriting during \u000A proofs by induction.  Formerly, during induction proofs, ACL2 suppressed \u000A rewriting of certain ``induction hypothesis'' terms, and forced expansion of \u000A certain ``induction conclusion'' terms, until rewriting had stabilized.  This \u000A meddling with the rewriter is still turned off when rewriting has stabilized, \u000A but it is now turned off earlier once an ancestor has been through the \u000A rewriter and the current goal is free of ``induction conclusion'' terms. \u000A Thanks to Dave Greve and Matt Wilding for providing an example and associated \u000A analysis that led us to look for a heuristic modification.</p> \u000A \u000A <p>A change has been made in the heuristics for handling certain ``weak'' \u000A <see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see> rules when building contexts.  Those who want to \u000A dig deeply into this change are welcome to look at the code following the call \u000A of <v>most-recent-enabled-recog-tuple</v> in the code for function \u000A <v>assume-true-false</v> in the ACL2 sources.</p> \u000A \u000A <p>The handling of free variables in a hypothesis of a <see topic=\"ACL2____REWRITE\">rewrite</see> rule \u000A (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>) has been improved in the case that the hypothesis \u000A is of the form <v>(equiv x y)</v>, where <v>equiv</v> is a known equivalence \u000A relation (see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>).  Previously, if the rewriter was attempting \u000A to rewrite the hypothesis <v>(equiv x y)</v> of a rewrite rule, in a context \u000A where <v>x'</v> is an instance of <v>x</v>, then the rewriter could fail to notice \u000A a term <tt>(equiv x' y')</tt> true in the current context where <v>y'</v> is an \u000A instance of <v>y</v>, in the case that <v>x'</v> precedes <v>y'</v> in the <tt><see topic=\"ACL2____TERM-ORDER\">term-order</see></tt>.  This has been remedied.  This improvement applies regardless of \u000A whether <v>x</v>, <v>y</v>, or (we believe) both are already fully instantiated in \u000A the present context.  Thanks to Joe Hendrix for bringing up an example and to \u000A Vernon Austel for providing another, simple example.</p> \u000A \u000A <p>A very minor change has been made to the rewriter in the case that an \u000A equality appears on the left-hand side of a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule. \u000A Formerly, when such an equality <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v> was commuted to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> y\u000A x)</v> in order for the rule to match the current term, then all equalities on \u000A the instantiated right-hand side of the rule were commuted, except for those \u000A occurring inside another equality.  The instantiated right-hand side is no \u000A longer modified.  It seems very unlikely that this change will cause proofs to \u000A fail, though we cannot completely rule out that possibility.</p> \u000A \u000A <p>We have modified how the ACL2 simplifier handles the application of a \u000A defined function symbol to constant arguments in certain cases, which we now \u000A describe.  As before, ACL2 attempts to simplify such a function application by \u000A evaluation, provided the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of the function \u000A is enabled.  And as before, if that evaluation fails due to a subroutine call \u000A of a constrained function (introduced by <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>), ACL2 may wrap a \u000A call of <v>hide</v> around this function application.  (See <see topic=\"ACL2____HIDE\">hide</see>.)  But \u000A now, ACL2 attempts to apply definitions and rewrite rules in the case that \u000A this evaluation fails, and only if the resulting term is unchanged does ACL2 \u000A wrap <tt><see topic=\"ACL2____HIDE\">hide</see></tt> around this function application.  Thanks to Matt Wilding \u000A for bringing up the idea of this modification.</p> \u000A \u000A <p>The generation of \"Goal\" for recursive (and mutually-recursive) \u000A definitions now uses the subsumption/replacement limitation (default 500). \u000A See <see topic=\"ACL2____CASE-SPLIT-LIMITATIONS\">case-split-limitations</see>.</p> \u000A \u000A <p>Default hints now apply to hints given in definitions, not just theorems. \u000A See <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p>Thanks to Robert Krug for implementing the following two improvements \u000A involving linear arithmetic reasoning: linear arithmetic now uses the \u000A conclusions of <tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules, and <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> now uses a \u000A small amount of linear reasoning when deciding inequalities.</p>"],

 "ACL2____NOTE-2-6-OTHER":[["Note-2-6"],"ACL2 Sources","ACL2","<p>Warning strings are now case-insensitive.  See <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>.</p> \u000A \u000A <p>ACL2 causes a warning when an <see topic=\"ACL2____IN-THEORY\">in-theory</see> hint or event causes a 0-ary \u000A function's definition to be disabled but its <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> to be enabled.</p> \u000A \u000A <p>A minor modification has been made to <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> that can have a \u000A positive impact on performance in Allegro Common Lisp.  (For Lisp hackers: the \u000A stobj name was formerly declared special, and that was disabling Allegro's \u000A tail-merging routing for compilation of some recursive functions using \u000A stobjs.)  The downside is that stobj names can no longer be evaluated in raw \u000A Lisp.  However, raw Lisp is not the right place to be evaluating ACL2 forms \u000A anyhow; see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>.  We thank Rob Sumners for bringing this issue \u000A to our attention.</p> \u000A \u000A <p>Before Version 2.6, there has been the following problem with <tt><see topic=\"ACL2____DEFSTUB\">defstub</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> in the case that the current package is not \u000A the ACL2 package.  If a <see topic=\"ACL2____SIGNATURE\">signature</see> was specified using the symbol \u000A <v>=&gt;</v>, then that symbol had have been imported into the current package from \u000A the ACL2 package when the current package was defined.  There are no longer \u000A any package restrictions on the use of <v>=&gt;</v>.  Thanks to John Cowles for \u000A bringing this problem to our attention.</p> \u000A \u000A <p>Bugs in <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> have been fixed.  <v>Defun-sk</v> forms introducing \u000A functions of no arguments were failing to be admitted, for example: \u000A <v>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> always-p1 () (<see topic=\"ACL2____FORALL\">forall</see> (x) (p1 x)))</v>.  Thanks to John Cowles for \u000A bringing this problem to our attention.  Also, <v>defun-sk</v> failed on an \u000A example in the documentation (see <see topic=\"ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE\">tutorial4-defun-sk-example</see>), as \u000A pointed out by Matyas Sustik; this bug has been fixed as well.</p> \u000A \u000A <p>The trace mechanism has been fixed to handle <see topic=\"ACL2____STOBJ\">stobj</see>s, and to avoid \u000A the printing of so-called <i>enabled structures</i>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____BRR\">brr</see></tt> command <v>:type-alist</v> now produces more readable \u000A output.</p> \u000A \u000A <p>An <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> of an uncertified book no longer loads an \u000A associated compiled file.</p> \u000A \u000A <p>We added a few checks to make sure that the underlying lisp is suitable, \u000A for example checking that the reader is case-insensitive and reads in symbols \u000A with upper-case names where appropriate.</p> \u000A \u000A <p>We now warn when forcing (see <see topic=\"ACL2____FORCE\">force</see>) or immediate force mode (see \u000A <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>) change state between enabled and disabled.  Also \u000A see <see topic=\"ACL2____ENABLE-IMMEDIATE-FORCE-MODEP\">enable-immediate-force-modep</see> and see <see topic=\"ACL2____DISABLE-IMMEDIATE-FORCE-MODEP\">disable-immediate-force-modep</see> for information about these new macros, which \u000A may be used to control immediate force mode.</p> \u000A \u000A <p>We have eliminated the use of a low-level raw Lisp constant, \u000A <v>*most-recent-multiplicity*</v>.  Our test suite saw a speed-up of \u000A approximately 2% as a result for an ACL2 image built on GCL (but no \u000A significant speed-up for an ACL2 image built on Allegro Common Lisp).  We \u000A thank Rob Sumners for suggesting this improvement.</p> \u000A \u000A <p>Fixnum declarations are now realized as <v>(signed-byte 29)</v> instead of \u000A <v>(signed-byte 27)</v>.  We check that the underlying Common Lisp recognizes \u000A objects of type <v>(signed-byte 29)</v> as fixnums, with the exception of CLISP, \u000A which is said to have an efficient bignum implementation.</p> \u000A \u000A <p>A new documentation topic <see topic=\"ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE\">functional-instantiation-example</see> \u000A illustrates functional instantiation.</p> \u000A \u000A <p>A bug has been fixed in the monitoring of runes (see <see topic=\"ACL2____MONITOR\">monitor</see>). \u000A Thanks to Dave Greve for sending an example that clearly showed the \u000A problem.</p> \u000A \u000A <p>A warning is now issued when it is detected that a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule may not be as strong as it appears because it is not \u000A sufficient to prove itself by type reasoning.</p> \u000A \u000A <p>An error is caused for rules of class <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> when the function \u000A symbol <v>IF</v> is among the <v>:trigger-fns</v>.  (<v>IF</v> was ignored anyhow; \u000A the point of this change is to avoid misleading the user.)</p> \u000A \u000A <p>A minor bug has been fixed in <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt>, evident for example if this \u000A command was applied to <v>IF</v>.</p> \u000A \u000A <p>A minor hole in <v>:</v><tt><see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see></tt> did not permit \u000A the acceptance of <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> forms that include constant \u000A function definitions.  This has been fixed.  Thanks to Eric Smith for coming \u000A up with a simple example illustrating the problem.</p> \u000A \u000A <p>The temporary files \"TMP.lisp\" and \"TMP1.lisp\" written out by \u000A <v>:</v><tt><see topic=\"ACL2____COMP\">comp</see></tt> are now written to the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>).</p> \u000A \u000A <p>Previously, the Allegro compiler was not eliminating tail recursion for \u000A executable counterparts of functions, because of the way one of its flags had \u000A been set.  As a result, calls of functions whose guards had not been verified \u000A could run out of stack space when this was not necessary.  This situation has \u000A been fixed.</p> \u000A \u000A <p>Executable counterparts could have slow array accesses.  This has been \u000A fixed (specifically, constants are no longer replaced with their values in the \u000A definitions of executable counterparts).</p> \u000A \u000A <p>Various improvements have been made to the documentation.  Thanks in \u000A particular to Eric Smith for pointing out a numbers of places where fixes were \u000A in order.</p> \u000A \u000A <p>File \"mcl-acl2-startup.lisp\" has been updated, thanks to feedback from \u000A Philippe Georgelin.</p> \u000A \u000A <p>Inefficiencies in GCL fixnum computations were remedied for macros <v>+f</v> \u000A and <v>*f</v>.  Thanks to Rob Sumners for pointing out this issue.</p>"],

 "ACL2____GOAL-SPEC":[["Hints"],"ACL2 Sources","ACL2","<code>Examples:\u000A&quot;Goal&quot;\u000A&quot;goal&quot;\u000A&quot;Subgoal *1/3''&quot;\u000A&quot;subgoal *1/3''&quot;\u000A&quot;[2]Subgoal *1/3''&quot;</code> \u000A \u000A <p>When <see topic=\"ACL2____HINTS\">hints</see> are given to the theorem prover, a goal-spec is provided \u000A to specify the goal to which the <see topic=\"ACL2____HINTS\">hints</see> are to be applied.  The <see topic=\"ACL2____HINTS\">hints</see> provided are carried along innocuously until the named goal arises. \u000A When it arises, the <see topic=\"ACL2____HINTS\">hints</see> are ``activated'' for that goal and its \u000A descendents.</p> \u000A \u000A <p>A legal goal specification may be extracted from the theorem prover's \u000A output.  Certain lines clearly label formulas, as in</p> \u000A \u000A <code>Subgoal *1/3.2'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> ... ...)</code> \u000A \u000A <p>and these lines all give rise to goal specifications.  In general, these \u000A lines all start either with ``Goal'' or ``Subgoal'' or else with those words \u000A preceded by a number in square brackets, as in</p> \u000A \u000A <code>[1]Subgoal *1/3.2'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> ... ...).</code> \u000A \u000A <p>A goal specification may be obtained by deleting any surrounding whitespace \u000A from such a line and embedding the text in string quotation marks.  Thus</p> \u000A \u000A <code>&quot;[1]Subgoal *1/3.2'&quot;</code> \u000A \u000A <p>is the goal specifier for the goal above.</p> \u000A \u000A <p>As noted, a hint is applied to a goal when the hint's goal specification \u000A matches the name ACL2 assigns to the goal.  The matching algorithm is \u000A case-insensitive.  Thus, alternative goal specifications for the goal above \u000A are <v>&quot;[1]subgoal *1/3.2'&quot;</v> and <v>&quot;[1]SUBGOAL *1/3.2'&quot;</v>.  The matching \u000A algorithm does not tolerate non-case discrepancies.  Thus, \u000A <v>&quot;[1]Subgoal*1/3.2'&quot;</v> and <v>&quot; [1]Subgoal *1/3.2'&quot;</v> are \u000A unacceptable.</p> \u000A \u000A <p>Sometimes a formula is given two names, e.g.,</p> \u000A \u000A <code>Subgoal *1/14.2'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> ...\u000A         ...)\u000AName the formula above *1.1.</code> \u000A \u000A <p>It is the first name (the one that starts with ``Goal'' or ``Subgoal'') and \u000A not the second which constitutes a legal goal-spec.  Roughly speaking, when \u000A the system prints the line containing the goal specification, it activates any \u000A <see topic=\"ACL2____HINTS\">hints</see> that are attached to that goal-spec.  Consider the example above. \u000A Suppose <v>Subgoal *1/14.2'</v> could be proved by using a certain lemma \u000A instance.  Then the appropriate entry in the <see topic=\"ACL2____HINTS\">hints</see> would be:</p> \u000A \u000A <code>(&quot;Subgoal *1/14.2'&quot; :use ...)</code> \u000A \u000A <p>This might surprise you because the system appears to do nothing to \u000A <v>*1/14.2'</v> besides push it for a subsequent induction.  But actually \u000A between the time the system printed the goal-spec line and the time it decides \u000A to push the goal, you can think of the system as trying everything it has.  So \u000A a <v>use</v> hint activated when <v>Subgoal *1/14.2'</v> arises is just what you \u000A want.</p> \u000A \u000A <p>But what if you want to give an <v>:induct</v> hint?  By the time induction \u000A is tried, the formula has been given the name <v>*1.1</v>.  Well, this is one \u000A you just have to remember:</p> \u000A \u000A <code>(&quot;Subgoal *1/14.2'&quot; :induct ...).</code> \u000A \u000A <p>When the above hint is activated the <v>:induct</v> directive short-circuits \u000A the rest of the processing and sends immediately the formula into the pool of \u000A goals to prove by induction.  The induct hint is attached to the formula in \u000A the pool and when the time comes to turn our attention to that goal, the \u000A induct advice is followed.</p> \u000A \u000A <p>We conclude by emphasizing a point made above, that a hint is applied to a \u000A goal when the hint's goal specification matches the name ACL2 assigns to the \u000A goal.  If there is no such match, then the hint is ignored.  Consider the \u000A following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A     :hints ((&quot;Subgoal *1/&quot; :in-theory nil)))</code> \u000A \u000A <p>Normally, <v>:in-theory</v> hints are inherited by subgoals (see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>), so you might expect that the empty theory is used \u000A in <v>Subgoal *1/2</v> and <v>Subgoal *1/1</v>.  But in fact, since there is no \u000A subgoal printed that is labeled <v>Subgoal *1/</v>, the above <v>:in-theory</v> \u000A hint is ignored.  The above example is in contrast to the following, where the \u000A hint makes the proof fail, because there really is a <v>Subgoal *1/</v> in the \u000A proof this time.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)))\u000A     :hints ((&quot;Subgoal *1/&quot; :in-theory nil)))</code>"],

 "ACL2____FUTURE-WORK-RELATED-TO-THE-TAU-SYSTEM":[["Introduction-to-the-tau-system"],"ACL2 Sources","ACL2","<p>The tau system is inexpressive compared to modern polymorphic type \u000A systems &mdash; something that may be unavoidable in an untyped first-order \u000A language.  However, we believe its effectiveness could be greatly increased by \u000A the development of some additional tools.  We also believe that most of these \u000A tools could be provided by ACL2 users creating certified community books that \u000A exploit the basic tools already provided.  It is likely the initial attempts \u000A to create such books will show the inadequacy of some of the current \u000A algorithms and data structures in the tau system and might point the way to \u000A improvements.</p> \u000A \u000A <p>As implementors of ACL2 our preference is to support the improvement of the \u000A core algorithms and data structures and provide customizable hooks allowing \u000A users to exploit them by developing effective and convenient interfaces. \u000A However, we want the further elaboration and optimization of the tau system to \u000A be based on user experience not just speculation.</p> \u000A \u000A <p>Some tools we <i>think</i> might help are listed below.  We invite \u000A volunteers and further suggestions.</p> \u000A \u000A <p><i>A tau-centric signature notation</i> for use in function definitions, \u000A exploited by a macro replacing <v>defun</v> so that input-output relationships \u000A phrased in tau terms are proved as <v>:tau-system</v> rules immediately after \u000A functions are introduced:</p> \u000A \u000A <p>We have, for example, experimented with a book defining a macro that allows \u000A the definition of the function <v>ap</v> (append) accompanied by a signature \u000A rule.  Subsequent <v>defsig</v> events can add other signatures in the same \u000A notation.</p> \u000A \u000A <code>(defsig ap (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> * true-listp ==&gt; true-listp) (x y)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (ap (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))\u000A       y))\u000A\u000A(defsig ap (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> * integer-listp ==&gt; integer-listp))</code> \u000A \u000A <p>This experimental book provides succinct syntax for all tau signatures. \u000A For example, that book parses the signature</p> \u000A \u000A <code>(<see topic=\"ACL2____NATP\">NATP</see> (<see topic=\"COMMON-LISP_____F2_D3\">/=</see> 3 5 7) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 16) * TRUE-LISTP ==&gt; BOOLEANP * INTEGER-LISTP * NATP)</code> \u000A \u000A <p>to be the signature of a function with two inputs and three outputs.  The \u000A first input must be a natural number, different from 3, 5, and 7, and less \u000A than 16.  The second input must be a true list.  The first output is a \u000A boolean, the second a list of integers, and the third a natural.</p> \u000A \u000A <p>To express this signature for function <v>fn</v> as a formula requires \u000A significantly more typing by the user:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____NATP\">NATP</see> X)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 3))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 5))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X 7))\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> X 16)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> Y))\u000A         (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____BOOLEANP\">BOOLEANP</see> (<see topic=\"ACL2____MV-NTH\">MV-NTH</see> 0 (FN X Y)))\u000A              (INTEGER-LISP (<see topic=\"ACL2____MV-NTH\">MV-NTH</see> 1 (FN X Y)))\u000A              (<see topic=\"ACL2____NATP\">NATP</see> (<see topic=\"ACL2____MV-NTH\">MV-NTH</see> 2 (FN X Y)))))</code> \u000A \u000A <p>We suspect that the provision of some succinct tau-centric notation (not \u000A necessarily the one above) for signatures at definition-time will mean users \u000A get more benefit from the tau system.</p> \u000A \u000A <p><i>Some tau inference mechanisms</i>: This phrase suggests two different \u000A improvements.  One is to implement a mechanism that adds or completes \u000A signatures for function definitions by exploiting knowledge of commonly used \u000A recursive schemas and the signatures of the subroutines in the body.  For \u000A example, the definition of <v>ap</v> above rather obviously has the \u000A signature</p> \u000A \u000A <code>(<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> * integer-listp ==&gt; integer-listp)</code> \u000A \u000A <p>and many others just based on the two recursive schemas that (a) collect \u000A certain elements from lists and (b) check that all elements have a certain \u000A property.</p> \u000A \u000A <p>The other ``tau inference'' improvement is to implement a mechanism for \u000A inferring relations between user-defined Booleans, perhaps by exploiting \u000A example generation, testing, and knowledge of recursive schemas.  For example, \u000A it is fairly obvious that <tt><see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see></tt> implies <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>. \u000A Making the user state these relations invites omissions that render the tau \u000A system very unpredictable.</p> \u000A \u000A <p><i>A tau assistant</i>: It would be useful to have a way to find out what \u000A tau rules are missing.  Given a term that the user believes should \u000A ``obviously'' have some tau (``type'') what rules might be added to make the \u000A tau algorithm compute that expected tau?  For example, if <v>(g x)</v> is known \u000A to satisfy <v>P</v> and <v>(f x)</v> is known to satisfy <v>R</v> when its argument \u000A satisfies <v>S</v>:</p> \u000A \u000A <code>g : T ==&gt; P\u000Af : S ==&gt; R</code> \u000A \u000A <p>then if the user asserts that <v>(f (g x))</v> ``ought'' to have tau <v>R</v>, \u000A one plausible suggestion is the simple tau rule that <v>(P x)</v> implies <v>(S\u000A x)</v>.  Such assistance &mdash; while still confronting an undecidable problem \u000A &mdash; might be easier to implement within the tau framework than more \u000A generally in ACL2.  (Many users have wanted such an assistant to suggest \u000A lemmas for the rewriter.)</p>"],

 "ACL2____FURTHER-INFORMATION-ON-REWRITING":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>In the following paragraphs we give some links to advanced topics, \u000A marked with ``<see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>''.  If you are reading this topic as part of the \u000A tutorial on the theorem prover, do not follow these links upon your first \u000A reading.  Just take note of the existence of the facilities and ideas \u000A mentioned.</p> \u000A \u000A <p><see topic=\"ACL2____ARITHMETIC\">Arithmetic</see>: If your goal theorem involves even trivial arithmetic, \u000A such as adding or subtracting <v>1</v>, we recommend that you do</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic/top-with-meta&quot; :dir :system)</code> \u000A \u000A <p>which loads into ACL2 all the rules in one of the so-called ACL2 \u000A ``community books''.  (<i>Books</i> are certified files of definitions, \u000A lemmas, etc., usually prepared by other ACL2 users and explicitly shared with \u000A the community.  The ACL2 installation instructions suggest downloading the \u000A community books.)  The book \"top-with-meta\" is the most elementary and most \u000A widely used arithmetic book.  Other community books include \u000A \"arithmetic-5/top\" and various hardware and floating-point arithmetic \u000A books.</p> \u000A \u000A <p><b>Rules Concluding with Arithmetic Inequalities</b>: If you are tempted to \u000A create a rewrite rule with an arithmetic inequality as its conclusion or \u000A left-hand side, think again.  Inequalities such as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"ACL2____LEN\">len</see> (delete e x)) (<see topic=\"ACL2____LEN\">len</see> x))</code> \u000A \u000A <p>make poor left-hand sides for rewrite rules.  For example, the inequality \u000A above does not match the target</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"ACL2____LEN\">LEN</see> (DELETE E X)) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">LEN</see> X)))</code> \u000A \u000A <p>even though it is sufficient to prove the target (given some simple \u000A arithmetic).  We recommend that if you have a theorem that establishes an \u000A arithmetic inequality, you make it a <i>linear</i> rule.  See <see topic=\"ACL2____LINEAR\">linear</see> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>Rearranging Formulas Before Making Rules</b>: It is possible to \u000A rearrange the propositional structure of a proved formula before processing it \u000A as a rule.  This allows you to state a theorem one way ``for publication'' and \u000A rearrange it to be stored as a more effective rule.  See <see topic=\"ACL2____INTRODUCTION-TO-THE-DATABASE\">introduction-to-the-database</see> (a tutorial topic you'll come to later) and its \u000A discussion of the concept of <v>corollary</v>.  Also, see the discussion of \u000A <v>corollary</v> in <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>Rewriting with New Equivalence Relations</b>: You may introduce new \u000A <i>equivalence</i> relations, like ``set-equal'' or ``is-a-permutation'' and \u000A cause the rewriter to replace equivalents by equivalents in suitable contexts, \u000A where you use <i>congruence</i> rules to inform ACL2 of where these more \u000A relaxed notions of equivalence may be used; see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A see <see topic=\"ACL2____CONGRUENCE\">congruence</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>Pragmatic Advice to Control Rules</b>: You may attach various \u000A <i>pragmas</i> to a rule that allow you rather fine heuristic control over \u000A whether and how the rule is applied.  For example, you may mark a hypothesis \u000A to be <i>forced</i> (see <see topic=\"ACL2____FORCE\">force</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A meaning that the rule is to be applied even if that hypothesis is not relieved \u000A -- but if the proof is successful the system will turn its attention to all \u000A forced subgoals.  You may similarly mark a hypothesis so as to cause a case \u000A split, allowing the relief of the hypothesis on one branch and spawning \u000A another branch explicitly denying the hypothesis; see <see topic=\"ACL2____CASE-SPLIT\">case-split</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A You may add a bogus hypothesis that looks at the intended application of the \u000A rule and decides whether to apply the rule or not, performing an arbitrary \u000A computation on the syntactic context of the application; see <see topic=\"ACL2____SYNTAXP\">syntaxp</see> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.  By providing a <v>:match-free</v> modifier to the \u000A <v>:rewrite</v> rule declaration in your rule-classes, you may tell ACL2 to try \u000A all or only the first free variable value it guesses (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>).  You may provide a bogus hypothesis that computes \u000A from the syntactic environment the values to guess for the free variables in a \u000A rule; see <see topic=\"ACL2____BIND-FREE\">bind-free</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A You may mark a term so that the rewriter does not dive into it; see <see topic=\"ACL2____HIDE\">hide</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>Programming Your Own Rewriter</b>: If you cannot find a way to use \u000A rewrite rules to make the transformations you desire, you might investigate \u000A the use of <i>metafunctions</i>.  A metafunction is just a little theorem \u000A prover of your own design.  It takes as input a list structure representing a \u000A term and returns a list structure representing a term.  If you can prove that \u000A the meaning of the input and output terms are equivalent, you can extend the \u000A ACL2 simplifier to call your metafunction.  See <see topic=\"ACL2____META\">meta</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p><b>The Order in which Targets are Rewritten</b>: The rewriter sweeps \u000A through terms ``inside-out'' otherwise known as ``left-most innermost first''. \u000A Thus, before trying to apply rewrite rules to <v>(</v><i>f a1 ... an</i><v>)</v>, \u000A rules are applied to the <i>ai</i>.  This has the good effect of normalizing \u000A the <i>ai</i>.</p> \u000A \u000A <p>This fact might help you understand why sometimes your rules ``don't seem \u000A to fire.''  For example, suppose you have a rule for rewriting <v>(<see topic=\"ACL2____LEN\">len</see> (rev\u000A x))</v> to <v>(<see topic=\"ACL2____LEN\">len</see> x)</v> and suppose you wish to prove a theorem about <v>(<see topic=\"ACL2____LEN\">LEN</see>\u000A (REV (<see topic=\"COMMON-LISP____CONS\">CONS</see> A B)))</v>.  Suppose <v>rev</v> is defined in terms of <v>append</v>, as \u000A shown in <see topic=\"ACL2____PROGRAMMING-KNOWLEDGE-TAKEN-FOR-GRANTED\">programming-knowledge-taken-for-granted</see>.  Then you might see a \u000A checkpoint in which the <v>(<see topic=\"ACL2____LEN\">LEN</see> (REV ...))</v> above has been simplified to \u000A <v>(<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV B) (<see topic=\"COMMON-LISP____LIST\">LIST</see> A)))</v> instead of to <v>(<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> A B))</v>.  Why \u000A wasn't your rule about <v>(<see topic=\"ACL2____LEN\">len</see> (rev x))</v> applied?  The reason is that <v>(REV\u000A (<see topic=\"COMMON-LISP____CONS\">CONS</see> A B))</v> rewrote to <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV B) (<see topic=\"COMMON-LISP____LIST\">LIST</see> A))</v> before rules were \u000A applied to <v>(<see topic=\"ACL2____LEN\">LEN</see> (REV ...))</v>.  You need a rule about <v>(<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x\u000A y))</v>, as you will see from the checkpoint.</p> \u000A \u000A <p><b>The Order in which Rules are Tried</b>: The rewriter tries the most \u000A recently proved rules first.  For example, suppose <v>f</v>, <v>g</v>, and <v>h</v> \u000A are functions defined so that the following two theorems are provable and \u000A suppose you executed the following two events in the order shown:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> rule1 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (g x)) (h 1 x)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rule2 (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (g x)) (h 2 X)))</code> \u000A \u000A <p>Then if rewrite rules are applied to <v>(F (G A))</v>, the result will be \u000A <v>(H 2 A)</v>, because the latter rule, <v>rule2</v>, is applied first.  It is \u000A generally best not to have conflicting rules or, at least, to understand how \u000A such conflicts are resolved.  The system will warn you when you propose a rule \u000A that conflicts with an existing one.</p> \u000A \u000A <p>If you were reading this topic as part of the tutorial introduction to the \u000A theorem prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see>.</p>"],

 "ACL2____FREQUENTLY-ASKED-QUESTIONS-BY-NEWCOMERS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This FAQ is for people who've read through all the other sections \u000A of the tutorial introduction to the theorem prover (see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> and all the links from it that are not \u000A marked with the little warning sign (``<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>''). \u000A Do not expect to understand our answers if you haven't taken the time to read \u000A through the tutorial.  In the answers below you will see more links into the \u000A hypertext reference manual.  While such links were marked ``<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>'' \u000A in the tutorial, they are not marked that way here.  When you enter the \u000A reference manual be prepared to explore and assemble a mini-course on the \u000A topic of interest, not a quick fix.</p> \u000A \u000A <p><b>Q</b>.  How do I find something in the <b>ACL2 documentation</b>? \u000A <b>A</b>.  Try going to the Search link on the ACL2 home page.</p> \u000A \u000A <p><b>Q</b>.  How does the theorem prover work?  <b>A</b>.  We really don't \u000A think you need to know much about the inner workings of the prover to become \u000A an effective user.  That doesn't mean the system is self-explanatory!  It \u000A means that stuff you need to learn is not how the theorem prover works but how \u000A to interact with it!  That is what <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> \u000A is about.  However, if you want the most basic overview of the prover, see \u000A <see topic=\"ACL2____ARCHITECTURE-OF-THE-PROVER\">architecture-of-the-prover</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>define a new function</b>?  <b>A</b>. See <see topic=\"COMMON-LISP____DEFUN\">defun</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>define a new predicate</b>?  <b>A</b>. See <see topic=\"COMMON-LISP____DEFUN\">defun</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>define a new relation</b>?  <b>A</b>. See <see topic=\"COMMON-LISP____DEFUN\">defun</see>.</p> \u000A \u000A <p><b>Q</b>. How do I define a <b>function or predicate that takes a varying \u000A number of arguments</b>?  <b>A</b>. You can't.  However, see <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> \u000A to learn how to define a macro that takes a varying number of arguments and \u000A expands into an arbitrary term that you compute.</p> \u000A \u000A <p><b>Q</b>. How do I define a <b>macro that is sensitive to the state</b>? \u000A <b>A</b>.  You can't.  However, advanced users should consider <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.</p> \u000A \u000A <p><b>Q</b>. How do I define <b>mutually recursive</b> functions?  <b>A</b>. \u000A See <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>.  However, you should realize that when two \u000A functions, say <v>f</v> and <v>g</v>, are mutually recursive, properties of <v>f</v> \u000A generally have to be stated simultaneously with properties of <v>g</v>, since \u000A inductive proofs about one function require inductive hypotheses about the \u000A other.  Furthermore, ACL2 does not know how to do inductions for mutually \u000A recursive functions and must be told.  See <see topic=\"ACL2____MUTUAL-RECURSION-PROOF-EXAMPLE\">mutual-recursion-proof-example</see>.</p> \u000A \u000A <p><b>Q</b>. How do I declare the <b>type signature of a function</b>? \u000A <b>A</b>.  You can't.  ACL2 is a syntactically untyped language and all \u000A functions are defined on all objects in the ACL2 universe.  We recommend that \u000A the new user get used to this and only then explore the use of ACL2 to express \u000A and enforce a type regime.  In ACL2, the <i>guard</i> of a function is akin to \u000A the type signature of a function in typed languages.  However, ACL2 guards may \u000A be arbitrary terms, not just type conditions, they only concern the inputs to \u000A the function (not its result), and do not affect the axiom defining the \u000A function -- all functions are defined on every combination of objects.  You \u000A may, of course, prove theorems that establish that every function called in a \u000A definition or theorem is supplied with input satisfying its guards (which \u000A necessarily involves describe the outputs too).  These formulas are called \u000A <i>guard conjectures</i> and the process of proving them is called <i>guard \u000A verification</i>.  Since guards are arbitrary ACL2 formulas, the ``type \u000A regimes'' one tends to enforce in ACL2 can be much for flexible and expressive \u000A than those in most programming languages.  However, that expressibility also \u000A means guard verification can be challenging (indeed undecidable).  On the \u000A other hand, if one limits oneself to simple type-like guards, lemmas can be \u000A proved that make most guard verification fully automatic and one can configure \u000A ACL2 to do guard verification automatically at <v>defun</v>-time.  One may also \u000A delay guard verification until ``the right'' lemmas have been proved.  By \u000A doing guard verification one can make functions execute faster by allowing the \u000A code to avoid runtime checks.  This is especially valuable to industrial users \u000A who have large models used both in verification and as simulation engines for \u000A designed artifacts.  In addition, guard verification can give you the \u000A assurance that you are using your functions within their intended domains and \u000A hence is a form of functional specification and verification.  However, all \u000A these advantages aside, it is remarkably easy to drift away from the simplest \u000A type regimes and write a guard that raises deep mathematical problems.  New \u000A users should not try to confront these problems until they are comfortable \u000A with the theorem prover and with lemma development.  Therefore, we \u000A <b>strongly</b> recommend that you forget about types and guards and get used \u000A to reasoning about total functions.  When you do decide to learn about them, \u000A be prepared for a complex story involving specification, execution efficiency, \u000A and proof management.  See <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p><b>Q</b>. How do I tell <v>defun</v> <b>what measure to use</b>? \u000A <b>A</b>. See <see topic=\"ACL2____XARGS\">xargs</see>, specifically <v>:measure</v>.</p> \u000A \u000A <p><b>Q</b>.  I specified a measure that always returns a natural number but \u000A ACL2 is acting like it's not a natural number.  <b>A</b>.  There are two \u000A likely problems.  The most likely one is that your measure isn't really always \u000A a natural!  Suppose the formals of your <v>defun</v> are <v>x</v> and <v>y</v> and \u000A your measure is <v>(m x y)</v>.  Suppose the recursive calls of your function \u000A are protected by tests that insure that <v>x</v> and <v>y</v> are naturals.  Then \u000A you might assume <v>x</v> and <v>y</v> are naturals in the measure.  But ACL2 has \u000A to prove <v>(<see topic=\"ACL2____O-P\">o-p</see> (m x y))</v>, where <tt><see topic=\"ACL2____O-P\">o-p</see></tt> is the predicate that \u000A recognizes ordinals (and naturals are ordinals).  Note that the theorem \u000A doesn't have any hypotheses!  You might intuitively think that your measure \u000A has to be an ordinal just under the conditions that lead to recursive calls. \u000A That's not what ACL2 enforces.  It has to be an ordinal, always.  So you must \u000A change your specified measure.  For example, consider wrapping <tt><see topic=\"ACL2____NFIX\">nfix</see></tt> \u000A around it or around its uses of <v>x</v> and <v>y</v> to coerce those quantities \u000A to naturals.  The second most likely explanation is that your measure returns \u000A a natural, always, but ACL2 doesn't know that and it takes induction to prove. \u000A This might happen if <v>m</v> involves some recursive functions.  In this case, \u000A prove <v>(<see topic=\"ACL2____NATP\">natp</see> (m x y))</v> before your <v>defun</v>.  Perhaps you should consider \u000A making the <v>natp</v> lemma a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> lemma to make \u000A ACL2's typing algorithm aware of it.</p> \u000A \u000A <p><b>Q</b>. How do I tell <v>defun</v> <b>what well-founded relation to \u000A use</b>?  <b>A</b>. See <see topic=\"ACL2____XARGS\">xargs</see>, specifically \u000A <v>:well-founded-relation</v>.</p> \u000A \u000A <p><b>Q</b>. How do I show that a <b>relation is well-founded</b>? \u000A <b>A</b>. Prove a theorem establishing that there is an order preserving \u000A embedding into the ordinals and store it with <v>:rule-classes</v> <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt>.</p> \u000A \u000A <p><b>Q</b>.  What is an <b>ordinal</b>?  What does it mean to be \u000A <b>well-founded</b>?  <b>A</b>.  Ordinals are an extension of the natural \u000A numbers used to insure that a process can't go on forever.  Like naturals, \u000A they can be added, multiplied, and exponentiated.  There is a sense of one \u000A ordinal being less than another.  Unlike the naturals, each of which is \u000A finite, the ordinals include infinite objects.  Now imagine ``standing'' on an \u000A ordinal and ``stepping'' to a smaller one.  Like the naturals, this ``walk \u000A down the ordinals'' can't go on forever, even if you start on an infinite \u000A ordinal.  That is because the ordinals are <i>well-founded</i>.  See <tt><see topic=\"ACL2____O-P\">o-p</see></tt> for more information about ordinals in ACL2 and about well-foundedness. \u000A See <see topic=\"ACL2____ORDINALS\">ordinals</see> for a deeper discussion and a discussion of books that can \u000A help configure ACL2 to reason about ordinals.</p> \u000A \u000A <p><b>Q</b>. How can provide <b>hints for the termination proofs</b> in \u000A <v>defun</v>?  <b>A</b>. See <see topic=\"ACL2____XARGS\">xargs</see>, specifically <v>:hints</v> (for the \u000A termination proofs) and <v>:guard-hints</v> (for the guard verification \u000A proofs).</p> \u000A \u000A <p><b>Q</b>. How do I define a <b>constant</b> (something like a <b>global \u000A variable</b>)?  <b>A</b>. See <see topic=\"ACL2____DEFCONST\">defconst</see>.  But remember that as an \u000A applicative programming language, ACL2 does not have global variables!  You \u000A can define a symbol to have a fixed value and use the symbol sort of like a \u000A global variable in function definitions: you may refer to the value of the \u000A symbol in your functions without passing the variable in as formal parameter. \u000A But you may not ever change the value of the symbol!</p> \u000A \u000A <p><b>Q</b>. How do I save the value of a top-level computation for future \u000A use?  <b>A</b>. See <see topic=\"ACL2____ASSIGN\">assign</see> and see <see topic=\"ACL2_____04\">@</see>.</p> \u000A \u000A <p><b>Q</b>. How do I introduce <b>new syntactic form</b> or \u000A <b>abbreviation</b>?  <b>A</b>. See <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>.</p> \u000A \u000A <p><b>Q</b>.  How can create and modify an array?  <b>A</b>.  ACL2 is a \u000A functional language, so it is impossible to destructively modify an existing \u000A object; technically, all ``updates'' to objects must be implemented by \u000A ``copy-on-write'' semantics.  That said, ACL2 provides support for <see topic=\"ACL2____ARRAYS\">arrays</see>, provided you use them in a restricted way.  They give you \u000A constant-time access and change under the use restrictions.</p> \u000A \u000A <p><b>Q</b>.  How do I read from or write to a file?  How do I do IO? \u000A <b>A</b>.  To manipulate files, your function must have <tt><see topic=\"ACL2____STATE\">state</see></tt> as an \u000A argument, so you should read about the restrictions that imposes.  For \u000A input/output facilities, see <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p><b>Q</b>. How do I define a <b>structure that can be destructively \u000A modified</b>?  <b>A</b>. ACL2 is an <i>applicative programming language</i>. \u000A You can't modify objects arbitrarily!  You basically have to ``copy on \u000A write,'' which means you construct new objects from old ones, making the \u000A changes you want in the new one.  If the <v>car</v> of some object is <v>1</v> at \u000A one moment and <v>2</v> later, then the basic logical axiom <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> = \u000A <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> is violated!  However, if the only reference to the old object, \u000A e.g., <i>x</i>, was to pass it to the code that copied and ``changed'' it, \u000A then ACL2 can re-use the old object to produce the new one and the axioms \u000A would not object.  Such syntactic restrictions can make <i>x</i> a modifiable \u000A structure but they will impose a heavy burden on you as a programmer: if pass \u000A such an <i>x</i> to a function and the function modifies it, then you must \u000A pass <i>x</i> only to that function and you must return the modified value and \u000A use it henceforth.  Such objects are said to be <i>single threaded</i>.  See \u000A <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.</p> \u000A \u000A <p><b>Q</b>.  How do I write a universal quantifier?  An existential \u000A quantifier?  How can I say ``for all'' or ``there exists''?  <b>A</b> You \u000A can't literally write quantifiers.  But ACL2 has the power of full first order \u000A logic with quantification.  See <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see>.</p> \u000A \u000A <p><b>Q</b>.  How do I introduce an undefined or uninterpreted function \u000A symbol?  Can I constrain it to have certain properties?  <b>A</b>.  See <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p><b>Q</b>.  How can I hide a lemma?  I want to prove a lemma temporarily to \u000A use in another proof but I don't want the lemma around thereafter.  <b>A</b>. \u000A One way to get a similar effect is to prove the lemma and then disable it with \u000A an <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> ...))</v> event; see <see topic=\"ACL2____IN-THEORY\">in-theory</see>.  Another way is \u000A to put the lemma and the theorem that needs it into an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and \u000A wrap a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> around the lemma.</p> \u000A \u000A <p><b>Q</b>. What is an <b>event</b>?  <b>A</b>. An <i>event</i> is a command \u000A that adds information to the ACL2 database (the ``logical world''), like \u000A <v>defun</v> or <v>defthm</v>.  See <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p><b>Q</b>. How do I say that I <b>do not want a rewrite rule</b> generated \u000A from a theorem?  <b>A</b>. The command</p> \u000A \u000A <code>(defthm <i>name</i> <i>formula</i> \u000A        :rule-classes nil) \u000A</code> \u000A \u000A <p>will attempt to prove <i>formula</i> and, if successful, give \u000A <i>formula</i> the name <i>name</i>, which you may use in hints as a theorem, \u000A but it will build no rules from the formula.</p> \u000A \u000A <p><b>Q</b>. How do I say that I want a formula to be <b>stored as a rewrite \u000A rule</b>?  <b>A</b>. The command</p> \u000A \u000A <code>(defthm <i>name</i> <i>formula</i>) \u000A</code> \u000A \u000A <p>will attempt to prove <i>formula</i> and, if successful, it will give \u000A <i>formula</i> the name <i>name</i> and generate a rewrite rule from it, with \u000A the runic name <v>(:rewrite </v><i>name</i>)].  It could happen that \u000A <i>formula</i> generates more than one rewrite rule, e.g., this happens if the \u000A conclusion is an <v>AND</v>.  In this case, each conjunctive branch through the \u000A conclusion generates a rule named <v>(:rewrite </v><i>name</i><v> . i)</v>, for \u000A <i>i</i>=1,2,...  For more details, see <see topic=\"ACL2____REWRITE\">rewrite</see>.</p> \u000A \u000A <p><b>Q</b>. How do I say that I want a formula to be <b>stored as a rewrite \u000A rule</b> <b>and some other kinds of rules</b>?  <b>A</b>. The command</p> \u000A \u000A <code>(defthm <i>name</i> <i>formula</i> \u000A        :rule-classes (<i>:class1</i> ... <i>classk</i>)) \u000A</code> \u000A \u000A <p>will attempt to prove <i>formula</i> and, if successful, it will give \u000A <i>formula</i> the name <i>name</i> and generate a rule of each <i>:classi</i> \u000A specified.  Each <i>:classi</i> should either be a keyword, like <v>:REWRITE</v> \u000A or <v>:GENERALIZE</v>, naming a rule class (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>), or else \u000A should be a list that starts with a rule class and then lists the relevant \u000A modifiers.  Be sure to include <v>:REWRITE</v> among the rule classes if you \u000A want the formula to generate a rewrite rule.  It doesn't do that automatically \u000A if you explicitly specify <v>:rule-classes</v>!</p> \u000A \u000A <p><b>Q</b>. How do I <b>rearrange</b> the shape of a formula before \u000A generating a rule from it?  <b>A</b>. See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> and read about \u000A the <v>:corollary</v> modifier.</p> \u000A \u000A <p><b>Q</b>. What is a <b>type-prescription</b>?  <b>A</b>. ACL2 has an \u000A algorithm for determining the type of object returned by a term, where a type \u000A is one of the Common Lisp primitive datatypes such as natural, integer, \u000A Boolean, cons, true-listp, etc.  Rules provided by you can influence this \u000A algorithm.  See <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>.</p> \u000A \u000A <p><b>Q</b>. How do <b>rewrite rules work</b>?  <b>A</b>. Re-read the tutorial \u000A sections: <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see> and <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>see what's in the database</b>?  <b>A</b>. You can't \u000A look at the entire database with user-friendly tools.  You can print the \u000A command that introduced a particular name, print the entire sequence of user \u000A commands, print the commands in the region between two commands, print all the \u000A rewrite rules that apply to a given term or function symbol, and many other \u000A options.  See <see topic=\"ACL2____HISTORY\">history</see>.  If you have loaded a book from another user, \u000A you might wish to read the source file.  For example, the source file for \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-5/top&quot; :dir :system)</v> is the file named \u000A <v>arithmetic-5/top.lisp</v> on the <v>acl2-sources/books/</v> directory where \u000A ever your ACL2 sources are installed.  Often, books are well-documented by \u000A comments by their authors.  Some books have <v>Readme</v> or <v>README</v> files \u000A on the same directory.</p> \u000A \u000A <p><b>Q</b>. How do I <b>undo</b> a command?  <b>A</b>. See <see topic=\"ACL2____HISTORY\">history</see>, \u000A especially see <see topic=\"ACL2____U\">u</see> (``undo'') and see <see topic=\"ACL2____UBT\">ubt</see> (``undo back \u000A through''). <b>Q</b>. What <b>rewrite rules match</b> a given term? \u000A <b>A</b>. See <see topic=\"ACL2____PL\">pl</see>.</p> \u000A \u000A <p><b>Q</b>.  What were those <b>questions to ask when looking at key \u000A checkpoints</b>?  <b>A</b>.  See <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p> \u000A \u000A <p><b>Q</b>. How do I figure out <b>why a rewrite rule won't fire</b>? \u000A <b>A</b>. If you activate rewrite rule monitoring (see <see topic=\"ACL2____BRR\">brr</see>) and then \u000A install a monitor (see <see topic=\"ACL2____MONITOR\">monitor</see>) on the rule in question, ACL2 will \u000A enter an interactive break whenever the pattern of the rule is matched against \u000A a target in a proof.  So after installing the monitor, re-try the proof and \u000A then interact with the rewriter via break commands (see <see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see>). \u000A Like all trace and break packages, you have to know what you're doing to use \u000A the break rewrite facility, so read the material in the reference manual.  If \u000A no interactive break happens after you've installed the monitor on your rule \u000A and re-tried the proof, it means no suitable target ever arose.  Don't forget \u000A to turn off monitoring when you're done as it slows down the system.</p> \u000A \u000A <p><b>Q</b>. <b>Why is a proof taking so long?</b> <b>A</b>. Unexpectedly poor \u000A performance on simple problems is usually a sign of cyclic rewriting or \u000A combinatoric explosion.  See <see topic=\"ACL2____DMR\">dmr</see> and see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>.</p> \u000A \u000A <p><b>Q</b>. How do I tell ACL2 <b>what induction to do</b> for a particular \u000A formula?  <b>A</b>. When issuing the <v>defthm</v> command for the formula, \u000A supply an <v>:induct</v> hint:</p> \u000A \u000A <code>(defthm <i>name</i> \u000A        <i>formula</i> \u000A        :hints ((\"Goal\" :induct (f x1 ... xn)))) \u000A</code> \u000A \u000A <p>where <v>f</v> is a function that recurs the way you want the induction to \u000A unfold and <v>x1 ... xn</v> are the relevant variables in <i>formula</i>.  You \u000A usually have to define <v>f</v> appropriately for each formula that needs an \u000A induct hint, though sometimes you can re-use an earlier <v>f</v> or one of the \u000A functions in the formula itself.  It doesn't matter what value <v>(f x1\u000A ... xn)</v> returns.  All that matters is how it recurs.  The termination \u000A analysis for <v>f</v> justifies the induction done.  See <see topic=\"ACL2____HINTS\">hints</see>, \u000A especially the section on <v>:induct</v> hints; also see <see topic=\"ACL2____INDUCTION\">induction</see>.</p> \u000A \u000A <p><b>Q</b>. ACL2 doesn't know <b>simple arithmetic</b> that can simplify the \u000A term <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 -1 x)</v>.  <b>A</b>.  You should load an arithmetic book whenever \u000A you're dealing with an arithmetic problem.  The simplest arithmetic book is \u000A typically loaded with the event <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic/top-with-meta&quot;\u000A :dir :system)</v>.  If you're using <v>floor</v> and <v>mod</v> or non-linear \u000A arithmetic like <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x y)</v> you should use <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>\u000A &quot;arithmetic-5/top&quot; :dir :system)</v>.  See also the discussion of arithmetic \u000A books under the ``Lemma Libraries and Utilities'' link of the ACL2 home page, \u000A and see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p> \u000A \u000A <p><b>Q</b>. ACL2 is not using an <b>arithmetic lemma</b> that I proved. \u000A <b>A</b>. Lemmas concluding with arithmetic inequalities, like</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____LEN\">len</see> (delete e x)) (<see topic=\"ACL2____LEN\">len</see> x)))</code> \u000A \u000A <p>are not good rewrite rules because they rarely match targets because of \u000A intervening arithmetic operators.  For example, the above conclusion doesn't \u000A match <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____LEN\">LEN</see> (DELETE E X)) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">LEN</see> X)))</v>.  You should store such lemmas \u000A as <v>:linear</v> rules by using the command:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> len-delete\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____LEN\">len</see> (delete e x)) (<see topic=\"ACL2____LEN\">len</see> x)))\u000A  :rule-classes :linear)</code> \u000A \u000A <p>See <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p><b>Q</b>.  What is a <b>linear rule</b>?  <b>A</b>.  See <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p><b>Q</b>. How do I make ACL2 <b>treat a relation as an equality</b>? \u000A <b>A</b>. We assume you mean to treat the relation as an equivalence, i.e., \u000A replace one term by another when they are equivalent under your relation.  See \u000A <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>, see <see topic=\"ACL2____CONGRUENCE\">congruence</see>, and see <see topic=\"ACL2____REFINEMENT\">refinement</see>.</p> \u000A \u000A <p><b>Q</b>. One of my rewrite rules has a <b>hypothesis that doesn't \u000A rewrite</b> to true.  What do I do?  <b>A</b>. Prove a rewrite rule that \u000A establishes that hypothesis under the relevant assumptions from the context of \u000A the intended target.  Alternatively, undo the rule and restate it with a \u000A <tt><see topic=\"ACL2____FORCE\">force</see></tt> around the problematic hypothesis, making ACL2 assume the \u000A hypothesis when the rule is applied and raising the truth of the hypothesis as \u000A an explicit subgoal of the proof.  See also <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt>.  Of course, \u000A you should always state the strongest rewrite rules you can think of, \u000A eliminating hypotheses or shifting them to the right-hand side inside of \u000A <v>IF</v>s; see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>.</p> \u000A \u000A <p><b>Q</b>. How do I make ACL2 ``guess'' the <b>right instantiation of a free \u000A variable</b>?  <b>A</b>. You can provide a <v>:restrict</v> hint that names the \u000A problematic lemma and provides an instantiation of the free variable.  See \u000A <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:restrict</v>.  You could alternatively give a \u000A hint that <v>:uses</v> the rule and provides the appropriate instantiation in \u000A full.  See <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:use</v>.  Recall that ACL2 guesses \u000A free variable instantiations by matching the problematic hypothesis to the \u000A assumptions in the context of the target.  If the appropriate assumption is \u000A present but ACL2 is finding another one, try undoing the problematic rule and \u000A proving it again, specifying the <v>:match-free :all</v> modifier of the \u000A <v>:rewrite</v> or <v>:linear</v> rule class.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>. \u000A Alternatively, undo and prove the problematic rule again and use a <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> form to compute the appropriate instantiation.</p> \u000A \u000A <p><b>Q</b>. How can I make ACL2 do a <b>case split</b> to prove a certain \u000A subgoal?  <b>A</b>. See <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:cases</v>.</p> \u000A \u000A <p><b>Q</b>. How can I <b>prevent ACL2 from using a rewrite rule</b>? \u000A <b>A</b>.  See <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> ...)</v>.  If \u000A the use of the rule is problematic in only one subgoal and the lemma is needed \u000A in other subgoals, disable the lemma only in the problematic subgoal by \u000A specifying the subgoal name (e.g., <v>&quot;Subgoal 1/3.2.1&quot;</v>) as the goal \u000A specifier in the hint.  If the rule isn't needed anywhere in the proof, you \u000A could use the specifier <v>&quot;Goal&quot;</v>.  If you don't think the rule will ever \u000A be needed for a while, you could globally disable it with the event \u000A <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> ...))</v>  (see <see topic=\"ACL2____IN-THEORY\">in-theory</see>) executed before the \u000A first problematic proof.  If the rule has never been used or must always be \u000A disabled, undo it and prove it again with <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> \u000A <v>nil</v>.</p> \u000A \u000A <p><b>Q</b>.  How can I prevent ACL2 from running a definition on constants? \u000A I tried disabling the function but that didn't work.  <b>A</b>.  If you have a \u000A function named <v>f</v> then disabling <v>f</v> will disable the definitional \u000A axiom about <v>f</v>.  But ACL2 has another kind of rule about <v>f</v>, telling \u000A it how to evaluate <v>f</v>.  The <see topic=\"ACL2____RUNE\">rune</see> of this rule is \u000A <v>(:executable-counterpart f)</v>.  Try disabling that, as in the <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> <v>((</v>...  <v>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart f))\u000A ...c[))</v>.</p> \u000A \u000A <p><b>Q</b>. How can I make ACL2 <b>use a rule</b> in a proof?  <b>A</b>. See \u000A <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:use</v>.</p> \u000A \u000A <p><b>Q</b>.  How can I make ACL2 expand a function call in a proof? \u000A <b>A</b>.  You can give an <v>:</v>See <see topic=\"ACL2____EXPAND\">expand</see> hint.</p> \u000A \u000A <p><b>Q</b>.  ACL2 sometimes aborts the proof attempt before showing me all of \u000A the subgoals.  How can I make it just keep going instead of aborting early? \u000A <b>A</b>. See <see topic=\"ACL2____OTF-FLG\">otf-flg</see>, which stands for Onward Thru the Fog FLaG.</p> \u000A \u000A <p><b>Q</b>. How can I <b>compute when I want a rule to fire</b>?  <b>A</b>. \u000A See <see topic=\"ACL2____SYNTAXP\">syntaxp</see>.</p> \u000A \u000A <p><b>Q</b>.  How can I add <b>pragmatic advice to a lemma after it has been \u000A proved</b>?  For example, how can add a forced hypothesis, a backchain limit, \u000A or a <v>syntaxp</v> test?  <b>A</b>.  You can't.  You can undo the lemma, \u000A restate it appropriately, and prove it again.  This produces the cleanest \u000A database.  Alternatively, you can prove the restated lemma under a new name -- \u000A a task that should be easy since the old version is in the database and will \u000A presumably make the proof trivial -- and then disable the old name.</p> \u000A \u000A <p><b>Q</b>. How can I <b>stop ACL2 from rewriting a term</b>?  <b>A</b>. If \u000A you need rewriting done but want to prevent some particular terms from being \u000A rewritten, see <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:hands-off</v>.  Alternatively, \u000A consider embedding the problematic term in a <tt><see topic=\"ACL2____HIDE\">hide</see></tt>.  Users sometime \u000A develop special theories (see <see topic=\"ACL2____THEORY\">theory</see>) containing just the rules they \u000A want and then use hints to switch to those theories on the appropriate \u000A subgoals.</p> \u000A \u000A <p><b>Q</b>.  Can I <b>compute which subgoals a hint refers to</b>?  <b>A</b>. \u000A Yes, see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.  This topic is for advanced users but knowing \u000A that it is possible might come in handy someday.</p> \u000A \u000A <p><b>Q</b>.  I want the rewriter to <b>always use one theory and then switch \u000A to another</b> so that it doesn't use my most complicated before until doing \u000A the simple things.  <b>A</b>.  This is possible but is something for the \u000A advanced user.  It can be done using a form of <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>.  See \u000A <see topic=\"ACL2____USING-COMPUTED-HINTS-7\">using-computed-hints-7</see>.</p> \u000A \u000A <p><b>Q</b>.  Is there a way to attach <b>the same hint to every defthm</b>? \u000A <b>A</b>.  See <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p><b>Q</b>.  How can I just tell ACL2 the proof steps?  <b>A</b>.  See <see topic=\"ACL2____VERIFY\">verify</see> and see <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.</p> \u000A \u000A <p><b>Q</b>. How can I write <b>my own simplifier</b>?  <b>A</b>.  See <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <p><b>Q</b>. How can I add an axiom or just assume some lemma without proof? \u000A <b>A</b>. This is very dangerous but is a good strategy for exploring whether \u000A or not a certain set of lemmas (and their rules) is sufficient to prove your \u000A goal.  See <see topic=\"ACL2____DEFAXIOM\">defaxiom</see> and see <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>.</p> \u000A \u000A <p><b>Q</b>. How can redefine a user-defined function?  <b>A</b>.  This is \u000A tricky.  What if you've already proved theorems about the old definition and \u000A then wish to change it?  There are several options.  See <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see> (and note specifically the discussion of updater \u000A function for it, <v>set-ld-redefinition-action</v>); also see <see topic=\"ACL2____REDEF\">redef</see>, see \u000A <see topic=\"ACL2____REDEF_12\">redef!</see>, see <see topic=\"ACL2____REDEF_B2\">redef+</see>, and see <see topic=\"ACL2____REDEF-\">redef-</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>change a function from</b> <v>:program</v> <b>mode \u000A to</b> <v>:logic</v> <b>mode</b>?  <b>A</b>. See <see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>change the guards</b> on a function?  <b>A</b>. You \u000A can't.  Undo it and redefine it.</p> \u000A \u000A <p><b>Q</b>. What is <b>program mode</b>?  <b>A</b>. See <see topic=\"ACL2____PROGRAM\">program</see>.</p> \u000A \u000A <p><b>Q</b>. What does the ACL2 <b>prompt</b> mean?  <b>A</b>.  See <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see> or, specifically, see <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <p><b>Q</b>. What is <b>logic mode</b>?  <b>A</b>. See <see topic=\"ACL2____LOGIC\">logic</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>get into or out of</b> <v>:program</v> <b>mode?</b> \u000A <v>:Logic</v> <b>mode?</b> <b>A</b>. See <see topic=\"ACL2____PROGRAM\">program</see> and see <see topic=\"ACL2____LOGIC\">logic</see>. \u000A You can enter these modes temporarily for a particular <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> by using \u000A <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))</v> or <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic))</v> \u000A after the list of formal parameters to the definition.</p> \u000A \u000A <p><b>Q</b>.  How do I quit from ACL2?  <b>A</b>.  This varies depending on \u000A the interface you're using.  See <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>load a file</b> of definitions and theorems created \u000A by someone else?  <b>A</b>. See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.</p> \u000A \u000A <p><b>Q</b>. How do I <b>create my own book</b> of definitions and theorems? \u000A <b>A</b>. See <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p><b>Q</b>. Where are the books referenced by <b>:dir :system</b> on my \u000A machine?  <b>A</b>. If your ACL2 is installed on the directory \u000A <i>dir</i><v>/acl2-sources</v> and you follow the standard installation \u000A instructions, then the books are typically the files under the directory \u000A <i>dir</i><v>/acl2-sources/books/</v>.</p> \u000A \u000A <p><b>Q</b>. How can I find out <b>what books are available</b>?  <b>A</b>. Go \u000A to the ACL2 home page, <v>http://www.cs.utexas.edu/u/moore/acl2/</v> and look at \u000A the link labeled ``Lemma Libraries and Utilities.''</p> \u000A \u000A <p><b>Q</b>. How do I <b>produce my own book</b>?  <b>A</b>. See <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p><b>Q</b>. What is a <b>decision procedure</b>?  What <b>decision procedures \u000A does ACL2 have</b>?  <b>A</b>. A <i>decision procedure</i> is an algorithm \u000A that given enough time and space can determine, for all the formulas in a \u000A certain syntactic class of formulas, whether the formula is a theorem or not. \u000A The most well-known decision procedure is for propositional calculus: by \u000A testing a formula under all the combinations Boolean assignments to the \u000A variables, one can decide if a propositional formula is a theorem.  The \u000A syntactic class consists of all formulas you can write using just variables, \u000A constants, and compositions of the functions <v>and</v>, <v>or</v>, <v>not</v>, \u000A <v>implies</v>, <v>iff</v>, and <v>if</v>.  There are, of course, an exponential \u000A number of different assignments so the algorithm can be slow.  ACL2 contains a \u000A propositional decision procedure based on symbolic normalization that can be \u000A faster than trying all the combinations of truthvalues -- but is not \u000A guaranteed to be faster.  ACL2 also contains an optional <see topic=\"ACL2____BDD\">bdd</see> procedure. \u000A ACL2 also contains a decision procedure for rational arithmetic involving \u000A variables, rational constants, addition, multiplication by rational constants, \u000A equality, and the various versions of arithmetic inequality (<v>&lt;</v>, <v>&lt;=</v>, \u000A <v>&gt;=</v>, and <v>&gt;</v>).  It can be extended with <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> lemmas. \u000A ACL2 is complete for equality over uninterpreted (e.g., undefined and \u000A unconstrained) function symbols using an algorithm based on transitive closure \u000A of equivalence classes under functional substitutivity.  Finally, you can make \u000A ACL2 use other decision procedures, even ones implemented outside of ACL2; see \u000A <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>.</p> \u000A \u000A <p><b>Q</b>. ACL2 has the <b>change of variable</b> trick (<b>destructor \u000A elimination</b>) that it does to get rid of <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> X)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> X)</v> by \u000A replacing <v>x</v> by <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> A B)</v>.  Is there a way to make ACL2 do that for \u000A other terms?  <b>A</b>. Yes.  See <see topic=\"ACL2____ELIM\">elim</see>.</p> \u000A \u000A <p><b>Q</b>. How can I <b>prevent destructor elimination</b>?  <b>A</b>. See \u000A <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:do-not '(eliminate-destructors)</v>.</p> \u000A \u000A <p><b>Q</b>. How can I <b>prevent cross-fertilization</b>?  <b>A</b>. See \u000A <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:do-not '(fertilize)</v>.</p> \u000A \u000A <p><b>Q</b>. How can I <b>prevent generalization</b>?  <b>A</b>. See <see topic=\"ACL2____HINTS\">hints</see>, specifically <v>:do-not '(<see topic=\"ACL2____GENERALIZE\">generalize</see>)</v>.</p> \u000A \u000A <p><b>Q</b>. How can I make ACL2 <b>impose a restriction on a new variable</b> \u000A introduced by destructor elimination or generalization?  <b>A</b>.  See <see topic=\"ACL2____GENERALIZE\">generalize</see>.</p> \u000A \u000A <p><b>Q</b>. What <b>rule classes</b> are there?  <b>A</b>. See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p><b>Q</b>. What is a <b>theory</b>?  <b>A</b>. See <see topic=\"ACL2____THEORIES\">theories</see>.</p> \u000A \u000A <p><b>Q</b>. How are <b>hints inherited by the children of a subgoal</b>? \u000A <b>A</b>. See <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>.</p> \u000A \u000A <p><b>Q</b>.  How do I use ACL2 under <b>Emacs</b>?  <b>A</b>. See <see topic=\"ACL2____EMACS\">emacs</see>.</p> \u000A \u000A <p><b>Q</b>.  How do I use ACL2 under <b>Eclipse</b>?  <b>A</b>. See <see topic=\"ACL2____ACL2-SEDAN\">ACL2-Sedan</see>.</p> \u000A \u000A <p><b>Q</b>.  How do I interrupt the prover?  <b>A</b>.  The keyboard sequence \u000A for interrupting a running process depends your operating system, host Common \u000A Lisp, and user interface (e.g., Emacs, Eclipse, etc.).  But perhaps a few \u000A examples will help you discover what you need to know.  If your host Common \u000A Lisp is GCL or Allegro and you are typing directly to the Common Lisp process, \u000A then you can interrupt a computation by typing ``ctrl-c'' (hold down the \u000A Control key and hit the ``c'' key once).  But other Lisps may require some \u000A other control sequence.  If you are typing to an Emacs process which is \u000A running the GCL or Allegro Common Lisp process in a shell buffer, you should \u000A type ctrl-c ctrl-c -- that is, you have to type the previously mentioned \u000A sequence twice in succession.  If you are running the ACL2 Sedan, you can use \u000A the <i>Interrupt Session</i> button on the tool bar.  The environment you \u000A enter when you interrupt depends on various factors and basically you should \u000A endeavor to get back to the top level ACL2 command loop, perhaps by typing \u000A some kind of Lisp depenent ``abort'' command like <v>A</v> or <v>:q</v>, or \u000A <v>:abort!</v>.  You can usually determine what environment you're in by paying \u000A attention to the prompt.</p> \u000A \u000A <p><b>Q</b>.  What is the <b>ACL2 loop</b>?  <b>A</b>.  That is the name given \u000A to the interactive environment ACL2 provides, a ``read-eval-print loop'' in \u000A which the user submits successive commands by typing ACL2 expressions and \u000A keyword commands.  See <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see>.</p> \u000A \u000A <p><b>Q</b>.  What is <b>raw lisp</b>?  <b>A</b>.  That is our name for the \u000A host Common Lisp in which ACL2 is implemented.  See <see topic=\"ACL2____INTRODUCTION-TO-A-FEW-SYSTEM-CONSIDERATIONS\">introduction-to-a-few-system-considerations</see>.  There is an ACL2 mode named \u000A <i>raw mode</i> which is different from ``raw lisp.''  See <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>.</p> \u000A \u000A <p><b>Q</b>.  Can I get a tree-like view of a proof?  <b>A</b>.  See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> for an Emacs utility that displays proof trees and allows you to \u000A navigate through a proof from the proof tree.  The ACL2 Sedan also supports \u000A proof trees and you should see the ACL2s documention on that topic.</p> \u000A \u000A <p><b>Q</b>.  I used the earlier Boyer-Moore theorem prover, Nqthm.  How is \u000A ACL2 different?  <b>A</b>. See <see topic=\"ACL2____NQTHM-TO-ACL2\">nqthm-to-ACL2</see>.</p> \u000A \u000A <p>If you are reading this as part of the tutorial introduction to the theorem \u000A prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____FAILED-FORCING":[["Force","Debugging"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see> for a background discussion of the notion \u000A of forcing rounds.  When a proof fails during a forcing round it means that \u000A the ``gist'' of the proof succeeded but some ``technical detail'' failed.  The \u000A first question you must ask yourself is whether the <see topic=\"ACL2____FORCE\">force</see>d goals are \u000A indeed theorems.  We discuss the possibilities below.</p> \u000A \u000A <p>If you believe the <see topic=\"ACL2____FORCE\">force</see>d goals are theorems, you should follow the \u000A usual methodology for ``fixing'' failed ACL2 proofs, e.g., the identification \u000A of key lemmas and their timely and proper use as rules.  See <see topic=\"ACL2____FAILURE\">failure</see>, \u000A see <see topic=\"ACL2____GAG-MODE\">gag-mode</see>, and see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>The rules designed for the goals of forcing rounds are often just what is \u000A needed to prove the <see topic=\"ACL2____FORCE\">force</see>d hypothesis at the time it is <see topic=\"ACL2____FORCE\">force</see>d. \u000A Thus, you may find that when the system has been ``taught'' how to prove the \u000A goals of the forcing round no forcing round is needed.  This is intended as a \u000A feature to help structure the discovery of the necessary rules.</p> \u000A \u000A <p>If a hint must be provided to prove a goal in a forcing round, the \u000A appropriate ``goal specifier'' (the string used to identify the goal to which \u000A the hint is to be applied) is just the text printed on the line above the \u000A formula, e.g., <v>&quot;[1]Subgoal *1/3''&quot;</v>.  See <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>.</p> \u000A \u000A <p>If you solve a forcing problem by giving explicit <see topic=\"ACL2____HINTS\">hints</see> for the \u000A goals of forcing rounds, you might consider whether you could avoid forcing \u000A the assumption in the first place by giving those <see topic=\"ACL2____HINTS\">hints</see> in the \u000A appropriate places of the main proof.  This is one reason that we print out \u000A the origins of each <see topic=\"ACL2____FORCE\">force</see>d assumption.  An argument against this style, \u000A however, is that an assumption might be <see topic=\"ACL2____FORCE\">force</see>d in hundreds of places in \u000A the main goal and proved only once in the forcing round, so that by delaying \u000A the proof you actually save time.</p> \u000A \u000A <p>We now turn to the possibility that some goal in the forcing round is not a \u000A theorem.</p> \u000A \u000A <p>There are two possibilities to consider.  The first is that the original \u000A theorem has insufficient hypotheses to ensure that all the <see topic=\"ACL2____FORCE\">force</see>d \u000A hypotheses are in fact always true.  The ``fix'' in this case is to amend the \u000A original conjecture so that it has adequate hypotheses.</p> \u000A \u000A <p>A more difficult situation can arise and that is when the conjecture has \u000A sufficient hypotheses but they are not present in the forcing round goal. \u000A This can be caused by what we call ``premature'' forcing.</p> \u000A \u000A <p>Because ACL2 rewrites from the inside out, it is possible that it will \u000A <see topic=\"ACL2____FORCE\">force</see> hypotheses while the context is insufficient to establish them. \u000A Consider trying to prove <v>(p x (foo x))</v>.  We first rewrite the formula in \u000A an empty context, i.e., assuming nothing.  Thus, we rewrite <v>(foo x)</v> in an \u000A empty context.  If rewriting <v>(foo x)</v> <see topic=\"ACL2____FORCE\">force</see>s anything, that <see topic=\"ACL2____FORCE\">force</see>d assumption will have to be proved in an empty context.  This will \u000A likely be impossible.</p> \u000A \u000A <p>On the other hand, suppose we did not attack <v>(foo x)</v> until after we \u000A had expanded <v>p</v>.  We might find that the value of its second argument, \u000A <v>(foo x)</v>, is relevant only in some cases and in those cases we might be \u000A able to establish the hypotheses <see topic=\"ACL2____FORCE\">force</see>d by <v>(foo x)</v>.  Our premature \u000A forcing is thus seen to be a consequence of our ``over eager'' rewriting.</p> \u000A \u000A <p>Here, just for concreteness, is an example you can try.  In this example, \u000A <v>(foo x)</v> rewrites to <v>x</v> but has a <see topic=\"ACL2____FORCE\">force</see>d hypothesis of \u000A <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</v>.  <v>P</v> does a case split on that very hypothesis and uses \u000A its second argument only when <v>x</v> is known to be rational.  Thus, the \u000A hypothesis for the <v>(foo x)</v> rewrite is satisfied.  On the false branch of \u000A its case split, <v>p</v> simplies to <v>(p1 x)</v> which can be proved under the \u000A assumption that <v>x</v> is not rational.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p (x y)(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y) (p1 x)))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-rewrite (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____FORCE\">force</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x) x)))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> foo))</code> \u000A \u000A <p>The attempt then to do <v>(<see topic=\"ACL2____THM\">thm</see> (p x (foo x)))</v> <see topic=\"ACL2____FORCE\">force</see>s the \u000A unprovable goal <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</v>.</p> \u000A \u000A <p>Since all ``formulas'' are presented to the theorem prover as single terms \u000A with no hypotheses (e.g., since <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> is a function), this problem \u000A would occur routinely were it not for the fact that the theorem prover expands \u000A certain ``simple'' definitions immediately without doing anything that can \u000A cause a hypothesis to be <see topic=\"ACL2____FORCE\">force</see>d.  See <see topic=\"ACL2____SIMPLE\">simple</see>.  This does not \u000A solve the problem, since it is possible to hide the propositional structure \u000A arbitrarily deeply.  For example, one could define <v>p</v>, above, recursively \u000A so that the test that <v>x</v> is rational and the subsequent first ``real'' use \u000A of <v>y</v> occurred arbitrarily deeply.</p> \u000A \u000A <p>Therefore, the problem remains: what do you do if an impossible goal is \u000A <see topic=\"ACL2____FORCE\">force</see>d and yet you know that the original conjecture was adequately \u000A protected by hypotheses?</p> \u000A \u000A <p>One alternative is to disable forcing entirely.  See <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>.  Another is to <see topic=\"ACL2____DISABLE\">disable</see> the rule that caused the <see topic=\"ACL2____FORCE\">force</see>.</p> \u000A \u000A <p>A third alternative is to prove that the negation of the main goal implies \u000A the <see topic=\"ACL2____FORCE\">force</see>d hypothesis.  For example,</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> not-p-implies-rationalp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (p x (foo x))) (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Observe that we make no rules from this formula.  Instead, we merely \u000A <v>:use</v> it in the subgoal where we must establish <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (p x (foo x))\u000A     :hints ((&quot;Goal&quot; :use not-p-implies-rationalp)))</code> \u000A \u000A <p>When we said, above, that <v>(p x (foo x))</v> is first rewritten in an empty \u000A context we were misrepresenting the situation slightly.  When we rewrite a \u000A literal we know what literal we are rewriting and we implicitly assume it \u000A false.  This assumption is ``dangerous'' in that it can lead us to simplify \u000A our goal to <v>nil</v> and give up &mdash; we have even seen people make the \u000A mistake of assuming the negation of what they wished to prove and then via a \u000A very complicated series of transformations convince themselves that the \u000A formula is false.  Because of this ``tail biting'' we make very weak use of \u000A the negation of our goal.  But the use we make of it is sufficient to \u000A establish the <see topic=\"ACL2____FORCE\">force</see>d hypothesis above.</p> \u000A \u000A <p>A fourth alternative is to weaken your desired theorem so as to make \u000A explicit the required hypotheses, e.g., to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> rationalp-implies-main\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (p x (foo x)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>This of course is unsatisfying because it is not what you originally \u000A intended.  But all is not lost.  You can now prove your main theorem from this \u000A one, letting the <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> here provide the necessary case split.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (p x (foo x))\u000A     :hints ((&quot;Goal&quot; :use rationalp-implies-main)))</code>"],

 "ACL2____DEALING-WITH-TAU-PROBLEMS":[["Introduction-to-the-tau-system"],"ACL2 Sources","ACL2","<p>For background on the tau system, see <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>.  The two most common problems caused by the \u000A tau system have to do with the system's interaction with ``legacy'' proof \u000A scripts.  Such scripts may suffer because they were not designed to exploit \u000A tau reasoning and which may configure the tau database in quite incomplete and \u000A arbitrary ways.  The two most common problems we have seen are (a) significant \u000A slow downs in a few proofs and (b) failed proof attempts due to hints being \u000A misapplied because the tau system caused subgoals to be renumbered.</p> \u000A \u000A <p>We discuss the rather limited means of dealing with these problems here. \u000A In <see topic=\"ACL2____FUTURE-WORK-RELATED-TO-THE-TAU-SYSTEM\">future-work-related-to-the-tau-system</see> we list some major \u000A inadequacies of the tau system.</p> \u000A \u000A <p>If the tau system contributes to a proof, the <see topic=\"ACL2____RUNE\">rune</see> <v>(:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt><v> tau-system)</v> will be listed among the Rules in the \u000A Summary.  However, merely by being attempted the tau system can slow down \u000A proofs in which it makes no contribution.</p> \u000A \u000A <p>The most brutal and fool-proof way to isolate a proof from the tau system \u000A is to disable the entire system.  This can be done globally by</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>)))  ; (:executable-counterpart tau-system)</code> \u000A \u000A <p>or locally with a subgoal specific hint:</p> \u000A \u000A <code>...\u000A:hints ((&quot;...subgoal id...&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>))))</code> \u000A \u000A <p>Conducting a proof with and without the participation of the tau system can \u000A help you determine whether tau reasoning is helping or hurting.</p> \u000A \u000A <p><i>Dealing with Slowdowns</i></p> \u000A \u000A <p>The <tt><see topic=\"ACL2____TIME-TRACKER\">time-tracker</see></tt> utility was added to allow users to investigate \u000A whether excessive amounts of time are being spent in a given function.  It was \u000A then used to annotate the code for the tau system as described in <see topic=\"ACL2____TIME-TRACKER-TAU\">time-tracker-tau</see>.  The result is that if ``excessive'' time is spent in tau \u000A reasoning, messages to that effect will be printed to the proof log.  The \u000A question is: aside from disabling the tau system how can the proof be sped \u000A up?</p> \u000A \u000A <p>There are two common causes of slowdown in the tau system.  The first stems \u000A from the system's use of <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt>s to determine \u000A whether a constant has a given tau.  Recall that a tau is a conjunction of \u000A monadic predicates.  To determine whether some constant satisfies the tau, the \u000A predicates are executed.  If you have a hard-to-compute predicate this can be \u000A very slow.  The most typical such predicates in ACL2 applications are those \u000A that check invariants, e.g., that recognize ``good states'' or ``well-formed \u000A data.''  These are often written inefficiently because they are intended only \u000A for used in theorems and, before the tau system was added, they may have never \u000A been applied to constants.  The most common constants tau predicates are \u000A applied to are <v>0</v>, <v>T</v>, and <v>NIL</v>, although different models may \u000A stress other constants.  To understand why <v>NIL</v> for example is frequently \u000A tested, if the test of an <v>IF</v>-expression is computed to have tau <i>s</i> \u000A then the next question we ask is ``does <v>nil</v> satisfy <i>s</i>?''</p> \u000A \u000A <p>You may determine whether the tau system is spending time executing tau \u000A predicates by observing the rewriter &mdash; see <see topic=\"ACL2____DMR\">dmr</see> &mdash; or by \u000A interrupting the system and getting a backtrace (see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>).</p> \u000A \u000A <p>If excessive time is being spent in a tau predicate, a draconian solution \u000A is to disable the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of that predicate, for \u000A example in either of these equivalent ways.  The tau system does not execute \u000A disabled <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt>s.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart foo)))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (foo)))</code> \u000A \u000A <p>In either case above, you may prefer to provide local <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> rather than <v>:in-theory</v> <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>Disabling the executable counterpart of expensive tau predicates will \u000A weaken the tau system, probably only negligibly, because it can no longer run \u000A the predicates to determine whether they admits given constants.</p> \u000A \u000A <p>A more sophisticated solution is to make the tau system record values of \u000A the <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode function in question, so that the system will \u000A look up the necessary values rather than running the function every time the \u000A question arises.  It will look up recorded values whether the executable \u000A counterpart of the tau predicate is enabled or disabled.  Here is an example \u000A of a lemma that can provide such a solution.  See the discussion of the \u000A <i>Eval</i> form of <v>:</v><tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> rules.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (foo 0)\u000A       (foo 17)\u000A       (foo t)\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (foo '(a b c))))\u000A  :rule-classes :tau-system)</code> \u000A \u000A <p>It might be difficult to determine <i>which</i> constants are being \u000A repeatedly tested, although tracing (<tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>) suspected tau predicates \u000A will show what they are being called on.</p> \u000A \u000A <p>At the moment there are no better user-level tools to discover this. \u000A However, some users may wish to consider the following hack: In the ACL2 \u000A source file <v>tau.lisp</v>, immediately after the definition of the system \u000A function <v>ev-fncall-w-tau-recog</v>, there is a comment which contains some \u000A raw Lisp code that can be used to investigate whether tau's use of evaluation \u000A on constants is causing a problem and to determine which constants are \u000A involved.</p> \u000A \u000A <p>The second main cause of slowdowns by the tau system is that the system \u000A contains ``too many'' conjunctive rules (see the <i>Conjunctive</i> form in \u000A <tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>).  Unfortunately, we have no tools for either identifying \u000A the problem or addressing it!  That said, let us tell you what we do know!</p> \u000A \u000A <p>Conjunctive rules are used to ``complete'' each tau as it is built. \u000A Referring to the <v>weekdayp</v> example in <tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>, if a tau is \u000A constructed that recognizes weekdays but not <v>MON</v>, <v>TUE</v>, <v>THU</v>, or \u000A <v>FRI</v>, it is completed by adding that the tau recognizes (only) <v>WED</v>. \u000A This means that when we construct a tau we scan all known conjunctive rules \u000A and see whether all but one of the literals of any conjunctive rule are \u000A present.  This can be expensive.  To mitigate this expense, the tau system \u000A caches the computation on a per proof basis (the cache is cleared after every \u000A proof).</p> \u000A \u000A <p>To learn what conjunctive rules there are in your system, evaluate</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> 'tau-conjunctive-rules (<see topic=\"ACL2____TAU-DATABASE\">tau-database</see> (w state)))</code> \u000A \u000A <p>Perhaps by sending the implementors that list, we can think of ways to \u000A index the conjunctive rules to save time.</p> \u000A \u000A <p><i>Dealing with Misapplied Hints</i></p> \u000A \u000A <p>The second common problem caused by the tau system in legacy proof scripts \u000A is that it can cause subgoals to be renumbered and thus cause hints to be \u000A missed.  The only ways to address this problem is either to disable the tau \u000A system (locally or globally by disabling <v>(:executable-counterpart\u000A tau-system)</v>) or change the legacy hints to use the new subgoal names.</p>"],

 "ACL2____BOOKDATA":[["Books"],"ACL2 Sources","ACL2","<p>ACL2 provides a primitive capability for writing out a file of data \u000A associated with a book.  This information might be useful, for example, in \u000A building a database that allows you to search for name conflicts.  See <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> directory <v>books/tools/book-conflicts/</v> for an application \u000A of this capability by Dave Greve.  If you use this capability and have ideas \u000A for enhancing it, please feel free to send them to the ACL2 developers.</p> \u000A \u000A <p>If the book has the name <v>BK</v>, then the output file is named \u000A <v>BK__bookdata.out</v>.  That file is generated in the same directory as \u000A <v>BK</v>, by certifying <v>BK</v> when <see topic=\"ACL2____STATE\">state</see> global <v>'write-bookdata</v> \u000A has a non-<v>nil</v> value, for example as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> write-bookdata t)\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;BK&quot; ...)</code> \u000A \u000A <p>The resulting file will contain a single form of the following shape, \u000A although not necessarily in the following order, according to the description \u000A that follows below.</p> \u000A \u000A <code>(&quot;...BK.lisp&quot;\u000A :PKGS          pkgs-val\u000A :BOOKS         book-val\u000A :PORT-BOOKS    port-book-val\u000A :CONSTS        consts-val\u000A :PORT-CONSTS   port-consts-val\u000A :FNS           fns-val\u000A :PORT-FNS      port-fns-val\u000A :LABELS        labels-val\u000A :PORT-LABELS   port-labels-val\u000A :MACROS        macros-val\u000A :PORT-MACROS   port-macros-val\u000A :STOBJS        stobjs-val\u000A :PORT-STOBJS   port-stobjs-val\u000A :THEORIES      theories-val\u000A :PORT-THEORIES port-theories-val\u000A :THMS          thms-val\u000A :PORT-THMS     port-thms-val</code>) \u000A \u000A <p>The first entry in the form will always be the full book name (see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>) of the certified book, <v>BK</v>.</p> \u000A \u000A <p>Subsequent values in the form are based on <see topic=\"ACL2____EVENTS\">events</see> introduced by \u000A including <v>BK</v>.  For various values of <v>xxx</v> as described below, \u000A <v>port-</v><i>xxx</i><v>-val</v> is a list of values corresponding to <see topic=\"ACL2____EVENTS\">events</see> introduced in the certification <see topic=\"ACL2____WORLD\">world</see> for <v>BK</v> (see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>), and <i>xxx</i><v>-val</v> is a list of values corresponding to \u000A <see topic=\"ACL2____EVENTS\">events</see> introduced non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly by <v>BK</v>.  These lists include \u000A only ``top-level'' events, not those that are introduced by a book included \u000A either in <v>BK</v> or its certification world.</p> \u000A \u000A <p><v>pkgs-val</v> is a list of names of packages introduced in the \u000A certification world (at the top level, not in an included book).  Note that no \u000A packages are introduced in a book itself, so no distinction is made between \u000A <v>pkgs-val</v> and <v>port-pkgs-val</v>.  Both <v>port-book-val</v> and \u000A <v>book-val</v> are lists of full book names (see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>) of \u000A included books.  The values associated with the other keywords are, \u000A themselves, association lists (see <see topic=\"ACL2____ALISTP\">alistp</see>) such that each key is a \u000A package name, which is associated with a list of <see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see>s for \u000A symbols in that package that are introduced for that keyword.  For example, \u000A <v>fns-val</v> may be the alist</p> \u000A \u000A <code>((&quot;ACL2&quot; &quot;F1&quot; &quot;F2&quot;)\u000A (&quot;MY-PKG&quot; &quot;G1&quot; &quot;G2&quot;))</code> \u000A \u000A <p>if the function symbols introduced in the book are <v>F1</v> and <v>F2</v> in \u000A the <v>&quot;ACL2&quot;</v> package, as well as <v>G1</v> and <v>G2</v> in the \u000A <v>&quot;MY-PKG&quot;</v> package.</p> \u000A \u000A <p>We next explain what kinds of symbols are introduced for each keyword \u000A <v>:xxx</v>.  Each such symbol would be associated with either the keyword \u000A <v>:port-xxx</v> or the keyword <v>:xxx</v> depending respectively on whether the \u000A symbol is introduced at the top level of the certification world for <v>BK</v> \u000A or <v>BK</v> itself.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o <v>:CONSTS</v><br></br> \u000A \u000A constant symbol introduced by <v>defconst</v></p> \u000A \u000A <p>o <v>:FNS</v><br></br> \u000A \u000A function symbol: introduced by <v>defun</v>, <v>defuns</v>, or <v>defchoose</v>; or \u000A constrained (by an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event)</p> \u000A \u000A <p>o <v>:LABELS</v><br></br> \u000A \u000A symbol introduced by <v>deflabel</v></p> \u000A \u000A <p>o <v>:MACROS</v><br></br> \u000A \u000A macro name introduced by <v>defmacro</v></p> \u000A \u000A <p>o <v>:STOBJS</v><br></br> \u000A \u000A <v>stobj</v> name introduced by <v>defstobj</v> or <v>defabsstobj</v></p> \u000A \u000A <p>o <v>:THEORIES</v><br></br> \u000A \u000A theory name introduced by <v>deftheory</v></p> \u000A \u000A <p>o <v>:THMS</v><br></br> theorem name, which may be introduced by <v>defthm</v> \u000A  or a macro call expanding to a call of <v>defthm</v>, such as see <see topic=\"ACL2____DEFEQUIV\">defequiv</see> or <v>defaxiom</v>; but may be introduced by <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, for \u000A  example, with name <v>&quot;MYPKG-PACKAGE&quot;</v> if the package name is \u000A  <v>&quot;MYPKG&quot;</v></p></blockquote> \u000A \u000A <p>Our hope is that people in the ACL2 community will generate and use this \u000A data to improve the ACL2 <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  Here is an example \u000A illustrating how to generate bookdata files for those books as a byproduct of \u000A a regression run.  Below, we write <v>{DIR}</v> as an abbreviation for the ACL2 \u000A sources directory, and assume that this command is run from that directory. \u000A Of course, you may wish to use <v>make</v> options like <v>-j 8</v> and <v>make</v> \u000A variable settings like <v>ACL2={DIR}/my-saved_acl2</v>; see <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see> for details.</p> \u000A \u000A <code>make regression-fresh \\\u000AACL2_CUSTOMIZATION={DIR}/acl2-customization-files/bookdata.lisp</code>"],

 "ACL2____ADVANCED-FEATURES":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Maybe you've been using ACL2 for awhile, and you wonder if there \u000A are lesser-known features that you might find useful.  Then this topic is for \u000A you.  We present below a ``laundry list'' of some such features, with brief \u000A descriptions and links to <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topics.</p> \u000A \u000A <p>Although the list below is long, it is not intended to be complete, and \u000A indeed some topics have been deliberately excluded.  Some have fallen out of \u000A use, perhaps for good reason, such as <see topic=\"ACL2____NU-REWRITER\">nu-rewriter</see> and <see topic=\"ACL2____OBDD\">obdd</see>. \u000A Others are already likely to be discovered when needed, such as <tt><see topic=\"ACL2____GETENV_42\">getenv$</see></tt> and perhaps <tt><see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see></tt>.  Some topics are referenced by \u000A documentation for others in the list, such as <tt><see topic=\"ACL2____MBT\">mbt</see></tt>, which is referenced \u000A by <tt><see topic=\"ACL2____MBE\">mbe</see></tt>.  Some utilities such as <tt><see topic=\"ACL2____PSTACK\">pstack</see></tt> and <tt><see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see></tt> seem too low-level to be worthy of inclusion below.</p> \u000A \u000A <p>For an extensive introduction to using the prover, which may include some \u000A aspects new to you, see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.  A shorter \u000A topic contains highlights for efficient prover usage: see <see topic=\"ACL2____TIPS\">tips</see>.  Also \u000A see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see> for an extension of ACL2 (written by others), ACL2s, \u000A that includes an Eclipse-based interface, more powerful and automatic \u000A termination reasoning, and other features.</p> \u000A \u000A <p>We now move on to the list.</p> \u000A \u000A <code>========================================\u000ATop-level commands and utilities:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____A_12\">a!</see> and see <see topic=\"ACL2____P_12\">p!</see> to abort or pop.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see> for initial commands to run at startup.</p> \u000A \u000A <p>o See <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see> for how keyword commands are processed.</p> \u000A \u000A <p>o See <see topic=\"ACL2____LD\">ld</see> for many ways to control the top-level loop.</p> \u000A \u000A <p>o See <see topic=\"ACL2____COMPILATION\">compilation</see> for a discussion of <v>set-compiler-enabled</v> and \u000A other compiler-related utilities.</p> \u000A \u000A <p>o For useful reader macros `<v>#!</v>', `<v>#.</v>', and `<v>#u</v>', see <see topic=\"ACL2____SHARP-BANG-READER\">sharp-bang-reader</see>, see <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>, and see <see topic=\"ACL2____SHARP-U-READER\">sharp-u-reader</see>.</p> \u000A \u000A <p>o To save and use an ACL2 executable, see <see topic=\"ACL2____ACL2-AS-STANDALONE-PROGRAM\">ACL2-as-standalone-program</see> \u000A and see <see topic=\"ACL2____SAVE-EXEC\">save-exec</see>.</p> \u000A \u000A <p>o For utilities related to timing, see <see topic=\"ACL2____TIME_42\">time$</see>, see <see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>, see <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>, and see <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see>.</p> \u000A \u000A <p>o To query and manage the database, see <see topic=\"ACL2____HISTORY\">history</see> (which discusses \u000A many useful utilities, such as <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> and <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>), and \u000A see <see topic=\"ACL2____DEAD-EVENTS\">dead-events</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> for linking keyword for <v>:dir</v> \u000A argument of <tt><see topic=\"ACL2____LD\">ld</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____REBUILD\">rebuild</see> for a fast way to load a file without waiting for \u000A proofs.</p> \u000A \u000A <p>o For parallel certification, see <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see> for use of the \u000A <v>-j</v> option of `make'; also see <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.</p></blockquote> \u000A \u000A <code>========================================\u000ASome relatively less common events:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> to reset the prehistory.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> to assert that a given form returns a non-nil \u000A value.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFATTACH\">defattach</see> to execute constrained functions using corresponding \u000A attached functions.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> to define a function whose body has an outermost \u000A quantifier.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFCHOOSE\">defchoose</see> to define a Skolem (witnessing) function.</p> \u000A \u000A <p>o For efficiency consider using <v>defconst-fast</v>; see <see topic=\"ACL2____DEFCONST\">defconst</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> to specify when <see topic=\"ACL2____GUARD\">guard</see> \u000A verification is tried by default.</p></blockquote> \u000A \u000A <code>======================================== \u000AOutput and its control (see <see topic=\"ACL2____IO\">io</see> for additional information): \u000A======================================== \u000A</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____WITH-OUTPUT\">with-output</see> to suppress or turn on specified \u000A output for an event.</p> \u000A \u000A <p>o See <see topic=\"ACL2____EVISC-TABLE\">evisc-table</see> for support for abbreviated output.</p> \u000A \u000A <p>o See <see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see> for a table used to associate names for \u000A <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt>/<tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt> printing.</p> \u000A \u000A <p>o See <see topic=\"ACL2____OUTPUT-TO-FILE\">output-to-file</see> to redirect output to a file.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PRINT-CONTROL\">print-control</see> to control ACL2 printing.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> to control suppression of details when \u000A printing.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> to control output by type.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-IPRINT\">set-iprint</see> to allow abbreviated output to be read back in.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see> (also <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> and <see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see>) to control the radix in which numbers are printed.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> to control whether symbols are printed in upper \u000A case or in lower case.</p></blockquote> \u000A \u000A <code>========================================\u000AOn proving termination for definitions:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____ORDINALS\">ordinals</see> for a discussion of ordinals in ACL2.</p> \u000A \u000A <p>o See <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> for a control on ACL2's termination and \u000A induction analyses.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> to set the default well-founded \u000A relation for termination analysis.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see> for a related tool that provides extra automation \u000A for termination proofs.</p></blockquote> \u000A \u000A <code>========================================\u000AProof debugging and output control:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> to get statistics on which \u000A runes are being tried.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> and see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> to associate a \u000A function name with a macro name.</p> \u000A \u000A <p>o See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> for how to monitor rewrite rules.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DMR\">dmr</see> for dynamic monitoring of rewriting and other prover \u000A activity.</p> \u000A \u000A <p>o See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> to see reports about the forward \u000A chaining process.</p> \u000A \u000A <p>o See <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see> to generate markers to indicate sources of <see topic=\"ACL2____GUARD\">guard</see> proof obligations.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> for support for low-level interaction.</p> \u000A \u000A <p>o See <see topic=\"ACL2____REDO-FLAT\">redo-flat</see> for redo on failure of a <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> and see <see topic=\"ACL2____PSO\">pso</see> to abbreviate or restore proof \u000A output.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>, see <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>, and \u000A see <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see> to inhibit various types of output.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-RAW-PROOF-FORMAT\">set-raw-proof-format</see> to make proof output display lists of \u000A <see topic=\"ACL2____RUNE\">rune</see>s.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> to skip proofs for a given form.</p></blockquote> \u000A \u000A <code>========================================\u000AProgram debugging:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____BREAK_42\">break$</see> to cause an immediate Lisp break.</p> \u000A \u000A <p>o See <see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> to break when encountering a hard or soft error \u000A caused by ACL2.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> to disassemble a function.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PRINT-GV\">print-gv</see> to print a form whose evaluation caused a guard \u000A violation.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PROFILE\">profile</see> to turn on profiling for one function.</p> \u000A \u000A <p>o See <see topic=\"ACL2____TRACE_42\">trace$</see> and see <see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see> to <see topic=\"COMMON-LISP____TRACE\">trace</see> function \u000A evaluations, possibly sending trace output to a file.</p> \u000A \u000A <p>o See <see topic=\"ACL2____WET\">wet</see> to evaluate a form and print a subsequent error \u000A trace.</p></blockquote> \u000A \u000A <code>======================================== \u000AProgramming and evaluation idioms, support, utilities (also see <see topic=\"ACL2____PROGRAMMING\">programming</see> \u000Afor more utilities, e.g., <tt><see topic=\"ACL2____RANDOM_42\">random$</see></tt>) \u000A======================================== \u000A</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____ARRAYS\">arrays</see> and See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see> for introductions \u000A to ACL2 arrays and single-threaded objects (stobjs), respectively, each of \u000A which provides efficient destructive operations in an applicative setting. \u000A Also see <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see> for a way to create local stobjs.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ASSERT_42\">assert$</see> to cause a hard error if the given test is false.</p> \u000A \u000A <p>o See <see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> to obtain the true absolute filename, with \u000A soft links resolved.</p> \u000A \u000A <p>o See <see topic=\"ACL2____CASE-MATCH\">case-match</see> for a utility providing pattern matching and \u000A destructuring.</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFPUN\">defpun</see> to define a tail-recursive function symbol.</p> \u000A \u000A <p>o See <see topic=\"ACL2____EC-CALL\">ec-call</see> to execute a call in the ACL2 logic instead of raw \u000A Lisp.</p> \u000A \u000A <p>o See <see topic=\"ACL2____ER\">er</see> to print an error message and ``cause an error''.</p> \u000A \u000A <p>o See <see topic=\"COMMON-LISP____FLET\">flet</see> to provide local binding of function symbols.</p> \u000A \u000A <p>o See <see topic=\"ACL2____GC_42\">gc$</see> to invoke the garbage collector.</p> \u000A \u000A <p>o See <see topic=\"ACL2____MBE\">mbe</see> to attach code for execution.</p> \u000A \u000A <p>o See <see topic=\"ACL2____MV-LIST\">mv-list</see> to convert a multiple-valued result to a single-valued \u000A list.</p> \u000A \u000A <p>o See <see topic=\"ACL2____MV_F3\">mv?</see> to return one or more values.</p> \u000A \u000A <p>o For non-executable code, see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see> and see <see topic=\"ACL2____NON-EXEC\">non-exec</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PROG2_42\">prog2$</see> and see <see topic=\"ACL2____PROGN_42\">progn$</see> to execute two or more forms and \u000A return the value of the last one.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see> for how to program using the von \u000A Neumannesque ACL2 <see topic=\"ACL2____STATE\">state</see> object.</p> \u000A \u000A <p>o See <see topic=\"ACL2____TOP-LEVEL\">top-level</see> to evaluate a top-level form as a function body.</p> \u000A \u000A <p>o See <see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> to suppress or enable guard-checking for a \u000A form.</p> \u000A \u000A <p>o For ways to fake access to the state see <see topic=\"ACL2____WORMHOLE\">wormhole</see>, see <see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see>, see <see topic=\"ACL2____CW\">cw</see>, see <see topic=\"ACL2____CW_12\">cw!</see>, see <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>, see <see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see>, and (dangerous!) see <see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see>.</p></blockquote> \u000A \u000A <code>========================================\u000AConnecting with the underlying host Lisp, and doing other evil:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____DEFTTAG\">defttag</see> to introduce a trust tag (ttag).</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see> to define a macro that returns its last \u000A argument, but with side effects.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PROGN_12\">progn!</see> to evaluate forms that are not necessarily <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____RETURN-LAST\">return-last</see> to return the last argument, perhaps with side \u000A effects.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> to enter or exit ``raw mode,'' a raw Lisp \u000A environment.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SYS-CALL\">sys-call</see> and <see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see> to make a system call to the \u000A host operating system.</p></blockquote> \u000A \u000A <code>========================================\u000AMacros and related utilities:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____DEFABBREV\">defabbrev</see> for a convenient form of macro \u000A definition for simple expansions.</p> \u000A \u000A <p>o See <see topic=\"ACL2____MACRO-ARGS\">macro-args</see> for the formals list of a macro definition (see \u000A <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>).</p> \u000A \u000A <p>o See <see topic=\"ACL2____MAKE-EVENT\">make-event</see> for a sort of extension of <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> that \u000A allows access to the <see topic=\"ACL2____STATE\">state</see>, by evaluating (expanding) a given form and \u000A then evaluate the result of that expansion.</p> \u000A \u000A <p>o See <see topic=\"ACL2____TRANS\">trans</see>, see <see topic=\"ACL2____TRANS_12\">trans!</see>, and see <see topic=\"ACL2____TRANS1\">trans1</see> to print the \u000A macroexpansion of a form.</p></blockquote> \u000A \u000A <code>========================================\u000AExperimental extensions:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see> for ACL2(h); in particular, \u000A see <see topic=\"ACL2____MEMOIZE\">memoize</see> for efficient function memoization and see <see topic=\"ACL2____PROFILE\">profile</see> \u000A for profiling.</p> \u000A \u000A <p>o See <see topic=\"COMMON-LISP____REAL\">real</see> for ACL2(r), which supports the real numbers.</p> \u000A \u000A <p>o See <see topic=\"ACL2____PARALLELISM\">parallelism</see> for ACL2(p), which supports parallel evaluation \u000A and proof.</p></blockquote> \u000A \u000A <code>========================================\u000ADatabase control and query:\u000A========================================</code> \u000A \u000A <blockquote><p>o See <see topic=\"ACL2____DISABLEDP\">disabledp</see> to determine whether a given name or \u000A rune is disabled.</p> \u000A \u000A <p>o For redefinition support see <see topic=\"ACL2____REDEF\">redef</see>, see <see topic=\"ACL2____REDEF_12\">redef!</see>, see <see topic=\"ACL2____REDEF_B2\">redef+</see>, see <see topic=\"ACL2____REDEF-\">redef-</see>, and see <see topic=\"ACL2____REDEFINED-NAMES\">redefined-names</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____TABLE\">table</see> for user-managed tables.</p> \u000A \u000A <p>o See <see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> to view a guard proof obligation without \u000A doing the proof.</p></blockquote> \u000A \u000A <code>========================================\u000AProver control\u000A========================================</code> \u000A \u000A <blockquote><p>o For congruence-based reasoning see <see topic=\"ACL2____DEFCONG\">defcong</see>, see <see topic=\"ACL2____CONGRUENCE\">congruence</see>, see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>, see <see topic=\"ACL2____DEFEQUIV\">defequiv</see>, and see <see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see>.</p> \u000A \u000A <p>o For meta rules and clause processors see <see topic=\"ACL2____META\">meta</see>, see <see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see>, see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>, see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see> (for connecting with external tools, such as \u000A SAT solvers), and See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see> (for <see topic=\"ACL2____STATE\">state</see> and \u000A context-sensitive metafunctions).</p> \u000A \u000A <p>o For theory control, see <see topic=\"ACL2____THEORIES\">theories</see> for detailed information, but in \u000A particular see <see topic=\"ACL2____DEFTHEORY\">deftheory</see>, see <see topic=\"ACL2____THEORY-FUNCTIONS\">theory-functions</see>, see <see topic=\"ACL2____IN-ARITHMETIC-THEORY\">in-arithmetic-theory</see> (and see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>), and see <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____HINTS\">hints</see> for a complete list of prover hints, including some of \u000A the more obscure ones such as <v>:restrict</v>, <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt>, \u000A <v>:nonlinearp</v>, <v>:backchain-limit-rw</v>, <v>:reorder</v>, and \u000A <v>:backtrack</v>.  Also see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see> for an explanation \u000A of how hints interact with the ACL2 proof process.  For other topics related \u000A to hints, see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, see <see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see>, see \u000A <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>, and see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see> and <see topic=\"ACL2____USING-COMPUTED-HINTS\">using-computed-hints</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____BIND-FREE\">bind-free</see> to bind <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> of a <see topic=\"ACL2____REWRITE\">rewrite</see> or \u000A <see topic=\"ACL2____LINEAR\">linear</see> rule.</p> \u000A \u000A <p>o See <see topic=\"ACL2____CASE-SPLIT\">case-split</see> for a utility like <tt><see topic=\"ACL2____FORCE\">force</see></tt> that immediately \u000A splits the top-level goal on the indicated hypothesis.</p> \u000A \u000A <p>o See <see topic=\"ACL2____CASE-SPLIT-LIMITATIONS\">case-split-limitations</see> for a way to the number of cases \u000A produced at once</p> \u000A \u000A <p>o See <see topic=\"ACL2____DEFAULT-BACKCHAIN-LIMIT\">default-backchain-limit</see> to specify the backchain limit for a \u000A rule.</p> \u000A \u000A <p>o See <see topic=\"ACL2____FORCE\">force</see> for an identity function used to force a hypothesis.</p> \u000A \u000A <p>o See <see topic=\"ACL2____OTF-FLG\">otf-flg</see> for a way to push more than one initial subgoal for \u000A induction.</p> \u000A \u000A <p>o See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> to add various kinds of rules to the database, \u000A including more unusual sorts such as <v>:</v><tt><see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see></tt> rules and \u000A <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> rules.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see> to set the backchain-limit used by the \u000A type-set and rewriting mechanisms.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-BODY\">set-body</see> to set an alternate definition body for <v>:expand</v> \u000A <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see> to set the <see topic=\"ACL2____REWRITE\">rewrite</see> stack depth \u000A used by the rewriter.</p> \u000A \u000A <p>o See <see topic=\"ACL2____SYNTAXP\">syntaxp</see> to attach a heuristic filter on a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>, or <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule.</p></blockquote> \u000A \u000A "],

 "ACL2____ADD-INCLUDE-BOOK-DIR":[["Books-reference"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A\u000A; For (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot; :dir :smith), prepend &quot;/u/smith/&quot; to &quot;foo&quot;.\u000A(<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> :smith &quot;/u/smith/&quot;)\u000A\u000A; For (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;bar&quot; :dir :util), prepend absolute directory pathname\u000A; corresponding to the relative pathname, &quot;utilities/&quot;.\u000A(<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> :util &quot;utilities&quot;)</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs.  See \u000A <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> kwd dir)</code> \u000A \u000A <p>where <v>kwd</v> is a <tt><see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see></tt> and <v>dir</v> is a relative or absolute \u000A <see topic=\"COMMON-LISP____PATHNAME\">pathname</see> for a directory.  (If the final '<v>/</v>' is missing, ACL2 will \u000A add it for you.)  The effect of this event is to modify the meaning of the \u000A <v>:dir</v> keyword argument of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or <tt><see topic=\"ACL2____LD\">ld</see></tt> as indicated \u000A by the examples above, that is, by associating the indicated directory with \u000A the indicated keyword for purposes of the <v>:dir</v> argument.  By the \u000A ``indicated directory'' we mean, in the case that the pathname is a relative \u000A pathname, the directory relative to the current connected book directory; see \u000A <see topic=\"ACL2____CBD\">cbd</see>.  See <see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see> for how to undo this \u000A effect.</p> \u000A \u000A <p>For a keyword already associated with a directory string by a previous \u000A invocation of <v>add-include-book-dir</v> or <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>, it \u000A is illegal to associate a different directory string until removing the \u000A existing association; see <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt> (and see <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt> if the existing association was made by <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>.  If however the new directory string is identical with \u000A the existing one, which was already assigned by <v>add-include-book-dir</v>, \u000A then the new call of <v>add-include-book-dir</v> will be redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).</p> \u000A \u000A <p>The keyword <v>:system</v> can never be redefined.  It will always point to \u000A the absolute pathname of the system books directory, which by default is \u000A immediately under the directory where the ACL2 executable was originally built \u000A (see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, in particular the discussion there of ``books \u000A directory'').</p> \u000A \u000A <p>This macro is essentially a call <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table\u000A :include-book-dir-alist ...)</v>, and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> \u000A and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs.  See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  Even if you invoke <v>add-include-book-dir</v> before \u000A certifying a book, so that this event is among the book's <see topic=\"ACL2____PORTCULLIS\">portcullis</see> \u000A commands rather than in the book itself, nevertheless that \u000A <v>add-include-book-dir</v> event will not be visible after the book is \u000A included.  If you want the effect to persist outside the book, see <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>.</p> \u000A \u000A <p>It is actually illegal to update the <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see> directly \u000A with a new <v>:include-book-dir-alist</v> value; instead, use \u000A <v>add-include-book-dir</v> and <v>delete-include-book-dir</v>.  This restriction \u000A allows ACL2 to function appropriately while including sub-books and in \u000A raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), where the associations of keywords to \u000A directories are stored outside the usual tables.  Upon exit from raw-mode, all \u000A effects disappear that were made by <v>add-include-book-dir</v>, <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>, <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt>, or <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt>.</p>"],

 "ACL2____ACCUMULATED-PERSISTENCE-SUBTLETIES":[["Accumulated-persistence"],"ACL2 Sources","ACL2","<p>In this topic we cover the overcounting of ``useful'' and of \u000A recursive <see topic=\"ACL2____RUNE\">rune</see> application attempts, and we describe how ``useless'' \u000A <see topic=\"ACL2____RUNE\">rune</see> application attempts can actually be critical for a proof's \u000A success.</p> \u000A \u000A <p><i>Overcounting of ``useful'' and of recursive rune application \u000A attempts.</i> Not every <see topic=\"ACL2____RUNE\">rune</see> application may be necessary for a proof's \u000A success.  Consider for example:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> a (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> b x))))\u000A            a))</code> \u000A \u000A <p>Then <v>show-accumulated-persistence</v> will tell us that <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules <v>car-cons</v> and <v>cdr-cons</v> each had one useful \u000A application.  However, the rule <v>cdr-cons</v> is used to simplify <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see>\u000A (<see topic=\"COMMON-LISP____CONS\">cons</see> b x))</v> to <v>x</v>, and this simplification is unecessary for the proof. \u000A Indeed, the proof succeeds even when preceded by the event: <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see>\u000A (<see topic=\"ACL2____DISABLE\">disable</see> cdr-cons))</v>.  We thus see that a <see topic=\"ACL2____RUNE\">rune</see> application labeled as \u000A ``useful'' may be simplifying a term that is not relevant to the proof.</p> \u000A \u000A <p>As of this writing, we consider every <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rule \u000A application to be ``useful'', for simplicity of the implementation.  Moreover, \u000A our counting of these rules is such that a single rule may be counted more \u000A than once.</p> \u000A \u000A <p>Next we show how recursive rule applications are overcounted.  Consider the \u000A following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mem (a x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (mem a (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code> \u000A \u000A <p>Now suppose we consider the sequence of theorems <v>(mem a (<see topic=\"COMMON-LISP____LIST\">list</see> a))</v>, \u000A <v>(mem a (<see topic=\"COMMON-LISP____LIST\">list</see> 1 a))</v>, <v>(mem a (<see topic=\"COMMON-LISP____LIST\">list</see> 1 2 a))</v>, <v>(mem a (<see topic=\"COMMON-LISP____LIST\">list</see> 1 2 3\u000A a))</v>, and so on.  We will see that the <v>:frames</v> reported for each \u000A increases quadratically, even though the <v>:tries</v> increases linearly; so in \u000A this case the <v>:tries</v> statistics are more appropriate.  Each time the \u000A definition of <v>mem</v> is applied, a new stack frame is pushed (see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>), and all subsequent applications of that definition \u000A are accumulated into the <v>:frames</v> count for that stack frame.  The final \u000A <v>:frames</v> count will be the sum of the counts for those individual frames, \u000A which form a linear sequence whose sum is therefore quadratic in the number of \u000A applications of the definition of <v>mem</v>.</p> \u000A \u000A <p><i>How ``useless'' attempts can be critical for a proof's success.</i> The \u000A command <v>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> :useless</v>)] will list rules that did not \u000A contribute directly to the proof (see <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>, in \u000A particular the discussion of ``useless'' there).  However, a ``useless'' rule \u000A can on rare occasions be critical to the success of a proof.  In the following \u000A example, we have a ``bad'' rule that can take the proof in the wrong \u000A direction, but a ``useless'' rule does a rewrite that prevents the succesful \u000A relieving of a hypothesis of the ``bad'' rule.  In summary:</p> \u000A \u000A <code>; Assume p0.  We want to prove p1.\u000A\u000A; Key rule:\u000Ap0 -&gt; p1 = t\u000A\u000A; Bad rule that could ruin the proof:\u000Ap3 -&gt; p1 = p2\u000A\u000A; But unfortunately, we know p3:\u000Ap0 -&gt; p3\u000A\u000A; Important ``useless'' rule, preventing ``bad rule'' above from firing:\u000Ap3 = p4</code> \u000A \u000A <p>The following event captures the rules described above.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((p0 (x) t)\u000A  (p1 (x) t)\u000A  (p2 (x) t)\u000A  (p3 (x) t)\u000A  (p4 (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p0 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p1 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p2 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p3 (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p4 (x) x))\u000A\u000A; Key rule:\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> p0-implies-p1\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (p0 x)\u000A            (p1 x)))\u000A\u000A; Bad rule that could ruin the proof:\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> p3-implies-p1-is-p2\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (p3 x)\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (p1 x) (p2 x))))\u000A\u000A; But unfortunately, we know p3:\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> p0-implies-p3\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (p0 x)\u000A            (p3 x)))\u000A\u000A; Important ``useless'' rule, preventing p3-implies-p1-is-p2 from firing:\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> p3-is-p4\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (p3 x) (p4 x))))</code> \u000A \u000A <p>Now we can see that <v>p3-is-p4</v> is labeled as ``useless'', by evaluating \u000A these commands.</p> \u000A \u000A <code>(<see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see> t)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (p0 x) (p1 x)))\u000A(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see>)</code> \u000A \u000A <p>If instead we first evaluate <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> p3-is-p4))</v> before the \u000A <v>thm</v> above, then the proof fails, even though <v>p3-is-p4</v> was labeled as \u000A ``useless''!</p> \u000A \u000A <p>Nevertheless, in general it is probably safe to disable rules reported as \u000A ``useless'' by <v>(<see topic=\"ACL2____SHOW-ACCUMULATED-PERSISTENCE\">show-accumulated-persistence</see> :useless)</v>, and doing so may \u000A speed up a proof considerably.</p> \u000A \u000A <p>Remark. The example above suggests a surprising fact: on rare occasions, a \u000A proof may fail when you give an <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint consisting of \u000A exactly the <see topic=\"ACL2____RUNE\">rune</see>s reported in a proof that succeeds.  For, imagine a \u000A rule R that is needed in part of the proof but is ``bad'' in a second part, \u000A and that some other, ``useless'' rule prevents the application of R in that \u000A second part.  The example above suggests that disabling this ``useless'' rule \u000A can allow the second application of R, thus preventing the proof.</p>"],

 "COMMON-LISP____FLET":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A; The following evaluates to (<see topic=\"ACL2____MV\">mv</see> 7 10):\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x)\u000A          (<see topic=\"COMMON-LISP_____B2\">+</see> x 3))\u000A       (g (x)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer x))\u000A          (<see topic=\"COMMON-LISP_____A2\">*</see> x 2)))\u000A  (<see topic=\"ACL2____MV\">mv</see> (f 4) (g 5)))\u000A\u000AGeneral Forms:\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> (def1 ... defk) body)\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> (def1 ... defk) declare-form1 .. declare-formk body)</code> \u000A \u000A <p>where <v>body</v> is a term, and each <v>defi</v> is a definition as in <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> but with the leading <v>defun</v> symbol omitted.  See <see topic=\"COMMON-LISP____DEFUN\">defun</see>.  If \u000A any <v>declare-formi</v> are supplied, then each must be of the form <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A decl1 ... decln)</v>, where each <v>decli</v> is of the form <v>(inline g1\u000A ... gm)</v> or <v>(notinline g1 ... gm)</v>, and each <v>gi</v> is defined by some \u000A <v>defi</v>.</p> \u000A \u000A <p>The only effect of the declarations is to provide advice to the host Lisp \u000A compiler.  The declarations are otherwise ignored by ACL2, so we mainly ignore \u000A them in the discussion below.</p> \u000A \u000A <p>The innermost <v>flet</v>-binding of a function symbol, <v>f</v>, above a call \u000A of <v>f</v>, is the one that provides the definition of <v>f</v> for that call. \u000A Note that <v>flet</v> does not provide recursion.  Consider the following \u000A example.</p> \u000A \u000A <code>; Give a global definition of f:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP_____B2\">+</see> x 3))\u000A; Evaluate an expression using a local binding of f:\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x (f (<see topic=\"COMMON-LISP____1_B2\">1+</see> x)))))\u000A  (f 4))</code> \u000A \u000A <p>In the above term <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> x (f (<see topic=\"COMMON-LISP____1_B2\">1+</see> x)))</v>, <v>f</v> refers to the global \u000A definition of <v>f</v> above the <v>flet</v> expression.  However, <v>(f 4)</v> \u000A refers to the <v>flet</v>-binding of <v>f</v>, <v>(f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x (f x)))</v>.  The \u000A result of the <v>flet</v> expression is thus obtained by evaluating <v>(f 4)</v> \u000A where <v>(f 4)</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 4 (f (<see topic=\"COMMON-LISP____1_B2\">1+</see> 4)))</v>, where the latter call of <v>f</v> \u000A refers to the global definition; thus we have <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 4 (f 5))</v>, which \u000A evaluates to <v>(4 . 8)</v>.</p> \u000A \u000A <p>Although <v>flet</v> behaves in ACL2 essentially as it does in Common Lisp, \u000A ACL2 imposes the following restrictions and qualifications.</p> \u000A \u000A <blockquote><p>o Every <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form for a local definition (<v>def1</v> \u000A through <v>defk</v>, above) must be an <v>ignore</v>, <v>ignorable</v>, or <v>type</v> \u000A expression.</p> \u000A \u000A <p>o Each <v>defi</v> must bind a different function symbol.</p> \u000A \u000A <p>o Each <v>defi</v> must bind a symbol that is a legal name for an ACL2 \u000A function symbol.  In particular, the symbol may not be in the keyword package \u000A or the main Lisp package.  Moreover, the symbol may not be a built-in ACL2 \u000A function or macro.</p> \u000A \u000A <p>o Every variable occurring in the body of a <v>defi</v> must be a formal \u000A parameter of that <v>defi</v>.  (This restriction is not enforced in Common \u000A Lisp.)</p> \u000A \u000A <p>o If the <v>flet</v>-binding <v>defi</v> is in the body of a function <v>f</v>, \u000A then the <see topic=\"ACL2____STOBJ\">stobj</see> inputs for <v>defi</v> are implicitly those of its inputs \u000A that are declared <see topic=\"ACL2____STOBJ\">stobj</see> inputs of <v>f</v>.</p></blockquote> \u000A \u000A <p><v>Flet</v> bindings are evaluated in parallel.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A       (g (x) (f x)))\u000A  (g 3))</code> \u000A \u000A <p>The binding of <v>g</v> refers to the global value of <v>f</v>, not the \u000A <v>flet</v>-binding of <v>f</v>.  Thus, the <v>flet</v> expression evaluates to 3. \u000A Compare the <v>flet</v> expression above to the following one, which instead \u000A evaluates to <v>(3 . 3)</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A(<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A  (<see topic=\"COMMON-LISP____FLET\">flet</see> ((g (x) (f x)))\u000A    (g 3)))</code> \u000A \u000A <p>Under the hood, ACL2 translates <v>flet</v> bindings to <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> \u000A expressions (see <see topic=\"ACL2____TERM\">term</see>), throwing away the <v>inline</v> and \u000A <v>notinline</v> declarations (if any).  The following example illustrates this \u000A point.</p> \u000A \u000A <code>ACL2 !&gt;:trans (<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A                     (g (x y) (<see topic=\"COMMON-LISP_____B2\">+</see> x y)))\u000A                (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (inline f))\u000A                (f (g 3 4)))\u000A\u000A((<see topic=\"COMMON-LISP____LAMBDA\">LAMBDA</see> (X) (<see topic=\"COMMON-LISP____CONS\">CONS</see> X X))\u000A ((<see topic=\"COMMON-LISP____LAMBDA\">LAMBDA</see> (X Y) (<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> X Y)) '3 '4))\u000A\u000A=&gt; *\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p><v>Flet</v> is part of Common Lisp.  See any Common Lisp documentation for \u000A more information.  We conclude by pointing out an important aspect of \u000A <v>flet</v> shared by ACL2 and Common Lisp: The binding is lexical, not dynamic. \u000A That is, the <v>flet</v> binding of a function symbol only applies to calls of \u000A that function symbol in the body of the <v>flet</v>, not other calls made in the \u000A course of evaluation.  Consider the following example.  Suppose we define:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x)\u000A  (<see topic=\"COMMON-LISP____FLET\">flet</see> ((f (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)))\u000A    (g x)))</code> \u000A \u000A <p>Then evaluation of <v>(h 3)</v> results in <v>3</v>, not in the <v>cons</v> pair \u000A <v>(3 . 3)</v>, because the <v>flet</v> binding of <v>f</v> only applies to calls of \u000A <v>f</v> that appear in the body of that <v>flet</v>.  In this case, only <v>g</v> \u000A is called in the body of that <v>flet</v>.</p>"],

 "ACL2____USING-COMPUTED-HINTS-4":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>So far we have used computed hints only to compute when a fixed set \u000A of keys and values are to be used as a hint.  But computed hints can, of \u000A course, compute the set of keys and values.  You might, for example, write a \u000A hint that recognizes when a clause ``ought'' to be provable by a <v>:BDD</v> \u000A hint and generate the appropriate hint.  You might build in a set of useful \u000A lemmas and check to see if the clause is provable <v>:BY</v> one of them.  You \u000A can keep all function symbols disabled and use computed hints to compute which \u000A ones you want to <v>:EXPAND</v>.  In general, you can write a theorem prover for \u000A use in your hints, provided you can get it to do its job by directing our \u000A theorem prover.</p> \u000A \u000A <p>Suppose for example we wish to find every occurrence of an instance of \u000A <v>(SWAP x)</v> and provide the corresponding instance of \u000A <v>ALL-SWAPS-HAVE-THE-PROPERTY</v>.  Obviously, we must explore the clause \u000A looking for instances of <v>(SWAP x)</v> and build the appropriate instances of \u000A the lemma.  We could do this in many different ways, but below we show a \u000A general purpose set of utilities for doing it.  The functions are not defined \u000A in ACL2 but could be defined as shown.</p> \u000A \u000A <p>Our plan is: (1) Find all instances of a given pattern (term) in a clause, \u000A obtaining a set of substitutions.  (2) Build a set of <v>:instance</v> \u000A expressions for a given lemma name and set of substitutions.  (3) Generate a \u000A <v>:use</v> hint for those instances when instances are found.</p> \u000A \u000A <p>The pair of functions below find all instances of a given pattern term in \u000A either a term or a list of terms.  The functions each return a list of \u000A substitutions, each substitution accounting for one of the matches of pat to a \u000A subterm.  At this level in ACL2 substitutions are lists of pairs of the form \u000A <v>(var . term)</v>.  All terms mentioned here are presumed to be in translated \u000A form.</p> \u000A \u000A <p>The functions take as their third argument a list of substitutions \u000A accumulated to date and add to it the substitutions produced by matching pat \u000A to the subterms of the term.  We intend this accumulator to be nil initially. \u000A If the returned value is nil, then no instances of pat occurred.</p> \u000A \u000A <code>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-all-instances (pat term alists)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A  (instancep alist)\u000A  (one-way-unify pat term)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((alists (<see topic=\"COMMON-LISP____IF\">if</see> instancep (<see topic=\"ACL2____ADD-TO-SET-EQUAL\">add-to-set-equal</see> alist alists) alists)))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A     ((variablep term) alists)\u000A     ((fquotep term) alists)\u000A     (t (find-all-instances-list pat (fargs term) alists))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-all-instances-list (pat list-of-terms alists)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A  ((<see topic=\"COMMON-LISP____NULL\">null</see> list-of-terms) alists)\u000A  (t (find-all-instances pat\u000A                         (<see topic=\"COMMON-LISP____CAR\">car</see> list-of-terms)\u000A                         (find-all-instances-list pat\u000A                                                  (<see topic=\"COMMON-LISP____CDR\">cdr</see> list-of-terms)\u000A                                                  alists))))))</code> \u000A \u000A <p>Caveat: The following aside has nothing to do with computed hints.  Does an \u000A instance of <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> x))</v> occur in <v>((<see topic=\"COMMON-LISP____LAMBDA\">LAMBDA</see> (V) (<see topic=\"COMMON-LISP____CAR\">CAR</see> V)) (<see topic=\"COMMON-LISP____CDR\">CDR</see> A))</v>? \u000A It does if one beta-reduces the lambda-expression to <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A))</v>; the \u000A appropriate substitution is to replace <v>x</v> by <v>A</v>.  But the definition \u000A of <v>find-all-instances</v> above does <i>not</i> find this instance because it \u000A does not do beta-reduction.</p> \u000A \u000A <p>We now turn our attention to converting a list of substitutions into a list \u000A of lemma instances, each of the form</p> \u000A \u000A <code>(:INSTANCE name (var1 term1) ... (vark termk))</code> \u000A \u000A <p>as written in <v>:use</v> hints.  In the code shown above, substitutions are \u000A lists of pairs of the form <v>(var . term)</v>, but in lemma instances we must \u000A write ``doublets.''  So here we show how to convert from one to the other:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pairs-to-doublets (alist)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> alist) nil)\u000A        (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAAR\">caar</see> alist) (<see topic=\"COMMON-LISP____CDAR\">cdar</see> alist))\u000A                 (pairs-to-doublets (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist))))))</code> \u000A \u000A <p>Now we can make a list of lemma instances:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-lemma-instances (<see topic=\"ACL2____NAME\">name</see> alists)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____NULL\">null</see> alists) nil)\u000A   (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____LIST_A2\">list*</see> :instance name (pairs-to-doublets (<see topic=\"COMMON-LISP____CAR\">car</see> alists)))\u000A            (make-lemma-instances name (<see topic=\"COMMON-LISP____CDR\">cdr</see> alists))))))</code> \u000A \u000A <p>Finally, we can package it all together into a hint function.  The function \u000A takes a pattern, <v>pat</v>, which must be a translated term, the name of a \u000A lemma, <v>name</v>, and a clause.  If some instances of <v>pat</v> occur in \u000A <v>clause</v>, then the corresponding instances of <v>name</v> are <v>:USE</v>d in \u000A the computed hint.  Otherwise, the hint does not apply.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> add-corresponding-instances (pat name clause)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((alists (find-all-instances-list pat clause nil)))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A     ((<see topic=\"COMMON-LISP____NULL\">null</see> alists) nil)\u000A     (t (<see topic=\"COMMON-LISP____LIST\">list</see> :use (make-lemma-instances name alists))))))</code> \u000A \u000A <p>The design of this particular hint function makes it important that the \u000A variables of the pattern be the variables of the named lemma and that all of \u000A the variables we wish to instantiate occur in the pattern.  We could, of \u000A course, redesign it to allow ``free variables'' or some sort of renaming.</p> \u000A \u000A <p>We could now use this hint as shown below:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> ... ...\u000A  :hints ((add-corresponding-instances\u000A           '(SWAP x)\u000A           'ALL-SWAPS-HAVE-THE-PROPERTY\u000A           clause)))</code> \u000A \u000A <p>The effect of the hint above is that any time a clause arises in which any \u000A instance of <v>(SWAP x)</v> appears, we add the corresponding instance of \u000A <v>ALL-SWAPS-HAVE-THE-PROPERTY</v>.  So for example, if Subgoal *1/3.5 contains \u000A the subterm <v>(SWAP (SWAP A))</v> then this hint fires and makes the system \u000A behave as though the hint:</p> \u000A \u000A <code>(&quot;Subgoal *1/3.5&quot;\u000A :USE ((:INSTANCE ALL-SWAPS-HAVE-THE-PROPERTY (X A))\u000A       (:INSTANCE ALL-SWAPS-HAVE-THE-PROPERTY (X (SWAP A)))))</code> \u000A \u000A <p>had been present.</p>"],

 "ACL2____USING-COMPUTED-HINTS-3":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>Sometimes it is desirable to supply a hint whenever a certain term \u000A arises in a conjecture.  For example, suppose we have proved</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> all-swaps-have-the-property\u000A   (the-property (swap x))\u000A   :rule-classes nil)</code> \u000A \u000A <p>and suppose that whenever <v>(SWAP A)</v> occurs in a goal, we wish to add \u000A the additional hypothesis that <v>(THE-PROPERTY (SWAP A))</v>.  Note that this \u000A is equivalent supplying the hint</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> test\u000A    '(:use (:instance all-swaps-have-the-property (x A)))\u000A    nil)</code> \u000A \u000A <p>where <v>test</v> answers the question ``does the clause contain <v>(SWAP\u000A A)</v>?''  That question can be asked with <v>(occur-lst '(SWAP A) clause)</v>. \u000A Briefly, <v>occur-lst</v> takes the representation of a translated term, x, and \u000A a list of translated terms, y, and determines whether x occurs as a subterm of \u000A any term in y.  (By ``subterm'' here we mean proper or improper, e.g., the \u000A subterms of <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> X)</v> are <v>X</v> and <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> X)</v>.)</p> \u000A \u000A <p>Thus, the computed hint:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (occur-lst '(swap a) clause)\u000A    '(:use (:instance all-swaps-have-the-property (x A)))\u000A    nil)</code> \u000A \u000A <p>will add the hypothesis <v>(THE-PROPERTY (SWAP A))</v> to every goal \u000A containing <v>(SWAP A)</v> -- except the children of goals to which the \u000A hypothesis was added.</p> \u000A \u000A <p><b>A COMMON MISTAKE</b> users are likely to make is to forget that they are \u000A dealing with translated terms.  For example, suppose we wished to look for \u000A <v>(SWAP (<see topic=\"COMMON-LISP____LIST\">LIST</see> 1 A))</v> with <v>occur-lst</v>.  We would never find it with</p> \u000A \u000A <code>(occur-lst '(SWAP (<see topic=\"COMMON-LISP____LIST\">LIST</see> 1 A)) clause)</code> \u000A \u000A <p>because that presentation of the term contains macros and other \u000A abbreviations.  By using :trans (see <see topic=\"ACL2____TRANS\">trans</see>) we can obtain the \u000A translation of the target term.  Then we can look for it with:</p> \u000A \u000A <code>(occur-lst '(SWAP (<see topic=\"COMMON-LISP____CONS\">CONS</see> '1 (<see topic=\"COMMON-LISP____CONS\">CONS</see> A 'NIL))) clause)</code> \u000A \u000A <p>Note in particular that you must</p> \u000A \u000A <code>* eliminate all macros and\u000A* explicitly quote all constants.</code> \u000A \u000A <p>We recommend using <v>:trans</v> to obtain the translated form of the terms \u000A in which you are interested, before programming your hints.</p> \u000A \u000A <p>An alternative is to use the expression <v>(prettyify-clause clause nil\u000A nil)</v> in your hint to convert the current goal clause into the s-expression \u000A that is actually printed.  For example, the clause</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)) (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> Y) (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> '1 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)) Y))</code> \u000A \u000A <p>``prettyifies'' to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> Y)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> 1 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)) Y))</code> \u000A \u000A <p>which is what you would see printed by ACL2 when the goal clause is that \u000A shown.</p> \u000A \u000A <p>However, if you choose to convert your clauses to prettyified form, you \u000A will have to write your own explorers (like our <v>occur-lst</v>), because all \u000A of the ACL2 term processing utilities work on translated and/or clausal forms. \u000A This should not be taken as a terrible burden.  You will, at least, gain the \u000A benefit of knowing what you are really looking for, because your explorers \u000A will be looking at exactly the s-expressions you see at your terminal.  And \u000A you won't have to wade through our still undocumented term/clause utilities. \u000A The approach will slow things down a little, since you will be paying the \u000A price of independently consing up the prettyified term.</p> \u000A \u000A <p>We make one more note on this example.  We said above that the computed \u000A hint:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (occur-lst '(swap a) clause)\u000A    '(:use (:instance all-swaps-have-the-property (x A)))\u000A    nil)</code> \u000A \u000A <p>will add the hypothesis <v>(THE-PROPERTY (SWAP A))</v> to every goal \u000A containing <v>(SWAP A)</v> -- except the children of goals to which the \u000A hypothesis was added.</p> \u000A \u000A <p>It bears noting that the subgoals produced by induction and top-level \u000A forcing round goals are not children.  For example, suppose the hint above \u000A fires on \"Subgoal 3\" and produces, say, \"Subgoal 3'\".  Then the hint will \u000A not fire on \"Subgoal 3'\" even though it (still) contains <v>(SWAP A)</v> \u000A because \"Subgoal 3'\" is a child of a goal on which the hint fired.</p> \u000A \u000A <p>But now suppose that \"Subgoal 3'\" is pushed for induction.  Then the \u000A goals created by that induction, i.e., the base case and induction step, are \u000A not considered children of \"Subgoal 3'\".  All of the original hints are \u000A available.</p> \u000A \u000A <p>Alternatively, suppose that \"Subgoal 3' is proved but forces some other \u000A subgoal, \"[1]Subgoal 1\" which is attacked in Forcing Round 1.  That \u000A top-level forced subgoal is not a child.  All the original hints are available \u000A to it.  Thus, if it contains <v>(SWAP A)</v>, the hint will fire and supply the \u000A hypothesis, producing \"[1]Subgoal 1'\".  This may be unnecessary, as the \u000A hypothesis might already be present in \"[1]Subgoal 1\".  In this case, no \u000A harm is done.  The hint won't fire on \"[1]Subgoal 1\" because it is a child \u000A of \"[1]Subgoal 1\" and the hint fired on that.</p>"],

 "ACL2____TUTORIAL3-PHONEBOOK-EXAMPLE":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>The other tutorial examples are rather small and entirely self \u000A contained.  The present example is rather more elaborate, and makes use of a \u000A feature that really adds great power and versatility to ACL2, namely: the use \u000A of previously defined collections of lemmas, in the form of ``<see topic=\"ACL2____BOOKS\">books</see>.''</p> \u000A \u000A <p>This example was written almost entirely by Bill Young of Computational \u000A Logic, Inc.</p> \u000A \u000A <p>This example is based on one developed by Ricky Butler and Sally Johnson of \u000A NASA Langley for the PVS system, and subsequently revised by Judy Crow, <i>et \u000A al</i>, at SRI.  It is a simple phone book specification.  We will not bother \u000A to follow their versions closely, but will instead present a style of \u000A specification natural for ACL2.</p> \u000A \u000A <p>The idea is to model an electronic phone book with the following \u000A properties.</p> \u000A \u000A <blockquote> \u000A \u000A <p>Our phone book will store the phone numbers of a city.</p> \u000A \u000A <p>It must be possible to retrieve a phone number, given a name.</p> \u000A \u000A <p>It must be possible to add and delete entries.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Of course, there are numerous ways to construct such a model.  A natural \u000A approach within the Lisp/ACL2 context is to use ``association lists'' or \u000A ``alists.''  Briefly, an alist is a list of pairs <v>(key .  value)</v> \u000A associating a value with a key.  A phone book could be an alist of pairs \u000A <v>(<see topic=\"ACL2____NAME\">name</see> . pnum)</v>.  To find the phone number associated with a given name, we \u000A merely search the alist until we find the appropriate pair.  For a large city, \u000A such a linear list would not be efficient, but at this point we are interested \u000A only in <b>modeling</b> the problem, not in deriving an efficient \u000A implementation.  We could address that question later by proving our alist \u000A model equivalent, in some desired sense, to a more efficient data \u000A structure.</p> \u000A \u000A <p>We could build a theory of alists from scratch, or we can use a previously \u000A constructed theory (book) of alist definitions and facts.  By using an \u000A existing book, we build upon the work of others, start our specification and \u000A proof effort from a much richer foundation, and hopefully devote more of our \u000A time to the problem at hand.  Unfortunately, it is not completely simple for \u000A the new user to know what <see topic=\"ACL2____BOOKS\">books</see> are available and what they contain. \u000A We hope later to improve the documentation of the growing collection of \u000A <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> that are typically downloaded with ACL2; for now, the \u000A reader is encouraged to look in the README.html file in the books' top-level \u000A directory.  For present purposes, the beginning user can simply take our word \u000A that a book exists containing useful alist definitions and facts.  These \u000A definitions and lemmas can be introduced into the current theory using the \u000A <see topic=\"ACL2____COMMAND\">command</see>:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;data-structures/alist-defthms&quot; :dir :system)</code> \u000A \u000A <p>This book has been ``certified,'' which means that the definitions and \u000A lemmas have been mechanically checked and stored in a safe manner.  (See <see topic=\"ACL2____BOOKS\">books</see> and see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> for details.)</p> \u000A \u000A <p>Including this book makes available a collection of functions including the \u000A following:</p> \u000A \u000A <code>(<see topic=\"ACL2____ALISTP\">ALISTP</see> A)    ; is A an alist (actually a primitive ACL2 function)\u000A\u000A(BIND X V A)  ; associate the key X with value V in alist A\u000A\u000A(BINDING X A) ; return the value associated with key X in alist A\u000A\u000A(BOUND? X A)  ; is key X associated with any value in alist A\u000A\u000A(DOMAIN A)    ; return the list of keys bound in alist A\u000A\u000A(RANGE A)     ; return the list of values bound to keys in alist A\u000A\u000A(REMBIND X A) ; remove the binding of key X in alist A</code> \u000A \u000A <p>Along with these function definitions, the book also provides a number of \u000A proved lemmas that aid in simplifying expressions involving these functions. \u000A (See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for the way in which lemmas are used in \u000A simplification and rewriting.)  For example,</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> bound?-bind\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bound? x (bind y v a))\u000A         (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)\u000A             (bound? x a))))</code> \u000A \u000A <p>asserts that <v>x</v> will be bound in <v>(bind y v a)</v> if and only if: \u000A either <v>x = y</v> or <v>x</v> was already bound in <v>a</v>.  Also,</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> binding-bind\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (binding x (bind y v a))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)\u000A             v\u000A           (binding x a))))</code> \u000A \u000A <p>asserts that the resulting binding will be <v>v</v>, if <v>x = y</v>, or the \u000A binding that <v>x</v> had in <v>a</v> already, if not.</p> \u000A \u000A <p>Thus, the inclusion of this book essentially extends our specification and \u000A reasoning capabilities by the addition of new operations and facts about these \u000A operations that allow us to build further specifications on a richer and \u000A possibly more intuitive foundation.</p> \u000A \u000A <p>However, it must be admitted that the use of a book such as this has two \u000A potential limitations:</p> \u000A \u000A <blockquote> \u000A \u000A <p>the definitions available in a book may not be ideal for your particular \u000A problem;</p> \u000A \u000A <p>it is (extremely) likely that some useful facts (especially, <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rules) are not available in the book and will have to be proved.</p> \u000A \u000A </blockquote> \u000A \u000A <p>For example, what is the value of <v>binding</v> when given a key that is not \u000A bound in the alist?  We can find out by examining the function definition. \u000A Look at the definition of the <v>binding</v> function (or any other defined \u000A function), using the <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> command:</p> \u000A \u000A <code>ACL2 !&gt;:pe binding\u000A   d     33  (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see>\u000A                  &quot;/slocal/src/acl2/v1-9/books/public/alist-defthms&quot;)\u000A\u000A&gt;V d          (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> BINDING (X A)\u000A                     &quot;The value bound to X in alist A.&quot;\u000A                     (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD (<see topic=\"ACL2____ALISTP\">ALISTP</see> A)))\u000A                     (<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"ACL2____ASSOC-EQUAL\">ASSOC-EQUAL</see> X A)))</code> \u000A \u000A <p>This tells us that <v>binding</v> was introduced by the given <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form, is currently <see topic=\"ACL2____DISABLE\">disable</see>d in the current theory, and \u000A has the definition given by the displayed <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form.  We see that \u000A <v>binding</v> is actually defined in terms of the primitive <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt> \u000A function.  If we look at the definition of <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>:</p> \u000A \u000A <code>ACL2 !&gt;:pe assoc-equal\u000A V     -489  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> ASSOC-EQUAL (X ALIST)\u000A                    (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD (<see topic=\"ACL2____ALISTP\">ALISTP</see> ALIST)))\u000A                    (<see topic=\"COMMON-LISP____COND\">COND</see> ((<see topic=\"COMMON-LISP____ENDP\">ENDP</see> ALIST) NIL)\u000A                          ((<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> ALIST)))\u000A                           (<see topic=\"COMMON-LISP____CAR\">CAR</see> ALIST))\u000A                          (T (<see topic=\"ACL2____ASSOC-EQUAL\">ASSOC-EQUAL</see> X (<see topic=\"COMMON-LISP____CDR\">CDR</see> ALIST)))))</code> \u000A \u000A <p>we can see that <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt> returns <v>nil</v> upon reaching the end \u000A of an unsuccessful search down the alist.  So <v>binding</v> returns <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see>\u000A nil)</v> in that case, which is <v>nil</v>.  Notice that we could also have \u000A investigated this question by trying some simple examples.</p> \u000A \u000A <code>ACL2 !&gt;(binding 'a nil)\u000ANIL\u000A\u000AACL2 !&gt;(binding 'a (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'b 2)))\u000ANIL</code> \u000A \u000A <p>These definitions aren't ideal for all purposes. For one thing, there's \u000A nothing that keeps us from having <v>nil</v> as a value bound to some key in the \u000A alist.  Thus, if <v>binding</v> returns <v>nil</v> we don't always know if that is \u000A the value associated with the key in the alist, or if that key is not bound. \u000A We'll have to keep that ambiguity in mind whenever we use <v>binding</v> in our \u000A specification.  Suppose instead that we wanted <v>binding</v> to return some \u000A error string on unbound keys.  Well, then we'd just have to write our own \u000A version of <v>binding</v>.  But then we'd lose much of the value of using a \u000A previously defined book.  As with any specification technique, certain \u000A tradeoffs are necessary.</p> \u000A \u000A <p>Why not take a look at the definitions of other alist functions and see how \u000A they work together to provide the ability to construct and search alists? \u000A We'll be using them rather heavily in what follows so it will be good if you \u000A understand basically how they work.  Simply start up ACL2 and execute the form \u000A shown earlier, but substituting our directory name for the top-level ACL2 \u000A directory with yours.  Alternatively, just</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;data-structures/alist-defthms&quot; :dir :system)</code> \u000A \u000A <p>Then, you can use <v>:</v><see topic=\"ACL2____PE\">pe</see> to look at function definitions.  You'll \u000A soon discover that almost all of the definitions are built on definitions of \u000A other, more primitive functions, as <v>binding</v> is built on <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>.  You can look at those as well, of course, or in many cases \u000A visit their documentation.</p> \u000A \u000A <p>The other problem with using a predefined book is that it will seldom be \u000A ``sufficiently complete,'' in the sense that the collection of <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rules supplied won't be adequate to prove everything we'd like to know about \u000A the interactions of the various functions.  If it were, there'd be no real \u000A reason to know that <v>binding</v> is built on top of <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>, \u000A because everything we'd need to know about <v>binding</v> would be nicely \u000A expressed in the collection of theorems supplied with the book.  However, \u000A that's very seldom the case.  Developing such a collection of rules is \u000A currently more art than science and requires considerable experience.  We'll \u000A encounter examples later of ``missing'' facts about <v>binding</v> and our other \u000A alist functions.  So, let's get on with the example.</p> \u000A \u000A <p>Notice that alists are mappings of keys to values; but, there is no notion \u000A of a ``type'' associated with the keys or with the values.  Our phone book \u000A example, however, does have such a notion of types; we map names to phone \u000A numbers.  We can introduce these ``types'' by explicitly defining them, e.g., \u000A names are strings and phone numbers are integers.  Alternatively, we can \u000A <b>partially define</b> or axiomatize a recognizer for names without giving a \u000A full definition.  A way to safely introduce such ``constrained'' function \u000A symbols in ACL2 is with the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form.  For example, consider \u000A the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ;; Introduce a recognizer for names and give a ``type'' lemma.\u000A  (((namep *) =&gt; *))\u000A  ;;\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> namep (x)\u000A           ;; This declare is needed to tell\u000A           ;; ACL2 that we're aware that the\u000A           ;; argument x is not used in the body\u000A           ;; of the function.\u000A           (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A           t))\u000A  ;;\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> namep-booleanp\u000A    (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (namep x))))</code> \u000A \u000A <p>This <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form introduces the new function <v>namep</v> of one \u000A argument and one result and constrains <v>(namep x)</v> to be Boolean, for all \u000A inputs <v>x</v>.  More generally, an encapsulation establishes an environment in \u000A which functions can be defined and theorems and rules added without \u000A necessarily introducing those functions, theorems, and rules into the \u000A environment outside the encapsulation.  To be admissible, all the events in \u000A the body of an encapsulate must be admissible.  But the effect of an \u000A encapsulate is to assume only the non-local events.</p> \u000A \u000A <p>The first ``argument'' to <v>encapsulate</v>, <v>((namep (x) t))</v> above, \u000A declares the intended <see topic=\"ACL2____SIGNATURE\">signature</see>s of new function symbols that will be \u000A ``exported'' from the encapsulation without definition.  The <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> of <v>name</v> defines name within the encapsulation always to \u000A return <v>t</v>.  The <v>defthm</v> event establishes that <v>namep</v> is Boolean. \u000A By making the <v>defun</v> local but the <v>defthm</v> non-<v>local</v> this \u000A encapsulate constrains the undefined function <v>namep</v> to be Boolean; the \u000A admissibility of the encapsulation establishes that there exists a Boolean \u000A function (namely the constant function returning <v>t</v>).</p> \u000A \u000A <p>We can subsequently use <v>namep</v> as we use any other Boolean function, \u000A with the proviso that we know nothing about it except that it always returns \u000A either <v>t</v> or <v>nil</v>.  We use <v>namep</v> to ``recognize'' legal keys for \u000A our phonebook alist.</p> \u000A \u000A <p>We wish to do something similar to define what it means to be a legal phone \u000A number.  We submit the following form to ACL2:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ;; Introduce a recognizer for phone numbers.\u000A  (((pnump *) =&gt; *))\u000A  ;;\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> pnump (x)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))))\u000A  ;;\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> pnump-booleanp\u000A    (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (pnump x)))\u000A  ;;\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> nil-not-pnump\u000A    (<see topic=\"COMMON-LISP____NOT\">not</see> (pnump nil)))).</code> \u000A \u000A <p>This introduces a Boolean-valued recognizer <v>pnump</v>, with the additional \u000A proviso that the constant <v>nil</v> is not a <v>pnump</v>.  We impose this \u000A restriction to guarantee that we'll never bind a name to <v>nil</v> in our phone \u000A book and thereby introduce the kind of ambiguity described above regarding the \u000A use of <v>binding</v>.</p> \u000A \u000A <p>Now a legal phone book is an alist mapping from <v>namep</v>s to <v>pnump</v>s. \u000A We can define this as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> name-phonenum-pairp (x)\u000A  ;; Recognizes a pair of (<see topic=\"ACL2____NAME\">name</see> . pnum).\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A       (namep (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A       (pnump (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> phonebookp (l)\u000A  ;; Recognizes a list of such pairs.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> l))\u000A      (<see topic=\"COMMON-LISP____NULL\">null</see> l)\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (name-phonenum-pairp (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A         (phonebookp (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))))</code> \u000A \u000A <p>Thus, a phone book is really a list of pairs <v>(<see topic=\"ACL2____NAME\">name</see> . pnum)</v>.  Notice \u000A that we have not assumed that the keys of the phone book are distinct.  We'll \u000A worry about that question later.  (It is not always desirable to insist that \u000A the keys of an alist be distinct.  But it may be a useful requirement for our \u000A specific example.)</p> \u000A \u000A <p>Now we are ready to define some of the functions necessary for our \u000A phonebook example.  The functions we need are:</p> \u000A \u000A <code>(IN-BOOK? NM BK)          ; does NM have a phone number in BK\u000A\u000A(FIND-PHONE NM BK)        ; find NM's phone number in phonebook BK\u000A\u000A(ADD-PHONE NM PNUM BK)    ; give NM the phone number PNUM in BK\u000A\u000A(CHANGE-PHONE NM PNUM BK) ; change NM's phone number to PNUM in BK\u000A\u000A(DEL-PHONE NM PNUM)       ; remove NM's phone number from BK</code> \u000A \u000A <p>Given our underlying theory of alists, it is easy to write these functions. \u000A But we must take care to specify appropriate ``boundary'' behavior.  Thus, \u000A what behavior do we want when, say, we try to change the phone number of a \u000A client who is not currently in the book?  As usual, there are numerous \u000A possibilities; here we'll assume that we return the phone book unchanged if we \u000A try anything ``illegal.''</p> \u000A \u000A <p>Possible definitions of our phone book functions are as follows. \u000A (Remember, an <v>include-book</v> form such as the ones shown earlier must be \u000A executed in order to provide definitions for functions such as \u000A <v>bound?</v>.)</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> in-book? (nm bk)\u000A  (bound? nm bk))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-phone (nm bk)\u000A  (binding nm bk))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> add-phone (nm pnum bk)\u000A  ;; If nm already in-book?, make no change.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (in-book? nm bk)\u000A      bk\u000A    (bind nm pnum bk)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> change-phone (nm pnum bk)\u000A  ;; Make a change only if nm already has a phone number.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (in-book? nm bk)\u000A      (bind nm pnum bk)\u000A    bk))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> del-phone (nm bk)\u000A  ;; Remove the binding from bk, if there is one.\u000A  (rembind nm bk))</code> \u000A \u000A <p>Notice that we don't have to check whether a name is in the book before \u000A deleting, because <v>rembind</v> is essentially a no-op if <v>nm</v> is not bound \u000A in <v>bk</v>.</p> \u000A \u000A <p>In some sense, this completes our specification.  But we can't have any \u000A real confidence in its correctness without validating our specification in \u000A some way.  One way to do so is by proving some properties of our \u000A specification.  Some candidate properties are:</p> \u000A \u000A <blockquote> \u000A \u000A <p>1. A name will be in the book after we add it.</p> \u000A \u000A <p>2. We will find the most recently added phone number for a client.</p> \u000A \u000A <p>3. If we change a number, we'll find the change.</p> \u000A \u000A <p>4. Changing and then deleting a number is the same as just deleting.</p> \u000A \u000A <p>5. A name will not be in the book after we delete it.</p></blockquote> \u000A \u000A <p>Let's formulate some of these properties.  The first one, for example, \u000A is:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> add-in-book\u000A  (in-book? nm (add-phone nm pnum bk))).</code> \u000A \u000A <p>You may wonder why we didn't need any hypotheses about the ``types'' of the \u000A arguments.  In fact, <v>add-in-book</v> is really expressing a property that is \u000A true of alists in general, not just of the particular variety of alists we are \u000A dealing with.  Of course, we could have added some extraneous hypotheses and \u000A proved:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> add-in-book\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (namep nm)\u000A                (pnump pnum)\u000A                (phonebookp bk))\u000A           (in-book? nm (add-phone nm pnum bk)))),</code> \u000A \u000A <p>but that would have yielded a weaker and less useful lemma because it would \u000A apply to fewer situations.  In general, it is best to state lemmas in the most \u000A general form possible and to eliminate unnecessary hypotheses whenever \u000A possible.  The reason for that is simple: lemmas are usually stored as rules \u000A and used in later proofs.  For a lemma to be used, its hypotheses must be \u000A relieved (proved to hold in that instance); extra hypotheses require extra \u000A work.  So we avoid them whenever possible.</p> \u000A \u000A <p>There is another, more important observation to make about our lemma.  Even \u000A in its simpler form (without the extraneous hypotheses), the lemma \u000A <v>add-in-book</v> may be useless as a <see topic=\"ACL2____REWRITE\">rewrite</see> rule.  Notice that it is \u000A stated in terms of the non-recursive functions <v>in-book?</v> and \u000A <v>add-phone</v>.  If such functions appear in the left hand side of the \u000A conclusion of a lemma, the lemma may not ever be used.  Suppose in a later \u000A proof, the theorem prover encountered a <see topic=\"ACL2____TERM\">term</see> of the form:</p> \u000A \u000A <code>(in-book? nm (add-phone nm pnum bk)).</code> \u000A \u000A <p>Since we've already proved <v>add-in-book</v>, you'd expect that this would \u000A be immediately reduced to true.  However, the theorem prover will often \u000A ``expand'' the non-recursive definitions of <v>in-book?</v>  and <v>add-phone</v> \u000A using their definitions <b>before</b> it attempts rewriting with lemmas. \u000A After this expansion, lemma <v>add-in-book</v> won't ``match'' the <see topic=\"ACL2____TERM\">term</see> \u000A and so won't be applied.  Look back at the proof script for <v>add-in-proof</v> \u000A and you'll notice that at the very end the prover warned you of this potential \u000A difficulty when it printed:</p> \u000A \u000A <code>Warnings:  Non-rec\u000ATime:  0.18 seconds (prove: 0.05, print: 0.00, other: 0.13)\u000AADD-IN-BOOK</code> \u000A \u000A <p>The ``Warnings'' line notifies you that there are non-recursive function \u000A calls in the left hand side of the conclusion and that this problem might \u000A arise.  Of course, it may be that you don't ever plan to use the lemma for \u000A rewriting or that your intention is to <see topic=\"ACL2____DISABLE\">disable</see> these functions.  <see topic=\"ACL2____DISABLE\">Disable</see>d functions are not expanded and the lemma should apply.  However, you \u000A should always take note of such warnings and consider an appropriate response. \u000A By the way, we noted above that <v>binding</v> is <see topic=\"ACL2____DISABLE\">disable</see>d.  If it were \u000A not, none of the lemmas about <v>binding</v> in the book we included would \u000A likely be of much use for exactly the reason we just gave.</p> \u000A \u000A <p>For our current example, let's assume that we're just investigating the \u000A properties of our specifications and not concerned about using our lemmas for \u000A rewriting.  So let's go on.  If we really want to avoid the warnings, we can \u000A add <v>:rule-classes nil</v> to each <v>defthm</v> event; see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p>Property 2 is: we always find the most recently added phone number for a \u000A client.  Try the following formalization:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> find-add-first-cut\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (find-phone nm (add-phone nm pnum bk))\u000A         pnum))</code> \u000A \u000A <p>and you'll find that the proof attempt fails.  Examining the proof attempt \u000A and our function definitions, we see that the lemma is false if <v>nm</v> is \u000A already in the book.  We can remedy this situation by reformulating our lemma \u000A in at least two different ways:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> find-add1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (in-book? nm bk))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (find-phone nm (add-phone nm pnum bk))\u000A                  pnum)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> find-add2\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (find-phone nm (add-phone nm pnum bk))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (in-book? nm bk)\u000A             (find-phone nm bk)\u000A             pnum)))</code> \u000A \u000A <p>For technical reasons, lemmas such as <v>find-add2</v>, i.e., which do not \u000A have hypotheses, are usually slightly preferable.  This lemma is stored as an \u000A ``unconditional'' <see topic=\"ACL2____REWRITE\">rewrite</see> rule (i.e., has no hypotheses) and, \u000A therefore, will apply more often than <v>find-add1</v>.  However, for our \u000A current purposes either version is all right.</p> \u000A \u000A <p>Property 3 says: If we change a number, we'll find the change.  This is \u000A very similar to the previous example.  The formalization is as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> find-change\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (find-phone nm (change-phone nm pnum bk))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (in-book? nm bk)\u000A             pnum\u000A           (find-phone nm bk))))</code> \u000A \u000A <p>Property 4 says: changing and then deleting a number is the same as just \u000A deleting.  We can model this as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> del-change\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (del-phone nm (change-phone nm pnum bk))\u000A         (del-phone nm bk)))</code> \u000A \u000A <p>Unfortunately, when we try to prove this, we encounter subgoals that seem \u000A to be true, but for which the prover is stumped.  For example, consider the \u000A following goal.  (Note: <v>endp</v> holds of lists that are empty.)</p> \u000A \u000A <code>Subgoal *1/4\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> BK))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> NM (<see topic=\"COMMON-LISP____CAAR\">CAAR</see> BK)))\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (BOUND? NM (<see topic=\"COMMON-LISP____CDR\">CDR</see> BK)))\u000A              (BOUND? NM BK))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (REMBIND NM (BIND NM PNUM BK))\u000A                (REMBIND NM BK))).</code> \u000A \u000A <p>Our intuition about <v>rembind</v> and <v>bind</v> tells us that this goal \u000A should be true even without the hypotheses.  We attempt to prove the following \u000A lemma.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> rembind-bind\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rembind nm (bind nm pnum bk))\u000A         (rembind nm bk)))</code> \u000A \u000A <p>The prover proves this by induction, and stores it as a rewrite rule. \u000A After that, the prover has no difficulty in proving <v>del-change</v>.</p> \u000A \u000A <p>The need to prove lemma <v>rembind-bind</v> illustrates a point we made early \u000A in this example: the collection of <see topic=\"ACL2____REWRITE\">rewrite</see> rules supplied by a \u000A previously certified book will almost never be everything you'll need.  It \u000A would be nice if we could operate purely in the realm of names, phone numbers, \u000A and phone books without ever having to prove any new facts about alists. \u000A Unfortunately, we needed a fact about the relation between <v>rembind</v> and \u000A <v>bind</v> that wasn't supplied with the alists theory.  Hopefully, such \u000A omissions will be rare.</p> \u000A \u000A <p>Finally, let's consider our property 5 above: a name will not be in the \u000A book after we delete it.  We formalize this as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> in-book-del\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (in-book? nm (del-phone nm bk))))</code> \u000A \u000A <p>This proves easily.  But notice that it's only true because <v>del-phone</v> \u000A (actually <v>rembind</v>) removes <b>all</b> occurrences of a name from the \u000A phone book.  If it only removed, say, the first one it encountered, we'd need \u000A a hypothesis that said that <v>nm</v> occurs at most once in <v>bk</v>.  Ah, maybe \u000A that's a property you hadn't considered.  Maybe you want to ensure that \u000A <b>any</b> name occurs at most once in any valid phonebook.</p> \u000A \u000A <p>To complete this example, let's consider adding an <b>invariant</b> to our \u000A specification.  In particular, suppose we want to assure that no client has \u000A more than one associated phone number.  One way to ensure this is to require \u000A that the domain of the alist is a ``set'' (has no duplicates).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> setp (l)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> l)\u000A      (<see topic=\"COMMON-LISP____NULL\">null</see> l)\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))\u000A         (setp (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-phonebookp (bk)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (phonebookp bk)\u000A       (setp (domain bk))))</code> \u000A \u000A <p>Now, we want to show under what conditions our operations preserve the \u000A property of being a <v>valid-phonebookp</v>.  The operations <v>in-book?</v>  and \u000A <v>find-phone</v> don't return a phone book, so we don't really need to worry \u000A about them.  Since we're really interested in the ``types'' of values \u000A preserved by our phonebook functions, let's look at the types of those \u000A operations as well.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> in-book-booleanp\u000A  (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (in-book? nm bk)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> in-book-namep\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (phonebookp bk)\u000A                (in-book? nm bk))\u000A           (namep nm))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> bound?))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> find-phone-pnump\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (phonebookp bk)\u000A                (in-book? nm bk))\u000A           (pnump (find-phone nm bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> bound? binding))))</code> \u000A \u000A <p>Note the ``<v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>'' on the last two lemmas.  Neither of these \u000A would prove without these <see topic=\"ACL2____HINTS\">hints</see>, because once again there are some \u000A facts about <v>bound?</v> and <v>binding</v> not available in our current context. \u000A Now, we could figure out what those facts are and try to prove them. \u000A Alternatively, we can <see topic=\"ACL2____ENABLE\">enable</see> <v>bound?</v> and <v>binding</v> and hope that \u000A by opening up these functions, the conjectures will reduce to versions that \u000A the prover does know enough about or can prove by induction.  In this case, \u000A this strategy works.  The hints tell the prover to <see topic=\"ACL2____ENABLE\">enable</see> the functions \u000A in question when considering the designated goal.</p> \u000A \u000A <p>Below we develop the theorems showing that <v>add-phone</v>, \u000A <v>change-phone</v>, and <v>del-phone</v> preserve our proposed invariant.  Notice \u000A that along the way we have to prove some subsidiary facts, some of which are \u000A pretty ugly.  It would be a good idea for you to try, say, \u000A <v>add-phone-preserves-invariant</v> without introducing the following four \u000A lemmas first.  See if you can develop the proof and only add these lemmas as \u000A you need assistance.  Then try <v>change-phone-preserves-invariant</v> and \u000A <v>del-phone-preserves-invariant</v>.  They will be easier.  It is illuminating \u000A to think about why <v>del-phone-preserves-invariant</v> does not need any \u000A ``type'' hypotheses.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> bind-preserves-phonebookp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (phonebookp bk)\u000A                (namep nm)\u000A                (pnump num))\u000A           (phonebookp (bind nm num bk))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-equal-strip-cars-bind\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> a))))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> (bind y z a))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bind-preserves-domain-setp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ALISTP\">alistp</see> bk)\u000A                (setp (domain bk)))\u000A           (setp (domain (bind nm num bk))))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> domain))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> phonebookp-alistp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (phonebookp bk)\u000A           (<see topic=\"ACL2____ALISTP\">alistp</see> bk)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ADD-PHONE-PRESERVES-INVARIANT\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (valid-phonebookp bk)\u000A                (namep nm)\u000A                (pnump num))\u000A           (valid-phonebookp (add-phone nm num bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> domain-bind))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> CHANGE-PHONE-PRESERVES-INVARIANT\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (valid-phonebookp bk)\u000A                (namep nm)\u000A                (pnump num))\u000A           (valid-phonebookp (change-phone nm num bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> domain-bind))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> remove-equal-preserves-setp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (setp l)\u000A           (setp (<see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see> x l))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> rembind-preserves-phonebookp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (phonebookp bk)\u000A           (phonebookp (rembind nm bk))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> DEL-PHONE-PRESERVES-INVARIANT\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (valid-phonebookp bk)\u000A           (valid-phonebookp (del-phone nm bk))))</code> \u000A \u000A <p>As a final test of your understanding, try to formulate and prove an \u000A invariant that says that no phone number is assigned to more than one name. \u000A The following hints may help.</p> \u000A \u000A <blockquote> \u000A \u000A <p>1. Define the appropriate invariant.  (Hint: remember the function \u000A <v>range</v>.)</p> \u000A \u000A <p>2. Do our current definitions of <v>add-phone</v> and <v>change-phone</v> \u000A necessarily preserve this property?  If not, consider what hypotheses are \u000A necessary in order to guarantee that they do preserve this property.</p> \u000A \u000A <p>3. Study the definition of the function <v>range</v> and notice that it is \u000A defined in terms of the function <tt><see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see></tt>.  Understand how this \u000A defines the range of an alist.</p> \u000A \u000A <p>4. Formulate the correctness theorems and attempt to prove them.  You'll \u000A probably benefit from studying the invariant proof above.  In particular, you \u000A may need some fact about the function <tt><see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see></tt> analogous to the \u000A lemma <v>member-equal-strip-cars-bind</v> above.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Below is one solution to this exercise.  Don't look at the solution, \u000A however, until you've struggled a bit with it.  Notice that we didn't actually \u000A change the definitions of <v>add-phone</v> and <v>change-phone</v>, but added a \u000A hypothesis saying that the number is ``new.''  We could have changed the \u000A definitions to check this and return the phonebook unchanged if the number was \u000A already in use.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pnums-in-use (bk)\u000A  (range bk))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> phonenums-unique (bk)\u000A  (setp (pnums-in-use bk)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> new-pnump (pnum bk)\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> pnum (pnums-in-use bk))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-equal-strip-cdrs-rembind\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> y)))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> (rembind z y))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> DEL-PHONE-PRESERVES-PHONENUMS-UNIQUE\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (phonenums-unique bk)\u000A           (phonenums-unique (del-phone nm bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> range))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> strip-cdrs-bind-non-member\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (bound? x a))\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> a))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> (bind x y a))\u000A                  (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> a) (<see topic=\"COMMON-LISP____LIST\">list</see> y))))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> bound?))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> setp-append-list\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (setp l)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (setp (<see topic=\"COMMON-LISP____APPEND\">append</see> l (<see topic=\"COMMON-LISP____LIST\">list</see> x)))\u000A                  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x l)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ADD-PHONE-PRESERVES-PHONENUMS-UNIQUE\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (phonenums-unique bk)\u000A                (new-pnump pnum bk)\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> bk))\u000A           (phonenums-unique (add-phone nm pnum bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> range))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-equal-strip-cdrs-bind\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> z (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> a)))\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z y)))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> z (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> (bind x y a))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> CHANGE-PHONE-PRESERVES-PHONENUMS-UNIQUE\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (phonenums-unique bk)\u000A                (new-pnump pnum bk)\u000A                (<see topic=\"ACL2____ALISTP\">alistp</see> bk))\u000A           (phonenums-unique (change-phone nm pnum bk)))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> range))))</code>"],

 "ACL2____TUTORIAL1-TOWERS-OF-HANOI":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>This example was written almost entirely by Bill Young of \u000A Computational Logic, Inc.</p> \u000A \u000A <p>We will formalize and prove a small theorem about the famous ``Towers of \u000A Hanoi'' problem.  This problem is illustrated by the following picture.</p> \u000A \u000A <code>   |        |        |\u000A   |        |        |\u000A  ---       |        |\u000A -----      |        |\u000A-------     |        |\u000A\u000A   A        B        C</code> \u000A \u000A <p>We have three pegs &mdash; <v>a</v>, <v>b</v>, and <v>c</v> &mdash; and <v>n</v> \u000A disks of different sizes.  The disks are all initially on peg <v>a</v>.  The \u000A goal is to move all disks to peg <v>c</v> while observing the following two \u000A rules.</p> \u000A \u000A <p>1. Only one disk may be moved at a time, and it must start and finish the \u000A move as the topmost disk on some peg;</p> \u000A \u000A <p>2. A disk can never be placed on top of a smaller disk.</p> \u000A \u000A <p>Let's consider some simple instances of this problem.  If <v>n</v> = 1, i.e., \u000A only one disk is to be moved, simply move it from <v>a</v> to <v>c</v>.  If <v>n</v> \u000A = 2, i.e., two disks are to be moved, the following sequence of moves \u000A suffices: move from <v>a</v> to <v>b</v>, move from <v>a</v> to <v>c</v>, move from \u000A <v>b</v> to <v>c</v>.</p> \u000A \u000A <p>In this doc topic we will show an ACL2 function that generates a suitable \u000A list of moves for a tower of <v>n</v> disks.  Then we will use ACL2 to prove \u000A that the number of moves is <v>(<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 n) 1)</v>.  For an ACL2 script that \u000A proves the correctness of (a version of) this function, see the community book \u000A <v>&quot;misc/hanoi.lisp&quot;</v> in the <v>books</v> directory of your ACL2 sources.</p> \u000A \u000A <p>In general, this problem has a straightforward recursive solution.  Suppose \u000A that we desire to move <v>n</v> disks from <v>a</v> to <v>c</v>, using <v>b</v> as the \u000A intermediate peg.  For the basis, we saw above that we can always move a \u000A single disk from <v>a</v> to <v>c</v>.  Now if we have <v>n</v> disks and assume \u000A that we can solve the problem for <v>n-1</v> disks, we can move <v>n</v> disks as \u000A follows.  First, move <v>n-1</v> disks from <v>a</v> to <v>b</v> using <v>c</v> as the \u000A intermediate peg; move the single disk from <v>a</v> to <v>c</v>; then move \u000A <v>n-1</v> disks from <v>b</v> to <v>c</v> using <v>a</v> as the intermediate peg.</p> \u000A \u000A <p>In ACL2, we can write a function that will return the sequence of moves. \u000A One such function is as follows.  Notice that we have two base cases.  If \u000A <v>(<see topic=\"ACL2____ZP\">zp</see> n)</v> then <v>n</v> is not a positive integer; we treat that case as if \u000A <v>n</v> were 0 and return an empty list of moves.  If <v>n</v> is 1, then we \u000A return a list containing the single appropriate move.  Otherwise, we return \u000A the list containing exactly the moves dictated by our recursive analysis \u000A above.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> move (a b)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'move a 'to b))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hanoi (a b c n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 1)\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> (move a c))\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (hanoi a c b (<see topic=\"COMMON-LISP____1-\">1-</see> n))\u000A              (<see topic=\"COMMON-LISP____CONS\">cons</see> (move a c)\u000A                    (hanoi b a c (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))))</code> \u000A \u000A <p>Notice that we give <v>hanoi</v> four arguments: the three pegs, and the \u000A number of disks to move.  It is necessary to supply the pegs because, in \u000A recursive calls, the roles of the pegs differ.  In any execution of the \u000A algorithm, a given peg will sometimes be the source of a move, sometimes the \u000A destination, and sometimes the intermediate peg.</p> \u000A \u000A <p>After submitting these functions to ACL2, we can execute the <v>hanoi</v> \u000A function on various specific arguments.  For example:</p> \u000A \u000A <code>ACL2 !&gt;(hanoi 'a 'b 'c 1)\u000A((MOVE A TO C))\u000A\u000AACL2 !&gt;(hanoi 'a 'b 'c 2)\u000A((MOVE A TO B)\u000A (MOVE A TO C)\u000A (MOVE B TO C))\u000A\u000AACL2 !&gt;(hanoi 'a 'b 'c 3)\u000A((MOVE A TO C)\u000A (MOVE A TO B)\u000A (MOVE C TO B)\u000A (MOVE A TO C)\u000A (MOVE B TO A)\u000A (MOVE B TO C)\u000A (MOVE A TO C))</code> \u000A \u000A <p>From the algorithm it is clear that if it takes <v>m</v> moves to transfer \u000A <v>n</v> disks, it will take <v>(m + 1 + m) = 2m + 1</v> moves for <v>n+1</v> disks. \u000A From some simple calculations, we see that we need the following number of \u000A moves in specific cases:</p> \u000A \u000A <code>Disks   0   1   2   3   4   5   6   7  ...\u000AMoves   0   1   3   7  15  31  63  127 ...</code> \u000A \u000A <p>The pattern is fairly clear.  To move <v>n</v> disks requires <v>(2^n - 1)</v> \u000A moves.  Let's attempt to use ACL2 to prove that fact.</p> \u000A \u000A <p>First of all, how do we state our conjecture?  Recall that <v>hanoi</v> \u000A returns a list of moves.  The length of the list (given by the function \u000A <v>len</v>) is the number of moves required.  Thus, we can state the following \u000A conjecture.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> hanoi-moves-required-first-try\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (hanoi a b c n))\u000A         (<see topic=\"COMMON-LISP____1-\">1-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 n))))</code> \u000A \u000A <p>When we submit this to ACL2, the proof attempt fails.  Along the way we \u000A notice subgoals such as:</p> \u000A \u000A <code>Subgoal *1/1'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 N))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> 0 (<see topic=\"COMMON-LISP_____B2\">+</see> -1 (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> 2 N)))).</code> \u000A \u000A <p>This tells us that the prover is considering cases that are uninteresting \u000A to us, namely, cases in which <v>n</v> might be negative.  The only cases that \u000A are really of interest are those in which <v>n</v> is a non-negative natural \u000A number.  Therefore, we revise our theorem as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> hanoi-moves-required\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n))    ;; n is at least 0\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (hanoi a b c n))\u000A                  (<see topic=\"COMMON-LISP____1-\">1-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 n)))))</code> \u000A \u000A <p>and submit it to ACL2 again.</p> \u000A \u000A <p>Again the proof fails.  Examining the proof script we encounter the \u000A following text.  (How did we decide to focus on this goal?  Some information \u000A is provided in ACLH, and the ACL2 documentation on <see topic=\"ACL2____TIPS\">tips</see> may be helpful. \u000A But the simplest answer is: this was the first goal suggested by the ``<see topic=\"ACL2____PROOF-TREE\">proof-tree</see>'' tool below the start of the proof by induction.  See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.)</p> \u000A \u000A <code>Subgoal *1/5''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> N)\u000A              (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 N)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> N 1))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (HANOI A C B (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N)))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> -1 (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> 2 (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N))))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (HANOI B A C (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N)))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> -1 (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> 2 (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N)))))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (HANOI A C B (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N))\u000A                             (<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____LIST\">LIST</see> 'MOVE A 'TO C)\u000A                                   (HANOI B A C (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N)))))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> -1 (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"COMMON-LISP____EXPT\">EXPT</see> 2 (<see topic=\"COMMON-LISP_____B2\">+</see> -1 N))))))</code> \u000A \u000A <p>It is difficult to make much sense of such a complicated goal.  However, we \u000A do notice something interesting.  In the conclusion is a <see topic=\"ACL2____TERM\">term</see> of the \u000A following shape.</p> \u000A \u000A <code>(<see topic=\"ACL2____LEN\">LEN</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> ... ...))</code> \u000A \u000A <p>We conjecture that the length of the <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> of two lists should be \u000A the sum of the lengths of the lists.  If the prover knew that, it could \u000A possibly have simplified this <see topic=\"ACL2____TERM\">term</see> earlier and made more progress in \u000A the proof.  Therefore, we need a <see topic=\"ACL2____REWRITE\">rewrite</see> rule that will suggest such a \u000A simplification to the prover.  The appropriate rule is:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> len-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A         (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"ACL2____LEN\">len</see> y))))</code> \u000A \u000A <p>We submit this to the prover, which proves it by a straightforward \u000A induction.  The prover stores this lemma as a <see topic=\"ACL2____REWRITE\">rewrite</see> rule and will \u000A subsequently (unless we <see topic=\"ACL2____DISABLE\">disable</see> the rule) replace <see topic=\"ACL2____TERM\">term</see>s matching \u000A the left hand side of the rule with the appropriate instance of the <see topic=\"ACL2____TERM\">term</see> on the right hand side.</p> \u000A \u000A <p>We now resubmit our lemma <v>hanoi-moves-required</v> to ACL2.  On this \u000A attempt, the proof succeeds and we are done.</p> \u000A \u000A <p>One bit of cleaning up is useful.  We needed the hypotheses that:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)).</code> \u000A \u000A <p>This is an awkward way of saying that <v>n</v> is a natural number; natural \u000A is not a primitive data type in ACL2.  We could define a function \u000A <v>naturalp</v>, but it is somewhat more convenient to define a macro as \u000A follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> naturalp (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'and (<see topic=\"COMMON-LISP____LIST\">list</see> 'integerp x)\u000A                (<see topic=\"COMMON-LISP____LIST\">list</see> '&lt;= 0 x)))</code> \u000A \u000A <p>Subsequently, we can use <v>(naturalp n)</v> wherever we need to note that a \u000A quantity is a natural number.  See <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> for more information about \u000A ACL2 macros.  With this macro, we can reformulate our theorem as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> hanoi-moves-required\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (naturalp n)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (hanoi a b c n))\u000A                  (<see topic=\"COMMON-LISP____1-\">1-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 n))))).</code> \u000A \u000A <p>Another interesting (but much harder) theorem asserts that the list of \u000A moves generated by our <v>hanoi</v> function actually accomplishes the desired \u000A goal while following the rules.  When you can state and prove that theorem, \u000A you'll be a very competent ACL2 user.</p> \u000A \u000A <p>By the way, the name ``Towers of Hanoi'' derives from a legend that a group \u000A of Vietnamese monks works day and night to move a stack of 64 gold disks from \u000A one diamond peg to another, following the rules set out above.  We're told \u000A that the world will end when they complete this task.  From the theorem above, \u000A we know that this requires 18,446,744,073,709,551,615 moves:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____1-\">1-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 64))\u000A18446744073709551615\u000AACL2 !&gt;</code> \u000A \u000A <p>We're guessing they won't finish any time soon.</p>"],

 "ACL2____SYNTAXP-EXAMPLES":[["Syntaxp"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SYNTAXP\">syntaxp</see> for a basic discussion of the use of \u000A <v>syntaxp</v> to control rewriting.</p> \u000A \u000A <p>A common syntactic restriction is</p> \u000A \u000A <code>(<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X) (<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'QUOTE)))</code> \u000A \u000A <p>or, equivalently,</p> \u000A \u000A <code>(<see topic=\"ACL2____SYNTAXP\">SYNTAXP</see> (QUOTEP X)).</code> \u000A \u000A <p>A rule with such a hypothesis can be applied only if <v>x</v> is bound to a \u000A specific constant.  Thus, if <v>x</v> is <v>23</v> (which is actually represented \u000A internally as <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> 23)</v>), the test evaluates to <v>t</v>; but if <v>x</v> \u000A prints as <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 11 12)</v> then the test evaluates to <v>nil</v> (because <v>(<see topic=\"COMMON-LISP____CAR\">car</see>\u000A x)</v> is the symbol <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>).  We see the use of this restriction in \u000A the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep c))\u000A              (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep d)))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d x)\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d) x))).</code> \u000A \u000A <p>If <v>c</v> and <v>d</v> are constants, then the <tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> \u000A of <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> will evaluate the sum of <v>c</v> and <v>d</v>.  For instance, \u000A under the influence of this rule</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> 11 12 foo)</code> \u000A \u000A <p>rewrites to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 11 12) foo)</code> \u000A \u000A <p>which in turn rewrites to <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 23 foo)</v>.  Without the syntactic \u000A restriction, this rule would loop with the built-in rules \u000A <v>ASSOCIATIVITY-OF-+</v> or <v>COMMUTATIVITY-OF-+</v>.</p> \u000A \u000A <p>We here recommend that the reader try the affects of entering expressions \u000A such as the following at the top level ACL2 prompt.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> 11 23)\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> '11 23)\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> '11 '23)\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> ''11 ''23)\u000A:trans (<see topic=\"COMMON-LISP_____B2\">+</see> 11 23)\u000A:trans (<see topic=\"COMMON-LISP_____B2\">+</see> '11 23)\u000A:trans (<see topic=\"COMMON-LISP_____B2\">+</see> ''11 23)\u000A:trans (<see topic=\"COMMON-LISP_____B2\">+</see> c d x)\u000A:trans (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d) x)</code> \u000A \u000A <p>We also recommend that the reader verify our claim above about looping by \u000A trying the affect of each of the following rules individually.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> good\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep c))\u000A                 (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep d)))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d x)\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d) x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> c)\u000A                 (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> d))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d x)\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d) x))))</code> \u000A \u000A <p>on (the false) theorems:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 11 12 x) y))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> c)\u000A                (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> d)\u000A                (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c d x) y))).</code> \u000A \u000A <p>One can use <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt>, perhaps in conjunction with <tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt>, to investigate any looping.</p> \u000A \u000A <p>Here is a simple example showing the value of rule <v>good</v> above. \u000A Without <v>good</v>, the <v>thm</v> form below fails.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x) t)\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4 x)) (foo (<see topic=\"COMMON-LISP_____B2\">+</see> 7 x))))</code> \u000A \u000A <p>The next three examples further explore the use of <v>quote</v> in <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> hypotheses.</p> \u000A \u000A <p>We continue the examples of <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> hypotheses with a rule from \u000A community book <v>books/finite-set-theory/set-theory.lisp</v>.  We will not \u000A discuss here the meaning of this rule, but it is necessary to point out that \u000A <v>(ur-elementp nil)</v> is true in this book.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> scons-nil\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a ''nil)))\u000A                (ur-elementp a))\u000A           (<see topic=\"COMMON-LISP_____D3\">=</see> (scons e a)\u000A              (scons e nil)))).</code> \u000A \u000A <p>Here also, <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> is used to prevent looping.  Without the \u000A restriction, <v>(scons e nil)</v> would be rewritten to itself since \u000A <v>(ur-elementp nil)</v> is true.<br></br> \u000A \u000A Question: Why the use of two quotes in <v>''nil</v>?<br></br> \u000A \u000A Hints: <v>Nil</v> is a constant just as 23 is.  Try <v>:trans (<see topic=\"COMMON-LISP____CONS\">cons</see> a nil)</v>, \u000A <v>:trans (<see topic=\"COMMON-LISP____CONS\">cons</see> 'a 'nil)</v>, and <v>:trans (<see topic=\"COMMON-LISP____CONS\">cons</see> ''a ''nil)</v>.  Also, don't \u000A forget that the arguments to a function are evaluated before the function is \u000A applied.</p> \u000A \u000A <p>The next two rules move negative constants to the other side of an \u000A inequality.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> |(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> c) x) y)|\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep c))\u000A                (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> c) 0))\u000A                (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____B2\">+</see> c x) y)\u000A                  (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____FIX\">fix</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> c) y)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> |(<see topic=\"COMMON-LISP_____C3\">&lt;</see> y (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> c) x))|\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep c))\u000A                (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> c) 0))\u000A                (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y (<see topic=\"COMMON-LISP_____B2\">+</see> c x))\u000A                  (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> c) y) (<see topic=\"ACL2____FIX\">fix</see> x)))))</code> \u000A \u000A <p>Questions: What would happen if <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> c) '0)</v> were used?  What about \u000A <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> c) ''0)</v>?</p> \u000A \u000A <p>One can also use <v>syntaxp</v> to restrict the application of a rule to a \u000A particular set of variable bindings as in the following taken from the <see topic=\"ACL2____IHS\">ihs</see> library.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A\u000A  (<see topic=\"ACL2____LOCAL\">local</see>\u000A   (<see topic=\"ACL2____DEFTHM\">defthm</see> floor-+-crock\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A           (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y)\u000A           (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> z)\u000A           (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'x) (<see topic=\"COMMON-LISP____EQ\">eq</see> y 'y) (<see topic=\"COMMON-LISP____EQ\">eq</see> z 'z))))\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z)\u000A             (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____MOD\">mod</see> x z) (<see topic=\"COMMON-LISP____MOD\">mod</see> y z))\u000A                       (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> x z) (<see topic=\"COMMON-LISP____FLOOR\">floor</see> y z)) z)) z)))))\u000A\u000A  (<see topic=\"ACL2____DEFTHM\">defthm</see> floor-+\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____FORCE\">force</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x))\u000A          (<see topic=\"ACL2____FORCE\">force</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y))\u000A          (<see topic=\"ACL2____FORCE\">force</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> z))\u000A          (<see topic=\"ACL2____FORCE\">force</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z 0))))\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z)\u000A            (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____MOD\">mod</see> x z) (<see topic=\"COMMON-LISP____MOD\">mod</see> y z)) z)\u000A               (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> x z) (<see topic=\"COMMON-LISP____FLOOR\">floor</see> y z))))))\u000A\u000A  )</code> \u000A \u000A <p>We recommend the use of <v>:</v><v>brr</v> to investigate the use of \u000A <v>floor-+-crock</v>.</p> \u000A \u000A <p>Another useful restriction is defined by</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rewriting-goal-literal (x mfc state)\u000A\u000A  ;; Are we rewriting a top-level goal literal, rather than rewriting\u000A  ;; to establish a hypothesis from a rewrite (<see topic=\"COMMON-LISP____OR\">or</see> other) rule?\u000A\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x state))\u000A  (<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"ACL2____ACCESS\">access</see> metafunction-context mfc :ancestors))).</code> \u000A \u000A <p>We use this restriction in the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> |(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y) 0)|\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (rewriting-goal-literal x mfc state))\u000A                  (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                  (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x y) 0)\u000A                    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)\u000A                           nil)\u000A                          ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 0)\u000A                           nil)\u000A                          ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A                           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 y))\u000A                          ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x)\u000A                           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y 0))))))</code> \u000A \u000A <p>which has been found to be useful, but which also leads to excessive \u000A thrashing in the linear arithmetic package if used indiscriminately.</p> \u000A \u000A <p>See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see> for information on the use of <v>mfc</v> \u000A and <v>metafunction-context</v>.</p>"],

 "ACL2____NOTE8":[["Release-notes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____NOTE8-UPDATE\">note8-update</see> for yet more recent changes.</p> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see>s have been eliminated from the ACL2 logic.  A summary is \u000A contained in this brief note.  Also see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.</p> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see>s may be included in <see topic=\"ACL2____DEFUNS\">defuns</see> as usual but are ignored \u000A from the perspective of admission to the logic: functions must terminate on \u000A all arguments.</p> \u000A \u000A <p>As in Nqthm, primitive functions, e.g., <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> and <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>, \u000A logically default unexpected arguments to convenient values.  Thus, <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 'abc\u000A 3)</v> is <v>3</v> and <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</v> is <v>nil</v>.  See <see topic=\"ACL2____PROGRAMMING\">programming</see>, and \u000A see the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for the individual primitive functions.</p> \u000A \u000A <p>In contrast to earlier versions of ACL2, Version 1.8 logical functions are \u000A executed at Nqthm speeds even when <see topic=\"ACL2____GUARD\">guard</see>s have not been verified.  In \u000A versions before 1.8, such functions were interpreted by ACL2.</p> \u000A \u000A <p>Colors have been eliminated.  Two ``<see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s'' are supported, \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  Roughly speaking, <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> does what <v>:red</v> used to do, namely, allow you to prototype \u000A functions for execution without any proof burdens.  <v>:</v><tt><see topic=\"ACL2____LOGIC\">Logic</see></tt> mode \u000A does what <v>:blue</v> used to do, namely, allow you to add a new definitional \u000A axiom to the logic.  A global <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> is comparable to the \u000A old default color.  The system comes up in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  To \u000A change the global <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, type <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> at the top-level.  To specify the <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of a \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> locally use</p> \u000A \u000A <code> <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode mode))</v>.  </code> \u000A \u000A <p>The <see topic=\"ACL2____PROMPT\">prompt</see> has changed.  The initial <see topic=\"ACL2____PROMPT\">prompt</see>, indicating \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, is</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>If you change to <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode the <see topic=\"ACL2____PROMPT\">prompt</see> becomes</p> \u000A \u000A <code>ACL2 p!&gt;</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see>s can be seen as having either of two roles: (a) they are a \u000A specification device allowing you to characterize the kinds of inputs a \u000A function ``should'' have, or (b) they are an efficiency device allowing \u000A logically defined functions to be executed directly in Common Lisp.  If a \u000A <see topic=\"ACL2____GUARD\">guard</see> is specified, as with <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt>, then it \u000A is ``verified'' at defun-time (unless you also specify <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> \u000A <v>:verify-guards nil</v>).  <see topic=\"ACL2____GUARD\">Guard</see> verification means what it always has: \u000A the input <see topic=\"ACL2____GUARD\">guard</see> is shown to imply the <see topic=\"ACL2____GUARD\">guard</see>s on all subroutines \u000A in the body.  If the <see topic=\"ACL2____GUARD\">guard</see>s of a function are verified, then a call of \u000A the function on inputs satisfying the <see topic=\"ACL2____GUARD\">guard</see> can be computed directly by \u000A Common Lisp.  Thus, verifying the <see topic=\"ACL2____GUARD\">guard</see>s on your functions will allow \u000A them to execute more efficiently.  But it does not affect their logical \u000A behavior and since you will automatically get Nqthm speeds on unverified \u000A logical definitions, most users will probably use <see topic=\"ACL2____GUARD\">guard</see>s either as a \u000A specification device or only use them when execution efficiency is extremely \u000A important.</p> \u000A \u000A <p>Given the presence of <see topic=\"ACL2____GUARD\">guard</see>s in the system, two issues are \u000A unavoidable.  Are <see topic=\"ACL2____GUARD\">guard</see>s verified as part of the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> process? \u000A And are <see topic=\"ACL2____GUARD\">guard</see>s checked when terms are evaluated?  We answer both of \u000A those questions below.</p> \u000A \u000A <p>Roughly speaking, in its initial <see topic=\"ACL2____STATE\">state</see> the system will try to verify \u000A the <see topic=\"ACL2____GUARD\">guard</see>s of a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> if a <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> is supplied in \u000A the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> and will not try otherwise.  However, <see topic=\"ACL2____GUARD\">guard</see> \u000A verification in <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> can be inhibited ``locally'' by supplying the \u000A <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> <v>nil</v>.  ``Global'' inhibition can \u000A be obtained via the <v>:</v><tt><see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see></tt>.  If you do not \u000A use the <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>, you will not need to think about \u000A <see topic=\"ACL2____GUARD\">guard</see> verification.</p> \u000A \u000A <p>We now turn to the evaluation of expressions.  Even if your functions \u000A contain no <see topic=\"ACL2____GUARD\">guard</see>s, the primitive functions do and hence you have the \u000A choice: when you submit an expression for evaluation do you mean for <see topic=\"ACL2____GUARD\">guard</see>s to be checked at runtime or not?  Put another way, do you mean for the \u000A expression to be evaluated in Common Lisp (if possible) or in the logic? \u000A Note: If Common Lisp delivers an answer, it will be the same as in the logic, \u000A but it might be erroneous to execute the form in Common Lisp.  For example, \u000A should <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</v> cause a <see topic=\"ACL2____GUARD\">guard</see> violation error or return \u000A <v>nil</v>?</p> \u000A \u000A <p>The top-level ACL2 loop has a variable which controls which sense of \u000A execution is provided.  To turn ``<see topic=\"ACL2____GUARD\">guard</see> checking on,'' by which we mean \u000A that <see topic=\"ACL2____GUARD\">guard</see>s are checked at runtime, execute the top-level form \u000A <v>:set-guard-checking t</v>.  To turn it off, do <v>:set-guard-checking nil</v>. \u000A The status of this variable is reflected in the <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>means <see topic=\"ACL2____GUARD\">guard</see> checking is on and</p> \u000A \u000A <code>ACL2 &gt;</code> \u000A \u000A <p>means <see topic=\"ACL2____GUARD\">guard</see> checking is off.  The exclamation mark can be thought of \u000A as ``barring'' certain computations.  The absence of the mark suggests the \u000A absence of error messages or unbarred access to the logical axioms.  Thus, for \u000A example</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will signal an error, while</p> \u000A \u000A <code>ACL2 &gt;(<see topic=\"COMMON-LISP____CAR\">car</see> 'abc)</code> \u000A \u000A <p>will return <v>nil</v>.</p> \u000A \u000A <p>Note that whether or not <see topic=\"ACL2____GUARD\">guard</see>s are checked at runtime is \u000A independent of whether you are operating in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode or \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode and whether theorems are being proved or not. \u000A (Although it must be added that functions defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode cannot help but check their <see topic=\"ACL2____GUARD\">guard</see>s because no logical definition \u000A exists.)</p> \u000A \u000A <p>Version 1.8 permits the verification of the <see topic=\"ACL2____GUARD\">guard</see>s of theorems, thus \u000A insuring that all instances of the theorem will evaluate without error in \u000A Common Lisp.  To verify the <see topic=\"ACL2____GUARD\">guard</see>s of a theorem named <v>name</v> execute \u000A the event</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> name).</code> \u000A \u000A <p>If a theorem's <see topic=\"ACL2____GUARD\">guard</see>s have been verified, the theorem is guaranteed \u000A to evaluate without error to non-<v>nil</v> in Common Lisp (provided resource \u000A errors do not arise).</p> \u000A \u000A <p>Caveat about <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>: <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> is a function symbol, \u000A so in the term <v>(<see topic=\"ACL2____IMPLIES\">implies</see> p q)</v>, <v>p</v> cannot be assumed true when <v>q</v> \u000A is evaluated; they are both evaluated ``outside.''  Hence, you cannot \u000A generally verify the <see topic=\"ACL2____GUARD\">guard</see>s on a theorem if <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> is used to \u000A state the hypotheses.  Use <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> instead.  In a future version of ACL2, \u000A <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> will likely be a macro.</p> \u000A \u000A <p>See sum-list-example.lisp for a nice example of the use of Version 1.8. \u000A This is roughly the same as the documentation for <see topic=\"ACL2____GUARD-EXAMPLE\">guard-example</see>.</p> \u000A \u000A <p>We have removed the capability to do ``old-style-forcing'' as existed \u000A before Version 1.5.  See <see topic=\"ACL2____NOTE5\">note5</see>.</p> \u000A \u000A <p>NOTE: Some low level details have, of course, changed.  One such change is \u000A that there are no longer two distinct type prescriptions stored when a \u000A function is admitted with its <see topic=\"ACL2____GUARD\">guard</see>s verified.  So for example, the \u000A type prescription <see topic=\"ACL2____RUNE\">rune</see> for <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> is now</p> \u000A \u000A <code>(:type-prescription binary-append)</code> \u000A \u000A <p>while in Versions 1.7 and earlier, there were two such <see topic=\"ACL2____RUNE\">rune</see>s:</p> \u000A \u000A <code>(:type-prescription binary-append . 1)\u000A(:type-prescription binary-append . 2)</code> \u000A \u000A <p>Nqthm-style forcing on <see topic=\"ACL2____LINEAR\">linear</see> arithmetic assumptions is no longer \u000A executed when forcing is <see topic=\"ACL2____DISABLE\">disable</see>d.</p> \u000A \u000A <p>Functional instantiation now benefits from a trick also used in Nqthm: once \u000A a <see topic=\"ACL2____CONSTRAINT\">constraint</see> generated by a <v>:functional-instance</v> lemma instance \u000A (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>) has been proved on behalf of a successful event, \u000A it will not have to be re-proved on behalf of a later event.</p> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____1_B2\">1+</see></tt> and <tt><see topic=\"COMMON-LISP____1-\">1-</see></tt> are now macros in the logic, not functions. \u000A Hence, for example, it is ``safe'' to use them on left-hand sides of rewrite \u000A rules, without invoking the common warning about the presence of nonrecursive \u000A function symbols.</p> \u000A \u000A <p>A new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> section <see topic=\"ACL2____FILE-READING-EXAMPLE\">file-reading-example</see> illustrates \u000A how to process forms in a file.</p> \u000A \u000A <p>A new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>forwardchain</v> has been added; see \u000A <see topic=\"ACL2-PC____FORWARDCHAIN\">ACL2-pc::forwardchain</see>.</p> \u000A \u000A <p>It is now possible to use quantifiers.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> and see <see topic=\"ACL2____DEFCHOOSE\">defchoose</see>.</p> \u000A \u000A <p>There is a new event <tt><see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see></tt>, which allows the user to \u000A turn off warnings of various types.  see <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>.</p> \u000A \u000A <p>An unsoundness relating <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and <v>:functional-instance</v> \u000A <see topic=\"ACL2____HINTS\">hints</see> has been remedied, with a few small effects visible at the user \u000A level.  The main observable effect is that <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> and non-local \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> <see topic=\"ACL2____EVENTS\">events</see> are no longer allowed in the scope of any \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event that has a non-empty <see topic=\"ACL2____SIGNATURE\">signature</see>.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is called, we now require that the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) be <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  On a \u000A related note, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is irrelevant to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>; the mode is always set to <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> initially, though \u000A it may be changed within the book and reverts to its original value at the \u000A conclusion of the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  A bug in <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A prevented it from acting this way even though the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> said \u000A otherwise.</p> \u000A \u000A <p>The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been substantially improved.  A new section \u000A ``Programming'' contains <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of many useful functions \u000A provided by ACL2; see <see topic=\"ACL2____PROGRAMMING\">programming</see>.  Also, the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has \u000A been ``marked up'' extensively.  Thus in particular, users of Mosaic will find \u000A many links in the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p>The symbols <tt><see topic=\"ACL2____FORCE\">force</see></tt>, <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt>, and <v>acl2-count</v> have been \u000A added to the list <v>*acl2-exports*</v>.</p> \u000A \u000A <p>We now permit most names from the main Lisp package to be used as names, \u000A except for names that define functions, macros, or constants.  See <see topic=\"ACL2____NAME\">name</see>.</p> \u000A \u000A <p>We have changed the list of imports from the Common Lisp package to ACL2, \u000A i.e., the list <v>*common-lisp-symbols-from-main-lisp-package*</v>, to be \u000A exactly those external symbols of the Common Lisp package as specified by the \u000A draft Common Lisp standard.  In order to accommodate this change, we have \u000A renamed some ACL2 functions as shown below, but these and other ramifications \u000A of this change should be transparent to most ACL2 users.</p> \u000A \u000A <code>warning      --&gt; warning$\u000Aprint-object --&gt; print-object$</code> \u000A \u000A <p>Proof trees are no longer enabled by default.  To start them up, \u000A <v>:</v><tt><see topic=\"ACL2____START-PROOF-TREE\">start-proof-tree</see></tt>.</p> \u000A \u000A <p>We have added the capability of building smaller images.  The easiest way \u000A to do this on a Unix (trademark of AT&amp;T) system is: <v>make small</v>.</p> \u000A \u000A <p>Here we will put some less important changes, additions, and so on.</p> \u000A \u000A <p>We have added definitions for the Common Lisp function <tt><see topic=\"COMMON-LISP____POSITION\">position</see></tt> \u000A (for the test <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>), as well as corresponding versions <tt><see topic=\"ACL2____POSITION-EQUAL\">position-equal</see></tt> and <tt><see topic=\"ACL2____POSITION-EQ\">position-eq</see></tt> that use tests <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, respectively.  See <see topic=\"COMMON-LISP____POSITION\">position</see>, see <see topic=\"ACL2____POSITION-EQUAL\">position-equal</see>, and \u000A see <see topic=\"ACL2____POSITION-EQ\">position-eq</see>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event <v>rational-listp-implies-rationalp-car</v> no \u000A longer exists.</p> \u000A \u000A <p>We fixed a bug in the hint mechanism that applied <v>:by</v>, <v>:cases</v>, \u000A and <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see> to the first induction goal when the prover \u000A reverted to proving the original goal by induction.</p> \u000A \u000A <p>We fixed a bug in the handling of <v>(<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see>\u000A :warn)</v>.</p> \u000A \u000A <p>In support of removing the old-style forcing capability, we deleted the \u000A initialization of <see topic=\"ACL2____STATE\">state</see> global <v>old-style-forcing</v> and deleted the \u000A definitions of <v>recover-assumptions</v>, <v>recover-assumptions-from-goal</v>, \u000A <v>remove-assumptions1</v>, <v>remove-assumptions</v>, and \u000A <v>split-on-assumptions</v>, and we renamed <v>split-on-assumptions1</v> to \u000A <v>split-on-assumptions</v>.</p> \u000A \u000A <p>The special value <v>'none</v> in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands \u000A <v>claim</v> and <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt> has been replaced by <v>:none</v>.</p> \u000A \u000A <p>A bug in the handling of <see topic=\"ACL2____HINTS\">hints</see> by subgoals has been fixed.  For \u000A example, formerly a <v>:do-not</v> hint could be ``erased'' by a <v>:use</v> hint \u000A on a subgoal.  Thanks go to Art Flatau for noticing the bug.</p> \u000A \u000A <p>The functions <v>weak-termp</v> and <v>weak-term-listp</v> have been deleted, \u000A and their calls have been replaced by corresponding calls of <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> and <v>pseudo-term-listp</v>.  The notion of <tt><see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see></tt> \u000A has been slightly strenthened by requiring that terms of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A ...)</v> have length 2.</p> \u000A \u000A <p>Performance has been improved in various ways.  At the prover level, \u000A backchaining through the recognizer alist has been eliminated in order to \u000A significantly speed up ACL2's rewriter.  Among the other prover changes (of \u000A which there are several, all technical): we no longer clausify the input term \u000A when a proof is interrupted in favor of inducting on the input term.  At the \u000A <see topic=\"ACL2____IO\">io</see> level, we have improved performance somewhat by suitable \u000A declarations and proclamations.  These include technical modifications to the \u000A macros <tt><see topic=\"ACL2____MV\">mv</see></tt> and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>, and introduction of a macro <v>the-mv</v> \u000A analogous to the macro <tt><see topic=\"COMMON-LISP____THE\">the</see></tt> but for forms returning multiple \u000A values.</p> \u000A \u000A <p>The function <v>spaces</v> now takes an extra argument, the current \u000A column.</p> \u000A \u000A <p>A bug in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <v>equiv</v> command was fixed.</p> \u000A \u000A <p>The function <v>intersectp</v> has been deleted, because it was essentially \u000A duplicated by the function <tt><see topic=\"ACL2____INTERSECTP-EQUAL\">intersectp-equal</see></tt>.</p> \u000A \u000A <p>We now proclaim functions in AKCL and GCL before compiling <see topic=\"ACL2____BOOKS\">books</see>. \u000A This should result in somewhat increased speed.</p> \u000A \u000A <p>The function <v>repeat</v> has been eliminated; use <tt><see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see></tt> \u000A instead.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>expand</v> has been fixed so that it \u000A eliminates <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> (lambda) expressions when one would expect it to.</p> \u000A \u000A <p>A new primitive function, <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt>, has been introduced.  <tt><see topic=\"ACL2____MV-NTH\">Mv-nth</see></tt> is equivalent to <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> and is used in place of <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> in \u000A the translation of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> expressions.  This allows the user to \u000A control the simplification of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> expressions without affecting how \u000A <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> is treated.  In that spirit, the rewriter has been modified so \u000A that certain <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> expressions, namely those produced in the \u000A translation of <v>(<see topic=\"ACL2____MV-LET\">mv-let</see> (a b c)(<see topic=\"ACL2____MV\">mv</see> x y z) p)</v>, are given special \u000A treatment.</p> \u000A \u000A <p>A minor bug in <v>untranslate</v> has been fixed, which for example will fix \u000A the printing of conjunctions.</p> \u000A \u000A <p><v>Translate</v> now takes a <v>logicp</v> argument, which indicates whether it \u000A enforces the restriction that <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions do not \u000A occur in the result.</p> \u000A \u000A <p>The modified version of <v>trace</v> provided by ACL2, for use in raw Lisp, \u000A has been modified so that the lisp special variable <v>*trace-alist*</v> has a \u000A slightly different functionality.  This alist associates, using <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, \u000A symbols with the print representations of their values.  For example, \u000A initially <v>*trace-alist*</v> is a one-element list containing the pair \u000A <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> 'state '|*the-live-state*|)</v>.  Thus, one may cons the pair <v>(<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A '*foo* &quot;It's a FOO!&quot;)</v> on to <v>*trace-alist*</v>; then until <v>*foo*</v> is \u000A defined, this change will have no effect, but after for example</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *foo* 17)</code> \u000A \u000A <p>then <v>trace</v> will print <v>17</v> as <v>&quot;It's a FOO!&quot;</v>.</p> \u000A \u000A <p><v>Trace</v> also traces the corresponding logic function.</p> \u000A \u000A <p><see topic=\"ACL2____PROOF-TREE\">Proof-tree</see> display has been improved slightly in the case of \u000A successful proofs and certain event failures.</p> \u000A \u000A <p>The function <v>positive-integer-log2</v> has been deleted.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> now prints a warning message when it is \u000A encountered in the context of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event or a book.  See \u000A <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>.</p> \u000A \u000A <p>Some functions related to <v>the-fn</v> and <v>wormhole1</v> now have <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, but this change is almost certain to be \u000A inconsequential to all users.</p>"],

 "ACL2____NOTE7":[["Release-notes"],"ACL2 Sources","ACL2","<p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> now takes (optionally) an additional keyword \u000A argument, indicating whether a compiled file is to be loaded.  The default \u000A behavior is unchanged, except that a warning is printed when a compiled file \u000A is not loaded.  See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>.</p> \u000A \u000A <p>A <see topic=\"ACL2____MARKUP\">markup</see> language for <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings has been \u000A implemented, and many of the source files have been marked up using this \u000A language (thanks largely to the efforts of Laura Lawless).  See <see topic=\"ACL2____MARKUP\">markup</see>. \u000A Moreover, there are translators that we have used to provide versions of the \u000A ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> in info (for use in emacs), html (for Mosaic), and \u000A tex (for hardcopy) formats.</p> \u000A \u000A <p>A new event <tt><see topic=\"ACL2____DEFDOC\">defdoc</see></tt> has been implemented.  It is like <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>, but allows redefinition of <see topic=\"ACL2____DOC\">doc</see> strings and has other \u000A advantages.  See <see topic=\"ACL2____DEFDOC\">defdoc</see>.</p> \u000A \u000A <p>We used to ignore corollaries when collecting up the axioms introduced \u000A about constrained functions.  That bug has been fixed.  We thank John Cowles \u000A for bringing this bug to our attention.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFSTUB\">defstub</see></tt> now allows a <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> keyword argument, \u000A so that <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> may be attached to the name being introduced.</p> \u000A \u000A <p>A new command <tt><see topic=\"ACL2____NQTHM-TO-ACL2\">nqthm-to-ACL2</see></tt> has been added to help Nqthm users to \u000A make the transition to ACL2.  See <see topic=\"ACL2____NQTHM-TO-ACL2\">nqthm-to-ACL2</see>, which also includes a \u000A complete listing of the relevant tables.</p> \u000A \u000A <p>Many function names, especially of the form ``foo<v>-lst</v>'', have been \u000A changed in order to support the following convention, for any ``foo'':</p> \u000A \u000A <p><v>(foo-listp lst)</v> represents the notion <v>(for x in lst always foop\u000A x)</v>.</p> \u000A \u000A <p>A complete list of these changes may be found at the end of this note.  All \u000A of them except <v>symbolp-listp</v> and <v>list-of-symbolp-listp</v> have the \u000A string ``<v>-lst</v>'' in their names.  Note also that <v>keyword-listp</v> has \u000A been renamed <tt><see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see></tt>.</p> \u000A \u000A <p>Accumulated persistence has been implemented.  It is not connected to \u000A <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt> or rule monitoring.  See <see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see>.</p> \u000A \u000A <p><v>:Trigger-terms</v> has been added for <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule classes, \u000A so you can hang a <see topic=\"ACL2____LINEAR\">linear</see> rule under any addend you want.  See <see topic=\"ACL2____LINEAR\">linear</see>, which has been improved and expanded.</p> \u000A \u000A <p>ACL2 now accepts <v>256</v> <see topic=\"ACL2____CHARACTERS\">characters</see> and includes the Common Lisp \u000A functions <tt><see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see></tt> and <tt><see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see></tt>.  However, ACL2 controls the \u000A lisp reader so that <v>#\\c</v> may only be used when <v>c</v> is a single \u000A standard character or one of <v>Newline</v>, <v>Space</v>, <v>Page</v>, <v>Rubout</v>, \u000A <v>Tab</v>.  If you want to enter other <see topic=\"ACL2____CHARACTERS\">characters</see> use <tt><see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see></tt>, \u000A e.g., <v>(<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 7) (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 240) #a) 'string)</v>.  See \u000A <see topic=\"ACL2____CHARACTERS\">characters</see>.  Note: our current handling of <see topic=\"ACL2____CHARACTERS\">characters</see> makes the \u000A set of theorems different under Macintosh Common Lisp (MCL) than under other \u000A Common Lisps.  We hope to rectify this situation before the final release of \u000A ACL2.</p> \u000A \u000A <p>A new <see topic=\"ACL2____TABLE\">table</see>, <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>, has been implemented, that \u000A associates macro names with function names.  So for example, since <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> is associated with <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt>, the form <v>(<see topic=\"ACL2____DISABLE\">disable</see>\u000A append)</v> it is interpreted as though it were <v>(<see topic=\"ACL2____DISABLE\">disable</see> binary-append)</v>. \u000A See <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>, see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> and see <see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see>.</p> \u000A \u000A <p>The implementation of conditional metalemmas has been modified so that the \u000A metafunction is applied before the hypothesis metafunction is applied.  See \u000A <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <p>The Common Lisp functions <tt><see topic=\"COMMON-LISP____ACONS\">acons</see></tt> and <tt><see topic=\"COMMON-LISP____ENDP\">endp</see></tt> have been defined \u000A in the ACL2 logic.</p> \u000A \u000A <p>We have added the symbol <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> to the list <v>*acl2-exports*</v>, \u000A and hence to the package <v>&quot;ACL2-USER&quot;</v>.</p> \u000A \u000A <p>A new hint, <v>:restrict</v>, has been implemented.  See <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>It used to be that if <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> were given a number that is greater \u000A than the largest current <see topic=\"ACL2____COMMAND\">command</see> number, it treated that number the \u000A same as <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt>.  Now, an error is caused.</p> \u000A \u000A <p>The <see topic=\"ACL2____TABLE\">table</see> <v>:force-table</v> has been eliminated.</p> \u000A \u000A <p>A command <v>:</v><tt><see topic=\"ACL2____DISABLEDP\">disabledp</see></tt> (and macro <tt><see topic=\"ACL2____DISABLEDP\">disabledp</see></tt>) has been \u000A added; see <see topic=\"ACL2____DISABLEDP\">disabledp</see>.</p> \u000A \u000A <p><see topic=\"ACL2____COMPILATION\">Compilation</see> via <v>:</v><tt><see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see></tt> is now suppressed \u000A during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  In fact, whenever the <see topic=\"ACL2____STATE\">state</see> global \u000A variable <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> has value <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>Here are some less important changes, additions, and so on.</p> \u000A \u000A <p>Unlike previous releases, we have not proved all the theorems in \u000A <v>axioms.lisp</v>; instead we have simply assumed them.  We have deferred such \u000A proofs because we anticipate a fairly major changed in Version 1.8 in how we \u000A deal with <see topic=\"ACL2____GUARD\">guard</see>s.</p> \u000A \u000A <p>We used to (accidentally) prohibit the ``redefinition'' of a <see topic=\"ACL2____TABLE\">table</see> \u000A as a function.  That is no longer the case.</p> \u000A \u000A <p>The check for whether a <see topic=\"ACL2____COROLLARY\">corollary</see> follows tautologically has been \u000A sped up, at the cost of making the check less ``smart'' in the following \u000A sense: no longer do we expand primitive functions such as <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> \u000A before checking this propositional implication.</p> \u000A \u000A <p>The <see topic=\"ACL2____COMMAND\">command</see> <tt><see topic=\"ACL2____UBT_12\">ubt!</see></tt> has been modified so that it never causes \u000A or reports an error.  See <see topic=\"ACL2____UBT_12\">ubt!</see>.</p> \u000A \u000A <p>ACL2 now works in Harlequin LispWorks.</p> \u000A \u000A <p>The user can now specify the <v>:trigger-terms</v> for <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> \u000A rules.  See <see topic=\"ACL2____LINEAR\">linear</see>.</p> \u000A \u000A <p>The name of the system is now ``ACL2''; no longer is it ``Acl2''.</p> \u000A \u000A <p>The raw lisp counterpart of <tt><see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see></tt> is now defined to be a \u000A no-op as is consistent with the idea that it is just a call of <tt><see topic=\"ACL2____TABLE\">table</see></tt>.</p> \u000A \u000A <p>A bug was fixed that caused <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <see topic=\"ACL2____INSTRUCTIONS\">instructions</see> to be \u000A executed when <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> was <v>t</v>.</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____RASSOC\">rassoc</see></tt> has been added, along with a corresponding \u000A function used in its <see topic=\"ACL2____GUARD\">guard</see>, <v>r-eqlable-alistp</v>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event and hint now print a warning not only when \u000A certain ``primitive'' <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules are <see topic=\"ACL2____DISABLE\">disable</see>d, but \u000A also when certain ``primitive'' <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> rules are \u000A <see topic=\"ACL2____DISABLE\">disable</see>d.</p> \u000A \u000A <p>The modified version of <v>trace</v> provided by ACL2, for use in raw Lisp, \u000A has been modified so that the lisp special variable <v>*trace-alist*</v> is \u000A consulted.  This alist associates, using <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, values with their print \u000A representations.  For example, initially <v>*trace-alist*</v> is a one-element \u000A list containing the pair <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> state '|*the-live-state*|)</v>.</p> \u000A \u000A <p>The system now prints an observation when a form is skipped because the \u000A default color is <v>:red</v> or <v>:pink</v>.  (Technically: <v>when-cool</v> has \u000A been modified.)</p> \u000A \u000A <p>Additional protection exists when you submit a form to raw Common Lisp that \u000A should only be submitted inside the ACL2 read-eval-print loop.</p> \u000A \u000A <p>Here is a complete list of the changes in function names described near the \u000A top of this note, roughly of the form</p> \u000A \u000A <code>foo-lst --&gt; foo-listp</code> \u000A \u000A <p>meaning: the name ``<v>foo-lst</v>'' has been changed to \u000A ``<v>foo-listp</v>.''</p> \u000A \u000A <code>symbolp-listp    --&gt; symbol-listp\u000Alist-of-symbolp-listp  --&gt; symbol-list-listp\u000A                       {for consistency with change to symbol-listp}\u000Arational-lst     --&gt; rational-listp\u000A                     {which in fact was already defined as well}\u000Ainteger-lst      --&gt; integer-listp\u000Acharacter-lst    --&gt; character-listp\u000Astringp-lst      --&gt; string-listp\u000A32-bit-integer-lst   --&gt; 32-bit-integer-listp\u000Atyped-io-lst     --&gt; typed-io-listp\u000Aopen-channel-lst --&gt; open-channel-listp\u000Areadable-files-lst   --&gt; readable-files-listp\u000Awritten-file-lst --&gt; written-file-listp\u000Aread-file-lst    --&gt; read-file-listp\u000Awriteable-file-lst   --&gt; writable-file-listp\u000A                     {note change in spelling of ``writable''}\u000Awriteable-file-lst1  --&gt; writable-file-listp1\u000Apseudo-termp-lst     --&gt; pseudo-term-listp\u000Ahot-termp-lst --&gt; hot-term-listp {by analogy with pseudo-term-listp}\u000Aweak-termp-lst   --&gt; weak-term-listp\u000Aweak-termp-lst-lst   --&gt; weak-termp-list-listp\u000Ats-builder-case-lstp -&gt; ts-builder-case-listp\u000Aquotep-lst       --&gt; quote-listp\u000Atermp-lst        --&gt; term-listp\u000Ainstr-lst        --&gt; instr-listp\u000Aspliced-instr-lst    --&gt; spliced-instr-listp\u000Arewrite-fncallp-lst  --&gt; rewrite-fncallp-listp\u000Aevery-occurrence-equiv-hittablep1-lst --&gt;\u000A            every-occurrence-equiv-hittablep1-listp\u000Asome-occurrence-equiv-hittablep1-lst  --&gt;\u000A            some-occurrence-equiv-hittablep1-listp\u000A            {by analogy with the preceding, even though it's a\u000A             ``some'' instead of ``all'' predicate]\u000Aalmost-quotep1-lst   --&gt; almost-quotep1-listp\u000Affnnames-subsetp-lst --&gt; ffnnames-subsetp-listp\u000Aboolean-lstp     --&gt; boolean-listp\u000Asubst-expr1-lst-okp  --&gt; subst-expr1-ok-listp</code>"],

 "ACL2____NOTE6":[["Release-notes"],"ACL2 Sources","ACL2","<p>A new key has been implemented for the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, \u000A <v>:ignore-ok</v>.  See <see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see>.</p> \u000A \u000A <p>It is now legal to have color <see topic=\"ACL2____EVENTS\">events</see>, such as <v>(red)</v>, in the \u000A <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of a book.  More generally, it is legal to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> in the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of a book.  For example, if you \u000A execute <v>:red</v> and then certify a book, the event <v>(red)</v> will show up \u000A in the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of that book, and hence the definitions in that book \u000A will all be red (except when overridden by appropriate declarations or <see topic=\"ACL2____EVENTS\">events</see>).  When that book is included, then as always, its <see topic=\"ACL2____PORTCULLIS\">portcullis</see> \u000A must first be ``raised,'' and that will cause the default color to become red \u000A before the <see topic=\"ACL2____EVENTS\">events</see> in the book are executed.  As always, the value of \u000A <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> immediately after execution of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form will be the \u000A same as it was immediately before execution (and hence, so will the default \u000A color).  See <see topic=\"ACL2____PORTCULLIS\">portcullis</see> and, for more about books, see <see topic=\"ACL2____BOOKS\">books</see>.</p> \u000A \u000A <p>A theory <tt><see topic=\"ACL2____GROUND-ZERO\">ground-zero</see></tt> has been defined to contain exactly those \u000A rules that are <see topic=\"ACL2____ENABLE\">enable</see>d when Acl2 starts up.  See <see topic=\"ACL2____GROUND-ZERO\">ground-zero</see>.</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> is now <see topic=\"ACL2____ENABLE\">enable</see>d, correcting an oversight \u000A from Version 1.5.</p> \u000A \u000A <p>Customization files no longer need to meet the syntactic restrictions put \u000A on <see topic=\"ACL2____BOOKS\">books</see>; rather, they can contain arbitrary Acl2 forms.  See <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>Structured directory names and structured file names are supported; see \u000A especially the documentation for <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>, <see topic=\"ACL2____BOOK-NAME\">book-name</see>, and <tt><see topic=\"ACL2____CBD\">cbd</see></tt>.</p> \u000A \u000A <p>Acl2 now works with some Common Lisp implementations other than akcl, \u000A including Lucid, Allegro, and MCL.</p> \u000A \u000A <p>A facility has been added for displaying proof trees, especially using \u000A emacs; see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>There is a considerable amount of new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, in particular \u000A for the printing functions <tt><see topic=\"ACL2____FMT\">fmt</see></tt>, <tt><see topic=\"ACL2____FMT1\">fmt1</see></tt>, and <tt><see topic=\"ACL2____FMS\">fms</see></tt>, and for \u000A the notion of Acl2 term (see <see topic=\"ACL2____TERM\">term</see>).</p> \u000A \u000A <p>It is possible to introduce new well-founded relations, to specify which \u000A relation should be used by <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, and to set a default relation.  See \u000A <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.</p> \u000A \u000A <p>It is possible to make functions suggest new inductions.  See <see topic=\"ACL2____INDUCTION\">induction</see>.</p> \u000A \u000A <p>It is possible to change how Acl2 expresses <see topic=\"ACL2____TYPE-SET\">type-set</see> information; in \u000A particular, this affects what clauses are proved when <see topic=\"ACL2____FORCE\">force</see>d \u000A assumptions are generated.  See <see topic=\"ACL2____TYPE-SET-INVERTER\">type-set-inverter</see>.</p> \u000A \u000A <p>A new restriction has been added to <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, having to do with \u000A undoing.  If you undo a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> and define the same package name again, \u000A the imports list must be identical to the previous imports or else an \u000A explanatory error will occur.  See <see topic=\"ACL2____PACKAGE-REINCARNATION-IMPORT-RESTRICTIONS\">package-reincarnation-import-restrictions</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____THEORY-INVARIANT\">Theory-invariant</see></tt> and <tt><see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see></tt> are now \u000A embedded event forms.</p> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt> may now be used to quit entirely out of \u000A Lisp, thus losing your work forever.  This command works in akcl but may not \u000A work in every Common Lisp.</p> \u000A \u000A <p>A theory <tt><see topic=\"ACL2____GROUND-ZERO\">ground-zero</see></tt> has been added that contains exactly the <see topic=\"ACL2____ENABLE\">enable</see>d rules in the <see topic=\"ACL2____STARTUP\">startup</see> theory.  See <see topic=\"ACL2____GROUND-ZERO\">ground-zero</see>.</p> \u000A \u000A <p><v>Define-pc-macro</v> and <v>define-pc-atomic-macro</v> now automatically \u000A define <v>:red</v> functions.  (It used to be necessary, in general, to change \u000A color to <v>:red</v> before invoking these.)</p> \u000A \u000A <p>For a proof of the well-foundedness of <v>e0-ord-&lt;</v> on the \u000A <v>e0-ordinalp</v>s, see <see topic=\"ACL2____PROOF-OF-WELL-FOUNDEDNESS\">proof-of-well-foundedness</see>.  [Note added later: \u000A Starting with Version_2.8, <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> and <tt><see topic=\"ACL2____O-P\">o-p</see></tt> replace <v>e0-ord-&lt;</v> \u000A and <v>e0-ordinalp</v>, respectively.]</p> \u000A \u000A <p>Free variables are now handled properly for hypotheses of <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.</p> \u000A \u000A <p>When the system is loaded or saved, <tt><see topic=\"ACL2____STATE\">state</see></tt> is now bound to \u000A <v>*the-live-state*</v>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____CERTIFY-BOOK\">Certify-book</see></tt> has been modified so that when it compiles a file, it \u000A loads that object file.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFSTUB\">Defstub</see></tt> has been modified so that it works when the color is hot \u000A (<v>:red</v> or <v>:pink</v>).</p> \u000A \u000A <p>Several basic, but not particularly commonly used, <see topic=\"ACL2____EVENTS\">events</see> have been \u000A added or changed.  The obscure axiom <v>symbol-name-intern</v> has been \u000A modified.  The definition of <v>firstn</v> has been changed.  <tt><see topic=\"COMMON-LISP____BUTLAST\">Butlast</see></tt> is \u000A now defined.  The definition of <tt><see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see></tt> has been modified.  The \u000A left-hand side of the rewrite rule <v>rational-implies2</v> has been changed \u000A from <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x) (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)))</v> to <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x))\u000A (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x))</v>, in order to respect the fact that <tt><see topic=\"ACL2____UNARY-_F2\">unary-/</see></tt> is \u000A invisible with respect to <tt><see topic=\"ACL2____BINARY-_A2\">binary-*</see></tt>.  See <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.</p> \u000A \u000A <p>The `preprocess' process in the waterfall (see <see topic=\"ACL2____HINTS\">hints</see> for a \u000A discussion of the <v>:do-not</v> hint) has been changed so that it works to \u000A avoid case-splitting.  The `simplify' process refuses to force (see <see topic=\"ACL2____FORCE\">force</see>) when there are <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> terms, including <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> \u000A terms, in the goal being simplified.</p> \u000A \u000A <p>The function <v>apply</v> is no longer introduced automatically by \u000A translation of user input to internal form when functions are called on \u000A inappropriate explicit values, e.g., <v>(<see topic=\"COMMON-LISP____CAR\">car</see> 3)</v>.</p> \u000A \u000A <p>The choice of which variable to use as the measured variable in a recursive \u000A definition has been very slightly changed.</p>"],

 "ACL2____NOTE4":[["Release-notes"],"ACL2 Sources","ACL2","<p>Once again <tt><see topic=\"ACL2____LD\">ld</see></tt> only takes one required argument, as the \u000A <v>bind-flg</v> has been deleted.</p> \u000A \u000A <p>Three commands have been added in the spirit of <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt>. \u000A <v>:</v><tt><see topic=\"ACL2____PE_12\">Pe!</see></tt> is similar to <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> but it prints all <see topic=\"ACL2____EVENTS\">events</see> with the given name, rather than just the most recent.  The command \u000A <v>:</v><tt><see topic=\"ACL2____PF\">pf</see></tt> prints the corollary formula corresponding to a name or <see topic=\"ACL2____RUNE\">rune</see>.  The command <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> (print lemmas) prints rules whose top \u000A function symbol is the given name.  See <see topic=\"ACL2____PE_12\">pe!</see>, see <see topic=\"ACL2____PF\">pf</see>, and see \u000A <see topic=\"ACL2____PL\">pl</see>.</p> \u000A \u000A <p>Book naming conventions have been changed somewhat.  The once-required \u000A <v>.lisp</v> extension is now prohibited!  Directories are supported, including \u000A a notion of ``connected book directory''.  See <see topic=\"ACL2____BOOK-NAME\">book-name</see>.  Also, the \u000A second argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is now optional, defaulting to \u000A <v>0</v>.</p> \u000A \u000A <p><see topic=\"ACL2____COMPILATION\">Compilation</see> is now supported inside the Acl2 loop.  See <see topic=\"ACL2____COMP\">comp</see> \u000A and see <see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see>.</p> \u000A \u000A <p>The default color is now part of the Acl2 <see topic=\"ACL2____WORLD\">world</see>; see <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> <v>default-color</v>.  <v>Ld-color</v> is no longer an <tt><see topic=\"ACL2____LD\">ld</see></tt> special. \u000A Instead, colors are <see topic=\"ACL2____EVENTS\">events</see>; see the documentation for <v>red</v>, \u000A <v>pink</v>, <v>blue</v>, and <v>gold</v>.</p> \u000A \u000A <p>A <see topic=\"ACL2____TABLE\">table</see> exists for controlling whether Acl2 prints comments when it \u000A <see topic=\"ACL2____FORCE\">force</see>s hypotheses of rules; see <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> <v>force-table</v>. \u000A Also, it is now possible to turn off the forcing of assumptions by disabling \u000A the definition of <see topic=\"ACL2____FORCE\">force</see>; see <see topic=\"ACL2____FORCE\">force</see>.</p> \u000A \u000A <p>The event <v>defconstant</v> is no longer supported, but a very similar \u000A event, <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, has been provided in its place.  See <see topic=\"ACL2____DEFCONST\">defconst</see>.</p> \u000A \u000A <p>The event for defining <see topic=\"ACL2____CONGRUENCE\">congruence</see> relations is now <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> \u000A (formerly, <v>defcon</v>).</p> \u000A \u000A <p>Patterns are now allowed in <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>.  See the \u000A documentation for <v>:expand</v> inside the documentation for <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>We have improved the way we report rules used by the simplifier.  All <see topic=\"ACL2____RUNE\">rune</see>s of the same type are reported together in the running commentary \u000A associated with each goal, so that for example, executable counterparts are \u000A listed separately from definitions, and rewrite rules are listed separately \u000A from <see topic=\"ACL2____LINEAR\">linear</see> rules.  The preprocessor now mentions ``simple'' rules; see \u000A <see topic=\"ACL2____SIMPLE\">simple</see>.</p> \u000A \u000A <p>The mechanism for printing warning messages for new rewrite rules, related \u000A to subsumption, now avoids worrying about nonrecursive function symbols when \u000A those symbols are <see topic=\"ACL2____DISABLE\">disable</see>d.  These messages have also been eliminated \u000A for the case where the old rule is a <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule.</p> \u000A \u000A <p>Backquote has been modified so that it can usually provide predictable \u000A results when used on the left side of a rewrite rule.</p> \u000A \u000A <p>Time statistics are now printed even when an event fails.</p> \u000A \u000A <p>The Acl2 trace package has been modified so that it prints using the values \u000A of the Lisp globals <v>*print-level*</v> and <v>*print-length*</v> \u000A (respectively).</p> \u000A \u000A <p><see topic=\"ACL2____TABLE\">Table</see> has been modified so that the <v>:clear</v> option lets you \u000A replace the entire <see topic=\"ACL2____TABLE\">table</see> with one that satisfies the <v>val</v> and key \u000A guards (if any); see <see topic=\"ACL2____TABLE\">table</see>.</p> \u000A \u000A <p>We have relaxed the translation rules for <v>:measure</v> <see topic=\"ACL2____HINTS\">hints</see> to \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, so that the the same rules apply to these terms that apply to \u000A terms in <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  In particular, in <v>:measure</v> <see topic=\"ACL2____HINTS\">hints</see> <tt><see topic=\"ACL2____MV\">mv</see></tt> is treated just like <tt><see topic=\"COMMON-LISP____LIST\">list</see></tt>, and <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A receives no special handling.</p> \u000A \u000A <p>The <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see> test has been relaxed.  The old test required that \u000A every new argument be strictly less than the corresponding old argument in a \u000A certain <see topic=\"ACL2____TERM-ORDER\">term-order</see>.  The new test uses a lexicographic order on term \u000A lists instead.  For example, consider the following rewrite rule.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see>\u000A (variable-update var1\u000A                  val1 (variable-update var2 val2 vs))\u000A (variable-update var2\u000A                  val2 (variable-update var1 val1 vs)))</code> \u000A \u000A <p>This rule is permutative.  Now imagine that we want to apply this rule to \u000A the term</p> \u000A \u000A <code>(variable-update u y (variable-update u x vs)).</code> \u000A \u000A <p>Since the actual corresponding to both <v>var1</v> and <v>var2</v> is <v>u</v>, \u000A which is not strictly less than itself in the <see topic=\"ACL2____TERM-ORDER\">term-order</see>, this rule \u000A would fail to be applied in this situation when using the old test.  However, \u000A since the pair <v>(<see topic=\"ACL2____U\">u</see> x)</v> is lexicographically less than the pair <v>(<see topic=\"ACL2____U\">u</see> y)</v> \u000A with respect to our <see topic=\"ACL2____TERM-ORDER\">term-order</see>, the rule is in fact applied using our \u000A new test.</p> \u000A \u000A <p>Messages about <see topic=\"ACL2____EVENTS\">events</see> now contain a space after certain left \u000A parentheses, in order to assist emacs users.  For example, the event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> abc (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____LEN\">len</see> x) 0) (<see topic=\"ACL2____LEN\">len</see> x)))</code> \u000A \u000A <p>leads to a summary containing the line</p> \u000A \u000A <code>Form:  ( DEFTHM ABC ...)</code> \u000A \u000A <p>and hence, if you search backwards for ``<v>(<see topic=\"ACL2____DEFTHM\">defthm</see> abc</v>'', you won't stop \u000A at this message.</p> \u000A \u000A <p>More tautology checking is done during a proof; in fact, no goal printed to \u000A the screen, except for the results of applying <v>:use</v> and <v>:by</v> <see topic=\"ACL2____HINTS\">hints</see> or the top-level goals from an induction proof, are known to Acl2 to be \u000A tautologies.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____LD-QUERY-CONTROL-ALIST\">ld-query-control-alist</see></tt> may now be used to suppress printing of \u000A queries; see <see topic=\"ACL2____LD-QUERY-CONTROL-ALIST\">ld-query-control-alist</see>.</p> \u000A \u000A <p>Warning messages are printed with short summary strings, for example the \u000A string ``<v>Use</v>'' in the following message.</p> \u000A \u000A <code>Acl2 Warning [Use] in DEFTHM:  It is unusual to :USE an enabled\u000A:REWRITE or :DEFINITION rule, so you may want to consider\u000Adisabling FOO.</code> \u000A \u000A <p>At the end of the event, just before the time is printed, all such summary \u000A strings are printed out.</p> \u000A \u000A <p>The keyword command <v>:u</v> has been introduced as an abbreviation for \u000A <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt>.  Printing of query messages is suppressed \u000A by <v>:u</v>.</p> \u000A \u000A <p>The keyword <v>:cheat</v> is no longer supported by any event form.</p> \u000A \u000A <p>Some irrelevant formals are detected; see <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see>.</p> \u000A \u000A <p>A bug in the application of metafunctions was fixed: now if the output of a \u000A metafunction is equal to its input, the application of the metafunction is \u000A deemed unsuccessful and the next metafunction is tried.</p> \u000A \u000A <p>An example has been added to the documentation for <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> to \u000A suggest how to make use of <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations in rewriting.</p> \u000A \u000A <p>The following Common Lisp functions have been added to Acl2: <tt><see topic=\"COMMON-LISP____ALPHA-CHAR-P\">alpha-char-p</see></tt>, <tt><see topic=\"COMMON-LISP____UPPER-CASE-P\">upper-case-p</see></tt>, <tt><see topic=\"COMMON-LISP____LOWER-CASE-P\">lower-case-p</see></tt>, <tt><see topic=\"COMMON-LISP____CHAR-UPCASE\">char-upcase</see></tt>, <tt><see topic=\"COMMON-LISP____CHAR-DOWNCASE\">char-downcase</see></tt>, <tt><see topic=\"COMMON-LISP____STRING-DOWNCASE\">string-downcase</see></tt>, <tt><see topic=\"COMMON-LISP____STRING-UPCASE\">string-upcase</see></tt>, and <v>digit-charp-p</v>.</p> \u000A \u000A <p>A documentation section called <tt><see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see></tt> has been added for the \u000A interactive facility, whose documentation has been slightly improved.  See in \u000A particular the documentation for <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>, <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, and \u000A <see topic=\"ACL2____MACRO-COMMAND\">macro-command</see>.</p> \u000A \u000A <p>A number of <see topic=\"ACL2____EVENTS\">events</see> that had been inadvertently disallowed in <see topic=\"ACL2____BOOKS\">books</see> are now permitted in <see topic=\"ACL2____BOOKS\">books</see>.  These are: <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt>, \u000A <v>defcor</v>, <tt><see topic=\"ACL2____DEFEQUIV\">defequiv</see></tt>, <tt><see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see></tt>, <tt><see topic=\"ACL2____DEFSTUB\">defstub</see></tt>, and \u000A <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt>.</p>"],

 "ACL2____NOTE3":[["Release-notes"],"ACL2 Sources","ACL2","<p><see topic=\"ACL2____PROGRAMMING\">Programming</see> mode has been eliminated.  Instead, all \u000A functions have a ``color'' which indicates what can be done with the function. \u000A For example, <v>:red</v> functions can be executed but have no axioms describing \u000A them.  Thus, <v>:red</v> functions can be introduced after passing a simple \u000A syntactic check and they can be redefined without undoing.  But nothing of \u000A consequence can be proved about them.  At the other extreme are <v>:gold</v> \u000A functions which can be executed and which also have passed both the \u000A termination and the <see topic=\"ACL2____GUARD\">guard</see> verification proofs.  The color of a function \u000A can be specified with the new <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword, <v>:color</v>, which, if \u000A omitted defaults to the global setting of <v>ld-color</v>.  <v>Ld-color</v> \u000A replaces <v>load-mode</v>.  Setting <v>ld-color</v> to <v>:red</v> causes behavior \u000A similar to the old <v>:g-mode</v>.  Setting <v>ld-color</v> to <v>:gold</v> causes \u000A behavior similar to the old <v>:v-mode</v>.  It is possible to prototype your \u000A system in <v>:red</v> and then convert <v>:red</v> functions to :<v>blue</v> \u000A individually by calling <tt><see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see></tt> on them.  They can then be \u000A converted to <v>:gold</v> with <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.  This allows us to \u000A undertake to verify the termination and <see topic=\"ACL2____GUARD\">guard</see>s of system functions. \u000A See <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> color for an introduction to the use of colors.</p> \u000A \u000A <p>Type prescription rules have been added.  Recall that in Nqthm, some <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules were actually stored as ``<see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>s.''  Such \u000A rules allow the user to inform Nqthm's primitive type mechanism as to the \u000A kinds of shells returned by a function.  Earlier versions of Acl2 did not have \u000A an analogous kind of rule because Acl2's type mechanism is complicated by \u000A <see topic=\"ACL2____GUARD\">guard</see>s.  Version 1.3 supports <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.  See \u000A <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>.</p> \u000A \u000A <p>Three more new <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> implement congruence-based rewriting. \u000A It is possible to identify a binary relation as an equivalence relation (see \u000A <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>), to show that one equivalence relation refines another \u000A (see <see topic=\"ACL2____REFINEMENT\">refinement</see>) and to show that a given equivalence relation is \u000A maintained when rewriting a given function call, e.g., <v>(fn ...xk...)</v>, by \u000A maintaining another equivalence relation while rewriting the <v>k</v>th argument \u000A (see <see topic=\"ACL2____CONGRUENCE\">congruence</see>).  If <v>r</v> has been shown to be an <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> \u000A relation and then <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps (r (foo x) (bar x)))</v> is proved as a \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule, then instances of <v>(foo x)</v> will be replaced by \u000A corresponding instances of <v>(bar x)</v> provided the instance occurs in a slot \u000A where the maintainence of <v>r-equivalence</v> is known to be sufficient and \u000A <v>hyps</v> can be established as usual.</p> \u000A \u000A <p>In Version 1.2, <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> were simple keywords, e.g., \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt>.  In Version 1.3, <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> have been elaborated to allow you to specify how the theorem \u000A ought to be used as a rule.  That is, the new <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> allows you \u000A to separate the mathematical statement of the formula from its interpretation \u000A as a rule.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p>Rules used to be named by symbols, e.g., <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <v>car-cons</v> were \u000A the names of rules.  Unfortunately, this was ambiguous because there are three \u000A rules associated with function symbols: the symbolic definition, the \u000A executable counterpart, and the <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>; many different rules \u000A might be associated with theorems, depending on the rule classes.  In Version \u000A 1.3 rules are named by ``<see topic=\"ACL2____RUNE\">rune</see>s'' (which is just short hand for ``rule \u000A names'').  Example <see topic=\"ACL2____RUNE\">rune</see>s are <v>(:definition car)</v>, \u000A <v>(:executable-counterpart car)</v>, and <v>(:type-prescription car . 1)</v>. \u000A Every rule added by an event has a different name and you can <see topic=\"ACL2____ENABLE\">enable</see> \u000A and <see topic=\"ACL2____DISABLE\">disable</see> them independently.  See <see topic=\"ACL2____RUNE\">rune</see> and see <see topic=\"ACL2____THEORIES\">theories</see>.</p> \u000A \u000A <p>The identity function <tt><see topic=\"ACL2____FORCE\">force</see></tt>, of one argument, has been added and \u000A given a special interpretation by the functions responsible for establishing \u000A hypotheses in backchaining: When the system fails to establish some hypothesis \u000A of the form <v>(<see topic=\"ACL2____FORCE\">force</see> term)</v>, it simply assumes it is true and goes on, \u000A delaying until later the establishment of term.  In particular, pushes a new \u000A subgoal to prove term in the current context.  When that subgoal is attacked, \u000A all of the resources of the theorem prover, not just rewriting, are brought to \u000A bear.  Thus, for example, if you wish to prove the rule <tt>(implies \u000A (good-statep s) (equal (exec s n) s'))</tt> and it is your expectation that \u000A every time <v>exec</v> appears its first argument is a <v>good-statep</v> then you \u000A might write the rule as <tt>(implies (force (good-statep s)) (equal (exec s n) \u000A s'))</tt>.  This rule is essentially an unconditional rewrite of <v>(exec s\u000A n)</v> to <v>s'</v> that spawns the new goal <v>(good-statep s)</v>.  See <see topic=\"ACL2____FORCE\">force</see>.  Because you can now specify independently how a theorem is used as a \u000A rule, you need not write the <tt><see topic=\"ACL2____FORCE\">force</see></tt> in the actual theorem proved.  See \u000A <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p>Version 1.3 supports a facility similar to Nqthm's <tt><see topic=\"ACL2____BREAK-LEMMA\">break-lemma</see></tt>. \u000A See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>.  You can install ``<see topic=\"ACL2____MONITOR\">monitor</see>s'' on <see topic=\"ACL2____RUNE\">rune</see>s \u000A that will cause interactive breaks under certain conditions.</p> \u000A \u000A <p>Acl2 also provides ``<see topic=\"ACL2____WORMHOLE\">wormhole</see>s'' which allow you to write functions \u000A that cause interaction with the user but which do not require that you have \u000A access to <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.</p> \u000A \u000A <p>The rewriter now automatically backchains to stronger recognizers.  There \u000A is no user hook to this feature but it may simplify some proofs with which \u000A older versions of Acl2 had trouble.  For example, if the rewriter is trying to \u000A prove <v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (foo a b c))</v> it is now smart enough to try lemmas that \u000A match with <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo a b c))</v>.</p>"],

 "ACL2____NOTE-6-5":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes to ACL2 since Version 6.4 into the \u000A following categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p>See also <see topic=\"ACL2____NOTE-6-5-BOOKS\">note-6-5-books</see> for a summary of changes made to the ACL2 \u000A Community Books since ACL2 6.4, including the build system.</p> \u000A \u000A <h3>Changes to Existing Features</h3> \u000A \u000A <p>The <tt><see topic=\"ACL2____BRR_04\">brr@</see></tt> command now supports inputs <v>:initial-ttree</v> and \u000A <v>:final-ttree</v>.  Thanks to David Rager for a request leading to these \u000A enhancements.  See <see topic=\"ACL2____TTREE\">ttree</see> for a discussion of the tag-tree structures \u000A returned for these inputs.</p> \u000A \u000A <p>(GCL only) A restriction for structure-sharing (as described in a remark, \u000A ``Remark on print-circle-files''; see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>) involving GCL has \u000A been removed, since this is no longer required for the latest GCL \u000A versions (2.6.8 and 2.6.10).  Thanks to Jared Davis and Camm Maguire for \u000A useful discussions and this GCL improvement.</p> \u000A \u000A <p>The keyword commands <v>:exit</v> and <v>:quit</v> are disabled inside <see topic=\"ACL2____BRR\">brr</see>, in order to avoid accidental exits from ACL2.  Thanks to David Rager for \u000A suggesting such a change.</p> \u000A \u000A <p>For calls of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> made during the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> phase \u000A of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, keyword <v>:check-expansion</v> no longer causes \u000A evaluation of the first argument of <v>make-event</v>; rather, the value of \u000A <v>:check-expansion</v> is simply used as the expansion.  Thanks to Sol Swords \u000A for suggesting this change, which made it reasonable to ignore such \u000A <v>make-event</v> forms when determining the ``rolling back'' before that \u000A <v>include-book</v> phase, as described in the next section.</p> \u000A \u000A <p>The form <v>(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> nil)</v> now is a no-op if <see topic=\"ACL2____STATE\">state</see> global \u000A <tt>'</tt><v>skip-reset-prehistory</v> has a non-<v>nil</v> value.  Thanks to Sol \u000A Swords for requesting this feature (in support of infrastructure for \u000A certifying the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>).</p> \u000A \u000A <p>The output from <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt>, <v>:</v><tt><see topic=\"ACL2____PR_12\">pr!</see></tt>, <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>, and \u000A <v>:</v><tt><see topic=\"ACL2____SHOW-BODIES\">show-bodies</see></tt> has been tweaked.  Thanks to a suggestion from Ben \u000A Selfridge, such output now shows the hypotheses (the <v>Hyps</v> field) above \u000A the left-hand side and right-hand side of a <see topic=\"ACL2____REWRITE\">rewrite</see> rule (<v>Lhs</v> and \u000A <v>Rhs</v>, respectively).  The <v>Equiv</v> field has also been moved up before \u000A the <v>Lhs</v> field in such output for rewrite rules, and this field has been \u000A added for <see topic=\"ACL2____ELIM\">elim</see> rules.  Finally, the <v>Hyps</v> field now appears above \u000A the <v>Term</v> field in such output for <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> may now be <see topic=\"ACL2____MONITOR\">monitor</see>ed.  Thanks to \u000A David Rager for requesting this enhancement.</p> \u000A \u000A <p>When forcing (see <tt><see topic=\"ACL2____FORCE\">force</see></tt>) is applied, we now always see the \u000A appropriate <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> <see topic=\"ACL2____RUNE\">rune</see>, <v>(:executable-counterpart force)</v>, in \u000A the proof output and summary; this hadn't previously been the case.  Thanks to \u000A Robert Krug for bringing this issue to our attention.</p> \u000A \u000A <p>The default value for the <v>:write-port</v> keyword argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is now <v>t</v>, as was already claimed by the documentation. \u000A Thus, a <v>.port</v> file will be written when a book is certified, making it \u000A more likely that a subsequent <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> will complete without error \u000A even if the book has been modified without recertification.  Note that a \u000A change by Sol Swords to the build system for the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> now \u000A sets an environment variable to guarantee that <v>write-port</v> is true, \u000A independent of this change.  On a related note: the unadvertised environment \u000A variable <v>ACL2_WRITE_PORT</v> is now handled more appropriately, by interning \u000A its upcased argument into the ACL2 package.</p> \u000A \u000A <p>The following changes can make it easier to include books that are \u000A uncertified or have corrupted compiled files.  Thanks to Sol Swords for \u000A sending helpful, relevant examples and for subsequent discussions helping to \u000A lead to these changes.</p> \u000A \u000A <ul> \u000A \u000A <li>It is no longer illegal to call <v>set-compiler-enabled</v> within <see topic=\"ACL2____BOOKS\">books</see>.  See <see topic=\"ACL2____COMPILATION\">compilation</see>, which shows how to do this in order to avoid \u000A loading the compiled file not only for a book specified by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> but also for all books included under that book.  Also see <see topic=\"ACL2____COMPILATION\">compilation</see> for an analogous utility, <v>(set-port-file-enabled nil state)</v>, \u000A to avoid loading <v>.port</v> files.  (For background on <v>.port</v> files, see \u000A <see topic=\"ACL2____UNCERTIFIED-BOOKS\">uncertified-books</see>).</li> \u000A \u000A <li>In situations where it is legal to include an uncertified book (typically, \u000A any time other than during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, ACL2 nonetheless could fail \u000A to do so when an error occurred while reading a <see topic=\"ACL2____CERTIFICATE\">certificate</see> file.  ACL2 \u000A can now instead include the book successfully as an uncertified book.</li> \u000A \u000A <li>When hard raw Lisp errors occur during loading of compiled files on behalf \u000A of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event, that event can now complete successfully \u000A much as though the compiled file were simply missing.</li> \u000A \u000A </ul> \u000A \u000A <p>(SBCL only) The function <tt><see topic=\"ACL2____SETENV_42\">setenv$</see></tt> is now supported in SBCL, which \u000A had been the one supported host Lisp for which <v>setenv$</v> had not been \u000A supported.  Thanks to Jared Davis for pointing the way to this \u000A improvement.</p> \u000A \u000A <p>Built-in <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> have been modified in order to support \u000A suitable guard-checking.  For example, evaluation of the form <v>(<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> 3\u000A '(4 5))</v>, or equivalently, <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> 3 '(4 5) :test 'eq)</v>, had produced a \u000A value of <v>nil</v>, but now we get what one should have expected: a guard \u000A violation (since <tt><see topic=\"ACL2____MEMBER-EQ\">member-eq</see></tt> is intended to compare symbols).  Thanks to \u000A Yan Peng for raising a question on the acl2-help mailing list that led us to \u000A make this improvement.</p> \u000A \u000A <p>When the <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> command is supplied the macro alias for a \u000A function symbol (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), the system will now print \u000A <see topic=\"ACL2____EVENTS\">events</see> not only for the macro symbol but also for the corresponding \u000A function symbol.  Thanks to Cuong Chau, Jared Davis, Shilpi Goel, Sol Swords, \u000A and Warren Hunt for requesting this enhancement.</p> \u000A \u000A <p>Some evaluations of forms <v>(<see topic=\"ACL2____MBE\">mbe</see> :logic L :exec E)</v> are more efficient or \u000A avoid guard violations, by evaluation of the <v>:exec</v> form (<v>E</v>) instead \u000A of the <v>:logic</v> form (<v>L</v>).  If the <tt><see topic=\"ACL2____MBE\">mbe</see></tt> call is in a \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function definition, the <v>:exec</v> code will be \u000A evaluated, where unlike before, this is true even when guard-checking is \u000A <v>:all</v> or <v>:none</v> (see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>).  If the <v>mbe</v> call \u000A is in a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function definition, the change is that the \u000A <v>:exec</v> code will be evaluated when there is a superior call of a \u000A <v>:program</v> mode function, except when guard-checking is <v>:all</v> or \u000A <v>:none</v> or in a couple of technical cases (see a comment in the source code \u000A for constant <v>*mbe-as-exec*</v> for details).  This use of <v>:exec</v> code had \u000A been defeated when the superior <v>:program</v> mode function had an \u000A ``invariant-risk'' of making ill-guarded stobj updates.  However, there is no \u000A change in the case of safe-mode, which is used during evaluation of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt>, and <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> forms, and during \u000A macroexpansion: the <v>:logic</v> and <v>:exec</v> forms are still both evaluated \u000A and checked for equality.  Thanks to Jared Davis for reporting this issue and \u000A for helpful chats about it.  For more information, see the comment about ``mbe \u000A and invariant-risk'' in the form <v>(defxdoc note-6-5 ...)</v> in <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> file <v>books/system/doc/acl2-doc.lisp</v>.</p> \u000A \u000A <p>The utility <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> has a new (optional) keyword argument, \u000A <v>:init-forms</v>, which specifies a list of forms to evaluate when first \u000A entering the ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LP\">lp</see></tt>.  Thanks to Jared Davis for \u000A requesting this enhancement.</p> \u000A \u000A <h3>New Features</h3> \u000A \u000A <p>A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event may now include the keyword argument \u000A <v>:non-memoizable</v>.  When its value if <v>t</v>, then for ACL2(h) builds (see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>), code will run somewhat faster.  In a little test \u000A doing just reads and writes to a stobj array in ACL2(h), it took 26% less time \u000A when the stobj to be written was introduced by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> using \u000A <v>:non-memoizable t</v>.  Thanks to Warren Hunt for requesting this \u000A feature and helping to develop the test.</p> \u000A \u000A <p>See <tt><see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see></tt> for a utility that may be preferable to \u000A <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt>, since it essentially calls <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt> \u000A automatically.  Thanks to David Rager, Shilpi Goel, and David Hardin for \u000A participating in an acl2-help mailing list discussion that motivated this \u000A feature.</p> \u000A \u000A <h3>Heuristic Improvements</h3> \u000A \u000A <p>One of the steps performed by <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> has traditionally been \u000A to check for local incompatibilities (see <see topic=\"ACL2____LOCAL-INCOMPATIBILITY\">local-incompatibility</see>) after \u000A admitting all the events in the book.  This step involved rolling back the \u000A logical <see topic=\"ACL2____WORLD\">world</see> to what it was at the start of certification, and then \u000A including the book (see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>).  As an optimization, we now \u000A avoid rolling back the world more than necessary; see <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, \u000A specifically the discussion of Step 3.  We thank Sol Swords for requesting \u000A this optimization and for making a suggestion that improved it, pertaining to \u000A <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> (described above).  We also thank Jared Davis for alerting \u000A us to an issue that turned out to be caused by a bug in our initial \u000A implementation.  Sol has reported a nearly 20% reduction in time for \u000A certifying a certain large collection of books.</p> \u000A \u000A <p>The test for redundancy of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> has been made \u000A more efficient.  Also, while the criterion itself remains unchanged, the \u000A documentation has been made more accurate; see <see topic=\"ACL2____REDUNDANT-ENCAPSULATE\">redundant-encapsulate</see>. \u000A We observed more than a 2% reduction in time for the ``everything'' regression \u000A target, but we observed more than 23% of the time eliminated for the form \u000A <v>(<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;doc/top&quot; :dir :system))</v> after building the \u000A documentation.  Thanks to David Rager for bringing that particular book to our \u000A attention, as one that took a long time to include.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____PAIRLIS_42\">pairlis$</see></tt> is now tail-recursive, hence potentially more \u000A efficient.  More precisely, its definition for the logic remains unchanged, \u000A but using <tt><see topic=\"ACL2____MBE\">mbe</see></tt>, for execution it calls a new tail-recursive function, \u000A <v>pairlis$-tailrec</v>.  Thanks to Jared Davis for requesting this \u000A improvement.</p> \u000A \u000A <p><see topic=\"ACL2____LINEAR\">Linear</see> arithmetic has been strengthened slightly so that immediately \u000A after simplification has ``settled down'' (see <see topic=\"ACL2____HINTS-AND-THE-WATERFALL\">hints-and-the-waterfall</see>), the unrewritten conclusion of a <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rule may be used when normally this would not be the case.  Thanks to \u000A Robert Krug for reminding us of examples we had encountered that could benefit \u000A from some such a change, and for pointing us to some relevant source code. \u000A This improvement generally causes an extra pass through the simplifier; hence \u000A we have observed approximately a 2% slowdown in the regression suite.  Note \u000A that machinery is now in place for installing additional ``desperation \u000A heuristics''; perhaps the ACL2 community will have some to suggest.</p> \u000A \u000A <h3>Bug Fixes</h3> \u000A \u000A <p>We fixed a soundness bug in how the <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see> processed calls of \u000A <v>if</v>.  Thanks to Dmitry Nadezhin for reporting this bug by sending a simple \u000A example that exploited it to prove <v>nil</v>.  To see that example, see the \u000A comment about ``tau soundess bug'' in the form <v>(defxdoc note-6-5 ...)</v> in \u000A <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> file <v>books/system/doc/acl2-doc.lisp</v>.</p> \u000A \u000A <p>We fixed a soundness bug for nested <see topic=\"ACL2____STOBJ\">stobj</see>s (see <see topic=\"ACL2____STOBJ-LET\">stobj-let</see>). \u000A In the case of a stobj producer variable that is not a child stobj, it had \u000A been possible to update that stobj without returning it.  Thanks to Sol Swords \u000A for reporting this bug and providing a corresponding proof of <v>nil</v>, which \u000A is included in a comment in the form <v>(defxdoc note-6-5 ...)</v> in <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> file <v>books/system/doc/acl2-doc.lisp</v>.)</p> \u000A \u000A <p>We fixed a soundness bug in the case of a <see topic=\"ACL2____STOBJ\">stobj</see> with a field that is \u000A a resizable array of stobjs.  Thanks to Sol Swords for sending a proof of \u000A <v>nil</v> exploiting this bug, together with some helpful analysis attributing \u000A the bug to a mistake in the generated logical definition of the corresponding \u000A resize function.  (His example is included in a comment in the form \u000A <v>(defxdoc note-6-5 ...)</v> in <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> file \u000A <v>books/system/doc/acl2-doc.lisp</v>.)</p> \u000A \u000A <p>(ACL2(h) only) We fixed a soundness bug involving the interaction of <tt><see topic=\"ACL2____STOBJ-LET\">stobj-let</see></tt> and <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>, which thus is only a bug in ACL2(h) (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>), not in ACL2.  The fix was to add code to \u000A <v>stobj-let</v> that clears the memoization table for the parent stobj if any \u000A child stobj is updated.  An example proof of <v>nil</v> before this fix may be \u000A found in the ACL2 source file <v>translate.lisp</v>, in a comment in the \u000A function <v>stobj-let-fn-raw</v>.</p> \u000A \u000A <p>(Allegro CL and CMUCL only) We fixed bugs in function <tt><see topic=\"ACL2____PRINC_42\">princ$</see></tt> when \u000A invoked in an ACL2 image with Allegro CL or CMUCL as the host Lisp \u000A implementation.  In Allegro CL, when printing a complex number in base 16, \u000A lower case characters were produced, for example printing <v>#C(c d)</v> instead \u000A of the characters predicted by the axioms, namely <v>#C(C D)</v>.  In some \u000A recent versions of CMUCL, after executing the forms <v>(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16\u000A state)</v> and <v>(<see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> :downcase state)</v>, hex digits were printed in \u000A lower case, unlike other host Lisps and contrary to the ACL2 axioms: for \u000A example, 1000 base 10 was printed as <v>3e8</v> rather than as predicted by the \u000A ACL2 axioms, <v>3E8</v>.  Thanks to Jared Davis for bringing these bugs to our \u000A attention, and also for pointing out excessive printing of a big note about \u000A such printing in Allegro CL.  We now avoid that note altogether, which had \u000A warned that printing numbers in base 16 can be slow for Allegro CL.  In fact, \u000A that performance issue has been eliminated: for example, after evaluating \u000A <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *c* (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 200000))</v> and <v>(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16 state)</v>, the \u000A form <v>(<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____PRINC_42\">princ$</see> *c* *standard-co* state) state))</v> reports \u000A taking about 3 seconds before the change but about 1/100 seconds after the \u000A change.  Thanks to David Margolies of Franz Inc. for passing along a remark \u000A from a colleague that showed how to make this improvement.</p> \u000A \u000A <p>The utility <tt><see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see></tt> can now only be called if there is an active \u000A <see topic=\"ACL2____TRUST-TAG\">trust-tag</see>, which is the restriction that was already in place for \u000A <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>.  This plugs a potential soundness hole.</p> \u000A \u000A <p>We have made several system functions untouchable (see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>), in order to prevent soundness bugs.  We thank Jared \u000A Davis and Sol Swords for sending us an example that used a call of one of \u000A these functions to prove <v>nil</v>.  We have placed that example into a comment \u000A in Community Books file <tt>system/doc/acl2-doc.lisp</tt>, form <v>(defxdoc\u000A note-6-5 ...)</v>.</p> \u000A \u000A <p>The <v>brr</v> command, <v>:wonp</v>, had not been installed for use after \u000A <v>:eval</v> even though this was claimed in the documentation (see <see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see>).  This has been fixed.  Thanks to David Rager for bringing this \u000A issue to our attention.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> that could cause <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to \u000A fail with a bogus message about ``the set of ttags permitted \u000A in the current context.''  Thanks to Sol Swords and Anna Slobodova for \u000A reporting this bug, and to Sol for sending a small example that illustrated \u000A the problem.</p> \u000A \u000A <p>Fixed a bug that was wrongly disallowing certain calls of exported \u000A functions for abstract stobjs (see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>).  Thanks to Sol Swords \u000A for reporting this bug, supplying an example illustrating the bug, and \u000A suggesting a nice fix.  A slight variant of his example is included in a \u000A comment in Community Books file <tt>system/doc/acl2-doc.lisp</tt>, form \u000A <v>(defxdoc note-6-5 ...)</v>.</p> \u000A \u000A <p>When a macro symbol <v>mac</v> was a macro-alias for a function symbol \u000A <v>f</v> (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), then the form <v>(<see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see>\u000A mac)</v> caused an error when encountered while including an uncertified book. \u000A This problem has been fixed.  Thanks to Jared Davis for pointing out the \u000A problem and sending a simple example to illustrate it.  Technical Note: the \u000A actual problem was that <tt><see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see></tt> generates a call of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> with an <v>:expansion?</v> argument, and that argument needed to be \u000A ignored (as it now is, after the fix) when including an uncertified book.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____WALKABOUT\">walkabout</see></tt> utility could cause hard Lisp errors when the current \u000A package is other than <v>&quot;ACL2&quot;</v>.  This has been fixed.  Thanks to Sol \u000A Swords for bringing this bug to our attention and suggesting a fix.</p> \u000A \u000A <h3>Changes at the System Level</h3> \u000A \u000A <p>(SBCL only) Fixed the executable script generated for SBCL so that \u000A <v>SBCL_HOME</v> is set to the SBCL <v>obj/sbcl-home/</v> directory if it exists, \u000A since that fix seems needed for some recent versions of SBCL (1.1.14 in \u000A particular).</p> \u000A \u000A <p>File <v>GNUmakefile</v> in the ACL2 sources directory wasn't passing the \u000A value of shell variable <v>ACL2</v> from target <v>certify-books-fresh</v> to \u000A target <v>certify-books</v>, and similarly for target \u000A <v>books/system/doc/render-doc.cert</v> (invoked by target <v>DOC</v>).  This has \u000A been fixed.  We thank Jared Davis and Camm Maguire for helpful \u000A discussions.</p> \u000A \u000A <p>The \"make DOC\" command was not completing correctly when variable \u000A <v>ACL2</v> was not set.  Also, for host Lisp CMUCL at least, it stalled at the \u000A terminal unless a quit command was issued twice (as ACL2 was stuck without \u000A exiting).  Those problems have been resolved.  Thanks to Jim Ward for bringing \u000A these issues to our attention</p> \u000A \u000A <p>Improved the error message when encountering an illegal comma while reading \u000A input (i.e., for a comma not within a suitable nesting of backquotes).  Thanks \u000A to Jared Davis for bringing this issue to our attention and for a helpful \u000A discussion.</p> \u000A \u000A <p>(GCL only) Arranged for state global <v>'tmp-dir</v> to be set to GCL's \u000A <v>si::*tmp-dir*</v>, even on Windows; this may be important for compilation on \u000A Windows.  Thanks to Camm Maguire for pointing out the need to set \u000A <v>'tmp-dir</v> for ACL2 built on GCL on mingw.</p> \u000A \u000A <p>(GCL only) Now, <tt><see topic=\"ACL2____GC_42\">gc$</see></tt> may be called with no arguments, in which case \u000A the missing argument for <v>system::gc</v> is <v>t</v>.</p> \u000A \u000A <h3>EMACS Support</h3> \u000A \u000A <p>Improvements to the <see topic=\"ACL2____ACL2-DOC\">ACL2-Doc</see> browser include the following.</p> \u000A \u000A <ul> \u000A \u000A <li>A new `<v>w</v>' command displays the topic name in the minibuffer, together \u000A with the manual name (ACL2+Books Manual or ACL2 User's Manual).</li> \u000A \u000A <li>Improved the `<v>control-t .</v>' command so that the default topic is \u000A selected just as in acl2-doc mode.  (Technically: the same syntax table is \u000A used for `<v>control-t .</v>' as is used in the <v>acl2-doc</v> buffer for that \u000A command and the `<v>g</v>' command.)</li> \u000A \u000A <li>Improved the search commands so that the display doesn't move when the \u000A next occurrence is already displayed on the screen.</li> \u000A \u000A <li>Eliminated the deprecated commands `<v>Control-x a A</v>', \u000A `<v>Control-x a a</v>', and `<v>Control-t G</v>'.</li> \u000A \u000A </ul> \u000A \u000A <h3>Experimental/Alternate Versions</h3> \u000A \u000A <p>For ACL2(r), Ruben Gamboa found a bug in constraints for functions \u000A introduced with <v>defun-std</v>, and also kindly provided a fix, which has been \u000A incorporated.</p> \u000A \u000A <p>Modified ACL2(hp) so that two system functions that had been <tt><see topic=\"ACL2____UNMEMOIZE\">unmemoize</see></tt>d when turning on <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see> &mdash; \u000A <v>fchecksum-obj</v> and <v>expansion-alist-pkg-names-memoize</v> &mdash; now \u000A remain memoized.  This can greatly improve performance when using ACL2(hp) \u000A with <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see> on, in particular community book \u000A <v>books/system/doc/render-doc-combined.lisp</v>.</p> \u000A \u000A <p>Made miscellaneous small improvements to ACL2(h).</p> \u000A \u000A <p>The utilities <v>hons-shrink-alist</v> and <v>hons-shrink-alist!</v> have \u000A essentially been renamed to <tt><see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see></tt> and <tt><see topic=\"ACL2____FAST-ALIST-FORK_12\">fast-alist-fork!</see></tt>, respectively.  The old names are deprecated but remain as \u000A macros that are macro-aliases for the respective new names (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).  New utilities, <tt><see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see></tt> and <tt><see topic=\"ACL2____FAST-ALIST-CLEAN_12\">fast-alist-clean!</see></tt> are similar to the above but take just one argument and, if \u000A it is a fast alist, associates the result with the hash table link of that \u000A argument.  Thanks to Jared Davis for providing a concrete proposal, together \u000A with the new names, for what had been only a concept.</p>"],

 "ACL2____NOTE-6-4":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes to ACL2 since Version 6.3 into the \u000A following categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p>See also <see topic=\"ACL2____NOTE-6-4-BOOKS\">note-6-4-books</see> for a summary of changes made to the ACL2 \u000A Community Books since ACL2 6.3, including the build system.</p> \u000A \u000A <h3>Changes to Existing Features</h3> \u000A \u000A <p><see topic=\"ACL2____GAG-MODE\">Gag-mode</see> no longer saves prover output when <v>PROVE</v> output is \u000A inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>).  This may improve performance \u000A slightly when certifying community books using their <v>Makefile</v> or <see topic=\"ACL2____CERT.PL\">cert.pl</see>; in particular, GCL 2.6.8 running on a Mac can now certify the book \u000A <v>books/tau/bounders/elementary-bounders.lisp</v> without running out of space, \u000A even without explicitly turning off <see topic=\"ACL2____GAG-MODE\">gag-mode</see> (which had been done \u000A shortly before the Version 6.3 release).</p> \u000A \u000A <p>When <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> fails to find a readable <see topic=\"ACL2____CERTIFICATE\">certificate</see> \u000A (<v>.cert</v>) file, the error message now distinguishes between the case that \u000A this file is missing and the case that read permission is missing.</p> \u000A \u000A <p>(GCL only) Time reporting has been improved when the host Lisp is Gnu \u000A Common Lisp.  A key change was made in the computation of runtime (for \u000A example, to report in event summaries), so that it includes the ``child \u000A runtime''.  See <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.  Also, the utility <tt><see topic=\"ACL2____TIME_42\">time$</see></tt> now \u000A gives improved information by including child runtime information, which can \u000A be significant; for example, it probably includes compile time, while the \u000A ``seconds runtime'' statistic (still) does not.  Recent versions of GCL might \u000A also provide system runtime and child system runtime.  See <see topic=\"ACL2____TIME_42\">time$</see>. \u000A Thanks to Camm Maguire for suggesting these improvements to <v>time$</v> and \u000A providing an initial implementation for them.</p> \u000A \u000A <p>Fixed a bug in an ACL2 system function, <v>our-truename</v>, which returns \u000A the ``true name'' for a file name, when supplied with an optional second \u000A argument.  Thanks to Camm Maguire for bringing this bug to our attention.</p> \u000A \u000A <p>The wording in theory warnings has been improved, to avoid giving the \u000A impression that you are newly disabling a built-in rule in the case that it \u000A merely remains disabled.</p> \u000A \u000A <p>As requested by Sol Swords, erroneous evaluations of system function \u000A <v>magic-ev-fncall</v> that had produced a message, <v>msg</v>, will now also \u000A return that message.  The following example, sent by Sol, illustrates the fix: \u000A before, evaluation of <v>(magic-ev-fncall 'cons '(a) state nil nil)</v> printed \u000A a message (to the comment window) and then returned <v>(<see topic=\"ACL2____MV\">mv</see> t nil)</v>, but now \u000A it returns <v>mv t &lt;msg&gt;</v>, shere <v>&lt;msg&gt;</v> denotes the message (printed with \u000A the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> <v>~@</v> directive).</p> \u000A \u000A <p>Fixed <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> to avoid printing the bodies of <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> \u000A forms.  Thanks to Jared Davis for pointing out the large output when the body \u000A is a large character string.</p> \u000A \u000A <p>A new output type, <v>history</v>, now controls the printing of \u000A history-related information; see <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>.  The unused \u000A output type, <v>expansion</v>, is no longer supported, i.e., is no longer a \u000A member of the list <v>*valid-output-names*</v>.  Because of this change, \u000A printing of information when undoing, as by <v>:</v><tt><see topic=\"ACL2____U\">u</see></tt>, will now take \u000A place even when <v>event</v> output is inhibited (if <v>history</v> output is not \u000A inhibited); we thank Sol Swords for requesting that change.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field of a rule of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, \u000A should still be a list of lists <v>(var1 var2 fn1 fn2 ... fnk)</v>, where \u000A <v>var1</v> and <v>var2</v> are variables and the <v>fni</v> are symbols.  But \u000A formerly each <v>fni</v> was required to be a function symbol; now, it can be a \u000A macro alias for a function symbol (see <tt><see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see></tt>).  For example, \u000A the following is a valid <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field: <v>((x y +))</v>.</p> \u000A \u000A <p>The restrictions on utilities <tt><see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see></tt> and <tt><see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see></tt> have been updated in order to avoid potentially confusing or \u000A inappropriate results, by imposing the following new requirements on their \u000A first argument, which must still be a function symbol.  That symbol must not \u000A be <v>if</v> (formerly the illegal symbol was <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>); it must not \u000A be a key of the alist, <v>*ttag-fns-and-macros*</v>; it must not be \u000A untouchable (see <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt>); and it must not be a <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> creator (see <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>).</p> \u000A \u000A <p>The <see topic=\"ACL2____TABLE\">table</see> guard on <tt><see topic=\"ACL2____DIVE-INTO-MACROS-TABLE\">dive-into-macros-table</see></tt> has been \u000A strengthened in order to avoid calling untouchable functions (see <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt>).</p> \u000A \u000A <h3>New Features</h3> \u000A \u000A <p>We have added a tool for writing out useful information about a book's \u000A event names when certifying the book.  See <see topic=\"ACL2____BOOKDATA\">bookdata</see>.  Thanks to Dave \u000A Greve for requesting this tool and participating in its specification.</p> \u000A \u000A <p>There are new analogues of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> and <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt>: <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt> and <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt>, respectively.  The new utilities are similar to \u000A their existing counterparts, except that their effects are not <see topic=\"ACL2____LOCAL\">local</see> to \u000A enclosing <see topic=\"ACL2____BOOKS\">books</see> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events.  Thanks to Shilpi Goel \u000A for requesting this enhancement.</p> \u000A \u000A <p>The class of <see topic=\"ACL2____CONGRUENCE\">congruence</see> rules has been broadened considerably, so \u000A that one can restrict to patterns.  For example, a congruence rule can now \u000A state that an equivalence is maintained for the term <v>(<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (f (<see topic=\"COMMON-LISP____CONS\">cons</see> u\u000A v) y 'a))</v> when rewriting <v>y</v>.  See <see topic=\"ACL2____PATTERNED-CONGRUENCE\">patterned-congruence</see>.  Thanks \u000A to Sol Swords for requesting this feature.</p> \u000A \u000A <h3>Heuristic Improvements</h3> \u000A \u000A <p>(None to report this time.)</p> \u000A \u000A <h3>Bug Fixes</h3> \u000A \u000A <p>Fixed a soundness bug in the handling of hypotheses of conditional \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules invoked during rewriting by applying \u000A <v>:</v><tt><see topic=\"ACL2____EXPAND\">expand</see></tt> <see topic=\"ACL2____HINTS\">hints</see>.  See (defxdoc note-6-4 ...) in community \u000A book <v>system/doc/acl2-doc.lisp</v> for a proof of <v>nil</v> in ACL2 Version_6.3 \u000A that exploits this bug.</p> \u000A \u000A <p>It had been possible to update a <see topic=\"ACL2____STOBJ\">stobj</see> (either an ordinary stobj or \u000A an abstract stobjs) so that it no longer satisfies its recognizer predicate. \u000A This soundness bug has been fixed.  Thanks to Jared Davis and Sol Swords for \u000A pointing out this bug, making useful observations about the issue, and sending \u000A proofs of <v>nil</v>, one of which may be found in a Lisp comment in the \u000A defxdoc form for <v>note-6-4</v>.</p> \u000A \u000A <p>Fixed a long-standing soundness bug (found at least as far back as Version \u000A 1.9!) in the checking done for <see topic=\"ACL2____CONGRUENCE\">congruence</see> rules.  There had failed to \u000A be a check that the new variable on the right-hand side of the conclusion is \u000A indeed new.  The following example is shown in detail as a comment in function \u000A <v>interpret-term-as-congruence-rule</v>, ACL2 source file <v>defthm.lisp</v>, \u000A where it is used to prove <v>nil</v> in Version  6.3.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (e y1 y2)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (h y2 y1)\u000A                (h y2 y2)))</code> \u000A \u000A <p>Fixed a bug in the ACL2 character reader that was causing an end-of-file \u000A error when reading from a string ending in <tt>\"#\\c\"</tt>, for <v>c</v> a \u000A character or non-terminating sequence of characters.  Thanks to Jared Davis \u000A for sending the following example, whose evaluation in raw Lisp had caused an \u000A error.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((*readtable* *acl2-readtable*)\u000A       (stream (make-string-input-stream &quot;#\\\\a&quot;))\u000A       (x1 (read stream nil :EOF))\u000A       (x2 (read stream nil :EOF)))\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> x1 x2))</code> \u000A \u000A <p>(GCL only) Improved the automatic proclaiming mechanism used for GCL \u000A builds, in particular to avoid computing a return type when a term is detected \u000A that could come from a call of <tt><see topic=\"ACL2____NON-EXEC\">non-exec</see></tt>, and to do a more complete job \u000A for calls of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>.</p> \u000A \u000A <p>(CCL only) A CCL bug was treating filenames of the form <v>&quot;~/user/...&quot;</v> \u000A as <v>&quot;~/...&quot;</v>.  Thanks to Gary Byers for sending us a CCL-specific form \u000A that is now included in the ACL2 sources, which avoids this problem.</p> \u000A \u000A <p>Attempts to submit <tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> rules had unfortunate consequences in \u000A the case that the function ``symbol'' is <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> or <tt><see topic=\"COMMON-LISP____QUOTE\">quote</see></tt>: for \u000A <v>if</v>, rules were appropriately ignored because of the special handling that \u000A ACL2 already gives to calls of <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> for congruence-based reasoning, \u000A while for <v>quote</v>, it was possible to get a hard Lisp error.  Now, attempts \u000A to submit such rules will result in a clear ACL2 error.</p> \u000A \u000A <p>It had been possible to get a confusing raw Lisp error when submitting a \u000A <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> event whose first argument is not a symbol, as in: \u000A <v>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> (fld :type integer :initially 0))</v>.  That user error is now \u000A reported gracefully by ACL2.</p> \u000A \u000A <p>When an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form is executed for a non-existent book, we \u000A no longer get a bogus warning about a missing compiled file.  (Of course the \u000A compiled file is missing when the book itself is missing!  So there's no need \u000A to report this fact.)  Thanks to Caleb Eggensperger for bringing this issue to \u000A our attention.</p> \u000A \u000A <h3>Changes at the System Level</h3> \u000A \u000A <p>The ACL2 system <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been reworked for the <see topic=\"ACL2____XDOC\">xdoc</see> \u000A framework developed by Jared Davis.  While the ACL2 User's Manual can still be \u000A built, it is now possible for the ACL2 community to contribute to the ACL2 \u000A system documentation, in community book \u000A <tt>books/system/doc/acl2-doc.lisp</tt>; see comments near the top of that \u000A book.  Now that both the ACL2 system documentation and much of the community \u000A books documentation are written in <see topic=\"ACL2____XDOC\">xdoc</see> format, we hope the ACL2 \u000A community will add links from the ACL2 system documentation topics to book \u000A documentation topics.  Note that <v>:</v><see topic=\"ACL2____DOC\">doc</see> still works at the terminal, \u000A but it is based on the new system, and other terminal-based access to the \u000A documentation has been eliminated (for example <v>:more</v>).  Thanks to Jared \u000A for all his work contributing to this enhancement.</p> \u000A \u000A <p>Fixed community books <v>books/Makefile-generic</v> and also \u000A <v>books/Makefile</v>, which now has filename <v>books/GNUmakefile</v>, to work \u000A with Cygwin (Windows).  Thanks to Harsh Raju Chamarthi for his substantial \u000A help.</p> \u000A \u000A <p>The guard for system function <v>ev-fncall-w</v> now includes an arity \u000A check.</p> \u000A \u000A <h3>EMACS Support</h3> \u000A \u000A <p>There is now an Emacs utility for browsing the hypertext documentation for \u000A ACL2 and the community books.  This browser, ACL2-Doc, essentially serves as a \u000A replacement for Emacs Info, which can no longer be used to browse the \u000A documentation.  ACL2-Doc can be used not just for the ACL2 User's Manual but \u000A also for the ACL2+Books Manual.  It is loaded automatically into Emacs if you \u000A load the file <tt>emacs/emacs-acl2.el</tt>.  See <see topic=\"ACL2____ACL2-DOC\">ACL2-doc</see> and <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <h3>Experimental/Alternate Versions</h3> \u000A \u000A <p>(None to report this time.)</p>"],

 "ACL2____NOTE-6-3":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 6.2 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>The evaluation of a term from a <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypothesis had been \u000A expected to produce an alist binding free variables to terms.  While that is \u000A still legal, it is also legal for that evaluation to produce a list of such \u000A alists: then each is considered, until one of them permits all remaining \u000A hypotheses to be relieved.  See <see topic=\"ACL2____BIND-FREE\">bind-free</see>.  Thanks to Sol Swords for \u000A requesting this enhancement.</p> \u000A \u000A <p>ACL2 continues to provide a way to specify keyword command abbreviations \u000A for the top-level loop; see <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see>.  However, <tt><see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see></tt> is now a <see topic=\"ACL2____TABLE\">table</see> rather than a <see topic=\"ACL2____STATE\">state</see> global; \u000A it is thus no longer a so-called <see topic=\"ACL2____LD\">ld</see> special.  The functionality of \u000A <v>set-ld-keyword-aliases</v> has essentially been preserved, except that it is \u000A now an event (see <see topic=\"ACL2____EVENTS\">events</see>), hence it may appear in a book; it is <see topic=\"ACL2____LOCAL\">local</see> to a book (or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event); and the <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A argument is optional, and deprecated.  A non-local version \u000A (<v>set-ld-keyword-aliases!</v>)  has been added, along with corresponding \u000A utilities <v>add-keyword-alias</v> and <v>add-keyword-alias!</v> for adding a \u000A single keyword alias.  See <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see>.  Thanks to Jared Davis \u000A for correspondence that led us to make this change.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>(<see topic=\"ACL2____EXIT\">exit</see> t)</v> now exits without a query \u000A (but still prints an event to show the <v>:INSTRUCTIONS</v>).  Thanks to Warren \u000A Hunt for feedback leading us to make this change.</p> \u000A \u000A <p>We made the following minor changes to the behavior of <v>dmr</v>; see <see topic=\"ACL2____DMR\">dmr</see>.  First, if <v>dmr</v> monitoring is enabled, then <v>(dmr-start)</v> will \u000A have no effect other than to print a corresponding observation, and if \u000A monitoring is disabled, then <v>(dmr-stop)</v> will have no effect other than to \u000A print a corresponding observation.  Second, it had been the case that when \u000A <v>(dmr-start)</v> is invoked, the debugger was always automatically enabled \u000A with value <v>t</v> (see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>), and the debugger remained \u000A enabled when <v>(dmr-stop)</v> was invoked.  Now, the debugger is only enabled \u000A by <v>(dmr-start)</v> if it is not already enabled and does not have setting \u000A <v>:never</v>.  Moreover, if such automatic enabling takes place, then the old \u000A setting for the debugger is restored by <v>(dmr-stop)</v> unless <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt> has first been called after that automatic enabling. \u000A Finally, if the value of <see topic=\"ACL2____STATE\">state</see> global variable <v>'debugger-enable</v> is \u000A <v>:bt</v>, then the new value will be <v>:break-bt</v>, not <v>t</v>.</p> \u000A \u000A <p>When a call of <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> is executed in the ACL2 loop, its constituent \u000A <see topic=\"ACL2____EVENTS\">events</see> and their results are printed, just as was already done for \u000A calls of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>.  Thanks to Jared Davis for a conversation \u000A causing us to consider this change.</p> \u000A \u000A <p>(CCL only) When <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt> is invoked with an argument \u000A that prints a backtrace and CCL is the host Lisp, the backtrace will be \u000A limited to 10,000 stack frames.  (We have seen more than 65,000 stack frames \u000A before this change.)  This limit is the value of raw Lisp variable \u000A <v>*ccl-print-call-history-count*</v>, which may be assigned another positive \u000A integer value to serve as the maximum number of stack frames to be \u000A printed.</p> \u000A \u000A <p>Improvements have been made pertaining to the disabling (inhibiting) of \u000A individual types of warning.  Now, inhibited warnings are implemented in a \u000A straightforward way using a separate <see topic=\"ACL2____TABLE\">table</see> for this purpose, the \u000A <v>inhibit-warnings-table</v>, rather than using the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.  See <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>, and see <see topic=\"ACL2____SET-INHIBIT-WARNINGS_12\">set-inhibit-warnings!</see> for a variant that is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or a book in which it occurs.  Thanks to Sol Swords for sending \u000A examples showing how <tt><see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see></tt> did not always behave as one \u000A might reasonably expect when books are involved.</p> \u000A \u000A <p>It had been the case that <tt><see topic=\"ACL2____LP\">lp</see></tt> took a single argument, <v>'raw</v>. \u000A This argument was not documented and also caused an error, so it has been \u000A eliminated.</p> \u000A \u000A <p>The functionality of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> has been significantly expanded. \u000A First: if the expansion is of the form <v>(:OR e1 e2 ...)</v>, then event forms \u000A <v>e1</v>, <v>e2</v>, and so on are evaluated, in order, until the evaluation of \u000A some <v>ek</v> completes without error.  In that case, the expansion is treated \u000A simply as <v>ek</v>.  With this capability, alternative expansions can be \u000A attempted and the successful one does not need to be evaluated again.  See the \u000A new version of community book <v>books/make-event/proof-by-arith.lisp</v> for an \u000A example.  Second, an expansion may be of the form <v>(:DO-PROOFS e)</v>, in \u000A which case the event <v>e</v> is evaluated with proofs <b>not</b> skipped; see \u000A <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.  Third, new keyword <v>:EXPANSION?</v> can be used to \u000A avoid storing expansions in certificate files.  See <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.</p> \u000A \u000A <p>When a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event prints a failure message in the summary, that \u000A message now indicates when the failure is due to a failed proof of guard \u000A verification or a failed proof of the measure theorem.  Thanks to Shilpi Goel \u000A for requesting this enhancement.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>ACL2 can now be instructed to time activities using real time (wall clock \u000A time) instead of run time (typically, cpu time).  See <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.  Thanks to Jared Davis for asking to be able to obtain \u000A real-time reports in event summaries.</p> \u000A \u000A <p>A new utility, <tt><see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see></tt>, is similar to existing utility <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> in that it executes a command.  Unlike <v>sys-call</v>, however, \u000A <v>sys-call+</v> returns values that include output from the command (in \u000A addition to the exit status), rather than simply printing the command.  See \u000A <see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see>.</p> \u000A \u000A <p>The new macro <tt><see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see></tt> extends the functionality of <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> by permitting macro-aliases (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>). \u000A See <see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see>.  Thanks to Jared Davis for requesting this feature \u000A and suggesting the use of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> in its implementation.  We have \u000A also modified <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> to print a friendlier error message when \u000A its argument is a macro-alias.</p> \u000A \u000A <p>See <see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see> for a new utility that returns the number of \u000A prover steps most recently taken.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>The processing of <v>:use</v> and <v>:by</v> <see topic=\"ACL2____HINTS\">hints</see> has been changed in \u000A the following two rather subtle ways, thanks to suggestions from Sol \u000A Swords.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o For <v>:by</v> hints, the simplest check was an equality check, rather than \u000A a more general subsumption check.  That equality check was made after removing \u000A so-called ``guard holders'' (<tt><see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see></tt>, <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt>, <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt>, <tt><see topic=\"COMMON-LISP____THE\">the</see></tt>) from both the previous theorem and the purported \u000A theorem.  Now, guard-holder removal has been strengthened, so that the results \u000A are also put into so-called quote-normal form, for example replacing <v>(<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A '3 '4)</v> by <v>'(3 . 4)</v>.</p> \u000A \u000A <p>o For a <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> provided to a <v>:use</v> or <v>:by</v> hint that \u000A is a <v>:functional-instance</v>, if a <v>:do-not</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>) has \u000A specified that <v>preprocess-clause</v> is not to be used, then preprocessing \u000A will not be used on the constraints.</p></blockquote> \u000A \u000A <p>We eliminated certain warnings about being ``weak'' for every <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule whose conclusion designates that the function call can \u000A be equal to one of its arguments, e.g., <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (foo y)) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo\u000A y) y))</v>.  In many cases (such as the one above), such warnings about ``weak'' \u000A simply aren't correct.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a soundness bug that was permitting a <see topic=\"ACL2____STOBJ\">stobj</see> to be bound by a \u000A <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> or <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> form, without being among the outputs of that \u000A form.  Thanks to Jen Davis and Dave Greve for reporting this bug.  Their \u000A report included an example which forms the basis for a proof of <v>nil</v>, \u000A included as a comment in the form <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-3 ...)</v> in ACL2 source \u000A file <v>ld.lisp</v>.</p> \u000A \u000A <p>(GCL only) Fixed an obscure soundness bug due to an error in the GCL \u000A implementation of <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt>.  For details, see the relevant \u000A comment in the ACL2 source code under <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-3 ...)</v>.</p> \u000A \u000A <p>Fixed a bug in the case of a field of a (concrete) stobj that is an \u000A abstract stobj (see <see topic=\"ACL2____NESTED-STOBJS\">nested-stobjs</see>).  Thanks to David Rager for bringing \u000A this bug to our attention.</p> \u000A \u000A <p>Splitter output for type <v>if-intro</v> (see <see topic=\"ACL2____SPLITTER\">splitter</see>) could formerly \u000A occur even when at most one subgoal is generated.  This has been fixed.</p> \u000A \u000A <p>Fixed a bug in <tt><see topic=\"ACL2____WOF\">wof</see></tt>, hence in <tt><see topic=\"ACL2____PSOF\">psof</see></tt> (which uses <v>wof</v>), \u000A that was causing the printing of a bogus error message.</p> \u000A \u000A <p>A small logical bug has been fixed in the logical definition of <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt>.  Formerly it always returned <v>(<see topic=\"ACL2____MV\">mv</see> nil state)</v> whenever \u000A the oracle of the state is non-empty (see <see topic=\"ACL2____STATE\">state</see>).</p> \u000A \u000A <p>Fixed a bug that was causing an error upon evaluation of the form \u000A <v>(<see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> nil)</v>.  Thanks to David Russinoff for reporting \u000A this error.</p> \u000A \u000A <p>The <v>:measure</v> (if supplied) is now ignored when checking redundancy \u000A with respect to a non-recursive definition that is not defined within a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>.  (See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see> and see <see topic=\"ACL2____XARGS\">xargs</see>.)  It \u000A had been possible to get a low-level ACL2 error in this situation.  Thanks to \u000A Jared Davis for reporting this bug with a helpful example.</p> \u000A \u000A <p>Eliminated a potential error when using <tt><see topic=\"ACL2____COMP\">comp</see></tt> to compile an \u000A uncompiled function defined under <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, which we observed in \u000A LispWorks.</p> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL</b></p> \u000A \u000A <p>The ACL2 sources are now publicly available between ACL2 releases, using \u000A svn; see the new ``<v>acl2-devel</v>'' project hosted by Google code at <a \u000A href='http://acl2-devel.googlecode.com'>http://acl2-devel.googlecode.com</a>. \u000A Although such a copy of ACL2 is likely to work well with the latest svn \u000A (trunk) revision of the ACL2 community books (see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>), \u000A please take seriously the warning message printed at startup: ``The authors of \u000A ACL2 consider svn distributions to be experimental; they may be incomplete, \u000A fragile, and unable to pass our own regression.''  That message also provides \u000A instructions for bug reports.  If you decide to use svn versions of either the \u000A community books or ACL2, then you should use both, as they tend to be kept in \u000A sync.  We fully expect ACL2 releases to continue from time to time, as usual. \u000A Thanks to Jared Davis for his efforts in setting up the new acl2-devel project \u000A and svn repository, and to him and David Rager for convincing us to distribute \u000A ACL2 sources via svn between releases.</p> \u000A \u000A <p>Thanks to a suggestion from Jared Davis, over 30 built-in functions are now \u000A declared to be inline in order to boost performance.  (The list may be found \u000A by searching ACL2 source file <v>axioms.lisp</v> for ``<v>(declaim\u000A (inline</v>''.)</p> \u000A \u000A <p>Better support has been provided for command line arguments, especially \u000A those supplied directly by the user when calling ACL2.  For one, problems with \u000A quoting have been solved using <v>&quot;$@&quot;</v> in place of <v>$*</v>.  Also, the \u000A function <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> now allows specification of arguments, both for the \u000A host Lisp as well as ``inert'' arguments that can be passed along to calls of \u000A programs (as with <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>).  A keyword argument, \u000A <v>:return-from-lp</v>, specifies a form to evaluate before quitting the \u000A read-eval-print loop at startup.  See <see topic=\"ACL2____SAVE-EXEC\">save-exec</see>.  Also see the source \u000A function <v>user-args-string</v> and its comments, source file \u000A <v>acl2-init.lisp</v>, for more information.  Thanks to Jared Davis for \u000A suggesting the use of <v>&quot;$@&quot;</v>, as well as modifications to <tt><see topic=\"ACL2____SAVE-EXEC\">save-exec</see></tt> and helpful conversations about that.</p> \u000A \u000A <p>A rather extensive overhaul has taken place for the function proclaiming \u000A mechanism.  As before, this is only used when the host Lisp is GCL.  However, \u000A building an executable is now faster for some Lisps, including GCL, by \u000A avoiding repeated recompilation and perhaps repeated initialization.</p> \u000A \u000A <p>(CCL only) We increased stack sizes when the host Lisp is CCL.  The default \u000A for recent CCL versions is equivalent to specifying `<v>-Z 2M</v>' on the \u000A command line, but saved ACL2 scripts (including experimental versions ACL2(h), \u000A ACL2(p), ACL2(r), and combinations of them) to `<v>-Z 64M</v>', representing a \u000A 32-fold increase.  Thanks to Jared Davis for pointing us to community books \u000A file <v>books/centaur/ccl-config.lsp</v> and to Sol Swords for helpful \u000A discussions.</p> \u000A \u000A <p>(SBCL only) Fixed <v>save-exec</v> for host Lisp SBCL to provide the same \u000A export of variable <v>SBCL_HOME</v> that was provided in the original \u000A <v>saved_acl2</v> script.</p> \u000A \u000A <p>(GCL only) We made changes, following suggestions from Camm Maguire (whom \u000A we thank for these suggestions), to support ACL2 builds on recent versions of \u000A GCL (2.6.8 and 2.6.10; we recommend against using GCL 2.6.9, since issues \u000A there were fixed in 2.6.10).  Specifically, we no longer set the hole size, \u000A and we allocate contiguous pages sufficient to run an ACL2 regression without \u000A failing due to memory limitations.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p>Modified file <v>emacs/emacs-acl2.el</v> to eliminate some warnings that were \u000A appearing in a recent Emacs version, replacing <v>(end-of-buffer)</v> by \u000A <v>(goto-char (point-max))</v> and <v>next-line</v> by <v>forward-line</v>.  Thanks \u000A to Warren Hunt for bringing the warnings to our attention.</p> \u000A \u000A <p><b>EXPERIMENTAL/ALTERNATE VERSIONS</b></p> \u000A \u000A <p>(Allegro CL only) ACL2(h) now avoids blow-ups in hash table sizes that \u000A could be caused by <see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see>.  Thanks to Jared Davis for \u000A helping to debug this problem, and to David Rager for contributing the \u000A community book <v>books/parsers/earley/earley-parser.lisp</v>, which highlighted \u000A this problem.</p> \u000A \u000A <p>(SBCL only) Fixed a bug that was causing a Lisp break after turning on \u000A <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see>.  Thanks to David Rager for confirming that our \u000A proposed fix is correct.</p>"],

 "ACL2____NOTE-6-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 6.1 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt> has been changed, so that evaluation of a form \u000A <v>(<see topic=\"ACL2____TOP-LEVEL\">top-level</see> x)</v> results in an error when evaluation of <v>x</v> results in an \u000A error.  Thanks to Jared Davis for observing that when evaluating a file using \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt>, an interrupt of a call of a <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt> call in that file \u000A would not prevent evaluation of later forms in the file.</p> \u000A \u000A <p>The macro <tt><see topic=\"COMMON-LISP____THE\">the</see></tt> no longer causes an error when <see topic=\"ACL2____GUARD\">guard</see>-checking \u000A is <v>:NONE</v>.  For example, it had been the case that evaluation of <v>(<see topic=\"COMMON-LISP____THE\">the</see>\u000A integer t)</v> always caused an error; but now, there is no error after \u000A executing command <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :NONE</v>.  Thanks to \u000A Jared Davis for asking for a way to avoid such errors.</p> \u000A \u000A <p>The error printed when attempting to ``reincarnate'' a package &mdash; that \u000A is, to define a package that is not part of the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> but \u000A exists in raw Lisp because it was once part of the world &mdash; is now much \u000A more instructive.  In particular, it shows pathnames for the previous and \u000A proposed <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events, and it shows the symbols that are imported by \u000A one but not the other.  Thanks to Jared Davis for requesting this \u000A improvement.</p> \u000A \u000A <p>Functions <tt><see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see></tt> and <tt><see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see></tt> no \u000A longer cause an error when failing to open a channel because of a permissions \u000A problem, but instead return <v>(<see topic=\"ACL2____MV\">mv</see> nil state)</v>.  Thanks to Jared Davis for \u000A requesting this change.  (Note: this change does not apply if the host Lisp is \u000A non-ANSI, i.e., if the host Lisp is non-ANSI GCL.)</p> \u000A \u000A <p>The advanced <see topic=\"ACL2____META-EXTRACT\">meta-extract</see> mechanisms, provided for using facts from \u000A the <see topic=\"ACL2____WORLD\">world</see> or metafunction context, have been enhanced in the following \u000A ways, in collaboration with Sol Swords.  See <see topic=\"ACL2____META-EXTRACT\">meta-extract</see> for more \u000A details.</p> \u000A \u000A <blockquote> \u000A \u000A <p>It is now permissible to use calls of <v>meta-extract-global-fact</v> in \u000A hypotheses of <tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> rules, much as they are used in \u000A hypotheses of <tt><see topic=\"ACL2____META\">meta</see></tt> rules.  See <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>.  Thanks to Sol \u000A Swords for requesting this feature.</p> \u000A \u000A <p>The utility <v>meta-extract-global-fact</v> is now a macro, which expands to \u000A a corresponding call of the new function, <v>meta-extract-global-fact+</v>. \u000A This new function takes an alternate, extra <see topic=\"ACL2____STATE\">state</see> as an argument; it is \u000A not to be executed, and it operates on the alternate state, whose logical \u000A <see topic=\"ACL2____WORLD\">world</see> is intended to be the same as that of the ``live'' (usual) \u000A state.</p> \u000A \u000A <p>A new sort of value for the <v>obj</v> argument is supported for \u000A <v>meta-extract-global-fact</v> (and <v>meta-extract-global-fact+</v>), which \u000A results in a term equating a function application to its result.  See <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>, in particular the discussion of <v>:fncall</v>.</p></blockquote> \u000A \u000A <p>It is now possible for <v>trace$</v> to avoid printing prefixes of the form \u000A <v>&quot;n&gt; &quot;</v> and <v>&quot;&lt;n &quot;</v>, while also (optionally) avoiding indentation. \u000A See <see topic=\"ACL2____TRACE_42\">trace$</see>, in particular the discussion of <v>:fmt!</v>.  Thanks to \u000A Shilpi Goel for requesting this feature.</p> \u000A \u000A <p>It was possible for the <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see> feature to generate duplicate \u000A calls of <v>extra-info</v> in hypotheses generated for guard verification.  We \u000A have eliminated duplicates of this sort.</p> \u000A \u000A <p>When <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> returns without error, it returns a value \u000A <v>(:NUMBER-OF-ENABLED-RUNES k)</v>, where <v>k</v> is the length of the new \u000A current theory.  (Formerly, <v>k</v> was returned.)  This value is thus printed \u000A when an <v>in-theory</v> event is submitted at the top level.  Thanks to Gisela \u000A Rossi for feedback that led us to make this change.</p> \u000A \u000A <p>A new keyword parameter for <tt><see topic=\"ACL2____LD\">ld</see></tt> is <v>:ld-missing-input-ok</v>.  Its \u000A default value is <v>nil</v>, which causes an error, as before, upon failure to \u000A open a specified file.  Other legal values are <v>t</v> and <v>:WARN</v>; see \u000A <see topic=\"ACL2____LD-MISSING-INPUT-OK\">ld-missing-input-ok</see> and see <see topic=\"ACL2____LD\">ld</see>.</p> \u000A \u000A <p>Extended <v>*acl2-exports*</v>, in particular adding <v>UNSIGNED-BYTE-P</v> and \u000A <v>SIGNED-BYTE-P</v> (thanks to a suggestion by Jared Davis)</p> \u000A \u000A <p>Even if the function <v>f</v> is defined to take one or more <see topic=\"ACL2____STOBJ\">stobj</see> \u000A arguments, the form <v>(<see topic=\"ACL2____EC-CALL\">ec-call</see> (f ...))</v> is now legal if all arguments of \u000A the call of <v>f</v> are non-stobj objects, in any context where only ordinary \u000A object return values are expected.</p> \u000A \u000A <p>When the second argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is a symbol, that symbol \u000A formerly needed to be <v>?</v> or <v>t</v>, in the <v>&quot;ACL2&quot;</v> package.  Now, \u000A the <see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> of the second argument is ignored: any symbol \u000A whose <see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> is <v>&quot;?&quot;</v> or <v>&quot;T&quot;</v> is treated the same in \u000A that argument position as the symbol <v>?</v> or <v>t</v> in the <v>&quot;ACL2&quot;</v> \u000A package, respectively.  Thanks to Warren Hunt and Nathan Wetzler for \u000A suggesting consideration of a more relaxed criterion for that second \u000A argument.</p> \u000A \u000A <p>(For system hackers, not standard ACL2 users:) Utilities <tt><see topic=\"ACL2____INITIALIZE-EVENT-USER\">initialize-event-user</see></tt> and <tt><see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see></tt> now each take a list of \u000A three arguments, <v>(ctx body state)</v>.  Thanks to Harsh Raju Chamarthi for \u000A requesting this change.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>It is now permissible to specify a <see topic=\"ACL2____STOBJ\">stobj</see> field that is itself either \u000A a stobj or an array of stobjs.  A new primitive, <tt><see topic=\"ACL2____STOBJ-LET\">stobj-let</see></tt>, is \u000A provided in order to access or update such fields; see <see topic=\"ACL2____STOBJ-LET\">stobj-let</see>. \u000A Thanks to Warren Hunt and Sol Swords for requesting the ability to specify \u000A nested stobjs.</p> \u000A \u000A <p>New accessor function <v>(mfc-world mfc)</v> returns the world component of a \u000A metafunction context.  See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.</p> \u000A \u000A <p>A new <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword, <v>:SPLIT-TYPES</v>, can be used to ``split'' \u000A the <see topic=\"COMMON-LISP____TYPE\">type</see> declarations from the <see topic=\"ACL2____GUARD\">guard</see> in the following sense.  By \u000A default, or when <v>:SPLIT-TYPES</v> has value <v>nil</v>, one gets the existing \u000A behavior: the terms corresponding to type declarations are conjoined into the \u000A guard.  However, if <v>:SPLIT-TYPES t</v> is specified, then that is not the \u000A case; instead, guard verification will require that these terms are proved \u000A under the hypothesis that the guard holds.  In this way, one can add type \u000A declarations to assist the host Lisp compiler without cluttering the \u000A function's guard.  See <see topic=\"ACL2____XARGS\">xargs</see>.  Thanks to Jared Davis for requesting \u000A this feature.</p> \u000A \u000A <p>Advanced users may want to see <see topic=\"ACL2____QUICK-AND-DIRTY-SUBSUMPTION-REPLACEMENT-STEP\">quick-and-dirty-subsumption-replacement-step</see> for a way to turn off a prover \u000A heuristic.  Thanks to those who have mentioned to us potential issues with \u000A this heuristic, most recently Dave Greve.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>We made changes to the ``ancestors check'' heuristic (source function \u000A <v>ancestors-check-builtin</v>), as follows.</p> \u000A \u000A <blockquote> \u000A \u000A <p>The heuristic could prevent a <see topic=\"ACL2____REWRITE\">rewrite</see> rule's hypothesis from being \u000A rewritten to true, even when that hypothesis is of the form <v>(<see topic=\"ACL2____FORCE\">force</see>\u000A &lt;term&gt;)</v>.  Now, forcing will take place as expected; see <see topic=\"ACL2____FORCE\">force</see>. \u000A Thanks to Robert Krug for bringing this issue to our attention and sending an \u000A example, which we include as a comment in the ACL2 source code (see \u000A <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-2 ...)</v>).</p> \u000A \u000A <p>The heuristic is now delayed until after we check whether the hypothesis is \u000A already known, using <see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning alone (in particular, not using \u000A rewriting), to be true or to be false.  We believe that this is now the \u000A ``right'' order for those two operations.  We saw a slight speed up in the \u000A regression tests (about a percent) with this change, but that might be in the \u000A noise.</p> \u000A \u000A <p>A technical change makes the heuristic slightly less aggressive in \u000A preventing backchaining.  Roughly speaking, ordering checks based on function \u000A symbol counts could suffice to permit backchaining, where now variable counts \u000A also suffice.  Thanks to Robert Krug for showing us an example where \u000A backchaining led to a term with no free variables that was nevertheless \u000A subject to the ancestors check, preventing it from being rewritten.</p> \u000A \u000A <p>(For those who use <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> to attach to <v>ancestors-check</v>) We \u000A have used <v>defrec</v> to introduce an `<v>ancestor</v>' data structure.  A new \u000A function, <v>strip-ancestor-literals</v>, should be used to obtain the literals \u000A from a list of ancestors, although <v>strip-cars</v> will still work at this \u000A time.</p></blockquote> \u000A \u000A <p>When we rewrite the current literal of the current clause we assume the \u000A falsity of the other literals and of the conclusions produced by forward \u000A chaining.  We have changed the order in which those assumptions are made, \u000A which affects the type-alist used during rewriting.  This has three effects: \u000A the new type-alist, which is sometimes stronger than the old one, may allow \u000A additional rules to fire, the choice of free vars may be different, and the \u000A order of the literals in forced subgoals may be different.  Should ``legacy'' \u000A proofs fail under the new type-alist, we recommend looking for rules that are \u000A fired in the new proof that were not fired (on that same subgoal) in the old \u000A one.  Thanks to Dave Greve for sending us an example that led us to make this \u000A change.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>We fixed a soundness bug that could be exploited by calling system \u000A functions <v>acl2-magic-mfc</v> or <v>acl2-magic-canonical-pathname</v>.  Thanks \u000A to Sol Swords for bringing this bug to our attention.</p> \u000A \u000A <p>We fixed a soundness bug in the handling of <see topic=\"ACL2____STOBJ\">stobj</see>s, in which strings \u000A were recognized as stobjs in raw Lisp.  Thanks to Jared Davis for sending us a \u000A proof of <v>nil</v> that exploited this bug.  We now have a much simpler example \u000A of this bug, as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad (stp &quot;abc&quot;) :rule-classes nil)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (stp)) :use bad))\u000A  :rule-classes nil)</code> \u000A \u000A <p>We fixed bugs in extended metafunctions (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>).  The macro <v>mfc-ap</v> no longer takes a \u000A <v>:TTREEP</v> keyword argument, because this argument could allow returning a \u000A tag tree that does not properly account for forcing.  The remaining \u000A <v>mfc-xx</v> macros &mdash; <v>mfc-relieve-hyp</v>, <v>mfc-rw+</v>, <v>mfc-rw</v>, \u000A and <v>mfc-ts</v> &mdash; still take a <v>:TTREEP</v> keyword argument, but the \u000A corresponding functions when <v>:TTREEP</v> is <v>t</v> &mdash; \u000A <v>mfc-relieve-hyp-ttree</v>, <v>mfc-rw+-ttree</v>, <v>mfc-rw-ttree</v>, and \u000A <v>mfc-ts-ttree</v> &mdash; were introduced with incorrect output signatures.  A \u000A complication is that <v>mfc-relieve-hyp-ttree</v> was improperly defined in raw \u000A Lisp in a way that actually matched the incorrect signature!  All of these \u000A bugs have been fixed.  Perhaps any of them could have made it possible to \u000A prove <v>nil</v>, though we have not tried to do so.</p> \u000A \u000A <p>(Windows only) On Windows, it had been possible for ACL2 not to consider \u000A two pathnames to name the same file when the only difference is the case of \u000A the drive, e.g., `<v>C:</v>' vs. `<v>c:</v>'.  This has been fixed.  Thanks to Sol \u000A Swords for reporting this issue.</p> \u000A \u000A <p>Fixed a bug in the storing of rules for the tau system; see <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>.  (The error message mentions \u000A PARTITION-SIGNATURE-HYPS-INTO-TAU-ALIST-AND-OTHERS.)  Thanks to Sol Swords for \u000A reporting this bug and sending a simple example to illustrate it.</p> \u000A \u000A <p>It had been possible to admit the missing <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events printed by \u000A <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>, and yet get an error when subsequently submitting the \u000A same <v>defabsstobj</v> event, stating: ``Note discrepancy with existing \u000A formula''.  The problem could occur when an expression of the form <v>(<see topic=\"COMMON-LISP____OR\">or</see> X\u000A Y)</v> occurred in one of those missing events, because ACL2 created it from the \u000A term <v>(<see topic=\"COMMON-LISP____IF\">if</see> X 't Y)</v> but then translated <v>(<see topic=\"COMMON-LISP____OR\">or</see> X Y)</v> to <v>(<see topic=\"COMMON-LISP____IF\">if</see> X X Y)</v>, \u000A resulting in a mismatch.  This has been fixed.  Thanks to Jared Davis for \u000A reporting this bug using a simple example.</p> \u000A \u000A <p>A hard Lisp error was possible for certain illegal functional substitutions \u000A (see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>).  Thanks to Sol Swords for reporting this bug.</p> \u000A \u000A <p>We fixed a bug in the case that an exported function of a <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> event had a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.  Thanks to Jared Davis for \u000A sending a simple example when reporting this bug.</p> \u000A \u000A <p>We now avoid an infinite loop that could occur when attempting to close the \u000A standard character output channel (see <see topic=\"ACL2____STANDARD-CO\">standard-co</see>).  Instead, an error \u000A message explains how to accomplish what was probably intended.  Thanks to \u000A Shilpi Goel for bringing this issue to our attention.</p> \u000A \u000A <p>(Windows only) Fixed a bug that was causing a hard error on Windows when \u000A ACL2 encountered filenames starting with the tilde character (<v>~</v>), for \u000A example, <v>(<see topic=\"ACL2____LD\">ld</see> &quot;~/acl2-customization.lsp&quot;)</v>.  Thanks to Sol Swords for \u000A bringing this bug to our attention.  Also thanks to Harsh Raju Chamarthi for a \u000A useful conversation that led to a better fix than our first one.</p> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL</b></p> \u000A \u000A <p>ACL2 may now be built on recent versions of a new host Lisp, ANSI Gnu \u000A Common Lisp (GCL).  Traditional (non-ANSI) GCL was the original host Lisp \u000A underlying ACL2, and we are grateful for GCL support that we received from the \u000A late Bill Schelter and, more recently and particularly for ANSI GCL, from Camm \u000A Maguire.</p> \u000A \u000A <p>The `make' process suggested for book certification has changed \u000A substantially, thanks in large part to contributions from Jared Davis and Sol \u000A Swords.  We have seen the new process provide better performance on machines \u000A with many cores, and we expect maintenance advantages such as eliminating the \u000A need for Makefiles in individual book directories.  The ``classic'' process, \u000A which was based on community books file <v>books/Makefile-generic</v>, is still \u000A supported (see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>) but may disappear in a \u000A future release of ACL2.  See <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see>.  Most changes should \u000A be invisible to the user, other than improved `make'-level parallelism, with \u000A the exception of the following.</p> \u000A \u000A <blockquote><p>o Variable <v>ACL2_JOBS</v> is no longer supported, nor is it \u000A necessary; simply use `make' option `-j' instead.</p> \u000A \u000A <p>o Regressions now use `make' option <v>-k</v> by default, which causes the \u000A regression to keep going after errors, rather than <v>-i</v>, which ignores \u000A errors.  If you encounter problems because of this change, use \u000A <v>ACL2_IGNORE=-i</v> with your `make' command.</p> \u000A \u000A <p>o The `regression' target works for the experimental extension, ACL2(h) \u000A (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>); target `regression-hons' no longer \u000A exists.</p></blockquote> \u000A \u000A <p>Please let us know if you run into problems with the new infrastructure, as \u000A we consider the legacy infrastructure to be deprecated and we will probably \u000A eliminate much of it in the future.  In particular, circular dependencies were \u000A formerly prohibited at the directory level, but that is no longer the case, \u000A and we expect such cycles to occur in the future.</p> \u000A \u000A <p>Although ACL2 users don't typically modify raw Lisp variables, we have \u000A arranged to reset Lisp variable <v>*default-pathname-defaults*</v> if necessary \u000A at startup so that it will not interfere with ACL2, in particular by messing \u000A up the initial connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>).  Thanks to Jared \u000A Davis, Sol Swords, and Raymond Toy for helping us to identify this issue.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p><b>EXPERIMENTAL/ALTERNATE VERSIONS</b></p> \u000A \u000A <p>In ACL2(h), <tt><see topic=\"ACL2____PRINT-OBJECT_42\">print-object$</see></tt> no longer uses the serialize printer \u000A except in system applications as before (e.g., write out <v>.cert</v> files). \u000A Thanks to Dave Greve for bringing this issue to our attention.</p> \u000A \u000A <p>Jared Davis contributed changes related to the <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> utility of \u000A ACL2(h), including some low-level changes as well as the following.</p> \u000A \u000A <p>o <tt><see topic=\"ACL2____NEVER-MEMOIZE\">Never-memoize</see></tt> specifies that a given function should never be \u000A memoized.</p> \u000A \u000A <p>o Removed <v>memoize-let</v>, which may never have ever been used.</p> \u000A \u000A <p>o Removed the <v>:inline</v> keyword option to memoize, which was just an \u000A   alias for the <v>:recursive</v> option.</p> \u000A \u000A <p>For ACL2(p), some anomalous behavior may no longer occur because prover \u000A calls (more specifically, trips through the ACL2 ``waterfall'') will return \u000A only after all sub-computations (threads) have finished.  Thanks to David \u000A Rager for contributing this improvement.</p> \u000A \u000A <p>ACL2(pr), which includes <see topic=\"ACL2____PARALLELISM\">parallelism</see> (as for ACL2(p)) and \u000A non-standard analysis support for the <see topic=\"COMMON-LISP____REAL\">real</see>s (as for ACL2(r)), now \u000A builds and can certify the community <v>nonstd/</v> books.  Thanks to David \u000A Rager for his contribution to this capability.</p>"],

 "ACL2____NOTE-6-0":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 5.0 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level, Emacs support, and \u000A experimental versions.  Each change is described in just one category, though \u000A of course many changes could be placed in more than one category.</p> \u000A \u000A <p>NOTE.  But we start with one major change that is outside the usual \u000A categories:</p> \u000A \u000A <p><b>LICENSE change</b></p> \u000A \u000A <p>The ACL2 license has been changed from GPL Version 2 to a 3-clause BSD \u000A license, found in the <v>LICENSE</v> file distributed with ACL2.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>Function <tt><see topic=\"ACL2____FMT-TO-STRING\">fmt-to-string</see></tt> and similar functions (see <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>) now use the default right margin settings; formerly the \u000A right margin had been set at 10,000.  If you want the former behavior, you can \u000A use the <v>:fmt-control-alist</v>, as illustrated below.</p> \u000A \u000A <code>(<see topic=\"ACL2____FMT-TO-STRING\">fmt-to-string</see> &quot;~x0&quot;\u000A               (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 30)))\u000A               :fmt-control-alist\u000A               `((fmt-soft-right-margin . 10000)\u000A                 (fmt-hard-right-margin . 10000)))</code> \u000A \u000A <p>The use of attachments (see <see topic=\"ACL2____DEFATTACH\">defattach</see>) has been made more efficient, \u000A avoiding some low-level checks (Common Lisp `<v>boundp</v>' checks).  Thanks to \u000A Shilpi Goel for constructing an example that we used to help direct us to \u000A remove inefficiency.  The following results for that example &mdash; a \u000A Fibonacci program run on a machine interpreter in raw-mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) &mdash; give a sense of the potential speedup, though we note \u000A that a full ACL2(h) regression showed no significant speedup.</p> \u000A \u000A <code>; Time before the change:\u000A; 0.89 seconds realtime, 0.90 seconds runtime\u000A\u000A; Time after the change:\u000A; 0.75 seconds realtime, 0.75 seconds runtime\u000A\u000A; Time when cheating to avoid the cost of attachments, by redefining a\u000A; function to BE its attachment (so, this gives a lower bound on possible\u000A; execution time):\u000A; 0.72 seconds realtime, 0.72 seconds runtime</code> \u000A \u000A <p>Functions <v>read-acl2-oracle</v> and <v>read-acl2-oracle@par</v> are no longer \u000A untouchable (see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>).  We reported this change for \u000A Version_5.0 but it was not made; thanks to Jared Davis for bringing this to \u000A our attention.  Function <v>get-timer</v> also is no longer untouchable.</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____BUTLAST\">butlast</see></tt> now behaves more reasonably on arguments \u000A violating its <see topic=\"ACL2____GUARD\">guard</see>.  For example, <v>(<see topic=\"COMMON-LISP____BUTLAST\">butlast</see> '(1 2 3) -1)</v> is now \u000A provably equal to <v>(1 2 3)</v> instead of to <v>(1 2 3 nil)</v>.  Thanks to \u000A Jared Davis for suggesting a change to the definition of <v>butlast</v>.</p> \u000A \u000A <p>The utilities <v>mfc-ts</v> and <v>mfc-ap</v> (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>) formerly never used forcing (see <see topic=\"ACL2____FORCE\">force</see>).  Now, \u000A by default, forcing is allowed during execution of these functions if and only \u000A if it is permitted in the rewriting environment where they are called. \u000A Moreover, these and the <v>mfc-xx</v> utilities &mdash; <v>mfc-rw</v>, \u000A <v>mfc-rw+</v>, and <v>mfc-relieve-hyp</v> &mdash; are now macros that take \u000A (optional) keyword arguments <v>:forcep</v> and <v>:ttreep</v>.  The <v>:forcep</v> \u000A argument is <v>:same</v> by default, providing the forcing behavior inherited \u000A from the environment (as described above); but it can be the symbol <v>t</v> or \u000A <v>nil</v>, indicating that forcing is to be enabled or disabled, respectively. \u000A The <v>:ttree</v> argument is <v>nil</v> by default, but when it is <v>t</v>, then a \u000A second value is returned, which is a tag-tree.  See <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>.</p> \u000A \u000A <p>Many improvements have been made to the tau-system (see <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>), \u000A including support for arithmetic intervals bounded by constants.  Thus, for \u000A example, <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 15))</v> is a tau predicate.  The <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has also been improved (see <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>). Also see <see topic=\"ACL2____TIME-TRACKER-TAU\">time-tracker-tau</see> for \u000A discussion of how the new <tt><see topic=\"ACL2____TIME-TRACKER\">time-tracker</see></tt> utility can help discover ways \u000A to detect slowdown related to the tau-system.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see topic=\"ACL2____EVENTS\">events</see> printed by <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>, namely \u000A those that remain to be proved, are now given with <v>:rule-classes nil</v> \u000A since there is probably no intention to use them as rules.  Thanks to Robert \u000A Krug for suggesting that we consider this change.</p> \u000A \u000A <p>The formal parameters for a macro definition (see <see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>) may now \u000A include <tt><see topic=\"ACL2____STATE\">state</see></tt> and user-defined <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s.  (However, macro \u000A formals may not be declared as stobjs; see <see topic=\"ACL2____XARGS\">xargs</see>.)  Thanks to Jose Luis \u000A Ruiz-Reina for raising this issue and to Rob Sumners for helpful conversations \u000A &mdash; both of these nearly 10 years ago!</p> \u000A \u000A <p>The utilities <tt><see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see></tt>, <tt><see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see></tt>, <tt><see topic=\"ACL2____DEFUND-INLINE\">defund-inline</see></tt>, and <tt><see topic=\"ACL2____DEFUND-NOTINLINE\">defund-notinline</see></tt> have been simplified, by taking \u000A advantage of the lifting of restrictions on formal parameters of macro \u000A definitions mentioned above (involving symbols that happen to be <see topic=\"ACL2____STOBJ\">stobj</see> \u000A names).  Now, when any of the above four utilities is called with a given set \u000A of formal parameters, those formals will be used not only for the generated \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event but also for the generated <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> event. \u000A (Previously, they had been renamed for the <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> event in order to \u000A respect the stobj name restriction that no longer exists.)  Thanks to Jared \u000A Davis for pointing out the value of making this change.</p> \u000A \u000A <p>The <see topic=\"ACL2____EVENTS\">events</see> <tt><see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see></tt> and <tt><see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see></tt> now convert arguments as appropriate using the <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>.  For example, the event <v>(<see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> append\u000A car)</v> is now legal (though probably not a good idea), because \u000A <v>add-invisible-fns</v> is now sensitive to the fact that <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> maps \u000A to <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> in the <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>.</p> \u000A \u000A <p>When <v>:</v><v>pe</v> is applied to a built-in function that does not have a \u000A defining event, such as <tt><see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see></tt>, <v>:pe</v> now gives more useful output \u000A that points to the documentation instead of printing a call of \u000A <v>ENTER-BOOT-STRAP-MODE</v>.  Thanks to Anthony Knape for bringing this issue \u000A to our attention.</p> \u000A \u000A <p>The macros <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> and <tt><see topic=\"ACL2____UNMEMOIZE\">unmemoize</see></tt> now cause a warning rather \u000A than an error in ACL2 (and work as before in ACL2(h)).</p> \u000A \u000A <p>Terms are now parsed into <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules in a manner \u000A that removes <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> bindings both at the top level and in the conclusion \u000A (but still not in the hypotheses of the rule).  See <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>. \u000A Thanks to Jared Davis for requesting such an enhancement.</p> \u000A \u000A <p>Printing of numbers is now appropriately sensitive to the print radix; see \u000A <see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see>.  Thanks to Shilpi Goel for requesting this \u000A enhancement.</p> \u000A \u000A <p>The system function <v>explode-atom</v> no longer includes the radix \u000A indicator.  The new function <v>explode-atom+</v> may be used for that \u000A purpose.</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>Among the new features for system hackers are analogues of system function \u000A <v>simple-translate-and-eval</v> that do not return <tt><see topic=\"ACL2____STATE\">state</see></tt>.  (Thanks to \u000A David Rager for requesting this feature and helpful conversations on its \u000A implementation.)  This and other low-level changes are typically documented in \u000A comments in the corresponding release note event, which in this case is \u000A <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-0 ...)</v>.</p> \u000A \u000A <p>More built-in functions are now <see topic=\"ACL2____GUARD\">guard</see>-verified (and in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode).  Furthermore, a mechanism exists for marking yet more built-in \u000A functions as guard-verified based on <see topic=\"ACL2____BOOKS\">books</see> contributed by users; see \u000A Part II of <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/open-architecture/'>http://www.cs.utexas.edu/users/moore/acl2/open-architecture/</a>. \u000A The current state of that enterprise may be viewed by evaluating the constant \u000A <v>*system-verify-guards-alist*</v>, which associates a community book name with \u000A a list of functions.  When ACL2 is built in the normal way, each of those \u000A functions is marked as guard-verified when ACL2 is started up; but a special \u000A developer build can be used to check that the indicated book, together with \u000A its sub-books, proves that those functions are guard-verified.</p> \u000A \u000A <p>Metatheorems (see <see topic=\"ACL2____META\">meta</see>) may now have additional hypotheses, called \u000A ``meta-extract hypotheses'', that allow metafunctions to depend on the \u000A validity of certain terms extracted from the context or the logical <see topic=\"ACL2____WORLD\">world</see>.  See <see topic=\"ACL2____META-EXTRACT\">meta-extract</see>.  Thanks to Sol Swords for providing an \u000A initial implementation, together with very helpful discussions as well as a \u000A community book, <v>books/clause-processors/meta-extract-user.lisp</v>, that \u000A extends the power of meta-extract hypotheses.</p> \u000A \u000A <p>New utilities <tt><see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see></tt>, <tt><see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see></tt>, and <tt><see topic=\"ACL2____ORACLE-APPLY-RAW\">oracle-apply-raw</see></tt> call a function argument on specified arguments.  Thanks to \u000A Jared Davis for requesting this utility.</p> \u000A \u000A <p>A new utility makes it convenient to track time spent inside specified \u000A function calls or, more generally, during specified evaluation.  See <see topic=\"ACL2____TIME-TRACKER\">time-tracker</see>.</p> \u000A \u000A <p>New runic designators make it easy to refer to macro names when building \u000A theories.  Thus, for example, the object <v>(:i append)</v> may be used in \u000A theory expressions to designate the <see topic=\"ACL2____RUNE\">rune</see> <v>(:induction\u000A binary-append)</v>.  See <see topic=\"ACL2____THEORIES\">theories</see>.  Thanks to Jared Davis for a useful \u000A discussion leading to this enhancement.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFABSSTOBJ\">Defabsstobj</see></tt> <see topic=\"ACL2____EVENTS\">events</see> now take an optional <v>:congruent-to</v> \u000A keyword argument, much like <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>.  Thanks to Sol Swords for \u000A requesting this feature and for suggesting a very nice optimization that \u000A avoids the need to prove additional lemmas.</p> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____FLET\">Flet</see></tt> may now include <v>inline</v> and <v>notinline</v> declarations. \u000A Thanks to Jared Davis for requesting this feature.</p> \u000A \u000A <p>The utility <v>gc-verbose</v> controls printing of messages by the garbage \u000A collector, for certain host Lisps.  See <see topic=\"ACL2____GC-VERBOSE\">gc-verbose</see>.  Thanks to Shilpi \u000A Goel for requesting this utility.</p> \u000A \u000A <p>Added definitions of functions <tt><see topic=\"ACL2____NAT-LISTP\">nat-listp</see></tt> and <tt><see topic=\"ACL2____ACL2-NUMBER-LISTP\">ACL2-number-listp</see></tt>.  Thanks to Harsh Raju Chamarthi for requesting these \u000A additions.  Many community books had varying definitions of these functions; \u000A these additions guarantee that all books must agree on how these two functions \u000A are defined.  (Some community books have been changed in order that they \u000A remain certifiable, given these additions.)  Note that a few built-in \u000A <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules were modified in order to accommodate \u000A these additions, and the definition of <tt><see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see></tt> was modified to \u000A call <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> instead of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, like the other such definitions.</p> \u000A \u000A <p>See <see topic=\"ACL2____GET-COMMAND-SEQUENCE\">get-command-sequence</see> for a new utility that returns a list of \u000A <see topic=\"ACL2____COMMAND\">command</see>s between two given command descriptors.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>We obtained a substantial speedup &mdash; 13% observed for the regression \u000A suite, and 8% observed for the ACL2(h) regression suite &mdash; by tweaking \u000A the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> implementation to eliminate virtually all of its \u000A overhead when it is not in use (the default, which holds until <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt><v> t</v> is evaluated).  Thanks to David Rager for a conversation involving \u000A ACL2(p) performance statistics that suggested looking at changing <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> to boost performance.</p> \u000A \u000A <p>The heuristics for automatically expanding recursive function calls have \u000A been changed during proofs by induction.  Now, during induction, more terms \u000A that suggested the induction scheme are automatically expanded.  Thanks to \u000A David Rager for providing an example and having discussions with us that \u000A spurred us to develop this heuristic improvement.</p> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a soundness bug in <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> based on <tt><see topic=\"ACL2____GUARD\">guard</see></tt>s that \u000A violated single-threadedness restrictions.  Thanks to Sol Swords for bringing \u000A this bug to our attention and supplying a proof of <v>nil</v>, which we include \u000A as a comment in source file <v>ld.lisp</v>, in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-6-0 ...)</v>.  We \u000A also thank Sol for helpful discussions about <see topic=\"ACL2____GUARD\">guard</see>s of functions \u000A introduced by <v>defabsstobj</v>, which has led us to enhance the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>; see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>.</p> \u000A \u000A <p>Fixed a soundness bug in <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> based on interrupted updates \u000A of abstract stobjs.  As part of the fix a new keyword, <v>:PROTECT</v>, has been \u000A introduced for <v>defabsstobj</v> exports, along with a new top-level \u000A <v>defabsstobj</v> keyword, <v>:PROTECT-DEFAULT</v>; see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>.  We \u000A do some analysis that we expect will avoid the use of <v>:PROTECT</v> in many \u000A cases, which is fortunate since the use of <v>:PROTECT t</v> may cause a slight \u000A slowdown in (abstract) stobj updates.  Thanks to Sol Swords for bringing this \u000A bug to our attention and supplying a proof of <v>nil</v>, which we include as a \u000A comment in source file <v>other-events.lisp</v>, in the definition of function \u000A <v>set-absstobj-debug</v>.</p> \u000A \u000A <p>Fixed a raw Lisp error that occurred when tracing a <i>stobj</i> resize \u000A function, thanks to an error report from Warren Hunt, Marijn Heule, and Nathan \u000A Wetzler.</p> \u000A \u000A <p>Fixed a raw Lisp error that occurred for certain ill-formed signatures, as \u000A in the following example.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A           (((f (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; * :guard t)))\u000A           (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> x))))\u000A\u000A***********************************************\u000A************ ABORTING from raw Lisp ***********\u000AError:  value (F (<see topic=\"COMMON-LISP_____A2\">*</see>) =&gt; * :GUARD T) is not of the expected type SYMBOL.\u000A***********************************************</code> \u000A \u000A <p>The notion of ``error triple'' (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) had been \u000A implemented ambiguously, with the result that for a <see topic=\"ACL2____STOBJ\">stobj</see>, <v>st</v>, the \u000A result of evaluating the following two forms was the same: <v>(<see topic=\"ACL2____MV\">mv</see> nil st\u000A state)</v> and <v>(<see topic=\"ACL2____MV\">mv</see> t st state)</v>.  Of course, these are just examples; in \u000A general, a result of <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> was sometimes treated as an error \u000A triple even when <v>val</v> is a <see topic=\"ACL2____STOBJ\">stobj</see>.  Now, <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> is \u000A an error triple only when <v>erp</v> and <v>val</v> are ordinary (non-<see topic=\"ACL2____STOBJ\">stobj</see>) values.  Thanks to Warren Hunt and Marijn Heule for bringing this \u000A problem to our attention.</p> \u000A \u000A <p>The ``with-error-trace'' utility, <tt><see topic=\"ACL2____WET\">wet</see></tt>, now works in the non-error \u000A case when given a form that returns multiple values.  (Note however that \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> will be printed as <v>REPLACED-STATE</v>; and similarly, a \u000A user-defined <see topic=\"ACL2____STOBJ\">stobj</see>, say <v>ST</v>, will be printed as \u000A <v>REPLACED-ST</v>.)</p> \u000A \u000A <p>Some possible error messages for <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> have been fixed that \u000A had been ill-formed.  Thanks to Sol Swords for bringing this bug to our \u000A attention.</p> \u000A \u000A <p>Fixed a bug that sometimes caused the times displayed in the summary for \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to be smaller than the actual times.</p> \u000A \u000A <p>Fixed a bug in the <see topic=\"ACL2____GUARD\">guard</see>s to system functions <v>fmt-char</v> and \u000A <v>fmt-var</v>, which are no longer <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode, guard-verified \u000A functions.</p> \u000A \u000A <p>(GCL only) Fixed a bug present in Gnu Common Lisp for <v>#u</v> (see <see topic=\"ACL2____SHARP-U-READER\">sharp-u-reader</see>).</p> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL</b></p> \u000A \u000A <p>The <see topic=\"ACL2____STATE\">state</see> global variable <v>'distributed-books-dir</v> has been \u000A renamed <v>'system-books-dir</v>.  On a related note, the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A now refers to ``community books'' rather than ``distributed books'', and there \u000A is a corresponding new documentation topic; see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p> \u000A \u000A <p>Fixed a bug in the implementation of <tt><see topic=\"ACL2____WET\">wet</see></tt> (which is actually in the \u000A community book <v>books/misc/wet.lisp</v>).</p> \u000A \u000A <p>A directory, <v>interface/</v>, is no longer part of the ACL2 distribution. \u000A Rather, it is a subdirectory of the ACL2 community books.  Thus, if you fetch \u000A those books in the usual way (see the installation instructions on the ACL2 \u000A home page), you will find a directory <v>books/interface/</v>.  Subdirectory \u000A <v>emacs/</v> of that <v>interface</v> directory provides Emacs support for <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>s as well an <v>acl2-mode</v>.  This change has been reflected in \u000A ACL2 file <v>emacs/emacs-acl2.el</v>, so users will probably not be impacted if \u000A they load that file into Emacs.</p> \u000A \u000A <p>The community books file <v>books/Makefile-generic</v> now causes, by \u000A default, a backtrace to be printed when there is a raw Lisp error.</p> \u000A \u000A <p>Some changes have been made to how regressions are run, i.e., to how the \u000A community books are certified.  (1) The standard regression now includes \u000A community books directory <v>books/centaur</v>.  To skip these (for example, a \u000A Windows system has encountered difficulty with them even after installing \u000A Perl), include <v>ACL2_CENTAUR=skip</v> with your `make' command.  (2) A new \u000A `make' (or environment) variable, <v>ACL2_JOBS</v>, specifies the number of \u000A parallel jobs to run, serving as a replacement for the <v>-j</v> argument of \u000A `make' that works for all community books, including those under directory \u000A <v>centaur</v>; see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>.  (3) It is no longer \u000A necessary to do an ACL2(h) regression in order to build a copy of the \u000A documentation generated by Jared Davis's xdoc utility at \u000A <v>books/xdoc-impl/manual/preview.html</v>; a vanilla ACL2 regression will build \u000A this manual.  (4) It is no longer necessary to set the <v>ACL2</v> environment \u000A variable for ACL2(h) regressions if you want to use the executable \u000A <v>saved_acl2h</v> in the ACL2 sources directory.</p> \u000A \u000A <p>The ACL2 home page now has a search utility for documentation and books. \u000A Thanks to Shilpi Goel and David Rager for feedback on a preliminary version of \u000A this utility.</p> \u000A \u000A <p>(only for SBCL with 64-bit ACL2(h)) The value of SBCL command line option \u000A <v>--dynamic-space-size</v> for ACL2(h) on 64-bit platforms has been increased \u000A from 2000 to 16000 (as explained in a comment in the ACL2 source definition of \u000A <v>*sbcl-dynamic-space-size*</v>).</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p><b>EXPERIMENTAL/ALTERNATE VERSIONS</b></p> \u000A \u000A <p>Among the enhancements for ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>) are the following.</p> \u000A \u000A <blockquote> \u000A \u000A <p>Thanks to Ruben Gamboa for his helpful role in making the following \u000A improvements made with Ruben Gamboa in support for non-standard analysis in \u000A ACL2(r).</p> \u000A \u000A <p>Constrained functions can now be introduce as non-classical.  See <see topic=\"ACL2____SIGNATURE\">signature</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____DEFUN-SK\">Defun-sk</see></tt> now takes a new keyword argument, <v>:CLASSICALP</v>, that \u000A determines whether or not the named function is classical.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>.</p> \u000A \u000A <p>Incorporated a bug fix from Ruben Gamboa for <tt><see topic=\"COMMON-LISP____CEILING\">ceiling</see></tt>.  The default \u000A (for `bad' arguments) had been 1, but now we follow normal ACL2 practice by \u000A returning 0 in that case.</p></blockquote> \u000A \u000A <p>Among the enhancements for the HONS extension (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>) are the following.</p> \u000A \u000A <blockquote> \u000A \u000A <p>Macros <tt><see topic=\"ACL2____WITH-FAST-ALIST\">with-fast-alist</see></tt>, <tt><see topic=\"ACL2____WITH-STOLEN-ALIST\">with-stolen-alist</see></tt>, and <tt><see topic=\"ACL2____FAST-ALIST-FREE-ON-EXIT\">fast-alist-free-on-exit</see></tt> are now defined in ACL2(h), rather than being defined \u000A in the community book <v>&quot;books/centaur/misc/hons-extra.lisp&quot;</v>.  Thanks to \u000A Jared Davis and Sol Swords for donating this code, and thanks to Jared for \u000A helpful discussions leading to this change.</p></blockquote> \u000A \u000A <p>Among the enhancements for ACL2(p) (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>) are the \u000A following.  We thank David Rager for his work in developing ACL2(p) and for \u000A his helpful role in these improvements.</p> \u000A \u000A <blockquote> \u000A \u000A <p>A bug has been fixed that could leave one in a <see topic=\"ACL2____WORMHOLE\">wormhole</see>, awaiting \u000A input, after an error, such as an error in an <v>:in-theory</v> hint during a \u000A proof.  Thanks to Shilpi Goel for bringing this bug to our attention.</p> \u000A \u000A <p>A key checkpoint for a given goal is now printed only once.  Previously, if \u000A a key checkpoint led to more than one goal pushed for proof by induction, the \u000A key checkpoint would be printed once for each such goal during the proof, and \u000A also once for each such goal in the summary at the end.</p></blockquote>"],

 "ACL2____NOTE-5-0":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Below we roughly organize the changes since Version 4.3 into the following \u000A categories of changes: existing features, new features, heuristic \u000A improvements, bug fixes, changes at the system level and to distributed books, \u000A Emacs support, and experimental versions.  Each change is described in just \u000A one category, though of course many changes could be placed in more than one \u000A category.</p> \u000A \u000A <p>NOTE: ACL2 is now distributed under Version 2 of the GNU General Public \u000A License.  [Added later: The license has changed since Version_5.0.  See \u000A LICENSE.]  Formerly, any later version had been acceptable.  Moreover, books \u000A are no longer distributed from a University of Texas website, but rather, from \u000A Google Code at the <a \u000A href='http://code.google.com/p/acl2-books/downloads/'>ACL2 Books Downloads</a> \u000A page.</p> \u000A \u000A <p><b>CHANGES TO EXISTING FEATURES</b></p> \u000A \u000A <p>A fatal error now occurs if environment variable <v>ACL2_CUSTOMIZATION</v> \u000A has a value other than <v>NONE</v> or the empty string, but is not the name of \u000A an existing file.  Thanks to Harsh Raju Chamarthi for requesting such a \u000A change.</p> \u000A \u000A <p>Functions <v>read-acl2-oracle</v> (and <v>read-acl2-oracle@par</v>), \u000A <v>read-run-time</v>, and <v>main-timer</v> are no longer untouchable (see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>).</p> \u000A \u000A <p>We now avoid certain duplicate conjuncts in the <see topic=\"ACL2____CONSTRAINT\">constraint</see> stored \u000A for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  For example, the constraint stored for \u000A the following event formerly included <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (FOOP (<see topic=\"COMMON-LISP____CONS\">CONS</see> X Y)) (FOOP Y))</v> \u000A and <v>(<see topic=\"ACL2____BOOLEANP\">BOOLEANP</see> (FOOP X))</v> twice each, but no more.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((foop (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foop (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foop-constraints\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (foop x))\u000A        (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foop (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) (foop y)))\u000A   :rule-classes\u000A   ((:type-prescription :corollary (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (foop x)))\u000A    (:rewrite :corollary (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foop (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) (foop y))))))</code> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> for a constrained function (see <see topic=\"ACL2____SIGNATURE\">signature</see>) \u000A may now mention function symbols introduced in the same <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A event that introduces that function.  Thanks to Nathan Wetzler for a helpful \u000A discussion leading to this improvement.</p> \u000A \u000A <p>The test for redundancy (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> has been improved in cases involving redefinition \u000A (see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>).  Thanks to Jared Davis for providing the \u000A following example, which illustrates the problem.</p> \u000A \u000A <code>(<see topic=\"ACL2____REDEF_12\">redef!</see>)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A    (<see topic=\"COMMON-LISP_____B2\">+</see> 3 x)))\u000A\u000A(g 0) ; 3, as expected\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A    (<see topic=\"COMMON-LISP_____B2\">+</see> 4 x)))\u000A\u000A(g 0) ; 4, as expected\u000A\u000A; Unfortunately, the following was flagged as redundant because it agreed\u000A; with the first encapsulate above.  That has been fixed; now, it is\u000A; recognized as not being redundant.\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A    (<see topic=\"COMMON-LISP_____B2\">+</see> 3 x)))</code> \u000A \u000A <p>The test for redundancy of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> events has \u000A been improved in the case that redefinition is active.  In that case, \u000A redundancy now additionally requires that the ``translated'' body is \u000A unchanged, i.e., even after expanding macro calls and replacing constants \u000A (defined by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>) with their values.  Thanks to Sol Swords for \u000A requesting this enhancement, and to Jared Davis for pointing out a bug in a \u000A preliminary change.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>, in particular the ``Note \u000A About Unfortunate Redundancies''.  Note that this additional requirement was \u000A already in force for redundancy of <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> events.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____DEFMACRO-LAST\">defmacro-last</see></tt> and the <see topic=\"ACL2____TABLE\">table</see> <tt><see topic=\"ACL2____RETURN-LAST-TABLE\">return-last-table</see></tt> have been modified so that when they give special treatment \u000A to a macro <v>mac</v> and its raw Lisp counterpart <v>mac-raw</v>, a call \u000A <v>(<see topic=\"ACL2____RETURN-LAST\">return-last</see> 'mac-raw ...)</v>  can be made illegal when encountered directly \u000A in the top level loop, as opposed to inside a function body.  See <see topic=\"ACL2____RETURN-LAST\">return-last</see>.  Thanks to Harsh Raju Chamarthi for showing us an example that \u000A led us to make this improvement.</p> \u000A \u000A <p>We removed a barrier to admitting function definitions, as we explain using \u000A the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (m state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> m)\u000A      (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'last-m m state)))\u000A        (foo (<see topic=\"COMMON-LISP____CDR\">cdr</see> m) state))\u000A    state))</code> \u000A \u000A <p>Previously, ACL2 complained that it could not determine the outputs of the \u000A <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> form, as is necessary in order to ensure that <tt><see topic=\"ACL2____STATE\">state</see></tt> is \u000A returned by it.  ACL2 now works harder to solve this problem as well as the \u000A analogous problem for <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> and, more generally for <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>.  (The main idea is to reverse the order of processing the \u000A <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> branches if necessary.)  We thank Sol Swords for contributing a \u000A version of the above example and requesting this improvement.</p> \u000A \u000A <p>It is no longer the case that <tt><see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see></tt> causes a Lisp break \u000A when encountering an error during translation of user input into internal \u000A (translated) form (see <see topic=\"ACL2____TERM\">term</see>).  The reason is that an improvement to the \u000A translation process, specifically the one described in the preceding \u000A paragraph, allows certain backtracking from ``errors'', which are intended to \u000A be silent rather than causing breaks into raw Lisp.  Thanks to Jared Davis for \u000A sending an example leading to this change.</p> \u000A \u000A <p>(CCL and SBCL only) When the host Lisp is CCL or SBCL, then since all \u000A functions are compiled, a <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command will no longer load the \u000A newly-compiled file (and similarly for <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> with argument \u000A <v>:load-compiled-file :comp</v>).</p> \u000A \u000A <p><tt><see topic=\"ACL2____SET-WRITE-ACL2X\">Set-write-acl2x</see></tt> now returns an error triple and can take more \u000A values, some of which automatically allow including uncertified books when \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> is called with argument :acl2x t.</p> \u000A \u000A <p>The environment variable <v>COMPILE_FLG</v> has been renamed \u000A <v>ACL2_COMPILE_FLG</v>; see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>.</p> \u000A \u000A <p>The macros <tt><see topic=\"ACL2____DEFTHMD\">defthmd</see></tt> and <tt><see topic=\"ACL2____DEFUND\">defund</see></tt> no longer return an error \u000A triple with value <v>:SKIPPED</v> when proofs are being skipped.  Rather, the \u000A value returned is the same as would be returned on success when proofs are not \u000A skipped.</p> \u000A \u000A <p>For those who use <tt><see topic=\"ACL2____SET-WRITE-ACL2X\">set-write-ACL2x</see></tt>: now, when <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A is called without a <v>:ttagsx</v> argument supplied, then the value of \u000A <v>:ttagsx</v> defaults to the (explicit or default) value of the <v>:ttags</v> \u000A argument.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt> <see topic=\"ACL2____COMMAND\">command</see>s can now accept \u000A <see topic=\"ACL2____TERM\">term</see>s that had previously been rejected.  For example, the command \u000A <v>:pl (<see topic=\"COMMON-LISP____MEMBER\">member</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))</v> had caused an error, but now it works as one \u000A might reasonably expect, treating <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> as <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> (see \u000A <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for relevant background).  Thanks to Jared Davis for \u000A reporting this problem by sending the above example.</p> \u000A \u000A <p>We have eliminated some hypotheses in built-in <see topic=\"ACL2____REWRITE\">rewrite</see> rules \u000A <v>characterp-nth</v> and <v>ordered-symbol-alistp-delete-assoc-eq</v>.</p> \u000A \u000A <p>Added the symbols <tt><see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see></tt>, <tt><see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see></tt>, and <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt> to <v>*acl2-exports*</v>.</p> \u000A \u000A <p>Added to the <see topic=\"ACL2____GUARD\">guard</see>s of <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt> and <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt> the requirement that the second argument must be a \u000A Boolean.  Thanks to Jared Davis for sending an example that led to this \u000A change.</p> \u000A \u000A <p>The built-in function <v>string-for-tilde-@-clause-id-phrase</v> has been put \u000A into <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode and had its guards verified, as have some \u000A subsidiary functions.  A few new rules have been added in support of this \u000A work; search for <v>string-for-tilde-@-clause-id-phrase</v> in ACL2 source file \u000A <v>boot-strap-pass-2.lisp</v> if interested.  Thanks to David Rager for \u000A contributing an initial version of this improvement.</p> \u000A \u000A <p>All trust tags are now in the <see topic=\"COMMON-LISP____KEYWORD\">keyword</see> package.  The <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> \u000A event may still take a symbol in an arbitrary package, but the trust tag \u000A created will be in the keyword package (with the same <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> as \u000A the symbol provided).  Similarly, non-<v>nil</v> symbols occurring in the \u000A <v>:ttags</v> argument of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> \u000A command will be converted to corresponding keywords.  See <see topic=\"ACL2____DEFTTAG\">defttag</see>.</p> \u000A \u000A <p>There have been several changes to <see topic=\"ACL2____GAG-MODE\">gag-mode</see>.  It is now is initially \u000A set to <v>:goals</v>, suppressing most proof commentary other than key \u000A checkpoints; see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>.  (As before, see <see topic=\"ACL2____PSO\">pso</see> for how to \u000A recover the proof output.)  Also, top-level induction schemes are once again \u000A printed when gag-mode is on, though these as well as printing of guard \u000A conjectures can be abbreviated (``eviscerated'') with a new <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>, in particular the discussion there \u000A of <v>:GAG-MODE</v>.  Finally, the commentary printed within <see topic=\"ACL2____GAG-MODE\">gag-mode</see> \u000A that is related to <see topic=\"ACL2____FORCING-ROUND\">forcing-round</see>s is now less verbose.  Thanks to Dave \u000A Greve and David Rager for discussions leading to the change in the printing of \u000A induction schemes under gag-mode; thanks to Warren Hunt for an email that led \u000A us to similar handling for printing of guard conjectures; and thanks to Robert \u000A Krug for a suggestion that led us to restore, in abbreviated form, important \u000A information about the sources of forcing round goals.</p> \u000A \u000A <p>An error now occurs if <tt><see topic=\"ACL2____LD\">ld</see></tt> is called while loading a compiled book. \u000A See <see topic=\"ACL2____CALLING-LD-IN-BAD-CONTEXTS\">calling-ld-in-bad-contexts</see>.  Thanks to David Rager for reporting a \u000A low-level assertion failure that led us to make this change.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> interactive loop is more robust: most errors will \u000A leave you in that loop, rather than kicking you out of the proof-checker and \u000A thus back to the main ACL2 read-eval-print loop.  Thanks to David Hardin for \u000A suggesting this improvement in the case of errors arising from extra right \u000A parentheses.</p> \u000A \u000A <p>The summary at the end of a proof now prints the following note when \u000A appropriate:</p> \u000A \u000A <code>[NOTE: A goal of NIL was generated.  See :DOC nil-goal.]</code> \u000A \u000A <p>See <see topic=\"ACL2____NIL-GOAL\">nil-goal</see>.</p> \u000A \u000A <p>Improved <tt><see topic=\"ACL2____DMR\">dmr</see></tt> to show the function being called in the case of \u000A explicit evaluation: ``<v>(EV-FNCALL function-being-called)</v>''.</p> \u000A \u000A <p>It is now permitted to bind any number of <see topic=\"ACL2____STOBJS\">stobjs</see> to themselves in \u000A the bindings of a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expression.  But if any stobj is bound to other \u000A than itself in <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> bindings, then there still must be only one binding \u000A in that <v>LET</v> expression.  The analogous relaxation holds for <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> expressions.  Thanks to Sol Swords for requesting such a change, which \u000A was needed for some code generated by macro calls.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt> now returns without error; See <see topic=\"ACL2____TOP-LEVEL\">top-level</see>.  Formerly, this macro always returned an error triple <v>(<see topic=\"ACL2____MV\">mv</see> t\u000A .. state)</v>, which meant that normal calls of <tt><see topic=\"ACL2____LD\">ld</see></tt> would stop after \u000A encountering a call of <v>top-level</v>.  Thanks to Jared Davis for bringing \u000A this issue to our attention.</p> \u000A \u000A <p>It is no longer the case that when you specify <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword \u000A <v>:non-executable t</v> in a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form rather than using <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt>, then the form of the body need match only the shape <v>(<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A (throw-nonexec-error ... ...) ...)</v>.  We now require that the body of the \u000A definition of a function symbol, <v>fn</v>, with formals <v>(x1 ... xk)</v>, be of \u000A the form <v>(<see topic=\"ACL2____PROG2_42\">prog2$</see> (throw-nonexec-error 'fn (<see topic=\"COMMON-LISP____LIST\">list</see> x1 ... xk)) ...)</v>.  This \u000A fixes the following odd behavior, which could be considered a bug.  Consider a \u000A book that contains the following two events.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :non-executable t :mode :logic))\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (throw-nonexec-error 'bar (<see topic=\"COMMON-LISP____LIST\">list</see> x))\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 x)))\u000A(<see topic=\"ACL2____DEFN\">defn</see> h (x)\u000A  (foo x))</code> \u000A \u000A <p>After certifying this book and then including it in a new session, the \u000A behavior occurred that is displayed below; notice the mention of <v>BAR</v>. \u000A However, if the two forms were submitted directly in the loop, then the error \u000A message had mentioned <v>FOO</v> instead of <v>BAR</v>.  This discrepancy has been \u000A eliminated, by rejecting the proposed definition of <v>foo</v> because the name \u000A in the first argument of <v>throw-nonexec-error</v> was <v>'bar</v> where now it \u000A must be <v>'foo</v>.</p> \u000A \u000A <code>ACL2 !&gt;(h 3)\u000A\u000AACL2 Error in TOP-LEVEL:  ACL2 cannot ev the call of undefined function\u000ABAR on argument list:\u000A\u000A(3)\u000A\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>A tautology checker used in the ACL2 sources (function <v>if-tautologyp</v>) \u000A has been limited somewhat in the effort it makes to recognize a tautology. \u000A While we expect it to be rare for the effect of this change to be noticeable, \u000A we thank Sol Swords for sending us an example that motivated this change: a \u000A <see topic=\"ACL2____GUARD\">guard</see> verification that took about 5 seconds in Version_4.3 now takes, \u000A on the same machine, about 0.07 seconds.</p> \u000A \u000A <p>The behavior of backquote (<v>`</v>) has been changed slightly to be \u000A compatible with its behavior in raw Lisp.  The change is to allow the use of \u000A comma-atsign (<v>,@</v>) at the end of a list, as in the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x 3) (y 2) (z 7)) `(,x ,y ,@z))</code> \u000A \u000A <p>Formerly, evaluation of this form had caused a guard violation in the ACL2 \u000A loop unless guard-checking was off (i.e., <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> was \u000A invoked with <v>nil</v> or <v>:none</v>), in which case it returned <v>(3 2)</v>. \u000A But we observed evaluation of this form to return <v>(3 2 . 7)</v> in every host \u000A Lisp on which ACL2 runs (Allegro CL, CCL, CLISP, CMUCL, GCL, LispWorks, and \u000A SBCL).  Now, ACL2 behaves like these Lisps.</p> \u000A \u000A <p>A call of the <tt><see topic=\"ACL2____THEORY\">theory</see></tt> macro had previously returned <v>nil</v> when \u000A applied to other than the name of name of a previously executed <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event.  Now, a hard error occurs.</p> \u000A \u000A <p>The <see topic=\"ACL2____TABLE\">table</see> <v>binop-table</v> has been replaced by the table <tt><see topic=\"ACL2____UNTRANS-TABLE\">untrans-table</see></tt>.  However, <tt><see topic=\"ACL2____ADD-BINOP\">add-binop</see></tt> and <tt><see topic=\"ACL2____REMOVE-BINOP\">remove-binop</see></tt> continue \u000A to have the same effect as before.  See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see>, which is a new \u000A feature discussed below.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____BOOLEANP\">booleanp</see></tt> is now defined using <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> instead of \u000A <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, which may increase its efficiency.  Thanks to Jared Davis for \u000A this change.</p> \u000A \u000A <p>For pairs <v>(key . val)</v> in the <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>, there had \u000A been a requirement that <v>val</v> is a known function symbol.  Now, it only \u000A needs to be a symbol.  (This change was made to support the new feature, \u000A <tt><see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see></tt>, described elsewhere in these release notes.)</p> \u000A \u000A <p><b>NEW FEATURES</b></p> \u000A \u000A <p>A new ``tau system'' provides a kind of ``type checker.''  See <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>.  Thanks to Dave Greve for supplying a motivating example (on \u000A which this system can provide significant speedup), and to Sol Swords for \u000A sending a very helpful bug report on a preliminary implementation.</p> \u000A \u000A <p>Users may now arrange for additional summary information to be printed at \u000A the end of <see topic=\"ACL2____EVENTS\">events</see>.  [Note added at Version_6.1: Formerly we pointed \u000A here to <v>print-summary-user</v>, but now, see <see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see>; also \u000A see <see topic=\"ACL2____NOTE-6-1\">note-6-1</see>].  Thanks to Harsh Raju Chamarthi for requesting this \u000A feature and participating in a design discussion.</p> \u000A \u000A <p>A new, advanced <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command, <v>geneqv</v>, shows the \u000A generated equivalence relation at the current subterm.  Thanks to Dave Greve \u000A for an inquiry leading to this enhancement.</p> \u000A \u000A <p>A new reader macro, <v>#u</v>, permits the use of underscore characters in a \u000A number.  See <see topic=\"ACL2____SHARP-U-READER\">sharp-u-reader</see>.  Thanks to Jared Davis for requesting this \u000A capability.</p> \u000A \u000A <p>New <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>pl</v> and <v>pr</v> provide interfaces to \u000A the ACL2 commands <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt>, respectively.  These \u000A can be useful if you want to see trivially-proved hypotheses, as now clarified \u000A in the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> documentation for its <v>show-rewrites</v> command. \u000A See <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  Thanks to Pete Manolios for suggesting \u000A such clarification and capability.</p> \u000A \u000A <p>It is now legal to call <see topic=\"ACL2____NON-EXECUTABLE\">non-executable</see> functions without the usual \u000A <see topic=\"ACL2____SIGNATURE\">signature</see> restrictions imposed on executable code.  For example, the \u000A third event below was not admissible, but now it is.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> foo fld)\u000A(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> id (x)\u000A  x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (foo)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs foo :verify-guards nil))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 (id foo)))</code> \u000A \u000A <p>Thanks to Jared Davis for requesting this enhancement, in particular for \u000A calling non-executable functions in the <v>:logic</v> part of an <tt><see topic=\"ACL2____MBE\">mbe</see></tt> \u000A call.  Here is Jared's example, which is admissible now but formerly was \u000A not.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> foo (fld))\u000A(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> my-identity (x) x)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-fld (foo)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs foo))\u000A  (<see topic=\"ACL2____MBE\">mbe</see> :logic (my-identity foo)\u000A       :exec (<see topic=\"COMMON-LISP____LET\">let</see> ((val (fld foo)))\u000A               (update-fld val foo))))</code> \u000A \u000A <p>A new macro, <tt><see topic=\"ACL2____NON-EXEC\">non-exec</see></tt>, allows the use of non-executable code, for \u000A example inside ordinary function definitions.  Thanks to Sol Swords for \u000A requesting this enhancement.</p> \u000A \u000A <p>A new ``provisional certification'' process is supported that can allow \u000A <see topic=\"ACL2____BOOKS\">books</see> to be certified before their included sub-books have been \u000A certified, thus allowing for potentially much greater `make'-level \u000A parallelism.  See <see topic=\"ACL2____PROVISIONAL-CERTIFICATION\">provisional-certification</see>.  Thanks to Jared Davis for \u000A requesting this feature and for helpful discussions, based in part on \u000A rudimentary provisional certification schemes that he developed first at \u000A Rockwell Collins and later for his `Milawa' project.  Also, thanks to Jared \u000A and to Sol Swords for testing this feature and for providing a fix for a bug \u000A in a preliminary implementation, and thanks to Sol for providing performance \u000A feedback and a crucial suggestion that led to an improved implementation.</p> \u000A \u000A <p>Event summaries now show the names of events that were mentioned in <see topic=\"ACL2____HINTS\">hints</see> of type <v>:use</v>, <v>:by</v>, or <v>:clause-processor</v>.  See <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see>.  Thanks to Francisco J. Martin Mateos for \u000A requesting such an enhancement (actually thanks to the community, as his \u000A request is the most recent but this has come up from time to time before).</p> \u000A \u000A <p>ACL2 now stores a data structure representing the relation ``Event A is \u000A used in the proof of Event B.''  See <see topic=\"ACL2____DEAD-EVENTS\">dead-events</see>, which explains this \u000A data structure and mentions one application: to identify dead code and unused \u000A theorems.  Thanks to Shilpi Goel for requesting such a feature and for helpful \u000A feedback.</p> \u000A \u000A <p>A new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic provides a guide to programming with \u000A state; see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.  Thanks to Sarah Weissman for \u000A suggesting that such a guide might be useful, and to David Rager for helpful \u000A feedback on a preliminary version.  There also has been some corresponding \u000A reorganization of the documentation as well as creation of additional \u000A documentation (e.g., see <see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>).  Now, most built-in \u000A functions and macros commonly used in programs (as opposed to <see topic=\"ACL2____EVENTS\">events</see> \u000A like <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, for example) are subtopics of a new topic &mdash; see \u000A <see topic=\"ACL2____ACL2-BUILT-INS\">ACL2-built-ins</see> &mdash; which is a subtopic of <see topic=\"ACL2____PROGRAMMING\">programming</see>, a \u000A topic that in turn has considerably fewer direct subtopics than before.</p> \u000A \u000A <p>It is now possible to bind extra variables in a <v>:USE</v> hint, thus \u000A avoiding the error message: ``The formula you wish to instantiate, ..., \u000A mentions only the variable(s) ...''.  See <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>, in particular \u000A the discussion of keyword <v>:extra-bindings-ok</v>.  Thanks to Sol Swords for \u000A requesting such an enhancement.</p> \u000A \u000A <p>The function <v>read-object-suppress</v> is like <v>read-object</v> except that \u000A it avoids errors and discards the value read.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p>A <see topic=\"ACL2____STOBJ\">stobj</see> may now be passed as an argument where another stobj is \u000A expected if the two are ``congruent''.  See <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>, in particular, \u000A its discussion of the new <v>:congruent-to</v> keyword of <v>defstobj</v>.  Thanks \u000A to Sol Swords for requesting this enhancement and for useful discussions \u000A contributing to its design.</p> \u000A \u000A <p>A new top-level utility has been provided that shows the assembly language \u000A for a defined function symbol; see <see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see>.  Thanks to Jared Davis \u000A for requesting such a utility and to Shilpi Goel for pointing out an \u000A inconvenience with the initial implementation.  Note that it uses the \u000A distributed book <v>books/misc/disassemble.lisp</v>, which users are welcome to \u000A modify (see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>).</p> \u000A \u000A <p>The macro <v>set-accumulated-persistence</v> is an alias for <tt><see topic=\"ACL2____ACCUMULATED-PERSISTENCE\">accumulated-persistence</see></tt>.  Thanks to Robert Krug for suggesting this \u000A addition.</p> \u000A \u000A <p>A new <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic lists lesser-known and advanced ACL2 \u000A features, intended for those with prior ACL2 experience who wish to extend \u000A their knowledge of ACL2 capabilities.  See <see topic=\"ACL2____ADVANCED-FEATURES\">advanced-features</see>.  Thanks \u000A to Warren Hunt and Anna Slobodova for requesting such information.</p> \u000A \u000A <p>A new macro, <tt><see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see></tt>, provides a variant of <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> such that the resulting theory is the same at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A time as it was at <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> time.  Thanks to Robert Krug for \u000A helpful discussions on this new feature and for updating his \u000A <v>books/arithmetic-5/</v> distributed books to use this feature.</p> \u000A \u000A <p>A new event, <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>, provides a new way to introduce \u000A single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see> and see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>).  These \u000A so-called ``abstract <see topic=\"ACL2____STOBJ\">stobj</see>s'' permit user-provided logical definitions \u000A for primitive operations on stobjs, for example using an alist-based \u000A representation instead of a list-based representation for array fields. \u000A Moreover, the proof obligations guarantee that the recognizer is preserved; \u000A hence the implementation avoids executing the recognizer, which may be an \u000A arbitrarily complex invariant that otherwise would be an expensive part of \u000A <see topic=\"ACL2____GUARD\">guard</see> checks.  Thanks to Warren Hunt for a request leading us to design \u000A and implement this new feature, and thanks to Rob Sumners for a request \u000A leading us to implement a related utility, <tt><see topic=\"ACL2____DEFABSSTOBJ-MISSING-EVENTS\">defabsstobj-missing-events</see></tt>. \u000A See <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>.  Also thanks to Sol Swords for sending an example \u000A exhibiting a bug in the initial implementation, which has been fixed.</p> \u000A \u000A <p>A new command, <v>:psof &lt;filename&gt;</v>, is like <v>:pso</v> but directs proof \u000A replay output to the specified file.  For large proofs, <v>:</v><tt><see topic=\"ACL2____PSOF\">psof</see></tt> may \u000A complete much more quickly than <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt>.  see <see topic=\"ACL2____PSOF\">psof</see>.  More \u000A generally, a new utility, <tt><see topic=\"ACL2____WOF\">wof</see></tt> (an acronym for ``With Output File''), \u000A directs standard output and proofs output to a file; see <see topic=\"ACL2____WOF\">wof</see>.</p> \u000A \u000A <p>The new macro <tt><see topic=\"ACL2____DEFND\">defnd</see></tt> defines a function with <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> \u000A <v>t</v> and <see topic=\"ACL2____DISABLE\">disable</see>s that function, in analogy to how <tt><see topic=\"ACL2____DEFUND\">defund</see></tt> \u000A defines with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and then <see topic=\"ACL2____DISABLE\">disable</see>s.  Thanks to Shilpi Goel \u000A for requesting this feature.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____PL2\">pl2</see></tt> command now shows <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules; and a \u000A new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command, <v>show-linears</v> (equivalently, <v>sls</v>), \u000A is an analogue of the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <v>show-rewrites</v> (<v>sr</v>) \u000A command, but for <see topic=\"ACL2____LINEAR\">linear</see> rules.  Thanks to Shilpi Goel for requesting \u000A this new proof-checker command.  Finally, a corresponding new proof-checker \u000A command, <v>apply-linear</v> (<v>al</v>), is an analogue of the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <v>rewrite</v> (<v>r</v>) command, but for <see topic=\"ACL2____LINEAR\">linear</see> rules.</p> \u000A \u000A <p>The macros <tt><see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see></tt> and <tt><see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see></tt> replace macros \u000A <tt><see topic=\"ACL2____ADD-BINOP\">add-binop</see></tt> and <tt><see topic=\"ACL2____REMOVE-BINOP\">remove-binop</see></tt>, respectively, though the latter \u000A continue to work.  The new macros allow you to decide whether or not to \u000A display calls of binary macros as flat calls for right-associated arguments, \u000A e.g., <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)</v> rather than <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))</v>.  See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see>.</p> \u000A \u000A <p>It is now possible to request that the host Lisp compiler inline calls of \u000A specified functions, or to direct that the host Lisp compiler not inline such \u000A calls.  See <see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> and see <see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see>.  We thank \u000A Jared Davis for several extensive, relevant conversations, and for finding a \u000A bug in a preliminary implementation.  We also thank others who have engaged in \u000A discussions with us about inlining for ACL2; besides Jared Davis, we recall \u000A such conversations with Rob Sumners, Dave Greve, and Shilpi Goel.</p> \u000A \u000A <p><b>HEURISTIC IMPROVEMENTS</b></p> \u000A \u000A <p>Reading of ACL2 <tt><see topic=\"ACL2____ARRAYS\">arrays</see></tt> (see <see topic=\"ACL2____AREF1\">aref1</see>, see <see topic=\"ACL2____AREF2\">aref2</see>) has \u000A been made more efficient (as tested with CCL as the host Lisp) in the case of \u000A consecutive repeated reads of the same named array.  Thanks to Jared Davis and \u000A Sol Swords for contributing this improvement.</p> \u000A \u000A <p>Slightly modified the induction schemes stored, so that calls of so-called \u000A ``guard-holders'' (such as <tt><see topic=\"ACL2____MBE\">mbe</see></tt> and <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> &mdash; indeed, any \u000A call of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt> &mdash; and <tt><see topic=\"COMMON-LISP____THE\">the</see></tt>) are expanded away.  In \u000A particular, calls of equality variants such as <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> are treated as \u000A their corresponding function calls, e.g., <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>; see <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see>.  Guard-holders are also now expanded away before storing \u000A <see topic=\"ACL2____CONSTRAINT\">constraint</see>s for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, which can sometimes \u000A result in simpler constraints.</p> \u000A \u000A <p>Improved the performance of <tt><see topic=\"ACL2____DMR\">dmr</see></tt> (technical note: by modifying raw \u000A Lisp code for function <v>dmr-flush</v>, replacing <v>finish-output</v> by \u000A <v>force-output</v>).</p> \u000A \u000A <p>We now avoid certain rewriting loops.  A long comment about this change, \u000A including an example of a loop that no longer occurs, may be found in source \u000A function <v>expand-permission-result</v>.</p> \u000A \u000A <p>Slightly strengthened <see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning at the level of literals \u000A (i.e., top-level hypotheses and conclusions).  See the comment in ACL2 source \u000A function <v>rewrite-atm</v> about the ``use of dwp = t'' for an example of a \u000A theorem provable only after this change.</p> \u000A \u000A <p>Strengthened the ability of <see topic=\"ACL2____TYPE-SET\">type-set</see> reasoning to make deductions \u000A about terms being integers or non-integer rationals.  The following example \u000A illustrates the enhancement: before the change, no simplification was \u000A performed, but after the change, the conclusion simplifies to <v>(foo t)</v>. \u000A Thanks to Robert Krug for conveying the problem to us and outlining a \u000A solution.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo (x) t)\u000A(<see topic=\"ACL2____THM\">thm</see> ; should reduce conclusion to (foo t)\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A               (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)\u000A               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> 1/3 y))))\u000A          (foo (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> y (<see topic=\"COMMON-LISP_____A2\">*</see> 3 x))))))</code> \u000A \u000A <p><b>BUG FIXES</b></p> \u000A \u000A <p>Fixed a class of soundness bugs involving each of the following functions: \u000A <tt><see topic=\"ACL2____GETENV_42\">getenv$</see></tt>, <tt><see topic=\"ACL2____GET-WORMHOLE-STATUS\">get-wormhole-status</see></tt>, <tt><see topic=\"ACL2____CPU-CORE-COUNT\">cpu-core-count</see></tt>, <tt><see topic=\"ACL2____WORMHOLE-P\">wormhole-p</see></tt>, <tt><see topic=\"ACL2____RANDOM_42\">random$</see></tt>, <v>file-write-date$</v>, and \u000A <v>serialize-read-fn</v>, and (for the HONS version of ACL2) <tt><see topic=\"ACL2____CLEAR-MEMOIZE-TABLE\">clear-memoize-table</see></tt> and <tt><see topic=\"ACL2____CLEAR-MEMOIZE-TABLES\">clear-memoize-tables</see></tt> as well as (possible \u000A soundness bug) <v>serialize-write-fn</v>.  For example, we were able to admit \u000A the following events, but that is no longer the case (neither for <v>getenv$</v> \u000A as shown, nor analogously for other functions listed above).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> not-true\u000A  (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____GETENV_42\">getenv$</see> &quot;PWD&quot; (build-state))))\u000A  :rule-classes nil)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> contradiction\u000A  nil\u000A  :hints ((&quot;Goal&quot;\u000A           :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____GETENV_42\">getenv$</see>))\u000A           :use not-true))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Fixed a soundness bug involving <v>with-live-state</v>, which could cause an \u000A error in the use of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> or <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt> in a book or its <see topic=\"ACL2____PORTCULLIS\">portcullis</see> commands.  See \u000A <see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see>, as the documentation for this macro has been updated; \u000A in particular it is now untouchable (see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>) and is \u000A intended only for system hackers.  Thanks to Jared Davis for reporting a bug \u000A in the use of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> after our first attempt at a \u000A fix.</p> \u000A \u000A <p>Fixed a soundness bug based on the use of <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> together with \u000A the little-used argument <v>k=t</v> for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>.  An example proof \u000A of <v>nil</v> appears in a comment in the ACL2 sources, in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-5-0\u000A ...)</v>.</p> \u000A \u000A <p>Fixed a soundness bug that allowed users to define new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> \u000A primitive commands.  Before this fix, a book proving <v>nil</v> could be \u000A certified, as shown in a comment now in the introduction of the <see topic=\"ACL2____TABLE\">table</see> \u000A <v>pc-command-table</v> in source file <v>proof-checker-a.lisp</v>.</p> \u000A \u000A <p>(Technical change, primarily related to <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>:) Plugged a \u000A security hole that allowed <see topic=\"ACL2____BOOKS\">books</see>' <see topic=\"ACL2____CERTIFICATE\">certificate</see>s to be out-of-date \u000A with respect to <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansions, but not recognized as such. \u000A The change is to include the so-called expansion-alist in the certificate's \u000A checksum.  An example appears in a comment in the ACL2 sources, in \u000A <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-5-0 ...)</v>.</p> \u000A \u000A <p>Fixed a bug in <see topic=\"ACL2____GUARD\">guard</see> verification due to expanding calls of \u000A primitives when translating user-level terms to internal form, so called \u000A ``translated terms'' (see <see topic=\"ACL2____TERM\">term</see>).  While we have not observed a \u000A soundness hole due to this bug, we have not ruled it out.  Before the bug fix, \u000A the following event was admissible, as guard verification succeeded (but \u000A clearly should not have).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____IDENTITY\">identity</see> 3)))</code> \u000A \u000A <p>For those who want details about this bug, we analyze how ACL2 generates \u000A <see topic=\"ACL2____GUARD\">guard</see> proof obligations for this example.  During that process, it \u000A evaluates ground subexpressions.  Thus, <v>(<see topic=\"COMMON-LISP____IDENTITY\">identity</see> '3)</v> is first simplified \u000A to <v>'3</v>; so a term must be built from the application of <v>car</v> to \u000A <v>'3</v>.  Guard-checking is always turned on when generating guard proof \u000A obligations, so now, ACL2 refuses to simplify <v>(<see topic=\"COMMON-LISP____CAR\">car</see> '3)</v> to <v>'nil</v>. \u000A However, before this bug fix, when ACL2 was building a term by applying \u000A <v>car</v> to argument <v>'3</v>, it did so directly without checking guards; \u000A source code function <v>cons-term</v> is `smart' that way.  After the fix, such \u000A term-building reduction is only performed when the primitive's guard is \u000A met.</p> \u000A \u000A <p>While calls of many event macros had been prohibited inside executable \u000A code, others should have been but were not.  For example, the following was \u000A formerly allowed.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program :stobjs state))\u000A  (<see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see> :my-hint (<see topic=\"COMMON-LISP____IDENTITY\">identity</see> nil)))\u000A(foo state) ; Caused hard raw Lisp error!</code> \u000A \u000A <p>Thus, several event macros (including for example <tt><see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see></tt>) may no longer be called inside executable code.</p> \u000A \u000A <p>Fixed an assertion that could occur, for example, after reverting to prove \u000A the original goal by induction and generating a goal of <v>NIL</v>.  Thanks to \u000A Jared Davis for sending us a helpful example to bring this bug to our \u000A attention.</p> \u000A \u000A <p>It was possible for <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> to generate raw Lisp code with \u000A excessively restrictive type declarations.  This has been fixed.  Thanks to \u000A Warren Hunt for reporting this bug and sending an example that illustrates it. \u000A See <see topic=\"ACL2____STOBJ-EXAMPLE-2\">stobj-example-2</see> for examples of such raw Lisp code; now, one finds \u000A <v>(<see topic=\"COMMON-LISP____AND\">and</see> fixnum (integer 0 *))</v> where formerly the type was restricted to \u000A <v>(integer 0 268435455)</v>.</p> \u000A \u000A <p>Fixed a bug in that was ignoring the use of <v>:computed-hint-replacement</v> \u000A in certain cases involving a combination of computed hints and custom keyword \u000A hints.  Thanks to Robert Krug for reporting this bug and sending a very \u000A helpful example.</p> \u000A \u000A <p>Fixed a bug in the output from <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt>, which was failing to list \u000A previous <see topic=\"ACL2____EVENTS\">events</see> in the message about ``bypassing constraints that have \u000A been proved when processing the event(s)''.</p> \u000A \u000A <p>(GCL only) Fixed a bug in <tt><see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see></tt> (which was only a bug \u000A in GCL, not an issue for other host Lisps).</p> \u000A \u000A <p>Fixed ACL2 trace output to indent properly for levels above 99 (up to \u000A 9999).  Thanks to Warren Hunt for bringing this bug to our attention.</p> \u000A \u000A <p>Fixed a bug in the reporting of times in event summaries &mdash; probably \u000A one that has been very long-standing!  The times reported had often been too \u000A small in the case of compound <see topic=\"ACL2____EVENTS\">events</see>, notably <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>. \u000A Thanks to everyone who reported this problem (we have a record of emails from \u000A Eric Smith and Jared Davis on this issue).</p> \u000A \u000A <p>Fixed a bug in <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>, where the use of <v>:lambdas</v> \u000A could prevent other parts of such a hint.  For example, the following \u000A invocation of <tt><see topic=\"ACL2____THM\">thm</see></tt> failed before this fix was made.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUND\">defund</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (foo x)) x)\u000A:hints ((&quot;Goal&quot; :expand (:lambdas (foo x)))))</code> \u000A \u000A <p>Certain ``program-only'' function calls will now cause hard Lisp errors. \u000A (The rather obscure reason for this fix is to support logical modeling of the \u000A ACL2 evaluator.  A relevant technical discussion may be found in source \u000A function <v>oneify-cltl-code</v>, at the binding of variable \u000A <v>fail_program-only-safe</v>.)</p> \u000A \u000A <p>There was an unnecessary restriction that <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt>-bound functions must \u000A return all <see topic=\"ACL2____STOBJ\">stobj</see>s among their inputs.  For example, the following \u000A definition was rejected because <v>state</v> was not among the outputs of \u000A <v>h</v>.  This restriction has been removed.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A  (<see topic=\"COMMON-LISP____FLET\">flet</see> ((h (<see topic=\"ACL2____STATE\">state</see>) (f-boundp-global 'x state)))\u000A    (h state)))</code> \u000A \u000A <p>We fixed a bug, introduced in the preceding release (Version 4.3), in the \u000A check for irrelevant formals (see <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see>).  That check had \u000A been too lenient in its handling of lambda (<see topic=\"COMMON-LISP____LET\">let</see>) expressions, for \u000A example allowing the following definition to be admitted in spite of its first \u000A formal parameter obviously being irrelevant.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x clk)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> clk)\u000A      :diverge\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((clk (<see topic=\"COMMON-LISP____1-\">1-</see> clk)))\u000A      (foo x clk))))</code> \u000A \u000A <p>Fixed a bug in the <v>mini-proveall</v> target in <v>GNUmakefile</v>.  The fix \u000A includes a slight change to the <v>:mini-proveall</v> <see topic=\"ACL2____COMMAND\">command</see> (an extra \u000A event at the end).  Thanks to Camm Maguire for reporting this bug.</p> \u000A \u000A <p>Fixed a bug that occurred when <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> was called after using \u000A <tt><see topic=\"ACL2____SET-FMT-SOFT-RIGHT-MARGIN\">set-fmt-soft-right-margin</see></tt> or <tt><see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see></tt> to set \u000A a small right margin.</p> \u000A \u000A <p>Fixed <tt><see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see></tt> so that it takes effect for a subsequent \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  Thanks to Jared Davis and David Rager for queries \u000A that led to this fix.</p> \u000A \u000A <p>Hard Lisp errors are now avoided for certain <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules: \u000A those whose <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation is other than <v>equal</v> when the rule \u000A is originally processed, but is no longer a known equivalence relation when \u000A the rule is to be stored.  Thanks to Jared Davis for sending a useful example, \u000A a minor variant of which is included in a comment in source function \u000A <v>interpret-term-as-rewrite-rule</v> (file <v>defthm.lisp</v>).</p> \u000A \u000A <p>Fixed a bug in the ACL2 evaluator (source function <v>raw-ev-fncall</v>), \u000A which was unlikely to be exhibited in practice.</p> \u000A \u000A <p>Fixed a hard Lisp error that could occur for ill-formed <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> \u000A <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>, e.g., <v>(:meta :trigger-fns '(foo))</v>.</p> \u000A \u000A <p>It is now an error to include a <see topic=\"ACL2____STOBJ\">stobj</see> name in the <v>:renaming</v> \u000A alist (see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>).</p> \u000A \u000A <p>Some bogus warnings about non-recursive function symbols have been \u000A eliminated for rules of class <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>.</p> \u000A \u000A <p>(Allegro CL host Lisp only) Fixed an obsolete setting of compiler variable \u000A <v>comp:declared-fixnums-remain-fixnums-switch</v>, which may have been \u000A responsible for intermittent (and infrequent) checksum errors encountered \u000A while including books during certification of the regression suite.</p> \u000A \u000A <p>Fixed a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> bug that could result in duplicate goal names \u000A in the case of forced hypotheses.  An example showing this bug, before the \u000A fix, appears in a comment in the ACL2 sources, in <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> note-5-0\u000A ...)</v>.</p> \u000A \u000A <p>We fixed a bug in a prover routine involved in <see topic=\"ACL2____TYPE-SET\">type-set</see> computations \u000A involving linear arithmetic.  This bug has been around since at least as far \u000A back as Version_3.3 (released November, 2007).  We are not aware of any \u000A resulting unsoundness, though it did have the potential to weaken the prover. \u000A For example, the following is proved now, but was not proved before the bug \u000A was fixed.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A               (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y)\u000A               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 1/3 y) x)))\u000A          (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> y (<see topic=\"COMMON-LISP_____A2\">*</see> 3 x))))\u000A :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> commutativity-of-+))))</code> \u000A \u000A <p>Although all bets are off when using redefinition (see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>), we wish to minimize negative effects of its use, \u000A especially raw Lisp errors.  The examples below had caused raw Lisp errors, \u000A but no longer.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld :inline t)\u000A(<see topic=\"ACL2____REDEF_12\">redef!</see>)\u000A(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st new0 fld)\u000A(<see topic=\"ACL2____U\">u</see>)\u000A(fld st) ; previously an error, which is now fixed\u000A\u000A; Fresh ACL2 session:\u000A(<see topic=\"ACL2____REDEF_12\">redef!</see>)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (x) `(<see topic=\"COMMON-LISP____QUOTE\">quote</see> ,x))\u000A(<see topic=\"ACL2____U\">u</see>)\u000A\u000A; Fresh ACL2 session:\u000A(<see topic=\"ACL2____REDEF_12\">redef!</see>)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> 'list x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)</code> \u000A \u000A <p>Fixed a bug that could cause hard Lisp errors in an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A event.  Thanks to Sol Swords for sending an example that exhibited this bug. \u000A Here is a simpler such example; the bug was in how it was checked whether the \u000A <see topic=\"ACL2____GUARD\">guard</see> for a guard-verified function (here, <v>g</v>) depends on some \u000A function introduced in the <see topic=\"ACL2____SIGNATURE\">signature</see> of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> (here, \u000A the function <v>f</v>).</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ((f (x) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (f x) t)))\u000A   x))</code> \u000A \u000A <p>Fixed a bug in <v>mfc-relieve-hyp</v> that we believe could prohibit its use \u000A on the last hypothesis.  Thanks to Sol Swords for reporting this bug and \u000A providing a fix.</p> \u000A \u000A <p>The syntax <v>#!</v> (see <see topic=\"ACL2____SHARP-BANG-READER\">sharp-bang-reader</see>) was broken after a \u000A skipped readtime conditional.  For example, the following input line caused an \u000A error.</p> \u000A \u000A <code>#+skip #!acl2(<see topic=\"COMMON-LISP____QUOTE\">quote</see> 3)</code> \u000A \u000A <p>This bug has been fixed.</p> \u000A \u000A <p>Fixed a bug in the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> utility, which was evidenced by \u000A error messages that could occur when dealing with free variables.  An example \u000A of such an error message is the following; we thank Robert Krug for sending us \u000A an example that produced this error and enabled us to produce a fix.</p> \u000A \u000A <code>HARD ACL2 ERROR in TILDE-@-FAILURE-REASON-PHRASE1:  Unrecognized failure\u000Areason, ((MEM-ARRAY . X86) (ADDR QUOTE 9)).</code> \u000A \u000A <p>We fixed an obscure bug that we believe could interfere with <tt><see topic=\"ACL2____DEFPROXY\">defproxy</see></tt> because of an incorrect <v>(declaim (notinline &lt;function&gt;))</v> \u000A form.</p> \u000A \u000A <p><b>CHANGES AT THE SYSTEM LEVEL AND TO DISTRIBUTED BOOKS</b></p> \u000A \u000A <p>Improvements have been made related to the reading of characters.  In \u000A particular, checks are now done for ASCII encoding and for the expected <tt><see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see></tt> values for <v>Space</v>, <v>Tab</v>, <v>Newline</v>, <v>Page</v>, and \u000A <v>Rubout</v>.  Also, an error no longer occurs with certain uses of \u000A non-standard characters.  For example, it had caused an error to certify a \u000A book after a single <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see> of <v>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A `(<see topic=\"ACL2____DEFCONST\">defconst</see> *my-null* ,(<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0)))</v>; but this is no longer an issue. \u000A Thanks to Jared Davis for helpful correspondence that led us to make these \u000A improvements.</p> \u000A \u000A <p>The character encoding for reading from files has been fixed at iso-8859-1. \u000A See <see topic=\"ACL2____CHARACTER-ENCODING\">character-encoding</see>.  Thanks to Jared Davis for bringing this \u000A portability issue to our attention (as this change arose in order to deal with \u000A a change in the default character encoding for the host Lisp, CCL), and \u000A pointing us in the right direction for dealing with it.  In many cases, the \u000A character encoding for reading from the terminal is also iso-8859-1; but this \u000A is not guaranteed.  In particular, when the host Lisp is SBCL this may not be \u000A the case.</p> \u000A \u000A <p>Although the HTML documentation is distributed with ACL2, it had not been \u000A possible for users to build that documentation without omitting graphics, for \u000A example on the ACL2 home page.  That has been fixed, as files \u000A <v>graphics/*.gif</v> are now distributed.</p> \u000A \u000A <p>Compiler warnings are suppressed more completely than they had been before. \u000A For example, the following had produced a compiler warning when the host Lisp \u000A is CCL, but no longer does so.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f () (<see topic=\"COMMON-LISP____CAR\">car</see> 3))\u000A(<see topic=\"ACL2____TRACE_42\">trace$</see> f)</code> \u000A \u000A <p>Removed support for ``tainted'' <see topic=\"ACL2____CERTIFICATE\">certificate</see>s.  One reason is that \u000A there are rarely incremental releases.  A stronger reason is that for the \u000A compatibility of a new release is with the previous non-incremental release, \u000A it's not particularly relevant whether or not the new release is \u000A incremental.</p> \u000A \u000A <p>The `make' variable <v>BOOKS</v> can now be defined above the line that \u000A includes Makefile-generic.  (For relevant background, see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>.)</p> \u000A \u000A <p>(SBCL only) ACL2 images built on SBCL now have an option, \u000A <v>--dynamic-space-size 2000</v>, that can avoid space problems that could \u000A previously have caused the session to die.</p> \u000A \u000A <p>The default value for variable <v>LISP</v> in file <v>GNUmakefile</v> is now \u000A <v>ccl</v>.  Thus, if you use `make' in the standard way to build an ACL2 \u000A executable, the default host Lisp is <v>ccl</v> rather than <v>gcl</v>.</p> \u000A \u000A <p><b>EMACS SUPPORT</b></p> \u000A \u000A <p><b>EXPERIMENTAL VERSIONS</b></p> \u000A \u000A <p>For the version supporting the reals, ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>), the \u000A supporting function <v>floor1</v> has been defined in raw Lisp.  This avoids an \u000A error such as in the following case.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f () (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)) (floor1 8/3))\u000A(f) ; had caused raw Lisp error, before the fix</code> \u000A \u000A <p>Among the enhancements for the parallel version, ACL2(p) (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>), are the following.  We thank David Rager for his work in \u000A developing ACL2(p) and these improvements in particular.</p> \u000A \u000A <blockquote> \u000A \u000A <p>The macro <v>set-parallel-evaluation</v> has been renamed <tt><see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see></tt>.</p> \u000A \u000A <p>Calls of the macro <tt><see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see></tt> are no longer <see topic=\"ACL2____EVENTS\">events</see>, so may not be placed at the top level of <see topic=\"ACL2____BOOKS\">books</see>.  However, it \u000A is easy to create events that have these effects; see <see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see>.  Note that now, <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> and similar \u000A commands do not change the settings for either waterfall-parallelism or \u000A waterfall-printing.</p> \u000A \u000A <p>The implementation of <tt><see topic=\"ACL2____DEFLOCK\">deflock</see></tt> has been improved.  Now, the macro it \u000A defines can provide a lock when invoked inside a <see topic=\"ACL2____GUARD\">guard</see>-verified or \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode function.  Previously, this was only the case if \u000A the function definition was loaded from raw Lisp, typically via a compiled \u000A file.</p> \u000A \u000A <p>The underlying implementation for waterfall parallelism (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>) has been improved.  As a result, even the largest \u000A proofs in the regression suite can be run efficiently in <v>:resource-based</v> \u000A waterfall parallelism mode.  Among these improvements is one that can prevent \u000A machines from rebooting because operating system limits have been exceeded; \u000A thanks to Robert Krug for bringing this issue to our attention.</p> \u000A \u000A <p>There is also a new flag for configuring the way waterfall parallelism \u000A behaves once underlying system resource limits are reached.  This flag is most \u000A relevant to <v>:full</v> waterfall parallelism.  see <see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> for more information.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DMR\">dmr</see></tt> utility has the same behavior in ACL2(p) as it has in ACL2 \u000A unless waterfall-parallelism has been set to a non-<v>nil</v> value (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>), in which case statistics about parallel execution \u000A are printed instead of the usual information.</p> \u000A \u000A <p>The user can now build the regression suite using waterfall <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  See the distributed file <v>acl2-customization-files/README</v> \u000A for details, and see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see> for a \u000A disclaimer related to building the regression suite using waterfall \u000A parallelism.</p> \u000A \u000A <p>When building ACL2 with both the hons and parallelism extensions (what is \u000A called ``ACL2(hp)''), the functions that are automatically memoized by the \u000A hons extension are now automatically unmemoized and memoized when the user \u000A toggles waterfall parallelism on and off, respectively.</p> \u000A \u000A <p>Calling <tt><see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see></tt> with a flag of <v>t</v> now results \u000A in the same settings as if it were called with a flag of <v>:resource-based</v>, \u000A which is now the recommended mode for waterfall parallelism.  Thanks to Shilpi \u000A Goel for requesting this feature.</p> \u000A \u000A <p>The prover now aborts in a timely way in response to interrupts issued \u000A during a proof with waterfall parallelism enabled.  (This had often not been \u000A the case.)  Thanks to Shilpi Goel for requesting this \u000A improvement.</p></blockquote> \u000A \u000A <p>Among the enhancements for the HONS extension (see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>) are the following.</p> \u000A \u000A <blockquote> \u000A \u000A <p>The compact-print code has been replaced by new serialization routines \u000A contributed by Jared Davis.  This may improve performance when including books \u000A that contain <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>s that expand to very large constants.  You can \u000A also now save objects to disk without going into raw lisp; see <see topic=\"ACL2____SERIALIZE\">serialize</see> for details.</p> \u000A \u000A <p>Printing of certain messages has been sped up (by using Lisp function \u000A <v>force-output</v> in place of <v>finish-output</v>).  Thanks to Jared Davis for \u000A contributing this improvement.</p> \u000A \u000A <p><see topic=\"ACL2____STOBJ\">Stobj</see> array writes are perhaps twice as fast.</p> \u000A \u000A <p>It is now permitted to <see topic=\"ACL2____MEMOIZE\">memoize</see> functions that take user-defined \u000A <see topic=\"ACL2____STOBJ\">stobj</see>s as inputs, provided that no <see topic=\"ACL2____STOBJ\">stobj</see>s are returned.  Even if \u000A stobjs are returned, memoization is permitted provided the condition is \u000A <v>nil</v>, as when profiling (see <see topic=\"ACL2____PROFILE\">profile</see>).  Thanks to Sol Swords for an \u000A observation that led to this improvement and for useful conversations, \u000A including follow-up leading us to improve our initial implementation.</p> \u000A \u000A <p>Fixes have been made for memoizing with a non-<v>nil</v> value of \u000A <v>:ideal-okp</v>.  Errors had occurred when memoizing with a <v>:condition</v> \u000A other than <v>t</v> for a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function that had not been \u000A <see topic=\"ACL2____GUARD\">guard</see>-verified, even with a non-<v>nil</v> value of <v>:ideal-okp</v>; and \u000A after successfully memoizing such a function (without such <v>:condition</v>), \u000A it had not been possible to <tt><see topic=\"ACL2____UNMEMOIZE\">unmemoize</see></tt> it.  Thanks to Sol Swords for \u000A reporting issues with the <v>:ideal-okp</v> argument of <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>.</p> \u000A \u000A <p>If a book defined a function that was subsequently <see topic=\"ACL2____MEMOIZE\">memoize</see>d in that \u000A book, the function would no longer behaves as memoized upon completion of book \u000A certification (unless that <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> command was undone and \u000A replaced by evaluation of a corresponding <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> command).  This \u000A has been fixed.  Thanks to David Rager for pointing out the problem by sending \u000A an example.</p> \u000A \u000A <p>We now support ACL2(h) built not only on 64-bit CCL but also on all \u000A supported host Ansi Common Lisps (i.e., all supported host Lisps except GCL). \u000A Thanks to Jared Davis for doing much of the work to make this improvement. \u000A Note that performance will likely be best for 64-bit CCL; for some Lisps, \u000A performance may be much worse, probably depending in part on the underlying \u000A implementation of hash tables.</p></blockquote>"],

 "ACL2-PC____REWRITE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> reverse-reverse)\u000A   -- apply the rewrite rule `reverse-reverse'\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> (:rewrite reverse-reverse))\u000A   -- same as above\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> 2)\u000A   -- apply the second rewrite rule, as displayed by show-rewrites\u000Arewrite\u000A   -- apply the first rewrite rule, as displayed by show-rewrites\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> transitivity-of-&lt; ((y 7)))\u000A   -- apply the rewrite rule transitivity-of-&lt; with the substitution\u000A      that associates 7 to the ``free variable'' y\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> foo ((x 2) (y 3)) t)\u000A   -- apply the rewrite rule foo by substituting 2 and 3 for free\u000A      variables x and y, respectively, and also binding all other\u000A      free variables possible by using the current context\u000A      (hypotheses and governors)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REWRITE\">rewrite</see> &amp;optional rule-id substitution instantiate-free)</code> \u000A \u000A <p>Replace the current subterm with a new term by applying a <see topic=\"ACL2____REWRITE\">rewrite</see> or \u000A <see topic=\"ACL2____DEFINITION\">definition</see> rule.  The replacement will be done according to the \u000A information provided by the <v>show-rewrites</v> (<v>sr</v>) command.</p> \u000A \u000A <p>See also the <v>linear</v> command, for an analogous command to use with \u000A <see topic=\"ACL2____LINEAR\">linear</see> rules.</p> \u000A \u000A <p>If <v>rule-id</v> is a positive integer <v>n</v>, then the <v>n</v>th rule as \u000A displayed by <v>show-rewrites</v> is the one that is applied.  If <v>rule-id</v> \u000A is <v>nil</v> or is not supplied, then it is treated as the number 1. \u000A Otherwise, <v>rule-id</v> should be either a symbol or else a <v>:rewrite</v> or \u000A <v>:definition</v> <see topic=\"ACL2____RUNE\">rune</see>.  If a symbol is supplied, then any \u000A (<v>:rewrite</v> or <v>:definition</v>) rule of that name may be used.  We say \u000A more about this, and describe the other optional arguments, below.</p> \u000A \u000A <p>Consider first the following example.  Suppose that the current subterm is \u000A <v>(<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> y))</v> and that there is a <see topic=\"ACL2____REWRITE\">rewrite</see> rule called \u000A <v>reverse-reverse</v> of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x)) x)) .</code> \u000A \u000A <p>Then the instruction <v>(<see topic=\"ACL2____REWRITE\">rewrite</see> reverse-reverse)</v> causes the current \u000A subterm to be replaced by <v>y</v> and creates a new goal with conclusion \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)</v>.  An exception is that if the top-level hypotheses imply \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)</v> using only ``trivial reasoning'' (more on this below), \u000A then no new goal is created.</p> \u000A \u000A <p>If the <v>rule-id</v> argument is a number or is not supplied, then the \u000A system will store an instruction of the form <v>(<see topic=\"ACL2____REWRITE\">rewrite</see> name ...)</v>, where \u000A <v>name</v> is the name of a rewrite rule; this is in order to make it easier to \u000A replay instructions when there have been changes to the history.  Except: \u000A instead of the name (whether the name is supplied or calculated), the system \u000A stores the <see topic=\"ACL2____RUNE\">rune</see> if there is any chance of ambiguity.  (Formally, \u000A ``ambiguity'' here means that the rune being applied is of the form \u000A <v>(:rewrite name . index)</v>, where index is not <v>nil</v>.)</p> \u000A \u000A <p>Speaking in general, then, a <v>rewrite</v> instruction works as follows:</p> \u000A \u000A <p>First, a <see topic=\"ACL2____REWRITE\">rewrite</see> or <see topic=\"ACL2____DEFINITION\">definition</see> rule is selected according to \u000A the arguments of the <v>rewrite</v> instruction.  The selection is made as \u000A explained under ``General Form'' above.</p> \u000A \u000A <p>Next, the left-hand side of the rule is matched with the current subterm, \u000A i.e., a substitution <v>unify-subst</v> is found such that if one instantiates \u000A the left-hand side of the rule with <v>unify-subst</v>, then one obtains the \u000A current subterm.  If this match fails, then the instruction fails.</p> \u000A \u000A <p>Next, an attempt is made to relieve (discharge) the hypotheses, much as the \u000A theorem prover relieves hypotheses except that there is no call to the \u000A rewriter.  First, the substitution <v>unify-subst</v> is extended with the \u000A <v>substitution</v> argument, which may bind free variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).  Each hypothesis of the rule is then considered in turn, \u000A from first to last.  For each hypothesis, first the current substitution is \u000A applied, and then the system checks whether the hypothesis is ``clearly'' true \u000A in the current context.  If there are variables in the hypotheses of the rule \u000A that are not bound by the current substitution, then a weak attempt is made to \u000A extend that substitution so that the hypothesis is present in the current \u000A context (see the documentation for the proof-checker <v>hyps</v> command under \u000A <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>), much as would be done by the theorem prover's \u000A rewriter.</p> \u000A \u000A <p>If in the process above there are free variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), but the proof-checker can see how to bind them to relieve \u000A all hypotheses, then it will do so in both the <v>show-rewrites</v> (<v>sr</v>) \u000A and <v>rewrite</v> commands.  But normally, if even one hypothesis remains \u000A unrelieved, then no automatic extension of the substitution is made.  Except, \u000A if <v>instantiate-free</v> is not <v>nil</v>, then that extension to the \u000A substitution is kept.  (Technical note: in the case of an unrelieved \u000A hypothesis and a non-<v>nil</v> value of <v>instantiate-free</v>, if a <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypothesis produces a list of binding alists, then the last of \u000A those alists is the one that is used to extend the substitution.)</p> \u000A \u000A <p>Finally, the instruction is applied as follows.  The current subterm is \u000A replaced by applying the final substitution described above to the right-hand \u000A side of the selected rule.  And, one new subgoal is created for each \u000A unrelieved hypothesis of the rule, whose top-level hypotheses are the \u000A governors and top-level hypotheses of the current goal and whose conclusion \u000A and current subterm are the instance, by that same final substitution, of that \u000A unrelieved hypothesis.</p> \u000A \u000A <p><b>Remark:</b> The substitution argument should be a list whose elements \u000A have the form <v>(variable term)</v>, where <v>term</v> may contain \u000A abbreviations.</p>"],

 "ACL2-PC____APPLY-LINEAR":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(apply-linear foo)\u000A   -- apply the linear rule `foo'\u000A(apply-linear (:linear foo))\u000A   -- same as above\u000A(apply-linear 2)\u000A   -- apply the second linear rule, as displayed by show-linears\u000Arewrite\u000A   -- apply the first rewrite rule, as displayed by show-rewrites\u000A(apply-linear foo ((y 7)))\u000A   -- apply the linear rule foo with the substitution\u000A      that associates 7 to the ``free variable'' y\u000A(apply-linear foo ((x 2) (y 3)) t)\u000A   -- apply the linear rule foo by substituting 2 and 3 for free\u000A      variables x and y, respectively, and also binding all other\u000A      free variables possible by using the current context\u000A      (hypotheses and governors)\u000A\u000AGeneral Form:\u000A(apply-linear &amp;optional rule-id substitution instantiate-free)</code> \u000A \u000A <p>Add a new top-level hypothesis by applying a <see topic=\"ACL2____LINEAR\">linear</see> rule to the \u000A current subterm.  The new hypothesis will be created according to the \u000A information provided by the <v>show-linears</v> (<v>sls</v>) command.</p> \u000A \u000A <p>A short name for this command is <v>al</v>.</p> \u000A \u000A <p>We assume familiarity with the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>rewrite</v> (<v>r</v>) \u000A command.  In brief, the <v>apply-linear</v> command is an analogue of the \u000A <v>rewrite</v> command, but for <see topic=\"ACL2____LINEAR\">linear</see> rules in place of <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rules.  There is a significant difference: for the <v>apply-linear</v> command, \u000A instead of rewriting the current subterm as is done by the <v>rewrite</v> \u000A command, the conclusion of the applicable linear rule, suitably instantiated, \u000A is added as a new (and last) top-level hypothesis of the goal.  There is \u000A another significant difference: the automatic application of <see topic=\"ACL2____LINEAR\">linear</see> \u000A rules in the theorem prover is somewhat more complex than the automatic \u000A application of <see topic=\"ACL2____REWRITE\">rewrite</see> rules, so the <v>apply-linear</v> command may not \u000A correspond as closely to the prover's automatic use of a linear rule as the \u000A <v>rewrite</v> command corresponds to the prover's automatic use of a rewrite \u000A rule.</p> \u000A \u000A <p>Below, we refer freely to the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for the proof-checker's \u000A <v>rewrite</v> command.</p> \u000A \u000A <p>The <v>rule-id</v> is treated just as it is by the <v>rewrite</v> command.  If \u000A <v>rule-id</v> is a positive integer <v>n</v>, then the <v>n</v>th rule as displayed \u000A by <v>show-linears</v> is the one that is applied.  If <v>rule-id</v> is <v>nil</v> \u000A or is not supplied, then it is treated as the number 1.  Otherwise, \u000A <v>rule-id</v> should be either a symbol or else a <v>:linear</v> <see topic=\"ACL2____RUNE\">rune</see>.  If \u000A a symbol is supplied, then any <see topic=\"ACL2____LINEAR\">linear</see> rule of that name may be \u000A used.</p> \u000A \u000A <p>Consider the following example.  Suppose that the current subterm is <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see>\u000A (g (h y)) y)</v> and that <v>foo</v> is the name of the following linear rule.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A         (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (g x) 15))</code> \u000A \u000A <p>Then the instruction <v>(apply-linear foo)</v> applies <v>foo</v> by adding a \u000A new hypothesis <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> (g (h y)) 15)</v>.  In addition, a new goal with conclusion \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)</v> is created unless the current context (top-level \u000A hypotheses and governors) implies <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)</v> using only ``trivial \u000A reasoning'', just as for the <v>rewrite</v> command.</p> \u000A \u000A <p>If the <v>rule-id</v> argument is a number or is not supplied, then the \u000A system will store an instruction of the form <v>(apply-linear name ...)</v>, \u000A where <v>name</v> is the name of a <see topic=\"ACL2____LINEAR\">linear</see> rule; this is in order to make \u000A it easier to replay instructions when there have been changes to the history. \u000A Except: instead of the name (whether the name is supplied or calculated), the \u000A system stores the <see topic=\"ACL2____RUNE\">rune</see> if there is any chance of ambiguity.  (Formally, \u000A ``ambiguity'' here means that the rune being applied is of the form \u000A <v>(:rewrite name . index)</v>, where index is not <v>nil</v>.)</p> \u000A \u000A <p>Speaking in general, then, an <v>apply-linear</v> instruction works as \u000A follows.</p> \u000A \u000A <p>First, a <see topic=\"ACL2____LINEAR\">linear</see> rule is selected according to the arguments of the \u000A instruction.  The selection is made as explained under ``General Form'' \u000A above.</p> \u000A \u000A <p>Next, a trigger term of the rule (see <see topic=\"ACL2____LINEAR\">linear</see>) is matched with the \u000A current subterm, i.e., a substitution <v>unify-subst</v> is found such that if \u000A one instantiates that trigger term of the rule with <v>unify-subst</v>, then one \u000A obtains the current subterm.  If this match fails, then the instruction \u000A fails.</p> \u000A \u000A <p>Next, an attempt is made to relieve (discharge) the hypotheses, possibly \u000A handling free variables (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), exactly as is done with \u000A hypotheses when applying the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command, <v>rewrite</v> \u000A (<v>r</v>).</p> \u000A \u000A <p>Finally, the instruction is applied exactly as the <v>rewrite</v> instruction \u000A is applied, except instead of replacing the current subterm, the rule's \u000A instantiated conclusion is added to the end of the list of top-level \u000A hypotheses of the goal.</p> \u000A \u000A <p>Note that as for the <v>rewrite</v> command, the substitution argument should \u000A be a list whose elements have the form <v>(variable term)</v>, where <v>term</v> \u000A may contain abbreviations.</p>"],

 "COMMON-LISP____MEMBER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst)\u000A(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____MEMBER\">Member</see> x lst)</v> equals the longest tail of the list <v>lst</v> that \u000A begins with <v>x</v>, or else <v>nil</v> if no such tail exists.  The optional \u000A keyword, <v>:TEST</v>, has no effect logically, but provides the test (default \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with successive elements of \u000A <v>lst</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>member</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> \u000A (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument \u000A must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>member</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____MEMBER-EQ\">member-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test\u000A 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>member-equal</v>.</p> \u000A \u000A <p><v>Member</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>member-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> member-equal (x lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) lst)\u000A             (t (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))))</code>"],

 "ACL2____SPECIFIC-KINDS-OF-FORMULAS-AS-REWRITE-RULES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Below we give you some guidelines for handling specific, commonly \u000A occurring situations.</p> \u000A \u000A <p>* <b>Associativity</b>: If a function <v>f</v> is associative, prove</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f (f x y) z) (f x (f y z)))</code> \u000A \u000A <p>ACL2 will use this to flatten <v>f</v>-nests ``to the right.''</p> \u000A \u000A <p>* <b>Commutativity</b>: If a function <v>f</v> is commutative, prove both</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x y) (f y x))</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x (f y z)) (f y (f x z)))</code> \u000A \u000A <p>ACL2's heuristics will use these rules to order the arguments \u000A alphabetically, so that <v>(f B (f D (f A C)))</v> becomes <v>(f A (f B (f C\u000A D)))</v>.</p> \u000A \u000A <p>* <b>Distributivity</b>: If you have a pair of functions <v>f</v> and <v>g</v> \u000A so that <v>(f x (g y z))</v> is <v>(g (f x y) (f x z))</v> or some other form of \u000A distributivity is provable, arrange your rules to move the lighter function \u000A symbol up and the heavier one toward the variable symbols.  For example, our \u000A arithmetic libraries drive multiplication through addition, producing sums of \u000A products rather than products of sums.</p> \u000A \u000A <p>* <b>Identity and Other Laws</b>: Prove the obvious identity and zero laws \u000A (or at least anticipate that you might need them down the road) so as to \u000A eliminate operators.</p> \u000A \u000A <p>* <b>Get Rid of Tail Recursive Functions</b>: A corollary to the above \u000A advice concerns tail recursive functions that use auxiliary variables.  New \u000A users often define concepts using tail recursions, accumulating partial \u000A results in auxiliary variables, because creating such functions is similar to \u000A programming with <v>while</v> loops.  Expert users will use tail recursion when \u000A necessary for execution efficiency.  But tail recursive functions are messy to \u000A reason about: their auxiliary variables have to be properly initialized to \u000A make the functions compute the expected results, but to state inductively \u000A provable properties of tail recursive functions you must identify the \u000A invariants on those auxiliary variables.  This problem tends not to happen \u000A with <i>primitive recursive functions</i>.  A primitive recursive function is \u000A one that recurs down one variable and holds all the other variables constant \u000A in recursion.  Most tail-recursive functions can be written elegantly as \u000A primitive recursive functions, though one might have to ignore the \u000A programmer's desire to make things efficient and define auxiliary functions to \u000A appropriately transform the value returned by the recursive call.  The classic \u000A example is reverse defined in terms of the auxiliary function <v>append</v> \u000A versus reverse defined tail recursively with an accumulator.  By introducing \u000A <v>append</v> you introduce a concept about which you can state lemmas and \u000A decompose the proofs of properties of reverse.  So if your problem involves \u000A tail recursive functions with auxiliary variables, define the primitive \u000A recursive version, prove that the tail recursive function is equivalent to the \u000A primitive recursive one, and arrange the rewrite rule to eliminate the tail \u000A recursive function.</p> \u000A \u000A <p>* <b>Get Rid of Mutually Recursive Functions</b>: Similarly, if you have \u000A used <v>mutual-recursion</v> to introduce a clique of mutually recursive \u000A functions, <v>f1</v>, <v>f2</v>, ..., you will find that to reason about any one \u000A function in the nest you have to reason about all of them.  Any mutually \u000A recursive function can be defined in a singly recursive way.  So do that and \u000A then prove a rewrite rule that gets rid of all the mutually recursive \u000A functions by proving</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f1 ...) (g1 ...))\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f2 ...) (g2 ...))\u000A     ...)</code> \u000A \u000A <p>where the <v>gi</v> are singly recursive.  You may need to appeal to a trick \u000A to define the <v>gi</v>: define a singly recursive function that takes a flag \u000A argument and mimics whichever mutually recursive function the flag specifies. \u000A See <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A see <see topic=\"ACL2____MUTUAL-RECURSION-PROOF-EXAMPLE\">mutual-recursion-proof-example</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>If you got to this documentation page from the tutorial discussion of \u000A rewrite rules, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-2\">introduction-to-rewrite-rules-part-2</see>.</p>"],

 "ACL2____MUTUAL-RECURSION-PROOF-EXAMPLE":[["Tutorial5-miscellaneous-examples"],"ACL2 Sources","ACL2","<p>Sometimes one wants to reason about mutually recursive functions. \u000A Although this is possible in ACL2, it can be a bit awkward.  This example is \u000A intended to give some ideas about how one can go about such proofs.</p> \u000A \u000A <p>For an introduction to mutual recursion in ACL2, see <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>.</p> \u000A \u000A <p>We begin by defining two mutually recursive functions: one that collects \u000A the variables from a <see topic=\"ACL2____TERM\">term</see>, the other that collects the variables from a \u000A list of <see topic=\"ACL2____TERM\">term</see>s.  We actually imagine the <see topic=\"ACL2____TERM\">term</see> argument to be a \u000A <v>pseudo-termp</v>; see <see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> free-vars1 (<see topic=\"ACL2____TERM\">term</see> ans)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> term)\u000A         (<see topic=\"ACL2____ADD-TO-SET-EQ\">add-to-set-eq</see> term ans))\u000A        ((fquotep term) ans)\u000A        (t (free-vars1-lst (<see topic=\"COMMON-LISP____CDR\">cdr</see> term) ans))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> free-vars1-lst (lst ans)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) ans)\u000A        (t (free-vars1-lst (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)\u000A                           (free-vars1 (<see topic=\"COMMON-LISP____CAR\">car</see> lst) ans)))))\u000A\u000A)</code> \u000A \u000A <p>Now suppose that we want to prove the following theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> symbol-listp-free-vars1-try-1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A                (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A           (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1 x ans))))</code> \u000A \u000A <p>Often ACL2 can generate a proof by induction based on the structure of \u000A definitions of function symbols occurring in the conjecture.  In this case, \u000A ACL2 chooses to use an induction scheme suggested by <v>(<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans)</v>, \u000A and sadly, that doesn't work.  If one were doing this proof with pencil and \u000A paper, one would be more likely to prove a combination of the conjecture above \u000A and an analogous conjecture about free-vars1-lst.  Feel free to try a pencil \u000A and paper proof!  Or you can read on, to see how one can get ACL2 to do such a \u000A proof after all.</p> \u000A \u000A <p>The trick is to define a function that suggests an appropriate induction. \u000A The induction suggested is based on the if-then-else structure of the \u000A function's definition, where inductive hypotheses are generated for recursive \u000A calls &mdash; below we explain how that works for this function.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> symbol-listp-free-vars1-induction (x ans)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A; then we just make sure x and ans aren't considered irrelevant:\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> x ans)\u000A    (<see topic=\"COMMON-LISP____LIST\">list</see> (symbol-listp-free-vars1-induction (<see topic=\"COMMON-LISP____CAR\">car</see> x) ans)\u000A          (symbol-listp-free-vars1-induction (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) ans)\u000A          (symbol-listp-free-vars1-induction (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A                                             (free-vars1 (<see topic=\"COMMON-LISP____CAR\">car</see> x) ans)))))</code> \u000A \u000A <p>The if-then-else structure of this function generates two cases.  In one \u000A case, <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v> is true, and the theorem to be proved should be proved \u000A under no additional hypotheses except for <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v>; in other words, \u000A <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v> gives us the base case of the induction.  In the other case, \u000A <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x))</v> is assumed together with three instances of the theorem to \u000A be proved, one for each recursive call.  So, one instance substitutes <v>(<see topic=\"COMMON-LISP____CAR\">car</see>\u000A x)</v> for <v>x</v>; one substitutes <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> for <v>x</v>; and the third \u000A substitutes <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> for <v>x</v> and <v>(free-vars1 (<see topic=\"COMMON-LISP____CAR\">car</see> x) ans)</v> for \u000A <v>ans</v>.  If you think about how you would go about a hand proof of the \u000A theorem to follow, you'll likely come up with a similar scheme.</p> \u000A \u000A <p>We now prove the two theorems together as a conjunction, because the \u000A inductive hypotheses for one are sometimes needed in the proof of the other \u000A (even when you do this proof on paper!).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> symbol-listp-free-vars1\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A                     (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A                (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1 x ans)))\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp x)\u000A                     (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A                (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1-lst x ans))))\u000A  :hints\u000A  ((&quot;Goal&quot; :induct (symbol-listp-free-vars1-induction x ans))))</code> \u000A \u000A <p>The above works, but we conclude by illustrating a more efficient approach, \u000A in which we restrict to appropriate inductive hypotheses for each case.</p> \u000A \u000A <code>(<see topic=\"ACL2____UBT\">ubt</see> 'symbol-listp-free-vars1-induction)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> symbol-listp-free-vars1-induction (flg x ans)\u000A\u000A; Flg is nil if we are ``thinking'' of a single term.\u000A\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) ; whether x is a single term or a list of terms\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> x ans)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> flg ; i.e., if x is a list of terms\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> (symbol-listp-free-vars1-induction nil (<see topic=\"COMMON-LISP____CAR\">car</see> x) ans)\u000A              (symbol-listp-free-vars1-induction t\u000A                                                 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A                                                 (free-vars1 (<see topic=\"COMMON-LISP____CAR\">car</see> x) ans)))\u000A      (symbol-listp-free-vars1-induction t (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) ans))))</code> \u000A \u000A <p>We now state the theorem as a conditional, so that it can be proved nicely \u000A using the <see topic=\"ACL2____INDUCTION\">induction</see> scheme that we have just coded.  The prover will \u000A not store an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> <see topic=\"ACL2____TERM\">term</see> as a <see topic=\"ACL2____REWRITE\">rewrite</see> rule, but that's OK \u000A (provided we tell it not to try), because we're going to derive the \u000A corollaries of interest later and make <b>them</b> into <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A rules.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> symbol-listp-free-vars1-flg\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> flg\u000A      (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp x)\u000A                    (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A               (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1-lst x ans)))\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A                  (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A             (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1 x ans))))\u000A  :hints\u000A  ((&quot;Goal&quot; :induct (symbol-listp-free-vars1-induction flg x ans)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>And finally, we may derive the theorems we are interested in as immediate \u000A corollaries.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> symbol-listp-free-vars1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)\u000A                (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A           (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1 x ans)))\u000A  :hints ((&quot;Goal&quot; :by (:instance symbol-listp-free-vars1-flg\u000A                                 (flg nil)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> symbol-listp-free-vars1-lst\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (pseudo-term-listp x)\u000A                (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> ans))\u000A           (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (free-vars1-lst x ans)))\u000A  :hints ((&quot;Goal&quot; :by (:instance symbol-listp-free-vars1-flg\u000A                                 (flg t)))))</code> \u000A \u000A <p>You may find <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> that help you to automate this kind of \u000A reasoning about mutually recursive functions.  See for example <see topic=\"ACL2____MAKE-FLAG\">make-flag</see>.</p>"],

 "ACL2____ARCHITECTURE-OF-THE-PROVER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Six built-in proof techniques are used by ACL2 to decompose the \u000A goal formula into subgoals.</p> \u000A \u000A <p><i>simplification</i> -- decision procedures and rewriting with previously \u000A  proved rules, but actually including a host of other techniques under your \u000A  control.  Simplification is the only proof technique that can reduce a \u000A  formula to 0 subgoals (i.e., prove it) rather than just transform it to other \u000A  formulas.  The predominant activity in most proofs is simplification.  There \u000A  are many ways you can affect what the simplifier does to your formulas.  Good \u000A  users spend most of their time thinking about how to control the \u000A  simplifier.</p> \u000A \u000A <p><i>destructor elimination</i> -- getting rid of ``destructor terms'' like \u000A  <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> X)</v> and <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> X)</v> by replacing a variable, e.g., <v>X</v>, by a \u000A  ``constructor'' term, e.g., <v>(<see topic=\"COMMON-LISP____CONS\">CONS</see> A B)</v>.  But you can tell ACL2 about new \u000A  destructor/constructor combinations.</p> \u000A \u000A <p><i>cross-fertilization</i> -- using an equivalence hypothesis by \u000A  substituting one side for the other into the conclusion <i>and then throwing \u000A  the hypothesis away</i>.  This is a heuristic that helps use an inductive \u000A  hypothesis and prepare for another induction.</p> \u000A \u000A <p><i>generalization</i> -- replacing a term by a new variable and restricting \u000A  the new variable to have some of the properties of the term.  You can control \u000A  the restrictions imposed on the new variable.  This is a heuristic that \u000A  prepares the goal for another induction.</p> \u000A \u000A <p><i>elimination of irrelevance</i> -- throwing away unnecessary hypotheses. \u000A  This is a heuristic that prepares the goal for another induction.</p> \u000A \u000A <p><i>induction</i> -- selecting an induction scheme to prove a formula. \u000A  Inductions are ``suggested'' by the recursive functions appearing in the \u000A  formula.  But you can control what inductions are suggested by terms.</p> \u000A \u000A <p>But you can add additional techniques, called <i>clause processors</i>.</p> \u000A \u000A <p>The various techniques are tried in turn, with simplification first and \u000A  induction last.  Each technique reports one of three outcomes: it found \u000A  nothing to change (i.e., the technique doesn't <i>apply</i> to that subgoal), \u000A  it decided to abort the proof attempt (typically because there is reason to \u000A  believe the proof is failing), or it decomposed the goal into <i>k</i> \u000A  subgoals.</p> \u000A \u000A <p>The last outcome has a special case: if <i>k</i> is 0 then the technique \u000A  proved the goal.  Whenever <i>k</i> is non-0, the process starts over again \u000A  with simplification on each of the <i>k</i> subgoals.  However, it saves up \u000A  all the subgoals for which induction is the only proof technique left to \u000A  try. That way you see how it performs on every base case and induction step \u000A  of one induction before it launches into another induction.</p> \u000A \u000A <p>It runs until you or one of the proof techniques aborts the proof attempt \u000A  or until all subgoals have been proved.</p> \u000A \u000A <p>Note that if simplification produces a subgoal, that subgoal is \u000A  re-simplified.  This process continues until the subgoal cannot be simplified \u000A  further.  Only then is the next proof technique is tried.  Such suboals are \u000A  said to be <i>stable under simplification</i>.</p> \u000A \u000A <p>While this is happening, the prover prints an English narrative describing \u000A  the process.  Basically, after each goal is printed, the system prints an \u000A  English paragraph that names the next applicable proof technique, gives a \u000A  brief description of what that technique does to the subgoal, and says how \u000A  many new subgoals are produced.  Then each subgoal is dealt with in turn.</p> \u000A \u000A <p>If the proof is successful, you could read this log as a proof of the \u000A  conjecture.  But output from successful proofs is generally never read \u000A  because it is not important to The Method described in <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p> \u000A \u000A <p>The output of an unsuccessful proof attempt concludes with some <i>key</i> \u000A <i>checkpoints</i> which usually bear looking at.</p>"],

 "COMMON-LISP____IF":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____IF\">if</see> x y z)</v> is equal to <v>y</v> if <v>x</v> is any value other \u000A than <v>nil</v>, and is equal to <v>z</v> if <v>x</v> is <v>nil</v>.</p> \u000A \u000A <p>Only one of <v>y</v>, <v>z</v> is evaluated when <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y z)</v> is \u000A evaluated.</p> \u000A \u000A <p><v>If</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><v>If</v> is part of Common Lisp.  See any Common Lisp documentation for more \u000A information.</p>"],

 "ACL2____STOBJ-EXAMPLE-1-PROOFS":[["Stobj"],"ACL2 Sources","ACL2","<p>Consider again the event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> counters\u000A  (NodeCnt     :type integer :initially 0)\u000A  (TipCnt      :type integer :initially 0)\u000A  (IntTipsSeen :type t       :initially nil))</code> \u000A \u000A <p>discussed in <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see>, followed by the definition</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> reset-counters (counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (seq counters\u000A       (update-NodeCnt 0 counters)\u000A       (update-TipCnt 0 counters)\u000A       (update-IntTipsSeen nil counters)))</code> \u000A \u000A <p>which, because of the <v>seq</v> macro in <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see>, is just \u000A syntactic sugar for</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> reset-counters (counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((counters (update-NodeCnt 0 counters)))\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((counters (update-TipCnt 0 counters)))\u000A      (update-IntTipsSeen nil counters)))).</code> \u000A \u000A <p>Here is a simple theorem about <v>reset-counters</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> reset-counters-is-constant\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (countersp x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (reset-counters x)\u000A                  '(0 0 nil))))</code> \u000A \u000A <p>Before we talk about how to prove this theorem, note that the theorem is \u000A unusual in two respects.</p> \u000A \u000A <p>First, it calls <v>reset-counters</v> on an argument other than the variable \u000A <v>counters</v>!  That is allowed in theorems; logically speaking, the stobj \u000A functions are indistinguishable from ordinary functions.  Their use is \u000A syntactically restricted only in <v>defun</v>s, which might be compiled and run \u000A in raw Lisp.  Those restrictions allow us to implement stobj modification \u000A destructively.  But logically speaking, <v>reset-counters</v> and other stobj \u000A ``modifying'' functions just create new objects, constructively.</p> \u000A \u000A <p>Second, the theorem above explicitly provides the hypothesis that \u000A <v>reset-counters</v> is being applied to an object satisfying <v>countersp</v>. \u000A Such a hypothesis is not always required: <v>reset-counters</v> is total and \u000A will do something no matter what <v>x</v> is.  But in this particular case, the \u000A result is not <v>'(0 0 nil)</v> unless <v>x</v> is, at least, a true-list of \u000A length three.</p> \u000A \u000A <p>To make a long story short, to prove theorems about stobj functions you \u000A behave in exactly the way you would to prove the same theorems about the same \u000A functions defined without the stobj features.</p> \u000A \u000A <p>How can we prove the above theorem?  Unfolding the definition of \u000A <v>reset-counters</v> shows that <v>(reset-counters x)</v> is equal to</p> \u000A \u000A <code>(update-IntTipsSeen nil\u000A  (update-TipCnt 0\u000A    (update-NodeCnt 0 x)))</code> \u000A \u000A <p>which in turn is</p> \u000A \u000A <code>(<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2 nil\u000A (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1 0\u000A  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 0 0 x))).</code> \u000A \u000A <p>Opening up the definition of <v>update-nth</v> reduces this to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LIST_A2\">list*</see> 0 0 nil (<see topic=\"COMMON-LISP____CDDDR\">cdddr</see> x)).</code> \u000A \u000A <p>This is clearly equal to <v>'(0 0 nil)</v>, provided we know that <v>(<see topic=\"COMMON-LISP____CDDDR\">cdddr</see>\u000A x)</v> is <v>nil</v>.</p> \u000A \u000A <p>Unfortunately, that last fact requires a lemma.  The most specific lemma we \u000A could provide is</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> special-lemma-for-counters\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (countersp x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDDDR\">cdddr</see> x) nil)))</code> \u000A \u000A <p>but if you try to prove that lemma you will find that it requires some \u000A reasoning about <v>len</v> and <v>true-listp</v>.  Furthermore, the special lemma \u000A above is of interest only for <v>counters</v>.</p> \u000A \u000A <p>The following lemma about <v>len</v> is the one we prefer.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> equal-len-n\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (quotep n))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> x) n)\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n 0)\u000A                          nil\u000A                        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 0)\u000A                            (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A                          (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                               (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____-\">-</see> n 1)))))\u000A                    nil))))</code> \u000A \u000A <p>This lemma will simplify any equality in which a <v>len</v> expression is \u000A equated to any explicitly given constant <i>n</i>, e.g., <v>3</v>, reducing the \u000A equation to a conjunction of <v>consp</v> terms about the first <i>n</i> \u000A <v>cdr</v>s.</p> \u000A \u000A <p>If the above lemma is available then ACL2 immediately proves</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> reset-counters-is-constant\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (countersp x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (reset-counters x)\u000A                  '(0 0 nil))))</code> \u000A \u000A <p>The point is presumably well made: proving theorems about single-threaded \u000A object accessors and updaters is no different than proving theorems about \u000A other recursively defined functions on lists.</p> \u000A \u000A <p>As we have seen, operations on <see topic=\"ACL2____STOBJ\">stobj</see>s turn into definitions \u000A involving <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> and <tt><see topic=\"ACL2____UPDATE-NTH\">update-nth</see></tt> in the logic.  Here are two \u000A lemmas that are useful for simplifying terms involving <v>nth</v> and \u000A <v>update-nth</v>, which are therefore useful in reasoning about single-threaded \u000A objects.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> update-nth-update-nth-same\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> i1) (<see topic=\"ACL2____NFIX\">nfix</see> i2))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i1 v1 (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i2 v2 l))\u000A                  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i1 v1 l))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> update-nth-update-nth-diff\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> i1) (<see topic=\"ACL2____NFIX\">nfix</see> i2)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i1 v1 (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i2 v2 l))\u000A                  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i2 v2 (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> i1 v1 l))))\u000A  :rule-classes ((:rewrite :loop-stopper ((i1 i2)))))</code> \u000A \u000A <p>These lemmas are due to Matt Wilding.  See <see topic=\"ACL2____NU-REWRITER\">nu-rewriter</see> for a \u000A discussion of the efficient simplification of terms of the form <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> n\u000A (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> key val lst))</v>, which can be critical in settings involving \u000A sequential bindings that commonly arise in operations involving stobjs.</p> \u000A \u000A <p>We now recommend that you see <see topic=\"ACL2____STOBJ-EXAMPLE-2\">stobj-example-2</see>.</p>"],

 "ACL2____PROOF-OF-WELL-FOUNDEDNESS":[["Ordinals"],"ACL2 Sources","ACL2","<p>The soundness of ACL2 rests in part on the well-foundedness of \u000A <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> on <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s.  This can be taken as obvious if one is willing to \u000A grant that those concepts are simply encodings of the standard mathematical \u000A notions of the ordinals below <v>epsilon-0</v> and its natural ordering \u000A relation.  But it is possible to prove that <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> is well-founded on \u000A <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s without having to assert any connection to the ordinals and that \u000A is what we do here.  The community book \u000A <v>books/ordinals/proof-of-well-foundedness</v> carries out the proof outlined \u000A below in ACL2, using only that the natural numbers are well-founded.</p> \u000A \u000A <p>Before outlining the above mentioned proof, we note that in the analogous \u000A documentation page of ACL2 Version_2.7, there is a proof of the \u000A well-foundedness of <v>e0-ord-&lt;</v> on <v>e0-ordinalp</v>s, the less-than relation \u000A and recognizer for the old ordinals (that is, for the ordinals appearing in \u000A ACL2 up through that version).  Manolios and Vroon have given a proof in ACL2 \u000A Version_2.7 that the current ordinals (based on <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> and <tt><see topic=\"ACL2____O-P\">o-p</see></tt>) \u000A are order-isomorphic to the old ordinals (based on <v>e0-ord-&lt;</v> and \u000A <v>e0-ordinalp</v>).  Their proof establishes that switching from the old \u000A ordinals to the current ordinals preserves the soundness of ACL2.  For details \u000A see their paper:</p> \u000A \u000A <code>Manolios, Panagiotis &amp; Vroon, Daron. \u000AOrdinal arithmetic in ACL2. \u000AKaufmann, Matt, &amp; Moore, J Strother (eds). \u000AFourth International Workshop on the ACL2 Theorem \u000AProver and Its Applications (ACL2-2003), \u000AJuly, 2003. \u000ASee <a href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/'>http://www.cs.utexas.edu/users/moore/acl2/workshop-2003/</a>. \u000A</code> \u000A \u000A <p>We now give an outline of the above mentioned proof of well-foundedness. \u000A We first observe three facts about <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> on ordinals that have been \u000A proved by ACL2 using only structural induction on lists.  These theorems can \u000A be proved by hand.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-o&lt;\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> x y)\u000A                (<see topic=\"ACL2____O_C3\">o&lt;</see> y z))\u000A           (<see topic=\"ACL2____O_C3\">o&lt;</see> x z))\u000A  :rule-classes nil)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> non-circularity-of-o&lt;\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> x y)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> y x)))\u000A  :rule-classes nil)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> trichotomy-of-o&lt;\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O-P\">o-p</see> x)\u000A                (<see topic=\"ACL2____O-P\">o-p</see> y))\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)\u000A               (<see topic=\"ACL2____O_C3\">o&lt;</see> x y)\u000A               (<see topic=\"ACL2____O_C3\">o&lt;</see> y x)))\u000A  :rule-classes nil)</code> \u000A \u000A <p>These three properties establish that <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> orders the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s. \u000A To put such a statement in the most standard mathematical nomenclature, we can \u000A define the macro:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> o&lt;= (x y)\u000A  `(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> ,y ,x)))</code> \u000A \u000A <p>and then establish that <v>o&lt;=</v> is a relation that is a simple, complete \u000A (i.e., total) order on ordinals by the following three lemmas, which have been \u000A proved:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> antisymmetry-of-o&lt;=\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O-P\">o-p</see> x)\u000A                (<see topic=\"ACL2____O-P\">o-p</see> y)\u000A                (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> x y)\u000A                (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> y x))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use non-circularity-of-o&lt;)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-o&lt;=\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O-P\">o-p</see> x)\u000A                (<see topic=\"ACL2____O-P\">o-p</see> y)\u000A                (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> x y)\u000A                (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> y z))\u000A           (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> x z))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use transitivity-of-o&lt;)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> trichotomy-of-o&lt;=\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O-P\">o-p</see> x)\u000A                (<see topic=\"ACL2____O-P\">o-p</see> y))\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> x y)\u000A               (<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> y x)))\u000A  :rule-classes nil\u000A  :hints ((&quot;Goal&quot; :use trichotomy-of-o&lt;)))</code> \u000A \u000A <p>Crucially important to the proof of the well-foundedness of <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> on \u000A <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s is the concept of ordinal-depth, abbreviated <v>od</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> od (l)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> l)\u000A      0\u000A    (<see topic=\"COMMON-LISP____1_B2\">1+</see> (od (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> l)))))</code> \u000A \u000A <p>If the <v>od</v> of an <tt><see topic=\"ACL2____O-P\">o-p</see></tt> <v>x</v> is smaller than that of an <tt><see topic=\"ACL2____O-P\">o-p</see></tt> <v>y</v>, then <v>x</v> is <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> <v>y</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> od-1 (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x)\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> x y)\u000A    (od-1 (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> x) (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> y))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> od-implies-ordlessp\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____O-P\">o-p</see> x)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (od x) (od y)))\u000A           (<see topic=\"ACL2____O_C3\">o&lt;</see> x y))\u000A  :hints ((&quot;Goal&quot;\u000A           :induct (od-1 x y))))</code> \u000A \u000A <p>Remark.  A consequence of this lemma is the fact that if <v>s = s(1)</v>, \u000A <v>s(2)</v>, ... is an infinite, <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> descending sequence of <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s, \u000A then <v>od(s(1))</v>, <v>od(s(2))</v>, ... is a ``weakly'' descending sequence of \u000A non-negative integers: <v>od(s(i))</v> is greater than or equal to \u000A <v>od(s(i+1))</v>.</p> \u000A \u000A <p><i>Lemma Main.</i> For each non-negative integer <v>n</v>, <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> \u000A well-orders the set of <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s with <v>od</v> less than or equal to <v>n</v> \u000A .</p> \u000A \u000A <code>Base Case.  n = 0.  The o-ps with 0 od are the non-negative\u000Aintegers.  On the non-negative integers, o&lt; is the same as &lt;.\u000A\u000AInduction Step.  n &gt; 0.  We assume that o&lt; well-orders the\u000Ao-ps with od less than n.\u000A\u000A  If o&lt; does not well-order the o-ps with od less than or equal to n,\u000A  consider, D, the set of infinite, o&lt; descending sequences of o-ps of od\u000A  less than or equal to n.  The first element of a sequence in D has od n.\u000A  Therefore, the o-first-expt of the first element of a sequence in D has od\u000A  n-1.  Since o&lt;, by IH, well-orders the o-ps with od less than n, the set\u000A  of o-first-expts of first elements of the sequences in D has a minimal\u000A  element, which we denote by B and which has od of n-1.\u000A\u000A  Let k be the minimum integer such that for some infinite, o&lt; descending\u000A  sequence s of o-ps with od less than or equal to n, the first element of s\u000A  has an o-first-expt of B and an o-first-coeff of k.  Notice that k is\u000A  positive.\u000A\u000A  Having fixed B and k, let s = s(1), s(2), ... be an infinite, o&lt;\u000A  descending sequence of o-ps with od less than or equal to n such that s(1)\u000A  has a o-first-expt of B and an o-first-coeff of k.\u000A\u000A  We show that each s(i) has a o-first-expt of B and an o-first-coeff of\u000A  k. For suppose that s(j) is the first member of s either with o-first-expt\u000A  B and o-first-coeff m (m neq k) or with o-first-expt B' and o-first-coeff\u000A  B' (B' neq B). If (<see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see> s(j)) = B', then B' has od n-1 (otherwise,\u000A  by IH, s would not be infinite) and B' is o&lt; B, contradicting the\u000A  minimality of B. If 0 &lt; m &lt; k, then the fact that the sequence beginning\u000A  at s(j) is infinitely descending contradicts the minimality of k. If m &gt;\u000A  k, then s(j) is greater than its predecessor; but this contradicts the\u000A  fact that s is descending.\u000A\u000A  Thus, by the definition of o&lt;, for s to be a decreasing sequence of o-ps,\u000A  (<see topic=\"ACL2____O-RST\">o-rst</see> s(1)), (<see topic=\"ACL2____O-RST\">o-rst</see> s(2)), ... must be a decreasing sequence. We end by\u000A  showing this cannot be the case. Let t = t(1), t(2), ... be an infinite\u000A  sequence of o-ps such that t(i) = (<see topic=\"ACL2____O-RST\">o-rst</see> s(i)). Then t is infinitely\u000A  descending. Furthermore, t(1) begins with an o-p B' that is o&lt; B. Since t\u000A  is in D, t(1) has od n, therefore, B' has od n-1. But this contradicts the\u000A  minimality of B. Q.E.D.</code> \u000A \u000A <p>Theorem.  <tt><see topic=\"ACL2____O_C3\">o&lt;</see></tt> well-orders the <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s.  Proof.  Every \u000A infinite,<v> o&lt;</v> descending sequence of <tt><see topic=\"ACL2____O-P\">o-p</see></tt>s has the property that \u000A each member has <v>od</v> less than or equal to the <v>od</v>, <v>n</v>, of the \u000A first member of the sequence.  This contradicts Lemma Main.  Q.E.D.</p>"],

 "ACL2____LOGICAL-NAME":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples:\u000Aassoc\u000Acaddr\u000A+\u000A&quot;ACL2-USER&quot;\u000A&quot;arith&quot;\u000A&quot;project/task-1/arith.lisp&quot;\u000A:here</code> \u000A \u000A <p>A logical name is either a name introduced by some event, such as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, or else is the keyword \u000A <v>:here</v>, which refers to the most recent such event.  See <see topic=\"ACL2____EVENTS\">events</see>. \u000A Every logical name is either a symbol or a string.  For the syntactic rules on \u000A names, see <see topic=\"ACL2____NAME\">name</see>.  The symbols name functions, macros, constants, \u000A axioms, theorems, labels, and <see topic=\"ACL2____THEORIES\">theories</see>.  The strings name packages or \u000A <see topic=\"ACL2____BOOKS\">books</see>.  We permit the keyword symbol <v>:here</v> to be used as a logical \u000A name denoting the most recently completed event.</p> \u000A \u000A <p>The logical name introduced by an <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> is the full book name \u000A string for the book (see <see topic=\"ACL2____FULL-BOOK-NAME\">full-book-name</see>).  Thus, under the appropriate \u000A setting for the current book directory (see <see topic=\"ACL2____CBD\">cbd</see>) the event \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arith&quot;)</v> may introduce the logical name</p> \u000A \u000A <code>&quot;/usr/home/smith/project/task-1/arith.lisp&quot; .</code> \u000A \u000A <p>Under a different <tt><see topic=\"ACL2____CBD\">cbd</see></tt> setting, it may introduce a different logical \u000A name, perhaps</p> \u000A \u000A <code>&quot;/local/src/acl2/library/arith.lisp&quot; .</code> \u000A \u000A <p>It is possible that identical <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> events forms in a \u000A session introduce two different logical names because of the current book \u000A directory.</p> \u000A \u000A <p>A logical name that is a string is either a package name or a book name. \u000A If it is not a package name, we support various conventions to interpret it as \u000A a book name.  If it does not end with the string <v>&quot;.lisp&quot;</v> we extend it \u000A appropriately.  Then, we search for any book name that has the given logical \u000A name as a terminal substring.  Suppose <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arith&quot;)</v> is the \u000A only <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> so far and that \u000A <v>&quot;/usr/home/smith/project/task-1/arith.lisp&quot;</v> is the source file it \u000A processed.  Then <v>&quot;arith&quot;</v>, <v>&quot;arith.lisp&quot;</v> and \u000A <v>&quot;task-1/arith.lisp&quot;</v> are all logical names identifying that <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event (unless they are package names).  Now suppose a second \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arith&quot;)</v> is executed and processes \u000A <v>&quot;/local/src/acl2/library/arith.lisp&quot;</v>.  Then <v>&quot;arith&quot;</v> is no longer \u000A a logical name, because it is ambiguous.  However, <v>&quot;task-1/arith&quot;</v> is a \u000A logical name for the first <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and <v>&quot;library/arith&quot;</v> is \u000A a logical name for the second.  Indeed, the first can be named by \u000A <v>&quot;1/arith&quot;</v> and the second by <v>&quot;y/arith&quot;</v>.</p> \u000A \u000A <p>Logical names are used primarily in the theory manipulation functions, \u000A e.g., <tt><see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see></tt> and <tt><see topic=\"ACL2____CURRENT-THEORY\">current-theory</see></tt> with which you may \u000A obtain some standard <see topic=\"ACL2____THEORIES\">theories</see> as of some point in the historical past. \u000A The reference points are the introductions of logical names, i.e., the past is \u000A determined by the <see topic=\"ACL2____EVENTS\">events</see> it contains.  One might ask, ``Why not discuss \u000A the past with the much more flexible language of <see topic=\"ACL2____COMMAND\">command</see> descriptors?'' \u000A (See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.)  The reason is that inside of such <see topic=\"ACL2____EVENTS\">events</see> as <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or macro <see topic=\"ACL2____COMMAND\">command</see>s that expand to <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>s of <see topic=\"ACL2____EVENTS\">events</see>, <see topic=\"ACL2____COMMAND\">command</see> descriptors provide too coarse a \u000A grain.</p> \u000A \u000A <p>When logical names are used as referents in theory expressions used in \u000A <see topic=\"ACL2____BOOKS\">books</see>, one must consider the possibility that the defining event within \u000A the book in question becomes redundant by the definition of the name prior to \u000A the assumption of the book.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p>"],

 "COMMON-LISP____REAL":[["ACL2"],"ACL2 Sources","ACL2","<p>ACL2 supports rational numbers but not real numbers.  However, \u000A starting with Version 2.5, a variant of ACL2 called ``ACL2(r)'' supports the \u000A real numbers by way of non-standard analysis.  ACL2(r) was conceived and first \u000A implemented by Ruben Gamboa in his Ph.D.  dissertation work, supervised by Bob \u000A Boyer with active participation by Matt Kaufmann.</p> \u000A \u000A <p>ACL2(r) has the same source files as ACL2.  After you download ACL2, you \u000A can build ACL2(r) by executing the following command on the command line in \u000A your acl2-sources directory, replacing <v>&lt;your_lisp&gt;</v> with a path to your \u000A Lisp executable:</p> \u000A \u000A <code>make large-acl2r LISP=&lt;your_lisp&gt;</code> \u000A \u000A <p>This will create an executable in your acl2-sources directory named \u000A <v>saved_acl2r</v>.</p> \u000A \u000A <p>Note that if you download <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> as tarfiles, then you will \u000A automatically be obtaining the books to be certified with ACL2(r).  They can \u000A be certified from your acl2-sources directory, shown here as <v>&lt;DIR&gt;</v>:</p> \u000A \u000A <code>make regression ACL2=&lt;DIR&gt;/saved_acl2r</code> \u000A \u000A <p>To check that you are running ACL2(r), see if the prompt includes the \u000A string ``<v>(r)</v>'', e.g.:</p> \u000A \u000A <code>ACL2(r) !&gt;</code> \u000A \u000A <p>Or, look at <v>(<see topic=\"ACL2_____04\">@</see> acl2-version)</v> and see if ``<v>(r)</v>'' is a \u000A substring.</p> \u000A \u000A <p>In ACL2 (as opposed to ACL2(r)), when we say ``real'' we mean \u000A ``rational.''</p> \u000A \u000A <p>Caution: ACL2(r) should be considered experimental: although we (Kaufmann \u000A and Moore) have carefully completed Gamboa's integration of the reals into the \u000A ACL2 source code, our primary concern has been to ensure unchanged behavior \u000A when ACL2 is compiled in the default manner, i.e., without the non-standard \u000A extensions.  As for every release of ACL2, at the time of a release we are \u000A unaware of soundness bugs in ACL2 or ACL2(r).</p> \u000A \u000A <p>There is only limited documentation on the non-standard features of \u000A ACL2(r).  We hope to provide more documentation for such features in future \u000A releases.  Please feel free to query the authors if you are interested in \u000A learning more about ACL2(r).  Gamboa's dissertation may also be helpful.</p>"],

 "ACL2____STRONG-REWRITE-RULES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Suppose you notice the following term in a Key Checkpoint:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A) (REV B)).</code> \u000A \u000A <p>You might think of two theorems for handling this term, which we'll call \u000A the ``weak'' and ``strong'' version of <v>member-rev</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-rev-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e b)\u000A           (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev b)))).\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-rev-strong\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (rev b))\u000A       (<see topic=\"COMMON-LISP____MEMBER\">member</see> e b))).</code> \u000A \u000A <p>The ``strong'' version logically implies the ``weak'' version and so \u000A deserves the adjective.  (Recall that ``<i>p</i> &lt;---&gt; <i>q</i>'' is \u000A just ``<i>p</i> ---&gt; <i>q</i>'' and ``<i>q</i> ---&gt; <i>p</i>.''  So the \u000A strong version quite literally says everything the weak version does and then \u000A some.)</p> \u000A \u000A <p>But the ``strong'' version also produces a <i>better</i> rewrite rule. \u000A Here are the rules generated from these two formulas, phrased as directives to \u000A ACL2's simplifier:</p> \u000A \u000A <p><v>member-rev-weak</v>: If you ever see an instance of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (REV\u000A b))</v>, backchain to <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e b)</v> (i.e., turn your attention to that term) \u000A and if you can show that it is true, replace <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (REV b))</v> by \u000A <v>T</v>.</p> \u000A \u000A <p><v>member-rev-strong</v>: If you ever see an instance of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (REV\u000A b))</v>, replace it by <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e b)</v>.</p> \u000A \u000A <p>Technical Note: Actually, both rules concern <i>propositional</i> \u000A occurrences of the ``target'' expression, <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (REV b))</v>, i.e., \u000A occurrences of the target in which its truthvalue is the only thing of \u000A relevance.  (Recall that <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> x y)</v> returns the tail of <v>y</v> starting \u000A with <v>x</v>!  Evaluate some simple <v>MEMBER</v> expressions, like <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> 3\u000A '(1 2 3 4 5))</v> to see what we mean.)  Both theorems tell us about \u000A circumstances in which the target is non-<v>NIL</v> (i.e., ``true'') without \u000A telling us its identity.  But ACL2 keeps track of when the target is in a \u000A propositional occurrence and can use such rules to rewrite the target to \u000A propositionally equivalent terms.</p> \u000A \u000A <p>So the strong version is better because it will always eliminate <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see>\u000A e (REV b))</v> in favor of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e b)</v>.  That simpler expression may be \u000A further reduced if the context allows it.  But in any case, the strong version \u000A eliminates <v>REV</v> from the problem.  The weak version only eliminates \u000A <v>REV</v> when a side-condition can be proved.</p> \u000A \u000A <p>While either version may permit us to prove the Key Checkpoint that \u000A ``suggested'' the rule, the strong version is a better rule to have in the \u000A database going forward.</p> \u000A \u000A <p>For example, suppose <v>NATS-BELOW</v> returns the list of natural numbers \u000A below its argument.  Imagine two alternative scenarios in which a key \u000A checkpoint is about to arise involving this term:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> K (REV (NATS-BELOW J)))</code> \u000A \u000A <p><b>Scenario 1</b> is when you've got the strong version in your database: \u000A it will rewrite the key checkpoint term to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> K (NATS-BELOW J))</code> \u000A \u000A <p>and that's what you'll see in the printed checkpoint unless other rules \u000A reduce it further.</p> \u000A \u000A <p><b>Scenario 2</b> is when you have only the weak version in your database: \u000A the weak rule will attempt to reduce the term above to <v>T</v> and will, if \u000A there are sufficient rules and hypotheses about <v>K</v>'s membership in \u000A <v>(NATS-BELOW J)</v>.  But if no such rules or hypotheses are available, you'll \u000A be confronted with a key checkpoint involving</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> K (REV (NATS-BELOW J)))</code> \u000A \u000A <p>and <i>it will not be obvious</i> that the problem would go away if you \u000A could establish that <v>K</v> is in <v>(NATS-BELOW J)</v>.  Clearly, Scenario 1 is \u000A better.</p> \u000A \u000A <p>We recommend that you now practice formulating strong versions of rules \u000A suggested by terms you might see.  See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p> \u000A \u000A <p>When you are finished with that, use your browser's <b>Back Button</b> to \u000A return to <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p>"],

 "ACL2____ACL2-TUTORIAL":[["ACL2"],"ACL2 Sources","ACL2","<p>To learn about ACL2, read at least the following two links.</p> \u000A \u000A <ul> \u000A \u000A <li><see topic='ACL2____INTERESTING-APPLICATIONS'>Industrial Applications of \u000A ACL2</see> (10 minutes) to help you understand what sophisticated users can \u000A do;</li> \u000A \u000A <li><see topic='ACL2____A_02Flying_02Tour_02of_02ACL2'>A Flying Tour</see> (10 \u000A minutes) to get an overview of the system and what skills the user must \u000A have.</li> \u000A \u000A </ul> \u000A \u000A <p>If you want to learn <i>how to use</i> ACL2, we recommend that you read a \u000A selection of the materials referenced below, depending on your learning style, \u000A and do suggested exercises.</p> \u000A \u000A <ul> \u000A \u000A <li><see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'>A Walking \u000A Tour</see> (1 hour) provides an overview of the theorem prover.</li> \u000A \u000A <li>The <a href='http://tryacl2.org'>Try ACL2</a> web site provides \u000A interactive lessons to get you started using ACL2.</li> \u000A \u000A <li>See <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> (10-40 hours) for \u000A instruction on how to interact with the system.  Unlike the three documents \u000A above, this document expects you to <i>think</i>!  It cites the necessary \u000A background pages on programming in ACL2 and on the logic and then instructs \u000A you in <see topic=\"ACL2____THE-METHOD\">the-method</see>, which is how expert users use ACL2.  It concludes \u000A with some challenge problems for the ACL2 beginner (including solutions) and \u000A an FAQ.  Most users will spend several hours a day for several days working \u000A through this material.</li> \u000A \u000A <li>The book <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/acl2-books/car/index.html'>Computer-Aided \u000A Reasoning: An Approach</a> is worth a careful read, as you work exercises and \u000A learn <see topic=\"ACL2____THE-METHOD\">the-method</see>.</li> \u000A \u000A <li><see topic='ACL2____ANNOTATED-ACL2-SCRIPTS'>Annotated ACL2 Scripts and \u000A Demos</see> contains relatively elementary proof scripts that have been \u000A annotated to help train the newcomer.</li> \u000A \u000A <li>Many files (``books'') in the ACL2 community books (see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>) are extensively annotated.</li> \u000A \u000A <li>An <see topic='ACL2____ALTERNATIVE-INTRODUCTION'>Alternative \u000A Introduction</see> document, while largely subsumed by the <see topic='ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER'>Introduction to the Theorem Prover</see> \u000A mentioned above, still might be useful because it covers much of the tutorial \u000A material in a different way.</li> \u000A \u000A </ul> \u000A \u000A <p>At this point you are probably ready to use ACL2 on your own <i>small</i> \u000A projects.  A common mistake for beginners is to browse the documentation and \u000A then try to do something that is too big!  Think of a very small project and \u000A then simplify it!</p> \u000A \u000A <p>Note that ACL2 has a very supportive user network.  See the link to \u000A ``Mailing Lists'' on the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2'>ACL2 home page</a>.</p> \u000A \u000A <p>The topics listed below are a hodge podge, developed over time.  Although \u000A some of these are not mentioned above, you might find some to be useful as \u000A well.</p>"],

 "ACL2-PC____SEQUENCE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(sequence (<see topic=\"ACL2____INDUCT\">induct</see> p prove) t)</code> \u000A \u000A <p>See also the definitions of commands <v>do-all</v>, <v>do-strict</v>, \u000A <v>protect</v>, and <v>succeed</v>.</p> \u000A \u000A <code>General Form:\u000A(sequence\u000A instruction-list\u000A &amp;optional\u000A strict-flg protect-flg success-expr no-prompt-flg no-restore-flg)</code> \u000A \u000A <p>Each instruction in the list <v>instruction-list</v> is run, and the \u000A instruction ``succeeds'' if every instruction in <v>instruction-list</v> \u000A ``succeeds''.  However, it might ``succeed'' even if some instructions in the \u000A list ``fail''; more generally, the various arguments control a number of \u000A aspects of the running of the instructions.  All this is explained in the \u000A paragraphs below.  First we embark on a general discussion of the instruction \u000A interpreter, including the notions of ``succeed'' and ``fail''.</p> \u000A \u000A <p><b>Remark:</b> The arguments are <b>not</b> evaluated, except (in a sense) \u000A for <v>success-expr</v>, as described below.</p> \u000A \u000A <p>Each primitive and meta instruction can be thought of as returning an error \u000A triple, say <v>(erp val state)</v>; see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.  An instruction \u000A (primitive or meta) ``succeeds'' if <v>erp</v> is <v>nil</v> and <v>val</v> is not \u000A <v>nil</v>; otherwise it ``fails''.  (When we use the words ``succeed'' or \u000A ``fail'' in this technical sense, we'll always include them in double quotes.) \u000A If an instruction ``fails,'' we say that that the failure is ``soft'' if \u000A <v>erp</v> is <v>nil</v>; otherwise the failure is ``hard''.  The <v>sequence</v> \u000A command gives the user control over how to treat ``success'' and ``failure'' \u000A when sequencing instructions, though we have created a number of handy macro \u000A commands for this purpose, notably <v>do-all</v>, <v>do-strict</v> and \u000A <v>protect</v>.</p> \u000A \u000A <p>Here is precisely what happens when a <v>sequence</v> instruction is run. \u000A The instruction interpreter is run on the instructions supplied in the \u000A argument <v>instruction-list</v> (in order).  The interpreter halts the first \u000A time there is a hard ``failure.'' except that if <v>strict-flg</v> is supplied \u000A and not <v>nil</v>, then the interpreter halts the first time there is any \u000A ``failure.''  The error triple <v>(erp val state)</v> returned by the \u000A <v>sequence</v> instruction is the triple returned by the last instruction \u000A executed (or, the triple <v>(nil t state)</v> if <v>instruction-list</v> is \u000A <v>nil</v>), except for the following provision.  If <v>success-expr</v> is \u000A supplied and not <v>nil</v>, then it is evaluated with the state global \u000A variables <v>pc-erp</v> and <v>pc-val</v> (in the \"ACL2\" package) bound to the \u000A corresponding components of the error triple returned (as described above). \u000A At least two values should be returned, and the first two of these will be \u000A substituted for <v>erp</v> and <v>val</v> in the triple finally returned by \u000A <v>sequence</v>.  For example, if <v>success-expr</v> is <v>(<see topic=\"ACL2____MV\">mv</see> erp val)</v>, then \u000A no change will be made to the error triple, and if instead it is <v>(<see topic=\"ACL2____MV\">mv</see> nil\u000A t)</v>, then the <v>sequence</v> instruction will ``succeed''.</p> \u000A \u000A <p>That concludes the description of the error triple returned by a \u000A <v>sequence</v> instruction, but it remains to explain the effects of the \u000A arguments <v>protect-flg</v> and <v>no-prompt-flg</v>.</p> \u000A \u000A <p>If <v>protect-flg</v> is supplied and not <v>nil</v> and if also the \u000A instruction ``fails'' (i.e., the error component of the triple is not <v>nil</v> \u000A or the value component is <v>nil</v>), then the state is reverted so that the \u000A proof-checker's state (including the behavior of <v>restore</v>) is set back to \u000A what it was before the <v>sequence</v> instruction was executed.  Otherwise, \u000A unless <v>no-restore-flg</v> is set, the state is changed so that the \u000A <v>restore</v> command will now undo the effect of this <v>sequence</v> \u000A instruction (even if there were nested calls to <v>sequence</v>).</p> \u000A \u000A <p>Finally, as each instruction in <v>instruction-list</v> is executed, the \u000A prompt and that instruction will be printed, unless the global state variable \u000A <v>pc-print-prompt-and-instr-flg</v> is unbound or <v>nil</v> and the parameter \u000A <v>no-prompt-flg</v> is supplied and not <v>nil</v>.</p>"],

 "ACL2____WORMHOLE-EVAL":[["Wormhole"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> 'demo\u000A   '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs)\u000A      (<see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see> whs\u000A                         (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> name info)\u000A                               (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs))))\u000A   (<see topic=\"ACL2____PROG2_42\">prog2$</see> info name))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see> name lambda varterm)</code> \u000A \u000A <p>where <v>name</v> must be a quoted wormhole name and <v>lambda</v> must be a \u000A quoted <v>lambda</v>-expression.  The <v>lambda</v>-expression must have at most \u000A one formal parameter but the body of the <v>lambda</v>-expression may contain \u000A other variables.  Note that in the example form given above, the <v>lambda</v> \u000A has one formal, <v>whs</v>, and uses <v>name</v> and <v>info</v> freely.  Note that \u000A the <v>lambda</v> is quoted.  The third argument of <v>wormhole-eval</v>, \u000A <v>varterm</v>, is an arbitrary term that should mention all of the free \u000A variables in the <v>lambda</v>-expression.  That term establishes your ``right'' \u000A to refer to those free variables in the environment in which the \u000A <v>wormhole-eval</v> expression occurs.  The value of <v>varterm</v> is irrelevant \u000A and if you provide <v>nil</v> ACL2 will automatically provide a suitable term, \u000A namely a <v>prog2$</v> form like the one shown in the example above.</p> \u000A \u000A <p>Aside: Exception for ACL2(p) (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>) to the irrelevance of \u000A <v>varterm</v>.  By default, calls of <v>wormhole-eval</v> employ a lock, \u000A <v>*wormhole-lock*</v>.  To avoid such a lock, include the symbol \u000A <v>:NO-WORMHOLE-LOCK</v> in <v>varterm</v>; for example, you might replace a last \u000A argument of <v>nil</v> in <v>wormhole-eval</v> by <v>:NO-WORMHOLE-LOCK</v>.  End of \u000A Aside.</p> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see> for a full explanation of wormholes.  Most relevant \u000A here is that every wormhole has a name and a status.  The status is generally \u000A a cons pair whose <v>car</v> is the keyword <v>:ENTER</v> or the keyword \u000A <v>:SKIP</v> and whose <v>cdr</v> is an arbitrary object used to store information \u000A from one wormhole call to the next.</p> \u000A \u000A <p>Here is a succinct summary of <v>wormhole-eval</v>.  If the \u000A <v>lambda</v>-expression has a local variable, <v>wormhole-eval</v> applies the \u000A <v>lambda</v>-expression to the wormhole status of the named wormhole and \u000A remembers the value as the new wormhole status.  If the <v>lambda</v> has no \u000A formal parameter, the <v>lambda</v> is applied to no arguments and the value is \u000A the new status.  <v>Wormhole-eval</v> returns <v>nil</v>.  Thus, the formal \u000A parameter of the <v>lambda</v>-expression, if provided, denotes the wormhole's \u000A hidden status information; the value of the <v>lambda</v> is the new status and \u000A is hidden away.</p> \u000A \u000A <p>The guard of a <v>wormhole-eval</v> call is the guard of the body of the \u000A <v>lambda</v>-expression, with a fresh variable symbol used in place of the \u000A formal so that no assumptions are possible about the hidden wormhole status. \u000A If the guard of a <v>wormhole-eval</v> is verified, the call is macroexpanded \u000A inline to the evaluation of the body in a suitable environment.  Thus, it can \u000A be a very fast way to access and change hidden state information, but the \u000A results must remain hidden.  To do arbitrary computations on the hidden state \u000A (i.e., to access the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> or logical <see topic=\"ACL2____WORLD\">world</see> or to interact \u000A with the user) see <see topic=\"ACL2____WORMHOLE\">wormhole</see>.</p> \u000A \u000A <p>Functions that are probably useful in the body of the <tt><see topic=\"COMMON-LISP____LAMBDA\">lambda</see></tt> or the \u000A guard of a function using <v>wormhole-eval</v> include the following: <tt><see topic=\"ACL2____WORMHOLE-STATUSP\">wormhole-statusp</see></tt>, <tt><see topic=\"ACL2____WORMHOLE-ENTRY-CODE\">wormhole-entry-code</see></tt>, <tt><see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see></tt>, <tt><see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see></tt>, <tt><see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see></tt>, and <tt><see topic=\"ACL2____MAKE-WORMHOLE-STATUS\">make-wormhole-status</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see> for a series of example uses of <v>wormhole-eval</v> and \u000A <v>wormhole</v>.</p> \u000A \u000A <p>For a behind-the-scenes description of how wormholes work, See <see topic=\"ACL2____WORMHOLE-IMPLEMENTATION\">wormhole-implementation</see>.</p>"],

 "ACL2____DEFCONST":[["Events","Programming"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *digits* '(0 1 2 3 4 5 6 7 8 9))\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *n-digits* (<see topic=\"COMMON-LISP____THE\">the</see> unsigned-byte (<see topic=\"COMMON-LISP____LENGTH\">length</see> *digits*)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> name term doc-string)</code> \u000A \u000A <p>where <v>name</v> is a symbol beginning and ending with the character <v>*</v>, \u000A <v>term</v> is a variable-free term that is evaluated to determine the value of \u000A the constant, and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).</p> \u000A \u000A <p>When a constant symbol is used as a <see topic=\"ACL2____TERM\">term</see>, ACL2 replaces it by its \u000A value; see <see topic=\"ACL2____TERM\">term</see>.</p> \u000A \u000A <p>Note that <v>defconst</v> uses a ``safe mode'' to evaluate its form, in order \u000A to avoids soundness issues but with an efficiency penalty (perhaps increasing \u000A the evaluation time by several hundred percent).  If efficiency is a concern, \u000A or if for some reason you need the form to be evaluated without safe mode \u000A (e.g., you are an advanced system hacker using trust tags to traffic in raw \u000A Lisp code), consider using <see topic=\"ACL2____DEFCONSTS\">defconsts</see> instead.  Also see <see topic=\"ACL2____USING-TABLES-EFFICIENTLY\">using-tables-efficiently</see> for an analogous issue with <tt><see topic=\"ACL2____TABLE\">table</see></tt> \u000A events.</p> \u000A \u000A <p>It may be of interest to note that <v>defconst</v> is implemented at the lisp \u000A level using <v>defparameter</v>, as opposed to <v>defconstant</v>. \u000A (Implementation note: this is important for proper support of undoing and \u000A redefinition.)</p> \u000A \u000A <p>We close with a technical remark, perhaps of interest only to users of \u000A ACL2(h), the experimental extension of ACL2 that supports hash cons, function \u000A memoization, and hash-table-based ``fast alists''; see <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  For an event of the form <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *C* (<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A OBJ))</v>, i.e., <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *C* 'OBJ)</v>, then the value associated with \u000A <v>*C*</v> is <v>OBJ</v>; that is, the value of <v>*C*</v> is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> to the \u000A actual object <v>OBJ</v> occurring in the <v>defconst</v> form.  So for example, \u000A if <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> is used to generate such a <v>defconst</v> event, as it is \u000A in the two books mentioned above, and <v>OBJ</v> is a fast alist (using \u000A ACL2(h)), then the value of <v>*C*</v> is a fast alist.  This guarantee \u000A disappears if the term in the <v>defconst</v> form is not a quoted object, i.e., \u000A if it is not of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> OBJ)</v>.</p>"],

 "ACL2____ADD-MATCH-FREE-OVERRIDE":[["Free-variables"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t)\u000A    ; Try only the first binding of free variables when relieving hypotheses\u000A    ; of any rule of class :rewrite, :linear, or :forward-chaining.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all (:rewrite foo) (:rewrite bar))\u000A    ; For rewrite rules foo and bar, try all bindings of free variables when\u000A    ; relieving hypotheses.\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :clear)\u000A    ; Restore :match-free to what was originally stored for each rule (either\u000A    ; :all or :once).</code> \u000A \u000A <p>As described elsewhere (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), a <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rule may have free variables in its \u000A hypotheses, and ACL2 can be directed either to try all bindings \u000A (``<v>:all</v>'') or just the first (``<v>:once</v>'') when relieving a \u000A hypothesis, as a basis for relieving subsequent hypotheses.  This direction is \u000A generally provided by specifying either <v>:match-free :once</v> or \u000A <v>:match-free :all</v> in the <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> of the rule, or by \u000A using the most recent <tt><see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see></tt> event.  Also see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p> \u000A \u000A <p>However, if a proof is going slowly, you may want to modify the behavior of \u000A some such rules so that they use only the first match for free variables in a \u000A hypothesis when relieving subsequent hypotheses, rather than backtracking and \u000A trying additional matches as necessary.  (But note: \u000A <v>add-match-free-override</v> is not relevant for <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> \u000A rules.)  The event <v>(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t)</v> has that effect.  Or \u000A at the other extreme, perhaps you want to specify all rules as <v>:all</v> rules \u000A except for a some specific exceptions.  Then you can execute \u000A <v>(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all t)</v> followed by, say, \u000A <v>(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once (:rewrite foo) (:linear bar))</v>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :clear)\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> flg t)\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> flg rune1 rune2 ... runek)</code> \u000A \u000A <p>where <v>flg</v> is <v>:once</v> or <v>:all</v> and the <v>runei</v> are <tt><see topic=\"ACL2____RUNE\">rune</see></tt>s.  If <v>:clear</v> is specified then all rules will have the \u000A <v>:all</v>/<v>:once</v> behavior from when they were first stored.  The second \u000A general form causes all <see topic=\"ACL2____REWRITE\">rewrite</see> <see topic=\"ACL2____LINEAR\">linear</see>, and <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules to have the behavior specified by <v>flg</v> (<v>:all</v> \u000A or <v>:once</v>).  Finally, the last of these, where runes are specified, is \u000A additive in the sense that only the indicated rules are affected; all others \u000A keep the behavior they had just before this event was executed (possible \u000A because of earlier <v>add-match-free-override</v> events).</p> \u000A \u000A <p>At the conclusion of this event, ACL2 prints out the list of all \u000A <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> runes whose rules contain free variables in hypotheses that \u000A are to be bound <v>:once</v>, except that if there are no overrides (value \u000A <v>:clear</v> was used), then <v>:clear</v> is printed.</p> \u000A \u000A <p>This event only affects rules that exist at the time it is executed. \u000A Future rules are not affected by the override.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It \u000A uses the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to \u000A the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs.</p> \u000A \u000A <p><i>Remarks</i></p> \u000A \u000A <p>Lists of the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, and <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s whose behavior was originally <v>:once</v> or \u000A <v>:all</v> are returned by the following forms, respectively.</p> \u000A \u000A <code>(free-var-runes :once (w state))\u000A(free-var-runes :all  (w state))</code> \u000A \u000A <p>The form</p> \u000A \u000A <code>(match-free-override (w state))</code> \u000A \u000A <p>evaluates to a pair, whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is a number used by ACL2 to \u000A determine whether a <see topic=\"ACL2____RUNE\">rune</see> is sufficiently old to be affected by the \u000A override, and whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is the list of <see topic=\"ACL2____RUNE\">rune</see>s whose behavior is \u000A specified as <v>:once</v> by <v>add-match-free-override</v>; except, if no runes \u000A have been overridden, then the keyword <v>:clear</v> is returned.</p>"],

 "COMMON-LISP____DEFMACRO":[["Macros","Events"],"ACL2 Sources","ACL2","<code>Example Defmacros:\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> xor (x y)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'if x (<see topic=\"COMMON-LISP____LIST\">list</see> 'not y) y))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> git (sym key)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'getprop sym key nil\u000A        '(<see topic=\"COMMON-LISP____QUOTE\">quote</see> current-acl2-world)\u000A        '(w state)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> one-of (x &amp;rest rst)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> rst)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> rst) nil)\u000A        (t (<see topic=\"COMMON-LISP____LIST\">list</see> 'or\u000A                 (<see topic=\"COMMON-LISP____LIST\">list</see> 'eq x (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____CAR\">car</see> rst)))\u000A                 (<see topic=\"COMMON-LISP____LIST_A2\">list*</see> 'one-of x (<see topic=\"COMMON-LISP____CDR\">cdr</see> rst))))))\u000A\u000AExample Expansions:\u000Aterm                    macroexpansion\u000A\u000A(<see topic=\"ACL2____XOR\">xor</see> a b)              (<see topic=\"COMMON-LISP____IF\">if</see> a (<see topic=\"COMMON-LISP____NOT\">not</see> b) b)\u000A(<see topic=\"ACL2____XOR\">xor</see> a (foo b))        (<see topic=\"COMMON-LISP____IF\">if</see> a (<see topic=\"COMMON-LISP____NOT\">not</see> (foo b)) (foo b))\u000A\u000A(git 'car 'lemmas)     (<see topic=\"ACL2____GETPROP\">getprop</see> 'car 'lemmas nil\u000A                                'current-acl2-world\u000A                                (w state))\u000A\u000A(one-of x a b c)       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'a)\u000A                           (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'b)\u000A                               (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'c) nil)))\u000A\u000A(one-of x 1 2 3)       ill-formed (<see topic=\"ACL2____GUARD\">guard</see> violation)\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> name macro-args doc-string dcl ... dcl body)</code> \u000A \u000A <p>where <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>), <v>macro-args</v> \u000A specifies the formal parameters of the macro, and <v>body</v> is a term.  The \u000A formal parameters can be specified in a much more general way than is allowed \u000A by ACL2 <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____MACRO-ARGS\">macro-args</see> for a description \u000A of keyword (<v>&amp;key</v>) and optional (<v>&amp;optional</v>) parameters as well as \u000A other so-called ``lambda-list keywords'', <v>&amp;rest</v> and <v>&amp;whole</v>.  <tt><see topic=\"ACL2____DOC-STRING\">Doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>. \u000A Each <v>dcl</v> is an optional declaration (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>) except that the \u000A only <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> keyword permitted by <v>defmacro</v> is <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt>.</p> \u000A \u000A <p>For compute-intensive applications see the community book \u000A <v>misc/defmac.lisp</v>, which can speed up macroexpansion by introducing an \u000A auxiliary <v>defun</v>.  For more information, evaluate the form \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/defmac&quot; :dir :system)</v> and then evaluate <v>:doc\u000A defmac</v>.</p> \u000A \u000A <p>Macroexpansion occurs when a form is read in, i.e., before the evaluation \u000A or proof of that form is undertaken.  To experiment with macroexpansion, see \u000A <see topic=\"ACL2____TRANS\">trans</see>.  When a form whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is <v>name</v> arises as the form \u000A is read in, the arguments are bound as described in CLTL pp. 60 and 145, the \u000A <see topic=\"ACL2____GUARD\">guard</see> is checked, and then the <v>body</v> is evaluated.  The result is \u000A used in place of the original form.</p> \u000A \u000A <p>In ACL2, macros do not have access to the ACL2 state, <tt><see topic=\"ACL2____STATE\">state</see></tt>.  (If \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> or any user-defined stobj (see <see topic=\"ACL2____STOBJ\">stobj</see>) is a macro \u000A argument, it is treated as an ordinary variable, bound at macro-expansion time \u000A to a piece of syntax.)  This is in part a reflection of CLTL, p. 143, ``More \u000A generally, an implementation of Common Lisp has great latitude in deciding \u000A exactly when to expand macro calls with a program. ...  Macros should be \u000A written in such a way as to depend as little as possible on the execution \u000A environment to produce a correct expansion.''  In ACL2, the product of \u000A macroexpansion is independent of the current environment and is determined \u000A entirely by the macro body and the functions and constants it references.  It \u000A is possible, however, to define macros that produce expansions that refer to \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> or other single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see>) or variables \u000A not among the macro's arguments.  See the <v>git</v> example above.  For a \u000A related utility that does have access to the ACL2 <see topic=\"ACL2____STATE\">state</see>, see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.</p>"],

 "ACL2____REMOVE-UNTOUCHABLE":[["Defttag"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> my-var nil) ; then state global my-var is not untouchable\u000A(<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> set-mem t)  ; then function set-mem is not untouchable</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>.</p> \u000A \u000A <p>This documentation topic is directed at those who build systems on top of \u000A ACL2.  We first describe a means for removing restrictions related to \u000A so-called ``untouchables'': functions (or macros) that cannot be called, or \u000A state global variables that cannot be modified or unbound, without \u000A intervention that requires an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  Then we \u000A describe the <v>remove-untouchable</v> event.</p> \u000A \u000A <p>We begin by discussing untouchable state global variables \u000A <v>temp-touchable-vars</v> and <v>temp-touchable-fns</v>, which initially have \u000A value <v>nil</v>.  These can often be used in place of <v>remove-untouchable</v>. \u000A When the value is <v>t</v>, no variable (respectively, no function or macro) is \u000A treated as untouchable, regardless of the set of initial untouchables or the \u000A <v>remove-untouchable</v> or <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt> <see topic=\"ACL2____EVENTS\">events</see> that have \u000A been admitted.  Otherwise the value of each of these two variables is a <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>, and no member of this list is treated as an untouchable \u000A variable (in the case of <v>temp-touchable-vars</v>) or as an untouchable \u000A function or macro (in the case of <v>temp-touchable-fns</v>).  These two state \u000A global variables can be set by <v>set-temp-touchable-vars</v> and \u000A <v>set-temp-touchable-fns</v>, respectively, provided there is an active trust \u000A tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  Here is an illustrative example.  This macro \u000A executes the indicated forms in a context where there are no untouchable \u000A variables, but requires an active trust tag when invoked.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> with-all-touchable (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> forms)\u000A  `(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A    :state-global-bindings\u000A    ((temp-touchable-vars t set-temp-touchable-vars))\u000A    (<see topic=\"ACL2____PROGN_12\">progn!</see> ,@forms)))</code> \u000A \u000A <p>An equivalent version, which however is not recommended since <tt><see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see></tt> may have surprising behavior in raw Lisp, is as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> with-all-touchable (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> forms)\u000A  `(<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A    (<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A     ((temp-touchable-vars t set-temp-touchable-vars))\u000A     (<see topic=\"ACL2____PROGN_12\">progn!</see> ,@forms))))</code> \u000A \u000A <p>Finally, the value <v>t</v> for <v>temp-touchable-vars</v> removes the \u000A requirement that built-in state globals cannot be made unbound (with \u000A <v>makunbound-global</v>).</p> \u000A \u000A <p>We now turn to the <v>remove-untouchable</v> event, in case the approach \u000A above is for some reason not adequate.  This event is illegal by default, \u000A since it can be used to provide access to ACL2 internal functions and data \u000A structures that are intentionally made untouchable for the user.  If you want \u000A to call it, you must first create an active trust tag; see <see topic=\"ACL2____DEFTTAG\">defttag</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> name{s}  fn-p :doc doc-string)</code> \u000A \u000A <p>where <v>name{s}</v> is a non-<v>nil</v> symbol or a non-<v>nil</v> true list of \u000A symbols, <v>fn-p</v> is any value (but generally <v>nil</v> or <v>t</v>), and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string not beginning with \u000A ``<v>:Doc-Section</v> ...''.  If <v>name{s}</v> is a symbol it is treated as the \u000A singleton list containing that symbol.  The effect of this event is to remove \u000A the given symbols from the list of ``untouchable variables'' in the current \u000A world if <v>fn-p</v> is <v>nil</v>, else to remove the symbols into the list of \u000A ``untouchable functions''.  This event is redundant if no symbol listed is a \u000A member of the appropriate untouchables list (variables or functions).</p>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-3":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (FOO A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (BAR B) (MUM C)))</code> \u000A \u000A <p>Since <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> contains all the members of <v>x</v> and all the \u000A members of <v>y</v>, <v>e</v> is a member of <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> precisely when \u000A <v>e</v> is a member of <v>x</v> or of <v>y</v>.  So a strong statement of this \u000A is:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append-strong-false\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A         (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A             (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y))))</code> \u000A \u000A <p>However, this is not a theorem because <v>member</v> is not Boolean. \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">Member</see> e x)</v>, for example, returns the first tail of <v>x</v> that starts \u000A with <v>e</v>, or else <v>nil</v>.  To see an example of this formula that \u000A evaluates to <v>nil</v>, let</p> \u000A \u000A <code>e = 3\u000Ax = '(1 2 3)\u000Ay = '(4 5 6).</code> \u000A \u000A <p>Then the left-hand side, <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))</v> evaluates to <v>(3 4\u000A 5 6)</v> while the right-hand side evaluates to <v>(3)</v>.</p> \u000A \u000A <p>However, the two sides are propositionally equivalent (both either <v>nil</v> \u000A or non-<v>nil</v> together).  So this is a useful :<tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append-strong\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A           (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y)))).</code> \u000A \u000A <p>It tells the system that whenever it encounters an instance of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e\u000A (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y))</v> in a propositional occurrence (where only its truthvalue is \u000A relevant), it should be replaced by this disjunction of <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e x)</v> and \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e y)</v>.</p> \u000A \u000A <p>The following two formulas are true but provide much weaker rules and we \u000A would not add them:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)))\u000A\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y) (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y)))</code> \u000A \u000A <p>because they each cause the system to backchain upon seeing <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e\u000A (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y))</v> expressions and will not apply unless one of the two \u000A side-conditions can be established.</p> \u000A \u000A <p>There is a rewrite rule that is even stronger than \u000A <v>member-append-strong</v>.  It is suggested by the counterexample, above, for \u000A the <v>EQUAL</v> version of the rule.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append-really-strong\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A             (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) y)\u000A             (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y))))</code> \u000A \u000A <p>While <v>member-append-strong</v> only rewrites <v>member-append</v> \u000A expressions occurring propositionally, the <v>-really-strong</v> version \u000A rewrites <i>every</i> occurrence.</p> \u000A \u000A <p>However, this rule will be more useful than <v>member-append-strong</v> only \u000A if you have occurrences of <v>member</v> in non-propositional places.  For \u000A example, suppose you encountered a term like:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CONS\">CONS</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y)) z).</code> \u000A \u000A <p>Then the <v>-strong</v> rule does not apply but the <v>-really-strong</v> rule \u000A does.</p> \u000A \u000A <p>Furthermore, the <v>-really-strong</v> rule, by itself, is not quite as good \u000A as the <v>-strong</v> rule in propositional settings!  For example, if you have \u000A proved the <v>-really-strong</v> rule, you'll notice that the system still has \u000A to use induction to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A)\u000A         (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> B A))).</code> \u000A \u000A <p>The <v>-really-strong</v> rule would rewrite it to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A)\u000A         (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A)\u000A             (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A) B)\u000A             (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E B)))</code> \u000A \u000A <p>which would further simplify to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A)\u000A         (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> E A) B))</code> \u000A \u000A <p>What lemma does this suggest?  The answer is the rather odd:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))</code> \u000A \u000A <p>which rewrites propositional occurrences of <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y)</v> to <v>T</v> if \u000A <v>x</v> is non-<v>nil</v>.  This is an inductive fact about <v>append</v>.</p> \u000A \u000A <p>A problem with the <v>-really-strong</v> rule is that it transforms even \u000A propositional occurrences of <v>member</v> into mixed propositional and \u000A non-propositional occurrences.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append-really-strong\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))      ; &lt;-- even if this is a propositional occurrence\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x)\u000A             (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x) y)  ; &lt;-- the member in here is not!\u000A             (<see topic=\"COMMON-LISP____MEMBER\">member</see> e y))))</code> \u000A \u000A <p>So if you are using the <v>-really-strong</v> lemma in a situation in which \u000A all your <v>member</v> expressions are used propositionally, you'll suddenly \u000A find yourself confronted with non-propositional uses of <v>member</v>.</p> \u000A \u000A <p>Our advice is not to use the <v>-really-strong</v> version unless your \u000A application is inherently using <v>member</v> in a non-propositional way.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-1":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (FOO A) (BAR B)))</code> \u000A \u000A <p>Obviously, you must think about the conditions under which <v>(<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x\u000A y)</v> returns a true-list.  Recall that <v>APPEND</v> concatentates <v>x</v> and \u000A <v>y</v>, with <v>y</v> being the terminal sublist.  Its definition is equivalent \u000A to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> append (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      y\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A            (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))</code> \u000A \u000A <p>Technical Note: <v>Append</v> is really a macro that permits you to write \u000A calls of <v>append</v> with more than two arguments.</p> \u000A \u000A <p>In a sense, <v>append</v> ``expects'' its arguments to be lists ending in \u000A <v>nil</v>, so-called <v>true-listp</v>s.  (Such expectations are formalized in \u000A ACL2 by the notion of the ``<see topic=\"ACL2____GUARD\">guard</see>'' <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A of the function, but we strongly recommend not investigating guards until \u000A you're good at using the system.)</p> \u000A \u000A <p>New users frequently start every new theorem by listing all their \u000A expectations on the arguments of functions in the problem.  For example, if \u000A the new user wants to make some statement about when <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> is a \u000A <v>true-listp</v>, it is not uncommon for him or her first to write:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))\u000A         ...)</code> \u000A \u000A <p>to get ``comfortable.''  Then, thinking about when <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> is a \u000A <v>true-listp</v> is easy: it always returns a <v>true-listp</v>.  It's always a \u000A <v>true-listp</v>.''  This thinking produces the theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-append-really-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A                (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))))</code> \u000A \u000A <p>You'll note we gave it a name suggesting it is ``really weak.''</p> \u000A \u000A <p>One sense in which it is weak is that it has an unnecessary hypothesis.  If \u000A <v>y</v> is a <v>true-listp</v>, then <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> is too, whether <v>x</v> is \u000A a <v>true-listp</v> or not.  In ACL2, all functions are total.  Logically \u000A speaking, it doesn't matter whether <v>endp</v> expects its argument to be a \u000A <v>true-listp</v> or not, it behaves.  <v>(<see topic=\"COMMON-LISP____APPEND\">Append</see> x y)</v> either returns <v>y</v> \u000A or a <v>cons</v> whose second argument is generated by <v>append</v>.  Thus, if \u000A <v>y</v> is a <v>true-listp</v>, the answer is too.  So here is an improved \u000A version of the rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-append-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))))</code> \u000A \u000A <p>We still think of it as ``weak'' because it has a hypothesis that limits \u000A its applicability.</p> \u000A \u000A <p>The strong version of the rule is</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-append-strong\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A         (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))).</code> \u000A \u000A <p>That is, <v>append</v> returns a <v>true-listp</v> <i>precisely</i> when its \u000A second argument is a <v>true-listp</v>.  We recommend that the strong version be \u000A made a :<tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> rule.</p> \u000A \u000A <p>The weak version of the rule allows us to reduce <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x\u000A y))</v> to true if we can show that <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> y)</v> is true.  But suppose \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> y)</v> is actually false.  Then <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> x y))</v> \u000A would not simplify under the weak version of the rule.  But under the strong \u000A version it would simplify to <v>NIL</v>.</p> \u000A \u000A <p>Technical Note: The weak version of the rule is a useful :<tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> rule.  The type mechanism cannot currently exploit \u000A the strong version of the rule.</p> \u000A \u000A <p>The strategy of ``getting comfortable'' by adding a bunch of hypotheses \u000A before you know you need them is not conducive to creating strong rules.  We \u000A tend to state the main relationship that we intuit about some function and \u000A then add the hypotheses we need to make it true.  In this case, there were no \u000A necessary hypotheses.  But if there are, we first identify them and then we \u000A ask ``what can I say about the function if these hypotheses aren't true?'' \u000A and try to strengthen the statement still further.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____LD-POST-EVAL-PRINT":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-post-eval-print</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>). \u000A The accessor is <v>(<see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see> state)</v> and the updater is \u000A <v>(set-ld-post-eval-print val state)</v>.  <v>Ld-post-eval-print</v> must be \u000A either <v>t</v>, <v>nil</v>, or <v>:command-conventions</v>.  The initial value of \u000A <v>ld-post-eval-print</v> is <v>:command-conventions</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-post-eval-print</v> is one of them.  If this global variable is <v>t</v>, \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the result.  In the case of a form that produces a multiple \u000A value, <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the list containing all the values (which, logically \u000A speaking, is what the form returned).  If <v>ld-post-eval-print</v> is <v>nil</v>, \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> does not print the values.  This is most useful when <tt><see topic=\"ACL2____LD\">ld</see></tt> is \u000A used to load a previously processed file.</p> \u000A \u000A <p>Finally, if <v>ld-post-eval-print</v> is <v>:command-conventions</v> and also \u000A <v>ld-error-triples</v> is <v>t</v>, then <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the result but treats \u000A ``error triples'' specially.  An ``error triple'' (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) \u000A is a result, <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>, that consists of two ordinary values and \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Many ACL2 functions use such triples to signal errors.  The \u000A convention is that if <v>erp</v> (the first value) is <v>nil</v>, then the \u000A function is returning <v>val</v> (the second value) as its conventional single \u000A result and possibly side-effecting <see topic=\"ACL2____STATE\">state</see> (as with some output).  If \u000A <v>erp</v> is not <v>nil</v>, then an error has been caused, <v>val</v> is \u000A irrelevant and the error message has been printed in the returned <see topic=\"ACL2____STATE\">state</see>.  Example ACL2 functions that follow this convention include <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt>.  If such ``error producing'' functions are \u000A evaluated while <v>ld-post-eval-print</v> is set to <v>t</v>, then you would see \u000A them producing lists of length 3.  This is disconcerting to users accustomed \u000A to Common Lisp (where these functions produce single results but sometimes \u000A cause errors or side-effect <see topic=\"ACL2____STATE\">state</see>).  For more information about error \u000A triples, see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.</p> \u000A \u000A <p>When <v>ld-post-eval-print</v> is <v>:command-conventions</v> and a form \u000A produces an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> as its value, <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A prints nothing if <v>erp</v> is non-<v>nil</v> and otherwise <tt><see topic=\"ACL2____LD\">ld</see></tt> prints \u000A just <v>val</v>.  Because it is a misrepresentation to suggest that just one \u000A result was returned, <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the value of the global variable \u000A <v>'triple-print-prefix</v> before printing <v>val</v>.  <v>'triple-print-prefix</v> \u000A is initially <v>&quot; &quot;</v>, which means that when non-erroneous error triples are \u000A being abbreviated to <v>val</v>, <v>val</v> appears one space off the left margin \u000A instead of on the margin.</p> \u000A \u000A <p>In addition, when <v>ld-post-eval-print</v> is <v>:command-conventions</v> and \u000A the value component of an error triple is the keyword <v>:invisible</v> then \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> prints nothing.  This is the way certain commands (e.g., \u000A <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt>) appear to return no value.</p> \u000A \u000A <p>By printing nothing when an error has been signalled, <tt><see topic=\"ACL2____LD\">ld</see></tt> makes it \u000A appear that the error (whose message has already appeared in <see topic=\"ACL2____STATE\">state</see>) has \u000A ``thrown'' the computation back to load without returning a value.  By \u000A printing just <v>val</v> otherwise, we suppress the fact that <see topic=\"ACL2____STATE\">state</see> has \u000A possibly been changed.</p>"],

 "COMMON-LISP____APPEND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Append</v>, which takes zero or more arguments, expects all the \u000A arguments except perhaps the last to be true (null-terminated) lists.  It \u000A returns the result of concatenating all the elements of all the given lists \u000A into a single list.  Actually, in ACL2 <v>append</v> is a macro that expands \u000A into calls of the binary function <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> if there are at least \u000A two arguments; if there is just one argument then the expansion is that \u000A argument; and finally, <v>(<see topic=\"COMMON-LISP____APPEND\">append</see>)</v> expands to <v>nil</v>.</p> \u000A \u000A <p><v>Append</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>append</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> append (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> rst)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> rst) nil)\u000A                ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> rst)) (<see topic=\"COMMON-LISP____CAR\">car</see> rst))\u000A                (t (xxxjoin 'binary-append rst))))</code> \u000A <p><b>Function: </b><srclink>binary-append</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-append (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) y)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                      (<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))))</code>"],

 "ACL2____SYS-CALL":[["Interfacing-tools","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;cp&quot; '(&quot;foo.lisp&quot; &quot;foo-copied.lisp&quot;))\u000A(<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;cp&quot; '(&quot;foo.lisp&quot; &quot;foo-copied.lisp&quot;))\u000A        (<see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see> state))</code> \u000A \u000A <p>The first argument of <v>sys-call</v> is a command for the host operating \u000A system, and the second argument is a list of strings that are the arguments \u000A for that command.  In GCL and perhaps some other lisps, you can put the \u000A arguments with the command; but this is not the case, for example, in Allegro \u000A CL running on Linux.</p> \u000A \u000A <p>For a related utility, see <see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see>.</p> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> above is optional, but illustrates a typical sort \u000A of use when one wishes to get the return status.  See <see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SYS-CALL\">sys-call</see> cmd args)</code> \u000A \u000A <p>This function logically returns <v>nil</v>.  However, it makes the indicated \u000A call to the host operating system, as described above, using a function \u000A supplied ``under the hood'' by the underlying Lisp system.  On occasions where \u000A one wishes to obtain the numeric status returned by the host operating system \u000A (or more precisely, by the Lisp function under the hood that passes the system \u000A call to the host operating system), one may do so; see <see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see>. \u000A The status value is the value returned by that Lisp function, which may well \u000A be the same numeric value returned by the host operating system for the \u000A underlying system call.</p> \u000A \u000A <p>Note that <v>sys-call</v> does not touch the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>; however, \u000A <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt> updates the <v>acl2-oracle</v> field of the \u000A <v>state</v>.</p> \u000A \u000A <p>Be careful if you use <v>sys-call</v>!  It can be used for example to \u000A overwrite files, or worse!  We view a use of <v>sys-call</v> as a call to the \u000A operating system that is made outside ACL2.  The following example from Bob \u000A Boyer shows how to use <v>sys-call</v> to execute, in effect, arbitrary Lisp \u000A forms.  ACL2 provides a ``trust tag'' mechanism that requires execution of a \u000A <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> form before you can use <v>sys-call</v>; see <see topic=\"ACL2____DEFTTAG\">defttag</see>. \u000A (Note: The setting of the raw Lisp variable <v>*features*</v> below is just to \u000A illustrate that any such mischief is possible.  Normally <v>*features*</v> is a \u000A list with more than a few elements.)</p> \u000A \u000A <code>% cat foo\u000Aprint *0x85d2064=0x838E920\u000Adetach\u000Aq\u000A% acl2\u000A... boilerplate deleted\u000AACL2 !&gt;(<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;gdb -p $PPID -w &lt; foo &gt;&amp; /dev/null &quot; nil)\u000ANIL\u000AACL2 !&gt;:q\u000A\u000AExiting the ACL2 read-eval-print loop.  To re-enter, execute (<see topic=\"ACL2____LP\">LP</see>).\u000AACL2&gt;*features*\u000A\u000A(:AKCL-SET-MV)\u000A\u000AACL2&gt;</code> \u000A \u000A <p>Finally, we make a comment about output redirection, which also applies to \u000A other related features that one may expect of a shell.  <v>Sys-call</v> does not \u000A directly support output redirection.  If you want to run a program, <v>P</v>, \u000A and redirect its output, one option is to create a wrapper script, <v>W</v> to \u000A call instead.  Thus <v>W</v> might be a shell script containing the line:</p> \u000A \u000A <code>P $* &gt;&amp; foo.out</code> \u000A \u000A <p>For a different, more direct solution, see <see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see>.</p>"],

 "ACL2____MBT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The macro <v>mbt</v> (``must be true'') can be used in order to add \u000A code in order to admit function definitions in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, \u000A without paying a cost in execution efficiency.  Examples below illustrate its \u000A intended use.</p> \u000A \u000A <p>Semantically, <v>(<see topic=\"ACL2____MBT\">mbt</see> x)</v> equals <v>x</v>.  However, in raw Lisp <v>(<see topic=\"ACL2____MBT\">mbt</see>\u000A x)</v> ignores <v>x</v> entirely, and macroexpands to <v>t</v>.  ACL2's <see topic=\"ACL2____GUARD\">guard</see> \u000A verification mechanism ensures that the raw Lisp code is only evaluated when \u000A appropriate, since a guard proof obligation <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x t)</v> is generated. \u000A See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> and, for general discussion of guards, see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____MBE\">mbe</see>, which stands for ``must be equal.''  Although <v>mbt</v> \u000A is more natural in many cases, <v>mbe</v> has more general applicability.  In \u000A fact, <v>(<see topic=\"ACL2____MBT\">mbt</see> x)</v> is essentially defined to be <v>(<see topic=\"ACL2____MBE\">mbe</see> :logic x :exec\u000A t)</v>.</p> \u000A \u000A <p>We can illustrate the use of <v>mbt</v> on the following generic example, \u000A where <v>&lt;g&gt;</v>, <v>&lt;test&gt;</v>, <v>&lt;rec-x&gt;</v>, and <v>&lt;base&gt;</v> are intended to be \u000A terms involving only the variable <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard &lt;g&gt;))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> &lt;test&gt;\u000A      (foo &lt;rec-x&gt;)\u000A    &lt;base&gt;))</code> \u000A \u000A <p>In order to admit this function, ACL2 needs to discharge the proof \u000A obligation that <v>&lt;rec-x&gt;</v> is smaller than <v>x</v>, namely:</p> \u000A \u000A <code>(implies &lt;test&gt; \u000A         (o&lt; (acl2-count <v>&lt;rec-x&gt;</v>) \u000A              (acl2-count x))) \u000A</code> \u000A \u000A <p>But suppose we need to know that <v>&lt;g&gt;</v> is true in order to prove this. \u000A Since <v>&lt;g&gt;</v> is only the <see topic=\"ACL2____GUARD\">guard</see>, it is not part of the logical \u000A definition of <v>foo</v>.  A solution is to add the guard to the definition of \u000A <v>foo</v>, as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard &lt;g&gt;))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____MBT\">mbt</see> &lt;g&gt;)\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> &lt;test&gt;\u000A          (foo &lt;rec-x&gt;)\u000A        &lt;base&gt;)\u000A    nil))</code> \u000A \u000A <p>If we do this using <v>&lt;g&gt;</v> rather than <v>(<see topic=\"ACL2____MBT\">mbt</see> &lt;g&gt;)</v>, then evaluation of \u000A every recursive call of <v>foo</v> will cause the evaluation of (the appropriate \u000A instance of) <v>&lt;g&gt;</v>.  But since <v>(<see topic=\"ACL2____MBT\">mbt</see> &lt;g&gt;)</v> expands to <v>t</v> in raw \u000A Lisp, then once we verify the guards of <v>foo</v>, the evaluations of <v>&lt;g&gt;</v> \u000A will be avoided (except at the top level, when we check the guard before \u000A allowing evaluation to take place in Common Lisp).</p> \u000A \u000A <p>Other times, the guard isn't the issue, but rather, the problem is that a \u000A recursive call has an argument that itself is a recursive call.  For example, \u000A suppose that <v>&lt;rec-x&gt;</v> is of the form <v>(foo &lt;expr&gt;)</v>.  There is no way \u000A we can hope to discharge the termination proof obligation shown above.  A \u000A standard solution is to add some version of this test:</p> \u000A \u000A <code>(mbt (o&lt; (acl2-count <v>&lt;rec-x&gt;</v>) (acl2-count x))) </code> \u000A \u000A <p>Here is a specific example based on one sent by Vernon Austel.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> recurX2 (n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n))\u000A                  :verify-guards nil))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> n) 0)\u000A        (t (<see topic=\"COMMON-LISP____LET\">let</see> ((call (recurX2 (<see topic=\"COMMON-LISP____1-\">1-</see> n))))\u000A             (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____MBT\">mbt</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> call) n))\u000A                 (recurX2 call)\u000A               1 ;; this branch is never actually taken\u000A               )))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> recurX2-0\u000A (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (recurX2 n) 0))\u000A\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> recurX2)</code> \u000A \u000A <p>If you <v>(</v><tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt><v> acl2-count)</v>, you will see that evaluation \u000A of <v>(recurX2 2)</v> causes several calls of <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> before the \u000A <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>.  But this evaluation does not call <v>acl2-count</v> \u000A after the <v>verify-guards</v>, because the ACL2 evaluation mechanism uses raw \u000A Lisp to do the evaluation, and the form <v>(<see topic=\"ACL2____MBT\">mbt</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> call) n))</v> \u000A macroexpands to <v>t</v> in Common Lisp.</p> \u000A \u000A <p>You may occasionally get warnings when you compile functions defined using \u000A <v>mbt</v>.  (For commands that invoke the compiler, see <see topic=\"ACL2____COMPILATION\">compilation</see>.) \u000A These can be inhibited by using an <v>ignorable</v> <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form.  Here \u000A is a simple but illustrative example.  Note that the declarations can \u000A optionally be separated into two <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORABLE\">ignorable</see> x)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x t)))\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____MBT\">mbt</see> x) y))</code>"],

 "ACL2____REDUNDANT-ENCAPSULATE":[["Encapsulate"],"ACL2 Sources","ACL2","<p>For this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic we assume familiarity with \u000A <v>encapsulate</v> events and the notion of redundancy for <see topic=\"ACL2____EVENTS\">events</see>; see \u000A <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> and see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p> \u000A \u000A <p>The typical way for an <v>encapsulate</v> event to be redundant is when a \u000A syntactically identical <v>encapsulate</v> has already been executed under the \u000A same <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt>, <tt><see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see></tt>, and <tt><see topic=\"ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS\">default-verify-guards-eagerness</see></tt>.  But more generally, the <v>encapsulate</v> \u000A events need not be syntactically identical; for example, it suffices that they \u000A agree when the contents of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> sub-events are ignored.  The precise \u000A criterion for redundancy is given below, but let us first look at a \u000A consequence of the point just made about ignoring the contents of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> sub-events.  Consider the following sequence of two events.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> f-identity\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) x))))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> false-claim\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) (<see topic=\"COMMON-LISP____NOT\">not</see> x)))))</code> \u000A \u000A <p>You might be surprised to learn that the second is actually redundant, even \u000A though <v>false-claim</v> is clearly not a theorem; indeed, its negation is a \u000A theorem!  The following two points may soften the blow.  First, this behavior \u000A is as specified above (and is specified more precisely below): the contents of \u000A <see topic=\"ACL2____LOCAL\">local</see> events are ignored when checking redundancy of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events.  Second, this behavior is sound, because the logical \u000A meaning of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event is taken from the events that it \u000A exports, which do not include events that are <see topic=\"ACL2____LOCAL\">local</see> to the \u000A <v>encapsulate</v> event.</p> \u000A \u000A <p>Some users, however, want to use <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> events for testing in \u000A a way that is thwarted by this ignoring of <see topic=\"ACL2____LOCAL\">local</see> sub-events.  Consider \u000A the following sort of example.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test1 ...)))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test2 ...)))</code> \u000A \u000A <p>Since the contents of local events are ignored when checking redundancy of \u000A an <v>encapsulate</v> event, the second form just above is indeed redundant, \u000A presumably not as expected by whomever wrote these two tests.  A solution is \u000A to add distinct non-local forms, for example as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A             (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> &quot;test1&quot;)\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test1 ...)))\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see> ()\u000A             (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> &quot;test2&quot;)\u000A             (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test2 ...)))</code> \u000A \u000A <p>A different solution is to use <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> for testing, as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test1 ...)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____DEFTHM\">defthm</see> test2 ...)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))</code> \u000A \u000A <p>Also see community books <v>misc/eval.lisp</v>, \u000A <v>make-event/eval-check.lisp</v>, and <v>make-event/eval-tests.lisp</v> for more \u000A ways to test in books.</p> \u000A \u000A <p>The precise criterion for redundancy of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> \u000A is as follows.  The two <see topic=\"ACL2____WORLD\">world</see>s in which each is evaluated must have the \u000A same <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>s, the same <see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see>, and \u000A the same <see topic=\"ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS\">default-verify-guards-eagerness</see>.  Moreover, the existing and \u000A proposed <v>encapsulate</v> <see topic=\"ACL2____EVENTS\">events</see> contain the same <see topic=\"ACL2____SIGNATURE\">signature</see>s and \u000A the same number of top-level events &mdash; let <v>k</v> be that number &mdash; \u000A and for each <v>i &lt; k</v>, the <v>i</v>th top-level events <v>E1</v> and <v>E2</v> \u000A from the earlier and current <v>encapsulate</v>s must be related in one of the \u000A following ways.</p> \u000A \u000A <p>o <v>E1</v> and <v>E2</v> are equal; or</p> \u000A \u000A <p>o <v>E1</v> is of the form <v>(record-expansion E2 ...)</v>; or else</p> \u000A \u000A <p>o <v>E1</v> and <v>E2</v> are equal after replacing each <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A sub-event by <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :elided))</v>, where a sub-event of an \u000A event <v>E</v> is either <v>E</v> itself, or a sub-event of a constituent event of \u000A <v>E</v> in the case that <v>E</v> is a call of <tt><see topic=\"ACL2____WITH-OUTPUT\">with-output</see></tt>, <tt><see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see></tt>, <tt><see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see></tt>, \u000A <v>record-expansion</v>, <tt><see topic=\"ACL2____TIME_42\">time$</see></tt>, <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt>, <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt>, or \u000A <v>encapsulate</v> itself.</p>"],

 "ACL2____DISABLE":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DISABLE\">disable</see> fact (fact) associativity-of-app)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DISABLE\">disable</see> name1 name2 ... namek)</code> \u000A \u000A <p>where each <v>namei</v> is a runic designator; see <see topic=\"ACL2____THEORIES\">theories</see>.  The \u000A result is the theory that contains all the names in the current theory except \u000A those listed.  Note that this is merely a function that returns a theory.  The \u000A result is generally a very long list of <see topic=\"ACL2____RUNE\">rune</see>s and you will probably \u000A regret printing it.</p> \u000A \u000A <p>The standard way to ``disable'' a fixed set of names, is as follows; see \u000A <see topic=\"ACL2____HINTS\">hints</see> and see <see topic=\"ACL2____IN-THEORY\">in-theory</see>.</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> name1 name2 ... namek)    ; in a hint\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> name1 name2 ... namek))   ; as an event\u000A(<see topic=\"ACL2____LOCAL\">local</see> ; often desirable, to avoid exporting from the current context\u000A (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> name1 name2 ... namek)))</code> \u000A \u000A <p>Note that all the names are implicitly quoted.  If you wish to disable a \u000A computed list of names, <v>lst</v>, use the theory expression \u000A <v>(<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here) lst)</v>.</p>"],

 "ACL2____About_02the_02Prompt":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The string ``<v>ACL2 !&gt;</v>'' is the ACL2 prompt.</p> \u000A \u000A <p>The prompt tells the user that an ACL2 <see topic=\"ACL2____COMMAND\">command</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>is \u000A expected.  In addition, the prompt tells us a little about the current state \u000A of the ACL2 command interpreter.  We explain the prompt briefly below.  But \u000A first we talk about the command interpreter.</p> \u000A \u000A <p>An ACL2 command is generally a Lisp expression to be evaluated.  There are \u000A some unusual commands (such as :<see topic=\"ACL2____Q\">q</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> for \u000A <b>quitting</b> ACL2) which cause other behavior.  But most commands are read, \u000A evaluated, and then have their results printed.  Thus, we call the command \u000A interpreter a ``read-eval-print loop.''  The ACL2 command interpreter is named \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> (after Lisp's ``load'').</p> \u000A \u000A <p>When a command is read, all the symbols in it are converted to uppercase. \u000A Thus, typing <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app ...)</v> is the same as typing <v>(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> APP ...)</v> \u000A or <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> App ...)</v>.  There are ways to force lowercase case characters \u000A into symbols but we won't discuss them here.  A consequence of Common Lisp's \u000A default uppercasing is that you'll see a general lack of concern over the case \u000A used when symbols are displayed in this documentation.</p> \u000A \u000A <p>In addition, symbols ``belong'' to ``packages'' which give the user a way \u000A to control namespaces.  The prompt tells us which package is the default one, \u000A namely <v>&quot;ACL2&quot;</v>.  That means when we call <v>car</v>, for example, we are \u000A invoking the standard definition of that symbol.  If the packager were \u000A <v>&quot;JONES&quot;</v> then <v>car</v> would refer to the definition of that symbol in \u000A that package (which may or may not be different depending on what symbols were \u000A imported into that package.</p> \u000A \u000A <p>A command like <b>(defun app (x y) ...)</b> causes ACL2 to evaluate the \u000A <see topic=\"COMMON-LISP____DEFUN\">defun</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> function on <b>app</b>, <b>(x y)</b> and \u000A <b>...</b>.  When that command is evaluated it prints some information to the \u000A terminal explaining the processing of the proposed definition.  It returns the \u000A symbol <v>APP</v> as its value, which is printed by the command interpreter. \u000A (Actually, <v>defun</v> is not a function but a <see topic='COMMON-LISP____DEFMACRO'>macro</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> which expands to a form that involves <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, a necessary precondition to printing output to the \u000A terminal and to ``changing'' the set of axioms.  But we do not discuss this \u000A further here.)</p> \u000A \u000A <p>The <v>defun</v> command is an example of a special kind of command called an \u000A ``event.''  <see topic=\"ACL2____EVENTS\">Events</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> are those commands that change the ``logical \u000A world'' by adding such things as axioms or theorems to ACL2's database.  See \u000A <see topic=\"ACL2____WORLD\">world</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.  But not every command is an event command.</p> \u000A \u000A <p>A command like <b>(app '(1 2 3) '(4 5 6 7))</b> is an example of a \u000A non-event.  It is processed the same general way: the function <b>app</b> is \u000A applied to the indicated arguments and the result is printed.  The function \u000A <b>app</b> does not print anything and does not change the ``world.''</p> \u000A \u000A <p>A third kind of command is one that display information about the current \u000A logical world or that ``roll back'' to previous versions of the world.  Such \u000A commands are called ``<see topic=\"ACL2____HISTORY\">history</see>'' <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A commands.</p> \u000A \u000A <p>What does the ACL2 prompt tell us about the read-eval-print loop?  The \u000A prompt ``<v>ACL2 !&gt;</v>'' tells us that the command will be read with <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> set to <v>&quot;ACL2&quot;</v>, that guard checking (see \u000A <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>) is on (``<v>!</v>''), and that we are at the \u000A top-level (there is only one ``<v>&gt;</v>'').  For more about the prompt, see \u000A <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>You should now return to <see topic='ACL2____Revisiting_02the_02Admission_02of_02App'>the Walking Tour</see>.</p>"],

 "ACL2____LINEAR-ARITHMETIC":[["Linear"],"ACL2 Sources","ACL2","<p>We describe the procedure very roughly here.  Fundamental to the \u000A procedure is the notion of a linear polynomial inequality.  A ``linear \u000A polynomial'' is a sum of terms, each of which is the product of a rational \u000A constant and an ``unknown.''  The ``unknown'' is permitted to be <v>1</v> simply \u000A to allow a term in the sum to be constant.  Thus, an example linear polynomial \u000A is <v>3*x + 7*a + 2</v>; here <v>x</v> and <v>a</v> are the (interesting) unknowns. \u000A However, the unknowns need not be variable symbols.  For example, <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see>\u000A x)</v> might be used as an unknown in a linear polynomial.  Thus, another linear \u000A polynomial is <v>3*(<see topic=\"COMMON-LISP____LENGTH\">length</see> x) + 7*a</v>.  A ``linear polynomial inequality'' is \u000A an inequality (either <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt> or <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>) relation between two linear \u000A polynomials.  Note that an equality may be considered as a pair of \u000A inequalities; e.q., <v>3*x + 7*a + 2 = 0</v> is the same as the conjunction of \u000A <v>3*x + 7*a + 2 &lt;= 0</v> and <v>0 &lt;= 3*x + 7*a + 2</v>.</p> \u000A \u000A <p>Certain linear polynomial inequalities can be combined by \u000A cross-multiplication and addition to permit the deduction of a third \u000A inequality with fewer unknowns.  If this deduced inequality is manifestly \u000A false, a contradiction has been deduced from the assumed inequalities.</p> \u000A \u000A <p>For example, suppose we have two assumptions</p> \u000A \u000A <code>p1:       3*x + 7*a &lt;  4\u000Ap2:               3 &lt;  2*x</code> \u000A \u000A <p>and we wish to prove that, given <v>p1</v> and <v>p2</v>, <v>a &lt; 0</v>.  As \u000A suggested above, we proceed by assuming the negation of our goal</p> \u000A \u000A <code>p3:               0 &lt;= a.</code> \u000A \u000A <p>and looking for a contradiction.</p> \u000A \u000A <p>Our first step will be to eliminate the variable <v>x</v>.  To that end, we \u000A cross-multiply based on the coefficients of <v>x</v> in the two inequalities. \u000A By multiplying <v>p1</v> by <v>2</v> and <v>p2</v> by <v>3</v>, and then adding the \u000A respective sides, we deduce the intermediate result</p> \u000A \u000A <code>p4:  6*x + 14*a + 9 &lt; 8 + 6*x</code> \u000A \u000A <p>which, after cancellation, is:</p> \u000A \u000A <code>p4:        14*a + 1 &lt;  0.</code> \u000A \u000A <p>If we then cross-multiply and add <v>p3</v> to <v>p4</v>, we get</p> \u000A \u000A <code>p5:               1 &lt;= 0,</code> \u000A \u000A <p>a contradiction.  Thus, we have proved that <v>p1</v> and <v>p2</v> imply the \u000A negation of <v>p3</v>.</p> \u000A \u000A <p>All of the unknowns of an inequality must be eliminated by cancellation in \u000A order to produce a constant inequality.  We can choose to eliminate the \u000A unknowns in any order, but we eliminate them in term-order, largest unknowns \u000A first.  (See <see topic=\"ACL2____TERM-ORDER\">term-order</see>.)  That is, two polys are cancelled against \u000A each other only when they have the same largest unknown.  For instance, in the \u000A above example we see that <v>x</v> is the largest unknown in each of <v>p1</v> and \u000A <v>p2</v>, and <v>a</v> in <v>p3</v> and <v>p4</v>.</p> \u000A \u000A <p>Now suppose that this procedure does not produce a contradiction but \u000A instead yields a set of nontrivial inequalities.  A contradiction might still \u000A be deduced if we could add to the set some additional inequalities allowing \u000A further cancellations.  That is where <v>:linear</v> lemmas come in.  When the \u000A set of inequalities has stabilized under cross-multiplication and addition and \u000A no contradiction is produced, we search the database of <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> \u000A rules for rules about the unknowns that are candidates for cancellation (i.e., \u000A are the largest unknowns in their respective inequalities).  See <see topic=\"ACL2____LINEAR\">linear</see> \u000A for a description of how <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> rules are used.</p> \u000A \u000A <p>See also <tt><see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see></tt> for a description of an extension to \u000A the linear-arithmetic procedure described here.</p>"],

 "ACL2____SET-INHIBIT-OUTPUT-LST":[["Prover-output"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> '(warning))\u000A(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> '(<see topic=\"ACL2____PROOF-TREE\">proof-tree</see> prove proof-checker))\u000A(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> *valid-output-names*) ; inhibit all prover output\u000A:set-inhibit-output-lst (<see topic=\"ACL2____PROOF-TREE\">proof-tree</see> prove)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> lst)</code> \u000A \u000A <p>where <v>lst</v> is a form (which may mention <tt><see topic=\"ACL2____STATE\">state</see></tt>) that evaluates \u000A to a list of names, each of which is the name of one of the following \u000A ``kinds'' of output produced by ACL2.</p> \u000A \u000A <code>error          error messages\u000Awarning        warnings other than those related to soundness\u000Awarning!       warnings (of all degrees of importance)\u000Aobservation    observations\u000Aprove          commentary produced by the theorem prover\u000Aproof-checker  commentary produced by the proof-checker\u000Aevent          non-proof commentary produced by events such as defun\u000A               and encapsulate\u000Ahistory        output from history commands such as :ubt and :pbt\u000Asummary        the summary at the successful conclusion of an event\u000Aproof-tree     proof-tree output</code> \u000A \u000A <p>It is possible to inhibit each kind of output by putting the corresponding \u000A name into <v>lst</v>.  For example, if <v>'warning</v> is included in (the value \u000A of) <v>lst</v>, then no warnings are printed except those related to soundness, \u000A e.g., the inclusion of an uncertified book.  Note that <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> \u000A output is affected by <v>set-inhibit-output-lst</v>; see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>Note that proof output can be controlled without inhibiting it using this \u000A utility, and indeed is already quite limited by default.  See <tt><see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____WITH-OUTPUT\">with-output</see> for a variant of this utility that can be used in \u000A <see topic=\"ACL2____BOOKS\">books</see>.  Also see <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> for how to inhibit \u000A individual warning types and see <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see> for how to \u000A inhibit individual parts of the summary.</p> \u000A \u000A <p>Printing of events on behalf of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is inhibited when both <v>'event</v> and <v>'prove</v> belong to \u000A <v>lst</v>.  Otherwise, printing of events is controlled by the <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A special <tt><see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see></tt>.</p> \u000A \u000A <p><i>Note for advanced users.</i> By including <v>warning!</v> in <v>lst</v>, you \u000A are automatically including <v>warning</v> as well: all warnings will be \u000A inhibited.  This is not the case if you modify value of state global variable \u000A <v>'inhibit-output-lst</v> directly (with <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> or <v>f-put-global</v>); \u000A then, if you include <v>warning!</v> but not <v>warning</v>, then warnings not \u000A related to soundness will still be printed (which is probably not what was \u000A intended).</p>"],

 "ACL2____REGENERATE-TAU-DATABASE":[["Events","Introduction-to-the-tau-system"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____REGENERATE-TAU-DATABASE\">regenerate-tau-database</see>)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REGENERATE-TAU-DATABASE\">regenerate-tau-database</see> :doc doc-string)</code> \u000A \u000A <p>where <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string not \u000A beginning with ``<v>:Doc-Section</v> ...''.  Because no unique name is \u000A associated with a <v>regenerate-tau-database</v> event, there is no way we can \u000A store the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> in our \u000A il[documentation] database.  Hence, we actually prohibit <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> \u000A from having the form of an ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p>The tau database is regenerated by scanning the current logical world and \u000A re-processing every rule-generating event in it relative to the current \u000A enabled theory and current tau auto mode settings.  See <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> for background details.</p> \u000A \u000A <p>This command was intended to allow the user to remove a fact from the tau \u000A database, by regenerating the database without the fact.  But as the following \u000A discussion highlights, <v>regenerate-tau-database</v> does not really solve the \u000A problem.  We regard it as a placeholder for a more sophisticated mechanism. \u000A However, we have trouble understanding why a user might wish to remove a fact \u000A from the database and are awaiting further user experiences before designing \u000A the more sophisticated mechanism.</p> \u000A \u000A <p>Suppose, for example, that you wanted to remove a signature rule provided \u000A by some rule with name <i>rune</i>.  You could disable <i>rune</i> and \u000A regenerate the database.  We discuss why you might &mdash; or might not \u000A &mdash; want to do this later.  But suppose you did it.  Unfortunately, the \u000A database you get will not be just like the one you started with minus the \u000A signature rule.  The reason is that the database you started with was \u000A generated incrementally and the current theory might have evolved.  To take a \u000A simple example, your starting database might have included a rule that has \u000A been disabled since it was first added.  Thus, the part of your starting \u000A database built before the disabling was constructed with the rule enabled and \u000A the part built afterwards has the rule disabled.  You are unlikely to get the \u000A same database whether you enable or disable that rule now.</p> \u000A \u000A <p>You might hope that the avoidance of <v>in-theory</v> events would eliminate \u000A the problem but it does not because even the <tt><see topic=\"ACL2____GROUND-ZERO\">ground-zero</see></tt> theory is \u000A constructed incrementally from the ``pre-history'' commands used to boot up \u000A ACL2.  Those pre-history commands include some global <v>in-theory</v> commands. \u000A Since every session starts from the <v>ground-zero</v> state, the starting \u000A database is already ``infected'' with global <v>in-theory</v> commands.</p> \u000A \u000A <p>The reason we hope that it will not be necessary to remove tau facts is \u000A that the tau system is designed merely to be fast and benign (see <i>Design \u000A Philosophy</i> in <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see>).  The tau system's \u000A coverage should grows monotonically with the addition of rules.  According to \u000A this understanding of tau, adding a signature rule, for example, may allow tau \u000A to prove some additional goals but will not prevent it from proving goals it \u000A could prove previously.  If this is understanding of tau is accurate, we see \u000A no fundamental reason to support the removal of a fact.  This, of course, \u000A ignores the possibility that the user wishes to explore alternative proof \u000A strategies or measure performance.</p> \u000A \u000A <p>We welcome user observations and experience on this issue.</p>"],

 "ACL2____LD-KEYWORD-ALIASES":[["Ld"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-LD-KEYWORD-ALIASES\">set-ld-keyword-aliases</see> '((:q 0 q-fn)\u000A                          (:e 0 exit-acl2-macro))\u000A                        state)\u000A(<see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see> state) ; current value of the ld-keyword-aliases table</code> \u000A \u000A <p><v>Ld-keyword-aliases</v> is the name of a ACL2 table (see <see topic=\"ACL2____TABLE\">table</see>) and \u000A also the name of a function of <v>state</v> that returns the value of this \u000A table.  That value must be an alist, each element of which is of the form \u000A <v>(:keyword n fn)</v>, where <v>:keyword</v> is a keyword, <v>n</v> is a \u000A nonnegative integer, and <v>fn</v> is a function symbol of arity <v>n</v>, a macro \u000A symbol, or a <v>lambda</v> expression of arity <v>n</v>.  When <v>keyword</v> is \u000A typed as an <tt><see topic=\"ACL2____LD\">ld</see></tt> command, <v>n</v> more forms are read, <v>x1, ..., xn</v>, \u000A and the form <v>(fn 'x1 ... 'xn)</v> is then evaluated.  The initial value of \u000A the <v>ld-keyword-aliases</v> <see topic=\"ACL2____TABLE\">table</see> is <v>nil</v>.</p> \u000A \u000A <p>ACL2 provides functions to modify the <v>ld-keyword-aliases</v> table, as \u000A follows.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>(<see topic=\"ACL2____SET-LD-KEYWORD-ALIASES\">Set-ld-keyword-aliases</see> val state)</v>: sets the table to <v>val</v>, which \u000A must be a legal alist as described above.  This is an event that may go into a \u000A book (see <see topic=\"ACL2____EVENTS\">events</see>), but its effect will be <see topic=\"ACL2____LOCAL\">local</see> to that \u000A book.</p> \u000A \u000A <p><v>Set-ld-keyword-aliases!</v> is the same as <v>set-ld-keyword-aliases</v>, \u000A except that its effect is not <see topic=\"ACL2____LOCAL\">local</see>.  Indeed, the form \u000A <v>(<see topic=\"ACL2____SET-LD-KEYWORD-ALIASES\">set-ld-keyword-aliases</see> val state)</v> is equivalent to the form <v>(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____SET-LD-KEYWORD-ALIASES_12\">set-ld-keyword-aliases!</see> val state)</v>.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ADD-LD-KEYWORD-ALIAS\">Add-ld-keyword-alias</see> key val state)</v>: modifies the table by binding \u000A the keyword <v>key</v> to <v>val</v>, which must be a legal value as described \u000A above.  This is an event that may go into a book (see <see topic=\"ACL2____EVENTS\">events</see>), but its \u000A effect will be <see topic=\"ACL2____LOCAL\">local</see> to that book.</p> \u000A \u000A <p><v>Add-ld-keyword-alias!</v> is the same as <v>add-ld-keyword-alias</v>, except \u000A that its effect is not <see topic=\"ACL2____LOCAL\">local</see>.  Indeed, the form \u000A <v>(<see topic=\"ACL2____ADD-LD-KEYWORD-ALIAS\">add-ld-keyword-alias</see> key val state)</v> is equivalent to the form <v>(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____ADD-LD-KEYWORD-ALIAS_12\">add-ld-keyword-alias!</see> key val state)</v>.</p> \u000A \u000A </blockquote> \u000A \u000A <p>Consider the first example above:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-LD-KEYWORD-ALIASES\">set-ld-keyword-aliases</see> '((:q 0 q-fn)\u000A                          (:e 0 exit-acl2-macro))\u000A                        state)</code> \u000A \u000A <p>With this event, <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> is redefined to have the effect of \u000A executing <v>(q-fn)</v>, so for example if you have defined <v>q-fn</v> with</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> q-fn ()\u000A  '(<see topic=\"ACL2____ER\">er</see> soft 'q &quot;You un-bound :q and now we have a soft error.&quot;))</code> \u000A \u000A <p>then <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> will cause an error, and if you have defined</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> exit-acl2-macro () '(exit-ld state))</code> \u000A \u000A <p>then <v>:e</v> will cause the effect (it so happens) that <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> \u000A normally has.  If you prefer <v>:e</v> to <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> for exiting the ACL2 \u000A loop, you might even want to put such definitions of <v>q-fn</v> and \u000A <v>exit-acl2-macro</v> together with the <v>set-ld-keyword-aliases</v> form above \u000A into your <v>&quot;acl2-customization.lsp&quot;</v> file; see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-keyword-aliases</v> is one of them.  <v>Ld-keyword-aliases</v> affects how \u000A keyword commands are parsed.  Generally speaking, <tt><see topic=\"ACL2____LD\">ld</see></tt>'s command \u000A interpreter reads ``<v>:fn x1 ... xn</v>'' as ``<v>(fn 'x1 ... 'xn)</v>'' when \u000A <v>:fn</v> is a keyword and <v>fn</v> is the name of an <v>n</v>-ary function; see \u000A <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.  But this parse is overridden, as described above, \u000A for the keywords bound in the <v>ld-keyword-aliases</v> <see topic=\"ACL2____TABLE\">table</see>.</p>"],

 "ACL2____GENERALIZED-BOOLEANS":[["Common-lisp"],"ACL2 Sources","ACL2","<p>The discussion below outlines a potential source of unsoundness in \u000A ACL2.  Although to our knowledge there is no existing Lisp implementation in \u000A which this problem can arise, nevertheless we feel compelled to explain this \u000A situation.</p> \u000A \u000A <p>Consider for example the question: What is the value of <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v>? \u000A According to the ACL2 axioms, it's <v>t</v>.  And as far as we know, based on \u000A considerable testing, the value is <v>t</v> in every Common Lisp implementation. \u000A However, according the Common Lisp draft proposed ANSI standard, any \u000A non-<v>nil</v> value could result.  Thus for example, <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v> is \u000A allowed by the standard to be <v>17</v>.</p> \u000A \u000A <p>The Common Lisp standard specifies (or soon will) that a number of Common \u000A Lisp functions that one might expect to return Boolean values may, in fact, \u000A return arbitrary values.  Examples of such functions are <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt>, and <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>; a much more complete list is given below.  Indeed, \u000A we dare to say that every Common Lisp function that one may believe returns \u000A only Booleans is, nevertheless, not specified by the standard to have that \u000A property, with the exceptions of the functions <v>not</v> and <v>null</v>.  The \u000A standard refers to such arbitrary values as ``generalized Booleans,'' but in \u000A fact this terminology makes no restriction on those values.  Rather, it merely \u000A specifies that they are to be viewed, in an informal sense, as being either \u000A <v>nil</v> or not <v>nil</v>.</p> \u000A \u000A <p>This situation is problematic for ACL2, which axiomatizes these functions \u000A to return Booleans.  The problem is that because (for efficiency and \u000A simplicity) ACL2 makes very direct use of compiled Common Lisp functions to \u000A support the execution of its functions, there is in principle a potential for \u000A unsoundness due to these ``generalized Booleans.''  For example, ACL2's <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> function is defined to be Common Lisp's <v>equal</v>.  Hence if in Common \u000A Lisp the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v> were to evaluate to 17, then in ACL2 we could \u000A prove (using the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>) \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3) 17)</v>.  However, ACL2 can also prove <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x\u000A x) t)</v>, and these two terms together are contradictory, since they imply \u000A (instantiating <v>x</v> in the second term by <v>3</v>) that <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v> is \u000A both equal to <v>17</v> and to <v>t</v>.</p> \u000A \u000A <p>To make matters worse, the standard allows <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v> to evaluate to \u000A <i>different</i> non-<v>nil</v> values every time.  That is: <v>equal</v> need not \u000A even be a function!</p> \u000A \u000A <p>Fortunately, no existing Lisp implementation takes advantage of the \u000A flexibility to have (most of) its predicates return generalized Booleans, as \u000A far as we know.  We may implement appropriate safeguards in future releases of \u000A ACL2, in analogy to (indeed, probably extending) the existing safeguards by \u000A way of guards (see <see topic=\"ACL2____GUARD\">guard</see>).  For now, we'll sleep a bit better knowing \u000A that we have been up-front about this potential problem.</p> \u000A \u000A <p>The following list of functions contains all those that are defined in \u000A Common Lisp to return generalized Booleans but are assumed by ACL2 to return \u000A Booleans.  In addition, the functions <tt><see topic=\"ACL2____ACL2-NUMBERP\">ACL2-numberp</see></tt> and <tt><see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see></tt> are directly defined in terms of respective Common Lisp \u000A functions <v>numberp</v> and <v>complexp</v>.</p> \u000A \u000A <code>/=\u000A&lt;\u000A=\u000Aalpha-char-p\u000Aatom\u000Achar-equal\u000Achar&lt;\u000Achar&lt;=\u000Achar&gt;\u000Achar&gt;=\u000Acharacterp\u000Aconsp\u000Adigit-char-p\u000Aendp\u000Aeq\u000Aeql\u000Aequal\u000Aevenp\u000Aintegerp\u000Akeywordp\u000Alistp\u000Alogbitp\u000Alogtest\u000Alower-case-p\u000Aminusp\u000Aoddp\u000Aplusp\u000Arationalp\u000Astandard-char-p\u000Astring-equal\u000Astring&lt;\u000Astring&lt;=\u000Astring&gt;\u000Astring&gt;=\u000Astringp\u000Asubsetp\u000Asymbolp\u000Aupper-case-p\u000Azerop</code>"],

 "ACL2____UNCERTIFIED-BOOKS":[["Books"],"ACL2 Sources","ACL2","<p>For relevant background see <see topic=\"ACL2____BOOKS\">books</see>, see <see topic=\"ACL2____CERTIFICATE\">certificate</see>, \u000A and see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____INCLUDE-BOOK\">Include-book</see></tt> has a special provision for dealing with an \u000A uncertified book, i.e., a file with no <see topic=\"ACL2____CERTIFICATE\">certificate</see> or an invalid <see topic=\"ACL2____CERTIFICATE\">certificate</see> (i.e., one whose check sums describe files other than the ones \u000A actually read).  In this case, a warning is printed and the book is otherwise \u000A processed much as though it were certified and had an open <see topic=\"ACL2____PORTCULLIS\">portcullis</see>.</p> \u000A \u000A <p>If a book <v>B.lisp</v> is uncertified and a file <v>B.port</v> exists, then \u000A the forms in <v>B.port</v> are evaluated before the forms in <v>B.lisp</v>.  Such \u000A a file <v>B.port</v> is typically created calling <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> on book \u000A <v>&quot;B&quot;</v> with argument <v>:write-port t</v>, so that <v>B.port</v> contains the \u000A <see topic=\"ACL2____PORTCULLIS\">portcullis</see> <see topic=\"ACL2____COMMAND\">command</see>s for <v>B</v> (the commands present in the \u000A <see topic=\"ACL2____WORLD\">world</see> when that certification was attempted).  To avoid loading \u000A <v>.port</v> files, see <see topic=\"ACL2____COMPILATION\">compilation</see>.</p> \u000A \u000A <p>Inclusion of uncertified books can be handy, but it can have disastrous \u000A consequences.</p> \u000A \u000A <p>The provision allowing uncertified <see topic=\"ACL2____BOOKS\">books</see> to be included can have \u000A disastrous consequences, ranging from hard lisp errors, to damaged memory, to \u000A quiet logical inconsistency.</p> \u000A \u000A <p>It is possible for the inclusion of an uncertified book to render the logic \u000A inconsistent.  For example, one of its non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see> might \u000A be <v>(<see topic=\"ACL2____DEFTHM\">defthm</see> t-is-nil (<see topic=\"COMMON-LISP____EQUAL\">equal</see> t nil))</v>.  It is also possible for the \u000A inclusion of an uncertified book to cause hard errors or <see topic=\"ACL2____BREAKS\">breaks</see> into \u000A raw Common Lisp.  For example, if the file has been edited since it was \u000A certified, it may contain too many open parentheses, causing Lisp to read past \u000A ``end of file.'' Similarly, it might contain non-ACL2 objects such as \u000A <v>3.1415</v> or ill-formed event forms that cause ACL2 code to break.</p> \u000A \u000A <p>Even if a book is perfectly well formed and could be certified (in a \u000A suitable extension of ACL2's initial <see topic=\"ACL2____WORLD\">world</see>), its uncertified inclusion \u000A might cause Lisp errors or inconsistencies!  For example, it might mention \u000A packages that do not exist in the host <see topic=\"ACL2____WORLD\">world</see>, especially if the \u000A <v>.port</v> file (discussed above) does not exist from an earlier certification \u000A attempt.  The <see topic=\"ACL2____PORTCULLIS\">portcullis</see> of a certified book ensures that the correct \u000A <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s have been admitted, but if a book is read without actually \u000A raising its <see topic=\"ACL2____PORTCULLIS\">portcullis</see>, symbols in the file, e.g., \u000A <v>acl2-arithmetic::fn</v>, could cause ``unknown package'' errors in Common \u000A Lisp.  Perhaps the most subtle disaster occurs if the host <see topic=\"ACL2____WORLD\">world</see> does \u000A have a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> for each package used in the book but the host <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> imports different symbols than those required by the <see topic=\"ACL2____PORTCULLIS\">portcullis</see>.  In this case, it is possible that formulas which were theorems \u000A in the certified book are non-theorems in the host <see topic=\"ACL2____WORLD\">world</see>, but those \u000A formulas can be read without error and will then be quietly assumed.</p> \u000A \u000A <p>In short, if you include an uncertified book, <b>all bets are off</b> \u000A regarding the validity of the future behavior of ACL2.</p> \u000A \u000A <p>That said, it should be noted that ACL2 is pretty tough and if errors don't \u000A occur, the chances are that deductions after the inclusion of an uncertified \u000A book are probably justified in the (possibly inconsistent) logical extension \u000A obtained by assuming the admissibility and validity of the definitions and \u000A conjectures in the book.</p>"],

 "ACL2____PSEUDO-TERMP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x 'nil)))      ; has value t\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(<see topic=\"COMMON-LISP____CAR\">car</see> x y z))              ; also has value t!\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(delta (h x)))            ; has value t\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(delta (h x) . 7))        ; has value nil (<see topic=\"COMMON-LISP____NOT\">not</see> a true-listp)\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)) b)) ; has value t\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(<see topic=\"COMMON-LISP____IF\">if</see> x y 123))             ; has value nil (123 is not quoted)\u000A(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> '(<see topic=\"COMMON-LISP____IF\">if</see> x y '123))            ; has value t</code> \u000A \u000A <p>If <v>x</v> is the quotation of a term, then <v>(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)</v> is <v>t</v>. \u000A However, if <v>x</v> is not the quotation of a term it is not necessarily the \u000A case that <v>(<see topic=\"ACL2____PSEUDO-TERMP\">pseudo-termp</see> x)</v> is <v>nil</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____TERM\">term</see> for a discussion of the various meanings of the word \u000A ``term'' in ACL2.  In its most strict sense, a term is either a legal variable \u000A symbol, a quoted constant, or the application of an <v>n</v>-ary function symbol \u000A or closed <v>lambda</v>-expression to <v>n</v> terms.  By ``legal variable \u000A symbol'' we exclude constant symbols, such as <v>t</v>, <v>nil</v>, and \u000A <v>*ts-rational*</v>.  By ``quoted constants'' we include <v>'t</v> (aka <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A t)</v>), <v>'nil</v>, <v>'31</v>, etc., and exclude constant names such as <v>t</v>, \u000A <v>nil</v> and <v>*ts-rational*</v>, unquoted constants such as <v>31</v> or \u000A <v>1/2</v>, and ill-formed <v>quote</v> expressions such as <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> 3 4)</v>.  By \u000A ``closed lambda expression'' we exclude expressions, such as <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x)\u000A (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))</v>, containing free variables in their bodies.  Terms typed by the \u000A user are translated into strict terms for internal use in ACL2.</p> \u000A \u000A <p>The predicate <v>termp</v> checks this strict sense of ``term'' with respect \u000A to a given ACL2 logical world; See <see topic=\"ACL2____WORLD\">world</see>.  Many ACL2 functions, such as \u000A the rewriter, require certain of their arguments to satisfy <v>termp</v>. \u000A However, as of this writing, <v>termp</v> is in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and \u000A thus cannot be used effectively in conjectures to be proved.  Furthermore, if \u000A regarded simply from the perspective of an effective <see topic=\"ACL2____GUARD\">guard</see> for a \u000A term-processing function, <v>termp</v> checks many irrelevant things.  (Does it \u000A really matter that the variable symbols encountered never start and end with \u000A an asterisk?)  For these reasons, we have introduced the notion of a \u000A ``pseudo-term'' and embodied it in the predicate <v>pseudo-termp</v>, which is \u000A easier to check, does not require the logical <see topic=\"ACL2____WORLD\">world</see> as input, has \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, and is often perfectly suitable as a <see topic=\"ACL2____GUARD\">guard</see> on \u000A term-processing functions.</p> \u000A \u000A <p>A <v>pseudo-termp</v> is either a symbol, a true list of length 2 beginning \u000A with the word <v>quote</v>, the application of an <v>n</v>-ary pseudo-<v>lambda</v> \u000A expression to a true list of <v>n</v> pseudo-terms, or the application of a \u000A symbol to a true list of <v>n</v> <v>pseudo-termp</v>s.  By an ``<v>n</v>-ary \u000A pseudo-<v>lambda</v> expression'' we mean an expression of the form <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see>\u000A (v1 ... vn) pterm)</v>, where the <v>vi</v> are symbols (but not necessarily \u000A distinct legal variable symbols) and <v>pterm</v> is a <v>pseudo-termp</v>.</p> \u000A \u000A <p>Metafunctions may use <v>pseudo-termp</v> as a <see topic=\"ACL2____GUARD\">guard</see>.</p>"],

 "ACL2____ENABLE":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ENABLE\">enable</see> fact (fact) associativity-of-app)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ENABLE\">enable</see> name1 name2 ... namek)</code> \u000A \u000A <p>where each <v>namei</v> is a runic designator; see <see topic=\"ACL2____THEORIES\">theories</see>.  The \u000A result is the theory that contains all the names in the current theory plus \u000A those listed.  Note that this is merely a function that returns a theory.  The \u000A result is generally a very long list of <see topic=\"ACL2____RUNE\">rune</see>s and you will probably \u000A regret printing it.</p> \u000A \u000A <p>The standard way to ``enable'' a fixed set of names, is as follows; see \u000A <see topic=\"ACL2____HINTS\">hints</see> and see <see topic=\"ACL2____IN-THEORY\">in-theory</see>.</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> name1 name2 ... namek)  ; in a hint\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> name1 name2 ... namek)) ; as an event\u000A(<see topic=\"ACL2____LOCAL\">local</see> ; often desirable, to avoid exporting from the current context\u000A (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> name1 name2 ... namek)))</code> \u000A \u000A <p>Note that all the names are implicitly quoted.  If you wish to enable a \u000A computed list of names, <v>lst</v>, use the theory expression <v>(<see topic=\"ACL2____UNION-THEORIES\">union-theories</see>\u000A (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here) lst)</v>.</p>"],

 "ACL2____NORMED":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In Common Lisp, we can tell whether an ACL2 object is <b>normed</b> or not, \u000A but there is no way for an ordinary ACL2 function to see whether an object is \u000A normed.  Hence, whether or not an object is normed is an implementation-level \u000A concept.</p> \u000A \u000A <p>The fundamental property of normed objects is that if A and B are both \u000A normed, then <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> A B)</v> holds if and only if <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> A B)</v> holds.  For \u000A strings and conses, <v>(<see topic=\"COMMON-LISP____EQL\">eql</see> A B)</v> holds only when <v>(<see topic=\"COMMON-LISP____EQ\">eq</see> A B)</v>, so any \u000A normed conses or strings are <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> precisely when they are <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>. \u000A The potential benefits of having normed objects include: constant-time \u000A equality comparisons, reduced memory usage, fast association lists, and \u000A function memoization.</p> \u000A \u000A <p>Note that in our implementation, all symbols, characters, and numbers are \u000A automatically normed, and whenever a cons is normed its car and cdr must also \u000A be normed.</p> \u000A \u000A <p>The Meaning of Normed in Common Lisp.</p> \u000A \u000A <p>Recall that the ACL2 Objects are certain \"good\" Common Lisp symbols, \u000A characters, strings, and numbers, and the conses which can be recursively \u000A formed from these good atoms.</p> \u000A \u000A <p>Not all properties of these objects are visible in the ACL2 logic.  An \u000A example of an invisible property is the pointer identity of an object.  For \u000A example, suppose we write the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *x* (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 2))\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *y* (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 2))</code> \u000A \u000A <p>In Common Lisp, <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> is guaranteed to provide a new pair that is \u000A distinct from any previously created pair, so we know that *x* and *y* will be \u000A distinct objects and we will be able to tell them apart from one another using \u000A <v>eq</v>.  But this difference is not visible in the ACL2 logic, and no ACL2 \u000A function can tell *x* apart from *y*.</p> \u000A \u000A <p>The normed-ness of an object is a similarly invisible property.  In Common \u000A Lisp, invisible to ACL2, we maintain a data structure called a \"Hons Space\" \u000A that records which objects are normed.  So, being normed is not an intrinsic \u000A property of an object, but instead is determined by whether the object is \u000A mentioned in this Hons Space.</p> \u000A \u000A <p>Notes about Garbage Collection.</p> \u000A \u000A <p>The Hons Space includes tables with pointers to every normed cons and \u000A string.  One consequence of this is that normed objects cannot be reclaimed by \u000A Lisp's ordinary garbage collector, even after they have become unreachable \u000A from the perspective of an ACL2 program.</p> \u000A \u000A <p>For CCL users only, <tt><see topic=\"ACL2____HONS-WASH\">hons-wash</see></tt> is a special kind of garbage \u000A collection that allows normed conses to be reclaimed.  For other Lisps, the \u000A only option is to occasionally, manually clear out these Hons Space tables \u000A with <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt>.</p>"],

 "ACL2____DEAD-EVENTS":[["Other"],"ACL2 Sources","ACL2","<p>Below, when we talk about ``an event <v>A</v>'', we mean an event \u000A whose name is <v>A</v>.</p> \u000A \u000A <p>When event <v>A</v> is used in a proof performed to admit event <v>B</v> that \u000A you submit to ACL2, we say that <v>A</v> is a ``proof-supporter'' of <v>B</v>. \u000A ACL2 stores an association list such that for every event <v>B</v> with at least \u000A one proof-supporter, <v>B</v> is associated with a list of all of its \u000A proof-supporters, sorted by <tt><see topic=\"ACL2____SYMBOL-_C3\">symbol-&lt;</see></tt>.  The following form evaluates to \u000A that alist, which is called the ``proof-supporters-alist''.</p> \u000A \u000A <code>(global-val 'proof-supporters-alist (w state))</code> \u000A \u000A <p>By ``used in a proof'' above, we mean: applied as a rule or supplied \u000A explicitly via <see topic=\"ACL2____HINTS\">hints</see> of type <v>:use</v>, <v>:by</v>, or \u000A <v>:clause-processor</v>.  That is, the <see topic=\"ACL2____EVENTS\">events</see> ``used in a proof'' for \u000A admitting an event <v>E</v> are those listed in the summary printed at the \u000A conclusion of admitting <v>E</v>.</p> \u000A \u000A <p>Note that if proofs are skipped when admitting event <v>E</v>, say because \u000A the last admission of <v>E</v> was done by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> (or \u000A <v>certify-book</v>, which ends with an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>), then there will \u000A be no entry in that alist for <v>E</v>.  (An exception is made however for \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, where proof-supporters are remembered from \u000A the first pass; see below.)  So if you want the proof-supporters-alist to \u000A include supporters for events in a book, use <tt><see topic=\"ACL2____LD\">ld</see></tt> rather than <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> to process the events in that book.  If \u000A however you are interested in the proof-supporters FROM a book that support a \u000A later event, then it is fine to include that book.</p> \u000A \u000A <p>The case for <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is slightly tricky.  Consider an example \u000A of the following form.</p> \u000A \u000A <code>A ; event preceding the encapsulate\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A B\u000A (<see topic=\"ACL2____LOCAL\">local</see> C) ; uses A and B in a proof\u000A D ; uses C in a proof\u000A )</code> \u000A \u000A <p>At the conclusion of this <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event, the \u000A proof-supporters-alist associates <v>D</v> with <v>A</v> and <v>B</v>, but not \u000A <v>C</v> (which has disappeared, since it is <see topic=\"ACL2____LOCAL\">local</see>).</p> \u000A \u000A <p>Note that this sort of ``transitive closure'' operation is only performed \u000A when necessary due to the disappearance of <see topic=\"ACL2____LOCAL\">local</see> <see topic=\"ACL2____EVENTS\">events</see>.  For \u000A example, if we replace <v>(<see topic=\"ACL2____LOCAL\">local</see> C)</v> above by just <v>C</v>, then <v>D</v> is \u000A associated in the proof-supporters-alist only with <v>C</v>, not with <v>A</v> or \u000A <v>B</v>.  If you want the transitive closure of the relation computed by the \u000A proof-supporters-alist, you have to compute it yourself. (This was a \u000A deliberate design decision, in order to avoid slowing down event processing.) \u000A However, there is help available on how to do such a computation:</p> \u000A \u000A <p>A community book, <v>books/misc/dead-events.lisp</v>, does such a transitive \u000A closure, and moreover uses that information to find ``dead events'' relative \u000A to a list of ``desired'' events.  For example, suppose you use <tt><see topic=\"ACL2____LD\">ld</see></tt> to \u000A process the events, with proofs, in a book intended to prove theorems \u000A <v>MAIN-1</v> and <v>MAIN-2</v>.  (Remember, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> will not save \u000A such information.)  Suppose furthermore that the book begins with some <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms followed by <v>(<see topic=\"ACL2____DEFLABEL\">deflabel</see> book-start)</v>.  You could \u000A evaluate this form:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEAD-EVENTS\">dead-events</see> '(main-1 main-2) :start 'book-start)</code> \u000A \u000A <p>The result is a list of events that you probably can delete from the book \u000A without causing any proofs to fail.  See the <v>dead-events.lisp</v> book for \u000A further documentation.</p> \u000A \u000A <p>You might also find the code in the above book to be helpful for writing \u000A your own utilities based on the proof-supporters-alist.</p>"],

 "ACL2____COMPRESS1":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____COMPRESS1\">compress1</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____COMPRESS1\">compress1</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is a symbol and <v>alist</v> is a 1-dimensional array, \u000A generally named <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.  Logically \u000A speaking, this function removes irrelevant pairs from <v>alist</v>, possibly \u000A shortening it.  The function returns a new array, <v>alist'</v>, with the same \u000A <tt><see topic=\"ACL2____HEADER\">header</see></tt> (including name and dimension) as <v>alist</v>, that, under \u000A <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>, is everywhere equal to <v>alist</v>.  That is, <v>(<see topic=\"ACL2____AREF1\">aref1</see> name\u000A alist' i)</v> is <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist i)</v>, for all legal indices <v>i</v>. \u000A <v>Alist'</v> may be shorter than <v>alist</v> and the non-irrelevant pairs may \u000A occur in a different order than in <v>alist</v>.</p> \u000A \u000A <p>Practically speaking, this function plays an important role in the \u000A efficient implementation of <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>.  In addition to creating the new \u000A array, <v>alist'</v>, <v>compress1</v> makes that array the ``semantic value'' of \u000A <v>name</v> and allocates a raw lisp array to <v>name</v>.  For each legal index, \u000A <v>i</v>, that raw lisp array contains <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist' i)</v> in slot \u000A <v>i</v>.  Thus, subsequent <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> operations can be executed in \u000A virtually constant time provided they are given <v>name</v> and the <v>alist'</v> \u000A returned by the most recently executed <v>compress1</v> or <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> on \u000A <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p>In general, <v>compress1</v> returns an alist whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is an \u000A association list whose keys are nonnegative integers in ascending order. \u000A However, if the <tt><see topic=\"ACL2____HEADER\">header</see></tt> specifies an <v>:order</v> of <v>&gt;</v> then the \u000A keys will occur in descending order, and if the <v>:order</v> is <v>:none</v> or \u000A <v>nil</v> then the keys will not be sorted, i.e., <v>compress1</v> is logically \u000A the identity function (though it still attaches an array under the hood). \u000A Note however that a <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> call is replaced by a hard error if the \u000A header specifies an <v>:order</v> of <v>:none</v> or <v>nil</v> and the array's \u000A length exceeds the <tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> field of its <tt><see topic=\"ACL2____HEADER\">header</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>compress1</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A compress1 (<see topic=\"ACL2____NAME\">name</see> l)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l)))\u000A (<see topic=\"COMMON-LISP____CASE\">case</see>\u000A  (array-order (<see topic=\"ACL2____HEADER\">header</see> name l))\u000A  (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____HEADER\">header</see> name l)\u000A           (compress11 name l 0 (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))\u000A                       (<see topic=\"ACL2____DEFAULT\">default</see> name l))))\u000A  (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____HEADER\">header</see> name l)\u000A           (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (compress11 name l 0 (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))\u000A                                (<see topic=\"ACL2____DEFAULT\">default</see> name l)))))\u000A  (t\u000A   (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A    (<see topic=\"COMMON-LISP____AND\">and</see>\u000A     (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> name l))\u000A     (<see topic=\"ACL2____HARD-ERROR\">hard-error</see>\u000A      'compress1\u000A      &quot;Attempted to compress a one-dimensional array named ~\u000A                        ~x0 whose header specifies :ORDER ~x1 and whose ~\u000A                        length, ~x2, exceeds its maximum-length, ~x3.&quot;\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 name)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\1 nil)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\3 (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> name l)))))\u000A    l))))</code>"],

 "ACL2____USING-COMPUTED-HINTS-2":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>Suppose the main proof completes with a forcing round on three \u000A subgoals, \"[1]Subgoal 3\", \"[1]Subgoal 2\", and \"[1]Subgoal 1\".  Suppose \u000A you wish to <v>:use lemma42</v> in all top-level goals of the first forcing \u000A round.  This can be done supplying the hint</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> test '(:use lemma42) nil),</code> \u000A \u000A <p>where <v>test</v> is an expression that returns <v>t</v> when <v>ID</v> is one of \u000A the clause ids in question.</p> \u000A \u000A <code>    goal-spec     (parse-clause-id goal-spec)\u000A\u000A&quot;[1]Subgoal 3&quot;        ((1) (3) . 0)\u000A&quot;[1]Subgoal 2&quot;        ((1) (2) . 0)\u000A&quot;[1]Subgoal 1&quot;        ((1) (1) . 0)</code> \u000A \u000A <p>Recall (see <see topic=\"ACL2____CLAUSE-IDENTIFIER\">clause-identifier</see>) that <v>parse-clause-id</v> maps from a \u000A goal spec to a clause id, so you can use that function on the goal specs \u000A printed in the failed proof attempt to determine the clause ids in \u000A question.</p> \u000A \u000A <p>So one acceptable <v>test</v> is</p> \u000A \u000A <code>(<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> id '(((1) (3) . 0)\u000A                   ((1) (2) . 0)\u000A                   ((1) (1) . 0)))</code> \u000A \u000A <p>or you could use <v>parse-clause-id</v> in your computed hint if you don't \u000A want to see clause ids in your script:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id (parse-clause-id &quot;[1]Subgoal 3&quot;))\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id (parse-clause-id &quot;[1]Subgoal 2&quot;))\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id (parse-clause-id &quot;[1]Subgoal 1&quot;)))</code> \u000A \u000A <p>or you could use the inverse function (see <see topic=\"ACL2____CLAUSE-IDENTIFIER\">clause-identifier</see>):</p> \u000A \u000A <code>(<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (string-for-tilde-@-clause-id-phrase id)\u000A              '(&quot;[1]Subgoal 3&quot;\u000A                &quot;[1]Subgoal 2&quot;\u000A                &quot;[1]Subgoal 1&quot;))</code> \u000A \u000A <p>Recall that what we've shown above are the tests to use in the computed \u000A hint.  The hint itself is <v>(<see topic=\"COMMON-LISP____IF\">if</see> test '(:use lemma42) nil)</v> or something \u000A equivalent like <v>(<see topic=\"COMMON-LISP____AND\">and</see> test '(:use lemma42))</v>.</p> \u000A \u000A <p>The three tests above are all equivalent.  They suffer from the problem of \u000A requiring the explicit enumeration of all the goal specs in the first forcing \u000A round.  A change in the script might cause more forced subgoals and the ones \u000A other than those enumerated would not be given the hint.</p> \u000A \u000A <p>You could write a test that recognizes all first round top-level subgoals \u000A no matter how many there are.  Just think of the programming problem: how do I \u000A recognize all the clause id's of the form <v>((1) (n) . 0)</v>?  Often you can \u000A come to this formulation of the problem by using <v>parse-clause-id</v> on a few \u000A of the candidate goal-specs to see the common structure.  A suitable test in \u000A this case is:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> id) '(1))     ; forcing round 1, top-level (pre-induction)\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> id)) 1) ; Subgoal n (<see topic=\"COMMON-LISP____NOT\">not</see> Subgoal n.i ...)\u000A     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> id) 0))      ; no primes</code> \u000A \u000A <p>The test above is ``overkill'' because it recognizes precisely the clause \u000A ids in question.  But recall that once a computed hint is used, it is (by \u000A default) removed from the hints available to the children of the clause. \u000A Thus, we can widen the set of clause ids recognized to include all the \u000A children without worrying that the hint will be applied to those children.</p> \u000A \u000A <p>In particular, the following test supplies the hint to every top-level goal \u000A of the first forcing round:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> id) '(1))</code> \u000A \u000A <p>You might worry that it would also supply the hint to the subgoal produced \u000A by the hint -- the cases we ruled out by the ``overkill'' above.  But that \u000A doesn't happen since the hint is unavailable to the children.  You could even \u000A write:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> id)) 1)</code> \u000A \u000A <p>which would supply the hint to every goal of the form \"[1]Subgoal ...\" \u000A and again, because we see and fire on the top-level goals first, we will not \u000A fire on, say, \"[1]Subgoal *1.3/2\", i.e., the id '((1 1 3) (2) . 0) even \u000A though the test recognizes that id.</p> \u000A \u000A <p>Finally, the following test supplies the hint to every top-level goal of \u000A every forcing round (except the 0th, which is the ``gist'' of the proof, not \u000A ``really'' a forcing round):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> id)) 0))</code> \u000A \u000A <p>Recall again that in all the examples above we have exhibited the <v>test</v> \u000A in a computed hint of the form <v>(<see topic=\"COMMON-LISP____IF\">if</see> test '(:key1 val1 ...) nil)</v>.</p>"],

 "ACL2____HONS-ACONS":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-acons</v> is like <tt><see topic=\"COMMON-LISP____ACONS\">acons</see></tt> except that its guard does \u000A not require <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>; we leave it enabled and would think it odd to ever \u000A prove a theorem about it.</p> \u000A \u000A <p>Under the hood, two things are done differently.  First, the key is copied \u000A with <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt>; this lets us use EQL-based hash tables instead of \u000A EQUAL-based hash tables for better performance.  Second, assuming there is a \u000A valid hash table associated with this alist, we destructively update the hash \u000A table by binding key to val.  The hash table will no longer be associated with \u000A alist, but will instead be associated with the new alist returned by \u000A <v>hons-acons</v>.</p> \u000A \u000A <p>Hash Table Creation</p> \u000A \u000A <p>A new hash table is created by <v>hons-acons</v> whenever alist is an atom. \u000A Unlike ordinary <v>acons</v>, we do not require that alist be nil, and in fact \u000A you may wish to use a non-nil value for one of two reasons.</p> \u000A \u000A <p>1.  As a size hint</p> \u000A \u000A <p>By default, the new hash table will be given a quite modest default \u000A capacity of 60 elements.  As more elements are added, the table may need to be \u000A resized to accommodate them, and this resizing has some runtime cost.</p> \u000A \u000A <p>When a natural number is used as a fast alist's name, we interpret it as a \u000A size hint.  For example, <v>(<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> 'foo 'bar 1000)</v> instructs us to use \u000A 1000 as the initial size for this hash table and binds 'foo to 'bar.  The \u000A resulting table should not need to be resized until more than 1000 elements \u000A are added.  We ignore size hints that request fewer than 60 elements.</p> \u000A \u000A <p>Because of hash collisions, hash tables typically need to have a larger \u000A size than the actual number of elements they contain.  The hash tables for \u000A fast alists are told to grow when they reach 70% full.  So, an easy rule of \u000A thumb might be: multiply the expected number of elements you need by 1.5 to \u000A keep your hash tables about 2/3 full.</p> \u000A \u000A <p>2.  As an alist name</p> \u000A \u000A <p>We also frequently use a symbol for alist, and think of this symbol as the \u000A name of the new alist.  We have found that naming alists can be valuable for \u000A two reasons:</p> \u000A \u000A <p>First, the name can be helpful in identifying fast alists that should have \u000A been freed, see <tt><see topic=\"ACL2____FAST-ALIST-SUMMARY\">fast-alist-summary</see></tt>.</p> \u000A \u000A <p>Second, names can sometimes be used to avoid a subtle and insidious \u000A table-stealing phenomenon that occurs when using fast-alists that are \u000A themselves normed; see <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt>.</p> \u000A \u000A <p>At the moment, there is no way to simultaneously name a fast alist and also \u000A give it a size hint.  We may eventually allow strings to include embedded name \u000A and size components, but for now we have not implemented this \u000A capability.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-acons</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-acons (key val alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> key val) alist))</code>"],

 "ACL2____GENERALIZING-KEY-CHECKPOINTS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Suppose <v>MEMBER</v> determines whether its first argument is a \u000A member of its second, and <v>SUBSETP</v> determines whether every element of its \u000A first argument is a member of its second.  Suppose that you're trying to prove \u000A some Main Theorem and are told the formula below is a key checkpoint.  What \u000A should you do?</p> \u000A \u000A <code>Key Checkpoint:\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> A)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A))\u000A              (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A) B)\u000A              (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) B)\u000A              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> B C))))\u000A         (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A) C))</code> \u000A \u000A <p>The key observation is that the third hypothesis implies the negation of \u000A the fourth.  Writing it in the contrapositive:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> ...\u000A              (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) B)\u000A              ...)\u000A         (<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> B C)))</code> \u000A \u000A <p>In fact, that is more complicated than it needs to be.  A ``correct'' \u000A response to the key checkpoint above is to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a b)\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)))).</code> \u000A \u000A <p>A still better response is to prove this:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a b)\u000A               (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a c))\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> b c)))).</code> \u000A \u000A <p>This version is better because it handles both of the possibilities \u000A regarding whether <v>a</v> is a subset of the arguments of the <v>append</v>.</p> \u000A \u000A <p>It would be nice if we could think of a ``strong'' version, one in which \u000A <v>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> a (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> b c))</v> is rewritten to some clearly simpler term, but \u000A nothing comes to mind.</p> \u000A \u000A <p>In any case, if you cannot see any obvious simplification of the individual \u000A terms in the Key Checkpoint, you should ask yourself whether there are \u000A connections beween the various propositional parts (as here, with the third \u000A and fourth hypotheses).  It is a good heuristic to look for relations between \u000A parts with the same top-level function symbol (as here, with <v>SUBSETP</v>). \u000A It is also a good heuristic to throw out parts of the formula that seem \u000A disconnected (as here, with the terms involving <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> A)</v>).</p> \u000A \u000A <p>This discussion suggests several ``modes'' of looking at key checkpoints \u000A and the idea of trying the modes in sequence:</p> \u000A \u000A <p>(1) look for simplifiable combinations of function symbols within \u000A propositional components,</p> \u000A \u000A <p>(2) look for relations between propositional components, and</p> \u000A \u000A <p>(3) throw out irrelevant or weakly connected components.</p> \u000A \u000A <p>In all cases you are bringing to bear your intuitions about the \u000A <i>semantics</i> of the terms.  That is, you're not just slinging symbols. \u000A You should be looking out for obvious truths about individual parts of the \u000A checkpoints...  truths that are obvious to you but not to ACL2!</p> \u000A \u000A <p>Ultimately the three ``modes'' are the same and we do not really recommend \u000A adhering to the given sequence.  You're just looking for simpler theorems \u000A that, in combination, imply the checkpoint.  Keeping the ``modes'' in mind may \u000A help focus your attention so you consider all the plausible possibilities. \u000A After a little experience you'll find yourself looking for all these things \u000A simultaneously as part ``cleaning up'' the checkpoints.</p> \u000A \u000A <p>When your main goal theorems are harder than these, your main concern will \u000A be looking at a Key Checkpoint and asking yourself ``why is this true?''  But \u000A you don't want to do that until you've cleaned it up ``locally'' as much as \u000A possible and sometimes -- more often than you might think -- local \u000A considerations can prove many of your checkpoints.</p> \u000A \u000A <p>If you have been working your way through the tutorial introduction to the \u000A theorem prover, use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p>"],

 "ACL2____DEFEVALUATOR":[["Events"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list\u000A  ((<see topic=\"COMMON-LISP____LENGTH\">length</see> x) (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x y)))</code> \u000A \u000A <p>See <see topic=\"ACL2____META\">meta</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> ev ev-list\u000A   ((g1 x1 ... xn_1)\u000A    ...\u000A    (gk x1 ... xn_k))</code> \u000A \u000A <p>where <v>ev</v> and <v>ev</v>-list are new function symbols and <v>g1</v>, ..., \u000A <v>gk</v> are old function symbols with the indicated number of formals, i.e., \u000A each <v>gi</v> has <v>n_i</v> formals.</p> \u000A \u000A <p>This function provides a convenient way to constrain <v>ev</v> and \u000A <v>ev-list</v> to be mutually-recursive evaluator functions for the symbols \u000A <v>g1</v>, ..., <v>gk</v>.  Roughly speaking, an evaluator function for a fixed, \u000A finite set of function symbols is a restriction of the universal evaluator to \u000A terms composed of variables, constants, lambda expressions, and applications \u000A of the given functions.  However, evaluator functions are constrained rather \u000A than defined, so that the proof that a given metafunction is correct vis-a-vis \u000A a particular evaluator function can be lifted (by functional instantiation) to \u000A a proof that it is correct for any larger evaluator function.  See <see topic=\"ACL2____META\">meta</see> \u000A for a discussion of metafunctions.</p> \u000A \u000A <p><v>Defevaluator</v> executes an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> after generating the \u000A appropriate <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events.  Perhaps the easiest way \u000A to understand what <v>defevaluator</v> does is to execute the keyword \u000A command</p> \u000A \u000A <code>:trans1 (<see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see> evl evl-list ((<see topic=\"COMMON-LISP____LENGTH\">length</see> x) (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> x y)))</code> \u000A \u000A <p>and inspect the output.  This trick is also useful in the rare case that \u000A the event fails because a hint is needed.  In that case, the output of \u000A <v>:</v><tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> can be edited by adding hints, then submitted \u000A directly.</p> \u000A \u000A <p>Formally, <v>ev</v> is said to be an ``evaluator function for <v>g1</v>, ..., \u000A <v>gk</v>, with mutually-recursive counterpart <v>ev-list</v>'' iff <v>ev</v> and \u000A <v>ev-list</v> are constrained functions satisfying just the <see topic=\"ACL2____CONSTRAINT\">constraint</see>s \u000A discussed below.</p> \u000A \u000A <p><v>Ev</v> and <v>ev-list</v> must satisfy <see topic=\"ACL2____CONSTRAINT\">constraint</see>s (0)-(4) and \u000A (k):</p> \u000A \u000A <code>(0) How to ev an arbitrary function application:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                  (<see topic=\"ACL2____SYNTAXP\">syntaxp</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a ''nil)))\u000A                  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'quote)))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a)\u000A                    (ev (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                              (<see topic=\"ACL2____KWOTE-LST\">kwote-lst</see> (ev-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) a)))\u000A                        nil)))\u000A\u000A(1) How to ev a variable symbol:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a) (<see topic=\"COMMON-LISP____AND\">and</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> x a)))))\u000A\u000A(2) How to ev a constant:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'quote))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a) (<see topic=\"COMMON-LISP____CADR\">cadr</see> x)))\u000A\u000A(3) How to ev a lambda application:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                  (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a)\u000A                    (ev (<see topic=\"COMMON-LISP____CADDAR\">caddar</see> x)\u000A                        (<see topic=\"ACL2____PAIRLIS_42\">pairlis$</see> (<see topic=\"COMMON-LISP____CADAR\">cadar</see> x)\u000A                                  (ev-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) a)))))\u000A\u000A(4) How to ev an argument list:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x-lst)\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev-list x-lst a)\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> (ev (<see topic=\"COMMON-LISP____CAR\">car</see> x-lst) a)\u000A                          (ev-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x-lst) a))))\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x-lst))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev-list x-lst a)\u000A                    nil))\u000A\u000A(k) For each i from 1 to k, how to ev an application of gi,\u000A    where gi is a function symbol of n arguments:\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'gi))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ev x a)\u000A                    (gi (ev x1 a)\u000A                        ...\u000A                        (ev xn a)))),\u000A    where xi is the (cad...dr x) expression equivalent to (<see topic=\"COMMON-LISP____NTH\">nth</see> i x).</code> \u000A \u000A <p><v>Defevaluator</v> defines suitable witnesses for <v>ev</v> and <v>ev-list</v>, \u000A proves the theorems about them, and constrains <v>ev</v> and <v>ev-list</v> \u000A appropriately.  We expect <v>defevaluator</v> to work without assistance from \u000A you, though the proofs do take some time and generate a lot of output.  The \u000A proofs are done in the context of a fixed theory, namely the value of the \u000A constant <v>*defevaluator-form-base-theory*</v>.</p> \u000A \u000A <p>(Aside: (3) above may seem surprising, since the bindings of <v>a</v> are not \u000A included in the environment that is used to evaluate the lambda body, \u000A <v>(<see topic=\"COMMON-LISP____CADDAR\">caddar</see> x)</v>.  However, ACL2 lambda expressions are all <i>closed</i>: in \u000A <v>(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (v1 ... vn) body)</v>, the only free variables in <v>body</v> are among \u000A the <v>vi</v>.  See <see topic=\"ACL2____TERM\">term</see>.)</p>"],

 "COMMON-LISP____PROGN":[["Events"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x) x)\u000A       (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> my-defun (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> 'defun args))\u000A       (my-defun bar (x) (foo x)))\u000A\u000AGeneral form:\u000A(<see topic=\"COMMON-LISP____PROGN\">progn</see> event1 event2 ... eventk)</code> \u000A \u000A <p>where <v>k</v> &gt;= 0 and each <v>eventi</v> is a legal embedded event form \u000A (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).  Each event is printed (by default) and \u000A evaluated, in sequence.  If any event fails, the entire <v>progn</v> call is \u000A deemed to have failed, and the logical <see topic=\"ACL2____WORLD\">world</see> is rolled back to what it \u000A was immediately before the <v>progn</v> call was evaluated.  A utility is \u000A provided to assist in debugging failures of such execution; see <see topic=\"ACL2____REDO-FLAT\">redo-flat</see>.</p> \u000A \u000A <p>NOTE: If the <v>eventi</v> above are not all legal embedded event forms (see \u000A <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>), consider using <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> or (with great \u000A care!) <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> instead.  If your goal is simply to execute a sequence \u000A of top-level forms that are not necessarily all legal embedded event forms, \u000A consider using <v>ld</v> instead; see <see topic=\"ACL2____LD\">ld</see>.</p> \u000A \u000A <p>For a related event form that does allow introduction of <see topic=\"ACL2____CONSTRAINT\">constraint</see>s \u000A and <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>ACL2 does not allow the use of <v>progn</v> in definitions.  Instead, the \u000A macro <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> can be used for sequencing <see topic=\"ACL2____STATE\">state</see>-oriented \u000A operations; see <see topic=\"ACL2____ER-PROGN\">er-progn</see> and see <see topic=\"ACL2____STATE\">state</see>.  If you are using \u000A single-threaded objects (see <see topic=\"ACL2____STOBJ\">stobj</see>) you may wish to define a version of \u000A <tt><see topic=\"ACL2____ER-PROGN\">er-progn</see></tt> that cascades the object through successive changes.  ACL2's \u000A <tt><see topic=\"ACL2____PPROGN\">pprogn</see></tt> is the <v>state</v> analogue of such a macro.</p>"],

 "ACL2____TIDBITS":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BOOKS\">books</see> for a discussion of books.  Briefly, a book is a \u000A file whose name ends in ``.lisp'' that contains ACL2 <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____HISTORY\">history</see> for a list of useful commands.  Some examples:</p> \u000A \u000A <code>:pbt :here      ; print the current event\u000A:pbt (:here -3) ; print the last four events\u000A:u              ; undo the last event\u000A:pe append      ; print the definition of append</code> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> to learn how to print documentation to the \u000A terminal.  There are also versions of the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for web \u000A browsers and for Emacs (see <see topic=\"ACL2____ACL2-DOC\">ACL2-Doc</see>).</p> \u000A \u000A <p>There are quite a few kinds of rules allowed in ACL2 besides <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules, though we hope that beginners won't usually need to be aware \u000A of them.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for details.  In particular, there is \u000A support for <see topic=\"ACL2____CONGRUENCE\">congruence</see> rewriting.  See <see topic=\"ACL2____RUNE\">rune</see> (``RUle NamE'') for \u000A a description of the various kinds of rules in the system.  Also see <see topic=\"ACL2____THEORIES\">theories</see> for a description of how to build <see topic=\"ACL2____THEORIES\">theories</see> of <see topic=\"ACL2____RUNE\">rune</see>s, \u000A which are often used in hints; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>A ``<see topic=\"ACL2____PROGRAMMING\">programming</see> mode'' is supported; see <see topic=\"ACL2____PROGRAM\">program</see>, see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, and see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>.  It can be useful to prototype \u000A functions after executing the command <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, which will cause \u000A definitions to be syntaxed-checked only.</p> \u000A \u000A <p>ACL2 supports mutual recursion, though this feature is not tied into the \u000A automatic discovery of <see topic=\"ACL2____INDUCTION\">induction</see> schemas and is often not the best way \u000A to proceed when you expect to be reasoning about the functions.  See <see topic=\"ACL2____DEFUNS\">defuns</see>; also see <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____LD\">ld</see> for discussion of how to load files of <see topic=\"ACL2____EVENTS\">events</see>.  There \u000A are many options to <tt><see topic=\"ACL2____LD\">ld</see></tt>, including ones to suppress proofs and to \u000A control output.</p> \u000A \u000A <p>The <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> (Onward Thru the Fog FLaG) is a useful feature \u000A that Nqthm users have often wished for.  It prevents the prover from aborting \u000A a proof attempt and inducting on the original conjecture.  See <see topic=\"ACL2____OTF-FLG\">otf-flg</see>.</p> \u000A \u000A <p>ACL2 supports redefinition and redundancy in <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see> and see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p> \u000A \u000A <p>A <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> display feature is available for use with Emacs.  This \u000A feature provides a view of ACL2 proofs that can be much more useful than \u000A reading the stream of <see topic=\"ACL2____CHARACTERS\">characters</see> output by the theorem prover as its \u000A ``proof.''  See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>An interactive feature similar to Pc-Nqthm is supported in ACL2.  See <see topic=\"ACL2____VERIFY\">verify</see> and see <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.</p> \u000A \u000A <p>ACL2 allows you to <see topic=\"ACL2____MONITOR\">monitor</see> the use of <see topic=\"ACL2____REWRITE\">rewrite</see> rules.  See \u000A <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____ARRAYS\">arrays</see> to read about applicative, fast <see topic=\"ACL2____ARRAYS\">arrays</see> in \u000A ACL2.</p> \u000A \u000A <p>To quit the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop, or (in akcl) to return to the ACL2 \u000A <see topic=\"ACL2____COMMAND\">command</see> loop after an interrupt, type <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt>.  To continue \u000A (resume) after an interrupt (in akcl), type <v>:r</v>.  To cause an interrupt \u000A (in akcl under Unix (trademark of AT&amp;T)), hit control-C (twice, if inside \u000A Emacs).  To exit ACL2 altogether, first type <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt> to exit the ACL2 \u000A <see topic=\"ACL2____COMMAND\">command</see> loop, and then exit Lisp (by typing <v>(user::bye)</v> in \u000A akcl).</p> \u000A \u000A <p>See <see topic=\"ACL2____STATE\">state</see> to read about the von Neumannesque ACL2 <see topic=\"ACL2____STATE\">state</see> \u000A object that records the ``current state'' of the ACL2 session.  Also see <see topic=\"ACL2_____04\">@</see>, and see <see topic=\"ACL2____ASSIGN\">assign</see>, to learn about reading and setting global <see topic=\"ACL2____STATE\">state</see> variables.</p> \u000A \u000A <p>If you want your own von Neumannesque object, e.g., a structure that can be \u000A ``destructively modified'' but which must be used with some syntactic \u000A restrictions, see <see topic=\"ACL2____STOBJ\">stobj</see>.</p>"],

 "ACL2____MACRO-ALIASES-TABLE":[["Macros"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TABLE\">table</see> macro-aliases-table 'append 'binary-append)</code> \u000A \u000A <p>This example associates the function symbol <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> with the \u000A macro name <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>.  As a result, the name <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> may be used \u000A as a runic designator (see <see topic=\"ACL2____THEORIES\">theories</see>) by the various theory functions. \u000A Thus, for example, it will be legal to write</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> append))</code> \u000A \u000A <p>as an abbreviation for</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> binary-append))</code> \u000A \u000A <p>which in turn really abbreviates</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                                    '(<see topic=\"ACL2____BINARY-APPEND\">binary-append</see>)))\u000A\u000AGeneral Form:\u000A\u000A(<see topic=\"ACL2____TABLE\">table</see> macro-aliases-table 'macro-name 'function-name)</code> \u000A \u000A <p>or very generally</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> macro-aliases-table macro-name-form function-name-form)</code> \u000A \u000A <p>where <v>macro-name-form</v> and <v>function-name-form</v> evaluate, \u000A respectively, to a macro name and a symbol in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>. \u000A See <see topic=\"ACL2____TABLE\">table</see> for a general discussion of tables and the <v>table</v> event \u000A used to manipulate tables.</p> \u000A \u000A <p>Note that <v>function-name-form</v> (above) does not need to evaluate to a \u000A function symbol, but only to a symbol.  As a result, one can introduce the \u000A alias before defining a recursive function, as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> macro-aliases-table 'mac 'fn)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (mac (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A    x))</code> \u000A \u000A <p>Although this is obviously contrived example, this flexibility can be \u000A useful to macro writers; see for example the definition of ACL2 system macro \u000A <tt><see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see></tt>.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____TABLE\">table</see></tt> <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt> is an alist that associates \u000A macro symbols with function symbols, so that macro names may be used as runic \u000A designators (see <see topic=\"ACL2____THEORIES\">theories</see>).  For a convenient way to add entries to \u000A this <see topic=\"ACL2____TABLE\">table</see>, see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>.  To remove entries from the \u000A <see topic=\"ACL2____TABLE\">table</see> with ease, see <see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see>.</p> \u000A \u000A <p>This <see topic=\"ACL2____TABLE\">table</see> is used by the theory functions; see <see topic=\"ACL2____THEORIES\">theories</see>. \u000A For example, in order that <v>(<see topic=\"ACL2____DISABLE\">disable</see> append)</v> be interpreted as <v>(<see topic=\"ACL2____DISABLE\">disable</see>\u000A binary-append)</v>, it is necessary that the example form above has been \u000A executed.  In fact, this <see topic=\"ACL2____TABLE\">table</see> does indeed associate many of the macros \u000A provided by the ACL2 system, including <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>, with function symbols. \u000A Loosely speaking, it only does so when the macro is ``essentially the same \u000A thing as'' a corresponding function; for example, <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> and \u000A <v>(<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> x y)</v> represent the same term, for any expressions <v>x</v> \u000A and <v>y</v>.</p>"],

 "ACL2____FINALIZE-EVENT-USER":[["Prover-output"],"ACL2 Sources","ACL2","<p>This utility is intended for system hackers, not standard ACL2 \u000A users.</p> \u000A \u000A <p>ACL2 prints summaries at the conclusions of processing <see topic=\"ACL2____EVENTS\">events</see> \u000A (unless summaries are inhibited; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> and also \u000A see <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see>).  You may arrange for processing to \u000A take place just after the summary, by defining a function with argument list \u000A <v>(ctx body state)</v> that returns one value, namely <v>state</v>.  We describe \u000A <v>ctx</v> and <v>body</v> at the end below, but you may simply prefer to ignore \u000A these arguments.)  Your function should normally be a <see topic=\"ACL2____GUARD\">guard</see>-verified \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function with no guard other than that provided by \u000A the input requirement on <tt><see topic=\"ACL2____STATE\">state</see></tt>, that is, <v>(state-p state)</v>; but \u000A later below we discuss how to avoid this requirement.  You then attach (see \u000A <see topic=\"ACL2____DEFATTACH\">defattach</see>) your function to the function <v>finalize-event-user</v>.  The \u000A following example illustrates how this all works.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> finalize-event-user-test (ctx body state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state)\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> ctx body))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (boundp-global 'abbrev-evisc-tuple state)\u000A              (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL-P\">open-output-channel-p</see> *standard-co*\u000A                                     :character\u000A                                     state))\u000A         (<see topic=\"ACL2____PPROGN\">pprogn</see>\u000A          (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'abbrev-evisc-tuple state) :DEFAULT)\u000A              (<see topic=\"ACL2____PRINC_42\">princ$</see> &quot;Abbrev-evisc-tuple has its default value.~%&quot;\u000A                      *standard-co*\u000A                      state)\u000A            (<see topic=\"ACL2____PRINC_42\">princ$</see> &quot;Abbrev-evisc-tuple has been modified.~%&quot;\u000A                    *standard-co*\u000A                    state))))\u000A        (t state)))\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> finalize-event-user finalize-event-user-test)</code> \u000A \u000A <p>After admission of the two events above, an event summary will conclude \u000A with extra printout, for example:</p> \u000A \u000A <code>Note: Abbrev-evisc-tuple has its default value.</code> \u000A \u000A <p>If the attachment function (above, <v>finalize-event-user-test</v>) does not \u000A meet all the requirements stated above, then you can use the <v>:skip-checks</v> \u000A argument of <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> to get around the requirement, as illustrated by \u000A the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> finalize-event-user-test2 (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                  :mode :program)\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> ctx body))\u000A  (<see topic=\"ACL2____OBSERVATION\">observation</see>\u000A   'my-test\u000A   &quot;~|Value of term-evisc-tuple: ~x0~|&quot;\u000A   (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'term-evisc-tuple state)))\u000A\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; needed for :skip-checks t\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> (<see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see> finalize-event-user-test2)\u000A           :skip-checks t)</code> \u000A \u000A <p>So for example:</p> \u000A \u000A <code>ACL2 !&gt;(set-term-evisc-tuple (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 2 7 nil nil) state)\u000A (:TERM)\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *foo6* '(a b c))\u000A\u000ASummary\u000AForm:  ( DEFCONST *FOO6* ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AACL2 Observation in MY-TEST:\u000AValue of term-evisc-tuple: (NIL 2 7 NIL)\u000A *FOO6*\u000AACL2 !&gt;</code> \u000A \u000A <p>Note that (as of this writing) the macro <tt><see topic=\"ACL2____OBSERVATION\">observation</see></tt> expands to a \u000A call of a <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>-mode function.  Thus, the trick shown above \u000A involving <v>:skip-checks</v> allows the use of <v>:program</v>-mode functions; \u000A for example, you can print with <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.</p> \u000A \u000A <p>See community book <v>books/misc/defattach-bang.lisp</v> for a variant of \u000A <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> that uses <tt><see topic=\"ACL2____EC-CALL\">ec-call</see></tt> to avoid issues of <see topic=\"ACL2____GUARD\">guard</see> \u000A verification.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____INITIALIZE-EVENT-USER\">initialize-event-user</see>, which discusses the handling of \u000A <see topic=\"ACL2____STATE\">state</see> globals by that utility as well as by \u000A <v>finalize-event-user</v>.</p> \u000A \u000A <p>Finally, as promised above, we briefly describe the arguments <v>ctx</v> and \u000A <v>body</v>.  These are the arguments passed to the call of macro \u000A <v>with-ctx-summarized</v> under which <v>finalize-event-user</v> (or \u000A <v>initialize-event-user</v>) was called.  Thus, they are unevaluated \u000A expressions.  For example, system function <v>defthm-fn1</v> has a body of the \u000A following form.</p> \u000A \u000A <code>(with-ctx-summarized\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (output-in-infixp state) event-form (<see topic=\"COMMON-LISP____CONS\">cons</see> 'defthm name))\u000A (<see topic=\"COMMON-LISP____LET\">let</see> ((wrld (w state))\u000A       (ens (ens state))\u000A       .....</code> \u000A \u000A <p>Thus, when <v>initialize-event-user</v> and <v>finalize-event-user</v> are \u000A called on behalf of <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, <v>ctx</v> is the s-expression</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (output-in-infixp state) event-form (<see topic=\"COMMON-LISP____CONS\">cons</see> 'defthm name))</code> \u000A \u000A <p>while <v>body</v> is the following s-expression (with most code elided).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((wrld (w state))\u000A      (ens (ens state))\u000A      .....</code> \u000A \u000A <p>You might find it helpful to use <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> to get a sense of <v>ctx</v> \u000A and <v>body</v>, for example, <v>(<see topic=\"ACL2____TRACE_42\">trace$</see> finalize-event-user)</v>.</p>"],

 "ACL2____BREAK-LEMMA":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example:\u000A:brr t                          ; if you haven't done that yet\u000A:monitor (:rewrite lemma12) t   ; to install a break point on the\u000A                                ; rule named (:rewrite lemma12)</code> \u000A \u000A <p>ACL2 does not support Nqthm's <v>break-lemma</v> but supports a very similar \u000A and more powerful break facility.  Suppose some proof is failing; apparently \u000A some particular rule is not being used and you wish to learn why.  Then you \u000A need the ACL2 <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> facility.  See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> and all \u000A of its associated <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> topics for details.  The following basic \u000A steps are required.</p> \u000A \u000A <p>(1) To enable the ``break rewrite'' feature, you must first execute</p> \u000A \u000A <code>ACL2 !&gt;:brr t</code> \u000A \u000A <p>at the top-level of ACL2.  Equivalently, evaluate <v>(<see topic=\"ACL2____BRR\">brr</see> t)</v>.  <see topic=\"ACL2____BREAK-REWRITE\">Break-rewrite</see> stays enabled until you disable it with <v>(<see topic=\"ACL2____BRR\">brr</see> nil)</v>.  When \u000A <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> is enabled the ACL2 rewriter will run slower than normal \u000A but you will be able to <see topic=\"ACL2____MONITOR\">monitor</see> the attempts to apply specified \u000A rules.</p> \u000A \u000A <p>(2) Decide what <see topic=\"ACL2____RUNE\">rune</see>s (see <see topic=\"ACL2____RUNE\">rune</see>) you wish to <see topic=\"ACL2____MONITOR\">monitor</see>. \u000A For example, you might want to know why <v>(:rewrite lemma12 . 2)</v> is not \u000A being used in the attempted proof.  That, by the way, is the name of the \u000A second rewrite rule generated from the event named <v>lemma12</v>.</p> \u000A \u000A <p>The command</p> \u000A \u000A <code>ACL2 !&gt;:monitor (:rewrite lemma12 . 2) t</code> \u000A \u000A <p>will install an ``unconditional'' break point on that rule.  The ``<v>t</v>'' \u000A at the end of the command means it is unconditional, i.e., a break will occur \u000A every time the rule is tried.  ACL2 supports conditional breaks also, in which \u000A case the <v>t</v> is replaced by an expression that evaluates to non-<v>nil</v> \u000A when you wish for a break to occur.  See <see topic=\"ACL2____MONITOR\">monitor</see>.  The above keyword \u000A command is, of course, equivalent to</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____MONITOR\">monitor</see> '(:rewrite lemma12 . 2) t)</code> \u000A \u000A <p>which you may also type.  You may install breaks on as many rules as you \u000A wish.  You must use <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt> on each rule.  You may also change the \u000A break condition on a rule with <tt><see topic=\"ACL2____MONITOR\">monitor</see></tt>.  Use <tt><see topic=\"ACL2____UNMONITOR\">unmonitor</see></tt> (see \u000A <see topic=\"ACL2____UNMONITOR\">unmonitor</see>) to remove a rule from the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed \u000A rules.</p> \u000A \u000A <p>(3) Then try the proof again.  When a <see topic=\"ACL2____MONITOR\">monitor</see>ed rule is tried by the \u000A rewriter you will enter an interactive break, called <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>. \u000A See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> for a detailed description.  Very simply, <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> lets you inspect the context of the attempted application both \u000A before and after the attempt.  When <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> is entered it will \u000A print out the ``target'' term being rewritten.  If you type <v>:go</v> <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> will try the rule and then exit, telling you (a) whether the \u000A rule was applied, (b) if so, how the target was rewritten, and (c) if not, why \u000A the rule failed.  There are many other commands.  See <see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see>.</p> \u000A \u000A <p>(4) When you have finished using the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> feature you \u000A should disable it to speed up the rewriter.  You can disable it with</p> \u000A \u000A <code>ACL2 !&gt;:brr nil</code> \u000A \u000A <p>The list of <see topic=\"ACL2____MONITOR\">monitor</see>ed rules and their break conditions persists but \u000A is ignored.  If you enable <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> later, the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed rules will be displayed and will be used again by rewrite.</p> \u000A \u000A <p>You should disable the <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> feature whenever you are not \u000A intending to use it, even if the list of <see topic=\"ACL2____MONITOR\">monitor</see>ed rules is empty, \u000A because the rewriter is slowed down as long as <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> is \u000A enabled.</p> \u000A \u000A <p>If you get a stack overflow, see <see topic=\"ACL2____CW-GSTACK\">cw-gstack</see>.</p>"],

 "ACL2____NOTE-2-6-RULES":[["Note-2-6"],"ACL2 Sources","ACL2","<p>The following symbols have been added to the list constant \u000A <v>*common-lisp-specials-and-constants*</v>: <v>REPLACE</v>, <v>FILL</v>, \u000A <v>CHARACTER</v>, <v>=</v>, <v>BREAK</v>, and <v>PRIN1</v>.  This was done in support \u000A of ports to Allegro 6.0 and Windows platforms (see <see topic=\"ACL2____NOTE-2-6-NEW-FUNCTIONALITY\">note-2-6-new-functionality</see>).</p> \u000A \u000A <p>The list of symbols in <v>*acl2-exports*</v> has been modified, for example \u000A to include <v>show-accumulated-persistence</v> and the legal arguments to <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>.</p> \u000A \u000A <p>Functions <tt><see topic=\"ACL2____ZP\">zp</see></tt> and <tt><see topic=\"ACL2____ZIP\">zip</see></tt> are now handled slightly differently. \u000A They are are now disabled, but each comes with a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule \u000A that allows their expansion on non-variable terms, and also with a \u000A <v>:</v><tt><see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see></tt> rule that avoids the need for opening up \u000A these functions when applied to variables.  The resulting behavior should be \u000A very similar to the behavior of previous versions, except that case splits \u000A will be avoided when these functions are applied to variables.</p> \u000A \u000A <p>Function <tt><see topic=\"ACL2____STANDARD-STRING-ALISTP\">standard-string-alistp</see></tt> replaces function \u000A <v>string-alistp</v>.  For further discussion, see <see topic=\"ACL2____NOTE-2-6-GUARDS\">note-2-6-guards</see>.</p> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> whose conclusion is a term of the form \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> lhs rhs)</v> have always been stored in the expected way: <v>lhs</v> \u000A rewrites to <v>rhs</v>.  This way of storing <v>:rewrite</v> rules has been \u000A extended to allow <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>, <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, or <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> in place of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.</p> \u000A \u000A <p>Rewrite rule <v>nth-update-nth</v>, in source file <v>axioms.lisp</v>, has been \u000A strengthened.</p> \u000A \u000A <p>A new rewrite rule <v>equal-constant-+</v> has been added to the book \u000A <v>arithmetic/equalities</v>.  This should generally be a beneficial change, but \u000A existing proofs involving the arithmetic books could conceivably be \u000A affected.</p> \u000A \u000A <p>Function <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> and constant \u000A <v>*main-lisp-package-name*</v> have undergone small changes.  This change \u000A should rarely be noticed by users and is discussed elsewhere; see <see topic=\"ACL2____NOTE-2-6-SYSTEM\">note-2-6-system</see>.</p> \u000A \u000A <p>We mention here that proofs involving <see topic=\"ACL2____STOBJ\">stobj</see>s may need to be modified \u000A because of changes in auxiliary functions generated by <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>. \u000A (These changes were made in support of a new resizing capability, mentioned \u000A elsewhere in these release notes; see <see topic=\"ACL2____NOTE-2-6-NEW-FUNCTIONALITY\">note-2-6-new-functionality</see>.</p> \u000A \u000A <p>In the distributed book directory <v>books/arithmetic/</v>, the book \u000A <v>rationals-with-axioms-proved.lisp</v> has been renamed \u000A <v>rationals.lisp</v>.</p> \u000A \u000A <p>(ACL2(r) only) Rewrite rules <v>realp-+</v>, <v>realp-*</v>, \u000A <v>realp-unary--</v>, and <v>realp-unary-/</v> have been added in analogy to \u000A existing rules <v>rationalp-+</v>, <v>rationalp-*</v>, <v>rationalp-unary--</v>, and \u000A <v>rationalp-unary-/</v>.  Thanks to Jun Sawada for suggesting this change.</p> \u000A \u000A <p>The definition of <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> has been modified slightly.  Previously, if \u000A <v>*my-a*</v> were an array then <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'some-name *my-a* :header)</v> would \u000A evaluate to the <v>cdr</v> of the <tt><see topic=\"ACL2____HEADER\">header</see></tt> of <v>*my-a*</v> rather than to \u000A its <tt><see topic=\"ACL2____DEFAULT\">default</see></tt>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p>Changes have been made in the <v>ihs</v> books, based on suggestions from Jun \u000A Sawada, that support its use with ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).  The primary \u000A change is to replace calls of <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt> with calls of <tt><see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see></tt>, which should have no effect on users of standard ACL2.</p>"],

 "ACL2____ANNOTATED-ACL2-SCRIPTS":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Beginning users may find these annotated scripts useful.  We \u000A suggest that you read these in the following order:</p> \u000A \u000A <code><see topic=\"ACL2____TUTORIAL1-TOWERS-OF-HANOI\">Tutorial1-Towers-of-Hanoi</see> \u000A<see topic=\"ACL2____TUTORIAL2-EIGHTS-PROBLEM\">Tutorial2-Eights-Problem</see> \u000A<see topic=\"ACL2____TUTORIAL3-PHONEBOOK-EXAMPLE\">Tutorial3-Phonebook-Example</see> \u000A<see topic=\"ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE\">Tutorial4-Defun-Sk-Example</see> \u000A<see topic=\"ACL2____TUTORIAL5-MISCELLANEOUS-EXAMPLES\">Tutorial5-Miscellaneous-Examples</see> \u000A</code> \u000A \u000A <p>You can also find useful demos in the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> directory, \u000A <v>books/demos/</v>, and its subdirectories.</p> \u000A \u000A <p>The web page <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/tutorial/rev3.html'>Brief \u000A ACL2 Tutorial</a> contains a script that illustrates how it feels to use The \u000A Method to prove an unusual list reverse function correct.  The screen shots of \u000A ACL2's proof output are outdated -- in the version shown, ACL2 does not print \u000A Key Checkpoints, but the concept of key checkpoint is clear in the discussion \u000A and the behavior of the user.</p> \u000A \u000A <p>See <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/contrib/POLISHING-PROOFS-TUTORIAL.html'>Polishing \u000A Proofs Tutorial</a> for a tutorial on becoming successful at approaching a \u000A formalization and proof problem in ACL2.  That tutorial, written by Shilpi \u000A Goel and Sandip Ray, has two parts: it illustrates how to guide the theorem \u000A prover to a successful proof, and it shows how to clean up the proof in order \u000A to facilitate maintenance and extension of the resulting book (see <see topic=\"ACL2____BOOKS\">books</see>).</p> \u000A \u000A <p>The <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/tutorial/kaufmann-TPHOLs08/index.html'>ACL2 \u000A Demo Given at TPHOLs 2008</a> by Matt Kaufmann includes scripts and a gzipped \u000A tar file containing the entire contents of the demos.</p> \u000A \u000A <p>The <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/tutorial/sort-equivalence'>sort \u000A equivalence demo</a> is a collection of scripts illustrating both high-level \u000A strategy and lower-level tactics dealing with the functional equivalence of \u000A various list sorting algorithms.  Start with the <v>README</v> on that \u000A directory.  There is also a <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/tutorial/sort-equivalence.tgz'>gzipped \u000A tar file</a> with all of these scripts.</p> \u000A \u000A <p>When you feel you have read enough examples, you might want to try the \u000A following very simple example on your own. (See <see topic=\"ACL2____SOLUTION-TO-SIMPLE-EXAMPLE\">solution-to-simple-example</see> for a solution, after you work on this example.) \u000A First define the notion of the ``fringe'' of a tree, where we identify trees \u000A simply as <see topic=\"COMMON-LISP____CONS\">cons</see> structures, with <see topic=\"COMMON-LISP____ATOM\">atom</see>s at the leaves.  For \u000A example:</p> \u000A \u000A <code>ACL2 !&gt;(fringe '((a . b) c . d))\u000A(A B C D)</code> \u000A \u000A <p>Next, define the notion of a ``leaf'' of a tree, i.e., a predicate \u000A <v>leaf-p</v> that is true of an atom if and only if that atom appears at the \u000A tip of the tree.  Define this notion without referencing the function \u000A <v>fringe</v>.  Finally, prove the following theorem, whose proof may well be \u000A automatic (i.e., not require any lemmas).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> leaf-p-iff-member-fringe\u000A  (<see topic=\"ACL2____IFF\">iff</see> (leaf-p atm x)\u000A       (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> atm (fringe x))))</code>"],

 "ACL2____ADD-CUSTOM-KEYWORD-HINT":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see> :my-hint (my-hint-fn val ...))\u000A\u000A(<see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see> :my-hint\u000A                         (my-hint-fn val ...)\u000A                         :checker (my-hint-checker-fn val ...))</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see> :key term1 :checker term2)</code> \u000A \u000A <p>where <v>:key</v> is a <tt><see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see></tt> not among the primitive keyword hints \u000A listed in <v>*hint-keywords*</v>, the <v>:checker</v> argument is optional, and \u000A <v>term1</v> and (if supplied) <v>term2</v> are terms with certain free-variable \u000A and signature restrictions described below.  Henceforth, <v>:key</v> is treated \u000A as a custom keyword hint, e.g., the user can employ <v>:key</v> in hints to \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, such as:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> name ...\u000A  :hints ((&quot;Subgoal *1/1'&quot; ... :key val ...))).</code> \u000A \u000A <p>Custom keyword hints are complicated.  To use them you must understand \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>, multiple values (e.g., <tt><see topic=\"ACL2____MV\">mv</see></tt> and <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>), ACL2's \u000A notion of error triples (see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>), how to generate \u000A ``soft'' errors with <tt><see topic=\"ACL2____ER\">er</see></tt>, how to use <tt><see topic=\"ACL2____FMT\">fmt</see></tt>-strings to control \u000A output, how to use computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) and some aspects \u000A of ACL2's internal event processing.  Furthermore, it is possible to implement \u000A a custom keyword hint that can make an event non-reproducible!  So we \u000A recommend that these hints be developed by ACL2 experts.  Basically the custom \u000A keyword feature allows the implementors and other experts to extend the hint \u000A facility without modifying the ACL2 sources.</p> \u000A \u000A <p><v>Term1</v> is called the ``generator'' term and <v>term2</v> is called the \u000A ``checker'' term of the custom keyword hint <v>:key</v>.  Together they specify \u000A the semantics of the new custom keyword hint <v>:key</v>.  Roughly speaking, \u000A when a custom keyword hint is supplied by the user, as in</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> name ...\u000A  :hints ((&quot;Subgoal *1/1'&quot; ... :my-hint val ...))).</code> \u000A \u000A <p>the checker term is evaluated on <v>val</v> to check that <v>val</v> is of the \u000A expected shape.  Provided <v>val</v> passes the check, the generator term is \u000A used to compute a standard hint.  Like computed hints, the generator of a \u000A custom keyword hint is allowed to inspect the actual clause on which it is \u000A being fired.  Indeed, it is allowed to inspect the entire list of hints \u000A (standard and custom) supplied for that clause.  Thus, in the most general \u000A case, a custom keyword hint is just a very special kind of computed hint.</p> \u000A \u000A <p>The generator, <v>term1</v>, must have no free variables other than:</p> \u000A \u000A <code>(val keyword-alist\u000A id clause world stable-under-simplificationp\u000A hist pspv ctx state).</code> \u000A \u000A <p>Moreover, either <v>term1</v> must evaluate to a single non-<see topic=\"ACL2____STOBJ\">stobj</see> \u000A value, or else it must be single-threaded in <v>state</v> and have the standard \u000A error-triple output signature, <v>(<see topic=\"ACL2____MV\">mv</see> * * state)</v>.</p> \u000A \u000A <p>The restrictions on the checker, <v>term2</v>, are that it be single-threaded \u000A in <v>state</v>, have the standard error-triple output signature, <v>(<see topic=\"ACL2____MV\">mv</see> * *\u000A state)</v>, and have no free variables other than:</p> \u000A \u000A <code>(val world ctx state).</code> \u000A \u000A <p>For examples, see the community books directory <v>books/hints/</v>, in \u000A particular <v>basic-tests.lisp</v>.</p> \u000A \u000A <p>To delete a previously added custom keyword hint, see <see topic=\"ACL2____REMOVE-CUSTOM-KEYWORD-HINT\">remove-custom-keyword-hint</see>.</p> \u000A \u000A <p>The community book <v>hints/merge-hint.lisp</v> can be useful in writing \u000A custom keyword hints.  See the examples near the of the file.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p>"],

 "ACL2____UBT":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:ubt :max      ; undo back through the most recent command\u000A               ; (which just means undo the most recent command)\u000A:ubt :x        ; same as :ubt :max\u000A:u             ; same as :ubt :max with no questions asked\u000A:ubt fn        ; undo back through the introduction of fn\u000A               ; (including all the other events in fn's block)\u000A:ubt 5         ; undo back through the fifth command executed\u000A:ubt (:max -4) ; undo back through the most recent five commands\u000A:ubt (:x -4)   ; undo back through the most recent five commands</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Ubt</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and undoes the \u000A <see topic=\"ACL2____COMMAND\">command</see>s from <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt> (aka <v>:x</v>) through the one \u000A described.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.  <tt><see topic=\"ACL2____PBT\">Pbt</see></tt> will print the <see topic=\"ACL2____COMMAND\">command</see>s that <v>ubt</v> will undo.  <v>:</v><tt><see topic=\"ACL2____OOPS\">Oops</see></tt> will undo the undo. \u000A See <see topic=\"ACL2____OOPS\">oops</see>.</p> \u000A \u000A <p><v>Ubt</v> can cause errors or queries.  To avoid these, see <see topic=\"ACL2____UBT_12\">ubt!</see>.</p> \u000A \u000A <p>It is important to remember that a <see topic=\"ACL2____COMMAND\">command</see> may create several <see topic=\"ACL2____EVENTS\">events</see>.  That is, the <see topic=\"ACL2____COMMAND\">command</see> that introduces <v>fn1</v> may also \u000A introduce <v>fn2</v>.  Undoing the <see topic=\"ACL2____COMMAND\">command</see> that created either of these \u000A will undo them both.  The <see topic=\"ACL2____EVENTS\">events</see> created by a <see topic=\"ACL2____COMMAND\">command</see> constitute \u000A the <see topic=\"ACL2____COMMAND\">command</see>'s ``block'' and we can only undo entire blocks.  Use <tt><see topic=\"ACL2____PCB\">pcb</see></tt> to print the <see topic=\"ACL2____COMMAND\">command</see> block of a <see topic=\"ACL2____COMMAND\">command</see> if you wish to see \u000A what will be lost by undoing the <see topic=\"ACL2____COMMAND\">command</see>.</p> \u000A \u000A <p><v>Ubt</v> will not undo into ``prehistory''.  <v>:Ubt 1</v> will undo all of \u000A your <see topic=\"ACL2____COMMAND\">command</see>s.  But <v>:ubt -5</v> will cause an error, warning you that \u000A <v>:ubt</v> cannot undo system initialization.</p> \u000A \u000A <p>See <see topic=\"ACL2____U\">u</see> for how to undo just the latest command, and see <see topic=\"ACL2____UBU\">ubu</see> \u000A and see <see topic=\"ACL2____UBU_12\">ubu!</see> for how to undo back up to, but not including, the current \u000A command.</p>"],

 "ACL2____DEFABBREV":[["Macros","Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFABBREV\">defabbrev</see> snoc (x y) (<see topic=\"COMMON-LISP____APPEND\">append</see> y (<see topic=\"COMMON-LISP____LIST\">list</see> x)))\u000A(<see topic=\"ACL2____DEFABBREV\">defabbrev</see> sq (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 8) x)) (<see topic=\"COMMON-LISP_____A2\">*</see> x x))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFABBREV\">defabbrev</see> name (v1 ... vn) doc-string decl1 ... declk body)</code> \u000A \u000A <p>where <v>name</v> is a new function symbol, the <v>vi</v> are distinct variable \u000A symbols, and <v>body</v> is a term.  The <v>decli</v>, if supplied, should be \u000A legal <v>declare</v> forms; see <see topic=\"COMMON-LISP____DECLARE\">declare</see>.  <v>Doc-string</v> is an optional \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p>Roughly speaking, the <v>defabbrev</v> event is akin to defining <v>f</v> so \u000A that <v>(f v1 ... vn) = body</v>.  But rather than do this by adding a new \u000A axiom, <v>defabbrev</v> defines <v>f</v> to be a macro so that <v>(f a1 ... an)</v> \u000A expands to <v>body</v>, with the ``formals,'' <v>vi</v>, replaced by the \u000A ``actuals,'' <v>ai</v>.</p> \u000A \u000A <p>For example, if <v>snoc</v> is defined as shown in the first example above, \u000A then <v>(snoc (<see topic=\"COMMON-LISP_____B2\">+</see> i j) temp)</v> is just an abbreviation for</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____APPEND\">append</see> temp (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP_____B2\">+</see> i j))).</code> \u000A \u000A <p>In order to generate efficiently executable Lisp code, the macro that \u000A <v>defabbrev</v> introduces uses a <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> to bind the ``formals'' to the \u000A ``actuals.''  Consider the second example above.  Logically speaking, <v>(sq\u000A (ack i j))</v> is an abbreviation for <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (ack i j) (ack i j))</v>.  But in fact \u000A the macro for <v>sq</v> introduced by <v>defabbrev</v> actually arranges for \u000A <v>(sq (ack i j))</v> to expand to:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((x (ack i j)))\u000A  (<see topic=\"COMMON-LISP_____A2\">*</see> x x))</code> \u000A \u000A <p>which executes more efficiently than <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (ack i j) (ack i j))</v>.</p> \u000A \u000A <p>In the theorem prover, the <v>let</v> above expands to</p> \u000A \u000A <code>((<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP_____A2\">*</see> x x)) (ack i j))</code> \u000A \u000A <p>and thence to <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (ack i j) (ack i j))</v>.</p> \u000A \u000A <p>It is important to note that the term in <v>body</v> should not contain a \u000A call of <v>name</v> &mdash; i.e., <v>defabbrev</v> should not be used in place of \u000A <v>defun</v> when the function is recursive.  ACL2 will not complain when the \u000A <v>defabbrev</v> form is processed, but instead ACL2 will more than likely go \u000A into an infinite loop during macroexpansion of any form that has a call of \u000A <v>name</v>.</p> \u000A \u000A <p>It is also important to note that the parameters of any call of a macro \u000A defined by defabbrev will, as is the case for the parameters of a function \u000A call, be evaluated before the body is evaluated, since this is the evaluation \u000A order of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>.  This may lead to some errors or unexpected \u000A inefficiencies during evaluation if the body contains any conditionally \u000A evaluted forms like <v>cond</v>, <v>case</v>, or <v>if</v>.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFABBREV\">defabbrev</see> foo (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (test x) (bar y) nil))</code> \u000A \u000A <p>Notice a typical one-step expansion of a call of <v>foo</v> (see <see topic=\"ACL2____TRANS1\">trans1</see>):</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (foo expr1 expr2)\u000A (<see topic=\"COMMON-LISP____LET\">LET</see> ((X EXPR1) (Y EXPR2))\u000A      (<see topic=\"COMMON-LISP____IF\">IF</see> (TEST X) (BAR Y) NIL))\u000AACL2 !&gt;</code> \u000A \u000A <p>Now imagine that <v>expr2</v> is a complicated expression whose evaluation is \u000A intended only when the predicate <v>test</v> holds of <v>expr1</v>.  The expansion \u000A above suggests that <v>expr2</v> will always be evaluated by the call <v>(foo\u000A expr1 expr2)</v>, which may be inefficient (since perhaps we only need that \u000A value when <v>test</v> is true of <v>expr1</v>).  The evaluation of <v>expr2</v> may \u000A even cause an error, for example in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode if the \u000A expression <v>expr2</v> has been constructed in a manner that could cause a \u000A guard violation unless <v>test</v> holds of <v>expr1</v>.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q1-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>The correct answer to Question 1 in <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see> is <i>Choice (iv)</i>.</p> \u000A \u000A <p>The Induction Step of the inductive proof of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>for an induction on the linear list <v>z</v> is:</p> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp z)) \u000A              (implies (true-listp (cdr z)) \u000A                       (equal (rev (rev (cdr z))) (cdr z)))) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>The second hypothesis above is the the <i>induction hypothesis</i>.  The \u000A conclusion above is the formula we are trying to prove.  Each induction \u000A hypothesis is <i>always</i> an <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instance</see> of the formula \u000A being proved, i.e., it is obtained from the formula being proved by uniformly \u000A replacing the variables in the formula with terms.  Notice how the induction \u000A hypothesis above is the same as the induction conclusion, except that all the \u000A <v>z</v>s have been replaced by <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> z)</v>.</p> \u000A \u000A <p>If you thought the right answer was</p> \u000A \u000A <code><i>Induction Step -- Choice (i)</i>: \u000A(implies (not (endp z)) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>then perhaps you didn't understand that we're doing an inductive proof. \u000A Certainly if you prove the Base Case already discussed and you prove <i>Choice \u000A (i)</i> above, then you will have proved the goal conjecture, but you would \u000A have done it by simple case analysis: prove it when <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> z)</v> and prove it \u000A when <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z))</v>.  While logically valid, you probably can't prove \u000A <i>Choice (i)</i> directly because you have no induction hypothesis to work \u000A with.</p> \u000A \u000A <p>If you thought the right answer was:</p> \u000A \u000A <code><i>Induction Step -- Choice (ii)</i>: \u000A(implies (true-listp (cdr z)) \u000A         (equal (rev (rev (cdr z))) (cdr z))) \u000A</code> \u000A \u000A <p>then perhaps you misunderstand the difference between the <i>Induction \u000A Step</i> and the <i>Induction Hypothesis</i>.  The Induction <i>Step</i> is \u000A the ``other half'' of the main proof, balancing the Base Case.  The Induction \u000A <i>Hypothesis</i> is just a hypothesis you get to use during the Induction \u000A Step.  The question Q1 asked what is the Induction Step.</p> \u000A \u000A <p>If you thought the right answer was:</p> \u000A \u000A <code><i>Induction Step -- Choice (iii)</i>: \u000A(implies (and (not (endp z)) \u000A              (equal (rev (rev (cdr x))) (cdr x))) ; <i>``induction hyp''</i> \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>then you are making the most common mistake newcomers make to induction. \u000A You are giving yourself an ``induction hypothesis'' that is not an instance of \u000A the conjecture you're proving.  This alleged induction hypothesis says that \u000A <v>(rev (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</v> is <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v>, whereas the correct induction \u000A hypothesis says those two terms are equal <i>if</i> <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</v>. \u000A This alleged induction hypothesis is a stronger claim than we're trying to \u000A prove.  It turns out that by making this mistake you can ``prove'' conjectures \u000A that are not always true!  Remember: the induction hypothesis is always an \u000A instance of the conjecture you're proving, not just some piece of it.  Of \u000A course, ACL2 ``knows'' this and will never make this mistake.  But we remind \u000A you of it because there may be times when you intuit a different hypothesis \u000A and don't understand why ACL2 doesn't use it.</p> \u000A \u000A <p>If this doesn't make sense, perhaps you should read about <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF'>induction</see> again.</p> \u000A \u000A <p>When you understand why <i>Choice (iv)</i> is the correct answer, use your \u000A browser's <b>Back Button</b> to return to <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see> and go to question Q2.</p>"],

 "ACL2____GUARD-EXAMPLE":[["Tutorial5-miscellaneous-examples","Guard"],"ACL2 Sources","ACL2","<p>This note addresses the question: what is the use of <see topic=\"ACL2____GUARD\">guard</see>s \u000A in ACL2?  Although we recommend that beginners try to avoid <see topic=\"ACL2____GUARD\">guard</see>s for \u000A a while, we hope that the summary here is reasonably self-contained and will \u000A provide a reasonable introduction to guards in ACL2.  For a more systematic \u000A discussion, see <see topic=\"ACL2____GUARD\">guard</see>.  For a summary of that topic, see <see topic=\"ACL2____GUARD-QUICK-REFERENCE\">guard-quick-reference</see>.</p> \u000A \u000A <p>Before we get into the issue of <see topic=\"ACL2____GUARD\">guard</see>s, let us note that there are \u000A two important ``modes'':</p> \u000A \u000A <p><see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> &mdash; ``Does this <see topic=\"COMMON-LISP____DEFUN\">defun</see> add an axiom (`:logic \u000A mode') or not (`:program mode')?''  (See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>.)  Only \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions can have their ``<see topic=\"ACL2____GUARD\">guard</see>s verified'' \u000A via mechanized proof; see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> &mdash; ``Should runtime <see topic=\"ACL2____GUARD\">guard</see> violations \u000A signal an error (<v>:all</v>, and usually with <v>t</v> or <v>:nowarn</v>) or go \u000A undetected (<v>nil</v>, <v>:none</v>)?  Equivalently, are expressions evaluated in \u000A Common Lisp or in the logic?''  (See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.)</p> \u000A \u000A <p><i>Prompt examples</i></p> \u000A \u000A <p>Here some examples of the relation between the ACL2 <see topic=\"ACL2____PROMPT\">prompt</see> and the \u000A ``modes'' discussed above.  Also see <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>.  The first \u000A examples all have <v>ld-skip-proofsp nil</v>; that is, proofs are <i>not</i> \u000A skipped.</p> \u000A \u000A <code>ACL2 !&gt;    ; logic mode with guard checking on\u000AACL2 &gt;     ; logic mode with guard checking off\u000AACL2 p!&gt;   ; program mode with guard checking on\u000AACL2 p&gt;    ; program mode with guard checking off</code> \u000A \u000A <p>Here are some examples with <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt> of <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.</p> \u000A \u000A <code>ACL2 &gt;     ; guard checking off, ld-skip-proofsp nil\u000AACL2 s&gt;    ; guard checking off, ld-skip-proofsp t\u000AACL2 !&gt;    ; guard checking on, ld-skip-proofsp nil\u000AACL2 !s&gt;   ; guard checking on, ld-skip-proofsp t</code> \u000A \u000A <p><i>Sample session</i></p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP_____B2\">+</see> 'abc 3)\u000A\u000AACL2 Error in TOP-LEVEL: The guard for the function symbol\u000ABINARY-+, which is (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X) (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> Y)),\u000Ais violated by the arguments in the call (<see topic=\"COMMON-LISP_____B2\">+</see> 'ABC 3).\u000A\u000AACL2 !&gt;:set-guard-checking nil\u000A;;;; verbose output omitted here\u000AACL2 &gt;(<see topic=\"COMMON-LISP_____B2\">+</see> 'abc 3)\u000A3\u000AACL2 &gt;(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 'abc 3)\u000AT\u000AACL2 &gt;(<see topic=\"COMMON-LISP_____C3\">&lt;</see> 3 'abc)\u000ANIL\u000AACL2 &gt;(<see topic=\"COMMON-LISP_____C3\">&lt;</see> -3 'abc)\u000AT\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> sum-list (x)\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> x)\u000A                        :verify-guards nil))\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) 0)\u000A              (t (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (sum-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))\u000A\u000AThe admission of SUM-LIST is trivial, using the relation\u000AO&lt; (which is known to be well-founded on the domain\u000Arecognized by O-P) and the measure (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X).\u000AWe observe that the type of SUM-LIST is described by the\u000Atheorem (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> (SUM-LIST X)).  We used primitive type\u000Areasoning.\u000A\u000ASummary\u000AForm:  ( DEFUN SUM-LIST ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.03 seconds\u000A   (prove: 0.00, print: 0.00, proof tree: 0.00, other: 0.03)\u000A SUM-LIST\u000AACL2 !&gt;(sum-list '(1 2 3))\u000A\u000AACL2 Warning [Guards] in TOP-LEVEL:  Guard-checking will be inhibited\u000Aon recursive calls of the executable counterpart (i.e., in the ACL2\u000Alogic) of SUM-LIST.  To check guards on all recursive calls:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :all)\u000ATo leave behavior unchanged except for inhibiting this message:\u000A  (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :nowarn)\u000A\u000A6\u000AACL2 !&gt;(sum-list '(1 2 abc 3))\u000A\u000AACL2 Error in TOP-LEVEL: The guard for the function symbol\u000ABINARY-+, which is (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> X) (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> Y)),\u000Ais violated by the arguments in the call (<see topic=\"COMMON-LISP_____B2\">+</see> 'ABC 3).\u000A\u000AACL2 !&gt;:set-guard-checking nil\u000A;;;; verbose output omitted here\u000AACL2 &gt;(sum-list '(1 2 abc 3))\u000A6\u000AACL2 &gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> sum-list-append\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (sum-list (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A              (<see topic=\"COMMON-LISP_____B2\">+</see> (sum-list a) (sum-list b))))\u000A\u000A&lt;&lt; Starting proof tree logging &gt;&gt;\u000A\u000AName the formula above *1.\u000A\u000APerhaps we can prove *1 by induction.  Three induction\u000Aschemes are suggested by this conjecture.  Subsumption\u000Areduces that number to two.  However, one of these is flawed\u000Aand so we are left with one viable candidate.\u000A\u000A...\u000A\u000AThat completes the proof of *1.\u000A\u000AQ.E.D.</code> \u000A \u000A <p><i>Guard verification vs. defun</i></p> \u000A \u000A <code>      Declare Form                        Guards Verified?\u000A\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program ...))          no\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard g))                    yes\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard g :verify-guards nil)) no\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> ...&lt;no :guard&gt;...))           no\u000A\u000AACL2 &gt;:pe sum-list\u000A l        8  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> SUM-LIST (X)\u000A              (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X)\u000A                              :VERIFY-GUARDS NIL))\u000A              (<see topic=\"COMMON-LISP____COND\">COND</see> ((<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X) 0)\u000A                    (T (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (SUM-LIST (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))))\u000AACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> sum-list)\u000AThe non-trivial part of the guard conjecture for SUM-LIST,\u000Agiven the :type-prescription rule SUM-LIST, is\u000A\u000AGoal\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X) (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X)))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X NIL))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X) (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X)))\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X) (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> X)))\u000A              (<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)))).\u000A\u000A...\u000A\u000AACL2 &gt;:pe sum-list\u000A lv       8  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> SUM-LIST (X)\u000A              (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X)\u000A                              :VERIFY-GUARDS NIL))\u000AACL2 &gt;:set-guard-checking t\u000A\u000ATurning guard checking on, value T.\u000A\u000AACL2 !&gt;(sum-list '(1 2 abc 3))\u000A\u000AACL2 Error in TOP-LEVEL: The guard for the function symbol\u000ASUM-LIST, which is (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> X), is violated by the\u000Aarguments in the call (SUM-LIST '(1 2 ABC ...)).  See :DOC trace for a useful\u000Adebugging utility.  See :DOC set-guard-checking for information about\u000Asuppressing this check with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none), as recommended for\u000Anew users.\u000A\u000AACL2 !&gt;:set-guard-checking nil\u000A;;;; verbose output omitted here\u000AACL2 &gt;(sum-list '(1 2 abc 3))\u000A6\u000AACL2 &gt;:comp sum-list\u000ACompiling gazonk0.lsp.\u000AEnd of Pass 1.\u000AEnd of Pass 2.\u000AFinished compiling gazonk0.lsp.\u000ALoading gazonk0.o\u000Astart address -T 1bbf0b4 Finished loading gazonk0.o\u000ACompiling gazonk0.lsp.\u000AEnd of Pass 1.\u000AEnd of Pass 2.\u000AFinished compiling gazonk0.lsp.\u000ALoading gazonk0.o\u000Astart address -T 1bc4408 Finished loading gazonk0.o\u000A SUM-LIST\u000AACL2 &gt;:q\u000A\u000AExiting the ACL2 read-eval-print loop.\u000AACL2&gt;(<see topic=\"COMMON-LISP____TRACE\">trace</see> sum-list)\u000A(SUM-LIST)\u000A\u000AACL2&gt;(<see topic=\"ACL2____LP\">lp</see>)\u000A\u000AACL2 Version 1.8.  Level 1.  Cbd &quot;/slocal/src/acl2/v1-9/&quot;.\u000AType :help for help.\u000AACL2 &gt;(sum-list '(1 2 abc 3))\u000A6\u000AACL2 &gt;(sum-list '(1 2 3))\u000A  1&gt; (SUM-LIST (1 2 3))&gt;\u000A    2&gt; (SUM-LIST (2 3))&gt;\u000A      3&gt; (SUM-LIST (3))&gt;\u000A        4&gt; (SUM-LIST NIL)&gt;\u000A        &lt;4 (SUM-LIST 0)&gt;\u000A      &lt;3 (SUM-LIST 3)&gt;\u000A    &lt;2 (SUM-LIST 5)&gt;\u000A  &lt;1 (SUM-LIST 6)&gt;\u000A6\u000AACL2 &gt;:pe sum-list-append\u000A          9  (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> SUM-LIST-APPEND\u000A                     (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (SUM-LIST (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> A B))\u000A                            (<see topic=\"COMMON-LISP_____B2\">+</see> (SUM-LIST A) (SUM-LIST B))))\u000AACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> sum-list-append)\u000A\u000AThe non-trivial part of the guard conjecture for\u000ASUM-LIST-APPEND, given the :type-prescription rule SUM-LIST,\u000Ais\u000A\u000AGoal\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> A)\u000A     (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> A B))\u000A     (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> A)\u000A     (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> B)).\u000A\u000A...\u000A\u000A****** FAILED ******* See :DOC failure ****** FAILED ******\u000AACL2 &gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> common-lisp-sum-list-append\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> a)\u000A                  (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> b))\u000A             (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (sum-list (<see topic=\"COMMON-LISP____APPEND\">append</see> a b))\u000A                    (<see topic=\"COMMON-LISP_____B2\">+</see> (sum-list a) (sum-list b)))\u000A             t)\u000A         :rule-classes nil)\u000A\u000A&lt;&lt; Starting proof tree logging &gt;&gt;\u000A\u000ABy the simple :rewrite rule SUM-LIST-APPEND we reduce the\u000Aconjecture to\u000A\u000AGoal'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> A)\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> B))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (SUM-LIST A) (SUM-LIST B))\u000A                (<see topic=\"COMMON-LISP_____B2\">+</see> (SUM-LIST A) (SUM-LIST B)))).\u000A\u000ABut we reduce the conjecture to T, by primitive type\u000Areasoning.\u000A\u000AQ.E.D.\u000A;;;; summary omitted here\u000AACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> common-lisp-sum-list-append)\u000A\u000AThe non-trivial part of the guard conjecture for\u000ACOMMON-LISP-SUM-LIST-APPEND, given the :type-prescription\u000Arule SUM-LIST, is\u000A\u000AGoal\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> A)\u000A                   (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> B))\u000A              (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> A))\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> A)\u000A                   (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> B))\u000A              (<see topic=\"ACL2____INTEGER-LISTP\">INTEGER-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> A B)))).\u000A\u000A...\u000A\u000AQ.E.D.\u000A\u000AThat completes the proof of the guard theorem for\u000ACOMMON-LISP-SUM-LIST-APPEND.  COMMON-LISP-SUM-LIST-APPEND\u000Ais compliant with Common Lisp.\u000A;;;; Summary omitted here.\u000AACL2 &gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> foo (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"ACL2____MV\">mv</see> x y)))\u000A\u000A...\u000A\u000AQ.E.D.</code> \u000A \u000A <code>ACL2 &gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)\u000A\u000AACL2 Error in (<see topic=\"ACL2____VERIFY-GUARDS\">VERIFY-GUARDS</see> FOO): The number of values we\u000Aneed to return is 1 but the number of values returned by the\u000Acall (<see topic=\"ACL2____MV\">MV</see> X Y) is 2.\u000A\u000A&gt; (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"ACL2____MV\">MV</see> X Y))\u000A\u000AACL2 Error in (<see topic=\"ACL2____VERIFY-GUARDS\">VERIFY-GUARDS</see> FOO): The guards for FOO cannot\u000Abe verified because the theorem has the wrong syntactic\u000Aform.  See :DOC verify-guards.</code>"],

 "ACL2____GRANULARITY":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>Some function calls are on arguments whose evaluation time is long enough \u000A to warrant parallel execution, while others are not.  A granularity form can \u000A be used to make appropriate restrictions on the use of parallelism.</p> \u000A \u000A <p>For example, consider the Fibonacci function.  Experiments have suggested \u000A that execution time can be reduced if whenever the argument is less than 27, a \u000A serial version of the Fibonacci function is called.  One way to utilize this \u000A information is to write two definitions of the Fibonacci function, one serial \u000A and one parallel.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                     (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 27) (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (fib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                            (fib (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A        (t (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                            (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))))</code> \u000A \u000A <p>We realize quickly that writing both of these function definitions is \u000A cumbersome and redundant.  This problem can be avoided by using a \u000A <v>granularity</v> declaration with a parallelism primitive.  This form ensures \u000A that a call is parallelized only if resources are available and the \u000A granularity form evaluates to a non-<v>nil</v> value at the time of the call. \u000A Below is a definition of the Fibonacci function using a granularity form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)) 0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 27)))\u000A                  (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                            (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))))</code> \u000A \u000A <p>A granularity form can reference an extra formal parameter that describes \u000A the call depth of the function the user is parallelizing.  Consider for \u000A example the following parallel <v>mergesort</v> function, based on Davis's \u000A Ordered Sets library.  It splits the data into symmetric chunks for \u000A computation, so we increment the <v>depth</v> argument during the recursive call \u000A on both the <v>car</v> and <v>cdr</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;finite-set-theory/osets/sets&quot; :dir :system)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> set::pmergesort-exec (x depth)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A        ((<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (set::insert (<see topic=\"COMMON-LISP____CAR\">car</see> x) nil))\u000A        (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (part1 part2)\u000A                   (set::split-list x nil nil)\u000A                   (<see topic=\"ACL2____PARGS\">pargs</see>\u000A                    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> depth 2)))\u000A                    (set::union (set::pmergesort-exec part1\u000A                                                        (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))\u000A                                 (set::pmergesort-exec part2\u000A                                                        (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))))))))</code> \u000A \u000A <p>A less intrusive method (i.e., not requiring an extra formal parameter like \u000A the <v>depth</v> argument just above), which however can be less efficient, \u000A involves analyzing the data itself for structural properties.  For \u000A example:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> some-depth-exceeds (x n)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> n)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A        t\u000A      (<see topic=\"COMMON-LISP____OR\">or</see> (some-depth-exceeds (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____1-\">1-</see> n))\u000A          (some-depth-exceeds (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____1-\">1-</see> n))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tip (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'A)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'T)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'C)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'G)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pvalid-tree (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      (valid-tip x)\u000A    (<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (some-depth-exceeds x 3)))\u000A          (pvalid-tree (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A          (pvalid-tree (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code> \u000A \u000A <p>If you experiment with calls of <v>pvalid-tree</v>, you are likely to find \u000A that the ``speedup'' it provides over a corresponding serial version is, in \u000A fact, a slowdown!  The problem is likely that <v>some-depth-exceeds</v> is an \u000A expensive function to run repeatedly.  Instead of the approach above, it is \u000A often handy to add an extra formal parameter in order to allow for more \u000A efficient granularity forms, as we have done above in the definition of \u000A <v>SET::pmergesort-exec</v>.</p>"],

 "ACL2____WHY-BRR":[["Break-rewrite"],"ACL2 Sources","ACL2","<p>Why isn't <tt><see topic=\"ACL2____BRR\">brr</see></tt> mode automatically disabled when there are no \u000A monitored runes?  The reason is that the list of monitored runes is kept in a \u000A wormhole state.</p> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see> for more information on wormholes in general.  But the \u000A fundamental property of the wormhole function is that it is a logical \u000A <v>no-op</v>, a constant function that does not take state as an argument.  When \u000A entering a wormhole, arbitrary information can be passed in (including the \u000A external state).  That information is used to construct a near copy of the \u000A external state and that ``wormhole state'' is the one with respect to which \u000A interactions occur during breaks.  But no information is carried by ACL2 out \u000A of a wormhole &mdash; if that were allowed wormholes would not be logical \u000A no-ops.  The only information carried out of a wormhole is in the user's \u000A head.</p> \u000A \u000A <p><tt><see topic=\"ACL2____BREAK-REWRITE\">Break-rewrite</see></tt> interacts with the user in a wormhole state because \u000A the signature of the ACL2 rewrite function does not permit it to modify <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Hence, only wormhole interaction is possible.  (This has the \u000A additional desirable property that the correctness of the rewriter does not \u000A depend on what the user does during interactive breaks within it; indeed, it \u000A is logically impossible for the user to affect the course of <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>.)</p> \u000A \u000A <p>Now consider the list of monitored runes.  Is that kept in the external \u000A state as a normal state global or is it kept in the wormhole state?  If it is \u000A in the external state then it can be inspected within the wormhole but not \u000A changed.  This is unacceptable; it is common to change the <see topic=\"ACL2____MONITOR\">monitor</see>ed \u000A rules as the proof attempt progresses, installing monitors when certain rules \u000A are about to be used in certain contexts.  Thus, the list of monitored runes \u000A must be kept as a wormhole variable.  Hence, its value cannot be determined \u000A outside the wormhole, where the proof attempt is ongoing.</p> \u000A \u000A <p>This raises another question: If the list of monitored runes is unknown to \u000A the rewriter operating on the external state, how does the rewriter know when \u000A to break?  The answer is simple: it breaks every time, for every rune, if \u000A <tt><see topic=\"ACL2____BRR\">brr</see></tt> mode is enabled.  The wormhole is entered (silently), computations \u000A are done within the wormhole state to determine if the user wants to see the \u000A break, and if so, interactions begin.  For unmonitored runes and runes with \u000A false break conditions, the silent wormhole entry is followed by a silent \u000A wormhole exit and the user perceives no break.</p> \u000A \u000A <p>Thus, the penalty for running with <tt><see topic=\"ACL2____BRR\">brr</see></tt> mode enabled when there are \u000A no monitored runes is high: a wormhole is entered on every application of \u000A every rune and the user is simply unware of it.  The user who has finally \u000A unmonitored all runes is therefore strongly advised to carry this information \u000A out of the wormhole and to do <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt> <v>nil</v> in the external state \u000A when the next opportunity arises.</p>"],

 "ACL2____TRACE_12":[["Trace"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TRACE_12\">trace!</see> (fact :native t :entry *foo*))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____TRACE_12\">trace!</see> spec1 ... specn)</code> \u000A \u000A <p>where the <v>speci</v> are suitable arguments to <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____TRACE_42\">trace$</see> for a way to trace function calls.  Some calls of \u000A <v>trace$</v> are potentially dangerous and thus require a trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  But it can be a nuisance to call <v>defttag</v> explicitly, so the \u000A <v>trace!</v> macro is provided in order to avoid the need to do that: \u000A <v>trace!</v> automatically defines a (temporary) trust tag.</p> \u000A \u000A <p>See <see topic=\"ACL2____UNTRACE_42\">untrace$</see> for how to undo the effect of <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt>.</p> \u000A \u000A <p>The evaluation of a <v>trace!</v> form causes temporary creation of an active \u000A trust tag, <v>:trace!</v>, followed by the corresponding <v>trace$</v> form.  The \u000A trust tag will disappear when the call to <v>trace!</v> completes.  Even though \u000A <v>trace!</v> will remove its temporary ttag, it will still print a ``<v>TTAG\u000A NOTE</v>'', which indicates that the session is suspect.  See <see topic=\"ACL2____DEFTTAG\">defttag</see> and \u000A see <see topic=\"ACL2____TTAGS-SEEN\">ttags-seen</see> for further remarks on this issue.</p> \u000A \u000A <p>Because of the active trust tag, it is possible to do things with \u000A <v>trace!</v>  that are useful but without logical justification.  Below is an \u000A example of how to use <v>trace!</v> to cause a function call to change <see topic=\"ACL2____STATE\">state</see>, even though the function does not take <tt><see topic=\"ACL2____STATE\">state</see></tt> as a \u000A parameter.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (n)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> n) :verify-guards nil))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A             1\u000A           (<see topic=\"COMMON-LISP_____A2\">*</see> n (fact (<see topic=\"COMMON-LISP____1-\">1-</see> n)))))\u000A\u000AThe admission of FACT is trivial, using the relation O&lt; (which is known\u000Ato be well-founded on the domain recognized by O-P) and the measure\u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> N).  We observe that the type of FACT is described by the\u000Atheorem (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> (FACT N)) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 (FACT N))).  We used the :compound-\u000Arecognizer rule ZP-COMPOUND-RECOGNIZER and primitive type reasoning.\u000A\u000ASummary\u000AForm:  ( DEFUN FACT ...)\u000ARules: ((:COMPOUND-RECOGNIZER ZP-COMPOUND-RECOGNIZER)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-foo (n value state)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state :verify-guards nil))\u000A         (<see topic=\"ACL2____ASSIGN\">assign</see> foo (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> n value) (<see topic=\"ACL2_____04\">@</see> foo))))\u000A\u000ASince UPDATE-FOO is non-recursive, its admission is trivial.  We observe\u000Athat the type of UPDATE-FOO is described by the theorem\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (UPDATE-FOO N VALUE STATE))\u000A     (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (UPDATE-FOO N VALUE STATE))).\u000AWe used primitive type reasoning.\u000A\u000A(UPDATE-FOO * * STATE) =&gt; (<see topic=\"ACL2____MV\">MV</see> * * STATE).\u000A\u000ASummary\u000AForm:  ( DEFUN UPDATE-FOO ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A UPDATE-FOO\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_12\">trace!</see> (fact :exit (update-foo n value state)))\u000A\u000ATTAG NOTE: Adding ttag :TRACE! from the top level loop.\u000A ((FACT :EXIT (UPDATE-FOO N VALUE STATE)))\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> foo nil)\u000A NIL\u000AACL2 !&gt;(fact 7)\u000A1&gt; (ACL2_*1*_ACL2::FACT 7)\u000A  2&gt; (ACL2_*1*_ACL2::FACT 6)\u000A    3&gt; (ACL2_*1*_ACL2::FACT 5)\u000A      4&gt; (ACL2_*1*_ACL2::FACT 4)\u000A        5&gt; (ACL2_*1*_ACL2::FACT 3)\u000A          6&gt; (ACL2_*1*_ACL2::FACT 2)\u000A            7&gt; (ACL2_*1*_ACL2::FACT 1)\u000A              8&gt; (ACL2_*1*_ACL2::FACT 0)\u000A              &lt;8 NIL\u000A            &lt;7 NIL\u000A          &lt;6 NIL\u000A        &lt;5 NIL\u000A      &lt;4 NIL\u000A    &lt;3 NIL\u000A  &lt;2 NIL\u000A&lt;1 NIL\u000A5040\u000AACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> foo)\u000A((7 . 5040)\u000A (6 . 720)\u000A (5 . 120)\u000A (4 . 24)\u000A (3 . 6)\u000A (2 . 2)\u000A (1 . 1)\u000A (0 . 1))\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)\u000A\u000AComputing the guard conjecture for FACT....\u000A\u000AThe guard conjecture for FACT is trivial to prove, given the :compound-\u000Arecognizer rules NATP-COMPOUND-RECOGNIZER and ZP-COMPOUND-RECOGNIZER,\u000Aprimitive type reasoning and the :type-prescription rule FACT.  FACT\u000Ais compliant with Common Lisp.\u000A\u000ASummary\u000AForm:  ( VERIFY-GUARDS FACT)\u000ARules: ((:COMPOUND-RECOGNIZER NATP-COMPOUND-RECOGNIZER)\u000A        (:COMPOUND-RECOGNIZER ZP-COMPOUND-RECOGNIZER)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)\u000A        (:TYPE-PRESCRIPTION FACT))\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FACT\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> foo nil)\u000A NIL\u000AACL2 !&gt;(fact 7)\u000A1&gt; (ACL2_*1*_ACL2::FACT 7)\u000A  2&gt; (FACT 7)\u000A    3&gt; (FACT 6)\u000A      4&gt; (FACT 5)\u000A        5&gt; (FACT 4)\u000A          6&gt; (FACT 3)\u000A            7&gt; (FACT 2)\u000A              8&gt; (FACT 1)\u000A                9&gt; (FACT 0)\u000A                &lt;9 NIL\u000A              &lt;8 NIL\u000A            &lt;7 NIL\u000A          &lt;6 NIL\u000A        &lt;5 NIL\u000A      &lt;4 NIL\u000A    &lt;3 NIL\u000A  &lt;2 NIL\u000A&lt;1 NIL\u000A5040\u000AACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> foo)\u000A((7 . 5040)\u000A (7 . 5040)\u000A (6 . 720)\u000A (5 . 120)\u000A (4 . 24)\u000A (3 . 6)\u000A (2 . 2)\u000A (1 . 1)\u000A (0 . 1))\u000AACL2 !&gt;(<see topic=\"ACL2____TRACE_12\">trace!</see> (fact :exit (<see topic=\"COMMON-LISP____PROGN\">progn</see> (update-foo n value state)\u000A                                  (<see topic=\"COMMON-LISP____CONS\">cons</see> traced-fn values))))\u000A\u000ATTAG NOTE: Adding ttag :TRACE! from the top level loop.\u000A ((FACT :EXIT (<see topic=\"COMMON-LISP____PROGN\">PROGN</see> (UPDATE-FOO N VALUE STATE)\u000A                     (<see topic=\"COMMON-LISP____CONS\">CONS</see> TRACED-FN VALUES))))\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> foo nil)\u000A NIL\u000AACL2 !&gt;(fact 7)\u000A1&gt; (ACL2_*1*_ACL2::FACT 7)\u000A  2&gt; (FACT 7)\u000A    3&gt; (FACT 6)\u000A      4&gt; (FACT 5)\u000A        5&gt; (FACT 4)\u000A          6&gt; (FACT 3)\u000A            7&gt; (FACT 2)\u000A              8&gt; (FACT 1)\u000A                9&gt; (FACT 0)\u000A                &lt;9 (FACT 1)\u000A              &lt;8 (FACT 1)\u000A            &lt;7 (FACT 2)\u000A          &lt;6 (FACT 6)\u000A        &lt;5 (FACT 24)\u000A      &lt;4 (FACT 120)\u000A    &lt;3 (FACT 720)\u000A  &lt;2 (FACT 5040)\u000A&lt;1 (ACL2_*1*_ACL2::FACT 5040)\u000A5040\u000AACL2 !&gt;(<see topic=\"ACL2_____04\">@</see> foo)\u000A((7 . 5040)\u000A (7 . 5040)\u000A (6 . 720)\u000A (5 . 120)\u000A (4 . 24)\u000A (3 . 6)\u000A (2 . 2)\u000A (1 . 1)\u000A (0 . 1))\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally, we remark that using <v>trace!</v> can cause errors in situations \u000A where tracing is automatically suspended and re-introduced.  This is likely to \u000A be a rare occurrence, but consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____TRACE_12\">trace!</see> (<see topic=\"ACL2____LEXORDER\">lexorder</see> :native t :multiplicity 1))\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;foo&quot; 0 t)</code> \u000A \u000A <p>If the certify-book causes compilation, you may see an error such as the \u000A following.</p> \u000A \u000A <code>ACL2 Error in (<see topic=\"ACL2____CERTIFY-BOOK\">CERTIFY-BOOK</see> &quot;foo&quot; ...):  The keyword :NATIVE cannot\u000Abe used in a trace spec unless there is an active trust tag.  The trace\u000Aspec (<see topic=\"ACL2____LEXORDER\">LEXORDER</see> :NATIVE T :MULTIPLICITY 1) is thus illegal.  Consider\u000Ausing trace! instead.  The complete list of keywords that require a\u000Atrust tag for use in a trace spec is: (:NATIVE :DEF :MULTIPLICITY).</code> \u000A \u000A <p>This error is harmless.  The function will appear, when calling \u000A <v>(<see topic=\"ACL2____TRACE_42\">trace$</see>)</v>, to remain traced, but in fact there will be no tracing \u000A behavior, so you may want to call <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt> on the function symbol in \u000A question.</p>"],

 "ACL2____SET-ENFORCE-REDUNDANCY":[["Redundant-events"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see> nil)   ; do not require redundancy (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(<see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see> t)     ; most events (see below) must be redundant\u000A(<see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see> :warn) ; warn for most non-redundant events</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see> flag)</code> \u000A \u000A <p>where <v>flag</v> is <v>nil</v>, <v>t</v>, or <v>:warn</v>, as indicated above. \u000A This macro is essentially equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :enforce-redundancy flag)</code> \u000A \u000A <p>and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  However, \u000A unlike the above simple call of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> event function (see <see topic=\"ACL2____TABLE\">table</see>), no output results from a <v>set-enforce-redundancy</v> event.</p> \u000A \u000A <p><v>Set-enforce-redundancy</v> may be thought of as an event that merely sets \u000A a flag as indicated above, which determines whether most <see topic=\"ACL2____EVENTS\">events</see>, \u000A including <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events, are allowed to be \u000A redundant; see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.  The exceptions are <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>, \u000A <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <tt><see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see></tt>, <tt><see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see></tt>, <tt><see topic=\"ACL2____SET-BODY\">set-body</see></tt>, and <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.  Any other type of non-redundant event will cause an \u000A error if <v>flag</v> is <v>t</v> and a warning if <v>flag</v> is <v>nil</v>, \u000A <i>except</i> in the course of carrying out an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form.</p> \u000A \u000A <p>Note that because <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see> that set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> are implicitly <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, <v>set-enforce-redundancy</v> \u000A events are ignored when including books.  However, the presence of the event \u000A <v>(<see topic=\"ACL2____SET-ENFORCE-REDUNDANCY\">set-enforce-redundancy</see> t)</v> in a book guarantees that its subsequent \u000A definitions and theorems are redundant.  This can be a useful property to \u000A maintain in library development, as we now describe.</p> \u000A \u000A <p>An example of the use of this form can be found in the community <see topic=\"ACL2____BOOKS\">books</see> under directory <v>books/rtl/rel4/</v>.  The intention in that directory \u000A has been to put all the gory details in subdirectories <v>support/</v> and \u000A <v>arithmetic/</v>, so that the books in subdirectory <v>lib/</v> contain only the \u000A ``exported'' definitions and theorems.  This approach is useful for human \u000A readability.  Moreover, suppose we want to prove new theorems in <v>lib/</v>. \u000A Typically we wish to prove the new theorems using the existing books in \u000A <v>lib/</v>; however, our methodology demands that the proofs go into books in \u000A <v>support/</v>.  If every theorem in <v>lib/</v> is redundant, then we can \u000A <i>develop</i> the proofs in <v>lib/</v> but then when we are done, <i>move</i> \u000A each book with such proofs into <v>support/</v> as follows.  In any such book, \u000A we first replace <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms referring to books in <v>lib/</v> by \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> forms referring to corresponding books in <v>support/</v> \u000A and/or <v>arithmetic/</v>.  Then, we add suitable <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> events to \u000A get us back into the original <v>lib/</v> proof environment.</p> \u000A \u000A <p>The default behavior of the system is as though the \u000A <v>:enforce-redundancy</v> value is <v>nil</v>.  The current behavior can be \u000A ascertained by evaluating the following form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> :enforce-redundancy (table-alist 'acl2-defaults-table wrld)))</code>"],

 "ACL2____PRINT-GV":[["Guard","Debugging"],"ACL2 Sources","ACL2","<p>By default, ACL2 checks input constraints on functions, known as \u000A <see topic=\"ACL2____GUARD\">guard</see>s.  When guards are violated, an informative message is printed; \u000A but sometimes it is useful to investigate why the guard check fails.  The \u000A utility <v>print-gv</v> is provided for that purpose.  (Alternatively, you may \u000A prefer to avoid guards entirely with <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none)</v>; see \u000A <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.)</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____PRINT-GV\">print-gv</see>)\u000A(<see topic=\"ACL2____PRINT-GV\">print-gv</see> :evisc-tuple (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 4 4 nil nil))\u000A(<see topic=\"ACL2____PRINT-GV\">print-gv</see> :evisc-tuple (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 4 ; print-level\u000A                                    5 ; print-length\u000A                                    (world-evisceration-alist state nil)\u000A                                    nil ; hiding-cars\u000A                                    ))</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____PRINT-GV\">print-gv</see> :evisc-tuple x)</code> \u000A \u000A <p>where the <v>:evisc-tuple</v> argument is optional and defaults to one that \u000A hides only the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>, by printing <v>&lt;world&gt;</v> instead of \u000A a very large structure.  See <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> for a discussion of \u000A evisc-tuples.</p> \u000A \u000A <p>To see how one might use <v>print-gv</v>, consider the following \u000A definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (my-first-predicate x)\u000A                              (my-second-predicate x))))\u000A  (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</code> \u000A \u000A <p>Now suppose we evaluate a call of <v>foo</v> and obtain a guard \u000A violation.</p> \u000A \u000A <code>ACL2 !&gt;(foo 3)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function call (FOO X),\u000Awhich is (<see topic=\"COMMON-LISP____AND\">AND</see> (MY-FIRST-PREDICATE X) (MY-SECOND-PREDICATE X)), is violated\u000Aby the arguments in the call (FOO 3).  To debug: See :DOC print-gv,\u000Asee :DOC trace, and see :DOC wet.  See :DOC set-guard-checking for\u000Ainformation about suppressing this check with (<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> :none),\u000Aas recommended for new users.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>We can obtain (in essence) the guard form that was false, as follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____PRINT-GV\">print-gv</see>)\u000A\u000A(<see topic=\"COMMON-LISP____LET\">LET</see> ((X '3))\u000A     (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> X))\u000A     (<see topic=\"COMMON-LISP____AND\">AND</see> (MY-FIRST-PREDICATE X)\u000A          (MY-SECOND-PREDICATE X)))\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>But which conjunct is to blame?  With a bit of editing we can try each \u000A conjunct in turn.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">LET</see> ((X '3))\u000A            (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> X))\u000A            (MY-FIRST-PREDICATE X))\u000AT\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">LET</see> ((X '3))\u000A            (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> X))\u000A            (MY-SECOND-PREDICATE X))\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p>Aha!  Now we can investigate why the second predicate fails for 3.</p> \u000A \u000A <p>The following hack will give you access in raw Lisp to the form printed by \u000A <v>(<see topic=\"ACL2____PRINT-GV\">print-gv</see>)</v>.  After a guard violation, just submit this form to raw \u000A Lisp:</p> \u000A \u000A <code>(print-gv1 (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> 'ev-fncall-guard-er-wormhole\u000A                                      *wormhole-status-alist*)))\u000A           state)</code> \u000A \u000A <p>If you use local <see topic=\"ACL2____STOBJ\">stobj</see>s (see <see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>) or stobj \u000A fields of stobjs, you may need to edit the output of <v>print-gv</v> in order to \u000A evaluate it.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x st)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) :stobjs st)\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A  (fld st))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> test ()\u000A  (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A   st\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A           (<see topic=\"ACL2____MV\">mv</see> (g 3 st) st)\u000A           result)))\u000A\u000A(test)</code> \u000A \u000A <p>Then <v>:print-gv</v> yields the result shown below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____FLET\">FLET</see> ((G{GUARD} (X ST)\u000A                 (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> X ST))\u000A                 (<see topic=\"COMMON-LISP____AND\">AND</see> (STP ST) (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))))\u000A      (G{GUARD} '3 |&lt;some-stobj&gt;|))</code> \u000A \u000A <p>In this example you could replace ``<v>|&lt;some-stobj&gt;|</v>'' by ``<v>st</v>'' to \u000A obtain a result of <v>nil</v>.  But similar cases may require the use of a local \u000A stobj that is no longer available, in which case you may need to be creative \u000A in order to take advantage of <v>:print-gv</v>.  Here is such an example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st2 fld2)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g2 (st2)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____NULL\">null</see> (fld2 st2)) :stobjs st2))\u000A  (<see topic=\"ACL2____MV\">mv</see> 0 st2))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> test2 ()\u000A  (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A   st2\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st2)\u000A           (<see topic=\"COMMON-LISP____LET\">let</see> ((st2 (update-fld2 17 st2)))\u000A             (g2 st2))\u000A           result)))\u000A\u000A(test2)</code> \u000A \u000A <p>In this case, <v>:print-gv</v> yields the following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____FLET\">FLET</see> ((G2{GUARD} (ST2)\u000A                  (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> ST2))\u000A                  (<see topic=\"COMMON-LISP____AND\">AND</see> (ST2P ST2) (<see topic=\"COMMON-LISP____NULL\">NULL</see> (FLD2 ST2)))))\u000A      (G2{GUARD} |&lt;some-stobj&gt;|))</code> \u000A \u000A <p>But if you replace ``<v>|&lt;some-stobj&gt;|</v>'' by ``<v>st</v>'', the guard holds; \u000A it is only the local stobj, which is no longer available, that produced a \u000A guard violation (because its field had been updated to a cons).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____FLET\">FLET</see> ((G2{GUARD} (ST2)\u000A                         (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"COMMON-LISP____IGNORABLE\">IGNORABLE</see> ST2))\u000A                         (<see topic=\"COMMON-LISP____AND\">AND</see> (ST2P ST2) (<see topic=\"COMMON-LISP____NULL\">NULL</see> (FLD2 ST2)))))\u000A             (G2{GUARD} st2))\u000AT\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally, we note that while <v>print-gv</v> is a utility for debugging guard \u000A violations, in contrast, see <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see> for a utility to assist in \u000A debugging failed proofs arising from guard verification.</p>"],

 "ACL2____LD-QUERY-CONTROL-ALIST":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-query-control-alist</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The accessor is <v>(<see topic=\"ACL2____LD-QUERY-CONTROL-ALIST\">ld-query-control-alist</see> state)</v> and the updater is \u000A <v>(set-ld-query-control-alist val state)</v>.  Roughly speaking, \u000A <v>ld-query-control-alist</v> is either <v>nil</v> (meaning all queries should be \u000A interactive), <v>t</v> (meaning all should default to the first accepted \u000A response), or an alist that pairs query ids to keyword responses.  The alist \u000A may end in either <v>t</v> or <v>nil</v>, indicating the default value for all ids \u000A not listed explicitly.  Formally, the <v>ld-query-control-alist</v> must satisfy \u000A <v>ld-query-control-alistp</v>.  The initial <v>ld-query-control-alist</v> is \u000A <v>nil</v>, which means all queries are handled interactively.</p> \u000A \u000A <p>When an ACL2 query is raised, a unique identifying symbol is printed in \u000A parentheses after the word ``Query''.  This symbol, called the ``query id,'' \u000A can be used in conjunction with <v>ld-query-control-alist</v> to prevent the \u000A query from being handled interactively.  By ``handled interactively'' we mean \u000A that the query is printed to <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt> and a response is read from \u000A <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt>.  The alist can be used to obtain a ``default value'' \u000A for each query id.  If this value is <v>nil</v>, then the query is handled \u000A interactively.  In all other cases, the system handles the query without \u000A interaction (although text may be printed to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> to make it \u000A appear that an interaction has occurred; see below).  If the default value is \u000A <v>t</v>, the system acts as though the user responded to the query by typing \u000A the first response listed among the acceptable responses.  If the default \u000A value is neither <v>nil</v> nor <v>t</v>, then it must be a keyword and one of the \u000A acceptable responses.  In that case, the system acts as though the user \u000A responded with the given keyword.</p> \u000A \u000A <p>Next, we discuss how the <v>ld-query-control-alist</v> assigns a default \u000A value to each query id.  It assigns each id the first value paired with the id \u000A in the alist, or, if no such pair appears in the alist, it assigns the final \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of the alist as the value.  Thus, <v>nil</v> assigns all ids \u000A <v>nil</v>.  <v>T</v> assigns all ids <v>t</v>.  <v>'((:filter . nil) (:sysdef . :n)\u000A . t)</v> assigns <v>nil</v> to the <v>:filter</v> query, <v>:n</v> to the <v>:sysdef</v> \u000A query, and <v>t</v> to all others.</p> \u000A \u000A <p>It remains only to discuss how the system prints text when the default \u000A value is not <v>nil</v>, i.e., when the query is handled without interaction. \u000A In fact, it is allowed to pair a query id with a singleton list containing a \u000A keyword, rather than a keyword, and this indicates that no printing is to be \u000A done.  Thus for the example above, <v>:sysdef</v> queries would be handled \u000A noninteractively, with printing done to simulate the interaction.  But if we \u000A change the example so that <v>:sysdef</v> is paired with <v>(:n)</v>, i.e., if \u000A <v>ld-query-control-alist</v> is <v>'((:filter . nil) (:sysdef :n) . t)</v>, then \u000A no such printing would take place for <v>sysdef</v> queries.  Instead, the \u000A default value of <v>:n</v> would be assigned ``quietly''.</p>"],

 "ACL2____IRRELEVANT-FORMALS":[["Programming"],"ACL2 Sources","ACL2","<p>Let <v>fn</v> be a function of <v>n</v> arguments.  Let <v>x</v> be the \u000A <v>i</v>th formal of <v>fn</v>.  We say <v>x</v> is ``irrelevant in <v>fn</v>'' if \u000A <v>x</v> does not occur in either the <see topic=\"ACL2____GUARD\">guard</see> or the measure for <v>fn</v>, \u000A and the value of <v>(fn a1...ai...an)</v> is independent of <v>ai</v>.</p> \u000A \u000A <p>The easiest way to define a function with an irrelevant formal is simply \u000A not to use the formal in the body of the function.  Such formals are said to \u000A be ``ignored'' by Common Lisp and a special declaration is provided to allow \u000A ignored formals.  ACL2 makes a distinction between ignored and irrelevant \u000A formals.  Note however that if a variable is <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt>d <v>ignore</v>d or \u000A <v>ignorable</v>, then it will not be reported as irrelevant.</p> \u000A \u000A <p>An example of an irrelevant formal is <v>x</v> in the definition of <v>fact</v> \u000A below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (i x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ZEROP\">zerop</see> i) 1 (<see topic=\"COMMON-LISP_____A2\">*</see> i (fact (<see topic=\"COMMON-LISP____1-\">1-</see> i) (<see topic=\"COMMON-LISP____CONS\">cons</see> i x))))).</code> \u000A \u000A <p>Observe that <v>x</v> is only used in recursive calls of <v>fact</v>; it never \u000A ``gets out'' into the result.  ACL2 can detect some irrelevant formals by a \u000A closure analysis on how the formals are used.  For example, if the <v>i</v>th \u000A formal is only used in the <v>i</v>th argument position of recursive calls, then \u000A it is irrelevant.  This is how <v>x</v> is used above.</p> \u000A \u000A <p>It is possible for a formal to appear only in recursive calls but still be \u000A relevant.  For example, <v>x</v> is <b>not</b> irrelevant below, even though it \u000A only appears in the recursive call.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (i x) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ZEROP\">zerop</see> i) 0 (fn x (<see topic=\"COMMON-LISP____1-\">1-</see> i))))</code> \u000A \u000A <p>The key observation above is that while <v>x</v> only appears in a recursive \u000A call, it appears in an argument position, namely <v>i</v>'s, that is relevant. \u000A (The function above can be admitted with a <v>:measure</v> of <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____NFIX\">nfix</see> i)\u000A (<see topic=\"ACL2____NFIX\">nfix</see> x))</v>.)</p> \u000A \u000A <p>Establishing that a formal is irrelevant, in the sense defined above, can \u000A be an arbitrarily hard problem because it requires theorem proving.  For \u000A example, is <v>x</v> irrelevant below?</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> test (i j k x) (<see topic=\"COMMON-LISP____IF\">if</see> (p i j k) 0 x))</code> \u000A \u000A <p>Note that the value of <v>(test i j k x)</v> is independent of <v>x</v> &mdash; \u000A thus making <v>x</v> irrelevant &mdash; precisely if <v>(p i j k)</v> is a \u000A theorem.  ACL2's syntactic analysis of a definition does not guarantee to \u000A notice all irrelevant formals.</p> \u000A \u000A <p>We regard the presence of irrelevant formals as an indication that \u000A something is wrong with the definition.  We cause an error on such definitions \u000A and suggest that you recode the definition so as to eliminate the irrelevant \u000A formals.  If you must have an irrelevant formal, one way to ``trick'' ACL2 \u000A into accepting the definition, without slowing down the execution of your \u000A function, is to use the formal in an irrelevant way in the <see topic=\"ACL2____GUARD\">guard</see>.  For \u000A example, to admit fact, above, with its irrelevant <v>x</v> one might use</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (i x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ZEROP\">zerop</see> i) 0 (<see topic=\"COMMON-LISP_____A2\">*</see> i (fact (<see topic=\"COMMON-LISP____1-\">1-</see> i) (<see topic=\"COMMON-LISP____CONS\">cons</see> i x)))))</code> \u000A \u000A <p>For those who really want to turn off this feature, we have provided a way \u000A to use the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> for this purpose; see <see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see>.</p>"],

 "COMMON-LISP_____B2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>+</v> is really a macro that expands to calls of the function \u000A <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt>.  So for example</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> x y 4 z)</code> \u000A \u000A <p>represents the same term as</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> y (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> 4 z))).</code> \u000A \u000A <p>See <see topic=\"ACL2____BINARY-_B2\">binary-+</see>.</p> \u000A \u000A <p><b>Macro: </b><srclink>+</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> + (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> rst)\u000A          (<see topic=\"COMMON-LISP____IF\">if</see> rst\u000A              (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> rst)\u000A                  (xxxjoin 'binary-+ rst)\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> 'binary-+\u000A                        (<see topic=\"COMMON-LISP____CONS\">cons</see> 0 (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> rst) nil))))\u000A              0))</code>"],

 "COMMON-LISP____EQL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____EQL\">eql</see> x y)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v>.</p> \u000A \u000A <p>Unlike <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <v>eql</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring at least one of \u000A its arguments to be a number, a symbol, or a character.  Generally, <v>eql</v> \u000A is executed more efficiently than <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.</p> \u000A \u000A <p>For a discussion of the various ways to test against 0, See <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>Eql</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>eql</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> eql (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> x) (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> y))))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))</code>"],

 "ACL2____SET-STATE-OK":[["State"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>In brief: The variable symbol <tt><see topic=\"ACL2____STATE\">state</see></tt> has an unusual status in ACL2. \u000A In order to use it, you either need to issue <v>:set-state-ok t</v>, as we \u000A explain below, or you need to declare it to be a <see topic=\"ACL2____STOBJ\">stobj</see>, as explained \u000A elsewhere (see <see topic=\"ACL2____DECLARE-STOBJS\">declare-stobjs</see>).  Now we explain in more detail.</p> \u000A \u000A <p>Because the variable symbol <tt><see topic=\"ACL2____STATE\">state</see></tt> denotes the ``current ACL2 \u000A state,'' ACL2 treats the symbol very restrictively when it occurs as a formal \u000A parameter of a defined function.  The novice user, who is unlikely to be aware \u000A of the special status of that symbol, is likely to be confused when error \u000A messages about <v>STATE</v> are printed in response to the innocent choice of \u000A that symbol as a formal variable.  Therefore the top-level ACL2 loop can \u000A operate in a mode in which <tt><see topic=\"ACL2____STATE\">state</see></tt> is simply disallowed as a formal \u000A parameter.</p> \u000A \u000A <p>For a discussion of <v>STATE</v>, See <see topic=\"ACL2____STATE\">state</see> and see <see topic=\"ACL2____STOBJ\">stobj</see>. \u000A Roughly speaking, at the top-level, the ``current ACL2 state'' is denoted by \u000A the variable symbol <v>STATE</v>.  Only the current state may be passed into a \u000A function expecting a state as an argument.  Furthermore, the name of the \u000A formal parameter into which the current state is passed must be <v>STATE</v> and \u000A nothing but the current state may be passed into a formal of that name. \u000A Therefore, only certain access and change functions can use that formal \u000A &mdash; namely those with a <v>STATE</v> formal &mdash; and if any such function \u000A produces a new state it becomes the ``current state'' and must be passed along \u000A in the <v>STATE</v> position thereafter.  Thus, ACL2 requires that the state be \u000A single-threaded.  This, in turn, allows us to represent only one state at a \u000A time and to produce new states from it destructively in a von Neumaneque \u000A fashion.  The syntactic restrictions on the variable <v>STATE</v> are enforced \u000A by the translate mechanism (see <see topic=\"ACL2____TRANS\">trans</see> and see <see topic=\"ACL2____TERM\">term</see>) when terms \u000A are read.</p> \u000A \u000A <p>To prevent the novice user from seeing messages prohibiting certain uses of \u000A the variable symbol <v>STATE</v> ACL2 has a mode in which it simply disallows \u000A the use of that symbol as a formal parameter.  Use of the symbol causes a \u000A simple error message.  The system is initially in that mode.</p> \u000A \u000A <p>To get out of that mode, execute:</p> \u000A \u000A <code>:set-state-ok t ;;; or, (<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)</code> \u000A \u000A <p>It is not recommended that you do this until you have read the \u000A documentation of <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>To enter the mode in which use of <v>state</v> is prohibited as a formal \u000A parameter, do:</p> \u000A \u000A <code>:set-state-ok nil</code> \u000A \u000A <p>The mode is stored in the defaults table, See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>. \u000A Thus, the mode may be set <tt><see topic=\"ACL2____LOCAL\">local</see></tt>ly in books.</p>"],

 "ACL2____PARALLEL-PROGRAMMING":[["Parallelism"],"ACL2 Sources","ACL2","<p>Here we document support for parallel programming in ACL2(p), an \u000A experimental extension of ACL2; also see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>One of ACL2's strengths lies in its ability to execute industrial models \u000A efficiently.  The ACL2 source code provides an experimental parallel execution \u000A capability that can increase the speed of explicit evaluation, including \u000A simulator runs using such models, and it can also decrease the time required \u000A for proofs that make heavy use of the evaluation of ground terms.</p> \u000A \u000A <p>The parallelism primitives are <tt><see topic=\"ACL2____PLET\">plet</see></tt>, <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, <tt><see topic=\"ACL2____PAND\">pand</see></tt>, \u000A <tt><see topic=\"ACL2____POR\">por</see></tt>, and <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt>.  <tt><see topic=\"ACL2____PAND\">Pand</see></tt> and <tt><see topic=\"ACL2____POR\">por</see></tt> terminate \u000A early when an argument is found to evaluate to <v>nil</v> or non-<v>nil</v>, \u000A respectively, thus potentially improving on the efficiency of lazy evaluation. \u000A <tt><see topic=\"ACL2____SPEC-MV-LET\">Spec-mv-let</see></tt> is a modification of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> that supports \u000A speculative and parallel execution.</p> \u000A \u000A <p>Of the above five parallelism primitives, all but <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt> allow \u000A for limiting parallel execution (spawning of so-called ``threads'') depending \u000A on resource availability.  Specifically, the primitives allow specification of \u000A a size condition to control the <see topic=\"ACL2____GRANULARITY\">granularity</see> under which threads are \u000A allowed to spawn.  You can use such <see topic=\"ACL2____GRANULARITY\">granularity</see> declarations in \u000A recursively-defined functions to implement data-dependent parallelism in their \u000A execution.</p> \u000A \u000A <p>We recommend that in order to learn to use the parallelism primitives, you \u000A begin by reading examples: see <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see>.  That section will \u000A direct you to further documentation topics.</p> \u000A \u000A <p>In addition to providing parallel programming primitives, ACL2(p) also \u000A provides the ability to execute the main ACL2 proof process in parallel.  See \u000A <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> for further details.</p>"],

 "ACL2____DEFUN-NX":[["Defun","Events"],"ACL2 Sources","ACL2","<code>Example:\u000A\u000A(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)\u000A(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> foo (x state)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (a b c)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> x state)\u000A          (<see topic=\"COMMON-LISP____LIST\">list</see> a b c b a)))\u000A; Note ``ill-formed'' call of foo just below.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar (<see topic=\"ACL2____STATE\">state</see> y)\u000A  (foo state y))</code> \u000A \u000A <p>The macro <v>defun-nx</v> introduces definitions using the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A macro, always in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode, such that the calls of the \u000A resulting function cannot be evaluated.  Such a definition is admitted without \u000A enforcing syntactic restrictions for executability, in particular for \u000A single-threadedness (see <see topic=\"ACL2____STOBJ\">stobj</see>) and multiple-values passing (see <see topic=\"ACL2____MV\">mv</see> and see <see topic=\"ACL2____MV-LET\">mv-let</see>).  After such a definition is admitted, the usual \u000A syntactic rules for <tt><see topic=\"ACL2____STATE\">state</see></tt> and user-defined <see topic=\"ACL2____STOBJ\">stobj</see>s are relaxed \u000A for calls of the function it defines.  Also see <see topic=\"ACL2____NON-EXEC\">non-exec</see> for a way to \u000A designate subterms of function bodies, or subterms of code to be executed at \u000A the top level, as non-executable.</p> \u000A \u000A <p>The syntax of <v>defun-nx</v> is identical to that of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  A \u000A form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> name (x1 ... xk) ... body)</code> \u000A \u000A <p>expands to the following form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> name (x1 ... xk)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :non-executable t :mode :logic))\u000A  ...\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (throw-nonexec-error 'name (<see topic=\"COMMON-LISP____LIST\">list</see> x1 ... xk))\u000A          body))</code> \u000A \u000A <p>Note that because of the insertion of the above call of \u000A <v>throw-nonexec-error</v>, no formal is ignored when using <v>defun-nx</v>.</p> \u000A \u000A <p>During proofs, the error is silent; it is ``caught'' by the proof mechanism \u000A and generally results in the introduction of a call of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> during a \u000A proof.  If an error message is produced by evaluating a call of the function \u000A on a list of arguments that includes <v>state</v> or user-defined <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s, these arguments will be shown as symbols such as <v>|&lt;state&gt;|</v> in \u000A the error message.  In the case of a user-defined stobj bound by <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt> or <tt><see topic=\"ACL2____STOBJ-LET\">stobj-let</see></tt>, the symbol printed will include the \u000A suffix <v>{instance}</v>, for example, <v>|&lt;st&gt;{instance}|</v>.</p> \u000A \u000A <p>It is harmless to include <v>:non-executable t</v> in your own <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> \u000A <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form; <v>defun-nx</v> will still lay down its own such \u000A declaration, but ACL2 can tolerate the duplication.</p> \u000A \u000A <p>Note that <v>defund-nx</v> is also available.  It has an effect identical to \u000A that of <v>defun-nx</v> except that as with <tt><see topic=\"ACL2____DEFUND\">defund</see></tt>, it leaves the \u000A function disabled.</p> \u000A \u000A <p>If you use guards (see <see topic=\"ACL2____GUARD\">guard</see>), please be aware that even though \u000A syntactic restrictions are relaxed for <v>defun-nx</v>, guard verification \u000A proceeds exactly as for <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  If you want ACL2 to skip a form for \u000A purposes of generating guard proof obligations, use the macro <tt><see topic=\"ACL2____NON-EXEC\">non-exec</see></tt>, which generates a call of <v>throw-nonexec-error</v> that differs \u000A somewhat from the one displayed above.  See <see topic=\"ACL2____NON-EXEC\">non-exec</see>.</p> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DEFUN\">defun</see> for documentation of <v>defun</v>.</p>"],

 "COMMON-LISP____SEARCH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot;)                   ; = 4, index of first match\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot; :test 'equal)      ; same as above\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot; :from-end t)       ; = 8, index of last match\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot; :start1 1)         ; = 1\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot; :start2 5)         ; = 8\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;cd&quot; &quot;Cdabcdefcde&quot; :test 'char-equal) ; = 0 (case-insensitive)\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> &quot;ac&quot; &quot;Cdabcdefcde&quot;)                   ; = nil\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> '(a b) '(9 8 a b 7 6))                    ; = 2\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____SEARCH\">search</see> seq1 seq2 &amp;key from-end test start1 start2 end1 end2)</code> \u000A \u000A <p><v>Search</v> indicates whether one string or list occurs as a (contiguous) \u000A subsequence of another string or list, respectively.  It returns <v>nil</v> if \u000A no such match is found; otherwise it returns the (zero-based) index of the \u000A first match by default, but a non-<v>nil</v> value of keyword argument \u000A <v>:from-end</v> causes it to return the last match.  The <v>:test</v> is \u000A <v>equal</v> by default.  The other legal value for <v>:test</v> is \u000A <v>char-equal</v>, which can be given only for two strings, in which case the \u000A match is case-insensitive.  Finally, values of <v>:start1</v> and <v>:end1</v> for \u000A the first sequence, and of <v>:start2</v> and <v>:end2</v> for the second \u000A sequence, bound the portion of the respective sequence used for deciding on a \u000A match, though the index returned is always an index into the second sequence \u000A as a whole.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for calls of <v>search</v> is given by a function, \u000A <v>search-fn-guard</v>, which has the following requirements.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o The two arguments much both satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> or else must both \u000A be strings, which must consist of standard characters (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>) if the <v>:test</v> is <tt><see topic=\"COMMON-LISP____CHAR-EQUAL\">char-equal</see></tt>.</p> \u000A \u000A <p>o The <v>:test</v> must evaluate to one of the symbols <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> or \u000A <tt><see topic=\"COMMON-LISP____CHAR-EQUAL\">char-equal</see></tt>, where the latter is only allowed if the (first) two \u000A arguments are strings.</p> \u000A \u000A <p>o The values of <v>:start1</v>, <v>:start2</v>, <v>:end1</v>, and <v>:end2</v> must \u000A all be natural numbers, where if omitted they default to 0, 0, the length \u000A <v>len1</v> of the first argument, and the length <v>len2</v> of the second \u000A argument, respectively.</p> \u000A \u000A <p>o If <v>start1</v> is the value of <v>:start1</v>, defaulting as described just \u000A above, and similarly for the other start and end keywords and for lengths \u000A <v>len1</v> and <v>len2</v> as described just above, then: <v>start1 &lt;= end1 &lt;=\u000A len1</v> and <v>start2 &lt;= end2 &lt;= len2</v>.</p> \u000A \u000A </blockquote> \u000A \u000A <p><v>Search</v> is a Common Lisp function (actually, a macro in ACL2).  See any \u000A Common Lisp documentation for more information.</p> \u000A \u000A <h3>Definition</h3> \u000A \u000A <p><b>Macro: </b><srclink>search</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>\u000A search\u000A (seq1 seq2 &amp;key from-end (test ''equal)\u000A       (start1 '0)\u000A       (start2 '0)\u000A       (end1 'nil end1p)\u000A       (end2 'nil end2p))\u000A (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A  'search-fn\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A   seq1\u000A   (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A    seq2\u000A    (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A     from-end\u000A     (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A      test\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see>\u000A          start1\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> start2\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> end1\u000A                      (<see topic=\"COMMON-LISP____CONS\">cons</see> end2\u000A                            (<see topic=\"COMMON-LISP____CONS\">cons</see> end1p (<see topic=\"COMMON-LISP____CONS\">cons</see> end2p 'nil))))))))))))</code> \u000A <p><b>Function: </b><srclink>search-fn</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A search-fn\u000A (seq1 seq2 from-end test\u000A       start1 start2 end1 end2 end1p end2p)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A  (<see topic=\"ACL2____XARGS\">xargs</see>\u000A      :guard (search-fn-guard seq1 seq2 from-end test\u000A                              start1 start2 end1 end2 end1p end2p)))\u000A (<see topic=\"COMMON-LISP____LET_A2\">let*</see>\u000A  ((end1 (<see topic=\"COMMON-LISP____IF\">if</see> end1p end1 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq1)))\u000A   (end2 (<see topic=\"COMMON-LISP____IF\">if</see> end2p end2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq2)))\u000A   (seq1 (<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> seq1 start1 end1)))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A   (seq1 seq2)\u000A   (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQ\">eq</see> test 'char-equal)\u000A          (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____STRING-DOWNCASE\">string-downcase</see> seq1)\u000A              (<see topic=\"COMMON-LISP____STRING-DOWNCASE\">string-downcase</see> seq2)))\u000A         (t (<see topic=\"ACL2____MV\">mv</see> seq1 seq2)))\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____-\">-</see> end1 start1) (<see topic=\"COMMON-LISP____-\">-</see> end2 start2))\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> (from-end (search-from-end seq1 seq2 start2 end2 nil))\u000A              (t (search-from-start seq1 seq2 start2 end2)))))))</code> \u000A <p><b>Function: </b><srclink>search-from-end</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A   search-from-end\u000A   (seq1 seq2 start2 end2 acc)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq1))\u000A                               (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq2) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq2))\u000A                               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> start2)\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 start2)\u000A                               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> end2)\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> end2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq2))\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> start2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq1)) end2))))\u000A   (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> end2))\u000A              (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> start2)))\u000A          nil)\u000A         (t (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((bound2 (<see topic=\"COMMON-LISP_____B2\">+</see> start2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq1)))\u000A                   (matchp (<see topic=\"COMMON-LISP____EQUAL\">equal</see> seq1 (<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> seq2 start2 bound2)))\u000A                   (new-acc (<see topic=\"COMMON-LISP____IF\">if</see> matchp start2 acc)))\u000A                  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> bound2 end2) new-acc)\u000A                        (t (search-from-end seq1 seq2 (<see topic=\"COMMON-LISP____1_B2\">1+</see> start2)\u000A                                            end2 new-acc)))))))</code> \u000A <p><b>Function: </b><srclink>search-from-start</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A   search-from-start\u000A   (seq1 seq2 start2 end2)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq1))\u000A                               (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq2) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq2))\u000A                               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> start2)\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 start2)\u000A                               (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> end2)\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> end2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq2))\u000A                               (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> start2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq1)) end2))))\u000A   (<see topic=\"COMMON-LISP____LET\">let</see> ((bound2 (<see topic=\"COMMON-LISP_____B2\">+</see> start2 (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq1))))\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> end2))\u000A                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> start2)))\u000A               nil)\u000A              ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> seq1 (<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> seq2 start2 bound2))\u000A               start2)\u000A              ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> bound2 end2) nil)\u000A              (t (search-from-start seq1 seq2 (<see topic=\"COMMON-LISP____1_B2\">1+</see> start2)\u000A                                    end2)))))</code>"],

 "ACL2____STATE-GLOBAL-LET_A2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see> for requisite background on \u000A programming with the ACL2 <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((inhibit-output-lst *valid-output-names*))\u000A (<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x)))\u000A\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A ((fmt-hard-right-margin 1000 set-fmt-hard-right-margin)\u000A  (fmt-soft-right-margin 1000 set-fmt-soft-right-margin))\u000A (mini-proveall))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> ((var1 form1) ; or (var1 form1 set-var1)\u000A                    ...\u000A                    (vark formk) ; or (vark formk set-vark)\u000A                   )\u000A                   body)</code> \u000A \u000A <p>where: each <v>vari</v> is a variable; each <v>formi</v> is an expression whose \u000A value is a single ordinary object (i.e. not multiple values, and not <see topic=\"ACL2____STATE\">state</see> or any other <see topic=\"ACL2____STOBJ\">stobj</see>); <v>set-vari</v>, if supplied, is a function \u000A with <see topic=\"ACL2____SIGNATURE\">signature</see> <v>((set-vari * state) =&gt; state)</v>; and <v>body</v> is an \u000A expression that evaluates to an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>).  Each \u000A <v>formi</v> is evaluated in order, starting with <v>form1</v>, and with each such \u000A binding the state global variable <v>vari</v> is bound to the value of \u000A <v>formi</v>, sequentially in the style of <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>.  More precisely, then \u000A meaning of this form is to set (in order) the global values of the indicated \u000A <see topic=\"ACL2____STATE\">state</see> global variables <v>vari</v> to the values of <v>formi</v> using \u000A <tt><see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see></tt>, execute <v>body</v>, restore the <v>vari</v> to their \u000A previous values (but see the discussion of setters below), and return the \u000A triple produced by body (with its state as modified by the restoration).  The \u000A restoration is guaranteed even in the face of aborts.  The ``bound'' variables \u000A may initially be unbound in state and restoration means to make them unbound \u000A again.</p> \u000A \u000A <p>Still referring to the General Form above, let <v>old-vali</v> be the value \u000A of state global variable <v>vari</v> at the time <v>vari</v> is about to be \u000A assigned the value of <v>formi</v>.  If <v>set-vari</v> is not supplied, then as \u000A suggested above, the following form is evaluated at the conclusion of the \u000A evaluation of the <v>state-global-let*</v> form, whether or not an error has \u000A occurred: <v>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'vari 'old-vali state)</v>.  However, if \u000A <v>set-vari</v> is supplied, then instead the form evaluated will be \u000A <v>(set-vari 'old-vali state)</v>.  This capability is particularly useful if \u000A <v>vari</v> is untouchable (see <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>), since the above call \u000A of <tt><see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see></tt> is illegal.</p> \u000A \u000A <p>Note that the scope of the bindings of a <v>state-global-let*</v> form is the \u000A body of that form.  This may seem obvious, but to drive the point home, let's \u000A consider the following example (see <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> and see <see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see>).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> ((print-base 16 set-print-base)\u000A                           (print-radix t set-print-radix))\u000A                          (<see topic=\"ACL2____MV\">mv</see> nil 10 state))\u000A 10\u000AACL2 !&gt;</code> \u000A \u000A <p>Why wasn't the result printed as <v>#xA</v>?  The reason is that the result \u000A was printed after evaluation of the entire form had completed.  If you want to \u000A see <v>#xA</v>, do the printing in the scope of the bindings, for example as \u000A follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see> ((print-base 16 set-print-base)\u000A                           (print-radix t set-print-radix))\u000A                          (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____FMS\">fms</see> &quot;~x0~%&quot;\u000A                                       (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0 10))\u000A                                       *standard-co* state nil)\u000A                                  (<see topic=\"ACL2____MV\">mv</see> nil 10 state)))\u000A\u000A#xA\u000A 10\u000AACL2 !&gt;</code>"],

 "ACL2____NOTE9":[["Release-notes"],"ACL2 Sources","ACL2","<p>By default, when the system is started it is illegal to use the \u000A variable <tt><see topic=\"ACL2____STATE\">state</see></tt> as a formal parameter of a function definition.  The \u000A aim is to prevent novice users from stumbling into the Byzantine syntactic \u000A restrictions on that variable symbol.  Use</p> \u000A \u000A <code>:set-state-ok t</code> \u000A \u000A <p>or, equivalently,</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)</code> \u000A \u000A <p>to switch back to the old default mode.  See <see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see></p> \u000A \u000A <p><v>Set-state-ok</v> is an event that affects the ACL2 defaults table (see \u000A <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>).  Recall that when books are included, the \u000A defaults table is restored to its pre-inclusion state.  Thus, while a \u000A <v>set-state-ok</v> form will permit the book to define a <v>state</v>-using \u000A function, it will not permit the user of the book to make such a definition. \u000A We recommend putting <v>(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)</v> in any book that defines a \u000A <v>state</v> using function.</p> \u000A \u000A <p>Books certified under Version 1.8 must be recertified under Version 1.9. \u000A See :DOC version.</p> \u000A \u000A <p>The simplifier has been made to look out for built-in clauses, whereas in \u000A past versions such clauses were only noticed by the ``preprocessor'' at the \u000A top of the waterfall.  THIS CHANGE MAY PREVENT OLD SCRIPTS FROM REPLAYING! \u000A The undesirable side-effect is caused by the fact that <v>:HINTS</v> require you \u000A to refer to clauses by their exact name (see <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>) and because the \u000A new simplifier proves more clauses than before, the goals produced have \u000A different names.  Thus, if a script uses <v>:HINTS</v> that refer to clauses \u000A other than \"Goal\", e.g., \"Subgoal 1.3\" then the hint may be applied to a \u000A different subgoal than originally intended.</p> \u000A \u000A <p>The use of built-in-clauses has been made more efficient.  If a set of \u000A clauses arise often in a piece of work, it might be advantageous to build them \u000A in even if that results in a large set (hundreds?) of built-in clauses.  See \u000A <see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see></p> \u000A \u000A <p>Wormholes can now be used in :logic mode functions. See <see topic=\"ACL2____WORMHOLE\">wormhole</see></p> \u000A \u000A <p>It is now possible to provide ``computed hints.''  For example, have you \u000A ever wished to say ``in all goals with a name like this, :use that'' or ``if \u000A this term is in the subgoal, then :use that''?  Well, see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see> and the extraordinarily long example in see <see topic=\"ACL2____USING-COMPUTED-HINTS\">using-computed-hints</see>.</p> \u000A \u000A <p><v>Hide</v> terms may be rewritten with :rewrite rules about <v>hide</v>.  See \u000A <see topic=\"ACL2____HIDE\">hide</see>, where we also now explain why <v>hide</v> terms are sometimes \u000A introduced into your proof attempts.</p> \u000A \u000A <p>A bug that sometimes caused the ``non-lazy IF'' hard error message was \u000A fixed.</p> \u000A \u000A <p>A bug that sometimes caused a hard error in forward chaining was fixed.</p> \u000A \u000A <p>A bug in print-rules (:pr) was fixed.</p> \u000A \u000A <p>We report the use of :executable-counterparts in the evaluation of SYNTAXP \u000A forms.</p> \u000A \u000A <p>Some documentation errors were fixed.</p> \u000A \u000A <p>A bug in parent-tree tracking in add-literal-and-pt was fixed.</p> \u000A \u000A <p>A bug in ok$, go$ and eval$ was fixed.</p> \u000A \u000A <p>Clausify now optimizes (mv-nth 'k (list x0 ... xk ... xn)) to xk.</p>"],

 "ACL2____MSG":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____FMT\">fmt</see> for background on formatted printing with ACL2.</p> \u000A \u000A <p>We document <v>msg</v> precisely below, but first, we give an informal \u000A introduction and illustrate with an example.  Suppose you are writing a \u000A program that is to do some printing.  Typically, you will either pass the ACL2 \u000A state around (see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>) and use formatted printing \u000A functions that take the state as an argument (see <see topic=\"ACL2____FMT\">fmt</see>)), or else you \u000A might avoid using state by calling the utility, <tt><see topic=\"ACL2____CW\">cw</see></tt>, to do you \u000A printing.  Alternatively, you might print error messages upon encountering \u000A illegal situations; see <see topic=\"ACL2____ER\">er</see>.  But there are times where instead of \u000A printing immediately, you may prefer to pass messages around, for example to \u000A accumulate them before printing a final message.  In such cases, it may be \u000A desirable to construct ``message'' objects to pass around.</p> \u000A \u000A <p>For example, consider the following pair of little programs.  The first \u000A either performs a computation or prints an error, and the second calls the \u000A first on two inputs.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> invert1 (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A    (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;ERROR: ~x0 expected a cons, but was given ~x1.~|&quot;\u000A                'invert1 x)\u000A            nil)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> invert2 (x1 x2)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> (invert1 x1) (invert1 x2)))</code> \u000A \u000A <p>For example:</p> \u000A \u000A <code>ACL2 !&gt;(invert1 '(3 . 4))\u000A(4 . 3)\u000AACL2 !&gt;(invert1 'a)\u000AERROR: INVERT1 expected a cons, but was given A.\u000ANIL\u000AACL2 !&gt;(invert2 '(3 . 4) '(5 . 6))\u000A((4 . 3) (6 . 5))\u000AACL2 !&gt;(invert2 'a 'b)\u000AERROR: INVERT1 expected a cons, but was given A.\u000AERROR: INVERT1 expected a cons, but was given B.\u000A(NIL NIL)\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that when there are errors, there is no attempt to explain that \u000A these are due to a call of <v>invert2</v>.  That could be fixed, of course, by \u000A arranging for <v>invert2</v> to print its own error; but for complicated \u000A programs it can be awkward to coordinate printing from many sources.  So let's \u000A try a different approach.  This time, the first function returns two results. \u000A The first result is an ``error indicator'' &mdash; either a message object or \u000A <v>nil</v> &mdash; while the second is the computed value (only of interest when \u000A the first result is <v>nil</v>).  Then the higher-level function can print a \u000A single error message that includes the error message(s) from the lower-level \u000A function, as shown below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> invert1a (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"ACL2____MV\">mv</see> nil\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A    (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"ACL2____MSG\">msg</see> &quot;ERROR: ~x0 expected a cons, but was given ~x1.~|&quot;\u000A             'invert1a x)\u000A        nil)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> invert2a (x1 x2)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp1 y1)\u000A          (invert1a x1)\u000A          (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp2 y2)\u000A                  (invert1a x2)\u000A                  (<see topic=\"COMMON-LISP____IF\">if</see> erp1\u000A                      (<see topic=\"COMMON-LISP____IF\">if</see> erp2\u000A                          (<see topic=\"ACL2____CW\">cw</see> &quot;~x0 failed with two errors:~|  ~@1  ~@2&quot;\u000A                              'invert2a erp1 erp2)\u000A                        (<see topic=\"ACL2____CW\">cw</see> &quot;~x0 failed with one error:~|  ~@1&quot;\u000A                            'invert2a erp1))\u000A                    (<see topic=\"COMMON-LISP____IF\">if</see> erp2\u000A                        (<see topic=\"ACL2____CW\">cw</see> &quot;~x0 failed with one error:~|  ~@1&quot;\u000A                            'invert2a erp2)\u000A                      (<see topic=\"COMMON-LISP____LIST\">list</see> y1 y2))))))</code> \u000A \u000A <p>For example:</p> \u000A \u000A <code>ACL2 !&gt;(invert2a '(3 . 4) '(5 . 6))\u000A((4 . 3) (6 . 5))\u000AACL2 !&gt;(invert2a '(3 . 4) 'b)\u000AINVERT2A failed with one error:\u000A  ERROR: INVERT1A expected a cons, but was given B.\u000ANIL\u000AACL2 !&gt;(invert2a 'a 'b)\u000AINVERT2A failed with two errors:\u000A  ERROR: INVERT1A expected a cons, but was given A.\u000A  ERROR: INVERT1A expected a cons, but was given B.\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p>If you study the example above, you might well understand <v>msg</v>.  But we \u000A conclude with precise documentation.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____MSG\">msg</see> str arg1 ... argk)</code> \u000A \u000A <p>where <v>str</v> is a string and <v>k</v> is at most 9.</p> \u000A \u000A <p>This macro returns a pair suitable for giving to the <v>fmt</v> directive \u000A <v>~@</v>.  Thus, suppose that <v>#\\c</v> is bound to the value of <v>(<see topic=\"ACL2____MSG\">msg</see> str\u000A arg1 ... argk)</v>, where <v>c</v> is a character and <v>k</v> is at most 9.  Then \u000A the <v>fmt</v> directive <v>~@c</v> will print out the string, <v>str</v>, in the \u000A context of the alist in which the successive <v>fmt</v> variables <v>#\\0</v>, \u000A <v>#\\1</v>, ..., <v>#\\k</v> are bound to the successive elements of <v>(arg1\u000A ... argk)</v>.</p>"],

 "ACL2____HONS-NOTE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Certain ``Hons-Notes'' are printed to the terminal to report \u000A implementation-level information about the management of <see topic=\"ACL2____HONS\">hons</see> \u000A structures.  Some of these may be low-level and of interest mainly to system \u000A developers.  But below we discuss what users can learn from a particular \u000A hons-note, ``ADDR-LIMIT reached''.  In short: If you are seeing a lot of such \u000A hons-notes, then you may be using a lot of memory.  (Maybe you can reduce that \u000A memory; for certain BDD operations, for example (as defined in community books \u000A <v>books/centaur/ubdds/</v>), variable ordering can make a big difference.)</p> \u000A \u000A <p>By way of background:</p> \u000A \u000A <blockquote><p>The ADDR-HT is the main hash table that lets us look up <see topic=\"ACL2____NORMED\">normed</see> conses, i.e., <see topic=\"ACL2____HONS\">hons</see>es.  It is an ordinary Common Lisp hash \u000A table, so when it starts to get too full the Lisp will grow it.  It can get \u000A really big.  (It has been seen to take more than 10 GB of memory just for the \u000A hash table's array, not to mention the actual conses!)</p> \u000A \u000A <p>Hons-Notes may be printed when the ADDR-HT is getting really full.  This \u000A growth can be expensive and can lead to memory problems.  Think about what it \u000A takes to resize a hash table:</p> \u000A \u000A <p>(1) allocate a new, bigger array<br></br> \u000A \u000A   (2) rehash elements, copying them into the new array<br></br> \u000A \u000A   (3) free the old array</p> \u000A \u000A <p>Until you reach step (3) and a garbage collection takes place, you have to \u000A have enough memory to have both the old and new arrays allocated.  If the old \u000A array was 10 GB and we want to allocate a new one that's 15 GB, this can get \u000A pretty bad.  Also, rehashing the elements is expensive time-wise when there \u000A are lots of elements.</p> \u000A \u000A <p>Since resizing a hash table is expensive, we want to avoid it.  There's a \u000A <tt><see topic=\"ACL2____HONS-RESIZE\">hons-resize</see></tt> command for this.  You may want your books to start with \u000A something like the following.</p> \u000A \u000A <code>(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (set-max-mem (<see topic=\"COMMON-LISP_____A2\">*</see> 30 (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 30))))      ; 30 GB heap\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2____HONS-RESIZE\">hons-resize</see> :addr-ht #u_100_000_000)) ; 100M honses</code> \u000A \u000A <p>Basically, if you roughly know how many honses your book will need, you can \u000A just get them up front and then never to resize.</p></blockquote> \u000A \u000A <p>The Hons-Notes about ``ADDR-LIMIT reached'' are basically there to warn you \u000A that the ADDR-HT is being resized.  This can help you realize that your <tt><see topic=\"ACL2____HONS-RESIZE\">hons-resize</see></tt> command had too small of an ADDR-HT size, or might suggest that \u000A your book should have a <tt><see topic=\"ACL2____HONS-RESIZE\">hons-resize</see></tt> command.  There are also commands \u000A like <v>(</v><tt><see topic=\"ACL2____HONS-SUMMARY\">hons-summary</see></tt><v>)</v>, <see topic=\"ACL2____SET-MAX-MEM\">set-max-mem</see>, and defined in \u000A community book <v>books/centaur/misc/memory-mgmt-logic.lisp</v>, \u000A <v>(hons-analyze-memory nil)</v>.  These can show you how many honses you \u000A currently have, how much space they are taking, and that sort of thing.  (A \u000A nice trick is to call <tt><see topic=\"ACL2____HONS-SUMMARY\">hons-summary</see></tt> at the end of a book, to get an \u000A idea of how many honses you should ask for in your <tt><see topic=\"ACL2____HONS-RESIZE\">hons-resize</see></tt> \u000A command).</p>"],

 "ACL2____DEFREC":[["Events"],"ACL2 Sources","ACL2","<p>The <v>defrec</v> macro is built into ACL2 and is frequently used in \u000A the ACL2 sources to define basic kinds of structures.</p> \u000A \u000A <h3>Better Alternatives to Defrec</h3> \u000A \u000A <p>While <v>defrec</v> may be a reasonable choice for writing <see topic=\"ACL2____PROGRAM\">program</see>-mode code, most users would likely be better served by using one of \u000A the many, richer alternatives to <see topic=\"ACL2____DEFREC\">defrec</see> that are available in various \u000A books.  See for instance macros such as:</p> \u000A \u000A <ul> \u000A <li>The <see topic=\"STD____DEFAGGREGATE\">std::defaggregate</see> macro from <see topic=\"ACL2____STD_F2UTIL\">std/util</see>,</li> \u000A <li>The <see topic=\"ACL2____DEFDATA\">defdata</see> macro from the <see topic=\"ACL2____DATA-DEFINITIONS\">data-definitions</see> framework,</li> \u000A <li>The <see topic=\"FTY____DEFPROD\">fty::defprod</see> macro from the <see topic=\"FTY____FTY\">fty::fty</see> library, and</li> \u000A <li>The <v>data-structures/structures</v> book.</li> \u000A </ul> \u000A \u000A <p>A major reason to favor these macros over <v>defrec</v> is that they \u000A introduce the constructors and accessors for the structure as proper \u000A functions, rather than mere macros.  This is very helpful when you are trying \u000A to use ACL2 to reason about code that works with structures.  For instance, it \u000A means that your proof goals will involve terms like <v>(employee-&gt;position\u000A x)</v> instead of <v>(<see topic=\"COMMON-LISP____CDDDR\">cdddr</see> x)</v>.</p> \u000A \u000A <p>Another reason is that <v>defrec</v> does not support putting constraints on \u000A the fields of your structure.  For instance, you might want to have an \u000A <v>employee</v> structure where the <v>name</v> field is always a string.  You \u000A can't do this with <v>defrec</v>, but any of the above macros support it.</p> \u000A \u000A <p>Some of the above macros also have other useful features, e.g., integration \u000A with <see topic=\"ACL2____B_A2\">b*</see>, support for <see topic=\"ACL2____XDOC\">xdoc</see> documentation, and so forth.</p> \u000A \u000A <h3>Usage</h3> \u000A \u000A <p>A typical use of <v>defrec</v> might look like this:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFREC\">defrec</see> employee             ;; name of the structure\u000A  (<see topic=\"ACL2____NAME\">name</see> salary . position)   ;; fields of the structure\u000A  nil)                       ;; &quot;cheap&quot; flag</code> \u000A \u000A <p>This will result in the introduction of:</p> \u000A \u000A <ul> \u000A \u000A <li>A \"weak\" recognizer function, <v>weak-employee-p</v>, which recognizes \u000A cons structures that have the right shape.  We call the recognizer <i>weak</i> \u000A because it doesn't impose any constraints on the fields, e.g., there is no \u000A requirement that the <v>name</v> of an employee must be a string or that the \u000A <v>salary</v> must be a number.</li> \u000A \u000A <li>A <see topic=\"ACL2____MAKE\">make</see> macro that can be used like this: \u000A     <code>(<see topic=\"ACL2____MAKE\">make</see> employee :name &quot;Jimmy&quot;\u000A               :salary 0\u000A               :position &quot;Unpaid Intern&quot;)</code></li> \u000A \u000A <li>A <see topic=\"ACL2____CHANGE\">change</see> macro that can be used like this: \u000A     <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((jimmy (make-employee :name &quot;Jimmy&quot; ...)))\u000A  (<see topic=\"ACL2____CHANGE\">change</see> employee jimmy :salary 300000\u000A                         :position &quot;Vice President&quot;))</code></li> \u000A \u000A <li>Suitable <see topic=\"ACL2____ACCESS\">access</see> macros that can be used like this: \u000A     <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((jimmy (make-employee :name &quot;Jimmy&quot; ...)))\u000A  (<see topic=\"ACL2____ACCESS\">access</see> employee jimmy :name))</code></li> \u000A \u000A </ul>"],

 "ACL2____CURRENT-PACKAGE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p><v>Current-package</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see> state)</v> and the updater is \u000A <v>(set-current-package val state)</v>, or more conventionally, <v>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see>\u000A val)</v>.  The value of <v>current-package</v> is actually the string that names \u000A the package.  (Common Lisp's ``package'' objects do not exist in ACL2.)  The \u000A current package must be known to ACL2, i.e., it must be one of the initial \u000A packages or a package defined with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> by the user.</p> \u000A \u000A <p>When printing symbols, the package prefix is displayed if it is not the \u000A <v>current-package</v> and may be optionally displayed otherwise.  Thus, if \u000A <v>current-package</v> is <v>&quot;ACL2&quot;</v> then the symbol <v>'ACL2::SYMB</v> may be \u000A printed as <v>SYMB</v> or <v>ACL2::SYMB</v>, while <v>'MY-PKG::SYMB</v> must be \u000A printed as <v>MY-PKG::SYMB</v>.  But if <v>current-package</v> is <v>&quot;MY-PKG&quot;</v> \u000A then the former symbol must be printed as <v>ACL2::SYMB</v> while the latter may \u000A be printed as <v>SYMB</v>.</p> \u000A \u000A <p>In Common Lisp, <v>current-package</v> also affects how objects are read from \u000A character streams.  Roughly speaking, read and print are inverses if the \u000A <v>current-package</v> is fixed, so reading from a stream produced by printing \u000A an object must produce an equal object.</p> \u000A \u000A <p>In ACL2, the situation is more complicated because we never read objects \u000A from character streams, we only read them from object ``streams'' (channels). \u000A Logically speaking, the objects in such a channel are fixed regardless of the \u000A setting of <v>current-package</v>.  However, our host file systems do not \u000A support the idea of Lisp object files and instead only support character \u000A files.  So when you open an object input channel to a given (character file) \u000A we must somehow convert it to a list of ACL2 objects.  This is done by a \u000A <i>deus ex machina</i> (``a person or thing that appears or is introduced \u000A suddenly and unexpectedly and provides a contrived solution to an apparently \u000A insoluble difficulty,'' Webster's Ninth New Collegiate Dictionary).  Roughly \u000A speaking, the <i>deus ex machina</i> determines what sequence of calls to \u000A <v>read-object</v> will occur in the future and what the <v>current-package</v> \u000A will be during each of those calls, and then produces a channel containing the \u000A sequence of objects produced by an analogous sequence of Common Lisp reads \u000A with <v>*current-package*</v> bound appropriately for each.</p> \u000A \u000A <p>A simple rule suffices to make sane file <see topic=\"ACL2____IO\">io</see> possible: before you \u000A read an object from an object channel to a file created by printing to a \u000A character channel, make sure the <v>current-package</v> at read-time is the same \u000A as it was at print-time.</p>"],

 "ACL2____BDD":[["ACL2"],"ACL2 Sources","ACL2","<p>Ordered binary decision diagrams (OBDDs, often simply called BDDs) \u000A are a technique, originally published by Randy Bryant, for the efficient \u000A simplification of Boolean expressions.  In ACL2 we combine this technique with \u000A rewriting to handle arbitrary ACL2 terms that can represent not only Boolean \u000A values, but non-Boolean values as well.  In particular, we provide a setting \u000A for deciding equality of bit vectors (lists of Boolean values).</p> \u000A \u000A <p>An introduction to BDDs for the automated reasoning community may be found \u000A in ``Introduction to the OBDD Algorithm for the ATP Community'' by J Moore, \u000A <i>Journal of Automated Reasoning</i> (1994), pp. 33-45.  (This paper also \u000A appears as Technical Report #84 from Computational Logic, Inc.)</p> \u000A \u000A <p>Further information about BDDs in ACL2 can be found in the subtopics of \u000A this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> section.  In particular, see <see topic=\"ACL2____BDD-INTRODUCTION\">bdd-introduction</see> \u000A for a good starting place that provides a number of examples.</p> \u000A \u000A <p>See <see topic=\"ACL2____HINTS\">hints</see> for a description of <v>:bdd</v> hints.  For quick \u000A reference, here is an example; but only the <v>:vars</v> part of the hint is \u000A required, as explained in the documentation for <see topic=\"ACL2____HINTS\">hints</see>.  The values \u000A shown are the defaults.</p> \u000A \u000A <code>(:vars nil :bdd-constructors (<see topic=\"COMMON-LISP____CONS\">cons</see>) :prove t :literal :all)</code> \u000A \u000A <p>We suggest that you next visit the documentation topic <see topic=\"ACL2____BDD-INTRODUCTION\">bdd-introduction</see>.</p>"],

 "ACL2-PC_____D3":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A=     -- replace the current subterm by a term equated to it in\u000A         one of the hypotheses (<see topic=\"COMMON-LISP____IF\">if</see> such a term exists)\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> x) -- replace the current subterm by x, assuming that the prover\u000A         can show that they are equal\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z)\u000A      -- replace the term (<see topic=\"COMMON-LISP_____B2\">+</see> x y) by the term z inside the current\u000A         subterm, assuming that the prover can prove\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z) from the current top-level hypotheses\u000A         or that this term or (<see topic=\"COMMON-LISP____EQUAL\">equal</see> z (<see topic=\"COMMON-LISP_____B2\">+</see> x y)) is among the\u000A         current top-level hypotheses or the current governors\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> &amp; z)\u000A      -- exactly the same as above, if (<see topic=\"COMMON-LISP_____B2\">+</see> x y) is the current\u000A         subterm\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z :hints :none)\u000A      -- same as (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z), except that a new subgoal is\u000A         created with the current goal's hypotheses and governors\u000A         as its top-level hypotheses and (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z) as its\u000A         conclusion\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z 0)\u000A      -- exactly the same as immediately above\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> (p x)\u000A   (p y)\u000A   :equiv iff\u000A   :otf-flg t\u000A   :hints ((&quot;Subgoal 2&quot; :BY FOO) (&quot;Subgoal 1&quot; :use bar)))\u000A      -- same as (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y) z), except that the prover uses\u000A         the indicated values for otf-flg and hints, and only\u000A         propositional (<see topic=\"ACL2____IFF\">iff</see>) equivalence is used (however, it\u000A         must be that only propositional equivalence matters at\u000A         the current subterm)\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP_____D3\">=</see> &amp;optional x y &amp;rest keyword-args)</code> \u000A \u000A <p>If terms <v>x</v> and <v>y</v> are supplied, then replace <v>x</v> by <v>y</v> \u000A inside the current subterm if they are ``known'' to be ``equal''.  Here \u000A ``known'' means the following: the prover is called as in the <v>prove</v> \u000A command (using <v>keyword-args</v>) to prove <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v>, except that a \u000A keyword argument <v>:equiv</v> is allowed, in which case <v>(equiv x y)</v> is \u000A proved instead, where <v>equiv</v> is that argument.  (See below for how \u000A governors are handled.)</p> \u000A \u000A <p>Actually, <v>keyword-args</v> is either a single non-keyword or is a list of \u000A the form <v>((kw-1 x-1) ... (kw-n x-n))</v>, where each <v>kw-i</v> is one of the \u000A keywords <v>:equiv</v>, <v>:otf-flg</v>, <v>:hints</v>.  Here <v>:equiv</v> defaults \u000A to <v>equal</v> if the argument is not supplied or is <v>nil</v>, and otherwise \u000A should be the name of an ACL2 equivalence relation.  <v>:Otf-flg</v> and \u000A <v>:hints</v> give directives to the prover, as explained above and in the \u000A documentation for the <v>prove</v> command; however, no prover call is made if \u000A <v>:hints</v> is a non-<v>nil</v> atom or if <v>keyword-args</v> is a single \u000A non-keyword (more on this below).</p> \u000A \u000A <p><i>Remarks on defaults</i></p> \u000A \u000A <p>(1) If there is only one argument, say <v>a</v>, then <v>x</v> defaults to the \u000A current subterm, in the sense that <v>x</v> is taken to be the current subterm \u000A and <v>y</v> is taken to be <v>a</v>.</p> \u000A \u000A <p>(2) If there are at least two arguments, then <v>x</v> may be the symbol \u000A <v>&amp;</v>, which then represents the current subterm.  Thus, <v>(<see topic=\"COMMON-LISP_____D3\">=</see> a)</v> is \u000A equivalent to <v>(<see topic=\"COMMON-LISP_____D3\">=</see> &amp; a)</v>.  (Obscure point: actually, <v>&amp;</v> can be in any \u000A package, except the keyword package.)</p> \u000A \u000A <p>(3) If there are no arguments, then we look for a top-level hypothesis or a \u000A governor of the form <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> c u)</v> or <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> u c)</v>, where <v>c</v> is the \u000A current subterm.  In that case we replace the current subterm by <v>u</v>.</p> \u000A \u000A <p>As with the <v>prove</v> command, we allow goals to be given ``bye''s in the \u000A proof, which may be generated by a <v>:hints</v> keyword argument in \u000A <v>keyword-args</v>.  These result in the creation of new subgoals.</p> \u000A \u000A <p>A proof is attempted unless the <v>:hints</v> argument is a non-<v>nil</v> atom \u000A other than :<v>none</v>, or unless there is one element of <v>keyword-args</v> and \u000A it is not a keyword.  In that case, if there are any hypotheses in the current \u000A goal, then what is attempted is a proof of the implication whose antecedent is \u000A the conjunction of the current hypotheses and governors and whose conclusion \u000A is the appropriate <v>equal</v> term.</p> \u000A \u000A <p><b>Remarks:</b> (1) It is allowed to use abbreviations in the hints.  (2) \u000A The keyword :<v>none</v> has the special role as a value of :<v>hints</v> that is \u000A shown clearly in an example above.  (3) If there are governors, then the new \u000A subgoal has as additional hypotheses the current governors.</p>"],

 "ACL2____USER-DEFINED-FUNCTIONS-TABLE":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TABLE\">table</see> user-defined-functions-table 'untranslate-preprocess 'my-preprocess)\u000A(<see topic=\"ACL2____TABLE\">table</see> user-defined-functions-table 'untranslate 'my-untranslate)</code> \u000A \u000A <p>This feature should perhaps only be used by advanced users who have a \u000A thorough understanding of the system functions being replaced.  There are \u000A currently two ways a user can affect the way ACL2 prints terms.</p> \u000A \u000A <p>The first example associates the user-defined function symbol \u000A <v>my-preprocess</v> with <v>untranslate-preprocess</v>.  As a result, when ACL2 \u000A prints a term, say during a proof, using its so-called ``untranslate'' process \u000A the first thing it does is to call <v>my-preprocess</v> on two arguments: that \u000A term and the current ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.  If the call produces a \u000A non-<v>nil</v> result, then that result is passed to the untranslate \u000A process.</p> \u000A \u000A <p>The second example associates the user-defined function symbol \u000A <v>my-untranslate</v> with the built-in function symbol <v>untranslate</v>.  As a \u000A result, the code for <v>my-untranslate</v> will be run whenever the untranslate \u000A process is run.  The formals of the two functions must agree and must not \u000A contain any <see topic=\"ACL2____STOBJ\">stobj</see> names.  Note that these overrides fail to occur upon \u000A guard violations and some other evaluation errors.</p> \u000A \u000A <p>The <v>untranslate-preprocess</v> approach may suffice for most cases in \u000A which a user wants to modify the way output is produced by the theorem prover. \u000A We present an example immediately below, but see <see topic=\"ACL2____UNTRANSLATE-PATTERNS\">untranslate-patterns</see> \u000A for a more elaborate example.  If the <v>untranslate-preprocess</v> approach \u000A does not seem sufficient for your purposes, you are invited to look at \u000A community book <v>books/misc/rtl-untranslate.lisp</v> or the source code for \u000A <see topic=\"ACL2____DEFINE\">define</see> for an example of user-defined <v>untranslate</v> (i.e., following \u000A the second example displayed above).</p> \u000A \u000A <p>Suppose you have a large constant that you would prefer not to see in \u000A proofs.  For example, you may have submitted the following definition (but \u000A imagine a much larger constant, say, a list of length 1,000,000).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* '(a b c d))</code> \u000A \u000A <p>If you submit the following (silly) theorem</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x *a*) (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> yyy zzz))))</code> \u000A \u000A <p>then you will see the following output:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X '(A B C D)) YYY).</code> \u000A \u000A <p>If <v>*a*</v> had represented a much larger structure, we would wish we could \u000A see the following instead.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> X *A*) YYY)</code> \u000A \u000A <p>That can be accomplished as follows.  First we make the following \u000A definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-preprocess (<see topic=\"ACL2____TERM\">term</see> wrld)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> wrld))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> term (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote *a*))\u000A      '*a*\u000A    nil))</code> \u000A \u000A <p>Now we submit the following <tt><see topic=\"ACL2____TABLE\">table</see></tt> event.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> user-defined-functions-table\u000A       'untranslate-preprocess\u000A       'my-preprocess)</code> \u000A \u000A <p>This will install <v>my-preprocess</v> as a preprocessor before the normal \u000A untranslation routine is applied to printing a term.  When the untranslation \u000A routine encounters the constant <v>(<see topic=\"COMMON-LISP____QUOTE\">QUOTE</see> (A B C D))</v>, it will replace it \u000A with <v>*a*</v>, and the usual untranlation routine will print this as \u000A <v>*A*</v>.</p>"],

 "ACL2____SET-ABSSTOBJ-DEBUG":[["Defabsstobj"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic assumes familiarity with abstract \u000A stobjs.  See <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>.</p> \u000A \u000A <p>Below we explain what is meant by an error message such as the \u000A following.</p> \u000A \u000A <code>ACL2 Error in CHK-ABSSTOBJ-INVARIANTS:  Possible invariance violation\u000Afor an abstract stobj!  See :DOC set-absstobj-debug, and PROCEED AT\u000AYOUR OWN RISK.</code> \u000A \u000A <p>The use of <v>(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t)</v> will make this error message more \u000A informative, as follows, at the cost of slower execution &mdash; but in \u000A practice, the slowdown may be negligible (more on that below).</p> \u000A \u000A <code>ACL2 Error in CHK-ABSSTOBJ-INVARIANTS:  Possible invariance violation\u000Afor an abstract stobj!  See :DOC set-absstobj-debug, and PROCEED AT\u000AYOUR OWN RISK.  Evaluation was aborted under a call of abstract stobj\u000Aexport UPDATE-FLD-NIL-BAD.</code> \u000A \u000A <p>You may be best off starting a new ACL2 session if you see one of the \u000A errors above.  But you can continue at your own risk.  With a trust tag (see \u000A <see topic=\"ACL2____DEFTTAG\">defttag</see>), you can even fool ACL2 into thinking nothing is wrong, and \u000A perhaps you can fix up the abstract stobj so that indeed, nothing really is \u000A wrong.  See the community book <v>books/misc/defabsstobj-example-4.lisp</v> for \u000A how to do that.  That book also documents the <v>:always</v> keyword and a \u000A special value for the first argument, <v>:RESET</v>.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t)                 ; obtain extra debug info, as above\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t :event-p t)      ; same as above\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t\u000A                    :on-skip-proofs t) ; as above, but even in include-book\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t :event-p nil)    ; returns one value, not error triple\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> nil)               ; avoid extra debug info (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> val\u000A                    :event-p        event-p        ; default t\u000A                    :always         always         ; default nil\u000A                    :on-skip-proofs on-skip-proofs ; default nil\u000A                    )</code> \u000A \u000A <p>where the keyword arguments are optional with defaults as indicated above, \u000A and all supplied arguments are evaluated except for <v>on-skip-proofsp</v>, \u000A which must be Boolean (if supplied).  Keyword arguments are discussed at the \u000A end of this topic.</p> \u000A \u000A <p>Recall (see <see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see>) that for any exported function whose \u000A <v>:EXEC</v> function might (according to ACL2's heuristics) modify the concrete \u000A stobj non-atomically, one must specify <v>:PROTECT t</v>.  This results in extra \u000A code generated for the exported function, which provides a check that \u000A atomicity was not actually violated by a call of the exported function.  The \u000A extra code might slow down execution, but perhaps only negligibly in typical \u000A cases.  If you can tolerate a bit extra slow-down, then evaluate the form \u000A <v>(<see topic=\"ACL2____SET-ABSSTOBJ-DEBUG\">set-absstobj-debug</see> t)</v>.  Subsequent such errors will provide additional \u000A information, as in the example displayed earlier in this documentation \u000A topic.</p> \u000A \u000A <p>Finally we document the keyword arguments, other than <v>:ALWAYS</v>, which \u000A is discussed in a book as mentioned above.  When the value of <v>:EVENT-P</v> is \u000A true, which it is by default, the call of <v>set-absstobj-debug</v> will expand \u000A to an event.  That event is a call of <tt><see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see></tt>.  In that case, \u000A <v>:ON-SKIP-PROOFS</v> is passed to that call so that <v>set-absstobj-debug</v> \u000A has an effect even when proofs are being skipped, as during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  That behavior is the default; that is, <v>:ON-SKIP-PROOFS</v> is \u000A <v>nil</v> by default.  Also see <see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see>.  The value of keyword \u000A <v>:ON-SKIP-PROOFS</v> must always be either <v>t</v> or <v>nil</v>, but other than \u000A that, it is ignored when <v>EVENT-P</v> is <v>nil</v>.</p>"],

 "ACL2____NOTE-2-6-GUARDS":[["Note-2-6"],"ACL2 Sources","ACL2","<p>When you <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> that a function treats certain formals as \u000A <v>:</v><tt><see topic=\"ACL2____STOBJ\">stobj</see></tt><v>s</v>, the <see topic=\"ACL2____GUARD\">guard</see> of the function is automatically \u000A extended to include the corresponding stobj-recognizer calls.  For example, if \u000A a definition includes <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (ST)))</v> then the guard of \u000A the function is changed by the addition of the conjunct <v>(ST-P ST)</v>.</p> \u000A \u000A <p>One impact of this is that if you use the built-in ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> as a \u000A formal parameter of a function, <v>(STATE-P STATE)</v> is added to the guard. \u000A This may introduce a guard where there was none in previous versions of the \u000A system.  In older versions, therefore, no attempt would be made to <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, while in the new version, we would attempt guard verification. \u000A You may wish to add <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :verify-guards nil))</v> to such \u000A definitions.</p> \u000A \u000A <p>A related change affects users who do not use stobjs or <v>state</v>.  In \u000A previous versions of the system &mdash; as now &mdash; a <v>type</v> declaration \u000A extended the guard you provided explicitly.  Thus, if you wrote <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A (<see topic=\"COMMON-LISP____TYPE\">type</see> integer n))</v> then <v>(<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> n)</v> was added to your guard.  This is \u000A still the case and <v>:stobjs</v> recognizers are similarly added.  But in older \u000A versions of the system we ``added'' the conjuncts without checking whether \u000A they were already present in the guard you provided.  This sometimes produced \u000A such guards as <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n))</v> where the first was \u000A produced by your <v>type</v> declaration and the second was your <v>:guard</v>. \u000A We now eliminate redundant conjuncts; this may rearrange the order of the \u000A conjuncts.</p> \u000A \u000A <p>The guard conjectures for functions using <v>stobj</v>s have been simplified \u000A somewhat by taking advantage of the syntactic restrictions checked for \u000A single-threaded objects.</p> \u000A \u000A <p>The following functions have been modified so that character and string \u000A arguments are restricted to standard characters.  (See <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> \u000A and see <see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see>.)</p> \u000A \u000A <blockquote> \u000A \u000A <p><tt><see topic=\"COMMON-LISP____UPPER-CASE-P\">upper-case-p</see></tt> <tt><see topic=\"COMMON-LISP____LOWER-CASE-P\">lower-case-p</see></tt> <tt><see topic=\"COMMON-LISP____CHAR-UPCASE\">char-upcase</see></tt> <tt><see topic=\"COMMON-LISP____CHAR-DOWNCASE\">char-downcase</see></tt> <v>string-downcase1</v> <tt><see topic=\"COMMON-LISP____STRING-DOWNCASE\">string-downcase</see></tt> \u000A <v>string-upcase1</v> <tt><see topic=\"COMMON-LISP____STRING-UPCASE\">string-upcase</see></tt> <tt><see topic=\"COMMON-LISP____CHAR-EQUAL\">char-equal</see></tt> \u000A <v>string-equal1</v> <tt><see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see></tt></p> \u000A \u000A </blockquote> \u000A \u000A <p>Also, function <tt><see topic=\"ACL2____STANDARD-STRING-ALISTP\">standard-string-alistp</see></tt> replaces function \u000A <v>string-alistp</v>, with concomitant changes in the guard to <tt><see topic=\"ACL2____ASSOC-STRING-EQUAL\">assoc-string-equal</see></tt>, and in variable <v>*acl2-exports*</v>.  Also, lemma \u000A <v>standard-string-alistp-forward-to-alistp</v> replaces lemma \u000A <v>string-alistp-forward-to-alistp</v>.  There is a new lemma \u000A <v>standard-char-p-nth</v>, which has also been added to \u000A <v>*acl2-exports*</v>.</p> \u000A \u000A <p>The guard had been inadvertently omitted from the definition of the \u000A function <tt><see topic=\"COMMON-LISP____SUBSTITUTE\">substitute</see></tt> (and its subroutine <v>substitute-ac</v>).  This \u000A omission has been corrected; also, the guard is slightly stronger than the \u000A documentation had claimed (and that has been corrected).</p>"],

 "ACL2____FAST-ALIST-FORK":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  It assumes familiarity with fast alists; see \u000A <see topic=\"ACL2____FAST-ALISTS\">fast-alists</see>.</p> \u000A \u000A <p>See <tt><see topic=\"ACL2____FAST-ALIST-FORK_12\">fast-alist-fork!</see></tt>, <tt><see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see></tt>, and <tt><see topic=\"ACL2____FAST-ALIST-CLEAN_12\">fast-alist-clean!</see></tt> for related utilities.</p> \u000A \u000A <p>Logically, <v>(<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> alist ans)</v> is defined as follows:</p> \u000A \u000A <p><b>Function: </b><srclink>fast-alist-fork</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fast-alist-fork (alist ans)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> alist) ans)\u000A             ((<see topic=\"COMMON-LISP____ATOM\">atom</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A              (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist) ans))\u000A             ((<see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)) ans)\u000A              (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist) ans))\u000A             (t (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)\u000A                                 (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist) ans)))))</code> \u000A \u000A <p>The alist argument need not be a fast alist.</p> \u000A \u000A <p>Typically ans is set to nil or some other atom.  In this case, shrinking \u000A produces a new, fast alist which is like alist except that (1) any \u000A \"malformed,\" atomic entries have been removed, (2) all \"shadowed pairs\" \u000A have been removed, and (3) incidentally, the surviving elements have been \u000A reversed.  This can be useful as a way to clean up any unnecessary bindings in \u000A alist, or as a way to obtain a \"deep copy\" of a fast alist that can extended \u000A independently from the original while maintaining discipline.</p> \u000A \u000A <p>Note that fast-alist-fork is potentially expensive, for the following two \u000A reasons.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o The alist is copied, dropping any shadowed pairs.  This process will \u000A require a hash table lookup for each entry in the alist; and it will require \u000A creating a new alist, which uses additional memory.</p> \u000A \u000A <p>o Unless ans is a fast alist that is stolen (see below), a new hash table \u000A is created, which uses additional memory.  This hash table is populated in \u000A time that is linear in the number of unique keys in the \u000A alist.</p></blockquote> \u000A \u000A <p>When ans is not an atom, good discipline requires that it is a fast alist. \u000A In this case, <v>fast-alist-fork</v> steals the hash table for ans and extends \u000A it with all of the bindings in alist that are not in ans.  From the \u000A perspective of <tt><see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see></tt>, you can think of the resulting alist \u000A as being basically similar to <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> ans alist)</v>, but in a different \u000A order.</p> \u000A \u000A <p>Note that when ans is not a fast alist (e.g., ans is an atom) then such \u000A stealing does not take place.</p> \u000A \u000A <p>A common idiom is to replace an alist by the result of shrinking it, in \u000A which case it is best to free the input alist, for example as follows.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((alist (<see topic=\"ACL2____FAST-ALIST-FREE-ON-EXIT\">fast-alist-free-on-exit</see> alist\u000A                                      (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> alist nil))))\u000A  ...)</code> \u000A \u000A <p>See <see topic=\"ACL2____FAST-ALIST-FREE-ON-EXIT\">fast-alist-free-on-exit</see> and see <see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see>.</p> \u000A \u000A "],

 "COMMON-LISP____PATHNAME":[["Books-reference"],"ACL2 Sources","ACL2","<p>The notion of pathname objects from Common Lisp is not supported in \u000A ACL2, nor is the function <v>pathname</v>.  However, ACL2 supports file \u000A operations, using conventions for naming files based on those of the \u000A Unix (trademark of AT&amp;T) operating system, so that the character <v>/</v> is \u000A used to terminate directory names.  Some file names are ``absolute'' \u000A (complete) descriptions of a file or directory; others are ``relative'' to the \u000A current working directory or to the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>). \u000A We emphasize that even for users of Windows-based systems or Macintosh \u000A computers, ACL2 file names are in the Unix style.  We will call these <i>ACL2 \u000A pathnames</i>, often omitting the ``ACL2.''</p> \u000A \u000A <p>Pathnames starting with the directory separator (<v>/</v>) or the tilde \u000A character (<v>~</v>) are absolute pathnames.  All other pathnames are relative \u000A pathnames.  An exception is in the Microsoft Windows operating system, where \u000A it is illegal for the pathname to start with a tilde character but the drive \u000A may be included, e.g., <v>&quot;c:/home/smith/acl2/book-1.lisp&quot;</v>.  In fact, the \u000A drive <i>must</i> be included in the portcullis of a book; see <see topic=\"ACL2____PORTCULLIS\">portcullis</see>.  Note also that some host Common Lisps will not support pathnames \u000A starting with <v>&quot;~&quot;</v>, for example <v>~smith</v>, though ACL2 will generally \u000A support those starting with <v>&quot;~/&quot;</v> regardless of the host Common \u000A Lisp.</p> \u000A \u000A <p>Consider the following examples.  The filename string</p> \u000A \u000A <code>&quot;/home/smith/acl2/book-1.lisp&quot;</code> \u000A \u000A <p>is an absolute pathname, with top-level directory <v>&quot;home&quot;</v>, under that \u000A the directory <v>&quot;smith&quot;</v> and then the directory <v>&quot;acl2&quot;</v>, and \u000A finally, within that directory the file <v>&quot;book-1.lisp&quot;</v>.  If the \u000A connected book directory is <v>&quot;/home/smith/&quot;</v> (see <see topic=\"ACL2____CBD\">cbd</see>), then the \u000A filename string above also corresponds to the relative filename string \u000A \"acl2/book1.lisp\".</p> \u000A \u000A <p>Finally, we note that (on non-Windows systems) the pathname <v>&quot;~&quot;</v> and \u000A pathnames starting with <v>&quot;~/&quot;</v> are illegal in books being certified. \u000A Otherwise, a subsidiary <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form would have a different \u000A meaning at certification time than at a later time when the certified book is \u000A included by a different user.</p>"],

 "COMMON-LISP____AND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>And</v> is the macro for conjunctions.  <v>And</v> takes any number \u000A of arguments.  <v>And</v> returns <v>nil</v> if one of the arguments is <v>nil</v>, \u000A but otherwise returns the last argument.  If there are no arguments, <v>and</v> \u000A returns <v>t</v>.</p> \u000A \u000A <p><v>And</v> is a Common Lisp macro.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>and</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> and (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (and-macro args))</code> \u000A <p><b>Function: </b><srclink>and-macro</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> and-macro (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> lst)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A               (<see topic=\"COMMON-LISP____LIST\">list</see> 'if\u000A                     (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A                     (and-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A                     nil)\u000A               (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A           t))</code>"],

 "ACL2____STOBJ-EXAMPLE-1-DEFUNS":[["Stobj"],"ACL2 Sources","ACL2","<p>Consider the event shown in <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see>:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> counters\u000A  (NodeCnt     :type integer :initially 0)\u000A  (TipCnt      :type integer :initially 0)\u000A  (IntTipsSeen :type t       :initially nil))</code> \u000A \u000A <p>Here is a complete list of the defuns added by the event.</p> \u000A \u000A <p>The careful reader will note that the <v>counters</v> argument below is \u000A <i>not</i> declared with the <v>:stobjs</v> <v>xarg</v> even though we insist that \u000A the argument be a stobj in calls of these functions.  This ``mystery'' is \u000A explained below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> NodeCntp (x)                 ;;; Recognizer for 1st field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> TipCntp (x)                  ;;; Recognizer for 2nd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> IntTipsSeenp (x)             ;;; Recognizer for 3rd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards t) (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A  t)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> countersp (counters)         ;;; Recognizer for object\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> counters)\u000A       (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> counters) 3)\u000A       (NodeCntp (<see topic=\"COMMON-LISP____NTH\">nth</see> 0 counters))\u000A       (TipCntp (<see topic=\"COMMON-LISP____NTH\">nth</see> 1 counters))\u000A       (IntTipsSeenp (<see topic=\"COMMON-LISP____NTH\">nth</see> 2 counters))\u000A       t))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> create-counters ()           ;;; Creator for object\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> '0 '0 'nil))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> NodeCnt (counters)           ;;; Accessor for 1st field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (countersp counters) :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____NTH\">nth</see> 0 counters))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-NodeCnt (v counters)  ;;; Updater for 1st field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v)\u000A                       (countersp counters))\u000A                  :verify-guards t))\u000A  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 0 v counters))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> TipCnt (counters)            ;;; Accessor for 2nd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (countersp counters) :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____NTH\">nth</see> 1 counters))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-TipCnt (v counters)   ;;; Updater for 2nd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v)\u000A                       (countersp counters))\u000A                  :verify-guards t))\u000A  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1 v counters))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> IntTipsSeen (counters)       ;;; Accessor for 3rd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (countersp counters) :verify-guards t))\u000A  (<see topic=\"COMMON-LISP____NTH\">nth</see> 2 counters))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-IntTipsSeen (v counters) ;;; Updater for 3rd field\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (countersp counters) :verify-guards t))\u000A  (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2 v counters))</code> \u000A \u000A <p>Observe that there is a recognizer for each of the three fields and then a \u000A recognizer for the <v>counters</v> object itself.  Then, for each field, there \u000A is an accessor and an updater.</p> \u000A \u000A <p>Observe also that the functions are guarded so that they expect a \u000A <v>countersp</v> for their <v>counters</v> argument and an appropriate value for \u000A the new field values.</p> \u000A \u000A <p>You can see all of the <v>defuns</v> added by a <v>defstobj</v> event by \u000A executing the event and then using the <v>:pcb!</v> command on the stobj name. \u000A E.g.,</p> \u000A \u000A <code>ACL2 !&gt;:pcb! counters</code> \u000A \u000A <p>will print the defuns above.</p> \u000A \u000A <p>We now clear up the ``mystery'' mentioned above.  Note, for example in \u000A <v>TipCnt</v>, that the formal <v>counters</v> is used.  From the discussion in \u000A <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see> it has been made clear that <v>TipCnt</v> can only be \u000A called on the <v>counters</v> object.  And yet, in that same discussion it was \u000A said that an argument is so treated only if it it declared among the \u000A <v>:stobjs</v> in the definition of the function.  So why doesn't <v>TipCnt</v> \u000A include something like <v>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (counters)))</v>?</p> \u000A \u000A <p>The explanation of this mystery is as follows.  At the time <v>TipCnt</v> was \u000A defined, during the introduction of the <v>counters</v> stobj, the name \u000A ``<v>counters</v>'' was not yet a single-threaded object.  The introduction of a \u000A new single-threaded object occurs in three steps: (1) The new primitive \u000A recognizers, accessors, and updaters are introduced as ``ordinary functions,'' \u000A producing their logical axiomatizations.  (2) The executable counterparts are \u000A defined in raw Lisp to support destructive updating.  (3) The new name is \u000A declared a single-threaded object to ensure that all future use of these \u000A primitives respects the single-threadedness of the object.  The functions \u000A defined as part of the introduction of a new single-threaded object are the \u000A only functions in the system that have undeclared stobj formals other than \u000A <v>state</v>.</p> \u000A \u000A <p>You may return to <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see> here.</p>"],

 "ACL2____SET-FC-CRITERIA":[["Forward-chaining-reports"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>)                 ; shut off all tracking\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> nil)\u000A\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> t)               ; to track all forward chaining\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> (t t t))\u000A\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>\u000A ((:FORWARD-CHAINING LEMMA1)      ; track all uses of LEMMA1,\u000A   t\u000A   t)\u000A  ((:FORWARD-CHAINING LEMMA2)     ; uses of LEMMA2 triggered\u000A   (<see topic=\"ACL2____ALISTP\">ALISTP</see> (BASIC-MAPPER A B))    ; by this specific ALISTP term\u000A   t)\u000A  (t t (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (DLT D))))     ; and every rule deriving\u000A                                  ; this TRUE-LISTP term.\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> nil)\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> t)\u000A(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> triple1 triple2 ...)</code> \u000A \u000A <p>where each triple is of the form <v>(<see topic=\"ACL2____RUNE\">rune</see> inst-trigger inst-concl)</v>.  If \u000A rune is non-<v>t</v> is must be a forward chaining <see topic=\"ACL2____RUNE\">rune</see>.  The other two \u000A components, <v>inst-trigger</v> and <v>inst-concl</v>, if non-<v>t</v>, must be \u000A terms.  The list of all the triples supplied is called the ``criteria.''  In \u000A the form <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> nil)</v>, the criteria used is the empty list of \u000A triples.  (Technically, supplying <v>nil</v> as a triple ``ought'' to be an \u000A error, but it is a more ``natural'' way to say the list of criteria is empty \u000A than to use the correct form <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>)</v>.)  In the form \u000A <v>(<see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see> t)</v> the criteria used is the list containing the single \u000A triple <v>(t t t)</v>.</p> \u000A \u000A <p>This function sets the tracking criteria for forward chaining reporting. \u000A See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for a general discussion of tracking and \u000A reporting forward chaining activity.</p> \u000A \u000A <p>The forward chaining criteria is a list of triples.  Think of it as \u000A representing a disjunction of conjunctions.  The activation of a <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rune by some triggering term in the current context \u000A <i>satisfies</i> the criteria if it satisfies one of the triples.  To satisfy \u000A the triple <v>(<see topic=\"ACL2____RUNE\">rune</see> inst-trigger inst-concl)</v>, the activation must satisfy \u000A each component of the triple.  Any <v>t</v> component is always satisfied.  If \u000A <v>rune</v> is non-<v>t</v> it is satisfied if the activation is for the given \u000A rune.  If <v>inst-trigger</v> is non-<v>t</v>, it is satisfied if the activation \u000A is triggered by the given term.  (``<v>Inst-trigger''</v> stands for \u000A ``instantiated trigger.''  It is not the trigger term of the rule but is \u000A supposed to be an instance of that term that you believe will arise in some \u000A proof attempt you are debugging -- an instance you want to ``watch'' as it \u000A fires the rule.)  If <v>inst-concl</v> is non-<v>t</v>, it is satisfied if the \u000A activation could possibly derive the conclusion given.  (Again, \u000A ``<v>inst-concl''</v> stands for ``instantiated conclusion'' and shows the term \u000A in your problem that you expect to be derived by forward chaining.)</p> \u000A \u000A <p>Note that if the criteria is empty, it is never satisfied, so tracking is \u000A turned off.  If the criteria is the singleton list containing just the triple \u000A <v>(t t t)</v>, then every activation satisfies it and so all <v>:forward\u000A chaining</v> rules are tracked.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for details.</p>"],

 "ACL2____IN-TAU-INTERVALP":[["Tau-system"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> e x)</code> \u000A \u000A <p>Here, <v>x</v> should be an interval (see <tt><see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see></tt>).  This \u000A function returns <v>t</v> or <v>nil</v> indicating whether <v>e</v>, which is \u000A generally but not necessarily a number, is an element of interval <v>x</v>.  By \u000A that is meant that <v>e</v> satisfies the domain predicate of the interval and \u000A lies between the two bounds.</p> \u000A \u000A <p>Suppose <v>x</v> is an interval with the components <i>dom</i>, \u000A <i>lo-rel</i>, <i>lo</i>, <i>hi-rel</i> and <i>hi</i>.  Suppose <v>(&lt;?\u000A </v><i>rel u v</i><v>)</v> means <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> </v><i>u v</i><v>)</v> when <i>rel</i> is true \u000A and <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> </v><i>u v</i><v>)</v> otherwise, with appropriate treatment of \u000A infinities.</p> \u000A \u000A <p>Then for <v>e</v> to be in interval <v>x</v>, it must be the case that <v>e</v> \u000A satisfies the domain predicate <i>dom</i> (where where <i>dom</i>=<v>nil</v> \u000A means there is no restriction on the domain) and <v>(&lt;? </v><i>lo-rel lo</i><v>\u000A e)</v> and <v>(&lt;? </v><i>hi-rel</i><v> e </v><i>hi</i><v>)</v>.  [Note: ``Appropriate \u000A treatment of infinities'' is slightly awkward if both infinities are \u000A represented by the same object, <v>nil</v>.  However, this is handled by \u000A coercing <v>e</v> to a number <i>after</i> checking that it is in the domain. \u000A By this trick, ``<v>&lt;?</v>''  is presented with at most one ``infinity'' and it \u000A is always negative when in the first argument and positive when in the \u000A second.]</p> \u000A \u000A <p>Note that every element in an <v>INTEGERP</v> interval is contained in the \u000A analogous <v>RATIONALP</v> interval (i.e., the interval obtained by just \u000A replacing the domain <v>INTEGERP</v> by <v>RATIONALP</v>).  That is because every \u000A integer is a rational.  Similarly, every rational is an ACL2 number.</p> \u000A \u000A <p>Note that an interval in which the relations are weak and the bounds are \u000A equal rationals is the ``unit'' or ``identity'' interval containing exactly \u000A that rational.</p> \u000A \u000A <p>Note that an interval in which the relations are strong and the bounds are \u000A equal rationals is empty: it contains no objects.</p> \u000A \u000A <p>Note that the interval <v>(<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> nil nil nil nil nil)</v> is the \u000A ``universal interval:'' it contains all ACL2 objects.  It contains all numbers \u000A because they statisfy the non-existent domain restriction and lie between \u000A minus infinity and plus infinity.  It contains all non-numbers because the \u000A interval contains <v>0</v> and ACL2's inequalities coerce non-numbers to <v>0</v>. \u000A The universal interval is useful if you are defining a bounder (see <see topic=\"ACL2____BOUNDERS\">bounders</see>) for a function and do not wish to address a certain case: return \u000A the universal interval.</p> \u000A \u000A <p>Recall that <tt><see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see></tt> constructs intervals.  Using \u000A <v>make-tau-interval</v> we give several self-explanatory examples of \u000A <v>in-tau-intervalp</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 3 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10))           = t\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 3 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'RATIONALP nil 0 nil 10))          = t\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 3 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> NIL nil 0 nil 10))                 = t\u000A\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> -3 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10))          = nil\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 30 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10))          = nil\u000A\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 3/5 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10))         = nil\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 3/5 (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'RATIONALP nil 0 nil 10))        = t\u000A\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> #c(3 5) (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'RATIONALP nil 0 nil 10))    = nil\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> #c(3 5) (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'ACL2-NUMBERP nil 0 nil 10)) = t\u000A\u000A(<see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see> 'ABC (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> NIL nil 0 nil 10))              = t</code>"],

 "ACL2_____04":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2_____04\">@</see> y) 1)\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> a (<see topic=\"ACL2____ASET1\">aset1</see> 'ascii-map-array (<see topic=\"ACL2_____04\">@</see> a) 66 'Upper-case-B))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2_____04\">@</see> symbol)</code> \u000A \u000A <p>where <v>symbol</v> is any symbol to which you have <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt>ed a global \u000A value.  This macro expands into <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'symbol state)</v>, which \u000A retrieves the stored value of the symbol.</p> \u000A \u000A <p>The macro <v>f-get-global</v> is closely related to <tt><see topic=\"ACL2_____04\">@</see></tt>: <v>(<see topic=\"ACL2_____04\">@</see> var)</v> \u000A macroexpands to <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'var state)</v>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> makes it convenient to set the value of a symbol. \u000A The <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> operation has no effect on the <v>global-table</v> of \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thus, you may use these globals to hang onto useful data \u000A structures even though you may undo back past where you computed and saved \u000A them.</p>"],

 "ACL2____NU-REWRITER":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>The rewriter contains special provisions for rewriting expressions \u000A composed of <v>nth</v>, <v>update-nth</v>, <v>update-nth-array</v>, together with \u000A <v>let</v> expressions and other applications of non-recursive functions or \u000A <v>lambda</v> expressions.  For details see the paper ``Rewriting for Symbolic \u000A Execution of State Machine Models'' by J Strother Moore.  Also see <see topic=\"ACL2____SET-NU-REWRITER-MODE\">set-nu-rewriter-mode</see>.</p> \u000A \u000A <p>The ``nu-rewriter'' is a recent addition to the main rewrite engine in \u000A ACL2.  Consider the expression</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1 (new-a x s) s)))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2 (new-b x s) s)))\u000A    (<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 3 (new-c x s) s)))\u000A      s)))</code> \u000A \u000A <p>If the <v>let</v>s in this expression are expanded, a very large expression \u000A results because of the duplicate occurrences of <v>s</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 3\u000A            (new-c x\u000A                   (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2\u000A                               (new-b x\u000A                                      (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1\u000A                                                  (new-a x s)\u000A                                                  s))\u000A                               (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1\u000A                                           (new-a x s)\u000A                                           s)))\u000A            (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2\u000A                        (new-b x\u000A                               (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1\u000A                                           (new-a x s)\u000A                                           s))\u000A                        (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1\u000A                                    (new-a x s)\u000A                                    s))).</code> \u000A \u000A <p>This expansion of the <v>let</v> expression can be very expensive in space \u000A and time.  In particular, the <v>(new-a x s)</v> expression might be rewritten \u000A many times.</p> \u000A \u000A <p>Now imagine asking what 2nd component of the structure is.  That is, \u000A consider</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____NTH\">nth</see> 2\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1 (new-a x s) s)))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 2 (new-b x s) s)))\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((s (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 3 (new-c x s) s)))\u000A           s))))</code> \u000A \u000A <p>The normal ACL2 rewrite engine would answer this question by first \u000A rewriting the arguments to the <v>nth</v> expression; in particular, it would \u000A expand the nested <v>let</v> expression to the large nested <v>update-nth</v> \u000A expression and then, using rules such as</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> nth-update-nth\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> m (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> n val l))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> m) (<see topic=\"ACL2____NFIX\">nfix</see> n))\u000A             val (<see topic=\"COMMON-LISP____NTH\">nth</see> m l))))</code> \u000A \u000A <p>would reduce the expression to <v>(new-b x (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 1 (new-a x s)\u000A s))</v>.</p> \u000A \u000A <p>The purpose of the nu-rewriter is to allow simplifications like this \u000A without first expanding the <v>let</v>s.  The ``nu'' in the name is an acronym \u000A for ``<v>nth/update-nth</v>''.  The nu-rewriter knows how to move an <v>nth</v> \u000A into a <v>let</v> without expanding the <v>let</v> and how to simplify it if it \u000A nestles up against an <v>update-nth</v>.</p> \u000A \u000A <p>There are four characteristics of this problem: the presence of <v>nth</v>, \u000A the presence of <v>update-nth</v>, the use of <v>let</v> to provide ``sequential'' \u000A updates, and the use of constant indices.  <v>Nth</v> and <v>update-nth</v> need \u000A not occur explicitly; they may be used inside of definitions of ``wrapper'' \u000A functions.</p> \u000A \u000A <p>Because the nu-rewriter changes the order in which things are rewritten, \u000A its routine use can make ACL2 unable to reproduce old proofs.  It is therefore \u000A switched off by default.  If your application exhibits the characteristics \u000A above, you might wish to switch the nu-rewriter on using <tt><see topic=\"ACL2____SET-NU-REWRITER-MODE\">set-nu-rewriter-mode</see></tt>.</p> \u000A \u000A <p>More will eventually be written about the nu-rewriter.  However, it is \u000A described in detail in the paper cited at the beginning of this documentation \u000A topic.</p>"],

 "ACL2____INTERSECTION_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> l1 l2 ... lk)\u000A(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> l1 l2 ... lk :test 'eql) ; same as above\u000A(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> l1 l2 ... lk :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> l1 l2 ... lk :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____INTERSECTION_42\">Intersection$</see> x y)</v> equals a list that contains the <v>member</v>s of \u000A <v>x</v> that are also <v>member</v>s of <v>y</v>.  More precisely, the resulting \u000A list is the result of deleting from <v>x</v> those members that that are not \u000A members of <v>y</v>.  The optional keyword, <v>:TEST</v>, has no effect logically, \u000A but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing members of the \u000A two lists.</p> \u000A \u000A <p><v>Intersection$</v> need not take exactly two arguments, though it must take \u000A at least one argument: <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x)</v> is <v>x</v>, <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x y\u000A z ... :test test)</v> is <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x (<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> y z ... :test test)\u000A :test test)</v>, and if <v>:TEST</v> is not supplied, then <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x y z\u000A ...)</v>  is <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x (<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> y z ...))</v>.  For the discussion \u000A below we restrict ourselves, then, to the cases <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x y)</v> and \u000A <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x y :test test)</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>intersection$</v> (in the two cases just \u000A above) depends on the test.  In all cases, both arguments must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then one of the arguments must \u000A satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then one of the \u000A arguments must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>intersection$</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____INTERSECTION-EQ\">intersection-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see>\u000A x lst :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> x lst\u000A :test 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>intersection-equal</v>.</p> \u000A \u000A <p>Note that <v>intersection-eq</v> can take any positive number of arguments, \u000A in analogy to <v>intersection$</v>; indeed, <v>(<see topic=\"ACL2____INTERSECTION-EQ\">intersection-eq</see> ...)</v> expands \u000A to <v>(<see topic=\"ACL2____INTERSECTION_42\">intersection$</see> ... :test 'eq)</v>.  However, <v>intersection-equal</v> is a \u000A function, not a macro, and takes exactly two arguments.</p> \u000A \u000A <p><v>Intersection$</v> is similar to the Common Lisp primitive \u000A <v>intersection</v>.  However, Common Lisp does not specify the order of \u000A elements in the result of a call of <v>intersection</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>intersection-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     intersection-equal (l1 l2)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l1) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l2))))\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l1) nil)\u000A           ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1) l2)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1)\u000A                  (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2)))\u000A           (t (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2))))</code>"],

 "ACL2____DEFTHEORY":[["Events","Theories"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> interp-theory\u000A           (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see>\u000A             (<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)\u000A             (<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> 'interp-section)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> name term :doc doc-string)</code> \u000A \u000A <p>where <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>), <v>term</v> is a \u000A term that when evaluated will produce a theory (see <see topic=\"ACL2____THEORIES\">theories</see>), and \u000A <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).  Except for the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, <v>term</v> must contain \u000A no free variables.  <v>Term</v> is evaluated with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the \u000A current world (see <see topic=\"ACL2____WORLD\">world</see>) and the resulting theory is then converted to \u000A a <i>runic theory</i> (see <see topic=\"ACL2____THEORIES\">theories</see>) and associated with <v>name</v>. \u000A Henceforth, this runic theory is returned as the value of the theory \u000A expression <v>(<see topic=\"ACL2____THEORY\">theory</see> name)</v>.</p> \u000A \u000A <p>The value returned is the length of the resulting theory.  For example, in \u000A the following, the theory associated with <v>'FOO</v> has 54 <see topic=\"ACL2____RUNE\">rune</see>s:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> foo (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> '(<see topic=\"ACL2____BINARY-APPEND\">binary-append</see>)\u000A                                      (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)))\u000A\u000ASummary\u000AForm:  ( DEFTHEORY FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A 54\u000AACL2 !&gt;</code> \u000A \u000A <p>Note that the theory being defined depends on the context.  For example, \u000A consider the following (contrived) example book.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFUND\">defund</see> foo (x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)) ; defund disables foo\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> foo)))\u000A(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> my-theory (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> foo))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-property\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (foo x))\u000A  :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> my-theory))))</code> \u000A \u000A <p>At the time <v>foo-property</v> is proved admissible during book \u000A certification (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>), the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> \u000A event has previously been evaluated, so the <see topic=\"ACL2____DEFINITION\">definition</see> of <v>foo</v> is \u000A <see topic=\"ACL2____ENABLE\">enable</see>d.  Thus, the <v>:in-theory</v> hint on <v>foo-property</v> will \u000A <see topic=\"ACL2____ENABLE\">enable</see> <v>foo</v>, and the theorem proves.  HOWEVER, when the book is \u000A later included (see <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>), the <tt><see topic=\"ACL2____LOCAL\">local</see></tt> event is skipped, \u000A so the definition of <v>foo</v> is <see topic=\"ACL2____DISABLE\">disable</see>d at the time the <see topic=\"ACL2____THEORY\">theory</see> \u000A <v>my-theory</v> is defined.  Hence, unlike the case for the admissibility pass \u000A of the book's certification, that theory does not include the definition of \u000A <v>foo</v> when the book is included.</p> \u000A \u000A <p>There is, however, a way to ensure that a <see topic=\"ACL2____THEORY\">theory</see> defined in a book \u000A is the same at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> time as it was during the admissibility \u000A pass of the book's certification; see <see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see>.</p> \u000A \u000A <p>Note that a <v>deftheory</v> event is never redundant, see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p>"],

 "ACL2____SET-WATERFALL-PRINTING":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see> :full)    ; print everything\u000A(<see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see> :limited) ; print a subset that's thought to be useful\u000A(<see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see> :very-limited) ; print an even smaller subset</code> \u000A \u000A <p><v>Set-waterfall-printing</v> evaluates its argument, which indicates how \u000A much printing should occur when executing ACL2 with the parallelized version \u000A of the waterfall.  It only affects the printing that occurs when parallelism \u000A mode is enabled for the waterfall (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>).</p> \u000A \u000A <p>A value of <v>:full</v> is intended to print the same output as in serial \u000A mode.  This output will be interleaved unless the waterfall-parallelism mode \u000A is one of <v>nil</v> or <v>:pseudo-parallel</v>.</p> \u000A \u000A <p>A value of <v>:limited</v> omits most of the output that occurs in the serial \u000A version of the waterfall.  Instead, the proof attempt prints key checkpoints \u000A (see <see topic=\"ACL2____ACL2P-KEY-CHECKPOINTS\">ACL2p-key-checkpoints</see>).  The value of <v>:limited</v> also prints \u000A messages that indicate which subgoal is currently being proved, along with the \u000A wall-clock time elapsed since the theorem began its proof; and if state global \u000A <v>'waterfall-printing-when-finished</v> has a non-<v>nil</v> value, then such a \u000A message will also be printed at the completion of each subgoal.  The function \u000A <v>print-clause-id-okp</v> may receive an attachment to limit such printing; see \u000A <see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see>.  Naturally, these subgoal numbers can appear out \u000A of order, because the subgoals can be proved in parallel.</p> \u000A \u000A <p>A value of <v>:very-limited</v> is treated the same as <v>:limited</v>, except \u000A that instead of printing subgoal numbers, the proof attempt prints a period \u000A (`<v>.</v>') each time it starts a new subgoal.</p> \u000A \u000A <p>Note that this form cannot be used at the top level of a book, or of a \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event.  Here is a workaround for use in \u000A such contexts; of course, you may replace <v>:very-limited</v> with any other \u000A legal argument for <v>set-waterfall-printing</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see> :very-limited)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))</code> \u000A \u000A <p>(For more about event contexts and the use of <v>make-event</v>, see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>, in particular the section ``Restriction to Event Contexts.'')</p> \u000A \u000A <p>The following form has the effect described above, except that it will \u000A affect waterfall-printing even when including a certified book that contains \u000A it.</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see> (<see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see> :very-limited)\u000A                      (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil)))\u000A            :check-expansion t)</code> \u000A \u000A <p>Note that <v>set-waterfall-printing</v> is automatically called by <tt><see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see></tt>.</p> \u000A \u000A <p>To enable the printing of information when a subgoal is finished, assign a \u000A non-<v>nil</v> value to global <v>waterfall-printing-when-finished</v>.  This can \u000A be accomplished by entering the following at the top level:</p> \u000A \u000A <code>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'waterfall-printing-when-finished t state)</code>"],

 "ACL2____SET-RW-CACHE-STATE":[["Rewrite"],"ACL2 Sources","ACL2","<p>The ACL2 rewriter uses a data structure, called the \u000A rw-cache (rewriter cache), to save failed attempts to apply conditional <see topic=\"ACL2____REWRITE\">rewrite</see> rules.  The regression suite has taken approximately 11% less time \u000A with this mechanism.  The rw-cache is active by default but this event allows \u000A it to be turned off or modified.  Note that this event is <see topic=\"ACL2____LOCAL\">local</see> to its \u000A context (from <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>).  For a non-local \u000A version, use <see topic=\"ACL2____SET-RW-CACHE-STATE_12\">set-rw-cache-state!</see>.</p> \u000A \u000A <code>Example forms:\u000A(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> :atom)     ; default: rw-cache cleared for each literal\u000A                               ;   (i.e., hypothesis or conclusion of a goal)\u000A(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> nil)       ; rw-cache is inactive\u000A(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> t)         ; rw-cache persists beyond each literal\u000A(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> :disabled) ; rw-cache is inactive, but the rw-cache-state\u000A                               ;   transitions to state t after\u000A                               ;   simplification takes place\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> val)</code> \u000A \u000A <p>where <v>val</v> evaluates to one of the four values shown in ``Example \u000A forms'' above.  The default is <v>:atom</v>, which enables the rw-cache but \u000A clears it before rewriting a hypothesis or conclusion of any goal.  The value \u000A <v>t</v> is provides more aggresive use of the rw-cache, basically preserving \u000A the rw-cache when there is a single subgoal.  The value <v>:disabled</v> is the \u000A same as <v>t</v>, except that the rw-cache is initially inactive and only \u000A becomes active when some simplification has taken place.  We have seen a few \u000A cases where value <v>t</v> will make a proof fail but <v>:disabled</v> does \u000A not.</p> \u000A \u000A <p>The following example illustrates the rw-cache in action.  You will see a \u000A break during evaluation of the <tt><see topic=\"ACL2____THM\">thm</see></tt> form.  Type <v>:eval</v> and you will \u000A see a failed rewriting attempt.  Type <v>:go</v> to continue, and at the next \u000A break type <v>:eval</v> again.  This time you will see the same failed rewriting \u000A attempt, but this time labeled with a notation saying that the failure was \u000A cached earlier, which indicates that this time the rewriter did not even \u000A attempt to prove the hypothesis of the <see topic=\"ACL2____REWRITE\">rewrite</see> rule <v>f1-&gt;f2</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f1 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f2 (x) t)\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> f1-&gt;f2\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (f1 x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f2 x) t)))\u000A:brr t\u000A:monitor (:rewrite f1-&gt;f2) t\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (f2 a)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> (f2 a))))</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs (see \u000A <see topic=\"ACL2____SET-RW-CACHE-STATE_12\">set-rw-cache-state!</see> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event).</p> \u000A \u000A <p>We also note that rw-cache-state changes may also be caused at the subgoal \u000A level; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>We welcome you to experiment with different rw-cache states.  If the more \u000A aggressive values of <v>t</v> and <v>:disabled</v> cause proofs to fail, then you \u000A can revert to the default of <v>:atom</v> or even turn off the rw-cache using \u000A <v>(<see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see> nil)</v>.  We don't expect users to need a deep knowledge \u000A of the rw-cache in order to do such experiments, but readers interested in \u000A details of the rw-cache implementation are invited to read the ``Essay on \u000A Rw-cache'' in the ACL2 source code.</p>"],

 "ACL2____PROGRAMMING":[["ACL2"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is a parent topic under which we \u000A include documentation topics for built-in functions, macros, and special forms \u000A (see <see topic=\"ACL2____ACL2-BUILT-INS\">ACL2-built-ins</see>) as well as topics for notions important to \u000A programming with ACL2.  If you don't find what you're looking for, see the \u000A Index or see individual topics that may be more directly appropriate; for \u000A example, see <see topic=\"ACL2____EVENTS\">events</see> for top-level event constructors like <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____DEBUGGING\">debugging</see> for utilities that can aid in programming.</p>"],

 "ACL2____PL":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pl foo     ; prints rules that rewrite some call of foo\u000A:pl (<see topic=\"COMMON-LISP_____B2\">+</see> x y) ; prints rules that rewrite (<see topic=\"COMMON-LISP_____B2\">+</see> x y)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PL2\">pl2</see>, which restricts output to rules that you specify for a \u000A given term.</p> \u000A \u000A <p><v>Pl</v> takes one argument, which should be a symbol or a term.</p> \u000A \u000A <p>First suppose that the argument is a symbol.  Then it should be either a \u000A function symbol or else a macro alias for a function symbol (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), which is treated as the corresponding function symbol. \u000A In this case <v>:pl</v> displays rules that apply to terms whose top function \u000A symbol is the one specified, specifically, rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>, <v>:</v><tt><see topic=\"ACL2____ELIM\">elim</see></tt>, and <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt>.</p> \u000A \u000A <p>Otherwise the argument should be a term (in user syntax, so that for \u000A example macros are permitted).  In this case, <v>:pl</v> displays the \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, and <v>:meta</v> rules that \u000A rewrite the specified term, followed by the applicable <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rules.  Each rule is displayed with additional information, \u000A such as the hypotheses that remain after applying some simple techniques to \u000A discharge them that are likely to apply in any context.  Note that for \u000A <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> rules, only those are displayed that meet two conditions: \u000A the application of the metafunction returns a term different from the input \u000A term, and if there is a hypothesis metafunction then it also returns a term. \u000A (A subtlety: In the case of extended metafunctions (see <see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see>), a trivial metafunction context is used for the \u000A application of the metafunction.)</p> \u000A \u000A <p>Note that some rule classes are not handled by <v>:pl</v>.  In particular, if \u000A you want to see all <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> rules, issue the command \u000A <v>:print-clause-processor-rules</v>, and for trusted clause-processors, \u000A <v>(<see topic=\"ACL2____TABLE\">table</see> trusted-clause-processor-table)</v>; see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> and \u000A see <see topic=\"ACL2____DEFINE-TRUSTED-CLAUSE-PROCESSOR\">define-trusted-clause-processor</see>.</p>"],

 "ACL2____GCL":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>See the installation instructions for basic information about \u000A building ACL2 on top of GCL, including information about where to fetch GCL. \u000A Here, we provide some tips that may be useful.</p> \u000A \u000A <p>1. You can place forms to evaluate at start-up into file <v>init.lsp</v> in \u000A the directory where you are starting ACL2 (GCL), or into file \u000A <v>acl2-init.lsp</v> in your home directory.  For example, in order to evaluate \u000A both of the lisp forms mentioned in 2 below, you could put them both into \u000A <v>init.lsp</v> in the current directory or in <v>~/acl2-init.lsp</v> (either way, \u000A without <v>(<see topic=\"ACL2____LP\">lp</see>)</v> or <v>:q</v>):</p> \u000A \u000A <code>(setq si::*optimize-maximum-pages* nil)\u000A(si::allocate 'cons 75000 t)</code> \u000A \u000A <p>Note that if you want to put ACL2 patches in this file, you should precede \u000A them with (in-package \"ACL2\").</p> \u000A \u000A <p>2. Suppose you run out of space, for example with an error like this:</p> \u000A \u000A <code>Error: The storage for CONS is exhausted.\u000A  Currently, 59470 pages are allocated.\u000A  Use ALLOCATE to expand the space.</code> \u000A \u000A <p>The following suggestion from Camm Maguire will minimize the heap size, at \u000A the cost of more garbage collection time.</p> \u000A \u000A <code>:q   ; exit the ACL2 loop\u000A(setq si::*optimize-maximum-pages* nil)\u000A(<see topic=\"ACL2____LP\">lp</see>) ; re-enter the ACL2 loop</code> \u000A \u000A <p>A second thing to try, suggested by several people, is to preallocate more \u000A pages before the run, e.g.:</p> \u000A \u000A <code>:q   ; exit the ACL2 loop\u000A(si::allocate 'cons 75000 t)\u000A(<see topic=\"ACL2____LP\">lp</see>) ; re-enter the ACL2 loop</code> \u000A \u000A <p>Also see <see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> for a suggestion on how to free up \u000A space.</p> \u000A \u000A <p>3. Windows users have seen this error:</p> \u000A \u000A <code>cc1.exe: unrecognized option `-fno-zero-initialized-in-bss'</code> \u000A \u000A <p>Camm Maguire suggests that a solution may be to evaluate the following in \u000A GCL before building ACL2.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> 'compiler)\u000A(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((x `-fno-zero-initialized-in-bss')\u000A       (i (<see topic=\"COMMON-LISP____SEARCH\">search</see> x *cc*)))\u000A        (setq *cc* (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string\u000A                                (<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> *cc* 0 i)\u000A                                (<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> *cc* (<see topic=\"COMMON-LISP_____B2\">+</see> i (<see topic=\"COMMON-LISP____LENGTH\">length</see> x))))))</code> \u000A \u000A <p>4. It is possible to profile using ACL2 built on GCL.  See file \u000A <v>save-gprof.lsp</v> in the ACL2 source directory.</p> \u000A \u000A <p>5. Some versions of GCL may have garbage-collector bugs that, on rare \u000A occasions, cause ACL2 (when built on GCL) to break.  If you run into this, a \u000A solution may be to execute the following:</p> \u000A \u000A <code>:q\u000A(si::sgc-on nil)\u000A(<see topic=\"ACL2____LP\">lp</see>)</code> \u000A \u000A <p>Alternatively, put <v>(si::sgc-on nil)</v> in your <v>~/acl2-init.lsp</v> \u000A file.</p> \u000A \u000A <p>A full regression test and found that this decreased performance by about \u000A 10%.  But even with that, GCL is probably one of the faster Common Lisp \u000A implementations for ACL2 on Linux.  Performance figures may often be found by \u000A following the ``Recent changes'' link on the ACL2 home page.</p> \u000A \u000A <p>6. GCL operations on numbers can sometimes be sped up, perhaps by up to two \u000A orders of magnitude, by suitable <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms (also see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>).  The following example, developed with Warren Hunt and Serita \u000A Nelesen, illustrates the use of such declarations.</p> \u000A \u000A <code>; File iplus.lisp:\u000A; Operations on naturals together with positive infinity (represented as -1).\u000A\u000A; After (<see topic=\"ACL2____LD\">ld</see> &quot;iplus.lisp&quot;), escape to raw Lisp with :q and then eavluate\u000A; (disassemble 'big-test).  You should see lots of arithmetic operations\u000A; in C code, but no calls of C functions CMPmake_fixnum or number_plus.\u000A\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> i-max ()\u000A  (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 (<see topic=\"COMMON-LISP____1-\">1-</see> 28)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> i+ (x y)\u000A  `(<see topic=\"COMMON-LISP____THE\">the</see> (signed-byte 28)\u000A        (<see topic=\"COMMON-LISP____LET\">let</see> ((x ,x)\u000A              (y ,y))\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 28) x y))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A                     (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y 0))\u000A                 -1)\u000A                (t (<see topic=\"COMMON-LISP____LET\">let</see> ((result\u000A                          (<see topic=\"COMMON-LISP____THE\">the</see> (signed-byte 29) (<see topic=\"COMMON-LISP_____B2\">+</see> x y))))\u000A                     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 29) result))\u000A                     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> result (i-max)) -1)\u000A                           (t (<see topic=\"COMMON-LISP____THE\">the</see> (signed-byte 28) result)))))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> imin (x y)\u000A  `(<see topic=\"COMMON-LISP____THE\">the</see> (signed-byte 28)\u000A        (<see topic=\"COMMON-LISP____LET\">let</see> ((x ,x)\u000A              (y ,y))\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 28) x y))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A                 (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> y 0) -1)\u000A                       (t y)))\u000A                ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> y 0)\u000A                 x)\u000A                (t\u000A                 (<see topic=\"COMMON-LISP____THE\">the</see> (signed-byte 28) (<see topic=\"COMMON-LISP____MIN\">min</see> x y)))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> big-test (x y z)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 28) x y z))\u000A  (imin (i+ x y)\u000A        (i+ y (imin x z))))</code>"],

 "ACL2____DEFDOC":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFDOC\">defdoc</see> interp-section\u000A   &quot;:Doc-Section ...&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFDOC\">defdoc</see> name doc-string)</code> \u000A \u000A <p>where <v>name</v> is a symbol or string to be documented and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).  This \u000A event adds the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string for symbol <v>name</v> to the \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> database.  It may also be used to change the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <v>name</v> if <v>name</v> already has <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>. \u000A The difference between this event and <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> is that, unlike <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> (but like <tt><see topic=\"ACL2____TABLE\">table</see></tt>), it does not mark the current <see topic=\"ACL2____HISTORY\">history</see> with the <v>name</v>.  But like <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>, <v>defdoc</v> <see topic=\"ACL2____EVENTS\">events</see> are never considered redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFLABEL\">deflabel</see> for a means of attaching a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string \u000A to a name that marks the current <see topic=\"ACL2____HISTORY\">history</see> with that name.  We now \u000A elaborate further on how <v>defdoc</v> may be useful in place of <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>.</p> \u000A \u000A <p>It is usually sufficient to use <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> when you might be tempted \u000A to use <v>defdoc</v>.  However, unlike <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt>, <v>defdoc</v> does not \u000A mark the current <see topic=\"ACL2____HISTORY\">history</see> with <v>name</v>.  Thus, <v>defdoc</v> is useful \u000A for introducing the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event, for example, several <see topic=\"ACL2____EVENTS\">events</see> before the function or \u000A theory is actually defined.</p> \u000A \u000A <p>For example, suppose you want to define a theory (using <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>). \u000A You need to prove the lemmas in that theory before executing the <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event.  However, it is quite natural to define a <v>:Doc-Section</v> \u000A (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>) whose name is the name of the theory to be defined, \u000A and put the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for that theory's lemmas into that \u000A <v>:Doc-Section</v>.  <v>Defdoc</v> is ideal for this purpose, since it can be \u000A used to introduce the <v>:Doc-Section</v>, followed by the lemmas referring to \u000A that <v>:Doc-Section</v>, and finally concluded with a <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event \u000A of the same name.  If <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> were used instead of <v>defdoc</v>, for \u000A example, then the <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> event would be disallowed because the name \u000A is already in use by the <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> event.</p> \u000A \u000A <p>We also imagine that some users will want to use <v>defdoc</v> to insert the \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for a function under development.  This <v>defdoc</v> event \u000A would be followed by definitions of all the subroutines of that function, \u000A followed in turn by the function definition itself.</p> \u000A \u000A <p>Any time <v>defdoc</v> is used to attach <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> to an \u000A already-documented name, the name must not be attached to a new \u000A <v>:Doc-Section</v>.  We make this requirement as a way of avoiding loops in the \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> tree.  When <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> is redefined, a warning \u000A will be printed to the terminal.</p>"],

 "ACL2____CW":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Example:</p> \u000A \u000A <code>(<see topic=\"ACL2____CW\">cw</see> &quot;The goal is ~p0 and the alist is ~x1.~%&quot;\u000A    (<see topic=\"ACL2____UNTRANSLATE\">untranslate</see> term t nil)\u000A    unify-subst)</code> \u000A \u000A <p>Logically, this expression is equivalent to <v>nil</v>.  However, it has the \u000A effect of first printing to the so-called ``comment window'' the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> \u000A string as indicated.  Thus, <v>cw</v> is like <v>fmt</v> (see <see topic=\"ACL2____FMT\">fmt</see>) except \u000A in three important ways.  First, it is a macro whose calls expand to calls of \u000A a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode function.  Second, it neither takes nor returns the \u000A ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>; logically <v>cw</v> simply returns <v>nil</v>, although it \u000A prints to a <i>comment window</i> that just happens to share the terminal \u000A screen with the standard character output <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>.  Third, its \u000A <v>fmt</v> args are positional references, so that for example</p> \u000A \u000A <code>(<see topic=\"ACL2____CW\">cw</see> &quot;Answers: ~p0 and ~p1&quot; ans1 ans2)</code> \u000A \u000A <p>prints in the same manner as:</p> \u000A \u000A <code>(<see topic=\"ACL2____FMT\">fmt</see> &quot;Answers: ~p0 and ~p1&quot;\u000A     (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 ans1) (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\1 ans2))\u000A     *standard-co* state nil)</code> \u000A \u000A <p>Typically, calls of <v>cw</v> are embedded in <tt><see topic=\"ACL2____PROG2_42\">prog2$</see></tt> forms, e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> ...)\u000A        (<see topic=\"ACL2____MV\">mv</see> a b c))</code> \u000A \u000A <p>which has the side-effect of printing to the comment window and logically \u000A returning <v>(<see topic=\"ACL2____MV\">mv</see> a b c)</v>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____CW\">cw</see> fmt-string arg1 arg2 ... argn)</code> \u000A \u000A <p>where n is between 0 and 9 (inclusive).  The macro uses <tt><see topic=\"ACL2____FMT-TO-COMMENT-WINDOW\">fmt-to-comment-window</see></tt>, passing it the column <v>0</v> and <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> \u000A <v>nil</v>, after assembling the appropriate alist binding the <tt><see topic=\"ACL2____FMT\">fmt</see></tt> vars \u000A #\\0 through #\\9; see <see topic=\"ACL2____FMT\">fmt</see>.  If you want</p> \u000A \u000A <code>(a) more than 10 vars,\u000A(b) vars other than the digit chars,\u000A(c) a different column, or\u000A(d) a different evisc-tuple,</code> \u000A \u000A <p>then call <tt><see topic=\"ACL2____FMT-TO-COMMENT-WINDOW\">fmt-to-comment-window</see></tt> instead.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____CW_12\">cw!</see>, which is useful if you want to be able to read the \u000A printed forms back in.</p> \u000A \u000A <p>Finally, we discuss another way to create formatted output that also avoids \u000A the need to pass in the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  The idea is to use wormholes; see \u000A <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  Below is a function you can write, along with some calls, \u000A providing an illustration of this approach.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-fmt-to-comment-window (str alist)\u000A  (<see topic=\"ACL2____WORMHOLE\">wormhole</see> 'my-fmt-to-comment-window\u000A            '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) whs)\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> str alist)\u000A            '(<see topic=\"ACL2____PPROGN\">pprogn</see>\u000A              (<see topic=\"ACL2____FMS\">fms</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2_____04\">@</see> wormhole-input))\u000A                   (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2_____04\">@</see> wormhole-input))\u000A                   *standard-co*\u000A                   state\u000A                   nil)\u000A              (value :q))\u000A            :ld-verbose nil\u000A            :ld-error-action :return ; harmless return on error\u000A            :ld-prompt nil))\u000A\u000A; A non-erroneous call:\u000A(my-fmt-to-comment-window &quot;Here is ~x0 for your inspection~%&quot;\u000A                          (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 'foo)))\u000A\u000A; An error inside the fmt string (unbound fmt var); note that even\u000A; with the error, the wormhole is exited.\u000A(my-fmt-to-comment-window &quot;Here is ~x1 for your inspection~%&quot;\u000A                          (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 'foo)))\u000A\u000A; A guard violation in the binding; note that even with the error,\u000A; the wormhole is exited.\u000A(my-fmt-to-comment-window &quot;Here is ~x0 for your inspection~%&quot;\u000A                          (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____CAR\">car</see> 'foo))))</code>"],

 "ACL2____TRUE-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>True-listp</v> is the function that checks whether its argument is \u000A a list that ends in, or equals, <v>nil</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>true-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> true-listp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A           (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil)))</code>"],

 "ACL2____PROG2_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____HARD-ERROR\">hard-error</see>, see <see topic=\"ACL2____ILLEGAL\">illegal</see>, and see <see topic=\"ACL2____CW\">cw</see> for \u000A examples of functions to call in the first argument of <v>prog2$</v>.  Also see \u000A <see topic=\"ACL2____PROGN_42\">progn$</see> for an extension of <v>prog2$</v> that handles than two \u000A arguments.</p> \u000A \u000A <p>Semantically, <v>(<see topic=\"ACL2____PROG2_42\">Prog2$</see> x y)</v> equals <v>y</v>; the value of <v>x</v> is \u000A ignored.  However, <v>x</v> is first evaluated for side effect.  Since the ACL2 \u000A <see topic=\"ACL2____PROGRAMMING\">programming</see> language is applicative, there can be no logical impact of \u000A evaluating <v>x</v>.  However, <v>x</v> may involve a call of a function such as \u000A <tt><see topic=\"ACL2____HARD-ERROR\">hard-error</see></tt> or <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt>, which can cause so-called ``hard \u000A errors'', or a call of <tt><see topic=\"ACL2____CW\">cw</see></tt> to perform output.</p> \u000A \u000A <p>Here is a simple, contrived example using <tt><see topic=\"ACL2____HARD-ERROR\">hard-error</see></tt>.  The \u000A intention is to check at run-time that the input is appropriate before calling \u000A function <v>bar</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo-a (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A   (<see topic=\"COMMON-LISP____OR\">or</see> (good-car-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A       (<see topic=\"ACL2____HARD-ERROR\">hard-error</see> 'foo-a\u000A                   &quot;Bad value for x: ~p0&quot;\u000A                   (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 x))))\u000A   (bar x)))</code> \u000A \u000A <p>The following similar function uses <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt> instead of \u000A <v>hard-error</v>.  Since <v>illegal</v> has a guard of <v>nil</v>, <see topic=\"ACL2____GUARD\">guard</see> \u000A verification would guarantee that the call of <v>illegal</v> below will never be \u000A made (at least when guard checking is on; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo-b (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (good-car-p (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A   (<see topic=\"COMMON-LISP____OR\">or</see> (good-car-p (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A       (<see topic=\"ACL2____ILLEGAL\">illegal</see> 'foo-b\u000A                &quot;Bad value for x: ~p0&quot;\u000A                (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 x))))\u000A   (bar x)))</code> \u000A \u000A <p>We conclude with a simple example using <tt><see topic=\"ACL2____CW\">cw</see></tt> from the ACL2 \u000A sources.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> print-terms (terms iff-flg wrld)\u000A\u000A; Print untranslations of the given terms with respect to iff-flg, following\u000A; each with a newline.\u000A\u000A; We use cw instead of the fmt functions because we want to be able to use this\u000A; function in print-type-alist-segments (used in brkpt1), which does not return\u000A; state.\u000A\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> terms)\u000A      terms\u000A    (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A     (<see topic=\"ACL2____CW\">cw</see> &quot;~q0&quot; (<see topic=\"ACL2____UNTRANSLATE\">untranslate</see> (<see topic=\"COMMON-LISP____CAR\">car</see> terms) iff-flg wrld))\u000A     (print-terms (<see topic=\"COMMON-LISP____CDR\">cdr</see> terms) iff-flg wrld))))</code>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-6":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (FOO A) (NATS-BELOW (BAR B)))</code> \u000A \u000A <p>The definition of <v>NATS-BELOW</v> is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nats-below (j)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> j)\u000A      '(0)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> j (nats-below (<see topic=\"COMMON-LISP____-\">-</see> j 1)))))</code> \u000A \u000A <p>Thus, <v>(nats-below 7)</v> is <v>(7 6 5 4 3 2 1 0)</v>.  So when is <v>k</v> a \u000A member of <v>(nats-below j)</v>?</p> \u000A \u000A <p>The weakest version is</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-nats-below-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k)\u000A                (<see topic=\"ACL2____NATP\">natp</see> j)\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))\u000A           (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))))</code> \u000A \u000A <p>But clearly we could improve this to:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-nats-below-weak-better\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k)\u000A                (<see topic=\"ACL2____NATP\">natp</see> j))\u000A           (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))\u000A                (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))))</code> \u000A \u000A <p>or even</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-nats-below-weak-better\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____NATP\">natp</see> j)\u000A           (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))\u000A                (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k)\u000A                     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j)))))</code> \u000A \u000A <p>Clearly though, we'd like to get rid of the <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v> hypothesis and \u000A the neatest plausible version is:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-nats-below-weak-neatest\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> j)\u000A            (<see topic=\"ACL2____NATP\">natp</see> k)\u000A            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))))</code> \u000A \u000A <p>But it is not a theorem!  For example, if <v>j</v> is <v>-1</v> and <v>k</v> is \u000A 0, then the left-hand side above returns <v>t</v>, because <v>(nats-below j)</v> \u000A is <v>(0)</v>, but the right-hand side is <v>nil</v>.</p> \u000A \u000A <p>But this suggests a strategy for dealing with necessary hypotheses, like \u000A <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v>.  We can move them into an <v>IF</v> on the right-hand side! \u000A Something like this might be a useful rewrite rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))\u000A     (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> j)\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))\u000A         ...)).</code> \u000A \u000A <p>We know, from <v>member-nats-below-weak-better</v>, that if <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v> is \u000A true, the <v>member</v> is equivalent to <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))</v>.  So now \u000A consider what we know if <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v> is false.  If we can think of some term \u000A it's equivalent to and that term is simpler than the <v>member</v> expression, \u000A we have a strong rule.</p> \u000A \u000A <p>But by inspection of the definition of <v>nats-below</v>, we see that when \u000A <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v> is false, <v>(nats-below j)</v> is the list <v>(0)</v> because \u000A <v>(<see topic=\"ACL2____ZP\">zp</see> j)</v> is t.  That is, <v>nats-below</v> treats all non-natural arguments \u000A like they were <v>0</v>.  Thus, when <v>(<see topic=\"ACL2____NATP\">natp</see> j)</v> is false, <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> k\u000A (nats-below j))</v> is <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> k '(0))</v>, which is <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> k 0)</v>.</p> \u000A \u000A <p>So the strong version is</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> member-nats-below-strong\u000A   (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> k (nats-below j))\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____NATP\">natp</see> j)\u000A            (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> k)\u000A                 (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> k j))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> k 0))))</code> \u000A \u000A <p>This is a great :<tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rule.  It gets rid of the <v>member</v> and <v>nats-below</v> and introduces \u000A arithmetic.</p> \u000A \u000A <p>This example illustrates the idea of putting an <v>if</v> on the \u000A right-hand-side of the equivalence.  Many users tend to limit themselves to \u000A propositional forms inside <v>iff</v> or to simple expressions inside of \u000A <v>equal</v>.  But it is quite natural to use <v>if</v> to express what the answer \u000A is: if <v>j</v> is a natural, then <v>k</v> is in <v>(nats-below j)</v> precisely if \u000A <v>k</v> is a natural less than or equal to <v>j</v>; if <v>j</v> is not a natural, \u000A then <v>k</v> is in <v>(nats-below j)</v> precisely if <v>k</v> is <v>0</v>.</p> \u000A \u000A <p>Use <v>if</v> to lay out the cases you must consider, if you can think of a \u000A simpler, equivalent expression for every possible case.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____PAND":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> x y)\u000A      (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> y x))\u000A\u000A(<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A       (<see topic=\"ACL2____GRANULARITY\">granularity</see>\u000A        (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) 500)\u000A             (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> y) 500))))\u000A       (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> x y)\u000A       (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> y x))</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> expr)) ; optional granularity declaration\u000A      arg1 ... argN)</code> \u000A \u000A <p>where <v>N &gt;= 0</v> and each <v>argi</v> and <v>expr</v> are arbitrary terms.</p> \u000A \u000A <p><v>Pand</v> evaluates its arguments in parallel.  It returns a Boolean \u000A result: <v>nil</v> if any argument evaluates to <v>nil</v>, else <v>t</v>.  Note \u000A that <v>pand</v> always returns a Boolean result, even though <v>and</v> can \u000A return a non-<v>nil</v> value other than <v>t</v>, namely the value of its last \u000A argument.  (A moment's reflection will make it clear that in order for <tt><see topic=\"ACL2____POR\">por</see></tt> to parallelize efficiently, it needs to return a Boolean value; so \u000A <v>pand</v> returns a Boolean value for consistency with <tt><see topic=\"ACL2____POR\">por</see></tt>.)</p> \u000A \u000A <p>Another difference between <v>pand</v> and <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> is that for a call of \u000A <v>pand</v>, even if an argument evaluates to <v>nil</v>, a subsequent argument \u000A may be evaluated.  Consider the following example (where <v>cw</v> prints a \u000A string; see <see topic=\"ACL2____CW\">cw</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar ()\u000A  (<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 100000) nil) ; evaluates to nil\u000A        (<see topic=\"ACL2____CW\">cw</see> &quot;hello world~%&quot;)))</code> \u000A \u000A <p>When <v>(bar)</v> is evaluated, the above arguments of <v>pand</v> can execute \u000A in parallel, causing ``<v>hello world</v>'' to be printed to the terminal.  If \u000A we had used <v>and</v> rather than <v>pand</v>, then since <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see>\u000A 100000) nil)</v> evaluates to <v>nil</v>, the above call of <tt><see topic=\"ACL2____CW\">cw</see></tt> would be \u000A avoided and no such printing would take place.  Even with <v>pand</v>, such \u000A printing <i>might</i> not take place, depending on resources, timing of thread \u000A creation, and whether or not parallel execution is enabled (see <see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see>).</p> \u000A \u000A <p>Note that unlike the case for <tt><see topic=\"COMMON-LISP____AND\">and</see></tt>, the definition of <v>pand</v> does \u000A not provide <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> as a <see topic=\"ACL2____GUARD\">guard</see> to <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> in the call of \u000A <v>pand</v> below:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'foo)))</code> \u000A \u000A <p>As a result, <see topic=\"ACL2____GUARD\">guard</see> verification will fail for the above definition. \u000A If <v>pand</v> were replaced by <v>and</v>, then <see topic=\"ACL2____GUARD\">guard</see> verification would \u000A succeed.</p> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see> for another example.  Also see <see topic=\"ACL2____PARALLELISM-AT-THE-TOP-LEVEL\">parallelism-at-the-top-level</see> for restrictions on evaluating parallelism \u000A primitives from within the ACL2 top-level loop.  Finally see <see topic=\"ACL2____EARLY-TERMINATION\">early-termination</see> to read how <v>pand</v> can offer more efficiency than <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> by avoiding evaluation of some of its arguments.</p>"],

 "ACL2____NOTE-2-2":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important changes:</p> \u000A \u000A <p>A bug was fixed in the compile command, <v>:comp</v>.  The compiled code \u000A produced by <v>:comp</v> in previous versions could be wildly incorrect because \u000A of a confusion between the printer and the reader regarding what was the \u000A current Lisp <v>*package*</v>.  This bug could manifest itself only if you used \u000A the <v>:comp</v> command to compile previously uncompiled functions while the \u000A current package was different from <v>&quot;ACL2&quot;</v>.  What happened in that \u000A situation depended upon what symbols were imported into your current package. \u000A The most likely behavior is that the compiler would break or complain or the \u000A resulting compiled code would call functions that did not exist.</p> \u000A \u000A <p>There have been no other important changes to the code.</p> \u000A \u000A <p>However, this release contains some useful new books, notably those on the \u000A <v>books</v> subdirectories <v>cli-misc</v> and <v>ihs</v>.  Both have <v>README</v> \u000A files.  The <v>ihs</v> books provide support for integer hardware \u000A specifications.  These books were crucial to Bishop Brock's successful \u000A modeling of the Motorola CAP.  We thank Bishop for producing them and we thank \u000A all those who worked so hard to get these books released.  We highly recommend \u000A the <v>ihs</v> books to those modeling ALUs and other arithmetic components of \u000A microprocessors or programming languages.</p> \u000A \u000A <p>In previous versions of ACL2, the arithmetic books, found on \u000A <v>books/arithmetic/</v>, included the addition of several unproved axioms \u000A stating properties of the rationals that we believed could be derived from our \u000A ``official'' axioms but which we had not mechanically proved.  The axioms were \u000A found in the book <v>rationals-with-axioms.lisp</v>, which was then used in the \u000A uppermost arithmetic books <v>top.lisp</v> and <v>top-with-meta.lisp</v>.  John \u000A Cowles has now provided us with ACL2 proofs of those ``axioms'' and so in this \u000A release you will find both <v>rationals-with-axioms.lisp</v> and \u000A <v>rationals-with-axioms-proved.lisp</v>.  The former is provided for \u000A compatibility's sake.  The latter is identical but contains <v>defthm</v>s where \u000A the former contains <v>defaxiom</v>s.  The top-most books have been rebuilt \u000A using ``<v>-axioms-proved</v>'' book.  Thanks John.</p> \u000A \u000A <p>Less important notes:</p> \u000A \u000A <p>Bishop Brock found a bug in <v>translated-acl2-unwind-protectp4</v>.  Jun \u000A Sawada reported a bug in linear arithmetic that caused us not to prove certain \u000A trivial theorems concluding with <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> i j))</v>.  We have fixed \u000A both.</p> \u000A \u000A <p>We now prohibit definitions that call certain event commands such as \u000A <v>DEFTHM</v> and <v>TABLE</v> because our Common Lisp implementations of them \u000A differ from their ACL2 meanings (so that compiled books can be loaded \u000A correctly and efficiently).</p> \u000A \u000A <p>Minor bugs in the documentation were fixed.</p>"],

 "ACL2____NAME":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples                 Counter-Examples\u000A\u000APRIMEP               91         (<see topic=\"COMMON-LISP____NOT\">not</see> a symbolp)\u000AF-AC-23              :CHK-LIST  (in KEYWORD package)\u000A1AX                  *PACKAGE*  (in the Lisp Package)\u000A|Prime Number|       1E3        (<see topic=\"COMMON-LISP____NOT\">not</see> a symbolp)</code> \u000A \u000A <p>Many different ACL2 entities have names, e.g., functions, macros, \u000A variables, constants, packages, theorems, <see topic=\"ACL2____THEORIES\">theories</see>, etc.  Package names \u000A are strings.  All other names are symbols and may not be in the \u000A <v>&quot;KEYWORD&quot;</v> package.  Moreover, names of functions, macros, constrained \u000A functions, and constants, other than those that are predefined, must not be in \u000A the ``main Lisp package'' (which is (<v>&quot;LISP&quot;</v> or <v>&quot;COMMON-LISP&quot;</v>, \u000A according to whether we are following CLTL1 or CLTL2).  An analogous \u000A restriction on variables is given below.</p> \u000A \u000A <p><v>T</v>, <v>nil</v>, and all names above except those that begin with \u000A ampersand (&amp;) are names of variables or constants.  <v>T</v>, <v>nil</v>, and \u000A those names beginning and ending with star (*) are constants.  All other such \u000A names are variables.</p> \u000A \u000A <p>Note that names that start with ampersand, such as <v>&amp;rest</v>, may be \u000A lambda list keywords and are reserved for such use whether or not they are in \u000A the CLTL constant <v>lambda-list-keywords</v>.  (See pg 82 of CLTL2.)  That is, \u000A these may not be used as variables.  Unlike constants, variables may be in the \u000A main Lisp package as long as they are in the original list of imports from \u000A that package to ACL2, the list \u000A <v>*common-lisp-symbols-from-main-lisp-package*</v>, and do not belong to a list \u000A of symbols that are potentially ``global.''  This latter list is the value of \u000A <v>*common-lisp-specials-and-constants*</v>.</p> \u000A \u000A <p>Our restrictions pertaining to the main Lisp package take into account that \u000A some symbols, e.g., <v>lambda-list-keywords</v> and <v>boole-c2</v>, are \u000A ``special.''  Permitting them to be bound could have harmful effects.  In \u000A addition, the Common Lisp language does not allow certain manipulations with \u000A many symbols of that package.  So, we stay away from them, except for allowing \u000A certain variables as indicated above.</p>"],

 "ACL2____HONS-RESIZE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>General form:</p> \u000A \u000A <code>(<see topic=\"ACL2____HONS-RESIZE\">hons-resize</see> [:str-ht size]\u000A             [:nil-ht size]\u000A             [:cdr-ht size]\u000A             [:cdr-ht-eql size]\u000A             [:addr-ht size]\u000A             [:other-ht size]\u000A             [:sbits size]\u000A             [:fal-ht size]\u000A             [:persist-ht size])</code> \u000A \u000A <p>Example:</p> \u000A \u000A <code>(<see topic=\"ACL2____HONS-RESIZE\">hons-resize</see> :str-ht 100000\u000A             :addr-ht (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 27))</code> \u000A \u000A <p>Logically, <v>hons-resize</v> just returns nil; we leave it enabled and would \u000A think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons-resize</v> can be used to change the sizes of certain \u000A hash tables in the Hons Space.</p> \u000A \u000A <p>All arguments are optional.  The size given to each argument should either \u000A be nil (the default) or a natural number.  A natural number indicates the \u000A newly desired size for the hash table, whereas nil means \"don't resize this \u000A table.\" Any non-natural argument is regarded as nil.</p> \u000A \u000A <p>You may wish to call this function for two reasons.</p> \u000A \u000A <p>1.  Improving Performance by Resizing Up</p> \u000A \u000A <p>Since the hash tables in the Hons Space automatically grow as new objects \u000A are normed, <v>hons-resize</v> is unnecessary.  But automatic growth can be slow \u000A because it is incremental: a particular hash table might be grown dozens of \u000A times over the course of your application.  Using <v>hons-resize</v> to pick a \u000A more appropriate initial size may help to reduce this overhead.</p> \u000A \u000A <p>2.  Reducing Memory Usage by Resizing Down</p> \u000A \u000A <p>Resizing can also be used to shrink the hash tables.  This might possibly \u000A be useful immediately after a <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt> to free up memory taken by \u000A the hash tables themselves.  (Of course, the tables will grow again as you \u000A create new normed objects.)</p> \u000A \u000A <p>Advice for using <v>hons-resize</v>.</p> \u000A \u000A <p>Note that hash tables that are already close to the desired size, or which \u000A have too many elements to fit into the desired size, will not actually be \u000A resized.  This makes resizing relatively \"safe.\"</p> \u000A \u000A <p>Note that the <tt><see topic=\"ACL2____HONS-SUMMARY\">hons-summary</see></tt> function can be used to see how large \u000A and how full your hash tables are.  This may be useful in deciding what sizes \u000A you want to give to <v>hons-resize</v>.</p> \u000A \u000A <p>Note that <v>hons-resize</v> operates by (1) allocating new hash tables, then \u000A (2) copying everything from the old hash table into the new table.  Because of \u000A this, for best performance you should ideally call it when the hash tables \u000A involved are minimally populated, i.e., at the start of your application, or \u000A soon after a <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt>.</p>"],

 "ACL2____FAST-ALISTS":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>The implementation of fast alists is, in many ways, similar to that of ACL2 \u000A arrays.  Logically, <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> is just like <v>acons</v>, and <tt><see topic=\"ACL2____HONS-GET\">hons-get</see></tt> is similar to <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>.  But under the hood, hash tables \u000A are associated with these alists, and, when a certain <b>discipline</b> is \u000A followed, these functions execute with hash table speeds.</p> \u000A \u000A <p>What is this discipline?  Each <v>hons-acons</v> operation \"steals\" the \u000A hash table associated with the alist that is being extended.  Because of this, \u000A one must be very conscious of which object is the most recent extension of an \u000A alist and use that extension exclusively.</p> \u000A \u000A <p>The only penalty for a failure to keep track of the most recent extension \u000A is a loss of execution speed, not of correctness, and perhaps the annoyance of \u000A some <tt><see topic=\"ACL2____SLOW-ALIST-WARNING\">slow-alist-warning</see></tt>s.</p> \u000A \u000A <p>Maintaining discipline may require careful passing of a fast alist up and \u000A down through function calls, as with any single threaded object in an \u000A applicative setting, but there is <b>no syntactic enforcement</b> that insists \u000A you only use the most recent extension of an alist as there is for single \u000A threaded objects (<tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s).  Also, even with perfectly proper code, \u000A discipline can sometimes be lost due to user interrupts and aborts.</p> \u000A \u000A <p>Performance Notes</p> \u000A \u000A <p>See <see topic=\"ACL2____HONS-ACONS\">hons-acons</see> for how the final <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of a fast alist can be \u000A used as a size hint or as a name for reports printed by calling <tt><see topic=\"ACL2____FAST-ALIST-SUMMARY\">fast-alist-summary</see></tt>.</p> \u000A \u000A <p>The keys of fast alists are always <see topic=\"ACL2____NORMED\">normed</see>.  Why?  In Common Lisp, \u000A equal-based hashing is relatively slow, so to allow the use of eql-based hash \u000A tables, <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> and <tt><see topic=\"ACL2____HONS-GET\">hons-get</see></tt> always <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> the \u000A keys involved.</p> \u000A \u000A <p>Since alist keys are frequently atoms, this norming activity may often be \u000A so fast that you do not need to think about it.  But if you are going to use \u000A large structures as the keys for your fast alist, this overhead can be \u000A significant, and you may want to ensure that your keys are normed ahead of \u000A time.</p> \u000A \u000A <p>There is <b>no automatic mechanism</b> for reclaiming the hash tables that \u000A are associated with alists.  Because of this, to avoid memory leaks, you \u000A should call <tt><see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see></tt> to remove the hash table associated with \u000A alists that will no longer be used.</p>"],

 "ACL2____FAILURE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>When ACL2 gives up it does not mean that the submitted conjecture \u000A is invalid, even if the last formula ACL2 printed in its proof attempt is \u000A manifestly false.  Since ACL2 sometimes <see topic=\"ACL2____GENERALIZE\">generalize</see>s the goal being \u000A proved, it is possible it adopted an invalid subgoal as a legitimate (but \u000A doomed) strategy for proving a valid goal.  Nevertheless, conjectures \u000A submitted to ACL2 are often invalid and the proof attempt often leads the \u000A careful reader to the realization that a hypothesis has been omitted or that \u000A some special case has been forgotten.  It is good practice to ask yourself, \u000A when you see a proof attempt fail, whether the conjecture submitted is \u000A actually a theorem.</p> \u000A \u000A <p>If you think the conjecture is a theorem, then you must figure out from \u000A ACL2's output what you know that ACL2 doesn't about the functions in the \u000A conjecture and how to impart that knowledge to ACL2 in the form of rules.  The \u000A ``key checkpoint'' information printed at the end of the summary provides a \u000A fine place to start.  See <see topic=\"ACL2____THE-METHOD\">the-method</see> for a general discussion of how to \u000A prove theorems with ACL2, and see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> \u000A for a more detailed tutorial.  Also see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> for discussion of \u000A key checkpoints and an abbreviated output mode that focuses attention on them. \u000A You may find it most useful to start by focusing on key checkpoints that are \u000A not under a proof by induction, if any, both because these are more likely to \u000A suggest useful lemmas and because they are more likely to be theorems; for \u000A example, generalization may have occurred before a proof by induction has \u000A begun.  If you need more information than is provided by the key checkpoints \u000A &mdash; although this should rarely be necessary &mdash; then you can look at \u000A the full proof, perhaps with the aid of certain utilities: see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>, see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>, and see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>.</p> \u000A \u000A <p>For information on a tool to help debug failures of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> events, as well as <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> failures, see <see topic=\"ACL2____REDO-FLAT\">redo-flat</see>.</p> \u000A \u000A <p>Again, see <see topic=\"ACL2____THE-METHOD\">the-method</see> for a general discussion of how to prove \u000A theorems with ACL2, and see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for a \u000A more detailed tutorial.  See also the book ``Computer-Aided Reasoning: An \u000A Approach'' (Kaufmann, Manolios, Moore), as well as the discussion of how to \u000A read Nqthm proofs and how to use Nqthm rules in ``A Computational Logic \u000A Handbook'' by Boyer and Moore (Academic Press, 1988).</p> \u000A \u000A <p>If the failure occurred during a forcing round, see <see topic=\"ACL2____FAILED-FORCING\">failed-forcing</see>.</p>"],

 "ACL2____THEORY-FUNCTIONS":[["Theories"],"ACL2 Sources","ACL2","<code>Example Calls of Theory Functions:\u000A(<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)\u000A(<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> th1 th2)\u000A(<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> th1 th2)</code> \u000A \u000A <p>The theory functions are documented individually:</p> \u000A \u000A <p>The functions (actually, macros) mentioned above are convenient ways to \u000A produce <see topic=\"ACL2____THEORIES\">theories</see>.  (See <see topic=\"ACL2____THEORIES\">theories</see>.) Some, like <tt><see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see></tt>, take a logical name (see <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>) as an \u000A argument and return the relevant theory as of the time that name was \u000A introduced.  Others, like <tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt>, take two <see topic=\"ACL2____THEORIES\">theories</see> and \u000A produce a new one.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see> for a caution about the use of \u000A logical names in theory expressions.</p> \u000A \u000A <p>Theory expressions are generally composed of applications of theory \u000A functions.  Formally, theory expressions are expressions that involve, at \u000A most, the free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt> and that when evaluated with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the current ACL2 world (see <see topic=\"ACL2____WORLD\">world</see>) return <see topic=\"ACL2____THEORIES\">theories</see>.  The ``theory functions'' are actually macros that expand into \u000A forms that involve the free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  Thus, for example \u000A <v>(<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)</v> actually expands to <v>(universal-theory-fn\u000A :here world)</v> and when that form is evaluated with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the \u000A current ACL2 <see topic=\"ACL2____WORLD\">world</see>, <v>universal-theory-fn</v> scans the ACL2 property \u000A lists and computes the current universal theory.  Because the theory functions \u000A all implicitly use <tt><see topic=\"ACL2____WORLD\">world</see></tt>, the variable does not generally appear in \u000A anything the user types.</p>"],

 "ACL2____GUARD-QUICK-REFERENCE":[["Guard"],"ACL2 Sources","ACL2","<p>For a careful introduction to guards, see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p><b>I. GUARD CHECKING DURING EXECUTION</b></p> \u000A \u000A <p><i>Effect</i></p> \u000A \u000A <p>Guards on definitions are checked at execution time (except for guards on \u000A subsidiary calls of recursive or mutually recursive functions).</p> \u000A \u000A <p><i>When does it happen</i></p> \u000A \u000A <p>By default, guards are checked for all forms submitted at the top \u000A level.</p> \u000A \u000A <p><i>To disable</i><br></br> \u000A \u000A <v>:set-guard-checking nil</v> ; skip raw Lisp if there is a guard violation \u000A <v>:set-guard-checking :none</v> ; skip guard checking entirely</p> \u000A \u000A <p><i>To (re-)enable</i><br></br> \u000A \u000A <v>:set-guard-checking t</v></p> \u000A \u000A <p>See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> for more options.</p> \u000A \u000A <p><b>II. GUARD VERIFICATION</b></p> \u000A \u000A <p><i>Effect</i></p> \u000A \u000A <p>A proof is attempted of the obligations arising from the guards of \u000A subsidiary functions in a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, or <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> \u000A event.  In the case of a <v>defun</v>, the guard itself is also verified (under \u000A an implicit guard of <v>t</v>).</p> \u000A \u000A <p><i>When does it happen</i></p> \u000A \u000A <p>Only names of defined functions, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>s, and <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>s are \u000A subject to guard verification.  Guard verification may occur when functions \u000A are defined (using <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>), but it requires an explicit call of <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> in order to verify guards for <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>s and <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>s.  Constrained functions (see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>) may not have their \u000A guards verified.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo ...)</v><br></br> \u000A \u000A causes guard verification for the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, or <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt> named by <v>foo</v>, if it has not already been successfully done. \u000A The default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) must be \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, or else this event is ignored.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo ...)</v><br></br> \u000A \u000A causes guard verification of <v>foo</v> if and only if the following two \u000A conditions are both met.  First, foo is processed in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> \u000A mode (either by setting mode <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> globally, or by including \u000A <v>:mode :logic</v> in the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration).  Second, at least one of \u000A the following sub-conditions is met.  Also see <see topic=\"ACL2____XARGS\">xargs</see>, and see <see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> for how to change this behavior.</p> \u000A \u000A <blockquote><p>1. The <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration specifies a <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt>.</p> \u000A \u000A <p>2. There is at least one <v>type</v> declaration (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>).</p> \u000A \u000A <p>3. The <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration specifies <v>:</v><tt><see topic=\"ACL2____STOBJ\">stobj</see></tt><v>s</v>.</p> \u000A \u000A <p>4. The <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> declaration specifies <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> \u000A <v>t</v>.</p></blockquote> \u000A \u000A <p><v>(<see topic=\"ACL2____VERIFY-TERMINATION\">verify-termination</see> foo ...)</v><br></br> \u000A \u000A causes guard verification of <v>foo</v> if <v>foo</v> is a function currently \u000A defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and the criteria for guard verification \u000A of a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form are met, as discussed above.  The default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> \u000A (see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>) must be <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, or else this \u000A event is ignored.</p>"],

 "ACL2____EXAMPLE-INDUCTIONS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Here are some pages illustrating various induction schemes \u000A suggested by recursive functions.</p> \u000A \u000A <p><b>Classical Induction on Natural Numbers</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-NAT-RECURSION\">example-induction-scheme-nat-recursion</see>.</p> \u000A \u000A <p><b>Induction Preserving Even/Odd Parity</b> or<br></br> \u000A \u000A <b>Induction Downwards by 2</b> or <br></br> \u000A \u000A <b>Induction with Multiple Base Cases</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-DOWN-BY-2\">example-induction-scheme-down-by-2</see> for an induction in which the induction \u000A hypothesis decreases the induction variable by an amount other than 1.  This \u000A illustrates that the induction hypothesis can be about whatever term or terms \u000A are needed to explain how the formula recurs.  The example also illustrates an \u000A induction with more than one Base Case.</p> \u000A \u000A <p><b>Classical Induction on Lists</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-ON-LISTS\">example-induction-scheme-on-lists</see> for an induction over linear lists, in \u000A which we inductively assume the conjecture for <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> and prove it for \u000A <v>x</v>.  It doesn't matter whether the list is <v>nil</v>-terminated or not; the \u000A Base Case addresses all the possibilities.</p> \u000A \u000A <p><b>Classical Induction on Binary (Cons) Trees</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-BINARY-TREES\">example-induction-scheme-binary-trees</see> for an induction over the simplest form \u000A of binary tree.  Here the Induction Step provides two hypotheses, one about \u000A the left subtree and one about the right subtree.</p> \u000A \u000A <p><b>Induction on Several Variables</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-ON-SEVERAL-VARIABLES\">example-induction-scheme-on-several-variables</see> for an induction in which \u000A several variables participate in the case analysis and induction \u000A hypotheses.</p> \u000A \u000A <p><b>Induction Upwards</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-UPWARDS\">example-induction-scheme-upwards</see> for \u000A an induction scheme in which the induction hypothesis is about something \u000A ``bigger than'' the induction conclusion.  This illustrates that the sense in \u000A which the hypothesis is about something ``smaller'' than the conclusion is \u000A determined by a measure of all the variables, not the magnitude or extent of \u000A some single variable.</p> \u000A \u000A <p><b>Induction with Auxiliary Variables</b> or<br></br> \u000A \u000A <b>Induction with Accumulators</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-WITH-ACCUMULATORS\">example-induction-scheme-with-accumulators</see> for an induction scheme in which \u000A one variable ``gets smaller'' but another is completely arbitrary.  Such \u000A schemes are common when dealing with tail-recursive functions that accumulate \u000A partial results in auxiliary variables.  This example also shows how to \u000A provide several arbitrary terms in a non-inductive variable of a scheme.</p> \u000A \u000A <p><b>Induction with Multiple Induction Steps</b>: see <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-WITH-MULTIPLE-INDUCTION-STEPS\">example-induction-scheme-with-multiple-induction-steps</see> for an induction in \u000A which we make different inductive hypotheses depending on which case we're in. \u000A This example also illustrates the handling of auxiliary variables or \u000A accumulators.</p>"],

 "ACL2____CASE-SPLIT":[["Miscellaneous","Rewrite"],"ACL2 Sources","ACL2","<p><v>Case-split</v> is an variant of <tt><see topic=\"ACL2____FORCE\">force</see></tt>, which has similar \u000A special treatment in hypotheses of rules for the same <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> as \u000A for <v>force</v> (see <see topic=\"ACL2____FORCE\">force</see>).  This treatment takes place for rule \u000A classes <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, <v>:</v><tt><see topic=\"ACL2____META\">meta</see></tt> (actually in \u000A that case, the result of evaluating the hypothesis metafunction call), and \u000A <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt>.</p> \u000A \u000A <p>When a hypothesis of a conditional rule (of one of the classes listed \u000A above) has the form <v>(<see topic=\"ACL2____CASE-SPLIT\">case-split</see> hyp)</v> it is logically equivalent to \u000A <v>hyp</v>.  However it affects the application of the rule generated as \u000A follows: if ACL2 attempts to apply the rule but cannot establish that the \u000A required instance of <v>hyp</v> holds in the current context, it considers the \u000A hypothesis true anyhow, but (assuming all hypotheses are seen to be true and \u000A the rule is applied) creates a subgoal in which that instance of <v>hyp</v> is \u000A assumed false.  (There are exceptions, noted below.)</p> \u000A \u000A <p>For example, given the rule</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> p1-&gt;p2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (p1 x))\u000A           (p2 x)))</code> \u000A \u000A <p>then an attempt to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (p3 x) (p2 (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</code> \u000A \u000A <p>can give rise to a single subgoal:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (P1 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))) (P3 X))\u000A         (P2 (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))).</code> \u000A \u000A <p>Unlike <tt><see topic=\"ACL2____FORCE\">force</see></tt>, <v>case-split</v> does not delay the ``false case'' to \u000A a forcing round but tackles it more or less immediately.</p> \u000A \u000A <p>The special ``split'' treatment of <v>case-split</v> can be disabled by \u000A disabling forcing: see <see topic=\"ACL2____FORCE\">force</see> for a discussion of disabling forcing, and \u000A also see <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>.  Finally, we should mention that the rewriter \u000A is never willing to split when there is an <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> term present in the goal \u000A being simplified.  Since <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> terms and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> terms are merely \u000A abbreviations for <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> terms, they also prevent splitting.  Note that \u000A <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> terms are ultimately eliminated using the ordinary flow of the \u000A proof (but see <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see>), so <v>case-split</v> will \u000A ultimately function as intended.</p> \u000A \u000A <p>When in the proof checker, <v>case-split</v> behaves like <v>force</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>case-split</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> case-split (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "ACL2____BRR_04":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____BRR_04\">brr@</see> :target)      ; the term being rewritten\u000A(<see topic=\"ACL2____BRR_04\">brr@</see> :unify-subst) ; the unifying substitution\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____BRR_04\">brr@</see> :symbol)</code> \u000A \u000A <p>where <v>:symbol</v> is one of the keywords displayed below.  This utility \u000A may be most useful for system hackers; see <see topic=\"ACL2____BRR-COMMANDS\">brr-commands</see> for queries \u000A that are more at a user level.  In particular, keywords marked below with \u000A <v>*</v> probably require an implementor's knowledge of the system to use \u000A effectively.  They are supported but not well documented.  More is said on \u000A this topic following the table.</p> \u000A \u000A <code>:symbol             (<see topic=\"ACL2____BRR_04\">brr@</see> :symbol)\u000A-------             ---------------------\u000A\u000A:target             the term to be rewritten.  This term is an\u000A                    instantiation of the left-hand side of the\u000A                    conclusion of the rewrite-rule being broken.\u000A                    This term is in translated form!  Thus, if\u000A                    you are expecting (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil) -- and your\u000A                    expectation is almost right -- you will see\u000A                    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 'nil); similarly, instead of (<see topic=\"COMMON-LISP____CADR\">cadr</see> a)\u000A                    you will see (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> a)).  In translated\u000A                    forms, all constants are quoted (even nil, t,\u000A                    strings and numbers) and all macros are\u000A                    expanded.\u000A\u000A:unify-subst        the substitution that, when applied to :target,\u000A                    produces the left-hand side of the rule being\u000A                    broken.  This substitution is an alist pairing\u000A                    variable symbols to translated (!) terms.\u000A\u000A:wonp               t or nil indicating whether the rune was\u000A                    successfully applied.  (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp) returns\u000A                    nil if evaluated before :EVALing the rule.\u000A\u000A:rewritten-rhs      the result of successfully applying the rewrite\u000A                    rule or else nil if (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp) is nil.  The\u000A                    result of successfully applying the rule is\u000A                    always a translated (!) term and is never nil.\u000A\u000A:poly-list          the result of successfully applying the linear\u000A                    rule or else nil if (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp) is nil.  This\u000A                    result represents the list of polynomials\u000A                    produced by the rule application.  The leading\u000A                    term of each polynomial is enclosed in an extra\u000A                    set of parentheses.\u000A\u000A:failure-reason     some non-nil lisp object indicating why the rule\u000A                    was not applied or else nil.  Before the rule is\u000A                    :EVALed, (<see topic=\"ACL2____BRR_04\">brr@</see> :failure-reason) is nil.  After\u000A                    :EVALing the rule, (<see topic=\"ACL2____BRR_04\">brr@</see> :failure-reason) is nil\u000A                    if (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp) is t.  Rather than document the\u000A                    various non-nil objects returned as the failure\u000A                    reason, we encourage you simply to evaluate\u000A                    (<see topic=\"ACL2____BRR_04\">brr@</see> :failure-reason) in the contexts of interest.\u000A                    Alternatively, study the ACL2 function tilde-@-\u000A                    failure-reason-phrase.\u000A\u000A:lemma           *  the rewrite rule being broken.  For example,\u000A                    (<see topic=\"ACL2____ACCESS\">access</see> rewrite-rule (<see topic=\"ACL2____BRR_04\">brr@</see> :lemma) :lhs) will\u000A                    return the left-hand side of the conclusion\u000A                    of the rule.\u000A\u000A:type-alist      *  a display of the type-alist governing :target.\u000A                    Elements on the displayed list are of the form\u000A                    (<see topic=\"ACL2____TERM\">term</see> type), where term is a term and type\u000A                    describes information about term assumed to hold in\u000A                    the current context.  (See also the documentation for\u000A                    type-alist.)  The type-alist may be used to determine\u000A                    the current assumptions, e.g., whether A is a CONSP.\u000A\u000A:ancestors       *  a stack of frames indicating the backchain history\u000A                    of the current context.  The theorem prover is in\u000A                    the process of trying to establish each hypothesis\u000A                    in this stack.  Thus, the negation of each hypothesis\u000A                    can be assumed false.  Each frame also records the\u000A                    rules on behalf of which this backchaining is being\u000A                    done and the weight (function symbol count) of the\u000A                    hypothesis.  All three items are involved in the\u000A                    heuristic for preventing infinite backchaining.\u000A                    Exception:  Some frames are ``binding hypotheses''\u000A                    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> var term) or (equiv var (<see topic=\"ACL2____DOUBLE-REWRITE\">double-rewrite</see> term))\u000A                    that bind variable var to the result of rewriting\u000A                    term.  The ACL2 source code has a definition\u000A                    (<see topic=\"ACL2____DEFREC\">defrec</see> ancestor ...) that may provide some relevant\u000A                    insight.\u000A\u000A:initial-ttree   *  the initial and (after :EVAL) final tag trees,\u000A:final-ttree        respectively.  (Tag trees are low-level data structures\u000A                    that store lemmas used and other information, as\u000A                    documented in topic TTREE.)\u000A\u000A:gstack          *  the current goal stack.  The gstack is maintained\u000A                    by rewrite and is the data structure printed as the\u000A                    current ``path.''  Thus, any information derivable\u000A                    from the :path brr command is derivable from gstack.\u000A                    For example, from gstack one might determine that\u000A                    the current term is the second hypothesis of a\u000A                    certain rewrite rule.</code> \u000A \u000A <p>In general <v>brr@-expressions</v> are used in break conditions, the \u000A expressions that determine whether interactive breaks occur when <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see>s are applied.  See <see topic=\"ACL2____MONITOR\">monitor</see>.  For example, you \u000A might want to break only those attempts in which one particular term is being \u000A rewritten or only those attempts in which the binding for the variable <v>a</v> \u000A is known to be a <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt>.  Such conditions can be expressed using ACL2 \u000A system functions and the information provided by <v>brr@</v>.  Unfortunately, \u000A digging some of this information out of the internal data structures may be \u000A awkward or may, at least, require intimate knowledge of the system functions. \u000A But since conditional expressions may employ arbitrary functions and macros, \u000A we anticipate that a set of convenient primitives will gradually evolve within \u000A the ACL2 community.  It is to encourage this evolution that <v>brr@</v> provides \u000A access to the <v>*</v>'d data.</p>"],

 "ACL2____WITH-SERIALIZE-CHARACTER":[["Serialize"],"ACL2 Sources","ACL2","<p>This documentation topic relates to an experimental extension of \u000A ACL2 that supports <tt><see topic=\"ACL2____HONS\">hons</see></tt>, memoization, and fast alists.  See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  See <see topic=\"ACL2____SERIALIZE\">serialize</see> for a discussion of \u000A ``serialization'' routines, contributed by Jared Davis for saving ACL2 objects \u000A in files for later loading.</p> \u000A \u000A <p>The expression <v>(<see topic=\"ACL2____WITH-SERIALIZE-CHARACTER\">with-serialize-character</see> char form)</v> evaluates to the \u000A value of <v>form</v>, but with the serialize-character of the <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A assigned to <v>char</v>, which should be one of <v>nil</v>, <v>#\\Y</v>, or \u000A <v>#\\Z</v>.  We describe the effect of that assignment below.  But note that if \u000A you are doing this because of one or more specific calls of \u000A <v>print-object$</v>, such as <v>(<see topic=\"ACL2____PRINT-OBJECT_42\">print-object$</see> x channel state)</v>, then you may \u000A wish instead to evaluate <v>(print-object$-ser x serialize-character channel\u000A state)</v>, in which case you will not need to use \u000A <v>with-serialize-character</v>.  (Note however that <v>serialize-character</v> is \u000A treated as <v>nil</v> for other than a HONS version.)</p> \u000A \u000A <code>General forms:\u000A(<see topic=\"ACL2____WITH-SERIALIZE-CHARACTER\">with-serialize-character</see> nil form)\u000A(<see topic=\"ACL2____WITH-SERIALIZE-CHARACTER\">with-serialize-character</see> #Y form)\u000A(<see topic=\"ACL2____WITH-SERIALIZE-CHARACTER\">with-serialize-character</see> #Z form)</code> \u000A \u000A <p>where <v>form</v> should evaluate to an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>).</p> \u000A \u000A <p>Note that if you prefer to obtain the same behavior (as described below) \u000A globally, rather than only within the scope of <v>with-serialize-character</v>, \u000A then use <v>set-serialize-character</v> in a corresponding manner:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-SERIALIZE-CHARACTER\">set-serialize-character</see> nil state)\u000A(<see topic=\"ACL2____SET-SERIALIZE-CHARACTER\">set-serialize-character</see> #\\Y state)\u000A(<see topic=\"ACL2____SET-SERIALIZE-CHARACTER\">set-serialize-character</see> #\\Z state)</code> \u000A \u000A <p>In each case above, calls of <v>print-object$</v> (see <see topic=\"ACL2____IO\">io</see>) in \u000A <v>form</v> will produce an object that can be read by the HONS version of ACL2. \u000A In the first case, that object is printed as one might expect at the terminal, \u000A as an ordinary Lisp s-expression.  But in the other cases, the object is \u000A printed by first laying down either <v>#Y</v> or <v>#Z</v> (respectively) and then \u000A calling <tt><see topic=\"ACL2____SERIALIZE-WRITE\">serialize-write</see></tt> (or more precisely, the underlying function \u000A called by <v>serialize-write</v> that prints to a stream).</p> \u000A \u000A <p>Consider what happens when the ACL2 reader encounters an object produced as \u000A described above (in the <v>#Y</v> or <v>#Z</v> case).  When the object was \u000A written, information was recorded on whether that object was a <see topic=\"ACL2____HONS\">hons</see>. \u000A In the case of <v>#Z</v>, the object will be read as a hons if and only if it \u000A was originally written as a hons.  But in the case of <v>#Y</v>, it will never \u000A be read as a hons.  Thus, <v>#Y</v> and <v>#Z</v> will behave the same if the \u000A original written object was not a hons, creating an object that is not a hons. \u000A For an equivalent explanation and a bit more discussion, see <see topic=\"ACL2____SERIALIZE-READ\">serialize-read</see>, in particular the discussion of the hons-mode.  The value \u000A <v>:smart</v> described there corresponds to <v>#Z</v>, while <v>:never</v> \u000A corresponds to <v>#Y</v>.</p>"],

 "ACL2____TUTORIAL2-EIGHTS-PROBLEM":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>This example was written almost entirely by Bill Young of \u000A Computational Logic, Inc.</p> \u000A \u000A <p>This simple example was brought to our attention as one that Paul Jackson \u000A has solved using the NuPrl system.  The challenge is to prove the theorem:</p> \u000A \u000A <code>for all n &gt; 7, there exist naturals i and j such that: n = 3i + 5j.</code> \u000A \u000A <p>In ACL2, we could phrase this theorem using quantification.  However we \u000A will start with a constructive approach, i.e., we will show that values of \u000A <v>i</v> and <v>j</v> exist by writing a function that will construct such values \u000A for given <v>n</v>.  Suppose we had a function <v>(split n)</v> that returns an \u000A appropriate pair <v>(i . j)</v>.  Our theorem would be as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> split-splits\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((i (<see topic=\"COMMON-LISP____CAR\">car</see> (split n)))\u000A        (j (<see topic=\"COMMON-LISP____CDR\">cdr</see> (split n))))\u000A    (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                  (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 7 n))\u000A             (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                  (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                  (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                  (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 j)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 3 i) (<see topic=\"COMMON-LISP_____A2\">*</see> 5 j))\u000A                         n)))))</code> \u000A \u000A <p>That is, assuming that <v>n</v> is a natural number greater than 7, <v>(split\u000A n)</v> returns values <v>i</v> and <v>j</v> that are in the appropriate relation to \u000A <v>n</v>.</p> \u000A \u000A <p>Let's look at a few cases:</p> \u000A \u000A <code> 8 = 3x1 + 5x1;    11 = 3x2 + 5x1;     14 = 3x3 + 5x1;   ...\u000A 9 = 3x3 + 5x0;    12 = 3x4 + 5x0;     15 = 3x5 + 5x0;   ...\u000A10 = 3x0 + 5x2;    13 = 3x1 + 5x2;     16 = 3x2 + 5x2;   ...</code> \u000A \u000A <p>Maybe you will have observed a pattern here; any natural number larger than \u000A 10 can be obtained by adding some multiple of 3 to 8, 9, or 10.  This gives us \u000A the clue to constructing a proof.  It is clear that we can write split as \u000A follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bump-i (x)\u000A  ;; Bump the i component of the pair\u000A  ;; (i . j) by 1.\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> split (n)\u000A  ;; Find a pair (i . j) such that\u000A  ;; n = 3i + 5j.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> n) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n 8))\u000A      nil ;; any value is really reasonable here\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 8)\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 1)\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 9)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 0)\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 10)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> 0 2)\u000A          (bump-i (split (<see topic=\"COMMON-LISP____-\">-</see> n 3))))))))</code> \u000A \u000A <p>Notice that we explicitly compute the values of <v>i</v> and <v>j</v> for the \u000A cases of 8, 9, and 10, and for the degenerate case when <v>n</v> is not a \u000A natural or is less than 8.  For all naturals greater than <v>n</v>, we decrement \u000A <v>n</v> by 3 and bump the number of 3's (the value of i) by 1.  We know that \u000A the recursion must terminate because any integer value greater than 10 can \u000A eventually be reduced to 8, 9, or 10 by successively subtracting 3.</p> \u000A \u000A <p>Let's try it on some examples:</p> \u000A \u000A <code>ACL2 !&gt;(split 28)\u000A(6 . 2)\u000A\u000AACL2 !&gt;(split 45)\u000A(15 . 0)\u000A\u000AACL2 !&gt;(split 335)\u000A(110 . 1)</code> \u000A \u000A <p>Finally, we submit our theorem <v>split-splits</v>, and the proof succeeds. \u000A In this case, the prover is ``smart'' enough to induct according to the \u000A pattern indicated by the function split.</p> \u000A \u000A <p>For completeness, we'll note that we can state and prove a quantified \u000A version of this theorem.  We introduce the notion <v>split-able</v> to mean that \u000A appropriate <v>i</v> and <v>j</v> exist for <v>n</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> split-able (n)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> (i j)\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 3 i) (<see topic=\"COMMON-LISP_____A2\">*</see> 5 j)))))</code> \u000A \u000A <p>Then our theorem is given below.  Notice that we prove it by observing that \u000A our previous function <v>split</v> delivers just such an <v>i</v> and <v>j</v> (as \u000A we proved above).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> split-splits2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 7 n))\u000A           (split-able n))\u000A  :hints ((&quot;Goal&quot; :use (:instance split-able-suff\u000A                                  (i (<see topic=\"COMMON-LISP____CAR\">car</see> (split n)))\u000A                                  (j (<see topic=\"COMMON-LISP____CDR\">cdr</see> (split n)))))))</code> \u000A \u000A <p>Unfortunately, understanding the mechanics of the proof requires knowing \u000A something about the way <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> works.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> or see \u000A <see topic=\"ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE\">Tutorial4-Defun-Sk-Example</see> for more on that subject.</p>"],

 "ACL2____STARTUP":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>When you start up ACL2, you'll probably find yourself inside the \u000A ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop, as indicated by the following <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <code>ACL2 !&gt;</code> \u000A \u000A <p>If not, you should type <v>(<see topic=\"ACL2____LP\">LP</see>)</v>.  See <see topic=\"ACL2____LP\">lp</see>, which has a lot more \u000A information about the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop.</p> \u000A \u000A <p>You should now be in ACL2.  The current ``<see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>'' is \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>; the other mode is <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, which would \u000A cause the letter <v>p</v> to be printed in the <see topic=\"ACL2____PROMPT\">prompt</see>.  <v>:</v><tt><see topic=\"ACL2____LOGIC\">Logic</see></tt> means that any function we define is not only executable but also is \u000A axiomatically defined in the ACL2 logic.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> and see <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>.  For example we can define a function <v>my-cons</v> as \u000A follows.  (You may find it useful to start up ACL2 and submit this and other \u000A <see topic=\"ACL2____COMMAND\">command</see>s below to the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop, as we won't include \u000A output below.)</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-cons (x y) (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))</code> \u000A \u000A <p>An easy theorem may then be proved: the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of <v>(my-cons a b)</v> \u000A is A.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFTHM\">defthm</see> car-my-cons (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (my-cons a b)) a))</code> \u000A \u000A <p>You can place raw Lisp forms to evaluate at start-up into file \u000A <v>acl2-init.lsp</v> in your home directory, except on Windows systems.  For \u000A example, if you put the following into <v>acl2-init.lsp</v>, then ACL2 will \u000A print \"HI\" when it starts up.</p> \u000A \u000A <code>(print &quot;HI&quot;)</code> \u000A \u000A <p>But be careful; all bets are off when you submit forms to raw Lisp, so this \u000A capability should only be used when you are hacking or when you are setting \u000A some Lisp parameters (e.g., <v>(setq si::*notify-gbc* nil)</v> to turn off \u000A garbage collection notices in GCL).</p> \u000A \u000A <p>Notice that unlike Nqthm, the theorem <see topic=\"ACL2____COMMAND\">command</see> is <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> \u000A rather than <v>prove-lemma</v>.  See <see topic=\"ACL2____DEFTHM\">defthm</see>, which explains (among other \u000A things) that the default is to turn theorems into <see topic=\"ACL2____REWRITE\">rewrite</see> rules.</p> \u000A \u000A <p>Various keyword commands are available to query the ACL2 ``<see topic=\"ACL2____WORLD\">world</see>'', \u000A or database.  For example, we may view the definition of <v>my-cons</v> by \u000A invoking a command to print <see topic=\"ACL2____EVENTS\">events</see>, as follows.</p> \u000A \u000A <code>ACL2 !&gt;:pe my-cons</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PE\">pe</see>.  We may also view all the lemmas that <see topic=\"ACL2____REWRITE\">rewrite</see> \u000A <see topic=\"ACL2____TERM\">term</see>s whose top function symbol is <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> by using the following \u000A command, whose output will refer to the lemma <v>car-my-cons</v> proved \u000A above.</p> \u000A \u000A <code>ACL2 !&gt;:pl car</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PL\">pl</see>.  Finally, we may print all the <see topic=\"ACL2____COMMAND\">command</see>s back \u000A through the initial <see topic=\"ACL2____WORLD\">world</see> as follows.</p> \u000A \u000A <code>ACL2 !&gt;:pbt 0</code> \u000A \u000A <p>See <see topic=\"ACL2____HISTORY\">history</see> for a list of commands, including these, for viewing the \u000A current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>Continue with the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <see topic=\"ACL2____ANNOTATED-ACL2-SCRIPTS\">annotated-ACL2-scripts</see> to \u000A see a simple but illustrative example in the use of ACL2 for reasoning about \u000A functions.</p>"],

 "ACL2____DEFTHEORY-STATIC":[["Events","Theories"],"ACL2 Sources","ACL2","<p>This macro provides a variant of <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>, such that the \u000A resulting theory is the same at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> time as it was at <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> time.</p> \u000A \u000A <p>We assume that the reader is familiar with <see topic=\"ACL2____THEORIES\">theories</see>; see <see topic=\"ACL2____DEFTHEORY\">deftheory</see>.  We begin here by illustrating how <v>deftheory-static</v> differs \u000A from <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>.  Suppose for example that the following events are the \u000A first two events in a book, where that book is certified in the initial ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see> (see <see topic=\"ACL2____GROUND-ZERO\">ground-zero</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> my-theory\u000A  (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))\u000A(<see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see> my-static-theory\u000A  (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))</code> \u000A \u000A <p>Now suppose we include that book after executing the following event.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> car-cons))</code> \u000A \u000A <p>Suppose that later we execute <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____THEORY\">theory</see> 'my-theory))</v>.  Then \u000A the rule <v>car-cons</v> will be disabled, because it was disabled at the time \u000A the expression <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v> was evaluated when processing the \u000A <v>deftheory</v> of <v>my-theory</v> while including the book.  However, if we \u000A execute <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____THEORY\">theory</see> 'my-static-theory))</v>, then the rule \u000A <v>car-cons</v> will be enabled, because the value of the theory \u000A <v>my-static-theory</v> was saved at the time the book was certified.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see> name term :doc doc-string)</code> \u000A \u000A <p>The arguments are handled the same as for <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>.  Thus, \u000A <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>), <v>term</v> is a term that \u000A when evaluated will produce a theory (see <see topic=\"ACL2____THEORIES\">theories</see>), and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).  Except for the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>, <v>term</v> must contain \u000A no free variables.  <v>Term</v> is evaluated with <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the \u000A current world (see <see topic=\"ACL2____WORLD\">world</see>) and the resulting theory is then converted to \u000A a <i>runic theory</i> (see <see topic=\"ACL2____THEORIES\">theories</see>) and associated with <v>name</v>. \u000A Henceforth, this runic theory is returned as the value of the theory \u000A expression <v>(<see topic=\"ACL2____THEORY\">theory</see> name)</v>.</p> \u000A \u000A <p>As for <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt>, the value returned is the length of the resulting \u000A theory.</p> \u000A \u000A <p>We conclude with an optional discussion about the implementation of \u000A <v>deftheory-static</v>, for those familiar with <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  The \u000A following macroexpansion of the <v>deftheory-static</v> form above shows how \u000A this works (see <see topic=\"ACL2____TRANS1\">trans1</see>).</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____DEFTHEORY-STATIC\">deftheory-static</see> my-static-theory\u000A                 (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))\u000A (<see topic=\"ACL2____MAKE-EVENT\">MAKE-EVENT</see> (<see topic=\"COMMON-LISP____LET\">LET</see> ((<see topic=\"ACL2____WORLD\">WORLD</see> (W STATE)))\u000A                  (<see topic=\"COMMON-LISP____LIST\">LIST</see> 'DEFTHEORY\u000A                        'MY-STATIC-THEORY\u000A                        (<see topic=\"COMMON-LISP____LIST\">LIST</see> 'QUOTE (<see topic=\"ACL2____CURRENT-THEORY\">CURRENT-THEORY</see> :HERE)))))\u000AACL2 !&gt;</code> \u000A \u000A <p>The idea is that upon evaluation of this <v>make-event</v> form, the first \u000A step is to evaluate the indicated <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> expression to obtain a form \u000A <v>(<see topic=\"ACL2____DEFTHEORY\">deftheory</see> my-theory '(...))</v>, where ``<v>(...)</v>'' is a list of all <see topic=\"ACL2____RUNE\">rune</see>s in current theory.  If this form is in a book being certified, then the \u000A resulting <v>deftheory</v> form is stored in the book's certificate, and is used \u000A when the book is included later.</p>"],

 "ACL2____UNION_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____UNION_42\">union$</see> l1 l2 ... lk)\u000A(<see topic=\"ACL2____UNION_42\">union$</see> l1 l2 ... lk :test 'eql) ; same as above\u000A(<see topic=\"ACL2____UNION_42\">union$</see> l1 l2 ... lk :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____UNION_42\">union$</see> l1 l2 ... lk :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____UNION_42\">Union$</see> x y)</v> equals a list that contains both the members of <v>x</v> \u000A and the members of <v>y</v>.  More precisely, the resulting list is the same as \u000A one would get by first deleting the members of <v>y</v> from <v>x</v>, and then \u000A concatenating the result to the front of <v>y</v>.  The optional keyword, \u000A <v>:TEST</v>, has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing members of the two lists.</p> \u000A \u000A <p><v>Union$</v> need not take exactly two arguments: <v>(<see topic=\"ACL2____UNION_42\">union$</see>)</v> is <v>nil</v>, \u000A <v>(<see topic=\"ACL2____UNION_42\">union$</see> x)</v> is <v>x</v>, <v>(<see topic=\"ACL2____UNION_42\">union$</see> x y z ... :test test)</v> is <v>(<see topic=\"ACL2____UNION_42\">union$</see> x\u000A (<see topic=\"ACL2____UNION_42\">union$</see> y z ... :test test) :test test)</v>, and if <v>:TEST</v> is not supplied, \u000A then <v>(<see topic=\"ACL2____UNION_42\">union$</see> x y z ...)</v> is <v>(<see topic=\"ACL2____UNION_42\">union$</see> x (<see topic=\"ACL2____UNION_42\">union$</see> y z ...))</v>.  For the \u000A discussion below we restrict ourselves, then, to the cases <v>(<see topic=\"ACL2____UNION_42\">union$</see> x y)</v> \u000A and <v>(<see topic=\"ACL2____UNION_42\">union$</see> x y :test test)</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>union$</v> (in the two cases just above) \u000A depends on the test.  In all cases, both arguments must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then one of the arguments must \u000A satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then one of the \u000A arguments must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>union$</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____UNION-EQ\">union-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____UNION_42\">union$</see> x lst :test\u000A 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____UNION-EQUAL\">union-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____UNION_42\">union$</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>union-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>union-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     union-equal (l1 l2)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l1) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l2))))\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l1) l2)\u000A           ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1) l2)\u000A            (<see topic=\"ACL2____UNION-EQUAL\">union-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2))\u000A           (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1)\u000A                    (<see topic=\"ACL2____UNION-EQUAL\">union-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2)))))</code> \u000A \u000A <p>Note that <v>union-eq</v> can take any number of arguments, in analogy to \u000A <v>union$</v>; indeed, <v>(<see topic=\"ACL2____UNION-EQ\">union-eq</see> ...)</v> expands to <v>(<see topic=\"ACL2____UNION_42\">union$</see> ... :test\u000A 'eq)</v>.  However, <v>union-equal</v> is a function, not a macro, and takes \u000A exactly two arguments.</p> \u000A \u000A <p><v>Union$</v> is similar to the Common Lisp primitive <v>union</v>.  However, \u000A Common Lisp does not specify the order of elements in the result of a call of \u000A <v>union</v>.</p>"],

 "ACL2____PUFF_A2":[["History"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;:puff* :max\u000AACL2 !&gt;:puff* :x\u000AACL2 !&gt;:puff* 15\u000AACL2 !&gt;:puff* &quot;book&quot;\u000A\u000AGeneral Form:\u000A:puff* cd</code> \u000A \u000A <p>where <v>cd</v> is a <see topic=\"ACL2____COMMAND\">command</see> descriptor (see <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>) for a ``puffable'' <see topic=\"ACL2____COMMAND\">command</see>.  See <see topic=\"ACL2____PUFF\">puff</see> for \u000A the definition of ``puffable'' and for a description of the basic act of \u000A ``puffing'' a <see topic=\"ACL2____COMMAND\">command</see>.  In particular, see <see topic=\"ACL2____PUFF\">puff</see> for a discussion \u000A of a sense in which <v>puff</v>, and hence <v>puff*</v>, should be viewed as a \u000A hack.  <v>Puff*</v> is just the recursive application of <see topic=\"ACL2____PUFF\">puff</see>. \u000A <v>Puff*</v> prints the region <see topic=\"ACL2____PUFF\">puff</see>ed, using <tt><see topic=\"ACL2____PCS\">pcs</see></tt>.</p> \u000A \u000A <p>To <see topic=\"ACL2____PUFF\">puff</see> a <see topic=\"ACL2____COMMAND\">command</see> is to replace it by its immediate \u000A subevents, each of which is executed as a <see topic=\"ACL2____COMMAND\">command</see>.  To <v>puff*</v> a \u000A <see topic=\"ACL2____COMMAND\">command</see> is to replace the <see topic=\"ACL2____COMMAND\">command</see> by each of its immediate \u000A subevents and then to <v>puff*</v> each of the puffable <see topic=\"ACL2____COMMAND\">command</see>s among \u000A the newly introduced ones.</p> \u000A \u000A <p>For example, suppose <v>&quot;ab&quot;</v> is a book containing the following</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;a&quot;)\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;b&quot;)</code> \u000A \u000A <p>Suppose that book <v>&quot;a&quot;</v> only contained <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s for the \u000A functions <v>a1</v> and <v>a2</v> and that <v>&quot;b&quot;</v> only contained <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s for <v>b1</v> and <v>b2</v>.</p> \u000A \u000A <p>Now consider an ACL2 <see topic=\"ACL2____STATE\">state</see> in which only two <see topic=\"ACL2____COMMAND\">command</see>s have \u000A been executed, the first being <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;ab&quot;)</v> and the second being \u000A <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;c&quot;)</v>.  Thus, the relevant part of the display produced by \u000A <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt> 1 would be:</p> \u000A \u000A <code>1 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;ab&quot;)\u000A2 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;c&quot;)</code> \u000A \u000A <p>Call this <see topic=\"ACL2____STATE\">state</see> the ``starting <see topic=\"ACL2____STATE\">state</see>'' in this example, \u000A because we will refer to it several times.</p> \u000A \u000A <p>Suppose <v>:puff 1</v> is executed in the starting <see topic=\"ACL2____STATE\">state</see>.  Then the \u000A first <see topic=\"ACL2____COMMAND\">command</see> is replaced by its immediate subevents and <v>:pbt 1</v> \u000A would show:</p> \u000A \u000A <code>1 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;a&quot;)\u000A2 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;b&quot;)\u000A3 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;c&quot;)</code> \u000A \u000A <p>Contrast this with the execution of <v>:puff* 1</v> in the starting <see topic=\"ACL2____STATE\">state</see>.  <v>Puff*</v> would first <see topic=\"ACL2____PUFF\">puff</see> <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;ab&quot;)</v> to get \u000A the <see topic=\"ACL2____STATE\">state</see> shown above.  But then it would recursively <v>puff*</v> the \u000A puffable <see topic=\"ACL2____COMMAND\">command</see>s introduced by the first <see topic=\"ACL2____PUFF\">puff</see>.  This continues \u000A recursively as long as any <see topic=\"ACL2____PUFF\">puff</see> introduced a puffable <see topic=\"ACL2____COMMAND\">command</see>. \u000A The end result of <v>:puff* 1</v> in the starting <see topic=\"ACL2____STATE\">state</see> is</p> \u000A \u000A <code>1 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A1 ...)\u000A2 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> A2 ...)\u000A3 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> B1 ...)\u000A4 (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> B2 ...)\u000A5 (<see topic=\"ACL2____INCLUDE-BOOK\">INCLUDE-BOOK</see> &quot;c&quot;)</code> \u000A \u000A <p>Observe that when <v>puff*</v> is done, the originally indicated <see topic=\"ACL2____COMMAND\">command</see>, <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;ab&quot;)</v>, has been replaced by the corresponding \u000A sequence of primitive <see topic=\"ACL2____EVENTS\">events</see>.  Observe also that puffable <see topic=\"ACL2____COMMAND\">command</see>s elsewhere in the <see topic=\"ACL2____HISTORY\">history</see>, for example, <see topic=\"ACL2____COMMAND\">command</see> 2 in \u000A the starting <see topic=\"ACL2____STATE\">state</see>, are not affected (except that their <see topic=\"ACL2____COMMAND\">command</see> \u000A numbers grow as a result of the splicing in of earlier <see topic=\"ACL2____COMMAND\">command</see>s).</p>"],

 "ACL2____PSTACK":[["Debugging"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____PSTACK\">pstack</see>)      ; inspect break\u000A(<see topic=\"ACL2____PSTACK\">pstack</see> t)    ; inspect break, printing all calls in abbreviated form\u000A(<see topic=\"ACL2____PSTACK\">pstack</see> :all) ; as above, but only abbreviating the ACL2 world</code> \u000A \u000A <p>When the form <v>(<see topic=\"ACL2____PSTACK\">pstack</see>)</v> is executed during a break from a proof, or at \u000A the end of a proof that the user has aborted, a ``process stack'' (or ``prover \u000A stack'') will be printed that gives some idea of what the theorem prover has \u000A been doing.  Moreover, by evaluating <v>(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> t)</v> before starting a \u000A proof (see <see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see>) one can get trace-like information about \u000A prover functions, including time summaries, printed to the screen during a \u000A proof.  This feature is currently quite raw and may be refined considerably as \u000A time goes on, based on user suggestions.  For example, the usual control of \u000A printing given by <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> is irrelevant for printing the \u000A pstack.</p> \u000A \u000A <p>The use of <v>(<see topic=\"ACL2____PSTACK\">pstack</see> t)</v> or <v>(<see topic=\"ACL2____PSTACK\">pstack</see> :all)</v> should only be used by \u000A those who are comfortable looking at functions in the ACL2 source code. \u000A Otherwise, simply use <v>(<see topic=\"ACL2____PSTACK\">pstack</see>)</v>.</p> \u000A \u000A <p>Entries in the pstack include the following (listed here alphabetically, \u000A except for the first).</p> \u000A \u000A <p><v>preprocess-clause</v>, <v>simplify-clause</v>, etc. (in \u000A general,<v>xxx-clause</v>): top-level processes in the prover ``waterfall''</p> \u000A \u000A <p><v>clausify</v>: splitting a goal into subgoals</p> \u000A \u000A <p><v>ev-fncall</v>: evaluating a function on explicit arguments</p> \u000A \u000A <p><v>ev-fncall-meta</v>: evaluating a metafunction</p> \u000A \u000A <p><v>forward-chain</v>: building a context for the current goal using <tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> rules</p> \u000A \u000A <p><v>induct</v>: finding an induction scheme</p> \u000A \u000A <p><v>pop-clause</v>: getting the next goal to prove by induction</p> \u000A \u000A <p><v>process-assumptions</v>: creating forcing rounds</p> \u000A \u000A <p><v>remove-built-in-clauses</v>: removing built-in clauses (see <see topic=\"ACL2____BUILT-IN-CLAUSE\">built-in-clause</see>)</p> \u000A \u000A <p><v>process-equational-polys</v>: deducing interesting equations</p> \u000A \u000A <p><v>remove-trivial-equivalences</v>: removing trivial equalities (and \u000A equivalences) from the current goal</p> \u000A \u000A <p><v>rewrite-atm</v>: rewriting a top-level term in the current goal</p> \u000A \u000A <p><v>setup-simplify-clause-pot-lst</v>: building the linear arithmetic database \u000A for the current goal</p> \u000A \u000A <p><v>strip-branches</v>, <v>subsumption-replacement-loop</v>: subroutines of \u000A <v>clausify</v></p> \u000A \u000A <p><v>waterfall</v>: top-level proof control</p>"],

 "ACL2____NOTE-2-7-SYSTEM":[["Note-2-7"],"ACL2 Sources","ACL2","<p>ACL2 now runs (once again) under LispWorks, specifically, LispWorks \u000A 4.2.0.  However, we needed a patch, which presumably will be unnecessary after \u000A 4.2.7.  From LispWorks support:</p> \u000A \u000A <blockquote><p>Users with LispWorks4.2.7 should ask us at \u000A lisp-support@xanalys.com for the transform-if-node patch. It will be helpful \u000A if they quote (Lisp Support Call #11372) when doing so. Also, they must send a \u000A bug form generated from their LispWorks image: instructions at \u000A http://www.lispworks.com/support/bug-report.html.</p></blockquote> \u000A \u000A <p>File <v>books/Makefile-generic</v> has been improved so that failed attempts \u000A to certify a book will cause the `make' to fail.  Previously, an existing \u000A <v>.cert</v> file was left in place, and that sufficed for the `make' to be \u000A considered a success.  Now, the old <v>.cert</v> file is first removed when \u000A recertification is found to be necessary.</p> \u000A \u000A <p>A change has been made to source file <v>acl2.lisp</v> to accommodate GCL \u000A 2.4.3.  (ACL2 Version 2.6 does not work with some versions of GCL 2.4.3.)</p> \u000A \u000A <p>The error message has been improved when certain forms are typed to raw \u000A Lisp and the ACL2 loop has never been entered (with <v>(</v><tt><see topic=\"ACL2____LP\">lp</see></tt><v>)</v>).</p> \u000A \u000A <p>The following symbols in the ACL2 package have been made untouchable, \u000A meaning that they are not available to the user: <v>ev-fncall</v>, <v>ev</v>, \u000A <v>ev-lst</v>, <v>ev-acl2-unwind-protect</v>, <v>ev-fncall!</v>, and \u000A <v>user-stobj-alist-safe</v>.  The reason is that these functions can not be \u000A called safely except under certain restrictions.  If you want to call the ACL2 \u000A evaluator, consider using the built-in system functions <v>trans-eval</v> or \u000A simple-translate-and-eval.</p> \u000A \u000A <p>CLISP Version_2.30 implements a notion of ``locking'' the \"LISP\" package \u000A that is incompatible with building ACL2.  (CLISP Version_2.27 does not appear \u000A to have had this feature.)  We have gotten around this problem by unlocking \u000A the \"LISP\" package in ACL2 images built on such CLISPs.</p> \u000A \u000A <p>Automatic proclaiming for GCL, which has (for a long time) been done for \u000A functions in compiled books, has been improved.  Formerly, the only time a \u000A non-trivial output type (i.e., other than <v>t</v>) was inferred was when \u000A macroexpansion produced an explicit call of <tt><see topic=\"COMMON-LISP____THE\">the</see></tt>.  Now, <tt><see topic=\"COMMON-LISP____IF\">if</see></tt> \u000A expressions can also generate non-<v>t</v> output types.  Consider the following \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> the-fixnum (n)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'the '(signed-byte 29) n))\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> 1+f (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'the-fixnum\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> '1+ (<see topic=\"COMMON-LISP____LIST\">list</see> 'the-fixnum x))))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (unsigned-byte 27) x))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> x)\u000A      0\u000A    (1+f (foo (1-f x)))))</code> \u000A \u000A <p>Formerly, the <v>proclaim</v> forms for <v>foo</v>, before and after this \u000A improvement, are as shown below.</p> \u000A \u000A <code>(PROCLAIM '(FTYPE (FUNCTION ((UNSIGNED-BYTE 27)) T) FOO))                ;old\u000A(PROCLAIM '(FTYPE (FUNCTION ((UNSIGNED-BYTE 27)) (SIGNED-BYTE 29)) FOO)) ;new</code> \u000A \u000A <p>Compiler info messages sent to error stream were eliminated for CMUCL.</p>"],

 "ACL2____EVISC-TUPLE":[["Io"],"ACL2 Sources","ACL2","<p>ACL2 output is generally printed in full.  However, ACL2 can be \u000A directed to abbreviate, or ``eviscerate'', objects before printing them.  To \u000A ``eviscerate'' an object we replace certain substructures within it by strings \u000A that are printed in their stead.  Such replacement is made relative to a \u000A so-called ``evisc-tuple'', which has four components: <v>(<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>\u000A print-level print-length alist hiding-cars)</v> is the same as the value of \u000A <v>(<see topic=\"COMMON-LISP____LIST\">list</see> alist print-level print-length hiding-cars)</v>, and the components are \u000A used as follows (with priority order as discussed below).  The alist component \u000A is used to replace any substructure occurring as a key by the corresponding \u000A string.  The print-level and print-length are analogous to Common Lisp \u000A variables <v>*print-level*</v> and <v>*print-length*</v>, respectively, and cause \u000A replacement of substructures deeper than print-level by `<v>#</v>' and those \u000A longer than print-length by `<v>...</v>'.  Finally, any <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> <v>x</v> \u000A that starts with one of the symbols in <v>hiding-cars</v> is printed as \u000A <v>&lt;hidden&gt;</v>.</p> \u000A \u000A <p>The following example illustrates the use of an evisc-tuple that limits the \u000A print-level to 3 &mdash; only three descents into list structures are \u000A permitted before replacing a subexpression by `<v>#</v>' &mdash; and limits the \u000A print-length to 4 &mdash; only the first four elements of any list structure \u000A will be printed before replacing its tail by `<v>...</v>'.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____FMS\">fms</see> &quot;~x0~%&quot;\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 '((a b ((c d)) e f g) u v w x y)))\u000A            *standard-co*\u000A            state\u000A            (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 3 4 nil nil))\u000A\u000A((A B (#) E ...) U V W ...)\u000A&lt;state&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p>Notice that it is impossible to read the printed value back into ACL2, \u000A since there is no way for the ACL2 reader to interpret `<v>#</v>' or `<v>...</v>'. \u000A To solve this problem, see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.</p> \u000A \u000A <p>In the above example we pass an evisc-tuple explicitly to a printing \u000A function, in this case, <tt><see topic=\"ACL2____FMS\">fms</see></tt> (see <see topic=\"ACL2____FMT\">fmt</see>).  But ACL2 also does its \u000A own printing, for example during a proof attempt.  There are global \u000A evisc-tuples that control ACL2's printing; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> and see \u000A <see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see>.</p>"],

 "ACL2____DEFAULT-DEFUN-MODE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>When a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is processed and no <v>:mode</v> <v>xarg</v> is \u000A supplied, the function <v>default-defun-mode</v> is used.  To find the default \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of the current ACL2 <see topic=\"ACL2____WORLD\">world</see>, type \u000A <v>(<see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> (w state))</v>.  See <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> for a discussion \u000A of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s.  To change the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> of the ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see>, type one of the keywords <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.</p> \u000A \u000A <p>The default ACL2 <see topic=\"ACL2____PROMPT\">prompt</see> displays the current default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> by showing the character <v>p</v> for <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, \u000A and omitting it for <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode; see <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>. \u000A The default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> may be changed using the keyword <see topic=\"ACL2____COMMAND\">command</see>s \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, which are equivalent to the \u000A <see topic=\"ACL2____COMMAND\">command</see>s <v>(<see topic=\"ACL2____PROGRAM\">program</see>)</v> and <v>(<see topic=\"ACL2____LOGIC\">logic</see>)</v>.  Each of these names is \u000A documented separately: see <see topic=\"ACL2____PROGRAM\">program</see> and see <see topic=\"ACL2____LOGIC\">logic</see>.  The default \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is stored in the <see topic=\"ACL2____TABLE\">table</see> <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> \u000A and hence may also be changed by a <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____COMMAND\">command</see>.  See <see topic=\"ACL2____TABLE\">table</see> and also see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  Both mode-changing <see topic=\"ACL2____COMMAND\">command</see>s are <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>While <see topic=\"ACL2____EVENTS\">events</see> that change the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> are permitted \u000A within an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the text of a book, their effects are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> in scope to the duration of the encapsulation or inclusion.  For \u000A example, if the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> and a book is \u000A included that contains the event <v>(<see topic=\"ACL2____PROGRAM\">program</see>)</v>, then subsequent <see topic=\"ACL2____EVENTS\">events</see> \u000A within the book are processed with the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>; but when the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event completes, the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> will still be <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.  <see topic=\"ACL2____COMMAND\">Command</see>s that change \u000A the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> are not permitted inside <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A forms.</p>"],

 "COMMON-LISP____LET_A2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Examples</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((x (<see topic=\"COMMON-LISP_____A2\">*</see> x x))\u000A       (y (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)))\u000A (<see topic=\"COMMON-LISP____LIST\">list</see> x y))\u000A\u000A(<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((x (<see topic=\"COMMON-LISP_____A2\">*</see> x x))\u000A       (y (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x))\u000A       (x (<see topic=\"COMMON-LISP_____A2\">*</see> x y))\u000A       (a (<see topic=\"COMMON-LISP_____A2\">*</see> x x)))\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> a))\u000A (<see topic=\"COMMON-LISP____LIST\">list</see> x y))</code> \u000A \u000A <p>If the forms above are executed in an environment in which <v>x</v> has the \u000A value <v>-2</v>, then the respective results are <v>'(4 8)</v> and <v>'(32 8)</v>. \u000A See <see topic=\"COMMON-LISP____LET\">let</see> for a discussion of both <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and <v>let*</v>, or read on \u000A for a briefer discussion.</p> \u000A \u000A <p>The difference between <tt><see topic=\"COMMON-LISP____LET\">let</see></tt> and <v>let*</v> is that the former binds \u000A its local variables in parallel while the latter binds them sequentially. \u000A Thus, in <v>let*</v>, the term evaluated to produce the local value of one of \u000A the locally bound variables is permitted to reference any locally bound \u000A variable occurring earlier in the binding list and the value so obtained is \u000A the newly computed local value of that variable.  See <see topic=\"COMMON-LISP____LET\">let</see>.</p> \u000A \u000A <p>In ACL2 the only <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms allowed for a <v>let*</v> form are \u000A <v>ignore</v>, <v>ignorable</v>, and <v>type</v>.  See <see topic=\"COMMON-LISP____DECLARE\">declare</see>.  Moreover, no \u000A variable declared <v>ignore</v>d or <v>ignorable</v> may be bound more than once. \u000A A variable with a type declaration may be bound more than once, in which case \u000A the type declaration is treated by ACL2 as applying to each binding occurrence \u000A of that variable.  It seems unclear from the Common Lisp spec whether the \u000A underlying Lisp implementation is expected to apply such a declaration to more \u000A than one binding occurrence, however, so performance in such cases may depend \u000A on the underlying Lisp.</p> \u000A \u000A <p><v>Let*</v> is a Common Lisp macro.  See any Common Lisp documentation for \u000A more information.</p>"],

 "ACL2____OTHER":[["ACL2"],"ACL2 Sources","ACL2","<p>This section contains an assortment of top-level functions that fit \u000A into none of the other categories and yet are suffiently useful as to merit \u000A ``<v>advertisement</v>'' in the <v>:</v><tt><see topic=\"ACL2____HELP\">help</see></tt> command.</p>"],

 "ACL2____HONS-ACONS_12":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-acons!</v> is like <tt><see topic=\"COMMON-LISP____ACONS\">acons</see></tt> except that its guard \u000A does not require <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>; we leave it enabled and would think it odd to \u000A ever prove a theorem about it.</p> \u000A \u000A <p>Ordinarily, <see topic=\"ACL2____FAST-ALISTS\">fast-alists</see> are constructed with <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> \u000A instead of <v>hons-acons!</v>.  In such alists, the keys are honsed, but the \u000A conses that make up the \"spine\" of the alist itself are ordinary conses.  In \u000A other words, it is basically correct to say:</p> \u000A \u000A <code>(<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> key val alist) == (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____HONS-COPY\">hons-copy</see> key) val) alist)</code> \u000A \u000A <p>In contrast, when <v>hons-acons!</v> is used, the conses making up the alist \u000A itself are also normed.  That is,</p> \u000A \u000A <code>(<see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see> key val alist) == (<see topic=\"ACL2____HONS\">hons</see> (<see topic=\"ACL2____HONS\">hons</see> key val) alist)</code> \u000A \u000A <p>Generally, you <b>should not use</b> <v>hons-acons!</v> unless you really \u000A know what you're doing.</p> \u000A \u000A <p>Drawback 1.  <v>hons-acons!</v> requires you to <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> all of the \u000A values that are being stored in the fast alist.  If you are storing large \u000A values, this may be expensive.</p> \u000A \u000A <p>Drawback 2.  It can be more difficult to maintain the proper discipline \u000A when using <v>hons-acons!</v>.  For instance, consider the following:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((al1 (<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> 1 'one (<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> 2 'two nil)))\u000A      (al2 (<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> 1 'one (<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> 2 'two nil))))\u000A   ...)</code> \u000A \u000A <p>Here, both al1 and al2 are valid fast alists and they can be extended \u000A independently without any trouble.  But if these alists had instead been \u000A constructed with <v>hons-acons!</v>, then since both al1 and al2 are equal, \u000A normed conses, they will literally be <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> and hence will refer to \u000A precisely the same hash table.  In other words, <v>hons-acons!</v> makes it \u000A relatively easy to <b>inadvertently</b> steal the hash table associated with \u000A some other fast alist.  This problem can be alleviated somewhat by uniquely \u000A naming alists; see the discussion in <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> for details.</p> \u000A \u000A <p>Despite these drawbacks, <v>hons-acons!</v> is the typical way to generate a \u000A fast alist that is normed (but also, for example, see <see topic=\"ACL2____FAST-ALIST-FORK_12\">fast-alist-fork!</see>).  It is not adequate to <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> a fast alist that \u000A was generated by ordinary <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt> calls, because this would produce \u000A an EQUAL-but-not-EQ object, and this new object would not be associated with \u000A the fast alist's hash table.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-acons!</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-acons! (key val alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> key val) alist))</code>"],

 "ACL2____COMMAND-DESCRIPTOR":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A:max      ; the command most recently typed by the user\u000A:x        ; synonymous with :max\u000A(:x -1)   ; the command before the most recent one\u000A(:x -2)   ; the command before that\u000A:x-2      ; synonymous with (:x -2)\u000A5         ; the fifth command typed by the user\u000A1         ; the first command typed by the user\u000A0         ; the last command of the system initialization\u000A-1        ; the next-to-last initialization command\u000A:min      ; the first command of the initialization\u000A:start    ; the last command of the initial ACL2 logical world\u000Afn        ; the command that introduced the logical name fn\u000A(:search (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo-bar))\u000A          ; the first command encountered in a search from :max to\u000A          ; 0 that either contains defmacro and foo-bar in the\u000A          ; command form or contains defmacro and foo-bar in some\u000A          ; event within its block.</code> \u000A \u000A <p>The recorded <see topic=\"ACL2____HISTORY\">history</see> of your interactions with the top-level ACL2 \u000A <see topic=\"ACL2____COMMAND\">command</see> loop is marked by the <see topic=\"ACL2____COMMAND\">command</see>s you typed that changed \u000A the logical <see topic=\"ACL2____WORLD\">world</see>.  Each such <see topic=\"ACL2____COMMAND\">command</see> generated one or more \u000A <see topic=\"ACL2____EVENTS\">events</see>, since the only way for you to change the logical <see topic=\"ACL2____WORLD\">world</see> \u000A is to execute an event function.  See <see topic=\"ACL2____COMMAND\">command</see> and see <see topic=\"ACL2____EVENTS\">events</see>. \u000A We divide <see topic=\"ACL2____HISTORY\">history</see> into ``<see topic=\"ACL2____COMMAND\">command</see> blocks,'' grouping together \u000A each <see topic=\"ACL2____WORLD\">world</see> changing <see topic=\"ACL2____COMMAND\">command</see> and its <see topic=\"ACL2____EVENTS\">events</see>.  A ``<see topic=\"ACL2____COMMAND\">command</see> descriptor'' is an object that can be used to describe a particular \u000A <see topic=\"ACL2____COMMAND\">command</see> in the <see topic=\"ACL2____HISTORY\">history</see> of the ongoing session.</p> \u000A \u000A <p>Each <see topic=\"ACL2____COMMAND\">command</see> is assigned a unique integer called its ``<see topic=\"ACL2____COMMAND\">command</see> number'' which indicates the <see topic=\"ACL2____COMMAND\">command</see>'s position in the \u000A chronological ordering of all of the <see topic=\"ACL2____COMMAND\">command</see>s ever executed in this \u000A session (including those executed to initialize the system).  We assign the \u000A number 1 to the first <see topic=\"ACL2____COMMAND\">command</see> you type to ACL2.  We assign 2 to the \u000A second and so on.  The non-positive integers are assigned to ``prehistoric'' \u000A <see topic=\"ACL2____COMMAND\">command</see>s, i.e., the <see topic=\"ACL2____COMMAND\">command</see>s used to initialize the ACL2 system: \u000A 0 is the last <see topic=\"ACL2____COMMAND\">command</see> of the initialization, -1 is the one before that, \u000A etc.</p> \u000A \u000A <p>The legal <see topic=\"ACL2____COMMAND\">command</see> descriptors are described below.  We use <v>n</v> to \u000A denote any integer, <v>sym</v> to denote any logical name (see <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>), and <v>cd</v> to denote, recursively, any <see topic=\"ACL2____COMMAND\">command</see> \u000A descriptor.</p> \u000A \u000A <code> command                   command\u000Adescriptor                described\u000A\u000A:max   -- the most recently executed command (i.e., the one with\u000A          the largest command number)\u000A:x     -- synonymous with :max\u000A:x-k   -- synonymous with (:x -k), if k is an integer and k&gt;0\u000A:min   -- the earliest command (i.e., the one with the smallest\u000A          command number and hence the first command of the system\u000A          initialization)\u000A:start -- the last command when ACL2 starts up\u000An      -- command number n  (<see topic=\"COMMON-LISP____IF\">If</see> n is not in the\u000A          range :min&lt;=n&lt;=:max, n is replaced by the nearest of :min\u000A          and :max.)\u000Asym    -- the command that introduced the logical name sym\u000A(cd n) -- the command whose number is n plus the command number of\u000A          the command described by cd\u000A(:search pat cd1 cd2)\u000A          In this command descriptor, pat must be either an atom or\u000A          a true list of atoms and cd1 and cd2 must be command\u000A          descriptors.  We search the interval from cd1 through cd2\u000A          for the first command that matches pat.  Note that if cd1\u000A          occurs chronologically after cd2, the search is\u000A          ``backwards'' through history while if cd1 occurs\u000A          chronologically before cd2, the search is ``forwards''.  A\u000A          backwards search will find the most recent match; a\u000A          forward search will find the chronologically earliest\u000A          match.  A command matches pat if either the command form\u000A          itself or one of the events in the block contains pat (<see topic=\"COMMON-LISP____OR\">or</see>\u000A          all of the atoms in pat if pat is a list).\u000A(:search pat)\u000A          the command found by (:search pat :max 0), i.e., the most\u000A          recent command matching pat that was part of the user's\u000A          session, not part of the system initialization.</code>"],

 "ACL2____ASET1":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____ASET1\">aset1</see> 'delta1 a (<see topic=\"COMMON-LISP_____B2\">+</see> i k) 27)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ASET1\">aset1</see> name alist index val)</code> \u000A \u000A <p>where <v>name</v> is a symbol, <v>alist</v> is a 1-dimensional array named \u000A <v>name</v>, <v>index</v> is a legal index into <v>alist</v>, and <v>val</v> is an \u000A arbitrary object.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.  Roughly speaking this \u000A function ``modifies'' <v>alist</v> so that the value associated with <v>index</v> \u000A is <v>val</v>.  More precisely, it returns a new array, <v>alist'</v>, of the same \u000A name and dimension as <v>alist</v> that, under <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>, is everywhere \u000A equal to <v>alist</v> except at <v>index</v> where the result is <v>val</v>.  That \u000A is, <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist' i)</v> is <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist i)</v> for all legal \u000A indices <v>i</v> except <v>index</v>, where <v>(<see topic=\"ACL2____AREF1\">aref1</see> name alist' i)</v> is \u000A <v>val</v>.</p> \u000A \u000A <p>In order to ``modify'' <v>alist</v>, <v>aset1</v> <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es a new pair \u000A onto the front.  If the length of the resulting alist exceeds the <v>:</v><tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> entry in the array <see topic=\"ACL2____HEADER\">header</see>, <v>aset1</v> compresses the \u000A array as with <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt>.</p> \u000A \u000A <p>It is generally expected that the ``semantic value'' of <v>name</v> will be \u000A <v>alist</v> (see <see topic=\"ACL2____ARRAYS\">arrays</see>).  This function operates in virtually constant \u000A time whether this condition is true or not (unless the <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> \u000A operation is required).  But the value returned by this function cannot be \u000A used efficiently by subsequent <v>aset1</v> operations unless <v>alist</v> is the \u000A semantic value of <v>name</v> when <v>aset1</v> is executed.  Thus, if the \u000A condition is not true, <v>aset1</v> prints a <b>slow array</b> warning to the \u000A comment window.  See <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>aset1</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     aset1 (<see topic=\"ACL2____NAME\">name</see> l n val)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l)\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))))))\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((l (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> n val) l)))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> name l))\u000A                 (<see topic=\"ACL2____COMPRESS1\">compress1</see> name l))\u000A                (t l))))</code>"],

 "ACL2-PC____EXIT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Aexit                        -- exit the interactive proof-checker\u000A(<see topic=\"ACL2____EXIT\">exit</see> t)                    -- exit after printing a bogus defthm event\u000A(<see topic=\"ACL2____EXIT\">exit</see> append-associativity) -- exit and create a defthm\u000A                               event named append-associativity\u000A\u000AGeneral Forms:\u000A\u000Aexit --  Exit without storing an event.\u000A\u000A(<see topic=\"ACL2____EXIT\">exit</see> t) -- Exit after printing a bogus defthm event, showing :INSTRUCTIONS.\u000A\u000A(<see topic=\"ACL2____EXIT\">exit</see> event-name &amp;optional rule-classes do-it-flg) --\u000AExit, and perhaps store an event</code> \u000A \u000A <p>The command <v>exit</v> returns you to the ACL2 loop.  At a later time, \u000A <v>(<see topic=\"ACL2____VERIFY\">verify</see>)</v> may be executed to get back into the same proof-checker state, \u000A as long as there hasn't been an intervening use of the proof-checker \u000A (otherwise see <v>save</v>).</p> \u000A \u000A <p>When given one or more arguments as shown above, <v>exit</v> still returns \u000A you to the ACL2 loop, but first, if the interactive proof is complete, then it \u000A attempts create a <v>defthm</v> event with the specified <v>event-name</v> and \u000A <v>rule-classes</v> (which defaults to <v>(:rewrite)</v> if not supplied).  The \u000A event will be printed to the terminal, and then normally the user will be \u000A queried whether an event should really be created.  However, if the final \u000A optional argument <v>do-it-flg</v> is supplied and not <v>nil</v>, then an event \u000A will be made without a query.</p> \u000A \u000A <p>For example, the form</p> \u000A \u000A <code>(<see topic=\"ACL2____EXIT\">exit</see> top-pop-elim (:elim :rewrite) t)</code> \u000A \u000A <p>causes a <v>defthm</v> event named <v>top-pop-elim</v> to be created with \u000A rule-classes <v>(:elim :rewrite)</v>, without a query to the user (because of \u000A the argument <v>t</v>).</p> \u000A \u000A <p><b>Remark:</b> it is permitted for <v>event-name</v> to be <v>nil</v>.  In that \u000A case, the name of the event will be the name supplied during the original call \u000A of <v>verify</v>.  (See the documentation for <v>verify</v> and <v>commands</v>.) \u000A Also in that case, if <v>rule-classes</v> is not supplied then it defaults to \u000A the rule-classes supplied in the original call of <v>verify</v>.</p> \u000A \u000A <p><v>Comments</v> on ``success'' and ``failure''.  An <v>exit</v> instruction \u000A will always ``fail'', so for example, if it appears as an argument of a \u000A <v>do-strict</v> instruction then none of the later (instruction) arguments will \u000A be executed.  Moreover, the ``failure'' will be ``hard'' if an event is \u000A successfully created or if the instruction is simply <v>exit</v>; otherwise it \u000A will be ``soft''.  See the documentation for <v>sequence</v> for an explanation \u000A of hard and soft ``failures''.  An obscure but potentially important fact is \u000A that if the ``failure'' is hard, then the error signal is a special signal \u000A that the top-level interactive loop can interpret as a request to exit.  Thus \u000A for example, a sequencing command that turns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val\u000A state)</v> into <v>(<see topic=\"ACL2____MV\">mv</see> t val state)</v> would never cause an exit from the \u000A interactive loop.</p> \u000A \u000A <p>If the proof is not complete, then <v>(<see topic=\"ACL2____EXIT\">exit</see> event-name ...)</v> will not \u000A cause an exit from the interactive loop.  However, in that case it will print \u000A out the original user-supplied goal (the one that was supplied with the call \u000A to <v>verify</v>) and the current list of instructions.</p>"],

 "ACL2____PRINTING-TO-STRINGS":[["Io","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Each of ACL2's formatted printing functions, <v>FM*</v>, has an \u000A analoguous macro <v>FM*-TO-STRING</v> indicated below.  These functions do not \u000A include a channel or <tt><see topic=\"ACL2____STATE\">state</see></tt> as an argument, and <v>FM*-TO-STRING</v> \u000A returns the string that <v>FM*</v> would print to the state, in place of \u000A <v>state</v>; see <see topic=\"ACL2____FMT\">fmt</see>.  The legal keyword arguments are described \u000A below.</p> \u000A \u000A <code>General Forms:                                 result\u000A(<see topic=\"ACL2____FMS-TO-STRING\">fms-to-string</see> string alist &amp;key ...)          ; string\u000A(<see topic=\"ACL2____FMT-TO-STRING\">fmt-to-string</see> string alist &amp;key ...)          ; (<see topic=\"ACL2____MV\">mv</see> col string)\u000A(<see topic=\"ACL2____FMT1-TO-STRING\">fmt1-to-string</see> string alist column &amp;key ...)  ; (<see topic=\"ACL2____MV\">mv</see> col string)\u000A(<see topic=\"ACL2____FMS_12-TO-STRING\">fms!-to-string</see> string alist &amp;key ...)         ; string\u000A(<see topic=\"ACL2____FMT_12-TO-STRING\">fmt!-to-string</see> string alist &amp;key ...)         ; (<see topic=\"ACL2____MV\">mv</see> col string)\u000A(<see topic=\"ACL2____FMT1_12-TO-STRING\">fmt1!-to-string</see> string alist column &amp;key ...) ; (<see topic=\"ACL2____MV\">mv</see> col string)</code> \u000A \u000A <p>The legal keyword arguments are as follows.  They are all optional with a \u000A default of <v>nil</v>.</p> \u000A \u000A <blockquote><p><v>Evisc-tuple</v> is evaluated, and corresponds exactly to the \u000A <v>evisc-tuple</v> argument of the corresponding <v>FM*</v> function; see <see topic=\"ACL2____FMT\">fmt</see>.</p> \u000A \u000A <p><v>Fmt-control-alist</v> should typically evaluate to an alist that maps \u000A print-control variables to values; see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see>.  Any alist \u000A mapping variables to values is legal, however.  By default the print controls \u000A are set according to the value of constant <v>*fmt-control-defaults*</v>; \u000A <v>fmt-control-alist</v> overrides these defaults.  For example, \u000A <v>*fmt-control-defaults*</v> sets the right margin just as it is set in the \u000A initial ACL2 <see topic=\"ACL2____STATE\">state</see>, by binding <v>fmt-soft-right-margin</v> and \u000A <v>fmt-hard-right-margin</v> to their respective defaults of \u000A <v>*fmt-soft-right-margin-default*</v> and <v>*fmt-hard-right-margin-default*</v>. \u000A The following example shows how you can override those defaults, in this case \u000A arranging to print flat by setting the right margin to a large number.</p> \u000A \u000A <code>(<see topic=\"ACL2____FMT-TO-STRING\">fmt-to-string</see> &quot;~x0&quot;\u000A               (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> 30)))\u000A               :fmt-control-alist\u000A               `((fmt-soft-right-margin . 10000)\u000A                 (fmt-hard-right-margin . 10000)))</code> \u000A \u000A <p><v>Iprint</v> is typically <v>nil</v>, but <v>t</v> is also a legal value.  See \u000A <see topic=\"ACL2____SET-IPRINT\">set-iprint</see> for the effect of value <v>t</v>, which however is local to \u000A this call of a <v>FM*-TO-STRING</v> function; the behavior if iprinting \u000A afterwards is not affected by this call.  In particular, you will not be able \u000A to look at the values of iprint tokens printed by this call, so the value \u000A <v>t</v> is probably of limited utility at best.</p></blockquote> \u000A \u000A <p>Also see <see topic=\"ACL2____IO\">io</see> for a discussion of the utility \u000A <v>get-output-stream-string$</v>, which allows for accumulating the results of \u000A more than one printing call into a single string but requires the use of \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p>"],

 "ACL2____MAKE-TAU-INTERVAL":[["Tau-system"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> doc lo-rel lo hi-rel hi)</code> \u000A \u000A <p>An interval is a structure of the form: <v>(</v><i>dom</i> \u000A <v>(</v><i>lo-rel</i> <v>.</v>  <i>lo</i><v>)</v> <v>.</v>  <v>(</v><i>hi-rel</i> <v>.</v> \u000A <i>hi</i><v>))</v>.  Every tau contains an interval used to represent the \u000A domain, the upper, and the lower bounds of the objects recognized by the \u000A tau.</p> \u000A \u000A <p><v>make-tau-interval</v> constructs well-formed intervals only if its five \u000A arguments satisfy certain restrictions given below.  When these restrictions \u000A are violated <v>make-tau-interval</v> can construct objects that are not \u000A intervals!  <v>make-tau-interval</v> does not attempt to coerce or adjust its \u000A arguments to make well-formed intervals.</p> \u000A \u000A <p>For examples of intervals (and non-intervals!) constructed by \u000A <v>make-tau-interval</v> see <tt><see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see></tt>.  For examples of what \u000A objects are contained in certain intervals, see <tt><see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see></tt>.</p> \u000A \u000A <p>The components of an interval are as follows:</p> \u000A \u000A <p><i>dom</i> (``domain'') -- must be one of four symbols: <v>INTEGERP</v>, \u000A <v>RATIONALP</v>, <v>ACL2-NUMBERP</v>, or <v>NIL</v> denoting no restriction on the \u000A domain.</p> \u000A \u000A <p>The two ``relations,'' <i>lo-rel</i> and <i>hi-rel</i> are Booleans, where \u000A <v>t</v> denotes less-than inequality (<tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt>) and <v>nil</v> represents \u000A less-than-or-equal inequality (<tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt>).  Think of <v>t</v> meaning \u000A ``strong'' and <v>nil</v> meaning ``weak'' inequality.</p> \u000A \u000A <p><i>Lo</i> and <i>hi</i> must be either <v>nil</v> or explicit rational \u000A numbers.  If <i>lo</i> is <v>nil</v> it denotes negative infinity; if <i>hi</i> \u000A is <v>nil</v> it denotes positive infinity.  <i>Lo</i> must be no greater than \u000A <i>hi</i>.  <i>Note</i>: Even though <v>ACL2-NUMBERP</v> intervals may contain \u000A complex rationals, the <i>lo</i> and <i>hi</i> bounds must be rational.  This \u000A is an arbitrary decision made by the implementors to simplify coding.</p> \u000A \u000A <p>Finally, if the <i>dom</i> is <v>INTEGERP</v>, then both relations should be \u000A weak and <i>lo</i> and <i>hi</i> must be integers when they are \u000A non-<v>nil</v>.</p> \u000A \u000A <p>For <i>x</i> to be ``in'' an interval it must be of the type described by \u000A the domain predicate <i>dom</i>, <i>lo</i> must be smaller than <i>x</i> in \u000A the strong or weak sense denoted by <i>lo-rel</i>, and <i>x</i> must be \u000A smaller than <i>hi</i> in the strong or weak sense denoted by \u000A <i>hi-rel</i>.</p> \u000A \u000A <p>The components of an interval may be accessed with the functions <tt><see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see></tt>, <tt><see topic=\"ACL2____TAU-INTERVAL-LO-REL\">tau-interval-lo-rel</see></tt>, <tt><see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see></tt>, \u000A <tt><see topic=\"ACL2____TAU-INTERVAL-HI-REL\">tau-interval-hi-rel</see></tt>, and <tt><see topic=\"ACL2____TAU-INTERVAL-HI\">tau-interval-hi</see></tt>.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q2-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>The correct answer to Question 2 in <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see> is <i>Subgoal (i)</i> plus any one of the \u000A other other three.  For your reference, the four choices were:</p> \u000A \u000A <code><i>Subgoal (i)</i>: \u000A(implies (and (not (endp z)) \u000A              (true-listp z)) \u000A         (true-listp (cdr z))) \u000A \u000A<i>Subgoal (ii)</i>: \u000A(implies (and (not (endp z)) \u000A              (true-listp z) \u000A              (equal (rev (rev (cdr z))) (cdr z))) \u000A         (equal (rev (rev z)) z)) \u000A \u000A<i>Subgoal (iii)</i>: \u000A(implies (and (not (endp z)) \u000A              (equal (rev (rev (cdr z))) (cdr z))) \u000A         (equal (rev (rev z)) z)) \u000A \u000A<i>Subgoal (iv)</i>: \u000A(implies (and (not (endp z)) \u000A              (true-listp (cdr z)) \u000A              (equal (rev (rev (cdr z))) (cdr z))) \u000A         (equal (rev (rev z)) z)) \u000A</code> \u000A \u000A <p>In particular, it is wrong to think the Induction Step of the proof of</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))</code> \u000A \u000A <p>can be established by proving just <i>Subgoal (ii)</i>, <i>Subgoal \u000A (iii)</i>, <i>Subgoal (iv)</i>, or combinations of those three.  You must also \u000A prove <i>Subgoal (i)</i> or something like it!</p> \u000A \u000A <p>The Inductive Step for the conjecture above is</p> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp z)) \u000A              ; <i>Induction Hypothesis</i>: \u000A              (implies (true-listp (cdr z)) \u000A                       (equal (rev (rev (cdr z))) (cdr z)))) \u000A         ; <i>Induction Conclusion</i>: \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>Note that the Inductive Hypothesis is an implication:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))) (<see topic=\"COMMON-LISP____CDR\">cdr</see> z)))</code> \u000A \u000A <p>This hypothesis can be true two different ways.  The ``normal'' way -- the \u000A way everybody remembers -- is that <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))</v> is true and thus \u000A <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))) (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))</v> is true.  But the way many people \u000A forget is that <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> z))</v> is false.  You must prove the \u000A Induction Step even in this ``forgetable'' case.</p> \u000A \u000A <p>In this case, the Induction Step simplifies to</p> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp z)) \u000A              (not (true-listp (cdr z)))) \u000A         (implies (true-listp z) \u000A                  (equal (rev (rev z)) z))) \u000A</code> \u000A \u000A <p>By Promotion (see the list of tautologies in our discussion of <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see>) this is</p> \u000A \u000A <code><i>Induction Step'</i>: \u000A(implies (and (not (endp z)) \u000A              (not (true-listp (cdr z))) \u000A              (true-listp z)) \u000A         (equal (rev (rev z)) z)) \u000A</code> \u000A \u000A <p>Using the Contrapositive and rearranging the order of the hypotheses (see \u000A <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see> again), this is</p> \u000A \u000A <code><i>Induction Step''</i>: \u000A(implies (and (not (endp z)) \u000A              (true-listp z) \u000A              (not (equal (rev (rev z)) z))) \u000A         (true-listp (cdr z))) \u000A</code> \u000A \u000A <p>Notice that <i>Subgoal (i)</i> implies <i>Induction Step''</i>:</p> \u000A \u000A <code><i>Subgoal (i)</i>: \u000A(implies (and (not (endp z)) \u000A              (truelistp z)) \u000A         (truelistp (cdr z))) \u000A</code> \u000A \u000A <p>Every inductive proof of an implication raises a case like this.  If we \u000A denote the conjecture <v>(<see topic=\"ACL2____IMPLIES\">implies</see> p q)</v> as <v>p ---&gt; q</v>, then the Induction \u000A Step will look like this:</p> \u000A \u000A <code>  ( c  &amp;  (p'  ---&gt;  q'))\u000A---&gt;\u000A  (p ---&gt; q)</code> \u000A \u000A <p>where <v>c</v> is the test that determines the inductive step, (e.g., <v>(<see topic=\"COMMON-LISP____NOT\">not</see>\u000A (<see topic=\"COMMON-LISP____ENDP\">endp</see> z))</v>) and <v>p'</v> and <v>q'</v> are inductive instances of <v>p</v> and \u000A <v>q</v>.  Promotion produces</p> \u000A \u000A <code>  ( c  &amp; p &amp; (p'  ---&gt;  q'))\u000A---&gt;\u000A  q</code> \u000A \u000A <p>It is then very common to prove that <v>p</v> implies <v>p'</v>,</p> \u000A \u000A <code><i>(i)</i>: \u000A(c &amp; p) ---&gt; p' \u000A</code> \u000A \u000A <p>and then prove that <v>q'</v> implies <v>q</v>,</p> \u000A \u000A <code><i>(ii)</i>: \u000A(c &amp; p &amp; q') ---&gt; q \u000A</code> \u000A \u000A <p>These correspond exactly to our choices <i>Subgoal (i)</i> and <i>Subgoal \u000A (ii)</i>.</p> \u000A \u000A <p>It is sometimes helpful to remember this diagram:</p> \u000A \u000A <code>(c  &amp;  (p'  ---&gt;  q')\u000A        ^         |\u000A        |         |\u000A        |         v\u000A --&gt;   (p   ---&gt;  q )</code> \u000A \u000A <p>When you understand why <i>Subgoals (i)</i> and <i>(ii)</i> are sufficient, \u000A use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see> and go to question Q3.</p>"],

 "ACL2____IMPLIES":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Implies</v> is the ACL2 implication function.  <v>(<see topic=\"ACL2____IMPLIES\">implies</see> P Q)</v> \u000A means that either <v>P</v> is false (i.e., <v>nil</v>) or <v>Q</v> is true (i.e., \u000A not <v>nil</v>).</p> \u000A \u000A <p><b>Function: </b><srclink>implies</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> implies (p q)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> p (<see topic=\"COMMON-LISP____IF\">if</see> q t nil) t))</code>"],

 "ACL2____IGNORED-ATTACHMENT":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Attachments provide a way to execute constrained functions.  But in \u000A some cases, ACL2 will not permit such execution.  We discuss this issue \u000A briefly here.  For more information about attachments, see <see topic=\"ACL2____DEFATTACH\">defattach</see>.</p> \u000A \u000A <p>We illustrate this issue with the following example, discussed below.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"ACL2____DEFSTUB\">defstub</see> foo () t)\u000A (<see topic=\"ACL2____DEFN\">defn</see> foo-impl-1 () t)\u000A (<see topic=\"ACL2____DEFATTACH\">defattach</see> foo foo-impl-1)\u000A (<see topic=\"ACL2____DEFN\">defn</see> foo-impl-2 () nil)\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFATTACH\">defattach</see> foo foo-impl-2))\u000A (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac () (foo)) ; nil in the first pass, t in the second pass\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar () (mac))    ; nil in the first pass, t in the second pass\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> bar-is-nil\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (bar) nil))\u000A )</code> \u000A \u000A <p>Here, a non-executable function <v>foo</v> is introduced with no constraints, \u000A and is provided two contradictory implementations, <v>foo-impl-1</v> and \u000A <v>foo-impl-2</v>.  A function, <v>bar</v>, is defined using a macro, <v>mac</v>, \u000A whose expansion depends on which of <v>foo-impl-1</v> or <v>foo-impl-2</v> is \u000A attached to <v>foo</v>.  If ACL2 were to allow this, then as indicated by the \u000A comments above, <v>(bar)</v> would be defined to be <v>nil</v> on the first pass \u000A of the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form, where <v>foo</v> is attached to \u000A <v>foo-impl-2</v>; but <v>(bar)</v> would be defined to be <v>t</v> on the second \u000A pass, where <v>foo</v> is attached to <v>foo-impl-1</v> because the second <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> call is <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  Thus, after execution of the \u000A <v>encapsulate</v> form, <v>(bar)</v> would be provably equal to <v>t</v> even \u000A though there would be a theorem, <v>bar-is-nil</v> &mdash; proved during the \u000A first pass of the <v>encapsulate</v> &mdash; saying that <v>(bar)</v> is \u000A <v>nil</v>!</p> \u000A \u000A <p>Fortunately, ACL2 does not permit this to happen.  The example above \u000A produces the following output.</p> \u000A \u000A <code>ACL2 !&gt;&gt;(<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> BAR NIL (MAC))\u000A\u000AACL2 Error in ( DEFUN BAR ...):  In the attempt to macroexpand the\u000Aform (MAC), evaluation of the macro body caused the following error:\u000A\u000AACL2 cannot ev the call of undefined function FOO on argument list:\u000A\u000ANIL\u000A\u000ANote that because of logical considerations, attachments (including\u000AFOO-IMPL-2) must not be called in this context.</code> \u000A \u000A <p>We see, then, the importance of disallowing evaluation using attachments \u000A during macroexpansion.  ACL2 is careful to avoid attachments in situations, \u000A like this one, where using attachments could be unsound.</p> \u000A \u000A <p>We conclude with an example illustrating how <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> can be used \u000A to work around the refusal of ACL2 to use attachments during macroexpansion. \u000A The idea is that <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansions are stored, and this avoids the \u000A issue of <tt><see topic=\"ACL2____LOCAL\">local</see></tt> attachments.  In particular, for the example below, the \u000A second <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> affects the body of <v>f2</v> even though that <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> is <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, because the expansion of the corresponding <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> is saved during the first pass of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, when full \u000A admissibility checks are done.  Then even after including the book, the \u000A definition of <v>f2</v> will be based on the second (<tt><see topic=\"ACL2____LOCAL\">local</see></tt>) <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> form below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> body-1 (<see topic=\"ACL2____NAME\">name</see> formals body)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> name))\u000A  `(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> ,(<see topic=\"COMMON-LISP____CAR\">car</see> formals))\u000A       ,body\u000A     nil))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> body-2 (<see topic=\"ACL2____NAME\">name</see> formals body)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> name))\u000A  `(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> ,(<see topic=\"COMMON-LISP____CAR\">car</see> formals))\u000A       ,body\u000A     t))\u000A\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> defun+ (<see topic=\"ACL2____NAME\">name</see> formals body)\u000A  `(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (foo) ; attachable stub\u000A        '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ,name ,formals ,(body-1 name formals body))\u000A      '(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ,name ,formals ,(body-2 name formals body)))))\u000A\u000A;;; (defun+ f1 (x y) (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) ; fails because foo has no attachment\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> foo () t)\u000A(<see topic=\"ACL2____DEFN\">defn</see> foo-true () t)\u000A(<see topic=\"ACL2____DEFN\">defn</see> foo-false () nil)\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> foo foo-true)\u000A\u000A(defun+ f1 (x y) (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DEFATTACH\">defattach</see> foo foo-false))\u000A\u000A(defun+ f2 (x y) (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))\u000A\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f1 3 t) nil))\u000A\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f2 3 t) (<see topic=\"COMMON-LISP____CONS\">cons</see> 3 t)))</code>"],

 "ACL2____ER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____FMT\">fmt</see> for a general discussion of formatted printing in \u000A ACL2.  All calls of <v>er</v> print formatted strings, just as is done by <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____ER\">er</see> hard  'top-level &quot;Illegal inputs, ~x0 and ~x1.&quot; a b)\u000A(<see topic=\"ACL2____ER\">er</see> hard? 'top-level &quot;Illegal inputs, ~x0 and ~x1.&quot; a b)\u000A(<see topic=\"ACL2____ER\">er</see> soft  'top-level &quot;Illegal inputs, ~x0 and ~x1.&quot; a b)</code> \u000A \u000A <p>The examples above all print an error message to standard output saying \u000A that <v>a</v> and <v>b</v> are illegal inputs.  However, the first two abort \u000A evaluation after printing an error message (while logically returning \u000A <v>nil</v>, though in ordinary evaluation the return value is never seen); while \u000A the third returns <v>(<see topic=\"ACL2____MV\">mv</see> t nil state)</v> after printing an error message.  The \u000A result in the third case can be interpreted as an ``error'' when programming \u000A with the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, something most ACL2 users will probably not want \u000A to do unless they are building systems of some sort; see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.  If state is not available in the current context \u000A then you will probably want to use <v>(<see topic=\"ACL2____ER\">er</see> hard ...)</v> or <v>(<see topic=\"ACL2____ER\">er</see> hard? ...)</v> \u000A to cause an error; for example, if you are returning two values, you may write \u000A <v>(<see topic=\"ACL2____MV\">mv</see> (<see topic=\"ACL2____ER\">er</see> hard ...) nil)</v>.</p> \u000A \u000A <p>The difference between the <v>hard</v> and <v>hard?</v> forms is one of guards. \u000A Use <v>hard</v> if you want the call to generate a (clearly impossible) guard \u000A proof obligation of (essentially) <v>NIL</v>.  But use <v>hard?</v> if you want to \u000A be able to call this function in guard-verified code, since the call generates \u000A a (trivially satisfied) guard proof obligation of <v>T</v>.</p> \u000A \u000A <p><v>Er</v> is a macro, and the above three examples expand to calls of ACL2 \u000A functions, as shown below.  See <see topic=\"ACL2____ILLEGAL\">illegal</see>, see <see topic=\"ACL2____HARD-ERROR\">hard-error</see>, and see \u000A <see topic=\"ACL2____ERROR1\">error1</see>.  The first two have guards of (essentially) <v>NIL</v> and \u000A <v>T</v>, respectively, while <tt><see topic=\"ACL2____ERROR1\">error1</see></tt> is in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> \u000A mode.</p> \u000A \u000A <code>General forms:\u000A(<see topic=\"ACL2____ER\">er</see> hard  ctx fmt-string arg1 arg2 ... argk)\u000A  ==&gt; {macroexpands, in essence, to:}\u000A(<see topic=\"ACL2____ILLEGAL\">ILLEGAL</see>    CTX FMT-STRING\u000A            (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\0 ARG1) (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\1 ARG2) ... (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\k ARGk)))\u000A\u000A(<see topic=\"ACL2____ER\">er</see> hard? ctx fmt-string arg1 arg2 ... argk)\u000A  ==&gt; {macroexpands, in essence, to:}\u000A(<see topic=\"ACL2____HARD-ERROR\">HARD-ERROR</see> CTX FMT-STRING\u000A            (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\0 ARG1) (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\1 ARG2) ... (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\k ARGk)))\u000A\u000A(<see topic=\"ACL2____ER\">er</see> soft  ctx fmt-string arg1 arg2 ... argk)\u000A  ==&gt; {macroexpands, in essence, to:}\u000A(<see topic=\"ACL2____ERROR1\">ERROR1</see>     CTX FMT-STRING\u000A            (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\0 ARG1) (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\1 ARG2) ... (<see topic=\"COMMON-LISP____CONS\">CONS</see> #\\k ARGk)))</code>"],

 "ACL2____BOOK-CONTENTS":[["Books-tour"],"ACL2 Sources","ACL2","<code>Example Book:\u000A\u000A; This book defines my app function and the theorem that it is\u000A; associative.  One irrelevant help lemma is proved first but\u000A; it is local and so not seen by include-book.  I depend on the\u000A; inferior book &quot;weird-list-primitives&quot; from which I get\u000A; definitions of hd and tl.\u000A\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;MY-PKG&quot;)\u000A\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;weird-list-primitives&quot;)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (hd x) (app (tl x) y)) y))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> help-lemma\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app x nil) x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> app-is-associative\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c) (app a (app b c))))</code> \u000A \u000A <p>The first form in a book must be <v>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;pkg&quot;)</v> where \u000A <v>&quot;pkg&quot;</v> is some package name known to ACL2 whenever the book is \u000A certified.  The rest of the forms in a book are embedded event forms, i.e., \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>s, etc., some of which may be marked <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  See <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>.  The usual Common Lisp commenting \u000A conventions are provided.  Note that since a book consists of embedded event \u000A forms, we can talk about the ``<tt><see topic=\"ACL2____LOCAL\">local</see></tt>'' and ``non-local'' <see topic=\"ACL2____EVENTS\">events</see> \u000A of a book.</p> \u000A \u000A <p>Because <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> is not an embedded event form, the only <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> in a book is the initial one.  Because <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> is not an \u000A embedded event form, a book can never contain a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> form.  Because \u000A <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> is an embedded event form, <see topic=\"ACL2____BOOKS\">books</see> may contain \u000A references to other <see topic=\"ACL2____BOOKS\">books</see>.  This makes <see topic=\"ACL2____BOOKS\">books</see> structured \u000A objects.</p> \u000A \u000A <p>When the forms in a book are read from the file, they are read with <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> set to the package named in the <tt><see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see></tt> form at \u000A the top of the file.  The effect of this is that all symbols are <see topic=\"COMMON-LISP____INTERN\">intern</see>ed in that package, except those whose packages are given explicitly \u000A with the ``::'' notation.  For example, if a book begins with <v>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see>\u000A &quot;ACL2-X&quot;)</v> and then contains the form</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn (x)\u000A  (<see topic=\"COMMON-LISP____LIST\">acl2::list</see> 'car x))</code> \u000A \u000A <p>then <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <v>fn</v>, <v>x</v>, and <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> are all <see topic=\"COMMON-LISP____INTERN\">intern</see>ed in the <v>&quot;ACL2-X&quot;</v> package.  I.e., it is as though the following \u000A form were read instead:</p> \u000A \u000A <code>(acl2-x::defun acl2-x::fn (acl2-x::x)\u000A    (<see topic=\"COMMON-LISP____LIST\">acl2::list</see> 'acl2-x::car acl2-x::x)).</code> \u000A \u000A <p>Of course, <v>acl2-x::defun</v> would be the same symbol as <v>acl2::defun</v> \u000A if the <v>&quot;ACL2-X&quot;</v> package imported <v>acl2::defun</v>.</p> \u000A \u000A <p>If each book has its own unique package name and all the names defined \u000A within the book are in that package, then name clashes between <see topic=\"ACL2____BOOKS\">books</see> \u000A are completely avoided.  This permits the construction of useful logical <see topic=\"ACL2____WORLD\">world</see>s by the successive inclusion of many <see topic=\"ACL2____BOOKS\">books</see>.  Although it is \u000A often too much trouble to manage several packages, their judicious use is a \u000A way to minimize name clashes.  Often, a better way is to use <v>local</v>; see \u000A <see topic=\"ACL2____LOCAL\">local</see>.</p> \u000A \u000A <p>How does <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> know the definitions of the packages used in \u000A a book, since <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>s cannot be among the forms?  More generally, how \u000A do we know that the forms in a book will be admissible in the host logical \u000A <see topic=\"ACL2____WORLD\">world</see> of an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>?  See <see topic=\"ACL2____CERTIFICATE\">certificate</see> for answers \u000A to these questions.</p>"],

 "COMMON-LISP____NOT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Not</v> is the ACL2 negation function.  The negation of <v>nil</v> \u000A is <v>t</v> and the negation of anything else is <v>nil</v>.</p> \u000A \u000A <p><v>Not</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>not</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> not (p)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> p nil t))</code>"],

 "ACL2____SET-DEFAULT-HINTS":[["Default-hints"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> '((computed-hint-1 clause)\u000A                     (computed-hint-2 clause\u000A                                      stable-under-simplificationp)))\u000A(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> nil)</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs; see \u000A <see topic=\"ACL2____SET-DEFAULT-HINTS_12\">set-default-hints!</see> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> lst)</code> \u000A \u000A <p>where <v>lst</v> is a list.  Generally speaking, the elements of <v>lst</v> \u000A should be suitable for use as <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>.</p> \u000A \u000A <p>Whenever a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> or <tt><see topic=\"ACL2____THM\">thm</see></tt> command is executed, the default \u000A hints are appended to the right of any explicitly provided <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> \u000A in the command.  The same applies to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s as well (<v>:hints</v>, \u000A <v>:guard-hints</v>, and (for ACL2(r)) <v>:std-hints</v>).  The hints are then \u000A translated and processed just as though they had been explicitly included.</p> \u000A \u000A <p>Technically, we do not put restrictions on <v>lst</v>, beyond that it is a \u000A true list.  It would be legal to execute</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> '((&quot;Goal&quot; :use lemma23)))</code> \u000A \u000A <p>with the effect that the given hint is added to subsequent hints supplied \u000A explicitly.  An explicit \"Goal\" hint would, however, take priority, as \u000A suggested by the mention above of ``appended to the right.''</p> \u000A \u000A <p>Note that <v>set-default-hints</v> sets the default hints as specified.  To \u000A add to or remove from the current default, see <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> and \u000A see <see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see>.  To see the current default hints, see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p>Finally, note that the effects of <v>set-default-hints</v>, <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt>, and <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see></tt> are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A book in which they appear.  Thus, users who include a book with such forms \u000A will not have their default hints affected by such forms.  In order to export \u000A the effect of setting the default hints, use <tt><see topic=\"ACL2____SET-DEFAULT-HINTS_12\">set-default-hints!</see></tt>, \u000A <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS_12\">add-default-hints!</see></tt>, or <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS_12\">remove-default-hints!</see></tt>.</p> \u000A \u000A <p>For a related feature, which however is only for advanced system builders, \u000A see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>.</p>"],

 "ACL2____SHOW-BDD":[["Bdd"],"ACL2 Sources","ACL2","<p>Attempts to use BDDs (see <see topic=\"ACL2____BDD\">bdd</see>), using <v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt> \u000A <see topic=\"ACL2____HINTS\">hints</see>, can fail for various reasons.  Sometimes it is useful to explore \u000A such failures.  To do so, one may simply execute the form</p> \u000A \u000A <code>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)</code> \u000A \u000A <p>inside the ACL2 loop.  The system's response is generally self-explanatory. \u000A Perhaps you have already seen <v>show-bdd</v> used in some examples (see <see topic=\"ACL2____BDD-INTRODUCTION\">bdd-introduction</see> and see <see topic=\"ACL2____IF_A2\">if*</see>).  Here we give some details about \u000A <v>show-bdd</v>.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____SHOW-BDD\">Show-bdd</see>)</v> prints the goal to which the BDD procedure was applied and \u000A reports the number of nodes created during the <see topic=\"ACL2____BDD\">bdd</see> computation, \u000A followed by additional information depending on whether or not the computation \u000A ran to completion or aborted (for reasons explained elsewhere; see <see topic=\"ACL2____BDD-ALGORITHM\">bdd-algorithm</see>).  If the computation did abort, a backtrace is printed that \u000A should be useful in understanding where the problem lies.  Otherwise, \u000A <v>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see>)</v> prints out ``falsifying constraints.''  This list of pairs \u000A associates <see topic=\"ACL2____TERM\">term</see>s with values and suggests how to construct a binding \u000A list for the variables in the conjecture that will falsify the conjecture.  It \u000A also prints out the <see topic=\"ACL2____TERM\">term</see> that is the result of simplifying the input \u000A <see topic=\"ACL2____TERM\">term</see>.  In each of these cases, parts of the object may be hidden during \u000A printing, in order to avoid creating reams of uninteresting output.  If so, \u000A the user will be queried about whether he wishes to see the entire object \u000A (alist or <see topic=\"ACL2____TERM\">term</see>), which may be quite large.  The following responses are \u000A legal:</p> \u000A \u000A <blockquote><p><v> w</v> &mdash; Walk around the object with a structure \u000A editor</p> \u000A \u000A <p><v> t</v> &mdash; Print the object in full</p> \u000A \u000A <p><v>nil</v> &mdash; Do not print any more of the object</p></blockquote> \u000A \u000A <p><v>Show-bdd</v> actually has four optional arguments, probably rarely used. \u000A The general form is</p> \u000A \u000A <code>(<see topic=\"ACL2____SHOW-BDD\">show-bdd</see> goal-name goal-ans falsifying-ans term-ans)</code> \u000A \u000A <p>where <v>goal-name</v> is the name of the goal on which the <v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt> \u000A hint was used (or, <v>nil</v> if the system should find such a goal), \u000A <v>goal-ans</v> is the answer to be used in place of the query for whether to \u000A print the input goal in full, <v>falsifying-ans</v> is the answer to be used in \u000A place of the query for whether to print the falsifying constraints in full, \u000A and <v>term-ans</v> is the answer to be used in place of the query for whether \u000A to print the resulting <see topic=\"ACL2____TERM\">term</see> in full.</p>"],

 "ACL2____SET-MATCH-FREE-DEFAULT":[["Free-variables"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :once)\u000A(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :all)\u000A(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> nil)</code> \u000A \u000A <p>Note: This utility does not apply to <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules; for a \u000A related topic pertinent to such rules, see <see topic=\"ACL2____FREE-VARIABLES-TYPE-PRESCRIPTION\">free-variables-type-prescription</see>.</p> \u000A \u000A <p>As described elsewhere (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), a <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rule may have free variables in its \u000A hypotheses, and ACL2 can be directed either to try all bindings \u000A (``<v>:all</v>'') or just the first (``<v>:once</v>'') when relieving that \u000A hypothesis, as a basis for relieving subsequent hypotheses.  This directing of \u000A <v>:all</v> or <v>:once</v> is generally provided by specifying either \u000A <v>:match-free :once</v> or <v>:match-free :all</v> in the <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> of the rule.  If neither of these is specified, then the most \u000A recent <v>set-match-free-default</v> is used by ACL2 to fill in this missing \u000A <v>:match-free</v> field.  See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.  Except: If the last \u000A <v>set-match-free-default</v> specifies <v>nil</v>, then ACL2 reverts to the \u000A behavior it had at start-up, as described in Remarks (2) and (3) below.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It \u000A uses the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to \u000A the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs.</p> \u000A \u000A <p>Remarks.</p> \u000A \u000A <p>(1) The use of <v>set-match-free-default</v> has no effect on existing rules. \u000A In order to change the behavior of existing rules with respect to \u000A free-variable matching, see <see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see>.</p> \u000A \u000A <p>(2) If you submit a <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rule with a free variable in a hypothesis, and no default \u000A setting was previously specified with <v>set-match-free-default</v> or the \u000A default setting is <v>nil</v>, and the rule is not within a book being processed \u000A with <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, or <tt><see topic=\"ACL2____REBUILD\">rebuild</see></tt>, then a \u000A warning or error is caused.  In order to make this an error instead of a \u000A warning, see <see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see>.</p> \u000A \u000A <p>(3) If you submit a <see topic=\"ACL2____REWRITE\">rewrite</see>, <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rule with a free variable in a hypothesis, and no default \u000A setting has been previously specified with <v>set-match-free-default</v> or the \u000A default setting is <v>nil</v>, and no error is caused (see (2) above), then the \u000A default <v>:all</v> is used.</p>"],

 "ACL2____ASSIGN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> x (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 10))\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> a (<see topic=\"ACL2____ASET1\">aset1</see> 'ascii-map-array (<see topic=\"ACL2_____04\">@</see> a) 66 'Upper-case-B))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> symbol term)</code> \u000A \u000A <p>where <v>symbol</v> is any symbol (with certain enforced exclusions to avoid \u000A overwriting ACL2 system ``globals'') and <v>term</v> is any ACL2 term that could \u000A be evaluated at the top-level.  <v>Assign</v> evaluates the term, stores the \u000A result as the value of the given symbol in the <v>global-table</v> of <tt><see topic=\"ACL2____STATE\">state</see></tt>, and returns the result.  (Note: the actual implementation of the \u000A storage of this value is much more efficient than this discussion of the logic \u000A might suggest.)  <v>Assign</v> is a macro that effectively expands to the more \u000A complicated but understandable:</p> \u000A \u000A <code>(<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'symbol term state)\u000A        (<see topic=\"ACL2____MV\">mv</see> nil (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'symbol state) state)).</code> \u000A \u000A <p>The macro <v>f-put-global</v> is closely related to <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt>: \u000A <v>(<see topic=\"ACL2____ASSIGN\">assign</see> var val)</v> macroexpands to <v>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'var val state)</v>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2_____04\">@</see></tt> gives convenient access to the value of such globals. \u000A The <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> operation has no effect on the <v>global-table</v> of \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thus, you may use these globals to hang onto useful data \u000A structures even though you may undo back past where you computed and saved \u000A them.</p>"],

 "ACL2____ACL2_02Symbols":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Common Lisp's symbols are a data type representing words.  They are \u000A frequently regarded as atomic objects in the sense that they are not \u000A frequently broken down into their constituents.  Often the only important \u000A properties of symbols is that they are not numbers, characters, strings, or \u000A lists and that two symbols are not equal if they look different (!).  Examples \u000A of symbols include <v>PLUS</v> and <v>SMITH::ABC</v>.  All function and variable \u000A names in ACL2 are symbols.  When symbols are used as constants they must be \u000A quoted, as in <v>'PLUS</v>.</p> \u000A \u000A <p>The symbol <v>T</v> is commonly used as the Boolean ``true.''  The symbol \u000A <v>NIL</v> is commonly used both as the Boolean ``false'' and as the ``empty \u000A list.''  Despite sometimes being called the ``empty list'' <v>NIL</v> is a \u000A <b>symbol</b> not an ``empty cons.''  Unlike other symbols, <v>T</v> and \u000A <v>NIL</v> may be used as constants without quoting them.</p> \u000A \u000A <p>Usually, symbols are written as sequences of alphanumeric characters other \u000A than those denoting numbers.  Thus, <v>A12</v>, <v>+1A</v> and <v>1+</v> are symbols \u000A but <v>+12</v> is a number.  Roughly speaking, when symbols are read lower case \u000A characters are converted to upper case, so we frequently do not distinguish \u000A <v>ABC</v> from <v>Abc</v> or <v>abc</v>.  Click <see topic='ACL2____Conversion'>here</see> for information about case conversion when symbols \u000A are read.  However, any character can be used in a symbol, but some characters \u000A must be ``escaped'' to allow the Lisp reader to parse the sequence as a \u000A symbol.  For example, <v>|Abc|</v> is a symbol whose first character is \u000A capitalized and whose remaining characters are in lower case.  <v>|An odd\u000A duck|</v> is a symbol containing two #\\Space characters.  See any Common Lisp \u000A documentation for the syntactic rules for symbols.</p> \u000A \u000A <p>Technically, a symbol is a special kind of pair consisting of a package \u000A name (which is a string) and a symbol name (which is also a string).  (See \u000A <see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A see <see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.)  The symbol SMITH::ABC is said to be in package \u000A \"SMITH\" and to have the symbol name \"ABC\".  The symbol <v>ABC</v> in package \u000A \"SMITH\" is generally not equal to the symbol <v>ABC</v> in package \"JONES\". \u000A However, it is possible to ``import'' symbols from one package into another \u000A one, but in ACL2 this can only be done when the package is created.  (See \u000A <see topic=\"ACL2____DEFPKG\">defpkg</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.)  If the <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> is \u000A \"SMITH\" then <v>SMITH::ABC</v> may be more briefly written as just <v>ABC</v>. \u000A <tt><see topic=\"COMMON-LISP____INTERN\">Intern</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> ``creates'' a symbol of a given name in a given \u000A package.</p>"],

 "ACL2____SET-VERIFY-GUARDS-EAGERNESS":[["Guard"],"ACL2 Sources","ACL2","<code>Example Forms:                        try guard verification?\u000A(<see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> 0) ; no, unless :verify-guards t\u000A(<see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> 1) ; yes if a guard or type is supplied\u000A(<see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> 2) ; yes, unless :verify-guards nil</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-VERIFY-GUARDS-EAGERNESS\">set-verify-guards-eagerness</see> n)</code> \u000A \u000A <p>where <v>n</v> is a variable-free term that evaluates to <v>0</v>, <v>1</v>, or \u000A <v>2</v>.  This macro is essentially equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :verify-guards-eagerness n)</code> \u000A \u000A <p>and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  However, \u000A unlike the above simple call of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> event function (see <see topic=\"ACL2____TABLE\">table</see>), no output results from a <v>set-verify-guards-eagerness</v> event.</p> \u000A \u000A <p><v>Set-verify-guards-eagerness</v> may be thought of as an event that merely \u000A sets a flag to <v>0</v>, <v>1</v>, or <v>2</v>.  The flag is used by certain <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> to determine whether <see topic=\"ACL2____GUARD\">guard</see> verification is tried. \u000A The flag is irrelevant to those <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode and to those <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which an explicit \u000A <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> setting is provided among the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt>.  In \u000A the former case, <see topic=\"ACL2____GUARD\">guard</see> verification is not done because it can only be \u000A done when logical functions are being defined.  In the latter case, the \u000A explicit <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> setting determines whether <see topic=\"ACL2____GUARD\">guard</see> \u000A verification is tried.  So consider a <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A in which no <v>:</v><tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> setting is provided.  Is <see topic=\"ACL2____GUARD\">guard</see> \u000A verification tried?  The answer depends on the eagerness setting as follows. \u000A If the eagerness is <v>0</v>, <see topic=\"ACL2____GUARD\">guard</see> verification is not tried.  If the \u000A eagerness is <v>1</v>, it is tried if and only if a guard is explicitly \u000A specified in the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, in the following sense: there is an <v>xargs</v> \u000A keyword <v>:guard</v> or <v>:stobjs</v> or a <tt><see topic=\"COMMON-LISP____TYPE\">type</see></tt> declaration.  If the \u000A eagerness is <v>2</v>, <see topic=\"ACL2____GUARD\">guard</see> verification is tried.</p> \u000A \u000A <p>The default behavior of the system is as though the \u000A <v>:verify-guards-eagerness</v> is <v>1</v>.  The current behavior can be \u000A ascertained by evaluating the form <v>(<see topic=\"ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS\">default-verify-guards-eagerness</see> (w\u000A state))</v>.</p>"],

 "ACL2____OBSERVATION":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Here is a typical application of <v>observation</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">let</see> ((ctx 'top-level)\u000A             (<see topic=\"ACL2____NAME\">name</see> 'foo))\u000A         (<see topic=\"ACL2____OBSERVATION\">observation</see> ctx\u000A                      &quot;Skipping processing of name ~x0.&quot;\u000A                      name))\u000A\u000AACL2 Observation in TOP-LEVEL:  Skipping processing of name FOO.\u000A&lt;state&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p><v>Observation</v> prints an initial ``<v>ACL2 Observation...: </v>'', and then \u000A prints the indicated message using formatted printing (see <see topic=\"ACL2____FMT\">fmt</see>). \u000A Notice in the example above that evaluation of a call of <v>observation</v> \u000A returns <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Indeed, <v>observation</v> is actually a macro whose \u000A expansion takes and returns the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  A similar utility, \u000A <v>observation-cw</v>, is available that does not take or return <v>state</v>; \u000A rather, it returns <v>nil</v> as the suffix ``<v>cw</v>'' suggests that a \u000A ``comment window'' is the target of this printing, rather than the state.  For \u000A example:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">let</see> ((ctx 'top-level)\u000A             (<see topic=\"ACL2____NAME\">name</see> 'foo))\u000A         (<see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see> ctx\u000A                         &quot;Skipping processing of name ~x0.&quot;\u000A                         name))\u000A\u000AACL2 Observation in TOP-LEVEL:  Skipping processing of name FOO.\u000ANIL\u000AACL2 !&gt;</code> \u000A \u000A <p><v>Observation-cw</v> takes exactly the same arguments as <v>observation</v>, \u000A but <v>observation-cw</v> does its printing in a so-called ``wormhole''; see \u000A <see topic=\"ACL2____WORMHOLE\">wormhole</see>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____OBSERVATION\">observation</see>    ctx fmt-string fmt-arg1 fmt-arg2 ... fmt-argk)\u000A(<see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see> ctx fmt-string fmt-arg1 fmt-arg2 ... fmt-argk)</code> \u000A \u000A <p>where <v>ctx</v> generally evaluates to a symbol (but see below), and \u000A <v>fmt-string</v> together with the <v>fmt-argi</v> are suitable for passing to \u000A <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.  Output begins and ends with a newline.</p> \u000A \u000A <p>Recall from the example above that the output from a call of \u000A <v>observation</v> (or <v>observation-cw</v>) begins with ``<v>ACL2\u000A Observation</v>'' and additional characters ending in ``<v>: </v>'', for example \u000A ``<v> in TOP-LEVEL: </v>'', followed by formatted output produced from \u000A <v>fmt-string</v> with the given <v>fmt-argi</v>.  The characters printed \u000A immediately following the string ``<v>ACL2 Observation</v>'' depend on the value \u000A of <v>ctx</v>.  If <v>ctx</v> is <v>nil</v>, nothing is printed.  If <v>ctx</v> is a \u000A non-<v>nil</v> symbol, it is printed using <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive <v>~x</v>.  If \u000A <v>ctx</v> is a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is a symbol, formatted \u000A printing is applied to the string \"(~x0 ~x1 ...)\", where <v>#\\0</v> and \u000A <v>#\\1</v> are bound respectively to that car and cdr.  Otherwise, <v>ctx</v> is \u000A printed using <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive <v>~@</v>.</p> \u000A \u000A <p>We next discuss situations in which printing is inhibited for \u000A <v>observation</v> and <v>observation-cw</v>.  No printing is done when \u000A <v>observation</v> is among the inhibited output types; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.  Moreover, no printing is done by <v>observation</v> \u000A during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  If you want to avoid printing from \u000A <v>observation-cw</v> during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, then you need to manage that \u000A yourself.</p>"],

 "COMMON-LISP____EQ":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Eq</v> is the function for determining whether two objects are \u000A identical (i.e., have the exact same store address in the current von Neumann \u000A implementation of Common Lisp).  It is the same as <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> in the ACL2 \u000A logic.</p> \u000A \u000A <p><v>Eq</v> is a Common Lisp function.  In order to ensure conformance with \u000A Common Lisp, the ACL2 <see topic=\"ACL2____GUARD\">guard</see> on <v>eq</v> requires at least one of the \u000A arguments to <v>eq</v> to be a symbol.  Common Lisp guarantees that if <v>x</v> is \u000A a symbol, then <v>x</v> is <v>eq</v> to <v>y</v> if and only if <v>x</v> is <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> to <v>y</v>.  Thus, the ACL2 user should think of <v>eq</v> as nothing \u000A besides a fast means for checking <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> when one argument is known to \u000A be a symbol.  In particular, it is possible that an <v>eq</v> test will not even \u000A require the cost of a function call but will be as fast as a single machine \u000A instruction.</p> \u000A \u000A <p><b>Function: </b><srclink>eq</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> eq (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x) t (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y))))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))</code>"],

 "ACL2_____A2STANDARD-CO_A2":[["Io"],"ACL2 Sources","ACL2","<p>The value of the ACL2 constant <v>*standard-co*</v> is an open \u000A character output channel that is synonymous to Common Lisp's \u000A <v>*standard-output*</v>.</p> \u000A \u000A <p>ACL2 character output to <v>*standard-co*</v> will go to the stream named by \u000A Common Lisp's <v>*standard-output*</v>.  That is, by changing the setting of \u000A <v>*standard-output*</v> in raw Common Lisp you can change the actual \u000A destination of ACL2 output on the channel named by <v>*standard-co*</v>. \u000A Observe that this happens without changing the logical value of \u000A <v>*standard-co*</v> (which is some channel symbol).  Changing the setting of \u000A <v>*standard-output*</v> in raw Common Lisp essentially just changes the map \u000A that relates ACL2 to the physical world of terminals, files, etc.</p> \u000A \u000A <p>To see the value of this observation, consider the following.  Suppose you \u000A write an ACL2 function which does character output to the constant channel \u000A <v>*standard-co*</v>.  During testing you see that the output actually goes to \u000A your terminal.  Can you use the function to output to a file?  Yes, if you are \u000A willing to do a little work in raw Common Lisp: open a stream to the file in \u000A question, set <v>*standard-output*</v> to that stream, call your ACL2 function, \u000A and then close the stream and restore <v>*standard-output*</v> to its nominal \u000A value.  Similar observations can be made about the two ACL2 input channels, \u000A <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt> and <tt><see topic=\"ACL2_____A2STANDARD-CI_A2\">*standard-ci*</see></tt>, which are analogues of \u000A <v>*standard-input*</v>.</p> \u000A \u000A <p>Another reason you might have for wanting to change the actual streams \u000A associated with <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt> and <v>*standard-co*</v> is to drive the \u000A ACL2 top-level loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, on alternative input and output streams. \u000A This end can be accomplished easily within ACL2 by either calling <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A on the desired channels or file names or by resetting the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A global variables <v>'</v><tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt> and <v>'</v><tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> which \u000A are used by <tt><see topic=\"ACL2____LD\">ld</see></tt>.  See <see topic=\"ACL2____STANDARD-OI\">standard-oi</see> and see <see topic=\"ACL2____STANDARD-CO\">standard-co</see>.</p>"],

 "ACL2____ZP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ZP\">Zp</see> n)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____NFIX\">nfix</see> n) 0)</v> and \u000A is the preferred termination test for recursion down the natural \u000A numbers. <v>(<see topic=\"ACL2____ZP\">Zp</see> n)</v> returns <v>t</v> if <v>n</v> is <v>0</v> or not a natural \u000A number; it returns <v>nil</v> otherwise.  Thus, in the ACL2 logic \u000A (ignoring the issue of <see topic=\"ACL2____GUARD\">guard</see>s):</p> \u000A \u000A <code> n       (<see topic=\"ACL2____ZP\">zp</see> n)\u000A3         nil\u000A0         t\u000A-1        t\u000A5/2       t\u000A#c(1 3)   t\u000A'abc      t</code> \u000A \u000A <p><v>(<see topic=\"ACL2____ZP\">Zp</see> n)</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring <v>n</v> to be a natural number.</p> \u000A \u000A <p>For a discussion of the various idioms for testing against <v>0</v>, see \u000A <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>Zp</v> is typically used as the termination test in recursions down the \u000A natural numbers.  It has the advantage of ``coercing'' its argument to a \u000A natural and hence allows the definition to be admitted without an explicit \u000A type check in the body.  <see topic=\"ACL2____GUARD\">Guard</see> verification allows <v>zp</v> to be \u000A compiled as a direct <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>-comparision with <v>0</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>zp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> zp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0) t))</code>"],

 "ACL2____WITH-LOCAL-STOBJ":[["Stobj"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____STOBJ\">stobj</see> for an introduction to single-threaded \u000A  objects.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A st\u000A (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A         (compute-with-st x st)\u000A         result))</code> \u000A \u000A <p><v>With-local-stobj</v> can be thought of as a macro, where the example form \u000A above expands as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A        (<see topic=\"COMMON-LISP____LET\">let</see> ((st (create-st)))\u000A          (compute-with-st x st))\u000A        (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> st))\u000A        result)</code> \u000A \u000A <p>However, ACL2 expects you to use <v>with-local-stobj</v>, not its expansion. \u000A More precisely, stobj creator functions are not allowed except (implicitly) \u000A via <v>with-local-stobj</v> and in logic-only situations (like theorems and \u000A hints).  Moreover, neither <v>with-local-stobj</v> nor its expansions are legal \u000A when typed directly at the top-level loop.  See <see topic=\"ACL2____TOP-LEVEL\">top-level</see> for a way to \u000A use <v>with-local-stobj</v> in the top-level loop.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see> stobj-name mv-let-form)\u000A(<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see> stobj-name mv-let-form creator-name)</code> \u000A \u000A <p>where <v>stobj-name</v> is the name of a <see topic=\"ACL2____STOBJ\">stobj</see>, <v>mv-let-form</v> is a \u000A call of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>, and if <v>creator-name</v> is supplied then it should be \u000A the name of the creator function for <v>stobj-name</v>; see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>. \u000A For the example form above, its expansion would use <v>creator-name</v>, if \u000A supplied, in place of <v>create-st</v>.  Note that <v>stobj-name</v> must not be \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> (the ACL2 state), except in special situations probably of \u000A interest only to system developers; see <see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see>.</p> \u000A \u000A <p><v>With-local-stobj</v> can be useful when a stobj is used to memoize \u000A intermediate results during a computation, yet it is desired not to make the \u000A <v>stobj</v> a formal parameter for the function and its callers.</p> \u000A \u000A <p>ACL2 can reason about these ``local stobjs,'' and in particular about stobj \u000A creator functions.  For technical reasons, ACL2 will not allow you to enable \u000A the <v>:EXECUTABLE-COUNTERPART</v> <see topic=\"ACL2____RUNE\">rune</see> of a stobj creator function.</p> \u000A \u000A <p>Finally, here is a small example concocted in order to illustrate that \u000A <v>with-local-stobj</v> calls can be nested.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld1)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo ()\u000A  (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A   st ; Let us call this the ``outer binding of st''.\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (val10 val20 st)\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((st (update-fld1 10 st)))\u000A       ;; At this point the outer binding of st has fld1 = 10.\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((result (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A                      st ; Let us call this the ``inner binding of st''.\u000A                      (<see topic=\"ACL2____MV-LET\">mv-let</see> (val st)\u000A                        (<see topic=\"COMMON-LISP____LET\">let</see> ((st (update-fld1 20 st)))\u000A                          ;; Now fld1 = 20 for the inner binding of st.\u000A                          (<see topic=\"ACL2____MV\">mv</see> (fld1 st) st))\u000A                        val))))\u000A         ;; So result has been bound to 20 above, but here we are once again\u000A         ;; looking at the outer binding of st, where fld1 is still 10.\u000A         (<see topic=\"ACL2____MV\">mv</see> (fld1 st) result st)))\u000A     (<see topic=\"ACL2____MV\">mv</see> val10 val20))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo) (<see topic=\"ACL2____MV\">mv</see> 10 20))) ; succeeds</code>"],

 "ACL2____SET-INHIBIT-WARNINGS":[["Prover-output"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> &quot;theory&quot; &quot;use&quot;)</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs; see \u000A <see topic=\"ACL2____SET-INHIBIT-WARNINGS_12\">set-inhibit-warnings!</see> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event. \u000A Indeed, <v>(<see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> ...)</v> is equivalent to <v>(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____SET-INHIBIT-WARNINGS_12\">set-inhibit-warnings!</see> ...))</v>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> string1 string2 ...)</code> \u000A \u000A <p>where each string is considered without regard to case.  This macro is \u000A equivalent to <v>(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____TABLE\">table</see> inhibit-warnings-table nil 'lst :clear))</v>, \u000A where <v>lst</v> is the list of strings supplied.  This macro is an event (see \u000A <see topic=\"ACL2____TABLE\">table</see>), but no output results from a <v>set-inhibit-warnings</v> \u000A event.</p> \u000A \u000A <p>ACL2 prints warnings that may, from time to time, seem excessive to \u000A experienced users.  Each warning is ``labeled'' with a string identifying the \u000A type of warning.  Consider for example</p> \u000A \u000A <code>ACL2 Warning [Use] in ( THM ...):  It is unusual to :USE ....</code> \u000A \u000A <p>Here, the label is \"Use\".  The argument list for \u000A <v>set-inhibit-warnings</v> is a list of such labels, each of which is a string. \u000A Any warning is suppressed if its label is a member of this list, where case is \u000A ignored.  Thus, for example, the warning above will not be printed after a \u000A call of <v>set-inhibit-warnings</v> that contains the string, <v>&quot;Use&quot;</v> (or \u000A any string that is <tt><see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see></tt> to <v>&quot;Use&quot;</v>, such as <v>&quot;use&quot;</v> \u000A or <v>&quot;USE&quot;</v>).  In summary: the effect of this event is to suppress any \u000A warning whose label is a member of the given argument list, where case is \u000A ignored.</p> \u000A \u000A <p>The list of currently inhibited warnings is the list of keys in the <see topic=\"ACL2____TABLE\">table</see> named <v>inhibit-warnings-table</v>.  (The values in the table are \u000A irrelevant.)  One way to get that value is to get the result from evaluating \u000A the following form: <v>(table-alist 'inhibit-warnings-table (w state))</v>.  Of \u000A course, if warnings are inhibited overall &mdash; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> &mdash; then this value is entirely irrelevant.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING-REPEATEDLY":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>We assume you've read about ``<see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE'>instances</see>'' and picked up \u000A our basic terminology including the ideas of <i>matching</i> a <i>pattern</i> \u000A term to a <i>target</i> term, obtaining a <i>substitution</i> and how to \u000A <i>instantiate</i> a term with a substitution.  We use these notions below \u000A without further citation.</p> \u000A \u000A <p>In addition, we assume you've read about ``<see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING'>rewriting</see>'' where we \u000A introduced the idea of treating a theorem (axiom, definition, or lemma) as a \u000A <i>conditional rewrite</i> rule and replaced one term by an equivalent one \u000A provided we can <i>relieve the hypotheses</i>.</p> \u000A \u000A <p>Suppose you're trying to prove <i>formula1</i> and you transform it to \u000A <i>formula2</i> by rewriting.  What happened?</p> \u000A \u000A <code><i>formula1</i>: \u000A(implies (and (not (consp z)) \u000A              (true-listp z)) \u000A         (equal (rev (rev z)) z)) \u000A \u000A<i>formula2</i>: \u000A(implies (and (not (consp z)) \u000A              (equal z nil)) \u000A         (equal (rev (rev z)) z)) \u000A</code> \u000A \u000A <p>Evidently we replaced <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)</v> by <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> z nil)</v>.  But how \u000A did that happen?  What really happened was the sequential application of \u000A several unconditional rewrites and the use of replacement of equals by \u000A equals.</p> \u000A \u000A <p>The definition of <v>true-listp</v> is:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> true-listp (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))).</code> \u000A \u000A <p>By rewriting once with the definition of <v>true-listp</v>, we transform \u000A <i>formula1</i> to:</p> \u000A \u000A <code><i>formula1'</i>: \u000A(implies (and (not (consp z)) \u000A              (if (consp z) \u000A                  (true-listp (cdr z)) \u000A                  (equal z nil))) \u000A         (equal (rev (rev z)) z)). \u000A</code> \u000A \u000A <p>Note how the call of <v>true-listp</v> has been replaced by the entire body \u000A of <v>true-listp</v>.</p> \u000A \u000A <p>Next, note that the first hypothesis above is that <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> z)</v> is false. \u000A That is, <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z))</v> is the same as <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> z) nil)</v>. \u000A Thus, replacement of equals by equals means we can transform <i>formula1'</i> \u000A to</p> \u000A \u000A <code><i>formula1''</i>: \u000A(implies (and (not (consp z)) \u000A              (if nil \u000A                  (true-listp (cdr z)) \u000A                  (equal z nil))) \u000A         (equal (rev (rev z)) z)). \u000A</code> \u000A \u000A <p>(We will explore replacement of equals by equals later.)</p> \u000A \u000A <p>Furthermore, we know the basic axiom about <v>if</v>:</p> \u000A \u000A <code><i>Axiom</i> if-nil: \u000A(if nil x y) = y. \u000A</code> \u000A \u000A <p>Rewriting with this particular axiom, using <v>(<see topic=\"COMMON-LISP____IF\">if</see> nil x y)</v> as the \u000A pattern and <v>y</v> as the replacement, will transform <i>formula1''</i> to</p> \u000A \u000A <code><i>formula2</i>: \u000A(implies (and (not (consp z)) \u000A              (equal z nil)) \u000A         (equal (rev (rev z)) z)). \u000A</code> \u000A \u000A <p>Often when we say we derived one formula from another by ``expansion'' and \u000A or by ``rewriting'' we take many rewrite steps, as here.  We typically use \u000A hypotheses of the formula without noting ``replacement of equals by equals'' \u000A as when we replaced <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> z)</v> by <v>nil</v>, and we typically omit to \u000A mention the use of basic axioms like <v>if-nil</v> above.</p> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the example proof in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____AREF1":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____AREF1\">aref1</see> 'delta1 a (<see topic=\"COMMON-LISP_____B2\">+</see> i k))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____AREF1\">aref1</see> name alist index)</code> \u000A \u000A <p>where <v>name</v> is a symbol, <v>alist</v> is a 1-dimensional array and \u000A <v>index</v> is a legal index into <v>alist</v>.  This function returns the value \u000A associated with <v>index</v> in <v>alist</v>, or else the default value of the \u000A array.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.</p> \u000A \u000A <p>This function executes in virtually constant time if <v>alist</v> is in fact \u000A the ``semantic value'' associated with <v>name</v> (see <see topic=\"ACL2____ARRAYS\">arrays</see>).  When it \u000A is not, <v>aref1</v> must do a linear search through <v>alist</v>.  In that case \u000A the correct answer is returned but a <b>slow array</b> comment is printed to \u000A the comment window.  See <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>aref1</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     aref1 (<see topic=\"ACL2____NAME\">name</see> l n)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l)\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))))))\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> n :header)) (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> n l))))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> x) (<see topic=\"ACL2____DEFAULT\">default</see> name l))\u000A                (t (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code>"],

 "ACL2____VERIFY-GUARDS_B2":[["Events"],"ACL2 Sources","ACL2","<p>We assume familiarity with <see topic=\"ACL2____GUARD\">guard</see> verification; see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.  Unlike <v>verify-guards</v>, the macro call <v>(<see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see>\u000A mac ...)</v> will verify guards for a function, <v>fn</v>, such that the macro \u000A <v>mac</v> is associated with the function symbol <v>fn</v> in <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt> \u000A (also see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>).  For example, if you define a macro \u000A <v>app</v> and list append function <v>binary-app</v>, and you associate macro \u000A <v>app</v> with function symbol <v>binary-app</v> in <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>, \u000A then evaluation of the form <v>(verify-guard+ app)</v> will have the effect of \u000A evaluating <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fn)</v>.  Note that in this setting, evaluation of \u000A <v>(verify-guard app)</v> would cause an error, because <v>app</v> is a macro and \u000A <v>verify-guards</v>, unlike <v>verify-guards+</v>, cannot handle macros.</p> \u000A \u000A <p>The rest of this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic discusses why we do not simply \u000A arrange that <v>verify-guards</v> be permitted to take a macro alias.  The \u000A following example shows a soundness issue in doing so.</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                   :verify-guards nil))\u000A   (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                   :verify-guards nil))\u000A   (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A (<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac (x)\u000A   x)\u000A (<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> mac f2) ; silly macro alias ;\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> mac f1)) ; alternate silly macro alias ;\u000A (<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> mac))</code> \u000A \u000A <p>If we were to allow macro aliases in <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, this event \u000A would be admitted, because on the first pass we are verifying guards of \u000A <v>f1</v>.  But after the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form completes evaluation, it \u000A would appear that <v>f2</v> is guard-verified.  That could of course cause a raw \u000A Lisp error.</p> \u000A \u000A <p>The enhanced functionality provided by <v>verify-guards+</v> does not have \u000A the above problem, because it takes advantage of <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> to avoid \u000A taking advantage of the contradictory results produced by the two calls of \u000A <v>add-macro-alias</v>.  See <see topic=\"ACL2____MAKE-EVENT\">make-event</see>.  If the specific example above \u000A is modified by replacing <v>verify-guards</v> with <v>verify-guards+</v>, then the \u000A first pass through the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form will expand the form \u000A <v>(<see topic=\"ACL2____VERIFY-GUARDS_B2\">verify-guards+</see> mac)</v> to <v>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> f1)</v>.  That same expansion \u000A will be used for the <v>verify-guards+</v> call during the second pass through \u000A the <v>encapsulate</v> form, which is evaluated successfully and leaves us in a \u000A <see topic=\"ACL2____WORLD\">world</see> where <v>f1</v> is guard-verified and <v>f2</v> is not.</p>"],

 "ACL2____PE":[["History"],"ACL2 Sources","ACL2","<code>Example:\u000A:pe fn   ; sketches the command that introduced fn and\u000A         ; prints in full the event within it that created fn.</code> \u000A \u000A <p>See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p> \u000A \u000A <p><v>Pe</v> takes one argument, a logical name, and prints in full the event \u000A corresponding to the name.  <v>Pe</v> also sketches the <see topic=\"ACL2____COMMAND\">command</see> \u000A responsible for that event if the <see topic=\"ACL2____COMMAND\">command</see> is different from the event \u000A itself.  See <see topic=\"ACL2____PC\">pc</see> for a description of the format used to display a <see topic=\"ACL2____COMMAND\">command</see>.  To remind you that the event is inferior to the <see topic=\"ACL2____COMMAND\">command</see>, \u000A i.e., you can only undo the entire <see topic=\"ACL2____COMMAND\">command</see>, not just the event, the \u000A event is indented slightly from the <see topic=\"ACL2____COMMAND\">command</see> and a slash (meant to \u000A suggest a tree branch) connects them.</p> \u000A \u000A <p>If the given logical name corresponds to more than one event, then <v>:pe</v> \u000A will print the above information for every such event.  Here is an example of \u000A such behavior.</p> \u000A \u000A <code>ACL2 !&gt;:pe nth\u000A      -4270  (<see topic=\"ACL2____ENCAPSULATE\">ENCAPSULATE</see> NIL ...)\u000A             \\\u000A&gt;V            (<see topic=\"ACL2____VERIFY-TERMINATION\">VERIFY-TERMINATION</see> NTH)\u000A\u000AAdditional events for the logical name NTH:\u000A PV   -4949  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> NTH (N L)\u000A                    &quot;Documentation available via :doc&quot;\u000A                    (<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (<see topic=\"ACL2____XARGS\">XARGS</see> :GUARD (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> N)\u000A                                                (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> N 0)\u000A                                                (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> L))))\u000A                    (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> L)\u000A                        NIL\u000A                        (<see topic=\"COMMON-LISP____IF\">IF</see> (<see topic=\"ACL2____ZP\">ZP</see> N)\u000A                            (<see topic=\"COMMON-LISP____CAR\">CAR</see> L)\u000A                            (<see topic=\"COMMON-LISP____NTH\">NTH</see> (<see topic=\"COMMON-LISP____-\">-</see> N 1) (<see topic=\"COMMON-LISP____CDR\">CDR</see> L)))))\u000AACL2 !&gt;</code> \u000A \u000A <p>If you prefer to see only the formula for the given name, for example if it \u000A is part of a large <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>, see <see topic=\"ACL2____PF\">pf</see>.</p>"],

 "ACL2____NOTE-2-8-OTHER":[["Note-2-8"],"ACL2 Sources","ACL2","<p>Execution of <tt><see topic=\"ACL2____TABLE\">table</see></tt> events has been sped up in many cases by \u000A avoiding excessive consing.</p> \u000A \u000A <p>ACL2 now warns if <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> (or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>) rules \u000A contain free variables on the right-hand side.  Thanks to Dave Greve for \u000A raising this issue.</p> \u000A \u000A <p>Emacs file <v>emacs/emacs-acl2.el</v> has been updated to better comprehend \u000A the notion of the ``ACL2 shell'', which is the buffer to which ACL2 forms are \u000A written by commands defined in the above file.  Thus, command <v>control-t e</v> \u000A has been modified always to write to the ACL2 shell (which is <v>&quot;*shell*&quot;</v> \u000A by default), and the following new commands have been defined.</p> \u000A \u000A <blockquote><p>o control-t c<br></br> \u000A \u000A   Set the ACL2 shell to the current buffer. \u000A o control-t b<br></br> \u000A \u000A   Change to the ACL2 shell.</p></blockquote> \u000A \u000A <p>The commands <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> and <v>:</v><tt><see topic=\"ACL2____PR\">pr</see></tt> may now be given a macro \u000A name that corresponds via the <v>macro-aliases-table</v> to a function name, so \u000A that for example <v>:pl append</v> is treated the same as <v>:pl\u000A binary-append</v>.  A more interesting improvement, for <v>:pl</v> only, is that \u000A <v>:pl</v> may now take any term.  When <v>:pl</v> is given a term other than a \u000A symbol, it will print all rewrite rules that match that term.  Thanks to David \u000A Russinoff, Robert Krug, and Bill Legato for getting this going.</p> \u000A \u000A <p>A new function, <tt><see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see></tt>, returns a symbol in the given \u000A package.</p> \u000A \u000A <p>The installation instructions have been updated, for example to give more \u000A guidance on obtaining Lisp implementations and to mention the acl2-help \u000A mailing list.</p> \u000A \u000A <p>Jared Davis has suggested some symbols to be added to <v>*acl2-exports*</v>, \u000A and we have done so.  Thanks, Jared.</p> \u000A \u000A <blockquote><p>o <v>MFC</v> (used in <tt><see topic=\"ACL2____SYNTAXP\">syntaxp</see></tt> and <tt><see topic=\"ACL2____EXTENDED-METAFUNCTIONS\">extended-metafunctions</see></tt>; thanks also to Robert Krug for this one) o <v>ID</v>, \u000A <v>CLAUSE</v>, <v>WORLD</v>, and <v>STABLE-UNDER-SIMPLIFICATIONP</v> (used in <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>) o <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt></p></blockquote> \u000A \u000A <p>The command <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> has been improved so that when the event is \u000A inside an included book, the path of included books (from the top-level book \u000A down to the one containing the event) is shown.  Thanks to Eric Smith (perhaps \u000A among others) for pointing out the utility of this improvement.</p> \u000A \u000A <p>A new release of the rtl library has been included: <v>books/rtl/rel4/</v>. \u000A See the <v>README</v> file in that directory.</p>"],

 "ACL2____GET-INTERNAL-TIME":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>The ACL2 system provides utilities that deal with elapsed time. \u000A The most visible of these is in the time summaries printed when completing \u000A evaluation of <see topic=\"ACL2____EVENTS\">events</see>.  For others, see <see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see>, \u000A see <see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see>, see <see topic=\"ACL2____TIME-TRACKER\">time-tracker</see>, see <see topic=\"ACL2____TIME-TRACKER-TAU\">time-tracker-tau</see>, and see <see topic=\"ACL2____PSTACK\">pstack</see>.</p> \u000A \u000A <p>By default, these utilities all use an underlying notion of run time \u000A provided by the host Common Lisp implementation: specifically, the Common Lisp \u000A function <v>get-internal-run-time</v>.  However, Common Lisp also provides the \u000A function <v>get-internal-real-time</v>, which returns the real time (wall clock \u000A time).  While the latter is specified to measure elapsed time, the former is \u000A left to the implementation, which might well only measure time spent in the \u000A Lisp process.  Consider the following example, which is a bit arcane but \u000A basically sleeps for 2 seconds.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> t) ; to allow sys-call\u000A(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____SYS-CALL\">sys-call</see> &quot;sleep&quot; '(&quot;2&quot;))\u000A         (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))</code> \u000A \u000A <p>A typical time summary might be as follows, drastically under-reporting the \u000A elapsed time.</p> \u000A \u000A <code>Time:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)</code> \u000A \u000A <p>However, you can instruct ACL2 to switch to using elapsed time (real time), \u000A in summaries and elsewhere, by evaluating the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> get-internal-time-as-realtime t)</code> \u000A \u000A <p>To return to using runtime:</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> get-internal-time-as-realtime nil)</code> \u000A \u000A <p>While the above example is rather silly, the issue becomes significant in \u000A time summaries for proofs that call out to external tools (see <see topic=\"ACL2____SYS-CALL\">sys-call</see> \u000A and see <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>).</p> \u000A \u000A <p>Note that a function <v>get-internal-time</v> is defined in raw Lisp but is \u000A not available inside the ACL2 loop.  However, the expression <v>(<see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see>\u000A state)</v> provides an interface to this function that is available inside the \u000A ACL2 loop; see <see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see>.</p> \u000A \u000A <p>We are open to changing the default to elapsed wall-clock time (realtime), \u000A and may do so in future ACL2 releases.</p> \u000A \u000A <p>Implementation note (GCL only): If the host Lisp is Gnu Common Lisp, then \u000A <v>get-internal-run-time</v> has a multiple value return, and the first two \u000A values (runtime and child runtime) are added together to produce a result for \u000A <v>get-internal-time</v>.</p>"],

 "ACL2____DEFLABEL":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____DEFLABEL\">deflabel</see> interp-section\u000A   :doc\u000A   &quot;:Doc-Section ...&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFLABEL\">deflabel</see> name :doc doc-string)</code> \u000A \u000A <p>where <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>) and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string (see <see topic=\"ACL2____DOC-STRING\">doc-string</see>).  This event adds the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string for symbol \u000A <v>name</v> to the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> database.  By virtue of the fact that \u000A <v>deflabel</v> is an event, it also marks the current <see topic=\"ACL2____HISTORY\">history</see> with the \u000A <v>name</v>.  Thus, even undocumented labels are convenient as landmarks in a \u000A proof development.  For example, you may wish to undo back through some label \u000A or compute a theory expression (see <see topic=\"ACL2____THEORIES\">theories</see>) in terms of some labels. \u000A <v>Deflabel</v> <see topic=\"ACL2____EVENTS\">events</see> are never considered redundant.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFDOC\">defdoc</see> for a means of attaching a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string to \u000A a name without marking the current <see topic=\"ACL2____HISTORY\">history</see> with that name.</p>"],

 "ACL2____COMMUNITY-BOOKS":[["Books"],"ACL2 Sources","ACL2","<p>ACL2 <see topic=\"ACL2____BOOKS\">books</see> are files of ACL2 <see topic=\"ACL2____EVENTS\">events</see> like definitions \u000A and theorems.</p> \u000A \u000A <p>The ACL2 <b>Community Books</b> are the canonical set of open-source books \u000A for ACL2, developed since the early 1990s by members of the ACL2 community. \u000A They include libraries for reasoning in many domains, macro libraries for more \u000A quickly writing and documenting code, interfacing tools for connecting ACL2 to \u000A other systems, productivity tools for better proof automation and debugging, \u000A and specialty libraries for areas like hardware verification.</p> \u000A \u000A <p>From the <a href='http://acl2-books.googlecode.com/'>ACL2 Books</a> web \u000A site you can:</p> \u000A \u000A <ul> \u000A <li>Download the Community Books;</li> \u000A <li>Learn how to contribute books to the ACL2 community; and</li> \u000A <li>Obtain updates between ACL2 releases.</li> \u000A </ul> \u000A \u000A <p>Once you have downloaded the Community Books, you should typically:</p> \u000A \u000A <ul> \u000A  <li>Extract them into the <v>books/</v> subdirectory of your ACL2 \u000A      installation, and </li> \u000A  <li>Certify them by following the instructions \u000A      in <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see>.</li> \u000A </ul>"],

 "COMMON-LISP____TRACE":[["Debugging"],"ACL2 Sources","ACL2","<p>ACL2 provides a trace utility, <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>, with corresponding \u000A reverse operation <tt><see topic=\"ACL2____UNTRACE_42\">untrace$</see></tt>.  These can be used without any dependence \u000A on the underlying Lisp utility, and are the tracing utilities of choice in \u000A ACL2; see <see topic=\"ACL2____TRACE_42\">trace$</see> and see <see topic=\"ACL2____UNTRACE_42\">untrace$</see>.</p> \u000A \u000A <p>However, for advanced users we note that the underlying host Lisp may also \u000A provide a trace utility, <v>trace</v>, and corresponding <v>untrace</v>. \u000A Moreover, these have been modified in the case that the host Lisp is GCL, \u000A Allegro CL, or CCL (OpenMCL), to provide limited support for <v>:entry</v>, \u000A <v>:exit</v>, and perhaps <v>:cond</v> keywords, to hide certain large data \u000A structures (world, enabled structure, rewrite constant), and to trace \u000A executable counterparts.  See source files <v>*-trace.lisp</v>.  For the above \u000A Lisps, you can invoke the original <v>trace</v> and <v>untrace</v> by invoking \u000A <v>old-trace</v> and <v>old-untrace</v>, respectively, in raw Lisp rather than in \u000A the normal ACL2 loop.</p>"],

 "COMMON-LISP____REMOVE-DUPLICATES":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> x)\u000A(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> x :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> x :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> x :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">Remove-duplicates</see> x)</v> returns the result of deleting duplicate \u000A elements from the beginning of the list or string <v>x</v>.  For example, \u000A <v>(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> '(1 2 3 2 4))</v> is equal to <v>'(1 3 2 4)</v>.  The \u000A optional keyword, <v>:TEST</v>, has no effect logically, but provides the test \u000A (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with successive elements of \u000A <v>lst</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>remove-duplicates</v> depends on the test. \u000A In all cases, the argument must satisfy <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> or <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>. \u000A If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then the argument must satisfy either <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> or <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then the \u000A argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>The relation between <v>remove-duplicates</v> and its variants is related to \u000A the usual pattern for equality variants; see <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see>. \u000A However, the possibility of a string argument changes the usual pattern a bit. \u000A As one might expect:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____REMOVE-DUPLICATES-EQ\">remove-duplicates-eq</see> lst)</v> is equivalent to \u000A <v>(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> lst :test 'eq)</v>.</p></blockquote> \u000A \u000A <p>However, <v>remove-duplicates-equal</v> is defined without consideration of \u000A strings, for backward compatibility with versions of ACL2 through Version_4.2. \u000A The macro <v>remove-duplicates-logic</v> has been introduced to model the \u000A behavior of <v>remove-duplicates</v> even on strings; use <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> if \u000A you wish to see its definition.  So we can say the following.</p> \u000A \u000A <blockquote><p><v>(remove-duplicates-logic lst)</v> is equivalent to \u000A <v>(<see topic=\"COMMON-LISP____REMOVE-DUPLICATES\">remove-duplicates</see> lst :test 'equal)</v>; and</p> \u000A \u000A <p><v>(remove-duplicates-logic lst)</v> is equal to <v>(<see topic=\"ACL2____REMOVE-DUPLICATES-EQUAL\">remove-duplicates-equal</see>\u000A lst)</v> when <v>lst</v> is not a string.</p></blockquote> \u000A \u000A <p>In particular, when the argument is not a string, reasoning about any of \u000A these primitives reduces to reasoning about the function \u000A <v>remove-duplicates-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>remove-duplicates-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> remove-duplicates-equal (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l) nil)\u000A             ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))\u000A              (<see topic=\"ACL2____REMOVE-DUPLICATES-EQUAL\">remove-duplicates-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l)\u000A                      (<see topic=\"ACL2____REMOVE-DUPLICATES-EQUAL\">remove-duplicates-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code> \u000A \u000A <p><v>Remove-duplicates</v> is defined by Common Lisp.  See any Common Lisp \u000A documentation for more information.</p>"],

 "ACL2____SYS-CALL_B2":[["Sys-call","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A; The following returns (<see topic=\"ACL2____MV\">mv</see> nil s state), where s is the standard output\u000A; from the command: ls -l ./\u000A(<see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see> &quot;ls&quot; '(&quot;-l&quot; &quot;./&quot;) state)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see> cmd args state)</code> \u000A \u000A <p>where <v>cmd</v> is a command to the host operating system and <v>args</v> is a \u000A list of strings.  Also see <see topic=\"ACL2____SYS-CALL\">sys-call</see>; but there are two differences \u000A between <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> and <v>sys-call+</v>.  First, the latter takes an extra \u000A argument, <v>state</v>.  Second, while <v>sys-call</v> returns <v>nil</v>, \u000A <v>sys-call+</v> returns three values: a so-called error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>), <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>.  While execution returns values as \u000A described just below, further below we explain the logical return values.  In \u000A the following, please keep in mind that the exact behavior depends on the \u000A platform; the description is only a guide.  For example, on some platforms \u000A <v>erp</v> might always be <v>nil</v>, even if in the error case, and <v>val</v> \u000A might or might not include error output.  (For details, look at the ACL2 \u000A source code for function <v>system-call+</v>, whose output is converted by \u000A replacing an <v>erp</v> of <v>nil</v> by 0.)</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>Erp</v> is either <v>nil</v> or a non-zero integer.  Normally, <v>nil</v> \u000A indicates that the command ran without error, and otherwise <v>erp</v> is the \u000A exit status.</p> \u000A \u000A <p><v>Val</v> is a string, typically the output generated by the call of \u000A <v>cmd</v>.</p> \u000A \u000A <p><v>State</v> is an ACL2 <see topic=\"ACL2____STATE\">state</see>.</p></blockquote> \u000A \u000A <p>While the description above pertains to the values returned by executing \u000A <v>sys-call+</v>, the logical values are as follows.  For a discussion of the \u000A <v>acl2-oracle</v> field of an ACL2 state, see <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt> and \u000A <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>Erp</v> is the first element of the <v>acl2-oracle</v> of the input state if \u000A that element is a nonzero integer, and otherwise is <v>nil</v>.</p> \u000A \u000A <p><v>Val</v> is the second element of the <v>acl2-oracle</v> of the input state \u000A if it is a string, else the empty string, <v>&quot;&quot;</v>.</p> \u000A \u000A <p><v>State</v> is the result of popping the <v>acl2-oracle</v> field twice from \u000A the input state.</p></blockquote> \u000A \u000A <p>Note that unlike <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt>, a call of <tt><see topic=\"ACL2____SYS-CALL_B2\">sys-call+</see></tt> has no \u000A effect on subsequent calls of <tt><see topic=\"ACL2____SYS-CALL-STATUS\">sys-call-status</see></tt>.</p> \u000A \u000A <p>As is the case for <v>sys-call</v>, a trust tag is required to call \u000A <v>sys-call+</v>.  For discussion of this and more, see <see topic=\"ACL2____SYS-CALL\">sys-call</see>.</p>"],

 "ACL2____GUARD-INTRODUCTION":[["Guard"],"ACL2 Sources","ACL2","<p>Most users can probably profit by avoiding dealing with guards most \u000A of the time.  If they seem to get in the way, they can be ``turned off'' using \u000A the command <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> <v>nil</v>; for more about this, \u000A see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  For more about guards in general, see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p>The guard on a function symbol is a formula about the formals of the \u000A function.  To see the guard on a function, use the keyword command \u000A <v>:</v><tt><see topic=\"ACL2____ARGS\">args</see></tt>.  See <see topic=\"ACL2____ARGS\">args</see>.  To specify the guard on a function at \u000A <v>defun-time</v>, use the <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> <v>xarg</v> (See <see topic=\"ACL2____XARGS\">xargs</see>) or \u000A <v>type</v> declarations (see <see topic=\"COMMON-LISP____DECLARE\">declare</see>).  The latter may be preferable if \u000A you want the host Lisp compiler to use this information.</p> \u000A \u000A <p>Guards can be seen as having either of two roles: (a) they are a \u000A specification device allowing you to characterize the kinds of inputs a \u000A function ``should'' have, or (b) they are an efficiency device allowing \u000A logically defined functions to be executed directly in Common Lisp.  Briefly: \u000A If the guards of a function definition are ``verified'' (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>), then the evaluation of a call of that function on arguments \u000A satisfying its guard will have the following property:</p> \u000A \u000A <blockquote><p>All subsequent function calls during that evaluation will be on \u000A arguments satisfying the guard of the called function.</p></blockquote> \u000A \u000A <p>The consequence of this fact for (a) is that your specification function is \u000A well-formed, in the sense that the values returned by this function on \u000A appropriate arguments only depend on the restrictions of the called functions \u000A to their intended domains.  The consequence of this fact for (b) is that in \u000A the ACL2 system, when a function whose guards have been verified is called on \u000A arguments that satisfy its guard, then the raw lisp function defined by this \u000A function's <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> event is used to evaluate the call.  Note however \u000A that even when the user-supplied <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is not used, ACL2 uses a \u000A corresponding ``executable counterpart'' that generally performs, we expect, \u000A nearly as well as the raw lisp function.  See <see topic=\"ACL2____COMP\">comp</see> to see how <see topic=\"ACL2____COMPILATION\">compilation</see> can speed up both kinds of execution.</p> \u000A \u000A <p>Let us turn next to the issue of the relationship between guards and \u000A evaluation.  See <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>.</p>"],

 "ACL2____GUARD-EVALUATION-EXAMPLES-SCRIPT":[["Guard"],"ACL2 Sources","ACL2","<p>Below is a script that illustrates the combination of <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s &mdash; <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode \u000A without <see topic=\"ACL2____GUARD\">guard</see>s verified, and <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode with <see topic=\"ACL2____GUARD\">guard</see>s verified &mdash; with values from <tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt> &mdash; \u000A <v>t</v> (the default), <v>:all</v>, <v>:none</v>, and <v>nil</v>.  (It does not \u000A illustrate the value <v>:nowarn</v>, which is the same as <v>t</v> except for \u000A inhibiting a warning.)  The script also illustrates cases where the guard is \u000A not, or is, <v>t</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____GUARD-EVALUATION-EXAMPLES-LOG\">guard-evaluation-examples-log</see> for result of running this script. \u000A Before presenting the script below, we give some instructions in case you want \u000A to run it yourself.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> for discussion of the interaction between \u000A <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>s and <see topic=\"ACL2____GUARD\">guard</see>-checking that is illustrated by this \u000A script.  Also see <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see> for a succinct table, with \u000A associated discussion, that covers in detail the interactions illustrated \u000A here.</p> \u000A \u000A <p>The script mentions the running of ``Tracing Code''.  The code is the \u000A following sequence of commands.</p> \u000A \u000A <code>(<see topic=\"ACL2____TRACE_42\">trace$</see> fact)\u000A:set-guard-checking t\u000A(fact 2)\u000A(fact t)\u000A:set-guard-checking :all\u000A(fact 2)\u000A(fact t)\u000A:set-guard-checking :none\u000A(fact 2)\u000A(fact t)\u000A:set-guard-checking nil\u000A(fact 2)\u000A(fact t)</code> \u000A \u000A <p>If you want to run the script yourself, you may find it handy to use the \u000A following Emacs keyboard macro for running the tracing code in 2-window mode, \u000A with the cursor in the window with the script and ACL2 running in the other \u000A window.</p> \u000A \u000A <code>(fset 'step-guard-script\u000A   [?C-a ?C-  ?C-e ?M-w ?C-a ?C-n\u000A    ?C-x ?o ?M-&gt; ?C-y return ?C-x ?o])\u000A\u000A; Put it on a key (<see topic=\"COMMON-LISP____IF\">if</see> you have defined the indicated keymap by using\u000A; emacs/emacs-acl2.el):\u000A(define-key ctl-t-keymap &quot;r&quot; 'step-guard-script)</code> \u000A \u000A <p>The script follows.</p> \u000A \u000A <code>;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Program mode\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :mode :program))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000A; Run the Tracing Code here.  It shows execution in raw Lisp in the t and nil\u000A; cases of :set-guard-checking, but not in the :all or :none cases.  We get a\u000A; guard violation for argument t in the case :set-guard-checking t.\u000A\u000A:u\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                         :mode :program))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000A; Run the Tracing Code here.  It should give the same results as above,\u000A; except that we no longer get a guard violation in the case\u000A; :set-guard-checking t.\u000A\u000A:u\u000A\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Logic mode, guard other than t\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :verify-guards nil\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000A; Run the Tracing Code here.  It should give guard violations for (fact t)\u000A; with guard-checking set to t or :all.  It should never run in raw Lisp,\u000A; because we have not verified guards.  In the t case, we can get a warning\u000A; about avoiding the guard check on recursive calls, but only if we do not\u000A; trace the function, fact.\u000A\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)\u000A\u000A; Run the Tracing Code here.  The results should be as described just above,\u000A; except that now we go into raw Lisp for (fact 2) with guard-checking other\u000A; than :none.\u000A\u000A:u\u000A:u\u000A\u000A; The following definition is the same as above, except that guards are\u000A; verified.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000A; Run the Tracing Code here.  We should get the same traces as in the\u000A; immediately preceding case, since guards had been verified in both cases.\u000A\u000A:u\u000A\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A;;; Logic mode, guard t\u000A;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t\u000A                         :verify-guards nil\u000A                         :mode :logic))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____POSP\">posp</see> x)\u000A             (<see topic=\"COMMON-LISP_____A2\">*</see> x (fact (<see topic=\"COMMON-LISP____1-\">1-</see> x)))\u000A           1))\u000A\u000A; Run the Tracing Code here.  We should never go in to raw Lisp, because\u000A; guards have not been verified.  We will see the same traces for (fact 2) as\u000A; with the (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) guard above with :verify-guards nil specified, except\u000A; that even without tracing, there is no warning for :set-guard-checking t\u000A; about recursive calls.  And, there are no guard violations for (fact t), of\u000A; course, since posp (necessarily, if we are to verify guards) has a guard of\u000A; t.\u000A\u000A(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> fact)\u000A\u000A; Run the Tracing Code here.  You shouldn't see any surprises.  Note however\u000A; that if we back up to the start (using :u :u) and then define fact as just\u000A; above but without :verify-guards nil, then the :none setting will allow us\u000A; to go into raw Lisp: although :none generally avoids execution of raw Lisp\u000A; counterparts, it allows this when the guard is T and guards have been\u000A; verified.</code>"],

 "ACL2____EQUIVALENT-FORMULAS-DIFFERENT-REWRITE-RULES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Consider the rewrite rules that would be generated from the three \u000A commands below.  In all three cases, the fact being stated relates the \u000A <v>n</v>th element of the reverse of <v>x</v> to the <v>n</v>th element of <v>x</v>. \u000A In fact, the three formulas are simple rearrangements of each other and are \u000A all equivalent.  The theorem prover treats all three formulas equivalently \u000A when proving them.  But the rules generated from them are very different.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> nth-rev-1\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> x)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n (rev x))\u000A                  (<see topic=\"COMMON-LISP____NTH\">nth</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> nth-rev-2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> x)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x)\u000A                  (<see topic=\"COMMON-LISP____NTH\">nth</see> n (rev x)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> nth-rev-3\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> n)\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NTH\">nth</see> n (rev x))\u000A                            (<see topic=\"COMMON-LISP____NTH\">nth</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">len</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x))))\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">len</see> x)))))</code> \u000A \u000A <p>Here are the three rewrite rules:</p> \u000A \u000A <p><b>nth-rev-1</b>:<br></br> \u000A \u000A Replace instances of <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> n (REV x))</v><br></br> \u000A \u000A by <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">LEN</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x)</v>,<br></br> \u000A \u000A if you can establish that <v>n</v> is a natural number less than the length of \u000A <v>x</v>.</p> \u000A \u000A <p><b>nth-rev-2</b>:<br></br> \u000A \u000A Replace instances of <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">LEN</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x)</v><br></br> \u000A \u000A by <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> n (REV x))</v>,<br></br> \u000A \u000A if you can establish that <v>n</v> is a natural number less than the length of \u000A <v>x</v>.</p> \u000A \u000A <p><b>nth-rev-3</b>:<br></br> \u000A \u000A Replace instances of <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"ACL2____LEN\">LEN</see> x))</v><br></br> \u000A \u000A by <v>NIL</v><br></br> \u000A \u000A if you can establish that <v>n</v> is a natural number and that <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> n (REV\u000A x))</v> is different from <v>(<see topic=\"COMMON-LISP____NTH\">NTH</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____LEN\">LEN</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 n)) x)</v>.</p> \u000A \u000A <p>As the driver of ACL2, you have to decide which rule you want <i>when you \u000A give the command to prove it</i>.</p> \u000A \u000A <p>If you tell the theorem prover to use both <v>nth-rev-1</v> and \u000A <v>nth-rev-2</v>, ACL2 will enter an infinite loop when it sees any term \u000A matching either <v>NTH</v> expression.</p> \u000A \u000A <p>Most users would choose form <v>nth-rev-1</v> of the rule.  It eliminates \u000A <v>rev</v> from the problem -- at the expense of introducing some arithmetic. \u000A But arithmetic is so fundamental it is rarely possible to avoid it and it is \u000A likely to be in the problem already since you're indexing into <v>(rev x)</v>. \u000A The <v>nth-rev-2</v> form of the rule is ``bad'' because it introduces <v>rev</v> \u000A into a problem where it might not have appeared.  The <v>nth-rev-3</v> version \u000A is ``bad'' because it makes the theorem prover shift its attention from a \u000A simple arithmetic inequality to a complicated property of <v>nth</v> and \u000A <v>rev</v>, which might not be in the problem.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see>.</p>"],

 "ACL2____DISASSEMBLE_42":[["Debugging","Programming"],"ACL2 Sources","ACL2","<p>The macro <v>disassemble$</v> provides a convenient interface to the \u000A underlying <v>disassemble</v> utility of the host Common Lisp implementation, \u000A which prints assembly code for a given function symbol at the terminal.  If \u000A the argument is instead a macro alias for a function symbol (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), then it prints assembly code for that function symbol \u000A instead.</p> \u000A \u000A <p><v>Disassemble$</v> works by including the community book \u000A <v>books/misc/disassemble.lisp</v>, which defines the supporting function \u000A <v>disassemble$-fn</v>, and then by calling that function.  Note that the \u000A arguments to <v>disassemble$</v> are evaluated.  Also note that \u000A <v>disassemble$</v> is intended as a top-level utility for the ACL2 loop, not to \u000A be called in code; for such a purpose, include the above book and call \u000A <v>disassemble$-fn</v> directly.</p> \u000A \u000A <code>Example Forms:\u000A\u000A(<see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> 'foo)\u000A(<see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> 'foo :recompile t)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> form)\u000A(<see topic=\"ACL2____DISASSEMBLE_42\">disassemble$</see> form :recompile flg)</code> \u000A \u000A <p>where <v>form</v> evaluates to a function symbol or a macro alias for a \u000A function symbol and <v>flg</v> evaluates to any value.  If <v>flg</v> is <v>nil</v>, \u000A then the existing definition of that function symbol is disassembled.  But if \u000A <v>flg</v> is supplied and has a value other than <v>nil</v> or <v>:default</v>, and \u000A if that function symbol is defined in the ACL2 loop (not merely in raw Lisp; \u000A for example, see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), then the disassembly will be based on a \u000A recompilation of that ACL2 definition.  Normally this recompilation is not \u000A necessary, but for some host Lisps, it may be useful; in particular, for CCL \u000A the above book arranges that source code information is saved, so that the \u000A output is annotated with such information.  When recompilation takes place, \u000A the previous definition is restored after disassembly is complete.  Finally, \u000A if <v>flg</v> is omitted or has the value <v>:default</v> &mdash; i.e., in the \u000A default case &mdash; then recompilation may take place or not, depending on \u000A the host Lisp.  The values of <v>(<see topic=\"ACL2_____04\">@</see> host-lisp)</v> for which recompilation takes \u000A place by default may be found by looking at the above book, or by including it \u000A and evaluating the constant <v>*host-lisps-that-recompile-by-default*</v>.  As \u000A of this writing, CCL is the only such Lisp (because that is the one for which \u000A we can obtain source annotation in the output by recompiling).</p>"],

 "ACL2____BIND-FREE-EXAMPLES":[["Bind-free"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BIND-FREE\">bind-free</see> for a basic discussion of the use of \u000A <v>bind-free</v> to control rewriting.</p> \u000A \u000A <p>Note that the examples below all illustrate the common case in which a \u000A <v>bind-free</v> hypothesis generates a binding alist.  See <see topic=\"ACL2____BIND-FREE\">bind-free</see>, in \u000A particular the final section, for a discussion of the case that instead a list \u000A of binding alists is generated.</p> \u000A \u000A <p>We give examples of the use of <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypotheses from the \u000A perspective of a user interested in reasoning about arithmetic, but it should \u000A be clear that <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> can be used for many other purposes also.</p> \u000A \u000A <p>EXAMPLE 1: Cancel a common factor.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bind-divisor (a b)\u000A\u000A; If a and b are polynomials with a common factor c, we return a\u000A; binding for x.  We could imagine writing get-factor to compute the\u000A; gcd, or simply to return a single non-invertible factor.\u000A\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((c (get-factor a b)))\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> c (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'x c)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> cancel-factor\u000A  ;; We use case-split here to ensure that, once we have selected\u000A  ;; a binding for x, the rest of the hypotheses will be relieved.\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> a)\u000A                (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> b)\u000A                (<see topic=\"ACL2____BIND-FREE\">bind-free</see> (bind-divisor a b) (x))\u000A                (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)))\u000A                (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)))\u000A           (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> a b)\u000A                (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____F2\">/</see> a x) (<see topic=\"COMMON-LISP_____F2\">/</see> b x)))))</code> \u000A \u000A <p>EXAMPLE 2: Pull integer summand out of floor.  Note: This example has an \u000A <i>extended</i> <tt><see topic=\"ACL2____BIND-FREE\">bind-free</see></tt> hypothesis, which uses the term \u000A <v>(find-int-in-sum sum mfc state)</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fl (x)\u000A  ;; This function is defined, and used, in the IHS books.\u000A  (<see topic=\"COMMON-LISP____FLOOR\">floor</see> x 1))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> int-binding (<see topic=\"ACL2____TERM\">term</see> mfc state)\u000A  ;; The call to mfc-ts returns the encoded type of term. ;\u000A  ;; Thus, we are asking if term is known by type reasoning to ;\u000A  ;; be an integer. ;\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (<see topic=\"ACL2____STATE\">state</see>) :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (ts-subsetp (mfc-ts term mfc state)\u000A                  *ts-integer*)\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'int term))\u000A    nil))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> find-int-in-sum (sum mfc state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (<see topic=\"ACL2____STATE\">state</see>) :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (nvariablep sum)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (fquotep sum))\u000A           (<see topic=\"COMMON-LISP____EQ\">eq</see> (ffn-symb sum) 'binary-+))\u000A      (<see topic=\"COMMON-LISP____OR\">or</see> (int-binding (fargn sum 1) mfc state)\u000A          (find-int-in-sum (fargn sum 2) mfc state))\u000A    (int-binding sum mfc state)))\u000A\u000A; Some additional work is required to prove the following.  So for\u000A; purposes of illustration, we wrap skip-proofs around the defthm.\u000A\u000A(<see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see>\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> cancel-fl-int\u000A  ;; The use of case-split is probably not needed, since we should\u000A  ;; know that int is an integer by the way we selected it.  But this\u000A  ;; is safer.\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> sum)\u000A                 (<see topic=\"ACL2____BIND-FREE\">bind-free</see> (find-int-in-sum sum mfc state) (int))\u000A                 (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> int)))\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fl sum)\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> int (fl (<see topic=\"COMMON-LISP____-\">-</see> sum int)))))\u000A   :rule-classes ((:rewrite :match-free :all)))\u000A)\u000A\u000A; Arithmetic libraries will have this sort of lemma.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> hack (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> x) x y) (<see topic=\"ACL2____FIX\">fix</see> y)))\u000A\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> fl))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (fl (<see topic=\"COMMON-LISP_____B2\">+</see> x y)) (<see topic=\"COMMON-LISP_____B2\">+</see> x (fl y)))))</code> \u000A \u000A <p>EXAMPLE 3: Simplify terms such as (equal (+ a (* a b)) 0)</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> factors (product)\u000A  ;; We return a list of all the factors of product.  We do not\u000A  ;; require that product actually be a product.\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (fn-symb product) 'BINARY-*)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (fargn product 1)\u000A            (factors (fargn product 2)))\u000A    (<see topic=\"COMMON-LISP____LIST\">list</see> product)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-product (factors)\u000A  ;; Factors is assumed to be a list of ACL2 terms.  We return an\u000A  ;; ACL2 term which is the product of all the ellements of the\u000A  ;; list factors.\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> factors)\u000A         ''1)\u000A        ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> factors))\u000A         (<see topic=\"COMMON-LISP____CAR\">car</see> factors))\u000A        ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> factors))\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'BINARY-* (<see topic=\"COMMON-LISP____CAR\">car</see> factors) (<see topic=\"COMMON-LISP____CADR\">cadr</see> factors)))\u000A        (t\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'BINARY-* (<see topic=\"COMMON-LISP____CAR\">car</see> factors) (make-product (<see topic=\"COMMON-LISP____CDR\">cdr</see> factors))))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> quotient (common-factors sum)\u000A  ;; Common-factors is a list of ACL2 terms.   Sum is an ACL2 term each\u000A  ;; of whose addends have common-factors as factors.  We return\u000A  ;; (<see topic=\"COMMON-LISP_____F2\">/</see> sum (make-product common-factors)).\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (fn-symb sum) 'BINARY-+)\u000A      (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"COMMON-LISP____FIRST\">first</see> (make-product (<see topic=\"ACL2____SET-DIFFERENCE-EQUAL\">set-difference-equal</see> (factors (fargn sum 1))\u000A                                                       common-factors))))\u000A        (<see topic=\"COMMON-LISP____LIST\">list</see> 'BINARY-+ first (quotient common-factors (fargn sum 2))))\u000A    (make-product (<see topic=\"ACL2____SET-DIFFERENCE-EQUAL\">set-difference-equal</see> (factors sum)\u000A                                        common-factors))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> intersection-equal (x y)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A         nil)\u000A        ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) y)\u000A         (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))\u000A        (t\u000A         (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> common-factors (factors sum)\u000A  ;; Factors is a list of the factors common to all of the addends\u000A  ;; examined so far.  On entry, factors is a list of the factors in\u000A  ;; the first addend of the original sum, and sum is the rest of the\u000A  ;; addends.  We sweep through sum, trying to find a set of factors\u000A  ;; common to all the addends of sum.\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> sum)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> factors)\u000A         nil)\u000A        ((<see topic=\"COMMON-LISP____EQ\">eq</see> (fn-symb sum) 'BINARY-+)\u000A         (common-factors (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> factors (factors (fargn sum 1)))\u000A                         (fargn sum 2)))\u000A        (t\u000A         (<see topic=\"ACL2____INTERSECTION-EQUAL\">intersection-equal</see> factors (factors sum)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> simplify-terms-such-as-a+ab-rel-0-fn (sum)\u000A  ;; If we can find a set of factors common to all the addends of sum,\u000A  ;; we return an alist binding common to the product of these common\u000A  ;; factors and binding quotient to (<see topic=\"COMMON-LISP_____F2\">/</see> sum common).\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (fn-symb sum) 'BINARY-+)\u000A      (<see topic=\"COMMON-LISP____LET\">let</see> ((common-factors (common-factors (factors (fargn sum 1))\u000A                                            (fargn sum 2))))\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> common-factors\u000A            (<see topic=\"COMMON-LISP____LET\">let</see> ((common (make-product common-factors))\u000A                  (quotient (quotient common-factors sum)))\u000A              (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'common common)\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> 'quotient quotient)))\u000A          nil))\u000A    nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> simplify-terms-such-as-a+ab-=-0\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BIND-FREE\">bind-free</see>\u000A                 (simplify-terms-such-as-a+ab-rel-0-fn sum)\u000A                 (common quotient))\u000A                (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> common))\u000A                (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> quotient))\u000A                (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> sum\u000A                                   (<see topic=\"COMMON-LISP_____A2\">*</see> common quotient))))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> sum 0)\u000A                  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> common 0)\u000A                      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> quotient 0)))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> u (<see topic=\"COMMON-LISP_____A2\">*</see> u v)) 0)\u000A      (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> u 0) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> v -1))))</code>"],

 "ACL2____ADD-INCLUDE-BOOK-DIR_12":[["Books-reference"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see>, which has completely \u000A analogous syntax and semantics, except that <v>add-include-book-dir!</v> is not \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs. \u000A Probably <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> is to be preferred unless you have a \u000A good reason for wanting to export the effect of this event outside the \u000A enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>This macro is essentially a <tt><see topic=\"ACL2____TABLE\">table</see></tt> event that updates the table \u000A <v>include-book-dir!-table</v>, which associates keywords with absolute \u000A pathnames.  However, as with <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>, direct table \u000A updates are disallowed; you must use <v>add-include-book-dir!</v> to add to the \u000A table and <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt> to remove from the table.</p> \u000A \u000A <p>It is illegal to call <v>add-include-book-dir!</v> in a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A context.  (If you are tempted to do that, consider using <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> instead.)  To understand this restriction, imagine a \u000A book that contains the following sequence of <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see> :my-dir &quot;path/to/BAD/dir&quot;)\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see> :my-dir))\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see> :my-dir &quot;path/to/GOOD/dir&quot;))\u000A(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;foo&quot; :dir :my-dir)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> f-def\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (f x) x))</code> \u000A \u000A <p>During the first (proof) pass of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, the book \u000A <v>path/to/GOOD/dir/foo.lisp</v> will be included.  But on the second pass, the \u000A book <v>path/to/BAD/dir/foo.lisp</v> will be included.  Now imagine that the \u000A ``good'' version contains the event <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)</v> but the ``bad'' \u000A version instead contains the event <v>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) (<see topic=\"COMMON-LISP____NOT\">not</see> x))</v>.  Then we can \u000A easily prove <v>nil</v> from the theorem <v>f-def</v>!  Although it is likely that \u000A checksums will catch this error at <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> time, we prefer not to \u000A rely on checksums for soundness.</p>"],

 "ACL2____USING-TABLES-EFFICIENTLY":[["Table"],"ACL2 Sources","ACL2","<p> \u000A (Thanks to Jared Davis for contributing this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic, to \u000A which we have made only minor modifications.)</p> \u000A \u000A <p>Suppose your book contains <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, or macros that \u000A expand into <v>table</v> events, of the following form:</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> my-table 'my-field &lt;computation&gt;)</code> \u000A \u000A <p>Then <v>&lt;computation&gt;</v> will be evaluated <i>twice</i> during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <i>again</i> every time you include the book with <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  In some cases this overhead can be avoided using <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.</p> \u000A \u000A <p>See also <see topic=\"ACL2____DEFCONSTS\">defconsts</see> for an analogous trick involving <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>.</p> \u000A \u000A <p>As an example, suppose we want to store numbers in a table only if they \u000A satisfy some computationally expensive predicate.  We'll introduce a new book, \u000A <v>number-table.lisp</v>, and create a table to store these numbers:</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> number-table 'data nil)</code> \u000A \u000A <p>Instead of implementing a ``computationally expensive predicate,'' we'll \u000A write a function that just prints a message when it is called and accepts even \u000A numbers:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> expensive-computation (n)\u000A  (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Expensive computation on ~x0.~%&quot; n)\u000A          (<see topic=\"COMMON-LISP____EVENP\">evenp</see> n)))</code> \u000A \u000A <p>Now we'll implement a macro, <v>add-number</v>, which will add its argument \u000A to the table only if it satisfies the expensive predicate:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> add-number (n)\u000A  `(<see topic=\"ACL2____TABLE\">table</see> number-table 'data\u000A          (<see topic=\"COMMON-LISP____LET\">let</see> ((current-data\u000A                 (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> 'data (table-alist 'number-table world)))))\u000A            (<see topic=\"COMMON-LISP____IF\">if</see> (expensive-computation ,n)\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> ,n current-data)\u000A              current-data))))</code> \u000A \u000A <p>Finally, we'll call <v>add-number</v> a few times to finish the book.</p> \u000A \u000A <code>(add-number 1)\u000A(add-number 2)\u000A(add-number 3)</code> \u000A \u000A <p>When we now invoke <v>(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;number-table&quot;)</v>, we see the \u000A expensive predicate being called twice for each number: once in Step 2, the \u000A main pass, then again in Step 3, the admissibility check.  Worse, the \u000A computation is performed again for each number when we use <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> to load <v>number-table</v>, e.g.,</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;number-table&quot;)\u000AExpensive computation on 1.\u000AExpensive computation on 2.\u000AExpensive computation on 3.</code> \u000A \u000A <p>To avoid these repeated executions, we can pull the test out of the \u000A <v>table</v> event using <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>.  Here's an alternate implementation \u000A of <v>add-number</v> that won't repeat the computation:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> add-number (n)\u000A  `(<see topic=\"ACL2____MAKE-EVENT\">make-event</see>\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (expensive-computation ,n)\u000A        '(<see topic=\"ACL2____TABLE\">table</see> number-table 'data\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> ,n (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> 'data\u000A                                     (table-alist 'number-table world)))))\u000A      '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :expensive-computation-failed))))</code> \u000A \u000A <p>When we recertify <v>number-table.lisp</v>, we'll see the expensive \u000A computation is still called once for each number in Step 2, but is no longer \u000A called during Step 3.  Similarly, the <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> no longer shows any \u000A calls of the expensive computation.</p>"],

 "ACL2____TYPE-ALIST":[["Type-set"],"ACL2 Sources","ACL2","<p>The ACL2 prover maintains many structures that need not be \u000A  understood by the user.  One of these, the <i>type-alist</i> structure, is \u000A  usually in this category.  But some utilities refer to the type-alist, so we \u000A  summarize it here.</p> \u000A \u000A  <p>A type-alist is is an association list, each element of which is of the \u000A  form <tt>(u ts . ttree)</tt>, where <v>u</v> is a <see topic=\"ACL2____TERM\">term</see> (in internal, \u000A  ``translated'' form), <v>ts</v> is a <see topic=\"ACL2____TYPE-SET\">type-set</see>, and <v>ttree</v> is a \u000A  tag-tree (see <see topic=\"ACL2____TTREE\">ttree</see>).  Such an element means that the term <v>u</v> has \u000A  the type-set <v>ts</v>, which is a number.  While you are welcome to see the \u000A  <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of <see topic=\"ACL2____TYPE-SET\">type-set</see> for details, this is not necessary, \u000A  since user-level utilities generally display a type-alist without displaying \u000A  numeric type-sets.  Instead, they often use a symbol of the form \u000A  <tt>*TS-typ*</tt> to denote the type described by <tt>typ</tt>, and for \u000A  complements they use <tt>(TS-COMPLEMENT *TS-typ*)</tt>.  For example, a \u000A  context in which the terms <tt>(p x y)</tt>, <v>(<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">no-duplicatesp-equal</see> y)</v>, \u000A  and <v>(<see topic=\"ACL2____NATP\">natp</see> x)</v> are assumed to be true (i.e., non-<v>nil</v>) may be \u000A  displayed using the following lines.</p> \u000A \u000A  <code>-----\u000ATerms with type *TS-T*:\u000A(<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">NO-DUPLICATESP-EQUAL</see> Y)\u000A-----\u000ATerms with type (TS-COMPLEMENT *TS-NIL*):\u000A(P X Y)\u000A-----\u000ATerms with type *TS-NON-NEGATIVE-INTEGER*:\u000AX\u000A-----</code> \u000A \u000A  <p>These lines says that <v>(<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">no-duplicatesp-equal</see> y)</v> is the symbol <v>t</v>, \u000A  <tt>(p x y)</tt> is not the symbol <v>nil</v>, and <v>x</v> is a non-negative \u000A  integer (i.e., a <tt><see topic=\"ACL2____NATP\">natp</see></tt>).</p> \u000A \u000A  <p>ACL2 computes a type-alist based on contextual information including the \u000A  hypotheses of the current goal (technically: the negations of the literals in \u000A  the goal that are not currently being rewritten), <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> \u000A  from those literals, and the surrounding tests in calls of <v>if</v>.  The \u000A  type-alist is of course not all that is known, since an entire logical <see topic=\"ACL2____WORLD\">world</see> of facts is also known; and additional known information may be \u000A  recorded in the current database of <see topic=\"ACL2____LINEAR\">linear</see> rules.</p>"],

 "ACL2____REFINEMENT":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bag-equal-refines-set-equal\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (bag-equal x y)\u000A           (set-equal y x))\u000A  :rule-classes :refinement)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 x y) (equiv2 x y))</code> \u000A \u000A <p><v>Equiv1</v> and <v>equiv2</v> must be known equivalence relations.  The \u000A effect of such a rule is to record that <v>equiv1</v> is a refinement of \u000A <v>equiv2</v>.  This means that <v>equiv1</v> <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules may be \u000A used while trying to maintain <v>equiv2</v>.  See <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> for a \u000A general discussion of the issues.</p> \u000A \u000A <p>The macro form <v>(<see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see> equiv1 equiv2)</v> is an abbreviation for a \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> of rule-class <v>:refinement</v> that establishes that \u000A <v>equiv1</v> is a refinement of <v>equiv2</v>.  See <see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see>.</p> \u000A \u000A <p>Suppose we have the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule</p> \u000A \u000A <code>(bag-equal (<see topic=\"COMMON-LISP____APPEND\">append</see> a b) (<see topic=\"COMMON-LISP____APPEND\">append</see> b a))</code> \u000A \u000A <p>which states that <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> is commutative modulo bag-equality. \u000A Suppose further we have established that bag-equality refines set-equality. \u000A Then when we are simplifying <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> expressions while maintaining \u000A set-equality we use <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>'s commutativity property, even though it \u000A was proved for bag-equality.</p> \u000A \u000A <p>Equality is known to be a refinement of all equivalence relations.  The \u000A transitive closure of the refinement relation is maintained, so if \u000A <v>set-equality</v>, say, is shown to be a refinement of some third sense of \u000A equivalence, then <v>bag-equality</v> will automatially be known as a refinement \u000A of that third equivalence.</p> \u000A \u000A <p><v>:refinement</v> lemmas cannot be disabled.  That is, once one equivalence \u000A relation has been shown to be a refinement of another, there is no way to \u000A prevent the system from using that information.  Of course, individual \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules can be disabled.</p> \u000A \u000A <p>More will be written about this as we develop the techniques.</p>"],

 "ACL2____PARALLEL-PUSHING-OF-SUBGOALS-FOR-INDUCTION":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>The following discussion, concerning the naming of subgoals pushed for \u000A proof by induction and the timeliness of aborting when two or more goals are \u000A pushed for proof by induction, only applies when waterfall parallelism is \u000A enabled (see <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>).</p> \u000A \u000A <p>When two sibling subgoals (e.g. 4.5 and 4.6) both push goals to be proved \u000A by induction (e.g., 4.6 pushes *1 and 4.5 pushes *2), a name is assigned to \u000A the second pushed subgoal (e.g., *2) as if the first push hasn't happened \u000A (e.g., *2 is mistakenly called *1).  In such a case, we say what the name \u000A _could_ be.  The following non-theorem illustrates how this works.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> :full)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x x)) y z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x x y)))</code> \u000A \u000A <p>There is another consequence of the way the parallelized waterfall pushes \u000A subgoals for proof by induction.  Without waterfall parallelism enabled, ACL2 \u000A sometimes decides to abort instead of pushing a goal for later proof by \u000A induction, preferring instead to induct on the original conjecture.  But with \u000A waterfall parallelism enabled, the prover no longer necessarily immediately \u000A aborts to prove the original conjecture.  Suppose for example that sibling \u000A subgoals, Subgoal 4.6 and Subgoal 4.5, each push a subgoal for induction.  If \u000A the waterfall is performing the proof of each of these subgoals in parallel, \u000A the proof will no longer abort immediately after the second push occurs, that \u000A is at Subgoal 4.5.  As a result, the prover will continue through Subgoal 4.4, \u000A Subgoal 4.3, and beyond.  It is not until the results of combining the proof \u000A results of Subgoal 4.6 with the results from the remaining sibling subgoals \u000A (4.5, 4.4, and so on), that the proof attempt will abort and revert to prove \u000A the original conjecture by induction.  This example illustrates behavior that \u000A is rather like the case that <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> is <v>t</v>, in the sense \u000A that the abort does not happen immediately, but also rather like the case that \u000A <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> is <v>nil</v>, in the sense that the abort does occur \u000A before getting to Subgoal 3.</p>"],

 "ACL2____KEYWORD-COMMANDS":[["Ld"],"ACL2 Sources","ACL2","<code>Examples:\u000Auser type-in                 form evaluated\u000A:pc 5                        (<see topic=\"ACL2____PC\">ACL2::PC</see> '5)\u000A:pcs app rev                 (<see topic=\"ACL2____PCS\">ACL2::PCS</see> 'app 'rev)\u000A:length (1 2 3)              (<see topic=\"COMMON-LISP____LENGTH\">ACL2::LENGTH</see> '(1 2 3))\u000A:quit                        (<see topic=\"ACL2____QUIT\">ACL2::QUIT</see>) ; Note: avoid optional argument</code> \u000A \u000A <p>When a keyword, <v>:key</v>, is read as a command, ACL2 determines whether \u000A the symbol with the same name in the <v>&quot;ACL2&quot;</v> package, <v>acl2::key</v>, is \u000A a function or simple macro of n arguments.  If so, ACL2 reads <v>n</v> more \u000A objects, <v>obj1</v>, ..., <v>objn</v>, and then acts as though it had read the \u000A following form (for a given <v>key</v>):</p> \u000A \u000A <code>(ACL2::key 'obj1 ... 'objn)</code> \u000A \u000A <p>Thus, by using the keyword command hack you avoid typing the parentheses, \u000A the <v>&quot;ACL2&quot;</v> package name, and the quotation marks.</p> \u000A \u000A <p>See <see topic=\"ACL2____LD-KEYWORD-ALIASES\">ld-keyword-aliases</see> for how to customize this behavior.</p> \u000A \u000A <p>Note the generality of this hack.  Any function or macro in the \u000A <v>&quot;ACL2&quot;</v> package can be so invoked, not just ``commands.''  Indeed, there \u000A is no such thing as a distinguished class of commands.  Users may take \u000A advantage of the keyword command hack by defining functions and macros in the \u000A <v>&quot;ACL2&quot;</v> package.</p> \u000A \u000A <p>The one caveat is that when the keyword hack is used to invoke a macro, \u000A only the required arguments for that macro are read before calling that macro: \u000A none of the <v>&amp;optional</v>, <v>&amp;rest</v>, <v>&amp;body</v>, or <v>&amp;key</v> arguments are \u000A read for that call.  The macro is thus called with only its required \u000A arguments.  The following log illustrates this caveat.</p> \u000A \u000A <code>ACL2 !&gt;:set-iprint t\u000A\u000AACL2 Query (:SET-IPRINT):  Action  (T, NIL, RESET, RESET-ENABLE, SAME,\u000AQ or ?):\u000AACL2 Observation in SET-IPRINT:  Iprinting has been enabled.\u000AACL2 !&gt;</code> \u000A \u000A <p>What happened?  First, the command <v>:set-iprint</v> was read.  Since the \u000A macro <tt><see topic=\"ACL2____SET-IPRINT\">set-iprint</see></tt> has no required arguments, the ACL2 evaluator was \u000A then called on the form <v>(<see topic=\"ACL2____SET-IPRINT\">set-iprint</see>)</v>, that is, calling the macro on no \u000A arguments.  <v>Set-iprint</v> is defined to query the ACL2 user when its first \u000A argument is omitted.  The log shows that query, which is set up to read the \u000A next form from the input stream.  That form was available immediately: the \u000A form <v>t</v> that had been supplied by the user.  So the query returned \u000A immediately and the <v>set-iprint</v> call was completed.</p>"],

 "ACL2____GUARD-OBLIGATION":[["Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>, and see <see topic=\"ACL2____GUARD\">guard</see> for a discussion of \u000A guards.  Also see <see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> for a utility provided for \u000A viewing the guard proof obligation, without proof.  <v>Guard-obligation</v> is a \u000A lower level function for use in system programs, not typically appropriate for \u000A most ACL2 users.  If you simply want to see the guard proof obligations, see \u000A <see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see>.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> 'foo nil 'top-level state)\u000A(<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> '(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (foo (<see topic=\"COMMON-LISP____CAR\">car</see> x)) t) nil 'my-function state)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> name guard-debug ctx state)\u000A(<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> term guard-debug ctx state)</code> \u000A \u000A <p>where the first argument is either the name of a function or theorem or is \u000A a non-variable term that may be in untranslated form; <v>guard-debug</v> is \u000A typically <v>nil</v> but may be <v>t</v> (see <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see>); <v>ctx</v> is a \u000A context (typically, a symbol used in error and warning messages); and <tt><see topic=\"ACL2____STATE\">state</see></tt> references the ACL2 <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <p>If you want to obtain the formula but you don't care about the so-called \u000A ``tag tree'':</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        (<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> x guard-debug 'top-level state)\u000A        (<see topic=\"COMMON-LISP____IF\">if</see> erp\u000A           ( .. code for handling error case, e.g., name is undefined .. )\u000A          (<see topic=\"COMMON-LISP____LET\">let</see> ((cl-set (<see topic=\"COMMON-LISP____CADR\">cadr</see> val))) ; to be proved for guard verification\u000A            ( .. code using cl-set, which is a list of clauses,\u000A                 implicitly conjoined, each of which is viewed as\u000A                 a disjunction .. ))))</code> \u000A \u000A <p>The form <v>(<see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> x guard-debug ctx state)</v> evaluates to a \u000A triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>, where <v>erp</v> is <v>nil</v> unless there is an \u000A error, and <tt><see topic=\"ACL2____STATE\">state</see></tt> is the ACL2 state.  Suppose <v>erp</v> is <v>nil</v>. \u000A Then <v>val</v> is the keyword <v>:redundant</v> if the corresponding <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> event would be redundant; see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>. \u000A Otherwise, <v>val</v> is a tuple <v>(<see topic=\"COMMON-LISP____LIST_A2\">list*</see> names cl-set ttree)</v>, where: \u000A <v>names</v> is <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> :term xt)</v> if <v>x</v> is not a variable, where <v>xt</v> \u000A is the translated form of <v>x</v>; and otherwise is a list containing <v>x</v> \u000A along with, if <v>x</v> is defined in a <v>mutual-recursion</v>, any other \u000A functions defined in the same <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> nest; <v>cl-set</v> is a \u000A list of lists of terms, viewed as a conjunction of clauses (each viewed (as a \u000A disjunction); and <v>ttree</v> is an assumption-free tag-tree that justifies \u000A cl-set.  (The notion of ``tag-tree'' may probably be ignored except for system \u000A developers.)</p> \u000A \u000A <p><v>Guard-obligation</v> is typically used for function names or non-variable \u000A terms, but as for <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, it may also be applied to theorem \u000A names.</p> \u000A \u000A <p>See the source code for <tt><see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see></tt> for an example of how \u000A to use <v>guard-obligation</v>.</p>"],

 "COMMON-LISP____NTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____NTH\">Nth</see> n l)</v> is the <v>n</v>th element of <v>l</v>, zero-based.  If \u000A <v>n</v> is greater than or equal to the length of <v>l</v>, then <v>nth</v> returns \u000A <v>nil</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____NTH\">Nth</see> n l)</v> has a <see topic=\"ACL2____GUARD\">guard</see> that <v>n</v> is a non-negative integer and \u000A <v>l</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.</p> \u000A \u000A <p><v>Nth</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>nth</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nth (n l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                                   (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> l)\u000A           nil\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A               (<see topic=\"COMMON-LISP____CAR\">car</see> l)\u000A               (<see topic=\"COMMON-LISP____NTH\">nth</see> (<see topic=\"COMMON-LISP____-\">-</see> n 1) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))))</code>"],

 "ACL2____TYPE-SET-INVERTER":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example Rule Class:\u000A(:type-set-inverter\u000A  :corollary (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____AND\">and</see> (counting-number x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)))\u000A                    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x)))\u000A  :type-set 2)\u000A\u000AGeneral Forms of Rule Class:\u000A:type-set-inverter, or\u000A(:type-set-inverter :type-set n)\u000A\u000AGeneral Form of Theorem or Corollary:\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> new-expr old-expr)</code> \u000A \u000A <p>where <v>n</v> is a <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> (see <see topic=\"ACL2____TYPE-SET\">type-set</see>) and <v>old-expr</v> \u000A is the term containing <v>x</v> as a free variable that ACL2 currently uses to \u000A recognize <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> <v>n</v>.  For a given <v>n</v>, the exact form of \u000A <v>old-expr</v> is generated by</p> \u000A \u000A <code>(convert-type-set-to-term 'x n (ens state) (w state) nil)].</code> \u000A \u000A <p>If the <v>:</v><tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> field of the rule-class is omitted, we \u000A attempt to compute it from the right-hand side, <v>old-expr</v>, of the \u000A corollary.  That computation is done by <v>type-set-implied-by-term</v> (see \u000A <see topic=\"ACL2____TYPE-SET\">type-set</see>).  However, it is possible that the type-set we compute from \u000A <v>lhs</v> does not have the required property that when inverted with \u000A <v>convert-type-set-to-term</v> the result is <v>lhs</v>.  If you omit \u000A <v>:</v><tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> and an error is caused because <v>lhs</v> has the \u000A incorrect form, you should manually specify both <v>:</v><tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> and \u000A the <v>lhs</v> generated by <v>convert-type-set-to-term</v>.</p> \u000A \u000A <p>The rule generated will henceforth make <v>new-expr</v> be the term used by \u000A ACL2 to recognize type-set <v>n</v>.  If this rule is created by a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event named <v>name</v> then the rune of the rule is \u000A <v>(:type-set-inverter name)</v> and by disabling that rune you can prevent its \u000A being used to decode type-sets.</p> \u000A \u000A <p>Type-sets are inverted when forced assumptions are turned into formulas to \u000A be proved.  In their internal form, assumptions are essentially pairs \u000A consisting of a context and a goal term, which was forced.  Abstractly a \u000A context is just a list of hypotheses which may be assumed while proving the \u000A goal term.  But actually contexts are alists which pair terms with type-sets, \u000A encoding the current hypotheses.  For example, if the original conjecture \u000A contained the hypothesis <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v> then the context used while working \u000A on that conjecture will include the assignment to <v>x</v> of the type-set \u000A <v>*ts-integer*</v>.</p>"],

 "ACL2____TIME-TRACKER-TAU":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic explains messages printing by the \u000A theorem prover about the <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>, as follows.</p> \u000A \u000A <p>During a proof you may see a message such as the following.</p> \u000A \u000A <code>TIME-TRACKER-NOTE [:TAU]: Elapsed runtime in tau is 4.95 secs; see\u000A:DOC time-tracker-tau.</code> \u000A \u000A <p>Just below a proof summary you may see a message such as the following.</p> \u000A \u000A <code>TIME-TRACKER-NOTE [:TAU]: For the proof above, the total runtime spent\u000Ain the tau system was 30.01 seconds.  See :DOC time-tracker-tau.</code> \u000A \u000A <p>Both of these messages are intended to let you know that certain prover \u000A heuristics (see <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>) may be slowing proofs down more than they \u000A are helping.  If you are satisfied with the prover's performance, you may \u000A ignore these messages or even turn them off by disabling time-tracking \u000A entirely (see <see topic=\"ACL2____TIME-TRACKER\">time-tracker</see>).  Otherwise, here are some actions that you \u000A can take to solve such a performance problem.</p> \u000A \u000A <p>The simplest solution is to disable the tau-system, in either of the \u000A following equivalent ways.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart tau-system)))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>)))</code> \u000A \u000A <p>But if you want to leave the tau-system enabled, you could investigate the \u000A possibility is that the tau-system is causing an expensive <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode function to be executed.  You can diagnose that problem by \u000A observing the rewriter &mdash; see <see topic=\"ACL2____DMR\">dmr</see> &mdash; or by interrupting the \u000A system and getting a backtrace (see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>).  A solution \u000A in that case is to disable the executable-counterpart of that function, for \u000A example in either of these equivalent ways.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart foo)))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (foo)))</code> \u000A \u000A <p>As a result, the tau-system will be weakened, but perhaps only \u000A negligibly.</p> \u000A \u000A <p>In either case above, you may prefer to provide <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> \u000A hints rather than <v>:in-theory</v> <see topic=\"ACL2____EVENTS\">events</see>; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>A more sophisticated solution is to record values of the <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>-mode function in question, so that the tau-system will look up the \u000A necessary values rather than calling the function, whether or not the \u000A executable-counterpart of that function is enabled.  Here is an example of a \u000A lemma that can provide such a solution.  See <see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (foo 0)\u000A       (foo 17)\u000A       (foo t)\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (foo '(a b c))))\u000A  :rule-classes :tau-system)</code>"],

 "ACL2____SET-COMPILE-FNS":[["Compilation"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see> t)    ; new functions compiled after DEFUN\u000A(<see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see> nil)  ; new functions not compiled after DEFUN</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____COMP\">comp</see>, because it may be more efficient in some Common Lisps \u000A to compile many functions at once rather than to compile each one as you go \u000A along.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-COMPILE-FNS\">set-compile-fns</see> term)</code> \u000A \u000A <p>where <v>term</v> is a variable-free term that evaluates to <v>t</v> or \u000A <v>nil</v>.  This macro is equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :compile-fns term)</code> \u000A \u000A <p>and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  However, \u000A unlike the above simple call of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> event function (see <see topic=\"ACL2____TABLE\">table</see>), no output results from a <v>set-compile-fns</v> event.</p> \u000A \u000A <p><v>Set-compile-fns</v> may be thought of as an event that merely sets a flag \u000A to <v>t</v> or <v>nil</v>.  The flag's effect is felt when functions are defined, \u000A as with <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>.  If the flag is <v>t</v>, functions are automatically \u000A compiled after they are defined, as are their executable counterparts (see \u000A <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see>).  Otherwise, functions are not automatically \u000A compiled.  Exception: The flag has no effect when explicit compilation is \u000A suppressed; see <see topic=\"ACL2____COMPILATION\">compilation</see>.</p> \u000A \u000A <p>Because <v>set-compile-fns</v> is an event, the old value of the flag is \u000A restored when a <v>set-compile-fns</v> event is undone.</p> \u000A \u000A <p>Even when <v>:set-compile-fns t</v> has been executed, functions are not \u000A individually compiled when processing an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> event.  If you \u000A wish to include a book of compiled functions, we suggest that you first \u000A certify it with the <see topic=\"ACL2____COMPILATION\">compilation</see> flag set (see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see>) or \u000A else compile the book by supplying the appropriate <v>load-compiled-file</v> \u000A argument to <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  More generally, <see topic=\"ACL2____COMPILATION\">compilation</see> via \u000A <v>set-compile-fns</v> is suppressed when the <see topic=\"ACL2____STATE\">state</see> global variable \u000A <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> has value <v>'</v><tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p>"],

 "ACL2____PKG-IMPORTS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-pkg-imports</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A           (<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> x)\u000A         nil))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)</code> \u000A \u000A <p><v>(<see topic=\"ACL2____PKG-IMPORTS\">Pkg-imports</see> pkg)</v> returns a duplicate-free list of all symbols \u000A imported into <v>pkg</v>, which should be the name of a package known to ACL2. \u000A For example, suppose <v>&quot;MY-PKG&quot;</v> was created by</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;MY-PKG&quot; '(ACL2::ABC LISP::CAR)).</code> \u000A \u000A <p>Then <v>(<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> &quot;MY-PKG&quot;)</v> equals the list <v>(ACL2::ABC\u000A LISP::CAR)</v>.</p> \u000A \u000A <p>If <v>pkg</v> is not a string, then <v>(<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> pkg)</v> is <v>nil</v>.  If \u000A <v>pkg</v> is a string but not the name of a package known to ACL2, then the \u000A value of the form <v>(<see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see> pkg)</v> is unspecified, and it evaluation \u000A will fail to yield a value.  By ``the symbols imported into <v>pkg</v>'' we mean \u000A the symbols imported into <v>pkg</v> by the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event that introduced \u000A <v>pkg</v>.  There are no imports for built-in packages except for the \u000A <v>&quot;ACL2&quot;</v> package, which imports the symbols in the list value of the \u000A constant <v>*common-lisp-symbols-from-main-lisp-package*</v>.  In particular, \u000A this is the case for the <v>&quot;COMMON-LISP&quot;</v> package.  Users familiar with \u000A Common Lisp may find this surprising, since in actual Common Lisp \u000A implementations it is often the case that many symbols in that package are \u000A imported from other packages.  However, ACL2 treats all symbols in the \u000A constant <v>*common-lisp-symbols-from-main-lisp-package*</v> as having a <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of <v>&quot;COMMON-LISP&quot;</v>, as though they were not \u000A imported.  ACL2 admits a symbol imported into in the <v>&quot;COMMON-LISP&quot;</v> \u000A package only if it belongs to that list: any attempt to read any other symbol \u000A imported into the <v>&quot;COMMON-LISP&quot;</v> package, or to produce such a symbol \u000A with <tt><see topic=\"ACL2____INTERN_42\">intern$</see></tt> or <tt><see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see></tt>, will cause an \u000A error.</p> \u000A \u000A <p>The following axioms formalize properties of <v>pkg-imports</v> discussed \u000A above (use <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> to view them).</p> \u000A \u000A <code>symbol-package-name-intern-in-package-of-symbol\u000Aintern-in-package-of-symbol-is-identity\u000Asymbol-listp-pkg-imports\u000Ano-duplicatesp-pkg-imports\u000Acompletion-of-pkg-imports</code>"],

 "ACL2____DOC":[["Documentation"],"ACL2 Sources","ACL2","<p>The <v>:doc</v> command may be used at the ACL2 prompt to access the \u000A ACL2 system <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.  Usually it may also access documentation \u000A defined in books.  However, most users will probably access the ACL2 \u000A documentation in other ways; see <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.  In particular, \u000A consider using the <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a>, for topics documented in \u000A the ACL2 community <see topic=\"ACL2____BOOKS\">books</see> or in the ACL2 system (where the latter are \u000A rearranged).</p> \u000A \u000A <p>Alternatively, consider using the ACL2-doc Emacs browser; see <see topic=\"ACL2____ACL2-DOC\">ACL2-doc</see>.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;:doc DEFTHM          ; print documentation of DEFTHM\u000AACL2 !&gt;:doc logical-name    ; print documentation of LOGICAL-NAME\u000A\u000AGeneral Form:\u000AACL2&gt;:doc name</code>"],

 "ACL2____BREAK-ON-ERROR":[["Trace"],"ACL2 Sources","ACL2","<code>General forms:\u000A(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> t)    ; installs a trace causing a continuable error (break)\u000A                      ;   when an error is invoked by ACL2.\u000A(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>)      ; same as above\u000A(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> :all) ; same as above, but even when inside the prover\u000A(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see> nil)  ; uninstall any above trace</code> \u000A \u000A <p><v>(<see topic=\"ACL2____BREAK-ON-ERROR\">Break-on-error</see>)</v> generates a suitable trace of error functions. \u000A Evaluate <v>(<see topic=\"ACL2____TRACE_42\">trace$</see>)</v> after <v>(<see topic=\"ACL2____BREAK-ON-ERROR\">break-on-error</see>)</v> if you want to see the \u000A specific trace forms (which you can modify and then submit directly to \u000A <v>trace$</v>, if you wish).  This <see topic=\"COMMON-LISP____TRACE\">trace</see> should cause entry to the Lisp \u000A debugger whenever ACL2 calls its error routines, except for certain errors \u000A when inside the theorem prover, and also at those times if option :all is \u000A supplied.</p> \u000A \u000A <p>NOTE: For technical reasons, you may see some error messages more than \u000A once.</p> \u000A \u000A <p>Finally, note that you are welcome to define your own version of \u000A <v>break-on-error</v> by modifying a copy of the source definition (search for \u000A ``<v>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> break-on-error</v>'' in ACL2 source file other-events.lisp). \u000A Please feel free to send your version of <v>break-on-error</v> to the ACL2 \u000A implementors, for possible inclusion into ACL2.</p> \u000A \u000A <p><v>Break-on-error</v> is implmented using ACL2 <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  See <see topic=\"ACL2____TRACE_12\">trace!</see> if you want an explanation of the ``<v>TTAG NOTE</v>'' that is \u000A printed.</p> \u000A \u000A <p>The argument, if supplied, is evaluated and must evaluate to <v>t</v>, \u000A <v>nil</v>, or <v>:all</v>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> for how to get raw-Lisp backtrace \u000A information when an error occurs as a result of <v>break-on-error</v>, or even \u000A of a raw Lisp error, by calling <v>set-debugger-enable</v> with argument \u000A <v>:bt</v>, <v>:bt-break</v>, or <v>:break-bt</v>.  Note that for ACL2 errors (as \u000A opposed to raw Lisp errors), i.e. errors affected by <v>break-on-error</v>, all \u000A three of those keyword values are treated equivalently (and, all are ignored \u000A for non-ANSI GCL; see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>).</p>"],

 "ACL2-PC____IN-THEORY":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see>\u000A   (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory) '(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> binary-append)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> &amp;optional atom-or-theory-expression)</code> \u000A \u000A <p>If the argument is not supplied, then this command sets the current \u000A proof-checker theory (see below for explanation) to agree with the current \u000A ACL2 theory.  Otherwise, the argument should be a theory expression, and in \u000A that case the proof-checker theory is set to the value of that theory \u000A expression.</p> \u000A \u000A <p>The current proof-checker theory is used in all calls to the ACL2 theorem \u000A prover and rewriter from inside the proof-checker.  Thus, the most recent \u000A <v>in-theory</v> instruction in the current <v>state-stack</v> has an effect in \u000A the proof-checker totally analogous to the effect caused by an <v>in-theory</v> \u000A hint or event in ACL2.  All <v>in-theory</v> instructions before the last are \u000A ignored, because they refer to the current theory in the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, \u000A not to the existing proof-checker theory.  For example:</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____ENABLE\">enable</see> bar)\u000A (<see topic=\"ACL2____UNION-THEORIES\">UNION-THEORIES</see> (<see topic=\"ACL2____CURRENT-THEORY\">CURRENT-THEORY</see> :HERE)\u000A                 '(BAR))\u000AACL2 !&gt;:trans1 (<see topic=\"ACL2____CURRENT-THEORY\">CURRENT-THEORY</see> :HERE)\u000A (CURRENT-THEORY-FN :HERE WORLD)\u000AACL2 !&gt;</code> \u000A \u000A <p>Thus <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> bar))</v> modifies the current theory of the \u000A current ACL2 world.  So for example, suppose that <v>foo</v> is disabled outside \u000A the proof checker and you execute the following instructions, in this \u000A order.</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> foo))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> bar))</code> \u000A \u000A <p>Then after the second of these, <v>bar</v> will be enabled in the \u000A proof-checker, but <v>foo</v> will be disabled.  The reason is that \u000A <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____ENABLE\">enable</see> bar))</v> instructs the proof-checker to modify the \u000A current theory (from outside the proof-checker, not from inside the \u000A proof-checker) by enabling <v>bar</v>.</p> \u000A \u000A <p>Note that <v>in-theory</v> instructions in the proof-checker have no effect \u000A outside the proof-checker's interactive loop.</p> \u000A \u000A <p>If the most recent <v>in-theory</v> instruction in the current state of the \u000A proof-checker has no arguments, or if there is no <v>in-theory</v> instruction \u000A in the current state of the proof-checker, then the proof-checker will use the \u000A current ACL2 theory.  This is true even if the user has interrupted the \u000A interactive loop by exiting and changing the global ACL2 theory.  However, if \u000A the most recent <v>in-theory</v> instruction in the current state of the \u000A proof-checker had an argument, then global changes to the current theory will \u000A have no effect on the proof-checker state.</p>"],

 "COMMON-LISP____INTERN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____INTERN\">intern</see> symbol-name symbol-package-name)</v> returns a symbol with \u000A the given <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> and the given <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt>.  We \u000A restrict Common Lisp's <v>intern</v> so that the second argument is either the \u000A symbol *main-lisp-package-name*, the value of that constant, or is one of \u000A \"ACL2\", \"ACL2-INPUT-CHANNEL\", \"ACL2-OUTPUT-CHANNEL\", or \"KEYWORD\".  To \u000A avoid that restriction, see <see topic=\"ACL2____INTERN_42\">intern$</see>.</p> \u000A \u000A <p>In ACL2 <v>intern</v> is actually implemented as a macro that expands to a \u000A call of a similar function whose second argument is a symbol.  Invoke <v>:pe\u000A intern</v> to see the definition, or see <see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see>.</p> \u000A \u000A <p>To see why is <v>intern</v> so restricted consider <v>(<see topic=\"COMMON-LISP____INTERN\">intern</see> &quot;X&quot; &quot;P&quot;)</v>. \u000A In particular, is it a symbol and if so, what is its <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt>?  One is tempted to say ``yes, it is a symbol in the \u000A package <v>&quot;P&quot;</v>.''  But if package <v>&quot;P&quot;</v> has not yet been defined, \u000A that would be premature because the imports to the package are unknown.  For \u000A example, if <v>&quot;P&quot;</v> were introduced with</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;P&quot; '(LISP::X))</code> \u000A \u000A <p>then in Common Lisp <v>(<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> (<see topic=\"COMMON-LISP____INTERN\">intern</see> &quot;X&quot; &quot;P&quot;))</v> returns \u000A <v>&quot;LISP&quot;</v>.</p> \u000A \u000A <p>The obvious restriction on <v>intern</v> is that its second argument be the \u000A name of a package known to ACL2.  We cannot express such a restriction \u000A (except, for example, by limiting it to those packages known at some fixed \u000A time, as we do).  Instead, we provide <tt><see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see></tt> \u000A which requires a ``witness symbol'' for the package instead of the package. \u000A The witness symbol is any symbol (expressible in ACL2) and uniquely specifies \u000A a package necessarily known to ACL2.</p>"],

 "ACL2____PARALLELISM-AT-THE-TOP-LEVEL":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>Calls of parallelism primitives made explicitly in the ACL2 top-level loop, \u000A as opposed to inside function bodies, will never cause parallel execution. \u000A Such calls will either execute with serial execution or will cause an error; \u000A see <see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see>.  For a way around this restriction, see \u000A <see topic=\"ACL2____TOP-LEVEL\">top-level</see>.</p> \u000A \u000A <p>Consider for example the following call of <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> in the ACL2 \u000A top-level loop.  Instead of executing <v>pargs</v>, ACL2 macroexpands away this \u000A call, leaving us with serial execution of the arguments to the <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> \u000A call, or else causes an error (see <see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see>).  If there \u000A is no error, then</p> \u000A \u000A <code>(<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (expensive-fn-1 4) (expensive-fn-2 5)))</code> \u000A \u000A <p>expands into:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CONS\">cons</see> (expensive-fn-1 4) (expensive-fn-2 5))</code> \u000A \u000A <p>One trivial way to enable parallel execution of a form is to surround it \u000A with a call to macro <see topic=\"ACL2____TOP-LEVEL\">top-level</see>.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____TOP-LEVEL\">top-level</see> (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (expensive-fn-1 4) (expensive-fn-2 5))))</code> \u000A \u000A <p>Then in an executable image that supports parallel execution &mdash; see \u000A <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see> for instructions on how to build such an executable \u000A &mdash; <v>(expensive-fn-1 4)</v> and <v>(expensive-fn-2 5)</v> can evaluate in \u000A parallel.</p> \u000A \u000A <p>A second way to enable parallel execution of a form is to place it inside a \u000A function body.  For example, consider the following definition.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y)\u000A  (<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (expensive-fn-1 x) (expensive-fn-2 y))))</code> \u000A \u000A <p>Then in an executable image that supports parallel execution, submission of \u000A the form <v>(foo 4 5)</v> can cause parallel execution of <v>(expensive-fn-1\u000A 4)</v> and <v>(expensive-fn-2 5)</v>.</p> \u000A \u000A <p>Note that <see topic=\"ACL2____GUARD\">guard</see>s need not be verified in order to obtain <see topic=\"ACL2____PARALLEL\">parallel</see> execution.  The only restrictions on parallel execution are to use \u000A an executable supporting it, to avoid calling parallelism primitives directly \u000A in the top-level loop, to have sufficient resources (especially, threads) \u000A available, and to avoid explicitly disabling parallel execution (see <see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see>).</p>"],

 "ACL2____GUARDS-FOR-SPECIFICATION":[["Guard"],"ACL2 Sources","ACL2","<p>A use of guard verification that has nothing to do with efficiency \u000A is as a way to gain confidence in specifications.  This use has the feel of \u000A ``types'' in many traditional <see topic=\"ACL2____PROGRAMMING\">programming</see> languages, though guards \u000A allow much greater expressiveness than most systems of types \u000A (and unfortunately, as a result they are not syntactically checkable).</p> \u000A \u000A <p>For more discussion of guards in general, see <see topic=\"ACL2____GUARD\">guard</see>.</p> \u000A \u000A <p>Suppose you have written a collection of function definitions that are \u000A intended to specify the behavior of some system.  Perhaps certain functions \u000A are only intended to be called on certain sorts of inputs, so you attach \u000A guards to those functions in order to ``enforce'' that requirement.  And then, \u000A you verify the guards for all those functions.</p> \u000A \u000A <p>Then what have you gained, other than somewhat increased efficiency of \u000A execution (as explained above), which quite possibly isn't your main concern? \u000A You have gained the confidence that when evaluating any call of a \u000A (specification) function whose arguments satisfy that function's guard, all \u000A subsequent function calls during the course of evaluation will have this same \u000A property, that the arguments satisfy the guard of the calling function.  In \u000A logical terms, we can say that the equality of the original call with the \u000A returned value is provable from weakened versions of the definitions, where \u000A each definitional axiom is replaced by an implication whose antecedent is the \u000A requirement that the arguments satisfy the guard and whose consequent is the \u000A original axiom.  For example,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</code> \u000A \u000A <p>originally generates the axiom</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x)\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))</code> \u000A \u000A <p>but in fact, when evaluation involves no guard violation then the following \u000A weaker axiom suffices in the justification of the evaluation.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo x)\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> 1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))</code> \u000A \u000A <p>If you are following links to read this documentation as a hypertext style \u000A document, then please see <see topic=\"ACL2____GUARD-MISCELLANY\">guard-miscellany</see>.  This concludes our \u000A discussion of guards with miscellaneous remarks, and also contains pointers to \u000A related topics.</p>"],

 "ACL2____COMMON-LISP":[["About-ACL2"],"ACL2 Sources","ACL2","<p>ACL2 is a logic, a theorem prover, and a programming language based \u000A on Common Lisp.  A connection with Common Lisp is established with guards (see \u000A <see topic=\"ACL2____GUARD\">guard</see>).</p> \u000A \u000A <p>However, here we document potential deviations from Common Lisp semantics \u000A even in the presence of verified guards.  Our view is that these deviations \u000A are extremely unlikely to manifest; indeed, as of this writing we are unaware \u000A of any cases in which these issues arise in practice.  However, we feel \u000A obligated to acknowledge their possibility, which could result in surprises \u000A during evaluation or even proof.</p> \u000A \u000A <p>The Common Lisp spec allows certain predicates to return what it calls \u000A ``generalized Booleans,'' which are really arbitrary values that are to be \u000A viewed as either <v>nil</v> or non-<v>nil</v>.  However, in ACL2 these functions \u000A are assumed to return <v>nil</v> or <v>t</v>.  For details,see <see topic=\"ACL2____GENERALIZED-BOOLEANS\">generalized-booleans</see>.</p> \u000A \u000A <p>The execution of forms with <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions can result \u000A in calls of functions on arguments that do not satisfy their <see topic=\"ACL2____GUARD\">guard</see>s. \u000A In practice, this simply causes hard Lisp errors.  But in principle one could \u000A imagine a damaged Lisp image that operates incorrectly.  See <see topic=\"ACL2____DEFUN-MODE-CAVEAT\">defun-mode-caveat</see>.</p> \u000A \u000A <p>The Common Lisp spec, specifically \u000A Section <a href='http://www.lispworks.com/documentation/HyperSpec/Body/03_bbc.htm'>Section 3.2.2.3</a> \u000A of the <a href='http://www.lispworks.com/documentation/HyperSpec/Front'>Common Lisp Hyperspec</a>, \u000A allows for undefined results when a function is ``multiply defined'' in a \u000A compiled file.  ACL2 allows redundant <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s in a book, and in general \u000A <see topic=\"ACL2____BOOKS\">books</see> are compiled by <v>certify-book</v> (but see <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> \u000A and see <see topic=\"ACL2____COMPILATION\">compilation</see> for how to control such compilation).  Moreover, \u000A ACL2 provides a redefinition capability (see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see> and \u000A see <see topic=\"ACL2____REDEF\">redef</see>), and the above section also allows for undefined results \u000A when a function is defined in a compiled file and then redefined, \u000A presumably (for example) because of inlining.</p>"],

 "ACL2____BREAKS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example:\u000ABroken at PROVE.  Type :H for Help.\u000A&gt;&gt;:Q\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>You may interrupt the system by typing various control character sequences. \u000A The precise sequences are determined by the host Lisp and operating system \u000A environment.  For example, in GCL and Allegro Common Lisp, a console interrupt \u000A is caused by typing ``<v>ctrl-c</v>''.  If, however, the GCL or Allegro is \u000A running in an Emacs shell buffer, one must type ``ctrl-c ctrl-c''.</p> \u000A \u000A <p>If a break occurs, for example because of a bug in ACL2 or a user \u000A interrupt, the break will run a Common Lisp read-eval-print loop, not an ACL2 \u000A read-eval-print loop.  This may not be obvious if the <see topic=\"ACL2____PROMPT\">prompt</see>s in the \u000A two loops are similar.  Because you are typing to a Common Lisp evaluator, you \u000A must be careful.  It is possible to damage your ACL2 state in irreparable ways \u000A by executing non-ACL2 Common Lisp.  It is even possible to disrupt and render \u000A inaccurate the interrupted evaluation of a simple ACL2 expression.</p> \u000A \u000A <p>For ACL2 built on most host Common Lisps, you will see the string <v>[RAW\u000A LISP]</v> in the <see topic=\"ACL2____PROMPT\">prompt</see> at a break, to emphasize that one is inside a \u000A break and hence should quit from the break.  For some host Common Lisps, the \u000A top-level prompt also contains the string <v>[RAW LISP]</v>.  See <see topic=\"ACL2____PROMPT\">prompt</see> \u000A for how to control printing of that string.</p> \u000A \u000A <p>The most reliable way to return to the ACL2 top level is by executing the \u000A following command: <v>(</v><tt><see topic=\"ACL2____ABORT_12\">abort!</see></tt><v>)</v>.  Appropriate cleanup will then \u000A be done, which should leave you in an appropriate state.</p> \u000A \u000A <p>However, you may be able to quit from the break in the normal Lisp manner \u000A (as with <v>:q</v> in GCL or CCL, <v>:reset</v> in Allegro CL, and <v>q</v> in CMU \u000A CL).  If this attempt to quit is successful, it will return you to the \u000A innermost ACL2 read-eval-print loop, with appropriate cleanup performed first. \u000A Note that if you are within a <tt><see topic=\"ACL2____BRR\">brr</see></tt> environment when the break occurs, \u000A quitting from the break will only return you to that environment, not to the \u000A top of ACL2's read-eval-print loop.</p>"],

 "ACL2____STANDARD-OI":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Standard-oi</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____STANDARD-OI\">standard-oi</see> state)</v> and the updater is <v>(set-standard-oi\u000A val state)</v>.  <v>Standard-oi</v> must be an open object input channel, a true \u000A list of objects, or a list of objects whose last <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is an open object \u000A input channel.  It is from this source that <tt><see topic=\"ACL2____LD\">ld</see></tt> takes the input forms \u000A to process.  When <tt><see topic=\"ACL2____LD\">ld</see></tt> is called, if the value specified for \u000A <v>standard-oi</v> is a string or a list of objects whose last <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> is a \u000A string, then <tt><see topic=\"ACL2____LD\">ld</see></tt> treats the string as a file name and opens an object \u000A input channel from that file, where the connected book directory (see <see topic=\"ACL2____CBD\">cbd</see>) is used to resolve relative pathnames.  The channel opened by <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A is closed by <tt><see topic=\"ACL2____LD\">ld</see></tt> upon termination.</p> \u000A \u000A <p>``Standard-oi'' stands for ``standard object input.'' The read-eval-print \u000A loop in <tt><see topic=\"ACL2____LD\">ld</see></tt> reads the objects in <v>standard-oi</v> and treats them as \u000A forms to be evaluated.  The initial value of <v>standard-oi</v> is the same as \u000A the value of <tt><see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see></tt> (see <see topic=\"ACL2_____A2STANDARD-OI_A2\">*standard-oi*</see>).</p>"],

 "ACL2____SET-SAVED-OUTPUT":[["Set-gag-mode"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> t t)    ; save proof output for later, but inhibit it now\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> t :all) ; save proof output for later, but inhibit all\u000A                          ;   output (except WARNING!, for critical warnings,\u000A                          ;   and ERROR, unless these are already inhibited)\u000A:set-saved-output t :all  ; same as the line above\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> t nil)  ; save proof output for later, but print it now too\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> nil t)  ; do not save proof output, and inhibit it\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> nil nil); do not save proof output or inhibit output\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> nil :same), (<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> t :same)\u000A                          ; save proof output or not, as indicated, but do\u000A                          ;   not change which output is inhibited\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> nil :normal)\u000A                          ; the behavior when ACL2 first starts up: do not\u000A                          ;   save output, and only inhibit proof-tree output\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> t '(warning observation proof-tree prove))\u000A                          ; save proof output for later, and inhibit the\u000A                          ;   indicated kinds of output\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> save-flg inhibit-flg)</code> \u000A \u000A <p>Parameter <v>save-flg</v> is <v>t</v> to cause output to be saved for later \u000A display using <v>pso</v> or <v>pso!</v>; see <see topic=\"ACL2____PSO\">pso</see> and see <see topic=\"ACL2____PSO_12\">pso!</see>, and \u000A see the documentation for <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands of the same names. \u000A Set <v>save-flg</v> to <v>nil</v> to turn off this feature; except, it always \u000A stays on in proof-checker sessions entered with <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>.  The other \u000A argument, <v>inhibit-flg</v>, controls whether output should be inhibited when \u000A it is created (normally, during a proof attempt).  So a common combination is \u000A to set both arguments to <v>t</v>, to indicate that output should be suppressed \u000A for now but saved for printing with <tt><see topic=\"ACL2____PSO\">pso</see></tt> or <tt><see topic=\"ACL2____PSO_12\">pso!</see></tt>.  The examples \u000A above give a good summary of the functionality for the second argument.</p> \u000A \u000A <p>Saved output is cleared at the start of every event, and also at the start \u000A of every <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands that invoke the prover.  Note that \u000A interactive <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands, that is, from a proof-checker \u000A session entered with <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, are always run with output saved.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>; and see <see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see>, which \u000A causes subgoal numbers to be printed during proof attempts when output is \u000A inhibited.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> if you want to inhibit certain output \u000A from the prover but not other output (e.g., not the summary), and you don't \u000A want to save any output.</p>"],

 "ACL2____REBUILD":[["Other"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot;)\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot; t)\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot; t :dir :system)\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot; :all)\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot; :query)\u000AACL2 !&gt;(<see topic=\"ACL2____REBUILD\">rebuild</see> &quot;project.lisp&quot; 'lemma-22)</code> \u000A \u000A <p><v>Rebuild</v> allows you to assume all the <see topic=\"ACL2____COMMAND\">command</see>s in a given file \u000A or list, supplied in the first argument.  Because <v>rebuild</v> processes an \u000A arbitrary sequence of <see topic=\"ACL2____COMMAND\">command</see>s with <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> <v>t</v>, it \u000A is unsound!  However, if each of these <see topic=\"ACL2____COMMAND\">command</see>s is in fact admissible, \u000A then processing them with <v>rebuild</v> will construct the same logical <see topic=\"ACL2____STATE\">state</see> that you would be in if you typed each <see topic=\"ACL2____COMMAND\">command</see> and waited \u000A through the proofs again.  Thus, <v>rebuild</v> is a way to reconstruct a <see topic=\"ACL2____STATE\">state</see> previously obtained by proving your way through the <see topic=\"ACL2____COMMAND\">command</see>s.</p> \u000A \u000A <p>The second, optional argument to <v>rebuild</v> is a ``filter'' (see <see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see>) that lets you specify which <see topic=\"ACL2____COMMAND\">command</see>s to process. \u000A You may specify <v>t</v>, <v>:all</v>, <v>:query</v>, or a new logical name.  <v>t</v> \u000A and <v>:all</v> both mean that you expect the entire file or list to be \u000A processed.  <v>:query</v> means that you will be asked about each <see topic=\"ACL2____COMMAND\">command</see> \u000A in turn.  A new name means that all <see topic=\"ACL2____COMMAND\">command</see>s will be processed as long \u000A as the name is new, i.e., <v>rebuild</v> will stop processing <see topic=\"ACL2____COMMAND\">command</see>s \u000A immediately after executing a <see topic=\"ACL2____COMMAND\">command</see> that introduces name. \u000A <v>Rebuild</v> will also stop if any <see topic=\"ACL2____COMMAND\">command</see> causes an error.  You may \u000A therefore wish to plant an erroneous form in the file, e.g., <v>(<see topic=\"ACL2____MV\">mv</see> t nil\u000A state)</v>, (see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see>), to cause <v>rebuild</v> to stop there. \u000A The form <v>(<see topic=\"ACL2____I-AM-HERE\">i-am-here</see>)</v> is such a pre-defined form.  If you do not specify a \u000A filter, <v>rebuild</v> will query you for one.</p> \u000A \u000A <p>Inspection of the definition of <v>rebuild</v>, e.g., via <v>:</v><tt><see topic=\"ACL2____PC\">pc</see></tt> \u000A <v>rebuild-fn</v>, will reveal that it is just a glorified call to the function \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt>.  See <see topic=\"ACL2____LD\">ld</see> if you find yourself wishing that <v>rebuild</v> had \u000A additional functionality.</p> \u000A \u000A <p>If you supply the above ``filter'' argument, then you may also supply the \u000A keyword argument <v>:dir</v>, which is then passed to <v>ld</v>; see <see topic=\"ACL2____LD\">ld</see>.</p>"],

 "ACL2____QUANTIFIERS":[["Defun-sk"],"ACL2 Sources","ACL2","<p>ACL2 supports first-order quantifiers <tt><see topic=\"ACL2____EXISTS\">exists</see></tt> and <tt><see topic=\"ACL2____FORALL\">forall</see></tt> by way of the <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> event.  However, proof support for \u000A quantification is quite limited.  Therefore, you may prefer using recursion in \u000A place of <v>defun-sk</v> when possible (following common ACL2 practice).</p> \u000A \u000A <p>For example, the notion ``every member of <v>x</v> has property <v>p</v>'' can \u000A be defined either with recursion or explicit quantification, but proofs may be \u000A simpler when recursion is used.  We illustrate this point with two proofs of \u000A the same informal claim, one of which uses recursion which the other uses \u000A explicit quantification.  Notice that with recursion, the proof goes through \u000A fully automatically; but this is far from true with explicit quantification \u000A (especially notable is the ugly hint).</p> \u000A \u000A <p>The informal claim for our examples is: If every member <v>a</v> of each of \u000A two lists satisfies the predicate <v>(p a)</v>, then this holds of their <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>; and, conversely.</p> \u000A \u000A <p>See <see topic=\"ACL2____QUANTIFIERS-USING-RECURSION\">quantifiers-using-recursion</see> for a solution to this example using \u000A recursion.</p> \u000A \u000A <p>See <see topic=\"ACL2____QUANTIFIERS-USING-DEFUN-SK\">quantifiers-using-defun-sk</see> for a solution to this example using \u000A <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>.  Also See <see topic=\"ACL2____QUANTIFIERS-USING-DEFUN-SK-EXTENDED\">quantifiers-using-defun-sk-extended</see> for an \u000A elaboration on that solution.</p> \u000A \u000A <p>But perhaps first, see <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> for an ACL2 utility to introduce \u000A first-order quantification in a direct way.  Examples of the use of \u000A <v>defun-sk</v> are also available: see <see topic=\"ACL2____DEFUN-SK-EXAMPLE\">defun-sk-example</see> and see <see topic=\"ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE\">Tutorial4-Defun-Sk-Example</see> for basic examples, and see <see topic=\"ACL2____QUANTIFIER-TUTORIAL\">quantifier-tutorial</see> for a more complete, careful beginner's introduction that \u000A takes you through typical kinds of quantifier-based reasoning in ACL2.</p>"],

 "ACL2____NOTE8-UPDATE":[["Release-notes"],"ACL2 Sources","ACL2","<p>ACL2 can now use Ordered Binary Decision Diagram technology.  See \u000A <see topic=\"ACL2____BDD\">bdd</see>.  There is also a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> <v>bdd</v> command.</p> \u000A \u000A <p>ACL2 is now more respectful of the intention of the function <tt><see topic=\"ACL2____HIDE\">hide</see></tt>. \u000A In particular, it is more careful not to dive inside any call of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> \u000A during equality substitution and case splitting.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>) <tt><see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see></tt> may now be \u000A used to turn off printing of input forms during processing of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> forms, by setting it to the value \u000A <v>:never</v>, i.e., <v>(set-ld-pre-eval-print :never state)</v>.  See <see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see>.</p> \u000A \u000A <p>The TUTORIAL documentation section (now obsolete) has, with much help from \u000A Bill Young, been substantially improved to a bona fide introduction.</p> \u000A \u000A <p>The term pretty-printer has been modified to introduce <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X Y)</v> as an \u000A abbreviation for <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> Y X))</v>.</p> \u000A \u000A <p>Forward chaining and linear arithmetic now both benefit from the evaluation \u000A of ground subterms.</p> \u000A \u000A <p>A new macro <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt> has been defined.  This should \u000A be used when setting the <see topic=\"ACL2____STATE\">state</see> global <v>inhibit-output-lst</v>; see \u000A <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> and see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>The test for redundancy in definitions includes the <see topic=\"ACL2____GUARD\">guard</see> and type \u000A declarations.  See <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____GENERALIZED-BOOLEANS\">generalized-booleans</see> for a discussion of a potential soundness \u000A problem for ACL2 related to the question: Which Common Lisp functions are \u000A known to return Boolean values?</p> \u000A \u000A <p>Here we will put some less important changes, additions, and so on.</p> \u000A \u000A <p>A bug has been fixed so that now, execution of <v>:comp t</v> (see <see topic=\"ACL2____COMP\">comp</see>) correctly handles non-standard characters.</p> \u000A \u000A <p>A bug in <tt><see topic=\"COMMON-LISP____DIGIT-CHAR-P\">digit-char-p</see></tt> has been fixed, so that the ``default'' is \u000A <v>nil</v> rather than <v>0</v>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____TRUE-LISTP\">True-listp</see></tt> now tests the final <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> against <v>nil</v> using \u000A <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt> instead of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, for improved efficiency.  The logical \u000A meaning is, however, unchanged.</p> \u000A \u000A <p><tt><see topic=\"ACL2____PUT-ASSOC-EQUAL\">Put-assoc-equal</see></tt> has been added to the logic (it used to have \u000A <v>:</v><tt><see topic=\"ACL2____DEFUN-MODE\">defun-mode</see></tt> <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, and has been documented.</p>"],

 "ACL2____NOTE-3-5_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please also see <see topic=\"ACL2____NOTE-3-5\">note-3-5</see> for changes in Version 3.5 of \u000A  ACL2.</p> \u000A \u000A <p>This release incorporates improvements from Ruben Gamboa in support for \u000A non-standard analysis in ACL2(r), in the following ways:</p> \u000A \u000A <p>ACL2(r) now supports non-classical objects that are not also numeric, e.g., \u000A non-classical cons pairs.  Consequently, the built-in <v>standard-numberp</v> \u000A has been replaced with <tt><see topic=\"ACL2____STANDARDP\">standardp</see></tt>.</p> \u000A \u000A <p>If <v>f</v> is a classical function, the value <v>(f x1 ... xn)</v> is \u000A guaranteed to be standard when the <v>xi</v> are standard.  ACL2(r) can now \u000A recognize this fact automatically, using <v>defun-std</v>.  For example, the \u000A following can be used to assert that the square root of 2 is a standard \u000A value.</p> \u000A \u000A <code>(defthm-std sqrt-2-rational\u000A  (<see topic=\"ACL2____STANDARDP\">standardp</see> (acl2-sqrt 2)))</code> \u000A \u000A <p>More generally, the expression <v>(f x1 ... xn)</v> can contain free \u000A variables, but the result is guaranteed to be standard when the variables take \u000A on standard variables, as in the following:</p> \u000A \u000A <code>(defthm-std sqrt-x-rational\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____STANDARDP\">standardp</see> x)\u000A           (<see topic=\"ACL2____STANDARDP\">standardp</see> (acl2-sqrt x))))</code> \u000A \u000A <p>A potential soundness bug in <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> was fixed.  Specifically, \u000A when a classical, constrained function is instantiated with a lambda \u000A expression containing free variables, it may produce non-standard values \u000A depending on the values of the free variables.  This means that the functional \u000A instantiation cannot be used to justify a non-classical theorem.  For example, \u000A consider the following sequence:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A  ((f (x) t))\u000A  (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x) x)))\u000A(defthm-std f-x-standard\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____STANDARDP\">standardp</see> x)\u000A           (<see topic=\"ACL2____STANDARDP\">standardp</see> (f x))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> plus-x-standard\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____STANDARDP\">standardp</see> x)\u000A           (<see topic=\"ACL2____STANDARDP\">standardp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x y)))\u000A  :hints ((&quot;Goal&quot;\u000A           :use ((:functional-instance f-x-standard\u000A                                       (f (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (x) (<see topic=\"COMMON-LISP_____B2\">+</see> x y))))))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> plus-x-eps-not-standard\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____STANDARDP\">standardp</see> x)\u000A           (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____STANDARDP\">standardp</see> (<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP_____F2\">/</see> (i-large-integer)))))))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> nil-iff-t\u000A  nil\u000A  :hints ((&quot;Goal&quot;\u000A          :use ((:instance plus-x-standard\u000A                           (y (<see topic=\"COMMON-LISP_____F2\">/</see> (i-large-integer))))\u000A                (:instance plus-x-eps-not-standard)))))</code> \u000A \u000A <p>ACL2(r) also supports the introduction of non-classical functions with \u000A <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>.  These behave just as normal functions introduced with \u000A <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt>, but they have a non-classical choice property.</p> \u000A \u000A <p>Finally, ACL2(r) now comes with a revamped library supporting non-standard \u000A analysis, still distributed separately as <v>books/nonstd/</v>.  The new library \u000A uses <tt><see topic=\"ACL2____DEFCHOOSE\">defchoose</see></tt> to state more natural and useful versions of the IVT, \u000A MVT, etc.  It also supports the introduction of inverse functions, e.g., \u000A logarithms.  Finally, the library has much more extensive support for \u000A differentiation.</p>"],

 "ACL2____NIL-GOAL":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>At the end of a failed proof, one typically sees so-called ``key \u000A checkpoints'' (see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>).  These may be annotated as \u000A follows.</p> \u000A \u000A <code>[NOTE: A goal of NIL was generated.  See :DOC nil-goal.]</code> \u000A \u000A <p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic gives some ideas about how to think about \u000A the situation described by that message: some goal has reduced to \u000A <v>nil</v>.</p> \u000A \u000A <p>Suppose then that you see the above NOTE.  If you look back at the proof \u000A log, even with <see topic=\"ACL2____GAG-MODE\">gag-mode</see> enabled, you will see a message saying that a \u000A goal of <v>NIL</v> ``has been generated''.  This may indicate that the original \u000A goal is not a theorem, since most of the prover's activity is to replace a \u000A goal by an equivalent conjunction of its child subgoals.  However, if some \u000A ancestor of the <v>nil</v> goal has undergone a process other than \u000A simplification or destructor elimination &mdash; fertilization (heuristic use \u000A of equalities), generalization, or elimination of irrelevance &mdash; then it \u000A is quite possible that the prover got to the <v>nil</v> goal by replacing a goal \u000A by a stronger (and perhaps false) conjunction of child subgoals.</p> \u000A \u000A <p>At present, if you are using <see topic=\"ACL2____GAG-MODE\">gag-mode</see> (the default) then you will \u000A need to issue the command <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> (``Print Saved Output'') if you \u000A want to see whether the situation above has occurred.  However, that might not \u000A be necessary.  A good rule of thumb is that if the <v>nil</v> goal is under more \u000A level of induction (e.g., with a prefix ``*i.j'' such as ``Subgoal \u000A *1.1/2.2''), then there is some likelihood that the situation above did indeed \u000A occur, and you can spend your time and energy looking at the key checkpoints \u000A printed in the summary to see if they suggest useful <see topic=\"ACL2____REWRITE\">rewrite</see> rules to \u000A prove.  On the other hand, if the <v>nil</v> goal is at the top level (e.g. with \u000A a name not starting with ``*'', such as ``Subgoal 3.2''), then the original \u000A conjecture is probably not a theorem.  If you do not quickly see why that is \u000A the case, then you might find it useful to issue the command <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> \u000A to see which case reduced to <v>nil</v>, in order to get insight about how the \u000A theorem might be falsified.</p>"],

 "ACL2____MORE":[["Legacy-documentation","Note1"],"ACL2 Sources","ACL2","<p>NOTE: The command <v>:more</v> only makes sense at the terminal.</p> \u000A \u000A <code>Example:\u000AACL2 !&gt;:more</code> \u000A \u000A <p>will continue printing whatever <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> was started by \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> or <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt>.</p> \u000A \u000A <p>When you type <v>:doc name</v>, for some documented <v>name</v>, the system \u000A responds by typing the one-liner and the notes sections of the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for <v>name</v>.  It then types ``<v>(<see topic=\"COMMON-LISP____TYPE\">type</see> :more for more, :more!\u000A for the rest)</v>''.  If you then type</p> \u000A \u000A <code>ACL2 !&gt;:more</code> \u000A \u000A <p>the system will start to print the details section of <v>name</v>.  The same \u000A thing could be achieved by typing <v>:more-doc name</v>, but that requires you \u000A to type name again.</p> \u000A \u000A <p>Similarly, if you have typed <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt> name, the system will \u000A print the first ``block'' of the details section and then print ``<v>(<see topic=\"COMMON-LISP____TYPE\">type</see>\u000A :more for more, :more! for the rest)</v>''.  Typing <v>:more</v> at that point \u000A will cause the next block of the details section to be printed.  Eventually \u000A <v>:more</v> will conclude by printing ``<v>*-</v>'' which is the indicator that \u000A the text has been exhausted.</p> \u000A \u000A <p>What is a ``block'' of text?  <v>:More</v> looks for the end of a paragraph \u000A (two adjacent newlines) after printing <v>n</v> lines.  If it doesn't find one \u000A before it has printed <v>k</v> lines, it just stops there.  <v>N</v> and <v>k</v> \u000A here are the values of the two <see topic=\"ACL2____STATE\">state</see> global variables \u000A <v>'more-doc-min-lines</v> and <v>'more-doc-max-lines</v>.  You may use <tt><see topic=\"ACL2_____04\">@</see></tt> \u000A and <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> to inspect and set these variables, e.g., <v>(<see topic=\"ACL2_____04\">@</see>\u000A more-doc-max-lines)</v> will return the current maximum number of lines printed \u000A by <v>:more</v> and <v>(<see topic=\"ACL2____ASSIGN\">assign</see> more-doc-max-lines 19)</v> will set it to 19.  On \u000A terminals having only 24 lines, we find min and max settings of 12 and 19 the \u000A most pleasant.</p> \u000A \u000A <p>If you want <v>:more</v> to print all of the details instead of feeding them \u000A to you one block at a time, type <v>:</v><tt><see topic=\"ACL2____MORE_12\">more!</see></tt> instead.</p>"],

 "ACL2____FUNCTIONAL-INSTANTIATION-EXAMPLE":[["Tutorial5-miscellaneous-examples"],"ACL2 Sources","ACL2","<p>The example below demonstrates the use of functional instantiation, \u000A that is, the use of a generic result in proving a result about specific \u000A functions.  In this example we constrain a function to be associative and \u000A commutative, with an identity or ``root,'' on a given domain.  Next, we define \u000A a corresponding function that applies the constrained associative-commutative \u000A function to successive elements of a list.  We then prove that the latter \u000A function gives the same value when we first reverse the elements of the list. \u000A Finally, we use functional instantiation to derive the corresponding result \u000A for the function that multiplies successive elements of a list.</p> \u000A \u000A <p>The details of the proof (such as the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event and \u000A particulars of the lemmas) are not the point here.  Rather, the point is to \u000A demonstrate the interaction of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> and \u000A <v>:functional-instance</v> <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>s.  Of course, if you are \u000A interested in details then you may find it helpful to run these <see topic=\"ACL2____EVENTS\">events</see> \u000A through ACL2.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____CONSTRAINT\">constraint</see> for more about <v>:functional-instance</v> and see \u000A <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> for general information about the use of \u000A previously-proved lemmas.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((ac-fn * *) =&gt; *)\u000A  ((ac-fn-domain *) =&gt; *)\u000A  ((ac-fn-root) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> ac-fn (x y) (<see topic=\"COMMON-LISP_____B2\">+</see> x y)))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> ac-fn-root () 0))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> ac-fn-domain (x) (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-comm\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn x y)\u000A          (ac-fn y x)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-assoc\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn (ac-fn x y) z)\u000A          (ac-fn x (ac-fn y z))))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-id\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (ac-fn-domain x)\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn (ac-fn-root) x)\u000A                   x)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-closed\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (ac-fn-domain (ac-fn x y))\u000A        (ac-fn-domain (ac-fn-root)))))\u000A\u000A;;;;;;;;;;;;;;;;;;;;;;;\u000A; End of encapsulate. ;\u000A;;;;;;;;;;;;;;;;;;;;;;;\u000A\u000A; Define a ``fold'' function that iteratively applies ac-fn over a list.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ac-fn-list (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      (ac-fn-root)\u000A    (ac-fn (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A           (ac-fn-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A; Recognize lists all of whose elements are in the intended domain.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ac-fn-domain-list (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      t\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (ac-fn-domain (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A         (ac-fn-domain-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A; Define a list reverse function.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      nil\u000A    (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A\u000A; The following is needed for proving ac-fn-list-append, which is\u000A; needed for proving ac-fn-list-rev.\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-list-closed\u000A   (ac-fn-domain (ac-fn-list x)))\u000A\u000A; Needed for proving ac-fn-list-rev:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-list-append\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (ac-fn-domain-list x)\u000A                (ac-fn-domain-list y))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn-list (<see topic=\"COMMON-LISP____APPEND\">append</see> x y))\u000A                  (ac-fn (ac-fn-list x)\u000A                         (ac-fn-list y)))))\u000A\u000A; Needed for proving ac-fn-list-rev:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-domain-list-rev\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn-domain-list (rev x))\u000A         (ac-fn-domain-list x)))\u000A\u000A; The following is a good idea because without it, the proof attempt\u000A; for ac-fn-list-rev (see just below) fails with the term (<see topic=\"ACL2____HIDE\">HIDE</see>\u000A; (AC-FN-LIST NIL)).  It is often a good idea to disable\u000A; executable-counterparts of functions that call constrained\u000A; functions.\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart ac-fn-list)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> ac-fn-list-rev\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (ac-fn-domain-list x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (ac-fn-list (rev x))\u000A                  (ac-fn-list x))))\u000A\u000A; Our goal now is to apply functional instantiation to ac-fn-list-rev\u000A; in order to prove the corresponding theorem, times-list-rev, based\u000A; on * instead of ac-fn.\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> times-list (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      1\u000A    (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A       (times-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> acl2-number-listp (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      t\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A         (<see topic=\"ACL2____ACL2-NUMBER-LISTP\">acl2-number-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A; The following relies on the following built-in rules for * (whose\u000A; statements correspond directly to their names): commutativity-of-*,\u000A; associativity-of-*, and unicity-of-1.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> times-list-rev\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____ACL2-NUMBER-LISTP\">acl2-number-listp</see> x)\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (times-list (rev x))\u000A                  (times-list x)))\u000A  :hints ((&quot;Goal&quot;\u000A           :use\u000A           ((:functional-instance\u000A             ac-fn-list-rev\u000A             ;; Instantiate the generic functions:\u000A             (ac-fn *)\u000A             (ac-fn-root (<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> () 1))\u000A             (ac-fn-domain acl2-numberp)\u000A             ;; Instantiate the other relevant functions:\u000A             (ac-fn-list times-list)\u000A             (ac-fn-domain-list acl2-number-listp))))))</code>"],

 "ACL2____DEFLOCK":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel evaluation and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"ACL2____DEFLOCK\">deflock</see> *my-lock*)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFLOCK\">deflock</see> *symbol*)</code> \u000A \u000A <p>where <v>*symbol*</v> is a symbol whose first and last characters are both \u000A the character <v>#\\*</v>.</p> \u000A \u000A <p>A call of this macro generates a definition of another macro, named \u000A <v>with-&lt;modified-lock-symbol&gt;</v>, where <v>&lt;modified-lock-symbol&gt;</v> is the \u000A given symbol with the leading and trailing <v>*</v> characters removed.  This \u000A newly defined macro will guarantee mutually exclusive execution when called in \u000A the body of the raw Lisp definition of a function, as is typically the case \u000A for <see topic=\"ACL2____GUARD\">guard</see>-verified functions, for <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions, \u000A and for calls of macro <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt>.  (See <see topic=\"ACL2____GUARD-EVALUATION-TABLE\">guard-evaluation-table</see> \u000A for details of how raw Lisp code might not be invoked when guard-checking (see \u000A <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>) has value <v>:none</v> or <v>:all</v>.)</p> \u000A \u000A <p>To see how mutual exclusion is guaranteed, consider the raw Lisp code \u000A generated for the macro, <v>with-&lt;modified-lock-symbol&gt;</v>, that is introduced \u000A by a call of <v>deflock</v>.  This code uses a lock (with the given \u000A <v>*symbol*</v> as its name), which guarantees that for any two forms that are \u000A each in the scope of a call of <v>with-&lt;modified-lock-symbol&gt;</v>, the forms do \u000A not execute concurrently.</p> \u000A \u000A <p>Note that a call of <v>deflock</v> expands into the application of <v>progn</v> \u000A to two events, as illustrated below.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____DEFLOCK\">deflock</see> *my-cw-lock*)\u000A (<see topic=\"COMMON-LISP____PROGN\">PROGN</see> (<see topic=\"ACL2____TABLE\">TABLE</see> LOCK-TABLE '*MY-CW-LOCK* T)\u000A        (<see topic=\"COMMON-LISP____DEFMACRO\">DEFMACRO</see> WITH-MY-CW-LOCK (<see topic=\"COMMON-LISP_____62REST\">&amp;REST</see> ARGS)\u000A                  (<see topic=\"COMMON-LISP____LIST_A2\">LIST*</see> 'WITH-LOCK '*MY-CW-LOCK* ARGS)))\u000AACL2 !&gt;</code> \u000A \u000A <p>Thus, <v>deflock</v> forms are legal embedded event forms (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>) for <see topic=\"ACL2____BOOKS\">books</see> as well as <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and \u000A <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> <see topic=\"ACL2____EVENTS\">events</see>.</p> \u000A \u000A <p>The following log shows a lock in action.  Recall that locks work as \u000A expected in <see topic=\"ACL2____GUARD\">guard</see>-verified and <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions; \u000A they do not, however, work in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions that have not \u000A been guard-verified, as illustrated below.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFLOCK\">deflock</see> *my-cw-lock*)\u000A[[.. output omitted ..]]\u000A WITH-MY-CW-LOCK\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (n)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> n) :verify-guards nil))\u000A         (<see topic=\"ACL2____PLET\">plet</see> ((x1 (with-my-cw-lock (<see topic=\"ACL2____CW\">cw</see> &quot;~x0&quot; (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> n))))\u000A                (x2 (with-my-cw-lock (<see topic=\"ACL2____CW\">cw</see> &quot;~x0&quot; (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> n)))))\u000A               (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NULL\">null</see> x1) (<see topic=\"COMMON-LISP____NULL\">null</see> x2))))\u000A[[.. output omitted ..]]\u000A FOO\u000AACL2 !&gt;(foo 20)\u000A(NIL NIL NIL NIL( NIL NIL NIL NIL NIL NILNIL  NIL NILNIL  NIL NILNIL\u000A     NIL NILNIL NIL  NIL NILNIL  NIL NIL\u000ANIL      NILNIL  NIL NILNIL )\u000ANIL NIL NIL NIL NIL NIL NIL NIL)\u000AT\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> foo)\u000A[[.. output omitted ..]]\u000A FOO\u000AACL2 !&gt;(foo 20)\u000A(NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A     NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL)\u000A(NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL\u000A     NIL NIL NIL NIL NIL NIL NIL NIL NIL NIL)\u000AT\u000AACL2 !&gt;</code>"],

 "ACL2____DEFAXIOM":[["Events"],"ACL2 Sources","ACL2","<p>WARNING: We strongly recommend that you not add axioms.  If at all \u000A possible you should use <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> to define \u000A new concepts recursively or use <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> to constrain them \u000A constructively.  If your goal is to defer a proof by using a top-down style, \u000A consider using <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt>; see the discussion on ``Top-Down Proof'' \u000A in Section B.1.2 of ``Computer-Aided Reasoning: An Approach.''  Adding new \u000A axioms frequently renders the logic inconsistent.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> sbar (<see topic=\"COMMON-LISP____EQUAL\">equal</see> t nil)\u000A          :rule-classes nil\u000A          :doc &quot;:Doc-Section ...&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> name term\u000A         :rule-classes rule-classes\u000A         :doc          doc-string)</code> \u000A \u000A <p>where <v>name</v> is a new symbolic name (see <see topic=\"ACL2____NAME\">name</see>), <v>term</v> is a \u000A term intended to be a new axiom, and <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> are as described in the corresponding <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topics \u000A .  The two keyword arguments are optional.  If <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> is \u000A not supplied, the list <v>(:rewrite)</v> is used; if you wish the axiom to \u000A generate no rules, specify <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>nil</v>.</p>"],

 "ACL2____DECLARE-STOBJS":[["Stobj"],"ACL2 Sources","ACL2","<p>When a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> uses one of its formals as a single-threaded \u000A object \u000A (<see topic=\"ACL2____STOBJ\">stobj</see>), the <v>defun</v> <i>must</i> include a declaration that the \u000A formal is to be so used.  An exception is the formal ``<tt><see topic=\"ACL2____STATE\">state</see></tt>,'' which \u000A if not declared as explained below, may still be used provided an appropriate \u000A global ``declaration'' is issued: see <see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see>.</p> \u000A \u000A <p>If the formal in question is <v>counters</v> then an appropriate declaration \u000A is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs counters))</code> \u000A \u000A <p>or, more generally,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs (... counters ...)))</code> \u000A \u000A <p>where all the single-threaded formals are listed.</p> \u000A \u000A <p>For such a declaration to be legal it must be the case that all the names \u000A have previously been defined as single-threaded objects with <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt>.</p> \u000A \u000A <p>When an argument is declared to be single-threaded the guard of the \u000A function is augmented by conjoining to it the condition that the argument \u000A satisfy the recognizer for the single-threaded object.  Furthermore, the \u000A syntactic checks done to enforce the legal use of single-threaded objects are \u000A also sufficient to allow these guard conjuncts to be automatically proved.</p> \u000A \u000A <p>The obvious question arises: Why does ACL2 insist that you declare stobj \u000A names before using them in <v>defun</v>s if you can only declare names that have \u000A already been defined with <v>defstobj</v>?  What would go wrong if a formal were \u000A treated as a single-threaded object if and only if it had already been so \u000A defined?</p> \u000A \u000A <p>Suppose that one user, say Jones, creates a book in which <v>counters</v> is \u000A defined as a single-threaded object.  Suppose another user, Smith, creates a \u000A book in which <v>counters</v> is used as an ordinary formal parameter.  Finally, \u000A suppose a third user, Brown, wishes to use both books.  If Brown includes \u000A Jones' book first and then Smith's, then Smith's function treats <v>counters</v> \u000A as single-threaded.  But if Brown includes Smith's book first, the argument is \u000A treated as ordinary.</p> \u000A \u000A <p>ACL2 insists on the declaration to ensure that the definition is processed \u000A the same way no matter what the context.</p>"],

 "ACL2____ACL2-AS-STANDALONE-PROGRAM":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>ACL2 is intended for interactive use.  It is generally unrealistic \u000A to expect it to prove theorems fully automatically; see <see topic=\"ACL2____THE-METHOD\">the-method</see>, and \u000A see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for a more detailed \u000A tutorial.</p> \u000A \u000A <p>Nevertheless, here we describe an approach for how to call the ACL2 theorem \u000A prover noninteractively.  These steps can of course be modified according to \u000A your needs.  Here, we illustrate how to call ACL2 from another Lisp program \u000A (or an arbitrary program) to attempt to prove an arithmetic theorem.</p> \u000A \u000A <p><b>=== STEP 1: ===</b></p> \u000A \u000A <p>Build a suitable ACL2 image by starting ACL2 and then executing the \u000A following forms.  In particular, these define a macro, <v>try-thm</v>, that \u000A causes ACL2 to exit with with an exit status indicating success or failure of \u000A a proof attempt.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-5/top&quot; :dir :system)\u000A(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> try-thm (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A  `(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A           (<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> 3 (<see topic=\"ACL2____THM\">thm</see> ,@args))\u000A           (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> val))\u000A           (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"COMMON-LISP____IF\">if</see> erp (<see topic=\"ACL2____EXIT\">exit</see> 1) (<see topic=\"ACL2____EXIT\">exit</see> 0)) state))))\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see>) ; optional\u000A:q\u000A(<see topic=\"ACL2____SAVE-EXEC\">save-exec</see> &quot;arith-acl2&quot; &quot;Included arithmetic-4/top&quot;)</code> \u000A \u000A <p>If you prefer, above you can replace 3 by some other number of seconds as a \u000A time limit for the prover.  Also, you can replace</p> \u000A \u000A <code>(<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> 3 (<see topic=\"ACL2____THM\">thm</see> ,@args))</code> \u000A \u000A <p>by</p> \u000A \u000A <code>(<see topic=\"ACL2____WITH-OUTPUT\">with-output</see> :off :all (<see topic=\"ACL2____WITH-PROVER-TIME-LIMIT\">with-prover-time-limit</see> 3 (<see topic=\"ACL2____THM\">thm</see> ,@args)))</code> \u000A \u000A <p>if you want to turn off output.  It may be best to leave the output on, \u000A instead eliminating it in the calling program (see Step 3 below).</p> \u000A \u000A <p><b>=== STEP 2: ===</b></p> \u000A \u000A <p>Try a little test.  In that same directory try this:</p> \u000A \u000A <code>echo '(try-thm (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x))' | ./arith-acl2\u000Aecho $?</code> \u000A \u000A <p>The exit status should be 0, indicating success.  Now try this:</p> \u000A \u000A <code>echo '(try-thm (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x)))' | ./arith-acl2\u000Aecho $?</code> \u000A \u000A <p>The exit status should be 1, indicating failure.</p> \u000A \u000A <p><b>=== STEP 3: ===</b></p> \u000A \u000A <p>Create a shell script that automates Step 2, for example:</p> \u000A \u000A <code>#!/bin/sh\u000A(echo &quot;(try-thm $1)&quot; | ./arith-acl2) &gt;&amp; /dev/null\u000Aexit $?</code> \u000A \u000A <p><b>=== STEP 4: ===</b></p> \u000A \u000A <p>Try your script from a Lisp program, if you like.  Here is how you can do \u000A it in SBCL, for example.  (Different Lisps have different ways to do this, as \u000A summarized in function <v>system-call</v> in ACL2 source file \u000A <v>acl2-init.lisp</v>.)</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> provable? (x)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((status\u000A         (process-exit-code\u000A          (sb-ext:run-program &quot;./try-thm.sh&quot; (<see topic=\"COMMON-LISP____LIST\">list</see> (format nil &quot;~s&quot; x))\u000A                              :output t :search t))))\u000A    (<see topic=\"COMMON-LISP____EQL\">eql</see> status 0)))</code> \u000A \u000A <p>Then here is a log:</p> \u000A \u000A <code>* (provable? '(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))\u000A\u000ANIL\u000A* (provable? '(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x x))\u000A\u000AT\u000A*</code> \u000A \u000A <p>Certainly refinements are possible -- for example the above doesn't \u000A distinguish between unprovable and ill-formed input.  But it's a start.</p> \u000A \u000A "],

 "ACL2-PC____TYPE-ALIST":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t t)     ; display type-alist based on conclusion and governors\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t t t)   ; as above, but also display forward-chaining report\u000Atype-alist           ; same as (<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil t) -- governors only\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil)     ; same as (<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil t) -- governors only\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t)       ; same as (<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t nil) -- conclusion only\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil nil) ; display type-alist without considering\u000A                     ; conclusion or governors\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> &amp;optional concl-flg govs-flg fc-report-flg)</code> \u000A \u000A <p>where if <v>govs-flg</v> is omitted then it defaults to <v>(<see topic=\"COMMON-LISP____NOT\">not</see> concl-flg)</v>, \u000A and <v>concl-flg</v> and <v>fc-report-flg</v> default to <v>nil</v>.</p> \u000A \u000A <p>Display the current assumptions as a <see topic=\"ACL2____TYPE-ALIST\">type-alist</see>.  Note that this \u000A display includes the result of forward chaining.  When <v>fc-report-flg</v> is \u000A supplied a non-<v>nil</v> value, the display also includes a forward-chaining \u000A report; otherwise,the presence or absence of such a report is controlled by \u000A the usual global settings (see <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see>).</p> \u000A \u000A <p>There are two basic reasons contemplated for using this command.</p> \u000A \u000A <p>1. The theorem prover has failed (either outside the proof-checker or using \u000A a proof-checker command such as <v>bash</v> or <v>reduce</v> and you want to debug \u000A by getting an idea of what the prover knows about the context.</p> \u000A \u000A <blockquote> \u000A \u000A <p>a. You really are interested in the context for the current term.  Include \u000A hypotheses and governors (i.e., accounting for tests of surrounding \u000A <v>if</v>-expressions that must be true or false) but not the current conclusion \u000A (which the theorem prover's heuristics would generally ignore for contextual \u000A information).  Command:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil t)</v> ; equivalently, <v>type-alist</v> or <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see>\u000A nil)</v></p> \u000A \u000A <p>b. You are not thinking in particular about the current term; you just want \u000A to get an idea of the context that the prover would build at the top-level, \u000A for forward-chaining.  Incorporate the conclusion but not the governors. \u000A Command:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t nil)</v> ; equivalently, <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> t)</v></p></blockquote> \u000A \u000A <p>2. You intend to use one of the <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see> that does \u000A simplification, such as <v>s</v> or <v>x</v>, and you want to see the context. \u000A Then include the surrounding <v>if</v>-term governors but not the goal's \u000A conclusion.  Command:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see> nil t)</v> ; equivalently, <v>type-alist</v> or <v>(<see topic=\"ACL2____TYPE-ALIST\">type-alist</see>\u000A nil)</v></p> \u000A \u000A <p>See <see topic=\"ACL2____TYPE-SET\">type-set</see> (also see <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>) for information \u000A about ACL2's type system, which can assist in understanding the output of the \u000A <v>type-alist</v> command.</p>"],

 "ACL2____PROOF-TREE-DETAILS":[["Proof-tree"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> for an introduction to proof trees, and for a \u000A list of related topics.  Here we present some details not covered \u000A elsewhere.</p> \u000A \u000A <p>1.  When proof tree display is enabled (because the command <v>:</v><tt><see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see></tt> has not been executed, or has been superseded by a later \u000A <v>:</v><tt><see topic=\"ACL2____START-PROOF-TREE\">start-proof-tree</see></tt> command), then time summaries will include the \u000A time for proof tree display.  This time includes the time spent computing with \u000A proof trees, such as the pruning process described briefly above.  Even when \u000A proof trees are not displayed, such as when their display is turned off in the \u000A middle of a proof, this time will be printed if it is not 0.</p> \u000A \u000A <p>2.  When a goal is given a <v>:bye</v> in a proof (see <see topic=\"ACL2____HINTS\">hints</see>), it is \u000A treated for the purpose of proof tree display just as though it had been \u000A proved.</p> \u000A \u000A <p>3.  Several <see topic=\"ACL2____STATE\">state</see> global variables affect proof tree display.  <v>(<see topic=\"ACL2_____04\">@</see>\u000A proof-tree-indent)</v> is initially the string <v>&quot;| &quot;</v>: it is the string \u000A that is laid down the appropriate number of times to effect indentation. \u000A <v>(<see topic=\"ACL2_____04\">@</see> proof-tree-buffer-width)</v> is initially the value of \u000A <v>(fmt-soft-right-margin state)</v>, and is used to prevent printing of the \u000A annotation ``(<see topic=\"ACL2____FORCE\">force</see>d ...)'' in any greater column than this value. \u000A However, <v>(<see topic=\"ACL2____ASSIGN\">assign</see> proof-tree-buffer-width nil)</v> to avoid any such \u000A suppression.  Finally, <v>(<see topic=\"ACL2_____04\">@</see> checkpoint-processors)</v> is a list of processors \u000A from the constant list <v>*preprocess-clause-ledge*</v>, together with \u000A <v>:induct</v>.  You may remove elements of <v>(<see topic=\"ACL2_____04\">@</see> checkpoint-processors)</v> to \u000A limit which processes are considered checkpoints, but note that this can \u000A affect what is printed by gag-mode (see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>).</p> \u000A \u000A <p>4.  When <v>:</v><tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> is not set to <v>t</v> in a proof, and the \u000A prover then decides to revert to the original goal and prove it by induction, \u000A the proof tree display will reflect this fact as shown here:</p> \u000A \u000A <code>c  0 |  |  Subgoal 2 PUSH (reverting)</code> \u000A \u000A <p>5. The usual <see topic=\"ACL2____FAILURE\">failure</see> message is printed as part of the prooftree \u000A display when a proof has failed.</p>"],

 "ACL2____POST-INDUCTION-KEY-CHECKPOINTS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Each post-induction key checkpoint is a theorem if and only if the \u000A original conjecture was a theorem.  The reason is that each subgoal produced \u000A by induction concludes with the original formula and simplification preserves \u000A equivalence.</p> \u000A \u000A <p>So if you see a post-induction key checkpoint that is not a theorem, stop \u000A looking at the checkpoints!  Your original conjecture is not a theorem!  Fix \u000A it.</p> \u000A \u000A <p>If you're convinced all the post-induction conjectures are theorems, ask \u000A whether each has the hypotheses you'd need to prove it.  If the case analysis \u000A feels inappropriate or induction hypotheses seem to be missing, then ACL2 \u000A might have done the wrong induction.  Find the induction scheme it did by \u000A reading the first induction message printed after the conjecture was \u000A submitted.  If it is wrong, then extend ACL2's induction analysis or tell ACL2 \u000A what induction to do, as explained shortly.</p> \u000A \u000A <p>But before you decide the induction hypothesis is missing, look closely for \u000A contradictions among the hypotheses of the checkpoint formula.  For example, \u000A perhaps one of the hypotheses is <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> e x)</v> and another is <tt>(NOT \u000A (MEMBER e' x'))</tt> where <v>e</v>, <v>x</v>, <v>e'</v>, and <v>x'</v> are possibly \u000A complicated expressions.</p> \u000A \u000A <p>Is it possible that <v>e</v> and <v>e'</v> are equal and <v>x</v> and <v>x'</v> are \u000A equal?  If so, then the two hypotheses are contradictory and the checkpoint \u000A would be proved if you could find rules that would simplify those expressions \u000A to their common forms.  So look for theorems about those subexpressions.</p> \u000A \u000A <p>Or maybe you can get <v>e</v> and <v>e'</v> to reduce to some common <v>d</v> but \u000A but find that <v>x</v> and <v>x'</v> are really different.  Then ask whether</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> d x) (<see topic=\"COMMON-LISP____MEMBER\">member</see> d x'))</code> \u000A \u000A <p>If you could prove that, the key checkpoint would be proved.  Of course, \u000A you may need other hypotheses from the checkpoint to state your theorems.</p> \u000A \u000A <p>If you have been working your way through the tutorial introduction to the \u000A theorem prover, use your browser's <b>Back Button</b> now to <see topic=\"ACL2____INTRODUCTION-TO-KEY-CHECKPOINTS\">introduction-to-key-checkpoints</see>.</p>"],

 "ACL2____NOTE-2-6-SYSTEM":[["Note-2-6"],"ACL2 Sources","ACL2","<p>We modified the tracking of <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> events and the use \u000A of <tt><see topic=\"ACL2____STATE\">state</see></tt> global <v>ld-skip-proofsp</v> in order to avoid some soundness \u000A issues.  For example, <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> events buried in locally-included \u000A books are now tracked.  The ``Essay on Skip-proofs'' in source file \u000A <v>axioms.lisp</v> gives several examples of dicey behavior that is no longer \u000A supported.</p> \u000A \u000A <p>We fixed a problem with some of the makefiles, so that recursive \u000A invocations of `make' now use the version of `make' specified on the command \u000A line.</p> \u000A \u000A <p>Files were fixed to help non-Unix/Linux users with book certification. \u000A Thanks to John Cowles for finding some problems and suggesting fixes to \u000A <v>books/certify-numbers.lisp</v>, <v>books/arithmetic/certify.lsp</v>, and \u000A <v>books/cowles/certify.lsp</v>.  We thank Scott Burson for noticing and fixing \u000A some other such problems.  Moreover, a bdd test was being ignored entirely in \u000A Version 2.5; this problem has been fixed as well.</p> \u000A \u000A <p>A minor change in system function save-acl2-in-allegro will allow this \u000A function to continue to work in Allegro CL versions starting (someday) with \u000A 10.0.  Thanks to Art Flatau for suggesting such a fix.</p> \u000A \u000A <p>The <v>books/case-studies/</v> directory has been removed.  These books are \u000A in support of the first (1998) ACL2 workshop, and are accessible via the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/'>ACL2 home page</a>.  Also, \u000A the <v>books/cli-misc</v> directory has been renamed <v>books/misc</v>, and the \u000A <v>books/nqthm</v> directory has been removed.</p> \u000A \u000A <p>The notion of ACL2 version has been slightly modified to catch unsoundness \u000A due to implementation dependencies.  See <see topic=\"ACL2____VERSION\">version</see>.  Another change to \u000A eliminate such unsoundness is that built-in symbols now have a <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of <v>&quot;COMMON-LISP&quot;</v>; formerly, this string was \u000A <v>&quot;LISP&quot;</v> for ACL2 images built on GCL.  See <see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see>. \u000A At a low level, the (undocumented) constant <v>*main-lisp-package-name*</v> is \u000A now <v>&quot;COMMON-LISP&quot;</v>; before, it was <v>&quot;LISP&quot;</v> for GCL.</p>"],

 "ACL2____NOTE-2-3":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important changes:</p> \u000A \u000A <p>Versions of ACL2 preceding this one contain a subtle soundness bug!  We \u000A found a flaw in our detection of <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.  The bug \u000A allowed some subversive recursions to slip through undetected.</p> \u000A \u000A <p>We believe it would have been difficult to have exploited this flaw \u000A inadvertently.  In particular, the following five conditions are necessary. \u000A <br></br> <br></br> \u000A \u000A (1) Introduce a constrained function, say <v>f</v>, via an <v>encapsulate</v>. \u000A <br></br> <br></br> \u000A \u000A (2) In the same encapsulation, define a clique of mutually recursive \u000A functions.  This clique must be non-<v>local</v> and in <v>:logic</v> mode. \u000A <br></br> <br></br> \u000A \u000A (3) In that mutually recursive clique, use the constrained function \u000A <v>f</v> (perhaps indirectly) so that the termination argument for the clique \u000A depends on properties of the <i>witness</i> for <v>f</v>.  Thus, <v>f</v> or some \u000A other function dependent upon <v>f</v>, must be used in an argument in a \u000A recursive call or in a term governing a recursive call.  Furthermore, the use \u000A of <v>f</v> must be such that the termination proof cannot be done without \u000A exploiting properties of the witness for <v>f</v>.  Other uses of the \u000A constrained functions in the clique are ok.  <br></br> <br></br> \u000A \u000A (4) Fail to include the exploited properties of <v>f</v> among the constraints \u000A of the encapsulation.  <br></br> <br></br> \u000A \u000A (5) Later, outside the encapsulation, explicitly use a functional \u000A instantiation in which <v>f</v> is replaced by a function not enjoying the \u000A crucial properties.  <br></br> <br></br> \u000A \u000A See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see> for details.</p> \u000A \u000A <p>Less important notes:</p> \u000A \u000A <p>We have begun to write some introductory tutorial material for those who \u000A wish to learn to program in ACL2.  Most of this material is HTML-based.  See \u000A the Hyper-Card on the ACL2 home page.</p> \u000A \u000A <p>The documentation of <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> was improved to explain why one \u000A might wish to verify the ``guards'' of a <v>defthm</v> event.  The missing \u000A documentation was noticed by John Cowles.</p> \u000A \u000A <p>A bug was fixed in cross fertilization.  The bug caused the system to \u000A report that it had substituted one term for another when in fact no \u000A substitution occurred.  The bug was noticed by Bill McCune.</p>"],

 "ACL2____NON-EXEC":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Non-exec</v> is a macro such that logically, <v>(<see topic=\"ACL2____NON-EXEC\">non-exec</see> x)</v> is \u000A equal to <v>x</v>.  However, the argument to a call of <v>non-exec</v> need not \u000A obey the usual syntactic restrictions for executable code, and indeed, \u000A evaluation of a call of <v>non-exec</v> will result in an error.  Moreover, for \u000A any form occurring in the body of a function (see <see topic=\"COMMON-LISP____DEFUN\">defun</see>) that is a call \u000A of <v>non-exec</v>, no guard proof obligations are generated for that form.</p> \u000A \u000A <p>The following example, although rather contrived, illustrates the use of \u000A <v>non-exec</v>.  One can imagine a less contrived example that efficiently \u000A computes return values for a small number of fixed inputs and, for other \u000A inputs, returns something logically ``consistent'' with those return \u000A values.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> double (x)\u000A  (<see topic=\"COMMON-LISP____CASE\">case</see> x\u000A    (1 2)\u000A    (2 4)\u000A    (3 6)\u000A    (otherwise (<see topic=\"ACL2____NON-EXEC\">non-exec</see> (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)))))</code> \u000A \u000A <p>We can prove that <v>double</v> is compliant with Common Lisp (see <see topic=\"ACL2____GUARD\">guard</see>) and that it always computes <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> double)\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (double x) (<see topic=\"COMMON-LISP_____A2\">*</see> 2 x)))</code> \u000A \u000A <p>We can evaluate double on the specified arguments.  But a call of \u000A <v>non-exec</v> results in an error message that reports the form that was \u000A supplied to <v>non-exec</v>.</p> \u000A \u000A <code>ACL2 !&gt;(double 3)\u000A6\u000AACL2 !&gt;(double 10)\u000A\u000AACL2 Error in TOP-LEVEL:  ACL2 has been instructed to cause an error\u000Abecause of an attempt to evaluate the following form (see :DOC non-\u000Aexec):\u000A\u000A  (<see topic=\"COMMON-LISP_____A2\">*</see> 2 X).\u000A\u000ATo debug see :DOC print-gv, see :DOC trace, and see :DOC wet.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>During proofs, the error is silent; it is ``caught'' by the proof mechanism \u000A and generally results in the introduction of a call of <tt><see topic=\"ACL2____HIDE\">hide</see></tt> during a \u000A proof.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFUN-NX\">defun-nx</see> for a utility that makes every call of a function \u000A non-executable, rather than a specified form.  The following examples contrast \u000A <v>non-exec</v> with <tt><see topic=\"ACL2____DEFUN-NX\">defun-nx</see></tt>, in particular illustratating the role of \u000A <tt><see topic=\"ACL2____NON-EXEC\">non-exec</see></tt> in avoiding guard proof obligations.</p> \u000A \u000A <code>; Guard verification fails:\u000A(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> f1 (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000A; Guard verification succeeds after changing the guard above:\u000A(<see topic=\"ACL2____DEFUN-NX\">defun-nx</see> f1 (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A  (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A\u000A; Guard verification succeeds:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"ACL2____NON-EXEC\">non-exec</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A\u000A; Evaluating (g1) prints &quot;Hello&quot; before signaling an error.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g1 ()\u000A  (f1 (<see topic=\"ACL2____CW\">cw</see> &quot;Hello&quot;)))\u000A\u000A; Evaluating (g2) does not print before signaling an error.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g2 ()\u000A  (<see topic=\"ACL2____NON-EXEC\">non-exec</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Hello&quot;)))\u000A\u000A; Evaluating (h1) gives a guard violation for taking reciprocal of 0.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h1 ()\u000A  (f1 (<see topic=\"COMMON-LISP_____F2\">/</see> 1 0)))\u000A\u000A; Evaluating (h2) does not take a reciprocal, hence there is no guard\u000A; violation for that; we just get the error expected from using non-exec.\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> h2 ()\u000A  (<see topic=\"ACL2____NON-EXEC\">non-exec</see> (<see topic=\"COMMON-LISP_____F2\">/</see> 0)))</code>"],

 "ACL2____MV-NTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____MV-NTH\">Mv-nth</see> n l)</v> is the <v>n</v>th element of <v>l</v>, zero-based. \u000A If <v>n</v> is greater than or equal to the length of <v>l</v>, then <v>mv-nth</v> \u000A returns <v>nil</v>.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MV-NTH\">Mv-nth</see> n l)</v> has a <see topic=\"ACL2____GUARD\">guard</see> that <v>n</v> is a non-negative \u000A integer.</p> \u000A \u000A <p><v>Mv-nth</v> is equivalent to the Common Lisp function <tt><see topic=\"COMMON-LISP____NTH\">nth</see></tt> (although \u000A without the guard condition that the list is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>), but is \u000A used by ACL2 to access the nth value returned by a multiply valued expression. \u000A For example, the following are logically equivalent:</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        (<see topic=\"ACL2____READ-OBJECT\">read-object</see> ch state)\u000A        (value (<see topic=\"COMMON-LISP____LIST\">list</see> erp val)))</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((erp (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 0 (<see topic=\"ACL2____READ-OBJECT\">read-object</see> ch state)))\u000A      (val (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 1 (<see topic=\"ACL2____READ-OBJECT\">read-object</see> ch state)))\u000A      (<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 2 (<see topic=\"ACL2____READ-OBJECT\">read-object</see> ch state))))\u000A  (value (<see topic=\"COMMON-LISP____LIST\">list</see> erp val)))</code> \u000A \u000A <p>To see the ACL2 definition of <v>mv-nth</v>, see <see topic=\"ACL2____PF\">pf</see>.</p> \u000A \u000A <p>If <v>EXPR</v> is an expression that is multiply valued, then the form \u000A <v>(<see topic=\"ACL2____MV-NTH\">mv-nth</see> n EXPR)</v> is illegal both in definitions and in forms submitted \u000A directly to the ACL2 loop.  Indeed, <v>EXPR</v> cannot be passed as an argument \u000A to any function (<v>mv-nth</v> or otherwise) in such an evaluation context.  The \u000A reason is that ACL2 code compiled for execution does not actually create a \u000A list for multiple value return; for example, the <v>read-object</v> call above \u000A logically returns a list of length 3, but when evaluated, it instead stores \u000A its three returned values without constructing a list.  In such cases you can \u000A use <v>mv-nth</v> to access the corresponding list by using <v>mv-list</v>, \u000A writing <v>(<see topic=\"ACL2____MV-NTH\">mv-nth</see> n (<see topic=\"ACL2____MV-LIST\">mv-list</see> k EXPR))</v> for suitable <v>k</v>, where \u000A <v>mv-list</v> converts a multiple value result into the corresponding list; see \u000A <see topic=\"ACL2____MV-LIST\">mv-list</see>.</p>"],

 "ACL2____LD-PROMPT":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-prompt</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____LD-PROMPT\">ld-prompt</see> state)</v> and the updater is <v>(set-ld-prompt val\u000A state)</v>.  <v>Ld-prompt</v> must be either <v>nil</v>, <v>t</v>, or a function \u000A symbol that, when given an open output character channel and <see topic=\"ACL2____STATE\">state</see>, \u000A prints the desired <see topic=\"ACL2____PROMPT\">prompt</see> to the channel and returns two values: the \u000A number of <see topic=\"ACL2____CHARACTERS\">characters</see> printed and the <see topic=\"ACL2____STATE\">state</see>.  The initial value \u000A of <v>ld-prompt</v> is <v>t</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-prompt</v> is one of them.  <v>Ld-prompt</v> determines whether <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A prints a <see topic=\"ACL2____PROMPT\">prompt</see> before reading the next form from <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>. \u000A If <v>ld-prompt</v> is <v>nil</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> prints no <see topic=\"ACL2____PROMPT\">prompt</see>.  If \u000A <v>ld-prompt</v> is <v>t</v>, the default <see topic=\"ACL2____PROMPT\">prompt</see> printer is used, which \u000A displays information that includes the current package, default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>, <see topic=\"ACL2____GUARD\">guard</see> checking status (on or off), and <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt>; see <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see>.</p> \u000A \u000A <p>If <v>ld-prompt</v> is neither <v>nil</v> nor <v>t</v>, then it should be a \u000A function name, <v>fn</v>, such that <v>(fn channel state)</v> will print the \u000A desired <see topic=\"ACL2____PROMPT\">prompt</see> to <v>channel</v> in <tt><see topic=\"ACL2____STATE\">state</see></tt> and return <v>(<see topic=\"ACL2____MV\">mv</see> col\u000A state)</v>, where <v>col</v> is the number of <see topic=\"ACL2____CHARACTERS\">characters</see> output (on the \u000A last line output).  You may define your own <see topic=\"ACL2____PROMPT\">prompt</see> printing \u000A function.</p> \u000A \u000A <p>If you supply an inappropriate <see topic=\"ACL2____PROMPT\">prompt</see> function, i.e., one that \u000A causes an error or does not return the correct number and type of results, the \u000A following odd <see topic=\"ACL2____PROMPT\">prompt</see> will be printed instead:</p> \u000A \u000A <code>Bad Prompt\u000ASee :DOC ld-prompt&gt;</code> \u000A \u000A <p>which will lead you to this message.  You should either call <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A appropriately next time or assign an appropriate value to \u000A <v>ld-prompt</v>.</p>"],

 "ACL2____LAST-PROVER-STEPS":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>For discussions of prover step limits, See <see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see> and see <see topic=\"ACL2____WITH-PROVER-STEP-LIMIT\">with-prover-step-limit</see>.  The value of \u000A the form <v>(<see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see> state)</v> indicates the number of prover steps \u000A taken, in the sense described below, for the most recent context in which an \u000A event summary would normally be printed.  Note that the value of \u000A <v>(<see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see> state)</v> is updated for all <see topic=\"ACL2____EVENTS\">events</see>, and for all \u000A other forms such as calls of <tt><see topic=\"ACL2____THM\">thm</see></tt> or <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, that would \u000A print a summary &mdash; regardless of whether or not such output is inhibited \u000A (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> and see <see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see>).  In particular, the value is updated (typically \u000A to <v>nil</v>) for <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, even when no summary is printed; \u000A for example, the value is updated to <v>nil</v> for <v>table</v> events such as \u000A <v>(</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt><v>)</v>, <v>(</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt><v>)</v>, and even calls of \u000A <tt><see topic=\"ACL2____SET-PROVER-STEP-LIMIT\">set-prover-step-limit</see></tt>.</p> \u000A \u000A <p>The value of <v>(<see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see> state)</v> is determined as follows, based \u000A on the most recent summary context (as described above):</p> \u000A \u000A <blockquote> \u000A \u000A <p><v>nil</v>, if no prover steps were taken; else,</p> \u000A \u000A <p>the (positive) number of steps taken, if the number of steps did not exceed \u000A the starting limit; else,</p> \u000A \u000A <p>the negative of the starting limit.</p></blockquote> \u000A \u000A <p>We conclude with a remark for advanced users who wish to invoke \u000A <v>last-prover-steps</v> in the development of utilities that track prover \u000A steps.  Suppose that you want to write a utility that takes some action based \u000A on the number of prover steps performed by the first <v>defun</v> event that is \u000A generated, among others, for example the number of prover steps taken to admit \u000A <v>f1</v> in the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 ...)\u000A       (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 ...))</code> \u000A \u000A <p>A solution is to record the steps taken by the first <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> before \u000A executing subsequent events, as follows (see <see topic=\"ACL2____MAKE-EVENT\">make-event</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f1 ...)\u000A       (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'my-step-count\u000A                                         (<see topic=\"ACL2____LAST-PROVER-STEPS\">last-prover-steps</see> state)\u000A                                         state)\u000A                           (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> nil))))\u000A       (<see topic=\"COMMON-LISP____DEFUN\">defun</see> f2 ...))</code>"],

 "ACL2____HIDDEN-DEATH-PACKAGE":[["Defpkg"],"ACL2 Sources","ACL2","<p>This documentation topic explains a little bit about certain errors \u000A users may see when attempting to evaluate a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event.  In brief, \u000A if you see an error that refers you to this topic, you are probably trying to \u000A admit a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event, and you should change the name of the package to \u000A be introduced by that event.</p> \u000A \u000A <p>Recall that <v>defpkg</v> events introduce axioms, for example as \u000A follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;PKG0&quot; '(a b))\u000A\u000ASummary\u000AForm:  ( DEFPKG &quot;PKG0&quot; ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A &quot;PKG0&quot;\u000AACL2 !&gt;:pr! &quot;PKG0&quot;\u000A\u000ARune:       (:REWRITE PKG0-PACKAGE)\u000AStatus:     Enabled\u000ALhs:        (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">SYMBOL-PACKAGE-NAME</see> (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">INTERN-IN-PACKAGE-OF-SYMBOL</see> X Y))\u000ARhs:        &quot;PKG0&quot;\u000AHyps:       (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____STRINGP\">STRINGP</see> X)\u000A                 (<see topic=\"COMMON-LISP____NOT\">NOT</see> (MEMBER-SYMBOL-NAME X '(A B)))\u000A                 (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> Y)\u000A                 (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">SYMBOL-PACKAGE-NAME</see> Y) &quot;PKG0&quot;))\u000AEquiv:      EQUAL\u000ABackchain-limit-lst:    NIL\u000ASubclass:   BACKCHAIN\u000ALoop-stopper: NIL\u000AACL2 !&gt;</code> \u000A \u000A <p>Now, a <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event may be executed underneath an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form that is marked <tt><see topic=\"ACL2____LOCAL\">local</see></tt>.  In \u000A that case, traces of the added axiom will disappear after the surrounding \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> form is admitted.  This can cause \u000A inconsistencies.  (You can take our word for it, or you can look at the \u000A example shown in the ``Essay on Hidden Packages'' in source file \u000A <v>axioms.lisp</v>.)</p> \u000A \u000A <p>In order to prevent unsoundness, then, ACL2 maintains the following \u000A invariant.  Let us say that a <v>defpkg</v> event is ``hidden'' if it is in \u000A support of the current logical <see topic=\"ACL2____WORLD\">world</see> but is not present in that world \u000A as an event, because it is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> as indicated above.  We maintain the \u000A invariant that all <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, even if ``hidden'', are \u000A tracked under-the-hood in the current logical <see topic=\"ACL2____WORLD\">world</see>.  Sometimes this \u000A property causes <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> events to be written to the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> \u000A of a book's <see topic=\"ACL2____CERTIFICATE\">certificate</see> (see <see topic=\"ACL2____BOOKS\">books</see>).  At any rate, if you then \u000A try to define the package in a manner inconsistent with the earlier such \u000A definition, that is, with a different imports list, you will see an error \u000A because of the above-mentioned tracking.</p> \u000A \u000A <p>(By the way, this topic's name comes from Holly Bell, who heard \"hidden \u000A death package\" instead of \"hidden defpkg\".  The description seemed to fit. \u000A Thanks, Holly!)</p>"],

 "ACL2____EVISC-TABLE":[["Events"],"ACL2 Sources","ACL2","<p>The <v>evisc-table</v> is an ACL2 table (see <see topic=\"ACL2____TABLE\">table</see>) whose \u000A purpose is to modify the print representations of specified non-<v>nil</v> \u000A objects.  When a key (some object) is associated with a string value, then \u000A that string is printed instead of that key (as an abbreviation).  For example, \u000A the following log shows how to abbreviate the key <v>(a b c)</v> with the token \u000A <v>&lt;my-abc-list&gt;</v>.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TABLE\">table</see> evisc-table '(a b c) &quot;&lt;my-abc-list&gt;&quot;)\u000A\u000ASummary\u000AForm:  ( TABLE EVISC-TABLE ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A EVISC-TABLE\u000AACL2 !&gt;'(a b c)\u000A&lt;my-abc-list&gt;\u000AACL2 !&gt;'(4 5 a b c)\u000A(4 5 . &lt;my-abc-list&gt;)\u000AACL2 !&gt;</code> \u000A \u000A <p>Every value in this <see topic=\"ACL2____TABLE\">table</see> must be either a string or <v>nil</v>, where \u000A <v>nil</v> eliminates any association of the key with an abbreviation. \u000A Continuing with the log above:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TABLE\">table</see> evisc-table '(a b c) nil)\u000A\u000ASummary\u000AForm:  ( TABLE EVISC-TABLE ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A EVISC-TABLE\u000AACL2 !&gt;'(a b c)\u000A(A B C)\u000AACL2 !&gt;'(4 5 a b c)\u000A(4 5 A B C)\u000AACL2 !&gt;</code> \u000A \u000A <p>It can be particularly helpful to use this table to abbreviate a constant \u000A introduced by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> by prefixing the constant name with \u000A <v>&quot;#,&quot;</v>, as we now describe.  Consider first the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *abc* '(1 2 3 4 5 6 7 8))\u000A(<see topic=\"ACL2____TABLE\">table</see> evisc-table *abc*\u000A  (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string &quot;#,&quot; (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> '*abc*)))</code> \u000A \u000A <p>Then the constant <v>*abc*</v> is printed as follows &mdash; very helpful if \u000A its associated structure is significantly larger than the 8-element list of \u000A numbers shown above!</p> \u000A \u000A <code>ACL2 !&gt;*abc*\u000A#,*ABC*\u000AACL2 !&gt;</code> \u000A \u000A <p>What's more, the ACL2 reader will replace <v>#,*C*</v>, where <v>*C*</v> is \u000A defined by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, by its value, regardless of <v>evisc-table</v>; see \u000A <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>.  Continuing with the example above, we have:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> #,*ABC*))\u000A(2 3 4 5 6 7 8)\u000AACL2 !&gt;</code> \u000A \u000A <p>Of course, more care needs to be taken if packages are involved (see <see topic=\"ACL2____DEFPKG\">defpkg</see>), as we now illustrate.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;FOO&quot; nil)\u000A\u000ASummary\u000AForm:  ( DEFPKG &quot;FOO&quot; ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A &quot;FOO&quot;\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> foo::*a* '(1 2 3))\u000A\u000ASummary\u000AForm:  ( DEFCONST FOO::*A* ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO::*A*\u000AACL2 !&gt;(<see topic=\"ACL2____TABLE\">table</see> evisc-table foo::*a* &quot;#,foo::*a*&quot;)\u000A\u000ASummary\u000AForm:  ( TABLE EVISC-TABLE ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A EVISC-TABLE\u000AACL2 !&gt;foo::*a*\u000A#,foo::*a*\u000AACL2 !&gt;'#,foo::*a*\u000A#,foo::*a*\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CDR\">cdr</see> '#,foo::*a*)\u000A(2 3)\u000AACL2 !&gt;</code> \u000A \u000A <p>We conclude by an example showing some extra care that may be important to \u000A consider taking.  We start with:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> |*BaR*| '(3 4))</code> \u000A \u000A <p>Then the following works just fine; but try it without the extra code for \u000A the <v>may-need-slashes</v> case and you'll see that the sharp-dot printing is \u000A missing.  First:</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> evisc-table\u000A       |*BaR*|\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> '|*BaR*|)))\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (may-need-slashes x)\u000A             (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string &quot;#.|&quot; x &quot;|&quot;)\u000A           (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string &quot;#.&quot; x))))</code> \u000A \u000A <p>Then:</p> \u000A \u000A <code>ACL2 !&gt;|*BaR*|\u000A#,|*BaR*|\u000AACL2 !&gt;</code>"],

 "ACL2____WALKABOUT":[["Debugging"],"ACL2 Sources","ACL2","<p>By typing <v>(<see topic=\"ACL2____WALKABOUT\">walkabout</see> x state)</v> for an ACL2 term <v>x</v> whose \u000A value is a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> tree, you can walk around that tree.  For example, \u000A ACL2 developers often use this utility to explore the ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>When you issue a <v>walkabout</v> command, a summary of commands will be \u000A printed before you enter an interactive loop.</p> \u000A \u000A <code>Commands:\u000A0, 1, 2, ..., nx, bk, pp, (pp n), (pp lev len), =, (<see topic=\"COMMON-LISP_____D3\">=</see> symb), and q.</code> \u000A \u000A <p>In the interactive <v>walkabout</v> loop, a positive integer n takes you to \u000A the nth position, while 0 takes you up a level.  The commands <v>nx</v> and \u000A <v>bk</v> take you to the next and previous position, respectively, at the same \u000A level.  The command <v>pp</v> prints the current object in full, while <v>(pp\u000A level length)</v> hides sub-objects below the indicated level and past the \u000A indicated length, if non-<v>nil</v>; see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>.  The command <v>(pp\u000A n)</v> abbreviates <v>(pp n n)</v>, so in particular <v>(pp nil)</v> is equivalent \u000A to <v>pp</v>.</p> \u000A \u000A <p>Note that the commands above work in any package: <v>nx</v>, <v>bk</v>, \u000A <v>pp</v>, <v>=</v>, and <v>q</v> are converted to the <v>&quot;ACL2&quot;</v> package if the \u000A current package is not <v>&quot;ACL2&quot;</v>.</p> \u000A \u000A <p>The following example illustrates the commands described above.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WALKABOUT\">walkabout</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> '(a (b1 b2 b3)) '(c d e f)) state)\u000A\u000ACommands:\u000A0, 1, 2, ..., nx, bk, pp, (pp n), (pp lev len), =, (<see topic=\"COMMON-LISP_____D3\">=</see> symb), and q.\u000A\u000A(A (B1 B2 B3) C ...)\u000A:2\u000A(B1 B2 B3)\u000A:3\u000AB3\u000A:0\u000A(B1 B2 B3)\u000A:nx\u000AC\u000A:nx\u000AD\u000A:0\u000A(A (B1 B2 B3) C ...)\u000A:pp\u000A(A (B1 B2 B3) C D E F)\u000A:(pp 4)\u000A(A (B1 B2 B3) C D ...)\u000A:(pp 1 4)\u000A(A # C D ...)\u000A:(pp nil 2)\u000A(A (B1 B2 ...) ...)\u000A:q\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally we describe the commands <v>q</v>, <v>=</v>, and <v>(<see topic=\"COMMON-LISP_____D3\">=</see> symb)</v>, where \u000A <v>symb</v> is a symbol.  The command <v>q</v> simply causes an exit from the \u000A <v>walkabout</v> loop.  The command <v>=</v> also exits, but causes the current \u000A object to be printed in full.  The command <v>(<see topic=\"COMMON-LISP_____D3\">=</see> symb)</v> saves an association \u000A of <v>symb</v> with the current object, which can be retrieved outside the \u000A <v>walkabout</v> loop using the macro <v>walkabout=</v>, as illustrated below.</p> \u000A \u000A <code>:2\u000A(B1 B2 B3)\u000A:(<see topic=\"COMMON-LISP_____D3\">=</see> my-list)\u000A(walkabout= MY-LIST) is\u000A(B1 B2 B3)\u000A:q\u000AACL2 !&gt;(walkabout= MY-LIST)\u000A(B1 B2 B3)\u000AACL2 !&gt;</code> \u000A \u000A <p>Finally, we remark that for trees that are not true-lists, <v>walkabout</v> \u000A treats the dot as an object that can be ``walked about''.  The following \u000A example illustrates this point.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WALKABOUT\">walkabout</see> '(c d e . f) state)\u000A\u000ACommands:\u000A0, 1, 2, ..., nx, bk, pp, (pp n), (pp lev len), =, (<see topic=\"COMMON-LISP_____D3\">=</see> symb), and q.\u000A\u000A(C D E . F)\u000A:3\u000AE\u000A:nx\u000A.\u000A:nx\u000AF\u000A:0\u000A(C D E . F)\u000A:4\u000A.\u000A:0\u000A(C D E . F)\u000A:5\u000AF\u000A:</code>"],

 "ACL2____TAU-INTERVALP":[["Tau-system"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> x)</code> \u000A \u000A <p>An interval is a structure of the form: <v>(</v><i>dom</i> \u000A <v>(</v><i>lo-rel</i> <v>.</v>  <i>lo</i><v>)</v> <v>.</v>  <v>(</v><i>hi-rel</i> <v>.</v> \u000A <i>hi</i><v>))</v>.  Every tau contains an interval used to represent the domain \u000A and the upper and lower bounds of the objects recognized by the tau.</p> \u000A \u000A <p>Restrictions on the components of an interval are as follows.  For an \u000A interpretation of the meaning of the components, see <tt><see topic=\"ACL2____IN-TAU-INTERVALP\">in-tau-intervalp</see></tt> \u000A or <tt><see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see></tt>.</p> \u000A \u000A <p><i>dom</i> (``domain'') -- must be one of four symbols: <v>INTEGERP</v>, \u000A <v>RATIONALP</v>, <v>ACL2-NUMBERP</v>, or <v>NIL</v>.</p> \u000A \u000A <p>The two ``relations,'' <i>lo-rel</i> and <i>hi-rel</i>, must be \u000A Booleans.</p> \u000A \u000A <p><i>Lo</i> and <i>hi</i> must be either <v>nil</v> or explicit rational \u000A numbers.  <i>Lo</i> must be no greater than <i>hi</i> (where <v>nil</v>s \u000A represent negative or positive infinity for <i>lo</i> and <i>hi</i> \u000A respectively.</p> \u000A \u000A <p>Finally, if the <i>dom</i> is <v>INTEGERP</v>, then both relations must \u000A <v>nil</v> and <i>lo</i> and <i>hi</i> must be integers when they are \u000A non-<v>nil</v>.</p> \u000A \u000A <p>Recall that <tt><see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see></tt> constructs intervals.  The intervals \u000A it constructs are well-formed only if the arguments to <v>make-tau-interval</v> \u000A satisfy the rules above; <v>make-tau-interval</v> does not coerce or adjust its \u000A arguments in any way.  Thus, it can be (mis-)used to create non-intervals. \u000A Here are examples of <v>tau-intervalp</v> using <v>make-tau-interval</v>.</p> \u000A \u000A <code>; integers: 0 &lt;= x &lt;= 10:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10))      = t\u000A\u000A; integers: 0 &lt;= x (i.e., the natural numbers):\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil nil))     = t\u000A\u000A; violations of domain rules:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP t 0 t 10))          = nil\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'INTEGERP nil 0 nil 10/11))   = nil\u000A\u000A; violation of rule that bounds must be rational if non-nil:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'ACL2-NUMBERP t 0 t #c(3 5))) = nil\u000A\u000A; violation of rule that lo &lt;= hi:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'ACL2-NUMBERP t 0 t -10))     = nil\u000A\u000A; rationals: 0 &lt; x &lt;= 22/7:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'RATIONALP t 0 nil 22/7))     = t\u000A\u000A; numbers: -10 &lt; x &lt; 10:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> 'ACL2-NUMBERP t -10 t 10))    = t\u000A\u000A; any: -10 &lt; x &lt; 10:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> nil t -10 t 10))              = t\u000A\u000A: any:\u000A(<see topic=\"ACL2____TAU-INTERVALP\">tau-intervalp</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> nil nil nil nil nil))         = t</code> \u000A \u000A <p>Note that the second-to-last interval, with domain <v>nil</v> contains all \u000A non-numbers as well as numbers strictly between -10 and 10.  The reason is \u000A that the interval contains <v>0</v> and all non-numbers are coerced to <v>0</v> by \u000A the inequality functions.</p> \u000A \u000A <p>Note that the last interval contains all ACL2 objects.  It is called the \u000A ``universal interval.''</p>"],

 "ACL2____STOBJ-EXAMPLE-2":[["Stobj"],"ACL2 Sources","ACL2","<p>The following event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> ms\u000A  (pcn  :type integer                  :initially 0)\u000A  (mem  :type (array integer (100000)) :initially -1)\u000A  (code :type t                        :initially nil))</code> \u000A \u000A <p>introduces a single-threaded object named <v>ms</v> (which stands for \u000A ``machine state'').  The object has three fields, a <v>pcn</v> or program \u000A counter, a <v>mem</v> or memory, and a <v>code</v> field.</p> \u000A \u000A <p>The <v>mem</v> field is occupied by an object initially of type <v>(array\u000A integer (100000))</v>.  Logically speaking, this is a list of length \u000A <v>100000</v>, each element of which is an integer.  But in the underlying \u000A implementation of the <v>ms</v> object, this field is occupied by a raw Lisp \u000A array, initially of size 100000.</p> \u000A \u000A <p>You might expect the above <v>defstobj</v> to define the accessor function \u000A <v>mem</v> and the updater <v>update-mem</v>.  <i>That does not happen!</i>.</p> \u000A \u000A <p>The above event defines the accessor function <v>memi</v> and the updater \u000A <v>update-memi</v>.  These functions do not access/update the <v>mem</v> field of \u000A the <v>ms</v> object; they access/update the individual elements of the array in \u000A that field.</p> \u000A \u000A <p>In particular, the logical definitions of the two functions are:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> memi (i ms)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (msp ms)\u000A                       (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                       (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (mem-length ms)))))\u000A  (<see topic=\"COMMON-LISP____NTH\">nth</see> i (<see topic=\"COMMON-LISP____NTH\">nth</see> 1 ms)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-memi (i v ms)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard\u000A                  (<see topic=\"COMMON-LISP____AND\">and</see> (msp ms)\u000A                       (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 i)\u000A                       (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (mem-length ms))\u000A                       (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> v))))\u000A  (update-nth-array 1 i v ms))</code> \u000A \u000A <p>For example, to access the 511th (0-based) memory location of the current \u000A <v>ms</v> you could evaluate:</p> \u000A \u000A <code>ACL2 !&gt;(memi 511 ms)\u000A-1</code> \u000A \u000A <p>The answer is <v>-1</v> initially, because that is the above-specified \u000A initial value of the elements of the <v>mem</v> array.</p> \u000A \u000A <p>To set that element you could do</p> \u000A \u000A <code>ACL2 !&gt;(update-memi 511 777 ms)\u000A&lt;ms&gt;\u000AACL2 !&gt;(memi 511 ms)\u000A777</code> \u000A \u000A <p>The raw Lisp implementing these two functions is shown below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> memi (i ms)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (<see topic=\"COMMON-LISP____AND\">and</see> fixnum (integer 0 *)) i))\u000A  (<see topic=\"COMMON-LISP____THE\">the</see> integer\u000A       (aref (<see topic=\"COMMON-LISP____THE\">the</see> (simple-array integer (<see topic=\"COMMON-LISP_____A2\">*</see>))\u000A                  (svref ms 1))\u000A             (<see topic=\"COMMON-LISP____THE\">the</see> (<see topic=\"COMMON-LISP____AND\">and</see> fixnum (integer 0 *)) i))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-memi (i v ms)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (<see topic=\"COMMON-LISP____AND\">and</see> fixnum (integer 0 *)) i)\u000A           (<see topic=\"COMMON-LISP____TYPE\">type</see> integer v))\u000A  (<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A   (setf (aref (<see topic=\"COMMON-LISP____THE\">the</see> (simple-array integer (<see topic=\"COMMON-LISP_____A2\">*</see>))\u000A                    (svref ms 1))\u000A               (<see topic=\"COMMON-LISP____THE\">the</see> (<see topic=\"COMMON-LISP____AND\">and</see> fixnum (integer 0 *)) i))\u000A         (<see topic=\"COMMON-LISP____THE\">the</see> integer v))\u000A   ms))</code> \u000A \u000A <p>If you want to see the raw Lisp supporting a <v>defstobj</v>, execute the \u000A <v>defstobj</v> and then evaluate the ACL2 form <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> 4 (global-val\u000A 'cltl-command (w state)))</v>.</p> \u000A \u000A <p>To continue the stobj tour, see <see topic=\"ACL2____STOBJ-EXAMPLE-3\">stobj-example-3</see>.</p>"],

 "ACL2____STOBJ-EXAMPLE-1-IMPLEMENTATION":[["Stobj"],"ACL2 Sources","ACL2","<p>the event</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> counters\u000A  (NodeCnt     :type integer :initially 0)\u000A  (TipCnt      :type integer :initially 0)\u000A  (IntTipsSeen :type t       :initially nil))</code> \u000A \u000A <p>discussed in <see topic=\"ACL2____STOBJ-EXAMPLE-1\">stobj-example-1</see>, creates a Common Lisp object to \u000A represent the current value of <v>counters</v>.  That object is created by \u000A evaluating either of the following ``raw'' (non-ACL2) Common Lisp forms:</p> \u000A \u000A <code>(create-counters)\u000A\u000A(vector (make-array 1 :element-type 'integer\u000A                      :initial-element '0)\u000A        (make-array 1 :element-type 'integer\u000A                      :initial-element '0)\u000A        'nil)</code> \u000A \u000A <p>and the value is stored in the Common Lisp global variable named \u000A <v>*the-live-counters*</v>.</p> \u000A \u000A <p>Thus, the <v>counters</v> object is an array of length three.  The first two \u000A elements are arrays of size 1 and are used to hold the <v>NodeCnt</v> and \u000A <v>TipCnt</v> fields.  The third element is the <v>IntTipsSeen</v> field.  The \u000A first two fields are represented by arrays so that we can implement the \u000A <v>integer</v> type specification efficiently.  Generally, integers are \u000A ``boxed'' in some Common Lisp implementations, for example, GCL.  Creating a \u000A new integer requires creating a new box to put it in.  But in some lisps, \u000A including GCL, the integers inside arrays of integers are not boxed.</p> \u000A \u000A <p>The function <v>NodeCnt</v> is defined in raw Lisp as:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> NodeCnt (counters)\u000A  (<see topic=\"COMMON-LISP____THE\">the</see> integer\u000A       (aref (<see topic=\"COMMON-LISP____THE\">the</see> (simple-array integer (1))\u000A                  (svref counters 0))\u000A             0)))</code> \u000A \u000A <p>Observe that the form <v>(svref counters 0)</v> is evaluated to get an array \u000A of size 1, which is followed by a call of <v>aref</v> to access the 0th element \u000A of that array.</p> \u000A \u000A <p>The function <v>update-NodeCnt</v> is defined in raw Lisp as:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-NodeCnt (v counters)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer v))\u000A  (<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A   (setf (aref (<see topic=\"COMMON-LISP____THE\">the</see> (simple-array integer (1))\u000A                    (svref counters 0))\u000A               0)\u000A         (<see topic=\"COMMON-LISP____THE\">the</see> integer v))\u000A   counters))</code> \u000A \u000A <p>Note that when this function is called, it does not create a new vector of \u000A length three, but ``smashes'' the existing one.</p> \u000A \u000A <p>One way to see all the raw Lisp functions defined by a given <v>defstobj</v> \u000A is to evaluate the <v>defstobj</v> event and then evaluate, in the ACL2 loop, \u000A the expression <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> 4 (global-val 'cltl-command (w state)))</v>.  Those \u000A functions that contain <v>(<see topic=\"COMMON-LISP____DECLARE\">DECLARE</see> (STOBJ-INLINE-FN T))</v> will generate <tt><see topic=\"ACL2____DEFABBREV\">defabbrev</see></tt> forms because the <v>:inline</v> keyword of <tt><see topic=\"ACL2____DEFSTOBJ\">defstobj</see></tt> was \u000A supplied the value <v>t</v>.  The rest will generate <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s.</p> \u000A \u000A <p>We now recommend that you look at <see topic=\"ACL2____STOBJ-EXAMPLE-1-PROOFS\">stobj-example-1-proofs</see>.</p>"],

 "ACL2____PROMPT":[["Ld"],"ACL2 Sources","ACL2","<p>The prompt printed by ACL2 conveys information about various \u000A ``modes.''  See <see topic=\"ACL2____DEFAULT-PRINT-PROMPT\">default-print-prompt</see> and see <see topic=\"ACL2____LD-PROMPT\">ld-prompt</see> for \u000A details.</p> \u000A \u000A <p>The prompt during raw Lisp breaks is, with most Common Lisp \u000A implementations, adjusted by ACL2 to include the string <v>&quot;[RAW LISP</v>\"], \u000A in order to reminder users not to submit ACL2 forms there; see <see topic=\"ACL2____BREAKS\">breaks</see>. \u000A For Lisps that seem to use the same code for printing prompts at the top-level \u000A as in <see topic=\"ACL2____BREAKS\">breaks</see>, the top-level prompt is similarly adjusted.  For Lisps \u000A with the above prompt adjustment, The following forms may be executed in raw \u000A Lisp (i.e., after typing <v>:q</v>).</p> \u000A \u000A <code>(install-new-raw-prompt) ; install prompt with [RAW LISP] as described above\u000A(install-old-raw-prompt) ; revert to original prompt from host Common Lisp</code>"],

 "ACL2____OTF-FLG":[["Defthm","Thm","Xargs"],"ACL2 Sources","ACL2","<p>The value of this flag is normally <v>nil</v>.  If you want to \u000A prevent the theorem prover from abandoning its initial work upon pushing the \u000A second subgoal, set <v>:otf-flg</v> to <v>t</v>.</p> \u000A \u000A <p>Suppose you submit a conjecture to the theorem prover and the system splits \u000A it up into many subgoals.  Any subgoal not proved by other methods is \u000A eventually set aside for an attempted induction proof.  But upon setting aside \u000A the second such subgoal, the system chickens out and decides that rather than \u000A prove n&gt;1 subgoals inductively, it will abandon its initial work and \u000A attempt induction on the originally submitted conjecture.  The <v>:otf-flg</v> \u000A (Onward Thru the Fog) allows you to override this chickening out. When \u000A <v>:otf-flg</v> is <v>t</v>, the system will push all the initial subgoals and \u000A proceed to try to prove each, independently, by induction.</p> \u000A \u000A <p>Even when you don't expect induction to be used or to succeed, setting the \u000A <v>:otf-flg</v> is a good way to force the system to generate and display all \u000A the initial subgoals.</p> \u000A \u000A <p>For <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> and <tt><see topic=\"ACL2____THM\">thm</see></tt>, <v>:otf-flg</v> is a keyword argument \u000A that is a peer to <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> and <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt>.  It may \u000A be supplied as in the following examples; also see <see topic=\"ACL2____DEFTHM\">defthm</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (my-predicate x y) :rule-classes nil :otf-flg t)\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> append-assoc\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))\u000A  :hints ((&quot;Goal&quot; :induct t))\u000A  :otf-flg t)</code> \u000A \u000A <p>The <v>:otf-flg</v> may be supplied to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> via the <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> \u000A declare option.  When you supply an <v>:otf-flg</v> hint to <v>defun</v>, the flag \u000A is effective for the termination proofs and the guard proofs, if any.</p>"],

 "ACL2____ERROR-TRIPLES-AND-PARALLELISM":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>ACL2 supports the use of error triples in many features; e.g., <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>.  (For background on error triples, see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>.)  However, ACL2(p) does not support the use of error \u000A triples in some of these features (e.g., <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>) while <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see> is enabled.</p> \u000A \u000A <p>You may see an error message like the following when running ACL2(p) with \u000A <see topic=\"ACL2____WATERFALL-PARALLELISM\">waterfall-parallelism</see> enabled:</p> \u000A \u000A <code>ACL2 Error in ( THM ...):  Since we are translating a form in ACL2(p)\u000Aintended to be executed with waterfall parallelism enabled, the form\u000A(MY-STATE-MODIFYING-COMPUTED-HINT ID STATE) was expected to represent\u000Aan ordinary value, not an error triple (<see topic=\"ACL2____MV\">mv</see> erp val state), as would\u000Abe acceptable in a serial execution of ACL2.  Therefore, the form returning\u000Aa tuple of the form (<see topic=\"COMMON-LISP_____A2\">*</see> * STATE) is an error.  See :DOC unsupported-\u000Awaterfall-parallelism-features and :DOC error-triples-and-parallelism\u000Afor further explanation.</code> \u000A \u000A <p>In this particular example, the cause of the error was trying to use a \u000A computed hint that returned state, which is not allowed when executing the \u000A waterfall in parallel (see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see> \u000A for other related information).</p> \u000A \u000A <p>Often, the only reason users need to return state is so they can perform \u000A some output during the proof process.  In this case, we suggest using one of \u000A the state-free output functions, like <tt><see topic=\"ACL2____CW\">cw</see></tt> or <tt><see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see></tt>. \u000A If the user is concerned about the interleaving of their output with other \u000A output, these calls can be surrounded with the macro <tt><see topic=\"ACL2____WITH-OUTPUT-LOCK\">with-output-lock</see></tt>.</p> \u000A \u000A <p>Another frequent reason users return state is so they can cause a <v>soft</v> \u000A error and halt the proof process.  In this case, we suggest instead calling \u000A <tt><see topic=\"ACL2____ER\">er</see></tt> with the <v>hard</v> or <v>hard?</v> severity.  By using these \u000A mechanisms, the user avoids modifying <tt><see topic=\"ACL2____STATE\">state</see></tt>, a requirement for much of \u000A the code written in ACL2(p).</p> \u000A \u000A <p>You may encounter other similar error messages when using <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>, <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>, or <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>. \u000A Chances are that you are somehow returning an error triple when an ordinary \u000A value is needed.  If this turns out not to be the case, please let the ACL2 \u000A implementors know.</p>"],

 "ACL2____ADD-MACRO-FN":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> append binary-append)\u000A(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> append binary-append t)</code> \u000A \u000A <p>These examples each associate the function symbol <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> \u000A with the macro name <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>.  As a result, theory functions will \u000A understand that <v>append</v> refers to <v>binary-append</v> &mdash; see <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> &mdash; and moreover, proof output will be printed using \u000A <v>append</v> rather than <v>binary-append</v>.  In the first case, <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x\u000A (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))</v> is printed rather than <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)</v>.  In the second \u000A case, right-associated arguments are printed flat: <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)</v>.  Such \u000A right-association is considered only for binary function symbols; otherwise \u000A the optional third argument is ignored.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> macro-name function-name)\u000A(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> macro-name function-name nil) ; same as above\u000A(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> macro-name function-name t)</code> \u000A \u000A <p>This is a convenient way to add an entry to <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt> and \u000A at the same time extend the <tt><see topic=\"ACL2____UNTRANS-TABLE\">untrans-table</see></tt>.  As suggested by the \u000A example above, calls of a function in this table will be printed as \u000A corresponding calls of macros, with right-associated arguments printed flat in \u000A the case of a binary function symbol if the optional third argument is t.  In \u000A that case, for a binary function symbol <v>fn</v> associated with macro name \u000A <v>mac</v>, then a call <v>(fn arg1 (fn arg2 (... (fn argk arg))))</v> will be \u000A displayed to the user as though the ``term'' were <v>(mac arg1 arg2 ... argk\u000A arg)</v>.  For a call <v>(f a1 ... ak)</v> of a function symbol that is not \u000A binary, or the optional argument is not supplied as <v>t</v>, then the effect is \u000A simply to replace <v>f</v> by the corresponding macro symbol.  See <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>, which is invoked on the first two arguments.  Also see <see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see>, see <see topic=\"ACL2____UNTRANS-TABLE\">untrans-table</see>, and see <see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see>.</p>"],

 "ACL2____ADD-DEFAULT-HINTS":[["Default-hints"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> '((computed-hint-1 clause)\u000A                     (computed-hint-2 clause\u000A                                      stable-under-simplificationp)))\u000A(<see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> '((computed-hint-3 id clause world))\u000A                   :at-end t)</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs (see \u000A <see topic=\"ACL2____ADD-DEFAULT-HINTS_12\">add-default-hints!</see> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event).</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> lst)\u000A(<see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> lst :at-end flg)</code> \u000A \u000A <p>where <v>lst</v> is a list.  Generally speaking, the elements of <v>lst</v> \u000A should be suitable for use as <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>.</p> \u000A \u000A <p>This event is completely analogous to <tt><see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see></tt>, the \u000A difference being that <v>add-default-hints</v> appends the indicated hints to \u000A the front of the list of default hints, so that they are tried first &mdash; \u000A or, if <v>flg</v> is supplied and evaluates to other than <v>nil</v>, at the end \u000A of the list, so that they are tried last &mdash; rather than <b>replacing</b> \u000A the default hints with the indicated hints.  Each new hint is thus considered \u000A after each existing hints when both are applied to the same goal.  Also See \u000A <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>, see <see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see>, and see <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p>Finally, note that the effects of <v>set-default-hints</v>, <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt>, and <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see></tt> are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A book in which they appear.  Thus, users who include a book with such forms \u000A will not have their default hints affected by such forms.  In order to export \u000A the effect of setting the default hints, use <tt><see topic=\"ACL2____SET-DEFAULT-HINTS_12\">set-default-hints!</see></tt>, \u000A <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS_12\">add-default-hints!</see></tt>, or <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS_12\">remove-default-hints!</see></tt>.</p> \u000A \u000A <p>For a related feature, which however is only for advanced system builders, \u000A see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>.</p>"],

 "ACL2-PC____S":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000AS  -- simplify the current subterm\u000A(S :backchain-limit 2 :normalize t :expand (<see topic=\"COMMON-LISP____APPEND\">append</see> x z))\u000A   -- simplify the current subterm, but during the rewriting\u000A      process first ``normalize'' it by pushing IFs to the\u000A      top-level, and also force the term (<see topic=\"COMMON-LISP____APPEND\">append</see> x z) to be\u000A      expanded during the rewriting process\u000A\u000AGeneral Form:\u000A(s &amp;key rewrite normalize backchain-limit repeat in-theory hands-off\u000A        expand)</code> \u000A \u000A <p>Simplify the current subterm according to the keyword parameters supplied. \u000A First if-normalization is applied (unless the <v>normalize</v> argument is \u000A <v>nil</v>), i.e., each subterm of the form <v>(f ... (<see topic=\"COMMON-LISP____IF\">if</see> test x y) ...)</v>  is \u000A replaced by the term <v>(<see topic=\"COMMON-LISP____IF\">if</see> test (f ... x ...) (f ... y ...))</v>  except, of \u000A course, when <v>f</v> is <v>if</v> and the indicated <v>if</v> subterm is in the \u000A second or third argument position.  Then rewriting is applied (unless the \u000A rewrite argument is <v>nil</v>).  Finally this pair of actions is repeated \u000A &mdash; until the rewriting step causes no change in the term.  A description \u000A of each parameter follows.</p> \u000A \u000A <code>:rewrite -- default t</code> \u000A \u000A <p>When non-<v>nil</v>, instructs the system to use ACL2's rewriter (or, \u000A something close to it) during simplification.</p> \u000A \u000A <code>:normalize -- default t</code> \u000A \u000A <p>When non-<v>nil</v>, instructs the system to use if-normalization (as \u000A described above) during simplification.</p> \u000A \u000A <code>:backchain-limit -- default 0</code> \u000A \u000A <p>Sets the number of recursive calls to the rewriter that are allowed for \u000A backchaining.  Even with the default of 0, some reasoning is allowed \u000A (technically speaking, type-set reasoning is allowed) in the relieving of \u000A hypotheses.  The value should be <v>nil</v> or a non-negative integer, and \u000A limits backchaining only for rewriting, not for type-set reasoning.</p> \u000A \u000A <code>:repeat -- default 0</code> \u000A \u000A <p>Sets the number of times the current term is to be rewritten.  If this \u000A value is <v>t</v>, then the default is used (as specified by the constant \u000A <v>*default-s-repeat-limit*</v>).</p> \u000A \u000A <code>:in-theory, :hands-off, :expand</code> \u000A \u000A <p>These have their usual meaning; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p><b>Remark:</b> if conditional rewrite rules are used that cause case splits \u000A because of the use of <v>force</v>, then appropriate new subgoals will be \u000A created, i.e., with the same current subterm (and address) but with each new \u000A (forced) hypothesis being negated and then used to create a corresponding new \u000A subgoal.  In that case, the current goal will have all such new hypotheses \u000A added to the list of top-level hypotheses.</p>"],

 "COMMON-LISP_____C3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-&lt;</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A                (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y))\u000A           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((x1 (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x) x 0))\u000A               (y1 (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y) y 0)))\u000A           (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x1) (<see topic=\"COMMON-LISP____REALPART\">realpart</see> y1))\u000A               (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x1) (<see topic=\"COMMON-LISP____REALPART\">realpart</see> y1))\u000A                    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x1) (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> y1)))))))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y))</code> \u000A \u000A <p>Notice that like all arithmetic functions, <v>&lt;</v> treats non-numeric inputs \u000A as <v>0</v>.</p> \u000A \u000A <p>This function has the usual meaning on the rational numbers, but is \u000A extended to the complex rational numbers using the lexicographic order: first \u000A the real parts are compared, and if they are equal, then the imaginary parts \u000A are compared.</p>"],

 "COMMON-LISP____CONSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> x)</v> is true if and only if <v>x</v> is a <see topic=\"COMMON-LISP____CONS\">cons</see> \u000A pair.</p>"],

 "ACL2____WITHOUT-EVISC":[["Io"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> form)</code> \u000A \u000A <p>where <v>form</v> is any expression to evaluate.  The effect is to evaluate \u000A <v>form</v> as though the <v>without-evisc</v> wrapper were absent, except that \u000A expressions are printed in full for the ensuing output, regardless of the \u000A current evisc-tuples (see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>).  See <see topic=\"ACL2____SET-IPRINT\">set-iprint</see> for \u000A an example.</p> \u000A \u000A <p>More precisely, <v>without-evisc</v> binds each of the term-evisc-tuple, \u000A ld-evisc-tuple, abbrev-evisc-tuple and gag-mode-evisc-tuple to <v>nil</v> (see \u000A <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>).  It does not modify the trace evisc-tuple, so trace \u000A output is not modified by <v>without-evisc</v>.  Also note that calls of \u000A printing functions such as <tt><see topic=\"ACL2____FMT\">fmt</see></tt> that include explicit evisc-tuples will \u000A not have those evisc-tuples overridden.  The following example illustrates \u000A this point.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see>\u000A        (<see topic=\"ACL2____FMS\">fms</see> &quot;~x0~%&quot;\u000A             (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0 '((a b ((c d)) e f g) u v w x y)))\u000A             *standard-co*\u000A             state\u000A             (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 2 3 nil nil)))\u000A\u000A((A B # ...) U V ...)\u000A&lt;state&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p>We conclude with two remarks.  (1) A call of <v>without-evisc</v> on \u000A expression <v>exp</v> actually invokes a specialized call of <tt><see topic=\"ACL2____LD\">ld</see></tt> on a \u000A one-element list containing <v>exp</v>, which prints the value returned by \u000A evaluation of <v>exp</v> but actually returns the useless value <v>(<see topic=\"ACL2____MV\">mv</see> nil\u000A :invisible state)</v>.  So do not use <v>without-evisc</v> in programs; just use \u000A it at the top level of the ACL2 read-eval-print loop, or at least the top \u000A level of <v>ld</v>.  (2) Even when using without-evisc, if the ACL2 logical \u000A <see topic=\"ACL2____WORLD\">world</see> is part of the value returned, it will be printed in abbreviated \u000A form because the ACL2 read-eval-print loop always arranges for this to be the \u000A case, regardless of the ld-evisc-tuple.  For example:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____WITHOUT-EVISC\">without-evisc</see> (w state))\u000A&lt;world&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p>An alternative to the use of <v>without-evisc</v> is to explore large objects \u000A using the ACL2 function <v>(<see topic=\"ACL2____WALKABOUT\">walkabout</see> object state)</v>.  Some brief \u000A documentation is printed when you enter an interactive loop upon evaluating a \u000A call of <v>walkabout</v>.  We may add documentation for <v>walkabout</v> if that \u000A is requested.</p>"],

 "ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  While the most common use of the limit described below is in \u000A parallel proof \u000A (see <see topic=\"ACL2____PARALLEL-PROOF\">parallel-proof</see>), it also applies to all parallelism primitives \u000A (see <see topic=\"ACL2____PARALLEL-PROGRAMMING\">parallel-programming</see>) except <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt> &mdash; though we \u000A expect that rather few programming applications will encouter this limit.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> :none)     ; disable the limit\u000A(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> &lt;integer&gt;) ; set limit to &lt;integer&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see>, Section ``Another Granularity Issue \u000A Related to Thread Limitations'', for an explanation of how the host Lisp can \u000A run out of threads.  Also see <see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR\">set-total-parallelism-work-limit-error</see>.</p> \u000A \u000A <p>If the underlying runtime system (the Operating System, the host Lisp, \u000A etc.)  is unable to provide enough threads to finish executing the parallelism \u000A work given to it, the runtime system can crash in a very unpleasant manner \u000A (ranging from a Lisp error to completely freezing the machine).  To avoid this \u000A unpleasantness, ACL2(p) will attempt to avoid creating so much parallelism \u000A that the runtime system crashes.</p> \u000A \u000A <p>ACL2 initially uses a conservative estimate to limit the number of threads. \u000A To tell ACL2(p) to use a different limit, call \u000A <v>set-total-parallelism-work-limit</v> with the new limit.  For example, if the \u000A current default limit is 10,000, then to allow 13,000 threads to exist, issue \u000A the following form at the top level.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> 13000)</code> \u000A \u000A <p>To disable this limit altogether, evaluate the following form:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> :none)</code> \u000A \u000A <p>The default value of total-parallelism-work-limit can be found by calling \u000A function <v>default-total-parallelism-work-limit</v>.  If the default value is \u000A too high for your system please notify the ACL2 maintainers with a limit that \u000A does work for your system, as they might then lower the default limit.</p>"],

 "ACL2____NQTHM-TO-ACL2":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A:nqthm-to-acl2 prove-lemma   ; Display ACL2 topic(s) and/or print\u000A                             ; information corresponding to Nqthm\u000A                             ; PROVE-LEMMA command.\u000A(<see topic=\"ACL2____NQTHM-TO-ACL2\">nqthm-to-acl2</see> 'prove-lemma) ; Same as above.\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____NQTHM-TO-ACL2\">nqthm-to-acl2</see> name)</code> \u000A \u000A <p>where <v>name</v> is a notion documented for Nqthm: either a function in the \u000A Nqthm logic, or a command.  If there is corresponding information available \u000A for ACL2, it will be printed in response to this command.  This information is \u000A not intended to be completely precise, but rather, is intended to help those \u000A familiar with Nqthm to make the transition to ACL2.</p> \u000A \u000A <p>We close with two tables that contain all the information used by this \u000A <v>nqthm-to-acl2</v> command.  The first shows the correspondence between \u000A functions in the Nqthm logic and corresponding ACL2 functions (when possible); \u000A the second is similar, but for commands rather than functions.</p> \u000A \u000A <code>Nqthm functions  --&gt;     ACL2\u000A----------------------------------------\u000AADD1          --&gt;  1+\u000AADD-TO-SET    --&gt;  ADD-TO-SET-EQUAL and ADD-TO-SET-EQ\u000AAND           --&gt;  AND\u000AAPPEND        --&gt;  APPEND and BINARY-APPEND\u000AAPPLY-SUBR    --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AAPPLY$        --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AASSOC         --&gt;  ASSOC-EQUAL, ASSOC and ASSOC-EQ\u000ABODY          --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000ACAR           --&gt;  CAR\u000ACDR           --&gt;  CDR\u000ACONS          --&gt;  CONS\u000ACOUNT         --&gt;  ACL2-COUNT\u000ADIFFERENCE    --&gt;  -\u000AEQUAL         --&gt;  EQUAL, EQ, EQL and =\u000AEVAL$         --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AFALSE         --&gt;  Nqthm's F corresponds to the ACL2 symbol NIL.\u000AFALSEP        --&gt;  NOT and NULL\u000AFORMALS       --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AGEQ           --&gt;  &gt;=\u000AGREATERP      --&gt;  &gt;\u000AIDENTITY      --&gt;  IDENTITY\u000AIF            --&gt;  IF\u000AIFF           --&gt;  IFF\u000AIMPLIES       --&gt;  IMPLIES\u000ALEQ           --&gt;  &lt;=\u000ALESSP         --&gt;  &lt;\u000ALISTP         --&gt;  CONSP\u000ALITATOM       --&gt;  SYMBOLP\u000AMAX           --&gt;  MAX\u000AMEMBER        --&gt;  MEMBER-EQUAL, MEMBER and MEMBER-EQ\u000AMINUS         --&gt;  - and UNARY--\u000ANEGATIVEP     --&gt;  MINUSP\u000ANEGATIVE-GUTS --&gt;  ABS\u000ANLISTP        --&gt;  ATOM\u000ANOT           --&gt;  NOT\u000ANUMBERP       --&gt;  ACL2-NUMBERP, INTEGERP and RATIONALP\u000AOR            --&gt;  OR\u000AORDINALP      --&gt;  O-P\u000AORD-LESSP     --&gt;  O&lt;\u000APACK          --&gt;  See intern and coerce.\u000APAIRLIST      --&gt;  PAIRLIS$\u000APLUS          --&gt;  + and BINARY-+\u000AQUOTIENT      --&gt;  /\u000AREMAINDER     --&gt;  REM and MOD\u000ASTRIP-CARS    --&gt;  STRIP-CARS\u000ASUB1          --&gt;  1-\u000ATIMES         --&gt;  * and BINARY-*\u000ATRUE          --&gt;  The symbol T.\u000AUNION         --&gt;  UNION-EQUAL and UNION-EQ\u000AUNPACK        --&gt;  See symbol-name and coerce.\u000AV&amp;C$          --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AV&amp;C-APPLY$    --&gt;  No correspondent, but see the documentation for\u000A                   DEFEVALUATOR and META.\u000AZERO          --&gt;  The number 0.\u000AZEROP         --&gt;  ZP\u000A\u000A========================================\u000A\u000ANqthm commands   --&gt;     ACL2\u000A----------------------------------------\u000AACCUMULATED-PERSISTENCE\u000A              --&gt;  ACCUMULATED-PERSISTENCE\u000AADD-AXIOM     --&gt;  DEFAXIOM\u000AADD-SHELL     --&gt;  There is no shell principle in ACL2.\u000AAXIOM         --&gt;  DEFAXIOM\u000ABACKQUOTE-SETTING\u000A              --&gt;  Backquote is supported in ACL2, but not\u000A                   currently documented.\u000ABOOT-STRAP    --&gt;  GROUND-ZERO\u000ABREAK-LEMMA   --&gt;  MONITOR\u000ABREAK-REWRITE --&gt;  BREAK-REWRITE\u000ACH            --&gt;  PBT\u000A                   See also :DOC history.\u000ACHRONOLOGY    --&gt;  PBT\u000A                   See also :DOC history.\u000ACOMMENT       --&gt;  DEFLABEL\u000ACOMPILE-UNCOMPILED-DEFNS\u000A              --&gt;  COMP\u000ACONSTRAIN     --&gt;  See :DOC encapsulate and :DOC local.\u000ADATA-BASE     --&gt;  Perhaps the closest ACL2 analogue of DATA-BASE\u000A                   is PROPS.  But see :DOC history for a collection\u000A                   of commands for querying the ACL2 database\u000A                   (``world'').  Note that the notions of\u000A                   supporters and dependents are not supported in\u000A                   ACL2.\u000ADCL           --&gt;  DEFSTUB\u000ADEFN          --&gt;  DEFUN and DEFMACRO\u000ADEFTHEORY     --&gt;  DEFTHEORY\u000ADISABLE       --&gt;  DISABLE\u000ADISABLE-THEORY\u000A              --&gt;  See :DOC theories.  The Nqthm command\u000A                   (DISABLE-THEORY FOO) corresponds roughly to the\u000A                   ACL2 command\u000A                   (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see>\u000A                                (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                                (<see topic=\"ACL2____THEORY\">theory</see> 'foo))).\u000ADO-EVENTS     --&gt;  LD\u000ADO-FILE       --&gt;  LD\u000AELIM          --&gt;  ELIM\u000AENABLE        --&gt;  ENABLE\u000AENABLE-THEORY --&gt;  See :DOC theories.  The Nqthm command\u000A                   (ENABLE-THEORY FOO) corresponds roughly to the\u000A                   ACL2 command\u000A                   (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see>\u000A                                (<see topic=\"ACL2____THEORY\">theory</see> 'foo)\u000A                                (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))).\u000AEVENTS-SINCE  --&gt;  PBT\u000AFUNCTIONALLY-INSTANTIATE\u000A              --&gt;  ACL2 provides a form of the :USE hint that\u000A                   corresponds roughly to the\u000A                   FUNCTIONALLY-INSTANTIATE event of Nqthm. See\u000A                   :DOC lemma-instance.\u000AGENERALIZE    --&gt;  GENERALIZE\u000AHINTS         --&gt;  HINTS\u000ALEMMA         --&gt;  DEFTHM\u000AMAINTAIN-REWRITE-PATH\u000A              --&gt;  BRR\u000AMAKE-LIB      --&gt;  There is no direct analogue of Nqthm's notion of\u000A                   ``library.''  See :DOC books for a description\u000A                   of ACL2's mechanism for creating and saving\u000A                   collections of events.\u000AMETA          --&gt;  META\u000ANAMES         --&gt;  NAME\u000ANOTE-LIB      --&gt;  INCLUDE-BOOK\u000APPE           --&gt;  PE\u000APROVE         --&gt;  THM\u000APROVEALL      --&gt;  See :DOC ld and :DOC certify-book.  The latter\u000A                   corresponds to Nqthm's PROVE-FILE,which may be\u000A                   what you're interested in,really.\u000APROVE-FILE    --&gt;  CERTIFY-BOOK\u000APROVE-FILE-OUT\u000A              --&gt;  CERTIFY-BOOK\u000APROVE-LEMMA   --&gt;  DEFTHM\u000A                   See also :DOC hints.\u000AR-LOOP        --&gt;  The top-level ACL2 loop is an evaluation loop as\u000A                   well, so no analogue of R-LOOP is necessary.\u000AREWRITE       --&gt;  REWRITE\u000ARULE-CLASSES  --&gt;  RULE-CLASSES\u000ASET-STATUS    --&gt;  IN-THEORY\u000ASKIM-FILE     --&gt;  LD-SKIP-PROOFSP\u000ATOGGLE        --&gt;  IN-THEORY\u000ATOGGLE-DEFINED-FUNCTIONS\u000A              --&gt;  EXECUTABLE-COUNTERPART-THEORY\u000ATRANSLATE     --&gt;  TRANS and TRANS1\u000AUBT           --&gt;  UBT and U\u000AUNBREAK-LEMMA --&gt;  UNMONITOR\u000AUNDO-BACK-THROUGH\u000A              --&gt;  UBT\u000AUNDO-NAME     --&gt;  See :DOC ubt.  There is no way to undo names in\u000A                   ACL2 without undoing back through such names.\u000A                   However, see :DOC ld-skip-proofsp for\u000A                   information about how to quickly recover the\u000A                   state.</code>"],

 "ACL2____CURRENT-THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> 'lemma3)</code> \u000A \u000A <p>See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> logical-name)</code> \u000A \u000A <p>Returns the current theory as it existed immediately after the introduction \u000A of <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt> provided it is evaluated in an environment in which \u000A the variable symbol WORLD is bound to the current ACL2 logical world, <v>(w\u000A state)</v>.  Thus,</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</code> \u000A \u000A <p>will cause an (unbound variable) error while</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____WORLD\">world</see> (w state))) (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here))</code> \u000A \u000A <p>will return the current theory in world.</p> \u000A \u000A <p>See <see topic=\"ACL2____THEORIES\">theories</see> and see <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see> for a discussion of \u000A theories in general and why the commonly used ``theory functions'' such as \u000A <v>current-theory</v> are really macros that expand into terms involving the \u000A variable <v>world</v>.</p> \u000A \u000A <p>The theory returned by <v>current-theory</v> is in fact the theory selected \u000A by the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event most recently preceding logical name, extended \u000A by the rules introduced up through <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt>.</p> \u000A \u000A <p>You may experience a fencepost problem in deciding which logical name to \u000A use.  <tt><see topic=\"ACL2____DEFLABEL\">Deflabel</see></tt> can always be used to mark unambiguously for future \u000A reference a particular point in the development of your theory.  The order of \u000A <see topic=\"ACL2____EVENTS\">events</see> in the vicinity of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is confusing.  See \u000A <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____CHECK-SUM":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>A ``check sum'' is an integer in some fixed range computed from the \u000A printed representation of an object, e.g., the sum, modulo <v>2**32</v>, of the \u000A ascii codes of all the <see topic=\"ACL2____CHARACTERS\">characters</see> in the printed representation.</p> \u000A \u000A <p>Ideally, you would like the check sum of an object to be uniquely \u000A associated with that object, like a fingerprint.  It could then be used as a \u000A convenient way to recognize the object in the future: you could remember the \u000A check sum (which is relatively small) and when an object is presented to you \u000A and alleged to be the special one you could compute its check sum and see if \u000A indeed it was.  Alas, there are many more objects than check sums (after all, \u000A each check sum is an object, and then there's <v>t</v>).  So you try to design a \u000A check sum algorithm that maps similar looking objects far apart, in the hopes \u000A that corruptions and counterfeits &mdash; which appear to be similar to the \u000A object &mdash; have different check sums.  Nevertheless, the best you can do \u000A is a many-to-one map.  If an object with a different check sum is presented, \u000A you can be positive it is not the special object.  But if an object with the \u000A same check sum is presented, you have no grounds for positive \u000A identification.</p> \u000A \u000A <p>The basic check sum algorithm in ACL2 is called <v>check-sum-obj</v>, which \u000A computes the check sum of an ACL2 object.  Roughly speaking, we scan the print \u000A representation of the object and, for each character encountered, we multiply \u000A the ascii code of the character times its position in the stream (modulo a \u000A certain prime) and then add (modulo a certain prime) that into the running \u000A sum.  This is inaccurate in many senses (for example, we don't always use the \u000A ascii code and we see numbers as though they were printed in base 127) but \u000A indicates the basic idea.</p> \u000A \u000A <p>ACL2 uses check sums to increase security in the <see topic=\"ACL2____BOOKS\">books</see> mechanism; \u000A see <see topic=\"ACL2____CERTIFICATE\">certificate</see>.</p>"],

 "ACL2____WITH-FAST-ALIST":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>Logically, <v>with-fast-alist</v> just returns <v>form</v>.</p> \u000A \u000A <p>Under the hood, we cause <v>alist</v> to become a fast alist before executing \u000A <v>form</v>.  If doing so caused us to introduce a new hash table, the hash \u000A table is automatically freed after <v>form</v> completes.</p> \u000A \u000A <p>More accurately, under the hood <v>(<see topic=\"ACL2____WITH-FAST-ALIST\">with-fast-alist</see> name form)</v> \u000A essentially expands to something like:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (already-fast-alist-p name)\u000A    form\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((&lt;temp&gt; (<see topic=\"ACL2____MAKE-FAST-ALIST\">make-fast-alist</see> name)))\u000A    (prog1 form\u000A           (<see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see> &lt;temp&gt;))))</code> \u000A \u000A <p>Practically speaking, <v>with-fast-alist</v> is frequently a better choice \u000A then just using <tt><see topic=\"ACL2____MAKE-FAST-ALIST\">make-fast-alist</see></tt>, and is particularly useful for \u000A writing functions that can take either fast or slow alists as arguments.  That \u000A is, consider the difference between:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bad (alist ...)\u000A  (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((fast-alist (<see topic=\"ACL2____MAKE-FAST-ALIST\">make-fast-alist</see> alist))\u000A         (answer     (expensive-computation fast-alist ...)))\u000A   (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see> fast-alist)\u000A           answer)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> good (alist ...)\u000A   (<see topic=\"ACL2____WITH-FAST-ALIST\">with-fast-alist</see> alist\u000A     (expensive-computation alist ...)))</code> \u000A \u000A <p>Either approach is fine if the caller provides a slow <v>alist</v>.  But if \u000A the input <v>alist</v> is already fast, <v>bad</v> will (perhaps unexpectedly) \u000A free it!  On the other hand, <v>good</v> is able to take advantage of an \u000A already-fast argument and will not cause it to be inadvertently freed.</p> \u000A \u000A <p>See also the macro <v>with-fast-alists</v> defined in the community book \u000A <v>&quot;books/centaur/misc/hons-extra.lisp&quot;</v>, which allows you to call <tt><see topic=\"ACL2____WITH-FAST-ALIST\">with-fast-alist</see></tt> on several alists simultaneously.</p> \u000A \u000A <p>The community book <v>&quot;books/centaur/misc/hons-extra.lisp&quot;</v> extends the \u000A <see topic=\"ACL2____B_A2\">b*</see> macro with the <v>with-fast</v> pattern binder.  That is, after \u000A executing <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;centaur/misc/hons-extra.lisp&quot; :dir :system)</v> \u000A you may write something like this:</p> \u000A \u000A <code>(b* (...\u000A     ((with-fast a b c ...))\u000A     ...)\u000A  ...)</code> \u000A \u000A <p>which causes <v>a</v>, <v>b</v>, and <v>c</v> to become fast alists until the \u000A completion of the <v>b*</v> form.</p> \u000A \u000A <p>Note that <v>with-fast-alist</v> will cause logically tail-recursive \u000A functions not to execute tail-recursively if its cleanup phase happens after \u000A the tail-recursive call returns.</p>"],

 "ACL2____The_02Event_02Summary":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>At the conclusion of most events (click <see \u000A topic='ACL2____About_02the_02Prompt'>here</see> for a brief discussion of events or \u000A see <see topic=\"ACL2____EVENTS\">events</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>), ACL2 prints a summary.  The summary for <v>app</v> \u000A is:</p> \u000A \u000A <code>Summary\u000AForm:  ( DEFUN APP ...)\u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL))\u000AWarnings:  None\u000ATime:  0.03 seconds (prove: 0.00, print: 0.00, other: 0.03)\u000A APP</code> \u000A \u000A <p>The ``rules'' listed are those used in function admission or proof \u000A summarized.  What is actually listed are ``runes'' (see <see topic=\"ACL2____RUNE\">rune</see>) <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) \u000A which are list-structured names for rules in the ACL2 database or ``<see topic=\"ACL2____WORLD\">world</see>'' <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.  Using <see topic=\"ACL2____THEORIES\">theories</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> you \u000A can ``enable'' and ``disable'' rules so as to make them available (or not) to \u000A the ACL2 theorem prover.</p> \u000A \u000A <p>The ``warnings'' mentioned (none are listed for <v>app</v>) remind the reader \u000A whether the event provoked any warnings.  The warnings themselves would have \u000A been printed earlier in the processing and this part of the summary just names \u000A the earlier warnings printed.</p> \u000A \u000A <p>The ``time'' indicates how much processing time was used and is divided \u000A into three parts: the time devoted to proof, to printing, and to syntactic \u000A checks, pre-processing and database updates.  Despite the fact that ACL2 is an \u000A applicative language it is possible to measure time with ACL2 programs.  The \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> contains a clock.  The times are printed in decimal \u000A notation but are actually counted in integral units.  Note that by default, \u000A each time is a runtime, also known as a cpu time, as opposed to being a real \u000A time, also known as a wall clock time.</p> \u000A \u000A <p>The final <v>APP</v> is the value of the <v>defun</v> command and was printed \u000A by the read-eval-print loop.  The fact that it is indented one space is a \u000A subtle reminder that the command actually returned an ``error triple'', \u000A consisting of a flag indicating (in this case) that no error occurred, a value \u000A (in this case the symbol <v>APP</v>), and the final <tt><see topic=\"ACL2____STATE\">state</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A See <see topic=\"ACL2____LD-POST-EVAL-PRINT\">ld-post-eval-print</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> for \u000A some details.  If you really want to follow that link, however, you might see \u000A <see topic=\"ACL2____LD\">ld</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> first.</p> \u000A \u000A <p>You should now return to <see topic='ACL2____Revisiting_02the_02Admission_02of_02App'>the Walking Tour</see>.</p>"],

 "ACL2____TRANS":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A:trans (<see topic=\"COMMON-LISP____LIST\">list</see> a b c)\u000A:trans (<see topic=\"COMMON-LISP____CADDR\">caddr</see> x)\u000A:trans (<see topic=\"COMMON-LISP____COND\">cond</see> (p q) (r))</code> \u000A \u000A <p>This function takes one argument, an alleged term, and translates it, \u000A expanding the macros in it completely.  Either an error is caused or the \u000A formal meaning of the term is printed.  We also print the ``output signature'' \u000A which indicates how many results are returned and which are single-threaded \u000A objects.  For example, a term that returns one ordinary object (e.g., an \u000A object other than <tt><see topic=\"ACL2____STATE\">state</see></tt> or a user-defined single-threaded object (see \u000A <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>)) has the output signature</p> \u000A \u000A <code>=&gt; *</code> \u000A \u000A <p>A term that returns the single-threaded object <v>STATE</v> has the output \u000A signature</p> \u000A \u000A <code>=&gt; STATE</code> \u000A \u000A <p>and a term that returns four results might have the output signature</p> \u000A \u000A <code>=&gt; (<see topic=\"ACL2____MV\">MV</see> $MEM * * STATE)</code> \u000A \u000A <p>This signature indicates that the first result is the (user defined) \u000A single-threaded object <v>$MEM</v>, that the next two results are ordinary, and \u000A that the last result is <v>STATE</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____TRANS_12\">trans!</see> for a corresponding command that does not enforce \u000A restrictions of single-threaded objects.</p> \u000A \u000A <p>It is sometimes more convenient to use <tt><see topic=\"ACL2____TRANS1\">trans1</see></tt> which is like trans \u000A but which only does top-level macroexpansion.</p> \u000A \u000A <p>For more, see <see topic=\"ACL2____TERM\">term</see>.</p>"],

 "ACL2____TOP-LEVEL":[["Other"],"ACL2 Sources","ACL2","<p>Some forms, such as calls of <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt>, are illegal \u000A when supplied directly to the ACL2 top-level loop.  The macro <v>top-level</v> \u000A provides a workaround in such cases, by defining a temporary <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>-mode function named <v>top-level-fn</v> whose only argument is \u000A <v>state</v> and whose body is the form to be evaluated.  When the call of \u000A <v>top-level</v> returns there is no change to the existing ACL2 logical <see topic=\"ACL2____WORLD\">world</see>.  The following edited log illustrates all of the above points.</p> \u000A \u000A <code>ACL2 !&gt;:pbt 0\u000A          0  (<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000AACL2 !&gt;(<see topic=\"ACL2____DEFSTOBJ\">defstobj</see> st fld)\u000A\u000ASummary\u000AForm:  ( DEFSTOBJ ST ...)\u000ARules: NIL\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A ST\u000AACL2 !&gt;(<see topic=\"ACL2____TOP-LEVEL\">top-level</see>\u000A        (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A         st\u000A         (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A                 (<see topic=\"COMMON-LISP____LET\">let</see> ((st (update-fld 17 st)))\u000A                   (<see topic=\"ACL2____MV\">mv</see> (fld st) st))\u000A                 result)))\u000A17\u000AACL2 !&gt;(<see topic=\"ACL2____TOP-LEVEL\">top-level</see>\u000A        (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A         st\u000A         (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A                 (<see topic=\"COMMON-LISP____LET\">let</see> ((st (update-fld 17 st)))\u000A                   (<see topic=\"ACL2____MV\">mv</see> (fld st) st))\u000A                 (<see topic=\"ACL2____MV\">mv</see> nil result state))))\u000A 17\u000AACL2 !&gt;(<see topic=\"ACL2____TOP-LEVEL\">top-level</see>\u000A        (<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A         st\u000A         (<see topic=\"ACL2____MV-LET\">mv-let</see> (result st)\u000A                 (<see topic=\"COMMON-LISP____LET\">let</see> ((st (update-fld 17 st)))\u000A                   (<see topic=\"ACL2____MV\">mv</see> (fld st) st))\u000A                 (<see topic=\"ACL2____MV\">mv</see> result state))))\u000A(17 &lt;state&gt;)\u000AACL2 !&gt;:pbt 0\u000A          0  (<see topic=\"ACL2____EXIT-BOOT-STRAP-MODE\">EXIT-BOOT-STRAP-MODE</see>)\u000A   d      1:x(<see topic=\"ACL2____DEFSTOBJ\">DEFSTOBJ</see> ST FLD)\u000AACL2 !&gt;</code> \u000A \u000A <p>Each argument of <v>top-level</v> after the first should be a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A form or <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string, as the list of these extra arguments will \u000A be placed before the first argument when forming the definition of \u000A <v>top-level-fn</v>.</p> \u000A \u000A <p><tt><see topic=\"ACL2____TOP-LEVEL\">Top-level</see></tt> generates a call of <tt><see topic=\"ACL2____LD\">ld</see></tt>, so that the value \u000A returned is printed in the normal way.  The call of <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt> itself \u000A actually evaluates to <v>(<see topic=\"ACL2____MV\">mv</see> erp :invisible state)</v>, where <v>erp</v> is <v>t</v> \u000A if and only evaluation of the call of <v>top-level-fn</v> caused an error, which \u000A normally results in no additional output.  (For details about ``caused an \u000A error'', see the definition of <v>top-level</v> in the ACL2 source code, and see \u000A <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see>.)</p>"],

 "ACL2____SPECIAL-CASES-FOR-REWRITE-RULES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>In principle, every rewrite rule is made from a formula of this \u000A shape:</p> \u000A \u000A <code>(IMPLIES (AND <i>hyp1</i> ... <i>hypk</i>) \u000A         (<i>eqv</i> <i>lhs</i> <i>rhs</i>)) \u000A</code> \u000A \u000A <p>where <i>eqv</i> is either <v>EQUAL</v> or <v>IFF</v> and the result of \u000A expanding any abbreviations in <i>lhs</i> is the application of some function \u000A symbol other than <v>IF</v>.</p> \u000A \u000A <p>* In the special case where there is only one <i>hyp</i> term, i.e., \u000A <i>k</i>=1, the <v>(<see topic=\"COMMON-LISP____AND\">AND</see></v> <i>hyp1</i><v>)</v> can be written <i>hyp1</i>.</p> \u000A \u000A <p>* In the special case where there are no <i>hyp</i> terms, <i>k</i>=0, the \u000A <v>(<see topic=\"COMMON-LISP____AND\">AND</see>)</v> term is logically just <v>T</v> and the whole <v>IMPLIES</v> can be \u000A dropped; such a formula may be written as an unconditional <v>EQUAL</v> or \u000A <v>IFF</v> term.</p> \u000A \u000A <p>* If you build a rewrite rule from a formula that concludes with <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see></v> \u000A <i>x</i><v>)</v>, it is treated as though it were <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see></v> <i>x</i> \u000A <v>NIL)</v>, which is logically equivalent to what you typed.</p> \u000A \u000A <p>* If you build a rewrite rule from a formula that concludes with an \u000A <v>AND</v>, ACL2 will build a rewrite rule for each conjunct of the <v>AND</v>. \u000A This is because</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyp (<see topic=\"COMMON-LISP____AND\">AND</see> concl1 concl2))</code> \u000A \u000A <p>is propositionally equivalent to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyp concl1)\u000A     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyp concl2)).</code> \u000A \u000A <p>However, if you use an <v>OR</v>-expression as a hypothesis, ACL2 does \u000A <i>not</i> do the dual transformation.  Thus, <v>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____OR\">OR</see> hyp1 hyp2)\u000A concl)</v> generates one rewrite rule.</p> \u000A \u000A <p>* Finally, if you build a rewrite rule from a formula that does not \u000A conclude with an <v>EQUAL</v>, an <v>IFF</v>, a <v>NOT</v>, or an <v>AND,</v> but with \u000A some other term, say, <i>lhs</i>, then ACL2 acts like you typed <v>(<see topic=\"ACL2____IFF\">IFF</see></v> \u000A <i>lhs</i> <v>T)</v>, which is logically equivalent to what you typed.</p> \u000A \u000A <p>Thus, regardless of what you type, every rule has <i>k</i> hypotheses.  For \u000A unconditional rules, <i>k</i> is 0 and the hypotheses are vacuously true. \u000A Whether or not you write an <v>EQUAL</v> or an <v>IFF</v> in the conclusion, every \u000A rule is either an equality or a propositional equivalence, every rule has a \u000A left-hand side, and every rule has a right-hand side.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-REWRITE-RULES-PART-1\">introduction-to-rewrite-rules-part-1</see>.</p>"],

 "ACL2-PC____COMM":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Acomm\u000A(comm 10)\u000A\u000AGeneral Form:\u000A(comm &amp;optional n)</code> \u000A \u000A <p>Prints out instructions in reverse order.  This is actually the same as \u000A <v>(commands n t)</v> &mdash; or, <v>(commands nil t)</v> if <v>n</v> is not \u000A supplied.  As explained in the documentation for <v>commands</v>, the final \u000A argument of <v>t</v> causes suppression of instructions occurring between \u000A so-called ``matching bookends,'' which we now explain.</p> \u000A \u000A <p>A ``begin bookend'' is an instruction of the form</p> \u000A \u000A <code>(COMMENT :BEGIN x . y).</code> \u000A \u000A <p>Similarly, an ``end bookend'' is an instruction of the form</p> \u000A \u000A <code>(COMMENT :END x' . y').</code> \u000A \u000A <p>The ``name'' of the first bookend is <v>x</v> and the ``name'' of the second \u000A bookend is <v>x'</v>.  When such a pair of instructions occurs in the current \u000A state-stack, we call them ``matching bookends'' provided that they have the \u000A same name (i.e. <v>x</v> equals <v>x'</v>) and if no other begin or end bookend \u000A with name <v>x</v> occurs between them.  The idea now is that <v>comm</v> hides \u000A matching bookends together with the instructions they enclose.  Here is a more \u000A precise explanation of this ``hiding''; probably there is no value in reading \u000A on!</p> \u000A \u000A <p>A <v>comm</v> instruction hides bookends in the following manner.  (So does a \u000A <v>comment</v> instruction when its second optional argument is supplied and \u000A non-<v>nil</v>.)  First, if the first argument <v>n</v> is supplied and not \u000A <v>nil</v>, then we consider only the last <v>n</v> instructions from the \u000A state-stack; otherwise, we consider them all.  Now the resulting list of \u000A instructions is replaced by the result of applying the following process to \u000A each pair of matching bookends: the pair is removed, together with everything \u000A in between the begin and end bookend of the pair, and all this is replaced by \u000A the ``instruction''</p> \u000A \u000A <code>(&quot;***HIDING***&quot; :COMMENT :BEGIN name ...)</code> \u000A \u000A <p>where <v>(comment begin name ...)</v> is the begin bookend of the pair. \u000A Finally, after applying this process to each pair of matching bookends, each \u000A begin bookend of the form <v>(comment begin name ...)</v>  that remains is \u000A replaced by</p> \u000A \u000A <code>(&quot;***UNFINISHED***&quot; :COMMENT :BEGIN name ...) .</code>"],

 "ACL2-PC____CASESPLIT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(casesplit (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)) -- assuming that we are at the top of the\u000A                       conclusion, add (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) as a new top-level\u000A                       hypothesis in the current goal, and create a\u000A                       subgoal identical to the current goal except\u000A                       that it has (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)) as a new top-level\u000A                       hypothesis\u000A\u000AGeneral Form:\u000A(casesplit expr &amp;optional use-hyps-flag do-not-flatten-flag)</code> \u000A \u000A <p>When the current subterm is the entire conclusion, this instruction adds \u000A <v>expr</v> as a new top-level hypothesis, and create a subgoal identical to the \u000A existing current goal except that it has the negation of <v>expr</v> as a new \u000A top-level hypothesis.  See also <v>claim</v>.  The optional arguments control \u000A the use of governors and the ``flattening'' of new hypotheses, as we now \u000A explain.</p> \u000A \u000A <p>The argument <v>use-hyps-flag</v> is only of interest when there are \u000A governors.  (To read about governors, see the documentation for the command \u000A <v>hyps</v>).  In that case, if <v>use-hyps-flag</v> is not supplied or is \u000A <v>nil</v>, then the description above is correct; but otherwise, it is not \u000A <v>expr</v> but rather it is <v>(<see topic=\"ACL2____IMPLIES\">implies</see> govs expr)</v> that is added as a new \u000A top-level hypothesis (and whose negation is added as a top-level hypothesis \u000A for the new goal), where <v>govs</v> is the conjunction of the governors.</p> \u000A \u000A <p>If <v>do-not-flatten-flag</v> is supplied and not <v>nil</v>, then that is all \u000A there is to this command.  Otherwise (thus this is the default), when the \u000A claimed term (first argument) is a conjunction (<v>and</v>) of terms and the \u000A <v>claim</v> instruction succeeds, then each (nested) conjunct of the claimed \u000A term is added as a separate new top-level hypothesis.  Consider the following \u000A example, assuming there are no governors.</p> \u000A \u000A <code>(casesplit (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a)) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> r s)) t)</code> \u000A \u000A <p>Three new top-level hypotheses are added to the current goal, namely <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see>\u000A x y)</v>, <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a)</v>, and <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> r s)</v>.  In that case, only one \u000A hypothesis is added to create the new goal, namely the negation of <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see>\u000A x y) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> a) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> r s))</v>.  If the negation of this term had been \u000A <v>claim</v>ed, then it would be the other way around: the current goal would \u000A get a single new hypothesis while the new goal would be created by adding \u000A three hypotheses.</p> \u000A \u000A <p><b>Remark:</b> It is allowed to use abbreviations in the hints.</p>"],

 "COMMON-LISP____OR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Or</v> is the macro for disjunctions.  <v>Or</v> takes any number of \u000A arguments and returns the first that is non-<v>nil</v>, or <v>nil</v> if there is \u000A no non-<v>nil</v> element.</p> \u000A \u000A <p>In the ACL2 logic, the macroexpansion of <v>(<see topic=\"COMMON-LISP____OR\">or</see> x y)</v> is an <v>IF</v> term \u000A that appears to cause <v>x</v> to be evaluated twice:</p> \u000A \u000A <code>ACL2 !&gt;:trans (<see topic=\"COMMON-LISP____OR\">or</see> x y)\u000A\u000A(<see topic=\"COMMON-LISP____IF\">IF</see> X X Y)\u000A\u000A=&gt; *\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>If <v>x</v> were replaced by an expression whose evaluation takes a long \u000A time, then such an expansion would be ineffecient.  However, don't be fooled: \u000A you can expect Common Lisp implementations to avoid this problem, say by \u000A generating a new variable, for example:</p> \u000A \u000A <code>ACL2 !&gt;:q ; Exit the ACL2 loop and go into raw Common Lisp\u000A\u000AExiting the ACL2 read-eval-print loop.  To re-enter, execute (<see topic=\"ACL2____LP\">LP</see>).\u000AACL2&gt;(macroexpand '(<see topic=\"COMMON-LISP____OR\">or</see> x y))\u000A\u000A(<see topic=\"COMMON-LISP____LET\">LET</see> ((#:G5374 X)) (<see topic=\"COMMON-LISP____IF\">IF</see> #:G5374 #:G5374 Y))\u000AT\u000A\u000AACL2&gt;</code> \u000A \u000A <p><v>Or</v> is a Common Lisp macro.  See any Common Lisp documentation for more \u000A information.</p> \u000A \u000A <p><b>Macro: </b><srclink>or</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> or (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (or-macro args))</code> \u000A <p><b>Function: </b><srclink>or-macro</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> or-macro (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> lst)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A               (<see topic=\"COMMON-LISP____LIST\">list</see> 'if\u000A                     (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A                     (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A                     (or-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)))\u000A               (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A           nil))</code>"],

 "COMMON-LISP____LIST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>List</v> is the macro for building a list of objects.  For \u000A example, <v>(<see topic=\"COMMON-LISP____LIST\">list</see> 5 6 7)</v> returns a list of length 3 whose elements are \u000A <v>5</v>, <v>6</v>, and <v>7</v> respectively.  Also see <see topic=\"COMMON-LISP____LIST_A2\">list*</see>.</p> \u000A \u000A <p><v>List</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> list (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (list-macro args))</code> \u000A <p><b>Function: </b><srclink>list-macro</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> list-macro (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> lst)\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> 'cons\u000A                 (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A                       (<see topic=\"COMMON-LISP____CONS\">cons</see> (list-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)) nil)))\u000A           nil))</code>"],

 "ACL2____UNSUPPORTED-PARALLELISM-FEATURES":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see> for an introduction to parallel \u000A programming in ACL2.</p> \u000A \u000A <p>For proof features of ACL2 that are not yet supported when parallel \u000A execution is enabled for the primary ACL2 proof process, generally known as \u000A ``the waterfall'', see <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">unsupported-waterfall-parallelism-features</see>.</p> \u000A \u000A <p>Please note that this topic discusses ACL2 features that are disabled when \u000A using ACL2(p) (see <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see>).  These features are disabled \u000A regardless of whether waterfall parallelism is enabled.</p> \u000A \u000A <p>Calls of <tt><see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see></tt> simply convert to calls of <tt><see topic=\"ACL2____CW\">cw</see></tt>, so \u000A suppressing <tt><see topic=\"ACL2____OBSERVATION\">observation</see></tt>s (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) will not \u000A suppress these messages.</p> \u000A \u000A <p>Memoization is not supported when executing in parallel.  See <see topic=\"ACL2____UNSUPPORTED-WATERFALL-PARALLELISM-FEATURES\">Unsupported-waterfall-parallelism-features</see> for memoization details related to \u000A waterfall parallelism.</p> \u000A \u000A <p>Since, as of April 2012, garbage collection is inherently sequential, \u000A ACL2(p) minimizes the use of garbage collection by setting a high garbage \u000A collection threshold.  As a result, ACL2(p) is not expected to perform well on \u000A machines with less memory than this threshold (1 gigabyte, as of April \u000A 2012).</p> \u000A \u000A <p>In CCL, the underlying parallel execution engine is tuned for the number of \u000A CPU cores (or hardware threads) actually available in the machine.  SBCL and \u000A LispWorks are tuned for a machine with 16 CPU cores.</p> \u000A \u000A <p>CCL is considered to be the ``flagship Lisp'' for parallel execution in \u000A ACL2.  The SBCL and LispWorks implementations are thought to be generally \u000A stable.  However, due to their relatively less common use, the SBCL and \u000A LispWorks implementations are likely less robust than the CCL \u000A implementation.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____TIME-TRACKER\">time-tracker</see></tt> utility is a no-op for ACL2(p).</p>"],

 "ACL2____THEORIES-AND-PRIMITIVES":[["Theories"],"ACL2 Sources","ACL2","<p>When you <see topic=\"ACL2____DISABLE\">disable</see> the <see topic=\"ACL2____DEFINITION\">definition</see> or <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> of a built-in function, you may see a warning, for \u000A example as follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> mv-nth))\u000A\u000AACL2 Warning [Theory] in ( IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ...)):  Although the\u000Atheory expression (<see topic=\"ACL2____DISABLE\">DISABLE</see> MV-NTH) disables the :DEFINITION rule for\u000AMV-NTH, some expansions involving this function may still occur.  See\u000A:DOC theories-and-primitives.</code> \u000A \u000A <p>This warning can be eliminated by turning off all theory warnings (see \u000A <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>) or simply by evaluating the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> verbose-theory-warning nil)</code> \u000A \u000A <p>But before you eliminate such warnings, you may wish to read the following \u000A to understand their significance.</p> \u000A \u000A <p>First consider the following example, evaluated after the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> \u000A event displayed above.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____MV-NTH\">mv-nth</see> 2 (<see topic=\"COMMON-LISP____LIST\">list</see> a b c d e)) c))\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: ((:DEFINITION MV-NTH)\u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL))\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000AProver steps counted:  19\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>Note that even though the <see topic=\"ACL2____DEFINITION\">definition</see> of <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> had been \u000A <see topic=\"ACL2____DISABLE\">disable</see>d, nevertheless its definition rule was used in proving this \u000A theorem.  It is as though <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> had not been been disabled after all! \u000A The warning is intended to indicate that expansion of <v>mv-nth</v> calls may be \u000A made by the theorem prover even when <v>mv-nth</v> is disabled.  Indeed, the \u000A prover has special-purpose code for simplifying certain <v>mv-nth</v> calls.</p> \u000A \u000A <p>A similar issue can arise for <v>executable-counterpart</v> rules, as the \u000A following log illustrates.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart symbolp)))\u000A\u000AACL2 Warning [Theory] in ( IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ...)):  Although the\u000Atheory expression (<see topic=\"ACL2____DISABLE\">DISABLE</see> (:EXECUTABLE-COUNTERPART SYMBOLP)) disables\u000Athe :EXECUTABLE-COUNTERPART rule for SYMBOLP, some calls involving\u000Athis function may still be made.  See :DOC theories-and-primitives.\u000A\u000ASummary\u000AForm:  ( IN-THEORY (<see topic=\"ACL2____DISABLE\">DISABLE</see> ...))\u000ARules: NIL\u000AWarnings:  Theory\u000ATime:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)\u000A 2921\u000AACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> 'a))\u000A\u000AQ.E.D.\u000A\u000ASummary\u000AForm:  ( THM ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A\u000AProof succeeded.\u000AACL2 !&gt;</code> \u000A \u000A <p>In general, ACL2 warns when <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> or <see topic=\"ACL2____HINTS\">hints</see> \u000A leave you in a theory where a rule for a built-in function is disabled but may \u000A be applied in some cases nonetheless, because of special-purpose prover code \u000A for handling calls of that function.  The built-in function symbols with such \u000A <see topic=\"ACL2____DEFINITION\">definition</see> rules or <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> rules are those in \u000A the following two lists, respectively.</p> \u000A \u000A <code>ACL2 !&gt;*definition-minimal-theory*\u000A(<see topic=\"ACL2____MV-NTH\">MV-NTH</see> IFF NOT\u000A        IMPLIES EQ ATOM EQL = /= NULL ENDP ZEROP\u000A        SYNP PLUSP MINUSP LISTP RETURN-LAST\u000A        MV-LIST THE-CHECK WORMHOLE-EVAL\u000A        FORCE CASE-SPLIT DOUBLE-REWRITE)\u000AACL2 !&gt;*built-in-executable-counterparts*\u000A(<see topic=\"ACL2____ACL2-NUMBERP\">ACL2-NUMBERP</see> BINARY-* BINARY-+ UNARY-- UNARY-/\u000A              &lt; CAR CDR CHAR-CODE CHARACTERP CODE-CHAR\u000A              COMPLEX COMPLEX-RATIONALP COERCE\u000A              CONS CONSP DENOMINATOR EQUAL IF IMAGPART\u000A              INTEGERP INTERN-IN-PACKAGE-OF-SYMBOL\u000A              NUMERATOR PKG-WITNESS PKG-IMPORTS\u000A              RATIONALP REALPART STRINGP SYMBOL-NAME\u000A              SYMBOL-PACKAGE-NAME SYMBOLP NOT)\u000AACL2 !&gt;</code>"],

 "ACL2____SPEC-MV-LET":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib-with-step-count (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 33))\u000A      (fib-with-step-count x)\u000A    (<see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see>\u000A     (a cnt1)\u000A     (pfib-with-step-count (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A     (<see topic=\"ACL2____MV-LET\">mv-let</see> (b cnt2)\u000A             (pfib-with-step-count (<see topic=\"COMMON-LISP____-\">-</see> x 2))\u000A             (<see topic=\"COMMON-LISP____IF\">if</see> t\u000A                 (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP_____B2\">+</see> a b)\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> 1 cnt1 cnt2))\u000A               (<see topic=\"ACL2____MV\">mv</see> &quot;speculative result is always needed&quot;\u000A                   -1))))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see>\u000A (v1 ... vn)  ; bind distinct variables\u000A &lt;spec&gt;       ; evaluate speculatively; return n values\u000A (<see topic=\"ACL2____MV-LET\">mv-let</see>      ; or, use mv?-let if k=1 below\u000A  (w1 ... wk) ; bind distinct variables\u000A  &lt;eager&gt;     ; evaluate eagerly\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> &lt;test&gt;  ; use results from &lt;spec&gt; if true\u000A      &lt;typical-case&gt; ; may mention v1 ... vn\u000A    &lt;abort-case&gt;)))  ; does not mention v1 ... vn</code> \u000A \u000A <p>Our design of <v>spec-mv-let</v> is guided by its use in ACL2 source code to \u000A parallelize part of ACL2's proof process, in the experimental parallel \u000A extension of ACL2.  The user can think of <v>spec-mv-let</v> as a speculative \u000A version of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt>.  (In ordinary ACL2, the semantics agree with this \u000A description but without speculative or parallel execution.)</p> \u000A \u000A <p>Evaluation of the above general form proceeds as suggested by the comments. \u000A First, <v>&lt;spec&gt;</v> is executed speculatively.  Control then passes immediately \u000A to the <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> call, without waiting for the result of evaluating \u000A <v>&lt;spec&gt;</v>.  The variables <v>(w1 ... wk)</v> are bound to the result of \u000A evaluating <v>&lt;eager&gt;</v>, and then <v>&lt;test&gt;</v> is evaluated.  If the value of \u000A <v>&lt;test&gt;</v> is true, then the values of <v>(v1 ... vn)</v> are needed, and \u000A <v>&lt;typical-case&gt;</v> blocks until they are available.  If the value of \u000A <v>&lt;test&gt;</v> is false, then the values of <v>(v1 ... vn)</v> are not needed, and \u000A the evaluation of <v>&lt;spec&gt;</v> may be aborted.</p> \u000A \u000A <p>The calls to <v>mv-let</v> and to <v>if</v> displayed above in the General Form \u000A are an essential part of the design of <v>spec-mv-let</v>, and are thus \u000A required.</p> \u000A \u000A <p>The following definition of <v>fib-with-step-count</v> completes the example \u000A above:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fib-with-step-count (x)\u000A(<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A(<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0)\u000A       (<see topic=\"ACL2____MV\">mv</see> 0 1))\u000A      ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) (<see topic=\"ACL2____MV\">mv</see> 1 1))\u000A      (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (a cnt1)\u000A                 (fib-with-step-count (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                 (<see topic=\"ACL2____MV-LET\">mv-let</see> (b cnt2)\u000A                         (fib-with-step-count (<see topic=\"COMMON-LISP____-\">-</see> x 2))\u000A                         (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP_____B2\">+</see> a b)\u000A                             (<see topic=\"COMMON-LISP_____B2\">+</see> 1 cnt1 cnt2)))))))</code>"],

 "ACL2____LD-PRE-EVAL-PRINT":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-pre-eval-print</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>). \u000A The accessor is <v>(<see topic=\"ACL2____LD-PRE-EVAL-PRINT\">ld-pre-eval-print</see> state)</v> and the updater is \u000A <v>(set-ld-pre-eval-print val state)</v>.  <v>Ld-pre-eval-print</v> must be either \u000A <v>t</v>, <v>nil</v>, or <v>:never</v>.  The initial value of <v>ld-pre-eval-print</v> \u000A is <v>nil</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-pre-eval-print</v> is one of them.  If this global variable is <v>t</v>, \u000A then before evaluating the form just read from <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A prints the form to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  If the variable is <v>nil</v>, no such \u000A printing occurs.  The <v>t</v> option is useful if you are reading from a file \u000A of <see topic=\"ACL2____COMMAND\">command</see>s and wish to assemble a complete script of the session in \u000A <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.</p> \u000A \u000A <p>The value <v>:never</v> of <v>ld-pre-eval-print</v> is rarely used.  During the \u000A evaluation of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> forms, \u000A subsidiary events are normally printed, even if <v>ld-pre-eval-print</v> is \u000A <v>nil</v>.  Thus for example, when the user submits an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A form, all subsidiary events are generally printed even in the default \u000A situation where <v>ld-pre-eval-print</v> is <v>nil</v>.  But occasionally one may \u000A want to suppress such printing.  In that case, <v>ld-pre-eval-print</v> should \u000A be set to <v>:never</v>.  As described elsewhere (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), another way to suppress such printing is to execute \u000A <v>(<see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> lst)</v> where <v>lst</v> evaluates to a list including \u000A <v>'prove</v> and <v>'event</v>.</p>"],

 "ACL2____INVISIBLE-FNS-TABLE":[["Loop-stopper"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see> (w state))\u000A((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> unary--)\u000A (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> unary-/)\u000A (<see topic=\"ACL2____UNARY--\">unary--</see> unary--)\u000A (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> unary-/))</code> \u000A \u000A <p>Among other things, the setting above has the effect of making <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> ``invisible'' for the purposes of applying permutative <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> trees.  Also see <see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> \u000A and see <see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see>, which manage macro aliases (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>), as well as see <see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____TABLE\">table</see> for a general discussion of tables.</p> \u000A \u000A <p>The ``invisible functions <see topic=\"ACL2____TABLE\">table</see>'' is an alist with elements of the \u000A following form, where <v>fn</v> is a function symbol and the <v>ufni</v> are unary \u000A function symbols in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>, and <v>k</v> is at least \u000A 1.</p> \u000A \u000A <code>(fn ufn1 ufn2 ... ufnk)</code> \u000A \u000A <p>This <see topic=\"ACL2____TABLE\">table</see> thus associates with certain function symbols, e.g., \u000A <v>fn</v> above, a set of unary functions, e.g., the <v>ufni</v> above.  The \u000A <v>ufni</v> associated with <v>fn</v> in the invisible functions table are said to \u000A be ``invisible with respect to <v>fn</v>.''  If <v>fn</v> is not the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> \u000A of any pair in the <v>alist</v>, then no function is invisible for it.  Thus for \u000A example, setting the invisible functions alist to <v>nil</v> completely \u000A eliminates the consideration of invisibility.</p> \u000A \u000A <p>The notion of invisibility is involved in the use of the <v>:</v><tt><see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see></tt> field of <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules to prevent the indefinite \u000A application of permutative rewrite rules.  Roughly speaking, if rewrite rules \u000A are being used to permute <v>arg</v> and (ufni arg) inside of a nest of <v>fn</v> \u000A calls, and <v>ufni</v> is invisible with respect to <v>fn</v>, then <v>arg</v> and \u000A <v>(ufni arg)</v> are considered to have the same ``weight'' and will be \u000A permuted so as to end up as adjacent tips in the <v>fn</v> nest.  See <see topic=\"ACL2____LOOP-STOPPER\">loop-stopper</see>.</p>"],

 "ACL2____FREE-VARIABLES-TYPE-PRESCRIPTION":[["Free-variables"],"ACL2 Sources","ACL2","<p>We assume familiarity with the issue of dealing with free variables \u000A in hypotheses; see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>.</p> \u000A \u000A <p>By default, starting with Version 4.3, ACL2 attempts all possible matches \u000A for free variables.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f1 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f2 (x y) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f3 (y) t)\u000A\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> f1-prop\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (f2 x y) ; &lt;-- y is free in this hypothesis\u000A                (f3 y))\u000A           (f1 x))       ; &lt;-- (f1 x) is the type-term (<see topic=\"COMMON-LISP____TYPE\">type</see> is `non-nil')\u000A  :rule-classes :type-prescription)\u000A\u000A; Succeeds:\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (f2 a b)\u000A                   (f3 b))\u000A              (f1 a)))\u000A\u000A; The following fails unless we try more than one match for free variables in\u000A; hypotheses.\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (f2 a b)\u000A                   (f2 a c)\u000A                   (f2 a d)\u000A                   (f3 b))\u000A              (f1 a)))</code> \u000A \u000A <p>There may be times when you want to match only the first free variable.  In \u000A that case, you can write a function of two arguments, the \u000A <i>type-prescription</i> <see topic=\"ACL2____RUNE\">rune</see> being applied and the current ACL2 world, \u000A that prohibits multiple matching for those times.  Your function is then \u000A `attached' to the built-in constrained function, <v>oncep-ts</v>.  The following \u000A examples are intended to explain how this works.</p> \u000A \u000A <p>First, let us disallow all mutliple matching of free variables (i.e., \u000A implement the behavior often referred to as ``<v>:match-free :once</v>''; see \u000A <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oncep-tp-always (<see topic=\"ACL2____RUNE\">rune</see> wrld)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> rune wrld)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic :guard t))\u000A  t)\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> oncep-tp oncep-tp-always)</code> \u000A \u000A <p>The second <v>thm</v> form above will now fail, because only one \u000A free-variable match is permitted for the first hypothesis of rule <v>f1-prop</v> \u000A above.</p> \u000A \u000A <p>Now suppose that instead, we want to disallow multiple matches for free \u000A variables in hypotheses of type-prescription rules <i>except</i> for the rule \u000A <v>f1-prop</v> above.  With the following events, the second <v>thm</v> form above \u000A once again succeeds.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oncep-tp-always-except-f1-prop (<see topic=\"ACL2____RUNE\">rune</see> wrld)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> wrld)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> rune)\u000A                                           (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> rune))\u000A                                           (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> (<see topic=\"COMMON-LISP____CADR\">cadr</see> rune)))))\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (base-symbol rune) 'f1-prop)))\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> oncep-tp oncep-tp-always-except-f1-prop)</code> \u000A \u000A <p>In general, your <tt><see topic=\"ACL2____DEFATTACH\">defattach</see></tt> event will attach a function symbol to \u000A <v>oncep-tp</v>.  The <see topic=\"ACL2____GUARD\">guard</see> of that function symbol must be implied by \u000A the guard of <v>oncep-tp</v>:</p> \u000A \u000A <code>ACL2 !&gt;:args oncep-tp\u000A\u000AFunction         ONCEP-TP\u000AFormals:         (<see topic=\"ACL2____RUNE\">RUNE</see> WRLD)\u000ASignature:       (ONCEP-TP * *)\u000A                 =&gt; *\u000AGuard:           (<see topic=\"COMMON-LISP____AND\">AND</see> (PLIST-WORLDP WRLD)\u000A                      (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> RUNE)\u000A                      (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> RUNE))\u000A                      (<see topic=\"COMMON-LISP____SYMBOLP\">SYMBOLP</see> (<see topic=\"COMMON-LISP____CADR\">CADR</see> RUNE)))\u000AGuards Verified: T\u000ADefun-Mode:      :logic\u000AType:            built-in (<see topic=\"COMMON-LISP____OR\">or</see> unrestricted)\u000A\u000A ONCEP-TP\u000AACL2 !&gt;</code>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-WITH-ACCUMULATORS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p>To prove <v>(p x a)</v> for all <v>x</v> and all <v>a</v>, prove each of the \u000A following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (endp x) \u000A         (p x a)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp x)) \u000A              (p (cdr x) (cons (car x) a))) \u000A         (p x a)) \u000A</code> \u000A \u000A <p>Note that in the induction hypothesis we assume <v>p</v> for a smaller <v>x</v> \u000A but a larger <v>a</v>.  In fact, we could include as many induction hypotheses \u000A as we want and use any terms we want in the <v>a</v> position as long as the \u000A <v>x</v> position is occupied by a smaller term.</p> \u000A \u000A <p>A function that suggests this particular induction is shown below.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev1 (x a)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      a\u000A      (rev1 (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a)))).</code> \u000A \u000A <p>A function that suggests a similar induction in which three induction \u000A hypotheses are provided, one in which the <v>a</v> position is occupied by \u000A <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a)</v>, another in which the <v>a</v> position is occupied by \u000A some arbitrary term <v>b</v>, and a third in which the <v>a</v> position is \u000A occupied by <v>a</v>, is suggested by the term <v>(rev1-modified x a b)</v> \u000A where</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev1-modified (x a b)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> x a b)\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> (rev1-modified (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a) b)\u000A            (rev1-modified (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) b b)\u000A            (rev1-modified (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) a b))))</code> \u000A \u000A <p>Remember that the value of this term or function is irrelevant to the \u000A induction suggested.  Because ACL2's definitional principle insists that all \u000A the formal parameters play a role in the computation (at least syntactically), \u000A it is common practice when defining functions for their induction schemes to \u000A return the <v>list</v> of all the formals (to insure all variables are involved) \u000A and to combine recursive calls on a given branch with <v>list</v> (to avoid \u000A introducing additional case analysis as would happen if <v>and</v> or <v>or</v> or \u000A other propositional functions are used).</p> \u000A \u000A <p>If you tried to prove <v>(p x a)</v> and suggested the induct hint \u000A <v>(rev1-modified x a (fact k))</v>, as by</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (p x a)\u000A     :hints ((&quot;Goal&quot; :induct (rev1-modified x a (fact k)))))</code> \u000A \u000A <p>the inductive argument would be:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (endp x) \u000A         (p x a)) \u000A</code> \u000A \u000A <code><i>Inductive Step</i>: \u000A(implies (and (not (endp x)) \u000A              (p (cdr x) (cons (car x) a)) \u000A              (p (cdr x) (fact k)) \u000A              (p (cdr x) a)) \u000A         (p x a)) \u000A</code>"],

 "ACL2____DOCUMENTATION-COPYRIGHT":[["Copyright","Documentation"],"ACL2 Sources","ACL2","<p>There are two manuals associated with ACL2: the ACL2 User's Manual \u000A and the ACL2+Books Manual.  See <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.  The former is \u000A distributed with ACL2, you can reach many links into it from the ACL2 home \u000A page.  The latter may be preferred for routine browsing, since it extends the \u000A ACL2 User's Manual with documentation obtained from the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p> \u000A \u000A <p>The ACL2 User's Manual is copyrighted under the terms of the <v>LICENSE</v> \u000A file distributed with ACL2.  Its original authors are the ACL2 authors, but it \u000A is now defined in an ACL2 community book, <v>books/system/doc/acl2-doc.lisp</v>, \u000A so that members of the ACL2 community may contribute to it.</p> \u000A \u000A <p>The ACL2+Books Manual is a mechanically generated mashup derived from both \u000A the ACL2 User's Manual and the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  The ACL2+Books Manual \u000A thus has contributions from many authors.  At the top of each topic, in a line \u000A under the topic name, you will generally find either ``ACL2 Sources'' or the \u000A name of a Community Book.  In the former case, the text is from the ACL2 \u000A User's Manual and is authored, copyrighted, and licensed as per the ACL2 <see topic=\"ACL2____COPYRIGHT\">copyright</see>.  When a book is named, the content was extracted from that book \u000A which may be inspected for authorship, copyright, and license terms.</p> \u000A \u000A <p>There are two standard tools for browsing these manuals, other than using \u000A the <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> command at the terminal.</p> \u000A \u000A <ul> \u000A \u000A <li><b>The ACL2 <see topic=\"ACL2____XDOC\">Xdoc</see> Fancy Viewer.</b> This tool, written by Jared \u000A Davis, is included with the web-based version of each manual.  Information on \u000A copyright and licensing are provided in <a href='LICENSE'>its LICENSE \u000A file</a>.</li> \u000A \u000A <li><b>The <see topic=\"ACL2____ACL2-DOC\">ACL2-doc</see> Emacs browser.</b> This tool, authored by Matt \u000A Kaufmann and J Strother Moore, is distributed with ACL2 and is licensed under \u000A the terms of the <v>LICENSE</v> file distributed with ACL2.</li> \u000A \u000A </ul>"],

 "ACL2____Common_02Lisp":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____An_02Example_02Common_02Lisp_02Function_02Definition'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p><img src='common-lisp.gif'></img></p> \u000A \u000A <p>The logic of ACL2 is based on Common Lisp.</p> \u000A \u000A <p>Common Lisp is the standard list processing programming language.  It is \u000A documented in: Guy L. Steele, <a \u000A href='https://www.cs.cmu.edu/Groups/AI/html/cltl/cltl2.html'><b>Common Lisp \u000A The Language</b></a>, Digital Press, 12 Crosby Drive, Bedford, MA 01730, \u000A 1990.</p> \u000A \u000A <p>ACL2 formalizes only a subset of Common Lisp.  It includes such familiar \u000A Lisp functions as <v>cons</v>, <v>car</v> and <v>cdr</v> for creating and \u000A manipulating list structures, various arithmetic primitives such as <v>+</v>, \u000A <v>*</v>, <v>expt</v> and <v>&lt;=</v>, and <v>intern</v> and <v>symbol-name</v> for \u000A creating and manipulating symbols.  Control primitives include <v>cond</v>, \u000A <v>case</v> and <v>if</v>, as well as function call, including recursion.  New \u000A functions are defined with <v>defun</v> and macros with <v>defmacro</v>.  See \u000A <see topic=\"ACL2____PROGRAMMING\">programming</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> for a list of the Common Lisp primitives supported \u000A by ACL2.</p> \u000A \u000A <p>ACL2 supports five of Common Lisp's datatypes:</p> \u000A \u000A <p>* the precisely represented, unbounded numbers (integers, rationals, and \u000A the complex numbers with rational components, called the ``complex rationals'' \u000A here),</p> \u000A \u000A <p>* the characters with ASCII codes between 0 and 255</p> \u000A \u000A <p>* strings of such characters</p> \u000A \u000A <p>* symbols (including packages)</p> \u000A \u000A <p>* conses</p> \u000A \u000A <p>ACL2 is a very small subset of full Common Lisp.  ACL2 does not include the \u000A Common Lisp Object System (CLOS), higher order functions, circular structures, \u000A and other aspects of Common Lisp that are <b>non-applicative</b>.  Roughly \u000A speaking, a language is applicative if it follows the rules of function \u000A application.  For example, <v>f(x)</v> must be equal to <v>f(x)</v>, which means, \u000A among other things, that the value of <v>f</v> must not be affected by ``global \u000A variables'' and the object <v>x</v> must not change over time.</p> \u000A \u000A <p><see topic='ACL2____An_02Example_02Common_02Lisp_02Function_02Definition'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____CW-GSTACK":[["Break-rewrite","Debugging"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see>)\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :frames 10)       ; show only the top 10 frames\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :frames '(1 10))  ; same as above:  show only frames 1 through 10\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :frames '(10 20)) ; show only frames 10 through 20\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :evisc-tuple (<see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> 3 4 nil nil))\u000A                             ; print with print-level 3 and print-length 4\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :evisc-tuple nil) ; print using default ``evisceration'',\u000A                             ;   essentially the same as just above\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :evisc-tuple '(nil 3 4 (<see topic=\"ACL2____HIDE\">hide</see>)))\u000A                             ; same as just above\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____CW-GSTACK\">cw-gstack</see> :frames frames :evisc-tuple evisc-tuple)</code> \u000A \u000A <p>where <v>:frames</v> and <v>:evisc-tuple</v> are optional, but if they are \u000A supplied, their values are evaluated.  The value of <v>frames</v> should be \u000A either a natural number or a list of two natural numbers, the first less than \u000A the second; and the value of <v>evisc-tuple</v> should be an evisc-tuple (see \u000A <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>).  If <v>:evisc-tuple</v> is omitted, then substructures \u000A deeper than 3 are replaced by ``<v>#</v>'' and those longer than 4 are replaced \u000A by ``<v>...</v>'', and terms of the form <v>(<see topic=\"ACL2____HIDE\">hide</see> ...)</v> are printed as \u000A <v>&lt;hidden&gt;</v>.  Also see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see> for an alternative to printing \u000A ``<v>#</v>'' and ``<v>...</v>''.</p> \u000A \u000A <p>Stack overflows may occur, perhaps caused by looping rewrite rules.  In \u000A some Lisps, stack overflows may manifest themselves as segmentation faults, \u000A causing the entire ACL2 image to crash.  Finding looping rewrite rules can be \u000A tricky, especially if you are using books supplied by other people.  (However, \u000A see <see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see> for a way to avoid stack overflows caused \u000A by rewriter loops.)</p> \u000A \u000A <p>Normally, a stack overflow will cause the printing of an error message that \u000A suggests how to proceed.  Just follow those instructions, and you will \u000A generally be able to see what is causing the loop.</p> \u000A \u000A <p>Suggestion: Once you have found the loop and fixed it, you should execute \u000A the ACL2 command <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt><v> nil</v>, so that you don't slow down \u000A subsequent proof attempts.</p>"],

 "ACL2____ACL2-USER":[["Programming"],"ACL2 Sources","ACL2","<p>This package imports the standard Common Lisp symbols that ACL2 \u000A supports and also a few symbols from package <v>&quot;ACL2&quot;</v> that are commonly \u000A used when interacting with ACL2.  You may prefer to select this as your \u000A current package so as to avoid colliding with ACL2 system names.</p> \u000A \u000A <p>This package imports the symbols listed in \u000A <v>*common-lisp-symbols-from-main-lisp-package*</v>, which contains hundreds of \u000A CLTL function and macro names including those supported by ACL2 such as <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>, and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.  It also imports the symbols in \u000A <v>*acl2-exports*</v>, which contains a few symbols that are frequently used \u000A while interacting with the ACL2 system, such as <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, and <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>.  It imports nothing else.</p> \u000A \u000A <p>Thus, names such as <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>, <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, and <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt>, which are defined in the <v>&quot;ACL2&quot;</v> package are not present \u000A here.  If you find yourself frequently colliding with names that are defined \u000A in <v>&quot;ACL2&quot;</v> you might consider selecting <v>&quot;ACL2-USER&quot;</v> as your \u000A current package (see <see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see>).  If you select <v>&quot;ACL2-USER&quot;</v> as \u000A the current package, you may then simply type <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt> to refer to \u000A <v>acl2-user::member-equal</v>, which you may define as you see fit.  Of course, \u000A should you desire to refer to the <v>&quot;ACL2&quot;</v> version of <tt><see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see></tt>, you will have to use the <v>&quot;ACL2::&quot;</v> prefix, e.g., \u000A <v>acl2::member-equal</v>.</p> \u000A \u000A <p>If, while using <v>&quot;ACL2-USER&quot;</v> as the current package, you find that \u000A there are symbols from <v>&quot;ACL2&quot;</v> that you wish we had imported into it \u000A (because they are frequently used in interaction), please bring those symbols \u000A to our attention.  For example, should <tt><see topic=\"ACL2____UNION-THEORIES\">union-theories</see></tt> and <tt><see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see></tt> be imported?  Except for stabilizing on the ``frequently \u000A used'' names from <v>&quot;ACL2&quot;</v>, we intend never to define a symbol whose \u000A <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> is <v>&quot;ACL2-USER&quot;</v>.</p>"],

 "COMMON-LISP____THE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____THE\">The</see> typ val)</v> checks that <v>val</v> satisfies the type \u000A specification <v>typ</v> (see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>).  An error is caused if the \u000A check fails, and otherwise, <v>val</v> is the value of this expression.  Here \u000A are some examples.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____THE\">the</see> integer 3)       ; returns 3\u000A(<see topic=\"COMMON-LISP____THE\">the</see> (integer 0 6) 3) ; returns 3\u000A(<see topic=\"COMMON-LISP____THE\">the</see> (integer 0 6) 7) ; causes an error (see below for exception)</code> \u000A \u000A <p>See <see topic=\"ACL2____TYPE-SPEC\">type-spec</see> for a discussion of the legal type specifications.</p> \u000A \u000A <p>There is an exception to the rule that failure of the type-check causes an \u000A error: there is no error when <see topic=\"ACL2____GUARD\">guard</see>-checking has been turned off with \u000A <v>:set-guard-checking :NONE</v> or <v>(<see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> :NONE ...)</v>.  See \u000A <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> and see <see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see>.</p> \u000A \u000A <p>The following remark is for those who verify guards for their functions \u000A (see <see topic=\"ACL2____GUARD\">guard</see> and see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>).  We remark that a call of \u000A <v>(<see topic=\"COMMON-LISP____THE\">the</see> TYPE EXPR)</v> in the body of a function definition generates a guard \u000A proof obligation that the type, <v>TYPE</v>, holds for the value of the \u000A expression, <v>EXPR</v>.  Consider the following example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> f (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (p1 x)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (p2 x)\u000A      (<see topic=\"COMMON-LISP____THE\">the</see> integer x)\u000A    17))</code> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> proof obligation generated for the <v>THE</v> expression \u000A above is as follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p1 x) (p2 x))\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((var x)) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> var)))</code> \u000A \u000A <p><v>THE</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p>"],

 "ACL2____WITH-LOCAL-STATE":[["Stobj"],"ACL2 Sources","ACL2","<p>This is an advanced topic, probably of interest only to system \u000A developers.</p> \u000A \u000A <p>Consider the following example form:</p> \u000A \u000A <code>(<see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see>\u000A (<see topic=\"ACL2____MV-LET\">mv-let</see> (result state)\u000A         (compute-with-state x state)\u000A         result))</code> \u000A \u000A <p>This is equivalent to the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see>\u000A state\u000A (<see topic=\"ACL2____MV-LET\">mv-let</see> (result state)\u000A         (compute-with-state x state)\u000A         result))</code> \u000A \u000A <p>By default, this form is illegal, because ACL2 does not have a way to \u000A unwind all changes to the ACL2 <see topic=\"ACL2____STATE\">state</see>; we say more on this issue below. \u000A There may however be situations where you are willing to manage or overlook \u000A this issue.  In that case you may execute the following form to enable the use \u000A of <v>with-local-state</v>, by enabling the use of <tt><see topic=\"ACL2____WITH-LOCAL-STOBJ\">with-local-stobj</see></tt> on \u000A <v>state</v>; but note that it requires an active trust tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> create-state t)</code> \u000A \u000A <p>Please be aware that no local <see topic=\"ACL2____STATE\">state</see> is actually created, however! \u000A In particular, users of <v>with-local-state</v> need either to ensure that \u000A channels are closed and state global variables are returned to their original \u000A values, or else be willing to live with changes made to state that are not \u000A justified by the code that has been evaluated.  You are welcome to look in the \u000A the ACL2 source code at the definition of macro <v>channel-to-string</v>, which \u000A employs <v>with-local-state</v> to create a local <see topic=\"ACL2____STATE\">state</see> for the purpose \u000A of creating a string.</p> \u000A \u000A <p>Here is an example use of <v>with-local-state</v>.  Notice the use of <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> &mdash; and indeed, please understand that we are just hacking here, \u000A and in general it takes significant effort to be sure that one is using \u000A <v>with-local-state</v> correctly!</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A\u000A(<see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> create-state t)\u000A\u000A(<see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see> t)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A   (channel state)\u000A   (<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> &quot;my-file&quot; :object state)\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (eofp obj state)\u000A           (<see topic=\"ACL2____READ-OBJECT\">read-object</see> channel state)\u000A           (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> eofp))\u000A           (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____CLOSE-INPUT-CHANNEL\">close-input-channel</see> channel state)))\u000A             (<see topic=\"ACL2____MV\">mv</see> obj state)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see> (<see topic=\"ACL2____MV-LET\">mv-let</see> (result state)\u000A                            (foo state)\u000A                            result)))\u000A\u000A; Multiple-value return version:\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo2 (<see topic=\"ACL2____STATE\">state</see>)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A   (channel state)\u000A   (<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> &quot;my-file&quot; :object state)\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (eofp obj state)\u000A           (<see topic=\"ACL2____READ-OBJECT\">read-object</see> channel state)\u000A           (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____CLOSE-INPUT-CHANNEL\">close-input-channel</see> channel state)))\u000A             (<see topic=\"ACL2____MV\">mv</see> eofp obj state)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bar2 ()\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A  (<see topic=\"ACL2____WITH-LOCAL-STATE\">with-local-state</see> (<see topic=\"ACL2____MV-LET\">mv-let</see> (eofp result state)\u000A                            (foo2 state)\u000A                            (<see topic=\"ACL2____MV\">mv</see> eofp result))))</code>"],

 "ACL2____VALUE-TRIPLE":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4))\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2____CW\">cw</see> &quot;hi&quot;) :on-skip-proofs t)\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2_____04\">@</see> ld-pre-eval-print))\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> (<see topic=\"ACL2_____04\">@</see> ld-pre-eval-print) :check t)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> form\u000A              :on-skip-proofs sp ; optional; nil by default\u000A              :check chk         ; optional; nil by default\u000A              )</code> \u000A \u000A <p><v>Value-triple</v> provides a convenient way to evaluate a form in an event \u000A context, including <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> and in <see topic=\"ACL2____BOOKS\">books</see>; \u000A see <see topic=\"ACL2____EVENTS\">events</see>.  The form should evaluate to a single, non-<see topic=\"ACL2____STOBJ\">stobj</see> \u000A value.</p> \u000A \u000A <p>Calls of <v>value-triple</v> are generally skipped when proofs are being \u000A skipped, in particular when ACL2 is performing the second pass through the \u000A <see topic=\"ACL2____EVENTS\">events</see> of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form or during an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>, or indeed any time <tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> is non-<v>nil</v>.  If \u000A you want the call evaluated during those times as well, use a non-<v>nil</v> \u000A value for <v>:on-skip-proofs</v>.  Note that the argument to \u000A <v>:on-skip-proofs</v> is not evaluated.</p> \u000A \u000A <p>If you expect the form to evaluate to a non-<v>nil</v> value and you want an \u000A error to occur when that is not the case, you can use <v>:check t</v>.  More \u000A generally, the argument of <v>:check</v> can be a form that evaluates to a \u000A single, non-<see topic=\"ACL2____STOBJ\">stobj</see> value.  If this value is not <v>nil</v>, then the \u000A aforementioned test is made (that the given form is not <v>nil</v>).  If an \u000A error occurs and the value of <v>:check</v> is a string or indeed any \u000A ``message'' suitable for printing by <tt><see topic=\"ACL2____FMT\">fmt</see></tt> when supplied as a value for \u000A tilde-directive <v>~@</v>, then that string or message is printed.</p>"],

 "ACL2____The_02Tours":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>ACL2 is a very large, multipurpose system.  You can use it as a \u000A programming language, a specification language, a modeling language, a formal \u000A mathematical logic, or a semi-automatic theorem prover, just to name its most \u000A common uses.  It has been used on a number of <see topic='ACL2____INTERESTING-APPLICATIONS'>industrial applications</see>.  If you're uncertain \u000A as to whether your project is appropriate for ACL2 we urge you to look over \u000A this list or contact the ACL2 developers.</p> \u000A \u000A <p>This home page includes all of ACL2's online documentation, which is quite \u000A extensive (over 4 megabytes).  To help ease your introduction to ACL2, we have \u000A built two tours through this documentation.</p> \u000A \u000A <p>If you are familiar with at least some of the <see topic='ACL2____INTERESTING-APPLICATIONS'>industrial applications</see> of ACL2, then you \u000A will understand the distance between the simple examples we talk about in \u000A these tours and the kinds of things ACL2 users do with the system.</p> \u000A \u000A <p>Newcomers to ACL2 should first take the ``Flying Tour.''  Then, if you want \u000A to know more, take the ``Walking Tour.''  On your first reading, follow the \u000A two Tours linearly, clicking only on the icon of the Tour you're on.  Beware \u000A of other links, which might jump you from one tour to the other or into the \u000A ACL2 User's Manual!  Once you've had a coherent overview of the system, you \u000A might quickly repeat both Tours to see if there are unvisited links you're \u000A interested in, using your brower's Back Button to return to your starting \u000A points.</p> \u000A \u000A <p>If after all this you want to learn how to use the theorem prover (!), see \u000A <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p> \u000A \u000A <p>To start a tour, click on the appropriate icon below.</p> \u000A \u000A <p><see topic='ACL2____A_02Flying_02Tour_02of_02ACL2'><img \u000A src='flying.gif'></img></see> <see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'><img src='walking.gif'></img></see></p> \u000A \u000A <p>If you take the tours in a text-based format (such as using the :DOC \u000A command in Emacs), they will probably be unsatisfying because we use gif files \u000A and assume you can navigate ``back.''</p>"],

 "ACL2____STANDARD-CO":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Standard-co</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____STANDARD-CO\">standard-co</see> state)</v> and the updater is <v>(set-standard-co\u000A val state)</v>.  <v>Standard-co</v> must be an open character output channel.  It \u000A is to this channel that <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the <see topic=\"ACL2____PROMPT\">prompt</see>, the form to be \u000A evaluated, and the results.  The event <see topic=\"ACL2____COMMAND\">command</see>s such as <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, etc., which print extensive commentary do not print to \u000A <v>standard-co</v> but rather to a different channel, <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt>, so that \u000A you may redirect this commentary while still interacting via <v>standard-co</v>. \u000A See <see topic=\"ACL2____PROOFS-CO\">proofs-co</see>.</p> \u000A \u000A <p>``Standard-co'' stands for ``standard character output.'' The initial value \u000A of <v>standard-co</v> is the same as the value of <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt> (see \u000A <see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see>).</p>"],

 "ACL2____RESET-PREHISTORY":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see>)     ; restart command numbering at 0\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> nil) ; same as above\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> t)   ; as above except also disable ubt-prehistory\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see>)\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> permanent-p)\u000A(<see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see> permanent-p doc-string)</code> \u000A \u000A <p>where <v>permanent-p</v> is <v>t</v> or <v>nil</v>, and <v>doc-string</v> is an \u000A optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string not beginning with ``<v>:Doc-Section</v> \u000A ...''.  After execution of this command, ACL2 will change the numbering \u000A provided by its <see topic=\"ACL2____HISTORY\">history</see> utilities so that this <v>reset-prehistory</v> \u000A command (or the top-level compound <see topic=\"ACL2____COMMAND\">command</see> containing it, which for \u000A example might be an <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>) is assigned the number 0.  The only \u000A way to undo this command is with command <tt><see topic=\"ACL2____UBT-PREHISTORY\">ubt-prehistory</see></tt>.  However, \u000A even that is disallowed if <v>permanent-p</v> is <v>t</v>.</p> \u000A \u000A <p>Note that the second argument of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, which specifies the \u000A number of commands in the certification world (i.e., since ground-zero), is \u000A not sensitive to <v>reset-prehistory</v>; rather, it expects the number of \u000A commands since ground-zero.  To see such commands, <v>:</v><tt><see topic=\"ACL2____PBT\">pbt</see></tt><v>\u000A :start</v>.</p> \u000A \u000A <p>A <v>reset-prehistory</v> event for which parameter <v>permanent-p</v> has the \u000A default value of <v>nil</v> is always skipped when any of the following \u000A conditions holds: during <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>; during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> or \u000A the second pass of <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> (indeed, whenever <see topic=\"ACL2____LD\">ld</see> special \u000A <tt>'</tt><tt><see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see></tt> has value <v>'include-book</v>); or when \u000A <v>state</v> global <v>'skip-reset-prehistory</v> has a non-<v>nil</v> value.  Thus, \u000A we avoid resetting the history numbering to 0 when including or certifying a \u000A book, since that would probably not be what was intended.</p> \u000A \u000A <p>See <see topic=\"ACL2____UBT-PREHISTORY\">ubt-prehistory</see> for how to undo a <v>reset-prehistory</v> command \u000A that does not have a <v>permanent-p</v> of <v>t</v>.</p>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-5":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (FOO A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (BAR B) (MUM C)))</code> \u000A \u000A <p>When is <v>x</v> a subset of <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> y z)</v>?  Clearly it is if <v>x</v> is a \u000A subset of <v>y</v> or <v>x</v> is a subset of <v>z</v>.  We could write that:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-2-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y)\u000A               (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x z))\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))))</code> \u000A \u000A <p>The rule generated from this is: ``if you ever encounter (an instance of) \u000A <v>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> x (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> y z))</v>, backchain to the <v>or</v> above and try to \u000A establish it.  If you can establish it, replace the target by <v>T</v>.''</p> \u000A \u000A <p>This does not fully characterize the situation though.  For example, <v>'(1\u000A 2 3 4)</v> is a subset of <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> '(1 3) '(2 4))</v> without being a subset of \u000A either argument of the <v>append</v>.</p> \u000A \u000A <p>However, no obvious equivalence comes to mind -- indeed, to express any of \u000A the ideas floating around here requires defining and introducing more \u000A functions, which is not recommended unless those functions are already in the \u000A problem.</p> \u000A \u000A <p>For example, if you defined the concept of ``<v>set-minus</v>'' so that \u000A <v>(set-minus x y)</v> consists of those elements of <v>x</v> not in <v>y</v>, then \u000A you could prove:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subset-append-2-strong-but-messy\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (set-minus x z) y)\u000A              (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (set-minus x y) z))))</code> \u000A \u000A <p>But this rewrite rule would ``trade'' <v>append</v> away and introduce \u000A <v>set-minus</v>.  That might be a good strategy if <v>set-minus</v> were already \u000A in the problem.  But if it were not, it might not be.  We wouldn't recommend \u000A this rule unless it were helpful in normalizing the expressions in the \u000A problem.</p> \u000A \u000A <p>We recommend sticking with the weak version of the rule,</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-2-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y)\u000A               (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x z))\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))).</code> \u000A \u000A <p>This illustrates the fact that sometimes there is no strong version of a \u000A rule!</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Start in a fresh ACL2, either by restarting your ACL2 image from \u000A scratch or executing <v>:ubt! 1</v>.</p> \u000A \u000A <p>This problem is much more open ended than the preceding ones.  The \u000A challenge is to define a function that collects exactly one copy of each \u000A element of a list and to prove that it returns a subset of the list with no \u000A duplications.</p> \u000A \u000A <p><b>Hint</b>: We recommend that you read this hint to align your function \u000A names with our solution, to make comparisons easier.  Our answer is shown in \u000A see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4-ANSWER\">introductory-challenge-problem-4-answer</see>.  In that page you'll see a \u000A definition of a function <v>collect-once</v> and the proofs of two theorems:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-1-about-collect-once\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (collect-once x) x))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main-theorem-2-about-collect-once\u000A  (<see topic=\"COMMON-LISP____NOT\">not</see> (dupsp (collect-once x))))</code> \u000A \u000A <p>The function <v>dupsp</v> is as defined in see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3\">introductory-challenge-problem-3</see>.  This is quite easy.</p> \u000A \u000A <p>Then, we define a tail-recursive version of the method based on the \u000A pseudo-code:</p> \u000A \u000A <code>a = nil;\u000Awhile (x not empty) {\u000A a = if (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a) then a else (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) a);\u000A x = (<see topic=\"COMMON-LISP____CDR\">cdr</see> x);\u000A }\u000Areturn a;</code> \u000A \u000A <p>We formalize this with the function <v>while-loop-version</v>, where \u000A <v>(while-loop-version x nil)</v> is the ``semantics'' of the code above.  I.e., \u000A the function <v>while-loop-version</v> captures the while loop in the \u000A pseudo-code above and returns the final value of <v>a</v>, and it should be \u000A invoked with the initial value of <v>a</v> being <v>nil</v>.</p> \u000A \u000A <p>We prove <v>(while-loop-version x nil)</v> returns a subset of <v>x</v> that \u000A contains no duplications.  Furthermore, we do it two ways: first \u000A ``indirectly'' by relating <v>while-loop-version</v> to <v>collect-once</v>, and \u000A second (``directly'') without using <v>collect-once</v>.  Both of these proofs \u000A are much harder than the <v>collect-once</v> approach, involving about a dozen \u000A lemmas each.</p> \u000A \u000A <p>Compare your solutions to ours at see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4-ANSWER\">introductory-challenge-problem-4-answer</see>.</p> \u000A \u000A <p>Then, use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see>.</p>"],

 "ACL2____FLUSH-COMPRESS":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____FLUSH-COMPRESS\">flush-compress</see> 'my-array)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____FLUSH-COMPRESS\">flush-compress</see> name)</code> \u000A \u000A <p>where <v>name</v> is a symbol.</p> \u000A \u000A <p>Recall that <v>(<see topic=\"ACL2____COMPRESS1\">compress1</see> nm alist)</v> associates an under-the-hood raw Lisp \u000A one-dimensional array of name <v>nm</v> with the given association list, \u000A <v>alist</v>, while <v>(<see topic=\"ACL2____COMPRESS2\">compress2</see> nm alist)</v> is the analogous function for \u000A two-dimensional arrays; see <see topic=\"ACL2____COMPRESS1\">compress1</see> and see <see topic=\"ACL2____COMPRESS2\">compress2</see>.  The \u000A only purpose of <v>flush-compress</v>, which always returns <v>nil</v>, is to \u000A remove the association of any under-the-hood array with the given name, thus \u000A eliminating slow array accesses (see <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>).  It is not \u000A necessary if the return values of <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> and <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> are \u000A always used as the ``current'' copy of the named array, and thus \u000A <v>flush-compress</v> should rarely, if ever, be needed in user \u000A applications.</p> \u000A \u000A <p>Nevertheless, we provide the following contrived example to show how \u000A <v>flush-compress</v> can be used to good effect.  Comments have been added to \u000A this log to provide explanation.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> a (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                            '((:header :dimensions (5)\u000A                                       :maximum-length 15\u000A                                       :default uninitialized\u000A                                       :name demo)\u000A                              (0 . zero)\u000A                              (1 . one))))\u000A ((:HEADER :DIMENSIONS (5)\u000A           :MAXIMUM-LENGTH\u000A           15 :DEFAULT UNINITIALIZED :NAME DEMO)\u000A  (0 . ZERO)\u000A  (1 . ONE))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)\u000AZERO\u000A; As expected, the above evaluation did not cause a slow array warning.  Now\u000A; we associate a different under-the-hood array with the name 'demo.\u000AACL2 !&gt;(<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                  '((:header :dimensions (5)\u000A                             :maximum-length 15\u000A                             :default uninitialized\u000A                             :name demo)\u000A                    (0 . zero)))\u000A((:HEADER :DIMENSIONS (5)\u000A          :MAXIMUM-LENGTH\u000A          15 :DEFAULT UNINITIALIZED :NAME DEMO)\u000A (0 . ZERO))\u000A; The following array access produces a slow array warning because (<see topic=\"ACL2_____04\">@</see> a) is\u000A; no longer associated under-the-hood with the array name 'demo.\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)\u000A\u000A**********************************************************\u000ASlow Array Access!  A call of AREF1 on an array named\u000ADEMO is being executed slowly.  See :DOC slow-array-warning\u000A**********************************************************\u000A\u000AZERO\u000A; Now we associate under-the-hood, with array name 'demo, an alist equal to\u000A; (<see topic=\"ACL2_____04\">@</see> a).\u000AACL2 !&gt;(<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo\u000A                  '((:header :dimensions (5)\u000A                             :maximum-length 15\u000A                             :default uninitialized\u000A                             :name demo)\u000A                    (0 . zero)\u000A                    (1 . one)))\u000A((:HEADER :DIMENSIONS (5)\u000A          :MAXIMUM-LENGTH\u000A          15 :DEFAULT UNINITIALIZED :NAME DEMO)\u000A (0 . ZERO)\u000A (1 . ONE))\u000A; The following array access is still slow, because the under-the-hood array\u000A; is merely associated with a copy of (<see topic=\"ACL2_____04\">@</see> a), not with the actual object\u000A; (<see topic=\"ACL2_____04\">@</see> a).\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)\u000A\u000A**********************************************************\u000ASlow Array Access!  A call of AREF1 on an array named\u000ADEMO is being executed slowly.  See :DOC slow-array-warning\u000A**********************************************************\u000A\u000AZERO\u000A; So we might try to fix the problem by recompressing. But this doesn't\u000A; work.  It would work, by the way, if we re-assign a:\u000A; (<see topic=\"ACL2____ASSIGN\">assign</see> a (<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a))).  That is why we usually will not need\u000A; flush-compress.\u000AACL2 !&gt;(<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a))\u000A((:HEADER :DIMENSIONS (5)\u000A          :MAXIMUM-LENGTH\u000A          15 :DEFAULT UNINITIALIZED :NAME DEMO)\u000A (0 . ZERO)\u000A (1 . ONE))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)\u000A\u000A**********************************************************\u000ASlow Array Access!  A call of AREF1 on an array named\u000ADEMO is being executed slowly.  See :DOC slow-array-warning\u000A**********************************************************\u000A\u000AZERO\u000A; Finally, we eliminate the warning by calling flush-compress before we call\u000A; compress1.  The call of flush-compress removes any under-the-hood\u000A; association of an array with the name 'demo.  Then the subsequent call of\u000A; compress1 associates the object (<see topic=\"ACL2_____04\">@</see> a) with that name.  (Technical point:\u000A; compress1 always associates the indicated name with the value that it\u000A; returns.  in this case, what compress1 returns is (<see topic=\"ACL2_____04\">@</see> a), because (<see topic=\"ACL2_____04\">@</see> a) is\u000A; already, logically speaking, a compressed array1p (starts with a :header\u000A; and the natural number keys are ordered).\u000AACL2 !&gt;(<see topic=\"ACL2____FLUSH-COMPRESS\">flush-compress</see> 'demo)\u000ANIL\u000AACL2 !&gt;(<see topic=\"ACL2____COMPRESS1\">compress1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a))\u000A((:HEADER :DIMENSIONS (5)\u000A          :MAXIMUM-LENGTH\u000A          15 :DEFAULT UNINITIALIZED :NAME DEMO)\u000A (0 . ZERO)\u000A (1 . ONE))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'demo (<see topic=\"ACL2_____04\">@</see> a) 0)\u000AZERO\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Function: </b><srclink>flush-compress</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> flush-compress (<see topic=\"ACL2____NAME\">name</see>)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> name))\u000A       nil)</code>"],

 "ACL2____About_02Types":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The universe of ACL2 objects includes objects of many different \u000A types.  For example, <v>t</v> is a ``symbol'' and 3 is an ``integer.''  Roughly \u000A speaking the objects of ACL2 can be partitioned into the following types:</p> \u000A \u000A <code> <see topic='ACL2____Numbers_02in_02ACL2'>Numbers</see> <v>3, -22/7, #c(3\u000A5/2)</v> <see topic='ACL2____ACL2_02Characters'>Characters</see> <v>#\\A, #\\a,\u000A#\\Space</v> <see topic='ACL2____ACL2_02Strings'>Strings</see> <v>&quot;This is a\u000Astring.&quot;</v> <see topic='ACL2____ACL2_02Symbols'>Symbols</see> <v>'abc,\u000A'smith::abc</v> <see topic='ACL2____ACL2_02Conses_02or_02Ordered_02Pairs'>Conses (or \u000AOrdered Pairs)</see> <v>'((a . 1) (b . 2))</v> </code> \u000A \u000A <p>When proving theorems it is important to know the types of object returned \u000A by a term.  ACL2 uses a complicated heuristic algorithm, called <tt><see topic=\"ACL2____TYPE-SET\">type-set</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, to determine what types of objects a term may \u000A produce.  The user can more or less program the <v>type-set</v> algorithm by \u000A proving <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rules.</p> \u000A \u000A <p>ACL2 is an ``untyped'' logic in the sense that the syntax is not typed: It \u000A is legal to apply a function symbol of n arguments to any n terms, regardless \u000A of the types of the argument terms.  Thus, it is permitted to write such odd \u000A expressions as <v>(<see topic=\"COMMON-LISP_____B2\">+</see> t 3)</v> which sums the symbol <v>t</v> and the integer 3. \u000A Common Lisp does not prohibit such expressions.  We like untyped languages \u000A because they are simple to describe, though proving theorems about them can be \u000A awkward because, unless one is careful in the way one defines or states \u000A things, unusual cases (like <v>(<see topic=\"COMMON-LISP_____B2\">+</see> t 3)</v>) can arise.</p> \u000A \u000A <p>To make theorem proving easier in ACL2, the axioms actually define a value \u000A for such terms.  The value of <v>(<see topic=\"COMMON-LISP_____B2\">+</see> t 3)</v> is 3; under the ACL2 axioms, \u000A non-numeric arguments to <v>+</v> are treated as though they were 0.</p> \u000A \u000A <p>You might immediately wonder about our claim that ACL2 is Common Lisp, \u000A since <v>(<see topic=\"COMMON-LISP_____B2\">+</see> t 3)</v> is ``an error'' (and will sometimes even ``signal an \u000A error'') in Common Lisp.  It is to handle this problem that ACL2 has \u000A <b>guards</b>.  We will discuss guards later in the Walking Tour.  However, \u000A many new users simply ignore the issue of guards entirely and that is what we \u000A recommend for now.</p> \u000A \u000A <p>You should now return to <see topic='ACL2____Revisiting_02the_02Admission_02of_02App'>the Walking Tour</see>.</p>"],

 "ACL2____ACL2":[["Top"],"ACL2 Sources","ACL2","<p>This is the ACL2 documentation.  For the ACL2+Books Manual, which \u000A  that includes both the ACL2 documentation and the ACL2 <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>, see the <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a>.</p>"],

 "COMMON-LISP_____A2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>*</v> is really a macro that expands to calls of the function \u000A <tt><see topic=\"ACL2____BINARY-_A2\">binary-*</see></tt>.  So for example</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____A2\">*</see> x y 4 z)</code> \u000A \u000A <p>represents the same term as</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_A2\">binary-*</see> x (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> y (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> 4 z))).</code> \u000A \u000A <p>See <see topic=\"ACL2____BINARY-_A2\">binary-*</see>.</p> \u000A \u000A <p><v>*</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p>"],

 "COMMON-LISP____ASSOC":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist)\u000A(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____ASSOC\">Assoc</see> x alist)</v> is the first member of <v>alist</v> whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> is \u000A <v>x</v>, or <v>nil</v> if no such member exists.  The optional keyword, \u000A <v>:TEST</v>, has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>s of successive elements \u000A of <v>alist</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>assoc</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> (see \u000A <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see></tt>. \u000A If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument must be a symbol or \u000A the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>assoc</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> x alist)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist\u000A :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> x alist)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>assoc-equal</v>.</p> \u000A \u000A <p><v>Assoc</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>assoc-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> assoc-equal (x alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> alist) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)))\u000A              (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A             (t (<see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)))))</code>"],

 "COMMON-LISP____-":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BINARY-_B2\">binary-+</see> for addition and see <see topic=\"ACL2____UNARY--\">unary--</see> for \u000A negation.</p> \u000A \u000A <p>Note that <v>-</v> represents subtraction as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____-\">-</see> x y)</code> \u000A \u000A <p>represents the same term as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP____-\">-</see> y))</code> \u000A \u000A <p>which is really</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x (<see topic=\"ACL2____UNARY--\">unary--</see> y)).</code> \u000A \u000A <p>Also note that <v>-</v> represents arithmetic negation as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____-\">-</see> x)</code> \u000A \u000A <p>expands to</p> \u000A \u000A <code>(<see topic=\"ACL2____UNARY--\">unary--</see> x).</code> \u000A \u000A <p><b>Macro: </b><srclink>-</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>\u000A     - (x &amp;optional (y 'nil binary-casep))\u000A     (<see topic=\"COMMON-LISP____IF\">if</see> binary-casep\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((y (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> y)\u000A                           (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> y) 'quote)\u000A                           (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))\u000A                           (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)))\u000A                           (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)) nil))\u000A                      (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> y))\u000A                      y)))\u000A              (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y)\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> 'binary-+\u000A                        (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____UNARY--\">unary--</see> y) (<see topic=\"COMMON-LISP____CONS\">cons</see> x nil)))\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> 'binary-+\u000A                        (<see topic=\"COMMON-LISP____CONS\">cons</see> x\u000A                              (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'unary-- (<see topic=\"COMMON-LISP____CONS\">cons</see> y nil))\u000A                                    nil)))))\u000A         (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A                           (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) 'quote)\u000A                           (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A                           (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A                           (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) nil))\u000A                      (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A                      x)))\u000A              (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                  (<see topic=\"ACL2____UNARY--\">unary--</see> x)\u000A                  (<see topic=\"COMMON-LISP____CONS\">cons</see> 'unary-- (<see topic=\"COMMON-LISP____CONS\">cons</see> x nil))))))</code>"],

 "ACL2____SET-PARALLEL-EXECUTION":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see> for an introduction to parallel \u000A execution in ACL2.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see> nil) ; default for images not built for parallelism\u000A(<see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see> t)   ; default for images built for parallelism\u000A(<see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see> :bogus-parallelism-ok)</code> \u000A \u000A <p><v>Set-parallel-execution</v> takes an argument that specifies the enabling \u000A or disabling of <see topic=\"ACL2____PARALLEL\">parallel</see> execution for the primitives <tt><see topic=\"ACL2____PAND\">pand</see></tt>, \u000A <tt><see topic=\"ACL2____POR\">por</see></tt>, <tt><see topic=\"ACL2____PLET\">plet</see></tt>, and <tt><see topic=\"ACL2____PARGS\">pargs</see></tt> (but not <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt>, \u000A whose parallel execution remains enabled).  However, without using <tt><see topic=\"ACL2____TOP-LEVEL\">top-level</see></tt>, calls of parallelism primitives made explicitly in the ACL2 \u000A top-level loop, as opposed to inside function bodies, will never cause \u000A parallel execution; see <see topic=\"ACL2____PARALLELISM-AT-THE-TOP-LEVEL\">parallelism-at-the-top-level</see>.  Parallel \u000A execution is determined by the value of the argument to \u000A <v>set-parallel-execution</v>, as follows.</p> \u000A \u000A <p>Value <v>t</v>:<br></br> \u000A \u000A All parallelism primitives used in bodies of function definitions are given \u000A the opportunity to execute in parallel.  However, the use of parallelism \u000A primitives directly in the ACL2 top-level loop causes an error.</p> \u000A \u000A <p>Value <v>:bogus-parallelism-ok</v>:<br></br> \u000A \u000A Parallel execution is enabled, as for value <v>t</v>.  However, the use of \u000A parallelism primitives directly in the ACL2 top-level loop does not cause an \u000A error, but rather, simply results in serial execution for these \u000A primitives.</p> \u000A \u000A <p>Value <v>nil</v>:<br></br> \u000A \u000A All parallelism primitives degrade to their serial equivalents, including \u000A their calls made directly in the ACL2 top-level loop.  Thus, uses of \u000A parallelism primitives do not in themselves cause errors.</p>"],

 "ACL2____SET-MATCH-FREE-ERROR":[["Free-variables"],"ACL2 Sources","ACL2","<code>Legal Forms:\u000A(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> nil)\u000A:set-match-free-error nil\u000A(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> t)\u000A:set-match-free-error t</code> \u000A \u000A <p>As described elsewhere (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), when a <see topic=\"ACL2____REWRITE\">rewrite</see>, \u000A <see topic=\"ACL2____LINEAR\">linear</see>, or <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rule has free variables in its \u000A hypotheses, the user can specify whether to try all bindings (``<v>:all</v>'') \u000A or just the first (``<v>:once</v>'') when relieving its hypotheses, as a basis \u000A for relieving subsequent hypotheses.  This direction of <v>:all</v> or \u000A <v>:once</v> is generally provided by specifying either <i>:match-free :once</i> \u000A or <i>:match-free :all</i> in the <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> of the rule.</p> \u000A \u000A <p>But suppose that neither of these is specified for such a rule.  (Note: \u000A <v>set-match-free-error</v> is not relevant for <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules.) \u000A Also suppose that <v>set-match-free-default</v> has not specified a default of \u000A <v>:once</v> or <v>:all</v> (see <see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see>).  In this case a \u000A warning will occur except when in the context of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  If you \u000A prefer to see an error in such cases, except in the context of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, execute <v>(<see topic=\"ACL2____SET-MATCH-FREE-ERROR\">set-match-free-error</see> t)</v>.  If there is no error, \u000A then a default of <v>:all</v> is used.</p> \u000A \u000A <p>Note: This is NOT an event!  Instead, <v>set-match-free-error</v> sets the \u000A state global <v>'match-free-error</v> (see <see topic=\"ACL2____STATE\">state</see> and see <see topic=\"ACL2____ASSIGN\">assign</see>). \u000A Thus, this form cannot be put into a book.  If you are tempted to put it into \u000A a book, consider the fact that it really isn't needed there, since the absence \u000A of <v>:match-free</v> does not cause an error in the context of <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> or <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.  If you still feel the need for such a \u000A form, consider using <v>set-match-free-default</v> to provide a default, at \u000A least within the scope of the current book (if any); see <see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see>.</p>"],

 "ACL2____PROFILE":[["Events"],"ACL2 Sources","ACL2","<p>NOTE: An alternative to this utility, which has a lot less \u000A functionality but doesn't depend on the experimental extension of ACL2 \u000A mentioned just below, may be found in community book \u000A <v>books/misc/profiling.lisp</v>.</p> \u000A \u000A <p>This documentation topic relates to an experimental extension of ACL2 under \u000A development by Bob Boyer and Warren Hunt.  See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see> for \u000A a general discussion of memoization, on which this <v>profile</v> utility is \u000A built, and the related features of hash consing and applicative hash \u000A tables.</p> \u000A \u000A <p><v>Profile</v> can be useful in Common Lisp debugging and performance \u000A analysis, including examining the behavior of ACL2 functions.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____PROFILE\">profile</see> 'fn)       ; keep count of the calls of fn\u000A(<see topic=\"ACL2____PROFILE\">profile</see> 'fn        ; as above, with with some memoize options\u000A         :trace t\u000A         :forget t)\u000A(<see topic=\"ACL2____MEMSUM\">memsum</see>) ; report statistics on calls of memoized functions (e.g., fn)\u000A(<see topic=\"ACL2____CLEAR-MEMOIZE-STATISTICS\">clear-memoize-statistics</see>) ; clear memoization (<see topic=\"COMMON-LISP____AND\">and</see> profiling) statistics</code> \u000A \u000A <p>Evaluation of <v>(<see topic=\"ACL2____PROFILE\">profile</see> 'fn)</v> redefines <v>fn</v> so that a count will be \u000A kept of the calls of FN.  The information recorded may be displayed, for \u000A example, by invoking <v>(</v><tt><see topic=\"ACL2____MEMOIZE-SUMMARY\">memoize-summary</see></tt><v>)</v> or (equivalently) \u000A <v>(<see topic=\"ACL2____MEMSUM\">memsum</see>)</v>.  If you wish to gather fresh statistics for the evaluation of a \u000A top-level form, see <see topic=\"ACL2____CLEAR-MEMOIZE-STATISTICS\">clear-memoize-statistics</see>.</p> \u000A \u000A <p><v>Profile</v> is just a macro that calls <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> to do its work. \u000A <v>Profile</v> gives the two keyword parameters <v>:condition</v> and \u000A <v>:recursive</v> of <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> the value <v>nil</v>.  Other keyword \u000A parameters for <v>memoize</v>, which must not include <v>:condition</v>, \u000A <v>:condition-fn</v>, or <v>:recursive</v>, are passed through.  To eliminate \u000A profiling, use <tt><see topic=\"ACL2____UNMEMOIZE\">unmemoize</see></tt>; for example, to eliminate profiling for \u000A function <v>fn</v>, evaluate <v>(<see topic=\"ACL2____UNMEMOIZE\">unmemoize</see> 'fn)</v>.</p> \u000A \u000A <p>You may find raw Lisp functions <v>profile-all</v> and <v>profile-acl2</v> to \u000A be useful.  These may be found in file <v>memoize/old/profile-raw.lsp</v> \u000A distributed with the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.</p>"],

 "ACL2____POR":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____POR\">por</see> (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> x y)\u000A     (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> y x))\u000A\u000A(<see topic=\"ACL2____POR\">por</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A      (<see topic=\"ACL2____GRANULARITY\">granularity</see>\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) 500)\u000A            (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> y) 500))))\u000A      (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> x y)\u000A      (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> y x))</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____POR\">por</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> expr)) ; optional granularity declaration\u000A     arg1 ... argN)</code> \u000A \u000A <p>where <v>N &gt;= 0</v> and each <v>argi</v> and <v>expr</v> are arbitrary terms.</p> \u000A \u000A <p><v>Por</v> evaluates its arguments in parallel.  It returns a Boolean result: \u000A <v>t</v> if any argument evaluates to non-<v>nil</v>, else <v>nil</v>.  Note that \u000A while <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> returns the first non-<v>nil</v> value from evaluating its \u000A arguments left-to-right (if any such value is not <v>nil</v>) <tt><see topic=\"ACL2____POR\">por</see></tt> always \u000A returns a Boolean result, in support of efficiency (see <see topic=\"ACL2____EARLY-TERMINATION\">early-termination</see>) in light of the nondeterministic order in which argument \u000A values are returned.</p> \u000A \u000A <p>Another difference between <v>por</v> and <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> is that for a call of \u000A <v>por</v>, even if the an argument's value is not <v>nil</v>, a subsequent \u000A argument may be evaluated.  See <see topic=\"ACL2____PAND\">pand</see> for a discussion of the analogous \u000A property of <v>pand</v>.  In particular, <see topic=\"ACL2____GUARD\">guard</see>s generated from calls of \u000A <v>por</v> may not assume for an argument that the preceding arguments evaluated \u000A to <v>nil</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see> for another example.  Also see <see topic=\"ACL2____PARALLELISM-AT-THE-TOP-LEVEL\">parallelism-at-the-top-level</see> for restrictions on evaluating parallelism \u000A primitives from within the ACL2 top-level loop.  Finally see <see topic=\"ACL2____EARLY-TERMINATION\">early-termination</see> to read how <v>por</v> can offer more efficiency than <tt><see topic=\"COMMON-LISP____OR\">or</see></tt> by avoiding evaluation of some of its arguments.</p>"],

 "ACL2____OK-IF":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example Form:\u000A:ok-if (<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"ACL2____BRR_04\">brr@</see> :wonp))\u000A\u000AGeneral Form:\u000A:ok-if expr</code> \u000A \u000A <p>where <v>expr</v> is a term involving no free variables other than <v>state</v> \u000A and returning one non-<v>state</v> result which is treated as Boolean.  This \u000A form is intended to be executed from within <v>break-rewrite</v> (see <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see>).</p> \u000A \u000A <p>Consider first the simple situation that the <v>(<see topic=\"ACL2____OK-IF\">ok-if</see> term)</v> is a command \u000A read by <v>break-rewrite</v>.  Then, if the term is non-<v>nil</v>, \u000A <v>break-rewrite</v> exits and otherwise it does not.</p> \u000A \u000A <p>More generally, <v>ok-if</v> returns an ACL2 error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val\u000A state)</v>.  (See <see topic=\"ACL2____LD\">ld</see> or see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see> for more on \u000A error triples.)  If any form being evaluated as a command by \u000A <v>break-rewrite</v> returns the triple returned by <v>(<see topic=\"ACL2____OK-IF\">ok-if</see> term)</v> then the \u000A effect of that form is to exit <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> if term is non-<v>nil</v>. \u000A Thus, one might define a function or macro that returns the value of \u000A <v>ok-if</v> expressions on all outputs and thus create a convenient new way to \u000A exit <v>break-rewrite</v>.</p> \u000A \u000A <p>The exit test, <v>term</v>, generally uses <v>brr@</v> to access context \u000A sensitive information about the attempted rule application.  See <see topic=\"ACL2____BRR_04\">brr@</see>. \u000A <v>Ok-if</v> is useful inside of command sequences produced by break conditions. \u000A See <see topic=\"ACL2____MONITOR\">monitor</see>.  <v>:ok-if</v> is most useful after an <v>:eval</v> command \u000A has caused <v>break-rewrite</v> to try to apply the rule because in the \u000A resulting break environment <v>expr</v> can access such things as whether the \u000A rule succeeded, if so, what term it produced, and if not, why.  There is no \u000A need to use <v>:ok-if</v> before <v>:eval</v>ing the rule since the same effects \u000A could be achieved with the break condition on the rule itself.  Perhaps we \u000A should replace this concept with <v>:eval-and-break-if</v>?  Time will \u000A tell.</p>"],

 "ACL2____HONS-COPY":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In the logic, <v>hons-copy</v> is just the identity function; we leave it \u000A enabled and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons-copy</v> does whatever is necessary to produce a \u000A <see topic=\"ACL2____NORMED\">normed</see> version of X.</p> \u000A \u000A <p>What might this involve?</p> \u000A \u000A <p>If X is a symbol, character, or number, then it is already normed and \u000A nothing is done.</p> \u000A \u000A <p>If X is a string, we check if any normed version of X already exists.  If \u000A so, we return the already-normed version; otherwise, we install X as the \u000A normed version for all strings that are <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> to X.</p> \u000A \u000A <p>If X is a cons, we must determine if there is a normed version of X, or \u000A recursively construct and install one.  Norming large cons trees can become \u000A expensive, but there are a couple of cheap cases.  In particular, if X is \u000A already normed, or if large subtrees of X are already normed, then not much \u000A needs to be done.  The slowest case is norming some large ACL2 cons structure \u000A that has no subtrees which are already normed.</p> \u000A \u000A <p>Note that running <v>hons-copy</v> on an object that was created with <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> is often slower than just using <tt><see topic=\"ACL2____HONS\">hons</see></tt> directly when constructing \u000A the object.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-copy</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-copy (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "ACL2____HONS":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In the logic, <v>hons</v> is just <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>; we leave it enabled and would \u000A think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons</v> does whatever is necessary to ensure that its \u000A result is <see topic=\"ACL2____NORMED\">normed</see>.</p> \u000A \u000A <p>What might this involve?</p> \u000A \u000A <p>Since the car and cdr of any normed cons must be normed, we need to <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> x and y.  This requires little work if x and y are already normed, \u000A but can be expensive if x or y contain large, un-normed cons structures.</p> \u000A \u000A <p>After that, we need to check whether any normed cons equal to <v>(x . y)</v> \u000A already exists.  If so, we return it; otherwise, we need to construct a new \u000A cons for <v>(x . y)</v> and install it as the normed version of <v>(x\u000A . y)</v>.</p> \u000A \u000A <p>Generally speaking, these extra operations make <v>hons</v> much slower than \u000A <v>cons</v>, even when given normed arguments.</p> \u000A \u000A <p><b>Function: </b><srclink>hons</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> x y))</code>"],

 "ACL2____FILE-READING-EXAMPLE":[["Tutorial5-miscellaneous-examples"],"ACL2 Sources","ACL2","<p>This example illustrates the use of ACL2's <see topic=\"ACL2____IO\">io</see> primitives to \u000A read the forms in a file.  See <see topic=\"ACL2____IO\">io</see>.</p> \u000A \u000A <p>This example provides a solution to the following problem.  Let's say that \u000A you have a file that contains s-expressions.  Suppose that you want to build a \u000A list by starting with <v>nil</v>, and updating it ``appropriately'' upon \u000A encountering each successive s-expression in the file.  That is, suppose that \u000A you have written a function <v>update-list</v> such that <v>(update-list obj\u000A current-list)</v> returns the list obtained by ``updating'' <v>current-list</v> \u000A with the next object, <v>obj</v>, encountered in the file.  The top-level \u000A function for processing such a file, returning the final list, could be \u000A defined as follows.  Notice that because it opens a channel to the given file, \u000A this function modifies <see topic=\"ACL2____STATE\">state</see> and hence must return <see topic=\"ACL2____STATE\">state</see>.  Thus \u000A it actually returns two values: the final list and the new <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> process-file (filename state)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see>\u000A   (channel state)\u000A   (<see topic=\"ACL2____OPEN-INPUT-CHANNEL\">open-input-channel</see> filename :object state)\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (result state)\u000A           (process-file1 nil channel state) ;see below\u000A           (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (<see topic=\"ACL2____CLOSE-INPUT-CHANNEL\">close-input-channel</see> channel state)))\u000A             (<see topic=\"ACL2____MV\">mv</see> result state)))))</code> \u000A \u000A <p>The function <v>process-file1</v> referred to above takes the currently \u000A constructed list (initially, <v>nil</v>), together with a channel to the file \u000A being read and the <see topic=\"ACL2____STATE\">state</see>, and returns the final updated list.  Notice \u000A that this function is tail recursive.  This is important because many Lisp \u000A compilers will remove tail recursion, thus avoiding the potential for stack \u000A overflows when the file contains a large number of forms.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> process-file1 (current-list channel state)\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (eofp obj state)\u000A          (<see topic=\"ACL2____READ-OBJECT\">read-object</see> channel state)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A           (eofp (<see topic=\"ACL2____MV\">mv</see> current-list state))\u000A           (t (process-file1 (update-list obj current-list)\u000A                             channel state)))))</code> \u000A \u000A <p>As an exercise, you might want to add <see topic=\"ACL2____GUARD\">guard</see>s to the functions above \u000A and verify the guards (see <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>).  See <see topic=\"ACL2____ARGS\">args</see> or make a \u000A call of the form <v>(<see topic=\"ACL2____GUARD\">guard</see> 'your-function nil (w state))</v> to see the guard of \u000A an existing function.</p>"],

 "ACL2____Corroborating_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Models_02of_02Computer_02Hardware_02and_02Software'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>After producing a model, it must be <b>corroborated</b> against reality. \u000A The Falling Body Model has been corroborated by a vast number of experiments \u000A in which the time and distance were measured and compared according to the \u000A formula.  In general all models must be corroborated by experiment.</p> \u000A \u000A <p>The Falling Body Model can be derived from deeper models, namely Newton's \u000A laws of motion and the assertion that, over the limited distances concerned, \u000A graviation exerts a constant acceleration on the object.  When the model in \u000A question can be derived from other models, it is the other models that are \u000A being corroborated by our experiments.</p> \u000A \u000A <p>Because nature is not formal, we cannot <b>prove</b> that our models of it \u000A are correct.  All we can do is test our models against nature's behavior.</p> \u000A \u000A <p>Such testing often exposes restrictions on the applicability of our models. \u000A For example, the Falling Body Model is inaccurate if air resistance is \u000A significant.  Thus, we learn not to use that model to predict how long it \u000A takes a feather to fall from a 200 foot tower in the earth's atmosphere.</p> \u000A \u000A <p>In addition, attempts at corroboration might reveal that the model is \u000A actually incorrect.  Careful measurements might expose the fact that the \u000A gravitational force increases as the body falls closer to earth.  Very careful \u000A measurements might reveal relativistic effects.  Technically, the familiar \u000A Falling Body Model is just wrong, even under excessive restrictions such as \u000A ``in a perfect vacuum'' and ``over small distances.''  But it is an incredibly \u000A useful model nonetheless.</p> \u000A \u000A <p>There are several morals here.</p> \u000A \u000A <p><b>Models need not be complete to be useful.</b></p> \u000A \u000A <p><b>Models need not be perfectly accurate to be useful.</b></p> \u000A \u000A <p><b>The user of a model must understand its limitations.</b></p> \u000A \u000A <p><see topic='ACL2____Models_02of_02Computer_02Hardware_02and_02Software'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____CASE-MATCH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____CASE-MATCH\">case-match</see> x\u000A  (pat1 dcl1 body1)\u000A  ...\u000A  (patk dclk bodyk))</code> \u000A \u000A <p>where <v>x</v> is a variable symbol, the <v>pati</v> are structural patterns as \u000A described below, the <v>dcli</v> are optional <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> forms and the \u000A <v>bodyi</v> are terms.  Return the value(s) of the <v>bodyi</v> corresponding to \u000A the first <v>pati</v> matching <v>x</v>, or <v>nil</v> if none matches.</p> \u000A \u000A <p>Pattern Language:<br></br> \u000A \u000A With the few special exceptions described below, matching requires that the \u000A <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> structure of <v>x</v> be isomorphic to that of the pattern, down to \u000A the <see topic=\"COMMON-LISP____ATOM\">atom</see>s in the pattern.  Non-symbol <see topic=\"COMMON-LISP____ATOM\">atom</see>s in the pattern match \u000A only themselves.  Symbols in the pattern denote variables which match anything \u000A and which are bound by a successful match to the corresponding substructure of \u000A <v>x</v>.  Variables that occur more than once must match the same (<tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>) structure in every occurrence.</p> \u000A \u000A <code>Exceptions:\u000A&amp;               Matches anything and is not bound.  Repeated\u000A                  occurrences of &amp; in a pattern may match different\u000A                  structures.\u000Anil, t, *sym*   These symbols cannot be bound and match only their\u000A                  global values.\u000A!sym            where sym is a symbol that is already bound in the\u000A                  context of the case-match, matches only the\u000A                  current binding of sym.\u000A'obj            Matches only itself.</code> \u000A \u000A <p>Some examples are shown below.</p> \u000A \u000A <p>Below we show some sample patterns and examples of things they match and do \u000A not match.</p> \u000A \u000A <code>pattern       matches         non-matches\u000A(x y y)       (ABC 3 3)       (ABC 3 4)    ; 3 is not 4\u000A(fn x . rst)  (P (A I) B C)   (ABC)        ; NIL is not (x . rst)\u000A              (J (A I))                    ; rst matches nil\u000A('fn (g x) 3) (FN (H 4) 3)    (GN (G X) 3) ; 'fn matches only itself\u000A(&amp; t &amp; !x)    ((A) T (B) (C))              ; provided x is '(C)</code> \u000A \u000A <p>Consider the two binary trees that contain three leaves.  They might be \u000A described as <v>(x . (y . z))</v> and <v>((x . y) . z)</v>, where <v>x</v>, <v>y</v>, \u000A and <v>z</v> are atomic.  Suppose we wished to recognize those trees.  The \u000A following <v>case-match</v> would do:</p> \u000A \u000A <code>(<see topic=\"ACL2____CASE-MATCH\">case-match</see> tree\u000A  ((x . (y . z))\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____ATOM\">atom</see> y) (<see topic=\"COMMON-LISP____ATOM\">atom</see> z)))\u000A  (((x . y) . z)\u000A   (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____ATOM\">atom</see> y) (<see topic=\"COMMON-LISP____ATOM\">atom</see> z))))</code> \u000A \u000A <p>Suppose we wished to recognize such trees where all three tips are \u000A identical.  Suppose further we wish to return the tip if the tree is one of \u000A those recognized ones and to return the number <v>7</v> otherwise.</p> \u000A \u000A <code>(<see topic=\"ACL2____CASE-MATCH\">case-match</see> tree\u000A  ((x . (x . x))\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) x 7))\u000A  (((x . x) . x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) x 7))\u000A  (&amp; 7))</code> \u000A \u000A <p>Note that <v>case-match</v> returns <v>nil</v> if no <v>pati</v> matches.  Thus \u000A if we must return <v>7</v> in that case, we have to add as the final pattern the \u000A <v>&amp;</v>, which always matches anything.</p>"],

 "ACL2____USING-COMPUTED-HINTS-8":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>None of the examples show the use of the variable <v>WORLD</v>, which \u000A is allowed in computed hints.  There are some (undocumented) ACL2 utilities \u000A that might be useful in programming hints, but these utilities need access to \u000A the ACL2 logical world (see <see topic=\"ACL2____WORLD\">world</see>).</p> \u000A \u000A <p>A very useful fact to know is that <v>(table-alist name world)</v> returns an \u000A alist representation of the current value of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> named \u000A <v>name</v>.</p> \u000A \u000A <p>The ACL2 source code is littered with <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode functions \u000A for manipulating world.  In our source code, the world is usually bound a \u000A variable named <v>wrld</v>; so searching our code for that name might be \u000A helpful.</p> \u000A \u000A <p>Using these utilities to look at the <v>WORLD</v> one can, for example, \u000A determine whether a symbol is defined recursively or not, get the body and \u000A formals of a defined function, or fetch the statement of a given lemma. \u000A Because these utilities are not yet documented, we do not expect users to \u000A employ <v>WORLD</v> in computed hints.  But experts might and it might lead to \u000A the formulation of a more convenient language for computed hints.</p> \u000A \u000A <p>None of our examples illustrated the 7 argument form of a computed hint, \u000A <v>(fn ID CLAUSE WORLD STABLE-UNDER-SIMPLIFICATIONP HIST PSPV CTX)</v>.  When \u000A used, the variables <v>HIST</v>, <v>PSPV</v>, and <v>CTX</v>, are bound to the \u000A clause history, the package of ``special variables'' governing the clause, and \u000A the ``error message context.''  These variables are commonly used throughout \u000A our source code but are, unfortunately, undocumented.  Again, we expect a few \u000A experts will find them useful in developing computed hints.</p> \u000A \u000A <p>If you start using computed hints extensively, please contact the \u000A developers of ACL2 and let us know what you are doing with them and how we can \u000A help.</p>"],

 "ACL2____SET-NU-REWRITER-MODE":[["Nu-rewriter"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>This event sets a flag that controls whether the ACL2 rewriter uses the \u000A special-purpose <v>nth</v>/<v>update-nth</v> rewriter (nu-rewriter).  The flag may \u000A have one of three values: <v>nil</v>, <v>t</v>, or <v>:literals</v>.</p> \u000A \u000A <code>:set-nu-rewriter-mode nil        ; do not use nu-rewriter\u000A:set-nu-rewriter-mode t          ; use nu-rewriter in rewriting\u000A:set-nu-rewriter-mode :literals  ; use nu-rewriter in rewriting after\u000A                                 ;  a pre-pass through every literal\u000A(<see topic=\"ACL2____SET-NU-REWRITER-MODE\">set-nu-rewriter-mode</see> :literals) ; same as above</code> \u000A \u000A <p>The value <v>nil</v> prevents the use of the nu-rewriter.  The other two \u000A values allow the use of the nu-rewriter.</p> \u000A \u000A <p>When the flag is non-<v>nil</v> and the rewriter encounters a term that \u000A ``begins with an <v>nth</v>'', the nu-rewriter is applied.  By ``begins with an \u000A <v>nth</v>'' here we mean either the term is an application of <v>nth</v> or is an \u000A application of some nonrecursive function or <v>lambda</v> expression whose body \u000A contains an expression that begins with an <v>nth</v>.</p> \u000A \u000A <p>Note that the use of the nu-rewriter here described above is driven by the \u000A rewriter, i.e., the nu-rewriter is applied only to terms visited by the \u000A rewriter in its inside-out sweep.  When the flag is set to <v>:literals</v> the \u000A system makes a pre-pass through every goal clause and applies the nu-rewriter \u000A to every subterm.  The rewriter is then used on the output of that \u000A pre-pass.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>We expect to write more documentation as we gain experience with the \u000A nu-rewriter.</p>"],

 "ACL2____PUSH-UNTOUCHABLE":[["Defttag"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see> my-var nil)\u000A(<see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see> set-mem t)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see> name{s}  fn-p :doc doc-string)</code> \u000A \u000A <p>where <v>name{s}</v> is a non-<v>nil</v> symbol or a non-<v>nil</v> true list of \u000A symbols, <v>fn-p</v> is any value (but generally <v>nil</v> or <v>t</v>), and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string not beginning with \u000A ``<v>:Doc-Section</v> ...''.  If <v>name{s}</v> is a symbol it is treated as the \u000A singleton list containing that symbol.  The effect of this event is to union \u000A the given symbols into the list of ``untouchable variables'' in the current \u000A world if <v>fn-p</v> is <v>nil</v>, else to union the symbols into the list of \u000A ``untouchable functions''.  This event is redundant if every symbol listed is \u000A already a member of the appropriate untouchables list (variables or \u000A functions).</p> \u000A \u000A <p>When a symbol is on the untouchables list it is syntactically illegal for \u000A any event to call a function or macro of that name, if <v>fn-p</v> is \u000A non-<v>nil</v>, or to change the value of a state global variable of that name, \u000A if <v>fn-p</v> is <v>nil</v>.  Thus, the effect of pushing a function symbol, \u000A <v>name</v>, onto untouchables is to prevent any future event from using that \u000A symbol as a function or macro, or as a state global variable (according to \u000A <v>fn-p</v>).  This is generally done to ``fence off'' some primitive function \u000A symbol from ``users'' after the developer has used the symbol freely in the \u000A development of some higher level mechanism.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>.</p>"],

 "ACL2____NOTE-4-0-WORMHOLE-CHANGES":[["Note-4-0"],"ACL2 Sources","ACL2","<p>Here we describe how to convert an ``old-style'' call of <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt> &mdash; that is, a call suitable for ACL2 versions preceding 4.0 \u000A &mdash; in which the <v>pseudo-flag</v> was <v>t</v>.  In order to convert such a \u000A call</p> \u000A \u000A <code>(<see topic=\"ACL2____WORMHOLE\">wormhole</see> t 'name input form ...)</code> \u000A \u000A <p>to a new-style call, the following steps must be carried out.  Note that \u000A the wormhole name must always be quoted now.</p> \u000A \u000A <p>First, eliminate the first argument, <v>t</v>, and add a new second argument \u000A that is the quoted lambda expression</p> \u000A \u000A <code>'(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) (<see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see> whs :ENTER))</code> \u000A \u000A <p>Setting the entry code to <v>:ENTER</v> is not necessary if you maintain the \u000A invariant (after initialization) that it is always <v>:ENTER</v>.  In that case, \u000A the simpler quoted lambda will suffice:</p> \u000A \u000A <code>'(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) whs)</code> \u000A \u000A <p>Second, change the <v>form</v> argument so that instead of talking about the \u000A state-global variable <v>wormhole-output</v> it talks about the state-global \u000A variable <v>wormhole-status</v>.  Look for <v>(<see topic=\"ACL2_____04\">@</see> wormhole-output)</v>, <v>(<see topic=\"ACL2____ASSIGN\">assign</see>\u000A wormhole-output ...)</v>, <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'wormhole-output ...)</v> and \u000A <v>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'wormhole-output ...)</v> in <v>form</v> and replace them with \u000A expressions involving <v>wormhole-status</v>.</p> \u000A \u000A <p>However, remember that the old data stored in <v>wormhole-output</v> is now \u000A in the <v>wormhole-data</v> component of the <v>wormhole-status</v>.  Thus, for \u000A example, an old use of <v>(<see topic=\"ACL2_____04\">@</see> wormhole-output)</v> will typically be replaced by \u000A <v>(<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> (<see topic=\"ACL2_____04\">@</see> wormhole-status))</v> and an old use of <v>(<see topic=\"ACL2____ASSIGN\">assign</see>\u000A wormhole-output ...)</v> will typically be replaced by</p> \u000A \u000A <code>(<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status (<see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see> (<see topic=\"ACL2_____04\">@</see> wormhole-status) ...))</code> \u000A \u000A <p>In summary, an old-style call like</p> \u000A \u000A <code>(<see topic=\"ACL2____WORMHOLE\">wormhole</see> t 'name\u000A          input\u000A          '(...1 (<see topic=\"ACL2_____04\">@</see> wormhole-output) ...2\u000A            ...3 (<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-output ...4) ...5)\u000A          ...6)</code> \u000A \u000A <p>can become</p> \u000A \u000A <code>(<see topic=\"ACL2____WORMHOLE\">wormhole</see> 'name\u000A          '(<see topic=\"COMMON-LISP____LAMBDA\">lambda</see> (whs) (<see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see> whs :ENTER))\u000A          input\u000A          '(...1 (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> (<see topic=\"ACL2_____04\">@</see> wormhole-status)) ...2\u000A            ...3 (<see topic=\"ACL2____ASSIGN\">assign</see> wormhole-status\u000A                        (<see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see> (<see topic=\"ACL2_____04\">@</see> wormhole-status)\u000A                                           ...4) ...5)\u000A          ...6)</code> \u000A \u000A <p>In any case, and especially if your <v>wormhole</v> call had a \u000A <v>pseudo-flag</v> other than <v>t</v>, we recommend that you see <see topic=\"ACL2____WORMHOLE\">wormhole</see>.</p>"],

 "ACL2____FAST-ALIST-CLEAN":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  It assumes familiarity with fast alists; see \u000A <see topic=\"ACL2____FAST-ALISTS\">fast-alists</see>.</p> \u000A \u000A <p>See <tt><see topic=\"ACL2____FAST-ALIST-CLEAN_12\">fast-alist-clean!</see></tt>, <tt><see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see></tt>, and <tt><see topic=\"ACL2____FAST-ALIST-FORK_12\">fast-alist-fork!</see></tt> for related utilities.</p> \u000A \u000A <p>Logically, <v>(<see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see> alist)</v> is defined as follows:</p> \u000A \u000A <p><b>Function: </b><srclink>fast-alist-clean</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fast-alist-clean (alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> alist\u000A                        (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> alist)\u000A                            (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____LAST\">last</see> alist))\u000A                            alist)))</code> \u000A \u000A <p>The result is thus a corresponding fast alist, with the order reversed and \u000A with atoms and shadowed pairs removed, as per the definition above; see <see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> for details.  Moreover, if <v>alist</v> is not a fast alist, \u000A then <v>(<see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see> alist)</v> is executed in raw Lisp by calling \u000A <v>fast-alist-fork</v> as indicated above.</p> \u000A \u000A <p>However, if <v>alist</v> is a fast alist, then a special definition under the \u000A hood provides a different handling of associated hash tables.  After running \u000A <v>(<see topic=\"ACL2____FAST-ALIST-CLEAN\">fast-alist-clean</see> alist)</v> to obtain a result, <v>cleaned-alist</v>, the hash \u000A table that had been associated with <v>alist</v> will now be associated with \u000A <v>cleaned-alist</v> instead.  This saves the expense of creating a new hash \u000A table, but there is still the expense of copying the alist, as for <tt><see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see></tt>.  However, unlike <v>fast-alist-fork</v>, there is no need to \u000A free the input alist.</p> \u000A \u000A <p>Note that the final <v>cdr</v> is preserved, so that the name is preserved \u000A for use by <tt><see topic=\"ACL2____FAST-ALIST-SUMMARY\">fast-alist-summary</see></tt> (also see <see topic=\"ACL2____HONS-ACONS\">hons-acons</see>).</p>"],

 "ACL2____EARLY-TERMINATION":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>The evaluation of <v>(<see topic=\"COMMON-LISP____AND\">and</see> expr1 expr2)</v> returns <v>nil</v> if <v>expr1</v> \u000A evaluates to <v>nil</v>, avoiding the evaluation of <v>expr2</v>.  More generally, \u000A the evaluation of <v>(<see topic=\"COMMON-LISP____AND\">and</see> expr1 expr2 ... exprk)</v> terminates with a return \u000A value of <v>nil</v> as soon as any <v>expri</v> evaluates to <v>nil</v> &mdash; no \u000A <v>exprj</v> is evaluated in this case for <v>j &gt; i</v>.  This so-called ``lazy \u000A evaluation'' of <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> terms can thus save some computation; roughly \u000A speaking, the smaller the <v>i</v>, the more computation is saved.</p> \u000A \u000A <p>If the above call of <tt><see topic=\"COMMON-LISP____AND\">and</see></tt> is replaced by its parallel version, \u000A <tt><see topic=\"ACL2____PAND\">pand</see></tt>, then there can be even more opportunity for skipping work.  The \u000A arguments to <tt><see topic=\"ACL2____PAND\">pand</see></tt> can be evaluated in parallel, in which case the \u000A first such evaluation that returns with a value of <v>nil</v>, if any, causes \u000A the remaining such evaluations to abort.</p> \u000A \u000A <p>Consider the following functions that compute whether a tree is valid (see \u000A <see topic=\"ACL2____GRANULARITY\">granularity</see> for a discussion of the granularity form).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> valid-tip (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'A)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'T)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'C)\u000A      (<see topic=\"COMMON-LISP____EQ\">eq</see> x 'G)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pvalid-tree (x depth)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> depth)))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      (valid-tip x)\u000A    (<see topic=\"ACL2____PAND\">pand</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> depth 10)))\u000A          (pvalid-tree (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))\u000A          (pvalid-tree (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth)))))</code> \u000A \u000A <p>We would like to stop execution as soon as any tip is found to be invalid. \u000A So, when computing the conjunction of terms by using <tt><see topic=\"ACL2____PAND\">pand</see></tt>, once one of \u000A those terms evaluates to <v>nil</v>, the computations for the other terms are \u000A aborted and the <tt><see topic=\"ACL2____PAND\">pand</see></tt> call returns <v>nil</v>.  By using <tt><see topic=\"ACL2____PAND\">pand</see></tt>, we \u000A can in principle attain a speedup factor greater than the number of available \u000A cores.</p> \u000A \u000A <p>The concept of early termination also applies to <tt><see topic=\"ACL2____POR\">por</see></tt>, except that \u000A early termination occurs when an argument evaluates to non-<v>nil</v>.</p>"],

 "ACL2____CLAUSE-IDENTIFIER":[["Goal-spec"],"ACL2 Sources","ACL2","<p>To each goal-spec, <v>str</v>, there corresponds a clause-identifier \u000A produced by <v>(parse-clause-id str)</v>.  For example,</p> \u000A \u000A <code>(parse-clause-id &quot;[2]Subgoal *4.5.6/7.8.9'''&quot;)</code> \u000A \u000A <p>returns <v>((2 4 5 6) (7 8 9) . 3)</v>.</p> \u000A \u000A <p>The function <v>string-for-tilde-@-clause-id-phrase</v> inverts \u000A <v>parse-clause-id</v> in the sense that given a clause identifier it returns \u000A the corresponding goal-spec.</p> \u000A \u000A <p>As noted in the documentation for <see topic=\"ACL2____GOAL-SPEC\">goal-spec</see>, each clause printed in \u000A the theorem prover's proof attempt is identified by a name.  When these names \u000A are represented as strings they are called ``goal specs.''  Such strings are \u000A used to specify where in the proof attempt a given hint is to be applied.  The \u000A function <v>parse-clause-id</v> converts goal-specs into clause identifiers, \u000A which are cons-trees containing natural numbers.</p> \u000A \u000A <p>Examples of goal-specs and their corresponding clause identifiers are shown \u000A below.</p> \u000A \u000A <code>             parse-clause-id\u000A                   --&gt;\u000A\u000A&quot;Goal&quot;                       ((0) NIL . 0)\u000A&quot;Subgoal 3.2.1'&quot;             ((0) (3 2 1) . 1)\u000A&quot;[2]Subgoal *4.5.6/7.8.9'''&quot; ((2 4 5 6) (7 8 9) . 3)\u000A\u000A                   &lt;--\u000A      string-for-tilde-@-clause-id-phrase</code> \u000A \u000A <p>The caar of a clause id specifies the forcing round, the cdar specifies the \u000A goal being proved by induction, the cadr specifies the particular subgoal, and \u000A the cddr is the number of primes in that subgoal.</p> \u000A \u000A <p>Internally, the system maintains clause ids, not goal-specs.  The system \u000A prints clause ids in the form shown by goal-specs.  When a goal-spec is used \u000A in a hint, it is parsed (before the proof attempt begins) into a clause id. \u000A During the proof attempt, the system watches for the clause id and uses the \u000A corresponding hint when the id arises.  (Because of the expense of creating \u000A and garbage collecting a lot of strings, this design is more efficient than \u000A the alternative.)</p>"],

 "ACL2-PC____HYPS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Ahyps               -- print all (<see topic=\"ACL2____TOP-LEVEL\">top-level</see>) hypotheses\u000A(hyps (1 3) (2 4)) -- print hypotheses 1 and 3 and governors 2 and 4\u000A(hyps (1 3) t)     -- print hypotheses 1 and 3 and all governors\u000A\u000AGeneral Form:\u000A(hyps &amp;optional hyps-indices govs-indices)</code> \u000A \u000A <p>Print the indicated top-level hypotheses and governors.  (The notion of \u000A ``governors'' is defined below.)  Here, <v>hyps-indices</v> and \u000A <v>govs-indices</v> should be lists of indices of hypotheses and governors \u000A (respectively), except that the atom <v>t</v> may be used to indicate that one \u000A wants all hypotheses or governors (respectively).</p> \u000A \u000A <p>The list of ``governors'' is defined as follows.  Actually, we define here \u000A the notion of the governors for a pair of the form <v>&lt;term</v>, address&gt;]; \u000A we're interested in the special case where the term is the conclusion and the \u000A address is the current address.  If the address is <v>nil</v>, then there are no \u000A governors, i.e., the list of governors is <v>nil</v>.  If the term is of the \u000A form <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y z)</v> and the address is of the form <v>(2 . rest)</v> or <v>(3\u000A . rest)</v>, then the list of governors is the result of <v>cons</v>ing <v>x</v> or \u000A its negation (respectively) onto the list of governors for the pair <v>&lt;y,\u000A rest&gt;</v> or the pair <v>&lt;z, rest&gt;</v> (respectively).  If the term is of the form \u000A <v>(<see topic=\"ACL2____IMPLIES\">implies</see> x y)</v> and the address is of the form <v>(2 . rest)</v>, then the \u000A list of governors is the result of <v>cons</v>ing <v>x</v> onto the list of \u000A governors for the pair <v>&lt;y, rest&gt;</v>.  Otherwise, the list of governors for \u000A the pair <v>&lt;term, (n .  rest)&gt;</v> is exactly the list of governors for the \u000A pair <v>&lt;argn, rest&gt;</v> where <v>argn</v> is the <v>n</v>th argument of \u000A <v>term</v>.</p> \u000A \u000A <p>If all goals have been proved, a message saying so will be printed.  (as \u000A there will be no current hypotheses or governors!).</p> \u000A \u000A <p>The <v>hyps</v> command never causes an error.  It ``succeeds'' (in fact its \u000A value is <v>t</v>) if the arguments (when supplied) are appropriate, i.e. \u000A either <v>t</v> or lists of indices of hypotheses or governors, respectively. \u000A Otherwise it ``fails'' (its value is <v>nil</v>).</p>"],

 "ACL2-PC____HELP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A(<see topic=\"ACL2____HELP\">help</see> all)     -- list all proof-checker commands\u000A(<see topic=\"ACL2____HELP\">help</see> rewrite) -- partial documentation on the rewrite command; the\u000A                  rest is available using more or more!\u000A\u000A(help! rewrite) -- full documentation on the rewrite command\u000A\u000Ahelp, help!     -- this documentation (in part, or in totality,\u000A                   respectively)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____HELP\">help</see> &amp;optional command)\u000A(help! &amp;optional command)\u000Amore\u000Amore!</code> \u000A \u000A <p>The proof checker supports the same kind of documentation as does ACL2 \u000A proper.  The main difference is that you need to type</p> \u000A \u000A <code>(<see topic=\"ACL2____HELP\">help</see> command)</code> \u000A \u000A <p>in a list rather than <v>:doc command</v>.  So, to get all the documentation \u000A on <v>command</v>, type <v>(help! command)</v> inside the interactive loop, but to \u000A get only a one-line description of the command together with some examples, \u000A type <v>(<see topic=\"ACL2____HELP\">help</see> command)</v>.  In the latter case, you can get the rest of the \u000A help by typing <v>more!</v>; or type <v>more</v> if you don't necessarily want all \u000A the rest of the help at once.  (Then keep typing <v>more</v> if you want to keep \u000A getting more of the help for that command.)</p> \u000A \u000A <p>An exception is <v>(<see topic=\"ACL2____HELP\">help</see> all)</v>, which prints the documentation topic \u000A <v>proof-checker-commands</v>, to show you all possible proof-checker commands. \u000A So for example, when you see <v>ACL2-PC::USE</v> in that list, you can then \u000A submit <v>(<see topic=\"ACL2____HELP\">help</see> use)</v> or <v>(help! use)</v> to get documentation for the \u000A proof-checker <v>use</v> command.</p> \u000A \u000A <p>But summarizing for other than the case of <v>all</v>: as with ACL2, you can \u000A type either of the following:</p> \u000A \u000A <code>more, more!\u000A-- to obtain more (<see topic=\"COMMON-LISP____OR\">or</see>, all the rest of) the documentation last\u000A   requested by help (<see topic=\"COMMON-LISP____OR\">or</see>, outside the proof-checker's loop, :doc)</code> \u000A \u000A <p>It has been arranged that the use of <v>(<see topic=\"ACL2____HELP\">help</see> command)</v> will tell you just \u000A about everything you could want to know about <v>command</v>, almost always by \u000A way of examples.  For more details about a command, use <v>help!</v>, <v>more</v>, \u000A or <v>more!</v>.</p> \u000A \u000A <p>We use the word ``command'' to refer to the name itself, e.g. \u000A <v>rewrite</v>.  We use the word ``instruction'' to refer to an input to the \u000A interactive system, e.g. <v>(<see topic=\"ACL2____REWRITE\">rewrite</see> foo)</v> or <v>(<see topic=\"ACL2____HELP\">help</see> split)</v>.  Of course, \u000A we allow commands with no arguments as instructions in many cases, \u000A e.g. <v>rewrite</v>.  In such cases, <v>command</v> is treated identically to \u000A <v>(<see topic=\"ACL2____COMMAND\">command</see>)</v>.</p>"],

 "ACL2-PC____ADD-ABBREVIATION":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Example: <v>(add-abbreviation v (<see topic=\"COMMON-LISP_____A2\">*</see> x y))</v> causes future \u000A occurrences of <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x y)</v> to be printed as <v>(? v)</v>, until (unless) a \u000A corresponding invocation of <v>remove-abbreviations</v> occurs.  In this case we \u000A say that <v>v</v> ``abbreviates'' <v>(<see topic=\"COMMON-LISP_____A2\">*</see> x y)</v>.</p> \u000A \u000A <code>General Form:\u000A(add-abbreviation var &amp;optional raw-term)</code> \u000A \u000A <p>Let <v>var</v> be an abbreviation for <v>raw-term</v>, if <v>raw-term</v> is \u000A supplied, else for the current subterm.  Note that <v>var</v> must be a variable \u000A that does not already abbreviate some term.</p> \u000A \u000A <p>A way to think of abbreviations is as follows.  Imagine that whenever an \u000A abbreviation is added, say <v>v</v> abbreviates <v>expr</v>, an entry associating \u000A <v>v</v> to <v>expr</v> is made in an association list, which we will call \u000A ``<v>*abbreviations-alist*</v>''.  Then simply imagine that <v>?</v> is a function \u000A defined by something like:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ? (v)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((pair (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> v *abbreviations-alist*)))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> pair (<see topic=\"COMMON-LISP____CDR\">cdr</see> pair)\u000A      (error ...))))</code> \u000A \u000A <p>Of course the implementation isn't exactly like that, since the \u000A ``constant'' <v>*abbreviations-alist*</v> actually changes each time an \u000A <v>add-abbreviation</v> instruction is successfully invoked.  Nevertheless, if \u000A one imagines an appropriate redefinition of the ``constant'' \u000A <v>*abbreviations-alist*</v> each time an <v>add-abbreviation</v> is invoked, then \u000A one will have a clear model of the meaning of such an instruction.</p> \u000A \u000A <p>The effect of abbreviations on output is that before printing a term, each \u000A subterm that is abbreviated by a variable <v>v</v> is first replaced by <v>(?\u000A v)</v>.</p> \u000A \u000A <p>The effect of abbreviations on input is that every built-in proof-checker \u000A command accepts abbreviations wherever a term is expected as an argument, \u000A i.e., accepts the syntax <v>(? v)</v> whenever <v>v</v> abbreviates a term.  For \u000A example, the second argument of <v>add-abbreviation</v> may itself use \u000A abbreviations that have been defined by previous <v>add-abbreviation</v> \u000A instructions.</p> \u000A \u000A <p>See also <v>remove-abbreviations</v> and <v>show-abbreviations</v>.</p>"],

 "COMMON-LISP____RASSOC":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x alist)\u000A(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x alist :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x alist :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x alist :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____RASSOC\">Rassoc</see> x alist)</v> is the first member of <v>alist</v> whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> \u000A is <v>x</v>, or <v>nil</v> if no such member exists.  <v>(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x alist)</v> is \u000A thus similar to <v>(<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x alist)</v>, the difference being that it looks for \u000A the first pair in the given alist whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>, rather than <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>, \u000A is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> to <v>x</v>.  See <see topic=\"COMMON-LISP____ASSOC\">assoc</see>.  The optional keyword, \u000A <v>:TEST</v>, has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s of successive elements \u000A of <v>lst</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>rassoc</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> (see \u000A <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____R-EQLABLE-ALISTP\">r-eqlable-alistp</see></tt>. \u000A If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument must be a symbol or \u000A the second argument must satisfy <tt><see topic=\"ACL2____R-SYMBOL-ALISTP\">r-symbol-alistp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>rassoc</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____RASSOC-EQ\">rassoc-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x lst :test\u000A 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____RASSOC-EQUAL\">rassoc-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____RASSOC\">rassoc</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>rassoc-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>rassoc-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rassoc-equal (x alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> alist) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)))\u000A              (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A             (t (<see topic=\"ACL2____RASSOC-EQUAL\">rassoc-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)))))</code> \u000A \u000A <p><v>Rassoc</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p>"],

 "COMMON-LISP____POSITION":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq)\u000A(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____POSITION\">Position</see> x seq)</v> is the least index (zero-based) of the element <v>x</v> \u000A in the string or list <v>seq</v>, if <v>x</v> is an element of <v>seq</v>. \u000A Otherwise <v>(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq)</v> is <v>nil</v>.  The optional keyword, \u000A <v>:TEST</v>, has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with items of <v>seq</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>position</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> or <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>. \u000A If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable \u000A for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy \u000A <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first \u000A argument must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>position</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____POSITION-EQ\">position-eq</see> x seq)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq\u000A :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____POSITION-EQUAL\">position-equal</see> x seq)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____POSITION\">position</see> x seq :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>position-equal</v>.</p> \u000A \u000A <p><v>Position</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>position-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> position-equal (item lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> lst) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> lst)\u000A           (position-ac item (<see topic=\"COMMON-LISP____COERCE\">coerce</see> lst 'list) 0)\u000A           (position-equal-ac item lst 0)))</code> \u000A <p><b>Function: </b><srclink>position-equal-ac</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> position-equal-ac (item lst acc)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                                   (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> acc))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> item (<see topic=\"COMMON-LISP____CAR\">car</see> lst)) acc)\u000A             (t (position-equal-ac item (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)\u000A                                   (<see topic=\"COMMON-LISP____1_B2\">1+</see> acc)))))</code>"],

 "ACL2____UNIVERSAL-THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> :here)\u000A(<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> 'lemma3)</code> \u000A \u000A <p>See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____UNIVERSAL-THEORY\">universal-theory</see> logical-name)</code> \u000A \u000A <p>Returns the theory consisting of all the <see topic=\"ACL2____RUNE\">rune</see>s that existed \u000A immediately after <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt> was introduced.  See <see topic=\"ACL2____THEORIES\">theories</see> \u000A and see <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.  The theory includes <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt> itself \u000A (if there is a rule by that name).  (Note that since some <see topic=\"ACL2____EVENTS\">events</see> do not \u000A introduce rules (e.g., <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt>, <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> \u000A with <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <v>nil</v>), the universal-theory does not \u000A necessarily include a <see topic=\"ACL2____RUNE\">rune</see> for every event name.)  The universal-theory \u000A is very long and you will probably regret printing it.</p> \u000A \u000A <p>You may experience a fencepost problem in deciding which <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see> to use.  <tt><see topic=\"ACL2____DEFLABEL\">Deflabel</see></tt> can always be used to mark \u000A unambiguously for future reference a particular point in the development of \u000A your theory.  This is convenient because <tt><see topic=\"ACL2____DEFLABEL\">deflabel</see></tt> does not introduce \u000A any rules and hence it doesn't matter if you count it as being in the interval \u000A or not.  The order of <see topic=\"ACL2____EVENTS\">events</see> in the vicinity of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A is confusing.  See <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____CURRENT-THEORY\">current-theory</see>.  <v>Current-theory</v> is much more commonly \u000A used than <v>universal-theory</v>.  The former includes only the <see topic=\"ACL2____ENABLE\">enable</see>d \u000A <see topic=\"ACL2____RUNE\">rune</see>s as of the given <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt>, which is probably what you \u000A want, while the latter includes <see topic=\"ACL2____DISABLE\">disable</see>d ones as well.</p>"],

 "ACL2____PR":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A:pr fn ; prints the rules from the definition of fn (including any\u000A       ; :type-prescription rule and :definition rule)\u000A\u000A:pr assoc-append ; if assoc-append is a rewrite rule, prints that rule</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PR_12\">pr!</see>, which is similar but works at the command level \u000A instead of at the event level, and see <see topic=\"ACL2____PL\">pl</see>, which displays all rewrite \u000A rules for calls of <v>fn</v>, not just those introduced at definition time.</p> \u000A \u000A <p><v>Pr</v> takes one argument, a logical name, and prints the rules associated \u000A with it.  In each case it prints the rune, the current enabled/disabled \u000A status, and other appropriate fields from the rule.  It may be helpful to read \u000A the documentation for various kinds of rules in order to understand the \u000A information printed by this command.  For example, the information printed for \u000A a linear rule might be:</p> \u000A \u000A <code>Rune:     (:LINEAR ABC)\u000AEnabled:  T\u000AHyps:     ((<see topic=\"COMMON-LISP____CONSP\">CONSP</see> X))\u000AConcl:    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X)) (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> X))\u000AMax-term: (<see topic=\"ACL2____ACL2-COUNT\">ACL2-COUNT</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))\u000ABackchain-limit-lst:    (3)</code> \u000A \u000A <p>The <v>hyps</v> and <v>concl</v> fields for this rule are fairly \u000A self-explanatory, but it is useful to see <see topic=\"ACL2____LINEAR\">linear</see> to learn about maximal \u000A terms (which, as one might guess, are stored under ``Max-term'').</p> \u000A \u000A <p>Currently, this function does not print congruence rules or equivalence \u000A rules.</p> \u000A \u000A <p>The expert user might also wish to use <tt><see topic=\"ACL2____FIND-RULES-OF-RUNE\">find-rules-of-rune</see></tt>.  See \u000A <see topic=\"ACL2____FIND-RULES-OF-RUNE\">find-rules-of-rune</see>.</p>"],

 "ACL2____INTERN-IN-PACKAGE-OF-SYMBOL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion \u000A  Axiom (<v>completion-of-intern-in-package-of-symbol</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y))\u000A           (<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y)\u000A         nil))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x) (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y))</code> \u000A \u000A <p>Intuitively, <v>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y)</v> creates a symbol with \u000A <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> <v>x</v> <see topic=\"COMMON-LISP____INTERN\">intern</see>ed in the package containing <v>y</v>. \u000A More precisely, suppose <v>x</v> is a string, <v>y</v> is a symbol with <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> pkg and that the <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> event creating pkg had \u000A the list of symbols imports as the value of its second argument.  Then \u000A <v>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x y)</v> returns a symbol, ans, the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of ans is <v>x</v>, and the <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of ans is \u000A pkg, unless <v>x</v> is the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of some member of imports with \u000A <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> ipkg, in which case the <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of ans is ipkg.  Because <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> requires that \u000A there be no duplications among the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt>s of the imports, \u000A <v>intern-in-package-of-symbol</v> is uniquely defined.</p> \u000A \u000A <p>For example, suppose <v>&quot;MY-PKG&quot;</v> was created by</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;MY-PKG&quot; '(ACL2::ABC LISP::CAR)).</code> \u000A \u000A <p>Let <v>w</v> be <v>'my-pkg::witness</v>.  Observe that</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> w) is t                     ; w is a symbol\u000A(<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> w) is &quot;WITNESS&quot;         ; w's name is &quot;WITNESS&quot;\u000A(<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> w) is &quot;MY-PKG&quot;  ; w is in the package &quot;MY-PKG&quot;</code> \u000A \u000A <p>The construction of <v>w</v> illustrates one way to obtain a symbol in a \u000A given package: write it down as a constant using the double-colon \u000A notation.</p> \u000A \u000A <p>But another way to obtain a symbol in a given package is to create it with \u000A <v>intern-in-package-of-symbol</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;XYZ&quot; w) is MY-PKG::XYZ\u000A\u000A(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;ABC&quot; w) is ACL2::ABC\u000A\u000A(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;CAR&quot; w) is LISP::CAR\u000A\u000A(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> &quot;car&quot; w) is MY-PKG::|car|</code>"],

 "ACL2____IN-ARITHMETIC-THEORY":[["Events"],"ACL2 Sources","ACL2","<p>We assume familiarity with <see topic=\"ACL2____THEORIES\">theories</see>; in particular, see \u000A <see topic=\"ACL2____IN-THEORY\">in-theory</see> for the normal way to set the current theory.  Here, we \u000A discuss an analogous event that pertains only to non-linear arithmetic \u000A (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>).</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____IN-ARITHMETIC-THEORY\">in-arithmetic-theory</see> '(lemma1 lemma2))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____IN-ARITHMETIC-THEORY\">in-arithmetic-theory</see> term :doc doc-string)</code> \u000A \u000A <p>where <v>term</v> is a term that when evaluated will produce a theory (see \u000A <see topic=\"ACL2____THEORIES\">theories</see>), and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is an optional <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string not beginning with ``<v>:Doc-Section</v> ...''.  Except for the variable \u000A <tt><see topic=\"ACL2____WORLD\">world</see></tt>, <v>term</v> must contain no free variables.  <v>Term</v> is \u000A evaluated with the variable <tt><see topic=\"ACL2____WORLD\">world</see></tt> bound to the current <see topic=\"ACL2____WORLD\">world</see> to \u000A obtain a theory and the corresponding runic theory (see <see topic=\"ACL2____THEORIES\">theories</see>) is \u000A then used by non-linear arithmetic (see <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>).</p> \u000A \u000A <p>Warning: If <v>term</v> involves macros such as <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> you will probably not get what you expect!  Those macros are defined \u000A relative to the <tt><see topic=\"ACL2____CURRENT-THEORY\">current-theory</see></tt>.  But in this context you might wish \u000A they were defined in terms of the ``<v>CURRENT-ARITHMETIC-THEORY</v>'' which is \u000A not actually a defined function.  We do not anticipate that users will \u000A repeatedly modify the arithmetic theory.  We expect <v>term</v> most often to be \u000A a constant list of runes and so have not provided ``arithmetic theory \u000A manipulation functions'' analogous to <tt><see topic=\"ACL2____CURRENT-THEORY\">current-theory</see></tt> and <tt><see topic=\"ACL2____ENABLE\">enable</see></tt>.</p> \u000A \u000A <p>Because no unique name is associated with an <v>in-arithmetic-theory</v> \u000A event, there is no way we can store the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> in our il[documentation] database.  Hence, we actually prohibit \u000A <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> from having the form of an ACL2 <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string; see <see topic=\"ACL2____DOC-STRING\">doc-string</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>.</p>"],

 "ACL2____DEFCONG":[["Events"],"ACL2 Sources","ACL2","<p><v>Defcong</v> is used to prove that one <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relation \u000A preserves another in a given argument position of a given function.</p> \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> set-equal iff (memb x y) 2)\u000A\u000Ais an abbreviation for\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> set-equal-implies-iff-memb-2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (set-equal y y-equiv)\u000A           (<see topic=\"ACL2____IFF\">iff</see> (memb x y) (memb x y-equiv)))\u000A  :rule-classes (:congruence))</code> \u000A \u000A <p>See <see topic=\"ACL2____CONGRUENCE\">congruence</see> and also see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>.</p> \u000A \u000A <p>NOTE: <v>defcong</v> may only be used to create classic <see topic=\"ACL2____CONGRUENCE\">congruence</see> \u000A rules, not <see topic=\"ACL2____PATTERNED-CONGRUENCE\">patterned-congruence</see> rules.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFCONG\">defcong</see> equiv1 equiv2 term k\u000A  :rule-classes rule-classes\u000A  :instructions instructions\u000A  :hints hints\u000A  :otf-flg otf-flg\u000A  :event-name event-name\u000A  :doc doc)</code> \u000A \u000A <p>where <v>equiv1</v> and <v>equiv2</v> are known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations; \u000A <v>term</v> is a call of a function <v>fn</v>, other than <v>if</v>, on the correct \u000A number of distinct variable arguments, <v>(fn x1 ... xn)</v>; <v>k</v> is a \u000A positive integer less than or equal to the arity of <v>fn</v>; and other \u000A arguments are as specified in the documentation for <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  The \u000A <v>defcong</v> macro expands into a call of <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  The name of the \u000A <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event is <v>equiv1-implies-equiv2-fn-k</v> unless an \u000A <v>:event-name</v> keyword argument is supplied for the name.  The term of the \u000A theorem is</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 xk yk)\u000A         (equiv2 (fn x1... xk ...xn)\u000A                 (fn x1... yk ...xn))).</code> \u000A \u000A <p>The rule-class <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> is added to the <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> specified, if it is not already there.  All other arguments to \u000A the generated <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> form are as specified by the keyword arguments \u000A above.</p>"],

 "ACL2____USING-ENABLED-RULES":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Consider the following (admittedly silly) example.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z) (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z))\u000A     :hints ((&quot;Subgoal *1/1&quot; :use cdr-cons)))</code> \u000A \u000A <p>ACL2's output includes the following warning.</p> \u000A \u000A <code>ACL2 Warning [Use] in ( THM ...):  It is unusual to :USE an enabled\u000A:REWRITE or :DEFINITION rule, so you may want to consider disabling\u000A(:REWRITE CDR-CONS) in the hint provided for Subgoal *1/1.</code> \u000A \u000A <p>The warning is saying that if you leave the rewrite rule enabled, ACL2 may \u000A simplify away the hypothesis added by the <v>:use</v> hint.  We now explain this \u000A danger in more detail and show how disabling the rule can solve this \u000A problem.</p> \u000A \u000A <p>Recall (see <see topic=\"ACL2____HINTS\">hints</see>) how <v>:use</v> <see topic=\"ACL2____HINTS\">hints</see> work.  Such a hint \u000A specifies a formula, <v>F</v>, which is based on an existing lemma.  Then the \u000A indicated goal, <v>G</v>, is replaced by the implication <v>(<see topic=\"ACL2____IMPLIES\">implies</see> F G)</v>. \u000A The intention is that the truth of <v>F</v> will help in the simplification of \u000A <v>G</v> to <v>T</v> (true).  The ``[Use]'' warning shown above is telling us of \u000A the danger that <v>F</v> may be rewritten to <v>T</v>, reducing the implication \u000A above to <v>(<see topic=\"ACL2____IMPLIES\">implies</see> T G)</v> &mdash; thus, sadly, <v>F</v> has disappeared and is \u000A not available to help with the simplification of <v>G</v>.</p> \u000A \u000A <p>Consider the following tiny example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> p (x) (<see topic=\"COMMON-LISP____CONS\">cons</see> x x))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> car-p\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (p x)) x))\u000A(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> p (:type-prescription p)))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (p x1) (p x2))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x1 x2))\u000A     :hints ((&quot;Goal&quot;\u000A              :use ((:instance car-p (x x1))\u000A                    (:instance car-p (x x2))))))</code> \u000A \u000A <p>The proof of the final <tt><see topic=\"ACL2____THM\">thm</see></tt> form fails, because the new hypotheses \u000A are rewritten to <v>t</v> using the <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rule <v>CAR-P</v>, in \u000A the manner described above.  The following proof log shows the new hypotheses \u000A and their disappearance via rewriting.</p> \u000A \u000A <code>We augment the goal with the hypotheses provided by the :USE hint.\u000AThese hypotheses can be derived from CAR-P via instantiation.  We are\u000Aleft with the following subgoal.\u000A\u000AGoal'\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (P X1)) X1)\u000A              (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> (P X2)) X2))\u000A         (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (P X1) (P X2))\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X1 X2))).\u000A\u000ABy the simple :rewrite rule CAR-P we reduce the conjecture to\u000A\u000AGoal''\u000A(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (P X1) (P X2))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> X1 X2)).</code> \u000A \u000A <p>When we disable the rule <v>CAR-P</v> as follows, the proof succeeds.</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (p x1) (p x2))\u000A              (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x1 x2))\u000A     :hints ((&quot;Goal&quot;\u000A              :use ((:instance car-p (x x1))\u000A                    (:instance car-p (x x2)))\u000A              :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> car-p))))</code> \u000A \u000A <p>In general, then, a solution is to disable the rewrite rule that you are \u000A supplying in a <v>:use</v> hint.</p>"],

 "ACL2____SET-DIFFERENCE_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2)\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____SET-DIFFERENCE_42\">Set-difference$</see> l1 l2)</v> equals a list that contains the <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>s of <v>l1</v> that are not <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>s of <v>l2</v>.  More precisely, \u000A the resulting list is the same as one gets by deleting the members of <v>l2</v> \u000A from <v>l1</v>, leaving the remaining elements in the same order as in <v>l1</v>. \u000A The optional keyword, <v>:TEST</v>, has no effect logically, but provides the \u000A test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing members of the two lists.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>set-difference$</v> depends on the test. \u000A In all cases, both arguments must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If \u000A the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>set-difference$</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____SET-DIFFERENCE-EQ\">set-difference-eq</see> l1 l2)</v> is equivalent to \u000A <v>(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____SET-DIFFERENCE-EQUAL\">set-difference-equal</see> l1 l2)</v> is equivalent to <v>(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1\u000A l2 :test 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>set-difference-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>set-difference-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     set-difference-equal (l1 l2)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l1) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l2))))\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l1) nil)\u000A           ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1) l2)\u000A            (<see topic=\"ACL2____SET-DIFFERENCE-EQUAL\">set-difference-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2))\u000A           (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l1)\u000A                    (<see topic=\"ACL2____SET-DIFFERENCE-EQUAL\">set-difference-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l1) l2)))))</code> \u000A \u000A <p><v>Set-difference$</v> is similar to the Common Lisp primitive \u000A <v>set-difference</v>.  However, Common Lisp does not specify the order of \u000A elements in the result of a call of <v>set-difference</v>.</p>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Consider these definitions:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nats-below (j)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> j)\u000A      '(0)\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> j (nats-below (<see topic=\"COMMON-LISP____-\">-</see> j 1)))))</code> \u000A \u000A <p>We assume you are familiar with such ACL2 built-ins as <v>append</v>, \u000A <v>member</v>, <v>subsetp</v> and <v>true-listp</v>.  When we use throw-away names \u000A like <v>FOO</v>, <v>BAR</v>, and <v>MUM</v> below we mean to suggest some arbitrary \u000A function you shouldn't think about!  We're just trying to train your eye to \u000A ignore irrelevant things.</p> \u000A \u000A <p>Below are some terms that should suggest rewrite rules to you.  Imagine \u000A that each of these terms occurs in some Key Checkpoint.  What rules come to \u000A mind?  Try to think of the strongest rules you can.</p> \u000A \u000A <p>Term 1:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (FOO A) (BAR B)))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-1\">practice-formulating-strong-rules-1</see></p> \u000A \u000A <p>Term 2:<br></br> \u000A \u000A <v>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV (FOO A)))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-2\">practice-formulating-strong-rules-2</see></p> \u000A \u000A <p>Term 3:<br></br> \u000A \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (FOO A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (BAR B) (MUM C)))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-3\">practice-formulating-strong-rules-3</see></p> \u000A \u000A <p>Term 4:<br></br> \u000A \u000A <v>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (FOO A) (BAR B)) (MUM C))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-4\">practice-formulating-strong-rules-4</see></p> \u000A \u000A <p>Term 5:<br></br> \u000A \u000A <v>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (FOO A) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (BAR B) (MUM C)))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-5\">practice-formulating-strong-rules-5</see></p> \u000A \u000A <p>Term 6:<br></br> \u000A \u000A <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (FOO A) (NATS-BELOW (BAR B)))</v></p> \u000A \u000A <p>Answers: See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-6\">practice-formulating-strong-rules-6</see></p> \u000A \u000A <p>We recommend doing all of these little exercises.  When you're finished, \u000A use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>.</p>"],

 "ACL2____MV-LIST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A; Returns the list (3 4):\u000A(<see topic=\"ACL2____MV-LIST\">mv-list</see> 2 (<see topic=\"ACL2____MV\">mv</see> 3 4))\u000A\u000A; Returns a list containing the three values returned by var-fn-count:\u000A(<see topic=\"ACL2____MV-LIST\">mv-list</see> 3 (var-fn-count '(<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y) z) nil))\u000A\u000AGeneral form:\u000A(<see topic=\"ACL2____MV-LIST\">mv-list</see> n term)</code> \u000A \u000A <p>Logically, <v>(<see topic=\"ACL2____MV-LIST\">mv-list</see> n term)</v> is just <v>term</v>; that is, in the logic \u000A <v>mv-list</v> simply returns its second argument.  However, the evaluation of a \u000A call of <v>mv-list</v> on explicit values always results in a single value, \u000A which is a (null-terminated) list.  For evaluation, the term <v>n</v> above (the \u000A first argument to an <v>mv-list</v> call) must ``essentially'' (see below) be an \u000A integer not less than 2, where that integer is the number of values returned \u000A by the evaluation of <v>term</v> (the second argument to that <v>mv-list</v> \u000A call).</p> \u000A \u000A <p>We say ``essentially'' above because it suffices that the translation of \u000A <v>n</v> to a term (see <see topic=\"ACL2____TRANS\">trans</see>) be of the form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> k)</v>, where \u000A <v>k</v> is an integer greater than 1.  So for example, if <v>term</v> above \u000A returns three values, then <v>n</v> can be the expression <v>3</v>, or <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A 3)</v>, or even <v>(mac 3)</v> if <v>mac</v> is a macro defined by <v>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> mac\u000A (x) x)</v>.  But <v>n</v> cannot be <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 2)</v>, because even though that \u000A expression evaluates to <v>3</v>, nevertheless it translates to <v>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> '1\u000A '2)</v>, not to <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> 3)</v>.</p> \u000A \u000A <p><v>Mv-list</v> is the ACL2 analogue of the Common Lisp construct \u000A <v>multiple-value-list</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>mv-list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mv-list (input-arity x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t)\u000A                (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> input-arity))\u000A       x)</code>"],

 "ACL2____MACRO-ARGS":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A(x y z)\u000A(x y z &amp;optional max (base '10 basep))\u000A(x y &amp;rest rst)\u000A(x y &amp;key max base)\u000A(<see topic=\"COMMON-LISP_____62WHOLE\">&amp;whole</see> sexpr x y)</code> \u000A \u000A <p>The ``lambda-list'' of a macro definition may include simple formal \u000A parameter names as well as appropriate uses of the following <v>lambda</v>-list \u000A keywords from CLTL (pp. 60 and 145), respecting the order shown:</p> \u000A \u000A <code>&amp;whole,\u000A&amp;optional,\u000A&amp;rest,\u000A&amp;body,\u000A&amp;key, and\u000A&amp;allow-other-keys.</code> \u000A \u000A <p>ACL2 does not support <v>&amp;aux</v> and <v>&amp;environment</v>.  In addition, we \u000A make the following restrictions:</p> \u000A \u000A <blockquote> \u000A \u000A <p>(1) initialization forms in <v>&amp;optional</v> and <v>&amp;key</v> specifiers must be \u000A quoted values;</p> \u000A \u000A <p>(2) <v>&amp;allow-other-keys</v> may only be used once, as the last specifier; \u000A and</p> \u000A \u000A <p>(3) destructuring is not allowed.</p> \u000A \u000A </blockquote> \u000A \u000A <p>You are encouraged to experiment with the macro facility.  One way to do so \u000A is to define a macro that does nothing but return the quotation of its \u000A arguments, e.g.,</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> demo (x y &amp;optional opt &amp;key key1 key2)\u000A  (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____LIST\">list</see> x y opt key1 key2)))</code> \u000A \u000A <p>You may then execute the macro on some sample forms, e.g.,</p> \u000A \u000A <code>  term                         value\u000A(demo 1 2)                (1 2 NIL NIL NIL)\u000A(demo 1 2 3)              (1 2 3 NIL NIL)\u000A(demo 1 2 :key1 3)        error:  non-even key/value arglist\u000A                          (because :key1 is used as opt)\u000A(demo 1 2 3 :key2 5)      (1 2 3 NIL 5)</code> \u000A \u000A <p>In particular, Common Lisp specifies that if you use both <v>&amp;rest</v> and \u000A <v>&amp;key</v>, then both will be bound using the same list of arguments.  The \u000A following example should serve to illustrate how this works.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> foo (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args &amp;key k1 k2 k3)\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____LIST\">list</see> args k1 k2 k3)))\u000A\u000ASummary\u000AForm:  ( DEFMACRO FOO ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A FOO\u000AACL2 !&gt;(foo :k1 3 :k2 4 :k3 5)\u000A((:K1 3 :K2 4 :K3 5) 3 4 5)\u000AACL2 !&gt;(foo :k1 3 :k2 4)\u000A((:K1 3 :K2 4) 3 4 NIL)\u000AACL2 !&gt;(foo :k1 3 :bad-key 7)\u000A\u000AACL2 Error in macro expansion:  Illegal key/value args (:BAD-KEY 7)\u000Ain macro expansion of (FOO :K1 3 :BAD-KEY 7).  The argument list for\u000AFOO is\u000A(<see topic=\"COMMON-LISP_____62REST\">&amp;REST</see> ARGS &amp;KEY K1 K2 K3).\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p>If we don't want to get the error above, we can use <v>&amp;allow-other-keys</v>, \u000A as follows.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> bar (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args &amp;key k1 k2 k3\u000A                            &amp;allow-other-keys)\u000A         (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote (<see topic=\"COMMON-LISP____LIST\">list</see> args k1 k2 k3)))\u000A\u000ASummary\u000AForm:  ( DEFMACRO BAR ...)\u000ARules: NIL\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A BAR\u000AACL2 !&gt;(bar :k1 3 :bad-key 7)\u000A((:K1 3 :BAD-KEY 7) 3 NIL NIL)\u000AACL2 !&gt;</code> \u000A \u000A <p>Also see <see topic=\"ACL2____TRANS\">trans</see>.</p>"],

 "ACL2____ERROR-TRIPLES":[["State"],"ACL2 Sources","ACL2","<p>When evaluation returns three values, where the first two are \u000A ordinary \u000A (non-<see topic=\"ACL2____STOBJ\">stobj</see>) objects and the third is the ACL2 <see topic=\"ACL2____STATE\">state</see>, the result \u000A may be called an ``error triple''.  If an error triple is <v>(<see topic=\"ACL2____MV\">mv</see> erp val\u000A state)</v>, we think of <v>erp</v> as an error flag and <v>val</v> as the returned \u000A value.  By default, if the result of evaluating a top-level form is an error \u000A triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>, then that result is not printed if <v>erp</v> is \u000A non-<v>nil</v> or if <v>val</v> is the keyword <v>:INVISIBLE</v>, and otherwise \u000A <v>val</v> is printed with a preceding space.  For example:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) ; ordinary value\u000A7\u000AACL2 !&gt;(<see topic=\"ACL2____MV\">mv</see> nil (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) state) ; error triple, error component of nil\u000A 7\u000AACL2 !&gt;(<see topic=\"ACL2____MV\">mv</see> t (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) state) ; error triple, non-nil error component\u000AACL2 !&gt;(<see topic=\"ACL2____MV\">mv</see> nil :invisible state) ; special case for :INVISIBLE\u000AACL2 !&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see> for a discussion of error triples and how \u000A to program with them.  Also see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see> and see <see topic=\"ACL2____LD\">ld</see> for \u000A a discussion of the value <v>:COMMAND-CONVENTIONS</v> for keyword \u000A <v>:LD-POST-EVAL-PRINT</v>.</p>"],

 "ACL2____DIVE-INTO-MACROS-TABLE":[["Proof-checker","ACL2-pc::dv"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____DIVE-INTO-MACROS-TABLE\">dive-into-macros-table</see> (w state))\u000A((CAT . EXPAND-ADDRESS-CAT)\u000A (LXOR . EXPAND-ADDRESS-LXOR)</code> \u000A \u000A <p>This table associates macro names with functions used by the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>DV</v> and numeric diving commands (e.g., <v>3</v>) in order \u000A to dive properly into subterms.  See <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>, in particular the \u000A documentation for <v>DV</v>.</p> \u000A \u000A <p>This table can be extended easily.  See <see topic=\"ACL2____ADD-DIVE-INTO-MACRO\">add-dive-into-macro</see> and also \u000A see <see topic=\"ACL2____REMOVE-DIVE-INTO-MACRO\">remove-dive-into-macro</see>.</p> \u000A \u000A <p>The symbol associated with a macro should be a function symbol taking four \u000A arguments, in this order:</p> \u000A \u000A <ul> \u000A <li><v>car-addr</v><br/> \u000A     <blockquote>the first number in the list given to the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>DV</v> command</blockquote></li> \u000A <li><v>raw-term</v><br/> \u000A     <blockquote>the untranslated term into which we will \u000A     dive</blockquote></li> \u000A <li><v>term</v> \u000A     <blockquote>the translated term into which we will dive</blockquote></li> \u000A <li><v>wrld</v> \u000A     <blockquote>the current ACL2 logical <see topic=\"ACL2____WORLD\">world</see></blockquote></li> \u000A </ul> \u000A \u000A <p>The function will normally return a list of positive integers, representing \u000A the (one-based) address for diving into <v>term</v> that corresponds to the \u000A single-address dive into <v>raw-term</v> by <v>car-address</v>.  However, it can \u000A return <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> str alist)</v>, where <v>str</v> is a string suitable for <tt><see topic=\"ACL2____FMT\">fmt</see></tt> and <v>args</v> is the corresponding alist for <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.</p> \u000A \u000A <p>Referring to the example above, <v>expand-address-cat</v> would be such a \u000A function, which will be called on <v>raw-term</v> values that are calls of \u000A <v>cat</v>.  See the community book <v>books/misc/rtl-untranslate.lisp</v> for the \u000A definition of such a function.</p> \u000A \u000A <p>See <see topic=\"ACL2____TABLE\">table</see> for a general discussion of tables.</p>"],

 "ACL2____COMPRESS2":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____COMPRESS2\">compress2</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____COMPRESS2\">compress2</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is a symbol and <v>alist</v> is a 2-dimensional array, \u000A generally named <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.  Logically \u000A speaking, this function removes irrelevant pairs from <v>alist</v>, possibly \u000A shortening it.  The function returns a new array, <v>alist'</v>, with the same \u000A <tt><see topic=\"ACL2____HEADER\">header</see></tt> (including name and dimension) as <v>alist</v>, that, under \u000A <tt><see topic=\"ACL2____AREF2\">aref2</see></tt>, is everywhere equal to <v>alist</v>.  That is, <v>(<see topic=\"ACL2____AREF2\">aref2</see> name\u000A alist' i j)</v> is <v>(<see topic=\"ACL2____AREF2\">aref2</see> name alist i j)</v>, for all legal indices <v>i</v> and \u000A <v>j</v>.  <v>Alist'</v> may be shorter than <v>alist</v> and the non-irrelevant \u000A pairs may occur in a different order in <v>alist'</v> than in <v>alist</v>.</p> \u000A \u000A <p>Practically speaking, this function plays an important role in the \u000A efficient implementation of <tt><see topic=\"ACL2____AREF2\">aref2</see></tt>.  In addition to creating the new \u000A array, <v>alist'</v>, <v>compress2</v> makes that array the ``semantic value'' of \u000A <v>name</v> and allocates a raw lisp array to <v>name</v>.  For all legal indices, \u000A <v>i</v> and <v>j</v>, that raw lisp array contains <v>(<see topic=\"ACL2____AREF2\">aref2</see> name alist' i j)</v> \u000A in slot <v>i</v>,<v>j</v>.  Thus, subsequent <tt><see topic=\"ACL2____AREF2\">aref2</see></tt> operations can be \u000A executed in virtually constant time provided they are given <v>name</v> and the \u000A <v>alist'</v> returned by the most recently executed <v>compress2</v> or <tt><see topic=\"ACL2____ASET2\">aset2</see></tt> on <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>compress2</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> compress2 (<see topic=\"ACL2____NAME\">name</see> l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l)))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____HEADER\">header</see> name l)\u000A             (compress21 name l 0 (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))\u000A                         (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))\u000A                         (<see topic=\"ACL2____DEFAULT\">default</see> name l))))</code>"],

 "ACL2-PC____EQUIV":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(equiv (<see topic=\"COMMON-LISP_____A2\">*</see> x y) 3) -- replace (<see topic=\"COMMON-LISP_____A2\">*</see> x y) by 3 everywhere inside the\u000A                     current subterm, if their equality is among the\u000A                     top-level hypotheses or the governors\u000A(equiv x t iff)   -- replace x by t everywhere inside the current\u000A                     subterm, where only propositional equivalence\u000A                     needs to be maintained at each occurrence of x\u000A\u000AGeneral form:\u000A(equiv old new &amp;optional relation)</code> \u000A \u000A <p>Substitute new for old everywhere inside the current subterm, provided that \u000A either (relation old new) or (relation new old) is among the top-level \u000A hypotheses or the governors (possibly by way of backchaining and/or \u000A refinement; see below).  If relation is <v>nil</v> or is not supplied, then it \u000A defaults to <v>equal</v>.  See also the command <v>=</v>, which is much more \u000A flexible.  Note that this command fails if no substitution is actually \u000A made.</p> \u000A \u000A <p><b>Remark:</b> No substitution takes place inside explicit values.  So for \u000A example, the instruction <v>(equiv 3 x)</v> will cause <v>3</v> to be replaced by \u000A <v>x</v> if the current subterm is, say, <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 3 y)</v>, but not if the current \u000A subterm is <v>(<see topic=\"COMMON-LISP_____A2\">*</see> 4 y)</v> even though <v>4 = (<see topic=\"COMMON-LISP____1_B2\">1+</see> 3)</v>.</p> \u000A \u000A <p>The following remarks are quite technical and mostly describe a certain \u000A weak form of ``backchaining'' that has been implemented for <v>equiv</v> in \u000A order to support the <v>=</v> command.  In fact neither the term <v>(relation\u000A old new)</v> nor the term <v>(relation new old)</v> needs to be <b>explicitly</b> \u000A among the current ``assumptions'', i.e., the top-level hypothesis or the \u000A governors.  Rather, there need only be such an assumption that ``tells us'' \u000A <v>(r old new)</v> or <v>(r new old)</v>, for <b>some</b> equivalence relation \u000A <v>r</v> that <b>refines</b> <v>relation</v>.  Here, ``tells us'' means that \u000A either one of the indicated terms is among those assumptions, or else there is \u000A an assumption that is an implication whose conclusion is one of the indicated \u000A terms and whose hypotheses (gathered up by appropriately flattening the first \u000A argument of the <v>implies</v> term) are all among the current \u000A assumptions.</p>"],

 "ACL2____TTAGS-SEEN":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>General Forms:\u000A:ttags-seen\u000A(<see topic=\"ACL2____TTAGS-SEEN\">ttags-seen</see>)</code> \u000A \u000A <p>Suppose the output is as follows.</p> \u000A \u000A <code>(T NIL)\u000A(FOO &quot;/home/bob/bar.lisp&quot;\u000A     &quot;/home/cindy/bar.lisp&quot;)\u000AWarning: This output is minimally trustworthy (see :DOC TTAGS-SEEN).</code> \u000A \u000A <p>This output indicates that the current logical <see topic=\"ACL2____WORLD\">world</see> has seen the \u000A declaration of trust tag <v>T</v> at the top-level (see <see topic=\"ACL2____DEFTTAG\">defttag</see>) and the \u000A declaration of trust tag <v>FOO</v> in the two books included from the listed \u000A locations.  The warning emphasizes that this command cannot be used to \u000A validate the ``purity'' of an ACL2 session, because using a ttag renders \u000A enough power to hide from this or any other command the fact that the ttag was \u000A ever declared.</p> \u000A \u000A <p>As discussed elsewhere (see <see topic=\"ACL2____DEFTTAG\">defttag</see>), the only reliable way to \u000A validate the ``purity'' of a session is to watch for ``<v>TTAG NOTE</v>'' \u000A output.</p> \u000A \u000A <p>Another shortcoming of this command is that it only checks the current \u000A logical <see topic=\"ACL2____WORLD\">world</see> for ttag declarations.  For example, one could execute a \u000A <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt> event; then use <tt><see topic=\"ACL2____PROGN_12\">progn!</see></tt> and <tt><see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see></tt> to \u000A replace system functions with corrupt definitions or to introduce inconsistent \u000A axioms in the <tt><see topic=\"ACL2____GROUND-ZERO\">ground-zero</see></tt> <see topic=\"ACL2____WORLD\">world</see>; and finally, execute \u000A <v>:</v><tt><see topic=\"ACL2____UBT_12\">ubt!</see></tt><v> 1</v> to remove all evidence of the ttag in the <see topic=\"ACL2____WORLD\">world</see> while leaving in place the corrupt definitions or axioms.  The base \u000A world is now tainted, meaning we could prove <v>nil</v> or certify a book that \u000A proves <v>nil</v>, but the resulting session or book would contain no trace of \u000A the ttag that tainted it!</p> \u000A \u000A <p>Despite shortcomings, this command might be useful to system hackers.  It \u000A also serves to illustrate the inherent flaw in asking a session whether or how \u000A it is ``tainted'', justifying the ``<v>TTAG NOTE</v>'' approach (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).</p>"],

 "ACL2____THM":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app a b) c)\u000A            (app a (app b c))))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____DEFTHM\">defthm</see>.  Unlike <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, <v>thm</v> does not create an \u000A event; it merely causes the theorem prover to attempt a proof.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____THM\">thm</see> term\u000A     :hints        hints\u000A     :otf-flg      otf-flg\u000A     :doc          doc-string)</code> \u000A \u000A <p>where <v>term</v> is a term alleged to be a theorem, and <tt><see topic=\"ACL2____HINTS\">hints</see></tt>, \u000A <tt><see topic=\"ACL2____OTF-FLG\">otf-flg</see></tt> and <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> are as described in the corresponding \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> entries.  The three keyword arguments above are all \u000A optional.</p>"],

 "ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR\">set-total-parallelism-work-limit-error</see> t)   ; cause error (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR\">set-total-parallelism-work-limit-error</see> nil) ; disable error and\u000A                                             ; continue serially</code> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see>, Section ``Another Granularity Issue \u000A Related to Thread Limitations'', for an explanation of how the host Lisp can \u000A run out of threads.  See <see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> for further \u000A details, including an explanation of how to manage the limit that triggers \u000A this error.</p> \u000A \u000A <p>The value of state global <v>total-parallelism-work-limit-error</v> dictates \u000A what occurs when the underlying runtime system runs reaches a limit on the \u000A number of threads for parallel computation.  By default, when this limit is \u000A reached, the ACL2(p) user will receive an error and computation will halt.  At \u000A this point, the ACL2(p) user has the following options.</p> \u000A \u000A <p>(1) Remove the limit by evaluating the following form.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> :none)</code> \u000A \u000A <p>(2) Disable the error so that execution continues serially once the \u000A available thread resources have been exhausted.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR\">set-total-parallelism-work-limit-error</see> nil)</code> \u000A \u000A <p>(3) Increase the limit on number of threads that ACL2(p) is willing to \u000A create, in spite of potential risk (see <see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see>).  In this case, the following query returns \u000A the current limit.</p> \u000A \u000A <code>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'total-parallelism-work-limit)</code> \u000A \u000A <p>Then to increase that limit, evaluate the following form:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> &lt;new-integer-value&gt;)</code> \u000A \u000A <p>For example, suppose that the value of <v>total-parallelism-work-limit</v> \u000A was originally 10,000.  Then evaluation of the following form increases that \u000A limit to 13,000.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see> 13000)</code>"],

 "ACL2____SET-INVISIBLE-FNS-TABLE":[["Loop-stopper"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see> ((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> unary--)\u000A                          (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> unary-/)\u000A                          (<see topic=\"ACL2____UNARY--\">unary--</see> unary--)\u000A                          (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> unary-/)))\u000A(<see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see> t) ; restore original invisible-fns-table</code> \u000A \u000A <p>Among other things, the setting above has the effect of making <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> ``invisible'' for the purposes of applying permutative <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> trees.  Thus, <v>arg</v> and <v>(<see topic=\"ACL2____UNARY--\">unary--</see>\u000A arg)</v> will be given the same weight and will be permuted so as to be \u000A adjacent.  The form <v>(<see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see> (w state))</v> returns the current \u000A value of the invisible functions table.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> and see <see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see> for \u000A events that add to and remove from the invisible functions table, while \u000A accounting for macro aliases (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see> alist)</code> \u000A \u000A <p>where <v>alist</v> is either <v>t</v> or a true list of pairs, each element of \u000A which is of the form <v>(fn ufn1 ... ufnk)</v>, where <v>fn</v> is a function \u000A symbol and each <v>ufni</v> is a unary function symbol.  When alist is <v>t</v>, \u000A the initial value of this table is used in its place.  Modulo the replacement \u000A of <v>alist</v> by the default setting when <v>alist</v> is <v>t</v>, this macro is \u000A equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> invisible-fns-table nil 'alist :clear)</code> \u000A \u000A <p>which is also an event (see <see topic=\"ACL2____TABLE\">table</see>).</p> \u000A \u000A <p>Note that <v>set-invisible-fns-table</v> does not evaluate its argument. \u000A However, you can call <tt><see topic=\"ACL2____TABLE\">table</see></tt> directly for that purpose.  For \u000A example,</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see> ((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> unary--)\u000A                          (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> unary-/)\u000A                          (<see topic=\"ACL2____UNARY--\">unary--</see> unary--)\u000A                          (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> unary-/)))</code> \u000A \u000A <p>ie equivalent to the following; see <see topic=\"ACL2____TABLE\">table</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> invisible-fns-table nil\u000A       (<see topic=\"COMMON-LISP____QUOTE\">quote</see> ((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> unary--)\u000A               (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> unary-/)\u000A               (<see topic=\"ACL2____UNARY--\">unary--</see> unary--)\u000A               (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> unary-/)))\u000A       :clear)</code> \u000A \u000A <p>See <see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see> for a description of the invisible functions \u000A table.</p>"],

 "ACL2____PARGS":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 1)) (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A\u000A(<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 35)))\u000A       (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 1)) (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____PARGS\">pargs</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> expr)) ; optional granularity declaration\u000A       (f arg1 ... argN))</code> \u000A \u000A <p>where <v>N &gt;= 0</v> and each <v>argi</v> and <v>expr</v> are arbitrary terms. \u000A Logically, <v>pargs</v> is just the identity macro, in the sense that the above \u000A forms can logically be replaced by <v>(f arg1 ... argN)</v>.  However, this \u000A <v>pargs</v> form may parallelize the evaluation of arguments <v>arg1</v> through \u000A <v>argN</v> before applying function <v>f</v> to those results.  If the above \u000A <tt><see topic=\"ACL2____GRANULARITY\">granularity</see></tt> declaration is present, then its expression (<v>expr</v> \u000A above) is first evaluated, and if the result is <v>nil</v>, then such \u000A parallelism is avoided.  Even if parallelism is not thus avoided, parallelism \u000A may be limited by available resources.</p> \u000A \u000A <p>Since macros can change expressions in unexpected ways, it is illegal to \u000A call <v>pargs</v> on a form that is a macro call.  To parallelize computation of \u000A arguments to a macro, see <see topic=\"ACL2____PLET\">plet</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-AT-THE-TOP-LEVEL\">parallelism-at-the-top-level</see> for restrictions on evaluating \u000A parallelism primitives from within the ACL2 top-level loop.</p>"],

 "ACL2____PACKAGE-REINCARNATION-IMPORT-RESTRICTIONS":[["Defpkg"],"ACL2 Sources","ACL2","<p>Suppose <v>(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;pkg&quot; imports)</v> is the most recently executed \u000A successful definition of <v>&quot;pkg&quot;</v> in this ACL2 session and that it has \u000A since been undone, as by <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>.  Any future attempt in this \u000A session to define <v>&quot;pkg&quot;</v> as a package must specify an identical imports \u000A list.</p> \u000A \u000A <p>The restriction stems from the need to implement the reinstallation of \u000A saved logical <see topic=\"ACL2____WORLD\">world</see>s as in error recovery and the <v>:</v><tt><see topic=\"ACL2____OOPS\">oops</see></tt> \u000A <see topic=\"ACL2____COMMAND\">command</see>.  Suppose that the new <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> attempts to import some \u000A symbol, <v>a::sym</v>, not imported by the previous definition of <v>&quot;pkg&quot;</v>. \u000A Because it was not imported in the original package, the symbol <v>pkg::sym</v>, \u000A different from <v>a::sym</v>, may well have been created and may well be used in \u000A some saved <see topic=\"ACL2____WORLD\">world</see>s.  Those saved <see topic=\"ACL2____WORLD\">world</see>s are Common Lisp objects \u000A being held for you ``behind the scenes.''  In order to import <v>a::sym</v> into \u000A <v>&quot;pkg&quot;</v> now we would have to unintern <v>pkg::sym</v>, rendering those \u000A saved <see topic=\"ACL2____WORLD\">world</see>s ill-formed.  It is because of saved <see topic=\"ACL2____WORLD\">world</see>s that we \u000A do not actually clear out a package when it is undone.</p> \u000A \u000A <p>At one point we thought it was sound to allow the new <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt> to \u000A import a subset of the old.  But that is incorrect.  Suppose the old \u000A definition of <v>&quot;pkg&quot;</v> imported <v>a::sym</v> but the new one does not. \u000A Suppose we allowed that and implemented it simply by setting the imports of \u000A <v>&quot;pkg&quot;</v> to the new subset.  Then consider the conjecture <v>(<see topic=\"COMMON-LISP____EQ\">eq</see> a::sym\u000A pkg::sym)</v>.  This ought not be a theorem because we did not import \u000A <v>a::sym</v> into <v>&quot;pkg&quot;</v>.  But in fact in AKCL it is a theorem because \u000A <v>pkg::sym</v> is read as <v>a::sym</v> because of the old imports.</p>"],

 "ACL2____DEFUND":[["Defun","Events"],"ACL2 Sources","ACL2","<p>Use <v>defund</v> instead of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> when you want to disable a \u000A function immediately after its definition in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode.  This \u000A macro has been provided for users who prefer working in a mode where functions \u000A are only enabled when explicitly directed by <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>. \u000A Specifically, the form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFUND\">defund</see> NAME FORMALS ...)</code> \u000A \u000A <p>expands to:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A  (<see topic=\"COMMON-LISP____DEFUN\">defun</see> NAME FORMALS ...)\u000A  (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A   :off summary\u000A   (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> NAME)))\u000A  (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> '(:defund NAME))).</code> \u000A \u000A <p>Only the <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rule (and, for recursively defined \u000A functions, the <v>:</v><tt><see topic=\"ACL2____INDUCTION\">induction</see></tt> rule) for the function are disabled. \u000A In particular, <v>defund</v> does not disable either the <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> or the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> rule.  Also, \u000A the summary for the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event is suppressed.</p> \u000A \u000A <p>If the function is defined in <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> mode, either because \u000A the <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt> or because <v>:mode\u000A :program</v> has been specified in an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> form of a <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> \u000A form, then no <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event is executed.  (More precisely, <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> events are ignored when the <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> is \u000A <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, and if <v>:mode :program</v> is specified then \u000A <v>defund</v> does not generate an <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event.)</p> \u000A \u000A <p>Note that <v>defund</v> commands are never redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>) when the <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> is <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>, \u000A because the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event will always be executed.</p> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DEFUN\">defun</see> for documentation of <v>defun</v>.</p>"],

 "ACL2____CANONICAL-PATHNAME":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For the name <v>fname</v> of a file, the form <v>(<see topic=\"ACL2____CANONICAL-PATHNAME\">Canonical-pathname</see>\u000A fname nil state)</v> evaluates to a Unix-style absolute filename representing \u000A the same file as <v>fname</v>, but generally without any use of soft links in \u000A the name.  (Below, we explain the qualifier ``generally''.)  If however the \u000A file indicated by <v>fname</v> does not exist, <v>(<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> fname nil\u000A state)</v> is <v>nil</v>.  Thus, <v>canonical-pathname</v> can be used as one would \u000A use the raw Lisp function <v>probe-file</v>.</p> \u000A \u000A <p>The specification of <v>(<see topic=\"ACL2____CANONICAL-PATHNAME\">Canonical-pathname</see> fname dir-p state)</v> when \u000A <v>dir-p</v> is not <v>nil</v> is simlar, except that if the specified file exists \u000A but is not a directory, then the result is <v>nil</v>.</p> \u000A \u000A <p>The function <v>canonical-pathname</v> has a guard of <v>t</v>, though the \u000A second argument must be the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  This function is introduced \u000A with the following properties.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> canonical-pathname-is-idempotent\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> (<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> x dir-p state) dir-p state)\u000A         (<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> x dir-p state)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> canonical-pathname-type\u000A  (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> x dir-p state) nil)\u000A      (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"ACL2____CANONICAL-PATHNAME\">canonical-pathname</see> x dir-p state)))\u000A  :rule-classes :type-prescription)</code> \u000A \u000A <p>We use the qualifier ``generally'', above, because there is no guarantee \u000A that the filename will be canonical without soft links, though we expect this \u000A to be true in practice.  ACL2 attempts to compute the desired result and then \u000A checks that the input and result have the same Common Lisp ``<v>truename</v>''. \u000A This check is expected to succeed, but if it fails then the input string is \u000A returned unchanged, and to be conservative, the value returned is <v>nil</v> in \u000A this case if <v>dir-p</v> is true.</p>"],

 "ACL2____An_02Example_02of_02ACL2_02in_02Use":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>To introduce you to ACL2 we will consider the <v>app</v> function discussed \u000A in the <see topic='ACL2____Common_02Lisp'>Common Lisp</see> page, <b>except</b> \u000A we will omit for the moment the <b>declare</b> form, which in ACL2 is called a \u000A <b>guard</b>.</p> \u000A \u000A <p>Guards are arbitrary ACL2 terms that express the ``intended domain'' of \u000A functions.  In that sense, guards are akin to type signatures.  However, \u000A Common Lisp and ACL2 are untyped programming languages: while the language \u000A supports several different data types and the types of objects can be \u000A determined by predicates at runtime, any type of object may be passed to any \u000A function.  Thus, guards are ``extra-logical.''  Recognizing both the practical \u000A and intellectual value of knowing that your functions are applied to the kinds \u000A of objects you intend, ACL2 imposes guards on Common Lisp and provides a means \u000A of proving that functions are used as intended.  But the story is necessarily \u000A complicated and we do not recommend it to the new user.  Get used to the fact \u000A that any ACL2 function may be applied to any objects and program accordingly. \u000A Read about guards later.</p> \u000A \u000A <p>Here is the definition again</p> \u000A \u000A <code><b>(defun app (x y)</b> \u000A  <b>(cond ((endp x) y)</b> \u000A        <b>(t (cons (car x) </b> \u000A                 <b>(app (cdr x) y)))))</b> \u000A</code> \u000A \u000A <p>The next few stops along the Walking Tour will show you</p> \u000A \u000A <code> * how to use the ACL2 documentation, * what happens when the above \u000Adefinition is submitted to ACL2, * what happens when you evaluate calls of \u000A<v>app</v>, * what one simple theorem about <v>app</v> looks like, * how ACL2 \u000Aproves the theorem, and * how that theorem can be used in another proof. \u000A</code> \u000A \u000A <p>Along the way we will talk about the <b>definitional principle</b>, \u000A <b>types</b>, the ACL2 <b>read-eval-print loop</b>, and how the <b>theorem \u000A prover</b> works.</p> \u000A \u000A <p>When we complete this part of the tour we will return briefly to the notion \u000A of <b>guards</b> and revisit several of the topics above in that context.</p> \u000A \u000A <p><see topic='ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____ASET2":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____ASET2\">aset2</see> 'delta1 a i j 27)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ASET2\">aset2</see> name alist i j val)</code> \u000A \u000A <p>where <v>name</v> is a symbol, <v>alist</v> is a 2-dimensional array named \u000A <v>name</v>, <v>i</v> and <v>j</v> are legal indices into <v>alist</v>, and <v>val</v> \u000A is an arbitrary object.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.  Roughly speaking this \u000A function ``modifies'' <v>alist</v> so that the value associated with <v>(i\u000A . j)</v> is <v>val</v>.  More precisely, it returns a new array, <v>alist'</v>, of \u000A the same name and dimension as <v>alist</v> that, under <tt><see topic=\"ACL2____AREF2\">aref2</see></tt>, is \u000A everywhere equal to <v>alist</v> except at <v>(i . j)</v> where the result is \u000A <v>val</v>.  That is, <v>(<see topic=\"ACL2____AREF2\">aref2</see> name alist' x y)</v> is <v>(<see topic=\"ACL2____AREF2\">aref2</see> name alist x\u000A y)</v> for all legal indices <v>x</v> <v>y</v> except <v>i</v> and <v>j</v> where \u000A <v>(<see topic=\"ACL2____AREF2\">aref2</see> name alist' i j)</v> is <v>val</v>.</p> \u000A \u000A <p>In order to ``modify'' <v>alist</v>, <v>aset2</v> <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>es a new pair \u000A onto the front.  If the length of the resulting <v>alist</v> exceeds the \u000A <v>:</v><tt><see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see></tt> entry in the array <see topic=\"ACL2____HEADER\">header</see>, <v>aset2</v> \u000A compresses the array as with <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt>.</p> \u000A \u000A <p>It is generally expected that the ``semantic value'' of <v>name</v> will be \u000A <v>alist</v> (see <see topic=\"ACL2____ARRAYS\">arrays</see>).  This function operates in virtually constant \u000A time whether this condition is true or not (unless the <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> \u000A operation is required).  But the value returned by this function cannot be \u000A used efficiently by subsequent <v>aset2</v> operations unless <v>alist</v> is the \u000A semantic value of <v>name</v> when <v>aset2</v> is executed.  Thus, if the \u000A condition is not true, <v>aset2</v> prints a <b>slow array</b> warning to the \u000A comment window.  See <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>aset2</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     aset2 (<see topic=\"ACL2____NAME\">name</see> l i j val)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l)\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l)))\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> j 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> j (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))))))\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((l (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> i j) val) l)))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> name l))\u000A                 (<see topic=\"ACL2____COMPRESS2\">compress2</see> name l))\u000A                (t l))))</code>"],

 "ACL2____SET-BOGUS-MUTUAL-RECURSION-OK":[["Mutual-recursion"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see> t)\u000A(<see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see> nil)\u000A(<see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see> :warn)</code> \u000A \u000A <p>By default, ACL2 checks that when a ``clique'' of more than one function is \u000A defined simultaneously (using <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt>), \u000A then every body calls at least one of the functions in the ``clique.''  Below, \u000A we refer to definitional events that fail this check as ``bogus'' mutual \u000A recursions.  The check is important because ACL2 does not store induction \u000A schemes for functions defined with other functions in a <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> or <tt><see topic=\"ACL2____DEFUNS\">defuns</see></tt> event.  Thus, ACL2 may have difficulty \u000A proving theorems by induction that involve such functions.  Moreover, the \u000A check can call attention to bugs, since users generally intend that their \u000A mutual recursions are not bogus.</p> \u000A \u000A <p>Nevertheless, there are times when it is advantageous to allow bogus mutual \u000A recursions, for example when they are generated mechanically, even at the \u000A expense of losing stored induction schemes.  The first example above allows \u000A bogus mutual recursion.  The second example disallows bogus mutual recursion; \u000A this is the default.  The third example allows bogus mutual recursion, but \u000A prints an appropriate warning.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-BOGUS-MUTUAL-RECURSION-OK\">set-bogus-mutual-recursion-ok</see> flg)</code> \u000A \u000A <p>where <v>flg</v> is either <v>t</v>, <v>nil</v>, or <v>:warn</v>.</p>"],

 "ACL2____OUTPUT-TO-FILE":[["Io"],"ACL2 Sources","ACL2","<p>For a general discussion of ACL2 input/output and of the ACL2 \u000A read-eval-print loop, see <see topic=\"ACL2____IO\">io</see> and see <see topic=\"ACL2____LD\">ld</see> (respectively).  Here we \u000A use an example to illustrate how to use some of the options provided by \u000A <v>ld</v> to redirect ACL2 output to a file, other than the printing of the \u000A prompt (which continues to go to the terminal).</p> \u000A \u000A <p>There are two <v>ld</v> specials that control output from the <v>ld</v> \u000A command: <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt> for proof output and <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> for other \u000A output.  The following example shows how to use these to redirect output to a \u000A file <v>&quot;tmp.out&quot;</v>.  The following command opens a character output channel \u000A to to the file <v>&quot;tmp.out&quot;</v> and redirects proof output to that channel, \u000A i.e., to file <v>&quot;tmp.out&quot;</v>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (chan state)\u000A        (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see> &quot;tmp.out&quot; :character state)\u000A        (set-proofs-co chan state))</code> \u000A \u000A <p>Next, we redirect standard output to that same channel.</p> \u000A \u000A <code>(set-standard-co (<see topic=\"ACL2____PROOFS-CO\">proofs-co</see> state) state)</code> \u000A \u000A <p>Now we can load an input file, in this case file <v>&quot;tmp.lisp&quot;</v>, and \u000A output will be redirected to file <v>&quot;tmp.out&quot;</v>.  (The use of \u000A <v>:ld-pre-eval-print t</v> is optional; see <see topic=\"ACL2____LD\">ld</see>.)</p> \u000A \u000A <code>(<see topic=\"ACL2____LD\">ld</see> &quot;tmp.lisp&quot; :ld-pre-eval-print t)</code> \u000A \u000A <p>Having completed our load operation, we restore both proof output and \u000A standard output to the terminal, as follows.</p> \u000A \u000A <code>(set-standard-co *standard-co* state)\u000A(<see topic=\"ACL2____CLOSE-OUTPUT-CHANNEL\">close-output-channel</see> (<see topic=\"ACL2____PROOFS-CO\">proofs-co</see> state) state)\u000A(set-proofs-co *standard-co* state)</code> \u000A \u000A <p>The following variant of the above example shows how to redirect output as \u000A above except without changing the global settings of the two <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A specials, <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt> and <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  This approach uses a \u000A notion of ``global variables'' stored in the ACL2 <see topic=\"ACL2____STATE\">state</see>; see <see topic=\"ACL2____ASSIGN\">assign</see> and see <see topic=\"ACL2_____04\">@</see>.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (chan state)\u000A        (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see> &quot;tmp.out&quot; :character state)\u000A        (<see topic=\"ACL2____ASSIGN\">assign</see> tmp-channel chan))\u000A(<see topic=\"ACL2____LD\">ld</see> &quot;tmp.lisp&quot; :ld-pre-eval-print t\u000A               :proofs-co (<see topic=\"ACL2_____04\">@</see> tmp-channel)\u000A               :standard-co (<see topic=\"ACL2_____04\">@</see> tmp-channel))\u000A(<see topic=\"ACL2____CLOSE-OUTPUT-CHANNEL\">close-output-channel</see> (<see topic=\"ACL2_____04\">@</see> tmp-channel) state)</code>"],

 "ACL2____Guards":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Common Lisp functions are partial; they are not defined for all \u000A possible inputs.  But ACL2 functions are total.  Roughly speaking, the logical \u000A function of a given name in ACL2 is a <b>completion</b> of the Common Lisp \u000A function of the same name obtained by adding some arbitrary but ``natural'' \u000A values on arguments outside the ``intended domain'' of the Common Lisp \u000A function.</p> \u000A \u000A <p>ACL2 requires that every ACL2 function symbol have a ``guard,'' which may \u000A be thought of as a predicate on the formals of the function describing the \u000A intended domain.  The guard on the primitive function <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A for example, is <v>(<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))</v>, which requires the argument \u000A to be either an ordered pair or <v>nil</v>.  We will discuss later how to \u000A specify a guard for a defined function; when one is not specified, the guard \u000A is <v>t</v> which is just to say all arguments are allowed.</p> \u000A \u000A <p><b>But guards are entirely extra-logical</b>: they are not involved in the \u000A axioms defining functions.  If you put a guard on a defined function, the \u000A defining axiom added to the logic defines the function on <b>all</b> \u000A arguments, not just on the guarded domain.</p> \u000A \u000A <p>So what is the purpose of guards?</p> \u000A \u000A <p>The key to the utility of guards is that we provide a mechanism, called \u000A ``guard verification,'' for checking that all the guards in a formula are \u000A true.  See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see>.  This mechanism will attempt to prove that \u000A all the guards encountered in the evaluation of a guarded function are true \u000A every time they are encountered.</p> \u000A \u000A <p>For a thorough discussion of guards, see the paper [km97] in the ACL2 <see topic=\"ACL2____BIBLIOGRAPHY\">bibliography</see>.</p>"],

 "ACL2____FULL-BOOK-NAME":[["Books-reference"],"ACL2 Sources","ACL2","<p>For this discussion we assume that the resident operating system is \u000A Unix (trademark of AT&amp;T), but analogous remarks apply to other operating \u000A systems supported by ACL2, in particular, the Macintosh operating system where \u000A `<v>:</v>' plays roughly the role of `<v>/</v>' in Unix; see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>.</p> \u000A \u000A <p>ACL2 defines a ``full book name'' to be an ``absolute filename string,'' \u000A that may be divided into contiguous sections: a ``directory string'', a \u000A ``familiar name'' and an ``extension''.  See <see topic=\"COMMON-LISP____PATHNAME\">pathname</see> for the \u000A definitions of ``absolute,'' ``filename string,'' and other notions pertaining \u000A to naming files.  Below we exhibit the three sections of one such string:</p> \u000A \u000A <code>&quot;/usr/home/smith/project/arith.lisp&quot;\u000A\u000A&quot;/usr/home/smith/project/&quot;           ; directory string\u000A                        &quot;arith&quot;      ; familiar name\u000A                             &quot;.lisp&quot; ; extension</code> \u000A \u000A <p>The sections are marked by the rightmost slash and rightmost dot, as shown \u000A below.</p> \u000A \u000A <code>&quot;/usr/home/smith/project/arith.lisp&quot;\u000A                        |     |\u000A                        slash dot\u000A                        |     |\u000A&quot;/usr/home/smith/project/&quot;           ; directory string\u000A                        &quot;arith&quot;      ; familiar name\u000A                             &quot;.lisp&quot; ; extension</code> \u000A \u000A <p>The directory string includes (and terminates with) the rightmost slash. \u000A The extension includes (and starts with) the rightmost dot.  The dot must be \u000A strictly to the right of the slash so that the familiar name is well-defined \u000A and nonempty.</p> \u000A \u000A <p>If you are using ACL2 on a system in which file names do not have this \u000A form, please contact the authors and we'll see what we can do about \u000A generalizing ACL2's conventions.</p>"],

 "ACL2____DEFSTUB":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____DEFSTUB\">defstub</see> subr1 (<see topic=\"COMMON-LISP_____A2\">*</see> * state) =&gt; (<see topic=\"ACL2____MV\">mv</see> * state))\u000AACL2 !&gt;(<see topic=\"ACL2____DEFSTUB\">defstub</see> add-hash (<see topic=\"COMMON-LISP_____A2\">*</see> * hashtable) =&gt; hashtable)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> name args-sig =&gt; output-sig)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> name args-sig =&gt; output-sig :doc doc-string)</code> \u000A \u000A <p><v>Name</v> is a new function symbol and <v>(<see topic=\"ACL2____NAME\">name</see> . args-sig) =&gt;\u000A output-sig)</v> is a <see topic=\"ACL2____SIGNATURE\">signature</see>.  If the optional <tt><see topic=\"ACL2____DOC-STRING\">doc-string</see></tt> is \u000A supplied it should be a documentation string.  See also the ``Old Style'' \u000A heading below.</p> \u000A \u000A <p><v>Defstub</v> macro expands into an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> event (see <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>).  Thus, no axioms are available about <v>name</v> but it may be \u000A used wherever a function of the given signature is permitted.  Exception: if \u000A <v>output-sig</v> is of the form <v>(<see topic=\"ACL2____MV\">mv</see> ...)</v>, then a <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule is introduced stating that <v>name</v> returns a value \u000A satisfying <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.</p> \u000A \u000A <p>Old Style:</p> \u000A \u000A <code>Old Style General Form:\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> name formals output)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> name formals output :doc doc-string)</code> \u000A \u000A <p>where <v>name</v> is a new function symbol, <v>formals</v> is its list of \u000A formal parameters, and <v>output</v> is either a symbol (indicating that the \u000A function returns one result) or a term of the form <v>(<see topic=\"ACL2____MV\">mv</see> s1 ... sn)</v>, where \u000A each <v>si</v> is a symbol (indicating that the function returns <v>n</v> \u000A results).  Whether and where the symbol <tt><see topic=\"ACL2____STATE\">state</see></tt> occurs in <v>formals</v> \u000A and <v>output</v> indicates how the function handles <see topic=\"ACL2____STATE\">state</see>.  It should be \u000A the case that <v>(<see topic=\"ACL2____NAME\">name</see> formals output)</v> is in fact a signature (see <see topic=\"ACL2____SIGNATURE\">signature</see>).</p> \u000A \u000A <p>Note that with the old style notation it is impossible to stub-out a \u000A function that uses any single-threaded object other than state.  The old style \u000A is preserved for compatibility with earlier versions of ACL2.</p>"],

 "ACL2____CALLING-LD-IN-BAD-CONTEXTS":[["Ld"],"ACL2 Sources","ACL2","<p>The macro <tt><see topic=\"ACL2____LD\">ld</see></tt> was designed to be called directly in the \u000A top-level ACL2 loop, although there may be a few occasions for calling it from \u000A functions.  ACL2 cannot cope with invocations of <tt><see topic=\"ACL2____LD\">ld</see></tt> during the process \u000A of loading a compiled file for a book, so this is an error.</p> \u000A \u000A <p>To see how that can happen, consider the following book, where file \u000A <v>const.lsp</v> contains the single form <v>(<see topic=\"ACL2____DEFCONST\">defconst</see> *foo* '(a b))</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> t)\u000A(<see topic=\"ACL2____PROGN_12\">progn!</see> (<see topic=\"ACL2____LD\">ld</see> &quot;const.lsp&quot;))</code> \u000A \u000A <p>An attempt to certify this book will cause an error, but that particular \u000A error can be avoided, as discussed below.  If the book is certified, however, \u000A with production of a corresponding compiled file (which is the default \u000A behavior for <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>), then any subsequent call of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> that loads this compiled file will cause an error.  Again, this \u000A error is necessary because of how ACL2 is designed; specifically, this <tt><see topic=\"ACL2____LD\">ld</see></tt> call would interfere with tracking of constant definitions when loading \u000A the compiled file for the book.</p> \u000A \u000A <p>Because including such a book (with a compiled file) causes an error, then \u000A as a courtesy to the user, ACL2 arranges that the certification will fail \u000A (thus avoiding a surprise later when trying to include the book).  The error \u000A in that case will look as follows.</p> \u000A \u000A <code>ACL2 Error in LD:  It is illegal to call LD in this context.  See DOC\u000Acalling-ld-in-bad-contexts.</code> \u000A \u000A <p>If you really think it is OK to avoid this error, you can get around it by \u000A setting <see topic=\"ACL2____STATE\">state</see> global variable <v>ld-okp</v> to t: <v>(<see topic=\"ACL2____ASSIGN\">assign</see> ld-okp t)</v>. \u000A You can then certify the book in the example above, but you will still not be \u000A able to include it with a compiled file.</p>"],

 "ACL2____BRR-COMMANDS":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>:a!             abort to ACL2 top-level \u000A:p!             pop one level (exits a top-level break-rewrite loop) \u000A:target         term being rewritten \u000A:unify-subst    substitution making :lhs equal :target \u000A:hyps           hypotheses of the rule \u000A:hyp i          ith hypothesis of the rule \u000A:lhs            left-hand side of rule's conclusion \u000A:rhs            right-hand side of rule's conclusion \u000A:type-alist     type assumptions governing :target \u000A:initial-ttree  ttree before :eval (see <see topic=\"ACL2____TTREE\">ttree</see>) \u000A:ancestors      negations of backchaining hypotheses being pursued \u000A:wonp           indicates whether application succeeded (after :eval) \u000A:rewritten-rhs  rewritten :rhs (after :eval) of a rewrite rule \u000A:poly-list      list of polynomials (after :eval) of a linear rule, \u000A                  where the leading term of each is enclosed in an extra set \u000A                  of parentheses \u000A:final-ttree    ttree after :eval (see <see topic=\"ACL2____TTREE\">ttree</see>) \u000A:failure-reason reason rule failed (after :eval) \u000A:path           rewriter's path from top clause to :target \u000A:frame i        ith frame in :path \u000A:top            top-most frame in :path \u000A:ok             exit break \u000A:go             exit break, printing result \u000A:eval           try rule and re-enter break afterwards \u000A:ok!            :ok but no recursive breaks \u000A:go!            :go but no recursive breaks \u000A:eval!          :eval but no recursive breaks \u000A:ok$ runes      :ok with runes monitored during recursion \u000A:go$ runes      :go with runes monitored during recursion \u000A:eval$ runes    :eval with runes monitored during recursion \u000A:help           this message \u000A:standard-help  :help message from ACL2 top-level \u000A</code> \u000A \u000A <p><see topic=\"ACL2____BREAK-REWRITE\">Break-rewrite</see> is just a call of the standard ACL2 read-eval-print \u000A loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, on a ``<see topic=\"ACL2____WORMHOLE\">wormhole</see>'' <see topic=\"ACL2____STATE\">state</see>.  Thus, you may \u000A execute most commands you might normally execute at the top-level of ACL2. \u000A However, all <see topic=\"ACL2____STATE\">state</see> changes you cause from within <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> \u000A are lost when you exit or <v>:eval</v> the rule.  You cannot modify <see topic=\"ACL2____STOBJ\">stobj</see>s from within the break.  See <see topic=\"ACL2____BREAK-REWRITE\">break-rewrite</see> for more details and \u000A see <see topic=\"ACL2____LD\">ld</see> for general information about the standard ACL2 read-eval-print \u000A loop.  Also see <see topic=\"ACL2____BRR_04\">brr@</see> for a utility that can return a value for many of \u000A the keywords above, instead of merely printing to the screen.</p> \u000A \u000A <p>Note that if you are breaking on a <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____LINEAR\">linear</see> rule, \u000A several of the commands listed above do not apply: <v>:lhs</v>, <v>:rhs</v>, \u000A <v>:initial-ttree</v>, and <v>:final-ttree</v>.  Moreover, <v>:rewritten-rhs</v> \u000A also does not apply, but instead, <v>:poly-list</v> shows the result of applying \u000A the linear lemma as a list of polynomials, implicitly conjoined.  The leading \u000A term of each polynomial is enclosed in an extra set of parentheses.</p>"],

 "ACL2____SET-DEFAULT-BACKCHAIN-LIMIT":[["Backchain-limit"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>This event sets the default <tt><see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see></tt> used when a new <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, <tt><see topic=\"ACL2____META\">meta</see></tt>, or <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> rule is \u000A admitted.  Its value may be a two-element list whose elements are each either \u000A <v>nil</v> or a non-negative integer.  Its value <v>x</v> may also be <v>nil</v> or \u000A a non-negative integer, which is treated as the two element list <v>(x\u000A x)</v>.</p> \u000A \u000A <p>The first element of the list is used to limit backchaining for a rule of \u000A class <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> while the second element is used to limit \u000A backchaining for the other three classes of rules mentioned above.  See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> for details about how backchain-limits are used.  The \u000A examples below assume that a new rule doesn't itself specify a value for \u000A <v>:backchain-limit-lst</v>.</p> \u000A \u000A <code>:set-default-backchain-limit nil  ; do not impose backchain limits for the\u000A                                  ; rule\u000A:set-default-backchain-limit 0    ; allow only type-set reasoning for\u000A                                  ; relieving a new rule's hypotheses\u000A:set-default-backchain-limit 500  ; allow backchaining through a new rewrite,\u000A                                  ; linear, or meta rule's hypotheses to a\u000A                                  ; depth of no more than 500\u000A(<see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see> 500) ; same as above\u000A:set-default-backchain-limit (nil 500)\u000A                                  ; same as above\u000A(<see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see> '(nil 500))\u000A                                  ; same as above\u000A(<see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see> '(3 500))\u000A                                  ; for a new :type-prescription rule, allow\u000A                                  ; type-set backchaining to a depth\u000A                                  ; of no more than 3; for a new\u000A                                  ; rule of class :rewrite, :linear,\u000A                                  ; or :meta, allow backchaining to\u000A                                  ; a depth of no more than 50\u000A(<see topic=\"ACL2____SET-DEFAULT-BACKCHAIN-LIMIT\">set-default-backchain-limit</see> '(nil 500))\u000A                                  ; do not limit backchaining for a\u000A                                  ; new :type-prescription rule; for\u000A                                  ; a new rule of class :rewrite,\u000A                                  ; :linear, or :meta, allow\u000A                                  ; backchaining to a depth of no\u000A                                  ; more than 50</code> \u000A \u000A <p>The initial default backchain-limit is <v>nil</v>.</p>"],

 "ACL2____PUT-ASSOC":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name val alist)\u000A(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name val alist :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name val alist :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name val alist :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____PUT-ASSOC\">Put-assoc</see> name val alist)</v> returns an alist that is the same as the \u000A list <v>alist</v>, except that the first pair in <v>alist</v> with a <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> \u000A of <v>name</v> is replaced by <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> name val)</v>, if there is one.  If there \u000A is no such pair, then <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> name val)</v> is added at the end.  Note that the \u000A order of the keys occurring in <v>alist</v> is unchanged (though a new key may \u000A be added).</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>put-assoc</v> depends on the test.  In all \u000A cases, the last argument must satisfy <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> (see \u000A <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the last argument must satisfy <tt><see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see></tt>.  If \u000A the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument must be a symbol or the \u000A last argument must satisfy <tt><see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>put-assoc</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____PUT-ASSOC-EQ\">put-assoc-eq</see> name val alist)</v> is equivalent to \u000A <v>(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name val alist :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____PUT-ASSOC-EQUAL\">put-assoc-equal</see> name val alist)</v> is equivalent to <v>(<see topic=\"ACL2____PUT-ASSOC\">put-assoc</see> name\u000A val alist :test 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>put-assoc-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>put-assoc-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> put-assoc-equal (<see topic=\"ACL2____NAME\">name</see> val alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> alist) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> name val)))\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> name (<see topic=\"COMMON-LISP____CAAR\">caar</see> alist))\u000A              (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> name val) (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)\u000A                      (<see topic=\"ACL2____PUT-ASSOC-EQUAL\">put-assoc-equal</see> name val (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist))))))</code>"],

 "ACL2____DEFAULT-HINTS":[["Hints"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see> (w state))\u000A((computed-hint-1 clause)\u000A (computed-hint-2 clause stable-under-simplificationp))</code> \u000A \u000A <p>The value returned by this function is added to the right of the \u000A <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> argument of every <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> and <tt><see topic=\"ACL2____THM\">thm</see></tt> command, \u000A and to hints provided to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s as well (<v>:hints</v>, \u000A <v>:guard-hints</v>, and (for ACL2(r)) <v>:std-hints</v>).</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> for a more general discussion.  Advanced users \u000A only: see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for an advanced variant of default hints that \u000A are not superseded by <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> arguments.</p>"],

 "ACL2____ACL2P-KEY-CHECKPOINTS":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>For printing output, the parallel version of the waterfall follows the \u000A precedent of <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt>.  The idea behind gag mode is to print only the \u000A subgoals most relevant to debugging a failed proof attempt.  These subgoals \u000A are called 'key checkpoints' (see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> for the definition of \u000A ``key'' and ``checkpoint''), and we restrict the default output mode for the \u000A parallel version of the waterfall to printing checkpoints similar to these key \u000A checkpoints.</p> \u000A \u000A <p>As of this writing, we are aware of exactly one discrepancy between gag \u000A mode's key checkpoints and the parallel version of the waterfall's \u000A checkpoints.  This discrepancy occurs when using ``by'' hints (see <see topic=\"ACL2____HINTS\">hints</see>).  As an example, take the following form, which attempts to prove a \u000A non-theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z) (<see topic=\"COMMON-LISP____APPEND\">append</see> z (<see topic=\"COMMON-LISP____APPEND\">append</see> y x)))\u000A     :hints ((&quot;Subgoal *1/2'''&quot; :by nil)))</code> \u000A \u000A <p>With waterfall parallelism enabled, <v>Subgoal *1/2''</v> will be printed as \u000A a key checkpoint.  This is different from using <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> while running \u000A the serial version of the waterfall, which skips printing the subgoal as a \u000A checkpoint.</p> \u000A \u000A <p>For those familiar with the ACL2 waterfall, we note that that the parallel \u000A version of the waterfall prints key checkpoints that are unproved in the \u000A following sense: a subgoal is a key checkpoint if it leads, in the current \u000A call of the waterfall, to a goal that is pushed for induction.</p>"],

 "ACL2-PC____X":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Ax --  expand and simplify.</code> \u000A \u000A <p>For example, if the current subterm is (append a b), then after <v>x</v> the \u000A current subterm will probably be (cons (car a) (append (cdr a) b)) if (consp \u000A a) and (true-listp a) are among the top-level hypotheses and governors.  If \u000A there are no top-level hypotheses and governors, then after <v>x</v> the current \u000A subterm will probably be:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> x\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))\u000A      y)\u000A  (apply 'binary-append (<see topic=\"COMMON-LISP____LIST\">list</see> x y))).\u000A\u000AGeneral Form:\u000A(X &amp;key\u000A   rewrite normalize backchain-limit in-theory hands-off expand)</code> \u000A \u000A <p>Expand the function call at the current subterm, and simplify using the \u000A same conventions as with the <v>s</v> command (see documentation for \u000A <v>s</v>).</p> \u000A \u000A <p>Unlike <v>s</v>, it is permitted to set both <v>:rewrite</v> and \u000A <v>:normalize</v> to <v>nil</v>, which will result in no simplification; see \u000A <v>x-dumb</v>.</p> \u000A \u000A <p><b>Remark</b> (obscure): On rare occasions the current address may be \u000A affected by the use of <v>x</v>.  For example, suppose we have the \u000A definition</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> g (x) (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) x 3))</code> \u000A \u000A <p>and then we enter the proof-checker with</p> \u000A \u000A <code>(<see topic=\"ACL2____VERIFY\">verify</see> (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (g x) 3) t)) .</code> \u000A \u000A <p>Then after invoking the instruction <v>(dive 2 1)</v>, so that the current \u000A subterm is <v>(g x)</v>, followed by the instruction <v>x</v>, we would expect the \u000A conclusion to be <v>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3) t)</v>.  However, the system \u000A actually replaces <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> 3 3)</v> with <v>t</v> (because we use the ACL2 \u000A term-forming primitives), and hence the conclusion is actually <v>(<see topic=\"COMMON-LISP____IF\">if</see>\u000A (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) t t)</v>.  Therefore, the current address is put at <v>(2)</v> rather \u000A than <v>(2 1)</v>.  In such cases, a warning ``<v>NOTE</v>'' will be printed to \u000A the terminal.</p> \u000A \u000A <p>The other primitive commands to which the above ``truncation'' note applies \u000A are <v>equiv</v>, <v>rewrite</v>, and <v>s</v>.</p>"],

 "COMMON-LISP____SUBSETP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y)\u000A(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x y :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____SUBSETP\">Subsetp</see> x y)</v> is true if and only if every <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> of the list \u000A <v>x</v> is a <v>member</v> of the list <v>y</v>.  The optional keyword, <v>:TEST</v>, \u000A has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for \u000A comparing members of the two lists.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>subsetp</v> depends on the test.  In all \u000A cases, both arguments must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the \u000A test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>subsetp</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____SUBSETP-EQ\">subsetp-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x lst\u000A :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>subsetp-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>subsetp-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> subsetp-equal (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) t)\u000A             ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) y)\u000A              (<see topic=\"ACL2____SUBSETP-EQUAL\">subsetp-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))\u000A             (t nil)))</code> \u000A \u000A <p><v>Subsetp</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p>"],

 "COMMON-LISP____REMOVE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst)\u000A(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____REMOVE\">Remove</see> x lst)</v> is equal to <v>lst</v> if <v>x</v> is not a member of \u000A <v>lst</v>, else is the result of removing all occurrences of <v>x</v> from \u000A <v>lst</v>.  The optional keyword, <v>:TEST</v>, has no effect logically, but \u000A provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with \u000A successive elements of <v>lst</v>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____REMOVE1\">remove1</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>remove</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> \u000A (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument \u000A must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>remove</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____REMOVE-EQ\">remove-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst :test\u000A 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"COMMON-LISP____REMOVE\">remove</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>remove-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>remove-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> remove-equal (x l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A              (<see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l)\u000A                      (<see topic=\"ACL2____REMOVE-EQUAL\">remove-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code> \u000A \u000A <p><v>Remove</v> is defined by Common Lisp.  See any Common Lisp documentation \u000A for more information.</p>"],

 "ACL2____SYMBOL-PACKAGE-NAME":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>WARNING: While <v>symbol-package-name</v> behaves properly on all \u000A ACL2 objects, it may give surprising results when called in raw Lisp.  For \u000A more details see <see topic=\"ACL2____PKG-IMPORTS\">pkg-imports</see>, in particular the discussion there of the \u000A <v>&quot;COMMON-LISP&quot;</v> package.</p> \u000A \u000A <p>Completion Axiom (<v>completion-of-symbol-package-name</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A           (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> x)\u000A         &quot;&quot;))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)</code> \u000A \u000A <p>Note: <v>Symbol-package-name</v> may diverge from the name of the symbol's \u000A package in raw Lisp, in the case that this package is the main Lisp package. \u000A For example, in GCL <v>(<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> 'car)</v> evaluates to \u000A \"COMMON-LISP\" even though the actual package name for the symbol, <v>car</v>, \u000A is \"LISP\".</p>"],

 "ACL2____SET-BACKCHAIN-LIMIT":[["Backchain-limit"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>This event sets the global <tt><see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see></tt> used by the ACL2 \u000A type-set and rewriting mechanisms.  Its value may be a cons whose car and cdr \u000A are each either <v>nil</v> or a non-negative integer.  Its value <v>x</v> may also \u000A be <v>nil</v> or a non-negative integer, which is treated as a cons whose car \u000A and cdr are both <v>x</v>.</p> \u000A \u000A <p>The car is used to limit backchaining used by the ACL2 type-set mechanism, \u000A while the cdr is used to limit backchaining used by the rewriting mechanism. \u000A See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see> for details about how backchain-limits are used. \u000A Rewrite backchain limits may also be installed at the level of hints; see \u000A <see topic=\"ACL2____HINTS\">hints</see> for a discussion of <v>:backchain-limit-rw</v>.</p> \u000A \u000A <code>:set-backchain-limit nil  ; do not impose any additional limits\u000A:set-backchain-limit 0    ; allow only type-set reasoning for rewriting\u000A                          ; hypotheses\u000A:set-backchain-limit 500  ; allow backchaining to a depth of no more\u000A                          ; than 500 for rewriting hypotheses\u000A(<see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see> 500) ; same as above\u000A:set-backchain-limit (500 500)\u000A                          ; same as above\u000A(<see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see> '(500 500))\u000A                          ; same as above\u000A(<see topic=\"ACL2____SET-BACKCHAIN-LIMIT\">set-backchain-limit</see> '(3 500))\u000A                          ; allow type-set backchaining to a depth of no more\u000A                          ; than 3 and rewriter backchaining to a depth of no\u000A                          ; more than 500</code> \u000A \u000A <p>The default limit is <v>(nil nil)</v>.</p>"],

 "ACL2____P_12":[["Ld"],"ACL2 Sources","ACL2","<p>Logically speaking, <v>(<see topic=\"ACL2____P_12\">p!</see>) = nil</v>.  If you are already at the top \u000A level of the ACL2 command loop, rather than being in a subsidiary call of \u000A <tt><see topic=\"ACL2____LD\">ld</see></tt>, then the keyword then a call of <v>(<see topic=\"ACL2____P_12\">p!</see>)</v> returns <v>nil</v> and has \u000A no other effect.</p> \u000A \u000A <p>Otherwise, <v>(<see topic=\"ACL2____P_12\">p!</see>)</v> is evaluated inside a call of <tt><see topic=\"ACL2____LD\">ld</see></tt> that was made \u000A inside ACL2's command loop.  In that case, the current computation is aborted \u000A and treating as causing an error, and control returns to the superior call of \u000A <v>ld</v>.</p> \u000A \u000A <p>Here is a more detailed description of the effect of <v>(<see topic=\"ACL2____P_12\">p!</see>)</v> when not at \u000A the top level of the ACL2 command loop.  The current call of <v>LD</v> is \u000A treated as though <v>ld-error-action</v> is <v>:RETURN!</v> (the default) and \u000A hence immediately returns control to the superior call of <tt><see topic=\"ACL2____LD\">ld</see></tt>.  If all \u000A calls of <tt><see topic=\"ACL2____LD\">ld</see></tt> were made with the default <v>ld-error-action</v> of \u000A <v>:RETURN!</v>, then all superior calls of <v>ld</v> will then complete until you \u000A are back at top level of the ACL2 loop.  For more information, see <see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see>.</p> \u000A \u000A <p>If you are at an ACL2 prompt (as opposed to a raw Lisp break), then you may \u000A type <v>:p!</v> in place of <v>(<see topic=\"ACL2____P_12\">p!</see>)</v>; see <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.</p>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-2":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (REV (FOO A)))</code> \u000A \u000A <p>The definition of <v>rev</v> in this problem is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))</code> \u000A \u000A <p>Since the definition terminates with an <v>endp</v> test and otherwise \u000A <v>cdr</v>s the argument, the author of <v>rev</v> was clearly expecting <v>x</v> to \u000A be a <v>true-listp</v>.  (Indeed, the ``<see topic=\"ACL2____GUARD\">guard</see>'' <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A for <v>rev</v> must include <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)</v> since that is <v>endp</v>'s \u000A guard.)  So you're naturally justified in limiting your thoughts about <v>(rev\u000A x)</v> to <v>x</v> that are true-lists.  This gives rise to the theorem:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-rev-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A           (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (rev x))))</code> \u000A \u000A <p>This is the kind of thinking illustrated in the earlier <v>append</v> example \u000A (see <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-1\">practice-formulating-strong-rules-1</see>), and, to paraphrase Z in \u000A <i>Men in Black</i>, it exemplifies ``everything we've come to expect from \u000A years of training with typed languages.''</p> \u000A \u000A <p>But logically speaking, the definition of <v>rev</v> does not require <v>x</v> \u000A to be a <v>true-listp</v>.  It can be any object at all: ACL2 functions are \u000A total.  <v>Rev</v> either returns <v>nil</v> or the result of appending a \u000A singleton list onto the right end of its recursive result.  That call of \u000A <v>append</v> always returns a <v>true-listp</v> since the singleton list is a \u000A true list.  (See <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-1\">practice-formulating-strong-rules-1</see>.)</p> \u000A \u000A <p>So this is a theorem and a very useful :<tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> true-listp-rev-strong\u000A  (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (rev x))).</code> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____NOTE-3-6-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>The essential changes to ACL2 since Version 3.6 are the two bug fixes \u000A described below.  There was also some functionality-neutral code refactoring, \u000A as requested by Daron Vroon in support of the ACL2 Sedan (see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see>).  Also see <a \u000A href='https://code.google.com/p/acl2-books/wiki/ReleaseVersionNumbers'>ReleaseVersionNumbers</a> \u000A for a list of books in Version 3.6.1 of ACL2 but not Version 3.6.  For \u000A <i>changes</i> to existing books rather than <i>additions</i>, see the <a \u000A href='http://code.google.com/p/acl2-books/source/list'>log entries</a> \u000A starting with revision r329 up through revision 350.</p> \u000A \u000A <p>Fixed a soundness bug in the handling of <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>, \u000A specifically in the handling of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>-expressions.  Thanks to Pete \u000A Manolios, who sent us a proof of <v>nil</v>, essentially as follows.  In the \u000A termination proof for <v>foo</v> below, the binding of <v>x</v> to <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> t x)</v> \u000A was not substituted into the recursive call of <v>foo</v> for purposes of the \u000A termination proof.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :ruler-extenders :all))\u000A  (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP____CONS\">cons</see> t x)))\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A        x\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> t (foo (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> foo-bad\u000A  nil\u000A  :hints ((&quot;Goal&quot;\u000A           :use ((:instance foo (x '(3))))\u000A           :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> foo (foo))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>Fixed a typo in code supporting <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> (specifically, \u000A swapped arguments in a recursive call of ACL2 source function \u000A <v>get-ruler-extenders2</v>, which could cause problems for functions defined \u000A using <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>).  Thanks to Daron Vroon for bringing this bug \u000A to our attention, pointing out the swapped arguments.</p>"],

 "ACL2____NOTE-2-8-RULES":[["Note-2-8"],"ACL2 Sources","ACL2","<p>The <see topic=\"ACL2____THEORY\">theory</see> <tt><see topic=\"ACL2____MINIMAL-THEORY\">minimal-theory</see></tt> has been changed by adding \u000A the <see topic=\"ACL2____DEFINITION\">definition</see> <see topic=\"ACL2____RUNE\">rune</see> for <tt><see topic=\"ACL2____MV-NTH\">mv-nth</see></tt> to the theory.  A \u000A corresponding change has been made to the theory warning mechanism, which was \u000A failing to warn if the definition of <v>mv-nth</v> is disabled, even though \u000A calls of <v>mv-nth</v> can be expanded by special-purpose code in the rewriter. \u000A Thanks to Serita Van Groningen for pointing out this problem with the theory \u000A warning mechanism.</p> \u000A \u000A <p>The <tt><see topic=\"ACL2____DEFEVALUATOR\">defevaluator</see></tt> event has been modified so that in the body of the \u000A evaluator function, to add a new case <v>(<see topic=\"COMMON-LISP____ATOM\">ATOM</see> X)</v> (returning <v>nil</v>) has \u000A been inserted immediately after the case <v>(<see topic=\"COMMON-LISP____EQ\">EQ</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) 'QUOTE)</v>.  This is a \u000A no-op semantically but may speed up proofs.  Thanks to Warren Hunt for \u000A suggesting this change.</p> \u000A \u000A <p>A new form of <v>:</v><tt><see topic=\"ACL2____COMPOUND-RECOGNIZER\">compound-recognizer</see></tt> rule is now allowed:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (fn x) concl1 concl2)</code> \u000A \u000A <p>This is equivalent to an existing form:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (fn x) concl1)\u000A     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (fn x)) concl2))</code> \u000A \u000A <p>Thanks to Josh Purinton for bringing this to our attention.</p> \u000A \u000A <p>Rewrite rules <v>realpart-+</v> and <v>imagpart-+</v> have been added in order \u000A to simplify the <tt><see topic=\"COMMON-LISP____REALPART\">realpart</see></tt> and <tt><see topic=\"COMMON-LISP____IMAGPART\">imagpart</see></tt> (respectively) of a sum. \u000A They follow from a theorem <v>add-def-complex</v> that equates a sum with the \u000A complex number formed by adding real and imaginary parts.  All three of these \u000A theorems may be found in source file <v>axioms.lisp</v>.  Thanks to Eric Smith \u000A for raising a question leading to these additions, as well as to Joe Hendrix \u000A and Vernon Austel for helpful suggestions.</p>"],

 "ACL2____LEN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Len</v> returns the length of a list.</p> \u000A \u000A <p>A Common Lisp function that is appropriate for both strings and proper \u000A lists is <v>length</v>; see <see topic=\"COMMON-LISP____LENGTH\">length</see>.  The guard for <v>len</v> is \u000A <v>t</v>.</p> \u000A \u000A <p>(Low-level implementation note.  ACL2 provides a highly-optimized \u000A implementation of <v>len</v>, which is tail-recursive and fixnum-aware, that \u000A differs from its simple ACL2 definition.)</p> \u000A \u000A <p><b>Function: </b><srclink>len</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> len (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))) 0))</code>"],

 "ACL2____INTRODUCTORY-CHALLENGES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Do each of the problems.  In each case, start with a fresh ACL2 (or \u000A undo all effects of previous events with <v>:ubt! 1</v>).  This may require that \u000A you ``re-discover'' the same lemma more than once in different problems, but \u000A recognizing the need for something you used in some previous project is part \u000A of the training.</p> \u000A \u000A <p>We recommend that you follow The Method and consult the documentation as \u000A needed -- but that you not look at our answers until you're well and truly \u000A baffled!</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1\">introductory-challenge-problem-1</see> (Answer: <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1-ANSWER\">introductory-challenge-problem-1-answer</see>)</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2\">introductory-challenge-problem-2</see> (Answer: <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2-ANSWER\">introductory-challenge-problem-2-answer</see>)</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3\">introductory-challenge-problem-3</see> (Answer: <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3-ANSWER\">introductory-challenge-problem-3-answer</see>)</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4\">introductory-challenge-problem-4</see> (Answer: <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-4-ANSWER\">introductory-challenge-problem-4-answer</see>)</p> \u000A \u000A <p>In addition to these explicit challenge problems designed for beginners, \u000A the ACL2 documentation has many example solutions to problems (not always \u000A phrased in the question/answer format here).  If you are looking for other \u000A examples, you should consider</p> \u000A \u000A <p><see topic=\"ACL2____ANNOTATED-ACL2-SCRIPTS\">annotated-ACL2-scripts</see> (Answer: the answers are given in the \u000A examples)</p> \u000A \u000A <p>When you've done the problems and compared your solutions to ours, use your \u000A browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p>"],

 "ACL2____HONS-WASH":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-wash</v> just returns nil; we leave it enabled and would \u000A think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, in CCL, <v>hons-wash</v> runs a garbage collection after \u000A taking special measures to allow normed conses to be collected.  In Lisps \u000A other than CCL, <v>hons-wash</v> does nothing.  This is because the efficient \u000A implementation of <v>hons-wash</v> is specific to the \"static honsing\" scheme \u000A which requires CCL-specific features.</p> \u000A \u000A <p>Why is this function needed?  Ordinarily, it is not possible to garbage \u000A collect any normed conses.  This is because the Hons Space includes pointers \u000A to any normed conses, and hence Lisp's garbage collector thinks these objects \u000A are live.  To correct for this, <v>hons-wash</v> (1) clears out these pointers, \u000A (2) runs a garbage collection, then (3) re-norms any previously-normed conses \u000A which have survived the garbage collection.</p> \u000A \u000A <p>Note.  It is not recommended to interrupt this function.  Doing so may \u000A cause persistently normed conses and fast alist keys to become un-normed, \u000A which might lead to less efficient re-norming and/or violations of the \u000A fast-alist discipline.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-wash</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-wash nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____DELETE-INCLUDE-BOOK-DIR":[["Books-reference"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A; Remove association of a directory with :smith for include-book and ld:\u000A(<see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see> :smith)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see> kwd)</code> \u000A \u000A <p>where <v>kwd</v> is a <tt><see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see></tt>.  The effect of this event is to \u000A modify the meaning of the <v>:dir</v> keyword argument of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> \u000A and <tt><see topic=\"ACL2____LD\">ld</see></tt> as indicated by the example above, namely by removing \u000A association of a directory with the indicated keyword for purposes of the \u000A <v>:dir</v> argument of <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and <tt><see topic=\"ACL2____LD\">ld</see></tt>.  See <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt> for how to associate a new directory with a keyword.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>This macro is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see> for a \u000A discussion of this aspect of both macros.  For non-local associations of \u000A keywords with directories, see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see> and <see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see>.  Note that <v>delete-include-book-dir</v> may only be \u000A used to remove keywords added by calls of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>, and \u000A <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR_12\">delete-include-book-dir!</see></tt> may only be used to remove keywords added by \u000A calls of <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt></p>"],

 "ACL2____CHARACTER-ENCODING":[["Io"],"ACL2 Sources","ACL2","<p>When the Common Lisp reader comes across bytes in a file or at the \u000A terminal, they are parsed into characters.  The simplest case is when each \u000A byte that is read is a standard character (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).  It is \u000A actually quite common that each byte that is read corresponds to a single \u000A character.  The parsing of bytes into characters is based on a <i>character \u000A encoding</i>, that is, a mapping that associates one or more bytes with each \u000A legal character.</p> \u000A \u000A <p>In order to help guarantee the portability of files (including <see topic=\"ACL2____BOOKS\">books</see>), ACL2 installs a common character encoding for reading files, often \u000A known as iso-8859-1 or latin-1.  For some host Lisps this character encoding \u000A is also used for reading from the terminal; but, sadly, this may not hold for \u000A all host Lisps, and may not even be possible for some of them.</p> \u000A \u000A <p>The use of the above encoding could in principle cause problems if one's \u000A editor produces files using an encoding other than iso-8859-1, at least if one \u000A uses non-standard characters.  In particular, the default Emacs buffer \u000A encoding may be utf-8.  If your file has non-standard characters, then in \u000A Emacs you can evaluate the form</p> \u000A \u000A <code>(setq save-buffer-coding-system 'iso-8859-1)</code> \u000A \u000A <p>before saving the buffer into a file.  This will happen automatically for \u000A users who load distributed file <v>emacs/emacs-acl2.el</v> into their Emacs \u000A sessions.</p> \u000A \u000A <p>For an example of character encodings in action, see the community book \u000A <v>books/misc/character-encoding-test.lisp</v>.</p>"],

 "ACL2____ASSERT-EVENT":[["Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) 7))\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) 7) :msg (<see topic=\"ACL2____MSG\">msg</see> &quot;Error: ~x0&quot; 'equal-check))\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____B2\">+</see> 3 4) 7) :on-skip-proofs t)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> form)\u000A(<see topic=\"ACL2____ASSERT-EVENT\">assert-event</see> form :on-skip-proofs t)</code> \u000A \u000A <p><v>Assert-event</v> takes a ground form, i.e., one with no free variables; \u000A <tt><see topic=\"ACL2____STOBJ\">stobj</see></tt>s are allowed but only a single non-<tt><see topic=\"ACL2____STOBJ\">stobj</see></tt> value can be \u000A returned.  The form is then evaluated and if the result is <v>nil</v>, then a \u000A so-called hard error (see <see topic=\"ACL2____ER\">er</see>) results.  This evaluation is however not \u000A done if proofs are being skipped, as during <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> (also see \u000A <see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see> and see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>), unless <v>:on-skip-proofs\u000A t</v> is supplied.</p> \u000A \u000A <p>Normally, if an <v>assert-event</v> call fails then a generic failure message \u000A is printed, showing the offending form.  However, if keyword argument \u000A <v>:msg</v> is supplied, then the failure message is printed as with <tt><see topic=\"ACL2____FMT\">fmt</see></tt> \u000A argument <v>~@0</v>; see <see topic=\"ACL2____FMT\">fmt</see>.  In particular, <v>:msg</v> is typically a \u000A string or a call <v>(<see topic=\"ACL2____MSG\">msg</see> str arg-0 arg-1 ... arg-k)</v>, where <v>str</v> is a \u000A string and each <v>arg-i</v> is the value to be associated with <v>#\\i</v> upon \u000A formatted printing (as with <tt><see topic=\"ACL2____FMT\">fmt</see></tt>) of the string <v>str</v>.</p> \u000A \u000A <p>This form may be put into a book to be certified (see <see topic=\"ACL2____BOOKS\">books</see>), \u000A because <v>assert-event</v> is a macro whose calls expand to calls of \u000A <v>value-triple</v> (see <see topic=\"ACL2____EMBEDDED-EVENT-FORM\">embedded-event-form</see>).  When certifying a book, \u000A guard-checking is off, as though <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> nil)</v> has been \u000A evaluated; see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>.  That, together with a ``safe \u000A mode,'' guarantees that <v>assert-event</v> forms are evaluated in the logic \u000A without guard violations while certifying a book.</p>"],

 "ACL2-PC____GENERALIZE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____GENERALIZE\">generalize</see>\u000A ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y)) 0)\u000A ((<see topic=\"COMMON-LISP____APPEND\">append</see> x y) w))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____GENERALIZE\">generalize</see> &amp;rest substitution)</code> \u000A \u000A <p>Generalize using the indicated substitution, which should be a non-empty \u000A list.  Each element of that list should be a two-element list of the form \u000A <v>(<see topic=\"ACL2____TERM\">term</see> variable)</v>, where <v>term</v> may use abbreviations.  The effect of \u000A the instruction is to replace each such term in the current goal by the \u000A corresponding variable.  This replacement is carried out by a parallel \u000A substitution, outside-in in each hypothesis and in the conclusion.  More \u000A generally, actually, the ``variable'' (second) component of each pair may be \u000A <v>nil</v> or a number, which causes the system to generate a new name of the \u000A form <v>_</v> or <v>_n</v>, with <v>n</v> a natural number; more on this below. \u000A However, when a variable is supplied, it must not occur in any goal of the \u000A current proof-checker state.</p> \u000A \u000A <p>When the ``variable'' above is <v>nil</v>, the system will treat it as the \u000A variable <v>|_|</v> if that variable does not occur in any goal of the current \u000A proof-checker state.  Otherwise it treats it as <v>|_0|</v>, or <v>|_1|</v>, or \u000A <v>|_2|</v>, and so on, until one of these is not among the variables of the \u000A current proof-checker state.  If the ``variable'' is a non-negative integer \u000A <v>n</v>, then the system treats it as <v>|_n|</v> unless that variable already \u000A occurs among the current goals, in which case it increments n just as above \u000A until it obtains a new variable.</p> \u000A \u000A <p><b>Remark:</b> The same variable may not occur as the variable component of \u000A two different arguments (though <v>nil</v> may occur arbitrarily many times, as \u000A may a positive integer).</p>"],

 "ACL2____USING-COMPUTED-HINTS-5":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>We have found that it is sometimes helpful to define hints so that \u000A they print out messages to the terminal when they fire, so you can see what \u000A hint was generated and which of your computed hints did it.</p> \u000A \u000A <p>To that end we have defined a macro we sometimes use.  Suppose you have a \u000A <v>:hints</v> specification such as:</p> \u000A \u000A <code>:hints (computed-hint-fn (hint-expr id))</code> \u000A \u000A <p>If you defmacro the macro below you could then write instead:</p> \u000A \u000A <code>:hints ((show-hint computed-hint-fn 1)\u000A        (show-hint (hint-expr id) 2))</code> \u000A \u000A <p>with the effect that whenever either hint is fired (i.e., returns \u000A non-<v>nil</v>), a message identifying the hint by the marker (1 or 2, above) \u000A and the non-<v>nil</v> value is printed.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> show-hint (hint &amp;optional marker)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A   ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> hint)\u000A         (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> hint)))\u000A    hint)\u000A   (t\u000A    `(<see topic=\"COMMON-LISP____LET\">let</see> ((marker ,marker)\u000A           (ans ,(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> hint)\u000A                     `(,hint id clause world stable-under-simplificationp)\u000A                   hint)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> ans\u000A           (<see topic=\"ACL2____PROG2_42\">prog2$</see>\u000A            (<see topic=\"ACL2____CW\">cw</see> &quot;~%***** Computed Hint~#0~[~/ (from hint ~x1)~]~%~x2~%~%&quot;\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____NULL\">null</see> marker) 0 1)\u000A                marker\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> (string-for-tilde-@-clause-id-phrase id)\u000A                      ans))\u000A            ans)\u000A         nil)))))</code> \u000A \u000A <p>Note that when <v>show-hint</v> is applied to a hint that is a symbol, e.g., \u000A <v>computed-hint-fn</v>, it applies the symbol to the four computed-hint \u000A arguments: <v>id</v>, <v>clause</v>, <v>world</v>, and \u000A <v>stable-under-simplificationp</v>.  If <v>computed-hint-fn</v> is of arity 3 the \u000A code above would cause an error.  One way to avoid it is to write</p> \u000A \u000A <code>:hints ((show-hints (computed-hint-fn id clause world) 1)\u000A        (show-hint (hint-expr id) 2)).</code> \u000A \u000A <p>If you only use computed hints of arity 3, you might eliminate the \u000A occurrence of <v>stable-under-simplificationp</v> in the definition of \u000A <v>show-hint</v> above.</p> \u000A \u000A <p>Putting a <v>show-hint</v> around a common hint has no effect.  If you find \u000A yourself using this utility let us know and we'll consider putting it into the \u000A system itself.  But it does illustrate that you can use computed hints to do \u000A unusual things.</p>"],

 "ACL2____SOLUTION-TO-SIMPLE-EXAMPLE":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>To see a statement of the problem solved below, see <see topic=\"ACL2____ANNOTATED-ACL2-SCRIPTS\">annotated-ACL2-scripts</see> (in particular the end of that topic).</p> \u000A \u000A <p>Here is a sequence of ACL2 <see topic=\"ACL2____EVENTS\">events</see> that illustrates the use of ACL2 \u000A to make definitions and prove theorems.  We will introduce the notion of the \u000A fringe of a tree, as well as the notion of a leaf of a tree, and then prove \u000A that the members of the fringe are exactly the leaves.</p> \u000A \u000A <p>We begin by defining the fringe of a tree, where we identify trees simply \u000A as <see topic=\"COMMON-LISP____CONS\">cons</see> structures, with <see topic=\"COMMON-LISP____ATOM\">atom</see>s at the leaves.  The definition is \u000A recursive, breaking into two cases.  If <v>x</v> is a <see topic=\"COMMON-LISP____CONS\">cons</see>, then the \u000A <v>fringe</v> of <v>x</v> is obtained by appending together the <v>fringe</v>s of \u000A the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> (left and right child) of <v>x</v>.  Otherwise, \u000A <v>x</v> is an <see topic=\"COMMON-LISP____ATOM\">atom</see> and its <v>fringe</v> is the one-element list \u000A containing only <v>x</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fringe (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (fringe (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A              (fringe (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A    (<see topic=\"COMMON-LISP____LIST\">list</see> x)))</code> \u000A \u000A <p>Now that <v>fringe</v> has been defined, let us proceed by defining the \u000A notion of an atom appearing as a ``leaf'', with the goal of proving that the \u000A leaves of a tree are exactly the members of its <v>fringe</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> leaf-p (atm x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A      (<see topic=\"COMMON-LISP____OR\">or</see> (leaf-p atm (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A          (leaf-p atm (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A    (<see topic=\"COMMON-LISP____EQUAL\">equal</see> atm x)))</code> \u000A \u000A <p>The main theorem is now as follows.  Note that the rewrite rule below uses \u000A the equivalence relation <tt><see topic=\"ACL2____IFF\">iff</see></tt> (see <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>) rather than \u000A <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, since <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> returns the tail of the given list that \u000A begins with the indicated member, rather than returning a Boolean.  (Use \u000A <v>:pe member</v> to see the definition of <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt>.)</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> leaf-p-iff-member-fringe\u000A  (<see topic=\"ACL2____IFF\">iff</see> (leaf-p atm x)\u000A       (<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> atm (fringe x))))</code>"],

 "ACL2____SET-PRINT-CLAUSE-IDS":[["Prover-output"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see> t)\u000A:set-print-clause-ids t\u000A(<see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see> nil)\u000A:set-print-clause-ids nil</code> \u000A \u000A <p>This command affects output from the theorem prover only when <v>'prove</v> \u000A output is inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) or gag-mode is on (but \u000A in that case the <v>:goals</v> setting issues this command automatically; see \u000A <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>).  Calling this macro with value <v>t</v> as shown above \u000A will cause subsequent proof attempts with <v>'prove</v> output inhibited to \u000A print the subgoal number, so that you can see the progress of the proof; value \u000A <v>nil</v> reverts to the default behavior, where this is not the case.  On a \u000A related note, we point out that you can cause output to be saved for later \u000A display; see <see topic=\"ACL2____PSO\">pso</see> and see <see topic=\"ACL2____PSO_12\">pso!</see>.</p> \u000A \u000A <p>If <v>'prove</v> output is inhibited or gag-mode is on, and if you issue \u000A <v>(<see topic=\"ACL2____SET-PRINT-CLAUSE-IDS\">set-print-clause-ids</see> t)</v> (either explicitly or with <v>(<see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>\u000A :goals)</v>), then you can restrict when subgoal numbers are printed.  In the \u000A following example we restrict to subgoals that are no more than four \u000A inductions deep, no more than four casesplits deep, and no more than four \u000A single-subgoals deep.  For additional relevant explanation, see <see topic=\"ACL2____CLAUSE-IDENTIFIER\">clause-identifier</see> and see <see topic=\"ACL2____DEFATTACH\">defattach</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> print-clause-id-okp-level-4 (cl-id)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic :guard (clause-id-p cl-id)))\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"ACL2____ACCESS\">access</see> clause-id cl-id :pool-lst))\u000A           4)\u000A       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"ACL2____ACCESS\">access</see> clause-id cl-id :case-lst))\u000A           4)\u000A       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"ACL2____ACCESS\">access</see> clause-id cl-id :primes)\u000A           4)))\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> print-clause-id-okp print-clause-id-okp-level-4)</code>"],

 "ACL2____SET-LET_A2-ABSTRACTIONP":[["Prover-output"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>When this flag is set to <v>t</v>, subterms that occur more than once in a \u000A clause are abstracted away with <tt><see topic=\"COMMON-LISP____LET_A2\">let*</see></tt>, generally shortening the \u000A displayed size of the clauses.  This flag only affects how clauses are \u000A printed.  It does not change what terms the theorem prover manipulates.</p> \u000A \u000A <code>:set-let*-abstractionp t ;;; or, (<see topic=\"ACL2____SET-LET_A2-ABSTRACTIONP\">set-let*-abstractionp</see> t)</code> \u000A \u000A <p>will cause the prettyprinter to do ``let* abstraction'' on clauses before \u000A they are printed.  The algorithm finds the maximal multiply-occuring subterm \u000A and extracts it, binding it to some new variable and replacing its occurrences \u000A by that variable.  This produces a <v>let*</v> form.  This process is iterated \u000A until no subterm occurs more than once.  This process generally takes a little \u000A time, but less time than to print large clauses.  The process can greatly \u000A reduce the amount of text produced by the prover.</p> \u000A \u000A <p>THIS ONLY AFFECTS HOW THE CLAUSES ARE PRINTED!  The unabstracted clauses \u000A are manipulated by the theorem prover.</p> \u000A \u000A <code>:set-let*-abstractionp nil</code> \u000A \u000A <p>restores normal clause printing.</p> \u000A \u000A <p>The mode is stored in the defaults table, See <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>. \u000A Thus, the mode may be set locally in books.</p>"],

 "ACL2____REMOVE1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst)\u000A(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____REMOVE1\">Remove1</see> x lst)</v> is equal to <v>lst</v> if <v>x</v> is not a member of \u000A <v>lst</v>, else is the result of removing the first occurrences of <v>x</v> from \u000A <v>lst</v>.  The optional keyword, <v>:TEST</v>, has no effect logically, but \u000A provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with \u000A successive elements of <v>lst</v>.</p> \u000A \u000A <p>Also see <see topic=\"COMMON-LISP____REMOVE\">remove</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>remove1</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> \u000A (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument \u000A must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>remove1</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____REMOVE1-EQ\">remove1-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst\u000A :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____REMOVE1-EQUAL\">remove1-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____REMOVE1\">remove1</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p><b>Function: </b><srclink>remove1-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> remove1-equal (x l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> x (<see topic=\"COMMON-LISP____CAR\">car</see> l)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l)\u000A                      (<see topic=\"ACL2____REMOVE1-EQUAL\">remove1-equal</see> x (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>remove1-equal</v>.</p>"],

 "ACL2____QUICK-AND-DIRTY-SUBSUMPTION-REPLACEMENT-STEP":[["Other"],"ACL2 Sources","ACL2","<p>This topic is probably only of interest to those who are \u000A undertaking particularly complex proof developments.</p> \u000A \u000A <p>The ACL2 prover's handling of propositional logic uses a heuristic that we \u000A believe might cause the prover to slow down significantly or even to trigger a \u000A stack overflow.  However, we believe these negative effects are only felt on \u000A very large formulas &mdash; formulas that are likely to cause similar \u000A degradation of many other parts of ACL2.</p> \u000A \u000A <p>The following two events turn off that heuristic (by eliminating calls to \u000A source function <v>quick-and-dirty-subsumption-replacement-step</v>, after which \u000A this <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic is named).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> quick-and-dirty-srs-off (cl1 ac)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> cl1 ac)\u000A           (<see topic=\"ACL2____XARGS\">xargs</see> :mode :logic :guard t))\u000A  nil)\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> quick-and-dirty-srs quick-and-dirty-srs-off)</code> \u000A \u000A <p>However, if you feel the need to try this out, please remember that the \u000A proof is likely to fail anyway since other parts of ACL2 will probably be \u000A stressed.  A more basic problem with your proof strategy may exist: the \u000A formulas are getting extraordinarily large.  A common approach for avoiding \u000A such problem include disabling functions (see <see topic=\"ACL2____IN-THEORY\">in-theory</see>).  Other less \u000A common approaches might help if you are sufficiently desperate, such as \u000A defining your own <v>IF</v> function to use in place of the built-in \u000A <v>IF</v>.</p> \u000A \u000A <p>If you do find an example for which the above two events provide \u000A significant benefit, we (the ACL2 implementors) would be interested in hearing \u000A about it.</p> \u000A \u000A <p>To turn the heuristic back on:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> quick-and-dirty-srs quick-and-dirty-srs-builtin)</code>"],

 "ACL2____OPEN-OUTPUT-CHANNEL_12":[["Io"],"ACL2 Sources","ACL2","<p>Use this function in place of <v>open-output-channel</v> if you want \u000A to open a channel for output at times this would otherwise be prohibited, for \u000A example during <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> expansion and <tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> <see topic=\"ACL2____HINTS\">hints</see>.  If this functionality doesn't quite seem like what you need, take a \u000A look at the definition of <v>open-output-channel!</v> in axioms.lisp, \u000A specifically the binding of <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable <v>writes-okp</v>. \u000A The following example, taken from community book \u000A <v>books/hons-archive/hons-archive.lisp</v>, illustrates the latter \u000A approach.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> har-zip! (x filename &amp;key sortp)\u000A  &quot;See :doc hons-archive&quot;\u000A  `(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A           (<see topic=\"ACL2____PROGN_12\">progn!</see>\u000A            :state-global-bindings\u000A            ((temp-touchable-vars t set-temp-touchable-vars))\u000A            (<see topic=\"ACL2____STATE-GLOBAL-LET_A2\">state-global-let*</see>\u000A             ((writes-okp t))\u000A             (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> (har-zip-fn ,x ,filename ,sortp state)))\u000A               (<see topic=\"ACL2____MV\">mv</see> nil nil state))))\u000A           (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> erp val))\u000A           state))</code> \u000A \u000A <p>The book below illustrates the soundness loophole plugged in ACL2 \u000A Version_3.2 related to file writes during book certification.</p> \u000A \u000A <code>; The following example is adapted (with only very slight changes)\u000A; from one written by Peter Dillinger.  It illustrates the prohibition\u000A; against writing files enforced by with-output-channel during book\u000A; certification (<see topic=\"ACL2____MORE\">more</see> specifically, during make-event expansion).\u000A\u000A; This book certifies in ACL2 Version_3.1 before the fix discussed in the\u000A; paragraph about it being ``possible to write files during book\u000A; certification'' in :DOC NOTE-3-2.  The fix was actually made to ACL2\u000A; function open-output-channel.\u000A\u000A; After the fix, in order for certification to succeed one needs to do\u000A; two things.  First, in raw lisp:\u000A;   (push :after-writes-okp-fix *features*)\u000A; Second, certify with this command:\u000A;   (<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &quot;writes-okp&quot; 0 nil :ttags (:writes-okp))\u000A\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> write-objects-to-channel (obj-lst chan state)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program\u000A                   :stobjs state\u000A                   :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> obj-lst)))\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> obj-lst)\u000A       (<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____PRINT-OBJECT_42\">print-object$</see> (<see topic=\"COMMON-LISP____CAR\">car</see> obj-lst) chan state)\u000A               (write-objects-to-channel (<see topic=\"COMMON-LISP____CDR\">cdr</see> obj-lst) chan state)\u000A               state)\u000A     state)))\u000A\u000A#+after-writes-okp-fix\u000A(<see topic=\"ACL2____DEFTTAG\">defttag</see> :writes-okp)\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> write-objects-to-file (obj-lst filename state)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program\u000A                   :stobjs state\u000A                   :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> filename)\u000A                               (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> obj-lst))))\u000A   (<see topic=\"ACL2____MV-LET\">mv-let</see> (chan state)\u000A           #-after-writes-okp-fix\u000A           (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL\">open-output-channel</see> filename :object state)\u000A           #+after-writes-okp-fix\u000A           (<see topic=\"ACL2____OPEN-OUTPUT-CHANNEL_12\">open-output-channel!</see> filename :object state)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> chan\u000A               (<see topic=\"ACL2____PPROGN\">pprogn</see> (write-objects-to-channel obj-lst chan state)\u000A                       (<see topic=\"ACL2____CLOSE-OUTPUT-CHANNEL\">close-output-channel</see> chan state)\u000A                       (value :done))\u000A             (<see topic=\"ACL2____ER\">er</see> soft 'write-object-to-file\u000A                 &quot;Could not open for writing: ~x0&quot;\u000A                 filename)))))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____DEFCONST\">defconst</see> *nil.lisp*\u000A   '((<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A     (<see topic=\"ACL2____DEFTHM\">defthm</see> bad nil :rule-classes nil))))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____DEFCONST\">defconst</see> *nil.cert*\u000A   '((<see topic=\"COMMON-LISP____IN-PACKAGE\">IN-PACKAGE</see> &quot;ACL2&quot;)\u000A     &quot;ACL2 Version 3.1&quot;\u000A     :BEGIN-PORTCULLIS-CMDS\u000A     :END-PORTCULLIS-CMDS\u000A     NIL\u000A     ((&quot;/home/peterd/test/nil.lisp&quot; &quot;nil&quot; &quot;nil&quot;\u000A       ((:SKIPPED-PROOFSP) (:AXIOMSP) (:TTAGS)) . 134094174))\u000A     62589544\u000A     )))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see>\u000A (<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____ER-PROGN\">er-progn</see>\u000A              (write-objects-to-file *nil.lisp* &quot;nil.lisp&quot; state)\u000A              (write-objects-to-file *nil.cert* &quot;nil.cert&quot; state)\u000A              (value '(<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> :invisible)))))\u000A\u000A(<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>\u000A        &quot;nil&quot; :load-compiled-file nil))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> bad nil :rule-classes nil)</code>"],

 "ACL2____NOTE-2-8-PROOF-CHECKER":[["Note-2-8"],"ACL2 Sources","ACL2","<p>Added new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>wrap1</v>, <v>wrap</v>, and \u000A <v>wrap-induct</v>.  <v>Wrap</v> replaces multiple goals by their conjunction: \u000A <v>(wrap instr1 instr2 ...)</v> employs <v>wrap1</v> so that the indicated \u000A instructions create only at most one new goal.  <v>Wrap-induct</v> is a simple \u000A example of the use of <v>wrap</v>, so that induction creates only one goal (the \u000A conjunction of the base and induction steps).  <v>Wrap1</v> can be used \u000A immediately after a prover call (<v>bash</v>, <v>prove</v>, <v>reduce</v>, <v>bdd</v>, \u000A or <v>induct</v>) to collapse the new goals into one.  See <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>=</v> failed to work as expected when a \u000A governing <v>IF</v>-test of the current term is T.  This has been fixed (by \u000A fixing source function <v>conjuncts-of</v>).  Thanks to Yoann Padioleau for \u000A bringing this problem to our attention.</p> \u000A \u000A <p>The <v>type-alist</v> command now takes optional arguments that control \u000A whether or not the governors and/or conclusion are used in computing the \u000A context that is printed (see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>, specifically \u000A subtopic <v>type-alist</v>).  Thanks to Rob Sumners for suggesting this \u000A improvement.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____TOGGLE-PC-MACRO\">toggle-pc-macro</see></tt> has always taken an optional second \u000A argument of <v>atomic-macro</v> or <v>macro</v>.  However, this was not clearly \u000A documented, and those two symbols had to be in the <v>ACL2</v> package.  Both of \u000A these problems have been remedied.  Thanks to John Erickson for bringing the \u000A lack of documentation of the second argument to our attention.</p>"],

 "ACL2____LD-PRE-EVAL-FILTER":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-pre-eval-filter</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>). \u000A The accessor is <v>(<see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see> state)</v> and the updater is \u000A <v>(set-ld-pre-eval-filter val state)</v>.  <v>Ld-pre-eval-filter</v> must be \u000A either <v>:all</v>, <v>:query</v>, or a new name that could be defined (e.g., by \u000A <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>).  The initial value of \u000A <v>ld-pre-eval-filter</v> is <v>:all</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-pre-eval-filter</v> is one of them.  If the filter is <v>:all</v>, then \u000A every form read is evaluated.  If the filter is <v>:query</v>, then after a form \u000A is read it is printed to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and the user is asked if the form \u000A is to be evaluated or skipped.  If the filter is a new name, then all forms \u000A are evaluated until that name is introduced, at which time <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A terminates normally.</p> \u000A \u000A <p>The <v>:all</v> filter is, of course, the normal one.  <v>:Query</v> is useful \u000A if you want to replay selected the <see topic=\"ACL2____COMMAND\">command</see>s in some file.  The new name \u000A filter is used if you wish to replay all the <see topic=\"ACL2____COMMAND\">command</see>s in a file up \u000A through the introduction of the given one.</p>"],

 "ACL2____LD-ERROR-TRIPLES":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-error-triples</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>). \u000A The accessor is <v>(<see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see> state)</v> and the updater is \u000A <v>(set-ld-error-triples val state)</v>.  <v>Ld-error-triples</v> must be either \u000A <v>t</v> or <v>nil</v>.  The initial value of <v>ld-error-triples</v> is <v>t</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-error-triples</v> is one of them.  If this variable has the value <v>t</v> \u000A then when a form evaluates to an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> (see \u000A <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) such that <v>erp</v> is non-<v>nil</v>, then an error is \u000A deemed to have occurred.  When an error occurs in evaluating a form, <tt><see topic=\"ACL2____LD\">ld</see></tt> rolls back the ACL2 <see topic=\"ACL2____WORLD\">world</see> to the configuration it had at the \u000A conclusion of the last error-free form.  Then <tt><see topic=\"ACL2____LD\">ld</see></tt> takes the action \u000A determined by <tt><see topic=\"ACL2____LD-ERROR-ACTION\">ld-error-action</see></tt>.</p>"],

 "ACL2____KEEP":[["Books-tour"],"ACL2 Sources","ACL2","<p>The certificate (see <see topic=\"ACL2____CERTIFICATE\">certificate</see> for general information) of \u000A a certified file is divided into two parts, a <see topic=\"ACL2____PORTCULLIS\">portcullis</see> and a keep. \u000A These names come from castle lore.  The keep is the strongest and usually \u000A tallest tower of a castle from which the entire courtyard can be surveyed by \u000A the defenders.  The keep of a book is a list of file names and check sums used \u000A after the book has been included, to determine if the files read were (up to \u000A check sum) those certified.</p> \u000A \u000A <p>Once the <see topic=\"ACL2____PORTCULLIS\">portcullis</see> is open, <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> can enter the book \u000A and read the event forms therein.  The non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event forms are in \u000A fact executed, extending the host theory.  That may read in other <see topic=\"ACL2____BOOKS\">books</see>.  When that has been finished, the keep of the <see topic=\"ACL2____CERTIFICATE\">certificate</see> is \u000A inspected.  The keep is a list of the book names which are included \u000A (hereditarily through all subbooks) in the certified book (including the \u000A certified book itself) together with the check sums of the objects in those \u000A <see topic=\"ACL2____BOOKS\">books</see> at the time of certification.  We compare the check sums of the \u000A <see topic=\"ACL2____BOOKS\">books</see> just included to the check sums of the <see topic=\"ACL2____BOOKS\">books</see> stored in the \u000A keep.  If differences are found then we know that the book or one of its \u000A subbooks has been changed since certification.</p> \u000A \u000A <p>See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> to continue the guided tour through <see topic=\"ACL2____BOOKS\">books</see>.</p>"],

 "ACL2____HONS-CLEAR":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-clear</v> just returns nil; we leave it enabled and would \u000A think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons-clear</v> brutally (1) clears all the tables that \u000A govern which conses are <see topic=\"ACL2____NORMED\">normed</see>, then (2) optionally garbage collects, \u000A per the <v>gc</v> argument, then finally (3) re-norms the keys of <tt><see topic=\"ACL2____FAST-ALISTS\">fast-alists</see></tt> and persistently normed conses; see <see topic=\"ACL2____HONS-COPY-PERSISTENT\">hons-copy-persistent</see>.</p> \u000A \u000A <p>Note.  The hash tables making up the Hons Space retain their sizes after \u000A being cleared.  If you want to shrink them, see <tt><see topic=\"ACL2____HONS-RESIZE\">hons-resize</see></tt>.</p> \u000A \u000A <p>Note.  CCL users might prefer <tt><see topic=\"ACL2____HONS-WASH\">hons-wash</see></tt>, which is relatively \u000A efficient and allows for the garbage collection of normed conses without \u000A impacting their normed status.</p> \u000A \u000A <p>Note.  It is not recommended to interrupt this function.  Doing so may \u000A cause persistently normed conses and fast alist keys to become un-normed, \u000A which might lead to less efficient re-norming and/or violations of the \u000A fast-alist discipline.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-clear</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-clear (gc)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> gc))\u000A       nil)</code>"],

 "ACL2____HARD-ERROR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____HARD-ERROR\">Hard-error</see> ctx str alist)</v> causes evaluation to halt with a \u000A short message using the ``context'' <v>ctx</v>.  An error message is first \u000A printed using the string <v>str</v> and alist <v>alist</v> that are of the same \u000A kind as expected by <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.  See <see topic=\"ACL2____FMT\">fmt</see>.  Also see <see topic=\"ACL2____ER\">er</see> for a \u000A macro that provides a unified way of signaling errors.</p> \u000A \u000A <p><v>Hard-error</v> has a guard of <v>t</v>.  Also see <see topic=\"ACL2____ILLEGAL\">illegal</see> for a \u000A similar capability which however has a guard of <v>nil</v> that supports static \u000A checking using <tt><see topic=\"ACL2____GUARD\">guard</see></tt> verification, rather than using dynamic \u000A (run-time) checking.  This distinction is illustrated elsewhere: see <see topic=\"ACL2____PROG2_42\">prog2$</see> for examples.</p> \u000A \u000A <p>Semantically, <v>hard-error</v> ignores its arguments and always returns \u000A <v>nil</v>.  But if a call <v>(<see topic=\"ACL2____HARD-ERROR\">hard-error</see> ctx str alist)</v> is encountered during \u000A evaluation, then the string <v>str</v> is printed using the association list \u000A <v>alist</v> (as in <tt><see topic=\"ACL2____FMT\">fmt</see></tt>), after which evaluation halts immediately. \u000A Here is a trivial, contrived example.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____CONS\">cons</see> 3 (<see topic=\"ACL2____HARD-ERROR\">hard-error</see> 'my-context\u000A                            &quot;Printing 4: ~n0&quot;\u000A                            (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 4))))\u000A\u000AHARD ACL2 ERROR in MY-CONTEXT:  Printing 4: four\u000A\u000AACL2 Error in TOP-LEVEL:  Evaluation aborted.\u000A\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Function: </b><srclink>hard-error</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hard-error (ctx str alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> ctx str alist))\u000A       nil)</code>"],

 "ACL2____GOOD-BYE":[["Other"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____GOOD-BYE\">good-bye</see>)\u000A; [ACL2 is exited]\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____GOOD-BYE\">good-bye</see> 3)\u000A; [ACL2 is exited with Unix exit status 3]</code> \u000A \u000A <p>Note: Your entire session will disappear forever when you evaluate \u000A <v>(<see topic=\"ACL2____GOOD-BYE\">good-bye</see>)</v>.</p> \u000A \u000A <p>The command <v>(<see topic=\"ACL2____GOOD-BYE\">good-bye</see>)</v> quits not only out of the ACL2 <see topic=\"ACL2____COMMAND\">command</see> \u000A loop, but in fact quits entirely out of the underlying Lisp.  Thus, there is \u000A no going back!  You will <b>not</b> be able to re-enter the <see topic=\"ACL2____COMMAND\">command</see> \u000A loop after typing <v>(<see topic=\"ACL2____GOOD-BYE\">good-bye</see>)</v>!  All your work will be lost!!!</p> \u000A \u000A <p>This command may not work in some underlying Common Lisp implementations. \u000A In such cases, there is no harm in trying; ACL2 will let you know how to \u000A proceed if it cannot exit.</p> \u000A \u000A <p>In some systems, typing <v>control-d</v> at the top-level ACL2 prompt \u000A (<v>control-c control-d</v> if inside emacs) will call this function.</p> \u000A \u000A <p>If you give <v>good-bye</v> an argument, it should be a natural number, and \u000A it indicates the Unix (Linux) exit status.</p> \u000A \u000A <p>If you merely want to exit the ACL2 <see topic=\"ACL2____COMMAND\">command</see> loop, use <v>:q</v> \u000A instead (see <see topic=\"ACL2____Q\">q</see>).</p>"],

 "ACL2-PC____LISP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(lisp (<see topic=\"ACL2____ASSIGN\">assign</see> xxx 3))\u000A\u000AGeneral Form:\u000A(lisp form)</code> \u000A \u000A <p>Evaluate <v>form</v>.  The <v>lisp</v> command is mainly of interest for side \u000A effects.  See also <v>print</v>, <v>skip</v>, and <v>fail</v>.</p> \u000A \u000A <p>The rest of the documentation for <v>lisp</v> is of interest only to those \u000A who use it in macro commands.  If the Lisp evaluation (by <v>trans-eval</v>) of \u000A form returns an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp\u000A ((NIL NIL STATE) . (erp-1 val-1 &amp;)) state)</v>, then the <v>lisp</v> command \u000A returns the appropriate error triple</p> \u000A \u000A <code>(<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____OR\">or</see> erp erp-1)\u000A    val-1\u000A    state) .</code> \u000A \u000A <p>Otherwise, the <v>trans-eval</v> of form must return an error triple of the \u000A form <v>(<see topic=\"ACL2____MV\">mv</see> erp (<see topic=\"COMMON-LISP____CONS\">cons</see> stobjs-out val) &amp;)</v>, and the <v>lisp</v> command returns \u000A the appropriate error triple</p> \u000A \u000A <code>(<see topic=\"ACL2____MV\">mv</see> erp\u000A    val\u000A    state).</code> \u000A \u000A <p>Note that the output signature of the form has been lost.  The user must \u000A know the signature in order to use the output of the <v>lisp</v> command. \u000A Trans-eval, which is undocumented except by comments in the ACL2 source code, \u000A has replaced, in <v>val</v>, any occurrence of the current state or the current \u000A values of stobjs by simple symbols such as <v>REPLACED-STATE</v>.  The actual \u000A values of these objects may be recovered, in principle, from the <v>state</v> \u000A returned and the <v>user-stobj-alist</v> within that state.  However, in \u000A practice, the stobjs cannot be recovered because the user is denied access to \u000A <v>user-stobj-alist</v>.  The moral is: do not try to write macro commands that \u000A manipulate stobjs.  Should the returned <v>val</v> contain <v>REPLACED-STATE</v> \u000A the value may simply be ignored and <v>state</v> used, since that is what \u000A <v>REPLACED-STATE</v> denotes.</p>"],

 "COMMON-LISP_____D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP_____D3\">=</see> x y)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v>.</p> \u000A \u000A <p>Unlike <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <v>=</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring both of its \u000A arguments to be numbers.  Generally, <v>=</v> is executed more efficiently than \u000A <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.</p> \u000A \u000A <p>For a discussion of the various ways to test against 0, See <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>=</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> = (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                                   (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))</code>"],

 "ACL2____UPDATE-NTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____UPDATE-NTH\">Update-nth</see> key val l)</v> returns a list that is the same as the \u000A list <v>l</v>, except that the value at the <v>0</v>-based position <v>key</v> \u000A (a natural number) is <v>val</v>.</p> \u000A \u000A <p>If <v>key</v> is an integer at least as large as the length of <v>l</v>, then \u000A <v>l</v> will be padded with the appropriate number of <v>nil</v> elements, as \u000A illustrated by the following example.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> 8 'z '(a b c d e))\u000A(A B C D E NIL NIL NIL Z)</code> \u000A \u000A <p>We have the following theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)\u000A              (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> key)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 key))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> key val l))\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> key (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))\u000A                    (<see topic=\"COMMON-LISP____LENGTH\">length</see> l)\u000A                  (<see topic=\"COMMON-LISP_____B2\">+</see> 1 key))))</code> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of <v>update-nth</v> requires that its first (position) \u000A argument is a natural number and its last (list) argument is a true list.</p> \u000A \u000A <p><b>Function: </b><srclink>update-nth</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> update-nth (key val l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))\u000A                (<see topic=\"COMMON-LISP____TYPE\">type</see> (integer 0 *) key))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> key) (<see topic=\"COMMON-LISP____CONS\">cons</see> val (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l)\u000A                      (<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> (<see topic=\"COMMON-LISP____1-\">1-</see> key) val (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____TAU-STATUS":[["Tau-system"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TAU-STATUS\">tau-status</see>)\u000A(<see topic=\"ACL2____TAU-STATUS\">tau-status</see> :system t)\u000A(<see topic=\"ACL2____TAU-STATUS\">tau-status</see> :auto-mode nil)\u000A(<see topic=\"ACL2____TAU-STATUS\">tau-status</see> :system t :auto-mode nil)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____TAU-STATUS\">tau-status</see> :system a :auto-mode b)</code> \u000A \u000A <p>where <v>a</v> and <v>b</v> are Booleans.  Both keyword arguments are optional \u000A and they may be presented in either order.  Value <v>a</v> controls whether the \u000A <tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt> is used during subsequent proofs.  Value <v>b</v> controls \u000A whether tau system rules are added automatically (``greedily'') when rules of \u000A other <tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> are added.  If no arguments are supplied, this is \u000A not an event and just returns an error-triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) \u000A indicating the current settings.  See <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> \u000A for background details.</p> \u000A \u000A <p>The two flags are independent.  For example, the tau system may be disabled \u000A in proof attempts even though it is automatically (and silently) extending its \u000A database as rules of other classes are added.</p> \u000A \u000A <p>Flag (a) is actually toggled by enabling or disabling the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of <tt><see topic=\"ACL2____TAU-SYSTEM\">tau-system</see></tt>.  Flag (b) is toggled with the \u000A function <tt><see topic=\"ACL2____SET-TAU-AUTO-MODE\">set-tau-auto-mode</see></tt>, which manipulates the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>.</p> \u000A \u000A <p>This macro expands into zero, one, or two <see topic=\"ACL2____EVENTS\">events</see>, as required by the \u000A supplied values of flags <v>a</v> and <v>b</v>.</p> \u000A \u000A <p>If no arguments are supplied the form is not an event and simply returns \u000A (as an error triple <v>(<see topic=\"ACL2____MV\">mv</see> nil ans state)</v>; see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) the \u000A current settings of the two flags.  For example:</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____TAU-SYSTEM\">tau-system</see>)\u000A ((:SYSTEM NIL) (:AUTO-MODE T))</code> \u000A \u000A <p>intended to be self-explanatory.</p>"],

 "ACL2____SET-SPLITTER-OUTPUT":[["Splitter"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> t)   ; enable  reports of ``splitter'' rules (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(<see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> nil) ; disable reports of ``splitter'' rules</code> \u000A \u000A <p>After evaluation of the form <v>(<see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> t)</v> (the default), \u000A then whenever <v>prove</v> output is not inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), ACL2 will report <see topic=\"ACL2____REWRITE\">rewrite</see> and <see topic=\"ACL2____DEFINITION\">definition</see> rules that may have reduced a goal to more than one subgoal.  See \u000A <see topic=\"ACL2____SPLITTER\">splitter</see> for how to interpret such reports.  We call such rules \u000A ``splitter rules'' for the goal that is being split.  This information can be \u000A useful in deciding how to eliminate large splits, for example of Goal into \u000A Subgoal 1000 through Subgoal 1, by disabling some splitter rules.  If you want \u000A to avoid the printing of such information, you can put the form \u000A <v>(<see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> t)</v> in your customization file; see <see topic=\"ACL2____ACL2-CUSTOMIZATION\">ACL2-customization</see>.</p> \u000A \u000A <p>Note that this command does not change the ACL2 <see topic=\"ACL2____WORLD\">world</see>; it only \u000A modifies the <see topic=\"ACL2____STATE\">state</see>.  More precisely, it sets a state global to the \u000A indicated value.  (See <see topic=\"ACL2____STATE\">state</see> for discussion of the ``global-table'' of \u000A the state.)  When <v>prove</v> output is enabled (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>), the value of that state global is the value of the \u000A form <v>(</v><tt><see topic=\"ACL2____SPLITTER-OUTPUT\">splitter-output</see></tt><v>)</v>; otherwise the value of that form is \u000A <v>nil</v>.</p> \u000A \u000A <p>Again, see <see topic=\"ACL2____SPLITTER\">splitter</see> for the effects of turning on the reporting of \u000A splitter rules.</p>"],

 "ACL2____SERIALIZE-READ":[["Serialize"],"ACL2 Sources","ACL2","<p>General form:</p> \u000A \u000A <code>(<see topic=\"ACL2____SERIALIZE-READ\">serialize-read</see> filename\u000A                [:hons-mode {:always, :never, :smart}]   ; :smart by default\u000A                [:verbosep  {t, nil}])                   ; nil by default\u000A --&gt;\u000A(<see topic=\"ACL2____MV\">mv</see> obj state)</code> \u000A \u000A <p>In the logic this is an oracle read.</p> \u000A \u000A <p>Under the hood, we try to read and return a serialized object from a file \u000A that was presumably created by <tt><see topic=\"ACL2____SERIALIZE-WRITE\">serialize-write</see></tt>.  On success we return \u000A the contents of the file.  Any failures (e.g., file not found, bad file \u000A contents, etc.) will result in a hard Lisp error.</p> \u000A \u000A <p>The <v>filename</v> should be a string that gives the path to the file.</p> \u000A \u000A <p>The <v>hons-mode</v> controls how whether to use <tt><see topic=\"ACL2____HONS\">hons</see></tt> or <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> \u000A to restore the object.  The default mode is <v>:smart</v>, which means that \u000A conses that were <see topic=\"ACL2____NORMED\">normed</see> at the time of the file's creation should be \u000A restored with <v>hons</v>.  But you can override this and insist that <v>hons</v> \u000A is to <v>:always</v> or <v>:never</v> be used, instead.</p> \u000A \u000A <p>Why would you use <v>:never</v>?  If your object previously had a lot of \u000A honses, but you no longer have any need for them to be normed, then using \u000A <v>:never</v> may sometimes be a lot faster since it can avoid <v>hons</v> calls. \u000A On the other hand, if you are going to <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> some part of the \u000A file's contents, then it is likely faster to use <v>:smart</v> or <v>:always</v> \u000A instead of first creating normal conses and then copying them to build \u000A honses.</p> \u000A \u000A <p>The <v>:verbosep</v> flag just controls whether to print some low-level \u000A details related to timing and memory usage as the file is being read.</p>"],

 "ACL2____SERIALIZE-IN-BOOKS":[["Serialize"],"ACL2 Sources","ACL2","<p>Our serialize scheme was developed in order to allow very large \u000A ACL2 objects to be loaded into books.  Ordinarily this is carried out using \u000A <tt><see topic=\"ACL2____SERIALIZE-READ\">serialize-read</see></tt> within a <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt>, e.g.,</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-EVENT\">make-event</see> (<see topic=\"ACL2____MV-LET\">mv-let</see> (obj state)\u000A                    (<see topic=\"ACL2____SERIALIZE-READ\">serialize-read</see> &quot;my-file&quot;)\u000A                    (value `(<see topic=\"ACL2____DEFCONST\">defconst</see> *my-file* ',obj))))</code> \u000A \u000A <p>But this scheme is not particularly efficient.</p> \u000A \u000A <p>During <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, the actual call of <v>serialize-read</v> is \u000A carried out, and this is typically pretty fast.  But then a number of possibly \u000A inefficient things occur.</p> \u000A \u000A <blockquote> \u000A \u000A <p>- The ACL2 function <v>bad-lisp-object</v> is run on the resulting object. \u000A This is memoized for efficiency, but may still take considerable time when the \u000A file is very large.</p> \u000A \u000A <p>- The checksum of the resulting object is computed.  This is also memoized, \u000A but as before may still take some time.</p> \u000A \u000A <p>- The object that was just read is then written into book.cert, essentially \u000A with <tt><see topic=\"ACL2____SERIALIZE-WRITE\">serialize-write</see></tt>.  This can take some time, and results in large \u000A certifiate files.</p></blockquote> \u000A \u000A <p>Then, during <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, the <v>make-event</v> expansion of is \u000A loaded.  This is now basically just a <v>serialize-read</v>.</p> \u000A \u000A <p>The moral of the story is that using serialize will only help your \u000A <v>certify-book</v> time, and it only impacts a portion of the overall time.</p> \u000A \u000A <p>To avoid this overhead, we have developed an UNSOUND alternative to \u000A <v>serialize-read</v>, which is available only by loading an additional book. \u000A So, if the above scheme is not performing well for you, you may wish to see \u000A <see topic=\"ACL2____UNSOUND-READ\">unsound-read</see>.</p>"],

 "ACL2____Numbers_02in_02ACL2":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>ACL2 numbers are precisely represented and unbounded.  They can be \u000A partitioned into the following subtypes:</p> \u000A \u000A <code> Rationals Integers Positive integers <v>3</v> Zero <v>0</v> Negative \u000AIntegers <v>-3</v> Non-Integral Rationals Positive Non-Integral Rationals \u000A<v>19/3</v> Negative Non-Integral Rationals <v>-22/7</v> Complex Rational Numbers \u000A<v>#c(3 5/2) ; = 3+(5/2)i</v> </code> \u000A \u000A <p>Signed integer constants are usually written (as illustrated above) as \u000A sequences of decimal digits, possibly preceded by <v>+</v> or <v>-</v>.  Decimal \u000A points are not allowed.  Integers may be written in binary, as in <v>#b1011</v> \u000A (= 23) and <v>#b-111</v> (= -7).  Octal may also be used, <v>#o-777</v> = -511. \u000A Non-integral rationals are written as a signed decimal integer and an unsigned \u000A decimal integer, separated by a slash.  Complex rationals are written as \u000A #c(rpart ipart) where rpart and ipart are rationals.</p> \u000A \u000A <p>Of course, 4/2 = 2/1 = 2 (i.e., not every rational written with a slash is \u000A a non-integer).  Similarly, #c(4/2 0) = #c(2 0) = 2.</p> \u000A \u000A <p>The common arithmetic functions and relations are denoted by <v>+</v>, \u000A <v>-</v>, <v>*</v>, <v>/</v>, <v>=</v>, <v>&lt;</v>, <v>&lt;=</v>, <v>&gt;</v> and <v>&gt;=</v>.  However \u000A there are many others, e.g., <v>floor</v>, <v>ceiling</v>, and <v>lognot</v>.  We \u000A suggest you see <see topic=\"ACL2____PROGRAMMING\">programming</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A where we list all of the primitive ACL2 functions.  Alternatively, see any \u000A Common Lisp language documentation.</p> \u000A \u000A <p>The primitive predicates for recognizing numbers are illustrated below. \u000A The following ACL2 function will classify an object, x, according to its \u000A numeric subtype, or else return 'NaN (not a number).  We show it this way just \u000A to illustrate programming in ACL2.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> classify-number (x)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x) 'positive-integer)\u000A                      ((<see topic=\"COMMON-LISP_____D3\">=</see> 0 x) 'zero)\u000A                      (t 'negative-integer)))\u000A               ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x) 'positive-non-integral-rational)\u000A               (t 'negative-non-integral-rational)))\u000A        ((<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> x) 'complex-rational)\u000A        (t 'NaN)))</code>"],

 "ACL2____DELETE-ASSOC":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key alist)\u000A(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key alist :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key alist :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key alist :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____DELETE-ASSOC\">Delete-assoc</see> key alist)</v> returns an alist that is the same as the list \u000A <v>alist</v>, except that the first pair in <v>alist</v> with a <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of \u000A <v>key</v> is deleted, if there is one; otherwise <v>alist</v> is returned.  Note \u000A that the order of the elements of <v>alist</v> is unchanged (though one may be \u000A deleted).</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>delete-assoc</v> depends on the test.  In \u000A all cases, the second argument must satisfy <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> \u000A (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument \u000A must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>delete-assoc</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____DELETE-ASSOC-EQ\">delete-assoc-eq</see> key alist)</v> is equivalent to \u000A <v>(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key alist :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____DELETE-ASSOC-EQUAL\">delete-assoc-equal</see> key alist)</v> is equivalent to <v>(<see topic=\"ACL2____DELETE-ASSOC\">delete-assoc</see> key\u000A alist :test 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>delete-assoc-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>delete-assoc-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> delete-assoc-equal (key alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> alist) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> key (<see topic=\"COMMON-LISP____CAAR\">caar</see> alist)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)\u000A                      (<see topic=\"ACL2____DELETE-ASSOC-EQUAL\">delete-assoc-equal</see> key (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist))))))</code>"],

 "ACL2____ALPHORDER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Alphorder</v> is a non-strict total order, a ``less than or \u000A equal,'' on atoms.  By ``non-strict total order'' we mean a function that \u000A always returns <v>t</v> or <v>nil</v> and satisfies the following properties.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o Antisymmetry: <v>XrY &amp; YrX -&gt; X=Y</v></p> \u000A \u000A <p>o Transitivity: <v>XrY &amp; YrZ -&gt; XrZ</v></p> \u000A \u000A <p>o Trichotomy: <v>XrY v YrX</v></p> \u000A \u000A </blockquote> \u000A \u000A <p>Also see <see topic=\"ACL2____LEXORDER\">lexorder</see>, which extends <v>alphorder</v> to all objects.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ALPHORDER\">Alphorder</see> x y)</v> has a guard of <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____ATOM\">atom</see> y))</v>.</p> \u000A \u000A <p>Within a single type: rationals are compared arithmetically, complex \u000A rationals are compared lexicographically, characters are compared via their \u000A char-codes, and strings and symbols are compared with alphabetic ordering. \u000A Across types, rationals come before complexes, complexes come before \u000A characters, characters before strings, and strings before symbols.  We also \u000A allow for ``bad atoms,'' i.e., atoms that are not legal Lisp objects but make \u000A sense in the ACL2 logic; these come at the end, after symbols.</p> \u000A \u000A <p><b>Function: </b><srclink>alphorder</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> alphorder (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____ATOM\">atom</see> y))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x y))\u000A                    (t t)))\u000A             ((<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y) nil)\u000A             ((<see topic=\"ACL2____COMPLEX_F2COMPLEX-RATIONALP\">complex/complex-rationalp</see> x)\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____COMPLEX_F2COMPLEX-RATIONALP\">complex/complex-rationalp</see> y)\u000A                     (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x) (<see topic=\"COMMON-LISP____REALPART\">realpart</see> y))\u000A                         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x) (<see topic=\"COMMON-LISP____REALPART\">realpart</see> y))\u000A                              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x) (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> y)))))\u000A                    (t t)))\u000A             ((<see topic=\"ACL2____COMPLEX_F2COMPLEX-RATIONALP\">complex/complex-rationalp</see> y) nil)\u000A             ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y)\u000A                     (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x) (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> y)))\u000A                    (t t)))\u000A             ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y) nil)\u000A             ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> y) (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRING_C3_D3\">string&lt;=</see> x y) t))\u000A                    (t t)))\u000A             ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> y) nil)\u000A             (t (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____SYMBOL-_C3\">symbol-&lt;</see> y x)))\u000A                             (t t)))\u000A                      ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y) nil)\u000A                      (t (bad-atom&lt;= x y))))))</code>"],

 "ACL2____ACL2-COUNT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ACL2-COUNT\">Acl2-count</see> x)</v> returns a nonnegative integer that indicates \u000A the ``size'' of its argument <v>x</v>.</p> \u000A \u000A <p>All <see topic=\"ACL2____CHARACTERS\">characters</see> and symbols have <v>acl2-count 0</v>.  The \u000A <v>acl2-count</v> of a string is the number of <see topic=\"ACL2____CHARACTERS\">characters</see> in it, i.e., \u000A its length.  The <v>acl2-count</v> of a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> is one greater than the sum \u000A of the <v>acl2-count</v>s of the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> and <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.  The \u000A <v>acl2-count</v> of an integer is its absolute value.  The <v>acl2-count</v> of a \u000A rational is the sum of the <v>acl2-count</v>s of the numerator and denominator. \u000A The <v>acl2-count</v> of a complex rational is one greater than the sum of the \u000A <v>acl2-count</v>s of the real and imaginary parts.</p> \u000A \u000A <p><b>Function: </b><srclink>acl2-count</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> acl2-count (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A              (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A               (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                   (integer-abs x)\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> (integer-abs (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x))\u000A                      (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)))\u000A               (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____COMPLEX_F2COMPLEX-RATIONALP\">complex/complex-rationalp</see> x)\u000A                   (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x))\u000A                      (<see topic=\"ACL2____ACL2-COUNT\">acl2-count</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x)))\u000A                   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x) (<see topic=\"COMMON-LISP____LENGTH\">length</see> x) 0)))))</code>"],

 "ACL2____The_02Proof_02of_02the_02Associativity_02of_02App":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Step_02to_02T'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Here is the theorem prover's output when it processes the <b>defthm</b> \u000A command for the associativity of <v>app</v>.  We have highlighted text for which \u000A we offer some explanation, and broken the presentation into several pages. \u000A (The most recent version of ACL2 may print slightly different output but the \u000A basics are the same.)  Just follow the Walking Tour after exploring the \u000A explanations.</p> \u000A \u000A <p>However, before exploring this output you should understand that ACL2 users \u000A rarely read successful proofs!  Instead, they look at certain subgoals printed \u000A in failed proofs, figure whether and how those subgoals can be proved, and \u000A give ACL2 directions for proving them, usually by simply proving other lemmas. \u000A Furthermore, to be a good user of ACL2 you do not have to understand how the \u000A theorem prover works.  You just have to understand how to interact with it. \u000A We explain this in great detail later.  But basically all new users are \u000A curious to know how ACL2 works and this little tour attempts to give some \u000A answers, just to satisfy your curiosity.</p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code>ACL2!&gt;<b>(defthm associativity-of-app</b> \u000A        <b>(equal (app (app a b) c)</b> \u000A               <b>(app a (app b c))))</b> \u000A \u000AName the formula above <see topic='ACL2____Name_02the_02Formula_02Above'>*1</see>. \u000A \u000A<see topic='ACL2____Perhaps'>Perhaps</see> we can prove *1 by induction.  Three induction schemes are \u000A<see topic='ACL2____Suggested_02Inductions_02in_02the_02Associativity_02of_02App_02Example'>suggested</see> by this conjecture.  <see topic='ACL2____Subsumption_02of_02Induction_02Candidates_02in_02App_02Example'>Subsumption</see> reduces that number to two. \u000AHowever, one of these is <see topic='ACL2____Flawed_02Induction_02Candidates_02in_02App_02Example'>flawed</see> and so we are left with one viable \u000Acandidate. \u000A \u000AWe will induct according to a scheme suggested by (APP A B).  If we \u000Alet  (:P A B C) denote *1 above then the induction scheme we'll use \u000Ais \u000A<see topic='ACL2____The_02Induction_02Scheme_02Selected_02for_02the_02App_02Example'>(</see>AND \u000A   <see topic='ACL2____The_02Induction_02Step_02in_02the_02App_02Example'>(</see>IMPLIES (AND (NOT (ENDP A)) \u000A                 (:P (CDR A) B C)) \u000A            (:P A B C)) \u000A   <see topic='ACL2____The_02Base_02Case_02in_02the_02App_02Example'>(</see>IMPLIES (ENDP A) (:P A B C))). \u000AThis induction is <see topic='ACL2____The_02Justification_02of_02the_02Induction_02Scheme'>justified</see> by the same argument used to admit APP, \u000Anamely, the measure (ACL2-COUNT A) is decreasing according to the relation \u000AO&lt; (which is known to be well-founded on the domain recognized \u000Aby O-P).  When <see topic='ACL2____The_02Instantiation_02of_02the_02Induction_02Scheme'>applied</see> to the goal at hand the above induction \u000Ascheme produces the following two <see topic='ACL2____Nontautological_02Subgoals'>nontautological subgoals</see>. \u000A \u000A</code> \u000A \u000A <p><see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Step_02to_02T'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<p>Examples:</p> \u000A \u000A <p><i>For use in <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> <see topic=\"ACL2____EVENTS\">events</see> or <v>:in-theory</v> <see topic=\"ACL2____HINTS\">hints</see>:</i><br/> \u000A <tt>(theory 'ground-zero)</tt></p> \u000A \u000A <p><i>For direct evaluation at the top-level loop:</i><br/> \u000A <tt>(let ((world (w state))) (theory 'ground-zero))</tt></p> \u000A \u000A <p>In the examples above, the theory returned is the one in force when ACL2 is \u000A started up (see <see topic=\"ACL2____GROUND-ZERO\">ground-zero</see>).</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____THEORY\">theory</see> name)</code> \u000A \u000A <p>where <v>name</v> is the name of a previously executed <tt><see topic=\"ACL2____DEFTHEORY\">deftheory</see></tt> \u000A event (otherwise a hard error occurs).  Returns the named theory.  See <see topic=\"ACL2____THEORIES\">theories</see>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____SIMPLE":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Example of simple rewrite rule:\u000A(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> x y)) x)\u000A\u000AExamples of simple definition:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> file-clock-p (x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x))\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> naturalp (x)\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0)))</code> \u000A \u000A <p>The theorem prover output sometimes refers to ``simple'' definitions and \u000A rewrite rules.  These rules can be used by the preprocessor, which is one of \u000A the theorem prover's ``processes'' understood by the <v>:do-not</v> hint; see \u000A <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>The preprocessor expands certain definitions and uses certain rewrite rules \u000A that it considers to be ``fast''.  There are two ways to qualify as fast.  One \u000A is to be an ``abbreviation'', where a rewrite rule with no hypotheses or loop \u000A stopper is an ``abbreviation'' if the right side contains no more variable \u000A occurrences than the left side, and the right side does not call the functions \u000A <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt> or <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt>.  Definitions and rewrite rules can \u000A both be abbreviations; the criterion for definitions is similar, except that \u000A the definition must not be recursive.  The other way to qualify applies only \u000A to a non-recursive definition, and applies when its body is a disjunction or \u000A conjunction, according to a perhaps subtle criterion that is intended to avoid \u000A case splits.</p>"],

 "ACL2____SET-PRINT-CASE":[["Io"],"ACL2 Sources","ACL2","<p>By default, symbols are printed in upper case when vertical bars \u000A are not required, as specified by Common Lisp.  As with Common Lisp, ACL2 \u000A supports printing in a \"downcase\" mode, where symbols are printed in lower \u000A case.  Many printing functions (some details below) print characters in lower \u000A case for a symbol when the ACL2 <see topic=\"ACL2____STATE\">state</see> global variable <v>print-case</v> \u000A has value <v>:downcase</v> and vertical bars are not necessary for printing that \u000A symbol.  (Thus, this state global functions in complete analogy to the Common \u000A Lisp global <v>*print-case*</v>.)  The value <v>print-case</v> is returned by \u000A <v>(print-case)</v>, and may be set using the function <v>set-print-case</v> as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> :upcase   state) ; Default printing\u000A(<see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> :downcase state) ; Print symbols in lower case when\u000A                                 ; vertical bars are not required</code> \u000A \u000A <p>The ACL2 user can expect that the <v>:downcase</v> setting will have an \u000A effect for formatted output (see <see topic=\"ACL2____FMT\">fmt</see> and see <see topic=\"ACL2____FMS\">fms</see>) when the \u000A directives are <v>~p</v>, <v>~P</v>, <v>~q</v>, or <v>~Q</v>, for built-in functions \u000A <v>princ$</v> and <v>prin1$</v>, and the <v>ppr</v> family of functions, and \u000A <i>not</i> for built-in function <v>print-object$</v>.  For other printing \u000A functions, the effect of <v>:downcase</v> is unspecified.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see> for other user-settable print controls.</p>"],

 "ACL2____SET-INHIBITED-SUMMARY-TYPES":[["Prover-output"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see> '(rules time))</code> \u000A \u000A <p>Note: This is not an event.  Rather, it changes the <see topic=\"ACL2____STATE\">state</see>, in \u000A analogy to <tt><see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see></tt>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-INHIBITED-SUMMARY-TYPES\">set-inhibited-summary-types</see> form)</code> \u000A \u000A <p>where form evaluates to a true-list of symbols, each of which is among the \u000A values of the constant <v>*summary-types*</v>, i.e.: <v>header</v>, <v>form</v>, \u000A <v>rules</v>, <v>hint-events</v> <v>warnings</v>, <v>time</v>, <v>steps</v>, <v>value</v>, \u000A and <v>splitter-rules</v>.  Each specified type inhibits printing of the \u000A corresponding portion of the summaries printed at the conclusions of <see topic=\"ACL2____EVENTS\">events</see>, where <v>header</v> refers to an initial newline followed by the line \u000A containing just the word <v>Summary</v>.</p> \u000A \u000A <p>Note the distinction between <v>rules</v> and <v>hint-events</v>.  <v>Rules</v> \u000A provides a record of automatic rule usage by the prover, while \u000A <v>hint-events</v> shows the names of events given to <v>:USE</v> or <v>:BY</v> \u000A <see topic=\"ACL2____HINTS\">hints</see>, as well as <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see> functions given to \u000A <v>:CLAUSE-PROCESSOR</v> hints that have an effect on the proof.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.  Note that \u000A <v>set-inhibited-summary-types</v> has no effect when <v>summary</v> is one of the \u000A types inhibited by <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>, because in that case none of \u000A the summary will be printed.</p> \u000A \u000A <p>To control summary types for a single event, see <see topic=\"ACL2____WITH-OUTPUT\">with-output</see>.</p>"],

 "ACL2____REMOVE-DEFAULT-HINTS":[["Default-hints"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see> '((computed-hint-1 clause)\u000A                        (computed-hint-2 clause\u000A                                         stable-under-simplificationp)))</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.  It is \u000A <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it occurs (see \u000A <see topic=\"ACL2____REMOVE-DEFAULT-HINTS_12\">remove-default-hints!</see> for a corresponding non-<tt><see topic=\"ACL2____LOCAL\">local</see></tt> event).</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see> lst)</code> \u000A \u000A <p>where <v>lst</v> is a list.  Generally speaking, the elements of <v>lst</v> \u000A should be suitable for use as <tt><see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see></tt>.  Also see <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see>.</p> \u000A \u000A <p>If some elements of the given list do not belong to the existing default \u000A hints, they will simply be ignored by this event.</p> \u000A \u000A <p>Also See <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>, see <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see>, and see \u000A <see topic=\"ACL2____DEFAULT-HINTS\">default-hints</see>.</p> \u000A \u000A <p>Finally, note that the effects of <v>set-default-hints</v>, <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see></tt>, and <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see></tt> are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A book in which they appear.  Thus, users who include a book with such forms \u000A will not have their default hints affected by such forms.  In order to export \u000A the effect of setting the default hints, use <tt><see topic=\"ACL2____SET-DEFAULT-HINTS_12\">set-default-hints!</see></tt>, \u000A <tt><see topic=\"ACL2____ADD-DEFAULT-HINTS_12\">add-default-hints!</see></tt>, or <tt><see topic=\"ACL2____REMOVE-DEFAULT-HINTS_12\">remove-default-hints!</see></tt>.</p>"],

 "ACL2____NOTE2":[["Release-notes"],"ACL2 Sources","ACL2","<p>Hacker mode has been eliminated and <see topic=\"ACL2____PROGRAMMING\">programming</see> mode has \u000A been added.  <see topic=\"ACL2____PROGRAMMING\">Programming</see> mode is unsound but does syntax checking and \u000A permits redefinitions of names.  See <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> <v>load-mode</v> and \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> <v>g-mode</v>.</p> \u000A \u000A <p>The arguments to <tt><see topic=\"ACL2____LD\">ld</see></tt> have changed.  <tt><see topic=\"ACL2____LD\">Ld</see></tt> is now much more \u000A sophisticated.  See <see topic=\"ACL2____LD\">ld</see>.</p> \u000A \u000A <p>For those occasions on which you wish to look at a large list structure \u000A that you are afraid to print, try <v>(<see topic=\"ACL2____WALKABOUT\">walkabout</see> x state)</v>, where <v>x</v> is an \u000A Acl2 expression that evaluates to the structure in question.  I am afraid \u000A there is no <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> yet, but it is similar in spirit to the \u000A Interlisp structure editor.  You are standing on an object and commands move \u000A you around in it.  E.g., 1 moves you to its first element, 2 to its second, \u000A etc.; 0 moves you up to its parent; <v>nx</v> and <v>bk</v> move you to its next \u000A sibling and previous sibling; <v>pp</v> prettyprints it; <tt><see topic=\"ACL2____Q\">q</see></tt> exits \u000A returning <v>nil</v>; <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt> exits returning the thing you're standing on; \u000A <v>(<see topic=\"COMMON-LISP_____D3\">=</see> symb)</v> assigns the thing you're standing on to the <see topic=\"ACL2____STATE\">state</see> global \u000A variable <v>symb</v>.</p> \u000A \u000A <p>Several new <see topic=\"ACL2____HINTS\">hints</see> have been implemented, including <v>:by</v> and \u000A <v>:do-not</v>.  The old <v>:do-not-generalize</v> has been scrapped in favor of \u000A such new <see topic=\"ACL2____HINTS\">hints</see> as <v>:do-not</v> <v>(<see topic=\"ACL2____GENERALIZE\">generalize</see> elim)</v>.  <v>:By</v> lets \u000A you say ``this goal is subsumed by'' a given lemma instance.  The <v>:by</v> \u000A hint also lets you say ``this goal can't be proved yet but skip it and see how \u000A the rest of the proof goes.'' See <see topic=\"ACL2____HINTS\">hints</see>.</p>"],

 "ACL2____NOTE-2-4":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important changes:</p> \u000A \u000A <p>We corrected a soundness bug in Version 2.3 related to the handling of \u000A <tt><see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see></tt>.  The bad behavior was noticed by Robert Krug. \u000A Thanks!</p> \u000A \u000A <p>We corrected a bug that permitted <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> to accept a \u000A function even though a subfunction had not yet had its guards verified. \u000A Thanks to John Cowles for noticing this.</p> \u000A \u000A <p>User defined single-threaded objects are now supported.  See <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>Less important notes:</p> \u000A \u000A <p>We corrected a bug that prevented the intended expansion of some recursive \u000A function calls.</p> \u000A \u000A <p>We changed the handling of the primitive function <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt>, which is \u000A logically defined to be <v>nil</v> but which is programmed to signal an error, \u000A so that when it is evaluated as part of a proof, it does not signal an error. \u000A The old handling of the function prevented some guard proofs involving <tt><see topic=\"COMMON-LISP____THE\">the</see></tt> or <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>s with internal declarations.</p> \u000A \u000A <p>We corrected a bug that permitted some <v>LOCAL</v> <v>DEFAXIOM</v> events to \u000A slip into certified books.</p> \u000A \u000A <p>We corrected a bug that prevented the correct undoing of certain \u000A <v>DEFPKG</v> forms.</p> \u000A \u000A <p>Changes were made to support CMU Lisp.  Pete Manolios helped with these \u000A changes.</p> \u000A \u000A <p>Changes were made to make the make files more compatible with Allegro \u000A Common Lisp.  Jun Sawada, who has been a great help with keeping ACL2 up and \u000A running at UT on various platforms, was especially helpful.  Thanks Jun.</p>"],

 "ACL2____LD-VERBOSE":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-verbose</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____LD-VERBOSE\">ld-verbose</see> state)</v> and the updater is <v>(set-ld-verbose val\u000A state)</v>.  <v>Ld-verbose</v> must be <v>t</v>, <v>nil</v> or a string or <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> suitable for <tt><see topic=\"ACL2____FMT\">fmt</see></tt> printing via the <v>~@</v> command.  The initial \u000A value of <v>ld-verbose</v> is a <tt><see topic=\"ACL2____FMT\">fmt</see></tt> message that prints the ACL2 version \u000A number, <tt><see topic=\"ACL2____LD\">ld</see></tt> level and connected book directory.</p> \u000A \u000A <p>Note: <v>Ld-verbose</v> has no effect on proofs.  See <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> and \u000A see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see> for how to control the size of proof \u000A output.</p> \u000A \u000A <p>Before processing the forms in <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, <tt><see topic=\"ACL2____LD\">ld</see></tt> may print a \u000A header.  The printing of this header is controlled by <v>ld-verbose</v>.  If \u000A <v>ld-verbose</v> is <v>nil</v>, no header is printed.  If it is <v>t</v>, <tt><see topic=\"ACL2____LD\">ld</see></tt> prints the message</p> \u000A \u000A <code>ACL2 loading &lt;file&gt;</code> \u000A \u000A <p>where <v>&lt;file&gt;</v> is the input channel supplied to <tt><see topic=\"ACL2____LD\">ld</see></tt>.  A similar \u000A message is printed when <tt><see topic=\"ACL2____LD\">ld</see></tt> completes.  If <v>ld-verbose</v> is neither \u000A <v>t</v> nor <v>nil</v> then it is presumably a header and is printed with the \u000A <v>~@</v> <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive before <tt><see topic=\"ACL2____LD\">ld</see></tt> begins to read and process \u000A forms.  In this case the <v>~@</v> <tt><see topic=\"ACL2____FMT\">fmt</see></tt> directive is interpreted in an \u000A environment in which <v>#\\v</v> is the ACL2 version string, <v>#\\l</v> is the \u000A level of the current recursion in <tt><see topic=\"ACL2____LD\">ld</see></tt> and/or <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>, and \u000A <v>#\\c</v> is the connected book directory <v>(<see topic=\"ACL2____CBD\">cbd</see>)</v>.</p>"],

 "ACL2____INITIALIZE-EVENT-USER":[["Prover-output"],"ACL2 Sources","ACL2","<p>This utility is intended for system hackers, not standard ACL2 \u000A users.</p> \u000A \u000A <p>See <see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see> to see how to supply code to be run at the \u000A end of <see topic=\"ACL2____EVENTS\">events</see>.  We assume familiarity with <v>finalize-event-user</v>; \u000A here we focus on how to supply code for the beginning as well as the end of \u000A events.</p> \u000A \u000A <p>As with <see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see>, you attach your own function of \u000A argument list <v>(ctx qbody state)</v> to <v>initialize-event-user</v>.  (See \u000A <see topic=\"ACL2____FINALIZE-EVENT-USER\">finalize-event-user</see> for discussion of <v>ctx</v> and <v>body</v>.)  The \u000A attachment should return <v>state</v> and have a trivial guard, requiring \u000A (implicitly) only that <v>state</v> satisfies <v>state-p</v> unless you use trust \u000A tags to avoid that requirement.  For example:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFATTACH\">defattach</see> initialize-event-user initialize-event-user-test)</code> \u000A \u000A <p>Why would you want to do this?  Presumably you are building a system on top \u000A of ACL2 and you want to track your own data.  For example, suppose you want to \u000A save the time in some <see topic=\"ACL2____STATE\">state</see> global variable, <v>my-time</v>.  You could \u000A do the following.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-init (ctx body state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                  :guard-hints\u000A                  ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> read-run-time))))\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> ctx body))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (seconds state)\u000A          (<see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see> state)\u000A          (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'start-time seconds state)))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-final (ctx body state)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                  :guard-hints\u000A                  ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____ENABLE\">enable</see> read-run-time))))\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> ctx body))\u000A  (<see topic=\"ACL2____MV-LET\">mv-let</see> (seconds state)\u000A          (<see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see> state)\u000A          (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"COMMON-LISP____IF\">if</see> (boundp-global 'start-time state)\u000A                      (<see topic=\"ACL2____CW\">cw</see> &quot;Time: ~x0 seconds~%&quot;\u000A                          (<see topic=\"COMMON-LISP____-\">-</see> seconds (<see topic=\"ACL2____FIX\">fix</see> (<see topic=\"ACL2_____04\">@</see> start-time))))\u000A                    (<see topic=\"ACL2____CW\">cw</see> &quot;BIG SURPRISE!~%&quot;))\u000A                  (<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'end-time seconds state))))\u000A\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> initialize-event-user my-init)\u000A(<see topic=\"ACL2____DEFATTACH\">defattach</see> finalize-event-user my-final)</code> \u000A \u000A <p>Here is an abbreviated log, showing the time being printed at the end.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) x y x y x y x y)\u000A                   (<see topic=\"COMMON-LISP____APPEND\">append</see> x y x y x y x y x y)))\u000A\u000A*1 (<see topic=\"COMMON-LISP____THE\">the</see> initial Goal, a key checkpoint) is pushed for proof by induction.\u000A....\u000AACL2 Error in ( THM ...):  See :DOC failure.\u000A\u000A******** FAILED ********\u000ATime: 869/100 seconds\u000AACL2 !&gt;</code>"],

 "ACL2____GENERALIZE":[["Rule-classes"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for a general discussion of rule classes, \u000A including how they are used to build rules from formulas and a discussion of \u000A the various keywords in a rule class description.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> integer-listp-rev\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> x)\u000A           (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> (rev x)))\u000A  :rule-classes :generalize)\u000A\u000AGeneral Form:\u000Aany theorem</code> \u000A \u000A <p>To use a <v>:generalize</v> rule, the system waits until it has decided to \u000A generalize some term, <v>term</v>, by replacing it with some new variable \u000A <v>v</v>.  If any <v>:generalize</v> formula can be instantiated so that some \u000A non-variable subterm becomes <v>term</v>, then that instance of the formula is \u000A added as a hypothesis.  Thus for the example above, if the term <v>(rev x2)</v> \u000A is generalized to the variable <v>rv</v> during a proof, then the following is \u000A added as a hypothesis when generalizing to a new goal.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> x2)\u000A         (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> rv))</code> \u000A \u000A <p>At the moment, the best description of how ACL2 <v>:generalize</v> rules are \u000A used may be found in the discussion of ``Generalize Rules,'' page 248 of A \u000A Computational Logic Handbook, or ``Generalization,'' page 132 of \u000A ``Computer-Aided Reasoning: An Approach.''  Also see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see> for detailed tutorial on using ACL2 to \u000A prove theorems, which includes some discussion of generalization.</p>"],

 "ACL2____FREE-VARIABLES-EXAMPLES-FORWARD-CHAINING":[["Free-variables-examples"],"ACL2 Sources","ACL2","<p>The following examples illustrate ACL2's handling of free variables \u000A in <see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see> rules, as well as user control over how such free \u000A variables are handled.  See <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> for a background \u000A discussion.</p> \u000A \u000A <code>; First let us introduce a transitive operation, op, and prove a\u000A; forward-chaining rule stating the transitivity of op.\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((op * *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op (x y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-op\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op x y) (op y z)) (op x z))\u000A   :rule-classes :forward-chaining))\u000A\u000A; The following theorem is proved by forward chaining, using the above rule.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v w) (op v a))\u000A          (op u w)))\u000A\u000A; The proof of the theorem just above succeeds because the term (op u v)\u000A; triggers the application of forward-chaining rule transitivity-of-op,\u000A; binding x to u and y to v.  Free variable z of that rule is bound to both w\u000A; and to a, resulting in the addition of both (op u w) and (op u a) to the\u000A; context.  However, (op v a) happens to be at the front of the context, so\u000A; if only one free-variable binding had been allowed, then z would have only\u000A; been bound to a, not to w, as we now illustrate.\u000A\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once (:forward-chaining transitivity-of-op))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> ; FAILS!\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v w) (op v a))\u000A          (op u w)))\u000A\u000A:ubt! 1\u000A\u000A; Starting over, this time we prove transitivity-of-op as a :match-free :once\u000A; forward-chaining rule.  Note that the presence of :match-free eliminates\u000A; the free-variables warning that we got the first time.\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((op * *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op (x y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-op\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op x y) (op y z)) (op x z))\u000A   :rule-classes ((:forward-chaining :match-free :once))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> ; FAILS!\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v w) (op v a))\u000A          (op u w)))\u000A\u000A; Notice that if we swap the order of the last two hypotheses the theorem\u000A; goes through, because this time (op v w) is first in the context.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> ; SUCCEEDS!\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v a) (op v w))\u000A          (op u w)))\u000A\u000A:u\u000A\u000A; Now let's try setting the default to :once.\u000A\u000A(<see topic=\"ACL2____SET-MATCH-FREE-DEFAULT\">set-match-free-default</see> :once)\u000A\u000A; We still get a free-variables warning when we admit this forward-chaining rule.\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((op * *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op (x y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> transitivity-of-op\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op x y) (op y z)) (op x z))\u000A   :rule-classes ((:forward-chaining))))\u000A\u000A; This theorem fails--as it should.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> ; FAILS!\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v w) (op v a))\u000A          (op u w)))\u000A\u000A; But if we convert this rule (<see topic=\"COMMON-LISP____OR\">or</see> here, all possible rules) to :all rules,\u000A; then the proof succeeds.\u000A\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :all t)\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> ; SUCCEEDS!\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op u v) (op v w) (op v a))\u000A          (op u w)))\u000A\u000A; Now let's test a relatively slow :all case (<see topic=\"COMMON-LISP____THE\">the</see> next thm below).\u000A\u000A:ubt! 1\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((op1 *) =&gt; *)\u000A  ((op3 * * *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op1 (x) (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x)) t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op3 (x0 x1 x2)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x0 x1 x2))\u000A          t))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> op1-op3-property\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op1 x0) (op1 x1) (op1 x2))\u000A            (op3 x0 x1 x2))\u000A   :rule-classes ((:forward-chaining :match-free :all))))\u000A\u000A; The following succeeds, but takes a little time (about a second in one run).\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (op1 a0) (op1 a1) (op1 a2) (op1 a3) (op1 a4) (op1 a5)\u000A           (op1 a6) (op1 a7) (op1 a8) (op1 a9) (op1 a10) (op1 a11)\u000A           (op1 a12) (op1 a13) (op1 a14) (op1 a15) (op1 a16)\u000A           (op1 a17) (op1 a18) (op1 a19) (op1 a20))\u000A      (op3 a5 a6 a0)))\u000A\u000A(<see topic=\"ACL2____ADD-MATCH-FREE-OVERRIDE\">add-match-free-override</see> :once t)\u000A\u000A; The same theorem now fails because of the add-match-free-override, but is\u000A; more than an order of magnitude faster.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (op1 a0) (op1 a1) (op1 a2) (op1 a3) (op1 a4) (op1 a5)\u000A           (op1 a6) (op1 a7) (op1 a8) (op1 a9) (op1 a10) (op1 a11)\u000A           (op1 a12) (op1 a13) (op1 a14) (op1 a15) (op1 a16)\u000A           (op1 a17) (op1 a18) (op1 a19) (op1 a20))\u000A      (op3 a5 a6 a0)))\u000A\u000A; A slight variant succeeds in a negligible amount of time (still with the\u000A; :once override above).\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see>\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (op1 a0) (op1 a1) (op1 a2) (op1 a3) (op1 a4) (op1 a5)\u000A           (op1 a6) (op1 a7) (op1 a8) (op1 a9) (op1 a10) (op1 a11)\u000A           (op1 a12) (op1 a13) (op1 a14) (op1 a15) (op1 a16)\u000A           (op1 a17) (op1 a18) (op1 a19) (op1 a20))\u000A      (op3 a5 a20 a20)))\u000A\u000A; Reality check: This shouldn't give a free-variables warning, and everything\u000A; should work great since there are no free variables with this trigger term.\u000A\u000A:ubt! 1\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((op1 *) =&gt; *)\u000A  ((op7 * * * * * * *) =&gt; *))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op1 (x)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x))\u000A          t))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> op7 (x0 x1 x2 x3 x4 x5 x6)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> x0 x1 x2 x3 x4 x5 x6))\u000A          t))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> op1-op7-property\u000A   (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op1 x0) (op1 x1) (op1 x2)\u000A                 (op1 x3) (op1 x4) (op1 x5) (op1 x6))\u000A            (op7 x0 x1 x2 x3 x4 x5 x6))\u000A   :rule-classes ((:forward-chaining\u000A                   :trigger-terms ((op7 x0 x1 x2 x3 x4 x5 x6))))))\u000A\u000A; The following then succeeds, and very quickly.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (op1 a0) (op1 a1) (op1 a2)\u000A                   (op1 a3) (op1 a4) (op1 a5) (op1 a6))\u000A              (op7 a4 a6 a5 a6 a6 a6 a6)))</code>"],

 "ACL2____ER-PROGN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ER-PROGN\">er-progn</see> (check-good-foo-p (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'my-foo state) state)\u000A          (value (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'my-foo state)\u000A                    (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'bar state))))</code> \u000A \u000A <p>This sequencing primitive is only useful when programming with <see topic=\"ACL2____STATE\">state</see>, something that very few users will probably want to do.  See <see topic=\"ACL2____STATE\">state</see>.</p> \u000A \u000A <p><v>Er-progn</v> is used much the way that <tt><see topic=\"COMMON-LISP____PROGN\">progn</see></tt> is used in Common \u000A Lisp, except that it expects each form within it to evaluate to an ``error \u000A triple'' of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>; see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>.  The \u000A first such form, if any, that evaluates to such a triple where <v>erp</v> is not \u000A <v>nil</v> yields the error triple returned by the <v>er-progn</v>.  If there is \u000A no such form, then the last form returns the value of the <v>er-progn</v> \u000A form.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ER-PROGN\">er-progn</see> &lt;expr1&gt; ... &lt;exprk&gt;)</code> \u000A \u000A <p>where each <v>&lt;expri&gt;</v> is an expression that evaluates to an error triple \u000A (see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>).  The above form is essentially equivalent \u000A to the following (``essentially'' because in fact, care is taken to avoid \u000A variable capture).</p> \u000A \u000A <code>(<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A        &lt;expr1&gt;\u000A        (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp val state))\u000A              (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A                         &lt;expr2&gt;\u000A                         (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp val state))\u000A                               (t ...\u000A                                      (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A                                              &lt;expr{k-1}&gt;\u000A                                              (<see topic=\"COMMON-LISP____COND\">cond</see> (erp (<see topic=\"ACL2____MV\">mv</see> erp val state))\u000A                                                    (t &lt;exprk&gt;)))))))))</code>"],

 "ACL2____DEBUGGING":[["Top","ACL2"],"ACL2 Sources","ACL2",""],

 "ACL2____About_02the_02Admission_02of_02Recursive_02Definitions":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>You can't just add any formula as an axiom or definition and expect \u000A the logic to stay sound!  For example, if we were permitted to define <v>(APP\u000A X Y)</v> so that it was equal to <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (APP X Y))</v> then we could prove \u000A anything.  The purported ``definition'' of <v>APP</v> must have several \u000A properties to be admitted to the logic as a new axiom.</p> \u000A \u000A <p>The key property a recursive definition must have is that the recursion \u000A terminate.  This, along with some syntactic criteria, ensures us that there \u000A exists a function satisfying the definition.</p> \u000A \u000A <p>Termination must be proved before the definition is admitted.  This is done \u000A in general by finding a measure of the arguments of the function and a \u000A well-founded relation such that the arguments ``get smaller'' every time a \u000A recursive branch is taken.</p> \u000A \u000A <p>For <v>app</v> the measure is the ``size'' of the first argument, <v>x</v>, as \u000A determined by the primitive function <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A The well-founded relation used in this example is <tt><see topic=\"ACL2____O-P\">o-p</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A which is the standard ordering on the ordinals less than ``epsilon naught.'' \u000A These particular choices for <v>app</v> were made ``automatically'' by ACL2. \u000A But they are in fact determined by various ``default'' settings.  The user of \u000A ACL2 can change the defaults or specify a ``hint'' to the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A command to specify the measure and relation.</p> \u000A \u000A <p>You should now return to <see topic='ACL2____Revisiting_02the_02Admission_02of_02App'>the Walking Tour</see>.</p>"],

 "COMMON-LISP____DIGIT-CHAR-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____DIGIT-CHAR-P\">digit-char-p</see> ch)</v> is the integer corresponding to the \u000A character <v>ch</v> in base <v>10</v>.  For example, <v>(<see topic=\"COMMON-LISP____DIGIT-CHAR-P\">digit-char-p</see> #\\3)</v> is \u000A equal to the integer <v>3</v>.  More generally, an optional second argument \u000A specifies the radix (default <v>10</v>, as indicated above).</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>digit-char-p</v> (more precisely, for the function \u000A <v>our-digit-char-p</v> that calls of this macro expand to) requires its second \u000A argument to be an integer between 2 and 36, inclusive, and its first argument \u000A to be a character.</p> \u000A \u000A <p><v>Digit-char-p</v> is a Common Lisp function, though it is implemented in \u000A the ACL2 logic as an ACL2 macro.  See any Common Lisp documentation for more \u000A information.</p> \u000A \u000A <p><b>Macro: </b><srclink>digit-char-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> digit-char-p (ch &amp;optional (radix '10))\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 'our-digit-char-p\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> ch (<see topic=\"COMMON-LISP____CONS\">cons</see> radix 'nil))))</code> \u000A <p><b>Function: </b><srclink>our-digit-char-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> our-digit-char-p (ch radix)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> ch)\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> radix)\u000A                                   (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 2 radix)\u000A                                   (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> radix 36))))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((l (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> ch\u000A                       '((#\\0 . 0)\u000A                         (#\\1 . 1)\u000A                         (#\\2 . 2)\u000A                         (#\\3 . 3)\u000A                         (#\\4 . 4)\u000A                         (#\\5 . 5)\u000A                         (#\\6 . 6)\u000A                         (#\\7 . 7)\u000A                         (#\\8 . 8)\u000A                         (#\\9 . 9)\u000A                         (#\\a . 10)\u000A                         (#\\b . 11)\u000A                         (#\\c . 12)\u000A                         (#\\d . 13)\u000A                         (#\\e . 14)\u000A                         (#\\f . 15)\u000A                         (#\\g . 16)\u000A                         (#\\h . 17)\u000A                         (#\\i . 18)\u000A                         (#\\j . 19)\u000A                         (#\\k . 20)\u000A                         (#\\l . 21)\u000A                         (#\\m . 22)\u000A                         (#\\n . 23)\u000A                         (#\\o . 24)\u000A                         (#\\p . 25)\u000A                         (#\\q . 26)\u000A                         (#\\r . 27)\u000A                         (#\\s . 28)\u000A                         (#\\t . 29)\u000A                         (#\\u . 30)\u000A                         (#\\v . 31)\u000A                         (#\\w . 32)\u000A                         (#\\x . 33)\u000A                         (#\\y . 34)\u000A                         (#\\z . 35)\u000A                         (#\\A . 10)\u000A                         (#\\B . 11)\u000A                         (#\\C . 12)\u000A                         (#\\D . 13)\u000A                         (#\\E . 14)\u000A                         (#\\F . 15)\u000A                         (#\\G . 16)\u000A                         (#\\H . 17)\u000A                         (#\\I . 18)\u000A                         (#\\J . 19)\u000A                         (#\\K . 20)\u000A                         (#\\L . 21)\u000A                         (#\\M . 22)\u000A                         (#\\N . 23)\u000A                         (#\\O . 24)\u000A                         (#\\P . 25)\u000A                         (#\\Q . 26)\u000A                         (#\\R . 27)\u000A                         (#\\S . 28)\u000A                         (#\\T . 29)\u000A                         (#\\U . 30)\u000A                         (#\\V . 31)\u000A                         (#\\W . 32)\u000A                         (#\\X . 33)\u000A                         (#\\Y . 34)\u000A                         (#\\Z . 35)))))\u000A            (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> l (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l) radix)) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))\u000A                  (t nil))))</code>"],

 "COMMON-LISP____COMPLEX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x 3) ; x + 3i, where i is the principal square root of -1\u000A(<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x y) ; x + yi\u000A(<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x 0) ; same as x, for rational numbers x</code> \u000A \u000A <p>The function <v>complex</v> takes two rational number arguments and returns \u000A an ACL2 number.  This number will be of type <v>(<see topic=\"COMMON-LISP____COMPLEX\">complex</see> rational)</v> [as \u000A defined in the Common Lisp language], except that if the second argument is \u000A zero, then <v>complex</v> returns its first argument.  The function <tt><see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see></tt> is a recognizer for complex rational numbers, i.e. for ACL2 \u000A numbers that are not rational numbers.</p> \u000A \u000A <p>The reader macro <v>#C</v> (which is the same as <v>#c</v>) provides a \u000A convenient way for typing in complex numbers.  For explicit rational numbers \u000A <v>x</v> and <v>y</v>, <v>#C(x y)</v> is read to the same value as <v>(<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x\u000A y)</v>.</p> \u000A \u000A <p>The functions <tt><see topic=\"COMMON-LISP____REALPART\">realpart</see></tt> and <tt><see topic=\"COMMON-LISP____IMAGPART\">imagpart</see></tt> return the real and \u000A imaginary parts (respectively) of a complex (possibly rational) number.  So \u000A for example, <v>(<see topic=\"COMMON-LISP____REALPART\">realpart</see> #C(3 4)) = 3</v>, <v>(<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> #C(3 4)) = 4</v>, \u000A <v>(<see topic=\"COMMON-LISP____REALPART\">realpart</see> 3/4) = 3/4</v>, and <v>(<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> 3/4) = 0</v>.</p> \u000A \u000A <p>The following built-in axiom may be useful for reasoning about complex \u000A numbers.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> complex-definition\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A                (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x y)\u000A                  (<see topic=\"COMMON-LISP_____B2\">+</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> #c(0 1) y))))\u000A  :rule-classes nil)</code> \u000A \u000A <p>A completion axiom that shows what <v>complex</v> returns on arguments \u000A violating its <see topic=\"ACL2____GUARD\">guard</see> (which says that both arguments are rational \u000A numbers) is the following, named <v>completion-of-complex</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____COMPLEX\">complex</see> x y)\u000A       (<see topic=\"COMMON-LISP____COMPLEX\">complex</see> (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) x 0)\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> y) y 0)))</code>"],

 "ACL2____WITH-OUTPUT-LOCK":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This documentation topic relates to an experimental extension of \u000A ACL2, ACL2(p), created initially by David L. Rager.  See <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see> for how to build an executable image that supports parallel \u000A execution.  Also see community books directory <v>books/parallel/</v> for \u000A examples.</p> \u000A \u000A <p>One may wish to perform output while executing code in parallel.  If \u000A threads are allowed to print concurrently, the output will be interleaved and \u000A often unreadable.  To avoid this, the user can surround forms that perform \u000A output with the <v>with-output-lock</v> macro.</p> \u000A \u000A <p>Take the following definition of <v>pfib</v> as an example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0))\u000A              :exec (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0))\u000A         0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PLET\">plet</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> t))\u000A                 ((a (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Computing pfib ~x0~%&quot; (<see topic=\"COMMON-LISP____-\">-</see> x 1))\u000A                             (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))))\u000A                  (b (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____CW\">cw</see> &quot;Computing pfib ~x0~%&quot; (<see topic=\"COMMON-LISP____-\">-</see> x 2))\u000A                             (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))\u000A                 (<see topic=\"COMMON-LISP_____B2\">+</see> a b)))))</code> \u000A \u000A <p>With <see topic=\"ACL2____PARALLEL-EXECUTION\">parallel-execution</see> enabled, a call of <v>(pfib 5)</v>results in \u000A non-deterministically interleaved output, for example as follows.</p> \u000A \u000A <code>ACL2 !&gt;(pfib 5)\u000ACComputing pfib 4\u000Aomputing pfib 3\u000AComCpuotmipnugt ipnfgi bp fib3\u000A2\u000AComputing pCfiobm put2i\u000Ang pfib 1\u000AComputing pfib Co1mp\u000Auting pfib 0\u000ACCoommppuuttiinngg  ppffiibb  12\u000A\u000AComCpuotmipnugt ipnfgi bp fib1\u000A0\u000ACoCmopmuptuitnign gp fpifbi b 1\u000A0\u000A5\u000AACL2 !&gt;</code> \u000A \u000A <p>If the user instead surrounds the calls to <tt><see topic=\"ACL2____CW\">cw</see></tt> with the macro \u000A <v>with-output-lock</v>, as in the following session, the output will no longer \u000A be interleaved.</p> \u000A \u000A <code>ACL2 !&gt;\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pfib (x)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____NATP\">natp</see> x)))\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0))\u000A              :exec (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0))\u000A         0)\u000A        ((<see topic=\"COMMON-LISP_____D3\">=</see> x 1) 1)\u000A        (t (<see topic=\"ACL2____PLET\">plet</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> t))\u000A                 ((a (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____WITH-OUTPUT-LOCK\">with-output-lock</see>\u000A                              (<see topic=\"ACL2____CW\">cw</see> &quot;Computing pfib ~x0~%&quot; (<see topic=\"COMMON-LISP____-\">-</see> x 1)))\u000A                             (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 1))))\u000A                  (b (<see topic=\"ACL2____PROG2_42\">prog2$</see> (<see topic=\"ACL2____WITH-OUTPUT-LOCK\">with-output-lock</see>\u000A                              (<see topic=\"ACL2____CW\">cw</see> &quot;Computing pfib ~x0~%&quot; (<see topic=\"COMMON-LISP____-\">-</see> x 2)))\u000A                             (pfib (<see topic=\"COMMON-LISP____-\">-</see> x 2)))))\u000A                 (<see topic=\"COMMON-LISP_____B2\">+</see> a b)))))\u000A\u000A&lt;snip&gt;\u000A\u000AACL2 !&gt;(pfib 5)\u000AComputing pfib 4\u000AComputing pfib 3\u000AComputing pfib 3\u000AComputing pfib 2\u000AComputing pfib 2\u000AComputing pfib 1\u000AComputing pfib 2\u000AComputing pfib 1\u000AComputing pfib 1\u000AComputing pfib 0\u000AComputing pfib 1\u000AComputing pfib 0\u000AComputing pfib 1\u000AComputing pfib 0\u000A5\u000AACL2 !&gt;</code>"],

 "ACL2____TUTORIAL4-DEFUN-SK-EXAMPLE":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>This example illustrates the use of <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> and <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see topic=\"ACL2____EVENTS\">events</see> to reason about quantifiers.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see>.  For \u000A a more through, systematic beginner's introduction to quantification in ACL2, \u000A see <see topic=\"ACL2____QUANTIFIER-TUTORIAL\">quantifier-tutorial</see>.</p> \u000A \u000A <p>Many users prefer to avoid the use of quantifiers, since ACL2 provides only \u000A very limited support for reasoning about quantifiers.</p> \u000A \u000A <p>Here is a list of <see topic=\"ACL2____EVENTS\">events</see> that proves that if there are arbitrarily \u000A large numbers satisfying the disjunction <v>(<see topic=\"COMMON-LISP____OR\">OR</see> P R)</v>, then either there are \u000A arbitrarily large numbers satisfying <v>P</v> or there are arbitrarily large \u000A numbers satisfying <v>R</v>.</p> \u000A \u000A <code>; Introduce undefined predicates p and r.\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> r (x) t)\u000A\u000A; Define the notion that something bigger than x satisfies p.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-bigger-p (x)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> y (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) (p y))))\u000A\u000A; Define the notion that something bigger than x satisfies r.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-bigger-r (x)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> y (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) (r y))))\u000A\u000A; Define the notion that arbitrarily large x satisfy p.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> arb-lg-p ()\u000A  (<see topic=\"ACL2____FORALL\">forall</see> x (some-bigger-p x)))\u000A\u000A; Define the notion that arbitrarily large x satisfy r.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> arb-lg-r ()\u000A  (<see topic=\"ACL2____FORALL\">forall</see> x (some-bigger-r x)))\u000A\u000A; Define the notion that something bigger than x satisfies p or r.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> some-bigger-p-or-r (x)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> y (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) (<see topic=\"COMMON-LISP____OR\">or</see> (p y) (r y)))))\u000A\u000A; Define the notion that arbitrarily large x satisfy p or r.\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> arb-lg-p-or-r ()\u000A  (<see topic=\"ACL2____FORALL\">forall</see> x (some-bigger-p-or-r x)))\u000A\u000A; Prove the theorem promised above.  Notice that the functions open\u000A; automatically, but that we have to provide help for some rewrite\u000A; rules because they have free variables in the hypotheses.  The\u000A; ``witness functions'' mentioned below were introduced by DEFUN-SK.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see>\u000A (<see topic=\"ACL2____IMPLIES\">implies</see> (arb-lg-p-or-r)\u000A          (<see topic=\"COMMON-LISP____OR\">or</see> (arb-lg-p)\u000A              (arb-lg-r)))\u000A :hints ((&quot;Goal&quot;\u000A          :use\u000A          ((:instance some-bigger-p-suff\u000A                      (x (arb-lg-p-witness))\u000A                      (y (some-bigger-p-or-r-witness\u000A                          (<see topic=\"COMMON-LISP____MAX\">max</see> (arb-lg-p-witness)\u000A                               (arb-lg-r-witness)))))\u000A           (:instance some-bigger-r-suff\u000A                      (x (arb-lg-r-witness))\u000A                      (y (some-bigger-p-or-r-witness\u000A                          (<see topic=\"COMMON-LISP____MAX\">max</see> (arb-lg-p-witness)\u000A                               (arb-lg-r-witness)))))\u000A           (:instance arb-lg-p-or-r-necc\u000A                      (x (<see topic=\"COMMON-LISP____MAX\">max</see> (arb-lg-p-witness)\u000A                              (arb-lg-r-witness))))))))\u000A\u000A; And finally, here's a cute little example.  We have already\u000A; defined above the notion (some-bigger-p x), which says that\u000A; something bigger than x satisfies p.  Let us introduce a notion\u000A; that asserts that there exists both y and z bigger than x which\u000A; satisfy p.  On first glance this new notion may appear to be\u000A; stronger than the old one, but careful inspection shows that y and\u000A; z do not have to be distinct.  In fact ACL2 realizes this, and\u000A; proves the theorem below automatically.\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> two-bigger-p (x)\u000A  (<see topic=\"ACL2____EXISTS\">exists</see> (y z) (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) (p y) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x z) (p z))))\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (some-bigger-p x) (two-bigger-p x)))\u000A\u000A; A technical point:  ACL2 fails to prove the theorem above\u000A; automatically if we take its contrapositive, unless we disable\u000A; two-bigger-p as shown below.  That is because ACL2 needs to expand\u000A; some-bigger-p before applying the rewrite rule introduced for\u000A; two-bigger-p, which contains free variables.  The moral of the\u000A; story is:  Don't expect too much automatic support from ACL2 for\u000A; reasoning about quantified notions.\u000A\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____NOT\">not</see> (two-bigger-p x)) (<see topic=\"COMMON-LISP____NOT\">not</see> (some-bigger-p x)))\u000A     :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> two-bigger-p))))</code>"],

 "ACL2____SET-TRACE-EVISC-TUPLE":[["Trace"],"ACL2 Sources","ACL2","<p>A trace evisc-tuple, which is set by this utility, provides a means \u000A to restrict printing during tracing.  See <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> for an \u000A introduction to evisc-tuples; also see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> and see <see topic=\"ACL2____SET-IPRINT\">set-iprint</see>.</p> \u000A \u000A <p>By default the ACL2 <see topic=\"COMMON-LISP____TRACE\">trace</see> mechanism, <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>, automatically \u000A deals with stobjs, the logical world, and certain other large structures.  See \u000A <see topic=\"ACL2____TRACE_42\">trace$</see>, in particular the documentation of <v>trace$</v> option \u000A <v>:hide</v>.  However, even with that default behavior you may want to restrict \u000A what is printed according to the print-level and print-length of an \u000A evisc-tuple; see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>.</p> \u000A \u000A <code>Examples: \u000A \u000A; Set trace evisc tuple to a standard value, using current Lisp *print-level* \u000A; and *print-length* variables: \u000A(set-trace-evisc-tuple t state) \u000A \u000A; Set trace evisc tuple back to its default: \u000A(set-trace-evisc-tuple nil state) \u000A \u000A; Set trace evisc tuple to restrict print-level to 3 and print-length to 4, \u000A; while hiding the logical world and suitably printing stobjs even if trace$ \u000A; option ``:hide nil'' is used.  (Note: calling <v>trace-evisceration-alist</v> \u000A; directly requires removing this function as `untouchable', which requires a \u000A; trust tag; see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>.) \u000A(set-trace-evisc-tuple \u000A (evisc-tuple 3 4 (trace-evisceration-alist state) nil) \u000A state) \u000A \u000AGeneral Forms: \u000A \u000A(set-trace-evisc-tuple nil state) ; trace evisc-tuple set to standard value \u000A(set-trace-evisc-tuple   t state) ; trace evisc-tuple set to hide the logical \u000A                                  ;   world and deal with stobjs even when \u000A                                  ;   trace$ option ``:hide nil'' is supplied \u000A(set-trace-evisc-tuple evisc-tuple state) \u000A                                  ; tracing set to use indicated evisc-tuple \u000A</code> \u000A \u000A <p>See <see topic=\"ACL2____TRACE_42\">trace$</see> for a discussion of ACL2 tracing.  The <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see> \u000A used by that trace utility is the one last installed by \u000A <v>set-trace-evisc-tuple</v> (or by <tt><see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see></tt> for the \u000A trace-evisc-tuple) &mdash; initially to the default of <v>nil</v> &mdash; unless \u000A overriden by trace option <v>:evisc-tuple</v>.</p> \u000A \u000A <p><b>Remark</b>.  If you use value <v>t</v>, then ACL2 will ensure that the \u000A logical world and stobjs are kept up-to-date in the trace evisc-tuple.</p>"],

 "ACL2____SET-PRINT-BASE":[["Io"],"ACL2 Sources","ACL2","<p>Also see <tt><see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see></tt> for a nice combined use of \u000A  <v>set-print-base</v> and <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt>.</p> \u000A \u000A <p>By default, integers and ratios are printed in base 10.  ACL2 also supports \u000A printing in radix 2, 8, or 16 by calling set-print-base with the desired \u000A radix (base).</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 10 state) ; Default printing\u000A(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16 state) ; Print integers and ratios in hex</code> \u000A \u000A <p>Here is a sample log.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25 25/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16 state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(19 19/3)\u000AACL2 !&gt;</code> \u000A \u000A <p>See <see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> for how to print the radix, for example, \u000A printing the decimal number 25 in print-base 16 as ``<v>#x25</v>'' rather than \u000A ``<v>25</v>''.  Also see <see topic=\"ACL2____PRINT-CONTROL\">print-control</see> for other user-settable print \u000A controls.</p> \u000A \u000A <p>Note: ACL2 <see topic=\"ACL2____EVENTS\">events</see> and some other top-level commands (for example, \u000A <tt><see topic=\"ACL2____THM\">thm</see></tt>, <tt><see topic=\"ACL2____VERIFY\">verify</see></tt>, and history commands such as <v>:</v><v>pe</v> and \u000A <v>:</v><v>pbt</v>) set the print base to 10 during their evaluation.  So <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt> has no effect while these forms are being processed.</p>"],

 "ACL2____REDEF_B2":[["Ld"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000AACL2 !&gt;:redef+\u000AACL2 p!&gt;</code> \u000A \u000A <p>This <see topic=\"ACL2____COMMAND\">command</see> is intended only for system hackers, not typical users. \u000A It sets <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> to <v>'(:warn! . :overwrite)</v>, sets \u000A the default <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see> to <v>:</v><tt><see topic=\"ACL2____PROGRAM\">program</see></tt>, and invokes <tt><see topic=\"ACL2____SET-STATE-OK\">set-state-ok</see></tt> with value <v>t</v>.  It also introduces <v>(<see topic=\"ACL2____DEFTTAG\">defttag</see> :redef+)</v>, \u000A so that redefinition of system functions will be permitted; see <see topic=\"ACL2____DEFTTAG\">defttag</see>.  Finally, it removes as untouchable (see <see topic=\"ACL2____PUSH-UNTOUCHABLE\">push-untouchable</see>) \u000A all variables and functions.</p> \u000A \u000A <p>WARNING: If the form <v>(<see topic=\"ACL2____REDEF_B2\">redef+</see>)</v> is used in a book, then including the \u000A book can leave you in a state in which dangerous actions are allowed, \u000A specifically: redefinition, and access to functions and variables normally \u000A prohibited because they are untouchable.  To avoid this problem, insert the \u000A form <v>(</v><tt><see topic=\"ACL2____REDEF-\">redef-</see></tt><v>)</v> into your book after <v>(<see topic=\"ACL2____REDEF_B2\">redef+</see>)</v>.</p> \u000A \u000A <p>To see the code for <v>redef+</v>, evaluate <v>:trans1 (<see topic=\"ACL2____REDEF_B2\">redef+</see>)</v>.  This \u000A <see topic=\"ACL2____COMMAND\">command</see> is intended for those who are modifying ACL2 source code \u000A definitions.  Thus, note that even system functions can be redefined with a \u000A mere warning.  Be careful!</p>"],

 "ACL2____REDEFINED-NAMES":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Example and General Forms:\u000A(<see topic=\"ACL2____REDEFINED-NAMES\">redefined-names</see> state)</code> \u000A \u000A <p>This function collects names that have been redefined in the current ACL2 \u000A <see topic=\"ACL2____STATE\">state</see>.  <v>:</v><tt><see topic=\"ACL2____PROGRAM\">Program</see></tt> mode functions that were reclassified to \u000A <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> functions are not collected, since such reclassification \u000A cannot imperil soundness because it is allowed only when the new and old \u000A definitions are identical.</p> \u000A \u000A <p>Thus, if <v>(<see topic=\"ACL2____REDEFINED-NAMES\">redefined-names</see> state)</v> returns <v>nil</v> then no unsafe \u000A definitions have been made, regardless of <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt>.  See \u000A <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>.</p> \u000A \u000A <p>WARNING: This function does not report names that are not explicitly \u000A redefined in the current logical <see topic=\"ACL2____WORLD\">world</see>.  Consider for example:</p> \u000A \u000A <code>(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A ()\u000A (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () t)\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo () nil))\u000A (<see topic=\"ACL2____DEFTHM\">defthm</see> foo-prop\u000A   (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (foo) nil)\u000A   :rule-classes nil))</code> \u000A \u000A <p>If you attempt to call <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> in the resulting world, ACL2 \u000A will appropriately refuse to do the certification:</p> \u000A \u000A <code>ACL2 Error in (<see topic=\"ACL2____CERTIFY-BOOK\">CERTIFY-BOOK</see> &quot;foo&quot; ...):  At least one command in the\u000Acurrent ACL2 world was executed while the value of state global variable\u000A'LD-REDEFINITION-ACTION was not nil:\u000A\u000A  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FOO NIL T)\u000A\u000ACertification is therefore not allowed in this world.  You can use\u000A:ubt to undo back through this command; see :DOC ubt.</code> \u000A \u000A <p>However, since the local redefinition of <v>foo</v> is gone in the \u000A certification world, <v>(<see topic=\"ACL2____REDEFINED-NAMES\">redefined-names</see> state)</v> will return <v>nil</v>. \u000A (Technical aside, to be ignored except for those interested in implementation \u000A details: ACL2 inhibits <v>certify-book</v> in this example because the \u000A definition of <v>foo</v> is the value of <v>(global-val 'redef-seen (w\u000A state))</v>.)</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-EQUALS-FOR-EQUALS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Anytime you have an equality hypothesis relating two terms, \u000A  e.g.,</p> \u000A \u000A <code>(equal <i>lhs</i> <i>rhs</i>) \u000A</code> \u000A \u000A <p>it is legal to substitute one for the other anyplace else in the formula. \u000A Doing so does not change the truthvalue of the formula.</p> \u000A \u000A <p>You can use a <i>negated equality</i> this way <i>provided</i> it appears \u000A in the conclusion.  For example, it is ok to transform</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 23)))</code> \u000A \u000A <p>to</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> 23)\u000A         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 23)))</code> \u000A \u000A <p>by substitutions of equals for equals.  That is because, by <see \u000A topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see>, we could rearrange the formulas into their \u000A <i>contrapositive</i> forms:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 23)\u000A         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 23)\u000A         (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> 23)))</code> \u000A \u000A <p>and see the equality as a hypothesis and the substitution of <v>23</v> for \u000A <v>x</v> as sensible.</p> \u000A \u000A <p>Sometimes people speak loosely and say ``substitution of equals for \u000A equals'' when they really mean ``substitutions of equivalents for \u000A equivalents.''  Equality, as tested by <v>EQUAL</v>, is only one example of an \u000A equivalence relation.  The next most common is propositional equivalence, as \u000A tested by <v>IFF</v>.  You can use propositional equivalence hypotheses to \u000A substitute one side for the other <i>provided</i> the target term occurs in a \u000A propositional place, as discussed at the bottom of <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-PROPOSITIONAL-CALCULUS'>propositional \u000A calculus</see>.</p> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the example proof in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____IMMEDIATE-FORCE-MODEP":[["Force"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____DISABLE-IMMEDIATE-FORCE-MODEP\">disable-immediate-force-modep</see> and see <see topic=\"ACL2____ENABLE-IMMEDIATE-FORCE-MODEP\">enable-immediate-force-modep</see>.</p> \u000A \u000A <p>This function symbol is defined simply to provide a <see topic=\"ACL2____RUNE\">rune</see> which can \u000A be <see topic=\"ACL2____ENABLE\">enable</see>d and <see topic=\"ACL2____DISABLE\">disable</see>d.  Enabling</p> \u000A \u000A <code>(:executable-counterpart immediate-force-modep)</code> \u000A \u000A <p>causes ACL2 to attack <see topic=\"ACL2____FORCE\">force</see>d hypotheses immediately instead of \u000A delaying them to the next forcing round.</p> \u000A \u000A <code>Example Hints\u000A:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart immediate-force-modep))\u000A           ; delay forced hyps to forcing round\u000A:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> (:executable-counterpart immediate-force-modep))\u000A           ; split on forced hyps immediately</code> \u000A \u000A <p>See <see topic=\"ACL2____FORCE\">force</see> for background information.  When a <tt><see topic=\"ACL2____FORCE\">force</see></tt>d \u000A hypothesis cannot be established a record is made of that fact and the proof \u000A continues.  When the proof succeeds a ``forcing round'' is undertaken in which \u000A the system attempts to prove each of the <see topic=\"ACL2____FORCE\">force</see>d hypotheses explicitly. \u000A However, if the <see topic=\"ACL2____RUNE\">rune</see> <v>(:executable-counterpart\u000A immediate-force-modep)</v> is <see topic=\"ACL2____ENABLE\">enable</see>d at the time the hypothesis is <see topic=\"ACL2____FORCE\">force</see>d, then ACL2 does not delay the attempt to prove that hypothesis but \u000A undertakes the attempt more or less immediately.</p>"],

 "ACL2____F-PUT-GLOBAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'x (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 10) state)\u000A(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'a (<see topic=\"ACL2____ASET1\">aset1</see> 'ascii-map-array (<see topic=\"ACL2_____04\">@</see> a) 66 'Upper-case-B) state)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> (<see topic=\"COMMON-LISP____QUOTE\">quote</see> symbol) term state)</code> \u000A \u000A <p>where <v>symbol</v> is any symbol (with certain enforced exclusions to avoid \u000A overwriting ACL2 system ``globals'') and <v>term</v> is any ACL2 term that could \u000A be evaluated at the top-level.  <v>F-put-global</v> evaluates the term, stores \u000A the result as the value of the given symbol in the <v>global-table</v> of <tt><see topic=\"ACL2____STATE\">state</see></tt>, and returns the new <v>state</v>.  (Note: the actual implementation of \u000A the storage of this value is much more efficient than this discussion of the \u000A logic might suggest.)</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt> is closely related to <v>f-put-global</v>: \u000A <v>(<see topic=\"ACL2____ASSIGN\">assign</see> var val)</v> macroexpands to <v>(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'var val state)</v>.</p> \u000A \u000A <p>The macros <tt><see topic=\"ACL2_____04\">@</see></tt> and <tt><see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see></tt> give convenient access to the \u000A value of such globals.  The <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> operation has no effect on the \u000A <v>global-table</v> of <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thus, you may use these globals to hang \u000A onto useful data structures even though you may undo back past where you \u000A computed and saved them.</p>"],

 "ACL2____ZIP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ZIP\">Zip</see> i)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____IFIX\">ifix</see> i) 0)</v> and \u000A is the preferred termination test for recursion through the integers.  <v>(<see topic=\"ACL2____ZIP\">Zip</see>\u000A i)</v> returns <v>t</v> if <v>i</v> is <v>0</v> or not an integer; it returns <v>nil</v> \u000A otherwise.  Thus,</p> \u000A \u000A <code>i         (<see topic=\"ACL2____ZIP\">zip</see> i)\u000A3         nil\u000A0         t\u000A-2        nil\u000A5/2       t\u000A#c(1 3)   t\u000A'abc      t</code> \u000A \u000A <p><v>(<see topic=\"ACL2____ZIP\">Zip</see> i)</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring <v>i</v> to be an integer.</p> \u000A \u000A <p>For a discussion of the various idioms for testing against <v>0</v>, see \u000A <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>Zip</v> is typically used as the termination test in recursions through \u000A the integers.  It has the advantage of ``coercing'' its argument to an integer \u000A and hence allows the definition to be admitted without an explicit type check \u000A in the body.  <see topic=\"ACL2____GUARD\">Guard</see> verification allows <v>zip</v> to be compiled as a \u000A direct <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>-comparision with <v>0</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>zip</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> zip (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____D3\">=</see> x 0) t))</code>"],

 "ACL2____SET-IGNORE-OK":[["Declare","Defun"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> t)\u000A(<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> nil)\u000A(<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> :warn)</code> \u000A \u000A <p>The first example above allows unused formals and locals, i.e., variables \u000A that would normally have to be <see topic=\"COMMON-LISP____DECLARE\">declare</see>d <v>ignore</v>d or <v>ignorable</v>. \u000A The second example disallows unused formals and locals; this is the default. \u000A The third example allows them, but prints an appropriate warning.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-IGNORE-OK\">set-ignore-ok</see> flg)</code> \u000A \u000A <p>where <v>flg</v> is either <v>t</v>, <v>nil</v>, or <v>:warn</v>.</p> \u000A \u000A <p>One might find this event useful when one is generating function \u000A definitions by an automated procedure, when that procedure does not take care \u000A to make sure that all formals are actually used in the definitions that it \u000A generates.</p> \u000A \u000A <p>Note: Defun will continue to report irrelevant formals even if \u000A <v>:set-ignore-ok</v> has been set to <v>t</v>, unless you also use <tt><see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see></tt> to instruct it otherwise.</p>"],

 "ACL2____Q":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;:Q</code> \u000A \u000A <p>The keyword command <v>:q</v> typed at the top-level of the ACL2 loop will \u000A terminate the loop and return control to the Common Lisp top-level (or, more \u000A precisely, to whatever program invoked <tt><see topic=\"ACL2____LP\">lp</see></tt>).  To reenter the ACL2 loop, \u000A execute <v>(<see topic=\"ACL2____LP\">acl2::lp</see>)</v> in Common Lisp.  You will be in the same state as you \u000A were when you exited with <v>:q</v>, unless during your stay in Common Lisp you \u000A messed the data structures representating the ACL2 <see topic=\"ACL2____STATE\">state</see> (including \u000A files, property lists, and single-threaded objects).</p> \u000A \u000A <p>Unlike all other keyword commands, typing <v>:q</v> is not equivalent to \u000A invoking the function <v>q</v>.  There is no function <v>q</v>.</p>"],

 "ACL2____GAG-MODE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> for an explanation of gag-mode, \u000A which can take any of the following values:</p> \u000A \u000A <code>(<see topic=\"ACL2____GAG-MODE\">gag-mode</see>) ; generally evaluates to t, nil, or :goals</code>"],

 "ACL2____ADD-TO-SET":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x lst)\u000A(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x lst :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x lst :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x lst :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p>For a symbol <v>x</v> and an object <v>lst</v>, <v>(<see topic=\"ACL2____ADD-TO-SET-EQ\">add-to-set-eq</see> x lst)</v> is \u000A the result of <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>ing <v>x</v> on to the front of <v>lst</v>, unless \u000A <v>x</v> is already a <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> of <v>lst</v>, in which case the result is \u000A <v>lst</v>. The optional keyword, <v>:TEST</v>, has no effect logically, but \u000A provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing <v>x</v> with \u000A successive elements of <v>lst</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>add-to-set</v> depends on the test.  In all \u000A cases, the second argument must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then either the first argument must be suitable for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> \u000A (see <see topic=\"ACL2____EQLABLEP\">eqlablep</see>) or the second argument must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then either the first argument \u000A must be a symbol or the second argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>add-to-set</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____ADD-TO-SET-EQ\">add-to-set-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x\u000A lst :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ADD-TO-SET-EQUAL\">add-to-set-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____ADD-TO-SET\">add-to-set</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>add-to-set-equal</v>.</p>"],

 "ACL2-PC____DV":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(dv 1)    -- assign the new current subterm to be the first argument\u000A             of the existing current subterm\u000A(dv 1 2)  -- assign the new current subterm to be the result of\u000A             first taking the 1st argument of the existing\u000A             current subterm, and then the 2nd argument of that</code> \u000A \u000A <p>For example, if the current subterm is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____B2\">+</see> a b) c),</code> \u000A \u000A <p>then after <v>(dv 1)</v> it is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> a b).</code> \u000A \u000A <p>If after that, then <v>(dv 2)</v> is invoked, the new current subterm will \u000A be</p> \u000A \u000A <code>b.</code> \u000A \u000A <p>Instead of <v>(dv 1)</v> followed by <v>(dv 2)</v>, the same current subterm \u000A could be obtained by instead submitting the single instruction <v>(dv 1\u000A 2)</v>.</p> \u000A \u000A <code>General Form:\u000A(dv &amp;rest naturals-list)</code> \u000A \u000A <p>If <v>naturals-list</v> is a non-empty list <v>(n_1 ... n_k)</v> of natural \u000A numbers, let the new current subterm be the result of selecting the \u000A <v>n_1</v>-st argument of the current subterm, and then the <v>n_2</v>-th subterm \u000A of that, ..., finally the <v>n_k</v>-th subterm.</p> \u000A \u000A <p><b>Remark:</b> <v>(dv n)</v> may be abbreviated by simply <v>n</v>, so we could \u000A have typed <v>1</v> instead of <v>(dv 1)</v> in the first example above.</p> \u000A \u000A <p><b>Remark:</b> See also <v>dive</v>, which is related to the command <v>pp</v>, \u000A in that the diving is done according to raw (translated, internal form) \u000A syntax.  Use the command <v>dv</v> if you want to dive according to the syntax \u000A displayed by the command <v>p</v>.  Thus, the command ``<v>up</v>'' is the inverse \u000A of <v>dive</v>, not of <v>dv</v>.  The following example illustrates this \u000A point.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____VERIFY\">verify</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____A2\">*</see> a b c) x))\u000A-&gt;: p ; print user-level term\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____A2\">*</see> A B C) X)\u000A-&gt;: pp ; print internal-form (translated) term\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> A (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> B C)) X)\u000A-&gt;: exit\u000AExiting....\u000A NIL\u000AACL2 !&gt;(<see topic=\"ACL2____VERIFY\">verify</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP_____A2\">*</see> a b c) x))\u000A-&gt;: p\u000A(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP_____A2\">*</see> A B C) X)\u000A-&gt;: 1 ; same as (dv 1)\u000A-&gt;: p ; print user-level term\u000A(<see topic=\"COMMON-LISP_____A2\">*</see> A B C)\u000A-&gt;: pp ; print internal-form (translated) term\u000A(<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> A (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> B C))\u000A-&gt;: 3 ; dive to third argument of (<see topic=\"COMMON-LISP_____A2\">*</see> A B C)\u000A-&gt;: p\u000AC\u000A-&gt;: up ; go up one level in (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> A (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> B C))\u000A-&gt;: p\u000A(<see topic=\"COMMON-LISP_____A2\">*</see> B C)\u000A-&gt;: pp\u000A(<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> B C)\u000A-&gt;:</code>"],

 "COMMON-LISP____CONCATENATE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string &quot;ab&quot; &quot;cd&quot; &quot;ef&quot;)     ; equals &quot;abcdef&quot;\u000A(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string &quot;ab&quot;)               ; equals &quot;ab&quot;\u000A(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'list '(a b) '(c d) '(e f)) ; equals '(a b c d e f)\u000A(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'list)                      ; equals nil\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> result-type x1 x2 ... xn)</code> \u000A \u000A <p>where <v>n &gt;= 0</v> and either: <v>result-type</v> is <v>'</v><tt><see topic=\"COMMON-LISP____STRING\">string</see></tt> and \u000A each <v>xi</v> is a string; or <v>result-type</v> is <v>'</v><tt><see topic=\"COMMON-LISP____LIST\">list</see></tt> and each \u000A <v>xi</v> is a true list.  <v>Concatenate</v> simply concatenates its arguments to \u000A form the result string or list.  Also see <see topic=\"COMMON-LISP____APPEND\">append</see> and see <see topic=\"ACL2____STRING-APPEND\">string-append</see>.  (The latter immediately generates a call to <v>concatenate</v> \u000A when applied to strings.)</p> \u000A \u000A <p>Note: We do *not* try to comply with the Lisp language's insistence that \u000A <v>concatenate</v> copies its arguments.  Not only are we in an applicative \u000A setting, where this issue shouldn't matter for the logic, but also we do not \u000A actually modify the underlying lisp implementation of <v>concatenate</v>; we \u000A merely provide a definition for it.</p> \u000A \u000A <p><v>Concatenate</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>concatenate</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> concatenate\u000A          (result-type &amp;rest sequences)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> result-type ''string)\u000A                                     (<see topic=\"COMMON-LISP____EQUAL\">equal</see> result-type ''list))))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> result-type ''string)\u000A                 (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> sequences (<see topic=\"COMMON-LISP____CDR\">cdr</see> sequences)\u000A                             (<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> sequences)))\u000A                        (<see topic=\"COMMON-LISP____LIST\">list</see> 'string-append\u000A                              (<see topic=\"COMMON-LISP____CAR\">car</see> sequences)\u000A                              (<see topic=\"COMMON-LISP____CADR\">cadr</see> sequences)))\u000A                       (t (<see topic=\"COMMON-LISP____LIST\">list</see> 'string-append-lst\u000A                                (<see topic=\"COMMON-LISP____CONS\">cons</see> 'list sequences)))))\u000A                ((<see topic=\"COMMON-LISP____ENDP\">endp</see> sequences) nil)\u000A                (t (<see topic=\"COMMON-LISP____CONS\">cons</see> 'append\u000A                         (<see topic=\"COMMON-LISP____APPEND\">append</see> sequences (<see topic=\"COMMON-LISP____LIST\">list</see> nil))))))</code>"],

 "COMMON-LISP____ATOM":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v> is true if and only if <v>x</v> is an atom, i.e., not a \u000A <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair.</p> \u000A \u000A <p><v>Atom</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>, and is a Common Lisp function.  See \u000A any Common Lisp documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>atom</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> atom (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))</code>"],

 "ACL2_____A2TERMINAL-MARKUP-TABLE_A2":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>The value of the ACL2 constant <v>*terminal-markup-table*</v> is an \u000A association list pairing <see topic=\"ACL2____MARKUP\">markup</see> keys with strings, to be used for \u000A printing to the terminal.  See <see topic=\"ACL2____MARKUP\">markup</see> for a description of the ACL2 \u000A <see topic=\"ACL2____MARKUP\">markup</see> language.</p> \u000A \u000A <p>The entries in <v>*terminal-markup-table*</v> are of the form</p> \u000A \u000A <code>(key flag . fmt-string)</code> \u000A \u000A <p>where <v>key</v> is one of the <see topic=\"ACL2____DOC-STRING\">doc-string</see> tilde directives (see <see topic=\"ACL2____MARKUP\">markup</see>), <v>flag</v> is a Boolean as described below, and <v>fmt-string</v> is a \u000A string as expected by the ACL2 printing function <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.  The system \u000A arranges that for any <v>arg</v>, when an expression ~key[arg] is encountered by \u000A the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> printer, <tt><see topic=\"ACL2____FMT\">fmt</see></tt> will print <v>fmt-string</v> in an \u000A association list, binding keys based on <v>arg</v> as follows.</p> \u000A \u000A <code>#\\p --- the `pointer'     ; only used if flag is t\u000A#\\s --- the print name version of the pointer, e.g., |abc| or ABC\u000A#\\c --- the parent file   ; only used if flag is t\u000A#\\t --- the displayed text\u000A#\\T --- uppercased displayed text</code> \u000A \u000A <p>The first three entries are used only when the <v>flag</v> associated with \u000A <v>key</v> is <v>t</v>, indicating that the argument <v>arg</v> of <v>~key</v> is to \u000A be parsed as starting with a symbol; for example, <v>~key[foo bar]</v> will bind \u000A <v>#\\p</v> to the symbol <v>FOO</v>.</p> \u000A \u000A <p>The discussion of the above association list for printing <v>fmt-string</v> \u000A applies when printing <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> to other than the terminal as well. \u000A Such tools exist for Texinfo and for HTML; see files \u000A <v>doc/write-acl2-html.lisp</v> and <v>doc/write-acl2-texinfo.lisp</v> distributed \u000A with ACL2.</p>"],

 "ACL2____WITH-LIVE-STATE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The macro <v>with-live-state</v> is an advanced feature that very few \u000A users will need (basically, only system hackers).  Indeed, it is untouchable; \u000A see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see> for how to enable calling <v>with-live-state</v> \u000A in the ACL2 loop.</p> \u000A \u000A <code>Example Form:\u000A(<see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see> (<see topic=\"ACL2____ASSIGN\">assign</see> y 3))\u000A\u000AGeneral form:\u000A(<see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see> form)</code> \u000A \u000A <p>where form is an arbitrary form with a free reference to the variable \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>Logically, <v>(<see topic=\"ACL2____WITH-LIVE-STATE\">with-live-state</see> FORM)</v> macroexpands to <v>FORM</v>.  However, \u000A in raw Lisp it expands to:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____STATE\">state</see> *the-live-state*))\u000A  FORM)</code> \u000A \u000A <p>If a form that mentions the variable <tt><see topic=\"ACL2____STATE\">state</see></tt> might be executed in raw \u000A Lisp &mdash; that is, either outside the ACL2 loop or in raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) &mdash; then the surrounding the form with <v>with-live-state</v> \u000A as shown above can avoid potential warnings or (much less likely) errors. \u000A Note however that if <v>state</v> is lexically bound to a state other than the \u000A usual ``live'' state, surprising behavior may occur when evaluating a call of \u000A <v>with-live-state</v> in raw Lisp or raw mode (either directly by evaluation or \u000A at compile time), because <v>with-live-state</v> will override that lexical \u000A binding of <tt><see topic=\"ACL2____STATE\">state</see></tt> by a lexical binding of <v>state</v> to the usual \u000A ``live'' state.</p>"],

 "ACL2____REWRITE-STACK-LIMIT":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>ACL2 users can create rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> (see \u000A <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) that have the potential of causing an infinite loop in \u000A the ACL2 rewriter.  This could lead to Lisp stack overflows and even \u000A segmentation faults.  For this reason, the depth of certain calls of functions \u000A in the ACL2 rewriter is limited by default using the value of the form \u000A <v>(<see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see> (w state))</v>.  To see the limit in action, execute the \u000A following forms.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc-1\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)))\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> app-assoc-2\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y z)\u000A         (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)))\u000A(<see topic=\"ACL2____THM\">thm</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> a b c) xxx)\u000A     ; try without these hints to see a slightly different error message\u000A     :hints ((&quot;Goal&quot; :do-not '(preprocess))))</code> \u000A \u000A <p>The ensuing error message shows a stack of one greater than the value of \u000A <v>(<see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see> (w state))</v>, which by default is the value of the \u000A constant <v>*default-rewrite-stack-limit*</v>.  The error message also explains \u000A how to use <v>:</v><tt><see topic=\"ACL2____BRR\">brr</see></tt><v> t</v> and <v>(</v><tt><see topic=\"ACL2____CW-GSTACK\">cw-gstack</see></tt><v>)</v> to find \u000A looping rewrite rules.</p> \u000A \u000A <p>This limit can be changed; see <see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see>.</p> \u000A \u000A <p>For a related limit, see <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p>"],

 "ACL2____PLET":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____PLET\">plet</see> ((a (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 1)))\u000A       (b (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> a b))\u000A\u000A(<see topic=\"ACL2____PLET\">plet</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 35)))\u000A      ((a (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 1)))\u000A       (b (fibonacci (<see topic=\"COMMON-LISP____-\">-</see> x 2))))\u000A      (<see topic=\"COMMON-LISP_____B2\">+</see> a b))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____PLET\">plet</see> (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> expr)) ; optional granularity declaration\u000A      ((var1 val1)\u000A       ...\u000A       (varN valN))\u000A      (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ... (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ; optional declarations\u000A      body)</code> \u000A \u000A <p>The syntax of <v>plet</v> is identical to the syntax of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>, except \u000A that <v>plet</v> permits an optional granularity declaration in the first \u000A argument position; see <see topic=\"ACL2____GRANULARITY\">granularity</see>.  In the logic a call of <v>plet</v> \u000A macroexpands to the corresponding call of <tt><see topic=\"COMMON-LISP____LET\">let</see></tt>, where the granularity \u000A declaration (if any) is dropped.</p> \u000A \u000A <p><v>Plet</v> cause the evaluation of each <v>vali</v> above to be done in \u000A parallel before processing the body.  If the above <tt><see topic=\"ACL2____GRANULARITY\">granularity</see></tt> \u000A declaration is present, then its expression (<v>expr</v> above) is first \u000A evaluated, and if the result is <v>nil</v>, then such parallelism is avoided. \u000A Even if parallelism is not thus avoided, parallelism may be limited by \u000A available resources.</p> \u000A \u000A <p>See <see topic=\"ACL2____PARALLELISM-AT-THE-TOP-LEVEL\">parallelism-at-the-top-level</see> for restrictions on evaluating \u000A parallelism primitives from within the ACL2 top-level loop.</p>"],

 "ACL2____NATP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The natural numbers is the set of all non-negative integers, \u000A <v>{0,1,2,3,...}</v>.  <v>Natp</v> returns <v>t</v> if and only its argument is a \u000A natural number, and <v>nil</v> otherwise.</p> \u000A \u000A <p>The community book <see topic=\"ACL2____ARITHMETIC_F2NATP-POSP\">arithmetic/natp-posp</see> has some lightweight rules \u000A for reasoning about <v>posp</v> and <v>natp</v>, and is included in the <see topic=\"ACL2____ARITHMETIC-1\">arithmetic-1</see> library.  For a somewhat heavier and more comprehensive \u000A alternative, you may wish to instead see the <see topic=\"ACL2____ARITH-EQUIVS\">arith-equivs</see> book.</p> \u000A \u000A <p><b>Function: </b><srclink>natp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> natp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 x)))</code>"],

 "ACL2____ILLEGAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ILLEGAL\">Illegal</see> ctx str alist)</v> causes evaluation to halt with a short \u000A message using the ``context'' <v>ctx</v>.  An error message is first printed \u000A using the string <v>str</v> and alist <v>alist</v> that are of the same kind as \u000A expected by <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.  See <see topic=\"ACL2____FMT\">fmt</see>, and see <see topic=\"ACL2____PROG2_42\">prog2$</see> for an example \u000A of how to use a related function, <tt><see topic=\"ACL2____HARD-ERROR\">hard-error</see></tt> \u000A (see <see topic=\"ACL2____HARD-ERROR\">hard-error</see>).  Also see <see topic=\"ACL2____ER\">er</see> for a macro that provides a \u000A unified way of signaling errors.</p> \u000A \u000A <p>The difference between <v>illegal</v> and <tt><see topic=\"ACL2____HARD-ERROR\">hard-error</see></tt> is that the \u000A former has a guard of <v>nil</v> while the latter has a <tt><see topic=\"ACL2____GUARD\">guard</see></tt> of <v>t</v>. \u000A Thus, you may want to use <v>illegal</v> rather than <v>hard-error</v> when you \u000A intend to do <tt><see topic=\"ACL2____GUARD\">guard</see></tt> verification at some point, and you expect the \u000A guard to guarantee that the <v>illegal</v> call is never executed.  See <see topic=\"ACL2____PROG2_42\">prog2$</see> for an example.</p> \u000A \u000A <p><b>Function: </b><srclink>illegal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> illegal (ctx str alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____HARD-ERROR\">hard-error</see> ctx str alist)))\u000A       (<see topic=\"ACL2____HARD-ERROR\">hard-error</see> ctx str alist))</code>"],

 "ACL2____Free_02Variables_02in_02Top-Level_02Input":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>ACL2 !&gt;<b>(equal (app (app a b) c)</b> \u000A              <b>(app a (app b c))))</b> \u000A \u000AACL2 Error in TOP-LEVEL:  Global variables, such as C, B, and A, are \u000Anot allowed. See :DOC ASSIGN and :DOC @. \u000A \u000A</code> \u000A \u000A <p>ACL2 does not allow ``global variables'' in top-level input.  There is no \u000A ``top-level binding environment'' to give meaning to these variables.</p> \u000A \u000A <p>Thus, expressions involving no variables can generally be evaluated,</p> \u000A \u000A <code>ACL2 !&gt;<b>(equal (app (app '(1 2) '(3 4)) '(5 6))</b> \u000A              <b>(app '(1 2) (app '(3 4) '(5 6))))</b> \u000A(1 2 3 4 5 6) \u000A</code> \u000A \u000A <p>but expressions containing variables cannot.</p> \u000A \u000A <p>There is an exception to this rule.  References to ``single-threaded \u000A objects'' may appear in top-level forms.  See <see topic=\"ACL2____STOBJ\">stobj</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>.  A \u000A single-threaded object is an ACL2 object, usually containing many fields, \u000A whose use is syntactically restricted so that it may be given as input only to \u000A certain functions and must be returned as output by certain functions.  These \u000A restrictions allow single- threaded objects to be efficiently manipulated. \u000A For example, only a single copy of the object actually exists, even though \u000A from a logical perspective one might expect the object to be ``copied on \u000A write.''</p> \u000A \u000A <p>The most commonly used single-threaded object in ACL2 is the ACL2 system \u000A state, whose current value is always held in the variable <tt><see topic=\"ACL2____STATE\">state</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>ACL2 provides a way for you to use <v>state</v> to save values of \u000A computations at the top-level and refer to them later.  See <see topic=\"ACL2____ASSIGN\">assign</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A <see topic=\"ACL2_____04\">@</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p>"],

 "ACL2____Common_02Lisp_02as_02a_02Modeling_02Language":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='common-lisp.gif'></img></p> \u000A \u000A <p>In ACL2 we have adopted Common Lisp as the basis of our modeling language. \u000A If you have already read our brief note on Common Lisp and recall the example \u000A of <v>app</v>, please proceed.  Otherwise click <see topic='ACL2____Common_02Lisp'>here</see> for an exceedingly brief introduction to Common \u000A Lisp and then come <b>back</b> here.</p> \u000A \u000A <p>In Common Lisp it is very easy to write systems of formulas that manipulate \u000A discrete, inductively constructed data objects.  In building a model you might \u000A need to formalize the notion of sequences and define such operations as \u000A concatenation, length, whether one is a permutation of the other, etc.  It is \u000A easy to do this in Common Lisp.  Furthermore, if you have a Common Lisp \u000A ``theory of sequences'' you can <b>run</b> the operations and relations you \u000A define.  That is, you can execute the functions on concrete data to see what \u000A results your formulas produce.</p> \u000A \u000A <p>If you define the function <v>app</v> as shown above and then type</p> \u000A \u000A <code>(app '(A B) '(C D E))</code> \u000A \u000A <p>in any Common Lisp, the answer will be computed and will be <v>(A B C D\u000A E)</v>.</p> \u000A \u000A <p>The <b>executable</b> nature of Common Lisp and thus of ACL2 is very handy \u000A when producing models.</p> \u000A \u000A <p>But executability is not enough for a modeling language because the purpose \u000A of models is to permit analysis.</p> \u000A \u000A <p>Click <see topic='ACL2____Analyzing_02Common_02Lisp_02Models'>here</see> to \u000A continue.</p>"],

 "ACL2____COPYRIGHT":[["About-ACL2"],"ACL2 Sources","ACL2","<p>This topic provides information about copyright, license, \u000A authorship, and sponsorship of the ACL2 system.  For information about \u000A copyright and authorship of <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>, see <see topic=\"ACL2____DOCUMENTATION-COPYRIGHT\">documentation-copyright</see>, which notes that there are many documentation \u000A authors.</p> \u000A \u000A <p>ACL2 Version 6.5 &mdash; A Computational Logic for Applicative \u000A Common Lisp</p> \u000A \u000A <p>Copyright (C) 2014, Regents of the University of Texas</p> \u000A \u000A <p>This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright (C) \u000A 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.</p> \u000A \u000A <p>This program is free software; you can redistribute it and/or modify it \u000A under the terms of the LICENSE file distributed with ACL2.</p> \u000A \u000A <p>This program is distributed in the hope that it will be useful, but WITHOUT \u000A ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS \u000A FOR A PARTICULAR PURPOSE.  See the LICENSE for more details.</p> \u000A \u000A <p>Written by: Matt Kaufmann and J Strother Moore<br/> \u000A email: Kaufmann@cs.utexas.edu and Moore@cs.utexas.edu<br/> \u000A Department of Computer Science<br/> \u000A University of Texas at Austin<br/> \u000A Austin, TX 78712 U.S.A.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____ACKNOWLEDGMENTS\">acknowledgments</see>.</p>"],

 "ACL2____ACL2-SEDAN":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Many successful ACL2 users run in an shell under Emacs; see <see topic=\"ACL2____EMACS\">emacs</see>.  However, those not familiar with Emacs may prefer to start with an \u000A Eclipse-based interface initiallly developed by Peter Dillinger and Pete \u000A Manolios called the <a href='http://acl2s.ccs.neu.edu/acl2s/doc/'>ACL2 \u000A Sedan</a> or ``ACL2s''.</p> \u000A \u000A <p>ACL2 sessions in the ACL2 Sedan can utilize non-standard extensions and \u000A enhancements, especially geared toward new users, termination reasoning, and \u000A attaching rich user interfaces.  These extensions are <a \u000A href='http://acl2s.ccs.neu.edu/acl2s/src/acl2-extensions'>generally \u000A available</a> as certifiable ACL2 books. \u000A (Some code originating from this project has been migrated to the ACL2 \u000A community books, but only after it was quite stable.)  Thanks to Peter \u000A Dillinger, Pete Manolios, Daron Vroon, and Harsh Raju Chamarthi for their work \u000A on the ACL2 Sedan and for making their books available to ACL2 users.</p>"],

 "ACL2____SET-WELL-FOUNDED-RELATION":[["Defun"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> lex2)</code> \u000A \u000A <p>provided <v>lex2</v> has been proved to be a well-founded relation (see <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>).  Note: This is an event!  It does not print the usual \u000A event summary but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so \u000A recorded.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-WELL-FOUNDED-RELATION\">set-well-founded-relation</see> rel)</code> \u000A \u000A <p>where <v>rel</v> has been proved to be a well-founded relation on objects \u000A satisfying some predicate, <v>mp</v>; see <see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see>.  This \u000A macro is equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :well-founded-relation\u000A 'rel)</v>, and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>This event sets the default well-founded relation to be that imposed on \u000A <v>mp</v>-measures by the relation <v>rel</v>.  Subsequently, if a recursively \u000A defined function is submitted to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> with no explicitly given \u000A <v>:</v><tt><see topic=\"ACL2____WELL-FOUNDED-RELATION\">well-founded-relation</see></tt> argument, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> uses the default \u000A relation, <v>rel</v>, and the associated domain predicate <v>mp</v> used in its \u000A well-foundedness theorem.  That is, the termination conditions generated will \u000A require proving that the measure used by the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is an \u000A <v>mp</v>-measure and that in every recursive call the measure of the arguments \u000A decreases according to <v>rel</v>.</p>"],

 "ACL2____RETURN-LAST-TABLE":[["Return-last"],"ACL2 Sources","ACL2","<p>Please first see <see topic=\"ACL2____RETURN-LAST\">return-last</see> for relevant background.</p> \u000A \u000A <p>This <see topic=\"ACL2____TABLE\">table</see> is for advanced users only, and requires an active trust \u000A tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  We recommend that you do not modify this table \u000A directly, but instead use the macro <v>defmacro-last</v>.  Here we augment that \u000A discussion with some highly technical observations that can probably be \u000A ignored if you use <v>defmacro-last</v>.</p> \u000A \u000A <p>This <see topic=\"ACL2____TABLE\">table</see> has a <v>:guard</v> requiring that each key be a symbol \u000A defined in raw Lisp, generally as a macro, and requiring that each \u000A non-<v>nil</v> value be associated either with a symbol that names a macro \u000A defined in ACL2, or else with a list of one element containing such a symbol. \u000A The table can only be modified when there is an active trust tag; see <see topic=\"ACL2____DEFTTAG\">defttag</see>.  If a key is associated with the value <v>nil</v>, then that key is \u000A treated as though it were not in the table.</p> \u000A \u000A <p>Note that keys of this table are not eligible to be bound by <tt><see topic=\"COMMON-LISP____FLET\">flet</see></tt>. \u000A The current value of this table may be obtained by evaluating the form \u000A <v>(table-alist 'return-last-table (w state))</v>.  The built-in constant \u000A <v>*initial-return-last-table*</v> holds the initial value of this table.</p>"],

 "ACL2____NO-DUPLICATESP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x)\u000A(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> lst)</v> is true if and only if no member of <v>lst</v> \u000A occurs twice in <v>lst</v>.  The optional keyword, <v>:TEST</v>, has no effect \u000A logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing \u000A elements of <v>lst</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>no-duplicatesp</v> depends on the test.  In \u000A all cases, the argument must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is \u000A <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then the argument must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the \u000A test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then the argument must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>no-duplicatesp</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____NO-DUPLICATESP-EQ\">no-duplicatesp-eq</see> x lst)</v> is equivalent to \u000A <v>(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x lst :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">no-duplicatesp-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____NO-DUPLICATESP\">no-duplicatesp</see> x lst\u000A :test 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>no-duplicatesp-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>no-duplicatesp-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> no-duplicatesp-equal (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l) t)\u000A             ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)) nil)\u000A             (t (<see topic=\"ACL2____NO-DUPLICATESP-EQUAL\">no-duplicatesp-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))))</code>"],

 "ACL2____MACROS":[["ACL2"],"ACL2 Sources","ACL2",""],

 "ACL2____LD-EVISC-TUPLE":[["Ld"],"ACL2 Sources","ACL2","<p><v>Ld-evisc-tuple</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____LD-EVISC-TUPLE\">ld-evisc-tuple</see> state)</v> and an updater is \u000A <v>(set-ld-evisc-tuple val state)</v>, although the use of <tt><see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see></tt> is preferred for updating.  <v>Ld-evisc-tuple</v> must be \u000A either <v>nil</v>, which is its initial value, or a legal evisc-tuple: see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, reads forms from \u000A <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, evaluates them and prints the result to <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  Note that the ACL2 top-level loop (see <tt><see topic=\"ACL2____LP\">lp</see></tt>) results from \u000A an invocation of <tt><see topic=\"ACL2____LD\">ld</see></tt>.</p> \u000A \u000A <p>However, there are various flags that control <tt><see topic=\"ACL2____LD\">ld</see></tt>'s behavior and \u000A <v>ld-evisc-tuple</v> is one of them.  <tt><see topic=\"ACL2____LD\">Ld</see></tt> may print the forms it is \u000A evaluating and/or the results of evaluation.  Depending on the value of \u000A <v>ld-evisc-tuple</v> <tt><see topic=\"ACL2____LD\">ld</see></tt> may ``eviscerate'' objects before printing \u000A them.  See <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see> for a discussion of evisceration and of how \u000A other evisc-tuples affect the printing of error messages and warnings, as well \u000A as other output not from <v>ld</v>.</p>"],

 "ACL2____INTERSECTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2)\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'eql)   ; same as above (<see topic=\"COMMON-LISP____EQL\">eql</see> as equality test)\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'eq)    ; same, but eq is equality test\u000A(<see topic=\"ACL2____SET-DIFFERENCE_42\">set-difference$</see> l1 l2 :test 'equal) ; same, but equal is equality test</code> \u000A \u000A <p><v>(<see topic=\"ACL2____INTERSECTP\">Intersectp</see> l1 l2)</v> returns <v>t</v> if <v>l1</v> and <v>l2</v> have a <tt><see topic=\"COMMON-LISP____MEMBER\">member</see></tt> in common, else it returns <v>nil</v>.  The optional keyword, \u000A <v>:TEST</v>, has no effect logically, but provides the test (default <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>) used for comparing members of the two lists.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for a call of <v>intersectp</v> depends on the test.  In all \u000A cases, both arguments must satisfy <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.  If the test is <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see></tt>.  If the \u000A test is <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, then one of the arguments must satisfy <tt><see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____EQUALITY-VARIANTS\">equality-variants</see> for a discussion of the relation between \u000A <v>intersectp</v> and its variants:</p> \u000A \u000A <blockquote><p><v>(<see topic=\"ACL2____INTERSECTP-EQ\">intersectp-eq</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____INTERSECTP\">intersectp</see> x\u000A lst :test 'eq)</v>;</p> \u000A \u000A <p><v>(<see topic=\"ACL2____INTERSECTP-EQUAL\">intersectp-equal</see> x lst)</v> is equivalent to <v>(<see topic=\"ACL2____INTERSECTP\">intersectp</see> x lst :test\u000A 'equal)</v>.</p></blockquote> \u000A \u000A <p>In particular, reasoning about any of these primitives reduces to reasoning \u000A about the function <v>intersectp-equal</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>intersectp-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> intersectp-equal (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A             ((<see topic=\"ACL2____MEMBER-EQUAL\">member-equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) y) t)\u000A             (t (<see topic=\"ACL2____INTERSECTP-EQUAL\">intersectp-equal</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))</code>"],

 "ACL2____DEFUN-SK-EXAMPLE":[["Defun-sk"],"ACL2 Sources","ACL2","<p>For a more through, systematic beginner's introduction to \u000A quantification in ACL2, see <see topic=\"ACL2____QUANTIFIER-TUTORIAL\">quantifier-tutorial</see>.</p> \u000A \u000A <p>The following example illustrates how to do proofs about functions defined \u000A with <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>.  The events below can be put into a certifiable book \u000A (see <see topic=\"ACL2____BOOKS\">books</see>).  The example is contrived and rather silly, in that it \u000A shows how to prove that a quantified notion implies itself, where the \u000A antecedent and conclusion are defined with different <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> events. \u000A But it illustrates the formulas that are generated by <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>, and \u000A how to use them.  Thanks to Julien Schmaltz for presenting this example as a \u000A challenge.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A(<see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>\u000A (((p *) =&gt; *)\u000A  ((expr *) =&gt; *))\u000A\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> p (x) x))\u000A (<see topic=\"ACL2____LOCAL\">local</see> (<see topic=\"COMMON-LISP____DEFUN\">defun</see> expr (x) x)))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-expr1 (x)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> (y) (<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (expr y))))\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-expr2 (x)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> (y) (<see topic=\"ACL2____IMPLIES\">implies</see> (p x) (expr y)))))\u000A\u000A; We want to prove the theorem my-theorem below.  What axioms are there that\u000A; can help us?  If you submit the command\u000A\u000A; :pcb! forall-expr1\u000A\u000A; then you will see the following two key events.  (They are completely\u000A; analogous of course for FORALL-EXPR2.)\u000A\u000A;   (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FORALL-EXPR1 (X)\u000A;     (<see topic=\"COMMON-LISP____LET\">LET</see> ((Y (FORALL-EXPR1-WITNESS X)))\u000A;          (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (P X) (EXPR Y))))\u000A;\u000A;   (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> FORALL-EXPR1-NECC\u000A;     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (P X) (EXPR Y)))\u000A;              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (FORALL-EXPR1 X)))\u000A;     :HINTS\u000A;     ((&quot;Goal&quot; :USE FORALL-EXPR1-WITNESS)))\u000A\u000A; We see that the latter has value when FORALL-EXPR1 occurs negated in a\u000A; conclusion, or (therefore) positively in a hypothesis.  A good rule to\u000A; remember is that the former has value in the opposite circumstance: negated\u000A; in a hypothesis or positively in a conclusion.\u000A\u000A; In our theorem, FORALL-EXPR2 occurs positively in the conclusion, so its\u000A; definition should be of use.  We therefore leave its definition enabled,\u000A; and disable the definition of FORALL-EXPR1.\u000A\u000A;   (<see topic=\"ACL2____THM\">thm</see>\u000A;     (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p x) (forall-expr1 x))\u000A;              (forall-expr2 x))\u000A;     :hints ((&quot;Goal&quot; :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> forall-expr1))))\u000A;\u000A;   ; which yields this unproved subgoal:\u000A;\u000A;   (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (P X) (FORALL-EXPR1 X))\u000A;            (EXPR (FORALL-EXPR2-WITNESS X)))\u000A\u000A; Now we can see how to use FORALL-EXPR1-NECC to complete the proof, by\u000A; binding y to (FORALL-EXPR2-WITNESS X).\u000A\u000A; We use defthmd below so that the following doesn't interfere with the\u000A; second proof, in my-theorem-again that follows.\u000A(<see topic=\"ACL2____DEFTHMD\">defthmd</see> my-theorem\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (p x) (forall-expr1 x))\u000A           (forall-expr2 x))\u000A  :hints ((&quot;Goal&quot;\u000A           :use ((:instance forall-expr1-necc\u000A                            (x x)\u000A                            (y (forall-expr2-witness x)))))))\u000A\u000A; The following illustrates a more advanced technique to consider in such\u000A; cases.  If we disable forall-expr1, then we can similarly succeed by having\u000A; FORALL-EXPR1-NECC applied as a :rewrite rule, with an appropriate hint in how\u000A; to instantiate its free variable.  See :doc hints.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> my-theorem-again\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (P x) (forall-expr1 x))\u000A           (forall-expr2 x))\u000A  :hints ((&quot;Goal&quot;\u000A           :in-theory (<see topic=\"ACL2____DISABLE\">disable</see> forall-expr1)\u000A           :restrict ((forall-expr1-necc\u000A                       ((y (forall-expr2-witness x))))))))</code>"],

 "ACL2____BOOLE_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers <v>x</v> and <v>y</v> are viewed in their two's \u000A complement representation, <v>(<see topic=\"ACL2____BOOLE_42\">boole$</see> op x y)</v> returns the result of applying \u000A the bit-wise logical operation specified by <v>op</v>.  The following table is \u000A adapted from documentation for the analogous Common Lisp function \u000A <a href='http://www.lispworks.com/documentation/HyperSpec/Body/f_boole.htm'>boole</a> in the <a href='http://www.lispworks.com/documentation/HyperSpec/'>Common Lisp Hyperspec</a>.  Note that the values of <v>op</v> for \u000A <v>boole$</v> are ACL2 constants, rather than corresponding values of <v>op</v> \u000A for the Common Lisp function <v>boole</v>.</p> \u000A \u000A <code>op               result\u000A-----------      ---------\u000A*boole-1*        x\u000A*boole-2*        y\u000A*boole-andc1*    and complement of x with y\u000A*boole-andc2*    and x with complement of y\u000A*boole-and*      and\u000A*boole-c1*       complement of x\u000A*boole-c2*       complement of y\u000A*boole-clr*      the constant 0 (all zero bits)\u000A*boole-eqv*      equivalence (exclusive nor)\u000A*boole-ior*      inclusive or\u000A*boole-nand*     not-and\u000A*boole-nor*      not-or\u000A*boole-orc1*     or complement of x with y\u000A*boole-orc2*     or x with complement of y\u000A*boole-set*      the constant -1 (all one bits)\u000A*boole-xor*      exclusive or</code> \u000A \u000A <p>The guard of <v>boole$</v> specifies that <v>op</v> is the value of one of the \u000A constants above and that <v>x</v> and <v>y</v> are integers.</p> \u000A \u000A <p>See any Common Lisp documentation for analogous information about Common \u000A Lisp function <v>boole</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>boole$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> boole$ (op i1 i2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (integer 0 15) op)\u000A                (<see topic=\"COMMON-LISP____TYPE\">type</see> integer i1 i2))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-1*) i1)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-2*) i2)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-and*) (<see topic=\"COMMON-LISP____LOGAND\">logand</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-andc1*)\u000A              (<see topic=\"COMMON-LISP____LOGANDC1\">logandc1</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-andc2*)\u000A              (<see topic=\"COMMON-LISP____LOGANDC2\">logandc2</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-c1*) (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i1))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-c2*) (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-clr*) 0)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-eqv*) (<see topic=\"COMMON-LISP____LOGEQV\">logeqv</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-ior*) (<see topic=\"COMMON-LISP____LOGIOR\">logior</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-nand*) (<see topic=\"COMMON-LISP____LOGNAND\">lognand</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-nor*) (<see topic=\"COMMON-LISP____LOGNOR\">lognor</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-orc1*) (<see topic=\"COMMON-LISP____LOGORC1\">logorc1</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-orc2*) (<see topic=\"COMMON-LISP____LOGORC2\">logorc2</see> i1 i2))\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-set*) 1)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> op *boole-xor*) (<see topic=\"COMMON-LISP____LOGXOR\">logxor</see> i1 i2))\u000A             (t 0)))</code>"],

 "COMMON-LISP____FLOOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____FLOOR\">floor</see> 14 3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____FLOOR\">floor</see> -14 3)\u000A-5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____FLOOR\">floor</see> 14 -3)\u000A-5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____FLOOR\">floor</see> -14 -3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____FLOOR\">floor</see> -15 -3)\u000A5</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____FLOOR\">Floor</see> i j)</v> returns the result of taking the quotient of <v>i</v> and \u000A <v>j</v> and returning the greatest integer not exceeding that quotient.  For \u000A example, the quotient of <v>-14</v> by <v>3</v> is <v>-4 2/3</v>, and the largest \u000A integer not exceeding that rational number is <v>-5</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see> i j)</v> requires that <v>i</v> and <v>j</v> are \u000A rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Floor</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  However, note that unlike Common Lisp, the ACL2 \u000A <v>floor</v> function returns only a single value,</p> \u000A \u000A <p><b>Function: </b><srclink>floor</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> floor (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> i)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> j)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> j 0)))))\u000A       (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((<see topic=\"ACL2____Q\">q</see> (<see topic=\"COMMON-LISP_____A2\">*</see> i (<see topic=\"COMMON-LISP_____F2\">/</see> j)))\u000A              (n (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> q))\u000A              (d (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> q)))\u000A             (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____D3\">=</see> d 1) n)\u000A                   ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                    (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> n d))\u000A                   (t (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> (<see topic=\"COMMON-LISP____-\">-</see> n) d))\u000A                         -1)))))</code>"],

 "ACL2____TTREE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>Many low-level ACL2 functions take and return ``tag trees'' or \u000A ``ttrees'' (pronounced ``tee-trees'') which contain various useful bits of \u000A information such as the lemmas used, the linearize assumptions made, etc.</p> \u000A \u000A <p>Abstractly a tag-tree represents a list of sets, each member set having a \u000A name given by one of the ``tags'' (which are symbols) of the ttree.  The \u000A elements of the set named <v>tag</v> are all of the objects tagged <v>tag</v> in \u000A the tree.  You are invited to browse the source code.  Definitions of \u000A primitives are labeled with the comment ``; Note: Tag-tree primitive''.</p> \u000A \u000A <p>The rewriter, for example, takes a term and a ttree (among other things), \u000A and returns a new term, term', and new ttree, ttree'.  Term' is equivalent to \u000A term (under the current assumptions) and the ttree' is an extension of ttree. \u000A If we focus just on the set associated with the tag <v>LEMMA</v> in the ttrees, \u000A then the set for ttree' is the extension of that for ttree obtained by \u000A unioning into it all the <see topic=\"ACL2____RUNE\">rune</see>s used by the rewrite.  The set associated \u000A with <v>LEMMA</v> can be obtained by <v>(tagged-objects 'LEMMA ttree)</v>.</p>"],

 "ACL2____SET-DEFERRED-TTAG-NOTES":[["Defttag"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see> val state)</code> \u000A \u000A <p>where <v>val</v> is <v>t</v> or <v>nil</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFTTAG\">defttag</see> for a discussion of trust tags (ttags).  By default, a \u000A ``<v>TTAG NOTE</v>'' is printed in order to indicate reliance on a ttag.  When \u000A many such notes are printed, it may be desirable to avoid seeing them all. \u000A Upon evaluation of the form</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see> t state)</code> \u000A \u000A <p>ACL2 will enter a deferred mode for the printing of ttag notes.  In this \u000A mode, only the first ttag note is printed for each top-level command, and the \u000A remainder are summarized before the next top-level prompt (if any) is printed, \u000A hence before the next command is evaluated.  That summary is merely a list of \u000A ttags, but the summary explains that the full ttag notes may be printed with \u000A the command <v>(show-ttag-notes)</v>.</p> \u000A \u000A <p>Note that <v>(show-ttag-notes)</v> spares you duplicate ttag notes.  If you \u000A want to see every ttag note as it would normally appear, for maximum security, \u000A do not evaluate the command <v>(<see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see> t state)</v>.  You can \u000A undo the effect of this command, returning to an immediate mode for printing \u000A ttag notes, by evaluating:</p> \u000A \u000A <code>(<see topic=\"ACL2____SET-DEFERRED-TTAG-NOTES\">set-deferred-ttag-notes</see> nil state)</code>"],

 "ACL2____RANDOM_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____RANDOM_42\">random$</see> 10 state) ==&gt; (<see topic=\"ACL2____MV\">mv</see> 7 &lt;state&gt;)</code> \u000A \u000A <p><v>(<see topic=\"ACL2____RANDOM_42\">Random$</see> limit state)</v>, where <v>limit</v> is a positive integer, returns \u000A a random non-negative integer together with a new <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Logically, \u000A it simply returns the first element of a list that is a field of the ACL2 \u000A <tt><see topic=\"ACL2____STATE\">state</see></tt>, called the <v>acl2-oracle</v> (see <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>), \u000A together with the new state resulting from removing that element from that \u000A list.  (Except, if that element is not in range as specified above, then 0 is \u000A returned.)  However, <v>random$</v> actually invokes a Common Lisp function to \u000A choose the integer returned.  Quoting from the Common Lisp HyperSpec(TM), <a \u000A href='http://www.lispworks.com/documentation/HyperSpec/Front'>http://www.lispworks.com/documentation/HyperSpec/Front</a>: \u000A ``An approximately uniform choice distribution is used...  each of the \u000A possible results occurs with (approximate) probability 1/limit.''</p> \u000A \u000A <p>Consider enabling rules <v>natp-random$</v> and <v>random$-linear</v> if you \u000A want to reason about <v>random$</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>random$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> random$ (limit state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (integer 1 *) limit)\u000A                (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state))\u000A       (<see topic=\"ACL2____MV-LET\">mv-let</see> (erp val state)\u000A               (<see topic=\"ACL2____READ-ACL2-ORACLE\">read-acl2-oracle</see> state)\u000A               (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NULL\">null</see> erp)\u000A                               (<see topic=\"ACL2____NATP\">natp</see> val)\u000A                               (<see topic=\"COMMON-LISP_____C3\">&lt;</see> val limit))\u000A                          val)\u000A                         (t 0))\u000A                   state)))</code>"],

 "ACL2____PCB":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pcb :max ; print the most recent command block\u000A:pcb :x   ; print the most recent command block\u000A:pcb fn   ; print the command block that introduced fn\u000A:pcb 5    ; print the fifth command block</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Pcb</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and prints the \u000A <see topic=\"ACL2____COMMAND\">command</see> block of the <see topic=\"ACL2____COMMAND\">command</see> described.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see> for details of <see topic=\"ACL2____COMMAND\">command</see> descriptors.  See <see topic=\"ACL2____PC\">pc</see> \u000A for description of the format in which <see topic=\"ACL2____COMMAND\">command</see>s are displayed.  The \u000A <see topic=\"ACL2____COMMAND\">command</see> block of a <see topic=\"ACL2____COMMAND\">command</see> consists of the <see topic=\"ACL2____COMMAND\">command</see> itself \u000A and all of the <see topic=\"ACL2____EVENTS\">events</see> it created.  If the <see topic=\"ACL2____COMMAND\">command</see> created a \u000A single event and that event is in fact the <see topic=\"ACL2____COMMAND\">command</see> (i.e., if the <see topic=\"ACL2____COMMAND\">command</see> typed was just an event such as a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> or <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> \u000A rather than a macro that expanded to some event forms), then <v>pcb</v> just \u000A prints the <see topic=\"ACL2____COMMAND\">command</see>.  <v>Pcb</v> sketches <see topic=\"ACL2____COMMAND\">command</see> and all of the \u000A <see topic=\"ACL2____EVENTS\">events</see> it created, rather than printing them fully.  If you wish to see \u000A just the <see topic=\"ACL2____COMMAND\">command</see>, in its entirety, use <tt><see topic=\"ACL2____PC\">pc</see></tt>.  If you wish to see \u000A one of the <see topic=\"ACL2____EVENTS\">events</see> within the block, in its entirety, use <tt><see topic=\"ACL2____PE\">pe</see></tt>. \u000A If you wish to see the <see topic=\"ACL2____COMMAND\">command</see> sketched and all of the <see topic=\"ACL2____EVENTS\">events</see> it \u000A created, in their entirety, use <tt><see topic=\"ACL2____PCB_12\">pcb!</see></tt>.</p>"],

 "ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions_02_82cont_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02Admission_02of_02App'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>You can always use the Index <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A icon below to find the documentation of functions.  Try it.  Click on the \u000A Index icon below.  Then use the Find command of your browser to find ``endp'' \u000A in that document and follow the link.  But remember to come back here.</p> \u000A \u000A <p>The ACL2 documentation is also available in Emacs, via the ACL2-Doc \u000A browser (see <see topic=\"ACL2____ACL2-DOC\">ACL2-Doc</see>) <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, \u000A allowing you to explore the hyperlinked documentation in the comfort of a text \u000A editor that can also interact with ACL2.</p> \u000A \u000A <p>In addition, runtime images of ACL2 have the hyperlinked text as a large \u000A ACL2 data structure that can be explored with ACL2's <b>:doc</b> command.  If \u000A you have ACL2 running, try the command <b>:doc endp</b>.</p> \u000A \u000A <p>Another way to find out about ACL2 functions, if you have an ACL2 image \u000A available, is to use the command :<tt><see topic=\"ACL2____ARGS\">args</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A which prints the formals, type, and guard of a function symbol.</p> \u000A \u000A <p>Of course, the ACL2 documentation can also be printed out as a very long \u000A book but we do not recommend that!  See the ACL2 Home Page to download the \u000A Postscript.</p> \u000A \u000A <p>Now let's continue with <v>app</v>.</p> \u000A \u000A <p><see topic='ACL2____The_02Admission_02of_02App'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____FIND-RULES-OF-RUNE":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____FIND-RULES-OF-RUNE\">find-rules-of-rune</see> rune wrld)</code> \u000A \u000A <p>This function finds all the rules in <v>wrld</v> with <v>:</v><tt><see topic=\"ACL2____RUNE\">rune</see></tt> \u000A rune.  It returns a list of rules in their internal form (generally as \u000A described by the corresponding <v>defrec</v>).  Decyphering these rules is \u000A difficult since one cannot always look at a rule object and decide what kind \u000A of record it is without exploiting many system invariants (e.g., that \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> runes only name rewrite-rules).</p> \u000A \u000A <p>At the moment this function returns <v>nil</v> if the rune in question is a \u000A <v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt> rune, because there is no object representing \u000A <v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt> rules.  (<v>:</v><tt><see topic=\"ACL2____REFINEMENT\">refinement</see></tt> rules cause changes \u000A in the <v>'coarsenings</v> properties.)  In addition, if the rune is an \u000A <v>:</v><tt><see topic=\"ACL2____EQUIVALENCE\">equivalence</see></tt> rune, then congruence rules with that rune will be \u000A returned &mdash; because <v>:</v><tt><see topic=\"ACL2____EQUIVALENCE\">equivalence</see></tt> lemmas generate some \u000A congruence rules &mdash; but the fact that a certain function is now known to \u000A be an equivalence relation is not represented by any rule object and so no \u000A such rule is returned.  (The fact that the function is an equivalence relation \u000A is encoded entirely in its presence as a <v>'coarsening</v> of <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.)</p>"],

 "ACL2____FAST-ALIST-FREE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, this function is the identity; we leave it enabled and would \u000A think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>fast-alist-free</v> removes the hash table associated with \u000A this alist, if one exists.  This effectively converts the alist into an \u000A ordinary alist.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____FAST-ALIST-FREE-ON-EXIT\">fast-alist-free-on-exit</see>.</p> \u000A \u000A <p>Because there is no automatic mechanism for freeing the hash tables used in \u000A fast alists, to avoid memory leaks you should manually free any alists that \u000A will no longer be used.  You may find <tt><see topic=\"ACL2____FAST-ALIST-SUMMARY\">fast-alist-summary</see></tt> useful in \u000A tracking down alists that were not properly freed.</p> \u000A \u000A <p>It is safe to call <v>fast-alist-free</v> on any argument, including fast \u000A alists that have already been freed and objects which are not alists at \u000A all.</p> \u000A \u000A <p><b>Function: </b><srclink>fast-alist-free</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fast-alist-free (alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       alist)</code>"],

 "ACL2____DEFINE-PC-MACRO":[["Proof-checker"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFINE-PC-MACRO\">define-pc-macro</see> ib (<see topic=\"COMMON-LISP_____62OPTIONAL\">&amp;optional</see> term)\u000A  (value\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> term\u000A       `(then (<see topic=\"ACL2____INDUCT\">induct</see> ,term) bash)\u000A     `(then induct bash))))</code> \u000A \u000A <p>The example above captures a common paradigm: one attempts to prove the \u000A current goal by inducting and then simplifying the resulting goals.  (see \u000A <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see> for documentation of the command <v>then</v>, \u000A which is itself a pc-macro command, and commands <v>induct</v> and <v>bash</v>.) \u000A Rather than issuing <v>(then induct bash)</v>, or worse yet issuing <v>induct</v> \u000A and then issuing <v>bash</v> for each resulting goals, the above definition of \u000A <v>ib</v> would let you issue <v>ib</v> and get the same effect.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFINE-PC-MACRO\">define-pc-macro</see> cmd args doc-string dcl ... dcl body)</code> \u000A \u000A <p>where <v>cmd</v> is the name of the pc-macro than you want to define, \u000A <v>args</v> is its list of formal parameters.  <v>Args</v> may include lambda-list \u000A keywords <v>&amp;optional</v> and <v>&amp;rest</v>; see <see topic=\"ACL2____MACRO-ARGS\">macro-args</see>, but note that \u000A here, <v>args</v> may not include <v>&amp;key</v> or <v>&amp;whole</v>.</p> \u000A \u000A <p>The value of <v>body</v> should be an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>), of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp xxx state)</v> for some <v>erp</v> and \u000A <v>xxx</v>.  If <v>erp</v> is <v>nil</v>, then <v>xxx</v> is handed off to the \u000A proof-checker's instruction interpreter.  Otherwise, evaluation typically \u000A halts.  We may write more on the full story later if there is interest in \u000A reading it.</p>"],

 "ACL2____COROLLARY":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>This is a low-level system function at the present time.  See <see topic=\"ACL2____PR\">pr</see> and see <see topic=\"ACL2____PR_12\">pr!</see> instead.  Also see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> for the use of \u000A the symbol <v>:corollary</v> in specifying a rule class.</p>"],

 "ACL2____ACL2_02as_02an_02Interactive_02Theorem_02Prover_02_82cont_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____ACL2_02System_02Architecture'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>When ACL2 proves a lemma, it is converted into one or more <b>rules</b> and \u000A stored in a <b>database</b>.  The theorem prover is <b>rule-driven</b>.  By \u000A proving lemmas you can configure ACL2 to behave in certain ways when it is \u000A trying to prove formulas in a certain problem domain.  The expert user can \u000A make ACL2 do amazingly ``smart'' looking things.</p> \u000A \u000A <p>But it would be wrong to think that ACL2 <i>knows</i> the mathematical \u000A content of a formula just because it has proved it.  What ACL2 knows &mdash; \u000A all ACL2 knows &mdash; is what is encoded in its rules.  There are many types \u000A of rules (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>).</p> \u000A \u000A <p>Many formulas can be effectively coded as rules.  But by the same token, it \u000A is possible to encode a formula as a rule that is so ineffective it cannot \u000A even prove itself!</p> \u000A \u000A <p>The way a formula is stored as a rule is entirely up to the user.  That is, \u000A <b>you</b> determine how ACL2 should use each formula that it proves.</p> \u000A \u000A <p>The most common kind of rule is the <b>rewrite rule</b>.  It is so common \u000A that if you don't tell ACL2 how to store a formula, it stores it as a rewrite \u000A rule.</p> \u000A \u000A <p><see topic='ACL2____ACL2_02System_02Architecture'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2-PC____SHOW-REWRITES":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Ashow-rewrites\u000A\u000AGeneral Form:\u000A(show-rewrites &amp;optional rule-id enabled-only-flg)</code> \u000A \u000A <p>This command displays <see topic=\"ACL2____REWRITE\">rewrite</see> rules whose left-hand side matches the \u000A current subterm, and shows how that command can be applied.  For each rule \u000A displayed, hypotheses are shown that would need to be proved after the rule is \u000A applied.  Note that hypotheses are omitted from the display when the system \u000A can trivially verify that they hold; to see all hypotheses for each rule in a \u000A display that is independent of the arguments of the current subterm, use the \u000A <v>pl</v> or <v>pr</v> command.</p> \u000A \u000A <p>Here are details on the arguments and the output.  If <v>rule-id</v> is \u000A supplied and is a name (non-<v>nil</v> symbol) or a <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>, then only the corresponding rewrite \u000A rule(s) will be displayed, while if <v>rule-id</v> is a positive integer <v>n</v>, \u000A then only the <v>n</v>th rule that would be in the list is displayed.  In each \u000A case, the display will point out when a rule is currently disabled (in the \u000A interactive environment), except that if <v>enabled-only-flg</v> is supplied and \u000A not <v>nil</v>, then disabled rules will not be displayed at all.  Finally, \u000A among the free variables of any rule (see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see>), those that \u000A would remain free if the rule were applied will be displayed.  Also see <see topic=\"ACL2____REWRITE\">rewrite</see>.</p>"],

 "ACL2-PC____PUT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(put x 17)\u000A\u000AGeneral Form:\u000A(put var expr)</code> \u000A \u000A <p>Substitute <v>expr</v> for the ``free variable'' <v>var</v>, as explained \u000A below.</p> \u000A \u000A <p>A ``free variable'' is, for our purposes, a variable <v>var</v> such that the \u000A instruction <v>(free var)</v> has been executed earlier in the state-stack. \u000A What <v>(free var)</v> really does is to let <v>var</v> be an abbreviation for the \u000A term <v>(<see topic=\"ACL2____HIDE\">hide</see> var)</v> (see documentation for <v>add-abbreviation</v>).  What \u000A <v>(put var expr)</v> really does is to unwind the state-stack, replacing that \u000A <v>free</v> instruction with the instruction <v>(add-abbreviation var expr)</v>, \u000A so that future references to <v>(? var)</v> become reference to <v>expr</v> rather \u000A than to <v>(<see topic=\"ACL2____HIDE\">hide</see> var)</v>, and then to replay all the other instructions that \u000A were unwound.  Because <v>hide</v> was used, the expectation is that in most \u000A cases, the instructions will replay successfully and <v>put</v> will \u000A ``succeed''.  However, if any replayed instruction ``fails'', then the entire \u000A replay will abort and ``fail'', and the state-stack will revert to its value \u000A before the <v>put</v> instruction was executed.</p> \u000A \u000A <p>If <v>(put var expr)</v> ``succeeds'', then <v>(remove-abbreviation var)</v> \u000A will be executed at the end.</p> \u000A \u000A <p><b>Remark</b>: The <v>restore</v> command will revert the state-stack to its \u000A value present before the <v>put</v> instruction was executed.</p>"],

 "ACL2-PC____CLAIM":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(claim (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y))   -- attempt to prove (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) from the current\u000A                     top-level hypotheses and if successful, then\u000A                     add (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) as a new top-level hypothesis in\u000A                     the current goal\u000A(claim (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y)\u000A       :otf-flg t\u000A       :hints ((&quot;Goal&quot; :induct t)))\u000A                  -- as above, but call the prover using the\u000A                     indicated values for the otf-flg and hints\u000A(claim (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) 0) -- as above, except instead of attempting to\u000A                     prove (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y), create a new subgoal with the\u000A                     same top-level hypotheses as the current goal\u000A                     that has (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) as its conclusion\u000A(claim (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) :hints :none)\u000A                  -- same as immediately above\u000A\u000AGeneral Form:\u000A(claim expr &amp;rest rest-args)</code> \u000A \u000A <p>This command creates a new subgoal with the same top-level hypotheses as \u000A the current goal but with a conclusion of <v>expr</v>.  If <v>rest-args</v> is a \u000A non-empty list headed by a non-keyword, then there will be no proof attempted \u000A for the new subgoal.  With that possible exception, <v>rest-args</v> should \u000A consist of keyword arguments.  The keyword argument <v>:do-not-flatten</v> \u000A controls the ``flattening'' of new hypotheses, just as with the <v>casesplit</v> \u000A command (as described in its documentation).  The remaining <v>rest-args</v> are \u000A used with a call the <v>prove</v> command on the new subgoal, except that if \u000A <v>:hints</v> is a non-<v>nil</v> atom, then the prover is not called &mdash; \u000A rather, this is the same as the situation described above, where \u000A <v>rest-args</v> is a non-empty list headed by a non-keyword.</p> \u000A \u000A <p><b>Remarks:</b> (1) Unlike the <v>casesplit</v> command, the <v>claim</v> \u000A command is completely insensitive to governors. (2) It is allowed to use \u000A abbreviations in the hints.  (3) The keyword :<v>none</v> has the special role \u000A as a value of :<v>hints</v> that is shown clearly in an example above.</p>"],

 "ACL2____SHARP-U-READER":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000A\u000AACL2 !&gt;#ub1000_1000_1000_\u000A2184\u000AACL2 !&gt;#b100010001000\u000A2184\u000AACL2 !&gt;#uo1_1\u000A9\u000AACL2 !&gt;#o11\u000A9\u000AACL2 !&gt;#u34_5\u000A345\u000AACL2 !&gt;#u345\u000A345\u000AACL2 !&gt;345\u000A345\u000AACL2 !&gt;#ux12_a\u000A298\u000AACL2 !&gt;#ux12a\u000A298\u000AACL2 !&gt;#u x12a\u000A298\u000AACL2 !&gt;#x12a\u000A298\u000AACL2 !&gt;#u123_456/7_919\u000A123456/7919\u000AACL2 !&gt;</code> \u000A \u000A <p>The ACL2 reader supports the use of <v>#ub</v>, <v>#uo</v>, and <v>#ux</v> where \u000A one would otherwise write <v>#b</v>, <v>#o</v>, and <v>#x</v>, respectively (for \u000A binary, octal, and hexadecimal numerals), but where underscore characters \u000A (`<v>_</v>') are allowed but ignored.  Also supported is the prefix <v>#u</v> in \u000A front of a an expression that is a decimal numeral except that underscore \u000A characteres are allowed but ignored.</p> \u000A \u000A <p>The precise specification of <v>#u</v> is as follows.  The Lisp reader reads \u000A one expression after the <v>#u</v>.  If the result is a number, then that number \u000A is returned by the reader.  Otherwise the result must be a symbol whose name \u000A begins with one of the characters `<v>B</v>', `<v>O</v>', or `<v>X</v>', or else a \u000A decimal digit (one of the characters `<v>0</v>, <v>1</v>, ..., <v>9</v>').  All \u000A underscores are removed from the name of that symbol to obtain a string and in \u000A the first three cases only, a `<v>#</v>' character is prepended to that string. \u000A The resulting string is then handed to the Lisp reader in order to obtain the \u000A final result, which must be a number or else an error occurs.</p>"],

 "ACL2____SET-PRINT-RADIX":[["Io"],"ACL2 Sources","ACL2","<p>Also see <tt><see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see></tt> for a nice combined use of \u000A  <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt> and <v>set-print-radix</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> for background on how the print base affects the \u000A printing of numbers.  <v>set-print-radix</v> affects whether a radix indicated \u000A when a number is printed.  The radix is not indicated by default, or after \u000A evaluating <v>(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> nil state)</v>.  But if <v>set-print-radix</v> is \u000A called with a first argument that evaluates to a non<v>nil</v> value &mdash; for \u000A example, <v>(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> t state)</v> &mdash; then the radix is shown when \u000A printing.  (This behavior is consistent with the handling of Common Lisp \u000A global <v>*print-radix*</v>.)  The following log illustrates how this works.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25 25/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16 state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(19 19/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> t state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(#x19 #x19/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 10 state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25. #10r25/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> nil state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25 25/3)\u000AACL2 !&gt;</code>"],

 "ACL2____SERIALIZE":[["Io"],"ACL2 Sources","ACL2","<p>This documentation topic relates to an experimental extension of \u000A ACL2 that supports <tt><see topic=\"ACL2____HONS\">hons</see></tt>, memoization, and fast alists.  See <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.  Thanks to Jared Davis for contributing the \u000A ``serialization'' routines for saving ACL2 objects in files for later \u000A loading.</p> \u000A \u000A <p>We implement some routines for writing arbitrary ACL2 objects to files, and \u000A for loading those files later.  We usually call these \".sao\" files, which \u000A stands for (S)erialized (A)CL2 (O)bject.</p> \u000A \u000A <p>Our serialization scheme uses a compact, binary format that preserves \u000A structure sharing in the original object.  We optimize for read \u000A performance.</p>"],

 "ACL2____NTH-ALIASES-TABLE":[["Stobj","Nth","Update-nth"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TABLE\">table</see> nth-aliases-table 'st0 'st)</code> \u000A \u000A <p>This example associates the symbol <v>st0</v> with the symbol <v>st</v>.  As a \u000A result, when the theorem prover prints terms of the form <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> n st0)</v> or \u000A <v>(<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> n val st0)</v>, where <v>st</v> is a <see topic=\"ACL2____STOBJ\">stobj</see> whose <v>n</v>th \u000A accessor function is <v>f-n</v>, then it will print <v>n</v> as <v>*f-n*</v>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____TABLE\">table</see> nth-aliases-table 'alias-name 'name)</code> \u000A \u000A <p>This event causes <v>alias-name</v> to be treated like <v>name</v> for purposes \u000A of the printing of terms that are calls of <v>nth</v> and <v>update-nth</v>. \u000A (Note however that <v>name</v> is not recursively looked up in this table.) \u000A Both must be symbols other than <tt><see topic=\"ACL2____STATE\">state</see></tt>.  See <see topic=\"ACL2____TERM\">term</see>, in particular \u000A the discussion there of untranslated terms.</p> \u000A \u000A <p>For a convenient way to add entries to this <see topic=\"ACL2____TABLE\">table</see>, see <see topic=\"ACL2____ADD-NTH-ALIAS\">add-nth-alias</see>.  To remove entries from the <see topic=\"ACL2____TABLE\">table</see> with ease, see <see topic=\"ACL2____REMOVE-NTH-ALIAS\">remove-nth-alias</see>.</p>"],

 "ACL2____NOTE-2-6":[["Release-notes"],"ACL2 Sources","ACL2","<p>Because of the large number of modifications, we have divided up \u000A the Version 2.6 notes into the following subtopics.</p> \u000A \u000A <blockquote> \u000A \u000A <p>o New functionality (see <see topic=\"ACL2____NOTE-2-6-NEW-FUNCTIONALITY\">note-2-6-new-functionality</see>):<br></br> \u000A \u000A o Changes in proof engine (see <see topic=\"ACL2____NOTE-2-6-PROOFS\">note-2-6-proofs</see>):<br></br> \u000A \u000A o Changes in rules and definitions (see <see topic=\"ACL2____NOTE-2-6-RULES\">note-2-6-rules</see>):<br></br> \u000A \u000A o Guard-related changes (see <see topic=\"ACL2____NOTE-2-6-GUARDS\">note-2-6-guards</see>):<br></br> \u000A \u000A o Proof-checker changes (see <see topic=\"ACL2____NOTE-2-6-PROOF-CHECKER\">note-2-6-proof-checker</see>):<br></br> \u000A \u000A o System-level changes (see <see topic=\"ACL2____NOTE-2-6-SYSTEM\">note-2-6-system</see>):<br></br> \u000A \u000A o Other (minor) changes (see <see topic=\"ACL2____NOTE-2-6-OTHER\">note-2-6-other</see>):<br></br></p> \u000A \u000A </blockquote>"],

 "ACL2____MACRO-COMMAND":[["Proof-checker"],"ACL2 Sources","ACL2","<p>The proof-checker (see <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>) allows the user to \u000A supply interactive commands.  Compound commands, called macro commands, may be \u000A defined; these expand into zero or more other commands.  Some of these are \u000A ``atomic'' macro commands; these are viewed as a single command step when \u000A completed successfully.</p> \u000A \u000A <p>More <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> will be written on the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.  For \u000A now, we simply point out that there are lots of examples of the use of \u000A <v>define-pc-macro</v> and <v>define-pc-atomic-macro</v> in the ACL2 source file \u000A <v>&quot;proof-checker-b.lisp&quot;</v>.  The former is used to create macro commands, \u000A which can be submitted to the interactive loop (see <see topic=\"ACL2____VERIFY\">verify</see>) and will \u000A ``expand'' into zero or more commands.  The latter is similar, except that the \u000A undoing mechanism (see <see topic=\"ACL2-PC____UNDO\">ACL2-pc::undo</see>) understands atomic macro commands \u000A to represent single interactive commands.  Also see <see topic=\"ACL2-PC____COMM\">ACL2-pc::comm</see> and \u000A see <see topic=\"ACL2-PC____COMMANDS\">ACL2-pc::commands</see> for a discussion of the display of interactive \u000A commands.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____TOGGLE-PC-MACRO\">toggle-pc-macro</see> for how to change a macro command to an \u000A atomic macro command, and vice versa.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-EVALUATION":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Any time you are proving a formula and see a subterm in the formula \u000A that contains no variables, you can just evaluate the subterm.</p> \u000A \u000A <p>This is familiar from algebra: It is not uncommon to rearrange a \u000A polynominal to collect all the constants and then add them up:</p> \u000A \u000A <code>(3x + 2 + 7y + 2)\u000A=\u000A(3x + 7y + (2 + 2))\u000A=\u000A(3x + 7y + 4).</code> \u000A \u000A <p>That last step is just evaluation.</p> \u000A \u000A <p>It happens often in ACL2 proofs because theorems involve constants and \u000A defined functions and when those constants ``drift into the maw'' of a \u000A function, the function call can be eliminated and replaced by a new constant. \u000A ACL2 does this automatically; you don't have to tell it.  In fact, there are a \u000A few occasions where you might prefer it <i>not</i> evaluate and those are the \u000A ones you have to look out for!  They'll be obvious when they happen because \u000A you'll see a mysterious constant crop up in the proof.</p> \u000A \u000A <p>Evaluation is legal because it is just the repeated use of unconditional \u000A rewriting to replace definitions by their instantiated bodies until no \u000A function calls remain.</p> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the example proof in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____DELETE-INCLUDE-BOOK-DIR_12":[["Books-reference"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see>, which has completely \u000A analogous syntax and semantics, but is used for removing associations \u000A previously placed by <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR\">add-include-book-dir</see></tt>.  By contrast, \u000A <v>delete-include-book-dir!</v> removes associations previously placed by \u000A <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt>.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <p>This macro is essentially a <tt><see topic=\"ACL2____TABLE\">table</see></tt> event that updates the table \u000A <v>include-book-dir!-table</v>, which associates keywords with absolute \u000A pathnames.  However, as with <tt><see topic=\"ACL2____DELETE-INCLUDE-BOOK-DIR\">delete-include-book-dir</see></tt>, direct table \u000A updates are disallowed; you must use <v>delete-include-book-dir!</v> to remove \u000A from the table and <tt><see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see></tt> to add to the table.</p> \u000A \u000A <p>It is illegal to call <v>delete-include-book-dir!</v> in a <tt><see topic=\"ACL2____LOCAL\">local</see></tt> \u000A context.  For an explanation, see <see topic=\"ACL2____ADD-INCLUDE-BOOK-DIR_12\">add-include-book-dir!</see>.</p>"],

 "ACL2____DEFABSSTOBJ-MISSING-EVENTS":[["Events"],"ACL2 Sources","ACL2","<p>We assume familiarity with <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt>. \u000A <v>Defabsstobj-missing-events</v> is a macro is for advanced users (who, for \u000A example, understand the role of the <v>translate</v> and <v>untranslate</v> \u000A functions), who want programmatic access to the <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> events required \u000A to admit a specific <v>defabsstobj</v> event.</p> \u000A \u000A <p>This macro has the same syntax as <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> &mdash; to use it, \u000A just replace a call of <tt><see topic=\"ACL2____DEFABSSTOBJ\">defabsstobj</see></tt> by a call of \u000A <v>defabsstobj-missing-events</v> on the same arguments.  The result is an error \u000A triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>.  If <v>erp</v> is <v>nil</v>, then <v>val</v> is the \u000A list of all objects <v>(<see topic=\"ACL2____NAME\">name</see> formula . old-formula)</v>, where a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> \u000A event named <v>name</v> remains to be admitted whose translated formula is \u000A <v>formula</v>, and where <v>old-formula</v> is <v>nil</v> unless the indicated \u000A event already exists (hence with a different formula), in which case \u000A <v>old-formula</v> is the existing translated formula.</p> \u000A \u000A <p>To build a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event from the above value, <v>val</v>, we suggest \u000A evaluating a form like <v>(<see topic=\"ACL2____UNTRANSLATE\">untranslate</see> formula t (w state))</v>.</p>"],

 "COMMON-LISP____TRUNCATE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> 14 3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> -14 3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> 14 -3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> -14 -3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> -15 -3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> 10/4 3/4)\u000A3</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____TRUNCATE\">Truncate</see> i j)</v> is the result of taking the quotient of <v>i</v> and \u000A <v>j</v> and dropping the fraction.  For example, the quotient of <v>-14</v> by \u000A <v>3</v> is <v>-4 2/3</v>, so dropping the fraction <v>2/3</v>, we obtain a result \u000A for <v>(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> -14 3)</v> of <v>-4</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> i j)</v> requires that <v>i</v> and <v>j</v> \u000A are rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Truncate</v> is a Common Lisp function.  However, note that unlike Common \u000A Lisp, the ACL2 <v>truncate</v> function returns only a single value, Also see \u000A <see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see>, which is appropriate for integers and may \u000A simplify reasoning, unless a suitable arithmetic library is loaded, but be \u000A less efficient for evaluation on concrete objects.</p> \u000A \u000A <p><b>Function: </b><srclink>truncate</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> truncate (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> i)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> j)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> j 0)))))\u000A       (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((<see topic=\"ACL2____Q\">q</see> (<see topic=\"COMMON-LISP_____A2\">*</see> i (<see topic=\"COMMON-LISP_____F2\">/</see> j)))\u000A              (n (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> q))\u000A              (d (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> q)))\u000A             (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____D3\">=</see> d 1) n)\u000A                   ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                    (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> n d))\u000A                   (t (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> (<see topic=\"COMMON-LISP____-\">-</see> n)\u000A                                                       d))))))</code>"],

 "COMMON-LISP____LENGTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Length</v> is the function for determining the length of a \u000A sequence.  In ACL2, the argument is required to be either a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> \u000A or a string.</p> \u000A \u000A <p><v>Length</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>length</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> length (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) t (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A           (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list))\u000A           (<see topic=\"ACL2____LEN\">len</see> x)))</code>"],

 "ACL2____SHARP-DOT-READER":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____DEFCONST\">defconst</see> *a* '(a b c))\u000A\u000ASummary\u000AForm:  ( DEFCONST *A* ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A *A*\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____QUOTE\">quote</see> (1 2 #.*a* 3 4))\u000A(1 2 (A B C) 3 4)\u000AACL2 !&gt;</code> \u000A \u000A <p>The ACL2 reader supports the syntax <v>#.*a*</v> where <v>*a*</v> was defined \u000A by <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>.  In this case, the reader treats <v>#.*a*</v> as though it \u000A were reading the value of <v>*a*</v>.  This feature can be useful in conjunction \u000A with the use of <tt><see topic=\"ACL2____EVISC-TABLE\">evisc-table</see></tt> to abbreviate large constants, so that the \u000A abbreviation can be read back in; see <see topic=\"ACL2____EVISC-TABLE\">evisc-table</see>.</p> \u000A \u000A <p>Remarks.</p> \u000A \u000A <p>(1) The ACL2 reader only supports `<v>#.</v>' as described above, unlike \u000A Common Lisp.  Older versions (preceding 3.5) used `<v>#.</v>' to abort, but that \u000A functionality is now carried out by <v>(<see topic=\"ACL2____A_12\">a!</see>)</v>; see <see topic=\"ACL2____A_12\">a!</see>.  For a related \u000A feature that only pops up one level, see <see topic=\"ACL2____P_12\">p!</see>.</p> \u000A \u000A <p>(2) If you call <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> on a book that contains a form \u000A `<v>#.*foo*</v>', the <v>*foo*</v> must already be defined in the <see topic=\"ACL2____WORLD\">world</see> in \u000A which you issue the <v>certify-book</v> command.  The reason is that \u000A <v>certify-book</v> reads the entire book before evaluating its forms.</p>"],

 "ACL2____SET-CHECKPOINT-SUMMARY-LIMIT":[["Set-gag-mode"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see> for a discussion of key checkpoints.</p> \u000A \u000A <code>Examples:\u000A\u000A; (<see topic=\"ACL2____DEFAULT\">Default</see>) When a proof fails, print all key checkpoints before\u000A; induction but at most 3 key checkpoints after induction:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (nil . 3))\u000A\u000A; When a proof fails, print at most 3 key checkpoints before\u000A; induction but all 3 key checkpoints after induction:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (3 . nil))\u000A\u000A; When a proof fails, print at most 3 key checkpoints before\u000A; induction and at most 5 key checkpoints after induction:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (3 . 5))\u000A\u000A; When a proof fails, print at most 3 key checkpoints before\u000A; induction and at most 3 key checkpoints after induction:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (3 . 3))\u000A; or equivalently,\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> 3)\u000A\u000A; When a proof fails, print all key checkpoints:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (nil . nil))\u000A; or equivalently,\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> nil)\u000A\u000A; When a proof fails, print no information at all about key checkpoints:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> t)\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> (n1 . n2))\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> n)\u000A(<see topic=\"ACL2____SET-CHECKPOINT-SUMMARY-LIMIT\">set-checkpoint-summary-limit</see> t)</code> \u000A \u000A <p>where each of <v>n1</v> and <v>n2</v> is a natural number or <v>nil</v>.  For the \u000A second form, <v>n</v> can be a natural number or <v>nil</v> and is treated as \u000A <v>(n . n)</v>.  The value <v>t</v> inhibits all printing of checkpoint summary \u000A information.  The values <v>n1</v> and <v>n2</v> determine printing of key \u000A checkpoints generated before the first induction and generated after the first \u000A induction, respectively, where at most <v>n1</v> or <v>n2</v> (respectively) such \u000A key checkpoints are printed unless the value is <v>nil</v>, in which case there \u000A is no limitation.</p> \u000A \u000A <p>The argument <v>x</v> for <v>set-checkpoint-summary-limit</v>, as described \u000A above, may be quoted, i.e. supplied as <v>'x</v> or <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> x)</v>.  Thus, you \u000A may use the keyword form (see <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>) if you prefer, for \u000A example:</p> \u000A \u000A <code>:set-checkpoint-summary-limit (nil . 3)</code>"],

 "ACL2____SET-BODY":[["Definition","Events"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-BODY\">set-body</see> foo (:definition foo)) ; restore original definition of foo\u000A(<see topic=\"ACL2____SET-BODY\">set-body</see> foo foo) ; same as just above\u000A(<see topic=\"ACL2____SET-BODY\">set-body</see> foo my-foo-def) ; use my-foo-def for the body of foo\u000A(<see topic=\"ACL2____SET-BODY\">set-body</see> foo (:definition my-foo-def)) ; same as just above</code> \u000A \u000A <p>Rules of class <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> can install a new definition body, \u000A used for example by <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>.  See <see topic=\"ACL2____DEFINITION\">definition</see> and also \u000A see <see topic=\"ACL2____HINTS\">hints</see> for a detailed discussion of the <v>:install-body</v> fields of \u000A <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules and their role in <v>:expand</v> hints.</p> \u000A \u000A <p>There may be several such definitions, but by default, the latest one is \u000A used by <v>:expand</v> hints.  Although the <v>:with</v> keyword may be used in \u000A <v>:expand</v> hints to override this behavior locally (see <see topic=\"ACL2____HINTS\">hints</see>), it \u000A may be convenient to install a definition for expansion other than the latest \u000A one &mdash; for example, the original definition.  <v>Set-body</v> may be used \u000A for this purpose.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-BODY\">set-body</see> function-symbol rule-name)</code> \u000A \u000A <p>where <v>rule-name</v> is either a <v>:definition</v> <see topic=\"ACL2____RUNE\">rune</see> or is a \u000A function symbol, <v>sym</v>, which represents the rune <v>(:definition\u000A sym)</v>.</p> \u000A \u000A <p>You can view all definitions available for expansion; see <see topic=\"ACL2____SHOW-BODIES\">show-bodies</see>.</p>"],

 "ACL2____PRINC_42":[["ACL2-built-ins","Io"],"ACL2 Sources","ACL2","<p>Use <v>princ$</v> to do basic printing of atoms (i.e., other than \u000A <v>cons</v> pairs).  In particular, <v>princ$</v> prints a string without the \u000A surrounding double-quotes and without escaping double-quote characters within \u000A the string.  Note that <v>princ$</v> is sensitive to the print-base, \u000A print-radix, and print-case; see <see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see> and see <see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see>.  <v>Princ$</v> returns <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> &quot;Howdy ho&quot; (<see topic=\"ACL2____STANDARD-CO\">standard-co</see> state) state)\u000AHowdy ho&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PPROGN\">pprogn</see> (<see topic=\"ACL2____PRINC_42\">princ$</see> &quot;Howdy ho&quot; (<see topic=\"ACL2____STANDARD-CO\">standard-co</see> state) state)\u000A               (newline (<see topic=\"ACL2____STANDARD-CO\">standard-co</see> state) state))\u000AHowdy ho\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> &quot;ab\\&quot;cd&quot; *standard-co* state)\u000Aab&quot;cd&lt;state&gt;\u000AACL2 !&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> 17 *standard-co* state)\u000A17&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see> 16 state) ; might be nicer to use set-print-base-radix\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> 17 *standard-co* state)\u000A11&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see> t state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> 17 *standard-co* state)\u000A#x11&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> 'xyz *standard-co* state)\u000AXYZ&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-CASE\">set-print-case</see> :downcase state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"ACL2____PRINC_42\">princ$</see> 'xyz *standard-co* state)\u000Axyz&lt;state&gt;\u000AACL2 !&gt;</code> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"ACL2____PRINC_42\">princ$</see> x channel state)</v> is essentially as \u000A follows; see <see topic=\"ACL2____IO\">io</see> for an explanation of guards of certain built-in \u000A functions that take <see topic=\"ACL2____STATE\">state</see>, such as <v>princ$</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A         (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A         (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A         (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x))\u000A     (state-p1 state-state)\u000A     (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> channel)\u000A     (open-output-channel-p1 channel :character state-state))</code> \u000A \u000A <p>See <see topic=\"ACL2____FMT\">fmt</see> for more sophisticated printing routines, and see <see topic=\"ACL2____IO\">io</see> \u000A for general information about input and output.</p>"],

 "ACL2____MUST-BE-EQUAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The form <v>(<see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see> logic exec)</v> evaluates to <v>logic</v> in \u000A the ACL2 logic but evaluates to <v>exec</v> in raw Lisp.  The point is to be \u000A able to write one definition to reason about logically but another for \u000A evaluation.  Please see <see topic=\"ACL2____MBE\">mbe</see> and see <see topic=\"ACL2____MBT\">mbt</see> for appropriate macros \u000A to use, rather than calling <v>must-be-equal</v> directly, since it is easy to \u000A commute the arguments of <v>must-be-equal</v> by accident.</p> \u000A \u000A <p>In essence, the guard for <v>(<see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see> x y)</v> is <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v>. \u000A However, note that <v>must-be-equal</v> is a macro: <v>(<see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see> logic\u000A exec)</v> expands to <v>(<see topic=\"ACL2____MBE1\">mbe1</see> exec logic)</v>, which expands to a call of <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>.</p>"],

 "ACL2____HEADER":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____HEADER\">header</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____HEADER\">header</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is arbitrary and <v>alist</v> is a 1- or 2-dimensional array. \u000A This function returns the header of the array <v>alist</v>.  The function \u000A operates in virtually constant time if <v>alist</v> is the semantic value of \u000A <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>header</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     header (<see topic=\"ACL2____NAME\">name</see> l)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l) (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l))))\u000A     (<see topic=\"ACL2____PROG2_42\">prog2$</see> name (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> :header l)))</code>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-NAT-RECURSION":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Classical Induction on Natural Numbers</b>: Induction is familiar in the \u000A arithmetic setting.  To prove <v>(p n)</v>, for all <v>n</v>, by classical \u000A induction on the construction of the natural numbers, prove each of the \u000A following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (zp n) (p n)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (zp n)) \u000A              (p (- n 1))) \u000A         (p n)) \u000A</code> \u000A \u000A <p>The Base Case establishes that <v>p</v> holds for <v>0</v>.  In fact, because \u000A of the definition of <tt><see topic=\"ACL2____ZP\">zp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>, it \u000A establishes that <v>(p n)</v> holds when <v>n</v> is <v>0</v> and it holds when \u000A <v>n</v> is not a natural number.</p> \u000A \u000A <p>The Induction Step establishes that if <v>n</v> is a natural number other \u000A than <v>0</v>, and if <v>p</v> holds for <v>n</v>-1, then <v>p</v> holds for <v>n</v>. \u000A The hypothesis <v>(p (<see topic=\"COMMON-LISP____-\">-</see> n 1))</v> above is called the <i>induction \u000A hypothesis</i>.</p> \u000A \u000A <p>A function that suggests this induction is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nat-recursion (n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      n\u000A      (nat-recursion (<see topic=\"COMMON-LISP____-\">-</see> n 1))))</code> \u000A \u000A <p>Similarly, the term <v>(fact n)</v> suggests this induction if <v>fact</v> is \u000A defined:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fact (k)\u000A (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> k)\u000A     1\u000A     (<see topic=\"COMMON-LISP_____A2\">*</see> k (fact (<see topic=\"COMMON-LISP____-\">-</see> k 1))))).</code> \u000A \u000A <p>even though the formal parameter of this definition of <v>fact</v> is <v>k</v>, \u000A not <v>n</v>.</p>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-DOWN-BY-2":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Classical Induction on Natural Numbers Preserving Parity</b>: Here is \u000A another way to decompose natural numbers.  To prove <v>(p n)</v>, for all \u000A <v>n</v>, prove each of the following:</p> \u000A \u000A <code><i>Base Case 1</i>: \u000A(implies (zp n) (p n)) \u000A</code> \u000A \u000A <code><i>Base Case 2</i>: \u000A(implies (equal n 1) (p n)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (zp n)) \u000A              (not (equal n 1)) \u000A              (p (- n 2))) \u000A         (p n)) \u000A</code> \u000A \u000A <p>Base Case 1 establishes that <v>p</v> holds for <v>0</v> (and all objects other \u000A than positive naturals).</p> \u000A \u000A <p>Base Case 2 establishes that <v>p</v> holds for <v>1</v>.</p> \u000A \u000A <p>The Induction Step establishes that if <v>n</v> is a natural number greater \u000A than <v>1</v>, and if <v>p</v> holds for <v>n</v>-2, then <v>p</v> holds for \u000A <v>n</v>.</p> \u000A \u000A <p>Note that we have thus proved that <v>(p n)</v> holds, for all <v>n</v>.  For \u000A example, <v>(p -7)</v>, <v>(p 'abc)</v>, and <v>(p 0)</v> are all established by \u000A Base Case 1.  <v>(p 1)</v> is established by Base Case 2.  <v>(p 2)</v> is \u000A established from <v>(p 0)</v> and the Induction Step.  Think about it!  <v>(p\u000A 3)</v> is established form <v>(p 1)</v> and the Induction Step, etc.</p> \u000A \u000A <p>A function that suggests this induction is:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> parity (n)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A      'even\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> n 1)\u000A          'odd\u000A          (parity (<see topic=\"COMMON-LISP____-\">-</see> n 2))))).</code>"],

 "ACL2____DEFPUN":[["Events"],"ACL2 Sources","ACL2","<p><v>Defpun</v> is a macro developed by Pete Manolios and J Moore that \u000A allows tail-recursive definitions.  It is defined in community book \u000A <v>books/misc/defpun.lisp</v>, so to use it, execute the following event.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/defpun&quot; :dir :system)</code> \u000A \u000A <p>Details of defpun are provided by Manolios and Moore in the ``Partial \u000A Functions in ACL2'' published with the <a \u000A href='http://www.cs.utexas.edu/users/moore/acl2/workshop-2000/'>ACL2 2000 \u000A workshop</a>.  Also see <a \u000A href='http://www.cs.utexas.edu/users/moore/publications/defpun/index.html'>Partial \u000A Functions in ACL2</a>.</p> \u000A \u000A <p>A variant, <v>defp</v>, has been developed by Matt Kaufmann to allow more \u000A general forms of tail recursion.  If <v>defpun</v> doesn't work for you, try \u000A <v>defp</v> by first executing the following event.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;misc/defp&quot; :dir :system)</code> \u000A \u000A <p>Sandip Ray has contributed a variant of <v>defpun</v>, <v>defpun-exec</v>, that \u000A supports executability.  See community book \u000A <v>books/defexec/defpun-exec/defpun-exec.lisp</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;defexec/defpun-exec/defpun-exec&quot; :dir :system)</code> \u000A \u000A <p>He has also contributed community book \u000A <v>books/misc/misc2/defpun-exec-domain-example.lisp</v>, for functions that are \u000A uniquely defined in a particular domain.</p>"],

 "ACL2____DEFEQUIV":[["Events"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> set-equal)\u000A\u000Ais an abbreviation for\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> set-equal-is-an-equivalence\u000A  (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____BOOLEANP\">booleanp</see> (set-equal x y))\u000A       (set-equal x x)\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (set-equal x y) (set-equal y x))\u000A       (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (set-equal x y)\u000A                     (set-equal y z))\u000A                (set-equal x z)))\u000A  :rule-classes (:equivalence))</code> \u000A \u000A <p>See <see topic=\"ACL2____EQUIVALENCE\">equivalence</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFEQUIV\">defequiv</see> fn\u000A  :rule-classes rule-classes\u000A  :instructions instructions\u000A  :hints hints\u000A  :otf-flg otf-flg\u000A  :event-name event-name\u000A  :doc doc)</code> \u000A \u000A <p>where <v>fn</v> is a function symbol of arity 2, <v>event-name</v>, if \u000A supplied, is a symbol, and all other arguments are as specified in the \u000A documentation for <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  The <v>defequiv</v> macro expands into a call \u000A of <v>defthm</v>.  The name of the <v>defthm</v> is <v>fn-is-an-equivalence</v>, \u000A unless <v>event-name</v> is supplied, in which case <v>event-name</v> is the name \u000A used.  The term generated for the <v>defthm</v> event states that <v>fn</v> is \u000A Boolean, reflexive, symmetric, and transitive.  The rule-class \u000A <v>:equivalence</v> is added to the <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> specified, if it is not \u000A already there.  All other arguments to the generated <v>defthm</v> form are as \u000A specified by the other keyword arguments above.</p>"],

 "ACL2____COMPILING-ACL2P":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.  See <see topic=\"ACL2____PARALLELISM-TUTORIAL\">parallelism-tutorial</see> for an introduction to parallel \u000A programming in ACL2.</p> \u000A \u000A <p>You can build an experimental version of ACL2 that supports parallel \u000A execution in the following host Common Lisp implementations:</p> \u000A \u000A <blockquote><p>* CCL (OpenMCL)</p> \u000A \u000A <p>* Lispworks 6.0</p> \u000A \u000A <p>* SBCL with threads (feature <v>:sb-thread</v>)</p></blockquote> \u000A \u000A <p>The command below will compile ACL2 to support parallel execution, \u000A including parallel execution during proofs.  Any non-empty string may be used \u000A in place of <v>t</v>, and the value of <v>LISP</v> (shown here as <v>ccl</v>) is any \u000A Lisp executable on which one can build ACL2(p) (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>).</p> \u000A \u000A <code>make ACL2_PAR=t LISP=ccl</code> \u000A \u000A <p>So for example, to make an executable image and also documentation (which \u000A will appear in subdirectories <v>doc/EMACS</v> and <v>doc/HTML</v>), using the \u000A Lisp executable <v>ccl</v>:</p> \u000A \u000A <code>make large DOC ACL2_PAR=t LISP=ccl</code>"],

 "ACL2____BINARY-_B2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-+</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y)\u000A               (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y)\u000A             x)\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y)\u000A             y\u000A           0)))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x) (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))</code> \u000A \u000A <p>Notice that like all arithmetic functions, <v>binary-+</v> treats non-numeric \u000A inputs as <v>0</v>.</p> \u000A \u000A <p>Calls of the macro <tt><see topic=\"COMMON-LISP_____B2\">+</see></tt> expand to calls of <v>binary-+</v>; see <see topic=\"COMMON-LISP_____B2\">+</see>.</p>"],

 "ACL2____A_12":[["Ld"],"ACL2 Sources","ACL2","<p>When <v>(<see topic=\"ACL2____A_12\">a!</see>)</v> is evaluated inside of ACL2's command loop, the \u000A current computation is aborted and control returns to the top of the command \u000A loop, exactly as though the user had interrupted and aborted the current \u000A computation.  (Note: Versions of ACL2 up to Version_3.4 provided `<v>#.</v>' for \u000A this purpose, but no longer; see <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>.)</p> \u000A \u000A <p>If you are at an ACL2 prompt (as opposed to a raw Lisp break), then you may \u000A type <v>:a!</v> in place of <v>(<see topic=\"ACL2____A_12\">a!</see>)</v>; see <see topic=\"ACL2____KEYWORD-COMMANDS\">keyword-commands</see>.</p> \u000A \u000A <p>For a related feature that only pops up one level, see <see topic=\"ACL2____P_12\">p!</see>.</p> \u000A \u000A <p>Logically speaking, <v>(<see topic=\"ACL2____A_12\">a!</see>) = nil</v>.  But imagine that it is defined in \u000A such a way that it causes a stack overflow or other resource exhaustion when \u000A called.</p>"],

 "ACL2____A_02Flying_02Tour_02of_02ACL2":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____About_02the_02ACL2_02Home_02Page'><img \u000A src='large-flying.gif'></img></see></p> \u000A \u000A <p>On this tour you will learn a little about what ACL2 is for rather than how \u000A ACL2 works.  At the top and bottom bottom of the ``page'' there are ``flying \u000A tour'' icons.  Click on either icon to go to the next page of the tour.</p> \u000A \u000A <p>The tour visits the following topics sequentially.  But on your first \u000A reading, don't navigate through the tour by clicking on these links; they are \u000A shown as live links only so that later you can determine what you've visited. \u000A Instead, just use the flying tour icons.</p> \u000A \u000A <code><b>The Flight Plan</b> \u000A* <see topic='ACL2____About_02the_02ACL2_02Home_02Page'>This Documentation</see> \u000A* <see topic='ACL2____What_02Is_02ACL2_82Q_92'>What is ACL2?</see> \u000A* <see topic='ACL2____What_02is_02a_02Mathematical_02Logic_82Q_92'>Mathematical Logic</see> \u000A* <see topic='ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92'>Mechanical Theorem Proving</see> \u000A* <see topic='ACL2____About_02Models'>Mathematical Models in General</see> \u000A* <see topic='ACL2____Models_02of_02Computer_02Hardware_02and_02Software'>Mathematical Models of Computing Machines</see> \u000A     <see topic='ACL2____A_02Typical_02State'>Formalizing Models</see> \u000A     <see topic='ACL2____Running_02Models'>Running Models</see> \u000A     <see topic='ACL2____Symbolic_02Execution_02of_02Models'>Symbolic Execution of Models</see> \u000A     <see topic='ACL2____Proving_02Theorems_02about_02Models'>Proving Theorems about Models</see> \u000A* Requirements of ACL2 \u000A     <see topic='ACL2____What_02is_02Required_02of_02the_02User_82Q_92'>The User's Skills</see> \u000A     <see topic='ACL2____How_02Long_02Does_02It_02Take_02to_02Become_02an_02Effective_02User_82Q_92'>Training</see> \u000A     <see topic='ACL2____Other_02Requirements'>Host System</see> \u000A</code> \u000A \u000A <p>On your first reading, don't explore other links you see in the tour.  Some \u000A of them lead to the Walking Tour, which you can take coherently when you \u000A finish this tour.  Others lead into the extensive hyptertext documentation and \u000A you are liable to get lost there unless you're trying to answer a specific \u000A question.  We intend the tour to take about 10 minutes of your time.</p> \u000A \u000A <p><see topic='ACL2____About_02the_02ACL2_02Home_02Page'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____AREF2":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____AREF2\">aref2</see> 'delta1 a i j)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____AREF2\">aref2</see> name alist i j)</code> \u000A \u000A <p>where <v>name</v> is a symbol, <v>alist</v> is a 2-dimensional array and <v>i</v> \u000A and <v>j</v> are legal indices into <v>alist</v>.  This function returns the value \u000A associated with <v>(i . j)</v> in <v>alist</v>, or else the default value of the \u000A array.  See <see topic=\"ACL2____ARRAYS\">arrays</see> for details.</p> \u000A \u000A <p>This function executes in virtually constant time if <v>alist</v> is in fact \u000A the ``semantic value'' associated with <v>name</v> (see <see topic=\"ACL2____ARRAYS\">arrays</see>).  When it \u000A is not, <v>aref2</v> must do a linear search through <v>alist</v>.  In that case \u000A the correct answer is returned but a <b>slow array</b> comment is printed to \u000A the comment window.  See <see topic=\"ACL2____SLOW-ARRAY-WARNING\">slow-array-warning</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>aref2</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     aref2 (<see topic=\"ACL2____NAME\">name</see> l i j)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l)\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l)))\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                                 (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> j 0)\u000A                                 (<see topic=\"COMMON-LISP_____C3\">&lt;</see> j (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name l))))))\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((x (assoc2 i j l)))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> x) (<see topic=\"ACL2____DEFAULT\">default</see> name l))\u000A                (t (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))</code>"],

 "ACL2____ABOUT-ACL2":[["ACL2"],"ACL2 Sources","ACL2","<p>This is ACL2 Version 6.5, <see topic=\"ACL2____COPYRIGHT\">copyright</see> (C) 2014, \u000A Regents of the University of Texas, authored by Matt Kaufmann and J Strother \u000A Moore.</p> \u000A \u000A <p>See the <a href='http://www.cs.utexas.edu/users/moore/acl2/'>ACL2 home \u000A page</a> for additional information including tutorials, installation \u000A instructions, mailing lists, related publications, ACL2 workshops and \u000A seminars, acknowledgements, and other ACL2 releases.</p> \u000A \u000A <p>See <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for how to access the ACL2 User's Manual.</p> \u000A \u000A <p>For statistics on ACL2 code size, see file <v>doc/acl2-code-size.txt</v>.</p>"],

 "ACL2-PC____SPLIT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Asplit</code> \u000A \u000A <p>For example, if the current goal has one hypothesis <v>(<see topic=\"COMMON-LISP____OR\">or</see> x y)</v> and a \u000A conclusion of <v>(<see topic=\"COMMON-LISP____AND\">and</see> a b)</v>, then <v>split</v> will create four new goals:</p> \u000A \u000A <code>one with hypothesis X and conclusion A\u000Aone with hypothesis X and conclusion B\u000Aone with hypothesis Y and conclusion A\u000Aone with hypothesis Y and conclusion B.\u000A\u000AGeneral Form:\u000ASPLIT</code> \u000A \u000A <p>Replace the current goal by subgoals whose conjunction is equivalent \u000A (primarily by propositional reasoning) to the original goal, where each such \u000A goal cannot be similarly split.</p> \u000A \u000A <p><b>Remark:</b> The new goals will all have their hypotheses promoted; in \u000A particular, no conclusion will have a top function symbol of <v>implies</v>. \u000A Also note that <v>split</v> will fail if there is exactly one new goal created \u000A and it is the same as the existing current goal.</p> \u000A \u000A <p>The way <v>split</v> really works is to call the ACL2 theorem prover with \u000A only simplification (and preprocessing) turned on, and with only a few \u000A built-in functions (especially, propositional ones) enabled, namely, the ones \u000A in the list <v>(<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)</v>.  However, because the prover is \u000A called, type-set reasoning can be used to eliminate some cases.  For example, \u000A if <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)</v> is in the hypotheses, then probably <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see>\u000A x))</v> will be reduced to <v>t</v>.</p>"],

 "COMMON-LISP____SYMBOL-NAME":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-symbol-name</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A           (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x)\u000A         &quot;&quot;))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)</code>"],

 "COMMON-LISP____SUBSEQ":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For any natural numbers <v>start</v> and <v>end</v>, where <v>start</v> \u000A <v>&lt;=</v> <v>end</v> <v>&lt;=</v> <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)</v>, <v>(<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> seq start end)</v> is the \u000A subsequence of <v>seq</v> from index <v>start</v> up to, but not including, index \u000A <v>end</v>.  <v>End</v> may be <v>nil</v>, in which case it is treated as though it \u000A is <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)</v>, i.e., we obtain the subsequence of <v>seq</v> from index \u000A <v>start</v> all the way to the end.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____SUBSEQ\">subseq</see> seq start end)</v> is that <v>seq</v> is a true \u000A list or a string, <v>start</v> and <v>end</v> are integers (except, <v>end</v> may \u000A be <v>nil</v>, in which case it is treated as <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)</v> for the rest of \u000A this discussion), and <v>0</v> <v>&lt;=</v> <v>start</v> <v>&lt;=</v> <v>end</v> <v>&lt;=</v> \u000A <v>(<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)</v>.</p> \u000A \u000A <p><v>Subseq</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  Note: In Common Lisp the third argument of <v>subseq</v> \u000A is optional, but in ACL2 it is required, though it may be <v>nil</v> as \u000A explained above.</p> \u000A \u000A <p><b>Function: </b><srclink>subseq</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     subseq (seq start end)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq))\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> start)\u000A                                 (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 start)\u000A                                 (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NULL\">null</see> end)\u000A                                     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> end)\u000A                                          (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> end (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq))))\u000A                                 (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> start (<see topic=\"COMMON-LISP____OR\">or</see> end (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq))))))\u000A     (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq)\u000A         (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (subseq-list (<see topic=\"COMMON-LISP____COERCE\">coerce</see> seq 'list)\u000A                              start (<see topic=\"COMMON-LISP____OR\">or</see> end (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)))\u000A                 'string)\u000A         (subseq-list seq start (<see topic=\"COMMON-LISP____OR\">or</see> end (<see topic=\"COMMON-LISP____LENGTH\">length</see> seq)))))</code>"],

 "COMMON-LISP____INTEGERP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v> is true if and only if <v>x</v> is an integer.</p>"],

 "ACL2____WATERFALL-PARALLELISM-FOR-BOOK-CERTIFICATION":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>There are books whose certification can be sped up significantly by using \u000A waterfall parallelism.  (See <see topic=\"ACL2____PARALLELISM\">parallelism</see>, including the caveat in its \u000A \"IMPORTANT NOTE\".)  One such example in the ACL2 community books is \u000A <v>models/jvm/m5/apprentice.lisp</v>, which is typically excluded from \u000A regressions because of how long it takes to certify.  In order to use \u000A waterfall parallelism during certification of a book <v>&lt;book-name&gt;.lisp</v> \u000A using `make' (see <see topic=\"ACL2____BOOKS-CERTIFICATION\">books-certification</see> and see <see topic=\"ACL2____BOOKS-CERTIFICATION-CLASSIC\">books-certification-classic</see>), we recommend creating a file \u000A <v>&lt;book-name&gt;.acl2</v> that includes the following forms.</p> \u000A \u000A <code>#+acl2-par\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see> t)\u000A\u000A(<see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> &lt;book-name&gt; ? t) ; other arguments may be preferable</code> \u000A \u000A <p>Note that there are books that will not certify when waterfall-parallelism \u000A is enabled.  See file <v>acl2-customization-files/README</v> for more \u000A information, including how to certify essentially all books using waterfall \u000A parallelism.</p>"],

 "ACL2____TAU-DATA":[["Tau-system"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____TAU-DATA\">tau-data</see> binary-+)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____TAU-DATA\">tau-data</see> fn)</code> \u000A \u000A <p>This macro returns a list structure that indicates what facts about the \u000A symbol <v>fn</v> are known to the tau system.  <v>Fn</v> should either be a \u000A function symbol or else a macro associated with <v>fn</v>; see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.  See <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> for \u000A background details.</p> \u000A \u000A <p>The list structure should be self-explanatory given the following brief \u000A comments.  The ``index'' of a function, when non-<v>nil</v>, means the function \u000A is a monadic Boolean function treated by the tau system as a tau \u000A predicate.</p> \u000A \u000A <p>The ``positive'' and ``negative implicants'' are conjunctions that indicate \u000A the tau implied by the given one or its negation.</p> \u000A \u000A <p>The ``signatures'' entry is a formula indicating all the known signatures. \u000A If the signatures formula is <v>T</v> it means there are no known signatures. \u000A (Signatures is the conjunction of all signature rules and the empty \u000A conjunction is <v>T</v>.)</p> \u000A \u000A <p>If you wish to see a long list of all the runes from which some tau \u000A information has been gleaned, evaluate <v>(global-val 'tau-runes (w\u000A state))</v>.</p>"],

 "ACL2____SET-MEASURE-FUNCTION":[["Defun"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-MEASURE-FUNCTION\">set-measure-function</see> nqthm::count)</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-MEASURE-FUNCTION\">set-measure-function</see> name)</code> \u000A \u000A <p>where <v>name</v> is a function symbol of one argument.  This macro is \u000A equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :measure-function 'name)</v>, and \u000A hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  Although this is \u000A thus an event (see <see topic=\"ACL2____TABLE\">table</see>), nevertheless no output results from a \u000A <v>set-measure-function</v> event.</p> \u000A \u000A <p>This event sets the default measure function to <v>name</v>.  Subsequently, \u000A if a recursively defined function is submitted to <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> with no \u000A explicitly given <v>:measure</v> argument, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> ``guesses'' the measure \u000A <v>(<see topic=\"ACL2____NAME\">name</see> var)</v>, where <v>name</v> is the then current default measure function \u000A and <v>var</v> is the first formal found to be tested along every branch and \u000A changed in every recursive call.</p> \u000A \u000A <p>Note that if <v>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :measure-function 'name)</v> has \u000A its default value of <v>nil</v>, then the default measure function is <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt>.</p>"],

 "ACL2____SET-IGNORE-DOC-STRING-ERROR":[["Legacy-documentation"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see> nil)   ; :doc strings must be well-formed\u000A(<see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see> t)     ; ill-formed :doc strings are ignored\u000A(<see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see> :warn) ; ill-formed :doc strings are ignored\u000A                                    ;   except for causing a warning</code> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-IGNORE-DOC-STRING-ERROR\">set-ignore-doc-string-error</see> flag)</code> \u000A \u000A <p>where <v>flag</v> is <v>nil</v>, <v>t</v>, or <v>:warn</v>, as indicated above. \u000A This macro is essentially equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :ignore-doc-string-error flag)</code> \u000A \u000A <p>and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> \u000A <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.  However, \u000A unlike the above simple call of the <tt><see topic=\"ACL2____TABLE\">table</see></tt> event function (see <see topic=\"ACL2____TABLE\">table</see>), no output results from a <v>set-ignore-doc-string-error</v> event.</p> \u000A \u000A <p>Note that since <tt><see topic=\"ACL2____DEFDOC\">defdoc</see></tt> <see topic=\"ACL2____EVENTS\">events</see> have the sole purpose of \u000A installing <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings, these require well-formed \u000A documentation strings even after executing a call of \u000A <v>ignore-doc-string-error</v>.</p> \u000A \u000A <p>The default behavior of the system is as though the \u000A <v>:ignore-doc-string-error</v> value is <v>nil</v>.  The current behavior can be \u000A ascertained by evaluating the following form.</p> \u000A \u000A <code>(ignore-doc-string-error (w state))</code>"],

 "ACL2____READ-ACL2-ORACLE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>ACL2 has a mutable `state' that contains a number of fields; see \u000A <see topic=\"ACL2____STATE\">state</see>.  One of these fields is a list, called the <v>acl2-oracle</v> \u000A field.  There are no constraints on the elements of this list; thus, you will \u000A not be able to prove anything about those elements.  However, as a \u000A convenience, ACL2 permits you to evaluate the form <tt>(read-acl2-oracle \u000A state)</tt>, which returns the multiple value <v>(<see topic=\"ACL2____MV\">mv</see> nil nil state)</v>.</p> \u000A \u000A <p>The ACL2 implementation uses <v>read-acl2-oracle</v>, but you will likely \u000A find this function to be useless unless you do advanced, scurrilous things \u000A using trust tags (see <tt><see topic=\"ACL2____DEFTTAG\">defttag</see></tt>).</p> \u000A \u000A <p>The following definition is the logical definition of \u000A <v>read-acl2-oracle</v>, but as explained above does not really characterize its \u000A behavior under the hood.</p> \u000A \u000A <p><b>Function: </b><srclink>read-acl2-oracle</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> read-acl2-oracle (state-state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (state-p1 state-state)))\u000A       (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____NULL\">null</see> (acl2-oracle state-state))\u000A           (<see topic=\"COMMON-LISP____CAR\">car</see> (acl2-oracle state-state))\u000A           (update-acl2-oracle (<see topic=\"COMMON-LISP____CDR\">cdr</see> (acl2-oracle state-state))\u000A                               state-state)))</code>"],

 "ACL2____PL2":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pl2 (<see topic=\"COMMON-LISP_____B2\">+</see> x y) nil ; prints rules that apply to (<see topic=\"COMMON-LISP_____B2\">+</see> x y)\u000A:pl2 (<see topic=\"COMMON-LISP_____B2\">+</see> x y) foo ; prints rules named foo that apply to (<see topic=\"COMMON-LISP_____B2\">+</see> x y)\u000A:pl2 (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (:rewrite foo) ; if the rule with rune (:rewrite foo) applies\u000A                            ;   to (<see topic=\"COMMON-LISP_____B2\">+</see> x y), then print it\u000A:pl2 (<see topic=\"COMMON-LISP_____B2\">+</see> x y) (:type-prescription foo)\u000A                            ; as above, but for the indicated\u000A                            ;   type-prescription rule</code> \u000A \u000A <p><v>Pl2</v> takes two arguments.  The first is a term.  The second is either \u000A <v>nil</v> or a ``rule-id'' that is either a symbol or a <see topic=\"ACL2____RUNE\">rune</see>.  The \u000A result is to print rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt>, <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>, <v>:meta</v>, <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt>, and <v>:</v><tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> that apply to the given term.  Indeed, <v>:pl2</v> prints \u000A exactly what is printed by applying <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt> to the first argument \u000A &mdash; see <see topic=\"ACL2____PL\">pl</see> &mdash; except that if the second argument is not \u000A <v>nil</v> then it is used to filter the rules printed, as follows.</p> \u000A \u000A <blockquote> \u000A \u000A <p>If the rule-id is a symbol, then only rules whose name is that symbol will \u000A be printed.</p> \u000A \u000A <p>If the rule-id is a <see topic=\"ACL2____RUNE\">rune</see>, then at most one rule will be printed: the \u000A rule named by that rune (if the rule would be printed by <v>:</v><tt><see topic=\"ACL2____PL\">pl</see></tt>).</p></blockquote>"],

 "ACL2____ORACLE-APPLY-RAW":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see>, as we assume familiarity with that \u000A function.  <v>Oracle-apply-raw</v> is a variant of <v>oracle-apply</v> that is \u000A untouchable, and hence requires a trust tag to remove the untouchability (see \u000A <see topic=\"ACL2____DEFTTAG\">defttag</see> and see <see topic=\"ACL2____REMOVE-UNTOUCHABLE\">remove-untouchable</see>).  Unlike <v>oracle-apply</v>, \u000A <v>oracle-apply-raw</v> simply calls the raw Lisp function <v>funcall</v> to \u000A compute the result, without restriction: the specified <v>:</v><tt><see topic=\"ACL2____GUARD\">guard</see></tt> is \u000A <v>t</v>, the function itself is applied (not its executable counterpart), there \u000A is no restriction for untouchable functions or <tt><see topic=\"ACL2____RETURN-LAST\">return-last</see></tt>, and safe \u000A mode is not used.  Thus, in general, <v>oracle-apply-raw</v> can be dangerous to \u000A use: any manner of error can occur!</p> \u000A \u000A <p>As is the case for <tt><see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see></tt>, the function symbol <tt><see topic=\"ACL2____ORACLE-APPLY-RAW\">oracle-apply-raw</see></tt> is defined in <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode and is <see topic=\"ACL2____GUARD\">guard</see>-verified.  <v>Oracle-apply-raw</v> is logically defined to be <tt><see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see></tt>; more precisely:</p> \u000A \u000A <code>(<see topic=\"ACL2____ORACLE-APPLY-RAW\">oracle-apply-raw</see> fn args state)\u000A= {logical definition}\u000A(<see topic=\"ACL2____EC-CALL\">ec-call</see> (<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> fn args state))</code> \u000A \u000A <p><b>Function: </b><srclink>oracle-apply-raw</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oracle-apply-raw (fn args state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state :guard t))\u000A       (<see topic=\"ACL2____EC-CALL\">ec-call</see> (<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> fn args state)))</code>"],

 "ACL2____NOTE-2-0":[["Release-notes"],"ACL2 Sources","ACL2","<p>This is the first version of ACL2 released under the copyright of \u000A the University of Texas (UT).  Future releases of ACL2 will be made from UT \u000A rather than Computational Logic, Inc. (CLI).  Version 2.0 is just Version 1.9 \u000A as released by CLI, with a few bugs fixed.</p> \u000A \u000A <p>A bug causing an infinite loop was fixed in functional instantiation.  The \u000A bug manifested itself when two conditions occurred simultaneously: First, the \u000A functional substitution replaces a function symbol, e.g., <v>FOO</v>, with a \u000A <v>LAMBDA</v> expression containing a free variable (a variable not among in the \u000A <v>LAMBDA</v> formals).  And, second, in one of the constraints being \u000A instantiated there is a call of the function symbol <v>FOO</v> within the scope \u000A of another <v>LAMBDA</v> expression.  Unless you used such a functional \u000A substitution, this bug fix will not affect you.</p> \u000A \u000A <p>Less important notes:</p> \u000A \u000A <p>The implementation of <v>PRINC$</v> was changed so that it was no longer \u000A sensitive to the external setting of <v>*print-base*</v> and other Common Lisp \u000A special variables.</p> \u000A \u000A <p>Typographical errors were fixed in the documentation.</p>"],

 "ACL2____MAKE-ORD":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Make-ord</v> is the ordinal constructor.  Its use is recommended \u000A instead of using <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> to make ordinals.  For a discussion of ordinals, \u000A see <see topic=\"ACL2____ORDINALS\">ordinals</see>.</p> \u000A \u000A <p>For any ordinal, <v>alpha &lt; epsilon-0</v>, there exist natural numbers <v>p</v> \u000A and <v>n</v>, positive integers <v>x1, x2, ..., xn</v> and ordinals <v>a1 &gt; a2 &gt;\u000A ... &gt; an &gt; 0</v> such that <v>alpha &gt; a1</v> and <v>alpha = w^(a1)x1 + w^(a2)x2 +\u000A ... + w^(an)xn + p</v>.  We call <v>a1</v> the ``first exponent'', <v>x1</v> the \u000A ``first coefficient'', and the remainder <v>(w^(a2)x2 + ... + w^(an)xn + p)</v> \u000A the ``rest'' of alpha.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____MAKE-ORD\">Make-ord</see> fe fco rst)</v> corresponds to the ordinal <v>(w^fe)fco + rst</v>. \u000A Thus the first infinite ordinal, <v>w</v> (<v>omega</v>), is constructed by</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-ORD\">make-ord</see> 1 1 0)</code> \u000A \u000A <p>and, for example, the ordinal <v>(w^2)5 + w2 + 7</v> is constructed by:</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE-ORD\">make-ord</see> 2 5 (<see topic=\"ACL2____MAKE-ORD\">make-ord</see> 1 2 7)) .</code> \u000A \u000A <p>The reason <v>make-ord</v> is used rather than <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> is that it allows \u000A us to reason more abstractly about the ordinals, without having to worry about \u000A the underlying representation.</p> \u000A \u000A <p><b>Function: </b><srclink>make-ord</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-ord (fe fco rst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____POSP\">posp</see> fco) (<see topic=\"ACL2____O-P\">o-p</see> fe) (<see topic=\"ACL2____O-P\">o-p</see> rst))))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> fe fco) rst))</code>"],

 "ACL2____HONS-EQUAL":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In the logic, <v>hons-equal</v> is just <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>; we leave it enabled \u000A and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, when <v>hons-equal</v> encounters two arguments that are both \u000A normed, it becomes a mere <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> check, and hence avoids the overhead of \u000A recursively checking large cons structures for equality.</p> \u000A \u000A <p>Note.  If <v>hons-equal</v> is given arguments that do not contain many \u000A normed objects, it can actually be much slower than <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>!  This is \u000A because it checks to see whether its arguments are normed at each recursive \u000A step, and so you are repeatedly paying the price of such checks.  Also see \u000A <see topic=\"ACL2____HONS-EQUAL-LITE\">hons-equal-lite</see>, which only checks at the top level whether its \u000A arguments are normed.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-equal (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))</code>"],

 "ACL2____ERROR1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ERROR1\">Error1</see> ctx str alist)</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> t nil state)</v>.  An \u000A error message is first printed using the the ``context'' <v>ctx</v>, as well as \u000A the string <v>str</v> and alist <v>alist</v> that are of the same kind as expected \u000A by <tt><see topic=\"ACL2____FMT\">fmt</see></tt>.  See <see topic=\"ACL2____FMT\">fmt</see>.</p> \u000A \u000A <p><v>Error1</v> can be interpreted as causing an ``error'' when programming \u000A with the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>, something most ACL2 users will probably not want \u000A to do; see <see topic=\"ACL2____LD-ERROR-TRIPLES\">ld-error-triples</see> and see <see topic=\"ACL2____ER-PROGN\">er-progn</see>.  In order to cause \u000A errors with <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt> mode functions, see <see topic=\"ACL2____HARD-ERROR\">hard-error</see> and see \u000A <see topic=\"ACL2____ILLEGAL\">illegal</see>.  Better yet, see <see topic=\"ACL2____ER\">er</see> for a macro that provides a unified \u000A way of signaling errors.</p> \u000A \u000A <p>As mentioned above, <v>error1</v> always returns <v>(<see topic=\"ACL2____MV\">mv</see> t nil state)</v>.  But \u000A if a call <v>(<see topic=\"ACL2____ERROR1\">error1</see> ctx str alist)</v> is encountered during evaluation, then \u000A the string <v>str</v> is first printed using the association list <v>alist</v> (as \u000A in <tt><see topic=\"ACL2____FMT\">fmt</see></tt>).  Here is a trivial, contrived example.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"ACL2____ERROR1\">error1</see> 'my-context\u000A               &quot;Printing 4: ~n0&quot;\u000A               (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\0 4))\u000A               state)\u000A\u000AACL2 Error in MY-CONTEXT:  Printing 4: four\u000A\u000AACL2 !&gt;</code>"],

 "ACL2____ENTER-BOOT-STRAP-MODE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>ACL2 functions, e.g., <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, that show \u000A <v>enter-boot-strap-mode</v> as their defining <see topic=\"ACL2____COMMAND\">command</see> are in fact \u000A primitives.  It is impossible for the system to display defining axioms about \u000A these symbols.</p> \u000A \u000A <p><v>Enter-boot-strap-mode</v> is a Common Lisp function but not an ACL2 \u000A function.  It magically creates from <v>nil</v> an ACL2 property list <see topic=\"ACL2____WORLD\">world</see> that lets us start the boot-strapping process.  That is, once \u000A <v>enter-boot-strap-mode</v> has created its <see topic=\"ACL2____WORLD\">world</see>, it is possible to \u000A process the <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>s, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s, and <tt><see topic=\"ACL2____DEFAXIOM\">defaxiom</see></tt>s, \u000A necessary to bring up the rest of the system.  Before that <see topic=\"ACL2____WORLD\">world</see> is \u000A created, the attempt by ACL2 even to translate a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> form, say, \u000A would produce an error because <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is undefined.</p> \u000A \u000A <p>Several ACL2 functions show <v>enter-boot-strap-mode</v> as their defining \u000A <see topic=\"ACL2____COMMAND\">command</see>.  Among them are <tt><see topic=\"COMMON-LISP____IF\">if</see></tt>, <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>, <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>, and \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.  These functions are characterized by axioms rather than \u000A definitional equations &mdash; axioms that in most cases are built into our \u000A code and hence do not have any explicit representation among the rules and \u000A formulas in the system.</p>"],

 "ACL2____DEFAULT-PRINT-PROMPT":[["Ld"],"ACL2 Sources","ACL2","<code>Example prompt:\u000AACL2 p!s&gt;</code> \u000A \u000A <p>The <see topic=\"ACL2____PROMPT\">prompt</see> printed by ACL2 displays the current package, followed by \u000A a space, followed by zero or more of the three <see topic=\"ACL2____CHARACTERS\">characters</see> as specified \u000A below, followed by the character <tt><see topic=\"COMMON-LISP_____E3\">&gt;</see></tt> printed one or more times, \u000A reflecting the number of recursive calls of <tt><see topic=\"ACL2____LD\">ld</see></tt>.  The three <see topic=\"ACL2____CHARACTERS\">characters</see> in the middle are as follows:</p> \u000A \u000A <code>p     ; when (<see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> (w state)) is :program\u000A!     ; when guard checking is on\u000As     ; when (<see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see> state) is t</code> \u000A \u000A <p>See <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see>, see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>, and see <see topic=\"ACL2____LD-SKIP-PROOFSP\">ld-skip-proofsp</see>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____LD-PROMPT\">ld-prompt</see> to see how to install your own <see topic=\"ACL2____PROMPT\">prompt</see>.</p> \u000A \u000A <p>Here are some examples with <v>ld-skip-proofsp nil</v>.</p> \u000A \u000A <code>ACL2 !&gt;    ; logic mode with guard checking on\u000AACL2 &gt;     ; logic mode with guard checking off\u000AACL2 p!&gt;   ; program mode with guard checking on\u000AACL2 p&gt;    ; program mode with guard checking off</code> \u000A \u000A <p>Here are some examples with <tt><see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see></tt> of <v>:</v><tt><see topic=\"ACL2____LOGIC\">logic</see></tt>.</p> \u000A \u000A <code>ACL2 &gt;     ; guard checking off, ld-skip-proofsp nil\u000AACL2 s&gt;    ; guard checking off, ld-skip-proofsp t\u000AACL2 !&gt;    ; guard checking on, ld-skip-proofsp nil\u000AACL2 !s&gt;   ; guard checking on, ld-skip-proofsp t</code> \u000A \u000A <p>Finally, here is the prompt in raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>), \u000A regardless of the settings above:</p> \u000A \u000A <code>ACL2 P&gt;</code>"],

 "ACL2____BROKEN-LINK":[["Documentation"],"ACL2 Sources","ACL2","<p>You are looking at the ACL2 User's Manual, which contains <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> from the ACL2 sources but not from the ACL2 community <see topic=\"ACL2____BOOKS\">books</see>.  Perhaps you came to this page by following a link to a topic that is \u000A documented in a book, as shown in the table below.  If so, then you may wish \u000A to go to a manual that includes documentation from the books.  If you are \u000A using the ACL2-Doc Emacs browser, then see <see topic=\"ACL2____ACL2-DOC\">ACL2-Doc</see>, in particular the \u000A documentation of the ``<v>I</v>'' command for switching to the \u000A <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a>.</p> \u000A \u000A <p>We include the following table in case you find it to be helpful.  It maps \u000A each topic name shown to the book in which it is documented.</p> \u000A \u000A <p><code>((&lt;&lt; &quot;[books]/misc/total-order.lisp&quot;)\u000A (oslib::argv &quot;[books]/oslib/logic-defs.lisp&quot;)\u000A (arith-equivs &quot;[books]/centaur/misc/arith-equiv-defs.lisp&quot;)\u000A (arithmetic &quot;[books]/doc/top.lisp&quot;)\u000A (arithmetic-1 &quot;[books]/arithmetic/top.lisp&quot;)\u000A (arithmetic/natp-posp &quot;[books]/arithmetic/natp-posp.lisp&quot;)\u000A (b* &quot;[books]/tools/bstar.lisp&quot;)\u000A (bridge &quot;[books]/centaur/bridge/top.lisp&quot;)\u000A (cert.pl &quot;[books]/build/doc.lisp&quot;)\u000A (data-definitions &quot;[books]/cgen/defdata.lisp&quot;)\u000A (std::defaggregate &quot;[books]/std/util/defaggregate.lisp&quot;)\u000A (defconsts &quot;[books]/std/util/defconsts.lisp&quot;)\u000A (defdata &quot;[books]/cgen/defdata.lisp&quot;)\u000A (define &quot;[books]/std/util/define.lisp&quot;)\u000A (fty::defprod &quot;[books]/centaur/fty/deftypes.lisp&quot;)\u000A (fty::fty &quot;[books]/centaur/fty/fixtype.lisp&quot;)\u000A (gl &quot;[books]/centaur/gl/doc.lisp&quot;)\u000A (hacker &quot;[books]/hacking/doc-section.lisp&quot;)\u000A (ihs &quot;[books]/ihs/ihs-doc-topic.lisp&quot;)\u000A (include-raw &quot;[books]/tools/include-raw.lisp&quot;)\u000A (make-flag &quot;[books]/tools/flag.lisp&quot;)\u000A (non-parallel-book &quot;[books]/std/system/non-parallel-book.lisp&quot;)\u000A (note-6-4-books &quot;[books]/doc/relnotes.lisp&quot;)\u000A (note-6-5-books &quot;[books]/doc/relnotes.lisp&quot;)\u000A (str::numbers &quot;[books]/std/strings/top.lisp&quot;)\u000A (oslib &quot;[books]/oslib/top.lisp&quot;)\u000A (quicklisp &quot;[books]/centaur/quicklisp/top.lisp&quot;)\u000A (satlink::sat-solver-options &quot;[books]/centaur/satlink/top.lisp&quot;)\u000A (satlink &quot;[books]/centaur/satlink/top.lisp&quot;)\u000A (xdoc::save &quot;[books]/xdoc/topics.lisp&quot;)\u000A (set-max-mem &quot;[books]/centaur/misc/memory-mgmt-logic.lisp&quot;)\u000A (std &quot;[books]/std/top.lisp&quot;)\u000A (std/io &quot;[books]/std/io/top.lisp&quot;)\u000A (std/util &quot;[books]/std/util/top.lisp&quot;)\u000A (unsound-read &quot;[books]/std/io/unsound-read.lisp&quot;)\u000A (untranslate-patterns &quot;[books]/misc/untranslate-patterns.lisp&quot;)\u000A (with-raw-mode &quot;[books]/hacking/hacker.lisp&quot;)\u000A (with-redef-allowed &quot;[books]/hacking/hacker.lisp&quot;)\u000A (xdoc &quot;[books]/xdoc/topics.lisp&quot;))</code></p>"],

 "ACL2____Analyzing_02Common_02Lisp_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>To analyze a model you must be able to reason about the operations \u000A and relations involved.  Perhaps, for example, some aspect of the model \u000A depends upon the fact that the concatenation operation is associative.</p> \u000A \u000A <p>In any Common Lisp you can confirm that</p> \u000A \u000A <code>(app '(A B) (app '(C D) '(E F)))</code> \u000A \u000A <p>and</p> \u000A \u000A <code>(app (app '(A B) '(C D)) '(E F)))</code> \u000A \u000A <p>both evaluate to the same thing, <v>(A B C D E F)</v>.</p> \u000A \u000A <p>But what distinguishes ACL2 (the logic) from applicative Common Lisp (the \u000A language) is that in ACL2 you can <b>prove</b> that the concatenation function \u000A <v>app</v> is associative when its arguments are true-lists, whereas in Common \u000A Lisp all you can do is test that proposition.</p> \u000A \u000A <p>That is, in ACL2 it makes sense to say that the following formula is a \u000A ``theorem.''</p> \u000A \u000A <code><b>Theorem</b> Associativity of App \u000A(implies (and (true-listp a) \u000A              (true-listp b)) \u000A         (equal (app (app a b) c) \u000A                (app a (app b c)))) \u000A</code> \u000A \u000A <p>Theorems about the properties of models are proved by symbolically \u000A manipulating the operations and relations involved.  If the concatenation of \u000A sequences is involved in your model, then you may well need the theorem above \u000A in order to that your model has some particular property.</p>"],

 "ACL2____About_02the_02ACL2_02Home_02Page":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____What_02Is_02ACL2_82Q_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>The ACL2 Home Page is integrated into the ACL2 online documentation.  Over \u000A 4 megabytes of hypertext is available here.</p> \u000A \u000A <p>The vast majority of the text is user-level documentation.  For example, to \u000A find out about <see topic=\"ACL2____REWRITE\">rewrite</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rules you could click on the link.  (If you do that, remember to use your \u000A browser's <b>Back Button</b> to come back here.)</p> \u000A \u000A <p>The tiny warning signs <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> mark links that lead out of the introductory-level \u000A material and into the user documentation.  We advise against following such \u000A links upon your first reading of the documentation.</p> \u000A \u000A <p>At the end of the tours you will have a chance to revisit them quickly to \u000A explore alternative paths more fully.</p> \u000A \u000A <p>Finally, every page contains two icons at the bottom.  The ACL2 icon leads \u000A you back to the ACL2 Home Page.  The Index icon allows you to browse an \u000A alphabetical listing of all the topics in ACL2's online documentation.  But \u000A both icons take you off the main route of the tour.</p> \u000A \u000A <p><see topic='ACL2____What_02Is_02ACL2_82Q_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____A_02Walking_02Tour_02of_02ACL2":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Common_02Lisp'><img \u000A src='large-walking.gif'></img></see></p> \u000A \u000A <p>On this tour you will learn a little more about the ACL2 logic, the theorem \u000A prover, and the user interface.</p> \u000A \u000A <p>This time we will stick with really simple things, such as the \u000A associativity of list concatenation.</p> \u000A \u000A <p>We assume you have taken the Flying Tour but that you did not necessarily \u000A follow all the ``off-tour'' links because we encouraged you not to.  With the \u000A Walking Tour we encourage you to visit off-tour links &mdash; provided they \u000A are not marked with the tiny warning sign (<see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>). \u000A But they are ``branches'' in the tour that lead to ``dead ends.''  When you \u000A reach a dead end, remember to use your browser's Back Button to return to the \u000A Walking Tour to continue.</p> \u000A \u000A <p>When you get to the end of the tour we'll give you a chance to repeat \u000A quickly both the Flying and the Walking Tours to visit any off-tour links \u000A still of interest.</p> \u000A \u000A <p><see topic='ACL2____Common_02Lisp'><img src='walking.gif'></img></see></p>"],

 "ACL2____ADD-RAW-ARITY":[["Set-raw-mode"],"ACL2 Sources","ACL2","<p>Technical note: This macro is a no-op, and is not necessary, when \u000A ACL2 is built with #-acl2-mv-as-values.</p> \u000A \u000A <p>Users of raw mode (see <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see>) can use arbitrary raw Lisp \u000A functions that are not known inside the usual ACL2 loop.  In such cases, ACL2 \u000A may not know how to display a multiple value returned by ACL2's <tt><see topic=\"ACL2____MV\">mv</see></tt> \u000A macro.  The following example should make this clear.</p> \u000A \u000A <code>ACL2 P&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> foo (x y) (<see topic=\"ACL2____MV\">mv</see> y x))\u000AFOO\u000AACL2 P&gt;(foo 3 4)\u000A\u000ANote: Unable to compute number of values returned by this evaluation\u000Abecause function FOO is not known in the ACL2 logical world.  Presumably\u000Ait was defined in raw Lisp or in raw mode.  Returning the first (perhaps\u000Aonly) value for calls of FOO.\u000A4\u000AACL2 P&gt;(<see topic=\"ACL2____ADD-RAW-ARITY\">add-raw-arity</see> foo 2)\u000A RAW-ARITY-ALIST\u000AACL2 P&gt;(foo 3 4)\u000A(4 3)\u000AACL2 P&gt;</code> \u000A \u000A <p>The first argument of <v>add-raw-arity</v> should be a symbol, representing \u000A the name of a function, macro, or special form, and the second argument should \u000A either be a non-negative integer (denoting the number of values returned by \u000A ACL2) or else the symbol <v>:LAST</v>, meaning that the number of values \u000A returned by the call is the number of values returned by the last \u000A argument.</p> \u000A \u000A <p>The current arity assignments can be seen by evaluating <v>(<see topic=\"ACL2_____04\">@</see>\u000A raw-arity-alist)</v>.  See <see topic=\"ACL2____REMOVE-RAW-ARITY\">remove-raw-arity</see> for how to undo a call of \u000A <v>add-raw-arity</v>.</p>"],

 "ACL2____ADD-OVERRIDE-HINTS":[["Override-hints"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for a discussion of override-hints.  Here \u000A we describe how to extend the list of override-hints.  Note that the effects \u000A of <v>add-override-hints</v> <see topic=\"ACL2____EVENTS\">events</see> are <see topic=\"ACL2____LOCAL\">local</see> to the <see topic=\"ACL2____BOOKS\">books</see> \u000A or <v>encapsulate</v> <see topic=\"ACL2____EVENTS\">events</see> in which they reside; see <see topic=\"ACL2____ADD-OVERRIDE-HINTS_12\">add-override-hints!</see> to avoid that restriction.  Also see <see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see> to set a new list of override-hints to it, ignoring the \u000A present list rather than adding to it.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> form)\u000A(<see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> form :at-end t)\u000A(<see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> form :at-end nil) ; default for :at-end</code> \u000A \u000A <p>where <v>form</v> evaluates to a list of computed hint forms.  The effect of \u000A this event is to extend the current list of <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> by appending \u000A the result of that evaluation.  The default is to append the evaluation result \u000A to the front of the current list of override-hints, but if <v>:at-end t</v> is \u000A specified, then the evaluation result is appended to the end of the current \u000A list.</p>"],

 "ACL2-PC____GENEQV":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>General Forms:\u000Ageneqv     ; show list of equivalence relations being maintained\u000A(geneqv t) ; as above, but pair each relation with a justifying rune</code> \u000A \u000A <p>This is an advanced command, whose effect is to print the so-called \u000A ``generated equivalence relation'' (or ``geneqv'') that is maintained at the \u000A current subterm of the conclusion.  That structure is a list of equivalence \u000A relations, representing the transitive closure <v>E</v> of the union of those \u000A relations, such that it suffices to maintain <v>E</v> at the current subterm: if \u000A that subterm, <v>u</v>, is replaced in the goal's conclusion, <v>G</v>, by another \u000A term equivalent to <v>u</v> with respect to <v>E</v>, then the resulting \u000A conclusion is Boolean equivalent to <v>G</v>.  Also see <see topic=\"ACL2____DEFCONG\">defcong</see>.</p> \u000A \u000A <p>The command `<v>geneqv</v>' prints the above list of equivalence relations, \u000A or more precisely, the list of function symbols for those relations.  If \u000A however <v>geneqv</v> is given a non-<v>nil</v> argument, then a list is printed \u000A whose elements are each of the form <v>(s r)</v>, where <v>s</v> is the symbol for \u000A an equivalence relation and <v>r</v> is a <v>:</v><tt><see topic=\"ACL2____CONGRUENCE\">congruence</see></tt> <see topic=\"ACL2____RUNE\">rune</see> \u000A justifying the inclusion of <v>s</v> in the list of equivalence relations being \u000A maintained at the current subterm.</p>"],

 "COMMON-LISP_____F2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BINARY-_A2\">binary-*</see> for multiplication and see <see topic=\"ACL2____UNARY-_F2\">unary-/</see> for \u000A reciprocal.</p> \u000A \u000A <p>Note that <v>/</v> represents division as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____F2\">/</see> x y)</code> \u000A \u000A <p>represents the same term as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____A2\">*</see> x (<see topic=\"COMMON-LISP_____F2\">/</see> y))</code> \u000A \u000A <p>which is really</p> \u000A \u000A <code>(<see topic=\"ACL2____BINARY-_A2\">binary-*</see> x (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> y)).</code> \u000A \u000A <p>Also note that <v>/</v> represents reciprocal as follows:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____F2\">/</see> x)</code> \u000A \u000A <p>expands to</p> \u000A \u000A <code>(<see topic=\"ACL2____UNARY-_F2\">unary-/</see> x).</code> \u000A \u000A <p><v>/</v> is a Common Lisp macro.  See any Common Lisp documentation for more \u000A information.</p> \u000A \u000A <p><b>Macro: </b><srclink>/</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> / (x &amp;optional (y 'nil binary-casep))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> (binary-casep (<see topic=\"COMMON-LISP____LIST\">list</see> 'binary-* x (<see topic=\"COMMON-LISP____LIST\">list</see> 'unary-/ y)))\u000A                (t (<see topic=\"COMMON-LISP____LIST\">list</see> 'unary-/ x))))</code>"],

 "COMMON-LISP____QUOTE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The form <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see> x)</v> evaluates to <v>x</v>.  See any Common Lisp \u000A documentation.</p>"],

 "ACL2____TERM-TABLE":[["Meta"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TABLE\">table</see> term-table t '((<see topic=\"ACL2____BINARY-_B2\">binary-+</see> x y) '3 'nil (<see topic=\"COMMON-LISP____CAR\">car</see> x)))</code> \u000A \u000A <p>See <see topic=\"ACL2____TABLE\">table</see> for a general discussion of tables and the <v>table</v> \u000A event used to manipulate tables.</p> \u000A \u000A <p>The ``<v>term-table</v>'' is used at the time a meta rule is checked for \u000A syntactic correctness.  Each proposed metafunction is run on each term in this \u000A table, and the result in each case is checked to make sure that it is a \u000A <v>termp</v> in the current world.  In each case where this test fails, a \u000A warning is printed.</p> \u000A \u000A <p>Whenever a metafunction is run in support of the application of a meta \u000A rule, the result must be a term in the current world.  When the result is not \u000A a term, a hard error arises.  The <v>term-table</v> is simply a means for \u000A providing feedback to the user at the time a meta rule is submitted, warning \u000A of the definite possibility that such a hard error will occur at some point in \u000A the future.</p> \u000A \u000A <p>The key used in <v>term-table</v> is arbitrary.  The top-most value is always \u000A the one that is used; it is the entire list of terms to be considered.  Each \u000A must be a <v>termp</v> in the current ACL2 world.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INSTANCE":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Let <i>p</i> and <i>q</i> be terms or formulas (there is no \u000A difference in ACL2).  Then we say <i>p</i> is an <i>instance</i> or \u000A <i>substitution instance</i> of <i>q</i> if and only if <i>p</i> can be \u000A obtained from <i>q</i> by uniformly replacing the variables of <i>q</i> by \u000A terms.  Sometimes we call <i>p</i> the <i>target</i> and <i>q</i> the \u000A <i>pattern</i> because by choosing appropriate replacements we can make the \u000A pattern <i>match</i> many different targets.</p> \u000A \u000A <p>For example, the following <i>target</i> is an instance of the given \u000A <i>pattern</i>:</p> \u000A \u000A <code><i>target</i>:      (APP (APP (REV A) (REV B)) (REV C)) \u000A<i>pattern</i>:     (APP (APP   x       y    ) (REV z)) \u000A</code> \u000A \u000A <p>The replacement or <i>substitution</i> used in this match of the pattern to \u000A the target is:</p> \u000A \u000A <code><i>variable in pattern</i>          <i>replacement term</i> \u000Ax                              (REV A) \u000Ay                              (REV B) \u000Az                              C \u000A</code> \u000A \u000A <p>Such substitutions are usually written this way in ACL2:</p> \u000A \u000A <code>((x  (REV A))\u000A (y  (REV B))\u000A (z  C)).</code> \u000A \u000A <p>Please use your browser's <b>Back Button</b> to return to the page that \u000A mentioned ``instance.''</p>"],

 "ACL2____HONS-EQUAL-LITE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In the logic, <v>hons-equal-lite</v> is just <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>; we leave it \u000A enabled and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons-equal-lite</v> checks whether its arguments are \u000A normed, and if so it effectively becomes a <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> check.  Otherwise, it \u000A immediately calls <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> to determine if its arguments are equal.</p> \u000A \u000A <p>The idea here is to strike a useful balance between <v>equal</v> and <tt><see topic=\"ACL2____HONS-EQUAL\">hons-equal</see></tt>.  If both arguments happen to be normed, we get to use a very fast \u000A equality comparison.  Otherwise, we just get out of the way and let <v>equal</v> \u000A do its work, without the extra overhead of recursively checking whether the \u000A subtrees of x and y are normed.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-equal-lite</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-equal-lite (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y))</code>"],

 "COMMON-LISP____ENDP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>In the ACL2 logic, <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> x)</v> is the same as <v>(<see topic=\"COMMON-LISP____ATOM\">atom</see> x)</v>.  See \u000A <see topic=\"COMMON-LISP____ATOM\">atom</see>.</p> \u000A \u000A <p>Unlike <tt><see topic=\"COMMON-LISP____ATOM\">atom</see></tt>, the <see topic=\"ACL2____GUARD\">guard</see> for <v>endp</v> requires that <v>x</v> is \u000A a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair or is <v>nil</v>.  Thus, <v>endp</v> is typically used as a \u000A termination test for functions that recur on a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> argument. \u000A See <see topic=\"ACL2____GUARD\">guard</see> for general information about <see topic=\"ACL2____GUARD\">guard</see>s.</p> \u000A \u000A <p><v>Endp</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>endp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> endp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))))\u000A       (<see topic=\"COMMON-LISP____ATOM\">atom</see> x))</code>"],

 "ACL2____VERIFY":[["Proof-checker"],"ACL2 Sources","ACL2","<p>For proof-checker command summaries, see <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____VERIFY\">VERIFY</see> (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))\u000A                      (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A                             (<see topic=\"COMMON-LISP____APPEND\">append</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z)))))\u000A   -- Attempt to prove the given term interactively.\u000A\u000A(<see topic=\"ACL2____VERIFY\">VERIFY</see> (p x)\u000A        :event-name p-always-holds\u000A        :rule-classes (:rewrite :generalize)\u000A        :instructions ((<see topic=\"ACL2____REWRITE\">rewrite</see> p-always-holds-lemma)\u000A                       change-goal))\u000A   -- Attempt to prove (p x), where the intention is to call the\u000A      resulting DEFTHM event by the name p-always-holds, with\u000A      rule-classes as indicated.  The two indicated instructions\u000A      will be run immediately to start the proof.\u000A\u000A(<see topic=\"ACL2____VERIFY\">VERIFY</see>)\u000A   -- Re-enter the proof-checker in the state at which is was last\u000A      left.\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____VERIFY\">VERIFY</see> &amp;OPTIONAL raw-term\u000A        &amp;KEY\u000A        event-name\u000A        rule-classes\u000A        instructions)</code> \u000A \u000A <p><v>Verify</v> is the function used for entering the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s \u000A interactive loop.</p>"],

 "ACL2____STANDARDP":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STANDARDP\">Standardp</see> x)</v> is true if and only if <v>x</v> is a ``standard'' \u000A object.  This notion of ``standard'' comes from non-standard analysis and is \u000A discussed in Ruben Gamboa's dissertation.  In brief, all the familiar objects \u000A are standard: e.g., the familiar real numbers are standard, but non-zero \u000A infinitesimals are not standard, and the familiar integers are standard, but \u000A not those that exceed every integer that you can express in the usual way (1, \u000A 2, 3, and so on).  Similarly, the familiar lists are standard, but not so a \u000A list that contains a large number of integers, where ``large'' means more than \u000A the standard integers.  The set of standard numbers is closed under the usual \u000A arithmetic operations, hence the sum of a standard number and a non-zero \u000A infinitesimal is not standard, though it is what is called ``limited'' (see \u000A <see topic=\"ACL2____I-LIMITED\">i-limited</see>).</p> \u000A \u000A <p>This predicate is only defined in ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____SET-CBD":[["Books-reference"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;:set-cbd &quot;/usr/home/smith/&quot;\u000AACL2 !&gt;:set-cbd &quot;my-acl2/books&quot;</code> \u000A \u000A <p>See <see topic=\"ACL2____CBD\">cbd</see> for a description of the connected book directory.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-CBD\">set-cbd</see> str)</code> \u000A \u000A <p>where <v>str</v> is a nonempty string that represents the desired directory \u000A (see <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>).  This command sets the connected book directory (see \u000A <see topic=\"ACL2____CBD\">cbd</see>) to the string representing the indicated directory.  Thus, this \u000A command may determine which files are processed by <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt> and \u000A <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> <see topic=\"ACL2____COMMAND\">command</see>s typed at the top-level.  However, the \u000A <tt><see topic=\"ACL2____CBD\">cbd</see></tt> is also temporarily set by those two book processing <see topic=\"ACL2____COMMAND\">command</see>s.</p> \u000A \u000A <p>IMPORTANT: Pathnames in ACL2 are in the Unix (trademark of AT&amp;T) style. \u000A That is, the character ``<v>/</v>'' separates directory components of a \u000A pathname, and pathnames are absolute when they start with this character, and \u000A relative otherwise.  See <see topic=\"COMMON-LISP____PATHNAME\">pathname</see>.</p>"],

 "ACL2____RESET-LD-SPECIALS":[["Ld"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____RESET-LD-SPECIALS\">reset-ld-specials</see> t)\u000A(<see topic=\"ACL2____RESET-LD-SPECIALS\">reset-ld-specials</see> nil)</code> \u000A \u000A <p>Roughly speaking, the <tt><see topic=\"ACL2____LD\">ld</see></tt> specials are certain <see topic=\"ACL2____STATE\">state</see> global \u000A variables, such as <tt><see topic=\"ACL2____CURRENT-PACKAGE\">current-package</see></tt>, <tt><see topic=\"ACL2____LD-PROMPT\">ld-prompt</see></tt>, and <tt><see topic=\"ACL2____LD-PRE-EVAL-FILTER\">ld-pre-eval-filter</see></tt>, which are managed by <tt><see topic=\"ACL2____LD\">ld</see></tt> as though they were local \u000A variables.  These variables determine the channels on which <tt><see topic=\"ACL2____LD\">ld</see></tt> reads \u000A and prints and control many options of <tt><see topic=\"ACL2____LD\">ld</see></tt>.  See <see topic=\"ACL2____LD\">ld</see> for the \u000A details on what the <tt><see topic=\"ACL2____LD\">ld</see></tt> specials are.</p> \u000A \u000A <p>This function, <v>reset-ld-specials</v>, takes one Boolean argument, \u000A <v>flg</v>.  The function resets all of the <tt><see topic=\"ACL2____LD\">ld</see></tt> specials to their \u000A initial, top-level values, except for the three channel variables, <tt><see topic=\"ACL2____STANDARD-OI\">standard-oi</see></tt>, <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>, and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt>, which are reset to \u000A their initial values only if <v>flg</v> is non-<v>nil</v>.  Of course, if you are \u000A in a recursive call of <tt><see topic=\"ACL2____LD\">ld</see></tt>, then when you exit that call, the <tt><see topic=\"ACL2____LD\">ld</see></tt> specials will be restored to the values they had at the time <tt><see topic=\"ACL2____LD\">ld</see></tt> \u000A was called recursively.  To see what the initial values are, inspect the value \u000A of the constant <v>*initial-ld-special-bindings*</v>.</p>"],

 "ACL2____RESET-KILL-RING":[["History"],"ACL2 Sources","ACL2","<p>By default, ACL2 holds on to old logical <see topic=\"ACL2____WORLD\">world</see>s when you \u000A undo <see topic=\"ACL2____COMMAND\">command</see>s (see <see topic=\"ACL2____UBT\">ubt</see>), as documented elswhere; see <see topic=\"ACL2____OOPS\">oops</see>.  You can free up memory by deleting those old worlds using \u000A <v>reset-kill-ring</v>.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> t state)   ; replace each element of the kill ring by nil\u000A(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> 2 state)   ; create a new kill ring of '(nil nil)\u000A(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> 0 state)   ; create a new kill ring that is empty\u000A(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> nil state) ; just return the length of the kill ring\u000A\u000AGeneral form:\u000A(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> n state)</code> \u000A \u000A <p>where <v>n</v> evaluates either to <v>t</v>, to <v>nil</v>, or to a nonnegative \u000A integer (a <tt><see topic=\"ACL2____NATP\">natp</see></tt>).  If <v>n</v> evaluates to <v>t</v>, it is treated as the \u000A length of the current kill ring.  If <v>n</v> is <v>nil</v>, then the length \u000A <v>k</v> of the current kill ring is returned as a value triple <v>(<see topic=\"ACL2____MV\">mv</see> nil k\u000A state)</v>.  If <v>n</v> is a <tt><see topic=\"ACL2____NATP\">natp</see></tt>, then the kill ring is replaced with a \u000A list of <v>n</v> <v>nil</v>s.</p> \u000A \u000A <p>In particular, use <v>(<see topic=\"ACL2____RESET-KILL-RING\">reset-kill-ring</see> 0 state)</v> to avoid saving any old \u000A logical <see topic=\"ACL2____WORLD\">world</see>s, at the cost of disabling the effect of the <tt><see topic=\"ACL2____OOPS\">oops</see></tt> \u000A <see topic=\"ACL2____COMMAND\">command</see>.</p>"],

 "ACL2____PSO":[["Other"],"ACL2 Sources","ACL2","<p>Evaluate <v>:pso</v> in order to print output that was generated in \u000A an environment where output was being saved; see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> for \u000A details.  However, <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output will be suppressed; use \u000A <v>:</v><tt><see topic=\"ACL2____PSO_12\">pso!</see></tt> if you want that output to be printed as well.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____PSOG\">psog</see>, for printing saved output in <see topic=\"ACL2____GAG-MODE\">gag-mode</see>.</p>"],

 "ACL2____MOD-EXPT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____MOD-EXPT\">mod-expt</see> r i m)</v> is the result of raising the number <v>r</v> to \u000A the integer power <v>i</v> and then taking the residue mod <v>m</v>.  That is, \u000A <v>(<see topic=\"ACL2____MOD-EXPT\">mod-expt</see> r i m)</v> is equal to <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> r i) m)</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"ACL2____MOD-EXPT\">mod-expt</see> r i m)</v> is that <v>r</v> is a rational \u000A number and <v>i</v> is an integer; if <v>r</v> is <v>0</v> then <v>i</v> is \u000A nonnegative; and <v>m</v> is a non-zero rational number.</p> \u000A \u000A <p>In some implementations (GCL Version 2.7.0 as of this writing), this \u000A function is highly optimized when <v>r</v> and <v>i</v> are natural numbers, not \u000A both zero, and <v>m</v> is a positive integer.  For other Lisp implementations, \u000A consider using function <v>mod-expt-fast</v>, defined in the community book \u000A <v>arithmetic-3/floor-mod/mod-expt-fast.lisp</v>, which should still provide \u000A significantly improved performance over this function.</p> \u000A \u000A <p><b>Function: </b><srclink>mod-expt</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     mod-expt (base exp mod)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> base)\u000A                                 (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> exp)\u000A                                 (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> base 0) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> exp 0)))\u000A                                 (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> mod)\u000A                                 (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> mod 0)))))\u000A     (<see topic=\"COMMON-LISP____MOD\">mod</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> base exp) mod))</code>"],

 "ACL2____FAST-ALIST-FREE-ON-EXIT":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>Logically, <v>(<see topic=\"ACL2____FAST-ALIST-FREE-ON-EXIT\">fast-alist-free-on-exit</see> alist form)</v> is the \u000A identity and returns <v>form</v>.  Also see <see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see>.</p> \u000A \u000A <p>Under the hood, this essentially expands to:</p> \u000A \u000A <code>(prog1 form\u000A       (<see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see> alist))</code> \u000A \u000A <p>In other words, <v>alist</v> is not freed immediately, but instead remains a \u000A fast alist until the form completes.  This may be useful when you are writing \u000A code that uses a fast alist but has many return points.</p> \u000A \u000A <p>See also the macro <v>fast-alists-free-on-exit</v> defined in the community \u000A book <v>&quot;books/centaur/misc/hons-extra.lisp&quot;</v>, which can be used to free \u000A several alists.</p> \u000A \u000A <p>The community book <v>&quot;books/centaur/misc/hons-extra.lisp&quot;</v> extends the \u000A <see topic=\"ACL2____B_A2\">b*</see> macro with the <v>free-on-exit</v> pattern binder.  That is, after \u000A executing <v>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;centaur/misc/hons-extra.lisp&quot; :dir :system)</v>, \u000A the form</p> \u000A \u000A <code>(b* (...\u000A     ((free-on-exit a b c))\u000A     ...)\u000A  ...)</code> \u000A \u000A <p>causes <v>a</v>, <v>b</v>, and <v>c</v> to be freed when the <v>b*</v> completes, \u000A but they remain fast alists until then.</p>"],

 "ACL2____DOCS":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>NOTE: The <v>:docs</v> command only makes sense at the terminal.</p> \u000A \u000A <p>When the <v>:docs</v> command is given a <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> argument it searches \u000A the text produced by <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> and <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt> and lists \u000A all the documented topics whose text contains the given string.  For purposes \u000A of this string matching we ignore distinctions of case and the amount and kind \u000A (but not presence) of white space.  We also treat hyphen as whitespace.</p> \u000A \u000A <p>However, the following examples show how <v>:docs</v> can be used on other \u000A than string patterns.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;:docs *           ; lists documentation sections\u000AACL2 !&gt;:docs **          ; lists all documented topics within all sections\u000AACL2 !&gt;:docs events      ; lists all topics in section EVENTS\u000AACL2 !&gt;:docs &quot;compil&quot;    ; lists topics ``apropos''</code> \u000A \u000A <p>The database of formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings is structured into \u000A sections.  Within a section are topics.  Each topic has a one-liner, some \u000A notes, and some detailed discussions.  The <v>:docs</v> command provides a view \u000A of the entire database.</p> \u000A \u000A <p><v>:docs</v> takes one argument, as described below:</p> \u000A \u000A <code>arg               effect\u000A\u000A*                 list all section headings in the database\u000A**                list all section headings and all topics within\u000A                  each section\u000Aname              list all topics in the section named name (where\u000A                  name is some symbol other than * and **).  This\u000A                  is always the same as :doc name.\u000Apattern           list all topics whose :doc or :more-doc text\u000A                  mentions the string pattern.  For purposes of this\u000A                  string matching we ignore distinctions of case and\u000A                  the amount and kind (but not presence) of white\u000A                  space.  We also treat hyphen as whitespace.</code>"],

 "ACL2____DISABLE-FORCING":[["Force"],"ACL2 Sources","ACL2","<code>General Form:\u000AACL2 !&gt;:disable-forcing   ; disallow forced case splits</code> \u000A \u000A <p>See <see topic=\"ACL2____FORCE\">force</see> and see <see topic=\"ACL2____CASE-SPLIT\">case-split</see> for a discussion of forced case \u000A splits, which are inhibited by this command.</p> \u000A \u000A <p><v>Disable-forcing</v> is actually a macro that <see topic=\"ACL2____DISABLE\">disable</see>s the \u000A executable counterpart of the function symbol <v>force</v>; see <see topic=\"ACL2____FORCE\">force</see>. \u000A When you want to use <see topic=\"ACL2____HINTS\">hints</see> to turn off forced case splits, use a form \u000A such as one of the following (these are equivalent).</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart force))\u000A:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____FORCE\">force</see>))</code> \u000A \u000A <p>The following example shows how this works.  First evaluate these \u000A forms.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFSTUB\">defstub</see> f1 (x) t)\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> f2 (x) t)\u000A(<see topic=\"ACL2____DEFAXIOM\">defaxiom</see> ax (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____CASE-SPLIT\">case-split</see> (f2 x)) (f1 x)))\u000A(<see topic=\"ACL2____THM\">thm</see> (f1 x))</code> \u000A \u000A <p>You will see the application of the rule, <v>ax</v>, in the proof of the \u000A <tt><see topic=\"ACL2____THM\">thm</see></tt> call above.  However, if you first evaluate \u000A <v>(<see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see>)</v>, then there will be no application of <v>ax</v>.  To \u000A restore forced case splitting, see <see topic=\"ACL2____ENABLE-FORCING\">enable-forcing</see>.</p>"],

 "ACL2____BOOKS-REFERENCE":[["Books"],"ACL2 Sources","ACL2"," \u000A \u000A"],

 "COMMON-LISP____STANDARD-CHAR-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">Standard-char-p</see> x)</v> is true if and only if <v>x</v> is a \u000A ``standard'' character, i.e., a member of the list <v>*standard-chars*</v>. \u000A This list includes <v>#\\Newline</v> and <v>#\\Space</v> <see topic=\"ACL2____CHARACTERS\">characters</see>, as \u000A well as the usual punctuation and alphanumeric <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><v>Standard-char-p</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring its argument to be a \u000A character.</p> \u000A \u000A <p><v>Standard-char-p</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>standard-char-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> standard-char-p (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> x *standard-chars*) t nil))</code>"],

 "COMMON-LISP____EXPT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____EXPT\">Expt</see> r i)</v> is the result of raising the number <v>r</v> to the \u000A integer power <v>i</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____EXPT\">expt</see> r i)</v> is that <v>r</v> is a number and <v>i</v> \u000A is an integer, and furthermore, if <v>r</v> is <v>0</v> then <v>i</v> is \u000A nonnegative.  When the type requirements of the <see topic=\"ACL2____GUARD\">guard</see> aren't met, \u000A <v>(<see topic=\"COMMON-LISP____EXPT\">expt</see> r i)</v> first coerces <v>r</v> to a number and <v>i</v> to an integer.</p> \u000A \u000A <p><v>Expt</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.  Note that <v>r</v> can be a complex number; this is \u000A consistent with Common lisp.</p> \u000A \u000A <p><b>Function: </b><srclink>expt</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> expt (r i)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> r)\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> r 0) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i 0))))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZIP\">zip</see> i) 1)\u000A             ((<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"ACL2____FIX\">fix</see> r) 0) 0)\u000A             ((<see topic=\"COMMON-LISP_____E3\">&gt;</see> i 0) (<see topic=\"COMMON-LISP_____A2\">*</see> r (<see topic=\"COMMON-LISP____EXPT\">expt</see> r (<see topic=\"COMMON-LISP_____B2\">+</see> i -1))))\u000A             (t (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____F2\">/</see> r) (<see topic=\"COMMON-LISP____EXPT\">expt</see> r (<see topic=\"COMMON-LISP_____B2\">+</see> i 1))))))</code>"],

 "ACL2____VERIFY-GUARDS-FORMULA":[["Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see> and see <see topic=\"ACL2____GUARD\">guard</see> for a discussion of \u000A guards.  This utility is provided for viewing a guard proof obligation, \u000A without doing a proof.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> foo)\u000A(<see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> foo :guard-debug t)\u000A(<see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> foo :otf-flg dont-care :xyz whatever)\u000A(<see topic=\"ACL2____VERIFY-GUARDS-FORMULA\">verify-guards-formula</see> (<see topic=\"COMMON-LISP_____B2\">+</see> (foo x) (bar y)) :guard-debug t)</code> \u000A \u000A <p><v>Verify-guards-formula</v> allows all keywords, but only pays attention to \u000A <v>:guard-debug</v>, which has the same effect as in <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt> (see \u000A <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see>).  Apply <v>verify-guards-formula</v> to a name just as you \u000A would use <tt><see topic=\"ACL2____VERIFY-GUARDS\">verify-guards</see></tt>, but when you only want to view the formula \u000A rather than creating an event.  If the first argument is not a symbol, then it \u000A is treated as the body of a <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> event for which you want the guard \u000A proof obligation.</p> \u000A \u000A <p>See <see topic=\"ACL2____GUARD-OBLIGATION\">guard-obligation</see> if you want to obtain guard proof obligations \u000A for use in a program.</p>"],

 "ACL2____USING-COMPUTED-HINTS":[["Hints"],"ACL2 Sources","ACL2","<p>Computed hints (see <see topic=\"ACL2____COMPUTED-HINTS\">computed-hints</see>) are extraordinarily \u000A powerful.  We show a few examples here to illustrate their use.  We recommend \u000A that the <v>using-computed-hints-n</v> topics be read in the order \u000A <v>using-computed-hints-1</v>, <v>using-computed-hints-2</v>, and so on.</p>"],

 "ACL2____UBU":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:ubu :x-3      ; undo the last three commands (same as :ubt :x-2)\u000A:ubu (:x -3)   ; same as above\u000A:ubu fn        ; undo back up to, but not including the introduction of fn\u000A               ; (so fn will continue to be defined)\u000A:ubu 5         ; undo back up to, but not including, the fifth command\u000A               ; executed (leaving the first five commands in place)</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Ubu</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and undoes the \u000A <see topic=\"ACL2____COMMAND\">command</see>s from <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt> (aka <v>:x</v>) up to, but not including, \u000A the indicated command.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Ubu</v> can cause errors or queries.  To avoid these, see <see topic=\"ACL2____UBU_12\">ubu!</see>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____UBT\">ubt</see>, which is similar but also undoes the indicated \u000A command.  As for <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>, <v>:</v><tt><see topic=\"ACL2____OOPS\">oops</see></tt> will undo the undo (see \u000A <see topic=\"ACL2____OOPS\">oops</see>) and <tt><see topic=\"ACL2____UBU\">ubu</see></tt> will not undo into ``prehistory''.</p> \u000A \u000A <p>See <see topic=\"ACL2____U\">u</see> for how to undo just the latest command, and see <see topic=\"ACL2____UBT\">ubt</see> \u000A and see <see topic=\"ACL2____UBT_12\">ubt!</see> for how to undo back through (that is, including) the \u000A current command.</p>"],

 "ACL2____The_02Theorem_02that_02App_02is_02Associative":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02Proof_02of_02the_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <code>ACL2!&gt;<b>(defthm associativity-of-app</b> \u000A        <b>(equal (app (app a b) c)</b> \u000A               <b>(app a (app b c))))</b> \u000A</code> \u000A \u000A <p>The formula above says <v>app</v> is associative.  The <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A command instructs ACL2 to prove the formula and to name it \u000A <v>associativity-of-app</v>.  Actually, the <v>defthm</v> command also builds the \u000A formula into the database as a <tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rule, but we won't go into that just yet.</p> \u000A \u000A <p>What we will consider is how the ACL2 theorem prover proves this \u000A formula.</p> \u000A \u000A <p>If you proceed you will find the actual output of ACL2 in response to the \u000A command above.  Some of the text is highlighted for the purposes of the tour. \u000A ACL2 does not highlight its output.</p> \u000A \u000A <p>You will note that we sometimes highlight a single open parenthesis.  This \u000A is our way of drawing your attention to the subformula that begins with that \u000A parenthesis.  By clicking on the parenthesis you will get an explanation of \u000A the subformula or its processing.</p> \u000A \u000A <p><see topic='ACL2____The_02Proof_02of_02the_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____The_02End_02of_02the_02Walking_02Tour":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='sitting.gif'></img></p> \u000A \u000A <p>This completes the Walking Tour.</p> \u000A \u000A <p>We intend to document many other parts of the system this way, but we just \u000A haven't gotten around to it.</p> \u000A \u000A <p>To start the two tours over again from the beginning, click on the icons \u000A below.  If you are really interested in learning how to use ACL2, we recommend \u000A that you repeat each tour at least once more to explore branches of the tour \u000A that you might have missed.</p> \u000A \u000A <p>If you want to learn how to use the theorem prover, we now recommend that \u000A you devote the time necessary to work your way through the extended \u000A introduction to how to use the prover.</p> \u000A \u000A <p>See <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>.</p> \u000A \u000A <p>This will explain how to interact with ACL2 and has some sample problems \u000A for you to solve including some challenge proofs to make ACL2 find.</p> \u000A \u000A <p>We hope you enjoy ACL2.  We do.</p> \u000A \u000A <p>Matt Kaufmann and J Strother Moore</p> \u000A \u000A <p><see topic='ACL2____A_02Flying_02Tour_02of_02ACL2'><img \u000A src='flying.gif'></img></see> <see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'><img src='walking.gif'></img></see></p>"],

 "ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED":[["Parallelism"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED\">set-waterfall-parallelism-hacks-enabled</see> t)\u000A(<see topic=\"ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED\">set-waterfall-parallelism-hacks-enabled</see> nil)</code> \u000A \u000A <p>Some features (e.g., <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> and <see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see>s) of \u000A serial ACL2 are by default not available in ACL2(p) with waterfall parallelism \u000A enabled, because they offer a mechanism to modify <see topic=\"ACL2____STATE\">state</see> that is \u000A unsound.  To allow or (once again) disallow the use the these features in \u000A ACL2(p), call <v>set-waterfall-parallelism-hacks-enabled</v> with argument \u000A <v>t</v> or <v>nil</v>, respectively.</p> \u000A \u000A <p><v>Set-waterfall-parallelism-hacks-enabled</v> requires the use of a trust \u000A tag (see <see topic=\"ACL2____DEFTTAG\">defttag</see>).  One can call <tt><see topic=\"ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED_12\">set-waterfall-parallelism-hacks-enabled!</see></tt>  instead, which will automatically \u000A install a trust tag named <v>:waterfall-parallelism-hacks</v>.</p> \u000A \u000A <p>See <see topic=\"ACL2____ERROR-TRIPLES-AND-PARALLELISM\">error-triples-and-parallelism</see> for further related \u000A discussion.</p>"],

 "ACL2____Proving_02Theorems_02about_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____What_02is_02Required_02of_02the_02User_82Q_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>But ACL2 is a <b>logic</b>.  We can <b>prove theorems about the \u000A model</b>.</p> \u000A \u000A <p><img src='computing-machine-xxy.gif'></img></p> \u000A \u000A <code><b>Theorem.  MC 'mult is a multiplier</b> \u000A(implies (and (natp x) \u000A              (natp y)) \u000A         (equal (lookup 'z (mc (s 'mult x y) (mclk x))) \u000A                (* x y))). \u000A</code> \u000A \u000A <p>This theorem says that a certain program running on the <b>mc</b> machine \u000A will correctly multiply <b>any two natural numbers</b>.</p> \u000A \u000A <p>It is a statement about an <b>infinite</b> number of test cases!</p> \u000A \u000A <p>We know it is true about the model because we <b>proved</b> it.</p> \u000A \u000A <p>Of course, models of actual machines usually only accept a finite number of \u000A different inputs.  For example, engineers at Advanced Micro Devices (AMD), \u000A Centaur, and IBM have ACL2 models of floating point units that operate on \u000A double precision IEEE floating point numbers.  These are finite models.  But \u000A the size of their inputs is sufficiently large that they are verified by the \u000A same mathematical methods used to prove theorems about infinite state systems \u000A like our little <v>mc</v>.</p> \u000A \u000A <p><see topic='ACL2____What_02is_02Required_02of_02the_02User_82Q_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____PARALLELISM-PERFORMANCE":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____GRANULARITY\">granularity</see> for an important construct that limits the spawning \u000A of parallel computations, which can be important when a computation is too \u000A short-lived to warrant a separate thread.</p> \u000A \u000A <p>There are times in which parallelism provides no speedup because of garbage \u000A collection in the underlying Lisp implementation.  The following example \u000A illustrates this phenomenon.  If you change the <tt><see topic=\"ACL2____GRANULARITY\">granularity</see></tt> \u000A declaration so that the depth bound is 3, 4, or larger instead of 2, you may \u000A still find no speedup.  In all cases you may find that parallelism results in \u000A a significantly greater time spent in garbage collection.</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;finite-set-theory/osets/sets&quot; :dir :system)\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> set::pmergesort-exec (x depth)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A          ((<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (set::insert (<see topic=\"COMMON-LISP____CAR\">car</see> x) nil))\u000A          (t (<see topic=\"ACL2____MV-LET\">mv-let</see> (part1 part2)\u000A                     (set::split-list x nil nil)\u000A                     (<see topic=\"ACL2____PARGS\">pargs</see>\u000A                      (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____GRANULARITY\">granularity</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> depth 2)))\u000A                      (set::union (set::pmergesort-exec part1\u000A                                                          (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))\u000A                                   (set::pmergesort-exec part2\u000A                                                          (<see topic=\"COMMON-LISP____1_B2\">1+</see> depth))))))))\u000A(<see topic=\"ACL2____DEFCONST\">defconst</see> *x* (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> (fromto 1 400000)))\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (set::pmergesort-exec *x* 0)))\u000A(<see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see> nil)\u000A(<see topic=\"ACL2____TIME_42\">time$</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (set::pmergesort-exec *x* 0)))</code>"],

 "ACL2____NOTE-2-8-SYSTEM":[["Note-2-8"],"ACL2 Sources","ACL2","<p>ACL2 now runs on OpenMCL, ``an opensourced Common Lisp \u000A implementation, derived from Digitool's Macintosh Common Lisp product.'' \u000A Thanks to Greg Wright and Robert Krug for doing most of the work for this \u000A port.</p> \u000A \u000A <p>When <v>(</v><tt><see topic=\"ACL2____LP\">lp</see></tt><v>)</v> is first executed, the underlying raw Lisp \u000A package will change to <v>&quot;ACL2&quot;</v> (if that is not already the current \u000A package in raw Lisp).  This is a minor change that will probably not be \u000A noticed, since up to now it has probably been the case that the ACL2 \u000A executable starts up with <v>&quot;ACL2&quot;</v> as the underlying raw Lisp package. \u000A But this change was made because we have been informed that ACL2 executables \u000A based on OpenMCL need not start up with <v>&quot;ACL2&quot;</v> as the underlying raw \u000A Lisp package.</p> \u000A \u000A <p>ACL2 now runs on MCL 5.0.  Thanks to Pascal Costanza for updates to the \u000A instructions in file <v>mcl-acl2-startup.lisp</v> and for an update to the ACL2 \u000A sources (parameter <v>*compiled-file-extension*</v>).</p>"],

 "ACL2____MV_F3-LET":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Mv?-let</v> is a macro that extends the macro <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> by \u000A allowing a single variable to be bound.  Thus, the syntax is the same as that \u000A of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> except that <v>mv?-let</v> is permitted to bind a single \u000A variable to a form that produces a single value.  The macros <v>mv?-let</v> and \u000A <tt><see topic=\"ACL2____MV_F3\">mv?</see></tt>  are provided to facilitate the writing of macros that traffic in \u000A expressions that could return one or more (multiple) values.</p> \u000A \u000A <p>For example, the form</p> \u000A \u000A <code>(<see topic=\"ACL2____MV_F3-LET\">mv?-let</see> (y)\u000A         (f x)\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer y))\u000A         (g x y z))</code> \u000A \u000A <p>is equivalent to the following form.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((y (f x)))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer y))\u000A  (g x y z))</code> \u000A \u000A <p>Calls of <v>mv?-let</v> and of <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> are equivalent when the first \u000A argument contains at least two variables; see <see topic=\"ACL2____MV-LET\">mv-let</see> for documentation. \u000A In the case of binding a single variable, the general form is</p> \u000A \u000A <code>(<see topic=\"ACL2____MV_F3-LET\">mv?-let</see> (var)\u000A         term\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ... (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...)\u000A         body)</code> \u000A \u000A <p>and this is equivalent to the following form (see <see topic=\"COMMON-LISP____LET\">let</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((var term))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...) ... (<see topic=\"COMMON-LISP____DECLARE\">declare</see> ...)\u000A  body)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____MV_F3\">mv?</see>.</p>"],

 "ACL2____MAKE-FAST-ALIST":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>Note: it is often better to use <v>with-fast-alist</v>; see <see topic=\"ACL2____WITH-FAST-ALIST\">with-fast-alist</see>.</p> \u000A \u000A <p>Logically, <v>make-fast-alist</v> is the identity function.</p> \u000A \u000A <p><b>Function: </b><srclink>make-fast-alist</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-fast-alist (alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       alist)</code> \u000A \u000A <p>Under the hood, we construct and return an object that is <v>equal</v> to \u000A <v>alist</v> and which is a fast alist.  If <v>alist</v> is already a fast alist, \u000A almost no work is required: we simply return it unchanged.</p> \u000A \u000A <p>When <v>alist</v> is not fast, we must minimally construct a hash table for \u000A its bindings.  It is often possible to bind this new hash table to <v>alist</v> \u000A itself.  But in certain cases when the alist keys are not <see topic=\"ACL2____NORMED\">normed</see>, a new \u000A alist must be constructed, also, and so we may return an <v>equal</v> but not \u000A <v>eq</v> alist.  (In these cases, we still try to avoid at least some consing \u000A by reusing the \"longest normed tail\" of the alist.)</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This answer is in the form of a script sufficient to lead ACL2 to a \u000A proof.</p> \u000A \u000A <code> ; Trying <v>dupsp-rev</v> at this point produces the key checkpoint: \u000A \u000A ; (IMPLIES (AND (CONSP X) \u000A ;               (NOT (MEMBER (CAR X) (CDR X))) \u000A ;               (EQUAL (DUPSP (REV (CDR X))) \u000A ;                      (DUPSP (CDR X)))) \u000A ;          (EQUAL (DUPSP (APPEND (REV (CDR X)) (LIST (CAR X)))) \u000A ;                 (DUPSP (CDR X)))) \u000A \u000A ; which suggests the lemma \u000A \u000A ; (defthm dupsp-append \u000A ;   (implies (not (member e x)) \u000A ;            (equal (dupsp (append x (list e))) \u000A ;                   (dupsp x)))) \u000A \u000A ; However, attempting to prove that, produces a key checkpoint \u000A ; containing <v>(<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (<see topic=\"COMMON-LISP____CDR\">CDR</see> X) (<see topic=\"COMMON-LISP____LIST\">LIST</see> E)))</v>. \u000A ; So we prove the lemma: \u000A \u000A (defthm member-append \u000A   (iff (member e (append a b)) \u000A        (or (member e a) \u000A            (member e b)))) \u000A \u000A ; Note that we had to use <v>iff</v> instead of <v>equal</v> since <v>member</v> is \u000Anot a \u000A ; Boolean function. \u000A \u000A ; Having proved this lemma, we return to <v>dupsp-append</v> and succeed: \u000A \u000A (defthm dupsp-append \u000A   (implies (not (member e x)) \u000A            (equal (dupsp (append x (list e))) \u000A                   (dupsp x)))) \u000A \u000A ; So now we return to <v>dups-rev</v>, expecting success.  But it fails \u000A ; with the same key checkpoint: \u000A \u000A ; (IMPLIES (AND (CONSP X) \u000A ;               (NOT (MEMBER (CAR X) (CDR X))) \u000A ;               (EQUAL (DUPSP (REV (CDR X))) \u000A ;                      (DUPSP (CDR X)))) \u000A ;          (EQUAL (DUPSP (APPEND (REV (CDR X)) (LIST (CAR X)))) \u000A ;                 (DUPSP (CDR X)))) \u000A \u000A ; Why wasn't our <v>dupsp-append</v> lemma applied?  We have two choices here: \u000A ; (1) Think.  (2) Use tools. \u000A \u000A ; Think:  When an enabled rewrite rule doesn't fire even though the left-hand \u000A ; side matches the target, the hypothesis couldn't be relieved.  The \u000A<v>dups-append</v> \u000A ; rule has the hypothesis <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> e x))</v> and after the match with the \u000Aleft-hand side, \u000A ; <v>e</v> is <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> X)</v> and <v>x</v> is <v>(REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))</v>.  So the system \u000Acouldn't rewrite \u000A ; <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X))))</v> to true, even though it knows that \u000A ; <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X) (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)))</v> from the second hypothesis of the \u000Acheckpoint. \u000A ; Obviously, we need to prove <v>member-rev</v> below. \u000A \u000A ; Use tools:  We could enable the ``break rewrite'' facility, with \u000A \u000A ; ACL2 !&gt;:brr t \u000A \u000A ; and then install an unconditional monitor on the rewrite rule \u000A ; <v>dupsp-append</v>, whose rune is (:REWRITE DUPSP-APPEND), with: \u000A \u000A ; :monitor (:rewrite dupsp-append) t \u000A \u000A ; Then we could re-try our main theorem, dupsp-rev.  At the resulting \u000A ; interactive break we type :eval to evaluate the attempt to relieve the \u000A ; hypotheses of the rule. \u000A \u000A ; (1 Breaking (:REWRITE DUPSP-APPEND) on \u000A ; (DUPSP (BINARY-APPEND (REV #) (CONS # #))): \u000A ; 1 ACL2 &gt;:eval \u000A \u000A ; 1x (:REWRITE DUPSP-APPEND) failed because :HYP 1 rewrote to \u000A ; (NOT (MEMBER (CAR X) (REV #))). \u000A \u000A ; Note that the report above shows that hypothesis 1 of the rule \u000A ; did not rewrite to T but instead rewrote to an expression \u000A ; involving <v>(<see topic=\"COMMON-LISP____MEMBER\">member</see> ... (rev ...))</v>.  Thus, we're led to the \u000A ; same conclusion that Thinking produced.  To get out of the \u000A ; interactive break we type: \u000A \u000A ; 1 ACL2 &gt;:a! \u000A ; Abort to ACL2 top-level \u000A \u000A ; and then turn off the break rewrite tool since we won't need it \u000A ; again right now, with: \u000A \u000A ; ACL2 !&gt;:brr nil \u000A \u000A ; In either case, by thinking or using tools, we decide to prove: \u000A \u000A (defthm member-rev \u000A   (iff (member e (rev x)) \u000A        (member e x))) \u000A \u000A ; which succeeds.  Now when we try to prove dups-rev, it succeeds. \u000A \u000A (defthm dupsp-rev \u000A   (equal (dupsp (rev x)) \u000A          (dupsp x))) \u000A \u000A </code> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3\">introductory-challenge-problem-3</see>.</p>"],

 "ACL2____HONS-SUMMARY":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-summary</v> just returns nil; we leave it enabled and \u000A would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, this function gathers and prints some basic information \u000A about the sizes of the tables in the Hons Space.</p> \u000A \u000A <p>This information may be useful for deciding if you want to garbage collect \u000A using functions such as <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt> or <tt><see topic=\"ACL2____HONS-WASH\">hons-wash</see></tt>.  It may also \u000A be useful for determining good initial sizes for the Hons Space hash tables \u000A for your particular computation; see <see topic=\"ACL2____HONS-RESIZE\">hons-resize</see>.  For information \u000A about <see topic=\"ACL2____FAST-ALISTS\">fast-alists</see>, see <see topic=\"ACL2____FAST-ALIST-SUMMARY\">fast-alist-summary</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-summary</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-summary\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____Functions_02for_02Manipulating_02these_02Objects":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Modeling_02in_02ACL2'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>Consider a typical ``stack'' of control frames.</p> \u000A \u000A <p><img src='stack.gif'></img></p> \u000A \u000A <p>Suppose the model required that we express the idea of ``the most recent \u000A frame whose return program counter points into <v>MAIN</v>.''</p> \u000A \u000A <p>The natural expression of this notion involves</p> \u000A \u000A <p><b>function application</b> &mdash; ``fetch the <v>return-pc</v> of this \u000A frame''</p> \u000A \u000A <p><b>case analysis</b> &mdash; ``<b>if</b> the pc is <v>MAIN</v>, <b>then</b> \u000A ...''</p> \u000A \u000A <p><b>iteration</b> or <b>recursion</b> &mdash; ``pop this frame off and \u000A repeat.''</p> \u000A \u000A <p>The designers of ACL2 have taken the position that a <b>programming</b> \u000A <b>language</b> is the natural language in which to define such notions, \u000A <b>provided</b> the language has a mathematical foundation so that models can \u000A be analyzed and properties derived logically.</p> \u000A \u000A <p>Common Lisp is the language supported by ACL2.  To be precise, a small \u000A applicative subset of Common Lisp is the language supported by ACL2.</p> \u000A \u000A <p><see topic='ACL2____Modeling_02in_02ACL2'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____FUNCTIONAL-INSTANTIATION-IN-ACL2R":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>This documentation topic relates to ACL2(r), the modification of \u000A ACL2 that supports the real numbers (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p> \u000A \u000A <p>See <see topic=\"ACL2____HINTS\">hints</see> and see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see> for a discussion of \u000A <v>:use</v> hints that employ the <v>:functional-instance</v> keyword.  Here, we \u000A document additional requirements for such hints that applies to ACL2(r).  We \u000A assume familiarity with lemma instances; see <see topic=\"ACL2____LEMMA-INSTANCE\">lemma-instance</see>.</p> \u000A \u000A <p>(1) When functionally instantiating a non-classical formula, it is illegal \u000A to use pseudo-lambda expressions in a lemma instance.</p> \u000A \u000A <p>(2) A classical function symbol must be bound either to a classical \u000A function symbol or to a lambda (or, if allowed, pseudo-lambda) expression with \u000A a classical body.  Similarly, a non-classical function symbol must be bound \u000A either to a non-classical function symbol or to a lambda (or, if allowed, \u000A pseudo-lambda) expression with a non-classical body.</p>"],

 "ACL2____EXECUTABLE-COUNTERPART-THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____EXECUTABLE-COUNTERPART-THEORY\">executable-counterpart-theory</see> :here)\u000A(<see topic=\"ACL2____EXECUTABLE-COUNTERPART-THEORY\">executable-counterpart-theory</see> 'lemma3)</code> \u000A \u000A <p>See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____EXECUTABLE-COUNTERPART-THEORY\">executable-counterpart-theory</see> logical-name)</code> \u000A \u000A <p>Returns the theory containing all the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> \u000A <see topic=\"ACL2____RUNE\">rune</see>s, whether <see topic=\"ACL2____ENABLE\">enable</see>d or not, that existed immediately after \u000A <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt> was introduced.  See the documentation for <see topic=\"ACL2____THEORIES\">theories</see>, <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>, <see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see> and <tt><see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see></tt>.</p> \u000A \u000A <p>You may experience a fencepost problem in deciding which logical name to \u000A use.  <tt><see topic=\"ACL2____DEFLABEL\">Deflabel</see></tt> can always be used to mark unambiguously for future \u000A reference a particular point in the development of your theory.  The order of \u000A <see topic=\"ACL2____EVENTS\">events</see> in the vicinity of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is confusing.  See \u000A <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-ON-LISTS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Classical Induction on Lists</b>: To prove <v>(p x)</v>, for all <v>x</v>, \u000A by classical induction on the linear list structure, prove each of the \u000A following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (endp x) (p x)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp x)) \u000A              (p (cdr x))) \u000A         (p x)) \u000A</code> \u000A \u000A <p>An argument analogous to that given for natural numbers, <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-NAT-RECURSION\">example-induction-scheme-nat-recursion</see>, establishes <v>(p x)</v> for every \u000A <v>x</v>.  For example, <v>(p -7)</v>, <v>(p 'abc)</v>, and <v>(p nil)</v> are all \u000A established by the Base Case.  <v>(p '(Friday))</v> follows from <v>(p nil)</v>, \u000A given the Induction Step.  That sentence bears thinking about!  Think about \u000A it!  Similarly, <v>(p '(Yellow))</v> holds for the same reason.  <v>(p\u000A '(Thursday Friday))</v> follows from <v>(p '(Friday))</v> and the Induction Step, \u000A etc.</p> \u000A \u000A <p>A function that suggests this induction is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      y\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A            (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))).</code>"],

 "ACL2____DISABLEDP":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples: \u000A \u000A:disabledp foo   ; returns a list of all disabled runes whose base \u000A                 ; symbol is foo (see <see topic=\"ACL2____RUNE\">rune</see>) \u000A(disabledp 'foo) ; same as above (i.e., :disabledp foo) \u000A:disabledp (:rewrite bar . 1) ; returns t if the indicated rune is \u000A                              ; disabled, else nil \u000A(disabledp (:rewrite bar . 1)); same as immediately above \u000A</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PR\">pr</see>, which gives much more information about the rules \u000A associated with a given event.</p> \u000A \u000A <p><v>Disabledp</v> takes one argument, an event name (see <see topic=\"ACL2____EVENTS\">events</see>) other \u000A than <v>nil</v> or a <see topic=\"ACL2____RUNE\">rune</see>.  In the former case it returns the list of \u000A disabled runes associated with that name, in the sense that the rune's ``base \u000A symbol'' is that name (see <see topic=\"ACL2____RUNE\">rune</see>) or, if the event named is a <tt><see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see></tt> event, then the list of disabled runes associated with the function \u000A corresponding to that macro name, if any (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).  In \u000A the latter case, where the argument is a <see topic=\"ACL2____RUNE\">rune</see>, <v>disabledp</v> returns \u000A <v>t</v> if the rune is disabled, and <v>nil</v> otherwise.</p>"],

 "ACL2____DEFUN-NOTINLINE":[["Defun","Events"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> for an analogous utility that supports \u000A inlining.  The present utility is probably far less useful; it tells the \u000A compiler <i>not</i> to inline calls of the function being defined.  Also see \u000A <see topic=\"ACL2____DEFUND-NOTINLINE\">defund-notinline</see> for a variant of this event that disables the \u000A newly-defined function symbol.</p> \u000A \u000A <p>Under the hood, <v>(<see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see> f ...)</v> and <v>(<see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see> f ...)</v> \u000A cause evaluation of Common Lisp forms <v>(declaim (inline f$inline))</v> and \u000A <v>(declaim (notinline f$notinline))</v>, respectively.  According to the Common \u000A Lisp spec, the compiler need not respect the first of these (for <v>inline</v>), \u000A but it must respect the second of these (for <v>notinline</v>).  Fortunately, \u000A Common Lisp implementations often do respect the first of these as well.</p>"],

 "ACL2____ADD-INVISIBLE-FNS":[["Loop-stopper"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> binary-+ unary-- foo)\u000A(<see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> + unary-- foo)</code> \u000A \u000A <p>Each of the <see topic=\"ACL2____EVENTS\">events</see> above makes unary functions <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> and \u000A <v>foo</v> ``invisible'' for the purposes of applying permutative <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> trees.  Thus, <v>arg</v> and <v>(<see topic=\"ACL2____UNARY--\">unary--</see>\u000A arg)</v> will be given the same weight and will be permuted so as to be \u000A adjacent.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> top-fn unary-fn1 ... unary-fnk)</code> \u000A \u000A <p>where <v>top-fn</v> is a function symbol and the <v>unary-fni</v> are unary \u000A function symbols, or more generally, these are all macro aliases for function \u000A symbols (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).</p> \u000A \u000A <p>For more information see <see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see>.  Also see <see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see>, which explains how to set the entire table in a \u000A single event, and see <see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see>.</p>"],

 "COMMON-LISP_____C3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>&lt;=</v> is a macro, and <v>(<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x y)</v> expands to the same thing as \u000A <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> y x))</v>.  See <see topic=\"COMMON-LISP_____C3\">&lt;</see>.</p> \u000A \u000A <p><v>&lt;=</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>&lt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> &lt;= (x y)\u000A          (<see topic=\"COMMON-LISP____LIST\">list</see> 'not (<see topic=\"COMMON-LISP____LIST\">list</see> '&lt; y x)))</code>"],

 "COMMON-LISP____ROUND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> 14 3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> -14 3)\u000A-5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> 14 -3)\u000A-5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> -14 -3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> 13 3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> -13 3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> 13 -3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> -13 -3)\u000A4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> -15 -3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____ROUND\">round</see> 15 -2)\u000A-8</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____ROUND\">Round</see> i j)</v> is the result of taking the quotient of <v>i</v> and <v>j</v> \u000A and rounding off to the nearest integer.  When the quotient is exactly halfway \u000A between consecutive integers, it rounds to the even one.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____ROUND\">round</see> i j)</v> requires that <v>i</v> and <v>j</v> are \u000A rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Round</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  However, note that unlike Common Lisp, the ACL2 \u000A <v>round</v> function returns only a single value,</p> \u000A \u000A <p><b>Function: </b><srclink>round</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     round (i j)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> i)\u000A                                 (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> j)\u000A                                 (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> j 0)))))\u000A     (<see topic=\"COMMON-LISP____LET\">let</see> ((<see topic=\"ACL2____Q\">q</see> (<see topic=\"COMMON-LISP_____A2\">*</see> i (<see topic=\"COMMON-LISP_____F2\">/</see> j))))\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> q) q)\u000A                ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> q 0)\u000A                 (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((fl (<see topic=\"COMMON-LISP____FLOOR\">floor</see> q 1)) (remainder (<see topic=\"COMMON-LISP____-\">-</see> q fl)))\u000A                       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____E3\">&gt;</see> remainder 1/2) (<see topic=\"COMMON-LISP_____B2\">+</see> fl 1))\u000A                             ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> remainder 1/2) fl)\u000A                             (t (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____A2\">*</see> fl (<see topic=\"COMMON-LISP_____F2\">/</see> 2))) fl)\u000A                                      (t (<see topic=\"COMMON-LISP_____B2\">+</see> fl 1)))))))\u000A                (t (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((cl (<see topic=\"COMMON-LISP____CEILING\">ceiling</see> q 1))\u000A                          (remainder (<see topic=\"COMMON-LISP____-\">-</see> q cl)))\u000A                         (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____-\">-</see> 1/2) remainder) cl)\u000A                               ((<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____-\">-</see> 1/2) remainder) (<see topic=\"COMMON-LISP_____B2\">+</see> cl -1))\u000A                               (t (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____A2\">*</see> cl (<see topic=\"COMMON-LISP_____F2\">/</see> 2))) cl)\u000A                                        (t (<see topic=\"COMMON-LISP_____B2\">+</see> cl -1))))))))))</code>"],

 "COMMON-LISP____NTHCDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____NTHCDR\">Nthcdr</see> n l)</v> removes the first <v>n</v> elements from the list \u000A <v>l</v>.</p> \u000A \u000A <p>The following is a theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n l))\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> n (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))\u000A                    (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) n)\u000A                  0)))</code> \u000A \u000A <p>For related functions, see <see topic=\"ACL2____TAKE\">take</see> and see <see topic=\"COMMON-LISP____BUTLAST\">butlast</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> n l)</v> requires that <v>n</v> is a nonnegative \u000A integer and <v>l</v> is a true list.</p> \u000A \u000A <p><v>Nthcdr</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>nthcdr</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nthcdr (n l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)\u000A                                   (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n) l (<see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> (<see topic=\"COMMON-LISP_____B2\">+</see> n -1) (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))</code>"],

 "COMMON-LISP____IN-PACKAGE":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;MY-PKG&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> str)</code> \u000A \u000A <p>where <v>str</v> is a string that names an existing ACL2 package, i.e., one \u000A of the initial packages such as <v>&quot;KEYWORD&quot;</v> or <v>&quot;ACL2&quot;</v> or a package \u000A introduced with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>.  For a complete list of the known packages \u000A created with <tt><see topic=\"ACL2____DEFPKG\">defpkg</see></tt>, evaluate</p> \u000A \u000A <code>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> (known-package-alist state)).</code> \u000A \u000A <p>See <see topic=\"ACL2____DEFPKG\">defpkg</see>.  An ACL2 book (see <see topic=\"ACL2____BOOKS\">books</see>) must contain a single \u000A <v>in-package</v> form, which must be the first form in that book.</p>"],

 "COMMON-LISP____CEILING":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> 14 3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> -14 3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> 14 -3)\u000A-4\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> -14 -3)\u000A5\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> -15 -3)\u000A5</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____CEILING\">Ceiling</see> i j)</v> is the result of taking the quotient of <v>i</v> and \u000A <v>j</v> and returning the smallest integer that is at least as great as that \u000A quotient.  For example, the quotient of <v>-14</v> by <v>3</v> is <v>-4 2/3</v>, and \u000A the smallest integer at least that great is <v>-4</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____CEILING\">ceiling</see> i j)</v> requires that <v>i</v> and <v>j</v> are \u000A rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Ceiling</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  However, note that unlike Common Lisp, the ACL2 \u000A <v>ceiling</v> function returns only a single value,</p> \u000A \u000A <p><b>Function: </b><srclink>ceiling</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ceiling (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> i)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> j)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> j 0)))))\u000A       (<see topic=\"COMMON-LISP____LET_A2\">let*</see> ((<see topic=\"ACL2____Q\">q</see> (<see topic=\"COMMON-LISP_____A2\">*</see> i (<see topic=\"COMMON-LISP_____F2\">/</see> j)))\u000A              (n (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> q))\u000A              (d (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> q)))\u000A             (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____D3\">=</see> d 1) n)\u000A                   ((<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                    (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> n d)\u000A                       1))\u000A                   (t (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> (<see topic=\"COMMON-LISP____-\">-</see> n)\u000A                                                       d))))))</code>"],

 "COMMON-LISP____BUTLAST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____BUTLAST\">Butlast</see> l n)</v> is the list obtained by removing the last <v>n</v> \u000A elements from the true list <v>l</v>.  The following is a theorem \u000A (though it takes some effort, including lemmas, to get ACL2 to prove it).</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"COMMON-LISP____BUTLAST\">butlast</see> l n))\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))\u000A                    (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) n)\u000A                  0)))</code> \u000A \u000A <p>For related functions, see <see topic=\"ACL2____TAKE\">take</see> and see <see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____BUTLAST\">butlast</see> l n)</v> requires that <v>n</v> is a \u000A nonnegative integer and <v>lst</v> is a true list.</p> \u000A \u000A <p><v>Butlast</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  Note: In Common Lisp the second argument of \u000A <v>butlast</v> is optional, but in ACL2 it is required.</p> \u000A \u000A <p><b>Function: </b><srclink>butlast</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> butlast (lst n)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n))))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((lng (<see topic=\"ACL2____LEN\">len</see> lst)) (n (<see topic=\"ACL2____NFIX\">nfix</see> n)))\u000A            (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> lng n)\u000A                nil (<see topic=\"ACL2____TAKE\">take</see> (<see topic=\"COMMON-LISP____-\">-</see> lng n) lst))))</code>"],

 "ACL2____WITH-STOLEN-ALIST":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>Logically, <v>with-stolen-alist</v> just returns <v>form</v>.</p> \u000A \u000A <p>Under the hood, we cause <v>alist</v> to become a fast alist before executing \u000A <v>form</v>, and we check the various conditions outlined above before returning \u000A the final value.</p> \u000A \u000A <p>Note that <v>with-stolen-alist</v> will cause logically tail-recursive \u000A functions not to execute tail-recursively if its cleanup phase happens after \u000A the tail-recursive call returns.</p>"],

 "ACL2____The_02Associativity_02of_02App":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02Theorem_02that_02App_02is_02Associative'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code>ACL2!&gt;<b>(let ((a '(1 2))</b> \u000A            <b>(b '(3 4))</b> \u000A            <b>(c '(5 6)))</b> \u000A        <b>(equal (app (app a b) c)</b> \u000A               <b>(app a (app b c))))</b> \u000AT \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Observe that, for the particular <v>a</v>, <v>b</v>, and <v>c</v> above, <v>(app\u000A (app a b) c)</v> returns the same thing as <v>(app a (app b c))</v>.  Perhaps \u000A <v>app</v> is <b>associative</b>.  Of course, to be associative means that the \u000A above property must hold for all values of <v>a</v>, <v>b</v>, and <v>c</v>, not \u000A just the ones <b>tested</b> above.</p> \u000A \u000A <p>Wouldn't it be cool if you could type</p> \u000A \u000A <code>ACL2!&gt;<b>(equal (app (app a b) c)</b> \u000A             <b>(app a (app b c)))</b> \u000A \u000A</code> \u000A \u000A <p>and have ACL2 compute the value <v>T</v>?  Well, <b>you can't!</b> If you try \u000A it, you'll get an error message!  The message says we can't evaluate that form \u000A because it contains <b>free</b> variables, i.e., variables not given values. \u000A Click <see topic='ACL2____Free_02Variables_02in_02Top-Level_02Input'>here</see> to \u000A see the message.</p> \u000A \u000A <p>We cannot evaluate a form on an infinite number of cases.  But we can prove \u000A that a form is a theorem and hence know that it will always evaluate to \u000A true.</p> \u000A \u000A <p><see topic='ACL2____The_02Theorem_02that_02App_02is_02Associative'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____SLOW-ALIST-WARNING":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Obtaining hash-table performance from <tt><see topic=\"ACL2____HONS-GET\">hons-get</see></tt> requires one to \u000A follow a certain discipline.  If this discipline is violated, you may see a \u000A \"slow alist warning\".  This warning means that the alist you are extending \u000A or accessing does not have a valid hash table associated with it, and hence \u000A any accesses must be carried out with <tt><see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see></tt> instead of \u000A <v>gethash</v>.</p> \u000A \u000A <p>You can control whether or not you get a warning and, if so, whether or not \u000A a break (an error from which you can continue) ensues.  For instance:</p> \u000A \u000A <code>(set-slow-alist-action :warning)  ; warn on slow access (<see topic=\"ACL2____DEFAULT\">default</see>)\u000A(set-slow-alist-action :break)    ; warn and also call break$\u000A(set-slow-alist-action nil)       ; do not warn or break</code> \u000A \u000A <p>The above forms expand to <tt><see topic=\"ACL2____TABLE\">table</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, so they can be \u000A embedded in <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt>s and <see topic=\"ACL2____BOOKS\">books</see>, wrapped in <tt><see topic=\"ACL2____LOCAL\">local</see></tt>, \u000A and so on.</p>"],

 "ACL2____PSOF":[["Other"],"ACL2 Sources","ACL2","<p>For a similar utility, see <see topic=\"ACL2____PSO\">pso</see>.  Like <v>:pso</v>, the \u000A <v>:psof</v> command prints output that was generated in an environment where \u000A output was being saved, typically <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt>; also see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>.  But unlike <v>:pso</v>, <v>:psof</v> takes a filename argument \u000A and saves output to that file, instead of to the terminal.  For large proofs, \u000A <v>:</v><tt><see topic=\"ACL2____PSOF\">psof</see></tt> may complete more quickly than <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt>.  Note \u000A that as with <v>:pso</v>, <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output will be suppressed.</p> \u000A \u000A <p>The first line of output from <v>:psof</v> directs the Emacs editor to use \u000A auto-revert mode.  You can change the frequency of auto-reverting the buffer \u000A connected to a file by evaluating a suitable command in Emacs.  For example, \u000A the command <v>(setq auto-revert-interval .1)</v> arranges for auto-revert mode \u000A to update as needed every 1/10 of a second.</p>"],

 "ACL2____PROVER-OUTPUT":[["ACL2"],"ACL2 Sources","ACL2"," \u000A \u000A"],

 "ACL2____Models_02of_02Computer_02Hardware_02and_02Software":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____A_02Typical_02State'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p><img src='computing-machine.gif'></img></p> \u000A \u000A <p>Computing machines, whether hardware or software or some combintation, are \u000A frequently modeled as ``state machines.''</p> \u000A \u000A <p>To so model a computing machine we must represent its <b>states</b> as \u000A objects in our mathematical framework.</p> \u000A \u000A <p><b>Transitions</b> are functions or relations on state objects.</p> \u000A \u000A <p>In what language shall we define these objects, functions, and \u000A relations?</p> \u000A \u000A <p>The mathematical languages we were taught in high school</p> \u000A \u000A <p><b>algebra</b>,</p> \u000A \u000A <p><b>geometry</b>,</p> \u000A \u000A <p><b>trignometry</b>, and</p> \u000A \u000A <p><b>calculus</b></p> \u000A \u000A <p>are often inappropriate for modeling digital systems.  They primarily let \u000A us talk about numbers and continuous functions.</p> \u000A \u000A <p>To see what kind of expressive power we need, take a closer look at what a \u000A typical state contains.</p> \u000A \u000A <p><see topic='ACL2____A_02Typical_02State'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____LEGACY-DOCUMENTATION":[["Documentation"],"ACL2 Sources","ACL2",""],

 "ACL2____HONS-GET":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>hons-get</v> is just an alias for <tt><see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see></tt>; we \u000A typically leave it enabled and prefer to reason about <v>hons-assoc-equal</v> \u000A instead.  One benefit of this approach is that it helps to avoids \"false\" \u000A discipline warnings that might arise from execution during theorem \u000A proving.</p> \u000A \u000A <p>Under the hood, when <v>alist</v> is a fast alist that is associated with a \u000A valid hash table, <v>hons-get</v> first norms <v>key</v> using <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt>, \u000A then becomes a <v>gethash</v> operation on the hidden hash table.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-get</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-get (key alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see> key alist))</code>"],

 "ACL2____FUNCTION-THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> :here)\u000A(<see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> 'lemma3)</code> \u000A \u000A <p>See <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> logical-name)</code> \u000A \u000A <p>Returns the theory containing all the <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>s, whether <see topic=\"ACL2____ENABLE\">enable</see>d or not, that existed immediately after <tt><see topic=\"ACL2____LOGICAL-NAME\">logical-name</see></tt> was introduced.  See the documentation for <see topic=\"ACL2____THEORIES\">theories</see>, \u000A <see topic=\"ACL2____LOGICAL-NAME\">logical-name</see> and <tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART-THEORY\">executable-counterpart-theory</see></tt>.</p> \u000A \u000A <p>You may experience a fencepost problem in deciding which logical name to \u000A use.  <tt><see topic=\"ACL2____DEFLABEL\">Deflabel</see></tt> can always be used to mark unambiguously for future \u000A reference a particular point in the development of your theory.  The order of \u000A <see topic=\"ACL2____EVENTS\">events</see> in the vicinity of an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> is confusing.  See \u000A <see topic=\"ACL2____ENCAPSULATE\">encapsulate</see>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-UPWARDS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Induction Upwards</b>: To <v>(p i max)</v> for all <v>i</v> and all \u000A <v>max</v>, prove each of the following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (not (and (natp i) \u000A                   (natp max) \u000A                   (&lt; i max))) \u000A         (p i max)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and  (natp i) \u000A               (natp max) \u000A               (&lt; i max) \u000A               (p (+ i 1) max)) \u000A         (p i max)) \u000A</code> \u000A \u000A <p>Note that the induction hypothesis is about an <v>i</v> that is <i>bigger</i> \u000A than the <v>i</v> in in the conclusion.  In induction, as in recursion, the \u000A sense of one thing being ``smaller'' than another is determined by an \u000A arbitrary measure of all the variables, not the magnitude or extent of some \u000A particular variable.</p> \u000A \u000A <p>A function that suggests this induction is shown below.  ACL2 has to be \u000A told the measure, namely the difference between <v>max</v> and <v>i</v> (coerced \u000A to a natural number to insure that the measure is an ordinal).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> count-up (i max)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :measure (<see topic=\"ACL2____NFIX\">nfix</see> (<see topic=\"COMMON-LISP____-\">-</see> max i))))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> i)\u000A           (<see topic=\"ACL2____NATP\">natp</see> max)\u000A           (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i max))\u000A      (<see topic=\"COMMON-LISP____CONS\">cons</see> i (count-up (<see topic=\"COMMON-LISP_____B2\">+</see> 1 i) max))\u000A      nil)).</code>"],

 "ACL2____A_02Sketch_02of_02How_02the_02Rewriter_02Works":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Below we show the first target term, extracted from the current \u000A conjecture.  Below it we show the associativity rule.</p> \u000A \u000A <p><img src='uaa-rewrite.gif'></img></p> \u000A \u000A <p>The variables of the rewrite rule are <b>instantiated</b> so that the \u000A <b>left-hand side</b> of the rule matches the target:</p> \u000A \u000A <code>variable          term from target\u000A  a                     x1\u000A  b                     x2\u000A  c                     (app x3 x4)</code> \u000A \u000A <p>Then the target is <b>replaced</b> by the instantiated <b>right-hand \u000A side</b> of the rule.</p> \u000A \u000A <p>Sometimes rules have <b>hypotheses</b>.  To make a long story short, if the \u000A rule has hypotheses, then after matching the left-hand side, the rewriter \u000A instantiates the hypotheses and rewrites them recursively.  This is called \u000A <b>backchaining</b>.  If they all rewrite to true, then the target is replaced \u000A as above.</p> \u000A \u000A <p>We discuss the rewriter in more detail in the extended introduction to how \u000A to use the theorem prover, see <see topic=\"ACL2____INTRODUCTION-TO-THE-THEOREM-PROVER\">introduction-to-the-theorem-prover</see>, \u000A which we will recommend you work through <b>after</b> you have finished the \u000A two tours.</p>"],

 "ACL2____ACL2_02System_02Architecture":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Rewrite_02Rules_02are_02Generated_02from_02DEFTHM_02Events'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p><img src='acl2-system-architecture.gif'></img></p> \u000A \u000A <p>The user interacts with the theorem prover by giving it definitions, \u000A theorems and advice.  Most often the advice is about how to store each proved \u000A theorem as a rule.  Sometimes the advice is about how to prove a specific \u000A theorem.</p> \u000A \u000A <p>The database consists of all the rules ACL2 ``knows.''  It is possible to \u000A include in the database all of the rules in some certified file of other \u000A events.  Such certified files are called <see topic=\"ACL2____BOOKS\">books</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p> \u000A \u000A <p>Interesting proofs are usually built on top of many books, some of which \u000A are written especially for that problem domain and others of which are about \u000A oft-used domains, like arithmetic or list processing.  ACL2's distribution \u000A includes many books written by users.  See the ``books'' link under the \u000A <b>Lemma Libraries and Utilities</b> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A link of the ACL2 home page.</p> \u000A \u000A <p><see topic='ACL2____Rewrite_02Rules_02are_02Generated_02from_02DEFTHM_02Events'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____ACL2_02Conses_02or_02Ordered_02Pairs":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The function <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A creates an ordered pair.  <tt><see topic=\"COMMON-LISP____CAR\">Car</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> and \u000A <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> return the first and second components, \u000A respectively, of an ordered pair.  The function <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A recognizes ordered pairs.</p> \u000A \u000A <p>Ordered pairs are used to represent lists and trees.  See any Common Lisp \u000A documentation for a discussion of how list constants are written and for the \u000A many list processing functions available.  Also, see <see topic=\"ACL2____PROGRAMMING\">programming</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A where we list all the ACL2 primitive functions.</p> \u000A \u000A <p>Here are some examples of list constants to suggest their syntax.</p> \u000A \u000A <code>'(a . b)                ; a pair whose car is 'a and cdr is 'b\u000A'(a . nil)              ; a pair whose car is 'a and cdr is nil\u000A'(a)                    ; another way to write the same thing\u000A'(a b)                  ; a pair whose car is 'a and cdr is '(b)\u000A'(a b c)                ; a pair whose car is 'a and cdr is '(b c)\u000A                        ;  i.e., a list of three symbols, a, b, and c.\u000A'((a . 1) (b . 2))      ; a list of two pairs</code> \u000A \u000A <p>It is useful to distinguish ``proper'' conses from ``improper'' ones, the \u000A former being those cons trees whose right-most branch terminates with \u000A <v>nil</v>.  A ``true list'' (see <see topic=\"ACL2____TRUE-LISTP\">true-listp</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) is \u000A either <v>nil</v> or a proper cons.  <v>(A b c . 7)</v> is an improper cons and \u000A hence not a true list.</p>"],

 "ACL2-PC____DIVE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(DIVE 1)    -- assign the new current subterm to be the first\u000A               argument of the existing current subterm\u000A(DIVE 1 2)  -- assign the new current subterm to be the result of\u000A               first taking the 1st argument of the existing\u000A               current subterm, and then the 2nd argument of that</code> \u000A \u000A <p>For example, if the current subterm is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP_____B2\">+</see> a b) c),</code> \u000A \u000A <p>then after <v>(dive 1)</v> it is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP_____B2\">+</see> a b).</code> \u000A \u000A <p>If after that, then <v>(dive 2)</v> is invoked, the new current subterm will \u000A be</p> \u000A \u000A <code>b.</code> \u000A \u000A <p>Instead of <v>(dive 1)</v> followed by <v>(dive 2)</v>, the same current \u000A subterm could be obtained by instead submitting the single instruction \u000A <v>(dive 1 2)</v>.</p> \u000A \u000A <code>General Form:\u000A(dive &amp;rest naturals-list)</code> \u000A \u000A <p>If <v>naturals-list</v> is a non-empty list <v>(n_1 ... n_k)</v> of natural \u000A numbers, let the new current subterm be the result of selecting the \u000A <v>n_1</v>-st argument of the current subterm, and then the <v>n_2</v>-th subterm \u000A of that, ..., finally the <v>n_k</v>-th subterm.</p> \u000A \u000A <p><b>Remark:</b> <v>Dive</v> is related to the command <v>pp</v>, in that the \u000A diving is done according to raw (translated, internal form) syntax.  Use the \u000A command <v>dv</v> if you want to dive according to the syntax displayed by the \u000A command <v>p</v>.  Note that <v>(dv n)</v> can be abbreviated by simply \u000A <v>n</v>.</p>"],

 "ACL2____VERBOSE-PSTACK":[["Pstack"],"ACL2 Sources","ACL2","<p>NOTE: This utility is a low-level debugging utility, which may not \u000A be useful except to those familiar with ACL2 source code.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> t)   ; get trace-like information on prover during proofs\u000A(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> '(fn1 fn2 ...))\u000A                     ; as above, but omit calls of the indicated functions\u000A(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> nil) ; turn off trace-like information on prover</code> \u000A \u000A <p>For example, (verbose-pstack '(ev-fncall)) will provide a trace of various \u000A prover functions during proofs, except for the function <v>ev-fncall</v>.</p> \u000A \u000A <p>By evaluating <v>(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> t)</v> one can get trace-like information \u000A during subsequent proofs about prover functions, including time summaries, \u000A printed to the screen during a proof.  To turn off this feature, evaluate \u000A <v>(<see topic=\"ACL2____VERBOSE-PSTACK\">verbose-pstack</see> nil)</v>.  Also see <see topic=\"ACL2____PSTACK\">pstack</see>.</p>"],

 "ACL2____UNMEMOIZE":[["Events","Hons-and-memoization"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____UNMEMOIZE\">unmemoize</see> 'foo) ; turn off memoization of foo\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____UNMEMOIZE\">unmemoize</see> fn)</code> \u000A \u000A <p>where <v>fn</v> evaluates to a function symbol that is currently memoized; \u000A see <see topic=\"ACL2____MEMOIZE\">memoize</see>.  An exception is that as with <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>, <v>fn</v> may \u000A evaluate to the name of a macro that is associated with such a function \u000A symbol; see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>.</p> \u000A \u000A <p>Calls of this macro generate events of the form <v>(<see topic=\"ACL2____TABLE\">table</see> memoize-table fn\u000A nil)</v>.  When successful, the returned value is of the form <v>(<see topic=\"ACL2____MV\">mv</see> nil\u000A function-symbol state)</v>.</p> \u000A \u000A <p>To remove the effects of all <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt> <see topic=\"ACL2____EVENTS\">events</see>, evaluate: \u000A <v>(clear-memo-table)</v>.  To save and restore memoization, see <see topic=\"ACL2____SAVE-AND-CLEAR-MEMOIZATION-SETTINGS\">save-and-clear-memoization-settings</see> and see <see topic=\"ACL2____RESTORE-MEMOIZATION-SETTINGS\">restore-memoization-settings</see>.</p>"],

 "ACL2____TAKE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For any natural number <v>n</v> not exceeding the length of <v>l</v>, \u000A <v>(<see topic=\"ACL2____TAKE\">take</see> n l)</v> collects the first <v>n</v> elements of the list <v>l</v>.</p> \u000A \u000A <p>The following is a theorem (though it takes some effort, including lemmas, \u000A to get ACL2 to prove it):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> (<see topic=\"ACL2____TAKE\">take</see> n l)) (<see topic=\"ACL2____NFIX\">nfix</see> n))</code> \u000A \u000A <p>If <v>n</v> is an integer greater than the length of <v>l</v>, then <v>take</v> \u000A pads the list with the appropriate number of <v>nil</v> elements.  Thus, the \u000A following is also a theorem.</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A              (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l)\u000A              (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> l) n))\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____TAKE\">take</see> n l)\u000A                (<see topic=\"COMMON-LISP____APPEND\">append</see> l (<see topic=\"COMMON-LISP____MAKE-LIST\">make-list</see> (<see topic=\"COMMON-LISP____-\">-</see> n (<see topic=\"COMMON-LISP____LENGTH\">length</see> l))))))</code> \u000A \u000A <p>For related functions, see <see topic=\"COMMON-LISP____NTHCDR\">nthcdr</see> and see <see topic=\"COMMON-LISP____BUTLAST\">butlast</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"ACL2____TAKE\">take</see> n l)</v> is that <v>n</v> is a nonnegative \u000A integer and <v>l</v> is a true list.</p> \u000A \u000A <p><b>Function: </b><srclink>take</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> take (n l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n 0))\u000A                                   (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> l))))\u000A       (first-n-ac n l nil))</code>"],

 "ACL2____SYS-CALL-STATUS":[["Sys-call","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This function returns two values, <v>(<see topic=\"ACL2____MV\">mv</see> status state)</v>.  The \u000A first is the status resulting from the most recent invocation of function \u000A <v>sys-call</v>; see <see topic=\"ACL2____SYS-CALL\">sys-call</see>.  The second is the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> \u000A object, which is also the input to this function.</p> \u000A \u000A <p>The function <tt><see topic=\"ACL2____SYS-CALL\">sys-call</see></tt> makes a system call to the host operating \u000A system using a function supplied ``under the hood'' by the underlying Lisp \u000A system.  The status value is the value returned by that Lisp function, which \u000A may well be the numeric value returned by the host operating system for the \u000A underlying system call.  For more information, see <see topic=\"ACL2____SYS-CALL\">sys-call</see>.</p>"],

 "ACL2____MORE-DOC":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>NOTE: The <v>:more-doc</v> command only makes sense at the \u000A  terminal.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;:more-doc DEFTHM\u000AACL2 !&gt;:more-doc logical-name</code> \u000A \u000A <p>Often it is assumed in the text provided by <v>:more-doc</v> name that you \u000A have read the text provided by <v>:doc name</v>.</p> \u000A \u000A <p><v>:More-doc</v> just continues spewing out at you the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string provided with a definition.  If the user has done his job, <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> will probably remind you of the basics and <v>:more-doc</v>, if read after \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>, will address obscure details that are nevertheless worth \u000A noting.</p> \u000A \u000A <p>When <v>:more-doc</v> types ``<v>(<see topic=\"COMMON-LISP____TYPE\">type</see> :more for more, :more! for the\u000A rest)</v>'' you can get the next block of the continuation by typing \u000A <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> or all of the remaining blocks by typing <v>:</v><tt><see topic=\"ACL2____MORE_12\">more!</see></tt>.  See <see topic=\"ACL2____MORE\">more</see>.</p>"],

 "ACL2____HONS-ASSOC-EQUAL":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>The definition of <v>hons-assoc-equal</v> is similar to that of <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>, except that (1) it does not require <tt><see topic=\"ACL2____ALISTP\">alistp</see></tt> as a guard, \u000A and (2) it \"skips over\" any non-conses when its alist argument is \u000A malformed.</p> \u000A \u000A <p>We typically leave <v>hons-get</v> enabled and reason about \u000A <v>hons-assoc-equal</v> instead.  One benefit of this approach is that it avoids \u000A certain \"false\" discipline warnings that might arise from execution during \u000A theorem proving.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-assoc-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-assoc-equal (key alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> alist) nil)\u000A             ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A                   (<see topic=\"ACL2____HONS-EQUAL\">hons-equal</see> key (<see topic=\"COMMON-LISP____CAAR\">caar</see> alist)))\u000A              (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A             (t (<see topic=\"ACL2____HONS-ASSOC-EQUAL\">hons-assoc-equal</see> key (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)))))</code>"],

 "ACL2____FAST-ALIST-FORK_12":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically this function is just <v>fast-alist-fork</v>; we leave it enabled \u000A and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, this is the same as <v>fast-alist-fork</v> except that it \u000A uses something like <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt> instead of <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt>.  You \u000A generally <b>should not use</b> this function unless you really know what \u000A you're doing and understand the drawbacks discussed in <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt>.</p>"],

 "ACL2____F-GET-GLOBAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'y state) 1)\u000A(<see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see> 'a\u000A              (<see topic=\"ACL2____ASET1\">aset1</see> 'ascii-map-array\u000A                     (<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'a state)\u000A                     66\u000A                     'Upper-case-B)\u000A              state)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'symbol state)</code> \u000A \u000A <p>where <v>symbol</v> is any symbol to which you have <tt><see topic=\"ACL2____ASSIGN\">assign</see></tt>ed a global \u000A value.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2_____04\">@</see></tt> is closely related to <v>f-get-global</v>: <v>(<see topic=\"ACL2_____04\">@</see> var)</v> \u000A macroexpands to <v>(<see topic=\"ACL2____F-GET-GLOBAL\">f-get-global</see> 'var state)</v>.</p> \u000A \u000A <p>The macro <tt><see topic=\"ACL2____F-PUT-GLOBAL\">f-put-global</see></tt> makes it convenient to set the value of a \u000A symbol.  The <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> operation has no effect on the \u000A <v>global-table</v> of <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Thus, you may use these globals to hang \u000A onto useful data structures even though you may undo back past where you \u000A computed and saved them.</p>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-WITH-MULTIPLE-INDUCTION-STEPS":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Several Induction Steps</b>: To <v>(p x i a)</v> for all <v>x</v>, <v>i</v>, \u000A and <v>a</v>, prove each of the following:</p> \u000A \u000A <code><i>Base Case 1</i>: \u000A(implies (zp i) \u000A         (p x i a)) \u000A</code> \u000A \u000A <code><i>Induction Step 1</i>: \u000A(implies (and (not (zp i)) \u000A              (equal (parity i) 'even) \u000A              (p (* x x) \u000A                 (floor i 2) \u000A                 a)) \u000A         (p x i a)) \u000A</code> \u000A \u000A <code><i>Induction Step 2</i>: \u000A(implies (and (not (zp i)) \u000A              (not (equal (parity i) 'even)) \u000A              (p x \u000A                 (- i 1) \u000A                 (* x a))) \u000A         (p x i a)) \u000A</code> \u000A \u000A <p>A function that suggests this induction is the binary exponentiation \u000A function for natural numbers.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> bexpt (x i a)\u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> i) a)\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (parity i) 'even)\u000A         (bexpt (<see topic=\"COMMON-LISP_____A2\">*</see> x x)\u000A                (<see topic=\"COMMON-LISP____FLOOR\">floor</see> i 2)\u000A                a))\u000A        (t (bexpt x\u000A                  (<see topic=\"COMMON-LISP____-\">-</see> i 1)\u000A                  (<see topic=\"COMMON-LISP_____A2\">*</see> x a)\u000A                  )))).</code> \u000A \u000A <p>In order to admit this function it is necessary to know that <v>(<see topic=\"COMMON-LISP____FLOOR\">floor</see> i\u000A 2)</v> is smaller than <v>i</v> in the case above.  This can be proved if the \u000A community book <v>&quot;arithmetic-5/top&quot;</v> has been included from the ACL2 \u000A system directory, i.e.,</p> \u000A \u000A <code>(<see topic=\"ACL2____INCLUDE-BOOK\">include-book</see> &quot;arithmetic-5/top&quot; :dir :system)</code> \u000A \u000A <p>should be executed before defining <v>bexpt</v>.</p>"],

 "ACL2____DEFUNS":[["Mutual-recursion"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFUNS\">DEFUNS</see>\u000A (evenlp (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (oddlp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) t))\u000A (oddlp (x)\u000A   (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (evenlp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) nil)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFUNS\">DEFUNS</see> defuns-tuple1 ... defuns-tuplen)</code> \u000A \u000A <p>is equivalent to</p> \u000A \u000A <code>(<see topic=\"ACL2____MUTUAL-RECURSION\">MUTUAL-RECURSION</see>\u000A  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> . defuns-tuple1)\u000A  ...\u000A  (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> . defuns-tuplen))</code> \u000A \u000A <p>In fact, <v>defuns</v> is the more primitive of the two and <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> is just a macro that expands to a call of <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> \u000A after stripping off the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> at the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of each argument to \u000A <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt>.  We provide and use <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> rather \u000A than <v>defuns</v> because by leaving the <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>s in place, <tt><see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see></tt> forms can be processed by the Emacs <v>tags</v> program.  See \u000A <see topic=\"ACL2____MUTUAL-RECURSION\">mutual-recursion</see>.</p>"],

 "ACL2____DEFREFINEMENT":[["Events"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see> equiv1 equiv2)\u000A\u000Ais an abbreviation for\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> equiv1-refines-equiv2\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (equiv1 x y) (equiv2 x y))\u000A  :rule-classes (:refinement))</code> \u000A \u000A <p>See <see topic=\"ACL2____REFINEMENT\">refinement</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____DEFREFINEMENT\">defrefinement</see> equiv1 equiv2\u000A  :rule-classes rule-classes\u000A  :instructions instructions\u000A  :hints hints\u000A  :otf-flg otf-flg\u000A  :event-name event-name\u000A  :doc doc)</code> \u000A \u000A <p>where <v>equiv1</v> and <v>equiv2</v> are known <see topic=\"ACL2____EQUIVALENCE\">equivalence</see> relations, \u000A <v>event-name</v>, if supplied, is a symbol and all other arguments are as \u000A specified in the documentation for <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>.  The <v>defrefinement</v> \u000A macro expands into a call of <v>defthm</v>.  The name supplied is \u000A <v>equiv1-refines-equiv2</v>, unless <v>event-name</v> is supplied, in which case \u000A it is used as the name.  The term supplied states that <v>equiv1</v> refines \u000A <v>equiv2</v>.  The rule-class <v>:refinement</v> is added to the \u000A <v>rule-classes</v> specified, if it is not already there.  All other arguments \u000A to the generated <v>defthm</v> form are as specified by the other keyword \u000A arguments above.</p>"],

 "ACL2____ADD-MACRO-ALIAS":[["Macros"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> append binary-append)</code> \u000A \u000A <p>This example associates the function symbol <tt><see topic=\"ACL2____BINARY-APPEND\">binary-append</see></tt> with the \u000A macro name <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>.  As a result, the name <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt> may be used \u000A as a runic designator (see <see topic=\"ACL2____THEORIES\">theories</see>) by the various theory functions. \u000A See <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see> for more details.  Also see <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> \u000A for an extension of this utility that also affects printing.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see> macro-name function-name)</code> \u000A \u000A <p>This is a convenient way to add an entry to <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt>. \u000A See <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see> and also see <see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see>.</p>"],

 "ACL2-PC____WRAP1":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A; Keep (main . 1) and (main . 2) if they exist, as well as the current goal;\u000A; and for each other goal, conjoin it into the current goal and delete it:\u000A(wrap1 ((main . 1) (main . 2)))\u000A\u000A; As explained below, conjoin all subsequent siblings of the current goal\u000A; into the current goal, and then delete them:\u000A(wrap1)\u000A\u000AGeneral Form:\u000A(wrap1 &amp;optional kept-goal-names)</code> \u000A \u000A <p>If <v>kept-goal-names</v> is not <v>nil</v>, the current goal is replaced by \u000A conjoining it with all goals other than the current goal and those indicated \u000A by <v>kept-goal-names</v>, and those other goals are deleted.  If \u000A <v>kept-goal-names</v> is omitted, then the the current goal must be of the form \u000A <v>(<see topic=\"ACL2____NAME\">name</see> . n)</v>, and the goals to conjoin into the current goal (and delete) \u000A are those with names of the form <v>(<see topic=\"ACL2____NAME\">name</see> . k)</v> for <v>k</v> &gt;= <v>n</v>.</p> \u000A \u000A <p>NOTE: <v>Wrap1</v> always ``succeeds'', even if there are no other goals to \u000A conjoin into the current goal (a message is printed in that case), and it \u000A always leaves you with no hypotheses at the top of the current goal's \u000A conclusion (as though <v>top</v> and <v>demote</v> had been executed, if \u000A necessary).</p> \u000A \u000A <p>Also see proof-checker documentation for <v>wrap</v> (see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>).</p>"],

 "COMMON-LISP____STRING":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRING\">String</see> x)</v> <see topic=\"COMMON-LISP____COERCE\">coerce</see>s <v>x</v> to a string.  If <v>x</v> is \u000A already a string, then it is returned unchanged; if <v>x</v> is a symbol, then \u000A its <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is returned; and if <v>x</v> is a character, the \u000A corresponding one-character string is returned.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string</v> requires its argument to be a string, a \u000A symbol, or a character.</p> \u000A \u000A <p><v>String</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                                  (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A                                  (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x) x)\u000A             ((<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x) (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x))\u000A             (t (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____LIST\">list</see> x) 'string))))</code>"],

 "COMMON-LISP____COUNT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> #\\D &quot;DabcDefcDe&quot;)                 ; = 3\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> #\\D &quot;DabcDefcDe&quot; :start 1)        ; = 2\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> #\\D &quot;DabcDefcDe&quot; :start 1 :end 5) ; = 1\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> #\\D &quot;DabcDefcDe&quot; :start 1 :end 4) ; = 0\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> #\\z &quot;DabcDefcDe&quot;)                 ; = 0\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> '(a b) '(17 (a b) 23 (a b) (c d)))   ; = 2\u000A\u000AGeneral Form:\u000A(<see topic=\"COMMON-LISP____COUNT\">count</see> item sequence &amp;key start end)</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____COUNT\">Count</see> item sequence)</v> returns the number of times <v>item</v> occurs in \u000A <v>sequence</v>.  The <see topic=\"ACL2____GUARD\">guard</see> for calls of <v>count</v> (which is actually a \u000A macro in ACL2) specifies that <v>sequence</v> is a string or a true-list, and \u000A that <v>start</v>, which defaults to 0, and <v>end</v>, which defaults to the \u000A length of <v>sequence</v>, are valid indices into sequence.</p> \u000A \u000A <p>See any Common Lisp documentation for more information about <v>count</v>, \u000A which is a Common Lisp utility.  At this time ACL2 does not support keyword \u000A arguments for <v>count</v> other than <v>:start</v> and <v>:end</v>; we may add \u000A support for the <v>:from-end</v> keyword upon request.</p> \u000A \u000A <p><b>Macro: </b><srclink>count</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> count\u000A          (item sequence &amp;key (start '0) end)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 'count-fn\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> item\u000A                      (<see topic=\"COMMON-LISP____CONS\">cons</see> sequence\u000A                            (<see topic=\"COMMON-LISP____CONS\">cons</see> start (<see topic=\"COMMON-LISP____CONS\">cons</see> end 'nil))))))</code>"],

 "ACL2____SHOW-BODIES":[["Definition"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SHOW-BODIES\">show-bodies</see> foo)\u000A:show-bodies foo</code> \u000A \u000A <p>A definition made using <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> installs a so-called ``body'' of a \u000A function symbol, as do certain <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules.  Such bodies \u000A are used in a number of ways, including the application of <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>; see <see topic=\"ACL2____DEFINITION\">definition</see>, in particular the discussion of ``body'' there, \u000A and see <see topic=\"ACL2____HINTS\">hints</see> for a discussion of the <v>:expand</v> hint.  Also see \u000A <see topic=\"ACL2____SET-BODY\">set-body</see> for how to change which of the available definitions (among \u000A the original definition and appropriate <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> rules) is the \u000A one that provides the body.  The <v>show-bodies</v> command displays the \u000A available such bodies in an appropriate format, starting with the one that is \u000A currently used as the body.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"ACL2____SHOW-BODIES\">show-bodies</see> function-symbol)\u000A:show-bodies function-symbol</code>"],

 "ACL2____SETENV_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____SETENV_42\">Setenv$</see> str val)</v>, where <v>str</v> and <v>val</v> are strings, \u000A sets the environment variable <v>str</v> to have value <v>val</v>, for subsequent \u000A read by <v>getenv$</v> (see <see topic=\"ACL2____GETENV_42\">getenv$</see>), and returns <v>nil</v>.  Or, if this \u000A operation is not implemented for the host Common Lisp, an error will \u000A occur.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____SETENV_42\">setenv$</see> &quot;FOO&quot; &quot;BAR&quot;)</code> \u000A \u000A <p>It may be surprising that <v>setenv$</v> returns <v>nil</v>; indeed, it neither \u000A takes nor returns the ACL2 <see topic=\"ACL2____STATE\">state</see>.  The reason is that <tt><see topic=\"ACL2____GETENV_42\">getenv$</see></tt> \u000A takes responsibility for trafficking in <see topic=\"ACL2____STATE\">state</see>; it is defined in the \u000A logic using the function <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>, which (again, in the logic) \u000A does modify state, by popping an entry from its acl2-oracle field.  <see topic=\"ACL2____GETENV_42\">getenv$</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>setenv$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> setenv$ (str val)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> val))))\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> str val))\u000A       nil)</code>"],

 "ACL2____SET-REWRITE-STACK-LIMIT":[["Rewrite-stack-limit"],"ACL2 Sources","ACL2","<p>Note: This is an event!  It does not print the usual event summary \u000A but nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see> 30)                            ; set to small limit\u000A:set-rewrite-stack-limit 30                             ; same as above\u000A(<see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see> *default-rewrite-stack-limit*) ; the default\u000A(<see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see> (<see topic=\"COMMON-LISP____1-\">1-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 28)))              ; maximum legal limit\u000A:set-rewrite-stack-limit nil         ; same as above -- essentially, no limit</code> \u000A \u000A <p>This event sets the maximum stack depth for calls of certain functions that \u000A implement the ACL2 rewriter; see <see topic=\"ACL2____REWRITE-STACK-LIMIT\">rewrite-stack-limit</see>.  It must be a \u000A non-negative integer less than 2^28.  A call <v>(<see topic=\"ACL2____SET-REWRITE-STACK-LIMIT\">set-rewrite-stack-limit</see>\u000A limit)</v> is equivalent to:</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> acl2-defaults-table :rewrite-stack-limit limit).</code> \u000A \u000A <p>The use of <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt> ensures that this event's effect is \u000A implicitly <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form in which it \u000A occurs.</p> \u000A \u000A <p>For a different but somewhat related concept, see <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p>"],

 "ACL2____SET-IRRELEVANT-FORMALS-OK":[["Defun"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> t)\u000A(<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> nil)\u000A(<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> :warn)</code> \u000A \u000A <p>The first example above allows irrelevant formals in definitions; see <see topic=\"ACL2____IRRELEVANT-FORMALS\">irrelevant-formals</see>.  The second example disallows irrelevant formals; this is \u000A the default.  The third example allows irrelevant formals, but prints an \u000A appropriate warning.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded. \u000A Moreover, its effect is to set the <tt><see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see></tt>, and hence its \u000A effect is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the book or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> form containing it; \u000A see <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-IRRELEVANT-FORMALS-OK\">set-irrelevant-formals-ok</see> flg)</code> \u000A \u000A <p>where <v>flg</v> is either <v>t</v>, <v>nil</v>, or <v>:warn</v>.</p>"],

 "ACL2____Running_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Symbolic_02Execution_02of_02Models'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>Suppose the machine being modeled is some kind of arithmetic unit.  Suppose \u000A the model can be initialized so as to <b>multiply</b> <b>x</b> times <b>y</b> \u000A and leave the answer in <b>z</b>.  Then if we initialize <b>s</b> to \u000A <b>multiply</b> with <b>x=5</b> and <b>y=7</b> and run the machine long \u000A enough, we can read the answer <b>35</b> in the final state. \u000A \u000A  <img src='computing-machine-5x7.gif'></img></p> \u000A \u000A <p>Because ACL2 is a programming language, our model can be <b>run</b> or \u000A <b>executed</b>.</p> \u000A \u000A <p>If you defined the model in ACL2 and then typed</p> \u000A \u000A <code>(lookup 'z (mc (s 'mult 5 7) 29))</code> \u000A \u000A <p>then ACL2 would compute 35.  (Here we assume that the function <v>s</v> \u000A creates a state ready to run a given application on given inputs <v>x</v> and \u000A <v>y</v>.)  You can <b>emulate</b> or <b>test</b> the model of your machine.</p> \u000A \u000A <p>This is <b>obvious</b> because ACL2 is Common Lisp; and Common Lisp is a \u000A <b>programming language</b>.</p> \u000A \u000A <p><see topic='ACL2____Symbolic_02Execution_02of_02Models'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____REMOVE-OVERRIDE-HINTS":[["Override-hints"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for a discussion of override-hints.  Here \u000A we describe how to delete from the list of override-hints.  Note that the \u000A effects of <v>remove-override-hints</v> <see topic=\"ACL2____EVENTS\">events</see> are <see topic=\"ACL2____LOCAL\">local</see> to the \u000A <see topic=\"ACL2____BOOKS\">books</see> or <v>encapsulate</v> <see topic=\"ACL2____EVENTS\">events</see> in which they reside; see <see topic=\"ACL2____REMOVE-OVERRIDE-HINTS_12\">remove-override-hints!</see>  to avoid that restriction.  Also see <see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> and see <see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____REMOVE-OVERRIDE-HINTS\">remove-override-hints</see> form)</code> \u000A \u000A <p>where <v>form</v> should evaluate to a list of computed hint forms.  The \u000A effect of this event is to set the list of <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> to the result \u000A of deleting each element of the evaluation result from the <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see>, if that element indeed belongs to the override-hints; no \u000A check is made that these elements are actually elements of the existing \u000A override-hints.</p>"],

 "ACL2____PROOFS-CO":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Proofs-co</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>).  The \u000A accessor is <v>(<see topic=\"ACL2____PROOFS-CO\">proofs-co</see> state)</v> and the updater is <v>(set-proofs-co val\u000A state)</v>.  <v>Proofs-co</v> must be an open character output channel.  It is to \u000A this channel that <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt>, and the other event <see topic=\"ACL2____COMMAND\">command</see>s print their commentary.</p> \u000A \u000A <p>``Proofs-co'' stands for ``proofs character output.'' The initial value of \u000A <v>proofs-co</v> is the same as the value of <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt> (see <see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see>).</p>"],

 "ACL2____PKG-WITNESS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For any string <v>pkg</v> that names a package currently known to \u000A ACL2, <v>(<see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see> pkg)</v> is a symbol in that package whose <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> is the value of constant <v>*pkg-witness-name*</v>.  Logically, \u000A this is the case even if the package is not currently known to ACL2.  However, \u000A if <v>pkg-witness</v> is called on a string that is not the name of a package \u000A known to ACL2, a hard Lisp error will result.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____PKG-WITNESS\">Pkg-witness</see> pkg)</v> has a guard of <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> pkg) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> pkg\u000A &quot;&quot;)))</v>.  If <v>pkg</v> is not a string, then <v>(<see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see> pkg)</v> is equal \u000A to <v>(<see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see> &quot;ACL2&quot;)</v></p>"],

 "ACL2____PBT":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pbt :max      ; print back through the most recent command\u000A:pbt :x        ; print back through the most recent command\u000A:pbt fn        ; print back through the introduction of fn\u000A:pbt 5         ; print back through the fifth command executed\u000A:pbt (:x -4)   ; print back through the most recent five commands</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Pbt</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and prints the \u000A <see topic=\"ACL2____COMMAND\">command</see>s from <v>:max</v> (aka <v>:x</v>) through the one described.  See \u000A <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see> for a description of what a <see topic=\"ACL2____COMMAND\">command</see> \u000A descriptor is.  See <see topic=\"ACL2____PC\">pc</see> for a description of the format used to display \u000A <see topic=\"ACL2____COMMAND\">command</see>s.  <v>Pbt</v> will print the <see topic=\"ACL2____COMMAND\">command</see>s that <tt><see topic=\"ACL2____UBT\">ubt</see></tt> \u000A will undo.</p>"],

 "ACL2____ORACLE-FUNCALL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Oracle-funcall</v> evaluates its first argument to produce an ACL2 \u000A function symbol, and then applies that function symbol to the values of the \u000A rest of the arguments.  The return value is of the form <v>(<see topic=\"ACL2____MV\">mv</see> call-result\u000A state)</v>.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see> 'cons 3 4) ==&gt; (<see topic=\"ACL2____MV\">mv</see> '(3 . 4) &lt;state&gt;)\u000A(<see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see> (<see topic=\"COMMON-LISP____CAR\">car</see> '(<see topic=\"COMMON-LISP____FLOOR\">floor</see> foo bar)) (<see topic=\"COMMON-LISP_____B2\">+</see> 6 7) 5) ==&gt; (<see topic=\"ACL2____MV\">mv</see> 2 &lt;state&gt;)</code> \u000A \u000A <p><v>Oracle-funcall</v> is a macro; each of its calls macroexpands to a call of \u000A the related utility <v>oracle-apply</v> that takes the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> as an \u000A argument, as follows:</p> \u000A \u000A <code>(<see topic=\"ACL2____ORACLE-FUNCALL\">oracle-funcall</see> fn x1 x2 .. xk)</code> \u000A \u000A <p>macroexpands to</p> \u000A \u000A <code>(<see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> fn (<see topic=\"COMMON-LISP____LIST\">list</see> x1 x2 .. xk) state)</code> \u000A \u000A <p>Note that calls of <v>oracle-funcall</v> and <v>oracle-apply</v> return two \u000A values: the result of the function application, and a modified <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____ORACLE-APPLY\">oracle-apply</see> for details, including information about <see topic=\"ACL2____GUARD\">guard</see>s.</p> \u000A \u000A <p><b>Macro: </b><srclink>oracle-funcall</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> oracle-funcall (fn &amp;rest args)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 'oracle-apply\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> fn\u000A                      (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> 'list args)\u000A                            (<see topic=\"COMMON-LISP____CONS\">cons</see> 'state 'nil)))))</code>"],

 "ACL2____NOTE-3-0":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____NOTE-2-9-5\">note-2-9-5</see> for a description of changes since \u000A Version 2.9.4.  These include the new <tt><see topic=\"ACL2____MAKE-EVENT\">make-event</see></tt> feature, a soundness \u000A bug fix, an improvement for <v>:expand</v> <see topic=\"ACL2____HINTS\">hints</see>, evaluation in the logic \u000A by way of <v>:</v><tt><see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see></tt><v> :none</v>, and many other \u000A improvements.</p> \u000A \u000A <p>More generally, there have been several incremental releases since Version \u000A 2.9: see <see topic=\"ACL2____NOTE-2-9-1\">note-2-9-1</see>, see <see topic=\"ACL2____NOTE-2-9-2\">note-2-9-2</see>, see <see topic=\"ACL2____NOTE-2-9-3\">note-2-9-3</see>, see \u000A <see topic=\"ACL2____NOTE-2-9-4\">note-2-9-4</see>, and see <see topic=\"ACL2____NOTE-2-9-5\">note-2-9-5</see>.</p> \u000A \u000A <p>A very few users have contributed books following the instructions on the \u000A web.  We expect that when more contributions come in, we will give more \u000A attention to the question of how to organize the distributed and workshop \u000A books.  For now, we have simply added the new contributions according to the \u000A old-style distribution methodology.</p>"],

 "ACL2____MAXIMUM-LENGTH":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is an arbitrary object and <v>alist</v> is a 1- or \u000A 2-dimensional array.  This function returns the contents of the \u000A <v>:maximum-length</v> field of the <see topic=\"ACL2____HEADER\">header</see> of <v>alist</v>.  Whenever an \u000A <tt><see topic=\"ACL2____ASET1\">aset1</see></tt> or <tt><see topic=\"ACL2____ASET2\">aset2</see></tt> would cause the length of the alist to exceed \u000A its maximum length, a <tt><see topic=\"ACL2____COMPRESS1\">compress1</see></tt> or <tt><see topic=\"ACL2____COMPRESS2\">compress2</see></tt> is done \u000A automatically to remove irrelevant pairs from the array.  <v>Maximum-length</v> \u000A operates in virtually constant time if <v>alist</v> is the semantic value of \u000A <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>maximum-length</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     maximum-length (<see topic=\"ACL2____NAME\">name</see> l)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l) (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l))))\u000A     (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :maximum-length (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____HEADER\">header</see> name l)))))</code>"],

 "ACL2____GETENV_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____GETENV_42\">Getenv$</see> str state)</v>, where <v>str</v> is a string, reads the \u000A value of environment variable <v>str</v>, returning a value of <v>nil</v> if none \u000A is found or if the read fails.  The logical story is that <v>getenv$</v> reads \u000A its value from the <v>oracle</v> field of the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.  The return \u000A value is thus a triple of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>, where <v>erp</v> \u000A will always be <v>nil</v> in practice, and logically, <v>val</v> is the top of the \u000A acl2-oracle field of the state (see <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>) and the returned \u000A state has the updated (popped) acl2-oracle.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____GETENV_42\">getenv$</see> &quot;PWD&quot; state) ==&gt; (<see topic=\"ACL2____MV\">mv</see> nil &quot;/u/joe/work&quot; state)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____SETENV_42\">setenv$</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>getenv$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> getenv$ (str state)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state\u000A                       :guard (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str)))\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> str))\u000A       (<see topic=\"ACL2____READ-ACL2-ORACLE\">read-acl2-oracle</see> state))</code>"],

 "ACL2____EQLABLEP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>eqlablep</v> tests whether its argument is suitable \u000A for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>, at least one of whose arguments must satisfy this predicate \u000A in Common Lisp.  <v>(<see topic=\"ACL2____EQLABLEP\">Eqlablep</see> x)</v> is true if and only if its argument is a \u000A number, a symbol, or a character.</p> \u000A \u000A <p><b>Function: </b><srclink>eqlablep</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> eqlablep (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A           (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))</code>"],

 "ACL2____DIMENSIONS":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____DIMENSIONS\">dimensions</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DIMENSIONS\">dimensions</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is arbitrary and <v>alist</v> is a 1- or 2-dimensional array. \u000A This function returns the dimensions list of the array <v>alist</v>.  That list \u000A will either be of the form <v>(dim1)</v> or <v>(dim1 dim2)</v>, depending on \u000A whether <v>alist</v> is a 1- or 2-dimensional array.  <v>Dim1</v> and <v>dim2</v> \u000A will be integers and each exceed by 1 the maximum legal corresponding index. \u000A Thus, if <v>dimensions</v> returns, say, <v>'(100)</v> for an array <v>a</v> named \u000A <v>'delta1</v>, then <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'delta1 a 99)</v> is legal but <v>(<see topic=\"ACL2____AREF1\">aref1</see> 'delta1 a\u000A 100)</v> violates the <see topic=\"ACL2____GUARD\">guard</see>s on <tt><see topic=\"ACL2____AREF1\">aref1</see></tt>.  <v>Dimensions</v> operates \u000A in virtually constant time if <v>alist</v> is the semantic value of <v>name</v>. \u000A See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>dimensions</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     dimensions (<see topic=\"ACL2____NAME\">name</see> l)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l) (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l))))\u000A     (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :dimensions (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____HEADER\">header</see> name l)))))</code>"],

 "ACL2____DEFAULT-RULER-EXTENDERS":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>When a <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> is processed and no <v>:ruler-extenders</v> \u000A <v>xarg</v> is supplied, the function <v>default-ruler-extenders</v> is used to \u000A obtain the current ruler-extenders; see <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see>.  To find the \u000A default <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> of the current ACL2 <see topic=\"ACL2____WORLD\">world</see>, type \u000A <v>(<see topic=\"ACL2____DEFAULT-RULER-EXTENDERS\">default-ruler-extenders</see> (w state))</v>.</p> \u000A \u000A <p>While <see topic=\"ACL2____EVENTS\">events</see> that change the default <see topic=\"ACL2____RULER-EXTENDERS\">ruler-extenders</see> are \u000A permitted within an <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the text of a book, their effects \u000A are <tt><see topic=\"ACL2____LOCAL\">local</see></tt> in scope to the duration of the encapsulation or inclusion. \u000A See <see topic=\"ACL2____DEFAULT-DEFUN-MODE\">default-defun-mode</see> for an analogous discussion for \u000A defun-modes.</p>"],

 "ACL2____ACL2_02is_02an_02Untyped_02Language":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The example</p> \u000A \u000A <code>ACL2 !&gt;<b>(app '(a b c) 27)</b> \u000A(A B C . 27) \u000A</code> \u000A \u000A <p>illustrates the fact that ACL2's logic is untyped (click <see topic='ACL2____About_02Types'>here</see> for a brief discussion of the typed versus untyped \u000A nature of the logic).</p> \u000A \u000A <p>The definition of <v>app</v> makes no restriction of the arguments to lists. \u000A The definition says that if the first argument satisfies <tt><see topic=\"COMMON-LISP____ENDP\">endp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A then return the second argument.  In this example, when <v>app</v> has recursed \u000A three times down the <v>cdr</v> of its first argument, <v>'(a b c)</v>, it reaches \u000A the final <v>nil</v>, which satisfies <v>endp</v>, and so 27 is returned.  It is \u000A naturally consed into the emerging list as the function returns from \u000A successive recursive calls (since <v>cons</v> does not require its arguments to \u000A be lists, either).  The result is an ``improper'' list, <v>(a b c . 27)</v>.</p> \u000A \u000A <p>You can think of <v>(app x y)</v> as building a binary tree by replacing the \u000A right-most tip of the tree <v>x</v> with the tree <v>y</v>.</p>"],

 "ACL2-PC____PROVE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Aprove -- attempt to prove the current goal\u000A(prove :otf-flg t\u000A       :hints ((&quot;Subgoal 2&quot; :by foo) (&quot;Subgoal 1&quot; :use bar)))\u000A      -- attempt to prove the current goal, with the indicated hints\u000A         and with OTF-FLG set\u000A\u000AGeneral Form:\u000A(prove &amp;rest rest-args)</code> \u000A \u000A <p>Attempt to prove the current goal, where <v>rest-args</v> is as in the \u000A keyword arguments to <v>defthm</v> except that only <v>:hints</v> and \u000A <v>:otf-flg</v> are allowed.  The command succeeds exactly when the \u000A corresponding <v>defthm</v> would succeed, except that it is all right for some \u000A goals to be given ``bye''s.  Each goal given a ``bye'' will be turned into a \u000A new subgoal.  (See <see topic=\"ACL2____HINTS\">hints</see> for an explanation of <v>:by</v> hints.)</p> \u000A \u000A <p><b>Remark:</b> Use <v>(<see topic=\"COMMON-LISP_____D3\">=</see> t)</v> instead if you are not at the top of the \u000A conclusion.  Also note that if there are any hypotheses in the current goal, \u000A then what is actually attempted is a proof of <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps conc)</v>, where \u000A <v>hyps</v> is the conjunction of the top-level hypotheses and <v>conc</v> is the \u000A goal's conclusion.</p> \u000A \u000A <p><b>Remark:</b> It is allowed to use abbreviations in the hints.</p>"],

 "ACL2-PC____DEMOTE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Ademote        -- demote all top-level hypotheses\u000A(demote 3 5)  -- demote hypotheses 3 and 5</code> \u000A \u000A <p>For example, if the top-level hypotheses are <v>x</v> and <v>y</v> and the \u000A conclusion is <v>z</v>, then after execution of <v>demote</v>, the conclusion will \u000A be <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> x y) z)</v> and there will be no (top-level) hypotheses.</p> \u000A \u000A <code>General Form:\u000A(demote &amp;rest hyps-indices)</code> \u000A \u000A <p>Eliminate the indicated (top-level) hypotheses, but replace the conclusion \u000A <v>conc</v> with <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps conc)</v> where <v>hyps</v> is the conjunction of \u000A the hypotheses that were eliminated.  If no arguments are supplied, then all \u000A hypotheses are demoted, i.e. <v>demote</v> is the same as <v>(demote 1 2\u000A ... n)</v> where <v>n</v> is the number of top-level hypotheses.</p> \u000A \u000A <p><b>Remark</b>: You must be at the top of the conclusion in order to use \u000A this command.  Otherwise, first invoke <v>top</v>.  Also, <v>demote</v> fails if \u000A there are no top-level hypotheses or if indices are supplied that are out of \u000A range.</p>"],

 "COMMON-LISP____LOGAND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logand</v> returns their bitwise logical `and'.  In ACL2 <v>logand</v> is a \u000A macro that expands into calls of the binary function <v>binary-logand</v>, \u000A except that <v>(<see topic=\"COMMON-LISP____LOGAND\">logand</see>)</v> expands to <v>-1</v> and <v>(<see topic=\"COMMON-LISP____LOGAND\">logand</see> x)</v> expands to \u000A <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>binary-logand</v> requires its arguments to be \u000A integers.  <v>Logand</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>logand</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> logand (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> args) -1)\u000A                ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> args)) (<see topic=\"COMMON-LISP____CAR\">car</see> args))\u000A                (t (xxxjoin 'binary-logand args))))</code> \u000A <p><b>Function: </b><srclink>binary-logand</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-logand (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZIP\">zip</see> i) 0)\u000A             ((<see topic=\"ACL2____ZIP\">zip</see> j) 0)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> i -1) j)\u000A             ((<see topic=\"COMMON-LISP____EQL\">eql</see> j -1) i)\u000A             (t (<see topic=\"COMMON-LISP____LET\">let</see> ((x (<see topic=\"COMMON-LISP_____A2\">*</see> 2 (<see topic=\"COMMON-LISP____LOGAND\">logand</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> i 2) (<see topic=\"COMMON-LISP____FLOOR\">floor</see> j 2)))))\u000A                     (<see topic=\"COMMON-LISP_____B2\">+</see> x\u000A                        (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EVENP\">evenp</see> i) 0)\u000A                              ((<see topic=\"COMMON-LISP____EVENP\">evenp</see> j) 0)\u000A                              (t 1)))))))</code>"],

 "COMMON-LISP____COND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Cond</v> is the construct for IF, THEN, ELSE IF, ...  The test is \u000A against <v>nil</v>.  The argument list for <v>cond</v> is a list of ``clauses'', \u000A each of which is a list.  In ACL2, clauses must have length 1 or 2.</p> \u000A \u000A <p><v>Cond</v> is a Common Lisp macro.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>cond</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> cond (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> clauses)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (cond-clausesp clauses)))\u000A          (cond-macro clauses))</code> \u000A <p><b>Function: </b><srclink>cond-macro</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> cond-macro (clauses)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (cond-clausesp clauses)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> clauses)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses)) t)\u000A                    (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> clauses) nil))\u000A               (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses))\u000A                   (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses)))\u000A                   (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses)))\u000A               (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses))\u000A                   (<see topic=\"COMMON-LISP____LIST\">list</see> 'if\u000A                         (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses))\u000A                         (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses)))\u000A                         (cond-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> clauses)))\u000A                   (<see topic=\"COMMON-LISP____LIST\">list</see> 'or\u000A                         (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> clauses))\u000A                         (cond-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> clauses)))))\u000A           nil))</code>"],

 "COMMON-LISP____CHAR-DOWNCASE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR-DOWNCASE\">Char-downcase</see> x)</v> is equal to <v>#\\a</v> when <v>x</v> is \u000A <v>#\\A</v>, <v>#\\b</v> when <v>x</v> is <v>#\\B</v>, ..., and <v>#\\z</v> when <v>x</v> \u000A is <v>#\\Z</v>, and is <v>x</v> for any other character.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char-downcase</v> requires its argument to be a \u000A standard character (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Char-downcase</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char-downcase</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char-downcase (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> x))))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((pair (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x\u000A                          '((#\\A . #\\a)\u000A                            (#\\B . #\\b)\u000A                            (#\\C . #\\c)\u000A                            (#\\D . #\\d)\u000A                            (#\\E . #\\e)\u000A                            (#\\F . #\\f)\u000A                            (#\\G . #\\g)\u000A                            (#\\H . #\\h)\u000A                            (#\\I . #\\i)\u000A                            (#\\J . #\\j)\u000A                            (#\\K . #\\k)\u000A                            (#\\L . #\\l)\u000A                            (#\\M . #\\m)\u000A                            (#\\N . #\\n)\u000A                            (#\\O . #\\o)\u000A                            (#\\P . #\\p)\u000A                            (#\\Q . #\\q)\u000A                            (#\\R . #\\r)\u000A                            (#\\S . #\\s)\u000A                            (#\\T . #\\t)\u000A                            (#\\U . #\\u)\u000A                            (#\\V . #\\v)\u000A                            (#\\W . #\\w)\u000A                            (#\\X . #\\x)\u000A                            (#\\Y . #\\y)\u000A                            (#\\Z . #\\z)))))\u000A            (<see topic=\"COMMON-LISP____COND\">cond</see> (pair (<see topic=\"COMMON-LISP____CDR\">cdr</see> pair))\u000A                  ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) x)\u000A                  (t (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0)))))</code>"],

 "ACL2____WITH-GUARD-CHECKING":[["Guard"],"ACL2 Sources","ACL2","<code>Example:\u000A\u000A; Turn off all guard-checking for the indicated calls of append and car:\u000A(<see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> :none\u000A                     (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> 3 4)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____WITH-GUARD-CHECKING\">with-guard-checking</see> val form)</code> \u000A \u000A <p>where <v>val</v> evaluates to a legal guard-checking value (see <see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see>, or evaluate <v>*guard-checking-values*</v> to see the list \u000A of such values), and <v>form</v> is a form to be evaluated as though we had \u000A first executed <v>(<see topic=\"ACL2____SET-GUARD-CHECKING\">set-guard-checking</see> val)</v>.  Of course, this gaurd-checking \u000A setting is active only during evaluation of <v>form</v>, and is ignored once \u000A evaluation passes into raw Lisp functions (see <see topic=\"ACL2____GUARDS-AND-EVALUATION\">guards-and-evaluation</see>).</p>"],

 "ACL2____SET-OVERRIDE-HINTS":[["Override-hints"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for a discussion of override-hints.  Here \u000A we describe how to set them.  Note that the effects of <v>set-override-hints</v> \u000A <see topic=\"ACL2____EVENTS\">events</see> are <see topic=\"ACL2____LOCAL\">local</see> to the <see topic=\"ACL2____BOOKS\">books</see> or <v>encapsulate</v> <see topic=\"ACL2____EVENTS\">events</see> in which they reside; see <see topic=\"ACL2____SET-OVERRIDE-HINTS_12\">set-override-hints!</see> to avoid that \u000A restriction.  Also see <see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> to add to the list of \u000A override-hints, rather than setting a new list and ignoring the present \u000A list.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see> form)</code> \u000A \u000A <p>where <v>form</v> evaluates to a list of computed hint forms.  The effect of \u000A this event is to set the list of <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> to the result of that \u000A evaluation.</p>"],

 "ACL2____SET-DIFFERENCE-THEORIES":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                         '(fact (fact)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> th1 th2)</code> \u000A \u000A <p>where <v>th1</v> and <v>th2</v> are <see topic=\"ACL2____THEORIES\">theories</see> (see <see topic=\"ACL2____THEORIES\">theories</see>).  To \u000A each of the arguments there corresponds a runic theory.  This function returns \u000A the set-difference of those two runic <see topic=\"ACL2____THEORIES\">theories</see>, represented as a list \u000A and ordered chronologically.  That is, a <see topic=\"ACL2____RUNE\">rune</see> is in the result iff it \u000A is in the first runic theory but not in the second.</p> \u000A \u000A <p>The standard way to ``disable'' a theory, <v>lst</v>, is: <v>(<see topic=\"ACL2____IN-THEORY\">in-theory</see>\u000A (<see topic=\"ACL2____SET-DIFFERENCE-THEORIES\">set-difference-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here) lst))</v>.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____READ-RUN-TIME":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>By default, <v>(<see topic=\"ACL2____READ-RUN-TIME\">read-run-time</see> state)</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> runtime\u000A state)</v>, where runtime is the elapsed runtime in seconds since the start of \u000A the current ACL2 session and <v>state</v> is the resulting ACL2 <see topic=\"ACL2____STATE\">state</see>. \u000A But <v>read-run-time</v> can be made to return elapsed realtime (wall clock \u000A time) instead; see <see topic=\"ACL2____GET-INTERNAL-TIME\">get-internal-time</see>.</p> \u000A \u000A <p>The logical definition probably won't concern many users, but for \u000A completeness, we say a word about it here.  That definition uses the function \u000A <tt><see topic=\"ACL2____READ-ACL2-ORACLE\">read-ACL2-oracle</see></tt>, which modifies state by popping the value to return \u000A from its acl2-oracle field.</p> \u000A \u000A <p><b>Function: </b><srclink>read-run-time</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A   read-run-time (state-state)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (state-p1 state-state)))\u000A   (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____NULL\">null</see> (acl2-oracle state-state))\u000A                  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (acl2-oracle state-state)))))\u000A              0)\u000A             (t (<see topic=\"COMMON-LISP____CAR\">car</see> (acl2-oracle state-state))))\u000A       (update-acl2-oracle (<see topic=\"COMMON-LISP____CDR\">cdr</see> (acl2-oracle state-state))\u000A                           state-state)))</code>"],

 "ACL2____PRACTICE-FORMULATING-STRONG-RULES-4":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>What rules come to mind when looking at the following subterm of a \u000A Key Checkpoint?  Think of <i>strong</i> rules (see <see topic=\"ACL2____STRONG-REWRITE-RULES\">strong-rewrite-rules</see>).</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____SUBSETP\">SUBSETP</see> (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (FOO A) (BAR B)) (MUM C))</code> \u000A \u000A <p>When is <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> x y)</v> a subset of <v>z</v>?  When everything in <v>x</v> is \u000A in <v>z</v> and everything in <v>y</v> is in <v>z</v>.  We would make it a rewrite \u000A rule:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-1-strong\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x z)\u000A              (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> y z))))</code> \u000A \u000A <p>We put the ``<v>-1-</v>'' in the name because there is a comparable theorem \u000A for when the <v>append</v> is in the second argument of the <v>subsetp</v>; see \u000A <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES-5\">practice-formulating-strong-rules-5</see>.</p> \u000A \u000A <p>This strong rule is better than the conditional rule;</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-append-1-weak\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x z)\u000A                (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> y z))\u000A           (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> x y) z)))</code> \u000A \u000A <p>for all the usual reasons.</p> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____PRACTICE-FORMULATING-STRONG-RULES\">practice-formulating-strong-rules</see>.</p>"],

 "ACL2____PPROGN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____PPROGN\">pprogn</see>\u000A (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) #\\) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) slash-char))\u000A        (<see topic=\"ACL2____PRINC_42\">princ$</see> #\\ channel state))\u000A       (t state))\u000A (<see topic=\"ACL2____PRINC_42\">princ$</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l) channel state)\u000A (<see topic=\"ACL2____MV\">mv</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l) state))</code> \u000A \u000A <p>The convention for <v>pprogn</v> usage is to give it a non-empty sequence of \u000A forms, each of which (except possibly for the last) returns state (see <see topic=\"ACL2____STATE\">state</see>) as its only value.  The <see topic=\"ACL2____STATE\">state</see> returned by each but the last is \u000A passed on to the next.  The value or values of the last form are returned as \u000A the value of the <v>pprogn</v>.</p> \u000A \u000A <p>If you are using single-threaded objects you may wish to define an analogue \u000A of this function for your own <see topic=\"ACL2____STOBJ\">stobj</see>.</p> \u000A \u000A <p>General Form:</p> \u000A \u000A <code>(<see topic=\"ACL2____PPROGN\">PPROGN</see> form1\u000A        form2\u000A        ...\u000A        formk\u000A        result-form)</code> \u000A \u000A <p>This general form is equivalent, via macro expansion, to:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">LET</see> ((<see topic=\"ACL2____STATE\">STATE</see> form1))\u000A     (<see topic=\"COMMON-LISP____LET\">LET</see> ((<see topic=\"ACL2____STATE\">STATE</see> form2))\u000A          ...\u000A          (<see topic=\"COMMON-LISP____LET\">LET</see> ((<see topic=\"ACL2____STATE\">STATE</see> formk))\u000A               result-form)))</code>"],

 "ACL2____NOTE-2-7-RULES":[["Note-2-7"],"ACL2 Sources","ACL2","<p>The <tt><see topic=\"ACL2____DEFCONG\">defcong</see></tt> macro has been slightly changed.  The \u000A difference is that the variable generated with suffix <v>-EQUIV</v> will now be \u000A in the same package as the name of the variable from which it is generated, \u000A rather than always belonging to the ACL2 package.  Thanks to Hanbing Liu for \u000A suggesting this change.  (Note that a couple of books have been modified to \u000A accommodate this change, e.g., <v>books/finite-set-theory/set-theory</v>.)</p> \u000A \u000A <p>In Version_2.6, a change was made for rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> \u000A whose conclusion is a term of the form <v>(EQV lhs rhs)</v>, where <v>EQV</v> is \u000A <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt>, <tt><see topic=\"COMMON-LISP____EQ\">eq</see></tt>, or <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>: the rule was stored as though <v>EQV</v> \u000A were <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.  (See <see topic=\"ACL2____NOTE-2-6-RULES\">note-2-6-rules</see>.)  This change has been \u000A extended to rules of class <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt>.</p>"],

 "ACL2____MINIMAL-THEORY":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<p>This <tt><see topic=\"ACL2____THEORY\">theory</see></tt> (see <see topic=\"ACL2____THEORIES\">theories</see>) enables only a few \u000A built-in functions and executable counterparts.  It can be useful when you \u000A want to formulate lemmas that rather immediately imply the theorem to be \u000A proved, by way of a <v>:use</v> hint (see <see topic=\"ACL2____HINTS\">hints</see>), for example as \u000A follows.</p> \u000A \u000A <code>:use (lemma-1 lemma-2 lemma-3)\u000A:in-theory (<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> '(f1 f2) (<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory))</code> \u000A \u000A <p>In this example, we expect the current goal to follow from lemmas \u000A <v>lemma-1</v>, <v>lemma-2</v>, and <v>lemma-3</v> together with rules <v>f1</v> and \u000A <v>f2</v> and some obvious facts about built-in functions (such as the <see topic=\"ACL2____DEFINITION\">definition</see> of <tt><see topic=\"ACL2____IMPLIES\">implies</see></tt> and the <v>:</v><tt><see topic=\"ACL2____EXECUTABLE-COUNTERPART\">executable-counterpart</see></tt> of \u000A <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>).  The <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint above is intended to speed up \u000A the proof by turning off all inessential rules.</p>"],

 "ACL2____IFF":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Iff</v> is the ACL2 biconditional, ``if and only if''.  <v>(<see topic=\"ACL2____IFF\">iff</see> P\u000A Q)</v> means that either <v>P</v> and <v>Q</v> are both false (i.e., <v>nil</v>) or \u000A both true \u000A (i.e., not <v>nil</v>).</p> \u000A \u000A <p><b>Function: </b><srclink>iff</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> iff (p q)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> p (<see topic=\"COMMON-LISP____IF\">if</see> q t nil) (<see topic=\"COMMON-LISP____IF\">if</see> q nil t)))</code>"],

 "ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions_02_82cont_92'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Most ACL2 primitives are documented.  Here is the definition of <v>app</v> \u000A again, with the documented topics highlighted.  <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A All of the links below lead into the ACL2 User's Manual.  So follow these \u000A links if you wish, but use your <b>Back Button</b> to return here!</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> app (x y) \u000A  (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) y) \u000A        (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) \u000A                 (app (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))))) \u000A</code> \u000A \u000A <p>By following the link on <see topic=\"COMMON-LISP____ENDP\">endp</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> we \u000A see that it is a Common Lisp function and is defined to be the same as <see topic=\"COMMON-LISP____ATOM\">atom</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>, which recognizes non-conses.  But <v>endp</v> has a \u000A guard.  Since we are ignorning guards for now, we'll ignore the guard issue on \u000A <v>endp</v>.</p> \u000A \u000A <p>So this definition reads ``to <v>app</v> <v>x</v> and <v>y</v>: if <v>x</v> is an \u000A atom, return <v>y</v>; otherwise, <v>app</v> <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> x)</v> and <v>y</v> and then cons \u000A <v>(<see topic=\"COMMON-LISP____CAR\">car</see> x)</v> onto that.''</p> \u000A \u000A <p><see topic='ACL2____How_02To_02Find_02Out_02about_02ACL2_02Functions_02_82cont_92'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____How_02Long_02Does_02It_02Take_02to_02Become_02an_02Effective_02User_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Other_02Requirements'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>We expect that a talented undergraduate majoring in computer science (or \u000A perhaps mathematics) will probably take several weeks to become an effective \u000A ACL2 user.  The time will depend, of course, on that student's familiarity \u000A with logic (or formal methods) and Lisp programming, as well as willingness to \u000A read and study the ACL2 User's Manual.</p> \u000A \u000A <p>Of course, it is critical to do some projects in order to gain proficiency. \u000A (Hence access to an ACL2 implementation is also a requirement, for example by \u000A downloading and installing following links from the ACL2 home page.)  But it \u000A is critical to start with ``toy'' projects before tackling a ``grand \u000A challenge.''</p> \u000A \u000A <p><see topic='ACL2____Other_02Requirements'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____HONS-COPY-PERSISTENT":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically <v>hons-copy-persistent</v> is the identity; we leave it enabled \u000A and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, <v>hons-copy-persistent</v> is virtually identical to <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt>.</p> \u000A \u000A <p>The only difference is that when <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt> is used, any \u000A persistently normed conses are automatically re-normed, and this re-norming \u000A can be carried out more efficiently than, say, an ordinary <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>hons-copy-persistent</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> hons-copy-persistent (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "ACL2____GC_42":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>This function is provided so that the user can call the garbage \u000A collector of the host Lisp from inside the ACL2 loop.  Specifically, a call of \u000A <v>gc$</v> is translated into a call of a function below on the same \u000A arguments.</p> \u000A \u000A <code>Allegro CL:            excl:gc\u000ACCL                    ccl::gc\u000ACLISP                  ext:gc\u000ACMU Common Lisp        system::gc\u000AGCL                    si::gbc [default argument list: (t)]\u000ALispWorks              hcl::gc-generation [default argument list:\u000A                                           (7) for 64-bit OS, else (3)]\u000ASBCL                   sb-ext:gc</code> \u000A \u000A <p>The arguments, if any, are as documented in the underlying Common Lisp.  It \u000A is up to the user to pass in the right arguments, although we may do some \u000A rudimentary checks.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____GC-VERBOSE\">gc-verbose</see>.</p> \u000A \u000A <p>Evaluation of a call of this macro always returns <v>nil</v>.</p>"],

 "ACL2____GC-VERBOSE":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____GC-VERBOSE\">gc-verbose</see> arg)</code> \u000A \u000A <p>Garbage collection (gc) is performed by every Lisp implementation; see \u000A <see topic=\"ACL2____GC_42\">gc$</see>.  However, different ACL2 builds might see more or fewer messages. \u000A This macro is intended to provide an interface for controlling the verbosity, \u000A which is off if the argument evaluates to <v>nil</v> and otherwise is on.</p> \u000A \u000A <p>The above functionality is only supported for the following host Common \u000A Lisp implementations: CCL, CMUCL, and GCL.  Otherwise, the only effect of this \u000A macro is to print a notice that it is not supported.  You are welcome to \u000A contact the ACL2 developers if you would like to help in adding such support \u000A for another host Common Lisp.</p> \u000A \u000A <p>Evaluation of a call of this macro always returns <v>nil</v>.</p>"],

 "ACL2____FAST-ALIST-SUMMARY":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, <v>fast-alist-summary</v> just returns nil; we leave it enabled \u000A and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, this function gathers and prints some basic statistics \u000A about the current fast alists.  It may be useful for identifying fast alists \u000A that should have been freed with <tt><see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>fast-alist-summary</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fast-alist-summary\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____DEFAULT":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____DEFAULT\">default</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFAULT\">default</see> name alist)</code> \u000A \u000A <p>where <v>name</v> is an arbitrary object and <v>alist</v> is a 1- or \u000A 2-dimensional array.  This function returns the contents of the <v>:default</v> \u000A field of the <see topic=\"ACL2____HEADER\">header</see> of <v>alist</v>.  When <tt><see topic=\"ACL2____AREF1\">aref1</see></tt> or <tt><see topic=\"ACL2____AREF2\">aref2</see></tt> \u000A is used to obtain a value for an index (or index pair) not bound in \u000A <v>alist</v>, the default value is returned instead.  Thus, the array <v>alist</v> \u000A may be thought of as having been initialized with the default value. \u000A <v>default</v> operates in virtually constant time if <v>alist</v> is the semantic \u000A value of <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>default</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     default (<see topic=\"ACL2____NAME\">name</see> l)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ARRAY1P\">array1p</see> name l) (<see topic=\"ACL2____ARRAY2P\">array2p</see> name l))))\u000A     (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :default (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____HEADER\">header</see> name l)))))</code>"],

 "ACL2____CPU-CORE-COUNT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting parallel execution and proof; see <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p> \u000A \u000A <p>Unless the ACL2 executable supports parallel execution (see <see topic=\"ACL2____PARALLELISM\">parallelism</see>), this function returns <v>(<see topic=\"ACL2____MV\">mv</see> 1 state)</v>.  Otherwise:</p> \u000A \u000A <p><v>(<see topic=\"ACL2____CPU-CORE-COUNT\">Cpu-core-count</see> state)</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> core-count state)</v>, where \u000A <v>core-count</v> is the number of cpu cores if ACL2 can get that information \u000A from the underlying Common Lisp implementation.  Otherwise an error occurs, \u000A unless global <v>'cpu-core-count</v> is assigned to a positive integer value \u000A (see <see topic=\"ACL2____ASSIGN\">assign</see>), in which case that value is returned as the \u000A <v>core-count</v>.</p> \u000A \u000A <code>Example:\u000A(<see topic=\"ACL2____CPU-CORE-COUNT\">cpu-core-count</see> state) ==&gt; (<see topic=\"ACL2____MV\">mv</see> 4 state)</code> \u000A \u000A <p><b>Function: </b><srclink>cpu-core-count</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> cpu-core-count (<see topic=\"ACL2____STATE\">state</see>)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :stobjs state :guard t))\u000A       (<see topic=\"ACL2____MV-LET\">mv-let</see> (nullp val state)\u000A               (<see topic=\"ACL2____READ-ACL2-ORACLE\">read-acl2-oracle</see> state)\u000A               (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> nullp))\u000A               (<see topic=\"ACL2____MV\">mv</see> val state)))</code>"],

 "ACL2____CHANGE":[["Defrec"],"ACL2 Sources","ACL2","<p>The <v>change</v> macro is built into ACL2, and allows you to \u000A \"modify\" instances of structures that have been introduced with <see topic=\"ACL2____DEFREC\">defrec</see>.  Of course, since ACL2 is applicative, the original structure is not \u000A actually changed&mdash;instead, a new structure is constructed, copying some \u000A fields from the original structure and installing new values for other \u000A fields.</p> \u000A \u000A <p>For instance, suppose we first use <see topic=\"ACL2____MAKE\">make</see> to create an employee \u000A structure, e.g.,:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFCONST\">defconst</see> *jimmy* (<see topic=\"ACL2____MAKE\">make</see> employee :name &quot;Jimmy&quot;\u000A                                 :salary 0\u000A                                 :position &quot;Unpaid Intern&quot;)</code> \u000A \u000A <p>Then we can use <v>change</v> to mutate this structure, e.g.,:</p> \u000A \u000A <code>(<see topic=\"ACL2____CHANGE\">change</see> employee *jimmy* :salary 300000\u000A                         :position &quot;Vice President&quot;)</code> \u000A \u000A <p>Produces a new <v>employee</v> structure where the <v>name</v> is still \u000A \"Jimmy\", but where the <v>salary</v> and <v>position</v> have been updated to \u000A 300000 and \"Vice President\", respectively.</p> \u000A \u000A <p>See <see topic=\"ACL2____DEFREC\">defrec</see> for more information.</p>"],

 "ACL2____BINARY-APPEND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This binary function implements <tt><see topic=\"COMMON-LISP____APPEND\">append</see></tt>, which is a macro in \u000A ACL2.  See <see topic=\"COMMON-LISP____APPEND\">append</see></p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>binary-append</v> requires the first argument to be a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>binary-append</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-append (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) y)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                      (<see topic=\"ACL2____BINARY-APPEND\">binary-append</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y)))))</code>"],

 "COMMON-LISP____ZEROP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ZEROP\">zerop</see> x)</v> is <v>t</v> if <v>x</v> is <v>0</v> and is <v>nil</v> \u000A otherwise.  Thus, it is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____ZEROP\">Zerop</see> x)</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring <v>x</v> to be numeric and can be \u000A expected to execute more efficiently than <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)</v> in properly <see topic=\"ACL2____GUARD\">guard</see>ed compiled code.</p> \u000A \u000A <p>In recursions down the natural numbers, <v>(<see topic=\"ACL2____ZP\">zp</see> x)</v> is preferred over \u000A <v>(<see topic=\"COMMON-LISP____ZEROP\">zerop</see> x)</v> because the former coerces <v>x</v> to a natural and allows the \u000A termination proof.  In recursions through the integers, <v>(<see topic=\"ACL2____ZIP\">zip</see> x)</v> is \u000A preferred.  See <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>Zerop</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>zerop</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> zerop (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)))\u000A       (<see topic=\"COMMON-LISP____EQL\">eql</see> x 0))</code>"],

 "COMMON-LISP____SUBSTITUTE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____SUBSTITUTE\">Substitute</see> new old seq)</v> is the result of replacing each \u000A occurrence of <v>old</v> in <v>seq</v>, which is a list or a string, with \u000A <v>new</v>.</p> \u000A \u000A <p>The guard for <v>substitute</v> requires that either <v>seq</v> is a string and \u000A <v>new</v> is a character, or else: <v>seq</v> is a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> such that \u000A either all of its members are <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt> or <v>old</v> is \u000A <v>eqlablep</v>.</p> \u000A \u000A <p><v>Substitute</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.  Since ACL2 functions cannot take keyword \u000A arguments (though macros can), the test used in <v>substitute</v> is \u000A <v>eql</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>substitute</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     substitute (new old seq)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> new))\u000A                                (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> seq)\u000A                                     (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> old)\u000A                                         (<see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see> seq))))))\u000A     (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> seq)\u000A         (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (substitute-ac new old (<see topic=\"COMMON-LISP____COERCE\">coerce</see> seq 'list)\u000A                                nil)\u000A                 'string)\u000A         (substitute-ac new old seq nil)))</code>"],

 "COMMON-LISP____MAX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____MAX\">Max</see> x y)</v> is the larger of the numbers <v>x</v> and <v>y</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>max</v> requires its arguments to be rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers.</p> \u000A \u000A <p><v>Max</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>max</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> max (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x y) x y))</code>"],

 "COMMON-LISP____CHAR-UPCASE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR-UPCASE\">Char-upcase</see> x)</v> is equal to <v>#\\A</v> when <v>x</v> is \u000A <v>#\\a</v>, <v>#\\B</v> when <v>x</v> is <v>#\\b</v>, ..., and <v>#\\Z</v> when <v>x</v> \u000A is <v>#\\z</v>, and is <v>x</v> for any other character.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char-upcase</v> requires its argument to be a \u000A standard character (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Char-upcase</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char-upcase</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char-upcase (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> x))))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((pair (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> x\u000A                          '((#\\a . #\\A)\u000A                            (#\\b . #\\B)\u000A                            (#\\c . #\\C)\u000A                            (#\\d . #\\D)\u000A                            (#\\e . #\\E)\u000A                            (#\\f . #\\F)\u000A                            (#\\g . #\\G)\u000A                            (#\\h . #\\H)\u000A                            (#\\i . #\\I)\u000A                            (#\\j . #\\J)\u000A                            (#\\k . #\\K)\u000A                            (#\\l . #\\L)\u000A                            (#\\m . #\\M)\u000A                            (#\\n . #\\N)\u000A                            (#\\o . #\\O)\u000A                            (#\\p . #\\P)\u000A                            (#\\q . #\\Q)\u000A                            (#\\r . #\\R)\u000A                            (#\\s . #\\S)\u000A                            (#\\t . #\\T)\u000A                            (#\\u . #\\U)\u000A                            (#\\v . #\\V)\u000A                            (#\\w . #\\W)\u000A                            (#\\x . #\\X)\u000A                            (#\\y . #\\Y)\u000A                            (#\\z . #\\Z)))))\u000A            (<see topic=\"COMMON-LISP____COND\">cond</see> (pair (<see topic=\"COMMON-LISP____CDR\">cdr</see> pair))\u000A                  ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) x)\u000A                  (t (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0)))))</code>"],

 "ACL2____UNMONITOR":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____UNMONITOR\">unmonitor</see> '(:rewrite assoc-of-app))\u000A:unmonitor (:rewrite assoc-of-app)\u000A:unmonitor :all\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____UNMONITOR\">unmonitor</see> rune)\u000A(<see topic=\"ACL2____UNMONITOR\">unmonitor</see> :all)</code> \u000A \u000A <p>Here, <v>rune</v> is a <see topic=\"ACL2____RUNE\">rune</see> that is currently among those with break \u000A points installed.  This function removes the break.</p> \u000A \u000A <p>Subtle point: Because you may want to unmonitor a ``<see topic=\"ACL2____RUNE\">rune</see>'' that is \u000A no longer a <see topic=\"ACL2____RUNE\">rune</see> in the current ACL2 <see topic=\"ACL2____WORLD\">world</see>, we don't actually \u000A check this about <see topic=\"ACL2____RUNE\">rune</see>.  Instead, we simply check that <see topic=\"ACL2____RUNE\">rune</see> is a \u000A <v>consp</v> beginning with a <v>keywordp</v>.  That way, you'll know you've made \u000A a mistake if you try to <v>:unmonitor binary-append</v> instead of <v>:unmonitor\u000A (:definition binary-append)</v>, for example.</p>"],

 "ACL2____UNARY--":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-unary-minus</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UNARY--\">unary--</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"ACL2____UNARY--\">unary--</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____UNARY--\">unary--</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)</code> \u000A \u000A <p>Notice that like all arithmetic functions, <v>unary--</v> treats non-numeric \u000A inputs as <v>0</v>.</p> \u000A \u000A <p>Calls of the macro <tt><see topic=\"COMMON-LISP____-\">-</see></tt> on one argument expand to calls of \u000A <v>unary--</v>; see <see topic=\"COMMON-LISP____-\">-</see>.</p>"],

 "ACL2____The_02Admission_02of_02App":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Revisiting_02the_02Admission_02of_02App'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Here is what it looks like to submit the definition of <v>app</v> to \u000A ACL2:</p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code>ACL2 !&gt;<b>(defun app (x y)</b> \u000A  <b>(cond ((endp x) y)</b> \u000A        <b>(t (cons (car x) </b> \u000A                 <b>(app (cdr x) y)))))</b> \u000A \u000AThe admission of APP is trivial, using the relation O&lt; (which \u000Ais known to be well-founded on the domain recognized by O-P) \u000Aand the measure (ACL2-COUNT X).  We observe that the type of APP is \u000Adescribed by the theorem (OR (CONSP (APP X Y)) (EQUAL (APP X Y) Y)). \u000AWe used primitive type reasoning. \u000A \u000ASummary \u000AForm:  ( DEFUN APP ...) \u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL)) \u000AWarnings:  None \u000ATime:  0.03 seconds (prove: 0.00, print: 0.00, other: 0.03) \u000A APP \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>The text between the lines above is one interaction with the ACL2 command \u000A loop.  Interacting with the latest version of ACL2 may not produce the very \u000A same output, but we trust you'll recognize the basics.</p> \u000A \u000A <p>Above you see the user's <b>input</b> and how the system responds.  This \u000A little example shows you what the syntax looks like and is a very typical \u000A <b>successful</b> interaction with the definitional principle.</p> \u000A \u000A <p>Let's look at it a little more closely.</p> \u000A \u000A <p><see topic='ACL2____Revisiting_02the_02Admission_02of_02App'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____SET-FMT-HARD-RIGHT-MARGIN":[["Io"],"ACL2 Sources","ACL2","<p>In this documentation topic we discuss setting of both a ``soft'' \u000A and a ``hard'' right margin.</p> \u000A \u000A <code>Example Forms:\u000A(<see topic=\"ACL2____SET-FMT-SOFT-RIGHT-MARGIN\">set-fmt-soft-right-margin</see> 55 state) ; set soft right margin to 55\u000A(<see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see> 68 state) ; set hard right margin to 68</code> \u000A \u000A <p><tt><see topic=\"ACL2____FMT\">Fmt</see></tt> and related functions insert linebreaks when lines get too \u000A long.  A linebreak is inserted at an aesthetically appropriate point once the \u000A column exceeds the value of <v>(<see topic=\"ACL2_____04\">@</see> fmt-soft-right-margin)</v>.  If however the \u000A column exceeds the value of <v>(<see topic=\"ACL2_____04\">@</see> fmt-hard-right-margin)</v>, then a linebreak \u000A is soon inserted.  Such a ``hard'' linebreak follows the insertion of a \u000A backslash (<v>\\</v>) character unless <tt><see topic=\"ACL2____FMT_12\">fmt!</see></tt>, <tt><see topic=\"ACL2____FMS_12\">fms!</see></tt>, or <tt><see topic=\"ACL2____FMT1_12\">fmt1!</see></tt> is used, or state global <v>write-for-read</v> is true.</p>"],

 "ACL2____NOTE-2-6-PROOF-CHECKER":[["Note-2-6"],"ACL2 Sources","ACL2","<p>The proof-checker command <v>=</v>, when used with no arguments, now \u000A reports which hypothesis is being used.</p> \u000A \u000A <p>The output from <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>type-alist</v> has been \u000A improved.</p> \u000A \u000A <p>A slight change has been made to the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> for commands \u000A <v>promote</v>, <v>casesplit</v>, <v>equiv</v>, and <v>=</v>, so that terms of the \u000A form <v>(<see topic=\"COMMON-LISP____IF\">if</see> x nil y)</v> are recognized as conjunctions, <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____NOT\">not</see> x) y)</v>. \u000A Thanks to Pete Manolios for suggesting that we consider such a change.</p> \u000A \u000A <p>There is a new <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command <v>print-all-concs</v> that \u000A prints all the conclusions of the unproved goals.</p> \u000A \u000A <p>A new <tt><see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see></tt> command, <v>runes</v>, has been added.  It \u000A reports the <see topic=\"ACL2____RUNE\">rune</see>s that have participated in the interactive proof up to \u000A the current point.</p>"],

 "ACL2____NONNEGATIVE-INTEGER-QUOTIENT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> 14 3) ; equals 4\u000A(<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> 15 3) ; equals 5</code> \u000A \u000A <p><v>(<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> i j)</v> returns the integer quotient of the \u000A integers <v>i</v> and (non-zero) <v>j</v>, i.e., the largest <v>k</v> such that \u000A <v>(<see topic=\"COMMON-LISP_____A2\">*</see> j k)</v> is less than or equal to <v>i</v>.  Also see <see topic=\"COMMON-LISP____FLOOR\">floor</see>, see \u000A <see topic=\"COMMON-LISP____CEILING\">ceiling</see> and see <see topic=\"COMMON-LISP____TRUNCATE\">truncate</see>, which are derived from this function \u000A and apply to rational numbers.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> i j)</v> requires that \u000A <v>i</v> is a nonnegative integer and <v>j</v> is a positive integer.</p> \u000A \u000A <p><b>Function: </b><srclink>nonnegative-integer-quotient</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nonnegative-integer-quotient (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> i 0))\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                                   (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 j))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP_____D3\">=</see> (<see topic=\"ACL2____NFIX\">nfix</see> j) 0) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"ACL2____IFIX\">ifix</see> i) j))\u000A           0\u000A           (<see topic=\"COMMON-LISP_____B2\">+</see> 1\u000A              (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> (<see topic=\"COMMON-LISP____-\">-</see> i j)\u000A                                            j))))</code>"],

 "ACL2____LEXORDER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Lexorder</v> is a non-strict total order, a ``less than or \u000A equal,'' on ACL2 objects.  Also see <see topic=\"ACL2____ALPHORDER\">alphorder</see>, the restriction of \u000A <v>lexorder</v> to atoms; the notion of ``non-strict total order'' is defined \u000A there.</p> \u000A \u000A <p><v>Lexorder</v> has a guard of <v>t</v>.</p> \u000A \u000A <p>For <v>lexorder</v>, an <see topic=\"COMMON-LISP____ATOM\">atom</see> and a <see topic=\"COMMON-LISP____CONS\">cons</see> are ordered so that the \u000A <see topic=\"COMMON-LISP____ATOM\">atom</see> comes first, and two <see topic=\"COMMON-LISP____CONS\">cons</see>es are ordered so that the one \u000A with the recursively smaller <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> comes first, with the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s \u000A being compared only if the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>s are equal.  <v>Lexorder</v> compares two \u000A atoms by using <tt><see topic=\"ACL2____ALPHORDER\">alphorder</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>lexorder</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> lexorder (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> y) (<see topic=\"ACL2____ALPHORDER\">alphorder</see> x y)) (t t)))\u000A             ((<see topic=\"COMMON-LISP____ATOM\">atom</see> y) nil)\u000A             ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> y))\u000A              (<see topic=\"ACL2____LEXORDER\">lexorder</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)))\u000A             (t (<see topic=\"ACL2____LEXORDER\">lexorder</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> y)))))</code>"],

 "ACL2____LD-MISSING-INPUT-OK":[["Ld"],"ACL2 Sources","ACL2","<p><v>ld-missing-input-ok</v> is an <tt><see topic=\"ACL2____LD\">ld</see></tt> special (see <see topic=\"ACL2____LD\">ld</see>). \u000A The accessor is <v>(<see topic=\"ACL2____LD-MISSING-INPUT-OK\">ld-missing-input-ok</see> state)</v> and the updater is \u000A <v>(set-ld-missing-input-ok val state)</v>.  <v>ld-missing-input-ok</v> must be \u000A either <v>nil</v>, <v>t</v>, or <v>:warn</v>.  The initial value of \u000A <v>ld-missing-input-ok</v> is <v>nil</v>.</p> \u000A \u000A <p>The general-purpose ACL2 read-eval-print loop, <tt><see topic=\"ACL2____LD\">ld</see></tt>, is controlled by \u000A various flags that control its behavior, and <v>ld-missing-input-ok</v> is one \u000A of them.  In brief, the first argument of <v>ld</v> can indicate a file from \u000A which to read input.  If the file does not exist, it is an error by default, \u000A but <v>ld</v> becomes essentially a no-op if <v>t</v> or <v>:warn</v> is supplied \u000A for <v>:ld-missing-input-ok</v>, where <v>:warn</v> prints a warning.  Also see \u000A <see topic=\"ACL2____LD\">ld</see>.</p>"],

 "ACL2____Guiding_02the_02ACL2_02Theorem_02Prover":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____ACL2_02as_02an_02Interactive_02Theorem_02Prover_02_82cont_92'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Now that you have seen the theorem prover in action you might be curious as \u000A to how you guide it.</p> \u000A \u000A <p><img src='interactive-theorem-prover.gif'></img></p> \u000A \u000A <p>Look at the picture above.  It is meant to suggest that <i>Q</i> is an \u000A important lemma needed for the proof of <i>P</i>.  Note that to lead the \u000A prover to the proof of <i>P</i> the user first proves <i>Q</i>.  In a way, the \u000A formulation and proof of <i>Q</i> is a hint to the prover about how to prove \u000A <i>P</i>.</p> \u000A \u000A <p>The user usually doesn't think of <i>Q</i> or recognize the need to prove \u000A it separately until he or she sees the theorem prover <b>fail</b> to prove \u000A <i>P</i> without it ``knowing'' <i>Q</i>.</p> \u000A \u000A <p>The way the user typically discovers the need for <i>Q</i> is to look at \u000A failed proofs.</p> \u000A \u000A <p><see topic='ACL2____ACL2_02as_02an_02Interactive_02Theorem_02Prover_02_82cont_92'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____FMT-TO-COMMENT-WINDOW":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____CW\">cw</see> for an introduction to the comment window and the \u000A usual way to print it.</p> \u000A \u000A <p>Function <v>fmt-to-comment-window</v> is identical to <v>fmt1</v> (see <see topic=\"ACL2____FMT\">fmt</see>), except that the channel is <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt> and the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt> is neither an input nor an output.  An analogous function, \u000A <v>fmt-to-comment-window!</v>, prints with <tt><see topic=\"ACL2____FMT_12\">fmt!</see></tt> instead of <tt><see topic=\"ACL2____FMT\">fmt</see></tt>, \u000A in order to avoid insertion of backslash (\\) characters for margins; also see \u000A <see topic=\"ACL2____CW_12\">cw!</see>.  Note that even if you change the value of <tt><see topic=\"ACL2____LD\">ld</see></tt> special \u000A <v>standard-co</v> (see <see topic=\"ACL2____STANDARD-CO\">standard-co</see>), <v>fmt-to-comment-window</v> will \u000A print to <tt><see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see></tt>, which is the original value of <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____FMT-TO-COMMENT-WINDOW\">fmt-to-comment-window</see> fmt-string alist col evisc-tuple)</code> \u000A \u000A <p>where these arguments are as desribed for <tt><see topic=\"ACL2____FMT1\">fmt1</see></tt>; see <see topic=\"ACL2____FMT\">fmt</see>.</p>"],

 "ACL2____FAST-ALIST-CLEAN_12":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically this function is just <v>fast-alist-clean</v>; we leave it enabled \u000A and would think it odd to ever prove a theorem about it.</p> \u000A \u000A <p>Under the hood, this is the same as <v>fast-alist-clean</v> except that it \u000A uses something like <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt> instead of <tt><see topic=\"ACL2____HONS-ACONS\">hons-acons</see></tt>.  You \u000A generally <b>should not use</b> this function unless you really know what \u000A you're doing and understand the drawbacks discussed in <tt><see topic=\"ACL2____HONS-ACONS_12\">hons-acons!</see></tt>.</p>"],

 "ACL2____E_F2D":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<p>The macro <v>e/d</v> creates theory expressions for use in <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hints and events.  It provides a convenient way to <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> \u000A and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> simultaneously, without having to write arcane theory \u000A expressions.</p> \u000A \u000A <code>Examples:\u000A(<see topic=\"ACL2____E_F2D\">e/d</see> (lemma1 lemma2))          ; equivalent to (<see topic=\"ACL2____ENABLE\">enable</see> lemma1 lemma2)\u000A(<see topic=\"ACL2____E_F2D\">e/d</see> () (lemma))               ; equivalent to (<see topic=\"ACL2____DISABLE\">disable</see> lemma)\u000A(<see topic=\"ACL2____E_F2D\">e/d</see> (lemma1) (lemma2 lemma3)) ; Enable lemma1 then disable lemma2, lemma3.\u000A(<see topic=\"ACL2____E_F2D\">e/d</see> () (lemma1) (lemma2))     ; Disable lemma1 then enable lemma2.\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____E_F2D\">e/d</see> enables-0 disables-0 ... enables-n disables-n)</code> \u000A \u000A <p>where each <v>enables-i</v> and <v>disables-i</v> is a list of runic \u000A designators; see <see topic=\"ACL2____THEORIES\">theories</see>, see <see topic=\"ACL2____ENABLE\">enable</see>, and see <see topic=\"ACL2____DISABLE\">disable</see>.</p> \u000A \u000A <p>The <v>e/d</v> macro takes any number of lists suitable for the <tt><see topic=\"ACL2____ENABLE\">enable</see></tt> and <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> macros, and creates a theory that is equal to \u000A <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v> after executing the following commands.</p> \u000A \u000A <p>(in-theory (enable . enables-0)) (in-theory (disable . disables-0)) [etc.] \u000A (in-theory (enable . enables-n)) (in-theory (disable . disables-n))</p>"],

 "ACL2____EXPLODE-ATOM":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><tt>(explode-atom x print-base)</tt> prints the atom <v>x</v> as a list of \u000A characters, using some <see topic=\"ACL2____PRINT-BASE-P\">print-base-p</see> to decide what base to print \u000A numbers in.</p> \u000A \u000A <p>Examples:</p> \u000A \u000A <code>(<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> 15 10) --&gt; (#\\1 #\\5)              ; 15 in decimal\u000A(<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> 15 16) --&gt; (#\\F)                  ; 15 in hex\u000A(<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> &quot;foo&quot; 10) --&gt; (#\\f #\\o #\\o)\u000A(<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> 'acl2::foo 10) --&gt; (#\\F #\\O #\\O)  ; note: no package</code> \u000A \u000A <p>See also <see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see>, <see topic=\"STR____NUMBERS\">str::numbers</see>, and <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>explode-atom</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A explode-atom (x print-base)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                                 (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                 (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                                 (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x))\u000A                             (<see topic=\"ACL2____PRINT-BASE-P\">print-base-p</see> print-base))))\u000A (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A  ((<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A   (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A    ((<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0)\u000A            (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\-\u000A                  (<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> (<see topic=\"COMMON-LISP____-\">-</see> x)\u000A                                               print-base nil)))\u000A           (t (<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> x print-base nil))))\u000A    (t\u000A     (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x) print-base)\u000A             (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\/\u000A                   (<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)\u000A                                                print-base nil))))))\u000A  ((<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> x)\u000A   (<see topic=\"COMMON-LISP____LIST_A2\">list*</see>\u000A        #\\# #\\C #\\(\u000A        (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x) print-base)\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> #\\Space\u000A                      (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x) print-base)\u000A                              '(#\\)))))))\u000A  ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) (<see topic=\"COMMON-LISP____LIST\">list</see> x))\u000A  ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x) (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list))\u000A  (t (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x) 'list))))</code>"],

 "ACL2____DEFINE-PC-HELP":[["Proof-checker"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____DEFINE-PC-HELP\">define-pc-help</see> pp ()\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (goals t)\u000A      (io? proof-checker nil state\u000A           (state-stack)\u000A           (fms0 &quot;~|~y0~|&quot;\u000A                 (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> #0\u000A                             (fetch-term (conc t)\u000A                                         (current-addr t))))))\u000A    (print-all-goals-proved-message state)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____DEFINE-PC-HELP\">define-pc-help</see> name args &amp;rest body)</code> \u000A \u000A <p>This defines a macro command named <v>name</v>, as explained further below. \u000A The <v>body</v> should (after removing optional declarations) be a form that \u000A returns <v>state</v> as its single value.  Typically, it will just print \u000A something.</p> \u000A \u000A <p>What <v>(<see topic=\"ACL2____DEFINE-PC-HELP\">define-pc-help</see> name args &amp;rest body)</v> really does is to create a \u000A call of <v>define-pc-macro</v> that defines <v>name</v> to take arguments \u000A <v>args</v>, to have the declarations indicated by all but the last form in \u000A <v>body</v>, and to have a body that (via <v>pprogn</v>) first executes the form \u000A in the last element of body and then returns a call to the command <v>skip</v> \u000A (which will return <v>(<see topic=\"ACL2____MV\">mv</see> nil t state)</v>).</p>"],

 "ACL2____ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ALISTP\">alistp</see> x)</v> is true if and only if <v>x</v> is a list of <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pairs.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____ALISTP\">alistp</see> x)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> alistp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____ALISTP\">alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2-PC____PROMOTE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Apromote\u000A(promote t)</code> \u000A \u000A <p>For example, if the conclusion is <v>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> x y) z)</v>, then after \u000A execution of <v>promote</v>, the conclusion will be <v>z</v> and the terms <v>x</v> \u000A and <v>y</v> will be new top-level hypotheses.</p> \u000A \u000A <code>General Form:\u000A(promote &amp;optional do-not-flatten-flag)</code> \u000A \u000A <p>Replace conclusion of <v>(<see topic=\"ACL2____IMPLIES\">implies</see> hyps exp)</v> or <v>(<see topic=\"COMMON-LISP____IF\">if</see> hyps exp t)</v> with \u000A simply <v>exp</v>, adding <v>hyps</v> to the list of top-level hypotheses. \u000A Moreover, if <v>hyps</v> is viewed as a conjunction then each conjunct will be \u000A added as a separate top-level hypothesis.  An exception is that if \u000A <v>do-not-flatten-flag</v> is supplied and not <v>nil</v>, then only one top-level \u000A hypothesis will be added, namely <v>hyps</v>.</p> \u000A \u000A <p><b>Remark</b>: You must be at the top of the conclusion in order to use \u000A this command.  Otherwise, first invoke <v>top</v>.</p>"],

 "COMMON-LISP____SUBLIS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____SUBLIS\">Sublis</see> alist tree)</v> is obtained by replacing every leaf of \u000A <v>tree</v> with the result of looking that leaf up in the association list \u000A <v>alist</v>.  However, a leaf is left unchanged if it is not found as a key in \u000A <v>alist</v>.</p> \u000A \u000A <p>Leaves are looked up using the function <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt>.  The <see topic=\"ACL2____GUARD\">guard</see> \u000A for <v>(<see topic=\"COMMON-LISP____SUBLIS\">sublis</see> alist tree)</v> requires <v>(<see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see> alist)</v>.  This <see topic=\"ACL2____GUARD\">guard</see> ensures that the <see topic=\"ACL2____GUARD\">guard</see> for <tt><see topic=\"COMMON-LISP____ASSOC\">assoc</see></tt> will be met for each \u000A lookup generated by <v>sublis</v>.</p> \u000A \u000A <p><v>Sublis</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>sublis</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> sublis (alist tree)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> tree)\u000A              (<see topic=\"COMMON-LISP____LET\">let</see> ((pair (<see topic=\"COMMON-LISP____ASSOC\">assoc</see> tree alist)))\u000A                   (<see topic=\"COMMON-LISP____COND\">cond</see> (pair (<see topic=\"COMMON-LISP____CDR\">cdr</see> pair)) (t tree))))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____SUBLIS\">sublis</see> alist (<see topic=\"COMMON-LISP____CAR\">car</see> tree))\u000A                      (<see topic=\"COMMON-LISP____SUBLIS\">sublis</see> alist (<see topic=\"COMMON-LISP____CDR\">cdr</see> tree))))))</code>"],

 "COMMON-LISP____STRING_C3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRING_C3\">String&lt;</see> str1 str2)</v> is non-<v>nil</v> if and only if the string \u000A <v>str1</v> precedes the string <v>str2</v> lexicographically, where character \u000A inequalities are tested using <tt><see topic=\"COMMON-LISP____CHAR_C3\">char&lt;</see></tt>.  When non-<v>nil</v>, <v>(<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see>\u000A str1 str2)</v> is the first position (zero-based) at which the strings differ. \u000A Here are some examples.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> &quot;abcd&quot; &quot;abu&quot;)\u000A2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> &quot;abcd&quot; &quot;Abu&quot;)\u000ANIL\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> &quot;abc&quot; &quot;abcde&quot;)\u000A3\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> &quot;abcde&quot; &quot;abc&quot;)\u000ANIL</code> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string&lt;</v> specifies that its arguments are \u000A strings.</p> \u000A \u000A <p><v>String&lt;</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string&lt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string&lt; (str1 str2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2))))\u000A       (string&lt;-l (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str1 'list)\u000A                  (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str2 'list)\u000A                  0))</code>"],

 "COMMON-LISP____RATIONALP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</v> is true if and only if <v>x</v> is an rational \u000A number.</p>"],

 "COMMON-LISP____LOGXOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logxor</v> returns their bitwise logical exclusive or.  In ACL2 <v>logxor</v> \u000A is a macro that expands into calls of the binary function <v>binary-logxor</v>, \u000A except that <v>(<see topic=\"COMMON-LISP____LOGXOR\">logxor</see>)</v> expands to <v>0</v> and <v>(<see topic=\"COMMON-LISP____LOGXOR\">logxor</see> x)</v> expands to \u000A <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>binary-logxor</v> requires its arguments to be \u000A integers.  <v>Logxor</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>logxor</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> logxor (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> args) 0)\u000A                ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> args)) (<see topic=\"COMMON-LISP____CAR\">car</see> args))\u000A                (t (xxxjoin 'binary-logxor args))))</code> \u000A <p><b>Function: </b><srclink>binary-logxor</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-logxor (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> (<see topic=\"COMMON-LISP____LOGEQV\">logeqv</see> i j)))</code>"],

 "COMMON-LISP____LOGIOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logior</v> returns their bitwise logical inclusive or.  In ACL2 <v>logior</v> \u000A is a macro that expands into calls of the binary function <v>binary-logior</v>, \u000A except that <v>(<see topic=\"COMMON-LISP____LOGIOR\">logior</see>)</v> expands to <v>0</v> and <v>(<see topic=\"COMMON-LISP____LOGIOR\">logior</see> x)</v> expands to \u000A <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>binary-logior</v> requires its arguments to be \u000A integers.  <v>Logior</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>logior</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> logior (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> args) 0)\u000A                ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> args)) (<see topic=\"COMMON-LISP____CAR\">car</see> args))\u000A                (t (xxxjoin 'binary-logior args))))</code> \u000A <p><b>Function: </b><srclink>binary-logior</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-logior (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> (<see topic=\"COMMON-LISP____LOGAND\">logand</see> (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i) (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> j))))</code>"],

 "COMMON-LISP____LOGEQV":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logeqv</v> returns their bitwise logical equivalence.  In ACL2 <v>logeqv</v> is \u000A a macro that expands into calls of the binary function <v>binary-logeqv</v>, \u000A except that <v>(<see topic=\"COMMON-LISP____LOGEQV\">logeqv</see>)</v> expands to <v>-1</v> and <v>(<see topic=\"COMMON-LISP____LOGEQV\">logeqv</see> x)</v> expands to \u000A <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>binary-logeqv</v> requires its arguments to be \u000A integers.  <v>Logeqv</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>logeqv</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> logeqv (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> args) -1)\u000A                ((<see topic=\"COMMON-LISP____NULL\">null</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> args)) (<see topic=\"COMMON-LISP____CAR\">car</see> args))\u000A                (t (xxxjoin 'binary-logeqv args))))</code> \u000A <p><b>Function: </b><srclink>binary-logeqv</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> binary-logeqv (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGAND\">logand</see> (<see topic=\"COMMON-LISP____LOGORC1\">logorc1</see> i j) (<see topic=\"COMMON-LISP____LOGORC1\">logorc1</see> j i)))</code>"],

 "ACL2____What_02Is_02ACL2_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____What_02is_02a_02Mathematical_02Logic_82Q_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>ACL2 is a <b>mathematical logic</b> together with a <b>mechanical theorem \u000A prover</b> to help you reason in the logic.</p> \u000A \u000A <p>The logic is just a subset of applicative <see topic='ACL2____Common_02Lisp'>Common Lisp</see>.  (This link takes you off the main route of \u000A the tour.  You'll see some Common Lisp on the tour, so visit this later!)</p> \u000A \u000A <p>The theorem prover is an ``industrial strength'' version of the Boyer-Moore \u000A theorem prover, Nqthm.</p> \u000A \u000A <p><b>Models</b> of all kinds of computing systems can be built in ACL2, just \u000A as in Nqthm, even though the formal logic is Lisp.</p> \u000A \u000A <p>Once you've built an ACL2 model of a system, you can run it.</p> \u000A \u000A <p>You can also use ACL2 to prove theorems about the model.</p> \u000A \u000A <p><see topic='ACL2____What_02is_02a_02Mathematical_02Logic_82Q_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____UNION-THEORIES":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> 'lemma3)\u000A                (<see topic=\"ACL2____THEORY\">theory</see> 'arith-patch))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____UNION-THEORIES\">union-theories</see> th1 th2)</code> \u000A \u000A <p>where <v>th1</v> and <v>th2</v> are theories (see <see topic=\"ACL2____THEORIES\">theories</see>).  To each of \u000A the arguments there corresponds a runic theory.  This function returns the \u000A union of those two runic <see topic=\"ACL2____THEORIES\">theories</see>, represented as a list and ordered \u000A chronologically.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____TRANS1":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A:trans1 (<see topic=\"COMMON-LISP____LIST\">list</see> a b c)\u000A:trans1 (<see topic=\"COMMON-LISP____CADDR\">caddr</see> x)\u000A:trans1 (<see topic=\"COMMON-LISP____COND\">cond</see> (p q) (r))</code> \u000A \u000A <p>This function takes one argument, an alleged term, and expands the \u000A top-level macro in it for one step only.  Either an error is caused, which \u000A happens when the form is not a call of a macro, or the result is printed. \u000A Also see <see topic=\"ACL2____TRANS\">trans</see>, which translates the given form completely.</p>"],

 "ACL2____SYMBOL-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>symbol-listp</v> tests whether its argument is a \u000A true list of symbols.</p> \u000A \u000A <p><b>Function: </b><srclink>symbol-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> symbol-listp (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) (<see topic=\"COMMON-LISP____EQ\">eq</see> lst nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                     (<see topic=\"ACL2____SYMBOL-LISTP\">symbol-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code>"],

 "ACL2____STOP-PROOF-TREE":[["Proof-tree"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> and see <see topic=\"ACL2____START-PROOF-TREE\">start-proof-tree</see>.  Note \u000A that <v>:stop-proof-tree</v> works by adding <v>'</v><tt><see topic=\"ACL2____PROOF-TREE\">proof-tree</see></tt> to the \u000A <v>inhibit-output-lst</v>; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.</p> \u000A \u000A <p>Proof tree displays are explained in the documentation for <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.  <v>:Stop-proof-tree</v> causes proof tree display to be turned \u000A off.</p> \u000A \u000A <p>It is permissible to submit the form <v>(<see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see>)</v> during a break. \u000A Thus, you can actually turn off proof tree display in the middle of a proof by \u000A interrupting ACL2 and submitting the form <v>(<see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see>)</v> in raw \u000A Lisp.</p>"],

 "ACL2____SET-FC-REPORT-ON-THE-FLY":[["Forward-chaining-reports"],"ACL2 Sources","ACL2","<code>Examples:  (<see topic=\"ACL2____SET-FC-REPORT-ON-THE-FLY\">set-fc-report-on-the-fly</see> t)\u000A           (<see topic=\"ACL2____SET-FC-REPORT-ON-THE-FLY\">set-fc-report-on-the-fly</see> nil)</code> \u000A \u000A <p>If the flag is set to <v>t</v>, forward chaining tracking reports are printed \u000A when forward chaining occurs.  If the flag is set to <v>nil</v>, very short \u000A reports (giving just the caller and the report number) are printed during \u000A forward chaining but you can use <tt><see topic=\"ACL2____FC-REPORT\">fc-report</see></tt> to see the full report \u000A afterwards.</p> \u000A \u000A <p>Since nothing is tracked when the criteria is <v>nil</v>, this function also \u000A prints out the current criteria to remind you of what it is.</p> \u000A \u000A <p>The flag manipulated by this function does not shut off tracking.  It only \u000A determines whether reports are printed on-the-fly or not.  To shut off \u000A tracking <tt><see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for details.</p>"],

 "ACL2____SERIALIZE-WRITE":[["Serialize"],"ACL2 Sources","ACL2","<p>General form:</p> \u000A \u000A <code>(<see topic=\"ACL2____SERIALIZE-WRITE\">serialize-write</see> filename obj\u000A                 [:verbosep  {t, nil}])    ; nil by default\u000A --&gt;\u000Astate</code> \u000A \u000A <p>In the logic this carries out an oracle read.</p> \u000A \u000A <p>Under the hood, we try to save <v>obj</v> into the file indicated by \u000A <v>filename</v>, which must be a string.  The object can later be recovered with \u000A <tt><see topic=\"ACL2____SERIALIZE-READ\">serialize-read</see></tt>.  We just return <v>state</v>, and any failures (e.g., \u000A file not openable) will result in a hard Lisp error.</p> \u000A \u000A <p>Writing objects to disk is generally slower than reading them back in since \u000A some analysis is required to convert an object into our <see topic=\"ACL2____SERIALIZE\">serialize</see>d \u000A object format.</p> \u000A \u000A <p>The <v>verbosep</v> flag just says whether to print some low-level details \u000A related to timing and memory usage as the file is being read.</p>"],

 "ACL2____NOTE1":[["Release-notes"],"ACL2 Sources","ACL2","<p>The new features are extensively documented.  The relevant topics \u000A are:</p> \u000A \u000A <p>It is especially important to read all of of the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for \u000A <see topic=\"ACL2____BOOKS\">books</see> before trying to use books.  However, the new <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> \u000A keyword command is so handy for reading long <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> strings that \u000A we recommend you start with <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> more if reading at the terminal. \u000A Some documentation has been written for <see topic=\"ACL2____GUARD\">guard</see>s which you might find \u000A interesting.</p>"],

 "ACL2____NOTE-3-2_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Changed the default distributed <see topic=\"ACL2____BOOKS\">books</see> directory for ACL2(r) \u000A from <v>books/</v> to <v>books/nonstd/</v>.  See <see topic=\"ACL2____INCLUDE-BOOK\">include-book</see>, in \u000A particular the discussion of ``Distributed Books Directory''.</p> \u000A \u000A <p>Added directory <v>books/arithmetic-3/</v> and its subdirectories to \u000A <v>books/nonstd/</v>.  (But a chunk of theorems from \u000A <v>arithmetic-3/extra/ext.lisp</v> are ``commented out'' using \u000A #-:non-standard-analysis because they depend on <v>books/rtl/rel7/</v>, which is \u000A not yet in <v>books/nonstd/</v>; feel free to volunteer to remedy this!)</p> \u000A \u000A <p>Incorporated changes from Ruben Gamboa to some (linear and non-linear) \u000A arithmetic routines in the theorem prover, to comprehend the reals rather than \u000A only the rationals.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-3-2\">note-3-2</see> for changes to Version 3.2 of ACL2.</p>"],

 "ACL2____MV_F3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Mv?</v> is designed to work with <v>mv?-let</v>, generalizing how \u000A <tt><see topic=\"ACL2____MV\">mv</see></tt> works with <tt><see topic=\"ACL2____MV-LET\">mv-let</see></tt> by allowing the binding of a single \u000A variable.  For example, the following form is legal.</p> \u000A \u000A <code>(<see topic=\"ACL2____MV_F3-LET\">mv?-let</see> (y)\u000A         (<see topic=\"ACL2____MV_F3\">mv?</see> (f x))\u000A         (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer y))\u000A         (g x y z))</code> \u000A \u000A <p>The expression above is equivalent to the following expression, because \u000A <v>(<see topic=\"ACL2____MV_F3\">mv?</see> form)</v> expands to <v>form</v> for any expression, <v>form</v>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____LET\">let</see> ((y (f x)))\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> integer y))\u000A  (g x y z))</code> \u000A \u000A <p>Logically, <v>(<see topic=\"ACL2____MV_F3\">mv?</see> x)</v> is the same as <v>x</v>, while <v>(<see topic=\"ACL2____MV_F3\">mv?</see> v1 v2 ...)</v> \u000A is the same as <v>(<see topic=\"COMMON-LISP____LIST\">list</see> v1 v2 ...)</v>.  Also see <see topic=\"ACL2____MV\">mv</see> and see <see topic=\"ACL2____MV_F3-LET\">mv?-let</see>.</p>"],

 "ACL2____MORE_12":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>NOTE: The command <v>:more!</v> only makes sense at the terminal.</p> \u000A \u000A <code>Example:\u000AACL2 !&gt;:more!</code> \u000A \u000A <p>will print all of the remaining <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> started by the last \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> or <v>:</v><tt><see topic=\"ACL2____MORE-DOC\">more-doc</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____MORE\">more</see> for some background.  Typing <v>:more!</v> will print all \u000A remaining blocks of <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p> \u000A \u000A <p><v>:More!</v> is like <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> except that it prints all the text \u000A at once.  For example, if you type <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> name you will see some \u000A text followed by ``<v>(<see topic=\"COMMON-LISP____TYPE\">type</see> :more for more, :more! for the rest)</v>''.  If you \u000A then type simply <v>:more!</v> you will see all of the details, while if you \u000A type <v>:</v><tt><see topic=\"ACL2____MORE\">more</see></tt> you will be fed the next block of details.</p>"],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-Q3-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>The correct answer to Question 3 in <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see> is that you need to prove</p> \u000A \u000A <code><i>Subgoal to Relieve Hyp 1</i>: \u000A(implies (and (q (f a)) \u000A              (r a)) \u000A         (p (f a))) \u000A</code> \u000A \u000A <p>in order to use</p> \u000A \u000A <code><i>Theorem</i>: \u000A(implies (p (f x)) \u000A         (equal (g (h x)) \u000A                x)) \u000A</code> \u000A \u000A <p>to rewrite the target <v>(g (h a))</v> to <v>a</v> in</p> \u000A \u000A <code><i>Goal Conjecture</i>: \u000A(implies (and (q (f a)) \u000A              (r a)) \u000A         (s (g (h a)))) \u000A</code> \u000A \u000A <p>If you don't see why, re-read the discussion of <see topic='ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-REWRITING'>rewriting</see> again. \u000A Forgetting about the need to relieve hypotheses is a common mistake in \u000A informal proofs.  ACL2 won't forget to relieve them.  But if you forget about \u000A the need to do it, you may be confused when ACL2 doesn't see the ``proof'' you \u000A see!</p> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the end of quiz in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____EXPLODE-NONNEGATIVE-INTEGER":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> 925 10 nil)\u000A(#9 #2 #5)\u000AACL2 !&gt;(<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> 325 16 nil)\u000A(#3 #9 #D)</code> \u000A \u000A <p>For a non-negative integer <v>n</v>, <v>(<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see> n r\u000A nil)</v> is the list of <see topic=\"ACL2____CHARACTERS\">characters</see> in the radix-<v>r</v> (base-<v>r</v>) \u000A representation of <v>n</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>explode-nonnegative-integer</v> requires the first \u000A argument to be a nonnegative integer and second argument to be a valid radix \u000A for ACL2 (2, 8, 10, or 16).</p> \u000A \u000A <p>See also <see topic=\"ACL2____EXPLODE-ATOM\">explode-atom</see>, <see topic=\"STR____NUMBERS\">str::numbers</see>, and <see topic=\"ACL2____PRINTING-TO-STRINGS\">printing-to-strings</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>explode-nonnegative-integer</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> explode-nonnegative-integer\u000A       (n print-base ans)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                                   (<see topic=\"ACL2____PRINT-BASE-P\">print-base-p</see> print-base))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A                  (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____PRINT-BASE-P\">print-base-p</see> print-base)))\u000A              (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____NULL\">null</see> ans) '(#\\0)) (t ans)))\u000A             (t (<see topic=\"ACL2____EXPLODE-NONNEGATIVE-INTEGER\">explode-nonnegative-integer</see>\u000A                     (<see topic=\"COMMON-LISP____FLOOR\">floor</see> n print-base)\u000A                     print-base\u000A                     (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____DIGIT-TO-CHAR\">digit-to-char</see> (<see topic=\"COMMON-LISP____MOD\">mod</see> n print-base))\u000A                           ans)))))</code>"],

 "ACL2____EMACS":[["ACL2-tutorial"],"ACL2 Sources","ACL2","<p>Many successful ACL2 users run in an shell under the Emacs editor. \u000A If you do so, then you may wish to load the distributed file \u000A <v>emacs/emacs-acl2.el</v>.  The file begins with considerable comments \u000A describing what it offers.  It is intended to work both with GNU Emacs and \u000A XEmacs.</p> \u000A \u000A <p>In particular, the above file provides the ACL2-Doc browser, a convenient \u000A tool for viewing, in Emacs, documentation for both the ACL2 system and the \u000A documented community books.  See <see topic=\"ACL2____ACL2-DOC\">ACL2-Doc</see>.</p> \u000A \u000A <p>If you are not comfortable with Emacs, you may prefer to use an \u000A Eclipse-based interface; see <see topic=\"ACL2____ACL2-SEDAN\">ACL2-sedan</see>.</p>"],

 "ACL2____Conversion":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>When symbols are read by Common Lisp they are converted to upper \u000A case.  Note carefully that this remark applies to the characters in \u000A <i>symbols</i>.  The characters in strings are not converted upper case.</p> \u000A \u000A <p>To type a symbol containing lower case characters you can enclose the \u000A symbol in vertical bars, as in <v>|AbC|</v> or you can put a ``backslash'' \u000A before each lower case character you wish to preserve, as in <v>A\\bC</v>. \u000A <v>|AbC|</v> and <v>A\\bC</v> are two different ways of writing the same symbol \u000A (just like 2/4 and 1/2 are two different ways of writing the same rational and \u000A 123 and 0123 are two different ways to write the same natural number).  The \u000A symbol has three characters in its name, the middle one of which is a lower \u000A case b.</p>"],

 "ACL2____CUSTOM-KEYWORD-HINTS":[["Hints"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see> for a discussion of how advanced \u000A users can define their own hint keywords.  For examples, see the community \u000A books directory <v>books/hints/</v>, in particular <v>basic-tests.lisp</v>.</p>"],

 "ACL2____An_02Example_02Common_02Lisp_02Function_02Definition":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____An_02Example_02of_02ACL2_02in_02Use'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Consider the binary trees <v>x</v> and <v>y</v> below.</p> \u000A \u000A <p><img src='binary-trees-x-y.gif'></img></p> \u000A \u000A <p>In Lisp, <v>x</v> is written as the list <v>'(A B)</v> or, equivalently, as \u000A <v>'(A B . NIL)</v>.  Similarly, <v>y</v> may be written <v>'(C D E)</v>.  Suppose \u000A we wish to replace the right-most tip of <v>x</v> by the entire tree <v>y</v>. \u000A This is denoted <v>(app x y)</v>, where <v>app</v> stands for ``append''.</p> \u000A \u000A <p><img src='binary-trees-app.gif'></img></p> \u000A \u000A <p>We can define <v>app</v> with:</p> \u000A \u000A <code><b>(defun app (x y)</b>                           <i>; Concatenate x and y.</i> \u000A  <b>(declare (type (satisfies true-listp) x))</b><i>; We expect x to end in NIL.</i> \u000A  <b>(cond ((endp x) y)</b>                       <i>; If x is empty, return y.</i> \u000A        <b>(t (cons (car x)</b>                   <i>; Else, copy first node</i> \u000A                 <b>(app (cdr x) y)))))</b>       <i>;  and recur into next.</i> \u000A</code> \u000A \u000A <p>If you defined this function in some Common Lisp, then to run <v>app</v> on \u000A the <v>x</v> and <v>y</v> above you could then type</p> \u000A \u000A <code>(app '(A B) '(C D E))</code> \u000A \u000A <p>and Common Lisp will print the result <v>(A B C D E)</v>.</p> \u000A \u000A <p><see topic='ACL2____An_02Example_02of_02ACL2_02in_02Use'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____ARRAYS-EXAMPLE":[["Arrays"],"ACL2 Sources","ACL2","<p>The example below illustrates the use of ACL2 arrays.  It is not, \u000A of course, a substitute for the detailed explanations provided elsewhere (see \u000A <see topic=\"ACL2____ARRAYS\">arrays</see>, including subtopics).</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____DEFUN\">defun</see> defarray (<see topic=\"ACL2____NAME\">name</see> size initial-element)\u000A         (<see topic=\"ACL2____COMPRESS1\">compress1</see> name\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____LIST\">list</see> :HEADER\u000A                                :DIMENSIONS (<see topic=\"COMMON-LISP____LIST\">list</see> size)\u000A                                :MAXIMUM-LENGTH (<see topic=\"COMMON-LISP____1_B2\">1+</see> size)\u000A                                :DEFAULT initial-element\u000A                                :NAME name)\u000A                          nil)))\u000A\u000ASince DEFARRAY is non-recursive, its admission is trivial.  We observe\u000Athat the type of DEFARRAY is described by the theorem\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (DEFARRAY NAME SIZE INITIAL-ELEMENT))\u000A     (<see topic=\"ACL2____TRUE-LISTP\">TRUE-LISTP</see> (DEFARRAY NAME SIZE INITIAL-ELEMENT))).\u000AWe used the :type-prescription rule COMPRESS1.\u000A\u000ASummary\u000AForm:  ( DEFUN DEFARRAY ...)\u000ARules: ((:TYPE-PRESCRIPTION COMPRESS1))\u000AWarnings:  None\u000ATime:  0.02 seconds (prove: 0.00, print: 0.02, other: 0.00)\u000A DEFARRAY\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> my-ar (defarray 'a1 5 17))\u000A ((:HEADER :DIMENSIONS (5)\u000A           :MAXIMUM-LENGTH 6 :DEFAULT 17 :NAME A1))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 3)\u000A17\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 8)\u000A\u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol AREF1,\u000Awhich is\u000A(<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"ACL2____ARRAY1P\">ARRAY1P</see> NAME L) (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> N) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> N 0) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> N (<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"ACL2____DIMENSIONS\">DIMENSIONS</see> NAME L)))),\u000Ais violated by the arguments in the call (<see topic=\"ACL2____AREF1\">AREF1</see> 'A1 '(#) 8).\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____ASSIGN\">assign</see> my-ar (<see topic=\"ACL2____ASET1\">aset1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 3 'xxx))\u000A ((3 . XXX)\u000A  (:HEADER :DIMENSIONS (5)\u000A           :MAXIMUM-LENGTH 6 :DEFAULT 17 :NAME A1))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 3)\u000AXXX\u000AACL2 !&gt;(<see topic=\"ACL2____ASET1\">aset1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 3 'yyy) ; BAD: (<see topic=\"ACL2_____04\">@</see> my-ar) now points to\u000A                                    ;      an old copy of the array!\u000A((3 . YYY)\u000A (3 . XXX)\u000A (:HEADER :DIMENSIONS (5)\u000A          :MAXIMUM-LENGTH 6 :DEFAULT 17 :NAME A1))\u000AACL2 !&gt;(<see topic=\"ACL2____AREF1\">aref1</see> 'a1 (<see topic=\"ACL2_____04\">@</see> my-ar) 3) ; Because of &quot;BAD&quot; above, the array\u000A                               ; access is done using assoc rather\u000A                               ; than Lisp aref, hence is slower;\u000A                               ; but the answer is still correct,\u000A                               ; reflecting the value in (<see topic=\"ACL2_____04\">@</see> my-ar),\u000A                               ; which was not changed above.\u000A\u000A**********************************************************\u000ASlow Array Access!  A call of AREF1 on an array named\u000AA1 is being executed slowly.  See :DOC slow-array-warning\u000A**********************************************************\u000A\u000AXXX\u000AACL2 !&gt;</code>"],

 "ACL2____ALLOCATE-FIXNUM-RANGE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ALLOCATE-FIXNUM-RANGE\">Allocate-fixnum-range</see> fixnum-lo fixnum-hi)</v> causes Gnu Common \u000A Lisp \u000A (GCL) to create a persistent table for the integers between <v>fixnum-lo</v> and \u000A <v>fixnum-hi</v> (both bounds inclusive). This table is referenced first when \u000A any integer is boxed and the existing box in the table is used if the integer \u000A is in bounds.  This can speed up GCL considerably by avoiding wasteful fixnum \u000A boxing.  Here, <v>fixnum-lo</v> and <v>fixnum-hi</v> should be fixnums.  On 32-bit \u000A machines it would be good for them to be of type <v>(signed-byte 30)</v>, with \u000A <v>fixnum-lo &lt;= fixnum-hi</v>.</p> \u000A \u000A <p>When this function is executed in a Lisp implementation other than GCL, it \u000A has no side effect.  This function always returns <v>nil</v>.</p>"],

 "ACL2-PC____SHOW-ABBREVIATIONS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(show-abbreviations v w)\u000A   -- assuming that v and w currently abbreviate terms,\u000A      then this instruction displays them together with\u000A      the terms they abbreviate\u000Ashow-abbreviations\u000A   -- display all abbreviations</code> \u000A \u000A <p>See also <v>add-abbreviation</v> and <v>remove-abbreviations</v>.  In \u000A particular, the documentation for <v>add-abbreviation</v> contains a general \u000A discussion of abbreviations.</p> \u000A \u000A <code>General Form:\u000A(show-abbreviations &amp;rest vars)</code> \u000A \u000A <p>Display each argument in <v>vars</v> together with the term it abbreviates \u000A (if any).  If there are no arguments, i.e. the instruction is simply \u000A <v>show-abbreviations</v>, then display all abbreviations together with the \u000A terms they abbreviate.</p> \u000A \u000A <p>If the term abbreviated by a variable, say <v>v</v>, contains a proper \u000A subterm that is also abbreviate by (another) variable, then both the \u000A unabbreviated term and the abbreviated term (but not using <v>(? v)</v> to \u000A abbreviate the term) are displayed with together with <v>v</v>.</p>"],

 "ACL2-PC____FORWARDCHAIN":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(forwardchain 2) ; Second hypothesis should be of the form\u000A                 ; (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyp concl), and the result is to replace\u000A                 ; that hypothesis with concl.\u000A\u000AGeneral Forms:\u000A(forwardchain hypothesis-number)\u000A(forwardchain hypothesis-number hints)\u000A(forwardchain hypothesis-number hints quiet-flg)</code> \u000A \u000A <p>This command replaces the hypothesis corresponding to given index, which \u000A should be of the form <v>(<see topic=\"ACL2____IMPLIES\">IMPLIES</see> hyp concl)</v>, with its consequent \u000A <v>concl</v>.  In fact, the given hypothesis is dropped, and the replacement \u000A hypothesis will appear as the final hypothesis after this command is \u000A executed.</p> \u000A \u000A <p>The prover must be able to prove the indicated hypothesis from the other \u000A hypotheses, or else the command will fail.  The <v>:hints</v> argument is used \u000A in this prover call, and should have the usual syntax of hints to the \u000A prover.</p> \u000A \u000A <p>Output is suppressed if <v>quiet-flg</v> is supplied and not <v>nil</v>.</p>"],

 "COMMON-LISP____STRING-EQUAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For strings <v>str1</v> and <v>str2</v>, <v>(<see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see> str1 str2)</v> \u000A is true if and only <v>str1</v> and <v>str2</v> are the same except perhaps for \u000A the cases of their <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> on <v>string-equal</v> requires that its arguments are \u000A strings consisting of standard characters (see <see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see>).</p> \u000A \u000A <p><v>String-equal</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A string-equal (str1 str2)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A     (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1)\u000A                        (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str1 'list))\u000A                        (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2)\u000A                        (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str2 'list)))))\u000A (<see topic=\"COMMON-LISP____LET\">let</see> ((len1 (<see topic=\"COMMON-LISP____LENGTH\">length</see> str1)))\u000A      (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP_____D3\">=</see> len1 (<see topic=\"COMMON-LISP____LENGTH\">length</see> str2))\u000A           (string-equal1 str1 str2 0 len1))))</code>"],

 "COMMON-LISP____KEYWORDP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____KEYWORDP\">Keywordp</see> x)</v> is true if and only if <v>x</v> is a keyword, i.e., \u000A a symbol in the \"KEYWORD\" package.  Such symbols are typically printed using \u000A a colon \u000A (:) followed by the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of the symbol.</p> \u000A \u000A <p><v>Keywordp</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><v>Keywordp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.  The following log may be illuminating.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____INTERN\">intern</see> &quot;ABC&quot; &quot;KEYWORD&quot;)\u000A:ABC\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> ':ABC)\u000A&quot;ABC&quot;\u000AACL2 !&gt;(<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> ':ABC)\u000A&quot;KEYWORD&quot;\u000AACL2 !&gt;</code> \u000A \u000A <p><b>Function: </b><srclink>keywordp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> keywordp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)\u000A            (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> x)\u000A                   &quot;KEYWORD&quot;)))</code>"],

 "COMMON-LISP____CASE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"COMMON-LISP____CASE\">case</see> typ\u000A  ((:character foo)\u000A   (open file-name :direction :output))\u000A  (bar (open-for-bar file-name))\u000A  (otherwise\u000A   (my-error &quot;Illegal.&quot;)))</code> \u000A \u000A <p>is the same as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____MEMBER\">member</see> typ '(:character foo))\u000A       (open file-name :direction :output))\u000A      ((<see topic=\"COMMON-LISP____EQL\">eql</see> typ 'bar)\u000A       (open-for-bar file-name))\u000A      (t (my-error &quot;Illegal.&quot;)))</code> \u000A \u000A <p>which in turn is the same as</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> typ '(:character foo))\u000A    (open file-name :direction :output)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> typ 'bar)\u000A        (open-for-bar file-name)\u000A        (my-error &quot;Illegal.&quot;)))</code> \u000A \u000A <p>Notice the quotations that appear in the example above: <v>'(:character\u000A foo)</v> and <v>'bar</v>.</p> \u000A \u000A <code>General Forms:\u000A(<see topic=\"COMMON-LISP____CASE\">case</see> expr\u000A  (x1 val-1)\u000A  ...\u000A  (xk val-k)\u000A  (otherwise val-k+1))\u000A\u000A(<see topic=\"COMMON-LISP____CASE\">case</see> expr\u000A  (x1 val-1)\u000A  ...\u000A  (xk val-k)\u000A  (t val-k+1))\u000A\u000A(<see topic=\"COMMON-LISP____CASE\">case</see> expr\u000A  (x1 val-1)\u000A  ...\u000A  (xk val-k))</code> \u000A \u000A <p>where each <v>xi</v> is either <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt> or a true list of <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt> objects.  The final <v>otherwise</v> or <v>t</v> case is optional.</p> \u000A \u000A <p><v>Case</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p>"],

 "COMMON-LISP____ABS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ABS\">Abs</see> x)</v> is <v>-x</v> if <v>x</v> is negative and is <v>x</v> \u000A otherwise.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>abs</v> requires its argument to be a rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) number.</p> \u000A \u000A <p><v>Abs</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p>From ``Common Lisp the Language'' page 205, we must not allow complex \u000A <v>x</v> as an argument to <v>abs</v> in ACL2, because if we did we would have to \u000A return a number that might be a floating point number and hence not an ACL2 \u000A object.</p> \u000A \u000A <p><b>Function: </b><srclink>abs</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> abs (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MINUSP\">minusp</see> x) (<see topic=\"COMMON-LISP____-\">-</see> x) x))</code>"],

 "ACL2_____A2STANDARD-OI_A2":[["Io"],"ACL2 Sources","ACL2","<p>The value of the ACL2 constant <v>*standard-oi*</v> is an open object \u000A input channel that is synonymous to Common Lisp's <v>*standard-input*</v>.</p> \u000A \u000A <p>ACL2 object input from <v>*standard-oi*</v> is actually obtained by reading \u000A from the stream named by Common Lisp's <v>*standard-input*</v>.  That is, by \u000A changing the setting of <v>*standard-input*</v> in raw Common Lisp you can \u000A change the source from which ACL2 reads on the channel <v>*standard-oi*</v>. \u000A See <see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see>.</p>"],

 "ACL2____You_02Must_02Think_02about_02the_02Use_02of_02a_02Formula_02as_02a_02Rule":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Using_02the_02Associativity_02of_02App_02to_02Prove_02a_02Trivial_02Consequence'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>This is <b>good</b> and <b>bad</b>.</p> \u000A \u000A <p>The good news is that you can <b>program</b> ACL2's simplifier.</p> \u000A \u000A <p>The bad news is that when you command ACL2 to prove a theorem you must give \u000A some thought to <b>how that theorem is to be used as a rule</b>!</p> \u000A \u000A <p>For example, if after proving <v>associativity-of-app</v> as previously \u000A shown, you engaged in the mathematically trivial act of proving it again but \u000A with the equality reversed, you would have programmed ACL2's rewriter to loop \u000A forever.</p> \u000A \u000A <p>You can avoid adding any rule by using the command:</p> \u000A \u000A <code>(defthm associativity-of-app \u000A  (equal (app (app a b) c) \u000A         (app a (app b c))) \u000A  <b>:rule-classes nil</b>) \u000A</code> \u000A \u000A <p><see topic='ACL2____Using_02the_02Associativity_02of_02App_02to_02Prove_02a_02Trivial_02Consequence'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____The_02WARNING_02about_02the_02Trivial_02Consequence":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>This <b>Warning</b> alerts us to the fact that when treated as a \u000A <b>rewrite</b> rule, the new rule <v>TRIVIAL-CONSEQUENCE</v>, rewrites terms of \u000A the same form as a rule we have already proved, namely \u000A <v>ASSOCIATIVITY-OF-APP</v>.</p> \u000A \u000A <p>When you see this warning you should <b>think about your rules</b>!</p> \u000A \u000A <p>In the current case, it would be a good idea <b>not</b> to make \u000A <v>TRIVIAL-CONSEQUENCE</v> a rule at all.  We could do this with <v>:</v><tt><see topic=\"ACL2____RULE-CLASSES\">rule-classes</see></tt> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> nil.</p> \u000A \u000A <p>ACL2 proceeds to try to prove the theorem, even though it printed some \u000A warnings.  The basic assumption in ACL2 is that the <b>user</b> <b>understands \u000A what he or she is doing</b> but may need a little reminding just to manage a \u000A complicated set of facts.</p>"],

 "ACL2____SYMBOL-_C3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____SYMBOL-_C3\">symbol-&lt;</see> x y)</v> is non-<v>nil</v> if and only if either the \u000A <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of the symbol <v>x</v> lexicographially precedes the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt> of the symbol <v>y</v> (in the sense of <tt><see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see></tt>) or else \u000A the <tt><see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see></tt>s are equal and the <tt><see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see></tt> of \u000A <v>x</v> lexicographically precedes that of <v>y</v> (in the same sense).  So for \u000A example, <v>(<see topic=\"ACL2____SYMBOL-_C3\">symbol-&lt;</see> 'abcd 'abce)</v> and <v>(<see topic=\"ACL2____SYMBOL-_C3\">symbol-&lt;</see> 'acl2::abcd\u000A 'foo::abce)</v> are true.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>symbol</v> specifies that its arguments are \u000A symbols.</p> \u000A \u000A <p><b>Function: </b><srclink>symbol-&lt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> symbol-&lt; (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x) (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> y))))\u000A       (<see topic=\"COMMON-LISP____LET\">let</see> ((x1 (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> x))\u000A             (y1 (<see topic=\"COMMON-LISP____SYMBOL-NAME\">symbol-name</see> y)))\u000A            (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> x1 y1)\u000A                (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x1 y1)\u000A                     (<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> x)\u000A                              (<see topic=\"ACL2____SYMBOL-PACKAGE-NAME\">symbol-package-name</see> y))))))</code>"],

 "ACL2____SET-RAW-PROOF-FORMAT":[["Prover-output"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-RAW-PROOF-FORMAT\">set-raw-proof-format</see> t)\u000A:set-raw-proof-format t\u000A(<see topic=\"ACL2____SET-RAW-PROOF-FORMAT\">set-raw-proof-format</see> nil)\u000A:set-raw-proof-format nil</code> \u000A \u000A <p>This command affects output from the theorem prover only when <v>'prove</v> \u000A output is not inhibited (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) and gag-mode is \u000A off (see <see topic=\"ACL2____SET-GAG-MODE\">set-gag-mode</see>).  Calling this macro with value <v>t</v> as shown \u000A above will cause simplification steps from proof output, including steps from \u000A preprocess (see <see topic=\"ACL2____SIMPLE\">simple</see>), to print the list of runes used in a list \u000A format, rather than in the English proof commentary.  This ``raw'' format can \u000A be handy when you want to use that list as a basis for <tt><see topic=\"ACL2____HINTS\">hints</see></tt> that you \u000A construct for a subsequent proof attempt.</p>"],

 "ACL2____REMOVE-INVISIBLE-FNS":[["Loop-stopper"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see> (<see topic=\"ACL2____BINARY-_B2\">binary-+</see> unary-- foo)\u000A(<see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see> (<see topic=\"COMMON-LISP_____B2\">+</see> unary-- foo)</code> \u000A \u000A <p>The setting above has makes unary functions <tt><see topic=\"ACL2____UNARY--\">unary--</see></tt> and <v>foo</v> no \u000A longer ``invisible'' for the purposes of applying permutative <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> rules to <tt><see topic=\"ACL2____BINARY-_B2\">binary-+</see></tt> trees.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____REMOVE-INVISIBLE-FNS\">remove-invisible-fns</see> top-fn unary-fn1 ... unary-fnk)</code> \u000A \u000A <p>where <v>top-fn</v> is a function symbol and the <v>unary-fni</v> are unary \u000A function symbols, or more generally, these are all macro aliases for function \u000A symbols (see <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>).</p> \u000A \u000A <p>See <see topic=\"ACL2____ADD-INVISIBLE-FNS\">add-invisible-fns</see> and also see <see topic=\"ACL2____INVISIBLE-FNS-TABLE\">invisible-fns-table</see> and \u000A see <see topic=\"ACL2____SET-INVISIBLE-FNS-TABLE\">set-invisible-fns-table</see>.</p>"],

 "ACL2____PARALLEL-PROOF":[["Parallelism"],"ACL2 Sources","ACL2","<p>Here we document support for parallel proof in ACL2(p), an \u000A experimental extension of ACL2; also see <see topic=\"ACL2____PARALLELISM\">parallelism</see>, and for parallel \u000A programming in particular, see <see topic=\"ACL2____PARALLEL-PROGRAMMING\">parallel-programming</see>.</p>"],

 "ACL2____PAIRLIS_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The Common Lisp language allows its <tt><see topic=\"COMMON-LISP____PAIRLIS\">pairlis</see></tt> function to \u000A construct an alist in any order!  So we have to define our own version, \u000A <v>pairlis$</v>.  It returns the list of pairs obtained by <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt>ing \u000A together successive respective members of the given lists until the first list \u000A runs out.  (Hence in particular, if the second argument is <v>nil</v> then each \u000A element of the first argument is paired with <v>nil</v>.)</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>pairlis$</v> requires that its arguments are true \u000A lists.</p> \u000A \u000A <p><b>Function: </b><srclink>pairlis$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> pairlis$ (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> y))))\u000A       (<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A                         (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CAR\">car</see> y))\u000A                                  (<see topic=\"ACL2____PAIRLIS_42\">pairlis$</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> y)))))\u000A            :exec (pairlis$-tailrec x y nil)))</code>"],

 "ACL2____Guessing_02the_02Type_02of_02a_02Newly_02Admitted_02Function":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>When a function is admitted to the logic, ACL2 tries to ``guess'' \u000A what type of object it returns.  This guess is codified as a term that \u000A expresses a property of the value of the function.  For <v>app</v> the term \u000A is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____OR\">OR</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (APP X Y))\u000A    (<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (APP X Y) Y))</code> \u000A \u000A <p>which says that <v>app</v> returns either a cons or its second argument. \u000A This formula is added to ACL2's rule base as a <tt><see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A rule.  Later we will discuss how rules are used by the ACL2 theorem prover. \u000A The point here is just that when you add a definition, the database of rules \u000A is updated, not just by the addition of the definitional axiom, but by several \u000A new rules.</p> \u000A \u000A <p>You should now return to <see topic='ACL2____Revisiting_02the_02Admission_02of_02App'>the Walking Tour</see>.</p>"],

 "ACL2____GROUND-ZERO":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<p>ACL2 concludes its initialization <v>(boot-strapping)</v> procedure \u000A by defining the theory <v>ground-zero</v>; see <see topic=\"ACL2____THEORIES\">theories</see>.  In fact, this \u000A theory is just the theory defined by <v>(<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)</v> at the \u000A conclusion of initialization; see <see topic=\"ACL2____CURRENT-THEORY\">current-theory</see>.</p> \u000A \u000A <p>Note that by evaluating the event</p> \u000A \u000A <code>(<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> 'ground-zero))</code> \u000A \u000A <p>you can restore the current theory to its value at the time you started up \u000A ACL2.</p>"],

 "ACL2____GET-WORMHOLE-STATUS":[["Wormhole"],"ACL2 Sources","ACL2","<p>General Form: \u000A (get-wormhole-status name state)</p> \u000A \u000A <p><v>Name</v> should be the name of a wormhole (see <see topic=\"ACL2____WORMHOLE\">wormhole</see>).  This \u000A function returns an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) of the form <v>(<see topic=\"ACL2____MV\">mv</see>\u000A nil s state)</v>, where <v>s</v> is the status of the named wormhole.  The status \u000A is obtained by reading the oracle in the ACL2 <tt><see topic=\"ACL2____STATE\">state</see></tt>.</p> \u000A \u000A <p>This function makes the status of a wormhole visible outside the wormhole. \u000A But since this function takes <tt><see topic=\"ACL2____STATE\">state</see></tt> and modifies it, the function may \u000A only be used in contexts in which you may change <tt><see topic=\"ACL2____STATE\">state</see></tt>.  Otherwise, \u000A the wormhole status may stay in the wormhole.  See <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt> and \u000A <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>.</p>"],

 "ACL2____FAST-ALIST-LEN":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically this function counts how many elements would remain in the alist \u000A were we to shrink it with <tt><see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see></tt>.</p> \u000A \u000A <p>Good discipline requires that the alist is a fast alist.  Under the hood, \u000A when the alist is a fast alist we can simply call the underlying Common Lisp \u000A function <v>hash-table-count</v> on the associated hash table, which is very \u000A fast and doesn't require us to actually shrink the alist.</p> \u000A \u000A <p><b>Function: </b><srclink>fast-alist-len</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fast-alist-len (alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see> alist nil)))</code>"],

 "ACL2____DEFTHMD":[["Events"],"ACL2 Sources","ACL2","<p>Use <v>defthmd</v> instead of <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> when you want to disable \u000A a theorem immediately after proving it.  This macro has been provided for \u000A users who prefer working in a mode where theorems are only enabled when \u000A explicitly directed by <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt>.  Specifically, the form</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHMD\">defthmd</see> NAME TERM ...)</code> \u000A \u000A <p>expands to:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____PROGN\">progn</see>\u000A  (<see topic=\"ACL2____DEFTHMD\">defthmd</see> NAME TERM ...)\u000A  (<see topic=\"ACL2____WITH-OUTPUT\">with-output</see>\u000A   :off summary\u000A   (<see topic=\"ACL2____IN-THEORY\">in-theory</see> (<see topic=\"ACL2____DISABLE\">disable</see> NAME)))\u000A  (<see topic=\"ACL2____VALUE-TRIPLE\">value-triple</see> '(:defthmd NAME))).</code> \u000A \u000A <p>Note that <v>defthmd</v> commands are never redundant (see <see topic=\"ACL2____REDUNDANT-EVENTS\">redundant-events</see>).  Even if the <v>defthm</v> event is redundant, then the \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event will still be executed.</p> \u000A \u000A <p>The summary for the <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> event is suppressed.  See <see topic=\"ACL2____DEFTHM\">defthm</see> for documentation of <v>defthm</v>.</p>"],

 "ACL2____BOOKS-TOUR":[["Books"],"ACL2 Sources","ACL2","<p>The tour begins with <see topic=\"ACL2____BOOK-EXAMPLE\">book-example</see>.</p>"],

 "ACL2____ACL2_02Characters":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>ACL2 accepts 256 distinct characters, which are the characters \u000A obtained by applying the function <tt><see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A to each integer from 0 to 255.  Among these, Common Lisp designates certain \u000A ones as <v>*standard-characters*</v>, namely those of the form <v>(<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see>\u000A n)</v> where n is from 33 to 126, together with <v>#\\Newline</v> and \u000A <v>#\\Space</v>.  The actual standard characters may be viewed by evaluating the \u000A constant expression <v>*standard-chars*</v>.</p> \u000A \u000A <p>The standard character constants are written by writing a hash mark \u000A followed by a backslash (#\\) followed by the character.</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____CHARACTERP\">characterp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A recognizes characters.  For more details, See <see topic=\"ACL2____CHARACTERS\">characters</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see>.</p>"],

 "ACL2-PC____SAVE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(save lemma3-attempt)\u000A\u000AGeneral Form:\u000A(save &amp;optional name do-it-flg)</code> \u000A \u000A <p>Saves the current proof-checker state by ``associating'' it with the given \u000A name.  Submit <v>(<see topic=\"ACL2____RETRIEVE\">retrieve</see> name)</v> to Lisp to get back to this proof-checker \u000A state.  If <v>verify</v> was originally supplied with an event name, then the \u000A argument can be omitted in favor of that name as the default.</p> \u000A \u000A <p><b>Remark</b> that if a <v>save</v> has already been done with the indicated \u000A name (or the default event name), then the user will be queried regarding \u000A whether to go ahead with the save &mdash; except, if <v>do-it-flg</v> is \u000A supplied and not <v>nil</v>, then there will be no query and the <v>save</v> will \u000A be effected.</p> \u000A \u000A <p>See also the documentation for <v>retrieve</v> and <v>unsave</v>.</p>"],

 "ACL2-PC____REDUCE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Areduce -- attempt to prove the current goal without using induction\u000A(reduce (&quot;Subgoal 2&quot; :by foo) (&quot;Subgoal 1&quot; :use bar))\u000A       -- attempt to prove the current goal without using\u000A          induction, with the indicated hints\u000A\u000AGeneral Form:\u000A(reduce &amp;rest hints)</code> \u000A \u000A <p>Attempt to prove the current goal without using induction, using the \u000A indicated hints (if any).  A subgoal will be created for each goal that would \u000A have been pushed for proof by induction in an ordinary proof.</p> \u000A \u000A <p>Notice that unlike <v>prove</v>, the arguments to <v>reduce</v> are spread out, \u000A and are all hints.</p> \u000A \u000A <p><v>Reduce</v> is similar to <v>bash</v> in that neither of these allows \u000A induction.  But <v>bash</v> only allows simplification, while <v>reduce</v> allows \u000A processes <v>eliminate-destructors</v>, <v>fertilize</v>, <v>generalize</v>, and \u000A <v>eliminate-irrelevance</v>.</p> \u000A \u000A <p><b>Remark:</b> Induction will be used to the extent that it is ordered \u000A explicitly in the hints.</p>"],

 "ACL2-PC____P-TOP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Ap-top</code> \u000A \u000A <p>For example, if the conclusion is <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____AND\">and</see> x (p y)) (foo z))</v> and \u000A the current subterm is <v>(p y)</v>, then <v>p-top</v> will print <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____AND\">and</see> x\u000A (*** (p y) ***)) (foo z))</v>.</p> \u000A \u000A <p>Prettyprint the the conclusion, highlighting the current term.  The usual \u000A user syntax is used, as with the command <v>p</v> (as opposed to <v>pp</v>).  This \u000A is illustrated in the example above, where one would <v>*not*</v> see <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see>\u000A (<see topic=\"COMMON-LISP____IF\">if</see> x (*** (p y) ***) 'nil) (foo z))</v>.</p> \u000A \u000A <p><b>Remark</b> (obscure): In some situations, a term of the form <v>(<see topic=\"COMMON-LISP____IF\">if</see> x t\u000A y)</v> occurring inside the current subterm will not print as <v>(<see topic=\"COMMON-LISP____OR\">or</see> x y)</v>, \u000A when <v>x</v> isn't a call of a boolean primitive.  There's nothing incorrect \u000A about this, however.</p>"],

 "ACL2-PC____DO-ALL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(do-all induct p prove)\u000A\u000AGeneral Form:\u000A(do-all &amp;rest instruction-list)</code> \u000A \u000A <p>Run the indicated instructions until there is a hard ``failure''.  The \u000A instruction ``succeeds'' if and only if each instruction in \u000A <v>instruction-list</v> does.  (See the documentation for <v>sequence</v> for an \u000A explanation of ``success'' and ``failure.'')  As each instruction is executed, \u000A the system will print the usual prompt followed by that instruction, unless \u000A the global state variable <v>pc-print-prompt-and-instr-flg</v> is <v>nil</v>.</p> \u000A \u000A <p><b>Remark:</b> If <v>do-all</v> ``fails'', then the failure is hard if and \u000A only if the last instruction it runs has a hard ``failure''.</p> \u000A \u000A <p>Obscure point: For the record, <v>(do-all ins_1 ins_2 ... ins_k)</v> is the \u000A same as <v>(sequence (ins_1 ins_2 ... ins_k))</v>.</p>"],

 "ACL2-PC____BASH":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Abash -- attempt to prove the current goal by simplification alone\u000A(bash (&quot;Subgoal 2&quot; :by foo) (&quot;Subgoal 1&quot; :use bar))\u000A     -- attempt to prove the current goal by simplification alone,\u000A        with the indicated hints\u000A\u000AGeneral Form:\u000A(bash &amp;rest hints)</code> \u000A \u000A <p>Call the theorem prover's simplifier, creating a subgoal for each resulting \u000A goal.</p> \u000A \u000A <p>Notice that unlike <v>prove</v>, the arguments to <v>bash</v> are spread out, \u000A and are all hints.</p> \u000A \u000A <p><v>Bash</v> is similar to <v>reduce</v> in that neither of these allows \u000A induction.  But <v>bash</v> only allows simplification, while <v>reduce</v> allows \u000A processes <v>eliminate-destructors</v>, <v>fertilize</v>, <v>generalize</v>, and \u000A <v>eliminate-irrelevance</v>.</p> \u000A \u000A <p><b>Remark:</b> All forcing rounds will be skipped (unless there are more \u000A than 15 subgoals generated in the first forcing round, an injustice that \u000A should be rectified, but might remain unless there is pressure to fix \u000A it).</p>"],

 "COMMON-LISP____STRING_E3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRING_E3_D3\">String&gt;=</see> str1 str2)</v> is non-<v>nil</v> if and only if the string \u000A <v>str2</v> precedes the string <v>str1</v> lexicographically or the strings are \u000A equal.  When non-<v>nil</v>, <v>(<see topic=\"COMMON-LISP____STRING_E3_D3\">string&gt;=</see> str1 str2)</v> is the first \u000A position (zero-based) at which the strings differ, if they differ, and \u000A otherwise is their common length.  See <see topic=\"COMMON-LISP____STRING_E3\">string&gt;</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string&gt;=</v> specifies that its arguments are \u000A strings.</p> \u000A \u000A <p><v>String&gt;=</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string&gt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string&gt;= (str1 str2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> str1 str2)\u000A           (<see topic=\"COMMON-LISP____LENGTH\">length</see> str1)\u000A           (<see topic=\"COMMON-LISP____STRING_E3\">string&gt;</see> str1 str2)))</code>"],

 "COMMON-LISP____STRING_C3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRING_C3_D3\">String&lt;=</see> str1 str2)</v> is non-<v>nil</v> if and only if the string \u000A <v>str1</v> precedes the string <v>str2</v> lexicographically or the strings are \u000A equal.  When non-<v>nil</v>, <v>(<see topic=\"COMMON-LISP____STRING_C3_D3\">string&lt;=</see> str1 str2)</v> is the first \u000A position (zero-based) at which the strings differ, if they differ, and \u000A otherwise is their common length.  See <see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string&lt;=</v> specifies that its arguments are \u000A strings.</p> \u000A \u000A <p><v>String&lt;=</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string&lt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string&lt;= (str1 str2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> str1 str2)\u000A           (<see topic=\"COMMON-LISP____LENGTH\">length</see> str1)\u000A           (<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> str1 str2)))</code>"],

 "COMMON-LISP____SIGNUM":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____SIGNUM\">Signum</see> x)</v> is <v>0</v> if <v>x</v> is <v>0</v>, <v>-1</v> if <v>x</v> is \u000A negative, and is <v>1</v> otherwise.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>signum</v> requires its argument to be rational \u000A (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) number.</p> \u000A \u000A <p><v>Signum</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p>From ``Common Lisp the Language'' page 206, we see a definition of \u000A <v>signum</v> in terms of <tt><see topic=\"COMMON-LISP____ABS\">abs</see></tt>.  As explained elsewhere (see <see topic=\"COMMON-LISP____ABS\">abs</see>), \u000A the <see topic=\"ACL2____GUARD\">guard</see> for <tt><see topic=\"COMMON-LISP____ABS\">abs</see></tt> requires its argument to be a rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) number; hence, we make the same restriction for \u000A <v>signum</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>signum</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> signum (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ZEROP\">zerop</see> x) 0 (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MINUSP\">minusp</see> x) -1 1)))</code>"],

 "COMMON-LISP____REVAPPEND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____REVAPPEND\">Revappend</see> x y)</v> <see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see>s the <see topic=\"COMMON-LISP____REVERSE\">reverse</see> of the \u000A list <v>x</v> to <v>y</v>, which is also typically a list.</p> \u000A \u000A <p>The following theorem characterizes this English description.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y)\u000A       (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x) y))</code> \u000A \u000A <p>Hint: This lemma follows immediately from the definition of <tt><see topic=\"COMMON-LISP____REVERSE\">reverse</see></tt> \u000A and the following lemma.</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> revappend-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y) z)\u000A         (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x (<see topic=\"COMMON-LISP____APPEND\">append</see> y z))))</code> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x y)</v> requires that <v>x</v> is a true \u000A list.</p> \u000A \u000A <p><v>Revappend</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>revappend</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> revappend (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A           y (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) y))))</code>"],

 "COMMON-LISP____MOD":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>ACL2 !&gt;(<see topic=\"COMMON-LISP____MOD\">mod</see> 14 3)\u000A2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____MOD\">mod</see> -14 3)\u000A1\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____MOD\">mod</see> 14 -3)\u000A-1\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____MOD\">mod</see> -14 -3)\u000A-2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____MOD\">mod</see> -15 -3)\u000A0\u000AACL2 !&gt;</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____MOD\">Mod</see> i j)</v> is that number <v>k</v> that <v>(<see topic=\"COMMON-LISP_____A2\">*</see> j (<see topic=\"COMMON-LISP____FLOOR\">floor</see> i j))</v> added to \u000A <v>k</v> equals <v>i</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____MOD\">mod</see> i j)</v> requires that <v>i</v> and <v>j</v> are \u000A rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Mod</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>mod</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> mod (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> y 0)))))\u000A       (<see topic=\"COMMON-LISP____-\">-</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> x y) y)))</code>"],

 "COMMON-LISP____MAKE-LIST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For a nonnegative integer <v>size</v>, <v>(<see topic=\"COMMON-LISP____MAKE-LIST\">Make-list</see> size)</v> is a \u000A list of elements of length <v>size</v>, each of which is initialized to the \u000A <v>:initial-element</v> (which defaults to <v>nil</v>).</p> \u000A \u000A <p><v>Make-list</v> is a macro in ACL2, defined in terms of a tail recursive \u000A function <v>make-list-ac</v> whose <see topic=\"ACL2____GUARD\">guard</see> requires <v>size</v> to be a \u000A nonnegative integer.  <v>Make-list</v> is a Common Lisp function.  See any \u000A Common Lisp documentation for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>make-list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> make-list (size &amp;key initial-element)\u000A          (<see topic=\"COMMON-LISP____CONS\">cons</see> 'make-list-ac\u000A                (<see topic=\"COMMON-LISP____CONS\">cons</see> size\u000A                      (<see topic=\"COMMON-LISP____CONS\">cons</see> initial-element (<see topic=\"COMMON-LISP____CONS\">cons</see> 'nil 'nil)))))</code> \u000A <p><b>Function: </b><srclink>make-list-ac</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-list-ac (n val ac)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZP\">zp</see> n) ac)\u000A             (t (make-list-ac (<see topic=\"COMMON-LISP____1-\">1-</see> n)\u000A                              val (<see topic=\"COMMON-LISP____CONS\">cons</see> val ac)))))</code>"],

 "COMMON-LISP____INTEGER-LENGTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For non-negative integers, <v>(<see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see> i)</v> is the minimum \u000A number of bits needed to represent the integer.  Any integer can be \u000A represented as a signed two's complement field with a minimum of \u000A <v>(<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see> i) 1)</v> bits.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>integer-length</v> requires its argument to be an \u000A integer.  <v>Integer-length</v> is defined in Common Lisp.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>integer-length</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> integer-length (i)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZIP\">zip</see> i)\u000A           0\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____D3\">=</see> i -1)\u000A               0 (<see topic=\"COMMON-LISP_____B2\">+</see> 1 (<see topic=\"COMMON-LISP____INTEGER-LENGTH\">integer-length</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> i 2))))))</code>"],

 "COMMON-LISP____COERCE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-coerce</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x y)\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see>\u000A        ((<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 'list)\u000A         (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A             (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list)\u000A           nil))\u000A        (t\u000A         (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"ACL2____MAKE-CHARACTER-LIST\">make-character-list</see> x) 'string))))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____COERCE\">coerce</see> x y)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 'list)\u000A    (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> y 'string)\u000A      (<see topic=\"ACL2____CHARACTER-LISTP\">character-listp</see> x)\u000A    nil))</code> \u000A \u000A <p>Also see community book <v>books/misc/fast-coerce.lisp</v>, contributed by \u000A Jared Davis, for a version of <v>coerce</v> that may be faster for Common Lisp \u000A implementations other than CCL 1.3 or later, if the second argument is \u000A <v>'list</v> (for coercing a string to a list).</p>"],

 "COMMON-LISP____CODE-CHAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-code-char</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A                (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0)\u000A                (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 256))\u000A           (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> x)\u000A         (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0)))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)\u000A     (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0)\u000A     (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 256))</code> \u000A \u000A <p>ACL2 supports 8-bit <see topic=\"ACL2____CHARACTERS\">characters</see>.  Inputs not between <v>0</v> and \u000A <v>255</v> are treated as <v>0</v>.</p>"],

 "ACL2____What_02is_02a_02Mathematical_02Logic_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p><img src='proof.gif'></img></p> \u000A \u000A <p>A mathematical logic is a formal system of formulas (<b>axioms</b>) and \u000A <b>rules</b> for deriving other formulas, called <b>theorems</b>.</p> \u000A \u000A <p>A <b>proof</b> is a derivation of a theorem.  To see a concrete proof tree, \u000A click <see topic='ACL2____A_02Trivial_02Proof'>here</see>.</p> \u000A \u000A <p>Why should you care?  The neat thing about Theorems is that they are \u000A ``true.''  More precisely, if all the axioms are valid and the rules are \u000A validity preserving, then anything derived from the axioms via the rules is \u000A valid.</p> \u000A \u000A <p>So, if you want to determine if some formula is true, <b>prove it</b>.</p> \u000A \u000A <p><see topic='ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____UNTRACE_42":[["Trace"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____UNTRACE_42\">untrace$</see>)         ; untrace all functions previously\u000A                   ; traced (e.g. with trace$ or trace!)\u000A(<see topic=\"ACL2____UNTRACE_42\">untrace$</see> foo bar) ; as above, except only untrace foo and bar\u000A\u000AGeneral Forms:\u000A(<see topic=\"ACL2____UNTRACE_42\">untrace$</see>)                 ; untrace all (as noted above)\u000A(<see topic=\"ACL2____UNTRACE_42\">untrace$</see> fn1 fn2 ... fnk) ; untrace the indicated functions</code> \u000A \u000A <p>where the <v>fni</v> were previously traced (e.g. with <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> or \u000A <tt><see topic=\"ACL2____TRACE_12\">trace!</see></tt>).</p> \u000A \u000A <p><v>Untrace$</v> undoes the effect of <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt>.  See <see topic=\"ACL2____TRACE_42\">trace$</see>.  The \u000A value returned by <v>untrace$</v> gives the list of functions for which tracing \u000A is being removed.</p>"],

 "ACL2____UBT_12":[["History"],"ACL2 Sources","ACL2","<code>Example:\u000A:ubt! :x-4</code> \u000A \u000A <p>The keyword <see topic=\"ACL2____COMMAND\">command</see> <v>:ubt!</v> is the same as <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt>, but \u000A with related queries suppressed appropriately, and with a guarantee that it is \u000A ``error-free.''  More precisely, the value returned by <v>:ubt!</v>  will always \u000A be of the form <v>(<see topic=\"ACL2____MV\">mv</see> nil val state)</v>.  <v>:</v><tt><see topic=\"ACL2____OOPS\">Oops</see></tt> will undo the last \u000A <v>:ubt!</v>.  See <see topic=\"ACL2____UBT\">ubt</see>, see <see topic=\"ACL2____UBU\">ubu</see>, and see <see topic=\"ACL2____U\">u</see>.</p>"],

 "ACL2____START-PROOF-TREE":[["Proof-tree"],"ACL2 Sources","ACL2","<p>Also see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> and see <see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see>.  Note \u000A that <v>:start-proof-tree</v> works by removing <v>'</v><tt><see topic=\"ACL2____PROOF-TREE\">proof-tree</see></tt> from \u000A the <v>inhibit-output-lst</v>; see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.</p> \u000A \u000A <p>Explanations of proof tree displays may be found elsewhere: see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.  In a nutshell: <v>:start-proof-tree</v> causes proof tree display \u000A to be turned on, once it has been turned off by <v>:</v><tt><see topic=\"ACL2____STOP-PROOF-TREE\">stop-proof-tree</see></tt>.</p> \u000A \u000A <p>Do not attempt to invoke <v>start-proof-tree</v> during an interrupt in the \u000A middle of a proof.</p>"],

 "ACL2____SET-PRINT-BASE-RADIX":[["Io"],"ACL2 Sources","ACL2","<p>See <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt> and <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt> for detailed \u000A discussions of those functions.  <v>Set-print-base-radix</v> combines their \u000A functionality by setting the radix (as is done by <tt><see topic=\"ACL2____SET-PRINT-BASE\">set-print-base</see></tt>), and \u000A then causing the radix to be printed (as is done by <tt><see topic=\"ACL2____SET-PRINT-RADIX\">set-print-radix</see></tt>) \u000A exactly when the specified radix is not 10.</p> \u000A \u000A <p>Here is a sample log.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25 25/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see> 16 state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(#x19 #x19/3)\u000AACL2 !&gt;(<see topic=\"ACL2____SET-PRINT-BASE-RADIX\">set-print-base-radix</see> 10 state)\u000A&lt;state&gt;\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LIST\">list</see> 25 25/3)\u000A(25 25/3)\u000AACL2 !&gt;</code>"],

 "ACL2____Revisiting_02the_02Admission_02of_02App":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Evaluating_02App_02on_02Sample_02Input'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>Here is the definition of <v>app</v> again with certain parts highlighted. \u000A If you are taking the Walking Tour, please read the text carefully and click \u000A on each of the links below, <b>except those marked</b> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>. \u000A Then come <b>back</b> here.</p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code><see topic='ACL2____About_02the_02Prompt'>ACL2 !&gt;</see><b>(defun app (x y)</b> \u000A  <b>(cond ((endp x) y)</b> \u000A        <b>(t (cons (car x) </b> \u000A                 <b>(app (cdr x) y)))))</b> \u000A \u000AThe <see topic='ACL2____About_02the_02Admission_02of_02Recursive_02Definitions'>admission</see> of APP is trivial, using the \u000Arelation <see topic=\"ACL2____O_C3\">o&lt;</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> (which is known to be well-founded on \u000Athe domain recognized by <see topic=\"ACL2____O-P\">o-p</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) and the measure \u000A(<see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> X).  We <see topic='ACL2____Guessing_02the_02Type_02of_02a_02Newly_02Admitted_02Function'>observe</see> that the \u000A<see topic='ACL2____About_02Types'>type</see> of APP is described by the theorem (OR \u000A(CONSP (APP X Y)) (EQUAL (APP X Y) Y)).  We used primitive type \u000Areasoning. \u000A \u000A<see topic='ACL2____The_02Event_02Summary'>Summary</see> \u000AForm:  ( DEFUN APP ...) \u000ARules: ((:FAKE-RUNE-FOR-TYPE-SET NIL)) \u000AWarnings:  None \u000ATime:  0.03 seconds (prove: 0.00, print: 0.00, other: 0.03) \u000A APP \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p><see topic='ACL2____Evaluating_02App_02on_02Sample_02Input'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____QUANTIFIERS-USING-DEFUN-SK":[["Quantifiers"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see> for the context of this example.  It should \u000A be compared to a corresponding example in which a simpler proof is attained by \u000A using recursion in place of explicit quantification; see <see topic=\"ACL2____QUANTIFIERS-USING-RECURSION\">quantifiers-using-recursion</see>.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A; We prove that if every member A of each of two lists satisfies the\u000A; predicate (P A), then this holds of their append; and, conversely.\u000A\u000A; Here is a solution using explicit quantification.\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p (x) t)\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-p (x)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> a (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x)\u000A                     (p a))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x1) (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x2))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-p-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (forall-p (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (forall-p x1) (forall-p x2)))\u000A  :hints ((&quot;Goal&quot; ; ``should'' disable forall-p-necc, but no need\u000A           :use\u000A           ((:instance forall-p-necc\u000A                       (x (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A                       (a (forall-p-witness x1)))\u000A            (:instance forall-p-necc\u000A                       (x (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A                       (a (forall-p-witness x2)))\u000A            (:instance forall-p-necc\u000A                       (x x1)\u000A                       (a (forall-p-witness (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))))\u000A            (:instance forall-p-necc\u000A                       (x x2)\u000A                       (a (forall-p-witness (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))))))))</code> \u000A \u000A <p>Also see <see topic=\"ACL2____QUANTIFIERS-USING-DEFUN-SK-EXTENDED\">quantifiers-using-defun-sk-extended</see> for an elaboration on \u000A this example.</p>"],

 "ACL2____PR_12":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000A:pr! fn ; prints the rules from the definition of fn (including any\u000A        ; :type-prescription rule and :definition rule), as well as all other\u000A        ; rules created by the command that created by fn (which could be\u000A        ; many rules if, for example, fn was defined by an include-book\u000A        ; command).\u000A\u000A:pr! :max ; prints all the rules stored by the most recent command</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PR\">pr</see>, which is similar but works at the event level instead \u000A of at the command level.</p> \u000A \u000A <p><tt><see topic=\"ACL2____PR\">Pr</see></tt> takes one argument, a command descriptor, and prints the rules \u000A created by the corresponding event.  In each case it prints the rune, the \u000A current enabled/disabled status, and other appropriate fields from the rule. \u000A See <see topic=\"ACL2____PR\">pr</see> for further details.</p>"],

 "ACL2____PRINT-DOC-START-COLUMN":[["Legacy-documentation"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> print-doc-start-column nil)\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> print-doc-start-column 17)</code> \u000A \u000A <p>This <see topic=\"ACL2____STATE\">state</see> global variable controls the column in which the \u000A ``one-liner'' of a formatted <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> string is printed. \u000A Generally, when <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> is used to print a <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> \u000A string, the name of the documented concept is printed and then <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> tabs over to <v>print-doc-start-column</v> and prints the one-liner.  If \u000A the name extends past the desired column, <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> outputs a carriage \u000A return and then tabs over to the column.  If <v>print-doc-start-column</v> is \u000A <v>nil</v>, <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> just starts the one-liner two spaces from the end \u000A of the name, on the same line.  The initial value of \u000A <v>print-doc-start-column</v> is 15.</p>"],

 "ACL2____POSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____POSP\">posp</see> x)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____ZP\">zp</see> x))</v> (see \u000A <see topic=\"ACL2____ZP\">zp</see>) and also to <v>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)))</v>.</p> \u000A \u000A <p>The community book <see topic=\"ACL2____ARITHMETIC_F2NATP-POSP\">arithmetic/natp-posp</see> has some lightweight rules \u000A for reasoning about <v>posp</v> and <v>natp</v>, and is included in the <see topic=\"ACL2____ARITHMETIC-1\">arithmetic-1</see> library.  For a somewhat heavier and more comprehensive \u000A alternative, you may wish to instead see the <see topic=\"ACL2____ARITH-EQUIVS\">arith-equivs</see> book.</p> \u000A \u000A <p><b>Function: </b><srclink>posp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> posp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 x)))</code>"],

 "ACL2____PCS":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pcs 1 5              ; print commands 1 through 5\u000A:pcs 5 1              ; same as above\u000A:pcs :x (:x -3)       ; print the 3 most recently executed commands\u000A:pcs fn assoc-of-fn   ; print the commands between the one that introduced\u000A                      ; fn and the one that introduced assoc-of-fn</code> \u000A \u000A <p><v>Pcs</v> takes two arguments, both of which are <see topic=\"ACL2____COMMAND\">command</see> descriptors, \u000A and prints the <see topic=\"ACL2____COMMAND\">command</see>s between them with <tt><see topic=\"ACL2____PC\">pc</see></tt>.  The order of the \u000A two descriptors is irrelevant.  See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see> for a \u000A description of <see topic=\"ACL2____COMMAND\">command</see> descriptors.  See <see topic=\"ACL2____PC\">pc</see> for a description of \u000A the format in which <see topic=\"ACL2____COMMAND\">command</see>s are displayed.</p> \u000A \u000A <p>For a related utility that simply returns a sequence of commands, see <see topic=\"ACL2____GET-COMMAND-SEQUENCE\">get-command-sequence</see>.</p>"],

 "ACL2____O-FIRST-COEFF":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>An ACL2 ordinal is either a natural number or, for an infinite \u000A ordinal, a list whose elements are exponent-coefficient pairs (see <see topic=\"ACL2____O-P\">o-p</see>).  In the latter case, this function returns the <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt> of the first \u000A pair in the list.  In the case of a natural number, this function returns the \u000A ordinal itself (since a natural number, n, can be thought of as (w^0)n).</p> \u000A \u000A <p>For the corresponding exponent, see <see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>o-first-coeff</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o-first-coeff (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x) x (<see topic=\"COMMON-LISP____CDAR\">cdar</see> x)))</code>"],

 "ACL2____NFIX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Nfix</v> simply returns any natural number argument unchanged, \u000A returning <v>0</v> on an argument that is not a natural number.  Also see <see topic=\"ACL2____IFIX\">ifix</see>, see <see topic=\"ACL2____RFIX\">rfix</see>, see <see topic=\"ACL2____REALFIX\">realfix</see>, and see <see topic=\"ACL2____FIX\">fix</see> for analogous \u000A functions that coerce to an integer, a rational number, a real, and a number, \u000A respectively.</p> \u000A \u000A <p><v>Nfix</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>nfix</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nfix (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x 0)) x 0))</code>"],

 "ACL2____NEVER-MEMOIZE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, this function just returns <v>nil</v>.  But execution of \u000A <v>(<see topic=\"ACL2____NEVER-MEMOIZE\">never-memoize</see> fn)</v> records that <v>fn</v> must never be memoized, so that \u000A any attempt to memoize <v>fn</v> will fail.</p> \u000A \u000A <p>Any function can be marked as unsafe to memoize; in fact, <v>fn</v> need not \u000A even be defined at the time it is marked.</p> \u000A \u000A <p>This is useful for prohibiting the memoization of functions that are known \u000A to involve destructive functions like <v>nreverse</v>.</p>"],

 "ACL2____INTERN_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Intern$</v> is a macro that behaves the same as the macro <tt><see topic=\"COMMON-LISP____INTERN\">intern</see></tt>, except for weakening the restriction to a fixed set of package names \u000A so that any package name other than <v>&quot;&quot;</v> is legal.  See <see topic=\"COMMON-LISP____INTERN\">intern</see>. \u000A Note that if you evaluate a call <v>(<see topic=\"ACL2____INTERN_42\">intern$</see> x y)</v> for which there is no \u000A package with name <v>y</v> that is known to ACL2, you will get an error.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____INTERN_42\">Intern$</see> x y)</v> expands to:</p> \u000A \u000A <code>(<see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> x (<see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see> y))</code> \u000A \u000A <p>See <see topic=\"ACL2____INTERN-IN-PACKAGE-OF-SYMBOL\">intern-in-package-of-symbol</see> and see <see topic=\"ACL2____PKG-WITNESS\">pkg-witness</see>.</p>"],

 "ACL2____FORALL":[["Defun-sk"],"ACL2 Sources","ACL2","<p>The symbol <v>forall</v> (in the ACL2 package) represents universal \u000A quantification in the context of a <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> form.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> \u000A and see <see topic=\"ACL2____EXISTS\">exists</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see> for an example illustrating how the use of \u000A recursion, rather than explicit quantification with <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>, may be \u000A preferable.</p>"],

 "ACL2____DISABLE-IMMEDIATE-FORCE-MODEP":[["Force"],"ACL2 Sources","ACL2","<code>General Form:\u000AACL2 !&gt;:disable-immediate-force-modep</code> \u000A \u000A <p>This event causes ACL2 to delay <see topic=\"ACL2____FORCE\">force</see>d hypotheses to the next \u000A forcing round, rather than attacking them immediately.  See <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>.  Or for more basic information, first see <see topic=\"ACL2____FORCE\">force</see> \u000A for a discussion of <see topic=\"ACL2____FORCE\">force</see>d case splits.</p> \u000A \u000A <p>Disable-immediate-force-modep is a macro that <see topic=\"ACL2____DISABLE\">disable</see>s the \u000A executable counterpart of the function symbol <tt><see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see></tt>. \u000A When you want to <see topic=\"ACL2____DISABLE\">disable</see> this mode in <see topic=\"ACL2____HINTS\">hints</see>, use a form such as \u000A one of the following (these are equivalent).</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (:executable-counterpart immediate-force-modep))\u000A:in-theory (<see topic=\"ACL2____DISABLE\">disable</see> (<see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>))</code>"],

 "ACL2____CASE-SPLIT-LIMITATIONS":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____CASE-SPLIT-LIMITATIONS\">case-split-limitations</see> (w state))\u000A(500 100)</code> \u000A \u000A <p>With the setting above, <v>clausify</v> will not try subsumption/replacement \u000A if more than 500 clauses are involved.  Furthermore, the simplifier, as it \u000A sweeps over a clause, will inhibit further case splits when it has accumulated \u000A 100 subgoals.  This inhibition is implemented by continuing to rewrite \u000A subsequent literals but not splitting out their cases.  This can produce \u000A literals containing <v>IF</v>s.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-CASE-SPLIT-LIMITATIONS\">set-case-split-limitations</see> for a more general discussion.</p>"],

 "ACL2____ACL2_02Strings":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Strings of ACL2 <see topic='ACL2____ACL2_02Characters'>characters</see> are written as sequences of characters \u000A delimited by ``double quotation marks'' \u000A (\").  To put a double quotation mark in a string (or, any other character \u000A such as backslash or newline that seems to cause problems), escape it by \u000A preceding it with a backslash (\\).</p> \u000A \u000A <p>The function <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A recognizes strings and <tt><see topic=\"COMMON-LISP____CHAR\">char</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A will fetch the nth character of a string.  There are many other primitives for \u000A handling strings, such as <tt><see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> for \u000A comparing two strings lexicographically.  We suggest you See <see topic=\"ACL2____PROGRAMMING\">programming</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> where we list all of the primitive ACL2 functions. \u000A Alternatively, see any Common Lisp language documentation.</p>"],

 "COMMON-LISP____REVERSE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____REVERSE\">Reverse</see> x)</v> is the result of reversing the order of the \u000A elements of the list or string <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>reverse</v> requires that its argument is a true list \u000A or a string.</p> \u000A \u000A <p><v>Reverse</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>reverse</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> reverse (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x))))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A              (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list) nil)\u000A                      'string))\u000A             (t (<see topic=\"COMMON-LISP____REVAPPEND\">revappend</see> x nil))))</code>"],

 "COMMON-LISP____LAMBDA":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____CHAR-EQUAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For <see topic=\"ACL2____CHARACTERS\">characters</see> <v>x</v> and <v>y</v>, <v>(<see topic=\"COMMON-LISP____CHAR-EQUAL\">char-equal</see> x y)</v> is \u000A true if and only if <v>x</v> and <v>y</v> are the same except perhaps for their \u000A case.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> on <v>char-equal</v> requires that its arguments are both \u000A standard <see topic=\"ACL2____CHARACTERS\">characters</see> (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Char-equal</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char-equal (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> x)\u000A                                   (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> y))))\u000A       (<see topic=\"COMMON-LISP____EQL\">eql</see> (<see topic=\"COMMON-LISP____CHAR-DOWNCASE\">char-downcase</see> x)\u000A            (<see topic=\"COMMON-LISP____CHAR-DOWNCASE\">char-downcase</see> y)))</code>"],

 "ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92_02_82cont_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>A <b>mechanical theorem prover</b> is a computer program that finds proofs \u000A of theorems.</p> \u000A \u000A <p><img src='automatic-theorem-prover.gif'></img></p> \u000A \u000A <p>The ideal mechanical theorem prover is <b>automatic</b>: you give it a \u000A formula and it gives you a proof of that formula or tells you there is no \u000A proof.</p> \u000A \u000A <p>Unfortunately, automatic theorem provers can be built only for very simple \u000A logics (e.g., <b>propositional calculus</b>) and even then practical \u000A considerations (e.g., how many centuries you are willing to wait) limit the \u000A problems they can solve.</p> \u000A \u000A <p><see topic='ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92_02_82cont_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____WOF":[["Io"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____WOF\">wof</see> &quot;tmp&quot; (<see topic=\"ACL2____PSO\">pso</see>)) ; same as (<see topic=\"ACL2____PSOF\">psof</see> &quot;tmp&quot;)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____WOF\">wof</see> filename form)</code> \u000A \u000A <p>where <v>filename</v> is a writable filename and <v>form</v> is any form that \u000A evaluates to an error triple (see <see topic=\"ACL2____PROGRAMMING-WITH-STATE\">programming-with-state</see>), that is, a \u000A multiple value of the form <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v>.  All output to channels \u000A <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt> and <tt><see topic=\"ACL2____PROOFS-CO\">proofs-co</see></tt> will be directed to the indicated \u000A file.  It is acceptable to replace <v>filename</v> with <v>(<see topic=\"COMMON-LISP____QUOTE\">quote</see>\u000A filename)</v>.</p> \u000A \u000A <p>Note that so-called comment-window output (see <see topic=\"ACL2____CW\">cw</see> and see <see topic=\"ACL2____OBSERVATION-CW\">observation-cw</see>) is not redirected by <v>wof</v> to a file, nor is printing from \u000A a <see topic=\"ACL2____WORMHOLE\">wormhole</see>.</p>"],

 "ACL2____U":[["History"],"ACL2 Sources","ACL2","<code>Example:\u000A:u</code> \u000A \u000A <p>The keyword <see topic=\"ACL2____COMMAND\">command</see> <v>:u</v> is the same as <v>:</v><tt><see topic=\"ACL2____UBT\">ubt</see></tt> \u000A <v>:</v><tt><see topic=\"COMMON-LISP____MAX\">max</see></tt>, but with related queries suppressed appropriately. \u000A <v>:</v><tt><see topic=\"ACL2____OOPS\">Oops</see></tt> will undo the last <v>:u</v>.  See <see topic=\"ACL2____UBT\">ubt</see>, see <see topic=\"ACL2____UBU\">ubu</see>, see <see topic=\"ACL2____UBT_12\">ubt!</see>, and see <see topic=\"ACL2____UBU_12\">ubu!</see>.</p>"],

 "ACL2____REDEF":[["Ld"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000AACL2 !&gt;:redef</code> \u000A \u000A <p>This command sets <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> to <v>'(:query\u000A . :overwrite)</v>.</p> \u000A \u000A <p>This command allows redefinition of functions and other <see topic=\"ACL2____EVENTS\">events</see> \u000A without undoing, but with a query that requires the user to acknowledge that \u000A the redefinition is intentional.  To avoid that query, see <see topic=\"ACL2____REDEF_12\">redef!</see> or \u000A for more options, see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>.</p>"],

 "ACL2____QUANTIFIERS-USING-DEFUN-SK-EXTENDED":[["Quantifiers"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____QUANTIFIERS-USING-DEFUN-SK\">quantifiers-using-defun-sk</see> for the context of this \u000A example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A; We prove that if every member A of each of two lists satisfies the\u000A; predicate (P A), then this holds of their append; and, conversely.\u000A\u000A; Here is a solution using explicit quantification.\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p (x) t)\u000A\u000A(<see topic=\"ACL2____DEFUN-SK\">defun-sk</see> forall-p (x)\u000A  (<see topic=\"ACL2____FORALL\">forall</see> a (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x)\u000A                     (p a))))\u000A\u000A; The defun-sk above introduces the following axioms.  The idea is that\u000A; (FORALL-P-WITNESS X) picks a counterexample to (forall-p x) if there is one.\u000A\u000A;   (<see topic=\"COMMON-LISP____DEFUN\">DEFUN</see> FORALL-P (X)\u000A;     (<see topic=\"COMMON-LISP____LET\">LET</see> ((A (FORALL-P-WITNESS X)))\u000A;          (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A X) (P A))))\u000A;\u000A;   (<see topic=\"ACL2____DEFTHM\">DEFTHM</see> FORALL-P-NECC\u000A;     (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____MEMBER\">MEMBER</see> A X) (P A)))\u000A;              (<see topic=\"COMMON-LISP____NOT\">NOT</see> (FORALL-P X)))\u000A;     :HINTS ((&quot;Goal&quot; :USE FORALL-P-WITNESS)))\u000A\u000A; The following lemma seems critical.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> member-append\u000A  (<see topic=\"ACL2____IFF\">iff</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x1) (<see topic=\"COMMON-LISP____MEMBER\">member</see> a x2))))\u000A\u000A; The proof of forall-p-append seems to go out to lunch, so we break into\u000A; directions as shown below.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-p-append-forward\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (forall-p (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (forall-p x1) (forall-p x2)))\u000A  :hints ((&quot;Goal&quot; ; ``should'' disable forall-p-necc, but no need\u000A           :use\u000A           ((:instance forall-p-necc\u000A                       (x (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A                       (a (forall-p-witness x1)))\u000A            (:instance forall-p-necc\u000A                       (x (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A                       (a (forall-p-witness x2)))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-p-append-reverse\u000A  (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____AND\">and</see> (forall-p x1) (forall-p x2))\u000A           (forall-p (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2)))\u000A  :hints ((&quot;Goal&quot;\u000A           :use\u000A           ((:instance forall-p-necc\u000A                       (x x1)\u000A                       (a (forall-p-witness (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))))\u000A            (:instance forall-p-necc\u000A                       (x x2)\u000A                       (a (forall-p-witness (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))))))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> forall-p-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (forall-p (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (forall-p x1) (forall-p x2)))\u000A  :hints ((&quot;Goal&quot; :use (forall-p-append-forward\u000A                        forall-p-append-reverse))))</code>"],

 "ACL2____PF":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pf (:definition fn) ; prints the definition of fn as an equality\u000A:pf fn               ; same as above\u000A\u000A:pf (:rewrite foo)   ; prints the statement of the rewrite rule foo\u000A:pf foo              ; same as above</code> \u000A \u000A <p><v>pf</v> takes one argument, an event name or a <see topic=\"ACL2____RUNE\">rune</see>, and prints the \u000A formula associated with name.  If the argument is the name of a macro \u000A associated with a function name by <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see>, then the \u000A function name is used as the argument.</p>"],

 "ACL2____O-FIRST-EXPT":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>An ACL2 ordinal is either a natural number or, for an infinite \u000A ordinal, a list whose elements are exponent-coefficient pairs (see <see topic=\"ACL2____O-P\">o-p</see>).  In the latter case, this function returns the <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt> of the first \u000A pair in the list.  In the case of a natural number, the value returned is \u000A 0 (since a natural number, <v>n</v>, can be thought of as (w^0)n).</p> \u000A \u000A <p>For the corresponding coefficient, see <see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>o-first-expt</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o-first-expt (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x) (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x) 0 (<see topic=\"COMMON-LISP____CAAR\">caar</see> x)))</code>"],

 "ACL2____NOTE-2-7_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>In source file <v>axioms.lisp</v>, in order for proofs to succeed, \u000A (<v>make proofs</v>), the definitions of <tt><see topic=\"ACL2____ACL2-COUNT\">ACL2-count</see></tt> and \u000A <v>explode-atom</v> have been modified slightly, and lemma \u000A <v>standard-numberp-one</v> [modified after Version_3.4 to become \u000A <v>standardp-one</v>] has been given <v>:rule-classes nil</v>.</p> \u000A \u000A <p>All <tt><see topic=\"ACL2____SKIP-PROOFS\">skip-proofs</see></tt> forms have been eliminated from the nonstd books, \u000A thanks to Ruben Gamboa.</p> \u000A \u000A <p>The directory <v>books/sqrt/</v>, which was intended for ACL2(r), has been \u000A moved to <v>books/nonstd/sqrt/</v> and added as appropriate to \u000A <v>books/nonstd/Makefile</v>.</p> \u000A \u000A <p>Please see <see topic=\"ACL2____NOTE-2-7\">note-2-7</see> for changes to Version_2.7 of ACL2.</p>"],

 "ACL2____I-AM-HERE":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Example Input File for Rebuild:\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fn1 (x y) ...)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma1 ...)\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma2 ...)\u000A(<see topic=\"ACL2____I-AM-HERE\">i-am-here</see>)\u000AThe following lemma won't go through.  I started\u000Atyping the hint but realized I need to prove a\u000Alemma first.  See the failed proof attempt in foo.bar.\u000AI'm going to quit for the night now and resume tomorrow\u000Afrom home.\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> lemma3 ...\u000A  :hints ((&quot;Goal&quot; :use (:instance ???\u000A...</code> \u000A \u000A <p>By putting an <v>(<see topic=\"ACL2____I-AM-HERE\">i-am-here</see>)</v> form at the ``frontier'' of an evolving file \u000A of <see topic=\"ACL2____COMMAND\">command</see>s, you can use <tt><see topic=\"ACL2____REBUILD\">rebuild</see></tt> to load the file up to the \u000A <v>(<see topic=\"ACL2____I-AM-HERE\">i-am-here</see>)</v>.  <v>I-am-here</v> simply returns an error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) that indicates an error, and any form that ``causes an error'' \u000A will do the same job.  Note that the text of the file after the \u000A <v>(<see topic=\"ACL2____I-AM-HERE\">i-am-here</see>)</v> need not be machine readable.</p>"],

 "ACL2____GET-COMMAND-SEQUENCE":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____GET-COMMAND-SEQUENCE\">get-command-sequence</see> 4 12)\u000A:gcs 4 12 ; same as above\u000A(<see topic=\"ACL2____GET-COMMAND-SEQUENCE\">get-command-sequence</see> 4 :x)\u000A:gcs 4 :x ; same as above</code> \u000A \u000A <p>See <see topic=\"ACL2____PCS\">pcs</see> for a utility that prints abbreviated information about the \u000A <see topic=\"ACL2____COMMAND\">command</see>s that are between two command descriptors.  The utility \u000A <v>get-command-sequence</v> &mdash; or simply <v>gcs</v>, so that you can just \u000A type <v>:gcs</v> at the prompt &mdash; has the same syntax but instead of \u000A printing, it simply returns the corresponding list of commands.  More \u000A precisely, it returns an error triple <v>(<see topic=\"ACL2____MV\">mv</see> erp val state)</v> (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) such that if <v>erp</v> is not <v>nil</v>, then <v>val</v> is the \u000A desired list of commands.</p>"],

 "ACL2____FREE-VARIABLES-EXAMPLES":[["Free-variables"],"ACL2 Sources","ACL2","<p>The examples in the two sub-topics of this topic illustrate the \u000A handling of free variables in rules of class <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> \u000A (see <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES-REWRITE\">free-variables-examples-rewrite</see>) and of class <v>:</v><tt><see topic=\"ACL2____FORWARD-CHAINING\">forward-chaining</see></tt> (see <see topic=\"ACL2____FREE-VARIABLES-EXAMPLES-FORWARD-CHAINING\">free-variables-examples-forward-chaining</see>), \u000A respectively.  These implicitly illustrate <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> handling in \u000A rules of class <v>:</v><tt><see topic=\"ACL2____LINEAR\">linear</see></tt> as well.  Also see <see topic=\"ACL2____FREE-VARIABLES\">free-variables</see> \u000A and see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>.</p>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-BINARY-TREES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Classical Induction on Binary Trees</b>: To prove <v>(p x)</v>, for all \u000A <v>x</v>, by classical induction on binary tree structures, prove each of the \u000A following:</p> \u000A \u000A <code><i>Base Case</i>: \u000A(implies (atom x) (p x)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (atom x)) \u000A              (p (car x)) \u000A              (p (cdr x))) \u000A         (p x)) \u000A</code> \u000A \u000A <p>An argument analogous to that given in <see topic=\"ACL2____EXAMPLE-INDUCTION-SCHEME-ON-LISTS\">example-induction-scheme-on-lists</see> should convince you that <v>(p x)</v> holds \u000A for every object.</p> \u000A \u000A <p>A function that suggests this induction is:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> flatten (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      (<see topic=\"COMMON-LISP____LIST\">list</see> x)\u000A      (app (flatten (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A           (flatten (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))).</code>"],

 "ACL2____EQLABLE-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>eqlable-listp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of objects satisfying <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>eqlable-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> eqlable-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> l)\u000A           (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                (<see topic=\"ACL2____EQLABLE-LISTP\">eqlable-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)))\u000A           (<see topic=\"COMMON-LISP____EQUAL\">equal</see> l nil)))</code>"],

 "ACL2____ENABLE-IMMEDIATE-FORCE-MODEP":[["Force"],"ACL2 Sources","ACL2","<code>General Form:\u000AACL2 !&gt;:enable-immediate-force-modep</code> \u000A \u000A <p>This event causes ACL2 to attack <see topic=\"ACL2____FORCE\">force</see>d hypotheses immediately \u000A instead of delaying them to the next forcing round.  See <see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>.  Or for more basic information, first see <see topic=\"ACL2____FORCE\">force</see> \u000A for a discussion of <see topic=\"ACL2____FORCE\">force</see>d case splits.</p> \u000A \u000A <p>Enable-immediate-force-modep is a macro that <see topic=\"ACL2____ENABLE\">enable</see>s the executable \u000A counterpart of the function symbol <tt><see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see></tt>.  When you \u000A want to <see topic=\"ACL2____ENABLE\">enable</see> this mode in <see topic=\"ACL2____HINTS\">hints</see>, use a form such as one of the \u000A following (these are equivalent).</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> (:executable-counterpart immediate-force-modep))\u000A:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> (<see topic=\"ACL2____IMMEDIATE-FORCE-MODEP\">immediate-force-modep</see>))</code>"],

 "ACL2____ENABLE-FORCING":[["Force"],"ACL2 Sources","ACL2","<code>General Form:\u000AACL2 !&gt;:enable-forcing    ; allowed forced case splits</code> \u000A \u000A <p>See <see topic=\"ACL2____FORCE\">force</see> and see <see topic=\"ACL2____CASE-SPLIT\">case-split</see> for a discussion of forced case \u000A splits, which are turned back on by this command.  See <see topic=\"ACL2____DISABLE-FORCING\">disable-forcing</see> \u000A for an example showing how to turn off forced case splits.</p> \u000A \u000A <p><v>Enable-forcing</v> is actually a macro that <see topic=\"ACL2____ENABLE\">enable</see>s the executable \u000A counterpart of the function symbol <v>force</v>; see <see topic=\"ACL2____FORCE\">force</see>.  When you \u000A want to use <see topic=\"ACL2____HINTS\">hints</see> to turn on forced case splits, use a form such as one \u000A of the following (these are equivalent).</p> \u000A \u000A <code>:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> (:executable-counterpart force))\u000A:in-theory (<see topic=\"ACL2____ENABLE\">enable</see> (<see topic=\"ACL2____FORCE\">force</see>))</code>"],

 "ACL2____CHECKPOINT-FORCED-GOALS":[["Proof-tree"],"ACL2 Sources","ACL2","<code>Example forms:\u000A(<see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see> t)\u000A(<see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see> nil)</code> \u000A \u000A <p>Also see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>.</p> \u000A \u000A <p>By default, goals are not marked as checkpoints by a proof tree display (as \u000A described elsewhere; see <see topic=\"ACL2____PROOF-TREE\">proof-tree</see>) merely because they <see topic=\"ACL2____FORCE\">force</see> \u000A some hypotheses, thus possibly contributing to a forcing round.  However, some \u000A users may want such behavior, which will occur once the command \u000A <v>(<see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see></v> <v>t</v>) has been executed.  To return to the \u000A default behavior, use the command <v>(<see topic=\"ACL2____CHECKPOINT-FORCED-GOALS\">checkpoint-forced-goals</see> nil)</v>.</p>"],

 "ACL2____About_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Models_02of_02Computer_02Hardware_02and_02Software'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>ACL2 is used to construct mathematical models of computer hardware and \u000A software (i.e., ``digital systems'').</p> \u000A \u000A <p><img src='computing-machine.gif'></img></p> \u000A \u000A <p>A <b>mathematical model</b> is a set of mathematical formulas used to \u000A predict the behavior of some artifact.</p> \u000A \u000A <p>The use of mathematical models allows <b>faster</b> and <b>cheaper</b> \u000A delivery of <b>better</b> systems.</p> \u000A \u000A <p>Models need not be <b>complete</b> or <b>perfectly accurate</b> to be \u000A useful to the trained engineer.</p> \u000A \u000A <p>Click <see topic='ACL2____Models_02in_02Engineering'>here</see> for more \u000A discussion of these assertions in an engineering context.</p> \u000A \u000A <p><see topic='ACL2____Models_02of_02Computer_02Hardware_02and_02Software'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2-PC____RETRIEVE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____RETRIEVE\">retrieve</see> associativity-of-permutationp)\u000Aretrieve\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____RETRIEVE\">retrieve</see> &amp;optional name)</code> \u000A \u000A <p>Must be used from <v>outside</v> the interactive proof-checker loop.  If name \u000A is supplied and not <v>nil</v>, this causes re-entry to the interactive \u000A proof-checker loop in the state at which <v>save</v> was last executed for the \u000A indicated name.  (See documentation for <v>save</v>.)  If <v>name</v> is <v>nil</v> \u000A or is not supplied, then the user is queried regarding which proof-checker \u000A state to re-enter.  The query is omitted, however, if there only one \u000A proof-checker state is present that was saved with <v>save</v>, in which case \u000A that is the one that is used.  See also <v>unsave</v>.</p>"],

 "ACL2-PC____PRINT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(print (<see topic=\"COMMON-LISP____APPEND\">append</see> '(a b) '(c d)))\u000APrint the list (a b c d) to the terminal\u000A\u000AGeneral Forms:\u000A(print form)\u000A(print form t)</code> \u000A \u000A <p>Prettyprints the result of evaluating form.  The evaluation of <v>form</v> \u000A should return a single value that is not <tt><see topic=\"ACL2____STATE\">state</see></tt> or a single-threaded \u000A object (see <see topic=\"ACL2____STOBJ\">stobj</see>).  The optional second argument causes printing to be \u000A done without elision (so-called ``evisceration''; see <see topic=\"ACL2____EVISC-TUPLE\">evisc-tuple</see>).</p> \u000A \u000A <p>If the form you want to evaluate does not satisfy the criterion above, you \u000A should create an appropriate call of the <v>lisp</v> command instead.  Notice \u000A that this command always returns <v>(<see topic=\"ACL2____MV\">mv</see> nil nil state)</v> where the second \u000A result will always be <v>REPLACED-STATE</v>.</p>"],

 "COMMON-LISP____SUBST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____SUBST\">Subst</see> new old tree)</v> is obtained by substituting <v>new</v> for \u000A every occurence of <v>old</v> in the given tree.</p> \u000A \u000A <p>Equality to <v>old</v> is determined using the function <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt>.  The \u000A <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____SUBST\">subst</see> new old tree)</v> requires <v>(<see topic=\"ACL2____EQLABLEP\">eqlablep</see> old)</v>, which \u000A ensures that the <see topic=\"ACL2____GUARD\">guard</see> for <tt><see topic=\"COMMON-LISP____EQL\">eql</see></tt> will be met for each comparison \u000A generated by <v>subst</v>.</p> \u000A \u000A <p><v>Subst</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>subst</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> subst (new old tree)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> old)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQL\">eql</see> old tree) new)\u000A             ((<see topic=\"COMMON-LISP____ATOM\">atom</see> tree) tree)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____SUBST\">subst</see> new old (<see topic=\"COMMON-LISP____CAR\">car</see> tree))\u000A                      (<see topic=\"COMMON-LISP____SUBST\">subst</see> new old (<see topic=\"COMMON-LISP____CDR\">cdr</see> tree))))))</code>"],

 "COMMON-LISP____LIST_A2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>List*</v> is the Common Lisp macro for building a list of objects \u000A from given elements and a tail.  For example, <v>(<see topic=\"COMMON-LISP____LIST_A2\">list*</see> 5 6 '(7 8 9))</v> equals \u000A the list <v>'(5 6 7 8 9)</v>.  Also see <see topic=\"COMMON-LISP____LIST\">list</see>.</p> \u000A \u000A <p><v>List*</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>list*</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> list* (<see topic=\"COMMON-LISP_____62REST\">&amp;rest</see> args)\u000A          (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> args)))\u000A          (list*-macro args))</code> \u000A <p><b>Function: </b><srclink>list*-macro</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> list*-macro (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst) (<see topic=\"COMMON-LISP____CONSP\">consp</see> lst))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A           (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> 'cons\u000A                 (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst)\u000A                       (<see topic=\"COMMON-LISP____CONS\">cons</see> (list*-macro (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst)) nil)))))</code>"],

 "COMMON-LISP____ASH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ASH\">ash</see> i c)</v> is the result of taking the two's complement \u000A representation of the integer <v>i</v> and shifting it by <v>c</v> bits: shifting \u000A left and padding with <v>c</v> <v>0</v> bits if <v>c</v> is positive, shifting right \u000A and dropping <v>(<see topic=\"COMMON-LISP____ABS\">abs</see> c)</v> bits if <v>c</v> is negative, and simply returning \u000A <v>i</v> if <v>c</v> is <v>0</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>ash</v> requires that its arguments are integers.</p> \u000A \u000A <p><v>Ash</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>ash</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ash (i c)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> c))))\u000A       (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"ACL2____IFIX\">ifix</see> i) (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 c)) 1))</code>"],

 "ACL2____What_02is_02Required_02of_02the_02User_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____How_02Long_02Does_02It_02Take_02to_02Become_02an_02Effective_02User_82Q_92'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>It is not easy to build ACL2 models of complex systems.  To do so, the user \u000A must <b>understand</b></p> \u000A \u000A <code>* the system being modeled, and\u000A\u000A* ACL2, at least as a programming language.</code> \u000A \u000A <p>It is not easy to get ACL2 to prove hard theorems.  To do so, the user must \u000A <b>understand</b></p> \u000A \u000A <code>* the model,\u000A\u000A* ACL2 as a mathematical logic, and\u000A\u000A* be able to construct a proof (in interaction with ACL2).</code> \u000A \u000A <p>ACL2 will help construct the proof but its primary role is to prevent \u000A logical mistakes.  The creative burden &mdash; the mathematical insight into \u000A <b>why the model has the desired property</b> &mdash; is the user's \u000A responsibility.</p> \u000A \u000A <p><see topic='ACL2____How_02Long_02Does_02It_02Take_02to_02Become_02an_02Effective_02User_82Q_92'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____USING-COMPUTED-HINTS-1":[["Using-computed-hints"],"ACL2 Sources","ACL2","<p>The common hint</p> \u000A \u000A <code>(&quot;Subgoal 3.2.1''&quot; :use lemma42)</code> \u000A \u000A <p>has the same effect as the computed hint</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id '((0) (3 2 1) . 2))\u000A    '(:use lemma42)\u000A    nil)</code> \u000A \u000A <p>which, of course, is equivalent to</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id '((0) (3 2 1) . 2))\u000A     '(:use lemma42))</code> \u000A \u000A <p>which is also equivalent to the computed hint</p> \u000A \u000A <code>my-special-hint</code> \u000A \u000A <p>provided the following <v>defun</v> has first been executed</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> my-special-hint (id clause world)\u000A  (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :mode :program)\u000A           (<see topic=\"COMMON-LISP____IGNORE\">ignore</see> clause world))\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id '((0) (3 2 1) . 2))\u000A      '(:use lemma42)\u000A      nil))</code> \u000A \u000A <p>It is permitted for the <v>defun</v> to be in :LOGIC mode (see <see topic=\"ACL2____DEFUN-MODE\">defun-mode</see>) also.</p> \u000A \u000A <p>Just to be concrete, the following three events all behave the same way (if \u000A <v>my-special-hint</v> is as above):</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> main (big-thm a b c)\u000A  :hints ((&quot;Subgoal 3.2.1''&quot; :use lemma42)))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main (big-thm a b c)\u000A  :hints ((<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> id '((0) (3 2 1) . 2)) '(:use lemma42))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> main (big-thm a b c)\u000A  :hints (my-special-hint))</code>"],

 "ACL2____SPLITTER-OUTPUT":[["Splitter"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SPLITTER\">splitter</see> for a discussion of splitter rules.  See <see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see> for how to turn off, or on, the reporting of splitter \u000A rules.  When splitter-output is off, because either <v>prove</v> output is \u000A inhibited \u000A (see <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>) or <v>(</v><tt><see topic=\"ACL2____SET-SPLITTER-OUTPUT\">set-splitter-output</see></tt><v>\u000A nil)</v> has been invoked, then the value of <v>(<see topic=\"ACL2____SPLITTER-OUTPUT\">splitter-output</see>)</v> is <v>nil</v>. \u000A Otherwise, such reporting is on and the value is non-<v>nil</v>.</p>"],

 "ACL2____SIGNED-BYTE-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____SIGNED-BYTE-P\">Signed-byte-p</see> bits x)</v> is <v>T</v> when <v>bits</v> is a positive \u000A integer and <v>x</v> is a signed integer whose 2's complement representation \u000A fits in a bit-width of <v>bits</v>, i.e., <v>-2^(bits-1) &lt;= x &lt;\u000A 2^(bits-1)</v>.</p> \u000A \u000A <p>Note that a <see topic=\"ACL2____TYPE-SPEC\">type-spec</see> of <v>(signed-byte i)</v> for a variable <v>x</v> \u000A in a function's <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form translates to a <see topic=\"ACL2____GUARD\">guard</see> condition of \u000A <v>(<see topic=\"ACL2____SIGNED-BYTE-P\">signed-byte-p</see> i x)</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>signed-byte-p</v> is <v>T</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>signed-byte-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> signed-byte-p (bits x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> bits)\u000A            (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 bits)\u000A            (integer-range-p (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 (<see topic=\"COMMON-LISP____1-\">1-</see> bits)))\u000A                             (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 (<see topic=\"COMMON-LISP____1-\">1-</see> bits))\u000A                             x)))</code>"],

 "ACL2____SET-WORMHOLE-ENTRY-CODE":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____SET-WORMHOLE-ENTRY-CODE\">set-wormhole-entry-code</see> whs code)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  <v>Whs</v> should be a well-formed wormhole status and \u000A <v>code</v> should be <v>:ENTER</v> or <v>:SKIP</v>.  This function returns a new \u000A status with the specified entry code but the same data as <v>whs</v>.  It avoids \u000A unnecessary consing if the entry code for <v>whs</v> is already set to \u000A <v>code</v>.  This function does not affect state or a wormhole's hidden status. \u000A It just returns a (possibly) new status object suitable as the value of the \u000A <v>lambda</v> expressions in <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt> and <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>.</p>"],

 "ACL2____SET-WORMHOLE-DATA":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____SET-WORMHOLE-DATA\">set-wormhole-data</see> whs data)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  <v>Whs</v> should be a well-formed wormhole status; \u000A <v>data</v> is arbitrary.  This function returns a new status with the same \u000A entry code as <v>whs</v> but with the new <v>data</v>.  It avoids unnecessary \u000A consing if the data for <v>whs</v> is already set to <v>data</v>.  This function \u000A does not affect state or a wormhole's hidden status.  It just returns a \u000A (possibly) new status object suitable as the value of the <v>lambda</v> \u000A expressions in <tt><see topic=\"ACL2____WORMHOLE-EVAL\">wormhole-eval</see></tt> and <tt><see topic=\"ACL2____WORMHOLE\">wormhole</see></tt>.</p>"],

 "ACL2____SET-NON-LINEARP":[["Non-linear-arithmetic"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see> t)\u000A(<see topic=\"ACL2____SET-NON-LINEARP\">set-non-linearp</see> nil)</code> \u000A \u000A <p>See <see topic=\"ACL2____NON-LINEAR-ARITHMETIC\">non-linear-arithmetic</see>.  This event is equivalent to <v>(<see topic=\"ACL2____TABLE\">table</see>\u000A acl2-defaults-table :non-linearp &lt;t-or-nil&gt;)</v>, and hence is <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to \u000A any <see topic=\"ACL2____BOOKS\">books</see> and <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs; see \u000A <see topic=\"ACL2____ACL2-DEFAULTS-TABLE\">ACL2-defaults-table</see>.</p> \u000A \u000A <p>The initial value is <v>nil</v>.</p>"],

 "ACL2____RESIZE-LIST":[["Stobj"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____RESIZE-LIST\">Resize-list</see> lst n default-value)</v> takes a list, <v>lst</v>, and \u000A a desired length, <v>n</v>, for the result list, as well as a <v>default-value</v> \u000A to use for the extra elements if <v>n</v> is greater than the length of \u000A <v>lst</v>.</p> \u000A \u000A <p><v>Resize-list</v> has a guard of <v>t</v>.  This function is called in the \u000A body of function, <v>resize-&lt;a&gt;</v> where <v>&lt;a&gt;</v> is an array field of a <see topic=\"ACL2____STOBJ\">stobj</see>.  See <see topic=\"ACL2____STOBJ\">stobj</see> and see <see topic=\"ACL2____DEFSTOBJ\">defstobj</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>resize-list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> resize-list (lst n default-value)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____E3\">&gt;</see> n 0))\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) default-value (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                 (<see topic=\"ACL2____RESIZE-LIST\">resize-list</see> (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) lst (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))\u000A                              (<see topic=\"COMMON-LISP____1-\">1-</see> n)\u000A                              default-value))\u000A           nil))</code>"],

 "ACL2____REDEF-":[["Ld"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000AACL2 !&gt;:redef-\u000AACL2 p!&gt;</code> \u000A \u000A <p>This macro, for system hackers only, is a convenient way to reverse the \u000A effects of <v>:redef+</v>.  See <see topic=\"ACL2____REDEF_B2\">redef+</see>.  We do not guarantee that \u000A <v>:redef-</v> will restore everything one might expect to its state before the \u000A earlier <v>:redef+</v>.  To see exactly what <v>:redef-</v> does, look at its \u000A code, for example by evaluating <v>:trans1 (<see topic=\"ACL2____REDEF-\">redef-</see>)</v></p>"],

 "ACL2____PSO_12":[["Other"],"ACL2 Sources","ACL2","<p>Evaluate <v>:pso!</v> in order to print output that was generated in \u000A an environment where output was being saved; see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> for \u000A details.  Note that <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> will be included; use <v>:</v><tt><see topic=\"ACL2____PSO\">pso</see></tt> \u000A if you want that output to be suppressed.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____PSOG\">psog</see>, for printing saved output in <see topic=\"ACL2____GAG-MODE\">gag-mode</see>.</p>"],

 "ACL2____MEMOIZE-SUMMARY":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, this function just returns <v>nil</v>, but it displays profiling \u000A and memoization table information.  The profiling statistics may be cleared \u000A with <v>(</v><tt><see topic=\"ACL2____CLEAR-MEMOIZE-STATISTICS\">clear-memoize-statistics</see></tt><v>)</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>memoize-summary</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> memoize-summary\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Start in a fresh ACL2, either by restarting your ACL2 image from \u000A scratch or executing <v>:ubt! 1</v>.</p> \u000A \u000A <p>Define the following functions and use The Method to prove the theorem at \u000A the bottom:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> dupsp (x)  ; does x contain duplicate elements?\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))\u000A          t\u000A          (dupsp (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> dupsp-rev\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (dupsp (rev x)) (dupsp x)))</code> \u000A \u000A <p>When you've solved this problem, compare your answer to ours; see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-3-ANSWER\">introductory-challenge-problem-3-answer</see>.</p> \u000A \u000A <p>Then, use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see>.</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Start in a fresh ACL2, either by restarting your ACL2 image from \u000A scratch or executing</p> \u000A \u000A <code>:ubt! 1</code> \u000A \u000A <p>which will undo everything since the first user event.</p> \u000A \u000A <p>Then define this function:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rev (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (rev (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)) (<see topic=\"COMMON-LISP____LIST\">list</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))))</code> \u000A \u000A <p>Then use The Method to prove:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> triple-rev\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev (rev x))) (rev x)))</code> \u000A \u000A <p>When you've solved this problem, compare your answer to ours; see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1-ANSWER\">introductory-challenge-problem-1-answer</see>.</p> \u000A \u000A <p>Then, use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see>.</p>"],

 "ACL2____INTERFACING-TOOLS":[["Top","ACL2"],"ACL2 Sources","ACL2",""],

 "ACL2____IFIX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Ifix</v> simply returns any integer argument unchanged, returning \u000A <v>0</v> on a non-integer argument.  Also see <see topic=\"ACL2____NFIX\">nfix</see>, see <see topic=\"ACL2____RFIX\">rfix</see>, see \u000A <see topic=\"ACL2____REALFIX\">realfix</see> and see <see topic=\"ACL2____FIX\">fix</see> for analogous functions that coerce to a \u000A natural number, a rational number, a real, and a number, respectively.</p> \u000A \u000A <p><v>Ifix</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>ifix</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> ifix (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) x 0))</code>"],

 "ACL2____Hey_02Wait_12_02_02Is_02ACL2_02Typed_02or_02Untyped_82Q_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The example</p> \u000A \u000A <code>ACL2 !&gt;<b>(app 7 27)</b> \u000A \u000AACL2 Error in TOP-LEVEL:  The guard for the function symbol ENDP, which \u000Ais (OR (CONSP X) (EQUAL X NIL)), is violated by the arguments in the \u000Acall (ENDP 7). \u000A</code> \u000A \u000A <p>illustrates the fact that while ACL2 is an untyped language the ACL2 \u000A evaluator can be configured so as to check ``types'' at runtime.  We should \u000A not say ``types'' here but ``guards.''  Click <see topic='ACL2____Undocumented_02Topic'>here</see> for a discussion of guards.</p> \u000A \u000A <p>The guard on <tt><see topic=\"COMMON-LISP____ENDP\">endp</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A requires its argument to be a true list.  Since 7 is not a true list, and \u000A since ACL2 is checking guards in this example, an error is signaled by ACL2. \u000A How do you know ACL2 is checking guards?  Because the prompt tells us (click \u000A <see topic='ACL2____About_02the_02Prompt'>here</see>) with its ``!''.</p>"],

 "ACL2____EXIT-BOOT-STRAP-MODE":[["Miscellaneous"],"ACL2 Sources","ACL2","<p><v>Exit-boot-strap-mode</v> is the last step in creating the ACL2 \u000A <see topic=\"ACL2____WORLD\">world</see> in which the user lives.  It has <see topic=\"ACL2____COMMAND\">command</see> number <v>0</v>. \u000A <see topic=\"ACL2____COMMAND\">Command</see>s before it are part of the system initialization and extend all \u000A the way back to <v>:</v><tt><see topic=\"COMMON-LISP____MIN\">min</see></tt>.  <see topic=\"ACL2____COMMAND\">Command</see>s after it are those of the \u000A user.</p> \u000A \u000A <p><v>Exit-boot-strap-mode</v> is a Common Lisp function but not an ACL2 \u000A function.  It is called when every <tt><see topic=\"ACL2____DEFCONST\">defconst</see></tt>, <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt>, etc., in \u000A our source code has been processed under ACL2 and the <see topic=\"ACL2____WORLD\">world</see> is all but \u000A complete.  <v>exit-boot-strap-mode</v> has only one job: to signal the \u000A completion of the boot-strapping.</p>"],

 "ACL2____DEFAULT-HINTS-TABLE":[["Default-hints"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>, see <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see>, \u000A and see <see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see> for how to use this table.  For \u000A completeness, we mention here that under the hood, these events all update the \u000A <v>default-hints-table</v> by updating its key, <v>t</v>, for example as \u000A follows.</p> \u000A \u000A <code>(<see topic=\"ACL2____TABLE\">table</see> default-hints-table t\u000A       '((computed-hint-1 clause)\u000A         (computed-hint-2 clause\u000A                          stable-under-simplificationp)))</code> \u000A \u000A <p>The use of default hints is explained elsewhere; see <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>.</p> \u000A \u000A <p>Advanced users only: see <see topic=\"ACL2____OVERRIDE-HINTS\">override-hints</see> for an advanced variant of \u000A default hints.</p>"],

 "ACL2____COMP-GCL":[["Comp","Programming"],"ACL2 Sources","ACL2","<p><v>Comp-gcl</v> is for use by experts who want to examine the results \u000A of GCL compilation, and it may only be used with ACL2 implementations built on \u000A top of GCL.  It takes exactly the same arguments as <tt><see topic=\"ACL2____COMP\">comp</see></tt>, and has the \u000A same basic functionality (see <see topic=\"ACL2____COMP\">comp</see>), but has two additional effects. \u000A First, files <v>&quot;TMP.lisp&quot;</v> and <v>&quot;TMP1.lisp&quot;</v> are always created, even \u000A when a single function is specified.  Second, <v>comp-gcl</v> always leaves \u000A files <v>&quot;TMP.c&quot;</v>, <v>&quot;TMP.h&quot;</v>, <v>&quot;TMP1.c&quot;</v>, and <v>&quot;TMP1.h&quot;</v> \u000A when compilation is complete.</p>"],

 "ACL2____CERTIFY-BOOK_12":[["Certify-book"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____CERTIFY-BOOK_12\">certify-book!</see> &quot;my-arith&quot; 3)     ;Certify in a world with 3\u000A                                   ; commands, starting in a world\u000A                                   ; with at least 3 commands.\u000A(<see topic=\"ACL2____CERTIFY-BOOK_12\">certify-book!</see> &quot;my-arith&quot;)       ;Certify in the initial world.\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____CERTIFY-BOOK_12\">certify-book!</see> book-name k compile-flg)</code> \u000A \u000A <p>where <v>book-name</v> is a book name (see <see topic=\"ACL2____BOOK-NAME\">book-name</see>), <v>k</v> is a \u000A nonnegative integer used to indicate the ``certification <see topic=\"ACL2____WORLD\">world</see>,'' and \u000A <v>compile-flg</v> indicates whether you wish to compile the (functions in the) \u000A book.</p> \u000A \u000A <p>This <see topic=\"ACL2____COMMAND\">command</see> is identical to <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt>, except that the \u000A second argument <v>k</v> may not be <v>t</v> in <v>certify-book!</v> and if <v>k</v> \u000A exceeds the current <see topic=\"ACL2____COMMAND\">command</see> number, then an appropriate <tt><see topic=\"ACL2____UBT_12\">ubt!</see></tt> \u000A will be executed first.  See <see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see> and see <see topic=\"ACL2____UBT_12\">ubt!</see>.</p>"],

 "ACL2____BREAK_42":[["ACL2-built-ins","Other"],"ACL2 Sources","ACL2","<p>ACL2 users are generally advised to avoid breaking into raw Lisp. \u000A Advanced users may, on occasion, see the need to do so.  Evaluating \u000A <v>(<see topic=\"ACL2____BREAK_42\">break$</see>)</v> will have that effect.  (Exception: <v>break$</v> is disabled \u000A after evaluation of <v>(<see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see> :never)</v>; see <see topic=\"ACL2____SET-DEBUGGER-ENABLE\">set-debugger-enable</see>.)  <v>Break$</v> returns <v>nil</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>break$</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> break$ nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____ASSERT_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____ASSERT_42\">assert$</see> test form)</code> \u000A \u000A <p>where <v>test</v> returns a single value and <v>form</v> is arbitrary. \u000A Semantically, this call of <v>assert$</v> is equivalent to <v>form</v>.  However, \u000A it causes a hard error if the value of <v>test</v> is <v>nil</v>.  That hard error \u000A invokes the function <tt><see topic=\"ACL2____ILLEGAL\">illegal</see></tt>, which has a <see topic=\"ACL2____GUARD\">guard</see> that is equal \u000A to <v>nil</v>; so if you use <v>assert$</v> in code for which you verify guards, \u000A then a proof obligation will be that the occurrence of <v>test</v> is never \u000A <v>nil</v>.</p>"],

 "ACL2-PC____WRAP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(wrap induct) ; induct, then replace first new goal by the conjunction of all\u000A              ; the new goals, and drop all new goals after the first\u000A\u000AGeneral Form:\u000A(wrap &amp;rest instrs)</code> \u000A \u000A <p>First the instructions in <v>instrs</v> are executed, as in <v>do-all</v>.  If \u000A this ``fails'' then no additional action is taken.  Otherwise, the current \u000A goal after execution of <v>instrs</v> is conjoined with all ``new'' goals, in \u000A the sense that their names are not among the names of goals at the time \u000A <v>instrs</v> was begun.  This conjunction becomes the new current goal and \u000A those ``new'' goals are dropped.</p> \u000A \u000A <p>See the code for the proof-checker command wrap-induct for an example of \u000A the use of <v>wrap</v>.</p>"],

 "ACL2-PC____UNDO":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(undo 7)\u000Aundo\u000A\u000AGeneral Forms:\u000A\u000A(undo n) -- Undo the last n instructions.  The argument n should be\u000A            a positive integer.\u000A\u000Aundo     -- Same as (undo 1).</code> \u000A \u000A <p><b>Remark:</b> To remove the effect of an <v>undo</v> command, use \u000A <v>restore</v>.  See the documentation for details.</p> \u000A \u000A <p><b>Remark:</b> If the argument <v>n</v> is greater than the total number of \u000A interactive instructions in the current session, then <v>(undo n)</v> will \u000A simply take you back to the start of the session.</p> \u000A \u000A <p>The <v>undo</v> meta command always ``succeeds''; it returns <v>(<see topic=\"ACL2____MV\">mv</see> nil t\u000A state)</v> unless its optional argument is supplied and of the wrong type \u000A (i.e. not a positive integer) or there are no instructions to undo.</p>"],

 "ACL2-PC____REPLAY":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000AREPLAY     -- replay all instructions in the current session\u000A              (i.e., state-stack)\u000A(REPLAY 5) -- replay the most recent 5 instructions\u000A(REPLAY 5\u000A        (COMMENT deleted dive command here))\u000A           -- replace the 5th most recent instruction with the\u000A              indicated comment instruction, and then replay it\u000A              followed by the remaining 4 instructions\u000A\u000AGeneral Form:\u000A(REPLAY &amp;OPTIONAL n replacement-instruction)</code> \u000A \u000A <p>Replay the last <v>n</v> instructions if <v>n</v> is a positive integer; else \u000A <v>n</v> should be <v>nil</v> or not supplied, and replay all instructions. \u000A However, if <v>replacement-instruction</v> is supplied and not <v>nil</v>, then \u000A before the replay, replace the <v>nth</v> instruction (from the most recent, as \u000A shown by <v>commands</v>) with <v>replacement-instruction</v>.</p> \u000A \u000A <p>If this command ``fails'', then the <v>restore</v> command will revert the \u000A state-stack to its value present before the <v>replay</v> instruction was \u000A executed.</p>"],

 "ACL2-PC____REDUCE-BY-INDUCTION":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Areduce-by-induction\u000A  -- attempt to prove the current goal after going into induction,\u000A     with no further inductions\u000A\u000A(reduce-by-induction (&quot;Subgoal 2&quot; :by foo) (&quot;Subgoal 1&quot; :use bar))\u000A  -- attempt to prove the current goal after going into induction,\u000A     with no further inductions, using the indicated hints\u000A\u000AGeneral Form:\u000A(reduce-by-induction &amp;rest hints)</code> \u000A \u000A <p>A subgoal will be created for each goal that would have been pushed for \u000A proof by induction in an ordinary proof, except that the proof begins with a \u000A top-level induction.</p> \u000A \u000A <p>Notice that unlike <v>prove</v>, the arguments to <v>reduce-by-induction</v> \u000A are spread out, and are all hints.  See also <v>prove</v>, <v>reduce</v>, and \u000A <v>bash</v>.</p> \u000A \u000A <p><b>Remark</b>: Induction and the various processes will be used to the \u000A extent that they are ordered explicitly in the <v>:induct</v> and <v>:do-not</v> \u000A hints.</p>"],

 "ACL2-PC____PSO_12":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Apso!</code> \u000A \u000A <p>Print the most recent proof attempt from inside the proof-checker, \u000A including <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output, assuming you are in <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> or \u000A have saved output (see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>).  This includes all calls to \u000A the prover, including for example <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>induct</v>, \u000A <v>split</v>, and <v>bash</v>, in addition to <v>prove</v>.  So for example, you can \u000A follow <v>(quiet prove)</v> with <v>pso!</v> to see the proof, including <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output, if it failed.</p> \u000A \u000A <p>See also documentation for related <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>pso</v> \u000A and <v>psog</v>.</p>"],

 "ACL2-PC____PSOG":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Apsog</code> \u000A \u000A <p>Print the most recent proof attempt from inside the proof-checker, \u000A including goal names, assuming you are in <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> or have saved \u000A output (see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>).  This includes all calls to the prover, \u000A including for example <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>induct</v>, <v>split</v>, \u000A and <v>bash</v>, in addition to <v>prove</v>.  So for example, you can follow \u000A <v>(quiet prove)</v> with <v>psog</v> to see the proof, including <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output, if it failed.</p> \u000A \u000A <p>See also documentation for related <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>pso</v> \u000A and <v>pso!</v>.</p>"],

 "ACL2-PC____INDUCT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Ainduct, (<see topic=\"ACL2____INDUCT\">induct</see> t)\u000A   -- induct according to a heuristically-chosen scheme, creating\u000A      a new subgoal for each base and induction step\u000A(<see topic=\"ACL2____INDUCT\">induct</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x) y))\u000A   -- as above, but choose an induction scheme based on the term\u000A      (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x) y) rather than on the current goal\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____INDUCT\">induct</see> &amp;optional term)</code> \u000A \u000A <p>Induct as in the corresponding <v>:induct</v> hint given to the theorem \u000A prover, creating new subgoals for the base and induction steps.  If term is \u000A <v>t</v> or is not supplied, then use the current goal to determine the \u000A induction scheme; otherwise, use that term.</p> \u000A \u000A <p><b>Remark:</b> As usual, abbreviations are allowed in the term.</p> \u000A \u000A <p><b>Remark:</b> <v>Induct</v> actually calls the <v>prove</v> command with all \u000A processes turned off.  Thus, you must be at top of the goal for an <v>induct</v> \u000A instruction.</p>"],

 "COMMON-LISP_____E3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>&gt;</v> is a macro, and <v>(<see topic=\"COMMON-LISP_____E3\">&gt;</see> x y)</v> expands to the same thing as \u000A <v>(<see topic=\"COMMON-LISP_____C3\">&lt;</see> y x)</v>.  See <see topic=\"COMMON-LISP_____C3\">&lt;</see>.</p> \u000A \u000A <p><v>&gt;</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>&gt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> &gt; (x y) (<see topic=\"COMMON-LISP____LIST\">list</see> '&lt; y x))</code>"],

 "COMMON-LISP____UPPER-CASE-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____UPPER-CASE-P\">Upper-case-p</see> x)</v> is true if and only if <v>x</v> is an upper \u000A case character, i.e., a member of the list <v>#\\A</v>, <v>#\\B</v>, ..., \u000A <v>#\\Z</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>upper-case-p</v> requires its argument to be a \u000A standard character (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Upper-case-p</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>upper-case-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> upper-case-p (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> x))))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> x\u000A                    '(#\\A #\\B #\\C #\\D #\\E #\\F #\\G\u000A                          #\\H #\\I #\\J #\\K #\\L #\\M #\\N #\\O #\\P #\\Q\u000A                          #\\R #\\S #\\T #\\U #\\V #\\W #\\X #\\Y #\\Z))\u000A            t))</code>"],

 "COMMON-LISP____STRINGP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)</v> is true if and only if <v>x</v> is a string.</p>"],

 "COMMON-LISP____STRING-UPCASE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For a string <v>x</v>, <v>(<see topic=\"COMMON-LISP____STRING-UPCASE\">string-upcase</see> x)</v> is the result of \u000A applying <tt><see topic=\"COMMON-LISP____CHAR-UPCASE\">char-upcase</see></tt> to each character in <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string-upcase</v> requires its argument to be a \u000A string containing only standard characters.</p> \u000A \u000A <p><v>String-upcase</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string-upcase</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A   string-upcase (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A        (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                           (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list)))))\u000A   (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (string-upcase1 (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list))\u000A           'string))</code>"],

 "COMMON-LISP____STRING-DOWNCASE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For a string <v>x</v>, <v>(<see topic=\"COMMON-LISP____STRING-DOWNCASE\">string-downcase</see> x)</v> is the result of \u000A applying <tt><see topic=\"COMMON-LISP____CHAR-DOWNCASE\">char-downcase</see></tt> to each character in <v>x</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>string-downcase</v> requires its argument to be a \u000A string containing only standard characters.</p> \u000A \u000A <p><v>String-downcase</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string-downcase</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A   string-downcase (x)\u000A   (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A        (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> x)\u000A                           (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list)))))\u000A   (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (string-downcase1 (<see topic=\"COMMON-LISP____COERCE\">coerce</see> x 'list))\u000A           'string))</code>"],

 "COMMON-LISP____LOWER-CASE-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____LOWER-CASE-P\">Lower-case-p</see> x)</v> is true if and only if <v>x</v> is a lower case \u000A character, i.e., a member of the list <v>#\\A</v>, <v>#\\B</v>, ..., \u000A <v>#\\Z</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>lower-case-p</v> requires its argument to be a \u000A standard character (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Lower-case-p</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>lower-case-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> lower-case-p (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A                                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> x))))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> x\u000A                    '(#\\a #\\b #\\c #\\d #\\e #\\f #\\g\u000A                          #\\h #\\i #\\j #\\k #\\l #\\m #\\n #\\o #\\p #\\q\u000A                          #\\r #\\s #\\t #\\u #\\v #\\w #\\x #\\y #\\z))\u000A            t))</code>"],

 "COMMON-LISP____LOGBITP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For a nonnegative integer <v>i</v> and an integer <v>j</v>, <v>(<see topic=\"COMMON-LISP____LOGBITP\">logbitp</see>\u000A i j)</v> is a Boolean, which is <v>t</v> if and only if the value of the <v>i</v>th \u000A bit is <v>1</v> in the two's complement representation of <v>j</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____LOGBITP\">Logbitp</see> i j)</v> has a <see topic=\"ACL2____GUARD\">guard</see> that <v>i</v> is a nonnegative integer \u000A and <v>j</v> is an integer.</p> \u000A \u000A <p><v>Logbitp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logbitp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logbitp (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                                   (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> i 0))))\u000A       (<see topic=\"COMMON-LISP____ODDP\">oddp</see> (<see topic=\"COMMON-LISP____FLOOR\">floor</see> (<see topic=\"ACL2____IFIX\">ifix</see> j) (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 (<see topic=\"ACL2____NFIX\">nfix</see> i)))))</code>"],

 "COMMON-LISP____ALPHA-CHAR-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ALPHA-CHAR-P\">Alpha-char-p</see> x)</v> is true if and only if <v>x</v> is a alphabetic \u000A character, i.e., one of the <see topic=\"ACL2____CHARACTERS\">characters</see> <v>#\\a</v>, <v>#\\b</v>, ..., \u000A <v>#\\z</v>, <v>#\\A</v>, <v>#\\B</v>, ..., <v>#\\Z</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>alpha-char-p</v> requires its argument to be a \u000A character.</p> \u000A \u000A <p><v>Alpha-char-p</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>alpha-char-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> alpha-char-p (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> x\u000A                    '(#\\a #\\b #\\c\u000A                          #\\d #\\e #\\f #\\g #\\h #\\i #\\j #\\k #\\l #\\m\u000A                          #\\n #\\o #\\p #\\q #\\r #\\s #\\t #\\u #\\v #\\w\u000A                          #\\x #\\y #\\z #\\A #\\B #\\C #\\D #\\E #\\F #\\G\u000A                          #\\H #\\I #\\J #\\K #\\L #\\M #\\N #\\O #\\P #\\Q\u000A                          #\\R #\\S #\\T #\\U #\\V #\\W #\\X #\\Y #\\Z))\u000A            t))</code>"],

 "ACL2_____A2STANDARD-CI_A2":[["Io"],"ACL2 Sources","ACL2","<p>The value of the ACL2 constant <v>*standard-ci*</v> is an open \u000A character input channel that is synonymous to Common Lisp's \u000A <v>*standard-input*</v>.</p> \u000A \u000A <p>ACL2 character input from <v>*standard-ci*</v> is actually obtained by \u000A reading <see topic=\"ACL2____CHARACTERS\">characters</see> from the stream named by Common Lisp's \u000A <v>*standard-input*</v>.  That is, by changing the setting of \u000A <v>*standard-input*</v> in raw Common Lisp you can change the source from which \u000A ACL2 reads on the channel <v>*standard-ci*</v>.  See <see topic=\"ACL2_____A2STANDARD-CO_A2\">*standard-co*</see>.</p>"],

 "ACL2____UNSIGNED-BYTE-P":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____UNSIGNED-BYTE-P\">Unsigned-byte-p</see> bits x)</v> is <v>T</v> when <v>bits</v> is a \u000A positive integer and <v>x</v> is a nonnegative integer that fits into a \u000A bit-width of <v>bits</v>, i.e., <v>0 &lt;= x &lt; 2^bits</v>.</p> \u000A \u000A <p>Note that a <see topic=\"ACL2____TYPE-SPEC\">type-spec</see> of <v>(unsigned-byte i)</v> for a variable \u000A <v>x</v> in a function's <tt><see topic=\"COMMON-LISP____DECLARE\">declare</see></tt> form translates to a <see topic=\"ACL2____GUARD\">guard</see> \u000A condition of <v>(<see topic=\"ACL2____UNSIGNED-BYTE-P\">unsigned-byte-p</see> i x)</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>unsigned-byte-p</v> is <v>T</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>unsigned-byte-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> unsigned-byte-p (bits x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> bits)\u000A            (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 bits)\u000A            (integer-range-p 0 (<see topic=\"COMMON-LISP____EXPT\">expt</see> 2 bits) x)))</code>"],

 "ACL2____The_02Falling_02Body_02Model":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='pisa.gif'></img></p> \u000A \u000A <p>One particularly famous and very simple model is the equation of a falling \u000A body: the distance d an object falls is proportional to the square of the time \u000A t.  If the time is measured in seconds and the distance in feet, the equation \u000A relating these two is</p> \u000A \u000A <code>       2\u000Ad = 16t</code> \u000A \u000A <p>This equation is a <b>model</b> of falling objects.  It can be used to \u000A predict how long it takes a cannonball to fall from the top of a 200 foot \u000A tower (3.5 seconds).  This might be important if your product is designed to \u000A drop cannonballs on moving targets.</p>"],

 "ACL2____TOGGLE-PC-MACRO":[["Proof-checker"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TOGGLE-PC-MACRO\">toggle-pc-macro</see> pro)</code> \u000A \u000A <p>Change <v>pro</v> from an atomic macro command to an ordinary one (or \u000A vice-versa, if <v>pro</v> happens to be an ordinary macro command)</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____TOGGLE-PC-MACRO\">toggle-pc-macro</see> name &amp;optional new-tp)</code> \u000A \u000A <p>If name is an atomic macro command then this turns it into an ordinary one, \u000A and vice-versa.  However, if <v>new-tp</v> is supplied and not <v>nil</v>, then it \u000A should be the new type (the symbol <v>macro</v> or <v>atomic-macro</v>, in any \u000A package), or else there is no change.</p>"],

 "ACL2____Rewrite_02Rules_02are_02Generated_02from_02DEFTHM_02Events":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____You_02Must_02Think_02about_02the_02Use_02of_02a_02Formula_02as_02a_02Rule'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>By reading the documentation of <tt><see topic=\"ACL2____DEFTHM\">defthm</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A (and especially of its :<see topic=\"ACL2____RULE-CLASSES\">rule-classes</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A argument) you would learn that when we submitted the command</p> \u000A \u000A <code><b>(defthm associativity-of-app</b> \u000A  <b>(equal (app (app a b) c)</b> \u000A         <b>(app a (app b c))))</b> \u000A \u000A</code> \u000A \u000A <p>we not only command the system to prove that <v>app</v> is an associative \u000A function but</p> \u000A \u000A <code>* <b>we commanded it to use that fact as a rewrite rule</b>. \u000A </code> \u000A \u000A <p>That means that every time the system encounters a term of the form</p> \u000A \u000A <code>(app (app <b>x</b> <b>y</b>) <b>z</b>) \u000A</code> \u000A \u000A <p>it will replace it with</p> \u000A \u000A <code>(app <b>x</b> (app <b>y</b> <b>z</b>))! \u000A</code> \u000A \u000A <p><see topic='ACL2____You_02Must_02Think_02about_02the_02Use_02of_02a_02Formula_02as_02a_02Rule'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____REDEF_12":[["Ld"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000AACL2 !&gt;:redef!</code> \u000A \u000A <p>This command sets <tt><see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see></tt> to <v>'(:warn!\u000A . :overwrite)</v>.</p> \u000A \u000A <p>This command allows redefinition of functions and other <see topic=\"ACL2____EVENTS\">events</see> \u000A without undoing, but with a warning.  The command <v>:</v><tt><see topic=\"ACL2____REDEF\">redef</see></tt> is \u000A similar, but queries the user first before doing the redefinition.  For more \u000A related options, see <see topic=\"ACL2____LD-REDEFINITION-ACTION\">ld-redefinition-action</see>.</p>"],

 "ACL2____O-RST":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>An ACL2 infinite ordinal is a list whose elements are \u000A exponent-coefficient pairs (see <see topic=\"ACL2____O-P\">o-p</see> and see <see topic=\"ACL2____O-INFP\">o-infp</see>).  The first \u000A exponent and first coefficient of an ordinal can be obtained by using <tt><see topic=\"ACL2____O-FIRST-EXPT\">o-first-expt</see></tt> and <tt><see topic=\"ACL2____O-FIRST-COEFF\">o-first-coeff</see></tt> respectively.  To obtain the rest of \u000A the ordinal (for recursive analysis), use the <v>o-rst</v> function. It returns \u000A the rest of the ordinal after the first exponent and coefficient are \u000A removed.</p> \u000A \u000A <p><b>Function: </b><srclink>o-rst</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o-rst (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)))\u000A       (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))</code>"],

 "ACL2____NUMBER-SUBTREES":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>In the logic, <v>number-subtrees</v> is defined as the length of <tt><see topic=\"ACL2____CONS-SUBTREES\">cons-subtrees</see></tt>.</p> \u000A \u000A <p>Under the hood, we first <tt><see topic=\"ACL2____HONS-COPY\">hons-copy</see></tt> X to obtain a normed version, \u000A then count the number of unique conses in X using an EQ hash table.</p> \u000A \u000A <p><b>Function: </b><srclink>number-subtrees</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> number-subtrees (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"ACL2____LEN\">len</see> (<see topic=\"ACL2____CONS-SUBTREES\">cons-subtrees</see> x 'number-subtrees)))</code>"],

 "ACL2____NOTE-2-7-GUARDS":[["Note-2-7"],"ACL2 Sources","ACL2","<p>It was possible for guard verification to succeed where it should \u000A have failed.  See the discussion under <see topic=\"ACL2____NOTE-2-7-BUG-FIXES\">note-2-7-bug-fixes</see>.</p> \u000A \u000A <p>There have been changes in the guards generated from type declarations for \u000A the following cases.  Thanks to Dave Greve and Matt Wilding for suggesting \u000A such changes.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte n) val)\u000A(<see topic=\"COMMON-LISP____TYPE\">type</see> (unsigned-byte n) val)\u000A(<see topic=\"COMMON-LISP____TYPE\">type</see> (integer m n) val)</code> \u000A \u000A <p>The following examples illustrate the changes.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____TYPE\">type</see> (signed-byte 4) x)\u000A==&gt; [old] (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> -8 X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X 7))\u000A==&gt; [new] (<see topic=\"ACL2____SIGNED-BYTE-P\">SIGNED-BYTE-P</see> 4 X)\u000A\u000A(<see topic=\"COMMON-LISP____TYPE\">type</see> (unsigned-byte 4) x)\u000A==&gt; [old] (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____INTEGERP\">INTEGERP</see> X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 X) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> X 15))\u000A==&gt; [new] (<see topic=\"ACL2____UNSIGNED-BYTE-P\">UNSIGNED-BYTE-P</see> 4 X)</code>"],

 "ACL2____MEMBER-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-BASE-CASE":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>According to the sentence, the conjecture being proved is \u000A ``reversing the reverse of a <v>true-listp</v> yields the original list.''  The \u000A formula corresponding to this conjecture is:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A         (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z)).</code> \u000A \u000A <p>We're also told that this is an inductive proof.  Evidently we're doing an \u000A induction on the structure of the list <v>z</v>.  Then the <i>Base Case</i> is \u000A the formula:</p> \u000A \u000A <code>(<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> z)\u000A         (<see topic=\"ACL2____IMPLIES\">implies</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> z)\u000A                  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (rev (rev z)) z))).</code> \u000A \u000A <p>Now use your browser's <b>Back Button</b> to return to the example proof in \u000A <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED\">logic-knowledge-taken-for-granted</see>.</p>"],

 "ACL2____KEYWORD-VALUE-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see> l)</v> is true if and only if <v>l</v> is a \u000A list of even length of the form <v>(k1 a1 k2 a2 ... kn an)</v>, where each \u000A <v>ki</v> is a keyword.</p> \u000A \u000A <p><b>Function: </b><srclink>keyword-value-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> keyword-value-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____NULL\">null</see> l))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))\u000A                     (<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see> (<see topic=\"COMMON-LISP____CDDR\">cddr</see> l))))))</code>"],

 "ACL2____EXISTS":[["Defun-sk"],"ACL2 Sources","ACL2","<p>The symbol <v>exists</v> (in the ACL2 package) represents existential \u000A quantification in the context of a <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt> form.  See <see topic=\"ACL2____DEFUN-SK\">defun-sk</see> \u000A and see <see topic=\"ACL2____FORALL\">forall</see>.</p> \u000A \u000A <p>See <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see> for an example illustrating how the use of \u000A recursion, rather than explicit quantification with <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>, may be \u000A preferable.</p>"],

 "ACL2____DIGIT-TO-CHAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;(<see topic=\"ACL2____DIGIT-TO-CHAR\">digit-to-char</see> 8)\u000A#\\8</code> \u000A \u000A <p>For an integer <v>n</v> from 0 to 15, <v>(<see topic=\"ACL2____DIGIT-TO-CHAR\">digit-to-char</see> n)</v> is the character \u000A corresponding to <v>n</v> in hex notation, using uppercase letters for digits \u000A exceeding 9.  If <v>n</v> is in the appropriate range, that result is of course \u000A also the binary, octal, and decimal digit.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>digit-to-char</v> requires its argument to be an \u000A integer between 0 and 15, inclusive.</p> \u000A \u000A <p><b>Function: </b><srclink>digit-to-char</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     digit-to-char (n)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> 0 n) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> n 15))))\u000A     (<see topic=\"COMMON-LISP____CASE\">case</see> n (1 #\\1)\u000A           (2 #\\2)\u000A           (3 #\\3)\u000A           (4 #\\4)\u000A           (5 #\\5)\u000A           (6 #\\6)\u000A           (7 #\\7)\u000A           (8 #\\8)\u000A           (9 #\\9)\u000A           (10 #\\A)\u000A           (11 #\\B)\u000A           (12 #\\C)\u000A           (13 #\\D)\u000A           (14 #\\E)\u000A           (15 #\\F)\u000A           (otherwise #\\0)))</code>"],

 "ACL2____BOOLEANP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____BOOLEANP\">Booleanp</see> x)</v> is <v>t</v> if <v>x</v> is <v>t</v> or <v>nil</v>, and is \u000A <v>nil</v> otherwise.</p> \u000A \u000A <p>See <see topic=\"ACL2____GENERALIZED-BOOLEANS\">generalized-booleans</see> for a discussion of a potential soundness \u000A problem for ACL2 related to the question: Which Common Lisp functions are \u000A known to return Boolean values?</p> \u000A \u000A <p><b>Function: </b><srclink>booleanp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> booleanp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> x t) t (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil)))</code>"],

 "ACL2____A_02Typical_02State":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Functions_02for_02Manipulating_02these_02Objects'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p><img src='state-object.gif'></img></p> \u000A \u000A <p>Observe that the states in typical models talk about</p> \u000A \u000A <code><b>booleans</b>    <b>integers</b>   <b>vectors</b>     <b>records</b>   <b>caches</b> \u000A<b>bits</b>        <b>symbols</b>    <b>arrays</b>      <b>stacks</b>    <b>files</b> \u000A<b>characters</b>  <b>strings</b>    <b>sequences</b>   <b>tables</b>    <b>directories</b> \u000A</code> \u000A \u000A <p>These objects are <b>discrete</b> rather than <b>continuous</b>; \u000A furthermore they are built incrementally or <b>inductively</b> by repeatedly \u000A using primitive operations to put together smaller pieces.</p> \u000A \u000A <p>The functions we need to manipulate these objects do things like \u000A <b>concatenate</b>, <b>reverse</b>, <b>sort</b>, <b>search</b>, <b>count</b>, \u000A etc.</p> \u000A \u000A <p><see topic='ACL2____Functions_02for_02Manipulating_02these_02Objects'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____ARRAY2P":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____ARRAY2P\">array2p</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ARRAY2P\">array2p</see> name alist)</code> \u000A \u000A <p>where <v>name</v> and <v>alist</v> are arbitrary objects.  This function \u000A returns <v>t</v> if <v>alist</v> is a 2-dimensional ACL2 array.  Otherwise it \u000A returns <v>nil</v>.  The function operates in constant time if <v>alist</v> is the \u000A semantic value of <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>array2p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A array2p (<see topic=\"ACL2____NAME\">name</see> l)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A (<see topic=\"COMMON-LISP____AND\">and</see>\u000A  (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> name)\u000A  (<see topic=\"ACL2____ALISTP\">alistp</see> l)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see>\u000A   ((header-keyword-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> :header l))))\u000A   (<see topic=\"COMMON-LISP____AND\">and</see>\u000A    (<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see> header-keyword-list)\u000A    (<see topic=\"COMMON-LISP____LET\">let</see>\u000A     ((<see topic=\"ACL2____DIMENSIONS\">dimensions</see>\u000A           (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :dimensions header-keyword-list)))\u000A      (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see>\u000A        (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :maximum-length header-keyword-list))))\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> dimensions)\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> dimensions) 2)\u000A          (<see topic=\"COMMON-LISP____LET\">let</see> ((d1 (<see topic=\"COMMON-LISP____CAR\">car</see> dimensions))\u000A                (d2 (<see topic=\"COMMON-LISP____CADR\">cadr</see> dimensions)))\u000A               (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> d1)\u000A                    (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> d2)\u000A                    (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> maximum-length)\u000A                    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 d1)\u000A                    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 d2)\u000A                    (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP_____A2\">*</see> d1 d2) maximum-length)\u000A                    (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> maximum-length\u000A                        *maximum-positive-32-bit-integer*)\u000A                    (bounded-integer-alistp2 l d1 d2)))))))))</code>"],

 "ACL2____ARRAY1P":[["Arrays","ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Example Form:\u000A(<see topic=\"ACL2____ARRAY1P\">array1p</see> 'delta1 a)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ARRAY1P\">array1p</see> name alist)</code> \u000A \u000A <p>where <v>name</v> and <v>alist</v> are arbitrary objects.  This function \u000A returns <v>t</v> if <v>alist</v> is a 1-dimensional ACL2 array.  Otherwise it \u000A returns <v>nil</v>.  The function operates in constant time if <v>alist</v> is the \u000A semantic value of <v>name</v>.  See <see topic=\"ACL2____ARRAYS\">arrays</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>array1p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A array1p (<see topic=\"ACL2____NAME\">name</see> l)\u000A (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A (<see topic=\"COMMON-LISP____AND\">and</see>\u000A  (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> name)\u000A  (<see topic=\"ACL2____ALISTP\">alistp</see> l)\u000A  (<see topic=\"COMMON-LISP____LET\">let</see>\u000A   ((header-keyword-list (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"ACL2____ASSOC-EQ\">assoc-eq</see> :header l))))\u000A   (<see topic=\"COMMON-LISP____AND\">and</see>\u000A    (<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see> header-keyword-list)\u000A    (<see topic=\"COMMON-LISP____LET\">let</see>\u000A     ((<see topic=\"ACL2____DIMENSIONS\">dimensions</see>\u000A           (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :dimensions header-keyword-list)))\u000A      (<see topic=\"ACL2____MAXIMUM-LENGTH\">maximum-length</see>\u000A        (<see topic=\"COMMON-LISP____CADR\">cadr</see> (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> :maximum-length header-keyword-list))))\u000A     (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> dimensions)\u000A          (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____LENGTH\">length</see> dimensions) 1)\u000A          (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> dimensions))\u000A          (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> maximum-length)\u000A          (<see topic=\"COMMON-LISP_____C3\">&lt;</see> 0 (<see topic=\"COMMON-LISP____CAR\">car</see> dimensions))\u000A          (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CAR\">car</see> dimensions) maximum-length)\u000A          (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> maximum-length\u000A              *maximum-positive-32-bit-integer*)\u000A          (bounded-integer-alistp l (<see topic=\"COMMON-LISP____CAR\">car</see> dimensions))))))))</code>"],

 "ACL2____ARGS":[["Documentation"],"ACL2 Sources","ACL2","<code>Example:\u000A:args assoc-eq</code> \u000A \u000A <p><v>Args</v> takes one argument, a symbol which must be the name of a function \u000A or macro, and prints out the formal parameters, the <see topic=\"ACL2____GUARD\">guard</see> expression, \u000A the output <see topic=\"ACL2____SIGNATURE\">signature</see>, the deduced type, the <see topic=\"ACL2____CONSTRAINT\">constraint</see> (if any), \u000A and whether <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> about the symbol is available via \u000A <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt>.</p>"],

 "ACL2-PC____UNSAVE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____UNSAVE\">unsave</see> assoc-of-append)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____UNSAVE\">unsave</see> &amp;optional name)</code> \u000A \u000A <p>Eliminates the association of a proof-checker state with <v>name</v>, if \u000A <v>name</v> is supplied and not <v>nil</v>.  The name may be <v>nil</v> or not \u000A supplied, in which case it defaults to the event name supplied with the \u000A original call to <v>verify</v> (if there is one &mdash; otherwise, the \u000A instruction ``fails'' and there is no change).  The ACL2 function <v>unsave</v> \u000A may also be executed outside the interactive loop, with the same syntax.</p> \u000A \u000A <p>See also documentation for <v>save</v> and <v>retrieve</v>.</p>"],

 "ACL2-PC____RESTORE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Arestore</code> \u000A \u000A <p><v>Restore</v> removes the effect of an <v>undo</v> command.  This always works \u000A as expected if <v>restore</v> is invoked immediately after <v>undo</v>, without \u000A intervening instructions.  However, other commands may also interact with \u000A <v>restore</v>, notably ``sequencing'' commands such as <v>do-all</v>, \u000A <v>do-strict</v>, <v>protect</v>, and more generally, <v>sequence</v>.</p> \u000A \u000A <p><b>Remark:</b> Another way to control the saving of proof-checker state is \u000A with the <v>save</v> command; see the documentation for <v>save</v>.</p> \u000A \u000A <p>The <v>restore</v> command always ``succeeds''; it returns <v>(<see topic=\"ACL2____MV\">mv</see> nil t\u000A state)</v>.</p>"],

 "ACL2-PC____REMOVE-ABBREVIATIONS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Aremove-abbreviations -- remove all abbreviations\u000A(remove-abbreviations v w)\u000A                     -- assuming that V and W currently abbreviate\u000A                        terms, then they are ``removed'' in the\u000A                        sense that they are no longer considered to\u000A                        abbreviate those terms\u000A\u000AGeneral Forms:\u000A(remove-abbreviations &amp;rest vars)</code> \u000A \u000A <p>If vars is not empty (i.e., not <v>nil</v>), remove the variables in \u000A <v>vars</v> from the current list of abbreviations, in the sense that each \u000A variable in <v>vars</v> will no longer abbreviate a term.</p> \u000A \u000A <p><b>Remark:</b> The instruction fails if at least one of the arguments fails \u000A to be a variable that abbreviates a term.</p> \u000A \u000A <p>See also the documentation for <v>add-abbreviation</v>, which contains a \u000A discussion of abbreviations in general, and <v>show-abbreviations</v>.</p>"],

 "ACL2-PC____PSO":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Apso</code> \u000A \u000A <p>Print the most recent proof attempt from inside the proof-checker assuming \u000A you are in <tt><see topic=\"ACL2____GAG-MODE\">gag-mode</see></tt> or have saved output (see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see>).  This includes all calls to the prover, including for \u000A example <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>induct</v>, <v>split</v>, and <v>bash</v>, \u000A in addition to <v>prove</v>.  So for example, you can follow <v>(quiet prove)</v> \u000A with <v>pso</v> to see the proof, including <see topic=\"ACL2____PROOF-TREE\">proof-tree</see> output, if it \u000A failed.</p> \u000A \u000A <p>See also documentation for related <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> commands <v>psog</v> \u000A and <v>pso!</v>.</p>"],

 "ACL2-PC____CONTRAPOSE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(contrapose 3)\u000A\u000AGeneral Form:\u000A(contrapose &amp;optional n)</code> \u000A \u000A <p>The (optional) argument <v>n</v> should be a positive integer that does not \u000A exceed the number of hypotheses.  Negate the current conclusion and make it \u000A the <v>n</v>th hypothesis, while negating the current <v>n</v>th hypothesis and \u000A making it the current conclusion.  If no argument is supplied then the effect \u000A is the same as for <v>(contrapose 1)</v>.</p> \u000A \u000A <p><b>Remark:</b> By ``negate'' we mean an operation that replaces <v>nil</v> by \u000A <v>t</v>, <v>x</v> by <v>nil</v> for any other explicit value <v>x</v>, <v>(<see topic=\"COMMON-LISP____NOT\">not</see> x)</v> \u000A by <v>x</v>, and any other <v>x</v> by <v>(<see topic=\"COMMON-LISP____NOT\">not</see> x)</v>.</p>"],

 "ACL2-PC____BK":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Abk</code> \u000A \u000A <p>For example, if the conclusion is <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z))</v> and the current \u000A subterm is <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z)</v>, then after executing <v>bk</v>, the current subterm \u000A will be <v>x</v>.</p> \u000A \u000A <p>Move to the previous argument of the enclosing term.</p> \u000A \u000A <p>This is the same as <v>up</v> followed by <v>(dive n-1)</v>, where <v>n</v> is \u000A the position of the current subterm in its parent term in the conclusion. \u000A Thus in particular, the <v>nx</v> command fails if one is already at the top of \u000A the conclusion.</p> \u000A \u000A <p>See also <v>up</v>, <v>dive</v>, <v>top</v>, and <v>bk</v>.</p>"],

 "COMMON-LISP_____F2_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP_____F2_D3\">/=</see> x y)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x\u000A  y))</v>.</p> \u000A \u000A <p>Unlike <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <v>/=</v> has a <see topic=\"ACL2____GUARD\">guard</see> requiring both of its \u000A arguments to be numbers.  Generally, <v>/=</v> is executed more efficiently than \u000A a combination of <tt><see topic=\"COMMON-LISP____NOT\">not</see></tt> and <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.</p> \u000A \u000A <p>For a discussion of the various ways to test against 0, See <see topic=\"ACL2____ZERO-TEST-IDIOMS\">zero-test-idioms</see>.</p> \u000A \u000A <p><v>/=</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>/=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> /= (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                                   (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))))\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)))</code>"],

 "COMMON-LISP____TYPE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____REM":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>ACL2 !&gt;(<see topic=\"COMMON-LISP____REM\">rem</see> 14 3)\u000A2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____REM\">rem</see> -14 3)\u000A-2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____REM\">rem</see> 14 -3)\u000A2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____REM\">rem</see> -14 -3)\u000A-2\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____REM\">rem</see> -15 -3)\u000A0\u000AACL2 !&gt;</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____REM\">Rem</see> i j)</v> is that number <v>k</v> for which <v>(<see topic=\"COMMON-LISP_____A2\">*</see> j (<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> i j))</v> \u000A added to <v>k</v> equals <v>i</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"COMMON-LISP____REM\">rem</see> i j)</v> requires that <v>i</v> and <v>j</v> are \u000A rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers and <v>j</v> is non-zero.</p> \u000A \u000A <p><v>Rem</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>rem</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rem (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y)\u000A                                   (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQL\">eql</see> y 0)))))\u000A       (<see topic=\"COMMON-LISP____-\">-</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____TRUNCATE\">truncate</see> x y) y)))</code>"],

 "COMMON-LISP____LOGTEST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers <v>x</v> and <v>y</v> are viewed in their two's \u000A complement representation, <v>(<see topic=\"COMMON-LISP____LOGTEST\">logtest</see> x y)</v> is true if and only if there is \u000A some position for which both <v>x</v> and <v>y</v> have a `1' bit in that \u000A position.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>logtest</v> requires its arguments to be integers. \u000A <v>Logtest</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logtest</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logtest (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> y))))\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____ZEROP\">zerop</see> (<see topic=\"COMMON-LISP____LOGAND\">logand</see> x y))))</code>"],

 "COMMON-LISP____LOGNOT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i)</v> is the two's complement bitwise <v>`not'</v> of the \u000A integer <v>i</v>.</p> \u000A \u000A <p><v>Lognot</v> is actually defined by coercing its argument to an integer (see \u000A <see topic=\"ACL2____IFIX\">ifix</see>), negating the result, and then subtracting <v>1</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>lognot</v> requires its argument to be an \u000A integer.</p> \u000A \u000A <p><v>Lognot</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>lognot</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> lognot (i)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)))\u000A       (<see topic=\"COMMON-LISP_____B2\">+</see> (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"ACL2____IFIX\">ifix</see> i)) -1))</code>"],

 "COMMON-LISP____CHAR-CODE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-char-code</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)\u000A           (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)</code> \u000A \u000A <p>This function maps all non-characters to <v>0</v>.</p>"],

 "COMMON-LISP____CHAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR\">Char</see> s n)</v> is the <v>n</v>th element of <v>s</v>, zero-based.  If \u000A <v>n</v> is greater than or equal to the length of <v>s</v>, then <v>char</v> \u000A returns <v>nil</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____CHAR\">Char</see> s n)</v> has a <see topic=\"ACL2____GUARD\">guard</see> that <v>n</v> is a non-negative integer \u000A and <v>s</v> is a <tt><see topic=\"COMMON-LISP____STRINGP\">stringp</see></tt>.</p> \u000A \u000A <p><v>Char</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char (s n)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> s)\u000A                                   (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> n)\u000A                                   (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> n 0)\u000A                                   (<see topic=\"COMMON-LISP_____C3\">&lt;</see> n (<see topic=\"COMMON-LISP____LENGTH\">length</see> s)))))\u000A       (<see topic=\"COMMON-LISP____NTH\">nth</see> n (<see topic=\"COMMON-LISP____COERCE\">coerce</see> s 'list)))</code>"],

 "COMMON-LISP____ACONS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ACONS\">Acons</see> key datum alist)</v> equals the result of consing the pair \u000A <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> key datum)</v> to the front of the association list <v>alist</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____ACONS\">Acons</see> key datum alist)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"ACL2____ALISTP\">alistp</see> alist)</v>. \u000A <v>Acons</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>acons</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> acons (key datum alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> alist)))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> key datum) alist))</code>"],

 "ACL2____UBU_12":[["History"],"ACL2 Sources","ACL2","<code>Example:\u000A:ubu! :x-4</code> \u000A \u000A <p>The keyword <see topic=\"ACL2____COMMAND\">command</see> <v>:ubu!</v> is the same as <v>:</v><tt><see topic=\"ACL2____UBU\">ubu</see></tt>, but \u000A with related queries suppressed appropriately, and with a guarantee that it is \u000A ``error-free.''  More precisely, the error triple (see <see topic=\"ACL2____ERROR-TRIPLES\">error-triples</see>) \u000A returned by <v>:ubu!</v>  will always be of the form <v>(<see topic=\"ACL2____MV\">mv</see> nil val state)</v>. \u000A <v>:</v><tt><see topic=\"ACL2____OOPS\">Oops</see></tt> will undo the last <v>:ubu!</v>.  Also see <see topic=\"ACL2____UBU\">ubu</see>, see \u000A <see topic=\"ACL2____UBT\">ubt</see>, and see <see topic=\"ACL2____U\">u</see>.</p>"],

 "ACL2____The_02Time_02Taken_02to_02do_02the_02Associativity_02of_02App_02Proof":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The time it took us to explain this proof may leave the impression \u000A that the proof is complicated.  In a way, it is.  But it happens quickly.</p> \u000A \u000A <p>The time taken to do this proof is about 1/10 second.  The rest of the time \u000A (about 2/10 seconds) is spent in pre- and post-processing.</p> \u000A \u000A <p>Basically, this proof flashes across your screen before you can read it; \u000A you see the <b>Q.E.D.</b> and don't bother to scroll back to read it.  You \u000A have more important things to do than read successful proofs.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_021_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP <b>(IF </b><see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_022_92'>(</see><b>CONSP A)</b> \u000A                         <b>(CONS (CAR A) (APP (CDR A) B))</b> \u000A                         <b>B)</b> \u000A                     C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Note that the <v>IF</v> expression above is the simplified body of <v>APP</v>. \u000A But we know the test <v>(<see topic=\"COMMON-LISP____CONSP\">CONSP</see> A)</v> is true, by the first hypothesis.  Click \u000A on the link above to replace the test by <v>T</v>.  Actually this step and \u000A several subsequent ones are done during the simplification of the body of \u000A <v>APP</v> but we want to illustrate the basic principles of simplification \u000A without bothering with every detail.</p>"],

 "ACL2____The_02Rules_02used_02in_02the_02Associativity_02of_02App_02Proof":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Note that under <b>Rules</b> we list the <see topic='ACL2____RUNE'>runes</see> <see topic='ACL2____A_02Tiny_02Warning_02Sign'><icon \u000A src='twarning.gif'/></see> of all the rules used in the proof.  This list says \u000A that we used the rewrite rules <v>CAR-CONS</v> and <v>CDR-CONS</v>, the \u000A definitions of the functions <v>NOT</v>, <v>ENDP</v> and <v>APP</v>, and primitive \u000A type reasoning (which is how we simplified the <v>IF</v> and <v>EQUAL</v> \u000A terms).</p> \u000A \u000A <p>For what it is worth, <v>IMPLIES</v> and <v>AND</v> are actually <see \u000A topic='COMMON-LISP____DEFMACRO'>macros</see> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see> \u000A that are expanded into <v>IF</v> expressions before the proof ever begins.  The \u000A use of macros is not reported among the rules.</p>"],

 "ACL2____TAU-INTERVAL-LO-REL":[["Tau-system"],"ACL2 Sources","ACL2","<p>It is the case that</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-INTERVAL-LO-REL\">tau-interval-lo-rel</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) = lo-rel</code> \u000A \u000A <p>For a well-formed interval, <v>lo-rel</v> is a Boolean, where <v>t</v> denotes \u000A the <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt> (strong inequality or ``less-than'') relation and <v>nil</v> \u000A denotes <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt> (weak inequality or ``less-than-or-equal'') relation \u000A between the lower bound and the elements of the interval.</p> \u000A \u000A <p>When the domain of an interval is <v>INTEGERP</v>, there are additional \u000A constraints on the other components.  See <see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see>.</p>"],

 "ACL2____TAU-INTERVAL-HI-REL":[["Tau-system"],"ACL2 Sources","ACL2","<p>It is the case that</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-INTERVAL-HI-REL\">tau-interval-hi-rel</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) = hi-rel</code> \u000A \u000A <p>For a well-formed interval, <v>hi-rel</v> is a Boolean, where <v>t</v> denotes \u000A the <tt><see topic=\"COMMON-LISP_____C3\">&lt;</see></tt> (strong inequality or ``less-than'') relation and <v>nil</v> \u000A denotes <tt><see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see></tt> (weak inequality or ``less-than-or-equal'') relation \u000A between the elements of the interval and the upper bound.</p> \u000A \u000A <p>When the domain of an interval is <v>INTEGERP</v>, there are additional \u000A constraints on the other components.  See <see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see>.</p>"],

 "ACL2____TAU-INTERVAL-HI":[["Tau-system"],"ACL2 Sources","ACL2","<p>It is the case that</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-INTERVAL-HI\">tau-interval-hi</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) = hi</code> \u000A \u000A <p>For a well-formed interval, <v>hi</v> is either <v>nil</v>, denoting positive \u000A infinity, or a rational number giving the upper bound of the interval.  It \u000A must be the case that the upper bound is weakly above the lower bound of a \u000A well-formed interval.</p> \u000A \u000A <p>When the domain of an interval is <v>INTEGERP</v>, there are additional \u000A constraints on the other components.  See <see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see>.</p>"],

 "ACL2____STRING-APPEND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>String-append</v> takes two arguments, which are both strings (if \u000A the <see topic=\"ACL2____GUARD\">guard</see> is to be met), and returns a string obtained by concatenating \u000A together the <see topic=\"ACL2____CHARACTERS\">characters</see> in the first string followed by those in the \u000A second.  Also see <see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see>, noting that the macro call</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string str1 str2).</code> \u000A \u000A <p>expands to the call</p> \u000A \u000A <code>(<see topic=\"ACL2____STRING-APPEND\">string-append</see> str1 str2).</code> \u000A \u000A <p><b>Function: </b><srclink>string-append</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string-append (str1 str2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2))))\u000A       (<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str1 'list)\u000A                                   (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str2 'list))\u000A                           'string)\u000A            :exec (<see topic=\"COMMON-LISP____CONCATENATE\">concatenate</see> 'string str1 str2)))</code>"],

 "ACL2____SHOW-CUSTOM-KEYWORD-HINT-EXPANSION":[["Custom-keyword-hints"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____SHOW-CUSTOM-KEYWORD-HINT-EXPANSION\">show-custom-keyword-hint-expansion</see> t)\u000A(<see topic=\"ACL2____SHOW-CUSTOM-KEYWORD-HINT-EXPANSION\">show-custom-keyword-hint-expansion</see> nil)</code> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SHOW-CUSTOM-KEYWORD-HINT-EXPANSION\">show-custom-keyword-hint-expansion</see> flg)</code> \u000A \u000A <p>If the value of <v>flg</v> is non-<v>nil</v>, then when custom keyword hints \u000A   are expanded, the system prints the results of each expansion.  This is \u000A   sometimes useful for debugging custom keyword hints and, from time to time, \u000A   may be useful in understanding how a custom hint affects some proof \u000A   attempt.</p> \u000A \u000A <p>The default setting is <v>nil</v>.</p> \u000A \u000A <p>For an explanation of how custom keyword hints are processed, see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>.</p>"],

 "ACL2____SHARP-BANG-READER":[["Other"],"ACL2 Sources","ACL2","<code>Examples:\u000A\u000AACL2 !&gt;(<see topic=\"ACL2____DEFPKG\">defpkg</see> &quot;FOO&quot; nil)\u000A\u000ASummary\u000AForm:  ( DEFPKG &quot;FOO&quot; ...)\u000ARules: NIL\u000AWarnings:  None\u000ATime:  0.00 seconds (prove: 0.00, print: 0.00, other: 0.00)\u000A &quot;FOO&quot;\u000AACL2 !&gt;'#!foo(a b)\u000A(FOO::A FOO::B)\u000AACL2 !&gt;'#!foo(a #!acl2 b)\u000A(FOO::A B)\u000AACL2 !&gt;'#!foo(#!acl2 a b)\u000A(A FOO::B)\u000AACL2 !&gt;'#!foo(#!&quot;ACL2&quot; a b)\u000A(A FOO::B)\u000AACL2 !&gt;</code> \u000A \u000A <p>The ACL2 reader supports the syntax <v>#!pkg-name expr</v> where \u000A <v>pkg-name</v> is a string or symbol that names a package known to ACL2.  As \u000A illustrated above, this syntax nests as one might expect.  In the special case \u000A that <v>expr</v> is a symbol, <v>#!pkg-name expr</v> is equivalent to \u000A <v>pkg-name::expr</v>.</p>"],

 "ACL2____SET-DEFAULT-HINTS_12":[["Default-hints"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see>, which is the same as \u000A <v>set-default-hints!</v>  except that the latter is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs.  Probably <see topic=\"ACL2____SET-DEFAULT-HINTS\">set-default-hints</see> is to be preferred unless you have a good reason for \u000A wanting to export the effect of this event outside the enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.</p>"],

 "ACL2____REMOVE-MACRO-FN":[["Macros"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see> binary-append)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see> macro-fn)</code> \u000A \u000A <p>See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> for a discussion of how to associate a macro name \u000A with a function name.  This form sets <tt><see topic=\"ACL2____UNTRANS-TABLE\">untrans-table</see></tt> to the result of \u000A deleting the association of a macro name with the given binary function name. \u000A If the function name has no such association, then this form still generates \u000A an event, but the event has no real effect.</p>"],

 "ACL2____REMOVE-MACRO-ALIAS":[["Macros"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see> append)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REMOVE-MACRO-ALIAS\">remove-macro-alias</see> macro-name)</code> \u000A \u000A <p>See <see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see> for a discussion of macro aliases; also see \u000A <see topic=\"ACL2____ADD-MACRO-ALIAS\">add-macro-alias</see>.  This form sets <tt><see topic=\"ACL2____MACRO-ALIASES-TABLE\">macro-aliases-table</see></tt> to the \u000A result of deleting the key <v>macro-name</v> from that <see topic=\"ACL2____TABLE\">table</see>.  If the \u000A name does not occur in the <see topic=\"ACL2____TABLE\">table</see>, then this form still generates an \u000A event, but the event has no real effect.</p>"],

 "ACL2____REMOVE-DEFAULT-HINTS_12":[["Default-hints"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see>, which is the same as \u000A <v>remove-default-hints!</v>  except that the latter is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs.  Probably <see topic=\"ACL2____REMOVE-DEFAULT-HINTS\">remove-default-hints</see> is to be preferred unless you have a good reason for \u000A wanting to export the effect of this event outside the enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.</p>"],

 "ACL2____REAL_F2RATIONALP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For most ACL2 users, this is a macro abbreviating <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt>.  In ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>), this macro abbreviates the \u000A predicate <v>realp</v>, which holds for real numbers as well (including \u000A rationals).  Most ACL2 users can ignore this macro and use <tt><see topic=\"COMMON-LISP____RATIONALP\">rationalp</see></tt> \u000A instead, but many community books use <v>real/rationalp</v> so that these books \u000A will be suitable for ACL2(r) as well.</p>"],

 "ACL2____INTERSECTION-THEORIES":[["Theories","Theory-functions"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____INTERSECTION-THEORIES\">intersection-theories</see> (<see topic=\"ACL2____CURRENT-THEORY\">current-theory</see> :here)\u000A                       (<see topic=\"ACL2____THEORY\">theory</see> 'arith-patch))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____INTERSECTION-THEORIES\">intersection-theories</see> th1 th2)</code> \u000A \u000A <p>where <v>th1</v> and <v>th2</v> are theories (see <see topic=\"ACL2____THEORIES\">theories</see>).  To each of \u000A the arguments there corresponds a runic theory.  This function returns the \u000A intersection of those two runic <see topic=\"ACL2____THEORIES\">theories</see>, represented as a list and \u000A ordered chronologically.</p> \u000A \u000A <p>This ``function'' is actually a macro that expands to a term mentioning the \u000A single free variable <tt><see topic=\"ACL2____WORLD\">world</see></tt>.  When theory expressions are evaluated by \u000A <tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> or the <v>:</v><tt><see topic=\"ACL2____IN-THEORY\">in-theory</see></tt> hint, <tt><see topic=\"ACL2____WORLD\">world</see></tt> is bound \u000A to the current ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____HELP":[["Legacy-documentation"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;:help</code> \u000A \u000A <p>See <see topic=\"ACL2____LP\">lp</see> for general information about the top-level <see topic=\"ACL2____COMMAND\">command</see> \u000A environment for ACL2.</p>"],

 "ACL2____FMT_12":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This function is nearly identical to <v>fmt</v>; see <see topic=\"ACL2____FMT\">fmt</see>.  The \u000A only difference is that <v>fmt</v> may insert backslash (\\) characters when \u000A forced to print past the right margin in order to make the output a bit \u000A clearer in that case.  Use <v>fmt!</v> instead if you want to be able to read \u000A the forms back in.</p>"],

 "ACL2____FINDING-DOCUMENTATION":[["Documentation"],"ACL2 Sources","ACL2","<p>The <v>:</v><tt><see topic=\"ACL2____DOC\">doc</see></tt> command will display, at the terminal, <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topics defined in ACL2 or in <see topic=\"ACL2____BOOKS\">books</see> that have already \u000A been included.  But how can you find documentation for books that are not \u000A included in the current ACL2 session?</p> \u000A \u000A <p>The <see topic=\"ACL2____XDOC\">xdoc</see> <a href='http://www.cs.utexas.edu/users/moore/acl2/v6-5/combined-manual/index.html'>ACL2+Books Manual</a> includes documentation \u000A for both the ACL2 system and the <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see>.  For more information \u000A on this manual and how to view it, see <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see>.</p>"],

 "ACL2____ASSOC-STRING-EQUAL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ASSOC-STRING-EQUAL\">Assoc-string-equal</see> x alist)</v> is similar to <tt><see topic=\"ACL2____ASSOC-EQUAL\">assoc-equal</see></tt>.  However, for string <v>x</v> and alist <v>alist</v>, the comparison \u000A of <v>x</v> with successive keys in <v>alist</v> is done using <tt><see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see></tt> rather than <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>assoc-string-equal</v> requires that <v>x</v> is a \u000A string and <v>alist</v> is an alist.</p> \u000A \u000A <p><b>Function: </b><srclink>assoc-string-equal</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A    assoc-string-equal (str alist)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see>\u000A         (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str)\u000A                            (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> str 'list))\u000A                            (<see topic=\"ACL2____STANDARD-STRING-ALISTP\">standard-string-alistp</see> alist))))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> alist) nil)\u000A          ((<see topic=\"COMMON-LISP____STRING-EQUAL\">string-equal</see> str (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> alist)))\u000A           (<see topic=\"COMMON-LISP____CAR\">car</see> alist))\u000A          (t (<see topic=\"ACL2____ASSOC-STRING-EQUAL\">assoc-string-equal</see> str (<see topic=\"COMMON-LISP____CDR\">cdr</see> alist)))))</code>"],

 "ACL2____ADD-DEFAULT-HINTS_12":[["Default-hints"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see>, which is the same as \u000A <v>add-default-hints!</v>  except that the latter is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs.  Probably <see topic=\"ACL2____ADD-DEFAULT-HINTS\">add-default-hints</see> is to be preferred unless you have a good reason for \u000A wanting to export the effect of this event outside the enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.</p>"],

 "ACL2____ADD-BINOP":[["Macros"],"ACL2 Sources","ACL2","<p>The form <v>(<see topic=\"ACL2____ADD-BINOP\">add-binop</see> macro macro-fn)</v> is an abbreviation for the \u000A form <v>(<see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> macro macro-fn t)</v>.  See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see>.</p>"],

 "ACL2-PC____WRAP-INDUCT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Awrap-induct\u000A(wrap-induct t)\u000A(wrap-induct (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____REVERSE\">reverse</see> x) y))\u000A\u000AGeneral Form:\u000A(wrap-induct &amp;optional term)</code> \u000A \u000A <p>The <v>wrap-induct</v> command is identical to the <tt><see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see></tt> \u000A <v>induct</v> command, except that only a single goal is created: the \u000A conjunction of the base and induction steps.</p> \u000A \u000A <p>Note: The output will generally indicate that more than goal has been \u000A created, e.g.:</p> \u000A \u000A <code>Creating two new goals:  (MAIN . 1) and (MAIN . 2).</code> \u000A \u000A <p>However, <v>wrap-induct</v> always creates a unique goal (when it succeeds). \u000A A subsequent message clarifies this, for example:</p> \u000A \u000A <code>NOTE: Created ONLY one new goal, which is the current goal:\u000A  (MAIN . 1)</code> \u000A \u000A "],

 "ACL2-PC____P":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Ap</code> \u000A \u000A <p>Prettyprint the current term.  The usual user syntax is used, so that for \u000A example one would see <v>(<see topic=\"COMMON-LISP____AND\">and</see> x y)</v> rather than <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y 'nil)</v>.  (See \u000A also <v>pp</v>.)  Also, abbreviations are inserted where appropriate; see \u000A <v>add-abbreviation</v>.</p> \u000A \u000A <p>The ``current term'' is the entire conclusion unless <v>dive</v> commands \u000A have been given, in which case it may be a subterm of the conclusion.</p> \u000A \u000A <p>If all goals have been proved, a message saying so will be printed (as \u000A there will be no current <v>term</v>!).</p>"],

 "ACL2-PC____EXPAND":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Aexpand -- expand and do not simplify.</code> \u000A \u000A <p>For example, if the current subterm is <v>(<see topic=\"COMMON-LISP____APPEND\">append</see> a b)</v>, then after \u000A <v>(<see topic=\"ACL2____EXPAND\">expand</see> t)</v> the current subterm will be the term:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> x\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"COMMON-LISP____APPEND\">append</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x) y))\u000A      y)\u000A  (apply 'binary-append (<see topic=\"COMMON-LISP____LIST\">list</see> x y)))</code> \u000A \u000A <p>regardless of the top-level hypotheses and the governors.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____EXPAND\">expand</see> &amp;optional do-not-expand-lambda-flg)</code> \u000A \u000A <p>Expand the function call at the current subterm, and do not simplify.  The \u000A options have the following meanings:</p> \u000A \u000A <code>do-not-expand-lambda-flg:   default is nil; otherwise, the result\u000A                            should be a lambda expression</code> \u000A \u000A <p>See also <v>x</v>, which allows simplification.</p>"],

 "COMMON-LISP_____E3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>&gt;=</v> is a macro, and <v>(<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> x y)</v> expands to the same thing as \u000A <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y))</v>.  See <see topic=\"COMMON-LISP_____C3\">&lt;</see>.</p> \u000A \u000A <p><v>&gt;=</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>&gt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> &gt;= (x y)\u000A          (<see topic=\"COMMON-LISP____LIST\">list</see> 'not (<see topic=\"COMMON-LISP____LIST\">list</see> '&lt; x y)))</code>"],

 "COMMON-LISP____LOGORC1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logorc1</v> returns the bitwise logical inclusive or of the second with the \u000A bitwise logical `not' of the first.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>logorc1</v> requires its arguments to be integers. \u000A <v>Logorc1</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logorc1</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logorc1 (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGIOR\">logior</see> (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i) j))</code>"],

 "COMMON-LISP____LAST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____LAST\">Last</see> l)</v> is the last <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> of a list.  Here are \u000A examples.</p> \u000A \u000A <code>ACL2 !&gt;(<see topic=\"COMMON-LISP____LAST\">last</see> '(a b . c))\u000A(B . C)\u000AACL2 !&gt;(<see topic=\"COMMON-LISP____LAST\">last</see> '(a b c))\u000A(C)</code> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____LAST\">Last</see> l)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"COMMON-LISP____LISTP\">listp</see> l)</v>; thus, <v>l</v> need not \u000A be a <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt>.</p> \u000A \u000A <p><v>Last</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.  Unlike Common Lisp, we do not allow an optional second \u000A argument for <v>last</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>last</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> last (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____LISTP\">listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l)) l (<see topic=\"COMMON-LISP____LAST\">last</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))</code>"],

 "COMMON-LISP____EVENP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____EVENP\">evenp</see> x)</v> is true if and only if the integer <v>x</v> is even. \u000A Actually, in the ACL2 logic <v>(<see topic=\"COMMON-LISP____EVENP\">evenp</see> x)</v> is defined to be true when <v>x/2</v> \u000A is an integer.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>evenp</v> requires its argument to be an integer.</p> \u000A \u000A <p><v>Evenp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>evenp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> evenp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)))\u000A       (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP_____A2\">*</see> x (<see topic=\"COMMON-LISP_____F2\">/</see> 2))))</code>"],

 "ACL2____TUTORIAL5-MISCELLANEOUS-EXAMPLES":[["Annotated-ACL2-scripts"],"ACL2 Sources","ACL2","<p>The following examples are more advanced examples of usage of ACL2. \u000A They are included largely for reference, in case someone finds them \u000A useful.</p>"],

 "ACL2____TAU-INTERVAL-LO":[["Tau-system"],"ACL2 Sources","ACL2","<p>It is the case that</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-INTERVAL-LO\">tau-interval-lo</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) = lo</code> \u000A \u000A <p>For a well-formed interval, <v>lo</v> is either <v>nil</v>, denoting negative \u000A infinity, or a rational number giving the lower bound of the interval.  It \u000A must be the case that the lower bound is weakly below the upper bound of a \u000A well-formed interval.</p> \u000A \u000A <p>When the domain of an interval is <v>INTEGERP</v>, there are additional \u000A constraints on the other components.  See <see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see>.</p>"],

 "ACL2____Subsumption_02of_02Induction_02Candidates_02in_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>After collecting induction suggestions from these three terms</p> \u000A \u000A <code>(app <b>a</b> b) \u000A \u000A(app <b>b</b> c) \u000A \u000A(app <b>a</b> (app b c)) \u000A</code> \u000A \u000A <p>the system notices that the first and last suggest the same decomposition \u000A of <v>a</v>: case split on whether <v>a</v> is empty (i.e., <v>(<see topic=\"COMMON-LISP____ENDP\">endp</see> a)</v>), and in \u000A the case where it is not empty, recursively process <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> a)</v>.  So we are \u000A left with two ideas about how to induct:</p> \u000A \u000A <p>Decompose <b>a</b> as we would to unwind (app <b>a</b> b).</p> \u000A \u000A <p>Decompose <b>b</b> as we would to unwind (app <b>b</b> c).</p>"],

 "ACL2____STRIP-CARS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> x)</v> is the list obtained by walking through the \u000A list <v>x</v> and collecting up all first components (<tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>s).  This \u000A function is implemented in a tail-recursive way, despite its logical \u000A definition.</p> \u000A \u000A <p><v>(<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> x)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"ACL2____ALISTP\">alistp</see> x)</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>strip-cars</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> strip-cars (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> x)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                      (<see topic=\"ACL2____STRIP-CARS\">strip-cars</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____STANDARD-STRING-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STANDARD-STRING-ALISTP\">Standard-string-alistp</see> x)</v> is true if and only if <v>x</v> is a \u000A list of pairs of the form <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> key val)</v> where <v>key</v> is a string all \u000A of whose characters are standard (see <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>).</p> \u000A \u000A <p><v>Standard-string-alistp</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>standard-string-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A    standard-string-alistp (x)\u000A    (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A    (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))\u000A          (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                  (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                  (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____COERCE\">coerce</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)) 'list))\u000A                  (<see topic=\"ACL2____STANDARD-STRING-ALISTP\">standard-string-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____SET-BOGUS-DEFUN-HINTS-OK":[["Defun"],"ACL2 Sources","ACL2","<code>General Forms:\u000A(<see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see> t)\u000A(<see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see> nil)\u000A(<see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see> :warn)</code> \u000A \u000A <p>By default, ACL2 causes an error when the keyword <v>:</v><tt><see topic=\"ACL2____HINTS\">hints</see></tt> is \u000A supplied in an <tt><see topic=\"ACL2____XARGS\">xargs</see></tt> <see topic=\"COMMON-LISP____DECLARE\">declare</see> form for a non-recursive \u000A definition (see <see topic=\"COMMON-LISP____DEFUN\">defun</see>).  This behavior can be defeated with \u000A <v>(<see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see> t)</v>, or if you still want to see a warning in \u000A such cases, <v>(<see topic=\"ACL2____SET-BOGUS-DEFUN-HINTS-OK\">set-bogus-defun-hints-ok</see> :warn)</v>.</p>"],

 "ACL2____RFIX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Rfix</v> simply returns any rational number argument unchanged, \u000A returning <v>0</v> on a non-rational argument.  Also see <see topic=\"ACL2____NFIX\">nfix</see>, see <see topic=\"ACL2____IFIX\">ifix</see>, see <see topic=\"ACL2____REALFIX\">realfix</see>, and see <see topic=\"ACL2____FIX\">fix</see> for analogous functions that \u000A coerce to a natural number, an integer, a real, and a number, \u000A respectively.</p> \u000A \u000A <p><v>Rfix</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>rfix</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rfix (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x) x 0))</code>"],

 "ACL2____PCB_12":[["History"],"ACL2 Sources","ACL2","<code>Examples:\u000A:pcb! :max ; print the most recent command block\u000A:pcb! :x   ; print the most recent command block\u000A:pcb! fn   ; print the command block that introduced fn\u000A:pcb! 5    ; print the fifth command block</code> \u000A \u000A <p>See <see topic=\"ACL2____COMMAND-DESCRIPTOR\">command-descriptor</see>.</p> \u000A \u000A <p><v>Pcb!</v> takes one argument, a <see topic=\"ACL2____COMMAND\">command</see> descriptor, and prints the \u000A <see topic=\"ACL2____COMMAND\">command</see> block of the <see topic=\"ACL2____COMMAND\">command</see> described.  Unlike <tt><see topic=\"ACL2____PCB\">pcb</see></tt>, \u000A <v>pcb!</v> prints the event forms in full; see <see topic=\"ACL2____PCB\">pcb</see> for details.</p>"],

 "ACL2____Other_02Requirements":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02End_02of_02the_02Flying_02Tour'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>ACL2 is distributed on the Web without fee.</p> \u000A \u000A <p>There is a <b>license</b> agreement based on the 3-clause BSD license.  See \u000A the file LICENSE in the ACL2 distribution.</p> \u000A \u000A <p>ACL2 currently runs on <b>Unix</b>, <b>Linux</b>, <b>Windows</b>, and \u000A <b>Macintosh OS X</b> operating systems.</p> \u000A \u000A <p>It can be built in any of the following Common Lisps:</p> \u000A \u000A <code>* <b>Allegro Common Lisp</b>, \u000A* <b>CCL</b> (formerly OpenMCL) \u000A* <b>CLISP</b>, \u000A* <b>CMU Common Lisp</b>, \u000A* <b>GCL</b> (Gnu Common Lisp), \u000A* <b>LispWorks</b>, and \u000A* <b>SBCL</b> (Steel Bank Common Lisp) \u000A </code> \u000A \u000A <p><see topic='ACL2____The_02End_02of_02the_02Flying_02Tour'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____OPEN-TRACE-FILE":[["Trace"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see> &quot;foo&quot;) ; trace output will go to file foo\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see> filename) ; trace output will go to file filename</code> \u000A \u000A <p>Output from <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> normally goes to the screen, i.e., <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  But it can be redirected to a file as shown above.  See <see topic=\"ACL2____CLOSE-TRACE-FILE\">close-trace-file</see> for how to send trace output back to the screen.</p>"],

 "ACL2____Models_02in_02Engineering":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='bridge.gif'></img></p> \u000A \u000A <p>Frequently, engineers use mathematical models.  Use of such models \u000A frequently lead to</p> \u000A \u000A <p><b>better designs</b>,</p> \u000A \u000A <p><b>faster completion of acceptable products</b>, and</p> \u000A \u000A <p><b>reduced overall cost</b>,</p> \u000A \u000A <p>because models allow the trained user to study the design before it is \u000A built and analyze its properties.  Usually, testing and analyzing a model is \u000A cheaper and faster than fabricating and refabricating the product.</p> \u000A \u000A <p><img src='bridge-analysis.gif'></img></p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This answer is in the form of an ACL2 script sufficient to lead \u000A ACL2 to a proof.</p> \u000A \u000A <code>(defun rev (x) \u000A  (if (endp x) \u000A      nil \u000A      (append (rev (cdr x)) (list (car x))))) \u000A \u000A; Trying <v>triple-rev</v> at this point produces a key checkpoint containing \u000A; <v>(REV (<see topic=\"COMMON-LISP____APPEND\">APPEND</see> (REV (<see topic=\"COMMON-LISP____CDR\">CDR</see> X)) (<see topic=\"COMMON-LISP____LIST\">LIST</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> X))))</v>, which suggests: \u000A \u000A(defthm rev-append \u000A  (equal (rev (append a b)) \u000A         (append (rev b) (rev a)))) \u000A \u000A; And now <v>triple-rev</v> succeeds. \u000A \u000A(defthm triple-rev \u000A  (equal (rev (rev (rev x))) (rev x))) \u000A \u000A; An alternative, and more elegant, solution is to prove the <v>rev-rev</v> \u000A; instead of <v>rev-append</v>: \u000A \u000A; (defthm rev-rev \u000A;   (implies (true-listp x) \u000A;            (equal (rev (rev x)) x))) \u000A \u000A; <v>Rev-rev</v> is also discoverable by The Method because it is \u000A; suggested by the statement of <v>triple-rev</v> itself: <v>rev-rev</v> \u000A; simplifies a simpler composition of the functions in <v>triple-rev</v>. \u000A \u000A; Both solutions produce lemmas likely to be of use in future proofs \u000A; about <v>rev</v>. \u000A \u000A</code> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-1\">introductory-challenge-problem-1</see>.</p>"],

 "ACL2____GETPROP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General form:\u000A(<see topic=\"ACL2____GETPROP\">getprop</see> symb key default world-name world-alist)</code> \u000A \u000A <p>See community book <v>books/misc/getprop.lisp</v> for an example that \u000A illustrates the use of ACL2 utilities <tt><see topic=\"ACL2____GETPROP\">getprop</see></tt> and <v>putprop</v> to take \u000A advantage of under-the-hood Lisp (hashed) property lists.</p> \u000A \u000A <p><b>Macro: </b><srclink>getprop</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see>\u000A    getprop\u000A    (symb key default world-name world-alist)\u000A    (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> world-name ''current-acl2-world)\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> 'fgetprop\u000A              (<see topic=\"COMMON-LISP____CONS\">cons</see> symb\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> key\u000A                          (<see topic=\"COMMON-LISP____CONS\">cons</see> default (<see topic=\"COMMON-LISP____CONS\">cons</see> world-alist 'nil)))))\u000A        (<see topic=\"COMMON-LISP____CONS\">cons</see> 'sgetprop\u000A              (<see topic=\"COMMON-LISP____CONS\">cons</see> symb\u000A                    (<see topic=\"COMMON-LISP____CONS\">cons</see> key\u000A                          (<see topic=\"COMMON-LISP____CONS\">cons</see> default\u000A                                (<see topic=\"COMMON-LISP____CONS\">cons</see> world-name\u000A                                      (<see topic=\"COMMON-LISP____CONS\">cons</see> world-alist 'nil))))))))</code>"],

 "ACL2____FIX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Fix</v> simply returns any numeric argument unchanged, returning \u000A <v>0</v> on a non-numeric argument.  Also see <see topic=\"ACL2____NFIX\">nfix</see>, see <see topic=\"ACL2____IFIX\">ifix</see>, and \u000A see <see topic=\"ACL2____RFIX\">rfix</see> for analogous functions that coerce to a natural number, an \u000A integer, and a rational number, respectively.</p> \u000A \u000A <p><v>Fix</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>fix</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fix (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x) x 0))</code>"],

 "ACL2____EXAMPLE-INDUCTION-SCHEME-ON-SEVERAL-VARIABLES":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____LOGIC-KNOWLEDGE-TAKEN-FOR-GRANTED-INDUCTIVE-PROOF\">logic-knowledge-taken-for-granted-inductive-proof</see> for an \u000A explanation of what we mean by the induction <i>suggested</i> by a recursive \u000A function or a term.</p> \u000A \u000A <p><b>Induction on Several Variables</b> To <v>(p n x)</v> for all <v>n</v> and \u000A all <v>x</v>, prove each of the following:</p> \u000A \u000A <code><i>Base Case 1</i>: \u000A(implies (endp x) \u000A         (p n x)) \u000A</code> \u000A \u000A <code><i>Base Case 2</i>: \u000A(implies (and (not (endp x)) \u000A              (zp n)) \u000A         (p n x)) \u000A</code> \u000A \u000A <code><i>Induction Step</i>: \u000A(implies (and (not (endp x)) \u000A              (not (zp n)) \u000A              (p (- n 1) (cdr x))) \u000A         (p n x)) \u000A</code> \u000A \u000A <p>A function that suggests this induction is</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nth (n x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ENDP\">endp</see> x)\u000A      nil\u000A      (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ZP\">zp</see> n)\u000A          (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A          (<see topic=\"COMMON-LISP____NTH\">nth</see> (<see topic=\"COMMON-LISP____-\">-</see> n 1) (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))).</code>"],

 "ACL2____COMPLEX_F2COMPLEX-RATIONALP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For most ACL2 users, this is a macro abbreviating \u000A <v>complex-rationalp</v>; see <see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see>.  In ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>), a complex number <v>x</v> may have irrational real and imaginary parts. \u000A This macro abbreviates the predicate <v>complexp</v> in ACL2(r), which holds for \u000A such <v>x</v>.  Most ACL2 users can ignore this macro and use <tt><see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see></tt> instead.  Some community books use \u000A <v>complex/complex-rationalp</v> so that they are suitable for ACL2(r) as \u000A well.</p>"],

 "ACL2____CLEAR-MEMOIZE-TABLES":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p><v>Clear-memoize-tables</v> is a logical no-op.  All memoized values are \u000A forgotten.  It returns <v>nil</v>, invoking <tt><see topic=\"ACL2____CLEAR-MEMOIZE-TABLE\">clear-memoize-table</see></tt> for each \u000A memoized function.</p> \u000A \u000A <p><b>Function: </b><srclink>clear-memoize-tables</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> clear-memoize-tables\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____ASSOC-KEYWORD":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>If <v>l</v> is a list of even length of the form <v>(k1 a1 k2 a2\u000A ... kn an)</v>, where each <v>ki</v> is a keyword, then <v>(<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> key l)</v> \u000A is the first tail of <v>l</v> starting with <v>key</v> if key is some <v>ki</v>, and \u000A is <v>nil</v> otherwise.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>(<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> key l)</v> is <v>(<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see>\u000A l)</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>assoc-keyword</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> assoc-keyword (key l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____KEYWORD-VALUE-LISTP\">keyword-value-listp</see> l)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> l) nil)\u000A             ((<see topic=\"COMMON-LISP____EQ\">eq</see> key (<see topic=\"COMMON-LISP____CAR\">car</see> l)) l)\u000A             (t (<see topic=\"ACL2____ASSOC-KEYWORD\">assoc-keyword</see> key (<see topic=\"COMMON-LISP____CDDR\">cddr</see> l)))))</code>"],

 "ACL2____ACTIVE-RUNEP":[["Theories"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> '(:rewrite left-to-right))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ACTIVE-RUNEP\">active-runep</see> rune)</code> \u000A \u000A <p>where <v>rune</v> has the shape of a <see topic=\"ACL2____RUNE\">rune</see>.  This macro expands to an \u000A expression using the variables <v>ens</v> and <v>state</v>, and returns \u000A non-<v>nil</v> when the given rune exists and is <see topic=\"ACL2____ENABLE\">enable</see>d (according to \u000A the given ``enabled structure,'' <v>ens</v>, and the current logical <see topic=\"ACL2____WORLD\">world</see> of the given <tt><see topic=\"ACL2____STATE\">state</see></tt>).  See <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> for how this \u000A macro can be of use.</p>"],

 "ACL2-PC____TOP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Atop</code> \u000A \u000A <p>For example, if the conclusion is <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z))</v> and the current \u000A subterm is <v>y</v>, then after executing <v>top</v>, the current subterm will be \u000A the same as the conclusion, i.e., <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z))</v>.</p> \u000A \u000A <p><v>Top</v> is the same as <v>(up n)</v>, where <v>n</v> is the number of times \u000A one needs to execute <v>up</v> in order to get to the top of the conclusion. \u000A The <v>top</v> command fails if one is already at the top of the conclusion.</p> \u000A \u000A <p>See also <v>up</v>, <v>dive</v>, <v>nx</v>, and <v>bk</v>.</p>"],

 "ACL2-PC____TH":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Ath               -- print all (<see topic=\"ACL2____TOP-LEVEL\">top-level</see>) hypotheses and the current\u000A                    subterm\u000A(th (1 3) (2 4)) -- print hypotheses 1 and 3 and governors 2 and 4,\u000A                    and the current subterm\u000A(th (1 3) t)     -- print hypotheses 1 and 3 and all governors, and\u000A                    the current subterm\u000A\u000AGeneral Form:\u000A(th &amp;optional hyps-indices govs-indices)</code> \u000A \u000A <p>Print hypotheses and the current subterm.  The printing of hypotheses (and \u000A perhaps governors) are controlled as in the <v>hyps</v> command; see its \u000A documentation.</p> \u000A \u000A <p>Historical note: The name <v>th</v> is adapted from the Gypsy Verification \u000A Environment, where <v>th</v> abbreviates the command <v>theorem</v>, which says to \u000A print information on the current goal.</p>"],

 "ACL2-PC____SL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Asl\u000A(sl 3)\u000A\u000AGeneral Form:\u000A(sl &amp;optional backchain-limit)</code> \u000A \u000A <p>Simplify, but with all function definitions disabled (see <see topic=\"ACL2____FUNCTION-THEORY\">function-theory</see> in the top-level ACL2 loop), except for a few basic functions \u000A (the ones in <v>(<see topic=\"ACL2____THEORY\">theory</see> 'minimal-theory)</v>).  The <v>backchain-limit</v> has a \u000A default of 0, but if is supplied and not <v>nil</v>, then it should be a \u000A nonnegative integer; see the documentation for <v>s</v>.</p> \u000A \u000A <p>WARNING: This command completely ignores <v>in-theory</v> commands that are \u000A executed inside the proof-checker.</p>"],

 "ACL2-PC____NX":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Anx</code> \u000A \u000A <p>For example, if the conclusion is <v>(<see topic=\"COMMON-LISP_____D3\">=</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z))</v> and the current \u000A subterm is <v>x</v>, then after executing <v>nx</v>, the current subterm will be \u000A <v>(<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z)</v>.</p> \u000A \u000A <p>This is the same as <v>up</v> followed by <v>(dive n+1)</v>, where <v>n</v> is \u000A the position of the current subterm in its parent term in the conclusion. \u000A Thus in particular, the <v>nx</v> command fails if one is already at the top of \u000A the conclusion.</p> \u000A \u000A <p>See also <v>up</v>, <v>dive</v>, <v>top</v>, and <v>bk</v>.</p>"],

 "ACL2-PC____NOISE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(noise induct prove)\u000A\u000AGeneral Form:\u000A(noise &amp;rest instruction-list)</code> \u000A \u000A <p>Run the <v>instruction-list</v> through the top-level loop with output.</p> \u000A \u000A <p>In fact, having output is the default.  <v>Noise</v> is useful inside a \u000A surrounding call of <v>quiet</v>, when one temporarily wants output.  For \u000A example, if one wants to see output for a <v>prove</v> command immediately \u000A following an <v>induct</v> command but before an <v>s</v> command, one may want to \u000A submit an instruction like <v>(quiet induct (noise prove) s)</v>.  See also \u000A <v>quiet</v>.</p>"],

 "ACL2-PC____BDD":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Abdd\u000A(<see topic=\"ACL2____BDD\">bdd</see> :vars nil :bdd-constructors (<see topic=\"COMMON-LISP____CONS\">cons</see>) :prove t :literal :all)</code> \u000A \u000A <p>The general form is as shown in the latter example above, but with any \u000A keyword-value pairs omitted and with values as described for the <v>:</v><tt><see topic=\"ACL2____BDD\">bdd</see></tt> hint; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>This command simply calls the theorem prover with the indicated bdd hint \u000A for the top-level goal.  Note that if <v>:prove</v> is <v>t</v> (the default), \u000A then the proof will succeed entirely using bdds or else it will fail \u000A immediately.  See <see topic=\"ACL2____BDD\">bdd</see>.</p>"],

 "COMMON-LISP____STRING_E3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____STRING_E3\">String&gt;</see> str1 str2)</v> is non-<v>nil</v> if and only if <v>str2</v> \u000A precedes <v>str1</v> lexicographically.  When non-<v>nil</v>, <v>(<see topic=\"COMMON-LISP____STRING_E3\">string&gt;</see> str1\u000A str2)</v> is the first position (zero-based) at which the strings differ.  See \u000A <see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see>.</p> \u000A \u000A <p><v>String&gt;</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>string&gt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string&gt; (str1 str2)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str1) (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> str2))))\u000A       (<see topic=\"COMMON-LISP____STRING_C3\">string&lt;</see> str2 str1))</code>"],

 "COMMON-LISP____LOGORC2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logorc2</v> returns the bitwise logical inclusive or of the first with the \u000A bitwise logical `not' of the second.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>logorc2</v> requires its arguments to be integers. \u000A <v>Logorc2</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logorc2</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logorc2 (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGIOR\">logior</see> i (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> j)))</code>"],

 "COMMON-LISP____LOGCOUNT":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____LOGCOUNT\">Logcount</see> x)</v> is the number of ``on'' bits in the two's \u000A complement representation of <v>x</v>.</p> \u000A \u000A <p><v>(<see topic=\"COMMON-LISP____LOGCOUNT\">Logcount</see> x)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)</v>.</p> \u000A \u000A <p><v>Logcount</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logcount</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     logcount (x)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)))\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"ACL2____ZIP\">zip</see> x) 0)\u000A           ((<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0) (<see topic=\"COMMON-LISP____LOGCOUNT\">logcount</see> (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> x)))\u000A           ((<see topic=\"COMMON-LISP____EVENP\">evenp</see> x)\u000A            (<see topic=\"COMMON-LISP____LOGCOUNT\">logcount</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> x 2)))\u000A           (t (<see topic=\"COMMON-LISP____1_B2\">1+</see> (<see topic=\"COMMON-LISP____LOGCOUNT\">logcount</see> (<see topic=\"ACL2____NONNEGATIVE-INTEGER-QUOTIENT\">nonnegative-integer-quotient</see> x 2))))))</code>"],

 "COMMON-LISP____LOGANDC2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logandc2</v> returns the bitwise logical `and' of the first with the bitwise \u000A logical `not' of the second.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>logandc2</v> requires its arguments to be integers. \u000A <v>Logandc2</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logandc2</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logandc2 (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGAND\">logand</see> i (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> j)))</code>"],

 "COMMON-LISP____LOGANDC1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>logandc1</v> returns the bitwise logical `and' of the second with the bitwise \u000A logical `not' of the first.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>logandc1</v> requires its arguments to be integers. \u000A <v>Logandc1</v> is defined in Common Lisp.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>logandc1</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> logandc1 (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGAND\">logand</see> (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> i) j))</code>"],

 "COMMON-LISP____CHAR_E3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR_E3_D3\">char&gt;=</see> x y)</v> is true if and only if the character code of \u000A <v>x</v> is greater than or equal to that of <v>y</v>.  See <see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char&gt;=</v> specifies that its arguments are <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><v>Char&gt;=</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char&gt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char&gt;= (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y))))\u000A       (<see topic=\"COMMON-LISP_____E3_D3\">&gt;=</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x) (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> y)))</code>"],

 "COMMON-LISP____CHAR_C3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR_C3\">char&lt;</see> x y)</v> is true if and only if the character code of \u000A <v>x</v> is less than that of <v>y</v>.  See <see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char&lt;</v> specifies that its arguments are <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><v>Char&lt;</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char&lt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char&lt; (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y))))\u000A       (<see topic=\"COMMON-LISP_____C3\">&lt;</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x) (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> y)))</code>"],

 "ACL2____UNTRANS-TABLE":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000AACL2 !&gt;(<see topic=\"ACL2____UNTRANS-TABLE\">untrans-table</see> (w state))\u000A((<see topic=\"ACL2____BINARY-_B2\">BINARY-+</see> + . T)\u000A (<see topic=\"ACL2____BINARY-_A2\">BINARY-*</see> * . T)\u000A (<see topic=\"ACL2____BINARY-APPEND\">BINARY-APPEND</see> APPEND . T)\u000A (BINARY-LOGAND LOGAND . T)\u000A (BINARY-LOGIOR LOGIOR . T)\u000A (BINARY-LOGXOR LOGXOR . T)\u000A (BINARY-LOGEQV LOGEQV . T)\u000A (BINARY-POR POR . T)\u000A (BINARY-PAND PAND . T))</code> \u000A \u000A <p>See <see topic=\"ACL2____TABLE\">table</see> for a general discussion of tables.</p> \u000A \u000A <p>See <see topic=\"ACL2____ADD-MACRO-FN\">add-macro-fn</see> for a more general discussion of this <see topic=\"ACL2____TABLE\">table</see> \u000A and for a way to associate a macro name with a function name in theory \u000A events.</p>"],

 "ACL2____TRANS_12":[["Macros"],"ACL2 Sources","ACL2","<code>Examples:\u000A:trans! (<see topic=\"COMMON-LISP____LIST\">list</see> a b c)\u000A:trans! (<see topic=\"COMMON-LISP____APPEND\">append</see> x state)</code> \u000A \u000A <p><v>:Trans!</v> is identical to <v>:</v><tt><see topic=\"ACL2____TRANS\">trans</see></tt>, except that unlike \u000A <v>:trans</v>, <v>:trans!</v> ignores single-threadedness restrictions.  Thus, the \u000A second form above is legal for <v>:trans!</v>.  Also see <see topic=\"ACL2____TRANS\">trans</see> and see \u000A <see topic=\"ACL2____TRANS1\">trans1</see>.</p>"],

 "ACL2____TAU-INTERVAL-DOM":[["Tau-system"],"ACL2 Sources","ACL2","<p>It is the case that</p> \u000A \u000A <code>(<see topic=\"ACL2____TAU-INTERVAL-DOM\">tau-interval-dom</see> (<see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see> dom lo-rel lo hi-rel hi)) = dom</code> \u000A \u000A <p>For a well-formed interval, <v>dom</v> is one of the symbols <v>INTEGERP</v>, \u000A <v>RATIONALP</v>, <v>ACL2-NUMBERP</v>, or <v>NIL</v>.  When the domain is <v>NIL</v> \u000A there is no domain restriction.</p> \u000A \u000A <p>When the domain is <v>INTEGERP</v>, there are additional constraints on the \u000A other components.  See <see topic=\"ACL2____MAKE-TAU-INTERVAL\">make-tau-interval</see>.</p>"],

 "ACL2____TAU-DATABASE":[["Tau-system","History"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____TAU-DATABASE\">tau-database</see> (w state))</code> \u000A \u000A <p>This function returns a large list object that shows in a human-readable \u000A way what the tau system knows about every function symbol.  It is supposed to \u000A be self-explanatory.  See <see topic=\"ACL2____INTRODUCTION-TO-THE-TAU-SYSTEM\">introduction-to-the-tau-system</see> for background \u000A details.</p> \u000A \u000A <p>If the output is not self-explanatory, please contact the implementors and \u000A we will improve the output or the documentation.</p>"],

 "ACL2____Symbolic_02Execution_02of_02Models":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Proving_02Theorems_02about_02Models'><img \u000A src='flying.gif'></img></see></p> \u000A \u000A <p>But ACL2 is more than a programming language.</p> \u000A \u000A <p>Initialize <b>x</b> to 5 and let <b>y</b> be <b>any legal value</b>.</p> \u000A \u000A <p><img src='computing-machine-5xy.gif'></img></p> \u000A \u000A <p>Because ACL2 is a mathematical language, we can simplify the expression</p> \u000A \u000A <code>(lookup 'z (mc (s 'mult 5 y) 29))</code> \u000A \u000A <p>and get (+ y y y y y).  This is <b>symbolic execution</b> because not all \u000A of the parameters are known.</p> \u000A \u000A <p><see topic='ACL2____Proving_02Theorems_02about_02Models'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____Suggested_02Inductions_02in_02the_02Associativity_02of_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>To find a plausible induction argument, the system studies the \u000A recursions exhibited by the terms in the conjecture.</p> \u000A \u000A <p>Roughly speaking, a call of a recursive function ``suggests'' an induction \u000A if the argument position decomposed in recursion is occupied by a \u000A variable.</p> \u000A \u000A <p>In this conjecture, three terms suggest inductions:</p> \u000A \u000A <code>(app <b>a</b> b) \u000A \u000A(app <b>b</b> c) \u000A \u000A(app <b>a</b> (app b c)) \u000A</code> \u000A \u000A <p>The variable recursively decomposed is indicated in <b>bold</b>.</p>"],

 "ACL2____SYMBOL-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____SYMBOL-ALISTP\">Symbol-alistp</see> x)</v> is true if and only if <v>x</v> is a list of \u000A pairs of the form <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> key val)</v> where <v>key</v> is a <tt><see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>symbol-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> symbol-alistp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                     (<see topic=\"ACL2____SYMBOL-ALISTP\">symbol-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____STRIP-CDRS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> x)</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>(<see topic=\"ACL2____ALISTP\">alistp</see> x)</v>, and \u000A returns the list obtained by walking through the list <v>x</v> and collecting up \u000A all second components (<tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s).  This function is implemented in a \u000A tail-recursive way, despite its logical definition.</p> \u000A \u000A <p><b>Function: </b><srclink>strip-cdrs</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> strip-cdrs (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ALISTP\">alistp</see> x)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> x) nil)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                      (<see topic=\"ACL2____STRIP-CDRS\">strip-cdrs</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____SET-INHIBIT-WARNINGS_12":[["Prover-output"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see>, which is the same as \u000A <v>set-inhibit-warnings!</v>  except that the latter is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs.  Probably <see topic=\"ACL2____SET-INHIBIT-WARNINGS\">set-inhibit-warnings</see> is to be preferred unless you have a good reason for \u000A wanting to export the effect of this event outside the enclosing <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or book.</p>"],

 "ACL2____SAVE-AND-CLEAR-MEMOIZATION-SETTINGS":[["Events","Hons-and-memoization"],"ACL2 Sources","ACL2","<p>For background on memoization, see <see topic=\"ACL2____MEMOIZE\">memoize</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____SAVE-AND-CLEAR-MEMOIZATION-SETTINGS\">save-and-clear-memoization-settings</see>)</code> \u000A \u000A <p>Calls of this macro achieve two changes.  The first copies the current \u000A memoization settings into an ACL2 <see topic=\"ACL2____TABLE\">table</see>, and the second unmemoizes all \u000A functions that were memoized by calls of <tt><see topic=\"ACL2____MEMOIZE\">memoize</see></tt>.  Also see <see topic=\"ACL2____RESTORE-MEMOIZATION-SETTINGS\">restore-memoization-settings</see>.</p>"],

 "ACL2____RESET-FC-REPORTING":[["Forward-chaining-reports"],"ACL2 Sources","ACL2","<code>Example:  (<see topic=\"ACL2____RESET-FC-REPORTING\">reset-fc-reporting</see>)</code> \u000A \u000A <p>This function erases all forward chaining tracking criteria and sets the \u000A on-the-fly reporting flag to <v>nil.</v>  The next time you set the criteria \u000A (see <see topic=\"ACL2____SET-FC-CRITERIA\">set-fc-criteria</see>) the short form of reports, in which only the \u000A caller and the fc-report number is printed, will appear in your proof \u000A logs.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for details.</p>"],

 "ACL2____REMOVE-NTH-ALIAS":[["Nth-aliases-table"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____REMOVE-NTH-ALIAS\">remove-nth-alias</see> append)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REMOVE-NTH-ALIAS\">remove-nth-alias</see> alias-name)</code> \u000A \u000A <p>See <see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see> for further discussion; also see <see topic=\"ACL2____ADD-NTH-ALIAS\">add-nth-alias</see>.  This form sets <tt><see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see></tt> to the result of \u000A deleting the key <v>alias-name</v> from that <see topic=\"ACL2____TABLE\">table</see>.  If the name does not \u000A occur in the <see topic=\"ACL2____TABLE\">table</see>, then this form still generates an event, but the \u000A event has no real effect.</p>"],

 "ACL2____REALFIX":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Realfix</v> simply returns any real number argument unchanged, \u000A returning <v>0</v> on a non-real argument.  Also see <see topic=\"ACL2____NFIX\">nfix</see>, see <see topic=\"ACL2____IFIX\">ifix</see>, see <see topic=\"ACL2____RFIX\">rfix</see>, and see <see topic=\"ACL2____FIX\">fix</see> for analogous functions that coerce \u000A to a natural number, an integer, a rational, and a number, respectively.</p> \u000A \u000A <p><v>Realfix</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>realfix</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> realfix (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x) x 0))</code>"],

 "ACL2____QUANTIFIERS-USING-RECURSION":[["Quantifiers"],"ACL2 Sources","ACL2","<p>The following example illustrates the use of recursion as a means \u000A of avoiding proof difficulties that can arise from the use of explicit \u000A quantification (via <tt><see topic=\"ACL2____DEFUN-SK\">defun-sk</see></tt>).  See <see topic=\"ACL2____QUANTIFIERS\">quantifiers</see> for more about \u000A the context of this example.</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____IN-PACKAGE\">in-package</see> &quot;ACL2&quot;)\u000A\u000A; We prove that if every member A of each of two lists satisfies the\u000A; predicate (P A), then this holds of their append; and, conversely.\u000A\u000A; Here is a solution using recursively-defined functions.\u000A\u000A(<see topic=\"ACL2____DEFSTUB\">defstub</see> p (x) t)\u000A\u000A(<see topic=\"COMMON-LISP____DEFUN\">defun</see> all-p (x)\u000A  (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> x)\u000A      t\u000A    (<see topic=\"COMMON-LISP____AND\">and</see> (p (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A         (all-p (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))))\u000A\u000A(<see topic=\"ACL2____DEFTHM\">defthm</see> all-p-append\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (all-p (<see topic=\"COMMON-LISP____APPEND\">append</see> x1 x2))\u000A         (<see topic=\"COMMON-LISP____AND\">and</see> (all-p x1) (all-p x2))))</code>"],

 "ACL2____PSOG":[["Other"],"ACL2 Sources","ACL2","<p>Evaluate <v>:psog</v> in order to print output in <see topic=\"ACL2____GAG-MODE\">gag-mode</see> \u000A that was generated in an environment where output was being saved; see <see topic=\"ACL2____SET-SAVED-OUTPUT\">set-saved-output</see> for details.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____PSO\">pso</see> and see <see topic=\"ACL2____PSO_12\">pso!</see> for printing the full output.</p>"],

 "ACL2____PROGN_42":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This macro expands to a corresponding nest of calls of <v>prog2$</v>; \u000A see <see topic=\"ACL2____PROG2_42\">prog2$</see>.  The examples below show how this works: the first case \u000A below is typical, but we conclude with two special cases.</p> \u000A \u000A <code>ACL2 !&gt;:trans1 (<see topic=\"ACL2____PROGN_42\">progn$</see> (f1 x) (f2 x) (f3 x))\u000A (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (F1 X) (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (F2 X) (F3 X)))\u000AACL2 !&gt;:trans1 (<see topic=\"ACL2____PROGN_42\">progn$</see> (f1 x) (f2 x))\u000A (<see topic=\"ACL2____PROG2_42\">PROG2$</see> (F1 X) (F2 X))\u000AACL2 !&gt;:trans1 (<see topic=\"ACL2____PROGN_42\">progn$</see> (f1 x))\u000A (F1 X)\u000AACL2 !&gt;:trans1 (<see topic=\"ACL2____PROGN_42\">progn$</see>)\u000A NIL\u000AACL2 !&gt;</code>"],

 "ACL2____Overview_02of_02the_02Proof_02of_02a_02Trivial_02Consequence":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02End_02of_02the_02Walking_02Tour'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code>ACL2 !&gt;<b>(defthm trivial-consequence</b> \u000A         <b>(equal (app (app (app (app x1 x2) (app x3 x4)) (app x5 x6)) x7)</b> \u000A                <b>(app x1 (app (app x2 x3) (app (app x4 x5) (app x6 x7))))))</b> \u000A \u000A<see topic='ACL2____The_02WARNING_02about_02the_02Trivial_02Consequence'>ACL2 Warning</see> [Subsume] in ( DEFTHM TRIVIAL-CONSEQUENCE ...):  The previously \u000Aadded rule ASSOCIATIVITY-OF-APP subsumes the newly proposed :REWRITE \u000Arule TRIVIAL-CONSEQUENCE, in the sense that the old rule rewrites a \u000Amore general target.  Because the new rule will be tried first, it \u000Amay nonetheless find application. \u000A \u000ABy the simple :rewrite rule <see topic='ACL2____The_02First_02Application_02of_02the_02Associativity_02Rule'>ASSOCIATIVITY-OF-APP</see> we reduce the conjecture \u000Ato \u000A \u000AGoal' \u000A(EQUAL (APP X1 \u000A            (APP X2 \u000A                 (APP X3 (APP X4 (APP X5 (APP X6 X7)))))) \u000A       (APP X1 \u000A            (APP X2 \u000A                 (APP X3 (APP X4 (APP X5 (APP X6 X7))))))). \u000A \u000ABut we reduce the conjecture to T, by primitive type reasoning. \u000A \u000AQ.E.D. \u000A \u000ASummary \u000AForm:  ( DEFTHM TRIVIAL-CONSEQUENCE ...) \u000ARules: ((:REWRITE ASSOCIATIVITY-OF-APP) \u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL)) \u000AWarnings:  <see topic='ACL2____The_02Summary_02of_02the_02Proof_02of_02the_02Trivial_02Consequence'>Subsume</see> \u000ATime:  0.20 seconds (prove: 0.02, print: 0.00, other: 0.18) \u000A TRIVIAL-CONSEQUENCE \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>You might explore the links before moving on.</p> \u000A \u000A <p><see topic='ACL2____The_02End_02of_02the_02Walking_02Tour'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____NOTE-3-4_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please also see <see topic=\"ACL2____NOTE-3-4\">note-3-4</see> for changes to Version 3.4 of \u000A  ACL2.</p> \u000A \u000A <p>Fixed makefiles, <v>books/nonstd/Makefile</v> and <v>GNUmakefile</v>.  The old \u000A set-up seemed to work fine as long as all books certified, but it was really \u000A broken, for example only certifying some of the books in \u000A <v>books/nonstd/nsa/</v>, and then only when required by books in other \u000A directories.  Also fixed the ``<v>clean</v>'' target to clean links rather than \u000A to make links.</p>"],

 "ACL2____MAKE":[["Defrec"],"ACL2 Sources","ACL2","<p>The <v>make</v> macro is built into ACL2, and allows you to construct \u000A new instances of structures that have been introduced with <see topic=\"ACL2____DEFREC\">defrec</see>.  For \u000A instance:</p> \u000A \u000A <code>(<see topic=\"ACL2____MAKE\">make</see> employee :name &quot;Jimmy&quot;\u000A               :salary 0\u000A               :position &quot;Unpaid Intern&quot;)</code> \u000A \u000A <p>Creates a new <v>employee</v> structure with the given values for its \u000A <v>name</v>, <v>salary</v>, and <v>position</v> fields.  See <see topic=\"ACL2____DEFREC\">defrec</see> for more \u000A information.</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2-ANSWER":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>This answer is in the form of a script sufficient to lead ACL2 to a \u000A proof.</p> \u000A \u000A <code>; Trying <v>subsetp-reflexive</v> at this point produces the key checkpoint: \u000A \u000A; (IMPLIES (AND (CONSP X) \u000A;               (SUBSETP (CDR X) (CDR X))) \u000A;          (SUBSETP (CDR X) X)) \u000A \u000A; which suggests the generalization: \u000A \u000A(defthm subsetp-cdr \u000A  (implies (subsetp a (cdr b)) \u000A           (subsetp a b))) \u000A \u000A; And now <v>subsetp-reflexive</v> succeeds. \u000A \u000A(defthm subsetp-reflexive \u000A  (subsetp x x)) \u000A \u000A; A weaker version of the lemma, namely the one in which we \u000A; add the hypothesis that <v>b</v> is a <v>cons</v>, is also sufficient. \u000A \u000A;    (defthm subsetp-cdr-weak \u000A;      (implies (and (consp b) \u000A;                    (subsetp a (cdr b))) \u000A;               (subsetp a b))) \u000A \u000A; But the <v>(<see topic=\"COMMON-LISP____CONSP\">consp</see> b)</v> hypothesis is not really necessary in \u000A; ACL2's type-free logic because <v>(<see topic=\"COMMON-LISP____CDR\">cdr</see> b)</v> is <v>nil</v> if <v>b</v> is \u000A; not a <v>cons</v>.  For the reasons explained in the tutorial, we \u000A; prefer the strong version. \u000A \u000A</code> \u000A \u000A <p>Use your browser's <b>Back Button</b> now to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2\">introductory-challenge-problem-2</see>.</p>"],

 "ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2":[["Introduction-to-the-theorem-prover"],"ACL2 Sources","ACL2","<p>Start in a fresh ACL2, either by restarting your ACL2 image from \u000A scratch or executing <v>:ubt! 1</v>.</p> \u000A \u000A <p>Use The Method to prove</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> subsetp-reflexive\u000A  (<see topic=\"COMMON-LISP____SUBSETP\">subsetp</see> x x))</code> \u000A \u000A <p>When you've solved this problem, compare your answer to ours; see <see topic=\"ACL2____INTRODUCTORY-CHALLENGE-PROBLEM-2-ANSWER\">introductory-challenge-problem-2-answer</see>.</p> \u000A \u000A <p>Then, use your browser's <b>Back Button</b> to return to <see topic=\"ACL2____INTRODUCTORY-CHALLENGES\">introductory-challenges</see>.</p>"],

 "ACL2____FMT1_12":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This function is nearly identical to <v>fmt1</v>; see <see topic=\"ACL2____FMT1\">fmt1</see>. \u000A The only difference is that <v>fmt1</v> may insert backslash (\\) characters \u000A when forced to print past the right margin in order to make the output a bit \u000A clearer in that case.  Use <v>fmt1!</v> instead if you want to be able to read \u000A the forms back in.</p>"],

 "ACL2____FMS_12":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This function is nearly identical to <v>fms</v>; see <see topic=\"ACL2____FMS\">fms</see>.  The \u000A only difference is that <v>fms</v> may insert backslash (\\) characters when \u000A forced to print past the right margin in order to make the output a bit \u000A clearer in that case.  Use <v>fms!</v> instead if you want to be able to read \u000A the forms back in.</p>"],

 "ACL2____Evaluating_02App_02on_02Sample_02Input":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <code>ACL2 !&gt;<b>(app nil '(x y z))</b> \u000A(X Y Z) \u000A \u000AACL2 !&gt;<b>(app '(1 2 3) '(4 5 6 7))</b> \u000A(1 2 3 4 5 6 7) \u000A \u000AACL2 !&gt;<b>(app '(a b c d e f g) '(x y z))</b>   ; click <see topic='ACL2____Conversion'>here</see> for an explanation \u000A(A B C D E F G X Y Z) \u000A \u000AACL2 !&gt;<b>(app (app '(1 2) '(3 4)) '(5 6))</b> \u000A(1 2 3 4 5 6) \u000A \u000AACL2 !&gt;<b>(app '(1 2) (app '(3 4) '(5 6)))</b> \u000A(1 2 3 4 5 6) \u000A \u000AACL2!&gt;<b>(let ((a '(1 2))</b> \u000A            <b>(b '(3 4))</b> \u000A            <b>(c '(5 6)))</b> \u000A        <b>(equal (app (app a b) c)</b> \u000A               <b>(app a (app b c))))</b> \u000AT \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>As we can see from these examples, ACL2 functions can be executed more or \u000A less as Common Lisp.</p> \u000A \u000A <p>The last three examples suggest an interesting property of <v>app</v>.</p> \u000A \u000A <p><see topic='ACL2____The_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____EVISCERATE-HIDE-TERMS":[["Io"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> eviscerate-hide-terms t)\u000A(<see topic=\"ACL2____ASSIGN\">assign</see> eviscerate-hide-terms nil)</code> \u000A \u000A <p><v>Eviscerate-hide-terms</v> is a <tt><see topic=\"ACL2____STATE\">state</see></tt> global variable whose value \u000A is either <v>t</v> or <v>nil</v>.  The variable affects how terms are displayed by \u000A default (but not if you have set the term-evisc-tuple to other than its \u000A default; see <see topic=\"ACL2____SET-EVISC-TUPLE\">set-evisc-tuple</see>).  If <v>t</v>, terms of the form <v>(<see topic=\"ACL2____HIDE\">hide</see>\u000A ...)</v>  are printed as <v>&lt;hidden&gt;</v>.  Otherwise, they are printed \u000A normally.</p>"],

 "ACL2____CONS-SUBTREES":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>cons-subtrees</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see>\u000A     cons-subtrees (x al)\u000A     (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A     (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) al)\u000A           ((<see topic=\"ACL2____HONS-GET\">hons-get</see> x al) al)\u000A           (t (<see topic=\"ACL2____CONS-SUBTREES\">cons-subtrees</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                             (<see topic=\"ACL2____CONS-SUBTREES\">cons-subtrees</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)\u000A                                            (<see topic=\"ACL2____HONS-ACONS\">hons-acons</see> x t al))))))</code>"],

 "ACL2____CLEAR-MEMOIZE-TABLE":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>This function returns its argument, <v>fn</v>, unchanged.  The values \u000A memoized for <v>fn</v> are forgotten.</p> \u000A \u000A <p><b>Function: </b><srclink>clear-memoize-table</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> clear-memoize-table (fn)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       fn)</code>"],

 "ACL2____CLEAR-MEMOIZE-STATISTICS":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Logically, this function just returns <v>nil</v>.  It clears all profiling \u000A info displayed by <v>(</v><tt><see topic=\"ACL2____MEMOIZE-SUMMARY\">memoize-summary</see></tt><v>)</v></p> \u000A \u000A <p><b>Function: </b><srclink>clear-memoize-statistics</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> clear-memoize-statistics\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____BINARY-_A2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-*</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> x y)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y)\u000A               (<see topic=\"ACL2____BINARY-_A2\">binary-*</see> x y)\u000A             0)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____BINARY-_A2\">binary-*</see> x y)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x) (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> y))</code> \u000A \u000A <p>Notice that like all arithmetic functions, <v>binary-*</v> treats non-numeric \u000A inputs as <v>0</v>.</p> \u000A \u000A <p>Calls of the macro <tt><see topic=\"COMMON-LISP_____A2\">*</see></tt> expand to calls of <v>binary-*</v>; see <see topic=\"COMMON-LISP_____A2\">*</see>.</p>"],

 "ACL2____ADD-NTH-ALIAS":[["Nth-aliases-table"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ADD-NTH-ALIAS\">add-nth-alias</see> st0 st)</code> \u000A \u000A <p>This example associates the symbol <v>st0</v> with the symbol <v>st</v> for \u000A purposes of printing certain terms of the form <v>(<see topic=\"COMMON-LISP____NTH\">nth</see> n st0)</v> and \u000A <v>(<see topic=\"ACL2____UPDATE-NTH\">update-nth</see> n val st0)</v>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____ADD-NTH-ALIAS\">add-nth-alias</see> alias-name name)</code> \u000A \u000A <p>This is a convenient way to add an entry to <tt><see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see></tt>.  See \u000A <see topic=\"ACL2____NTH-ALIASES-TABLE\">nth-aliases-table</see> and also see <see topic=\"ACL2____REMOVE-NTH-ALIAS\">remove-nth-alias</see>.</p>"],

 "ACL2-PC____SHOW-TYPE-PRESCRIPTIONS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Ashow-type-prescriptions\u000A\u000AGeneral Form:\u000A(show-type-prescriptions &amp;optional rule-id)</code> \u000A \u000A <p>Display <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see> rules that apply to the current subterm. \u000A If <v>rule-id</v> is supplied and is a name (non-<v>nil</v> symbol) or a \u000A <v>:</v><tt><see topic=\"ACL2____REWRITE\">rewrite</see></tt> or <v>:</v><tt><see topic=\"ACL2____DEFINITION\">definition</see></tt> <see topic=\"ACL2____RUNE\">rune</see>, then only the \u000A corresponding rewrite rule(s) will be displayed.  In each case, the display \u000A will point out when a rule is currently disabled (in the interactive \u000A environment).  Also see <see topic=\"ACL2____TYPE-PRESCRIPTION\">type-prescription</see>.</p>"],

 "ACL2-PC____PR":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Apr\u000A(<see topic=\"ACL2____PR\">pr</see> foo)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____PR\">pr</see> &amp;optional x)</code> \u000A \u000A <p>This command simply invokes the corresponding command of the top-level ACL2 \u000A loop; see <see topic=\"ACL2____PR\">pr</see>.  If no argument is given, or if the argument is <v>nil</v>, \u000A then the current subterm should be a call of a function symbol, and the \u000A argument is taken to be that symbol.</p> \u000A \u000A <p>If you want information about applying rewrite rules to the current \u000A subterm, consider the <v>show-rewrites</v> (or equivalently, <v>sr</v>) \u000A command.</p>"],

 "ACL2-PC____PL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Apl\u000A(<see topic=\"ACL2____PL\">pl</see> foo)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____PL\">pl</see> &amp;optional x)</code> \u000A \u000A <p>This command simply invokes the corresponding command of the top-level ACL2 \u000A loop; see <see topic=\"ACL2____PL\">pl</see>.  If no argument is given, or if the argument is <v>nil</v>, \u000A then the current subterm should be a call of a function symbol, and the \u000A argument is taken to be that symbol.</p> \u000A \u000A <p>If you want information about applying rewrite rules to the current \u000A subterm, consider the <v>show-rewrites</v> (or equivalently, <v>sr</v>) \u000A command.</p>"],

 "ACL2-PC____FINISH":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(finish induct prove bash)\u000A\u000AGeneral Form:\u000A(finish &amp;rest instructions)</code> \u000A \u000A <p>Run the indicated instructions, stopping at the first failure.  If there is \u000A any failure, or if any new goals are created and remain at the end of the \u000A indicated instructions, then consider the call of <v>finish</v> to be a failure. \u000A See <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see> and visit the documentation for \u000A <v>sequence</v> for a discussion of the notion of ``failure'' for proof-checker \u000A commands.</p>"],

 "ACL2-PC____CHANGE-GOAL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(change-goal (main . 1)) -- change to the goal (main . 1)\u000Achange-goal              -- change to the next-to-top goal\u000A\u000AGeneral Form:\u000A(change-goal &amp;optional goal-name end-flg)</code> \u000A \u000A <p>Change to the goal with the name <v>goal-name</v>, i.e. make it the current \u000A goal.  However, if <v>goal-name</v> is <v>nil</v> or is not supplied, then it \u000A defaults to the next-to-top goal, i.e., the second goal in the stack of goals. \u000A If <v>end-flg</v> is supplied and not <v>nil</v>, then move the current goal to \u000A the end of the goal stack; else merely swap it with the next-to-top goal. \u000A Also see documentation for <v>cg</v>.</p>"],

 "ACL2-PC____BOOKMARK":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(bookmark final-goal)\u000A\u000AGeneral Form:\u000A(bookmark name &amp;rest instruction-list)</code> \u000A \u000A <p>Run the instructions in <v>instruction-list</v> (as though this were a call \u000A of <v>do-all</v>; see the documentation for <v>do-all</v>), but first insert a \u000A begin bookend with the given name and then, when the instructions have been \u000A completed, insert an end bookend with that same name.  See the documentation \u000A of <v>comm</v> for an explanation of bookends and how they can affect the \u000A display of instructions.</p>"],

 "COMMON-LISP____NULL":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Null</v> is the function that checks whether its argument is \u000A <v>nil</v>.  For recursive definitions it is often preferable to test for the \u000A end of a list using <tt><see topic=\"COMMON-LISP____ENDP\">endp</see></tt> instead of <v>null</v>; see <see topic=\"COMMON-LISP____ENDP\">endp</see>.</p> \u000A \u000A <p><v>Null</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>null</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> null (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))</code>"],

 "COMMON-LISP____MIN":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____MIN\">Min</see> x y)</v> is the smaller of the numbers <v>x</v> and <v>y</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>min</v> requires its arguments to be rational (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) numbers.</p> \u000A \u000A <p><v>Min</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>min</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> min (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)\u000A                                   (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> y))))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x y) x y))</code>"],

 "COMMON-LISP____LOGNOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>lognor</v> returns the bitwise logical `nor' of the first with the \u000A second.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>lognor</v> requires its arguments to be integers. \u000A <v>Lognor</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>lognor</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> lognor (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> (<see topic=\"COMMON-LISP____LOGIOR\">logior</see> i j)))</code>"],

 "COMMON-LISP____CHAR_E3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR_E3\">char&gt;</see> x y)</v> is true if and only if the character code of \u000A <v>x</v> is greater than that of <v>y</v>.  See <see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char&gt;</v> specifies that its arguments are <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><v>Char&gt;</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char&gt;</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char&gt; (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y))))\u000A       (<see topic=\"COMMON-LISP_____E3\">&gt;</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x) (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> y)))</code>"],

 "COMMON-LISP____CHAR_C3_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHAR_C3_D3\">char&lt;=</see> x y)</v> is true if and only if the character code of \u000A <v>x</v> is less than or equal to that of <v>y</v>.  See <see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>char&lt;=</v> specifies that its arguments are <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><v>Char&lt;=</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>char&lt;=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> char&lt;= (x y)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x) (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> y))))\u000A       (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> x) (<see topic=\"COMMON-LISP____CHAR-CODE\">char-code</see> y)))</code>"],

 "COMMON-LISP____1-":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____1-\">1-</see> x)</v> is the same as <v>(<see topic=\"COMMON-LISP____-\">-</see> x 1)</v>.  See <see topic=\"COMMON-LISP____-\">-</see>.</p> \u000A \u000A <p><v>1-</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>1-</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> 1- (x) (<see topic=\"COMMON-LISP____LIST\">list</see> '- x 1))</code>"],

 "ACL2____ZPF":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Zpf</v> is exactly the same as <tt><see topic=\"ACL2____ZP\">zp</see></tt>, except that <v>zpf</v> is \u000A intended for, and faster for, fixnum arguments.  Its guard is specified with a \u000A type declaration, <v>(<see topic=\"COMMON-LISP____TYPE\">type</see> (unsigned-byte 29) x)</v>.  (See <see topic=\"COMMON-LISP____DECLARE\">declare</see> and \u000A see <see topic=\"ACL2____TYPE-SPEC\">type-spec</see>.)  Also see <see topic=\"ACL2____ZP\">zp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>zpf</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> zpf (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"COMMON-LISP____TYPE\">type</see> (unsigned-byte 29) x))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x) (<see topic=\"COMMON-LISP_____C3_D3\">&lt;=</see> x 0) t))</code>"],

 "ACL2____UNARY-_F2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-unary-/</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A                (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)))\u000A           (<see topic=\"ACL2____UNARY-_F2\">unary-/</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"ACL2____UNARY-_F2\">unary-/</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A     (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x 0)))</code> \u000A \u000A <p>Notice that like all arithmetic functions, <v>unary-/</v> treats non-numeric \u000A inputs as <v>0</v>.</p> \u000A \u000A <p>Calls of the macro <tt><see topic=\"COMMON-LISP_____F2\">/</see></tt> on one argument expand to calls of \u000A <v>unary-/</v>; see <see topic=\"COMMON-LISP_____F2\">/</see>.</p>"],

 "ACL2____UBT-PREHISTORY":[["History"],"ACL2 Sources","ACL2","<p>This command is only used to eliminate a <tt><see topic=\"ACL2____RESET-PREHISTORY\">reset-prehistory</see></tt> \u000A event.  If your most recent <v>reset-prehistory</v> event does not have a flag \u000A argument of <v>t</v>, then <v>:ubt-prehistory</v> undoes all command back through, \u000A and including, that <v>reset-prehistory</v> event.</p>"],

 "ACL2____The_02Induction_02Step_02in_02the_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>This formula is the <b>Induction Step</b>.  It basically consists \u000A of three parts, a test identifying the inductive case, an induction hypothesis \u000A and an induction conclusion.</p> \u000A \u000A <code>(IMPLIES (AND (NOT (ENDP A))      <b>; Test</b> \u000A              (:P (CDR A) B C))   <b>; Induction Hypothesis</b> \u000A         (:P A B C))              <b>; Induction Conclusion</b> \u000A</code> \u000A \u000A <p>When we prove this we can assume</p> \u000A \u000A <code> * <v>A</v> is not empty, and that \u000A \u000A  * the associativity conjecture holds for a ``smaller'' version of <v>A</v>, \u000A    namely, <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> A)</v>. \u000A \u000A</code> \u000A \u000A <p>Under those hypotheses we have to prove the associativity conjecture for \u000A <v>A</v> itself.</p>"],

 "ACL2____TOP":[[],"[books]/system/doc/acl2-manual.lisp","ACL2","<p>This manual is generated from ACL2 documentation only; it excludes \u000A  documentation from the community books.</p>"],

 "ACL2____STANDARD-CHAR-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> x)</v> is true if and only if <v>x</v> is a \u000A null-terminated list all of whose members are standard <see topic=\"ACL2____CHARACTERS\">characters</see>.  See \u000A <see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see>.</p> \u000A \u000A <p><v>Standard-char-listp</v> has a <see topic=\"ACL2____GUARD\">guard</see> of <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>standard-char-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> standard-char-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____CONSP\">consp</see> l)\u000A              (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                   (<see topic=\"COMMON-LISP____STANDARD-CHAR-P\">standard-char-p</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                   (<see topic=\"ACL2____STANDARD-CHAR-LISTP\">standard-char-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))\u000A             (t (<see topic=\"COMMON-LISP____EQUAL\">equal</see> l nil))))</code>"],

 "ACL2____SET-RAW-MODE-ON_12":[["Defttag"],"ACL2 Sources","ACL2","<p>This is the same as <v>(</v><tt><see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see></tt><v> t)</v> except that \u000A it first introduces a so-called ``trust tag'' (``ttag'') so that \u000A <v>set-raw-mode</v> will be legal.  See <see topic=\"ACL2____DEFTTAG\">defttag</see> for a discussion of ttags \u000A and how they affect <tt><see topic=\"ACL2____CERTIFY-BOOK\">certify-book</see></tt> and <tt><see topic=\"ACL2____INCLUDE-BOOK\">include-book</see></tt>.</p> \u000A \u000A <p>See <see topic=\"ACL2____SET-RAW-MODE\">set-raw-mode</see> for a discussion of raw-mode.</p>"],

 "ACL2____REMOVE-CUSTOM-KEYWORD-HINT":[["Events"],"ACL2 Sources","ACL2","<code>Example Forms:\u000A(<see topic=\"ACL2____REMOVE-CUSTOM-KEYWORD-HINT\">remove-custom-keyword-hint</see> :my-hint)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____REMOVE-CUSTOM-KEYWORD-HINT\">remove-custom-keyword-hint</see> keyword)</code> \u000A \u000A <p>where <v>keyword</v> is a <tt><see topic=\"COMMON-LISP____KEYWORDP\">keywordp</see></tt>.</p> \u000A \u000A <p>For an explanation of how custom keyword hints are processed, see <see topic=\"ACL2____CUSTOM-KEYWORD-HINTS\">custom-keyword-hints</see>; also see <see topic=\"ACL2____ADD-CUSTOM-KEYWORD-HINT\">add-custom-keyword-hint</see>.</p> \u000A \u000A <p>Note: This is an event!  It does not print the usual event summary but \u000A nevertheless changes the ACL2 logical <see topic=\"ACL2____WORLD\">world</see> and is so recorded.</p>"],

 "ACL2____PUTPROP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>General form:\u000A(<see topic=\"ACL2____PUTPROP\">putprop</see> symbol key value world-alist)</code> \u000A \u000A <p>See community book <v>books/misc/getprop.lisp</v> for an example that \u000A illustrates the use of ACL2 utilities <tt><see topic=\"ACL2____GETPROP\">getprop</see></tt> and <v>putprop</v> to take \u000A advantage of under-the-hood Lisp (hashed) property lists.</p> \u000A \u000A <p><b>Function: </b><srclink>putprop</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> putprop (symb key value world-alist)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> symb)\u000A                                   (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> key)\u000A                                   (plist-worldp world-alist))))\u000A       (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CONS\">cons</see> symb (<see topic=\"COMMON-LISP____CONS\">cons</see> key value))\u000A             world-alist))</code>"],

 "ACL2____O-FINP":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p>We introduce the function <v>o-finp</v> which returns <v>t</v> for any \u000A ordinal that is finite, else <v>nil</v>.  This function is equivalent to the \u000A function <tt><see topic=\"COMMON-LISP____ATOM\">atom</see></tt>, and is introduced so that we can <tt><see topic=\"ACL2____DISABLE\">disable</see></tt> its \u000A definition when dealing with ordinals (also see <see topic=\"ACL2____MAKE-ORD\">make-ord</see>).</p> \u000A \u000A <p><b>Function: </b><srclink>o-finp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> o-finp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____ATOM\">atom</see> x))</code>"],

 "ACL2____Modeling_02in_02ACL2":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Running_02Models'><img \u000A  src='flying.gif'></img></see></p> \u000A \u000A <p><img src='computing-machine-a.gif'></img></p> \u000A \u000A <p>Below we define <b>mc(s,n)</b> to be the function that <b>single-step</b>s \u000A <b>n</b> times from a given starting state, <b>s</b>.  In Common Lisp, \u000A ``mc(s,n)'' is written <v>(mc s n)</v>.</p> \u000A \u000A <code><b>(defun mc (s n)</b>                     ; To step <b>s</b> <b>n</b> times: \u000A <b>(if (zp n)</b>                         ; If <b>n</b> is 0 \u000A     <b>s</b>                              ;    then return <b>s</b> \u000A     <b>(mc (single-step s) (- n 1))))</b> ;    else step <b>single-step(s)</b> \u000A                                                      <b>n-1</b> times. \u000A</code> \u000A \u000A <p>This is an example of a formal model in ACL2.</p> \u000A \u000A <p><see topic='ACL2____Running_02Models'><img \u000A src='flying.gif'></img></see></p>"],

 "ACL2____MAKE-WORMHOLE-STATUS":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____MAKE-WORMHOLE-STATUS\">make-wormhole-status</see> whs code data)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  <v>Whs</v> should be a well-formed wormhole status, \u000A <v>code</v> should be <v>:ENTER</v> or <v>:SKIP</v>, and <v>data</v> is arbitrary. \u000A This function returns a new status with the specified entry code and data, \u000A reusing <v>whs</v> if it is appropriate.</p>"],

 "ACL2____GOOD-ATOM-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>good-atom-listp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of ``good'' <see topic=\"COMMON-LISP____ATOM\">atom</see>s, i.e., where each element is a \u000A number, a symbol, a character, or a string.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____ATOM-LISTP\">atom-listp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>good-atom-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> good-atom-listp (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) (<see topic=\"COMMON-LISP____EQ\">eq</see> lst nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                         (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                         (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                         (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst)))\u000A                     (<see topic=\"ACL2____GOOD-ATOM-LISTP\">good-atom-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code>"],

 "ACL2____EQLABLE-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>eqlable-alistp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> objects whose <tt><see topic=\"COMMON-LISP____CAR\">car</see></tt>s all satisfy \u000A <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>eqlable-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> eqlable-alistp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                     (<see topic=\"ACL2____EQLABLE-ALISTP\">eqlable-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____DEFINE-PC-META":[["Proof-checker"],"ACL2 Sources","ACL2","<p>Built-in proof-checker meta commands include <v>undo</v> and \u000A <v>restore</v>, and others (<v>lisp</v>, <v>exit</v>, and <v>sequence</v>); see <see topic=\"ACL2____PROOF-CHECKER-COMMANDS\">proof-checker-commands</see>.  The advanced proof-checker user can define these as \u000A well.  See ACL2 source file <v>proof-checker-b.lisp</v> for examples, and \u000A contact the ACL2 implementors if those examples do not provide sufficient \u000A documentation.</p>"],

 "ACL2____COMMAND-LINE":[["Interfacing-tools"],"ACL2 Sources","ACL2","<p>You may provide command-line arguments when invoking ACL2, which \u000A are passed to the host Lisp.  For more information on this topic, along with a \u000A discussion of how to save an ACL2 executable that avoids passing command-line \u000A arguments to the host Lisp, see <see topic=\"ACL2____SAVE-EXEC\">save-exec</see>.</p>"],

 "ACL2____ACL2-HELP":[["About-ACL2"],"ACL2 Sources","ACL2","<p>You can email questions about ACL2 usage to the acl2-help mailing \u000A list: <v>acl2-help@utlists.utexas.edu</v>.  If you have more general questions \u000A about ACL2, for example, about projects completed using ACL2, you may prefer \u000A the acl2 mailing list, <v>acl2@utlists.utexas.edu</v>, which tends to have wider \u000A distribution.</p>"],

 "ACL2-PC____USE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____USE\">USE</see> true-listp-append\u000A     (:instance assoc-of-append (x a) (y b) (z c)))\u000A-- Add two top-level hypotheses, one the lemma called\u000A   true-listp-append, and the other an instance of the lemma called\u000A   assoc-of-append by the substitution in which x is assigned a, y\u000A   is assigned b, and z is assigned c.\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____USE\">use</see> &amp;rest args)</code> \u000A \u000A <p>Add the given lemma instances to the list of top-level hypotheses.  See \u000A <see topic=\"ACL2____HINTS\">hints</see> for the syntax of <v>:use</v> hints in <v>defthm</v>, which is \u000A essentially the same as the syntax here (see the example above).</p> \u000A \u000A <p>This command calls the <v>prove</v> command, and hence should only be used at \u000A the top of the conclusion.</p>"],

 "ACL2-PC____THEN":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(then induct prove)\u000A\u000AGeneral Form:\u000A(then first-instruction &amp;optional completion must-succeed-flg)</code> \u000A \u000A <p>Run <v>first-instruction</v>, and then run <v>completion</v> (another \u000A instruction) on each subgoal created by <v>first-instruction</v>.  If \u000A <v>must-succeed-flg</v> is supplied and not <v>nil</v>, then halt at the first \u000A ``failure'' and remove the effects of the invocation of <v>completion</v> that \u000A ``failed''.</p> \u000A \u000A <p>The default for completion is <v>reduce</v>.</p>"],

 "ACL2-PC____SHOW-LINEARS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Ashow-linears\u000A\u000AGeneral Form:\u000A(show-linears &amp;optional rule-id enabled-only-flg)</code> \u000A \u000A <p>This command displays <see topic=\"ACL2____LINEAR\">linear</see> rules with a trigger term that matches \u000A the current subterm, and shows how they can be applied.  This command is \u000A analogous to the <v>show-rewrites</v> <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> command; see its \u000A <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for details.  Also see the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> for \u000A proof-checker command <v>apply-linear</v> for how to apply linear rules.</p>"],

 "ACL2-PC____S-PROP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000As-prop\u000A\u000AGeneral Form:\u000A(s-prop &amp;rest names)</code> \u000A \u000A <p>Simplify, using the default settings for <v>s</v> (which include \u000A if-normalization and rewriting without real backchaining), but with respect to \u000A a theory in which only basic functions and rules (the ones in <v>(<see topic=\"ACL2____THEORY\">theory</see>\u000A 'minimal-theory)</v>), together with the names (or parenthesized names) in the \u000A <v>&amp;rest</v> argument <v>names</v>, are enabled.</p> \u000A \u000A <p>See also the documentation for <v>s</v>.</p>"],

 "ACL2-PC____COMMANDS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Acommands\u000A(commands 10 t)\u000A\u000AGeneral Forms:\u000A\u000Acommands or (commands nil)\u000APrint out all the instructions (in the current state-stack) in\u000Areverse order, i.e. from the most recent instruction to the starting\u000Ainstruction.\u000A\u000A(commands n) [n a positive integer]\u000APrint out the most recent n instructions (in the current\u000Astate-stack), in reverse order.\u000A\u000A(commands x abbreviate-flag)\u000ASame as above, but if abbreviate-flag is non-NIL, then do not\u000Adisplay commands between ``matching bookends''.  See documentation\u000Afor comm for an explanation of matching bookends.</code> \u000A \u000A <p><b>Remark</b>: If there are more than <v>n</v> instructions in the \u000A state-stack, then <v>(commands n)</v> is the same as <v>commands</v> (and also, \u000A <v>(commands n abb)</v> is the same as <v>(commands nil abb)</v>).</p>"],

 "COMMON-LISP____ODDP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____ODDP\">oddp</see> x)</v> is true if and only if <v>x</v> is odd, i.e., not even \u000A in the sense of <tt><see topic=\"COMMON-LISP____EVENP\">evenp</see></tt>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>oddp</v> requires its argument to be an integer.</p> \u000A \u000A <p><v>Oddp</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>oddp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> oddp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> x)))\u000A       (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"COMMON-LISP____EVENP\">evenp</see> x)))</code>"],

 "COMMON-LISP____MINUSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____MINUSP\">Minusp</see> x)</v> is true if and only if <v>x &lt; 0</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of <v>minusp</v> requires its argument to be a rational \u000A (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) number.</p> \u000A \u000A <p><v>Minusp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>minusp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> minusp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)))\u000A       (<see topic=\"COMMON-LISP_____C3\">&lt;</see> x 0))</code>"],

 "COMMON-LISP____LOGNAND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>When integers are viewed in their two's complement representation, \u000A <v>lognand</v> returns their bitwise logical `nand'.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> for <v>lognand</v> requires its arguments to be integers. \u000A <v>Lognand</v> is defined in Common Lisp.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Function: </b><srclink>lognand</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> lognand (i j)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i) (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j))))\u000A       (<see topic=\"COMMON-LISP____LOGNOT\">lognot</see> (<see topic=\"COMMON-LISP____LOGAND\">logand</see> i j)))</code>"],

 "COMMON-LISP____LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____LISTP\">listp</see> x)</v> is true when <v>x</v> is either a <tt><see topic=\"COMMON-LISP____CONS\">cons</see></tt> pair or \u000A is <v>nil</v>.</p> \u000A \u000A <p><v>Listp</v> has no <see topic=\"ACL2____GUARD\">guard</see>, i.e., its <see topic=\"ACL2____GUARD\">guard</see> is <v>t</v>.</p> \u000A \u000A <p><v>Listp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> listp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil)))</code>"],

 "COMMON-LISP____1_B2":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____1_B2\">1+</see> x)</v> is the same as <v>(<see topic=\"COMMON-LISP_____B2\">+</see> 1 x)</v>.  See <see topic=\"COMMON-LISP_____B2\">+</see>.</p> \u000A \u000A <p><v>1+</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>1+</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> 1+ (x) (<see topic=\"COMMON-LISP____LIST\">list</see> '+ 1 x))</code>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_029_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0210_92'>(</see>EQUAL (CONS (CAR A) \u000A                      (APP (APP (CDR A) B) \u000A                           C)) \u000A                <b>(CONS (CAR A)</b> \u000A                      <b>(APP (CDR A) (APP B C))</b>))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom that <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> x y) (<see topic=\"COMMON-LISP____CONS\">CONS</see>\u000A u v))</v> is equal to the conjunction of <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> x u)</v> and <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> y v)</v>. \u000A In this case, <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> x u)</v> is trivial, <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> (<see topic=\"COMMON-LISP____CAR\">CAR</see> A) (<see topic=\"COMMON-LISP____CAR\">CAR</see> A))</v>.</p>"],

 "ACL2____The_02Induction_02Scheme_02Selected_02for_02the_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>(<see topic=\"COMMON-LISP____AND\">AND</see>\u000A   (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____AND\">AND</see> (<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> A))         ; Induction Step: test\u000A                 (:P (<see topic=\"COMMON-LISP____CDR\">CDR</see> A) B C))      ;  and induction hypothesis\u000A            (:P A B C))                 ;  implies induction conclusion.\u000A   (<see topic=\"ACL2____IMPLIES\">IMPLIES</see> (<see topic=\"COMMON-LISP____ENDP\">ENDP</see> A) (:P A B C)))       ; Base Case</code> \u000A \u000A <p>The formula beginning with this parenthesis is the induction scheme \u000A suggested by <v>(APP A B)</v> applied to <v>(P A B C)</v>.</p> \u000A \u000A <p>It is a <b>conjunction</b> (<tt><see topic=\"COMMON-LISP____AND\">and</see></tt> <see \u000A topic='ACL2____A_02Tiny_02Warning_02Sign'><icon src='twarning.gif'/></see>) of \u000A two formulas.</p> \u000A \u000A <p>The first is the <b>induction step</b> and the second is the <b>base \u000A case</b>.</p>"],

 "ACL2____The_02First_02Application_02of_02the_02Associativity_02Rule":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>So here we see our associativity rule being used!</p> \u000A \u000A <p>The rewriter sweeps the conjecture in a <b>leftmost innermost</b> fashion, \u000A applying rewrite rules as it goes.</p> \u000A \u000A <p>The associativity rule is used many times in this sweep.  The first \u000A ``target'' is highlighted below.  Click on it to see what happens:</p> \u000A \u000A <code><b>Current Conjecture</b>: \u000A(equal (app (app <see topic='ACL2____A_02Sketch_02of_02How_02the_02Rewriter_02Works'>(app (app x1 x2) (app x3 x4))</see> (app x5 x6)) x7) \u000A       (app x1 (app (app x2 x3) (app (app x4 x5) (app x6 x7))))) \u000A</code>"],

 "ACL2____SHOW-ACCUMULATED-PERSISTENCE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SERIALIZE-ALTERNATIVES":[["Serialize"],"ACL2 Sources","ACL2","<p><see topic=\"ACL2____HONS\">Hons</see> users could previously use the routines \u000A <v>compact-print-file</v> and <v>compact-read-file</v>, which were once built into \u000A ACL2.  Another option was to use the <v>hons-archive</v> library.  These tools \u000A were deprecated and ultimately removed from ACL2.  If for some reason you need \u000A to use them, see <see topic=\"ACL2____COMMUNITY-BOOKS\">community-books</see> SVN revisions prior to 1314.</p>"],

 "ACL2____REMOVE-OVERRIDE-HINTS_12":[["Override-hints"],"ACL2 Sources","ACL2","<p><v>Remove-override-hints!</v> is the same as <tt><see topic=\"ACL2____REMOVE-OVERRIDE-HINTS\">remove-override-hints</see></tt>, except that the former is not <see topic=\"ACL2____LOCAL\">local</see> to <see topic=\"ACL2____BOOKS\">books</see> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs.  See <see topic=\"ACL2____REMOVE-OVERRIDE-HINTS\">remove-override-hints</see>; also see <see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see> and see <see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see>.</p>"],

 "ACL2____REMOVE-DIVE-INTO-MACRO":[["Dive-into-macros-table"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____REMOVE-DIVE-INTO-MACRO\">remove-dive-into-macro</see> logand)</code> \u000A \u000A <p>This feature undoes the effect of <tt><see topic=\"ACL2____ADD-DIVE-INTO-MACRO\">add-dive-into-macro</see></tt>, which is \u000A used so that the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>DV</v> command and numeric diving \u000A commands (e.g., <v>3</v>) will dive properly into subterms.  Please see <see topic=\"ACL2____ADD-DIVE-INTO-MACRO\">add-dive-into-macro</see> and especially see <see topic=\"ACL2____DIVE-INTO-MACROS-TABLE\">dive-into-macros-table</see>.</p>"],

 "ACL2____Overview_02of_02the_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>In this message the system is saying that <v>Subgoal *1/2</v> has \u000A been rewritten to the <v>Subgoal *1/2'</v>, by expanding the definition of \u000A <b>endp</b>.  This is an example of <b>simplification</b>, one of the main \u000A proof techniques used by the theorem prover.</p> \u000A \u000A <p>Click <see topic='ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_020_92'>here</see> if you \u000A would like to step through the simplification of <v>Subgoal *1/2</v>.</p>"],

 "ACL2____NOTE-3-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>NOTE!  New users can ignore these release notes, because the \u000A documentation has been updated to reflect all changes that are recorded \u000A here.</p> \u000A \u000A <p>Please see <see topic=\"ACL2____NOTE-3-0-2\">note-3-0-2</see> for a description of changes since Version \u000A 3.0.1, and also see <see topic=\"ACL2____NOTE-3-0-1\">note-3-0-1</see> for additional changes since Version \u000A 3.0.</p>"],

 "ACL2____NOTE-2-8_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>The <v>Makefile</v> has been modified by adding a new target, \u000A <v>clean-links</v>.  This can be used in order to remove all soft links, which \u000A is useful if the directory is copied or moved to a new location or if there \u000A are file system changes that cause problems with link pathnames.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-2-8\">note-2-8</see> for changes to Version_2.8 of ACL2.</p>"],

 "ACL2____KWOTE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>For any object <v>x</v>, <v>(<see topic=\"ACL2____KWOTE\">kwote</see> x)</v> returns the two-element list \u000A whose elements are the symbol <v>quote</v> and the given <v>x</v>, respectively. \u000A The guard of <v>(<see topic=\"ACL2____KWOTE\">kwote</see> x)</v> is <v>t</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>kwote</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> kwote (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"ACL2____MBE\">mbe</see> :logic (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote x)\u000A            :exec (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____EQ\">eq</see> x nil) *nil*)\u000A                        ((<see topic=\"COMMON-LISP____EQ\">eq</see> x t) *t*)\u000A                        ((<see topic=\"COMMON-LISP____EQL\">eql</see> x 0) *0*)\u000A                        ((<see topic=\"COMMON-LISP____EQL\">eql</see> x 1) *1*)\u000A                        ((<see topic=\"COMMON-LISP____EQL\">eql</see> x -1) *-1*)\u000A                        (t (<see topic=\"COMMON-LISP____LIST\">list</see> 'quote x)))))</code>"],

 "ACL2____INT_D3":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____INT_D3\">int=</see> x y)</v> is logically equivalent to <v>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> x y)</v>.</p> \u000A \u000A <p>Unlike <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt>, <v>int=</v> requires its arguments to be numbers (or \u000A else causes a <see topic=\"ACL2____GUARD\">guard</see> violation; see <see topic=\"ACL2____GUARD\">guard</see>).  Generally, <v>int=</v> \u000A is executed more efficiently than <tt><see topic=\"COMMON-LISP____EQUAL\">equal</see></tt> or <tt><see topic=\"COMMON-LISP_____D3\">=</see></tt> on \u000A integers.</p> \u000A \u000A <p><b>Macro: </b><srclink>int=</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> int= (i j)\u000A          (<see topic=\"COMMON-LISP____LIST\">list</see> 'eql\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> i)\u000A                    i (<see topic=\"COMMON-LISP____LIST\">list</see> 'the 'integer i))\u000A                (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> j)\u000A                    j (<see topic=\"COMMON-LISP____LIST\">list</see> 'the 'integer j))))</code>"],

 "ACL2____INCOMPATIBLE":[["Theories"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see> (<see topic=\"ACL2____INCOMPATIBLE\">incompatible</see> (:rewrite left-to-right)\u000A                                (:rewrite right-to-left)))\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____INCOMPATIBLE\">incompatible</see> rune1 rune2)</code> \u000A \u000A <p>where <v>rune1</v> and <v>rune2</v> are two specific <see topic=\"ACL2____RUNE\">rune</see>s.  The \u000A arguments are not evaluated.  <v>Invariant</v> is just a macro that expands into \u000A a term that checks that not both <see topic=\"ACL2____RUNE\">rune</see>s are enabled.  See <see topic=\"ACL2____THEORY-INVARIANT\">theory-invariant</see>.</p>"],

 "ACL2____Flawed_02Induction_02Candidates_02in_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Induction on <b>a</b> is unflawed: every occurrence of <b>a</b> in \u000A the conjecture</p> \u000A \u000A <code>(equal (app (app <b>a</b> b) c) \u000A       (app <b>a</b> (app b c))) \u000A</code> \u000A \u000A <p>is in a position being recursively decomposed!</p> \u000A \u000A <p>Now look at the occurrences of <v>b</v>.  The first (shown in <b>bold</b> \u000A below) is in a position that is held constant in the recursion of <v>(app a\u000A b)</v>.  It would be ``bad'' to induct on <v>b</v> here.</p> \u000A \u000A <code>(equal (app (app a <b>b</b>) c) \u000A       (app a (app b c))) \u000A</code>"],

 "ACL2____FMT1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____FMT\">fmt</see> for further explanation, including documentation of \u000A the tilde-directives.</p>"],

 "ACL2____FMS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____FMT\">fmt</see> for further explanation, including documentation of \u000A the tilde-directives.</p>"],

 "ACL2____CW_12":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>This is the same as <tt><see topic=\"ACL2____CW\">cw</see></tt>, except that <tt><see topic=\"ACL2____CW\">cw</see></tt> inserts \u000A backslash (\\) characters when forced to print past the right margin, in order \u000A to make the output a bit clearer in that case.  Use <v>cw!</v> instead if you \u000A want to be able to read the forms back in.</p>"],

 "ACL2____CLEAR-HASH-TABLES":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Deprecated.  Calls <tt><see topic=\"ACL2____CLEAR-MEMOIZE-TABLES\">clear-memoize-tables</see></tt> and then <tt><see topic=\"ACL2____HONS-CLEAR\">hons-clear</see></tt> \u000A or <tt><see topic=\"ACL2____HONS-WASH\">hons-wash</see></tt>, whichever makes sense for the underlying Common \u000A Lisp.</p> \u000A \u000A <p><b>Function: </b><srclink>clear-hash-tables</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> clear-hash-tables\u000A       nil (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       nil)</code>"],

 "ACL2____BACKCHAINING":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>When the theorem prover attempts to apply a rule (e.g., a <see topic=\"ACL2____REWRITE\">rewrite</see> rule), it must relieve (prove) the hypotheses of that rule.  In the \u000A  ACL2 community, this process of relieving hypotheses is called \u000A  backchaining.</p> \u000A \u000A  <p>There is no such thing as a backchaining or backward-chaining rule \u000A  class (see <see topic=\"ACL2____RULE-CLASSES\">rule-classes</see>) in ACL2.</p>"],

 "ACL2____ASSOC-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ADD-DIVE-INTO-MACRO":[["Dive-into-macros-table"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____ADD-DIVE-INTO-MACRO\">add-dive-into-macro</see> cat expand-address-cat)</code> \u000A \u000A <p>This feature is used so that the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see>'s <v>DV</v> command and \u000A numeric diving commands (e.g., <v>3</v>) will dive properly into subterms. \u000A Please see <see topic=\"ACL2____DIVE-INTO-MACROS-TABLE\">dive-into-macros-table</see>.</p>"],

 "ACL2____ACL2_02as_02an_02Interactive_02Theorem_02Prover":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The ACL2 theorem prover finds proofs in the ACL2 logic.  It can be \u000A automatic.  But most often the user must help it.</p> \u000A \u000A <p><img src='interactive-theorem-prover.gif'></img></p> \u000A \u000A <p>The user usually guides ACL2 by suggesting that it first prove key \u000A <b>lemmas</b>.  Lemmas are just theorems used in the proofs of other \u000A theorems.</p>"],

 "ACL2-PC____UP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:  if the conclusion is (<see topic=\"COMMON-LISP_____D3\">=</see> x (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z)) and the\u000A           current subterm is y, then we have:\u000Aup or (up 1) -- the current subterm becomes (<see topic=\"COMMON-LISP____-\">-</see> y)\u000A(up 2)       -- the current subterm becomes (<see topic=\"COMMON-LISP_____A2\">*</see> (<see topic=\"COMMON-LISP____-\">-</see> y) z)\u000A(up 3)       -- the current subterm becomes the entire conclusion\u000A(up 4)       -- no change; can't go up that many levels\u000A\u000AGeneral Form:\u000A(up &amp;optional n)</code> \u000A \u000A <p>Move up <v>n</v> levels in the conclusion from the current subterm, where \u000A <v>n</v> is a positive integer.  If <v>n</v> is not supplied or is <v>nil</v>, then \u000A move up 1 level, i.e., treat the instruction as <v>(up 1)</v>.</p> \u000A \u000A <p>See also <v>dive</v>, <v>top</v>, <v>nx</v>, and <v>bk</v>.</p>"],

 "ACL2-PC____RUNES":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples and general forms: \u000A(runes t)   ; print all <see topic=\"ACL2____RUNE\">rune</see>s used during this interactive proof \u000A(runes nil) ; print all <see topic=\"ACL2____RUNE\">rune</see>s used by the most recent command \u000A(runes)     ; same as (runes nil) \u000Arunes       ; same as (runes nil) \u000A</code> \u000A \u000A <p>This command does not change the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> state.  Rather, it \u000A simply reports runes (see <see topic=\"ACL2____RUNE\">rune</see>) that have participated in the \u000A interactive proof.</p> \u000A \u000A <p>Note that <v>(runes nil)</v> will show the <see topic=\"ACL2____RUNE\">rune</see>s used by the most \u000A recent primitive or macro command (as displayed by <v>:comm</v>).</p>"],

 "ACL2-PC____PROTECT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(protect induct p prove)\u000A\u000AGeneral Form:\u000A(protect &amp;rest instruction-list)</code> \u000A \u000A <p><v>Protect</v> is the same as <v>do-strict</v>, except that as soon as an \u000A instruction ``fails'', the state-stack reverts to what it was before the \u000A <v>protect</v> instruction began, and <v>restore</v> is given the same meaning \u000A that it had before the <v>protect</v> instruction began.  See the documentation \u000A for <v>do-strict</v>.</p>"],

 "ACL2-PC____ORELSE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(orelse top (print &quot;Couldn't move to the top&quot;))\u000A\u000AGeneral form:\u000A(orelse instr1 instr2)</code> \u000A \u000A <p>Run the first instruction.  Then if it ``fails'', run the second \u000A instruction also; otherwise, stop after the first.</p> \u000A \u000A <p>This instruction ``succeeds'' if and only if either <v>instr1</v> \u000A ``succeeds'', or else <v>instr2</v> ``succeeds''.  If it ``fails'', then the \u000A failure is soft.</p>"],

 "ACL2-PC____FAIL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000Afail\u000A(fail t)\u000A\u000AGeneral Form:\u000A(fail &amp;optional hard)</code> \u000A \u000A <p>This is probably only of interest to writers of macro commands.  The only \u000A function of <v>fail</v> is to fail to ``succeed''.</p> \u000A \u000A <p>The full story is that <v>fail</v> and <v>(fail nil)</v> simply return <v>(<see topic=\"ACL2____MV\">mv</see>\u000A nil nil state)</v>, while <v>(fail hard)</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> hard nil state)</v> if \u000A <v>hard</v> is not <v>nil</v>.  See also <v>do-strict</v>, <v>do-all</v>, and \u000A <v>sequence</v>.</p>"],

 "ACL2-PC____CLAUSE-PROCESSOR":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example: \u000A(cl-proc :function \u000A         note-fact-clause-processor \u000A        :hint '(equal a a)) -- Invoke the indicated clause processor function \u000A        with the indicated hint argument (see the beginning of community book \u000A        <v>books/clause-processors/basic-examples.lisp</v>. \u000A \u000AGeneral Form: \u000A(cl-proc &amp;rest cl-proc-args) \u000A</code> \u000A \u000A <p>Invoke a clause-processor as indicated by cl-proc-args, which is a list of \u000A arguments that can serve as the value of a <v>:</v><tt><see topic=\"ACL2____CLAUSE-PROCESSOR\">clause-processor</see></tt> \u000A hint; see <see topic=\"ACL2____HINTS\">hints</see>.</p> \u000A \u000A <p>This command calls the <v>prove</v> command, and hence should only be used at \u000A the top of the conclusion.</p>"],

 "COMMON-LISP____PLUSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____PLUSP\">Plusp</see> x)</v> is true if and only if <v>x &gt; 0</v>.</p> \u000A \u000A <p>The <see topic=\"ACL2____GUARD\">guard</see> of <v>plusp</v> requires its argument to be a rational \u000A (<see topic=\"COMMON-LISP____REAL\">real</see>, in ACL2(r)) number.</p> \u000A \u000A <p><v>Plusp</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>plusp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> plusp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____REAL_F2RATIONALP\">real/rationalp</see> x)))\u000A       (<see topic=\"COMMON-LISP_____E3\">&gt;</see> x 0))</code>"],

 "COMMON-LISP____IDENTITY":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____IDENTITY\">Identity</see> x)</v> equals <v>x</v>; what else can we say?</p> \u000A \u000A <p><v>Identity</v> is a Common Lisp function.  See any Common Lisp documentation \u000A for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>identity</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> identity (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       x)</code>"],

 "COMMON-LISP____CONJUGATE":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Conjugate</v> takes an ACL2 number as an argument, and returns its \u000A complex conjugate (i.e., the result of negating its imaginary part.).</p> \u000A \u000A <p><v>Conjugate</v> is a Common Lisp function.  See any Common Lisp \u000A documentation for more information.</p> \u000A \u000A <p><b>Function: </b><srclink>conjugate</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> conjugate (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____COMPLEX_F2COMPLEX-RATIONALP\">complex/complex-rationalp</see> x)\u000A           (<see topic=\"COMMON-LISP____COMPLEX\">complex</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x) (<see topic=\"COMMON-LISP____-\">-</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x)))\u000A           x))</code>"],

 "ACL2____WATERFALL-PARALLELISM":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-WATERFALL-PARALLELISM\">set-waterfall-parallelism</see>.</p>"],

 "ACL2____Using_02the_02Associativity_02of_02App_02to_02Prove_02a_02Trivial_02Consequence":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Overview_02of_02the_02Proof_02of_02a_02Trivial_02Consequence'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <p>If we have proved the <v>associativity-of-app</v> rule, then the following \u000A theorem is trivial:</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFTHM\">defthm</see> trivial-consequence\u000A  (<see topic=\"COMMON-LISP____EQUAL\">equal</see> (app (app (app (app x1 x2) (app x3 x4)) (app x5 x6)) x7)\u000A         (app x1 (app (app x2 x3) (app (app x4 x5) (app x6 x7))))))</code> \u000A \u000A <p>Below we show the proof</p> \u000A \u000A <p><see topic='ACL2____Overview_02of_02the_02Proof_02of_02a_02Trivial_02Consequence'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_028_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL <b>(CONS (CAR A)</b> \u000A                      <b>(APP (APP (CDR A) B)</b> \u000A                           <b>C))</b> \u000A                <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_029_92'>(</see>APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to expand the definition of <v>APP</v> here.  This \u000A time, we'll do the whole expansion at once, including the simplification of \u000A the resulting <v>IF</v>.  This is how ACL2 actually does it.</p>"],

 "ACL2____The_02Instantiation_02of_02the_02Induction_02Scheme":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The induction scheme just shown is just an abbreviation for our \u000A real goal.</p> \u000A \u000A <p>To obtain our actual goals we have to replace the schema <v>:P</v> by the \u000A associativity conjecture (instantiated as shown in the scheme).</p> \u000A \u000A <p>This produces two actual goals, the induction step and the base case.</p>"],

 "ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_020_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/1' \u000A(IMPLIES (NOT (CONSP A)) \u000A         (EQUAL (APP <see topic='ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_021_92'>(</see>APP A B) C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to replace <v>(APP A B)</v> by its definition.  Note \u000A that the hypothesis <v>(<see topic=\"COMMON-LISP____NOT\">NOT</see> (<see topic=\"COMMON-LISP____CONSP\">CONSP</see> A))</v> allows us to simplify the <v>IF</v> in \u000A <v>APP</v> to its <b>false branch</b> this time.</p>"],

 "ACL2____TRUE-LIST-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>True-list-listp</v> is the function that checks whether its \u000A argument is a list that ends in, or equals, <v>nil</v>, and furthermore, all of \u000A its elements have that property.  Also see <see topic=\"ACL2____TRUE-LISTP\">true-listp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>true-list-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> true-list-listp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"ACL2____TRUE-LIST-LISTP\">true-list-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____SYNTAX":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>For the details of ACL2 syntax, see CLTL.  For examples of ACL2 \u000A syntax, use <v>:</v><tt><see topic=\"ACL2____PE\">pe</see></tt> to print some of the ACL2 system code.  For \u000A example:</p> \u000A \u000A <code>:pe assoc-equal\u000A:pe dumb-occur\u000A:pe var-fn-count\u000A:pe add-linear-variable-to-alist</code> \u000A \u000A <p>There is no comprehensive description of the ACL2 syntax yet, except that \u000A found in CLTL.  Also see <see topic=\"ACL2____TERM\">term</see>.</p>"],

 "ACL2____SET-OVERRIDE-HINTS_12":[["Override-hints"],"ACL2 Sources","ACL2","<p><v>Set-override-hints!</v> is the same as <tt><see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see></tt>, \u000A except that the former is not <see topic=\"ACL2____LOCAL\">local</see> to <see topic=\"ACL2____BOOKS\">books</see> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs.  See <see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see>; \u000A also see <see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see>.</p>"],

 "ACL2____REMOVE-RAW-ARITY":[["Set-raw-mode"],"ACL2 Sources","ACL2","<p>Technical note: This macro is a no-op, and is not necessary, when \u000A ACL2 is built with #-acl2-mv-as-values.</p> \u000A \u000A <p>The form <v>(<see topic=\"ACL2____REMOVE-RAW-ARITY\">remove-raw-arity</see> fn)</v> undoes the effect of an earlier \u000A <v>(<see topic=\"ACL2____REMOVE-RAW-ARITY\">remove-raw-arity</see> fn val)</v>.  See <see topic=\"ACL2____ADD-RAW-ARITY\">add-raw-arity</see>.</p>"],

 "ACL2____R-SYMBOL-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____R-SYMBOL-ALISTP\">R-symbol-alistp</see> x)</v> is true if and only if <v>x</v> is a list of \u000A pairs of the form <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> key val)</v> where <v>val</v> is a <tt><see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>r-symbol-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> r-symbol-alistp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                     (<see topic=\"ACL2____R-SYMBOL-ALISTP\">r-symbol-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____R-EQLABLE-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>r-eqlable-alistp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of <tt><see topic=\"COMMON-LISP____CONSP\">consp</see></tt> objects whose <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>s all satisfy \u000A <tt><see topic=\"ACL2____EQLABLEP\">eqlablep</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>r-eqlable-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> r-eqlable-alistp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"ACL2____EQLABLEP\">eqlablep</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                     (<see topic=\"ACL2____R-EQLABLE-ALISTP\">r-eqlable-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Step_02to_02T":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Base_02Case_02to_02T'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <code><see topic='ACL2____On_02the_02Naming_02of_02Subgoals'>Subgoal *1/2</see> \u000A(IMPLIES (AND (NOT (ENDP A)) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A \u000ABy the simple :definition <see topic='ACL2____Overview_02of_02the_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step'>ENDP</see> we reduce the conjecture to \u000A \u000ASubgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A \u000A<see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Conclusion'>But</see> simplification reduces this to T, using the :definition APP, the \u000A:rewrite rules CDR-CONS and CAR-CONS and primitive type reasoning. \u000A</code> \u000A \u000A <p><see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Base_02Case_02to_02T'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Conclusion":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>In this message the system is saying that <v>Subgoal *1/2'</v> has \u000A been rewritten to T using the rules noted.  The word ``<b>But</b>'' at the \u000A beginning of the sentence is a signal that the goal has been proved.</p> \u000A \u000A <p>Click <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_020_92'>here</see> to step \u000A through the proof of <v>Subgoal *1/2'</v>.</p>"],

 "ACL2____On_02the_02Naming_02of_02Subgoals":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><v>Subgoal *1/2</v> is the <b>induction step</b> from the scheme, \u000A obtained by instantiating the scheme with our conjecture.</p> \u000A \u000A <p>We number the cases ``backward'', so this is case ``2'' of the proof of \u000A ``*1''.  We number them backward so you can look at a subgoal number and get \u000A an estimate for how close you are to the end.</p>"],

 "ACL2____Nontautological_02Subgoals":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The theorem prover's proof output is intended to suggest an outline \u000A of the reasoning process employed by its proof engine, which is virtually \u000A always more than is necessary for the ACL2 user.  In particular, the output \u000A often omits subgoals that are sufficiently trivial, including \u000A tautologies.</p>"],

 "ACL2____MEMSUM":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>This macro is an abbreviation for <tt><see topic=\"ACL2____MEMOIZE-SUMMARY\">memoize-summary</see></tt>.  Logically, it \u000A just returns <v>nil</v>.</p>"],

 "ACL2____DEFUND-NOTINLINE":[["Defun","Events"],"ACL2 Sources","ACL2","<p><v>Defund-notinline</v> is a variant of <tt><see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see></tt>, the \u000A difference being that <v>defund-notinline</v> disables the newly-defined \u000A function symbol.  See <see topic=\"ACL2____DEFUN-NOTINLINE\">defun-notinline</see>.</p>"],

 "ACL2____COMPLEX-RATIONALP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> 3)       ; nil, as 3 is rational, not complex rational\u000A(<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> #c(3 0)) ; nil, since #c(3 0) is the same as 3\u000A(<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> t)       ; nil\u000A(<see topic=\"ACL2____COMPLEX-RATIONALP\">complex-rationalp</see> #c(3 1)) ; t, as #c(3 1) is the complex number 3 + i</code> \u000A \u000A <p>See <see topic=\"COMMON-LISP____COMPLEX\">complex</see> for more about complex rationals in ACL2.</p>"],

 "ACL2____CLOSE-TRACE-FILE":[["Trace"],"ACL2 Sources","ACL2","<code>General Form:\u000A(<see topic=\"ACL2____CLOSE-TRACE-FILE\">close-trace-file</see>) ; trace output is no longer redirected to a file</code> \u000A \u000A <p>Output from <tt><see topic=\"ACL2____TRACE_42\">trace$</see></tt> normally goes to the screen, or more precisely, \u000A <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.  It can be redirected to a file; see <see topic=\"ACL2____OPEN-TRACE-FILE\">open-trace-file</see>.  Use <v>close-trace-file</v> to redirect trace output to \u000A <tt><see topic=\"ACL2____STANDARD-CO\">standard-co</see></tt>.</p>"],

 "ACL2____CHARACTER-ALISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____CHARACTER-ALISTP\">Character-alistp</see> x)</v> is true if and only if <v>x</v> is a list \u000A of pairs of the form <v>(<see topic=\"COMMON-LISP____CONS\">cons</see> key val)</v> where <v>key</v> is a <tt><see topic=\"COMMON-LISP____CHARACTERP\">characterp</see></tt>.</p> \u000A \u000A <p><b>Function: </b><srclink>character-alistp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> character-alistp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)))\u000A                     (<see topic=\"ACL2____CHARACTER-ALISTP\">character-alistp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____ADD-OVERRIDE-HINTS_12":[["Override-hints"],"ACL2 Sources","ACL2","<p><v>Add-override-hints!</v> is the same as <tt><see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see></tt>, \u000A except that the former is not <see topic=\"ACL2____LOCAL\">local</see> to <see topic=\"ACL2____BOOKS\">books</see> or <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> <see topic=\"ACL2____EVENTS\">events</see> in which it occurs.  See <see topic=\"ACL2____ADD-OVERRIDE-HINTS\">add-override-hints</see>; \u000A also see <see topic=\"ACL2____SET-OVERRIDE-HINTS\">set-override-hints</see>.</p>"],

 "ACL2____ACL2-NUMBERP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)</v> is true if and only if <v>x</v> is a number, \u000A i.e., a rational or complex rational number.</p>"],

 "ACL2____ACCESS":[["Defrec"],"ACL2 Sources","ACL2","<p>The <v>access</v> macro is built into ACL2, and allows you to access \u000A particular fields from structures that have been introduced with <see topic=\"ACL2____DEFREC\">defrec</see>.  For instance:</p> \u000A \u000A <code>(<see topic=\"ACL2____ACCESS\">access</see> employee x :name)</code> \u000A \u000A <p>would return the <v>name</v> field from the employee <v>x</v>.  See <see topic=\"ACL2____DEFREC\">defrec</see> for more information.</p>"],

 "ACL2-PC____RETAIN":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(RETAIN 2 3) -- keep the second and third hypotheses, and drop\u000A                the rest\u000A\u000AGeneral Form:\u000A(retain &amp;rest args)</code> \u000A \u000A <p>Drop all top-level hypotheses <b>except</b> those with the indicated \u000A indices.</p> \u000A \u000A <p>There must be at least one argument, and all must be in range (i.e. \u000A integers between one and the number of top-level hypotheses, inclusive).</p>"],

 "ACL2-PC____REPEAT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(repeat promote)\u000A\u000AGeneral Form:\u000A(repeat instruction)</code> \u000A \u000A <p>The given <v>instruction</v> is run repeatedly until it ``fails''.</p> \u000A \u000A <p><b>Remark:</b> There is nothing here in general to prevent the instruction \u000A from being run after all goals have been proved, though this is indeed the \u000A case for primitive instructions.</p>"],

 "ACL2-PC____DROP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(drop 2 3) -- drop the second and third hypotheses\u000Adrop       -- drop all top-level hypotheses\u000A\u000AGeneral Forms:\u000A(drop n1 n2 ...) -- Drop the hypotheses with the indicated indices.\u000A\u000Adrop             -- Drop all the top-level hypotheses.</code> \u000A \u000A <p><b>Remark:</b> If there are no top-level hypotheses, then the instruction \u000A <v>drop</v> will fail.  If any of the indices is out of range, i.e. is not an \u000A integer between one and the number of top-level hypotheses <v>(inclusive)</v>, \u000A then <v>(drop n1 n2 ...)</v> will fail.</p>"],

 "ACL2-PC____DO-STRICT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(do-strict induct p prove)\u000A\u000AGeneral Form:\u000A(do-strict &amp;rest instruction-list)</code> \u000A \u000A <p>Run the indicated instructions until there is a (hard or soft) ``failure''. \u000A In fact <v>do-strict</v> is identical in effect to <v>do-all</v>, except that \u000A <v>do-all</v> only halts once there is a hard ``failure''.  See the \u000A documentation for <v>do-all</v>.</p>"],

 "ACL2-PC____DO-ALL-NO-PROMPT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(do-all-no-prompt induct p prove)\u000A\u000AGeneral Form:\u000A(do-all-no-prompt &amp;rest instruction-list)</code> \u000A \u000A <p><v>Do-all-no-prompt</v> is the same as <v>do-all</v>, except that the prompt \u000A and instruction are not printed each time, regardless of the value of \u000A <v>pc-print-prompt-and-instr-flg</v>.  Also, restoring is disabled.  See the \u000A documentation for <v>do-all</v>.</p>"],

 "COMMON-LISP____REALPART":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-realpart</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"COMMON-LISP____REALPART\">realpart</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____REALPART\">realpart</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)</code>"],

 "COMMON-LISP____IMAGPART":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-imagpart</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)\u000A           (<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____IMAGPART\">imagpart</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> x)</code>"],

 "ACL2____What_02is_02a_02Mechanical_02Theorem_02Prover_82Q_92_02_82cont_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____About_02Models'><img \u000A src='flying.gif'></img></see> To get around this, mechanical theorem provers \u000A often require help from the <b>user</b>.</p> \u000A \u000A <p><img src='interactive-theorem-prover-a.gif'></img></p> \u000A \u000A <p>Click <see topic='ACL2____ACL2_02as_02an_02Interactive_02Theorem_02Prover'>here</see> to continue downward.</p> \u000A \u000A <p><see topic='ACL2____About_02Models'><img src='flying.gif'></img></see></p>"],

 "ACL2____WORMHOLE-STATUSP":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____WORMHOLE-STATUSP\">wormhole-statusp</see> whs)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  This predicate is useful in guards for wormholes.  It \u000A checks whether <v>whs</v> is either <v>nil</v> or a cons whose car is <v>:ENTER</v> \u000A or <v>:SKIP</v>.</p>"],

 "ACL2____WORMHOLE-P":[["Wormhole"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see> for a discussion of wormholes.  <v>(<see topic=\"ACL2____WORMHOLE-P\">Wormhole-p</see>\u000A state)</v> returns <v>(<see topic=\"ACL2____MV\">mv</see> nil t state)</v> when evaluated inside a wormhole, else \u000A <v>(<see topic=\"ACL2____MV\">mv</see> nil nil state)</v>.</p>"],

 "ACL2____WORMHOLE-ENTRY-CODE":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____WORMHOLE-ENTRY-CODE\">wormhole-entry-code</see> whs)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  Returns <v>:ENTER</v> or <v>:SKIP</v> given a well-formed \u000A wormhole status <v>whs</v>.  If <v>whs</v> is <v>nil</v> or not well-formed, the \u000A entry code is <v>:ENTER</v>.</p>"],

 "ACL2____WORMHOLE-DATA":[["Wormhole"],"ACL2 Sources","ACL2","<code>General Form:  (<see topic=\"ACL2____WORMHOLE-DATA\">wormhole-data</see> whs)</code> \u000A \u000A <p>See <see topic=\"ACL2____WORMHOLE\">wormhole</see>.  Returns the wormhole data from a well-formed wormhole \u000A status <v>whs</v>.  If <v>whs</v> is <v>nil</v> or not well-formed, the data is \u000A <v>nil</v>.</p>"],

 "ACL2____UNSAVE":[["Proof-checker"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____UNSAVE\">unsave</see> assoc-of-append)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____UNSAVE\">unsave</see> name)</code> \u000A \u000A <p>Eliminates the association of a <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> state with <v>name</v>. \u000A See <see topic=\"ACL2____UNSAVE\">unsave</see> or see <see topic=\"ACL2-PC____UNSAVE\">ACL2-pc::unsave</see>.</p> \u000A \u000A <p>Also see <see topic=\"ACL2-PC____SAVE\">ACL2-pc::save</see> and see <see topic=\"ACL2____RETRIEVE\">retrieve</see>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0212_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A<b>T</b> \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>So, indeed, <v>Subgoal *1/2'</v> <b>does</b> simplify to T!</p> \u000A \u000A <p>You can see that even in an example as simple as this one, quite a lot \u000A happens in simplification.</p> \u000A \u000A <p>You may click <see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Step_02to_02T'>here</see> to \u000A return to the main proof.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0210_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0211_92'>(</see><b>EQUAL (APP (APP (CDR A) B) C)</b> \u000A                <b>(APP (CDR A) (APP B C)))</b>). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to use the Induction Hypothesis (which is the \u000A second of the two hypotheses above and which is identical to the rewritten \u000A conclusion).</p>"],

 "ACL2____The_02End_02of_02the_02Flying_02Tour":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='landing.gif'></img></p> \u000A \u000A <p>This completes the Flying Tour.</p> \u000A \u000A <p>We recommend that you now take <see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'>A Walking Tour of ACL2</see>.</p> \u000A \u000A <p>Thanks.<br></br> \u000A \u000A Matt Kaufmann and J Moore</p> \u000A \u000A <p><see topic='ACL2____A_02Walking_02Tour_02of_02ACL2'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____The_02Base_02Case_02in_02the_02App_02Example":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>This formula is the <b>Base Case</b>.  It consists of two parts, a \u000A test identifying the non-inductive case and the conjecture to prove.</p> \u000A \u000A <code>(IMPLIES (ENDP A)                 <b>; Test</b> \u000A         (:P A B C))              <b>; Conjecture</b> \u000A</code> \u000A \u000A <p>When we prove this we can assume</p> \u000A \u000A <code> * <v>A</v> is empty </code> \u000A \u000A <p>and we have to prove the conjecture for <v>A</v>.</p>"],

 "ACL2____STANDARD-PART":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____STANDARD-PART\">Standard-part</see> x)</v> is, for a given <tt><see topic=\"ACL2____I-LIMITED\">i-limited</see></tt> number \u000A <v>x</v>, the unique real number infinitesimally close (see <see topic=\"ACL2____I-CLOSE\">i-close</see>) to \u000A <v>x</v>.  This function is only defined in ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____SET-RW-CACHE-STATE_12":[["Rewrite"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____SET-RW-CACHE-STATE\">set-rw-cache-state</see>, which is the same as \u000A <v>set-rw-cache-state!</v>  except that the latter is not <tt><see topic=\"ACL2____LOCAL\">local</see></tt> to the \u000A <tt><see topic=\"ACL2____ENCAPSULATE\">encapsulate</see></tt> or the book in which it occurs.</p>"],

 "ACL2____RETRIEVE":[["Proof-checker"],"ACL2 Sources","ACL2","<code>Examples:\u000A(<see topic=\"ACL2____RETRIEVE\">retrieve</see> associativity-of-permutationp)\u000Aretrieve\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____RETRIEVE\">retrieve</see> &amp;optional name)</code> \u000A \u000A <p>See <see topic=\"ACL2-PC____RETRIEVE\">ACL2-pc::retrieve</see>, or use <v>(<see topic=\"ACL2____HELP\">help</see> retrieve)</v> inside the \u000A interactive <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> loop.  Also see <see topic=\"ACL2____UNSAVE\">unsave</see>.</p>"],

 "ACL2____RESTORE-MEMOIZATION-SETTINGS":[["Events","Hons-and-memoization"],"ACL2 Sources","ACL2","<p>For background on memoization, see <see topic=\"ACL2____MEMOIZE\">memoize</see>.</p> \u000A \u000A <code>General Form:\u000A(<see topic=\"ACL2____RESTORE-MEMOIZATION-SETTINGS\">restore-memoization-settings</see>)</code> \u000A \u000A <p>Calls of this macro restore the memoization settings saved by <tt><see topic=\"ACL2____SAVE-AND-CLEAR-MEMOIZATION-SETTINGS\">save-and-clear-memoization-settings</see></tt>.</p>"],

 "ACL2____REMOVE-BINOP":[["Macros"],"ACL2 Sources","ACL2","<p>The form <v>(<see topic=\"ACL2____REMOVE-BINOP\">remove-binop</see> macro-fn)</v> is an abbreviation for the \u000A form <v>(<see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see> macro-fn)</v>.  See <see topic=\"ACL2____REMOVE-MACRO-FN\">remove-macro-fn</see>.</p>"],

 "ACL2____PROPER-CONSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Proper-consp</v> is the function that checks whether its argument \u000A is a non-empty list that ends in <v>nil</v>.  Also see <see topic=\"ACL2____TRUE-LISTP\">true-listp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>proper-consp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> proper-consp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x)))</code>"],

 "ACL2____NOTE-2-1":[["Release-notes"],"ACL2 Sources","ACL2","<p>The identity function <tt><see topic=\"ACL2____CASE-SPLIT\">case-split</see></tt> has been added.  It is \u000A similar to <tt><see topic=\"ACL2____FORCE\">force</see></tt> but causes an immediate split of the top-level goal \u000A on whether the indicated hypothesis is true.</p> \u000A \u000A <p>Less important notes:</p> \u000A \u000A <p>Minor bugs in the documentation were fixed.</p>"],

 "ACL2____KWOTE-LST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The function <v>kwote-lst</v> applies the function <v>kwote</v> to each \u000A element of a given list.  The guard of <v>(<see topic=\"ACL2____KWOTE-LST\">kwote-lst</see> lst)</v> is <v>(<see topic=\"ACL2____TRUE-LISTP\">true-listp</see>\u000A lst)</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>kwote-lst</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> kwote-lst (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> lst)))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ENDP\">endp</see> lst) nil)\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"ACL2____KWOTE\">kwote</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                      (<see topic=\"ACL2____KWOTE-LST\">kwote-lst</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code>"],

 "ACL2____IMPROPER-CONSP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Improper-consp</v> is the function that checks whether its \u000A argument is a non-empty list that ends in other than <v>nil</v>.  See <see topic=\"ACL2____PROPER-CONSP\">proper-consp</see> and also see <see topic=\"ACL2____TRUE-LISTP\">true-listp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>improper-consp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> improper-consp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x) (<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____TRUE-LISTP\">true-listp</see> x))))</code>"],

 "ACL2____FC-REPORT":[["Forward-chaining-reports"],"ACL2 Sources","ACL2","<code>Example: (<see topic=\"ACL2____FC-REPORT\">fc-report</see> 15)\u000A\u000AGeneral Form: (<see topic=\"ACL2____FC-REPORT\">fc-report</see> k)</code> \u000A \u000A <p>where <v>k</v> is the number of some forward chaining report printed in the \u000A most recent event.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for details.</p>"],

 "ACL2____EXIT":[["Good-bye"],"ACL2 Sources","ACL2","<p>Same as <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt>.</p>"],

 "ACL2____ESCAPE-TO-COMMON-LISP":[["Miscellaneous"],"ACL2 Sources","ACL2","<code>Example:\u000AACL2 !&gt;:Q</code> \u000A \u000A <p>There is no Common Lisp escape feature in the <tt><see topic=\"ACL2____LP\">lp</see></tt>.  This is part of \u000A the price of purity.  To execute a form in Common Lisp as opposed to ACL2, \u000A exit <tt><see topic=\"ACL2____LP\">lp</see></tt> with <v>:</v><tt><see topic=\"ACL2____Q\">q</see></tt>, submit the desired forms to the Common \u000A Lisp read-eval-print loop, and reenter ACL2 with <v>(<see topic=\"ACL2____LP\">lp</see>)</v>.</p>"],

 "ACL2____DEFUND-INLINE":[["Defun","Events"],"ACL2 Sources","ACL2","<p><v>Defund-inline</v> is a variant of <tt><see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see></tt>, the \u000A difference being that <v>defund-inline</v> disables the newly-defined function \u000A symbol.  See <see topic=\"ACL2____DEFUN-INLINE\">defun-inline</see>.</p>"],

 "ACL2____DEFAULT-BACKCHAIN-LIMIT":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>.</p> \u000A \u000A <p>The initial value is <v>(nil nil)</v>.  To inspect the current value (as \u000A explained elsewhere; see <see topic=\"ACL2____BACKCHAIN-LIMIT\">backchain-limit</see>):</p> \u000A \u000A <code>(<see topic=\"ACL2____DEFAULT-BACKCHAIN-LIMIT\">default-backchain-limit</see> wrld :ts) ; for type-set reasoning\u000A(<see topic=\"ACL2____DEFAULT-BACKCHAIN-LIMIT\">default-backchain-limit</see> wrld :rewrite) ; for rewriting</code>"],

 "ACL2____ATOM-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>atom-listp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of <see topic=\"COMMON-LISP____ATOM\">atom</see>s, i.e., of non-conses.</p> \u000A \u000A <p>Also see <see topic=\"ACL2____GOOD-ATOM-LISTP\">good-atom-listp</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>atom-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> atom-listp (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) (<see topic=\"COMMON-LISP____EQ\">eq</see> lst nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____ATOM\">atom</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst))\u000A                     (<see topic=\"ACL2____ATOM-LISTP\">atom-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code>"],

 "ACL2-PC____SLS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Asrs\u000A\u000AGeneral Form:\u000A(srs &amp;optional rule-id enabled-only-flg)</code> \u000A \u000A <p>See the documentation for <v>show-linears</v>, as <v>sls</v> and \u000A <v>show-linears</v> are identical.  NOTE: In analogy to the <v>sr</v> abbreviation \u000A for <v>show-rewrites</v>, one might expect this command to be <v>sl</v>; but that \u000A name was taken (``simplify with lemmas'') before <v>sls</v> was \u000A implemented.</p>"],

 "ACL2-PC____NIL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General form:\u000Anil</code> \u000A \u000A <p>(or, <v>control-d</v>).</p> \u000A \u000A <p>The whole point of this command is that in some Lisps (including akcl), if \u000A you type <v>control-d</v> then it seems, on occasion, to get interpreted as \u000A <v>nil</v>.  Without this command, one seems to get into an infinite loop.</p>"],

 "ACL2-PC____NEGATE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Example: \u000A (negate prove)</p> \u000A \u000A <code>General form:\u000A(negate &amp;rest instruction-list)</code> \u000A \u000A <p>Run the indicated instructions exactly in the sense of <v>do-all</v>, and \u000A ``succeed'' if and only if they ``fail''.</p> \u000A \u000A <p><b>Remark:</b> <v>Negate</v> instructions will never produce hard \u000A ``failures''.</p>"],

 "ACL2-PC____ILLEGAL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(<see topic=\"ACL2____ILLEGAL\">illegal</see> -3)\u000A\u000AGeneral Form:\u000A(<see topic=\"ACL2____ILLEGAL\">illegal</see> instruction)</code> \u000A \u000A <p>Probably not of interest to most users; always ``fails'' since it expands \u000A to the <v>fail</v> command.</p> \u000A \u000A <p>The <v>illegal</v> command is used mainly in the implementation.  For \u000A example, the instruction <v>0</v> is ``read'' as <v>(<see topic=\"ACL2____ILLEGAL\">illegal</see> 0)</v>, since \u000A <v>dive</v> expects positive integers.</p>"],

 "ACL2-PC____GOALS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Agoals</code> \u000A \u000A <p><v>Goals</v> lists the names of all goals that remain to be proved.  They are \u000A listed in the order in which they appear on the stack of remaining goals, \u000A which is relevant for example to the effect of a <v>change-goal</v> \u000A instruction.</p>"],

 "ACL2-PC____ELIM":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Aelim</code> \u000A \u000A <p>Upon running the <v>elim</v> command, the system will create a subgoal will \u000A be created for each goal that would have been pushed for proof by induction in \u000A an ordinary proof, where <b>only</b> elimination is used; not even \u000A simplification is used!</p>"],

 "COMMON-LISP____SYMBOLP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____SYMBOLP\">symbolp</see> x)</v> is true if and only if <v>x</v> is a symbol.</p>"],

 "COMMON-LISP____PAIRLIS":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The Common Lisp language allows its <v>pairlis</v> function to \u000A construct an alist in any order!  So we have to define our own version: See \u000A <see topic=\"ACL2____PAIRLIS_42\">pairlis$</see>.</p>"],

 "COMMON-LISP____CADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "ACL2____XOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Xor</v> is the ACL2 exclusive-or function.  <v>(<see topic=\"ACL2____XOR\">xor</see> P Q)</v> means \u000A that either <v>P</v> or <v>Q</v>, but not both, is false (i.e., <v>nil</v>).</p> \u000A \u000A <p><b>Function: </b><srclink>xor</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> xor (p q)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> p (<see topic=\"COMMON-LISP____IF\">if</see> q nil t) (<see topic=\"COMMON-LISP____IF\">if</see> q t nil)))</code>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0211_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A<see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_0212_92'>(</see>IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         <b>T</b>) \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to use the definition of <v>IMPLIES</v>.  Since the \u000A conclusion of the implication is now identically <v>T</v>, the implication \u000A simplifies to <v>T</v>.</p>"],

 "ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_023_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/1' \u000A(IMPLIES (NOT (CONSP A)) \u000A         <b>T</b>) \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Now that its conclusion is identically <v>T</v> the <v>IMPLIES</v> will \u000A simplify to <v>T</v> (not shown) and we are done with <v>Subgoal *1/1'</v>.</p> \u000A \u000A <p>You may click <see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Base_02Case_02to_02T'>here</see> to return \u000A to the main proof.</p>"],

 "ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_021_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2 \u000A(IMPLIES (AND <see topic='ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_022_92'>(</see>NOT <b>(NOT (CONSP A))</b>) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>The <b>bold</b> text is the instantiated definition of <v>ENDP</v>.</p> \u000A \u000A <p>Now click on the link above to simplify (NOT (NOT (CONSP A)))</p>"],

 "ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_020_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2 \u000A(IMPLIES (AND (NOT <see topic='ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_021_92'>(</see>ENDP A)) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above (the open parenthesis before <v>ENDP</v>) to replace \u000A <v>(<see topic=\"COMMON-LISP____ENDP\">ENDP</see> A)</v> by its definition.</p>"],

 "ACL2____The_02End_02of_02the_02Proof_02of_02the_02Associativity_02of_02App":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____Guiding_02the_02ACL2_02Theorem_02Prover'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <code>That <see topic='ACL2____Popping_02out_02of_02an_02Inductive_02Proof'>completes</see> the proof of *1. \u000A \u000A<see topic='ACL2____The_02Q.E.D._02Message'>Q.E.D.</see> \u000A \u000ASummary \u000AForm:  ( DEFTHM ASSOCIATIVITY-OF-APP ...) \u000A<see topic='ACL2____The_02Rules_02used_02in_02the_02Associativity_02of_02App_02Proof'>Rules</see>: ((:REWRITE CDR-CONS) \u000A        (:REWRITE CAR-CONS) \u000A        (:DEFINITION NOT) \u000A        (:DEFINITION ENDP) \u000A        (:FAKE-RUNE-FOR-TYPE-SET NIL) \u000A        (:DEFINITION APP)) \u000AWarnings:  None \u000ATime:  0.27 seconds (prove: <see topic='ACL2____The_02Time_02Taken_02to_02do_02the_02Associativity_02of_02App_02Proof'>0.10</see>, print: 0.05, other: 0.12) \u000A ASSOCIATIVITY-OF-APP \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p><see topic='ACL2____Guiding_02the_02ACL2_02Theorem_02Prover'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____SHOW-FC-CRITERIA":[["Forward-chaining-reports"],"ACL2 Sources","ACL2","<code>Example:  (<see topic=\"ACL2____SHOW-FC-CRITERIA\">show-fc-criteria</see>)</code> \u000A \u000A <p>This function prints the list of triples being used to determine what is \u000A tracked during forward chaining.</p> \u000A \u000A <p>See <see topic=\"ACL2____FORWARD-CHAINING-REPORTS\">forward-chaining-reports</see> for details.</p>"],

 "ACL2____Overview_02of_02the_02Simplification_02of_02the_02Base_02Case_02to_02T":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><see topic='ACL2____The_02End_02of_02the_02Proof_02of_02the_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p> \u000A \u000A <code><see topic='ACL2____Overview_02of_02the_02Expansion_02of_02ENDP_02in_02the_02Base_02Case'>Subgoal *1/1</see> \u000A(IMPLIES (ENDP A) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A \u000ABy the simple :definition ENDP we reduce the conjecture to \u000A \u000ASubgoal *1/1' \u000A(IMPLIES (NOT (CONSP A)) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A \u000A<see topic='ACL2____Overview_02of_02the_02Final_02Simplification_02in_02the_02Base_02Case'>But</see> simplification reduces this to T, using the :definition APP and \u000Aprimitive type reasoning. \u000A \u000A</code> \u000A \u000A <p><see topic='ACL2____The_02End_02of_02the_02Proof_02of_02the_02Associativity_02of_02App'><img \u000A src='walking.gif'></img></see></p>"],

 "ACL2____NOTE-2-9_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>No changes have been made for support of non-standard analysis, \u000A other than a minor modification or two in <v>books/nonstd/</v> books.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-2-9\">note-2-9</see> for changes to Version_2.9 of ACL2.</p>"],

 "ACL2____NOTE-2-6_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important changes to non-standard version: None since Version \u000A 2.5.</p> \u000A \u000A <p>Please see <see topic=\"ACL2____NOTE-2-6\">note-2-6</see> for changes to Version 2.6 of ACL2.  We hope to \u000A write more documentation for ACL2(r) in the future.</p>"],

 "ACL2____NAT-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>nat-listp</v> tests whether its argument is a true \u000A list of natural numbers.</p> \u000A \u000A <p><b>Function: </b><srclink>nat-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> nat-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____NATP\">natp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____NAT-LISTP\">nat-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____MONITORED-RUNES":[["Break-rewrite"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000A:monitored-runes</code> \u000A \u000A <p>This macro prints a list, each element of which is of the form <v>(<see topic=\"ACL2____RUNE\">rune</see>\u000A expr)</v>, showing each <see topic=\"ACL2____MONITOR\">monitor</see>ed <see topic=\"ACL2____RUNE\">rune</see> and its current break \u000A condition.</p>"],

 "ACL2____MBE1":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The form <v>(<see topic=\"ACL2____MBE1\">mbe1</see> exec logic)</v> is equivalent to the forms \u000A <v>(<see topic=\"ACL2____MBE\">mbe</see> :logic logic :exec exec)</v> and <v>(<see topic=\"ACL2____MUST-BE-EQUAL\">must-be-equal</see> logic exec)</v>.  See \u000A <see topic=\"ACL2____MBE\">mbe</see>.</p>"],

 "ACL2____MANAGING-ACL2-PACKAGES":[["Defpkg"],"ACL2 Sources","ACL2","<p>Jared Davis has contributed documentation on <a \u000A  href='http://www.cs.utexas.edu/users/moore/acl2/contrib/managing-acl2-packages.html'>managing \u000A  ACL2 packages</a>.</p>"],

 "ACL2____INTEGER-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>integer-listp</v> tests whether its argument is a \u000A true list of integers.</p> \u000A \u000A <p><b>Function: </b><srclink>integer-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> integer-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____INTEGERP\">integerp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____INTEGER-LISTP\">integer-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____I-LIMITED":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____I-LIMITED\">I-limited</see> x)</v> is true if and only if <v>x</v> is a number that \u000A is not infinitely large.  This predicate is only defined in ACL2(r) \u000A (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____I-CLOSE":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____I-CLOSE\">I-close</see> x y)</v> is true if and only if <v>x-y</v> is an \u000A infinitesimal number.  This predicate is only defined in ACL2(r) (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____FIX-TRUE-LIST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>Fix-true-list</v> is the identity function on <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> \u000A objects.  It converts every list to a true list by dropping the final <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>, and it converts every <see topic=\"COMMON-LISP____ATOM\">atom</see> to <v>nil</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>fix-true-list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> fix-true-list (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____CONSP\">consp</see> x)\u000A           (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x) (<see topic=\"ACL2____FIX-TRUE-LIST\">fix-true-list</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x)))\u000A           nil))</code>"],

 "ACL2____DEFMACRO-LAST":[["Events"],"ACL2 Sources","ACL2","<p>This is an advanced feature that requires a trust tag.  For \u000A explanation, including an example, see <see topic=\"ACL2____RETURN-LAST\">return-last</see>.</p>"],

 "ACL2____CHARACTER-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>character-listp</v> tests whether its argument is a \u000A true list of <see topic=\"ACL2____CHARACTERS\">characters</see>.</p> \u000A \u000A <p><b>Function: </b><srclink>character-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> character-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQUAL\">equal</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____CHARACTER-LISTP\">character-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____BOOLEAN-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>boolean-listp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of objects each or which satisfyies <tt><see topic=\"ACL2____BOOLEANP\">booleanp</see></tt>, i.e., \u000A is <v>t</v> or <v>nil</v>.</p> \u000A \u000A <p><b>Function: </b><srclink>boolean-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> boolean-listp (lst)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> lst) (<see topic=\"COMMON-LISP____EQ\">eq</see> lst nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____OR\">or</see> (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst) t) (<see topic=\"COMMON-LISP____EQ\">eq</see> (<see topic=\"COMMON-LISP____CAR\">car</see> lst) nil))\u000A                     (<see topic=\"ACL2____BOOLEAN-LISTP\">boolean-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> lst))))))</code>"],

 "ACL2____ACL2-NUMBER-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>acl2-number-listp</v> tests whether its argument is \u000A a true list of numbers.</p> \u000A \u000A <p><b>Function: </b><srclink>acl2-number-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> acl2-number-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"ACL2____ACL2-NUMBERP\">acl2-numberp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____ACL2-NUMBER-LISTP\">acl2-number-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____ABORT_12":[["Ld"],"ACL2 Sources","ACL2","<p>This is an alias for <v>a!</v>; see <see topic=\"ACL2____A_12\">a!</see>.  For a related feature \u000A that only pops up one level, see <see topic=\"ACL2____P_12\">p!</see>.</p>"],

 "ACL2-PC____SUCCEED":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(succeed induct p prove)\u000A\u000AGeneral Form:\u000A(succeed &amp;rest instruction-list)</code> \u000A \u000A <p>Run the indicated instructions until there is a hard ``failure'', and \u000A ``succeed''.  (See the documentation for <v>sequence</v> for an explanation of \u000A ``success'' and ``failure''.)</p>"],

 "ACL2-PC____PRO":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Apro</code> \u000A \u000A <p>Apply the <v>promote</v> command until there is no change.  This command \u000A ``succeeds'' exactly when at least one call of <v>promote</v> ``succeeds''.  In \u000A that case, only a single new proof-checker state will be created.</p>"],

 "ACL2-PC____PRINT-ALL-CONCS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Example and General Form: print-all-concs</p> \u000A \u000A <p>Prints all the conclusions of goals that remain to be proved, in a pleasant \u000A format.  See also the proof-checker command <v>print-all-goals</v>.</p>"],

 "ACL2-PC____PP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000App</code> \u000A \u000A <p>This is the same as <v>p</v> (see its documentation), except that raw syntax \u000A (internal form) is used.  So for example, one would see <v>(<see topic=\"COMMON-LISP____IF\">if</see> x y 'nil)</v> \u000A rather than <v>(<see topic=\"COMMON-LISP____AND\">and</see> x y)</v>.  Abbreviations are however still inserted, as with \u000A <v>p</v>.</p>"],

 "ACL2-PC____MORE_12":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Continues documentation of last proof-checker command visited with \u000A <v>help</v>, until all documentation on that command is printed out.</p> \u000A \u000A <p>Invoke <v>help</v> for documentation about the proof-checker help \u000A facility.</p>"],

 "ACL2-PC____COMMENT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(comment now begin difficult final goal)\u000A\u000AGeneral Form:\u000A(comment &amp;rest x)</code> \u000A \u000A <p>This instruction makes no change in the state except to insert the \u000A <v>comment</v> instruction.</p> \u000A \u000A <p>Some comments can be used to improve the display of commands; see \u000A documentation for <v>comm</v>.</p>"],

 "COMMON-LISP____NUMERATOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-numerator</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A           (<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x)\u000A         0))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____NUMERATOR\">numerator</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</code>"],

 "COMMON-LISP____DENOMINATOR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Completion Axiom (<v>completion-of-denominator</v>):</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____EQUAL\">equal</see> (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)\u000A       (<see topic=\"COMMON-LISP____IF\">if</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)\u000A           (<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)\u000A         1))</code> \u000A \u000A <p><see topic=\"ACL2____GUARD\">Guard</see> for <v>(<see topic=\"COMMON-LISP____DENOMINATOR\">denominator</see> x)</v>:</p> \u000A \u000A <code>(<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> x)</code>"],

 "COMMON-LISP____CHARACTERP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> x)</v> is true if and only if <v>x</v> is a \u000A  character.</p>"],

 "ACL2____The_02Summary_02of_02the_02Proof_02of_02the_02Trivial_02Consequence":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Note that at the conclusion of the proof, the system reminds you of \u000A the earlier <b>Warning</b>.</p> \u000A \u000A <p>It is a good idea, when the <b>Q.E.D.</b> flys by, to see if there were any \u000A Warnings.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_027_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_028_92'>(</see>IF T \u000A                    (CONS (CAR A) \u000A                          (APP <b>(APP (CDR A) B)</b> \u000A                               C)) \u000A                    C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom <v>(<see topic=\"COMMON-LISP____IF\">IF</see> T x y) = x</v>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_026_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (IF T \u000A                    (CONS <b>(CAR A)</b> \u000A                          (APP <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_027_92'>(</see>CDR (CONS (CAR A) (APP (CDR A) B))) \u000A                               C)) \u000A                    C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom <v>(<see topic=\"COMMON-LISP____CDR\">CDR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> x y)) = y</v>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_025_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (IF <b>T</b> \u000A                    (CONS <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_026_92'>(</see>CAR (CONS (CAR A) (APP (CDR A) B))) \u000A                          (APP (CDR (CONS (CAR A) (APP (CDR A) B))) \u000A                               C)) \u000A                    C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom <v>(<see topic=\"COMMON-LISP____CAR\">CAR</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> x y)) = x</v>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_024_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (IF <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_025_92'>(</see><b>CONSP (CONS (CAR A) (APP (CDR A) B)))</b> \u000A                    <b>(CONS (CAR (CONS (CAR A) (APP (CDR A) B)))</b> \u000A                          <b>(APP (CDR (CONS (CAR A) (APP (CDR A) B)))</b> \u000A                               <b>C))</b> \u000A                    <b>C)</b> \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom <v>(<see topic=\"COMMON-LISP____CONSP\">CONSP</see> (<see topic=\"COMMON-LISP____CONS\">CONS</see> x y)) =\u000A T</v>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_022_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_023_92'>(</see>IF <b>T</b> \u000A                         (CONS (CAR A) (APP (CDR A) B)) \u000A                         B) \u000A                     C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to apply the Axiom <v>(<see topic=\"COMMON-LISP____IF\">IF</see> T x y) = x</v>.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_020_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_021_92'>(</see>APP A B) C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to replace <v>(APP A B)</v> by its definition.</p>"],

 "ACL2____The_02Justification_02of_02the_02Induction_02Scheme":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>This paragraph explains why the induction selected is legal.  The \u000A explanation is basically the same as the explanation for why the recursion in \u000A <v>(APP A B)</v> terminates.</p>"],

 "ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_021_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/1' \u000A(IMPLIES (NOT (CONSP A)) \u000A         (EQUAL (APP <b>B</b> C) \u000A                <see topic='ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_022_92' use-tsee='yes'>(</see>APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to expand the definition of <v>APP</v>.  Again, we \u000A come out through the false branch because of the hypothesis.</p>"],

 "ACL2____The_02Expansion_02of_02ENDP_02in_02the_02Induction_02Step_02_82Step_022_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND <b>(CONSP A)</b> \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL (APP (APP A B) C) \u000A                (APP A (APP B C)))). \u000A \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Note that this is <v>Subgoal *1/2'.</v></p> \u000A \u000A <p>You may click <see topic='ACL2____Overview_02of_02the_02Simplification_02of_02the_02Induction_02Step_02to_02T'>here</see> to \u000A return to the main proof.</p>"],

 "ACL2____STRING-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>string-listp</v> tests whether its argument is a \u000A <tt><see topic=\"ACL2____TRUE-LISTP\">true-listp</see></tt> of strings.</p> \u000A \u000A <p><b>Function: </b><srclink>string-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> string-listp (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) (<see topic=\"COMMON-LISP____EQ\">eq</see> x nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____STRINGP\">stringp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A                     (<see topic=\"ACL2____STRING-LISTP\">string-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____RATIONAL-LISTP":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>The predicate <v>rational-listp</v> tests whether its argument is a \u000A true list of rational numbers.</p> \u000A \u000A <p><b>Function: </b><srclink>rational-listp</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> rational-listp (l)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> l) (<see topic=\"COMMON-LISP____EQ\">eq</see> l nil))\u000A             (t (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____RATIONALP\">rationalp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> l))\u000A                     (<see topic=\"ACL2____RATIONAL-LISTP\">rational-listp</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> l))))))</code>"],

 "ACL2____QUIT":[["Good-bye"],"ACL2 Sources","ACL2","<p>Same as <tt><see topic=\"ACL2____GOOD-BYE\">good-bye</see></tt>.</p>"],

 "ACL2____Popping_02out_02of_02an_02Inductive_02Proof":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>Recall that our induction scheme (click <see topic='ACL2____The_02Proof_02of_02the_02Associativity_02of_02App'>here</see> to revisit it) had two \u000A cases, the induction step (<v>Subgoal *1/2</v>) and the base case (<v>Subgoal\u000A *1/1</v>).  Both have been proved!</p>"],

 "ACL2____PRINT-BASE-P":[["Io"],"ACL2 Sources","ACL2","<p><b>Function: </b><srclink>print-base-p</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> print-base-p (print-base)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____AND\">and</see> (<see topic=\"COMMON-LISP____MEMBER\">member</see> print-base '(2 8 10 16))\u000A            t))</code>"],

 "ACL2____PARALLEL-EXECUTION":[["Parallel-programming"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-PARALLEL-EXECUTION\">set-parallel-execution</see> for how to configure parallel \u000A execution for calls of <tt><see topic=\"ACL2____PLET\">plet</see></tt>, <tt><see topic=\"ACL2____PARGS\">pargs</see></tt>, <tt><see topic=\"ACL2____PAND\">pand</see></tt>, <tt><see topic=\"ACL2____POR\">por</see></tt> (but not <tt><see topic=\"ACL2____SPEC-MV-LET\">spec-mv-let</see></tt>).</p>"],

 "ACL2____Overview_02of_02the_02Expansion_02of_02ENDP_02in_02the_02Base_02Case":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><v>Subgoal *1/1</v> is the <b>Base Case</b> of our induction.  It \u000A simplifies to <v>Subgoal *1/1'</v> by expanding the <b>ENDP</b> term in the \u000A hypothesis, just as we saw in the earlier proof of <v>Subgoal *1/2</v>.</p>"],

 "ACL2____O_E3_D3":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>O&gt;=</v> is a macro and <v>(<see topic=\"ACL2____O_E3_D3\">o&gt;=</see> x y)</v> expands to <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> x\u000A y))</v>.  See <see topic=\"ACL2____O_C3\">o&lt;</see>.</p>"],

 "ACL2____O_C3_D3":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>o&lt;=</v> is a macro and <v>(<see topic=\"ACL2____O_C3_D3\">o&lt;=</see> x y)</v> expands to <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____O_C3\">o&lt;</see> y\u000A x))</v>.  See <see topic=\"ACL2____O_C3\">o&lt;</see>.</p>"],

 "ACL2____O-INFP":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>O-infp</v> is a macro.  <v>(<see topic=\"ACL2____O-INFP\">O-infp</see> x)</v> opens up to \u000A  <v>(<see topic=\"COMMON-LISP____NOT\">not</see> (<see topic=\"ACL2____O-FINP\">o-finp</see> x))</v>.</p>"],

 "ACL2____NOTE-3-1_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>No significant changes have been made since Version 3.0 for support \u000A of non-standard analysis in particular.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-3-1\">note-3-1</see> for changes to Version 3.1 of ACL2.</p>"],

 "ACL2____NOTE-3-0_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>No significant changes have been made since Version 2.9 for support \u000A of non-standard analysis in particular.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-3-0\">note-3-0</see> for changes to Version 3.0 of ACL2.</p>"],

 "ACL2____NOTE-3-0-1_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>No significant changes have been made since Version 3.0 for support \u000A of non-standard analysis in particular.</p> \u000A \u000A <p>Please also see <see topic=\"ACL2____NOTE-3-0-1\">note-3-0-1</see> for changes to Version 3.0.1 of \u000A ACL2.</p>"],

 "ACL2____NOTE-2-7-PROOF-CHECKER":[["Note-2-7"],"ACL2 Sources","ACL2","<p>Output from the <see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see> can now be inhibited by \u000A supplying the symbol <v>proof-checker</v> in the list given to <see topic=\"ACL2____SET-INHIBIT-OUTPUT-LST\">set-inhibit-output-lst</see>.</p>"],

 "ACL2____NOTE-2-5_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Important changes to non-standard version:</p> \u000A \u000A <p>Please see <see topic=\"ACL2____NOTE-2-5\">note-2-5</see> for changes to Version 2.5 of ACL2.  We hope to \u000A write more documentation for ACL2(r) in the future.</p>"],

 "ACL2____MEMBER-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____MEASURE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____MAKE-CHARACTER-LIST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>Non-characters in the given list are <see topic=\"COMMON-LISP____COERCE\">coerce</see>d to the \u000A character with code 0.</p> \u000A \u000A <p><b>Function: </b><srclink>make-character-list</srclink></p><code>(<see topic=\"COMMON-LISP____DEFUN\">defun</see> make-character-list (x)\u000A       (<see topic=\"COMMON-LISP____DECLARE\">declare</see> (<see topic=\"ACL2____XARGS\">xargs</see> :guard t))\u000A       (<see topic=\"COMMON-LISP____COND\">cond</see> ((<see topic=\"COMMON-LISP____ATOM\">atom</see> x) nil)\u000A             ((<see topic=\"COMMON-LISP____CHARACTERP\">characterp</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x))\u000A              (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CAR\">car</see> x)\u000A                    (<see topic=\"ACL2____MAKE-CHARACTER-LIST\">make-character-list</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))\u000A             (t (<see topic=\"COMMON-LISP____CONS\">cons</see> (<see topic=\"COMMON-LISP____CODE-CHAR\">code-char</see> 0)\u000A                      (<see topic=\"ACL2____MAKE-CHARACTER-LIST\">make-character-list</see> (<see topic=\"COMMON-LISP____CDR\">cdr</see> x))))))</code>"],

 "ACL2____I-SMALL":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____I-SMALL\">I-small</see> x)</v> is true if and only if <v>x</v> is an infinitesimal \u000A number (possibly 0).  This predicate is only defined in ACL2(r) \u000A (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____I-LARGE":[["Real"],"ACL2 Sources","ACL2","<p><v>(<see topic=\"ACL2____I-LARGE\">I-large</see> x)</v> is true if and only if <v>x</v> is non-zero and \u000A <v>1/x</v> is an infinitesimal number.  This predicate is only defined in \u000A ACL2(r) \u000A (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____HONS-SHRINK-ALIST_12":[["Fast-alist-fork!"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Deprecated.  Alias for <tt><see topic=\"ACL2____FAST-ALIST-FORK_12\">fast-alist-fork!</see></tt>.</p>"],

 "ACL2____HONS-SHRINK-ALIST":[["Fast-alist-fork"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Deprecated.  Alias for <tt><see topic=\"ACL2____FAST-ALIST-FORK\">fast-alist-fork</see></tt>.</p>"],

 "ACL2____FLUSH-HONS-GET-HASH-TABLE-LINK":[["Hons-and-memoization"],"ACL2 Sources","ACL2","<p>This <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> topic relates to the experimental \u000A extension of ACL2 supporting hash cons, fast alists, and memoization; see \u000A <see topic=\"ACL2____HONS-AND-MEMOIZATION\">hons-and-memoization</see>.</p> \u000A \u000A <p>Deprecated.  Alias for <tt><see topic=\"ACL2____FAST-ALIST-FREE\">fast-alist-free</see></tt>.</p>"],

 "ACL2____EXTRA-INFO":[["Guard"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____GUARD-DEBUG\">guard-debug</see> for a discussion of this function, which is \u000A useful for debugging failures during <see topic=\"ACL2____GUARD\">guard</see> verification.</p>"],

 "ACL2____DOC_12":[["Legacy-documentation"],"ACL2 Sources","ACL2","<p>NOTE: The <v>:doc!</v> command only makes sense at the terminal.</p> \u000A \u000A <code>Examples:\u000AACL2 !&gt;:doc! defthm\u000AACL2 !&gt;:doc! certificate</code> \u000A \u000A <p>This command is like <v>:doc name</v> followed by <v>:</v><tt><see topic=\"ACL2____MORE_12\">more!</see></tt>.  It \u000A prints all the <see topic=\"COMMON-LISP____DOCUMENTATION\">documentation</see> of <v>name</v>.</p>"],

 "ACL2-PC____X-DUMB":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>General Form:\u000Ax-dumb:  expand without simplification.</code> \u000A \u000A <p>Same as <v>(<see topic=\"ACL2____EXPAND\">expand</see> t new-goals-flg keep-all-guards-flg)</v>.  See \u000A documentation for <v>expand</v>.</p> \u000A \u000A <p>See also <v>x</v>, which allows simplification.</p>"],

 "ACL2-PC____PRINT-ALL-GOALS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Example and General Form: print-all-goals</p> \u000A \u000A <p>Prints all the goals that remain to be proved, in a pleasant format.  See \u000A also the proof-checker command <v>print-all-concs</v>.</p>"],

 "ACL2-PC____HELP_12":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Same as <v>help</v>, except that the entire help message is printed \u000A without any need to invoke <v>more!</v> or <v>more</v>.</p> \u000A \u000A <p>Invoke <v>help</v> for documentation about the proof-checker help \u000A facility.</p>"],

 "ACL2-PC____EX":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Aex</code> \u000A \u000A <p>Same as <v>exit</v>, except that first the instruction <v>save</v> is \u000A executed.</p> \u000A \u000A <p>If <v>save</v> queries the user and is answered negatively, then the exit is \u000A aborted.</p>"],

 "ACL2-PC____ACL2-WRAP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(acl2-wrap (<see topic=\"ACL2____PE\">pe</see> :here))\u000A\u000AGeneral Form:\u000A(acl2-wrap form)</code> \u000A \u000A <p>Same as <v>(lisp form)</v>.  This is provided for interface tools that want \u000A to be able to execute the same form in raw Lisp, in the proof-checker, or in \u000A the ACL2 top-level loop <v>(<see topic=\"ACL2____LP\">lp</see>)</v>.</p>"],

 "COMMON-LISP____REST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>In the logic, <v>rest</v> is just a macro for <tt><see topic=\"COMMON-LISP____CDR\">cdr</see></tt>.</p> \u000A \u000A <p><v>Rest</v> is a Common Lisp function.  See any Common Lisp documentation for \u000A more information.</p> \u000A \u000A <p><b>Macro: </b><srclink>rest</srclink></p><code>(<see topic=\"COMMON-LISP____DEFMACRO\">defmacro</see> rest (x) (<see topic=\"COMMON-LISP____LIST\">list</see> 'cdr x))</code>"],

 "COMMON-LISP____CDDDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CADDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_023_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/2' \u000A(IMPLIES (AND (CONSP A) \u000A              (EQUAL (APP (APP (CDR A) B) C) \u000A                     (APP (CDR A) (APP B C)))) \u000A         (EQUAL <see topic='ACL2____The_02Simplification_02of_02the_02Induction_02Conclusion_02_82Step_024_92'>(</see>APP <b>(CONS (CAR A) (APP (CDR A) B))</b> \u000A                     C) \u000A                (APP A (APP B C)))). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to expand the definition of <v>APP</v> here.</p>"],

 "ACL2____The_02Q.E.D._02Message":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><b>Q.E.D.</b> stands for ``quod erat demonstrandum'' which is Latin \u000A for ``which was to be demonstrated'' and is the signal that a proof is \u000A completely done.</p>"],

 "ACL2____STOBJ-LET":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-FMT-SOFT-RIGHT-MARGIN":[["Io"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-FMT-HARD-RIGHT-MARGIN\">set-fmt-hard-right-margin</see> for a discussion of the soft \u000A and hard right margin for formatted output.</p>"],

 "ACL2____REAL-LISTP":[["Real"],"ACL2 Sources","ACL2","<p>The predicate <v>real-listp</v> tests whether its argument is a true \u000A list of real numbers.  This predicate is only defined in ACL2(r) \u000A (see <see topic=\"COMMON-LISP____REAL\">real</see>).</p>"],

 "ACL2____PROPS":[["Other"],"ACL2 Sources","ACL2","<code>Example:\u000A:props assoc-eq</code> \u000A \u000A <p><v>Props</v> takes one argument, a symbol, and prints all of the properties \u000A that are on that symbol in the ACL2 <see topic=\"ACL2____WORLD\">world</see>.</p>"],

 "ACL2____PE_12":[["History"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____PE\">pe</see>.  <v>:Pe</v> now has the functionality formerly \u000A provided by <v>:pe!</v>.</p>"],

 "ACL2____PARALLEL":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____PARALLELISM\">parallelism</see>.</p>"],

 "ACL2____Overview_02of_02the_02Final_02Simplification_02in_02the_02Base_02Case":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The <b>But</b> is our signal that the goal is proved.</p> \u000A \u000A <p>Click <see topic='ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_020_92'>here</see> to step \u000A through the proof.  It is very simple.</p>"],

 "ACL2____O_E3":[["Ordinals","ACL2-built-ins"],"ACL2 Sources","ACL2","<p><v>O&gt;</v> is a macro and <v>(<see topic=\"ACL2____O_E3\">o&gt;</see> x y)</v> expands to <v>(<see topic=\"ACL2____O_C3\">o&lt;</see> y x)</v>.  See \u000A <see topic=\"ACL2____O_C3\">o&lt;</see>.</p>"],

 "ACL2____EXPAND":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DEFAULT-VERIFY-GUARDS-EAGERNESS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____BIBLIOGRAPHY":[["About-ACL2"],"ACL2 Sources","ACL2","<p>The ACL2 home page includes a <a \u000A  href='http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html'>list \u000A  of notes and reports about ACL2</a>.</p>"],

 "ACL2-PC____MORE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>Continues documentation of last proof-checker command visited with \u000A <v>help</v>.</p> \u000A \u000A <p>Invoke <v>help</v> for documentation about the proof-checker help \u000A facility.</p>"],

 "ACL2-PC____HELP-LONG":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>See the documentation for <v>help!</v>.</p> \u000A \u000A <p><v>Help-long</v> has been included in addition to <v>help!</v> for historical \u000A reasons.  (Such a command is included in Pc-Nqthm).</p>"],

 "ACL2-PC____FREE":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(free x)\u000A\u000AGeneral Form:\u000A(free var)</code> \u000A \u000A <p>Mark <v>var</v> as a ``free variable''.  Free variables are only of interest \u000A for the <v>put</v> command; see its documentation for an explanation.</p>"],

 "ACL2-PC____CG":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Examples:\u000A(cg (main . 1)) -- change to the goal (main . 1)\u000Acg              -- change to the next-to-top goal\u000A\u000AGeneral Form:\u000A(CG &amp;OPTIONAL goal-name)</code> \u000A \u000A <p>Same as <v>(change-goal goal-name t)</v>, i.e. change to the indicated and \u000A move the current goal to the end of the goal stack.</p>"],

 "COMMON-LISP____CDDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDDAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CADAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_022_92":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<code>Subgoal *1/1' \u000A(IMPLIES (NOT (CONSP A)) \u000A         <see topic='ACL2____The_02Final_02Simplification_02in_02the_02Base_02Case_02_82Step_023_92'>(</see>EQUAL (APP B C) \u000A                <b>(APP B C)</b>)). \u000A</code> \u000A \u000A <p><img src='green-line.gif'></img></p> \u000A \u000A <p>Click on the link above to use the Axiom <v>(<see topic=\"COMMON-LISP____EQUAL\">EQUAL</see> x x) = t</v></p>"],

 "ACL2____SHARP-COMMA-READER":[["Other"],"ACL2 Sources","ACL2","<p>The use of `<v>#,</v>' has been deprecated.  Please use `<v>#.</v>' \u000A instead; see <see topic=\"ACL2____SHARP-DOT-READER\">sharp-dot-reader</see>.</p>"],

 "ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED_12":[["Parallelism"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-WATERFALL-PARALLELISM-HACKS-ENABLED\">set-waterfall-parallelism-hacks-enabled</see>.</p>"],

 "ACL2____Perhaps":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>The theorem prover's proof is printed in real time.  At the time it \u000A prints ``Perhaps'' it does not know the proof will succeed.</p>"],

 "ACL2____POSITION-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____OBDD":[["Bdd"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____BDD\">bdd</see> for information on this topic.</p>"],

 "ACL2____Name_02the_02Formula_02Above":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>When the theorem prover explicitly assigns a name, like <v>*1</v>, to \u000A a formula, it has decided to prove the formula by induction.</p>"],

 "ACL2____NOTE-2-8-ORDINALS":[["Note-2-8"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____ORDINALS\">ordinals</see>.</p>"],

 "ACL2____NOTE-2-8-GUARDS":[["Note-2-8"],"ACL2 Sources","ACL2","<p>All the guard-related changes may be found elsewhere; in \u000A particular, see <see topic=\"ACL2____NOTE-2-8-BUG-FIXES\">note-2-8-bug-fixes</see>.</p>"],

 "ACL2____NON-EXECUTABLE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____HIDDEN-DEFPKG":[["Defpkg"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____HIDDEN-DEATH-PACKAGE\">hidden-death-package</see></p>"],

 "ACL2____DEFAULT-TOTAL-PARALLELISM-WORK-LIMIT":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-TOTAL-PARALLELISM-WORK-LIMIT\">set-total-parallelism-work-limit</see>.</p>"],

 "ACL2____ASSOC-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2-PC____ST":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Asr\u000A\u000AGeneral Form:\u000A(st &amp;optional rule-id)</code> \u000A \u000A <p>See the documentation for <v>show-type-prescriptions</v>, as <v>st</v> and \u000A <v>show-type-prescriptions</v> are identical.</p>"],

 "ACL2-PC____SR":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000Asr\u000A\u000AGeneral Form:\u000A(sr &amp;optional rule-id enabled-only-flg)</code> \u000A \u000A <p>See the documentation for <v>show-rewrites</v>, as <v>sr</v> and \u000A <v>show-rewrites</v> are identical.</p>"],

 "ACL2-PC____SKIP":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Askip</code> \u000A \u000A <p>Make no change in the state-stack, but ``succeed''.  Same as <v>(sequence\u000A nil)</v>.</p>"],

 "ACL2-PC____QUIET":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(quiet induct prove)\u000A\u000AGeneral Form:\u000A(quiet &amp;rest instruction-list)</code> \u000A \u000A <p>Run the <v>instruction-list</v> through the top-level loop with no \u000A output.</p> \u000A \u000A <p>See also <v>noise</v>.</p>"],

 "ACL2-PC____CL-PROC":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>See the documentation for <tt><see topic=\"ACL2____PROOF-CHECKER\">proof-checker</see></tt> command \u000A <v>clause-processor</v>, which is identical to <v>cl-proc</v>.</p>"],

 "ACL2-PC____AL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(al 3)</code> \u000A \u000A <p>See the documentation for <v>apply-linear</v>, as <v>al</v> and \u000A <v>apply-linear</v> are identical.</p>"],

 "COMMON-LISP____KEYWORD":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____CADDDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CADDAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "ACL2____WATERFALL-PRINTING":[["Parallel-proof"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SET-WATERFALL-PRINTING\">set-waterfall-printing</see>.</p>"],

 "ACL2____Undocumented_02Topic":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p>This topic has not yet been documented.  Sorry</p>"],

 "ACL2____UNION-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SUBVERSIVE-INDUCTIONS":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____SUBVERSIVE-RECURSIONS\">subversive-recursions</see>.</p>"],

 "ACL2____STOBJS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PRINT-OBJECT_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____POSITION-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PARALLELISM-BUILD":[["Miscellaneous"],"ACL2 Sources","ACL2","<p>See <see topic=\"ACL2____COMPILING-ACL2P\">compiling-ACL2p</see>.</p>"],

 "ACL2____OBSERVATION-CW":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NOTE-4-3_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____NOTE-4-3\">note-4-3</see> for changes in Version 4.3 of \u000A  ACL2.</p>"],

 "ACL2____NOTE-4-2_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____NOTE-4-2\">note-4-2</see> for changes in Version 4.2 of \u000A ACL2.</p>"],

 "ACL2____NOTE-4-1_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____NOTE-4-1\">note-4-1</see> for changes in Version 4.1 of ACL2.</p>"],

 "ACL2____NOTE-4-0_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please see <see topic=\"ACL2____NOTE-4-0\">note-4-0</see> for changes in Version 4.0 of ACL2.</p>"],

 "ACL2____NOTE-3-6_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please also see <see topic=\"ACL2____NOTE-3-6\">note-3-6</see> for changes in Version 3.6 of \u000A  ACL2.</p>"],

 "ACL2____NOTE-3-3_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please also see <see topic=\"ACL2____NOTE-3-3\">note-3-3</see> for changes to Version 3.3 of \u000A  ACL2.</p>"],

 "ACL2____NOTE-3-2-1_82R_92":[["Release-notes"],"ACL2 Sources","ACL2","<p>Please also see <see topic=\"ACL2____NOTE-3-2-1\">note-3-2-1</see> for changes to Version 3.2.1 of \u000A  ACL2.</p>"],

 "ACL2____INTERSECTP-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DEFND":[["Events"],"ACL2 Sources","ACL2","<p><v>Defnd</v> is <tt><see topic=\"ACL2____DEFUND\">defund</see></tt> with <see topic=\"ACL2____GUARD\">guard</see> <v>t</v>.</p>"],

 "ACL2____DEFN":[["Events"],"ACL2 Sources","ACL2","<p><v>Defn</v> is <tt><see topic=\"COMMON-LISP____DEFUN\">defun</see></tt> with <see topic=\"ACL2____GUARD\">guard</see> <v>t</v>.</p>"],

 "ACL2-PC____R":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example:\u000A(r 3)</code> \u000A \u000A <p>See the documentation for <v>rewrite</v>, as <v>r</v> and <v>rewrite</v> are \u000A identical.</p>"],

 "ACL2-PC____PRINT-MAIN":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<code>Example and General Form:\u000Aprint-main</code> \u000A \u000A <p>Print the goal as originally entered.</p>"],

 "ACL2-PC____LEMMAS-USED":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>This is just an alias for <v>runes</v>.</p> \u000A \u000A "],

 "COMMON-LISP____THIRD":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____TENTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____SIXTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____SEVENTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____SECOND":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____OPTIMIZE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____NINTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____IGNORE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____IGNORABLE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____FOURTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____FIRST":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____FIFTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____EIGHTH":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDDDDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDDDAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDDADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDDAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDADDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDADAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDAADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CDAAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CADADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CADAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAADDR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAADAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAAADR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "COMMON-LISP____CAAAAR":[["ACL2-built-ins"],"ACL2 Sources","ACL2","<p>See any Common Lisp documentation for details.</p>"],

 "ACL2____UNTRANSLATE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____TYPESPEC-CHECK":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SUBSETP-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SPLIT-TYPES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LD-SKIP-PROOFSP":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LD-REDEFINITION-ACTION":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LD-KEYWORD-ALIASES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-DIFFERENCE-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-COMPILER-ENABLED":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____RULE-NAMES":[["Theories"],"ACL2 Sources","ACL2","<code>Examples:\u000A(:rewrite assoc-of-app)\u000A(:linear delta-aref . 2)\u000A(:definition length)\u000A(:executable-counterpart length)</code> \u000A \u000A <p>See <see topic=\"ACL2____RUNE\">rune</see>.</p>"],

 "ACL2____RESTRICT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REORDER":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE1-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____RASSOC-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____RASSOC-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PUT-ASSOC-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____OPEN-OUTPUT-CHANNEL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____OPEN-INPUT-CHANNEL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NO-DUPLICATESP-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NO-DUPLICATESP-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____INTERSECTP-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____INTERSECTION-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____INDUCT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____GUARD-HINTS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____CASES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____BACKCHAIN-LIMIT-RW":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____A_02Trivial_02Proof":[["Pages Written Especially for the Tours"],"ACL2 Sources","ACL2","<p><img src='concrete-proof.gif'></img></p>"],

 "ACL2____ADD-TO-SET-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ADD-LD-KEYWORD-ALIAS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2-PC____RUN-INSTR-ON-NEW-GOALS":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>See documentation for <v>then</v>.</p>"],

 "ACL2-PC____RUN-INSTR-ON-GOAL":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>See documentation for <v>then</v>.</p>"],

 "ACL2-PC____REPEAT-REC":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>See documentation for <v>repeat</v>.</p> \u000A \u000A "],

 "ACL2-PC____CONTRADICT":[["Proof-checker-commands"],"ACL2 Sources","ACL2","<p>see documentation for <v>contrapose</v></p>"],

 "ACL2____WRITE-BYTE_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____WATERFALL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____VERIFY-GUARDS-EAGERNESS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____USE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____UNTRANSLATE-PREPROCESS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____UNION-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____TRUST-TAG":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SUBSETP-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-SERIALIZE-CHARACTER":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-RULER-EXTENDERS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-NON-LINEAR":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LET_A2-ABSTRACTION":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LD-SKIP-PROOFS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-LD-KEYWORD-ALIASES_12":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-DIFFERENCE-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-ACCUMULATED-PERSISTENCE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SAVING-AND-RESTORING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____RESET-PRINT-CONTROL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE1-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE-DUPLICATES-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REMOVE-DUPLICATES-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PUT-ASSOC-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NORMALIZE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NONLINEARP":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____NO-THANKS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____MODE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____META-EXTRACT-GLOBAL-FACT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____META-EXTRACT-FORMULA":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____META-EXTRACT-CONTEXTUAL-FACT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____LEGACY-ORPHANS":[["Legacy-documentation"],"ACL2 Sources","ACL2",""],

 "ACL2____INTERSECTION-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____HANDS-OFF":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____GUARDS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____GET-OUTPUT-STREAM-STRING_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMT-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DO-NOT-INDUCT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DO-NOT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DELETE-ASSOC-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DELETE-ASSOC-EQ":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____CLOSE-OUTPUT-CHANNEL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____BACKTRACK":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ADD-TO-SET-EQUAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ADD-TO-SET-EQL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ADD-LD-KEYWORD-ALIAS_12":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62WHOLE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62REST":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62OPTIONAL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62KEY":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62BODY":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP_____62ALLOW-OTHER-KEYS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "COMMON-LISP____APROPOS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SINGLE-THREADED-OBJECTS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-RIGHT-MARGIN":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-READABLY":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-LINES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-LEVEL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-LENGTH":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-ESCAPE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____SET-PRINT-CIRCLE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____REGRESSION":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____READ-OBJECT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____READ-CHAR_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____READ-BYTE_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PUT-ASSOC-EQL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PROOF-SUPPORTERS-ALIST":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PRINT-SUMMARY-USER":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____PEEK-CHAR_42":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____OPEN-OUTPUT-CHANNEL-P":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____OPEN-INPUT-CHANNEL-P":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____META-EXTRACT-RW_B2-TERM":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____META-EXTRACT-GLOBAL-FACT_B2":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____IPRINTING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____IPRINT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____IMMED-FORCED":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____IF-INTRO":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____GETTING-STARTED":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____GCS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FORCED":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FNCALL-TERM":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMT_12-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMT1_12-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMT1-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMS_12-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FMS-TO-STRING":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____FAKE-RUNE":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____EXTERNAL-FORMAT":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____DYNAMICALLY-MONITOR-REWRITES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____CLOSE-INPUT-CHANNEL":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____CERTIFYING-BOOKS":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____BY":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____BOOK-MAKEFILES":[["Pointers"],"ACL2 Sources","ACL2",""],

 "ACL2____ACL2S":[["Pointers"],"ACL2 Sources","ACL2",""]};